package com.platform.metascheduler.client;

import org.globus.axis.gsi.GSIConstants;
import org.globus.wsrf.impl.security.authentication.Constants;
import org.globus.wsrf.impl.security.authorization.HostAuthorization;
import org.globus.wsrf.impl.security.authorization.NoAuthorization;
import org.globus.wsrf.impl.security.authorization.GridMapAuthorization;
import org.globus.wsrf.impl.security.authorization.SelfAuthorization;
import org.globus.gsi.proxy.IgnoreProxyPolicyHandler;

//import org.globus.ogsa.NotificationSinkCallback;

import java.io.FileReader;
import java.io.File;
import java.io.BufferedReader;
import java.io.IOException;

import java.net.URL;

import com.platform.metascheduler.job.JobPortType;
import com.platform.metascheduler.job.JobFaultType;
import javax.xml.namespace.QName;
import javax.xml.rpc.Stub;

import org.apache.axis.message.addressing.Address;
import org.apache.axis.message.addressing.EndpointReferenceType;
import org.apache.axis.message.MessageElement;
import org.apache.axis.message.addressing.ReferencePropertiesType;

import com.platform.metascheduler.job.CreateResourceResponse;
import com.platform.metascheduler.job.CreateResource;
import com.platform.metascheduler.job.Start;
import com.platform.metascheduler.job.StartResponse;
import com.platform.metascheduler.job.Modify;
import com.platform.metascheduler.job.ModifyResponse;
import com.platform.metascheduler.job.Cancel;
import com.platform.metascheduler.job.CancelResponse;
import com.platform.metascheduler.job.GetStatus;
import com.platform.metascheduler.job.GetStatusResponse;
import com.platform.metascheduler.job.Stop;
import com.platform.metascheduler.job.StopResponse;
import com.platform.metascheduler.job.Resume;
import com.platform.metascheduler.job.ResumeResponse;
import com.platform.metascheduler.job.Submit;
import com.platform.metascheduler.job.SubmitResponse;
import com.platform.metascheduler.job.GetJobData;
import com.platform.metascheduler.job.GetJobDataResponse;
import com.platform.metascheduler.job.service.JobServiceAddressingLocator;

import org.oasis.wsrf.lifetime.Destroy;
import org.globus.wsrf.ResourceException;
import org.globus.wsrf.encoding.ObjectSerializer;
import org.globus.wsrf.encoding.ObjectDeserializer;
import org.globus.wsrf.encoding.DeserializationException;

import java.rmi.RemoteException;
import javax.xml.rpc.ServiceException;
import java.io.FileOutputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import org.xml.sax.InputSource;
import com.platform.metascheduler.utils.MetaSchedulerConstants;

public class JobServiceClient {
    static{
        org.globus.axis.util.Util.registerTransport();
    }

    static String    userhome= System.getProperty("user.home") + "/";

    private static void printUsage() {
        System.err.println("Usage: JobServiceClient <JobService_handle> <JobService_operation>");
        System.err.println("Usage: JobServiceClient <job_name> <service_operation>");
        System.err.println("Usage: JobServiceClient <JobServiceClient_handle> <job_name> <JobServiceClient_operation>");
        System.err.println("JobService Handle:  URI to Jobservice");
        System.err.println("JobServiceClient Handle:  URI to JobServiceClient");
        System.err.println("Jobservice operation:");
        System.err.println("    createService rsl <rsl_file> [job_name] ");
        System.err.println("service operation:");
        System.err.println("    start <cluster_name>");
        System.err.println("    start <ResourceManagerFactoryService_handle> <cluster_name>");
        System.err.println("JobServiceClient operation:");
        System.err.println("    submit [queue_name]");
        System.err.println("    cancel");
        System.err.println("    stop");
        System.err.println("    resume");
        System.err.println("    getStatus");
        System.err.println("    getJobData");
        return;
    }

    private static String readRsl(String rslFile) {
        BufferedReader rslReader = null;
        StringBuffer rsl = null;
        try {
            rslReader = new BufferedReader(new FileReader(rslFile));
            rsl = new StringBuffer();
            String rslLine = rslReader.readLine();
            while (rslLine != null) {
                rsl.append(rslLine);
                rslLine = rslReader.readLine();
            }
        } catch (IOException e) {
            System.err.println("Error reading rsl file: " + e.getMessage());
            System.exit(2);
        } finally {
            if (rslReader != null) {
                try {
                    rslReader.close();
                } catch (IOException ee) {
                    // ignore this error
                }
            }
        }
        return (rsl.toString());
    }

    public static void main (String[] args) {
	String handle = null;
	String op = null;
	String oparg[] = new String[3];
	JobServiceAddressingLocator locator = new JobServiceAddressingLocator();
	EndpointReferenceType endpoint = null;
        JobPortType job = null;
        JobPortType jobd= null;

        if (args.length < 2) {
	    printUsage();
            return;
        }

	handle = args[0];
	op = args[1];
	if (args.length > 2) {
            oparg[0] = args[2];
        }
	if (args.length > 3) {
	    oparg[1] = args[3];
	}

	if (args.length > 4) {
	    oparg[2] = args[4];
	}

	if (op.equalsIgnoreCase("createService")) { 
	    try{
		if (oparg[0] == null || oparg[1] == null) {
		    printUsage();
		    return;
		}
	    
		endpoint = new EndpointReferenceType();
                endpoint.setAddress(new Address(args[0]));
                job = locator.getJobPortTypePort(endpoint);
		
		((Stub)job)._setProperty(Constants.GSI_SEC_CONV, Constants.SIGNATURE);
		((Stub)job)._setProperty(GSIConstants.GSI_MODE, GSIConstants.GSI_MODE_LIMITED_DELEG);
		((Stub)job)._setProperty(Constants.AUTHORIZATION, new HostAuthorization());
		//((Stub)job)._setProperty(Constants.GRIM_POLICY_HANDLER,
		//				   new IgnoreProxyPolicyHandler());
		//((org.apache.axis.client.Stub)job).setTimeout(0);			   
		String		  createOp = null;
		String            instanceID = null;

		if (oparg[0].equalsIgnoreCase("rsl")) {
		    createOp = "RSL:";
		    createOp = createOp + (readRsl(oparg[1]));
		    if ( oparg[2] != null) { 
			instanceID = oparg[2];
		    }
		} else if (oparg[0].equalsIgnoreCase("id")) {
		    createOp = "ID:" +  oparg[1];
		} else {
		    printUsage();
		    return;
		}
	
		CreateResourceResponse createResponse = job.createResource(new CreateResource(createOp,instanceID));
                endpoint = createResponse.getEndpointReference();
		String status   = createResponse.getStatus();
                if (status.startsWith(MetaSchedulerConstants.JOB_SERVICE_ERR_EXIST)) {
                      System.out.println("Error :" + MetaSchedulerConstants.JOB_SERVICE_ERR_EXIST);
                      return;
                }

                // Serialize endpoint
                if (instanceID == null) {
			System.out.println("user didn't specify job name! substitute with Resourcekey!");
                        ReferencePropertiesType rp= endpoint.getProperties();
                        MessageElement [] key = rp.get_any();
                        instanceID = key[0].getValue();
                }
                FileOutputStream out = null;
                try {
                       out = new FileOutputStream(userhome + "Job_" + instanceID + ".epr");
                       out.write(ObjectSerializer.toString(endpoint,
                                          new QName("", "JobEPR")).getBytes());
                } catch (IOException e) {
                       System.err.println("Failed to write subscription epr: " +
                                      e.getMessage());
                } finally {
                       if (out != null) {
                             try { out.close(); } catch (Exception ee) {}
                       }
                }

                System.out.println("Service location:" + args[0]);
                System.out.println("CreateJob Successfully: " + instanceID);
               
            }catch (RemoteException rme){
                        System.out.println("RemoteEx catched! Create Job error!");
                        System.out.println(rme.getMessage());
            }catch (Exception e) {
                        e.printStackTrace();
            }
            return;
	} // END of "createService"

        //Deserialize endpoint
        FileInputStream in = null;
        try {
            in = new FileInputStream(userhome + "Job_" + args[0] + ".epr");
            endpoint = (EndpointReferenceType)ObjectDeserializer.deserialize(
                new InputSource(in),
                EndpointReferenceType.class);
        }catch ( FileNotFoundException fe) {
            System.out.println(" Job :" + args[0] + "hasn't been created !");
        } catch (DeserializationException e){
        }finally {
            if (in != null) {
                try { in.close(); } catch (Exception e) {}
            }
        }
        if ( endpoint == null ) {
            System.out.println(" Job :" + args[0] + "hasn't been created !");
            return ;
        }

	try {
	    job = locator.getJobPortTypePort(endpoint);
            //((Stub)job)._setProperty(Constants.GSI_SEC_MSG,
            //                         Constants.SIGNATURE);
	    ((Stub)job)._setProperty(Constants.GSI_SEC_CONV,
				     Constants.SIGNATURE);
	    ((Stub)job)._setProperty(GSIConstants.GSI_MODE,
	   			     GSIConstants.GSI_MODE_LIMITED_DELEG);
	    ((Stub)job)._setProperty(Constants.AUTHORIZATION,
			             new HostAuthorization());
	    //((Stub)job)._setProperty(Constants.GRIM_POLICY_HANDLER,
	    //			     new IgnoreProxyPolicyHandler());
	    //((org.apache.axis.client.Stub)job).setTimeout(300000);
	    
	    if (op.equalsIgnoreCase("submit")) {
                if (oparg[0] == null) {
                    System.out.println(op + "() => " + job.submit(new Submit(endpoint,new String(""))).getJobId());
                } else {
                    System.out.println(op + "(" + oparg[0] + ") => "
                               + job.submit(new Submit(endpoint,oparg[0])).getJobId());
                }
	    } else if (op.equalsIgnoreCase("start")) {
	        System.out.println("begin to invoke start!");
		if (oparg[0] == null) {
		    System.err.println("Oprand need for " + op +"()");
		    return;
		}
		// 1 arg  : cluster 
		// 2 args : cluster factoryURL 
		// 3 args : cluster factoryURL type 
		// 4 args : cluster factoryURL rsvId type 
		// start method parameter: cluster factoryURL rsvId type 
                if (oparg[1] == null) {
		    System.out.println(op + "(" + oparg[0] + ") => "); 
		    System.out.println(job.start(new Start(oparg[0],null, null, null)).getValue());
                } else if (oparg[2] != null) { // 3 args
		    System.out.println(op + "(" + oparg[0] + ") => "); 
		    System.out.println(job.start(new Start(oparg[2], oparg[0],null, oparg[1])).getValue());
                } else {
		    System.out.println(op + "(" + oparg[0] + ", "+oparg[1]+") => ");
		    System.out.println(job.start(new Start(null,oparg[0] , null, oparg[1])).getValue());
                }
	    } else if (op.equalsIgnoreCase("modify")) {
		if (oparg[0] == null) {
		    System.err.println("Oprand need for " + op +"()");
		    return;
		}
		System.out.println("NOT IMPLEMENTED YET");
		System.out.println(op + "(" + oparg[0] + ") => " +
			(job.modify(new Modify(oparg[0])).isValue() ? "Success!" : "Fail!"));
	    } else if (op.equalsIgnoreCase("cancel")) {
	        System.out.println("begin to invoke cancel!");
		System.out.println(op + "() => " + (job.cancel(new Cancel()).isValue() ?"Success!" : "Fail!"));
	    } else if (op.equalsIgnoreCase("getStatus")) {
		System.out.println(op + "() => " + job.getStatus(new GetStatus()).getValue());
	    } else if (op.equalsIgnoreCase("stop")) {
		System.out.println(op + "() => " + (job.stop(new Stop()).isValue() ? "Success!" : "Fail!"));
	    } else if (op.equalsIgnoreCase("resume")) {
		System.out.println(op + "() => " + (job.resume(new Resume()).isValue() ? "Success!" : "Fail!"));
	    } else if (op.equalsIgnoreCase("getJobData")) {
		System.out.println(op + "() => " + job.getJobData(new GetJobData()).getValue());
	    } else if (op.equalsIgnoreCase("destroy")) {
                job.destroy(new Destroy());
                //delete epr file 
                File test = new File(userhome + "Job_"+ args[0]+".epr");
                if (test.exists()) {
                      	if(test.delete()){
	                	System.out.println("deleted " + args[0] + "'s epr file");
                	} else{
                        	System.out.println("Delete epr file failed!");
                	}
                }
		System.out.println("Job instance destroyed");
	    } else {
		System.err.println("Operation <" + op +"> not supported");
	    }
	} catch (java.rmi.RemoteException e) {
	    String msg = e.getMessage(); 
	    if ( msg != null ) { 
	       System.out.println("Job fault type: " + msg);
	    } else { 
	       e.printStackTrace();
	    }
	} catch (Exception e) {
	    System.out.println(e.getMessage());
	    e.printStackTrace();
	}
    }
}
