package com.platform.metascheduler.client;

import org.globus.axis.gsi.GSIConstants;
import org.globus.wsrf.impl.security.authentication.Constants;
import org.globus.wsrf.impl.security.authorization.HostAuthorization;
import org.globus.wsrf.impl.security.authorization.NoAuthorization;
import org.globus.wsrf.impl.security.authorization.GridMapAuthorization;
import org.globus.wsrf.impl.security.authorization.SelfAuthorization;
import org.globus.gsi.proxy.IgnoreProxyPolicyHandler;

import java.net.URL;
import org.apache.axis.message.addressing.Address;
import org.apache.axis.message.addressing.EndpointReferenceType;

import com.platform.metascheduler.queuing.QueueConfigParamsType;
import com.platform.metascheduler.queuing.QueueDataType;
import com.platform.metascheduler.queuing.QueuingRequestType;
import com.platform.metascheduler.queuing.QueuingPortType;
import com.platform.metascheduler.queuing.QueuingFaultType;

import com.platform.metascheduler.queuing.CreateResourceResponse;
import com.platform.metascheduler.queuing.CreateResource;
import com.platform.metascheduler.queuing.GetQueueConfigParams;
import com.platform.metascheduler.queuing.GetQueueConfigParamsResponse;
import com.platform.metascheduler.queuing.GetQueueData;
import com.platform.metascheduler.queuing.GetQueueDataResponse;
import com.platform.metascheduler.queuing.Open;
import com.platform.metascheduler.queuing.OpenResponse;
import com.platform.metascheduler.queuing.Close;
import com.platform.metascheduler.queuing.CloseResponse;
import com.platform.metascheduler.queuing.Submit;
import com.platform.metascheduler.queuing.SubmitResponse;
import org.oasis.wsrf.lifetime.Destroy;

import com.platform.metascheduler.queuing.service.QueuingServiceAddressingLocator;
import com.platform.metascheduler.utils.MetaSchedulerConstants;

import javax.xml.namespace.QName;
import javax.xml.rpc.Stub;
import org.xml.sax.InputSource;

import org.globus.wsrf.ResourceException;
import org.globus.wsrf.encoding.ObjectSerializer;
import org.globus.wsrf.encoding.ObjectDeserializer;
import org.globus.wsrf.encoding.DeserializationException;

import java.rmi.RemoteException;
import java.util.StringTokenizer;
import javax.xml.rpc.ServiceException; 
import java.net.MalformedURLException; 
import java.io.FileOutputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.File;
import java.io.FileNotFoundException;
import javax.xml.rpc.ServiceException;

public class QueuingServiceClient {
    static {
 	org.globus.axis.util.Util.registerTransport();
    }
    static String    userhome= System.getProperty("user.home") + "/";

    public static void main (String[] args) {
	String op = null;
	String oparg = null;
	EndpointReferenceType endpoint = null;

        if (args.length < 2) {
            System.err.println("Usage: QueuingServiceClient <QueuingService_handle> create <queue_name>");
            System.err.println("Usage: QueuingServiceClient <QueuingService_handle> <getQueueData|getQueueConfigParams> <queue_name>");
            return;
        }
	op = args[1];
	if (args.length > 2) {
            oparg = args[2];
        }

	QueuingServiceAddressingLocator locator = new QueuingServiceAddressingLocator();

	    if (op.equalsIgnoreCase("create")) {
		/* We create a service handle. */
		String status = null;
		try {
			// Create endpoint reference (sans resource)
			endpoint = new EndpointReferenceType();
			endpoint.setAddress(new Address(args[0]));
			QueuingPortType queue = locator.getQueuingPortTypePort(endpoint);
	                ((Stub)queue)._setProperty(Constants.GSI_SEC_CONV,
       		                            Constants.SIGNATURE);
                	((Stub)queue)._setProperty(GSIConstants.GSI_MODE,
                                            GSIConstants.GSI_MODE_LIMITED_DELEG);
                	((Stub)queue)._setProperty(Constants.AUTHORIZATION,
                                           new HostAuthorization());
                	((org.apache.axis.client.Stub)queue).setTimeout(0);

			// Create resource and get endpoint reference (with resource)
			if (oparg == null) {
				System.err.println("Error : No queue name specified");
				System.err.println("Usage: QueuingServiceClient <service_handle> create <queueName>");
				return ;
			}
			CreateResourceResponse createResponse = queue.createResource(new CreateResource(oparg));
			status = createResponse.getStatus();
			endpoint = createResponse.getEndpointReference();
			if (status.startsWith(MetaSchedulerConstants.QUEUE_SERVICE_ERR_EXIST)) {
				System.out.println("Error :" + MetaSchedulerConstants.QUEUE_SERVICE_ERR_EXIST);
				return;
			}
			
			// Serialize endpoint
			FileOutputStream out = null;
                	try {
	                     out = new FileOutputStream(userhome + "Queue_" + oparg + ".epr");
       		             out.write(ObjectSerializer.toString(endpoint, 
					new QName("", "QueueEPR")).getBytes());
       		         } catch (IOException e) {
       	             		System.err.println("Failed to write subscription epr: " +
                                       e.getMessage());
       		         } finally {
       		             if (out != null) {
       		                 try { out.close(); } catch (Exception ee) {}
       	             		}
       		         }

			System.out.println("Service location:" + args[0]);
			System.out.println("CreateQueuing Successfully: " + oparg);
		}catch (RemoteException rme){
			System.out.println("RemoteEx catched! Create Queue error!");
                        System.out.println(rme.getMessage());
		}catch (Exception e) {
			e.printStackTrace();
		}
			return;
	    }

	    //Deserialize endpoint
            FileInputStream in = null;
            try {
                in = new FileInputStream(userhome + "Queue_" + oparg + ".epr");
                endpoint = (EndpointReferenceType)ObjectDeserializer.deserialize(
                    new InputSource(in),
                    EndpointReferenceType.class);
            }catch ( FileNotFoundException fe) {
		System.out.println(" Queue: " + oparg + " hasn't been created !");
		return ;
	    } catch (DeserializationException e){
	    }finally {
                if (in != null) {
                    try { in.close(); } catch (Exception e) {}
                }
            }

	    if ( endpoint == null ) {
		System.out.println(" Queue:" + oparg + "hasn't been created !");
		return ;
	    }
	    try {
	    	QueuingPortType queue = locator.getQueuingPortTypePort(endpoint);
                ((Stub)queue)._setProperty(Constants.GSI_SEC_CONV,
                                   Constants.SIGNATURE);
                ((Stub)queue)._setProperty(GSIConstants.GSI_MODE,
                                                   GSIConstants.GSI_MODE_LIMITED_DELEG);
                ((Stub)queue)._setProperty(Constants.AUTHORIZATION,
                                           new HostAuthorization());
                ((org.apache.axis.client.Stub)queue).setTimeout(300000);

	    	if (op.equalsIgnoreCase("getQueueData")) {
			QueueDataType qData = queue.getQueueData(new GetQueueData()).getQueue_data();

			printQueueData(qData);
	    	} else if (op.equalsIgnoreCase("getQueueConfigParams")) {
			QueueConfigParamsType qConfig = queue.getQueueConfigParams(new GetQueueConfigParams()).getValue();
			if ( qConfig == null) { 
			    System.err.println("Cannot get queue configuration"); 
			} else { 
			    printQueueConfigParams(qConfig);
			}
	    	} else if (op.equalsIgnoreCase("open")) {
			queue.open(new Open());
	    	} else if (op.equalsIgnoreCase("close")) {
			queue.close(new Close());
	    	} else if (op.equalsIgnoreCase("submit")) {
			if (oparg == null) {
		    		System.err.println("Operation <"+ op +"> requires oprand");
		    		return;
			}
		
			EndpointReferenceType jobepr = readjobEpr(oparg);
			EndpointReferenceType rsvepr = null;
			//modify here after implemented reservation service
			/*if ( st.hasMoreTokens() ) { 
		    		rsvHandle = st.nextToken();
			}*/
			QueuingRequestType req = new QueuingRequestType( ); 
			req.setJobEpr(jobepr);
			req.setRsvEpr(rsvepr);
			queue.submit(new Submit(req));
	    	} else if (op.equalsIgnoreCase("modify")) {
			if (oparg == null) {
		    		System.err.println("Operation <"+ op +"> requires oprand");
		    		return;
			}
			// queue.submit();
	    	} else if (op.equalsIgnoreCase("remove")) {
			if (oparg == null) {
		    		System.err.println("Operation <"+ op +"> requires oprand");
		    	return;
			}
			// queue.submit();
	    	} else if (op.equalsIgnoreCase("destroy")) {
			queue.destroy(new Destroy());
			//delete epr file
			File test = new File(userhome + "Queue_"+ args[0]+".epr");
			if (test.exists()) {
				if(test.delete()){
					System.out.println("deleted " + oparg + "'s epr file");
				} else{
				System.out.println("Delete epr file failed!");
				}
			}
			System.out.println("Queue resource destroyed");
	    	} else {
			System.err.println("Operation <" + args[1] +"> not supported");
	    	} 
	    } catch (RemoteException e){
		System.err.println("Error: " + e.getMessage());
	    } catch (Exception e){
		e.printStackTrace();
	    }
    }

    public static void printQueueData(QueueDataType qData) { 
	System.out.println("Queue run-time information"); 
	System.out.println("\tname: " + qData.getName().getLocalPart());
	System.out.println("\tstatus: " + qData.getStatus());
	return;
    } 

    public static void printQueueConfigParams(QueueConfigParamsType qConfig) { 
	
	System.out.println("Queue configuration information"); 
	System.out.println("\tname: " + qConfig.getName().getLocalPart());
	String[]  plugins = qConfig.getPluginArray();
	
	System.out.println("\tplugins loaded:");
	for(int k=0; k < plugins.length; k++) { 
	    System.out.println("\t\t" + plugins[k]);
	}
	System.out.println("\tscheduleInterval: " + qConfig.getScheduleInterval() + " seconds");
        System.out.println("\tthrottle:         " + qConfig.getThrottle() + " jobs");
	return;
    }
    
    public static EndpointReferenceType readjobEpr(String name) {
        FileInputStream in = null;
	EndpointReferenceType endpoint = null;
        try {
            in = new FileInputStream("Job_" + name + ".epr");
            endpoint = (EndpointReferenceType)ObjectDeserializer.deserialize(
                new InputSource(in), EndpointReferenceType.class);
        }catch ( FileNotFoundException fe) {
            System.out.println(" Job :" + name + "hasn't been created !");
        } catch (DeserializationException e){
        }finally {
            if (in != null) {
                try { in.close(); } catch (Exception e) {}
            }
        }
	return endpoint;
    }
}
