/**
 * JobImpl - Implementation for the JobService.
 * 
 */
package com.platform.metascheduler.impl;

import java.rmi.RemoteException;
import com.platform.metascheduler.impl.JobImpl;
import com.platform.metascheduler.utils.JNDIHelper;
import com.platform.metascheduler.job.GridJobStatus;
import javax.naming.NamingException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.io.*;
import java.lang.Boolean;
import java.net.URL;
import java.net.InetAddress;
import java.math.BigDecimal;
import java.util.ArrayList;

import org.globus.gram.GramException;
import org.globus.gsi.gssapi.auth.*;

import org.globus.wsrf.container.Semaphore;

import org.apache.axis.message.MessageElement;
import org.globus.axis.gsi.GSIConstants;
import org.apache.commons.logging.*;
import javax.xml.soap.SOAPException;
import java.net.MalformedURLException;
import javax.xml.namespace.QName;

import java.util.Timer;
import java.util.TimerTask;

import org.globus.axis.gsi.GSIConstants;

import org.ietf.jgss.GSSCredential;
import org.ietf.jgss.GSSContext;
import javax.xml.rpc.Stub;
import java.util.Calendar;

import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.apache.axis.utils.XMLUtils;

import org.globus.exec.generated.StateEnumeration;
import org.globus.exec.utils.FaultUtils;
import org.globus.exec.utils.ManagedJobFactoryConstants;
import org.globus.exec.utils.Resources;
import org.globus.exec.utils.client.ManagedJobFactoryClientHelper;
import org.globus.exec.utils.rsl.RSLHelper;
import org.globus.exec.client.GramJob;
import org.globus.exec.client.GramJobListener;
import org.globus.wsrf.encoding.ObjectSerializer;
import org.globus.wsrf.impl.security.authentication.Constants;
import org.globus.wsrf.impl.security.authorization.Authorization;
import org.globus.wsrf.impl.security.authorization.HostAuthorization;
import org.globus.wsrf.impl.security.authorization.SelfAuthorization;

import org.apache.axis.components.uuid.UUIDGen;
import org.apache.axis.components.uuid.UUIDGenFactory;

import org.globus.wsrf.ResourceKey;
import org.globus.wsrf.ResourceHome;
import org.globus.wsrf.ResourceException;
import org.apache.axis.message.addressing.EndpointReferenceType;

import org.globus.wsrf.impl.SimpleResourceKey;
import org.globus.exec.utils.ManagedJobConstants;
import org.globus.wsrf.utils.AddressingUtils;
/**
 * Handle job dispatching to GRAM and collect job status from GRAM.
 */
public class JobGramTask extends Thread implements GramJobListener {

    static Log logger = LogFactory.getLog(JobGramTask.class);

    private boolean destroy;
    private boolean done = false;
    private ResourceKey m_key=null;
    private String m_factoryUrl = null;
    private String m_type = null;
    private String m_rslString = null;
    private GramJob m_gramjob = null;
    private Semaphore m_semaphore = new Semaphore();
    private Element m_message;
    private boolean m_cancelFlag = false;
    private boolean m_sigFlag = false;
    private int m_sigValue = 0;
    private EndpointReferenceType factoryEndpoint = null;
    private GSSCredential m_credential = null;


    /**
     * JobGramTask: constructor
     *
     * Create a thread that submit job to a ManagedJobService to run
     */
    public JobGramTask(ResourceKey jobkey, String rsl, String factory, String type, GSSCredential credential) {
        super("JobGramTask");    // explicit call to superclass (ServiceS keleton) 
                         // default constructor
	m_key = jobkey;
        m_rslString = rsl;
        m_factoryUrl = factory;
        m_type = type;
	logger.debug("Create JobGramTask type=" + m_type);
	m_credential = credential;
    }

    public boolean submitToGram(String rslfile, String factoryUrl, String type) 
                         throws RemoteException {
        Integer xmlSecurity = Constants.SIGNATURE;
	logger.debug("Self Auth");
        Authorization authorization = (Authorization) HostAuthorization.getInstance();
        boolean destroy = true;
        String destType = type;

        if ( factoryUrl == null ) {
            // Use local default
            throw new RemoteException("NULL factory URL");
        }

        if ( type == null ) {
            destType = ManagedJobFactoryConstants.DEFAULT_FACTORY_TYPE;
        }

        try {
            init(factoryUrl,
                   m_rslString,
                   authorization,
                   xmlSecurity,
                   destroy,
		   destType);
        } catch(Exception e) {
            logger.error("Failed to send job to gram", e);
            return false;
        }

        return (true);
    }

    protected void init(String factoryUrl,
                   String rslFile,
                   Authorization authorization,
                   Integer xmlSecurity,
                   boolean destroy,
		   String type) 
		   throws RemoteException{
	JobResource resource = getResource();
        m_factoryUrl = factoryUrl;
	try{
        	URL factoryURL = ManagedJobFactoryClientHelper.getServiceURL(factoryUrl).getURL();
        	factoryEndpoint = ManagedJobFactoryClientHelper.getFactoryEndpoint(factoryURL,type);
	} catch (Exception e) {
		logger.error("Could not create factory endpoint reference!");
		throw new RemoteException("Could not create factory endpoint reference",e);
	}
	logger.debug("factoryEndpoint=" + factoryEndpoint);

        if (!destroy) {
            logger.error("Warning: Will not destroy the created service.");
        }

	GramJob job = null;
	try {
        	job = new GramJob(rslFile);
	} catch (Exception e) {
                String errorMessage =
                    "Unable to parse RSL from file " + rslFile;
                logger.debug(errorMessage, e);
                logger.error(errorMessage + " - " + e.getMessage());
	}
	job.setTimeOut(GramJob.DEFAULT_TIMEOUT);
        job.setAuthorization(authorization);
        job.setMessageProtectionType(xmlSecurity);
	job.setCredentials(m_credential);
	job.setDelegationEnabled(false);

        m_gramjob = job;
	
	resource.setGramJob(job);

        job.addListener(this);

        UUIDGen uuidgen = UUIDGenFactory.getUUIDGen();
        String submissionID = "uuid:" + uuidgen.nextUUID();
	//resource.getJobdata().setJobId(submissionID);

	boolean submitted = false;
	int tries = 0;

        while (!submitted) {
            tries++;
            try {
        	logger.debug("Submit job!");
                job.submit(factoryEndpoint, false, true, submissionID);
	        /*ManagedJobFactoryPortType factoryPort = 
			ManagedJobFactoryClientHelper.getPort(factoryEndpoint);
        	this.jobEndpointReference
            		= createJobEndpoint(factoryPort, jobID, batch);*/
                submitted = true;
            } catch (Exception e) {
                logger.debug("Exception while submitting the job request: ", e);
                logger.error("Error submitting job request: " + e.getMessage());
                try {
                	job.destroy();
                	logger.error("Error starting job: " + e.getMessage());
            	} catch (Exception ee) {
                	logger.error("Error destroying job", ee);
            	}
            	try {
                	jobExit(resource);
            	} catch (Exception ee) {
                	logger.error("Error set job status ", ee);
            	}
            }
        }

        logger.debug(   "CREATED MANAGED JOB SERVICE WITH HANDLE: "
                            + job.getHandle());

        if (logger.isDebugEnabled()) {
            long millis = System.currentTimeMillis();
            BigDecimal seconds = new BigDecimal(((double)millis)/1000);
            seconds = seconds.setScale(3, BigDecimal.ROUND_HALF_DOWN);
            logger.debug( "submission time, in seconds from the Epoch:"
                          + "\nafter: " + seconds.toString());
            logger.debug( "\nafter, in milliseconds: " + millis);
        }

        synchronized (this) {
            while (!this.done) {
                try {
                    wait(60000);
                    //wait();
                }
                catch (InterruptedException ie) {
                    logger.error("interrupted waiting for job to finish",
                                 ie);
                }
                catch (Exception e) {
                    logger.error("Error get gram job status", e);
                }
                try {
                    logger.debug("Wait timed out, pulling job status");
                    m_gramjob.refreshStatus();
                } catch (Exception e) {
                    logger.error("Error refresh gram job status", e);
                }
            }
        }

    }

    public void stateChanged(GramJob job) {
        StateEnumeration jobState = job.getState();
	JobResource resource = this.getResource();
        boolean holding = job.isHolding();
        logger.debug("========== State Notification ==========");
        String holdString = "";
        if (holding) {
		holdString = "HOLD ";
	}
        logger.debug("Job state: " + holdString + jobState.getValue());
        logger.debug("========================================");

        synchronized (this) {
            if ( jobState.equals(StateEnumeration.Done)) {
                jobDone(resource);
                this.done = true;
            } else 
	    if (jobState.equals(StateEnumeration.Failed)) {
                jobExit(resource);
                this.done = true;
                logger.debug("Exit Code: " + Integer.toString(job.getExitCode()));
	    }
            notifyAll();
            //if we a running an interractive job,
            //prevent a hold from hanging the client
            if (holding) {
                logger.debug("Automatically releasing hold for interactive job");
                try {
                    job.release();
                } catch (Exception e) {
                   String errorMessage = "Unable to release job from hold";
                   logger.debug(errorMessage, e);
                   logger.error(errorMessage + " - " + e.getMessage());
                }
            } // end if
        } //end synchronized
    }

    public void run() {
        try {
            submitToGram(m_rslString, m_factoryUrl, m_type);
        } catch (Exception e) {
            logger.error("Error running job", e);
        }
    }
   
    public JobResource getResource() {
        ResourceHome home = null;
        JobResource resource = null;
        try{
              home= JNDIHelper.getResourceHome("metascheduler/JobService");
        } catch(NamingException e) {
              logger.error("Get job ResourceHome error!" + e.getMessage());
        }
        try{
              resource = (JobResource)home.find(m_key);
        } catch(ResourceException e) {
              logger.error(e);
        }
	return resource;
    }

    public void jobDone(JobResource resource){
        GridJobStatus state = resource.getJobdata().getState();
	if ( state == GridJobStatus.Run_susp) {
            // For GRAM job suspended means kill it
            // and restart it later so we need to
            // ignore the exit status update.
            return ;
        } else {
	    resource.acquire();
            resource.getJobdata().setState(GridJobStatus.PDone);
 	    resource.release();
        }
    }

    public void jobExit(JobResource resource){
        GridJobStatus state = resource.getJobdata().getState();
        if ( state == GridJobStatus.Run_susp) {
            // For GRAM job suspended means kill it
            // and restart it later so we need to
            // ignore the exit status update.
            return ;
        } else {
	    resource.acquire();
            resource.getJobdata().setState(GridJobStatus.PExit);
 	    resource.release();
        }
    }
}
