package com.platform.metascheduler.impl;

import java.util.TimerTask;
import java.util.Calendar;
import java.rmi.RemoteException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.platform.metascheduler.impl.JobImpl;
import com.platform.metascheduler.job.GridJobStatus;
import com.platform.metascheduler.job.JobDataType;
import com.platform.metascheduler.resourcemanager.JobQuery;
import com.platform.metascheduler.resourcemanager.ResourceManagerLsfPortType;
import com.platform.metascheduler.resourcemanager.ResourceManagerJobStatus;
import com.platform.metascheduler.resourcemanager.JobInfoType;
import com.platform.metascheduler.resourcemanager.impl.JobInfoTypeHelper;
import com.platform.metascheduler.job.GetStatus;
import com.platform.metascheduler.job.GetJobDataType;
import org.globus.wsrf.ResourceKey;
import org.globus.wsrf.ResourceHome;

import org.ietf.jgss.GSSCredential;
import com.platform.metascheduler.utils.JNDIHelper;
import javax.naming.NamingException;
import org.globus.wsrf.ResourceException;
/**
 * Pull job info from ResourceManager periodically
 */
public class JobInfoTask extends TimerTask {
    
//    private JobImpl m_job;
    private ResourceKey m_key;
    private JobDataType jobdata;
    GSSCredential m_credential;
    static Log logger = LogFactory.getLog(JobInfoTask.class);
    private static final String TYPE_RMADAPTER = "rmadapter";

    public JobInfoTask(ResourceKey key,GSSCredential credential) {
        m_key = key;
        m_credential=credential;
    }
  
    private GridJobStatus translateJobStatus(ResourceManagerJobStatus rmInfo) {
        GridJobStatus retstatus = null;

        ResourceManagerJobStatus rmStatus = rmInfo;

        logger.debug("In translateJobStatus, rm state ="+rmStatus);
        if ( rmStatus.equals(ResourceManagerJobStatus.PEND) ||
             rmStatus.equals(ResourceManagerJobStatus.WAIT)) {
            retstatus = GridJobStatus.Forwarded;
        } else if ( rmStatus.equals(ResourceManagerJobStatus.RUN) ||
                    rmStatus.equals(ResourceManagerJobStatus.UNKWN)) {
            retstatus = GridJobStatus.Run;
        } else if ( rmStatus.equals(ResourceManagerJobStatus.PSUSP)) {
            retstatus = GridJobStatus.Forw_susp;
        } else if ( rmStatus.equals(ResourceManagerJobStatus.SSUSP) ||
                    rmStatus.equals(ResourceManagerJobStatus.USUSP) ) {
            retstatus = GridJobStatus.Run_susp;
        } else if ( rmStatus.equals(ResourceManagerJobStatus.DONE)) {
            retstatus = GridJobStatus.Done;
        } else if ( rmStatus.equals(ResourceManagerJobStatus.EXIT) ||
                    rmStatus.equals(ResourceManagerJobStatus.ERROR)) {
            retstatus = GridJobStatus.Exit;
        } else {
            retstatus = null;
        }
        logger.debug("In translateJobStatus, return state ="+retstatus);
        return retstatus;
    }

    private boolean updateJobData(JobInfoType jobinfo) throws RemoteException{
        boolean changed = false;
        String tmprm = null;
        String tmpjob = null;
        int tmprmint = 1;
        int tmpjobint = 1;

        logger.debug("In updateJobData. Trying to acquire lock");
	Calendar time, tmprmTime;

        ResourceHome home = null;
        JobResource resource = null;
        try{
              home= JNDIHelper.getResourceHome("metascheduler/JobService");
        } catch(NamingException e) {
              logger.error("Get job ResourceHome error!" + e.getMessage());
        }
        try{
              resource = (JobResource)home.find(m_key);
        } catch(ResourceException e) {
              logger.error(e.getMessage());
        }

        resource.acquire();

        try {
            // queueName
            tmpjob = jobdata.getRmQueueName();
            tmprm = jobinfo.getQueue();
            if ( (tmpjob != null) && (tmprm != null)) {
                if (tmpjob.equals(tmprm)) {
                    jobdata.setRmQueueName(tmprm);
                    changed = true;
                }
            } else if ( tmprm != null ) {
               jobdata.setRmQueueName(tmprm);
               changed = true;
            }
        } catch(Exception e) {
            logger.debug("In updateJobData queueName. got exception " + e);
        }

        try {
            tmpjob = jobdata.getRunAs();
            tmprm = jobinfo.getUser();
            if ( (tmpjob != null) && (tmprm != null)) {
                if (tmpjob.equals(tmprm)) {
                    jobdata.setRunAs(tmprm);
                    changed = true;
                }
            } else if ( tmprm != null ) {
               jobdata.setRunAs(tmprm);
               changed = true;
            }
        } catch(Exception e) {
            logger.debug("In updateJobData RunAs. got exception " + e);
        }

        try {
            tmpjob = jobdata.getProject();
            tmprm = jobinfo.getProject();
            if ( (tmpjob != null) && (tmprm != null)) {
                if (tmpjob.equals(tmprm)) {
                    jobdata.setProject(tmprm);
                    changed = true;
                }
            } else if ( tmprm != null ) {
               jobdata.setProject(tmprm);
               changed = true;
            }
        } catch(Exception e) {
            logger.debug("In updateJobData Project. got exception " + e);
        }

        try {
            tmpjob = jobdata.getRsvId();
            tmprm = jobinfo.getRsvId();
            if ( (tmpjob != null) && (tmprm != null)) {
                if (tmpjob.equals(tmprm)) {
                    jobdata.setRsvId(tmprm);
                    changed = true;
                }
            } else if ( tmprm != null ) {
               jobdata.setRsvId(tmprm);
               changed = true;
            }
        } catch(Exception e) {
            logger.debug("In updateJobData RsvId. got exception " + e);
        }

        try {
            tmpjobint = jobdata.getCount().intValue();
            tmprmint = jobinfo.getCount().intValue();
            if ( (tmpjobint != 1) && (tmprmint != 1)) {
                if (tmpjobint == tmprmint) {
                    jobdata.setCount(jobinfo.getCount());
                    changed = true;
                }
            } else if ( tmprmint != 1 ) {
               jobdata.setCount(jobinfo.getCount());
               changed = true;
            }
        } catch(Exception e) {
            logger.debug("In updateJobData Count. got exception " + e);
        }

        try {
            tmpjob = jobdata.getReason();
            tmprm = jobinfo.getReason();
            if ( (tmpjob != null) && (tmprm != null)) {
                if (tmpjob.equals(tmprm)) {
                    jobdata.setReason(tmprm);
                    changed = true;
                }
            } else if ( tmprm != null ) {
               jobdata.setReason(tmprm);
               changed = true;
            }
        } catch(Exception e) {
            logger.debug("In updateJobData Reason. got exception " + e);
        }

        try {
            tmpjob = jobdata.getExecutionHosts();
            tmprm = jobinfo.getExecutionHosts();
            if ( (tmpjob != null) && (tmprm != null)) {
                if (tmpjob.equals(tmprm)) {
                    jobdata.setExecutionHosts(tmprm);
                    changed = true;
                }
            } else if ( tmprm != null ) {
               jobdata.setExecutionHosts(tmprm);
               changed = true;
            }
        } catch(Exception e) {
            logger.debug("In updateJobData hosts. got exception " + e);
        }

        try {
            time = jobdata.getStartTime();
            tmprmTime = jobinfo.getStartTime();
            if ( (time != null) && (tmprmTime != null)) {
                if (tmpjob.equals(tmprmTime)) {
                    jobdata.setStartTime(tmprmTime);
                    changed = true;
                }
            } else if ( tmprmTime != null ) {
                jobdata.setStartTime(tmprmTime);
                changed = true;
            }
        } catch(Exception e) {
            logger.debug("In updateJobData starttime. got exception " + e);
        }

        logger.debug("In updateJobData. releasing lock ");
        resource.release();
        return changed;
    }

    public void run()  {
        // Call the resource manager to get job info.
        jobdata = null; 
        String jobinfo =null;
        boolean resourcePropertyChanged = false;
        ResourceManagerLsfPortType adapter = null;
        ResourceHome home = null;
	JobResource resource = null;
        try{
              home= JNDIHelper.getResourceHome("metascheduler/JobService");
        } catch(NamingException e) {
              logger.error("Get job ResourceHome error!" + e.getMessage());
        }
        try{
              resource = (JobResource)home.find(m_key);
        } catch(ResourceException e) {
              logger.error(e.getMessage());
	      System.out.println("Thread:" + Thread.currentThread());
	      Thread.currentThread().interrupt();
        }
	if (resource==null) {
	      logger.debug("The Job isn't exist, it has been destroied!");
	      this.cancel();
	      return;
	}

        logger.debug("Java memory in use (before free) :" + (Runtime.getRuntime().totalMemory() -
                     Runtime.getRuntime().freeMemory()));
        System.gc();
        logger.debug("Java memory in use (after free) :" + (Runtime.getRuntime().totalMemory() -
                     Runtime.getRuntime().freeMemory()));
        logger.debug("Pulling job information from resource manager");

	jobdata = resource.getJobdata();

	String cn = jobdata.getClusterName(); 
        try {
            adapter = JobImpl.getRMAdapter(JobImpl.getFactoryUri(),cn,m_credential);
        } catch (Exception e) {
            logger.error("Error get job info:"+e);
	}
        try {
            jobinfo = adapter.jobQuery(new JobQuery(jobdata.getRmJobId())).getJobInfo();
        } catch (Exception e) {
            logger.error("Error get job info:"+e);
        } 

	GridJobStatus currentState = null;
	       resource.acquire();
       	try {
       		currentState = resource.getJobdata().getState();
       	} catch (Exception e) {
          	logger.error("Error get job state"+e);
        }
        resource.release();

        logger.debug("Got job info from resource manager: " + jobinfo);
        if ( jobinfo != null) {
            try {
                JobInfoTypeHelper jobinfoHelper = new JobInfoTypeHelper(jobinfo);
                JobInfoType info = jobinfoHelper.getJob(jobdata.getRmJobId());

                resourcePropertyChanged = updateJobData(info);

                GridJobStatus newStatus = null;
                newStatus = translateJobStatus(info.getStatus());
                if (( currentState!= null ) && currentState.toString().equals(newStatus.toString())) {
                    // Update job service data if it's changed
                    if ( resourcePropertyChanged ) {
                       // m_job.setServiceData();
		        resource.acquire();
        		resource.getJobdata().setState(newStatus);
        		resource.release();
                    }
                } else {
		    resource.acquire();
        	    resource.getJobdata().setState(newStatus);
        	    resource.release();
                    if ( newStatus == GridJobStatus.Done) {
		        if ( currentState == GridJobStatus.Run_susp &&
             			resource.getDestType() != TYPE_RMADAPTER) {
            			// For GRAM job suspended means kill it
           			 // and restart it later so we need to
            			// ignore the exit status update.
        	    	} else {
	                        resource.acquire();
       		                resource.getJobdata().setState(GridJobStatus.PDone);
       		                resource.release();
        	    	}
                        logger.debug("Job finished, destroy job instance and exit thread");
                        this.cancel();
                    } else if ( newStatus == GridJobStatus.Exit) {
        		if ( currentState == GridJobStatus.Run_susp && 
				resource.getDestType() != TYPE_RMADAPTER) {
            		// For GRAM job suspended means kill it
            		// and restart it later so we need to
            		// ignore the exit status update.
        		} else {
	                        resource.acquire();
       		                resource.getJobdata().setState(GridJobStatus.PExit);
       		                resource.release();
        		}
                        logger.debug("Job exited, destroy job instance and exit thread");
                        this.cancel();
                    }
                }
            } catch (Exception e) {
                logger.debug("Got exception: " + e);
            }
        }
        jobinfo = null;
        logger.debug("Leaving JobInfoTask.run " );
    }
}
