package com.platform.metascheduler.impl;

import org.globus.wsrf.Resource;
import org.globus.wsrf.RemoveCallback;
import org.globus.wsrf.ResourceIdentifier;
import org.globus.wsrf.ResourceProperties;
import org.globus.wsrf.ResourceProperty;
import org.globus.wsrf.ResourcePropertySet;
import org.globus.wsrf.impl.SimpleResourcePropertySet;
import org.globus.wsrf.impl.SimpleResourceProperty;
import org.globus.wsrf.ResourceException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import javax.xml.namespace.QName;

import com.platform.metascheduler.job.JobConfigParamsType;
import com.platform.metascheduler.job.JobDataType;
import com.platform.metascheduler.queuing.RmFactoryType;
import org.apache.axis.message.addressing.EndpointReferenceType;

import org.globus.exec.client.GramJob;
import com.platform.metascheduler.impl.JobGramTask;

public class JobResource
		implements
			Resource,
			ResourceIdentifier,
			ResourceProperties,
			RemoveCallback {

	private ResourcePropertySet propSet;
	private Object key;
	private ResourceProperty jobdata;
	private ResourceProperty rsl;
	private ResourceProperty createuser;
	private ResourceProperty destType;
	private ResourceProperty rmtype;
	private ResourceProperty queueName;
	private ResourceProperty jobConfig;
	private ResourceProperty jobGramTask;
	private ResourceProperty gramJob;

    	private Thread m_owner = null;
    	private int m_lock_count = 0;
    	private static Log logger = LogFactory.getLog(JobResource.class);
	public static final String DEFAULT_RMTYPE = "RMFS";

	// GET/SETTERS
	// jobdata
	public JobDataType getJobdata() {
		return (JobDataType) this.jobdata.get(0);
	}
	public void setJobdata(JobDataType i_jobdata) {
		this.jobdata.set(0, i_jobdata);
	}
	// rsl
	public String getRsl() {
		return ((String) this.rsl.get(0));
	}
	public void setRsl(String i_rsl) {
		this.rsl.set(0, i_rsl);
	}
	// createuser 
	public String getCreateuser() {
		return ((String) this.createuser.get(0));
	}
	public void setCreateuser(String i_createuser) {
		this.createuser.set(0, i_createuser);
	}
	// destType
	public String getDestType() {
		return (String) this.destType.get(0);
	}
	public void setDestType(String i_destType) {
		this.destType.set(0, i_destType);
	}
	// rmtype 
	public RmFactoryType getRmtype() {
		return (RmFactoryType) this.rmtype.get(0);
	}
	public void setRmtype(RmFactoryType i_rmtype) {
		this.rmtype.set(0, i_rmtype);
	}
	// queueName
	public String getQueueName() {
		return (String) this.queueName.get(0);
	}
	public void setQueueName(String i_queueName) {
		this.queueName.set(0, i_queueName);
	}
	// jobconfig
	public JobConfigParamsType getJobConfig() {
		return (JobConfigParamsType) this.jobConfig.get(0);
	}
	public void setJobConfig(JobConfigParamsType i_jobConfig) {
		this.jobConfig.set(0, i_jobConfig);
	}
	// jobGramTask
	public JobGramTask getJobGramTask() {
		return (JobGramTask) this.jobGramTask.get(0);
	}
	public void setJobGramTask(JobGramTask i_jobGramTask) {
		this.jobGramTask.set(0, i_jobGramTask);
	}
	// gramJob
	public GramJob getGramJob() {
		return (GramJob) this.gramJob.get(0);
	}
	public void setGramJob(GramJob i_gramJob) {
		this.gramJob.set(0, i_gramJob);
	}

    	public synchronized void acquire()
   	{
       		boolean ret = false;
        
        	logger.debug("In acquire, lockcount="+m_lock_count);
        	while ( ret != true ) { 
            	if( m_owner == null )
            	{   
                	m_owner = Thread.currentThread();
                	m_lock_count = 1;
                	ret = true;
                	logger.debug("acquired lock, lockcount="+m_lock_count);
            	} else if( m_owner == Thread.currentThread() ) {   
                	++m_lock_count;
                	ret = true;
                	logger.debug("acquired lock, lockcount="+m_lock_count);
            	}
            	if ( !ret ) {
                	try {
                    		logger.debug("acquiring lock, waiting");
                    		this.wait(1000);
                	} catch (Exception e) {
                	}
            	}
        	} 
    	}

    	public synchronized void release()
    	{
        	logger.debug("Releasing lock");
        	if( m_owner == Thread.currentThread() && --m_lock_count <= 0 )
        	{   
            		m_owner = null;
            		logger.debug("Releasd lock, lockcount="+m_lock_count);
            		notify();
        	}
        	logger.debug("Lock released, lockcount="+m_lock_count);
    	}

	public Object create(JobDataType i_jobdata,String i_rsl,String i_createUser,JobConfigParamsType i_config) throws Exception {
		this.key = new Integer(hashCode());
		this.propSet = new SimpleResourcePropertySet(new QName(
			"http://www.platform.com/namespaces/2003/05/metascheduler/job",
			"JobResourceProperties"));

		try {
			this.jobdata = new SimpleResourceProperty(new QName(
			"http://www.platform.com/namespaces/2003/05/metascheduler/job",
			"Jobdata"));
			this.propSet.add(this.jobdata);
			this.jobdata.add(i_jobdata);

			this.rsl = new SimpleResourceProperty(new QName(
			"http://www.platform.com/namespaces/2003/05/metascheduler/job",
			"Rsl"));
			this.propSet.add(this.rsl);
			this.rsl.add(i_rsl);

			this.createuser = new SimpleResourceProperty(new QName(
			"http://www.platform.com/namespaces/2003/05/metascheduler/job",
			"Createuser"));
			this.propSet.add(this.createuser);
			this.createuser.add(i_createUser);

			this.destType = new SimpleResourceProperty(new QName(
			"http://www.platform.com/namespaces/2003/05/metascheduler/job",
			"DestType"));
			this.propSet.add(this.destType);
			this.destType.add("NONE");

			this.rmtype = new SimpleResourceProperty(new QName(
			"http://www.platform.com/namespaces/2003/05/metascheduler/job",
			"Rmtype"));
			this.propSet.add(this.rmtype);
			this.rmtype.add(RmFactoryType._RMFS);

			this.queueName = new SimpleResourceProperty(new QName(
			"http://www.platform.com/namespaces/2003/05/metascheduler/job",
			"QueueName"));
			this.propSet.add(this.queueName);
			this.queueName.add("NONE");

			this.jobConfig= new SimpleResourceProperty(new QName(
			"http://www.platform.com/namespaces/2003/05/metascheduler/job",
			"JobConfig"));
			this.propSet.add(this.jobConfig);
			this.jobConfig.add(i_config);

			this.jobGramTask= new SimpleResourceProperty(new QName(
			"http://www.platform.com/namespaces/2003/05/metascheduler/job",
			"JobGramTask"));
			this.propSet.add(this.jobGramTask);
			this.jobGramTask.add(null);

			this.gramJob= new SimpleResourceProperty(new QName(
			"http://www.platform.com/namespaces/2003/05/metascheduler/job",
			"GramJob"));
			this.propSet.add(this.gramJob);
			this.gramJob.add(null);
		} catch (Exception e) {
			throw new RuntimeException(e.getMessage());
		}
		return key;
	}

	public ResourcePropertySet getResourcePropertySet() {
		return this.propSet;
	}

	public Object getID() {
		return this.key;
	}
    	public void remove() throws ResourceException {
		logger.debug("resource removed!");
    	}
}
