package com.platform.metascheduler.impl;

import com.platform.metascheduler.queuing.QueuingPortType;
import com.platform.metascheduler.queuing.QueueDataType;
import com.platform.metascheduler.queuing.QueuingFaultType;
import com.platform.metascheduler.queuing.QueuingRequestType;
import com.platform.metascheduler.queuing.QueueConfigParamsType;
import com.platform.metascheduler.queuing.RmFactoryType;
import com.platform.metascheduler.queuing.DispatchDecision;
import com.platform.metascheduler.queuing.SchedDecisionType;
import com.platform.metascheduler.queuing.SchedDecision;
import com.platform.metascheduler.job.JobDataType;
import com.platform.metascheduler.job.JobPortType;
import com.platform.metascheduler.job.service.JobServiceAddressingLocator;
import com.platform.metascheduler.reservation.GridReservationType;
import com.platform.metascheduler.utils.MetaSchedulerConstants;
import com.platform.metascheduler.utils.ConfigParamsTypeHelper;
import com.platform.metascheduler.utils.JNDIHelper;
import com.platform.metascheduler.impl.schedPlugin;

import com.platform.metascheduler.queuing.ConfigParamsType;

//import org.globus.ogsa.impl.ogsi.GridServiceImpl;
//import org.globus.ogsa.ServiceProperties;
//import org.globus.ogsa.ServiceDataSet;
//import org.globus.ogsa.ServiceData;
//import org.globus.ogsa.ServiceDataAttributes;


import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


import java.util.Date;
import java.util.Vector;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Hashtable;
import java.net.URL;
import java.lang.Thread; 
import javax.xml.namespace.QName;
import java.lang.IllegalAccessException; 
import java.net.MalformedURLException; 
import java.lang.ClassNotFoundException;
import java.lang.InstantiationException; 
import java.lang.IllegalAccessException; 
import java.lang.SecurityException;
import java.rmi.RemoteException;
import java.util.Calendar;
import javax.xml.rpc.Stub;

import java.io.File;
import java.io.InputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;

import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import org.w3c.dom.Element;
import javax.naming.NamingException;

/************* wsrf  classes ****************/
import org.globus.wsrf.container.ServiceHost;
import org.globus.wsrf.ResourceContext;
import org.globus.wsrf.ResourceKey;
import org.globus.wsrf.utils.AddressingUtils;
import org.globus.wsrf.ResourceException;
import org.globus.wsrf.ResourceHome;
import org.oasis.wsrf.lifetime.Destroy;
import org.globus.wsrf.config.ConfigException;

import org.apache.axis.message.addressing.EndpointReferenceType;
import org.apache.axis.utils.XMLUtils;
/************* stubs classes ****************/
import com.platform.metascheduler.queuing.CreateResourceResponse;
import com.platform.metascheduler.queuing.CreateResource;
import com.platform.metascheduler.queuing.GetQueueConfigParams;
import com.platform.metascheduler.queuing.GetQueueConfigParamsResponse;
import com.platform.metascheduler.queuing.GetQueueData;
import com.platform.metascheduler.queuing.GetQueueDataResponse;
import com.platform.metascheduler.queuing.Submit;
import com.platform.metascheduler.queuing.SubmitResponse;
import com.platform.metascheduler.queuing.Open;
import com.platform.metascheduler.queuing.OpenResponse;
import com.platform.metascheduler.queuing.Close;
import com.platform.metascheduler.queuing.CloseResponse;
import com.platform.metascheduler.queuing.Remove;
import com.platform.metascheduler.queuing.RemoveResponse;
import com.platform.metascheduler.job.Start;
import com.platform.metascheduler.job.StartResponse;

/*************** GSI classes ******************/
import org.globus.axis.gsi.GSIConstants;
import org.globus.wsrf.impl.security.authentication.Constants;
import org.globus.wsrf.impl.security.authorization.HostAuthorization;
import org.globus.wsrf.impl.security.authorization.SelfAuthorization;
import org.globus.gsi.proxy.IgnoreProxyPolicyHandler;
import org.globus.wsrf.impl.security.authentication.secureconv.service.SecurityContext;
import org.globus.wsrf.impl.security.authentication.Constants;
import org.globus.wsrf.security.SecurityManager;
import org.ietf.jgss.GSSCredential;
import org.ietf.jgss.GSSContext;
import org.apache.axis.MessageContext;

/************* wsrf callback classes ****************/
import org.globus.wsrf.RemoveCallback;


/** 
 * Queuing service framework. It loads queue configuration passed from 
 * QueuingFactoryCallback and validates the parameters.  
 * <p>
 * It loads all configured scheduler plugins. The default plugin is loaded
 * no matter it is configured or not. For other plugins, they are invoked
 * in the order they are configured in the queue configuration file..
 * The plugin has to implement interface schedPlugin.
 * 
 * <p> 
 * <ul>
 * <li> Invoke schedInit()  after loading the plugin successfully.
 * <li> Invoke schedOrder() once a new job is submitted to the queue.
 * <li> Invoke schedMatch() during the scheduling session.
 * <li> Invoke schedPost()  before the scheduling session ends. This is after
 *      all scheduling are sent to job service instances.
 * </ul>
 *
 * @see QueuingPortType
 * @see schedPlugin
 */     
public class QueuingServiceImpl {
    private static Log            logger    = LogFactory.getLog(QueuingServiceImpl.class);
    private QueueConfigParamsType m_qconfig  = null;               // queue configuration 
    private QueuingThread         m_qThread = null;
    private Vector                m_schedPlugins = null;
    private String           	  m_configFile = null;
    private Document         	  m_configDoc = null;

    private static ConfigParamsType m_config = null;
//  m_queues save all queue name and queue resourcekey , name is key , resourcekey is value
    static Hashtable        m_queues = null;
//  m_threads save all schedThread and queue resourcekey , resourcekey is key, schedThread is value
    private static Hashtable        m_threads = null;
    private static String         m_handle  = null; 
    private static boolean	  m_inited  = false;

    private final int             DEF_SCHEDULE_INTERVAL = 30; 
    private final int             TIME_OUT              = 600000; // 10 minutes
    private final int             MIN_SCHEDULE_INTERVAL =  5; 
    private final int             MAX_SCHEDULE_INTERVAL = 600;  // 10 minutes
    private final int             DEF_THROTTLE          = 100;  // schedule throttle

    private static final String DEFAULT_NAMESPACE
        = "http://www.platform.com/namespaces/2003/05/metascheduler/queuing";
    private static final String CONFIG_FILE = "etc/metascheduler_config.xml";
    private static final String QUEUING_NAME = "metascheduler/QueuingService";
    private static final String QUEUING_FACTORY_CONFIG = "queuingFactoryConfig";
    private static final String DEFAULT_QUEUE = "DEFAULT";


    /**
     * Load the specified plugin class
     * 
     * @param plugin plugin name
     * 
     * @return schedPlugin if the load of the class is successful otherwise null.
     * 
     * @see loadPlugins
     */ 
    private schedPlugin loadPlugin(String plugin) { 
	if ( plugin == null) { 
	    logger.warn("plugin is null"); 
	    return (null);
	} 

	Class        definition = null; 
	schedPlugin  instance = null; 

	try { 
	    definition = Class.forName(plugin);
	    Class[]  interfaces = definition.getInterfaces( );
	    boolean  valid = false; 

	    for (int k=0; k < interfaces.length; k++) {
		String iname = interfaces[k].getName();
		
		valid = false;
		if ( iname.equalsIgnoreCase(MetaSchedulerConstants.SCHED_PLUGIN_INTERFACE)) { 
		    valid = true; 
		    break;
		}
	    }

	    if ( valid == true) { 
		instance = (schedPlugin) definition.newInstance();
		return (instance);
	    } else { 
		logger.error("Plugin " + plugin + " skipped: does not implement interface schedPlugin");
	    }
	} catch ( ClassNotFoundException e) { 
	    logger.error("Cannot load plugin: class not found: " + plugin);
	} catch ( InstantiationException e) { 
	    logger.error("Cannot instantiate instance for plugin: " + plugin);
	    
	} catch ( IllegalAccessException e) { 
	    logger.error("Cannot access plugin: " + plugin);
	}

	return (null);
    }

    /**
     * Load all plugins configured at the queue level. 
     * <p> 
     * If the plugin doesn't exist, or implement schePlugin interface
     * or cannot be instanciated, the plugin is removed from configuration.o
     * 
     * @see loadPlugin
     */
    private Vector loadPlugins() throws RemoteException{ 
	String []  plugins = this.m_qconfig.getPluginArray();
	schedPlugin  instance = null;
	String     defPlugin = MetaSchedulerConstants.SCHED_PLUGIN_DEFAULT; 
	Vector	   schedPlugins = new Vector();

	// Load the default plugin
	instance = loadPlugin(defPlugin); 
	if ( instance == null ) { 
	    // Cannot load default plugin 
	    logger.error("Queue " + this.m_handle + "exits: cannot load default plugin: " + defPlugin);
	    throw new RemoteException("Load default plugin error!");
/*	    try { 
		logger.debug("Queue:" + getResource().getConfig().getName() + "will be destroy!" );
		this.destroy(new Destroy()); 
		logger.debug("Queue destroied!" );
	    } catch ( RemoteException e) { 
		logger.error("Queue destroy error!" );
		throw e;
	    } */
	}

	schedPlugins.add(instance); 
	logger.debug("Default plugin " + defPlugin + " is loaded successfully"); 
	instance.schedInit(this.m_handle,m_qconfig); 
	
	if ( plugins.length <= 0) { 
	    logger.debug("No other plugin is defined: default plugin only");
	    return schedPlugins;
	} 

	// Load customer defined plugins
	for(int i=0; i < plugins.length; i++) { 
	    if ( plugins[i].equalsIgnoreCase(defPlugin)) { 
		// This makes sure that default plugin is not loaded more than once
		continue;
	    }
	    
	    instance = loadPlugin(plugins[i]);
	    if ( instance != null) { 
		schedPlugins.add(instance); 

		logger.debug("Plugin " + plugins[i] + " is loaded successfully"); 
		instance.schedInit(this.m_handle,m_qconfig); 
	    } 
	}

	// Update plugin in this.m_qconfig based on schedPlugins.
	String[]  newPluginArray = new String[schedPlugins.size()];
	int k=0;
	Iterator entry = schedPlugins.iterator(); 
	while ( entry.hasNext()) {
	    newPluginArray[k] = entry.next().getClass().getName();
	    k++;
	}
	this.m_qconfig.setPluginArray(newPluginArray);
	return schedPlugins;
    }

    private QueuingThread startThread(ResourceKey key ) { 
	
	// Build other parameters , ensure m_qconfig is valid
	logger.debug("build ScheduleInterval!");
	Integer tmp = this.m_qconfig.getScheduleInterval(); 
	if ( tmp != null) { 
	    int interval = tmp.intValue(); 
	    if ( interval < MIN_SCHEDULE_INTERVAL  || interval > MAX_SCHEDULE_INTERVAL) { 
		logger.debug("Ingore invalid queue scheduleInterval: " + interval); 
		this.m_qconfig.setScheduleInterval(new Integer(DEF_SCHEDULE_INTERVAL));
	    } 
	} else { 
	    // Not defined in the configuration: use default
	    this.m_qconfig.setScheduleInterval(new Integer(DEF_SCHEDULE_INTERVAL));
	} 

	logger.debug("build Throttle!");
	tmp = this.m_qconfig.getThrottle(); 
	if ( tmp != null) { 
	    int throttle = tmp.intValue(); 
	    if ( throttle < 0  ) {
		logger.info("Ingore invalid queue throttle: " + throttle); 
		this.m_qconfig.setThrottle(new Integer(DEF_THROTTLE));
	    } 
	} else { 
	    // Not defined in the configuration: use default
	    this.m_qconfig.setThrottle(new Integer(DEF_THROTTLE));
	} 

	String [] plugins = this.m_qconfig.getPluginArray(); 
	for(int k=0; k < plugins.length; k++) { 
	    logger.info("processConfig: Plugin " + k + " : " + plugins[k]);
	}
	logger.info("processConfig: scheduleInterval: " + this.m_qconfig.getScheduleInterval()); 

	// get client credential ,pass it to thread,
	// because the thread can't start job with credential
        MessageContext ctx = MessageContext.getCurrentContext();
        SecurityContext sec = (SecurityContext)ctx.getProperty(Constants.CONTEXT);
        GSSCredential credential = null;
        try{
              credential = sec.getDelegatedCredential();
        }catch (Exception e) {
              logger.debug(e.getMessage());
        }

	// Start the scheduling thread
	logger.debug("start scheduler thread!");

	// Save resource key to Thread	
	logger.debug("Ready to create schedThread, key is :" + (Integer)key.getValue());
        if ( credential != null) {
                try {
			logger.debug("Ready to create schedThread, credential is :" + credential.getName().toString());
                } catch (org.ietf.jgss.GSSException e) { }
        } else{
                logger.debug("client dosen't have credential !");
        }
	m_qThread = new QueuingThread(key,credential); 
	m_qThread.start();
	logger.debug("start scheduler ok!");
	return m_qThread;
    }

/*    public void postCreate(GridContext context)	throws GridServiceException  {
	super.postCreate(context);

	// Handle is available after postCreate
	this.m_handle = (String)this.getProperty(ServiceProperties.HANDLE);
	logger.debug("Queue handle is: " + this.m_handle); 

	this.processConfig(); 

	SecurityManager manager = SecurityManager.getManager();
    	manager.setServiceOwnerFromContext(this);

	this.createServiceData(); 
    }
*/

    /** 
     * Handle a job submission requestion from job service. 
     * <p>
     * Upon receiving a new job, queuing service invokes schedOrder()
     * method of all plugins. 
     *
     * @return job submission status. "okay" if submission is 
     * successful. 
     *
     * @see QueuingRequestType
     */
    public synchronized SubmitResponse submit(Submit request)
	throws RemoteException, QueuingFaultType {
	QueuingRequestType submitRequest = request.getSubmitRequest();
	logger.debug("submit a job:");

	EndpointReferenceType jobEpr = submitRequest.getJobEpr();
	if (jobEpr != null) {
		logger.debug("job epr: " + jobEpr);
	}
	if (submitRequest.getRsvEpr() != null) { 
		logger.debug("rsv handle: " + submitRequest.getRsvEpr());
	}

	// make sure the element doesn't exist before
	if ( getResource().getJobs().contains(jobEpr)) { 
	    logger.debug("job already in the queue: " + jobEpr.getAddress());
	    return (new SubmitResponse("failed"));
	} 

	// Add the new job to the end of the Vector and then order
	// it by invoking scheduler plugin
	getResource().acquire();
	getResource().addJob(jobEpr);
	getResource().release();
	Iterator entry = getResource().getSchedplugins().iterator(); 
	logger.info("schedOrder jobs starts: job number =" + getResource().getJobs().size());
	while ( entry.hasNext() && getResource().getJobs().size() > 1) { 
	    schedPlugin plugin = (schedPlugin ) entry.next(); 
	    try { 
		logger.debug("Invoke " + plugin.getClass().getName() + ".schedOrder"); 
		getResource().acquire();
		getResource().setJobs(plugin.schedOrder(getResource().getJobs()));
		getResource().release();
	    } catch (RemoteException e) { 
		logger.error("plugin has error: " + e); 
		// remove the job from the list 
		getResource().acquire();
		getResource().getJobs().removeElement(jobEpr);
		getResource().release();
		break;
	    }
	}
	logger.debug("schedOrder jobs completes");
	return (new SubmitResponse("okay"));
    }


    /** 
     * Modify a job request. 
     * <p>
     * Not implemented yet.
     */
    public boolean modify(QueuingRequestType modifyRequest)
	throws RemoteException, QueuingFaultType {
	return true;
    }

    /** 
     * Handle a job removal from queue request. Upon receiving this request,
     * queuing service removes the job from its internal job list.
     * 
     * @throws RemoteException
     * @throws QueuingFaultType
     * 
     * @return true if removal is successful.
     *
     * @see removeQuest
     */
    public synchronized RemoveResponse remove(Remove request)
	throws RemoteException, QueuingFaultType {
        QueuingRequestType removeRequest = request.getRemoveRequest();

	EndpointReferenceType jobEpr = removeRequest.getJobEpr();
	getResource().acquire();
	Iterator e = getResource().getJobs().iterator();
	getResource().release();
	while ( e.hasNext() ) {
            EndpointReferenceType epr = (EndpointReferenceType) e.next();
	    if ( epr.toString().equalsIgnoreCase(jobEpr.toString())) { 
		logger.debug("Job is removed from queue");
		getResource().acquire();
		getResource().removeJob(jobEpr); 
		getResource().release();
		return (new RemoveResponse(true));
	    }
        }

	logger.debug("Cannot find job in the queue: " + jobEpr);
	return new RemoveResponse(false);
    }

    /**
     * Close a queue. 
     * <p>
     * Not implemented yet.
     */
    public CloseResponse close(Close request) throws RemoteException, QueuingFaultType {
	return new CloseResponse(true);
    }

    /**
     * Open a queue. 
     * <p>
     * Not implemented yet.
     */
    public OpenResponse open(Open request) throws RemoteException, QueuingFaultType {
	return new OpenResponse(true);
    }

    /**
     * Get queue data. 
     *
     * @throws RemoteException
     * @throws QueuingFaultType
     * @return QueueDataType
     * @see QueueDataType
     */
    public  GetQueueDataResponse getQueueData( GetQueueData request) throws RemoteException, QueuingFaultType {
	QueueDataType qData = new QueueDataType(); 

	QueuingResource resource = this.getResource();

	qData.setName(resource.getConfig().getName());
	qData.setStatus(resource.getStatus());
	GetQueueDataResponse response = new GetQueueDataResponse(qData);
        return response;


    }

    /**
     * Get queue configuration information. 
     *  
     * @throws RemoteException
     * @throws QueuingFaultType
     * 
     * @return getQueueConfigParams
     * @see getQueueConfigParams
     * 
     */
    public  GetQueueConfigParamsResponse getQueueConfigParams(GetQueueConfigParams request) 
	throws RemoteException, QueuingFaultType {
	QueuingResource resource = this.getResource();
         GetQueueConfigParamsResponse response = new GetQueueConfigParamsResponse(resource.getConfig());
	logger.debug("ready to invoke test "+ resource.getConfig().getCommunityGisHandle());
	//this.test(resource.getConfig().getCommunityGisHandle());
        return response ;
    }

    class QueuingThread extends Thread { 
	Log  logger   = null;
	int  interval = 0;
	String  name  = null;
	ResourceKey queuekey = null;
	QueuingResource resource = null;
	GSSCredential credential = null;

	public QueuingThread( ResourceKey key, GSSCredential clientcredential) { 
	    logger =  QueuingServiceImpl.this.logger; 
	    queuekey = key;
	    credential = clientcredential;
	    logger.debug("QueuingThread " + name + " is created: interval is " + interval);
	} 
	
	public QueuingThread(int t, ResourceKey key, GSSCredential clientcredential) { 
	    logger =  QueuingServiceImpl.this.logger; 
	    interval = t;
	    queuekey = key;
	    credential = clientcredential;
	    logger.debug("QueuingThread " + name + " is created: interval is " + interval);
	} 

	public void run() {
	    boolean done = false; 
	    if (resource == null) {
		logger.debug("In schedThread: get Queue Resource!");
		ResourceHome home = null;
		try{
			home= JNDIHelper.getResourceHome(QueuingServiceImpl.QUEUING_NAME);
		} catch(NamingException e) {
			logger.error("Get Queue ResourceHome error!" + e.getMessage());
		}
		try{
			resource = (QueuingResource)home.find(queuekey);
		} catch(ResourceException e) {
			logger.error(e.getMessage());
		}
		logger.debug("In schedThread: get Queue Resource successfully!");
		name = resource.getConfig().getName().getLocalPart();
		interval = resource.getConfig().getScheduleInterval().intValue();
	    }
	    
	    logger.debug("QueuingThread " + name + " is started");
	    while ( !done) { 
		try { 
		/*comment this line because m_schedplugin hasn't be setup*/
		    if (schedule() < 0) { 
			/* something wrong during scheduling: exit */
			break;
		    }
		    sleep(interval * 1000);
                    logger.debug("sched thread : sleep "+ interval + "seconds");
		} catch (InterruptedException e) { 
		    // nobody cares
	            logger.info("QueuingThread is interrupted");
		    done = true;
		}
	    }
	    
	    logger.error("QueuingThread is exiting");
	}

    // Only schedule the first job in the queue if the job number is more than one

        public synchronized int schedule() {
	
	int number = resource.getJobs().size(); 

	Date     time1 = Calendar.getInstance().getTime();

	logger.debug("Queue " + resource.getConfig().getName().getLocalPart() 
			+ " scheduling cycle begins at " + time1.toString() + " with " + number + " jobs.");

	// Get job information and then call each plugin to get 
	// scheduling decision and then execute the decision if exist.
	Vector decisions = new Vector();
	resource.acquire();
	Iterator  entry = resource.getJobs().iterator();
	resource.release();
	while ( entry.hasNext()) { 
	    EndpointReferenceType epr = (EndpointReferenceType) entry.next(); 
	    SchedDecision jobDecision = new SchedDecision(); 
	    jobDecision.setJobEpr(epr); 
	    decisions.add(jobDecision); 
	}

	System.out.println("Begin to Match: jobs:" + decisions.size());

	// Match jobs
	entry = resource.getSchedplugins().iterator(); 

	logger.info("schedMatch all jobs");
	while ( entry.hasNext() && resource.getJobs().size() > 0) { 
	    schedPlugin plugin = (schedPlugin ) entry.next(); 
	    try { 
		logger.debug("Invoke " + plugin.getClass().getName() + ".schedMatch"); 
		decisions = plugin.schedMatch(decisions);
	    } catch ( RemoteException e) { 
		logger.error("schedMatch failure: plugin " + plugin.getClass());
	    }
	}

	System.out.println("End with Match: jobs:" + decisions.size());

	// Go through decision list and dispatch all jobs in the list
	Iterator elem = decisions.iterator();
	while ( elem.hasNext() ) {
    	    SchedDecision decision = (SchedDecision)  elem.next(); 
    	    SchedDecisionType type = decision.getDecisionType(); 

	    if ( type == null) { 
		continue; 
	    }

    	    if  ( type.equals(SchedDecisionType.DISPATCH)) { 
        	// A dispatch decision: dispatch the job
		EndpointReferenceType endpoint  = decision.getJobEpr(); 
		DispatchDecision disp = decision.getDispDecision(0); 

		RmFactoryType  rmType = disp.getType();	
		String cluster = null; 
		String factory = disp.getFactory(); 
		if ( disp.getCluster() != null) { 
		    cluster = disp.getCluster(0); 
	        }
		String rmRsvId = disp.getRmRsvId(); 

	        logger.debug("Dispatch decision: job " + endpoint);	
	 	logger.debug("rmType: " + rmType);
		logger.debug("factory: " + factory);
		logger.debug("cluster: " + cluster);
		logger.debug("rmRsvId: " + rmRsvId);

		//fortransport-levelsecurity
 		org.globus.axis.util.Util.registerTransport();

	        try { 
		    JobServiceAddressingLocator locator = new JobServiceAddressingLocator();
		    JobPortType jobPort = locator.getJobPortTypePort(endpoint); 
		    if ( credential != null) {
    		    	try {
 				logger.debug("Credential is for identity : " + credential.getName().toString());
 				((Stub)jobPort)._setProperty(GSIConstants.GSI_CREDENTIALS, credential);
 		    	} catch (org.ietf.jgss.GSSException e) { }
 		    } else{
 			logger.debug("Thread hasn't get credential!!!");
 		    }

                    ((Stub)jobPort)._setProperty(Constants.GSI_SEC_CONV,
                                        Constants.SIGNATURE);
                    ((Stub)jobPort)._setProperty(GSIConstants.GSI_MODE,
                                        GSIConstants.GSI_MODE_LIMITED_DELEG);
                    ((Stub)jobPort)._setProperty(Constants.AUTHORIZATION,
                                        HostAuthorization.getInstance());
                    //((Stub)jobPort)._setProperty(Constants.GRIM_POLICY_HANDLER,
                    //                    new IgnoreProxyPolicyHandler());
		    ((org.apache.axis.client.Stub)jobPort).setTimeout(TIME_OUT);

		    jobPort.start(new Start(cluster, factory, rmRsvId, rmType.toString())); 
		    resource.acquire();
	            resource.removeJob(endpoint);
		    resource.release();
		    //jobPort.destroy(new org.oasis.wsrf.lifetime.Destroy());
	        } catch (RemoteException e) { 
                    String msg = e.getMessage();
		    logger.error("Job service cannot start the job " 
				+  endpoint + " msg: " + e.getMessage());
		} catch (Exception e) {
		    e.printStackTrace();
		}
		
/*	         catch ( MalformedURLException e) { 
		    logger.error("Wrong URL for the job " + endpoint); 
		}*/
	    }
        }	

	// Post scheduling
	logger.info("call schedPost of all plugins");
	entry = resource.getSchedplugins().iterator(); 
	while ( entry.hasNext() ) { 
	    schedPlugin plugin = (schedPlugin ) entry.next(); 
	    plugin.schedPost( );
	}

	Date     time2 = Calendar.getInstance().getTime();
	logger.info("Scheduling cycle ends at " + time2.toString());

	return (0);
    }

	public void interrupt() throws SecurityException {
		logger.debug("schedThread interrupte!");
		super.interrupt();
	}
    }

    /**************************************/
    /* process etc/metascheduler/metascheduler_config.xml */
    /**************************************/

    public void init() throws RemoteException{
         ResourceContext ctx = null;
         QueuingResourceHome home = null;
         ResourceKey key = null;

         try {
         this.m_handle = "https://" + ServiceHost.getBaseURL().getHost()
			 + ":" + ServiceHost.getBaseURL().getPort()
			 + "/" + MetaSchedulerConstants.QUEUING_URL;

         logger.debug("QueuingService handle: " + this.m_handle);
         } catch ( Exception e ) { }
         this.m_queues = new Hashtable();
         this.m_threads = new Hashtable();
          try {
         	this.loadConfigDoc();
         	this.processConfigMeta();
         } catch (Exception e) {
		logger.info("error at process metascheduler_config.xml!");
		throw new RemoteException(e.getMessage());
         }
	 this.m_inited=true;
    }
    
    /******************************************************/
    /* load metascheduler_config.xml to m_configDoc      */
    /* invoked by init()			 	 */
    /******************************************************/

    private void loadConfigDoc() throws ConfigException{
	logger.debug("Enter loadConfigDoc");
 	InputStream input = null;
	String fileName = "etc/metascheduler/metascheduler-config.xml";
 	m_configFile = fileName;

 	try
 	{
            input = new FileInputStream(m_configFile);

            if (input == null) {
                throw new ConfigException("Could not load config file: "
                                          + m_configFile);
            }

            this.m_configDoc = XMLUtils.newDocument(input);
            logger.debug("loadConfigDoc(): m_configDoc loaded");

        } catch (FileNotFoundException e) {
            logger.debug("Configuration file (" + fileName
                         + "not found, using default values");
        } catch (Exception e) {
            throw new ConfigException(e);

        } finally {
            if (input != null) {
                try {
                    input.close();
                } catch (Exception e) {
                    throw new ConfigException(e);
                }
            }
        }
    }

    /******************************************************/
    /* read queuing part from m_configDoc		  */
    /* process to m_config				  */
    /* invoked by init()			 	  */
    /******************************************************/
    private void processConfigMeta() throws Exception {
        NodeList nodes = this.m_configDoc.getElementsByTagNameNS(this.DEFAULT_NAMESPACE,
                                QUEUING_FACTORY_CONFIG);

        int nodesLength = nodes.getLength();
	logger.debug("Enter processConfigMeta!");
        if (nodesLength == 0) {
            logger.warn("No queue factory configuration found: queuing service is disabled: no queue can be created");
            return;
        }

        // Process only the first QUEUING_FACTORY_CONFIG node
        Element configElement = (Element)nodes.item(0);

        logger.debug("Processing: "+ XMLUtils.ElementToString(configElement));
        this.m_config = ConfigParamsTypeHelper.getQueuingConfigParamsType(configElement, this.m_handle);

        if ( this.m_config == null) {
            // Add the default queue to queue factory service
            logger.info("No queue is configured: create default queue");
            QName queue = new QName(this.m_handle, DEFAULT_QUEUE);
            this.m_config = ConfigParamsTypeHelper.createDefaultQueuingFactoryConfig(queue, null);
        } else {
            logger.debug("Total configured queues: " + this.m_config.getQueueConfigArray().length);
        }

        return;
    } // END of processConfigMeta()

    /*
     * Create a resource which hasn't been assign value
    */
       public CreateResourceResponse createResource(CreateResource request)
			throws RemoteException {

		String name = request.getQueueName();
		CreateResourceResponse response = new CreateResourceResponse();
		Vector plugins = null;
		EndpointReferenceType epr = null;
		ResourceContext ctx = null;
		QueuingResourceHome home = null;
		ResourceKey key = null;

		if (!this.m_inited){
			this.init();
		}

		try {
			ctx = ResourceContext.getResourceContext();
		} catch (Exception e) {
			throw new RemoteException("",e);
		}

		String queue="{"+this.m_handle+"}"+name;
	        QName queue_name = QName.valueOf(queue);

	        // Check if the queue is created or not
		// if queue is create already , get the queue epr ,and return it
       		String queueStr = queue_name.getNamespaceURI() + "/" + queue_name.getLocalPart();
		logger.debug("New queuestr is:" + queueStr);
		logger.debug("m_queues length is:" + m_queues.size());

       		if ( m_queues.containsKey(queueStr) && m_queues.size()>0) {
       		    logger.debug(MetaSchedulerConstants.QUEUE_SERVICE_ERR_EXIST + ": " + queue_name);
		    key = (ResourceKey)m_queues.get(queueStr);
		    logger.debug("get Key!!!: " + (Integer)key.getValue());
		    try {
			epr = AddressingUtils.createEndpointReference(ctx, key);
			logger.debug(epr);
		    } catch (Exception e) {
			throw new RemoteException("", e);
		    }
		    response.setEndpointReference(epr);
		    response.setStatus(MetaSchedulerConstants.QUEUE_SERVICE_ERR_EXIST);
		    return response;
       		}

       		this.m_qconfig = getQueueConfig(queue_name);
		logger.debug("communityGis = " + m_qconfig.getCommunityGisHandle());

		// Load plugins to m_schedPlugins which handle all schedulers' references
		// and update m_qconfig to remove invalid plugins.
		// comment it because schedPluginDefault.java hasn't modified.
		plugins = this.loadPlugins(); 

		logger.debug("Begin createResource!");
		try {
			home = (QueuingResourceHome) ctx.getResourceHome();
			key = home.create(m_qconfig,plugins,"open:active");
			
		} catch (Exception e) {
			throw new RemoteException("", e);
		}
		try {
			epr = AddressingUtils.createEndpointReference(ctx, key);
		} catch (Exception e) {
			throw new RemoteException("", e);
		}
		response.setEndpointReference(epr);
		response.setStatus(MetaSchedulerConstants.QUEUE_CREATE_SUCCESS);
		// queueStr is key of hash table, the Resourcekey is value of hash table.
		// because we must get Resourcekey by queueStr

		logger.debug("m_queues add! value is :" + (Integer)key.getValue() +":key is" + queueStr);
		m_queues.put(queueStr,key);

		/* start sched thread */
		m_threads.put(key,this.startThread(key));
		
		return response;
	}

	/* get Resource reference , invoked when access RPs*/
    	private QueuingResource getResource() throws RemoteException {
		logger.debug("Enter getResrouce!");
		Object resource = null;
		try {
			resource = ResourceContext.getResourceContext().getResource();
		} catch (Exception e) {
			logger.info("getResource() error!");
			throw new RemoteException("", e);
		}
		QueuingResource qResource = (QueuingResource) resource;
		return qResource;
	}


    private QueueConfigParamsType getQueueConfig(QName queue_name) {
        if (this.m_config == null) {
	    logger.info("m_config = null!");
            return null;
        }

        QueueConfigParamsType[] queueConfigArray = this.m_config.getQueueConfigArray();
        if (queueConfigArray == null) {
	    logger.info("m_config.queueConfigArray = null!");
            return null;
        }

        logger.debug("Number of queues: " + queueConfigArray.length);
        for (int i=0; i<queueConfigArray.length; i++) {
            QueueConfigParamsType queueConfig = queueConfigArray[i];
            QName queueName = queueConfig.getName();

            logger.debug("Configured queue: " + queueName);
            if (queueName.equals(queue_name)) {
                return (queueConfig);
            }
        }
        return null;
    }
    // for multi hosts co-work test , should be removed in the future
    private void test(String gis) {
	try {	
		com.platform.metascheduler.utils.MetaSchedulerHelper.getRemoteRM(gis);
	} catch(RemoteException e) {
		logger.error("In test!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
		logger.error(e.getMessage());
	}
    }
}
