package com.platform.metascheduler.impl; 

import java.util.Vector;
/*import org.globus.ogsa.GridServiceException;*/
import java.rmi.RemoteException;
import com.platform.metascheduler.impl.QueuingServiceImpl;
import com.platform.metascheduler.queuing.QueueConfigParamsType;

/**
 * Interface class for scheduler plugin. All scheduler plugins configured for queues
 * are loaded and executed during each scheduling cycle of queuing service
 * interfaces. 
 * <p> 
 * Every scheduling plugin has to implement those four interfaces so that queuing
 * service can load the plugin and invoke methods during its job scheduling.
 */
public interface schedPlugin { 

    /**
     * Scheduler initialization method. Suggest printing out plugin version. 
     * This method is invoked only once after queuing service
     * loads the plugin successfully.
     * 
     * @param queueURL  string to queue service instance
     * @param queueConfig queue config information
     * @return 0 if initialization is successful. Queuing service doesn't
     *  take any actions even if non-zero value is returned.
     *
     * @see QueueServiceImpl 
     */
    int    schedInit(String queueURL,QueueConfigParamsType queueConfig);

    /**
     * Order all jobs in the vector. The default schedule plugin 
     * already sorts all jobs based on their queuing time. This
     * method is invoked whenever a new job instance is submitted
     * to the queue.
     * 
     * @param jobs  vector of handles to each job service instance
     *  
     * @return a new vector of job handles
     *
     * @throws GridServiceException grid sevice exception 
     * @see schedPluginDefault
     */
    /*Vector schedOrder(Vector jobs) throws GridServiceException;     */
    Vector schedOrder(Vector jobs) throws RemoteException;

    /**
     * Match jobs in the vector with resource manager(s) and gram 
     * factories. The default schedule plugin creates job decision.
     * Customized plugin can modify the decision or remove the decision.
     * 
     * @param jobDecision  vector of jobDecisions made by previous plugin.
     *
     * @return a new vector of job decisions
     *
     * @throws GridServiceException grid sevice exception
     * 
     * @see schedPluginDefault
     * @see SchedDecision
     * @see SchedDecisionType
     */
    /*Vector schedMatch(Vector jobDecisions) throws GridServiceException;   */
    Vector schedMatch(Vector jobDecisions) throws RemoteException;

    /**
     * Post scheduling method. Use this method to clean up plugin specific
     * date created during each scheduling session. 
     * <p>
     * This method is invoked every scheduling cycle by queuing service after 
     * invoking scheMatch method of all scheduling plugins.
     *
     * @return 0 if method is successful. Queuing service doesn't
     *  take any actions even if non-zero value is returned.
     *
     * @see QueueServiceImpl
     */
    int    schedPost();                 
} 

