package com.platform.metascheduler.impl;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.util.Vector;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.StringTokenizer;
import java.util.Hashtable;
import java.util.ArrayList;
import java.util.Calendar;
import java.rmi.RemoteException;
import java.net.MalformedURLException;
import java.net.URL; 
import javax.xml.namespace.QName;

import com.platform.metascheduler.utils.JobDataHelper;
import com.platform.metascheduler.utils.MetaSchedulerConstants;
import com.platform.metascheduler.utils.MetaSchedulerHelper;
import com.platform.metascheduler.reservation.GridReservationType;
import com.platform.metascheduler.reservation.GridReservationStatus;
import com.platform.metascheduler.reservation.ReservationType;
import com.platform.metascheduler.queuing.RmFactoryType;
import com.platform.metascheduler.queuing.DispatchDecision;
import com.platform.metascheduler.queuing.QueueConfigParamsType;
import com.platform.metascheduler.queuing.SchedDecision;
import com.platform.metascheduler.queuing.SchedDecisionType;
import com.platform.metascheduler.job.JobPortType;
import com.platform.metascheduler.job.JobDataType;

import com.platform.metascheduler.resourcemanager.ResourceManagerInfoType;

import com.platform.metascheduler.impl.schedPlugin;
import com.platform.metascheduler.impl.QueuingServiceImpl;

import com.platform.metascheduler.job.service.JobServiceAddressingLocator;
import org.apache.axis.message.addressing.EndpointReferenceType;

import com.platform.metascheduler.job.GetJobDataType;
import com.platform.metascheduler.job.GetJobDataTypeResponse;

/** 
 * Default scheduler plugin.
 * <p> 
 * This plugin is always loaded by each queuing service no matter it is defined in the 
 * queue configuration file or not. Furthermore, it is the first scheduling plugin got 
 * invoked by queuing service framework because this plugin orders all jobs in FCSF order
 * and creates some basic scheduling decisions. 
 *
 */
public class schedPluginDefault implements schedPlugin { 
    private static Log         logger = LogFactory.getLog(schedPluginDefault.class);
    private static final int   version = 1;           // Plugin version
    private static int         lastClusterIndex = 0;  // last cluster that sends job to.	

    private String             m_handle = null;       // Handle to queue service
    private String             m_hosting    = null; 
    private Hashtable 	       m_clusterMap = null;   // Hash table stores all
						      // clusters to RM factory mapping
    private Hashtable          m_gramMap  = null;     // Hash table stores all 
						      // master gram factories
    private Hashtable 	       m_rmtClusterMap = null;   // Hash table stores all
						      // clusters to RM factory mapping
    private Hashtable          m_rmtGramMap  = null;     // Hash table stores all 
						      // master gram factories
    private String [] 	       m_clusters = null;     // Cluster in an array format contained
						      // in both m_clusterMap and m_gramMap
	

    /** 
     * Print current version of the plugin
     * @return 0 if initialization is successful.
     */
    public int schedInit(String url,QueueConfigParamsType queueConfig){
	logger.debug("Default plugin version " + version);
	m_handle = url;

	logger.debug("Queuing service handle: " + m_handle);
        StringTokenizer st = new StringTokenizer(m_handle, "/");
	if ( st.countTokens() > 2 ) { 
	    m_hosting = st.nextToken();                           // http:
	    m_hosting = m_hosting + "//" + st.nextToken() + "/";  // host_ip:portnumber
	} else { 
	    m_hosting = "https://localhost:8080/";
	}

	String     rmFactoryURL  = m_hosting + MetaSchedulerConstants.RESOURCE_MANAGER_FACTORY_URL;
	String	   rmFactory = "metascheduler/ResourceManagerFactoryService";
	logger.debug("rmFactory is :" + rmFactoryURL);
	ArrayList  rmFactories   = new ArrayList(); 
	ArrayList  gramFactories = new ArrayList(); 
	int        k;

	// Get rmFactories from local hosting environment
	try{
		m_clusterMap = MetaSchedulerHelper.getResourceManagerInfoMap(rmFactory);
	}catch (RemoteException e) {
		logger.error("getRMInfoMap Error:"+ e.getMessage());
	}
	if ( m_clusterMap == null || m_clusterMap.isEmpty()) { 
	    logger.info("No cluster is configured in ResourceManagerFactoryService");
        } else { 
	    rmFactories = MetaSchedulerHelper.getResourceManagerInfoArray(m_clusterMap);	    
	    for (k=0; k < rmFactories.size(); k++) { 
		    logger.debug("ResourceManagerFactoryService manages cluster: " 
				 + (String) rmFactories.get(k));
	    }
	}

	// Get gramFork service from local hosting environment as well.
        ResourceManagerInfoType tmpinfo = new ResourceManagerInfoType();
        tmpinfo.setName("GramFork");
        tmpinfo.setType("GramFork");
        tmpinfo.setVersion("");
	QName  tmpkey = new QName(m_hosting + MetaSchedulerConstants.GRAM_FORK_URL,"GramFork");
	//m_gramMap = MetaSchedulerHelper.getGramFactoryMap(m_hosting + MetaSchedulerConstants.CONTAINER_REGISTRY_SERVICE_URL); 
	m_gramMap = new Hashtable();
	m_gramMap.put(tmpkey,tmpinfo);
	if ( m_gramMap == null || m_gramMap.isEmpty()) { 
	    logger.info("No gram is configured in ResourceManagerFactoryService");
	} else {
	    gramFactories = MetaSchedulerHelper.getResourceManagerInfoArray(m_gramMap);
	    logger.debug("ResourceManagerFactoryService manages no gram factories");
	    for (k=0; k < gramFactories.size(); k++) { 
		logger.debug("ResourceManagerFactoryService manages gram factories: " 
			     + (String) gramFactories.get(k));
	    }
	}
	
	// Get remote rmFactories and gramFactories
	String      communityGisHandle = queueConfig.getCommunityGisHandle();
	Hashtable   remoteFactories = null;
	try{
		remoteFactories = MetaSchedulerHelper.getRemoteRM(communityGisHandle);
	} catch (RemoteException e) {
		logger.error("getRemoteRM Error:"+ e.getMessage());
	}
	ArrayList   rmtGramFactories = new ArrayList(); 
	ArrayList   rmtRmFactories   = new ArrayList(); 
	m_rmtClusterMap = new Hashtable(); 
	m_rmtGramMap    = new Hashtable();

	if ( remoteFactories != null && !remoteFactories.isEmpty()) { 
	    for (java.util.Enumeration e = remoteFactories.keys();
		 e.hasMoreElements() ;) {
		QName  key = (QName)e.nextElement();
		String  cn = key.getLocalPart();
		String  ns = key.getNamespaceURI();
		String  full = ns + ";" + cn; 

		Object info = remoteFactories.get(key);
		if ( cn.equals("GramFork") || cn.equals("GramPbs") || cn.equals("GramLsf")) { 
		    // Gram factories
		    logger.debug("Remote gram factory: " + full);
		    rmtGramFactories.add(full);
		    m_rmtGramMap.put(key, info);
		} else { 
		    logger.debug("Remote resource manager: " + full);
		    rmtRmFactories.add(full);
		    m_rmtClusterMap.put(key, info);
		}
	    }

	    for(java.util.Enumeration e = m_rmtGramMap.keys();
		e.hasMoreElements() ;) {
		QName  key = (QName)e.nextElement();
		logger.debug("rmtGramMap includes factory: " + key.toString());
	    }

	    for(java.util.Enumeration e = m_rmtClusterMap.keys();
		e.hasMoreElements() ;) {
		QName  key = (QName)e.nextElement();
		logger.debug("rmtClusterMap includes factory: " + key.toString());
	    }
        }
	
	// merge clusters and gramFactories into one array
	int size = rmFactories.size() + gramFactories.size() 
	           + rmtGramFactories.size() + rmtRmFactories.size(); 
	ArrayList allFactories = new ArrayList(size); 
	
	allFactories.addAll(rmFactories);
	allFactories.addAll(gramFactories);
	allFactories.addAll(rmtRmFactories);
	allFactories.addAll(rmtGramFactories);

	m_clusters = (String []) allFactories.toArray(new String[0]);
	for (k=0; k < m_clusters.length; k++) { 
	    logger.info("ResourceManagerFactoryService manages: " + m_clusters[k]);
	}
	
	// this.test(); 
	return (0);
    } 

    /** 
     * Order all jobs based on their submission time (FCFS). 
     * <p> 
     * We assume all jobs except the last one in the Vector are
     * sorted already. Just need to insert the last job into 
     * the vector based on its submission type.
     * <p> 
     * Binary search algorithm is used.
     * <p> 
     * This function is invoked by queuing service when a job 
     * is submitted to the queue.
     *
     * @return sorted jobs list based on FCFS
     */
    public Vector schedOrder(Vector jobs) throws RemoteException{
        logger.debug("schedPluginDefault: schedOrder is invoked");

	int size = jobs.size(); 

	if ( size <= 1) { 
	    // There is only one job: nothing to do
	    return (jobs); 
	}

	EndpointReferenceType epr = (EndpointReferenceType)jobs.lastElement(); 
	Calendar subTime; 
	long 	 subSec; 

 	// In the most case, new job is put at the end of the job list.
	// So try to compare with the last job. If current job's submisssion time
	// is after submssion time of the last job in the list, then nothing
	// need to be done. 
	try { 
            subSec  = this.getJobSubmissionTime(epr).getTimeInMillis();
	    long    tmpSec  = this.getJobSubmissionTime((EndpointReferenceType)jobs.elementAt(size-2)).getTimeInMillis();
	    if ( tmpSec < subSec) { 
	        return (jobs); 
	    }
	} catch (RemoteException e) { 
	    throw (e);
	} 

	// Reach here: need to find a position for the job. 

	// 1. Remove the job from the end of the list first.
	jobs.removeElementAt(size-1); 

	// 2. Find position in the existing job list using binary search
	// into the list
	size = jobs.size(); 
	int  low   = 0; 
	int  high  = size-1; 
	int  mid   = low; 

	while ( low < high ) { 
	    mid = (low + high) / 2;

	    long   seconds = 0;
	    try { 
		seconds = this.getJobSubmissionTime(epr).getTimeInMillis();	    
		if ( seconds < 0) { 
		    throw new RemoteException("Cannot get job submission time: " + epr);
		} 
	    } catch (RemoteException e) { 
		throw (e);
	    }

	    // Compare with current job's submission time 
	    if ( subSec < seconds ) { 
		// Search the first half
		high = mid - 1;
	    } else { // subSec >= seconds: search the second half
		low = mid + 1; 
	    } 
	}

	if ( high < 0) { 
	    // Make sure high > 0
	    high = 0; 
	}
	if ( low > size-1 ) { 
	    low = size - 1;
	}

	// 3. Find the insertion position
	int pos = size; // By default, the end
	try { 
	    EndpointReferenceType jobepr; 
	    long    lowSubTime, highSubTime; 

	    jobepr = (EndpointReferenceType) jobs.elementAt(low);
	    lowSubTime =  this.getJobSubmissionTime(jobepr).getTimeInMillis();	    

	    jobepr = (EndpointReferenceType) jobs.elementAt(high);
	    highSubTime = this.getJobSubmissionTime(jobepr).getTimeInMillis();

	    if ( subSec < lowSubTime) { 
		pos = low;
	    } else if ( subSec >= highSubTime) {
		pos = high+1;
	    } else {
		pos = mid;
	    }
	    
	} catch (RemoteException e) { 
	    throw (e);
	}

	// 4. Insert the job
	logger.debug("Insert job at pos " + pos);
	jobs.insertElementAt(epr, pos);

	return (jobs);
    } 

    /**
     * <p> 
     * Match jobs with resource managers. Two major fucntionalities:
     * <ul>
     * <li> Try to create a decision if the job doesn't have a decision. If
     *      the job can't be dispatched. Plugin notifies job services with reason
     *      and remove the job from decision list so that the job won't be considered
     *      by other plugins eithers. </li>
     * <li> Try to check if the decision, made by previous plugins, is valid. If
     *      yes, pass on; otherwise, notifies job services with reason and remove
     *      the job from decision list just like in previous situation.
     * </ul>

     * <p> 
     * For default plugin, it creates the initial decisions: it matches jobs 
     * with specified clusters/reservation. If the job doesn't specify cluster/
     * reservation, one cluster is chosen.  
     * 
     * @input decisions list of job decisions. If a job has no decision, 
     *  previous plugins don't know how to make a decision for the job; if the job
     *  has a decision, current plugin checks if the decision breaks its own policy. 
     *
     * @return job decision list
     */
    public Vector schedMatch(Vector decisions) throws RemoteException{ 
        logger.debug("schedPluginDefault: schedMatch is invoked");

	Iterator    entry = decisions.iterator(); 
	String      candCluster = null;
	int         index = -1; 
	int         numClusters = 0; 

	if ( m_clusters != null ) { 
	    numClusters = m_clusters.length; 
		for (int i=0;i< numClusters;i++) {
			logger.debug(m_clusters[i]);
		}
	    logger.debug("numberCluster=" + numClusters);
	    index       = lastClusterIndex;
            candCluster = m_clusters[index];
	    logger.info("Next cluster for dispatching jobs: " +  m_clusters[lastClusterIndex]);
	}

	while (entry.hasNext()) { 
	    SchedDecision schedDesc = (SchedDecision) entry.next(); 
	    JobDataType   jData     = this.getJobData(schedDesc.getJobEpr()); 
	    int           ret       = this.checkRsrcManager(jData); 
	    EndpointReferenceType epr = schedDesc.getJobEpr();

	    if ( ret < 0) { 
		// Invalid rsrc manager is defined: remove the job
		// from the decision list
		if ( jData != null) { 
		    logger.error("Job " + epr
		                 + " cannot be scheduled: invalid resource manager: " 
		                 +  jData.getClusterName());
		} 
		entry.remove( ); 
	    } else if ( ret > 0) { 
		// Valid resource manager: create dispatch decision
		String   defCluster = jData.getClusterName(); 
		logger.debug("cluster in JobData!" + defCluster);
		SchedDecisionType  type = schedDesc.getDecisionType(); 
		if ( type == null || !type.equals(SchedDecisionType.DISPATCH)) { 
		    String[] clusters = new String[1];
		    clusters[0]  = defCluster; 
			
		    schedDesc.setDecisionType(SchedDecisionType.DISPATCH);
                    DispatchDecision  dispDecision = this.createDispDecision(clusters);
                    DispatchDecision[] dispDecisionV = new DispatchDecision[1];
                    dispDecisionV[0] = dispDecision;

                    schedDesc.setDispDecision(dispDecisionV);
		    logger.debug("Dispatch decision (job with cluster): job " + epr + " to: " 
				 + dispDecision.getType().toString() + ":"
				 + dispDecision.getFactory() + ":"
				 + dispDecision.getCluster(0) + ":" 
				 + dispDecision.getRmRsvId());
		}
	    } else { 
		// Check if the job has a valid reservation id
		String id = jData.getGridRsvId();

		if ( id == null) { 
		    // Job has no cluster name or reservation 
		    logger.info("Job " + epr + " has no cluster/reservation: send to next available cluster");

		    // If rslRootName is metascheduler:job then
		    // The job can only be dispatched to RM Adapter not gram as
		    // gram doesn't understand metascheduler:job
		    // This requires adjusts of candCluster if necessary
		    String rslRootName = jData.getRslRootName(); 
		    logger.debug("new! rslRootName =" + rslRootName);
		    if ( rslRootName.equals("metascheduler:job")) { 
			boolean found = false;
			int     lastIndex  = index; 
			
			while ( found == false) { 
			    String tmp = m_clusters[index]; 
			    int semiColonIdx = tmp.indexOf(';');	    
			    String f = tmp.substring(0, semiColonIdx);
			    
			    if ( f.endsWith(MetaSchedulerConstants.GRAM_FORK_URL)
				 || f.endsWith(MetaSchedulerConstants.GRAM_PBS_URL)
				 || f.endsWith(MetaSchedulerConstants.GRAM_LSF_URL)) { 
				// gram factory: skip it
				index = ( index + 1) % numClusters; 
				if ( index == lastIndex) { 
				    // There is no RMFS in all factories
				    candCluster = null;
				    break;
				}
			    } else { 
				// gram factory
				found = true; 
				candCluster = tmp;
			    }
			}
			logger.debug("found?" + (found?"true ":"false ") + candCluster);
		    }

                    if ( candCluster != null ) {
                        int semiColonIdx2 = candCluster.indexOf(';');
                        String f2 = candCluster.substring(0, semiColonIdx2);
                        // gram job can't be dispatch to RMFS
                        if (rslRootName.equals("gram:job")) {
                                if(!f2.endsWith(MetaSchedulerConstants.RESOURCE_MANAGER_FACTORY_URL)) {
                                        String[]  clusters = new String [1];
                                        clusters[0] = candCluster;
                                        schedDesc.setDecisionType(SchedDecisionType.DISPATCH);
                                        DispatchDecision  dispDecision = this.createDispDecision(clusters);
                                        DispatchDecision[] dispDecisionV = new DispatchDecision[1];
                                        dispDecisionV[0] = dispDecision;
                                        schedDesc.setDispDecision(dispDecisionV);
                                        logger.debug("Dispatch decision (round-robin): job " + epr + " to: "
                                                + dispDecision.getType().toString() + ":"
                                                + dispDecision.getFactory() + ":"
                                                + dispDecision.getCluster(0));
                                }
                        } else {
                                String[]  clusters = new String [1];
                                clusters[0] = candCluster;
                                schedDesc.setDecisionType(SchedDecisionType.DISPATCH);
                                DispatchDecision  dispDecision = this.createDispDecision(clusters);
                                DispatchDecision[] dispDecisionV = new DispatchDecision[1];
                                dispDecisionV[0] = dispDecision;
                                schedDesc.setDispDecision(dispDecisionV);
                                logger.debug("Dispatch decision (round-robin): job " + epr + " to: "
                                        + dispDecision.getType().toString() + ":"
                                        + dispDecision.getFactory() + ":"
                                        + dispDecision.getCluster(0));
                        }
                        // Adjust candCluster to the next cluster
                        index = ( index + 1) % numClusters;
                        candCluster = m_clusters[index];
                    } else {
                        // No cluster available
                        entry.remove();
                    }

		} else { 
		    logger.info("Grid reservation id is: " + id);

		    // Reservation is defined by the job: try to dispatch the job
		    // to the cluster where the reservation is made
		    GridReservationType rsv = null;
		    try {
                    	rsv = this.getReservation(id);
		    } catch (RemoteException e) {
			logger.error("getReservation Error:" + e.getMessage());
		    }
                    if ( rsv == null ) {
                        logger.debug("Job " + epr + " cannot be scheduled: invalid reservation");
                        entry.remove();
                    } else { 
                        // Valid reservation ticket: create dispatch decision
                        // based on reservation information
			QName  rmRsvId = this.getFirstRmRsvFromRsvId(rsv); 
			String cluster = null; 
			String rsvTicketId = null; 

			if ( rmRsvId == null) { 
			    // Something is wrong with the reservation
			    entry.remove();
			    continue;
			}
			logger.info("Reservation: cluster: " + cluster + " rmRsvId: " + rmRsvId);

			// rsvId is in the format of http://.../rmfactoryservice;123@cluster
			String rmFactoryHandle = rmRsvId.getNamespaceURI(); 
			String localId = rmRsvId.getLocalPart();
			int    atIdx   = localId.indexOf('@');            

			if ( atIdx > 0) {     
			    // Provide both factory and cluster name makes it easier 
			    // for creating dispatch decision
			    cluster     = rmFactoryHandle + ";" + localId.substring(atIdx+1);
			    rsvTicketId = localId.substring(0, atIdx);
			}

			if ( cluster != null) { 
		 	    String[]  clusters = new String [1]; 
			    clusters[0] = cluster; 

                            schedDesc.setDecisionType(SchedDecisionType.DISPATCH);
                       	    DispatchDecision  dispDecision = this.createDispDecision(clusters, rsvTicketId);
                            DispatchDecision[] dispDecisionV = new DispatchDecision[1];
                            dispDecisionV[0] = dispDecision;

                            schedDesc.setDispDecision(dispDecisionV);
                            logger.debug("Dispatch decision (job with gridReservationId): job " + epr + " to: "
                                     + dispDecision.getType().toString() + ":"
                                     + dispDecision.getFactory() + ":"
                                     + dispDecision.getCluster(0) + ":" 
				     + dispDecision.getRmRsvId());
			} else { 
                            logger.debug("Job " + epr + " cannot be scheduled: invalid RM reservation");
			}
			   
		    }
		}
	    } 
	} 

	if ( index >= 0) { 
	   lastClusterIndex = index;
	}
	return (decisions);
    }  

    private DispatchDecision createDispDecision(String[] clusters, String rmRsvId) { 
	DispatchDecision disp = this.createDispDecision(clusters); 

	if ( disp != null) { 
	    disp.setRmRsvId(rmRsvId); 
	} 
	return (disp);
    } 

    // Create dispatch decision based on the first cluster information
    // We are using cluster array here because we might support
    // span-cluster parallel jobs in the future.
    private DispatchDecision createDispDecision(String[] clusters) { 
	if ( clusters == null ) { 
	    return (null); 
	} 

	if ( clusters[0] == null) { 
	    return (null); 
	} 
	logger.debug("In createDispDecision cluster =" + clusters[0]);

	// Find factory type based on the first cluster
	// The cluster name could be in three formats:
	//     - factory URL only 
	//            https://.../MasterLsfManagedFactoryService
	//            https://.../ResourceManagerFactoryService
	//     - localName like LSF51
	//     - fullName  like 
	//           https://.../ResourceManagerFactoryService;LSF51
	//           https://.../MasterLsfManagedFactoryService;LSF51
	String        tmp   = clusters[0]; 
	DispatchDecision disp = new DispatchDecision();
	int semiColonIdx = tmp.indexOf(';');
	
	String factory = null;
	String cluster = null; 
	if (semiColonIdx > 0) {
	    // both factory and clusters are specified
	    factory = tmp.substring(0, semiColonIdx);
	    cluster = tmp.substring(semiColonIdx+1);
	} else { 
	    // either factory or cluster 		
	    if ( tmp.startsWith("https://")) { 
		factory = tmp;
		cluster = null;
	    } else { 
		// set factory based on cluster
		cluster = tmp;
		logger.debug("m_clusterMap size =" + m_clusterMap.size());

	        factory = MetaSchedulerHelper.getFactoryByClusterName(m_clusterMap, cluster);
		if ( factory == null) { 
	            factory = MetaSchedulerHelper.getFactoryByClusterName(m_rmtClusterMap, cluster);
		    if (factory == null) {
			// Set to default
		        factory = m_hosting + MetaSchedulerConstants.RESOURCE_MANAGER_FACTORY_URL;
		    }
		}
 	    }	
	}
	logger.debug("In createDispDecision factory =" + factory);

	// Set decision's RM type based on factory 
	if ( factory != null) { 
	    if (factory.indexOf(MetaSchedulerConstants.RESOURCE_MANAGER_FACTORY_URL) > 0) { 
		// Resource manager factory service URL
		disp.setType(RmFactoryType.RMFS);
	    } else { 
		// Gram URL
		if ( factory.endsWith(MetaSchedulerConstants.GRAM_FORK_URL)) { 
		    disp.setType(RmFactoryType.Fork);
		} else if ( factory.endsWith(MetaSchedulerConstants.GRAM_PBS_URL)) { 
		    disp.setType(RmFactoryType.PBS);
		} else if ( factory.endsWith(MetaSchedulerConstants.GRAM_LSF_URL)) { 
		    disp.setType(RmFactoryType.LSF);
		} else { 
		    disp.setType(RmFactoryType.Conder);
		}
		    
	    }
	} else { 
	    // factory is null: this shouldn't happen
	    disp.setType(RmFactoryType.RMFS);
	    factory = m_hosting + MetaSchedulerConstants.RESOURCE_MANAGER_FACTORY_URL;
	}

	// set factory
	disp.setFactory(factory);

	// set cluster
	clusters[0] = cluster; 
	disp.setCluster(clusters);

	return (disp);
    }

    /** 
     * Perform scheduling-cycle specific data clean up.
     * @return 0 if post operation is successful.
     */
    public int schedPost()  { 
        logger.debug("schedPluginDefault: schedPost is invoked");

        return(0);
    }

    // Given a cluster id, try to locate the factory service to access
    // the service. The factory service could be:
    // ResourceManagerFactoryService, 
    // MasterForkManagedJobFactoryService,
    // MasterLsfManagedJobFactoryService, 
    // MasterPbsManagedJobFactoryService
    // 
    // If the cluster is handled by ResourceManagerFactoryService, then 
    // return the factory service handle; otherwise, return cluster directly
    // because Queuing service has no idea which resource manager type
    // the cluster is using.
    // 
    // 
    private String getRmHandle(String cluster) { 
	if ( cluster.startsWith("https://") == true) { 
	    // URL is given, just return;
	    return (cluster); 
	} else { 
	    // Return URL for resource manger factory service on 
	    // local machine. 
	    return ( m_hosting + MetaSchedulerConstants.RESOURCE_MANAGER_FACTORY_URL);
	}
    }

    private Calendar getJobSubmissionTime(EndpointReferenceType epr) 
	throws RemoteException { 
	//fortransport-levelsecurity
	 org.globus.axis.util.Util.registerTransport();

	JobServiceAddressingLocator locator;
	JobPortType jobPort; 
	JobDataType jData = null;
	Calendar subTime = null;

 	try { 
	    locator = new JobServiceAddressingLocator();
	    jobPort = locator.getJobPortTypePort(epr);
	    jData = jobPort.getJobDataType(new GetJobDataType()).getValue(); 
	    
	    if ( jData == null) { 
		logger.error("Job doesn't exist: " + epr); 
		throw new RemoteException("Job doesn't exist: " + epr);
	    }
	    subTime = jData.getSubmissionTime();
	} catch (RemoteException e) {
	    logger.error("Cannot start the job " + epr);
	} catch (Exception e) {
	    e.printStackTrace();
	}
/*	} catch ( MalformedURLException e) {
	    logger.error("Wrong URL for the job " + epr);
	} */
	return (subTime);
    }

    private JobDataType getJobData(EndpointReferenceType epr) { 
	JobServiceAddressingLocator locator = null;
	JobPortType           jobPort = null; 
	JobDataType           jData = null;
	//fortransport-levelsecurity
 	org.globus.axis.util.Util.registerTransport();


	if ( epr == null) { 
	    return (null); 
	}

 	try { 
	    locator = new JobServiceAddressingLocator();
	    jobPort = locator.getJobPortTypePort(epr);
	    jData = jobPort.getJobDataType(new GetJobDataType()).getValue(); 
	    
	    if ( jData == null) { 
		logger.error("Job doesn't exist: " + epr); 
		throw new RemoteException("Job doesn't exist: " + epr);
	    }
	} catch (RemoteException e) {
	    logger.error("Cannot start the job " + epr);
	}catch(Exception e){
 	    e.printStackTrace();
 	}
/*	} catch ( MalformedURLException e) {
	    logger.error("Wrong URL for the job " + epr);
	}*/

	return jData;
    }

    /**
     * Check if resource manager specified by the job is valid or not. 
     * 
     * @input jData: job data
     * 
     * @return 0: if the job doesn't specify resource manager; 
     *         1: specified resource manger is valid
     *        <0: specified resource manger is invalid
     *         
     */
    private int checkRsrcManager(JobDataType jData) {
	if ( jData == null) { 
	    return (-1);
	}

	String cluster = jData.getClusterName();
	logger.debug("cluster=" + cluster);
	if ( cluster == null) { 
	    return (0);
	}

	// Check if the cluster name is defined from 
	// Resource Manager Factory Service
	if ( m_clusterMap == null && m_gramMap == null 
	     && m_rmtClusterMap == null && m_rmtGramMap == null ) { 
	    logger.error("Not cluster/gram is configured in resource manager factory service");
	    return (-2);
	} 

	// Check the handle in RM's cluster map
	logger.debug("m_clusterMap size="+m_clusterMap.size());
	if ( m_clusterMap != null && ! m_clusterMap.isEmpty()) { 
	    if (  MetaSchedulerHelper.getFactoryByClusterName(m_clusterMap, cluster) != null) { 
		logger.debug("Find cluster in local cluster map");
		return (1); 
	    }
	}
	
	// Check if it is in gramMap
	if ( m_gramMap != null && ! m_gramMap.isEmpty()) { 
	    if ( MetaSchedulerHelper.getFactoryByClusterName(m_gramMap, cluster) != null) {
		logger.debug("Find cluster in local gram map");
		return (1); 
	    }
	} 

	// Check the handle in remote cluster map
	
	if ( m_rmtClusterMap != null && ! m_rmtClusterMap.isEmpty()) { 
	    if ( MetaSchedulerHelper.getFactoryByClusterName(m_rmtClusterMap, cluster) != null) {
		logger.debug("Find cluster in remote cluster map");
		return (1); 
	    }
	} 
	
	// Check the handle in remote gram map
	if ( m_rmtGramMap != null && ! m_rmtGramMap.isEmpty()) { 
	    if ( MetaSchedulerHelper.getFactoryByClusterName(m_rmtGramMap, cluster) != null) {
		logger.debug("Find cluster in remote gram map");
		return (1); 
	    }
	} 
	
	logger.error("Specified cluster/gram doesn't exist: " + cluster);
	return (-3);
    }

    /**
     * Get reservation information based on reservationId
     * 
     * @param id reservation id assigned by reservation service,
     * which contains namespaceURI and local id                 
     * 
     * @return reservation if rsvId is valid otherwise null
     */
    private GridReservationType getReservation(String id) throws RemoteException{
        if ( id == null) {
            return (null);
        }

	String rsvHandle = null; 
	String rsvId = null; 

	// Reservation id is either ID_20030625_3 or {factoryURI}ID_20030625_3
	QName  rsvQName = QName.valueOf(id);
	rsvHandle = rsvQName.getNamespaceURI(); 
	rsvId      = rsvQName.getLocalPart(); 
	if ( rsvHandle == null) {
	    rsvHandle = m_hosting + MetaSchedulerConstants.RESERVATION_URL; 
	    rsvId = id; 
	}

	logger.debug("Call getReservationInfo " + " rsvHandle: " + rsvHandle
		     + " rsvId: " + rsvId); 
	GridReservationType rsvInfo = MetaSchedulerHelper.getReservationInfo(rsvHandle, rsvId); 
	if ( rsvInfo != null) {
            logger.debug("rsvInfo: id " + rsvInfo.getGridRsvID() + " user: " + rsvInfo.getUserID());
	}
	return (rsvInfo);
    }

    // If the reservation is not at RESERVED state, we consider the reservation
    // is not usable. null will be returned.
    //
    //
    private QName getFirstRmRsvFromRsvId(GridReservationType rsv) { 
        ReservationType[]  rsvTypeArray = rsv.getRmRsvs();

	if ( rsv == null) { 
	    return (null); 
	}

	if ( ! rsv.getStatus( ).equals(GridReservationStatus.RESERVED)) { 
	    return (null);
	}

        if ( rsvTypeArray != null && rsvTypeArray[0] != null) {
            // Only interested in the first reservation     
            QName  rsvId  = rsvTypeArray[0].getRsvID();     

	    return (rsvId); 
	}
        return (null);
    }
} 

