package com.platform.metascheduler.impl;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import com.platform.metascheduler.impl.schedPlugin;
import com.platform.metascheduler.queuing.SchedDecision;
import com.platform.metascheduler.queuing.QueueConfigParamsType;

import java.util.Vector; 
import java.util.Enumeration; 

/** 
 * Throttle scheduler plugin. Throttle job number for scheduling 
 * considerations. Only the first 100 jobs are considered for scheduling. 
 */
public class schedThrottle implements schedPlugin { 
    private static Log       logger = LogFactory.getLog(schedThrottle.class);
    private static final int m_version  = 1;       // Plugin version

    private int              m_throttle = 100;      // Should be 100. 5 is for testing.
    private String           m_handle   = null;    // Handle to its queue service

    
    /** 
     * Print current version of the plugin
     * @param url queuing service handle
     *
     * @return 0 if initialization is successful.
     */
    public int schedInit(String url,QueueConfigParamsType queueConfig) {
	logger.debug("schedThrottle: version " + m_version + " throttle = " + m_throttle);
	m_handle = url;

	Integer tmp = null; 

	tmp = queueConfig.getThrottle();

	// Queue framework has performed error checking already.
	// Double check here to make sure throttle value is valid
	if ( tmp != null) { 
	    int throttle = tmp.intValue(); 
	    
	    if ( throttle > 0) { 
		m_throttle = throttle;
		logger.info("Use configured throttle value " + m_throttle);
	    } else { 
		logger.warn("Incorrect throttle value: use default value " + m_throttle);
	    }
	} else { 
	    logger.info("Use default throttle value " + m_throttle);
	}
	
	logger.debug("schedThrottle init ok");
	return (0);
    } 
               
    /** 
     * Don't need to do anything. Just use FCFS job list created by 
     * default plugin.
     */
    public Vector schedOrder(Vector jobs) {
        logger.debug("schedThrottle: schedOrder is invoked");

	return (jobs);
    } 

    /**
     * Keep throttle jobs for scheduling considerations and remove all 
     * job decision afterwards.
     * @return job decision list
     */
    public Vector schedMatch(Vector schedDecisions)  { 
        logger.debug("schedThrottle: schedMatch is invoked");

	if ( schedDecisions.size() <= m_throttle) { 
	    logger.debug("Keep all " + schedDecisions.size() 
			 + " decisions: smaller than throttle " + m_throttle);
	    return schedDecisions;
	}

	logger.debug("Keep only the first " + m_throttle + " dispatch decisions");
	Vector      newSchedDecs = new Vector(); 
	Enumeration entry = schedDecisions.elements(); 
	int         i=0; 
	while ( entry.hasMoreElements() && i < m_throttle) { 
	    SchedDecision tmp = (SchedDecision) entry.nextElement(); 
	    logger.debug("in Throttle!!! type=" + tmp.getDecisionType());
	    if ( tmp != null) { 
	        if ((tmp.getDecisionType()==null )||(! tmp.getDecisionType().equals( com.platform.metascheduler.queuing.SchedDecisionType.DISPATCH))) { 
		    // Skip non dispatch decisions
		    continue;
		}
		// Find a dispatch decision
	        newSchedDecs.add(tmp);
	        i++; 
	    }
	} 

	return (newSchedDecs);
    }  

    /** 
     * Perform scheduling-cycle specific data clean up.
     * @return 0 if post operation is successful.
     */
    public int schedPost()  { 
        logger.debug("schedThrottle: schedPost is invoked");
        return(0);
    }
} 

