/**
 * JobInfo.java
 *
 * This class convert Job to a XML string which conforms to the 
 * metascheduler.xsd schema. 
 */
package com.platform.metascheduler.resourcemanager;

import java.io.IOException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.platform.metascheduler.utils.MetaSchedulerConstants;

import com.platform.drm.wm.Job;

public class JobInfo {
    static Log logger = LogFactory.getLog(JobInfo.class);

    public static final String DEFAULT_NS = "rmgr";
    public static final String JOBINFOLIST = "jobInfo";
    public static final String JOBINFO = "jobItem";
    public static final String JOBID = "jobId";
    public static final String QUEUE = "queue";
    public static final String PROJECT = "project";
    public static final String RSVID = "rsvId";
    public static final String MAXCPUTIME = "maxCpuTime";
    public static final String MAXWALLTIME = "maxWallTime";
    public static final String MAXMEMORY   = "maxMemory";
    public static final String STDIN = "stdin";
    public static final String STDOUT = "stdout";
    public static final String STDERR = "stderr";
    public static final String COUNT = "count";
    public static final String USER = "user";
    public static final String EXECUTABLE = "executable";
    public static final String STATUS = "status";
    public static final String SUBMITTIME = "submitTime";
    public static final String REASON = "reason";
    public static final String EXECINFO = "executionInfo";
    public static final String EXECHOSTS = "Hosts";
    public static final String STARTTIME = "startTime";

    public static final String RESOURCEMANAGER_NS = "http://com.platform.metascheduler/2003/05/resourcemanager";
    
    protected static final String HEADER
        = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
	+ "<metascheduler:metascheduler\n"
        + "    xmlns:metascheduler=\""
        + MetaSchedulerConstants.METASCHEDULER_RSL_NS + "\"\n"
	+ "    xmlns:rmgr=\"" + RESOURCEMANAGER_NS + "\"\n"
        + "    xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n"
        + "    xsi:schemaLocation=\"\n"
        + "         " + MetaSchedulerConstants.METASCHEDULER_RSL_NS + "\n"
        + "         " + "./metascheduler/metascheduler.xsd\n"
	+ "         " + RESOURCEMANAGER_NS + "\n"
        + "         " + "./metascheduler/resourcemanager.xsd\">\n";

    protected static final String FOOTER = "</metascheduler:metascheduler>\n";
    protected static final String INFO_HEADER = "<rmgr:rmgr>\n";
    protected static final String INFO_FOOTER = "</rmgr:rmgr>\n";
    protected static final String JOBINFO_HEADER = "<rmgr:jobInfo>\n";
    protected static final String JOBINFO_FOOTER = "</rmgr:jobInfo>\n";

    private Job [] m_jobInfoList = null;

    public JobInfo() {
	m_jobInfoList = null;
    }

    public JobInfo(Job job) {
	if (job == null) {
	    m_jobInfoList = null;
	} else {
	    m_jobInfoList = new Job[1];
	    m_jobInfoList[0] = job;
	}
    }

    public JobInfo(Job[] jobList) {
	if (jobList == null || jobList.length <= 0) {
	    m_jobInfoList = null;
	} else {
	    m_jobInfoList = jobList;
	}
    }

    public String toString () {

	if (m_jobInfoList == null || m_jobInfoList.length <= 0) {
	    return (null);
	}

        StringBuffer jobInfo = new StringBuffer();
	for (int i = 0; i < m_jobInfoList.length; i++){ 
	    jobInfo.append(m_jobInfoList[i].getAttributes().toString());
	}
	return (jobInfo.toString());
    }

    public String toXMLString() {

	if (m_jobInfoList == null || m_jobInfoList.length <= 0) {
	    return (null);
	}

	StringBuffer allJobInfo = new StringBuffer();
	for (int i = 0; i < m_jobInfoList.length; i++) {
	    String jobInfo = jobToXMLString(m_jobInfoList[i], 1);

	    if (jobInfo != null) {
		allJobInfo.append(jobInfo);
	    }
	}

	StringBuffer xmlRsl = new StringBuffer();	
	if (allJobInfo.length() > 0) {
	    xmlRsl.append(HEADER);
	    xmlRsl.append(INFO_HEADER);
	    xmlRsl.append(JOBINFO_HEADER);	    
	    xmlRsl.append(allJobInfo);
	    xmlRsl.append(JOBINFO_FOOTER);
	    xmlRsl.append(INFO_FOOTER);
	    xmlRsl.append(FOOTER);
	}

	return(xmlRsl.toString());
    }

    public String jobToXMLString(Job job, int indentLevel) {

	if (job == null) {
	    return null;
	}

	StringBuffer result = new StringBuffer();
	String  value = null;

	try {
	    value = job.getAttribute(Job.JobId).trim();
	    if (value != null) {
		result.append(getSimpleElementMarkup(JOBID, value, 
						     indentLevel + 1));	
	    } else {
		return null;
	    }
	} catch (Exception e) {
	    logger.debug(e.getMessage());	
	}    

	try {
	    value = job.getAttribute(Job.Queue).trim();
	    if (value != null) {
		result.append(getSimpleElementMarkup(QUEUE, value, 
						     indentLevel + 1));
	    }
	} catch (Exception e) {
	    logger.debug(e.getMessage());	
	}

	try {
	    value = job.getAttribute(Job.ProjectName).trim();
	    if (value != null) {
		result.append(getSimpleElementMarkup(PROJECT, value, 
						     indentLevel + 1));
	    }
	} catch (Exception e) {
	    logger.debug(e.getMessage());	
	}
	
	//value = job.getAttribute(Job.Submit_rsvId).trim();
	try {
	    value = job.getAttribute("submit_rsvId").trim();
	    if (value != null) {
		result.append(getSimpleElementMarkup(RSVID, value,
						     indentLevel + 1));
	    }
	} catch (Exception e) {
	    logger.debug(e.getMessage());	
	}
	
	// miss MAXCPUTIME, MAXWALLTIME and MAXMEMORY 
	
	try {
	    value = job.getAttribute("InputFile").trim();
	    if (value != null) {
		result.append(getSimpleElementMarkup(STDIN, value,
						     indentLevel + 1));
	    }
	} catch (Exception e) {
	    logger.debug(e.getMessage());	
	}

	try {
	    value = job.getAttribute("OutputFile").trim();
	    if (value != null) {
		result.append(getSimpleElementMarkup(STDOUT, value,
						     indentLevel + 1));
	    }
	} catch (Exception e) {
	    logger.debug(e.getMessage());	
	}

	try {
	    value = job.getAttribute("ErrorFile").trim();
	    if (value != null) {
		result.append(getSimpleElementMarkup(STDERR, value,
						     indentLevel + 1));
	    }
	} catch (Exception e) {
	    logger.debug(e.getMessage());	
	}

	try {
	    value = job.getAttribute(Job.Submit_maxNumProcessors).trim();
	    if (value != null) {
		result.append(getSimpleElementMarkup(COUNT, value,
						     indentLevel + 1));
	    }
	} catch (Exception e) {
	    logger.debug(e.getMessage());	
	}

	try {
	    value = job.getAttribute(Job.UserName).trim();
	    if (value != null) {
		result.append(getSimpleElementMarkup(USER, value,
						     indentLevel + 1));
	    }
	} catch (Exception e) {
	    logger.debug(e.getMessage());	
	}

	try {
	    value = job.getAttribute(Job.Command).trim();
	    if (value != null) {
		result.append(getSimpleElementMarkup(EXECUTABLE, value,
						     indentLevel + 1));
	    }
	} catch (Exception e) {
	    logger.debug(e.getMessage());	
	}

	try {
	    value = job.getAttribute(Job.Status).trim();
	    if (value != null) {
		result.append(getSimpleElementMarkup(STATUS, value,
						     indentLevel + 1));
	    }
	} catch (Exception e) {
	    logger.debug(e.getMessage());	
	}
       
	try {
	    value = job.getAttribute(Job.SubmitTime).trim();
	    if (value != null) {
		result.append(getSimpleElementMarkup(SUBMITTIME, value,
						     indentLevel + 1));
	    }
	} catch (Exception e) {
	    logger.debug(e.getMessage());	
	}
	 
	try {
	    value = job.getAttribute(Job.PendReason).trim();
	    if (value != null) {
		    result.append(getSimpleElementMarkup(REASON, value,
						 indentLevel + 1));
	    }
	} catch (Exception e) {
	    logger.debug(e.getMessage());	
	}
	
	value = jobRuntimeInfoToXMLString(job, indentLevel + 1);
	if (value != null) {
	    result.append(value);
	}

	if (result.length() > 0) { 
	    result.insert(0, getOpenTag(JOBINFO, indentLevel));
	    result.append(getCloseTag(JOBINFO, indentLevel));
	}

	return (result.toString());
    }

    private String jobRuntimeInfoToXMLString(Job job, int indentLevel)
    {
	if (job == null) {
	    return null;
	}

	StringBuffer result = new StringBuffer();
	String value = null;

	try {
	    value = job.getAttribute(Job.ExHosts).trim();
	    if (value != null) {
		result.append(getSimpleElementMarkup(EXECHOSTS, value,
						     indentLevel + 1));
	    }
	} catch (Exception e) {
	    logger.debug(e.getMessage());
	}

	try {
	    value = job.getAttribute(Job.StartTime).trim();
	    if (value != null) {
		result.append(getSimpleElementMarkup(STARTTIME, value,
						     indentLevel + 1));
	    }
	} catch (Exception e) {
	    logger.debug(e.getMessage());
	}
	
	if (result.length() > 0) {
	    result.insert(0, getOpenTag(EXECINFO, indentLevel));
	    result.append(getCloseTag(EXECINFO, indentLevel));
	}

	return (result.toString());
    }

    private String getIndent(int level)
    {
	StringBuffer space = new StringBuffer();
	for (int i = 0; i < level; i++) {
	    space.append("  ");
	}
	return (space.toString());
    }

    private String getOpenTag(String elementName, int indentLevel)
    {
	StringBuffer tag = new StringBuffer();
	String space = getIndent(indentLevel);

	tag.append("<" + DEFAULT_NS + ":" + elementName + ">");
	
	if (indentLevel > 0){
	    tag.insert(0, space);
	    tag.append("\n");
	}
	return (tag.toString());
    }

    private String getCloseTag(String elementName, int indentLevel)
    {
	StringBuffer tag = new StringBuffer();
	String space = getIndent(indentLevel);

	tag.append("</" + DEFAULT_NS + ":" + elementName + ">");
	
	if (indentLevel > 0){
	    tag.insert(0, space);
	    tag.append("\n");
	}
	return (tag.toString());
    }

    private StringBuffer getSimpleElementMarkup(String elementName, 
						String value,
						int indentLevel)
    {
	StringBuffer elementMarkup = new StringBuffer();

	elementMarkup.append("<");
	elementMarkup.append(DEFAULT_NS);
	elementMarkup.append(":");
	elementMarkup.append(elementName);
	elementMarkup.append(">");
	elementMarkup.append(value);
	elementMarkup.append("</");
	elementMarkup.append(DEFAULT_NS);
	elementMarkup.append(":");
	elementMarkup.append(elementName);
	elementMarkup.append(">");

	if (indentLevel > 0) {
	    String space = getIndent(indentLevel);
	    elementMarkup.insert(0, space);
	    elementMarkup.append("\n");
	}
	return (elementMarkup);
    }

}
