package com.platform.metascheduler.resourcemanager;

// Java
import java.io.IOException;
import java.util.Properties;
import java.util.Iterator;
import java.lang.StringBuffer;
import java.util.Date;
import java.util.Calendar;
import java.util.StringTokenizer;
import java.text.SimpleDateFormat;
import java.io.StringWriter;

// Logging 
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

// WebGui API
import com.platform.drm.rm.Request;
import com.platform.drm.rm.Response;
import com.platform.drm.rm.AdvRsv;
import com.platform.drm.wm.Queue;
import com.platform.drm.wm.Job;
import com.platform.drm.ServiceInvokerFactory;
import com.platform.drm.DRMException;
import com.platform.drm.ServiceInvoker;
import com.platform.drm.wm.WMJobMgtService;
import com.platform.drm.rm.WMHostMgtService;

import  com.platform.drm.wm.JobMgtServiceInvoker;
import  com.platform.drm.SimpleSOAPRequestParams;
import  com.platform.drm.SimpleSOAPRespondParams;
import  com.platform.drm.wm.JobFilter;

import com.platform.drm.wm.JobDefinition;
import com.platform.drm.util.Debug;

import com.platform.metascheduler.resourcemanager.JobInfoType;
import com.platform.metascheduler.resourcemanager.JobsInfoType;
import com.platform.metascheduler.resourcemanager.ResourceManagerJobStatus;
import com.platform.metascheduler.resourcemanager.RsvInfoType;
import com.platform.metascheduler.resourcemanager.RsvsInfoType;
import com.platform.metascheduler.resourcemanager.RsvHostInfoType;

import org.globus.wsrf.encoding.ObjectSerializer;

// XML parserWMJobMgtService
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Element;
import org.w3c.dom.Text;
import org.w3c.dom.Document;
import org.apache.axis.utils.XMLUtils;
import javax.xml.namespace.QName;

public class LsfCluster {
    static Log logger = LogFactory.getLog(LsfCluster.class.getName());
    private String  clusterName = null;
    private String  contactHost = null; 
    private int     contactPort;
    private int     contactTimeout;

     /**
     * Constructor 
     * @param name the name of the cluster.
     * @param host the name of the host running the webBroker for the cluster
     * @param port the listening port of the webBroker
     * @param timeout the timeout interval (in ms)
     **/

    public LsfCluster(String name, String host, int port, int timeout)
    {  
	this.clusterName = new String (name);
	this.contactHost = new String (host);
	this.contactPort = port;
	this.contactTimeout = timeout;

	// Turn on webGui API debug 
	if (logger.isDebugEnabled()) {
	    Debug.setDebugMode(true);
	    Debug.setDebugModules(true);
	    Debug.setDebugLevel(5);
	    Debug.addDebugModule("com.platform.drm");
	    Debug.addDebugModule("com.platform.drm.wm");
	    Debug.addDebugModule("com.platform.drm.rm");
	}
    }

     /**
     * Submit a job to the cluster.
     * @param jobDef the job specified in DRM job definition. 
     * @return 	     jobId in the cluster.
     * @throws       SecurityException, IOException, DRMException
     **/
    public String submitJob(JobDefinition jobDef)
        throws SecurityException,
               IOException,
               DRMException
    {    

	if (jobDef == null) {
	    logger.debug("null job definition");
	    throw new NullPointerException();
	}
	
	Job job = null;
	Queue queue = new Queue(null, contactHost, contactPort, 
				contactTimeout);
	try {
	    job = queue.submitJob(jobDef);
	} catch (Exception e) {
	    throw new DRMException (2000, e.getMessage());
	}

	return (job.getJobId());
    } // End of submitJob

     /**
     * Submit a reservation to the cluster.
     * @param advrsvDef the reservation specified in DRM reservation 
     *                  definition. 
     * @return 		reservation ID in the cluster.
     * @throws          SecurityException, IOException, DRMException
     **/
    public String addAdvRsv(AdvRsv advrsvDef)
        throws SecurityException,
               IOException,
               DRMException
    {

	if (advrsvDef == null) {
	    logger.debug("null reservation definition");
	    throw new NullPointerException();
	}

        Request request = new AdvRsvAddRequest(advrsvDef);
        Response response = null;
        ServiceInvoker invoker = null;

        try {
            invoker = ServiceInvokerFactory.getInstance().getServiceInvoker( 
			       "com.platform.drm.rm.WMHostMgtService");
            
	    ((WMHostMgtService)invoker).setContactInfo(contactHost, 
					       contactPort, contactTimeout);
	    response = (Response)invoker.invokeMethod("WM_AddAdvRsv",request);
        } catch(Exception e) {    
	    logger.debug("AddAdvRsv: " + e);   
            throw new DRMException(2000, e.getMessage());
        }

        if ( response == null ) {
	    logger.debug("AddAdvRsv: no response");
            return  null;
        }

	// parse the response
        NodeList list = response.getNodeList();
        Node node = list.item(0);
        Node rsvidNode = node.getFirstChild();
        if (rsvidNode != null && rsvidNode.getNodeName().equals("RsvId")) {
           String rsvId = getNodeValue(rsvidNode);
           return rsvId;
        }   

        return null;
    } // AddAdvRsv()

    private String getNodeValue( Node node )
    {
	String value = null;
	if ( node != null && node.getNodeType() == Node.ELEMENT_NODE )
	{
	    Node valueNode = node.getFirstChild();
	    if ( valueNode != null 
		 && valueNode.getNodeType() == Node.TEXT_NODE )
	    {
		value = valueNode.getNodeValue();
		logger.debug("getNodeValue: " + node.getNodeName() +
			     " = " + value );
	    }
	}
	return value;
    } // End of getNodeValue

     /**
     * Cancel a reservation from the cluster.
     * @param rsvId 	the reservation Id to be cancelled. 
     * @return 		true if successfully.
     * @throws          SecurityException, IOException, DRMException
     **/
    public boolean delAdvRsv(String rsvId) 
	throws SecurityException,
	       IOException,
	       DRMException {

        Request request = new AdvRsvDeleteRequest(rsvId);
        ServiceInvoker invoker = null;
	
        try {
            invoker = ServiceInvokerFactory.getInstance().getServiceInvoker(
			       "com.platform.drm.rm.WMHostMgtService");

	    ((WMHostMgtService)invoker).setContactInfo(contactHost, 
						       contactPort, 
						       contactTimeout);
	    invoker.invokeMethod("WM_DeleteAdvRsv",request);
        } catch ( Exception e ){
	    logger.debug("delAdvRsv: " + e);   
            throw new DRMException(2000, e.getMessage());
        }

	return (true);
    } // End of delAdvRsv

     /**
     * Retreive reservation information from the cluster.
     * @param rsvId 	the reservation Id to be queryed. 
     *                  If the Id is empty, get all reservation 
     *                  information existed.
     * @return 		the string of reservation info in XML format.
     *                  If it is null, the rsvID doesnot exist in the cluster
     * @throws          SecurityException, IOException, DRMException
     **/
    public String getAdvRsvs(String rsvId) 
	throws SecurityException,
	       IOException,
               DRMException
    {
        Request request = new Request();
        Response response = null;
        ServiceInvoker invoker = null;

	// Call webBroker to get reservation information
        try {
            invoker = ServiceInvokerFactory.getInstance().
                getServiceInvoker( "com.platform.drm.rm.WMHostMgtService" );
	    ((WMHostMgtService)invoker).setContactInfo(contactHost, 
						       contactPort, 
						       contactTimeout);
            response = (Response)invoker.invokeMethod("WM_ListAdvRsvs",
						      request);
        } catch(Exception e) {
            logger.debug("getAdvRsvs: " + e);
            throw new DRMException( 2000, e.getMessage() );
        }

        if ( response == null ) {
	    logger.debug("getAdvRsvs: no response got");
            return  null;
        }

	// Parse output
        NodeList list = response.getNodeList();
        Element  advrsvNodes = (Element)list.item(0);
        NodeList nodes = advrsvNodes.getElementsByTagName("NoReservation");
    
	// no reservation, return
        if ( nodes != null && nodes.getLength() > 0 ) {
	    logger.debug("getAdvRsvs: no reservation exist");
	    return null;
        }
 
        nodes = advrsvNodes.getChildNodes(); 
        if (nodes != null && nodes.getLength() > 0) {

	    // parse the response
            int curIdx = 0;
            AdvRsv [] advrsvDef = new AdvRsv[nodes.getLength()];
            for( int nodeIndex = 0; 
		 nodeIndex < nodes.getLength(); nodeIndex++ ) {
                Node node = nodes.item( nodeIndex );
                if ( node == null ||
                     node.getNodeType() != Node.ELEMENT_NODE ) {
                    continue;
                }
                try {
		    AdvRsv oneItem = new AdvRsv(); 
		    oneItem.updateAttributes((Element)node);
		    advrsvDef[nodeIndex] = oneItem;
                } catch(Exception e){
		    throw new DRMException( 2000, e.getMessage() );
                }
                curIdx++;
	    }

	    // convert to a string in XML format
	    RsvsInfoType rsvsInfo = null;
	    
	    if (rsvId == null || rsvId.trim().length() <= 0) {
		rsvsInfo = convertToRsvsInfo(advrsvDef);
	    } else {
		for (int kk = 0; kk < advrsvDef.length; kk++) {
		    if (advrsvDef[kk].getAttribute(AdvRsv.Id).equals(rsvId)) {
			AdvRsv [] tmp = new AdvRsv [1];
			tmp[0] = advrsvDef[kk];
			rsvsInfo = convertToRsvsInfo(tmp);
			break;
		    }
                }
            }

	    if (rsvsInfo != null) {
		try {
                    Element element = ObjectSerializer.toElement(rsvsInfo,new QName("","RsvInfo"));
		    Document doc = XMLUtils.newDocument();
		    Node node = doc.importNode(element, true);
		    doc.appendChild(node);
		    return (XMLUtils.DocumentToString(doc));
		} catch (Exception e) {
		    logger.debug("Exception:" + e);
		    return (null);
		}
	    } else {
		return (null);
	    }
	    
	    /*
	    ReservationInfo rsvList = null;
	    
	    if (rsvId == null || rsvId.trim().length() <= 0) {
		rsvList = new ReservationInfo(advrsvDef);
	    } else {
		for (int kk = 0; kk < advrsvDef.length; kk++) {
		    if (advrsvDef[kk].getAttribute(AdvRsv.Id).equals(rsvId)) {
			rsvList = new ReservationInfo(advrsvDef[kk]);
			break;
		    }
                }
            }
	    if (rsvList == null) {
		return null;
	    } else {
		return (rsvList.toXMLString());
	    }
	    */
	}

	return (null); 
    } // getAdvRsvs()


    private String converDocToPrettyXMLString(Document doc) {
	try {
	    StringWriter writer = new StringWriter();
	    XMLUtils.PrettyDocumentToWriter(doc, writer);
	    return (writer.toString());
	} catch (Exception e) {
	    return (null);
	}    
    }
    
    /**
     * Convert DRM AdvRsv to ResourceManager rsvInfoType 
     *
     * @param  rsv 	the DRM AdvRsv
     * @return rsvsInfoType  the object to hold reservation information
     **/
    private RsvsInfoType convertToRsvsInfo(AdvRsv [] rsvs) {

	logger.debug("Come to convertToRsvsInfo");

	if (rsvs.length <= 0) {
	    return null;
	}

	RsvInfoType [] rsvInfoArray = new RsvInfoType [rsvs.length];
	String value = null;
	int    index = 0;

	for (int i = 0; i < rsvs.length; i++){ 
	    RsvInfoType rsvInfo = null;
	    AdvRsv rsv = rsvs[i];

	    try {
		value = rsv.getAttribute(AdvRsv.Id).trim();
		if (value != null) {
		    rsvInfo = new RsvInfoType();
		    rsvInfo.setRsvId(value);
		} else {
		    continue;
		}
	    } catch (Exception e) {
		logger.debug("Exception:" + e);
		continue;
	    }    
	    
	    try {
		value = rsv.getAttribute(AdvRsv.ResReq).trim();
		if (value != null) {
		    rsvInfo.setRsrc(value);
		}
	    } catch (Exception e) {
		logger.debug("Exception:" + e);	
	    }

	    try {
		value = rsv.getAttribute(AdvRsv.AskedHosts).trim();
		if (value != null) {
		    StringTokenizer st = new StringTokenizer(value, " ");
		    int num = st.countTokens();
		    
		    RsvHostInfoType [] hosts = new RsvHostInfoType [num];

		    for (int kk = 0; kk < num; kk++) {
			String hostStr =  new String(st.nextToken());

			int  j = hostStr.indexOf(":");
			String  tmpStr = null;

			tmpStr = hostStr.substring(0, j);
			if (tmpStr != null) {
			    RsvHostInfoType host = new RsvHostInfoType();
			    host.setName(tmpStr);
			    
			    tmpStr = hostStr.substring(j + 1);
			    int number = Integer.parseInt(tmpStr);
			    if (number <= 0) {
				number = 1;
			    }
			    host.setNumber(new Integer(number));
			    hosts[kk] = host;
			}
		    }
		    rsvInfo.setHostInfo(hosts);
		}
	    } catch (Exception e) {
		logger.debug("Exception:" + e);	
	    }
	
	    try {
		value = rsv.getAttribute(AdvRsv.UserName).trim();
		if (value != null) {
		    rsvInfo.setUser(value);
		}
	    } catch (Exception e) {
		logger.debug("Exception:" + e);	
	    }
	
	    try {
		value = rsv.getAttribute(AdvRsv.Ncpus).trim();
		if (value != null) {
		    rsvInfo.setNumber(Long.valueOf(value));
		}
	    } catch (Exception e) {
		logger.debug("Exception:" + e);	
	    }

	    try {
		value = rsv.getAttribute(AdvRsv.TimeWindow).trim();

		if (value != null) {
		    int kk = value.indexOf("-");
		    String str1 = value.substring(0, kk);
		    
		    // assume it if current year ???
		    Calendar nowCal = Calendar.getInstance();
		    
		    Calendar startCal = Calendar.getInstance();
		    startCal.setTime(new SimpleDateFormat("MM/dd/kk/mm").parse(str1));
		    startCal.set(Calendar.YEAR, nowCal.get(Calendar.YEAR));
		    Calendar startTime = startCal;
		    rsvInfo.setBeginTime(startTime);

		    String str2 = value.substring(kk + 1);
		    Calendar endCal = Calendar.getInstance();
		    endCal.setTime(new SimpleDateFormat("MM/dd/kk/mm").parse(str2));
		    endCal.set(Calendar.YEAR, nowCal.get(Calendar.YEAR)); 
    		    Calendar endTime = endCal;
		    rsvInfo.setEndTime(endTime);
		    
		}
	    } catch (Exception e) {
		logger.debug("Exception:" + e);	
	    }

	    rsvInfoArray[index] = rsvInfo;
	    index ++;
	}

	if (rsvInfoArray.length > 0) {
	    RsvsInfoType ret = new RsvsInfoType();
	    ret.setRsvsInfo(rsvInfoArray);
	    return (ret);
	} else {
	    return (null);
	}
    }

     /**
     * Retreive a set of jobs from the cluster.
     * @param filter 	the filter used to determine what jobs to return.
     * @return 		an array of the jobs matching the filter 
     *			or null if no job found.
     **/
    public Job[] getJobs(JobFilter filter)
	throws SecurityException,
	       IOException,
               DRMException
    {
        SimpleSOAPRequestParams reqParams = new SimpleSOAPRequestParams();
        if (filter != null) {
            reqParams.addCompoundParam(filter.getId(), filter);
        } else {
            return null; // Dangerous to return all jobs
	}

        SimpleSOAPRespondParams rspParams = null;

        try
        {
            ServiceInvoker invoker = null;
	    invoker = new JobMgtServiceInvoker(contactHost,
                                contactPort, contactTimeout);
            rspParams = (SimpleSOAPRespondParams)
			invoker.invokeMethod(JobMgtServiceInvoker.WM_QUERYJOB, 
						reqParams );

            if (rspParams == null) {
                return null;
	    }

            Object[] objs = rspParams.getCompoundParams(Job.SOAP_ID);
            Job[] jobs = new Job[objs.length];
            for (int i = 0; i < objs.length; i++) {
                jobs[i] = (Job)objs[i];
	    }
            return (jobs);
	} catch (Exception e) {
	    throw new DRMException(2000, e.getMessage()); 		
	}
    }

    /**
     * Retreive a specific job form the cluster.
     *
     * @param jobId 	the native jobId to be retrieved
     * @return Job      reference to the job retrieved
     **/
    public String getJobById(String jobId)
	throws SecurityException,
	       IOException,
               DRMException
    {
        logger.trace("getJobById with " + jobId );

        JobFilter filter= new JobFilter();
	if (jobId == null || jobId.length() <= 0) {
	    filter.addField(Job.JobName, "ResourceManager");
	} else {
	    filter.addField(Job.JobId, jobId);
	}

	Job [] jobs = getJobs(filter);

	JobsInfoType jobInfo = convertToJobInfos(jobs);
	if (jobInfo != null) {
	    try {
                Element element = ObjectSerializer.toElement(jobInfo,new QName("","JobInfo"));
		Document doc = XMLUtils.newDocument();
		Node node = doc.importNode(element, true);
		doc.appendChild(node);
		return (XMLUtils.DocumentToString(doc));
	    } catch (Exception e) {
		logger.debug("Exception:" + e);
		return (null);
	    }
	} else {
	    return (null);
	}
	
	/* Do not use JobInfo to generate XML string
	JobInfo jobInfo = new JobInfo(jobs);

	return (jobInfo.toXMLString());
	*/
    }

    /**
     * Convert DRM Job array to ResourceManager JobsInfoType 
     *
     * @param  jobs 	the array of Job
     * @return JobsInfoType  the object to hold Job information
     **/
    private JobsInfoType convertToJobInfos(Job [] jobs) {

	if (jobs.length <= 0) {
	    return null;
	}

	JobInfoType [] jobInfoArray = new JobInfoType [jobs.length];
	String value = null;
	int    index = 0;

	for (int i = 0; i < jobs.length; i++) { 
	    JobInfoType jobInfo = null;
	    Job job = jobs[i];

	    if (job == null) {
		continue;
	    }

	    try {
		value = job.getAttribute(Job.JobId).trim();
		if (value != null && value.length() > 0) {
		    jobInfo = new JobInfoType();
		    jobInfo.setJobId(value);
		} else {
		    continue;
		}
	    } catch (Exception e) {
		logger.debug("Exception:" + e);
		continue;
	    }    
	    
	    try {
		value = job.getAttribute(Job.Queue).trim();
		if (value != null) {
		    jobInfo.setQueue(value);
		}
	    } catch (Exception e) {
		logger.debug("Exception:" + e);	
	    }

	    try {
		value = job.getAttribute(Job.ProjectName).trim();
		if (value != null) {
		    jobInfo.setProject(value);
		}
	    } catch (Exception e) {
		logger.debug("Exception:" + e);	
	    }
	
	    try {
		value = job.getAttribute("submit_rsvId").trim();
		if (value != null) {
		    jobInfo.setRsvId(value);
		}
	    } catch (Exception e) {
		logger.debug("Exception:" + e);	
	    }
	
	    try {
		value = job.getAttribute("InputFile").trim();
		if (value != null) {
		    jobInfo.setStdin(value);
		}
	    } catch (Exception e) {
		logger.debug("Exception:" + e);	
	    }

	    try {
		value = job.getAttribute("OutputFile").trim();
		if (value != null) {
		    jobInfo.setStdout(value);
		}
	    } catch (Exception e) {
		logger.debug("Exception:" + e);	
	    }

	    try {
		value = job.getAttribute("ErrorFile").trim();
		if (value != null) {
		    jobInfo.setStderr(value);
		}
	    } catch (Exception e) {
		logger.debug("Exception:" + e);	
	    }

	    try {
		value = job.getAttribute(Job.Submit_maxNumProcessors).trim();
		if (value != null) {
		    jobInfo.setCount(Integer.valueOf(value));
		}
	    } catch (Exception e) {
		logger.debug("Exception:" + e);	
	    }

	    try {
		value = job.getAttribute(Job.UserName).trim();
		if (value != null) {
		    jobInfo.setUser(value);

		}
	    } catch (Exception e) {
		logger.debug("Exception:" + e);	
	    }

	    try {
		value = job.getAttribute(Job.Command).trim();
		if (value != null) {
		    jobInfo.setExecutable(value);
		}
	    } catch (Exception e) {
		logger.debug("Exception:" + e);	
	    }

	    try {
		value = job.getAttribute(Job.Status).trim();
		if (value != null) {
		    int status = Integer.parseInt(value);
		    jobInfo.setStatus(convertToRMJobStatus(status));
		}
	    } catch (Exception e) {
		logger.debug("Exception:" + e);	
	    }
       
	    try {
		value = job.getAttribute(Job.SubmitTime).trim();
		if (value != null) {
		    long time = Long.parseLong(value);
		    if (time > 0) {
			jobInfo.setSubmitTime(
			        convertToExtendedDateTimeType(time));
		    }
		}
	    } catch (Exception e) {
		logger.debug("Exception:" + e);	
	    }

	    try {
		value = job.getAttribute(Job.PendReason).trim();
		if (value != null) {
		    jobInfo.setReason(value);
		}
	    } catch (Exception e) {
		logger.debug("Exception:" + e);	
	    }

	    try {
		value = job.getAttribute(Job.ExHosts).trim();
		if (value != null) {
		    jobInfo.setExecutionHosts(value);
		}
	    } catch (Exception e) {
		logger.debug("Exception:" + e);	
	    }
 
  	    try {
		value = job.getAttribute(Job.StartTime).trim();
		if (value != null) {
		    long time = (long)Long.parseLong(value);
		    if (time > 0) {
			jobInfo.setStartTime(
			        convertToExtendedDateTimeType(time));
		    }
		}
	    } catch (Exception e) {
		logger.debug("Exception:" + e);	
	    }
 
	    jobInfoArray[index] = jobInfo;
	    index ++;
	}

	if (jobInfoArray.length > 0 && index > 0) {
	    JobsInfoType ret = new JobsInfoType();
	    ret.setJobsInfo(jobInfoArray);
	    return (ret);
	} else {
	    return (null);
	}
    }

    private Calendar convertToExtendedDateTimeType(long time)
    {
	Calendar cal = Calendar.getInstance();
	Date now = new Date(time * 1000);

	cal.setTime(now);
	return(cal);	
    }

    private ResourceManagerJobStatus convertToRMJobStatus(int status) 
    {
	ResourceManagerJobStatus value = null;

	switch (status) {
	    
	case 0x01:   //JOB_STAT_PEND
            value = ResourceManagerJobStatus.PEND;
            break;
        case 0x02:   //JOB_STAT_PSUSP
            value = ResourceManagerJobStatus.PSUSP;
            break;
        case 0x04:   //JOB_STAT_RUN
            value = ResourceManagerJobStatus.RUN;
            break;
        case 0x04 | 0x200: //JOB_STAT_RUN | JOB_STAT_WAIT
            value = ResourceManagerJobStatus.WAIT;
            break;
        case 0x08: //JOB_STAT_SSUSP
            value = ResourceManagerJobStatus.SSUSP;
            break;
        case 0x10: //JOB_STAT_USUSP
            value = ResourceManagerJobStatus.USUSP;
            break;
	case 0x20: //JOB_STAT_EXIT
	    value = ResourceManagerJobStatus.EXIT;
	    break;
	case 0x40: //JOB_STAT_DONE
	case 0x40 | 0x80: //JOB_STAT_DONE|JOB_STAT_PDONE
        case 0x40 | 0x200: //JOB_STAT_DONE|JOB_STAT_WAIT
	case 0x40 | 0x100: //JOB_STAT_DONE|JOB_STAT_PERR
	    value = ResourceManagerJobStatus.DONE;
	    break;
	case 0x1000: //JOB_STAT_UNKWN
	    value = ResourceManagerJobStatus.UNKWN;
	    break;
	default:
	    value = ResourceManagerJobStatus.ERROR;
	    break;
        }

	return (value);
    }

    // The following code should be shared by both webGui and Adapter
    class AdvRsvAddRequest extends Request
    {
        public AdvRsvAddRequest( AdvRsv advrsvDef)
        {
          super();
          Element element = m_document.createElement( "numCpus" );
          Text    value   = m_document.createTextNode(advrsvDef.getAttribute(AdvRsv.Ncpus));
          element.appendChild( value );
          m_documentFragment.appendChild( element );

          element = m_document.createElement( "askedHosts" );
          value   = m_document.createTextNode(advrsvDef.getAttribute(AdvRsv.AskedHosts));
          element.appendChild( value );
          m_documentFragment.appendChild( element );

          element = m_document.createElement( "askedHostNum" );
          value   = m_document.createTextNode(advrsvDef.getAttribute(AdvRsv.AskedHostsNum));
          element.appendChild( value );
          m_documentFragment.appendChild( element );

          element = m_document.createElement( "Name" );
          value   = m_document.createTextNode(advrsvDef.getAttribute(AdvRsv.UserName));
          element.appendChild( value );
          m_documentFragment.appendChild( element );

          element = m_document.createElement( "Type" );
          value   = m_document.createTextNode(advrsvDef.getAttribute(AdvRsv.Type));
          element.appendChild( value );
          m_documentFragment.appendChild( element );

          element = m_document.createElement( "IsRecur" );
          value   = m_document.createTextNode(advrsvDef.getAttribute(AdvRsv.IsRecur));
          element.appendChild( value );
          m_documentFragment.appendChild( element );

          element = m_document.createElement( "startTime" );
          value   = m_document.createTextNode(advrsvDef.getAttribute(AdvRsv.StartTime));
          element.appendChild( value );
          m_documentFragment.appendChild( element );

          element = m_document.createElement( "endTime" );
          value   = m_document.createTextNode(advrsvDef.getAttribute(AdvRsv.EndTime));
          element.appendChild( value );
          m_documentFragment.appendChild( element );

          element = m_document.createElement("resReq");
          value   = m_document.createTextNode(advrsvDef.getAttribute(AdvRsv.ResReq));
          element.appendChild( value );
          m_documentFragment.appendChild( element );
       }

    }

    class AdvRsvDeleteRequest extends Request
    {
        public AdvRsvDeleteRequest(String rsvId)
        {
            super();
            Element element = m_document.createElement( "advrsvName" );
            Text    value   = m_document.createTextNode( rsvId );
            element.appendChild( value );
            m_documentFragment.appendChild( element );
        }
    }  
   
} 



















