
/**
 * ReservationInfo.java
 *
 * This class convert AdvRsv to a XML string which conforms to the 
 * metascheduler.xsd schema. 
 */
package com.platform.metascheduler.resourcemanager;

import java.io.IOException;
import java.util.*;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.platform.drm.rm.AdvRsv;
import com.platform.metascheduler.utils.MetaSchedulerConstants;

public class ReservationInfo {
    static Log logger = LogFactory.getLog(ReservationInfo.class);

    public static final String DEFAULT_NS = "rmgr";
    public static final String RSVINFOLIST = "reservationInfo";
    public static final String RSVINFO = "reservationItem";
    public static final String RSVID = "rsvId";
    public static final String RSRC = "rsrc";
    public static final String HOSTSINFO = "hostsInfo";
    public static final String HOSTINFO = "hostInfo";
    public static final String HOSTNAME = "name";
    public static final String USER = "user";
    public static final String NUMBER = "number";
    public static final String RSVTIME = "reservationTime";
    public static final String BEGINTIME = "beginTime";
    public static final String ENDTIME = "endTime";
    public static final String RESOURCEMANAGER_NS = "http://com.platform.metascheduler/2003/05/resourcemanager";
    
    protected static final String HEADER
        = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
	+ "<metascheduler:metascheduler\n"
        + "    xmlns:metascheduler=\""
        + MetaSchedulerConstants.METASCHEDULER_RSL_NS + "\"\n"
	+ "    xmlns:rmgr=\"" + RESOURCEMANAGER_NS + "\"\n"
        + "    xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n"
        + "    xsi:schemaLocation=\"\n"
        + "         " + MetaSchedulerConstants.METASCHEDULER_RSL_NS + "\n"
        + "         " + "./metascheduler/metascheduler.xsd\n"
	+ "         " + RESOURCEMANAGER_NS + "\n"
        + "         " + "./metascheduler/resourcemanager.xsd\">\n";

    protected static final String FOOTER = "</metascheduler:metascheduler>\n";
    protected static final String INFO_HEADER = "<rmgr:rmgr>\n";
    protected static final String INFO_FOOTER = "</rmgr:rmgr>\n";
    protected static final String RSVINFO_HEADER = "<rmgr:reservationInfo>\n";
    protected static final String RSVINFO_FOOTER = "</rmgr:reservationInfo>\n";

    private AdvRsv [] m_rsvInfoList = null;

    public ReservationInfo() {
	m_rsvInfoList = null;
    }

    public ReservationInfo(AdvRsv rsv) {
	if (rsv == null) {
	    m_rsvInfoList = null;
	} else {
	    m_rsvInfoList = new AdvRsv[1];
	    m_rsvInfoList[0] = rsv;
	}
    }
    
    public ReservationInfo(AdvRsv[] rsvList) {
	if (rsvList == null || rsvList.length <= 0) {
	    m_rsvInfoList = null;
	} else {
	    m_rsvInfoList = rsvList;
	}
    }
    
    public String toString () {

	if (m_rsvInfoList == null) {
	    return (null);
	}

	StringBuffer rsv = new StringBuffer("");
	for (int i = 0; i < m_rsvInfoList.length; i++) {
	    rsv.append(" rsvId:");
	    rsv.append(m_rsvInfoList[i].getAttribute(AdvRsv.Id));       
	    rsv.append(" number:");
	    rsv.append(m_rsvInfoList[i].getAttribute(AdvRsv.Ncpus));       
	    rsv.append(" hosts:");
	    rsv.append(m_rsvInfoList[i].getAttribute(AdvRsv.AskedHosts));
	    rsv.append(" timeWindow:");
	    rsv.append(m_rsvInfoList[i].getAttribute(AdvRsv.TimeWindow));
	    rsv.append(" user:");
	    rsv.append(m_rsvInfoList[i].getAttribute(AdvRsv.UserName));
	    rsv.append("\n");
	} 
	return (rsv.toString());
    }

    public String toXMLString() {

	if (m_rsvInfoList == null) {
	    return (null);
	}
	
	StringBuffer allRsvInfo = new StringBuffer();
	for (int i = 0; i < m_rsvInfoList.length; i++) {
	    String rsvInfo = rsvToXMLString(m_rsvInfoList[i], 1);
	    if (rsvInfo != null) {
		allRsvInfo.append(rsvInfo);
	    }
	}
	
	StringBuffer xmlRsl = new StringBuffer();

	if (allRsvInfo.length() > 0) {
	    xmlRsl.append(HEADER);
	    xmlRsl.append(INFO_HEADER);
	    xmlRsl.append(RSVINFO_HEADER);
	    xmlRsl.append(allRsvInfo);
	    xmlRsl.append(RSVINFO_FOOTER);
	    xmlRsl.append(INFO_FOOTER);
	    xmlRsl.append(FOOTER);
	}
	return(xmlRsl.toString());
    }


    public String rsvToXMLString(AdvRsv rsv, int indentLevel) {

	if (rsv == null) {
	    return null;
	}
	
	StringBuffer result = new StringBuffer();
	String  value = null;

       	try {
	    value = rsv.getAttribute(AdvRsv.Id).trim();
	    if (value != null) {
		result.append(getSimpleElementMarkup(RSVID, value, 
						     indentLevel + 1));	
	    } else {
		return null;
	    }
	} catch (Exception e) {
	    logger.debug(e.getMessage());
	}

	try {
	    value = rsv.getAttribute(AdvRsv.ResReq).trim();
	    if (value != null) {
		result.append(getSimpleElementMarkup(RSRC, value, 
						     indentLevel + 1));	
	    } else {
		return null;
	    }
	} catch (Exception e) {
	    logger.debug(e.getMessage());
	}
	
	try {
	    value = rsv.getAttribute(AdvRsv.AskedHosts).trim();
	    if (value != null) {
		String hostString = hostsToXMLString(value, indentLevel + 1);
		if (hostString != null) {
		    result.append(hostString);
		}
	    }
	} catch (Exception e) {
	    logger.debug(e.getMessage());
	}

	try {
	    value = rsv.getAttribute(AdvRsv.UserName).trim();
	    if (value != null) {
		result.append(getSimpleElementMarkup(USER, value, 
						     indentLevel + 1));
	    }
	} catch (Exception e) {
	    logger.debug(e.getMessage());
	}

	try {
	    value = rsv.getAttribute(AdvRsv.Ncpus).trim();
	    if (value != null) {
		result.append(getSimpleElementMarkup(NUMBER, value,
						     indentLevel + 1));
	    }
	} catch (Exception e) {
	    logger.debug(e.getMessage());
	}

	try {
	    value = rsv.getAttribute(AdvRsv.TimeWindow).trim();
	    if (value != null) {
		String rsvWindow = timeWindowToXMLString(value, indentLevel + 1);
	    
		if (rsvWindow != null) {
		    result.append(rsvWindow);
		}	
	    }
	} catch (Exception e) {
	    logger.debug(e.getMessage());
	}

	if (result.length() > 0) {
	    result.insert(0, getOpenTag(RSVINFO, indentLevel));
	    result.append(getCloseTag(RSVINFO, indentLevel));
	}
	return (result.toString());
    }

    private String timeWindowToXMLString(String window, int indentLevel)
    {
	if (window == null) {
	    return null;
	}

	StringBuffer result = new StringBuffer();

	result.append(getOpenTag(RSVTIME, indentLevel));

	int idx = window.indexOf("-");

	String beginTimeOrg = window.substring(0, idx);
	String endTimeOrg = window.substring(idx+1);

	String beginTime = convertFromLSFTimeFormat(beginTimeOrg);
	result.append(getSimpleElementMarkup(BEGINTIME, beginTime,
					     indentLevel + 1));

	String endTime = convertFromLSFTimeFormat(endTimeOrg);
	result.append(getSimpleElementMarkup(ENDTIME, endTime,
					     indentLevel + 1));

	result.append(getCloseTag(RSVTIME, indentLevel));

	return (result.toString());
    }

    private String hostsToXMLString(String hosts, int indentLevel)
    {
	if (hosts == null || hosts.trim().length() <= 0) {
	    return null;
	}
	
	StringTokenizer st = new StringTokenizer(hosts, " ");
	int num = st.countTokens();

	StringBuffer result = new StringBuffer();

	result.append(getOpenTag(HOSTSINFO, indentLevel));
       
	for (int i = 0; i < num; i++) {
	    String hostInfo =  new String(st.nextToken());
	    String value = hostToXMLString(hostInfo, indentLevel + 1);
	    if (value != null) {
		result.append(value);
	    }
	}
	result.append(getCloseTag(HOSTSINFO, indentLevel));

	return (result.toString());
	    	    
    }
	
    private String hostToXMLString(String host, int indentLevel)
    {
	if (host == null) {
	    return null;
	}

	int    idx = host.indexOf(":");
	String  value = null;

	value = host.substring(0, idx);
	if (value != null) {

	    StringBuffer result = new StringBuffer();

	    result.append(getOpenTag(HOSTINFO, indentLevel));

	    result.append(getSimpleElementMarkup(HOSTNAME, value,
						 indentLevel + 1));
	    
	    value = host.substring(idx + 1);
	    result.append(getSimpleElementMarkup(NUMBER, value,
						 indentLevel + 1));
	    
	    result.append(getCloseTag(HOSTINFO, indentLevel));
	    
	    return(result.toString());
	} else {
	    return null;
	}
    }
	
    private String getIndent(int level)
    {
	StringBuffer space = new StringBuffer();
	for (int i = 0; i < level; i++) {
	    space.append("  ");
	}
	return (space.toString());
    }

    private String getOpenTag(String elementName, int indentLevel)
    {
	StringBuffer tag = new StringBuffer();
	String space = getIndent(indentLevel);

	tag.append("<" + DEFAULT_NS + ":" + elementName + ">");
	
	if (indentLevel > 0){
	    tag.insert(0, space);
	    tag.append("\n");
	}
	return (tag.toString());
    }

    private String getCloseTag(String elementName, int indentLevel)
    {
	StringBuffer tag = new StringBuffer();
	String space = getIndent(indentLevel);

	tag.append("</" + DEFAULT_NS + ":" + elementName + ">");
	
	if (indentLevel > 0){
	    tag.insert(0, space);
	    tag.append("\n");
	}
	return (tag.toString());
    }

    private StringBuffer getSimpleElementMarkup(String elementName, 
						String value,
						int indentLevel)
    {
	StringBuffer elementMarkup = new StringBuffer();

	elementMarkup.append("<");
	elementMarkup.append(DEFAULT_NS);
	elementMarkup.append(":");
	elementMarkup.append(elementName);
	elementMarkup.append(">");
	elementMarkup.append(value);
	elementMarkup.append("</");
	elementMarkup.append(DEFAULT_NS);
	elementMarkup.append(":");
	elementMarkup.append(elementName);
	elementMarkup.append(">");

	if (indentLevel > 0) {
	    String space = getIndent(indentLevel);
	    elementMarkup.insert(0, space);
	    elementMarkup.append("\n");
	}
	return (elementMarkup);
    }

    private static String convertFromLSFTimeFormat(String timeString)
    {
	StringBuffer rmTime = new StringBuffer();
	StringTokenizer st = new StringTokenizer(timeString, "//");
	int num = st.countTokens();

	for (int i = 0; i < 3; i++) {
	    if (i > num - 1) {
		rmTime.append("00");
	    } else {
		rmTime.append(st.nextToken());
	    }
	    if ( i < 2) {
		rmTime.append(":");
	    }
	}
	return (rmTime.toString());
    }

}
