package com.platform.metascheduler.resourcemanager.client;

import org.globus.axis.gsi.GSIConstants;
import org.globus.wsrf.impl.security.authentication.Constants;
import org.globus.wsrf.impl.security.authorization.HostAuthorization;
import org.globus.wsrf.impl.security.authorization.SelfAuthorization;
import org.globus.gsi.proxy.IgnoreProxyPolicyHandler;

import java.io.FileReader;
import java.io.File;
import java.io.BufferedReader;
import java.net.URL;
import java.util.Calendar;
import java.util.Date;
import javax.xml.rpc.Stub;
import java.io.FileOutputStream;
import java.io.FileInputStream;

import java.net.MalformedURLException;
import javax.xml.soap.SOAPException;
import java.io.IOException;
import java.rmi.RemoteException;
import java.io.FileNotFoundException;

import javax.xml.namespace.QName;
import org.xml.sax.InputSource;
import org.apache.axis.message.MessageElement;
import org.apache.axis.message.addressing.ReferencePropertiesType;
import org.apache.axis.message.addressing.Address;
import org.apache.axis.message.addressing.EndpointReferenceType;

import com.platform.metascheduler.resourcemanager.service.ResourceManagerFactoryServiceAddressingLocator;
import com.platform.metascheduler.resourcemanager.service.ResourceManagerLsfServiceAddressingLocator;
import com.platform.metascheduler.resourcemanager.ResourceManagerFactoryPortType;
import com.platform.metascheduler.resourcemanager.ResourceManagerLsfPortType;
import com.platform.metascheduler.resourcemanager.CreateResourceManagerResponse;
import com.platform.metascheduler.resourcemanager.CreateResourceManager;
import com.platform.metascheduler.resourcemanager.JobSubmit;
import com.platform.metascheduler.resourcemanager.JobSubmitResponse;
import com.platform.metascheduler.resourcemanager.JobQuery;
import com.platform.metascheduler.resourcemanager.JobQueryResponse;
import com.platform.metascheduler.resourcemanager.JobControl;
import com.platform.metascheduler.resourcemanager.JobControlResponse;
import com.platform.metascheduler.resourcemanager.RsvSubmit;
import com.platform.metascheduler.resourcemanager.RsvSubmitResponse;
import com.platform.metascheduler.resourcemanager.RsvQuery;
import com.platform.metascheduler.resourcemanager.RsvQueryResponse;
import com.platform.metascheduler.resourcemanager.RsvCancel;
import com.platform.metascheduler.resourcemanager.RsvCancelResponse;

import org.oasis.wsrf.lifetime.Destroy;
import org.globus.wsrf.ResourceException;
import org.globus.wsrf.encoding.ObjectSerializer;
import org.globus.wsrf.encoding.ObjectDeserializer;
import org.globus.wsrf.encoding.DeserializationException;

public class ResourceManagerServiceClient {
    static {
        org.globus.axis.util.Util.registerTransport();
    }
    
    static final int OP_CREATE_SERVICE  = 0; 
    static final int OP_JOB_SUBMIT      = 1; 
    static final int OP_JOB_QUERY       = 2; 
    static final int OP_JOB_CONTROL     = 3; 
    static final int OP_RSV_SUBMIT      = 4; 
    static final int OP_RSV_QUERY       = 5; 
    static final int OP_RSV_CANCEL      = 6; 
    
    static boolean   isFactoryURI = false; 
    static int       operation    = -1;
    static String    URI = null; 
    static String    rm_type = null;
    static String    userhome= System.getProperty("user.home") + "/";

    private static String readRsl(String rslFile) {
	
	BufferedReader rslReader = null;
	StringBuffer rsl = null;
	try {
	    rslReader = new BufferedReader(new FileReader(rslFile));
	    rsl = new StringBuffer();
	    String rslLine = rslReader.readLine();
	    while (rslLine != null) {
		rsl.append(rslLine);
		rslLine = rslReader.readLine();
	    }
	} catch (IOException e) {
	    System.err.println("Error reading rsl file: " + e);
	    System.exit(2);
	} finally {
	    if (rslReader != null) {
		try {
		    rslReader.close();
		} catch (IOException ee) {
		    // ignore this error
		}
	    }
	}
	return (rsl.toString());
    }

    public static void help() {
	System.err.println("Usage: ResourceManagerServiceClient <handle> <operation>");
        System.err.println("Usage: ResourceManagerServiceClient <service_name> <service_operation>");
        System.err.println("Handle:");
        System.err.println("    URI to service ");
        System.err.println("operation:");
        System.err.println("    createService [cluster] [service name]");
        System.err.println("service_operation:");
        System.err.println("    jobSubmit     job.xml");
        System.err.println("    jobControl    [KILL|STOP|RESUME] jobId");
        System.err.println("    jobQuery      [jobId]");
        System.err.println("    rsvSubmit     rsv.xml");
        System.err.println("    rsvQuery      [rsvId]");
        System.err.println("    rsvCancel     rsvId");
        System.exit (-1);
    }

    public static void parseArgs(String[] args) { 
	int length = args.length; 

	if ( length < 2) { 
	    help(); 
	}

	// Check if the handle is to the factory service or instance
	URI = args[0]; 
	if ( URI.endsWith("ResourceManagerFactoryService")) { 
	    isFactoryURI = true; 
	} 

	// Get operation name
	String op = args[1]; 
	if ( op.equalsIgnoreCase("createService")) { 
	    operation = OP_CREATE_SERVICE; 	    
	} else if ( op.equalsIgnoreCase("jobSubmit")) { 
	    operation = OP_JOB_SUBMIT;
	} else if ( op.equalsIgnoreCase("jobControl")) { 
	    operation = OP_JOB_CONTROL;
	} else if ( op.equalsIgnoreCase("jobQuery")) { 
	    operation = OP_JOB_QUERY;
	} else if ( op.equalsIgnoreCase("rsvSubmit")) { 
	    operation = OP_RSV_SUBMIT;
	} else if ( op.equalsIgnoreCase("rsvQuery")) { 
	    operation = OP_RSV_QUERY;
	} else if ( op.equalsIgnoreCase("rsvCancel")) { 
	    operation = OP_RSV_CANCEL;
	} else { 
	    System.err.println("Invalid operation: " + op); 
	    help();
	}
    } 

    public static void CreateResourceManager(String[] args,String key) {

        ResourceManagerFactoryServiceAddressingLocator locator = 
			new ResourceManagerFactoryServiceAddressingLocator();
	ResourceManagerFactoryPortType rmf = null;
	EndpointReferenceType endpoint = null;

	try { 
	       // Pass in URL to factory service: create an service instance
	       String name    = null; 
	       String cluster = null;
	       if ( args.length >= 4) { 
			name = args[3];
	       }
	       if ( args.length >= 3) { 
			cluster = args[2];
	       }
	       System.out.println("Create instance with name: " 
				  + name + " in cluster " + cluster);

              // Create endpoint reference (sans resource)
              endpoint = new EndpointReferenceType();
              endpoint.setAddress(new Address(args[0]));
              rmf = locator.getResourceManagerFactoryPortTypePort(endpoint);

		// enable GSI secure conversation
		// Setup client delegation
		((Stub)rmf)._setProperty(Constants.GSI_SEC_CONV,
						   Constants.SIGNATURE);
		((Stub)rmf)._setProperty(GSIConstants.GSI_MODE,
						   GSIConstants.GSI_MODE_LIMITED_DELEG);
		((Stub)rmf)._setProperty(Constants.AUTHORIZATION, 
       	                                	   HostAuthorization.getInstance());
		//((Stub)rmf)._setProperty(Constants.GRIM_POLICY_HANDLER,
		//				   new IgnoreProxyPolicyHandler());
 
              // Create resource and get endpoint reference (with resource)
              CreateResourceManagerResponse response = rmf.createResourceManager(new CreateResourceManager(cluster,key));
              endpoint = response.getEndpointReference();
	      rm_type = response.getType();

              // Serialize endpoint to file
              FileOutputStream out = null;
              try {
                   out = new FileOutputStream(userhome + "RM_" + name + ".epr");
                   out.write(ObjectSerializer.toString(endpoint,
                              new QName("", "RMEPR")).getBytes());
               } catch (IOException e) {
                      System.err.println("Failed to write subscription epr: " +
                             e.getMessage());
               } finally {
                   if (out != null) {
                       try { out.close(); } catch (Exception ee) {}
                      }
               }

              System.out.println("Service location:" + args[0]);
              System.out.println("Create " + rm_type + " ResourceManager Successfully: ");

       }catch (RemoteException rme){
                System.out.println("RemoteException catched! CreateResourceManager error!");
		System.out.println(rme.getMessage());
		//rmf.destroy();
	    System.exit(2);
       }catch (Exception e) {
                e.printStackTrace();
	    System.exit(2);
        }
    }

    public static void main (String[] args) {

	Object rmporttype = null;

	parseArgs(args);
	String ResourceKey = null;

	if (operation == OP_CREATE_SERVICE) {
		//check if created already
		File test = new File(userhome + "RM_"+args[3]+".epr");
		if (test.exists()) {
			//deserialize epr file
			EndpointReferenceType epr = ReadEpr(args[3]);
			//get key
			ResourceKey = getKey(epr);
			System.out.println("epr file exist! key is :" + ResourceKey);
		}
		CreateResourceManager(args,ResourceKey);
		System.exit(0);
	}
	
        rmporttype = findResourceManagerPortType(args);
	ResourceManagerLsfPortType rm = (ResourceManagerLsfPortType)rmporttype;

//	when complete Gram type , modify here
//        if (rm_type.equalsIgnoreCase("LSF")){
//	} else {
//		//ResourceManagerGramPortType rm = (ResourceManagerGramPortType)rmporttype;
//	}

	try { 
	    // Get the operation
            if (operation == OP_JOB_SUBMIT) {
               if ( args.length >= 3) {
                    String jobFile = new String(args[args.length-1]);
		    System.out.println("Invoke jobSubmit: read rslFile " 
				       + jobFile);
		    String rsl = readRsl(jobFile);

		    System.out.println("Invoke jobSubmit: jobId is " 
				       + rm.jobSubmit(new JobSubmit(rsl)).getJobId());
                } else { 
		    System.out.println("    jobSubmit     job.xml");
                }
            }
 
	else if ( operation == OP_JOB_CONTROL ) { 
		String jobId = null;
		String signal = null;

		if (args.length >= 4) {
		    jobId = new String (args[args.length-1]);
		    signal = new String (args[args.length-2]);
		} else if ( args.length == 3) {
		    // Cluster name not specified
		    jobId = new String (args[args.length-1]);
		    signal = new String ("KILL");
		} else { 
		    System.out.println("    jobControl [KILL|STOP|RESUME] jobId");
		    System.exit(2);
		}
	       
		if (!(signal.equalsIgnoreCase("KILL") 
		      || signal.equalsIgnoreCase("STOP")
		      || signal.equalsIgnoreCase("RESUME"))) {
		    System.out.println("    jobControl [KILL|STOP|RESUME] jobId");
		    System.exit(2);
		}

		int ret = rm.jobControl(new JobControl(jobId,signal)).getReply();
		if (ret == 0) {
		    System.out.println("Invoke jobControl " 
				       + signal + " " + jobId + ":"  
				       + "Done");
		} else {
		     System.out.println("Invoke jobControl " 
				       + signal + " " + jobId + ":"  
				       + "Failed");
		}

            } else if (operation == OP_JOB_QUERY ) {
		String output = null;
		if ( args.length >= 3) {
		    String jobId = new String (args[args.length-1]);
		    System.out.println("Invoke jobQuery " + jobId + ": ");
		    output = rm.jobQuery(new JobQuery(jobId)).getJobInfo();
		} else { 
		    System.out.println("Invoke jobQuery: ");
		    output = rm.jobQuery(new JobQuery("")).getJobInfo();
		}
		if (output == null) {
		    System.out.println("Not found");
		} else {
		    System.out.println("\n" + output);
		}

            } else if ( operation == OP_RSV_SUBMIT ) { 
		if ( args.length >=3 ) { 
		    System.out.println("Invoke RsvSubmit: read rslFile " 
				       + args[args.length-1]);
	
		    String rsl = readRsl(args[args.length-1]);
		    System.out.println("Invoke RsvSubmit: rsvId is " 
				       + rm.rsvSubmit(new RsvSubmit(rsl)).getRsvId());	    
		} else { 
		    System.out.println("    rsvSubmit     rsv.xml");
		}
            } else if ( operation == OP_RSV_QUERY) { 
		String output = null;
		if ( args.length >= 3) {
		    System.out.println("Invoke rsvQuery " 
				       + args[args.length-1] + ":\n");
		    output = rm.rsvQuery(new RsvQuery(args[args.length-1])).getRsvInfo();
		} else { 
		    System.out.println("Invoke rsvQuery :\n");
		    output = rm.rsvQuery(new RsvQuery("")).getRsvInfo();
		}

		if (output == null) {
		    System.out.println("Not found"); 
		} else {
		    System.out.println(output);
		}
            } else if (  operation == OP_RSV_CANCEL) {
		if ( args.length >= 3) {
		    int ret = rm.rsvCancel(new RsvCancel(args[args.length-1])).getReply();
		    if (ret == 0) {
			System.out.println("Invoke rsvCancel " 
					+ args[args.length-1] + ":" + 
					"Done");
		    } else {
		       System.out.println("Invoke rsvCancel " 
					+ args[args.length-1] + ":" +
					"Failed");
		    }
		} else {
		    System.err.println("    rsvCancel     rsvId");
		}
	    }
      	} catch (RemoteException e) { 
	    System.err.println("RemoteException: " + e);
	    System.exit(2);
	} catch (Exception e) {
            //e.printStackTrace();
	    System.err.println("Exception " + e);
        }
    }
    public static EndpointReferenceType ReadEpr(String rmname) {

        //Deserialize endpoint
	FileInputStream in = null;
	EndpointReferenceType endpoint = null;
        try {
		in = new FileInputStream(userhome + "RM_" + rmname + ".epr");
                endpoint = (EndpointReferenceType)ObjectDeserializer.deserialize(
                    new InputSource(in),
                    EndpointReferenceType.class);
        }catch ( FileNotFoundException fe) {
                System.out.println(" ResouceManager: " + rmname + "hasn't been created !");
        }catch (DeserializationException e){
        }finally {
                if (in != null) {
                    try { in.close(); } catch (Exception e) {}
        	}
        }
	return endpoint;
    }
    public static Object findResourceManagerPortType(String[] args) {

	EndpointReferenceType epr = ReadEpr(new String(args[0]));
	Object rm = null;

	try{
		ResourceManagerLsfServiceAddressingLocator locator = new 
				ResourceManagerLsfServiceAddressingLocator();
		String address = epr.getAddress().toString();
		System.out.println("Address is :" + epr.getAddress());
		if ( address.indexOf("LSF") != -1 ) {
			rm_type = "LSF";
		} else {
			rm_type = "GRAM";
		}

		rm = (Object)locator.getResourceManagerLsfPortTypePort(epr);
	
  	     	((Stub)rm)._setProperty(Constants.GSI_SEC_CONV,
                                        Constants.SIGNATURE);
       		((Stub)rm)._setProperty(Constants.GSI_SEC_MSG,
                                        Constants.SIGNATURE);
       		((Stub)rm)._setProperty(GSIConstants.GSI_MODE,
                                        GSIConstants.GSI_MODE_LIMITED_DELEG);
       		((Stub)rm)._setProperty(Constants.AUTHORIZATION,
                                        HostAuthorization.getInstance());
       		//((Stub)rm)._setProperty(Constants.GRIM_POLICY_HANDLER,
       		//                      new IgnoreProxyPolicyHandler());
	} catch (Exception e) {
		System.out.println("getResourceManagerPortType Error!" + e.getMessage());
		System.exit(2);
	}
	return rm;
    }
    public static String getKey(EndpointReferenceType epr) {
        ReferencePropertiesType rp= epr.getProperties();
        MessageElement [] key = rp.get_any();
        return  key[0].getValue();
    }
}
