package com.platform.metascheduler.resourcemanager.gram;

// Logging 
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

// Java classes
import java.rmi.RemoteException;
import java.util.*;
import java.lang.Long;
import java.lang.StringBuffer;

// Grid service
/*import org.globus.ogsa.impl.ogsi.GridServiceImpl;
import org.globus.ogsa.GridContext;
import org.globus.ogsa.GridServiceException;
import org.gridforum.ogsi.ExtendedDateTimeType;
import org.gridforum.ogsi.TerminationTimeType;*/

// GSI 
import org.globus.axis.gsi.GSIConstants;

import org.globus.wsrf.security.SecurityManager;
import org.globus.wsrf.impl.security.authentication.secureconv.service.SecurityContext;
//import org.globus.ogsa.impl.security.authentication.SecContext;
import org.globus.wsrf.impl.security.authentication.Constants;


import org.ietf.jgss.GSSCredential;
import org.ietf.jgss.GSSContext;
import org.ietf.jgss.GSSException;
import org.apache.axis.MessageContext;

import javax.security.auth.Subject;

// Implementation
import com.platform.metascheduler.resourcemanager.ResourceManagerFaultType;

// Imports
import com.platform.metascheduler.utils.MetaSchedulerRsl;
import com.platform.metascheduler.utils.RsvAttributes;
import com.platform.metascheduler.utils.JobAttributes;

// RSL 
import org.globus.exec.utils.rsl.RSLParseException;
import org.w3c.dom.Element;

// Java CoG & gram stuff
import org.globus.gram.internal.GRAMConstants;
import org.globus.gram.GramException;

public class ResourceManagerGram {
	
    static Log logger = LogFactory.getLog(ResourceManagerGram.class);
//  private LsfCluster cluster = null;
    private Subject subject = null;
    private String  globusID = null;
    private GSSCredential proxy;
//  private String  clusterName = null; //gram has no such idea
    private String  contactHost = null; 
    private String  globusContact;
    private int     contactPort;
    private int     contactTimeout;
	
    /** 
		* Constructor to create a Resource Manager LSF service instance that 
		* connect to default LSF cluster, that the cluster URI  is 
	* lsf60:localhost:1966.
		*/
    public ResourceManagerGram() {
		// explicit call to superclass (ServiceSkeleton) default constructor
		
		logger.debug("Invoke ResourceManagerGram Constructor!"); 
		// Set default LSF cluster information
		
		setClusterContact("", "localhost", 2119);
    }
	
	
    /** 
		* Constructor to create a Resource Manager Gram service instance that 
		* connect to a Globus 2.x resource. If the URI format is wrong, default
		* constructor will be called.
		* 
		* @param URI  the Gram resource URI in format of 
		*             hostname:portnumber
		*/
    public ResourceManagerGram(String URI) {
		// explicit call to superclass (ServiceSkeleton) default constructor
		
		logger.debug("Invoke ResourceManagerLsf: URI:" + URI); 
		// clusterName:hostname:portnumber
		
		StringTokenizer st = new StringTokenizer(URI, "/:");
        int num = st.countTokens();
		
		if ( num == 3) { 
			setClusterContact(new String(st.nextToken()), new String(st.nextToken()),
							  Integer.parseInt(new String(st.nextToken())));
		} else {
			logger.error ("Wrong URI (" + URI + "). Default value is set."); 
			setClusterContact("", "localhost", 2119);  
		}
    }
	
    /** 
		* Call super class postCreate, and do GSI authentication.
		* Only the creator can use 
		* this instance within the life time of the instance. The terminate
		* time is 10 minutes.
		* 
		* @param context  the GridContext of the service instance.
		* @throws GridServiceException
		*/
/*    public void postCreate(GridContext context)	throws GridServiceException {
		super.postCreate(context);
		
		logger.debug("Invoke postCreate"); 
		
		// Authentication
		SecurityManager manager = SecurityManager.getManager();
		manager.setServiceOwnerFromContext(this);
		
		this.globusID = manager.getCaller();
		logger.info("Authenticated globus user is:" + this.globusID);
		
 *****removing lsf auth stuff
		
		String userName = null;
		try {
			ServiceProperties serviceProp = context.getServiceProperties();
			
			GridMap gridMap =
				SecureServicePropertiesHelper.getGridMap(serviceProp);
			
			gridMap.refresh();
			userName = gridMap.getUserID(globusID);
			
			if (userName == null) {
				String errMsg = "User " + globusID + ":not in gridmap file.";
				logger.error(errMsg);
				throw new GridServiceException(errMsg);
			} else {
				logger.debug("User " + globusID  
							 + ":authorized as local user:" + userName);
			}
		} catch (Exception e) {
			logger.debug("Exception:" + e);
			throw new GridServiceException("Exception:" + e);
		}
		
		User user = new User(userName);
		user.setToken(globusID);
		
		this.subject = new Subject();
		subject.getPrincipals().add(user);     ************** 
		//	AuthenticationController.getInstance().authenticate(this.subject); 

		// Set termination time for the instance to 10 minutes
		try { 
			ExtendedDateTimeType terminationTime = new ExtendedDateTimeType();
			Calendar timeout = Calendar.getInstance(); 
			timeout.add(Calendar.MINUTE, 10);
			terminationTime.setValue(timeout);
			
			TerminationTimeType termTimeType = this.requestTerminationBefore(terminationTime); 
			Date  date  = termTimeType.getAfter().getDateTimeValue().getTime(); 
			logger.debug("Instance termination time: " + date.toString());
		} catch (RemoteException e) { 
			e.printStackTrace();
		} 
		
    } // END of postCreate()
*/	
    // Get cluster contact information from URL
    private void setClusterContact(String cluster, String hn, int pn) {
		this.contactHost = hn;
		this.contactPort = pn;
		this.globusContact = hn + ":" + pn;
		this.contactTimeout = 120 * 1000;
		logger.debug("URI: " + globusContact);
		/***** change to globus thingy
		this.cluster = new LsfCluster(this.clusterName, this.contactHost,
									  this.contactPort, this.contactTimeout);
		*******/
		//initialized = true;
    }
	
    private void checkClientContext() throws Exception {
		
        MessageContext ctx = MessageContext.getCurrentContext();
	SecurityContext sec = (SecurityContext)ctx.getProperty(Constants.CONTEXT);
        if (sec == null) {
            throw new Exception("Service requires secure access.");
		}
		
		logger.debug("extracting proxy cert from SecurityContext");
		
		//extract the user's proxy certificate to pass to the gram job
		this.proxy = sec.getDelegatedCredential();
		if(this.proxy == null) {
			throw new RemoteException("Client proxy certificate could not be extracted");
		}
		
		GSSContext context = sec.getContext();
		try {
			String clientID = context.getSrcName().toString();
			
			if (clientID.equals(this.globusID)) {
				logger.debug("the client ID is: " + clientID);
			} else {
				logger.debug("the client ID is: " + clientID);
				logger.debug("the owner of the instance is: " +  globusID);
				throw new Exception("User cannot access the service instance."); 
			}
			//AuthenticationController.getInstance().authenticate(subject);
		} catch (Exception e){
			logger.debug("checkClientContext: " + e);
			throw new Exception ("checkClientContext:" + e);
		}
		
    }
	
    /** 
		* Submit a job to a gram service.
		*
		* @param rsl the job RSL (2) string to be submited
		* @return    the jobID return from the Gram gatekeeper
		* @throws    ResourceManagerFaultType
		*/
    public String jobSubmit(String rsl) 
		throws RemoteException,ResourceManagerFaultType {
			
			try {
				checkClientContext();
			} catch (Exception e) {
				throw new RemoteException( "Authentication Failed:" + e.getMessage());
			}
			
			if (rsl == null) {
				throw new RemoteException( "Empty RSL string" );
			} else {
				logger.debug("Invoke jobSumit: rsl: " + rsl);
			}
			
			JobAttributes job  = null;
			String jobId = null;
			try {
				// Read in the rsl file
				MetaSchedulerRsl msRsl = new MetaSchedulerRsl(rsl);
				Element msRslRootElement = msRsl.getRootElement();
				
				// Get reservation information
				job  = new JobAttributes(msRslRootElement); 
			} catch (Exception e) {
				throw new RemoteException("Wrong RSL string" + e.getMessage() );
			}
			
			if (job != null) { 
				// Here we do all the silly business of converting RSL2
				// to old-style RSL1
				
				//JobDefinition  jobDef = new JobDefinition();
				
				StringBuffer rsl1 = new StringBuffer("&");
				
				try {
					rsl1 = rsl1.append("(executable=" + job.getExecutable() +")");
				} catch ( Exception e) {
					throw new RemoteException( "Invalid Input:" + "No executable in RSL");
				}
				
				try {
					String [] arguments = job.getArguments();
					if(arguments.length > 0) {
						rsl1.append("(arguments=");
						for (int i = 0; i < arguments.length; i++) {
							rsl1.append(" ").append(arguments[i]);
						}
						rsl1.append(")");
					}
				} catch (Exception e) {
				}
				
				//set queue rsl attribute
				try {
				if(job.getQueue() != null) {
					rsl1.append("(queue=" + job.getQueue() + ")");
				}
				} catch (Exception e) {
				}
								
				try {
					Hashtable environment = job.getEnvironment();
					StringBuffer buffer = new StringBuffer();
					Enumeration e = environment.keys();
					if(e.hasMoreElements()) {
						rsl1.append("(environment=");
					
						while (e.hasMoreElements()) {
							String key = (String) e.nextElement();
							rsl1.append("(" + makeString(key) + ") (" 
										  + makeString((String)environment.get(key)) + ")");
						}
						rsl1.append(")");
					}
				} catch (Exception  e) {
				}
				
				try {
					rsl1.append("(project=" + job.getProject() + ")");
				} catch (Exception e) {
				}
			
				try {
					rsl1.append("(stdin=" + job.getStdin() + ")");
				} catch (Exception e) {
				}
				
				try {
					String [] stdOut = job.getStdout();
					// Gram can only accept one stdout file
					if (stdOut.length > 0){
						rsl1.append("(stdout=" + stdOut[0] + ")");
					}
				} catch (Exception e) {
				}
				
				try {
					String [] stdErr = job.getStderr();
					// LSF can only accept a stderr file
					if (stdErr.length > 0){
						rsl1.append("(stderr=" + stdErr[0] + ")");
						//jobDef.setAttribute(JobDefinition.Stderr, stdErr[0]);
					}
				} catch (Exception e){
				}
				
				try {
					rsl1.append("(count=" + Integer.toString(job.getCount()) + ")");
				} catch (Exception e) {
				}
				
				try {
					rsl1.append("(maxTime=" + Long.toString(job.getMaxTime()) + ")");
				} catch (Exception e){
				}
				
				try {
					rsl1.append("(maxWallTime=" + Long.toString(job.getMaxWallTime()) + ")");

				} catch (Exception e){
				}
				
				try {
					rsl1.append("(maxCpuTime=" + Long.toString(job.getMaxCpuTime()) + ")");
				} catch (Exception e){
				}	
				
				try {
					rsl1.append("(maxMemory=" + Integer.toString(job.getMaxMemory()) + ")");
				} catch (Exception e){
				}	
				
				try {
					rsl1.append("(directory=" + job.getDirectory() + ")");
				} catch (Exception e) {
				}
				
				//jobDef.setAttribute(JobDefinition.JobName,"ResourceManager");
				
				//submit job in background to gram, set jobId
				GridJob gramJob = new GridJob(globusContact, true);
				gramJob.GlobusRun(new String(rsl1));
				
				jobId = gramJob.getJobId();
				
			}
			return (jobId);
		} // End of jobSubmit
	
    /** 
		* Control a LSF job.
		*
		* @param jobId  the native LSF jobId string to be controlled.
		* @param signal the signal string. In this version, STOP, RESUME, KILL
		*               signals are supported
		* @return    0 means the request is sent to the LSF cluster successfully.
		* @throws     ResourceManagerFaultType
		*/
    public int jobControl(String jobId, String signal) 
		throws RemoteException,ResourceManagerFaultType  {
			
			logger.trace("Invoke jobControl: jobId " + jobId + "Signal " + signal);
			
			try {
				checkClientContext();
			} catch (Exception e) {
				throw new RemoteException("Authentication Failed:" + e.getMessage() );
			}
			
			//create new job object from jobid
			GridJob job = new GridJob(globusContact, true);
			job.setJobId(jobId);

			// Process the control request. Current requests include KILL, STOP, and RESUME
			// in string form.
			try {
				if(signal.equals("STOP")) {
					return job.signal(GRAMConstants.SIGNAL_SUSPEND);
				} else if (signal.equals("RESUME")) {
					return job.signal(GRAMConstants.SIGNAL_RESUME);
				} else if (signal.equals("KILL")) {
					return job.signal(GRAMConstants.SIGNAL_CANCEL);
				} else {
				
				}
			} catch (GSSException ge) {
				throw new RemoteException("Sending job signal failed:" + ge.getMessage() );
			} catch (GramException gre) {
				throw new RemoteException("Sending job signal " + signal + " failed:" + gre.getMessage());
			}
			
			return (0);
		} // End ofjobControl 
	
	
    /** 
		* Query a or all LSF jobs.
		*
		* @param jobId  the native LSF jobId string. If it is null, get all
		*               jobs from the cluster
		* @return       job information string 
		* @throws       ResourceManagerFaultType
		*/
    public String jobQuery(String jobId)
		throws RemoteException,ResourceManagerFaultType {
			
			logger.trace("Invoke jobQuery: jobId: " + jobId);
			
			try {
				checkClientContext();
			} catch (Exception e) {
				throw new RemoteException( "Authentication Failed:" + e.getMessage());
			}
			
			String jobInfo = null;
			
			//create new job object from jobid
			GridJob job = new GridJob(globusContact, true);
			job.setJobId(jobId);
			
			// Get information from the resource manager about the job,
			jobInfo = job.getStatus();
			
			return (jobInfo);
			
		} // End of jobQuery
	
	public String rsvSubmit(String rsl) throws RemoteException{
		throw new RemoteException("rsvSubmit not implemented for gram.");
	}
	
	public int rsvCancel(String rsvId) throws RemoteException{
		throw new RemoteException( "rsvCancel not implemented for gram.");
	}
	
	public String rsvQuery(String rsvId) throws RemoteException{
			throw new RemoteException( "rsvQuery not implemented for gram.");
	}
		
	private String makeString(String s) {
        char [] chars = s.toCharArray();
        StringBuffer str = new StringBuffer();
        int j = 0;
		
        for (int i = 0; i < chars.length; i++) {
            if (chars[i] == '\''  || chars[i] == '\\') {
                str.append('\\');
                str.append(chars[i]);
            } else {
                str.append(chars[i]);
            }
        }
		
        return str.toString();
    }
}
