package com.platform.metascheduler.resourcemanager.impl;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.CharacterData;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;
import org.w3c.dom.NamedNodeMap;
import org.apache.xerces.parsers.DOMParser;
import org.xml.sax.InputSource;
import org.apache.axis.message.MessageElement;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import javax.xml.namespace.QName;
import java.rmi.RemoteException;
import java.util.Vector;
import java.util.Enumeration;
import java.util.Calendar;
import java.util.StringTokenizer;

//import com.platform.metascheduler.resourcemanager.lsf.ResourceManagerLsf;
//import com.platform.metascheduler.resourcemanager.gram.ResourceManagerGram;
import com.platform.metascheduler.resourcemanager.ResourceManagerAccessType;
import com.platform.metascheduler.resourcemanager.ResourceManagerInfoType;
import com.platform.metascheduler.resourcemanager.ResourceManagerFaultType;
import com.platform.metascheduler.resourcemanager.CreateResourceManagerResponse;
import com.platform.metascheduler.resourcemanager.CreateResourceManager;
import com.platform.metascheduler.utils.JNDIHelper;
import com.platform.metascheduler.utils.MetaSchedulerConstants;
import com.platform.metascheduler.resourcemanager.ResourceManagerHome;
import com.platform.metascheduler.resourcemanager.impl.ResourceManagerFactoryHome;
import com.platform.metascheduler.resourcemanager.impl.ResourceManagerFactoryResource;
//import com.platform.metascheduler.resourcemanager.impl.ResourceManagerFactory;

/************* wsrf classes **************/
import org.globus.wsrf.ResourceKey;
import org.globus.wsrf.ResourceContext;
import org.apache.axis.message.addressing.EndpointReferenceType;
import org.globus.wsrf.utils.AddressingUtils;
import org.globus.wsrf.ResourceException;
import org.globus.wsrf.impl.ResourceHomeImpl;
import org.globus.wsrf.container.ServiceHost;
import org.globus.wsrf.security.SecurityManager;
import org.globus.wsrf.impl.SimpleResourceKey;
import org.globus.wsrf.Resource;

public class ResourceManagerFactory {
    private static Log logger = LogFactory.getLog(ResourceManagerFactory.class);
    private final  String RM_NAMESPACE_URI = "http://www.platform.com/namespaces/2003/05/metascheduler/resourcemanager";
    private final  String DEFAULT_RM_CONFIG = "etc/metascheduler/resourcemanager-config.xml";
    private final  String SERVICE_NAME_LSF = "metascheduler/ResourceManagerLsfService";
    private final  String SERVICE_NAME_GRAM = "metascheduler/ResourceManagerGramService";

    private Vector clusters = new Vector(); 
    private ResourceManagerAccessType rmAccess = null;
	
    private String handle = null;
    private String gis_handle = null;
    private QName serviceDataName = null;
    private String subscriptionID = null;
    private String rm_type = null;
    
    public ResourceManagerFactory(){
	logger.debug("ResourceManagerFactory construction!");
    }

    public CreateResourceManagerResponse createResourceManager(CreateResourceManager request)
		throws RemoteException , ResourceManagerFaultType {
			String clustername = request.getCluster();
			String resourcekey = request.getKey();
			//GridServiceBase rmImpl = null;
			EndpointReferenceType  epr = null;
			logger.debug("ResourceManagerFactory::createResourceManager called");
			
			try { 
			        // Process Security 
		                //get the caller's user subject
		                SecurityManager securityManager = SecurityManager.getManager();
      		                String userSubject = securityManager.getCaller();
				logger.debug("CallerSubject" + userSubject);

        		        //get the caller's local user ID via system grid-mapfile lookup
				//if caller subject map multi users, use first user
            		        String localUserId = null;
            		        String[] localUserIds = securityManager.getLocalUsernames();
                		localUserId = localUserIds[0];
				logger.debug("Local Authorized User is: " + localUserId);

				//process here by read cluster resource
				this.getClusters();
				Enumeration e = clusters.elements(); 
				ClusterContact firstCluster = null; 
				
				e = clusters.elements(); 
				firstCluster = (ClusterContact) e.nextElement();
				
				if ( clustername != null ) { 
					//String msg = AnyHelper.getAsString(extension);
					// logger.debug("Msg string: " + msg);
					//String str = (String) AnyHelper.getAsSingleObject(extension); 
					logger.debug("Ready to Create RM :" + clustername);
					String str = clustername;
					String URI = str.trim();
					ClusterContact cluster  = null;
					
					if (URI == null) {
						// The user doesn't specify which cluster to run the job
						// Use the first cluster available
						cluster = firstCluster;
						if ( cluster == null ) { 
							throw new RemoteException("No resource manager configured");
						}
						
					} else {
						logger.debug("User input cluster URI is : " + URI);
						
						boolean     found = false; 
						
						// Assume Cluster Name is unique, currently
						e = clusters.elements();
						logger.debug("config cluster number:" + clusters.size());
						logger.debug("config cluster :" + clusters.toString());
						while ( e.hasMoreElements() && found == false ) { 
							cluster = (ClusterContact) e.nextElement();
							
							if ( URI.compareTo(cluster.getName()) == 0) { 
								logger.debug("Cluster " + URI + " is defined in RMFS");
								if (cluster.getType().equalsIgnoreCase("LSF") || cluster.getType().equalsIgnoreCase("GRAM")) {
									found = true; 
								} else {
									logger.debug("Resource manager " + URI + " not supported");
									throw new RemoteException("Resource manager " + URI + " not supported");
								}
							}
						} 
						
						if (found == false) { 
							throw new RemoteException("Resource manager " + URI + " not configured");
						}
					}
					epr = this.createRM(cluster,localUserId,userSubject,resourcekey);
				} else { 
					if ( firstCluster == null ) { 
						throw new RemoteException("No resource manager configured");
					}
					epr = this.createRM(firstCluster,localUserId,userSubject,resourcekey);
				}
			} catch (RemoteException re) {
				logger.debug("Error creating ResourceManager:" + re);
				throw new RemoteException("Error creating ResourceManager", re);
			}
			
			//return rmImpl;
			CreateResourceManagerResponse response = new CreateResourceManagerResponse(epr,this.rm_type);
			logger.debug("createResourceManager done! created a Lsf RM resource.");
			logger.debug("the Lsf RM epr is " + epr);
			return response;
		}

/**
 * createRM
 * Create a ResourceManager 
 * @ params ClusterContact cc
 * @ return EndpointReferenceType
 */
	private EndpointReferenceType createRM( ClusterContact cc, String localuser, String usersubject, String resourcekey)
		throws RemoteException{
		ResourceContext ctx = null;
		ResourceHomeImpl home = null;
		ResourceKey key = null;
		EndpointReferenceType epr = null;
		String	rm_handle = null;

		ctx = ResourceContext.getResourceContext();

		try {
         		rm_handle = "https://" + ServiceHost.getBaseURL().getHost()
                       		  + ":" + ServiceHost.getBaseURL().getPort()
                         	  + "/";
		} catch( Exception e) {}

		if (cc.getType().equalsIgnoreCase("LSF")) {
			try {
				home = (ResourceHomeImpl) JNDIHelper.getResourceHome(SERVICE_NAME_LSF);
			} catch (javax.naming.NamingException ne) {
				throw new RemoteException("",ne);
			}
			this.rm_type = "LSF";
			rm_handle = rm_handle + MetaSchedulerConstants.RESOURCE_MANAGER_LSF_URL;
		}
		else if (cc.getType().equalsIgnoreCase("GRAM")) {
			try {
				home = (ResourceHomeImpl) JNDIHelper.getResourceHome(SERVICE_NAME_GRAM);
			} catch (javax.naming.NamingException ne) {
				throw new RemoteException("",ne);
			}
			this.rm_type = "GRAM";
			rm_handle = rm_handle + MetaSchedulerConstants.RESOURCE_MANAGER_GRAM_URL;
		}

		if (resourcekey != null) {
			ResourceKey tmpkey = new SimpleResourceKey(home.getKeyTypeName(), 
						Integer.valueOf(resourcekey));
			Resource resource = null;
			try {
				resource = home.find(tmpkey);
			}catch (ResourceException e) {
				logger.debug("Resource not exist!");
			}
			if (resource!=null) {
				logger.error("Resource exist, can't create before destroy!");
				throw new RemoteException("Resource exist, can't create before destroy!");
			}
		}

		logger.debug("Ready to create ResourceManager!");
		try {
			key = ((ResourceManagerHome) home).create(cc.getContactInfo(),localuser,usersubject);
		} catch (Exception e) {
                        throw new RemoteException("", e);
                }


		try {
			epr = AddressingUtils.createEndpointReference(ctx, key);
			epr.setAddress(new org.apache.axis.message.addressing.AttributedURI(rm_handle));
		} catch (Exception ee) {
			throw new RemoteException("", ee);
 		}
		return epr;
	}

/**
 * get Resource reference , invoked when access RPs
 */
	private void getClusters() throws RemoteException {

		String name;
		String type;
		String host;
		int    port;
		String clustersstr = null;
		String cluster = null;

                logger.debug("Enter getClusters!");
                ResourceManagerFactoryResource resource = null;
                try {
			ResourceContext rtx = ResourceContext.getResourceContext();
			ResourceManagerFactoryHome home = (ResourceManagerFactoryHome) rtx.getResourceHome();
                        resource = (ResourceManagerFactoryResource)home.find(home.frkey);
			clustersstr = (String)resource.getResourcePropertySet().get(new 
				QName(MetaSchedulerConstants.RESOURCE_MANAGER_NS,"Clusters")).get(0);
			logger.debug("clusters:" + clustersstr);
                } catch (Exception e) {
                	logger.error("getFactoryResrouce error!");
                        throw new RemoteException("", e);
		}

		clusters.clear();
		StringTokenizer stcluster = new StringTokenizer(clustersstr," ");
		StringTokenizer st = null;
		while(stcluster.hasMoreTokens()){
			cluster = stcluster.nextToken();
			st  = new StringTokenizer(cluster,":");
			while(st.hasMoreTokens()){
				name = st.nextToken();
				type = st.nextToken();
				host = st.nextToken();
				port = Integer.valueOf(st.nextToken().trim()).intValue();
				ClusterContact cc = new ClusterContact(name,type,host,port);
				logger.debug("add clustercontact to hash table:" + cc.toString());
				clusters.add(cc);
			}
		}
        }
}
