package com.platform.metascheduler.utils;

import com.platform.metascheduler.queuing.QueueConfigParamsType;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.xml.namespace.QName;
import java.util.ArrayList;

import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class ConfigParamsTypeHelper {

    private static Log logger 
	= LogFactory.getLog(ConfigParamsTypeHelper.class);

    public static String QUEUING_FACTORY_CONFIG_TAG = "queuingFactoryConfig";
    public static String QUEUING_QUEUE_CONFIG_TAG = "queueConfig";
    public static String QUEUING_QUEUE_CONFIG_NAME_ATTR = "name";
    public static String QUEUING_QUEUE_CONFIG_PLUGIN_TAG = "plugin";
    public static String QUEUING_QUEUE_CONFIG_PLUGIN_CLASS_ATTR = "class";
    public static String QUEUING_QUEUE_CONFIG_PARAMETER = "parameter";
    public static String QUEUING_QUEUE_CONFIG_PARAMETER_SCHEDULE_INTERVAL="scheduleInterval";
    public static String QUEUING_QUEUE_CONFIG_PARAMETER_THROTTLE="throttle";
    public static String QUEUING_QUEUE_CONFIG_PARAMETER_COMMUNITYGIS="communityGisHandle";

    public static com.platform.metascheduler.queuing.ConfigParamsType
	getQueuingConfigParamsType(Element element, String prefix) {

	ArrayList list = new ArrayList();

	String tag = element.getTagName();
	if (tag == null || 
	    ! (tag.equals(QUEUING_FACTORY_CONFIG_TAG) 
	       || tag.endsWith(":" + QUEUING_FACTORY_CONFIG_TAG)) )  {
	    String error_msg ="getQueuingConfigParamsType: Tag name not match";
	    logger.error(error_msg);
	    return null;
	}

	Node childNode = element.getFirstChild();
	while (childNode != null) {	    

	    if (childNode.getNodeType() != Node.ELEMENT_NODE) {
		childNode = childNode.getNextSibling();
		continue;
	    }
	    Element child = (Element)childNode;
	    String childTag = child.getTagName();

	    if (childTag == null) {
		childNode = childNode.getNextSibling();
		continue;
	    }
	    if (childTag.equals(QUEUING_QUEUE_CONFIG_TAG) 
		|| childTag.endsWith(":" + QUEUING_QUEUE_CONFIG_TAG) ) {

		QueueConfigParamsType queueConfig = getQueuingQueueConfigParamsType(child, prefix);
		if (queueConfig != null) {
		    logger.debug("Add queue okay: " + queueConfig.getName());
		    list.add(queueConfig);
		} else {
		    // ignore
		}
	    }
	    
	    childNode = childNode.getNextSibling();
	}

       	com.platform.metascheduler.queuing.ConfigParamsType config 
	    = new com.platform.metascheduler.queuing.ConfigParamsType();

	QueueConfigParamsType[] a = new QueueConfigParamsType[list.size()];
	a = (QueueConfigParamsType[]) list.toArray(a);
	config.setQueueConfigArray(a);

	return (config);
    }


    /**
     * Create a default queue configuration for queue factory, which 
     * contains only one default queue.
     */
    public static com.platform.metascheduler.queuing.ConfigParamsType
	createDefaultQueuingFactoryConfig(QName name, String[] plugins) {

        QueueConfigParamsType[] queueConfigArray = null;
        queueConfigArray = new QueueConfigParamsType[1];
        queueConfigArray[0]  = new QueueConfigParamsType();
        queueConfigArray[0].setName(name);
        queueConfigArray[0].setPluginArray(plugins);

       	com.platform.metascheduler.queuing.ConfigParamsType 
		config = new com.platform.metascheduler.queuing.ConfigParamsType();
	config.setQueueConfigArray(queueConfigArray);

	return (config);
    }



    private static QueueConfigParamsType
	getQueuingQueueConfigParamsType(Element element, String prefix) {

	String name = element.getAttribute(QUEUING_QUEUE_CONFIG_NAME_ATTR);
	if (name == null) {
	    String error_msg 
		= "getQueuingQueueConfigParamsType: queue name not found";
	    logger.error(error_msg);
	    return null;
	}

	ArrayList list = new ArrayList();
	Integer  scheduleInterval = null;
	Integer  throttle = null; 
	String   gisStr = null;

	Node childNode = element.getFirstChild();
	while (childNode != null) {

	    if (childNode.getNodeType() != Node.ELEMENT_NODE) {
		childNode = childNode.getNextSibling();
		continue;
	    }

	    Element child = (Element)childNode;
	    String childTag = child.getTagName();
	    if (childTag == null) {
		childNode = childNode.getNextSibling();
		continue;
	    }

	    if (childTag.equals(QUEUING_QUEUE_CONFIG_PLUGIN_TAG) ||
		childTag.endsWith(":" + QUEUING_QUEUE_CONFIG_PLUGIN_TAG)) {

		String pluginClass 
	          = child.getAttribute(QUEUING_QUEUE_CONFIG_PLUGIN_CLASS_ATTR);

		if (pluginClass == null) {
		    String error_msg = "getQueuingQueueConfigParamsType: class not found";
		    logger.error(error_msg);
		    // ignore this child
		} else {
		    list.add(pluginClass);
		}
	    }

	    if (childTag.equals(QUEUING_QUEUE_CONFIG_PARAMETER) 
		|| childTag.endsWith(":" + QUEUING_QUEUE_CONFIG_PARAMETER)) { 
		// Some queue parameter
		
		String intervalStr = null; 
		String throttleStr = null; 

		intervalStr = child.getAttribute(QUEUING_QUEUE_CONFIG_PARAMETER_SCHEDULE_INTERVAL);
		if ( intervalStr != null && intervalStr.length() > 0) { 
   	            try {
  		        scheduleInterval = new Integer(intervalStr);
	            } catch (Exception e) {
	                logger.error("Interval value is invalid. The default value is 30 seconds.");
	            }
		} 

		throttleStr = child.getAttribute(QUEUING_QUEUE_CONFIG_PARAMETER_THROTTLE);
		if ( throttleStr != null && throttleStr.length() > 0) { 
   	            try {
		        throttle = new Integer(throttleStr);
	            } catch (Exception e) {
	                logger.error("Throttle value is invalid. The default value is 100.");
	            }
		}

		gisStr = child.getAttribute(QUEUING_QUEUE_CONFIG_PARAMETER_COMMUNITYGIS);
	    }

	    childNode = childNode.getNextSibling();
	}

	QueueConfigParamsType queueConfig = new QueueConfigParamsType();

	queueConfig.setName(new QName (prefix, name));
	queueConfig.setPluginArray((String[])list.toArray(new String[0]));
	if ( scheduleInterval != null ) { 
	    queueConfig.setScheduleInterval(scheduleInterval);
	    logger.debug("queueConfig.schedulerInterval= " + queueConfig.getScheduleInterval().intValue());
	}

	if ( throttle != null ) { 
	    queueConfig.setThrottle(throttle);
	    logger.debug("queueConfig.throttle= " + queueConfig.getThrottle().intValue());
	}

	if ( gisStr != null ) { 
	    queueConfig.setCommunityGisHandle(gisStr);
	    logger.debug("queueConfig.CommunityGisHandle= " + queueConfig.getCommunityGisHandle());
	}

	return queueConfig;
    }
}
