package com.platform.metascheduler.utils;

import com.platform.metascheduler.utils.MetaSchedulerRsl;
import com.platform.metascheduler.utils.RsvAttributes;

import com.platform.metascheduler.reservation.GridReservationType;
import com.platform.metascheduler.reservation.GridReservationStatus;
import com.platform.metascheduler.reservation.ReservationType;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.Text;
import org.w3c.dom.Element;
import org.w3c.dom.DOMImplementation;

import org.apache.xerces.dom.DOMImplementationImpl;
import org.apache.axis.message.MessageElement;
import org.apache.axis.utils.XMLUtils;

import javax.xml.namespace.QName;

import java.text.SimpleDateFormat;
import java.text.ParseException;
import java.util.Date;
import java.util.Calendar;
import java.rmi.RemoteException;
import java.net.URL;
import java.io.StringWriter;

import org.globus.exec.utils.rsl.RSLParseException;

public class GridReservationTypeHelper {
    private static Log logger 
        = LogFactory.getLog(GridReservationTypeHelper.class);
    private static final String GRID_RESERVATION = "GridReservation";
    private static final String GRID_RESERVATION_ID = "ID";
    private static final String GRID_RESERVATION_STATUS = "Status";
    private static final String GRID_RESERVATION_USERID = "UserID";
    private static final String GRID_RESERVATION_RSL = "RSL";
    private static final String GRID_RESERVATION_ACL = "ACL";
    private static final String GRID_RESERVATION_ACL_TEXT = "ACLItem";
    private static final String GRID_RESERVATION_TIME = "ActiveTime";
    private static final String GRID_RESERVATION_TIME_START = "Start";
    private static final String GRID_RESERVATION_TIME_DURATION = "Duration";
    private static final String GRID_RESERVATION_RM_RSVS = "RMReservationInfo";
    private static final String GRID_RESERVATION_RM_RSV = "RMReservation";
    private static final String GRID_RESERVATION_RM_RSV_ID = "ID";
    private static final String GRID_RESERVATION_RM_RSV_LOCATION = "Location";

    public static GridReservationType 
	getGridReservationType(QName gridRsvID, String rsl) 
	throws ParseException, RSLParseException {

	logger.debug("ready to new MetaSchedulerRsl");
	MetaSchedulerRsl msRsl = new MetaSchedulerRsl(rsl);
	Element        msRslRootElement = msRsl.getRootElement();

	RsvAttributes rsvAttr = new RsvAttributes(msRslRootElement);

	GridReservationType rsv = new GridReservationType();
	logger.debug("ready to initialize");
	initialize(rsv);

        logger.debug("rsvAttr: " + rsvAttr.toString());
        boolean timeFlag = false;

        try {
		rsv.setGridRsvID(gridRsvID);
		rsv.setStatus(GridReservationStatus.CREATED);
		rsv.setGridACL(null); // not set
		rsv.setUserID(rsvAttr.getUser());

		Calendar startCal = rsvAttr.getBeginTime();
		Calendar endCal = rsvAttr.getEndTime();
		Calendar nowCal = Calendar.getInstance();
        	if ( startCal.after(nowCal) && endCal.after(nowCal) ) {
            		timeFlag = true;
        	}
        
		// assume it if current year
		startCal.set(Calendar.YEAR, nowCal.get(Calendar.YEAR));
		startCal.set(Calendar.SECOND, 0);
		startCal.set(Calendar.MILLISECOND, 0);
		endCal.set(Calendar.YEAR,   nowCal.get(Calendar.YEAR));
		endCal.set(Calendar.SECOND, 0);
		endCal.set(Calendar.MILLISECOND, 0);

		rsv.setStartTime(startCal);

		long start = startCal.getTimeInMillis();
		long end = endCal.getTimeInMillis();

		rsv.setDuration((end - start) / 1000);
		rsv.setRSL(rsl);
		rsv.setRmRsvs(null); // not set
	} catch ( RSLParseException e) { 
		logger.debug("Cannot get reservation user name");
		throw new ParseException("error in get rsv attributes", 0);
        } 
        if (!timeFlag) {
		throw new ParseException("Invalid reservation time", 0);
        }
	return (rsv);
    }
    public static GridReservationType
	getGridReservationType(String nsURI, String localID, String rsl)
	throws ParseException, RSLParseException {

        return (getGridReservationType(new QName(nsURI, localID), rsl));
    }
/*    public static GridReservationType
	getGridReservationType(String handle, QName gridRsvID) 
	throws RemoteException {

	logger.debug("getGridReservationType(" + gridRsvID.toString() + ", " 
		     + handle + ")");

	GridReservationType rsv = null;

	OGSIServiceGridLocator locator = new OGSIServiceGridLocator();
	GridService service = 
	    locator.getGridServicePort(new HandleType(handle));
	ExtensibilityType queryResult =
	    service.findServiceData(QueryHelper.getNamesQuery(gridRsvID));

	if (queryResult.get_any() == null) {
	    // should not happen, it should at least return an empty 
	    // ServiceDataValuesType.
	    logger.debug("getGridReservationType(): No value found");
	    return null;
	}
	ServiceDataValuesType serviceData 
	    = AnyHelper.getAsServiceDataValues(queryResult);

	if (serviceData == null || serviceData.get_any() == null) {
	    logger.debug("getGridReservationType(): No value found");
	    return null;
	}

        ServiceDataAttributes[] attrs = AnyHelper.getAsServiceData(serviceData);
	if (attrs == null || attrs.length < 1) {
	    logger.debug("getGridReservationType(): No value found");
	    return null;
	}
        
	Object obj = attrs[0].getValue();
	if (! (obj instanceof Element) ) {
	    logger.warn("getGridReservationType(): value not Element");
	    return null;
	}

	Element rsvElement = (Element)obj;
	rsv = (GridReservationType)ObjectSerializer.toObject(rsvElement,
					       GridReservationType.class);

	logger.debug("getGridReservationType(): got " + toString(rsv)); 
	
	return rsv;
    }
*/

/*    public static void updateGridReservationType (GridReservationType rsv,
						  ServiceData serviceData,
						  HandleType orginator) 
    {
        ServiceDataAttributes attributes = null;
        try {
	    attributes =
	      new ServiceDataAttributes(ObjectSerializer.toElement(rsv));
	} catch (Exception e) {
	    e.printStackTrace();
	    return;
	}

	Calendar nowCal = Calendar.getInstance();
        Calendar hourLaterCal = Calendar.getInstance();
	hourLaterCal.add(Calendar.HOUR_OF_DAY, 1);

	attributes.setAvailableUntil(new ExtendedDateTimeType(hourLaterCal));
	attributes.setGoodFrom(new ExtendedDateTimeType(nowCal));
	attributes.setGoodUntil(new ExtendedDateTimeType(hourLaterCal));
	attributes.setOriginator(orginator);
	serviceData.setValue(attributes);
	
	serviceData.notifyChange();
	return;
    }
*/
/*    public static ServiceData putGridReservationType (GridReservationType rsv,
						      GridServiceBase service)
	throws GridServiceException 
    {
	logger.debug("RSV: " + toString(rsv));

	ServiceData serviceData = null;
	ServiceDataSet serviceDataSet = service.getServiceDataSet();
	QName serviceDataName = rsv.getGridRsvID();
        try {
	    String handle = 
		(String)service.getProperty(ServiceProperties.HANDLE);
            serviceData = serviceDataSet.create(serviceDataName);
	    updateGridReservationType(rsv, serviceData, new HandleType(handle));
	    serviceDataSet.add(serviceData);
        } catch (Exception e) {
            throw new GridServiceException(e);
        }

	return serviceData;
    }*/

    private static void initialize(GridReservationType rsv) {
	rsv.setGridRsvID(new QName("", ""));
	rsv.setStatus(GridReservationStatus.CREATED);
	rsv.setRSL("");
	rsv.setGridACL(new String[0]);
	rsv.setUserID("");
	rsv.setStartTime(Calendar.getInstance());
	rsv.setDuration(0);
	rsv.setRmRsvs(new ReservationType[0]);
    }

    /**
     * Generate a xml document to represent the GridReservationType
     *
     * @param rsv Target grid reservation type 
     *
     * @return A xml document, or null if fail
     */
    public static Document toDocument(GridReservationType rsv) {
	Document xmlDoc = null;
	QName    qname = rsv.getGridRsvID();

	logger.debug("RSV_ID: " + qname.toString());

	try {
	    DOMImplementation impl
		= DOMImplementationImpl.getDOMImplementation();

	    xmlDoc = impl.createDocument(null, GRID_RESERVATION, null);

	    Element rsvElement = xmlDoc.getDocumentElement();
	    rsvElement.setAttribute(GRID_RESERVATION_ID, 
				    rsv.getGridRsvID().toString());
	    rsvElement.setAttribute(GRID_RESERVATION_STATUS, 
				    rsv.getStatus().toString());
	    rsvElement.setAttribute(GRID_RESERVATION_USERID, rsv.getUserID());

	    Element rslElement =
		xmlDoc.createElement(GRID_RESERVATION_RSL);
	    String rsl = rsv.getRSL();
	    if (rsl != null && !rsl.equals("")) {
		MetaSchedulerRsl msRsl = new MetaSchedulerRsl(rsv.getRSL());
		Element         msRslRootElement = msRsl.getRootElement();

		Node newNode = xmlDoc.importNode(msRslRootElement, true);
		rslElement.appendChild(newNode);
	    }
	    rsvElement.appendChild(rslElement);

	    Element aclElement =
		xmlDoc.createElement(GRID_RESERVATION_ACL);
	    String[] acl = rsv.getGridACL();
	    if (acl != null) {
		for (int i=0; i<acl.length; i++) {
		    Element aclTextElement =
			xmlDoc.createElement(GRID_RESERVATION_ACL_TEXT);
		    Text aclText = xmlDoc.createTextNode(acl[i]);
		    aclTextElement.appendChild(aclText);
		    aclElement.appendChild(aclTextElement);
		}
	    }
	    rsvElement.appendChild(aclElement);

	    Element timeElement = 
		xmlDoc.createElement(GRID_RESERVATION_TIME);
	    Element startTimeElement = 
		xmlDoc.createElement(GRID_RESERVATION_TIME_START);
	    Text startTimeText = 
		xmlDoc.createTextNode(rsv.getStartTime().toString());
	    startTimeElement.appendChild(startTimeText);
	    timeElement.appendChild(startTimeElement);
	    Element durationTimeElement =
		xmlDoc.createElement(GRID_RESERVATION_TIME_DURATION);
	    Text durationTimeText = 
		xmlDoc.createTextNode(String.valueOf(rsv.getDuration()));
	    durationTimeElement.appendChild(durationTimeText);
	    timeElement.appendChild(durationTimeElement);
	    rsvElement.appendChild(timeElement);

	    Element rmRsvsElement = 
		xmlDoc.createElement(GRID_RESERVATION_RM_RSVS);
	    ReservationType[] rm_rsvs = rsv.getRmRsvs();
	    if (rm_rsvs != null) {
		for (int i=0; i<rm_rsvs.length; i++) {
		    Element rmRsvElement =
			xmlDoc.createElement(GRID_RESERVATION_RM_RSV);
		    rmRsvElement.setAttribute(GRID_RESERVATION_RM_RSV_ID,
				      rm_rsvs[i].getRsvID().toString());
		    String[] locations = rm_rsvs[i].getLocations();
		    if (locations != null) {
			Element locationElement = null;
			for (int k=0; k<locations.length; k++) {
			    locationElement = xmlDoc.createElement
				(GRID_RESERVATION_RM_RSV_LOCATION);
			    Text locationText = 
				xmlDoc.createTextNode(locations[k]);
			    locationElement.appendChild(locationText);
			    rmRsvElement.appendChild(locationElement);
			}
		    }
		    rmRsvsElement.appendChild(rmRsvElement);
		}
	    }
	    rsvElement.appendChild(rmRsvsElement);

	} catch (Exception e) {
	    e.printStackTrace();
	    return null;
	}
	return (xmlDoc);
    }

    /**
     * Generate a xml document string to represent the GridReservationType
     *
     * @param rsv Target grid reservation type 
     *
     * @return A xml document string, or null if fail
     */
    public static String toXMLString(GridReservationType rsv) {

	if ( rsv == null){
	    return (null);
        }

        try {
            StringWriter writer = new StringWriter();
            XMLUtils.PrettyDocumentToWriter(toDocument(rsv), writer);
            return (writer.toString());
        } catch (Exception e) {
            return (null);
        }

    }

    public static String toString(GridReservationType rsv) {
	if ( rsv == null) { 
	    return (null); 
	}
	return toXMLString(rsv);
    }

    public static long getRSLNumber(GridReservationType rsv) 
	throws RSLParseException {
	MetaSchedulerRsl msRsl = new MetaSchedulerRsl(rsv.getRSL());
	Element        msRslRootElement = msRsl.getRootElement();
	RsvAttributes rsvAttr = new RsvAttributes(msRslRootElement);
	long number = 0;

	try {
	    number = rsvAttr.getNumber();
	} catch (Exception e) {
	    return -1;
	}

	return number;
    }
}
