/*
 * This file or a portion of this file is licensed under the terms of the
 * Globus Toolkit Public  License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without modifications,
 * you must include this notice in the file.
 */
package com.platform.metascheduler.utils;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.globus.wsrf.ResourceHome;
import org.globus.wsrf.ResourceContext;
import org.globus.wsrf.ResourceContextException;
import org.globus.wsrf.Constants;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class JNDIHelper {

    public static ResourceHome getResourceHome(String service)
        throws NamingException
    {
        return (ResourceHome)getJNDIObject(service, Constants.HOME_NAME);
    }

    public static Object getJNDIObjectForCurrentService(String objectName)
        throws NamingException, ResourceContextException
    {
        ResourceContext ctx = ResourceContext.getResourceContext();
        return getJNDIObject(ctx.getService(), objectName);
    }

    public static Object getResourceHomeConfiguration(String service)
        throws NamingException
    {
        return getJNDIObject(service, "homeConfig");
    }

    /**
     *
     * @param service String The service within the JNDI context of which
     *                       the object is located
     * @param objectName String the name of the JNDI resource to look up.
     * @throws NamingException if the JNDI lookup fails
     * @return Object
     */
    public static Object getJNDIObject(String service, String objectName)
        throws NamingException
    {
        String jndiLookupString = null;
        Object jndiObject;
        try {
            Context initialContext = new InitialContext();
            if (!objectName.startsWith("/")) {
                objectName = "/" + objectName;
            }
            jndiLookupString = getServiceJNDILocation(service) + objectName;
            jndiObject = initialContext.lookup(jndiLookupString);
        } catch (NamingException e) {
            String errorMessage =
                "Problem getting GRAM JNDI configuration resource " +
                "with lookup string: " +  jndiLookupString;
            logger.error(errorMessage, e);
            throw e;
        }
        return jndiObject;
    }

    public static String getServiceJNDILocation(String service) {
        return Constants.JNDI_SERVICES_BASE_NAME + service;
    }

    public static String getServiceJNDILocation()
        throws ResourceContextException
    {
        ResourceContext ctx = ResourceContext.getResourceContext();
        return Constants.JNDI_SERVICES_BASE_NAME + ctx.getService();
    }

    private static Log logger =
        LogFactory.getLog(JNDIHelper.class.getName());

}
