package com.platform.metascheduler.utils;

import java.util.Hashtable;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.w3c.dom.Attr;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;
import org.w3c.dom.Document;

import javax.xml.namespace.QName;
import org.globus.exec.utils.rsl.RSLParseException;
//import org.globus.ogsa.impl.base.gram.utils.rsl.UnresolvedSubstitutionReferencesException;
//import org.globus.ogsa.impl.base.gram.utils.rsl.AttributeNotFoundException;

public class JobAttributes extends MsAttributesImpl {
    static Log logger = LogFactory.getLog(JobAttributes.class);

    static String JOB_NAMESPACE_URI = MetaSchedulerConstants.METASCHEDULER_RSL_NS;

    public JobAttributes() {
    }

    public JobAttributes(Element rslRootElement) {
        setRslRootElement(rslRootElement, MetaSchedulerConstants.JOB);
    }

    // Gram specific attributes
    public String getExecutable()
            throws RSLParseException { 
        return (String) getAttribute(new QName(MetaSchedulerConstants.GRAM_RSL_NS,
                                               MetaSchedulerConstants.JOB_EXECUTABLE));
    }

    public String getDirectory()
            throws  RSLParseException {
        return (String) getAttribute(new QName(MetaSchedulerConstants.GRAM_RSL_NS,
                                               MetaSchedulerConstants.JOB_DIRECTORY)); 
    } 

    public String[] getArguments()
            throws  RSLParseException {
        return (String[]) getAttribute(new QName(MetaSchedulerConstants.GRAM_RSL_NS,
                                               MetaSchedulerConstants.JOB_ARGUMENTS));
    }

    public Hashtable getEnvironment()
            throws RSLParseException { 
        return (Hashtable) getAttribute(new QName(MetaSchedulerConstants.GRAM_RSL_NS,
                                               MetaSchedulerConstants.JOB_ENVIRONMENT));
    }

    public String getStdin()
            throws  RSLParseException {
        return (String) getAttribute(new QName(MetaSchedulerConstants.GRAM_RSL_NS,
                                               MetaSchedulerConstants.JOB_STDIN));
    }

    public String[] getStdout()
            throws  RSLParseException {
        return (String []) getAttribute(new QName(MetaSchedulerConstants.GRAM_RSL_NS,
                                               MetaSchedulerConstants.JOB_STDOUT));
    }

    // MetaScheduler specific attributes
    public String getCluster()
            throws  RSLParseException {
        return (String) getAttribute(new QName(MetaSchedulerConstants.METASCHEDULER_RSL_NS,
                                               MetaSchedulerConstants.JOB_CLUSTER));
    }

    public String getReservationId()
            throws  RSLParseException {
        return (String) getAttribute(new QName(MetaSchedulerConstants.METASCHEDULER_RSL_NS,
                                               MetaSchedulerConstants.JOB_RESERVATIONID));
    }

    public String getGridReservationId()
            throws  RSLParseException {
        return (String) getAttribute(new QName(MetaSchedulerConstants.METASCHEDULER_RSL_NS,
                                               MetaSchedulerConstants.JOB_GRIDRESERVATIONID));
    }

    public String getJobInstanceId()
            throws  RSLParseException {
        return (String) getAttribute(
                    new QName(MetaSchedulerConstants.METASCHEDULER_RSL_NS,
                              MetaSchedulerConstants.JOB_INSTANCEID));
    }

    public String getFactory()
            throws  RSLParseException {
        return (String) getAttribute(
                    new QName(MetaSchedulerConstants.METASCHEDULER_RSL_NS,
                              MetaSchedulerConstants.JOB_FACTORY));
    }

    public String[] getStderr()
            throws  RSLParseException {
        return (String []) getAttribute(new QName(MetaSchedulerConstants.GRAM_RSL_NS,
                                               MetaSchedulerConstants.JOB_STDERR));
    }

    public int getCount()
	    throws  RSLParseException {
        return ((Integer) getAttribute(new QName(MetaSchedulerConstants.GRAM_RSL_NS, 
						 MetaSchedulerConstants.JOB_COUNT))).intValue();
    }

    public String getProject()
            throws  RSLParseException {
        return (String) getAttribute(new QName(MetaSchedulerConstants.GRAM_RSL_NS,
                                               MetaSchedulerConstants.JOB_PROJECT));
    }

    public String getQueue()
            throws  RSLParseException {
        return (String) getAttribute(new QName(MetaSchedulerConstants.GRAM_RSL_NS,
                                               MetaSchedulerConstants.JOB_QUEUE));
    }

    public long getMaxTime()
            throws  RSLParseException {
        return ((Long) getAttribute(new QName(MetaSchedulerConstants.GRAM_RSL_NS,
                                               MetaSchedulerConstants.JOB_MAXTIME))).longValue();
    }

    public long getMaxWallTime()
            throws  RSLParseException {
        return ((Long) getAttribute(new QName(MetaSchedulerConstants.GRAM_RSL_NS,
                                               MetaSchedulerConstants.JOB_MAXWALLTIME))).longValue();
    }

    public long getMaxCpuTime()
            throws  RSLParseException {
        return ((Long) getAttribute(new QName(MetaSchedulerConstants.GRAM_RSL_NS,
                                               MetaSchedulerConstants.JOB_MAXCPUTIME))).longValue();
    }

    public int getMaxMemory()
            throws  RSLParseException {
        return ((Integer) getAttribute(new QName(MetaSchedulerConstants.GRAM_RSL_NS,
                                               MetaSchedulerConstants.JOB_MAXMEMORY))).intValue();
    }

    public String getFactoryType()
            throws  RSLParseException {
        return (String) getAttribute(
                    new QName(MetaSchedulerConstants.METASCHEDULER_RSL_NS,
                              MetaSchedulerConstants.JOB_FACTORY_TYPE));
    }

    public String toString() {
 
       StringBuffer infoBuffer = new StringBuffer();

        try {
            infoBuffer.append("executable: " + this.getExecutable() + "\n");
        } catch ( RSLParseException e) {
            infoBuffer.append("executable: NULL\n");
        }

	try {
	    infoBuffer.append("directory: " + this.getDirectory() + "\n");
        } catch ( RSLParseException e) {
            infoBuffer.append("directory: NULL\n");
        }

	try {
	    infoBuffer.append("stdin: " + this.getStdin() + "\n");
        } catch ( RSLParseException e) {
            infoBuffer.append("stdin: NULL\n");
        }

	try {
	    String[] stdout = this.getStdout();
	    for (int i = 0; i<stdout.length; i++) {
		infoBuffer.append("stdout[" + i  + "]: " + stdout[i] + "\n");
	    }
        } catch ( RSLParseException e) {
            infoBuffer.append("stdout: NULL\n");
        }

	try {
	    String[] stderr = this.getStderr();
	    for (int i = 0; i<stderr.length; i++) {
		infoBuffer.append("stderr[" + i  + "]: " + stderr[i] + "\n");
	    }
        } catch ( RSLParseException e) {
            infoBuffer.append("stderr: NULL\n");
        }

	try {
	    String[] arguments = this.getArguments();
	    for (int i = 0; i<arguments.length; i++) {
		infoBuffer.append("arguments[" + i + "]: " + arguments[i] +
				  "\n");
	    }
        } catch ( RSLParseException e) {
            infoBuffer.append("arguments: NULL\n");
        }

	try {
	    infoBuffer.append("factory: " + this.getFactory() + "\n");
        } catch ( RSLParseException e) {
            infoBuffer.append("factory: NULL\n");
        }

	try {
	    infoBuffer.append("jobInstanceId: " + this.getJobInstanceId() + "\n");
        } catch ( RSLParseException e) {
            infoBuffer.append("jobInstanceId: NULL\n");
        }

	try {
	    Hashtable environment = this.getEnvironment();

	    for (java.util.Enumeration e = environment.keys() ; 
		 e.hasMoreElements() ;) {
		String key = (String)e.nextElement();
		infoBuffer.append("environment: " + key + "=" + 
				  environment.get(key) + "\n");
	    }
 
        } catch ( RSLParseException e) {
            infoBuffer.append("environments: NULL\n");
        }

	try {
	    infoBuffer.append("count: " + this.getCount()+ "\n");
        } catch ( RSLParseException e) {
            infoBuffer.append("count: NULL\n");
        }

	try {
	    infoBuffer.append("project: " + this.getProject() + "\n");
        } catch ( RSLParseException e) {
            infoBuffer.append("project: NULL\n");
        }

	try {
	    infoBuffer.append("queue: " + this.getQueue() + "\n" );
        } catch ( RSLParseException e) {
            infoBuffer.append("queue: NULL\n");
        }

	try {
	    infoBuffer.append("maxTime: " + this.getMaxTime()   + "\n" );
        } catch ( RSLParseException e) {
            infoBuffer.append("maxTime: NULL\n");
        }

	try {
	    infoBuffer.append("maxWallTime: " + this.getMaxWallTime() + "\n" );
        } catch ( RSLParseException e) {
            infoBuffer.append("maxWallTime: NULL\n");
        }

	try {
	    infoBuffer.append("maxCpuTime: " + this.getMaxCpuTime() + "\n" );
        } catch ( RSLParseException e) {
            infoBuffer.append("maxCpuTime: NULL\n");
        }

	try {
	    infoBuffer.append("maxMemory" +  this.getMaxMemory() + "\n" );
        } catch ( RSLParseException e) {
            infoBuffer.append("maxMemory: NULL\n");
        }

	try {
	    infoBuffer.append("cluster:" + this.getCluster() + "\n"  );
        } catch ( RSLParseException e) {
            infoBuffer.append("cluster: NULL\n");
        }


        try {       
            infoBuffer.append("gridReservationId:" + this.getGridReservationId() + "\n");
        } catch ( RSLParseException e) {
            infoBuffer.append("gridReservationId: NULL\n");
        }   
                 
	try {
	    infoBuffer.append("reservationId:" + this.getReservationId() + "\n");
        } catch ( RSLParseException e) {
            infoBuffer.append("reservationId: NULL\n");
        }

        return (infoBuffer.toString());
    }
}


