package com.platform.metascheduler.utils;

import com.platform.metascheduler.utils.MetaSchedulerRsl;
import com.platform.metascheduler.utils.JobAttributes;

import com.platform.metascheduler.job.JobDataType;
import com.platform.metascheduler.job.GridJobStatus;

import org.w3c.dom.Document;
import org.w3c.dom.Text;
import org.w3c.dom.Element;
import org.w3c.dom.DOMImplementation;

import org.apache.xerces.dom.DOMImplementationImpl;
import org.apache.axis.message.MessageElement;
import org.apache.axis.utils.XMLUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.xml.namespace.QName;

import java.text.SimpleDateFormat;
import java.text.ParseException;
import java.util.Date;
import java.util.Calendar;
import java.rmi.RemoteException;
import java.net.URL;

public class JobDataHelper {

    private Thread m_owner = null;
    private int m_lock_count = 0;
    
    private static Log logger 
        = LogFactory.getLog(JobDataHelper.class);

    public JobDataHelper() {
        logger.trace("In JobDataHelper");
    }

    /**
     * Generate a xml document string from the JobData
     *
     * @return A xml document string, or null if fail
     */
    public static String toXmlString(JobDataType jobdata) {
        String xml = null;

        DOMImplementation impl
                = DOMImplementationImpl.getDOMImplementation();

        Document xmlDoc
                = impl.createDocument(null, "JobData", null);

        Element jobElement = xmlDoc.getDocumentElement();
        try {

            Element jobIdElement =
                xmlDoc.createElement("jobId");
            Text jobIdText = xmlDoc.createTextNode(jobdata.getJobId().toString());
            jobIdElement.appendChild(jobIdText);
            jobElement.appendChild(jobIdElement);
        } catch (Exception e) {
            //do nothing
        }

        try {
            Element stateElement =
                xmlDoc.createElement("state");
            Text stateText = xmlDoc.createTextNode(jobdata.getState().toString());
            stateElement.appendChild(stateText);
            jobElement.appendChild(stateElement);
        } catch (Exception e) {
            // do nothing
        }

        try {
            Element handleIdElement =
                xmlDoc.createElement("handleId");
            Text handleIdText = xmlDoc.createTextNode(jobdata.getHandleId().toString());
            handleIdElement.appendChild(handleIdText);
            jobElement.appendChild(handleIdElement);
        } catch (Exception e) {
            // do nothing
        }

        try {
            Element rmJobIdElement =
                xmlDoc.createElement("rmJobId");
            Text rmJobIdText = xmlDoc.createTextNode(jobdata.getRmJobId().toString());
            rmJobIdElement.appendChild(rmJobIdText);
            jobElement.appendChild(rmJobIdElement);
        } catch (Exception e) {
            // do nothing
        }

        try {
            Element clusterNameElement =
                xmlDoc.createElement("clusterName");
            Text clusterNameText = xmlDoc.createTextNode(jobdata.getClusterName().toString());
            clusterNameElement.appendChild(clusterNameText);
            jobElement.appendChild(clusterNameElement);
        } catch (Exception e) {
            // do nothing
        }

        try {
            Element queueNameElement =
                xmlDoc.createElement("queueName");
            Text queueNameText = xmlDoc.createTextNode(jobdata.getQueueName().toString());
            queueNameElement.appendChild(queueNameText);
            jobElement.appendChild(queueNameElement);
        } catch (Exception e) {
            // do nothing
        }

        try {
            Element runAsElement =
                xmlDoc.createElement("runAs");
            Text runAsText = xmlDoc.createTextNode(jobdata.getRunAs().toString());
            runAsElement.appendChild(runAsText);
            jobElement.appendChild(runAsElement);
        } catch (Exception e) {
            // do nothing
        }

        try {
            Element rslElement =
                xmlDoc.createElement("rsl");
            MetaSchedulerRsl msRsl = new MetaSchedulerRsl(jobdata.getRsl());
            Element         msRslRootElement = msRsl.getRootElement();
            JobAttributes jobAttr = new JobAttributes(msRslRootElement);
            Text rslText = xmlDoc.createTextNode(jobAttr.toString());
            rslElement.appendChild(rslText);
            jobElement.appendChild(rslElement);
        } catch (Exception e) {
            // do nothing
        }

        try {
            xml = XMLUtils.DocumentToString(xmlDoc);

        } catch (Exception e) {
            return null;
        }
        return xml;

    }
}

