/**
 * MetaSchedulerRsl.java
 *
 * This class reads/writes an RSL file which conforms to the 
 * metascheduler.xsd schema. 
 */
package com.platform.metascheduler.utils;

import java.io.BufferedReader;
import java.io.Reader;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.StringReader;
import java.io.StringWriter;
import java.io.IOException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.apache.xml.serialize.LineSeparator;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.ErrorHandler;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXNotRecognizedException;
import org.xml.sax.SAXNotSupportedException;
import org.xml.sax.SAXParseException;
//import org.globus.ogsa.impl.base.gram.utils.rsl.Xerces2RslParserImpl;
import org.globus.exec.utils.rsl.RSLParseException;

import com.platform.metascheduler.utils.MetaSchedulerConstants;

//public class MetaSchedulerRsl extends Xerces2RslParserImpl {
public class MetaSchedulerRsl extends org.xml.sax.helpers.DefaultHandler {
    static Log logger = LogFactory.getLog(MetaSchedulerRsl.class);
    private Element           m_element = null;
    private Document          m_document = null;
    private boolean           m_reparse = true;

    public MetaSchedulerRsl() {
        super( );
    }

    public MetaSchedulerRsl(String rslString) throws RSLParseException{
	logger.debug("in MetaSchedulerRsl ready to parse rsl");
        m_element = parse(new StringReader(rslString));
    }

    public void setReparse(boolean flag) {
        m_reparse = flag;
    }

    public Element getRootElement( ) { 
	return (m_element);
    }

    public Document getDocument() {
	return (m_document);
    }

    public Element parse(Reader rslReader)
            throws  RSLParseException {

        /**
         *  Do not parse the document if it is already parsed.
         *  (Unless the m_reparse flag is set)
         */
        if ( m_document != null && 
             m_element != null && !m_reparse) {
            return m_element;
        }

        DocumentBuilderFactory parserFactory = null;
        DocumentBuilder xmlParser = null;
    
        try {
            parserFactory = DocumentBuilderFactory.newInstance();
            parserFactory.setNamespaceAware(true);
            parserFactory.setValidating(true);
            parserFactory.setAttribute(
                "http://java.sun.com/xml/jaxp/properties/schemaLanguage",
                "http://www.w3.org/2001/XMLSchema");
            xmlParser = parserFactory.newDocumentBuilder();
            xmlParser.setErrorHandler(this);

            BufferedReader bufferedReader = new BufferedReader(rslReader);
            m_document = xmlParser.parse(new InputSource(bufferedReader));

        } catch (Exception e) {
		logger.error(e.getMessage());
            throw new RSLParseException("exception during RSL parsing", e);
        }

        m_element = m_document.getDocumentElement();
	m_reparse = false; 
        return m_element;
    }

    public void readRslFile(String fileName) {
        try {
            parse(new FileReader(fileName));
        } catch (Exception e) {
            logger.error("RslFile:readRslFile parsing error:" , e);
        }
    }

    public void writeRslFile(String fileName) {
        OutputFormat format = new OutputFormat(m_document);
        format.setLineSeparator(LineSeparator.Web);
        // format.setIdenting(true);
        format.setLineWidth(80);
        format.setPreserveSpace(true);
	try { 
	    XMLSerializer serializer = new XMLSerializer(new FileWriter(fileName), format);
	    serializer.asDOMSerializer();
	    serializer.serialize(m_document);
	} catch (IOException e) { 
	    logger.error("cannot write rsl to file " + fileName);
	}
    }

    public String writeRslString() {
        StringWriter writer = new StringWriter();
        OutputFormat format = new OutputFormat(m_document);
        format.setLineSeparator(LineSeparator.Web);
        // format.setIdenting(true);
        format.setLineWidth(80);
        format.setPreserveSpace(true);

	try { 
	    XMLSerializer serializer = new XMLSerializer(writer, format);
	    serializer.asDOMSerializer();
	    serializer.serialize(m_document);
	} catch (IOException e) { 
	    logger.error("cannot write rsl to a string");
	}
        return writer.toString();
    }

    public NodeList getElementByName(String elementName) 
                   throws RSLParseException {
        return m_element.getElementsByTagNameNS(
                         MetaSchedulerConstants.METASCHEDULER_RSL_NS, 
                         elementName);
    }

    /**
     * setElementValue
     * 
     * Add a new element value, ie. Add jobInstanceId element into
     * the rsl file.
     */
    public Object setElementValue(String elementName, Object value) {
        String   tempVal = null;
	NodeList nodeList = null;
        
        try {
            nodeList = getElementByName(elementName);
	} catch (Exception e) {
        }
	
	return (nodeList);
    }

    /**
     * setAttributeValue
     * 
     * Set a new element value, ie. Set/change the value of a element.
     * the rsl file.
     */
    public Object setAttributeValue(String attributeName, Object value) {
	return (null);
    }

}
