package com.platform.metascheduler.utils;

import java.io.File;
import java.io.StringReader;
import java.io.StringWriter;

import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Calendar;
import java.util.Vector;
import java.util.List;
import java.util.ArrayList;
import java.util.StringTokenizer;

import javax.xml.soap.Name;
import javax.xml.soap.SOAPElement;
import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;

import org.apache.xerces.dom.DocumentImpl;

import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.CharacterData;
import org.w3c.dom.Text;
import org.xml.sax.InputSource;

import org.globus.exec.utils.rsl.RSLParseException;
//import org.globus.ogsa.impl.base.gram.utils.rsl.UnresolvedSubstitutionReferencesException;
//import org.globus.ogsa.impl.base.gram.utils.rsl.AttributeNotFoundException;


public abstract class MsAttributesImpl implements MsAttributes {
    static Log logger = LogFactory.getLog(MsAttributesImpl.class);

    protected static String MS_NAMESPACE_URI = "http://com.platform.metascheduler/2003/05/rsl/metascheduler";
    protected static String RSL_NAMESPACE_URI = "http://www.globus.org/namespaces/2004/02/rsl";
    protected static String MS_REQUEST = "request";
    protected static String VALUE_ATTRIBUTE_NAME = "value";
    protected static String NAME_ATTRIBUTE_NAME = "name";
    protected static String GRAM_EXEC = "executable";

    Element rslRootElement = null;
    Element msElement = null;

    //NodeList cachedSubScopes = null;
    NodeList cachedSubDefs = null;
    NodeList cachedSubRefs = null;

    Hashtable implicitSubstitutions = new Hashtable();

    boolean allSubstitutionsResolved = false;
    private boolean isMetaSchedulerJob = true;

    public MsAttributesImpl() { }

    public boolean isMetaSchedulerJob() {
        return isMetaSchedulerJob;
    }

    // Set msElement to REQUEST 
    public void setRslRootElement(Element rslRootElement) {
        this.rslRootElement = rslRootElement;
	logger.debug(rslRootElement);

        cacheSubstitutionEntities();

        int subDefCount = cachedSubDefs.getLength();
	logger.debug("subDefCount=" + subDefCount);
        Node msSearchNode = null;
        if (subDefCount > 0) {
            msSearchNode = cachedSubDefs.item(subDefCount-1);
        } else {
            msSearchNode = this.rslRootElement.getFirstChild();
        }
	if (msSearchNode == null) {
		logger.debug("msSearchNode == null");
	}
	logger.debug("msSearchNode =" + msSearchNode.getLocalName());
	logger.debug(msSearchNode.getNodeName());
	logger.debug(msSearchNode.getNodeValue());
        boolean foundMsElement = false;
        while ((msSearchNode != null) && !foundMsElement) {
            if (msSearchNode.getNodeName().endsWith(MS_REQUEST) ||
                msSearchNode.getNodeName().endsWith("job") ||
                msSearchNode.getNodeName().endsWith("executable")) {
                foundMsElement = true;
		logger.debug("found" + msSearchNode.getNodeName());
                if ( msSearchNode.getNodeName().endsWith("executable")) {
                    isMetaSchedulerJob = false;
		    logger.debug("isn't meta");
                }
            } else {
                msSearchNode = msSearchNode.getNextSibling();
            }
        }
        if (msSearchNode != null) {
		if (msSearchNode.getNodeName().endsWith("executable") ) {
			this.msElement = this.rslRootElement;
	    	} else {
            		this.msElement = (Element) msSearchNode;
		}
        } else { 
	    logger.error("There is no defined element: " + MS_REQUEST + " or job");
	}
    }

    // Set msElement to specified name. This name has to be the direct
    // child of REQUEST
    public void setRslRootElement(Element rslRootElement, String name) {
	setRslRootElement(rslRootElement); 
        Node    msSearchNode = this.msElement.getFirstChild();
	boolean foundMsElement = false;
        while ((msSearchNode != null) && !foundMsElement) {
            if (msSearchNode.getNodeName().endsWith(name)) {
                foundMsElement = true;
            } else {
                msSearchNode = msSearchNode.getNextSibling();
            }
        }
        if (msSearchNode != null) {
	    logger.debug("Find element with name " + name);
            this.msElement = (Element) msSearchNode;
        } else { 
	    logger.debug("Cannot find node with name: " + name);
	}
    }

    public Element getRslRootElement() {
        return this.rslRootElement;
    }

    public Element getMsElement() { 
	return this.msElement; 
    } 

    private void cacheSubstitutionEntities() {
        this.cachedSubDefs = this.rslRootElement.getElementsByTagNameNS(
                RSL_NAMESPACE_URI,
                "substitutionDef");

        this.cachedSubRefs = this.rslRootElement.getElementsByTagNameNS(
                RSL_NAMESPACE_URI,
                "substitutionRef");
    }

    public Iterator substitutionDefinitionNames() {
        int subDefCount = this.cachedSubDefs.getLength();

        HashSet subDefNames = new HashSet(subDefCount);
        for (int index=0; index<subDefCount; index++) {
            Element subDefElement = (Element) this.cachedSubDefs.item(index);
            subDefNames.add(subDefElement.getAttribute("name"));
        }
        return subDefNames.iterator();
    }

    public boolean isSubstitutionDefined(String substitutionName) {
        return
          (getSubstitutionDefinitionElement(substitutionName)!=null)?true:false;
    }

    public Iterator substitutionReferences() {
        int subRefCount = this.cachedSubRefs.getLength();

        HashSet subRefNames = new HashSet(subRefCount);
        for (int index=0; index<subRefCount; index++) {
            Element subRefElement = (Element) this.cachedSubRefs.item(index);
            subRefNames.add(subRefElement.getAttribute("name"));
        }
        return subRefNames.iterator();
    }

    public boolean isSubstitutionReferenced(String substitutionName) {
        int subRefCount = this.cachedSubRefs.getLength();

        for (int index=0; index<subRefCount; index++) {
            Element subRefElement = (Element) this.cachedSubRefs.item(index);
            if (subRefElement.getAttribute("name").equals(substitutionName)) {
                return true;
            }
        }

        return false;
    }

    public void setSubstitutionDefinition(
            String substitutionName,
            String substitutionValue) {

        //Parse the substitutionValue as an XML blob to get an Element
        DocumentBuilderFactory parserFactory = null;
        DocumentBuilder xmlParser = null;
        Document subDefDocument = null;
        try {
            parserFactory = DocumentBuilderFactory.newInstance();
            xmlParser = parserFactory.newDocumentBuilder();

            StringReader stringReader = new StringReader(substitutionValue);
            subDefDocument = xmlParser.parse(new InputSource(stringReader));
        } catch (Exception e) {
            logger.error("exception during substitution definition parsing", e);
            return;
        }

        Element subDefRootElement = subDefDocument.getDocumentElement();
        subDefRootElement.getAttribute(VALUE_ATTRIBUTE_NAME);
        setSubstitutionDefinition(substitutionName, subDefRootElement);
    }

    public void setSubstitutionDefinition(
            String substitutionName,
            Element substitutionValue) {

        Element existingSubDefElement
            = getSubstitutionDefinitionElement(substitutionName);
        Document rslDocument
            = (Document) this.rslRootElement.getParentNode();

        Node subDefValueNode
            = ((DocumentImpl)rslDocument).adoptNode(substitutionValue);

        if (existingSubDefElement != null) {
            //substitution definition exists, so just set the value
            existingSubDefElement.removeChild(
                    existingSubDefElement.getFirstChild());
            existingSubDefElement.appendChild(subDefValueNode);
        } else {
            //create the substitution definition element
            Element subDefElement = rslDocument.createElementNS(
                    RSL_NAMESPACE_URI,
                    "rsl:substitutionDef");
            subDefElement.setAttribute("name", substitutionName);
            subDefElement.appendChild(subDefValueNode);

            //add the sub def element to the current scope
            Element firstSubDef = (Element) cachedSubDefs.item(0);
            if (firstSubDef != null) {
                this.rslRootElement.insertBefore(
                        subDefElement,
                        firstSubDef);
            } else {
                this.rslRootElement.insertBefore(
                        subDefElement,
                        this.msElement);
            }

            cacheSubstitutionEntities();
        }
    }

    public NodeList getSubstitutionDefinition(String substitutionName)
            throws RSLParseException {
        Element subDefElement
            = getSubstitutionDefinitionElement(substitutionName);
        if (subDefElement == null) return null;

        NodeList subDefValueNodes = subDefElement.getChildNodes();
        int subDefValueNodeCount = subDefValueNodes.getLength();
        Vector elementVector = new Vector();

        Vector unresolvedSubRefs = new Vector(subDefValueNodeCount);
        StringBuffer valueBuffer = new StringBuffer();
        for (int index=0; index<subDefValueNodeCount; index++) {
            Node subDefValueNode = subDefValueNodes.item(index);
            if (subDefValueNode instanceof Element) {
                //found unresolved substitution reference
                if (subDefValueNode.getNodeName().indexOf("substitutionRef")
                    >= 0) {
                    unresolvedSubRefs.add(
                        ((Element) subDefValueNode).getAttribute("name"));
                } else {
                    elementVector.add(subDefValueNode);
                }
            }
        }
        if (unresolvedSubRefs.size() > 0) {
            //report all unresolved substitution references
            throw new RSLParseException(substitutionName);
        }

        return subDefValueNodes;
    }

    protected Element getSubstitutionDefinitionElement(
            String substitutionName) {
        int subDefCount = this.cachedSubDefs.getLength();

        for (int index=0; index<subDefCount; index++) {
            Element subDefElement = (Element) this.cachedSubDefs.item(index);
            if (subDefElement.getAttribute("name").equals(substitutionName)) {
                return subDefElement;
            }
        }

        return null;
    }

    public void resolveAllSubstitutions()
            throws RSLParseException {
        resolveSubstitutionsInSubtree(this.rslRootElement);
        allSubstitutionsResolved = true;
    }

    protected void resolveSubstitutionsInSubtree(Element subtreeRootElement)
            throws RSLParseException {

        if (allSubstitutionsResolved) return;

        boolean isUnresolvedReferences = false;
        Vector unresolvedReferences = new Vector();
        Element unresolvedElement = null;

        //get a list of all the substitution definitions
        int defCount = cachedSubDefs.getLength();

        //find all unresolved substitution references
        NodeList subtreeSubRefs = subtreeRootElement.getElementsByTagNameNS(
                RSL_NAMESPACE_URI,
                "substitutionRef");
        int refCount = subtreeSubRefs.getLength();

        //loop through all the unresolved substitution references
        for (int index=0; index<refCount; index++) {

            //item(0) is correct here, since we essentially remove each
            //sub ref from the node list when we replace it with the sub value
            Element refElement = (Element) subtreeSubRefs.item(0);
            //

            Element parentElement = (Element) refElement.getParentNode();

            //notify the calling method of the unresolved refs in this element
            if (isUnresolvedReferences && (unresolvedElement!=parentElement)) {
                throw new RSLParseException( unresolvedElement.getTagName());
            }

            //Search for the referenced substitution definition
            boolean foundSubDef = false;
            Element defElement = null;
            for (int defIndex=0; !foundSubDef && (defIndex<defCount);
                                defIndex++) {
                defElement = (Element) cachedSubDefs.item(defIndex);
                if (defElement.getAttribute("name").equals(
                    refElement.getAttribute("name"))) {
                    foundSubDef = true;
                }
            }

            if (!foundSubDef) {
                //uh oh, the referenced substitution definition wasn't found
                isUnresolvedReferences = true;
                unresolvedElement = parentElement;
                unresolvedReferences.add(refElement.getAttribute("name"));
            } else {
                //first cache the sub ref nodes sibling, if it exists
                Node siblingNode = refElement.getNextSibling();

                //make room for the sub definition
                parentElement.removeChild(refElement);

                //put the sub definition's contents where the ref was
                Node defElementChild = defElement.getFirstChild();
                while (defElementChild != null) {
                    //figure out where to stick the sub definition
                    if (siblingNode != null) {
                        parentElement.insertBefore(
                            defElementChild.cloneNode(true),
                            siblingNode);
                    } else {
                        parentElement.appendChild(
                            defElementChild.cloneNode(true));
                    }

                    defElementChild = defElementChild.getNextSibling();
                }
                //parentElement.normalize();
            }
        }

        //notify the calling method of the unresolved refs in this element
        if (isUnresolvedReferences) {
            Object[] unresolvedRefObjects = unresolvedReferences.toArray();
            String[] unresolvedRefStrings
                = new String[unresolvedRefObjects.length];
            for (int index=0; index<unresolvedRefObjects.length; index++) {
                unresolvedRefStrings[index]
                    = (String) unresolvedRefObjects[index];
            }
            throw new RSLParseException( unresolvedElement.getTagName());
        }
    }

    protected String getTextData(Element element) {
        //node has only text data
        NodeList textNodes = element.getChildNodes();
        int textNodeCount = textNodes.getLength();

        StringBuffer textData = new StringBuffer();
        for (int index=0; index<textNodeCount; index++) {
            Node textNode = textNodes.item(index);
            if (textNode instanceof Text) {
                textData.append(((Text) textNode).getData());
            }
        }

        return textData.toString();
    }

    protected String getDefaultAttributeValue(Element element) {
        return element.getAttribute(VALUE_ATTRIBUTE_NAME);
    }

    protected Object getValueNodeData(Element valueParentElement) {
        valueParentElement.normalize();

        StringBuffer attributeValue = new StringBuffer();

        NodeList attributeValueNodes = null;
        if (    valueParentElement.hasAttributes()
            && !valueParentElement.hasChildNodes()) {
            //We have a basic type which is using an attribute
            //instead of child nodes to express it's value
            attributeValueNodes
                = valueParentElement.getParentNode().getChildNodes();
        } else {
            //We have a type which uses child nodes to express it's value
            attributeValueNodes = valueParentElement.getChildNodes();
        }
        int attributeValueNodeCount = attributeValueNodes.getLength();

        Hashtable hashtable = null;

        for (int index=0; index<attributeValueNodeCount; index++) {
            Node attributeValueNode = attributeValueNodes.item(index);
            if (attributeValueNode instanceof Text) {
                attributeValue.append(
                    ((Text) attributeValueNode).getData().trim());
            } else if (attributeValueNode instanceof Element) {
                boolean hasAttributes = attributeValueNode.hasAttributes();
                boolean hasChildren = attributeValueNode.hasChildNodes();
                if (!hasAttributes && !hasChildren) {
                    //completely empty tag--use tag name
                    attributeValue.append(attributeValueNode.getLocalName());
                } else if (!hasAttributes && hasChildren) {
                    //node has a child that represents the value by it's name
                    return getValueNodeData((Element) attributeValueNode);
                } else if (hasAttributes && !hasChildren) {
                    //node has value in "value" attribute
                    attributeValue.append(
                            ((Element) attributeValueNode).getAttribute(
                                    VALUE_ATTRIBUTE_NAME));
                } else if (hasAttributes && hasChildren) {
                    //name/value pair node
                    if (hashtable == null) {
                        hashtable = new Hashtable();
                    }

                    String key = ((Element) attributeValueNode).getAttribute(
                            NAME_ATTRIBUTE_NAME).trim();
                    attributeValue.append(getValueNodeData(
                        (Element) attributeValueNode).toString().trim());

                    hashtable.put(key, attributeValue.toString().trim());
                    attributeValue.delete(0, attributeValue.length());
                }
            }
        }

        if (hashtable != null) {
            return hashtable;
        }

        return attributeValue.toString().trim();
    }

    /**
     * Returns an iteration of all the RSL attribute names
     *
     * @return java.util.Iterator
     */
    public Iterator attributeNames() {
        NodeList attributes = this.msElement.getChildNodes();
        int nodeCount = attributes.getLength();

        Vector attributeNames = new Vector(nodeCount);
        for (int index=0; index<nodeCount; index++) {
            Node attributeNode = attributes.item(index);
            if (attributeNode instanceof Element) {
                Element attribute = (Element) attributeNode;
                attributeNames.add(new QName(   attribute.getNamespaceURI(),
                                                attribute.getLocalName()));
            }
        }

        return attributeNames.iterator();
    }

    protected Object[] getArrayAttribute(Element dataTypeElement) {
        NodeList valueElements = dataTypeElement.getChildNodes();

        int nodeCount = valueElements.getLength();
        Vector valueVector = new Vector(nodeCount/2);

        for (int index=0; index<nodeCount; index++) {
            Node valueElement = valueElements.item(index);
            if (valueElement instanceof Element) {
                valueVector.add(
                    (String) getValueNodeData((Element) valueElement));
            }
        }

        return valueVector.toArray();
    }

    protected String[] getStringArrayAttribute(Element dataTypeElement) {
        Object[] objectArray = getArrayAttribute(dataTypeElement);

        int arrayLength = objectArray.length;

        String[] stringArray = new String[arrayLength];
        for (int index=0; index<arrayLength; index++) {
            stringArray[index] = (String) objectArray[index];
        }

        return stringArray;
    }

    protected String[][] getTwoDimensionalStringArrayAttribute(Element dataTypeElement) {
        Vector stringArrays = new Vector();

        Node child = dataTypeElement.getFirstChild();
        while (child != null) {
            if (child instanceof Element) {
                stringArrays.add(getStringArrayAttribute((Element) child));
            }
            child = child.getNextSibling();
        }

        Iterator stringArrayIterator = stringArrays.iterator();
        int largestArrayLength = 0;
        while (stringArrayIterator.hasNext()) {
            String[] stringArray = (String[]) stringArrayIterator.next();
            if (stringArray.length > largestArrayLength) {
                largestArrayLength = stringArray.length;
            }
        }

        int arrayCount = stringArrays.size();
        String[][] twoDimensionalStringArray
            = new String[arrayCount][largestArrayLength];
        for (int outerIndex=0; outerIndex<arrayCount; outerIndex++) {
            String[] stringArray = (String[])stringArrays.elementAt(outerIndex);
            for (int innerIndex=0;innerIndex<largestArrayLength; innerIndex++) {
                if (innerIndex < stringArray.length) {
                    twoDimensionalStringArray[outerIndex][innerIndex]
                        = stringArray[innerIndex];
                } else {
                    twoDimensionalStringArray[outerIndex][innerIndex] = null;
                }
            }
        }

        return twoDimensionalStringArray;
    }

    protected String[] getFileIoArrayAttribute(Element dataTypeElement) {
        Vector fileInputs = new Vector();

        Node child = dataTypeElement.getFirstChild();
        while ((child != null) && !(child instanceof Element)) {
            child = child.getNextSibling();
        }
        fileInputs.add(getStringArrayAttribute((Element) child));

        int arrayLength = fileInputs.size() * 2;
        String[] fileInputArray = new String[arrayLength];
        for (int index=0; index<arrayLength; index+=2) {
            String[] fileInput = (String[]) fileInputs.elementAt(index/2);
            fileInputArray[index] = fileInput[0];
            fileInputArray[index+1] = fileInput[1];
        }

        return fileInputArray;
    }

    //RSL schema type methods
    public String getMsNamespaceURI() {
        return MS_NAMESPACE_URI;
    }

    protected Object getAttribute(Element attribute) {
        Node walkNode = attribute.getFirstChild();

        while ((walkNode != null) && !(walkNode instanceof Element)) {
            walkNode = walkNode.getNextSibling();
        }
	
	if ( walkNode == null) { 
	    return (null); 
	}

        Element dataTypeElement = (Element) walkNode;
        String dataTypeName = dataTypeElement.getTagName();

        if (dataTypeName.endsWith("string")) {
            return (String) getValueNodeData(dataTypeElement);
        } else if (dataTypeName.endsWith("integer")) {
            return new Integer((String) getValueNodeData(dataTypeElement));
        } else if (dataTypeName.endsWith("long")) {
            return new Long((String) getValueNodeData(dataTypeElement));
        } else if (dataTypeName.endsWith("float")) {
            return new Float((String) getValueNodeData(dataTypeElement));
        } else if (dataTypeName.endsWith("double")) {
            return new Double((String) getValueNodeData(dataTypeElement));
        } else if (dataTypeName.endsWith("boolean")) {
            return new Boolean((String) getValueNodeData(dataTypeElement));
        } else if (dataTypeName.endsWith("url")) {
            return (String) getValueNodeData(dataTypeElement);
        } else if (dataTypeName.endsWith("path")) {
            return (String) getValueNodeData(dataTypeElement);
        } else if (dataTypeName.endsWith("hashtable")) {
            return (Hashtable) getValueNodeData(dataTypeElement);
        } else if (dataTypeName.endsWith("enumeration")) {
            return (String) getValueNodeData(dataTypeElement);
        } else if (dataTypeName.endsWith("fileInput")) {
            return getStringArrayAttribute(dataTypeElement);
        } else if (dataTypeName.endsWith("fileOutput")) {
            return getStringArrayAttribute(dataTypeElement);
        } else if (dataTypeName.endsWith("stringArray")) {
            return getStringArrayAttribute(dataTypeElement);
        } else if (dataTypeName.endsWith("pathArray")) {
            return getStringArrayAttribute(dataTypeElement);
        } else if (dataTypeName.endsWith("fileInputArray")) {
            return getTwoDimensionalStringArrayAttribute(dataTypeElement);
        } else if (dataTypeName.endsWith("fileOutputArray")) {
            return getTwoDimensionalStringArrayAttribute(dataTypeElement);
        } else if (dataTypeName.endsWith("twoDimensionalStringArray")) {
            return getTwoDimensionalStringArrayAttribute(dataTypeElement);
        } else if (dataTypeName.endsWith("dateTime")) {
//            return new Calendar((String) getValueNodeData(dataTypeElement));
	    return ReservationHelper.stringToCalendar((String) getValueNodeData(dataTypeElement));
        } 

        return attribute;
    }

    public Object getAttribute(QName name)
            throws  RSLParseException {

        //find the first occurrence of the requested attribute
        String namespaceURI = name.getNamespaceURI();
        String attributeName = name.getLocalPart();

        Element attribute = null;

        Node attributeNode = getDirectChildNode(this.msElement,
                                                namespaceURI,
                                                attributeName);

        if (attributeNode != null) {
            attribute = (Element) attributeNode;
        } else { 
	    logger.debug("Couldn't find the attribute with QNAME: " + name);
	}
	

        if (attribute != null) {
            resolveSubstitutionsInSubtree(attribute);
	
            Object obj = getAttribute(attribute);
	    logger.debug("QNAME: " + name);
	    logger.debug("Value: " + obj);
	    return (obj);
        }

        throw new RSLParseException (
                  (namespaceURI!=null)?"{"+namespaceURI+"}":""
                + attributeName);
    }

    //Primitives
    public Integer getIntegerAttribute(QName name)
        throws  RSLParseException {
        Object attributeValue = getAttribute(name);
        if (!(attributeValue instanceof Integer)) {
            String namespaceURI = name.getNamespaceURI();
            throw new RSLParseException(name.toString());
        }

        return (Integer) attributeValue;
    }

    public Long getLongAttribute(QName name)
        throws  RSLParseException {
        Object attributeValue = getAttribute(name);
        if (!(attributeValue instanceof Long)) {
            throw new RSLParseException(name.toString());
        }

        return (Long) attributeValue;
    }

    public Float getFloatAttribute(QName name)
        throws  RSLParseException {
        Object attributeValue = getAttribute(name);
        if (!(attributeValue instanceof Float)) {
            throw new RSLParseException(name.toString());
        }

        return (Float) attributeValue;
    }

    public Double getDoubleAttribute(QName name)
        throws  RSLParseException {
        Object attributeValue = getAttribute(name);
        if (!(attributeValue instanceof Double)) {
            throw new RSLParseException(name.toString());
        }

        return (Double) attributeValue;
    }

    public Boolean getBooleanAttribute(QName name)
        throws  RSLParseException {
        Object attributeValue = getAttribute(name);
        if (!(attributeValue instanceof Boolean)) {
            throw new RSLParseException(name.toString());
        }

        return (Boolean) attributeValue;
    }

    public String getStringAttribute(QName name)
        throws  RSLParseException {
        Object attributeValue = getAttribute(name);
        if (!(attributeValue instanceof String)) {
            throw new RSLParseException(name.toString());
        }

        return (String) attributeValue;
    }

    public String getPathAttribute(QName name)
        throws  RSLParseException {
        return getStringAttribute(name);
    }

    public String getUrlAttribute(QName name)
        throws  RSLParseException {
        return getStringAttribute(name);
    }

    //Complex
    public Hashtable getHashtableAttribute(QName name)
        throws  RSLParseException {
        Object attributeValue = getAttribute(name);
        if (!(attributeValue instanceof Hashtable)) {
            throw new RSLParseException(name.toString());
        }

        return (Hashtable) attributeValue;
    }

    public String[] getFileInputAttribute(QName name)
        throws  RSLParseException {
        Object attributeValue = getAttribute(name);
        if (!(attributeValue instanceof String[])) {
            throw new RSLParseException(name.toString());
        }

        return (String[]) attributeValue;
    }

    public String[] getFileOutputAttribute(QName name)
        throws  RSLParseException {
        return getStringArrayAttribute(name);
    }
    public String getEnumerationAttribute(QName name)
        throws  RSLParseException {
        return getStringAttribute(name);
    }

    //Arrays
    public String[] getStringArrayAttribute(QName name)
        throws  RSLParseException {
        Object attributeValue = getAttribute(name);
        if (!(attributeValue instanceof String[])) {
            throw new RSLParseException(name.toString());
        }

        return (String[]) attributeValue;
    }

    public String[][] getTwoDimensionalStringArrayAttribute(QName name)
        throws  RSLParseException {
        Object attributeValue = getAttribute(name);
        if (!(attributeValue instanceof String[][])) {
            throw new RSLParseException(name.toString());
        }

        return (String[][]) attributeValue;
    }

    public String[] getPathArrayAttribute(QName name)
        throws  RSLParseException {
        return getStringArrayAttribute(name);
    }

    public String[][] getFileInputArrayAttribute(QName name)
        throws  RSLParseException {
        return getTwoDimensionalStringArrayAttribute(name);
    }

    public String[][] getFileOutputArrayAttribute(QName name)
        throws  RSLParseException {
        return getTwoDimensionalStringArrayAttribute(name);
    }

    protected String arrayToString(Object attribute) {
        StringBuffer string = new StringBuffer();

        if ((attribute != null) && attribute.getClass().isArray()) {
            string.append("(");
            Object[] arrayAttribute = (Object[]) attribute;
            for (int index=0; index<arrayAttribute.length; index++) {
                String nestedArrayString = arrayToString(arrayAttribute[index]);
                if (nestedArrayString != null) {
                    string.append(nestedArrayString);
                } else {
                    string.append((String) arrayAttribute[index]);
                    if (index < (arrayAttribute.length-1)) {
                        string.append(", ");
                    }
                }
            }
            string.append(")");
        } else {
            return null;
        }

        return string.toString();
    }

    public String toString() {
        StringBuffer string = new StringBuffer();

        Iterator attributeNamesIter = attributeNames();
        while (attributeNamesIter.hasNext()) {
            QName attributeName = (QName) attributeNamesIter.next();
            string.append(attributeName.toString());
            string.append("-->");
            try {
                Object attribute = getAttribute(attributeName);
                String arrayString = arrayToString(attribute);
                if (arrayString != null) {
                    string.append(arrayString);
                } else {
                    string.append(attribute.toString());
                }
            } catch (RSLParseException anfe) {
                logger.error("problem getting RSL attribute", anfe);
                string.append("NULL");
            }
            string.append('\n');
        }

        return string.toString();
    }

    /**
     * Returns an RSL string for the MS attributes in the current RSL format.
     * Shields the consumer from the implementation of the attributes structure.
     * @return the RSL string for the job attributes.
     */
    public String toRSL() {
        String xmlRSL = null;
        Element rootElement = rslRootElement;
        OutputFormat format = new OutputFormat(
                rootElement.getOwnerDocument());
        format.setLineWidth(65);
        format.setIndenting(true);
        format.setIndent(2);
        StringWriter writerToString = new StringWriter();
        try {
            XMLSerializer serializer = new XMLSerializer(writerToString,format);
            serializer.asDOMSerializer();
            serializer.serialize(rootElement);
            xmlRSL = writerToString.toString();
        }
        catch (Exception e) {
            logger.error("Could not serialize RSL string", e);
        }
        return xmlRSL;
    }

    /**
     * Extract RSL element from XML element which contains it directly.
     */
    public static Element getRslRootElement(Element directParent) {
        return (Element)getDirectChildNode(directParent,
                                           RSL_NAMESPACE_URI,
                                           "rsl");
        //later on we can add recursive behavior if useful.
    }

    protected static Node getDirectChildNode(Node parent,
                                             String namespaceURI,
                                             String name) {
        Node currentNode = parent.getFirstChild();
        boolean foundNode = false;
        while (!foundNode && (currentNode != null)) {
            String nodeName = currentNode.getLocalName();
            String nodeNamespaceURI = currentNode.getNamespaceURI();
            if (            (namespaceURI == null)
                        &&  (nodeName != null)
                        &&  nodeName.equals(name) ) {
                foundNode = true;
            } else if (     (namespaceURI != null)
                        &&  (nodeNamespaceURI != null)
                        &&  nodeNamespaceURI.equals(namespaceURI)
                        &&  (nodeName != null)
                        &&  nodeName.equals(name) ) {
                foundNode = true;
            } else {
                currentNode = currentNode.getNextSibling();
            }
        }
        return currentNode;
    }

    protected static List getDirectChildrenNodes(Node parent,
                                                 String namespaceURI,
                                                 String name) {
        List listNodes = new ArrayList();
        Node currentNode = parent.getFirstChild();
        while (currentNode != null) {
            String nodeName = currentNode.getLocalName();
            String nodeNamespaceURI = currentNode.getNamespaceURI();
            if (            (namespaceURI == null)
                        &&  (nodeName != null)
                        &&  nodeName.equals(name) ) {
                listNodes.add(currentNode);
            } else if (     (namespaceURI != null)
                        &&  (nodeNamespaceURI != null)
                        &&  nodeNamespaceURI.equals(namespaceURI)
                        &&  (nodeName != null)
                        &&  nodeName.equals(name) ) {
                listNodes.add(currentNode);
            }

            currentNode = currentNode.getNextSibling();

        }
        return listNodes;
    }
}
