#!/usr/bin/env perl

use strict;
use Getopt::Long;

my $GL = $ENV{'GLOBUS_LOCATION'};
my $LD = $ENV{'LD_LIBRARY_PATH'};
my $username;
my $hash;
my $passwd;
my $rc;
my $yes;
my $help;
my $pwd=`pwd`;
chomp($pwd);

GetOptions( 'y!' => \$yes,
            'pass=s' => \$passwd,
            'hash=s' => \$hash,
            'user=s' => \$username,
            'help|h' => \$help);

if ($help)
{
    print "Usage:\n";
    print "$0 [-y] [-pass=<pass>] [-hash=<hash>] [-user=<user>] [-help]\n";
    print;
    print "   -y:     Confirm that creating new usercerts is okay.  Required\n";
    print "   -pass:  Supply a password.  If not specified, will use a \n";
    print "           password from \$HOME/.globus/.simplecapass.  If that\n";
    print "           file does not exist, will prompt for the password\n";
    print "   -hash:  Specify a CA hash to request a cert from.  If not\n";
    print "           specified, will use the default CA.\n";
    print "   -user:  Specify the user account to create a cert for.  If not\n";
    print "           specified, will prompt for a username.\n";
    print "   -help:  Print this help message.\n";
    exit 0;
}

if(!$yes)
{
    print "This script will destroy any usercerts you may have in\n";
    print "\$GLOBUS_LOCATION/etc.  If this is ok (ie a new build)\n";
    print "Then run with the -y option.\n";

    exit 0;
}

if ( $GL )
{
    print "Setting up $GL\n";
    $ENV{'PATH'} = "$GL/bin:$ENV{'PATH'}";
    if ( $LD )
    {
        $ENV{'LD_LIBRARY_PATH'} = "$GL/lib:$LD";
    }
    else
    {
        $ENV{'LD_LIBRARY_PATH'} = "$GL/lib";
    }
} 
else 
{
   print "Please set GLOBUS_LOCATION to your install dir.\n";
   exit 1;
}

if(!$ENV{'GPT_LOCATION'})
{
    $ENV{'GPT_LOCATION'} = $GL;
}

open(PASSFILE, "<$ENV{HOME}/.globus/.simplecapass");
if ( $? eq 0 )
{
   $passwd = <PASSFILE>;
   chomp $passwd;
   print "Using password from $ENV{HOME}/.globus/.simplecapass\n";
}
close(PASSFILE);

if(!$passwd)
{
    print "Please enter a password for the CA: ";
    system("stty -echo");
    $passwd = <STDIN>;
    system("stty echo");
    chomp($passwd);
}

if(!$username)
{
    print "\nUser account:";
    $username = <STDIN>;
    chomp($username);
    print;
} else
{
    print "\nCreating certificate for user $username\n";
}

#if(!$hash)
#{
    #print "\nENter CA hash:";
    #$hash = <STDIN>;
    #chomp($hash);
#}

my $logfile="gt-add-user.log";
system("touch $logfile");

#
# make a fake dir for certs to tar up and give to user
#
my $tmpdir="$pwd/tmp_gtset";

if( -d "$tmpdir" )
{
    system("rm -rf $tmpdir");
}
system("mkdir $tmpdir");
system("mkdir $tmpdir/certificates/");

# create the request for the cert
$rc = system("$GL/bin/grid-cert-request -cn \"$username\" -nopw -dir $tmpdir > $logfile 2>&1");
if($rc != 0)
{
    printf "FAILED\n";
    print "$GL/bin/grid-cert-request -cn \"$username\" -nopw -dir $tmpdir\n";
    exit 1;
}

# sign the cert
$rc = system("$GL/bin/grid-ca-sign -in $tmpdir/usercert_request.pem -out $tmpdir/usercert.pem -passin pass:$passwd -force  > $logfile 2> $logfile");
if($rc != 0)
{
    printf "FAILED\n";
    print "$GL/bin/grid-ca-sign -in $tmpdir/usercert_request.pem -out $tmpdir/usercert.pem -passin pass:<passwd> -force\n";
    exit 1;
}
print "cp $GL/share/certificates/$hash.* $tmpdir/certificates\n";
system("cp $GL/share/certificates/$hash.* $tmpdir/certificates");
system("tar -C $tmpdir/ -zcf \"$username\"_dot_globus.tar.gz .");

print("The tarball ${username}_dot_globus.tar.gz has been created.\n");
print("The user for which you are running this script should untar it in their ~/.globus  directory\n");

my $dn = `$GL/bin/grid-cert-info -file $tmpdir/usercert.pem -subject`;
chomp $dn;

print "Creating a grid-mapfile in $ENV{'HOME'}/.gridmap\n";

system("$GL/sbin/grid-mapfile-delete-entry -dn \"$dn\" -ln $username -mapfile $ENV{'HOME'}/.gridmap > /dev/null 2>&1");

$rc = system("$GL/sbin/grid-mapfile-add-entry -dn \"$dn\" -ln $username -mapfile $ENV{'HOME'}/.gridmap > $logfile 2> $logfile");
if($rc != 0)
{
    printf "FAILED\n";
    print "$GL/sbin/grid-mapfile-add-entry -dn \"$dn\" -ln $username -mapfile $ENV{'HOME'}/.gridmap\n";
    exit 1;
}
