#! /bin/sh

#
# Portions of this file Copyright 1999-2005 University of Chicago
# Portions of this file Copyright 1999-2005 The University of Southern California.
#
# This file or a portion of this file is licensed under the
# terms of the Globus Toolkit Public License, found at
# http://www.globus.org/toolkit/download/license.html.
# If you redistribute this file, with or without
# modifications, you must include this notice in the file.
#

	
trap _cleanup 0 

_cleanup () {
  : # Just a stub, just in case we bomb right away
}


# The following is not working correctly.
#trap "_cleanup ; kill -CHLD" INT TERM



ID="echo ${PROGRAM_NAME}[$$]"
PROGRAM_ID="${ID}"

GLOBUS_HOSTNAME="`${bindir}/globus-hostname`"

globus_date () {
   TZ=GMT
   ${GLOBUS_SH_DATE-date} -u | ${GLOBUS_SH_SED-sed} 's/UTC/GMT/'
}

globus_source () {
  FILE="$1"

  if [ ! -f ${FILE} ] ; then
    ${GLOBUS_SH_PRINTF-printf} "${FILE} not found." >&2
    exit 1
  fi
  . ${FILE}
}



if [ -z "$GLOBUS_MDS_DEFAULT_TTL" ] ; then
  _DEFAULT_TTL="undefined"
else
  _DEFAULT_TTL="$GLOBUS_MDS_DEFAULT_TTL"
fi

# Read in site specific information
if [ ! -f ${sysconfdir}/grid-info.conf ] ; then
   ${GLOBUS_SH_PRINTF-printf} "${sysconfdir}/grid-info.conf not found."
   exit 1
fi
. ${sysconfdir}/grid-info.conf


# Setup default meta-values for all objects

# Name of the object
objectName=

# Default time to live
ttl=

# Time Zone is force to be GMT
lastupdate=

   
field (){ 
  if [ "${GLOBUS_SH_CUT-cut}" != "" ]
  then
	${GLOBUS_SH_CUT-cut} -d" " -f$1 
  fi
}

get_value () {
       seperator=$1 
       ${GLOBUS_SH_CUT-cut} -d"$1" -f2-
}


############################################################
#                         CLEANUP
############################################################
# Call in case of some error

_cleanup () {
  # make a call to user define function "cleanup," if it is defined


  type cleanup 2>&1 | ${GLOBUS_SH_GREP-grep} function 2>&1 >/dev/null
  if [ $? -eq 0 ] ; then
    cleanup
  fi
  test -z "$WRAP_STDIN" || ${GLOBUS_SH_RM-rm} -f $WRAP_STDIN >/dev/null 2>&1
  test -z "$TMP_STDIN"  || ${GLOBUS_SH_RM-rm} -f $TMP_STDIN  >/dev/null 2>&1
}


_error () {
  # Print an error message ($1) and then exit 

  echo `globus_date` `$ID` "Error ($PROGRAM_NAME): $1" 1>&2
  exit 1
}


_warning () {
  # Print an error message ($1) and then exit 

  echo "Warning ($PROGRAM_NAME): $1" 1>&2
}



########################################
#  Set Default Values for command line options
PIPELINE=TRUE
CHECK=FALSE
BOOT=FALSE
RDN=

########################################
#  Clear LOCAL_ARGS so shell can process its own arguments

LOCAL_ARGS=

# To accept local args a script must set the following ENV VAR to 'true'
ACCEPT_LOCAL_ARGS='no'

########################################
# This is a support script that is sourced by all main
# MDS-build related scripts.  It provides two usefull routines:
#       validateInput and  generateOuput.
#
# "readCommandLine"  reads the command line for valid
#    options and sets appropriate Environment variables
#
# "validateInput" determines if the input from STDIN 
#    is of the correct form, i.e., it is a valid template.
#    The decision is based on the fields defined in the
#    FIELDS variable, which is defined in the parent script.
#
#  "generateOuput" processes all the defined FIELDS and 
#     either echos them to STDOUT (in LDIF format) or
#     or substitutes (via sed) the value within the 
#     template.
#

# This script depends on the following environment variables
# being set:  
#
#     PIPELINE : Indicates the type of processing
#                TRUE = Pipeline,  FALSE = echo
#     CHECK    : If TRUE, input validation occurs
#     BOOT     : If TRUE, automatically bootstraps the pipeline
#     FIELDS   : The names of all relevent fields 
#                That need to be processed
#
# It also assume that values that are to be process are
# defined with the "eval $FIELDS"
#
# For example, 
#     if   FIELDS="hn hostname" then
#        $hn and $hostname
#     are defined (IF) these fields are to be present 
#     in the output



############################################################
#                         printUsage
############################################################

printUsage () {
  echo "Usage: $PROGRAM_NAME [options]" 
  echo "  where options are"
  echo "       -rdn value"
  echo "           Sets the RDN value to \"value.\""
  echo "       -usage"
  echo "           Displays this message" 
  echo "       -version"
  echo "           Displays the current version number"
  echo "       -boot"
  echo "           Bootstraps the pipeline procedure by"
  echo "           obtaining the correct CLDIF file"
  echo "       -e"
  echo "           Echos each attribute (name-value pair)"
  echo "           to standard out (stdout) "
  echo "       -check"
  echo "           Performs input validation"
#  printLocalUsage
  echo ""
}


############################################################
#                         readCommandLine
############################################################

readCommandLine () {

  while [ "X$1" !=  "X" ]
  do
    case $1 in
      -version )
            echo "$PROGRAM_NAME version $PROGRAM_VERSION" 1>&2
            exit 0 ;;
      -usage )
            printUsage 1>&2
	    exit 0 ;;
      -boot )
            BOOT=TRUE ;;
      -e )
            PIPELINE=FALSE ;;
      -check )
            CHECK=TRUE ;;
      -rdn )
            RDN="$2"
            shift ;;
      -* )
	   if [ "X$ACCEPT_LOCAL_ARGS" != "Xtrue" ] ; then
              LOCAL_ARGS="$LOCAL_ARGS $1"
              # Read the argument if present
              if [ -n "$2" ] ; then 
                case $2 in
                   -* )  ;;
                    * )  
                         LOCAL_ARGS="$LOCAL_ARGS $2" 
                         shift ;;
                esac 
              fi
            else
              _warning "unknown command line option"
              printUsage 1>&2
              exit 1
            fi ;;
      * )
            _warning "unknown command line option" 
            printUsage 1>&2
            exit 1
    esac
    shift ;
  done
  return
}



############################################################
#                         validateInput
############################################################

validateInput () {


  ########################################
  # Validate input if in PIPELINE mode
  
  if [ "X$PIPELINE" != "XTRUE" ]
  then
     # No validation on input since no input
     :
  else
     WRAP_STDIN=${tmpdir}/$$_wrap
     if [ "X$BOOT" = "XTRUE" ]
     then
        if [ "X$OBJECT_CLASS" = "XGlobusTop" ]
        then
          _error "No default template for GlobusTop" 
          exit 1
        fi
        if [ ! -f $TEMPLATEDIR/${OBJECT_CLASS}.template ]
        then
           _error "Missing template file missing ${OBJECT_CLASS}.template"
           exit 1
        fi
	${GLOBUS_SH_CAT-cat} $TEMPLATEDIR/${OBJECT_CLASS}.template >$WRAP_STDIN
     else
        ${GLOBUS_SH_CAT-cat} >$WRAP_STDIN
     fi


     # If the optimization switch is on don't validate input
     if [ "X$CHECK" = "XFALSE" ] 
     then 
        return
     fi

     FIELDS=$*  

     for f in $FIELDS
     do
       # The -q (quit) option to grep is not portable
       # The -s (suppress error) option is handled the same way
       ${GLOBUS_SH_GREP-grep} $f  $WRAP_STDIN  > /dev/null  2>&1
       if [ "$?" != "0" ] 
       then
         _error "Erroneous Input file, field \"$f\" is listed"
         exit 1
       fi
     done
  fi

}


############################################################
#                         EPILOGUE
############################################################

generateOutput () {

  FIELDS=$*
  objectName="$dn"

   # Setup meta-values for all objectes

   # Name of the object
   if [ -z "$objectName" ] ; then 
     objectName="$dn"
   fi

   # Default time to live
   if [ -z "$ttl" ] ; then
     ttl="${_DEFAULT_TTL}"
   fi

   if [ -z "$lastupdate" ] ; then 
     lastupdate="`globus_date`"
   fi


  if [ "X$PIPELINE" != "XTRUE" ]
    then
      # Echo each defined field in LDIF formt
      for f in $FIELDS
      do 
        eval value="\$$f"
        if [ -n "$value" ] 
          then
            echo "$f: $value"
  	fi
      done
    else
      TMP_STDIN=${tmpdir}/$$_tmp
      # Insert each defined field into the template
      for f in $FIELDS
      do
        eval value="\$$f"
        if [ -n "$value" ]
          then
            # Warning we overwrite values that are defined
	    value=`echo "$value" | ${GLOBUS_SH_SED-sed} -e 1p -e d`
            ${GLOBUS_SH_SED-sed} -e "/^$f:/ s|:.*$|: $value|" $WRAP_STDIN >$TMP_STDIN
            mv $TMP_STDIN $WRAP_STDIN
          fi
      done
      ${GLOBUS_SH_CAT-cat} $WRAP_STDIN
      #Force a blank line
      echo

      test -z "$WRAP_STDIN" || ${GLOBUS_SH_RM-rm} -f $WRAP_STDIN
      test -z "$TMP_STDIN" || ${GLOBUS_SH_RM-rm} -f $TMP_STDIN

    fi
}




###########################################################
#                   SOME LIBRARY FUNCTIONS
###########################################################


add_attribute () {
  # inputs: name value
  # Inserts an additional attribute to a template

  ${GLOBUS_SH_SED-sed} -e "
/$1: *$/ a\\
$1: $2\
\
"

  # SED finds the first BLANK attribute of the appropriate type
  #     then appends the desired field.
  # We expect that the BLANK attribute is removed in later processing.
}




ip2name () {
   _IP=${1:?"IP number is required"}

  echo `${GLOBUS_SH_NSLOOKUP-nslookup} $_IP | ${GLOBUS_SH_SED-sed} -n '/^Name/p' | ${GLOBUS_SH_AWK-awk} '{print $2}'`
}



_build_network_ip () {

  _IP=${1:?"IP number is required"}

  _network=`echo $_IP | ${GLOBUS_SH_AWK-awk} -F. '{print $1":"$2":"$3":"$4}'`
  #Colons(:) are used for sanity -- Dot (.) is a sed metacharater

  _network=`echo $_network       |\
            ${GLOBUS_SH_SED-sed}  -e 's/::/:0:/' \
                   -e 's/::/:0:/' \
                   -e 's/:$/:0/'  \
                   -e 's/:/./g'`

  echo $_network
}



#Assuming I'm getting all four octets of the IP number
build_network_dn () {

  _IP=${1:?"IP number is required"}
  _AT=${2:-"nn"}


  # Determine the Network Class
  _class=`echo $_IP | ${GLOBUS_SH_AWK-awk} -F. '{print $1}'`

  if [ -z "$_class" ] ; then
    _error "IP number is corrupted"
    exit 1
  fi

  # Determine the root IP number in the hierarchy
  # This is based on the "class" of the network
  if [ $_class -lt 128 ] ; then
    _LAST_IP=
  elif [ $_class -lt 192 ] ; then
    _LAST_IP=`echo $_IP | ${GLOBUS_SH_AWK-awk} -F. '{print $1}'`
  else
    _LAST_IP=`echo $_IP | ${GLOBUS_SH_AWK-awk} -F. '{print $1"."$2}'`
  fi

  # Modify the IP number to get rid of ".0$" segments.
  # Worst case is 128.0.0.0 - so do it 3 times
  _IP=`echo $_IP | ${GLOBUS_SH_SED-sed} -e 's/\.0$//' -e 's/\.0$//' -e 's/\.0$//' `


  _network=
  _rdn=
  _count=5 
  while [ "X$_IP" != "X$_LAST_IP" ] ; do

    _network="`_build_network_ip $_IP`"
    _rdn="$_rdn $_AT=$_network,"
    _IP=`echo $_IP | ${GLOBUS_SH_SED-sed} -e 's/[0-9]*$//' -e 's/\.$//'`
    _count=`expr $_count - 1`

    if [ $_count -le 0 ] ; then
       _error "runaway loop -- probable cause: output from netstat" 
       exit 1
    fi
  done


  #Potentially put in internet node
  echo $_rdn ${GRID_INFO_ORGANIZATION_DN}

}



build_networkimage_dn () {
  _IP=${1:?"IP number is required"}

  echo `build_network_dn $_IP "nn_image"`

}




build_host_dn () {
  GLOBUS_HOSTNAME=${1:-`${bindir}/globus-hostname`}

  echo "hn=$GLOBUS_HOSTNAME, ${GRID_INFO_ORGANIZATION_DN}"

}

build_hostimage_dn () {
  # A hostname image is a DIT child of a Network node
  # The name is based on its IP address and the name associated with
  # that IP address.  not that the value of
  # `${bindir}/globus-hostname` is not
  # necessarilly the name associated with the IP address.


  _IP=${1:?"IP address is required"}
  _CANONICAL_NAME="${2:-`${bindir}/globus-hostname`}"

  #Remove the last octet to build the network tree
  _NETWORK_IP=`echo $_IP | ${GLOBUS_SH_AWK-awk} -F. '{print $1"."$2"."$3}'`

  echo "hn_image=$_CANONICAL_NAME, `build_networkimage_dn $_NETWORK_IP`"
}


build_networkinterface_dn () {
  # An interface is a DIT child of a host
  # The name is based on the interface, and hostname

  _INTERFACE=${1:?"Interface name is required"}
  
  echo "nin=$_INTERFACE, `build_host_dn $GLOBUS_HOSTNAME`"
}



build_networkinterfaceimage_dn () {
  # An interface is a DIT child of a host-image
  # Network-image --> host-image --> interface-image
  # The name is based on the interface, hostname, and IP

  _IP=${1:?"IP number is required"}
  _CANONICAL_NAME=${2:-`${bindir}/globus-hostname`}
  
  echo "nin_image=$_IP, `build_hostimage_dn $_IP $_CANONICAL_NAME`"

}



