/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.replica.rls;

/**
 * Queries that may return large result lists may use excessive server
 * or client resources (eg memory).  RLS supports both server and client
 * side limits on the number of results returned by a query.  The following
 * class is used to retrieve results incrementally.  If clients do net
 * specify an offset and limit using this class then the API will retrieve
 * all results on behalf of the client.  Alternatively the client can
 * pass one of the following objects to the query function and loop through
 * the results a chunk at a time.  The following are equivlent:
 *
 * // Get all results
 *   list = rls.LRCGetPFNWC("*");
 *   // process results
 *
 * // Get results 5 at a time
 *   RLSOffsetLimit offlim = newRLSOffsetLimit(0,5);
 *   while (true) {
 *     list = rls.LRCGetLFNWC("*", offlim);
 *     // process results
 *     if (offlim.offset == -1)	// offset is set to -1 when no more results
 *       break;
 *   }
 */
public class RLSOffsetLimit {
  public int offset;
  public int reslimit;

  public RLSOffsetLimit() {
    offset = 0;
    reslimit = 0;
  }

  public RLSOffsetLimit(int ioffset, int ireslimit) {
    offset = ioffset;
    reslimit = ireslimit;
  }
}

