/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */

#include <globus_config.h>
#include <globus_io.h>
#include <stdarg.h>
#include <termio.h>
#include <string.h>

#include "globus_rls_client.h"
#include "misc.h"

extern char	*pidfile;
static int	detached = 0;

/*
 * writepid - Write pid to file.  Globus isn't activated yet so use standard
 *   C lib functions.
 */
void
writepid(char *prog)

{
  FILE	*f;

  if ((f = fopen(pidfile, "w")) == NULL)
    logit(LOG_WARNING, "writepid: Can't write pid file %s", pidfile);
  else {
    fchmod(fileno(f), 0644);
    fprintf(f, "%d\n", (int) getpid());
    fclose(f);
  }
}

/*
 * detach - Detach from controlling tty, open syslog.  Globus isn't activated
 *   yet so use C lib functions.
 */
void
detach(char *prog)

{
  int	i;

  if (fork())
    exit(0);
#ifndef TIOCNOTTY
  setsid();
#else
  if ((i = open("/dev/tty", O_RDONLY)) >= 0)
    ioctl(i, TIOCNOTTY, 0);
#endif
  for (i = 0; i < 10; i++)	/* Close existing file descriptors	*/
    close(i);
  open("/dev/null", 0);
  dup2(0, 1);
  dup2(0, 2);
  detached++;
  openlog(prog, LOG_PID, LOG_DAEMON);
}

/*
 * log - Write log message.
 */
void
logit(int level, char *fmt, ...)

{
  va_list	ap;
  char		buf[BUFLEN];
  char		tbuf[100];

  va_start(ap, fmt);
  vsnprintf(buf, BUFLEN, fmt, ap);
  if (detached)
    syslog(level, "T%lu: %s", (unsigned long int) pthread_self(), buf);
  else {
    mycftime(tbuf, 100, "%Y-%m-%d %H:%M:%S", time(0));
    globus_libc_printf("%s T%lu: %s\n", tbuf, (unsigned long int) pthread_self(), buf);
  }
  va_end(ap);
}

char *
globuserr(globus_result_t r, char *errmsg)

{
  globus_object_t	*eo;
  char			*s;

  if ((eo = globus_error_get(r)) == NULL)
    return "unknown globus error";
  s = globus_error_print_friendly(eo);
  globus_object_free(eo);
  if (!s)
    return "unknown globus error";
  strncpy(errmsg, s, MAXERRMSG);
  globus_libc_free(s);
  return errmsg;
}
