/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.cas.impl.client;

import org.globus.cas.CASPortType;

import org.globus.cas.impl.CasConstants;

import org.globus.wsrf.utils.FaultHelper;

import org.globus.cas.types.ManageServiceAction;

import org.globus.cas.faults.CasFault;
import org.globus.cas.faults.NoPermissionFault;

/**
 * Command line client to add or remove service type to Action mapping
 */
public class ActionMappingHandler {
    
    static String msg = 
        "Usage: cas-action [<options>] <operationType> <serviceTypeName> \n"
        + "     <actionName> \n"
        + " Options are  -debug Runs with debug trace \n"
        + "              -c <instanceURl> Specify URL for CAS server\n"
        + "              -help Prints this message\n"
        + "              -v Prints version number\n"
        + "              -s Identity of CAS server. If not specified, will \n"
        + "                 defualt to host authorization.\n"
        + "              -m security mechanism. 'msg' for secure message\n"
        + "                 or 'conv' for secure conversation or 'trans' for\n"
        + "                 transport security. If unspecified, if instance\n"
        + "                 url is 'https', then trasport security is used \n"
        + "                 else mechanism defaults to secure message\n"
        + "              -p protection type, 'sig' signature and 'enc' \n"
        + "                 encryption, defaults to signature\n"
        + " <operationType> : indicates the type of operation that needs to\n"
        + "                   be done. Should be \"add\" or \"remove\" \n"
        + " <serviceTypeName> : name of the service type on which the action\n"
        + "                     mapping needs to be or removed \n "
        + " <actionName> : should be the name of the action ";

    static boolean debug = false;

    public static void main(String args[]) throws Exception {
        
        if (args.length<1) {    
            System.out.println(msg);
            System.exit(0);
        }

        CasClientSetup clientSetup = new CasClientSetup(args, msg);
        String instanceURL =  clientSetup.getInstanceURL();     
        String serverIdentity = clientSetup.getServerIdentity();
        int startIndex = clientSetup.argsStartIndex;
        debug = clientSetup.debug;

        printMessage("CAS client to add or remove service type/action\n"
                           + " mappings ");

        if (args.length < startIndex + 2) {
            System.err.println("Not enough parameters");
            System.err.println(msg);
            System.exit(-1);
        }

        String operation = args[startIndex];

        if ((!(operation.equals(CasConstants.ADD_OPERATION))) &&
            (!(operation.equals(CasConstants.REMOVE_OPERATION)))) {
            System.err.println("Erroneous operation type " + operation);
            System.exit(-1);
        }           
        CASPortType casPort = 
            clientSetup.getCASPort(instanceURL, 3, serverIdentity);
        printMessage("Parameters " + args[startIndex] + " " 
                     + args[startIndex+1] + " " + args[startIndex+2]);
        try {
            ManageServiceAction mngService = 
                new ManageServiceAction();
            mngService.setRequestType(args[startIndex]);
            mngService.setServiceTypeName(args[startIndex+1]);
            mngService.setActionName(args[startIndex+2]);
            casPort.manageServiceAction(mngService);
        } catch (NoPermissionFault noPermFault) {
            System.err.println(noPermFault.getDescription());
            System.exit(-1);
        } catch (CasFault casFault) {
            if (!debug) {
                String desc = 
                    (new FaultHelper(casFault)).getDescriptionAsString();
                System.err.println(desc);
            } else {
                System.err.println((new FaultHelper(casFault))
                                   .getStackTrace());
            }
            System.exit(-1);
        } catch (Exception exp) {
            exp.printStackTrace();
            if (debug) {
                System.err.println(exp);
            } else {
                System.err.println(CasClientSetup.getErrorMsg(exp));
            }
            System.exit(-1);
        }
        System.out.println("Completed successfully");
    }

    private static void printMessage(String msg) {
        if (debug) {
            System.out.println(msg);
        }
    }
}
