/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.cas.impl.client;

import org.globus.wsrf.impl.security.authentication.Constants;

/**
 * This class is used to define parameters for generating CAS
 * assertions.
 */
public class ClientParams {

    private int assertionLifetime = 24 * 60 * 60;
    private String proxyFilename = null;
    private String casProxyFilename = null;
    private String casProxyTag = null;
    private ResourceActionsMap[] resActions = null;
    private String securityType = Constants.GSI_SEC_MSG;
    private Object protectionType = Constants.SIGNATURE;
    /**
     * By default, assertion lifetime is set to 24 hours,
     * default proxy file is used, the cas proxy is named with a .cas
     * tag at the end of proxy file.
     */
    public ClientParams() {
    }

    /**
     * @param assertionLifetime
     *        Requested lifetime of the generated CAS assertion
     */
    public void setAssertionLifetime(int assertionLifetime) {
        this.assertionLifetime = assertionLifetime;
    }
    
    /**
     * @param proxyFilename
     *        Name of the file to get credential from.
     */
    public void setProxyFileName(String proxyFilename) {
        this.proxyFilename = proxyFilename;
    }
    
    /**
     * @param casProxyFilename
     *        Name of the file store proxy with CAS assertions.
     */
    public void setCasProxyFileName(String casProxyFilename) {
        this.casProxyFilename = casProxyFilename;
    }

    /**
     * @param <code>ResourceActionsMap</code>
     *        Array of resource:actions mapping to generate assertions on
     */
    public void setResourceActionsMap(ResourceActionsMap[] resActions) {
        this.resActions = resActions;
    }

    /**
     * @param tag
     *        Tag to append to proxy file name to generate file name
     *        for proxy with CAS assertion. Used only if
     *        casProxyFileName is null.
     */
    public void setCasProxyTag(String tag) {
        this.casProxyTag = tag;
    }

    public void setSecurityType(String secType) {
        this.securityType = secType;
    }

    public void setProtectionType(Object protType) {
        if (protType != null) {
            this.protectionType = protType;
        }
    }

    public String getSecurityType() {
        return this.securityType;
    }

    public Object getProtectionType() {
        return this.protectionType;
    }

    public int getAssertionLifetime() {
        return this.assertionLifetime;
    }
    
    public String getProxyFileName() {
        return this.proxyFilename;
    }

    public String getCasProxyTag() {
        return this.casProxyTag;
    }
    
    public String getCasProxyFileName() {
        return this.casProxyFilename;
    }

    public ResourceActionsMap[] getResourceActionsMap() {
        return this.resActions;
    }
    
    public String toString() {
        String returnString;
        if (this.proxyFilename == null) 
            returnString = "Proxy file name not set \n";
        else 
            returnString = "Proxy file name is " + this.proxyFilename + "\n";

        if (this.casProxyFilename == null) 
            returnString = returnString + "CAS Proxy file name not set \n";
        else 
            returnString = returnString + "CAS Proxy file name is " 
                + this.casProxyFilename + "\n";
        
        returnString = returnString + "Requested assertion life time is " + 
            this.assertionLifetime + "\n";

        returnString = returnString + "Cas proxy tag is " + this.casProxyTag 
            + "\n";

        returnString = returnString + "Security type " + this.securityType
            + "Protection type " + this.protectionType;

        if (this.resActions != null) {
            returnString = returnString + "Resource/Actions: \n";
            for (int i=0; i<resActions.length; i++) {
                returnString = returnString + resActions[i].toString();
            }
        }
        return returnString;
    }
}

