/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.cas.impl.client;

import org.globus.cas.CASPortType;

import org.globus.cas.faults.CasFault;
import org.globus.cas.faults.NoPermissionFault;

import org.globus.cas.types.ArrayOfString;

import org.globus.cas.impl.CasConstants;

import org.globus.wsrf.utils.FaultHelper;

/**
 * Command line client to get a list of trustAnchor/namespace/
 * user/userGroup/object/objectGroup/serviceType/serviceTypeAction/
 * serviceActionGroup. 
 */
public class ListCasObjects {
    
    static String msg = 
        " Usage : cas-list-object [<options>] <objectType>\n"
        + " Options are  -debug Runs with debug trace \n"
        + "              -c <instanceURl> Specify URL for CAS server\n"
        + "              -help Prints this message\n"
        + "              -v Prints version number\n"
        + "              -s Identity of CAS server. If not specified, will \n"
        + "                 defualt to host authorization.\n"
        + "              -m security mechanism. 'msg' for secure message\n"
        + "                 or 'conv' for secure conversation or 'trans' for\n"
        + "                 transport security. If unspecified, if instance\n"
        + "                 url is 'https', then trasport security is used \n"
        + "                 else mechanism defaults to secure message\n"
        + "              -p protection type, 'sig' signature and 'enc' \n"
        + "                 encryption, defaults to signature\n"
        + " <objectType> : Type of object that needs to be listed. Should be\n"
        + "                \"user\" or \"userGroup\" or \"object\" or \n"
        + "                \"objectGroup\" or \"namespace\" or \n"
        + "                \"serviceType\" or \"serviceAction\" or \n"
        + "                \"serviceActionGroup\" or \"policy\"";

    static boolean debug = false;

    public static void main(String args[]) throws Exception {

        if (args.length<1) {    
            System.out.println(msg);
            System.exit(0);
        }
        CasClientSetup clientSetup = new CasClientSetup(args, msg);
        String instanceURL =  clientSetup.getInstanceURL();     
        String serverIdentity = clientSetup.getServerIdentity();
        int startIndex = clientSetup.argsStartIndex;
        debug = clientSetup.debug;

        if (debug)
            System.out.println("CAS client to return a list of CAS objects.");

        if (args.length < startIndex + 1) {
            System.err.println("Not enough parameters");
            System.err.println(msg);
            System.exit(-1);
        }

        String type = args[startIndex].trim();
        if ( !(type.equals(CasConstants.TRUSTANCHOR_SPEC) || 
               type.equals(CasConstants.USER_SPEC) || 
               type.equals(CasConstants.USERGP_SPEC) || 
               type.equals(CasConstants.OBJECT_SPEC) || 
               type.equals(CasConstants.OBJECTGP_SPEC) || 
               type.equals(CasConstants.NAMESPACE_SPEC) ||
               type.equals(CasConstants.POLICY_SPEC) ||
               type.equals(CasConstants.SERVICEACTION_SPEC) ||
               type.equals(CasConstants.SERVICEACTIONGP_SPEC) ||
               type.equals(CasConstants.SERVICETYPE_SPEC))) {

            System.err.println("Erroneous type " + type);
            System.err.println(msg);
            System.exit(-1);
        }

        
        CASPortType casPort = 
            clientSetup.getCASPort(instanceURL, 1, serverIdentity);

        String[] objects = null;
        try {
             ArrayOfString arrayOfString =  casPort.list(type);
             if (arrayOfString != null) {
                 objects = arrayOfString.getStrings();
             }
        } catch (NoPermissionFault noPermFault) {
            System.err.println(noPermFault.getDescription());
            System.exit(-1);
        } catch (CasFault casFault) {
            if (!debug) {
                String desc = 
                    (new FaultHelper(casFault)).getDescriptionAsString();
                System.err.println(desc);
            } else {
                System.err.println((new FaultHelper(casFault))
                                   .getStackTrace());
            }
            System.exit(-1);
        } catch (Exception exp) {
            if (debug) {
                System.err.println(exp.toString());
            } else {
                System.err.println(CasClientSetup.getErrorMsg(exp));
            }
            System.exit(-1);
        }

        if (objects == null) {
            System.out.println("No CasObject of type " +  type + " found");
        }
        else {
            System.out.println("List:");
            for (int i=0; i<objects.length; i++) {
                System.out.println(objects[i]);
            }
        }
        System.out.println("Completed successfully");
    }
}
