/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.cas.impl.client;

import java.util.Vector;

/**
 * Class to store resource: actions mapping
 */
public class ResourceActionsMap {

    private String resource;
    private Vector actions;

    /**
     * @param resource
     *        Name of resource (stored as &quot;Namespace|resourceName&quot;)
     * @param action
     *        Service action (stored as &quot;serviceType actionName&quot;)
     */
    public ResourceActionsMap(String resource, String action) 
        throws Exception {
        if ((resource == null) || 
            ((action == null) || (action.trim().equals("")))) {
            throw new Exception("Resource and action cannot be null");
        }
        this.resource = resource;
        this.actions = new Vector();
        this.actions.add(action);
    }

    /**
     * @param resource
     *        Name of resource (stored as &quot;Namespace|resourceName&quot;)
     * @param actions
     *        Vector of service actions (each action is stored as 
     *        &quot;serviceType actionName&quot;)
     */
    ResourceActionsMap(String resource, Vector actions) 
        throws Exception {
        if ((resource == null) || (actions == null)) {
            throw new Exception("Resource and action cannot be null");
        }
        this.resource = resource;
        this.actions = actions;
    }

    /**
     * @param action
     *        Service action (stored as &quot;serviceType actionName&quot;)
     */
    public void addAction(String action) {
        if ((action != null) && (!action.trim().equals("")))
            this.actions.add(action);
    }

    /*
     * @param actions
     *        Vector of service actions (each action is stored as 
     *        &quot;serviceType actionName&quot;)
     */
    public void addActions(Vector actions) {
        if (actions != null) 
            this.actions.addAll(actions);
    }

    public String getResource() {
        return resource;
    }

    public String[] getActions() {
        String[] actionStrs = new String[actions.size()];
        actions.toArray(actionStrs);
        return actionStrs;
    }

    public Vector getActionsAsVector() {
        return actions;
    }

    public String toString() {
        String returnString = "Resource: " + resource + "\n Actions: \n";
        for (int i=0; i<actions.size(); i++) {
            returnString = returnString + (String)actions.get(i);
        }
        return returnString;
    }
}
