/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.cas.impl.bootstrap;

import java.io.File;
import java.io.FileInputStream;

import java.util.Properties;

import org.globus.cas.impl.CasConstants;

import org.globus.cas.types.UserData;
import org.globus.cas.types.PolicyData;
import org.globus.cas.types.UserGroupData;
import org.globus.cas.types.TrustAnchorData;
import org.globus.cas.types.ArrayOfString;

import org.globus.cas.impl.databaseAccess.UserDataHandler;
import org.globus.cas.impl.databaseAccess.PolicyDataHandler;
import org.globus.cas.impl.databaseAccess.ObjectDataHandler;
import org.globus.cas.impl.databaseAccess.UserGroupDataHandler;
import org.globus.cas.impl.databaseAccess.TrustAnchorDataHandler;
import org.globus.cas.impl.databaseAccess.ServiceTypeActionHandler;

// Code now expects that a trust anchor, user group and user are added
// using this. Does not support scenario where only trust anchor or
// user needs to be added.
public class PopulateUserData  {

    public static void populateDb(String bootstrapFilename) throws Exception {

        File bootstrapFile = new File(bootstrapFilename);
        if (!bootstrapFile.exists()) {
            throw new Exception("bootstrap file \"" + bootstrapFilename
                                + "\" does not exist ");
        }
        Properties prop = new Properties();
        prop.load(new FileInputStream(bootstrapFilename));

        String taName = prop.getProperty("ta-name");
        String taAuthMethod = prop.getProperty("ta-authMethod");
        String taAuthData = prop.getProperty("ta-authData");
        // To store trust anchor data, none of this shld be null
        emptyStringPruning(taName, "Trust anchor name");
        emptyStringPruning(taAuthMethod, "Trust anchor authentication method");
        emptyStringPruning(taAuthData, "Trust anchor authentication data");

        String userName = prop.getProperty("user-name");
        String userSubject = prop.getProperty("user-subject");
        emptyStringPruning(userName, "User name");
        emptyStringPruning(userSubject, "User subject");

        String userGroup = prop.getProperty("userGroupname");
        emptyStringPruning(userGroup, "User group name");
        
        TrustAnchorData trustAnchor = new TrustAnchorData();
        trustAnchor.setNickname(taName.trim());
        trustAnchor.setAuthMethod(taAuthMethod.trim());
        trustAnchor.setAuthData(taAuthData.trim());
        TrustAnchorDataHandler.storeObject(trustAnchor);
        
        UserData user = new UserData();
        user.setNickname(userName.trim());
        user.setSubjectName(userSubject.trim());
        user.setTrustAnchorName(taName.trim());
        UserDataHandler.storeObject(user);

        // Add user group
        UserGroupData userGpData = new UserGroupData();
        userGpData.setGroupName(userGroup.trim());
        UserGroupDataHandler.storeObject(userGpData);
        
        // Add user to group
        UserGroupDataHandler.addGroupMember(userGroup.trim(), 
                                            userName.trim());
        
        // Get CAS object id
        int objectId = 
            ObjectDataHandler.getObjectId(CasConstants.OBJECT_SELF,
                                          CasConstants.NAMESPACE_SELF);
        if (objectId == -1) {
            // policy cannot be stored, so return
            throw new Exception("Implicit object initialization has not been "
                                + " done can't find cas server object");
        }
        
        // Set super user permissions on this user
        int serviceActionId = ServiceTypeActionHandler
            .getServiceActionId(CasConstants.SERVICETYPE_CAS,
                                CasConstants.ACTION_SU);
        
        if (serviceActionId == -1) {
            // policy cannot be stored, so return
            throw new Exception("Implicit object initialization has not been "
                                + " done can't find cas server object");
        }
        
        PolicyData policyData = new PolicyData();
        policyData.setUserGroupName(userGroup.trim());
        policyData.setObjectSpec(Integer.toString(objectId));
        policyData.setObjectSpecDesc("object");
        policyData.setActionSpec(Integer.toString(serviceActionId));
        policyData.setActionSpecDesc("serviceAction");
        PolicyDataHandler.storeObject(policyData);
    }

    private static boolean stringInArray(String[] userNames, String userName) {
        for (int i=0; i<userNames.length; i++) {
            if (userNames[i].equals(userName))
                return true;
        }
        return false;
    }

    private static void emptyStringPruning(String str, String strName)
        throws Exception {
        
        if ((str == null) || (str.equals(""))) {
            throw new Exception(strName + " cannot be null or empty");
        }
    }
}
