/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.cas.impl.databaseAccess;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Connection;
import java.sql.Statement;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.globus.cas.impl.CasConstants;

import org.globus.cas.types.CasObjectData;
import org.globus.cas.types.NamespaceData;

import org.globus.util.I18n;

/**
 * Used to manipulate NamespaceData in database
 */
public class NamespaceDataHandler {
    
    static Log logger = LogFactory.getLog(NamespaceData.class.getName() );

    private static I18n i18n =
        I18n.getI18n("org.globus.cas.impl.databaseAccess.errors",
                     NamespaceDataHandler.class.getClassLoader());

    /**
     * Store the name space object
     * Manipulates only the namespace_table in the daabase
     */
    public static String storeObject(NamespaceData dataObject) 
        throws CasDBException {

        String columnString = "insert into" + CasDBConstants.TABLE_NAMESPACE 
            + " (" + CasDBConstants.COL_NS_NICKNAME + "," 
            + CasDBConstants.COL_BASENAME + "," 
            + CasDBConstants.COL_COMPARISON_ALG + ") values ('";
        StringBuffer query = new StringBuffer(columnString);
        query = query.append(dataObject.getNickname()).append("','")
            .append(dataObject.getBasename()).append("','")
            .append(dataObject.getComparisonAlg()).append("')");
        CasDBStorage.runUpdateQuery(query.toString());
        return dataObject.getNickname();
    }

    /**
     * List of all Namespace
     */
    public static String[] list() throws CasDBException {
        
        String query = "select" + CasDBConstants.COL_NS_NICKNAME + " from" 
            + CasDBConstants.TABLE_NAMESPACE;
        return CasDBStorage.runListQuery(query, 
                                         CasDBConstants.COL_NS_NICKNAME);
    }

    /*
     * Retrieves Namespace data
     * Picks up values frm trust anchor table
     */
    public static CasObjectData retrieveObject(String nickname) 
        throws CasDBException {
        
        NamespaceData returnObject = null;
        String query = "select * from" + CasDBConstants.TABLE_NAMESPACE 
            + " where" + CasDBConstants.COL_NS_NICKNAME + " = '" 
            + nickname.trim() + "'";
        Connection connection = CasDBStorage.getDBConnection();
        Statement statement = null;
        ResultSet resultSet = null;
        try {
            statement = connection.createStatement();
            resultSet = statement.executeQuery(query);
            if ((resultSet!=null) && (resultSet.next())) {
                returnObject = new NamespaceData();
                returnObject.setNickname(resultSet
                                         .getString(CasDBConstants
                                                    .COL_NS_NICKNAME.trim()));
                returnObject.setBasename(resultSet.getString(
                                         CasDBConstants.COL_BASENAME.trim()));
                returnObject.setComparisonAlg(resultSet.getString(
                             CasDBConstants.COL_COMPARISON_ALG.trim()));
            }
        }
        catch (SQLException exp) {
            logger.error(i18n.getMessage("retrieveErr", new Object[] {
                "namespace", query }), exp);
            throw new CasDBException(i18n.getMessage("retrieveErr", 
                                                     new Object[] {
                                                         "namespace", "" })
                                     + exp.getMessage(), exp);
        }
        finally {
            // returning connection irrespective of whether stmt
            // and result set are closed or not.
            CasDBStorage.returnDBConnection(connection);
            try {
                if (resultSet != null)
                    resultSet.close();
                if (statement != null)
                    statement.close();
            }
            catch (SQLException exp) {
                String err = "Error retrieving namespace object. Could not "
                    + " close SQL statement.";
                logger.warn(err, exp);
            }
        }
        return returnObject;
    }
    
    /**
     * Remove namespace entry from database
     * Manipulates only namespace table
     */
    public static void deleteObject(String nickname) throws CasDBException {
        
        // Namespace can be a member of object group, no SQL constraint
        // defines that. So check.
        if (ObjectGroupDataHandler.isMember(nickname.trim(), 
                                            CasConstants.NAMESPACE_SPEC)) {
            String errMesg = i18n.getMessage("memberOfGpErr", new Object [] { 
                    "namespace", nickname.trim(), "object"});
            logger.debug(errMesg);
            throw new CasDBException(errMesg);
        }

        String query = "delete from" + CasDBConstants.TABLE_NAMESPACE 
            + " where" + CasDBConstants.COL_NS_NICKNAME + " = '" 
            + nickname.trim() + "'";
        logger.debug("query is " + query);
        int updated = CasDBStorage.runUpdateQuery(query);
        if (updated <= 0) {
            String err = i18n.getMessage("nsDelError", nickname);
            logger.error(err);
            throw new CasDBException(err);
        }
    }
}
