/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.cas.impl.databaseAccess;

import java.sql.Connection;
import java.sql.Statement;
import java.sql.SQLException;
import java.sql.ResultSet;

import java.util.Vector;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.globus.cas.impl.CasConstants;

import org.globus.cas.impl.service.ObjectComparison;

import org.globus.cas.types.ObjectData;
import org.globus.cas.types.CasObjectData;
import org.globus.cas.types.ArrayOfString;

import org.globus.util.I18n;

/**
 * Describes the data needed for object
 */
public class ObjectDataHandler {

    static Log logger = LogFactory.getLog(ObjectDataHandler.class.getName() );

    private static I18n i18n =
        I18n.getI18n("org.globus.cas.impl.databaseAccess.errors",
                     ObjectDataHandler.class.getClassLoader());
    
    /**
     * Returns object id given onject name and namespace
     * Returns -1 if object is not in database
     */
    public static int getObjectId(String objectName, String namespaceNick) 
        throws CasDBException {
        Connection connection = CasDBStorage.getDBConnection();
        int result = getObjectId(connection, objectName, namespaceNick);
        CasDBStorage.returnDBConnection(connection);
        return result;
    }

    public static int getObjectId(Connection conn, String objectName, 
                                   String namespaceNick) 
        throws CasDBException {
        int result = -1;
        String query = "select " + CasDBConstants.COL_OBJECT_ID + " from" 
            + CasDBConstants.TABLE_OBJECT + " where" 
            + CasDBConstants.COL_OBJECT_NAME + "='" + objectName.trim() 
            + "' and " + CasDBConstants.COL_NS_NICKNAME + "='" 
            + namespaceNick.trim() + "'";
        Statement statement = null;
        ResultSet resultSet = null;
        try {
            statement = conn.createStatement();
            logger.debug("object exists query is " + query);
            resultSet = statement.executeQuery(query);
            if ((resultSet != null) && (resultSet.next())) {
                result = resultSet.getInt(CasDBConstants.COL_OBJECT_ID.trim());
            }
        }
        catch (SQLException exp) {
            CasDBStorage.returnDBConnection(conn);
            logger.error(i18n.getMessage("existenceErr", new Object[] {
                "object", query} ), exp);
            throw new CasDBException(i18n.getMessage("existenceErr", 
                                                     new Object[] { "object",
                                                                    "" }) 
                                     + exp.getMessage(), 
                                     exp);
        }
        finally {
            try {
                if (resultSet != null)
                    resultSet.close();
                if (statement != null)
                    statement.close();
            }
            catch (SQLException exp) {
                logger.warn(i18n.getMessage("dataExistenceCloseErr"), exp);
            }
        }
        return result;
    }

    // Returns the object comparison class
    public static ObjectComparison getComparisonClass(String compAlgClassName) 
        throws CasDBException {

        ObjectComparison objectComp = null;
        try {
            Class comparisonClass = Class.forName(compAlgClassName);
            objectComp = (ObjectComparison)comparisonClass.newInstance();
        }
        catch (ClassNotFoundException classNFE) {
            logger.error(i18n.getMessage("compClassNotFound", 
                                         compAlgClassName), classNFE);
            throw new CasDBException(i18n.getMessage("compClassNotFound", 
                                                     compAlgClassName)
                                     + classNFE.getMessage(), classNFE);
        }
        catch (InstantiationException instantiateExp) {
            logger.error(i18n.getMessage("compClassInst", compAlgClassName),
                         instantiateExp);
            throw new CasDBException(i18n.getMessage("compClassInst", 
                                                     compAlgClassName)
                                     + instantiateExp.getMessage(),
                                     instantiateExp);
        }
        catch (IllegalAccessException illegalExp) {
            logger.error(i18n.getMessage("compClassInst", compAlgClassName),
                         illegalExp);
            throw new CasDBException(i18n.getMessage("compClassInst", 
                                                     compAlgClassName)
                                     + illegalExp.getMessage(),
                                     illegalExp);
        }
        return objectComp;
    }

    // Returns the comparison algorithm for the specified objectId
    public static String getComparisonAlg(String namespace) 
        throws CasDBException {

        Connection connection = CasDBStorage.getDBConnection();
        try {
            return getComparisonAlg(connection, namespace);
        } finally {
            // returning connection irrespective of whether stmt
            // and result set are closed or not.
            CasDBStorage.returnDBConnection(connection);
        }
    }

    private static String getComparisonAlg(Connection connection,
                                           String namespace) 
        throws CasDBException {

        String query = "select" + CasDBConstants.COL_COMPARISON_ALG + " from" 
            + CasDBConstants.TABLE_NAMESPACE + " where" 
            + CasDBConstants.COL_NS_NICKNAME + "='" + namespace + "'";

        String compAlg = null;
        Statement statement = null;
        ResultSet resultSet = null;
        try {
            statement  = connection.createStatement();
            resultSet  = statement.executeQuery(query);
            if ((resultSet!=null) && (resultSet.next())) {
                compAlg = resultSet.getString(
                                    CasDBConstants.COL_COMPARISON_ALG.trim());
            }
        }
        catch (SQLException exp) {
            logger.error(i18n.getMessage("retrCompAlgErr", namespace) 
                         + query, exp);
            throw new CasDBException(i18n.getMessage("retrCompAlgErr", 
                                                     namespace)
                                     + exp.getMessage(), exp);
        }
        finally {

            try {
                if (resultSet != null)
                    resultSet.close();
                if (statement != null)
                    statement.close();
            }
            catch (SQLException exp) {
                String err = i18n.getMessage("compAlgNSClose");
                logger.warn(err, exp);
            }
        }

        if (compAlg == null) {
            String err = i18n.getMessage("doesNotExist", new Object[] {
                "Namespace", namespace });
            logger.error(err);
            throw new CasDBException(err);
        }
        if (compAlg.equals(CasConstants.EXACT_ALGNAME)) {
            logger.debug("Exact comparison");
            compAlg = CasConstants.EXACT_ALG_CLASSNAME;
        }
        if (compAlg.equals(CasConstants.WILD_ALGNAME)) {
            logger.debug("WildCard comparison");
            compAlg = CasConstants.WILD_ALG_CLASSNAME;
        }
        logger.debug("comparison Algorithm " + compAlg);
        return compAlg;
    } 

    private static void storeInObjectTable(Connection conn, String objectName, 
                                           String namespaceNick) 
        throws CasDBException {

        // Insert object data
        String columnString = "insert into " + CasDBConstants.TABLE_OBJECT 
            + " (" + CasDBConstants.COL_OBJECT_NAME + "," 
            + CasDBConstants.COL_NS_NICKNAME + ") values ('";
        StringBuffer query = new StringBuffer(columnString);
        query.append(objectName).append("','").append(namespaceNick)
            .append("')");
        Statement statement = null;
        try {
            statement = conn.createStatement();
            logger.debug("store query is " + query.toString());
            statement.executeUpdate(query.toString());
        }
        catch (SQLException exp) {
            CasDBStorage.returnDBConnection(conn);
            logger.error(i18n.getMessage("storeErr", new Object[] { "object", 
                                         query.toString() }), exp);
            throw new CasDBException(i18n.getMessage("storeErr", new Object[] {
                "object", "" } ) + exp.getMessage(), exp);
        }
        finally {
            try {
                if (statement != null)
                    statement.close();
            }
            catch (SQLException exp) {
                logger.warn(i18n.getMessage("storeErrClose", "object"), exp);
            }
        }
    }

    /**
     * Insert into user data into object table 
     * Assuming checks for object already existing we made
     */
    public static String storeObject(ObjectData objectData) 
        throws CasDBException {

        String objectName = objectData.getObjectName();
        String namespaceNick = objectData.getObjectNamespace();
        Connection connection = CasDBStorage.getDBConnection();
        storeInObjectTable(connection, objectName, namespaceNick);
        int result = getObjectId(connection, objectName, namespaceNick);
        CasDBStorage.returnDBConnection(connection);
        return Integer.toString(result);
    }

    /**
     * List of ObjectData
     */
    public static String[] list() throws CasDBException {
        String query = "select" + CasDBConstants.COL_OBJECT_NAME + " from" 
            + CasDBConstants.TABLE_OBJECT;
        return CasDBStorage.runListQuery(query, CasDBConstants.COL_OBJECT_NAME);
    }

    /**
     * Retrieves object data given onjectName and namespace
     * Picks up values from obejct table and object group entry table
     */
    public static CasObjectData retrieveObject(String objectName, 
                                               String namespaceNick) 
        throws CasDBException {

        ObjectData returnObject = null;
        Connection connection = CasDBStorage.getDBConnection();
        int result = getObjectId(connection, objectName, namespaceNick);
        if (result <= 0) {
            String err = i18n.getMessage("doesNotExist", new Object[] {
                "Object data ",  namespaceNick + "/" + objectName });
            logger.debug(err);
            throw new CasDBException(err);
        }
        returnObject =  retrieveObjectForId(Integer.toString(result));
        CasDBStorage.returnDBConnection(connection);
        return returnObject;
    }

    /**  
     * Retrieves object data given objectId
     * Picks up values from obejct table and object group entry table
     */
    public static ObjectData retrieveObjectForId(String objectId) 
        throws CasDBException {

        ObjectData objData = null;
        Vector temp = null;
        String query = "select" + CasDBConstants.COL_OBJECTGP_NAME + " from" 
            + CasDBConstants.TABLE_OBJECTGP_ENTRY + " where " 
            + CasDBConstants.COL_OBJECT_SPEC + " ='" + objectId + "'";
        Connection connection = CasDBStorage.getDBConnection();
        Statement statement = null;
        ResultSet resultSet = null;
        try {
            statement = connection.createStatement();
            resultSet = statement.executeQuery(query);
            if ((resultSet!=null) && (resultSet.next())) {
                temp = new Vector(resultSet.getFetchSize());
                do {
                    temp.add(
                             resultSet.getString(
                             CasDBConstants.COL_OBJECTGP_NAME.trim()));
                } while (resultSet.next());
            }
        }
        catch (SQLException exp) {
            logger.error(i18n.getMessage("stmtExecErr", query), exp);
            throw new CasDBException(i18n.getMessage("stmtExecErr", query)
                                     + exp.getMessage(), exp);
        }
        finally {
            // returning connection irrespective of whether stmt
            // and result set are closed or not.
            CasDBStorage.returnDBConnection(connection);
            try {
                if (resultSet != null)
                    resultSet.close();
                if (statement != null)
                    statement.close();
            }
            catch (SQLException exp) {
                String err = "Error retrieving object. Could not "
                    + " close SQL statement.";
                logger.warn(err, exp);
            }
        }

        ArrayOfString arrayOfGpNames = null;
        if (temp != null) {
            String groupNamesString[] = new String[temp.size()];
            temp.toArray(groupNamesString);
            arrayOfGpNames = new ArrayOfString(groupNamesString);
        }

        connection = CasDBStorage.getDBConnection();
        query = "select * from" + CasDBConstants.TABLE_OBJECT + " where" 
            + CasDBConstants.COL_OBJECT_ID + " = " + objectId.trim();
        try {
            statement = connection.createStatement();
            resultSet = statement.executeQuery(query);
            if (resultSet !=null) {
                if (resultSet.next()) {
                    objData = new ObjectData();
                    objData.setObjectId(objectId);
                    objData.setObjectName(
                            resultSet.getString(
                                      CasDBConstants.COL_OBJECT_NAME.trim()));
                    objData.setObjectNamespace(
                            resultSet.getString(
                                      CasDBConstants.COL_NS_NICKNAME.trim()));
                    objData.setGroupNames(arrayOfGpNames);
                }
            }
        }
        catch (SQLException exp) {
            logger.error(i18n.getMessage("retrieveErr", new Object[] { 
                "object", query }), exp);
            throw new CasDBException(i18n
                                     .getMessage("retrieveErr", 
                                                 new Object[] {"object", ""}) 
                                     + exp.getMessage(), exp);
        }
        finally {
            // returning connection irrespective of whether stmt
            // and result set are closed or not.
            CasDBStorage.returnDBConnection(connection);
            try {
                if (resultSet != null)
                    resultSet.close();
                if (statement != null)
                    statement.close();
            }
            catch (SQLException exp) {
                logger.warn(i18n.getMessage("retrieveErrClose", "object"), 
                            exp);
            }
        }
        return objData;
    }

    /**
     * Deletes object data gievn ojectName and namespace
     * Manipulates object table 
     */
    public static int deleteObject(String objectName, String namespaceNick) 
        throws CasDBException {
        int newObjectId = -1;
        Connection connection = CasDBStorage.getDBConnection();
        newObjectId = getObjectId(connection, objectName, namespaceNick);
        deleteObjectForId(connection, newObjectId);
        CasDBStorage.returnDBConnection(connection);    
        return newObjectId;
    }

    /**
     * Deletes object data given objectId
     * Manipulates object table 
     */
    public static void deleteObjectForId(String objectId) 
        throws CasDBException {

        Connection connection = CasDBStorage.getDBConnection();
        deleteObjectForId(connection, Integer.parseInt(objectId));
        CasDBStorage.returnDBConnection(connection);    
    }

    private static void deleteObjectForId(Connection connection, int objectId) 
        throws CasDBException {

        // Object can be a member of object group, no SQL constraint
        // defines that. So check.
        if (ObjectGroupDataHandler.isMember(connection, 
                                            Integer.toString(objectId),
                                            CasConstants.OBJECT_SPEC)) {
            String errMesg = 
                i18n.getMessage("memberOfGpErr", 
                                new Object[] { "object", 
                                               new Integer(objectId),
                                               "object" });
            logger.debug(errMesg);
            throw new CasDBException(errMesg);
        }

        String queryString = "delete from" + CasDBConstants.TABLE_OBJECT 
            + " where" + CasDBConstants.COL_OBJECT_ID + " =" + objectId;
        Statement statement = null;
        try {
            statement = connection.createStatement();
            statement.executeUpdate(queryString);
        }
        catch (SQLException exp) {
            CasDBStorage.returnDBConnection(connection);
            logger.error(i18n.getMessage("delObjErr", 
                                         new Object[] { "object",  
                                                        queryString }),
                         exp);
            throw new CasDBException(i18n.getMessage("delObjErr", 
                                                     new Object[] { "object",
                                                                    "" })
                                     + exp.getMessage(), exp);
        }
        finally {
            try {
                if (statement != null)
                    statement.close();
            }
            catch (SQLException exp) {
                logger.warn(i18n.getMessage("delObjErrClose", "object"), exp);
            }
        }
    }

    // Returns id of all objects in the db that match this object. Relevant
    // comparison algorithm is used.
    public static Vector retrieveMatchingObjects(String namespace, 
                                                 String objectName) 
        throws CasDBException {

        logger.debug("Namespace " + namespace + " objName " + objectName);
        Vector nsMatch = null;
        // Retrieve all objects that belong to the same namespace, then use
        // macthes method to match.
        String query = "select * from" + CasDBConstants.TABLE_OBJECT 
            + " where " + CasDBConstants.COL_NS_NICKNAME + "='" + namespace 
            + "'";
        logger.debug("Retrieve Matching objects query: "+ query);
        Connection connection = CasDBStorage.getDBConnection();
        Statement statement = null;
        ResultSet resultSet = null;
        try {
            statement = connection.createStatement();
            resultSet = statement.executeQuery(query);
            if ((resultSet != null) && (resultSet.next())) {
                nsMatch = new Vector(resultSet.getFetchSize());
                do {
                    ObjectData objData = new ObjectData();
                    objData.setObjectId(resultSet.getString(
                                  CasDBConstants.COL_OBJECT_ID.trim()));
                    objData.setObjectName(resultSet.getString(
                                  CasDBConstants.COL_OBJECT_NAME.trim()));
                    objData.setObjectNamespace(resultSet.getString(
                                  CasDBConstants.COL_NS_NICKNAME.trim()));
                    nsMatch.add(objData);
                } while (resultSet.next());
            }
            
            if (nsMatch == null) {
                logger.debug("No objects in the same namesapce, "
                             + "so no matches");
                return null;
            } else {
                Vector matchedObjects = null;
                ObjectData objToBeCompared = new ObjectData();
                objToBeCompared.setObjectNamespace(namespace);
                objToBeCompared.setObjectName(objectName);
                String compAlgClassName = 
                    ObjectDataHandler.getComparisonAlg(connection, namespace);
                ObjectComparison objectComp =
                    ObjectDataHandler.getComparisonClass(compAlgClassName);
                for (int i=0; i<nsMatch.size(); i++) {
                    ObjectData objInDb = (ObjectData)nsMatch.get(i);
                    if (objectComp.objectMatches(objInDb, objToBeCompared)) {
                        logger.debug("Object matches " 
                                     + objInDb.getObjectId());
                        if (matchedObjects == null) {
                            matchedObjects = new Vector();
                        }
                        matchedObjects.add(objInDb.getObjectId());
                    } else
                        logger.debug("Object does not match " 
                                     + objInDb.getObjectId());
                }
                return matchedObjects;
            }
        }
        catch (SQLException exp) {
            logger.error(i18n.getMessage("dataExistenceErr", query), exp);
            throw new CasDBException(i18n.getMessage("dataExistenceErr", query)
                                     + exp.getMessage(), exp);
        }
        finally {
            // returning connection irrespective of whether stmt
            // and result set are closed or not.
            CasDBStorage.returnDBConnection(connection);
            try {
                if (resultSet != null)
                    resultSet.close();
                if (statement != null)
                    statement.close();
            }
            catch (SQLException exp) {
                String err = i18n.getMessage("dataExistenceCloseErr");
                logger.warn(err, exp);
            }
        }
    }

    // Assumed parameter is an object name
    public static String getObjectDescription(String objectName) 
        throws CasDBException {
        ObjectData tempObject = retrieveObjectForId(objectName);
        if (tempObject == null) {
            String err = i18n.getMessage("objIdRetrErr", objectName);
            logger.error(err);
            throw new CasDBException(err);
        }
        return tempObject.getObjectNamespace() + 
            CasConstants.OBJECTSPEC_DELIMITER 
            + tempObject.getObjectName();
    }
}
