/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.cas.impl.databaseAccess;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Statement;
import java.sql.SQLException;

import java.util.Vector;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.globus.cas.impl.CasConstants;

import org.globus.cas.types.CasObjectData;
import org.globus.cas.types.ArrayOfString;
import org.globus.cas.types.ServiceTypeData;

import org.globus.util.I18n;

/**
 * Describes the data needed for service type
 */
public class ServiceTypeDataHandler {
    
    static Log logger = 
        LogFactory.getLog(ServiceTypeDataHandler.class.getName() );

    private static I18n i18n =
        I18n.getI18n("org.globus.cas.impl.databaseAccess.errors",
                     ServiceTypeDataHandler.class.getClassLoader());

    /** 
     * Insert into service type table 
     */
     public static String storeObject(ServiceTypeData serviceTypeData) 
         throws CasDBException {

        logger.debug("Service type data storeObject called");
        Connection connection = CasDBStorage.getDBConnection();
        String serviceTypeName = serviceTypeData.getName();
        // Store service type data
        String query = "insert into " + CasDBConstants.TABLE_SERVICETYPE + " ("
            + CasDBConstants.COL_SERVICETYPE_NAME + ") values ('" 
            + serviceTypeName + "')"; 
        Statement statement = null;
        try {
            statement = connection.createStatement();
            logger.debug("store query is " + query);
            statement.executeUpdate(query);
        }
        catch (SQLException exp) {
            logger.error(i18n.getMessage("storeErr", new Object[] {
                "service type", query }), exp);
            throw new CasDBException(i18n.getMessage("storeErr", new Object[] {
                "service type", exp.getMessage() }), exp);
        }
        finally {
            // returning connection irrespective of whether stmt
            // and result set are closed or not.
            CasDBStorage.returnDBConnection(connection);
            try {
                if (statement != null)
                    statement.close();
            }
            catch (SQLException exp) {
                logger.warn(i18n.getMessage("storeErrClose", "service type "),
                            exp);
            }
        }
        return serviceTypeName;
    }

    /** 
     * List of service type
     */
    public static String[] list() throws CasDBException {

        String query = "select" + CasDBConstants.COL_SERVICETYPE_NAME + " from" 
            + CasDBConstants.TABLE_SERVICETYPE;
        return CasDBStorage.runListQuery(query, 
                                         CasDBConstants.COL_SERVICETYPE_NAME);
    }

    /**
     * Retieves service type data
     * Picks up values from service type table and service type action table
     */
    public static CasObjectData retrieveObject(String name) 
        throws CasDBException {
        
        ServiceTypeData returnObject = null;
        Vector actionNames = null;
        Connection connection = CasDBStorage.getDBConnection();

        String query = "select" + CasDBConstants.COL_ACTION_NAME + " from" 
            + CasDBConstants.TABLE_SERVICETYPE_ACTION + " where" 
            + CasDBConstants.COL_SERVICETYPE_NAME + "='" + name.trim() + "'";

        Statement statement = null;
        ResultSet resultSet = null;
        try {
            statement = connection.createStatement();
            resultSet = statement.executeQuery(query);
            if ((resultSet!=null) && (resultSet.next())) {
                actionNames = new Vector(resultSet.getFetchSize());
                do {
                    actionNames.add(resultSet.getString(
                                    CasDBConstants.COL_ACTION_NAME.trim()));
                } while (resultSet.next());
            }
        }
        catch (SQLException exp) {
            CasDBStorage.returnDBConnection(connection);
            logger.error(i18n.getMessage("retrieveErr", new Object[] {
                "service type", query }), exp);
            throw new CasDBException(i18n.getMessage("retrieveErr", 
                      new Object[] { "service type", exp.getMessage()}), exp);
        }
        finally {
            try {
                if (resultSet != null)
                    resultSet.close();
                if (statement != null)
                    statement.close();
            }
            catch (SQLException exp) {
                logger.warn(i18n.getMessage("retrieveErrClose", 
                                            "service type"), exp);
            }
        }
        
        ArrayOfString arrayOfActionNames = null;
        if (actionNames != null) {
            String stringActionNames[] = new String[actionNames.size()];
            actionNames.toArray(stringActionNames);
            arrayOfActionNames = new ArrayOfString(stringActionNames);
        }

        query = "select * from" + CasDBConstants.TABLE_SERVICETYPE + " where" 
            + CasDBConstants.COL_SERVICETYPE_NAME + " = '" + name.trim() + "'";
        
        try {
            statement = connection.createStatement();
            resultSet = statement.executeQuery(query);
            if (resultSet!=null) {
                if (resultSet.next()) {
                    returnObject = new ServiceTypeData();
                    returnObject.setName(resultSet.getString(
                                 CasDBConstants.COL_SERVICETYPE_NAME.trim()));
                    returnObject.setActionNames(arrayOfActionNames);
                }
            }
        }
        catch (SQLException exp) {
            logger.error("Error retrieving service type data\n" + query, exp);
            logger.error(i18n.getMessage("retrieveErr", new Object[] {
                "service type", query }), exp);
            throw new CasDBException(i18n.getMessage("retrieveErr", 
                      new Object[] { "service type", exp.getMessage()}), exp);
        }
        finally {
            // returning connection irrespective of whether stmt
            // and result set are closed or not.
            CasDBStorage.returnDBConnection(connection);
            try {
                if (resultSet != null)
                    resultSet.close();
                if (statement != null)
                    statement.close();
            }
            catch (SQLException exp) {
                logger.warn(i18n.getMessage("retrieveErrClose", 
                                            "service type"), exp);
            }
        }
        return returnObject;
    }

    /**
     * Deletes service type data
     * Manipulates service type table 
     */
    public static void deleteObject(String serviceName) throws CasDBException {
        
        // Service type can be a member of object group, no SQL constraint
        // defines that. So check.
        if (ObjectGroupDataHandler.isMember(serviceName.trim(), 
                                            CasConstants.SERVICETYPE_SPEC)) {
            String errMesg = i18n.getMessage("memberOfGpErr", new Object[] {
                "Service type", serviceName.trim(), "object"});
            logger.debug(errMesg);
            throw new CasDBException(errMesg);
        }

        Connection connection = CasDBStorage.getDBConnection();
        String query = "delete from" + CasDBConstants.TABLE_SERVICETYPE 
            + " where" + CasDBConstants.COL_SERVICETYPE_NAME + " ='" 
            + serviceName.trim() + "'";
        Statement statement = null;
        int result = -1;
        try {
            statement = connection.createStatement();
            result = statement.executeUpdate(query);
        }
        catch (SQLException exp) {
            logger.error(i18n.getMessage("delObjErr", new Object[] { 
                "servicetype", query }), exp);
            throw new CasDBException(i18n.getMessage("delObjErr", 
                  new Object[] { "servicetype", exp.getMessage()}), exp);
        }
        finally {
            // returning connection irrespective of whether stmt
            // and result set are closed or not.
            CasDBStorage.returnDBConnection(connection);
            try {
                if (statement != null)
                    statement.close();
            }
            catch (SQLException exp) {
                logger.warn(i18n.getMessage("delObjErrClose", "servicetype"),
                            exp);
            }
        }
        if (result <= 0) {
            String err = i18n.getMessage("serviceTypeDelError", serviceName);
            logger.error(err);
            throw new CasDBException(err);
        }
    }
}
