/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.cas.impl.databaseAccess;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Connection;
import java.sql.Statement;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.globus.cas.impl.CasConstants;

import org.globus.cas.types.CasObjectData;
import org.globus.cas.types.TrustAnchorData;

import org.globus.util.I18n;

/**
 * TrustAnchor Data manipulation in the database
 */
public class TrustAnchorDataHandler {
    
    static Log logger = LogFactory.getLog(TrustAnchorData.class.getName() );

    private static I18n i18n =
        I18n.getI18n("org.globus.cas.impl.databaseAccess.errors",
                     TrustAnchorDataHandler.class.getClassLoader());

    /**
     * Store the trust anchor object
     * Manipulates only the trust_anchor_table in the daabase
     */
    public static String storeObject(TrustAnchorData dataObject) 
        throws CasDBException {

        String columnString = "insert into" + CasDBConstants.TABLE_TRUSTANCHOR 
            + " (" + CasDBConstants.COL_TRUST_NICKNAME + "," 
            + CasDBConstants.COL_AUTHMETHOD + "," 
            + CasDBConstants.COL_AUTHDATA + ") values ('";
        StringBuffer query = new StringBuffer(columnString);
        query = query.append(dataObject.getNickname()).append("','")
            .append(dataObject.getAuthMethod()).append("','")
            .append(dataObject.getAuthData()).append("')");
        CasDBStorage.runUpdateQuery(query.toString());
        return dataObject.getNickname();
    }

    /**
     * List of all Trust Anchors
     */
    public static String[] list() throws CasDBException {
        
        String query = "select" + CasDBConstants.COL_TRUST_NICKNAME + " from" 
            + CasDBConstants.TABLE_TRUSTANCHOR;
        return CasDBStorage.runListQuery(query, 
                                         CasDBConstants.COL_TRUST_NICKNAME);
    }

    /*
     * Retrieves TrustAnchor data
     * Picks up values frm trust anchor table
     */
    public static CasObjectData retrieveObject(String nickname) 
        throws CasDBException {
        
        TrustAnchorData returnObject = null;
        String query = "select * from" + CasDBConstants.TABLE_TRUSTANCHOR 
            + " where" + CasDBConstants.COL_TRUST_NICKNAME + " = '" 
            + nickname.trim() + "'";
        Connection connection = CasDBStorage.getDBConnection();
        Statement statement = null;
        ResultSet resultSet = null;
        try {
            statement = connection.createStatement();
            resultSet = statement.executeQuery(query);
            if (resultSet!=null) {
                if (resultSet.next()) {
                    returnObject = new TrustAnchorData();
                    returnObject.setNickname(
                                 resultSet.getString(
                                 CasDBConstants.COL_TRUST_NICKNAME.trim()));
                    returnObject.setAuthMethod(
                                 resultSet.getString(
                                 CasDBConstants.COL_AUTHMETHOD.trim()));
                    returnObject.setAuthData(
                                 resultSet.getString(
                                 CasDBConstants.COL_AUTHDATA.trim()));
                }
            }
        }
        catch (SQLException exp) {
            logger.error(i18n.getMessage("retrieveErr", new Object[] {
                "trust anchor", query }), exp);
            throw new CasDBException(i18n.getMessage("retrieveErr", 
                                                     new Object[] {
                                                         "trust anchor",
                                                         exp.getMessage() }),
                                     exp);
        }
        finally {
            // returning connection irrespective of whether stmt
            // and result set are closed or not.
            CasDBStorage.returnDBConnection(connection);
            try {
                if (resultSet != null)
                    resultSet.close();
                if (statement != null)
                    statement.close();
            }
            catch (SQLException exp) {
                logger.error(i18n.getMessage("retrieveErrClose", 
                                             "trust anchor"), exp); 
                throw new CasDBException(i18n.getMessage("retrieveErrClose", 
                                                         "trust anchor") 
                                         + exp.getMessage(), exp);
            }
        }
        return returnObject;
    }
    
    /**
     * Remove trust anchor entry from database
     * Manipulates only trust anchor table
     */
    public static void deleteObject(String nickname) throws CasDBException {

        // Trust anchor can be a member of object group, no SQL constraint
        // defines that. So check.
        if (ObjectGroupDataHandler.isMember(nickname.trim(), 
                                            CasConstants.TRUSTANCHOR_SPEC)) {
            String errMesg = i18n.getMessage("memberOfGpErr", new Object[] {
                "Trust anchor", nickname.trim(), "object group" });
            logger.debug(errMesg);
            throw new CasDBException(errMesg);
        }
        String query = "delete from" + CasDBConstants.TABLE_TRUSTANCHOR 
            + " where" + CasDBConstants.COL_TRUST_NICKNAME + " = '" 
            + nickname.trim() + "'";
        int result = CasDBStorage.runUpdateQuery(query);
        if (result <= 0) {
            String err = i18n.getMessage("trustAnchorDelError", nickname);
            logger.error(err);
            throw new CasDBException(err);
        }
    }

    public static String getNickname(String authData, String authMeth)
        throws CasDBException {

        logger.debug("authData " + authData + " authMeth " + authMeth);
        String query = "select" + CasDBConstants.COL_TRUST_NICKNAME + " from"
            + CasDBConstants.TABLE_TRUSTANCHOR + " where (" 
            + CasDBConstants.COL_AUTHMETHOD + " ='" + authMeth.trim() 
            + "') and (" + CasDBConstants.COL_AUTHDATA + " ='"
            + authData.trim() + "')";
        Connection connection = CasDBStorage.getDBConnection();
        Statement statement = null;
        ResultSet resultSet = null;
        String nickname = null;
        try {
            statement = connection.createStatement();
            resultSet = statement.executeQuery(query);
            if (resultSet!=null) {
                if (resultSet.next()) {
                    nickname =  
                        resultSet.getString(
                                  CasDBConstants.COL_TRUST_NICKNAME.trim());
                }
            }
        }
        catch (SQLException exp) {
            logger.error(i18n.getMessage("retrieveErr", new Object[] { 
                "trust anchor nickname", query }), exp);
            throw new CasDBException(i18n.getMessage("retrieveErr", 
                      new Object[] { "trust anchor nickname", 
                                     exp.getMessage() }), exp);
        }
        finally {
            // returning connection irrespective of whether stmt
            // and result set are closed or not.
            CasDBStorage.returnDBConnection(connection);
            try {
                if (resultSet != null)
                    resultSet.close();
                if (statement != null)
                    statement.close();
            }
            catch (SQLException exp) {
                logger.warn(i18n.getMessage("retrieveErrClose", 
                                            "trust anchor name"), exp);
            }
        }
        return nickname;
    }
}
