/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.cas.impl.databaseAccess;

import java.sql.Connection;
import java.sql.Statement;
import java.sql.SQLException;
import java.sql.ResultSet;

import java.util.Vector;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.globus.cas.impl.CasConstants;

import org.globus.cas.types.CasObjectData;
import org.globus.cas.types.UserGroupData;
import org.globus.cas.types.ArrayOfString;

import org.globus.util.I18n;

/**
 * Used to manipulate the UserGroupData in the database
 */
public class UserGroupDataHandler {

    static Log logger = LogFactory.getLog(UserGroupData.class.getName() );

    private static I18n i18n =
        I18n.getI18n("org.globus.cas.impl.databaseAccess.errors",
                     UserGroupDataHandler.class.getClassLoader());

    /**
     * Insert user group data into group table 
     * Does not create a new user entry and will fail if the user does not 
     * already exist in the user table. 
     */
    public static String storeObject(UserGroupData userGpData) 
        throws CasDBException {
        Connection connection = CasDBStorage.getDBConnection();
        String groupName = userGpData.getGroupName();
        // Store user group data
        String query = "insert into " + CasDBConstants.TABLE_USERGP + " (" 
            + CasDBConstants.COL_USERGP_NAME + ") values ('" 
            + userGpData.getGroupName() + "')";
        Statement statement = null;
        try {
            statement = connection.createStatement();
            logger.debug("store query is " + query);
            statement.executeUpdate(query);
        }
        catch (SQLException exp) {
            logger.error(i18n.getMessage("storeErr", new Object[] { 
                "user group", query }), exp);
            throw new CasDBException(i18n.getMessage("storeErr", new Object[] {
                "user group", exp.getMessage() }), exp);
        }
        finally {
            // returning connection irrespective of whether stmt
            // and result set are closed or not.
            CasDBStorage.returnDBConnection(connection);
            try {
                if (statement != null)
                    statement.close();
            }
            catch (SQLException exp) {
                logger.warn(i18n.getMessage("storeErrClose", "user group"),
                            exp);
            }
        }
        return groupName;
    }
    
    /**
     * List of user groups
     */
    public static String[] list() throws CasDBException {
        StringBuffer query = new StringBuffer("select ");
        query.append(CasDBConstants.COL_USERGP_NAME).append(" from")
            .append(CasDBConstants.TABLE_USERGP);
        return CasDBStorage.runListQuery(query.toString(), 
                                         CasDBConstants.COL_USERGP_NAME);
    }

    /** 
     * Retrieve user group object
     * Picks up values from user group table and user group entries table
     */
    public static CasObjectData retrieveObject(String groupName) 
        throws CasDBException {
        
        UserGroupData returnObject = null;
        Connection connection = CasDBStorage.getDBConnection();
        Vector temp = null;
        String query = "select" + CasDBConstants.COL_USER_NICKNAME + " from" 
            + CasDBConstants.TABLE_USERGP_ENTRY + " where " 
            + CasDBConstants.COL_USERGP_NAME + " ='" + groupName.trim() + "'";
        Statement statement = null;
        ResultSet resultSet = null;
        try {
            statement = connection.createStatement();
            resultSet = statement.executeQuery(query);
            if ((resultSet!=null) && (resultSet.next())) {
                temp = new Vector(resultSet.getFetchSize());
                do {
                    temp.add(resultSet.getString(
                            CasDBConstants.COL_USER_NICKNAME.trim()));
                } while (resultSet.next());
            }
        }
        catch (SQLException exp) {
            CasDBStorage.returnDBConnection(connection);
            logger.error(i18n.getMessage("retrieveErr", new Object[] {
                "user group", query }), exp);
            throw new CasDBException(i18n.getMessage("retrieveErr", 
                      new Object[] { "user group", exp.getMessage() }), exp);
        }
        finally {
            try {
                if (resultSet != null)
                    resultSet.close();
                if (statement != null)
                    statement.close();
            }
            catch (SQLException exp) {
                logger.warn(i18n.getMessage("retrieveErrClose", "user group"), 
                            exp);
            }
        }

        ArrayOfString arrayOfUserNames = null;
        if (temp != null) {
            String userNamesString[] = new String[temp.size()];
            temp.toArray(userNamesString);
            arrayOfUserNames = new ArrayOfString(userNamesString);
        }
        query = "select * from" + CasDBConstants.TABLE_USERGP + " where" 
            + CasDBConstants.COL_USERGP_NAME + " = '" + groupName.trim() + "'";

        try {
            statement = connection.createStatement();
            resultSet = statement.executeQuery(query);
            if (resultSet!=null) {
                if (resultSet.next()) {
                    returnObject = new UserGroupData();
                    returnObject.setGroupName(resultSet.getString(
                                 CasDBConstants.COL_USERGP_NAME.trim()));
                    returnObject
                        .setUserNames(arrayOfUserNames);
                }
            }
        }
        catch (SQLException exp) {
            logger.error(i18n.getMessage("retrieveErr", new Object[] {
                "user group", query }), exp);
            throw new CasDBException(i18n.getMessage("retrieveErr", 
                      new Object[] { "user group", exp.getMessage() }), exp);
        }
        finally {
            // returning connection irrespective of whether stmt
            // and result set are closed or not.
            CasDBStorage.returnDBConnection(connection);
            try {
                if (resultSet != null)
                    resultSet.close();
                if (statement != null)
                    statement.close();
            }
            catch (SQLException exp) {
                logger.warn(i18n.getMessage("retrieveErrClose", "user group"), 
                            exp);
            }
        }
        return returnObject;
    }

    /** 
     * Deletes user group data
     * Manipulates user group table 
     */
    public static void deleteObject(String groupName) throws CasDBException {

        logger.debug("delete object called");
        // User group can be a member of object group, no SQL constraint
        // defines that. So check.
        if (ObjectGroupDataHandler.isMember(groupName.trim(), 
                                            CasConstants.USERGP_SPEC)) {
            String errMesg = i18n.getMessage("memberOfGpErr", new Object[] {
                "User group ", groupName.trim(), "object" });
            logger.error(errMesg);
            throw new CasDBException(errMesg);
        }

        Connection connection = CasDBStorage.getDBConnection();
        String query = "delete from" + CasDBConstants.TABLE_USERGP + " where" 
            + CasDBConstants.COL_USERGP_NAME + " ='" + groupName.trim() + "'";
        Statement statement = null;
        int result = -1;
        try {
            statement = connection.createStatement();
            logger.debug("delete object query is " + query);
            result = statement.executeUpdate(query);
        }
        catch (SQLException exp) {
            logger.error(i18n.getMessage("delObjErr", new Object[] {
                "user group", query}), exp);
            throw new CasDBException(i18n.getMessage("delObjErr", 
                  new Object[] { "user group", exp.getMessage()}), exp);
        }
        finally {
            // returning connection irrespective of whether stmt
            // and result set are closed or not.
            CasDBStorage.returnDBConnection(connection);
            try {
                if (statement != null)
                    statement.close();
            }
            catch (SQLException exp) {
                logger.warn(i18n.getMessage("delObjErrClose", "user group"), 
                            exp);
            }
        }
        if (result <= 0) {
            String err = i18n.getMessage("userGpDelError", groupName);
            logger.error(err);
            throw new CasDBException(err);
        }
    }

    /**
     * Adds a user a member of the group
     * Manipulates user group entry table
     */
    public static void addGroupMember(String groupName, String userName) 
        throws CasDBException {
        
        if ((groupName == null) || (userName == null)) {
            String err = i18n.getMessage("paramNull", "Groupname or Username");
            logger.error(err);
            throw new CasDBException(err);
        }
        
        String query = "insert into" + CasDBConstants.TABLE_USERGP_ENTRY
            + " ( " + CasDBConstants.COL_USERGP_NAME + "," 
            + CasDBConstants.COL_USER_NICKNAME + ") values ( '" 
            + groupName.trim() + "','" + userName.trim() + "')";
        Connection connection = CasDBStorage.getDBConnection();
        Statement statement = null;
        try {
            statement = connection.createStatement();
            logger.debug(" query is " + query);
            statement.executeUpdate(query);
        }
        catch (SQLException exp) {
            logger.error(i18n.getMessage("addGpErr", "user"), exp);
            throw new CasDBException(i18n.getMessage("addGpErr", "user") +
                                     exp.getMessage(), exp);
        }
        finally {
            // returning connection irrespective of whether stmt
            // and result set are closed or not.
            CasDBStorage.returnDBConnection(connection);
            try {
                if (statement != null)
                    statement.close();
            }
            catch (SQLException exp) {
                logger.warn(i18n.getMessage("addGpErrClose", "user"), exp);
            }
        }
    }

    /**
     * Removes a user a member of the group
     * Manipulates user group entry table
     */
    public static void removeGroupMember(String groupName, String userName) 
        throws CasDBException {
        
        if ((groupName == null) || (userName == null)) {
            String err = i18n.getMessage("paramNull", "Groupname or Username");
            logger.error(err);
            throw new CasDBException(err);
        }
        
        String query = "delete from" + CasDBConstants.TABLE_USERGP_ENTRY
            + " where (" + CasDBConstants.COL_USERGP_NAME + "='" 
            + groupName.trim() + "') and (" + CasDBConstants.COL_USER_NICKNAME 
            + " ='" + userName.trim() + "')";
        Connection connection = CasDBStorage.getDBConnection();
        Statement statement = null;
        int result = -1;
        try {
            statement = connection.createStatement();
            logger.debug(" query is " + query);
            result = statement.executeUpdate(query);
        }
        catch (SQLException exp) {
            String err = i18n.getMessage("removeGpErr", 
                                         new Object[] { userName , groupName});
            logger.error(err, exp);
            throw new CasDBException(err, exp);
        }
        finally {
            // returning connection irrespective of whether stmt
            // and result set are closed or not.
            CasDBStorage.returnDBConnection(connection);
            try {
                if (statement != null)
                    statement.close();
            }
            catch (SQLException exp) {
                logger.warn(i18n.getMessage("removeGpErrClose", "user"), exp);
            }
        }
        if (result <= 0) {
            String err = i18n.getMessage("userMemDelError", new Object[] 
                { userName , groupName});
            logger.error(err);
            throw new CasDBException(err);
        }
    }
    
    /**
     * Method that checks if a group exists
     */
    public static boolean groupExists(String groupName) throws CasDBException {
        
        if (groupName == null) {
            logger.error(i18n.getMessage("paramNull", "Group name"));
            throw new CasDBException(i18n.getMessage("paramNull", 
                                                     "Group name"));
        } 
        String query = "select * from" + CasDBConstants.TABLE_USERGP 
            + " where" + CasDBConstants.COL_USERGP_NAME + "='"
            + groupName.trim() + "'";
        Connection connection = CasDBStorage.getDBConnection();
        Statement statement = null;
        ResultSet resultSet = null;
        try {
            statement = connection.createStatement();
            resultSet = statement.executeQuery(query);
            if ((resultSet!=null) && (resultSet.next())) {
                logger.debug("There is an entry for this user group ");
                return true;
            }
        }
        catch (SQLException exp) {
            logger.error(i18n.getMessage("verifyGpPresence", new Object[] {
                                         groupName, query }), exp);
            throw new CasDBException(i18n.getMessage("verifyGpPresence", 
                  new Object[] { groupName, query }), exp);
        }
        finally {
            // returning connection irrespective of whether stmt
            // and result set are closed or not.
            CasDBStorage.returnDBConnection(connection);
            try {
                if (resultSet != null)
                    resultSet.close();
                if (statement != null)
                    statement.close();
            }
            catch (SQLException exp) {
                logger.warn(i18n.getMessage("verifyGpPresenceClose", 
                                            groupName), exp);
            }
        }
        return false;
    }
}
