/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.cas.impl.databaseAccess;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.globus.cas.types.UserData;
import org.globus.cas.types.ObjectData;
import org.globus.cas.types.UserGroupData;
import org.globus.cas.types.ServiceTypeData;
import org.globus.cas.types.TrustAnchorData;
import org.globus.cas.types.ObjectGroupData;
import org.globus.cas.types.NamespaceData;
import org.globus.cas.types.ArrayOfString;

import org.globus.cas.utils.CasStringUtils;

import java.util.Vector;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class TestObjectData extends TestCase {

    static Log logger =
              LogFactory.getLog(TestObjectData.class.getName());

    public TestObjectData(String name){
        super(name);
    }

    public static Test suite() {
        return new TestSuite(TestObjectData.class);
    }

    public void testObjectData() throws Exception {

        String nickname = "nickname";
        String basename = "basename";
        String compAlg = "ExactComparison";

        // Store name space data
        NamespaceData nsData = new NamespaceData();
        nsData.setNickname(nickname);
        nsData.setBasename(basename);
        nsData.setComparisonAlg(compAlg);
        NamespaceDataHandler.storeObject(nsData);
        logger.debug("Store object went through " + nsData.toString());

        String objectName = "objectName";
        String objectNamespace = nickname;
        // Store object data without groups
        ObjectData objectData = new ObjectData();
        objectData.setObjectName(objectName);
        objectData.setObjectNamespace(objectNamespace);
        String objectId = ObjectDataHandler.storeObject(objectData);
        objectData.setObjectId(objectId);
        assertTrue(objectId!=null);
        logger.debug("Store  object data went through " 
                     + CasStringUtils.objectToString(objectData));

        // try storing data that matches the above
        boolean exp = false;
        try {
            ObjectDataHandler.storeObject(objectData);
        }
        catch (CasDBException dbExp) {
            exp = true;
        }
        assertTrue(exp);



        // Retrieve object data without groups
        ObjectData retrievedObjectData = 
            (ObjectData)ObjectDataHandler.retrieveObject(objectName, 
                                                         objectNamespace);
        assertTrue(retrievedObjectData!=null);
        logger.debug("Retrieve object data  " 
                     + CasStringUtils.objectToString(retrievedObjectData));
        assertTrue(retrievedObjectData.equals(objectData));
        logger.debug("Retrieve object data went through");

        // List of objects
        String[] list = ObjectDataHandler.list();
        assertTrue(list.length==1);
        logger.debug("List objects went through" + list.length);

        // Delete object data without groups
        ObjectDataHandler.deleteObject(objectName, objectNamespace);
        exp = false;
        try {
            ObjectData objData = 
                (ObjectData)ObjectDataHandler.retrieveObject(objectName, 
                                                             objectNamespace);
        } catch (CasDBException e) {
            if ((e.getMessage().indexOf("does not exist")) != -1)
                exp = true;
        }
        assertTrue(exp);
        logger.debug("Delete object data went through");

        // Object group data storage
        String objectGroupName = "objectGroupName";
        ObjectGroupData objGroup = new ObjectGroupData();
        objGroup.setGroupName(objectGroupName);
        String objGroupId = ObjectGroupDataHandler.storeObject(objGroup);
        assertTrue(objGroupId != null);
        assertTrue(objGroupId.equals(objectGroupName));
        logger.debug(" object group data went through " + objGroup.toString());

        // retrieve ObjectGroup
        ObjectGroupData retrievedData = 
            (ObjectGroupData)ObjectGroupDataHandler.retrieveObject(
                                                    objectGroupName);
        assertTrue(retrievedData!=null);
        logger.debug("obj group retrieved data is " 
                     + retrievedData.toString());
        assertTrue(retrievedData.equals(objGroup));

        // Store object data with group
        ObjectData objectData1 = new ObjectData();
        objectData1.setObjectName(objectName);
        objectData1.setObjectNamespace(objectNamespace);
        String temp[] = new String[1];
        temp[0] = objGroupId;
        objectData1.setGroupNames(new ArrayOfString(temp));
        String objectId1 = ObjectDataHandler.storeObject(objectData1);
        assertTrue(objectId1!=null);
        logger.debug(" store object data with group went through " 
                     + objectId1);

        // Retrieve object data with groups
        ObjectData retrievedObjectData1 = 
            (ObjectData)ObjectDataHandler.retrieveObject(objectName, 
                                                         objectNamespace);
        assertTrue(retrievedObjectData1!=null);
        logger.debug(" retrieve object dat awith group went through");

        // List objectGroup
        String gpList[] = ObjectGroupDataHandler.list();
        assertTrue(gpList.length==1);
        logger.debug("grouplist went through " + gpList.length);

        // delete ObjectGroup data
        ObjectGroupDataHandler.deleteObject(objectGroupName);
        ObjectGroupData deletedData = 
            (ObjectGroupData)ObjectGroupDataHandler.retrieveObject(
                                                    objectGroupName);
        assertTrue(deletedData==null);
        
        // delete object with group
        ObjectDataHandler.deleteObject(objectName, objectNamespace);

        // To test adding of group members 
        // adding ObjectGroup data
        objectGroupName = "objectGroupName1";
        objGroup = new ObjectGroupData();
        objGroup.setGroupName(objectGroupName);
        String objGroupId1 = ObjectGroupDataHandler.storeObject(objGroup);
        assertTrue(objGroupId1 != null);
        logger.debug("Store object goupr went through " + objGroupId1);

        // Second group to test listing of groups
        String objGpName = "objectGroupName2";
        objGroup = new ObjectGroupData();
        objGroup.setGroupName(objGpName);
        ObjectGroupDataHandler.storeObject(objGroup);

        testGroupMemberObject(objectGroupName, objGpName, objectNamespace);
        testGroupMemberUser(objectGroupName, objGpName);
        testGroupMemberUserGp(objectGroupName, objGpName);
        testGroupMemberServiceType(objectGroupName, objGpName);
        testGroupMemberNamespace(objectGroupName, objGpName);
        testGroupMemberTrustAnchor(objectGroupName, objGpName);

        // add a wild card namespace
        String wildNickname = "nickname1";
        nsData.setNickname(wildNickname);
        nsData.setBasename(basename);
        nsData.setComparisonAlg("WildCardComparison");
        NamespaceDataHandler.storeObject(nsData);

        // store obj with wildcard
        String wildCardObjName = "foo/bar/*";
        objectData.setObjectName(wildCardObjName);
        objectData.setObjectNamespace(wildNickname);
        String wildObj = ObjectDataHandler.storeObject(objectData);
        assertTrue(wildObj != null);
        // try storing a object covered by this
        /*      exp = false;
        objectData.setObjectName("foo/bar/c/*");
        objectData.setObjectNamespace(wildNickname);
        try {
            ObjectDataHandler.storeObject(objectData);
        }
        catch (CasDBException dbExp) {
            exp = true;
        }
        assertTrue(exp);

        // try obj exists
        objectData.setObjectName("foo/*");
        objectData.setObjectNamespace(wildNickname);
        exp = false;
        try {
            ObjectDataHandler.storeObject(objectData);
        }
        catch (CasDBException dbExp) {
            exp = true;
        }
        assertTrue(exp);
        */

        // Get a list of matches for the above
        Vector matchedObjs = 
            ObjectDataHandler.retrieveMatchingObjects(wildNickname, "foo/*");
        assertTrue(matchedObjs == null);
        
        Vector matchedObjs1 = 
            ObjectDataHandler.retrieveMatchingObjects(wildNickname, "foo/bar/d");
        assertTrue(matchedObjs1 != null);
        assertTrue(matchedObjs1.size() == 1);
        String retrievedId = (String)matchedObjs1.get(0);
        assertTrue(retrievedId.equals(wildObj));
        
        // store obj not covered
        objectData.setObjectName("bar/c");
        objectData.setObjectNamespace(wildNickname);
        String wildObj2 = ObjectDataHandler.storeObject(objectData);
        assertTrue(wildObj2 != null);

        ObjectDataHandler.deleteObjectForId(wildObj);
        ObjectDataHandler.deleteObjectForId(wildObj2);
        NamespaceDataHandler.deleteObject(nickname);
        NamespaceDataHandler.deleteObject(wildNickname);
        ObjectGroupDataHandler.deleteObject(objectGroupName);
        ObjectGroupDataHandler.deleteObject(objGpName);
    }

    private void testGroupMemberObject(String objectGroupName, 
                                       String objGpName,
                                       String namespaceNick) throws Exception {

        // adding an object withput group
        String objectName = "objectName1";
        String objectNamespace = namespaceNick;

        // Store object data without groups
        ObjectData objectData = new ObjectData();
        objectData.setObjectName(objectName);
        objectData.setObjectNamespace(objectNamespace);
        String objectId2 = ObjectDataHandler.storeObject(objectData);
        assertTrue(objectId2!=null);
        logger.debug("Store  object data went through " + objectId2);

        // Add a member to the group - object
        ObjectGroupDataHandler.addGroupMember(objectGroupName, objectId2, 
                                              "object");
        
        // verify that it was added.
        ObjectGroupData objGroupData = 
            (ObjectGroupData)ObjectGroupDataHandler.retrieveObject(
                                                    objectGroupName);
        assertTrue(objGroupData.getObjectSpecs() != null);
        String objectSpecs[] = objGroupData.getObjectSpecs().getStrings();
        assertTrue(objectSpecs!=null);
        assertTrue(CasStringUtils.stringInArray(objectSpecs, objectId2));
        logger.debug("member added to object group successfully");

        // Add to second group
        ObjectGroupDataHandler.addGroupMember(objGpName, objectId2, 
                                              "object");

        // Ensure that the object cannot be deleted.
        boolean exp= false;
        try {
            ObjectDataHandler.deleteObjectForId(objectId2);
        } catch (Exception e) {
            String err = e.getMessage();
            if ((err.indexOf("member of \"object\" group") != -1) &&
                (err.indexOf("Cannot delete") != -1)) {
                exp = true;
            } else {
                logger.error("Error: " + err);
            }
        }
        assertTrue(exp);
        
        // Retrieve group names
        String[] groupNames = 
            ObjectGroupDataHandler.getObjectGroupsForMember(objectId2, 
                                                            "object");
        assertTrue(groupNames != null);
        assertTrue(groupNames.length == 2);
        assertTrue(CasStringUtils.stringInArray(groupNames, objGpName));
        assertTrue(CasStringUtils.stringInArray(groupNames, objectGroupName));

        // remove the entry
        ObjectGroupDataHandler.removeGroupMember(objectGroupName, objectId2, 
                                                 "object");
        logger.debug("member removed from object group");

        // verify that it was removed .
        objGroupData = 
            (ObjectGroupData)ObjectGroupDataHandler.retrieveObject(
                                                    objectGroupName);
        assertTrue(objGroupData.getObjectSpecs() == null);
        logger.debug("member removed from object group successfully");

        // Ensure that the object cannot be still deleted.
        exp= false;
        try {
            ObjectDataHandler.deleteObjectForId(objectId2);
        } catch (Exception e) {
            String err = e.getMessage();
            if ((err.indexOf("member of \"object\" group") != -1) &&
                (err.indexOf("Cannot delete") != -1)) {
                exp = true;
            } else {
                logger.error("Error:" + err);
            }
        }
        assertTrue(exp);

        ObjectGroupDataHandler.removeGroupMember(objGpName, objectId2, 
                                                 "object");
        
        // Add object to table
        ObjectGroupDataHandler.addObjectToGroup(objectGroupName, objectName, 
                                                objectNamespace);
        objGroupData = 
            (ObjectGroupData)ObjectGroupDataHandler.retrieveObject(
                                                    objectGroupName);
        assertTrue(objGroupData.getObjectSpecs() != null);
        objectSpecs = objGroupData.getObjectSpecs().getStrings();
        assertTrue(objectSpecs!=null);
        assertTrue(CasStringUtils.stringInArray(objectSpecs, objectId2));
        logger.debug("object added to the table");

        int objId = ObjectDataHandler.getObjectId(objectName, objectNamespace);
        ObjectGroupDataHandler.removeGroupMember(objectGroupName, 
                                                 Integer.toString(objId), 
                                                 "object");
        
        ObjectDataHandler.deleteObject(objectName, objectNamespace);
    }

    private void testGroupMemberUser(String objectGroupName, 
                                     String objGpName) throws Exception {

        // Store Trust anchor data
        String nickname = "nickname";
        String authMethod = "authMethod";
        String authData ="authData";

        String userNick ="userNick";
        String subjectName = "userSubjectName";

        boolean exp = false;
        try {
            ObjectGroupDataHandler.addGroupMember(objectGroupName, userNick, 
                                                  "user");
        }
        catch (CasDBException e) {
            if ((e.getMessage()).indexOf("does not exist") != -1)
                exp = true;
        }
        assertTrue(exp);

        TrustAnchorData trustAnchor = new TrustAnchorData();
        trustAnchor.setNickname(nickname);
        trustAnchor.setAuthMethod(authMethod);
        trustAnchor.setAuthData(authData);
        TrustAnchorDataHandler.storeObject(trustAnchor);
        logger.debug("Store trust anchor object went through");

        // Store user data
        UserData userData = new UserData();
        userData.setNickname(userNick);
        userData.setSubjectName(subjectName);
        userData.setTrustAnchorName(nickname);
        UserDataHandler.storeObject(userData);
        logger.debug("Store user data went through " + userData.toString());

        // Add a member to the group - object
        ObjectGroupDataHandler.addGroupMember(objectGroupName, userNick, 
                                              "user");
        
        // verify that it was added.
        ObjectGroupData objGroupData = 
            (ObjectGroupData)ObjectGroupDataHandler.retrieveObject(
                                                    objectGroupName);
        assertTrue(objGroupData.getObjectSpecs() != null);
        String objectSpecs[] = objGroupData.getObjectSpecs().getStrings();
        assertTrue(objectSpecs!=null);
        assertTrue(CasStringUtils.stringInArray(objectSpecs, userNick));
        logger.debug("member added to object group successfully");

        // Add to second group
        ObjectGroupDataHandler.addGroupMember(objGpName, userNick, 
                                              "user");
        
        // Retrieve group names
        String[] groupNames = 
            ObjectGroupDataHandler.getObjectGroupsForMember(userNick, 
                                                            "user");
        assertTrue(groupNames != null);
        assertTrue(groupNames.length == 2);
        assertTrue(CasStringUtils.stringInArray(groupNames, objGpName));
        assertTrue(CasStringUtils.stringInArray(groupNames, objectGroupName));

        // Ensure that the user cannot be deleted.
        exp= false;
        try {
            UserDataHandler.deleteObject(userNick);
        } catch (Exception e) {
            String err = e.getMessage();
            if ((err.indexOf("member of \"object\" group") != -1) &&
                (err.indexOf("Cannot delete") != -1))
                exp = true;
            else
                logger.error(err);
        }
        assertTrue(exp);

        // remove the entry
        ObjectGroupDataHandler.removeGroupMember(objectGroupName, userNick, 
                                                 "user");
        logger.debug("member removed from object group");

        // verify that it was removed .
        objGroupData = 
            (ObjectGroupData)ObjectGroupDataHandler.retrieveObject(
                                                    objectGroupName);
        assertTrue(objGroupData.getObjectSpecs() == null);
        logger.debug("member removed from object group successfully");

        // Ensure that the user cannot be still deleted.
        exp= false;
        try {
            UserDataHandler.deleteObject(userNick);
        } catch (Exception e) {
            String err = e.getMessage();
            if ((err.indexOf("member of \"object\" group") != -1) &&
                (err.indexOf("Cannot delete") != -1))
                exp = true;
        }
        assertTrue(exp);

        ObjectGroupDataHandler.removeGroupMember(objGpName, userNick, 
                                                 "user");

        UserDataHandler.deleteObject(userNick);
        TrustAnchorDataHandler.deleteObject(nickname);
    }

    private void testGroupMemberUserGp(String objectGroupName, 
                                       String objGpName) throws Exception {

        // UserGroup data store without usernames
        String groupName = "userGroup";

        boolean exp = false;
        try {
            ObjectGroupDataHandler.addGroupMember(objectGroupName, groupName, 
                                                  "userGroup");
        }
        catch (CasDBException e) {
            if ((e.getMessage()).indexOf("does not exist") != -1)
                exp = true;
        }
        assertTrue(exp);

        UserGroupData userGpData = new UserGroupData();
        userGpData.setGroupName(groupName);
        UserGroupDataHandler.storeObject(userGpData);
        logger.debug("Store user group data went through " 
                     + userGpData.toString());

        // Add a member to the group - object
        ObjectGroupDataHandler.addGroupMember(objectGroupName, groupName, 
                                              "userGroup");
        
        // verify that it was added.
        ObjectGroupData objGroupData = 
            (ObjectGroupData)ObjectGroupDataHandler.retrieveObject(
                                                    objectGroupName);
        assertTrue(objGroupData.getObjectSpecs() != null);
        String objectSpecs[] = objGroupData.getObjectSpecs().getStrings();
        assertTrue(objectSpecs!=null);
        assertTrue(CasStringUtils.stringInArray(objectSpecs, groupName));
        logger.debug("member added to object group successfully");

        // Add to second group
        ObjectGroupDataHandler.addGroupMember(objGpName, groupName, 
                                              "userGroup");

        // Ensure that the user group cannot be deleted.
        exp= false;
        try {
            UserGroupDataHandler.deleteObject(groupName);
        } catch (Exception e) {
            String err = e.getMessage();
            if ((err.indexOf("member of \"object\" group") != -1) &&
                (err.indexOf("Cannot delete") != -1))
                exp = true;
        }
        assertTrue(exp);
        
        // Retrieve group names
        String[] groupNames = 
            ObjectGroupDataHandler.getObjectGroupsForMember(groupName, 
                                                            "userGroup");
        assertTrue(groupNames != null);
        assertTrue(groupNames.length == 2);
        assertTrue(CasStringUtils.stringInArray(groupNames, objGpName));
        assertTrue(CasStringUtils.stringInArray(groupNames, objectGroupName));

        // remove the entry
        ObjectGroupDataHandler.removeGroupMember(objectGroupName, groupName, 
                                                 "userGroup");
        logger.debug("member removed from object group");

        // verify that it was removed .
        objGroupData = 
            (ObjectGroupData)ObjectGroupDataHandler.retrieveObject(
                                                    objectGroupName);
        assertTrue(objGroupData.getObjectSpecs() == null);
        logger.debug("member removed from object group successfully");

        // Ensure that the user group cannot be still deleted.
        exp= false;
        try {
            UserGroupDataHandler.deleteObject(groupName);
        } catch (Exception e) {
            String err = e.getMessage();
            if ((err.indexOf("member of \"object\" group") != -1) &&
                (err.indexOf("Cannot delete") != -1))
                exp = true;
        }
        assertTrue(exp);

        ObjectGroupDataHandler.removeGroupMember(objGpName, groupName, 
                                                 "userGroup");

        UserGroupDataHandler.deleteObject(groupName);
    }

    private void testGroupMemberServiceType(String objectGroupName, 
                                            String objGpName) 
        throws Exception {

        String serviceId = "-1";
        boolean exp = false;
        try {
            ObjectGroupDataHandler.addGroupMember(objectGroupName, serviceId, 
                                                  "serviceType");
        }
        catch (CasDBException e) {
            if ((e.getMessage()).indexOf("does not exist") != -1)
                exp = true;
        }
        assertTrue(exp);

        // Service type entry
        String serviceTypeName = "serviceTypeName";
        ServiceTypeData serviceTypeData = new ServiceTypeData();
        serviceTypeData.setName(serviceTypeName);
        serviceId = ServiceTypeDataHandler.storeObject(serviceTypeData);
        assertTrue(serviceId!=null);
        logger.debug("Store  sercice type data went through");

        // Add a member to the group - object
        ObjectGroupDataHandler.addGroupMember(objectGroupName, serviceId, 
                                              "serviceType");
        
        // verify that it was added.
        ObjectGroupData objGroupData = 
            (ObjectGroupData)ObjectGroupDataHandler.retrieveObject(
                                                    objectGroupName);
        assertTrue(objGroupData.getObjectSpecs() != null);
        String objectSpecs[] = objGroupData.getObjectSpecs().getStrings();
        assertTrue(objectSpecs!=null);
        assertTrue(CasStringUtils.stringInArray(objectSpecs, serviceId));
        logger.debug("member added to object group successfully");

        // Add to second group
        ObjectGroupDataHandler.addGroupMember(objGpName, serviceId, 
                                              "serviceType");

        // Ensure that the service type cannot be deleted.
        exp= false;
        try {
            ServiceTypeDataHandler.deleteObject(serviceId);
        } catch (Exception e) {
            String err = e.getMessage();
            if ((err.indexOf("member of \"object\" group") != -1) &&
                (err.indexOf("Cannot delete") != -1))
                exp = true;
        }
        assertTrue(exp);        

        // Retrieve group names
        String[] groupNames = 
            ObjectGroupDataHandler.getObjectGroupsForMember(serviceId, 
                                                            "serviceType");
        assertTrue(groupNames != null);
        assertTrue(groupNames.length == 2);
        assertTrue(CasStringUtils.stringInArray(groupNames, objGpName));
        assertTrue(CasStringUtils.stringInArray(groupNames, objectGroupName));

        // remove the entry
        ObjectGroupDataHandler.removeGroupMember(objectGroupName, serviceId, 
                                                 "serviceType");
        logger.debug("member removed from object group");

        // verify that it was removed .
        objGroupData = 
            (ObjectGroupData)ObjectGroupDataHandler.retrieveObject(
                                                    objectGroupName);
        assertTrue(objGroupData.getObjectSpecs() == null);
        logger.debug("member removed from object group successfully");

        // Ensure that the service type cannot be still deleted.
        exp= false;
        try {
            ServiceTypeDataHandler.deleteObject(serviceId);
        } catch (Exception e) {
            String err = e.getMessage();
            if ((err.indexOf("member of \"object\" group") != -1) &&
                (err.indexOf("Cannot delete") != -1))
                exp = true;
        }
        assertTrue(exp);

        ObjectGroupDataHandler.removeGroupMember(objGpName, serviceId, 
                                                 "serviceType");

        ServiceTypeDataHandler.deleteObject(serviceTypeName);
    }

    private void testGroupMemberNamespace(String objectGroupName, 
                                          String objGpName) 
        throws Exception {

        String nickname = "newNickname";
        String basename = "basename";
        String compAlg = "compAlg";

        boolean exp = false;
        try {
            ObjectGroupDataHandler.addGroupMember(objectGroupName, nickname, 
                                                  "namespace");
        }
        catch (CasDBException e) {
            if ((e.getMessage()).indexOf("does not exist") != -1)
                exp = true;
        }
        assertTrue(exp);

        // Store name space data
        NamespaceData nsData = new NamespaceData();
        nsData.setNickname(nickname);
        nsData.setBasename(basename);
        nsData.setComparisonAlg(compAlg);
        NamespaceDataHandler.storeObject(nsData);
        NamespaceData retrieveNSData = 
            (NamespaceData)NamespaceDataHandler.retrieveObject(nickname);
        assertTrue(retrieveNSData!=null);
        assertTrue(retrieveNSData.equals(nsData));
        logger.debug("Store object went through " + nsData.toString());

        // Add a member to the group - namespace
        ObjectGroupDataHandler.addGroupMember(objectGroupName, nickname, 
                                              "namespace");
        
        // verify that it was added.
        ObjectGroupData objGroupData = 
            (ObjectGroupData)ObjectGroupDataHandler.retrieveObject(
                                                    objectGroupName);
        assertTrue(objGroupData.getObjectSpecs() != null);
        String objectSpecs[] = objGroupData.getObjectSpecs().getStrings();
        assertTrue(objectSpecs!=null);
        assertTrue(CasStringUtils.stringInArray(objectSpecs, nickname));
        logger.debug("member added to object group successfully");

        // Add to second group
        ObjectGroupDataHandler.addGroupMember(objGpName, nickname, 
                                              "namespace");

        // Ensure that the namespace cannot be deleted.
        exp= false;
        try {
            NamespaceDataHandler.deleteObject(nickname);
        } catch (Exception e) {
            String err = e.getMessage();
            if ((err.indexOf("member of \"object\" group") != -1) &&
                (err.indexOf("Cannot delete") != -1)) {
                exp = true;
            } else {
                logger.error("Error: " + err);
            }
        }
        assertTrue(exp);
        
        // Retrieve group names
        String[] groupNames = 
            ObjectGroupDataHandler.getObjectGroupsForMember(nickname, 
                                                            "namespace");
        assertTrue(groupNames != null);
        assertTrue(groupNames.length == 2);
        assertTrue(CasStringUtils.stringInArray(groupNames, objGpName));
        assertTrue(CasStringUtils.stringInArray(groupNames, objectGroupName));

        // remove the entry
        ObjectGroupDataHandler.removeGroupMember(objectGroupName, nickname, 
                                                 "namespace");
        logger.debug("member removed from object group");

        // verify that it was removed .
        objGroupData = 
            (ObjectGroupData)ObjectGroupDataHandler.retrieveObject(
                                                    objectGroupName);
        assertTrue(objGroupData.getObjectSpecs() == null);
        logger.debug("member removed from object group successfully");

        // Ensure that the namespace cannot be still deleted.
        exp= false;
        try {
            NamespaceDataHandler.deleteObject(nickname);
        } catch (Exception e) {
            String err = e.getMessage();
            if ((err.indexOf("member of \"object\" group") != -1) &&
                (err.indexOf("Cannot delete") != -1))
                exp = true;
        }
        assertTrue(exp);

        ObjectGroupDataHandler.removeGroupMember(objGpName, nickname, 
                                                 "namespace");

        NamespaceDataHandler.deleteObject(nickname);
    }

    private void testGroupMemberTrustAnchor(String objectGroupName, 
                                            String objGpName) 
        throws Exception {

        String nickname = "nickname";
        String authMethod = "authMethod";
        String authData ="authData";

        boolean exp = false;
        try {
            ObjectGroupDataHandler.addGroupMember(objectGroupName, nickname, 
                                                  "trustAnchor");
        }
        catch (CasDBException e) {
            if ((e.getMessage()).indexOf("does not exist") != -1)
                exp = true;
        }
        assertTrue(exp);

        // Store trust anchor data
        TrustAnchorData trustAnchor = new TrustAnchorData();
        trustAnchor.setNickname(nickname);
        trustAnchor.setAuthMethod(authMethod);
        trustAnchor.setAuthData(authData);
        TrustAnchorDataHandler.storeObject(trustAnchor);
        TrustAnchorData retrieveAnchorData = 
            (TrustAnchorData)TrustAnchorDataHandler.retrieveObject(nickname);
        assertTrue(retrieveAnchorData!=null);
        logger.debug("Store object went through " + trustAnchor.toString());
        
        // Add a member to the group - trustanchor
        ObjectGroupDataHandler.addGroupMember(objectGroupName, nickname, 
                                              "trustAnchor");
        
        // verify that it was added.
        ObjectGroupData objGroupData = 
            (ObjectGroupData)ObjectGroupDataHandler.retrieveObject(
                                                    objectGroupName);
        assertTrue(objGroupData.getObjectSpecs() != null);
        String objectSpecs[] = objGroupData.getObjectSpecs().getStrings();
        assertTrue(objectSpecs!=null);
        assertTrue(CasStringUtils.stringInArray(objectSpecs, nickname));
        logger.debug("member added to object group successfully");

        // Add to second group
        ObjectGroupDataHandler.addGroupMember(objGpName, nickname, 
                                              "trustAnchor");
        
        // Retrieve group names
        String[] groupNames = 
            ObjectGroupDataHandler.getObjectGroupsForMember(nickname, 
                                                            "trustAnchor");
        assertTrue(groupNames != null);
        assertTrue(groupNames.length == 2);
        assertTrue(CasStringUtils.stringInArray(groupNames, objGpName));
        assertTrue(CasStringUtils.stringInArray(groupNames, objectGroupName));

        // Ensure that the trust anchor cannot be deleted.
        exp= false;
        try {
            TrustAnchorDataHandler.deleteObject(nickname);
        } catch (Exception e) {
            String err = e.getMessage();
            if ((err.indexOf("member of \"object group\" group") != -1) &&
                (err.indexOf("Cannot delete") != -1))
                exp = true;
            else
                logger.error(e);
        }
        assertTrue(exp);

        // remove the entry
        ObjectGroupDataHandler.removeGroupMember(objectGroupName, nickname, 
                                                 "trustAnchor");
        logger.debug("member removed from object group");

        // verify that it was removed .
        objGroupData = 
            (ObjectGroupData)ObjectGroupDataHandler.retrieveObject(
                                                    objectGroupName);
        assertTrue(objGroupData.getObjectSpecs() == null);
        logger.debug("member removed from object group successfully");

        // Ensure that the trust anchor cannot be still deleted.
        exp= false;
        try {
            TrustAnchorDataHandler.deleteObject(nickname);
        } catch (Exception e) {
            String err = e.getMessage();
            if ((err.indexOf("member of \"object group\" group") != -1) &&
                (err.indexOf("Cannot delete") != -1))
                exp = true;
            else
                logger.error(e);
        }
        assertTrue(exp);

        ObjectGroupDataHandler.removeGroupMember(objGpName, nickname, 
                                                 "trustAnchor");

        TrustAnchorDataHandler.deleteObject(nickname);
    }
}
