/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.cas.impl.service;

import junit.framework.Test;
import junit.framework.TestSuite;
import junit.framework.TestResult;

import java.io.FileInputStream;

import java.util.Properties;

import org.globus.cas.CASPortType;

import org.globus.cas.impl.databaseAccess.CasDBOptions;
import org.globus.cas.impl.databaseAccess.CasDBStorage;

import org.globus.axis.util.Util;

import org.globus.cas.utils.TestSetup;

/**
 * Tests for CAS server package
 */
public class PackageTests extends TestSuite {

    static {
        Util.registerTransport();
        org.globus.wsrf.impl.security.authentication.wssec.GSSConfig.init();
    }

    public PackageTests(String name) {
        super(name);
    }

    public void run(TestResult result) {
       try {
           String propFileName = System.getProperty("casTestProperties");
           Properties prop = new Properties();
           prop.load(new FileInputStream(propFileName));
           String host = (String)prop.getProperty("host");
           String port = (String)prop.getProperty("port");
           CasDBOptions casDbOption = 
               new CasDBOptions(prop.getProperty("dbDriver"), 
                                prop.getProperty("dbConnectionURL"),
                                prop.getProperty("dbUsername"), 
                                prop.getProperty("dbPassword"));
           System.out.println("dbConnection " 
                              + prop.getProperty("dbConnectionURL"));
           CasDBStorage.setupDBConnection(casDbOption);
           
           String securityType = (String)prop.getProperty("securityType");
           String protType = (String)prop.getProperty("protectionType");

           TestSetup testSetup = new TestSetup(host, port, securityType, 
                                               protType);
           CASPortType casPort = testSetup.getCasPort();
           TestSelfEnrollPermissions.setParams(casPort);
           TestSelfUnenrollPermissions.setParams(casPort);
           TestQueries.setParams(casPort);
           TestAssertionGeneration.setParams(casPort);
           TestGrantUser1.setParams(casPort);
           super.run(result);
       } catch(Exception e) {
           result.addError(this, e);
       }
    }
    
    public static Test suite() throws Exception {
        TestSuite suite = new PackageTests("CAS Service Tests");
        suite.addTestSuite(TestSelfEnrollPermissions.class);
        suite.addTestSuite(TestSelfUnenrollPermissions.class);
        suite.addTestSuite(TestQueries.class);
        suite.addTestSuite(TestAssertionGeneration.class);
        suite.addTestSuite(TestGrantUser1.class);
        return suite;
    }
}
