/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.cas.impl.service;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.globus.cas.faults.CasFault;
import org.globus.cas.faults.NoPermissionFault;

import org.globus.cas.types.ServiceTypeData;
import org.globus.cas.types.TrustAnchorData;
import org.globus.cas.types.UserData;
import org.globus.cas.types.UserGroupData;
import org.globus.cas.types.PolicyData;
import org.globus.cas.types.ObjectData;
import org.globus.cas.types.NamespaceData;
import org.globus.cas.types.ObjectGroupData;
import org.globus.cas.types.ArrayOfString;
import org.globus.cas.types.GetGroupMembers;
import org.globus.cas.types.VoidType;
import org.globus.cas.types.CasObjectDesc;

import org.globus.cas.impl.databaseAccess.ObjectDataHandler;
import org.globus.cas.impl.databaseAccess.ObjectGroupDataHandler;
import org.globus.cas.impl.databaseAccess.UserDataHandler;
import org.globus.cas.impl.databaseAccess.UserGroupDataHandler;
import org.globus.cas.impl.databaseAccess.TrustAnchorDataHandler;
import org.globus.cas.impl.databaseAccess.ServiceTypeActionHandler;
import org.globus.cas.impl.databaseAccess.PolicyDataHandler;
import org.globus.cas.impl.databaseAccess.CasDBException;

import org.globus.cas.impl.CasConstants;
import org.globus.cas.utils.CasStringUtils;

import org.globus.cas.CASPortType;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class TestQueries extends TestCase {

    static Log logger = 
	LogFactory.getLog(TestQueries.class.getName());

    static CASPortType casPort = null;
    
    String objId = null;
    String query = null;
    UserData userData = null;
    UserGroupData userGpData = null;
    TrustAnchorData trustAnchor = null;
    ObjectGroupData objectGpData = null;
    String policyId = null;

    public TestQueries(String name){
	super(name);
    }

    public static Test suite() {
        return new TestSuite(TestQueries.class);
    }

    public static void setParams(CASPortType casPort_) {
	casPort = casPort_;
    }


    private void initialSetup() throws Exception {
	
	objId = 
	    Integer.toString(ObjectDataHandler.getObjectId(
					       CasConstants.OBJECT_SELF, 
					       CasConstants.NAMESPACE_SELF));
	query =
	    Integer.toString(ServiceTypeActionHandler.getServiceActionId(
			     CasConstants.SERVICETYPE_CAS, 
			     CasConstants.ACTION_QUERY));
    }
    
    public void testQueries() throws Exception {
	initialSetup();
	// positive test for whoami
	String userName = casPort.whoami(new VoidType());
	assertTrue(userName != null);
	assertTrue(userName.equals(TestConstants.user1));
	listQueriesTest();
	cleanUp();
    }

    // List tests 
    private void listQueriesTest() throws Exception {

	boolean exp = false;
	// Test permission
	try {
	    casPort.list(CasConstants.USER_SPEC);
	}
	catch (NoPermissionFault noPerm) {
	    exp= true;
	}
	assertTrue(exp);

	// add another user
	userData = new UserData();
	String dummyUser = "Dummy user";
	userData.setNickname(dummyUser);
	userData.setSubjectName("dummy subject");
	userData.setTrustAnchorName(TestConstants.defTrustNick);
	UserDataHandler.storeObject(userData);

	// set policy 
	policySetup(true);

	// test wrong type
	exp = false;
	try {
	    casPort.list("weirdType");
	}
	catch (CasFault casFault) {
	    exp= true;
	}
	assertTrue(exp);

        ArrayOfString arrayOfUsers = casPort.list(CasConstants.USER_SPEC);
        assertTrue(arrayOfUsers != null);
	String[] users = arrayOfUsers.getStrings();
	assertTrue(users != null);
	logger.debug("User list length " + users.length);
	assertTrue(users.length == 2);
	assertTrue(CasStringUtils.stringInArray(users, TestConstants.user1));
	assertTrue(CasStringUtils.stringInArray(users, dummyUser));
	
	// add user group
	String groupName = "userGroup1";
	userGpData = new UserGroupData();
	userGpData.setGroupName(groupName);
	UserGroupDataHandler.storeObject(userGpData);
        ArrayOfString listOfUsers = casPort.list(CasConstants.USERGP_SPEC);
        assertTrue(listOfUsers != null);
	users = listOfUsers.getStrings();
	assertTrue(users != null);
	logger.debug("UserGroup list length " + users.length);
	assertTrue(users.length == 2);
	assertTrue(CasStringUtils.stringInArray(users, 
                                                TestConstants.defUserGp));
	assertTrue(CasStringUtils.stringInArray(users, groupName));
	
	// add second user
	String secondUser = "dummy user1";
	userData.setNickname(secondUser);
	userData.setSubjectName("dummy subject1");
	UserDataHandler.storeObject(userData);
	// add member to user group 
	UserGroupDataHandler.addGroupMember(groupName, dummyUser);
	UserGroupDataHandler.addGroupMember(groupName, secondUser);

        ArrayOfString arrayOfMembers = 
            casPort.getGroupMembers(new GetGroupMembers(groupName, "user"));
        assertTrue(arrayOfMembers != null);
	String[] userGpMembers = arrayOfMembers.getStrings();
	assertTrue(userGpMembers != null);
	assertTrue(userGpMembers.length == 2);
	assertTrue(CasStringUtils.stringInArray(userGpMembers, dummyUser));
	assertTrue(CasStringUtils.stringInArray(userGpMembers, secondUser));
	
	// Delete user group entry
	UserGroupDataHandler.removeGroupMember(groupName, dummyUser);
	UserGroupDataHandler.removeGroupMember(groupName, secondUser);

	// Delete user
	UserDataHandler.deleteObject(secondUser);
	UserDataHandler.deleteObject(dummyUser);

	// add another trust anchor
	String trustNick = "dummyTrust";
	trustAnchor = new TrustAnchorData();
	trustAnchor.setNickname(trustNick);
	trustAnchor.setAuthMethod("dummy");
	trustAnchor.setAuthData("dummy");
	TrustAnchorDataHandler.storeObject(trustAnchor);
	
	// test Trust anchor
        ArrayOfString arrayOfAnchors = 
            casPort.list(CasConstants.TRUSTANCHOR_SPEC);
        assertTrue(arrayOfAnchors != null);
	String[] anchors = arrayOfAnchors.getStrings();
	assertTrue(anchors != null);
	logger.debug("TrustAnchor list length " + anchors.length);
	assertTrue(anchors.length == 2);
	assertTrue(CasStringUtils.stringInArray(anchors, 
						TestConstants.defTrustNick));
	assertTrue(CasStringUtils.stringInArray(anchors, trustNick));
	
	// test object
        ArrayOfString arrayOfObj = casPort.list(CasConstants.OBJECT_SPEC);
        assertTrue(arrayOfObj != null);
	String[] objects = arrayOfObj.getStrings();
	assertTrue(objects != null);
	logger.debug("Object list length " + objects.length);
	assertTrue(objects.length == 1);
	
	// test objectGroup
        ArrayOfString arrayOfObjGp = casPort.list(CasConstants.OBJECTGP_SPEC);
        assertTrue(arrayOfObjGp != null);
	String[] objectGp = arrayOfObjGp.getStrings();
	assertTrue(objectGp == null);
	
	// add object group
	String gpName = "dummy";
	objectGpData = new ObjectGroupData();
	objectGpData.setGroupName(gpName);
	ObjectGroupDataHandler.storeObject(objectGpData);
        ArrayOfString arrayGpSpec = casPort.list(CasConstants.OBJECTGP_SPEC);
        assertTrue(arrayGpSpec != null);
	objectGp = arrayGpSpec.getStrings();
	assertTrue(objectGp != null);
	logger.debug("objGp list length " + objectGp.length);
	assertTrue(objectGp.length == 1);
	assertTrue(CasStringUtils.stringInArray(objectGp, gpName));
	
	// add an object
	ObjectData objData = new ObjectData();
	objData.setObjectName("dummyObjName");
	objData.setObjectNamespace(CasConstants.NAMESPACE_SELF);
	String dummyObjId = ObjectDataHandler.storeObject(objData);

	// adding member to object group
	ObjectGroupDataHandler.addGroupMember(gpName, dummyObjId, "object");

	// get group members
        ArrayOfString arrayOfObjMem = 
            casPort.getGroupMembers(new GetGroupMembers(gpName, "object"));
        assertTrue(arrayOfObjMem != null);
	String[] objGpMembers = arrayOfObjMem.getStrings();
	assertTrue(objGpMembers != null);
	assertTrue(objGpMembers.length == 1);
	System.out.println("***" + objGpMembers[0]);
	assertTrue(CasStringUtils.stringInArray(objGpMembers, 
						CasConstants.NAMESPACE_SELF 
						+ CasConstants.OBJECTSPEC_DELIMITER
						+ "dummyObjName, object"));

	// remove object group member
	ObjectGroupDataHandler.removeGroupMember(gpName, dummyObjId, "object");

	// delete object
	ObjectDataHandler.deleteObject("dummyObjName", 
				       CasConstants.NAMESPACE_SELF);

	// delete object group
	ObjectGroupDataHandler.deleteObject(gpName);
    
	// test namespace
        ArrayOfString arrayOfNS = casPort.list(CasConstants.NAMESPACE_SPEC);
        assertTrue(arrayOfNS != null);
    	String[] nsList = arrayOfNS.getStrings();
	assertTrue(nsList!=null);
	logger.debug("namespace list length " + nsList.length);
	assertTrue(nsList.length == 3);
	
	// test service type
        ArrayOfString arrayOfService = 
            casPort.list(CasConstants.SERVICETYPE_SPEC);
        assertTrue(arrayOfService != null);
    	String[] serviceType = arrayOfService.getStrings();
	assertTrue(serviceType !=null);
	logger.debug("service type list length " + serviceType.length);
	assertTrue(serviceType.length == 2);
	
	// add action
	String actionName = "queryTest";
	ServiceTypeActionHandler.addServiceActionMapping(serviceType[0],
							 actionName);
	String actionName1 = "queryTest1";
	ServiceTypeActionHandler.addServiceActionMapping(serviceType[0],
							 actionName1);

	// add service action group
	String dummyServiceGp = "dummyServiceGp";
	ServiceTypeActionHandler.createServiceActionGroup(dummyServiceGp);
	ServiceTypeActionHandler.addServiceActionGpEntry(dummyServiceGp,
							 serviceType[0],
							 actionName);
	ServiceTypeActionHandler.addServiceActionGpEntry(dummyServiceGp,
							 serviceType[0],
							 actionName1);
	
	// Get service action gp members
        ArrayOfString arrayOfSa =             
            casPort.getGroupMembers(new GetGroupMembers(dummyServiceGp,
                                                        "serviceAction"));
        assertTrue(arrayOfSa != null);
	String[] saGpMembers = arrayOfSa.getStrings();
	assertTrue(saGpMembers != null);
	assertTrue(saGpMembers.length == 2);
	assertTrue(CasStringUtils.stringInArray(saGpMembers, serviceType[0] 
                                                + "/" + actionName));
	assertTrue(CasStringUtils.stringInArray(saGpMembers, serviceType[0] 
                                                + "/" + actionName1));

	// clean up
	ServiceTypeActionHandler.removeServiceActionGpEntry(dummyServiceGp,
							 serviceType[0],
							 actionName1);
	ServiceTypeActionHandler.removeServiceActionGpEntry(dummyServiceGp,
							 serviceType[0],
							 actionName);
	ServiceTypeActionHandler.deleteServiceActionGroup(dummyServiceGp);

	ServiceTypeActionHandler.removeServiceActionMapping(serviceType[0],
							 actionName1);
	ServiceTypeActionHandler.removeServiceActionMapping(serviceType[0],
							 actionName);
    }

    public void getObjectTest() throws Exception {

	boolean exp = false;	
	// test Permission
	try {
	    casPort.getCasObject(new CasObjectDesc(CasConstants.USER_SPEC, 
                                                   TestConstants.user1));
	}
	catch (NoPermissionFault noPerm) {
	    exp= true;
	}
	assertTrue(exp);

	// Permission is set
	policySetup(true);

	// Test wrong type
	exp = false;
	try {
	    casPort.getCasObject(new CasObjectDesc("weirdType", 
                                                   TestConstants.user1));
	}
	catch (CasFault casFault) {
	    exp= true;
	}
	assertTrue(exp);
	
	// user
	UserData getUserData = 
	    (UserData)casPort
            .getCasObject(new CasObjectDesc(CasConstants.USER_SPEC, 
                                            userData.getNickname()));
	assertTrue(getUserData != null);
	assertTrue(getUserData.equals(userData));

	// user group
	UserGroupData getUserGpData = 
	    (UserGroupData)casPort
            .getCasObject(new CasObjectDesc(CasConstants.USERGP_SPEC, 
                                            userGpData.getGroupName()));
	assertTrue(getUserGpData != null);
	assertTrue(getUserGpData.equals(userGpData));

	// trust anchor
	TrustAnchorData getTrustAnchor = 
	    (TrustAnchorData)casPort
            .getCasObject(new CasObjectDesc(CasConstants.TRUSTANCHOR_SPEC, 
                                            trustAnchor.getNickname()));
        assertTrue(getTrustAnchor != null);
	assertTrue(getTrustAnchor.equals(trustAnchor));
                          
	// object gp
	ObjectGroupData getObjGpData =
	    (ObjectGroupData)casPort
            .getCasObject(new CasObjectDesc(CasConstants.OBJECTGP_SPEC,
                                            objectGpData.getGroupName()));
	assertTrue(getObjGpData != null);
	assertTrue(getObjGpData.equals(objectGpData));
	
	// object
	String objectStr = CasConstants.NAMESPACE_SELF + "|" 
	    + CasConstants.OBJECT_SELF;
	ObjectData getObjData = (ObjectData)casPort
            .getCasObject(new CasObjectDesc(CasConstants.OBJECT_SPEC,
                                            objectStr));
	assertTrue(getObjData != null);
	assertTrue(getObjData.getObjectNamespace().equals(
			      CasConstants.NAMESPACE_SELF));
	
	assertTrue(getObjData.getObjectName()
                   .equals(CasConstants.OBJECT_SELF));

	// Namespace
	NamespaceData getNamespace =
	    (NamespaceData)casPort
            .getCasObject(new CasObjectDesc(CasConstants.NAMESPACE_SPEC,
                                            CasConstants.NAMESPACE_SELF));
	assertTrue(getNamespace != null);
	assertTrue(getNamespace.getNickname().equals(
					      CasConstants.NAMESPACE_SELF));
	assertTrue(getNamespace.getComparisonAlg().equals(
						   TestConstants.compAlg));

	// Service type
	ServiceTypeData getServiceType =
	    (ServiceTypeData)casPort
            .getCasObject(new CasObjectDesc(CasConstants.SERVICETYPE_SPEC,
                                            CasConstants.SERVICETYPE_CAS));
	assertTrue(getServiceType != null);
	assertTrue(getServiceType.getName().equals(
					    CasConstants.SERVICETYPE_CAS));
	assertTrue(getServiceType.getActionNames() != null);

	policySetup(false);
    }

    private void cleanUp() throws Exception {

        try {
            // delete user
            UserDataHandler.deleteObject(userData.getNickname());

            // delete TA
            TrustAnchorDataHandler.deleteObject(trustAnchor.getNickname());
        } catch (CasDBException exp) {
            if (exp.getMessage().indexOf("No database rows update") == -1) {
                throw exp;
            }
        }

	// delete Policy
	policySetup(false);
    }
    
    private void policySetup(boolean setup) throws Exception {

	if (setup) {
	    PolicyData policyData = new PolicyData();
	    policyData.setUserGroupName(TestConstants.defUserGp);
	    policyData.setObjectSpec(objId);
	    policyData.setObjectSpecDesc(CasConstants.OBJECT_SPEC);
	    policyData.setActionSpec(query);
	    policyData.setActionSpecDesc(CasConstants.SERVICEACTION_SPEC);
	    policyId = PolicyDataHandler.storeObject(policyData);
	}
	else {
	    PolicyDataHandler.deleteObject(policyId);
	}
    }
}
