/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.cas.impl.service;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.globus.cas.types.PolicyData;
import org.globus.cas.types.UserGroupData;
import org.globus.cas.types.ServiceTypeData;
import org.globus.cas.types.ObjectGroupData;
import org.globus.cas.types.ArrayOfString;

import org.globus.cas.impl.CasConstants;

import org.globus.cas.utils.CasStringUtils;

import org.globus.cas.impl.databaseAccess.ObjectDataHandler;
import org.globus.cas.impl.databaseAccess.ObjectGroupDataHandler;
import org.globus.cas.impl.databaseAccess.UserDataHandler;
import org.globus.cas.impl.databaseAccess.UserGroupDataHandler;
import org.globus.cas.impl.databaseAccess.TrustAnchorDataHandler;
import org.globus.cas.impl.databaseAccess.ServiceTypeDataHandler;
import org.globus.cas.impl.databaseAccess.ServiceTypeActionHandler;
import org.globus.cas.impl.databaseAccess.NamespaceDataHandler;

import org.globus.cas.types.DeleteGroup;
import org.globus.cas.types.DeleteObject;
import org.globus.cas.types.ManageUserGroups;
import org.globus.cas.types.ManageObjectGroups;
import org.globus.cas.types.ManageServiceAction;
import org.globus.cas.types.ManageServiceActionGroups;

import org.globus.cas.CASPortType;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class TestSelfUnenrollPermissions extends TestCase {

    static Log logger = 
        LogFactory.getLog(TestSelfUnenrollPermissions.class.getName());

    static CASPortType casPort = null;
    
    String objId = null;
    String unenrollId = null;
    String deleteGroupId = null;
    String deleteGroupEntry = null;
    
    public TestSelfUnenrollPermissions(String name){
        super(name);
    }

    public static Test suite() {
        return new TestSuite(TestSelfUnenrollPermissions.class);
    }

    public static void setParams(CASPortType casPort_) {
        casPort = casPort_;
    }

    private void initialSetup() throws Exception {

        objId = 
            Integer.toString(
                    ObjectDataHandler.getObjectId(
                                                  CasConstants.OBJECT_SELF, 
                                                  CasConstants.NAMESPACE_SELF));
        unenrollId = 
            Integer.toString(
                    ServiceTypeActionHandler.getServiceActionId(
                                             CasConstants.SERVICETYPE_CAS, 
                                             CasConstants.ACTION_UNENROLL));
        deleteGroupId = 
            Integer.toString(
                    ServiceTypeActionHandler.getServiceActionId(
                                             CasConstants.SERVICETYPE_CAS, 
                                             CasConstants.ACTION_DELETE_GROUP));
        deleteGroupEntry = 
            Integer.toString(
                    ServiceTypeActionHandler.getServiceActionId(
                    CasConstants.SERVICETYPE_CAS, 
                    CasConstants.ACTION_REMOVE_GROUP_ENTRY));
    }
    
    // all operations must go through since enroll sets grantAll
    public void testUnenrollData() throws Exception {

        initialSetup();
        PolicyData policyData = new PolicyData();
        policyData.setUserGroupName(TestConstants.defUserGp);

        // attempt removing from serviceActionGp
        casPort
            .manageServiceActionGroups(
            new ManageServiceActionGroups(TestConstants.actionName,
                                          TestConstants.serviceActionGpName, 
                                          "remove", 
                                          TestConstants.serviceName));
        // Ensure it was removed
        String[] serviceActionGpEntry 
            = ServiceTypeActionHandler.retrieveServiceActionGpEntries(
                                       TestConstants.serviceActionGpName);
        assertTrue(!CasStringUtils.stringInArray(serviceActionGpEntry, 
                                                 TestConstants.serviceName 
                                                 + "/" 
                                                 + TestConstants.actionName));

        // attempt deleting serviceActionGp
        casPort
            .deleteGroup(new DeleteGroup(TestConstants.serviceActionGpName,
                                         "serviceAction"));
        // Ensure it was removed
        String[] serviceActionGp = 
            ServiceTypeActionHandler.listServiceActionGroups();
        assertTrue(!CasStringUtils.stringInArray(serviceActionGp, 
                                  TestConstants.serviceActionGpName));

        // attempting to remove serviceAction mapping
        casPort.manageServiceAction(
                new ManageServiceAction(TestConstants.actionName, "remove",
                                        TestConstants.serviceName));
        // Ensure it was removed
        ServiceTypeData serviceType = new ServiceTypeData();
        serviceType.setName(TestConstants.serviceName);
        assertTrue(serviceType.equals(
                  (ServiceTypeData)ServiceTypeDataHandler.retrieveObject(
                                   TestConstants.serviceName)));

        // attempting to delete ServiceType
        casPort.deleteServiceType(TestConstants.serviceName);
        // Ensure it was removed
        String serviceTypeNames[] = ServiceTypeDataHandler.list();
        assertTrue(!CasStringUtils.stringInArray(serviceTypeNames, 
                                                 TestConstants.serviceName));

        // attempt removing from object group
        // Direct DBaccess to get object Id
        String addedObj = 
            Integer.toString(
                    ObjectDataHandler.getObjectId(TestConstants.objName, 
                                                  TestConstants.nsNick));
        casPort.manageObjectGroups(
                new ManageObjectGroups(TestConstants.objectGpName, 
                                       TestConstants.nsNick + "|"
                                       + TestConstants.objName, "object",
                                       "remove"));
        // Ensure it was removed
        ObjectGroupData retrievedObjGp = 
        (ObjectGroupData)ObjectGroupDataHandler
        .retrieveObject(TestConstants.objectGpName);
        ArrayOfString arrayOfSpec = retrievedObjGp.getObjectSpecs();
        assertTrue(arrayOfSpec == null);

        // attempt deleting objectGP
        casPort.deleteGroup(new DeleteGroup(TestConstants.objectGpName, 
                                            "object"));
        // Ensure it was deleted
        String[] listOfObjects = ObjectGroupDataHandler.list();
        assertTrue(!CasStringUtils.stringInArray(listOfObjects, 
                                                 TestConstants.objectGpName));


        // attempt removing from user group
        casPort.manageUserGroups(new ManageUserGroups(TestConstants.userGpName,
                                                      "remove",
                                                      TestConstants.userNick));
        // Ensure it was removed
        UserGroupData userGpData = 
            (UserGroupData)UserGroupDataHandler.retrieveObject(
                                                TestConstants.userGpName);
        ArrayOfString arrayOfUser = userGpData.getUserNames();
        assertTrue(arrayOfUser == null);

        // attempt deleting userGp
        casPort.deleteGroup(new DeleteGroup(TestConstants.userGpName, "user"));
        // Ensure it was deleted
        String userGpList[] = UserGroupDataHandler.list();
        assertTrue(!CasStringUtils.stringInArray(userGpList, 
                                                 TestConstants.userGpName));

        // attempt removing object
        casPort.deleteObject(new DeleteObject(TestConstants.objName, 
                                              TestConstants.nsNick));
        // Ensure it was remove
        listOfObjects = ObjectDataHandler.list();
        assertTrue(!CasStringUtils.stringInArray(listOfObjects, addedObj));

        // attempt removing namespace
        casPort.deleteObjectNamespace(TestConstants.nsNick);
        // Ensure it was remove
        String listOfNS[] = NamespaceDataHandler.list();
        for (int i=0; i<listOfNS.length; i++) { 
            logger.debug("list" + listOfNS[i]);
        }
        assertTrue(!CasStringUtils.stringInArray(listOfNS, 
                                                 TestConstants.nsNick));

        // attempt removing user
        casPort.removeUser(TestConstants.userNick);     
        // Ensure it was remove
        String listOfUsers[] = UserDataHandler.list();
        assertTrue(!CasStringUtils.stringInArray(listOfUsers, 
                                                 TestConstants.userNick));

        // attempt removing trust anchor
        casPort.removeTrustAnchor(TestConstants.trustNick);
        // Ensure it was removed
        String listOfTrustAnchor[] = TrustAnchorDataHandler.list();
        assertTrue(!CasStringUtils.stringInArray(listOfTrustAnchor, 
                                                 TestConstants.trustNick));

    }   
}
