/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.cas.utils;

import java.util.Vector;

import org.globus.cas.types.TrustAnchorData;
import org.globus.cas.types.NamespaceData;
import org.globus.cas.types.ServiceTypeData;
import org.globus.cas.types.UserData;
import org.globus.cas.types.ObjectData;
import org.globus.cas.types.PolicyData;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/** 
 * Utlity class for cleaning up strings before storing and restoring them
 */
public class CasStringUtils {

    static Log logger =
              LogFactory.getLog(CasStringUtils.class.getName());

    // Single quote needs to be escaped 
    static String singleQuote = "'";
    static String escapedQuote = "\\'";

    public static String escapeSingleQuote(String str) {
        logger.debug("escapeSingleQuote called with " + str);
        int index;
        int indexFrom = 0;
        if (str!=null) {
            StringBuffer strBuf = new StringBuffer(str);
            while ((index = str.indexOf(singleQuote, indexFrom)) != -1) {
                strBuf = strBuf.replace(index,index+1,escapedQuote);
                str = strBuf.toString();
                indexFrom = index+2;
                logger.debug("string is " + str + "indexFrom is " + indexFrom);
            }
        }
        return str;
    }

    public static String restoreSingleQuote(String str) {
        logger.debug("restoreSingleQuote called with " + str);
        int index;
        if (str!=null) {
            StringBuffer strBuf = new StringBuffer(str);
            while ((index = str.indexOf(escapedQuote)) != -1) {
                strBuf.delete(index, index+1);
                str = strBuf.toString();
                logger.debug("string is " + str);
            }
        }
        return str;
    }

    public static String[] restoreSingleQuote(String strArray[]) {
        if (strArray != null) {
            for (int i=0; i<strArray.length; i++) {
                logger.debug("restoreSingleQuote called with " + strArray[i]);
                int index;
                if (strArray[i]!=null) {
                    StringBuffer strBuf = new StringBuffer(strArray[i]);
                    while ((index = strArray[i].indexOf(escapedQuote)) != -1) {
                        strBuf.delete(index, index+1);
                        strArray[i] = strBuf.toString();
                        logger.debug("string " + i + " is " + strArray[i]);
                    }
                }
            }
        }
        return strArray;
    }
    
    public static String userToString(UserData userData) {
        String returnString = " User data: \n Name: " + userData.getNickname() 
            + "\n SubjectName: " + userData.getSubjectName() 
            + "\n TrustAnchor " + userData.getTrustAnchorName() 
            + "\n GroupNames: ";
        if (userData.getGroupNames() != null) {
            String gpNames[] = userData.getGroupNames().getStrings();
            if (gpNames != null) {
                StringBuffer buf = new StringBuffer(returnString);
                for (int i=0; i<gpNames.length; i++) {
                    buf.append(" ");
                    buf.append(gpNames[i]);
                }
                returnString = buf.toString();
            }
        }
        return returnString;
    }

    public static String namespaceToString(NamespaceData nsData) {
        return " Namespace data: \n Name: " 
            + nsData.getNickname() + "\n ComparisonAlg: " 
            + nsData.getComparisonAlg() + "\n Basename: "
            + nsData.getBasename();
    }

    public static String trustAnchorToString(TrustAnchorData anchorData) {
        return " TrustAnchor data: \n Name: " 
            + anchorData.getNickname() + "\n AuthMethod: " 
            + anchorData.getAuthMethod() + "\n AuthData: "
            + anchorData.getAuthData();
    }

    public static String objectToString(ObjectData objData) {
        String returnString = " Object data:\n ObjectId: " 
            + objData.getObjectId() + "\n Name: " + objData.getObjectName() 
            + "\n Namespace: " + objData.getObjectNamespace() 
            + "\n GroupNames: ";
        if (objData.getGroupNames() != null) {
            String[] gpNames = objData.getGroupNames().getStrings();
            if (gpNames != null) {
                StringBuffer buf = new StringBuffer(returnString);
                for (int i=0; i<gpNames.length; i++) {
                    buf.append(gpNames[i]);
                }
                returnString = buf.toString();
            }
        }
        return returnString;
    }

    public static String serviceTypeToString(ServiceTypeData serviceData) {
        String returnString = " ServiceType: " + serviceData.getName();
        if (serviceData.getActionNames() != null) {
            String actionNames[] = serviceData.getActionNames().getStrings();
            if (actionNames != null) {
                StringBuffer buf = new StringBuffer(returnString);
                buf.append("\n Action Names: ");
                for (int i=0; i<actionNames.length; i++) {
                    buf.append(actionNames[i]);
                }
                returnString = buf.toString();
            }
        }
        return returnString;
    }

    public static String policyToString(PolicyData policyData) {
        
        return " Policy data: \n PolicyId: " + policyData.getPolicyId() 
            + "\n UserGroupName: " + policyData.getUserGroupName()
            + "\n Object Specification: " + policyData.getObjectSpec() 
            + "\n Object Specification Description: " 
            + policyData.getObjectSpecDesc() + "\n Action Specification: " 
            + policyData.getActionSpec() 
            + "\n Action Specification Description: " 
            + policyData.getActionSpecDesc();
    }

    public static boolean stringInVector(Vector array, String str) {
        if ((str == null) || (array == null)) 
            return false;
        logger.debug("String is " + str + " array length " + array.size());
        str = str.trim();
        for (int i=0; i<array.size(); i++) {
            if ((array.get(i)!=null) && (array.get(i).equals(str))) 
                return true;
        }
        return false;
    }

    public static boolean stringInArray(String[] array, String str) {
        if ((str == null) || (array == null)) 
            return false;
        logger.debug("String is " + str + " array length " + array.length);
        str = str.trim();
        for (int i=0; i<array.length; i++) {
            if ((array[i]!=null) && (array[i].equals(str))) 
                return true;
        }
        return false;
    }
}
