/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.delegation.client;

import org.globus.delegation.DelegationUtil;

import org.apache.axis.message.addressing.EndpointReferenceType;

import org.globus.wsrf.impl.security.authorization.Authorization;
import org.globus.wsrf.impl.security.authorization.HostAuthorization;

import java.security.cert.X509Certificate;

import org.globus.wsrf.utils.AddressingUtils;

import org.globus.gsi.GlobusCredential;

import org.globus.axis.util.Util;

import org.globus.wsrf.impl.security.authentication.Constants;
import org.globus.wsrf.impl.security.descriptor.ClientSecurityDescriptor;

public class BaseClient {

    protected static String proxyFile = null;
    protected static int lifetime = 12 * 60 * 60;
    protected static Boolean fullDelegation = Boolean.TRUE;
    protected static Authorization authzVal = HostAuthorization.getInstance();
    protected static String cmd[];
    protected static String outFileName = null;
    protected static Object authType = Constants.GSI_TRANSPORT;
    protected static Object authVal = Constants.SIGNATURE;

    protected static X509Certificate getCertificateToSign(String factoryUrl)
        throws Exception {

        // Get the public key to delegate on.
        EndpointReferenceType delegEpr =
            AddressingUtils.createEndpointReference(factoryUrl, null);
        X509Certificate[] certsToDelegateOn =
            DelegationUtil.getCertificateChainRP(delegEpr,
                                                 getClientSecDesc());
        return certsToDelegateOn[0];
    }

    protected static GlobusCredential getCredential() throws Exception {

        GlobusCredential credential = null;
        if (proxyFile == null) {
            credential = GlobusCredential.getDefaultCredential();
        } else {
            credential = new GlobusCredential(proxyFile);
        }
        return credential;
    }

    protected static String getValue(int i) {
        if (i + 1 >= cmd.length) {
            System.err.println(cmd[i] + " needs a argument");
            System.exit(-1);
        }
        return cmd[i+1];
    }

    protected static ClientSecurityDescriptor getClientSecDesc() {
        ClientSecurityDescriptor desc = new ClientSecurityDescriptor();
        if (authType.equals(Constants.GSI_SEC_MSG)) {
            desc.setGSISecureMsg((Integer)authVal);
        } else if (authType.equals(Constants.GSI_SEC_CONV)) {
            desc.setGSISecureConv((Integer)authVal);
        } else if (authType.equals(Constants.GSI_TRANSPORT)) {
            desc.setGSITransport((Integer)authVal);
            Util.registerTransport();
        }
        desc.setAuthz(authzVal);
        return desc;
    }
}
