/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.delegation.client;

import org.globus.delegation.DelegationUtil;
import org.globus.delegation.DelegationConstants;

import org.apache.axis.message.addressing.EndpointReferenceType;

import org.globus.wsrf.encoding.ObjectSerializer;

import org.globus.wsrf.impl.security.util.AuthUtil;

import java.security.cert.X509Certificate;

import org.globus.wsrf.utils.AddressingUtils;

import org.globus.gsi.GlobusCredential;

import javax.xml.namespace.QName;

import java.io.FileWriter;

import org.globus.wsrf.impl.security.authentication.Constants;
import org.globus.wsrf.impl.security.descriptor.ClientSecurityDescriptor;

public class Delegate extends BaseClient {

    static String optionString = "Delegate [options] fileName\n"
        + " where options can be\n"
        + " -help/--help Displays usage\n"
        + " -h host, defaults to localhost. \n"
        + " -p port, defaults to 8443 if secure transport is used, else \n"
        + "    defaults to 8080\n"
        + " -c credential as proxy file, if not default credential is used\n"
        + " -l lifetime in seconds, defaults to 12 hours\n"
        + " -d if set to true full delegation is done, any other value \n"
        + "    limiited delegation is done. If option is not specified, \n"
        + "    defaults to full delegation \n"
        + " -m security mechanism. 'msg' for secure message or 'conv' for \n"
        + "    secure conversation and 'trans' for transport. Defaults to\n"
        + "    secure transport.\n"
        + " -n protection type, 'sig' signature and 'enc' encryption, \n"
        + "    defaults to signature\n"
        + " -a authz, defaults to Host Authorization (none, host, self or dn\n"
        + "    for identity authz)\n"
        + " and fileName is  file to write EPR of delegated credential";

    static String host = "127.0.0.1";
    static String port = null;

    public static void main(String[] args) throws Exception {

        if (args.length < 1) {
            System.out.println(optionString);
            System.exit(0);
        }

        cmd = args;
        parseArgs();

        if (port == null) {
            if (authType.equals(Constants.GSI_TRANSPORT)) {
                port = "8443";
            } else {
                port = "8080";
            }
        }
        String protocol = "http";
        if (authType.equals(Constants.GSI_TRANSPORT)) {
            protocol = "https";
        }

        String factoryUrl = protocol + "://" + host + ":" + port +
            DelegationConstants.SERVICE_BASE_PATH +
            DelegationConstants.FACTORY_PATH;
        X509Certificate certToSign = getCertificateToSign(factoryUrl);
        EndpointReferenceType delegatedCredEPR =
            delegate(factoryUrl, certToSign);
        System.out.println("Delegated credential EPR:\n" + delegatedCredEPR);

        if (outFileName != null) {
            FileWriter writer = null;
            try {
                writer = new FileWriter(outFileName);
                QName qName = new QName("", "DelegatedEPR");
                writer.write(ObjectSerializer.toString(delegatedCredEPR,
                                                       qName));
            } finally {
                if (writer != null) {
                    writer.close();
                }
            }
        }
    }

    private static void parseArgs() {

        boolean done = false;
        int i=0;
        for (i=0; i<cmd.length; i++) {

            if (done) {
                System.out.println("Invalid parameter, ignored " + cmd[i]);
                continue;
            } 
            if (cmd[i].equals("-h")) {
                host = getValue(i);
                i++;
            } else if (cmd[i].equals("-p")) {
                port = getValue(i);
                i++;
            } else if (cmd[i].equals("-c")) {
                proxyFile = getValue(i);
                i++;
            } else if (cmd[i].equals("-l")) {
                String lifetimeStr = getValue(i);
                lifetime = Integer.parseInt(lifetimeStr);
                i++;
            } else if (cmd[i].equals("-d")) {
                String delegStr = getValue(i);
                fullDelegation = new Boolean(delegStr);
                i++;
            } else if (cmd[i].equals("-a")) {
                String authzStr = getValue(i);
                authzVal = AuthUtil.getClientAuthorization(authzStr);
                i++;
            } else if (cmd[i].equals("-m")) {
                String secType = getValue(i);
                if (secType.equals("msg")) {
                    authType = Constants.GSI_SEC_MSG;
                } else if (secType.equals("conv")) {
                    authType = Constants.GSI_SEC_CONV;
                } else if (secType.equals("trans")) {
                    authType = Constants.GSI_TRANSPORT;
                } else {
                    System.err.println("Invalid entry for -m: " + secType);
                    System.err.println("-m should be 'msg', 'conv' or "
                                       + "'trans'");
                    System.exit(-1);
                }
                i++;
            } else if (cmd[i].equals("-n")) {
                String prot = getValue(i);
                if (prot.equals("sig")) {
                    authVal = Constants.SIGNATURE;
                } else if (prot.equals("enc")) {
                    authVal = Constants.ENCRYPTION;
                } else {
                    System.err.println("Invalid value for -n: " + prot);
                    System.err.println("-n should be 'sig' or 'enc'");
                    System.exit(-1);
                }
                i++;
            } else if ((cmd[i].equals("-help")) || (cmd[i].equals("--help"))) {
                System.out.println(optionString);
                System.exit(0);
            } else {
                outFileName = cmd[i];
                System.out.println("EPR will be written to: " + outFileName);
                done = true;
            }
        }
    }

    private static EndpointReferenceType delegate(String factoryUrl,
                                                  X509Certificate certToSign)
        throws Exception {
        ClientSecurityDescriptor desc = getClientSecDesc();
        GlobusCredential credential = null;
        if (proxyFile == null) {
            credential = GlobusCredential.getDefaultCredential();
        } else {
            credential = new GlobusCredential(proxyFile);
        }
        return DelegationUtil.delegate(factoryUrl, credential, certToSign,
                                       lifetime, fullDelegation.booleanValue(),
                                       desc);
    }
}
