/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.delegation.client;

import org.globus.delegation.DelegationUtil;
import org.globus.delegation.DelegationConstants;

import org.apache.axis.message.addressing.EndpointReferenceType;

import org.globus.wsrf.encoding.ObjectDeserializer;

import org.globus.wsrf.impl.security.util.AuthUtil;

import org.xml.sax.InputSource;

import java.io.FileInputStream;

import java.security.cert.X509Certificate;

import org.globus.gsi.GlobusCredential;

import org.globus.wsrf.impl.security.authentication.Constants;
import org.globus.wsrf.impl.security.descriptor.ClientSecurityDescriptor;

public class Refresh extends BaseClient {

    static String optionString = 
        " -help/--help prints help message\n"
        + " -c credential as proxy file, if not default credential is used\n"
        + " -l lifetime in seconds, defaults to 12 hours\n"
        + " -d if set to true full delegation is done, any other value \n"
        + "    liminited delegation is done. If option is not specified,\n "
        + "    defaults to full delegation \n"
        + " -a authz, defaults to Host Authorization\n"
        + " -e eprFileName file that contains delegation credential EPR,\n "
        + "    defaults to delegatedCredEPR\n"
        + " -m security mechanism. 'msg' for secure message or 'conv' for \n"
        + "    secure conversation and 'trans' for transport. Defaults to\n"
        + "    secure transport.\n"
        + " -n protection type, 'sig' signature and 'enc' encryption, \n"
        + "    defaults to signature";

    static String eprFileName = "delegatedCredEPR";

    public static void main(String[] args) throws Exception {

        cmd = args;
        parseArgs();
        System.out.println("Arguments: " + lifetime + " " + fullDelegation
                           + " " + eprFileName);


        EndpointReferenceType delegatedCredEPR = getEndpoint();
        
        System.out.println("Delegated epr " + delegatedCredEPR);

        String host = delegatedCredEPR.getAddress().getHost();
        int port = delegatedCredEPR.getAddress().getPort();
        String scheme =  delegatedCredEPR.getAddress().getScheme();
        String factoryUrl = scheme+ "://" + host + ":" + port + 
            DelegationConstants.SERVICE_BASE_PATH + 
            DelegationConstants.FACTORY_PATH;

        X509Certificate certToSign = getCertificateToSign(factoryUrl);

        refresh(certToSign, delegatedCredEPR);

        System.out.println("Refresh done");
    }

    private static EndpointReferenceType getEndpoint() 
        throws Exception {

        EndpointReferenceType epr = null;
        FileInputStream is = null;
        try {
            is = new FileInputStream(eprFileName);
            InputSource in = new InputSource(is);
            epr = (EndpointReferenceType)ObjectDeserializer
                .deserialize(in, EndpointReferenceType.class);
        } finally {
            if (is != null) {
                try { is.close(); } catch (Exception e) {}
            }
        }
        return epr;
    }

    private static void parseArgs() {
        
        for (int i=0; i<cmd.length; i++) {
            
            if ((cmd[i].startsWith("-help")) || 
                (cmd[i].startsWith("--help"))) {
                System.out.println(optionString);
                System.exit(0);
            } else if (cmd[i].equals("-c")) {
                proxyFile = getValue(i);
                i++;
            } else if (cmd[i].equals("-l")) {
                String lifetimeStr = getValue(i);
                lifetime = Integer.parseInt(lifetimeStr);
                i++;
            } else if (cmd[i].equals("-d")) {
                String delegStr = getValue(i);
                fullDelegation = new Boolean(delegStr);
                i++;
            } else if (cmd[i].equals("-e")) {
                eprFileName = getValue(i);
                i++;
            } else if (cmd[i].equals("-a")) {
                String authzStr = getValue(i);
                authzVal = AuthUtil.getClientAuthorization(authzStr);
                i++;
            } else if (cmd[i].equals("-m")) {
                String secType = getValue(i);
                if (secType.equals("msg")) {
                    authType = Constants.GSI_SEC_MSG;
                } else if (secType.equals("conv")) {
                    authType = Constants.GSI_SEC_CONV;
                } else if (secType.equals("trans")) {
                    authType = Constants.GSI_TRANSPORT;
                } else {
                    System.err.println("Invalid entry for -m: " + secType);
                    System.err.println("-m should be 'msg', 'conv' or "
                                       + "'trans'");
                    System.exit(-1);
                }
                i++;
            } else if (cmd[i].equals("-n")) {
                String prot = getValue(i);
                if (prot.equals("sig")) {
                    authVal = Constants.SIGNATURE;
                } else if (prot.equals("enc")) {
                    authVal = Constants.ENCRYPTION;
                } else {
                    System.err.println("Invalid value for -n: " + prot);
                    System.err.println("-n should be 'sig' or 'enc'");
                    System.exit(-1);
                }
                i++;
            } else {
                System.err.println("Invalid option " + cmd[i]);
                System.err.println("Use -help or --help to look at options");
                System.exit(-1);
            }
        }
    }

    private static void refresh(X509Certificate certToSign,
                                EndpointReferenceType delegatedCredEpr) 
        throws Exception {

        ClientSecurityDescriptor desc = getClientSecDesc();        
        GlobusCredential credential = null;
        if (proxyFile == null) {
            credential = GlobusCredential.getDefaultCredential();
        } else {
            credential = new GlobusCredential(proxyFile);
        }

        DelegationUtil.refresh(credential, certToSign, 
                               lifetime, fullDelegation.booleanValue(),
                               desc, delegatedCredEpr );
    }
}
