/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.delegation;

import org.globus.wsrf.test.GridTestCase;

import org.globus.wsrf.impl.security.authentication.Constants;
import org.globus.wsrf.impl.security.authorization.Authorization;
import org.globus.wsrf.impl.security.authorization.SelfAuthorization;
import org.globus.wsrf.impl.security.descriptor.ClientSecurityDescriptor;
import org.globus.wsrf.impl.security.authentication.encryption.EncryptionCredentials;

import org.globus.delegationService.CertType;
import javax.security.auth.Subject;
import org.globus.gsi.GlobusCredential;
import java.security.cert.X509Certificate;

import javax.xml.namespace.QName;

import org.globus.wsrf.utils.AddressingUtils;

import org.apache.axis.message.addressing.EndpointReferenceType;

public class PerformanceTests extends GridTestCase {

    public static int TEST_ITERATIONS = 400;

    static String mechanism = null;
    static boolean protection;
    Authorization clientAuthz = null;
    EncryptionCredentials encryptionCreds = null;
    X509Certificate certificate = null;
    int lifetime = 12 * 60 * 60;
    String serviceAddress = null;
    EndpointReferenceType delegEpr = null;
    static ClientSecurityDescriptor desc = null;

    public static final String FACTORY_SERVICE_PATH = 
        "DelegationFactoryService";

    public static final String SERVICE_PATH = 
        "DelegationService";

    GlobusCredential credential;

    public PerformanceTests(String name) {
        super(name);
    }

    // For now mechanism setting is ignored since Delegation service
    // only support secure message
    public static void setSecurityReq(String securityType, boolean protType) {

        mechanism = securityType;
        protection = protType;
        Integer protectionType = Constants.ENCRYPTION;
        if (protType) {
            protectionType = Constants.SIGNATURE;
        }
        System.out.println("Security machanism: " + mechanism 
                           + " with protection signature: " + protType); 
        desc = new ClientSecurityDescriptor();
        desc.setAuthz(SelfAuthorization.getInstance());
        if (mechanism.equalsIgnoreCase("message")) {
            desc.setGSISecureMsg(protectionType);
        } else if (mechanism.equalsIgnoreCase("conversation")) {
            desc.setGSISecureConv(protectionType);
        } else if (mechanism.equalsIgnoreCase("transport")) {
            desc.setGSITransport(protectionType);
        } 
    }
    
    public void testGetRP() throws Exception {

        long time1;
        long time2;
        long accumulator=0;

        QName certChainRp =
            new QName(DelegationConstants.NS, "CertificateChain");        

        for(int i = 0; i < TEST_ITERATIONS; i++) {
            time1 = System.currentTimeMillis();
            X509Certificate[] certsToDelegateOn = DelegationUtil
                .getCertificateChainRP(delegEpr, certChainRp, CertType.class, 
                                       desc);
            time2 = System.currentTimeMillis();
            accumulator += time2 - time1;
        }
        System.out.println("Get Resource property:  " +
                           String.valueOf(accumulator/TEST_ITERATIONS));
    }
    
    public void testDelegate() throws Exception {

        long time1;
        long time2;
        long accumulator=0;

        for(int i = 0; i < TEST_ITERATIONS; i++) {
            System.out.println("Iteration" + i);
            time1 = System.currentTimeMillis();
            DelegationUtil.delegate(this.serviceAddress, this.credential, 
                                    certificate, false, desc);
            time2 = System.currentTimeMillis();
            accumulator += time2 - time1;
        }
        System.out.println("Delegate:  " +
                           String.valueOf(accumulator/TEST_ITERATIONS));
    }

    protected void setUp() throws Exception {

        super.setUp();

        if ((mechanism.equals(Constants.GSI_SEC_MSG)) && (!protection)) {
            GlobusCredential cred = 
                GlobusCredential.getDefaultCredential();
            X509Certificate array[] = cred.getCertificateChain();
            this.encryptionCreds = new EncryptionCredentials(array);
            Subject subject = new Subject();
            subject.getPublicCredentials().add(encryptionCreds);
            desc.setPeerSubject(subject);
        }

        this.credential = GlobusCredential.getDefaultCredential();

        this.serviceAddress = TEST_CONTAINER.getBaseURL() 
            + FACTORY_SERVICE_PATH;
        this.delegEpr = 
            AddressingUtils.createEndpointReference(this.serviceAddress, null);

        QName certChainRp =
            new QName(DelegationConstants.NS, "CertificateChain");
        //Warmup
        for(int i = 0; i < 5; i++) {
            X509Certificate[] certsToDelegateOn = DelegationUtil
                .getCertificateChainRP(delegEpr, certChainRp, CertType.class, 
                                       desc);
            certificate = certsToDelegateOn[0];
            DelegationUtil.delegate(this.serviceAddress, this.credential, 
                                    certificate, false, desc);
        }
    }
}
