/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.delegation.service;

import junit.framework.Test;
import junit.framework.TestSuite;
import junit.framework.TestResult;

import org.globus.wsrf.test.GridTestCase;
import org.globus.wsrf.test.GSITestContainer;

import org.globus.wsrf.test.TestContainer;

import org.globus.wsrf.impl.security.authentication.Constants;

public class PostPackageTests extends TestSuite {

    protected TestContainer testContainer;

    public PostPackageTests(String name) {
        super(name);
    }

    public void run(TestResult result) {

        DelegationServiceTest.setPostPackage(true);
        String securityType = (String)System.getProperty("securityType");
        String mechanism = Constants.GSI_TRANSPORT;
        Object protection = Constants.SIGNATURE;
        if (securityType != null) { 
            if (securityType.equalsIgnoreCase("message")) {
                mechanism = Constants.GSI_SEC_MSG;
            } else if (securityType.equalsIgnoreCase("conversation")) {
                mechanism = Constants.GSI_SEC_CONV;
            } else if (securityType.equalsIgnoreCase("transport")) {
                mechanism = Constants.GSI_TRANSPORT;
            } 
            
            String protType = (String)System.getProperty("protectionType");
            if ((protType != null) && 
                   (protType.equalsIgnoreCase("signature"))) {
                protection = Constants.SIGNATURE;
            } else if ((protType != null) && 
                       (protType.equalsIgnoreCase("encryption"))) {
                protection = Constants.ENCRYPTION;
            }
        }
        DelegationServiceTest.setParameters(mechanism, protection);
        
        if (Constants.GSI_TRANSPORT.equals(mechanism)) {
            this.testContainer = new GSITestContainer();
        } else {
            this.testContainer = new TestContainer();
        }
         
        try {
            this.testContainer.start();
            GridTestCase.setTestServer(this.testContainer);
            super.run(result);
        } catch (Exception e) {
            e.printStackTrace();
            result.addError(this, e);
        }  finally {
            try {
                this.testContainer.stop();
            } catch(Exception e) {
                e.printStackTrace();
            }
        }
    }

    public static Test suite() throws Exception {
        TestSuite suite = new PostPackageTests("Delegation Service Tests");
        suite.addTestSuite(DelegationServiceTest.class);
        return suite;
    }
}
