/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.delegation.service;

import java.io.File;
import java.io.StringReader;            
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import org.xml.sax.InputSource;

import org.globus.wsrf.utils.FilePersistenceHelper;

import org.globus.util.Util;

import org.globus.wsrf.InvalidResourceKeyException;
import org.globus.wsrf.NoSuchResourceException;
import org.globus.wsrf.ResourceException;
import org.globus.wsrf.PersistenceCallback;
import org.globus.wsrf.ResourceIdentifier;
import org.globus.wsrf.ResourceKey;
import org.globus.wsrf.Resource;

import org.globus.wsrf.Constants;

import org.globus.delegation.DelegationUtil;

import javax.security.auth.Subject;

import org.globus.gsi.GlobusCredential;
import org.globus.gsi.jaas.GlobusPrincipal;

import org.apache.axis.message.addressing.EndpointReferenceType;

import org.globus.wsrf.encoding.ObjectSerializer;
import org.globus.wsrf.encoding.ObjectDeserializer;

import java.rmi.RemoteException;

import javax.xml.namespace.QName;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class TestResource implements Resource, ResourceIdentifier, 
                                     PersistenceCallback {

    static Log logger = 
        LogFactory.getLog(TestResource.class.getName());

    QName qName = new QName("", "DelegatedEPR");
    int callbackCount = 0;
    String resourceId;
    GlobusCredential credential = null;
    EndpointReferenceType delegatedEpr = null;
    String listenerId = null;
    private FilePersistenceHelper persistenceHelper;

    String HOME_LOCATION = Constants.JNDI_SERVICES_BASE_NAME +
        TestService.SERVICE_NAME + Constants.HOME_NAME;

    public TestResource() {
    }

    public TestResource(String resId) {
        this.resourceId = resId;
    }

    public Object getID() {
        return this.resourceId;
    }

    public void initialize(EndpointReferenceType epr, int counter, 
                           ResourceKey key) throws Exception {
        
        this.delegatedEpr = epr;
        // register listener
        TestListener listener = new TestListener(HOME_LOCATION, key);
        if (counter == 2) {
            DelegationUtil.registerDelegationListener(epr, listener, null);
        } else if (counter == 3) {
            Subject testSubject = new Subject();
            testSubject.getPrincipals().add(new GlobusPrincipal("test user"));
            DelegationUtil.registerDelegationListener(epr, listener, 
                                                      testSubject);
        } else {
            DelegationUtil.registerDelegationListener(epr, listener);
        }
        this.listenerId = listener.getId();
        store();
    }

    public void setCredential(GlobusCredential _credential) {
        System.out.println("Called set credential");
        this.credential = _credential;
        callbackCount++;
        try {
            store();
        } catch (ResourceException exp) {
            throw new RuntimeException("Error storing resource " 
                                       + exp.getMessage());
        }
    }

    public GlobusCredential getCredential() {
        return this.credential;
    }

    public synchronized void store() throws ResourceException {

        FileOutputStream fos = null;
        File tmpFile = null;
        try {
            tmpFile = File.createTempFile(
                "testDelegation", ".tmp",
                getPersistenceHelper().getStorageDirectory());
            fos = new FileOutputStream(tmpFile);
            ObjectOutputStream oos = new ObjectOutputStream(fos);
            oos.writeInt(this.callbackCount);
            oos.writeObject(this.listenerId);
            oos.writeObject(ObjectSerializer.toString(this.delegatedEpr, 
                                                      qName));
            this.credential.save(oos);
        } catch (Exception e) {
            tmpFile.delete();
            throw new ResourceException("Error storing resource ", e);
        } finally {
            if (fos != null) {
                try { fos.close();} catch (Exception ee) {}
            }
        }

        // FIXME: set permissions
        // Overwrite old file 
        File file = getKeyAsFile(this.resourceId);
        if (file.exists()) {
            file.delete();
        }

        if (!tmpFile.renameTo(file)) {
            tmpFile.delete();
            throw new ResourceException("Errpr storing resource");
        }

        if (!Util.setFilePermissions(file.getAbsolutePath(), 700)) {
            System.out.println("Check file permissions " + 
                               file.getAbsolutePath());
        }
    }

    public void load(ResourceKey key) throws ResourceException,
                                             NoSuchResourceException,
                                             InvalidResourceKeyException {

        this.resourceId = (String)key.getValue();
        File file = getKeyAsFile(this.resourceId);
        if (!file.exists()) {
            throw new NoSuchResourceException();
        }
       
        FileInputStream fis = null;
        try {
            fis = new FileInputStream(file);
            ObjectInputStream ois = new ObjectInputStream(fis);
            this.callbackCount = ois.readInt();
            this.listenerId = (String)ois.readObject();
            String delStr = (String)ois.readObject();
            System.out.println("Loading delegated EPR");
            InputSource is = new InputSource(new StringReader(delStr));
            this.delegatedEpr = (EndpointReferenceType)ObjectDeserializer
                .deserialize(is, EndpointReferenceType.class);
            this.credential = new GlobusCredential(ois);
            System.out.println("On load " + key + " " + this.callbackCount);
        } catch (Exception e) {
            logger.error("Error loading resource trace ", e);
            throw new ResourceException("Error loading resource", e);
        } finally {
            if (fis != null) {
                try { fis.close(); } catch (Exception ee) {}
            }
        }
    }

    protected synchronized FilePersistenceHelper getPersistenceHelper() {
        if (this.persistenceHelper == null) {
            try {
                this.persistenceHelper =
                    new FilePersistenceHelper(getClass(), ".ser");
            } catch (Exception e) {
                throw new RuntimeException(e.getMessage());
            }
        }
        return this.persistenceHelper;
    }

    private File getKeyAsFile(Object key)
        throws InvalidResourceKeyException {
        if (key instanceof String) {
            return getPersistenceHelper().getKeyAsFile(key);
        } else {
            throw new InvalidResourceKeyException();
        }
    }

    public void remove() throws ResourceException {
        System.out.println("Remove called");
        getPersistenceHelper().remove(this.resourceId);
    }

    public int getCallbackCount() throws RemoteException {
        if (this.callbackCount == 4) {
            System.out.println("Calling remove on " + this.listenerId);
            try {
                DelegationUtil.removeDelegationListener(this.delegatedEpr,
                                                        this.listenerId);
            } catch (Exception exp) {
                throw new RemoteException(exp.getMessage());
            }
        }
        return this.callbackCount;
    }
}

