/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */


#include "globus_i_rendezvous.h"
#include "globus_wsrf_core_tools.h"
#include "ManagedJobService_client.h"
#include "ManagedMultiJobService_client.h"
#include "version.h"
#include "wsnt_TopicExpressionType.h"
#include "globus_notification_consumer.h"
#include "wsrendezvous_RendezvousCompleted.h"
#include "wsrendezvous_RegistrantData.h"
#include "wsrendezvous_StateChangeNotificationMessageWrapperType.h"
#include "globus_service_engine.h"

static
void
rz_l_Multi_rp_callback(
    ManagedMultiJobService_client_handle_t   client_handle,
    void *                              arg,
    globus_result_t                     result,
    const wsrp_GetMultipleResourcePropertiesResponseType * 
                                        rz_rps,
    ManagedMultiJobPortType_GetMultipleResourceProperties_fault_t 
                                        fault_type,
    const xsd_any *                     fault);

static
void
rz_l_Multi_poll_callback(
    ManagedMultiJobService_client_handle_t   client_handle,
    void *                              arg,
    globus_result_t                     result,
    const wsrp_GetMultipleResourcePropertiesResponseType * 
                                        rz_rps,
    ManagedMultiJobPortType_GetMultipleResourceProperties_fault_t 
                                        fault_type,
    const xsd_any *                     fault);

static
void
rz_l_Multi_do_poll_callback(
    void *                              arg);


globus_result_t
globus_rz_multi_register(
    globus_rz_handle_t                  handle,
    const char *                        buffer,
    size_t                              length,
    int *                               rank)
{
    wsrendezvous_registerInputType      registerInput;
    wsrendezvous_registerResponseType * registerResponse;
    ManagedMultiJobPortType_register_fault_t fault_type;
    xsd_any *                           fault;
    ManagedMultiJobService_client_handle_t   client_handle;
    globus_result_t                     result = GLOBUS_SUCCESS;

    result = ManagedMultiJobService_client_init(
        &client_handle,
        (handle->attr ? handle->attr->soap_message_attr : NULL),
        NULL);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusRZErrorRegisterFailed(result);
        goto exit;
    }

    registerInput.data.value = globus_malloc(length);
    registerInput.data.length = length;
    memcpy(registerInput.data.value, buffer, length);
    registerInput.rank = *rank;

    result = ManagedMultiJobPortType_register_epr(
        client_handle,
        handle->epr,
        &registerInput,
        &registerResponse,
        &fault_type,
        &fault);
    if(result == GLOBUS_SUCCESS)
    {
        *rank = registerResponse->rank;
        wsrendezvous_registerResponseType_destroy(registerResponse);
    }
    else
    {
        if(fault)
        {
            xsd_any_destroy(fault);
        }
        result = GlobusRZErrorRegisterFailed(result);
        goto destroy_registerInput_contents;
    }

destroy_registerInput_contents:

    wsrendezvous_registerInputType_destroy_contents(&registerInput);
    ManagedMultiJobService_client_destroy(client_handle);

exit:
    return result;
}

globus_result_t
globus_rz_multi_data_request_begin(
    globus_rz_handle_t                  handle,
    globus_reltime_t *                  poll_freq,
    globus_rz_callback_func_t           callback,
    void *                              args)
{
    wsnt_SubscribeType *                Subscribe;
    wsnt_SubscribeResponseType *        SubscribeResponse;
    globus_result_t                     result = GLOBUS_SUCCESS;
    ManagedMultiJobPortType_Subscribe_fault_t fault_type;
    xsd_any *                           fault = NULL;
    wsrp_GetMultipleResourcePropertiesType rz_rps;
    xsd_QName *                         rz_rp;
    ManagedMultiJobService_client_handle_t   client_handle = NULL;
    int                                 https = GLOBUS_FALSE;

    if(handle->callback_handle)
    {
        result = GlobusRZErrorRequestAlreadyBegun();
        goto exit;
    }

    handle->callback_handle = globus_malloc(sizeof(rz_l_callback_handle_t));
    if(!handle->callback_handle)
    {
        result = GlobusRZErrorOutOfMemory(sizeof(rz_l_callback_handle_t));
        goto exit;
    }
    memset(handle->callback_handle, 0, sizeof(rz_l_callback_handle_t));

    if(((globus_soap_message_auth_method_t)globus_soap_message_attr_get(
           handle->attr->soap_message_attr,
           GLOBUS_SOAP_MESSAGE_AUTHENTICATION_METHOD_KEY)) ==
       GLOBUS_SOAP_MESSAGE_AUTH_SECURE)
    {
        https = GLOBUS_TRUE;
    }

    result = globus_service_engine_init(
        &handle->callback_handle->service_engine,
        (handle->attr ? handle->attr->soap_message_attr : NULL),
        NULL,
        NULL,
        https);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusRZErrorDataRequestFailed(result);
        goto destroy_callback_handle;
    }

    globus_mutex_init(&handle->callback_handle->mutex, NULL);
    globus_cond_init(&handle->callback_handle->cond, NULL);

    handle->callback_handle->callback = callback;
    handle->callback_handle->arg = args;

    result = globus_service_engine_register_start(
        handle->callback_handle->service_engine,
        rz_l_engine_stop_callback,
        handle);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusRZErrorDataRequestFailed(result);
        goto destroy_service_engine;
    }

    result = wsa_EndpointReferenceType_init(
        &handle->callback_handle->consumer_reference);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusRZErrorDataRequestFailed(result);
        goto stop_engine;
    }

    result = globus_notification_create_consumer(
        handle->callback_handle->consumer_reference,
        handle->callback_handle->service_engine,
        rz_l_notify_callback,
        handle->callback_handle);
    if (result != GLOBUS_SUCCESS)
    {
        result = GlobusRZErrorSubscribeFailed(result);
        goto destroy_consumer_reference;
    }

    result = wsnt_SubscribeType_init(&Subscribe);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusRZErrorSubscribeFailed(result);
        goto destroy_consumer;
    }

    result = wsa_EndpointReferenceType_copy_contents(
        &Subscribe->ConsumerReference,
        handle->callback_handle->consumer_reference);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusRZErrorSubscribeFailed(result);
        goto destroy_Subscribe;
    }

    result = xsd_any_init(&Subscribe->TopicExpression.any);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusRZErrorSubscribeFailed(result);
        goto destroy_Subscribe;
    }

    Subscribe->TopicExpression.any->any_info = &xsd_QName_contents_info;
    result = xsd_QName_copy(
        (xsd_QName **) &Subscribe->TopicExpression.any->value,
        &wsrendezvous_RendezvousCompleted_qname);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusRZErrorSubscribeFailed(result);
        goto destroy_Subscribe;
    }

    result = xsd_anyURI_init_cstr(
        &Subscribe->TopicExpression._Dialect,
        globus_libc_strdup(SIMPLE_TOPIC_EXPRESSION));
    if (result != GLOBUS_SUCCESS)
    {
        result = GlobusRZErrorSubscribeFailed(result);
        goto destroy_Subscribe;
    }

    result = ManagedMultiJobService_client_init(
        &client_handle,
        (handle->attr ? handle->attr->soap_message_attr : NULL),
        NULL);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusRZErrorDataRequestFailed(result);
        goto destroy_Subscribe;
    }

    result = ManagedMultiJobPortType_Subscribe_epr(
        client_handle,
        handle->epr,
        Subscribe,
        &SubscribeResponse,
        &fault_type,
        &fault);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusRZErrorSubscribeFailed(result);
        if(fault)
        {
            xsd_any_destroy(fault);
        }
        goto destroy_client_handle;
    }

    result = wsa_EndpointReferenceType_copy(
        &handle->callback_handle->subscription_reference,
        SubscribeResponse->SubscriptionReference);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusRZErrorSubscribeFailed(result);
        goto destroy_SubscribeResponse;
    }

    /* now do non-blocking poll */
    wsrp_GetMultipleResourcePropertiesType_init_contents(&rz_rps);
    rz_rp = xsd_QName_array_push(&rz_rps.ResourceProperty);
    xsd_QName_copy_contents(rz_rp, &wsrendezvous_RendezvousCompleted_qname);
    rz_rp = xsd_QName_array_push(&rz_rps.ResourceProperty);
    xsd_QName_copy_contents(rz_rp, &wsrendezvous_RegistrantData_qname);

    ManagedMultiJobService_client_destroy(
        client_handle);

    result = ManagedMultiJobService_client_init(
        &client_handle,
        (handle->attr ? handle->attr->soap_message_attr : NULL),
        NULL);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusRZErrorDataRequestFailed(result);
        goto destroy_rps;
    }

    result = ManagedMultiJobPortType_GetMultipleResourceProperties_epr_register(
        client_handle,
        handle->epr,
        &rz_rps,
        rz_l_Multi_rp_callback,
        handle);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusRZErrorPollFailed(result);
        goto destroy_rps;
    }

    client_handle = NULL;

    /* now register periodic callback */
    result = globus_callback_register_periodic(
        &handle->callback_handle->poll_handle,
        poll_freq,
        poll_freq,
        rz_l_Multi_do_poll_callback,
        handle);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusRZErrorPollFailed(result);
        goto destroy_rps;
    }

destroy_rps:
    wsrp_GetMultipleResourcePropertiesType_destroy_contents(&rz_rps);

destroy_SubscribeResponse:
    wsnt_SubscribeResponseType_destroy(SubscribeResponse);

destroy_client_handle:
    if(client_handle)
    {
        ManagedMultiJobService_client_destroy(
            client_handle);
    }

destroy_Subscribe:
    wsnt_SubscribeType_destroy(Subscribe);

    if(result == GLOBUS_SUCCESS)
    {
        goto exit;
    }

    if(handle->callback_handle->subscription_reference)
    {
        wsa_EndpointReferenceType_destroy(
            handle->callback_handle->subscription_reference);
    }

destroy_consumer:

    globus_notification_destroy_consumer(
        handle->callback_handle->consumer_reference);

destroy_consumer_reference:

    wsa_EndpointReferenceType_destroy(
        handle->callback_handle->consumer_reference);

stop_engine:
    result = globus_service_engine_register_stop(
        handle->callback_handle->service_engine);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusRZErrorShutdownFailed(result);
        goto exit;
    }

    globus_mutex_lock(&handle->callback_handle->mutex);
    while(!handle->callback_handle->done)
    {
        globus_cond_wait(&handle->callback_handle->cond,
                         &handle->callback_handle->mutex);
    }
    globus_mutex_unlock(&handle->callback_handle->mutex);

destroy_service_engine:
    globus_service_engine_destroy(handle->callback_handle->service_engine);
    globus_cond_destroy(&handle->callback_handle->cond);
    globus_mutex_destroy(&handle->callback_handle->mutex);

destroy_callback_handle:
    globus_free(handle->callback_handle);

exit:
    return result;
}


static
void
rz_l_Multi_poll_callback(
    ManagedMultiJobService_client_handle_t   client_handle,
    void *                              arg,
    globus_result_t                     result,
    const wsrp_GetMultipleResourcePropertiesResponseType *
                                        rz_rps,
    ManagedMultiJobPortType_GetMultipleResourceProperties_fault_t
                                        fault_type,
    const xsd_any *                     fault)
{
    globus_result_t                     local_result = GLOBUS_SUCCESS;
    rz_l_callback_handle_t *            callback_handle =
    (rz_l_callback_handle_t *) arg;

    if(result != GLOBUS_SUCCESS)
    {
        globus_mutex_lock(&callback_handle->mutex);
        callback_handle->result = result;
        callback_handle->callback(result, NULL, 0, callback_handle->arg);
        globus_mutex_unlock(&callback_handle->mutex);
    }

    globus_assert(rz_rps);

    if(rz_rps->any.length < 1)
    {
        local_result = GlobusRZErrorInvalidRPArrayLength(
	    rz_rps->any.length);
        globus_mutex_lock(&callback_handle->mutex);
        callback_handle->result = local_result;
        callback_handle->callback(local_result, NULL, 0, callback_handle->arg);
        globus_mutex_unlock(&callback_handle->mutex);
        goto exit;
    }

    if((rz_rps->any.elements[0].any_info != 
        &wsrendezvous_RendezvousCompleted_info) &&
       (rz_rps->any.elements[0].any_info !=
        &ManagedMultiJobPortType_RendezvousCompleted_rp_info))
    {
	local_result = GlobusRZErrorInvalidRPValue(
	    "Expected RendezvousCompleted as first "
	    "RP value instead of: {%s}%s",
	    rz_rps->any.elements[0].any_info->type);
        globus_mutex_lock(&callback_handle->mutex);
        callback_handle->result = local_result;
        callback_handle->callback(local_result, NULL, 0, callback_handle->arg);
        globus_mutex_unlock(&callback_handle->mutex);
        goto exit;
    }

    if(!*(xsd_boolean *)rz_rps->any.elements[0].value)
    {
        goto exit;
    }
    
    if(rz_rps->any.length < 2)
    {
        local_result = GlobusRZErrorInvalidRPArrayLength(
	    rz_rps->any.length);
        globus_mutex_lock(&callback_handle->mutex);
        callback_handle->result = local_result;
        callback_handle->callback(local_result, NULL, 0, callback_handle->arg);
        globus_mutex_unlock(&callback_handle->mutex);
        goto exit;
    }
    
    if((rz_rps->any.elements[1].any_info !=
        &wsrendezvous_RegistrantData_info) &&
       (rz_rps->any.elements[1].any_info !=
	&ManagedMultiJobPortType_RegistrantData_rp_info))
    {
	local_result = GlobusRZErrorInvalidRPValue(
            "Expected RegistrantData as second "
	    "RP value instead of: {%s}%s",
	    rz_rps->any.elements[1].any_info->type);
        globus_mutex_lock(&callback_handle->mutex);
        callback_handle->result = local_result;
        callback_handle->callback(local_result, NULL, 0, callback_handle->arg);
        globus_mutex_unlock(&callback_handle->mutex);
        goto exit;
    }
    
    globus_mutex_lock(&callback_handle->mutex);

    if(!callback_handle->done)
    {
        callback_handle->callback(
            result,
            ((xsd_base64Binary *)
             rz_rps->any.elements[1].value)->value,
            ((xsd_base64Binary *)
             rz_rps->any.elements[1].value)->length,
            callback_handle->arg);
        callback_handle->done = 1;
    }

    globus_mutex_unlock(&callback_handle->mutex);

exit:

    ManagedMultiJobService_client_destroy(client_handle);
}

static
void
rz_l_Multi_rp_callback(
    ManagedMultiJobService_client_handle_t   client_handle,
    void *                              arg,
    globus_result_t                     result,
    const wsrp_GetMultipleResourcePropertiesResponseType *
                                        rz_rps,
    ManagedMultiJobPortType_GetMultipleResourceProperties_fault_t
                                        fault_type,
    const xsd_any *                     fault)
{
    globus_i_rz_handle_t *              handle = (globus_i_rz_handle_t *) arg;

    globus_mutex_lock(&handle->poll_mutex);
    handle->poll_ready = 1;
    globus_mutex_unlock(&handle->poll_mutex);

    rz_l_Multi_poll_callback(
        client_handle, handle->callback_handle,
        result, rz_rps, fault_type, fault);
}

static
void
rz_l_Multi_do_poll_callback(
    void *                              arg)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    ManagedMultiJobService_client_handle_t   client_handle;
    wsrp_GetMultipleResourcePropertiesType rz_rps;
    xsd_QName *                         rz_rp;
    globus_i_rz_handle_t *              handle = (globus_i_rz_handle_t *) arg;

    globus_mutex_lock(&handle->poll_mutex);
    if(!handle->poll_ready)
    {
        globus_mutex_unlock(&handle->poll_mutex);
        return;
    }
    handle->poll_ready = 0;
    globus_mutex_unlock(&handle->poll_mutex);

    wsrp_GetMultipleResourcePropertiesType_init_contents(&rz_rps);
    rz_rp = xsd_QName_array_push(&rz_rps.ResourceProperty);
    xsd_QName_copy_contents(rz_rp, &wsrendezvous_RendezvousCompleted_qname);
    rz_rp = xsd_QName_array_push(&rz_rps.ResourceProperty);
    xsd_QName_copy_contents(rz_rp, &wsrendezvous_RegistrantData_qname);

    result = ManagedMultiJobService_client_init(
        &client_handle,
        (handle->attr ? handle->attr->soap_message_attr : NULL),
        NULL);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusRZErrorDataRequestFailed(result);
        goto destroy_rps;
    }

    result = ManagedMultiJobPortType_GetMultipleResourceProperties_epr_register(
        client_handle,
        handle->epr,
        &rz_rps,
        rz_l_Multi_rp_callback,
        handle);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusRZErrorPollFailed(result);
        goto destroy_client_handle;
    }

    goto destroy_rps;

destroy_client_handle:
    ManagedMultiJobService_client_destroy(
        client_handle);
destroy_rps:
    wsrp_GetMultipleResourcePropertiesType_destroy_contents(&rz_rps);
    handle->callback_handle->result = result;
}
