/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */

#include "globus_i_globusrun_ws.h"
#include "globus_error_gssapi.h"
#include "version.h"

#define globus_l_args_error(a__)                                            \
{                                                                           \
    fprintf(stderr,                                                         \
        "ERROR: %s\n\nSyntax: %s\nUse -help to display full usage\n",       \
        (a__), usage);                                                      \
    globus_module_deactivate_all();                                         \
    exit(1);                                                                \
}

#define globus_l_args_error_fmt(fmt__, arg__)                               \
{                                                                           \
    fprintf(stderr,                                                         \
        "ERROR: " fmt__ "\n\nSyntax: %s\nUse -help to display full usage\n",\
        (arg__), usage);                                                    \
    globus_module_deactivate_all();                                         \
    exit(1);                                                                \
}

#define globus_l_args_error_fmt2(fmt__, arg1__, arg2__)                     \
{                                                                           \
    fprintf(stderr,                                                         \
        "ERROR: " fmt__ "\n\nSyntax: %s\nUse -help to display full usage\n",\
        (arg1__), (arg2__), usage);                                         \
    globus_module_deactivate_all();                                         \
    exit(1);                                                                \
}

#define globus_l_args_error_fmt3(fmt__, arg1__, arg2__, arg3__)             \
{                                                                           \
    fprintf(stderr,                                                         \
        "ERROR: " fmt__ "\n\nSyntax: %s\nUse -help to display full usage\n",\
        (arg1__), (arg2__), (arg3__), usage);                               \
    globus_module_deactivate_all();                                         \
    exit(1);                                                                \
}

#define globus_l_args_error_fmt4(fmt__, arg1__, arg2__, arg3__, arg4__)     \
{                                                                           \
    fprintf(stderr,                                                         \
        "ERROR: " fmt__ "\n\nSyntax: %s\nUse -help to display full usage\n",\
        (arg1__), (arg2__), (arg3__), (arg4__), usage);                     \
    globus_module_deactivate_all();                                         \
    exit(1);                                                                \
}

#define listname(x)                     x##_aliases
#define defname(x)                      x##_definition
#define funcname(x)                     x##_predicate_test
#define paramsname(x)                   x##_predicate_params

#define namedef(id, alias1, alias2)                                         \
    static char * listname(id)[] = {alias1, alias2, NULL}

#define flagdef(id, alias1, alias2)                                         \
    namedef(id, alias1, alias2);                                            \
    static globus_args_option_descriptor_t defname(id) = {                  \
         id, listname(id), 0, NULL, NULL}

#define oneargdef(id, alias1, alias2)                                       \
    namedef(id, alias1, alias2);                                            \
    static globus_args_valid_predicate_t funcname(id)[] = {NULL};           \
    static void * paramsname(id)[] = {NULL};                                \
    globus_args_option_descriptor_t defname(id) = {(int) id,                \
        (char **) listname(id), 1, funcname(id), (void **) paramsname(id)}

/* argument processing stuff */
enum
{
    arg_submit = 1,
    arg_validate,
    arg_monitor,
    arg_status,
    arg_kill,
    arg_streaming,
    arg_streaming_out,
    arg_streaming_err,
    arg_batch,
    arg_factory,
    arg_factory_type,
    arg_factory_epr,
    arg_job_epr_output,
    arg_submission_id,
    arg_submission_id_file,
    arg_submission_id_output,
    arg_job_delegate,
    arg_staging_delegate,
    arg_terminate,
    arg_job_description,
    arg_job_command,
    arg_job_epr,
    arg_quiet,
    arg_host_authz,
    arg_self_authz,
    arg_subject_authz,
    arg_private,
    arg_timeout,
    arg_no_cleanup,
    arg_debug,
    arg_use_mode_s,
    arg_nodcau,
    arg_print_fault_type,
    arg_allow_ipv6,
    arg_job_credential,
    arg_staging_credential,
    arg_transfer_credential,
    arg_num = arg_transfer_credential
};

flagdef(arg_submit, "-submit", "-submit");
flagdef(arg_validate, "-validate", "-validate");
flagdef(arg_monitor, "-monitor", "-monitor");
flagdef(arg_status, "-status", "-status");
flagdef(arg_kill, "-kill", "-kill");

flagdef(arg_streaming, "-s", "-streaming");
flagdef(arg_batch, "-b", "-batch");
flagdef(arg_job_command, "-c", "-job-command");
flagdef(arg_job_delegate, "-J", "-job-delegate");
flagdef(arg_staging_delegate, "-S", "-staging-delegate");
flagdef(arg_host_authz, "-host", "-host-authz");
flagdef(arg_self_authz, "-self", "-self-authz");
flagdef(arg_private, "-p", "-private");
flagdef(arg_no_cleanup, "-n", "-no-cleanup");
flagdef(arg_debug, "-dbg", "-debug");
flagdef(arg_quiet, "-q", "-quiet");
flagdef(arg_use_mode_s, "-passive", "-passive");
flagdef(arg_nodcau, "-nodcau", "-nodcau");
flagdef(arg_print_fault_type, "-pft", "-print-fault-type");
flagdef(arg_allow_ipv6, "-ipv6", "-allow-ipv6");

oneargdef(arg_factory, "-F", "-factory");
oneargdef(arg_factory_type, "-Ft", "-factory-type");
oneargdef(arg_factory_epr, "-Ff", "-factory-epr-file");
oneargdef(arg_job_description, "-f", "-job-description-file");
oneargdef(arg_job_epr, "-j", "-job-epr-file");
oneargdef(arg_streaming_out, "-so", "-stdout-file");
oneargdef(arg_streaming_err, "-se", "-stderr-file");
oneargdef(arg_job_epr_output, "-o", "-job-epr-output-file");
oneargdef(arg_submission_id, "-I", "-submission-id");
oneargdef(arg_submission_id_file, "-If", "-submission-id-file");
oneargdef(arg_submission_id_output, "-Io", "-submission-id-output-file");
oneargdef(arg_terminate, "-term", "-termination");
oneargdef(arg_subject_authz, "-subject", "-subject-authz");
oneargdef(arg_timeout, "-T", "-http-timeout");
oneargdef(arg_job_credential, "-Jf", "-job-credential-file");
oneargdef(arg_staging_credential, "-Sf", "-staging-credential-file");
oneargdef(arg_transfer_credential, "-Tf", "-transfer-credential-file");

#define setupopt(id) args_options[id-1] = defname(id)
#define globus_l_args_init()                                                \
    setupopt(arg_submit);                                                   \
    setupopt(arg_validate);                                                 \
    setupopt(arg_monitor);                                                  \
    setupopt(arg_status);                                                   \
    setupopt(arg_kill);                                                     \
    setupopt(arg_streaming);                                                \
    setupopt(arg_batch);                                                    \
    setupopt(arg_job_command);                                              \
    setupopt(arg_job_delegate);                                             \
    setupopt(arg_staging_delegate);                                         \
    setupopt(arg_host_authz);                                               \
    setupopt(arg_self_authz);                                               \
    setupopt(arg_private);                                                  \
    setupopt(arg_no_cleanup);                                               \
    setupopt(arg_debug);                                                    \
    setupopt(arg_quiet);                                                    \
    setupopt(arg_factory);                                                  \
    setupopt(arg_factory_type);                                             \
    setupopt(arg_factory_epr);                                              \
    setupopt(arg_job_description);                                          \
    setupopt(arg_job_epr);                                                  \
    setupopt(arg_streaming_out);                                            \
    setupopt(arg_streaming_err);                                            \
    setupopt(arg_job_epr_output);                                           \
    setupopt(arg_submission_id);                                            \
    setupopt(arg_submission_id_file);                                       \
    setupopt(arg_submission_id_output);                                     \
    setupopt(arg_terminate);                                                \
    setupopt(arg_subject_authz);                                            \
    setupopt(arg_timeout);                                                  \
    setupopt(arg_use_mode_s);                                               \
    setupopt(arg_nodcau);                                                   \
    setupopt(arg_print_fault_type);                                         \
    setupopt(arg_allow_ipv6);                                               \
    setupopt(arg_job_credential);                                           \
    setupopt(arg_staging_credential);                                       \
    setupopt(arg_transfer_credential)

#define globus_l_check_not_set(var__, arg__, set__)                         \
{                                                                           \
    if((var__))                                                             \
    {                                                                       \
        globus_l_args_error_fmt4("%s (%s) not compatible with %s (%s)",     \
            listname(arg__)[1], listname(arg__)[0],                         \
            listname(set__)[1], listname(set__)[0]);                        \
    }                                                                       \
}

#define globus_l_get_mode_flag(mode__, str__)                               \
{                                                                           \
    switch((mode__))                                                        \
    {                                                                       \
        case GLOBUS_I_VALIDATE:                                             \
            str__ = "-validate";                                            \
            break;                                                          \
        case GLOBUS_I_SUBMIT:                                               \
            str__ = "-submit";                                              \
            break;                                                          \
        case GLOBUS_I_MONITOR:                                              \
            str__ = "-monitor";                                             \
            break;                                                          \
        case GLOBUS_I_STATUS:                                               \
            str__ = "-status";                                              \
            break;                                                          \
        case GLOBUS_I_KILL:                                                 \
            str__ = "-kill";                                                \
            break;                                                          \
        default:                                                            \
            str__ = NULL;                                                   \
            break;                                                          \
    }                                                                       \
}

#define globus_l_check_mode_not_set(mode__, set__)                          \
{                                                                           \
    char *                              str__;                              \
                                                                            \
    globus_l_get_mode_flag((mode__), str__)                                 \
    if(str__)                                                               \
    {                                                                       \
        usage = oneline_usage;                                              \
        globus_l_args_error_fmt2(                                           \
            "%s not compatible with %s",                                    \
            listname(set__)[1], str__);                                     \
    }                                                                       \
}

#define globus_l_get_auth_mode_flag(mode__, str__)                          \
{                                                                           \
    switch((mode__))                                                        \
    {                                                                       \
        case GLOBUS_I_AUTH_HOST:                                            \
            str__ = "-host-authz (-host)";                                  \
            break;                                                          \
        case GLOBUS_I_AUTH_SELF:                                            \
            str__ = "-self-authz (-self)";                                  \
            break;                                                          \
        case GLOBUS_I_AUTH_SUBJECT:                                         \
            str__ = "-subject-authz (-subject)";                            \
            break;                                                          \
        default:                                                            \
            str__ = NULL;                                                   \
            break;                                                          \
    }                                                                       \
}

#define globus_l_check_auth_mode_not_set(mode__, set__)                     \
{                                                                           \
    char *                              str__;                              \
                                                                            \
    globus_l_get_auth_mode_flag((mode__), str__)                            \
    if(str__)                                                               \
    {                                                                       \
        globus_l_args_error_fmt3(                                           \
            "%s (%s) not compatible with %s",                               \
            listname(set__)[1], listname(set__)[0], str__);                 \
    }                                                                       \
}

#define globus_l_check_mode_req(var__, req__, mode__)                       \
{                                                                           \
    if(!(var__))                                                            \
    {                                                                       \
        char *                          str__;                              \
                                                                            \
        globus_l_get_mode_flag((mode__), str__);                            \
        globus_l_args_error_fmt3("%s (%s) is required with %s",             \
            listname(req__)[1], listname(req__)[0], str__);                 \
    }                                                                       \
}

#define globus_l_check_req(var__, req__, arg__)                             \
{                                                                           \
    if(!(var__))                                                            \
    {                                                                       \
        globus_l_args_error_fmt4("%s (%s) is required with %s (%s)",        \
            listname(req__)[1], listname(req__)[0],                         \
            listname(arg__)[1], listname(arg__)[0]);                        \
    }                                                                       \
}

static globus_args_option_descriptor_t  args_options[arg_num];

static const char * validate_usage =
"globusrun-ws -validate -job-description-file <filename>\n";

static const char * status_usage =
"globusrun-ws -status -job-epr-file <filename>\n"
"   [-host-authz | -self-authz | -subject-authz <subject name>]\n"
"   [-private] [-http-timeout <milliseconds>] [-debug]\n";

static const char * kill_usage =
"globusrun-ws -kill -job-epr-file <filename>\n"
"   [-host-authz | -self-authz | -subject-authz <subject name>]\n"
"   [-private] [-http-timeout <milliseconds>] [-debug]\n";

static const char * monitor_usage =
"globusrun-ws -monitor -job-epr-file <filename>\n"
"   [-quiet] [-no-cleanup]\n"
"   [-streaming [-stdout-file <filename>] [-stderr-file <filename>]]\n"
"   [-host-authz | -self-authz | -subject-authz <subject name>]\n"
"   [-private] [-http-timeout <milliseconds>] [-debug]\n"
"   [-allow-ipv6] [-passive] [-nodcau]\n";

static const char * submit_usage =
"globusrun-ws -submit\n"
"   [-batch] [-quiet] [-no-cleanup]\n"
"   [-streaming [-stdout-file <filename>] [-stderr-file <filename>]]\n"
"   [-host-authz | -self-authz | -subject-authz <subject name>]\n"
"   [-private] [-http-timeout <milliseconds>] [-debug]\n"
"   [-allow-ipv6] [-passive] [-nodcau]\n"
"   [-factory-epr-file <filename> |\n"
"       [-factory <contact>] [-factory-type <type>]]\n"
"   [-submission-id <uuid> | -submission-id-file <filename>]\n"
"   [-submission-id-output-file <filename>] [-job-epr-output-file <filename>]\n"
"   [-job-delegate] [-staging-delegate]\n"
"   [-print-fault-type | -pft ]\n"
"   [-job-credential-file <filename>] [-staging-credential-file <filename>]\n"
"       [-transfer-credential-file <filename>]\n"
"   [-termination +<HH:MM>|<mm/dd/yyyy HH:MM>]\n"
"   -job-description-file <filename> | -job-command [--] <program> [arg ...]\n";

static const char * oneline_usage =
"globusrun-ws -help | -version[s]\n"
"globusrun-ws -usage [-validate | -submit | -monitor | -status | -kill]\n";

static const char * long_usage =
"globusrun-ws -help | -version[s]\n"
"globusrun-ws -usage [-validate | -submit | -monitor | -status | -kill]\n"
"  Use the above to see which of the following are valid with each mode.\n"
"\n"
"Modes of operation: (one should appear first on command line)\n"
" -validate:\n"
"       checks the job description for syntax errors and a subset\n"
"       of semantic errors without making any service requests.\n"
" -submit:\n"
"       submits (or resubmits) a job to a job host in one of\n"
"       three output modes: batch, interactive, or interactive-streaming.\n"
" -monitor:\n"
"       attaches to an existing job in interactive or\n"
"       interactive-streaming output modes.\n"
" -status:\n"
"       reports the current state of the job and exits.\n"
" -kill:\n"
"       requests the immediate cancellation of the job and exits.\n"
"\n"
"Options:\n"
" -F, -factory <contact>:\n"
"       If supplied, this option causes an EPR to be constructed using\n"
"       ad-hoc methods that depend on GT implementation details.\n"
"       For interoperability to other implementations of WS_GRAM,\n"
"       the -factory-epr-file option should be used instead.\n"
"      *format: [protocol://]{hostname|hostaddr}[:port][/service]\n"
"      *default: https://localhost:8443/wsrf/services/ManagedJobFactoryService\n"
" -Ft, -factory-type <type>:\n"
"       in the absence of -factory-epr-file, this specifies a\n"
"       type of scheduler.\n"
"      *default: 'Fork' for single jobs and 'Multi' for multijobs\n"
" -Ff, -factory-epr-file <filename>:\n"
"       causes the EPR for the ManagedJobFactory to be read from the given\n"
"       file. This EPR is used as the service endpoint for submission of the\n"
"       job.\n"
" -f, -job-description-file <filename>:\n"
"       causes the job description to be read from the given file.\n"
"       This description is modified according to the other options and\n"
"       passed in the WS_GRAM submission messages.\n"
" -c, -job-command [--] <program> [arg ...]:\n"
"       takes all remaining globusrun-ws arguments as its arguments;\n"
"       therefore it must appear last among globusrun-ws options. This\n"
"       option causes globusrun-ws to generate a simple job description with\n"
"       the named program and arguments.\n"
" -o, -job-epr-output-file <filename>:\n"
"       the created ManagedJob EPR will be written to the given\n"
"       file following successful submission. The file will not be written\n"
"       if the submission fails.\n"
" -j, -job-epr-file <filename>:\n"
"       causes the EPR for the ManagedJob to be read from the given file.\n"
"       This EPR is used as the endpoint for service requests.\n"
" -s, -streaming:\n"
"       The standard output and standard error files of the job are\n"
"       monitored and data is written to the corresponding output of\n"
"       globusrun-ws. The standard output will contain ONLY job output data,\n"
"       while the standard error may be a mixture of job error output as\n"
"       well as globusrun-ws messages, unless -quiet is specified.\n"
"      *implies: -staging-delegate if job description does not already\n"
"       contain stdout and stderr endpoints\n"
"      *note: use of -streaming (with or without -batch) places a hold state\n"
"       on the job before cleanup.  you must let the job complete\n"
"       or come back to it with -monitor or -kill\n"
" -so, -stdout-file <filename>:\n"
"       append stdout out stream to the specified file instead of to stdout.\n"
" -se, -stderr-file <filename>:\n"
"       append stderr out stream to the specified file instead of to stderr.\n"
" -I, -submission-id <uuid>:\n"
"       causes the job to be (re)submitted using the given uuid in the\n"
"       reliability protocol.\n"
" -If, -submission-id-file <filename>:\n"
"       causes the uuid to be read from the given file. It is an error\n"
"       to use with -submission-id\n"
" -Io, -submission-id-output-file <filename>:\n"
"       the uuid in use is written to the given file, whether this uuid\n"
"       was generated for the user or given by one of the above\n"
"       input options.\n"
" -J, -job-delegate:\n"
"       If supplied AND the job description does not already provide a\n"
"       jobCredential element, globusrun-ws will delegate the\n"
"       client credential to WS_GRAM and introduce the corresponding\n"
"       element to the submission input.\n"
" -S, -staging-delegate:\n"
"       If supplied AND the job description does include staging or\n"
"       cleanup directives AND the job description does not already provide\n"
"       the necessary stagingCredential or transferCredential element(s),\n"
"       globusrun-ws will delegate the client credential to WS_GRAM and RFT,\n"
"       and introduce the corresponding elements to the submission input.\n"
" -Jf, -job-credential-file <filename>:\n"
"       If supplied AND the job description does not already provide a\n"
"       jobCredential element, globusrun-ws will copy the supplied epr into\n"
"       the job description.  This should be an epr returned from the\n"
"       DelegationFactoryService intended for use by the job (or, in the\n"
"       case of a multijob, for authenticating to the subjobs).\n"
"      *note: for multijob descriptions, only the top level jobCredential\n"
"       will be copied into.\n"
" -Sf, -staging-credential-file <filename>:\n"
"       If supplied AND the job description does not already provide a\n"
"       stagingCredential element, globusrun-ws will copy the supplied epr\n"
"       into the job description.  This should be an epr returned from the\n"
"       DelegationFactoryService intended for use with the RFT service\n"
"       associated with the ManagedJobService.\n"
"      *note: this option is ignored for multijobs.\n"
" -Tf, -transfer-credential-file <filename>:\n"
"       If supplied, globusrun-ws will copy the epr into each of the\n"
"       stage in, stage out, and cleanup elements that do not already\n"
"       contain a transferCredential element.  This should be an epr\n"
"       returned from the DelegationFactoryService intended for use by\n"
"       RFT to authenticate with the target gridftp server.\n"
"      *note: this option is ignored for multijobs.\n"
" -b, -batch:\n"
"       enables batch mode. the tool prints the resulting ManagedJob EPR as\n"
"       the sole standard output (unless in quiet mode) and exits.\n"
"      *note: without this, -submit is equivalent to\n"
"       -submit -batch immediately followed by -monitor.\n"
" -q, -quiet:\n"
"       all non-fatal status and protocol-related messages are suppressed.\n"
" -n, -no-cleanup:\n"
"       the default behavior of trapping interrupts (SIGINT) and canceling\n"
"       the job is disabled. Instead, the interrupt simply causes the tool\n"
"       to exit without affecting the ManagedJob resource.\n"
" -host, -host-authz:\n"
"       The GSI 'host authorization' rule is used to verify that the\n"
"       service is using a host credential appropriate for the underlying\n"
"       service address information.\n"
"      *default\n"
" -self, -self-authz:\n"
"       The GSI 'self authorization' rule is used to verify that the\n"
"       service is using a (proxy) credential derived from the same\n"
"       identity as the client's.\n"
" -subject, -subject-authz <subject name>:\n"
"       The service must be using a credential with the exact subject\n"
"       name provided by this option.\n"
" -p, -private:\n"
"       If supplied, privacy-protection is enabled between globusrun-ws\n"
"       and WS_GRAM or GridFTP services. It is a fatal error to select\n"
"       privacy protection if it is not available due to build options or\n"
"       other security settings.\n"
"      *note: currently only works with https endpoints\n"
" -T, -http-timeout <milliseconds>:\n"
"       Set timeout for HTTP socket, in milliseconds, for all Web services\n"
"       interactions.\n"
"      *default: 120000 (2 minutes).\n"
" -term, -termination +<HH:MM>|<mm/dd/yyyy HH:MM>:\n"
"       Set a termination time (+relative to now).\n"
"      *default: +24:00\n"
" -dbg, -debug:\n"
"       Display message and GridFTP debug output on stderr\n"
" -pft, -print-fault-type:\n"
"       When a fault occurs, display a line containing\n"
"       Fault Type: fault-type\n"
"       on stderr\n"
" -ipv6, -allow-ipv6:\n"
"       Allow streaming transfers to use IPV6.\n"
" -passive:\n"
"       Force streaming transfers to use MODE S to allow for passive mode\n"
"       transfers. (Useful if you're behind a firewall, but expensive\n"
"       because there is no connection caching).\n"
" -nodcau:\n"
"       Disable data channel authentication on streaming transfers.\n"
"\n";

/* end argument processing stuff */

globus_result_t
globus_l_input_submission_id(
    const char *                        filename,
    char **                             value)
{
    FILE *                              file;
    char                                buf[50];

    file = fopen(filename, "r");
    if(!file)
    {
        goto error;
    }

    if(fscanf(file, "%49s", buf) != 1)
    {
        goto error;
    }

    fclose(file);

    *value = globus_libc_strdup(buf);

    return GLOBUS_SUCCESS;

error:
    return GlobusLSystemError("reading submission id", errno);
}

void
globus_i_parse_arguments(
    int                                 argc,
    char **                             argv,
    globus_i_info_t *                   info)
{
    int                                 rc;
    char *                              program;
    globus_bool_t                       usage_mode = GLOBUS_FALSE;
    globus_list_t *                     options_found = NULL;
    char *                              tmp;
    globus_args_option_instance_t *     instance;
    globus_list_t *                     list;
    globus_xio_contact_t                contact;
    globus_bool_t                       contact_specified = GLOBUS_FALSE;
    globus_bool_t                       submission_id_specified = GLOBUS_FALSE;
    globus_bool_t                       terminate = GLOBUS_FALSE;
    globus_bool_t                       duration = GLOBUS_FALSE;
    globus_bool_t                       command_line = GLOBUS_FALSE;
    const char *                        usage = oneline_usage;
    globus_bool_t                       timeout_specified = GLOBUS_FALSE;
    
    globus_l_args_init();

    program = strrchr(argv[0], '/');
    program = program ? program + 1 : argv[0];

    if(argc > 1)
    {
        if(argc > 2 && strcmp(argv[1], "-usage") == 0)
        {
            usage_mode = GLOBUS_TRUE;
            argv++;
            argc--;
        }
        else if(strcmp(argv[1], "-validate") == 0)
        {
            usage = validate_usage;
        }
        else if(strcmp(argv[1], "-submit") == 0)
        {
            usage = submit_usage;
        }
        else if(strcmp(argv[1], "-monitor") == 0)
        {
            usage = monitor_usage;
        }
        else if(strcmp(argv[1], "-status") == 0)
        {
            usage = status_usage;
        }
        else if(strcmp(argv[1], "-kill") == 0)
        {
            usage = kill_usage;
        }
    }

    if((rc = globus_args_scan(
        &argc,
        &argv,
        arg_num,
        args_options,
        program,
        &local_version,
        usage,
        long_usage,
        &options_found,
        NULL)) < 0)  /* error on argument line */
    {
        globus_module_deactivate_all();
        exit(rc == GLOBUS_FAILURE ? -4 : 0);
    }

    if(argc > 1)
    {
        info->command_argc = argc - 1;
        info->command_argv = argv + 1;
    }

    for(list = options_found;
        !globus_list_empty(list);
        list = globus_list_rest(list))
    {
        instance = globus_list_first(list);

        switch(instance->id_number)
        {
          case arg_submit:
            globus_l_check_mode_not_set(info->mode, arg_submit);
            info->mode = GLOBUS_I_SUBMIT;
            break;
          case arg_validate:
            globus_l_check_mode_not_set(info->mode, arg_validate);
            info->mode = GLOBUS_I_VALIDATE;
            break;
          case arg_monitor:
            globus_l_check_mode_not_set(info->mode, arg_monitor);
            info->mode = GLOBUS_I_MONITOR;
            break;
          case arg_status:
            globus_l_check_mode_not_set(info->mode, arg_status);
            info->mode = GLOBUS_I_STATUS;
            break;
          case arg_kill:
            globus_l_check_mode_not_set(info->mode, arg_kill);
            info->mode = GLOBUS_I_KILL;
            break;
          case arg_streaming:
            info->streaming = GLOBUS_TRUE;
            break;
          case arg_streaming_out:
            info->stream_out_file = globus_libc_strdup(instance->values[0]);
            break;
          case arg_streaming_err:
            info->stream_err_file = globus_libc_strdup(instance->values[0]);
            break;
          case arg_batch:
            info->batch = GLOBUS_TRUE;
            break;
          case arg_factory:
            globus_l_check_not_set(
                info->factory_epr, arg_factory_epr, arg_factory);
            
            globus_i_test_result("parsing factory",
                globus_xio_contact_parse(&contact, instance->values[0]));
            if(!contact.host && contact.resource)
            {
                /* single strings are interpreted as files */
                contact.host = contact.resource;
                contact.resource = NULL;
                
                if(contact.scheme)
                {
                    globus_free(contact.scheme);
                    contact.scheme = NULL;
                }
            }
            contact_specified = GLOBUS_TRUE;
            break;
          case arg_factory_type:
            globus_l_check_not_set(
                info->factory_epr, arg_factory_epr, arg_factory_type);
            info->factory_type = globus_libc_strdup(instance->values[0]);
            break;
          case arg_factory_epr:
            globus_l_check_not_set(
                info->factory_type, arg_factory_type, arg_factory_epr);
            globus_l_check_not_set(
                contact_specified, arg_factory, arg_factory_epr);
            
            globus_i_test_result("loading factory epr",
                globus_i_input_epr(instance->values[0], &info->factory_epr));
            break;
          case arg_job_epr_output:
            info->job_epr_output_file =
                globus_libc_strdup(instance->values[0]);
            break;
          case arg_submission_id:
            globus_l_check_not_set(submission_id_specified,
                arg_submission_id_file, arg_submission_id);
            if(globus_uuid_import(
                &info->submission_id, instance->values[0]) != GLOBUS_SUCCESS)
            {
                globus_l_args_error_fmt(
                    "%s is not a valid UUID", instance->values[0]);
            }
            submission_id_specified = GLOBUS_TRUE;
            break;
          case arg_submission_id_file:
            globus_l_check_not_set(submission_id_specified,
                arg_submission_id, arg_submission_id_file);
            globus_l_input_submission_id(instance->values[0], &tmp);
            if(globus_uuid_import(&info->submission_id, tmp) != GLOBUS_SUCCESS)
            {
                globus_l_args_error_fmt(
                    "%s is not a valid UUID", instance->values[0]);
            }
            globus_free(tmp);
            submission_id_specified = GLOBUS_TRUE;
            break;
          case arg_submission_id_output:
            info->submission_id_output_file =
                globus_libc_strdup(instance->values[0]);
            break;
          case arg_job_delegate:
            info->delegate_job = GLOBUS_TRUE;
            break;
          case arg_staging_delegate:
            info->delegate_staging = GLOBUS_TRUE;
            break;
          case arg_terminate:
            if(*instance->values[0] == '+')
            {
                globus_strptime(
                    instance->values[0] + 1, "%H:%M", &info->terminate);
                /* will add current time to this below */
                duration = GLOBUS_TRUE;
            }
            else
            {            
                globus_strptime(
                    instance->values[0], "%m/%d/%Y %H:%M", &info->terminate);
                terminate = GLOBUS_TRUE;
            }
            break;
          case arg_job_description:
            globus_l_check_not_set(command_line,
                arg_job_command, arg_job_description);
            info->job_description_file =
                globus_libc_strdup(instance->values[0]);
            break;
          case arg_job_command:
            globus_l_check_not_set(info->job_description_file,
                arg_job_description, arg_job_command);
            command_line = GLOBUS_TRUE;
            break;
          case arg_job_epr:
            globus_i_test_result("loading job epr",
                globus_i_input_epr(instance->values[0], &info->job_epr));
            break;
          case arg_quiet:
            info->quiet = GLOBUS_TRUE;
            break;
          case arg_host_authz:
            globus_l_check_auth_mode_not_set(info->auth_mode, arg_host_authz);
            info->auth_mode = GLOBUS_I_AUTH_HOST;
            break;
          case arg_self_authz:
            globus_l_check_auth_mode_not_set(info->auth_mode, arg_self_authz);
            info->auth_mode = GLOBUS_I_AUTH_SELF;
            break;
          case arg_subject_authz:
            globus_l_check_auth_mode_not_set(
                info->auth_mode, arg_subject_authz);
            info->auth_mode = GLOBUS_I_AUTH_SUBJECT;
            info->subject = globus_libc_strdup(instance->values[0]);
            {
                OM_uint32               major_status;
                OM_uint32               minor_status;
                gss_buffer_desc         name_buffer;
                gss_OID                 name_type = GSS_C_NO_OID;
                
                name_buffer.value = info->subject;
                name_buffer.length = strlen(info->subject);

                if(strchr(info->subject,'@') && !strstr(info->subject,"CN="))
                {
                    name_type = GSS_C_NT_HOSTBASED_SERVICE;
                }
                else if(!strncmp("<anonymous>", info->subject, 11))
                {
                    name_type = GSS_C_NT_ANONYMOUS;
                }

                major_status = gss_import_name(&minor_status,
                    &name_buffer, name_type, &info->identity);
                if(GSS_ERROR(major_status))
                {
                    globus_i_test_result("importing subject name",
                        globus_error_put(
                            globus_error_wrap_gssapi_error(                                      \
                                NULL, major_status, minor_status, 0,
                                __FILE__, "globusrun-ws",__LINE__,
                                "gss_import_name failed.")));
                }
            }
            break;
          case arg_private:
            info->encrypt = GLOBUS_TRUE;
            break;
          case arg_timeout:
            GlobusTimeReltimeSet(info->http_timeout,
                0, atoi(instance->values[0]) * 1000);
            timeout_specified = GLOBUS_TRUE;
            break;
          case arg_no_cleanup:
            info->no_cleanup_on_interrupt = GLOBUS_TRUE;
            break;
          case arg_debug:
            info->debug = GLOBUS_TRUE;
            break;
          case arg_use_mode_s:
            info->use_mode_s = GLOBUS_TRUE;
            break;
          case arg_nodcau:
            info->nodcau = GLOBUS_TRUE;
            break;
          case arg_print_fault_type:
            info->print_fault_type = GLOBUS_TRUE;
            break;
          case arg_allow_ipv6:
            info->allow_ipv6 = GLOBUS_TRUE;
            break;
          case arg_job_credential:
            globus_i_test_result("loading job credential epr",
                globus_i_input_epr(
                    instance->values[0], &info->job_cred_epr));
            break;
          case arg_staging_credential:
            globus_i_test_result("loading staging credential epr",
                globus_i_input_epr(
                    instance->values[0], &info->staging_cred_epr));
            break;
          case arg_transfer_credential:
            globus_i_test_result("loading transfer credential epr",
                globus_i_input_epr(
                    instance->values[0], &info->transfer_cred_epr));
            break;
          default:
            globus_l_args_error_fmt(
                "parse panic, arg id = %d", instance->id_number);
            break;
        }
    }

    globus_args_option_instance_list_free(&options_found);

    if(!info->mode)
    {
        globus_l_args_error(
        "Must specify one of -validate, -submit, -status, -monitor, or -kill");
    }
    
    if(usage_mode)
    {
        switch(info->mode)
        {
          case GLOBUS_I_VALIDATE:
            fputs(validate_usage, stderr);
            break;
          case GLOBUS_I_SUBMIT:
            fputs(submit_usage, stderr);
            break;
          case GLOBUS_I_MONITOR:
            fputs(monitor_usage, stderr);
            break;
          case GLOBUS_I_STATUS:
            fputs(status_usage, stderr);
            break;
          case GLOBUS_I_KILL:
            fputs(kill_usage, stderr);
            break;
          default:
            break;
        }
        
        globus_module_deactivate_all();
        exit(-4);
    }

    if(info->mode == GLOBUS_I_VALIDATE)
    {
        globus_l_check_mode_req(
            info->job_description_file, arg_job_description, info->mode);
    }
    
    if(info->mode == GLOBUS_I_STATUS || info->mode == GLOBUS_I_KILL ||
        info->mode == GLOBUS_I_MONITOR)
    {
        globus_l_check_mode_req(info->job_epr, arg_job_epr, info->mode);
    }
    
    if(info->mode == GLOBUS_I_SUBMIT)
    {
        if(command_line)
        {
            globus_l_check_mode_req((info->command_argc > 0),
                arg_job_command, info->mode);
        }
        else
        {
            globus_l_check_mode_req(info->job_description_file,
                arg_job_description, info->mode);
        }
    }
    
    if(info->stream_out_file)
    {
        globus_l_check_req(info->streaming, arg_streaming, arg_streaming_out);
    }
    if(info->stream_err_file)
    {
        globus_l_check_req(info->streaming, arg_streaming, arg_streaming_err);
    }

    /* setup defaults */
    if(!timeout_specified)
    {
        GlobusTimeReltimeSet(info->http_timeout, 0, 120000000);
    }
    
    if(!info->auth_mode)
    {
        info->auth_mode = GLOBUS_I_AUTH_HOST;
    }
    
    if(info->mode == GLOBUS_I_SUBMIT)
    {
        if(!submission_id_specified &&
            globus_uuid_create(&info->submission_id) != GLOBUS_SUCCESS)
        {
            fprintf(stderr, "globusrun-ws: couldn't generate uuid\n");
            exit(1);
        }

        if(!terminate)
        {
            time_t                      now;

            if(!duration)
            {
                info->terminate.tm_hour = 24;
            }

            time(&now);
            now += info->terminate.tm_hour * 60 * 60;
            now += info->terminate.tm_min * 60;
            globus_libc_gmtime_r(&now, &info->terminate);
        }
        
        if(!contact_specified && !info->factory_epr)
        {
            memset(&contact, 0, sizeof(contact));
            contact_specified = GLOBUS_TRUE;
        }
        
        if(contact_specified)
        {
            if(!contact.scheme)
            {
                contact.scheme = globus_libc_strdup("https");
            }
            if(!contact.host)
            {
                contact.host = globus_libc_strdup("localhost");
            }
            if(!contact.port)
            {
                if(strcmp(contact.scheme, "https") == 0)
                {
                    contact.port = globus_libc_strdup("8443");
                }
                else
                {
                    contact.port = globus_libc_strdup("8080");
                }
            }
            if(!contact.resource)
            {
                contact.resource = globus_libc_strdup(
                    MANAGEDJOBFACTORYSERVICE_BASE_PATH);
            }
            
            globus_i_test_result("constructing factory url",
                globus_xio_contact_info_to_url(&contact, &info->factory));
            globus_xio_contact_destroy(&contact);
        }
    }
}
