#ifndef GLOBUS_I_GLOBUSRUN_WS_H_
#define GLOBUS_I_GLOBUSRUN_WS_H_

#include "globus_common.h"
#include "globus_gsi_cert_utils.h"
#include "globus_gsi_credential.h"
#include "globus_xio.h"
#include "globus_ftp_client.h"
#include "globus_ftp_client_debug_plugin.h"
#include "globus_soap_message.h"
#include "ManagedJobFactoryService_client.h"
#include "ManagedJobService_client.h"
#include "ManagedMultiJobService_client.h"
#include "globus_notification_consumer.h"
#include "globus_wsrf_core_tools.h"
#include "SubscriptionManagerService_client.h"
#include "globus_i_streaming.h"
#include "wsgram_FaultResourcePropertyType.h"

#define GlobusLErrorWrapResult(doing__, result__)                           \
    (globus_error_put(                                                      \
        globus_error_construct_error(                                       \
            NULL,                                                           \
            globus_error_get(result__),                                     \
            0, __FILE__, _globus_func_name, __LINE__,                       \
            "Error %s", (doing__))))

#define GlobusLErrorInvalid(what__, reason__)                               \
    (globus_error_put(                                                      \
        globus_error_construct_error(                                       \
            NULL,                                                           \
            NULL,                                                           \
            0, __FILE__, _globus_func_name, __LINE__,                       \
            "Error: invalid %s. %s", (what__), (reason__))))

#define GlobusLErrorCanceled()                                              \
    (globus_error_put(                                                      \
        globus_error_construct_error(                                       \
            NULL,                                                           \
            NULL,                                                           \
            0, __FILE__, _globus_func_name, __LINE__,                       \
            "Operation was canceled")))

#define GlobusLErrorCanceledObj()                                           \
    (globus_error_construct_error(                                          \
        NULL,                                                               \
        NULL,                                                               \
        0, __FILE__, _globus_func_name, __LINE__,                           \
        "Operation was canceled"))

#define GlobusLErrorMemory(what__)                                          \
    (globus_error_put(                                                      \
        globus_error_construct_error(                                       \
            NULL,                                                           \
            NULL,                                                           \
            0, __FILE__, _globus_func_name, __LINE__,                       \
            "Error: out of memory allocating %s", (what__))))

#define GlobusLErrorWrapFault(doing__, fault__)                             \
    (globus_error_put(                                                      \
        globus_error_construct_error(                                       \
            NULL,                                                           \
            globus_error_convert_wsrf_fault((wsbf_BaseFaultType *)(fault__)),\
            0, __FILE__, _globus_func_name, __LINE__,                       \
            "Error %s", (doing__))))

#define GlobusLSystemError(doing, _errno)                                   \
    globus_error_put(                                                       \
        globus_error_wrap_errno_error(                                      \
            NULL,                                                           \
            (_errno),                                                       \
            0, __FILE__, _globus_func_name, __LINE__,                       \
            "Error %s", (doing)))

#define GLOBUS_I_STATE_UPDATE_POLL 60 /* seconds */

typedef void
(*globus_i_cancel_notify_t)(
    void *                              arg);

/* modes of operation */
typedef enum
{
    GLOBUS_I_VALIDATE = 1,
    GLOBUS_I_SUBMIT,
    GLOBUS_I_MONITOR,
    GLOBUS_I_STATUS,
    GLOBUS_I_KILL
} globus_i_mode_t;

typedef enum
{
    GLOBUS_I_AUTH_HOST = 1, /* default */
    GLOBUS_I_AUTH_SELF,
    GLOBUS_I_AUTH_SUBJECT
} globus_i_auth_mode_t;

typedef struct
{
    /* parsed options */
    globus_i_mode_t                     mode;
    globus_i_auth_mode_t                auth_mode;
    globus_bool_t                       streaming;
    globus_bool_t                       batch;
    globus_bool_t                       delegate_job;
    globus_bool_t                       delegate_staging;
    globus_bool_t                       no_cleanup_on_interrupt;
    globus_bool_t                       quiet;
    globus_bool_t                       debug;
    globus_bool_t                       encrypt;
    globus_bool_t                       use_mode_s;
    globus_bool_t                       nodcau;
    globus_bool_t                       allow_ipv6;
    globus_bool_t                       print_fault_type;
    
    char *                              factory;
    char *                              factory_type;
    char *                              job_description_file;
    char **                             command_argv;
    int                                 command_argc;
    char *                              job_epr_output_file;
    wsa_EndpointReferenceType *         job_cred_epr;
    wsa_EndpointReferenceType *         staging_cred_epr;
    wsa_EndpointReferenceType *         transfer_cred_epr;
    wsa_EndpointReferenceType *         factory_epr;
    wsa_EndpointReferenceType *         job_epr;
    
    globus_uuid_t                       submission_id;
    char *                              submission_id_output_file;
    struct tm                           terminate;
    char *                              subject;
    gss_name_t                          identity;
    
    char *                              stream_out_file;
    char *                              stream_err_file;
    globus_reltime_t                    http_timeout;
    
    globus_handler_chain_t              secmess_handler_chain;
    globus_handler_chain_t              tls_handler_chain;
    globus_handler_chain_t              service_handler_chain;
    globus_handler_chain_t              secmess_service_handler_chain;
    globus_soap_message_attr_t          message_attr;
    
    globus_ftp_client_operationattr_t   ftp_attr;
    
    globus_object_t *                   job_fault;
    int                                 job_exit_code;
} globus_i_info_t;

typedef enum
{
    GLOBUS_I_JOB_STATE_UNKNOWN,
    GLOBUS_I_JOB_STATE_UNSUBMITTED,
    GLOBUS_I_JOB_STATE_STAGEIN,
    GLOBUS_I_JOB_STATE_PENDING,
    GLOBUS_I_JOB_STATE_ACTIVE,
    GLOBUS_I_JOB_STATE_SUSPENDED,
    GLOBUS_I_JOB_STATE_STAGEOUT,
    GLOBUS_I_JOB_STATE_CLEANUP_HOLD,
    GLOBUS_I_JOB_STATE_CLEANUP,
    GLOBUS_I_JOB_STATE_DONE,
    GLOBUS_I_JOB_STATE_FAILED
} globus_i_monitor_job_state_t;

typedef enum
{
    GLOBUS_I_MY_STATE_NONE,
    GLOBUS_I_MY_STATE_QUERYING,
    GLOBUS_I_MY_STATE_QUERYING_NEED_FINISH,
    GLOBUS_I_MY_STATE_STREAMING,
    GLOBUS_I_MY_STATE_STREAMING_FINISHING,
    GLOBUS_I_MY_STATE_STREAMING_FINISHED,
    GLOBUS_I_MY_STATE_REMOVE_HOLD,
    GLOBUS_I_MY_STATE_CLEANUP
} globus_i_monitor_my_state_t;

typedef struct
{
    globus_mutex_t                      lock;
    globus_cond_t                       cond;
    int                                 cb_count;
    int                                 stream_query_count;
    int                                 stream_count;
    globus_i_monitor_job_state_t        job_state;
    globus_i_monitor_my_state_t         my_state;
    globus_i_info_t *                   command_info;
    globus_object_t *                   error;
    globus_bool_t                       use_https;
    
    globus_i_streaming_t                streaming;
    
    wsa_EndpointReferenceType *         job_epr;
    globus_service_engine_t             engine;
    wsnt_SubscribeType                  subscribe;
    wsa_EndpointReferenceType *         subscribe_epr;
    globus_xio_handle_t                 stdout_handle;
    globus_xio_handle_t                 stderr_handle;
    ManagedJobService_client_handle_t   job_handle;
    ManagedMultiJobService_client_handle_t multijob_handle;
    globus_callback_handle_t            poll_handle;
    globus_bool_t                       is_idle;
    globus_bool_t                       update_pending;
    globus_list_t *                     active_queries;
} globus_i_monitor_info_t;

globus_result_t
globus_i_load_and_validate(
    globus_i_info_t *                   info,
    wsgram_CreateManagedJobInputType *  input);

globus_result_t
globus_i_prepare_job(
    globus_i_info_t *                   info,
    wsgram_CreateManagedJobInputType *  input);

globus_result_t
globus_i_delegate(
    globus_i_info_t *                   info,
    wsgram_CreateManagedJobInputType *  input);

globus_result_t
globus_i_delegate_cleanup(
    globus_i_info_t *                   info,
    wsgram_CreateManagedJobInputType *  input);

globus_result_t
globus_i_submit(
    globus_i_info_t *                   info,
    wsnt_SubscribeType *                subscribe,
    wsa_EndpointReferenceType **        subscribe_epr,
    wsgram_CreateManagedJobInputType *  input,
    wsa_EndpointReferenceType **        job_epr);

globus_result_t
globus_i_input_epr(
    const char *                        filename,
    wsa_EndpointReferenceType **        job_epr);
    
globus_result_t
globus_i_output_epr(
    const char *                        filename,
    wsa_EndpointReferenceType *         job_epr);

globus_result_t
globus_i_monitor_init(
    globus_i_monitor_info_t *           monitor,
    globus_i_info_t *                   info);
    
void
globus_i_monitor_destroy(
    globus_i_monitor_info_t *           monitor);

globus_result_t
globus_i_monitor(
    globus_i_monitor_info_t *           monitor);

globus_result_t
globus_i_query_job_description(
    globus_i_info_t *                   info,
    wsgram_CreateManagedJobInputType *  input);

globus_result_t
globus_i_status(
    globus_i_info_t *                   info);

globus_result_t
globus_i_kill(
    globus_i_info_t *                   info);

globus_object_t *
globus_i_gram_fault_to_error(
    wsgram_FaultResourcePropertyType *  gram_fault);
    
void
globus_i_parse_arguments(
    int                                 argc,
    char **                             argv,
    globus_i_info_t *                   info);
    
/* returns true if already canceled */
globus_bool_t
globus_i_set_cancel_notify(
    globus_i_cancel_notify_t            cancel_cb,
    void *                              cancel_arg);

globus_bool_t
globus_i_check_canceled(void);


globus_object_t *
globus_i_error_fault_chain(
    wsbf_BaseFaultType *                fault);

void
globus_i_print_error(
    globus_result_t                     result);

void
globus_i_print_warning(
    globus_result_t                     result);

void
globus_i_test_result(
    const char *                        str,
    globus_result_t                     result);

void
globus_i_test_rc(
    const char *                        str,
    int                                 rc);
    
#endif
