/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */

#include "globus_i_globusrun_ws.h"
#include "wstransfer_DeleteType.h"

static xsd_QName globus_l_factory_type_ResourceID_qname = 
{
    "http://www.globus.org/namespaces/2004/10/gram/job",
    "ResourceID"
};

static
globus_result_t
globus_l_load_job_description(
    const char *                        file,
    wsgram_CreateManagedJobInputType *  input)
{
    globus_result_t                     result;
    globus_soap_message_handle_t        message;
    xsd_QName                           element;
    
    result = globus_soap_message_handle_init_from_file(&message, file);
    if(result != GLOBUS_SUCCESS)
    {
        goto error_init;
    }
    result = globus_soap_message_handle_set_attr(
            message,
            GLOBUS_SOAP_MESSAGE_VERBOSE_ERRORS_KEY,
            NULL,
            NULL,
            (void *) GLOBUS_TRUE);
    if (result != GLOBUS_SUCCESS)
    {
        goto error_set_attr;
    }
    
    result = globus_soap_message_deserialize_element_unknown(
        message, &element);
    if(result != GLOBUS_SUCCESS)
    {
        goto error_deserialize_root;
    }
    
    globus_soap_message_deserialize_push_element(message);
    
    if(strcmp(element.local, "job") == 0)
    {
        wsgram_JobDescriptionType *     jd;
        
        input->choice_value.type = wsgram_CreateManagedJobInputType_job;
        jd = &input->choice_value.value.job;
    
        result = wsgram_JobDescriptionType_deserialize(
            &element, jd, message, 0);
        if(result != GLOBUS_SUCCESS)
        {
            goto error_deserialize;
        }
    }
    else if(strcmp(element.local, "multiJob") == 0)
    {
        wsgram_MultiJobDescriptionType *       mjd;
        
        input->choice_value.type = wsgram_CreateManagedJobInputType_multiJob;
        mjd = &input->choice_value.value.multiJob;
        
        result = wsgram_MultiJobDescriptionType_deserialize(
            &element, mjd, message, 0);
        if(result != GLOBUS_SUCCESS)
        {
            goto error_deserialize;
        }
    }
    else
    {
        result = GlobusLErrorInvalid(
            "RSL file", "Root element must be 'job' or 'multiJob'");
        goto error_deserialize;
    }
    
    xsd_QName_destroy_contents(&element);
    globus_soap_message_handle_destroy(message);

    return GLOBUS_SUCCESS;

error_deserialize:
    xsd_QName_destroy_contents(&element);
error_deserialize_root:
error_set_attr:
    globus_soap_message_handle_destroy(message);
error_init:
    return result;
}

static
globus_result_t
globus_l_create_job_description(
    char **                             argv,
    int                                 argc,
    wsgram_CreateManagedJobInputType *  input)
{
    wsgram_JobDescriptionType *         jd;
    globus_result_t                     result;
    int                                 i;
    
    input->choice_value.type = wsgram_CreateManagedJobInputType_job;
    jd = &input->choice_value.value.job;
    
    result = xsd_string_copy_cstr(&jd->executable, argv[0]);
    if(result != GLOBUS_SUCCESS)
    {
        goto error;
    }
    
    for(i = 1; i < argc; i++)
    {
        xsd_string *                    arg;
        
        arg = xsd_string_array_push(&jd->argument);
        if(!arg)
        {
            result = GlobusLErrorMemory("argument slot");
            goto error;
        }
        result = xsd_string_copy_contents_cstr(arg, argv[i]);
        if(result != GLOBUS_SUCCESS)
        {
            goto error;
        }
    }
    
    return GLOBUS_SUCCESS;
    /* don't need to clean anything up as it will be done when input is
     * destroyed
     */
error:
    return result;
}

static
globus_result_t
globus_l_validate_job_description(
    wsgram_CreateManagedJobInputType *  input)
{
    globus_result_t                     result;
    
    /* only extra validation is to ensure multi-jobs have eprs for all
     * subjobs
     */
    if(input->choice_value.type == wsgram_CreateManagedJobInputType_multiJob)
    {
        wsgram_MultiJobDescriptionType *mjd;
        int                             i;
        
        mjd = &input->choice_value.value.multiJob;
        for(i = 0; i < mjd->job.length; i++)
        {
            wsgram_JobDescriptionType * jd = &mjd->job.elements[i];
            
            if(!jd->factoryEndpoint)
            {
                result = GlobusLErrorInvalid("factoryEndpoint",
                    "factoryEndpoint must exist for all subjobs "
                    " of a multiJob RSL");
                goto error;
            }
        }
    }
    
    return GLOBUS_SUCCESS;
error:
    return result;
}

globus_result_t
globus_i_load_and_validate(
    globus_i_info_t *                   info,
    wsgram_CreateManagedJobInputType *  input)
{
    globus_result_t                     result;
    
    if(info->job_description_file)
    {
        result = globus_l_load_job_description(
            info->job_description_file, input);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusLErrorWrapResult("loading rsl", result);
            goto error;
        }
    }
    else
    {
        result = globus_l_create_job_description(
            info->command_argv, info->command_argc, input);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusLErrorWrapResult(
                "creating job description", result);
            goto error;
        }
    }
    
    result = globus_l_validate_job_description(input);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusLErrorWrapResult("validating rsl", result);
        goto error;
    }
    
    return GLOBUS_SUCCESS;
    
error:
    return result;
}

static
globus_result_t
globus_l_staging_set_subject(
    wstransfer_TransferRequestType *    request,
    const char *                        subject)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    
    if(request)
    {
        if(!request->rftOptions)
        {
            result = wstransfer_RFTOptionsType_init(&request->rftOptions);
            if(result != GLOBUS_SUCCESS)
            {
                goto error;
            }
        }
        
        if(!request->rftOptions->subjectName &&
            !request->rftOptions->destinationSubjectName &&
            !request->rftOptions->sourceSubjectName)
        {
            result = xsd_string_copy_cstr(
                &request->rftOptions->subjectName, subject);
            if(result != GLOBUS_SUCCESS)
            {
                goto error;
            }
        }
    }
    
error:
    return result;
}

globus_result_t
globus_i_prepare_job(
    globus_i_info_t *                   info,
    wsgram_CreateManagedJobInputType *  input)
{
    globus_result_t                     result;
    wsgram_JobDescriptionType *         jd;
    int                                 jobcount;
    wsa_EndpointReferenceType **        pepr;
    wsgram_StateEnumeration **          pholdState;
    globus_bool_t                       ismultijob = GLOBUS_FALSE;
    int                                 i;
    
    if(input->choice_value.type == wsgram_CreateManagedJobInputType_job)
    {
        jd = &input->choice_value.value.job;
        jobcount = 1;
        pepr = &jd->factoryEndpoint;
        pholdState = &jd->holdState;
    }
    else
    {
        wsgram_MultiJobDescriptionType *mjd;
        
        mjd = &input->choice_value.value.multiJob;
        jd = mjd->job.elements;
        jobcount = mjd->job.length;
        pepr = &mjd->factoryEndpoint;
        pholdState = &mjd->holdState;
        ismultijob = GLOBUS_TRUE;
    }
    
    if(!*pepr)
    {
        wsa_EndpointReferenceType *     epr;
        
        if(info->factory_epr)
        {
            result = wsa_EndpointReferenceType_copy(&epr, info->factory_epr);
            if(result != GLOBUS_SUCCESS)
            {
                result = GlobusLErrorWrapResult("copying epr", result);
                goto error;
            }
        }
        else
        {
            xsd_any *                   any_entry;
            
            result = wsa_EndpointReferenceType_init(&epr);
            if(result != GLOBUS_SUCCESS)
            {
                result = GlobusLErrorWrapResult("initializing epr", result);
                goto error;
            }
        
            result = wsa_ReferencePropertiesType_init(
                &epr->ReferenceProperties);
            if(result != GLOBUS_SUCCESS)
            {
                wsa_EndpointReferenceType_destroy(epr);
                result = GlobusLErrorWrapResult("initializing epr", result);
                goto error;
            }
            
            result = xsd_anyURI_copy_contents_cstr(
                &epr->Address.base_value, info->factory);
            if(result != GLOBUS_SUCCESS)
            {
                wsa_EndpointReferenceType_destroy(epr);
                result = GlobusLErrorWrapResult("initializing epr", result);
                goto error;
            }
        
            any_entry = xsd_any_array_push(&epr->ReferenceProperties->any);
            if(!any_entry)
            {
                wsa_EndpointReferenceType_destroy(epr);
                result = GlobusLErrorMemory("ReferenceProperties->any");
                goto error;
            }
            any_entry->any_info = &xsd_string_info;
            result = xsd_QName_copy(
                &any_entry->element, &globus_l_factory_type_ResourceID_qname);
            if(result != GLOBUS_SUCCESS)
            {
                wsa_EndpointReferenceType_destroy(epr);
                result = GlobusLErrorWrapResult(
                    "copying ResourceID qname", result);
                goto error;
            }
            
            result = xsd_string_copy_cstr((xsd_string **) &any_entry->value, 
                info->factory_type ? 
                    info->factory_type : ismultijob ? "Multi" : "Fork");
            if(result != GLOBUS_SUCCESS)
            {
                wsa_EndpointReferenceType_destroy(epr);
                result = GlobusLErrorWrapResult("copying ResourceID", result);
                goto error;
            }
        }
        
        *pepr = epr;
    }
    
    result = xsd_dateTime_copy(
        &input->InitialTerminationTime, &info->terminate);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusLErrorWrapResult("copying termination time", result);
        goto error;
    }
    
    result = wsa_AttributedURI_init(&input->JobID);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusLErrorWrapResult("initializing JobID", result);
        goto error;
    }
    result = xsd_anyURI_init_contents_cstr(
        &input->JobID->base_value,
        globus_common_create_string("uuid:%s", info->submission_id.text));
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusLErrorWrapResult("initializing JobID", result);
        goto error;
    }
    
    if(info->streaming)
    {
        result = wsgram_StateEnumeration_copy(
            pholdState, &wsgram_StateEnumeration_CleanUp);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusLErrorWrapResult("setting holdState", result);
            goto error;
        }
    }
    
    for(i = 0; i < jobcount; i++)
    {
        if(info->streaming)
        {
            wstransfer_DeleteType *     entry;
            char *                      filename;
            
            if(!jd->Stdout)
            {
                filename = globus_common_create_string(
                    "file:///${GLOBUS_USER_HOME}/%s.%d.stdout",
                    info->submission_id.text, i);
                if(!filename)
                {
                    result = GlobusLErrorMemory("stdout filename");
                    goto error;
                }
                
                /* copy non-url portion of filename */
                result = xsd_string_copy_cstr(&jd->Stdout, filename + 7);
                if(result != GLOBUS_SUCCESS)
                {
                    globus_free(filename);
                    result = GlobusLErrorWrapResult(
                        "setting up stdout", result);
                    goto error;
                }
                
                if(!jd->fileCleanUp)
                {
                    result = wstransfer_DeleteRequestType_init(
                        &jd->fileCleanUp);
                    if(result != GLOBUS_SUCCESS)
                    {
                        globus_free(filename);
                        result = GlobusLErrorWrapResult(
                            "setting up stdout cleanup", result);
                        goto error;
                    }
                }
                
                entry = wstransfer_DeleteType_array_push(
                    &jd->fileCleanUp->deletion);
                if(!entry)
                {
                    globus_free(filename);
                    result = GlobusLErrorMemory("cleanup entry");
                    goto error;
                }
                xsd_string_init_contents_cstr(&entry->file, filename);
            }
            if(!jd->Stderr)
            {
                filename = globus_common_create_string(
                    "file:///${GLOBUS_USER_HOME}/%s.%d.stderr",
                    info->submission_id.text, i);
                if(!filename)
                {
                    result = GlobusLErrorMemory("stderr filename");
                    goto error;
                }
                
                /* copy non-url portion of filename */
                result = xsd_string_copy_cstr(&jd->Stderr, filename + 7);
                if(result != GLOBUS_SUCCESS)
                {
                    globus_free(filename);
                    result = GlobusLErrorWrapResult(
                        "setting up stderr", result);
                    goto error;
                }
                
                if(!jd->fileCleanUp)
                {
                    result = wstransfer_DeleteRequestType_init(
                        &jd->fileCleanUp);
                    if(result != GLOBUS_SUCCESS)
                    {
                        globus_free(filename);
                        result = GlobusLErrorWrapResult(
                            "setting up stderr cleanup", result);
                        goto error;
                    }
                }
                
                entry = wstransfer_DeleteType_array_push(
                    &jd->fileCleanUp->deletion);
                if(!entry)
                {
                    globus_free(filename);
                    result = GlobusLErrorMemory("cleanup entry");
                    goto error;
                }
                xsd_string_init_contents_cstr(&entry->file, filename);
            }
            
            info->delegate_staging = GLOBUS_TRUE;
        }
        
        if(info->auth_mode == GLOBUS_I_AUTH_SELF ||
            info->auth_mode == GLOBUS_I_AUTH_SUBJECT)
        {
            result = globus_l_staging_set_subject(
                jd->fileStageIn, info->subject);
            if(result != GLOBUS_SUCCESS)
            {
                result = GlobusLErrorWrapResult(
                    "setting staging subject", result);
                goto error;
            }
            
            result = globus_l_staging_set_subject(
                jd->fileStageOut, info->subject);
            if(result != GLOBUS_SUCCESS)
            {
                result = GlobusLErrorWrapResult(
                    "setting staging subject", result);
                goto error;
            }
            
            if(jd->fileCleanUp)
            {
                if(!jd->fileCleanUp->deleteOptions)
                {
                    result = wstransfer_DeleteOptionsType_init(
                        &jd->fileCleanUp->deleteOptions);
                    if(result != GLOBUS_SUCCESS)
                    {
                        result = GlobusLErrorWrapResult(
                            "initializing deleteOptions", result);
                        goto error;
                    }
                }
                
                if(!jd->fileCleanUp->deleteOptions->subjectName)
                {
                    result = xsd_string_copy_cstr(
                        &jd->fileCleanUp->deleteOptions->subjectName,
                        info->subject);
                    if(result != GLOBUS_SUCCESS)
                    {
                        result = GlobusLErrorWrapResult(
                            "setting staging subject", result);
                        goto error;
                    }
                }
            }
        }
        
        jd++;
    }
    
    /* copy in credential eprs */
    if(ismultijob)
    {
        wsgram_MultiJobDescriptionType *mjd;
        
        mjd = &input->choice_value.value.multiJob;
        
        /* only copy in job cred epr for multijobs and only if one doesnt
         * already exist
         */
        if(info->job_cred_epr && !mjd->jobCredentialEndpoint)
        {
            result = wsa_EndpointReferenceType_copy(
                &mjd->jobCredentialEndpoint, info->job_cred_epr);
            if(result != GLOBUS_SUCCESS)
            {
                result = GlobusLErrorWrapResult("copying cred epr", result);
                goto error;
            }
        }
    }
    else
    {
        jd = &input->choice_value.value.job;
        
        if(info->job_cred_epr && !jd->jobCredentialEndpoint)
        {
            result = wsa_EndpointReferenceType_copy(
                &jd->jobCredentialEndpoint, info->job_cred_epr);
            if(result != GLOBUS_SUCCESS)
            {
                result = GlobusLErrorWrapResult("copying cred epr", result);
                goto error;
            }
        }
        
        if(info->staging_cred_epr && !jd->stagingCredentialEndpoint)
        {
            result = wsa_EndpointReferenceType_copy(
                &jd->stagingCredentialEndpoint, info->staging_cred_epr);
            if(result != GLOBUS_SUCCESS)
            {
                result = GlobusLErrorWrapResult("copying cred epr", result);
                goto error;
            }
        }
        
        if(info->transfer_cred_epr)
        {
            if(jd->fileStageIn &&
                !jd->fileStageIn->transferCredentialEndpoint)
            {
                result = wsa_EndpointReferenceType_copy(
                    &jd->fileStageIn->transferCredentialEndpoint,
                    info->transfer_cred_epr);
                if(result != GLOBUS_SUCCESS)
                {
                    result = GlobusLErrorWrapResult(
                        "copying cred epr", result);
                    goto error;
                }
            }
            
            if(jd->fileStageOut &&
                !jd->fileStageOut->transferCredentialEndpoint)
            {
                result = wsa_EndpointReferenceType_copy(
                    &jd->fileStageOut->transferCredentialEndpoint,
                    info->transfer_cred_epr);
                if(result != GLOBUS_SUCCESS)
                {
                    result = GlobusLErrorWrapResult(
                        "copying cred epr", result);
                    goto error;
                }
            }
            
            if(jd->fileCleanUp &&
                !jd->fileCleanUp->transferCredentialEndpoint)
            {
                result = wsa_EndpointReferenceType_copy(
                    &jd->fileCleanUp->transferCredentialEndpoint,
                    info->transfer_cred_epr);
                if(result != GLOBUS_SUCCESS)
                {
                    result = GlobusLErrorWrapResult(
                        "copying cred epr", result);
                    goto error;
                }
            }
        }
    }
    
    return GLOBUS_SUCCESS;

error:
    return result;
}

static
void
globus_l_cancel_cb(
    void *                              arg)
{
    ManagedJobFactoryService_client_handle_t factory_handle;
    
    factory_handle = (ManagedJobFactoryService_client_handle_t) arg;
    ManagedJobFactoryService_client_operation_cancel(factory_handle);
}

globus_result_t
globus_i_submit(
    globus_i_info_t *                   info,
    wsnt_SubscribeType *                subscribe,
    wsa_EndpointReferenceType **        subscribe_epr,
    wsgram_CreateManagedJobInputType *  input,
    wsa_EndpointReferenceType **        job_epr)
{
    wsgram_CreateManagedJobOutputType * output;
    ManagedJobFactoryPortType_createManagedJob_fault_t fault_type;
    wsa_EndpointReferenceType *         epr;
    xsd_any *                           fault;
    globus_result_t                     result;
    ManagedJobFactoryService_client_handle_t factory_handle;
    globus_bool_t                       tls = GLOBUS_FALSE;

    
    if(input->choice_value.type == wsgram_CreateManagedJobInputType_job)
    {
        epr = input->choice_value.value.job.factoryEndpoint;
    }
    else
    {
        epr = input->choice_value.value.multiJob.factoryEndpoint;
    }
    if (strncmp(epr->Address.base_value, "https:", 6) == 0)
    {
        tls = GLOBUS_TRUE;
    }
    
    result = ManagedJobFactoryService_client_init(
        &factory_handle,
        info->message_attr,
        tls
            ? info->tls_handler_chain
            : info->secmess_handler_chain);

    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusLErrorWrapResult("initializing MJFS client", result);
        goto error_client;
    }
    
    if(globus_i_set_cancel_notify(globus_l_cancel_cb, factory_handle))
    {
        result = GlobusLErrorCanceled();
        goto error_canceled;
    }
    
    if(subscribe)
    {
        result = wsnt_SubscribeType_copy(&input->Subscribe, subscribe);
        if(result != GLOBUS_SUCCESS)
        {
            goto error;
        }
    }
    
    result = ManagedJobFactoryPortType_createManagedJob_epr(
        factory_handle, epr, input, &output, &fault_type, &fault);
    if(result != GLOBUS_SUCCESS)
    {
        if(fault_type != GLOBUS_SUCCESS)
        {
            xsd_any_destroy(fault);
        }
        result = GlobusLErrorWrapResult("submitting job", result);
        goto error;
    }
    
    if(input->InitialTerminationTime)
    {
        xsd_dateTime_copy_contents(
            input->InitialTerminationTime, &output->NewTerminationTime);
    }
    else
    {
        xsd_dateTime_copy(
            &input->InitialTerminationTime, &output->NewTerminationTime);
    }
    
    if(subscribe)
    {
        *subscribe_epr = output->subscriptionEndpoint;
        output->subscriptionEndpoint = NULL;
    }
    
    result = wsa_EndpointReferenceType_copy(
        job_epr, &output->managedJobEndpoint);
    if(result != GLOBUS_SUCCESS)
    {
        /* must be a memory error, but since job was actually started, I dont
         * want to kill it for nothing.  possibly leak a little memory here
         * and let something else fail if it's really memory.
         */
        *job_epr = &output->managedJobEndpoint;
        result = GLOBUS_SUCCESS;
    }
    else
    {
        wsgram_CreateManagedJobOutputType_destroy(output);
    }

error:
    globus_i_set_cancel_notify(NULL, NULL);
error_canceled:
    ManagedJobFactoryService_client_destroy(factory_handle);
error_client:
    return result;
}
