/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */

#include "globus_i_globusrun_ws.h"

globus_result_t
globus_i_input_epr(
    const char *                        filename,
    wsa_EndpointReferenceType **        epr)
{
    globus_result_t                     result;
    globus_soap_message_handle_t        message;
    wsa_EndpointReferenceType *         lepr;
    
    result = wsa_EndpointReferenceType_init(&lepr);
    if(result != GLOBUS_SUCCESS)
    {
        goto error_init;
    }
    
    result = globus_soap_message_handle_init_from_file(&message, filename);
    if(result != GLOBUS_SUCCESS)
    {
        goto error_load;
    }
    
    result = wsa_EndpointReferenceType_deserialize(
        NULL, lepr, message, 0);
    if(result != GLOBUS_SUCCESS)
    {
        goto error_deserialize;
    }
        
    globus_soap_message_handle_destroy(message);
    
    *epr = lepr;
    return GLOBUS_SUCCESS;

error_deserialize:
    globus_soap_message_handle_destroy(message);
error_load:
    wsa_EndpointReferenceType_destroy(lepr);
error_init:
    return result;
}

globus_result_t
globus_i_output_epr(
    const char *                        filename,
    wsa_EndpointReferenceType *         epr)
{
    globus_result_t                     result;
    globus_soap_message_handle_t        message;
    
    result = globus_soap_message_handle_init_to_file(&message, filename, 0);
    if(result != GLOBUS_SUCCESS)
    {
        goto error_init;
    }
    
    result = wsa_EndpointReferenceType_serialize(
        &wsa_EndpointReferenceType_qname, epr, message, 0);
    if(result != GLOBUS_SUCCESS)
    {
        goto error_serialize;
    }
        
    globus_soap_message_handle_destroy(message);

    return GLOBUS_SUCCESS;

error_serialize:
    globus_soap_message_handle_destroy(message);
error_init:
    return result;
}

void
globus_i_test_result(
    const char *                        str,
    globus_result_t                     result)
{
    if(result != GLOBUS_SUCCESS)
    {
        char *                          tmp;
        
        tmp = globus_error_print_friendly(globus_error_peek(result));
        fprintf(stderr, "globusrun-ws: Error while %s: %s", str, tmp);
        globus_free(tmp);
        globus_module_deactivate_all();
        exit(-4);
    }
}

void
globus_i_test_rc(
    const char *                        str,
    int                                 rc)
{
    if(rc != GLOBUS_SUCCESS)
    {
        fprintf(stderr, "globusrun-ws: Error while %s: (%d)\n", str, rc);
        globus_module_deactivate_all();
        exit(-4);
    }
}
