/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */

#include "globus_scheduler_event_generator.h"
#include "globus_gram_protocol_constants.h"
/** @page seg_api_test API Test
 * 
 * Test event portions of the SEG API. Passed as an argument,
 * the name of a file containing SEG messages. These are parsed and passed
 * to the event api. The standard output should be identical to the input file.
 */

int main(int argc, char *argv[])
{
    int rc;
    FILE * testfile;
    char *p;
    char line[256];
    globus_result_t result;

    if (argc < 2)
    {
        fprintf(stderr, "Usage: %s testfile\n", argv[0]);

        return 1;
    }

    
    rc = globus_module_activate(GLOBUS_SCHEDULER_EVENT_GENERATOR_MODULE);

    if (rc != GLOBUS_SUCCESS)
    {
        goto error;
    }

    testfile = fopen(argv[1], "r");
    if (testfile == NULL)
    {
        fprintf(stderr, "error openeing %s\n", argv[1]);
        goto deactivate_error;
    }

    while ((p = fgets(line, sizeof(line)-1, testfile)) != NULL)
    {
        int protocol_msg_type;
        time_t stamp;
        char jobid[80];
        int state;
        int exit_code;

        rc = sscanf(line, "%d;%ld;%[^;];%d;%d\n", 
            &protocol_msg_type,
            &stamp,
            jobid,
            &state,
            &exit_code);
        if (rc != 5)
        {
            goto close_error;
        }

        if (protocol_msg_type != 1)
        {
            fprintf(stderr, "unknown msg type %d\n", protocol_msg_type);

            goto close_error;
        }

        switch(state)
        {
        case GLOBUS_GRAM_PROTOCOL_JOB_STATE_PENDING:
            result = globus_scheduler_event_pending(stamp, jobid);
            if (result != GLOBUS_SUCCESS)
            {
                fprintf(stderr, "Error sending PENDING notification\n");

                goto close_error;
            }
            break;

        case GLOBUS_GRAM_PROTOCOL_JOB_STATE_ACTIVE:
            result = globus_scheduler_event_active(stamp, jobid);
            if (result != GLOBUS_SUCCESS)
            {
                fprintf(stderr, "Error sending ACTIVE notification\n");

                goto close_error;
            }
            break;

        case GLOBUS_GRAM_PROTOCOL_JOB_STATE_DONE:
            result = globus_scheduler_event_done(stamp, jobid, exit_code);
            if (result != GLOBUS_SUCCESS)
            {
                fprintf(stderr, "Error sending DONE notification\n");

                goto close_error;
            }
            break;

        case GLOBUS_GRAM_PROTOCOL_JOB_STATE_FAILED:
            result = globus_scheduler_event_failed(stamp, jobid, exit_code);
            if (result != GLOBUS_SUCCESS)
            {
                fprintf(stderr, "Error sending FAILED notification\n");

                goto close_error;
            }
            break;

        default:
            fprintf(stderr, "unknown job state\n");
            goto close_error;
        }
    }
    fclose(testfile);
    globus_module_deactivate_all();
    return 0;

close_error:
    fclose(testfile);
deactivate_error:
    globus_module_deactivate_all();
error:
    return 1;
}
