/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */

#include "globus_common.h"
#include "globus_gss_assist.h"

static
void
usage(char * cmd)
{
    fprintf(stderr, "%s: [-g GRIDMAP-PATH] [--] command [app-args ...]\n", cmd);
}
/* usage() */

#define DEFAULT_GRIDMAP "/etc/grid-security/grid-mapfile"

int main(int argc, char *argv[])
{
    int                                 i;
    struct passwd                       pwd;
    struct passwd *                     pwd_result;
    globus_byte_t                       pwd_buffer[256];
    char *                              gridmap = DEFAULT_GRIDMAP;
    char *                              globusid;
    int                                 rc;

    rc = globus_module_activate(GLOBUS_COMMON_MODULE);
    if (rc != GLOBUS_SUCCESS)
    {
        fprintf(stderr, "Error initializing Globus components\n");

        goto out;
    }

    rc = globus_module_activate(GLOBUS_GSI_GSS_ASSIST_MODULE);
    if (rc != GLOBUS_SUCCESS)
    {
        fprintf(stderr, "Error initializing Globus components\n");

        goto deactivate_out;
    }

    for (i = 1; i < argc; i++)
    {
        if (!strcmp(argv[i], "--"))
        {
            i++;
            break;
        }
        else if (!strcmp(argv[i], "-g"))
        {
            if (i+1 < argc)
            {
                gridmap = argv[++i];
            }
        }
        else if (argv[i][0] == '-')
        {
            fprintf(stderr, "Unknown option: %s\n", argv[i]);
            usage(argv[0]);
            rc = 1;

            goto deactivate_out;
        }
        else
        {
            break;
        }
    }

    if (i == argc)
    {
        fprintf(stderr, "Missing command to execute.\n");
        usage(argv[0]);
        rc = 2;

        goto deactivate_out;
    }

    if (geteuid() != getuid() || getuid() == 0)
    {
        fprintf(stderr, "%s should not be run as root or setuid\n", argv[0]);

        rc = 6;

        goto deactivate_out;
    }

    rc = globus_libc_getpwuid_r(
            getuid(),
            &pwd,
            pwd_buffer,
            sizeof(pwd_buffer),
            &pwd_result);

    if (rc != GLOBUS_SUCCESS)
    {
        fprintf(stderr, "Cannot determine current user name\n");
        rc = 3;

        goto deactivate_out;
    }

    rc = globus_libc_setenv("GRIDMAP", gridmap, 1);
    if (rc != GLOBUS_SUCCESS)
    {
        fprintf(stderr, "Unable to set GRIDMAPFILE environment\n");
        rc = 4;

        goto deactivate_out;
    }

    rc = globus_gss_assist_map_local_user(pwd.pw_name, &globusid);
    if (rc != GLOBUS_SUCCESS || globusid == NULL)
    {
        fprintf(stderr, "%s is not in the grid mapfile\n", pwd.pw_name);
        rc = 5;

        goto deactivate_out;
    }

    rc = execv(argv[i], &argv[i]);
    if (rc != GLOBUS_SUCCESS)
    {
        fprintf(stderr, "Unable to execute %s\n", argv[i]);
    }

deactivate_out:
    globus_module_deactivate_all();
out:
    return 1;
}
/* main() */
