#!/usr/bin/perl

use strict;
use IO::Handle;

my $file=$ARGV[0];

STDOUT->autoflush(1);

my %timings_hash = (
    'submission' => [0, 0],
    'fetchDelegFactoryEndoints' => [0, 0],
    'fetchDelegCertChainRP' => [0, 0],
    'delegate' => [0, 0],
    'createManagedJob' => [0, 0],
    'subscriptionDestroy' => [0, 0],
    'destroy' => [0, 0],
    'delegatedCredentialDestroy' => [0, 0]);
my %duration_total_hash = (
    'submission' => 0,
    'fetchDelegFactoryEndoints' => 0,
    'fetchDelegCertChainRP' => 0,
    'delegate' => 0,
    'createManagedJob' => 0,
    'subscriptionDestroy' => 0,
    'destroy' => 0,
    'delegatedCredentialDestroy' => 0);
my $timingCount = 0;
my $timingType;
my $timingName;
my $timingValue;

open FILE, "$file" or die "Error: unable to open file ".$file." for reading";
while (<FILE>)
{
    chomp;

    $timingType = $_;
    if ($timingType =~ /startTime/) { $timingType = 0 };
    if ($timingType =~ /endTime/) { $timingType = 1 };

    $timingName = $_;
    $timingName =~ s/^..*<..*Time name=\"//;
    $timingName =~ s/\"..*//;

    $timingValue = $_;
    $timingValue =~ s/^..*<..*Time name=\"..*\">//;
    $timingValue =~ s/<..*//;

    $timings_hash{$timingName}[$timingType] = $timingValue;
#print "Found " . ($timingType==0?"start":"end")
#. " timing for \"$timingName\""
#. ": " . $timingValue . "\n";

    if (/endTime name=\"destroy/)
    {
        my @keys = keys(%duration_total_hash);
        foreach my $key (@keys) {
            $duration_total_hash{$key}
                += (  $timings_hash{$key}[1]
                    - $timings_hash{$key}[0]);
        }

#my @keys = keys(%duration_total_hash);
#foreach my $key (@keys) {
#print "$key duration sub-total: $duration_total_hash{$key}\n";
#}

        $timingCount++;
    }
}
close FILE;

my $total_duration = 0;

my @keys = keys(%duration_total_hash);
foreach my $key (@keys) {
    if ($duration_total_hash{$key} > 0)
    {
        $duration_total_hash{$key} /= $timingCount;
    }
    print "$key duration average: $duration_total_hash{$key}\n";

    $total_duration += $duration_total_hash{$key};
}

print "total duration average: " . $total_duration . "\n";

my $execution_duration = $duration_total_hash{'submission'}
                       - $duration_total_hash{'createManagedJob'}
                       - $duration_total_hash{'delegate'}
                       - $duration_total_hash{'fetchDelegCertChainRP'}
                       - $duration_total_hash{'fetchDelegFactoryEndoints'};

print "execution duration average: " . $execution_duration . "\n";
