/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.exec.service.test;

import java.io.File;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.globus.exec.service.exec.utils.FileMapping;
import org.globus.exec.service.exec.utils.FileMappingException;
import org.globus.util.GlobusURL;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;


public class FileMappingTest extends TestCase {
    private static final String GLOBUS_LOCATION
        = System.getProperty("GLOBUS_LOCATION");
    private static final String SHARE_PATH
        = GLOBUS_LOCATION + File.separator + "test" + File.separator
        + System.getProperty("package.name") + File.separator;
    private static final String SIMPLE_MAP
        = SHARE_PATH + "simple_file_mapping.xml";
    private static final String SPARSE_MAP
        = SHARE_PATH + "sparse_file_mapping.xml";

    private String testUrlStrings[] = {
        "file:///",
        "file:///home/globus/test",
        "file:///etc/group",
        "file:///etc/passwd",
        "file:///mnt/home/globus/test"
    };

    private GlobusURL testUrls[];

    private static Log logger = LogFactory.getLog(
            FileMappingTest.class.getName());

    protected void setUp() throws java.net.MalformedURLException {
        logger.debug("setUp called");

        testUrls = new GlobusURL[testUrlStrings.length];

        for (int i = 0; i < testUrlStrings.length; i++) {
            logger.debug("parsing " + testUrlStrings[i]);
            testUrls[i] = new GlobusURL(testUrlStrings[i]);
        }
    }
    public FileMappingTest(String name) {
        super(name);
    }

    public static void main(String [] args) {
        TestSuite testSuite = null;

        // run all
        if (testSuite == null) {
            testSuite = new TestSuite(FileMappingTest.class);
        }

        junit.textui.TestRunner.run (testSuite);
    }

    public static Test suite() {
        return new TestSuite(FileMappingTest.class);
    }

    public void testSimpleFileMapping() throws Exception {
        FileMapping fileMap = new FileMapping(SIMPLE_MAP, "Fork");
        String resultUrls[] = {
            "gsiftp://localhost:2811/",
            "gsiftp://localhost:2811/home/globus/test",
            "gsiftp://localhost:2811/etc/group",
            "gsiftp://localhost:2811/etc/passwd",
            "gsiftp://localhost:2811/mnt/home/globus/test"
        };

        for (int i = 0; i < testUrlStrings.length; i++) {
            logger.debug("testing sparse mapping for "
                + testUrlStrings[i]);
            String result = fileMap.mapFileName(testUrls[i]);

            assertEquals(result, resultUrls[i]);
        }
    }

    public void testSparseFileMapping() throws Exception {
        FileMapping fileMap = new FileMapping(SPARSE_MAP, "Fork");
        String resultUrls[] = {
            null,
            "gsiftp://localhost:2811/ftp1/home/globus/test",
            "gsiftp://localhost:2811/ftp2/etc/group",
            "gsiftp://localhost:2811/ftp2/etc/passwd",
            null
        };

        for (int i = 0; i < testUrlStrings.length; i++)
        {
            logger.debug("testing sparse mapping for "
                + testUrlStrings[i]);
            String result = null;
            try
            {
                result = fileMap.mapFileName(testUrls[i]);
            }
            catch (FileMappingException fme)
            {
                assertNotNull(fme);
            }

            assertEquals(result, resultUrls[i]);
        }
    }

    public void testMissingScheduler()
        throws                              Exception
    {
        try
        {
            FileMapping fileMap = new FileMapping(SPARSE_MAP, "NonFork");
        }
        catch (FileMappingException fme)
        {
            assertNotNull(fme);
        }
    }
}
