/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.exec.service.test;


import java.net.URL;
import java.io.StringWriter;
import java.io.StringReader;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.FileReader;
import java.io.File;
import java.security.cert.X509Certificate;
import java.util.Calendar;
import java.util.List;
import java.util.LinkedList;
import java.util.Vector;

import javax.xml.namespace.QName;
import javax.xml.soap.SOAPElement;
import javax.xml.rpc.Stub;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.globus.axis.gsi.GSIConstants;
import org.apache.axis.message.MessageElement;
import org.apache.axis.message.Text;
import org.apache.axis.message.addressing.Address;
import org.apache.axis.message.addressing.EndpointReferenceType;
import org.apache.axis.message.addressing.ReferencePropertiesType;
import org.apache.axis.types.URI;

import org.globus.delegation.DelegationConstants;
import org.globus.delegation.DelegationUtil;

import org.globus.exec.generated.StateChangeNotificationMessageType;
import org.globus.exec.generated.StateChangeNotificationMessageWrapperType;
import org.globus.exec.generated.ManagedJobFactoryResourceProperties;
import org.globus.exec.generated.ManagedJobFactoryPortType;
import org.globus.exec.generated.CreateManagedJobInputType;
import org.globus.exec.generated.CreateManagedJobOutputType;
import org.globus.exec.generated.ReleaseInputType;
import org.globus.exec.generated.ManagedExecutableJobResourcePropertiesType;
import org.globus.exec.generated.ManagedJobPortType;
import org.globus.exec.generated.StateEnumeration;
import org.globus.exec.generated.JobDescriptionType;
import org.globus.exec.generated.service.ManagedJobFactoryServiceAddressingLocator;
import org.globus.exec.generated.service.ManagedJobServiceAddressingLocator;
import org.globus.exec.service.factory.ManagedJobFactoryHome;
import org.globus.exec.utils.ManagedJobConstants;
import org.globus.exec.utils.ManagedJobFactoryConstants;
import org.globus.exec.utils.rsl.RSLHelper;
import org.globus.exec.utils.client.ManagedJobFactoryClientHelper;
import org.globus.exec.utils.client.ManagedJobClientHelper;

import org.globus.gsi.GlobusCredential;

import org.globus.wsrf.WSNConstants;
import org.globus.wsrf.WSRFConstants;
import org.globus.wsrf.NotificationConsumerManager;
import org.globus.wsrf.NotifyCallback;
import org.globus.wsrf.encoding.ObjectDeserializer;
import org.globus.wsrf.encoding.ObjectSerializer;
import org.globus.wsrf.utils.XmlUtils;
import org.globus.wsrf.utils.AnyHelper;
import org.globus.wsrf.utils.AddressingUtils;

import org.globus.wsrf.impl.security.authentication.Constants;
import org.globus.wsrf.impl.security.authorization.Authorization;
import org.globus.wsrf.impl.security.authorization.SelfAuthorization;
import org.globus.wsrf.impl.security.authorization.IdentityAuthorization;
import org.globus.wsrf.impl.security.authorization.SelfAuthorization;
import org.globus.wsrf.impl.security.descriptor.GSISecureMsgAuthMethod;
import org.globus.wsrf.impl.security.descriptor.ResourceSecurityDescriptor;

import org.oasis.wsn.TopicExpressionType;
import org.oasis.wsn.GetCurrentMessage;
import org.oasis.wsn.GetCurrentMessageResponse;

import org.oasis.wsrf.faults.BaseFaultType;
import org.oasis.wsrf.lifetime.Destroy;
import org.oasis.wsrf.lifetime.SetTerminationTime;
import org.oasis.wsrf.lifetime.SetTerminationTimeResponse;
import org.oasis.wsrf.properties.DeleteType;
import org.oasis.wsrf.properties.InsertType;
import org.oasis.wsrf.properties.QueryExpressionType;
import org.oasis.wsrf.properties.UpdateType;
import org.oasis.wsrf.properties.GetMultipleResourceProperties_Element;
import org.oasis.wsrf.properties.GetMultipleResourcePropertiesResponse;
import org.oasis.wsrf.properties.GetResourcePropertyResponse;
import org.oasis.wsrf.properties.QueryResourceProperties_Element;
import org.oasis.wsrf.properties.QueryResourcePropertiesResponse;
import org.globus.wsrf.test.GridTestCase;
import org.globus.wsrf.test.GridTestSuite;

import org.w3c.dom.Element;


public class HoldReleaseTest extends JobSubmissionTest {

    private static Log logger =
        LogFactory.getLog(HoldReleaseTest.class.getName());

    public HoldReleaseTest(String name) {
        super(name);
    }

    public static void main(String [] args) {
        GridTestSuite testSuite = null;

        // run all
        if (testSuite == null) {
            testSuite = new GridTestSuite(
                HoldReleaseTest.class.toString());
            testSuite.addTestSuite(HoldReleaseTest.class);
        }

        junit.textui.TestRunner.run (testSuite);
    }

    /* Should fail.
     */
    public void testUnsubmittedHold()
        throws                              Exception
    {
        logger.debug("");

        //Should get Failed
        submitJobWithHold(StateEnumeration.Unsubmitted, 1);
    }

    /*
    public void testStageInHold()
        throws                              Exception
    {
        logger.debug("entering testStageInHold()");
        System.out.println("testStageInHold");

        //Should get StageIn + holding
        submitJobWithHold(StateEnumeration.StageIn, 1);

        assertTrue(wasNotified(StateEnumeration.StageIn));
        assertTrue(getCallback().isHolding());

        //Should get StageIn -> Active -> StageOut -> CleanUp -> Done
        releaseJob(5);

        logger.debug("leaving testStageInHold()");
    }
    */

    public void testPendingHold()
        throws                              Exception
    {
        logger.debug("entering testPendingHold()");
        System.out.println("testPendingHold");

        //Should get StageIn -> Pending + holding
        //submitJobWithHold(StateEnumeration.Pending, 2);

        //Should get Pending + holding
        submitJobWithHold(StateEnumeration.Pending, 1);

        assertTrue(wasNotified(StateEnumeration.Pending));
        assertTrue(getCallback().isHolding());

        //Should get Pending -> Active -> CleanUp -> Done
        releaseJob(4);

        logger.debug("leaving testPendingHold()");
    }

    /* Should fail.
     */
    public void testActiveHold()
        throws                              Exception
    {
        logger.debug("entering testActiveHold()");
        System.out.println("testActiveHold");

        //Should get StageIn -> Failed
        //submitJobWithHold(StateEnumeration.Active, 2);

        //Should get Failed
        submitJobWithHold(StateEnumeration.Active, 1);

        assertTrue(wasNotified(StateEnumeration.Failed));
    }

    /*
    public void testStageOutHold()
        throws                              Exception
    {
        logger.debug("entering testStageOutHold()");
        System.out.println("testStageOutHold");

        //Should get StageIn -> Active -> StageOut + holding
        submitJobWithHold(StateEnumeration.StageOut, 3);

        assertTrue(wasNotified(StateEnumeration.StageOut));
        assertTrue(getCallback().isHolding());

        //Should get StageOut -> CleanUp -> Done
        releaseJob(3);

        logger.debug("leaving testStageOutHold()");
    }
    */

    public void testCleanUpHold()
        throws                              Exception
    {
        logger.debug("entering testCleanUpHold()");
        System.out.println("testCleanUpHold");

        //Should get StageIn -> Active -> StageOut -> CleanUp + holding
        //submitJobWithHold(StateEnumeration.CleanUp, 4);

        //Should get Active -> CleanUp + holding
        submitJobWithHold(StateEnumeration.CleanUp, 2);

        assertTrue(wasNotified(StateEnumeration.CleanUp));
        assertTrue(getCallback().isHolding());

        //Should get CleanUp -> Done
        releaseJob(2);
    }

    /* Should fail.
     */
    public void testDoneHold()
        throws                              Exception
    {
        logger.debug("entering testDoneHold()");
        System.out.println("testDoneHold");

        //Should get StageIn -> Active -> StageOut -> CleanUp -> Failed
        //submitJobWithHold(StateEnumeration.Done, 5);

        //Should get Active -> CleanUp -> Failed
        submitJobWithHold(StateEnumeration.Done, 3);

        assertTrue(wasNotified(StateEnumeration.Failed));
    }

    /* Should fail.
     */
    public void testFailedHold()
        throws                              Exception
    {
        logger.debug("entering testFailedHold()");
        System.out.println("testFailedHold");

        //Should get Failed
        submitJobWithHold(StateEnumeration.Failed, 1);

        assertTrue(wasNotified(StateEnumeration.Failed));
    }

    private void submitJobWithHold(
        StateEnumeration                    holdState,
        int                                 expectedNotificationCount)
        throws                              Exception
    {
        logger.debug("entering submitJobWithHold()");

        //get job description template and add holdState
        JobDescriptionType jobDescription = getDefaultJobDescription();

        jobDescription.setHoldState(holdState);
        //If we need to test Failed, doctor the RSL so the job will fail
        if (holdState.equals(StateEnumeration.Failed)) {
            jobDescription.setExecutable("/BaD/ExEcUtAbLe");
        }

        this.submitJobWithStateSubscription(jobDescription);

        waitForCount(expectedNotificationCount, getWaitTimeMillis());

        /*
        if (   (expectedNotificationCount==1)
            && (expectedNotificationCount != getCallback().getNotifyCount()))
        {
            getCallback().addState(getJobState());
        }
        */

        //assert that the correct nofication progression occurred

    }

    protected int getWaitTimeMillis() {
        return 30000;
    }

    private void releaseJob(
        int                                 expectedNotificationCount)
        throws                              Exception
    {
        logger.debug("entering releaseJob()");

        getCallback().setHolding(false);

        int expectedTotalNotificationCount
            = getCallback().getNotifyCount() + expectedNotificationCount;

        ManagedJobPortType jobPort = this.getSecureJobPort();
        jobPort.release(new ReleaseInputType());

        //wait for hold or Failed notification
        waitForCount(expectedTotalNotificationCount, getWaitTimeMillis());

        //stop listening for state notifications
        //this.consumerManager.stopListening();
        this.unsubscribe();

        //assert that the correct nofication progression occurred
        assertTrue(wasNotified(StateEnumeration.Done));
        assertEquals(expectedTotalNotificationCount,
                     getCallback().getNotifyCount());

        logger.debug("leaving releaseJob()");
    }

}
