/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.exec.service.test;


import java.io.File;
import java.net.URL;
import java.security.cert.X509Certificate;
import java.util.Calendar;
import java.util.LinkedList;
import java.util.List;
import java.util.Vector;

import javax.xml.namespace.QName;
import javax.xml.rpc.Stub;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.apache.axis.components.uuid.UUIDGen;
import org.apache.axis.components.uuid.UUIDGenFactory;
import org.apache.axis.message.MessageElement;
import org.apache.axis.message.addressing.AttributedURI;
import org.apache.axis.message.addressing.EndpointReferenceType;
import org.apache.axis.message.addressing.ReferencePropertiesType;
import org.apache.axis.types.URI;

import org.oasis.wsn.TopicExpressionType;
import org.oasis.wsrf.faults.BaseFaultType;
import org.oasis.wsrf.lifetime.Destroy;
import org.oasis.wsrf.lifetime.SetTerminationTime;
import org.oasis.wsrf.lifetime.SetTerminationTimeResponse;
import org.oasis.wsrf.properties.QueryExpressionType;
import org.oasis.wsrf.properties.GetMultipleResourceProperties_Element;
import org.oasis.wsrf.properties.GetMultipleResourcePropertiesResponse;
import org.oasis.wsrf.properties.GetResourcePropertyResponse;
import org.oasis.wsrf.properties.QueryResourceProperties_Element;
import org.oasis.wsrf.properties.QueryResourcePropertiesResponse;
import org.w3c.dom.Element;

import org.globus.delegation.DelegationConstants;
import org.globus.delegation.DelegationUtil;
import org.globus.exec.generated.CreateManagedJobInputType;
import org.globus.exec.generated.CreateManagedJobOutputType;
import org.globus.exec.generated.JobDescriptionType;
import org.globus.exec.generated.ManagedJobFactoryPortType;
import org.globus.exec.generated.ManagedJobPortType;
import org.globus.exec.generated.MultiJobDescriptionType;
import org.globus.exec.generated.ServiceLevelAgreementType;
import org.globus.exec.generated.StateChangeNotificationMessageType;
import org.globus.exec.generated.StateEnumeration;
import org.globus.exec.utils.ManagedJobConstants;
import org.globus.exec.utils.ManagedJobFactoryConstants;
import org.globus.exec.utils.service.ManagedJobHelper;
import org.globus.exec.utils.client.ManagedJobClientHelper;
import org.globus.exec.utils.client.ManagedJobFactoryClientHelper;
import org.globus.exec.utils.rsl.RSLHelper;
import org.globus.gsi.GlobusCredential;
import org.globus.wsrf.NotificationConsumerManager;
import org.globus.wsrf.NotifyCallback;
import org.globus.wsrf.WSNConstants;
import org.globus.wsrf.WSRFConstants;
import org.globus.wsrf.encoding.ObjectDeserializer;
import org.globus.wsrf.impl.security.authentication.Constants;
import org.globus.wsrf.impl.security.authorization.Authorization;
import org.globus.wsrf.impl.security.authorization.HostAuthorization;
import org.globus.wsrf.impl.security.descriptor.GSISecureMsgAuthMethod;
import org.globus.wsrf.impl.security.descriptor.ResourceSecurityDescriptor;
import org.globus.wsrf.test.GridTestCase;
import org.globus.wsrf.test.GridTestSuite;
import org.globus.wsrf.utils.AddressingUtils;
import org.globus.wsrf.utils.AnyHelper;
import org.globus.wsrf.utils.XmlUtils;


public class ManagedJobTest extends JobSubmissionTest {

    public ManagedJobTest(String name) {
        super(name);
    }

    public static void main(String [] args) {
        GridTestSuite testSuite = null;

        // run all
        if (testSuite == null) {
            testSuite = new GridTestSuite(
                ManagedJobTest.class.toString());
            testSuite.addTestSuite(ManagedJobTest.class);
        }

        junit.textui.TestRunner.run (testSuite);
    }

    /**
     * Create a job then destroy it.
     * @throws Exception
     */
    public void testDestroyJob() throws Exception {
        logger.debug("");
        this.submitJob();
        this.destroyJob();
        assertTrue(getJobEndpoint() == null);
    }

    public void testSimpleJobSubmissionAndMonitoring() throws Exception {
        logger.debug("");
        this.submitJobAndWaitForCompletion(this.getDefaultJobDescription());
    }

    // get multiple resource properties from wsrl:ScheduledTermination
    public void testMultipleResourceProperties_Job_WSRL() throws Exception {
        logger.debug("");
        //first create job with non-null TerminationTime
        submitJob();

        ManagedJobPortType port = getSecureJobPort();


        QName [] propNames =
            new QName[] {WSRFConstants.TERMINATION_TIME,
                         WSRFConstants.CURRENT_TIME};

        GetMultipleResourceProperties_Element requestProp =
            new GetMultipleResourceProperties_Element();
        requestProp.setResourceProperty(propNames);

        GetMultipleResourcePropertiesResponse responseProp =
            port.getMultipleResourceProperties(requestProp);

        MessageElement [] any = responseProp.get_any();
        assertTrue(any != null);
        System.out.println("Resources Properties:\n");
        for (int i=0; i<any.length; i++) {
            Element rpElement = AnyHelper.toElement(any[i]);
            System.out.println(XmlUtils.toString(rpElement));
        }
        assertTrue(any.length > 1);

        Object obj1 = ObjectDeserializer.toObject(any[0], Calendar.class);
        assertTrue(obj1 instanceof Calendar);

        Object obj2 = ObjectDeserializer.toObject(any[1], Calendar.class);
        assertTrue(obj2 instanceof Calendar);
        assertTrue(obj2 != null);
    }


    public void testResourceProperty_State()
        throws Exception
    {
        logger.debug("");
        submitJob();
        //=========== test RPs of job
        ManagedJobPortType jobPortType =
            getSecureJobPort();
        GetResourcePropertyResponse response =
            jobPortType.getResourceProperty(ManagedJobConstants.RP_STATE);
        MessageElement [] any = response.get_any();
        assertTrue(any != null);
        assertTrue(any.length > 0);
        StateEnumeration state = (StateEnumeration) ObjectDeserializer.toObject(
            any[0], StateEnumeration.class);
        assertTrue(state != null);

    }


    public void testResourceProperty_ServiceLevelAgreement()
        throws Exception
    {
        logger.debug("");
        submitJob();
        //=========== test RPs of job
        ManagedJobPortType jobPortType =
            getSecureJobPort();
        GetResourcePropertyResponse response = jobPortType.getResourceProperty(
            ManagedJobConstants.RP_SERVICE_LEVEL_AGREEMENT);
        MessageElement [] any = response.get_any();
        assertTrue(any != null);
        assertTrue(any.length > 0);
        Element rp = any[0].getAsDOM();
        //or AnyHelper.getFirstAsElement(), but seems not as good...
        System.out.println("Returned SLA RP:\n" + XmlUtils.toString(rp));
        //rsl.normalize(); sla.normalize(); //doesn't work as we'd like
        //assertTrue(rsl.equals(sla)); //fails even if same RSL
        //TODO: we need XML deep comparison...only for RSL w/o substitutions
    }

    // get multiple resource properties from gram namespace
    public void testMultipleResourceProperties_Job_GRAM() throws Exception {
        logger.debug("");
        //first create job with non-null TerminationTime
        submitJob();

        ManagedJobPortType port = getSecureJobPort();

        QName [] propNames =
            new QName[] {ManagedJobConstants.RP_STATE,
                         ManagedJobConstants.RP_SERVICE_LEVEL_AGREEMENT,
                         ManagedJobConstants.RP_LOCAL_USER_ID};
                         //ManagedJobConstants.RP_CREDENTIAL_PATH};
        //TODO: test stdOut and stdErr RPs with specific RSL
        //We assume there is no file streaming in this RSL.
        GetMultipleResourceProperties_Element requestProp =
            new GetMultipleResourceProperties_Element();
        requestProp.setResourceProperty(propNames);

        GetMultipleResourcePropertiesResponse responseProp =
            port.getMultipleResourceProperties(requestProp);

        MessageElement [] any = responseProp.get_any();
        assertTrue(any != null);
        System.out.println(
            "[Job Resources Properties:\n");
        for (int i=0; i<any.length; i++) {
            Element rpElement = AnyHelper.toElement(any[i]);
            System.out.println(XmlUtils.toString(rpElement));
        }
        assertTrue(any.length > 2); //assuming none is null - might not be
        //assertTrue(any.length > 3); //assuming none is null - might not be
         //true if no security.

        Object obj0 = ObjectDeserializer.toObject(
            any[0], StateEnumeration.class);
        assertTrue(obj0 != null);

        Object obj1 = ObjectDeserializer.toObject(
            any[1], ServiceLevelAgreementType.class);
        assertNotNull(obj1);
        assertNotNull(((ServiceLevelAgreementType)obj1).getJob());

        Object obj2 = ObjectDeserializer.toObject(any[2], String.class);
        assertNotNull(obj1);


        //Object obj3 = ObjectDeserializer.toObject(any[3], String.class);
        //assertTrue(obj3 instanceof String);

    }



    // query resource property
    public void testQueryResourceProperty_TerminationTime_Job()
        throws Exception
    {
        logger.debug("");
        submitJob();

        ManagedJobPortType port =
            getSecureJobPort();

        // TODO: this can be put in a helper function

        String dialect = "http://www.w3.org/TR/1999/REC-xpath-19991116";

        QueryExpressionType query = new QueryExpressionType();
        query.setDialect(new URI(dialect));
        query.setValue(queryString);

        QueryResourceProperties_Element request
            = new QueryResourceProperties_Element();
        request.setQueryExpression(query);

        QueryResourcePropertiesResponse response =
            port.queryResourceProperties(request);

        MessageElement [] any = response.get_any();
        assertTrue(any != null);
        assertTrue(any.length > 0);

        assertTrue(any[0] instanceof MessageElement);
        logger.debug("termination time MessageElement: " + any[0].toString());
        String value = ((MessageElement)any[0]).getAsString();
        assertTrue(value != null);
        assertEquals("true", value.trim());
    }

    /**
     * Create a job, set termination time of job and compare input
     * with return value (which should be same or later).
     * @throws Exception
     */
    public void testSetTermTime1_Job() throws Exception
    {
        logger.debug("");
        submitJob();

        ManagedJobPortType port = getSecureJobPort();

        Calendar termTime = this.getDefaultTerminationTime();

        SetTerminationTime request = new SetTerminationTime();
        request.setRequestedTerminationTime(termTime);

        SetTerminationTimeResponse response =
            port.setTerminationTime(request);

        Calendar newTermTime = response.getNewTerminationTime();

        logger.debug("requested: " + termTime.getTime());
        logger.debug("scheduled: " + newTermTime.getTime());

        assertTrue(newTermTime.getTime().equals(termTime.getTime()) ||
                   newTermTime.getTime().after(termTime.getTime()));
    }


    /**
     * Create job, set its termination time, compare new return value with
     * value of TerminationTime RP.
     * @throws Exception
     */
    public void testSetTermTime2_Job() throws Exception
    {
        logger.debug("");
        submitJob();

        ManagedJobPortType port = getSecureJobPort();

        Calendar termTime = this.getDefaultTerminationTime();

        SetTerminationTime request = new SetTerminationTime();
        request.setRequestedTerminationTime(termTime);

        SetTerminationTimeResponse response =
            port.setTerminationTime(request);

        Calendar newTermTime = response.getNewTerminationTime();

        logger.debug("requested: " + termTime.getTime());
        logger.debug("scheduled: " + newTermTime.getTime());


        GetResourcePropertyResponse propResponse =
            port.getResourceProperty(WSRFConstants.TERMINATION_TIME);

        MessageElement [] any = propResponse.get_any();
        assertTrue(any != null);
        assertTrue(any.length > 0);

        Object obj = ObjectDeserializer.toObject(any[0], Calendar.class);
        assertTrue(obj instanceof Calendar);
        assertEquals(newTermTime.getTime(), ((Calendar)obj).getTime());
    }


    private static Log logger =
        LogFactory.getLog(ManagedJobTest.class.getName());

    private static String queryString =
        "boolean(/*/*[namespace-uri()='http://docs.oasis-open.org/wsrf/2004/06/wsrf-WS-ResourceLifetime-1.2-draft-01.xsd' and local-name()='TerminationTime'])";


}
