/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.exec.service.test;

import java.io.File;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.axis.types.PositiveInteger;

import org.globus.exec.generated.MultiJobDescriptionType;
import org.globus.exec.generated.JobDescriptionType;
import org.globus.exec.generated.NameValuePairType;
import org.globus.exec.generated.JobTypeEnumeration;
import org.globus.exec.utils.rsl.RSLHelper;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;


public class RSLTest extends TestCase {

    public RSLTest(String name) {
        super(name);
    }

    public static void main(String [] args) {
        TestSuite testSuite = null;

        // run all
        if (testSuite == null) {
            testSuite = new TestSuite(RSLTest.class);
        }

        junit.textui.TestRunner.run (testSuite);
    }

    public static Test suite() {
        return new TestSuite(RSLTest.class);
    }

    public void testSimpleRSL() throws Exception {
        JobDescriptionType jobDesc = readRSLFromFile(SIMPLE_RSL);
        assertTrue(jobDesc != null);
        assertTrue(jobDesc.getExecutable() != null);
        assertTrue(jobDesc.getExecutable().equals("/bin/true"));
    }

    public void testSimpleMultiRSL() throws Exception {
        JobDescriptionType jobDesc = readRSLFromFile(SIMPLE_MULTI_RSL);
        assertTrue(jobDesc != null);
        assertTrue(jobDesc instanceof MultiJobDescriptionType);
        MultiJobDescriptionType jobArray = (MultiJobDescriptionType) jobDesc;
        jobDesc = jobArray.getJob(0);
        assertTrue(jobDesc != null);
        assertTrue(jobDesc.getExecutable() != null);
        assertTrue(jobDesc.getExecutable().equals("/bin/true"));
    }

    public void testTestRSL() throws Exception {
        JobDescriptionType jobDesc = readRSLFromFile(TEST_RSL);
        assertTrue(jobDesc != null);

        //executable
        assertTrue(jobDesc.getExecutable() != null);
        assertTrue(jobDesc.getExecutable().equals("/bin/echo"));

        //directory
        assertTrue(jobDesc.getDirectory() != null);
        assertTrue(jobDesc.getDirectory().equals("${GLOBUS_USER_HOME}"));

        //arguments
        assertTrue(jobDesc.getArgument() != null);
        assertTrue(jobDesc.getArgument(0).equals("12"));
        assertTrue(jobDesc.getArgument(1).equals("abc"));
        assertTrue(jobDesc.getArgument(2).equals("34"));
        assertTrue(jobDesc.getArgument(3).equals(
            "pdscaex_instr_GrADS_grads23_28919.cfg"));
        assertTrue(jobDesc.getArgument(4).equals(
            "pgwynnel was here"));

        //environment
        assertTrue(jobDesc.getEnvironment() != null);
        NameValuePairType binding1 = jobDesc.getEnvironment(0);
        assertTrue(binding1 != null);
        assertTrue(binding1.getName() != null);
        assertTrue(binding1.getName().equals("PI"));
        assertTrue(binding1.getValue() != null);
        assertTrue(binding1.getValue().equals("3.141"));
        NameValuePairType binding2 = jobDesc.getEnvironment(1);
        assertTrue(binding2 != null);
        assertTrue(binding2.getName() != null);
        assertTrue(binding2.getName().equals("GLOBUS_DUROC_SUBJOB_INDEX"));
        assertTrue(binding2.getValue() != null);
        assertTrue(binding2.getValue().equals("0"));

        //stdin
        /*
        assertTrue(jobDesc.getStdin() != null);
        assertTrue(jobDesc.getStdin().equals("${GLOBUS_USER_HOME}/stdin"));
        */

        //stdout
        assertTrue(jobDesc.getStdout() != null);
        assertTrue(jobDesc.getStdout().equals("${GLOBUS_USER_HOME}/stdout"));

        //stderr
        assertTrue(jobDesc.getStderr() != null);
        assertTrue(jobDesc.getStderr().equals("${GLOBUS_USER_HOME}/stderr"));

        //count
        assertTrue(jobDesc.getCount() != null);
        assertTrue(jobDesc.getCount().equals(new PositiveInteger("1")));

        //job type
        assertTrue(jobDesc.getJobType() != null);
        //Axis-generated class does not specialize equals().... :-(
        assertTrue(JobTypeEnumeration.fromValue(jobDesc.getJobType().getValue())
                   == JobTypeEnumeration.multiple);
    }

    public JobDescriptionType readRSLFromFile(String rslFileName)
        throws Exception
    {
        return RSLHelper.readRSL(new File(rslFileName));
    }

    private static final String GLOBUS_LOCATION
        = System.getProperty("GLOBUS_LOCATION");
    private static final String SHARE_PATH
        = GLOBUS_LOCATION + File.separator + "test" + File.separator
        + System.getProperty("package.name") + File.separator;
    private static final String SIMPLE_RSL
        = SHARE_PATH + "simplest_job.xml";
    private static final String SIMPLE_MULTI_RSL
        = SHARE_PATH + "simplest_multi_job.xml";
    private static final String TEST_RSL
        = SHARE_PATH + "test.xml";

    private static Log logger = LogFactory.getLog(RSLTest.class.getName());
}
