/*
* Portions of this file Copyright 1999-2005 University of Chicago
* Portions of this file Copyright 1999-2005 The University of Southern California.
*
* This file or a portion of this file is licensed under the
* terms of the Globus Toolkit Public License, found at
* http://www.globus.org/toolkit/download/license.html.
* If you redistribute this file, with or without
* modifications, you must include this notice in the file.
*/

package org.globus.mds.aggregator.impl;

/**
 * Service developers incorporating the Aggregator framework may implement 
 * this interface, specifying the implementing class as the "entryRemovedCallback" 
 * parameter in the AggregatorConfiguration JNDI bean of a service jndi-config.xml
 * file.  
 * 
 * Whenever a AggregatorServiceGroupEntryResource is removed from an 
 * AggregatorServiceGroupEntryHome, the corresponding remove method will be 
 * invoked on the supplied class, passing as a parameter the instance of the 
 * resource that is about to be removed.
 *
 * NOTE: 
 * Implementing classes should excercise caution when copying and storing 
 * references to members of the supplied AggregatorServiceGroupEntryResource,
 * and should not retain such references beyond the scope of the remove 
 * function call; otherwise the AggregatorServiceGroupEntryResource will not be
 * eligible for garbage collection and undesired heap growth will occur.
 */
public interface AggregatorServiceGroupEntryRemovedCallback {
    
    public void remove(AggregatorServiceGroupEntryResource resource) 
        throws Exception;
}
