/*
* Portions of this file Copyright 1999-2005 University of Chicago
* Portions of this file Copyright 1999-2005 The University of Southern California.
*
* This file or a portion of this file is licensed under the
* terms of the Globus Toolkit Public License, found at
* http://www.globus.org/toolkit/download/license.html.
* If you redistribute this file, with or without
* modifications, you must include this notice in the file.
*/

/*
 * ExecutionAggregatorSource.java
 */

package org.globus.mds.aggregator.impl;

import org.globus.mds.aggregator.types.AggregatorContent;
import org.globus.mds.aggregator.types.AggregatorConfig;
import org.globus.mds.aggregator.types.AggregatorData;
import org.globus.mds.aggregator.types.AggregatorPollType;
import org.globus.mds.aggregator.types.ExecutionPollType;



import org.globus.wsrf.Constants;
import org.globus.wsrf.utils.AnyHelper;
import org.globus.wsrf.encoding.DeserializationException;
import org.globus.wsrf.encoding.ObjectDeserializer;
import org.globus.wsrf.encoding.ObjectSerializer;

import org.globus.wsrf.config.ContainerConfig;

import org.globus.wsrf.impl.timer.TimerManagerImpl;
import org.globus.wsrf.impl.timer.TimerListenerWrapper;

import org.apache.axis.encoding.AnyContentType;
import org.apache.axis.utils.XMLUtils;

import org.apache.axis.message.MessageElement;
import org.apache.axis.message.addressing.Address;
import org.apache.axis.message.addressing.EndpointReferenceType;


import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import commonj.timers.Timer;
import commonj.timers.TimerListener;

import java.io.OutputStream;
import java.io.PrintStream;
import java.io.File;

import java.util.Iterator;
import java.util.Hashtable;

import javax.xml.namespace.QName;

import org.w3c.dom.Element;
import org.w3c.dom.Document;


/**
 *  An AggregatorSource which generates data using synchronous poll-style 
 *  execution of an administrator-configured script.
 */
public class ExecutionAggregatorSource extends PollingAggregatorSource
{
    private static Log logger =
            LogFactory.getLog(ExecutionAggregatorSource.class.getName());
    
    private Hashtable probeTable;
    private String baseDir;
  
    /** Creates a new instance of ExecutionAggregatorSource */    
    public ExecutionAggregatorSource(AggregatorSink sink) {
        super(sink);
        this.setAggregatorSink(sink, null);
        this.baseDir = ContainerConfig.getBaseDirectory();
    }
   
    public void initialize(Object parameters) 
        throws AggregatorException, Exception 
    {
        if ((parameters != null) && (!(parameters instanceof AggregatorConfiguration))) {
            throw new AggregatorException("Unexpected parameter type in initialize");
        }
        this.probeTable = 
            ((AggregatorConfiguration)parameters).getExecutableMappingTable();
        
        super.initialize(parameters);
    }    

    public AnyContentType poll(AggregatorPollType pollParameters,
        EndpointReferenceType epr, AggregatorConfig config)  
    {

            ExecutionPollType parameters = (ExecutionPollType) pollParameters;
            if(parameters == null || 
                parameters.getProbeName() == null ||
                parameters.getProbeName().getLocalPart() == null) {

                logger.warn("Could not extract probe name from parameter block. Will not call script.");
                return null;
            }

            String probeName = parameters.getProbeName().getLocalPart();
            String probeFile = (String)this.probeTable.get(probeName);              
            
            if (probeFile == null) {
                logger.warn(
                    "Could not locate a suitable executable file for probe name (no action will be taken): " 
                            + probeName);
                return null;                
            }

            // construct the commandline and pass in the URL in the 
            // EPR's address as arg[1]                 
            String commandline= this.baseDir +
                                "/libexec/aggrexec/" +
                                probeFile +
                                " " + epr.getAddress();

            // prepare XML input document
            StringBuffer input = new StringBuffer("<inputDocument>");
            try {
                input.append(
                    ObjectSerializer.toString(epr, 
                        epr.getTypeDesc().getXmlType()));
                input.append(
                    ObjectSerializer.toString(config, 
                        config.getTypeDesc().getXmlType()));
            } catch (Exception e) {
                logger.error("Exception while serializing input stream: " + e);
            }
            input.append("</inputDocument>");

            if (logger.isDebugEnabled()) {
                logger.debug("Executing script " + commandline + 
                             " with the following input: " + input.toString());
            }
            
            // run the process
            // TODO: need to catch if process hangs:
            //   kill and/or do not start a new one whilst one is outstanding 
            Element response;
            Process process;
            Runtime runtime = Runtime.getRuntime();
            try {
                process = runtime.exec(commandline);
            } catch(Exception e) {
                logger.error("Could not execute " +commandline+" with exception: "+e);
                return null;
            }
            
            PrintStream p = null;
            try {
                p = new PrintStream(process.getOutputStream());
                p.println(input);
            } catch(Exception e) {
                logger.error("Exception writing input stream to " +commandline+": "+e);
                return null;
            } finally {
                try {  
                    if (p!=null) {
                        p.close();
                    }
                } catch (Exception e) { }
            }
            
            try
            {
                Document doc = XMLUtils.newDocument(process.getInputStream());
                response = doc.getDocumentElement();
            }
            catch (Exception e)
            {
                logger.error("Could not parse output from "
                    +commandline+". Will not deliver. Exception: "+e);
                return null;
            }
            
            AggregatorData data = new AggregatorData();
            data.set_any(AnyHelper.toAnyArray(response));
            
            return data;
        }


    public AggregatorPollType deserializeParameter(MessageElement m)
        throws DeserializationException
    {

        if (ObjectDeserializer.contains(m, ExecutionPollType.class))
        {
                return (ExecutionPollType)
                    ObjectDeserializer.toObject(m, ExecutionPollType.class);
        } else {
            return null;
        }
    }
}
