/*
* Portions of this file Copyright 1999-2005 University of Chicago
* Portions of this file Copyright 1999-2005 The University of Southern California.
*
* This file or a portion of this file is licensed under the
* terms of the Globus Toolkit Public License, found at
* http://www.globus.org/toolkit/download/license.html.
* If you redistribute this file, with or without
* modifications, you must include this notice in the file.
*/

package org.globus.mds.aggregator.impl;

import org.globus.axis.gsi.GSIConstants;

import org.globus.mds.aggregator.types.AggregatorContent;
import org.globus.mds.aggregator.types.AggregatorConfig;
import org.globus.mds.aggregator.types.AggregatorData;

import org.globus.mds.aggregator.types.AggregatorPollType;
import org.globus.mds.aggregator.types.GetResourcePropertyPollType;
import org.globus.mds.aggregator.types.GetMultipleResourcePropertiesPollType;
import org.globus.mds.aggregator.types.QueryResourcePropertiesPollType;

import org.oasis.wsrf.properties.WSResourcePropertiesServiceAddressingLocator;
import org.oasis.wsrf.properties.GetResourceProperty;
import org.oasis.wsrf.properties.GetResourcePropertyResponse;
import org.oasis.wsrf.properties.GetMultipleResourceProperties_Element;
import org.oasis.wsrf.properties.GetMultipleResourceProperties_PortType;
import org.oasis.wsrf.properties.GetMultipleResourcePropertiesResponse;
import org.oasis.wsrf.properties.QueryExpressionType;
import org.oasis.wsrf.properties.QueryResourceProperties_Element;
import org.oasis.wsrf.properties.QueryResourcePropertiesResponse;
import org.oasis.wsrf.properties.QueryResourceProperties_PortType;


import org.globus.wsrf.utils.AnyHelper;
import org.globus.wsrf.utils.FaultHelper;
import org.globus.wsrf.encoding.DeserializationException;
import org.globus.wsrf.encoding.ObjectDeserializer;
import org.globus.wsrf.impl.security.authentication.Constants;
import org.globus.wsrf.impl.security.authorization.NoAuthorization;
import org.globus.wsrf.impl.timer.TimerManagerImpl;

import org.apache.axis.encoding.AnyContentType;
import org.apache.axis.message.MessageElement;
import org.apache.axis.message.addressing.Address;
import org.apache.axis.message.addressing.EndpointReferenceType;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import commonj.timers.Timer;
import commonj.timers.TimerListener;

import java.util.Iterator;
import java.util.Hashtable;

import javax.xml.namespace.QName;
import javax.xml.rpc.Stub;

/**
 * A PollingAggregatorSource that polls using WSRF resource property 
 * operations.
 */

public class QueryAggregatorSource extends PollingAggregatorSource
{
    private static Log logger =
            LogFactory.getLog(QueryAggregatorSource.class.getName());

    private static WSResourcePropertiesServiceAddressingLocator wsloc =
            new WSResourcePropertiesServiceAddressingLocator();

    public QueryAggregatorSource(AggregatorSink sink, Object sinkParameters) {
        super(sink, sinkParameters);
    }

    public QueryAggregatorSource(AggregatorSink sink) {
        super(sink);
    }


    public AnyContentType poll(AggregatorPollType parameters, EndpointReferenceType epr, AggregatorConfig config)
    {
        // switch on the parameter type
        if (parameters instanceof GetResourcePropertyPollType)
        {
            return pollGet(parameters,epr);
        }
        else if (parameters instanceof GetMultipleResourcePropertiesPollType)
        {
            return pollGetMultiple(parameters,epr);
        }
        else if (parameters instanceof QueryResourcePropertiesPollType)
        {
            return pollQuery(parameters,epr);
        } else {
            logger.error("Unknown poll parameter type: "+parameters.getClass());
            return null;
        }
    }


    public AnyContentType pollGet(AggregatorPollType parameters,
        EndpointReferenceType epr) 
    {
        try {
            GetResourceProperty grp =
                wsloc.getGetResourcePropertyPort(epr);

            setAnonymous((Stub)grp);

            QName request =
                ((GetResourcePropertyPollType)
                parameters).getResourcePropertyName();

            return grp.getResourceProperty(request);                       
        }
        catch (Exception e)
        {
            logger.error("Exception Getting Resource Property from " + 
                         epr.getAddress().toString() + ": " + 
                         FaultHelper.getMessage(e));
            return null;
        }
    }

    public AnyContentType pollGetMultiple(AggregatorPollType parameters,
        EndpointReferenceType epr) 
    {
        try
        {
            GetMultipleResourceProperties_PortType port =
                wsloc.getGetMultipleResourcePropertiesPort(epr);

            setAnonymous((Stub)port);

            GetMultipleResourceProperties_Element request =
                new GetMultipleResourceProperties_Element();

            request.setResourceProperty(
                ((GetMultipleResourcePropertiesPollType)
                parameters).getResourcePropertyNames());

            return port.getMultipleResourceProperties(request);

            } catch (Exception e) {
                logger.error(
                    "Exception Getting Multiple Resource Properties from " + 
                         epr.getAddress().toString() + ": " + 
                         FaultHelper.getMessage(e));
                return null;
            }
    }

    public AnyContentType pollQuery(AggregatorPollType parameters,
        EndpointReferenceType epr) 
    {
        try
        {
            QueryResourceProperties_PortType port =
                wsloc.getQueryResourcePropertiesPort(epr);

            setAnonymous((Stub)port);

            QueryResourceProperties_Element request
                = new QueryResourceProperties_Element();

            request.setQueryExpression(
                ((QueryResourcePropertiesPollType)
                parameters).getQueryExpression()
                );
             
            return port.queryResourceProperties(request);

        } catch (Exception e) {
            logger.error(
                "Exception Querying Resource Properties from " + 
                         epr.getAddress().toString() + ": " + 
                         FaultHelper.getMessage(e));
                return null;
        }
    }


    public AggregatorPollType deserializeParameter(MessageElement m) 
        throws DeserializationException {

        if (ObjectDeserializer.contains(
            m,
            GetResourcePropertyPollType.class))
        {
            return (GetResourcePropertyPollType)
             ObjectDeserializer.toObject(
             m,
             GetResourcePropertyPollType.class
             );

        }
        else if (ObjectDeserializer.contains(
                    m,
                    GetMultipleResourcePropertiesPollType.class))
        {
            return (GetMultipleResourcePropertiesPollType)
                ObjectDeserializer.toObject(
                    m,
                    GetMultipleResourcePropertiesPollType.class
                );
        }
        else if (ObjectDeserializer.contains(
                    m,
                    QueryResourcePropertiesPollType.class))
        {
            return (QueryResourcePropertiesPollType)
                ObjectDeserializer.toObject(
                    m,
                    QueryResourcePropertiesPollType.class);
        } else {
            return null;
        }

    }


    private void setAnonymous(Stub stub) {
        stub._setProperty(Constants.GSI_ANONYMOUS, Boolean.TRUE);
        stub._setProperty(Constants.AUTHORIZATION,
            NoAuthorization.getInstance());
        stub._setProperty(GSIConstants.GSI_AUTHORIZATION,
            org.globus.gsi.gssapi.auth.NoAuthorization.getInstance());
     }

}

