/*
* Portions of this file Copyright 1999-2005 University of Chicago
* Portions of this file Copyright 1999-2005 The University of Southern California.
*
* This file or a portion of this file is licensed under the
* terms of the Globus Toolkit Public License, found at
* http://www.globus.org/toolkit/download/license.html.
* If you redistribute this file, with or without
* modifications, you must include this notice in the file.
*/

/*
 * SubscriptionAggregatorSource.java
 */

package org.globus.mds.aggregator.impl;

import org.globus.mds.aggregator.types.AggregatorContent;
import org.globus.mds.aggregator.types.AggregatorConfig;
import org.globus.mds.aggregator.types.AggregatorData;
import org.globus.mds.aggregator.types.AggregatorSubscriptionType;

import org.oasis.wsn.WSBaseNotificationServiceAddressingLocator;
import org.oasis.wsn.NotificationProducer;
import org.oasis.wsn.Subscribe;
import org.oasis.wsn.SubscribeResponse;
import org.oasis.wsn.TopicExpressionType;

import org.oasis.wsrf.lifetime.WSResourceLifetimeServiceAddressingLocator;
import org.oasis.wsrf.lifetime.ImmediateResourceTermination;
import org.oasis.wsrf.lifetime.Destroy;

import org.globus.wsrf.NotifyCallback;
import org.globus.wsrf.NotificationConsumerManager;
import org.globus.wsrf.Constants;
import org.globus.wsrf.utils.AnyHelper;
import org.globus.wsrf.utils.XmlUtils;
import org.globus.wsrf.utils.FaultHelper;
import org.globus.wsrf.encoding.ObjectSerializer;
import org.globus.wsrf.encoding.ObjectDeserializer;
import org.globus.wsrf.core.notification.ResourcePropertyValueChangeNotificationElementType;

import java.util.Calendar;
import java.util.Iterator;
import java.util.Hashtable;
import javax.xml.namespace.QName;
import javax.xml.rpc.Stub;

import org.apache.axis.encoding.AnyContentType;
import org.apache.axis.message.MessageElement;
import org.apache.axis.message.addressing.EndpointReferenceType;

import org.globus.wsrf.impl.security.descriptor.ResourceSecurityConfig;
import org.globus.wsrf.impl.security.descriptor.ResourceSecurityDescriptor;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.globus.axis.gsi.GSIConstants;


/**
 *  An AggregatorSource which generates data using asyncrhonous 
 *  subscription-based messaging.
 */
public class SubscriptionAggregatorSource implements AggregatorSource
{
    private boolean initialized = false;
    private AggregatorSink sink = null;
    private Object sinkParameters = null;
    private String resDescFilename = null;
    private Hashtable sessions = new Hashtable();   
    private NotificationConsumerManager consumer = null;
    
    WSBaseNotificationServiceAddressingLocator locator =
        new WSBaseNotificationServiceAddressingLocator();

    WSResourceLifetimeServiceAddressingLocator lifetimeLocator =
        new WSResourceLifetimeServiceAddressingLocator();
        
    private static Log logger =
            LogFactory.getLog(SubscriptionAggregatorSource.class.getName());
    
    /* Internal class that handles message processing */
    protected class NotificationCallback implements NotifyCallback
    {
        public EndpointReferenceType sourceEPR = null;
        public EndpointReferenceType consumerEPR = null;
        
        private AggregatorServiceGroupEntryResource entry = null;
        
        public NotificationCallback(AggregatorServiceGroupEntryResource entry)
        {
            this.entry = entry;       
        }
        
        public Object getEntry() 
        {
            return this.entry;
        }            
        
        public void deliver(java.util.List topicPath,
                            EndpointReferenceType producer,
                            Object message) 
        {
            AggregatorSink sink = 
                SubscriptionAggregatorSource.this.getAggregatorSink();            
            
            /*if (!(message instanceof AnyContentType)) {
                logger.warn("Dont know how to handle message type: " + message.toString());
                return;
            }*/
            
            AggregatorData data = new AggregatorData();
            data.set_any(AnyHelper.toAnyArray(message));
            
            try {
                sink.deliver(data, this.entry);
            } catch (Exception e) {
                logger.error("Exception during sink message delivery");
            }
        }
        
    }
    
    /** Creates a new instance of SubscriptionAggregatorSource */
    public SubscriptionAggregatorSource(AggregatorSink sink, 
                                        Object sinkParameters) 
    {
        this.setAggregatorSink(sink, sinkParameters);
    }
    
    public SubscriptionAggregatorSource(AggregatorSink sink) 
    {
        this.setAggregatorSink(sink, null);
    }
    
    public void initialize(Object parameters) 
        throws AggregatorException, Exception 
    {
        if (this.initialized) {
            if (logger.isDebugEnabled()) {
                logger.debug("Reinitializing");
            }
            this.terminate();
        }
        
        if (this.getAggregatorSink() == null) {
            throw new AggregatorException("Aggregator Sink not set!");
        } else {
            this.sink.initialize(this.sinkParameters);
        }
        
        this.consumer = NotificationConsumerManager.getInstance();
        this.consumer.startListening();
            
        this.initialized = true;             
    }
    
    public AggregatorServiceGroupEntryResource getSessionContext(String sessionKey) 
    {
        return (AggregatorServiceGroupEntryResource)this.sessions.get(sessionKey);
    }    
    
    public String getSessionKey(AggregatorServiceGroupEntryResource entry) 
        throws AggregatorException 
    {
        if (entry == null) {
            throw new AggregatorException("Null Argument");
        }
        
        String key = "";
        for (Iterator keys = this.sessions.keySet().iterator(); keys.hasNext();) 
        {
            key = (String) keys.next();
            NotificationCallback callback = 
                (NotificationCallback) this.sessions.get(key);
            if (entry.equals(callback.getEntry())) {
                break;
            }
        }
        return key;
    }
    
    public boolean isValidSession(String sessionKey) 
        throws AggregatorException 
    {
        return this.sessions.containsKey(sessionKey);
    }
    
    public AggregatorSink getAggregatorSink() 
    {
        return this.sink;
    }
    
    public void setAggregatorSink(AggregatorSink sink, Object sinkParameters) 
    {
        this.sink = sink;
        if (this.sink != null) {
            this.sink.setSource(this);
            this.sinkParameters = sinkParameters;
        }        
    }
    
    public String addAggregation(Object parameters, 
                                 AggregatorServiceGroupEntryResource entry) 
        throws AggregatorParamTypeNotSupportedException,
               AggregatorException,
               Exception 
    {                
        if (!initialized) {
            throw new AggregatorException("Not Initialized");
        }
        
        AggregatorConfig config = (AggregatorConfig) parameters;
        MessageElement[] any = config.get_any();
        
        // not sure this will ever happen, but check just in case
        if (any == null) {
           throw new AggregatorException("Got null any from AggregatorConfig");
        }
        
        AggregatorSubscriptionType aggrSub = null;            
        for (int i=0; i<any.length; i++) 
        {
            try 
            {
                if (ObjectDeserializer.contains(
                    any[i],
                    AggregatorSubscriptionType.class))
                {
                    aggrSub = (AggregatorSubscriptionType) 
                        ObjectDeserializer.toObject(
                            any[i], 
                            AggregatorSubscriptionType.class
                        );
                }                
            } catch (Exception e) {
                logger.error("Exception deserializing parameters: " + 
                             e.toString());
            }            
        }         
        
        if (aggrSub == null) {
            throw new AggregatorParamTypeNotSupportedException();
        } else if (logger.isDebugEnabled()) {
            logger.debug("Got AggregatorSubscriptionType from AggregatorConfig: " + 
                         XmlUtils.toString(
                            ObjectSerializer.toElement(
                                aggrSub,aggrSub.getTypeDesc().getXmlType())));
        }
        
        try 
        {
        
            NotificationProducer port = 
                this.locator.getNotificationProducerPort(entry.getMemberEPR());

            setAnonymous((Stub)port);

            ResourceSecurityDescriptor resDesc = null;

            if (resDescFilename != null) {
                ResourceSecurityConfig secConfig = 
                new ResourceSecurityConfig(resDescFilename);
                secConfig.init();
                resDesc = secConfig.getSecurityDescriptor();
            }

            NotificationCallback callback = new NotificationCallback(entry);
            callback.consumerEPR =
                consumer.createNotificationConsumer(callback, resDesc);
                    
            Subscribe request = new Subscribe();
            
            request.setUseNotify(Boolean.TRUE);
            request.setConsumerReference(callback.consumerEPR);
            request.setTopicExpression(aggrSub.getTopicExpression());
            request.setPrecondition(aggrSub.getPrecondition());
            request.setSelector(aggrSub.getSelector());
            request.setSubscriptionPolicy(aggrSub.getSubscriptionPolicy());
            
            Calendar cal = aggrSub.getInitialTerminationTime();
            request.setInitialTerminationTime(cal);
            if ((cal == null) && (logger.isDebugEnabled())) {
                logger.debug("Subscribe request for " + 
                             entry.getMemberEPR().getAddress() + 
                             " has INFINITE lifetime");
            }

            SubscribeResponse response = port.subscribe(request);
            
            callback.sourceEPR = response.getSubscriptionReference();            
            
            String sessionKey = Integer.toString(callback.hashCode());
            this.sessions.put(sessionKey, callback);
            
            return sessionKey;            
        } 
        catch(Exception e) 
        {
            logger.error("Exception while trying to subscribe to " + 
                         entry.getMemberEPR().getAddress() + ": " + 
                         FaultHelper.getMessage(e));
            throw e;
        }
        finally 
        {
            if (consumer != null) {
                //try { consumer.stopListening(); } catch (Exception ee) {}
            }
        }         
    }
    
    public void removeAggregation(String sessionKey) 
        throws AggregatorException, Exception 
    {
        NotificationCallback callback = 
            (NotificationCallback) this.sessions.remove(sessionKey);
                
        try {
            ImmediateResourceTermination port = 
                lifetimeLocator.getImmediateResourceTerminationPort(
                    callback.sourceEPR);
       
            setAnonymous((Stub)port); 
            port.destroy(new Destroy());
        }
        catch(Exception e) 
        {
            logger.error("Exception while trying to remove subscription to " + 
                         callback.sourceEPR.getAddress() + ": " + 
                         FaultHelper.getMessage(e));
            throw e;
        }
        finally 
        {   
            this.consumer.removeNotificationConsumer(callback.consumerEPR);
        }
    }
   
    public void terminate() 
    {
        if (!this.initialized) {
            return;
        }       
        
        this.getAggregatorSink().terminate();
        
        for (Iterator keys = this.sessions.keySet().iterator(); keys.hasNext();) 
        {
            try {
                this.removeAggregation((String) keys.next());
            } catch (Exception e) {
                logger.error("Exception during removeAggregation: " + e);
            }
        }
        
        try {
            this.consumer.stopListening();
        } catch (Exception e) {
            logger.error("Exception: " + e.toString());
        }        
        
        this.initialized = false;        
    }       
   

    private void setAnonymous(Stub stub) {

        stub._setProperty(
            org.globus.wsrf.impl.security.authentication.Constants.GSI_ANONYMOUS, 
            Boolean.TRUE);
        stub._setProperty(
            org.globus.wsrf.impl.security.authentication.Constants.AUTHORIZATION,
            org.globus.wsrf.impl.security.authorization.NoAuthorization.getInstance());
        stub._setProperty(GSIConstants.GSI_AUTHORIZATION,
            org.globus.gsi.gssapi.auth.NoAuthorization.getInstance());
     }

}
