/*
* Portions of this file Copyright 1999-2005 University of Chicago
* Portions of this file Copyright 1999-2005 The University of Southern California.
*
* This file or a portion of this file is licensed under the
* terms of the Globus Toolkit Public License, found at
* http://www.globus.org/toolkit/download/license.html.
* If you redistribute this file, with or without
* modifications, you must include this notice in the file.
*/
package org.globus.mds.index.test.junit;

import org.globus.mds.aggregator.impl.AggregatorConstants;
import org.globus.mds.aggregator.impl.AggregatorConstants;
import javax.xml.namespace.QName;
import javax.xml.soap.SOAPElement;
import javax.xml.rpc.Stub;
import org.globus.wsrf.encoding.ObjectSerializer;
import org.globus.wsrf.encoding.SerializationException;

import com.widgets.service.WidgetNotificationServiceAddressingLocator;
import com.widgets.WidgetNotificationPortType;
import com.widgets.GenerateNotification;

import java.net.URL;
import java.rmi.RemoteException;
import javax.xml.namespace.QName;
import junit.textui.TestRunner;

import org.apache.axis.message.MessageElement;
import org.apache.axis.message.addressing.Address;
import org.apache.axis.message.addressing.EndpointReferenceType;
import org.apache.axis.message.addressing.ReferencePropertiesType;
import org.apache.axis.types.URI;
import org.apache.axis.utils.XMLUtils;

import org.globus.interop.widget.Widget;
import org.globus.interop.widget.WidgetNotificationService;
import org.globus.mds.aggregator.test.junit.AggregatorTestCase;
import org.globus.mds.aggregator.types.AggregatorContent;
import org.globus.mds.aggregator.types.AggregatorConfig;
import org.globus.mds.aggregator.types.AggregatorData;
import org.globus.mds.aggregator.types.AggregatorSubscriptionType;
import org.globus.wsrf.Constants;
import org.globus.wsrf.WSNConstants;
import org.globus.wsrf.encoding.ObjectDeserializer;
import org.globus.wsrf.encoding.ObjectSerializer;
import org.globus.wsrf.impl.servicegroup.ServiceGroupConstants;
import org.globus.wsrf.impl.servicegroup.client.ServiceGroupRegistrationClient;
import org.globus.wsrf.utils.AddressingUtils;
import org.globus.wsrf.utils.AnyHelper;
import org.globus.wsrf.utils.XmlUtils;
import org.globus.mds.servicegroup.client.ServiceGroupRegistrationParameters;
import org.globus.mds.servicegroup.stubs.ServiceGroupFactoryPortType;
import org.globus.mds.servicegroup.stubs.ServiceGroupFactoryServiceAddressingLocator;

import org.oasis.wsn.TopicExpressionType;
import org.oasis.wsrf.servicegroup.ServiceGroupServiceAddressingLocator;
import org.oasis.wsrf.servicegroup.ServiceGroupRegistration;




public class SubscriptionIndexTest extends AggregatorTestCase {

    private String indexServiceAddress =
        "https://127.0.0.1:8443/wsrf/services/mds/test/subsource/IndexService";

    private String indexServiceAddressNoSecurity =
        "http://127.0.0.1:8080/wsrf/services/mds/test/subsource/IndexService";
    
    public static void main(String args[]) throws Exception
    {
        TestRunner.run(SubscriptionIndexTest.class);
    }

    public void decorateRegistrationParams(ServiceGroupRegistrationParameters 
                                           params) 
        throws SerializationException
    {
        TopicExpressionType topicExpr = null;
        try {
            topicExpr = new TopicExpressionType(
                WSNConstants.SIMPLE_TOPIC_DIALECT,
                WidgetNotificationService.TEST_TOPIC);
        } catch (Exception e) {
            fail("Exception constructing TopicExpressionType");
        }

        AggregatorSubscriptionType sub = new AggregatorSubscriptionType();
        sub.setTopicExpression(topicExpr);

        try {
            System.out.println("Subscription Parameters: " +
                               XmlUtils.toString(
                                ObjectSerializer.toElement(sub,
                                    sub.getTypeDesc().getXmlType())));
        } catch (Exception e) {};

        AggregatorContent content =
            new AggregatorContent(new AggregatorConfig(),
                                  new AggregatorData(),
                                  null);
        SOAPElement subElement = ObjectSerializer.toSOAPElement(sub,
            AggregatorConstants.SUBSCRIPTION_RP_QNAME);
        content.getAggregatorConfig().set_any(
            new MessageElement[] { AnyHelper.toAny(subElement) });

        params.setContent(content);
    }


    public EndpointReferenceType getAggregator()
    {
        try {
            EndpointReferenceType epr = new EndpointReferenceType();
            
            String serviceAddr = 
                (this.isSecure) ? 
                this.indexServiceAddress : 
                this.indexServiceAddressNoSecurity;

            epr.setAddress(new Address(serviceAddr));
            
            return epr;
        } catch(Exception e) {
            System.err.println("Exception when forming default EPR");
            e.printStackTrace();
            fail("Couldn't form default EPR");
            return null;
        }
    }


    /* the below replaces the default widget handling with 
       a notification generating resource. */

    final String widgetNotificationServiceAddress =
        "https://127.0.0.1:8443/wsrf/services/WidgetNotificationService";
    
    final String widgetNotificationServiceAddressNoSecurity =
        "http://127.0.0.1:8080/wsrf/services/WidgetNotificationService";    

    protected EndpointReferenceType createWidget()
            throws Exception
    {
        String serviceAddr = 
            (this.isSecure) ? 
            this.widgetNotificationServiceAddress : 
            this.widgetNotificationServiceAddressNoSecurity;
            
        EndpointReferenceType endpoint = 
            new EndpointReferenceType(new URI(serviceAddr));
        return endpoint;
    }

    protected void setWidget(EndpointReferenceType epr, String newvalue)
            throws Exception
    {
        WidgetNotificationServiceAddressingLocator notificationLocator =
            new WidgetNotificationServiceAddressingLocator();

        String serviceAddr = 
            (this.isSecure) ? 
            this.widgetNotificationServiceAddress : 
            this.widgetNotificationServiceAddressNoSecurity;
                
        EndpointReferenceType endpoint = new EndpointReferenceType(
            new URI(serviceAddr));
        
        WidgetNotificationPortType port =
            notificationLocator.getWidgetNotificationPortTypePort(endpoint);

        setAnonymous((Stub)port);
        port.generateNotification(new GenerateNotification());
    }

}

