/*
* Portions of this file Copyright 1999-2005 University of Chicago
* Portions of this file Copyright 1999-2005 The University of Southern California.
*
* This file or a portion of this file is licensed under the
* terms of the Globus Toolkit Public License, found at
* http://www.globus.org/toolkit/download/license.html.
* If you redistribute this file, with or without
* modifications, you must include this notice in the file.
*/

package org.globus.wsrf.impl.servicegroup.client;

import java.io.InputStream;
import java.io.FileInputStream;
import java.util.List;

import javax.xml.namespace.QName;
import javax.xml.rpc.Stub;

import org.apache.axis.message.addressing.EndpointReferenceType;
import org.apache.axis.types.URI;
import org.apache.axis.utils.XMLUtils;

import org.oasis.wsrf.servicegroup.ServiceGroupServiceAddressingLocator;
import org.oasis.wsrf.servicegroup.ServiceGroupRegistration;

import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.CommandLine;

import org.globus.wsrf.Constants;
import org.globus.wsrf.client.BaseClient;
import org.globus.wsrf.encoding.ObjectDeserializer;
import org.globus.wsrf.encoding.ObjectSerializer;
import org.globus.wsrf.utils.AnyHelper;
import org.globus.wsrf.utils.FaultHelper;

import org.w3c.dom.Document;

public class ServiceGroupAdd extends BaseClient {

    public static void main(String[] args) 
    {
        ServiceGroupAdd baseClient = new ServiceGroupAdd();
        baseClient.setCustomUsage("[filename of registration file]");
        String registrationFilename = null;

        try 
        {
            CommandLine line = baseClient.parse(args);

            List options = line.getArgList();
            if (options == null || options.isEmpty()) {
                System.err.println("Must specify a registration file");
                System.exit(1);
            } else  {
                registrationFilename =(String)options.get(0);
            } 
        } catch (ParseException e) {
            System.err.println("Parsing failed: " + e.getMessage());
            System.exit(1);
        } catch (Exception e) {
            System.err.println("Error: " + e.getMessage());
            System.exit(1);
        }

        try
        {
            ServiceGroupRegistrationClient regClient = 
                new ServiceGroupRegistrationClient(baseClient);
            
            System.out.println("Processing configuration file...");            
            regClient.processConfigFile(registrationFilename);           
            
            synchronized(regClient) {
                try {
                    regClient.wait();
                } catch (InterruptedException ex) {
                    regClient.terminate();
                    System.out.println("Exiting...");
                    System.exit(1);
                }
            }
        }
        catch(Exception e) 
        {
            if (baseClient.isDebugMode()) {
                FaultHelper.printStackTrace(e);
            } else {
                System.err.println("Error: " + FaultHelper.getMessage(e));
            }
        }
    }

}

