/*
* Portions of this file Copyright 1999-2005 University of Chicago
* Portions of this file Copyright 1999-2005 The University of Southern California.
*
* This file or a portion of this file is licensed under the
* terms of the Globus Toolkit Public License, found at
* http://www.globus.org/toolkit/download/license.html.
* If you redistribute this file, with or without
* modifications, you must include this notice in the file.
*/

/*
 * ServiceGroupRegistrationClientCallback.java
 *
 * Created on April 5, 2005, 9:34 PM
 */
package org.globus.wsrf.impl.servicegroup.client;

import java.lang.Exception;
import org.globus.mds.servicegroup.client.ServiceGroupRegistrationParameters;

/**
 * Interface used for receiving registration status asynchronously
 */
public interface ServiceGroupRegistrationClientCallback 
{
    /* This method should be implemented by the caller of the 
     * ServiceGroupRegistrationClient, in order to receive asynchronous status 
     * regarding registrations that have been submitted.
     *
     * @param regParams this parameter is expected to be the same runtime 
     * reference to an existing ServiceGroupRegistrationParameters object 
     * which has been previously passed to the ServiceGroupRegistrationClient.register
     * method.
     *
     * @param wasSuccessful this parameter indicates whether the registration
     * completed successfully.  If set to false, the implementing code should
     * check the @param exception parameter for a non-null value.
     *
     * @param wasRenewalAttempt this parameter indicates whether the registration
     * was a renewal of an existing registration or a rather a new ServiceGroup 
     * add() operation was invoked. The implmenting code may wish to take a 
     * different action in this case.
     *
     * @param exception this parameter indicates the exception that was raised during 
     * new registration or existing registration renewal, if any. 
     * If the registration was successful, it will be null.
     *
     * @return a boolean value indicating whether the reg client should
     * continue to attempt to add/renew this registration, or cancel it.
     */
    public boolean setRegistrationStatus(ServiceGroupRegistrationParameters regParams,
                                         boolean wasSuccessful,
                                         boolean wasRenewalAttempt,
                                         Exception exception);
    
}
