/*
* Portions of this file Copyright 1999-2005 University of Chicago
* Portions of this file Copyright 1999-2005 The University of Southern California.
*
* This file or a portion of this file is licensed under the
* terms of the Globus Toolkit Public License, found at
* http://www.globus.org/toolkit/download/license.html.
* If you redistribute this file, with or without
* modifications, you must include this notice in the file.
*/
package org.globus.wsrf.impl.servicegroup.inmemory;

import java.util.Hashtable;

import javax.naming.Context;
import javax.naming.InitialContext;

import javax.xml.namespace.QName;

import org.apache.axis.MessageContext;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.globus.wsrf.ResourceHome;
import org.globus.wsrf.Resource;
import org.globus.wsrf.ResourceKey;
import org.globus.wsrf.Constants;

import org.globus.wsrf.impl.SimpleResourceKey;

import org.globus.wsrf.impl.servicegroup.ServiceGroupResource;
import org.globus.wsrf.impl.servicegroup.ServiceGroupConstants;

import org.globus.mds.inmemory._2004._07._13.PairedKeyType;

/** Looks up a service group entry. A key for this service consists of the
    key to the containing service group and then a unique-in-servicegroup
    key. <br>

    TODO: could probably use simple resource home here
*/

public class InMemoryServiceGroupEntryHome implements ResourceHome {

    private static Log logger =
            LogFactory.getLog(InMemoryServiceGroupEntryHome.class.getName());


    public InMemoryServiceGroupEntryHome() {
    //     this.resources = new Hashtable();
    }
   

    // TODO: should probably cache some of the values used internally 
    // within this method.
    public Resource find(ResourceKey key) {
        Resource retVal = null;

        // extract ServiceGroup key from the composite key

        SimpleResourceKey k = (SimpleResourceKey)key;

        PairedKeyType pk = (PairedKeyType)k.getValue();

        ResourceKey groupKey = 
        new SimpleResourceKey(ServiceGroupConstants.KEY, pk.getGroupKey());

        // get the servicegroup home
        try { 
            ResourceHome groupHome = getGroupHome();

        // retrieve servicegroup resource


            InMemoryServiceGroupResource group = (InMemoryServiceGroupResource) groupHome.find(groupKey);

        // look up full key in servicegroup resource

            retVal = (Resource)group.entries.get(k);
        } catch(Exception e) {

            logger.error("Exception looking up an entry key: "+e);
            return null;
        }

        logger.debug("InMemoryServiceGroupEntryHome found: "+retVal);
        return retVal;
    }

    public void remove(ResourceKey key) {
        // NOTIMPL
        // this.resources.remove(key);
    }

    public Class getKeyTypeClass() {
        return PairedKeyType.class;
    }
  
    public QName getKeyTypeName() {
        return ServiceGroupConstants.ENTRY_KEY;
    }
   



    public ResourceHome getGroupHome()
        throws Exception {

        MessageContext ctx = MessageContext.getCurrentContext();

        String homeLocation = getGroupHomeLocation(ctx);
        Context initialContext = new InitialContext();

        return (ResourceHome) initialContext.lookup(homeLocation);
    }

    public String getGroupHomeLocation(MessageContext ctx)
        throws Exception {
        if (ctx == null) {
            throw new IllegalArgumentException("ctx == null");
        }
        if (!(ctx instanceof MessageContext)) {
            throw new Exception("ctx must be Axis instance");
        }

        org.apache.axis.MessageContext context =
            (org.apache.axis.MessageContext)ctx;

        String servicePath = context.getTargetService();
        return Constants.JNDI_BASE_NAME + servicePath +
               "/groupHome";
    }







 
}

