/*
* Portions of this file Copyright 1999-2005 University of Chicago
* Portions of this file Copyright 1999-2005 The University of Southern California.
*
* This file or a portion of this file is licensed under the
* terms of the Globus Toolkit Public License, found at
* http://www.globus.org/toolkit/download/license.html.
* If you redistribute this file, with or without
* modifications, you must include this notice in the file.
*/
package org.globus.wsrf.impl.servicegroup.inmemory;

import org.apache.axis.MessageContext;

import org.apache.axis.message.addressing.EndpointReferenceType;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.globus.wsrf.ResourceKey;
import org.globus.wsrf.ResourceProperties;
import org.globus.wsrf.ResourceProperty;
import org.globus.wsrf.ResourcePropertySet;

import org.globus.wsrf.impl.SimpleResourceKey;
import org.globus.wsrf.impl.SimpleResourcePropertySet;
import org.globus.wsrf.impl.ReflectionResourceProperty;

import org.globus.wsrf.utils.AddressingUtils;
import org.oasis.wsrf.servicegroup.EntryType;
import org.oasis.wsrf.servicegroup.Add;

import org.globus.wsrf.impl.servicegroup.EntryResourceProperty;
import org.globus.wsrf.impl.servicegroup.ServiceGroupConstants;
import org.globus.wsrf.impl.servicegroup.ServiceGroupRegistrationResource;

import java.rmi.RemoteException;
import java.util.HashMap;
import java.util.Iterator;

import javax.xml.namespace.QName;

/** Implements a simple in-memory service group resource.
 *  <br>
 *  Entries will be stored in a hash table under each service group resource
 *  so that we can easily find them; the key for a service group entry
 *  must first contain the key to the appropriate service group and then
 *  the key within that to the appropriate entry. <br />
 * 
 */

public class InMemoryServiceGroupResource 
       implements ResourceProperties, ServiceGroupRegistrationResource {

    private static Log logger =
            LogFactory.getLog(InMemoryServiceGroupResource.class.getName());

    public static final QName RP_SET =
        new QName("http://mds.globus.org/inmemory/2004/07/13", "InMemoryServiceGroupRP");

    ResourcePropertySet propSet;

    /** Maps entry keys to entry resource objects. */
    HashMap entries = new HashMap();

    public InMemoryServiceGroupResource() {
        this.propSet = new SimpleResourcePropertySet(RP_SET);


        // initialise the content RP; this will need to be generated
        // dynamically from all of the entries within this ServiceGroup
        try {
            ResourceProperty prop;

            prop=new EntryResourceProperty(this);
            this.propSet.add(prop);
        } catch(Exception e) {
            logger.error("Couldn't set up Entry RP");
        }

        
    }

   public ResourcePropertySet getResourcePropertySet() {
        return this.propSet;
    }

    private ResourceKey key =
        new SimpleResourceKey(ServiceGroupConstants.KEY, String.valueOf(hashCode()));

    public ResourceKey getKey() {
        return key;
    }

    // to be a servicegroup
    // NOTE: not thread-safe
    public Iterator iterateEntryKeys() {
        return entries.keySet().iterator();
    }
    
    // to be a servicegroup
    // NOTE: not thread-safe
    public Iterator iterateEntryValues() {
        return entries.entrySet().iterator();
    }

    // to be a servicegroup registration

    public EndpointReferenceType serviceGroupAdd(Add add) throws RemoteException
    {
        // create an entry resource

        EndpointReferenceType entryEPR = null;
        EndpointReferenceType serviceGroupEPR = null;        
        
        Object initialContent = add.getContent();
        try
        {        
            serviceGroupEPR = AddressingUtils.createEndpointReference(null);
        }
        catch(Exception e1)
        {
            throw new RemoteException("", e1);
        }
        
        InMemoryServiceGroupEntryResource entry = 
            new InMemoryServiceGroupEntryResource(getKey(),
                                                  add,
                                                  serviceGroupEPR);

        // invent a key
        ResourceKey key = entry.getKey();
        System.out.println("resource key is "+key);

        // construct an EPR to entry through the entry service.

        MessageContext msgContext = MessageContext.getCurrentContext();
        String transportURL =
            (String) msgContext.getProperty(MessageContext.TRANS_URL);
        // TODO: is there a better way to do this?
        transportURL = transportURL + "Entry" ;
        logger.debug("Will use transport URL in entry EPRs: "+transportURL);
        try
        {
            entryEPR = AddressingUtils.createEndpointReference(transportURL, key);            
        }
        catch(Exception e1)
        {
            throw new RemoteException("", e1);
        }

        entry.setEntryEPR(entryEPR);

        // put it in the entry map
        entries.put(key,entry);        

        return entryEPR;

    }

}

