/*
* Portions of this file Copyright 1999-2005 University of Chicago
* Portions of this file Copyright 1999-2005 The University of Southern California.
*
* This file or a portion of this file is licensed under the
* terms of the Globus Toolkit Public License, found at
* http://www.globus.org/toolkit/download/license.html.
* If you redistribute this file, with or without
* modifications, you must include this notice in the file.
*/
package org.globus.mds.trigger.impl;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import javax.xml.namespace.QName;

import org.apache.axis.message.addressing.Address;
import org.apache.axis.message.addressing.EndpointReferenceType;
import org.apache.axis.message.addressing.ReferencePropertiesType;
import org.apache.axis.MessageContext;
import org.apache.axis.utils.XMLUtils;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.globus.wsrf.Constants;
import org.globus.wsrf.ResourceKey;
import org.globus.wsrf.ResourceProperty;
import org.globus.wsrf.Resource;
import org.globus.wsrf.ResourceProperties;
import org.globus.wsrf.ResourcePropertySet;

import org.globus.wsrf.utils.FaultHelper;

import org.globus.mds.servicegroup.stubs.CreateResponse;
import org.globus.mds.servicegroup.stubs.Create;
import org.globus.mds.servicegroup.stubs.ServiceGroupFactoryCreateFaultType;

/**
 * Trigger factory. 
 */
public class TriggerFactoryService implements Resource
{
    private static Log logger =
            LogFactory.getLog(TriggerFactoryService.class.getName());

    public CreateResponse create(Create request) 
        throws ServiceGroupFactoryCreateFaultType 
    {

        CreateResponse response = new CreateResponse();
        
        EndpointReferenceType epr = new EndpointReferenceType();

        TriggerHome home = null;
        TriggerResource w = null;
        String triggerHomeName = 
            Constants.JNDI_BASE_NAME + "triggerHome";

        try {
            Context initialContext = new InitialContext();
            home =
                (TriggerHome) initialContext.lookup(triggerHomeName);
            w = home.create();
        } catch (NamingException e) {
            ServiceGroupFactoryCreateFaultType fault = 
                new ServiceGroupFactoryCreateFaultType();
            FaultHelper faultHelper = new FaultHelper(fault);
            faultHelper.addFaultCause(e);
            faultHelper.addDescription("when performing JNDI lookup for "+triggerHomeName);
            throw fault;  
        } catch (Exception e) {
            ServiceGroupFactoryCreateFaultType fault = 
                new ServiceGroupFactoryCreateFaultType();
            FaultHelper faultHelper = new FaultHelper(fault);
            faultHelper.addFaultCause(e);
            throw fault;   
        }
        
        ResourceKey key = w.getResourceKey();
        ReferencePropertiesType props = new ReferencePropertiesType();

        try {
            props.add(key.toSOAPElement());
        }
        catch(Exception e) {
            ServiceGroupFactoryCreateFaultType fault = 
                new ServiceGroupFactoryCreateFaultType();
            FaultHelper faultHelper = new FaultHelper(fault);
            faultHelper.addFaultCause(e);
            faultHelper.addDescription("Exception when serializing key");
            throw fault;  
        }
        MessageContext msgContext = MessageContext.getCurrentContext();
        String transportURL = 
            (String) msgContext.getProperty(MessageContext.TRANS_URL);
        // this is the URL to the TriggerFactoryService
        // but what we actually want is the URL to the TriggerService
        // XXX: yuck
        transportURL = transportURL.substring(0,transportURL.indexOf("Factory"))
                    + transportURL.substring(transportURL.indexOf("Factory")+7);
        logger.debug("Trigger factory will use transport URL "+transportURL);
        try {
            epr.setAddress(new Address(transportURL));
        } catch (Exception e) {
            ServiceGroupFactoryCreateFaultType fault = 
                new ServiceGroupFactoryCreateFaultType();
            FaultHelper faultHelper = new FaultHelper(fault);
            faultHelper.addFaultCause(e);
            throw fault;  
        }
        epr.setProperties(props);

        response.setEndpointReference(epr);
        return response;
    }    
}

