/*
* Portions of this file Copyright 1999-2005 University of Chicago
* Portions of this file Copyright 1999-2005 The University of Southern California.
*
* This file or a portion of this file is licensed under the
* terms of the Globus Toolkit Public License, found at
* http://www.globus.org/toolkit/download/license.html.
* If you redistribute this file, with or without
* modifications, you must include this notice in the file.
*/

package org.globus.mds.usefulrp.glue;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.io.InputStream;
import java.io.FileInputStream;

import java.net.ConnectException;
import java.net.Socket;

import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.stream.StreamSource;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import org.globus.wsrf.config.ContainerConfig;


/** An instance of this class will provide an XML element for a cluster
    in the GLUE schema using data gathered from a ganglia gmetad that 
    should be running elsewhere. This can then be used, for example,
    as a ResourceProperty value. */

public class GangliaElementProducer implements GLUEElementProducer
{

    private static final Log logger =
            LogFactory.getLog(GangliaElementProducer.class.getName());

    /** hostname or IP address to connect to for Ganglia data.
        defaults to localhost. */
    String gangliaHostName = "localhost"; 

    /** TCP port to connect to for Ganglia data.
        defaults to port 8649 */
    int gangliaTCPPort = 8649;

    /** Cluster to report info for. XML from the Ganglia daemon may
        contain information for multiple clusters, but we want to 
        select a specific one. For the moment, just use a cluster 
        name -- potentially it would be necessary to match against
        Grid name too if Ganglia has such an attribute? */
    String gangliaClusterIdentifier;

    /** Points to the XSLT file that will perform the glue to ganglia
        transformation. TODO Probably should be configurable? */
    String xsltFilename = getBaseDirectory() + 
        "/etc/globus_wsrf_mds_usefulrp/ganglia_to_glue.xslt";

    /** Creates an instance with all default params. */
    public GangliaElementProducer()
    {
        ;
    }

    private static String getBaseDirectory() 
    {
        return ContainerConfig.getBaseDirectory();
    }


    public Element getElement() throws Exception
    {
        Socket gangliaSocket = null;
        InputStream xsltInputStream = null; 
        
        try 
        {
            // make tcp conn to ganglia host and get an InputStream for it
             try {
                  gangliaSocket = new Socket(gangliaHostName, gangliaTCPPort);
             } catch(ConnectException ce) {
                 logger.warn("Could not connect to ganglia, on host "+gangliaHostName+", port "+gangliaTCPPort+" with exception "+ce);
                 throw ce;
             }

            InputStream gangliaInputStream = gangliaSocket.getInputStream();

            StreamSource gangliaSource = new StreamSource(gangliaInputStream);

            // somewhere to put the result
            DOMResult result = new DOMResult();

            // could probably cache some of this in constructor rather than
            // running each iteration?
            xsltInputStream = new FileInputStream(xsltFilename);
            StreamSource xsltSource = new StreamSource(xsltInputStream);
            TransformerFactory transformerFactory= TransformerFactory.newInstance();
            Transformer transformer = transformerFactory.newTransformer(xsltSource);

            // transform!
            transformer.transform(gangliaSource,result);

            // extract Element
            Document resultNode = (Document)result.getNode();
            Element resultElement = resultNode.getDocumentElement();

            return resultElement;
        } 
        catch (Exception e) 
        {
            throw e;
        }
        finally
        {
            try {
                if (gangliaSocket != null) {
                    gangliaSocket.close();
                }
                if (xsltInputStream != null) {
                    xsltInputStream.close();
                }
            } catch (Exception e) { }
        }

    }
    
}

