/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.mds.webmds;
import java.lang.reflect.Method;
import org.globus.mds.webmds.ReflectionException;
import java.net.URL;

public class Reflection {

    public static Object call(Class theClass,
			      String methodName,
			      Object[] arguments,
			      Class[] argumentTypes)
	throws ReflectionException, java.lang.reflect.InvocationTargetException
    {
	if ((theClass == null) || (methodName == null)) {
	    throw new ReflectionException("Class or method name is null");
	}

	Method method = null;
	try {	
	    method = theClass.getMethod(methodName, argumentTypes);
	} catch (java.lang.NoSuchMethodException e) {
	    throw new ReflectionException(
		"Class " + theClass.getName() + " does not have method " +
		methodName,
		e);
	}

	return invokeMethod(theClass, method, arguments);
    }

    public static Object call(Class theClass,
			      String methodName,
			      Object[] arguments)
	throws ReflectionException, java.lang.reflect.InvocationTargetException
    {
	if ((theClass == null) || (methodName == null)) {
	    throw new ReflectionException("Class or method name is null");
	}
	Class[] argumentTypes;

	if (arguments == null) {
	    argumentTypes = null;
	} else {
	    argumentTypes = new Class[arguments.length];
	    for (int i = 0; i < arguments.length; i++) {
		if (arguments[i] == null) {
		    throw new ReflectionException(
			"Can't determine argument type from null argument");
		}
		argumentTypes[i] = arguments[i].getClass();
	    }
	}
	return call(theClass, methodName, arguments, argumentTypes);
    }

    public static Object call(String className,
			      String methodName,
			      Object[] arguments,
			      Class[] argumentTypes)
	throws ReflectionException, java.lang.reflect.InvocationTargetException
    {
	if (className == null) {
	    throw new ReflectionException("Null class name");
	}
	Class theClass;
	try {
	    theClass = Class.forName(className);
	} catch (java.lang.ClassNotFoundException e) {
	    throw new ReflectionException("Class " + className + " not found.",
					  e);
	}
	return call(theClass, methodName, arguments, argumentTypes);
    }

    public static Object call(String className,
			      String methodName,
			      Object[] arguments)
	throws ReflectionException, java.lang.reflect.InvocationTargetException
    {
	if (className == null) {
	    throw new ReflectionException("Null class name");
	}
	Class theClass;
	try {
	    theClass = Class.forName(className);
	} catch (java.lang.ClassNotFoundException e) {
	    throw new ReflectionException("Class " + className + " not found.",
					  e);
	}
	return call(theClass, methodName, arguments);
    }

    public static Object invokeMethod(Class theClass,
				      Method method,
				      Object[] arguments)
	throws ReflectionException, java.lang.reflect.InvocationTargetException
    {
	if (method == null) {
	    throw new ReflectionException("Attempt to invoke null method");
	}
	Object instance;
	try {
	    instance = theClass.newInstance();
	} catch (java.lang.IllegalAccessException e) {
	    throw new ReflectionException("error instantiating class " +
					  theClass.getName(),
					  e);
	} catch (java.lang.InstantiationException e) {
	    throw new ReflectionException("error instantiating class " +
					  theClass.getName(),
					  e);
	}

	try {
	    return method.invoke(instance, arguments);
	} catch (java.lang.IllegalAccessException e) {
	    throw new ReflectionException("error invoking method",
				      e);
// 	} catch (java.lang.reflect.InvocationTargetException e) {
// 	    throw new ReflectionException("error invoking method",
// 				      e);
	}
    }
}
