/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.mds.webmds;

import java.util.Vector;
import org.globus.mds.webmds.Parameter;
import org.globus.mds.webmds.ParameterMap;
import org.globus.mds.webmds.WebmdsConfigException;
import javax.servlet.ServletException;
import java.io.IOException;
import org.globus.mds.webmdsConfig.WebmdsConfigType;
import org.globus.mds.webmdsConfig.NamedParamType;
import java.lang.ClassNotFoundException;

public class WebmdsConfigEntry {
    public static final String SUITABLE_FOR_XSL_NAME="suitableForXSL";
    private Class sourceClass = null;
    private String name = null;
    private ParameterMap params = null;
    private static final String XML_SOURCE_PREFIX_NAME="xmlSource";
    private static final String XML_SOURCE_PARAM_SEGMENT="param";
    private static final String XML_SOURCE_CLASS_NAME_SEGMENT="class";
    private static final String CONTENT_TYPE_PARAM_NAME_SEGMENT="contentType";
    private WebmdsConfigType config;


    public WebmdsConfigEntry(String name, WebmdsConfigType config) {
	this.name = name;
	this.config = config;
    }

    public void init() throws WebmdsException {
	if (this.config == null) {
	    throw new WebmdsConfigException("null config");
	}
	if (this.config.getClassName() == null) {
	    throw new WebmdsException("null class name in " +
		name + " config entry");
	}
	try {
	    this.sourceClass = Class.forName(this.config.getClassName());
	} catch (ClassNotFoundException e) {
	    throw new WebmdsException(e);
	}
	this.params = new ParameterMap();
	NamedParamType[] configParams = this.config.getParameter();
	if (configParams != null) {
	    for (int i = 0; i < configParams.length; i++) {
		params.putParameter(
		    Parameter.createSingleValuedParameter(
			configParams[i].getName(),
			configParams[i].getValue()));
	    }
	}
    }
    
    public String getName() {
	return(this.name);
    }

    public Class getSourceClass() {
	return(this.sourceClass);
    }

    public ParameterMap getParams() {
	return(this.params);
    }

    public boolean isSuitableForXSL() {
	Boolean result = this.config.getSuitableForXSL();
	if (result == null) {
	    return(false);
	}
	return(result.booleanValue());
    }

    public static String getCustomParamPrefix(String name) {
	return(new String(getSourcePrefix(name) + "." +
			  XML_SOURCE_PARAM_SEGMENT));
    }
    public static String getContentTypeParamName(String name) {
	return(new String(getSourcePrefix(name) + "." +
			  CONTENT_TYPE_PARAM_NAME_SEGMENT));
    }

    public static String getSourcePrefix(String name) {
	return(new String(XML_SOURCE_PREFIX_NAME + "." + name));
    }



}
