/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */

package org.globus.mds.webmds.test;

import junit.framework.TestCase;
import junit.framework.Test;
import junit.framework.TestSuite;

import org.globus.mds.webmdsConfig.WebmdsConfigType;
import org.globus.mds.webmdsConfig.NamedParamType;
import java.io.PrintStream;
import java.io.FileOutputStream;
import java.io.FileInputStream;
import org.globus.wsrf.encoding.ObjectSerializer;
import org.globus.wsrf.encoding.ObjectDeserializer;

import javax.xml.parsers.DocumentBuilder; 
import javax.xml.parsers.DocumentBuilderFactory;  
import javax.xml.parsers.FactoryConfigurationError;  
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.namespace.QName;
import org.xml.sax.SAXException;  
import org.xml.sax.SAXParseException;
import org.xml.sax.InputSource;
import java.io.File;
import java.io.IOException;
import org.w3c.dom.Document;
import org.w3c.dom.DOMException;

public class ConfigTest extends TestCase {

    public ConfigTest(String name) {
	super(name);
    }

    public static Test suite() 
    {
        return new TestSuite(ConfigTest.class);
    }

    // put this in so test could be run outside of junit
    public static void main(String[] args) 
    {
        ConfigTest test = new ConfigTest("ConfigTest");
        try {
            test.testConfigTest();
        } catch (Exception e) {
            System.out.print(e);
            e.printStackTrace();
        }
    }

    public void testConfigTest() throws Exception {
	WebmdsConfigType myConfig = new WebmdsConfigType();
	myConfig.setDescription("testDescription");
	myConfig.setClassName("myclass");
	NamedParamType[] params = new NamedParamType[2];
	params[0] = makeNamedParam("param1", "value1");
	params[1] = makeNamedParam("param2", "value2");
	myConfig.setParameter(params);
	QName name = new QName("http://mds.globus.org/webmdsConfig.wsdl",
			       "WebmdsConfig");
	String serialized = ObjectSerializer.toString(myConfig, name);
	PrintStream outfile = new PrintStream(new FileOutputStream("./test.in"));
	outfile.print(serialized);
	outfile.close();

	FileInputStream infile = new FileInputStream("./test.in");
	InputSource source = new InputSource(infile);
	WebmdsConfigType newConfig =
	    (WebmdsConfigType)ObjectDeserializer.deserialize(
		source, WebmdsConfigType.class);
	testEquals(myConfig, newConfig);
    }

    public void testEquals(WebmdsConfigType myConfig, WebmdsConfigType newConfig) {
	assertNotNull(newConfig);
	assertEquals(myConfig.getDescription(), newConfig.getDescription());
	assertEquals(myConfig.getClassName(), newConfig.getClassName());
	NamedParamType[] params = myConfig.getParameter();
	NamedParamType[] newParams = newConfig.getParameter();
	assertNotNull(newParams);
	assertEquals(params.length, newParams.length);
	for (int i = 0; i < params.length; i++) {
	    assertNotNull(newParams[i]);
	    assertEquals(params[i].getName(), newParams[i].getName());
	    assertEquals(params[i].getValue(), newParams[i].getValue());
	}
    }

    public NamedParamType makeNamedParam(String name,
					 String value) {
	NamedParamType param = new NamedParamType();
	param.setName(name);
	param.setValue(value);
	return(param);
    }
    
}
