/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */

package org.globus.mds.webmds.test;

import junit.framework.TestCase;
import junit.framework.Test;
import junit.framework.TestSuite;

import org.globus.mds.webmds.Parameter;
import org.globus.mds.webmds.ParameterMap;

public class ParameterMapTest extends TestCase {

    public ParameterMapTest(String name) {
	super(name);
    }

    public static Test suite() 
    {
        return new TestSuite(ParameterMapTest.class);
    }

    // put this in so test could be run outside of junit
    public static void main(String[] args) 
    {
        ParameterMapTest test = new ParameterMapTest("ParameterMapTest");
        try {
            test.testParameterMapTest();
        } catch (Exception e) {
            System.out.print(e);
            e.printStackTrace();
        }
    }

    public void testParameterMapTest() throws Exception {
	ParameterMap paramMap = new ParameterMap();
	Parameter param1 =
	    Parameter.createSingleValuedParameter("abc.def", "ghi");
	paramMap.putParameter(param1);

	Parameter param2 = Parameter.createSingleValuedParameter("foo.x", "x1");
	paramMap.putParameter(param2);

	Parameter param3 = Parameter.createSingleValuedParameter("foo.y", "y1");
	paramMap.putParameter(param3);

	Parameter param4 =
	    Parameter.createSingleValuedParameter("foosball", "hello");
	paramMap.putParameter(param4);

	checkContains(paramMap, param1);
	checkContains(paramMap, param2);
	checkContains(paramMap, param3);
	checkContains(paramMap, param4);

	ParameterMap newParamMap = paramMap.prefixedSubMap(new String("foo"));
	assertEquals(2, newParamMap.size());
	checkNotContains(newParamMap, param1);
	checkNotContains(newParamMap, param2);
	checkNotContains(newParamMap, param3);
	checkNotContains(newParamMap, param4);

	String defString = new String("def");
	String xString = new String("x");
	String yString = new String("y");
	String foosballString = new String("foosball");

	checkNotContainsKey(newParamMap, defString);
	checkContainsKey(newParamMap, xString);
	checkContainsKey(newParamMap, yString);
	checkNotContainsKey(newParamMap, foosballString);

	newParamMap = paramMap.prefixedSubMap(new String("abc"));
	assertEquals(1, newParamMap.size());
	checkContainsKey(newParamMap, defString);
	checkNotContainsKey(newParamMap, xString);
	checkNotContainsKey(newParamMap, yString);
	checkNotContainsKey(newParamMap, foosballString);

	newParamMap = paramMap.prefixedSubMap(new String("xyz"));
	assertNull(newParamMap);

	newParamMap = paramMap.prefixedSubMap(null);
	assertNull(newParamMap);
    }

    private void checkContains(ParameterMap paramMap, Parameter param) {
	String name = param.getName();
	assertTrue(paramMap.containsKey(name));
	Parameter testParam = (Parameter)paramMap.get(name);
	assertEquals(name, testParam.getName());
    }

    private void checkNotContains(ParameterMap paramMap, Parameter param) {
	String name = param.getName();
	assertFalse(paramMap.containsKey(name));
	Parameter testParam = (Parameter)paramMap.get(name);
	assertNull(testParam);
    }

    private void checkContainsKey(ParameterMap paramMap, String key) {
	assertTrue(paramMap.containsKey(key));
	Parameter testParam = (Parameter)paramMap.get(key);
	assertEquals(key, testParam.getName());
    }

    private void checkNotContainsKey(ParameterMap paramMap, String key) {
	assertFalse(paramMap.containsKey(key));
	Parameter testParam = (Parameter)paramMap.get(key);
	assertNull(testParam);
    }

}
