/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */

package org.globus.mds.webmds.test;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import com.meterware.httpunit.WebClient;
import com.meterware.httpunit.WebConversation;
import com.meterware.httpunit.WebRequest;
import com.meterware.httpunit.WebResponse;
import com.meterware.httpunit.WebTable;
import com.meterware.httpunit.GetMethodWebRequest;
import com.meterware.servletunit.ServletRunner;
import com.meterware.servletunit.ServletUnitClient;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.MalformedURLException;

import java.net.URL;

import org.globus.mds.webmds.WebmdsServlet;
import org.globus.mds.webmds.test.ServletTestFramework;

public class SimpleServletTest extends ServletTestFramework {

    private ServletUnitClient simClient = null;
    private WebConversation httpClient = null;
    private String urlString = null;

    public SimpleServletTest (String s) {
	super(s);
    }

    public static Test suite() 
    {
        return new TestSuite(SimpleServletTest.class);
    }

    public void testWithServletRunner() throws Exception {
	System.out.println("Running " + this.getClass().getName() +
			   " tests with servlet simulator.");
	GetMethodWebRequest gr = makeSRGetMethodWebRequest("testfileinfo");
	doTestRaw(this.getDefaultSimClient(), gr);
    }

    public void testWithHttp() throws Exception {
	if (this.getDefaultHttpClient() != null) {
	    System.out.println("Running " + this.getClass().getName() +
			       " tests against server at " +
			       this.getServletURL());
	    GetMethodWebRequest gr =
		makeGetMethodWebRequest(this.getServletURL(), "testfileinfo");
	    doTestRaw(this.getDefaultHttpClient(), gr);
	} else {
	    System.out.println("No " + SERVLET_URL_PROPERTY_NAME +
			       " property specified; skipping " +
			       this.getClass().getName() + " http tests");
	}
    }

    private void doTestRaw(WebClient client, GetMethodWebRequest request)
    throws Exception {
	/*
	 * It would be nicer to read the testfileinfo config file rather
	 * than having the test/sample.xml path hardwired in.  But this will
	 * only cause a problem if someone changes the testfileinfo config
	 * to read a different file.
	 */
	File sourceFile = new File(testFileDir(), "sample.xml");
	WebResponse response = runFormTest(client, request, CONTENT_TYPE_XML);
	assertResultsMatchFile(sourceFile, response);
    }

}
