/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.mds.webmds.xmlSources.file;
import java.util.Vector;
import org.globus.mds.webmds.Parameter;
import org.globus.mds.webmds.ParameterException;
import org.globus.mds.webmds.WebmdsXmlSource;
import org.globus.mds.webmds.WebmdsXmlSourceException;
import org.globus.mds.webmds.WebmdsServlet;
import org.globus.mds.webmds.WebmdsUtil;
import org.globus.mds.webmds.ParameterMap;
import java.io.IOException;
import java.io.FileReader;
import java.io.File;
import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;


/**
 * FileXmlSource -- reads an xml document from a file.
 * This uses one configuration parameter from the servlet config file:
 * <dl>
 * <dt><b>file</b>
 * (specified as <b>xmlSource.</b><i>sourceName</i>.<b>param.file</b> in
 * the servlet configuration)
 * <dd>The name of the file to read</dd>
 * </dl>
 */


public class FileXmlSource implements WebmdsXmlSource
{
    private ParameterMap initParameters = null;

    private final String FILE_PARAM_NAME = "file";

    private String requestInfoString = null;

    private WebmdsUtil webmdsUtil = null;

    /**
     * Get an xml Source object corresponding to the contents of an XML file.
     * @param name nickname of the xml source.
     * @param initParams initialization parameters.  This must include a
     * "file" parameter with the path name to the file.
     * @param requestParams parameters from the http request.  Ignored.
     */
    public Source getXmlSource(String name,
			       ParameterMap initParams,
			       ParameterMap requestParams)
	throws WebmdsXmlSourceException, IOException {
	Parameter fileParam = null;
	Parameter baseDirParam = null;
	if (initParams == null) {
	    throw new WebmdsXmlSourceException("No configuration for '" +
					       name +
					       "'");
	}
	baseDirParam = (Parameter)initParams.get(WebmdsServlet.BASE_DIRECTORY_NAME);
	String directory = null;
	if (baseDirParam != null) {
	    try {
		directory = baseDirParam.getSingleValue();
	    } catch (ParameterException e) {
		throw new WebmdsXmlSourceException(
		    "wrong number of values for " +
		    WebmdsServlet.BASE_DIRECTORY_NAME);
	    }
	}
	fileParam = (Parameter)initParams.get(FILE_PARAM_NAME);
	if (fileParam == null) {
	    throw new WebmdsXmlSourceException("No " + FILE_PARAM_NAME +
				      " configured for '" +
				      name + "'");
	}

	String fileName = null;

	try {
	    fileName = fileParam.getSingleValue();
	} catch (ParameterException e) {
	    throw new WebmdsXmlSourceException("Error getting " +
					       FILE_PARAM_NAME +
					       " parameter value",
					       e);
	}

	File file = this.webmdsUtil.newFile(fileName);

	if (! file.canRead()) {
	    throw new WebmdsXmlSourceException("Can't read xml source file " +
					       file.getAbsolutePath());
	}
	
	this.requestInfoString = fileName;
	return(new StreamSource(file));
    }

    public void init(WebmdsUtil webmdsUtil) {
	this.webmdsUtil = webmdsUtil;
    }

    private String getFileParamName() {
	return(this.FILE_PARAM_NAME);
    }

    /**
     * Returns the name of the requested file.
     */
    public String getRequestInfo() {
	return(requestInfoString);
    }
}
