/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */

package org.globus.mds.webmds.xmlSources.resourceProperties;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Document;
import org.w3c.dom.DOMException;
import org.w3c.dom.NamedNodeMap;
import java.io.IOException;
import java.io.FileReader;
import java.io.File;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.FactoryConfigurationError;  
import javax.xml.parsers.ParserConfigurationException;
import org.xml.sax.SAXException;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import org.apache.xalan.transformer.TransformerIdentityImpl;
import javax.xml.transform.TransformerException;

public class BuildClientConfig {

    public BuildClientConfig() {
    }

    public static void main(String argv[]) {
	String Usage = "Usage:  BuildClientConfig infile outfile";
	Document doc = null;
	String inFileName = argv[0];
	String outFileName = argv[1];
	if (argv.length != 2) {
	    System.err.println(Usage);
	    System.exit(1);
	}
	File inFile = new File(inFileName);
	File outFile = new File(outFileName);
	try {
	    doc = readSource(inFile);
	} catch (SAXException e) {
	    System.err.println("Error reading " + inFileName + ": " +
			       e.getMessage());
	    System.exit(1);
	} catch (ParserConfigurationException e) {
	    System.err.println("Error reading " + inFileName + ": " +
			       e.getMessage());
	    System.exit(1);
	} catch (java.io.IOException e) {
	    System.err.println("Error reading " + inFileName + ": " +
			       e.getMessage());
	    System.exit(1);
	}
	trim(doc, doc);
	doc.normalize();
	removeExtraWhiteSpace(doc);
	try {
	    printResult(doc, outFile);
	} catch (TransformerException e) {
	    System.err.println("Error formatting results and writing " +
			       outFileName + ": " +
			       e.getMessage());
	    System.exit(1);
	}
    }

    public static Document readSource(File file)
	throws SAXException, ParserConfigurationException, java.io.IOException {
	Document doc = null;
	DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
	DocumentBuilder builder = factory.newDocumentBuilder();
	return(builder.parse(file));
    }

    public static void printResult(Document doc, File outFile)
	throws TransformerException {
	Transformer transformer = new TransformerIdentityImpl();
	Source source = new DOMSource(doc);
	StreamResult result = new StreamResult(outFile);
	transformer.transform(source, result);
    }

    public static void trim(Node node, Node parent) {
	String[] goodPrefixes = {
	    "java:org.apache.axis.",
	    "java:org.globus.mds.",
	    "java:org.globus.ws.sc.",
	    "java:org.globus.ws.trust.",
	    "java:org.oasis.wsn.",
	    "java:org.oasis.wsrf.faults.",
	    "java:org.oasis.wsrf.properties.",
	    "java:java."
	};

	boolean keep = true;
	String value = null;
	if (node.getNodeType() == Node.ELEMENT_NODE) {
	    if (node.getNodeName() == "typeMapping") {
		keep = false;
		NamedNodeMap attributes = node.getAttributes();
		Node typeAttr = attributes.getNamedItem("type");
		value = typeAttr.getNodeValue();
		if (value != null) {
		    for (int i = 0; (!keep && (i < goodPrefixes.length)); i++) {
			if (value.startsWith(goodPrefixes[i])) {
			    keep = true;
			}
		    }
		}
	    }
	}
	Node child = null;
	Node nextChild = null;
	for (child = node.getFirstChild(); child != null; )
	{
	    nextChild = child.getNextSibling();
	    trim(child, node);
	    child = nextChild;
	}

	if (! keep) {
	    parent.removeChild(node);
	}
    }

    public static void removeExtraWhiteSpace(Node node) {
	if (node.getNodeType() == Node.TEXT_NODE) {
	    String value = node.getNodeValue();
	    if (value.matches("^\\s*\\n\\s*\\n\\s*$")) {
		node.setNodeValue("\n");
	    }
	}
	for (Node child = node.getFirstChild(); child != null; child = child.getNextSibling())
	{
	    removeExtraWhiteSpace(child);
	}
    }
}
