/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */

package org.globus.mds.webmds.xmlSources.resourceProperties;

import javax.xml.namespace.QName;
import org.globus.mds.webmds.WebmdsException;
import org.oasis.wsrf.properties.GetResourcePropertyResponse;
import java.net.URL;
import org.apache.axis.client.Call;
import org.apache.axis.client.Service;
import javax.xml.namespace.QName;
import javax.xml.rpc.ParameterMode;
import org.apache.axis.description.OperationDesc;
import org.apache.axis.encoding.ser.BeanSerializerFactory;
import org.apache.axis.encoding.ser.BeanDeserializerFactory;
import org.apache.axis.description.ParameterDesc;
import org.w3c.dom.Node;
import javax.xml.rpc.ServiceException;
import java.net.MalformedURLException;
import java.rmi.RemoteException;
import org.globus.mds.webmds.WebmdsConfigException;
import org.globus.mds.webmds.WebmdsXmlSourceException;
import org.apache.axis.message.MessageElement;
import org.oasis.wsrf.properties.QueryResourceProperties_PortType;
import org.oasis.wsrf.properties.QueryResourceProperties_Element;
import org.oasis.wsrf.properties.QueryResourcePropertiesResponse;
import org.apache.axis.message.addressing.EndpointReferenceType;
import org.oasis.wsrf.properties.QueryExpressionType;
import org.globus.wsrf.WSRFConstants;
import org.oasis.wsrf.properties.WSResourcePropertiesServiceAddressingLocator;
import org.oasis.wsrf.properties.WSResourcePropertiesServiceLocator;
import org.oasis.wsrf.properties.GetResourceProperty;
import org.apache.axis.types.URI.MalformedURIException;
import org.apache.axis.message.addressing.EndpointReferenceType;
import org.globus.wsrf.impl.security.authentication.Constants;
import javax.xml.rpc.Stub;
import org.globus.axis.gsi.GSIConstants;
import org.globus.axis.util.Util;
import org.globus.wsrf.impl.security.authorization.NoAuthorization;

public class ResourcePropertyHelper {
    static final String RP_NS = new String("http://docs.oasis-open.org/wsrf/2004/06/wsrf-WS-ResourceProperties-1.2-draft-01.xsd");
    static String GET_RP_STRING = "GetResourceProperty";
    static final QName GET_RP_QNAME = new QName(RP_NS,
						GET_RP_STRING);
    static final QName RP_RESPONSE_QNAME =
        new QName(RP_NS, ">GetResourcePropertyResponse");

    static final QName QNAME_QNAME =
        new QName("http://www.w3.org/2001/XMLSchema", "QName");

    static final QName INVALID_RP_QNAME_FAULT_QNAME =
       new QName(RP_NS, "InvalidResourcePropertyQNameFault");

    static final String INVALID_RP_QNAME_FAULT_CLASS_NAME =
       new String(
	   "org.oasis.wsrf.properties.InvalidResourcePropertyQNameFaultType");

    static final QName INVALID_RP_QNAME_FAULT_TYPE_QNAME =
        new QName(RP_NS, "InvalidResourcePropertyQNameFaultType");

    static final QName RESOURCE_UNKNOWN_FAULT_QNAME =
       new QName(RP_NS, "ResourceUnknownFault");

    static final String RESOURCE_UNKNOWN_FAULT_CLASS_NAME =
       new String("org.oasis.wsrf.properties.ResourceUnknownFaultType");

    static final QName RESOURCE_UNKNOWN_FAULT_TYPE_QNAME =
        new QName(RP_NS, "ResourceUnknownFaultType");

    static {
        Util.registerTransport();
    }

    public static Node queryResourceProperties(EndpointReferenceType endpoint)
        throws ResourcePropertySourceException {
        String dialect = WSRFConstants.XPATH_1_DIALECT;
        String expression = "/";

        WSResourcePropertiesServiceAddressingLocator locator =
            new WSResourcePropertiesServiceAddressingLocator();

	QueryExpressionType query = new QueryExpressionType();

	try {
	    query.setDialect(dialect);
	} catch (MalformedURIException e) {
	    throw new ResourcePropertySourceException(e);
	}

	query.setValue(expression);
	
	QueryResourceProperties_PortType port;

	try {
	    port = locator.getQueryResourcePropertiesPort(endpoint);
	} catch (ServiceException e) {
	    throw new ResourcePropertySourceException(e);
	}
	
	noSecurity((Stub)port);

	QueryResourceProperties_Element request
	    = new QueryResourceProperties_Element();
	request.setQueryExpression(query);
	
	QueryResourcePropertiesResponse response;

	try {
	    response = port.queryResourceProperties(request);
	} catch (RemoteException e) {
	    throw new ResourcePropertySourceException(e);
	}

	MessageElement messageElements[] = response.get_any();
	if (messageElements == null) {
	    return(null);
	}
	if (messageElements.length > 1) {
	    throw new ResourcePropertySourceException(
		"Resource property query returned " +
		Integer.toString(messageElements.length) +
		" elements; I only know how to deal with one");
	}
	Node node;
	try {
	    node = messageElements[0].getAsDocument();
	} catch (Exception e) {
	    throw new ResourcePropertySourceException("Error parsing message element",
					       e);
	}
	return(node);
	
    }

    public static Node getResourceProperty(EndpointReferenceType endpoint,
					   QName rpName)
	throws ResourcePropertySourceException {

	GetResourceProperty port;

        WSResourcePropertiesServiceAddressingLocator locator =
            new WSResourcePropertiesServiceAddressingLocator();

	try {
	    port = locator.getGetResourcePropertyPort(endpoint);
	} catch (ServiceException e) {
	    throw new ResourcePropertySourceException(e);
	}

	noSecurity((Stub)port);

	GetResourcePropertyResponse response;

	try {
	    response = port.getResourceProperty(rpName);
	} catch (RemoteException e) {
	    throw new ResourcePropertySourceException(
		"Error getting resource property; " +
		"endpoint was '" + endpoint +
		"', name was '" + rpName.toString(),
		e);
	}
	MessageElement[] messageElements = response.get_any();
	if (messageElements == null) {
	    return(null);
	}
	if (messageElements.length > 1) {
	    throw new ResourcePropertySourceException(
		"Get resource property returned " +
		Integer.toString(messageElements.length) +
		" elements; I only know how to deal with one");
	}
	Node node;
	try {
	    node = messageElements[0].getAsDocument();
	} catch (Exception e) {
	    throw new ResourcePropertySourceException("Error parsing message element",
					       e);
	}
	return(node);
    }

    private static void noSecurity(Stub stub) {
	stub._setProperty(Constants.GSI_ANONYMOUS, Boolean.TRUE);
	stub._setProperty(Constants.AUTHORIZATION,
			  NoAuthorization.getInstance());
	stub._setProperty(GSIConstants.GSI_AUTHORIZATION,
                    org.globus.gsi.gssapi.auth.NoAuthorization.getInstance());
    }
}
