/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */

package org.globus.mds.webmds.xmlSources.resourceProperties.test;

import org.globus.mds.webmds.test.ServletTestFramework;
import com.meterware.httpunit.WebClient;
import com.meterware.httpunit.WebConversation;
import com.meterware.httpunit.WebRequest;
import com.meterware.httpunit.WebResponse;
import com.meterware.httpunit.WebTable;
import com.meterware.httpunit.GetMethodWebRequest;
import com.meterware.servletunit.ServletRunner;
import com.meterware.servletunit.ServletUnitClient;
import org.globus.mds.webmds.WebmdsServlet;
import org.globus.mds.webmds.test.ServletTestFramework;
import junit.framework.TestCase;
import junit.framework.Test;
import junit.framework.TestSuite;


public class ResourcePropertyServletTest extends ServletTestFramework {
    private ServletUnitClient simClient = null;
    private WebConversation httpClient = null;
    private String openQuerySource = "openEndedQuery";
    private String openRPSource = "openEndedRP";
    private String rpNamespace = "http://mds.globus.org/metadata/2005/02";
    private String rpName = "ServiceMetaDataInfo";
    private String secureURL = null;
    private String insecureURL = null;

    public ResourcePropertyServletTest (String s) {
	super(s);
    }

    public static Test suite() 
    {
        return new TestSuite(ResourcePropertyServletTest.class);
    }

    public void setUp() throws Exception {
	super.setUp();
	this.secureURL = findPropertyValue(Constants.SECURE_SERVICE_PREFIX_PROPERTY_NAME) + "/DefaultIndexService";
	this.insecureURL = findPropertyValue(Constants.INSECURE_SERVICE_PREFIX_PROPERTY_NAME) + "/DefaultIndexService";
	System.out.println("Tests will use Globus servers at " +
			   this.secureURL +
			   " and " +
			   this.insecureURL);
    }

    public void testWithServletRunner() throws Exception {
	System.out.println("Running " + this.getClass().getName() +
			   " tests with servlet simulator.");
	// Test that open-ended query works.
	GetMethodWebRequest gr = makeSRGetMethodWebRequest(this.openQuerySource);
	doQueryTest(this.getDefaultSimClient(), gr, this.secureURL);
	gr = makeSRGetMethodWebRequest(this.openQuerySource);
	doQueryTest(this.getDefaultSimClient(), gr, this.insecureURL);

	// Test that open-ended resource property request works.
	gr = makeSRGetMethodWebRequest(this.openRPSource);
	doGetRPTest(this.getDefaultSimClient(), gr, this.secureURL);
	gr = makeSRGetMethodWebRequest(this.openRPSource);
	doGetRPTest(this.getDefaultSimClient(), gr, this.insecureURL);

    }

    public void testWithHttp() throws Exception {
	if (this.getDefaultHttpClient() == null) {
	    System.out.println("No " + SERVLET_URL_PROPERTY_NAME +
			       " property specified; skipping " +
			       this.getClass().getName() + " http tests");
	    return;
	}
	System.out.println("Running " + this.getClass().getName() +
			   " tests against server at " +
			   this.getServletURL());

	// Test that open-ended query works.
	GetMethodWebRequest gr =
	    makeGetMethodWebRequest(this.getServletURL(), this.openQuerySource);
	doQueryTest(this.getDefaultHttpClient(), gr, this.secureURL);
	gr =
	    makeGetMethodWebRequest(this.getServletURL(), this.openQuerySource);
	doQueryTest(this.getDefaultHttpClient(), gr, this.insecureURL);

	// Test that open-ended resource property request works.
	gr =
	    makeGetMethodWebRequest(this.getServletURL(), this.openRPSource);
	doGetRPTest(this.getDefaultHttpClient(), gr, this.secureURL);
	gr =
	    makeGetMethodWebRequest(this.getServletURL(), this.openRPSource);
	doGetRPTest(this.getDefaultHttpClient(), gr, this.insecureURL);
    }

    public void doQueryTest(WebClient client, GetMethodWebRequest request, String url)
    throws Exception {
	request.setParameter(Constants.QUERY_ENDPOINT_FORM_ARG, url);
	WebResponse response = runFormTest(client, request, CONTENT_TYPE_XML);
	assertNotNull(response.getText());
    }

    public void doGetRPTest(WebClient client, GetMethodWebRequest request, String url)
    throws Exception {
	request.setParameter(Constants.GETRP_ENDPOINT_FORM_ARG, url);
	request.setParameter(Constants.GETRP_RP_NAMESPACE_FORM_ARG,
			     this.rpNamespace);
	request.setParameter(Constants.GETRP_RP_NAME_FORM_ARG, this.rpName);
	WebResponse response = runFormTest(client, request, CONTENT_TYPE_XML);
	assertNotNull(response.getText());
    }
}
