#include "globus_common.h"
#include "version.h"
#include "RendezvousFactoryService_client.h"

typedef int (*test_case)(void);
#define SIZEOF_ARRAY(x) sizeof(x) / sizeof(x[0])

static const char BINARY_DATA_1[] = {1, 2, 3};
static char * endpoint = NULL;

static
int 
rendezvous_test_init(
    RendezvousFactoryService_client_handle_t *
                                        handle,
    wsrendezvous_registerInputType **   registerInput)
{
    int                                 rc;
    globus_result_t                     result;

    rc = globus_module_activate(GLOBUS_COMMON_MODULE);
    if (rc != GLOBUS_SUCCESS)
    {
        goto out;
    }
    rc = globus_module_activate(RENDEZVOUSFACTORYSERVICE_MODULE);
    if (rc != GLOBUS_SUCCESS)
    {
        goto deactivate_common_out;
    }

    result = RendezvousFactoryService_client_init(
            handle,
            NULL,
            NULL);
    if (result != GLOBUS_SUCCESS)
    {
        rc = 1;

        goto deactivate_rendezvous_out;
    }

    result = wsrendezvous_registerInputType_init(registerInput);
    if (result != GLOBUS_SUCCESS)
    {
        rc = 1;

        goto destroy_client_out;
    }
    (*registerInput)->data.value = globus_libc_malloc(sizeof(BINARY_DATA_1));
    (*registerInput)->data.length = sizeof(BINARY_DATA_1);
    if ((*registerInput)->data.value == NULL)
    {
        goto destroy_register_input_out;
    }
    memcpy((*registerInput)->data.value, BINARY_DATA_1, sizeof BINARY_DATA_1);

    return 0;

destroy_register_input_out:
    wsrendezvous_registerInputType_destroy(*registerInput);
destroy_client_out:
    RendezvousFactoryService_client_destroy(*handle);
deactivate_rendezvous_out:
    globus_module_deactivate(RENDEZVOUSFACTORYSERVICE_MODULE);
deactivate_common_out:
    globus_module_deactivate(GLOBUS_COMMON_MODULE);
out:
    return rc;
}

static
int
rendezvous_test_destroy(
    RendezvousFactoryService_client_handle_t
                                        handle,
    wsrendezvous_registerInputType *    registerInput)
{
    int                                 rc = 0;

    RendezvousFactoryService_client_destroy(handle);
    wsrendezvous_registerInputType_destroy(registerInput);
    globus_module_deactivate(RENDEZVOUSFACTORYSERVICE_MODULE);
    globus_module_deactivate(GLOBUS_COMMON_MODULE);

    return rc;
}

static
int
testRegistrationWhenCapacityIsZero(void)
{
    int                                 rc;
    globus_result_t                     result;
    RendezvousFactoryService_client_handle_t
                                        handle;
    wsrendezvous_createSyncType *       createSync;
    wsrendezvous_createSyncResponseType *
                                        createSyncResponse;
    int                                 fault_type = 0;
    xsd_any *                           fault = NULL;
    wsrendezvous_registerInputType *    registerInput;
    wsrendezvous_registerResponseType * registerResponse;

    rc = rendezvous_test_init(&handle, &registerInput);

    if (rc != GLOBUS_SUCCESS)
    {
        goto out;
    }

    result = wsrendezvous_createSyncType_init(&createSync);
    if (result != GLOBUS_SUCCESS)
    {
        rc = 1;

        goto destroy_out;
    }
    createSync->Capacity = 0;

    result = RendezvousFactoryPortType_createSync(
        handle,
        endpoint,
        createSync,
        &createSyncResponse,
        (RendezvousFactoryPortType_createSync_fault_t *) &fault_type,
        &fault);
    if (result != GLOBUS_SUCCESS || fault != NULL)
    {
        rc = 1;

        goto destroy_createSync_out;
    }

    registerInput->rank = 0;

    result = RendezvousFactoryPortType_register_epr(
        handle,
        &createSyncResponse->EndpointReference,
        registerInput,
        &registerResponse,
        (RendezvousFactoryPortType_register_fault_t *) &fault_type,
        &fault);

    if (result != GLOBUS_SUCCESS)
    {
        if (fault_type !=
                RENDEZVOUSFACTORYPORTTYPE_REGISTER_CAPACITYREACHEDFAULT)
        {
            rc = 1;

            goto destroy_createSyncResponse_out;
        }
    }
destroy_createSyncResponse_out:
    wsrendezvous_createSyncResponseType_destroy(createSyncResponse);
destroy_createSync_out:
    wsrendezvous_createSyncType_destroy(createSync);
destroy_out:
    rendezvous_test_destroy(handle, registerInput);
out:
    return rc;
}

static
int
testRegistrationWithDesiredRank(void)
{
    int                                 rc;
    globus_result_t                     result;
    RendezvousFactoryService_client_handle_t
                                        handle;
    wsrendezvous_createSyncType *       createSync;
    wsrendezvous_createSyncResponseType *
                                        createSyncResponse;
    int                                 fault_type = 0;
    xsd_any *                           fault = NULL;
    wsrendezvous_registerInputType *    registerInput;
    wsrendezvous_registerResponseType * registerResponse;

    rc = rendezvous_test_init(&handle, &registerInput);

    if (rc != GLOBUS_SUCCESS)
    {
        goto out;
    }

    result = wsrendezvous_createSyncType_init(&createSync);
    if (result != GLOBUS_SUCCESS)
    {
        rc = 1;

        goto destroy_out;
    }
    createSync->Capacity = 10;

    result = RendezvousFactoryPortType_createSync(
        handle,
        endpoint,
        createSync,
        &createSyncResponse,
        (RendezvousFactoryPortType_createSync_fault_t *) &fault_type,
        &fault);
    if (result != GLOBUS_SUCCESS || fault != NULL)
    {
        rc = 1;

        goto destroy_createSync_out;
    }

    registerInput->rank = 0;

    result = RendezvousFactoryPortType_register_epr(
        handle,
        &createSyncResponse->EndpointReference,
        registerInput,
        &registerResponse,
        (RendezvousFactoryPortType_register_fault_t *) &fault_type,
        &fault);
    if (result != GLOBUS_SUCCESS ||
            registerInput->rank != registerResponse->rank)
    {
        rc = 1;
        goto destroy_createSyncResponse_out;
    }
    wsrendezvous_registerResponseType_destroy(registerResponse);

    registerInput->rank = 1;
    result = RendezvousFactoryPortType_register_epr(
        handle,
        &createSyncResponse->EndpointReference,
        registerInput,
        &registerResponse,
        (RendezvousFactoryPortType_register_fault_t *) &fault_type,
        &fault);
    if (result != GLOBUS_SUCCESS ||
            registerInput->rank != registerResponse->rank)
    {
        rc = 1;
        goto destroy_createSyncResponse_out;
    }
    wsrendezvous_registerResponseType_destroy(registerResponse);

    registerInput->rank = 3;
    result = RendezvousFactoryPortType_register_epr(
        handle,
        &createSyncResponse->EndpointReference,
        registerInput,
        &registerResponse,
        (RendezvousFactoryPortType_register_fault_t *) &fault_type,
        &fault);

    if (result != GLOBUS_SUCCESS ||
            registerInput->rank != registerResponse->rank)
    {
        rc = 1;
        goto destroy_createSyncResponse_out;
    }
    wsrendezvous_registerResponseType_destroy(registerResponse);

    registerInput->rank = 5;
    result = RendezvousFactoryPortType_register_epr(
        handle,
        &createSyncResponse->EndpointReference,
        registerInput,
        &registerResponse,
        (RendezvousFactoryPortType_register_fault_t *) &fault_type,
        &fault);

    if (result != GLOBUS_SUCCESS ||
            registerInput->rank != registerResponse->rank)
    {
        rc = 1;
        goto destroy_createSyncResponse_out;
    }
    wsrendezvous_registerResponseType_destroy(registerResponse);

    registerInput->rank = -1;
    result = RendezvousFactoryPortType_register_epr(
        handle,
        &createSyncResponse->EndpointReference,
        registerInput,
        &registerResponse,
        (RendezvousFactoryPortType_register_fault_t *) &fault_type,
        &fault);

    registerInput->rank = 4; /* why do we expect 4? */
    if (result != GLOBUS_SUCCESS ||
            registerInput->rank != registerResponse->rank)
    {
        rc = 1;
        goto destroy_createSyncResponse_out;
    }
    wsrendezvous_registerResponseType_destroy(registerResponse);

    registerInput->rank = -1;
    result = RendezvousFactoryPortType_register_epr(
        handle,
        &createSyncResponse->EndpointReference,
        registerInput,
        &registerResponse,
        (RendezvousFactoryPortType_register_fault_t *) &fault_type,
        &fault);

    registerInput->rank = 2; /* why do we expect 2? */
    if (result != GLOBUS_SUCCESS ||
            registerInput->rank != registerResponse->rank)
    {
        rc = 1;
        goto destroy_createSyncResponse_out;
    }
    wsrendezvous_registerResponseType_destroy(registerResponse);

    registerInput->rank = -1;
    result = RendezvousFactoryPortType_register_epr(
        handle,
        &createSyncResponse->EndpointReference,
        registerInput,
        &registerResponse,
        (RendezvousFactoryPortType_register_fault_t *) &fault_type,
        &fault);

    registerInput->rank = 6;
    if (result != GLOBUS_SUCCESS ||
            registerInput->rank != registerResponse->rank)
    {
        rc = 1;
        goto destroy_createSyncResponse_out;
    }
    wsrendezvous_registerResponseType_destroy(registerResponse);
destroy_createSyncResponse_out:
    wsrendezvous_createSyncResponseType_destroy(createSyncResponse);
destroy_createSync_out:
    wsrendezvous_createSyncType_destroy(createSync);
destroy_out:
    rendezvous_test_destroy(handle, registerInput);
out:
    return rc;
}

static
int
testRegistrationWithTakenRank(void)
{
    int                                 rc;
    globus_result_t                     result;
    RendezvousFactoryService_client_handle_t
                                        handle;
    wsrendezvous_createSyncType *       createSync;
    wsrendezvous_createSyncResponseType *
                                        createSyncResponse;
    int                                 fault_type = 0;
    xsd_any *                           fault = NULL;
    wsrendezvous_registerInputType *    registerInput;
    wsrendezvous_registerResponseType * registerResponse;

    rc = rendezvous_test_init(&handle, &registerInput);

    if (rc != GLOBUS_SUCCESS)
    {
        goto out;
    }

    result = wsrendezvous_createSyncType_init(&createSync);
    if (result != GLOBUS_SUCCESS)
    {
        rc = 1;

        goto destroy_out;
    }
    createSync->Capacity = 10;

    result = RendezvousFactoryPortType_createSync(
        handle,
        endpoint,
        createSync,
        &createSyncResponse,
        (RendezvousFactoryPortType_createSync_fault_t *) &fault_type,
        &fault);
    if (result != GLOBUS_SUCCESS || fault != NULL)
    {
        rc = 1;

        goto destroy_createSync_out;
    }

    registerInput->rank = 0;

    result = RendezvousFactoryPortType_register_epr(
        handle,
        &createSyncResponse->EndpointReference,
        registerInput,
        &registerResponse,
        (RendezvousFactoryPortType_register_fault_t *) &fault_type,
        &fault);
    if (result != GLOBUS_SUCCESS ||
            registerInput->rank != registerResponse->rank)
    {
        rc = 1;
        goto destroy_createSyncResponse_out;
    }
    wsrendezvous_registerResponseType_destroy(registerResponse);
    registerResponse = NULL;

    registerInput->rank = 0;
    result = RendezvousFactoryPortType_register_epr(
        handle,
        &createSyncResponse->EndpointReference,
        registerInput,
        &registerResponse,
        (RendezvousFactoryPortType_register_fault_t *) &fault_type,
        &fault);
    if (registerResponse != NULL)
    {
        wsrendezvous_registerResponseType_destroy(registerResponse);
        registerResponse = NULL;
    }

    if (fault_type != RENDEZVOUSFACTORYPORTTYPE_REGISTER_RANKTAKENFAULT)
    {
        rc = 1;
        goto destroy_createSyncResponse_out;
    }

destroy_createSyncResponse_out:
    wsrendezvous_createSyncResponseType_destroy(createSyncResponse);
destroy_createSync_out:
    wsrendezvous_createSyncType_destroy(createSync);
destroy_out:
    rendezvous_test_destroy(handle, registerInput);
out:
    return rc;
}

int main(int argc, char * argv[])
{
    int i;
    int rc;
    int failed = 0;
    test_case tests[] = {
        testRegistrationWhenCapacityIsZero,
        testRegistrationWithDesiredRank,
        testRegistrationWithTakenRank
    };

    endpoint = argv[1];
    printf("1..%d\n", (int) SIZEOF_ARRAY(tests)); 

    for (i = 0; i < SIZEOF_ARRAY(tests); i++)
    {
        rc = (tests[i])();

        if (rc != 0)
        {
            failed++;
            printf("not ");
        }
        printf("ok\n");
    }

    return failed;
}
/* main() */
