/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.replica.replicator.impl;

import commonj.work.Work;
import commonj.work.WorkManager;

import org.globus.replica.replicator.ReplicationController;
import org.globus.replica.replicator.Replication;
import org.globus.replica.replicator.ReplicationStatus;
import org.globus.replica.replicator.ActivateException;

/**
 * IMPLEMENTATION SPECIFIC.
 */
class ReplicationControllerImpl implements ReplicationController {

    // FIELDS //
    ReplicationManagerImpl manager;

    // per ReplicationController interface
    ReplicationImpl replication;

    // CONSTRUCTORS //
    ReplicationControllerImpl(ReplicationManagerImpl m, ReplicationImpl r) {
        this.manager = m;
        this.replication = r;
    }

    // METHODS //

    // per ReplicationController interface
    public Replication getReplication() { return this.replication; }

    public void activate() throws IllegalStateException, ActivateException {
        synchronized (this.replication.statusLock) {
            if (this.replication.status != ReplicationStatus.PENDING)
                throw new IllegalStateException();
            try {
                WorkManager wm = this.manager.getWorkManager();
                DiscoverWork work = new DiscoverWork(this.manager, this.replication);
                wm.schedule(work, new ReplicationWorkListener(this.replication));
                this.replication.status = ReplicationStatus.ACTIVE;
                this.replication.stage = ReplicationImpl.DISCOVER;
                this.replication.setDirty();
            } catch (commonj.work.WorkException wex) {
                throw new ActivateException(wex);
            }
        }
        this.replication.notifyStatusObservers();
    }

    public void terminate() throws IllegalStateException {
        synchronized (this.replication.statusLock) {
            if (this.replication.status != ReplicationStatus.ACTIVE)
                throw new IllegalStateException();
            // Can only set the flag to indicate that the workers should terminate
            this.replication.status = ReplicationStatus.TERMINATING;
            this.replication.setDirty();
        }
        this.replication.notifyStatusObservers();
    }

    public void suspend() throws IllegalStateException {
        synchronized (this.replication.statusLock) {
            if (this.replication.status != ReplicationStatus.ACTIVE)
                throw new IllegalStateException();
            // Can only set the flag to indicate that the workers should terminate
            this.replication.status = ReplicationStatus.SUSPENDING;
            this.replication.setDirty();
        }
        this.replication.notifyStatusObservers();
    }

    public void resume() throws IllegalStateException, ActivateException {
        synchronized (this.replication.statusLock) {
            if (this.replication.status != ReplicationStatus.SUSPENDED)
                throw new IllegalStateException();
            try {
                WorkManager wm = this.manager.getWorkManager();
                Work work = null;
                if (this.replication.stage == ReplicationImpl.DISCOVER)
                    work = new DiscoverWork(this.manager, this.replication);
                else if (this.replication.stage == ReplicationImpl.TRANSFER)
                    work = new TransferWork(this.manager, this.replication);
                else if (this.replication.stage == ReplicationImpl.REGISTER)
                    work = new RegisterWork(this.manager, this.replication);
                else
                    throw new IllegalStateException();
                wm.schedule(work, new ReplicationWorkListener(this.replication));
                this.replication.status = ReplicationStatus.ACTIVE;
                this.replication.setDirty();
            } catch (commonj.work.WorkException wex) {
                throw new ActivateException(wex);
            }
        }
        this.replication.notifyStatusObservers();
    }
}
