/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.replica.replicator.impl;

import java.util.Iterator;
import java.util.Map;
import java.util.List;

import javax.naming.InitialContext;

import org.apache.axis.message.addressing.EndpointReferenceType;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.globus.util.I18n;

import commonj.work.WorkManager;
import commonj.work.WorkItem;

import org.globus.delegation.DelegationException;

import org.globus.replica.replicator.ReplicationManager;
import org.globus.replica.replicator.ReplicationFactory;
import org.globus.replica.replicator.ReplicationController;
import org.globus.replica.replicator.Replication;
import org.globus.replica.replicator.ReplicationItem;
import org.globus.replica.replicator.ReplicationStatus;
import org.globus.replica.replicator.ReplicationCount;
import org.globus.replica.replicator.ReplicationOptions;
import org.globus.replica.replicator.TransferOptions;
import org.globus.replica.replicator.ReplicationCredentials;
import org.globus.replica.replicator.CreateException;
import org.globus.replica.replicator.NotFoundException;

import org.globus.replica.replicator.util.DelegatedCredential;
import org.globus.replica.replicator.util.ReplicaCatalogFilter;
import org.globus.replica.replicator.util.SourceSelector;

/**
 * IMPLEMENTATION SPECIFIC.
 */
public /* TODO: final*/ class ReplicationManagerImpl implements ReplicationManager,
        org.globus.wsrf.jndi.Initializable {

    // FIELDS //

    private static Log logger =
        LogFactory.getLog(ReplicationManagerImpl.class.getName());
    private static I18n i18n =
        I18n.getI18n(ReplicationConstants.MESSAGE_RESOURCE_BUNDLE,
        ReplicationManagerImpl.class.getClassLoader());

    // bean fields
    public String wmLocation;
    public String catalogFilterLocation;
    public String sourceSelectorLocation;
    public String defaultIndexUrl;
    public String defaultRegistrationUrl;
    public String defaultReliableTransferUrl;
    public int locateRetryMax;
    public int locateRetryWait;
    public boolean locateRetryBackoff;
    public int rliMaxPerQuery;
    public int rliQueryTimeout;
    public int lrcMaxPerQuery;
    public int lrcQueryTimeout;
    public int transferRequestMax;
    public int transferWaitInt;
    public int transferWaitMax;
    public boolean transferWaitBackoff;
    public int defaultTransferConcurrency;
    public int defaultTransferTcpBufferSize;
    public int defaultTransferParallelStreams;
    public int registerRetryMax;
    public int registerRetryWait;
    public boolean registerRetryBackoff;
    public int registerMaxPerUpdate;
    public int registerUpdateTimeout;

    // other fields
    private FactoryImpl factory;
    private WorkManager wm;
    private ReplicaCatalogFilter catalogFilter;
    private SourceSelector sourceSelector;


    // CONSTRUCTORS //

    public ReplicationManagerImpl() {
        // empty constructor per Bean requirements (I think)
    }


    // METHODS //

    // per Bean requirements

    public String getWmLocation() { return this.wmLocation; }
    public void setWmLocation(String s) { this.wmLocation = s; }

    public String getCatalogFilterLocation() { return this.catalogFilterLocation; }
    public void setCatalogFilterLocation(String s) { this.catalogFilterLocation = s; }

    public String getSourceSelectorLocation() { return this.sourceSelectorLocation; }
    public void setSourceSelectorLocation(String s) { this.sourceSelectorLocation = s; }

    public String getDefaultIndexUrl() { return this.defaultIndexUrl; }
    public void setDefaultIndexUrl(String s) { this.defaultIndexUrl = s; }

    public String getDefaultRegistrationUrl() { return this.defaultRegistrationUrl; }
    public void setDefaultRegistrationUrl(String s) { this.defaultRegistrationUrl = s; }

    public String getDefaultReliableTransferUrl() { return this.defaultReliableTransferUrl; }
    public void setDefaultReliableTransferUrl(String s) { this.defaultReliableTransferUrl = s; }

    public int getLocateRetryMax() { return this.locateRetryMax; }
    public void setLocateRetryMax(int n) { this.locateRetryMax = n; }

    public int getLocateRetryWait() { return this.locateRetryWait; }
    public void setLocateRetryWait(int n) { this.locateRetryWait = n; }

    public boolean getLocateRetryBackoff() { return this.locateRetryBackoff; }
    public void setLocateRetryBackoff(boolean b) { this.locateRetryBackoff = b; }

    public int getRliMaxPerQuery() { return this.rliMaxPerQuery; }
    public void setRliMaxPerQuery(int n) { this.rliMaxPerQuery = n; }

    public int getRliQueryTimeout() { return this.rliQueryTimeout; }
    public void setRliQueryTimeout(int n) { this.rliQueryTimeout = n; }

    public int getLrcMaxPerQuery() { return this.lrcMaxPerQuery; }
    public void setLrcMaxPerQuery(int n) { this.lrcMaxPerQuery = n; }

    public int getLrcQueryTimeout() { return this.lrcQueryTimeout; }
    public void setLrcQueryTimeout(int n) { this.lrcQueryTimeout = n; }

    public int getTransferRequestMax() { return this.transferRequestMax; }
    public void setTransferRequestMax(int n) { this.transferRequestMax = n; }

    public int getTransferWaitInt() { return this.transferWaitInt; }
    public void setTransferWaitInt(int n) { this.transferWaitInt = n; }

    public int getTransferWaitMax() { return this.transferWaitMax; }
    public void setTransferWaitMax(int n) { this.transferWaitMax = n; }

    public boolean getTransferWaitBackoff() { return this.transferWaitBackoff; }
    public void setTransferWaitBackoff(boolean b) { this.transferWaitBackoff = b; }

    public int getDefaultTransferConcurrency() { return this.defaultTransferConcurrency; }
    public void setDefaultTransferConcurrency(int n) { this.defaultTransferConcurrency = n; }

    public int getDefaultTransferTcpBufferSize() { return this.defaultTransferTcpBufferSize; }
    public void setDefaultTransferTcpBufferSize(int n) { this.defaultTransferTcpBufferSize = n; }

    public int getDefaultTransferParallelStreams() { return this.defaultTransferParallelStreams; }
    public void setDefaultTransferParallelStreams(int n) { this.defaultTransferParallelStreams = n; }

    public int getRegisterRetryMax() { return this.registerRetryMax; }
    public void setRegisterRetryMax(int n) { this.registerRetryMax = n; }

    public int getRegisterRetryWait() { return this.registerRetryWait; }
    public void setRegisterRetryWait(int n) { this.registerRetryWait = n; }

    public boolean getRegisterRetryBackoff() { return this.registerRetryBackoff; }
    public void setRegisterRetryBackoff(boolean b) { this.registerRetryBackoff = b; }

    public int getRegisterMaxPerUpdate() { return this.registerMaxPerUpdate; }
    public void setRegisterMaxPerUpdate(int n) { this.registerMaxPerUpdate = n; }

    public int getRegisterUpdateTimeout() { return this.registerUpdateTimeout; }
    public void setRegisterUpdateTimeout(int n) { this.registerUpdateTimeout = n; }

    // per Initializable interface

    /**
     * INTERNAL USE ONLY.
     * Initializes the FactoryImpl and looks up the WorkManager.
     */
    public void initialize() throws Exception {
        // Dump bean settings
        if (logger.isTraceEnabled()) {
            logger.debug("Initializing ReplicationManagerImpl");
            _dumpBeanFields();
        }

        // Create factory
        this.factory = new FactoryImpl();

        // Locate a WorkManager
        try {
            InitialContext ctx = new InitialContext();
            this.wm = (WorkManager) ctx.lookup(this.wmLocation);
        }
        catch (Exception e) {
            logger.error("Failed to initialize Replication Manager: " + e.getMessage());
            throw e;
        }
    }

    private void _dumpBeanFields() {
        logger.debug("wmLocation: " + this.wmLocation);
        logger.debug("catalogFilterLocation: " + this.catalogFilterLocation);
        logger.debug("sourceSelectorLocation: " + this.sourceSelectorLocation);
        logger.debug("defaultIndexUrl: " + this.defaultIndexUrl);
        logger.debug("defaultRegistrationUrl: " + this.defaultRegistrationUrl);
        logger.debug("defaultReliableTransferUrl: " + this.defaultReliableTransferUrl);
        logger.debug("locateRetryMax: " + this.locateRetryMax);
        logger.debug("locateRetryWait: " + this.locateRetryWait);
        logger.debug("locateRetryBackoff: " + this.locateRetryBackoff);
        logger.debug("rliMaxPerQuery: " + this.rliMaxPerQuery);
        logger.debug("rliQueryTimeout: " + this.rliQueryTimeout);
        logger.debug("lrcMaxPerQuery: " + this.lrcMaxPerQuery);
        logger.debug("lrcQueryTimeout: " + this.lrcQueryTimeout);
        logger.debug("transferRequestMax: " + this.transferRequestMax);
        logger.debug("transferWaitInt: " + this.transferWaitInt);
        logger.debug("transferWaitBackoff: " + this.transferWaitBackoff);
        logger.debug("defaultTransferConcurrency: " + this.defaultTransferConcurrency);
        logger.debug("defaultTransferTcpBufferSize: " + this.defaultTransferTcpBufferSize);
        logger.debug("defaultTransferParallelStreams: " + this.defaultTransferParallelStreams);
        logger.debug("registerRetryMax: " + this.registerRetryMax);
        logger.debug("registerRetryWait: " + this.registerRetryWait);
        logger.debug("registerRetryBackoff: " + this.registerRetryBackoff);
        logger.debug("registerMaxPerUpdate: " + this.registerMaxPerUpdate);
        logger.debug("registerUpdateTimeout: " + this.registerUpdateTimeout);
    }

    // per ReplicationManager interface

    public ReplicationFactory getFactory() {
        return this.factory;
    }

    public ReplicationController createController(Replication r) {
        if (r == null || !(r instanceof ReplicationImpl))
            throw new IllegalArgumentException();

        return new ReplicationControllerImpl(this, (ReplicationImpl) r);
    }


    // Non-Inteface related methods

    /**
     * INTERNAL USE ONLY.
     * @return The instance of the WorkManager.
     */
    WorkManager getWorkManager() { return this.wm; }

    /**
     * INTERNAL USE ONLY.
     * This method starts a replication and activates its worker.
     */
    void startReplication(ReplicationImpl r) {
        synchronized (r.statusLock) {
            if (r.status != ReplicationStatus.PENDING)
                throw new IllegalStateException();
            try {
                DiscoverWork work = new DiscoverWork(this, r);
                wm.schedule(work);
                r.status = ReplicationStatus.ACTIVE;
                // persist state?
            } catch (commonj.work.WorkException wex) {
                logger.error("Failed to start replicaiton work: " + wex.getMessage());
            }
            r.notifyStatusObservers();
        }
    }

    /**
     * INTERNAL USE ONLY.
     * This method returns an instnace of the ReplicationCatalogFilter as 
     * configured for this ReplicationManagerImpl.
     */
    ReplicaCatalogFilter lookupReplicationCatalogFilter()
            throws javax.naming.NamingException {
        if (this.catalogFilter == null && this.catalogFilterLocation != null 
                && this.catalogFilterLocation.length() > 0) {
            InitialContext ctx = new InitialContext();
            this.catalogFilter = (ReplicaCatalogFilter) ctx.lookup(this.catalogFilterLocation);
        }
        return this.catalogFilter;
    }

    SourceSelector lookupSourceSelector()
            throws javax.naming.NamingException {
        if (this.sourceSelector == null && this.sourceSelectorLocation != null 
                && this.sourceSelectorLocation.length() > 0) {
            InitialContext ctx = new InitialContext();
            this.sourceSelector = (SourceSelector) ctx.lookup(this.sourceSelectorLocation);
        }
        return this.sourceSelector;
    }



    // INNER CLASSES //


    /**
     * IMPLEMENTATION SPECIFIC.
     * The factory implementation. Currently stores instances in memory.
     */
    class FactoryImpl implements ReplicationFactory {

        Object lock;
        Map active;
        Map cached;

        FactoryImpl() {
            lock = new Object();
            active = new java.util.HashMap(); // dont' need to gaurantee order
            cached = new java.util.HashMap(); // ditto
        }

        /**
         * IMPLEMENTATION SPECIFIC.
         * Creates a Replication and places it on the pending list.
         */
        public Replication createReplication(
                ReplicationCredentials credentials,
                List items,
                ReplicationOptions options)
                throws CreateException {

            logger.trace("createReplication");

            // Validate credentials
            if (credentials == null || 
                    !(credentials instanceof DelegatedCredential))
                throw new CreateException("Invalid credentials");

            // Check if options exist
            if (options == null) {
                options = createReplicationOptions(null, null, null, null);
            }

            // Cast options to replications, because we are reusing
            // if the user had any options to set it will already be 
            // in the passed in options object.
            ReplicationImpl replication = (ReplicationImpl) options;
            replication.credentials = (DelegatedCredential) credentials;

            // Validate and initialize items related fields
            if (items == null || items.isEmpty())
                throw new CreateException("Empty replication items");
            replication.addItems(items);
            replication.total = items.size();

            // Generate key
            replication.key = new Integer(replication.hashCode());

            // Add to cache
            synchronized (this.lock) {
                this.cached.put(replication.key, replication);
            }

            return replication;
        }

        /**
         * IMPLEMENTATION SPECIFIC.
         */
        public ReplicationOptions createReplicationOptions(
                String indexUrl,
                String reliableTransferUrl,
                String registrationUrl,
                TransferOptions transferOptions)
            throws
                CreateException {
            logger.trace("createReplicationOptions");

            // Use ReplicationImpl for replication options
            ReplicationImpl options = new ReplicationImpl();

            if (indexUrl == null || indexUrl.length() == 0) {
                options.indexUrl = defaultIndexUrl;
            }
            else {
                options.indexUrl = indexUrl;
            }

            if (reliableTransferUrl == null || reliableTransferUrl.length() == 0) {
                options.reliableTransferUrl = defaultReliableTransferUrl;
            }
            else {
                options.reliableTransferUrl = reliableTransferUrl;
            }

            if (registrationUrl == null || registrationUrl.length() == 0) {
                options.registrationUrl = defaultRegistrationUrl;
            }
            else {
                options.registrationUrl = registrationUrl;
            }

            // Process transfer options and use defaults if needed
            if (transferOptions != null) {
                options.transferOptions = (TransferOptionsImpl) transferOptions;
            }
            else {
                options.transferOptions = new TransferOptionsImpl();
            }
            // Check key values
            if (options.transferOptions.transferConcurrency == null) {
                options.transferOptions.transferConcurrency = new Integer(defaultTransferConcurrency);
            }
            if (options.transferOptions.parallelStreams == null) {
                options.transferOptions.parallelStreams = new Integer(defaultTransferParallelStreams);
            }
            if (options.transferOptions.tcpBufferSize == null) {
                options.transferOptions.tcpBufferSize = new Integer(defaultTransferTcpBufferSize);
            }
            return options;
        }

        /**
         * IMPLEMENTATION SPECIFIC.
         */
        public TransferOptions createTransferOptions(
                Integer transferConcurrency,
                Boolean binary,
                Integer blockSize,
                Integer tcpBufferSize,
                Integer parallelStreams,
                Boolean noTPT,
                Boolean dcau,
                String subjectName,
                String destinationSubjectName,
                String sourceSubjectName,
                String username)
            throws CreateException {

            return new TransferOptionsImpl(
                    transferConcurrency,
                    binary,
                    blockSize,
                    tcpBufferSize,
                    parallelStreams,
                    noTPT,
                    dcau,
                    subjectName,
                    destinationSubjectName,
                    sourceSubjectName,
                    username);
        }

        /**
         * IMPLEMENTATION SPECIFIC.
         */
        public ReplicationItem createReplicationItem(
                String uri,
                Integer priority,
                String[] destinationUris)
                throws CreateException {
            return new ReplicationItemImpl(uri, priority, destinationUris);
        }

        /**
         * IMPLEMENTATION SPECIFIC.
         */
        public void destroy( Object key ) throws NotFoundException,
                                                 IllegalStateException {
            logger.debug("destroy");

            // Change replication status
            ReplicationImpl r = (ReplicationImpl) this.find(key);
            if (r.getStatus() != ReplicationStatus.TERMINATED)
                throw new IllegalStateException();
            r.changeStatus(ReplicationStatus.DESTROYED);
            r.notifyStatusObservers();
            r.destroy();

            // Remove from factory collections
            Object activeRep = null;
            Object cachedRep = null;
            synchronized (this.lock) {
                if (this.active.containsKey(key))
                    activeRep = this.active.remove(key);
                if (this.cached.containsKey(key))
                    cachedRep = this.cached.remove(key);
            }
            if (activeRep == null && cachedRep == null)
                throw new NotFoundException();
        }

        /**
         * IMPLEMENTATION SPECIFIC.
         */
        public Replication find( Object key )
                throws NotFoundException {
            logger.debug("find");
            ReplicationImpl r;
            synchronized (this.lock) {
                r = (ReplicationImpl) this.active.get(key);
                if (r==null)
                    r = (ReplicationImpl) this.cached.get(key);
            }
            if (r==null)
                throw new NotFoundException();
            return r;
        }
    }
}
