/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.replica.replicator.impl;

import commonj.work.WorkEvent;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.globus.util.I18n;

import org.globus.replica.replicator.ReplicationStatus;

/**
 * INTERNAL USE ONLY.
 * Class implements <code>WorkListener</code> interfaces to listen
 * for replication work completion. This is primarily useful for
 * adjusting replication status when in between steady states, such
 * as when suspending or terminating.
 */
class ReplicationWorkListener implements commonj.work.WorkListener {

    // STATIC //

    private static Log logger =
        LogFactory.getLog(ReplicationWorkListener.class.getName());
    private static I18n i18n =
        I18n.getI18n(ReplicationConstants.MESSAGE_RESOURCE_BUNDLE,
                ReplicationWorkListener.class.getClassLoader());

    // FIELDS //

    ReplicationImpl replication;


    // CONSTRUCTORS //

    ReplicationWorkListener(ReplicationImpl r) {
        this.replication = r;
    }


    //  METHODS //

    public void workAccepted(WorkEvent we) {
        // Nothing to do here
    }

    public void workRejected(WorkEvent we) {
        if (logger.isDebugEnabled())
            logger.debug("Work rejected, key: " + this.replication.key);
        _adjustStatus();
    }

    public void workStarted(WorkEvent we) {
        // Nothing to do here
    }

    public void workCompleted(WorkEvent we) {
        if (logger.isDebugEnabled())
            logger.debug("Work completed, key: " + this.replication.key);
        _adjustStatus();
    }

    private void _adjustStatus() {
        boolean changed = false;
        synchronized(this.replication.statusLock) {
            if (this.replication.status == ReplicationStatus.SUSPENDING) {
                if (logger.isDebugEnabled())
                    logger.debug("Replication suspended, key: " + this.replication.key);
                this.replication.changeStatus(ReplicationStatus.SUSPENDED);
                changed = true;
            }
            if (this.replication.status == ReplicationStatus.TERMINATING) {
                if (logger.isDebugEnabled())
                    logger.debug("Replication terminated, key: " + this.replication.key);
                this.replication.changeStatus(ReplicationStatus.TERMINATED);
                changed = true;
            }
        }
        if (changed)
            this.replication.notifyStatusObservers();
    }
}
