/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.replica.replicator.service;

import java.util.Calendar;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.globus.util.I18n;
import org.globus.wsrf.impl.ResourceHomeImpl;
import org.globus.wsrf.impl.SimpleResourceKey;
import org.globus.wsrf.ResourceKey;

import org.globus.replica.replicator.generated.BaseRequestType;

/**
 * A <code>ResourceHome</code> implementation based on <code>ResourceHomeImpl</code>
 * for use with <code>ReplicatorResource</code> resources.
 */
public class ReplicatorHome extends ResourceHomeImpl {

    private static Log logger = LogFactory.getLog(ReplicatorHome.class.getName());
    private static I18n i18n = I18n.getI18n("org.globus.replica.replicator.errors",
            ReplicatorHome.class.getClassLoader());

    /**
     * Initializable interface method to initialize the resource home at
     * container startup.
     */
    public synchronized void initialize() throws Exception {
        super.initialize();
        logger.debug("initialize");
        // TODO initialization procedures go here

        /*
         * 
        try {
            PersistenceHelper.initialize();
        } catch ( Exception e ) {
            logger.error("Initialization failed: " + e.getMessage());
            throw e;
        }
         *
         */
    }

    /**
     * Create the <code>ReplicatorResource</code> resource.
     *
     * @param currentTime The current time.
     * @param terminationTime The initial termination time. Null value
     *      indicates no termination time is to be set for this resource.
     * @param request The request body.
     * @param userSubject The caller's user subject.
     * @param localUsername The caller's local username.
     * @return The resource key for the newly created replicator resource.
     * @exception Exception Indicates that in internal error occurred.
     */
    public ResourceKey create(
            Calendar currentTime,
            Calendar terminationTime,
            BaseRequestType request,
            String userSubject,
            String localUsername) throws Exception {

        logger.debug("create");
        // Create resource instance
        ReplicatorResource replicator = (ReplicatorResource) createNewInstance();
        replicator.create(currentTime, terminationTime, request, userSubject,
                localUsername);
        // Create resource key based on resource instance's ID
        ResourceKey key = new SimpleResourceKey(keyTypeName, replicator.getID());
        // Add resource to resource home
        add(key, replicator);
        return key;
    }
}
