/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.replica.replicator.util;

import java.util.Collection;

/**
 * This interface specifies methods for selecting sources. Users of
 * ReplicationManager may wish to implement custom logic for selecting
 * a source file from all available sources mapping to a given logical
 * file name. This selector, for instance, is invoked when creating
 * transfer requests, which permits the user to specify the desired
 * source to use in the transfer. Implementations of this interface
 * MUST be thread safe.
 */
public interface SourceSelector {
    /**
     * This method selects the desired source(s) from a collection of
     * all available source(s). <b>Note:</b> Implementaors <b>MAY modify</b>
     * the original collection.
     * @param sources A <code>Collection</code> of <code>String</code>s that
     * represent the URI of the source.
     * @return A <code>Collection</code> containing <code>String</code>s
     * that represent the URIs of sources. <b>IMPORTANT:</b> 
     * Implementations <b>MUST NOT</b> return a <code>null</code> collection.
     * If no sources match the user criteria, the appropriate return
     * value will be an empty <code>Collection</code> object. <b>Note:</b> At
     * this time, only the first source in the collection will be used.
     * @since 1.0
     */
    public Collection selectSources(Collection sources);
}
