/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.transfer.reliable.setup;

import java.util.Properties;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.FileNotFoundException;

import java.sql.Connection;
import java.sql.Statement;
import java.sql.SQLException;

import org.globus.transfer.reliable.service.database.RFTDatabaseSetup;
import org.globus.transfer.reliable.service.database.RFTDatabaseOptions;
import org.globus.transfer.reliable.service.database.RftDBException;

public class RFTDbBootstrap {
    //TODO: Read SQL from a file
    private static String requestSequence = "create sequence request_seq";
    private static String requestTable = " create table request ( " +
        " id int primary key default nextval('request_seq'), " +
        " concurrency int default 1, termination_time float, " +
        "proxy_loc text, username text, started boolean default false, " +
        "all_or_none boolean default false, maxattempts int default 1," +
        "delegated_epr text, user_subject text )";
    private static String transferSequence = "create sequence transfer_seq";

    private static String transferTable= "create table transfer (" 
        + "id int primary key default nextval('transfer_seq'), "
        + " request_id int default 0, source_url text not null,"
        + " dest_url text null,status int default 4,attempts int default 0,"
        + " user_name text default null, dcau boolean default false, "
        + " parallel_streams int default 1, tcp_buffer_size int default 0, "
        + " block_size integer default 16000, notpt boolean default false, "
        + " binary_mode boolean default true, source_subject text, " 
        + " dest_subject text, retry_time float default 0, size float default 0, "
        + " fault text, permissions int, ignore_file_perm_errors boolean "
        + " default false) ";
    private static String restartTable = "create table restart (" 
        + " transfer_id int, marker text )";
    private static String factoryTable = "create table factory ("
        + " total_transfers float, total_bytes float)";
    private static String index = " create index transfer_tindex on transfer"
        + " using btree(request_id, attempts)";
        

    public static void createTables(Connection c) throws SQLException {
        Statement st = c.createStatement();
        st.executeUpdate(requestSequence);
        st.executeUpdate(requestTable);
        st.executeUpdate(transferSequence);
        st.executeUpdate(transferTable);
        st.executeUpdate(restartTable);
        st.executeUpdate(factoryTable);
        st.executeUpdate(index);
        st.close();
        c.close();
    }
    public static void main(String args[]) {
        String userName = args[0]; 
        String password = args[1];
        String driverName = args[2];
        String connectionString = args[3];
        System.out.println( "Using driverName,userName,password, connectionString: ");
        System.out.println( driverName + " " + userName + " " + password 
                + " " + connectionString);
        /*String dbPropsFileName = args[0];
        if (dbPropsFileName == null) {
            System.err.println("Error: Need a properties file");
            System.exit(-1);
        }
        Properties dbProperties = new Properties();
        try {
            dbProperties.load(new FileInputStream(dbPropsFileName));
        } catch(FileNotFoundException fnfe) {
            System.err.println("Error: database properties file  " +
                         dbPropsFileName + " not found");
        } catch (IOException ioe) {
            System.err.println("Error loading file:" + dbPropsFileName 
                    + "\n" + ioe.getMessage());
            System.exit(-1);
        }*/
        RFTDatabaseOptions rftDbOptions = new RFTDatabaseOptions(
                driverName,userName,password,connectionString);
        try {
            RFTDatabaseSetup.setupDBConnection(rftDbOptions);
            createTables(RFTDatabaseSetup.getDBConnection());
        } catch (Exception dbEx) {
            System.err.println("Error initializing database " +
                    dbEx.getMessage());
            dbEx.printStackTrace();

            System.exit(-1);
        }
    }
}
