/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.transfer.reliable.service;

import org.globus.delegation.DelegationListener;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.globus.wsrf.Resource;
import org.globus.wsrf.ResourceKey;
import org.globus.wsrf.ResourceHome;
import org.globus.wsrf.ResourceException;

import org.globus.gsi.GlobusCredential;

import org.globus.util.I18n;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.globus.transfer.reliable.service.exception.RftException;
/**
 * CredentialRefreshListener
 */
public class CredentialRefreshListener implements DelegationListener {

    String listenerId = null;
    String home = null;
    ResourceKey key = null;
    private static I18n i18n =
        I18n.getI18n("org.globus.transfer.reliable.errors", 
                     CredentialRefreshListener.class.getClassLoader());
    private static Log logger = LogFactory
            .getLog(CredentialRefreshListener.class.getName());
    
    public CredentialRefreshListener(String home, ResourceKey key) {
        this.home = home;
        this.key = key;
    }

    /* (non-Javadoc)
     * @see org.globus.delegation.DelegationListener#setCredential(org.globus.gsi.GlobusCredential)
     */
    // Called on register and refresh.
    public void setCredential(GlobusCredential _credential) {

        try {
            Context initialContext = new InitialContext();
            ResourceHome resHome = (ResourceHome)initialContext.lookup(home);
            Resource resource = resHome.find(key);
            if (resource == null) {
                throw new RuntimeException(i18n
                                           .getMessage("resourceNull",
                                                       this.key.toString()));
            }
            ((ReliableFileTransferResource)resource)
                .setCredential(_credential);
        } catch (ResourceException exp) {
            logger.error(exp);
            throw new RuntimeException(i18n.getMessage("resourceRtrErr")
                                       + exp.getMessage());
        } catch (NamingException exp) {
            logger.error(exp);
            throw new RuntimeException(i18n.getMessage("resourceRtrErr")
                                       + exp.getMessage());
        } catch (RftException re) {
            throw new RuntimeException(i18n.getMessage("resourceRtrErr")
                                        + re.getMessage());
        }
    }

    public void setId(String id) {
        this.listenerId = id;
    }

    public String getId() {
        return this.listenerId;
    }

    public void credentialDeleted() {
        // FILLME: Called when delegated credential is destroyed. 
        // Add any cleanups, that maybe  required.
    }
}
