/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.transfer.reliable.service;

import java.util.Calendar;

import org.apache.axis.message.MessageElement;
import org.apache.axis.message.Text;
import org.apache.axis.message.addressing.EndpointReferenceType;
import org.apache.axis.utils.JavaUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.globus.rft.generated.RFTAuthenticationFaultType;
import org.globus.rft.generated.RFTAuthorizationFaultType;
import org.globus.rft.generated.RFTDatabaseFaultType;
import org.globus.rft.generated.RFTFaultResourcePropertyType;
import org.globus.rft.generated.RepeatedlyStartedFaultType;
import org.globus.rft.generated.TransferFaultType;
import org.globus.rft.generated.TransferTransientFaultType;
import org.globus.wsrf.ResourceKey;
import org.globus.wsrf.container.ServiceHost;
import org.globus.wsrf.impl.SimpleResourceKey;
import org.globus.wsrf.utils.AddressingUtils;
import org.globus.wsrf.utils.FaultHelper;
import org.oasis.wsrf.faults.BaseFaultType;
import org.oasis.wsrf.faults.BaseFaultTypeDescription;
import org.oasis.wsrf.faults.BaseFaultTypeErrorCode;

/**
 * FaultUtils
 */
public class FaultUtil {
    private static Log logger = LogFactory.getLog(
            FaultUtil.class.getName());

    /**
     * @param resourceKey
     * @param cause
     * @return <code> RFTFaultResourcePropertyType </code>
     */
    public static RFTFaultResourcePropertyType 
    createRepeatedlyStartedFault(
            int resourceKey, Exception cause) {
        RFTFaultResourcePropertyType faultRP = 
            new RFTFaultResourcePropertyType();
        String description = "Repeatedly Started fault";
        RepeatedlyStartedFaultType fault = (RepeatedlyStartedFaultType)
            makeFault(
                RepeatedlyStartedFaultType.class, resourceKey, 
                description, cause);
        faultRP.setRftRepeatedlyStartedFaultType(fault);

        return faultRP;
    }
    
    /**
     * @param resourceKey
     * @param cause
     * @return <code> RFTFaultResourcePropertyType </code>
     */
    public static RFTFaultResourcePropertyType createRFTDatabaseFault(
            int resourceKey, Exception cause) {
        RFTFaultResourcePropertyType faultRP = 
            new RFTFaultResourcePropertyType();
        String description = "RFT Database Fault";
        RFTDatabaseFaultType fault = (RFTDatabaseFaultType)
            makeFault(RFTDatabaseFaultType.class, resourceKey, 
                    description, cause);
        faultRP.setRftDatabaseFaultType(fault);
        return faultRP;
    }
    
    /**
     * @param resourceKey
     * @param cause
     * @return <code> RFTFaultResourcePropertyType </code>
     */
    public static RFTFaultResourcePropertyType createTransferTransientFault(
            int resourceKey, Exception cause) {
        RFTFaultResourcePropertyType faultRP =
            new RFTFaultResourcePropertyType();
        String description = "RFT Transient Fault";
        TransferTransientFaultType fault = (TransferTransientFaultType)
            makeFault(TransferTransientFaultType.class, resourceKey, 
                    description, cause);
        faultRP.setTransferTransientFaultType(fault);
        return faultRP;
    }
   /**
     * @param resourceKey
     * @param cause
     * @return <code> RFTFaultResourcePropertyType </code>
     */
    public static RFTFaultResourcePropertyType createTransferTransientFault(
            int resourceKey, 
            String description,
            Exception cause) {
        RFTFaultResourcePropertyType faultRP =
            new RFTFaultResourcePropertyType();
        TransferTransientFaultType fault = (TransferTransientFaultType)
            makeFault(TransferTransientFaultType.class, resourceKey, 
                    description, cause);
        faultRP.setTransferTransientFaultType(fault);
        return faultRP;
    }

    /**
     * @param resourceKey
     * @param cause
     * @return <code> RFTFaultResourcePropertyType </code>
     */
    public static RFTFaultResourcePropertyType createTransferFault(
            int resourceKey, 
            String description,
            Exception cause) {
        RFTFaultResourcePropertyType faultRP = 
            new RFTFaultResourcePropertyType();
        TransferFaultType fault = (TransferFaultType) makeFault(
                TransferFaultType.class, resourceKey, description , cause);
        faultRP.setRftTransferFaultType(fault);
        return faultRP;
    }

    /**
     * @param resourceKey
     * @param cause
     * @return <code> RFTFaultResourcePropertyType </code>
     */
    public static RFTFaultResourcePropertyType createTransferFault(
            int resourceKey, Exception cause) {
        RFTFaultResourcePropertyType faultRP = 
            new RFTFaultResourcePropertyType();
        String description = "RFT Transfer Fault"; 
        TransferFaultType fault = (TransferFaultType) makeFault(
                TransferFaultType.class, resourceKey, description , cause);
        faultRP.setRftTransferFaultType(fault);
        return faultRP;
    }
    /**
     * @param resourceKey
     * @param cause
     * @return <code> RFTFaultResourcePropertyType </code>
     */
    public static RFTFaultResourcePropertyType createAuthenticationFault (
            int resourceKey, Exception cause) {
        RFTFaultResourcePropertyType faultRP = 
            new RFTFaultResourcePropertyType();
        RFTAuthenticationFaultType fault =  (RFTAuthenticationFaultType)
            makeFault(RFTAuthenticationFaultType.class, resourceKey, 
                    null, cause);
        faultRP.setRftAuthenticationFaultType(fault);
        return faultRP;
    }
    
    /**
     * @param resourceKey
     * @param cause
     * @return <code> RFTFaultResourcePropertyType </code>
     */
    public static RFTFaultResourcePropertyType createAuthorizationFault (
            int resourceKey, Exception cause) {
        RFTFaultResourcePropertyType faultRP = 
            new RFTFaultResourcePropertyType();
        RFTAuthorizationFaultType fault =  (RFTAuthorizationFaultType)
            makeFault(RFTAuthorizationFaultType.class, resourceKey, 
                    null, cause);
        faultRP.setRftAuthorizationFaultType(fault);
        return faultRP;
    }
    /**
     * @param faultClass
     * @param requestId
     * @param description
     * @param cause
     * @return <code> BaseFaultType </code>
     */
    public static BaseFaultType makeFault(Class faultClass,
            int requestId, String description, Exception cause) {
        Calendar timestamp = Calendar.getInstance();
        SimpleResourceKey resourceKey = new SimpleResourceKey
            (RFTConstants.RFT_KEY_QNAME, Integer.toString(requestId));
        //originator
        EndpointReferenceType originator = getEndpointReference(resourceKey);

        //errorCode
        BaseFaultTypeErrorCode errorCode = new BaseFaultTypeErrorCode();
        if (cause != null) {
            MessageElement[] any = new MessageElement[] { 
                    new MessageElement(
                            new Text(JavaUtils.stackToString(cause))) 
                            };
            errorCode.set_any(any);
        }
        errorCode.setDialect(FaultHelper.STACK_TRACE);

        BaseFaultTypeDescription[] faultDescription = 
            new BaseFaultTypeDescription[] { 
                new BaseFaultTypeDescription(description) 
                };

        //faultCause
        BaseFaultType[] faultCause;
        if (cause != null) {
            faultCause = new BaseFaultType[] { 
                    FaultHelper.toBaseFault(cause) 
                    };
        } else {
            faultCause = new BaseFaultType[0];
        }

        BaseFaultType f = null;
        try {
            f = (BaseFaultType) faultClass.newInstance();
            f.setTimestamp(timestamp);
            f.setOriginator(originator);
            f.setErrorCode(errorCode);
            f.setDescription(faultDescription);
            f.setFaultCause(faultCause);
        } catch (Exception e) {
            logger.error(e);
        }

        return f;
    }

    private static EndpointReferenceType getEndpointReference(
            ResourceKey resourceKey) {
        try {
            EndpointReferenceType epr = AddressingUtils
                    .createEndpointReference(ServiceHost.getBaseURL()
                    + "ReliableFileTransferFactoryService", resourceKey);

            return epr;
        } catch (Exception e) {
            logger.error(e);
        }

        return null;
    }
}
