/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.transfer.reliable.service;

import org.apache.log4j.Logger;
import org.globus.ftp.ByteRangeList;
import org.globus.ftp.GridFTPRestartMarker;
import org.globus.ftp.Marker;
import org.globus.ftp.MarkerListener;
import org.globus.ftp.PerfMarker;
import org.globus.transfer.reliable.service.database.ReliableFileTransferDbAdapter;

/**
 * Listens for Performance and Restart Markers
 */
public class MyMarkerListener implements MarkerListener {

    private static Logger logger = Logger.getLogger(MyMarkerListener.class
            .getName());

    private ReliableFileTransferDbAdapter dbAdapter;

    private ByteRangeList list;

    private long size;

    private int transferid;

    /**
     * Constructor for the MyMarkerListener object
     */
    public MyMarkerListener() {
        this.list = new ByteRangeList();
    }

    /**
     * Constructor for the MyMarkerListener object
     * 
     * @param transferId
     * @param size
     */
    public MyMarkerListener(int transferId, long size) {
        try {
            this.dbAdapter = ReliableFileTransferDbAdapter.getDbAdapter();
            this.list = new ByteRangeList();
            this.size = size;
            this.transferid = transferId;
        } catch (Exception re) {
            logger.error("Error creating marker listener" + re.getMessage());
        }
    }

    /**
     * DOCUMENT ME!
     * 
     * @return DOCUMENT ME!
     */
    public ByteRangeList getByteRangeList() {

        return this.list;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.globus.ftp.MarkerListener#markerArrived(org.globus.ftp.Marker)
     */
    public void markerArrived(Marker m) {

        if (m instanceof GridFTPRestartMarker) {
            restartMarkerArrived((GridFTPRestartMarker) m);
        } else if (m instanceof PerfMarker) {
            perfMarkerArrived((PerfMarker) m);
        } else {
            if (logger.isDebugEnabled()) {
                logger.debug("Unknown format of restart marker");
            }
        }
    }

    /**
     * @param perfMarker
     */
    private void perfMarkerArrived(PerfMarker perfMarker) {
        if (logger.isDebugEnabled()) {
            logger.debug("--> perf marker arrived " + perfMarker.getTimeStamp());
        }
    }

    /**
     * @param marker
     *  
     */
    private void restartMarkerArrived(GridFTPRestartMarker marker) {

        try {
            this.list.merge(marker.toVector());

            String temp = this.list.toFtpCmdArgument();
            temp = temp.substring(temp.lastIndexOf("-") + 1);
            temp = "0-" + temp;
            if (logger.isDebugEnabled()) {
                logger.debug("Current transfer state: " 
                        + temp + " for transferId: "
                        + this.transferid);
            }
            this.dbAdapter.setRestartMarker(this.transferid, temp);
        } catch (Exception e) {
            logger.error("Exception in MarkerListener" + e.getMessage(), e);
        }
    }

    /**
     * Sets the size attribute of the MyMarkerListener object
     * 
     * @param size
     *            The new size value
     */
    public void setSize(long size) {
        this.size = size;
    }
}

