/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.transfer.reliable.service;

import javax.xml.namespace.QName;

/**
 * Constants for reuse
 */
public class RFTConstants {

    /**
     * Transfer finished
     */
    public static final  int STATUS_FINISHED = 0;

    /**
     * Transfer is being retried
     */
    public static final  int STATUS_RETRYING = 1;

    /**
     * Transfer Failed
     */
    public static final  int STATUS_FAILED = 2;

    /**
     * Transfer is Active
     */
    public static final  int STATUS_ACTIVE = 3;

    /**
     * Transfer is pending
     */
    public static final  int STATUS_PENDING = 4;

    /**
     * Transfer is cancelled
     */
    public static final int STATUS_CANCELLED = 5;

    /**
     * Expanding a directory URL
     */
    public static final int STATUS_EXPANDING = 6;

    /**
     * When a directory url is done expanding
     *  <code>STATUS_EXPANDING_DONE</code>
     */
    public static final int STATUS_EXPANDING_DONE = 7;

    /**
     * Marked for deletion <code>STATUS_DIRTY</code>
     */
    public static final int STATUS_DIRTY = 8;

    public static final int STATUS_DIRTY_DIR = 9;

    /**
    * Internal state
    */
    public static final int STATUS_CACHED = 10;

    /**
     * QName for RFT resource key
     */
    public static final QName RFT_KEY_QNAME = new QName(
            "http://www.globus.org/namespaces/2004/10/rft", "TransferKey");

    /**
     * QName for RFT Resource Property set
     */
    public static final QName RFT_RP_SET = new QName(
            "http://www.globus.org/namespaces/2004/10/rft", "RFT_RP_SET");

    /**
     * QName for OverallStatus Resource Property
     * For the resource
     */
    public static final QName OVERALL_STATUS_RESOURCE = new QName(
            "http://www.globus.org/namespaces/2004/10/rft", "OverallStatusProperty");

    /**
     * QName for TotalBytes Resource Property
     * For the resource
     */
    public static final QName TOTAL_BYTES_RESOURCE = new QName(
            "http://www.globus.org/namespaces/2004/10/rft", "TotalBytes");

    /**
     * QName for request status Resource Property
     * For  the resource
     */
    public static final QName REQUEST_STATUS_RESOURCE = new QName(
            "http://www.globus.org/namespaces/2004/10/rft", "RequestStatusProperty");
    
    /**
     * QName for total time taken by the resource
     */
    public static final QName TOTAL_TIME_RESOURCE = new QName(
            "http://www.globus.org/namespaces/2004/10/rft", "TotalTime");
    
    /**
     * Jndi look up for container's work manager
     */
    public static final String WORKMANAGER = 
        "java:comp/env/wm/ContainerWorkManager";

    /**
     * Jndi look up for database look up 
     * TODO: make this configurable
     */
    public static final String DS_LOOK_UP = 
        "ReliableFileTransferService/jdbc/postgres";

    /**
     * QName for number of active resources
     */
    public static final QName ACTIVE_RESOURCE_COUNT_FACTORY = new QName(
            "http://www.globus.org/namespaces/2004/10/rft",
            "ActiveResourceInstances");

    /**
     * QName for Total Transfer Count
     * Factory RP
     */
    public static final QName TOTAL_TRANSFER_COUNT_FACTORY = new QName(
            "http://www.globus.org/namespaces/2004/10/rft",
            "TotalNumberOfTransfers");

    /**
     * Factory RP
     * QName of <code>DELEGATION_ENDPOINT</code>
     */
    public static final QName DELEGATION_ENDPOINT_FACTORY = new QName(
            "http://www.globus.org/namespaces/2004/10/rft",
            "DelegationServiceEPR");
    
    /**
     * Factory RP
     *  <code>RFT_FACTORY_START_TIME</code>
     */
    public static final QName RFT_FACTORY_START_TIME = new QName(
            "http://www.globus.org/namespaces/2004/10/rft",
            "RFTFactoryStartTime");
    
    /**
     * Factory RP
     *  <code>TOTAL_ACTIVE_TRANSFERS</code>
     */
    public static final QName TOTAL_ACTIVE_TRANSFERS_FACTORY = new QName(
            "http://www.globus.org/namespaces/2004/10/rft",
            "TotalNumberOfActiveTransfers");
    
    /**
     * QName for TotalBytes Resource Property
     * For the factory
     */
    public static final QName TOTAL_BYTES_FACTORY = new QName(
            "http://www.globus.org/namespaces/2004/10/rft", 
            "TotalNumberOfBytesTransferred");

    /**
     *  <code>SERVICE_NAME</code>
     */
    public static final String SERVICE_NAME = "ReliableFileTransferService";

    /**
     *  <code>FACTORY_NAME</code>
     */
    public static final String FACTORY_NAME = "ReliableFileTransferFactoryService";


    /**
     * Path to client-security-config.xml on the disk
     */
    public static final String CLIENT_DESC = 
    "org/globus/transfer/reliable/client/client-security-config.xml";


    /**
     * Maximum number of retries
     */
    public static final String maxAttempts = "maxAttempts";

    /**
     * value <code>maxAttemptsValue</code>
     */
    public static final int maxAttemptsValue = 0;

}

