/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.transfer.reliable.service;

import java.util.Calendar;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.globus.rft.generated.BaseRequestType;
import 
    org.globus.transfer.reliable.service.database.ReliableFileTransferDbAdapter;
import org.globus.transfer.reliable.service.database.RftDBException;
import org.globus.util.I18n;

import org.globus.wsrf.impl.ResourceHomeImpl;
import org.globus.wsrf.impl.SimpleResourceKey;

/**
 * ReliableTransferHome.java
 *  
 */
public class ReliableFileTransferHome extends ResourceHomeImpl {

    private static Log logger = LogFactory
            .getLog(ReliableFileTransferHome.class.getName());
    private static I18n i18n = 
        I18n.getI18n("org.globus.transfer.reliable.errors",
                ReliableFileTransferHome.class.getClassLoader());
    
    ReliableFileTransferDbAdapter dbAdapter = ReliableFileTransferDbAdapter
            .getDbAdapter();

    /**
     * Constructor for the ReliableFileTransferHome object
     */
    public ReliableFileTransferHome() {
        //Empty constructor
    }

    /**
     * Called during recovery
     * 
     */
    public synchronized void initialize() {
        try {
            super.initialize();
            this.keyTypeName = RFTConstants.RFT_KEY_QNAME;
            List resourceKeyValueList = null;
            if (logger.isDebugEnabled()) {
                logger.debug("Loading the RFT home's list of resource keys");
            }
            resourceKeyValueList = new ReliableFileTransferResource()
                        .getStoredResourceKeyValues();
            if (resourceKeyValueList != null) {
                Iterator it = resourceKeyValueList.iterator();
                if (logger.isDebugEnabled()) {
                    logger.debug("Number of RFT resources to be recovered:"
                            + resourceKeyValueList.size());
                }
                while (it.hasNext()) {
                    Integer keyValue = (Integer) it.next();
                    if (keyValue != null) {
                        ReliableFileTransferResource res = 
                            new ReliableFileTransferResource();
                        res.load(getResourceKey(keyValue.intValue()));
                        this.add(getResourceKey(keyValue.intValue()), res);
                        this.dbAdapter = 
                            ReliableFileTransferDbAdapter.getDbAdapter();
                        res.setTerminationTime(this.dbAdapter
                                .getTerminationTime(keyValue.intValue()));
                    } else {
                        if (logger.isDebugEnabled()) {
                            logger.debug("no RFT Resources to recover");
                        }
                    }
                }
            }
        } catch (RftDBException ex) {
            logger.warn(i18n.getMessage("termErrorString")
                    + ex.getMessage());
            if (logger.isDebugEnabled()) {
                logger.debug(i18n.getMessage("termErrorString")
                        + ex.getMessage(), ex);
            }
        } catch (Exception re) {
            logger.warn(i18n.getMessage("termErrorString")
                    + re.getMessage());
            if (logger.isDebugEnabled()) {
                logger.debug(i18n.getMessage("termErrorString")
                        + re.getMessage(), re);
            }
        }
    }

    /**
     * Create the rft resource 
     * @param initialTerminationTime
     * @param currentTime
     * @param transferRequest
     * @param userSubject
     * @param localUserId
     * @return <code> SimpleResourceKey </code> 
     * @exception Exception
     */
    public synchronized SimpleResourceKey 
        create(Calendar initialTerminationTime,
            Calendar currentTime, BaseRequestType transferRequest,
            String userSubject, String localUserId) throws Exception {
        ReliableFileTransferResource res = new ReliableFileTransferResource(
                transferRequest, userSubject, localUserId,
                initialTerminationTime);
        SimpleResourceKey key = getResourceKey(res.getRequestId());
        if (logger.isDebugEnabled()) {
            logger.debug("Created a ReliableTransferResource with key "
                    + key.getValue());
        }

        res.initialize(ReliableFileTransferResource.
                createResourceBean(initialTerminationTime, currentTime),
                RFTConstants.RFT_RP_SET, key);
        res.store();
        this.add(key, res);
        res.processDelegatedCredential();
        return key;
    }

    /**
     * Gets the resourceKey attribute of the ReliableFileTransferHome class
     * 
     * @param requestId
     * @return The resourceKey value
     */
    public static SimpleResourceKey getResourceKey(int requestId) {
        return new SimpleResourceKey(RFTConstants.RFT_KEY_QNAME, Integer
                .toString(requestId));
    }
}

