/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.transfer.reliable.service;

import java.io.IOException;
import java.util.Vector;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.globus.ftp.GridFTPClient;
import org.globus.ftp.HostPort;
import org.globus.ftp.MlsxEntry;
import org.globus.ftp.exception.ServerException;
import org.globus.rft.generated.RFTOptionsType;
import org.globus.rft.generated.TransferStatusTypeEnumeration;
import org.globus.rft.generated.TransferType;
import org.globus.transfer.reliable.service.exception.RftException;
import org.globus.util.GlobusURL;
import org.globus.util.I18n;

import org.globus.transfer.reliable.service.database.ReliableFileTransferDbAdapter;
/**
 * This class should take the sourceUrl and destination Url from RftImpl of the
 * Class and return a Vector of corresponding Fully Qualified gsiftp urls
 * 
 */
public class URLExpander extends Thread {

    private GridFTPClient sourceHost, destinationHost;

    private String sourcePath, destinationPath;

    private Vector sourceUrlsEx;

    private GlobusURL sourceGlobusUrl, destinationGlobusUrl;

    private boolean done = false;

    private RFTOptionsType rftOptions = null;

    private boolean isDelete = false;

    private Vector dirVector = new Vector();

    private int requestId = 0;

    // this includes the url being expanded
    private int numberOfTransfers = 0;
    
    // This transferJob should include directory in source and dest
    private static Log logger = LogFactory.getLog(URLExpander.class.getName());

    private static I18n i18n = I18n.getI18n(
            "org.globus.transfer.reliable.errors", URLExpander.class
                    .getClassLoader());

    /**
     * Constructor for the URLExpander object
     * 
     * @param sourceHost
     *            source gridftp server
     * @param destinationHost
     *          destination gridftp server
     * @param sourceGlobusUrl
     *            sourceGlobusUrl
     * @param destinationGlobusUrl
     *            destination globus url.
     * @param rftOptions
     *            <code> RFTOptionsType </code>
     * @throws RftException
     */
    public URLExpander(GridFTPClient sourceHost, GridFTPClient destinationHost,
            GlobusURL sourceGlobusUrl, GlobusURL destinationGlobusUrl,
            RFTOptionsType rftOptions) throws RftException {
        this.sourceHost = sourceHost;
        this.destinationHost = destinationHost;
        this.sourceGlobusUrl = sourceGlobusUrl;
        this.destinationGlobusUrl = destinationGlobusUrl;
        this.sourcePath = "/" + sourceGlobusUrl.getPath();
        this.destinationPath = "/" + destinationGlobusUrl.getPath();
        if(logger.isDebugEnabled()) {
            logger.debug("Expanding: " + this.sourcePath + " " 
                    + this.destinationPath  );
        }
        this.rftOptions = rftOptions;
        this.sourceUrlsEx = new Vector();
        this.sourceUrlsEx.add(this.sourcePath);
        this.changeDir(this.destinationPath);
    }

    /**
     * This is to be used from the DeleteClient
     * 
     * @param sourceHost
     * @param sourceGlobusUrl
     * @param rftOptions
     */
    public URLExpander(GridFTPClient sourceHost, GlobusURL sourceGlobusUrl,
            RFTOptionsType rftOptions) {
        this.sourceHost = sourceHost;
        this.sourceGlobusUrl = sourceGlobusUrl;
        this.rftOptions = rftOptions;
        this.sourceUrlsEx = new Vector();
        this.sourcePath = "/" + sourceGlobusUrl.getPath();
        this.sourceUrlsEx.add(this.sourcePath);
        this.isDelete = true;
    }

    public void setRequestId(int requestId) {
        logger.debug("RequestId in Expander:" + requestId);
        this.requestId = requestId;
    }

    public Vector getSourceUrlList() {
        return this.sourceUrlsEx;
    }
    
    public int getNumberOfTransfers() {
        if (logger.isDebugEnabled()) {
            logger.debug("Returning number of transfers : "
                + this.numberOfTransfers);
        }
        return this.numberOfTransfers;
    }
    /**
     * do MLSD 
     * @param localSourcePath
     * @return MLSD results
     */
    public Vector doMlsd(String localSourcePath) {
        Vector returnThis = new Vector();
        try {
            HostPort hp = this.sourceHost.setLocalPassive();
            this.sourceHost.setActive(hp);
            this.sourceHost.changeDir(localSourcePath);
            returnThis = this.sourceHost.mlsd();
        } catch (Exception e) {
            logger.error(i18n.getMessage("urlExpansionErr")
                    + e.getMessage(), e);
        }
        return returnThis;
    }

    /**
     * Gets the status attribute of the URLExpander object
     * 
     * @return The status value
     */
    public boolean getStatus() {
        return this.done;
    }

    /**
     * this invokes the MLST command from GridFTPClient on the source hands it
     * over to parser utility which gives set of directories that need to be
     * made at the destination.Recursive directory traversal or Iterative?
     */
    public void run() {
        try {
            while (this.sourceUrlsEx.size() > 0) {
                logger
                        .debug("Size of SourceUrlsEx "
                                + this.sourceUrlsEx.size());
                String currentUrl = (String) this.sourceUrlsEx.remove(0);
                dirVector.add(currentUrl);
                Vector v = doMlsd(currentUrl);

                while (!v.isEmpty()) {
                    MlsxEntry f = (MlsxEntry) v.remove(0);
                    if (f.get(MlsxEntry.TYPE).equals(MlsxEntry.TYPE_DIR)) {
                        String dirName = f.getFileName();
                        if (!this.isDelete) {
                            this.makeDirectory(dirName);
                        } 
                        String newSourcePath = currentUrl + "/"
                            + dirName;
                        this.sourceUrlsEx.add(newSourcePath);
                    } else if (f.get(MlsxEntry.TYPE)
                            .equals(MlsxEntry.TYPE_FILE)) {
                        TransferType transferType = new TransferType();
                        String newSourceUrl = "gsiftp://"
                                + this.sourceGlobusUrl.getHost() + ":"
                                + this.sourceGlobusUrl.getPort() + currentUrl
                                + "/" + f.getFileName();
                        transferType.setSourceUrl(newSourceUrl);
                        int temp = currentUrl.lastIndexOf("//");
                        String mkdir = "";
                        if (temp != -1) {
                            mkdir = currentUrl.substring(temp + 2);
                        }
                        if (!this.isDelete) {
                            String newDestinationUrl = "gsiftp://"
                                    + this.destinationGlobusUrl.getHost() + ":"
                                    + this.destinationGlobusUrl.getPort()
                                    + this.destinationPath + "/" + mkdir + "/"
                                    + f.getFileName();
                            transferType.setDestinationUrl(newDestinationUrl);
                            transferType.setStatus(
                            TransferStatusTypeEnumeration.Pending);
                        } else {
                            transferType.setStatus(
                            TransferStatusTypeEnumeration.Dirty);
                        }
                        transferType.setAttempts(new Integer(0));
                        transferType.setRftOptions(this.rftOptions);
                        ReliableFileTransferDbAdapter.
                        getDbAdapter().
                        storeTransferJob(requestId, transferType);
                        this.numberOfTransfers++;
                    }
                }
                if (this.sourceUrlsEx.size() > 0) {
                    currentUrl = (String) this.sourceUrlsEx.elementAt(0);
                    if (!this.isDelete) {
                        String dirChange = currentUrl.substring(currentUrl
                                .lastIndexOf("//") + 2);
                        dirChange = dirChange.trim();
                        if (!dirChange.equals("")) {
                            dirChange = this.destinationPath + dirChange;
                            this.changeDir(dirChange);
                        }
                    } else {
                        if (logger.isDebugEnabled()) {
                            logger.debug("Delete this: " + currentUrl);
                        }
                    }
                }
            }
        } catch (Exception e) {
            logger.error(i18n.getMessage("urlExpansionErr"), e);
        }
        if (this.sourceUrlsEx.size() == 0) {
            if (logger.isDebugEnabled()) {
                logger.debug("UrlExpander is done");
            }
            this.done = true;
        }
    }

    private void makeDirectory(String dir) throws IOException {
        try {
            this.destinationHost.makeDir(dir);
        } catch (ServerException se) {
            //We don't do anything about this
            logger.error(i18n.getMessage("dirExistsErr", dir) 
                    + se.getMessage());
        }
    }

    public void close() {
        try {
            if (this.sourceHost != null) {
                this.sourceHost.close();
            }
            if (this.destinationHost != null) {
                this.destinationHost.close();
            }
        } catch (Exception e) {
            logger.error(i18n.getMessage("closeClientErr")
                    + e.getMessage(), e);
        }
    }
/*
     * this method takes a string and cd's to that location
     * @param dirString
     */
    private void changeDir(String dirString) {
        try {
            this.destinationHost.changeDir(dirString);
        } catch (ServerException e) {
            try {
                this.makeDirectory(dirString);
                this.destinationHost.changeDir(dirString);
            } catch (ServerException se) {
                logger.error(i18n.getMessage("urlExpansionErr"), e);
            } catch (IOException ioe) {
                logger.error(i18n.getMessage("urlExpansionErr"), e);
            }
        } catch (IOException ioe) {
            logger.error(i18n.getMessage("urlExpansionErr"), ioe);
        }
    }
}

