/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.transfer.reliable.service.database;

import java.util.Calendar;

import javax.naming.InitialContext;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.globus.rft.generated.TransferType;
import org.globus.transfer.reliable.service.RFTConstants;
import org.globus.transfer.reliable.service.TransferWork;
import org.globus.transfer.reliable.service.RFTResourceManager;
import org.globus.util.I18n;
import org.globus.wsrf.ResourcePropertySet;

import commonj.work.Work;
import commonj.work.WorkManager;

/**
 * Database poller for a transfer
 * TODO: Make this a singleton
 */
public class RFTDatabasePoller implements Work {
    
    private static Log logger = 
        LogFactory.getLog(RFTDatabasePoller.class.getName());
    
    private static I18n i18n =
        I18n.getI18n("org.globus.transfer.reliable.errors", 
                RFTDatabasePoller.class.getClassLoader());
    private boolean started = false;
    
    private long pollTime = 0;
    
    private ResourcePropertySet rpSet = null;
    
    private int requestId = 0;

    private RFTResourceManager resourceManager;
    
     /* @param pollTime
     * @param rpSet
     * @param requestId
     */
    public  RFTDatabasePoller(long pollTime, ResourcePropertySet rpSet, 
            int requestId, RFTResourceManager resourceManager) {
        this.pollTime = pollTime;
        this.rpSet = rpSet;
        this.requestId = requestId;
        this.resourceManager = resourceManager;
    }
    
    /* (non-Javadoc)
     * @see commonj.work.Work#release()
     */
    public void release() {
        // just for the interface
    }
    
    public boolean isDaemon() {
        return false;
    }
    
    /**
     * @return
     */
    public  boolean isStarted() {
        return this.started;
    }
    /**
     * @param pollTime
     */
    public void setPollTime(long pollTime) {
        this.pollTime = pollTime;
    }
    /*
     * main processing method
     */
    public void run()  {
        try {
            this.started = true;
            while(this.pollTime > Calendar.getInstance().getTimeInMillis()) {
                if (logger.isDebugEnabled()) {
                    logger.debug("Sleeping here ");
                    logger.debug(" pollTime: " + this.pollTime);
                    logger.debug( "Current time: " + 
                            Calendar.getInstance().getTimeInMillis());
                }
                // TODO:Find a better solution to actually sleep until
                // polltime = current time
                Thread.sleep(1000);
            }
            
            TransferType newTransfer = 
                this.resourceManager.getNext();
            if (newTransfer != null) {
                InitialContext initialCtx = new InitialContext();
                WorkManager workManager = (WorkManager) initialCtx
                        .lookup(RFTConstants.WORKMANAGER);
                TransferWork transferWork = new TransferWork(this.requestId,
                        this.rpSet, newTransfer);
                transferWork.setWorkManager(workManager);
                transferWork.setResourceManager(this.resourceManager);
                workManager.schedule(transferWork);
            }
        this.started = false;
        } catch(Exception ie) {
            logger.error(i18n.getMessage("dbPollingErr"),ie);
        } 
    }
}
