/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.transfer.reliable.service.factory;

import javax.xml.namespace.QName;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.globus.transfer.reliable.service.database.ReliableFileTransferDbAdapter;
import org.globus.transfer.reliable.service.database.RftDBException;
import org.globus.util.I18n;
import org.globus.wsrf.impl.ReflectionResourceProperty;
import org.globus.wsrf.impl.SimpleResourcePropertyMetaData;


/**
 * TotalNumberOfActiveTransfers in the containner
 */
public class TotalNumberOfActiveTransfers extends ReflectionResourceProperty {
    private static Log logger = 
        LogFactory.getLog(TotalNumberOfActiveTransfers.class.getName());
    private static I18n i18n =
        I18n.getI18n("org.globus.transfer.reliable.errors", 
                TotalNumberOfActiveTransfers.class.getClassLoader());
    private long totalNumberOfActiveTransfers = 0;
    
    /**
     * @param qname
     * @throws Exception
     */
    public TotalNumberOfActiveTransfers(QName qname)
    throws Exception {
        super(new SimpleResourcePropertyMetaData(qname));
        setObject(this);
        setPropertyName("TotalNumberOfActiveTransfers");
        initialize();
    }
    /**
     * @return Returns the totalNumberOfActiveTransfers.
     */
    public long getTotalNumberOfActiveTransfers() {
        try {
            this.totalNumberOfActiveTransfers = ReliableFileTransferDbAdapter.
            getDbAdapter().getNumberActive();
        } catch (RftDBException dbEx) {
            if (logger.isDebugEnabled()) {
                logger.error(i18n.getMessage("dbRetrieveErr"), dbEx);
            }
        }
        return this.totalNumberOfActiveTransfers;
    }
    /**
     * @param totalNumberOfActiveTransfers 
     * The totalNumberOfActiveTransfers to set.
     */
    public void setTotalNumberOfActiveTransfers(
            long totalNumberOfActiveTransfers) {
        this.totalNumberOfActiveTransfers = 
            totalNumberOfActiveTransfers;
    }
}
