/*
 * Portions of this file Copyright 1999-2005 University of Chicago 
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the 
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without 
 * modifications, you must include this notice in the file.
 */
package org.globus.transfer.reliable.service.usage;

import java.net.Inet6Address;
import java.nio.ReadOnlyBufferException;
import java.sql.Timestamp;
import java.util.Date;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.globus.usage.packets.IPTimeMonitorPacket;
import org.globus.usage.packets.CustomByteBuffer;

import org.globus.rft.generated.OverallStatus;
/*RFT GRAM usage monitor packets, in addition to the fields in IPTimeMonitorPacket,
include the following:
    - RFT Factory start time
    - transfer request created timestamp
    - request type (transfer,delete)
    - total number of files transferred
    - total number of bytes transferred
    - total number of RFT resources created
    - fault class name or identifier if Failed
*/
public class RFTUsageMonitorPacket extends IPTimeMonitorPacket {
    static Log log = LogFactory.getLog(RFTUsageMonitorPacket.class);

    public final static byte TRANSFER = 0;
    public final static byte DELETE = 1;

    private static byte FALSE = 0;
    private static byte TRUE = 1;

    private byte requestType; 
    private long numberOfFiles;
    private long numberOfBytes;
    private long numberOfResources;
    private long totalNumberOfFiles;
    private long totalNumberOfBytes;
    private Date resourceCreationTime;
    private Date factoryStartTime;
    private OverallStatus overallStatus;

    private static short COMPONENT_CODE = 5;
    private static short PACKET_VERSION = 1;

    public RFTUsageMonitorPacket() {
        setTimestamp(System.currentTimeMillis());
        setComponentCode(COMPONENT_CODE);
        setPacketVersion(PACKET_VERSION);
    }
    public long getTotalNumberOfFiles() {
        return this.totalNumberOfFiles;
    }

    public void setTotalNumberOfFiles(long totalNumberOfFiles) {
        this.totalNumberOfFiles = totalNumberOfFiles;
    }
    public void setTotalNumberOfBytes(long totalNumberOfBytes) {
        this.totalNumberOfBytes = totalNumberOfBytes;
    }
    public long getTotalNumberOfBytes() {
        return this.totalNumberOfBytes;
    }
    public long getNumberOfFiles() {
        return this.numberOfFiles;
    }
    public void setNumberOfFiles(long numberOfFiles) {
        this.numberOfFiles = numberOfFiles;
    }
    public long getNumberOfBytes() {
        return this.numberOfBytes;
    }
    public void setNumberOfBytes(long numberOfBytes) {
        this.numberOfBytes = numberOfBytes;
    }
    public void setOverallStatus(OverallStatus overallStatus) {
        this.overallStatus = overallStatus;
    }
    public OverallStatus getOverallStatus() {
        return this.overallStatus;
    }
    public long getNumberOfResources() {
        return this.numberOfResources;
    }
    public void setNumberOfResources(long numberOfResources) {
        this.numberOfResources = numberOfResources;
    }
    public Date getResourceCreationTime() {
        return this.resourceCreationTime;
    }
    public void setResourceCreationTime(Date resourceCreationTime) {
        this.resourceCreationTime = resourceCreationTime;
    }
    public Date getFactoryStartTime() {
        return this.factoryStartTime;
    }
    public void setFactoryStartTime(Date factoryStartTime) {
        this.factoryStartTime = factoryStartTime;
    }
    public void setRequestType(boolean requestType) {
        if(requestType) {
            // DELETE Request
            this.requestType = TRUE;
        } else {
            this.requestType = FALSE;
        }
    }
    public boolean isTransfer() {
        return this.requestType == TRANSFER;
    }
    public boolean isDelete() {
        return this.requestType == DELETE;
    }
    public void packCustomFields(CustomByteBuffer buf) {
        super.packCustomFields(buf);
        buf.put(this.requestType);
        buf.putLong(this.numberOfFiles);
        buf.putLong(this.numberOfBytes);
        buf.putLong(this.numberOfResources);
        buf.putLong(this.resourceCreationTime.getTime());
        buf.putLong(this.factoryStartTime.getTime());
        buf.putLong(this.totalNumberOfBytes);
        buf.putLong(this.totalNumberOfFiles);
    }

    public void unpackCustomFields(CustomByteBuffer buf) {
        super.unpackCustomFields(buf);
        this.requestType = buf.get();
        this.numberOfFiles = buf.getLong();
        this.numberOfBytes = buf.getLong();
        this.numberOfResources = buf.getLong();
        this.resourceCreationTime = new Date(buf.getLong());
        this.factoryStartTime = new Date(buf.getLong());
        this.totalNumberOfBytes = buf.getLong();
        this.totalNumberOfFiles = buf.getLong();
    }

    public void display() {
        log.info(super.toString());
        log.info("Request Type : " + this.requestType);
        log.info("Number of Files : " + this.totalNumberOfFiles);
        log.info("Number of Bytes : " + this.totalNumberOfBytes);
        log.info("Number of RFT Resources : " + this.numberOfResources);
        log.info("Resource creation Time: " + this.resourceCreationTime);
        log.info("RFT Factory start time : " + this.factoryStartTime);
    }

    public String toSQL() {
        StringBuffer buffer = new StringBuffer("(");
        buffer.append("request_type,number_of_files,");
        buffer.append("number_of_bytes,number_of_resources,");
        buffer.append("creation_time,factory_start_time,");
        buffer.append("total_bytes,total_files");
        buffer.append(") VALUES ('");
        buffer.append(this.requestType).append("','");
        buffer.append(this.numberOfFiles).append("','");
        buffer.append(this.numberOfBytes).append("','");
        buffer.append(this.numberOfResources).append("','");
        buffer.append(this.resourceCreationTime).append("','");
        buffer.append(this.factoryStartTime);
        buffer.append(this.totalNumberOfBytes).append("','");
        buffer.append(this.totalNumberOfFiles);
        buffer.append("')");
        // Is everything string ?
        return buffer.toString();
    }
} 
