/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.transfer.reliable.service.test;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.apache.axis.message.MessageElement;
import org.apache.axis.message.addressing.Address;
import org.apache.axis.message.addressing.EndpointReferenceType;
import org.apache.axis.message.addressing.ReferencePropertiesType;

import org.globus.wsrf.utils.XmlUtils;
import org.globus.wsrf.utils.AnyHelper;

import javax.xml.rpc.Stub;

import org.globus.rft.generated.ReliableFileTransferPortType;
import org.globus.transfer.reliable.client.BaseRFTClient;
import org.globus.rft.generated.TransferRequestType;
import org.globus.rft.generated.Start;
import org.globus.rft.generated.StartOutputType;
import org.globus.rft.generated.service.ReliableFileTransferServiceAddressingLocator;

public class RFTOptionsTest extends TestCase {
    
    private static ReliableFileTransferServiceAddressingLocator rftLocator = 
        new ReliableFileTransferServiceAddressingLocator();

    public RFTOptionsTest(String name) {
        super(name);
    }
    public static void main(String[] as) {
        TestSuite testSuite = null;

        if (testSuite == null) {
            testSuite = new TestSuite(RFTOptionsTest.class);
        }
        junit.textui.TestRunner.run(testSuite);
    }

    public void testNoOptions() throws Exception {
    }
    public EndpointReferenceType init() throws Exception {
        RFTTest.setOptions();
        return BaseRFTClient.delegateCredential(
                TestConfig.HOST, TestConfig.PORT);
    }

    public void testTransferEPR() throws Exception {
        EndpointReferenceType delegEPR = init();    
        TransferRequestType transfer = TestConfig.getSingleTransfer();
        transfer.setTransferCredentialEndpoint(delegEPR);
        EndpointReferenceType transferEPR = 
        BaseRFTClient.createRFT(TestConfig.FACTORY_URL,
                    transfer);
        //======= test EPR of transfer
        assertTrue(transferEPR != null);
        assertTrue(transferEPR.getAddress() != null);
        String address = transferEPR.getAddress().toString();
        System.out.println("rft service URL:" + address);
        ReferencePropertiesType props = transferEPR.getProperties();
        assertTrue(props.get_any() != null);
        assertTrue(props.get_any().length > 0);
        for (int i = 0; i < props.get_any().length; i++) {
            String referenceProperty = XmlUtils.toString(AnyHelper
                    .toElement(props.get_any()[i]));
            System.out.println("reference property #" + i + "\n"
                    + referenceProperty);
        }
        assertTrue(props.get_any()[0] != null);
        String keyValue = props.get_any()[0].getValue();
        System.out.println("transfer resource ID = " + keyValue);
        transferEPR.setAddress(new Address(TestConfig.SERVICE_URL));

        ReliableFileTransferPortType rft = rftLocator
                .getReliableFileTransferPortTypePort(transferEPR);
        BaseRFTClient.setSecurity((Stub) rft);

        StartOutputType startResponse = rft.start(new Start());
        assertTrue(startResponse != null);

    }
}

