/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.transfer.reliable.service.test;

import java.io.IOException;
import java.io.InputStream;
import java.io.FileInputStream;
import java.util.Properties;

import org.globus.rft.generated.RFTOptionsType;
import org.globus.rft.generated.TransferRequestType;
import org.globus.rft.generated.TransferType;

public class TestConfig {
    public static String PORT;
    public static String HOST;
    public static String sourceHost;
    public static String destinationHost;
    public static int sourcePort;
    public static int destPort;
    public static String authType;
    public static String authzVal;
    public static String sourceSubject = null;
    public static String destinationSubject = null;
    public static String subject;
    public static Boolean binary;
    public static Integer tcpBufferSize;
    public static Integer blockSize ;
    public static Boolean noTpt;
    public static Integer parallelStreams;
    public static Boolean dcau;
    public static Integer concurrency ;
    public static Boolean allOrNone;
    public static Boolean backwardsCompatible;
    public static Integer maxAttempts;
    
    public static String sourcePath = "/etc/group";
    public static String destPath = "/tmp/rftTest.tmp";
    
    public static String PROTOCOL;
    public static String URL_ROOT; 
    public static String FACTORY_URL; 
    public static String SERVICE_URL;
    public static String GLOBUS_LOCATION =
        System.getProperty("GLOBUS_LOCATION");
    public static String GLOBUS_SHARE= "/share/globus_wsrf_rft_test/";
    public static String CONFIG = GLOBUS_LOCATION 
        + GLOBUS_SHARE + "test.properties";
    
    static {
        InputStream in = null;
        try {
            System.out.println( "Loading: " + CONFIG);
            in = new FileInputStream(CONFIG);
            
            if( in == null ) {
                throw new IOException(
                        "Test Configuration File not found: " 
                        + CONFIG);
            }
            
            Properties props = new Properties();
            props.load(in);
            
            // Populate the config file 
            // TODO: if any of the props are null go back to default
            HOST = props.getProperty("HOST");
            PORT = props.getProperty("PORT");
            PROTOCOL = props.getProperty("PROTOCOL");
            URL_ROOT = PROTOCOL + "://" + HOST + ":" + PORT + "/wsrf/services/";
            FACTORY_URL = URL_ROOT + "ReliableFileTransferFactoryService";
            SERVICE_URL = URL_ROOT + "ReliableFileTransferService";
            sourceHost = props.getProperty("sourceHost");
            destinationHost = props.getProperty("destinationHost");
            sourcePort = toInt(props.getProperty("sourcePort"));
            destPort = toInt(props.getProperty("destinationPort"));
            authType = props.getProperty("authType");
            authzVal = props.getProperty("authzVal");
            subject = props.getProperty("subject");
            sourceSubject = props.getProperty("sourceSubject");
            destinationSubject = props.getProperty("destinationSubject");
            binary = new Boolean(props.getProperty("binary"));
            if(!props.getProperty("tcpBufferSize").equals("")) {
                tcpBufferSize = new Integer(props.getProperty("tcpBufferSize"));
            }
            noTpt = new Boolean(props.getProperty("noTpt"));
            if(!props.getProperty("parallelStreams").equals("")) {
                parallelStreams = new Integer(props.getProperty("parallelStreams"));
            }
            dcau = new Boolean(props.getProperty("dcau"));
            concurrency = new Integer(props.getProperty("concurrency"));
            allOrNone = new Boolean(props.getProperty("allOrNone"));
            backwardsCompatible = new Boolean(props.getProperty("compatible"));
            maxAttempts = new Integer(props.getProperty("maxAttempts"));
            sourcePath = props.getProperty("sourcePath");
            destPath = props.getProperty("destPath");
             
        } catch (Exception e) {
            System.err.println("error " + e.getMessage());
            if (e instanceof NumberFormatException) {
                System.err.println("Error:Badly formatted " +
                "numbers in properties file" + e.getMessage());
            }
        } finally {
            if (in != null) {
                try {
                    in.close();
                } catch (Exception ee) {
                }
            }
        }
    }
    
    private boolean checkForNull(String property) {
        if(property == null || property.equals("")) {
            return true;
        }
        return false;
    }
    public static int toInt(String str) 
    throws NumberFormatException {
        Integer integer = new Integer(str);
        return integer.intValue();
    }
    
    public static boolean toBoolean(String str) {
        Boolean bool =  new Boolean(str);
        return bool.booleanValue();
    }
    public static String getSourceUrl() {
        String sourceUrl = "gsiftp://" + sourceHost + ":" +
        sourcePort + sourcePath;
        return sourceUrl;
    }
    public static String getDestUrl() {
        String destUrl = "gsiftp://" 
            + destinationHost + ":" + destPort + destPath;
        return destUrl;
    }
    public static TransferRequestType 
    setDefaultOptions(TransferRequestType transfer) {
        transfer.setAllOrNone(allOrNone);
        transfer.setConcurrency(concurrency);
        transfer.setMaxAttempts(maxAttempts);
        RFTOptionsType rftOptions = new RFTOptionsType();
        //rftOptions.setBinary(binary);
        //rftOptions.setBlockSize(blockSize);
        //rftOptions.setDcau(dcau);
        rftOptions.setSubjectName(subject);
        //rftOptions.setDestinationSubjectName(sourceSubject);
        //rftOptions.setNotpt(noTpt);
        //rftOptions.setParallelStreams(parallelStreams);
        rftOptions.setIgnoreFilePermErr(backwardsCompatible);
        rftOptions.setSourceSubjectName(sourceSubject);
        //rftOptions.setTcpBufferSize(tcpBufferSize);
        transfer.setRftOptions(rftOptions);
        return transfer;
    }
    public static TransferRequestType getSingleTransfer() {
        TransferRequestType transfer = new TransferRequestType();
        TransferType[] transferArray = new TransferType[1];
        setDefaultOptions(transfer);
        transferArray[0] = new TransferType();
        transferArray[0].setSourceUrl(getSourceUrl());
        transferArray[0].setDestinationUrl(getDestUrl());
        transfer.setTransfer(transferArray);
        System.out.println( getSourceUrl() + getDestUrl());
        return transfer;
    }
    public static void printConfig() {
        String output = "HOST: " + HOST + " PORT: " + PORT;
        output = output + "AUTHZ " + authzVal + " AUTH: "+ authType;
        output = output + " FACTORY_URL" + FACTORY_URL + " " ;
        output = output + " SERVICE_URL" + SERVICE_URL + "binary:" + binary;
        getSingleTransfer();
        System.out.println( output );
    }

    public static void main(String args[]) {
        printConfig();
        getSingleTransfer();
    }
}
