/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */

#include "ResourcePropertiesTestService_client.h"
#include "globus_service_engine.h"
#include "globus_notification_consumer.h"
#include "globus_wsrf_resource.h"

#include "rptest_TestCaseType.h"
#include "rptest_TestStepType.h"
#include "rptest_CreateResourcePropertiesTestType.h"
#include "rptest_CreateResourcePropertiesTestResponseType.h"
#include "rptest_SetResourcePropertiesTestType_choice.h"
#include "wsrl_DestroyType.h"
#include "wsrl_DestroyResponseType.h"

#include "globus_xml_buffer.h"

static
globus_result_t
globus_l_rp_any_match(
    xsd_any_array *                     expected,
    xsd_any_array *                     actual);

int main(int argc, char * argv[])
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    ResourcePropertiesTestService_client_handle_t
                                        client_handle;
    rptest_CreateResourcePropertiesTestType *  CreateResourcePropertiesTest = NULL;
    rptest_CreateResourcePropertiesTestResponseType *
                                        CreateResourcePropertiesTestResponse
                                            = NULL;
    wsrp_SetResourcePropertiesResponseType *
                                        SetResourcePropertiesResponse;
    wsrp_GetResourcePropertyResponseType *
                                        GetResourcePropertyResponse;
    wsrp_GetMultipleResourcePropertiesResponseType *
                                        GetMultipleResourcePropertiesResponse;
    wsrl_DestroyType *                  Destroy;
    wsrl_DestroyResponseType *          DestroyResponse = NULL;
    int                                 fault_type = 0;
    xsd_any *                           fault = NULL;
    globus_soap_message_handle_t        test_handle;
    rptest_TestCaseType *               test_case;
    int                                 i;
    xsd_QName                           test_case_qname = {
        "http://www.globus.org/namespaces/2004/10/ResourcePropertiesTest",
        "TestCase"
    };
    char *                              url;
    char *                              file_name;
    globus_bool_t                       non_persistent = GLOBUS_FALSE;
    globus_soap_message_attr_t          attrs = NULL;

    if (argc > 2 && !strcmp(argv[1], "-non-persistent"))
    {
        if (argc < 4)
        {
            fprintf(stderr,
                    "usage: %s [-non-persistent] <URL> test-script-file\n",
                    argv[0]);
            exit(1);

        }
        url = argv[2];
        file_name = argv[3];
        non_persistent = GLOBUS_TRUE;
    }
    else if(argc < 3)
    {
        fprintf(stderr,
                "usage: %s [-non-persistent] <URL> test-script-file\n",
                argv[0]);
        exit(1);

    }
    else
    {
        url = argv[1];
        file_name = argv[2];
    }

    globus_module_activate(GLOBUS_SOAP_MESSAGE_MODULE);
    globus_module_activate(GLOBUS_WSRF_RESOURCE_MODULE);
    globus_module_activate(RESOURCEPROPERTIESTESTSERVICE_MODULE);

    result = globus_soap_message_handle_init_from_file(
        &test_handle,
        file_name);

    if (result != GLOBUS_SUCCESS)
    {
        goto out;
    }
    rptest_TestCaseType_init(&test_case);
    memset(test_case, '\0', sizeof(rptest_TestCaseType));
    result = rptest_TestCaseType_deserialize(
            &test_case_qname, 
            test_case, 
            test_handle,
            0);
    if(result != GLOBUS_SUCCESS)
    {
        goto destroy_test_handle;
    }

    result = globus_soap_message_attr_init(&attrs);
    if (non_persistent)
    {
        result = globus_soap_message_attr_set(
            attrs,
            GLOBUS_SOAP_MESSAGE_CONNECTION_KEY,
            NULL,
            NULL,
            "close");
    }

    result = ResourcePropertiesTestService_client_init(
            &client_handle,
            attrs,
            NULL);
    if(result != GLOBUS_SUCCESS)
    {
        goto destroy_test_case_out;
    }
    rptest_CreateResourcePropertiesTestType_init(&CreateResourcePropertiesTest);
    CreateResourcePropertiesTest->ResourcePropertyDefinition =
            test_case->ResourcePropertyDefinition;

    result = ResourcePropertiesTestPortType_CreateResourcePropertiesTest(
        client_handle,
        url,
        CreateResourcePropertiesTest,
        &CreateResourcePropertiesTestResponse,
        (ResourcePropertiesTestPortType_CreateResourcePropertiesTest_fault_t *)&fault_type,
        &fault);

    if(result != GLOBUS_SUCCESS)
    {
        goto destroy_test_case_out;
    }

    printf("%d..%d\n", 1, test_case->TestStep.length);

    for (i = 0; i < test_case->TestStep.length; i++)
    {
        rptest_TestStepType test_step = test_case->TestStep.elements[i];
        rptest_GetResourcePropertyTestType     get_resource_property_test;
        rptest_GetMultipleResourcePropertiesTestType
                                        get_multiple_resource_properties_test;
        rptest_QueryResourcePropertiesTestType query_resource_properties_test;
        rptest_SetResourcePropertiesTestType   set_resource_properties_test;

        switch (test_step.choice_value.type)
        {
            case rptest_TestStepType_undefined:
                globus_assert(test_step.choice_value.type
                        != rptest_TestStepType_undefined);
                break;

            case rptest_TestStepType_GetResourcePropertyTest:
                get_resource_property_test =
                        test_step.choice_value.value.GetResourcePropertyTest;
                result = ResourcePropertiesTestPortType_GetResourceProperty_epr(
                    client_handle,
                    &CreateResourcePropertiesTestResponse->EndpointReference,
                    &get_resource_property_test.GetResourceProperty,
                    &GetResourcePropertyResponse,
                    (ResourcePropertiesTestPortType_GetResourceProperty_fault_t *) &fault_type,
                    &fault);

                if (get_resource_property_test.choice_value.type ==
                    rptest_GetResourcePropertyTestType_GetResourcePropertyResponse)
                {
                    if (result == GLOBUS_SUCCESS)
                    {
                        result = globus_l_rp_any_match(
                            &get_resource_property_test.choice_value.value.GetResourcePropertyResponse.any,
                            &GetResourcePropertyResponse->any);
                        /* No result expected, we're all good */
                        if (result != GLOBUS_SUCCESS)
                        {
                            printf("not ok%s%s\n",
                                test_step._name ? "# " : "",
                                test_step._name ? *test_step._name : ""); 
                        }
                        else
                        {
                            printf ("ok\n");
                        }
                    }
                    else
                    {
                        if (fault != NULL)
                        {
                            printf("not ok%s%s\n",
                                test_step._name ? "# " : "",
                                test_step._name ? *test_step._name : ""); 

                            fprintf(stderr, "Expected GLOBUS_SUCCESS, got {%s}%s\n",
                                fault->any_info->type->Namespace,
                                fault->any_info->type->local);
                        }
                        else
                        {
                            printf("not ok%s%s\n",
                                test_step._name ? "# " : "",
                                test_step._name ? *test_step._name : ""); 

                            fprintf(stderr, "Expected GLOBUS_SUCCESS, got error: %s\n",
                                globus_error_print_friendly(globus_error_peek(result)));
                        }
                    }
                }
                else
                {
                    if (result == GLOBUS_SUCCESS)
                    {
                        printf("not ok%s%s\n",
                            test_step._name ? "# " : "",
                            test_step._name ? *test_step._name : ""); 

                        fprintf(stderr, "Expected {%s}%s, got GLOBUS_SUCCESS\n",
                            get_resource_property_test.choice_value.value.Fault._Type.Namespace,
                            get_resource_property_test.choice_value.value.Fault._Type.local);
                    }
                    else if (fault != NULL)
                    {
                        if (xsd_QName_keyeq(fault->any_info->type,
                            &get_resource_property_test.choice_value.value.Fault))
                        {
                            printf("ok\n");

                            /* Good fault match */
                        }
                        else
                        {
                            printf("not ok%s%s\n",
                                test_step._name ? "# " : "",
                                test_step._name ? *test_step._name : ""); 

                            fprintf(stderr, "Expected {%s}%s, got {%s}%s\n",
                                get_resource_property_test.choice_value.value.Fault._Type.Namespace,
                                get_resource_property_test.choice_value.value.Fault._Type.local,
                                fault->any_info->type->Namespace,
                                fault->any_info->type->local);
                        }
                    }
                    else
                    {
                        printf("not ok%s%s\n",
                            test_step._name ? "# " : "",
                            test_step._name ? *test_step._name : ""); 

                        fprintf(stderr, "Expected {%s}%s, got error: %s\n",
                            get_resource_property_test.choice_value.value.Fault._Type.Namespace,
                            get_resource_property_test.choice_value.value.Fault._Type.local,
                            globus_error_print_friendly(globus_error_peek(result)));
                    }
                }

                if(fault != NULL)
                {
                    xsd_any_destroy(fault);
                    fault = NULL;
                    fault_type = 0;
                }
                if (result != GLOBUS_SUCCESS)
                {
                    globus_object_free(globus_error_get(result));
                    result = GLOBUS_SUCCESS;
                }
                break;
                
            case rptest_TestStepType_GetMultipleResourcePropertiesTest:
                get_multiple_resource_properties_test = 
                        test_step.choice_value.value.GetMultipleResourcePropertiesTest;
                result = ResourcePropertiesTestPortType_GetMultipleResourceProperties_epr(
                    client_handle,
                    &CreateResourcePropertiesTestResponse->EndpointReference,
                    &get_multiple_resource_properties_test.GetMultipleResourceProperties,
                    &GetMultipleResourcePropertiesResponse,
                    (ResourcePropertiesTestPortType_GetMultipleResourceProperties_fault_t *) &fault_type,
                    &fault);

                if (get_multiple_resource_properties_test.choice_value.type ==
                    rptest_GetMultipleResourcePropertiesTestType_GetMultipleResourcePropertiesResponse)
                {
                    if (result == GLOBUS_SUCCESS)
                    {
                        result = globus_l_rp_any_match(
                            &get_multiple_resource_properties_test.choice_value.value.GetMultipleResourcePropertiesResponse.any,
                            &GetMultipleResourcePropertiesResponse->any);
                        if (result != GLOBUS_SUCCESS)
                        {
                            printf("not ok%s%s\n",
                                test_step._name ? "# " : "",
                                test_step._name ? *test_step._name : ""); 
                        }
                        else
                        {
                            printf ("ok\n");
                        }
                    }
                    else
                    {
                        if (fault != NULL)
                        {
                            printf("not ok%s%s\n",
                                test_step._name ? "# " : "",
                                test_step._name ? *test_step._name : ""); 

                            fprintf(stderr, "Expected GLOBUS_SUCCESS, got {%s}%s\n",
                                fault->any_info->type->Namespace,
                                fault->any_info->type->local);
                        }
                        else
                        {
                            printf("not ok%s%s\n",
                                test_step._name ? "# " : "",
                                test_step._name ? *test_step._name : ""); 

                            fprintf(stderr, "Expected GLOBUS_SUCCESS, got error: %s\n",
                                globus_error_print_friendly(globus_error_peek(result)));
                        }
                    }
                }
                else
                {
                    if (result == GLOBUS_SUCCESS)
                    {
                        printf("not ok%s%s\n",
                            test_step._name ? "# " : "",
                            test_step._name ? *test_step._name : ""); 

                        fprintf(stderr, "Expected {%s}%s, got GLOBUS_SUCCESS\n",
                            get_multiple_resource_properties_test.choice_value.value.Fault._Type.Namespace,
                            get_multiple_resource_properties_test.choice_value.value.Fault._Type.local);
                    }
                    else if (fault != NULL)
                    {
                        if (xsd_QName_keyeq(fault->any_info->type,
                            &get_multiple_resource_properties_test.choice_value.value.Fault))
                        {
                            printf("ok\n");

                            /* Good fault match */
                        }
                        else
                        {
                            printf("not ok%s%s\n",
                                test_step._name ? "# " : "",
                                test_step._name ? *test_step._name : ""); 

                            fprintf(stderr, "Expected {%s}%s, got {%s}%s\n",
                                get_multiple_resource_properties_test.choice_value.value.Fault._Type.Namespace,
                                get_multiple_resource_properties_test.choice_value.value.Fault._Type.local,
                                fault->any_info->type->Namespace,
                                fault->any_info->type->local);
                        }
                    }
                    else
                    {
                        printf("not ok%s%s\n",
                            test_step._name ? "# " : "",
                            test_step._name ? *test_step._name : ""); 

                        fprintf(stderr, "Expected {%s}%s, got error: %s\n",
                            get_multiple_resource_properties_test.choice_value.value.Fault._Type.Namespace,
                            get_multiple_resource_properties_test.choice_value.value.Fault._Type.local,
                            globus_error_print_friendly(globus_error_peek(result)));
                    }
                }

                if(fault != NULL)
                {
                    xsd_any_destroy(fault);
                    fault = NULL;
                    fault_type = 0;
                }
                if (result != GLOBUS_SUCCESS)
                {
                    globus_object_free(globus_error_get(result));
                    result = GLOBUS_SUCCESS;
                }
                break;

            case rptest_TestStepType_QueryResourcePropertiesTest:
                query_resource_properties_test =
                        test_step.choice_value.value.QueryResourcePropertiesTest;
                globus_assert(test_step.choice_value.type
                        != rptest_TestStepType_QueryResourcePropertiesTest);
                break;

            case rptest_TestStepType_SetResourcePropertiesTest:
                set_resource_properties_test =
                        test_step.choice_value.value.SetResourcePropertiesTest;

                result = ResourcePropertiesTestPortType_SetResourceProperties_epr(
                    client_handle,
                    &CreateResourcePropertiesTestResponse->EndpointReference,
                    &set_resource_properties_test.SetResourceProperties,
                    &SetResourcePropertiesResponse,
                    (ResourcePropertiesTestPortType_SetResourceProperties_fault_t *) &fault_type,
                    &fault);

                if (set_resource_properties_test.choice_value.type ==
                    rptest_SetResourcePropertiesTestType_SetResourcePropertiesResponse)
                {
                    if (result == GLOBUS_SUCCESS)
                    {
                        /* No result expected, we're all good */
                        printf("ok\n");
                    }
                    else
                    {
                        if (fault != NULL)
                        {
                            printf("not ok%s%s\n",
                                test_step._name ? "# " : "",
                                test_step._name ? *test_step._name : ""); 

                            fprintf(stderr, "Expected GLOBUS_SUCCESS, got {%s}%s\n",
                                fault->any_info->type->Namespace,
                                fault->any_info->type->local);
                        }
                        else
                        {
                            printf("not ok%s%s\n",
                                test_step._name ? "# " : "",
                                test_step._name ? *test_step._name : ""); 

                            fprintf(stderr, "Expected GLOBUS_SUCCESS, got error: %s\n",
                                globus_error_print_friendly(globus_error_peek(result)));
                        }
                    }
                }
                else
                {
                    if (result == GLOBUS_SUCCESS)
                    {
                        printf("not ok%s%s\n",
                            test_step._name ? "# " : "",
                            test_step._name ? *test_step._name : ""); 

                        fprintf(stderr, "Expected {%s}%s, got GLOBUS_SUCCESS\n",
                            set_resource_properties_test.choice_value.value.Fault._Type.Namespace,
                            set_resource_properties_test.choice_value.value.Fault._Type.local);
                    }
                    else if (fault != NULL)
                    {
                        if (xsd_QName_keyeq(fault->any_info->type,
                            &set_resource_properties_test.choice_value.value.Fault))
                        {
                            printf("ok\n");

                            /* Good fault match */
                        }
                        else
                        {
                            printf("not ok%s%s\n",
                                test_step._name ? "# " : "",
                                test_step._name ? *test_step._name : ""); 

                            fprintf(stderr, "Expected {%s}%s, got {%s}%s\n",
                                set_resource_properties_test.choice_value.value.Fault._Type.Namespace,
                                set_resource_properties_test.choice_value.value.Fault._Type.local,
                                fault->any_info->type->Namespace,
                                fault->any_info->type->local);
                        }
                    }
                    else
                    {
                        printf("not ok%s%s\n",
                            test_step._name ? "# " : "",
                            test_step._name ? *test_step._name : ""); 

                        fprintf(stderr, "Expected {%s}%s, got error: %s\n",
                            set_resource_properties_test.choice_value.value.Fault._Type.Namespace,
                            set_resource_properties_test.choice_value.value.Fault._Type.local,
                            globus_error_print_friendly(globus_error_peek(result)));
                    }
                }

                if(fault != NULL)
                {
                    xsd_any_destroy(fault);
                    fault = NULL;
                    fault_type = 0;
                }
                if (result != GLOBUS_SUCCESS)
                {
                    globus_object_free(globus_error_get(result));
                    result = GLOBUS_SUCCESS;
                }
        }
    }

    wsrl_DestroyType_init(&Destroy);

    result = ResourcePropertiesTestPortType_Destroy_epr(
        client_handle,
        &CreateResourcePropertiesTestResponse->EndpointReference,
        Destroy,
        &DestroyResponse,
        (ResourcePropertiesTestPortType_Destroy_fault_t *)&fault_type,
        &fault);

    if (result != GLOBUS_SUCCESS)
    {
        printf("Destroy failed: %s\n",
                globus_object_printable_to_string(globus_error_peek(result)));
        goto destroy_test_case_out;
    }


    if(CreateResourcePropertiesTestResponse)
    {
        rptest_CreateResourcePropertiesTestResponseType_destroy(
                CreateResourcePropertiesTestResponse);
    }

    ResourcePropertiesTestService_client_destroy(client_handle);

destroy_test_case_out:
    rptest_TestCaseType_destroy(test_case);
destroy_test_handle:
    globus_soap_message_handle_destroy(test_handle);
out:
    if(result != GLOBUS_SUCCESS)
    {
        char *                          errorstr;

        errorstr = globus_error_print_chain(globus_error_get(result));
        fprintf(stderr, "ERROR:\n\n%s\n\n", errorstr);
        globus_module_deactivate_all();
        return 1;
    }

    globus_module_deactivate_all();

    return 0;
}

static
globus_result_t
globus_l_rp_any_match(
    xsd_any_array *                     expected,
    xsd_any_array *                     actual)
{
    int                                 i;
    int                                 j;
    globus_bool_t *                     expectations;
    globus_bool_t *                     actuals;
    globus_result_t                     result = GLOBUS_SUCCESS;

    expectations = calloc(
            expected ? expected->length : 0,
            sizeof(globus_bool_t));
    actuals = calloc(
            actual ? actual->length : 0,
            sizeof(globus_bool_t));

    for (i = 0; i < expected->length; i++)
    {
        for (j = 0; j < actual->length; j++)
        {
            if (expectations[i] || actuals[j])
            {
                continue;
            }
            if (xsd_QName_keyeq(
                        expected->elements[i].element,
                        actual->elements[j].element))
            {
                if (expected->elements[i].any_info ==
                        actual->elements[j].any_info)
                {
                    if (expected->elements[i].any_info == &xsd_string_info)
                    {
                        if (!strcmp(*(xsd_string*) expected->elements[i].value,
                                *(xsd_string*) actual->elements[j].value))
                        {
                            expectations[i] = GLOBUS_TRUE;
                            actuals[j] = GLOBUS_TRUE;
                        }
                    }
                    else if (expected->elements[i].any_info == &xsd_int_info)
                    {
                        if (*(xsd_int*) expected->elements[i].value ==
                            *(xsd_int*) actual->elements[j].value)
                        {
                            expectations[i] = GLOBUS_TRUE;
                            actuals[j] = GLOBUS_TRUE;
                        }
                    }
                    else if (expected->elements[i].any_info == &globus_xml_buffer_contents_info)
                    {
                        globus_xml_buffer * expected_buf;
                        globus_xml_buffer * actual_buf;

                        expected_buf = expected->elements[i].value;
                        actual_buf = actual->elements[j].value;
                        
                        if (expected_buf->length == actual_buf->length)
                        {
                            if (memcmp(
                                    expected_buf->buffer,
                                    actual_buf->buffer,
                                    expected_buf->length) == 0)
                            {
                                expectations[i] = GLOBUS_TRUE;
                                actuals[j] = GLOBUS_TRUE;
                            }
                        }
                    }
                }
            }
        }
    }

    for (i = 0; i < expected->length; i++)
    {
        if (! expectations[i])
        {
            fprintf(stderr, "Missing value for {%s}%s\n",
                expected->elements[i].element->Namespace,
                expected->elements[i].element->local);
            result = GLOBUS_FAILURE;
        }
    }
    for (i = 0; i < actual->length; i++)
    {
        if (! actuals[i])
        {
            fprintf(stderr, "Unexpected value for {%s}%s\n",
                actual->elements[i].element->Namespace,
                actual->elements[i].element->local);
            result = GLOBUS_FAILURE;
        }
    }
    free(actuals);
    free(expectations);

    return result;
}
/* globus_l_rp_any_match() */
