/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */


#ifndef _GLOBUS_I_WS_SECURITY_H_
#define _GLOBUS_I_WS_SECURITY_H_

#include "globus_common.h"

EXTERN_C_BEGIN

enum
{
    GLOBUS_WS_SECURITY_ERROR_TYPE_CREATING_PKIPATH = 1024,
    GLOBUS_WS_SECURITY_ERROR_TYPE_IMPORTING_CREDENTIAL,
    GLOBUS_WS_SECURITY_ERROR_TYPE_OPENSSL,
    GLOBUS_WS_SECURITY_ERROR_TYPE_OUT_OF_MEMORY
};

GlobusDebugDeclare(GLOBUS_WS_SECURITY);

#define GLOBUS_L_WS_SECURITY_DEBUG_PKIPATH  1
#define GLOBUS_L_WS_SECURITY_DEBUG_INFO     2
#define GLOBUS_L_WS_SECURITY_DEBUG_DEBUG    4
#define GLOBUS_L_WS_SECURITY_DEBUG_TRACE    8
#define GLOBUS_L_WS_SECURITY_DEBUG_WARN     16

#define GlobusWSSecurityDebug(LEVEL) \
    (GlobusDebugTrue(GLOBUS_WS_SECURITY, LEVEL))

#define GlobusWSSecurityDebugPrintf(LEVEL, MESSAGE) \
    GlobusDebugPrintf(GLOBUS_WS_SECURITY, LEVEL, MESSAGE)

#define GlobusWSSecurityDebugEnter() \
    GlobusWSSecurityDebugPrintf(GLOBUS_L_WS_SECURITY_DEBUG_TRACE, \
                                 ("[%s] Entering\n", _globus_func_name))

#define GlobusWSSecurityDebugExit() \
    GlobusWSSecurityDebugPrintf(GLOBUS_L_WS_SECURITY_DEBUG_TRACE, \
                                 ("[%s] Exiting\n", _globus_func_name))

#define GlobusWSSecurityErrorCreatingPKIPath(cause, message)    \
    globus_error_put(                                           \
        globus_error_construct_error(                           \
           GLOBUS_WS_SECURITY_MODULE,                           \
           (cause ? globus_error_get(cause) : NULL),            \
           GLOBUS_WS_SECURITY_ERROR_TYPE_CREATING_PKIPATH,      \
           __FILE__,                                            \
           _globus_func_name,                                   \
           __LINE__,                                            \
           "Error creating X509PKIPath: %s",                    \
           message))

#define GlobusWSSecurityErrorImportingCredential(cause, message)        \
    globus_error_put(                                                   \
        globus_error_construct_error(                                   \
           GLOBUS_WS_SECURITY_MODULE,                                   \
           (cause ? globus_error_get(cause) : NULL),                    \
           GLOBUS_WS_SECURITY_ERROR_TYPE_IMPORTING_CREDENTIAL,          \
           __FILE__,                                                    \
           _globus_func_name,                                           \
           __LINE__,                                                    \
           "Error importing credential from PKIPath: %s",               \
           message))

#define GlobusWSSecurityErrorOpenSSL(message)           \
    globus_error_put(                                   \
        globus_error_wrap_openssl_error(                \
           GLOBUS_WS_SECURITY_MODULE,                   \
           GLOBUS_WS_SECURITY_ERROR_TYPE_OPENSSL,       \
           __FILE__,                                    \
           _globus_func_name,                           \
           __LINE__,                                    \
           "Error with PKIPath import/export: %s",      \
           message))

#define GlobusWSSecurityErrorOutOfMemory(length)        \
    globus_error_put(                                   \
        globus_error_wrap_openssl_error(                \
           GLOBUS_WS_SECURITY_MODULE,                   \
           GLOBUS_WS_SECURITY_ERROR_TYPE_OUT_OF_MEMORY, \
           __FILE__,                                    \
           _globus_func_name,                           \
           __LINE__,                                    \
           "Failed to allocate %d bytes", length))

EXTERN_C_END

#endif
