/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */

#include "WidgetService_client.h"
#include "wsrl_SetTerminationTimeType.h"
#include "wsrl_SetTerminationTimeResponseType.h"
#include "wsrp_GetMultipleResourcePropertiesType.h"
#include "wsrp_GetMultipleResourcePropertiesResponseType.h"

#include "interop_test.h"

int
main(int argc, char *argv[])
{
    int                                 rc;
    WidgetService_client_handle_t       client_handle;
    createWidgetType                    createWidget;
    createWidgetResponseType *          createWidgetResponse = NULL;
    wsrl_SetTerminationTimeType         SetTerminationTime;
    wsrl_SetTerminationTimeResponseType *
                                        SetTerminationTimeResponse = NULL;
    wsrp_GetMultipleResourcePropertiesType
                                        GetMultipleResourceProperties;
    wsrp_GetMultipleResourcePropertiesResponseType *
                                        GetMultipleResourcePropertiesResponse = NULL;
    int                                 fault_type;
    xsd_any *                           fault = NULL;
    time_t                              term_time;
    struct tm *                         result_tm = NULL;
    globus_result_t                     result = GLOBUS_SUCCESS;
    xsd_QName *                         qname = NULL;
    int                                 i;
    int                                 got[2];

    if (argc != 2)
    {
        fprintf(stderr, "Usage: %s ServiceURI\n", argv[0]);
        rc = 1;

        goto out;
    }

    rc = globus_module_activate(GLOBUS_SOAP_MESSAGE_MODULE);
    if (rc != GLOBUS_SUCCESS)
    {
        rc = 2;

        goto out;
    }
    result = WidgetService_client_init(&client_handle, NULL, NULL);
    if (result != GLOBUS_SUCCESS)
    {
        rc = 3;

        goto deactivate_out;
    }
    memset(&GetMultipleResourceProperties,
            '\0',
            sizeof(wsrp_GetMultipleResourcePropertiesType));

    /* Add deserializers for RP QNames not in the WSDL */
    result = interop_test_init_registry(GLOBUS_GLOBAL_TYPE_REGISTRY);
    if (result != GLOBUS_SUCCESS)
    {
        rc = 7;

        goto handle_destroy_out;
    }
    result = WidgetPortType_createWidget(
        client_handle,
        argv[1],
        &createWidget,
        &createWidgetResponse,
        (WidgetPortType_createWidget_fault_t *) &fault_type,
        &fault);

    if (result != GLOBUS_SUCCESS)
    {
        rc = 9;

        goto handle_destroy_out;
    }

    term_time = time(NULL) + 30;
    result_tm = globus_libc_gmtime_r(&term_time,
            &SetTerminationTime.RequestedTerminationTime);

    result = WidgetPortType_SetTerminationTime_epr(
        client_handle,
        &createWidgetResponse->EndpointReference,
        &SetTerminationTime,
        &SetTerminationTimeResponse,
        (WidgetPortType_SetTerminationTime_fault_t *) &fault_type,
        &fault);

    if (result != GLOBUS_SUCCESS || fault != NULL)
    {
        rc = 10;

        goto handle_destroy_out;
    }
    qname = xsd_QName_array_push(
            &GetMultipleResourceProperties.ResourceProperty);
    qname->Namespace = globus_libc_strdup(termination_time_rp_qname.Namespace);
    qname->local = globus_libc_strdup(termination_time_rp_qname.local);
    qname = xsd_QName_array_push(
            &GetMultipleResourceProperties.ResourceProperty);
    qname->Namespace = globus_libc_strdup(current_time_rp_qname.Namespace);
    qname->local = globus_libc_strdup(current_time_rp_qname.local);

    result = WidgetPortType_GetMultipleResourceProperties_epr(
        client_handle,
        &createWidgetResponse->EndpointReference,
        &GetMultipleResourceProperties,
        &GetMultipleResourcePropertiesResponse,
        (WidgetPortType_GetMultipleResourceProperties_fault_t *) &fault_type,
        &fault);

    if (result != GLOBUS_SUCCESS)
    {
        rc = 11;

        goto handle_destroy_out;
    }

    for (i = 0; i < GetMultipleResourcePropertiesResponse->any.length; i++)
    {
        xsd_any * any = &GetMultipleResourcePropertiesResponse->any.elements[i];

        if (xsd_QName_keyeq(any->any_info->type, xsd_dateTime_info.type) ||
            xsd_QName_keyeq(any->any_info->type, any->element))
        {
            if (xsd_QName_keyeq(any->element,
                    &GetMultipleResourceProperties.ResourceProperty.elements[0]))
            {
                if (memcmp(
                        any->value, 
                        &SetTerminationTimeResponse->NewTerminationTime,
                        sizeof(xsd_dateTime)) != 0)
                {
                    rc = 12;

                    goto handle_destroy_out;
                }
                else 
                {
                    got[0] = 1;
                }
            }
            else if (xsd_QName_keyeq(any->element, 
                    &GetMultipleResourceProperties.ResourceProperty.elements[1]))
            {
                got[1] = 1;
            }
            else
            {
                rc = 13;

                goto handle_destroy_out;
            }
        }
        else
        {
            rc = 14;

            goto handle_destroy_out;
        }
    }

    if ((!got[0]) || (!got[1]))
    {
        rc = 15;

        goto handle_destroy_out;
    }
handle_destroy_out:
    wsrp_GetMultipleResourcePropertiesType_destroy_contents(&GetMultipleResourceProperties);
    wsrl_SetTerminationTimeResponseType_destroy(SetTerminationTimeResponse);
    createWidgetResponseType_destroy(createWidgetResponse);
    WidgetService_client_destroy(client_handle);
deactivate_out:
    if(result != GLOBUS_SUCCESS)
    {
        char * error = globus_error_print_chain(globus_error_peek(result));
        fprintf(stderr, "%s\n", error);
        free(error);
    }

    globus_module_deactivate(GLOBUS_SOAP_MESSAGE_MODULE);
out:
    if (GetMultipleResourcePropertiesResponse != NULL)
    {
        wsrp_GetMultipleResourcePropertiesResponseType_destroy(
            GetMultipleResourcePropertiesResponse);
        GetMultipleResourcePropertiesResponse = NULL;
    }
    return rc;
}
/* main() */
