/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */

#include "WidgetService_client.h"
#include "wsrp_QueryResourcePropertiesType.h"
#include "wsrp_QueryResourcePropertiesResponseType.h"
#include "interop_test.c"

int
main(int argc, char *argv[])
{
    int                                 rc;
    WidgetService_client_handle_t       client_handle;
    createWidgetType                    createWidget;
    createWidgetResponseType *          createWidgetResponse = NULL;
    wsrp_QueryResourcePropertiesType *  QueryResourceProperties;
    wsrp_QueryResourcePropertiesResponseType *
                                        QueryResourcePropertiesResponse = NULL;
    int                                 fault_type;
    xsd_any *                           fault = NULL;
    globus_result_t                     result = GLOBUS_SUCCESS;

    if (argc != 2)
    {
        fprintf(stderr, "Usage: %s ServiceURI\n", argv[0]);
        rc = 1;

        goto out;
    }

    rc = globus_module_activate(GLOBUS_SOAP_MESSAGE_MODULE);
    if (rc != GLOBUS_SUCCESS)
    {
        rc = 2;

        goto out;
    }
    result = WidgetService_client_init(&client_handle, NULL, NULL);
    if (result != GLOBUS_SUCCESS)
    {
        rc = 3;

        goto deactivate_out;
    }
    /* Add deserializers for RP QName not in the WSDL */
    result = interop_test_init_registry(GLOBUS_GLOBAL_TYPE_REGISTRY);
    if (result != GLOBUS_SUCCESS)
    {
        rc = 6;
        goto handle_destroy_out;
    }

    result = WidgetPortType_createWidget(
        client_handle,
        argv[1],
        &createWidget,
        &createWidgetResponse,
        (WidgetPortType_createWidget_fault_t *) &fault_type,
        &fault);

    if (result != GLOBUS_SUCCESS)
    {
        rc = 7;

        goto handle_destroy_out;
    }

    wsrp_QueryResourcePropertiesType_init(&QueryResourceProperties);
    xsd_anyURI_init(&QueryResourceProperties->QueryExpression._Dialect);
    *QueryResourceProperties->QueryExpression._Dialect = globus_libc_strdup(
        "http://www.w3.org/TR/1999/REC-xpath-19991116");
    xsd_any_init(&QueryResourceProperties->QueryExpression.any);
    QueryResourceProperties->QueryExpression.any->registry = NULL;
    QueryResourceProperties->QueryExpression.any->any_info =
        &xsd_string_contents_info;
    QueryResourceProperties->QueryExpression.any->element = NULL;
    xsd_string_init_cstr(
        (xsd_string **) &QueryResourceProperties->QueryExpression.any->value,
        "boolean(/*/*[namespace-uri()='http://docs.oasis-open.org/wsrf/2004/06/wsrf-WS-ResourceLifetime-1.2-draft-01.xsd' and local-name()='TerminationTime'])");

    result = WidgetPortType_QueryResourceProperties_epr(
        client_handle,
        &createWidgetResponse->EndpointReference,
        QueryResourceProperties,
        &QueryResourcePropertiesResponse,
        (WidgetPortType_QueryResourceProperties_fault_t *) &fault_type,
        &fault);

    if (result != GLOBUS_SUCCESS)
    {
        rc = 9;

        if (fault != NULL && fault->element != NULL)
        {
            fprintf(stderr, "Got a {%s}%s fault\n",
                    fault->element->Namespace,
                    fault->element->local);
        }
        if (fault != NULL)
        {
            xsd_any_destroy(fault);
        }

        goto handle_destroy_out;
    }

handle_destroy_out:
    createWidgetResponseType_destroy(createWidgetResponse);
    WidgetService_client_destroy(client_handle);
deactivate_out:
    if(result != GLOBUS_SUCCESS)
    {
        char * error = globus_error_print_chain(globus_error_peek(result));
        fprintf(stderr, "%s\n", error);
        free(error);
    }
    globus_module_deactivate(GLOBUS_SOAP_MESSAGE_MODULE);
out:
    return rc;
}
/* main() */
