/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */

#include "WidgetService_client.h"
#include "wsrp_SetResourcePropertiesType.h"
#include "wsrp_SetResourcePropertiesResponseType.h"
#include "wsrp_GetResourcePropertyResponseType.h"
#include "wsrp_InsertType.h"
#include "interop_test.h"

static
globus_bool_t
match_string_rp_value(
    wsrp_GetResourcePropertyResponseType *
                                        response,
    xsd_QName *                         qname,
    xsd_string                          value);

#define STRING1 "string1"
#define STRING2 "string2"
#define STRING3 "string3"

int
main(int argc, char *argv[])
{
    int                                 rc;
    WidgetService_client_handle_t       client_handle;
    createWidgetType                    createWidget;
    createWidgetResponseType *          createWidgetResponse = NULL;
    wsrp_SetResourcePropertiesType_choice *
                                        SetResourcePropertiesType_choice;
    wsrp_SetResourcePropertiesType *    SetResourceProperties = NULL;
    wsrp_SetResourcePropertiesResponseType * 
                                        SetResourcePropertiesResponse = NULL;
    wsrp_GetResourcePropertyResponseType *
                                        GetResourcePropertyResponse = NULL;
    int                                 fault_type;
    xsd_any *                           any;
    xsd_any *                           fault = NULL;
    globus_result_t                     result = GLOBUS_SUCCESS;

    if (argc != 2)
    {
        fprintf(stderr, "Usage: %s ServiceURI\n", argv[0]);
        rc = 1;

        goto out;
    }

    rc = globus_module_activate(GLOBUS_SOAP_MESSAGE_MODULE);
    if (rc != GLOBUS_SUCCESS)
    {
        rc = 2;

        goto out;
    }
    result = WidgetService_client_init(&client_handle, NULL, NULL);
    if (result != GLOBUS_SUCCESS)
    {
        rc = 3;

        goto deactivate_out;
    }
    result = interop_test_init_registry(GLOBUS_GLOBAL_TYPE_REGISTRY);
    if (result != GLOBUS_SUCCESS)
    {
        rc = 5;
        goto handle_destroy_out;
    }

    result = WidgetPortType_createWidget(
        client_handle,
        argv[1],
        &createWidget,
        &createWidgetResponse,
        (WidgetPortType_createWidget_fault_t *) &fault_type,
        &fault);

    if (result != GLOBUS_SUCCESS)
    {
        rc = 4;

        goto handle_destroy_out;
    }

    result = WidgetPortType_GetResourceProperty_epr(
        client_handle,
        &createWidgetResponse->EndpointReference,
        &foo_rp_qname,
        &GetResourcePropertyResponse,
        (WidgetPortType_GetResourceProperty_fault_t *) &fault_type,
        &fault);

    if (result != GLOBUS_SUCCESS)
    {
        rc = 5;

        goto handle_destroy_out;
    }
    if (!match_string_rp_value(GetResourcePropertyResponse, &foo_rp_qname, NULL))
    {
        rc = 6;

        goto handle_destroy_out;
    }
    wsrp_GetResourcePropertyResponseType_destroy(
        GetResourcePropertyResponse);
    GetResourcePropertyResponse = NULL;

    wsrp_SetResourcePropertiesType_init(&SetResourceProperties);

    SetResourcePropertiesType_choice =
        wsrp_SetResourcePropertiesType_choice_array_push(
            &SetResourceProperties->choice_value);
    memset(SetResourcePropertiesType_choice,
            '\0',
            sizeof(wsrp_SetResourcePropertiesType_choice));
    SetResourcePropertiesType_choice->type =
        wsrp_SetResourcePropertiesType_Insert;
    any = xsd_any_array_push(
        &SetResourcePropertiesType_choice->value.Insert.any);
    any->registry = NULL;
    any->any_info = &xsd_string_info;
    xsd_QName_init(&any->element);
    any->element->Namespace = globus_libc_strdup(foo_rp_qname.Namespace);
    any->element->local = globus_libc_strdup(foo_rp_qname.local);;
    xsd_string_init_cstr(
            (xsd_string **) &any->value,
            globus_libc_strdup(STRING1));

    result = WidgetPortType_SetResourceProperties_epr(
        client_handle,
        &createWidgetResponse->EndpointReference,
        SetResourceProperties,
        &SetResourcePropertiesResponse,
        (WidgetPortType_SetResourceProperties_fault_t *) &fault_type,
        &fault);
    if (result != GLOBUS_SUCCESS)
    {
        rc = 7;

        goto handle_destroy_out;
    }

    result = WidgetPortType_GetResourceProperty_epr(
        client_handle,
        &createWidgetResponse->EndpointReference,
        &foo_rp_qname,
        &GetResourcePropertyResponse,
        (WidgetPortType_GetResourceProperty_fault_t *) &fault_type,
        &fault);
    if (result != GLOBUS_SUCCESS)
    {
        rc = 8;

        goto handle_destroy_out;
    }

    if (!match_string_rp_value(
                GetResourcePropertyResponse,
                &foo_rp_qname,
                STRING1))
    {
        rc = 9;

        goto handle_destroy_out;
    }
    wsrp_GetResourcePropertyResponseType_destroy(
        GetResourcePropertyResponse);
    GetResourcePropertyResponse = NULL;

    wsrp_SetResourcePropertiesType_destroy(SetResourceProperties);
    wsrp_SetResourcePropertiesType_init(&SetResourceProperties);

    SetResourcePropertiesType_choice =
        wsrp_SetResourcePropertiesType_choice_array_push(
            &SetResourceProperties->choice_value);
    memset(SetResourcePropertiesType_choice,
            '\0',
            sizeof(wsrp_SetResourcePropertiesType_choice));
    SetResourcePropertiesType_choice->type =
        wsrp_SetResourcePropertiesType_Update;
    any = xsd_any_array_push(
        &SetResourcePropertiesType_choice->value.Update.any);
    any->registry = NULL;
    any->any_info = &xsd_string_info;
    xsd_QName_init(&any->element);
    any->element->Namespace = globus_libc_strdup(foo_rp_qname.Namespace);
    any->element->local = globus_libc_strdup(foo_rp_qname.local);;
    xsd_string_init_cstr(
            (xsd_string **) &any->value,
            globus_libc_strdup(STRING2));
    any = xsd_any_array_push(
        &SetResourcePropertiesType_choice->value.Update.any);
    any->registry = NULL;
    any->any_info = &xsd_string_info;
    xsd_QName_init(&any->element);
    any->element->Namespace = globus_libc_strdup(foo_rp_qname.Namespace);
    any->element->local = globus_libc_strdup(foo_rp_qname.local);;
    xsd_string_init_cstr(
            (xsd_string **) &any->value,
            globus_libc_strdup(STRING3));

    wsrp_SetResourcePropertiesResponseType_destroy(
        SetResourcePropertiesResponse);
    SetResourcePropertiesResponse = NULL;

    result = WidgetPortType_SetResourceProperties_epr(
        client_handle,
        &createWidgetResponse->EndpointReference,
        SetResourceProperties,
        &SetResourcePropertiesResponse,
        (WidgetPortType_SetResourceProperties_fault_t *) &fault_type,
        &fault);
    if (result != GLOBUS_SUCCESS)
    {
        rc = 7;

        goto handle_destroy_out;
    }

    wsrp_GetResourcePropertyResponseType_destroy(
        GetResourcePropertyResponse);

    GetResourcePropertyResponse = NULL;

    result = WidgetPortType_GetResourceProperty_epr(
        client_handle,
        &createWidgetResponse->EndpointReference,
        &foo_rp_qname,
        &GetResourcePropertyResponse,
        (WidgetPortType_GetResourceProperty_fault_t *) &fault_type,
        &fault);
    if (result != GLOBUS_SUCCESS)
    {
        rc = 8;

        goto handle_destroy_out;
    }

    if (!match_string_rp_value(
                GetResourcePropertyResponse,
                &foo_rp_qname,
                STRING2))
    {
        rc = 9;

        goto handle_destroy_out;
    }
    if (!match_string_rp_value(
                GetResourcePropertyResponse,
                &foo_rp_qname,
                STRING3))
    {
        rc = 10;

        goto handle_destroy_out;
    }

    wsrp_SetResourcePropertiesType_destroy(SetResourceProperties);

    wsrp_SetResourcePropertiesType_init(&SetResourceProperties);
    SetResourcePropertiesType_choice =
        wsrp_SetResourcePropertiesType_choice_array_push(
            &SetResourceProperties->choice_value);
    memset(SetResourcePropertiesType_choice,
            '\0',
            sizeof(wsrp_SetResourcePropertiesType_choice));
    SetResourcePropertiesType_choice->type =
        wsrp_SetResourcePropertiesType_Delete;
    SetResourcePropertiesType_choice->value.Delete._ResourceProperty.Namespace =
        globus_libc_strdup(foo_rp_qname.Namespace);
    SetResourcePropertiesType_choice->value.Delete._ResourceProperty.local =
        globus_libc_strdup(foo_rp_qname.local);

    wsrp_SetResourcePropertiesResponseType_destroy(
        SetResourcePropertiesResponse);
    SetResourcePropertiesResponse = NULL;

    result = WidgetPortType_SetResourceProperties_epr(
        client_handle,
        &createWidgetResponse->EndpointReference,
        SetResourceProperties,
        &SetResourcePropertiesResponse,
        (WidgetPortType_SetResourceProperties_fault_t *) &fault_type,
        &fault);
    if (result != GLOBUS_SUCCESS)
    {
        rc = 11;

        goto handle_destroy_out;
    }
    wsrp_GetResourcePropertyResponseType_destroy(
        GetResourcePropertyResponse);

    GetResourcePropertyResponse = NULL;

    result = WidgetPortType_GetResourceProperty_epr(
        client_handle,
        &createWidgetResponse->EndpointReference,
        &foo_rp_qname,
        &GetResourcePropertyResponse,
        (WidgetPortType_GetResourceProperty_fault_t *) &fault_type,
        &fault);
    if (result != GLOBUS_SUCCESS)
    {
        rc = 12;

        goto handle_destroy_out;
    }
    if (!match_string_rp_value(
                GetResourcePropertyResponse,
                &foo_rp_qname,
                NULL))
    {
        rc = 13;

        goto handle_destroy_out;
    }

handle_destroy_out:
    wsrp_GetResourcePropertyResponseType_destroy(GetResourcePropertyResponse);
    wsrp_SetResourcePropertiesResponseType_destroy(SetResourcePropertiesResponse);
    wsrp_SetResourcePropertiesType_destroy(SetResourceProperties);
    createWidgetResponseType_destroy(createWidgetResponse);
    WidgetService_client_destroy(client_handle);
deactivate_out:
    if(result != GLOBUS_SUCCESS)
    {
        char * error = globus_error_print_chain(globus_error_peek(result));
        fprintf(stderr, "%s\n", error);
        free(error);
    }
    globus_module_deactivate(GLOBUS_SOAP_MESSAGE_MODULE);
out:
    return rc;
}
/* main() */

static
globus_bool_t
match_string_rp_value(
    wsrp_GetResourcePropertyResponseType *
                                        response,
    xsd_QName *                         qname,
    xsd_string                          value)
{
    int i;

    for (i = 0; i < response->any.length; i++)
    {
        xsd_any *                       any;
        any = &response->any.elements[i];

        if (xsd_QName_keyeq(any->any_info->type, xsd_string_info.type) ||
            xsd_QName_keyeq(any->any_info->type, qname))
        {
            if (value != NULL)
            {
                if (strcmp(value, *(xsd_string *) any->value) == 0)
                {
                    return 1;
                }
            }
            else
            {
                return (* (xsd_string *) value) == NULL;
            }
        }
        else
        {
            return GLOBUS_FALSE;
        }
    }
    return (value == NULL);
}
/* match_string_rp_value() */
