/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */


#include "globus_service_engine.h"
#include "globus_service_registry.h"
#include "globus_wsrf_core_tools.h"
#include "globus_i_wsrf_core_tools.h"
#include "version.h"
#include "libxml/uri.h"
#include "globus_wsrf_resource.h"
#include "globus_error_string.h"
#include "wsbf_DescriptionType.h"
#include "globus_ws_addressing.h"


static int
globus_l_wsrf_core_tools_activate();

static int
globus_l_wsrf_core_tools_deactivate();

globus_module_descriptor_t              globus_l_wsrf_core_tools_module =
{
    "globus_wsrf_core_tools_module",
    globus_l_wsrf_core_tools_activate,
    globus_l_wsrf_core_tools_deactivate,
    NULL,
    NULL,
    &local_version
};

GlobusDebugDefine(GLOBUS_WSRF_CORE_TOOLS);



static int
globus_l_wsrf_core_tools_activate()
{
    int                                 rc = 0;
    GlobusFuncName(globus_l_wsrf_core_tools_activate);

    rc = globus_module_activate(GLOBUS_COMMON_MODULE);
    if(rc != GLOBUS_SUCCESS)
    {
        return rc;
    }

    GlobusDebugInit(GLOBUS_WSRF_CORE_TOOLS, DEBUG INFO TRACE WARN ERROR);
    GlobusWSRFCoreToolsDebugEnter();

    rc = globus_module_activate(GLOBUS_SOAP_MESSAGE_MODULE);
    if(rc != GLOBUS_SUCCESS)
    {
        globus_module_deactivate(GLOBUS_COMMON_MODULE);
        goto error_exit;
    }

    goto exit;

 error_exit:

    GlobusWSRFCoreToolsDebugExit();
    GlobusDebugDestroy(GLOBUS_WSRF_CORE_TOOLS);
    return rc;

 exit:

    GlobusWSRFCoreToolsDebugExit();
    return rc;
}

static int
globus_l_wsrf_core_tools_deactivate()
{
    int                                 rc = 0;
    GlobusFuncName(globus_l_wsrf_core_tools_deactivate);
    GlobusWSRFCoreToolsDebugEnter();

    rc = globus_module_deactivate(GLOBUS_SOAP_MESSAGE_MODULE);
    if(rc != GLOBUS_SUCCESS)
    {
        goto exit;
    }

 exit:

    GlobusWSRFCoreToolsDebugExit();
    GlobusDebugDestroy(GLOBUS_WSRF_CORE_TOOLS);
    globus_module_deactivate(GLOBUS_COMMON_MODULE);

    return rc;
}

#include "globus_xsd_primitives.h"
#include "globus_wsrf_core_tools.h"

globus_result_t
globus_wsrf_core_create_endpoint_reference(
    globus_service_engine_t             engine,
    const char *                        path,
    xsd_any * *                         resource_id,
    wsa_EndpointReferenceType   *       endpoint_reference)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    xsd_any *                           new_rp;
    char *                              contact;
    GlobusFuncName(globus_wsrf_core_create_endpoint_reference);
    GlobusWSRFCoreToolsDebugEnter();
    
    globus_service_engine_get_contact(engine, &contact);

    wsa_EndpointReferenceType_init_contents(endpoint_reference);

    endpoint_reference->Address.base_value = 
    globus_common_create_string("%s%s",
                                contact,
                                path);
    globus_free(contact);
    
    wsa_ReferencePropertiesType_init(&endpoint_reference->ReferenceProperties);
    new_rp = xsd_any_array_push(&endpoint_reference->ReferenceProperties->any);
    
    new_rp->value = (*resource_id)->value;
    new_rp->element = (*resource_id)->element;
    new_rp->any_info = (*resource_id)->any_info;

    (*resource_id)->value = NULL;
    (*resource_id)->element = NULL;
    xsd_any_destroy(*resource_id);
    
    *resource_id = NULL;

    GlobusWSRFCoreToolsDebugExit();
    return result;
}

globus_result_t
globus_wsrf_core_unwrap_endpoint_reference(
    const char *                        wrapped_uri,
    xsd_QName *                         id_element,
    wsa_EndpointReferenceType   *       endpoint_reference)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    xsd_any *                           new_rp;
    xmlURIPtr                           parsed_uri;
    GlobusFuncName(globus_wsrf_core_unwrap_endpoint_reference);
    GlobusWSRFCoreToolsDebugEnter();
    
    parsed_uri = xmlParseURI(wrapped_uri);
    
    wsa_ReferencePropertiesType_init(&endpoint_reference->ReferenceProperties);
    new_rp = xsd_any_array_push(&endpoint_reference->ReferenceProperties->any);
   
    xsd_string_copy_cstr((xsd_string **) &new_rp->value, parsed_uri->query);
    xsd_QName_copy(&new_rp->element, id_element);
    new_rp->any_info = &xsd_string_info;

    xmlFree(parsed_uri->query);
    parsed_uri->query = NULL; 
    endpoint_reference->Address.base_value = xmlSaveUri(parsed_uri);

    xmlFreeURI(parsed_uri);

    GlobusWSRFCoreToolsDebugExit();
    return result;
}

globus_result_t
globus_wsrf_core_wrap_endpoint_reference(
    wsa_EndpointReferenceType *         endpoint_reference,
    char **                             wrapped_uri)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_wsrf_core_wrap_endpoint_reference);
    GlobusWSRFCoreToolsDebugEnter();

    /* verify endpoint reference property is xsd_string type */

    if(endpoint_reference->ReferenceProperties)
    {
        result = GlobusWSRFCoreToolsErrorNullReferenceProperty();
        goto exit;
    }
    
    if(endpoint_reference->ReferenceProperties->any.length != 1)
    {
        result = GlobusWSRFCoreToolsErrorReferencePropertiesWrongCount(
            endpoint_reference->ReferenceProperties->any.length);
        goto exit;
    }

    if(endpoint_reference->ReferenceProperties->any.elements[0].any_info !=
       (&xsd_string_info))
    {
        result = GlobusWSRFCoreToolsErrorInvalidReferenceProperties(
            endpoint_reference->ReferenceProperties->
            any.elements[0].any_info->type);
        goto exit;
    }

    *wrapped_uri = globus_common_create_string(
        "%s?%s",
        endpoint_reference->Address.base_value,
        endpoint_reference->ReferenceProperties->any.elements[0].value);

 exit:

    GlobusWSRFCoreToolsDebugExit();
    return result;
}

globus_result_t
globus_wsrf_core_get_resource(
    globus_soap_message_handle_t        message,
    globus_service_descriptor_t *       service,
    globus_resource_t *                 resource)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    char *                              id = NULL;
    wsa_EndpointReferenceType *         epr;
    GlobusFuncName(globus_wsrf_core_get_resource_id);
    GlobusWSRFCoreToolsDebugEnter();
    
    epr = globus_soap_message_handle_get_attr(
        message, 
        WSADDR_EPR_KEY);
    if(!epr)
    {
        result = GlobusWSRFCoreToolsErrorGetResourceFailed(
            GLOBUS_SUCCESS, 
            "No EndpointReference found "
            "in message properties");
        goto exit;
    }

    result = service->get_resource_id(&epr->ReferenceProperties->any, &id);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusWSRFCoreToolsErrorGetResourceFailed(
            result, "Failed to get resource id from ReferenceProperties");
        goto exit;
    }

    result = globus_resource_find(id, resource);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusWSRFCoreToolsErrorGetResourceFailed(
            result, "No resource found in resource registry");
        goto exit;
    }

 exit:

    if(id)
    {
        free(id);
    }
    
    GlobusWSRFCoreToolsDebugExit();
    return result;
}

globus_result_t
globus_wsrf_core_get_resource_from_epr(
    wsa_EndpointReferenceType *         epr,
    globus_resource_t *                 resource)
{
    xmlURIPtr                           uri = NULL;
    xmlChar *                           base_path = NULL;
    char *                              service_module_path = NULL;
    globus_extension_handle_t           ext;
    globus_service_descriptor_t *       service = NULL;
    char *                              id = NULL;
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_wsrf_core_get_resource_from_epr);
    GlobusWSRFCoreToolsDebugEnter();
    
    globus_assert(epr);
    globus_assert(epr->Address.base_value);

    uri = xmlParseURI(epr->Address.base_value);
    if(!uri)
    {
        result = GlobusWSRFCoreToolsErrorGetResourceFailed(
            GLOBUS_SUCCESS, "Failed to parse EPR Address");
        goto exit;
    }

    base_path = uri->path;
    while(*base_path == '/')
    {
        base_path++;
    }

    service_module_path = globus_common_create_string(
            GLOBUS_SERVICE_ENGINE_MODULE_PATH_PREFIX "/%s",
            base_path);

    if (service_module_path == NULL)
    {
        result = GlobusWSRFCoreToolsErrorGetResourceFailed(
            GLOBUS_SUCCESS, "Failed to parse EPR Address");
        goto exit;
    }

    service = globus_extension_lookup(
        &ext, GLOBUS_SERVICE_REGISTRY, service_module_path); 
    if(!service)
    {
        result = GlobusWSRFCoreToolsErrorGetResourceFailed(
            result, "Failed to get service descriptor from EPR Address");
        goto exit;
    }

    result = service->get_resource_id(&epr->ReferenceProperties->any, &id);
    if(result != GLOBUS_SUCCESS)
    {
        globus_extension_release(ext);
        result = GlobusWSRFCoreToolsErrorGetResourceFailed(
            result, "Failed to get resource id from ReferenceProperties");
        goto exit;
    }

    globus_extension_release(ext);

    result = globus_resource_find(id, resource);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusWSRFCoreToolsErrorGetResourceFailed(
            result, "No resource found in resource registry");
        goto exit;
    }

 exit:    
    if(service_module_path)
    {
        globus_libc_free(service_module_path);
    }

    if(uri)
    {
        xmlFreeURI(uri);
    }

    if(id)
    {
        free(id);
    }

    GlobusWSRFCoreToolsDebugExit();
    return result;
}

char *
globus_wsrf_core_export_timestamp(
    xsd_dateTime *                      val)
{
    return globus_common_create_string(
        "%.4d-%.2d-%.2dT%.2d:%.2d:%.2dZ", 
            val->tm_year + 1900,
            val->tm_mon + 1,
            val->tm_mday,
            val->tm_hour,
            val->tm_min,
            val->tm_sec);
}

#include "libxml/xmlschemastypes.h"

xsd_dateTime *
globus_wsrf_core_import_timestamp(
    const char *                        timestamp_string)
{
    xsd_dateTime *                      val = NULL;
    xmlSchemaValPtr                     schema_val = NULL;
    xmlChar *                           tmp_value = NULL;
    int                                 res = 0;

    xsd_dateTime_init(&val);

    res = xmlSchemaValidatePredefinedType(
        xmlSchemaGetBuiltInType(XML_SCHEMAS_DATETIME),
        tmp_value,
        &schema_val);
    if(res != 0)
    {
        xsd_dateTime_destroy(val);
        return NULL;
    }

    val->tm_year = schema_val->value.date.year - 1900;
    val->tm_mon = schema_val->value.date.mon;
    val->tm_mday = schema_val->value.date.day;
    val->tm_hour = schema_val->value.date.hour;
    val->tm_min = schema_val->value.date.min;
    val->tm_sec = schema_val->value.date.sec;
    if(schema_val->value.date.tz_flag)
    {
        val->tm_min += schema_val->value.date.tzo;
    }

    xmlSchemaFreeValue(schema_val);

    return val;
}

static
globus_object_t *
globus_l_error_fault_multiple(
    const wsbf_BaseFaultType_array *    fault_array)
{
    globus_object_t *                   error;
    int                                 i;
    
    error = globus_error_construct_multiple(NULL, 0, "");
    
    for(i = 0; i < fault_array->length; i++)
    {
        globus_error_mutliple_add_chain(
            error,
            globus_error_convert_wsrf_fault(&fault_array->elements[i]),
            "");
    }
    
    return error;
}

globus_object_t *
globus_error_convert_wsrf_fault(
    const wsbf_BaseFaultType *          fault)
{
    globus_object_t *                   cause;
    char *                              description = NULL;
    char **                             array;
    
    if(fault->FaultCause.length == 0)
    {
        cause = NULL;
    }
    else if(fault->FaultCause.length == 1)
    {
        cause = globus_error_convert_wsrf_fault(&fault->FaultCause.elements[0]);
    }
    else
    {
        cause = globus_l_error_fault_multiple(&fault->FaultCause);
    }
    
    array = (char **) globus_malloc(
        sizeof(char *) * fault->Description.length * 2);
    if(array)
    {
        int                             i;
        int                             j = 0;
        
        for(i = 0; i < fault->Description.length; i++)
        {
            if(i > 0)
            {
                array[j++] = "\n";
            }
            array[j++] = fault->Description.elements[i].base_value;
        }
        
        description = globus_libc_join((const char **)array, j);
        globus_free(array);
    }
    
    return globus_error_construct_string(NULL, cause, "%s", description);
}

static xsd_QName                               generic_epr =
{
    WSA_ENDPOINTREFERENCETYPE_NS,
    "EndpointReference"
};

globus_result_t
globus_wsrf_core_export_endpoint_reference(
    const wsa_EndpointReferenceType *   endpoint_reference,
    const char *                        filename,
    xsd_QName *                         element)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    xsd_QName *                         elem = element;
    GlobusFuncName(globus_wsrf_core_export_endpoint_reference);
    GlobusWSRFCoreToolsDebugEnter();
    
    if(!elem)
    {
        elem = &generic_epr;
    }

    GlobusXSDTypeExport(filename, 
                        wsa_EndpointReferenceType,
                        endpoint_reference,
                        elem,
                        result);

    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusWSRFCoreToolsErrorExportFailed(result, filename);
    }

    GlobusWSRFCoreToolsDebugExit();
    return result;
}

globus_result_t
globus_wsrf_core_import_endpoint_reference(
    const char *                        filename,
    wsa_EndpointReferenceType **        endpoint_reference,
    xsd_QName *                         element)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_wsrf_core_import_endpoint_reference);
    GlobusWSRFCoreToolsDebugEnter();
    
    wsa_EndpointReferenceType_init(endpoint_reference);

    GlobusXSDTypeImport(filename,
                        wsa_EndpointReferenceType,
                        (*endpoint_reference),
                        element,
                        result);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusWSRFCoreToolsErrorImportFailed(result, filename);
    }

    GlobusWSRFCoreToolsDebugExit();
    return result;
}
