/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */


#include "globus_ws_addressing.h"
#include "globus_extension.h"
#include "globus_common.h"
#include "globus_handler.h"
#include "globus_soap_message_fault.h"
#include "globus_xml_buffer.h"
#include "globus_soap_message_attrs.h"

#include "wsa_AttributedURI.h"
#include "wsa_Relationship.h"
#include "wsa_EndpointReferenceType.h"

#include "wsa_MessageID.h"
#include "wsa_RelatesTo.h"
#include "wsa_To.h"
#include "wsa_Action.h"
#include "wsa_From.h"
#include "wsa_ReplyTo.h"
#include "wsa_FaultTo.h"

#include "version.h"

#define WS_ADDRESSING_CLIENT_REQUEST_ALREADY_INVOKED \
        "WS_ADDRESSING_CLIENT_REQUEST_ALREADY_INVOKED"

#define WS_ADDRESSING_CLIENT_MESSAGEID_ALREADY_INVOKED \
        "WS_ADDRESSING_CLIENT_MESSAGEID_ALREADY_INVOKED"

#define WS_ADDRESSING_CLIENT_RELATESTO_ALREADY_INVOKED \
        "WS_ADDRESSING_CLIENT_RELATESTO_ALREADY_INVOKED"

#define WS_ADDRESSING_CLIENT_TO_ALREADY_INVOKED \
        "WS_ADDRESSING_CLIENT_TO_ALREADY_INVOKED"

#define WS_ADDRESSING_CLIENT_ACTION_ALREADY_INVOKED \
        "WS_ADDRESSING_CLIENT_ACTION_ALREADY_INVOKED"

#define WS_ADDRESSING_CLIENT_FROM_ALREADY_INVOKED \
        "WS_ADDRESSING_CLIENT_FROM_ALREADY_INVOKED"

#define WS_ADDRESSING_CLIENT_RESPONSE_ALREADY_INVOKED \
        "WS_ADDRESSING_CLIENT_RESPONSE_ALREADY_INVOKED"

#define WS_ADDRESSING_SERVER_RESPONSE_ALREADY_INVOKED \
        "WS_ADDRESSING_SERVER_RESPONSE_ALREADY_INVOKED"

#define WS_ADDRESSING_SERVER_MESSAGEID_ALREADY_INVOKED \
        "WS_ADDRESSING_SERVER_MESSAGEID_ALREADY_INVOKED"

#define WS_ADDRESSING_SERVER_RELATESTO_ALREADY_INVOKED \
        "WS_ADDRESSING_SERVER_RELATESTO_ALREADY_INVOKED"

#define WS_ADDRESSING_SERVER_TO_ALREADY_INVOKED \
        "WS_ADDRESSING_SERVER_TO_ALREADY_INVOKED"

#define WS_ADDRESSING_SERVER_ACTION_ALREADY_INVOKED \
        "WS_ADDRESSING_SERVER_ACTION_ALREADY_INVOKED"

#define WS_ADDRESSING_SERVER_FROM_ALREADY_INVOKED \
        "WS_ADDRESSING_SERVER_FROM_ALREADY_INVOKED"

#define WS_ADDRESSING_SERVER_REQUEST_ALREADY_INVOKED \
        "WS_ADDRESSING_SERVER_REQUEST_ALREADY_INVOKED"


int
globus_l_handler_ws_addressing_activate(void);

int
globus_l_handler_ws_addressing_deactivate(void);


GlobusExtensionDefineModule(globus_handler_ws_addressing) =
{
    GLOBUS_HANDLER_WS_ADDRESSING_MODULE,
    globus_l_handler_ws_addressing_activate,
    globus_l_handler_ws_addressing_deactivate,
    NULL,
    NULL,
    &local_version
};

#define GLOBUS_L_WS_ADDRESSING_DEBUG_TRACE 2
#define GLOBUS_L_WS_ADDRESSING_DEBUG_DEBUG 4

GlobusDebugDefine(GLOBUS_HANDLER_WS_ADDRESSING);

#define GlobusWSAddressingDebugPrintf(LEVEL, MESSAGE) \
    GlobusDebugPrintf(GLOBUS_HANDLER_WS_ADDRESSING, LEVEL, MESSAGE)

#define GlobusWSAddressingDebugEnter() \
    GlobusWSAddressingDebugPrintf(GLOBUS_L_WS_ADDRESSING_DEBUG_TRACE, \
                      ("[%s] Entering", _globus_func_name))

#define GlobusWSAddressingDebugExit() \
    GlobusWSAddressingDebugPrintf(GLOBUS_L_WS_ADDRESSING_DEBUG_TRACE, \
                      ("[%s] Exiting", _globus_func_name))

#define GlobusWSAddressingErrorSerializeFailed(ERROR, MESSAGE)          \
    globus_error_put(                                                   \
        globus_error_construct_error(                                   \
            GlobusExtensionMyModule(globus_handler_ws_addressing),      \
            (ERROR ? globus_error_get(ERROR) : NULL),                   \
            0,                                                          \
            __FILE__,                                                   \
            _globus_func_name,                                          \
            __LINE__,                                                   \
            MESSAGE))

#define GlobusWSAddressingErrorDeserializeFailed(ERROR, MESSAGE)        \
    globus_error_put(                                                   \
        globus_error_construct_error(                                   \
            GlobusExtensionMyModule(globus_handler_ws_addressing),      \
            (ERROR ? globus_error_get(ERROR) : NULL),                   \
            0,                                                          \
            __FILE__,                                                   \
            _globus_func_name,                                          \
            __LINE__,                                                   \
            globus_common_create_string MESSAGE))

#define GlobusWSAddressingErrorRequiredElementNotFound(MESSAGE)         \
    globus_error_put(                                                   \
        globus_error_construct_error(                                   \
            GlobusExtensionMyModule(globus_handler_ws_addressing),      \
            NULL,                                                       \
            0,                                                          \
            __FILE__,                                                   \
            _globus_func_name,                                          \
            __LINE__,                                                   \
            "Required element %s not found", MESSAGE))

#define GlobusWSAddressingErrorMemoryAllocationFailed(LENGTH, MESSAGE) \
    globus_error_put(                                                  \
        globus_error_construct_error(                                  \
            GlobusExtensionMyModule(globus_handler_ws_addressing),     \
            NULL,                                                      \
            0,                                                         \
            __FILE__,                                                  \
            _globus_func_name,                                         \
            __LINE__,                                                  \
            "Memory allocation failed (%d bytes): %s",                 \
            LENGTH, MESSAGE))

#define GlobusWSAddressingErrorFailedDefaultTrigger(ELEMENT)        \
    globus_error_put(                                               \
	globus_error_construct_error(                               \
	    GlobusExtensionMyModule(globus_handler_ws_addressing),  \
	    NULL,                                                   \
	    0,                                                      \
	    __FILE__,                                               \
	    _globus_func_name,                                      \
	    __LINE__,                                               \
	    "Failed to handle {%s}%s element",                      \
	    ELEMENT->Namespace,                                     \
	    ELEMENT->local))

static
globus_result_t
globus_l_uuid_copy(
    void **                             new_id,
    void *                              id);

void
globus_handler_ws_addressing_client_request_invoke(
    globus_handler_session_descriptor_t session,
    globus_soap_message_handle_t        message_handle,
    globus_result_t                     result)
{
    globus_uuid_t *                     uuid = NULL;
    char *                              uuid_string = NULL;
    wsa_AttributedURI                   MessageID;
    wsa_AttributedURI                   To;
    wsa_AttributedURI                   Action;
    char *                              action_uri = NULL;
    wsa_EndpointReferenceType           default_From;
    wsa_EndpointReferenceType *         source_endpoint = NULL;
    wsa_EndpointReferenceType *         reply_endpoint = NULL;
    wsa_EndpointReferenceType *         fault_endpoint = NULL;
    wsa_EndpointReferenceType *         endpoint_reference = NULL;
    GlobusFuncName(globus_ws_addressing_handler_client_request_invoke);
    GlobusWSAddressingDebugEnter();

    if(result != GLOBUS_SUCCESS)
    {
        goto exit;
    }

    if(globus_soap_message_handle_get_attr(
	    message_handle, 
	    WS_ADDRESSING_CLIENT_REQUEST_ALREADY_INVOKED))
    {
	goto exit;
    }

    endpoint_reference = (wsa_EndpointReferenceType *)
        globus_soap_message_handle_get_attr(
            message_handle, WSADDR_OUTBOUND_EPR_KEY);
    if(!endpoint_reference)
    {
	endpoint_reference = (wsa_EndpointReferenceType *)
	    globus_soap_message_handle_get_attr(
		message_handle, WSADDR_EPR_KEY);
    }
 
    uuid = globus_malloc(sizeof(globus_uuid_t));
    if(!uuid)
    {
        result = GlobusWSAddressingErrorMemoryAllocationFailed(
            sizeof(globus_uuid_t), "uuid");
        goto exit;
    }

    globus_uuid_create(uuid);

    uuid_string = globus_common_create_string("uuid:%s", uuid->text);
    MessageID.base_value = uuid_string;
    MessageID.anyAttributes = NULL;

    result = wsa_AttributedURI_serialize(
        (xsd_QName *)&wsa_MessageID_qname,
        &MessageID,
        message_handle,
        0);
    if(result != GLOBUS_SUCCESS)
    {
	globus_free(uuid_string);
	globus_free(uuid);
        result = GlobusWSAddressingErrorSerializeFailed(
            result, "MessageID element");
        goto exit;
    }

    globus_free(uuid_string);

    globus_soap_message_handle_set_attr(
        message_handle,
        WSADDR_OUTBOUND_MESSAGE_ID_KEY,
        globus_l_uuid_copy,
        globus_libc_free,
        (void *) uuid);

    globus_free(uuid);

    To.base_value = (char *) globus_soap_message_handle_get_attr(
        message_handle, WSADDR_DESTINATION_KEY);
    if(!To.base_value)
    {
        result = GlobusWSAddressingErrorSerializeFailed(
            GLOBUS_SUCCESS, "No destination endpoint");
        goto exit;
    }
    
    To.anyAttributes = NULL;

    result = wsa_AttributedURI_serialize(
        (xsd_QName *)&wsa_To_qname,
        &To,
        message_handle,
        0);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusWSAddressingErrorSerializeFailed(
            result, "To element");
        goto exit;
    }
    
    action_uri = (char *) globus_soap_message_handle_get_attr(
        message_handle, WSADDR_ACTION_REQUEST_KEY);
    if(!action_uri)
    {
        result = GlobusWSAddressingErrorSerializeFailed(
            GLOBUS_SUCCESS, "No SOAPAction defined");
        goto exit;
    }

    Action.base_value = action_uri;
    Action.anyAttributes = NULL;

    result = wsa_AttributedURI_serialize(
        (xsd_QName *)&wsa_Action_qname,
        &Action,
        message_handle,
        0);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusWSAddressingErrorSerializeFailed(
            result, "Action element");
        goto exit;
    }

    source_endpoint = globus_soap_message_handle_get_attr(
        message_handle,
        WSADDR_SOURCE_ENDPOINT_KEY);
    if(!source_endpoint)
    {
        memset(&default_From, 0, sizeof(wsa_EndpointReferenceType));
        default_From.Address.base_value = WSADDRESSING_ANON_NS;
        source_endpoint = &default_From;
    }

    result = wsa_EndpointReferenceType_serialize(
        (xsd_QName *)&wsa_From_qname,
        source_endpoint,
        message_handle,
        0);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusWSAddressingErrorSerializeFailed(
            result, "From element");
        goto exit;
    }

    reply_endpoint = globus_soap_message_handle_get_attr(
        message_handle,
        WSADDR_REPLY_ENDPOINT_KEY);
    if(reply_endpoint)
    {
        
        result = wsa_EndpointReferenceType_serialize(
            (xsd_QName *)&wsa_ReplyTo_qname,
            reply_endpoint,
            message_handle,
            0);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusWSAddressingErrorSerializeFailed(
                result, "ReplyTo element");
            goto exit;
        }
    }

    fault_endpoint = globus_soap_message_handle_get_attr(
        message_handle,
        WSADDR_FAULT_ENDPOINT_KEY);
    if(fault_endpoint)
    {
        result = wsa_EndpointReferenceType_serialize(
            (xsd_QName *)&wsa_FaultTo_qname,
            fault_endpoint,
            message_handle,
            0);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusWSAddressingErrorSerializeFailed(
                result, "FaultTo element");
            goto exit;
        }
    }

    if(endpoint_reference && endpoint_reference->ReferenceProperties)
    {
        wsa_ReferencePropertiesType * props;
        int i = 0;

        props = endpoint_reference->ReferenceProperties;
        for(; i < props->any.length; ++i)
        {
            result = props->any.elements[i].any_info->serialize(
                props->any.elements[i].element,
                props->any.elements[i].value,
                message_handle,
                0);
            if(result != GLOBUS_SUCCESS)
            {
                result = GlobusWSAddressingErrorSerializeFailed(
                    result, "Reference Properties");
                goto exit;
            }
        }
    }

    globus_soap_message_handle_set_attr(
	message_handle, 
	WS_ADDRESSING_CLIENT_REQUEST_ALREADY_INVOKED,
	NULL, NULL, (void *)1);

 exit:
	
    globus_handler_finished(session, result); 
    GlobusWSAddressingDebugExit();
}

static
void
globus_l_handler_ws_addressing_client_invoke_MessageID(
    globus_handler_session_descriptor_t session,
    globus_soap_message_handle_t        message_handle,
    globus_result_t                     result)
{
    wsa_AttributedURI                   value;
    globus_uuid_t *                     message_uuid = NULL;
    GlobusFuncName(globus_ws_addressing_handler_client_invoke_MessageID);
    GlobusWSAddressingDebugEnter();

    if(result != GLOBUS_SUCCESS)
    {
        goto exit;
    }

    if(globus_soap_message_handle_get_attr(
	    message_handle,
	    WS_ADDRESSING_CLIENT_MESSAGEID_ALREADY_INVOKED))
    {
	goto exit;
    }
    
    wsa_AttributedURI_init_contents(&value);

    result = wsa_AttributedURI_deserialize(
        &wsa_MessageID_qname,
        &value,
        message_handle,
        0);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusWSAddressingErrorDeserializeFailed(
            result,
            ("Failed to deserialize MessageID"));
        goto exit;
    }

    message_uuid = globus_malloc(sizeof(globus_uuid_t));
    if(!message_uuid)
    {
        result = GlobusWSAddressingErrorMemoryAllocationFailed(
            sizeof(globus_uuid_t), "message uuid");
        goto exit;
    }

    result = globus_uuid_import(
        message_uuid, value.base_value + 5);
    if(result != GLOBUS_SUCCESS)
    {
        globus_free(message_uuid);
        result = GlobusWSAddressingErrorDeserializeFailed(
            GLOBUS_SUCCESS,
            ("Failed to import uuid from string; %s", 
             value.base_value));
        goto MessageID_destroy;
    }

    globus_soap_message_handle_set_attr(
        message_handle,
        WSADDR_MESSAGE_ID_KEY,
        globus_l_uuid_copy,
        globus_libc_free,
        (void *) message_uuid);

    globus_free(message_uuid);
    
 MessageID_destroy:

    wsa_AttributedURI_destroy_contents(&value);

    globus_soap_message_handle_set_attr(
	message_handle,
	WS_ADDRESSING_CLIENT_MESSAGEID_ALREADY_INVOKED,
	NULL, NULL, (void *)1);

 exit:
   
    GlobusWSAddressingDebugExit();
    globus_handler_finished(session, result);
    return;
}

static
void
globus_l_handler_ws_addressing_client_invoke_RelatesTo(
    globus_handler_session_descriptor_t session,
    globus_soap_message_handle_t        message_handle,
    globus_result_t                     result)
{
    wsa_Relationship                    value;
    globus_uuid_t *                     message_uuid;
    globus_uuid_t *                     outbound_uuid = NULL;
    GlobusFuncName(globus_ws_addressing_handler_client_invoke_RelatesTo);
    GlobusWSAddressingDebugEnter();

    if(result != GLOBUS_SUCCESS)
    {
        goto exit;
    }

    if(globus_soap_message_handle_get_attr(
	    message_handle,
	    WS_ADDRESSING_CLIENT_RELATESTO_ALREADY_INVOKED))
    {
	goto exit;
    }
    
    wsa_Relationship_init_contents(&value);

    result = wsa_Relationship_deserialize(
        &wsa_RelatesTo_qname,
        &value,
        message_handle,
        0);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusWSAddressingErrorDeserializeFailed(
            result,
            ("Failed to deserialize RelatesTo"));
        goto exit;
    }

    if(!value.base_value)
    {
	result = GlobusWSAddressingErrorDeserializeFailed(
	    GLOBUS_SUCCESS,
	    ("Failed to get required UUID value from "
	     "message information header element: RelatesTo"));
	goto exit;
    }

    message_uuid = globus_malloc(sizeof(globus_uuid_t));
    if(!message_uuid)
    {
        result = GlobusWSAddressingErrorMemoryAllocationFailed(
            sizeof(globus_uuid_t), "message uuid");
        goto exit;
    }

    result = globus_uuid_import(
        message_uuid, value.base_value + 5);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusWSAddressingErrorDeserializeFailed(
            GLOBUS_SUCCESS,
            ("Failed to import uuid from string; %s", 
             value));
        goto RelatesTo_destroy;
    }

    outbound_uuid = 
    (globus_uuid_t *) globus_soap_message_handle_get_attr(
        message_handle,
        WSADDR_OUTBOUND_MESSAGE_ID_KEY);
    if(!outbound_uuid)
    {
        result = GlobusWSAddressingErrorDeserializeFailed(
            GLOBUS_SUCCESS,
            ("Failed to get request message UUID"
             " from message properties"));
        goto RelatesTo_destroy;
    }

    if(! GLOBUS_UUID_MATCH((*outbound_uuid), (*message_uuid)))
    {
        result = GlobusWSAddressingErrorDeserializeFailed(
            GLOBUS_SUCCESS, 
            ("UUID match failure: request message UUID: %s\n"
             "                    response message UUID: %s\n",
             outbound_uuid->text, message_uuid->text));
        goto RelatesTo_destroy;
    }

    globus_soap_message_handle_set_attr(
        message_handle,
        WSADDR_RELATES_TO_KEY,
        globus_l_uuid_copy,
        globus_libc_free,
        (void *)message_uuid);
   
    globus_free(message_uuid);
    
 RelatesTo_destroy:

    wsa_Relationship_destroy_contents(&value);

    globus_soap_message_handle_set_attr(
	message_handle,
	WS_ADDRESSING_CLIENT_RELATESTO_ALREADY_INVOKED,
	NULL, NULL, (void *)1);

 exit:
    
    GlobusWSAddressingDebugExit();
    globus_handler_finished(session, result);
    return;
}

static
void
globus_l_handler_ws_addressing_client_invoke_To(
    globus_handler_session_descriptor_t session,
    globus_soap_message_handle_t        message_handle,
    globus_result_t                     result)
{
    wsa_AttributedURI                   value;
    char *                              source_endpoint = NULL;
    GlobusFuncName(globus_ws_addressing_handler_client_invoke_To);
    GlobusWSAddressingDebugEnter();

    if(result != GLOBUS_SUCCESS)
    {
        goto exit;
    }

    if(globus_soap_message_handle_get_attr(
	    message_handle,
	    WS_ADDRESSING_CLIENT_TO_ALREADY_INVOKED))
    {
	goto exit;
    }
    
    wsa_AttributedURI_init_contents(&value);

    result = wsa_AttributedURI_deserialize(
        &wsa_To_qname,
        &value,
        message_handle,
        0);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusWSAddressingErrorDeserializeFailed(
            result,
            ("Failed to deserialize Action"));
        goto exit;
    }

    source_endpoint = globus_soap_message_handle_get_attr(
        message_handle,
        WSADDR_SOURCE_ENDPOINT_KEY);
    if(source_endpoint && strcmp(source_endpoint, value.base_value))
    {
        result = GlobusWSAddressingErrorDeserializeFailed(
            GLOBUS_SUCCESS,
            ("Expected source endpoint: %s in "
             "To field of reply, received: %s",
             source_endpoint,
             value.base_value));
        goto To_destroy;
    }
    else if(!source_endpoint && strncmp(WSADDRESSING_ANON_NS, 
                                        value.base_value, 
                                        sizeof(WSADDRESSING_ANON_NS)))
    {
        result = GlobusWSAddressingErrorDeserializeFailed(
            GLOBUS_SUCCESS,
            ("Expected anonymous endpoint in "
             "To field of reply"));
        goto To_destroy;
    }

 To_destroy:

    wsa_AttributedURI_destroy_contents(&value);

    globus_soap_message_handle_set_attr(
	message_handle,
	WS_ADDRESSING_CLIENT_TO_ALREADY_INVOKED,
	NULL, NULL, (void *)1);

 exit:
    
    GlobusWSAddressingDebugExit();
    globus_handler_finished(session, result);
    return;
}

static
void
globus_l_handler_ws_addressing_client_invoke_Action(
    globus_handler_session_descriptor_t session,
    globus_soap_message_handle_t        message_handle,
    globus_result_t                     result)
{
    wsa_AttributedURI                   value;
    char *                              response_action = NULL;
    GlobusFuncName(globus_ws_addressing_handler_client_invoke_Action);
    GlobusWSAddressingDebugEnter();

    if(result != GLOBUS_SUCCESS)
    {
        goto exit;
    }

    if(globus_soap_message_handle_get_attr(
	    message_handle,
	    WS_ADDRESSING_CLIENT_ACTION_ALREADY_INVOKED))
    {
	goto exit;
    }
    
    wsa_AttributedURI_init_contents(&value);

    result = wsa_AttributedURI_deserialize(
        &wsa_Action_qname,
        &value,
        message_handle,
        0);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusWSAddressingErrorDeserializeFailed(
            result,
            ("Failed to deserialize Action"));
        goto exit;
    }

    if(!strncmp(WSADDRESSING_FAULT_NS, 
                value.base_value, 
                sizeof(WSADDRESSING_FAULT_NS)))
    {
        /* found a fault */
        result = GLOBUS_SUCCESS;
        goto Action_destroy;
    }

    response_action = globus_soap_message_handle_get_attr(
        message_handle,
        WSADDR_ACTION_RESPONSE_KEY);
    if(!response_action || strcmp(response_action, value.base_value))
    {
        result = GlobusWSAddressingErrorDeserializeFailed(
            GLOBUS_SUCCESS,
            ("\nExpected Action element: %s"
             "\n             But received: %s",
             response_action,
             (value.base_value ? value.base_value : "(null)")));

        goto Action_destroy;
    }

    globus_soap_message_handle_set_attr(
        message_handle,
        WSADDR_ACTION_RESPONSE_KEY,
        globus_soap_message_attr_copy_string,
        globus_libc_free,
        value.base_value);

 Action_destroy:

    wsa_AttributedURI_destroy_contents(&value);

    globus_soap_message_handle_set_attr(
	message_handle,
	WS_ADDRESSING_CLIENT_ACTION_ALREADY_INVOKED,
	NULL, NULL, (void *)1);

 exit:
    
    GlobusWSAddressingDebugExit();
    globus_handler_finished(session, result);
    return;
}

static
void
globus_l_handler_ws_addressing_client_invoke_From(
    globus_handler_session_descriptor_t session,
    globus_soap_message_handle_t        message_handle,
    globus_result_t                     result)
{
    char *                              destination;
    wsa_EndpointReferenceType *         from_epr;
    GlobusFuncName(globus_ws_addressing_handler_client_invoke_From);
    GlobusWSAddressingDebugEnter();

    if(result != GLOBUS_SUCCESS)
    {
        goto exit;
    }

    if(globus_soap_message_handle_get_attr(
	    message_handle,
	    WS_ADDRESSING_CLIENT_FROM_ALREADY_INVOKED))
    {
	goto exit;
    }
    
    wsa_EndpointReferenceType_init(
        &from_epr);

    result = wsa_EndpointReferenceType_deserialize(
        &wsa_From_qname,
        from_epr,
        message_handle,
        0);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusWSAddressingErrorDeserializeFailed(
            result, ("Failed to deserialize From element"));
        goto exit;
    }

    destination = 
    globus_soap_message_handle_get_attr(
        message_handle,
        WSADDR_DESTINATION_KEY);
    if(!destination)
    {
        result = GlobusWSAddressingErrorDeserializeFailed(
            GLOBUS_SUCCESS,
            ("No destination endpoint"));
        goto destroy_From;
    }

    if(xmlStrcmp(destination, from_epr->Address.base_value))
    {
        result = GlobusWSAddressingErrorDeserializeFailed(
            GLOBUS_SUCCESS,
            ("Received From:Address: %s\nExpected: %s",
             from_epr->Address.base_value, destination));
        goto destroy_From;
    }

    result = globus_soap_message_handle_set_attr(
        message_handle, WSADDR_EPR_KEY,
        wsa_EndpointReferenceType_copy_wrapper,
        wsa_EndpointReferenceType_destroy_wrapper, 
        (void *)from_epr);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusWSAddressingErrorDeserializeFailed(
            result, ("Failed to set EndpointReference "
                     "in message properties"));
        goto destroy_From;
    }

 destroy_From:

    wsa_EndpointReferenceType_destroy(from_epr);

    globus_soap_message_handle_set_attr(
	message_handle,
	WS_ADDRESSING_CLIENT_FROM_ALREADY_INVOKED,
	NULL, NULL, (void *)1);

 exit:
    
    GlobusWSAddressingDebugExit();
    globus_handler_finished(session, result);
    return;
}

static
void
globus_handler_ws_addressing_client_response_invoke(
    globus_handler_session_descriptor_t session,
    globus_soap_message_handle_t        message_handle,
    globus_result_t                     result)
{
    GlobusFuncName(globus_ws_addressing_handler_client_response_invoke);
    GlobusWSAddressingDebugEnter();

    if(globus_soap_message_handle_get_attr(
	    message_handle,
	    WS_ADDRESSING_CLIENT_RESPONSE_ALREADY_INVOKED))
    {
	goto exit;
    }
    
    if(!globus_soap_message_handle_get_attr(
           message_handle,
           WSADDR_ACTION_RESPONSE_KEY))
    {
        result = GlobusWSAddressingErrorRequiredElementNotFound(
            "{"WSADDRESSING_NS"}Action");
        goto exit;
    }

    if(!globus_soap_message_handle_get_attr(
           message_handle,
           WSADDR_DESTINATION_KEY))
    {
        result = GlobusWSAddressingErrorRequiredElementNotFound(
            "{"WSADDRESSING_NS"}To");
        goto exit;
    }

    if(!globus_soap_message_handle_get_attr(
           message_handle,
           WSADDR_RELATES_TO_KEY))
    {
        result = GlobusWSAddressingErrorRequiredElementNotFound(
            "{"WSADDRESSING_NS"}RelatesTo");
        goto exit;
    }
 
    globus_soap_message_handle_set_attr(
	message_handle,
	WS_ADDRESSING_CLIENT_RESPONSE_ALREADY_INVOKED,
	NULL, NULL, (void *)1);
   
 exit:
    
    GlobusWSAddressingDebugExit();
    globus_handler_finished(session, result);
    return;
}
    

void
globus_handler_ws_addressing_server_response_invoke(
    globus_handler_session_descriptor_t session,
    globus_soap_message_handle_t        message_handle,
    globus_result_t                     result)
{
    globus_uuid_t *                     uuid = NULL;
    char *                              uuid_string = NULL;
    wsa_AttributedURI                   MessageID;
    wsa_AttributedURI                   To;
    wsa_AttributedURI                   Action;
    wsa_AttributedURI                   RelatesTo;
    char *                              soap_action = NULL;
    wsa_EndpointReferenceType *         endpoint_reference = NULL;
    wsa_EndpointReferenceType *         source_endpoint = NULL;
    wsa_EndpointReferenceType           default_From;
    GlobusFuncName(globus_handler_ws_addressing_server_response_invoke);
    GlobusWSAddressingDebugEnter();

    if(result != GLOBUS_SUCCESS)
    {
        goto exit;
    }

    if(globus_soap_message_handle_get_attr(
	    message_handle,
	    WS_ADDRESSING_SERVER_RESPONSE_ALREADY_INVOKED))
    {
	goto exit;
    }
    
    endpoint_reference = (wsa_EndpointReferenceType *)
        globus_soap_message_handle_get_attr(
            message_handle, WSADDR_OUTBOUND_EPR_KEY);
    if(!endpoint_reference)
    {
	endpoint_reference = (wsa_EndpointReferenceType *)
	    globus_soap_message_handle_get_attr(
		message_handle, WSADDR_EPR_KEY);
    }
    
    uuid = malloc(sizeof(globus_uuid_t));
    if(!uuid)
    {
        result = GlobusWSAddressingErrorMemoryAllocationFailed(
            sizeof(globus_uuid_t), "uuid");
        goto exit;
    }
    globus_uuid_create(uuid);

    uuid_string = globus_common_create_string("uuid:%s", uuid->text);
    MessageID.base_value = uuid_string;
    MessageID.anyAttributes = NULL;
    
    result = wsa_AttributedURI_serialize(
        (xsd_QName *)&wsa_MessageID_qname,
        &MessageID,
        message_handle,
        0);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusWSAddressingErrorSerializeFailed(
            result, "MessageID element");
        goto exit;
    }

    To.anyAttributes = NULL;
    To.base_value = globus_soap_message_handle_get_attr(
        message_handle, WSADDR_REQUEST_ENDPOINT_KEY);
    result = wsa_AttributedURI_serialize(
        (xsd_QName *)&wsa_To_qname,
        &To,
        message_handle,
        0);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusWSAddressingErrorSerializeFailed(
            result, "To element");
        goto exit;
    }
    
    source_endpoint = globus_soap_message_handle_get_attr(
        message_handle, WSADDR_SOURCE_ENDPOINT_KEY);
    if(!source_endpoint)
    {
	char *                          base_service_endpoint;
        memset(&default_From, 0, sizeof(wsa_EndpointReferenceType));
        base_service_endpoint = 
        globus_soap_message_handle_get_attr(
            message_handle, 
            GLOBUS_SOAP_MESSAGE_SERVICE_ENDPOINT_KEY);
        if(default_From.Address.base_value)
        {
            default_From.ReferenceProperties = 
            (endpoint_reference ? 
             endpoint_reference->ReferenceProperties : NULL);
            source_endpoint = &default_From;
        }
    }
    
    if(source_endpoint)
    {
        result = wsa_EndpointReferenceType_serialize(
            (xsd_QName *)&wsa_From_qname,
            source_endpoint,
            message_handle,
            0);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusWSAddressingErrorSerializeFailed(
                GLOBUS_FAILURE, "{"WSADDRESSING_NS"}From element");
            goto exit;
        }
    }

    soap_action = (char *) globus_soap_message_handle_get_attr(
        message_handle, WSADDR_ACTION_RESPONSE_KEY);
    if(!soap_action)
    {
        result = GlobusWSAddressingErrorSerializeFailed(
            GLOBUS_FAILURE, 
            "No "WSADDRESSING_NS
            "#ResponseAction property defined");
        goto exit;
    }

    Action.base_value = soap_action;
    Action.anyAttributes = NULL;
    
    result = wsa_AttributedURI_serialize(
        (xsd_QName *)&wsa_Action_qname,
        &Action,
        message_handle,
        0);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusWSAddressingErrorSerializeFailed(
            result, "Action element");
        goto exit;
    }
    
    uuid_string = globus_soap_message_handle_get_attr(
        message_handle,
        WSADDR_INBOUND_MESSAGE_ID_KEY);
    if(!uuid_string)
    {
        result = GlobusWSAddressingErrorSerializeFailed(
            result, "RelatesTo element");
        goto exit;
    }

    RelatesTo.base_value = uuid_string;
    RelatesTo.anyAttributes = NULL;

    result = wsa_AttributedURI_serialize(
        (xsd_QName *)&wsa_RelatesTo_qname,
        &RelatesTo,
        message_handle,
        0);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusWSAddressingErrorSerializeFailed(
            result, "RelatesTo element");
        goto exit;
    }
 
    globus_soap_message_handle_set_attr(
	message_handle,
	WS_ADDRESSING_SERVER_RESPONSE_ALREADY_INVOKED,
	NULL, NULL, (void *)1);
   
 exit:

    globus_handler_finished(session, result);
    GlobusWSAddressingDebugExit();
    return;
}

static
void
globus_l_handler_ws_addressing_server_invoke_MessageID(
    globus_handler_session_descriptor_t session,
    globus_soap_message_handle_t        message_handle,
    globus_result_t                     result)
{
    wsa_AttributedURI                   messID;
    GlobusFuncName(globus_handler_ws_addressing_server_invoke_MessageID);
    GlobusWSAddressingDebugEnter();

    if(result != GLOBUS_SUCCESS)
    {
        goto exit;
    }

    if(globus_soap_message_handle_get_attr(
	    message_handle,
	    WS_ADDRESSING_SERVER_MESSAGEID_ALREADY_INVOKED))
    {
	goto exit;
    }
    
    
    wsa_AttributedURI_init_contents(&messID);

    result = wsa_AttributedURI_deserialize(
        &wsa_MessageID_qname,
        &messID,
        message_handle,
        0);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusWSAddressingErrorDeserializeFailed(
            result, ("Failed to deserialize MessageID element"));
        goto exit;
    }

    globus_soap_message_handle_set_attr(
        message_handle,
        WSADDR_INBOUND_MESSAGE_ID_KEY,
        globus_soap_message_attr_copy_string,
        globus_libc_free, 
        messID.base_value);

    wsa_AttributedURI_destroy_contents(&messID);

    globus_soap_message_remove_required_header_element(
        message_handle, &wsa_MessageID_qname);

    globus_soap_message_handle_set_attr(
	message_handle,
	WS_ADDRESSING_SERVER_MESSAGEID_ALREADY_INVOKED,
	NULL, NULL, (void *)1);

 exit:
    
    globus_handler_finished(session, result);
    GlobusWSAddressingDebugExit();
    return;
}
    
static
void
globus_l_handler_ws_addressing_server_invoke_From(
    globus_handler_session_descriptor_t session,
    globus_soap_message_handle_t        message_handle,
    globus_result_t                     result)
{
    wsa_EndpointReferenceType           from_epr;
    GlobusFuncName(globus_handler_ws_addressing_server_invoke_From);
    GlobusWSAddressingDebugEnter();

    if(result != GLOBUS_SUCCESS)
    {
	goto exit;
    }

    if(globus_soap_message_handle_get_attr(
	    message_handle,
	    WS_ADDRESSING_SERVER_FROM_ALREADY_INVOKED))
    {
	goto exit;
    }

    wsa_EndpointReferenceType_init_contents(&from_epr);

    result = wsa_EndpointReferenceType_deserialize(
        &wsa_From_qname,
        &from_epr,
        message_handle,
        0);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusWSAddressingErrorDeserializeFailed(
            result, ("Failed to deserialize From element"));
        goto exit;
    }

    globus_soap_message_handle_set_attr(
        message_handle, 
        WSADDR_REQUEST_ENDPOINT_KEY,
        globus_soap_message_attr_copy_string,
        globus_libc_free, 
        from_epr.Address.base_value);
    
    wsa_EndpointReferenceType_destroy_contents(
        &from_epr);

    globus_soap_message_remove_required_header_element(
        message_handle, &wsa_From_qname);
 
    globus_soap_message_handle_set_attr(
	message_handle,
	WS_ADDRESSING_SERVER_FROM_ALREADY_INVOKED,
	NULL, NULL, (void *)1);
   
 exit:
   
    globus_handler_finished(session, result);
    GlobusWSAddressingDebugExit();
    return;
}

static
void
globus_l_handler_ws_addressing_server_invoke_To(
    globus_handler_session_descriptor_t session,
    globus_soap_message_handle_t        message_handle,
    globus_result_t                     result)
{
    wsa_EndpointReferenceType *         dest_epr;
    wsa_AttributedURI                   To;
    GlobusFuncName(globus_handler_ws_addressing_server_invoke_To);
    GlobusWSAddressingDebugEnter();

    if(result != GLOBUS_SUCCESS)
    {
	goto exit;
    }

    if(globus_soap_message_handle_get_attr(
	    message_handle,
	    WS_ADDRESSING_SERVER_TO_ALREADY_INVOKED))
    {
	goto exit;
    }
    
    wsa_AttributedURI_init_contents(&To);

    result = wsa_AttributedURI_deserialize(
        &wsa_To_qname,
        &To,
        message_handle,
        0);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusWSAddressingErrorDeserializeFailed(
            result, ("Failed to deserialize To element"));
        goto exit;
    }

    globus_soap_message_handle_set_attr(
        message_handle,
        GLOBUS_SOAP_MESSAGE_SERVICE_ENDPOINT_KEY,
        globus_soap_message_attr_copy_string,
        globus_libc_free, 
        To.base_value);

    wsa_EndpointReferenceType_init(&dest_epr);
    
    dest_epr->Address.base_value = globus_libc_strdup(To.base_value);

    wsa_AttributedURI_destroy_contents(&To);

    globus_soap_message_handle_set_attr(
        message_handle,
        WSADDR_EPR_KEY,
        wsa_EndpointReferenceType_copy_wrapper,
        wsa_EndpointReferenceType_destroy_wrapper,
        (void *) dest_epr);

    wsa_EndpointReferenceType_destroy(dest_epr);
    
    globus_soap_message_remove_required_header_element(
        message_handle, &wsa_To_qname);

    globus_soap_message_handle_set_attr(
	message_handle,
	WS_ADDRESSING_SERVER_TO_ALREADY_INVOKED,
	NULL, NULL, (void *)1);

 exit:
    
    globus_handler_finished(session, result);
    GlobusWSAddressingDebugExit();
    return;
}
   
static
void
globus_l_handler_ws_addressing_server_invoke_Action(
    globus_handler_session_descriptor_t session,
    globus_soap_message_handle_t        message_handle,
    globus_result_t                     result)
{
    wsa_AttributedURI                   Action;
    GlobusFuncName(globus_handler_ws_addressing_server_invoke_Action);
    GlobusWSAddressingDebugEnter();

    if(result != GLOBUS_SUCCESS)
    {
	goto exit;
    }

    if(globus_soap_message_handle_get_attr(
	    message_handle,
	    WS_ADDRESSING_SERVER_ACTION_ALREADY_INVOKED))
    {
	goto exit;
    }
    
    wsa_AttributedURI_init_contents(&Action);

    result = wsa_AttributedURI_deserialize(
        &wsa_Action_qname,
        &Action,
        message_handle,
        0);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusWSAddressingErrorDeserializeFailed(
            result, ("Failed to deserialize Action element"));
        goto exit;
    }

    globus_soap_message_handle_set_attr(
        message_handle,
        WSADDR_ACTION_REQUEST_KEY,
        globus_soap_message_attr_copy_string,
        globus_libc_free, 
        Action.base_value);
    
    globus_soap_message_remove_required_header_element(
        message_handle, &wsa_Action_qname);

    globus_soap_message_handle_set_attr(
	message_handle,
	WS_ADDRESSING_SERVER_ACTION_ALREADY_INVOKED,
	NULL, NULL, (void *)1);

 exit:
    
    globus_handler_finished(session, result);
    GlobusWSAddressingDebugExit();
    return;
}

static
void
globus_handler_ws_addressing_server_request_default_trigger(
    globus_handler_session_descriptor_t session,
    globus_soap_message_handle_t        message_handle,
    globus_result_t                     result)
{
    wsa_EndpointReferenceType *         dest_epr;
    GlobusFuncName(
        globus_handler_ws_addressing_server_request_default_trigger);
    GlobusWSAddressingDebugEnter();

    if(result != GLOBUS_SUCCESS)
    {
	goto exit;
    }
   
    dest_epr = globus_soap_message_handle_get_attr(
        message_handle,
        WSADDR_OUTBOUND_EPR_KEY);
    if(!dest_epr)
    {
	dest_epr = globus_soap_message_handle_get_attr(
	    message_handle,
	    WSADDR_EPR_KEY);
    }

    if(dest_epr)
    {
        wsa_ReferencePropertiesType_init(&dest_epr->ReferenceProperties);

        result = xsd_any_array_deserialize(
            NULL,
            &dest_epr->ReferenceProperties->any,
            message_handle,
            0);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusWSAddressingErrorDeserializeFailed(
                result, 
                ("Failed to deserialize ReferenceProperties elements"));
            goto exit;
        }
    }
    else
    {
	xsd_QName                       header_elem;
        result = globus_soap_message_deserialize_element_unknown(
	    message_handle, &header_elem);
	if(result != GLOBUS_SUCCESS)
	{
	    result = GlobusWSAddressingErrorDeserializeFailed(
		result,
		("Failed skipping unknown header element"));
	    goto exit;
	}
	
	result = globus_soap_message_deserialize_skip(
	    message_handle);
	if(result != GLOBUS_SUCCESS)
	{
	    result = GlobusWSAddressingErrorDeserializeFailed(
		result,
		("Failed skipping unknown header element"));
	    goto exit;
	}

	result = globus_soap_message_deserialize_element_end(
	    message_handle,
            &header_elem);
	if(result != GLOBUS_SUCCESS)
	{
	    result = GlobusWSAddressingErrorDeserializeFailed(
		result,
		("Failed skipping unknown header element"));
	    goto exit;
	}
    }

 exit:
    
    globus_handler_finished(session, result);
    GlobusWSAddressingDebugExit();
    return;
}

static
void
globus_handler_ws_addressing_server_request_invoke(
    globus_handler_session_descriptor_t session,
    globus_soap_message_handle_t        message_handle,
    globus_result_t                     result)
{
    int                                 To_handled = 0;
    int                                 Action_handled = 0;
    GlobusFuncName(globus_handler_ws_addressing_server_request_invoke);
    GlobusWSAddressingDebugEnter();

    if(globus_soap_message_handle_get_attr(
	    message_handle,
	    WS_ADDRESSING_SERVER_REQUEST_ALREADY_INVOKED))
    {
	goto exit;
    }
    
    if(globus_soap_message_handle_get_attr(
           message_handle,
           GLOBUS_SOAP_MESSAGE_SERVICE_ENDPOINT_KEY))
    {
        To_handled = 1;
    }

    if(globus_soap_message_handle_get_attr(
           message_handle,
           WSADDR_ACTION_REQUEST_KEY))
    {
        Action_handled = 1;
    }

    if(!To_handled || !Action_handled)
    {
        xsd_QName *                     missing_header_qname;
        xsd_any *                       missing_header;
        globus_soap_message_fault_t     soap_fault;

        globus_soap_message_fault_init(&soap_fault);
        
        xsd_QName_init(&missing_header_qname);
        if(!To_handled)
        {
            missing_header_qname->local =
                globus_libc_strdup(wsa_To_qname.local);
            missing_header_qname->Namespace =
                globus_libc_strdup(wsa_To_qname.Namespace);
        }
        else
        {
            missing_header_qname->local =
                globus_libc_strdup(wsa_Action_qname.local);
            missing_header_qname->Namespace = 
                globus_libc_strdup(wsa_Action_qname.Namespace);
        }

        xsd_any_init((void *)&missing_header);
        missing_header->any_info = &xsd_QName_contents_info;
        missing_header->value = (void *)missing_header_qname;

        soap_fault->faultcode =
            globus_libc_strdup("MessageInformationHeaderRequired");
        soap_fault->faultstring = globus_libc_strdup(
            "A required message information header: To, MessageID, "
            "or Action is not present.");
        soap_fault->detail = missing_header;

        result = GlobusSoapMessageErrorFault(soap_fault);
        goto exit;
    }

    globus_soap_message_handle_set_attr(
	message_handle,
	WS_ADDRESSING_SERVER_REQUEST_ALREADY_INVOKED,
	NULL, NULL, (void *)1);

 exit:
    
    globus_handler_finished(session, result);
    GlobusWSAddressingDebugExit();
    return;
}

#define GLOBUS_L_HANDLER_WSADD_DEFINE_CLIENT_TRIGGER(__ELEMENT__)       \
    static globus_handler_trigger_t                                     \
    globus_l_handler_ws_addressing_client_trigger_##__ELEMENT__ =       \
    {                                                                   \
        &wsa_##__ELEMENT__##_qname,                                     \
        globus_l_handler_ws_addressing_client_invoke_##__ELEMENT__      \
    }

GLOBUS_L_HANDLER_WSADD_DEFINE_CLIENT_TRIGGER(MessageID);
GLOBUS_L_HANDLER_WSADD_DEFINE_CLIENT_TRIGGER(RelatesTo);
GLOBUS_L_HANDLER_WSADD_DEFINE_CLIENT_TRIGGER(To);
GLOBUS_L_HANDLER_WSADD_DEFINE_CLIENT_TRIGGER(Action);
GLOBUS_L_HANDLER_WSADD_DEFINE_CLIENT_TRIGGER(From);

#define GLOBUS_L_HANDLER_WSADD_DEFINE_SERVER_TRIGGER(__ELEMENT__)       \
    static globus_handler_trigger_t                                     \
    globus_l_handler_ws_addressing_server_trigger_##__ELEMENT__ =       \
    {                                                                   \
        &wsa_##__ELEMENT__##_qname,                                     \
        globus_l_handler_ws_addressing_server_invoke_##__ELEMENT__      \
    }

GLOBUS_L_HANDLER_WSADD_DEFINE_SERVER_TRIGGER(MessageID);
GLOBUS_L_HANDLER_WSADD_DEFINE_SERVER_TRIGGER(To);
GLOBUS_L_HANDLER_WSADD_DEFINE_SERVER_TRIGGER(Action);
GLOBUS_L_HANDLER_WSADD_DEFINE_SERVER_TRIGGER(From);

globus_handler_descriptor_t globus_l_handler_ws_addressing_client_descriptor = 
{
    NULL, 
    globus_handler_ws_addressing_client_request_invoke,
    NULL,
    NULL,
    globus_handler_ws_addressing_client_response_invoke,
    NULL,
    NULL,
    NULL
};

globus_handler_descriptor_t globus_l_handler_ws_addressing_server_descriptor = 
{
    NULL,
    globus_handler_ws_addressing_server_request_invoke,
    NULL,
    NULL,
    globus_handler_ws_addressing_server_response_invoke,
    NULL,
    NULL,
    globus_handler_ws_addressing_server_request_default_trigger
};

int
globus_l_handler_ws_addressing_activate(void)
{
    int                                 res = 0;
    GlobusFuncName(globus_l_handler_ws_addressing_activate);

    GlobusDebugInit(GLOBUS_HANDLER_WS_ADDRESSING, TRACE DEBUG);
    GlobusWSAddressingDebugEnter();

    globus_l_handler_ws_addressing_client_descriptor.trigger_list = NULL;

    globus_list_insert(
        &globus_l_handler_ws_addressing_client_descriptor.trigger_list,
        &globus_l_handler_ws_addressing_client_trigger_MessageID);
    globus_list_insert(
        &globus_l_handler_ws_addressing_client_descriptor.trigger_list,
        &globus_l_handler_ws_addressing_client_trigger_RelatesTo);
    globus_list_insert(
        &globus_l_handler_ws_addressing_client_descriptor.trigger_list,
        &globus_l_handler_ws_addressing_client_trigger_To);
    globus_list_insert(
        &globus_l_handler_ws_addressing_client_descriptor.trigger_list,
        &globus_l_handler_ws_addressing_client_trigger_Action);
    globus_list_insert(
        &globus_l_handler_ws_addressing_client_descriptor.trigger_list,
        &globus_l_handler_ws_addressing_client_trigger_From);

    res = globus_extension_registry_add(
        GLOBUS_HANDLER_REGISTRY,
        GLOBUS_HANDLER_WS_ADDRESSING_CLIENT,
        GlobusExtensionMyModule(globus_handler_ws_addressing),
        &globus_l_handler_ws_addressing_client_descriptor);
    if(res != GLOBUS_SUCCESS)
    {
        goto exit;
    }

    globus_l_handler_ws_addressing_server_descriptor.trigger_list = NULL;
    globus_list_insert(
        &globus_l_handler_ws_addressing_server_descriptor.trigger_list,
        &globus_l_handler_ws_addressing_server_trigger_MessageID);
    globus_list_insert(
        &globus_l_handler_ws_addressing_server_descriptor.trigger_list,
        &globus_l_handler_ws_addressing_server_trigger_To);
    globus_list_insert(
        &globus_l_handler_ws_addressing_server_descriptor.trigger_list,
        &globus_l_handler_ws_addressing_server_trigger_Action);
    globus_list_insert(
        &globus_l_handler_ws_addressing_server_descriptor.trigger_list,
        &globus_l_handler_ws_addressing_server_trigger_From);

    res = globus_extension_registry_add(
        GLOBUS_HANDLER_REGISTRY,
        GLOBUS_HANDLER_WS_ADDRESSING_SERVER,
        GlobusExtensionMyModule(globus_handler_ws_addressing),
        &globus_l_handler_ws_addressing_server_descriptor);

 exit:
    
    GlobusWSAddressingDebugExit();
    return res;
}

int
globus_l_handler_ws_addressing_deactivate(void)
{
    GlobusFuncName(globus_l_handler_ws_addressing_deactivate);
    GlobusWSAddressingDebugEnter();

    globus_extension_registry_remove(
        GLOBUS_HANDLER_REGISTRY,
        GLOBUS_HANDLER_WS_ADDRESSING_CLIENT);

    globus_extension_registry_remove(
        GLOBUS_HANDLER_REGISTRY,
        GLOBUS_HANDLER_WS_ADDRESSING_SERVER);

    globus_list_free(
        globus_l_handler_ws_addressing_client_descriptor.trigger_list);

    globus_list_free(
        globus_l_handler_ws_addressing_server_descriptor.trigger_list);
    
    globus_l_handler_ws_addressing_client_descriptor.trigger_list = NULL;
    globus_l_handler_ws_addressing_server_descriptor.trigger_list = NULL;

    GlobusWSAddressingDebugExit();
    return 0;
}

static
globus_result_t
globus_l_uuid_copy(
    void **                             new_id,
    void *                              id)
{
    globus_uuid_t *                     uuid = id;
    globus_uuid_t *                     new_uuid;

    new_uuid = globus_malloc(sizeof(globus_uuid_t));
    globus_uuid_import(new_uuid, uuid->text);

    *new_id = new_uuid;
    return GLOBUS_SUCCESS;
}
