/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */


#include "globus_handler.h"
#include "globus_soap_message.h"
#include "globus_soap_message_handle.h"
#include "version.h"

void
globus_handler_debug_request_invoke(
    globus_handler_session_descriptor_t session,
    globus_soap_message_handle_t        message_handle,
    globus_result_t                     result)
{
    char *                              buffer;
    size_t                              length;
    globus_abstime_t                    current_time;
    char                                buf[256];

    result = globus_soap_message_get_marked_buffers(
        message_handle,
        NULL, NULL,
        &buffer,
        &length);
    if(result == GLOBUS_SUCCESS)
    {
        GlobusTimeAbstimeGetCurrent(current_time);
        sprintf(buf, "%lu.%.9lu",
                (unsigned long) current_time.tv_sec,
                (unsigned long) current_time.tv_nsec);
        fprintf(stderr, 
                "\n=== REQUEST MESSAGE (length %lu) (time %s) ===\n",
                (unsigned long) length, buf);
        fwrite(buffer, length, 1, stderr);
        fprintf(stderr, 
                "\n----------------------------------------------\n");
    }

    globus_handler_finished(session, result);
    return;
}                

void
globus_handler_debug_response_invoke(
    globus_handler_session_descriptor_t session,
    globus_soap_message_handle_t        message_handle,
    globus_result_t                     result)
{
    char *                              buffer;
    size_t                              length;
    globus_abstime_t                    current_time;
    char                                buf[256];

    result = globus_soap_message_get_marked_buffers(
        message_handle,
        NULL, NULL,
        &buffer,
        &length);
    if(result == GLOBUS_SUCCESS)
    {
        GlobusTimeAbstimeGetCurrent(current_time);
        sprintf(buf, "%lu.%.9lu",
                (unsigned long) current_time.tv_sec, 
                (unsigned long) current_time.tv_nsec);
        fprintf(stderr,
                "\n=== RESPONSE MESSAGE (length %lu) (time %s) ===\n",
                (unsigned long) length, buf);
        fwrite(buffer, length, 1, stderr);
        fprintf(stderr, 
                "\n----------------------------------------------\n");
    }

    globus_handler_finished(session, result);
    return;
}

globus_handler_descriptor_t globus_l_handler_debug_descriptor = 
{
    NULL,
    globus_handler_debug_request_invoke,
    NULL, NULL,
    globus_handler_debug_response_invoke,
    NULL, NULL, NULL
};

int
globus_l_handler_debug_activate(void);

int
globus_l_handler_debug_deactivate(void);


GlobusExtensionDefineModule(globus_handler_debug) =
{
    "debug",
    globus_l_handler_debug_activate,
    globus_l_handler_debug_deactivate,
    NULL,
    NULL,
    &local_version
};

int
globus_l_handler_debug_activate(void)
{
    int                                 res = 0;

    res = globus_extension_registry_add(
        GLOBUS_HANDLER_REGISTRY,
        "globus_handler_debug",
        GlobusExtensionMyModule(globus_handler_debug),
        &globus_l_handler_debug_descriptor);
    if(res != GLOBUS_SUCCESS)
    {
        goto exit;
    }
    
 exit:
    return res;
}

int
globus_l_handler_debug_deactivate(void)
{
    globus_extension_registry_remove(
        GLOBUS_HANDLER_REGISTRY,
        "globus_handler_debug");

    return 0;
}
