/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */


#include "globus_i_handler_wsse_cred.h"
#include "globus_extension.h"
#include "globus_common.h"
#include "globus_handler.h"
#include "globus_soap_message.h"
#include "globus_soap_message_handle.h"
#include "globus_soap_message_markers.h"
#include "globus_xml_buffer.h"
#include "globus_wsrf_core_tools.h"
#include "libxml/tree.h"
#include "xsd_any.h"
#include "wsse_BinarySecurityTokenType.h"
#include "wsseu_TimestampType.h"
#include "ds_SignedInfoType.h"
#include "ds_SignatureValueType.h"
#include "ds_SignatureType.h"
#include "ds_CanonicalizationMethodType.h"
#include "ds_SignatureMethodType.h"
#include "ds_ReferenceType.h"
#include "ds_TransformType.h"
#include "ds_KeyInfoType.h"
#include "wsse_SecurityTokenReferenceType.h"
#include "wsse_ReferenceType.h"
#include "globus_gsi_credential.h"
#include "globus_gsi_system_config.h"
#include "globus_ws_security.h"
#include "openssl/evp.h"
#include "openssl/err.h"
#include "libxml/c14n.h"
#include "globus_ws_addressing.h"

#include "wsa_MessageID.h"
#include "wsa_RelatesTo.h"
#include "wsa_ReplyTo.h"
#include "wsa_From.h"
#include "wsa_FaultTo.h"
#include "wsa_To.h"
#include "wsa_Action.h"

#include "wsse_SecurityHeaderType.h"
#include "wsse_BinarySecurityTokenType.h"
#include "ds_SignatureType.h"
#include "wsseu_TimestampType.h"

#include "wsse_Security.h"
#include "wsse_BinarySecurityToken.h"
#include "ds_Signature.h"
#include "ds_SignedInfo.h"
#include "ds_SignatureValue.h"
#include "wsse_SecurityTokenReference.h"
#include "ds_Reference.h"
#include "ds_KeyInfo.h"
#include "wsseu_Timestamp.h"
#include "wsse_Reference.h"

#include "version.h"

#include "exc_c14n_InclusiveNamespaces.h"
#include "exc_c14n_InclusiveNamespacesType.h"

#define SIGNED_INFO_MARKER "SIGNED_INFO_MARKER"
#define SIGNED_INFO_MARKER_END "SIGNED_INFO_MARKER_END"

#define SIGNATURE_CONTENTS_MARKER     "SIGNATURE_CONTENTS"
#define SIGNATURE_CONTENTS_END_MARKER "SIGNATURE_CONTENTS_END"

#define SECURITY_HEADER_ELEMENT_KEY "SECURITY_HEADER_ELEMENT_KEY"
#define WSSE_DONE_WITH_BODY_KEY "SECURITY_HEADER_DONE_WITH_BODY_KEY"

#define XMLSCHEMA_NS "http://www.w3.org/2001/XMLSchema"

#define SOAP_ENV_NS "http://schemas.xmlsoap.org/soap/envelope/"

#define WSSE_NS "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd"

#define BASE64BINARY_ENCODING_NS "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-soap-message-security-1.0#Base64Binary"

#define X509_TOKEN_NS "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-x509-token-profile-1.0#X509PKIPathv1"

#define WSSEU_NS "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd"

#define DS_NS "http://www.w3.org/2000/09/xmldsig#"

#define EXC_C14N_NS "http://www.w3.org/2001/10/xml-exc-c14n#"

#define RSA_SHA1_NS DS_NS "rsa-sha1"

#define SHA1_NS DS_NS "sha1"

#define GLOBUS_HANDLER_WSSE_TOKEN_IDS_KEY \
        "GLOBUS_SOAP_MESSAGE_WSSE_TOKEN_IDS"

#define GLOBUS_L_HANDLER_WSSE_REFERENCE_PROPS_KEY \
        "GLOBUS_L_HANDLER_WSSE_REFERENCE_PROPS_KEY"

#define WS_SECMESS_INIT_ALREADY_INVOKED \
       "WS_SECMESS_INIT_ALREADY_INVOKED"

#define WS_SECMESS_CLIENT_REQUEST_ALREADY_INVOKED \
       "WS_SECMESS_CLIENT_REQUEST_ALREADY_INVOKED"

#define WS_SECMESS_CLIENT_RESPONSE_ALREADY_INVOKED \
       "WS_SECMESS_CLIENT_RESPONSE_ALREADY_INVOKED"

#define WS_SECMESS_SERVER_REQUEST_ALREADY_INVOKED \
       "WS_SECMESS_SERVER_REQUEST_ALREADY_INVOKED"

#define WS_SECMESS_SERVER_RESPONSE_ALREADY_INVOKED \
       "WS_SECMESS_SERVER_RESPONSE_ALREADY_INVOKED"

int
globus_l_handler_ws_security_activate(void);

int
globus_l_handler_ws_security_deactivate(void);

GlobusExtensionDeclareModule(globus_handler_ws_security);

GlobusDebugDefine(GLOBUS_HANDLER_WS_SECURE_MESSAGE);

extern globus_module_descriptor_t       globus_extension_module;

static xsd_QName                        mustUnderstand_attr_qname =
{
    SOAP_ENV_NS,
    "mustUnderstand"
};

static xsd_QName                        wsseu_Id_qname =
{
    WSSEU_NS,
    "Id"
};

#define CERT_ID_PREFIX "CertId-"
#define GlobusHandlerWSSENewCertId() \
        globus_soap_message_construct_id(CERT_ID_PREFIX)

#define KEY_ID_PREFIX "KeyId-"
#define GlobusHandlerWSSENewKeyId() \
        globus_soap_message_construct_id(KEY_ID_PREFIX)

#define STR_ID_PREFIX "STRId-"
#define GlobusHandlerWSSENewSTRId() \
        globus_soap_message_construct_id(STR_ID_PREFIX)

#define ID_PREFIX "id-"
#define GlobusHandlerWSSENewId() \
        globus_soap_message_construct_id(ID_PREFIX)

#include "globus_i_handler_wsse_cred.h"

typedef struct
{
   globus_xml_buffer *                  xml_buffer;
   char *                               id;
} globus_l_handler_wsse_id_refprop_t;

static
globus_result_t
globus_l_handler_wsse_id_refprops_copy(
    void **                             newprop,
    void *                              prop);

static
void
globus_l_handler_wsse_id_refprops_destroy(
    void *                              prop);

static
globus_result_t
globus_l_handler_wsse_id_refprop_copy(
    void **                             newprop,
    void *                              prop);

static
void
globus_l_handler_wsse_id_refprop_destroy(
    void *                              prop);

static
globus_result_t
globus_l_handler_ws_security_c14n_signed_info(
    globus_soap_message_handle_t        message_handle,
    const char *                        input_buff,
    size_t                              input_length,
    char **                             output_buff,
    size_t *                            output_buff_length,
    xsd_string_array *                  prefixes);

#ifndef GLOBUS_HANDLER_WSSE_INLINED_C14N
static
globus_result_t
globus_i_handler_ws_security_c14n_buffer(
    const char *                        input_buff,
    size_t                              input_length,
    char **                             output_buff,
    size_t *                            output_buff_length);
#endif

static
globus_result_t
globus_l_handler_wsse_search_tree(
    xmlDocPtr                           doc,
    xsd_QName *                         element,
    xmlNodePtr *                        node);

static
globus_result_t
globus_l_handler_wsse_lookup_prefix(
    xmlDocPtr                           doc,
    xmlNodePtr                          node,
    const xmlChar *                     prefix,
    const xmlChar **                    href);

static
globus_result_t
globus_l_handler_wsse_get_referenced_node(
    const char *                        id,
    xmlDocPtr                           doc,
    xmlNodePtr *                        reference_node);

static
globus_result_t
globus_i_handler_ws_security_compute_signature(
    globus_gsi_cred_handle_t            cred,
    const char *                        input_buff,
    size_t                              input_length,
    char **                             signed_buff,
    size_t *                            signed_buff_length);

static
globus_result_t
globus_i_handler_ws_security_verify_signature(
    globus_gsi_cred_handle_t            cred,
    const unsigned char *               input_buff,
    size_t                              input_length,
    const unsigned char *               signed_buff,
    size_t                              signed_buff_length);

static
globus_result_t
globus_i_handler_ws_security_compute_digest(
    const char *                        buff,
    size_t                              buff_length,
    char **                             digest_value,
    size_t *                            digest_length);

static
globus_result_t
globus_i_handler_ws_security_verify_digest(
    const char *                        buff,
    size_t                              buff_length,
    const char *                        digest_value,
    size_t                              digest_length);

static
globus_result_t
globus_i_handler_ws_security_add_references(
    globus_soap_message_handle_t        message_handle,
    ds_ReferenceType_array *            reference_array);

static
globus_result_t
globus_i_handler_ws_security_verify_references(
    globus_soap_message_handle_t        message_handle,
    ds_ReferenceType_array *            reference_array);

static
globus_bool_t
globus_l_ws_security_is_reference_property_element(
    const xsd_QName *                   type);

static
globus_result_t
globus_handler_ws_security_sign_message(
    globus_soap_message_handle_t        message_handle)
{
    wsse_BinarySecurityTokenType        bst;
    wsseu_TimestampType                 Timestamp;
    ds_SignedInfoType                   SignedInfo;
    ds_SignatureValueType               SignatureValue;
    ds_KeyInfoType                      KeyInfo;
    char *                              key_info_id = NULL;
    char *                              key_info_id_end = NULL;
    char *                              BST_Id = NULL;
    char *                              BST_Id_end = NULL;
    globus_list_t **                    security_attrs = NULL;
    ds_KeyInfoType_choice *             key_info_choice = NULL;
    wsse_SecurityTokenReferenceType *   SecurityTokenReference = NULL;
    wsse_ReferenceType *                wsse_Reference = NULL;
    struct tm                           tv;
    time_t                              ct;
    char *                              signed_info_buff = NULL;
    size_t                              signed_info_buff_length = 0;
    int                                 mustUnderstand;
    int                                 message_duration;
    char *                              pkipath = NULL;
    int                                 pkipath_length;
    globus_gsi_cred_handle_t            cred = NULL;
    globus_result_t                     result = GLOBUS_SUCCESS;
    globus_xsd_element_options_t        options = 0;
    GlobusFuncName(globus_handler_ws_security_sign_message);
    GlobusWSSEDebugEnter();

    result = globus_soap_message_set_write_position_to_marker(
	message_handle, 
	GLOBUS_SOAP_MESSAGE_MARKER_HEADER_CONTENT);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusWSSEErrorFailedSign(result);
	goto exit;
    }

    globus_soap_message_element_ns_set(
	message_handle,
	wsse_Security_qname.Namespace,
	&options);

    result = globus_soap_message_serialize_element(
	message_handle,
	&wsse_Security_qname,
	options,
	NULL);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusWSSEErrorFailedSign(result);
	goto exit;
    }

    result = globus_soap_message_element_ns_add(
	message_handle,
	wsse_Security_qname.Namespace,
	&options);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusWSSEErrorFailedSign(result);
	goto exit;
    }

    mustUnderstand = 1;
    result = globus_soap_message_serialize_int_attribute(
	message_handle,
	&mustUnderstand_attr_qname,
	&mustUnderstand);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusWSSEErrorFailedSign(result);
	goto exit;
    }

    result = globus_soap_message_serialize_element_content_close(
	message_handle);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusWSSEErrorFailedSign(result);
	goto exit;
    }

    /* Start BinarySecurityToken */

    globus_soap_message_serialize_start_c14n_subset(
	message_handle);

    wsse_BinarySecurityTokenType_init_contents(&bst);

    result = xsd_anyURI_init_cstr(&bst._EncodingType,
            globus_libc_strdup(BASE64BINARY_ENCODING_NS));
    if (result != GLOBUS_SUCCESS)
    {
        result = GlobusWSSEErrorFailedSign(result);

        goto destroy_BST;
    }
    result = xsd_anyURI_init_cstr(&bst._ValueType,
            globus_libc_strdup(X509_TOKEN_NS));
    if (result != GLOBUS_SUCCESS)
    {
        result = GlobusWSSEErrorFailedSign(result);

        goto destroy_BST;
    }

    xsd_ID_init(&bst._Id);
    BST_Id = GlobusHandlerWSSENewCertId();
    *bst._Id = BST_Id;
    
    result = globus_soap_message_set_marker(
	message_handle,
	BST_Id);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusWSSEErrorFailedSign(result);
	goto exit;
    }

    /* get pkipath stuff */
    result = globus_i_handler_wsse_load_local_cred(
	message_handle, &cred, &pkipath, &pkipath_length);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusWSSEErrorFailedSign(result);
	goto destroy_BST;
    }

    bst.base_value = globus_malloc(pkipath_length + 1);
    memcpy(bst.base_value, pkipath, pkipath_length);
    bst.base_value[pkipath_length] = '\0';

    result = wsse_BinarySecurityTokenType_serialize(
	&wsse_BinarySecurityToken_qname,
	&bst,
	message_handle,
	options);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusWSSEErrorFailedSign(result);
	goto destroy_pkipath;
    }

    globus_soap_message_serialize_end_c14n_subset(
	message_handle);

    BST_Id_end = globus_common_create_string("%s-end", BST_Id);
    result = globus_soap_message_set_marker(
	message_handle,
	BST_Id_end);
    if(result != GLOBUS_SUCCESS)
    {
	globus_free(BST_Id_end);
	result = GlobusWSSEErrorFailedSign(result);
	goto destroy_KeyInfo;
    }
    globus_free(BST_Id_end);
    
    security_attrs = globus_soap_message_handle_get_attr(
	message_handle,
	GLOBUS_HANDLER_WSSE_TOKEN_IDS_KEY);
    globus_assert(security_attrs);

    globus_list_insert(security_attrs,
		       globus_libc_strdup(BST_Id));

    /* End BinarySecurityToken */

    /* Start Timestamp */

    wsseu_TimestampType_init_contents(&Timestamp);

    wsseu_AttributedDateTime_init(&Timestamp.Created);

    ct = time(NULL);
    gmtime_r(&ct, &tv);
    Timestamp.Created->base_value = globus_wsrf_core_export_timestamp(&tv);

    wsseu_AttributedDateTime_init(&Timestamp.Expires);

    message_duration = (int)globus_soap_message_handle_get_attr(
	message_handle,
	GLOBUS_HANDLER_WSSE_MESSAGE_DURATION_KEY);
    if(message_duration)
    {
	tv.tm_min += message_duration;
    }
    else
    {
	tv.tm_min += 5;
    }

    Timestamp.Expires->base_value = globus_wsrf_core_export_timestamp(&tv);

    result = wsseu_TimestampType_serialize(
	&wsseu_Timestamp_qname,
	&Timestamp,
	message_handle,
	0);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusWSSEErrorFailedSign(result);
	goto destroy_Timestamp;
    }

    /* End Timestamp */

    /* Start Signature element */

    options = 0;
    result = globus_soap_message_element_ns_set(
	message_handle,
	ds_Signature_qname.Namespace,
	&options);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusWSSEErrorFailedSign(result);
	goto destroy_Timestamp;
    }

    result = globus_soap_message_serialize_element(
	message_handle,
	&ds_Signature_qname,
	0,
	NULL);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusWSSEErrorFailedSign(result);
	goto destroy_Timestamp;
    }

    result = globus_soap_message_element_ns_add(
	message_handle,
	ds_Signature_qname.Namespace,
	&options);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusWSSEErrorFailedSign(result);
	goto destroy_Timestamp;
    }

    result = globus_soap_message_serialize_element_begin_close(
	message_handle);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusWSSEErrorFailedSign(result);
	goto destroy_Timestamp;
    }

    result = globus_soap_message_set_marker(
	message_handle,
	SIGNATURE_CONTENTS_MARKER);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusWSSEErrorFailedSign(result);
	goto destroy_Timestamp;
    }
    
    globus_soap_message_serialize_start_c14n_subset(
	message_handle);
    
    /* Add KeyInfo element */

    result = ds_KeyInfoType_init_contents(&KeyInfo);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusWSSEErrorFailedSign(result);
	goto destroy_Timestamp;
    }

    /* Add Id attribute */
    key_info_id = GlobusHandlerWSSENewKeyId();
    xsd_ID_init(&KeyInfo._Id);
    *KeyInfo._Id = key_info_id;

    result = globus_soap_message_set_marker(
	message_handle,
	key_info_id);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusWSSEErrorFailedSign(result);
	goto destroy_KeyInfo;
    }

    key_info_choice = ds_KeyInfoType_choice_array_push(
	&KeyInfo.choice_value);
    key_info_choice->type = ds_KeyInfoType_any;
    xsd_QName_copy(&key_info_choice->value.any.element,
		   &wsse_SecurityTokenReference_qname);
    key_info_choice->value.any.any_info = 
	&wsse_SecurityTokenReferenceType_info;
    wsse_SecurityTokenReferenceType_init(&SecurityTokenReference); 
    xsd_ID_init(&SecurityTokenReference->_Id); 
    *SecurityTokenReference->_Id = GlobusHandlerWSSENewSTRId(); 
    xsd_any_init_contents(&SecurityTokenReference->any); 
    xsd_QName_copy(&SecurityTokenReference->any.element, &wsse_Reference_qname);
    SecurityTokenReference->any.any_info = &wsse_ReferenceType_info;

    wsse_ReferenceType_init(&wsse_Reference);
    xsd_anyURI_init(&wsse_Reference->_URI);
    *wsse_Reference->_URI = globus_common_create_string("#%s", *bst._Id);
    result = xsd_anyURI_init_cstr(
            &wsse_Reference->_ValueType,
            globus_libc_strdup(X509_TOKEN_NS));
    if (result != GLOBUS_SUCCESS)
    {
       result = GlobusWSSEErrorFailedSign(result);
       goto destroy_KeyInfo;
    }


    SecurityTokenReference->any.value = wsse_Reference;

    key_info_choice->value.any.value = SecurityTokenReference;

    result = ds_KeyInfoType_serialize(
	&ds_KeyInfo_qname,
	&KeyInfo,
	message_handle,
	options);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusWSSEErrorFailedSign(result);
	goto destroy_KeyInfo;
    }

    globus_soap_message_serialize_end_c14n_subset(
	message_handle);

    key_info_id_end = globus_common_create_string("%s-end", key_info_id);
    result = globus_soap_message_set_marker(
	message_handle,
	key_info_id_end);
    if(result != GLOBUS_SUCCESS)
    {
	globus_free(key_info_id_end);
	result = GlobusWSSEErrorFailedSign(result);
	goto destroy_KeyInfo;
    }
    globus_free(key_info_id_end);
  
    globus_list_insert(security_attrs, globus_libc_strdup(key_info_id));

    result = globus_soap_message_set_marker(
	message_handle,
	SIGNATURE_CONTENTS_END_MARKER);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusWSSEErrorFailedSign(result);
	goto destroy_KeyInfo;
    }
    
    result = globus_soap_message_set_write_position_to_marker(
	message_handle,
	SIGNATURE_CONTENTS_MARKER);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusWSSEErrorFailedSign(result);
	goto destroy_KeyInfo;
    }
    
    ds_SignedInfoType_init_contents(&SignedInfo);

    ds_CanonicalizationMethodType_init_contents(
	&SignedInfo.CanonicalizationMethod);
    SignedInfo.CanonicalizationMethod._Algorithm = 
	globus_libc_strdup(EXC_C14N_NS);

    ds_SignatureMethodType_init_contents(
	&SignedInfo.SignatureMethod);
    SignedInfo.SignatureMethod._Algorithm =
	globus_libc_strdup(RSA_SHA1_NS);

    result = globus_i_handler_ws_security_add_references(
	message_handle, &SignedInfo.Reference);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusWSSEErrorFailedSign(result);
	goto destroy_SignedInfo;
    }
    
    globus_soap_message_serialize_start_c14n_subset(
	message_handle);

    result = globus_soap_message_set_marker(
	message_handle,
	GLOBUS_HANDLER_WSSE_MARKER_SIGNED_INFO_BEGIN);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusWSSEErrorFailedSign(result);
	goto destroy_SignedInfo;
    }    

    result = ds_SignedInfoType_serialize(
	&ds_SignedInfo_qname,
	&SignedInfo,
	message_handle,
	options);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusWSSEErrorFailedSign(result);
	goto destroy_SignedInfo;
    }

    globus_soap_message_serialize_end_c14n_subset(
	message_handle);

    result = globus_soap_message_set_marker(
	message_handle,
	GLOBUS_HANDLER_WSSE_MARKER_SIGNED_INFO_END);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusWSSEErrorFailedSign(result);
	goto destroy_SignedInfo;
    }    

    /* Set mark for SignatureValue */
   
    /* Add SignatureValue element */

    result = globus_soap_message_get_marked_buffers(
	message_handle, 
	GLOBUS_HANDLER_WSSE_MARKER_SIGNED_INFO_BEGIN,
	GLOBUS_HANDLER_WSSE_MARKER_SIGNED_INFO_END,
	&signed_info_buff,
	&signed_info_buff_length);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusWSSEErrorFailedSign(result);
	goto destroy_SignedInfo;
    } 
    
    result = globus_i_handler_ws_security_compute_signature(
	cred,
	signed_info_buff,
	signed_info_buff_length,
	&SignatureValue.base_value.value,
	&SignatureValue.base_value.length);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusWSSEErrorFailedSign(result);
	goto destroy_signed_info_buff;
    }
    
    SignatureValue._Id = NULL;
    result = ds_SignatureValueType_serialize(
	&ds_SignatureValue_qname,
	&SignatureValue,
	message_handle,
	0);
    if(result != GLOBUS_SUCCESS)
    {
	globus_free(SignatureValue.base_value.value);
	SignatureValue.base_value.value = NULL;
	SignatureValue.base_value.length = 0;
	result = GlobusWSSEErrorFailedSign(result);
	goto destroy_signed_info_buff;
    }

    globus_free(SignatureValue.base_value.value);
    SignatureValue.base_value.value = NULL;
    SignatureValue.base_value.length = 0;

    result = globus_soap_message_set_write_position_to_marker(
	message_handle,
	SIGNATURE_CONTENTS_END_MARKER);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusWSSEErrorFailedSign(result);
	goto destroy_signed_info_buff;
    }
 
    result = globus_soap_message_serialize_element_end(
	message_handle,
	&ds_Signature_qname,
	0,
	NULL);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusWSSEErrorFailedSign(result);
	goto destroy_signed_info_buff;
    }


    globus_soap_message_element_ns_remove(
	message_handle,
	ds_Signature_qname.Namespace,
	&options);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusWSSEErrorFailedSign(result);
	goto destroy_signed_info_buff;
    }

    result = globus_soap_message_serialize_element_end(
	message_handle,
	&wsse_Security_qname,
	0,
	NULL);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusWSSEErrorFailedSign(result);
	goto destroy_signed_info_buff;
    }

    globus_soap_message_element_ns_remove(
	message_handle,
	wsse_Security_qname.Namespace,
	&options);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusWSSEErrorFailedSign(result);
	goto destroy_signed_info_buff;
    }

destroy_signed_info_buff:

    if(signed_info_buff)
    {
	globus_free(signed_info_buff);
    }

destroy_SignedInfo:

    ds_SignedInfoType_destroy_contents(&SignedInfo);

destroy_KeyInfo:

    ds_KeyInfoType_destroy_contents(&KeyInfo);

destroy_Timestamp:

    wsseu_TimestampType_destroy_contents(&Timestamp);

destroy_pkipath:

destroy_BST:

    wsse_BinarySecurityTokenType_destroy_contents(&bst);

exit:

    GlobusWSSEDebugExit();
    return result;
}

static
globus_result_t
globus_l_handler_ws_security_mark_signed_info(
    globus_soap_message_handle_t        message_handle,
    const xsd_QName *                   type)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_handler_ws_security_mark_signed_info);
    GlobusWSSEDebugEnter();

    result = globus_soap_message_deserialize_mark(
	message_handle, SIGNED_INFO_MARKER);

    GlobusWSSEDebugExit();
    return result;
}

static
globus_result_t
globus_l_handler_ws_security_mark_signed_info_end(
    globus_soap_message_handle_t        message_handle,
    const xsd_QName *                   type)
{
    xsd_QName                           next_qn;
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_handler_ws_security_mark_signed_info);
    GlobusWSSEDebugEnter();

    xsd_QName_init_contents(&next_qn);

    result = globus_soap_message_deserialize_element_unknown(
	message_handle, &next_qn);
    if(result != GLOBUS_SUCCESS)
    {
	goto exit;
    }

    globus_soap_message_deserialize_push_element(message_handle);

    result = globus_soap_message_deserialize_mark(
	message_handle, SIGNED_INFO_MARKER_END);

exit:

    xsd_QName_destroy_contents(&next_qn);
    GlobusWSSEDebugExit();
    return result;
}

static
globus_result_t
globus_handler_ws_security_verify_message(
    globus_soap_message_handle_t        message_handle)
{
    globus_gsi_cred_handle_t            peer_cred = NULL;
    char *                              signed_info_buff = NULL;
    size_t                              signed_info_buff_length = 0;
    char *                              soap_message_buff = NULL;
    size_t                              soap_message_buff_length = 0;
    char *                              c14n_signed_info_buff = NULL;
    size_t                              c14n_signed_info_buff_length = 0;
    globus_result_t                     result = GLOBUS_SUCCESS;
    int                                 index = 0;
    wsseu_TimestampType *               Timestamp = NULL;
    ds_SignatureType *                  Signature = NULL;
    wsse_SecurityHeaderType *           SecurityHeader = NULL;
    GlobusFuncName(globus_handler_ws_security_verify_message);
    GlobusWSSEDebugEnter();

    if(result != GLOBUS_SUCCESS)
    {
	goto exit;
    }

    result = globus_soap_message_add_callout(
	message_handle,
	&ds_SignedInfo_qname,
	globus_l_handler_ws_security_mark_signed_info,
	globus_l_handler_ws_security_mark_signed_info_end,
	NULL);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusWSSEErrorDeserializeFailed(
	    result, "Failed to set marker for SignedInfo element");
	goto exit;
    }

    result = wsse_SecurityHeaderType_init(
	&SecurityHeader);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusWSSEErrorDeserializeFailed(
	    result, "Failed to init Security header");
	goto remove_callout_exit;
    }

    result = wsse_SecurityHeaderType_deserialize(
	&wsse_Security_qname,
	SecurityHeader,
	message_handle,
	0);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusWSSEErrorDeserializeFailed(
	    result, "Failed to deserialize Security header element");
	goto remove_callout_exit;
    }

    for(; index < SecurityHeader->any.length; ++index)
    {
	if(xsd_QName_keyeq(SecurityHeader->any.elements[index].element,
			   &wsse_BinarySecurityToken_qname))
	{
	    wsse_BinarySecurityTokenType * bst;

	    if(SecurityHeader->any.elements[index].any_info != 
	       &wsse_BinarySecurityToken_info)
	    {
		result = GlobusWSSEErrorFailedBinarySecurityToken(
		    "<BinarySecurityToken> element did not get "
		    "deserialized into a BinarySecurityTokenType");
		goto SecurityHeader_destroy;
	    }

	    bst = (wsse_BinarySecurityTokenType *)
		SecurityHeader->any.elements[index].value;

	    result = globus_ws_security_get_credential(
		bst->base_value,
		strlen(bst->base_value),
		&peer_cred);
	    if(result != GLOBUS_SUCCESS)
	    {
		result = GlobusWSSEErrorFailedBinarySecurityToken(
		    "Failed to get credential from PKIPath buffer");
		goto SecurityHeader_destroy;
	    }
	}
	else if(xsd_QName_keyeq(
		SecurityHeader->any.elements[index].element,
		&ds_Signature_qname))
	{
	    if(SecurityHeader->any.elements[index].any_info != 
	       &ds_Signature_info)
	    {
		result = GlobusWSSEErrorFailedVerify(
		    GLOBUS_SUCCESS,
		    "<Signature> element did not get "
		    "deserialized into a SignatureType");
		goto SecurityHeader_destroy;
	    }

	    Signature = (ds_SignatureType *)
		SecurityHeader->any.elements[index].value;
	}
	else if(xsd_QName_keyeq(
		SecurityHeader->any.elements[index].element,
		&wsseu_Timestamp_qname))
	{
	    if(SecurityHeader->any.elements[index].any_info != 
	       &wsseu_Timestamp_info)
	    {
		result = GlobusWSSEErrorFailedVerify(
		    GLOBUS_SUCCESS,
		    "<Timestamp> element did not get "
		    "deserialized into a TimestampType");
		goto SecurityHeader_destroy;
	    }

	    Timestamp = (wsseu_TimestampType *)
		SecurityHeader->any.elements[index].value;
	}
    }

    if(!peer_cred)
    {
	result = GlobusWSSEErrorFailedVerify(
	    GLOBUS_SUCCESS,
	    "Invalid Security header element: "
	    "no BinarySecurityToken element found");
	goto SecurityHeader_destroy;
    }

    if(!Signature)
    {
	result = GlobusWSSEErrorFailedVerify(
	    GLOBUS_SUCCESS,
	    "Invalid Security header element: no Signature element found");
	goto SecurityHeader_destroy;
    }

    if(!Timestamp)
    {
	result = GlobusWSSEErrorFailedVerify(
	    GLOBUS_SUCCESS,
	    "Invalid Security header element: no Timestamp found");
	goto SecurityHeader_destroy;
    }

    if(!globus_soap_message_handle_get_attr(
	    message_handle, GLOBUS_SOAP_MESSAGE_AUTH_ANONYMOUS_KEY))
    {
       char *                                certdir;
       globus_gsi_callback_data_t            callback_data;

       result = globus_gsi_callback_data_init(&callback_data);
       if(result != GLOBUS_SUCCESS)
       {
           result = GlobusWSSEErrorFailedVerify(
               result, "Peer credential verification failed");
           goto SecurityHeader_destroy;
       }

       result = GLOBUS_GSI_SYSCONFIG_GET_CERT_DIR(&certdir);
       if(result != GLOBUS_SUCCESS)
       {
           globus_gsi_callback_data_destroy(callback_data);
           result = GlobusWSSEErrorFailedVerify(
               result, "Peer credential verification failed");
           goto SecurityHeader_destroy;
       }

       result = globus_gsi_callback_set_cert_dir(
           callback_data,
           certdir);
       if(result != GLOBUS_SUCCESS)
       {
           globus_free(certdir);
           globus_gsi_callback_data_destroy(callback_data);
           result = GlobusWSSEErrorFailedVerify(
               result, "Peer credential verification failed");
           goto SecurityHeader_destroy;
       }

       globus_free(certdir);

       result = globus_gsi_cred_verify_cert_chain(
           peer_cred,
           callback_data);

        globus_gsi_callback_data_destroy(callback_data);

	if(result != GLOBUS_SUCCESS)
	{
	    result = GlobusWSSEErrorFailedVerify(
		result, "Peer credential verification failed");
	    goto SecurityHeader_destroy;
	}
    }

    /* get SignedInfo buffer */
    result = globus_soap_message_get_marked_buffers(
	message_handle, SIGNED_INFO_MARKER, SIGNED_INFO_MARKER_END,
	&signed_info_buff, &signed_info_buff_length);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusWSSEErrorFailedVerify(
	    result, "Failed to get SignedInfo buffer");
	goto SecurityHeader_destroy;
    }

    /* verify SignedInfo with what's in SignatureValue */
    result = globus_i_handler_ws_security_verify_signature(
	peer_cred,
	signed_info_buff,
	signed_info_buff_length,
	Signature->SignatureValue.base_value.value,
	Signature->SignatureValue.base_value.length);
    if(result != GLOBUS_SUCCESS)
    {
	xsd_string_array *              inc_ns_prefixes = NULL;

	GlobusWSSEDebugPrintf(
	    GLOBUS_L_WS_SECURITY_DEBUG_VERIFY,
	    ("Direct Verify of SignedInfo element failed.\n"));

	GlobusWSSEDebugPrintf(
	    GLOBUS_L_WS_SECURITY_DEBUG_C14N,
	    ("-----NON-C14N SignedInfo (length: %d)-----\n%.*s\n"
	     "---------------------------------------------\n", 
	     signed_info_buff_length, 
	     signed_info_buff_length, 
	     signed_info_buff));

	if(Signature->SignedInfo.CanonicalizationMethod.any.length > 0)
	{
	    if(Signature->SignedInfo.CanonicalizationMethod.
	       any.elements[0].any_info == 
	       (&exc_c14n_InclusiveNamespaces_info))
	    {
		exc_c14n_InclusiveNamespacesType * incs = 
		    (exc_c14n_InclusiveNamespacesType *)
		    Signature->SignedInfo.CanonicalizationMethod.
		    any.elements[0].value;

		inc_ns_prefixes = incs->_PrefixList;
	    }
	}

	result = globus_soap_message_get_marked_buffers(
	    message_handle, 
	    NULL, NULL,
	    &soap_message_buff, &soap_message_buff_length);
	if(result != GLOBUS_SUCCESS)
	{
	    result = GlobusWSSEErrorFailedVerify(
		result, "Failed to get SOAP message buffer");
	    goto SecurityHeader_destroy;
	}

	result = globus_l_handler_ws_security_c14n_signed_info(
	    message_handle,
	    soap_message_buff,
	    soap_message_buff_length,
	    &c14n_signed_info_buff,
	    &c14n_signed_info_buff_length,
	    inc_ns_prefixes);
	if(result != GLOBUS_SUCCESS)
	{
	    result = GlobusWSSEErrorFailedVerify(
		result,
		"Failed to perform c14n of the SignedInfo node");
	    goto signed_info_destroy;
	}

	GlobusWSSEDebugPrintf(
	    GLOBUS_L_WS_SECURITY_DEBUG_C14N,
	    ("-----C14N SignedInfo (length: %d)-----\n%.*s\n"
	     "---------------------------------------------\n", 
	     c14n_signed_info_buff_length, 
	     c14n_signed_info_buff_length, 
	     c14n_signed_info_buff));

	result = globus_i_handler_ws_security_verify_signature(
	    peer_cred,
	    c14n_signed_info_buff,
	    c14n_signed_info_buff_length,
	    Signature->SignatureValue.base_value.value,
	    Signature->SignatureValue.base_value.length);
	if(result != GLOBUS_SUCCESS)
	{
	    result = GlobusWSSEErrorFailedVerify(
		result,
		"Failed to verify the signature of the message");
	    goto signed_info_destroy;
	}
    }

    globus_soap_message_handle_set_attr(
	message_handle, 
	SECURITY_HEADER_ELEMENT_KEY,
	wsse_SecurityHeaderType_copy_wrapper,
	wsse_SecurityHeaderType_destroy_wrapper,
	(void *)SecurityHeader);
    
signed_info_destroy:

    if(signed_info_buff)
    {
	globus_free(signed_info_buff);
    }

    if(c14n_signed_info_buff)
    {
	globus_free(c14n_signed_info_buff);
    }

SecurityHeader_destroy:

    if(SecurityHeader)
    {
        wsse_SecurityHeaderType_destroy(SecurityHeader);
    }
    
remove_callout_exit:
    globus_soap_message_remove_callouts(
	message_handle,
	&ds_SignedInfo_qname);
exit:

    if(soap_message_buff)
    {
	globus_free(soap_message_buff);
    }

    if(peer_cred)
    {
	globus_gsi_cred_handle_destroy(peer_cred);
    }

    GlobusWSSEDebugExit();
    return result;
}

static
globus_result_t
globus_handler_ws_security_verify_message_references(
    globus_soap_message_handle_t        message_handle)
{
    int                                 index = 0;
    globus_result_t                     result = GLOBUS_SUCCESS;
    ds_SignatureType *                  Signature;
    wsse_SecurityHeaderType *           SecurityHeader;
    GlobusFuncName(globus_handler_ws_security_verify_message_references);
    GlobusWSSEDebugEnter();

    if(result != GLOBUS_SUCCESS)
    {
	goto exit;
    }

    SecurityHeader = globus_soap_message_handle_get_attr(
	message_handle,
	SECURITY_HEADER_ELEMENT_KEY);
    if(!SecurityHeader)
    {
	result = GlobusWSSEErrorFailedVerify(
	    result,
	    "Unable to get Security header element from "
	    "message attributes.");
	goto exit;
    }

    for(; index < SecurityHeader->any.length; ++index)
    {
	if(xsd_QName_keyeq(
		SecurityHeader->any.elements[index].element,
		&ds_Signature_qname))
	{
	    Signature = (ds_SignatureType *)
		SecurityHeader->any.elements[index].value;
	}
    }

    result = globus_i_handler_ws_security_verify_references(
	message_handle, &Signature->SignedInfo.Reference);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusWSSEErrorFailedVerify(
	    result,
	    "Unable to verify digest values of references "
	    "for SignedInfo element");
	goto exit;
    }

exit:

    GlobusWSSEDebugExit();
    return result;
}

static
globus_result_t
globus_l_ws_security_reference_ns_set(
    globus_soap_message_handle_t        message_handle,
    const xsd_QName *                   element,
    globus_xsd_element_options_t *      options)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    globus_bool_t                       done_with_body;
    GlobusFuncName(globus_l_ws_security_reference_ns_set);
    GlobusWSSEDebugEnter();

    /* Skip instances of reference properties in the message body */
    done_with_body = (globus_bool_t) globus_soap_message_handle_get_attr(
	message_handle,
        WSSE_DONE_WITH_BODY_KEY);

    if ((!done_with_body) &&
            globus_l_ws_security_is_reference_property_element(element))
    {
        return GLOBUS_SUCCESS;
    }
    globus_soap_message_serialize_start_c14n_subset(
	message_handle);

    result = globus_soap_message_element_ns_set(
	message_handle,
	WSSEU_NS,
	options);

    GlobusWSSEDebugExit();
    return result;
}

static
globus_result_t
globus_l_ws_security_reference_ns_add(
    globus_soap_message_handle_t        message_handle,
    const xsd_QName *                   element,
    globus_xsd_element_options_t *      options)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    globus_bool_t                       done_with_body;
    GlobusFuncName(globus_l_ws_security_reference_ns_set);
    GlobusWSSEDebugEnter();

    /* Skip instances of reference properties in the message body */
    done_with_body = (globus_bool_t) globus_soap_message_handle_get_attr(
	message_handle,
        WSSE_DONE_WITH_BODY_KEY);

    if ((!done_with_body) &&
            globus_l_ws_security_is_reference_property_element(element))
    {
        return GLOBUS_SUCCESS;
    }
    result = globus_soap_message_element_ns_add(
	message_handle,
	WSSEU_NS,
	options);

    GlobusWSSEDebugExit();
    return result;
}    

static
globus_result_t
globus_l_ws_security_reference_ns_remove(
    globus_soap_message_handle_t        message_handle,
    const xsd_QName *                   element,
    globus_xsd_element_options_t *      options)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    globus_bool_t                       done_with_body;
    GlobusFuncName(globus_l_ws_security_reference_ns_remove);
    GlobusWSSEDebugEnter();

    /* Skip instances of reference properties in the message body */
    done_with_body = (globus_bool_t) globus_soap_message_handle_get_attr(
	message_handle,
        WSSE_DONE_WITH_BODY_KEY);

    if ((!done_with_body) &&
            globus_l_ws_security_is_reference_property_element(element))
    {
        return GLOBUS_SUCCESS;
    }
    globus_soap_message_element_ns_remove(
	message_handle,
	WSSEU_NS,
	options);

    GlobusWSSEDebugExit();
    return result;
}    

static
globus_result_t
globus_l_ws_security_reference_begin(
    globus_soap_message_handle_t        message_handle,
    const xsd_QName *                   type)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    char *                              idval;
    char *                              idkey;
    globus_xsd_element_options_t *      options;
    globus_bool_t                       done_with_body;
    GlobusFuncName(globus_l_ws_security_reference_begin);
    GlobusWSSEDebugEnter();

    /* Skip instances of reference properties in the message body */
    done_with_body = (globus_bool_t) globus_soap_message_handle_get_attr(
	message_handle,
        WSSE_DONE_WITH_BODY_KEY);

    if ((!done_with_body) &&
            globus_l_ws_security_is_reference_property_element(type))
    {
        return GLOBUS_SUCCESS;
    }

    idval = globus_soap_message_construct_id("id-");

    result = globus_soap_message_set_marker(
	message_handle, idval);
    if(result != GLOBUS_SUCCESS)
    {
	globus_free(idval);
	result = GlobusWSSEErrorWithIdAttr(
	    result, type);
	goto exit;
    }

    idkey = globus_common_create_string(
	"{%s}%s#Id",
	type->Namespace,
	type->local);

    result = globus_soap_message_handle_set_attr(
	message_handle,
	idkey,
	globus_soap_message_attr_copy_string,
	globus_libc_free,
	(void *)idval);
    if(result != GLOBUS_SUCCESS)
    {
	globus_free(idval);
	globus_free(idkey);
	result = GlobusWSSEErrorWithIdAttr(
	    result, type);
	goto exit;
    }

    globus_free(idval);
    globus_free(idkey);

    result = globus_soap_message_attribute_ns_set(
	message_handle, wsseu_Id_qname.Namespace, options);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusWSSEErrorWithIdAttr(
	    result, type);
    }

exit:

    GlobusWSSEDebugExit();
    return result;
}

static
globus_result_t
globus_l_ws_security_reference_attr(
    globus_soap_message_handle_t        message_handle,
    const xsd_QName *                   type)
{
    char *                              idkey;
    char *                              idval;
    globus_result_t                     result = GLOBUS_SUCCESS;
    globus_bool_t                       done_with_body;
    GlobusFuncName(globus_l_ws_security_reference_attr);
    GlobusWSSEDebugEnter();

    /* Skip instances of reference properties in the message body */
    done_with_body = (globus_bool_t) globus_soap_message_handle_get_attr(
	message_handle,
        WSSE_DONE_WITH_BODY_KEY);
    if ((!done_with_body) &&
            globus_l_ws_security_is_reference_property_element(type))
    {
        return GLOBUS_SUCCESS;
    }
    idkey = globus_common_create_string(
	"{%s}%s#Id", 
	type->Namespace,
	type->local);
    if(!idkey)
    {
	result = GlobusWSSEErrorWithIdAttr(GLOBUS_SUCCESS, type);
	goto exit;
    }

    idval = globus_soap_message_handle_get_attr(
	message_handle,
	idkey);
    if(!idval)
    {
	result = GlobusWSSEErrorWithIdAttr(GLOBUS_SUCCESS, type);
	goto exit;
    }

    globus_free(idkey);

    result = globus_soap_message_serialize_string_attribute(
	message_handle, &wsseu_Id_qname, &idval);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusWSSEErrorWithIdAttr(result, type);
	goto exit;
    }

exit:
    GlobusWSSEDebugExit();
    return result;
}

static
globus_result_t
globus_l_ws_security_reference_end(
    globus_soap_message_handle_t        message_handle,
    const xsd_QName *                   type)
{
    char *                              idkey = NULL;
    char *                              idval;
    char *                              end_idval = NULL;
    globus_result_t                     result = GLOBUS_SUCCESS;
    globus_bool_t                       done_with_body;
    globus_list_t **                    security_attrs;
    GlobusFuncName(globus_i_ws_security_reference_end);
    GlobusWSSEDebugEnter();

    /* Skip instances of reference properties in the message body */
    done_with_body = (globus_bool_t) globus_soap_message_handle_get_attr(
	message_handle,
        WSSE_DONE_WITH_BODY_KEY);
    if ((!done_with_body) &&
            globus_l_ws_security_is_reference_property_element(type))
    {
        return GLOBUS_SUCCESS;
    }
    idkey = globus_common_create_string(
	"{%s}%s#Id", 
	type->Namespace,
	type->local);
    if(!idkey)
    {
	result = GlobusWSSEErrorWithIdAttr(
	    GLOBUS_SUCCESS, type);
	goto exit;
    }

    idval = globus_soap_message_handle_get_attr(
	message_handle,
	idkey);
    if(!idval)
    {
	result = GlobusWSSEErrorWithIdAttr(
	    GLOBUS_SUCCESS, type);
	goto exit;
    }

    end_idval = globus_common_create_string("%s-end", idval);
    result = globus_soap_message_set_marker(
	message_handle, end_idval);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusWSSEErrorWithIdAttr(
	    result, type);
	goto exit;
    }

    security_attrs = globus_soap_message_handle_get_attr(
	message_handle,
	GLOBUS_HANDLER_WSSE_TOKEN_IDS_KEY);
    globus_assert(security_attrs);

    globus_list_insert(security_attrs,
		       globus_libc_strdup(idval));

    globus_soap_message_serialize_end_c14n_subset(
	message_handle);

    if (xsd_QName_keyeq((xsd_QName *) type, &soap_body_qname))
    {
        result = globus_soap_message_handle_set_attr(
            message_handle,
            WSSE_DONE_WITH_BODY_KEY,
            NULL,
            NULL,
            (void *)1);
    }
exit:

    if(idkey)
    {
	globus_free(idkey);
    }
    
    if(end_idval)
    {
	globus_free(end_idval);
    }
    
    GlobusWSSEDebugExit();
    return result;
}

#define GLOBUS_L_HANDLER_WSSE_ADD_CALLOUTS(__QNAME__)   \
    result = globus_soap_message_add_callout(           \
	message_handle,                                 \
	__QNAME__,                                      \
	globus_l_ws_security_reference_begin,           \
	globus_l_ws_security_reference_end,             \
	globus_l_ws_security_reference_attr);           \
    if(result != GLOBUS_SUCCESS)                        \
    {                                                   \
        result = GlobusWSSEErrorCalloutInit(result);    \
        goto exit;                                      \
    }                                                   \
                                                        \
    result = globus_soap_message_add_ns_callout(        \
	message_handle,                                 \
	__QNAME__,                                      \
	globus_l_ws_security_reference_ns_set,          \
	globus_l_ws_security_reference_ns_add,          \
	globus_l_ws_security_reference_ns_remove);      \
    if(result != GLOBUS_SUCCESS)                        \
    {                                                   \
        result = GlobusWSSEErrorCalloutInit(result);    \
        goto exit;                                      \
    }

static
void
globus_handler_ws_security_init(
    globus_handler_session_descriptor_t session,
    globus_soap_message_handle_t        message_handle,
    globus_result_t                     result)
{
    char *                              prefix = NULL;
    char *                              idval = NULL;
    xmlDocPtr                           doc = NULL;
    globus_xml_buffer *                 xmlbuff;
    globus_list_t **                    security_attrs;
    globus_list_t *                     id_refprops = NULL;
    wsa_EndpointReferenceType *         endpoint_reference = NULL;
    wsa_EndpointReferenceType *         outbound_epr = NULL;
    GlobusFuncName(globus_handler_ws_security_init);
    GlobusWSSEDebugEnter();

    if(result != GLOBUS_SUCCESS)
    {
	goto exit;
    }
    
    security_attrs = globus_malloc(sizeof(globus_list_t *));
    if(!security_attrs)
    {
	result = GlobusWSSEErrorOutOfMemory(sizeof(globus_list_t *));
	goto exit;
    }

    *security_attrs = NULL;

    result = globus_soap_message_handle_set_attr(
	message_handle,
	GLOBUS_HANDLER_WSSE_TOKEN_IDS_KEY,
	NULL, NULL,
	security_attrs);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusWSSEErrorCalloutInit(result);
	goto exit;
    }

    if(globus_soap_message_handle_get_attr(
	    message_handle,
	    WS_SECMESS_INIT_ALREADY_INVOKED))
    {
	goto exit;
    }

    GLOBUS_L_HANDLER_WSSE_ADD_CALLOUTS(&soap_body_qname);
    GLOBUS_L_HANDLER_WSSE_ADD_CALLOUTS(&wsa_MessageID_qname);
    GLOBUS_L_HANDLER_WSSE_ADD_CALLOUTS(&wsa_RelatesTo_qname);
    GLOBUS_L_HANDLER_WSSE_ADD_CALLOUTS(&wsa_ReplyTo_qname);
    GLOBUS_L_HANDLER_WSSE_ADD_CALLOUTS(&wsa_From_qname);
    GLOBUS_L_HANDLER_WSSE_ADD_CALLOUTS(&wsa_FaultTo_qname);
    GLOBUS_L_HANDLER_WSSE_ADD_CALLOUTS(&wsa_To_qname);
    GLOBUS_L_HANDLER_WSSE_ADD_CALLOUTS(&wsa_Action_qname);
    GLOBUS_L_HANDLER_WSSE_ADD_CALLOUTS(&wsseu_Timestamp_qname);

    /* are we server side? */
    if(!globus_soap_message_handle_get_attr(
	    message_handle,
	    GLOBUS_SOAP_MESSAGE_SERVICE_ENDPOINT_KEY))
    {

	endpoint_reference = (wsa_EndpointReferenceType *)
	    globus_soap_message_handle_get_attr(
		message_handle, WSADDR_EPR_KEY);
	if(endpoint_reference)
	{
	    result = globus_soap_message_handle_set_attr(
		message_handle,
		WSADDR_OUTBOUND_EPR_KEY,
		wsa_EndpointReferenceType_copy_wrapper,
		wsa_EndpointReferenceType_destroy_wrapper,
		endpoint_reference);
	    if(result != GLOBUS_SUCCESS)
	    {
		result = GlobusWSSEErrorCalloutInit(result);
		goto exit;
	    }

	    outbound_epr = (wsa_EndpointReferenceType *)
		globus_soap_message_handle_get_attr(
		    message_handle, WSADDR_OUTBOUND_EPR_KEY);
	}
    }

    id_refprops = globus_soap_message_handle_remove_attr(
	message_handle,
	GLOBUS_L_HANDLER_WSSE_REFERENCE_PROPS_KEY);
    if(id_refprops)
    {
        globus_l_handler_wsse_id_refprops_destroy(id_refprops);
	id_refprops = NULL;
    }

    /* inject Id attr into RefProps */
    if(outbound_epr && outbound_epr->ReferenceProperties)
    {
	wsa_ReferencePropertiesType * props;
	int i = 0;

	props = outbound_epr->ReferenceProperties;
	for(; i < props->any.length; ++i)
	{
	    if(props->any.elements[i].any_info ==
	       (&globus_xml_buffer_info) ||
	       props->any.elements[i].any_info ==
	       (&globus_xml_buffer_contents_info))
	    {
		char *                      new_buff;
		int                         new_length;
		int                         created = 0;
		xmlNsPtr                    ns = NULL;
		globus_l_handler_wsse_id_refprop_t * id_refprop;

		xmlbuff =
		    (globus_xml_buffer *)props->any.elements[i].value;

		idval = globus_soap_message_construct_id("id-");
		doc = xmlRecoverMemory(xmlbuff->buffer, xmlbuff->length);

		prefix = globus_soap_message_serialize_lookup_prefix(
		    message_handle, wsseu_Id_qname.Namespace, &created);

		ns = xmlNewNs(doc->children, 
			      wsseu_Id_qname.Namespace,
			      prefix);
	
                xmlUnsetNsProp(doc->children, 
			       ns, 
			       (const xmlChar *)wsseu_Id_qname.local);

                xmlNewNsProp(doc->children,
			     ns,
			     wsseu_Id_qname.local,
			     idval);
		new_length = xmlC14NDocDumpMemory(
		    doc, NULL, 1, NULL, 0, (xmlChar **)&new_buff); 
		if(new_length < 0)
		{
		    result = GlobusWSSEErrorCalloutInit(result);
		    goto exit;
		}

		if(xmlbuff->buffer)
		{
		    globus_free(xmlbuff->buffer);
		}
		xmlbuff->buffer = new_buff;
		xmlbuff->length = new_length;

		id_refprop = globus_malloc(
		    sizeof(globus_l_handler_wsse_id_refprop_t));
		if(!id_refprop)
		{
		    result = GlobusWSSEErrorOutOfMemory(
			sizeof(globus_l_handler_wsse_id_refprop_t));
		    goto exit;
		}

		result = globus_xml_buffer_copy(
		    &id_refprop->xml_buffer, xmlbuff);
		if(result != GLOBUS_SUCCESS)
		{
		    result = GlobusWSSEErrorCalloutInit(result);
		    goto exit;
		}

		id_refprop->id = idval;
		idval = NULL;

		globus_list_insert(&id_refprops, (void *)id_refprop);

		if(doc)
		{
		    xmlFreeDoc(doc);
		    doc = NULL;
		}

		if(prefix)
		{
		    globus_free(prefix);
		    prefix = NULL;
		}
	    }
	    else
	    {
		GLOBUS_L_HANDLER_WSSE_ADD_CALLOUTS(
		    props->any.elements[i].element);
	    }	
	}
    }
   
    if(id_refprops)
    {
	globus_soap_message_handle_set_attr(
	    message_handle,
	    GLOBUS_L_HANDLER_WSSE_REFERENCE_PROPS_KEY, 
	    globus_l_handler_wsse_id_refprops_copy,
	    globus_l_handler_wsse_id_refprops_destroy,
	    id_refprops);
    }

    globus_soap_message_handle_set_attr(
	message_handle,
	WS_SECMESS_INIT_ALREADY_INVOKED,
	NULL, NULL, (void *)1);
    
exit:

    if(id_refprops)
    {
	globus_l_handler_wsse_id_refprops_destroy(id_refprops);
    }

    if(idval)
    {
	globus_free(idval);
    }

    if(doc)
    {
	xmlFreeDoc(doc);
    }

    if(prefix)
    {
	globus_free(prefix);
    }

    globus_handler_finished(session, result);
    GlobusWSSEDebugExit();
    return;
}

static
void
globus_handler_ws_security_destroy(
    globus_handler_session_descriptor_t session,
    globus_soap_message_handle_t        message_handle,
    globus_result_t                     result)
{
    globus_list_t **                    security_attrs;
    wsa_EndpointReferenceType *         outbound_epr = NULL;

    GlobusFuncName(globus_handler_ws_security_destroy);
    GlobusWSSEDebugEnter();

    if(result != GLOBUS_SUCCESS)
    {
	goto exit;
    }

    security_attrs = globus_soap_message_handle_remove_attr(
	message_handle, GLOBUS_HANDLER_WSSE_TOKEN_IDS_KEY);
    if(security_attrs)
    {
	globus_list_destroy_all(*security_attrs, globus_libc_free);
        globus_free(security_attrs);
    }

    globus_soap_message_remove_callouts(message_handle, &soap_body_qname);
    globus_soap_message_remove_callouts(message_handle, &wsa_MessageID_qname);
    globus_soap_message_remove_callouts(message_handle, &wsa_RelatesTo_qname);
    globus_soap_message_remove_callouts(message_handle, &wsa_From_qname);
    globus_soap_message_remove_callouts(message_handle, &wsa_ReplyTo_qname);
    globus_soap_message_remove_callouts(message_handle, &wsa_FaultTo_qname);
    globus_soap_message_remove_callouts(message_handle, &wsa_To_qname);
    globus_soap_message_remove_callouts(message_handle, &wsa_Action_qname);
    globus_soap_message_remove_callouts(
	message_handle, &wsseu_Timestamp_qname);

    outbound_epr = globus_soap_message_handle_get_attr(
        message_handle,
        WSADDR_OUTBOUND_EPR_KEY);

    /* remove callouts for outbound epr reference property elements */
    if(outbound_epr && outbound_epr->ReferenceProperties)
    {
	wsa_ReferencePropertiesType * props;
	int i = 0;

	props = outbound_epr->ReferenceProperties;
	for(; i < props->any.length; ++i)
	{
	    if((props->any.elements[i].any_info !=
	       (&globus_xml_buffer_info)) &&
	       (props->any.elements[i].any_info !=
	       (&globus_xml_buffer_contents_info)))
	    {
                globus_soap_message_remove_callouts(
                    message_handle, props->any.elements[i].element);
	    }	
	}
    }

exit:

    globus_handler_finished(session, result);
    GlobusWSSEDebugExit();
    return;
}

static                        
void
globus_handler_ws_security_client_request_invoke(
    globus_handler_session_descriptor_t session,
    globus_soap_message_handle_t        message_handle,
    globus_result_t                     result)
{
    GlobusFuncName(globus_handler_ws_security_client_request_invoke);
    GlobusWSSEDebugEnter();

    if(result != GLOBUS_SUCCESS)
    {
	goto exit;
    }

    if(globus_soap_message_handle_get_attr(
	    message_handle,
	    WS_SECMESS_CLIENT_REQUEST_ALREADY_INVOKED))
    {
	goto exit;
    }

    result = globus_handler_ws_security_sign_message(
	message_handle);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusWSSEErrorFailedClientRequest(result);
	goto exit;
    }

    globus_soap_message_handle_set_attr(
	message_handle,
	WS_SECMESS_CLIENT_REQUEST_ALREADY_INVOKED,
	NULL, NULL, (void *)1);

exit:

    globus_handler_finished(session, result);
    GlobusWSSEDebugExit();
    return;
}

static                        
void
globus_handler_ws_security_client_response_trigger(
    globus_handler_session_descriptor_t session,
    globus_soap_message_handle_t        message_handle,
    globus_result_t                     result)
{
    GlobusFuncName(globus_handler_ws_security_client_response_trigger);
    GlobusWSSEDebugEnter();

    if(result != GLOBUS_SUCCESS)
    {
	goto exit;
    }

    result = globus_handler_ws_security_verify_message(
	message_handle);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusWSSEErrorFailedClientRequest(result);
	goto exit;
    }

    result = globus_soap_message_remove_required_header_element(
	message_handle, &wsse_Security_qname);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusWSSEErrorFailedClientRequest(result);
	goto exit;
    }

exit:

    globus_handler_finished(session, result);
    GlobusWSSEDebugExit();
    return;
}

static                        
void
globus_handler_ws_security_client_response_invoke(
    globus_handler_session_descriptor_t session,
    globus_soap_message_handle_t        message_handle,
    globus_result_t                     result)
{
    GlobusFuncName(globus_handler_ws_security_client_response_invoke);
    GlobusWSSEDebugEnter();

    if(result != GLOBUS_SUCCESS)
    {
	goto exit;
    }

    if(globus_soap_message_handle_get_attr(
	    message_handle,
	    WS_SECMESS_CLIENT_RESPONSE_ALREADY_INVOKED))
    {
	goto exit;
    }
    
    result = globus_handler_ws_security_verify_message_references(
	message_handle);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusWSSEErrorFailedClientResponse(result);
	goto exit;
    }

    globus_soap_message_handle_set_attr(
	message_handle,
	WS_SECMESS_CLIENT_RESPONSE_ALREADY_INVOKED,
	NULL, NULL, (void *)1);

exit:

    globus_handler_finished(session, result);
    GlobusWSSEDebugExit();
    return;
}

static                        
void
globus_handler_ws_security_server_response_invoke(
    globus_handler_session_descriptor_t session,
    globus_soap_message_handle_t        message_handle,
    globus_result_t                     result)
{
    GlobusFuncName(globus_handler_ws_security_server_response_invoke);
    GlobusWSSEDebugEnter();

    if(result != GLOBUS_SUCCESS)
    {
	goto exit;
    }

    if(globus_soap_message_handle_get_attr(
	    message_handle,
	    WS_SECMESS_SERVER_RESPONSE_ALREADY_INVOKED))
    {
	goto exit;
    }
    
    result = globus_handler_ws_security_sign_message(
	message_handle);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusWSSEErrorFailedServerResponse(result);
	goto exit;
    }

    globus_soap_message_handle_set_attr(
	message_handle,
	WS_SECMESS_SERVER_RESPONSE_ALREADY_INVOKED,
	NULL, NULL, (void *)1);
   
exit:

    globus_handler_finished(session, result);
    GlobusWSSEDebugExit();
    return;
}

static                        
void
globus_handler_ws_security_server_request_trigger(
    globus_handler_session_descriptor_t session,
    globus_soap_message_handle_t        message_handle,
    globus_result_t                     result)
{
    GlobusFuncName(globus_handler_ws_security_server_request_trigger);
    GlobusWSSEDebugEnter();

    if(result != GLOBUS_SUCCESS)
    {
	goto exit;
    }

    result = globus_handler_ws_security_verify_message(
	message_handle);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusWSSEErrorFailedClientRequest(result);
	goto exit;
    }

    result = globus_soap_message_remove_required_header_element(
	message_handle, &wsse_Security_qname);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusWSSEErrorFailedClientRequest(result);
	goto exit;
    }

exit:

    globus_handler_finished(session, result);
    GlobusWSSEDebugExit();
    return;
}

static                        
void
globus_handler_ws_security_server_request_invoke(
    globus_handler_session_descriptor_t session,
    globus_soap_message_handle_t        message_handle,
    globus_result_t                     result)
{
    GlobusFuncName(globus_handler_ws_security_server_request_invoke);
    GlobusWSSEDebugEnter();

    if(result != GLOBUS_SUCCESS)
    {
	goto exit;
    }

    if(globus_soap_message_handle_get_attr(
	    message_handle,
	    WS_SECMESS_SERVER_REQUEST_ALREADY_INVOKED))
    {
	goto exit;
    }
    
    result = globus_handler_ws_security_verify_message_references(
	message_handle);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusWSSEErrorFailedServerRequest(result);
	goto exit;
    }

    globus_soap_message_handle_set_attr(
	message_handle,
	WS_SECMESS_SERVER_REQUEST_ALREADY_INVOKED,
	NULL, NULL, (void *)1);
    
exit:

    globus_handler_finished(session, result);
}

globus_handler_trigger_t globus_l_handler_ws_security_client_trigger =
{
    &wsse_Security_qname,
    globus_handler_ws_security_client_response_trigger
};

globus_handler_descriptor_t globus_l_handler_ws_security_client_descriptor = 
{
    globus_handler_ws_security_init, 
    globus_handler_ws_security_client_request_invoke,
    globus_handler_ws_security_destroy,
    NULL,
    globus_handler_ws_security_client_response_invoke,
    NULL,
    NULL,
    NULL
};

globus_handler_trigger_t globus_l_handler_ws_security_server_trigger =
{
    &wsse_Security_qname,
    globus_handler_ws_security_server_request_trigger
};

globus_handler_descriptor_t globus_l_handler_ws_security_server_descriptor = 
{
    NULL,
    globus_handler_ws_security_server_request_invoke,
    NULL,
    globus_handler_ws_security_init,
    globus_handler_ws_security_server_response_invoke,
    globus_handler_ws_security_destroy,
    NULL,
    NULL
};

GlobusExtensionDefineModule(globus_handler_ws_security) =
{
    GLOBUS_HANDLER_WS_SECMESS_LIB,
    globus_l_handler_ws_security_activate,
    globus_l_handler_ws_security_deactivate,
    NULL,
    NULL,
    &local_version
};

int
globus_l_handler_ws_security_activate(void)
{
    int                                 res = 0;
    GlobusFuncName(globus_l_handler_ws_security_activate);

    GlobusDebugInit(GLOBUS_HANDLER_WS_SECURE_MESSAGE, 
		    TRACE DEBUG PKIPATH C14N SIGN VERIFY);
    GlobusWSSEDebugEnter();

    globus_l_handler_ws_security_client_descriptor.trigger_list = NULL;
    globus_l_handler_ws_security_server_descriptor.trigger_list = NULL;

    globus_i_handler_wsse_cred_pkipath_table_init();

    globus_list_insert(
	&globus_l_handler_ws_security_client_descriptor.trigger_list,
	&globus_l_handler_ws_security_client_trigger);

    res = globus_extension_registry_add(
	GLOBUS_HANDLER_REGISTRY,
	GLOBUS_HANDLER_WS_SECMESS_CLIENT,
	GlobusExtensionMyModule(globus_handler_ws_security),
	&globus_l_handler_ws_security_client_descriptor);
    if(res != GLOBUS_SUCCESS)
    {
	goto exit;
    }

    globus_list_insert(
	&globus_l_handler_ws_security_server_descriptor.trigger_list,
	&globus_l_handler_ws_security_server_trigger);

    res = globus_extension_registry_add(
	GLOBUS_HANDLER_REGISTRY,
	GLOBUS_HANDLER_WS_SECMESS_SERVER,
	GlobusExtensionMyModule(globus_handler_ws_security),
	&globus_l_handler_ws_security_server_descriptor);

    res = globus_xsd_type_registry_insert(
	GLOBUS_GLOBAL_TYPE_REGISTRY,
	&wsse_BinarySecurityToken_info,
	NULL);
    if(res != GLOBUS_SUCCESS)
    {
	goto exit;
    }

    res = globus_xsd_type_registry_insert(
	GLOBUS_GLOBAL_TYPE_REGISTRY,
	&ds_Signature_info,
	NULL);
    if(res != GLOBUS_SUCCESS)
    {
	goto exit;
    }

    res = globus_xsd_type_registry_insert(
	GLOBUS_GLOBAL_TYPE_REGISTRY,
	&wsseu_Timestamp_info,
	NULL);
    if(res != GLOBUS_SUCCESS)
    {
	goto exit;
    }

    res = globus_xsd_type_registry_insert(
	GLOBUS_GLOBAL_TYPE_REGISTRY,
	&wsse_SecurityTokenReference_info,
	NULL);
    if(res != GLOBUS_SUCCESS)
    {
	goto exit;
    }

    res = globus_xsd_type_registry_insert(
	GLOBUS_GLOBAL_TYPE_REGISTRY,
	&ds_Reference_info,
	NULL);
    if(res != GLOBUS_SUCCESS)
    {
	goto exit;
    }

    res = globus_xsd_type_registry_insert(
	GLOBUS_GLOBAL_TYPE_REGISTRY,
	&exc_c14n_InclusiveNamespaces_info,
	NULL);
    if(res != GLOBUS_SUCCESS)
    {
	goto exit;
    }

exit:

    GlobusWSSEDebugExit();
    return res;
}

    int
globus_l_handler_ws_security_deactivate(void)
{
    GlobusFuncName(globus_l_handler_ws_security_deactivate);
    GlobusWSSEDebugEnter();

    globus_extension_registry_remove(
	GLOBUS_HANDLER_REGISTRY,
	GLOBUS_HANDLER_WS_SECMESS_CLIENT);

    globus_extension_registry_remove(
	GLOBUS_HANDLER_REGISTRY,
	GLOBUS_HANDLER_WS_SECMESS_SERVER);

    globus_i_handler_wsse_cred_pkipath_table_destroy();

    GlobusWSSEDebugExit();
    return 0;
}

static
globus_result_t
globus_i_handler_ws_security_compute_digest(
    const char *                        buff,
    size_t                              buff_length,
    char **                             digest_value,
    size_t *                            digest_length)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    char *                              digest;
    char *                              c14n_buff;
    size_t                              c14n_buff_length;
    GlobusFuncName(globus_i_handler_ws_security_compute_digest);
    GlobusWSSEDebugEnter();

    GlobusWSSEDebugPrintf(
	GLOBUS_L_WS_SECURITY_DEBUG_C14N,
	("-----INLINED C14N (length: %d)-----\n%.*s\n"
	 "---------------------------------------------\n", 
	 buff_length, buff_length, buff));

    c14n_buff = (char *)buff;
    c14n_buff_length = buff_length;

#ifndef GLOBUS_HANDLER_WSSE_INLINED_C14N
    result = globus_i_handler_ws_security_c14n_buffer(
	buff,
	buff_length,
	&c14n_buff,
	&c14n_buff_length);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusWSSEErrorFailedSign(result);
	goto exit;
    }

    GlobusWSSEDebugPrintf(
	GLOBUS_L_WS_SECURITY_DEBUG_C14N,
	("-----COMPUTED C14N (length: %d)-----\n%.*s\n"
	 "---------------------------------------------\n", 
	 c14n_buff_length, c14n_buff_length, 
	 c14n_buff));
#endif

    /* compute SHA1 digest */
    digest = globus_malloc(SHA_DIGEST_LENGTH);
    if(!digest)
    {
	result = GlobusWSSEErrorOutOfMemory(SHA_DIGEST_LENGTH);
	globus_free(c14n_buff);
	goto exit;
    }

    if(SHA1(c14n_buff, c14n_buff_length, 
	    digest) == NULL)
    {
	result = GlobusWSSEErrorOpenSSL(
	    "Failed to compute SHA1 digest");
	globus_free(digest);
	globus_free(c14n_buff);
	goto exit;
    }

#ifndef GLOBUS_HANDLER_WSSE_INLINED_C14N
    globus_free(c14n_buff);
#endif

    if(digest_length)
    {
	*digest_length = SHA_DIGEST_LENGTH;
    }

    *digest_value = digest;

exit:

    GlobusWSSEDebugExit();
    return result;
}

static
globus_result_t
globus_i_handler_ws_security_verify_digest(
    const char *                        buff,
    size_t                              buff_length,
    const char *                        digest_value,
    size_t                              digest_length)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    char *                              digest;
    GlobusFuncName(globus_i_handler_ws_security_verify_digest);
    GlobusWSSEDebugEnter();

    GlobusWSSEDebugPrintf(
	GLOBUS_L_WS_SECURITY_DEBUG_VERIFY,
	("-----------------DIGEST BUFFER (length: %d)--------------\n%.*s\n"
	 "=====================================================\n\n",
	 buff_length, buff_length, buff));

    if(digest_length != SHA_DIGEST_LENGTH)
    {
	result = GlobusWSSEErrorFailedDigestVerify(result);
	goto exit;
    }

    digest = globus_malloc(SHA_DIGEST_LENGTH);
    memset(digest, 0, SHA_DIGEST_LENGTH);

    if(SHA1(buff, buff_length,
	    digest) == NULL)
    {
	result = GlobusWSSEErrorOpenSSL(
	    "Could not compute SHA1 digest");
	result = GlobusWSSEErrorFailedDigestVerify(
	    result);
	goto exit;
    }

    if(memcmp(digest, digest_value, SHA_DIGEST_LENGTH))
    {
	globus_free(digest);
	result = GlobusWSSEErrorFailedDigestVerify(
	    result);
	goto exit;
    }

    globus_free(digest);

exit:

    GlobusWSSEDebugExit();
    return result;
}

static
globus_result_t
globus_i_handler_ws_security_verify_signature(
    globus_gsi_cred_handle_t            cred,
    const unsigned char *               input_buff,
    size_t                              input_length,
    const unsigned char *               signed_buff,
    size_t                              signed_buff_length)
{
    EVP_MD_CTX                          rsa_sha1_evp;
    globus_result_t                     result = GLOBUS_SUCCESS;
    int                                 res = 0;
    X509 *                              peer_cert;
    EVP_PKEY *                          pubkey;
    GlobusFuncName(globus_i_handler_ws_security_verify_signature);
    GlobusWSSEDebugEnter();

    if(EVP_VerifyInit(&rsa_sha1_evp, EVP_sha1()) == 0)
    {
	result = GlobusWSSEErrorOpenSSL(
	    "Failed to verify the RSA-SHA1 signature");
	goto exit;
    }

    if(EVP_VerifyUpdate(&rsa_sha1_evp,
			input_buff,
			input_length) == 0)
    {
	result = GlobusWSSEErrorOpenSSL(
	    "Failed to verify the RSA-SHA1 signature");
	goto evp_free;
    }

    result = globus_gsi_cred_get_cert(cred, &peer_cert);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusWSSEErrorFailedSignatureVerify();
	goto evp_free;
    }

    pubkey = X509_get_pubkey(peer_cert);
    res = EVP_VerifyFinal(
	&rsa_sha1_evp,
	(unsigned char *)signed_buff,
	signed_buff_length,
	pubkey);
    if(res == 0)
    {
	result = GlobusWSSEErrorFailedSignatureVerify();
	goto peer_cert_free;
    }
    else if(res < 0)
    {
	result = GlobusWSSEErrorOpenSSL(
	    "Failed to verify the RSA-SHA1 signature");
	goto peer_cert_free;
    }

peer_cert_free:

    EVP_PKEY_free(pubkey);
    X509_free(peer_cert);

evp_free:

    EVP_MD_CTX_cleanup(&rsa_sha1_evp);

exit:

    GlobusWSSEDebugExit();
    ERR_clear_error();
    return result;
}

static
globus_result_t
globus_i_handler_ws_security_compute_signature(
    globus_gsi_cred_handle_t            cred,
    const char *                        input_buff,
    size_t                              input_length,
    char **                             signed_buff,
    size_t *                            signed_buff_length)
{
    EVP_MD_CTX                          rsa_sha1_evp;
    EVP_PKEY *                          user_key = NULL;
    char *                              c14n_buff = NULL;
    const char *                        compute_buff;
    size_t                              compute_length;
    char *                              signature_value_buff;
    size_t                              signature_value_buff_length;
    globus_result_t                     result = GLOBUS_SUCCESS;
#ifndef GLOBUS_HANDLER_WSSE_INLINED_C14N
    size_t                              c14n_buff_length = 0;
#endif
    GlobusFuncName(globus_i_handler_ws_security_compute_signature);
    GlobusWSSEDebugEnter();

    GlobusWSSEDebugPrintf(
	GLOBUS_L_WS_SECURITY_DEBUG_C14N,
	("-----INLINED C14N (length: %d)-----\n%.*s\n"
	 "----------------------------------------\n", 
	 input_length, input_length, input_buff));

    compute_buff = input_buff;
    compute_length = input_length;

#ifndef GLOBUS_HANDLER_WSSE_INLINED_C14N
    result = globus_i_handler_ws_security_c14n_buffer(
	input_buff,
	input_length,
	&c14n_buff,
	&c14n_buff_length);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusWSSEErrorFailedSign(result);
	goto exit;
    }

    GlobusWSSEDebugPrintf(
	GLOBUS_L_WS_SECURITY_DEBUG_C14N,
	("-----COMPUTED C14N (length: %d)-----\n%.*s\n"
	 "----------------------------------------\n", 
	 c14n_buff_length, 
	 c14n_buff_length, 
	 c14n_buff));

    compute_buff = c14n_buff;
    compute_length = c14n_buff_length;
#endif

    if(EVP_SignInit(&rsa_sha1_evp, 
		    EVP_sha1()) == 0)
    {
	result = GlobusWSSEErrorOpenSSL(
	    "Could not initialize openssl signing context");
	goto exit;
    }

    if(EVP_SignUpdate(&rsa_sha1_evp, 
		      compute_buff, 
		      compute_length) == 0)
    {
	result = GlobusWSSEErrorOpenSSL(
	    "Could not update openssl signing context "
	    "with content");
	goto evp_free;
    }

    result = globus_gsi_cred_get_key(
	cred, &user_key);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusWSSEErrorFailedSign(result);
	goto evp_free;
    }

    signature_value_buff = globus_malloc(EVP_PKEY_size(user_key));
    if(EVP_SignFinal(&rsa_sha1_evp,
		     signature_value_buff,
		     (unsigned int *)&signature_value_buff_length,
		     user_key) == 0)
    {
	result = GlobusWSSEErrorOpenSSL(
	    "Could not sign content");
	goto user_key_free;
    }

    *signed_buff = signature_value_buff;
    *signed_buff_length = EVP_PKEY_size(user_key);

user_key_free:

    EVP_PKEY_free(user_key);

evp_free:

    EVP_MD_CTX_cleanup(&rsa_sha1_evp);

exit:

    if(c14n_buff)
    {
	globus_free(c14n_buff);
    }

    GlobusWSSEDebugExit();
    return result;
}

static
globus_result_t
globus_i_handler_ws_security_verify_references(
    globus_soap_message_handle_t        message_handle,
    ds_ReferenceType_array *            reference_array)
{
    char *                              buffer;
    size_t                              length;
    char *                              id;
    char *                              idend;
    int                                 i = 0;
    xmlNodePtr                          referenced_node = NULL;
    xmlDocPtr                           referenced_doc = NULL;
    xmlChar **                          inc_ns_prefixes = NULL;
    xmlDocPtr                           doc = NULL;
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_i_handler_ws_security_verify_references);
    GlobusWSSEDebugEnter();

    for(i = 0; i < reference_array->length; ++i)
    {
	if(reference_array->elements[i]._URI)
	{
	    if(**reference_array->elements[i]._URI == '#')
	    {
		id = (*reference_array->elements[i]._URI) + 1;
		idend = globus_common_create_string("%s-end", id);

		result = globus_soap_message_get_marked_buffers(
		    message_handle, id, idend, &buffer, &length);
		if(result != GLOBUS_SUCCESS)
		{
		    result = GlobusWSSEErrorFailedIdBufferGet(
			result, id);
		    globus_free(idend);
		    goto exit;
		}

		globus_free(idend);

		/* check Transform Algorithm type and IncNS for exc c14n */
		/* At somepoint this could be a callout to do transform 
		   based on Algorithm type */

		/* compute c14n */

		/* check DigestMethod */
		/* At some point this could be a callout to do Digest
		   based on Digest Algorithm */

		result = globus_i_handler_ws_security_verify_digest(
		    buffer, length, 
		    reference_array->elements[i].DigestValue.value,
		    reference_array->elements[i].DigestValue.length);
		if(result != GLOBUS_SUCCESS)
		{
		    char *              output_buff = NULL;
		    int                 res = 0;
		    char *              soap_message_buff = NULL;
		    int                 soap_message_buff_length = 0;

		    GlobusWSSEDebugPrintf(
			GLOBUS_L_WS_SECURITY_DEBUG_VERIFY,
			("========== INLINED DIGEST "
			 "VERIFY FAILED (id: %s) ========\n"
			 "%.*s\n"
			 "===============================================\n",
			 id, length, buffer));

		    globus_free(buffer);
		    buffer = NULL;
		    
		    result = globus_soap_message_get_marked_buffers(
			message_handle, 
			NULL, NULL,
			&soap_message_buff, &soap_message_buff_length);
		    if(result != GLOBUS_SUCCESS)
		    {
			result = GlobusWSSEErrorFailedReferenceVerify(
			    result, id);
			goto exit;
		    }

		    doc = xmlRecoverMemory(
			soap_message_buff, soap_message_buff_length);
		    if(!doc)
		    {
			if(soap_message_buff)
			{
			    globus_free(soap_message_buff);
			    soap_message_buff = NULL;
			}

			result = GlobusWSSEErrorFailedReferenceVerify(
			    result, id);
			goto exit;
		    }

		    if(soap_message_buff)
		    {
			globus_free(soap_message_buff);
			soap_message_buff = NULL;
		    }

		    if(reference_array->elements[i].Transforms->
		       Transform.length == 1 &&
		       reference_array->elements[i].Transforms->
		       Transform.elements[0].choice_value.length == 1 &&
		       reference_array->elements[i].Transforms->
		       Transform.elements[0].choice_value.
		       elements[0].type == ds_TransformType_any &&
		       reference_array->elements[i].Transforms->
		       Transform.elements[0].choice_value.
		       elements[0].value.any.any_info ==
		       &exc_c14n_InclusiveNamespaces_info)
		    {
			exc_c14n_InclusiveNamespacesType * namespaces =
			    (exc_c14n_InclusiveNamespacesType *)
			    reference_array->elements[i].Transforms->
			    Transform.elements[0].choice_value.
			    elements[0].value.any.value;
			int                             ind = 0;
			xsd_string_array *              prefixes = 
			    namespaces->_PrefixList;
			inc_ns_prefixes = xmlMalloc(
			    (prefixes->length + 1) * sizeof(char *));
			if(!inc_ns_prefixes)
			{
			    result = GlobusWSSEErrorOutOfMemory(
				((prefixes->length + 1) * sizeof(char *)));
			    goto exit;
			}
			memset(inc_ns_prefixes, 0, 
			       (sizeof(char *) * (prefixes->length + 1)));

			for(; ind < prefixes->length; ++ind)
			{
			    inc_ns_prefixes[ind] = xmlStrdup(
				prefixes->elements[ind]);
			}
		    }

		    result = globus_l_handler_wsse_get_referenced_node(
			id, doc, &referenced_node);
		    if(result != GLOBUS_SUCCESS)
		    {
			result = GlobusWSSEErrorFailedReferenceVerify(
			    result, id);
			goto exit;
		    }

		    referenced_doc = xmlNewDoc(NULL);
		    if(!referenced_doc)
		    {
			result = GlobusWSSEErrorFailedReferenceVerify(
			    result, id);
			goto exit;
		    }

		    xmlDocSetRootElement(
			referenced_doc,
			xmlDocCopyNode(
			    referenced_node, referenced_doc, 1));
		    
		    if(inc_ns_prefixes)
		    {
			/* XXX - this needs to be looked at..seems
			 * like this shouldn't be necessary
			 */
			int i = 0;
			while(inc_ns_prefixes[i])
			{
			    const xmlChar * href = NULL;
			    globus_l_handler_wsse_lookup_prefix(
				doc, doc->children, 
				inc_ns_prefixes[i],
				&href);
			    if(!href)
			    {
				result = GlobusWSSEErrorFailedReferenceVerify(
				    result, id);
				goto exit;
			    }

			    xmlNewNs(referenced_doc->children,
				     href,
				     inc_ns_prefixes[i]);
			    ++i;
			}
		    }

		    if(doc)
		    {
			xmlFreeDoc(doc);
			doc = NULL;
		    }

		    res = xmlC14NDocDumpMemory(
			referenced_doc, 
			NULL, 
			1, 
			inc_ns_prefixes, 
			0, 
			(xmlChar **)&output_buff); 
		    if(res < 0)
		    {
			result = GlobusWSSEErrorFailedReferenceVerify(
			    result, id);
			goto exit;
		    }

		    if(inc_ns_prefixes)
		    {
			int i = 0;
			while(inc_ns_prefixes[i])
			{
			    xmlFree(inc_ns_prefixes[i]);
			    ++i;
			}
			xmlFree(inc_ns_prefixes);
			inc_ns_prefixes = NULL;
		    }
		    
		    if(referenced_doc)
		    {
                        xmlFreeDoc(referenced_doc);
			referenced_doc = NULL;
		    }

		    result = globus_i_handler_ws_security_verify_digest(
			output_buff, res, 
			reference_array->elements[i].DigestValue.value,
			reference_array->elements[i].DigestValue.length);
		    if(result != GLOBUS_SUCCESS)
		    {
			GlobusWSSEDebugPrintf(
			    GLOBUS_L_WS_SECURITY_DEBUG_VERIFY,
			    ("============= COMPUTED C14N DIGEST "
			     "VERIFY FAILED (id: %s) ==============\n"
			     "%.*s\n"
			     "============================================\n",
			     id, res, output_buff));

			if(output_buff)
			{
			    globus_free(output_buff);
			}

			result = GlobusWSSEErrorFailedReferenceVerify(
			    result, id);
			goto exit;
		    }

		    if(output_buff)
		    {
			globus_free(output_buff);
		    }
		}

		if(buffer)
		{
		    globus_free(buffer);
		}
	    }
	}
    }

exit:

    if(inc_ns_prefixes)
    {
	int i = 0;
	while(inc_ns_prefixes[i])
	{
	    xmlFree(inc_ns_prefixes[i]);
	    ++i;
	}
	xmlFree(inc_ns_prefixes);
    }

    if(referenced_doc)
    {
	xmlFreeDoc(referenced_doc);
    }

    if(doc)
    {
	xmlFreeDoc(doc);
    }

    GlobusWSSEDebugExit();
    return result;
}

static
globus_result_t
globus_i_handler_ws_security_add_references(
    globus_soap_message_handle_t        message_handle,
    ds_ReferenceType_array *            reference_array)
{
    char *                              idvalue;
    char *                              idvalue_end;
    char *                              buffer;
    size_t                              buffer_length;
    globus_list_t **                    security_ids;
    globus_list_t *                     secids;
    ds_ReferenceType *                  reference;
    ds_TransformType *                  transform;
    globus_list_t *                     id_refprops;
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_i_handler_ws_security_add_references);
    GlobusWSSEDebugEnter();

    security_ids = (globus_list_t **)globus_soap_message_handle_get_attr(
	message_handle, 
	GLOBUS_HANDLER_WSSE_TOKEN_IDS_KEY);
    globus_assert(security_ids && *security_ids);
    secids = *security_ids;
    while(secids)
    {
	idvalue = (char *)globus_list_first(secids);

	reference = ds_ReferenceType_array_push(
	    reference_array);

	xsd_ID_init(&reference->_URI);
	*reference->_URI = globus_common_create_string("#%s", idvalue);
	reference->DigestMethod._Algorithm = globus_libc_strdup(SHA1_NS);
	ds_TransformsType_init(&reference->Transforms);
	transform = ds_TransformType_array_push(
	    &reference->Transforms->Transform);
	transform->_Algorithm = globus_libc_strdup(EXC_C14N_NS);

	idvalue_end = globus_common_create_string("%s-end", idvalue);

	result = globus_soap_message_get_marked_buffers(
	    message_handle,
	    idvalue,
	    idvalue_end,
	    &buffer,
	    &buffer_length);
	if(result != GLOBUS_SUCCESS)
	{
	    globus_free(idvalue_end);
	    result = GlobusWSSEErrorFailedSign(result);
	    goto exit;
	}

	globus_free(idvalue_end);

	GlobusWSSEDebugPrintf(
	    GLOBUS_L_WS_SECURITY_DEBUG_DEBUG,
	    ("====================== ID: %s (length: %d) ========\n"
	     "%.*s\n"
	     "========================================================\n",
	     idvalue, buffer_length, buffer_length, buffer));

	result = globus_i_handler_ws_security_compute_digest(
	    buffer, buffer_length,
	    &reference->DigestValue.value,
	    &reference->DigestValue.length);
	if(result != GLOBUS_SUCCESS)
	{
	    result = GlobusWSSEErrorFailedSign(result);
	    globus_free(buffer);
	    goto exit;
	}

	globus_free(buffer);

	secids = globus_list_rest(secids);
    }

    id_refprops = globus_soap_message_handle_get_attr(
	message_handle,
	GLOBUS_L_HANDLER_WSSE_REFERENCE_PROPS_KEY);
    while(id_refprops && !globus_list_empty(id_refprops))
    {
	globus_l_handler_wsse_id_refprop_t * id_refprop;

	id_refprop = globus_list_first(id_refprops);

	reference = ds_ReferenceType_array_push(
	    reference_array);

	xsd_ID_init(&reference->_URI);
	*reference->_URI = globus_common_create_string(
	    "#%s", id_refprop->id);
	reference->DigestMethod._Algorithm = globus_libc_strdup(SHA1_NS);
	ds_TransformsType_init(&reference->Transforms);
	transform = ds_TransformType_array_push(
	    &reference->Transforms->Transform);
	transform->_Algorithm = globus_libc_strdup(EXC_C14N_NS);

	GlobusWSSEDebugPrintf(
	    GLOBUS_L_WS_SECURITY_DEBUG_DEBUG,
	    ("====================== ID: %s (length: %d) ========\n"
	     "%*s\n"
	     "========================================================\n",
	     id_refprop->id, 
	     id_refprop->xml_buffer->length, 
	     id_refprop->xml_buffer->length, 
	     id_refprop->xml_buffer->buffer));

	result = globus_i_handler_ws_security_compute_digest(
	    id_refprop->xml_buffer->buffer, 
	    id_refprop->xml_buffer->length,
	    &reference->DigestValue.value,
	    &reference->DigestValue.length);
	if(result != GLOBUS_SUCCESS)
	{
	    result = GlobusWSSEErrorFailedSign(result);
	    globus_free(buffer);
	    goto exit;
	}

	id_refprops = globus_list_rest(id_refprops);
    }

exit:

    GlobusWSSEDebugExit();
    return result;
}

static
void
globus_l_handler_wsse_doc_free(
    void *                              doc)
{
    xmlFreeDoc((void *)doc);
}

static
globus_result_t
globus_l_handler_ws_security_c14n_signed_info(
    globus_soap_message_handle_t        message_handle,
    const char *                        input_buff,
    size_t                              input_length,
    char **                             output_buff,
    size_t *                            output_buff_length,
    xsd_string_array *                  prefixes)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    xmlDocPtr                           doc = NULL;
    xmlDocPtr                           signed_info_doc = NULL;
    xmlNodePtr                          signed_info_node = NULL;
    int                                 res = 0;
    xmlChar **                          inc_ns_prefixes = NULL;
    GlobusFuncName(globus_i_handler_ws_security_c14n_signed_info);
    GlobusWSSEDebugEnter();

    doc = xmlRecoverMemory(input_buff, input_length);
    if(!doc)
    {
	result = GLOBUS_FAILURE;
	goto exit;
    }

    if(prefixes)
    {
	int                             ind = 0;
	inc_ns_prefixes = xmlMalloc(
	    (prefixes->length + 1) * sizeof(char *));
	if(!inc_ns_prefixes)
	{
	    result = GlobusWSSEErrorOutOfMemory(
		((prefixes->length + 1) * sizeof(char *)));
	    goto exit;
	}
	memset(inc_ns_prefixes, 0, 
	       (sizeof(char *) * (prefixes->length + 1)));

	for(; ind < prefixes->length; ++ind)
	{
	    inc_ns_prefixes[ind] = xmlStrdup(
		prefixes->elements[ind]);
	}
    }

    result = globus_l_handler_wsse_search_tree(
	doc, &ds_SignedInfo_qname, &signed_info_node);
    if(result != GLOBUS_SUCCESS)
    {
	goto exit;
    }

    signed_info_doc = xmlNewDoc(NULL);
    if(!signed_info_doc)
    {
	result = GLOBUS_FAILURE;
	goto exit;
    }

    xmlUnlinkNode(signed_info_node);

    xmlDocSetRootElement(signed_info_doc, signed_info_node);
    xmlSetTreeDoc(signed_info_node, signed_info_doc); 

    res = xmlC14NDocDumpMemory(
	signed_info_doc, NULL, 1, inc_ns_prefixes, 0, (xmlChar **)output_buff);
    if(res < 0)
    {
	result = GLOBUS_FAILURE;
	goto exit;
    }

    GlobusWSSEDebugPrintf(
	GLOBUS_L_WS_SECURITY_DEBUG_C14N,
	("-----C14N SIGNED INFO (length: %d)-----\n%.*s\n"
	 "---------------------------------------------\n", 
	 res, res, *output_buff));

    *output_buff_length = res;

exit:

    if(signed_info_doc)
    {
	globus_l_handler_wsse_doc_free(signed_info_doc);
    }

    if(doc)
    {
	xmlFreeDoc(doc);
    }

    if(inc_ns_prefixes)
    {
	int                             ind = 0;
	while(inc_ns_prefixes[ind])
	{
	    xmlFree(inc_ns_prefixes[ind]);
	    ind++;
	}
	xmlFree(inc_ns_prefixes);
    }

    GlobusWSSEDebugExit();
    return result;
}

#ifndef GLOBUS_HANDLER_WSSE_INLINED_C14N
static
globus_result_t
globus_i_handler_ws_security_c14n_buffer(
    const char *                        input_buff,
    size_t                              input_length,
    char **                             output_buff,
    size_t *                            output_buff_length)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    size_t                              res = 0;
    xmlDocPtr                           doc = NULL;
    GlobusFuncName(globus_i_handler_ws_security_c14n_buffer);
    GlobusWSSEDebugEnter();

    doc = xmlParseMemory(input_buff, input_length);
    if(!doc)
    {
	result = GLOBUS_FAILURE;
	goto exit;
    }

    res = xmlC14NDocDumpMemory(
	doc, NULL, 1, NULL, 0, (xmlChar **)output_buff);
    if(res < 0)
    {
	result = GLOBUS_FAILURE;
	goto exit;
    }

    *output_buff_length = res;

exit:

    if(doc)
    {
	xmlFreeDoc(doc);
    }

    GlobusWSSEDebugExit();
    return result;
}
#endif

static
globus_result_t
globus_l_handler_wsse_get_referenced_node(
    const char *                        id,
    xmlDocPtr                           doc,
    xmlNodePtr *                        referenced_node)
{
    xmlNodePtr                          iter;
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_l_handler_wsse_get_referenced_node);
    GlobusWSSEDebugEnter();

    iter = doc->children;
    while(iter)
    {
	if(iter->type == XML_ELEMENT_NODE && xmlHasProp(iter, "Id"))
	{
	    xmlChar * idvalue;
	    idvalue = xmlGetProp(iter, "Id");
	    if(idvalue && !xmlStrcmp(idvalue, id))
	    {
		xmlFree(idvalue);
		break;
	    }
	    xmlFree(idvalue);
	}

	if(iter->children)
	{
	    iter = iter->children;
	}
	else if(iter->next)
	{
	    iter = iter->next;
	}
	else if(iter->parent && iter->parent->next)
	{
	    iter = iter->parent->next;
	}
	else
	{
	    while(iter && iter->parent && !iter->parent->next)
	    {
		iter = iter->parent;
	    }
	    if(iter && iter->parent && iter->parent->next)
	    {
		iter = iter->parent->next;
	    }
	    else
	    {
		iter = NULL;
	    }
	}
    }

    if(!iter)
    {
	result = GlobusWSSEErrorFailedReferenceVerify(
	    result, id);
    }

    *referenced_node = iter;

    GlobusWSSEDebugExit();
    return result;
}

static
globus_result_t
globus_l_handler_wsse_lookup_prefix(
    xmlDocPtr                           doc,
    xmlNodePtr                          node,
    const xmlChar *                     prefix,
    const xmlChar **                    href)
{
    xmlNodePtr                          iter;
    xmlNsPtr                            iterns;
    GlobusFuncName(globus_l_handler_wsse_lookup_prefix);
    GlobusWSSEDebugEnter();

    *href = NULL;
    iter = node;
    while(iter)
    {
	iterns = iter->ns;
	while(iterns)
	{
	    if(!xmlStrcmp(iterns->prefix, prefix))
	    {
		*href = iterns->href;
		return GLOBUS_SUCCESS;
	    }
	    iterns = iterns->next;
	}

	if(iter->children)
	{
	    iter = iter->children;
	}
	else if(iter->next)
	{
	    iter = iter->next;
	}
	else if(iter->parent && iter->parent->next)
	{
	    iter = iter->parent->next;
	}
    }
    return GLOBUS_SUCCESS;
}

static
globus_result_t
globus_l_handler_wsse_search_tree(
    xmlDocPtr                           doc,
    xsd_QName *                         element,
    xmlNodePtr *                        node)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    xmlNodePtr                          iter;
    GlobusFuncName(globus_l_handler_wsse_search_tree);
    GlobusWSSEDebugEnter();

    iter = doc->children;
    while(iter)
    {
	if(iter->type == XML_ELEMENT_NODE &&
	   !xmlStrcmp((xmlChar *)ds_SignedInfo_qname.Namespace,
		      iter->ns->href) &&
	   !xmlStrcmp((xmlChar *)ds_SignedInfo_qname.local,
		      iter->name))
	{
	    break;
	}

	if(iter->children)
	{
	    iter = iter->children;
	}
	else if(iter->next)
	{
	    iter = iter->next;
	}
	else if(iter->parent && iter->parent->next)
	{
	    iter = iter->parent->next;
	}
	else
	{
	    while(iter && iter->parent && !iter->parent->next)
	    {
		iter = iter->parent;
	    }
	    if(iter && iter->parent && iter->parent->next)
	    {
		iter = iter->parent->next;
	    }
	    else
	    {
		iter = NULL;
	    }
	}
    }

    if(!iter)
    {
	result = GlobusWSSEErrorFailedTreeSearch(element);
    }

    *node = iter;

    GlobusWSSEDebugExit();
    return result;
}

static
globus_result_t
globus_l_handler_wsse_id_refprop_copy(
    void **                             np,
    void *                              p)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    globus_l_handler_wsse_id_refprop_t * nprop;
    globus_l_handler_wsse_id_refprop_t * prop =
	(globus_l_handler_wsse_id_refprop_t *) p;

    nprop = globus_malloc(sizeof(globus_l_handler_wsse_id_refprop_t));
    if(!nprop)
    {
	result = GlobusWSSEErrorOutOfMemory(
	    sizeof(globus_l_handler_wsse_id_refprop_t));
	return result;
    }
    memset(nprop, 0, sizeof(globus_l_handler_wsse_id_refprop_t));

    nprop->id = globus_libc_strdup(prop->id);
    result = globus_xml_buffer_copy(&nprop->xml_buffer, prop->xml_buffer);
    if(result != GLOBUS_SUCCESS)
    {
	return result;
    }

    *np = (void *)nprop;
    return result;
}

static
void
globus_l_handler_wsse_id_refprop_destroy(
    void *                              p)
{
    globus_l_handler_wsse_id_refprop_t * prop = 
	(globus_l_handler_wsse_id_refprop_t *) p;

    if(prop)
    {
	if(prop->xml_buffer)
	{
	    globus_xml_buffer_destroy(prop->xml_buffer);
	}

	if(prop->id)
	{
	    globus_free(prop->id);
	}

	globus_free(prop);
    }
}

static
void
globus_l_handler_wsse_id_refprops_destroy(
    void *                              rps)
{
    globus_list_t *                     list = (globus_list_t *) rps;

    globus_list_destroy_all(list, globus_l_handler_wsse_id_refprop_destroy);
}

static
globus_result_t
globus_l_handler_wsse_id_refprops_copy(
    void **                             nrps,
    void *                              rps)
{
    globus_list_t *                     newlist = NULL;
    globus_list_t *                     list = (globus_list_t *) rps;

    while(list)
    {
	globus_l_handler_wsse_id_refprop_t * new_refprop;
        globus_l_handler_wsse_id_refprop_copy(
	    (void **)&new_refprop,
	    globus_list_first(list));
	globus_list_insert(&newlist, new_refprop);
        list = globus_list_rest(list);
    }
    *nrps = (void *)newlist;
    return GLOBUS_SUCCESS;
}

static
globus_bool_t
globus_l_ws_security_is_reference_property_element(
    const xsd_QName *                   type)
{
    if (xsd_QName_keyeq((xsd_QName *) type, &soap_body_qname) ||
        xsd_QName_keyeq((xsd_QName *) type, &wsa_MessageID_qname) ||
        xsd_QName_keyeq((xsd_QName *) type, &wsa_RelatesTo_qname) ||
        xsd_QName_keyeq((xsd_QName *) type, &wsa_From_qname) ||
        xsd_QName_keyeq((xsd_QName *) type, &wsa_ReplyTo_qname) ||
        xsd_QName_keyeq((xsd_QName *) type, &wsa_FaultTo_qname) ||
        xsd_QName_keyeq((xsd_QName *) type, &wsa_To_qname) ||
        xsd_QName_keyeq((xsd_QName *) type, &wsa_Action_qname) ||
        xsd_QName_keyeq((xsd_QName *) type, &wsseu_Timestamp_qname))
    {
        return GLOBUS_FALSE;
    }
    else
    {
        return GLOBUS_TRUE;
    }
}
