/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */

#ifndef GLOBUS_I_XSD_H
#define GLOBUS_I_XSD_H

#include "globus_common.h"
#include "globus_soap_message.h"
#include "globus_i_soap_message.h"
#include "globus_soap_message_utils.h"
#include "globus_xsd_type_info.h"
#include "xsd_QName.h"

EXTERN_C_BEGIN

#define GLOBUS_I_XSD_DEFINE_QNAME(prefix, type) \
    xsd_QName prefix##_##type##_qname =         \
    {                                           \
        "http://www.w3.org/2001/XMLSchema",     \
        #type                                   \
    }
 
#define GLOBUS_I_XSD_DEFINE_TYPE_INFO(type)     \
    struct globus_xsd_type_info_s               \
    type##_array_info =                         \
    {                                           \
        &type##_qname,                          \
        type##_array_serialize_wrapper,         \
        type##_array_deserialize_wrapper,       \
        type##_array_init_wrapper,              \
        type##_array_destroy_wrapper,           \
        type##_array_copy_wrapper,              \
        type##_array_init_contents_wrapper,     \
        type##_array_destroy_contents_wrapper,  \
        type##_array_copy_contents_wrapper,     \
        sizeof(type),                           \
        type##_array_push_wrapper,              \
        NULL,                                   \
        NULL                                    \
    };                                          \
                                                \
    struct globus_xsd_type_info_s               \
    type##_contents_info =                      \
    {                                           \
        &type##_qname,                          \
        type##_serialize_contents_wrapper,      \
        type##_deserialize_contents_wrapper,    \
        type##_init_wrapper,                    \
        type##_destroy_wrapper,                 \
        type##_copy_wrapper,                    \
        type##_init_contents_wrapper,           \
        type##_destroy_contents_wrapper,        \
        type##_copy_contents_wrapper,           \
        sizeof(type),                           \
        NULL,                                   \
        NULL,                                   \
        NULL                                    \
    };                                          \
                                                \
    struct globus_xsd_type_info_s               \
    type##_info =                               \
    {                                           \
        &type##_qname,                          \
        type##_serialize_wrapper,               \
        type##_deserialize_pointer_wrapper,     \
        type##_init_wrapper,                    \
        type##_destroy_wrapper,                 \
        type##_copy_wrapper,                    \
        type##_init_contents_wrapper,           \
        type##_destroy_contents_wrapper,        \
        type##_copy_contents_wrapper,           \
        sizeof(type),                           \
        NULL,                                   \
        &type##_contents_info,                  \
        &type##_array_info                      \
    }

#define GLOBUS_I_XSD_DEFINE_TYPE_INIT(type)                             \
    globus_result_t                                                     \
    type##_init(type **object)                                          \
    {                                                                   \
        globus_result_t                 result = GLOBUS_SUCCESS;        \
        GlobusFuncName(type##_init);                                    \
        type * instance;                                                \
                                                                        \
        GlobusSoapMessageDebugEnter();                                  \
                                                                        \
        instance = malloc(sizeof(type));                                \
        if (instance == NULL)                                           \
        {                                                               \
            GlobusSoapMessageDebugExit();                               \
            return GlobusSoapMessageErrorOutOfMemory;                   \
        }                                                               \
                                                                        \
        memset(instance, '\0', sizeof(type));                           \
                                                                        \
        *object = instance;                                             \
        GlobusSoapMessageDebugExit();                                   \
        return result;                                                  \
    }                                                                   \
    globus_result_t                                                     \
    type##_init_wrapper(void **object)                                  \
    {                                                                   \
        return type##_init((type **)object);                            \
    }                                                                   \
    globus_result_t                                                     \
    type##_init_contents_wrapper(void *object)                          \
    {                                                                   \
        return type##_init_contents((type *)object);                    \
    }
    
#define GLOBUS_I_XSD_DEFINE_TYPE_DESTROY(type)          \
    void                                                \
    type##_destroy(                                     \
        type *                          object)         \
    {                                                   \
        GlobusFuncName(type##_destroy);                 \
        GlobusSoapMessageDebugEnter();                  \
                                                        \
        if(!object)                                     \
        {                                               \
            goto exit;                                  \
        }                                               \
                                                        \
        type##_destroy_contents(object);                \
                                                        \
        globus_free(object);                            \
                                                        \
     exit:                                              \
        GlobusSoapMessageDebugExit();                   \
    }                                                   \
    void                                                \
    type##_destroy_wrapper(                             \
        void *                          object)         \
    {                                                   \
        type##_destroy((type *) object);                \
    }                                                   \
    void                                                \
    type##_destroy_contents_wrapper(                    \
        void *                          object)         \
    {                                                   \
        type##_destroy_contents((type *) object);       \
    }

#define GLOBUS_I_XSD_DEFINE_TYPE_COPY(type)                             \
    globus_result_t                                                     \
    type##_copy(                                                        \
        type **                         dest_obj,                       \
        const type *                    src_obj)                        \
    {                                                                   \
        globus_result_t                 result = GLOBUS_SUCCESS;        \
        GlobusFuncName(type##_copy);                                    \
        type * instance;                                                \
                                                                        \
        GlobusSoapMessageDebugEnter();                                  \
                                                                        \
        if(!src_obj)                                                    \
        {                                                               \
            instance = NULL;                                            \
            goto exit;                                                  \
        }                                                               \
                                                                        \
        instance = malloc(sizeof(type));                                \
        if (instance == NULL)                                           \
        {                                                               \
            GlobusSoapMessageDebugExit();                               \
            return GlobusSoapMessageErrorOutOfMemory;                   \
        }                                                               \
                                                                        \
        result = type##_copy_contents(instance, src_obj);               \
        if(result != GLOBUS_SUCCESS)                                    \
        {                                                               \
            globus_free(instance);                                      \
            GlobusSoapMessageDebugExit();                               \
            return GlobusSoapMessageErrorOutOfMemory;                   \
        }                                                               \
                                                                        \
exit:                                                                   \
        *dest_obj = instance;                                           \
        GlobusSoapMessageDebugExit();                                   \
        return result;                                                  \
    }                                                                   \
    globus_result_t                                                     \
    type##_copy_wrapper(                                                \
        void **                         dest_obj,                       \
        void *                          src_obj)                        \
    {                                                                   \
        return type##_copy((type **) dest_obj, (type *) src_obj);       \
    }                                                                   \
    globus_result_t                                                     \
    type##_copy_contents_wrapper(                                       \
        void *                          dest_obj,                       \
        void *                          src_obj)                        \
    {                                                                   \
        return type##_copy_contents(                                    \
            (type *) dest_obj, (const type *) src_obj);                 \
    }

#define GLOBUS_I_XSD_DEFINE_TYPE_SERIALIZE(type)                        \
    globus_result_t                                                     \
    type##_serialize(                                                   \
        xsd_QName *                         element_qname,              \
        type *                              instance,                   \
        globus_soap_message_handle_t        message_handle,             \
        globus_xsd_element_options_t        options)                    \
    {                                                                   \
        globus_result_t                     result = GLOBUS_SUCCESS;    \
        GlobusFuncName(type##_serialize);                               \
        GlobusSoapMessageDebugEnter();                                  \
                                                                        \
        if(!(options & GLOBUS_XSD_ELEMENT_CONTENTS_ONLY))               \
        {                                                               \
            /* add begin element */                                     \
            result = globus_soap_message_invoke_ns_callouts(            \
                    message_handle,                                     \
                    element_qname,                                      \
                    GLOBUS_SOAP_MESSAGE_CALLOUT_NS_SET,                 \
                    &options);                                          \
            if(result != GLOBUS_SUCCESS)                                \
            {                                                           \
                result = GlobusSoapMessageErrorSerializeFailed(         \
                    result, NULL, element_qname);                       \
                goto exit;                                              \
            }                                                           \
                                                                        \
            if(options & GLOBUS_XSD_ELEMENT_TYPE_ATTR &&                \
	       !message_handle->c14n_subset)                            \
            {                                                           \
                result = globus_soap_message_attribute_ns_set(          \
                    message_handle,                                     \
                    "http://www.w3.org/2001/XMLSchema-instance",        \
                    &options);                                          \
                if(result != GLOBUS_SUCCESS)                            \
                {                                                       \
                    result = GlobusSoapMessageErrorSerializeFailed(     \
                        result, NULL, element_qname);                   \
                    goto exit;                                          \
                }                                                       \
                                                                        \
                result = globus_soap_message_attribute_ns_set(          \
                    message_handle,                                     \
                    type##_qname.Namespace,                             \
                    &options);                                          \
                if(result != GLOBUS_SUCCESS)                            \
                {                                                       \
                    result = GlobusSoapMessageErrorSerializeFailed(     \
                        result, NULL, element_qname);                   \
                    goto exit;                                          \
                }                                                       \
            }                                                           \
                                                                        \
            result = globus_soap_message_element_ns_set(                \
                message_handle,                                         \
                element_qname->Namespace,                               \
                &options);                                              \
            if(result != GLOBUS_SUCCESS)                                \
            {                                                           \
                result = GlobusSoapMessageErrorSerializeFailed(         \
                    result, NULL, element_qname);                       \
                goto exit;                                              \
            }                                                           \
            result = globus_soap_message_invoke_callouts(               \
                    message_handle,                                     \
                    element_qname,                                      \
                    GLOBUS_SOAP_MESSAGE_CALLOUT_BEGIN);                 \
            if(result != GLOBUS_SUCCESS)                                \
            {                                                           \
                result = GlobusSoapMessageErrorSerializeFailed(         \
                    result, NULL, element_qname);                       \
                goto exit;                                              \
            }                                                           \
            result = globus_soap_message_serialize_element(             \
                message_handle,                                         \
                element_qname,                                          \
                options,                                                \
                &type##_qname);                                         \
            if(result != GLOBUS_SUCCESS)                                \
            {                                                           \
                result = GlobusSoapMessageErrorSerializeFailed(         \
                    result, NULL, element_qname);                       \
                goto exit;                                              \
            }                                                           \
            result = globus_soap_message_invoke_ns_callouts(            \
                message_handle,                                         \
                element_qname,                                          \
                GLOBUS_SOAP_MESSAGE_CALLOUT_NS_ADD,                     \
                &options);                                              \
            if(result != GLOBUS_SUCCESS)                                \
            {                                                           \
                result = GlobusSoapMessageErrorSerializeFailed(         \
                    result, NULL, element_qname);                       \
                goto exit;                                              \
            }                                                           \
                                                                        \
            /* serialize namespaces */                                  \
            if(options & GLOBUS_XSD_ELEMENT_TYPE_ATTR &&                \
	       !message_handle->c14n_subset)                            \
            {                                                           \
                result = globus_soap_message_attribute_ns_add(          \
                    message_handle,                                     \
                    "http://www.w3.org/2001/XMLSchema-instance",        \
                    &options);                                          \
                if(result != GLOBUS_SUCCESS)                            \
                {                                                       \
                    result = GlobusSoapMessageErrorSerializeFailed(     \
                        result, NULL, element_qname);                   \
                    goto exit;                                          \
                }                                                       \
                                                                        \
                result = globus_soap_message_attribute_ns_add(          \
                    message_handle,                                     \
                    type##_qname.Namespace,                             \
                    &options);                                          \
                if(result != GLOBUS_SUCCESS)                            \
                {                                                       \
                    result = GlobusSoapMessageErrorSerializeFailed(     \
                        result, NULL, element_qname);                   \
                    goto exit;                                          \
                }                                                       \
            }                                                           \
                                                                        \
            result = globus_soap_message_element_ns_add(                \
                message_handle,                                         \
                element_qname->Namespace,                               \
                &options);                                              \
            if(result != GLOBUS_SUCCESS)                                \
            {                                                           \
                result = GlobusSoapMessageErrorSerializeFailed(         \
                    result, NULL, element_qname);                       \
                goto exit;                                              \
            }                                                           \
                                                                        \
            if(options & GLOBUS_XSD_ELEMENT_TYPE_ATTR &&                \
	       !message_handle->c14n_subset)                            \
            {                                                           \
                xsd_QName                   type_attr;                  \
                type_attr.local = "type";                               \
                type_attr.Namespace =                                   \
                        "http://www.w3.org/2001/XMLSchema-instance";    \
                result = globus_soap_message_serialize_QName_attribute( \
                    message_handle,                                     \
                    &type_attr,                                         \
                    &type##_qname);                                     \
                if(result != GLOBUS_SUCCESS)                            \
                {                                                       \
                    result = GlobusSoapMessageErrorSerializeFailed(     \
                        result, NULL, element_qname);                   \
                    goto exit;                                          \
                }                                                       \
            }                                                           \
                                                                        \
            /* serialize attributes */                                  \
            result = globus_soap_message_invoke_callouts(               \
                message_handle,                                         \
                element_qname,                                          \
                GLOBUS_SOAP_MESSAGE_CALLOUT_ATTR);                      \
            if(result != GLOBUS_SUCCESS)                                \
            {                                                           \
                result = GlobusSoapMessageErrorSerializeFailed(         \
                    result, NULL, element_qname);                       \
                goto exit;                                              \
            }                                                           \
                                                                        \
            result = globus_soap_message_serialize_element_begin_close( \
                message_handle);                                        \
            if(result != GLOBUS_SUCCESS)                                \
            {                                                           \
                result = GlobusSoapMessageErrorSerializeFailed(         \
                    result, NULL, element_qname);                       \
                goto exit;                                              \
            }                                                           \
        }                                                               \
                                                                        \
        /* serialize contents of type */                                \
        result = type##_serialize_contents(                             \
            NULL, instance, message_handle, 0);                         \
        if(result != GLOBUS_SUCCESS)                                    \
        {                                                               \
            result = GlobusSoapMessageErrorSerializeFailed(             \
                result, NULL, element_qname);                           \
            goto exit;                                                  \
        }                                                               \
                                                                        \
        if(!(options & GLOBUS_XSD_ELEMENT_CONTENTS_ONLY))               \
        {                                                               \
            /* add end element */                                       \
            result = globus_soap_message_serialize_element_end(         \
                message_handle,                                         \
                element_qname,                                          \
                options,                                                \
                &type##_qname);                                         \
            if(result != GLOBUS_SUCCESS)                                \
            {                                                           \
                result = GlobusSoapMessageErrorSerializeFailed(         \
                    result, NULL, element_qname);                       \
                goto exit;                                              \
            }                                                           \
                                                                        \
            globus_soap_message_element_ns_remove(                      \
                message_handle,                                         \
                element_qname->Namespace,                               \
                &options);                                              \
                                                                        \
            if(options & GLOBUS_XSD_ELEMENT_TYPE_ATTR &&                \
	       !message_handle->c14n_subset)                            \
            {                                                           \
                globus_soap_message_attribute_ns_remove(                \
                    message_handle,                                     \
                    "http://www.w3.org/2001/XMLSchema-instance",        \
                    &options);                                          \
                                                                        \
                globus_soap_message_attribute_ns_remove(                \
                    message_handle,                                     \
                    type##_qname.Namespace,                             \
                    &options);                                          \
            }                                                           \
            result = globus_soap_message_invoke_ns_callouts(            \
                message_handle,                                         \
                element_qname,                                          \
                GLOBUS_SOAP_MESSAGE_CALLOUT_NS_REMOVE,                  \
                &options);                                              \
            if(result != GLOBUS_SUCCESS)                                \
            {                                                           \
                result  = GlobusSoapMessageErrorSerializeFailed(        \
                    result, NULL, element_qname);                       \
                goto exit;                                              \
            }                                                           \
            result = globus_soap_message_invoke_callouts(               \
                message_handle,                                         \
                element_qname,                                          \
                GLOBUS_SOAP_MESSAGE_CALLOUT_END);                       \
            if(result != GLOBUS_SUCCESS)                                \
            {                                                           \
                result  = GlobusSoapMessageErrorSerializeFailed(        \
                    result, NULL, element_qname);                       \
                goto exit;                                              \
            }                                                           \
	}                                                               \
                                                                        \
     exit:                                                              \
                                                                        \
        GlobusSoapMessageDebugExit();                                   \
        return result;                                                  \
    }                                                                   \
    globus_result_t                                                     \
    type##_serialize_wrapper(                                           \
        xsd_QName *                         element_qname,              \
        void *                              instance,                   \
        globus_soap_message_handle_t        message_handle,             \
        globus_xsd_element_options_t        options)                    \
    {                                                                   \
        return type##_serialize(                                        \
            element_qname, (type *) instance, message_handle, options); \
    }                                                                   \
    globus_result_t                                                     \
    type##_serialize_contents_wrapper(                                  \
        xsd_QName *                         element_qname,              \
        void *                              instance,                   \
        globus_soap_message_handle_t        message_handle,             \
        globus_xsd_element_options_t        options)                    \
    {                                                                   \
        return type##_serialize_contents(                               \
            element_qname, (type *) instance, message_handle, options); \
    }
    
#define GLOBUS_I_XSD_DEFINE_ATTR_SERIALIZE(type, base_name)             \
    globus_result_t                                                     \
    type##_serialize_attribute(                                         \
        xsd_QName *                         element_qname,              \
        type *                              inst,                       \
        globus_soap_message_handle_t        message_handle,             \
        globus_xsd_element_options_t        options)                    \
    {                                                                   \
        globus_result_t                     result = GLOBUS_SUCCESS;    \
        GlobusFuncName(type##_serialize_attribute);                     \
        GlobusSoapMessageDebugEnter();                                  \
                                                                        \
        result = globus_soap_message_serialize_##base_name##_attribute( \
            message_handle,                                             \
            element_qname, inst);                                       \
        if(result != GLOBUS_SUCCESS)                                    \
        {                                                               \
            result = GlobusSoapMessageErrorSerializeFailed(             \
                result, NULL, element_qname);                           \
            goto exit;                                                  \
        }                                                               \
                                                                        \
     exit:                                                              \
                                                                        \
        GlobusSoapMessageDebugExit();                                   \
        return result;                                                  \
    }

#define GLOBUS_I_XSD_DEFINE_LIST_SERIALIZE(type, base_name)             \
    globus_result_t                                                     \
    type##_list_serialize(                                              \
        xsd_QName *                         element_qname,              \
        type##_array *                      inst,                       \
        globus_soap_message_handle_t        message_handle,             \
        globus_xsd_element_options_t        options)                    \
    {                                                                   \
        globus_result_t                     result = GLOBUS_SUCCESS;    \
        GlobusFuncName(type##_list_serialize);                          \
        GlobusSoapMessageDebugEnter();                                  \
                                                                        \
        if(!(options & GLOBUS_XSD_ELEMENT_CONTENTS_ONLY))               \
        {                                                               \
            /* add begin element */                                     \
            result = globus_soap_message_element_ns_set(                \
                message_handle,                                         \
                element_qname->Namespace,                               \
                &options);                                              \
            if(result != GLOBUS_SUCCESS)                                \
            {                                                           \
                result = GlobusSoapMessageErrorSerializeFailed(         \
                    result, NULL, element_qname);                       \
                goto exit;                                              \
            }                                                           \
                                                                        \
            result = globus_soap_message_serialize_element(             \
                message_handle,                                         \
                element_qname,                                          \
                options,                                                \
                &type##_qname);                                         \
            if(result != GLOBUS_SUCCESS)                                \
            {                                                           \
                result = GlobusSoapMessageErrorSerializeFailed(         \
                    result, NULL, element_qname);                       \
                goto exit;                                              \
            }                                                           \
                                                                        \
            if(options & GLOBUS_XSD_ELEMENT_TYPE_ATTR &&                \
	       !message_handle->c14n_subset)                            \
            {                                                           \
                xsd_QName                   type_attr;                  \
                type_attr.local = "type";                               \
                type_attr.Namespace =                                   \
                        "http://www.w3.org/2001/XMLSchema-instance";    \
                result = globus_soap_message_serialize_QName_attribute( \
                    message_handle,                                     \
                    &type_attr,                                         \
                    &type##_qname);                                     \
                if(result != GLOBUS_SUCCESS)                            \
                {                                                       \
                    result = GlobusSoapMessageErrorSerializeFailed(     \
                        result, NULL, element_qname);                   \
                    goto exit;                                          \
                }                                                       \
            }                                                           \
                                                                        \
            result = globus_soap_message_serialize_element_begin_close( \
                message_handle);                                        \
            if(result != GLOBUS_SUCCESS)                                \
            {                                                           \
                result = GlobusSoapMessageErrorSerializeFailed(         \
                    result, NULL, element_qname);                       \
                goto exit;                                              \
            }                                                           \
        }                                                               \
                                                                        \
        result = type##_list_serialize_contents(                        \
            inst,                                                       \
            message_handle,                                             \
            options);                                                   \
        if(result != GLOBUS_SUCCESS)                                    \
        {                                                               \
            result = GlobusSoapMessageErrorSerializeFailed(             \
                result, NULL, element_qname);                           \
            goto exit;                                                  \
        }                                                               \
                                                                        \
        if(!(options & GLOBUS_XSD_ELEMENT_CONTENTS_ONLY))               \
        {                                                               \
            /* add end element */                                       \
            result = globus_soap_message_serialize_element_end(         \
                message_handle,                                         \
                element_qname,                                          \
                options,                                                \
                &type##_qname);                                         \
            if(result != GLOBUS_SUCCESS)                                \
            {                                                           \
                result = GlobusSoapMessageErrorSerializeFailed(         \
                    result, NULL, element_qname);                       \
                goto exit;                                              \
            }                                                           \
                                                                        \
            globus_soap_message_element_ns_remove(                      \
                message_handle,                                         \
                element_qname->Namespace,                               \
                &options);                                              \
        }                                                               \
                                                                        \
     exit:                                                              \
                                                                        \
        GlobusSoapMessageDebugExit();                                   \
        return result;                                                  \
    }

#define GLOBUS_I_XSD_DEFINE_LIST_SERIALIZE_CONTENTS(type, base_name)    \
    globus_result_t                                                     \
    type##_list_serialize_contents(                                     \
        type##_array *                      inst,                       \
        globus_soap_message_handle_t        message_handle,             \
        globus_xsd_element_options_t        options)                    \
    {                                                                   \
        globus_result_t                     result = GLOBUS_SUCCESS;    \
        GlobusFuncName(type##_list_serialize_contents);                 \
        GlobusSoapMessageDebugEnter();                                  \
                                                                        \
        result = globus_soap_message_serialize_##base_name##_list(      \
            message_handle,                                             \
            inst);                                                      \
        if(result != GLOBUS_SUCCESS)                                    \
        {                                                               \
            result = GlobusSoapMessageErrorSerializeFailed(             \
                result, "could not serialize as list", &type##_qname);  \
            goto exit;                                                  \
        }                                                               \
                                                                        \
     exit:                                                              \
                                                                        \
        GlobusSoapMessageDebugExit();                                   \
        return result;                                                  \
    }

#define GLOBUS_I_XSD_DEFINE_LIST_ATTR_SERIALIZE(type, base_name)             \
    globus_result_t                                                          \
    type##_list_serialize_attribute(                                         \
        xsd_QName *                     attr_qname,                          \
        type##_array *                  inst,                                \
        globus_soap_message_handle_t    message_handle,                      \
        globus_xsd_element_options_t    options)                             \
    {                                                                        \
        globus_result_t                 result = GLOBUS_SUCCESS;             \
        GlobusFuncName(type##_list_serialize_attribute);                     \
        GlobusSoapMessageDebugEnter();                                       \
                                                                             \
        result = globus_soap_message_serialize_##base_name##_attribute_list( \
            message_handle,                                                  \
            attr_qname,                                                      \
            inst);                                                           \
        if(result != GLOBUS_SUCCESS)                                         \
        {                                                                    \
            result = GlobusSoapMessageErrorSerializeFailed(                  \
                result, NULL, attr_qname);                                   \
            goto exit;                                                       \
        }                                                                    \
                                                                             \
    exit:                                                                    \
                                                                             \
        GlobusSoapMessageDebugExit();                                        \
        return result;                                                       \
    }

#define GLOBUS_I_XSD_DEFINE_TYPE_DESERIALIZE(type)			 \
    globus_result_t							 \
    type##_deserialize(							 \
        xsd_QName *                         element_qname,		 \
        type *                              instance,			 \
        globus_soap_message_handle_t        message_handle,		 \
        globus_xsd_element_options_t        options)			 \
    {									 \
        globus_result_t                     result = GLOBUS_SUCCESS;	 \
        GlobusFuncName(type##_deserialize);				 \
        GlobusSoapMessageDebugEnter();					 \
									 \
        if(!(options & GLOBUS_XSD_ELEMENT_CONTENTS_ONLY))		 \
        {								 \
            result = globus_soap_message_deserialize_element(		 \
                message_handle, element_qname);				 \
            if(result != GLOBUS_SUCCESS)				 \
            {								 \
                goto error_exit;					 \
            }								 \
									 \
            result =							 \
            globus_soap_message_deserialize_element_begin_close(	 \
                message_handle);					 \
            if(result != GLOBUS_SUCCESS)				 \
            {								 \
                goto error_exit;					 \
            }								 \
        }								 \
									 \
        result = type##_deserialize_contents(				 \
            NULL,							 \
            instance,							 \
            message_handle,						 \
            0);								 \
        if(result != GLOBUS_SUCCESS)					 \
        {								 \
            goto error_exit;						 \
        }								 \
									 \
        if(!(options & GLOBUS_XSD_ELEMENT_CONTENTS_ONLY))		 \
        {								 \
            result =							 \
            globus_soap_message_deserialize_element_end(		 \
                message_handle,                                          \
                element_qname);					         \
            if(result != GLOBUS_SUCCESS)				 \
            {								 \
                goto error_exit;					 \
            }								 \
        }								 \
									 \
        goto exit;							 \
									 \
     error_exit:							 \
									 \
        if(!GlobusSoapMessageStatusCheck(result))			 \
        {								 \
	    result = GlobusSoapMessageErrorDeserializeFailed(		 \
              result, element_qname);					 \
        }								 \
     exit:								 \
									 \
        GlobusSoapMessageDebugExit();					 \
        return result;							 \
    }									 \
    globus_result_t							 \
    type##_deserialize_contents_wrapper(				 \
        xsd_QName *                         element_qname,		 \
        void **                             instance,			 \
        globus_soap_message_handle_t        message_handle,		 \
        globus_xsd_element_options_t        options)			 \
    {									 \
        globus_result_t                     result;			 \
        GlobusFuncName(type##_deserialize_contents_wrapper);		 \
        GlobusSoapMessageDebugEnter();					 \
									 \
        if(!*instance)							 \
        {								 \
            result = type##_init((type **)instance);			 \
            if(result != GLOBUS_SUCCESS)				 \
            {								 \
                goto error;						 \
            }								 \
        }								 \
									 \
        result = type##_deserialize_contents(				 \
            element_qname, *(type **)instance, message_handle, options); \
    error:								 \
        GlobusSoapMessageDebugExit();					 \
        return result;							 \
    }
        
#define GLOBUS_I_XSD_DEFINE_TYPE_DESERIALIZE_POINTER(type)		\
    globus_result_t							\
    type##_deserialize_pointer(						\
    xsd_QName *                         element_qname,			\
    type **                             ip,				\
    globus_soap_message_handle_t        message_handle,			\
    globus_xsd_element_options_t        options)			\
    {									\
        globus_result_t                 result = GLOBUS_SUCCESS;	\
        GlobusFuncName(type##_deserialize_pointer);			\
        GlobusSoapMessageDebugEnter();					\
									\
        *ip = NULL;							\
									\
        result = globus_soap_message_deserialize_element(		\
            message_handle, element_qname);				\
        if(GlobusSoapMessageStatusElementNotFoundCheck(result) ||	\
           GlobusSoapMessageStatusFailedElementCheck(result) ||		\
           GlobusSoapMessageStatusFailedWithTextCheck(result))		\
        {								\
            result = GLOBUS_SUCCESS;					\
            goto exit;							\
        }								\
									\
        if(result != GLOBUS_SUCCESS)					\
        {								\
            goto error_exit;						\
        }								\
									\
        result = type##_init(ip);					\
									\
        if(result != GLOBUS_SUCCESS)					\
        {								\
            goto error_exit;						\
        }								\
									\
        result = type##_deserialize_contents(				\
            NULL, *ip, message_handle, 0);				\
        if(result != GLOBUS_SUCCESS)					\
        {								\
            goto error_exit;						\
        }								\
									\
        result = globus_soap_message_deserialize_element_end(		\
            message_handle,						\
            element_qname);						\
        if(result != GLOBUS_SUCCESS)					\
        {								\
            goto error_exit;						\
        }								\
									\
        goto exit;							\
									\
     error_exit:							\
									\
        if(!GlobusSoapMessageStatusCheck(result))			\
        {								\
            result = GlobusSoapMessageErrorDeserializeFailed(		\
                result, element_qname);					\
        }								\
									\
     exit:								\
									\
        if(result != GLOBUS_SUCCESS && *ip)				\
        {								\
            type##_destroy(*ip);					\
            *ip = NULL;							\
        }								\
									\
        GlobusSoapMessageDebugExit();					\
        return result;							\
    }									\
    globus_result_t							\
    type##_deserialize_pointer_wrapper(					\
    xsd_QName *                         element_qname,			\
    void **                             ip,				\
    globus_soap_message_handle_t        message_handle,			\
    globus_xsd_element_options_t        options)			\
    {									\
        return type##_deserialize_pointer(				\
            element_qname, (type **) ip, message_handle, options);	\
    }
    
#define GLOBUS_I_XSD_DEFINE_ATTR_DESERIALIZE(type, base_name)           \
    globus_result_t                                                     \
    type##_deserialize_attribute(                                       \
    xsd_QName *                         element_qname,                  \
    type *                              ip,                             \
    globus_soap_message_handle_t        message_handle,                 \
    globus_xsd_element_options_t        options)                        \
    {                                                                   \
        globus_result_t                 result = GLOBUS_SUCCESS;        \
        GlobusFuncName(type##_deserialize_attribute);                   \
        GlobusSoapMessageDebugEnter();                                  \
                                                                        \
        result =                                                        \
        globus_soap_message_deserialize_##base_name##_attribute(        \
            message_handle, element_qname, ip);                         \
                                                                        \
        if(result != GLOBUS_SUCCESS)                                    \
        {                                                               \
            goto error_exit;                                            \
        }                                                               \
                                                                        \
        goto exit;                                                      \
                                                                        \
     error_exit:                                                        \
                                                                        \
        GlobusSoapMessageErrorDeserializeFailed(result, element_qname); \
                                                                        \
     exit:                                                              \
                                                                        \
        GlobusSoapMessageDebugExit();                                   \
        return result;                                                  \
    }

#define GLOBUS_I_XSD_DEFINE_LIST_ATTR_DESERIALIZE(type, base_name)      \
    globus_result_t                                                     \
    type##_list_deserialize_attribute(                                  \
        xsd_QName *                     attr_qname,                     \
        type##_array *                  ip,                             \
        globus_soap_message_handle_t    message_handle,                 \
        globus_xsd_element_options_t    options)                        \
    {                                                                   \
        globus_result_t                 result = GLOBUS_SUCCESS;        \
        GlobusFuncName(type##_list_deserialize_attribute);              \
        GlobusSoapMessageDebugEnter();                                  \
                                                                        \
        result =                                                        \
        globus_soap_message_deserialize_##base_name##_attribute_list(   \
            message_handle, attr_qname, ip);                            \
        if(result != GLOBUS_SUCCESS)                                    \
        {                                                               \
            result = GlobusSoapMessageErrorDeserializeFailed(           \
                result, attr_qname);                                    \
        }                                                               \
                                                                        \
        return result;                                                  \
    }

#define GLOBUS_I_XSD_DEFINE_LIST_DESERIALIZE(type, base_name)		\
    globus_result_t							\
    type##_list_deserialize(						\
        xsd_QName *                         element_qname,		\
        type##_array *                      ip,				\
        globus_soap_message_handle_t        message_handle,		\
        globus_xsd_element_options_t        options)			\
    {									\
        globus_result_t                 result = GLOBUS_SUCCESS;	\
        GlobusFuncName(type##_list_deserialize);			\
        GlobusSoapMessageDebugEnter();					\
									\
        if(!(options & GLOBUS_XSD_ELEMENT_CONTENTS_ONLY))		\
        {								\
            result = globus_soap_message_deserialize_element(		\
                message_handle, element_qname);				\
            if(result != GLOBUS_SUCCESS)				\
            {								\
                goto error_exit;					\
            }								\
									\
            result =							\
            globus_soap_message_deserialize_element_begin_close(	\
                message_handle);					\
            if(result != GLOBUS_SUCCESS)				\
            {								\
                goto error_exit;					\
            }								\
        }								\
									\
        result =							\
        type##_list_deserialize_contents(				\
            ip, message_handle, options);				\
        if(result != GLOBUS_SUCCESS)					\
        {								\
            goto error_exit;						\
        }								\
									\
        if(!(options & GLOBUS_XSD_ELEMENT_CONTENTS_ONLY))		\
        {								\
            result =							\
            globus_soap_message_deserialize_element_end(		\
                message_handle,                                         \
                element_qname);                                         \
            if(result != GLOBUS_SUCCESS)				\
            {								\
                goto error_exit;					\
            }								\
        }								\
									\
        goto exit;							\
									\
     error_exit:							\
									\
        if(!GlobusSoapMessageStatusCheck(result))			\
        {								\
            result = GlobusSoapMessageErrorDeserializeFailed(		\
                result, element_qname);					\
        }								\
									\
     exit:								\
									\
        GlobusSoapMessageDebugExit();					\
        return result;							\
    }

#define GLOBUS_I_XSD_DEFINE_LIST_DESERIALIZE_CONTENTS(type, base_name)  \
    globus_result_t                                                     \
    type##_list_deserialize_contents(                                   \
        type##_array *                      ip,                         \
        globus_soap_message_handle_t        message_handle,             \
        globus_xsd_element_options_t        options)                    \
    {                                                                   \
        globus_result_t                 result = GLOBUS_SUCCESS;        \
        GlobusFuncName(type##_list_deserialize_contents);               \
        GlobusSoapMessageDebugEnter();                                  \
                                                                        \
        result =                                                        \
        globus_soap_message_deserialize_##base_name##_list(             \
            message_handle, ip);                                        \
        if(result != GLOBUS_SUCCESS)                                    \
        {                                                               \
            goto exit;                                                  \
        }                                                               \
                                                                        \
    exit:                                                               \
                                                                        \
        result = GlobusSoapMessageErrorDeserializeFailed(               \
            result, &type##_qname);                                     \
        return result;                                                  \
    }
         

#define GLOBUS_I_XSD_DEFINE_ATTR_DESERIALIZE_POINTER(type, base_name)   \
    globus_result_t                                                     \
    type##_deserialize_attribute_pointer(                               \
    xsd_QName *                         element_qname,                  \
    type **                             ip,                             \
    globus_soap_message_handle_t        message_handle,                 \
    globus_xsd_element_options_t        options)                        \
    {                                                                   \
        type *                  instance = NULL;                        \
        globus_result_t                 result = GLOBUS_SUCCESS;        \
        GlobusFuncName(type##_deserialize_attribute_pointer);           \
        GlobusSoapMessageDebugEnter();                                  \
                                                                        \
        *ip = NULL;                                                     \
                                                                        \
        result = type##_init(&instance);                                \
                                                                        \
        if(result != GLOBUS_SUCCESS)                                    \
        {                                                               \
            goto error_exit;                                            \
        }                                                               \
                                                                        \
        result =                                                        \
        globus_soap_message_deserialize_##base_name##_attribute(        \
            message_handle, element_qname, instance);                   \
        if(GlobusSoapMessageStatusAttributeNotFoundCheck(result))       \
        {                                                               \
            result = GLOBUS_SUCCESS;                                    \
            type##_destroy(instance);                                   \
            instance = NULL;                                            \
            goto exit;                                                  \
        }                                                               \
                                                                        \
        if(result != GLOBUS_SUCCESS)                                    \
        {                                                               \
            goto error_exit;                                            \
        }                                                               \
                                                                        \
        *ip = instance;                                                 \
                                                                        \
        goto exit;                                                      \
                                                                        \
     error_exit:                                                        \
                                                                        \
        GlobusSoapMessageErrorDeserializeFailed(result, element_qname); \
                                                                        \
     exit:                                                              \
                                                                        \
        if(result != GLOBUS_SUCCESS && instance)                        \
        {                                                               \
            type##_destroy(instance);                                   \
            *ip = NULL;                                                 \
        }                                                               \
                                                                        \
        GlobusSoapMessageDebugExit();                                   \
        return result;                                                  \
    }

#define GLOBUS_I_XSD_DEFINE_TYPE_SERIALIZE_ARRAY(type)                  \
    globus_result_t                                                     \
    type##_array_serialize(                                             \
        xsd_QName *                     element,                        \
        type##_array *                  array,                          \
        globus_soap_message_handle_t    message,                        \
        globus_xsd_element_options_t    options)                        \
    {                                                                   \
        globus_result_t                     result = GLOBUS_SUCCESS;    \
        int                                 i = 0;                      \
        GlobusFuncName(type##_array_serialize);                         \
        GlobusSoapMessageDebugEnter();                                  \
                                                                        \
        for(; i < array->length; ++i)                                   \
        {                                                               \
            result = type##_serialize(                                  \
                element,                                                \
                &array->elements[i],                                    \
                message,                                                \
                options);                                               \
            if(result != GLOBUS_SUCCESS)                                \
            {                                                           \
                goto exit;                                              \
            }                                                           \
        }                                                               \
                                                                        \
      exit:                                                             \
                                                                        \
        GlobusSoapMessageDebugExit();                                   \
        return result;                                                  \
    }                                                                   \
    globus_result_t                                                     \
    type##_array_serialize_wrapper(                                     \
        xsd_QName *                     element,                        \
        void *                          instance,                       \
        globus_soap_message_handle_t    message,                        \
        globus_xsd_element_options_t    options)                        \
    {                                                                   \
        return type##_array_serialize(                                  \
            element, (type##_array *) instance, message, options);      \
    }

#define GLOBUS_I_XSD_DEFINE_TYPE_DESERIALIZE_ARRAY(type)                \
    globus_result_t                                                     \
    type##_array_deserialize(                                           \
        xsd_QName *                         element_qname,              \
        type##_array *                      array,                      \
        globus_soap_message_handle_t        message,                    \
        globus_xsd_element_options_t        options)                    \
    {                                                                   \
        globus_result_t                     result = GLOBUS_SUCCESS;    \
        int                                 i = 0;                      \
        GlobusFuncName(type##_array_deserialize);                       \
        GlobusSoapMessageDebugEnter();                                  \
                                                                        \
        while(result == GLOBUS_SUCCESS)                                 \
        {                                                               \
            if(array->length > i)                                       \
            {                                                           \
                result = type##_deserialize(                            \
                    element_qname,                                      \
                    &array->elements[i],                                \
                    message,                                            \
                    options);                                           \
                if(result != GLOBUS_SUCCESS)                            \
                {                                                       \
                    if(GlobusSoapMessageStatusFailedElementCheck(       \
                           result) ||                                   \
                       GlobusSoapMessageStatusElementNotFoundCheck(     \
                           result) ||                                   \
                       GlobusSoapMessageStatusFailedWithTextCheck(      \
                           result))                                     \
                    {                                                   \
                        result = GLOBUS_SUCCESS;                        \
                        break;                                          \
                    }                                                   \
                                                                        \
                    result = GlobusSoapMessageErrorDeserializeFailed(   \
                        result, element_qname);                         \
                    goto exit;                                          \
                }                                                       \
            }                                                           \
            else                                                        \
            {                                                           \
                type                    next;                           \
                type *                  tmp;                            \
                memset(&next, 0, sizeof(type));                         \
                                                                        \
                result = type##_deserialize(                            \
                    element_qname,                                      \
                    &next,                                              \
                    message,                                            \
                    options);                                           \
                if(result != GLOBUS_SUCCESS)                            \
                {                                                       \
                    if(GlobusSoapMessageStatusFailedElementCheck(       \
                           result) ||                                   \
                       GlobusSoapMessageStatusElementNotFoundCheck(     \
                           result) ||                                   \
                       GlobusSoapMessageStatusFailedWithTextCheck(      \
                           result))                                     \
                    {                                                   \
                        result = GLOBUS_SUCCESS;                        \
                        break;                                          \
                    }                                                   \
                                                                        \
                    result = GlobusSoapMessageErrorDeserializeFailed(   \
                        result, element_qname);                         \
                    goto exit;                                          \
                }                                                       \
                                                                        \
                tmp = type##_array_push(array);                         \
                globus_assert(tmp);                                     \
                *tmp = next;                                            \
            }                                                           \
                                                                        \
            ++i;                                                        \
        }                                                               \
                                                                        \
      exit:                                                             \
                                                                        \
        if(result != GLOBUS_SUCCESS)                                    \
        {                                                               \
            int id = 0;                                                 \
            for(; id < i; ++id)                                         \
            {                                                           \
                type##_destroy_contents(&array->elements[id]);          \
            }                                                           \
            free(array->elements);                                      \
            array->length = 0;                                          \
            array->elements = NULL;                                     \
        }                                                               \
                                                                        \
        GlobusSoapMessageDebugExit();                                   \
        return result;                                                  \
    }                                                                   \
    globus_result_t                                                     \
    type##_array_deserialize_wrapper(                                   \
        xsd_QName *                         element_qname,              \
        void **                             inst,                       \
        globus_soap_message_handle_t        message,                    \
        globus_xsd_element_options_t        options)                    \
    {                                                                   \
        if(!*inst)                                                      \
        {                                                               \
            globus_result_t                 result;                     \
            result = type##_array_init((type##_array **)inst);          \
            if(result != GLOBUS_SUCCESS)                                \
            {                                                           \
                return result;                                          \
            }                                                           \
        }                                                               \
                                                                        \
        return type##_array_deserialize(                                \
            element_qname, *(type##_array **)inst, message, options);   \
    }
    
#define GLOBUS_I_XSD_DEFINE_TYPE_INIT_ARRAY(type)                       \
    globus_result_t                                                     \
    type##_array_init(                                                  \
        type##_array **                      arr)                       \
    {                                                                   \
        globus_result_t                 result;                         \
        type##_array *              array;                              \
        GlobusFuncName(type##_array_init);                              \
        GlobusSoapMessageDebugEnter();                                  \
                                                                        \
        array = (type##_array *)malloc(                                 \
            sizeof(type##_array));                                      \
        if(!array)                                                      \
        {                                                               \
            result = GlobusSoapMessageErrorOutOfMemory;                 \
            goto exit;                                                  \
        }                                                               \
                                                                        \
        result = type##_array_init_contents(array);                     \
        if(result != GLOBUS_SUCCESS)                                    \
        {                                                               \
            globus_free(array);                                         \
            array = NULL;                                               \
        }                                                               \
                                                                        \
        *arr = array;                                                   \
                                                                        \
      exit:                                                             \
        GlobusSoapMessageDebugExit();                                   \
        return result;                                                  \
    }                                                                   \
    globus_result_t                                                     \
    type##_array_init_wrapper(                                          \
        void **                             arr)                        \
    {                                                                   \
        return type##_array_init((type##_array **) arr);                \
    }                                                                   \
    globus_result_t                                                     \
    type##_array_init_contents(                                         \
        type##_array *                      array)                      \
    {                                                                   \
        GlobusFuncName(type##_array_init_contents);                     \
        GlobusSoapMessageDebugEnter();                                  \
                                                                        \
        memset(array, 0, sizeof(type##_array));                         \
                                                                        \
        GlobusSoapMessageDebugExit();                                   \
        return GLOBUS_SUCCESS;                                          \
    }                                                                   \
    globus_result_t                                                     \
    type##_array_init_contents_wrapper(                                 \
        void *                      array)                              \
    {                                                                   \
        return type##_array_init_contents((type##_array *) array);      \
    }
    
#define GLOBUS_I_XSD_DEFINE_TYPE_DESTROY_ARRAY(type)            \
    void                                                        \
    type##_array_destroy(                                       \
        type##_array *                              array)      \
    {                                                           \
        GlobusFuncName(type##_array_destroy);                   \
        GlobusSoapMessageDebugEnter();                          \
                                                                \
        type##_array_destroy_contents(array);                   \
        globus_free(array);                                     \
                                                                \
        GlobusSoapMessageDebugExit();                           \
    }                                                           \
    void                                                        \
    type##_array_destroy_wrapper(                               \
        void *                              arr)                \
    {                                                           \
        type##_array_destroy((type##_array *) arr);             \
    }                                                           \
    void                                                        \
    type##_array_destroy_contents(                              \
        type##_array *                              array)      \
    {                                                           \
        int                                 i = 0;              \
        GlobusFuncName(type##_array_destroy_contents);          \
        GlobusSoapMessageDebugEnter();                          \
                                                                \
        for(; i < array->length; ++i)                           \
        {                                                       \
            type##_destroy_contents(&array->elements[i]);       \
        }                                                       \
                                                                \
        if(array->elements)                                     \
        {                                                       \
            globus_free(array->elements);                       \
            array->elements = NULL;                             \
        }                                                       \
        array->length = 0;                                      \
                                                                \
        GlobusSoapMessageDebugExit();                           \
    }                                                           \
    void                                                        \
    type##_array_destroy_contents_wrapper(                      \
        void *                              array)              \
    {                                                           \
        type##_array_destroy_contents((type##_array *) array);  \
    }
    
#define GLOBUS_I_XSD_DEFINE_TYPE_COPY_ARRAY(type)                       \
    globus_result_t                                                     \
    type##_array_copy(                                                  \
        type##_array **                     dest_arr,                   \
        const type##_array *                src_arr)                    \
    {                                                                   \
        type##_array *              array;                              \
        globus_result_t             result;                             \
        GlobusFuncName(type##_array_copy);                              \
        GlobusSoapMessageDebugEnter();                                  \
                                                                        \
        array = (type##_array *)malloc(                                 \
            sizeof(type##_array));                                      \
        if(!array)                                                      \
        {                                                               \
            result = GlobusSoapMessageErrorOutOfMemory;                 \
            GlobusSoapMessageDebugExit();                               \
            return result;                                              \
        }                                                               \
                                                                        \
        result = type##_array_copy_contents(array, src_arr);            \
        if(result != GLOBUS_SUCCESS)                                    \
        {                                                               \
            globus_free(array);                                         \
            GlobusSoapMessageDebugExit();                               \
            return result;                                              \
        }                                                               \
                                                                        \
        *dest_arr = array;                                              \
                                                                        \
        GlobusSoapMessageDebugExit();                                   \
        return GLOBUS_SUCCESS;                                          \
    }                                                                   \
    globus_result_t                                                     \
    type##_array_copy_wrapper(                                          \
        void **                             dest_arr,                   \
        void *                              src_arr)                    \
    {                                                                   \
        return type##_array_copy(                                       \
            (type##_array **) dest_arr, (type##_array *) src_arr);      \
    }                                                                   \
                                                                        \
    globus_result_t                                                     \
    type##_array_copy_contents(                                         \
        type##_array *                      dest_arr,                   \
        const type##_array *                src_arr)                    \
    {                                                                   \
        int                             i;                              \
        globus_result_t                 result = GLOBUS_SUCCESS;        \
        GlobusFuncName(type##_array_copy_contents);                     \
        GlobusSoapMessageDebugEnter();                                  \
                                                                        \
        dest_arr->length = src_arr->length;                             \
        if(dest_arr->length > 0)                                        \
        {                                                               \
            dest_arr->elements = (type *)malloc(                        \
                sizeof(type) * dest_arr->length);                       \
            if(!dest_arr->elements)                                     \
            {                                                           \
                 result = GlobusSoapMessageErrorOutOfMemory;            \
                 goto exit;                                             \
            }                                                           \
        }                                                               \
        else                                                            \
        {                                                               \
            dest_arr->elements = NULL;                                  \
        }                                                               \
                                                                        \
        for(i = 0; i < dest_arr->length; i++)                           \
        {                                                               \
            result = type##_copy_contents(                              \
                &dest_arr->elements[i], &src_arr->elements[i]);         \
            if(result != GLOBUS_SUCCESS)                                \
            {                                                           \
                while(i--)                                              \
                {                                                       \
                    type##_destroy_contents(&dest_arr->elements[i]);    \
                }                                                       \
                                                                        \
                globus_free(dest_arr->elements);                        \
                dest_arr->elements = NULL;                              \
                dest_arr->length = 0;                                   \
                break;                                                  \
            }                                                           \
        }                                                               \
                                                                        \
      exit:                                                             \
        GlobusSoapMessageDebugExit();                                   \
        return result;                                                  \
    }                                                                   \
    globus_result_t                                                     \
    type##_array_copy_contents_wrapper(                                 \
        void *                             dest_arr,                    \
        void *                             src_arr)                     \
    {                                                                   \
        return type##_array_copy_contents(                              \
            (type##_array *) dest_arr, (const type##_array *) src_arr); \
    }

#define GLOBUS_I_XSD_DEFINE_TYPE_ARRAY_PUSH(type)                       \
    type *                                                              \
    type##_array_push(                                                  \
        type##_array *              array)                              \
    {                                                                   \
        GlobusFuncName(type##_array_push);                              \
        type * tmp;                                                     \
        GlobusSoapMessageDebugEnter();                                  \
                                                                        \
        tmp = realloc(array->elements,                                  \
                                  sizeof(type) * (array->length + 1));  \
        if (tmp == NULL) return NULL;                                   \
        array->elements = tmp;                                          \
        memset(&array->elements[array->length], 0, sizeof(type));       \
        array->length++;                                                \
                                                                        \
        GlobusSoapMessageDebugExit();                                   \
        return (&array->elements[array->length - 1]);                   \
    }                                                                   \
    void *                                                              \
    type##_array_push_wrapper(                                          \
        void *                         array)                           \
    {                                                                   \
        return type##_array_push((type##_array *) array);               \
    }

#define GLOBUS_I_XSD_DEFINE_ATTR_FUNCTIONS(type, base_name)        \
    GLOBUS_I_XSD_DEFINE_ATTR_SERIALIZE(type, base_name);           \
    GLOBUS_I_XSD_DEFINE_ATTR_DESERIALIZE(type, base_name);         \
    GLOBUS_I_XSD_DEFINE_ATTR_DESERIALIZE_POINTER(type, base_name)

#define GLOBUS_I_XSD_DEFINE_LIST_FUNCTIONS(type, base_name)             \
    GLOBUS_I_XSD_DEFINE_LIST_SERIALIZE(type, base_name);                \
    GLOBUS_I_XSD_DEFINE_LIST_SERIALIZE_CONTENTS(type, base_name);       \
    GLOBUS_I_XSD_DEFINE_LIST_DESERIALIZE(type, base_name);              \
    GLOBUS_I_XSD_DEFINE_LIST_DESERIALIZE_CONTENTS(type, base_name)

#define GLOBUS_I_XSD_DEFINE_LIST_ATTR_FUNCTIONS(type, base_name)        \
    GLOBUS_I_XSD_DEFINE_LIST_ATTR_SERIALIZE(type, base_name);           \
    GLOBUS_I_XSD_DEFINE_LIST_ATTR_DESERIALIZE(type, base_name)

#define GLOBUS_I_XSD_DEFINE_TYPE_FUNCTIONS(type)        \
    GLOBUS_I_XSD_DEFINE_TYPE_INIT(type);                \
    GLOBUS_I_XSD_DEFINE_TYPE_DESTROY(type);             \
    GLOBUS_I_XSD_DEFINE_TYPE_COPY(type);                \
    GLOBUS_I_XSD_DEFINE_TYPE_SERIALIZE(type);           \
    GLOBUS_I_XSD_DEFINE_TYPE_DESERIALIZE(type);         \
    GLOBUS_I_XSD_DEFINE_TYPE_DESERIALIZE_POINTER(type); \
    GLOBUS_I_XSD_DEFINE_TYPE_INIT_ARRAY(type);          \
    GLOBUS_I_XSD_DEFINE_TYPE_DESTROY_ARRAY(type);       \
    GLOBUS_I_XSD_DEFINE_TYPE_COPY_ARRAY(type);          \
    GLOBUS_I_XSD_DEFINE_TYPE_SERIALIZE_ARRAY(type);     \
    GLOBUS_I_XSD_DEFINE_TYPE_DESERIALIZE_ARRAY(type);   \
    GLOBUS_I_XSD_DEFINE_TYPE_ARRAY_PUSH(type)

#define GLOBUS_I_XSD_DEFINE_INIT_CONTENTS_GENERIC(type) \
    globus_result_t                                     \
    type##_init_contents(                               \
        type *                  instance)               \
    {                                                   \
        GlobusFuncName(type##_init_contents);           \
        GlobusSoapMessageDebugEnter();                  \
                                                        \
        memset(instance, 0, sizeof(type));              \
                                                        \
        GlobusSoapMessageDebugExit();                   \
        return GLOBUS_SUCCESS;                          \
    }

#define GLOBUS_I_XSD_DEFINE_DESTROY_CONTENTS_GENERIC(type)                  \
    void                                                                    \
    type##_destroy_contents(                                                \
        type *                          instance)                           \
    {                                                                       \
        GlobusFuncName(ype##_destroy_contents);                             \
        GlobusSoapMessageDebugEnter();                                      \
                                                                            \
        GlobusSoapMessageDebugExit();                                       \
    }

#define GLOBUS_I_XSD_DEFINE_COPY_CONTENTS_GENERIC(type)                     \
    globus_result_t                                                         \
    type##_copy_contents(                                                   \
        type *                          dest,                               \
        const type *                    src)                                \
    {                                                                       \
        GlobusFuncName(type##_copy_contents);                               \
        GlobusSoapMessageDebugEnter();                                      \
        if(dest && src)                                                     \
        {                                                                   \
            *dest = *src;                                                   \
        }                                                                   \
        GlobusSoapMessageDebugExit();                                       \
                                                                            \
        return GLOBUS_SUCCESS;                                              \
    }

EXTERN_C_END

#endif /* GLOBUS_I_XSD_H */
