/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */


#include "globus_i_operation_provider.h"
#include "globus_soap_message.h"

globus_extension_registry_t             globus_operation_provider_registry =
{
    NULL,
    GLOBUS_FALSE,
    GLOBUS_FALSE
};

GlobusDebugDefine(GLOBUS_OPERATION_PROVIDER);

static int
globus_l_operation_provider_activate(void);

static int
globus_l_operation_provider_deactivate(void);

static
void
globus_l_operation_entry_destroy(
    void *                              value);

#include "version.h"

globus_module_descriptor_t              globus_l_operation_provider_module =
{
    "globus_operation_provider",
    globus_l_operation_provider_activate,
    globus_l_operation_provider_deactivate,
    GLOBUS_NULL,
    GLOBUS_NULL,
    &local_version
};

typedef struct globus_l_operation_entry_s
{
    globus_operation_provider_descriptor_t * descriptor;
    globus_extension_handle_t                extension_handle;
} globus_l_operation_entry_t;

static int
globus_l_operation_provider_activate(void)
{
    int rc;

    rc = globus_module_activate(GLOBUS_COMMON_MODULE);
    if(rc != GLOBUS_SUCCESS)
    {
        return rc;
    }
    
    GlobusDebugInit(GLOBUS_OPERATION_PROVIDER, TRACE ERROR);

    rc = globus_module_activate(GLOBUS_SOAP_MESSAGE_MODULE);
    if(rc != GLOBUS_SUCCESS)
    {
        globus_module_deactivate(GLOBUS_COMMON_MODULE);
        return rc;
    }
    
    rc = globus_extension_registry_set_hashing(
        GLOBUS_OPERATION_PROVIDER_REGISTRY,
        xsd_QName_hash,
        xsd_QName_keyeq);
    if(rc != GLOBUS_SUCCESS)
    {
        globus_module_deactivate(GLOBUS_COMMON_MODULE);
        return rc;
    }

    return 0;
}

static int
globus_l_operation_provider_deactivate(void)
{
    GlobusFuncName(globus_l_operation_provider_deactivate);
    GlobusOperationProviderDebugEnter();

    globus_module_deactivate(GLOBUS_SOAP_MESSAGE_MODULE);

    GlobusOperationProviderDebugExit();

    GlobusDebugDestroy(GLOBUS_OPERATION_PROVIDER);
    globus_module_deactivate(GLOBUS_COMMON_MODULE);

    return GLOBUS_SUCCESS;
}

globus_result_t
globus_operation_table_init(
    globus_operation_table_t *          table,
    const char *                        Namespace)
{
    int                                 rc = 0;
    globus_result_t                     result = GLOBUS_SUCCESS;
    globus_operation_table_t            newtable;
    GlobusFuncName(globus_operation_table_init);
    GlobusOperationProviderDebugEnter();
    
    newtable = globus_malloc(sizeof(globus_l_operation_table_t));
    if(!newtable)
    {
        result = GlobusOperationProviderErrorOutOfMemory;
        goto exit;
    }
    memset(newtable, 0, sizeof(globus_l_operation_table_t));
    
    newtable->Namespace = globus_libc_strdup(Namespace);
    rc = globus_hashtable_init(&newtable->table,
                            10,
                            globus_hashtable_string_hash,
                            globus_hashtable_string_keyeq);
    if(rc != GLOBUS_SUCCESS)
    {
        result = GlobusOperationProviderErrorFailedInit((globus_result_t)rc);
        goto exit;
    }

    *table = newtable;

 exit:

    GlobusOperationProviderDebugExit();
    return result;
}

globus_result_t
globus_operation_table_get_operation(
    globus_operation_table_t            table,
    const char *                        operation_id,
    void **                             init_operation,
    void **                             operation)
{
    globus_l_operation_entry_t *        op_entry;
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_operation_table_call);
    GlobusOperationProviderDebugEnter();

    op_entry = (globus_l_operation_entry_t *) globus_hashtable_lookup(
        &table->table, (void *)operation_id);
    if(!op_entry)
    {
        result = GlobusOperationProviderErrorOpNotFound(operation_id);
        goto exit;
    }

    *init_operation = op_entry->descriptor->init_operation_func;
    *operation = op_entry->descriptor->operation_func;

 exit:

    GlobusOperationProviderDebugExit();
    return result;
}

globus_result_t
globus_l_operation_table_add_provider(
    globus_operation_table_t                 table,
    xsd_QName *                              operation_name,
    globus_operation_provider_descriptor_t * op_desc,
    globus_extension_handle_t                ext_handle)
{
    int                                 rc = 0;
    globus_result_t                     result = GLOBUS_SUCCESS;
    globus_l_operation_entry_t *        op_entry = NULL;
    GlobusFuncName(globus_l_operation_table_add_default_provider);
    GlobusOperationProviderDebugEnter();

    /* lookup operation name to see if we already have one of those */
    op_entry = globus_hashtable_lookup(&table->table, (void *)op_desc->id);
    if(op_entry)
    {
        /* remove it so that we can store the new value */
        op_entry = globus_hashtable_remove(&table->table, (void *)op_desc->id);
        if(op_entry->extension_handle)
        {
            globus_extension_release(op_entry->extension_handle);
        }

        free(op_entry);
    }

    op_entry = globus_malloc(sizeof(globus_l_operation_entry_t));
    if(!op_entry)
    {
        result = GlobusOperationProviderErrorOutOfMemory;
        goto exit;
    }
    memset(op_entry, 0, sizeof(globus_l_operation_entry_t));
    
    op_entry->descriptor = op_desc;
    op_entry->extension_handle = ext_handle;

    rc = globus_hashtable_insert(&table->table, 
                                 (void *)op_desc->id, op_entry);
    if(rc != GLOBUS_SUCCESS)
    {
        result = GlobusOperationProviderErrorFailedTableInsert(operation_name);
        goto exit;
    }
    
 exit:

    GlobusOperationProviderDebugExit();
    return result;
}

globus_result_t
globus_operation_table_add_default_provider(
    globus_operation_table_t            table,
    xsd_QName *                         operation_name,
    globus_operation_provider_descriptor_t * op_desc)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_operation_table_add_default_provider);
    GlobusOperationProviderDebugEnter();

    globus_assert(op_desc);

    result = globus_l_operation_table_add_provider(
        table, operation_name, op_desc, NULL);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusOperationProviderErrorFailedTableInsert(operation_name);
        goto exit;
    }

 exit:
        
    GlobusOperationProviderDebugExit();
    return result;
}

globus_result_t
globus_operation_table_add_provider(
    globus_operation_table_t            table,
    xsd_QName *                         operation_name)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    globus_extension_handle_t           ext_handle;
    globus_operation_provider_descriptor_t * op_desc;
    GlobusFuncName(globus_operation_table_add_provider);
    GlobusOperationProviderDebugEnter();

    op_desc = globus_extension_lookup(&ext_handle, 
                                      GLOBUS_OPERATION_PROVIDER_REGISTRY,
                                      operation_name);
    if(!op_desc)
    {
        result = GlobusOperationProviderErrorRegistryLookupFailed(
            operation_name);
        goto exit;
    }

    result = globus_l_operation_table_add_provider(
        table, operation_name, op_desc, ext_handle);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusOperationProviderErrorFailedTableInsert(operation_name);
        goto release_extension;
    }

    goto exit;

 release_extension:

    globus_extension_release(ext_handle);

 exit:

    GlobusOperationProviderDebugExit();
    return result;
}

static
void
globus_l_operation_entry_destroy(
    void *                              value)
{
    globus_l_operation_entry_t *  entry;
    GlobusFuncName(globus_l_operation_entry_destroy);
    GlobusOperationProviderDebugEnter();

    entry = (globus_l_operation_entry_t *) value;

    globus_extension_registry_remove(
        GLOBUS_OPERATION_PROVIDER_REGISTRY,
        entry->descriptor->name);

    if(entry->extension_handle)
    {
        globus_extension_release(entry->extension_handle);
    }

    free(entry);

    GlobusOperationProviderDebugExit();
}

void
globus_operation_table_destroy(
    globus_operation_table_t            table)
{
    GlobusFuncName(globus_operation_table_destroy);
    GlobusOperationProviderDebugEnter();

    if(table)
    {
        if(table->table)
        {
            globus_hashtable_destroy_all(&table->table, 
                                         globus_l_operation_entry_destroy);
        }
        
        if(table->Namespace)
        {
            free(table->Namespace);
        }

        free(table);
    }
    
    GlobusOperationProviderDebugExit();
}

globus_result_t
globus_operation_table_init_resource(
    globus_operation_table_t            table,
    const char *                        id)
{
    globus_l_operation_entry_t *             entry;
    globus_operation_provider_descriptor_t * op;
    globus_result_t                          result = GLOBUS_SUCCESS;

    GlobusFuncName(globus_operation_table_init_resource);
    GlobusOperationProviderDebugEnter();

    entry = globus_hashtable_first(&table->table);
    while(entry)
    {
        op = entry->descriptor;
        if(op && op->init_resource)
        {
            result = op->init_resource(id);
            if(result != GLOBUS_SUCCESS)
            {
                GlobusOperationProviderErrorFailedResourceInit(
                    result, op->name);
                goto exit;
            }
        }

        entry = globus_hashtable_next(&table->table);
    }

 exit:
    GlobusOperationProviderDebugExit();
    return result;
}
