/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */


#include "globus_xml_buffer.h"
#include "globus_service_registry.h"

globus_result_t
globus_resource_id_get_as_string(
    xsd_any_array *                     reference_properties,
    char **                             id)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    globus_soap_message_handle_t        memory_message;
    globus_xml_buffer *                 buffer_handle;
    GlobusFuncName(globus_resource_id_get_as_string);
    GlobusSoapMessageDebugEnter();

    if(!reference_properties || reference_properties->length < 1)
    {
        result = GlobusSoapMessageErrorNoReferenceProperties();
        goto exit;
    }
    else if(reference_properties->length > 1)
    {
        result = GlobusSoapMessageErrorTooManyReferenceProperties(
            reference_properties->length);
        goto exit;
    }

    if(reference_properties->elements[0].any_info == 
       (&globus_xml_buffer_contents_info))
    {
        buffer_handle = 
        (globus_xml_buffer *)reference_properties->elements[0].value;

        result = globus_soap_message_handle_init_from_memory(
            &memory_message, buffer_handle->buffer, buffer_handle->length);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusSoapMessageErrorFailedResourceGet(
                result, "Failed to initalize memory message");
            goto exit;
        }

        result = xsd_string_deserialize(
            NULL,
            id,
            memory_message,
            0);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusSoapMessageErrorFailedResourceGet(
                result, "Failed deserialize of CounterKey");
            goto exit;
        }

        globus_soap_message_handle_destroy(memory_message);
        memory_message = NULL;
    }

    if(reference_properties->elements[0].any_info ==
       (&xsd_string_info))
    {
        *id = globus_libc_strdup(
            *(char **)reference_properties->elements[0].value);
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}
