/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */


#include "globus_common.h"
#include "globus_i_soap_message.h"
#include "globus_soap_message.h"
#include "globus_soap_message_markers.h"
#include "globus_soap_message_handle.h"
#include "globus_soap_message_fault.h"
#include "globus_xml_buffer.h"
#include "openssl/evp.h"
#include "globus_xio_buffer.h"
#include "math.h"

#include "libxml/xmlschemastypes.h"

#ifdef	__alpha		/* Tru64 Unix does not define these */
#ifndef	INT32_MIN
#define	INT32_MIN	INT_MIN
#endif
#ifndef	INT32_MAX
#define	INT32_MAX	INT_MAX
#endif
#ifndef	INT64_MIN
#define	INT64_MIN	LONG_MIN
#endif
#ifndef	INT64_MAX
#define	INT64_MAX	LONG_MAX
#endif
#ifndef	UINT32_MAX
#define	UINT32_MAX	UINT_MAX
#endif
#ifndef	UINT64_MAX
#define	UINT64_MAX	ULONG_MAX
#endif
#endif	/* __alpha */

#define XSI_NS "http://www.w3.org/2001/XMLSchema-instance"
#define XSI_NIL "nil"

static
char *
globus_l_soap_message_my_strtok(
    const char *                        str,
    size_t                              str_length,
    int *                               token_start_index,
    int *                               token_end_index,
    const char *                        delims)
{
    const char *                        locator;
    const char *                        start_token;
    int                                 delim_count, 
                                        ind, 
                                        i, 
                                        between_tokens, 
                                        len;
    char *                              new_token = NULL;
    ind = 0;
    i = 0;
    between_tokens = 0;
    delim_count = strlen(delims);
    locator = str;
    
    for(i = 0; i < delim_count; ++i)
    {
        if(*locator == delims[i])
        {
            between_tokens = 1;
        }
    }

    while(between_tokens && locator && ind < str_length)
    {
        for(i = 0; i < delim_count; ++i)
        {
            if(*locator == delims[i])
            {
                break;
            }
        }

        if(i == delim_count)
        {
            between_tokens = 0;
            break;
        }

        locator++;
        ind++;
    }

    start_token = locator;
    if(token_start_index)
    {
        *token_start_index = ind;
    }

    while(!between_tokens && *locator && ind < str_length)
    {
        for(i = 0; i < delim_count; ++i)
        {
            if(*locator == delims[i])
            {
                if(token_end_index)
                {
                    *token_end_index = ind;
                }
                between_tokens = 1;
                break;
            }
        }

        locator++;
        ind++;
    }

    if(ind == str_length)
    {
        *token_end_index = -1;
    }
    else
    {
        ind--;
    }

    len = ind + (str - start_token);
    if(len > 0)
    {
        new_token = malloc(len+1);
        memcpy(new_token, start_token, len + 1);
        new_token[len] = '\0';
    }

    return new_token;
}

static
int
globus_i_soap_message_split_qnames(
    const xmlChar *                     qname_string,
    globus_list_t **                    qnames)
{
    xmlChar *                           tmpstr;
    xmlChar *                           newstr;
    int                                 start_token;
    int                                 end_token;
    char *                              token;

    newstr = globus_libc_strdup(qname_string);
    tmpstr = newstr;
    token = globus_l_soap_message_my_strtok(
        tmpstr, strlen(tmpstr), &start_token, &end_token, "\n\t ");
    tmpstr += end_token;
        
    while(1)
    {
        globus_list_insert(qnames, token);
        if(end_token < 0)
        {
            break;
        }

        token = globus_l_soap_message_my_strtok(
            tmpstr, strlen(tmpstr), &start_token, &end_token, "\n\t ");
        tmpstr += end_token;
        if(!token)
        {
            break;
        }

        globus_list_insert(qnames, token);
    }

    globus_free(newstr);
    return 0;
}

void
globus_soap_message_deserialize_push_element(
    globus_soap_message_handle_t        handle)
{
    GlobusFuncName(globus_soap_message_deserialize_push_element);
    GlobusSoapMessageDebugEnter();

    handle->next_ready++;

    GlobusSoapMessageDebugExit();
}

globus_result_t
globus_i_soap_message_deserialize_read(
    globus_soap_message_handle_t        handle)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_i_soap_message_deserialize_next_element);
    GlobusSoapMessageDebugEnter();

    if(xmlTextReaderNodeType(handle->reader) != XML_READER_TYPE_NONE &&
       xmlTextReaderIsEmptyElement(handle->reader))
    {
        if(handle->next_ready > 0)
        {
            handle->next_ready--;
        }
        else
        {
            result = GLOBUS_SOAP_MESSAGE_STATUS_FAILED_ELEMENT;
        }

        goto exit;
    }

    if(handle->next_ready > 0)
    {
	handle->next_ready--;
    }
    else
    {
	if(xmlTextReaderRead(handle->reader) <= 0)
	{
	    result = GlobusSoapMessageErrorFailedNextNode(handle->result);
	    goto exit;
	}
    }

exit:
    GlobusSoapMessageDebugExit();
    return result;
}
    
globus_result_t
globus_i_soap_message_deserialize_next_element(
    globus_soap_message_handle_t        handle)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    char *                              idval = NULL;
    GlobusFuncName(globus_i_soap_message_deserialize_next_element);
    xmlReaderTypes                      type = XML_READER_TYPE_NONE;
    
    GlobusSoapMessageDebugEnter();

    if(xmlTextReaderNodeType(handle->reader) != type &&
       xmlTextReaderIsEmptyElement(handle->reader))
    {
        if(handle->next_ready > 0)
        {
            handle->next_ready--;
        }
        else
        {
            result = GLOBUS_SOAP_MESSAGE_STATUS_FAILED_ELEMENT;
        }

        goto exit;
    }

    while(type != XML_READER_TYPE_ELEMENT)
    {
        if(handle->next_ready > 0)
        {
            handle->next_ready--;
        }
        else
        {
	    if(xmlTextReaderRead(handle->reader) <= 0)
            {
                result = GlobusSoapMessageErrorFailedNextNode(handle->result);
                goto exit;
            }
        }
        
        type = xmlTextReaderNodeType(handle->reader);
        if(type == XML_READER_TYPE_END_ELEMENT)
        {
            result = GLOBUS_SOAP_MESSAGE_STATUS_FAILED_ELEMENT;
            goto exit;
        }
        else if(type == XML_READER_TYPE_TEXT)
        {
            xmlChar *                   str = NULL;
            if (handle->verbose)
            {
                str = xmlTextReaderValue(handle->reader);
            }
            result = GlobusSoapMessageErrorFailedWithText(handle->verbose, str);

            if (str)
            {
                xmlFree(str);
            }

            goto exit;
        }
    }

    if(type == XML_READER_TYPE_ELEMENT)
    {
        if(handle->callout_table)
        {
            xsd_QName               qn;
            
            qn.local = (char *)
            xmlTextReaderConstLocalName(handle->reader);
            qn.Namespace = (char *)
            xmlTextReaderConstNamespaceUri(handle->reader);
            
            result = globus_soap_message_invoke_callouts(
                handle, 
                &qn,
                GLOBUS_SOAP_MESSAGE_CALLOUT_BEGIN);
            if(result != GLOBUS_SUCCESS)
            {
                result = GlobusSoapMessageErrorFailedNextNode(result);
                goto exit;
            }
        }

        if((idval = xmlTextReaderGetAttribute(
                handle->reader, 
                "Id")) != NULL)
        {
            result = globus_soap_message_deserialize_mark(
                handle,
                idval);
            if(result != GLOBUS_SUCCESS)
            {
                result = GlobusSoapMessageErrorSettingId(
                    result);
                goto exit;
            }
	    xmlFree(idval);
        }
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_deserialize_element_end(
    globus_soap_message_handle_t        handle,
    xsd_QName *                         qname)
{
    char *                              idval = NULL;
    globus_result_t                     result = GLOBUS_SUCCESS;
    xmlReaderTypes                      type = XML_READER_TYPE_NONE;
    int                                 res = 0;
    GlobusFuncName(globus_soap_message_deserialize_element_end);
    GlobusSoapMessageDebugEnter();

    if(xmlTextReaderIsEmptyElement(handle->reader))
    {
        if (qname)
        {
            xsd_QName                   tmp;
            xmlChar *                   qname_ns;
            xmlChar *                   qname_local;

            tmp.Namespace = (char *)
                    xmlTextReaderConstNamespaceUri(handle->reader);
            tmp.local = (char *)
                    xmlTextReaderConstLocalName(handle->reader);
            qname_ns = xmlTextReaderConstString(handle->reader,
                    qname->Namespace);
            qname_local = xmlTextReaderConstString(handle->reader,
                    qname->local);
            if (qname_ns != tmp.Namespace || qname_local != tmp.local)
            {
                result = GlobusSoapMessageErrorFailedEndElement(
                        handle->verbose,
                        &tmp);
                goto exit;
            }
        }
        res = xmlTextReaderRead(handle->reader);
        if(res < 0)
        {
            result = GlobusSoapMessageErrorFailedNextNode(handle->result);
            goto exit;
        }
        handle->next_ready++;    

        goto exit;
    }

    while(type != XML_READER_TYPE_END_ELEMENT)
    {
        if(handle->next_ready > 0)
        {
            handle->next_ready--;
        }
        else
        {
	    res = xmlTextReaderRead(handle->reader);
            if(res < 0)
            {
                result = GlobusSoapMessageErrorFailedNextNode(handle->result);
                goto exit;
            }
            else if(res == 0)
            {
                goto exit;
            }
        }

        type = xmlTextReaderNodeType(handle->reader);
        if(type == XML_READER_TYPE_ELEMENT)
        {
            xsd_QName                   unexpected_qn;

            if (handle->verbose)
            {
                unexpected_qn.Namespace = (char *)
                        xmlTextReaderConstNamespaceUri(handle->reader);
                unexpected_qn.local = (char *)
                        xmlTextReaderConstLocalName(handle->reader);
            }
            result = GlobusSoapMessageErrorFailedEndElement(
                    handle->verbose,
                    &unexpected_qn);

            goto exit;
        }
        else if (type == XML_READER_TYPE_TEXT)
        {
            xmlChar *                   str = NULL;
            if (handle->verbose)
            {
                str = xmlTextReaderValue(handle->reader);
            }
            result = GlobusSoapMessageErrorFailedWithText(handle->verbose, str);

            goto exit;
        }
    }

    if(type == XML_READER_TYPE_END_ELEMENT)
    {
        xsd_QName               qn;
        
        if(handle->callout_table)
        {
            qn.local = (char *)
            xmlTextReaderConstLocalName(handle->reader);
            qn.Namespace = (char *)
            xmlTextReaderConstNamespaceUri(handle->reader);

            result = globus_soap_message_invoke_callouts(
                handle,
                &qn,
                GLOBUS_SOAP_MESSAGE_CALLOUT_END);
        }
                    
        if((idval = xmlTextReaderGetAttribute(
                handle->reader, 
                "Id")) != NULL)
        {
            char * end_idval = 
            globus_common_create_string("%s-end", idval);

	    if(handle->next_ready == 0)
	    {
		result = globus_i_soap_message_deserialize_read(handle);
		if(result != GLOBUS_SUCCESS &&
		   !GlobusSoapMessageStatusFailedElementCheck(result))
		{
		    result = GLOBUS_SUCCESS;
		    goto exit;
		}

		globus_soap_message_deserialize_push_element(handle);
	    }

	    result = globus_soap_message_deserialize_mark(
                handle, end_idval);
            if(result != GLOBUS_SUCCESS)
            {
                result = GlobusSoapMessageErrorSettingId(
                    result);
                goto exit;
            }
	    xmlFree(idval);
            globus_free(end_idval);
        }
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_deserialize_skip(
    globus_soap_message_handle_t        handle)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_deserialize_skip);
    GlobusSoapMessageDebugEnter();

    if(xmlTextReaderSkip(handle->reader, handle->next_ready) <= 0)
    {
        result = GlobusSoapMessageErrorFailedNextNode(handle->result);
        goto exit;
    }

    if(!handle->next_ready)
    {
        handle->next_ready++;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_i_soap_message_deserialize_next_content(
    globus_soap_message_handle_t             handle)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    xmlReaderTypes                      type = XML_READER_TYPE_NONE;
    GlobusFuncName(globus_i_soap_message_deserialize_next_content);
    GlobusSoapMessageDebugEnter();

    if(xmlTextReaderIsEmptyElement(handle->reader))
    {
        goto exit;
    }

    while(type != XML_READER_TYPE_TEXT)
    {
        if(handle->next_ready > 0)
        {
            handle->next_ready--;
        }
        else if(xmlTextReaderRead(handle->reader) <= 0)
        {
            result = GlobusSoapMessageErrorFailedNextNode(handle->result);
            goto exit;
        }

        type = xmlTextReaderNodeType(handle->reader);
        if(type == XML_READER_TYPE_END_ELEMENT)
        {
            /* no content in between nodes */
            globus_soap_message_deserialize_push_element(handle);
            goto exit;
        }

        if(type != XML_READER_TYPE_TEXT &&
           type != XML_READER_TYPE_ATTRIBUTE)
        {
            result = GlobusSoapMessageErrorFailedContent();
            goto exit;
        }
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}
        
globus_result_t
globus_i_soap_message_deserialize_next_attribute(
    globus_soap_message_handle_t        handle)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    xmlReaderTypes                      type = XML_READER_TYPE_NONE;
    GlobusFuncName(globus_i_soap_message_deserialize_next_attribute);
    GlobusSoapMessageDebugEnter();

    if(xmlTextReaderRead(handle->reader) <= 0)
    {
        result = GlobusSoapMessageErrorFailedNextNode(handle->result);
        goto exit;
    }

    type = xmlTextReaderNodeType(handle->reader);
    if(type != XML_READER_TYPE_ATTRIBUTE)
    {
        result = GlobusSoapMessageErrorFailedAttribute();
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_deserialize_envelope(
    globus_soap_message_handle_t        handle)
{
    xsd_QName                           local_qn;
    const xmlChar *                     local;
    const xmlChar *                     soap_env;
    globus_result_t                     result = GLOBUS_SUCCESS;

    GlobusFuncName(globus_soap_message_deserialize_envelope);
    GlobusSoapMessageDebugEnter();

    result = globus_soap_message_set_marker(
	handle,
	GLOBUS_SOAP_MESSAGE_MARKER_ENVELOPE);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusSoapMessageErrorDeserializeFailed(
	    result, &soap_env_qname);
	goto exit;
    }
	    
    result = globus_i_soap_message_deserialize_next_element(handle);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            result, &soap_env_qname);
        goto exit;
    }

    soap_env = xmlTextReaderConstString(handle->reader, SOAP_ENV);
    local = xmlTextReaderConstLocalName(handle->reader);
    if(soap_env != local)
    {
        if (handle->verbose)
        {
            local_qn.Namespace = (char *)
                xmlTextReaderConstNamespaceUri(handle->reader);
            local_qn.local = (char *) local;
        }
        result = GlobusSoapMessageErrorElementNotFound(
                handle->verbose,
                &soap_env_qname,
                &local_qn);
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_deserialize_envelope_end(
    globus_soap_message_handle_t        handle)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_deserialize_envelope_end);
    GlobusSoapMessageDebugEnter();

    result = globus_soap_message_deserialize_element_end(
            handle,
            &soap_env_qname);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            result, &soap_env_qname);
        goto exit;
    }

    result = globus_xio_handle_cntl(
	handle->xio_handle, globus_i_soap_message_buffer_driver,
	GLOBUS_XIO_BUFFER_SET_READ_MARKER_AT_INDEX,
	GLOBUS_SOAP_MESSAGE_MARKER_ENVELOPE_END,
	xmlTextReaderReadPosition(handle->reader));
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusSoapMessageErrorWithMarker(
	    result, "Failed to set read marker: %s", 
	    GLOBUS_SOAP_MESSAGE_MARKER_ENVELOPE_END);
	goto exit;
    }
   
 exit:

    GlobusSoapMessageDebugExit();
    return result;
}
    
globus_result_t
globus_soap_message_deserialize_header(
    globus_soap_message_handle_t        handle)
{
    const xmlChar *                     local;
    const xmlChar *                     soap_header;
    globus_result_t                     result = GLOBUS_SUCCESS;

    GlobusFuncName(globus_soap_message_deserialize_header);
    GlobusSoapMessageDebugEnter();

    result = globus_i_soap_message_deserialize_next_element(handle);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            result, &soap_header_qname);
        goto exit;
    }
    
    soap_header = xmlTextReaderConstString(handle->reader, SOAP_HEADER);
    local = xmlTextReaderConstLocalName(handle->reader);
    if(soap_header != local)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            GLOBUS_SUCCESS, &soap_header_qname);
        goto exit;
    }

 exit:
    
    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_deserialize_header_end(
    globus_soap_message_handle_t        handle)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_deserialize_header_end);
    GlobusSoapMessageDebugEnter();

    result = globus_soap_message_deserialize_element_end(
            handle,
            &soap_header_qname);
    if(result != GLOBUS_SUCCESS)
    {
        GlobusSoapMessageErrorDeserializeFailed(
            result, &soap_header_qname);
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_add_required_header_element(
    globus_soap_message_handle_t        handle,
    const xsd_QName *                   qn)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    xsd_QName *                         value;
    GlobusFuncName(globus_soap_message_add_required_header_element);
    GlobusSoapMessageDebugEnter();

    if(!handle->mustUnderstand_hash)
    {
        globus_hashtable_init(
            &handle->mustUnderstand_hash,
            10,
            xsd_QName_hash,
            xsd_QName_keyeq);
    }

    xsd_QName_copy(&value, qn);

    globus_hashtable_insert(
        &handle->mustUnderstand_hash,
        value, value);

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_remove_required_header_element(
    globus_soap_message_handle_t        handle,
    const xsd_QName *                   qn)
{
    xsd_QName *                         entry;
    if(handle->mustUnderstand_hash)
    {
        entry = globus_hashtable_remove(
            &handle->mustUnderstand_hash, (void *)qn);
	xsd_QName_destroy(entry);
    }
    return GLOBUS_SUCCESS;
}

globus_result_t
globus_soap_message_check_required_headers(
    globus_soap_message_handle_t        handle)
{
    if(!handle->mustUnderstand_hash)
    {
        return GLOBUS_SUCCESS;
    }

    if(globus_hashtable_empty(&handle->mustUnderstand_hash))
    {
        return GLOBUS_SUCCESS;
    }
    else
    {
        xsd_QName *                     element;
        globus_result_t                 result = GLOBUS_SUCCESS;

        element = globus_hashtable_first(&handle->mustUnderstand_hash);

        while(element)
        {
            result = GlobusSoapMessageErrorMustUnderstand(
                result, element);
            element = globus_hashtable_next(&handle->mustUnderstand_hash);
        }
        
        return result;
    }
}

globus_result_t
globus_soap_message_deserialize_body(
    globus_soap_message_handle_t        handle)
{
    const xmlChar *                     local;
    const xmlChar *                     soap_body;
    xsd_QName                           local_qn;
    globus_result_t                     result = GLOBUS_SUCCESS;

    GlobusFuncName(globus_soap_message_deserialize_body);
    GlobusSoapMessageDebugEnter();

    result = globus_i_soap_message_deserialize_next_element(handle);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            result, &soap_body_qname);
        goto exit;
    }

    soap_body = xmlTextReaderConstString(handle->reader, SOAP_BODY);
    local = xmlTextReaderConstLocalName(handle->reader);
    if(soap_body != local)
    {
        if (handle->verbose)
        {
            local_qn.Namespace = (char *)
                xmlTextReaderNamespaceUri(handle->reader);
            local_qn.local = (char *) local;
        }

        result = GlobusSoapMessageErrorElementNotFound(
                    handle->verbose,
                    &soap_body_qname,
                    &local_qn);
        goto exit;
    }
        
 exit:
    
    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_deserialize_fault(
    globus_soap_message_handle_t        handle,
    globus_soap_message_fault_t *       soap_fault_handle,
    globus_soap_fault_callback_func_t   fault_callback,
    xsd_any **                          fault_any,
    void *                              user_data)
{
    const xmlChar *                     local;
    const xmlChar *                     soap_fault;
    const xmlChar *                     element_name;
    const xmlChar *                     faultcode_name;
    const xmlChar *                     faultstring_name;
    const xmlChar *                     faultactor_name;
    const xmlChar *                     detail_name;
    globus_xml_buffer *                 detail_buffer;
    globus_result_t                     result = GLOBUS_SUCCESS;
    globus_soap_message_fault_t         fault = NULL;

    GlobusFuncName(globus_soap_message_deserialize_fault);
    GlobusSoapMessageDebugEnter();

    result = globus_i_soap_message_deserialize_next_element(handle);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            result, &soap_fault_qname);
        goto exit;
    }

    soap_fault = xmlTextReaderConstString(handle->reader, SOAP_FAULT);
    local = xmlTextReaderConstLocalName(handle->reader);
    if(soap_fault != local)
    {
        globus_soap_message_deserialize_push_element(handle);
        goto exit;
    }

    globus_soap_message_fault_init(&fault);

    faultcode_name = xmlTextReaderConstString(
        handle->reader, SOAP_FAULTCODE);
    faultactor_name = xmlTextReaderConstString(
        handle->reader, SOAP_FAULTACTOR);
    faultstring_name = xmlTextReaderConstString(
        handle->reader, SOAP_FAULTSTRING);
    detail_name = xmlTextReaderConstString(
        handle->reader, SOAP_FAULT_DETAIL);

    while(result == GLOBUS_SUCCESS)
    {
        result = globus_i_soap_message_deserialize_next_element(handle);
        if(result != GLOBUS_SUCCESS)
        {
            if(GlobusSoapMessageStatusFailedElementCheck(result) ||
               GlobusSoapMessageStatusFailedWithTextCheck(result))
            {
                globus_soap_message_deserialize_push_element(handle);
                result = GLOBUS_SUCCESS;
                break;
            }
        
            result = GlobusSoapMessageErrorDeserializeFailed(
                result, &soap_fault_qname);
            goto exit;
        }

        element_name = xmlTextReaderConstLocalName(handle->reader);
        if(element_name == faultcode_name)
        {
            result = globus_i_soap_message_deserialize_next_content(handle);
            if(result != GLOBUS_SUCCESS)
            {
                result = GlobusSoapMessageErrorDeserializeFailed(
                    result, &soap_fault_qname);
                goto exit;
            }

            fault->faultcode = 
            xmlTextReaderValue(handle->reader);
        }
        else if(element_name == faultstring_name)
        {
            result = globus_i_soap_message_deserialize_next_content(handle);
            if(result != GLOBUS_SUCCESS)
            {
                result = GlobusSoapMessageErrorDeserializeFailed(
                    result, &soap_faultstring_qname);
                goto exit;
            }

            fault->faultstring = 
            xmlTextReaderValue(handle->reader);
        }
        else if(element_name == faultactor_name)
        {
            result = globus_i_soap_message_deserialize_next_content(handle);
            if(result != GLOBUS_SUCCESS)
            {
                result = GlobusSoapMessageErrorDeserializeFailed(
                    result, &soap_fault_qname);
                goto exit;
            }

            fault->faultactor = 
            xmlTextReaderValue(handle->reader);
        }
        else if(element_name == detail_name)
        {
            xsd_any *                   dummy_any = NULL;
            xsd_QName                   fault_qname;

            result = globus_soap_message_deserialize_element_unknown(
                handle, &fault_qname);
            if(result != GLOBUS_SUCCESS)
            {
                result = GlobusSoapMessageErrorDeserializeFailed(
                    result, &soap_fault_detail_qname);
                goto exit;
            }

            globus_soap_message_deserialize_push_element(handle);

            globus_soap_message_set_marker(handle, "FAULT_DETAIL_BEGIN");

            xsd_any_init(fault_any);

            result = xsd_any_deserialize(
                NULL, (*fault_any), handle, 0);
            if(result != GLOBUS_SUCCESS)
            {
                result = GlobusSoapMessageErrorDeserializeFailed(
                    result, &soap_fault_detail_qname);
                goto exit;
            }
            
            globus_soap_message_set_marker(handle, "FAULT_DETAIL_END");

            xsd_any_init(&fault->detail);
            fault->detail->any_info = (&globus_xml_buffer_contents_info);

            globus_xml_buffer_init(&detail_buffer);
            result = globus_soap_message_get_marked_buffers(
                handle, 
                "FAULT_DETAIL_BEGIN",
                "FAULT_DETAIL_END",
                &detail_buffer->buffer,
                (size_t *)&detail_buffer->length);
            if(result != GLOBUS_SUCCESS)
            {
                globus_xml_buffer_destroy(detail_buffer);
                result = GlobusSoapMessageErrorDeserializeFailed(
                    result, &soap_fault_detail_qname);
                goto exit;
            }
            
            fault->detail->value = detail_buffer;

            fault_callback(handle, 
                           (*fault_any)->any_info->type, 
                           user_data, 
                           NULL);

            do
            {
                result = xsd_any_deserialize_pointer(
                    NULL, &dummy_any, handle, 0);
                if(result != GLOBUS_SUCCESS)
                {
                    if(GlobusSoapMessageStatusFailedElementCheck(result))
                    {
                        result = GLOBUS_SUCCESS;
                        goto exit;
                    }

                    result = GlobusSoapMessageErrorDeserializeFailed(
                        result, &soap_fault_detail_qname);
                    goto exit;
                }
                xsd_any_destroy(dummy_any);
                dummy_any = NULL;

            } while(result == GLOBUS_SUCCESS);

            break;
        }
        else
        {
            result = GlobusSoapMessageErrorUnexpectedElement(
                &soap_fault_qname);
            goto exit;
        }

        result = globus_soap_message_deserialize_element_end(
                handle,
                &soap_fault_qname);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusSoapMessageErrorDeserializeFailed(
                result, &soap_fault_qname);
            goto exit;
        }        
    }
    
 exit:
    
    *soap_fault_handle = fault;

    GlobusSoapMessageDebugExit();
    return result;
}

globus_bool_t
globus_soap_message_deserialize_element_is_nil(
    globus_soap_message_handle_t        handle)
{
    xmlChar *                           nil_value;
    GlobusFuncName(globus_soap_message_deserialize_element_is_nil);
    GlobusSoapMessageDebugEnter();

    nil_value = xmlTextReaderGetAttributeNs(
        handle->reader, 
        XSI_NIL,
        XSI_NS);
    if(nil_value && !(xmlStrcmp(nil_value, "true")))
    {
        xmlFree(nil_value);
        return GLOBUS_TRUE;
    }
    
    if(!nil_value)
    {
        xmlFree(nil_value);
    }

    return GLOBUS_FALSE;
}    
        
globus_result_t
globus_soap_message_deserialize_element_unknown(
    globus_soap_message_handle_t        handle,
    xsd_QName  *                        qname)
{
    globus_result_t                     result = GLOBUS_SUCCESS;

    GlobusFuncName(globus_soap_message_deserialize_element);
    GlobusSoapMessageDebugEnter();

    result = globus_i_soap_message_deserialize_next_element(handle);
    if(result != GLOBUS_SUCCESS)
    {
        xsd_QName *                     q = NULL;

        if(GlobusSoapMessageStatusFailedElementCheck(result) ||
           GlobusSoapMessageStatusFailedWithTextCheck(result))
        {
            globus_soap_message_deserialize_push_element(handle);
            goto exit;
        }
        else
        {
            result = GlobusSoapMessageErrorDeserializeFailed(
                result, q);
        }
        goto exit;
    }

    qname->local = xmlTextReaderLocalName(handle->reader);
    qname->Namespace = xmlTextReaderNamespaceUri(handle->reader);

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_bool_t
globus_soap_message_deserialize_element_is_empty(
    globus_soap_message_handle_t        handle)
{
    return ((handle->next_ready == 0) &&
            xmlTextReaderIsEmptyElement(handle->reader));
}

globus_result_t
globus_soap_message_deserialize_element(
    globus_soap_message_handle_t        handle,
    const xsd_QName *                   qname)
{
    const xmlChar *                     local;
    const xmlChar *                     local_ns;
    const xmlChar *                     element;
    const xmlChar *                     element_ns;
    xsd_QName                           local_qn;
    globus_result_t                     result = GLOBUS_SUCCESS;

    GlobusFuncName(globus_soap_message_deserialize_element);
    GlobusSoapMessageDebugEnter();

    result = globus_i_soap_message_deserialize_next_element(handle);
    if(result != GLOBUS_SUCCESS)
    {
        if(GlobusSoapMessageStatusFailedElementCheck(result) ||
           GlobusSoapMessageStatusFailedWithTextCheck(result))
        {
            if (!xmlTextReaderIsEmptyElement(handle->reader))
            {
                globus_soap_message_deserialize_push_element(handle);
            }

            goto exit;
        }
        else
        {
            result = GlobusSoapMessageErrorDeserializeFailed(
                result, qname);
        }
        goto exit;
    }

    if(qname)
    {
        element = xmlTextReaderConstString(handle->reader, qname->local);
        local = xmlTextReaderConstString(
            handle->reader, xmlTextReaderConstLocalName(handle->reader));
        if(element != local)
        {
            if (handle->verbose)
            {
                local_qn.Namespace = (char *) xmlTextReaderConstString(
                        handle->reader,
                        xmlTextReaderConstNamespaceUri(handle->reader));
                local_qn.local = (char *) local;
            }
            globus_soap_message_deserialize_push_element(handle);

            result = GlobusSoapMessageErrorElementNotFound(
                    handle->verbose,
                    qname,
                    &local_qn);
            goto exit;
        }
        
        element_ns = xmlTextReaderConstString(
            handle->reader, qname->Namespace);
        local_ns = xmlTextReaderConstString(
            handle->reader, xmlTextReaderConstNamespaceUri(handle->reader));
        if(local_ns && element_ns != local_ns)
        {
            if (handle->verbose)
            {
                local_qn.Namespace = (char *) local_ns;
                local_qn.local = (char *) local;
            }
            result = GlobusSoapMessageErrorElementNotFound(
                    handle->verbose,
                    qname,
                    &local_qn);
            goto exit;
        }
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_deserialize_anyURI(
    globus_soap_message_handle_t        handle,
    xsd_anyURI *                        str)
{
    const xmlChar *                     constval;
    globus_result_t                     result = GLOBUS_SUCCESS;
    int                                 start_token, end_token;
    GlobusFuncName(globus_soap_message_deserialize_anyURI);
    GlobusSoapMessageDebugEnter();

    result = globus_i_soap_message_deserialize_next_content(handle);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            result, &xsd_string_qname);
        goto exit;
    }

    constval = xmlTextReaderConstValue(handle->reader);
    if(constval)
    {
	*str = globus_l_soap_message_my_strtok(
	    constval, strlen(constval), &start_token, &end_token, "\n\t ");
	if (*str == NULL)
	{
	    result = GlobusSoapMessageErrorDeserializeFailed(
		result, &xsd_anyURI_qname);
	    goto exit;
	}
    }
    else
    {
	*str = NULL;
    }
    
 exit:
    
    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_deserialize_string(
    globus_soap_message_handle_t        handle,
    xsd_string *                        str)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_deserialize_string);
    GlobusSoapMessageDebugEnter();

    result = globus_i_soap_message_deserialize_next_content(handle);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            result, &xsd_string_qname);
        goto exit;
    }

    *str = (xsd_string) xmlTextReaderValue(handle->reader);
    
 exit:
    
    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_deserialize_string_attribute(
    globus_soap_message_handle_t        handle,
    const xsd_QName *                   attr_name,
    xsd_string *                        str)
{
    const xmlChar *                     attr_ns;
    const xmlChar *                     local_ns;
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_deserialize_string_attribute);
    GlobusSoapMessageDebugEnter();

    attr_ns = xmlTextReaderConstString(
        handle->reader, attr_name->Namespace);
    local_ns = xmlTextReaderConstNamespaceUri(handle->reader);

    if(attr_ns != local_ns && (attr_ns != NULL) && (*attr_ns != '\0'))
    {
        *str = xmlTextReaderGetAttributeNs(
            handle->reader, attr_name->local, attr_name->Namespace);
    }
    else
    {
        *str = xmlTextReaderGetAttribute(
            handle->reader, attr_name->local);
    }
    if(!*str)
    {
        result = GlobusSoapMessageErrorAttributeNotFound(
                handle->verbose,
                attr_name);
        goto exit;
    }
    
 exit:
    
    GlobusSoapMessageDebugExit();
    return result;
}


globus_result_t
globus_soap_message_deserialize_string_list(
    globus_soap_message_handle_t        handle,
    xsd_string_array *                  inst)
{
    const xmlChar *                     tmp_value = NULL;
    char *                              newstr;
    char *                              tmpstr;
    char *                              token;
    int                                 start_token, end_token;
    xsd_string *                        new_element = NULL;
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_deserialize_string_list);
    GlobusSoapMessageDebugEnter();

    result = globus_i_soap_message_deserialize_next_content(handle);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            result, &xsd_QName_qname);
        goto exit;
    }
    
    tmp_value = xmlTextReaderConstValue(handle->reader);
    if(!tmp_value)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            result, &xsd_QName_qname);
        goto exit;
    }

    newstr = globus_libc_strdup(tmp_value);
    tmpstr = newstr;
    token = globus_l_soap_message_my_strtok(
        tmpstr, strlen(tmpstr), &start_token, &end_token, "\n\t ");
    tmpstr += end_token;
        
    while(token)
    {
        new_element = xsd_string_array_push(inst);
        *new_element = token;

        if(end_token < 0)
        {
            break;
        }

        token = globus_l_soap_message_my_strtok(
            tmpstr, strlen(tmpstr), &start_token, &end_token, "\n\t ");
        tmpstr += end_token;
    }

    globus_free(newstr);

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_deserialize_string_attribute_list(
    globus_soap_message_handle_t        handle,
    const xsd_QName *                   attr_qname,
    xsd_string_array *                  inst)
{
    const xmlChar *                     attr_ns = NULL;
    const xmlChar *                     local_ns = NULL;
    xmlChar *                           tmp_value = NULL;
    char *                              newstr;
    char *                              tmpstr;
    char *                              token;
    int                                 start_token, end_token;
    xsd_string *                        new_element = NULL;
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_deserialize_string_attribute_list);
    GlobusSoapMessageDebugEnter();

    attr_ns = xmlTextReaderConstString(
        handle->reader, attr_qname->Namespace);
    local_ns = xmlTextReaderConstNamespaceUri(handle->reader);

    if(attr_ns != local_ns  && (attr_ns != NULL) && (*attr_ns != '\0'))
    {
        tmp_value = xmlTextReaderGetAttributeNs(
            handle->reader, attr_qname->local, attr_qname->Namespace);
    }
    else
    {
        tmp_value = xmlTextReaderGetAttribute(
            handle->reader, attr_qname->local);
    }

    if(!tmp_value)
    {
        goto exit;
    }

    if(!(*tmp_value))
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            result, &xsd_string_qname);
        goto exit;
    }

    newstr = globus_libc_strdup(tmp_value);
    tmpstr = newstr;
    token = globus_l_soap_message_my_strtok(
        tmpstr, strlen(tmpstr), &start_token, &end_token, "\n\t ");
    tmpstr += end_token;
        
    while(token)
    {
        new_element = xsd_string_array_push(inst);
        *new_element = token;

        if(end_token < 0)
        {
            break;
        }

        token = globus_l_soap_message_my_strtok(
            tmpstr, strlen(tmpstr), &start_token, &end_token, "\n\t ");
        tmpstr += end_token;
    }

    globus_free(newstr);

 exit:

    if(tmp_value)
    {
	xmlFree(tmp_value);
    }
    
    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_deserialize_duration_attribute(
    globus_soap_message_handle_t        handle,
    const xsd_QName *                   attr_name,
    xsd_duration *                      val)
{
    const xmlChar *                     attr_ns;
    const xmlChar *                     local_ns;
    xmlChar *                           tmp_value;
    xmlSchemaValPtr                     schema_val = NULL;
    int                                 res;
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_deserialize_duration_attribute);
    GlobusSoapMessageDebugEnter();

    attr_ns = xmlTextReaderConstString(
        handle->reader, attr_name->Namespace);
    local_ns = xmlTextReaderConstNamespaceUri(handle->reader);

    if(attr_ns != local_ns && (attr_ns != NULL) && (*attr_ns != '\0'))
    {
        tmp_value = xmlTextReaderGetAttributeNs(
            handle->reader, attr_name->local, attr_name->Namespace);
    }
    else
    {
        tmp_value = xmlTextReaderGetAttribute(
            handle->reader, attr_name->local);
    }

    if(!tmp_value)
    {
        result = GlobusSoapMessageErrorAttributeNotFound(
                handle->verbose,
                attr_name);
        goto exit;
    }

    res = xmlSchemaValidatePredefinedType(
        xmlSchemaGetBuiltInType(XML_SCHEMAS_DURATION),
        tmp_value,
        &schema_val);
    if(res != 0)
    {
        result = GlobusSoapMessageErrorDeserializeContentFailed(
            &xsd_duration_qname, tmp_value);
        goto exit;
    }

    val->tm_mon = schema_val->value.date.mon - 1;
    val->tm_mday = schema_val->value.date.day;
    val->tm_sec = (int) schema_val->value.date.sec;
    
    xmlSchemaFreeValue(schema_val);
    
 exit:

    if(tmp_value)
    {
        xmlFree(tmp_value);
    }
    
    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_deserialize_dateTime_attribute(
    globus_soap_message_handle_t        handle,
    const xsd_QName *                   attr_name,
    xsd_dateTime *                      val)
{
    const xmlChar *                     attr_ns;
    const xmlChar *                     local_ns;
    xmlChar *                           tmp_value = NULL;
    xmlSchemaValPtr                     schema_val = NULL;
    int                                 res;
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_deserialize_dateTime_attribute);
    GlobusSoapMessageDebugEnter();

    attr_ns = xmlTextReaderConstString(
        handle->reader, attr_name->Namespace);
    local_ns = xmlTextReaderConstNamespaceUri(handle->reader);

    if(attr_ns != local_ns && (attr_ns != NULL) && (*attr_ns != '\0'))
    {
        tmp_value = xmlTextReaderGetAttributeNs(
            handle->reader, attr_name->local, attr_name->Namespace);
    }
    else
    {
        tmp_value = xmlTextReaderGetAttribute(
            handle->reader, attr_name->local);
    }
    if(!tmp_value)
    {
        result = GlobusSoapMessageErrorAttributeNotFound(
                handle->verbose,
                attr_name);
        goto exit;
    }

    res = xmlSchemaValidatePredefinedType(
        xmlSchemaGetBuiltInType(XML_SCHEMAS_DATETIME),
        tmp_value,
        &schema_val);
    if(res != 0)
    {
        result = GlobusSoapMessageErrorDeserializeContentFailed(
            &xsd_dateTime_qname, tmp_value);
        goto exit;
    }

    val->tm_year = schema_val->value.date.year - 1900;
    val->tm_mon = schema_val->value.date.mon;
    val->tm_mday = schema_val->value.date.day;
    val->tm_hour = schema_val->value.date.hour;
    val->tm_min = schema_val->value.date.min;
    val->tm_sec = schema_val->value.date.sec;
    if(schema_val->value.date.tz_flag)
    {
        val->tm_min += schema_val->value.date.tzo;
    }

    xmlSchemaFreeValue(schema_val);
    
 exit:

    if(tmp_value)
    {
        xmlFree(tmp_value);
    }
    
    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_deserialize_date_attribute(
    globus_soap_message_handle_t        handle,
    const xsd_QName *                   attr_name,
    xsd_date *                          val)
{
    const xmlChar *                     attr_ns;
    const xmlChar *                     local_ns;
    xmlChar *                           tmp_value = NULL;
    xmlSchemaValPtr                     schema_val = NULL;
    int                                 res;
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_deserialize_date_attribute);
    GlobusSoapMessageDebugEnter();

    attr_ns = xmlTextReaderConstString(
        handle->reader, attr_name->Namespace);
    local_ns = xmlTextReaderConstNamespaceUri(handle->reader);

    if(attr_ns != local_ns && (attr_ns != NULL) && (*attr_ns != '\0'))
    {
        tmp_value = xmlTextReaderGetAttributeNs(
            handle->reader, attr_name->local, attr_name->Namespace);
    }
    else
    {
        tmp_value = xmlTextReaderGetAttribute(
            handle->reader, attr_name->local);
    }
    if(!tmp_value)
    {
        result = GlobusSoapMessageErrorAttributeNotFound(
                handle->verbose,
                attr_name);
        goto exit;
    }


    res = xmlSchemaValidatePredefinedType(
        xmlSchemaGetBuiltInType(XML_SCHEMAS_DATE),
        tmp_value,
        &schema_val);
    if(res != 0)
    {
        result = GlobusSoapMessageErrorDeserializeContentFailed(
            &xsd_date_qname, tmp_value);
        goto exit;
    }

    val->tm_year = schema_val->value.date.year - 1900;
    val->tm_mon = schema_val->value.date.mon - 1;
    val->tm_mday = schema_val->value.date.day;

    xmlSchemaFreeValue(schema_val);
    
 exit:

    if(tmp_value)
    {
        xmlFree(tmp_value);
    }
    
    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_deserialize_int_attribute(
    globus_soap_message_handle_t        handle,
    const xsd_QName *                   attr_name,
    xsd_int *                           val)
{
    const xmlChar *                     attr_ns;
    const xmlChar *                     local_ns;
    xmlChar *                           tmp_value;
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_deserialize_int_attribute);
    GlobusSoapMessageDebugEnter();

    attr_ns = xmlTextReaderConstString(
        handle->reader, attr_name->Namespace);
    local_ns = xmlTextReaderConstNamespaceUri(handle->reader);

    if(attr_ns != local_ns && (attr_ns != NULL) && (*attr_ns != '\0'))
    {
        tmp_value = xmlTextReaderGetAttributeNs(
            handle->reader, attr_name->local, attr_name->Namespace);
    }
    else
    {
        tmp_value = xmlTextReaderGetAttribute(
            handle->reader, attr_name->local);
    }
    if(!tmp_value)
    {
        result = GlobusSoapMessageErrorAttributeNotFound(
                handle->verbose,
                attr_name);
        goto exit;
    }

    *val = strtol((const char *)tmp_value, NULL, 0);
    if(*val == INT32_MAX || *val == INT32_MIN)
    {
        result = GlobusSoapMessageErrorDeserializeContentFailed(
            &xsd_int_qname, tmp_value);
        goto exit;
    }
    
 exit:

    if(tmp_value)
    {
        xmlFree(tmp_value);
    }
    
    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_deserialize_integer_attribute(
    globus_soap_message_handle_t        handle,
    const xsd_QName *                   attr_name,
    xsd_integer *                       val)
{
    int                                 rc;
    xmlChar *                           tmp_value = NULL;
    char *                              tmp = NULL;
    char *                              tmp_start;
    char *                              tmp_end;
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_deserialize_integer_attribute);
    GlobusSoapMessageDebugEnter();

    tmp_value = xmlTextReaderGetAttributeNs(
        handle->reader, attr_name->local, attr_name->Namespace);
    if(!tmp_value)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            result, &xsd_integer_qname);
        goto exit;
    }

    tmp = globus_libc_strdup(tmp_value);
    tmp_start = tmp;

    while(*tmp_start && isspace(*tmp_start))
    {
        tmp_start++;
    }
    if (*tmp_start == '+')
    {
        tmp_start++;
    }
    if (*tmp_start != '-' && !isdigit(*tmp_start))
    {
        result = GlobusSoapMessageErrorDeserializeContentFailed(
            &xsd_integer_qname, tmp_value);
        goto exit;
    }
    tmp_end = tmp_start;

    if (*tmp_end == '-')
    {
        tmp_end++;
    }
    while (*tmp_end && isdigit(*tmp_end))
    {
        tmp_end++;
    }

    if (*tmp_end && isspace(*tmp_end))
    {
        *tmp_end = '\0';
        tmp_end++;
    }
    while (*tmp_end && isspace(*tmp_end))
    {
        tmp_end++;
    }
    if (*tmp_end)
    {
        /* garbage at end */
        result = GlobusSoapMessageErrorDeserializeContentFailed(
            &xsd_integer_qname, tmp_value);
        goto exit;
    }
    rc = BN_dec2bn(val, tmp_start);
    if (rc == 0)
    {
        result = GlobusSoapMessageErrorDeserializeContentFailed(
            &xsd_integer_qname, tmp_value);
        goto exit;
    }
    
 exit:

    if(tmp_value)
    {
        xmlFree(tmp_value);
    }
    if(tmp)
    {
        free(tmp);
    }
    
    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_deserialize_long_attribute(
    globus_soap_message_handle_t        handle,
    const xsd_QName *                   attr_name,
    xsd_long *                          val)
{
    const xmlChar *                     attr_ns;
    const xmlChar *                     local_ns;
    xmlChar *                           tmp_value;
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_deserialize_long_attribute);
    GlobusSoapMessageDebugEnter();

    attr_ns = xmlTextReaderConstString(
        handle->reader, attr_name->Namespace);
    local_ns = xmlTextReaderConstNamespaceUri(handle->reader);

    if(attr_ns != local_ns && (attr_ns != NULL) && (*attr_ns != '\0'))
    {
        tmp_value = xmlTextReaderGetAttributeNs(
            handle->reader, attr_name->local, attr_name->Namespace);
    }
    else
    {
        tmp_value = xmlTextReaderGetAttribute(
            handle->reader, attr_name->local);
    }
    if(!tmp_value)
    {
        result = GlobusSoapMessageErrorAttributeNotFound(
                handle->verbose,
                attr_name);
        goto exit;
    }

    *val = strtoll(tmp_value, NULL, 0);
    if(*val == INT64_MAX || *val == INT64_MIN)
    {
        if(errno == ERANGE)
        {
            result = GlobusSoapMessageErrorDeserializeContentFailed(
                &xsd_long_qname, tmp_value);
            goto exit;
        }
    }
    
 exit:
    
    if(tmp_value)
    {
        xmlFree(tmp_value);
    }

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_deserialize_float_attribute(
    globus_soap_message_handle_t        handle,
    const xsd_QName *                   attr_name,
    xsd_float *                         val)
{
    const xmlChar *                     attr_ns;
    const xmlChar *                     local_ns;
    xmlChar *                           tmp_value = NULL;
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_deserialize_float_attribute);
    GlobusSoapMessageDebugEnter();

    attr_ns = xmlTextReaderConstString(
        handle->reader, attr_name->Namespace);
    local_ns = xmlTextReaderConstNamespaceUri(handle->reader);

    if(attr_ns != local_ns && (attr_ns != NULL) && (*attr_ns != '\0'))
    {
        tmp_value = xmlTextReaderGetAttributeNs(
            handle->reader, attr_name->local, attr_name->Namespace);
    }
    else
    {
        tmp_value = xmlTextReaderGetAttribute(
            handle->reader, attr_name->local);
    }
    if(!tmp_value)
    {
        result = GlobusSoapMessageErrorAttributeNotFound(
                handle->verbose,
                attr_name);
        goto exit;
    }

    *val = (float) strtod(tmp_value, NULL);

 exit:
    
    if(tmp_value)
    {
        xmlFree(tmp_value);
    }

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_deserialize_base64Binary_attribute(
    globus_soap_message_handle_t        handle,
    const xsd_QName *                   attr_qname,
    xsd_base64Binary *                  val)
{
    xmlChar *                           tmp_value = NULL;
    int                                 length;
    globus_result_t                     result = GLOBUS_SUCCESS;
    
    GlobusFuncName(globus_soap_message_deserialize_base64Binary_attribute);
    GlobusSoapMessageDebugEnter();

    tmp_value = xmlTextReaderGetAttributeNs(
        handle->reader, attr_qname->local, attr_qname->Namespace);
    if(!tmp_value)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            result, &xsd_float_qname);
        goto exit;
    }

    length = strlen(tmp_value);

    val->length = (int)ceil((length * 3) / 4) + 1;
    val->value = globus_malloc(val->length);
    if(!val->value)
    {
	result = GlobusSoapMessageErrorOutOfMemory;
	goto exit;
    }
    
    if(xmlBase64Decode(
        tmp_value, 
	(unsigned long *)&length, 
	val->value, 
	(unsigned long *)&val->length) < 0)
    {
	result = GlobusSoapMessageErrorDeserializeContentFailed(
	    attr_qname, tmp_value);
	goto exit;
    }

 exit:
 
    if(tmp_value)
    {
        xmlFree(tmp_value);
    }

    GlobusSoapMessageDebugExit();
    return result;
}    

globus_result_t
globus_soap_message_deserialize_boolean(
    globus_soap_message_handle_t        handle,
    xsd_boolean *                       val)
{
    const xmlChar *                     tmp_value = NULL;
    globus_result_t                     result = GLOBUS_SUCCESS;
    
    GlobusFuncName(globus_soap_message_deserialize_boolean);
    GlobusSoapMessageDebugEnter();

    result = globus_i_soap_message_deserialize_next_content(handle);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            result, &xsd_boolean_qname);
        goto exit;
    }
    
    tmp_value = xmlTextReaderConstValue(handle->reader);
    if(!tmp_value)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            GLOBUS_SUCCESS, &xsd_boolean_qname);
        goto exit;
    }

    if(!xmlStrcmp(tmp_value, "false"))
    {
        *val = 0;
    }
    else
    {
        *val = 1;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_deserialize_boolean_attribute(
    globus_soap_message_handle_t        handle,
    const xsd_QName *                   attr_name,
    xsd_boolean *                       val)
{
    const xmlChar *                     attr_ns;
    const xmlChar *                     local_ns;
    xmlChar *                           tmp_value;
    globus_result_t                     result = GLOBUS_SUCCESS;
    
    GlobusFuncName(globus_soap_message_deserialize_boolean_attribute);
    GlobusSoapMessageDebugEnter();

    attr_ns = xmlTextReaderConstString(
        handle->reader, attr_name->Namespace);
    local_ns = xmlTextReaderConstNamespaceUri(handle->reader);

    if(attr_ns != local_ns  && (attr_ns != NULL) && (*attr_ns != '\0'))
    {
        tmp_value = xmlTextReaderGetAttributeNs(
            handle->reader, attr_name->local, attr_name->Namespace);
    }
    else
    {
        tmp_value = xmlTextReaderGetAttribute(
            handle->reader, attr_name->local);
    }
    if(!tmp_value)
    {
        result = GlobusSoapMessageErrorAttributeNotFound(
                handle->verbose,
                attr_name);
        goto exit;
    }
    
    if(!xmlStrcmp(tmp_value, "false"))
    {
        *val = 0;
    }
    else
    {
        *val = 1;
    }

 exit:

    if(tmp_value)
    {
        xmlFree(tmp_value);
    }

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_deserialize_dateTime(
    globus_soap_message_handle_t        handle,
    xsd_dateTime *                      val)
{
    xmlSchemaValPtr                     schema_val = NULL;
    int                                 res = 0;
    const xmlChar *                     tmp_value;
    globus_result_t                     result = GLOBUS_SUCCESS;

    GlobusFuncName(globus_soap_message_deserialize_dateTime);
    GlobusSoapMessageDebugEnter();

    result = globus_i_soap_message_deserialize_next_content(handle);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            result, &xsd_dateTime_qname);
        goto exit;
    }
    
    tmp_value = xmlTextReaderConstValue(handle->reader);
    if(!tmp_value)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            result, &xsd_dateTime_qname);
        goto exit;
    }

    res = xmlSchemaValidatePredefinedType(
        xmlSchemaGetBuiltInType(XML_SCHEMAS_DATETIME),
        tmp_value,
        &schema_val);
    if(res != 0)
    {
        result = GlobusSoapMessageErrorDeserializeContentFailed(
            &xsd_dateTime_qname, tmp_value);
        goto exit;
    }

    val->tm_year = schema_val->value.date.year - 1900;
    val->tm_mon = schema_val->value.date.mon - 1;
    val->tm_mday = schema_val->value.date.day;
    val->tm_hour = schema_val->value.date.hour;
    val->tm_min = schema_val->value.date.min;
    val->tm_sec = schema_val->value.date.sec;
    if(schema_val->value.date.tz_flag)
    {
        val->tm_min += schema_val->value.date.tzo;
    }

    xmlSchemaFreeValue(schema_val);

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}
        
globus_result_t
globus_soap_message_deserialize_date(
    globus_soap_message_handle_t        handle,
    xsd_date *                          val)
{
    int                                 res = 0;
    xmlSchemaValPtr                     schema_val = NULL;
    const xmlChar *                     tmp_value;
    globus_result_t                     result = GLOBUS_SUCCESS;

    GlobusFuncName(globus_soap_message_deserialize_date);
    GlobusSoapMessageDebugEnter();

    result = globus_i_soap_message_deserialize_next_content(handle);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            result, &xsd_date_qname);
        goto exit;
    }
    
    tmp_value = xmlTextReaderConstValue(handle->reader);
    if(!tmp_value)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            GLOBUS_SUCCESS, &xsd_date_qname);
        goto exit;
    }

    res = xmlSchemaValidatePredefinedType(
        xmlSchemaGetBuiltInType(XML_SCHEMAS_DATE),
        tmp_value,
        &schema_val);
    if(res != 0)
    {
        result = GlobusSoapMessageErrorDeserializeContentFailed(
            &xsd_date_qname, tmp_value);
        goto exit;
    }

    val->tm_year = schema_val->value.date.year - 1900;
    val->tm_mon = schema_val->value.date.mon - 1;
    val->tm_mday = schema_val->value.date.day;

    xmlSchemaFreeValue(schema_val);

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_deserialize_time(
    globus_soap_message_handle_t        handle,
    xsd_time *                          val)
{
    xmlSchemaValPtr                     schema_val = NULL;
    int                                 res;
    const xmlChar *                     tmp_value;
    globus_result_t                     result = GLOBUS_SUCCESS;

    GlobusFuncName(globus_soap_message_deserialize_time);
    GlobusSoapMessageDebugEnter();

    result = globus_i_soap_message_deserialize_next_content(handle);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            result, &xsd_time_qname);
        goto exit;
    }
    
    tmp_value = xmlTextReaderConstValue(handle->reader);
    if(!tmp_value)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            result, &xsd_time_qname);
        goto exit;
    }

    res = xmlSchemaValidatePredefinedType(
        xmlSchemaGetBuiltInType(XML_SCHEMAS_TIME),
        tmp_value,
        &schema_val);
    if(res != 0)
    {
        result = GlobusSoapMessageErrorDeserializeContentFailed(
            &xsd_time_qname, tmp_value);
        goto exit;
    }

    val->tm_hour = schema_val->value.date.hour;
    val->tm_min = schema_val->value.date.min;
    val->tm_sec = schema_val->value.date.sec;

    if(schema_val->value.date.tz_flag)
    {
        val->tm_sec += (schema_val->value.date.tzo * 60);
    }

    xmlSchemaFreeValue(schema_val);

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}
    

globus_result_t
globus_soap_message_deserialize_duration(
    globus_soap_message_handle_t        handle,
    xsd_duration *                      val)
{
    xmlSchemaValPtr                     schema_val = NULL;
    int                                 res;
    const xmlChar *                     tmp_value;
    globus_result_t                     result = GLOBUS_SUCCESS;

    GlobusFuncName(globus_soap_message_deserialize_duration);
    GlobusSoapMessageDebugEnter();

    result = globus_i_soap_message_deserialize_next_content(handle);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            result, &xsd_duration_qname);
        goto exit;
    }
    
    tmp_value = xmlTextReaderConstValue(handle->reader);
    if(!tmp_value)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            result, &xsd_duration_qname);
        goto exit;
    }

    res = xmlSchemaValidatePredefinedType(
        xmlSchemaGetBuiltInType(XML_SCHEMAS_DURATION),
        tmp_value,
        &schema_val);
    if(res != 0)
    {
        result = GlobusSoapMessageErrorDeserializeContentFailed(
            &xsd_duration_qname, tmp_value);
        goto exit;
    }

    val->tm_mon = schema_val->value.date.mon - 1;
    val->tm_mday = schema_val->value.date.day;
    val->tm_sec = (int) schema_val->value.date.sec;
    
    xmlSchemaFreeValue(schema_val);

 exit:
    
    GlobusSoapMessageDebugExit();
    return result;
}
    

globus_result_t
globus_soap_message_deserialize_float(
    globus_soap_message_handle_t        handle,
    xsd_float *                         val)
{
    const xmlChar *                     tmp_value;
    globus_result_t                     result = GLOBUS_SUCCESS;

    GlobusFuncName(globus_soap_message_deserialize_float);
    GlobusSoapMessageDebugEnter();

    result = globus_i_soap_message_deserialize_next_content(handle);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            result, &xsd_float_qname);
        goto exit;
    }
    
    tmp_value = xmlTextReaderConstValue(handle->reader);
    if(!tmp_value)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            result, &xsd_float_qname);
        goto exit;
    }

    *val = (float) strtod(tmp_value, NULL);

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_deserialize_double(
    globus_soap_message_handle_t        handle,
    xsd_double *                        val)
{
    const xmlChar *                     tmp_value;
    globus_result_t                     result = GLOBUS_SUCCESS;

    GlobusFuncName(globus_soap_message_deserialize_double);
    GlobusSoapMessageDebugEnter();

    result = globus_i_soap_message_deserialize_next_content(handle);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            result, &xsd_double_qname);
        goto exit;
    }
    
    tmp_value = xmlTextReaderConstValue(handle->reader);
    if(!tmp_value)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            result, &xsd_double_qname);
        goto exit;
    }

    *val = strtod(tmp_value, NULL);

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_deserialize_int(
    globus_soap_message_handle_t        handle,
    xsd_int *                           val)
{
    const xmlChar *                     tmp_value;
    globus_result_t                     result = GLOBUS_SUCCESS;

    GlobusFuncName(globus_soap_message_deserialize_int);
    GlobusSoapMessageDebugEnter();

    result = globus_i_soap_message_deserialize_next_content(handle);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            result, &xsd_int_qname);
        goto exit;
    }
    
    tmp_value = xmlTextReaderConstValue(handle->reader);
    if(!tmp_value)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            result, &xsd_int_qname);
        goto exit;
    }

    *val = strtol((const char *)tmp_value, NULL, 0);
    if(*val == INT32_MAX || *val == INT32_MIN)
    {
        result = GlobusSoapMessageErrorDeserializeContentFailed(
            &xsd_int_qname, tmp_value);
        goto exit;
    }
        
 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_deserialize_long(
    globus_soap_message_handle_t        handle,
    xsd_long *                          val)
{
    const xmlChar *                     tmp_value;
    globus_result_t                     result = GLOBUS_SUCCESS;

    GlobusFuncName(globus_soap_message_deserialize_long);
    GlobusSoapMessageDebugEnter();

    result = globus_i_soap_message_deserialize_next_content(handle);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            result, &xsd_long_qname);
        goto exit;
    }
    
    tmp_value = xmlTextReaderConstValue(handle->reader);
    if(!tmp_value)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            GLOBUS_SUCCESS, &xsd_long_qname);
        goto exit;
    }

    *val = strtoll(tmp_value, NULL, 0);
    if(*val == INT64_MAX || *val == INT64_MIN)
    {
        if(errno == ERANGE)
        {
            result = GlobusSoapMessageErrorDeserializeContentFailed(
                &xsd_long_qname, tmp_value);
            goto exit;
        }
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}


globus_result_t
globus_soap_message_deserialize_decimal(
    globus_soap_message_handle_t        handle,
    xsd_decimal *                       val)
{
    GlobusFuncName(globus_soap_message_deserialize_decimal);
    GlobusSoapMessageDebugEnter();

    /* not implemented */
    globus_panic(GLOBUS_SOAP_MESSAGE_MODULE, GLOBUS_SUCCESS,
                 "%s NOT IMPLEMENTED", _globus_func_name);

    GlobusSoapMessageDebugExit();
    return GLOBUS_FAILURE;
}

globus_result_t
globus_soap_message_deserialize_integer(
    globus_soap_message_handle_t        handle,
    xsd_integer *                       val)
{
    const xmlChar *                     tmp_value;
    char *                              tmp = NULL;
    char *                              tmp_start;
    char *                              tmp_end;
    int                                 rc;
    globus_result_t                     result = GLOBUS_SUCCESS;
    
    GlobusFuncName(globus_soap_message_deserialize_integer);
    GlobusSoapMessageDebugEnter();

    result = globus_i_soap_message_deserialize_next_content(handle);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            result, &xsd_integer_qname);
        goto exit;
    }
    
    tmp_value = xmlTextReaderConstValue(handle->reader);
    if(!tmp_value)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            GLOBUS_SUCCESS, &xsd_integer_qname);
        goto exit;
    }

    tmp = globus_libc_strdup(tmp_value);
    tmp_start = tmp;

    while(*tmp_start && isspace(*tmp_start))
    {
        tmp_start++;
    }
    if (*tmp_start == '+')
    {
        tmp_start++;
    }
    if (*tmp_start != '-' && !isdigit(*tmp_start))
    {
        result = GlobusSoapMessageErrorDeserializeContentFailed(
            &xsd_integer_qname, tmp_value);
        goto exit;
    }
    tmp_end = tmp_start;

    if (*tmp_end == '-')
    {
        tmp_end++;
    }
    while (*tmp_end && isdigit(*tmp_end))
    {
        tmp_end++;
    }

    if (*tmp_end && isspace(*tmp_end))
    {
        *tmp_end = '\0';
        tmp_end++;
    }
    while (*tmp_end && isspace(*tmp_end))
    {
        tmp_end++;
    }
    if (*tmp_end)
    {
        /* garbage at end */
        result = GlobusSoapMessageErrorDeserializeContentFailed(
            &xsd_integer_qname, tmp_value);
        goto exit;
    }
    rc = BN_dec2bn(val, tmp_start);
    if (rc == 0)
    {
        result = GlobusSoapMessageErrorDeserializeContentFailed(
            &xsd_integer_qname, tmp_value);
        goto exit;
    }

 exit:
    if (tmp)
    {
        free(tmp);
    }

    GlobusSoapMessageDebugExit();
    return result;
}
    
globus_result_t
globus_soap_message_deserialize_byte(
    globus_soap_message_handle_t        handle,
    xsd_byte *                          val)
{
    const xmlChar *                     tmp_value;
    short                               sval;
    globus_result_t                     result = GLOBUS_SUCCESS;

    GlobusFuncName(globus_soap_message_deserialize_byte);
    GlobusSoapMessageDebugEnter();

    result = globus_i_soap_message_deserialize_next_content(handle);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            result, &xsd_byte_qname);
        goto exit;
    }
    
    tmp_value = xmlTextReaderConstValue(handle->reader);
    if(!tmp_value)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            result, &xsd_byte_qname);
        goto exit;
    }

    sscanf(tmp_value, "%hd", &sval);
    *val = (char) sval;

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_deserialize_unsignedByte(
    globus_soap_message_handle_t        handle,
    xsd_unsignedByte *                  val)
{
    const xmlChar *                     tmp_value;
    unsigned short                      sval;
    globus_result_t                     result = GLOBUS_SUCCESS;

    GlobusFuncName(globus_soap_message_deserialize_unsignedByte);
    GlobusSoapMessageDebugEnter();

    result = globus_i_soap_message_deserialize_next_content(handle);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            result, &xsd_unsignedByte_qname);
        goto exit;
    }
    
    tmp_value = xmlTextReaderConstValue(handle->reader);
    if(!tmp_value)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            GLOBUS_SUCCESS, &xsd_unsignedByte_qname);
        goto exit;
    }

    if(sscanf(tmp_value, "%hu", &sval) == 0)
    {
        result = GlobusSoapMessageErrorDeserializeContentFailed(
            &xsd_unsignedByte_qname, tmp_value);
        goto exit;
    }

    *val = (unsigned char) sval;

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}  

globus_result_t
globus_soap_message_deserialize_unsignedLong(
    globus_soap_message_handle_t        handle,
    xsd_unsignedLong *                  val)
{
    const xmlChar *                     tmp_value;
    globus_result_t                     result = GLOBUS_SUCCESS;

    GlobusFuncName(globus_soap_message_deserialize_unsignedLong);
    GlobusSoapMessageDebugEnter();

    result = globus_i_soap_message_deserialize_next_content(handle);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            result, &xsd_unsignedLong_qname);
        goto exit;
    }
    
    tmp_value = xmlTextReaderConstValue(handle->reader);
    if(!tmp_value)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            GLOBUS_SUCCESS, &xsd_unsignedLong_qname);
        goto exit;
    }

    *val = strtoull(tmp_value, NULL, 0);
    if(*val == UINT64_MAX)
    {
        result = GlobusSoapMessageErrorDeserializeContentFailed(
            &xsd_unsignedLong_qname, tmp_value);
        goto exit;
    }

 exit:
    
    GlobusSoapMessageDebugExit();
    return result;
}  

globus_result_t
globus_soap_message_deserialize_unsignedInt(
    globus_soap_message_handle_t        handle,
    xsd_unsignedInt *                   val)
{
    const xmlChar *                     tmp_value;
    globus_result_t                     result = GLOBUS_SUCCESS;

    GlobusFuncName(globus_soap_message_deserialize_unsignedInt);
    GlobusSoapMessageDebugEnter();

    result = globus_i_soap_message_deserialize_next_content(handle);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            result, &xsd_unsignedInt_qname);
        goto exit;
    }
    
    tmp_value = xmlTextReaderConstValue(handle->reader);
    if(!tmp_value)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            GLOBUS_SUCCESS, &xsd_unsignedInt_qname);
        goto exit;
    }

    *val = strtoul(tmp_value, NULL, 0);
    if(*val == UINT32_MAX)
    {
        result = GlobusSoapMessageErrorDeserializeContentFailed(
            &xsd_unsignedInt_qname, tmp_value);
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}  
    

globus_result_t
globus_soap_message_deserialize_unsignedShort(
    globus_soap_message_handle_t        handle,
    xsd_unsignedShort *                 val)
{
    const xmlChar *                     tmp_value;
    globus_result_t                     result = GLOBUS_SUCCESS;

    GlobusFuncName(globus_soap_message_deserialize_unsignedShort);
    GlobusSoapMessageDebugEnter();

    result = globus_i_soap_message_deserialize_next_content(handle);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            result, &xsd_unsignedShort_qname);
        goto exit;
    }
    
    tmp_value = xmlTextReaderConstValue(handle->reader);
    if(!tmp_value)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            GLOBUS_SUCCESS, &xsd_unsignedShort_qname);
        goto exit;
    }

    if(sscanf(tmp_value, "%hu", val) < 1)
    {
        result = GlobusSoapMessageErrorDeserializeContentFailed(
            &xsd_unsignedShort_qname, tmp_value);
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}
    
globus_result_t
globus_soap_message_deserialize_base64Binary(
    globus_soap_message_handle_t        handle,
    xsd_base64Binary *                  val)
{
    const xmlChar *                     tmp_value;
    int                                 length = 0;
    globus_result_t                     result = GLOBUS_SUCCESS;

    GlobusFuncName(globus_soap_message_deserialize_base64Binary);
    GlobusSoapMessageDebugEnter();

    result = globus_i_soap_message_deserialize_next_content(handle);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            result, &xsd_base64Binary_qname);
        goto exit;
    }
    
    tmp_value = xmlTextReaderConstValue(handle->reader);
    if(!tmp_value)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            GLOBUS_SUCCESS, &xsd_base64Binary_qname);
        goto exit;
    }
    
    length = strlen(tmp_value);

    val->length = (int)ceil((length * 3) / 4) + 1;
    val->value = globus_malloc(val->length);
    if(!val->value)
    {
	result = GlobusSoapMessageErrorOutOfMemory;
	goto exit;
    }
    
    if(xmlBase64Decode(
	tmp_value, 
	(unsigned long *)&length, 
	val->value, 
	(unsigned long *)&val->length) < 0)
    {
	result = GlobusSoapMessageErrorDeserializeContentFailed(
	    &xsd_base64Binary_qname, tmp_value);
	goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_deserialize_hexBinary(
    globus_soap_message_handle_t        handle,
    xsd_hexBinary *                     val)
{
    int                                 res = 0;
    xmlSchemaValPtr                     schema_val = NULL;
    const xmlChar *                     tmp_value;
    globus_result_t                     result = GLOBUS_SUCCESS;

    GlobusFuncName(globus_soap_message_deserialize_hexBinary);
    GlobusSoapMessageDebugEnter();

    result = globus_i_soap_message_deserialize_next_content(handle);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            result, &xsd_hexBinary_qname);
        goto exit;
    }
    
    tmp_value = xmlTextReaderConstValue(handle->reader);
    if(!tmp_value)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            result, &xsd_hexBinary_qname);
        goto exit;
    }

    res = xmlSchemaValidatePredefinedType(
        xmlSchemaGetBuiltInType(XML_SCHEMAS_HEXBINARY),
        tmp_value,
        &schema_val);
    if(res != 0)
    {
        result = GlobusSoapMessageErrorDeserializeContentFailed(
            &xsd_hexBinary_qname, tmp_value);
        goto exit;
    }

    val->value = globus_libc_strdup(schema_val->value.hex.str);
    val->length = schema_val->value.hex.total;

    xmlSchemaFreeValue(schema_val);

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_deserialize_QName(
    globus_soap_message_handle_t        handle,
    xsd_QName *                         val)
{
    xmlChar *                           prefix;
    const xmlChar *                     tmp_value;
    char *                              token;
    globus_result_t                     result = GLOBUS_SUCCESS;
    int                                 start_token;
    int                                 end_token;

    GlobusFuncName(globus_soap_message_deserialize_QName);
    GlobusSoapMessageDebugEnter();

    result = globus_i_soap_message_deserialize_next_content(handle);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            result, &xsd_QName_qname);
        goto exit;
    }
    
    tmp_value = xmlTextReaderConstValue(handle->reader);
    if(!tmp_value)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            result, &xsd_QName_qname);
        goto exit;
    }
    token = globus_l_soap_message_my_strtok(
        tmp_value, strlen(tmp_value), &start_token, &end_token, "\n\t ");
    if (token == NULL)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            result, &xsd_QName_qname);
        goto exit;
    }

    val->local = xmlSplitQName2(token, &prefix);
    if(val->local && prefix)
    {
        val->Namespace = xmlTextReaderLookupNamespace(handle->reader, prefix);
        xmlFree(prefix);
    }
    else if (val->local)
    {
        val->Namespace = NULL;
    }
    else
    {
        val->local = globus_libc_strdup(tmp_value);
    }

    free(token);

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_deserialize_QName_attribute(
    globus_soap_message_handle_t        handle,
    const xsd_QName *                   attr_name,
    xsd_QName *                         val)
{
    xmlChar *                           str = NULL;
    xmlChar *                           prefix;
    const xmlChar *                     attr_ns;
    const xmlChar *                     local_ns;
    globus_result_t                     result = GLOBUS_SUCCESS;

    GlobusFuncName(globus_soap_message_deserialize_QName_attribute);
    GlobusSoapMessageDebugEnter();

    attr_ns = xmlTextReaderConstString(
        handle->reader, attr_name->Namespace);
    local_ns = xmlTextReaderConstNamespaceUri(handle->reader);

    /* is it the default or a null namespace? */
    if((attr_ns != local_ns) && (attr_ns != NULL) && (*attr_ns != '\0'))
    {
        str = xmlTextReaderGetAttributeNs(
            handle->reader, attr_name->local, attr_name->Namespace);
    }
    else
    {
        str = xmlTextReaderGetAttribute(
            handle->reader, attr_name->local);
    }

    if((str == NULL || (*str == '\0')))
    {
        result = GlobusSoapMessageErrorAttributeNotFound(
                handle->verbose,
                attr_name);
        goto exit;
    }
    else
    {
        val->local = xmlSplitQName2(str, &prefix);
        if(prefix)
        {
            val->Namespace = xmlTextReaderLookupNamespace(
                handle->reader, prefix);
            xmlFree(prefix);
        }
        else
        {
            val->Namespace = NULL;
        }
    }

 exit:

    if (str != NULL)
    {
        xmlFree(str);
    }

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_deserialize_QName_list(
    globus_soap_message_handle_t        handle,
    xsd_QName_array *                   inst)
{
    const xmlChar *                     tmp_value;
    globus_list_t *                     qnames;
    globus_list_t *                     qnlist;
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_deserialize_QName_list);
    GlobusSoapMessageDebugEnter();

    result = globus_i_soap_message_deserialize_next_content(handle);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            result, &xsd_QName_qname);
        goto exit;
    }
    
    tmp_value = xmlTextReaderConstValue(handle->reader);
    if(!tmp_value)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            result, &xsd_QName_qname);
        goto exit;
    }

    globus_i_soap_message_split_qnames(tmp_value, &qnames);
    qnlist = qnames;
    while(!globus_list_empty(qnlist))
    {
        char *                          str = globus_list_first(qnlist);
        xmlChar *                       prefix = NULL;
        xsd_QName *                     val;

        val = xsd_QName_array_push(inst);
        
        val->local = xmlSplitQName2(str, &prefix);
        val->Namespace = xmlTextReaderLookupNamespace(
            handle->reader, prefix);

        globus_free(str);
        qnlist = globus_list_rest(qnlist);
    }
    
    globus_list_destroy_all(qnames, globus_libc_free);
    
 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_deserialize_QName_attribute_list(
    globus_soap_message_handle_t        handle,
    const xsd_QName *                   attr_qname,
    xsd_QName_array *                   inst)
{
    const xmlChar *                     attr_ns = NULL;
    const xmlChar *                     local_ns = NULL;
    char *                              tmp_value;
    globus_list_t *                     qnames;
    globus_list_t *                     qnlist;
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_deserialize_QName_attribute_list);
    GlobusSoapMessageDebugEnter();

    attr_ns = xmlTextReaderConstString(
        handle->reader, attr_qname->Namespace);
    local_ns = xmlTextReaderConstNamespaceUri(handle->reader);

    if(attr_ns != local_ns  && (attr_ns != NULL) && (*attr_ns != '\0'))
    {
        tmp_value = xmlTextReaderGetAttributeNs(
            handle->reader, attr_qname->local, attr_qname->Namespace);
    }
    else
    {
        tmp_value = xmlTextReaderGetAttribute(
            handle->reader, attr_qname->local);
    }

    if((!tmp_value) || !(*tmp_value))
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            result, &xsd_QName_qname);
        goto exit;
    }

    globus_i_soap_message_split_qnames(tmp_value, &qnames);
    qnlist = qnames;
    while(!globus_list_empty(qnlist))
    {
        char *                          str = globus_list_first(qnlist);
        xmlChar *                       prefix = NULL;
        xsd_QName *                     val;

        val = xsd_QName_array_push(inst);
        
        val->local = xmlSplitQName2(str, &prefix);
        val->Namespace = xmlTextReaderLookupNamespace(
            handle->reader, prefix);

        globus_free(str);
        qnlist = globus_list_rest(qnlist);
    }
    
    globus_list_destroy_all(qnames, globus_libc_free);
    
 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_deserialize_mark(
    globus_soap_message_handle_t        handle,
    const char *                        idval)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_deserialize_mark);
    GlobusSoapMessageDebugEnter();

    GlobusSoapMessageDebugPrintf(
	GLOBUS_SOAP_MESSAGE_DEBUG_DEBUG,
	("SETTING MARK: %s\n", idval));

    if(handle->xio_handle)
    {
	result = globus_xio_handle_cntl(
	    handle->xio_handle, globus_i_soap_message_buffer_driver,
	    GLOBUS_XIO_BUFFER_SET_READ_MARKER_AT_INDEX,
	    idval,
	    xmlTextReaderReadPosition(handle->reader));
	if(result != GLOBUS_SUCCESS)
	{
	    result = GlobusSoapMessageErrorWithMarker(
		result, "Failed to set read marker: %s", idval);
	    goto exit;
	}
    }

 exit:
    
    GlobusSoapMessageDebugExit();
    return result;
}
