/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */


#include "globus_soap_message_utils.h"
#include "globus_soap_message_handle.h"
#include "globus_soap_message.h"
#include "globus_i_soap_message.h"
#include "globus_soap_message_attrs.h"
#include "xsd_QName.h"
#include "globus_xml_buffer.h"
#include "globus_xio_buffer.h"
#include "globus_xio_http.h"
#include "globus_xio_file_driver.h"
#include "globus_xio.h"

xsd_QName                               soap_env_qname = 
{
    SOAP_ENV_NS,
    SOAP_ENV
};

xsd_QName                               soap_header_qname =
{
    SOAP_ENV_NS,
    SOAP_HEADER
};

xsd_QName                               soap_body_qname =
{
    SOAP_ENV_NS,
    SOAP_BODY
};

xsd_QName                               soap_mustUnderstand_qname =
{
    SOAP_ENV_NS,
    "mustUnderstand"
};

#include "globus_soap_message_fault.h"

xsd_QName                               soap_fault_qname =
{
    SOAP_ENV_NS,
    SOAP_FAULT
};

xsd_QName                               soap_faultcode_qname =
{
    SOAP_ENV_NS,
    SOAP_FAULTCODE
};

xsd_QName                               soap_faultactor_qname =
{
    SOAP_ENV_NS,
    SOAP_FAULTACTOR
};

xsd_QName                               soap_faultstring_qname =
{
    SOAP_ENV_NS,
    SOAP_FAULTSTRING
};

xsd_QName                               soap_fault_detail_qname =
{
    SOAP_ENV_NS,
    SOAP_FAULT_DETAIL
};

void
globus_l_soap_message_error_callback(
    void *                              arg,
    const char *                        msg,
    xmlParserSeverities                 severity,
    xmlTextReaderLocatorPtr             locator);

void
globus_l_soap_message_structured_error_callback(
    void *                              userdata,
    xmlErrorPtr                         error);

globus_result_t
globus_soap_message_handle_init_from_dom(
    globus_soap_message_handle_t *      new_handle,
    xmlDocPtr                           dom)
{
    xmlTextReaderPtr                    reader = NULL;
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_handle_init_from_dom);
    GlobusSoapMessageDebugEnter();

    result = globus_soap_message_handle_init(new_handle, NULL);
    if(result != GLOBUS_SUCCESS)
    {
        goto exit;
    }

    reader = xmlReaderWalker(dom);
    if(!reader)
    {
        result = GlobusSoapMessageErrorInitFailed(
            "Failed to initialize xmlReader");
        goto destroy_handle;
    }
    xmlTextReaderCurrentDoc(reader);

    (*new_handle)->reader = reader;

    goto exit;

 destroy_handle:

    globus_soap_message_handle_destroy(*new_handle);
    *new_handle = NULL;

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_handle_init_from_file(
    globus_soap_message_handle_t *      new_handle,
    const char *                        file_contact)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    globus_xio_handle_t                 xio_handle;
    globus_xio_attr_t                   attr;
    GlobusFuncName(globus_soap_message_handle_init_from_file);
    GlobusSoapMessageDebugEnter();

    result = globus_xio_attr_init(&attr);
    if(result != GLOBUS_SUCCESS)
    {
        goto error_attr;
    }
    result = globus_xio_attr_cntl(
        attr,
        globus_i_soap_message_file_driver,
        GLOBUS_XIO_FILE_SET_FLAGS,
        GLOBUS_XIO_FILE_RDONLY);
    if(result != GLOBUS_SUCCESS)
    {
        goto error_attr_cntl;
    }
    
    result = globus_xio_handle_create(
        &xio_handle, globus_i_soap_message_xio_file_stack);
    if(result != GLOBUS_SUCCESS)
    {
        goto error_handle_create;
    }
    
    result = globus_xio_open(xio_handle, file_contact, attr);
    if(result != GLOBUS_SUCCESS)
    {
        goto error_open;
    }
    
    result = globus_soap_message_handle_init(new_handle, xio_handle);
    if(result != GLOBUS_SUCCESS)
    {
        goto error_init;
    }
    
    /* message handle now owns xio handle */
    xio_handle = NULL;
    
    result = globus_i_soap_message_reset_reader(*new_handle);
    if(result != GLOBUS_SUCCESS)
    {
        goto error_reset;
    }
    
    globus_xio_attr_destroy(attr);
    
    GlobusSoapMessageDebugExit();
    return GLOBUS_SUCCESS;

error_reset:
    globus_soap_message_handle_destroy(*new_handle);
error_init:
error_open:
    if(xio_handle)
    {
        globus_xio_close(xio_handle, NULL);
    }
error_handle_create:
error_attr_cntl:
    globus_xio_attr_destroy(attr);
error_attr:
    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_handle_init_from_memory(
    globus_soap_message_handle_t *      new_handle,
    const char *                        buffer,
    int                                 size)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    xmlTextReaderPtr                    reader = NULL;
    GlobusFuncName(globus_soap_message_handle_init_from_memory);
    GlobusSoapMessageDebugEnter();

    result = globus_soap_message_handle_init(new_handle, NULL);
    if(result != GLOBUS_SUCCESS)
    {
        goto exit;
    }

    reader = xmlReaderForMemory(
        buffer, size, NULL, NULL, 0);
    if(!reader)
    {
        result = GlobusSoapMessageErrorInitFailed(
            "Failed to initialize xmlReader");
        goto destroy_handle;
    }

    xmlTextReaderSetErrorHandler(reader, 
                                 globus_l_soap_message_error_callback, 
                                 (*new_handle));
    
    xmlTextReaderSetStructuredErrorHandler(
        reader,
        globus_l_soap_message_structured_error_callback,
        (*new_handle));

    (*new_handle)->reader = reader;

    goto exit;

 destroy_handle:

    globus_soap_message_handle_destroy(*new_handle);
    *new_handle = NULL;

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_handle_init_to_file(
    globus_soap_message_handle_t *      new_handle,
    const char *                        file_contact,
    globus_xio_file_flag_t              open_flags)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    globus_xio_handle_t                 xio_handle;
    globus_xio_attr_t                   attr;
    globus_soap_message_handle_t        message = NULL;
    GlobusFuncName(globus_soap_message_handle_init_from_file);
    GlobusSoapMessageDebugEnter();

    result = globus_xio_attr_init(&attr);
    if(result != GLOBUS_SUCCESS)
    {
        goto error_attr;
    }
    
    if(open_flags)
    {
        open_flags &= ~(GLOBUS_XIO_FILE_RDONLY|GLOBUS_XIO_FILE_RDWR);
        open_flags |= GLOBUS_XIO_FILE_WRONLY;
    }
    else
    {
        open_flags = GLOBUS_XIO_FILE_CREAT|
            GLOBUS_XIO_FILE_TRUNC|GLOBUS_XIO_FILE_WRONLY;
    }

    result = globus_xio_attr_cntl(
        attr,
        globus_i_soap_message_file_driver,
        GLOBUS_XIO_FILE_SET_FLAGS,
        open_flags);
    if(result != GLOBUS_SUCCESS)
    {
        goto error_attr_cntl;
    }
    
    result = globus_xio_handle_create(
        &xio_handle, globus_i_soap_message_xio_file_stack);
    if(result != GLOBUS_SUCCESS)
    {
        goto error_handle_create;
    }
    
    result = globus_xio_open(xio_handle, file_contact, attr);
    if(result != GLOBUS_SUCCESS)
    {
        goto error_open;
    }
    
    result = globus_soap_message_handle_init(&message, xio_handle);
    if(result != GLOBUS_SUCCESS)
    {
        goto error_init;
    }
    
    /* message handle now owns xio handle */
    xio_handle = NULL;
    
    result = globus_i_soap_message_setup_unbuffered_writer(
        message);
    if(result != GLOBUS_SUCCESS)
    {
        goto error_reset;
    }

    globus_xio_attr_destroy(attr);
    
    *new_handle = message;

    GlobusSoapMessageDebugExit();
    return GLOBUS_SUCCESS;

error_reset:
    globus_soap_message_handle_destroy(message);
error_init:
error_open:
    if(xio_handle)
    {
        globus_xio_close(xio_handle, NULL);
    }
error_handle_create:
error_attr_cntl:
    globus_xio_attr_destroy(attr);
error_attr:
    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_handle_init(
    globus_soap_message_handle_t *      new_handle,
    globus_xio_handle_t                 xio_handle)
{
    globus_soap_message_handle_t        handle = NULL;
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_handle_init);
    GlobusSoapMessageDebugEnter();

    handle = globus_malloc(sizeof(globus_i_soap_message_handle_t));
    if(!handle)
    {
        result = GlobusSoapMessageErrorOutOfMemory;
        goto exit;
    }
    memset(handle, 0, sizeof(globus_i_soap_message_handle_t));
          
    globus_handler_chain_init(&handle->chain);

    globus_soap_message_attr_init(
        &handle->attrs);

    handle->mustUnderstand_hash = NULL;
    
    globus_hashtable_init(&handle->nsprefix_table, 50,
                          globus_hashtable_voidp_hash,
                          globus_hashtable_voidp_keyeq);

    globus_hashtable_init(&handle->c14n_nsprefix_table, 50,
                          globus_hashtable_voidp_hash,
                          globus_hashtable_voidp_keyeq);

    globus_mutex_init(&handle->mutex, NULL);
    globus_mutex_init(&handle->cancel_mutex, NULL);

    handle->dict = xmlDictCreate();
    handle->default_ns = globus_malloc(
        sizeof(globus_i_soap_message_namespace_t));
    memset(handle->default_ns, 0, sizeof(globus_i_soap_message_namespace_t));
    handle->default_ns->ns = NULL;
    handle->default_ns->ref = 1;

    if(xio_handle)
    {
        handle->xio_handle = xio_handle;
    }

    *new_handle = handle;

  exit:

    GlobusSoapMessageDebugExit();
    return result;
}

void
globus_i_soap_message_namespace_destroy(
    globus_i_soap_message_namespace_t * ns_handle)
{
    GlobusFuncName(globus_i_soap_message_namespace_destroy);
    GlobusSoapMessageDebugEnter();

    if(ns_handle)
    {
        if(ns_handle->next)
        {
            globus_i_soap_message_namespace_destroy(ns_handle->next);
        }

        globus_free(ns_handle);
    }
}

globus_result_t
globus_soap_message_handle_reset(
    globus_soap_message_handle_t             handle)
{
    GlobusFuncName(globus_soap_message_handle_reset);
    GlobusSoapMessageDebugEnter();

    if(handle)
    {
        if(handle->attrs)
        {
            globus_soap_message_attr_destroy(handle->attrs);
            handle->attrs = NULL;
            globus_soap_message_attr_init(&handle->attrs);
        }

        if(handle->nsprefix_table)
        {
            globus_hashtable_destroy_all(
                &handle->nsprefix_table, globus_libc_free);
            globus_hashtable_init(&handle->nsprefix_table, 50,
                                  globus_hashtable_voidp_hash,
                                  globus_hashtable_voidp_keyeq);
        }
        
        if(handle->c14n_nsprefix_table)
        {
            globus_hashtable_destroy_all(
                &handle->c14n_nsprefix_table, globus_libc_free);
            globus_hashtable_init(&handle->c14n_nsprefix_table, 50,
                                  globus_hashtable_voidp_hash,
                                  globus_hashtable_voidp_keyeq);
        }

        if(handle->c14n_default_ns)
        {
            globus_i_soap_message_namespace_destroy(handle->c14n_default_ns);
            handle->c14n_default_ns = calloc(
                    1, sizeof(globus_i_soap_message_namespace_t));
        }

        if(handle->default_ns)
        {
            globus_i_soap_message_namespace_destroy(handle->default_ns);
            handle->default_ns = calloc(
                    1, sizeof(globus_i_soap_message_namespace_t));
        }

        if(handle->callout_table)
        {
            globus_hashtable_destroy_all(
                &handle->callout_table, 
                globus_i_soap_message_callouts_destroy);
            handle->callout_table = NULL;
        }

        if(handle->writer)
        {
            xmlTextWriterFlush(handle->writer);
            xmlFreeTextWriter(handle->writer);
            handle->writer = NULL;
        }

        if(handle->reader)
        {
            xmlFreeTextReader(handle->reader);
            handle->reader = NULL;
        }

        handle->nsp_count = 0;
    }
    GlobusSoapMessageDebugExit();
    return GLOBUS_SUCCESS;
}

void
globus_l_soap_message_handle_destroy(
    globus_soap_message_handle_t             handle)
{
    GlobusFuncName(globus_l_soap_message_handle_destroy);
    GlobusSoapMessageDebugEnter();

    if(handle)
    {
        if(handle->dict)
        {
            xmlDictFree(handle->dict);
        }
        
        globus_mutex_destroy(&handle->mutex);
        globus_mutex_destroy(&handle->cancel_mutex);
    
        if(handle->attrs)
        {
            globus_soap_message_attr_destroy(handle->attrs);
        }

        if(handle->nsprefix_table)
        {
            globus_hashtable_destroy_all(
                &handle->nsprefix_table, globus_libc_free);
        }
        
        if(handle->c14n_nsprefix_table)
        {
            globus_hashtable_destroy_all(
                &handle->c14n_nsprefix_table, globus_libc_free);
        }

        if(handle->c14n_default_ns)
        {
            globus_i_soap_message_namespace_destroy(handle->c14n_default_ns);
        }

        if(handle->default_ns)
        {
            globus_i_soap_message_namespace_destroy(handle->default_ns);
        }

        if(handle->writer)
        {
            xmlTextWriterFlush(handle->writer);
            xmlFreeTextWriter(handle->writer);
        }

        if(handle->reader)
        {
            xmlFreeTextReader(handle->reader);
        }

        if(handle->callout_table)
        {
            globus_hashtable_destroy_all(
                &handle->callout_table,
                globus_i_soap_message_callouts_destroy);
            handle->callout_table = NULL;
        }
        if(handle->mustUnderstand_hash != NULL)
        {
            globus_hashtable_destroy_all(
                    &handle->mustUnderstand_hash,
                    xsd_QName_destroy_wrapper);
            handle->mustUnderstand_hash = NULL;
        }
        if (handle->chain != NULL)
        {
            globus_handler_chain_destroy(handle->chain);
        }
    }

    GlobusSoapMessageDebugExit();
}

static
void
globus_l_soap_message_close_callback(
    globus_xio_handle_t                 xio_handle,
    globus_result_t                     result,
    void *                              user_arg)
{
    globus_soap_message_handle_t        handle = user_arg;

    free(handle);
}
/* globus_l_soap_message_close_callback() */

void
globus_soap_message_handle_destroy(
    globus_soap_message_handle_t        handle)
{
    globus_result_t                     result;

    GlobusFuncName(globus_soap_message_handle_destroy);
    GlobusSoapMessageDebugEnter();

    globus_l_soap_message_handle_destroy(handle);

    if(handle)
    {
        if(handle->xio_handle)
        {
            result = globus_xio_register_close(
                    handle->xio_handle,
                    NULL,
                    globus_l_soap_message_close_callback,
                    handle);

            if (result != GLOBUS_SUCCESS)
            {
                free(handle);
            }
        }
        else
        {
            free(handle);
        }
        
    }

    GlobusSoapMessageDebugExit();
}

typedef struct
{
    globus_soap_message_handle_t        handle;
    globus_callback_func_t              callback;
    void *                              args;
} globus_l_soap_message_destroy_callback_handle_t;

static void
globus_l_soap_message_handle_destroy_callback(
    globus_xio_handle_t                 handle,
    globus_result_t                     result,
    void *                              user_arg)
{
    globus_l_soap_message_destroy_callback_handle_t *
    callback_handle = 
    (globus_l_soap_message_destroy_callback_handle_t *) 
                                        user_arg;
    GlobusFuncName(globus_l_soap_message_handle_destroy_callback);
    GlobusSoapMessageDebugEnter();

    globus_l_soap_message_handle_destroy(callback_handle->handle);
    free(callback_handle->handle);

    callback_handle->callback(callback_handle->args);
    
    free(callback_handle);

    GlobusSoapMessageDebugExit();
}

void
globus_soap_message_handle_register_destroy(
    globus_soap_message_handle_t        handle,
    globus_callback_func_t              destroy_callback,
    void *                              destroy_args)
{
    GlobusFuncName(globus_soap_message_handle_register_destroy);
    GlobusSoapMessageDebugEnter();
    
    if(handle->xio_handle)
    {
        globus_l_soap_message_destroy_callback_handle_t * callback_handle;
        callback_handle = 
        (globus_l_soap_message_destroy_callback_handle_t *)
        globus_malloc(sizeof(globus_l_soap_message_destroy_callback_handle_t));

        callback_handle->handle = handle;
        callback_handle->callback = destroy_callback;
        callback_handle->args = destroy_args;

        globus_xio_register_close(
            handle->xio_handle, 
            NULL,
            globus_l_soap_message_handle_destroy_callback,
            callback_handle);
    }
    else
    {
        globus_l_soap_message_handle_destroy(handle);
        free(handle);
        
        globus_callback_register_oneshot(
            NULL,
            &globus_i_reltime_zero,
            destroy_callback,
            destroy_args);
    }

    GlobusSoapMessageDebugExit();
}

globus_result_t
globus_soap_message_set_marker(
    globus_soap_message_handle_t        handle,
    const char *                        marker)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_set_marker);
    GlobusSoapMessageDebugEnter();

    globus_assert_string(handle, "Handle passed to function is NULL");
    globus_assert_string(marker, "marker passed to function is NULL");
    
    if(handle->writer)
    {
        int res = xmlTextWriterWriteRawLen(handle->writer, NULL, 0);
        if(res < 0)
        {
            result = GlobusSoapMessageErrorWithMarker(
                (handle->result ? handle->result : GLOBUS_SUCCESS),
                "Failed to set flush writer before setting marker: %s",
                marker);
            goto exit;
        }

        xmlTextWriterFlush(handle->writer);
        result = globus_xio_handle_cntl(
            handle->xio_handle, globus_i_soap_message_buffer_driver,
            GLOBUS_XIO_BUFFER_SET_MARKER,
            marker);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusSoapMessageErrorWithMarker(
                result, "Failed to set write marker: %s", marker);
            goto exit;
        }
    }
    else if(handle->reader)
    {
        result = globus_i_soap_message_deserialize_next_element(handle);
        if(result != GLOBUS_SUCCESS && 
           !GlobusSoapMessageStatusFailedElementCheck(result))
        {
            result = GlobusSoapMessageErrorWithMarker(
                result, "Failed to set read marker: %s", marker);
            goto exit;
        }

        globus_soap_message_deserialize_push_element(handle);

        result = globus_xio_handle_cntl(
            handle->xio_handle, globus_i_soap_message_buffer_driver,
            GLOBUS_XIO_BUFFER_SET_READ_MARKER_AT_INDEX,
            marker,
            xmlTextReaderReadPosition(handle->reader));
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusSoapMessageErrorWithMarker(
                result, "Failed to set read marker: %s", marker);
            goto exit;
        }
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_get_marked_buffer(
    globus_soap_message_handle_t             handle,
    const char *                        marker,
    char **                             out_buffer,
    size_t *                            out_length)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_get_marked_buffer);
    GlobusSoapMessageDebugEnter();

    globus_assert_string(handle, "Handle passed to function is NULL");
    globus_assert_string(marker, "marker passed to function is NULL");

    result = globus_xio_handle_cntl(handle->xio_handle, 
                                    globus_i_soap_message_buffer_driver,
                                    GLOBUS_XIO_BUFFER_GET_MARKED_BUFFER,
                                    marker,
                                    out_buffer,
                                    out_length);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorWithMarker(
            result, 
            "Failed to get marked buffer: %s",
            marker);
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_get_marked_buffers(
    globus_soap_message_handle_t        handle,
    const char *                        begin_marker,
    const char *                        end_marker,
    char **                             out_buffer,
    size_t *                            out_length)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_get_marked_buffers);
    GlobusSoapMessageDebugEnter();

    globus_assert_string(handle, "Handle passed to function is NULL");

    result = globus_xio_handle_cntl(handle->xio_handle, 
                                    globus_i_soap_message_buffer_driver,
                                    GLOBUS_XIO_BUFFER_GET_MARKED_BUFFERS,
                                    begin_marker,
                                    end_marker,
                                    out_buffer,
                                    out_length);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorWithMarkers(
            result, 
            "Failed to get marked buffers between: %s and: %s",
            begin_marker, end_marker);
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_replace_marked_buffer(
    globus_soap_message_handle_t             handle,
    const char *                        marker,
    char *                              in_buffer,
    size_t                              in_length)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_replace_marked_buffer);
    GlobusSoapMessageDebugEnter();

    globus_assert_string(handle, "Handle passed to function is NULL");
    globus_assert_string(marker, "marker passed to function is NULL");

    result = globus_xio_handle_cntl(handle->xio_handle, 
                                    globus_i_soap_message_buffer_driver,
                                    GLOBUS_XIO_BUFFER_REPLACE_MARKED_BUFFER,
                                    marker,
                                    in_buffer,
                                    in_length);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorWithMarker(
            result, 
            "Failed to set marker: %s",
            marker);
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_remove_marked_buffer(
    globus_soap_message_handle_t             handle,
    const char *                        marker)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_remove_marked_buffer);
    GlobusSoapMessageDebugEnter();

    globus_assert_string(handle, "Handle passed to function is NULL");
    globus_assert_string(marker, "marker passed to function is NULL");

    result = globus_xio_handle_cntl(handle->xio_handle, 
                                    globus_i_soap_message_buffer_driver,
                                    GLOBUS_XIO_BUFFER_REMOVE_MARKED_BUFFER,
                                    marker);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorWithMarker(
            result, 
            "Failed to remove marked buffer: %s",
            marker);
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}



globus_result_t
globus_soap_message_reset_read_position(
    globus_soap_message_handle_t             handle)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_reset_read_position);
    GlobusSoapMessageDebugEnter();

    globus_assert_string(handle, "Handle passed to function is NULL");

    result = globus_xio_handle_cntl(handle->xio_handle, 
                                    globus_i_soap_message_buffer_driver,
                                    GLOBUS_XIO_BUFFER_RESET_READ_POSITION);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorWithMarker(
            result, 
            "Failed to reset read position%s", "");

        goto exit;
    }

    result = globus_i_soap_message_reset_reader(handle);
    if(result != GLOBUS_SUCCESS)
    {
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_set_read_position_to_marker(
    globus_soap_message_handle_t        handle,
    const char *                        marker)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_set_read_position_to_marker);
    GlobusSoapMessageDebugEnter();

    globus_assert_string(handle, "Handle passed to function is NULL");
    globus_assert_string(marker, "marker passed to function is NULL");

    result = globus_xio_handle_cntl(
        handle->xio_handle, 
        globus_i_soap_message_buffer_driver,
        GLOBUS_XIO_BUFFER_SET_READ_POSITION_TO_MARKER,
        marker);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorWithMarker(
            result, 
            "Failed to set read position to marker: %s",
            marker);
        goto exit;
    }

    result = globus_i_soap_message_reset_reader(handle);
    if(result != GLOBUS_SUCCESS)
    {
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_set_write_position_to_marker(
    globus_soap_message_handle_t        handle,
    const char *                        marker)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_set_write_position_to_marker);
    GlobusSoapMessageDebugEnter();

    globus_assert_string(handle, "Handle passed to function is NULL");
    globus_assert_string(marker, "marker passed to function is NULL");

    if(handle->writer)
    {
        xmlTextWriterFlush(handle->writer);
    }

    result = globus_xio_handle_cntl(
        handle->xio_handle, 
        globus_i_soap_message_buffer_driver,
        GLOBUS_XIO_BUFFER_SET_WRITE_POSITION_TO_MARKER,
        marker);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorWithMarker(
            result, 
            "Failed to set write position to marker: %s",
            marker);
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

#include "globus_soap_message_fault.h"

globus_result_t
globus_soap_message_fault_init(
    globus_soap_message_fault_t *       fault)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_fault_init);
    GlobusSoapMessageDebugEnter();

    *fault = globus_malloc(sizeof(struct globus_soap_message_fault_s));
    if(!*fault)
    {
        result = GlobusSoapMessageErrorOutOfMemory;
        goto exit;
    }

    memset(*fault, 0, sizeof(struct globus_soap_message_fault_s));

 exit:
    GlobusSoapMessageDebugExit();
    return result;
}

static
void
globus_soap_message_fault_copy(
    void *                              src,
    void **                             dst)
{
    globus_soap_message_fault_t         src_fault =
    (globus_soap_message_fault_t) src;
    globus_soap_message_fault_t         dest_fault;
    GlobusFuncName(globus_soap_message_fault_copy);
    GlobusSoapMessageDebugEnter();
    
    dest_fault = globus_calloc(1, sizeof(struct globus_soap_message_fault_s));
    if(dest_fault)
    {
        if(src_fault->faultcode)
        {
            dest_fault->faultcode = globus_libc_strdup(src_fault->faultcode);
        }

        if(src_fault->faultstring)
        {
            dest_fault->faultstring =
                globus_libc_strdup(src_fault->faultstring);
        }

        if(src_fault->faultactor)
        {
            dest_fault->faultactor = globus_libc_strdup(src_fault->faultactor);
        }

        if(src_fault->detail)
        {
            xsd_any_copy(&dest_fault->detail, src_fault->detail);
        }
    }
    
    *dst = dest_fault;
    GlobusSoapMessageDebugExit();
}

void
globus_soap_message_fault_destroy(
    void *                              f)
{
    globus_soap_message_fault_t         fault;
    GlobusFuncName(globus_soap_message_fault_destroy);
    GlobusSoapMessageDebugEnter();

    fault = (globus_soap_message_fault_t) f;

    if(fault)
    {
        if(fault->faultcode)
        {
            free(fault->faultcode);
        }

        if(fault->faultstring)
        {
            free(fault->faultstring);
        }

        if(fault->faultactor)
        {
            free(fault->faultactor);
        }

        if(fault->detail)
        {
            xsd_any_destroy(fault->detail);
        }
    
        free(fault);
    }

    GlobusSoapMessageDebugExit();
}

static
char *
globus_l_soap_message_fault_printable(
    globus_object_t *                   error)
{
    globus_soap_message_fault_t         fault;
    const char *                        layout[9];
    char *                              buffer = NULL;
    char *                              str;
    int                                 i = 0;
    globus_bool_t                       newline = GLOBUS_FALSE;
    
    fault = (globus_soap_message_fault_t) 
        globus_object_get_local_instance_data(error);
    
    if(fault->faultcode)
    {
        layout[i++] = "Fault code: ";
        layout[i++] = fault->faultcode;
        newline = GLOBUS_TRUE;
    }
    
    if(fault->faultstring)
    {
        layout[i++] = newline ? "\nFault string: " : "Fault string: ";
        layout[i++] = fault->faultstring;
        newline = GLOBUS_TRUE;
    }
    
    if(fault->faultactor)
    {
        layout[i++] = newline ? "\nFault actor: " : "Fault actor: ";
        layout[i++] = fault->faultactor;
        newline = GLOBUS_TRUE;
    }
    
    if(globus_i_error_verbose && fault->detail &&
        fault->detail->any_info == &globus_xml_buffer_contents_info)
    {
        globus_xml_buffer *             xml_buffer;
        
        xml_buffer = (globus_xml_buffer *) fault->detail->value;
        if(xml_buffer->length > 0)
        {
            buffer = (char *) globus_malloc(
                sizeof(char) * xml_buffer->length + 1);
            if(buffer)
            {
                memcpy(buffer, xml_buffer->buffer, xml_buffer->length);
                buffer[xml_buffer->length] = 0;
               layout[i++] = newline ? "\nFault detail:\n" : "Fault detail:\n";
                layout[i++] = buffer;
            }
        }
    }
    
    str = globus_libc_join(layout, i);
        
    if(buffer)
    {
        globus_free(buffer);
    }
    
    return str;
}

globus_object_type_t                    globus_l_soap_fault_type =
{
    GLOBUS_ERROR_TYPE_BASE,
    globus_soap_message_fault_copy,
    globus_soap_message_fault_destroy,
    globus_l_soap_message_fault_printable
};

globus_object_t *
globus_soap_message_fault_object_initialize(
    globus_soap_message_fault_t         fault)
{
    globus_object_t *                   obj;
    GlobusFuncName(globus_soap_message_fault_object_initialize);
    GlobusSoapMessageDebugEnter();

    obj = globus_object_construct(GLOBUS_SOAP_FAULT_TYPE);
    globus_object_set_local_instance_data(obj, fault);
    obj = globus_error_initialize_base(obj, GLOBUS_SOAP_MESSAGE_MODULE, NULL);
    
    GlobusSoapMessageDebugExit();
    return obj;
}

globus_bool_t
globus_soap_message_is_fault(
    globus_object_t *                   obj)
{
    globus_object_t *                   errobj;
    globus_bool_t                       res = GLOBUS_FALSE;
    GlobusFuncName(globus_soap_message_is_fault);
    GlobusSoapMessageDebugEnter();

    errobj = obj;
    while(globus_object_get_type(errobj) == GLOBUS_ERROR_TYPE_GLOBUS &&
          globus_error_get_cause(errobj))
    {
        errobj = globus_error_get_cause(errobj);
    }
    
    if(globus_object_get_type(errobj) == GLOBUS_SOAP_FAULT_TYPE)
    {
        res = GLOBUS_TRUE;
    }
    
    GlobusSoapMessageDebugExit();
    return res;
}
    

globus_soap_message_fault_t
globus_soap_message_fault_get(
    globus_object_t *                   obj)
{
    globus_object_t *                   errobj;
    globus_soap_message_fault_t         soap_fault = NULL;
    globus_object_t *                   parentobj = NULL;
    GlobusFuncName(globus_soap_message_fault_get);
    GlobusSoapMessageDebugEnter();

    errobj = obj;
    while(globus_object_get_type(errobj) == GLOBUS_ERROR_TYPE_GLOBUS &&
          globus_error_get_cause(errobj))
    {
        parentobj = errobj;
        errobj = globus_error_get_cause(errobj);
    }
    
    if(globus_object_get_type(errobj) == GLOBUS_SOAP_FAULT_TYPE)
    {
        soap_fault = errobj->instance_data;
        errobj->instance_data = NULL;
        globus_object_free(errobj);
        if(parentobj)
        {
            globus_error_set_cause(
                parentobj, (globus_object_t *)GLOBUS_FAILURE);
        }
    }
    
    GlobusSoapMessageDebugExit();
    return soap_fault;
}


    
void
globus_soap_message_handle_set_registry(
    globus_soap_message_handle_t        handle,
    globus_xsd_type_registry_t          registry)
{
    GlobusFuncName(globus_soap_message_handle_set_registry);
    GlobusSoapMessageDebugEnter();

    handle->registry = registry;

    GlobusSoapMessageDebugExit();
}

void
globus_soap_message_handle_get_registry(
    globus_soap_message_handle_t        handle,
    globus_xsd_type_registry_t *        registry)
{
    GlobusFuncName(globus_soap_message_handle_get_registry);
    GlobusSoapMessageDebugEnter();

    *registry = handle->registry;

    GlobusSoapMessageDebugExit();
}
    
globus_result_t
globus_soap_message_get_handler_chain(
    globus_soap_message_handle_t        handle,
    globus_handler_chain_t *            chain)
{
    GlobusFuncName(globus_soap_messager_get_handler_chain);
    GlobusSoapMessageDebugEnter();

    *chain = handle->chain;

    GlobusSoapMessageDebugExit();
    return GLOBUS_SUCCESS;
}

globus_result_t
globus_soap_message_set_handler_chain(
    globus_soap_message_handle_t        handle,
    globus_handler_chain_t              chain)
{
    GlobusFuncName(globus_soap_messager_set_handler_chain);
    GlobusSoapMessageDebugEnter();

    if(handle->chain)
    {
        globus_handler_chain_destroy(handle->chain);
    }

    globus_handler_chain_copy(&handle->chain, chain);

    GlobusSoapMessageDebugExit();
    return GLOBUS_SUCCESS;
}
    
globus_result_t
globus_soap_message_get_transport_handle(
    globus_soap_message_handle_t        message,
    globus_xio_handle_t *               handle)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_get_transport_handle);
    GlobusSoapMessageDebugEnter();

    *handle = message->xio_handle;

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_add_callout(
    globus_soap_message_handle_t        message,
    const xsd_QName *                   type,
    globus_soap_message_callout_func_t  begin_callout,
    globus_soap_message_callout_func_t  end_callout,
    globus_soap_message_callout_func_t  attr_callout)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    globus_i_soap_message_callouts_t *  callouts = NULL;
    globus_i_soap_message_callout_entry_t * entry;
    GlobusFuncName(globus_soap_message_add_callout);
    GlobusSoapMessageDebugEnter();

    if(!message->callout_table)
    {
        globus_hashtable_init(
            &message->callout_table,
            20,
            xsd_QName_hash,
            xsd_QName_keyeq);
    }
    
    entry = globus_malloc(sizeof(globus_i_soap_message_callout_entry_t));
    if(entry == NULL)
    {
        result = GlobusSoapMessageErrorOutOfMemory;
        goto exit;
    }
    memset(entry, 0, sizeof(globus_i_soap_message_callout_entry_t));

    entry->begin_callout = begin_callout;
    entry->end_callout = end_callout;
    entry->attr_callout = attr_callout;

    callouts = (globus_i_soap_message_callouts_t *)
    globus_hashtable_lookup(&message->callout_table, (void *)type);
    if(callouts == NULL)
    {
        callouts = globus_malloc(sizeof(globus_i_soap_message_callouts_t));
        if(callouts == NULL)
        {
            result = GlobusSoapMessageErrorOutOfMemory;
            goto exit;
        }
        memset(callouts, 0, sizeof(globus_i_soap_message_callouts_t));

        xsd_QName_copy(&callouts->type, type);

        globus_list_insert(&callouts->entries, entry);

        globus_hashtable_insert(
            &message->callout_table, (void *)type, callouts);
    }
    else
    {
        globus_list_insert(&callouts->entries, entry);
    }

exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_invoke_callouts(
    globus_soap_message_handle_t         handle,
    const xsd_QName *                    element_qname,
    globus_soap_message_callout_type_t   type)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    globus_i_soap_message_callout_entry_t * entry;
    globus_list_t *                     entries;
    globus_i_soap_message_callouts_t *  callouts;
    GlobusFuncName(globus_i_soap_message_call_begin_callouts);
    GlobusSoapMessageDebugEnter();
    
    if(!handle->callout_table)
    {
        goto exit;
    }

    callouts = globus_hashtable_lookup(
        &handle->callout_table, 
        (void *)element_qname);
    if(callouts)
    {
        entries = callouts->entries;
        while(entries)
        {
            entry = globus_list_first(entries);

            switch(type)
            {
            case GLOBUS_SOAP_MESSAGE_CALLOUT_BEGIN:
                if(entry->begin_callout)
                {
                    result = entry->begin_callout(handle, element_qname);
                }
                break;

            case GLOBUS_SOAP_MESSAGE_CALLOUT_END:
                if(entry->end_callout)
                {
                    result = entry->end_callout(handle, element_qname);
                }
                break;

            case GLOBUS_SOAP_MESSAGE_CALLOUT_ATTR:
                if(entry->attr_callout)
                {
                    result = entry->attr_callout(handle, element_qname);
                }
                break;

            default:
                break;
            }

            if (result != GLOBUS_SUCCESS)
            {
                break;
            }

            entries = globus_list_rest(entries);
        }
    }

 exit:
    GlobusSoapMessageDebugExit();
    return result;
}


void    
globus_soap_message_remove_callouts(
    globus_soap_message_handle_t        message,
    const xsd_QName *                   type)
{
    globus_i_soap_message_callouts_t *  callouts;
    GlobusFuncName(globus_soap_message_remove_callouts);
    GlobusSoapMessageDebugEnter();

    if(message->callout_table)
    {
	callouts = globus_hashtable_remove(&message->callout_table, (void *)type);
	if(callouts)
	{
	    globus_i_soap_message_callouts_destroy(
		callouts);
	}
    }

    GlobusSoapMessageDebugExit();
}

void
globus_i_soap_message_callouts_destroy(
    void *                              vcallouts)
{
    globus_i_soap_message_callouts_t *  callouts = 
    (globus_i_soap_message_callouts_t *) vcallouts;

    globus_list_destroy_all(
        callouts->entries, 
        globus_libc_free);
    
    xsd_QName_destroy(callouts->type);

    globus_free(callouts);
}
