/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */


#include "globus_common.h"
#include "version.h"
#include "libxml/xmlschemastypes.h"
#include "globus_xio.h"
#include "globus_xio_buffer.h"
#include "globus_i_soap_message.h"
#include "globus_soap_message_utils.h"
#include "globus_xsd_type_registry.h"

globus_xsd_type_registry_t              GLOBUS_XSD_BASE_REGISTRY = NULL;
globus_xsd_type_registry_t              GLOBUS_GLOBAL_TYPE_REGISTRY = NULL;

static int
globus_l_soap_message_activate();

static int
globus_l_soap_message_deactivate();

globus_module_descriptor_t
globus_i_soap_message_module =
{
    "globus_soap_message_module",
    globus_l_soap_message_activate,
    globus_l_soap_message_deactivate,
    NULL,
    NULL,
    &local_version
};

GlobusDebugDefine(GLOBUS_SOAP_MESSAGE);

globus_xio_stack_t                  globus_i_soap_message_xio_stack;
globus_xio_driver_t                 globus_i_soap_message_tcp_driver;
globus_xio_driver_t                 globus_i_soap_message_http_driver;
globus_xio_driver_t                 globus_i_soap_message_buffer_driver;

globus_xio_stack_t                  globus_i_soap_message_xio_file_stack;
globus_xio_driver_t                 globus_i_soap_message_file_driver;

globus_xio_stack_t                  globus_i_soap_message_xio_https_stack;
globus_xio_driver_t                 globus_i_soap_message_gsi_driver;


static int
globus_l_soap_message_activate()
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    int                                 rc = 0;
    GlobusFuncName(globus_l_soap_message_activate);
    
    globus_i_soap_message_xio_stack = NULL;
    globus_i_soap_message_tcp_driver = NULL;
    globus_i_soap_message_http_driver = NULL;
    globus_i_soap_message_buffer_driver = NULL;

    globus_i_soap_message_xio_file_stack = NULL;
    globus_i_soap_message_file_driver = NULL;

    globus_i_soap_message_xio_https_stack = NULL;
    globus_i_soap_message_gsi_driver = NULL;

    rc = globus_module_activate(GLOBUS_COMMON_MODULE);
    if(rc != GLOBUS_SUCCESS)
    {
        return rc;
    }

    GlobusDebugInit(GLOBUS_SOAP_MESSAGE, 
                    MESSAGES INFO DEBUG TRACE WARN REGISTRY WILDCARDS);
    GlobusSoapMessageDebugEnter();

    rc = globus_module_activate(GLOBUS_XIO_MODULE);
    if(rc != GLOBUS_SUCCESS)
    {
        return rc;
    }

    result = globus_xio_stack_init(
        &globus_i_soap_message_xio_stack, NULL);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorFailedTransport(
            result, "Failed xio stack initialization");
        goto error;
    }

    result = globus_xio_driver_load(
        "tcp", &globus_i_soap_message_tcp_driver);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorFailedTransport(
            result, "Failed to load tcp driver");
        goto error;
    }
    
    result = globus_xio_stack_push_driver(
        globus_i_soap_message_xio_stack, globus_i_soap_message_tcp_driver);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorFailedTransport(
            result, "Failed to push tcp driver onto stack");
        goto error;
    }
    
    result = globus_xio_driver_load(
        "http", &globus_i_soap_message_http_driver);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorFailedTransport(
            result, "Failed to load http driver");
        goto error;
    }
    
    result = globus_xio_driver_load(
        "buffer", &globus_i_soap_message_buffer_driver);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorFailedTransport(
            result, "Failed to load buffer driver");
        goto error;
    }
    
    result = globus_xio_stack_push_driver(
        globus_i_soap_message_xio_stack, globus_i_soap_message_http_driver);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorFailedTransport(
            result, "Failed to push http driver");
        goto error;
    }
    
    result = globus_xio_stack_push_driver(
        globus_i_soap_message_xio_stack, globus_i_soap_message_buffer_driver);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorFailedTransport(
            result, "Failed to push buffer driver");
        goto error;
    }
    
    result = globus_xio_stack_init(
        &globus_i_soap_message_xio_file_stack, NULL);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorFailedTransport(
            result, "Failed xio stack initialization");
        goto error;
    }

    result = globus_xio_driver_load(
        "file", &globus_i_soap_message_file_driver);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorFailedTransport(
            result, "Failed to load file driver");
        goto error;
    }
    
    result = globus_xio_stack_push_driver(
        globus_i_soap_message_xio_file_stack,
        globus_i_soap_message_file_driver);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorFailedTransport(
            result, "Failed to push file driver onto stack");
        goto error;
    }

    result = globus_xio_stack_init(
        &globus_i_soap_message_xio_https_stack, NULL);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorFailedTransport(
            result, "Failed xio stack initialization");
        goto error;
    }

    result = globus_xio_driver_load(
        "gsi", &globus_i_soap_message_gsi_driver);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorFailedTransport(
            result, "Failed to load gsi driver");
        goto error;
    }
    
    result = globus_xio_stack_push_driver(
        globus_i_soap_message_xio_https_stack,
        globus_i_soap_message_tcp_driver);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorFailedTransport(
            result, "Failed to push gsi driver onto stack");
        goto error;
    }

    result = globus_xio_stack_push_driver(
        globus_i_soap_message_xio_https_stack,
        globus_i_soap_message_gsi_driver);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorFailedTransport(
            result, "Failed to push gsi driver onto stack");
        goto error;
    }

    result = globus_xio_stack_push_driver(
        globus_i_soap_message_xio_https_stack,
        globus_i_soap_message_http_driver);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorFailedTransport(
            result, "Failed to push gsi driver onto stack");
        goto error;
    }

    result = globus_xio_stack_push_driver(
        globus_i_soap_message_xio_https_stack,
        globus_i_soap_message_buffer_driver);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorFailedTransport(
            result, "Failed to push gsi driver onto stack");
        goto error;
    }

    xmlSchemaInitTypes();

    globus_xsd_type_registry_init(&GLOBUS_XSD_BASE_REGISTRY);
    globus_xsd_type_registry_add_base_types(GLOBUS_XSD_BASE_REGISTRY);
     
    globus_xsd_type_registry_init(&GLOBUS_GLOBAL_TYPE_REGISTRY);
    globus_xsd_type_registry_add_base_types(GLOBUS_GLOBAL_TYPE_REGISTRY);

    goto exit;

 error:

    if(globus_i_soap_message_xio_stack)
    {
        globus_xio_stack_destroy(globus_i_soap_message_xio_stack);
    }

    if(globus_i_soap_message_buffer_driver)
    {
        globus_xio_driver_unload(globus_i_soap_message_buffer_driver);
    }

    if(globus_i_soap_message_http_driver)
    {
        globus_xio_driver_unload(globus_i_soap_message_http_driver);
    }

    if(globus_i_soap_message_tcp_driver)
    {
        globus_xio_driver_unload(globus_i_soap_message_tcp_driver);
    }
    
    if(globus_i_soap_message_file_driver)
    {
        globus_xio_driver_unload(globus_i_soap_message_file_driver);
    }
    
    if(globus_i_soap_message_xio_file_stack)
    {
        globus_xio_stack_destroy(globus_i_soap_message_xio_file_stack);
    }

    if(globus_i_soap_message_gsi_driver)
    {
        globus_xio_driver_unload(globus_i_soap_message_gsi_driver);
    }

    if(globus_i_soap_message_xio_https_stack)
    {
        globus_xio_stack_destroy(globus_i_soap_message_xio_https_stack);
    }

 exit:

    GlobusSoapMessageDebugExit();
    return (int) result;
}

static int
globus_l_soap_message_deactivate()
{
    int                                 rc = 0;
    GlobusFuncName(globus_l_soap_message_deactivate);
    GlobusSoapMessageDebugEnter();

    if(GLOBUS_XSD_BASE_REGISTRY)
    {
        globus_xsd_type_registry_destroy(GLOBUS_XSD_BASE_REGISTRY);
        GLOBUS_XSD_BASE_REGISTRY = NULL;
    }
 
    if(GLOBUS_GLOBAL_TYPE_REGISTRY)
    {
        globus_xsd_type_registry_destroy(GLOBUS_GLOBAL_TYPE_REGISTRY);
        GLOBUS_GLOBAL_TYPE_REGISTRY = NULL;
    }

    xmlSchemaCleanupTypes();
    
    if(globus_i_soap_message_file_driver)
    {
        globus_xio_driver_unload(globus_i_soap_message_file_driver);
    }

    if(globus_i_soap_message_xio_https_stack)
    {
        globus_xio_stack_destroy(globus_i_soap_message_xio_https_stack);
    }
    
    if(globus_i_soap_message_xio_file_stack)
    {
        globus_xio_stack_destroy(globus_i_soap_message_xio_file_stack);
    }
    
    if(globus_i_soap_message_xio_stack)
    {
        globus_xio_stack_destroy(globus_i_soap_message_xio_stack);
    }
    
    if(globus_i_soap_message_buffer_driver)
    {
        globus_xio_driver_unload(globus_i_soap_message_buffer_driver);
    }

    if(globus_i_soap_message_http_driver)
    {
        globus_xio_driver_unload(globus_i_soap_message_http_driver);
    }

    if(globus_i_soap_message_tcp_driver)
    {
        globus_xio_driver_unload(globus_i_soap_message_tcp_driver);
    }

    if(globus_i_soap_message_gsi_driver)
    {
        globus_xio_driver_unload(globus_i_soap_message_gsi_driver);
    }

    rc = globus_module_deactivate(GLOBUS_XIO_MODULE);
    if(rc != GLOBUS_SUCCESS)
    {
        return rc;
    }

    rc = globus_module_deactivate(GLOBUS_COMMON_MODULE);
    if(rc != GLOBUS_SUCCESS)
    {
        return rc;
    }

    return GLOBUS_SUCCESS;
}
