/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */


#include "globus_soap_message.h"
#include "globus_i_soap_message.h"
#include "globus_soap_message_handle.h"
#include "globus_soap_message_fault.h"
#include "globus_soap_message_markers.h"
#include "openssl/evp.h"
#include "openssl/buffer.h"
#include "xsd_anyAttributes.h"

const char                              _prefix_table[] =
    "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";

#define GLOBUS_I_SOAP_MESSAGE_GET_UNIQUE_PREFIX(HANDLE, PREFIX)         \
    {                                                                   \
        globus_assert(HANDLE->nsp_count < (62*62));                     \
        PREFIX[2] = _prefix_table[(int)(HANDLE->nsp_count / 62)];       \
        PREFIX[3] = _prefix_table[(int)(HANDLE->nsp_count % 62)];       \
        HANDLE->nsp_count++;                                            \
    }

#define GLOBUS_I_SOAP_MESSAGE_DEFAULT_NS(HANDLE)                        \
    (HANDLE->c14n_subset ?                                              \
     (HANDLE->c14n_default_ns ? HANDLE->c14n_default_ns->ns : NULL) :   \
     (HANDLE->default_ns ? HANDLE->default_ns->ns : NULL))

#define GLOBUS_I_SOAP_MESSAGE_NS_IS_DEFAULT(HANDLE, NS) \
    (HANDLE->c14n_subset ?                              \
     (((HANDLE->c14n_default_ns == NULL && !NS) ||      \
       (HANDLE->c14n_default_ns &&                      \
        HANDLE->c14n_default_ns->ns == NS)) ? 1 : 0) :  \
     (((HANDLE->default_ns == NULL && !NS) ||           \
       (HANDLE->default_ns &&                           \
        HANDLE->default_ns->ns == NS)) ? 1 : 0))

#define GLOBUS_I_SOAP_MESSAGE_NS_IS_NULL(HANDLE)        \
    (HANDLE->c14n_subset ?                              \
     ((HANDLE->c14n_default_ns->ns == NULL) ? 1 : 0) :  \
     ((HANDLE->default_ns->ns == NULL) ? 1 : 0))

#define GLOBUS_I_SOAP_MESSAGE_NS_SET_DEFAULT(HANDLE, NS)                \
    {                                                                   \
        globus_i_soap_message_namespace_t * NAMESPACE_REF;              \
        NAMESPACE_REF = globus_malloc(                                  \
            sizeof(globus_i_soap_message_namespace_t));                 \
        globus_assert(NAMESPACE_REF);                                   \
                                                                        \
        NAMESPACE_REF->ns = xmlDictLookup(HANDLE->dict, NS, -1);        \
        NAMESPACE_REF->ref = 0;                                         \
                                                                        \
        if(HANDLE->c14n_subset)                                         \
        {                                                               \
            NAMESPACE_REF->next = HANDLE->c14n_default_ns;              \
            HANDLE->c14n_default_ns = NAMESPACE_REF;                    \
        }                                                               \
        else                                                            \
        {                                                               \
            NAMESPACE_REF->next = HANDLE->default_ns;                   \
            HANDLE->default_ns = NAMESPACE_REF;                         \
        }                                                               \
    }

#define GLOBUS_I_SOAP_MESSAGE_DEFAULT_NS_REFINC(HANDLE) \
    {                                                   \
        if(HANDLE->c14n_subset)                         \
        {                                               \
            HANDLE->c14n_default_ns->ref++;             \
        }                                               \
        else                                            \
        {                                               \
            HANDLE->default_ns->ref++;                  \
        }                                               \
    }

#define GLOBUS_I_SOAP_MESSAGE_DEFAULT_NS_REFDEC(HANDLE) \
    {                                                   \
        if(HANDLE->c14n_subset)                         \
        {                                               \
            if(HANDLE->c14n_default_ns->ref > 0)        \
                HANDLE->c14n_default_ns->ref--;         \
            if(HANDLE->c14n_default_ns->ref < 0)        \
            {                                           \
                globus_i_soap_message_namespace_t * ns; \
                ns = HANDLE->c14n_default_ns;           \
                HANDLE->c14n_default_ns = ns->next;     \
                globus_free(ns);                        \
            }                                           \
        }                                               \
        else                                            \
        {                                               \
            if(HANDLE->default_ns->ref > 0)             \
                HANDLE->default_ns->ref--;              \
            if(HANDLE->default_ns->ref <= 0)            \
            {                                           \
                globus_i_soap_message_namespace_t * ns; \
                ns = HANDLE->default_ns;                \
                HANDLE->default_ns = ns->next;          \
                globus_free(ns);                        \
            }                                           \
                                                        \
        }                                               \
    }

#define GLOBUS_I_SOAP_MESSAGE_DEFAULT_NS_REFCOUNT(HANDLE, REFCOUNT)     \
    {                                                                   \
        if(HANDLE->c14n_subset)                                         \
        {                                                               \
                REFCOUNT = HANDLE->c14n_default_ns->ref;                \
        }                                                               \
        else                                                            \
        {                                                               \
                REFCOUNT = HANDLE->default_ns->ref;                     \
                                                                        \
        }                                                               \
    }

#define GLOBUS_I_SOAP_MESSAGE_NSPREFIX_LOOKUP(HANDLE, NS, PREFIX)       \
    {                                                                   \
        const xmlChar * INTERN_NS;                                      \
        globus_l_soap_message_prefix_ref_t * PREFIX_REF;                \
        INTERN_NS = (xmlChar *) xmlDictLookup(                          \
            HANDLE->dict, (const xmlChar *) NS, -1);                    \
        if(HANDLE->c14n_subset)                                         \
        {                                                               \
            PREFIX_REF = globus_hashtable_lookup(                       \
                &HANDLE->c14n_nsprefix_table,                           \
                (void *) INTERN_NS);                                    \
        }                                                               \
        else                                                            \
        {                                                               \
            PREFIX_REF = globus_hashtable_lookup(                       \
                &HANDLE->nsprefix_table,                                \
                (void *) INTERN_NS);                                    \
        }                                                               \
        PREFIX = (xmlChar *)(PREFIX_REF ? PREFIX_REF->prefix : NULL);   \
    }

#define GLOBUS_I_SOAP_MESSAGE_NSPREFIX_REFCOUNT(HANDLE, NS, REFCOUNT)   \
    {                                                                   \
        globus_l_soap_message_prefix_ref_t * PREFIX_REF;                \
        const xmlChar * INTERN_NS;                                      \
        INTERN_NS = (xmlChar *) xmlDictLookup(                          \
            HANDLE->dict, (const xmlChar *) NS, -1);                    \
        if(HANDLE->c14n_subset)                                         \
        {                                                               \
            PREFIX_REF = globus_hashtable_lookup(                       \
                &HANDLE->c14n_nsprefix_table,                           \
                (void *) INTERN_NS);                                    \
        }                                                               \
        else                                                            \
        {                                                               \
            PREFIX_REF = globus_hashtable_lookup(                       \
                &HANDLE->nsprefix_table,                                \
                (void *) INTERN_NS);                                    \
        }                                                               \
        REFCOUNT = (PREFIX_REF ? PREFIX_REF->ref : 0);                  \
    }
                                                                          
#define GLOBUS_I_SOAP_MESSAGE_NSPREFIX_REFINC(HANDLE, NS)       \
    {                                                           \
        globus_l_soap_message_prefix_ref_t * PREFIX_REF;        \
        if(HANDLE->c14n_subset)                                 \
        {                                                       \
            PREFIX_REF = globus_hashtable_lookup(               \
                &HANDLE->c14n_nsprefix_table,                   \
                (void *) NS);                                   \
        }                                                       \
        else                                                    \
        {                                                       \
            PREFIX_REF = globus_hashtable_lookup(               \
                &HANDLE->nsprefix_table,                        \
                (void *) NS);                                   \
        }                                                       \
                                                                \
        globus_assert(PREFIX_REF);                              \
        PREFIX_REF->ref++;                                      \
    }

#define GLOBUS_I_SOAP_MESSAGE_NSPREFIX_REFDEC(HANDLE, NS)               \
    {                                                                   \
        globus_l_soap_message_prefix_ref_t * PREFIX_REF;                \
        if(HANDLE->c14n_subset)                                         \
        {                                                               \
            PREFIX_REF = globus_hashtable_lookup(                       \
                &HANDLE->c14n_nsprefix_table,                           \
                (void *) NS);                                           \
        }                                                               \
        else                                                            \
        {                                                               \
            PREFIX_REF = globus_hashtable_lookup(                       \
                &HANDLE->nsprefix_table,                                \
                (void *) NS);                                           \
        }                                                               \
        if(PREFIX_REF && (PREFIX_REF->ref > 0)) PREFIX_REF->ref--;      \
    }

#define GLOBUS_I_SOAP_MESSAGE_REFINC(HANDLE, NS)                        \
    {                                                                   \
        const xmlChar *                 INTERN_NS = xmlDictLookup(      \
            HANDLE->dict, (const xmlChar *) NS, -1);                    \
        if(GLOBUS_I_SOAP_MESSAGE_NS_IS_DEFAULT(HANDLE, INTERN_NS))      \
        {                                                               \
            GLOBUS_I_SOAP_MESSAGE_DEFAULT_NS_REFINC(HANDLE);            \
        }                                                               \
        else if(INTERN_NS)                                              \
        {                                                               \
            GLOBUS_I_SOAP_MESSAGE_NSPREFIX_REFINC(HANDLE, INTERN_NS);   \
        }                                                               \
    }

#define GLOBUS_I_SOAP_MESSAGE_REFDEC(HANDLE, NS)                        \
    {                                                                   \
        const xmlChar *                 INTERN_NS = xmlDictLookup(      \
            HANDLE->dict, (const xmlChar *) NS, -1);                    \
        if(GLOBUS_I_SOAP_MESSAGE_NS_IS_DEFAULT(HANDLE, INTERN_NS))      \
        {                                                               \
            GLOBUS_I_SOAP_MESSAGE_DEFAULT_NS_REFDEC(HANDLE);            \
        }                                                               \
        else if(INTERN_NS)                                              \
        {                                                               \
            GLOBUS_I_SOAP_MESSAGE_NSPREFIX_REFDEC(HANDLE, INTERN_NS);   \
        }                                                               \
    }
                                                                          
#define GLOBUS_I_SOAP_MESSAGE_NSPREFIX_ADD(HANDLE, NS, PREFIX)          \
    {                                                                   \
        const xmlChar * INTERN_NS;                                      \
        const xmlChar * INTERN_PREFIX;                                  \
        globus_l_soap_message_prefix_ref_t * PREFIX_REF;                \
        PREFIX_REF = globus_malloc(                                     \
            sizeof(globus_l_soap_message_prefix_ref_t));                \
        globus_assert(PREFIX_REF);                                      \
                                                                        \
        INTERN_NS = xmlDictLookup(HANDLE->dict, (const xmlChar *) NS, -1); \
        INTERN_PREFIX = xmlDictLookup(                                  \
                HANDLE->dict,                                           \
                (const xmlChar *) PREFIX,                               \
                -1);                                                    \
        PREFIX_REF->prefix = INTERN_PREFIX;                             \
        PREFIX_REF->ref = 0;                                            \
                                                                        \
        if(HANDLE->c14n_subset)                                         \
        {                                                               \
            globus_hashtable_insert(&HANDLE->c14n_nsprefix_table,       \
                                    (void *) INTERN_NS,                 \
                                    (void *) PREFIX_REF);               \
        }                                                               \
        else                                                            \
        {                                                               \
            globus_hashtable_insert(&HANDLE->nsprefix_table,            \
                                    (void *) INTERN_NS,                 \
                                    (void *) PREFIX_REF);               \
        }                                                               \
    }

#define GLOBUS_I_SOAP_MESSAGE_GET_ATTR_PREFIX(HANDLE, ATTR_QNAME, PREFIX) \
    if(ATTR_QNAME && ATTR_QNAME->Namespace)                               \
    {                                                                     \
        GLOBUS_I_SOAP_MESSAGE_NSPREFIX_LOOKUP(                            \
            HANDLE, ATTR_QNAME->Namespace, PREFIX);                       \
    }                                                                     \
    else                                                                  \
    {                                                                     \
        PREFIX = NULL;                                                    \
    }

#define GLOBUS_I_SOAP_MESSAGE_GET_ELEMENT_PREFIX(HANDLE, QNAME, PREFIX)    \
    if(QNAME && QNAME->Namespace)                                          \
    {                                                                      \
        const xmlChar * NS_REF = xmlDictLookup(                            \
            HANDLE->dict, (const xmlChar *) QNAME->Namespace, -1);         \
        if(GLOBUS_I_SOAP_MESSAGE_NS_IS_DEFAULT(HANDLE, NS_REF))            \
        {                                                                  \
            PREFIX = NULL;                                                 \
        }                                                                  \
        else                                                               \
        {                                                                  \
            GLOBUS_I_SOAP_MESSAGE_NSPREFIX_LOOKUP(HANDLE, NS_REF, PREFIX); \
        }                                                                  \
    }                                                                      \
    else                                                                   \
    {                                                                      \
        PREFIX = NULL;                                                     \
    }

#define GLOBUS_I_SOAP_MESSAGE_DEFINE_PREFIX(HANDLE, NS, PREFIX) \
    ((xmlTextWriterWriteAttributeNS(                            \
        handle->writer,                                         \
        (const xmlChar *) "xmlns",                              \
        PREFIX,                                                 \
        NULL, (const xmlChar *) NS) < 0) ?                      \
                                                                \
        (GlobusSoapMessageErrorSerializeNamespaceFailed(        \
            NS, PREFIX)) :                                      \
        GLOBUS_SUCCESS)

#define GLOBUS_I_SOAP_MESSAGE_DEFINE_DEFAULT_NS(HANDLE, NS)     \
    ((xmlTextWriterWriteAttributeNS(                            \
        handle->writer,                                         \
        NULL,                                                   \
        (const xmlChar *) "xmlns",                              \
        NULL, NS) < 0) ?                                        \
                                                                \
        (GlobusSoapMessageErrorSerializeNamespaceFailed(        \
            NS, NULL)) :                                        \
        GLOBUS_SUCCESS)
 
#define GLOBUS_I_SOAP_MESSAGE_DEFINE_NULL_NS(HANDLE)            \
    ((xmlTextWriterWriteAttributeNS(                            \
        handle->writer,                                         \
        NULL,                                                   \
        (const xmlChar *) "xmlns",                              \
        NULL, (const xmlChar *) "") < 0) ?                      \
                                                                \
        (GlobusSoapMessageErrorSerializeNamespaceFailed(        \
            NULL, NULL)) :                                      \
        GLOBUS_SUCCESS)


typedef struct globus_l_soap_message_prefix_ref_s
{
    const xmlChar *                             prefix;
    int                                         ref;
} globus_l_soap_message_prefix_ref_t;

void
globus_soap_message_serialize_start_c14n_subset(
    globus_soap_message_handle_t        handle)
{
    handle->c14n_subset = 1;
    handle->c14n_default_ns = globus_malloc(
        sizeof(globus_i_soap_message_namespace_t));
    memset(handle->c14n_default_ns, 0, 
           sizeof(globus_i_soap_message_namespace_t));
}

void
globus_soap_message_serialize_end_c14n_subset(
    globus_soap_message_handle_t        handle)
{
    handle->c14n_subset = 0;
    
    globus_hashtable_destroy_all(
        &handle->c14n_nsprefix_table, globus_libc_free);

    globus_hashtable_init(
        &handle->c14n_nsprefix_table,
        50,
        globus_hashtable_voidp_hash,
        globus_hashtable_voidp_keyeq);

    globus_i_soap_message_namespace_destroy(
        handle->c14n_default_ns);
    handle->c14n_default_ns = NULL;
}

globus_bool_t
globus_soap_message_serialize_is_c14n_subset(
    globus_soap_message_handle_t        handle)
{
    return handle->c14n_subset;
}

char *
globus_soap_message_serialize_lookup_prefix(
    globus_soap_message_handle_t        handle,
    const char *                        Namespace,
    int *                               created)
{
    void *                              nsref;
    xmlChar                             pref[6] = "ns\0\0\0\0";
    const xmlChar *                     prefix = NULL;
    GlobusFuncName(globus_soap_message_serialize_lookup_prefix);
    GlobusSoapMessageDebugEnter();

    *created = 0;
    
    nsref = (void *) xmlDictLookup(
            handle->dict,
            (const xmlChar *) Namespace,
            -1);

    if(GLOBUS_I_SOAP_MESSAGE_NS_IS_DEFAULT(handle, nsref))
    {
        return NULL;
    }

    GLOBUS_I_SOAP_MESSAGE_NSPREFIX_LOOKUP(handle, nsref, prefix);
    if(!prefix)
    {
        
        *created = 1;
        GLOBUS_I_SOAP_MESSAGE_GET_UNIQUE_PREFIX(handle, pref);
        prefix = pref;
        GLOBUS_I_SOAP_MESSAGE_NSPREFIX_ADD(handle, nsref, prefix);
    }

    return (char *) globus_libc_strdup((char *)prefix);
}

globus_result_t
globus_soap_message_element_ns_set(
    globus_soap_message_handle_t        handle,
    const char *                        Namespace,
    globus_xsd_element_options_t *      options)
{
    const xmlChar *                     prefix = NULL;
    xmlChar                             pref[6] = "ns\0\0\0\0";
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_element_ns_set);
    GlobusSoapMessageDebugEnter();

    if(Namespace && (*options & GLOBUS_XSD_ELEMENT_SET_DEFAULT_NS) &&
       !(*options & GLOBUS_XSD_ELEMENT_ATTR_HAS_NULL_NAMESPACE))
    {
        GLOBUS_I_SOAP_MESSAGE_NS_SET_DEFAULT(
                handle,
                (const xmlChar *)Namespace);
    }
    else if(Namespace)
    {
        GLOBUS_I_SOAP_MESSAGE_NSPREFIX_LOOKUP(
                handle,
                (const xmlChar *) Namespace,
                prefix);

        if((!prefix) && 
                   (!GLOBUS_I_SOAP_MESSAGE_NS_IS_DEFAULT(
                       handle, xmlDictLookup(handle->dict, Namespace, -1))))
        {
            GLOBUS_I_SOAP_MESSAGE_GET_UNIQUE_PREFIX(handle, pref);
            prefix = pref;
            GLOBUS_I_SOAP_MESSAGE_NSPREFIX_ADD(
                    handle,
                    (const xmlChar *) Namespace,
                    prefix);
        }
    }

    if(!Namespace && 
       !(GLOBUS_I_SOAP_MESSAGE_NS_IS_NULL(handle)))
    {
        GLOBUS_I_SOAP_MESSAGE_NS_SET_DEFAULT(handle, NULL);
    }

    if(*options & GLOBUS_XSD_ELEMENT_ATTR_HAS_NULL_NAMESPACE)
    {
        *options ^= GLOBUS_XSD_ELEMENT_ATTR_HAS_NULL_NAMESPACE;
    }

    GlobusSoapMessageDebugExit();
    return result;
}
    

globus_result_t
globus_soap_message_element_ns_add(
    globus_soap_message_handle_t        handle,
    const char *                        Namespace,
    globus_xsd_element_options_t *      options)
{
    const xmlChar *                     prefix = NULL;
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_element_ns_add);
    GlobusSoapMessageDebugEnter();

    if(Namespace && (*options & GLOBUS_XSD_ELEMENT_SET_DEFAULT_NS) &&
       !(*options & GLOBUS_XSD_ELEMENT_ATTR_HAS_NULL_NAMESPACE))
    {
        result = GLOBUS_I_SOAP_MESSAGE_DEFINE_DEFAULT_NS(
                handle,
                (const xmlChar *) Namespace);
        if(result != GLOBUS_SUCCESS)
        {
            goto exit;
        }
    }
    else if(Namespace)
    {
        int refcount;
        GLOBUS_I_SOAP_MESSAGE_NSPREFIX_LOOKUP(handle, Namespace, prefix);
        GLOBUS_I_SOAP_MESSAGE_NSPREFIX_REFCOUNT(
            handle, Namespace, refcount);
        if(refcount == 0 &&
                   !GLOBUS_I_SOAP_MESSAGE_NS_IS_DEFAULT(
                       handle, xmlDictLookup(handle->dict, Namespace, -1)))
        {
            result = GLOBUS_I_SOAP_MESSAGE_DEFINE_PREFIX(
                handle, Namespace, prefix);
            if(result != GLOBUS_SUCCESS)
            {
                goto exit;
            }
        }
    }

    if(!Namespace && 
       !(*options & GLOBUS_XSD_ELEMENT_ATTR_HAS_NULL_NAMESPACE))
    {
        int refcount = 0;
        GLOBUS_I_SOAP_MESSAGE_DEFAULT_NS_REFCOUNT(handle, refcount);
        if(refcount < 1)
        {
            result = GLOBUS_I_SOAP_MESSAGE_DEFINE_NULL_NS(handle);
            if(result != GLOBUS_SUCCESS)
            {
                goto exit;
            }
        }
    }
            
    if(*options & GLOBUS_XSD_ELEMENT_SET_DEFAULT_NS)
    {
        *options ^= GLOBUS_XSD_ELEMENT_SET_DEFAULT_NS;
    }

    if(*options & GLOBUS_XSD_ELEMENT_ATTR_HAS_NULL_NAMESPACE)
    {
        *options ^= GLOBUS_XSD_ELEMENT_ATTR_HAS_NULL_NAMESPACE;
    }

    GLOBUS_I_SOAP_MESSAGE_REFINC(handle, Namespace);

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

void
globus_soap_message_element_ns_remove(
    globus_soap_message_handle_t        handle,
    const char *                        Namespace,
    globus_xsd_element_options_t *      options)
{
    GlobusFuncName(globus_soap_message_element_ns_remove);
    GlobusSoapMessageDebugEnter();
    GLOBUS_I_SOAP_MESSAGE_REFDEC(handle, Namespace);
    GlobusSoapMessageDebugExit();
}

globus_result_t
globus_soap_message_attribute_ns_set(
    globus_soap_message_handle_t        handle,
    const char *                        Namespace,
    globus_xsd_element_options_t *      options)
{
    xmlChar *                           prefix = NULL;
    xmlChar                             pref[6] = "ns\0\0\0\0";
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_attribute_ns_set);
    GlobusSoapMessageDebugEnter();

    if(Namespace)
    {
        GLOBUS_I_SOAP_MESSAGE_NSPREFIX_LOOKUP(
            handle, Namespace, prefix);
        if(!prefix)
        {
            GLOBUS_I_SOAP_MESSAGE_GET_UNIQUE_PREFIX(handle, pref);
            prefix = pref;
            GLOBUS_I_SOAP_MESSAGE_NSPREFIX_ADD(handle, Namespace, prefix);
        }
    }
/*     else */
/*     { */
/*         const xmlChar *                 default_ns =  */
/*         GLOBUS_I_SOAP_MESSAGE_DEFAULT_NS(handle); */
        
/*         if(default_ns) */
/*         { */
/*             GLOBUS_I_SOAP_MESSAGE_NS_SET_DEFAULT(handle, NULL); */

/*             *options |= GLOBUS_XSD_ELEMENT_ATTR_HAS_NULL_NAMESPACE; */
/*         } */
/*     } */

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_attribute_ns_add(
    globus_soap_message_handle_t        handle,
    const char *                        Namespace,
    globus_xsd_element_options_t *      options)
{
    xmlChar *                           prefix = NULL;
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_attribute_ns_add);
    GlobusSoapMessageDebugEnter();

    if(Namespace &&
       !GLOBUS_I_SOAP_MESSAGE_NS_IS_DEFAULT(
           handle, xmlDictLookup(handle->dict, Namespace, -1)))
    {
        int refcount;
        GLOBUS_I_SOAP_MESSAGE_NSPREFIX_LOOKUP(
            handle, Namespace, prefix);
        GLOBUS_I_SOAP_MESSAGE_NSPREFIX_REFCOUNT(
            handle, Namespace, refcount);
        if(refcount == 0)
        {
            result = GLOBUS_I_SOAP_MESSAGE_DEFINE_PREFIX(
                handle, Namespace, prefix);
            if(result != GLOBUS_SUCCESS)
            {
                goto exit;
            }
        }

        GLOBUS_I_SOAP_MESSAGE_REFINC(handle, Namespace);
    }
/*     else if(!Namespace) */
/*     { */
/*         int refcount = 0; */
/*         GLOBUS_I_SOAP_MESSAGE_DEFAULT_NS_REFCOUNT(handle, refcount); */
/*         if(refcount < 1) */
/*         { */
/*             result = GLOBUS_I_SOAP_MESSAGE_DEFINE_NULL_NS(handle); */
/*             if(result != GLOBUS_SUCCESS) */
/*             { */
/*                 goto exit; */
/*             } */
/*         } */

/*         GLOBUS_I_SOAP_MESSAGE_REFINC(handle, Namespace); */
/*     } */

 exit:
    GlobusSoapMessageDebugExit();
    return result;
}

void
globus_soap_message_attribute_ns_remove(
    globus_soap_message_handle_t        handle,
    const char *                        Namespace,
    globus_xsd_element_options_t *      options)
{
    GlobusFuncName(globus_soap_message_attribute_ns_remove);
    GlobusSoapMessageDebugEnter();
    if(Namespace)
    {
        GLOBUS_I_SOAP_MESSAGE_REFDEC(handle, Namespace);
    }
    GlobusSoapMessageDebugExit();
}

globus_result_t
globus_soap_message_subelement_ns_set(
    globus_soap_message_handle_t        handle,
    const char *                        Namespace,
    globus_xsd_element_options_t *      options)
{
    xmlChar                             pref[6] = "ns\0\0\0\0";
    xmlChar *                           prefix = NULL;
    
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_subelement_ns_set);
    GlobusSoapMessageDebugEnter();

    if(Namespace)
    {
        GLOBUS_I_SOAP_MESSAGE_NSPREFIX_LOOKUP(
            handle, Namespace, prefix);
        if(!prefix)
        {
            GLOBUS_I_SOAP_MESSAGE_GET_UNIQUE_PREFIX(handle, pref);
            prefix = pref;
            GLOBUS_I_SOAP_MESSAGE_NSPREFIX_ADD(handle, Namespace, prefix);
        }
    }
    
    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_subelement_ns_add(
    globus_soap_message_handle_t        handle,
    const char *                        Namespace,
    globus_xsd_element_options_t *      options)
{
    xmlChar *                           prefix = NULL;
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_subelement_ns_add);
    GlobusSoapMessageDebugEnter();

    if(Namespace && 
       !GLOBUS_I_SOAP_MESSAGE_NS_IS_DEFAULT(
           handle, xmlDictLookup(handle->dict, Namespace, -1)))
    {
        int refcount;
        GLOBUS_I_SOAP_MESSAGE_NSPREFIX_LOOKUP(
            handle, Namespace, prefix);
        GLOBUS_I_SOAP_MESSAGE_NSPREFIX_REFCOUNT(
            handle, Namespace, refcount);
        if(refcount == 0)
        {
            result = GLOBUS_I_SOAP_MESSAGE_DEFINE_PREFIX(
                handle, Namespace, prefix);
            if(result != GLOBUS_SUCCESS)
            {
                    goto exit;
            }
        }
        
        GLOBUS_I_SOAP_MESSAGE_REFINC(handle, Namespace);
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

void
globus_soap_message_subelement_ns_remove(
    globus_soap_message_handle_t        handle,
    const char *                        Namespace,
    globus_xsd_element_options_t *      options)
{
    if(Namespace)
    {
        GLOBUS_I_SOAP_MESSAGE_REFDEC(handle, Namespace);
    }
}


globus_result_t
globus_soap_message_serialize_envelope(
   globus_soap_message_handle_t         handle)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    globus_xsd_element_options_t        options = 0;
    GlobusFuncName(globus_soap_message_serialize_envelope_begin);
    GlobusSoapMessageDebugEnter();

    result = globus_soap_message_element_ns_set(
        handle, SOAP_ENV_NS, &options);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorSerializeFailed(
            result,
            NULL, 
            &soap_env_qname);
        goto exit;
    }

    result = globus_soap_message_serialize_element(
        handle,
        &soap_env_qname,
        options,
        NULL);

    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorSerializeFailed(
            result, "", &soap_env_qname);
        goto exit;
    }

    result = globus_soap_message_element_ns_add(
        handle,
        SOAP_ENV_NS,
        &options);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorSerializeFailed(
            result, "", &soap_env_qname);
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_serialize_envelope_end(
    globus_soap_message_handle_t             handle)
{
    globus_xsd_element_options_t        options = 0;
    int                                 res = 0;
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_serialize_envelope_end);
    GlobusSoapMessageDebugEnter();

    result = globus_soap_message_serialize_element_end(
        handle,
        &soap_env_qname,
        options,
        NULL);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorSerializeFailed(
            result,
            "",
            &soap_env_qname);
        goto exit;
    }

    res = xmlTextWriterFlush(handle->writer);
    if(res < 0)
    {
        xmlErrorPtr                     err = xmlGetLastError();
        result = GlobusSoapMessageErrorSerializeFailed(
            (handle->result ? handle->result : GLOBUS_SUCCESS),
            (err ? err->message : ""),
            &soap_env_qname);
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}
    
globus_result_t
globus_soap_message_serialize_header(
    globus_soap_message_handle_t        handle)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    globus_xsd_element_options_t        options = 0;
    GlobusFuncName(globus_soap_message_serialize_header);
    GlobusSoapMessageDebugEnter();

    result = globus_soap_message_invoke_ns_callouts(
        handle, 
        &soap_header_qname, 
        GLOBUS_SOAP_MESSAGE_CALLOUT_NS_SET,
        &options);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorSerializeFailed(
            result, "", &soap_header_qname);
        goto exit;
    }

    result = globus_soap_message_element_ns_set(
        handle, soap_header_qname.Namespace, &options);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorSerializeFailed(
            result, "", &soap_header_qname);
        goto exit;
    }

    result = globus_soap_message_invoke_callouts(
        handle, 
        &soap_header_qname, 
        GLOBUS_SOAP_MESSAGE_CALLOUT_BEGIN);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorSerializeFailed(
            result, "", &soap_header_qname);
        goto exit;
    }

    result = globus_soap_message_serialize_element(
        handle,
        &soap_header_qname,
        options,
        NULL);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorSerializeFailed(
            result, "", &soap_header_qname);
        goto exit;
    }
    
    result = globus_soap_message_element_ns_add(
        handle,
        soap_header_qname.Namespace,
        &options);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorSerializeFailed(
            result, "", &soap_header_qname);
        goto exit;
    }

    result = globus_soap_message_invoke_ns_callouts(
        handle,
        &soap_header_qname,
        GLOBUS_SOAP_MESSAGE_CALLOUT_NS_ADD,
        &options);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorSerializeFailed(
            result, "", &soap_header_qname);
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_serialize_header_begin_close(
    globus_soap_message_handle_t        message)
{
    int                                 res = 0;
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_serialize_header_begin_close);
    GlobusSoapMessageDebugEnter();

    res = xmlTextWriterWriteRawLen(message->writer, (xmlChar *) "", 0);
    if(res < 0)
    {
        xmlErrorPtr                     err = xmlGetLastError();
        result = GlobusSoapMessageErrorSerializeFailed(
            (message->result ? message->result : GLOBUS_SUCCESS),
            (err ? err->message : ""), &soap_header_qname);
        goto exit;
    }

    result = globus_soap_message_invoke_callouts(
        message,
        &soap_header_qname,
        GLOBUS_SOAP_MESSAGE_CALLOUT_ATTR);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorSerializeFailed(
            result,  "", &soap_header_qname);
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_serialize_header_end(
    globus_soap_message_handle_t        handle)
{
    globus_xsd_element_options_t        options = 0;
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_serialize_header_end);
    GlobusSoapMessageDebugEnter();

    result = globus_soap_message_serialize_element_end(
        handle, &soap_header_qname, options, NULL);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorSerializeFailed(
            result, "", &soap_header_qname);
        goto exit;
    }

    result = globus_soap_message_invoke_ns_callouts(
        handle,
        &soap_header_qname,
        GLOBUS_SOAP_MESSAGE_CALLOUT_NS_REMOVE,
        &options);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorSerializeFailed(
            result, NULL, &soap_header_qname);
        goto exit;
    }

    result = globus_soap_message_invoke_callouts(
        handle, &soap_header_qname, GLOBUS_SOAP_MESSAGE_CALLOUT_END);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorSerializeFailed(
            result, NULL, &soap_header_qname);
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_serialize_body(
    globus_soap_message_handle_t        handle)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    globus_xsd_element_options_t        options = 0;
    GlobusFuncName(globus_soap_message_serialize_body);
    GlobusSoapMessageDebugEnter();

    result = globus_soap_message_invoke_ns_callouts(
        handle, 
        &soap_body_qname, 
        GLOBUS_SOAP_MESSAGE_CALLOUT_NS_SET,
        &options);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorSerializeFailed(
            result, "", &soap_body_qname);
        goto exit;
    }

    result = globus_soap_message_element_ns_set(
        handle, soap_body_qname.Namespace, &options);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorSerializeFailed(
            result, "", &soap_body_qname);
        goto exit;
    }

    result = globus_soap_message_invoke_callouts(
        handle, 
        &soap_body_qname, 
        GLOBUS_SOAP_MESSAGE_CALLOUT_BEGIN);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorSerializeFailed(
            result, "", &soap_body_qname);
        goto exit;
    }

    result = globus_soap_message_serialize_element(
        handle, &soap_body_qname, options, NULL);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorSerializeFailed(
            result, "", &soap_body_qname);
        goto exit;
    }
    
    result = globus_soap_message_invoke_ns_callouts(
        handle,
        &soap_body_qname,
        GLOBUS_SOAP_MESSAGE_CALLOUT_NS_ADD,
        &options);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorSerializeFailed(
            result, "", &soap_body_qname);
        goto exit;
    }

    result = globus_soap_message_element_ns_add(
        handle,
        soap_body_qname.Namespace,
        &options);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorSerializeFailed(
            result, "", &soap_body_qname);
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_serialize_fault(
    globus_soap_message_handle_t        handle,
    globus_soap_message_fault_t         fault)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    globus_xsd_element_options_t        options = 0;
    GlobusFuncName(globus_soap_message_serialize_fault);
    GlobusSoapMessageDebugEnter();

    result = globus_soap_message_invoke_ns_callouts(
        handle,
        &soap_fault_qname,
        GLOBUS_SOAP_MESSAGE_CALLOUT_NS_SET,
        &options);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorSerializeFailed(
            result, "", &soap_fault_qname);
        goto exit;
    }

    result = globus_soap_message_element_ns_set(
        handle, soap_fault_qname.Namespace, &options);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorSerializeFailed(
            result, "", &soap_fault_qname);
        goto exit;
    }

    result = globus_soap_message_invoke_callouts(
        handle,
        &soap_fault_qname,
        GLOBUS_SOAP_MESSAGE_CALLOUT_BEGIN);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorSerializeFailed(
            result, "", &soap_fault_qname);
        goto exit;
    }

    result = globus_soap_message_serialize_element(
        handle, &soap_fault_qname, 0, NULL);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorSerializeFailed(
            result, NULL, &soap_fault_qname);
        goto exit;
    }

    result = globus_soap_message_element_ns_add(
        handle, soap_fault_qname.Namespace, &options);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorSerializeFailed(
            result, "", &soap_fault_qname);
        goto exit;
    }

    result = globus_soap_message_invoke_ns_callouts(
        handle,
        &soap_fault_qname,
        GLOBUS_SOAP_MESSAGE_CALLOUT_NS_ADD,
        &options);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorSerializeFailed(
            result, "", &soap_fault_qname);
        goto exit;
    }

    if(fault->faultcode)
    {
        result = globus_soap_message_serialize_element(
            handle, &soap_faultcode_qname, 0, NULL);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusSoapMessageErrorSerializeFailed(
                result, NULL, &soap_faultcode_qname);
            goto exit;
        }
        
        result = globus_soap_message_serialize_string(
            handle, &fault->faultcode);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusSoapMessageErrorSerializeFailed(
                result, NULL, &soap_faultcode_qname);
            goto exit;
        }
        
        result = globus_soap_message_serialize_element_end(
            handle,
            &soap_faultcode_qname, 0, NULL);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusSoapMessageErrorSerializeFailed(
                result, NULL, &soap_faultcode_qname);
            goto exit;
        }
    }

    if(fault->faultstring)
    {

        result = globus_soap_message_serialize_element(
            handle, &soap_faultstring_qname, 0, NULL);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusSoapMessageErrorSerializeFailed(
                result, NULL, &soap_faultstring_qname);
            goto exit;
        }
        
        result = globus_soap_message_serialize_string(
            handle, &fault->faultstring);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusSoapMessageErrorSerializeFailed(
                result, NULL, &soap_faultstring_qname);
            goto exit;
        }
        
        result = globus_soap_message_serialize_element_end(
            handle,
            &soap_faultstring_qname,
            0,
            NULL);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusSoapMessageErrorSerializeFailed(
                result, NULL, &soap_faultstring_qname);
            goto exit;
        }
    }

    if(fault->faultactor)
    {

        result = globus_soap_message_serialize_element(
            handle, &soap_faultactor_qname, 0, NULL);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusSoapMessageErrorSerializeFailed(
                result, NULL, &soap_faultactor_qname);
            goto exit;
        }
        
        result = globus_soap_message_serialize_string(
            handle, &fault->faultactor);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusSoapMessageErrorSerializeFailed(
                result, NULL, &soap_faultactor_qname);
            goto exit;
        }
        
        result = globus_soap_message_serialize_element_end(
            handle, &soap_faultactor_qname, 0, NULL);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusSoapMessageErrorSerializeFailed(
                result, NULL, &soap_faultactor_qname);
            goto exit;
        }
    }

    if(fault->detail)
    {
        
        result = globus_soap_message_serialize_element(
            handle, &soap_fault_detail_qname, 0, NULL);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusSoapMessageErrorSerializeFailed(
                result, NULL, &soap_fault_detail_qname);
            goto exit;
        }
        
        result = xsd_any_serialize(
            NULL, fault->detail, handle, 0);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusSoapMessageErrorSerializeFailed(
                result, NULL, &soap_fault_detail_qname);
            goto exit;
        }
        
        result = globus_soap_message_serialize_element_end(
            handle, &soap_fault_detail_qname, 0, NULL);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusSoapMessageErrorSerializeFailed(
                result, NULL, &soap_fault_detail_qname);
            goto exit;
        }
    }

    result = globus_soap_message_invoke_ns_callouts(
        handle,
        &soap_fault_qname,
        GLOBUS_SOAP_MESSAGE_CALLOUT_NS_REMOVE,
        &options);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorSerializeFailed(
            result, NULL, &soap_fault_qname);
        goto exit;
    }

    result = globus_soap_message_invoke_callouts(
        handle,
        &soap_fault_qname,
        GLOBUS_SOAP_MESSAGE_CALLOUT_END);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorSerializeFailed(
            result, NULL, &soap_fault_qname);
        goto exit;
    }

    result = globus_soap_message_serialize_element_end(
        handle, &soap_fault_qname, 0, NULL);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorSerializeFailed(
            result, NULL, &soap_fault_qname);
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_serialize_body_begin_close(
    globus_soap_message_handle_t        handle)
{
    int                                 res = 0;
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_serialize_body_begin_close);
    GlobusSoapMessageDebugEnter();

    result = globus_soap_message_invoke_callouts(
        handle,
        &soap_body_qname,
        GLOBUS_SOAP_MESSAGE_CALLOUT_ATTR);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorSerializeFailed(
            result,  "", &soap_body_qname);
        goto exit;
    }

    res = xmlTextWriterWriteRawLen(handle->writer, (xmlChar *) "", 0);
    if(res < 0)
    {
        xmlErrorPtr                     err = xmlGetLastError();
        result = GlobusSoapMessageErrorSerializeFailed(
            (handle->result ? handle->result : GLOBUS_SUCCESS),
            (err ? err->message : ""), &soap_body_qname);
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_serialize_body_end(
    globus_soap_message_handle_t        handle)
{
    globus_xsd_element_options_t        options = 0;
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_serialize_body_end);
    GlobusSoapMessageDebugEnter();

    result = globus_soap_message_serialize_element_end(
        handle, &soap_body_qname, options, NULL);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorSerializeFailed(
            result, NULL, &soap_body_qname);
        goto exit;
    }        

    result = globus_soap_message_invoke_ns_callouts(
        handle,
        &soap_body_qname,
        GLOBUS_SOAP_MESSAGE_CALLOUT_NS_REMOVE,
        &options);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorSerializeFailed(
            result, NULL, &soap_body_qname);
        goto exit;
    }

    result = globus_soap_message_invoke_callouts(
        handle, &soap_body_qname, GLOBUS_SOAP_MESSAGE_CALLOUT_END);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorSerializeFailed(
            result, NULL, &soap_body_qname);
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_serialize_element(
    globus_soap_message_handle_t        handle,
    xsd_QName *                         element_qname,
    globus_xsd_element_options_t        options,
    xsd_QName *                         type_qname)
{
    const xmlChar *                     prefix = NULL;
    int                                 res = 0;
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_serialize_element);
    GlobusSoapMessageDebugEnter();

    GLOBUS_I_SOAP_MESSAGE_GET_ELEMENT_PREFIX(
        handle, element_qname, prefix);
    res = xmlTextWriterStartElementNS(
        handle->writer, prefix, (const xmlChar *) element_qname->local, NULL);
    if(res < 0)
    {
        xmlErrorPtr                     err = xmlGetLastError();
        result = GlobusSoapMessageErrorSerializeFailed(
            (handle->result ? handle->result : GLOBUS_SUCCESS),
            (err ? err->message : ""), 
            element_qname);
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_serialize_dateTime(
    globus_soap_message_handle_t        handle,
    const xsd_dateTime *                val)
{
    xmlChar                             strval[25];
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_serialize_dateTime);
    GlobusSoapMessageDebugEnter();

    sprintf((char *) strval, "%.4d-%.2d-%.2dT%.2d:%.2d:%.2dZ", 
            val->tm_year + 1900,
            val->tm_mon + 1,
            val->tm_mday,
            val->tm_hour,
            val->tm_min,
            val->tm_sec);
    if(xmlTextWriterWriteString(handle->writer, strval) < 0)
    {
        xmlErrorPtr                     err = xmlGetLastError();
        result = GlobusSoapMessageErrorSerializeFailed(
            (handle->result ? handle->result : GLOBUS_SUCCESS), 
            (err ? err->message : ""), 
            &xsd_dateTime_qname);
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_serialize_date(
    globus_soap_message_handle_t        handle,
    const xsd_date *                    val)
{
    xmlChar                             strval[25];
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_serialize_date);
    GlobusSoapMessageDebugEnter();

    sprintf((char *) strval, "%.4d-%.2d-%.2d", 
            val->tm_year + 1900,
            val->tm_mon + 1,
            val->tm_mday);
    if(xmlTextWriterWriteString(handle->writer, strval) < 0)
    {
        xmlErrorPtr                     err = xmlGetLastError();
        result = GlobusSoapMessageErrorSerializeFailed(
            (handle->result ? handle->result : GLOBUS_SUCCESS), 
            (err ? err->message : ""), 
            &xsd_date_qname);
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_serialize_time(
    globus_soap_message_handle_t        handle,
    const xsd_time *                    val)
{
    xmlChar                             strval[25];
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_serialize_time);
    GlobusSoapMessageDebugEnter();

    sprintf((char *) strval, "%.2d:%.2d:%.2dZ", 
            val->tm_hour,
            val->tm_min,
            val->tm_sec);
    if(xmlTextWriterWriteString(handle->writer, strval) < 0)
    {
        xmlErrorPtr                     err = xmlGetLastError();
        result = GlobusSoapMessageErrorSerializeFailed(
            (handle->result ? handle->result : GLOBUS_SUCCESS), 
            (err ? err->message : ""), 
            &xsd_date_qname);
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}
    

globus_result_t
globus_soap_message_serialize_duration(
    globus_soap_message_handle_t        handle,
    const xsd_duration *                val)
{
    xmlChar                             strval[25];
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_serialize_duration);
    GlobusSoapMessageDebugEnter();

    sprintf((char *) strval, "P%dY%dM%dDT%dH%dM%dS",
            val->tm_year + 1900,
            val->tm_mon + 1,
            val->tm_mday,
            val->tm_hour,
            val->tm_min,
            val->tm_sec);
    if(xmlTextWriterWriteString(handle->writer, strval) < 0)
    {
        xmlErrorPtr                     err = xmlGetLastError();
        result = GlobusSoapMessageErrorSerializeFailed(
            (handle->result ? handle->result : GLOBUS_SUCCESS), 
            (err ? err->message : ""), 
            &xsd_duration_qname);
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_serialize_float(
    globus_soap_message_handle_t        handle,
    const xsd_float *                   val)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    xmlChar                             strval[25];
    GlobusFuncName(globus_soap_message_serialize_float);
    GlobusSoapMessageDebugEnter();

    sprintf((char *) strval, "%E", (double) *val);
    if(xmlTextWriterWriteString(handle->writer, strval) < 0)
    {
        xmlErrorPtr                     err = xmlGetLastError();
        result = GlobusSoapMessageErrorSerializeFailed(
            (handle->result ? handle->result : GLOBUS_SUCCESS), 
            (err ? err->message : ""), 
            &xsd_float_qname);
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_serialize_double(
    globus_soap_message_handle_t        handle,
    const xsd_double *                  val)
{
    xmlChar                             strval[25];
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_serialize_double);
    GlobusSoapMessageDebugEnter();
    
    sprintf((char *) strval, "%E", (double) *val);
    if(xmlTextWriterWriteString(handle->writer, strval) < 0)
    {
        xmlErrorPtr                     err = xmlGetLastError();
        result = GlobusSoapMessageErrorSerializeFailed(
            (handle->result ? handle->result : GLOBUS_SUCCESS), 
            (err ? err->message : ""), 
            &xsd_double_qname);
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_serialize_int(
    globus_soap_message_handle_t        handle,
    const xsd_int *                     val)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    xmlChar                             strval[25];
    
    GlobusFuncName(globus_soap_message_serialize_int);
    GlobusSoapMessageDebugEnter();

    sprintf((char *) strval, "%d", (int) *val);
    if(xmlTextWriterWriteString(handle->writer, strval) < 0)
    {
        xmlErrorPtr                     err = xmlGetLastError();
        result = GlobusSoapMessageErrorSerializeFailed(
            (handle->result ? handle->result : GLOBUS_SUCCESS), 
            (err ? err->message : ""), 
            &xsd_int_qname);
        goto exit;
    }

 exit:
    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_serialize_unsignedInt(
    globus_soap_message_handle_t        handle,
    const xsd_unsignedInt *             val)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    xmlChar                             strval[25];
    
    GlobusFuncName(globus_soap_message_serialize_unsignedInt);
    GlobusSoapMessageDebugEnter();

    sprintf((char *) strval, "%u", (unsigned int) *val);
    if(xmlTextWriterWriteString(handle->writer, strval) < 0)
    {
        xmlErrorPtr                     err = xmlGetLastError();
        result = GlobusSoapMessageErrorSerializeFailed(
            (handle->result ? handle->result : GLOBUS_SUCCESS), 
            (err ? err->message : ""), 
            &xsd_unsignedInt_qname);
        goto exit;
    }

 exit:
    GlobusSoapMessageDebugExit();
    return result;
}
    
globus_result_t
globus_soap_message_serialize_long(
    globus_soap_message_handle_t        handle,
    const xsd_long *                    val)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    xmlChar                                strval[25];

    GlobusFuncName(globus_soap_message_serialize_long);
    GlobusSoapMessageDebugEnter();
    
    sprintf((char *) strval, "%lld", (long long) *val);
    if(xmlTextWriterWriteString(handle->writer, strval) < 0)
    {
        xmlErrorPtr                     err = xmlGetLastError();
        result = GlobusSoapMessageErrorSerializeFailed(
            (handle->result ? handle->result : GLOBUS_SUCCESS), 
            (err ? err->message : ""), 
            &xsd_long_qname);
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_serialize_short(
    globus_soap_message_handle_t        handle,
    const xsd_short *                   val)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    xmlChar                             strval[25];

    GlobusFuncName(globus_soap_message_serialize_short);
    GlobusSoapMessageDebugEnter();
    
    sprintf((char *) strval, "%hd", (short) *val);
    if(xmlTextWriterWriteString(handle->writer, strval) < 0)
    {
        xmlErrorPtr                     err = xmlGetLastError();
        result = GlobusSoapMessageErrorSerializeFailed(
            (handle->result ? handle->result : GLOBUS_SUCCESS), 
            (err ? err->message : ""), 
            &xsd_short_qname);
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_serialize_decimal(
    globus_soap_message_handle_t        handle,
    const xsd_decimal *                 val)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    
    GlobusFuncName(globus_soap_message_serialize_decimal);
    GlobusSoapMessageDebugEnter();

    globus_assert_string(
        0, "globus_soap_message_serialize_decimal NOT IMPLEMENTED");


    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_serialize_integer(
    globus_soap_message_handle_t        handle,
    const xsd_integer *                 val)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    
    GlobusFuncName(globus_soap_message_serialize_integer);
    GlobusSoapMessageDebugEnter();
    
    if(xmlTextWriterWriteString(
                handle->writer,
                (const xmlChar *) BN_bn2dec(*val)) < 0)
    {
        xmlErrorPtr                     err = xmlGetLastError();
        result = GlobusSoapMessageErrorSerializeFailed(
            (handle->result ? handle->result : GLOBUS_SUCCESS), 
            (err ? err->message : ""), 
            &xsd_integer_qname);
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_serialize_byte(
    globus_soap_message_handle_t        handle,
    const xsd_byte *                    val)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    xmlChar                             strval[25];

    GlobusFuncName(globus_soap_message_serialize_byte);
    GlobusSoapMessageDebugEnter();
    
    sprintf((char *) strval, "%hhd", (char) *val);
    if(xmlTextWriterWriteString(handle->writer, strval) < 0)
    {
        xmlErrorPtr                     err = xmlGetLastError();
        result = GlobusSoapMessageErrorSerializeFailed(
            (handle->result ? handle->result : GLOBUS_SUCCESS), 
            (err ? err->message : ""), 
            &xsd_byte_qname);
        goto exit;
    }

 exit:
    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_serialize_base64Binary(
    globus_soap_message_handle_t        handle,
    const xsd_base64Binary *            val)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_serialize_base64Binary);
    GlobusSoapMessageDebugEnter();

    if(xmlTextWriterWriteBase64(
           handle->writer, val->value, 0, val->length) < 0)
    {
        xmlErrorPtr                     err = xmlGetLastError();
        result = GlobusSoapMessageErrorSerializeFailed(
            (handle->result ? handle->result : GLOBUS_SUCCESS), 
            (err ? err->message : ""), 
            &xsd_base64Binary_qname);
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}
        
globus_result_t
globus_soap_message_serialize_hexBinary(
    globus_soap_message_handle_t        handle,
    const xsd_hexBinary *               val)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_serialize_hexbinary);
    GlobusSoapMessageDebugEnter();


    if(xmlTextWriterWriteBinHex(handle->writer, val->value, 0, val->length) < 0)
    {
        xmlErrorPtr                     err = xmlGetLastError();
        result = GlobusSoapMessageErrorSerializeFailed(
            (handle->result ? handle->result : GLOBUS_SUCCESS), 
            (err ? err->message : ""), 
            &xsd_hexBinary_qname);
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_serialize_QName(
    globus_soap_message_handle_t        handle,
    const xsd_QName *                   val)
{
    xmlChar *                           prefix  = NULL;
    int                                 res = 0;
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_serialize_QName);
    GlobusSoapMessageDebugEnter();

    if (val->local == NULL)
    {
        result = GlobusSoapMessageErrorSerializeFailed(
            result, NULL, val);
        goto exit;
    }

    GLOBUS_I_SOAP_MESSAGE_GET_ELEMENT_PREFIX(
        handle, val, prefix);
    res = xmlTextWriterWriteFormatString(
        handle->writer, "%s%s%s",
        (prefix ? ((char *) prefix) : ""), (prefix ? ":" : ""), val->local);
    if(res < 0)
    {
        xmlErrorPtr                     err = xmlGetLastError();
        result = GlobusSoapMessageErrorSerializeFailed(
            (handle->result ? handle->result : GLOBUS_SUCCESS), 
            (err ? err->message : ""), 
            val);
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_serialize_QName_attribute(
    globus_soap_message_handle_t        handle,
    const xsd_QName *                   attr_name,
    const xsd_QName *                   val)
{
    int                                 res = 0;
    xmlChar *                           prefix = NULL;
    xmlChar *                           attr_prefix = NULL;
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_serialize_QName_attribute);
    GlobusSoapMessageDebugEnter();

    GLOBUS_I_SOAP_MESSAGE_GET_ATTR_PREFIX(
        handle, attr_name, attr_prefix);
    GLOBUS_I_SOAP_MESSAGE_GET_ELEMENT_PREFIX(
        handle, val, prefix);

    res = xmlTextWriterWriteFormatAttributeNS(
        handle->writer, attr_prefix, (xmlChar *) attr_name->local, NULL,
        (char *) "%s%s%s",
        (prefix ? ((char *)prefix) : ""), 
        (prefix ? ":" : ""), val->local);
    if(res < 0)
    {
        xmlErrorPtr                     err = xmlGetLastError();
        result = GlobusSoapMessageErrorSerializeFailed(
            (handle->result ? handle->result : GLOBUS_SUCCESS), 
            (err ? err->message : ""), 
            val);
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_serialize_QName_list(
    globus_soap_message_handle_t        handle,
    xsd_QName_array *                   inst)
{
    int                                 i = 0;
    int                                 pos = 0;
    int                                 len = 0;
    const xmlChar **                    prefix_array;
    globus_result_t                     result = GLOBUS_SUCCESS;
    char *                              value;
    int                                 res;
    GlobusFuncName(globus_soap_message_serialize_QName_list);
    GlobusSoapMessageDebugEnter();

    prefix_array = malloc(inst->length * (sizeof(const xmlChar *)));
    memset(prefix_array, 0, 
           (inst->length * (sizeof(const xmlChar *))));
    
    for(i = 0; i < inst->length; ++i)
    {
        xsd_QName *                     element = &inst->elements[i];
        GLOBUS_I_SOAP_MESSAGE_GET_ELEMENT_PREFIX(
            handle, element, prefix_array[i]);
        len += strlen((char *) prefix_array[i])
                + strlen(inst->elements[i].local) + 1;
    }

    value = globus_malloc(len + 1);
    memset(value, 0, len + 1);
    pos = 0;
    for(i = 0; i < inst->length; ++i)
    {
        pos += sprintf(
            value + pos, "%s%s%s",
            (prefix_array[i] ? prefix_array[i] : (const xmlChar *)""),
            (prefix_array[i] ? ":" : ""),
            inst->elements[i].local);
    }

    res = xmlTextWriterWriteFormatString(
        handle->writer, "%s", value);
    if(res < 0)
    {
        xsd_QName                       errqname = inst->elements[0];
        xmlErrorPtr                     err = xmlGetLastError();
        result = GlobusSoapMessageErrorSerializeFailed(
            (handle->result ? handle->result : GLOBUS_SUCCESS), 
            (err ? err->message : ""), 
            &errqname);
        goto exit;
    }

    globus_free(value);

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_serialize_QName_attribute_list(
    globus_soap_message_handle_t        handle,
    const xsd_QName *                   attr_qname,
    xsd_QName_array *                   inst)
{
    const xmlChar **                    prefix_array;
    globus_result_t                     result = GLOBUS_SUCCESS;
    int                                 i = 0;
    xmlChar *                           attr_prefix = NULL;
    int                                 len = 0;
    char *                              value = NULL;
    int                                 pos = 0;
    int                                 res;
    GlobusFuncName(globus_soap_message_serialize_QName_attribute_list);
    GlobusSoapMessageDebugEnter();

    prefix_array = globus_malloc(inst->length * (sizeof(const xmlChar *)));
    memset(prefix_array, 0, 
           (inst->length * (sizeof(const xmlChar *))));
    
    for(i = 0; i < inst->length; ++i)
    {
        xsd_QName *                     element = &inst->elements[i];
        GLOBUS_I_SOAP_MESSAGE_GET_ELEMENT_PREFIX(
            handle, element, prefix_array[i]);
        len += strlen((char *) prefix_array[i])
                + strlen(inst->elements[i].local) + 1;
    }

    GLOBUS_I_SOAP_MESSAGE_GET_ATTR_PREFIX(
        handle, attr_qname, attr_prefix);

    value = malloc(len + 1);
    memset(value, 0, len + 1);
    for(i = 0; i < inst->length; ++i)
    {
        pos += sprintf(
            value + pos, "%s%s%s",
            (prefix_array[i] ? (const char *)prefix_array[i] : ""),
            (prefix_array[i] ? ":" : ""),
            inst->elements[i].local);
    }

    res = xmlTextWriterWriteFormatAttributeNS(
        handle->writer, attr_prefix, (xmlChar *) attr_qname->local,
        NULL, "%s", value);
    if(res < 0)
    {
        xmlErrorPtr                     err = xmlGetLastError();
        result = GlobusSoapMessageErrorSerializeFailed(
            (handle->result ? handle->result : GLOBUS_SUCCESS), 
            (err ? err->message : ""), 
            &inst->elements[0]);
        goto exit;
    }

    globus_free(value);

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_serialize_duration_attribute(
    globus_soap_message_handle_t        handle,
    const xsd_QName *                   attr_name,
    const xsd_duration *                val)
{
    int                                 res = 0;
    const xmlChar *                     prefix = NULL;
    globus_result_t                     result = GLOBUS_SUCCESS;
    xmlChar                             strval[25];
    GlobusFuncName(globus_soap_message_serialize_duration_attribute);
    GlobusSoapMessageDebugEnter();

    GLOBUS_I_SOAP_MESSAGE_GET_ATTR_PREFIX(handle, attr_name, prefix);

    sprintf((char *) strval, "P%dY%dM%dDT%dH%dM%dS",
            val->tm_year + 1900,
            val->tm_mon + 1,
            val->tm_mday,
            val->tm_hour,
            val->tm_min,
            val->tm_sec);

    res = xmlTextWriterWriteAttributeNS(
        handle->writer, prefix, (xmlChar *) attr_name->local, NULL, strval);
    if(res < 0)
    {
        xmlErrorPtr                     err = xmlGetLastError();
        GlobusSoapMessageErrorSerializeFailed(
            GLOBUS_SUCCESS, (err ? err->message : ""), &xsd_duration_qname);
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_serialize_dateTime_attribute(
    globus_soap_message_handle_t        handle,
    const xsd_QName *                   attr_name,
    const xsd_dateTime *                val)
{
    int                                 res = 0;
    const xmlChar *                     prefix = NULL;
    globus_result_t                     result = GLOBUS_SUCCESS;
    xmlChar                             strval[25];
    GlobusFuncName(globus_soap_message_serialize_dateTime_attribute);
    GlobusSoapMessageDebugEnter();

    GLOBUS_I_SOAP_MESSAGE_GET_ATTR_PREFIX(handle, attr_name, prefix);

    sprintf((char *) strval, "%.4d-%.2d-%.2dT%.2d:%.2d:%.2dZ", 
            val->tm_year + 1900,
            val->tm_mon + 1,
            val->tm_mday,
            val->tm_hour,
            val->tm_min,
            val->tm_sec);

    res = xmlTextWriterWriteAttributeNS(
        handle->writer, prefix, (xmlChar *) attr_name->local, NULL, strval);
    if(res < 0)
    {
        xmlErrorPtr                     err = xmlGetLastError();
        GlobusSoapMessageErrorSerializeFailed(
            GLOBUS_SUCCESS, (err ? err->message : ""), &xsd_dateTime_qname);
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_serialize_date_attribute(
    globus_soap_message_handle_t        handle,
    const xsd_QName *                   attr_name,
    const xsd_date *                    val)
{
    const xmlChar *                     prefix = NULL;
    globus_result_t                     result = GLOBUS_SUCCESS;
    xmlChar                             strval[25];
    GlobusFuncName(globus_soap_message_serialize_date_attribute);
    GlobusSoapMessageDebugEnter();

    GLOBUS_I_SOAP_MESSAGE_GET_ATTR_PREFIX(handle, attr_name, prefix);

    sprintf((char *) strval, "%.4d-%.2d-%.2d", 
            val->tm_year + 1900,
            val->tm_mon + 1,
            val->tm_mday);
    if(xmlTextWriterWriteAttributeNS(
        handle->writer, prefix, (xmlChar *) attr_name->local,
        NULL, strval) < 0)
    {
        xmlErrorPtr                     err = xmlGetLastError();
        result = GlobusSoapMessageErrorSerializeFailed(
            (handle->result ? handle->result : GLOBUS_SUCCESS), 
            (err ? err->message : ""), 
            &xsd_date_qname);
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_serialize_base64Binary_attribute(
    globus_soap_message_handle_t        handle,
    const xsd_QName *                   attr_name,
    const xsd_base64Binary *            val)
{
    int                                 res = 0;
    const xmlChar *                     prefix = NULL;
    unsigned char *                     attr_buff = NULL;
    int                                 attr_buff_length;
    globus_result_t                     result = GLOBUS_SUCCESS;
    BIO *                               b64 = NULL;
    BIO *                               mem = NULL;
    BIO *                               bio = NULL;
    BUF_MEM *                           bm =  NULL;
    GlobusFuncName(globus_soap_message_serialize_base64Binary_attribute);
    GlobusSoapMessageDebugEnter();

    GLOBUS_I_SOAP_MESSAGE_GET_ATTR_PREFIX(handle, attr_name, prefix);

    b64 = BIO_new(BIO_f_base64());
    mem = BIO_new(BIO_s_mem());
    bio = BIO_push(b64, mem);

    if(BIO_write(bio, val->value, val->length) <= 0)
    {
        goto exit;
    }

    BIO_get_mem_ptr(bio, &bm);

    attr_buff = (unsigned char *) bm->data;
    attr_buff_length = bm->length;
    bm->data = NULL;
    bm->length = 0;
    
    res = xmlTextWriterWriteFormatAttributeNS(
        handle->writer, prefix, (xmlChar *) attr_name->local, NULL, 
        "%*s", attr_buff_length, attr_buff);
    if(res < 0)
    {
        xmlErrorPtr                     err = xmlGetLastError();
        GlobusSoapMessageErrorSerializeFailed(
            GLOBUS_SUCCESS, (err ? err->message : ""), 
            &xsd_base64Binary_qname);
        goto exit;
    }

 exit:

    if(attr_buff)
    {
        OPENSSL_free(attr_buff);
    }

    if(bio)
    {
        BIO_free_all(bio);
    }

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_serialize_long_attribute(
    globus_soap_message_handle_t        handle,
    const xsd_QName *                   attr_name,
    const xsd_long *                    val)
{
    int                                 res = 0;
    const xmlChar *                     prefix = NULL;
    globus_result_t                     result = GLOBUS_SUCCESS;
    xmlChar                             strval[25];
    GlobusFuncName(globus_soap_message_serialize_long_attribute);
    GlobusSoapMessageDebugEnter();

    GLOBUS_I_SOAP_MESSAGE_GET_ATTR_PREFIX(handle, attr_name, prefix);

    sprintf((char *) strval, "%lld", (long long) *val);

    res = xmlTextWriterWriteAttributeNS(
        handle->writer, prefix, (xmlChar *) attr_name->local, NULL, strval);
    if(res < 0)
    {
        xmlErrorPtr                     err = xmlGetLastError();
        GlobusSoapMessageErrorSerializeFailed(
            GLOBUS_SUCCESS, (err ? err->message : ""), &xsd_long_qname);
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_serialize_int_attribute(
    globus_soap_message_handle_t        handle,
    const xsd_QName *                   attr_name,
    const xsd_int *                     val)
{
    int                                 res = 0;
    const xmlChar *                     prefix = NULL;
    globus_result_t                     result = GLOBUS_SUCCESS;
    xmlChar                             strval[25];
    GlobusFuncName(globus_soap_message_serialize_int_attribute);
    GlobusSoapMessageDebugEnter();

    GLOBUS_I_SOAP_MESSAGE_GET_ATTR_PREFIX(handle, attr_name, prefix);

    sprintf((char *) strval, "%d", (int) *val);

    res = xmlTextWriterWriteAttributeNS(
        handle->writer, prefix, (xmlChar *) attr_name->local, NULL, strval);
    if(res < 0)
    {
        xmlErrorPtr                     err = xmlGetLastError();
        GlobusSoapMessageErrorSerializeFailed(
            GLOBUS_SUCCESS, (err ? err->message : ""), &xsd_int_qname);
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_serialize_float_attribute(
    globus_soap_message_handle_t        handle,
    const xsd_QName *                   attr_name,
    const xsd_float *                   val)
{
    int                                 res = 0;
    const xmlChar *                     prefix = NULL;
    globus_result_t                     result = GLOBUS_SUCCESS;
    xmlChar                             strval[25];
    GlobusFuncName(globus_soap_message_serialize_float_attribute);
    GlobusSoapMessageDebugEnter();

    GLOBUS_I_SOAP_MESSAGE_GET_ATTR_PREFIX(handle, attr_name, prefix);

    sprintf((char *) strval, "%E", (double) *val);

    res = xmlTextWriterWriteAttributeNS(
        handle->writer, prefix, (xmlChar *) attr_name->local, NULL, strval);
    if(res < 0)
    {
        xmlErrorPtr                     err = xmlGetLastError();
        GlobusSoapMessageErrorSerializeFailed(
            GLOBUS_SUCCESS, (err ? err->message : ""), &xsd_float_qname);
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_serialize_integer_attribute(
    globus_soap_message_handle_t        handle,
    const xsd_QName *                   attr_name,
    const xsd_integer *                 val)
{
    int                                 res = 0;
    const xmlChar *                     prefix = NULL;
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_serialize_integer_attribute);
    GlobusSoapMessageDebugEnter();

    GLOBUS_I_SOAP_MESSAGE_GET_ATTR_PREFIX(handle, attr_name, prefix);

    res = xmlTextWriterWriteAttributeNS(
        handle->writer, prefix, (xmlChar *) attr_name->local, NULL,
        (xmlChar *) BN_bn2dec(*val));
    if(res < 0)
    {
        xmlErrorPtr                     err = xmlGetLastError();
        GlobusSoapMessageErrorSerializeFailed(
            GLOBUS_SUCCESS, (err ? err->message : ""), &xsd_integer_qname);
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_serialize_element_end(
    globus_soap_message_handle_t        handle,
    xsd_QName *                         element,
    globus_xsd_element_options_t        options,
    xsd_QName *                         type)
{
    int                                 res = 0;
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_message_serialize_element_end);
    GlobusSoapMessageDebugEnter();
    
    if(handle->c14n_subset)
    {
        res = xmlTextWriterFullEndElement(handle->writer);
    }
    else
    {
        res = xmlTextWriterEndElement(handle->writer);
    }

    if(res < 0)
    {
        xmlErrorPtr                     err = xmlGetLastError();
        result = GlobusSoapMessageErrorSerializeElementEndFailed(
            (err ? err->message : ""));
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_serialize_element_content_close(
    globus_soap_message_handle_t        handle)
{
    int                                 res = 0;
    xsd_QName                           element_qname = {NULL, NULL};
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_serialize_element_begin_close);
    GlobusSoapMessageDebugEnter();

    res = xmlTextWriterWriteRawLen(handle->writer, (xmlChar *) "", 0);
    if(res < 0)
    {
        xmlErrorPtr                     err = xmlGetLastError();
        result = GlobusSoapMessageErrorSerializeFailed(
            (handle->result ? handle->result : GLOBUS_SUCCESS),
            (err ? err->message : ""), &element_qname);
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_serialize_element_begin_close(
    globus_soap_message_handle_t        handle)
{
/*     int                                 res = 0; */
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_serialize_element_begin_close);
    GlobusSoapMessageDebugEnter();

/*     res = xmlTextWriterWriteRawLen(handle->writer, NULL, 0); */
/*     if(res < 0) */
/*     { */
/*         xsd_QName                       element_qname; */
/*         xmlErrorPtr                     err = xmlGetLastError(); */
/*         element_qname.local = NULL; */
/*         element_qname.Namespace = NULL; */
/*         result = GlobusSoapMessageErrorSerializeFailed( */
/*             (handle->result ? handle->result : GLOBUS_SUCCESS), */
/*             (err ? err->message : ""), &element_qname); */
/*     } */

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_serialize_string(
    globus_soap_message_handle_t        handle,
    const xsd_string *                  str)
{
    int                                 res = 0;
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_serialize_string);
    GlobusSoapMessageDebugEnter();

    res = xmlTextWriterWriteString(handle->writer, (const xmlChar *) *str);
    if(res < 0)
    {
        xmlErrorPtr                     err = xmlGetLastError();
        GlobusSoapMessageErrorSerializeFailed(
            GLOBUS_SUCCESS, (err ? err->message : ""), &xsd_string_qname);
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_serialize_string_attribute(
    globus_soap_message_handle_t        handle,
    const xsd_QName *                   attr_name,
    const xsd_string *                  str)
{
    int                                 res = 0;
    const xmlChar *                     prefix = NULL;
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_serialize_string_attribute);
    GlobusSoapMessageDebugEnter();

    GLOBUS_I_SOAP_MESSAGE_GET_ATTR_PREFIX(handle, attr_name, prefix);

    res = xmlTextWriterWriteAttributeNS(
        handle->writer, prefix, (xmlChar *) attr_name->local, NULL,
        (const xmlChar *) *str);
    if(res < 0)
    {
        xmlErrorPtr                     err = xmlGetLastError();
        GlobusSoapMessageErrorSerializeFailed(
            GLOBUS_SUCCESS, (err ? err->message : ""), &xsd_string_qname);
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_serialize_string_list(
    globus_soap_message_handle_t        handle,
    xsd_string_array *                  inst)
{
    int                                 i = 0;
    int                                 pos = 0;
    int                                 len = 0;
    globus_result_t                     result = GLOBUS_SUCCESS;
    char *                              value;
    int                                 res;
    GlobusFuncName(globus_soap_message_serialize_string_list);
    GlobusSoapMessageDebugEnter();

    for(i = 0; i < inst->length; ++i)
    {
        len += strlen(inst->elements[i]);
    }

    value = globus_malloc(len + 1);
    pos = 0;
    for(i = 0; i < inst->length; ++i)
    {
        pos += sprintf(
            value + pos, "%s",
            inst->elements[i]);
    }

    res = xmlTextWriterWriteFormatString(
        handle->writer, "%s", value);
    if(res < 0)
    {
        xmlErrorPtr                     err = xmlGetLastError();
        result = GlobusSoapMessageErrorSerializeFailed(
            (handle->result ? handle->result : GLOBUS_SUCCESS), 
            (err ? err->message : ""), 
            &xsd_string_qname);
        goto exit;
    }

    globus_free(value);

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_serialize_string_attribute_list(
    globus_soap_message_handle_t        handle,
    const xsd_QName *                   attr_qname,
    xsd_string_array *                  inst)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    int                                 i = 0;
    xmlChar *                           attr_prefix = NULL;
    int                                 len = 0;
    char *                              value = NULL;
    int                                 pos = 0;
    int                                 res;
    GlobusFuncName(globus_soap_message_serialize_string_attribute_list);
    GlobusSoapMessageDebugEnter();

    for(i = 0; i < inst->length; ++i)
    {
        len += strlen(inst->elements[i]);
    }

    GLOBUS_I_SOAP_MESSAGE_GET_ATTR_PREFIX(
        handle, attr_qname, attr_prefix);

    value = malloc(len + 1);
    memset(value, 0, len + 1);
    for(i = 0; i < inst->length; ++i)
    {
        pos += sprintf(
            value + pos, "%s",
            inst->elements[i]);
    }

    res = xmlTextWriterWriteFormatAttributeNS(
        handle->writer, attr_prefix, (xmlChar *) attr_qname->local,
        NULL, "%s", value);
    if(res < 0)
    {
        xmlErrorPtr                     err = xmlGetLastError();
        result = GlobusSoapMessageErrorSerializeFailed(
            (handle->result ? handle->result : GLOBUS_SUCCESS), 
            (err ? err->message : ""), 
            &xsd_string_qname);
        goto exit;
    }

    globus_free(value);

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_serialize_boolean(
    globus_soap_message_handle_t        handle,
    const xsd_boolean *                 val)
{
    int                                 res = 0;
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_serialize_boolean);
    GlobusSoapMessageDebugEnter();
    
    res = xmlTextWriterWriteString(handle->writer, 
        (const xmlChar *) (*val ? "true" : "false"));
    if(res < 0)
    {
        xmlErrorPtr                     err = xmlGetLastError();
        GlobusSoapMessageErrorSerializeFailed(
            GLOBUS_SUCCESS, (err ? err->message : ""), &xsd_boolean_qname);
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_serialize_boolean_attribute(
    globus_soap_message_handle_t        handle,
    const xsd_QName *                   attr_name,
    const xsd_boolean *                 val)
{
    int                                 res = 0;
    const xmlChar *                     prefix = NULL;
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_soap_message_serialize_boolean_attribute);
    GlobusSoapMessageDebugEnter();

    GLOBUS_I_SOAP_MESSAGE_GET_ATTR_PREFIX(handle, attr_name, prefix);

    res = xmlTextWriterWriteAttributeNS(
        handle->writer, prefix, (xmlChar *) attr_name->local, 
        NULL, (const xmlChar *) (*val ? "true" : "false"));
    if(res < 0)
    {
        xmlErrorPtr                     err = xmlGetLastError();
        GlobusSoapMessageErrorSerializeFailed(
            GLOBUS_SUCCESS, (err ? err->message : ""), &xsd_boolean_qname);
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

char *
globus_soap_message_construct_id(
    const char *                        prefix)
{
    struct timeval current_time;
    gettimeofday(&current_time, NULL);
    return globus_common_create_string(
        "%s%lu",
        prefix,
        (current_time.tv_sec * 1000) +
         ((current_time.tv_usec * 0.001)));
}


globus_result_t
globus_soap_message_add_ns_callout(
    globus_soap_message_handle_t          message,
    const xsd_QName *                     type,
    globus_soap_message_callout_ns_func_t ns_set_callout,
    globus_soap_message_callout_ns_func_t ns_add_callout,
    globus_soap_message_callout_ns_func_t ns_remove_callout)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    globus_i_soap_message_callouts_t *  callouts = NULL;
    globus_i_soap_message_callout_entry_t * entry;
    GlobusFuncName(globus_soap_message_add_ns_callout);
    GlobusSoapMessageDebugEnter();
    
    if(!message->callout_table)
    {
        globus_hashtable_init(
            &message->callout_table,
            20,
            xsd_QName_hash,
            xsd_QName_keyeq);
    }

    entry = globus_malloc(sizeof(globus_i_soap_message_callout_entry_t));
    if(entry == NULL)
    {
        result = GlobusSoapMessageErrorOutOfMemory;
        goto exit;
    }
    memset(entry, 0, sizeof(globus_i_soap_message_callout_entry_t));

    entry->ns_set_callout = ns_set_callout;
    entry->ns_add_callout = ns_add_callout;
    entry->ns_remove_callout = ns_remove_callout;

    callouts = (globus_i_soap_message_callouts_t *)
    globus_hashtable_lookup(&message->callout_table, (void *)type);
    if(callouts == NULL)
    {
        callouts = globus_malloc(sizeof(globus_i_soap_message_callouts_t));
        if(callouts == NULL)
        {
            result = GlobusSoapMessageErrorOutOfMemory;
            goto exit;
        }
        memset(callouts, 0, sizeof(globus_i_soap_message_callouts_t));

        xsd_QName_copy(&callouts->type, type);

        globus_list_insert(&callouts->entries, entry);

        globus_hashtable_insert(
            &message->callout_table, (void *)type, callouts);
    }
    else
    {
        globus_list_insert(&callouts->entries, entry);
    }

exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_soap_message_invoke_ns_callouts(
    globus_soap_message_handle_t        handle,
    const xsd_QName *                   element_qname,
    globus_soap_message_callout_type_t  type,
    globus_xsd_element_options_t *      options)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    globus_i_soap_message_callout_entry_t * entry;
    globus_list_t *                     entries;
    globus_i_soap_message_callouts_t *  callouts;
    GlobusFuncName(globus_i_soap_message_call_begin_callouts);
    GlobusSoapMessageDebugEnter();
    
    if(!handle->callout_table)
    {
        goto exit;
    }

    callouts = globus_hashtable_lookup(
        &handle->callout_table, 
        (void *)element_qname);
    if(callouts)
    {
        entries = callouts->entries;
        while(entries)
        {
            entry = globus_list_first(entries);

            switch(type)
            {

            case GLOBUS_SOAP_MESSAGE_CALLOUT_NS_SET:
                if(entry->ns_set_callout)
                {
                    result =
                        entry->ns_set_callout(handle, element_qname, options);
                }
                break;

            case GLOBUS_SOAP_MESSAGE_CALLOUT_NS_ADD:
                if(entry->ns_add_callout)
                {
                    result =
                        entry->ns_add_callout(handle, element_qname, options);
                }
                break;

            case GLOBUS_SOAP_MESSAGE_CALLOUT_NS_REMOVE:
                if(entry->ns_remove_callout)
                {
                    result = entry->ns_remove_callout(
                            handle,
                            element_qname,
                            options);
                }
                break;

            default:
                break;
            }

            if (result != GLOBUS_SUCCESS)
            {
                break;
            }

            entries = globus_list_rest(entries);
        }
    }

 exit:
    GlobusSoapMessageDebugExit();
    return result;
}
