/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */


#include "globus_soap_message_utils.h"
#include "globus_i_soap_message.h"
#include "globus_soap_message_handle.h"
#include "xsd_QName.h"
#include "globus_xsd_type_info.h"
#include "globus_xml_buffer.h"
#include "globus_i_xsd.h"

xsd_QName                               globus_xml_buffer_qname =
{
    "GLOBUS_XML_BUFFER",
    "GLOBUS_XML_BUFFER"
};

GLOBUS_I_XSD_DEFINE_TYPE_FUNCTIONS(globus_xml_buffer);
GLOBUS_I_XSD_DEFINE_TYPE_INFO(globus_xml_buffer);
GLOBUS_I_XSD_DEFINE_INIT_CONTENTS_GENERIC(globus_xml_buffer);

globus_result_t
globus_xml_buffer_copy_contents(
    globus_xml_buffer *                 dest,
    const globus_xml_buffer *           src)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_xml_buffer_copy_contents);
    GlobusSoapMessageDebugEnter();
    
    if(dest && src)
    {
        dest->length = src->length;
        dest->buffer = globus_malloc(src->length + 1);
        if(!dest->buffer)
        {
            result = GlobusSoapMessageErrorOutOfMemory;
        }
        memcpy(dest->buffer, src->buffer, src->length);
        dest->buffer[src->length] = 0;
    }
    
    GlobusSoapMessageDebugExit();
    
    return result;
}

void
globus_xml_buffer_destroy_contents(
    globus_xml_buffer *                 instance)
{
    GlobusFuncName(globus_xml_buffer_destroy_contents);
    GlobusSoapMessageDebugEnter();
    
    if (instance->buffer != NULL)
    {
        free(instance->buffer);
        instance->buffer = NULL;
    }

    GlobusSoapMessageDebugExit();
}

globus_result_t
globus_xml_buffer_serialize_contents(
    xsd_QName *                         element,
    globus_xml_buffer *                 buffer_handle,
    globus_soap_message_handle_t        message_handle,
    globus_xsd_element_options_t        options)
{
    int                                 res = 0;
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_xml_buffer_serialize_contents);
    GlobusSoapMessageDebugEnter();

    res = xmlTextWriterWriteRawLen(message_handle->writer, 
                                   (const xmlChar *)buffer_handle->buffer,
                                   buffer_handle->length);
    if(res < 0)
    {
        result = GlobusSoapMessageErrorSerializeFailed(
            message_handle->result, 
            "Failed to serialize xml buffer", 
            &globus_xml_buffer_qname);
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_xml_buffer_deserialize_contents(
    xsd_QName *                         eqn,
    globus_xml_buffer *                 buffer_handle,
    globus_soap_message_handle_t        message_handle,
    globus_xsd_element_options_t        options)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_xml_buffer_deserialize_contents);
    GlobusSoapMessageDebugEnter();

    if(xmlTextReaderNodeType(message_handle->reader) ==
       XML_READER_TYPE_TEXT)
    {
	if(xmlTextReaderHasValue(message_handle->reader))
	{
	    buffer_handle->buffer = globus_libc_strdup(
		xmlTextReaderConstValue(message_handle->reader));
	    buffer_handle->length = strlen(buffer_handle->buffer);
	}
    
	if(message_handle->next_ready)
	{
	    message_handle->next_ready--;
	}
        goto exit;
    }
    
    if(message_handle->next_ready)
    {
	if(!xmlTextReaderIsEmptyElement(message_handle->reader))
	{
	    message_handle->next_ready--;
	}
	
	buffer_handle->buffer = xmlTextReaderReadInnerXml(
	    message_handle->reader);
	if(!buffer_handle->buffer)
	{
	    result = GlobusSoapMessageErrorDeserializeFailed(
		message_handle->result, 
		&globus_xml_buffer_qname);
	    goto exit;
	}
    }
    else
    {
	buffer_handle->buffer = xmlTextReaderReadOuterXml(
	    message_handle->reader);
	if(!buffer_handle->buffer)
	{
	    result = GlobusSoapMessageErrorDeserializeFailed(
		message_handle->result, 
		&globus_xml_buffer_qname);
	    goto exit;
	}
    }

    buffer_handle->length = strlen(buffer_handle->buffer);

 exit:
    
    GlobusSoapMessageDebugExit();
    return result;
}
