/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */

#include "globus_i_xsd.h"
#include "xsd_NCName.h"

GLOBUS_I_XSD_DEFINE_QNAME(xsd, NCName);
GLOBUS_I_XSD_DEFINE_TYPE_INFO(xsd_NCName);
GLOBUS_I_XSD_DEFINE_TYPE_FUNCTIONS(xsd_NCName);
GLOBUS_I_XSD_DEFINE_INIT_CONTENTS_GENERIC(xsd_NCName);

void
xsd_NCName_destroy_contents(
    xsd_NCName *                        instance)
{
    GlobusFuncName(xsd_NCName_destroy_contents);
    GlobusSoapMessageDebugEnter();

    if(*instance)
    {
        free(*instance);
        *instance = NULL;
    }

    GlobusSoapMessageDebugExit();
}

globus_result_t
xsd_NCName_copy_contents(
    xsd_NCName *                        dest,
    const xsd_NCName *                  src)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(xsd_NCName_copy_contents);
    GlobusSoapMessageDebugEnter();

    if(dest)
    {
        if(!src || !*src)
        {
            *dest = NULL;
        }
        else
        {
            *dest = globus_libc_strdup(*src);
            if(!*dest)
            {
                result = GlobusSoapMessageErrorOutOfMemory;
            }
        }
    }

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
xsd_NCName_serialize_contents(
    xsd_QName *                         element,
    xsd_NCName *                        instance,
    globus_soap_message_handle_t        message_handle,
    globus_xsd_element_options_t        options)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(xsd_NCName_serialize_contents);
    GlobusSoapMessageDebugEnter();

    /* serialize contents of type */
    result = globus_soap_message_serialize_string(
        message_handle, instance);

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
xsd_NCName_deserialize_contents(
    xsd_QName *                         eqn,
    xsd_NCName *                        instance,
    globus_soap_message_handle_t        message_handle,
    globus_xsd_element_options_t        options)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(xsd_NCName_deserialize_contents);
    GlobusSoapMessageDebugEnter();
    
    result = globus_soap_message_deserialize_string(
        message_handle, instance);

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
xsd_NCName_serialize_attribute(
    xsd_QName *                         element_qname,
    xsd_NCName *                        instance,
    globus_soap_message_handle_t        message_handle,
    globus_xsd_element_options_t        options)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(xsd_NCName_serialize_attribute);
    GlobusSoapMessageDebugEnter();

    result = globus_soap_message_serialize_string_attribute(
        message_handle, element_qname, instance);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorSerializeFailed(
            result, NULL, element_qname);
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
xsd_NCName_deserialize_attribute(
    xsd_QName *                         element_qname,
    xsd_NCName *                        ip,
    globus_soap_message_handle_t        message_handle,
    globus_xsd_element_options_t        options)
{
    globus_result_t                 result = GLOBUS_SUCCESS;
    GlobusFuncName(xsd_NCName_deserialize_attribute);
    GlobusSoapMessageDebugEnter();

    result = globus_soap_message_deserialize_string_attribute(
        message_handle, element_qname, ip);
    if(GlobusSoapMessageStatusAttributeNotFoundCheck(result))
    {
        result = GLOBUS_SUCCESS;
        goto exit;
    }

    if(result != GLOBUS_SUCCESS)
    {
        goto error_exit;
    }

    goto exit;

 error_exit:

    GlobusSoapMessageErrorDeserializeFailed(result, element_qname);

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
xsd_NCName_deserialize_attribute_pointer(
    xsd_QName *                         element_qname,
    xsd_NCName **                       ip,
    globus_soap_message_handle_t        message_handle,
    globus_xsd_element_options_t        options)
{
    xsd_NCName *                        instance = NULL;
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(xsd_NCName_deserialize_attribute_pointer);
    GlobusSoapMessageDebugEnter();

    *ip = NULL;

    result = xsd_NCName_init(&instance);
    if(result != GLOBUS_SUCCESS)
    {
        goto error_exit;
    }

    result = globus_soap_message_deserialize_string_attribute(
        message_handle, element_qname, instance);
    if(GlobusSoapMessageStatusAttributeNotFoundCheck(result))
    {
	xsd_NCName_destroy(instance);
        result = GLOBUS_SUCCESS;
        goto exit;
    }

    if(result != GLOBUS_SUCCESS)
    {
        goto error_exit;
    }

    *ip = instance;

    goto exit;

 error_exit:

    GlobusSoapMessageErrorDeserializeFailed(result, element_qname);

 exit:

    if(result != GLOBUS_SUCCESS && instance)
    {
        xsd_NCName_destroy(instance);
        *ip = NULL;
    }

    GlobusSoapMessageDebugExit();
    return result;
}
