/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */


#include "xsd_QName.h"
#include "globus_soap_message.h"
#include "globus_i_xsd.h"

GLOBUS_I_XSD_DEFINE_QNAME(xsd, QName);

GLOBUS_I_XSD_DEFINE_TYPE_INFO(xsd_QName);

GLOBUS_I_XSD_DEFINE_TYPE_INIT(xsd_QName);
GLOBUS_I_XSD_DEFINE_TYPE_DESTROY(xsd_QName);
GLOBUS_I_XSD_DEFINE_TYPE_COPY(xsd_QName);
GLOBUS_I_XSD_DEFINE_TYPE_DESERIALIZE(xsd_QName);
GLOBUS_I_XSD_DEFINE_TYPE_DESERIALIZE_POINTER(xsd_QName);
GLOBUS_I_XSD_DEFINE_TYPE_INIT_ARRAY(xsd_QName);
GLOBUS_I_XSD_DEFINE_TYPE_DESTROY_ARRAY(xsd_QName);
GLOBUS_I_XSD_DEFINE_TYPE_COPY_ARRAY(xsd_QName);
GLOBUS_I_XSD_DEFINE_TYPE_SERIALIZE_ARRAY(xsd_QName);
GLOBUS_I_XSD_DEFINE_TYPE_DESERIALIZE_ARRAY(xsd_QName);
GLOBUS_I_XSD_DEFINE_TYPE_ARRAY_PUSH(xsd_QName)

GLOBUS_I_XSD_DEFINE_ATTR_FUNCTIONS(xsd_QName, QName);
GLOBUS_I_XSD_DEFINE_INIT_CONTENTS_GENERIC(xsd_QName);
GLOBUS_I_XSD_DEFINE_LIST_FUNCTIONS(xsd_QName, QName);
GLOBUS_I_XSD_DEFINE_LIST_ATTR_FUNCTIONS(xsd_QName, QName);

globus_result_t
xsd_QName_copy_contents(
    xsd_QName *                         dest,
    const xsd_QName *                   src)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(xsd_QName_copy_contents);
    GlobusSoapMessageDebugEnter();
    
    if(dest && src)
    {
        if(src->Namespace)
        {
            dest->Namespace = globus_libc_strdup(src->Namespace);
            if(!dest->Namespace)
            {
                result = GlobusSoapMessageErrorOutOfMemory;
                goto error;
            }
        }
        else
        {
            dest->Namespace = NULL;
        }
        
        if(src->local)
        {
            dest->local = globus_libc_strdup(src->local);
            if(!dest->local)
            {
                result = GlobusSoapMessageErrorOutOfMemory;
                if(dest->Namespace)
                {
                    globus_free(dest->Namespace);
                }
                goto error;
            }
        }
        else
        {
            dest->local = NULL;
        }
    }

error:
    GlobusSoapMessageDebugExit();
    return result;
}

void
xsd_QName_destroy_contents(
    xsd_QName *                         instance)
{
    GlobusFuncName(xsd_QName_destroy_contents);
    GlobusSoapMessageDebugEnter();

    if(instance)
    {
        if(instance->Namespace)
        {
            free(instance->Namespace);
            instance->Namespace = NULL;
        }

        if(instance->local)
        {
            free(instance->local);
            instance->local = NULL;
        }
    }

    GlobusSoapMessageDebugExit();
}

globus_result_t
xsd_QName_serialize(
    xsd_QName *                         element_qname,
    xsd_QName *                         instance,
    globus_soap_message_handle_t        message_handle,
    globus_xsd_element_options_t        options)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(xsd_QName_serialize);
    GlobusSoapMessageDebugEnter();

    if(!(options & GLOBUS_XSD_ELEMENT_CONTENTS_ONLY))
    {
        /* add begin element */
        result = globus_soap_message_invoke_ns_callouts(
                message_handle,
                element_qname,
                GLOBUS_SOAP_MESSAGE_CALLOUT_NS_SET,
                &options);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusSoapMessageErrorSerializeFailed(
                result, NULL, element_qname);
            goto exit;
        }

        if(options & GLOBUS_XSD_ELEMENT_TYPE_ATTR &&
	   !message_handle->c14n_subset)
        {
            result = globus_soap_message_attribute_ns_set(
                message_handle,
                "http://www.w3.org/2001/XMLSchema-instance",
                &options);
            if(result != GLOBUS_SUCCESS)
            {
                result = GlobusSoapMessageErrorSerializeFailed(
                    result, NULL, element_qname);
                goto exit;
            }

            result = globus_soap_message_attribute_ns_set(
                message_handle,
                xsd_QName_qname.Namespace,
                &options);
            if(result != GLOBUS_SUCCESS)
            {
                result = GlobusSoapMessageErrorSerializeFailed(
                    result, NULL, element_qname);
                goto exit;
            }
        }
	
	result = globus_soap_message_element_ns_set(
	    message_handle,
	    element_qname->Namespace,
	    &options);
	if(result != GLOBUS_SUCCESS)
	{
	    result = GlobusSoapMessageErrorSerializeFailed(
		result, NULL, element_qname);
	    goto exit;
	}
    }

    result = globus_soap_message_attribute_ns_set(
	message_handle,
	instance->Namespace,
	&options);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusSoapMessageErrorSerializeFailed(
	    result, NULL, element_qname);
	goto exit;
    }

    if(!(options & GLOBUS_XSD_ELEMENT_CONTENTS_ONLY))
    {
        result = globus_soap_message_invoke_callouts(
                message_handle,
                element_qname,
                GLOBUS_SOAP_MESSAGE_CALLOUT_BEGIN);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusSoapMessageErrorSerializeFailed(
                result, NULL, element_qname);
            goto exit;
        }

        result = globus_soap_message_serialize_element(
            message_handle,
            element_qname,
            options,
            &xsd_QName_qname);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusSoapMessageErrorSerializeFailed(
                result, NULL, element_qname);
            goto exit;
        }

        result = globus_soap_message_invoke_ns_callouts(
                message_handle,
                element_qname,
                GLOBUS_SOAP_MESSAGE_CALLOUT_NS_ADD,
                &options);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusSoapMessageErrorSerializeFailed(
                result, NULL, element_qname);
            goto exit;
        }
        if(options & GLOBUS_XSD_ELEMENT_TYPE_ATTR &&
	   !message_handle->c14n_subset)
        {
            result = globus_soap_message_attribute_ns_add(
                message_handle,
                "http://www.w3.org/2001/XMLSchema-instance",
                &options);
            if(result != GLOBUS_SUCCESS)
            {
                result = GlobusSoapMessageErrorSerializeFailed(
                    result, NULL, element_qname);
                goto exit;
            }
        }

        result = globus_soap_message_element_ns_add(
            message_handle,
            element_qname->Namespace,
            &options);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusSoapMessageErrorSerializeFailed(
                result, NULL, element_qname);
            goto exit;
        }
    }
    
    result = globus_soap_message_attribute_ns_add(
	message_handle,
	instance->Namespace,
	&options);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusSoapMessageErrorSerializeFailed(
	    result, NULL, instance);
	goto exit;
    }

    if(!(options & GLOBUS_XSD_ELEMENT_CONTENTS_ONLY))
    {
        if(options & GLOBUS_XSD_ELEMENT_TYPE_ATTR &&
	   !message_handle->c14n_subset)
        {
            xsd_QName                   type_attr;
            type_attr.local = "type";
            type_attr.Namespace =
            "http://www.w3.org/2001/XMLSchema-instance";

            result = globus_soap_message_attribute_ns_add(
                message_handle,
                type_attr.Namespace,
                &options);
            if(result != GLOBUS_SUCCESS)
            {
                result = GlobusSoapMessageErrorSerializeFailed(
                    result, NULL, element_qname);
                goto exit;
            }

            result = globus_soap_message_attribute_ns_add(
                message_handle,
                xsd_QName_qname.Namespace,
                &options);
            if(result != GLOBUS_SUCCESS)
            {
                result = GlobusSoapMessageErrorSerializeFailed(
                    result, NULL, element_qname);
                goto exit;
            }

            result = globus_soap_message_serialize_QName_attribute(
                message_handle,
                &type_attr,
                &xsd_QName_qname);
            if(result != GLOBUS_SUCCESS)
            {
                result = GlobusSoapMessageErrorSerializeFailed(
                    result, NULL, element_qname);
                goto exit;
            }
        }
        result = globus_soap_message_invoke_callouts(
            message_handle,
            element_qname,
            GLOBUS_SOAP_MESSAGE_CALLOUT_ATTR);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusSoapMessageErrorSerializeFailed(
                result, NULL, element_qname);
            goto exit;
        }

        result = globus_soap_message_serialize_element_begin_close(
            message_handle);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusSoapMessageErrorSerializeFailed(
                result, NULL, element_qname);
            goto exit;
        }
    }

    /* serialize contents of type */
    result = globus_soap_message_serialize_QName(
        message_handle, instance);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorSerializeFailed(
            result, NULL, element_qname);
        goto exit;
    }

    if(!(options & GLOBUS_XSD_ELEMENT_CONTENTS_ONLY))
    {
        /* add end element */
        result = globus_soap_message_serialize_element_end(
            message_handle,
            element_qname,
            options,
            &xsd_QName_qname);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusSoapMessageErrorSerializeFailed(
                result, NULL, element_qname);
            goto exit;
        }

    }

    globus_soap_message_attribute_ns_remove(
        message_handle, instance->Namespace, &options);

    if (!(options & GLOBUS_XSD_ELEMENT_CONTENTS_ONLY))
    {
        globus_soap_message_element_ns_remove(
            message_handle, element_qname->Namespace, &options);

        if(options & GLOBUS_XSD_ELEMENT_TYPE_ATTR &&
	   !message_handle->c14n_subset)
        {
            globus_soap_message_attribute_ns_remove(
                message_handle, 
                "http://www.w3.org/2001/XMLSchema-instance",
                &options);
        
            globus_soap_message_attribute_ns_remove(
                message_handle, xsd_QName_qname.Namespace, &options);
        }
        result = globus_soap_message_invoke_ns_callouts(
            message_handle,
            element_qname,
            GLOBUS_SOAP_MESSAGE_CALLOUT_NS_REMOVE,
            &options);
        if(result != GLOBUS_SUCCESS)
        {
            result  = GlobusSoapMessageErrorSerializeFailed(
                result, NULL, element_qname);
            goto exit;
        }
        result = globus_soap_message_invoke_callouts(
            message_handle,
            element_qname,
            GLOBUS_SOAP_MESSAGE_CALLOUT_END);
        if(result != GLOBUS_SUCCESS)
        {
            result  = GlobusSoapMessageErrorSerializeFailed(
                result, NULL, element_qname);
            goto exit;
        }
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}
    
globus_result_t
xsd_QName_serialize_contents(
    xsd_QName *                         element,
    xsd_QName *                         instance,
    globus_soap_message_handle_t        message_handle,
    globus_xsd_element_options_t        options)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(xsd_QName_serialize_contents);
    GlobusSoapMessageDebugEnter();

    /* serialize contents of type */
    result = xsd_QName_serialize(
        element, instance, message_handle,
        (options | GLOBUS_XSD_ELEMENT_CONTENTS_ONLY));

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
xsd_QName_deserialize_contents(
    xsd_QName *                         eqn,
    xsd_QName *                         instance,
    globus_soap_message_handle_t        message_handle,
    globus_xsd_element_options_t        options)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(xsd_QName_deserialize_contents);
    GlobusSoapMessageDebugEnter();
    
    result = globus_soap_message_deserialize_QName(
        message_handle, instance);

    GlobusSoapMessageDebugExit();
    return result;
}

int
xsd_QName_strcmp(
    char *                              s1,
    char *                              s2)
{
    GlobusFuncName(globus_l_xsd_safe_strcmp);
    return (s1 == s2 || 
        (s1 && s2 && strcmp(s1, s2) == 0));
}

int
xsd_QName_keyeq(
    void *                              qname1,
    void *                              qname2)
{
    xsd_QName *                         q1 = qname1;
    xsd_QName *                         q2 = qname2;
    GlobusFuncName(xsd_QName_keyeq);

    return (q1 == q2) || (q1 && q2 &&
        xsd_QName_strcmp(q1->Namespace, q2->Namespace) &&
            xsd_QName_strcmp(q1->local, q2->local));
}

int
xsd_QName_hash(
    void *                              qname,
    int                                 limit)
{
    unsigned long                       h = 0;
    unsigned long                       g;
    char *                              key;
    xsd_QName *                         q = qname;
    GlobusFuncName(xsd_QName_hash);

    key = q->Namespace;

    if (key)
    {
        while (*key)
        {
            h = (h << 4) + *key++;

            if ((g = (h & 0xF0000000UL)))
            {
                h ^= g >> 24;
                h ^= g;
            }
        }
    }
    key = q->local;
    if (key)
    {
        while (*key)
        {
            h = (h << 4) + *key++;

            if ((g = (h & 0xF0000000UL)))
            {
                h ^= g >> 24;
                h ^= g;
            }
        }
    }

    return h % limit;
}

globus_result_t
xsd_QName_serialize_wrapper(
    xsd_QName *                         element_qname,
    void *                              instance,
    globus_soap_message_handle_t        message_handle,
    globus_xsd_element_options_t        options)
{
    return xsd_QName_serialize(
        element_qname, (xsd_QName *) instance, message_handle, options);
}

globus_result_t
xsd_QName_serialize_contents_wrapper(
    xsd_QName *                         element_qname,
    void *                              instance,
    globus_soap_message_handle_t        message_handle,
    globus_xsd_element_options_t        options)
{
    return xsd_QName_serialize_contents(
        element_qname, (xsd_QName *) instance, message_handle, options);
}
