/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */


#include "xsd_any.h"
#include "globus_soap_message.h"
#include "globus_soap_message_utils.h"
#include "globus_i_soap_message.h"
#include "globus_xml_buffer.h"
#include "globus_i_soap_message_wildcards.h"
#include "globus_i_xsd.h"

GLOBUS_I_XSD_DEFINE_QNAME(xsd, any);
GLOBUS_I_XSD_DEFINE_TYPE_INIT_ARRAY(xsd_any);
GLOBUS_I_XSD_DEFINE_TYPE_DESTROY_ARRAY(xsd_any);
GLOBUS_I_XSD_DEFINE_TYPE_COPY_ARRAY(xsd_any);
GLOBUS_I_XSD_DEFINE_TYPE_SERIALIZE_ARRAY(xsd_any);
GLOBUS_I_XSD_DEFINE_TYPE_DESERIALIZE_ARRAY(xsd_any);
GLOBUS_I_XSD_DEFINE_TYPE_ARRAY_PUSH(xsd_any);
GLOBUS_I_XSD_DEFINE_TYPE_INIT(xsd_any);
GLOBUS_I_XSD_DEFINE_TYPE_DESTROY(xsd_any);
GLOBUS_I_XSD_DEFINE_TYPE_COPY(xsd_any);
GLOBUS_I_XSD_DEFINE_INIT_CONTENTS_GENERIC(xsd_any);

globus_result_t
xsd_any_serialize_wrapper(
    xsd_QName *                         element_qname,
    void *                              instance,
    globus_soap_message_handle_t        message_handle,
    globus_xsd_element_options_t        options)
{
    return xsd_any_serialize(
        element_qname,
        (xsd_any *) instance,
        message_handle, options);
}

globus_result_t
xsd_any_deserialize_pointer_wrapper(
    xsd_QName *                         element_qname,
    void **                             ip,
    globus_soap_message_handle_t        message_handle,
    globus_xsd_element_options_t        options)
{
    return xsd_any_deserialize_pointer(
        element_qname,
        (xsd_any **) ip,
        message_handle, options);
}

struct globus_xsd_type_info_s xsd_any_info =
{
    &xsd_any_qname,
    xsd_any_serialize_wrapper,
    xsd_any_deserialize_pointer_wrapper,
    xsd_any_init_wrapper,
    xsd_any_destroy_wrapper,
    xsd_any_copy_wrapper,
    xsd_any_init_contents_wrapper,
    xsd_any_destroy_contents_wrapper,
    xsd_any_copy_contents_wrapper,
    sizeof(xsd_any),
    NULL,
    NULL,
    &xsd_any_array_info
};

struct globus_xsd_type_info_s xsd_any_array_info =
{
    &xsd_any_qname,
    xsd_any_array_serialize_wrapper,
    xsd_any_array_deserialize_wrapper,
    xsd_any_array_init_wrapper,
    xsd_any_array_destroy_wrapper,
    xsd_any_array_copy_wrapper,
    xsd_any_array_init_contents_wrapper,
    xsd_any_array_destroy_contents_wrapper,
    xsd_any_array_copy_contents_wrapper,
    sizeof(xsd_any),
    xsd_any_array_push_wrapper,
    NULL,
    NULL
};

void
xsd_any_destroy_contents(
    xsd_any *                           instance)
{
    GlobusFuncName(xsd_any_destroy_contents);
    GlobusSoapMessageDebugEnter();
    
    if(instance)
    {
        if(instance->value && instance->any_info)
        {
            instance->any_info->destroy(instance->value);
            instance->value = NULL;
        }
    
        if(instance->element)
        {
            xsd_QName_destroy(instance->element);
            instance->element = NULL;
        }
    }

    GlobusSoapMessageDebugExit();
}

globus_result_t
xsd_any_copy_contents(
    xsd_any *                           dest,
    const xsd_any *                     src)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(xsd_any_copy_contents);
    GlobusSoapMessageDebugEnter();

    if(dest && src)
    {
        *dest = *src;
        if(src->element)
        {
            result = xsd_QName_copy(&dest->element, src->element);
            if(result != GLOBUS_SUCCESS)
            {
                goto error;
            }
        }
        
        if(src->value && src->any_info)
        {
            result = src->any_info->copy(&dest->value, src->value);
            if(result != GLOBUS_SUCCESS)
            {
                if(dest->element)
                {
                    xsd_QName_destroy(dest->element);
                    dest->element = NULL;
                    goto error;
                }
            }
        }
    }

error:
    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
xsd_any_serialize(
    xsd_QName *                         element_qname,
    xsd_any *                           anyinst,
    globus_soap_message_handle_t        message_handle,
    globus_xsd_element_options_t        options)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(xsd_any_serialize);
    GlobusSoapMessageDebugEnter();

/*     if(!anyinst->element) */
/*     { */
/*         result = GlobusSoapMessageErrorSerializeFailed( */
/*             GLOBUS_FAILURE,  */
/*             "element field in xsd:any variable is NULL.", */
/*             element_qname); */
/*         goto exit; */
/*     } */

    /* serialize contents of type */
    result = anyinst->any_info->serialize(
        anyinst->element,
        anyinst->value,
        message_handle,
        options | GLOBUS_XSD_ELEMENT_TYPE_ATTR);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorSerializeFailed(
            result, 
            "Serialization of xsd:any failed",
            anyinst->element);
        goto exit;
    }

 exit:
 
    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
globus_i_xsd_deserialize_from_element_qname(
    xsd_QName *                         deselem_qname,
    globus_xsd_type_info_t *            found_any_info,
    globus_xsd_type_registry_t          registry,
    globus_soap_message_handle_t        message_handle)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    globus_xsd_type_info_t              any_info = NULL;
    GlobusFuncName(globus_i_xsd_deserialize_from_element_qname);
    GlobusSoapMessageDebugEnter();

    GlobusSoapMessageDebugPrintf(
        GLOBUS_SOAP_MESSAGE_DEBUG_WILDCARDS, 
        ("\tLooking for element {%s}%s:\n",
         deselem_qname->Namespace, 
         deselem_qname->local));

    GlobusSoapMessageDebugPrintf(
        GLOBUS_SOAP_MESSAGE_DEBUG_WILDCARDS, 
        ("\t\tin instance registry\n"));
    
    if(registry)
    {
        /* look for element QName in instance registry */
        result = globus_xsd_type_registry_get(
            registry, deselem_qname, &any_info);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusSoapMessageErrorFailedRegistryLookup(
                result, deselem_qname);
            goto exit;
        }

        if(GlobusSoapMessageDebug(GLOBUS_SOAP_MESSAGE_DEBUG_WILDCARDS))
        {
            if(any_info)
            {
                GlobusSoapMessageDebugPrintf(
                    GLOBUS_SOAP_MESSAGE_DEBUG_WILDCARDS,
                    ("\tFound type info: {%s}%s\n",
                     any_info->type->Namespace,
                     any_info->type->local));
            }
        }
    }

    if(!any_info)
    {
        globus_xsd_type_registry_t      message_registry;

        GlobusSoapMessageDebugPrintf(
            GLOBUS_SOAP_MESSAGE_DEBUG_WILDCARDS, 
            ("\t\tin message registry\n")); 

        globus_soap_message_handle_get_registry(message_handle,
                                                &message_registry);
        if(message_registry)
        {
            result = globus_xsd_type_registry_get(
                message_registry, deselem_qname, &any_info);
            if(result != GLOBUS_SUCCESS)
            {
                result = GlobusSoapMessageErrorFailedRegistryLookup(
                    result, deselem_qname);
                goto exit;
            }

            if(GlobusSoapMessageDebug(GLOBUS_SOAP_MESSAGE_DEBUG_WILDCARDS))
            {
                if(any_info)
                {
                    GlobusSoapMessageDebugPrintf(
                        GLOBUS_SOAP_MESSAGE_DEBUG_WILDCARDS,
                        ("\tFound type info: {%s}%s\n",
                         any_info->type->Namespace,
                         any_info->type->local));
                }
            }
        }
    }

    if(!any_info)
    {
        GlobusSoapMessageDebugPrintf(
            GLOBUS_SOAP_MESSAGE_DEBUG_WILDCARDS, 
            ("\t\tin global registry\n"));

        result = globus_xsd_type_registry_get(
            GLOBUS_GLOBAL_TYPE_REGISTRY, deselem_qname, &any_info);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusSoapMessageErrorFailedRegistryLookup(
                result, deselem_qname);
            goto exit;
        }

        if(GlobusSoapMessageDebug(GLOBUS_SOAP_MESSAGE_DEBUG_WILDCARDS))
        {
            if(any_info)
            {
                GlobusSoapMessageDebugPrintf(
                    GLOBUS_SOAP_MESSAGE_DEBUG_WILDCARDS, 
                    ("Found type info: {%s}%s\n",
                     any_info->type->Namespace,
                     any_info->type->local));
            }
        }
    }

 exit:

    *found_any_info = any_info;

    GlobusSoapMessageDebugExit();
    return result;
}
    

globus_result_t
globus_i_xsd_deserialize_from_xsi_type(
    globus_xsd_type_info_t *            found_any_info,
    globus_xsd_type_registry_t          registry,
    globus_soap_message_handle_t        message_handle)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    globus_xsd_type_info_t              any_info = NULL;
    xsd_QName                           type;
    xmlChar *                           type_string = NULL;
    xmlChar *                           type_prefix = NULL;
    xmlChar *                           type_local = NULL;
    xmlChar *                           type_ns = NULL;
    GlobusFuncName(globus_i_xsd_deserialize_from_xsi_type);
    GlobusSoapMessageDebugEnter();
    
    type.Namespace = NULL;
    type.local = NULL;

    type_string = xmlTextReaderGetAttributeNs(
        message_handle->reader, "type", XSI_NS);
    if(!type_string)
    {
        goto exit;
    }

    type_local = xmlSplitQName2(type_string, &type_prefix);
    if(type_prefix)
    {
        type_ns = xmlTextReaderLookupNamespace(
            message_handle->reader, type_prefix);
        if(!type_ns)
        {
            result = GlobusSoapMessageErrorUndefinedNamespace(type_prefix);
            goto exit;
        }

        type.Namespace = globus_libc_strdup(type_ns);
    }
    else
    {
        type.Namespace = NULL;
    }

    type.local = globus_libc_strdup(type_local);

    GlobusSoapMessageDebugPrintf(
        GLOBUS_SOAP_MESSAGE_DEBUG_WILDCARDS, 
        ("\tLooking for xsi:type {%s}%s:\n",
         type.Namespace, type.local));

    if(!any_info && registry)
    {
        GlobusSoapMessageDebugPrintf(
            GLOBUS_SOAP_MESSAGE_DEBUG_WILDCARDS, 
            ("\t\tin instance registry\n"));
            
        result = globus_xsd_type_registry_get(
            registry, &type, &any_info);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusSoapMessageErrorFailedRegistryLookup(result, &type);
            goto exit;
        }

        if(GlobusSoapMessageDebug(GLOBUS_SOAP_MESSAGE_DEBUG_WILDCARDS))
        {
            if(any_info)
            {
                GlobusSoapMessageDebugPrintf(
                    GLOBUS_SOAP_MESSAGE_DEBUG_WILDCARDS, 
                    ("\t\tFound any info: {%s}%s\n",
                     any_info->type->Namespace, any_info->type->local));
            }
        }
    }

    if(!any_info)
    {
        globus_xsd_type_registry_t      message_registry;
 
        GlobusSoapMessageDebugPrintf(
            GLOBUS_SOAP_MESSAGE_DEBUG_WILDCARDS, ("\t\tin message registry\n"));
        
        globus_soap_message_handle_get_registry(message_handle,
                                                &message_registry);
        if(message_registry)
        {
            result = globus_xsd_type_registry_get(
                message_registry, &type, &any_info);
            if(result != GLOBUS_SUCCESS)
            {
                result = GlobusSoapMessageErrorFailedRegistryLookup(
                    result, &type);
                goto exit;
            }
        }

        if(GlobusSoapMessageDebug(GLOBUS_SOAP_MESSAGE_DEBUG_WILDCARDS))
        {
            if(any_info)
            {
                GlobusSoapMessageDebugPrintf(
                    GLOBUS_SOAP_MESSAGE_DEBUG_WILDCARDS, 
                    ("\tFound any info: {%s}%s\n",
                     any_info->type->Namespace,
                     any_info->type->local));
            }
        }
    }

    if(!any_info)
    {
        GlobusSoapMessageDebugPrintf(
            GLOBUS_SOAP_MESSAGE_DEBUG_WILDCARDS, 
            ("\t\tin global registry\n"));

        result = globus_xsd_type_registry_get(
            GLOBUS_GLOBAL_TYPE_REGISTRY, &type, &any_info);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusSoapMessageErrorFailedRegistryLookup(
                result, &type);
            goto exit;
        }

        if(GlobusSoapMessageDebug(GLOBUS_SOAP_MESSAGE_DEBUG_WILDCARDS))
        {
            if(any_info)
            {
                GlobusSoapMessageDebugPrintf(
                    GLOBUS_SOAP_MESSAGE_DEBUG_WILDCARDS, 
                    ("\tFound any info: {%s}%s\n",
                    any_info->type->Namespace,
                     any_info->type->local));
            }
        }
    }

    if(!any_info)
    {
        any_info = &globus_xml_buffer_contents_info;
    }

 exit:

    *found_any_info = any_info;

    if(type.local)
    {
        free(type.local);
    }

    if(type.Namespace)
    {
        free(type.Namespace);
    }

    if (type_string)
    {
        xmlFree(type_string);
    }

    if (type_local)
    {
        xmlFree(type_local);
    }

    if (type_prefix)
    {
        xmlFree(type_prefix);
    }

    if (type_ns)
    {
        xmlFree(type_ns);
    }

    GlobusSoapMessageDebugEnter();
    return result;
}

globus_result_t
xsd_any_deserialize(
    xsd_QName *                         element_qname,
    xsd_any *                           instance,
    globus_soap_message_handle_t        message_handle,
    globus_xsd_element_options_t        options)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    xsd_QName *                         deselem_qname = NULL;
    int                                 element_exists = 0;
    GlobusFuncName(xsd_any_deserialize);
    GlobusSoapMessageDebugEnter();

    result = globus_soap_message_deserialize_element(
        message_handle, NULL);
    if(result != GLOBUS_SUCCESS)
    {
        if(GlobusSoapMessageStatusFailedWithTextCheck(result))
        {
            element_exists = 0;
            instance->any_info = &globus_xml_buffer_contents_info;
        }
        else if(!GlobusSoapMessageStatusCheck(result))
        {
            result = GlobusSoapMessageErrorDeserializeFailed(
                result, &xsd_any_qname);
            goto exit;
        }
        else
        {
	    goto exit;
        }
    }
    else
    {
        element_exists = 1;
        globus_soap_message_deserialize_push_element(message_handle);
    }

    GlobusSoapMessageDebugPrintf(
        GLOBUS_SOAP_MESSAGE_DEBUG_WILDCARDS, ("\n\nDeserializing xsd:any\n"));

    /* only if an element exists, set the instance->element to it */
    if(element_exists)
    {
        xsd_QName_init(&deselem_qname);
        
        deselem_qname->local = globus_libc_strdup(
            xmlTextReaderConstLocalName(
                message_handle->reader));
        deselem_qname->Namespace = globus_libc_strdup(
            xmlTextReaderConstNamespaceUri(message_handle->reader));
        
        instance->element = deselem_qname;

        /* if instance->any_info isn't set, then look for info in registry
         * based on QName of element
         */
        if(!instance->any_info)
        {
            result = globus_i_xsd_deserialize_from_element_qname(
                deselem_qname,
                &instance->any_info,
                instance->registry,
                message_handle);
            if(result != GLOBUS_SUCCESS)
            {
                result = GlobusSoapMessageErrorDeserializeFailed(
                    result, &xsd_any_qname);
                goto exit;
            }
        }
    }
        
    /* no any_info, look for xsi:type */
    if(!instance->any_info)
    {
        result = globus_i_xsd_deserialize_from_xsi_type(
            &instance->any_info,
            instance->registry,
            message_handle);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusSoapMessageErrorDeserializeFailed(
                result, &xsd_any_qname);
            goto exit;
        }
    }

    if(!instance->any_info)
    {
        instance->any_info = (&globus_xml_buffer_contents_info);
    }

    GlobusSoapMessageDebugPrintf(
        GLOBUS_SOAP_MESSAGE_DEBUG_WILDCARDS, 
        ("Using any info: {%s}%s\n",
         instance->any_info->type->Namespace,
         instance->any_info->type->local));

    result = instance->any_info->deserialize(
        deselem_qname,
        &instance->value,
        message_handle,
        options);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            result, instance->any_info->type);
        goto exit;
    }

 exit:

    if(result != GLOBUS_SUCCESS)
    {
        xsd_any_destroy_contents(instance);
    }

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
xsd_any_deserialize_pointer(
    xsd_QName *                         element_qname,
    xsd_any **                          ip,
    globus_soap_message_handle_t        message_handle,
    globus_xsd_element_options_t        options)
{
    xsd_any *                           instance;
    xsd_QName *                         deselem_qname;
    int                                 element_exists;
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(xsd_any_deserialize_pointer);
    GlobusSoapMessageDebugEnter();

    result = xsd_any_init(&instance);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            result, &xsd_any_qname);
        goto exit;
    }

    result = globus_soap_message_deserialize_element(
        message_handle, NULL);
    if(result != GLOBUS_SUCCESS)
    {
        if(GlobusSoapMessageStatusElementNotFoundCheck(result) ||
           GlobusSoapMessageStatusFailedWithTextCheck(result))
        {
            element_exists = 0;
            instance->any_info = &globus_xml_buffer_contents_info;
            result = GLOBUS_SUCCESS;
        }
        else if(!GlobusSoapMessageStatusCheck(result))
        {
            result = GlobusSoapMessageErrorDeserializeFailed(
                result, &xsd_any_qname);
            goto exit;
        }
        else
        {
            goto exit;
        }
    }
    else
    {
        element_exists = 1;
        globus_soap_message_deserialize_push_element(message_handle);
    }

    GlobusSoapMessageDebugPrintf(
        GLOBUS_SOAP_MESSAGE_DEBUG_WILDCARDS, ("\n\nDeserializing xsd:any\n"));
    
    if(element_exists)
    {
        xsd_QName_init(&deselem_qname);
        
        deselem_qname->local = globus_libc_strdup(
            xmlTextReaderConstLocalName(message_handle->reader));
        deselem_qname->Namespace = globus_libc_strdup(
            xmlTextReaderConstNamespaceUri(message_handle->reader));
        
        instance->element = deselem_qname;

        /* if instance->any_info isn't set, then look for info in registry
         * based on QName of element
         */
        if(!instance->any_info)
        {
            result = globus_i_xsd_deserialize_from_element_qname(
                deselem_qname,
                &instance->any_info,
                instance->registry,
                message_handle);
            if(result != GLOBUS_SUCCESS)
            {
                result = GlobusSoapMessageErrorDeserializeFailed(
                    result, &xsd_any_qname);
                goto exit;
            }
        }
    }
        
    /* no any_info, look for xsi:type */
    if(!instance->any_info)
    {
        result = globus_i_xsd_deserialize_from_xsi_type(
            &instance->any_info,
            instance->registry,
            message_handle);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusSoapMessageErrorDeserializeFailed(
                result, &xsd_any_qname);
            goto exit;
        }
    }

    if(!instance->any_info)
    {
        instance->any_info = (&globus_xml_buffer_contents_info);
    }

    GlobusSoapMessageDebugPrintf(
        GLOBUS_SOAP_MESSAGE_DEBUG_WILDCARDS, 
        ("Using any info: {%s}%s\n",
         instance->any_info->type->Namespace,
         instance->any_info->type->local));

    result = instance->any_info->deserialize(
        deselem_qname,
        &instance->value,
        message_handle,
        options);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            result, instance->any_info->type);
        goto exit;
    }

    *ip = instance;

 exit:

    if(result != GLOBUS_SUCCESS)
    {
        xsd_any_destroy(instance);
    }

    GlobusSoapMessageDebugExit();
    return result;
}

    
