/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */

#include "xsd_anyURI.h"
#include "globus_i_xsd.h"

GLOBUS_I_XSD_DEFINE_QNAME(xsd, anyURI);
GLOBUS_I_XSD_DEFINE_TYPE_FUNCTIONS(xsd_anyURI);
GLOBUS_I_XSD_DEFINE_TYPE_INFO(xsd_anyURI);
GLOBUS_I_XSD_DEFINE_INIT_CONTENTS_GENERIC(xsd_anyURI);
GLOBUS_I_XSD_DEFINE_LIST_SERIALIZE(xsd_anyURI, anyURI);
GLOBUS_I_XSD_DEFINE_LIST_DESERIALIZE(xsd_anyURI, anyURI);

globus_result_t
xsd_anyURI_copy_contents(
    xsd_anyURI *                        dest,
    const xsd_anyURI *                  src)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(xsd_anyURI_copy_contents);
    GlobusSoapMessageDebugEnter();

    if(dest)
    {
        if(!src || !*src)
        {
            *dest = NULL;
        }
        else
        {
            *dest = globus_libc_strdup(*src);
            if(!*dest)
            {
                result = GlobusSoapMessageErrorOutOfMemory;
            }
        }
    }

    GlobusSoapMessageDebugExit();
    return result;
}

void
xsd_anyURI_destroy_contents(
    xsd_anyURI * instance)
{
    GlobusFuncName(xsd_anyURI_destroy_contents);
    GlobusSoapMessageDebugEnter();

    if(*instance)
    {
        free(*instance);
        *instance = NULL;
    }

    GlobusSoapMessageDebugExit();
}
    
globus_result_t
xsd_anyURI_serialize_contents(
    xsd_QName *                         element_name,
    xsd_anyURI *                        instance,
    globus_soap_message_handle_t        message_handle,
    globus_xsd_element_options_t        options)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(xsd_anyURI_serialize_contents);
    GlobusSoapMessageDebugEnter();

    /* serialize contents of type */
    result = globus_soap_message_serialize_string(
        message_handle, instance);

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
xsd_anyURI_deserialize_contents(
    xsd_QName *                         element_qname,
    xsd_anyURI *                        instance,
    globus_soap_message_handle_t        message_handle,
    globus_xsd_element_options_t        options)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(xsd_anyURI_deserialize_contents);
    GlobusSoapMessageDebugEnter();
    
    result = globus_soap_message_deserialize_anyURI(
        message_handle, instance);

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
xsd_anyURI_serialize_attribute(
    xsd_QName *                         element_qname,
    xsd_anyURI *                        instance,
    globus_soap_message_handle_t        message_handle,
    globus_xsd_element_options_t        options)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(xsd_anyURI_serialize_attribute);
    GlobusSoapMessageDebugEnter();

    result = globus_soap_message_serialize_string_attribute(
        message_handle, element_qname, instance);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorSerializeFailed(
            result, NULL, element_qname);
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
xsd_anyURI_deserialize_attribute(
    xsd_QName *                         element_qname,
    xsd_anyURI *                        ip,
    globus_soap_message_handle_t        message_handle,
    globus_xsd_element_options_t        options)
{
    globus_result_t                 result = GLOBUS_SUCCESS;
    GlobusFuncName(xsd_anyURI_deserialize_attribute);
    GlobusSoapMessageDebugEnter();

    result = globus_soap_message_deserialize_string_attribute(
        message_handle, element_qname, ip);
    if(GlobusSoapMessageStatusAttributeNotFoundCheck(result))
    {
        result = GLOBUS_SUCCESS;
        goto exit;
    }

    if(result != GLOBUS_SUCCESS)
    {
        goto error_exit;
    }

    goto exit;

 error_exit:

    GlobusSoapMessageErrorDeserializeFailed(result, element_qname);

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
xsd_anyURI_deserialize_attribute_pointer(
    xsd_QName *                         element_qname,
    xsd_anyURI **                       ip,
    globus_soap_message_handle_t        message_handle,
    globus_xsd_element_options_t        options)
{
    xsd_anyURI *                        instance = NULL;
    globus_result_t                 result = GLOBUS_SUCCESS;
    GlobusFuncName(xsd_anyURI_deserialize_attribute_pointer);
    GlobusSoapMessageDebugEnter();

    *ip = NULL;

    result = xsd_anyURI_init(&instance);

    if(result != GLOBUS_SUCCESS)
    {
        goto error_exit;
    }

    result = globus_soap_message_deserialize_string_attribute(
        message_handle, element_qname, instance);
    if(GlobusSoapMessageStatusAttributeNotFoundCheck(result))
    {
	xsd_anyURI_destroy(instance);
        result = GLOBUS_SUCCESS;
        goto exit;
    }

    if(result != GLOBUS_SUCCESS)
    {
        goto error_exit;
    }

    *ip = instance;

    goto exit;

 error_exit:

    GlobusSoapMessageErrorDeserializeFailed(result, element_qname);

 exit:

    if(result != GLOBUS_SUCCESS && instance)
    {
        xsd_anyURI_destroy(instance);
        *ip = NULL;
    }

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
xsd_anyURI_list_serialize_contents(
    xsd_anyURI_array *                  inst,
    globus_soap_message_handle_t        message_handle,
    globus_xsd_element_options_t        options)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(xsd_anyURI_list_serialize);
    GlobusSoapMessageDebugEnter();

    result = globus_soap_message_serialize_string_list(
        message_handle,
        (xsd_string_array *)inst);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorSerializeFailed(
            result, "could not serialize as list", &xsd_anyURI_qname);
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
xsd_anyURI_list_deserialize_contents(
    xsd_anyURI_array *                  ip,
    globus_soap_message_handle_t        message_handle,
    globus_xsd_element_options_t        options)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(xsd_anyURI_list_deserialize_contents);
    GlobusSoapMessageDebugEnter();

    result = globus_soap_message_deserialize_string_list(
        message_handle, (xsd_string_array *) ip);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            result, &xsd_anyURI_qname);
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
xsd_anyURI_list_serialize_attribute(
    xsd_QName *                         attr_qname,
    xsd_anyURI_array *                  inst,
    globus_soap_message_handle_t        message_handle,
    globus_xsd_element_options_t        options)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(xsd_anyURI_list_serialize_attribute);
    GlobusSoapMessageDebugEnter();

    result = globus_soap_message_serialize_string_attribute_list(
        message_handle, attr_qname, (xsd_string_array *) inst);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorSerializeFailed(
            result, "could not serialize list of xsd:anyURI as attribute",
            &xsd_anyURI_qname);
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
xsd_anyURI_list_deserialize_attribute(
    xsd_QName *                         attr_qname,
    xsd_anyURI_array *                  inst,
    globus_soap_message_handle_t        message_handle,
    globus_xsd_element_options_t        options)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(xsd_anyURI_list_deserialize_attribute);
    GlobusSoapMessageDebugEnter();

    result = globus_soap_message_deserialize_string_attribute_list(
        message_handle, attr_qname, (xsd_string_array *) inst);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            result, &xsd_anyURI_qname);
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}
