/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */

#include "globus_i_xsd.h"
#include "globus_i_xsd_type_info.h"
#include "globus_soap_message.h"

GLOBUS_I_XSD_DEFINE_QNAME(xsd, base64Binary);
GLOBUS_I_XSD_DEFINE_TYPE_INFO(xsd_base64Binary);
GLOBUS_I_XSD_DEFINE_TYPE_FUNCTIONS(xsd_base64Binary);
GLOBUS_I_XSD_DEFINE_ATTR_FUNCTIONS(xsd_base64Binary, base64Binary);

GLOBUS_I_XSD_DEFINE_INIT_CONTENTS_GENERIC(xsd_base64Binary);

void
xsd_base64Binary_destroy_contents(
    xsd_base64Binary *                  instance)
{
    GlobusFuncName(xsd_base64Binary_destroy_contents);
    GlobusSoapMessageDebugEnter();

    if(instance)
    {
        if(instance->value)
        {
            free(instance->value);
            instance->value = NULL;
        }
    }

    GlobusSoapMessageDebugExit();
}

globus_result_t
xsd_base64Binary_copy_contents(
    xsd_base64Binary *                  dest,
    const xsd_base64Binary *            src)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(xsd_base64Binary_copy_contents);
    GlobusSoapMessageDebugEnter();

    if(dest && src)
    {
        dest->length = src->length;
        dest->value = globus_libc_malloc(src->length);
        if(!dest->value)
        {
            result = GlobusSoapMessageErrorOutOfMemory;
        }
        else
        {
            memcpy(dest->value, src->value, src->length);
        }
    }

    GlobusSoapMessageDebugExit();

    return result;
}

globus_result_t
xsd_base64Binary_serialize_contents(
    xsd_QName *                         element_qname,
    xsd_base64Binary *                  instance,
    globus_soap_message_handle_t        soap_message_handle,
    globus_xsd_element_options_t        options)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(xsd_base64Binary_serialize_contents);
    GlobusSoapMessageDebugEnter();

    /* serialize contents of type */
    result = globus_soap_message_serialize_base64Binary(
        soap_message_handle, instance);

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
xsd_base64Binary_deserialize_contents(
    xsd_QName *                         element_qn,
    xsd_base64Binary *                  instance,
    globus_soap_message_handle_t        soap_message_handle,
    globus_xsd_element_options_t        options)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(xsd_base64Binary_deserialize_contents);
    GlobusSoapMessageDebugEnter();
    
    result = globus_soap_message_deserialize_base64Binary(
        soap_message_handle, instance);

    GlobusSoapMessageDebugExit();
    return result;
}
