/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */

#include "globus_i_xsd.h"
#include "globus_i_xsd_type_info.h"
#include "globus_soap_message.h"
#include "xsd_language.h"

GLOBUS_I_XSD_DEFINE_QNAME(xsd, language);
GLOBUS_I_XSD_DEFINE_TYPE_INFO(xsd_language);
GLOBUS_I_XSD_DEFINE_TYPE_FUNCTIONS(xsd_language);
GLOBUS_I_XSD_DEFINE_INIT_CONTENTS_GENERIC(xsd_language);

globus_result_t
xsd_language_copy_contents(
    xsd_language *                      dest,
    const xsd_language *                src)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(xsd_language_copy_contents);
    GlobusSoapMessageDebugEnter();

    if(dest)
    {
        if(!src || !*src)
        {
            *dest = NULL;
        }
        else
        {
            *dest = globus_libc_strdup(*src);
            if(!*dest)
            {
                result = GlobusSoapMessageErrorOutOfMemory;
            }
        }
    }

    GlobusSoapMessageDebugExit();
    return result;
}
void
xsd_language_destroy_contents(
    xsd_language *                      instance)
{
    GlobusFuncName(xsd_language_destroy_contents);
    GlobusSoapMessageDebugEnter();

    if(*instance)
    {
        free(*instance);
        *instance = NULL;
    }

    GlobusSoapMessageDebugExit();
}

globus_result_t
xsd_language_serialize_contents(
    xsd_QName *                         element,
    xsd_language *                      instance,
    globus_soap_message_handle_t        soap_message_handle,
    globus_xsd_element_options_t        options)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(xsd_language_serialize_contents);
    GlobusSoapMessageDebugEnter();

    /* serialize contents of type */
    result = globus_soap_message_serialize_string(
        soap_message_handle, instance);

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
xsd_language_deserialize_contents(
    xsd_QName *                         eqn,
    xsd_language *                      instance,
    globus_soap_message_handle_t        soap_message_handle,
    globus_xsd_element_options_t        options)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(xsd_language_deserialize_contents);
    GlobusSoapMessageDebugEnter();
    
    result = globus_soap_message_deserialize_string(
        soap_message_handle, instance);

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
xsd_language_serialize_attribute(
    xsd_QName *                         element_qname,
    xsd_language *                      instance,
    globus_soap_message_handle_t        message_handle,
    globus_xsd_element_options_t        options)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(xsd_language_serialize_attribute);
    GlobusSoapMessageDebugEnter();

    result = globus_soap_message_serialize_string_attribute(
        message_handle, element_qname, instance);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorSerializeFailed(
            result, NULL, element_qname);
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
xsd_language_deserialize_attribute(
    xsd_QName *                         element_qname,
    xsd_language *                      ip,
    globus_soap_message_handle_t        message_handle,
    globus_xsd_element_options_t        options)
{
    globus_result_t                 result = GLOBUS_SUCCESS;
    GlobusFuncName(xsd_language_deserialize_attribute);
    GlobusSoapMessageDebugEnter();

    result = globus_soap_message_deserialize_string_attribute(
        message_handle, element_qname, ip);
    if(GlobusSoapMessageStatusAttributeNotFoundCheck(result))
    {
        result = GLOBUS_SUCCESS;
        goto exit;
    }

    if(result != GLOBUS_SUCCESS)
    {
        goto error_exit;
    }

    goto exit;

 error_exit:

    GlobusSoapMessageErrorDeserializeFailed(result, element_qname);

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
xsd_language_deserialize_attribute_pointer(
    xsd_QName *                         element_qname,
    xsd_language **                     ip,
    globus_soap_message_handle_t        message_handle,
    globus_xsd_element_options_t        options)
{
    xsd_language *                  instance = NULL;
    globus_result_t                 result = GLOBUS_SUCCESS;
    GlobusFuncName(xsd_language_deserialize_attribute_pointer);
    GlobusSoapMessageDebugEnter();

    *ip = NULL;

    result = xsd_language_init(&instance);

    if(result != GLOBUS_SUCCESS)
    {
        goto error_exit;
    }

    result = globus_soap_message_deserialize_string_attribute(
        message_handle, element_qname, instance);
    if(GlobusSoapMessageStatusAttributeNotFoundCheck(result))
    {
        result = GLOBUS_SUCCESS;
        goto exit;
    }

    if(result != GLOBUS_SUCCESS)
    {
        goto error_exit;
    }

    *ip = instance;

    goto exit;

 error_exit:

    GlobusSoapMessageErrorDeserializeFailed(result, element_qname);

 exit:

    if(result != GLOBUS_SUCCESS && instance)
    {
        xsd_language_destroy(instance);
        *ip = NULL;
    }

    GlobusSoapMessageDebugExit();
    return result;
}
