/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */

#include "globus_i_xsd.h"
#include "globus_i_xsd_type_info.h"
#include "globus_soap_message.h"

GLOBUS_I_XSD_DEFINE_QNAME(xsd, string);
GLOBUS_I_XSD_DEFINE_TYPE_INFO(xsd_string);
GLOBUS_I_XSD_DEFINE_TYPE_FUNCTIONS(xsd_string);
GLOBUS_I_XSD_DEFINE_ATTR_FUNCTIONS(xsd_string, string);
GLOBUS_I_XSD_DEFINE_INIT_CONTENTS_GENERIC(xsd_string);
GLOBUS_I_XSD_DEFINE_LIST_FUNCTIONS(xsd_string, string);
GLOBUS_I_XSD_DEFINE_LIST_ATTR_FUNCTIONS(xsd_string, string);

globus_result_t
xsd_string_copy_contents(
    xsd_string *                        dest,
    const xsd_string *                  src)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(xsd_string_copy_contents);
    GlobusSoapMessageDebugEnter();
    
    if(dest)
    {
        if(!src || !*src)
        {
            *dest = NULL;
        }
        else
        {
            *dest = globus_libc_strdup(*src);
            if(!*dest)
            {
                result = GlobusSoapMessageErrorOutOfMemory;
            }
        }
    }

    GlobusSoapMessageDebugExit();
    return result;
}

void
xsd_string_destroy_contents(
    xsd_string *                        instance)
{
    GlobusFuncName(xsd_string_destroy_contents);
    GlobusSoapMessageDebugEnter();

    if(*instance)
    {
        free(*instance);
        *instance = NULL;
    }

    GlobusSoapMessageDebugExit();
}

globus_result_t
xsd_string_serialize_contents(
    xsd_QName *                         element,
    xsd_string *                        instance,
    globus_soap_message_handle_t        soap_message_handle,
    globus_xsd_element_options_t        options)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(xsd_string_serialize_contents);
    GlobusSoapMessageDebugEnter();

    /* serialize contents of type */
    result = globus_soap_message_serialize_string(
        soap_message_handle, instance);

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
xsd_string_deserialize_contents(
    xsd_QName *                         eqn,
    xsd_string *                        instance,
    globus_soap_message_handle_t        soap_message_handle,
    globus_xsd_element_options_t        options)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(xsd_string_deserialize_contents);
    GlobusSoapMessageDebugEnter();

    result = globus_soap_message_deserialize_string(
        soap_message_handle, instance);

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
xsd_string_init_cstr(
    xsd_string **                       ip,
    char *                              str)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(xsd_string_init_cstr);
    GlobusSoapMessageDebugEnter();

    result = xsd_string_init(ip);
    if(result == GLOBUS_SUCCESS)
    {
        **ip = str;
    }

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
xsd_string_init_contents_cstr(
    xsd_string *                        instance,
    char *                              str)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(xsd_string_init_contents_cstr);
    GlobusSoapMessageDebugEnter();

    *instance = str;

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
xsd_string_copy_cstr(
    xsd_string **                       ip,
    const char *                        str)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(xsd_string_copy_cstr);
    GlobusSoapMessageDebugEnter();

    result = xsd_string_copy(ip, (const xsd_string *) &str);

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
xsd_string_copy_contents_cstr(
    xsd_string *                        instance,
    const char *                        str)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(xsd_string_copy_cstr);
    GlobusSoapMessageDebugEnter();

    result = xsd_string_copy_contents(instance, (const xsd_string *) &str);

    GlobusSoapMessageDebugExit();
    return result;
}
