/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */

#include "globus_soap_message.h"
#include "globus_xsd_type_info.h"
#include "xsd_string.h"

typedef int (*test_case)(void);

#define SIZEOF_ARRAY(x) sizeof(x) / sizeof(x[0])

xsd_QName foo_name = 
{
    "http://www.globus.org/namespaces",
    "foo"
};

static
int
test_init()
{
    int rc;
    rc = globus_module_activate(GLOBUS_COMMON_MODULE);
    if (rc != 0)
    {
        return rc;
    }
    rc = globus_module_activate(GLOBUS_SOAP_MESSAGE_MODULE);
    return rc;
}

static
void
test_finalize()
{
    globus_module_deactivate(GLOBUS_COMMON_MODULE);
    globus_module_deactivate(GLOBUS_SOAP_MESSAGE_MODULE);
}

int
keyeq_missing_namespace(void)
{
    xsd_QName                   key1 = { NULL, "local" };
    xsd_QName                   key2 = { NULL, "local" };
    int                         rc;

    rc = test_init();

    if (rc != GLOBUS_SUCCESS)
    {
        goto out;
    }

    if(!xsd_QName_keyeq(&key1, &key2))
    {
        rc = 1;
        goto finalize_out;
    }
    if(xsd_QName_keyeq(&foo_name, &key2))
    {
        rc = 1;
        goto finalize_out;
    }
    if(xsd_QName_keyeq(&key1, &foo_name))
    {
        rc = 1;
        goto finalize_out;
    }
    if(xsd_QName_keyeq(&key1, NULL))
    {
        rc = 1;
        goto finalize_out;
    }

finalize_out:
    test_finalize();
out:
    return rc;
}

int
keyeq_missing_local(void)
{
    xsd_QName                   key1 = { "namespace", NULL };
    xsd_QName                   key2 = { "namespace", NULL };
    int                         rc;

    rc = test_init();

    if (rc != GLOBUS_SUCCESS)
    {
        goto out;
    }

    if(!xsd_QName_keyeq(&key1, &key2))
    {
        rc = 1;
        goto finalize_out;
    }
    if(xsd_QName_keyeq(&foo_name, &key2))
    {
        rc = 1;
        goto finalize_out;
    }
    if(xsd_QName_keyeq(&key1, &foo_name))
    {
        rc = 1;
        goto finalize_out;
    }
    if(xsd_QName_keyeq(&key1, NULL))
    {
        rc = 1;
        goto finalize_out;
    }

finalize_out:
    test_finalize();
out:
    return rc;
}

int
hash_missing_namespace(void)
{
    xsd_QName                   key1 = { NULL, "local" };
    int                         rc;

    rc = test_init();

    if (rc != GLOBUS_SUCCESS)
    {
        goto out;
    }

    xsd_QName_hash(&key1, 16);

    rc = 0;

    test_finalize();
out:
    return rc;
}
int
hash_missing_local(void)
{
    xsd_QName                   key1 = { "namespace", NULL };
    int                         rc;

    rc = test_init();

    if (rc != GLOBUS_SUCCESS)
    {
        goto out;
    }

    xsd_QName_hash(&key1, 16);

    rc = 0;

    test_finalize();
out:
    return rc;
}

int
hash_quality_test(void)
{
    globus_xsd_type_info_t  types[] = {
        &xsd_anyURI_info,
        &xsd_base64Binary_info,
        &xsd_boolean_info,
        &xsd_byte_info,
        &xsd_date_info,
        &xsd_dateTime_info,
        &xsd_decimal_info,
        &xsd_double_info,
        &xsd_float_info,
        &xsd_hexBinary_info,
        &xsd_int_info,
        &xsd_integer_info,
        &xsd_long_info,
        &xsd_negativeInteger_info,
        &xsd_nonNegativeInteger_info,
        &xsd_nonPositiveInteger_info,
        &xsd_positiveInteger_info,
        &xsd_short_info,
        &xsd_string_info,
        &xsd_time_info,
        &xsd_unsignedByte_info,
        &xsd_unsignedInt_info,
        &xsd_unsignedLong_info,
        &xsd_unsignedShort_info
    };
    int bucket_lengths[17];
    int i;
    int max_bucket_length = 0;
    int rc;

    memset(bucket_lengths, '\0', sizeof(bucket_lengths));

    rc = test_init();

    if (rc != GLOBUS_SUCCESS)
    {
        goto out;
    }

    for (i = 0; i < SIZEOF_ARRAY(types); i++)
    {
        int tmp;

        tmp = xsd_QName_hash(types[i]->type, SIZEOF_ARRAY(bucket_lengths));
        bucket_lengths[tmp]++;
    }

    for (i = 0; i < SIZEOF_ARRAY(bucket_lengths); i++)
    {
        if (bucket_lengths[i] > max_bucket_length)
        {
            max_bucket_length = bucket_lengths[i];
        }
    }

    if (max_bucket_length > 3)
    {
        rc = 1;
        fprintf(stderr, "bucket length is %d\n", max_bucket_length);
    }

    test_finalize();
out:
    return rc;
}

int main()
{
int i;
    int rc;
    int failed = 0;
    test_case tests[] =
    {
        keyeq_missing_namespace,
        keyeq_missing_local,
        hash_missing_namespace,
        hash_missing_local,
        hash_quality_test
    };

    printf("1..%d\n", SIZEOF_ARRAY(tests));

    for (i = 0; i < SIZEOF_ARRAY(tests); i++) {
        rc = (tests[i])();
        if (rc == 0)
        {
            printf("ok\n");
        }
        else
        {
            printf("not ok\n");
            failed++;
        }
    }
    return failed;

}
