/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */


#ifndef _GLOBUS_I_WSDL_CONFIG_H_
#define _GLOBUS_I_WSDL_CONFIG_H_

#include "globus_wsdl_config.h"

#define GlobusWSDLConfigErrorFileOpen(_FILENAME_)                              \
  globus_error_put(                                                            \
      globus_error_construct_error(                                            \
          GLOBUS_WSDL_CONFIG_MODULE,                                           \
          GLOBUS_NULL,                                                         \
          GLOBUS_WSDL_CONFIG_ERROR_FILE,                                       \
          __FILE__,                                                            \
          _globus_func_name,                                                   \
          __LINE__,                                                            \
          "Failed to open config file %s", _FILENAME_))                        \

#define GlobusWSDLConfigErrorNoEqualsProp(_PROP_)                              \
    globus_error_put(                                                          \
        globus_error_construct_error(                                          \
            GLOBUS_WSDL_CONFIG_MODULE,                                         \
            GLOBUS_NULL,                                                       \
            GLOBUS_WSDL_CONFIG_ERROR_NO_EQUALS,                                \
            __FILE__,                                                          \
            _globus_func_name,                                                 \
            __LINE__,                                                          \
            "No '=' found in property definition: %s",                         \
            _PROP_))                                               

#define GlobusWSDLConfigErrorNoEndQuoteProp(_PROP_, _DELIM_)                   \
    globus_error_put(                                                          \
        globus_error_construct_error(                                          \
            GLOBUS_WSDL_CONFIG_MODULE,                                         \
            GLOBUS_NULL,                                                       \
            GLOBUS_WSDL_CONFIG_ERROR_NO_ENDQUOTE,                              \
            __FILE__,                                                          \
            _globus_func_name,                                                 \
            __LINE__,                                                          \
            "No end quote (%s) found in property definition: %s",              \
            _DELIM_, _PROP_))                                                  \

#define GlobusWSDLConfigErrorNoEquals(_FILE_, _LINENUM_)                       \
    globus_error_put(                                                          \
        globus_error_construct_error(                                          \
            GLOBUS_WSDL_CONFIG_MODULE,                                         \
            GLOBUS_NULL,                                                       \
            GLOBUS_WSDL_CONFIG_ERROR_NO_EQUALS,                                \
            __FILE__,                                                          \
            _globus_func_name,                                                 \
            __LINE__,                                                          \
            "No '=' found in property definition at %s:%d",                    \
            _FILE_, _LINENUM_))                                                \

#define GlobusWSDLConfigErrorNoEndQuote(_FILE_, _LINENUM_, _DELIM_)            \
    globus_error_put(                                                          \
        globus_error_construct_error(                                          \
            GLOBUS_WSDL_CONFIG_MODULE,                                         \
            GLOBUS_NULL,                                                       \
            GLOBUS_WSDL_CONFIG_ERROR_NO_ENDQUOTE,                              \
            __FILE__,                                                          \
            _globus_func_name,                                                 \
            __LINE__,                                                          \
            "No end quote (%s) found in property definition at %s:%d",         \
            _DELIM_, _FILE_, _LINENUM_))                                       \

#define GlobusWSDLConfigErrorHashInsertFailed()                                \
    globus_error_put(                                                          \
        globus_error_construct_error(                                          \
            GLOBUS_WSDL_CONFIG_MODULE,                                         \
            GLOBUS_NULL,                                                       \
            GLOBUS_WSDL_CONFIG_ERROR_HASH_INSERT,                              \
            __FILE__,                                                          \
            _globus_func_name,                                                 \
            __LINE__,                                                          \
            "Failed to insert property entry into hashtable"))                 \

#define GlobusWSDLConfigErrorNoCurly()                                         \
    globus_error_put(                                                          \
        globus_error_construct_error(                                          \
            GLOBUS_WSDL_CONFIG_MODULE,                                         \
            GLOBUS_NULL,                                                       \
            GLOBUS_WSDL_CONFIG_ERROR_NO_CURLY,                                 \
            __FILE__,                                                          \
            _globus_func_name,                                                 \
            __LINE__,                                                          \
            "No closing curly bracket '}' found"))                             \

#define GlobusWSDLConfigErrorNoEnv(_ENVSTRING_)                                \
    globus_error_put(                                                          \
        globus_error_construct_error(                                          \
            GLOBUS_WSDL_CONFIG_MODULE,                                         \
            GLOBUS_NULL,                                                       \
            GLOBUS_WSDL_CONFIG_ERROR_NO_ENV,                                   \
            __FILE__,                                                          \
            _globus_func_name,                                                 \
            __LINE__,                                                          \
            "No property or environment variable found named %s",              \
            _ENVSTRING_))

#define GlobusWSDLConfigErrorExpandingRefs(_RESULT_, _FILENAME_, _LINENUM_)    \
    globus_error_put(                                                          \
        globus_error_construct_error(                                          \
            GLOBUS_WSDL_CONFIG_MODULE,                                         \
            (_RESULT_ ? (globus_error_get(_RESULT_)) : GLOBUS_NULL),           \
             GLOBUS_WSDL_CONFIG_ERROR_EXPANDING_REFS,                          \
             __FILE__,                                                         \
             _globus_func_name,                                                \
             __LINE__,                                                         \
             "Failed to expand property references in property def at %s:%d",  \
             _FILENAME_, _LINENUM_))

#define GlobusWSDLConfigErrorOutOfMemory               \
    globus_error_put(                                  \
        globus_error_construct_error(                  \
            GLOBUS_WSDL_CONFIG_MODULE,                 \
            NULL,                                      \
            GLOBUS_WSDL_CONFIG_ERROR_OUT_OF_MEMORY,    \
            __FILE__,                                  \
            _globus_func_name,                         \
            __LINE__,                                  \
            "Failed to allocate memory"))

GlobusDebugDeclare(GLOBUS_WSDL_CONFIG);

#ifdef GLOBUS_DEBUG                                                            

#define GlobusWSDLConfigDebugEnter()                                           \
    GlobusDebugPrintf(GLOBUS_WSDL_CONFIG, GLOBUS_WSDL_CONFIG_DEBUG_TRACE,      \
                      ("[%s] Entering\n", _globus_func_name))                  \

#define GlobusWSDLConfigDebugExit()                                            \
    GlobusDebugPrintf(GLOBUS_WSDL_CONFIG, GLOBUS_WSDL_CONFIG_DEBUG_TRACE,      \
                      ("[%s] Exiting\n", _globus_func_name))                   \

#define GlobusWSDLConfigDebug(_LEVEL_)                                         \
    GlobusDebugTrue(GLOBUS_WSDL_CONFIG, _LEVEL_)                               \

#define GlobusWSDLConfigDebugPrintf(_LEVEL_, _MESSAGE_)                        \
    GlobusDebugPrintf(GLOBUS_WSDL_CONFIG, _LEVEL_, _MESSAGE_)

#else

#define GlobusWSDLConfigDebugEnter()
#define GlobusWSDLConfigDebugExit()
#define GlobusWSDLConfigDebug(_LEVEL_) 0
#define GlobusWSDLConfigDebugPrintf(_LEVEL_, _MESSAGE_)

#endif

#endif
