/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */


#ifndef _GLOBUS_I_WSDL_PARSER_H_
#define _GLOBUS_I_WSDL_PARSER_H_

#include "globus_wsdl_parser.h"

#define GlobusWSDLErrorJSRuntime(_RESULT_)              \
    globus_error_put(                                   \
        globus_error_construct_error(                   \
            GLOBUS_WSDL_PARSER_MODULE,                  \
            (_RESULT_ ? globus_error_get(_RESULT_) :    \
            GLOBUS_NULL),                               \
            GLOBUS_WSDL_ERROR_JSRUNTIME,                \
            __FILE__,                                   \
            _globus_func_name,                          \
            __LINE__,                                   \
            "Failed to intialize javascript runtime"))

#define GlobusWSDLErrorJSContext(_RESULT_)              \
    globus_error_put(                                   \
        globus_error_construct_error(                   \
            GLOBUS_WSDL_PARSER_MODULE,                  \
            (_RESULT_ ? globus_error_get(_RESULT_) :    \
            GLOBUS_NULL),                               \
            GLOBUS_WSDL_ERROR_JSCONTEXT,                \
            __FILE__,                                   \
            _globus_func_name,                          \
            __LINE__,                                   \
            "Failed to initialize javascript context"))

#define GlobusWSDLErrorJSLoadSchema(_RESULT_)           \
    globus_error_put(                                   \
        globus_error_construct_error(                   \
            GLOBUS_WSDL_PARSER_MODULE,                  \
            (_RESULT_ ? globus_error_get(_RESULT_) :    \
            GLOBUS_NULL),                               \
            GLOBUS_WSDL_ERROR_JSLOAD,                   \
            __FILE__,                                   \
            _globus_func_name,                          \
            __LINE__,                                   \
            "Failed to load WSDL schema "               \
            "into javascript"))

#define GlobusWSDLErrorTemplateLoad(_RESULT_, _FN_)     \
    globus_error_put(                                   \
        globus_error_construct_error(                   \
            GLOBUS_WSDL_PARSER_MODULE,                  \
            (_RESULT_ ? globus_error_get(_RESULT_) :    \
            GLOBUS_NULL),                               \
            GLOBUS_WSDL_ERROR_TEMPLATE_LOAD,            \
            __FILE__,                                   \
            _globus_func_name,                          \
            __LINE__,                                   \
            "Failed to load and "                       \
            "execute template: %s",                     \
            _FN_))

#define GlobusWSDLErrorScanningTypes(_RESULT_, _NAME_)                        \
    globus_error_put(                                                         \
        globus_error_construct_error(GLOBUS_WSDL_PARSER_MODULE,               \
                                     (_RESULT_ ? globus_error_get(_RESULT_) : \
                                      GLOBUS_NULL),                           \
                                     GLOBUS_WSDL_ERROR_SCANNING_TYPES,        \
                                     __FILE__,                                \
                                     _globus_func_name,                       \
                                     __LINE__,                                \
                                     "Failed while scanning xsd type: %s",    \
                                     _NAME_))

#define GlobusWSDLErrorScanningImports(_RESULT_, _NAME_)                      \
    globus_error_put(                                                         \
        globus_error_construct_error(GLOBUS_WSDL_PARSER_MODULE,               \
                                     (_RESULT_ ? globus_error_get(_RESULT_) : \
                                      GLOBUS_NULL),                           \
                                     GLOBUS_WSDL_ERROR_SCANNING_IMPORTS,      \
                                     __FILE__,                                \
                                     _globus_func_name,                       \
                                     __LINE__,                                \
                                     "Failed while scanning xsd import: %s", \
                                     _NAME_))

#define GlobusWSDLErrorScanningElements(_RESULT_, _NAME_)                     \
    globus_error_put(                                                         \
        globus_error_construct_error(GLOBUS_WSDL_PARSER_MODULE,               \
                                     (_RESULT_ ? globus_error_get(_RESULT_) : \
                                      GLOBUS_NULL),                           \
                                     GLOBUS_WSDL_ERROR_SCANNING_ELEMENTS,     \
                                     __FILE__,                                \
                                     _globus_func_name,                       \
                                     __LINE__,                                \
                                     "Failed while scanning xsd element: %s", \
                                     _NAME_))

#define GlobusWSDLErrorScanningAttributes(_RESULT_, _NAME_)     \
    globus_error_put(                                           \
        globus_error_construct_error(                           \
            GLOBUS_WSDL_PARSER_MODULE,                          \
            (_RESULT_ ? globus_error_get(_RESULT_) :            \
            GLOBUS_NULL),                                       \
            GLOBUS_WSDL_ERROR_SCANNING_ATTRIBUTES,              \
            __FILE__,                                           \
            _globus_func_name,                                  \
            __LINE__,                                           \
            "Failed while scanning xsd attribute: %s",          \
            _NAME_))

#define GlobusWSDLErrorScanningMessages(_RESULT_, _NAME_)       \
    globus_error_put(                                           \
        globus_error_construct_error(                           \
            GLOBUS_WSDL_PARSER_MODULE,                          \
            (_RESULT_ ? globus_error_get(_RESULT_) :            \
            GLOBUS_NULL),                                       \
            GLOBUS_WSDL_ERROR_SCANNING_MESSAGES,                \
            __FILE__,                                           \
            _globus_func_name,                                  \
            __LINE__,                                           \
            "Failed while scanning xsd messages: %s",           \
            _NAME_))

#define GlobusWSDLErrorScanningPortTypes(_RESULT_, _NAME_)      \
    globus_error_put(                                           \
        globus_error_construct_error(                           \
            GLOBUS_WSDL_PARSER_MODULE,                          \
            (_RESULT_ ? globus_error_get(_RESULT_) :            \
            GLOBUS_NULL),                                       \
            GLOBUS_WSDL_ERROR_SCANNING_PORTTYPES,               \
            __FILE__,                                           \
            _globus_func_name,                                  \
            __LINE__,                                           \
            "Failed while scanning xsd port type: %s",          \
            _NAME_))

#define GlobusWSDLErrorScanningBindings(_RESULT_, _NAME_)                     \
    globus_error_put(                                                         \
        globus_error_construct_error(GLOBUS_WSDL_PARSER_MODULE,               \
                                     (_RESULT_ ? globus_error_get(_RESULT_) : \
                                      GLOBUS_NULL),                           \
                                     GLOBUS_WSDL_ERROR_SCANNING_BINDINGS,     \
                                     __FILE__,                                \
                                     _globus_func_name,                       \
                                     __LINE__,                                \
                                     "Failed while scanning xsd binding: %s", \
                                     _NAME_))

#define GlobusWSDLErrorScanningServices(_RESULT_, _NAME_)                     \
    globus_error_put(                                                         \
        globus_error_construct_error(GLOBUS_WSDL_PARSER_MODULE,               \
                                     (_RESULT_ ? globus_error_get(_RESULT_) : \
                                      GLOBUS_NULL),                           \
                                     GLOBUS_WSDL_ERROR_SCANNING_SERVICES,     \
                                     __FILE__,                                \
                                     _globus_func_name,                       \
                                     __LINE__,                                \
                                     "Failed while scanning xsd service: %s", \
                                     _NAME_))

#define GlobusWSDLErrorParsingSchema(_RESULT_, _MESSAGE_)                     \
    globus_error_put(                                                         \
        globus_error_construct_error(GLOBUS_WSDL_PARSER_MODULE,               \
                                     (_RESULT_ ? globus_error_get(_RESULT_) : \
                                      GLOBUS_NULL),                           \
                                     GLOBUS_WSDL_ERROR_PARSING_SCHEMA,        \
                                     __FILE__,                                \
                                     _globus_func_name,                       \
                                     __LINE__,                                \
                                     _MESSAGE_))

#define GlobusWSDLErrorLoadingSchema(_RESULT_)                                \
    globus_error_put(                                                         \
        globus_error_construct_error(GLOBUS_WSDL_PARSER_MODULE,               \
                                     (_RESULT_ ? globus_error_get(_RESULT_) : \
                                      GLOBUS_NULL),                           \
                                     GLOBUS_WSDL_ERROR_PARSING_SCHEMA,        \
                                     __FILE__,                                \
                                     _globus_func_name,                       \
                                     __LINE__,                                \
                                     "Failed to load WSDL schema at %s into " \
                                     "javascript context."))

#define GlobusWSDLErrorExecutingScript(_RESULT_, _FN_)                        \
    globus_error_put(                                                         \
        globus_error_construct_error(GLOBUS_WSDL_PARSER_MODULE,               \
                                     (_RESULT_ ? globus_error_get(_RESULT_) : \
                                      GLOBUS_NULL),                           \
                                     GLOBUS_WSDL_ERROR_PARSING_SCHEMA,        \
                                     __FILE__,                                \
                                     _globus_func_name,                       \
                                     __LINE__,                                \
                                     "Failed to execute javascript in "       \
                                     "template %s",                           \
                                     _FN_))

#define GlobusWSDLErrorJSDebugFailed()                  \
    globus_error_put(                                   \
        globus_error_construct_error(                   \
            GLOBUS_WSDL_PARSER_MODULE,                  \
            GLOBUS_NULL,                                \
            GLOBUS_WSDL_ERROR_JSDEBUG_FAILED,           \
            __FILE__,                                   \
            _globus_func_name,                          \
            __LINE__,                                   \
            "Failed to enable javascript debug mode"))

#define GlobusWSDLErrorJSDebugCompile()                 \
    globus_error_put(                                   \
        globus_error_construct_error(                   \
            GLOBUS_WSDL_PARSER_MODULE,                  \
            GLOBUS_NULL,                                \
            GLOBUS_WSDL_ERROR_JSDEBUG_COMPILE,          \
            __FILE__,                                   \
            _globus_func_name,                          \
            __LINE__,                                   \
            "Javascript debugging requires "            \
            "debug mode enabled during compilation."))

#define GlobusWSDLErrorDirCreate(_DIR_)                                 \
    globus_error_put(                                                   \
        globus_error_construct_error(GLOBUS_WSDL_PARSER_MODULE,         \
                                     GLOBUS_NULL,                       \
                                     GLOBUS_WSDL_ERROR_DIR_CREATE,      \
                                     __FILE__,                          \
                                     _globus_func_name,                 \
                                     __LINE__,                          \
                                     "Failed to create directory: %s",  \
                                     _DIR_))

#define GlobusWSDLErrorAccessingTemplateFile(_ERRNO_, _FILE_)              \
    globus_error_put(                                                      \
        globus_error_wrap_errno_error(GLOBUS_WSDL_PARSER_MODULE,           \
                                     _ERRNO_,                              \
                                     GLOBUS_WSDL_ERROR_ACCESSING_TEMPLATE, \
                                     __FILE__,                             \
                                     _globus_func_name,                    \
                                     __LINE__,                             \
                                     "Error with template file: %s",       \
                                     _FILE_))

#define GlobusWSDLErrorOutOfMemory                                      \
    globus_error_put(                                                   \
        globus_error_construct_error(GLOBUS_WSDL_PARSER_MODULE,         \
                                     NULL,                              \
                                     GLOBUS_WSDL_ERROR_OUT_OF_MEMORY,   \
                                     __FILE__,                          \
                                     _globus_func_name,                 \
                                     __LINE__,                          \
                                     "Error out of memory"))

GlobusDebugDeclare(GLOBUS_WSDL_PARSER);

#define GlobusWSDLDebugEnter()                                          \
    GlobusDebugPrintf(GLOBUS_WSDL_PARSER, GLOBUS_WSDL_DEBUG_TRACE,      \
                      ("[%s] Entering\n", _globus_func_name))

#define GlobusWSDLDebugExit()                                           \
    GlobusDebugPrintf(GLOBUS_WSDL_PARSER, GLOBUS_WSDL_DEBUG_TRACE,      \
                      ("[%s] Exiting\n", _globus_func_name))

#define GlobusWSDLDebug(_LEVEL_)                        \
    GlobusDebugTrue(GLOBUS_WSDL_PARSER, _LEVEL_)

#define GlobusWSDLDebugPrintf(_LEVEL_, _MESSAGE_)               \
    GlobusDebugPrintf(GLOBUS_WSDL_PARSER, _LEVEL_, _MESSAGE_)

#define GlobusWSDLDebugWrite(_LEVEL_, _LENGTH_, _BUFFER_)                  \
    GlobusDebugFwrite(GLOBUS_WSDL_PARSER, _LEVEL_, _BUFFER_, _LENGTH_, 1);

#define GlobusWSDLDebugStream           GlobusDebugMyFile(GLOBUS_WSDL_PARSER)

#include "wsdl.h"


typedef enum
{
    type,
    element,
    name,
    id,
    attributes,
    minOccurs,
    maxOccurs,
    occurs,
    ref,
    refNs,
    refQname,
    subtypes,
    typeName,
    typeNs,
    base,
    baseQName,
    defValue,
    targetNamespace,
    namedType,
    namedTypeNs,
    substGroup,
    substGroupNs,
    scope,
    value,
    fixed,
    contentType,
    parts,
    operations,
    message,
    binding,
    input,
    output,
    inputType,
    outputType,
    faults,
    ports,
    soap,
    extensionType,
    Namespace,
    use,
    encodingStyle,
    header,
    body,
    fault,
    style,
    transport,
    soapAction,
    location,
    baseNs,
    baseType,
    qname,
    typeQname,
    attributeWildcard,
    any,
    ResourceProperties,
    implements,
    facets,
    Action
} globus_i_js_id_t;

globus_result_t
globus_i_wsdl_parse_schema(
    const char *                        schema_filename, 
    wsdlSchemaPtr *                     schema);

char *
globus_i_wsdl_my_strtok(
    const char *                        str,
    size_t                              str_length,
    int *                               token_start_index,
    int *                               token_end_index,
    const char *                        delims);

int
globus_i_wsdl_split_qnames(
    xmlChar *                           default_ns,
    xmlNodePtr                          node,
    const xmlChar *                     qname_string,
    xsdQNamePtr *                       qnames);

#endif
