/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */


#include "globus_common.h"
#include "libxml/xmlschemas.h"
#include "libxml/schemasInternals.h"

#include "wsdl.h"

#include "globus_i_wsdl_parser.h"

#include "js/jsstddef.h"
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "js/jstypes.h"
#include "js/jsarena.h"
#include "js/jsutil.h"
#include "js/jsprf.h"
#include "js/jsapi.h"
#include "js/jsatom.h"
#include "js/jscntxt.h"
#include "js/jsdbgapi.h"
#include "js/jsemit.h"
#include "js/jsfun.h"
#include "js/jsgc.h"
#include "js/jslock.h"
#include "js/jsobj.h"
#include "js/jsparse.h"
#include "js/jsscope.h"
#include "js/jsscript.h"

#define PROPFLAGS                       JSPROP_ENUMERATE

JSPropertySpec xsdType_properties[] =
{
    {"type",              type,              PROPFLAGS},
    {"name",              name,              PROPFLAGS},
    {"id",                id,                PROPFLAGS},
    {"attributes",        attributes,        PROPFLAGS},
    {"minOccurs",         minOccurs,         PROPFLAGS},
    {"maxOccurs",         maxOccurs,         PROPFLAGS},
    {"ref",               ref,               PROPFLAGS},
    {"refNs",             refNs,             PROPFLAGS},
    {"subtypes",          subtypes,          PROPFLAGS},
    {"contentType",       contentType,       PROPFLAGS},
    {"base",              base,              PROPFLAGS},
    {"baseNs",            baseNs,            PROPFLAGS},
    {"baseQName",         baseQName,         PROPFLAGS},
    {"baseType",          baseType,          PROPFLAGS},
    {"targetNamespace",   targetNamespace,   PROPFLAGS},
    {"namedType",         namedType,         PROPFLAGS},
    {"namedTypeNs",       namedTypeNs,       PROPFLAGS},
    {"qname",             qname,             PROPFLAGS},
    {"refQname",          refQname,          PROPFLAGS},
    {"attributeWildcard", attributeWildcard, PROPFLAGS},
    {"facets",            facets,            PROPFLAGS},
    {0}
};

JSPropertySpec xsdAttr_properties[] =
{
    {"type",            type,            PROPFLAGS},
    {"name",            name,            PROPFLAGS},
    {"id",              id,              PROPFLAGS},
    {"attributes",      attributes,      PROPFLAGS},
    {"occurs",          occurs,          PROPFLAGS},
    {"ref",             ref,             PROPFLAGS},
    {"refNs",           refNs,           PROPFLAGS},
    {"subtypes",        subtypes,        PROPFLAGS},
    {"typeName",        typeName,        PROPFLAGS},
    {"typeNs",          typeNs,          PROPFLAGS},
    {"defValue",        defValue,        PROPFLAGS},
    {"base",            base,            PROPFLAGS},
    {"targetNamespace", targetNamespace, PROPFLAGS},
    {"qname",           qname,           PROPFLAGS},
    {"refQname",        refQname,        PROPFLAGS},
    {"typeQname",       typeQname,       PROPFLAGS},
    {0}
};

JSPropertySpec xsdElement_properties[] =
{
    {"type",            type,            PROPFLAGS},
    {"name",            name,            PROPFLAGS},
    {"id",              id,              PROPFLAGS},
    {"attributes",      attributes,      PROPFLAGS},
    {"minOccurs",       minOccurs,       PROPFLAGS},
    {"maxOccurs",       maxOccurs,       PROPFLAGS},
    {"ref",             ref,             PROPFLAGS},
    {"refNs",           refNs,           PROPFLAGS},
    {"subtypes",        subtypes,        PROPFLAGS},
    {"targetNamespace", targetNamespace, PROPFLAGS},
    {"namedType",       namedType,       PROPFLAGS},
    {"namedTypeNs",     namedTypeNs,     PROPFLAGS},
    {"substGroup",      substGroup,      PROPFLAGS},
    {"substGroupNs",    substGroupNs,    PROPFLAGS},
    {"scope",           scope,           PROPFLAGS},
    {"value",           value,           PROPFLAGS},
    {"contentType",     contentType,     PROPFLAGS},
    {"qname",           qname,           PROPFLAGS},
    {"refQname",        refQname,        PROPFLAGS},
    {0}
};

JSPropertySpec xsdWildcard_properties[] =
{
    {"type",            type,            PROPFLAGS},
    {"id",              id,              PROPFLAGS},
    {"minOccurs",       minOccurs,       PROPFLAGS},
    {"maxOccurs",       maxOccurs,       PROPFLAGS},
    {"any",             any,             PROPFLAGS},
    {0}
};

JSPropertySpec xsdFacet_properties[] =
{
    {"type",            type,            PROPFLAGS},
    {"id",              id,              PROPFLAGS},
    {"value",           value,           PROPFLAGS},
    {"fixed",           fixed,           PROPFLAGS},
    {0}
};
    
JSPropertySpec xsdContentTypeEnum_properties[] =
{
    {"content_unknown",           XML_SCHEMA_CONTENT_UNKNOWN,  PROPFLAGS},
    {"content_empty",             XML_SCHEMA_CONTENT_EMPTY,    PROPFLAGS},
    {"content_elements",          XML_SCHEMA_CONTENT_ELEMENTS, PROPFLAGS},
    {"content_mixed",             XML_SCHEMA_CONTENT_MIXED,    PROPFLAGS},
    {"content_simple",            XML_SCHEMA_CONTENT_SIMPLE,   PROPFLAGS},
    {"content_mixed_or_elements", XML_SCHEMA_CONTENT_MIXED_OR_ELEMENTS, 
                                      PROPFLAGS},
    {"content_basic",             XML_SCHEMA_CONTENT_BASIC,    PROPFLAGS},
    {"content_any",               XML_SCHEMA_CONTENT_ANY,      PROPFLAGS},
    {0}
};

JSPropertySpec xsdTypeEnum_properties[] =
{
    {"basic",           XML_SCHEMA_TYPE_BASIC,            PROPFLAGS},
    {"any",             XML_SCHEMA_TYPE_ANY,              PROPFLAGS},
    {"facet",           XML_SCHEMA_TYPE_FACET,            PROPFLAGS},
    {"simple",          XML_SCHEMA_TYPE_SIMPLE,           PROPFLAGS},
    {"complex",         XML_SCHEMA_TYPE_COMPLEX,          PROPFLAGS},
    {"sequence",        XML_SCHEMA_TYPE_SEQUENCE,         PROPFLAGS},
    {"choice",          XML_SCHEMA_TYPE_CHOICE,           PROPFLAGS},
    {"all",             XML_SCHEMA_TYPE_ALL,              PROPFLAGS},
    {"simple_content",  XML_SCHEMA_TYPE_SIMPLE_CONTENT,   PROPFLAGS},
    {"complex_content", XML_SCHEMA_TYPE_COMPLEX_CONTENT,  PROPFLAGS},
    {"ur",              XML_SCHEMA_TYPE_UR,               PROPFLAGS},
    {"restriction",     XML_SCHEMA_TYPE_RESTRICTION,      PROPFLAGS},
    {"extension",       XML_SCHEMA_TYPE_EXTENSION,        PROPFLAGS},
    {"element",         XML_SCHEMA_TYPE_ELEMENT,          PROPFLAGS},
    {"attribute",       XML_SCHEMA_TYPE_ATTRIBUTE,        PROPFLAGS},
    {"attributegroup",  XML_SCHEMA_TYPE_ATTRIBUTEGROUP,   PROPFLAGS},
    {"group",           XML_SCHEMA_TYPE_GROUP,            PROPFLAGS},
    {"notation",        XML_SCHEMA_TYPE_NOTATION,         PROPFLAGS},
    {"list",            XML_SCHEMA_TYPE_LIST,             PROPFLAGS},
    {"union",           XML_SCHEMA_TYPE_UNION,            PROPFLAGS},
    {0}
};

JSPropertySpec xsdAttrEnum_properties[] =
{
    {"skip",           XML_SCHEMAS_ANYATTR_SKIP,         PROPFLAGS},
    {"lax",            XML_SCHEMAS_ANYATTR_LAX,          PROPFLAGS},
    {"strict",         XML_SCHEMAS_ANYATTR_STRICT,       PROPFLAGS},
    {"use_prohibited", XML_SCHEMAS_ATTR_USE_PROHIBITED,  PROPFLAGS},
    {"use_required",   XML_SCHEMAS_ATTR_USE_REQUIRED,    PROPFLAGS},
    {"use_optional",   XML_SCHEMAS_ATTR_USE_OPTIONAL,    PROPFLAGS},
    {"nsdefault",      XML_SCHEMAS_ATTR_NSDEFAULT,       PROPFLAGS},
    {0}
};

JSPropertySpec xsdQName_properties[] =
{
    {"name",        name,        PROPFLAGS},
    {"namespace",   Namespace,   PROPFLAGS},
    {"qname",       qname,       PROPFLAGS},
    {0}
};

JSPropertySpec part_properties[] =
{
    {"name",        name,        PROPFLAGS},
    {"type",        type,        PROPFLAGS},
    {"element",     element,     PROPFLAGS},
    {0}
};

JSPropertySpec message_properties[] =
{
    {"name",        name,        PROPFLAGS},
    {"parts",       parts,       PROPFLAGS},
    {"targetNamespace", targetNamespace,      PROPFLAGS},
    {0}
};

JSPropertySpec operationTypeEnum_properties[] =
{
    {"one_way",           ONE_WAY_OPERATION,           PROPFLAGS},
    {"request_response",  REQUEST_RESPONSE_OPERATION,  PROPFLAGS},
    {"solicit_response",  SOLICIT_RESPONSE_OPERATION,  PROPFLAGS},
    {"notification",      NOTIFICATION_OPERATION,      PROPFLAGS},
    {0}
};

JSPropertySpec bindingType_properties[] =
{
    {"soap",              SOAP_BINDING,                PROPFLAGS},
    {0}
};

JSPropertySpec soapOperationStyleType_properties[] =
{
    {"rpc",               RPC_STYLE,                   PROPFLAGS},
    {"document",          DOCUMENT_STYLE,              PROPFLAGS},
    {0}
};

JSPropertySpec soapUseType_properties[] =
{
    {"literal",           LITERAL_USE,                 PROPFLAGS},
    {"encoded",           ENCODED_USE,                 PROPFLAGS},
    {0}
};

JSPropertySpec param_properties[] =
{
    {"name",        name,        PROPFLAGS},
    {"Action",      Action,      PROPFLAGS},
    {"message",     message,     PROPFLAGS},
    {0}
};

JSPropertySpec fault_properties[] =
{
    {"name",        name,        PROPFLAGS},
    {"message",     message,     PROPFLAGS},
    {0}
};

JSPropertySpec operation_properties[] =
{
    {"name",        name,        PROPFLAGS},
    {"type",        type,        PROPFLAGS},
    {"input",       input,       PROPFLAGS},
    {"output",      output,      PROPFLAGS},
    {"faults",      faults,      PROPFLAGS},
    {0}
};

JSPropertySpec portType_properties[] =
{
    {"name",               name,               PROPFLAGS},
    {"operations",         operations,         PROPFLAGS},
    {"ResourceProperties", ResourceProperties, PROPFLAGS},
    {"implements",         implements,         PROPFLAGS},
    {"targetNamespace",             targetNamespace,             PROPFLAGS},
    {0}
};

JSPropertySpec bindingOperation_properties[] =
{
    {"name",          name,        PROPFLAGS},
    {"input",         input,       PROPFLAGS},
    {"inputType",     inputType,   PROPFLAGS},
    {"output",        output,      PROPFLAGS},
    {"outputType",    outputType,  PROPFLAGS},
    {"faults",        faults,      PROPFLAGS},
    {"soap",          soap,        PROPFLAGS},
    {"extensionType", extensionType, PROPFLAGS},
    {0}
};

JSPropertySpec bindingOperationFault_properties[] =
{
    {"name",           name,           PROPFLAGS},
    {"soap",           soap,           PROPFLAGS},
    {"extensionType",  extensionType,  PROPFLAGS},
    {0}
};

JSPropertySpec binding_properties[] =
{
    {"name",           name,           PROPFLAGS},
    {"type",           type,           PROPFLAGS},
    {"operations",     operations,     PROPFLAGS},
    {"soap",           soap,           PROPFLAGS},
    {"extensionType",  extensionType,  PROPFLAGS},
    {"targetNamespace",         targetNamespace,         PROPFLAGS},
    {0}
};

JSPropertySpec port_properties[] =
{
    {"name",          name,          PROPFLAGS},
    {"binding",       binding,       PROPFLAGS},
    {"operations",    operations,    PROPFLAGS},
    {"soap",          soap,          PROPFLAGS},
    {"extensionType", extensionType, PROPFLAGS},
    {0}
};

JSPropertySpec service_properties[] =
{
    {"name",          name,          PROPFLAGS},
    {"ports",         ports,         PROPFLAGS},
    {"soap",          soap,          PROPFLAGS},
    {"extensionType", extensionType, PROPFLAGS},
    {"targetNamespace",        targetNamespace,        PROPFLAGS},
    {0}
};

JSPropertySpec soapBinding_properties[] =
{
    {"style",     style,     PROPFLAGS},
    {"transport", transport, PROPFLAGS},
    {0}
};

JSPropertySpec soapOperation_properties[] =
{
    {"soapAction", soapAction, PROPFLAGS},
    {"style",      style,      PROPFLAGS},
    {0}
};

JSPropertySpec soapBody_properties[] =
{
    {"encodingStyle",   encodingStyle,   PROPFLAGS},
    {"parts",           parts,           PROPFLAGS},
    {"use",             use,             PROPFLAGS},
    {"namespace",       Namespace,       PROPFLAGS},
    {0}
};

JSPropertySpec soapFault_properties[] =
{
    {"encodingStyle",   encodingStyle,   PROPFLAGS},
    {"use",             use,             PROPFLAGS},
    {"namespace",       Namespace,       PROPFLAGS},
    {0}
};

JSPropertySpec soapHeader_properties[] =
{
    {"message",         message,         PROPFLAGS},
    {"parts",           parts,           PROPFLAGS},
    {"use",             use,             PROPFLAGS},
    {"encodingStyle",   encodingStyle,   PROPFLAGS},
    {"namespace",       Namespace,       PROPFLAGS},
    {0}
};

JSPropertySpec soapBindingParam_properties[] =
{
    {"header",          header,          PROPFLAGS},
    {"body",            body,            PROPFLAGS},
    {"fault",           fault,           PROPFLAGS},
    {0}
};

JSPropertySpec soapAddress_properties[] =
{
    {"location",        location,        PROPFLAGS},
    {0}
};
