/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */


#ifndef __WSDL_H__
#define __WSDL_H__

#ifdef __cplusplus
extern "C" { 
#endif

#if 0
}
#endif

#include "wsdlInternals.h"

typedef enum {
    WSDL_ERR_OK = 0,
    WSDL_ERR_NOROOT,
    WSDL_ERR_DICT,
    WSDL_ERR_PREFIX_UNDEFINED,
    WSDL_ERR_FAILED_LOAD,
    WSDL_ERR_FAILED_PARSE,
    WSDL_ERR_NOTHING_TO_PARSE,
    WSDL_ERR_NOT_WSDL,
    WSDL_ERR_NO_NODE,
    WSDL_ERR_PORT,
    WSDL_ERR_SERVICE,
    WSDL_ERR_TYPES,
    WSDL_ERR_PART,
    WSDL_ERR_BINDING,
    WSDL_ERR_SOAP_BINDING,
    WSDL_ERR_MESSAGE,
    WSDL_ERR_IMPORT,
    WSDL_ERR_OP,
    WSDL_ERR_PARAM,
    WSDL_ERR_FAULT,
    WSDL_ERR_PORT_TYPE,
    WSDL_ERR_TOPLEVEL

} wsdlError;

typedef void (*wsdlErrorFunc) (void *ctx, const char *msg, ...);
typedef void (*wsdlWarningFunc) (void *ctxt, const char *msg, ...);

struct _wsdlParserCtxt {
    void *                              userData;
    wsdlSchemaPtr                       schema;
    struct _wsdlParserCtxt *            enclosingCtxt;
    const xmlChar *                     enclosingNs;
    xmlDocPtr                           doc;
    xmlDictPtr                          dict;
    wsdlErrorFunc                       err;
    wsdlWarningFunc                     warn;
    int                                 nberrors;
    char *                              buffer;
    int                                 size;
    const xmlChar *                     url;
    int                                 preserve;
};

typedef struct _wsdlParserCtxt          wsdlParserCtxt;
typedef wsdlParserCtxt *                wsdlParserCtxtPtr;

wsdlParserCtxtPtr wsdlNewParserCtxt(const char *     URL);
void wsdlFreeParserCtxt(wsdlParserCtxtPtr ctxt);

wsdlParserCtxtPtr wsdlNewDocParserCtxt(xmlDocPtr doc);
wsdlParserCtxtPtr wsdlNewMemParserCtxt(const char * buffer, int size);

void
wsdlSetParserErrors(wsdlParserCtxtPtr ctxt,
                    wsdlErrorFunc err,
                    wsdlWarningFunc warn, void *ctx);

void
xsdQNameFree(
    xsdQNamePtr                         qname);

wsdlSchemaPtr
wsdlParse(wsdlParserCtxtPtr             ctxt);

wsdlSchemaPtr
wsdlParseXmlSchema(wsdlParserCtxtPtr    ctxt);

void
wsdlFreeParserCtxt(wsdlParserCtxtPtr    ctxt);

void
wsdlSchemaFree(
    wsdlSchemaPtr                       schema);

xmlSchemaTypePtr
wsdlFindType(wsdlParserCtxtPtr          ctxt,
             wsdlSchemaPtr              schema,
             const xmlChar *            typeLocal,
             const xmlChar *            typeNs);

xmlSchemaElementPtr
wsdlFindElement(wsdlParserCtxtPtr       ctxt,
                wsdlSchemaPtr           schema,
                const xmlChar *         elementLocal,
                const xmlChar *         elementNs);

wsdlMessagePtr
wsdlFindMessage(wsdlParserCtxtPtr       ctxt,
                wsdlSchemaPtr           schema,
                const xmlChar *         messageLocal,
                const xmlChar *         messageNs);

wsdlPortTypePtr
wsdlFindPortType(wsdlParserCtxtPtr      ctxt,
                 wsdlSchemaPtr          schema,
                 const xmlChar *        portTypeLocal,
                 const xmlChar *        portTypeNs);

wsdlBindingPtr
wsdlFindBinding(wsdlParserCtxtPtr       ctxt,
                wsdlSchemaPtr           schema,
                const xmlChar *         bindingLocal,
                const xmlChar *         bindingNs);

wsdlServicePtr
wsdlFindService(wsdlParserCtxtPtr       ctxt,
                wsdlSchemaPtr           schema,
                const xmlChar *         serviceLocal,
                const xmlChar *         serviceNs);

#ifdef __cplusplus
}
#endif

#endif
