/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */

#include "globus_wsrf_resource.h"
#include "xsd_string.h"

typedef int (*test_case)(void);

#define SIZEOF_ARRAY(x) sizeof(x) / sizeof(x[0])

int dynamic_property_suffix;

int test_init(globus_resource_t * resource)
{
    int rc;
    globus_result_t result;
    
    rc = globus_module_activate(GLOBUS_WSRF_RESOURCE_MODULE);

    if (rc != GLOBUS_SUCCESS)
    {
        goto error;
    }

    result = globus_resource_create("resourceid", resource);

    if (result != GLOBUS_SUCCESS)
    {
        goto deactivate_error;
    }

    dynamic_property_suffix = 0;

    return GLOBUS_SUCCESS;

deactivate_error:
    globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);

error:
    return 1;
}

int test_destroy(globus_resource_t resource)
{
    globus_result_t result;
    
    result = globus_resource_destroy(resource);

    if (result != GLOBUS_SUCCESS)
    {
        goto deactivate_error;
    }
    globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);

    return GLOBUS_SUCCESS;

deactivate_error:
    globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);

    return 1;
}

int
create_property_test(void)
{
    int rc;
    globus_result_t result;
    globus_resource_t resource;
    xsd_QName test_qname = {
         "http://test.globus.org/",
         "test"
    };

    rc = test_init(&resource);
    
    if (rc != GLOBUS_SUCCESS)
    {
        return rc;
    }

    result = globus_resource_create_property(
            resource,
            &test_qname,
            &xsd_string_info,
            NULL);

    if (result != GLOBUS_SUCCESS)
    {
        goto destroy_test_error;
    }
    test_destroy(resource);
    return 0;

destroy_test_error:
    test_destroy(resource);
    return 1;
}

int
create_destroy_property_test(void)
{
    int rc;
    globus_result_t result;
    globus_resource_t resource;
    xsd_QName test_qname = {
         "http://test.globus.org/",
         "test"
    };

    rc = test_init(&resource);
    
    if (rc != GLOBUS_SUCCESS)
    {
        return rc;
    }

    result = globus_resource_create_property(
            resource,
            &test_qname,
            &xsd_string_info,
            NULL);

    if (result != GLOBUS_SUCCESS)
    {
        goto destroy_test_error;
    }

    result = globus_resource_destroy_property(
            resource,
            &test_qname);

    if (result != GLOBUS_SUCCESS)
    {
        goto destroy_test_error;
    }

    test_destroy(resource);
    return 0;

destroy_test_error:
    test_destroy(resource);
    return 1;
}

int
create_delete_property_test(void)
{
    int rc;
    globus_result_t result;
    globus_resource_t resource;
    xsd_QName test_qname = {
         "http://test.globus.org/",
         "test"
    };
    void * property;
    globus_xsd_type_info_t info;

    rc = test_init(&resource);
    
    if (rc != GLOBUS_SUCCESS)
    {
        return rc;
    }

    result = globus_resource_create_property(
            resource,
            &test_qname,
            &xsd_string_info,
            NULL);

    if (result != GLOBUS_SUCCESS)
    {
        goto destroy_test_error;
    }

    result = globus_resource_delete_property(
            resource,
            &test_qname,
            &property,
            &info);

    if (result != GLOBUS_SUCCESS)
    {
        goto destroy_test_error;
    }

    if (property != NULL || info->initialize != xsd_string_info.initialize)
    {
        goto destroy_test_error;
    }

    test_destroy(resource);
    return 0;

destroy_test_error:
    test_destroy(resource);
    return 1;
}

int
create_set_property_test(void)
{
    int rc;
    globus_result_t result;
    globus_resource_t resource;
    xsd_QName test_qname = {
         "http://test.globus.org/",
         "test"
    };
    xsd_string * property;

    rc = test_init(&resource);
    
    if (rc != GLOBUS_SUCCESS)
    {
        return rc;
    }

    result = (*xsd_string_info.initialize)((void *)&property);

    if (result != GLOBUS_SUCCESS)
    {
        goto destroy_test_error;
    }

    (*property) = globus_libc_strdup("property");

    result = globus_resource_create_property(
            resource,
            &test_qname,
            &xsd_string_info,
            NULL);

    if (result != GLOBUS_SUCCESS)
    {
        goto destroy_property_error;
    }

    result = globus_resource_set_property(
            resource,
            &test_qname,
            property);

    if (result != GLOBUS_SUCCESS)
    {
        goto destroy_test_error;
    }

    test_destroy(resource);
    return 0;

destroy_property_error:
    xsd_string_info.destroy(property);
destroy_test_error:
    test_destroy(resource);
    return 1;
}

int
create_set_destroy_property_test(void)
{
    int rc;
    globus_result_t result;
    globus_resource_t resource;
    xsd_QName test_qname = {
         "http://test.globus.org/",
         "test"
    };
    xsd_string * property;

    rc = test_init(&resource);
    
    if (rc != GLOBUS_SUCCESS)
    {
        return rc;
    }

    result = xsd_string_info.initialize((void *)&property);

    if (result != GLOBUS_SUCCESS)
    {
        goto destroy_test_error;
    }

    (*property) = globus_libc_strdup("property");

    result = globus_resource_create_property(
            resource,
            &test_qname,
            &xsd_string_info,
            NULL);

    if (result != GLOBUS_SUCCESS)
    {
        goto destroy_property_error;
    }

    result = globus_resource_set_property(
            resource,
            &test_qname,
            property);

    if (result != GLOBUS_SUCCESS)
    {
        goto destroy_test_error;
    }

    result = globus_resource_destroy_property(
            resource,
            &test_qname);

    if (result != GLOBUS_SUCCESS)
    {
        goto destroy_test_error;
    }

    test_destroy(resource);
    return 0;

destroy_property_error:
    xsd_string_info.destroy(property);
destroy_test_error:
    test_destroy(resource);
    return 1;
}

int
create_get_property_test(void)
{
    int rc;
    globus_result_t result;
    globus_resource_t resource;
    xsd_QName test_qname = {
         "http://test.globus.org/",
         "test"
    };
    xsd_string * property = (void *) 0x1;
    globus_xsd_type_info_t type_info;

    rc = test_init(&resource);
    
    if (rc != GLOBUS_SUCCESS)
    {
        return rc;
    }

    result = globus_resource_create_property(
            resource,
            &test_qname,
            &xsd_string_info,
            NULL);

    if (result != GLOBUS_SUCCESS)
    {
        goto destroy_test_error;
    }

    result = globus_resource_get_property(
            resource,
            &test_qname,
            (void *) &property,
            NULL);

    if (result != GLOBUS_SUCCESS || property != NULL)
    {
        goto destroy_test_error;
    }
    result = globus_resource_get_property(
            resource,
            &test_qname,
            (void *) &property,
            &type_info);

    if (result != GLOBUS_SUCCESS || property != NULL || type_info == NULL)
    {
        goto destroy_test_error;
    }
    if (type_info->initialize != xsd_string_info.initialize)
    {
        goto destroy_test_error;
    }

    result = globus_resource_destroy_property(
            resource,
            &test_qname);

    if (result != GLOBUS_SUCCESS)
    {
        goto destroy_test_error;
    }

    test_destroy(resource);
    return 0;

destroy_test_error:
    test_destroy(resource);
    return 1;
}
int
create_get_set_get_property_test(void)
{
    int rc;
    globus_result_t result;
    globus_resource_t resource;
    xsd_QName test_qname = {
         "http://test.globus.org/",
         "test"
    };
    xsd_string * property1;
    xsd_string * property2;
    xsd_string * res_property;

    rc = test_init(&resource);
    
    if (rc != GLOBUS_SUCCESS)
    {
        return rc;
    }

    result = xsd_string_info.initialize((void *)&property1);

    if (result != GLOBUS_SUCCESS)
    {
        goto destroy_test_error;
    }

    *property1 = globus_libc_strdup("property1");

    if (*property1 == NULL)
    {
        goto destroy_property1_error;
    }

    result = xsd_string_info.initialize((void *)&property2);

    if (result != GLOBUS_SUCCESS)
    {
        goto destroy_property1_error;
    }

    *property2 = globus_libc_strdup("property2");

    if (*property2 == NULL)
    {
        goto destroy_property2_error;
    }

    result = globus_resource_create_property(
            resource,
            &test_qname,
            &xsd_string_info,
            property1);

    if (result != GLOBUS_SUCCESS)
    {
        goto destroy_test_error;
    }

    result = globus_resource_get_property(
            resource,
            &test_qname,
            (void *) &res_property,
            NULL);

    if (result != GLOBUS_SUCCESS || property1 != res_property)
    {
        goto destroy_test_error;
    }

    result = globus_resource_set_property(
            resource,
            &test_qname,
            property2);

    if (result != GLOBUS_SUCCESS)
    {
        goto destroy_test_error;
    }

    result = globus_resource_get_property(
            resource,
            &test_qname,
            (void *) &res_property,
            NULL);

    if (result != GLOBUS_SUCCESS || property2 != res_property)
    {
        goto destroy_test_error;
    }
    test_destroy(resource);
    return 0;

destroy_property2_error:
    xsd_string_info.destroy(property2);
destroy_property1_error:
    xsd_string_info.destroy(property1);
destroy_test_error:
    test_destroy(resource);
    return 1;
}

int
multi_create_property_test(void)
{
    int rc;
    globus_result_t result;
    globus_resource_t resource;
    xsd_QName test_qname = {
         "http://test.globus.org/",
         NULL
    };
    char test_localname[10];
    int i;

    rc = test_init(&resource);
    
    if (rc != GLOBUS_SUCCESS)
    {
        return rc;
    }

    test_qname.local = test_localname;

    for (i = 0; i < 256; i++)
    {
        sprintf(test_localname, "test%03d", i);

        result = globus_resource_create_property(
                resource,
                &test_qname,
                &xsd_string_info,
                NULL);

        if (result != GLOBUS_SUCCESS)
        {
            goto destroy_test_error;
        }
    }

    test_destroy(resource);
    return 0;

destroy_test_error:
    test_destroy(resource);
    return 1;
}

int
multi_set_property_test(void)
{
    int rc;
    globus_result_t result;
    globus_resource_t resource;
    xsd_QName test_qname = {
         "http://test.globus.org/",
         "test"
    };
    char test_localname[10];
    xsd_string * property[256];
    int i;

    rc = test_init(&resource);
    
    if (rc != GLOBUS_SUCCESS)
    {
        return rc;
    }

    result = globus_resource_create_property(
            resource,
            &test_qname,
            &xsd_string_info,
            NULL);

    if (result != GLOBUS_SUCCESS)
    {
        goto destroy_test_error;
    }

    for (i = 0; i < 256; i++)
    {
        sprintf(test_localname, "test%03d", i);
        result = xsd_string_info.initialize((void *) &property[i]);

        *(property[i]) = globus_libc_strdup(test_localname);

        result = globus_resource_set_property(
                resource,
                &test_qname,
                property[i]);

        if (result != GLOBUS_SUCCESS)
        {
            goto destroy_test_error;
        }
    }

    test_destroy(resource);
    return 0;

destroy_test_error:
    test_destroy(resource);
    return 1;
}

void
property_callback(
    void *                              arg,
    xsd_QName *                         qname,
    void **                             property)
{
    if (arg != (void *) 0x1)
    {
        *property = NULL;

        return;
    }

    
    xsd_string_init_cstr((xsd_string **) property,
        globus_common_create_string("test%02d", dynamic_property_suffix++));
}
/* property_callback() */

int
create_dynamic_property_test(void)
{
    int rc;
    globus_result_t result;
    globus_resource_t resource;
    xsd_QName test_qname = {
         "http://test.globus.org/",
         "test"
    };

    rc = test_init(&resource);
    
    if (rc != GLOBUS_SUCCESS)
    {
        return rc;
    }

    result = globus_resource_create_property_callback(
            resource,
            &test_qname,
            &xsd_string_info,
            property_callback,
            (void*) 0x1);

    if (result != GLOBUS_SUCCESS)
    {
        goto destroy_test_error;
    }
    test_destroy(resource);
    return 0;

destroy_test_error:
    test_destroy(resource);
    return 1;
}

int
create_get_dynamic_property_test(void)
{
    int rc;
    globus_result_t result;
    globus_resource_t resource;
    xsd_QName test_qname = {
         "http://test.globus.org/",
         "test"
    };
    xsd_string * res_property;

    rc = test_init(&resource);
    
    if (rc != GLOBUS_SUCCESS)
    {
        return rc;
    }

    result = globus_resource_create_property_callback(
            resource,
            &test_qname,
            &xsd_string_info,
            property_callback,
            (void *) 0x1);

    if (result != GLOBUS_SUCCESS)
    {
        goto destroy_test_error;
    }

    result = globus_resource_get_property(
            resource,
            &test_qname,
            (void *) &res_property,
            NULL);

    if (result != GLOBUS_SUCCESS || res_property == NULL)
    {
        goto destroy_test_error;
    }

    if (strcmp(*res_property, "test00"))
    {
        goto destroy_test_error;
    }

    result = globus_resource_get_property(
            resource,
            &test_qname,
            (void *) &res_property,
            NULL);

    if (result != GLOBUS_SUCCESS || res_property == NULL)
    {
        goto destroy_test_error;
    }

    if (strcmp(*res_property, "test01"))
    {
        goto destroy_test_error;
    }

    test_destroy(resource);
    return 0;

destroy_test_error:
    test_destroy(resource);
    return 1;
}

int
null_create_property_test(void)
{
    int rc;
    globus_result_t result;
    globus_resource_t resource;
    xsd_QName test_qname = {
         "http://test.globus.org/",
         "test"
    };

    rc = test_init(&resource);
    
    if (rc != GLOBUS_SUCCESS)
    {
        return rc;
    }

    result = globus_resource_create_property(
            NULL,
            &test_qname,
            &xsd_string_info,
            NULL);

    if (result == GLOBUS_SUCCESS)
    {
        goto destroy_test_error;
    }
    else if (! globus_error_match(
                globus_error_peek(result),
                GLOBUS_WSRF_RESOURCE_MODULE,
                GLOBUS_WSRF_RESOURCE_ERROR_TYPE_NULL_PARAM))
    {
        goto destroy_test_error;
    }

    result = globus_resource_create_property(
            resource,
            NULL,
            &xsd_string_info,
            NULL);

    if (result == GLOBUS_SUCCESS)
    {
        goto destroy_test_error;
    }
    else if (! globus_error_match(
                globus_error_peek(result),
                GLOBUS_WSRF_RESOURCE_MODULE,
                GLOBUS_WSRF_RESOURCE_ERROR_TYPE_NULL_PARAM))
    {
        goto destroy_test_error;
    }

    result = globus_resource_create_property(
            resource,
            &test_qname,
            NULL,
            NULL);

    if (result == GLOBUS_SUCCESS)
    {
        goto destroy_test_error;
    }
    else if (! globus_error_match(
                globus_error_peek(result),
                GLOBUS_WSRF_RESOURCE_MODULE,
                GLOBUS_WSRF_RESOURCE_ERROR_TYPE_NULL_PARAM))
    {
        goto destroy_test_error;
    }

    test_destroy(resource);
    return 0;

destroy_test_error:
    test_destroy(resource);
    return 1;
}

int
null_create_property_callback_test(void)
{
    int rc;
    globus_result_t result;
    globus_resource_t resource;
    xsd_QName test_qname = {
         "http://test.globus.org/",
         "test"
    };

    rc = test_init(&resource);
    
    if (rc != GLOBUS_SUCCESS)
    {
        return rc;
    }

    result = globus_resource_create_property_callback(
            NULL,
            &test_qname,
            &xsd_string_info,
            property_callback,
            NULL);

    if (result == GLOBUS_SUCCESS)
    {
        goto destroy_test_error;
    }
    else if (! globus_error_match(
                globus_error_peek(result),
                GLOBUS_WSRF_RESOURCE_MODULE,
                GLOBUS_WSRF_RESOURCE_ERROR_TYPE_NULL_PARAM))
    {
        goto destroy_test_error;
    }

    result = globus_resource_create_property_callback(
            resource,
            NULL,
            &xsd_string_info,
            property_callback,
            NULL);

    if (result == GLOBUS_SUCCESS)
    {
        goto destroy_test_error;
    }
    else if (! globus_error_match(
                globus_error_peek(result),
                GLOBUS_WSRF_RESOURCE_MODULE,
                GLOBUS_WSRF_RESOURCE_ERROR_TYPE_NULL_PARAM))
    {
        goto destroy_test_error;
    }

    result = globus_resource_create_property_callback(
            resource,
            &test_qname,
            NULL,
            property_callback,
            NULL);

    if (result == GLOBUS_SUCCESS)
    {
        goto destroy_test_error;
    }
    else if (! globus_error_match(
                globus_error_peek(result),
                GLOBUS_WSRF_RESOURCE_MODULE,
                GLOBUS_WSRF_RESOURCE_ERROR_TYPE_NULL_PARAM))
    {
        goto destroy_test_error;
    }

    result = globus_resource_create_property_callback(
            resource,
            &test_qname,
            &xsd_string_info,
            NULL,
            NULL);

    if (result == GLOBUS_SUCCESS)
    {
        goto destroy_test_error;
    }
    else if (! globus_error_match(
                globus_error_peek(result),
                GLOBUS_WSRF_RESOURCE_MODULE,
                GLOBUS_WSRF_RESOURCE_ERROR_TYPE_NULL_PARAM))
    {
        goto destroy_test_error;
    }

    test_destroy(resource);
    return 0;

destroy_test_error:
    test_destroy(resource);
    return 1;
}

int
null_set_property_test(void)
{
    int rc;
    globus_result_t result;
    globus_resource_t resource;
    xsd_QName test_qname = {
         "http://test.globus.org/",
         "test"
    };

    rc = test_init(&resource);
    
    if (rc != GLOBUS_SUCCESS)
    {
        return rc;
    }

    result = globus_resource_create_property(
            resource,
            &test_qname,
            &xsd_string_info,
            NULL);

    if (result != GLOBUS_SUCCESS)
    {
        goto destroy_test_error;
    }

    result = globus_resource_set_property(
            NULL,
            &test_qname,
            NULL);

    if (result == GLOBUS_SUCCESS)
    {
        goto destroy_test_error;
    }
    else if (! globus_error_match(
                globus_error_peek(result),
                GLOBUS_WSRF_RESOURCE_MODULE,
                GLOBUS_WSRF_RESOURCE_ERROR_TYPE_NULL_PARAM))
    {
        goto destroy_test_error;
    }

    result = globus_resource_set_property(
            resource,
            NULL,
            NULL);
    if (result == GLOBUS_SUCCESS)
    {
        goto destroy_test_error;
    }
    else if (! globus_error_match(
                globus_error_peek(result),
                GLOBUS_WSRF_RESOURCE_MODULE,
                GLOBUS_WSRF_RESOURCE_ERROR_TYPE_NULL_PARAM))
    {
        goto destroy_test_error;
    }

    test_destroy(resource);
    return 0;

destroy_test_error:
    test_destroy(resource);
    return 1;
}


int
null_get_property_test(void)
{
    int rc;
    globus_result_t result;
    globus_resource_t resource;
    xsd_QName test_qname = {
         "http://test.globus.org/",
         "test"
    };
    xsd_string * property = (void *) 0x1;

    rc = test_init(&resource);
    
    if (rc != GLOBUS_SUCCESS)
    {
        return rc;
    }

    result = globus_resource_create_property(
            resource,
            &test_qname,
            &xsd_string_info,
            NULL);

    if (result != GLOBUS_SUCCESS)
    {
        goto destroy_test_error;
    }

    result = globus_resource_get_property(
            NULL,
            &test_qname,
            (void *) &property,
            NULL);

    if (result == GLOBUS_SUCCESS)
    {
        goto destroy_test_error;
    }
    else if (! globus_error_match(
                globus_error_peek(result),
                GLOBUS_WSRF_RESOURCE_MODULE,
                GLOBUS_WSRF_RESOURCE_ERROR_TYPE_NULL_PARAM))
    {
        goto destroy_test_error;
    }
    result = globus_resource_get_property(
            resource,
            NULL,
            (void *) &property,
            NULL);

    if (result == GLOBUS_SUCCESS)
    {
        goto destroy_test_error;
    }
    else if (! globus_error_match(
                globus_error_peek(result),
                GLOBUS_WSRF_RESOURCE_MODULE,
                GLOBUS_WSRF_RESOURCE_ERROR_TYPE_NULL_PARAM))
    {
        goto destroy_test_error;
    }
    result = globus_resource_get_property(
            resource,
            &test_qname,
            NULL,
            NULL);

    if (result == GLOBUS_SUCCESS)
    {
        goto destroy_test_error;
    }
    else if (! globus_error_match(
                globus_error_peek(result),
                GLOBUS_WSRF_RESOURCE_MODULE,
                GLOBUS_WSRF_RESOURCE_ERROR_TYPE_NULL_PARAM))
    {
        goto destroy_test_error;
    }


    test_destroy(resource);
    return 0;

destroy_test_error:
    test_destroy(resource);
    return 1;
}


int
null_destroy_property_test(void)
{
    int rc;
    globus_result_t result;
    globus_resource_t resource;
    xsd_QName test_qname = {
         "http://test.globus.org/",
         "test"
    };

    rc = test_init(&resource);
    
    if (rc != GLOBUS_SUCCESS)
    {
        return rc;
    }

    result = globus_resource_create_property(
            resource,
            &test_qname,
            &xsd_string_info,
            NULL);

    if (result != GLOBUS_SUCCESS)
    {
        goto destroy_test_error;
    }

    result = globus_resource_destroy_property(
            NULL,
            &test_qname);

    if (result == GLOBUS_SUCCESS)
    {
        goto destroy_test_error;
    }
    else if (! globus_error_match(
                globus_error_peek(result),
                GLOBUS_WSRF_RESOURCE_MODULE,
                GLOBUS_WSRF_RESOURCE_ERROR_TYPE_NULL_PARAM))
    {
        goto destroy_test_error;
    }

    result = globus_resource_destroy_property(
            resource,
            NULL);

    if (result == GLOBUS_SUCCESS)
    {
        goto destroy_test_error;
    }
    else if (! globus_error_match(
                globus_error_peek(result),
                GLOBUS_WSRF_RESOURCE_MODULE,
                GLOBUS_WSRF_RESOURCE_ERROR_TYPE_NULL_PARAM))
    {
        goto destroy_test_error;
    }
    test_destroy(resource);
    return 0;

destroy_test_error:
    test_destroy(resource);
    return 1;
}

int
null_delete_property_test(void)
{
    int rc;
    globus_result_t result;
    globus_resource_t resource;
    xsd_QName test_qname = {
         "http://test.globus.org/",
         "test"
    };
    void * property;
    globus_xsd_type_info_t info;

    rc = test_init(&resource);
    
    if (rc != GLOBUS_SUCCESS)
    {
        return rc;
    }

    result = globus_resource_create_property(
            resource,
            &test_qname,
            &xsd_string_info,
            NULL);

    if (result != GLOBUS_SUCCESS)
    {
        goto destroy_test_error;
    }

    result = globus_resource_delete_property(
            NULL,
            &test_qname,
            &property,
            &info);

    if (result == GLOBUS_SUCCESS)
    {
        goto destroy_test_error;
    }
    else if (! globus_error_match(
                globus_error_peek(result),
                GLOBUS_WSRF_RESOURCE_MODULE,
                GLOBUS_WSRF_RESOURCE_ERROR_TYPE_NULL_PARAM))
    {
        goto destroy_test_error;
    }
    result = globus_resource_delete_property(
            resource,
            NULL,
            &property,
            &info);

    if (result == GLOBUS_SUCCESS)
    {
        goto destroy_test_error;
    }
    else if (! globus_error_match(
                globus_error_peek(result),
                GLOBUS_WSRF_RESOURCE_MODULE,
                GLOBUS_WSRF_RESOURCE_ERROR_TYPE_NULL_PARAM))
    {
        goto destroy_test_error;
    }

    result = globus_resource_delete_property(
            resource,
            &test_qname,
            NULL,
            &info);

    if (result == GLOBUS_SUCCESS)
    {
        goto destroy_test_error;
    }
    else if (! globus_error_match(
                globus_error_peek(result),
                GLOBUS_WSRF_RESOURCE_MODULE,
                GLOBUS_WSRF_RESOURCE_ERROR_TYPE_NULL_PARAM))
    {
        goto destroy_test_error;
    }

    result = globus_resource_delete_property(
            resource,
            &test_qname,
            &property,
            NULL);
    if (result == GLOBUS_SUCCESS)
    {
        goto destroy_test_error;
    }
    else if (! globus_error_match(
                globus_error_peek(result),
                GLOBUS_WSRF_RESOURCE_MODULE,
                GLOBUS_WSRF_RESOURCE_ERROR_TYPE_NULL_PARAM))
    {
        goto destroy_test_error;
    }
    test_destroy(resource);
    return 0;

destroy_test_error:
    test_destroy(resource);
    return 1;
}

int
unknown_get_property_test(void)
{
    int rc;
    globus_result_t result;
    globus_resource_t resource;
    xsd_QName test_qname = {
         "http://test.globus.org/",
         "test"
    };
    xsd_string * property = (void *) 0x1;

    rc = test_init(&resource);
    
    if (rc != GLOBUS_SUCCESS)
    {
        return rc;
    }

    result = globus_resource_get_property(
            resource,
            &test_qname,
            (void *) &property,
            NULL);
    if (result == GLOBUS_SUCCESS)
    {
        goto destroy_test_error;
    }
    else if (! globus_error_match(
                globus_error_peek(result),
                GLOBUS_WSRF_RESOURCE_MODULE,
                GLOBUS_WSRF_RESOURCE_ERROR_TYPE_UNKNOWN_PROPERTY))
    {
        goto destroy_test_error;
    }
    test_destroy(resource);
    return 0;

destroy_test_error:
    test_destroy(resource);
    return 1;
}

int
unknown_set_property_test(void)
{
    int rc;
    globus_result_t result;
    globus_resource_t resource;
    xsd_QName test_qname = {
         "http://test.globus.org/",
         "test"
    };
    xsd_string * property = (void *) 0x1;

    rc = test_init(&resource);
    
    if (rc != GLOBUS_SUCCESS)
    {
        return rc;
    }

    result = globus_resource_set_property(
            resource,
            &test_qname,
            (void *) &property);
    if (result == GLOBUS_SUCCESS)
    {
        goto destroy_test_error;
    }
    else if (! globus_error_match(
                globus_error_peek(result),
                GLOBUS_WSRF_RESOURCE_MODULE,
                GLOBUS_WSRF_RESOURCE_ERROR_TYPE_UNKNOWN_PROPERTY))
    {
        goto destroy_test_error;
    }
    test_destroy(resource);
    return 0;

destroy_test_error:
    test_destroy(resource);
    return 1;
}

int
unknown_delete_property_test(void)
{
    int rc;
    globus_result_t result;
    globus_resource_t resource;
    xsd_QName test_qname = {
         "http://test.globus.org/",
         "test"
    };
    void * property;
    globus_xsd_type_info_t info;

    rc = test_init(&resource);
    
    if (rc != GLOBUS_SUCCESS)
    {
        return rc;
    }

    result = globus_resource_delete_property(
            resource,
            &test_qname,
            &property,
            &info);
    if (result == GLOBUS_SUCCESS)
    {
        goto destroy_test_error;
    }
    else if (! globus_error_match(
                globus_error_peek(result),
                GLOBUS_WSRF_RESOURCE_MODULE,
                GLOBUS_WSRF_RESOURCE_ERROR_TYPE_UNKNOWN_PROPERTY))
    {
        goto destroy_test_error;
    }

    test_destroy(resource);
    return 0;

destroy_test_error:
    test_destroy(resource);
    return 1;
}

int
unknown_destroy_property_test(void)
{
    int rc;
    globus_result_t result;
    globus_resource_t resource;
    xsd_QName test_qname = {
         "http://test.globus.org/",
         "test"
    };

    rc = test_init(&resource);
    
    if (rc != GLOBUS_SUCCESS)
    {
        return rc;
    }

    result = globus_resource_destroy_property(
            resource,
            &test_qname);

    if (result == GLOBUS_SUCCESS)
    {
        goto destroy_test_error;
    }
    else if (! globus_error_match(
                globus_error_peek(result),
                GLOBUS_WSRF_RESOURCE_MODULE,
                GLOBUS_WSRF_RESOURCE_ERROR_TYPE_UNKNOWN_PROPERTY))
    {
        goto destroy_test_error;
    }
    test_destroy(resource);
    return 0;

destroy_test_error:
    test_destroy(resource);
    return 1;
}

int
duplicate_create_property_test(void)
{
    int rc;
    globus_result_t result;
    globus_resource_t resource;
    xsd_QName test_qname = {
         "http://test.globus.org/",
         "test"
    };

    rc = test_init(&resource);
    
    if (rc != GLOBUS_SUCCESS)
    {
        return rc;
    }

    result = globus_resource_create_property(
            resource,
            &test_qname,
            &xsd_string_info,
            NULL);

    if (result != GLOBUS_SUCCESS)
    {
        goto destroy_test_error;
    }

    result = globus_resource_create_property(
            resource,
            &test_qname,
            &xsd_string_info,
            NULL);

    if (result == GLOBUS_SUCCESS)
    {
        goto destroy_test_error;
    }
    else if (! globus_error_match(
                globus_error_peek(result),
                GLOBUS_WSRF_RESOURCE_MODULE,
                GLOBUS_WSRF_RESOURCE_ERROR_TYPE_DUPLICATE_PROPERTY))
    {
        goto destroy_test_error;
    }
    test_destroy(resource);
    return 0;

destroy_test_error:
    test_destroy(resource);
    return 1;
}

int
duplicate_create_property_callback_test(void)
{
    int rc;
    globus_result_t result;
    globus_resource_t resource;
    xsd_QName test_qname = {
         "http://test.globus.org/",
         "test"
    };

    rc = test_init(&resource);
    
    if (rc != GLOBUS_SUCCESS)
    {
        return rc;
    }

    result = globus_resource_create_property(
            resource,
            &test_qname,
            &xsd_string_info,
            NULL);

    if (result != GLOBUS_SUCCESS)
    {
        goto destroy_test_error;
    }

    result = globus_resource_create_property_callback(
            resource,
            &test_qname,
            &xsd_string_info,
            property_callback,
            NULL);

    if (result == GLOBUS_SUCCESS)
    {
        goto destroy_test_error;
    }
    else if (! globus_error_match(
                globus_error_peek(result),
                GLOBUS_WSRF_RESOURCE_MODULE,
                GLOBUS_WSRF_RESOURCE_ERROR_TYPE_DUPLICATE_PROPERTY))
    {
        goto destroy_test_error;
    }
    test_destroy(resource);
    return 0;

destroy_test_error:
    test_destroy(resource);
    return 1;
}
int main()
{
    int i;
    int rc;
    int failed = 0;
    test_case tests[] =
    {
        /* Verify good behavior */
        create_property_test,
        create_destroy_property_test,
        create_delete_property_test,
        create_set_property_test,
        create_set_destroy_property_test,
        create_get_property_test,
        create_get_set_get_property_test,
        multi_create_property_test,
        multi_set_property_test,
        create_dynamic_property_test,
        create_get_dynamic_property_test,

        /* Verify error handling */
        null_create_property_test,
        null_create_property_callback_test,
        null_destroy_property_test,
        null_delete_property_test,
        null_set_property_test,
        null_get_property_test,
        unknown_get_property_test,
        unknown_set_property_test,
        unknown_delete_property_test,
        unknown_destroy_property_test,
        duplicate_create_property_test,
        duplicate_create_property_callback_test
    };

    printf("1..%d\n", SIZEOF_ARRAY(tests));

    for (i = 0; i < SIZEOF_ARRAY(tests); i++) {
        rc = (tests[i])();
        if (rc == 0)
        {
            printf("ok\n");
        }
        else
        {
            printf("not ok\n");
            failed++;
        }
    }
    return failed;
}
