/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */

#include "globus_wsrf_resource.h"

typedef int (*test_case)(void);

void
simple_free(void * x)
{
    free(x);
}

int
activate_deactivate_test(void)
{
    int rc;
    rc = globus_module_activate(GLOBUS_WSRF_RESOURCE_MODULE);
    if (rc != GLOBUS_SUCCESS)
    {
        goto error;
    }
    rc = globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);
    if (rc != GLOBUS_SUCCESS)
    {
        goto error;
    }
    return 0;
error:
    return 1;
}

int
create_destroy_test(void)
{
    globus_resource_t                   resource;
    globus_result_t                     result;
    int rc;

    rc = globus_module_activate(GLOBUS_WSRF_RESOURCE_MODULE);
    if (rc != GLOBUS_SUCCESS)
    {
        goto error;
    }

    result = globus_resource_create("resource_id", &resource);
    if (result != GLOBUS_SUCCESS)
    {
        goto deactivate_error;
    }

    result = globus_resource_destroy(resource);
    if (result != GLOBUS_SUCCESS)
    {
        goto deactivate_error;
    }

    rc = globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);
    if (result != GLOBUS_SUCCESS)
    {
        goto error;
    }

    return 0;
deactivate_error:
    globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);
error:
    return 1;
}

int
create_find_finish_test(void)
{
    globus_resource_t                   resource;
    globus_result_t                     result;
    int rc;

    rc = globus_module_activate(GLOBUS_WSRF_RESOURCE_MODULE);
    if (rc != GLOBUS_SUCCESS)
    {
        goto error;
    }

    result = globus_resource_create("resource_id", &resource);
    if (result != GLOBUS_SUCCESS)
    {
        goto deactivate_error;
    }

    result = globus_resource_find("resource_id", &resource);
    if (result != GLOBUS_SUCCESS)
    {
        goto deactivate_error;
    }

    result = globus_resource_finish(resource);
    if (result != GLOBUS_SUCCESS)
    {
        goto deactivate_error;
    }
    result = globus_resource_destroy(resource);
    if (result != GLOBUS_SUCCESS)
    {
        goto deactivate_error;
    }

    rc = globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);
    if (result != GLOBUS_SUCCESS)
    {
        goto error;
    }

    return 0;
deactivate_error:
    globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);
error:
    return 1;
}

int
create_nodestroy_test(void)
{
    globus_resource_t                   resource;
    globus_result_t                     result;
    int rc;

    rc = globus_module_activate(GLOBUS_WSRF_RESOURCE_MODULE);
    if (rc != GLOBUS_SUCCESS)
    {
        goto error;
    }

    result = globus_resource_create("resource_id", &resource);
    if (result != GLOBUS_SUCCESS)
    {
        goto deactivate_error;
    }

    rc = globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);
    if (result != GLOBUS_SUCCESS)
    {
        goto error;
    }

    return 0;
deactivate_error:
    globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);
error:
    return 1;
}

int
set_resource_specific_test(void)
{
    globus_resource_t                   resource;
    globus_result_t                     result;
    int rc;
    char *                              foo;

    rc = globus_module_activate(GLOBUS_WSRF_RESOURCE_MODULE);
    if (rc != GLOBUS_SUCCESS)
    {
        goto error;
    }

    result = globus_resource_create("resource_id", &resource);
    if (result != GLOBUS_SUCCESS)
    {
        goto deactivate_error;
    }

    foo = malloc(1);

    if (foo == NULL)
    {
        goto destroy_error;
    }

    result = globus_resource_set_resource_specific(
        resource,
        foo,
        simple_free);

    if (result != GLOBUS_SUCCESS)
    {
        goto destroy_error;
    }
        
    result = globus_resource_destroy(resource);
    if (result != GLOBUS_SUCCESS)
    {
        goto deactivate_error;
    }

    rc = globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);
    if (result != GLOBUS_SUCCESS)
    {
        goto error;
    }

    return 0;
destroy_error:
    globus_resource_destroy(resource);
deactivate_error:
    globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);
error:
    return 1;

}

int
set_get_resource_specific_test(void)
{
    globus_resource_t                   resource;
    globus_result_t                     result;
    int rc;
    void *                              foo;
    void *                              bar;

    rc = globus_module_activate(GLOBUS_WSRF_RESOURCE_MODULE);
    if (rc != GLOBUS_SUCCESS)
    {
        goto error;
    }

    result = globus_resource_create("resource_id", &resource);
    if (result != GLOBUS_SUCCESS)
    {
        goto deactivate_error;
    }

    foo = malloc(1);

    if (foo == NULL)
    {
        goto destroy_error;
    }

    result = globus_resource_set_resource_specific(
        resource,
        foo,
        simple_free);

    if (result != GLOBUS_SUCCESS)
    {
        goto destroy_error;
    }

    result = globus_resource_get_resource_specific(
        resource,
        &bar);

    if (result != GLOBUS_SUCCESS)
    {
        goto destroy_error;
    }

    if (foo != bar)
    {
        goto destroy_error;
    }
        
    result = globus_resource_destroy(resource);
    if (result != GLOBUS_SUCCESS)
    {
        goto deactivate_error;
    }

    rc = globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);
    if (result != GLOBUS_SUCCESS)
    {
        goto error;
    }

    return 0;
destroy_error:
    globus_resource_destroy(resource);
deactivate_error:
    globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);
error:
    return 1;
}

int
set_get_set_resource_specific_test(void)
{
    globus_resource_t                   resource;
    globus_result_t                     result;
    int rc;
    void *                              foo;
    void *                              bar;
    void *                              baz;

    rc = globus_module_activate(GLOBUS_WSRF_RESOURCE_MODULE);
    if (rc != GLOBUS_SUCCESS)
    {
        goto error;
    }

    result = globus_resource_create("resource_id", &resource);
    if (result != GLOBUS_SUCCESS)
    {
        goto deactivate_error;
    }

    foo = malloc(1);

    if (foo == NULL)
    {
        goto destroy_error;
    }

    baz = malloc(1);

    if (baz == NULL)
    {
        free(foo);

        goto destroy_error;
    }

    result = globus_resource_set_resource_specific(
        resource,
        foo,
        simple_free);

    if (result != GLOBUS_SUCCESS)
    {
        goto destroy_error;
    }

    result = globus_resource_get_resource_specific(
        resource,
        &bar);

    if (result != GLOBUS_SUCCESS)
    {
        goto destroy_error;
    }

    if (foo != bar)
    {
        goto destroy_error;
    }

    result = globus_resource_set_resource_specific(
        resource,
        baz,
        simple_free);
    if (foo != bar)
    {
        free(baz);
        goto destroy_error;
    }
        
    result = globus_resource_destroy(resource);
    if (result != GLOBUS_SUCCESS)
    {
        goto deactivate_error;
    }

    rc = globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);
    if (result != GLOBUS_SUCCESS)
    {
        goto error;
    }

    return 0;
destroy_error:
    globus_resource_destroy(resource);
deactivate_error:
    globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);
error:
    return 1;
}

int
set_get_registry_test(void)
{
    globus_resource_t                   resource;
    globus_result_t                     result;
    int rc;
    globus_xsd_type_registry_t          registry;
    globus_xsd_type_registry_t          registry2;

    rc = globus_module_activate(GLOBUS_WSRF_RESOURCE_MODULE);
    if (rc != GLOBUS_SUCCESS)
    {
        goto error;
    }

    result = globus_xsd_type_registry_init(
        &registry);
    if (result != GLOBUS_SUCCESS)
    {
        goto deactivate_error;
    }

    result = globus_resource_create("resource_id", &resource);
    if (result != GLOBUS_SUCCESS)
    {
        goto destroy_registry_error;
    }
    result = globus_resource_get_type_registry(
        resource,
        &registry2);

    if (result != GLOBUS_SUCCESS || registry2 != NULL)
    {
        goto destroy_error;
    }

    result = globus_resource_set_type_registry(
        resource,
        registry);

    if (result != GLOBUS_SUCCESS)
    {
        goto destroy_error;
    }

    result = globus_resource_get_type_registry(
        resource,
        &registry2);

    if (result != GLOBUS_SUCCESS || registry2 != registry)
    {
        goto destroy_error;
    }
        
    result = globus_resource_destroy(resource);
    if (result != GLOBUS_SUCCESS)
    {
        goto destroy_registry_error;
    }
    result = globus_xsd_type_registry_destroy(
        registry);
    if (result != GLOBUS_SUCCESS)
    {
        goto deactivate_error;
    }

    rc = globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);
    if (result != GLOBUS_SUCCESS)
    {
        goto error;
    }

    return 0;
destroy_error:
    globus_resource_destroy(resource);
destroy_registry_error:
    globus_xsd_type_registry_destroy(registry);
deactivate_error:
    globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);
error:
    return 1;
}

int
multi_create_test(void)
{
    globus_resource_t                   resource[256];
    char                                resource_id[80];
    int                                 i;
    globus_result_t                     result;
    int                                 rc;

    rc = globus_module_activate(GLOBUS_WSRF_RESOURCE_MODULE);
    if (rc != GLOBUS_SUCCESS)
    {
        goto error;
    }

    for (i = 0; i < sizeof(resource)/sizeof(globus_resource_t); i++)
    {
        sprintf(resource_id, "resource%03d", i);
        result = globus_resource_create(resource_id, &resource[i]);
        if (result != GLOBUS_SUCCESS)
        {
            goto deactivate_error;
        }
    }

    rc = globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);
    if (result != GLOBUS_SUCCESS)
    {
        goto error;
    }

    return 0;
deactivate_error:
    globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);
error:
    return 1;
}

typedef struct
{
    char * resource_id;
    int found_times;
    globus_bool_t done;
    globus_mutex_t mutex;
    globus_cond_t cond;
} destroy_time_t;

static
void
destroy_time_callback(void * arg)
{
    globus_resource_t                   resource;
    destroy_time_t *                    destroy = arg;
    globus_result_t                     result;

    globus_mutex_lock(&destroy->mutex);
    result = globus_resource_find(destroy->resource_id, &resource);
    if (result != GLOBUS_SUCCESS)
    {
        destroy->done = GLOBUS_TRUE;

        globus_cond_signal(&destroy->cond);
    }
    else
    {
        destroy->found_times++;
        globus_resource_finish(resource);
    }
    globus_mutex_unlock(&destroy->mutex);
}

int
create_set_destroy_time_test(void)
{
    globus_callback_handle_t            callback;
    globus_resource_t                   resource;
    globus_abstime_t                    destroy_time;
    globus_reltime_t                    periodic_time;
    globus_abstime_t                    wait_time;
    globus_result_t                     result;
    destroy_time_t                      destroy;
    int rc = 0;

    rc = globus_module_activate(GLOBUS_WSRF_RESOURCE_MODULE);
    if (rc != GLOBUS_SUCCESS)
    {
        goto error;
    }

    globus_mutex_init(&destroy.mutex, NULL);
    globus_cond_init(&destroy.cond, NULL);
    destroy.resource_id = "resource_id";
    destroy.found_times = 0;
    destroy.done = GLOBUS_FALSE;

    result = globus_resource_create("resource_id", &resource);
    if (result != GLOBUS_SUCCESS)
    {
        goto deactivate_error;
    }

    GlobusTimeReltimeSet(periodic_time, 1, 0);
    GlobusTimeAbstimeSet(destroy_time, 3, 0);
    GlobusTimeAbstimeSet(wait_time, 6, 0);

    result = globus_resource_set_destroy_time(resource, &destroy_time);
    if (result != GLOBUS_SUCCESS)
    {
        goto deactivate_error;
    }
    globus_mutex_lock(&destroy.mutex);
    result = globus_callback_register_periodic(
        &callback,
        &globus_i_reltime_zero,
        &periodic_time,
        destroy_time_callback,
        &destroy);
    if (result != GLOBUS_SUCCESS)
    {
        goto deactivate_error;
    }

    while ((!destroy.done) && rc != ETIMEDOUT)
    {
        rc = globus_cond_timedwait(
                &destroy.cond,
                &destroy.mutex,
                &wait_time);
    }
    if (rc == ETIMEDOUT)
    {
        goto unregister_periodic_error;
    }
    if ((!destroy.done) ||
            (destroy.found_times < 0 || destroy.found_times > 3))
    {
        goto unregister_periodic_error;
    }
    globus_mutex_destroy(&destroy.mutex);
    globus_cond_destroy(&destroy.cond);
    globus_callback_unregister(callback, NULL, NULL, NULL);
    rc = globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);
    if (result != GLOBUS_SUCCESS)
    {
        goto error;
    }

    return 0;

unregister_periodic_error:
    globus_mutex_destroy(&destroy.mutex);
    globus_cond_destroy(&destroy.cond);
    globus_callback_unregister(callback, NULL, NULL, NULL);
deactivate_error:
    globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);
error:
    return 1;
}

int
destroy_while_timeout_set_test(void)
{
    globus_resource_t                   resource;
    globus_result_t                     result;
    globus_abstime_t                    destroy_time;
    int rc;

    rc = globus_module_activate(GLOBUS_WSRF_RESOURCE_MODULE);
    if (rc != GLOBUS_SUCCESS)
    {
        goto error;
    }

    GlobusTimeAbstimeSet(destroy_time, 3, 0);

    result = globus_resource_create("resource_id", &resource);
    if (result != GLOBUS_SUCCESS)
    {
        goto deactivate_error;
    }

    result = globus_resource_set_destroy_time(resource, &destroy_time);
    if (result != GLOBUS_SUCCESS)
    {
        goto deactivate_error;
    }
    result = globus_resource_destroy(resource);
    if (result != GLOBUS_SUCCESS)
    {
        goto deactivate_error;
    }

    rc = globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);
    if (result != GLOBUS_SUCCESS)
    {
        goto error;
    }

    return 0;
deactivate_error:
    globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);
error:
    return 1;
}

int
create_set_destroy_time_and_deactivate_test(void)
{
    globus_resource_t                   resource;
    globus_result_t                     result;
    globus_abstime_t                    destroy_time;
    int rc;

    rc = globus_module_activate(GLOBUS_WSRF_RESOURCE_MODULE);
    if (rc != GLOBUS_SUCCESS)
    {
        goto error;
    }

    GlobusTimeAbstimeSet(destroy_time, 30, 0);

    result = globus_resource_create("resource_id", &resource);
    if (result != GLOBUS_SUCCESS)
    {
        goto deactivate_error;
    }

    result = globus_resource_set_destroy_time(resource, &destroy_time);
    if (result != GLOBUS_SUCCESS)
    {
        goto deactivate_error;
    }

    rc = globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);
    if (result != GLOBUS_SUCCESS)
    {
        goto error;
    }

    return 0;
deactivate_error:
    globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);
error:
    return 1;
}

int
finish_and_destroy_test(void)
{
    globus_resource_t                   resource;
    globus_resource_t                   resource2;
    globus_result_t                     result;
    int rc;

    rc = globus_module_activate(GLOBUS_WSRF_RESOURCE_MODULE);
    if (rc != GLOBUS_SUCCESS)
    {
        goto error;
    }

    result = globus_resource_create("resource_id", &resource);
    if (result != GLOBUS_SUCCESS)
    {
        goto deactivate_error;
    }

    result = globus_resource_find("resource_id", &resource2);
    if (result != GLOBUS_SUCCESS || resource2 != resource)
    {
        goto deactivate_error;
    }
    result = globus_resource_finish_and_destroy(resource2);
    if (result != GLOBUS_SUCCESS)
    {
        goto deactivate_error;
    }

    rc = globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);
    if (result != GLOBUS_SUCCESS)
    {
        goto error;
    }

    return 0;
deactivate_error:
    globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);
error:
    return 1;
}
int
null_create_test(void)
{
    globus_resource_t                   resource;
    globus_result_t                     result;
    int rc;

    rc = globus_module_activate(GLOBUS_WSRF_RESOURCE_MODULE);
    if (rc != GLOBUS_SUCCESS)
    {
        goto error;
    }

    result = globus_resource_create(NULL, NULL);
    if (result == GLOBUS_SUCCESS)
    {
        goto deactivate_error;
    }

    if (! globus_error_match(
            globus_error_peek(result),
            GLOBUS_WSRF_RESOURCE_MODULE,
            GLOBUS_WSRF_RESOURCE_ERROR_TYPE_NULL_PARAM))
    {
        goto deactivate_error;
    }

    result = globus_resource_create("resource id", NULL);
    if (result == GLOBUS_SUCCESS)
    {
        goto deactivate_error;
    }

    if (! globus_error_match(
            globus_error_peek(result),
            GLOBUS_WSRF_RESOURCE_MODULE,
            GLOBUS_WSRF_RESOURCE_ERROR_TYPE_NULL_PARAM))
    {
        goto deactivate_error;
    }

    result = globus_resource_create(NULL, &resource);
    if (result == GLOBUS_SUCCESS)
    {
        goto deactivate_error;
    }

    if (! globus_error_match(
            globus_error_peek(result),
            GLOBUS_WSRF_RESOURCE_MODULE,
            GLOBUS_WSRF_RESOURCE_ERROR_TYPE_NULL_PARAM))
    {
        goto deactivate_error;
    }

    rc = globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);
    if (rc != GLOBUS_SUCCESS)
    {
        goto error;
    }

    return 0;

deactivate_error:
    globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);
error:
    return 1;
}

int
null_destroy_test(void)
{
    globus_result_t                     result;
    int rc;

    rc = globus_module_activate(GLOBUS_WSRF_RESOURCE_MODULE);
    if (rc != GLOBUS_SUCCESS)
    {
        goto error;
    }

    result = globus_resource_destroy(NULL);
    if (result == GLOBUS_SUCCESS)
    {
        goto deactivate_error;
    }

    if (! globus_error_match(
            globus_error_peek(result),
            GLOBUS_WSRF_RESOURCE_MODULE,
            GLOBUS_WSRF_RESOURCE_ERROR_TYPE_NULL_PARAM))
    {
        goto deactivate_error;
    }

    rc = globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);
    if (rc != GLOBUS_SUCCESS)
    {
        goto error;
    }

    return 0;

deactivate_error:
    globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);
error:
    return 1;
}

int
null_finish_test(void)
{
    globus_result_t                     result;
    int rc;

    rc = globus_module_activate(GLOBUS_WSRF_RESOURCE_MODULE);
    if (rc != GLOBUS_SUCCESS)
    {
        goto error;
    }

    result = globus_resource_finish(NULL);
    if (result == GLOBUS_SUCCESS)
    {
        goto deactivate_error;
    }

    if (! globus_error_match(
            globus_error_peek(result),
            GLOBUS_WSRF_RESOURCE_MODULE,
            GLOBUS_WSRF_RESOURCE_ERROR_TYPE_NULL_PARAM))
    {
        goto deactivate_error;
    }

    rc = globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);
    if (rc != GLOBUS_SUCCESS)
    {
        goto error;
    }

    return 0;

deactivate_error:
    globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);
error:
    return 1;
}

int
null_finish_and_destroy_test(void)
{
    globus_result_t                     result;
    int rc;

    rc = globus_module_activate(GLOBUS_WSRF_RESOURCE_MODULE);
    if (rc != GLOBUS_SUCCESS)
    {
        goto error;
    }

    result = globus_resource_finish_and_destroy(NULL);
    if (result == GLOBUS_SUCCESS)
    {
        goto deactivate_error;
    }

    if (! globus_error_match(
            globus_error_peek(result),
            GLOBUS_WSRF_RESOURCE_MODULE,
            GLOBUS_WSRF_RESOURCE_ERROR_TYPE_NULL_PARAM))
    {
        goto deactivate_error;
    }

    rc = globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);
    if (rc != GLOBUS_SUCCESS)
    {
        goto error;
    }

    return 0;

deactivate_error:
    globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);
error:
    return 1;
}

int
null_set_destroy_time_test(void)
{
    globus_result_t                     result;
    globus_resource_t                   resource = (globus_resource_t) 0x1;
    globus_abstime_t                    timeout;
    int rc;

    rc = globus_module_activate(GLOBUS_WSRF_RESOURCE_MODULE);
    if (rc != GLOBUS_SUCCESS)
    {
        goto error;
    }

    result = globus_resource_set_destroy_time(NULL, &timeout);
    if (result == GLOBUS_SUCCESS)
    {
        goto deactivate_error;
    }

    if (! globus_error_match(
            globus_error_peek(result),
            GLOBUS_WSRF_RESOURCE_MODULE,
            GLOBUS_WSRF_RESOURCE_ERROR_TYPE_NULL_PARAM))
    {
        goto deactivate_error;
    }
    result = globus_resource_set_destroy_time(resource, NULL);
    if (result == GLOBUS_SUCCESS)
    {
        goto deactivate_error;
    }

    if (! globus_error_match(
            globus_error_peek(result),
            GLOBUS_WSRF_RESOURCE_MODULE,
            GLOBUS_WSRF_RESOURCE_ERROR_TYPE_NULL_PARAM))
    {
        goto deactivate_error;
    }

    rc = globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);
    if (rc != GLOBUS_SUCCESS)
    {
        goto error;
    }

    return 0;

deactivate_error:
    globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);
error:
    return 1;
}
int
null_set_resource_specific_test(void)
{
    globus_resource_t                   resource;
    globus_result_t                     result;
    int rc;
    void *                              foo;

    rc = globus_module_activate(GLOBUS_WSRF_RESOURCE_MODULE);
    if (rc != GLOBUS_SUCCESS)
    {
        goto error;
    }

    result = globus_resource_create("resource_id", &resource);
    if (result != GLOBUS_SUCCESS)
    {
        goto deactivate_error;
    }

    foo = malloc(1);

    if (foo == NULL)
    {
        goto destroy_error;
    }

    result = globus_resource_set_resource_specific(
        NULL,
        foo,
        simple_free);

    if (result == GLOBUS_SUCCESS)
    {
        free(foo);
        goto destroy_error;
    }
    if (! globus_error_match(
            globus_error_peek(result),
            GLOBUS_WSRF_RESOURCE_MODULE,
            GLOBUS_WSRF_RESOURCE_ERROR_TYPE_NULL_PARAM))
    {
        free(foo);
        goto deactivate_error;
    }
    
    result = globus_resource_set_resource_specific(
        resource,
        NULL,
        simple_free);

    if (result != GLOBUS_SUCCESS)
    {
        free(foo);
        goto destroy_error;
    }

    free(foo);

    result = globus_resource_destroy(resource);
    if (result != GLOBUS_SUCCESS)
    {
        goto deactivate_error;
    }

    rc = globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);
    if (result != GLOBUS_SUCCESS)
    {
        goto error;
    }

    return 0;
destroy_error:
    globus_resource_destroy(resource);
deactivate_error:
    globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);
error:
    return 1;

}

int
null_get_resource_specific_test(void)
{
    globus_resource_t                   resource;
    globus_result_t                     result;
    int rc;
    void *                              foo;

    rc = globus_module_activate(GLOBUS_WSRF_RESOURCE_MODULE);
    if (rc != GLOBUS_SUCCESS)
    {
        goto error;
    }

    result = globus_resource_create("resource_id", &resource);
    if (result != GLOBUS_SUCCESS)
    {
        goto deactivate_error;
    }

    result = globus_resource_get_resource_specific(
        NULL,
        &foo);

    if (result == GLOBUS_SUCCESS)
    {
        goto destroy_error;
    }
    if (! globus_error_match(
            globus_error_peek(result),
            GLOBUS_WSRF_RESOURCE_MODULE,
            GLOBUS_WSRF_RESOURCE_ERROR_TYPE_NULL_PARAM))
    {
        goto deactivate_error;
    }
    
    result = globus_resource_get_resource_specific(
        resource,
        NULL);

    if (result == GLOBUS_SUCCESS)
    {
        goto destroy_error;
    }
    if (! globus_error_match(
            globus_error_peek(result),
            GLOBUS_WSRF_RESOURCE_MODULE,
            GLOBUS_WSRF_RESOURCE_ERROR_TYPE_NULL_PARAM))
    {
        goto deactivate_error;
    }

    result = globus_resource_destroy(resource);
    if (result != GLOBUS_SUCCESS)
    {
        goto deactivate_error;
    }

    rc = globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);
    if (result != GLOBUS_SUCCESS)
    {
        goto error;
    }

    return 0;
destroy_error:
    globus_resource_destroy(resource);
deactivate_error:
    globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);
error:
    return 1;

}

int
null_registry_test(void)
{
    globus_resource_t                   resource;
    globus_result_t                     result;
    int rc;
    globus_xsd_type_registry_t          registry;

    rc = globus_module_activate(GLOBUS_WSRF_RESOURCE_MODULE);
    if (rc != GLOBUS_SUCCESS)
    {
        goto error;
    }

    result = globus_resource_create("resource_id", &resource);
    if (result != GLOBUS_SUCCESS)
    {
        goto deactivate_error;
    }
    result = globus_resource_get_type_registry(
        resource,
        NULL);

    if (result == GLOBUS_SUCCESS ||
            (! globus_error_match(
                    globus_error_peek(result),
                    GLOBUS_WSRF_RESOURCE_MODULE,
                    GLOBUS_WSRF_RESOURCE_ERROR_TYPE_NULL_PARAM)))
    {
        goto destroy_error;
    }

    result = globus_resource_get_type_registry(
        NULL,
        &registry);

    if (result == GLOBUS_SUCCESS ||
            (! globus_error_match(
                    globus_error_peek(result),
                    GLOBUS_WSRF_RESOURCE_MODULE,
                    GLOBUS_WSRF_RESOURCE_ERROR_TYPE_NULL_PARAM)))
    {
        goto destroy_error;
    }

    result = globus_resource_destroy(resource);
    if (result != GLOBUS_SUCCESS)
    {
        goto deactivate_error;
    }
    rc = globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);
    if (result != GLOBUS_SUCCESS)
    {
        goto error;
    }

    return 0;
destroy_error:
    globus_resource_destroy(resource);
deactivate_error:
    globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);
error:
    return 1;
}

int
bad_find_test(void)
{
    globus_resource_t                   resource;
    globus_result_t                     result;
    int rc;

    rc = globus_module_activate(GLOBUS_WSRF_RESOURCE_MODULE);
    if (rc != GLOBUS_SUCCESS)
    {
        goto error;
    }

    result = globus_resource_find("resource_id", &resource);
    if (result == GLOBUS_SUCCESS)
    {
        goto deactivate_error;
    }
    if (! globus_error_match(
            globus_error_peek(result),
            GLOBUS_WSRF_RESOURCE_MODULE,
            GLOBUS_WSRF_RESOURCE_ERROR_TYPE_UNKNOWN_RESOURCE))
    {
        goto deactivate_error;
    }

    rc = globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);
    if (rc != GLOBUS_SUCCESS)
    {
        goto error;
    }

    return 0;
deactivate_error:
    globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);
error:
    return 1;
}

int
duplicate_create_test(void)
{
    globus_resource_t                   resource1;
    globus_resource_t                   resource2;
    globus_result_t                     result;
    int rc;

    rc = globus_module_activate(GLOBUS_WSRF_RESOURCE_MODULE);
    if (rc != GLOBUS_SUCCESS)
    {
        goto error;
    }

    result = globus_resource_create("resource_id", &resource1);
    if (result != GLOBUS_SUCCESS)
    {
        goto deactivate_error;
    }
    result = globus_resource_create("resource_id", &resource2);
    if (result == GLOBUS_SUCCESS)
    {
        goto destroy_error;
    }
    if (! globus_error_match(
            globus_error_peek(result),
            GLOBUS_WSRF_RESOURCE_MODULE,
            GLOBUS_WSRF_RESOURCE_ERROR_TYPE_DUPLICATE_RESOURCE))
    {
        goto destroy_error;
    }
    result = globus_resource_destroy(resource1);
    if (result != GLOBUS_SUCCESS)
    {
        goto deactivate_error;
    }

    rc = globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);
    if (rc != GLOBUS_SUCCESS)
    {
        goto error;
    }

    return 0;

destroy_error:
    globus_resource_destroy(resource1);
deactivate_error:
    globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);
error:
    return 1;
}

int
finish_without_find_test(void)
{
    globus_resource_t                   resource;
    globus_result_t                     result;
    int rc;

    rc = globus_module_activate(GLOBUS_WSRF_RESOURCE_MODULE);
    if (rc != GLOBUS_SUCCESS)
    {
        goto error;
    }

    result = globus_resource_create("resource_id", &resource);
    if (result != GLOBUS_SUCCESS)
    {
        goto deactivate_error;
    }
    result = globus_resource_finish(resource);

    if (result == GLOBUS_SUCCESS ||
            (! globus_error_match(
                    globus_error_peek(result),
                    GLOBUS_WSRF_RESOURCE_MODULE,
                    GLOBUS_WSRF_RESOURCE_ERROR_TYPE_NO_REFERENCES)))
    {
        goto destroy_error;
    }

    result = globus_resource_destroy(resource);
    if (result != GLOBUS_SUCCESS)
    {
        goto deactivate_error;
    }
    rc = globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);
    if (result != GLOBUS_SUCCESS)
    {
        goto error;
    }

    return 0;
destroy_error:
    globus_resource_destroy(resource);
deactivate_error:
    globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);
error:
    return 1;
}

int
finish_and_destroy_without_find_test(void)
{
    globus_resource_t                   resource;
    globus_result_t                     result;
    int rc;

    rc = globus_module_activate(GLOBUS_WSRF_RESOURCE_MODULE);
    if (rc != GLOBUS_SUCCESS)
    {
        goto error;
    }

    result = globus_resource_create("resource_id", &resource);
    if (result != GLOBUS_SUCCESS)
    {
        goto deactivate_error;
    }
    result = globus_resource_finish_and_destroy(resource);

    if (result == GLOBUS_SUCCESS ||
            (! globus_error_match(
                    globus_error_peek(result),
                    GLOBUS_WSRF_RESOURCE_MODULE,
                    GLOBUS_WSRF_RESOURCE_ERROR_TYPE_NO_REFERENCES)))
    {
        goto destroy_error;
    }

    result = globus_resource_destroy(resource);
    if (result != GLOBUS_SUCCESS)
    {
        goto deactivate_error;
    }
    rc = globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);
    if (result != GLOBUS_SUCCESS)
    {
        goto error;
    }

    return 0;
destroy_error:
    globus_resource_destroy(resource);
deactivate_error:
    globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);
error:
    return 1;
}

#define SIZEOF_ARRAY(x) sizeof(x) / sizeof(x[0])

int main()
{
    int i;
    int rc;
    int failed = 0;
    test_case tests[] =
    {
        /* Verify good behavior */
        activate_deactivate_test,
        create_destroy_test,
        create_find_finish_test,
        create_nodestroy_test,
        set_resource_specific_test,
        set_get_resource_specific_test,
        set_get_set_resource_specific_test,
        set_get_registry_test,
        multi_create_test,
        create_set_destroy_time_test,
        destroy_while_timeout_set_test,
        create_set_destroy_time_and_deactivate_test,
        finish_and_destroy_test,

        /* Verify error handling */
        null_create_test,
        null_destroy_test,
        null_finish_test,
        null_finish_and_destroy_test,
        null_set_destroy_time_test,
        null_set_resource_specific_test,
        null_get_resource_specific_test,
        null_registry_test,
        bad_find_test,
        duplicate_create_test,
        finish_without_find_test,
        finish_and_destroy_without_find_test
    };

    printf("1..%d\n", SIZEOF_ARRAY(tests));

    for (i = 0; i < SIZEOF_ARRAY(tests); i++) {
        rc = (tests[i])();
        if (rc == 0)
        {
            printf("ok\n");
        }
        else
        {
            printf("not ok\n");
            failed++;
        }
    }
    return failed;
}
