
#include "SecureCounterService_skeleton.h"
#include "SecureCounterService_internal_skeleton.h"

#include "globus_wsrf_resource.h"
#include "globus_wsrf_core_tools.h"
#include "globus_soap_message_handle.h"

#include "globus_ws_addressing.h"
#include "globus_ws_security.h"

extern globus_service_descriptor_t SecureCounterService_descriptor;


globus_result_t
SecureCounterService_init(
    globus_service_descriptor_t *       service_desc)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(SecureCounterService_init);
    SecureCounterServiceDebugEnter();

    result = (globus_result_t)
	globus_module_activate(GLOBUS_WSRF_RESOURCE_MODULE);
    if(result != GLOBUS_SUCCESS)
    {
	goto exit;
    }
    
    if(!service_desc->handlers)
    {
        globus_handler_chain_init(&service_desc->handlers);
    }

    result = globus_extension_activate(
	GLOBUS_HANDLER_WS_SECMESS_LIB);
    if(result != GLOBUS_SUCCESS)
    {
	goto exit;
    }

    result = globus_extension_activate(
	GLOBUS_HANDLER_WS_ADDRESSING_LIB);
    if(result != GLOBUS_SUCCESS)
    {
	goto exit;
    }
    
    result = globus_handler_chain_push(
	service_desc->handlers,
	GLOBUS_HANDLER_TYPE_ALL,
	GLOBUS_HANDLER_WS_ADDRESSING_SERVER);
    if(result != GLOBUS_SUCCESS)
    {
        goto exit;
    }
 
    result = globus_handler_chain_push(
	service_desc->handlers,
	GLOBUS_HANDLER_TYPE_ALL,
	GLOBUS_HANDLER_WS_SECMESS_SERVER);
    if(result != GLOBUS_SUCCESS)
    {
	goto exit;
    }
   
exit:
    SecureCounterServiceDebugExit();
    return result;
}

globus_result_t
SecureCounterService_finalize(
    globus_service_descriptor_t *       service_desc)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(SecureCounterService_finalize);

    SecureCounterServiceDebugEnter();

    /* do any service specific finalize stuff here, 
     * opposite of CounterService_init
     */

    if(service_desc->handlers)
    {
        globus_handler_chain_destroy(
	    service_desc->handlers);
	service_desc->handlers = NULL;
    }

    result = globus_extension_deactivate(
	GLOBUS_HANDLER_WS_SECMESS_LIB);
    if(result != GLOBUS_SUCCESS)
    {
	goto exit;
    }

    result = globus_extension_deactivate(
	GLOBUS_HANDLER_WS_ADDRESSING_LIB);
    if(result != GLOBUS_SUCCESS)
    {
	goto exit;
    }
 
    result = globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);

exit:
    SecureCounterServiceDebugExit();
    return result;
}

/* these functions are the service implementation and
 * must be implemented by the service implementer
 */

globus_result_t
CounterPortType_GetCurrentMessage_init(
    globus_service_engine_t             engine,
    globus_soap_message_handle_t        message,
    wsnt_GetCurrentMessageType * GetCurrentMessage)
{
    /* add function local variable declarations here */
    globus_result_t                     result = GLOBUS_SUCCESS;

    /* initialize trace debugging info */
    GlobusFuncName(CounterPortType_GetCurrentMessage_init);
    SecureCounterServiceDebugEnter();

    /* Service implementer may implmenent this function.  Since this
     * function gets called before deserialization of the request
     * message, the input variable GetCurrentMessage can be initialized
     * however the implementer wants, as well as making any configuration
     * changes to the service engine or soap message handles.
     *
     * If no configuration or initialization needs to be done, this
     * call can remain empty.
     */

    SecureCounterServiceDebugExit();
    return GLOBUS_SUCCESS;
}

globus_result_t
CounterPortType_GetCurrentMessage_impl(
    globus_service_engine_t             engine,
    globus_soap_message_handle_t        message,
    globus_service_descriptor_t *       service,
    wsnt_GetCurrentMessageType * GetCurrentMessage,
    wsnt_GetCurrentMessageResponseType * GetCurrentMessageResponse,
    const char **                       fault_name,
    void ** fault)
{
    /* add function local variable declarations here */
    globus_result_t                     result = GLOBUS_SUCCESS;

    /* initialize trace debugging info */
    GlobusFuncName(CounterPortType_GetCurrentMessage_impl);
    SecureCounterServiceDebugEnter();
     
    /* This is where it all happens.  Service implementer must 
     * implmenent this function.  Asume that GetCurrentMessage has
     * been initialized and filled with request values.  
     * GetCurrentMessageResponse must be set by the implementer.
     */

    result = SecureCounterServiceErrorNotImplemented(
        "CounterPorType_GetCurrentMessage_impl");

    SecureCounterServiceDebugExit();
    return result;     
}



/* these functions are the service implementation and
 * must be implemented by the service implementer
 */

globus_result_t
CounterPortType_Subscribe_init(
    globus_service_engine_t             engine,
    globus_soap_message_handle_t        message,
    wsnt_SubscribeType * Subscribe)
{
    /* add function local variable declarations here */
    globus_result_t                     result = GLOBUS_SUCCESS;

    /* initialize trace debugging info */
    GlobusFuncName(CounterPortType_Subscribe_init);
    SecureCounterServiceDebugEnter();

    /* Service implementer may implmenent this function.  Since this
     * function gets called before deserialization of the request
     * message, the input variable Subscribe can be initialized
     * however the implementer wants, as well as making any configuration
     * changes to the service engine or soap message handles.
     *
     * If no configuration or initialization needs to be done, this
     * call can remain empty.
     */

    SecureCounterServiceDebugExit();
    return GLOBUS_SUCCESS;
}

globus_result_t
CounterPortType_Subscribe_impl(
    globus_service_engine_t             engine,
    globus_soap_message_handle_t        message,
    globus_service_descriptor_t *       service,
    wsnt_SubscribeType * Subscribe,
    wsnt_SubscribeResponseType * SubscribeResponse,
    const char **                       fault_name,
    void ** fault)
{
    /* add function local variable declarations here */
    globus_result_t                     result = GLOBUS_SUCCESS;

    /* initialize trace debugging info */
    GlobusFuncName(CounterPortType_Subscribe_impl);
    SecureCounterServiceDebugEnter();
     
    /* This is where it all happens.  Service implementer must 
     * implmenent this function.  Asume that Subscribe has
     * been initialized and filled with request values.  
     * SubscribeResponse must be set by the implementer.
     */

    result = SecureCounterServiceErrorNotImplemented(
        "CounterPortType_Subscribe_impl");

    SecureCounterServiceDebugExit();
    return result;     
}

/* these functions are the service implementation and
 * must be implemented by the service implementer
 */

globus_result_t
CounterPortType_GetMultipleResourceProperties_init(
    globus_service_engine_t             engine,
    globus_soap_message_handle_t        message,
    wsrp_GetMultipleResourcePropertiesType * GetMultipleResourceProperties)
{
    /* add function local variable declarations here */
    globus_result_t                     result = GLOBUS_SUCCESS;

    /* initialize trace debugging info */
    GlobusFuncName(CounterPortType_GetResourceProperties_init);
    SecureCounterServiceDebugEnter();

    /* Service implementer may implmenent this function.  Since this
     * function gets called before deserialization of the request
     * message, the input variable GetResourceProperty can be initialized
     * however the implementer wants, as well as making any configuration
     * changes to the service engine or soap message handles.
     *
     * If no configuration or initialization needs to be done, this
     * call can remain empty.
     */

    SecureCounterServiceDebugExit();
    return GLOBUS_SUCCESS;
}

globus_result_t
CounterPortType_GetMultipleResourceProperties_impl(
    globus_service_engine_t             engine,
    globus_soap_message_handle_t        message,
    globus_service_descriptor_t *       service,
    wsrp_GetMultipleResourcePropertiesType * GetResourceProperty,
    wsrp_GetMultipleResourcePropertiesResponseType * GetMultipleResourcePropertiesResponse,
    const char **                       fault_name,
    void ** fault)
{
    /* add function local variable declarations here */
    globus_result_t                     result = GLOBUS_SUCCESS;

    /* initialize trace debugging info */
    GlobusFuncName(CounterPortType_GetMultipleResourceProperties_impl);
    SecureCounterServiceDebugEnter();
     
    /* This is where it all happens.  Service implementer must 
     * implmenent this function.  Asume that GetResourceProperty has
     * been initialized and filled with request values.  
     * GetResourcePropertyResponse must be set by the implementer.
     */

    result = SecureCounterServiceErrorNotImplemented(
        "CounterPortType_GetMultipleResourceProperties_impl");

    SecureCounterServiceDebugExit();
    return result;     
}


/* these functions are the service implementation and
 * must be implemented by the service implementer
 */

globus_result_t
CounterPortType_GetResourceProperty_init(
    globus_service_engine_t             engine,
    globus_soap_message_handle_t        message,
    xsd_QName * GetResourceProperty)
{
    /* add function local variable declarations here */
    globus_result_t                     result = GLOBUS_SUCCESS;

    /* initialize trace debugging info */
    GlobusFuncName(CounterPortType_GetResourceProperty_init);
    SecureCounterServiceDebugEnter();

    /* Service implementer may implmenent this function.  Since this
     * function gets called before deserialization of the request
     * message, the input variable GetResourceProperty can be initialized
     * however the implementer wants, as well as making any configuration
     * changes to the service engine or soap message handles.
     *
     * If no configuration or initialization needs to be done, this
     * call can remain empty.
     */

    SecureCounterServiceDebugExit();
    return GLOBUS_SUCCESS;
}

globus_result_t
CounterPortType_GetResourceProperty_impl(
    globus_service_engine_t             engine,
    globus_soap_message_handle_t        message,
    globus_service_descriptor_t *       service,
    xsd_QName * GetResourceProperty,
    wsrp_GetResourcePropertyResponseType * GetResourcePropertyResponse,
    const char **                       fault_name,
    void ** fault)
{
    /* add function local variable declarations here */
    globus_result_t                     result = GLOBUS_SUCCESS;

    /* initialize trace debugging info */
    GlobusFuncName(CounterPortType_GetResourceProperty_impl);
    SecureCounterServiceDebugEnter();
     
    /* This is where it all happens.  Service implementer must 
     * implmenent this function.  Asume that GetResourceProperty has
     * been initialized and filled with request values.  
     * GetResourcePropertyResponse must be set by the implementer.
     */

    result = SecureCounterServiceErrorNotImplemented(
        "CounterPortType_GetResourceProperty_impl");

    SecureCounterServiceDebugExit();
    return result;     
}

/* these functions are the service implementation and
 * must be implemented by the service implementer
 */

globus_result_t
CounterPortType_QueryResourceProperties_init(
    globus_service_engine_t             engine,
    globus_soap_message_handle_t        message,
    wsrp_QueryResourcePropertiesType * QueryResourceProperties)
{
    /* add function local variable declarations here */
    globus_result_t                     result = GLOBUS_SUCCESS;

    /* initialize trace debugging info */
    GlobusFuncName(CounterPortType_QueryResourceProperties_init);
    SecureCounterServiceDebugEnter();

    /* Service implementer may implmenent this function.  Since this
     * function gets called before deserialization of the request
     * message, the input variable QueryResourceProperties can be initialized
     * however the implementer wants, as well as making any configuration
     * changes to the service engine or soap message handles.
     *
     * If no configuration or initialization needs to be done, this
     * call can remain empty.
     */

    SecureCounterServiceDebugExit();
    return GLOBUS_SUCCESS;
}

globus_result_t
CounterPortType_QueryResourceProperties_impl(
    globus_service_engine_t             engine,
    globus_soap_message_handle_t        message,
    globus_service_descriptor_t *       descriptor,
    wsrp_QueryResourcePropertiesType * QueryResourceProperties,
    wsrp_QueryResourcePropertiesResponseType * QueryResourcePropertiesResponse,
    const char **                       fault_name,
    void ** fault)
{
    /* add function local variable declarations here */
    globus_result_t                     result = GLOBUS_SUCCESS;

    /* initialize trace debugging info */
    GlobusFuncName(CounterPortType_QueryResourceProperties_impl);
    SecureCounterServiceDebugEnter();
     
    /* This is where it all happens.  Service implementer must 
     * implmenent this function.  Asume that QueryResourceProperties has
     * been initialized and filled with request values.  
     * QueryResourcePropertiesResponse must be set by the implementer.
     */

    result = SecureCounterServiceErrorNotImplemented(
        "CounterPortType_QueryResourceProperties_impl");

    SecureCounterServiceDebugExit();
    return result;     
}



/* these functions are the service implementation and
 * must be implemented by the service implementer
 */

globus_result_t
CounterPortType_Destroy_init(
    globus_service_engine_t             engine,
    globus_soap_message_handle_t        message,
    wsrl_DestroyType * Destroy)
{
    /* add function local variable declarations here */
    globus_result_t                     result = GLOBUS_SUCCESS;

    /* initialize trace debugging info */
    GlobusFuncName(CounterPortType_Destroy_init);
    SecureCounterServiceDebugEnter();

    /* Service implementer may implmenent this function.  Since this
     * function gets called before deserialization of the request
     * message, the input variable Destroy can be initialized
     * however the implementer wants, as well as making any configuration
     * changes to the service engine or soap message handles.
     *
     * If no configuration or initialization needs to be done, this
     * call can remain empty.
     */

    SecureCounterServiceDebugExit();
    return GLOBUS_SUCCESS;
}

globus_result_t
CounterPortType_Destroy_impl(
    globus_service_engine_t             engine,
    globus_soap_message_handle_t        message,
    globus_service_descriptor_t *       service,
    wsrl_DestroyType * Destroy,
    wsrl_DestroyResponseType * DestroyResponse,
    const char **                       fault_name,
    void ** fault)
{
    /* add function local variable declarations here */
    globus_result_t                     result = GLOBUS_SUCCESS;

    /* initialize trace debugging info */
    GlobusFuncName(CounterPortType_Destroy_impl);
    SecureCounterServiceDebugEnter();
     
    /* This is where it all happens.  Service implementer must 
     * implmenent this function.  Asume that Destroy has
     * been initialized and filled with request values.  
     * DestroyResponse must be set by the implementer.
     */

    result = SecureCounterServiceErrorNotImplemented(
        "CounterPortType_Destroy_impl");

    SecureCounterServiceDebugExit();
    return result;     
}



/* these functions are the service implementation and
 * must be implemented by the service implementer
 */

globus_result_t
CounterPortType_SetTerminationTime_init(
    globus_service_engine_t             engine,
    globus_soap_message_handle_t        message,
    wsrl_SetTerminationTimeType * SetTerminationTime)
{
    /* add function local variable declarations here */
    globus_result_t                     result = GLOBUS_SUCCESS;

    /* initialize trace debugging info */
    GlobusFuncName(CounterPortType_SetTerminationTime_init);
    SecureCounterServiceDebugEnter();

    /* Service implementer may implmenent this function.  Since this
     * function gets called before deserialization of the request
     * message, the input variable SetTerminationTime can be initialized
     * however the implementer wants, as well as making any configuration
     * changes to the service engine or soap message handles.
     *
     * If no configuration or initialization needs to be done, this
     * call can remain empty.
     */

    SecureCounterServiceDebugExit();
    return GLOBUS_SUCCESS;
}

globus_result_t
CounterPortType_SetTerminationTime_impl(
    globus_service_engine_t             engine,
    globus_soap_message_handle_t        message,
    globus_service_descriptor_t *       service,
    wsrl_SetTerminationTimeType * SetTerminationTime,
    wsrl_SetTerminationTimeResponseType * SetTerminationTimeResponse,
    const char **                       fault_name,
    void ** fault)
{
    /* add function local variable declarations here */
    globus_result_t                     result = GLOBUS_SUCCESS;

    /* initialize trace debugging info */
    GlobusFuncName(CounterPortType_SetTerminationTime_impl);
    SecureCounterServiceDebugEnter();
     
    /* This is where it all happens.  Service implementer must 
     * implmenent this function.  Asume that SetTerminationTime has
     * been initialized and filled with request values.  
     * SetTerminationTimeResponse must be set by the implementer.
     */

    result = SecureCounterServiceErrorNotImplemented(
        "CounterPortType_SetTerminationTime_impl");

    SecureCounterServiceDebugExit();
    return result;     
}



/* these functions are the service implementation and
 * must be implemented by the service implementer
 */

globus_result_t
CounterPortType_add_init(
    globus_service_engine_t             engine,
    globus_soap_message_handle_t        message,
    xsd_int * add)
{
    /* add function local variable declarations here */
    globus_result_t                     result = GLOBUS_SUCCESS;

    /* initialize trace debugging info */
    GlobusFuncName(CounterPortType_add_init);
    SecureCounterServiceDebugEnter();

    /* Service implementer may implmenent this function.  Since this
     * function gets called before deserialization of the request
     * message, the input variable add can be initialized
     * however the implementer wants, as well as making any configuration
     * changes to the service engine or soap message handles.
     *
     * If no configuration or initialization needs to be done, this
     * call can remain empty.
     */

    SecureCounterServiceDebugExit();
    return GLOBUS_SUCCESS;
}

globus_result_t
CounterPortType_add_impl(
    globus_service_engine_t             engine,
    globus_soap_message_handle_t        message,
    globus_service_descriptor_t *       service,
    xsd_int * add,
    xsd_int * addResponse,
    const char **                       fault_name,
    void ** fault)
{
    /* add function local variable declarations here */
    xsd_int *                           counter_value = NULL;
    globus_soap_message_handle_t        memory_message = NULL;
    globus_resource_t                   resource = NULL;
    globus_result_t                     result = GLOBUS_SUCCESS;

    /* initialize trace debugging info */
    GlobusFuncName(CounterPortType_add_init);
    SecureCounterServiceDebugEnter();

    result = globus_wsrf_core_get_resource(
        message,
        service,
        &resource);
    if(result != GLOBUS_SUCCESS)
    {
        result = CounterPortType_add_chain_error(
            result,
            "Failed to get resource from EPR");
        goto exit;
    }

    result = globus_resource_get_property(resource,
        &CounterPortType_Value_rp_qname, (void **)&counter_value, NULL);
    if(result != GLOBUS_SUCCESS)
    {
        result = CounterPortType_add_chain_error(
            result, 
            "Failed to get CounterRP resource property from resource");
        goto finish_exit;
    }

    *counter_value += *add;
    *addResponse = *counter_value;

finish_exit:
    globus_resource_finish(resource);
 exit:

    if(memory_message)
    {
        globus_soap_message_handle_destroy(memory_message);
    }
    
    SecureCounterServiceDebugExit();
    return result;     
}



/* these functions are the service implementation and
 * must be implemented by the service implementer
 */

globus_result_t
CounterPortType_createCounter_init(
    globus_service_engine_t             engine,
    globus_soap_message_handle_t        message,
    createCounterType * createCounter)
{
    /* add function local variable declarations here */
    globus_result_t                     result = GLOBUS_SUCCESS;

    /* initialize trace debugging info */
    GlobusFuncName(CounterPortType_createCounter_init);
    SecureCounterServiceDebugEnter();

    /* Service implementer may implmenent this function.  Since this
     * function gets called before deserialization of the request
     * message, the input variable createCounter can be initialized
     * however the implementer wants, as well as making any configuration
     * changes to the service engine or soap message handles.
     *
     * If no configuration or initialization needs to be done, this
     * call can remain empty.
     */

    SecureCounterServiceDebugExit();
    return GLOBUS_SUCCESS;
}

globus_result_t
CounterPortType_createCounter_impl(
    globus_service_engine_t             engine,
    globus_soap_message_handle_t        message,
    globus_service_descriptor_t *       service,
    createCounterType * createCounter,
    createCounterResponseType * createCounterResponse,
    const char **                       fault_name,
    void ** fault)
{
    /* add function local variable declarations here */
    xsd_any *                           counter_key;
    char *                              resource_id;
    xsd_int *                           counter_value;
    globus_resource_t                   resource;
    char *                              endpoint;
    globus_result_t                     result = GLOBUS_SUCCESS;

    /* initialize trace debugging info */
    GlobusFuncName(CounterPortType_createCounter_init);
    SecureCounterServiceDebugEnter();
     
    resource_id = globus_common_create_string("%d", random());

    globus_resource_create(
        resource_id, &resource);
    
    result = SecureCounterServiceInitResource(resource_id);
    if(result != GLOBUS_SUCCESS)
    {
        result = CounterPortType_add_chain_error(
            result, "Failed to initialize resource with service");
        goto exit;
    }

    memset(createCounterResponse, 0, sizeof(createCounterResponseType));

    wsa_ReferencePropertiesType_init(
        &createCounterResponse->EndpointReference.ReferenceProperties);

    counter_key = xsd_any_array_push(
        &createCounterResponse->EndpointReference.ReferenceProperties->any);

    counter_key->any_info = &xsd_string_info;
    xsd_QName_init(&counter_key->element);
    counter_key->element->local = globus_libc_strdup("CounterKey");
    counter_key->element->Namespace = globus_libc_strdup(
        SecureCounterService_service_qname.Namespace);
    xsd_string_init_cstr((xsd_string **)&counter_key->value, resource_id);

    globus_service_engine_get_contact(engine, &endpoint);

    createCounterResponse->EndpointReference.Address.base_value =
    globus_common_create_string("%s/%s", endpoint, 
				SECURECOUNTERSERVICE_BASE_PATH);

    xsd_int_init(&counter_value);
    *counter_value = 0;

    result = globus_resource_create_property(
        resource, 
        &CounterPortType_Value_rp_qname, 
        &CounterPortType_Value_rp_info, counter_value);
    if(result != GLOBUS_SUCCESS)
    {
        result = CounterPortType_add_chain_error(
            result, "Failed to create resource property for CounterRP");
        goto exit;
    }

 exit:

    SecureCounterServiceDebugExit();
    return result;     
}


