/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */


#include "CounterService_client.h"
#include "globus_xml_buffer.h"

int main(int argc, char * argv[])
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    CounterService_client_handle_t      client_handle;
    createCounterType                   createCounter;
    createCounterResponseType *         createCounterResponse = NULL;
    int                                 fault_type;
    xsd_any *                           fault;
    int *                               addResponse = NULL;
    globus_xml_buffer *                 counterKey = NULL;
    int                                 add_value;
    xsd_any_array                       referenceProperties;
    globus_handler_chain_t              chain;

    if(argc < 2)
    {
        fprintf(stderr, "usage: test-secure-transport-counter <URL>\n");
        exit(1);
    }

    globus_module_activate(COUNTERSERVICE_MODULE);

    globus_handler_chain_init(&chain);

    result = globus_extension_activate("globus_handler_debug");
    if(result != GLOBUS_SUCCESS)
    {
        goto exit;
    }

    result = globus_extension_activate("globus_handler_ws_addressing");
    if(result != GLOBUS_SUCCESS)
    {
        goto exit;
    }

    result = globus_handler_chain_push(
        chain, 
        GLOBUS_HANDLER_TYPE_REQUEST_ALL,
        "globus_handler_debug");
    if(result != GLOBUS_SUCCESS)
    {
        goto exit;
    }

    result = globus_handler_chain_push(
        chain, GLOBUS_HANDLER_TYPE_REQUEST_ALL,
        "globus_handler_ws_addressing_client");
    if(result != GLOBUS_SUCCESS)
    {
        goto exit;
    }

    result = globus_handler_chain_push(
        chain,
        GLOBUS_HANDLER_TYPE_RESPONSE_ALL,
        "globus_handler_ws_addressing_client");
    if(result != GLOBUS_SUCCESS)
    {
        goto exit;
    }

    result = globus_handler_chain_push(
        chain, 
        GLOBUS_HANDLER_TYPE_RESPONSE_ALL,
        "globus_handler_debug");
    if(result != GLOBUS_SUCCESS)
    {
        goto exit;
    }

    result = CounterService_client_init(&client_handle, NULL, chain);
    if(result != GLOBUS_SUCCESS)
    {
        goto exit;
    }

    CounterService_client_attr_set(
        client_handle,
        GLOBUS_SOAP_MESSAGE_AUTHENTICATION_METHOD_KEY,
        NULL,
        NULL,
        (void *)GLOBUS_SOAP_MESSAGE_AUTH_SECURE);

    result = CounterPortType_createCounter(
        client_handle,
        argv[1],
        &createCounter,
        &createCounterResponse,
        (CounterPortType_createCounter_fault_t *)&fault_type,
        &fault);
    if(result != GLOBUS_SUCCESS)
    {
        goto exit;
    }

    if(createCounterResponse && 
       createCounterResponse->EndpointReference.ReferenceProperties)
    {
        referenceProperties = 
        createCounterResponse->EndpointReference.ReferenceProperties->any;

        if(referenceProperties.length != 1)
        {
            fprintf(stderr, 
                    "ERROR: Invalid number (%d) of counterKey elements"
                    " in createCounterResponse\n", 
                    referenceProperties.length);
            exit(1);
        }

        printf("COUNTER KEY TYPE: {%s}%s\n", 
               referenceProperties.elements[0].any_info->type->Namespace, 
               referenceProperties.elements[0].any_info->type->local);
               
        if(referenceProperties.elements[0].any_info->type == 
           xsd_string_info.type)
        {
            printf("COUNTER KEY: %s\n", 
                   *(xsd_string *)referenceProperties.elements[0].value);
        }
        else if(referenceProperties.elements[0].any_info->type == 
                globus_xml_buffer_info.type)
        {
            counterKey = (globus_xml_buffer *)
            referenceProperties.elements[0].value;
            if(counterKey)
            {
                printf("COUNTER KEY: %*s\n", 
                       counterKey->length, counterKey->buffer);
            }
        }
    }

    xsd_int_init((void *)&addResponse);

    add_value = 10;
    result = CounterPortType_add_epr(
        client_handle,
        &createCounterResponse->EndpointReference,
        &add_value,
        &addResponse,
        (CounterPortType_add_fault_t *)&fault_type,
        &fault);
    if(result != GLOBUS_SUCCESS)
    {
        goto exit;
    }

    printf("ADD RESPONSE: %d\n", (addResponse ? *addResponse : -1));

    xsd_int_destroy(addResponse);
        
 exit:

    if(createCounterResponse)
    {
        createCounterResponseType_destroy(createCounterResponse);
    }

    CounterService_client_destroy(client_handle);

    if(result != GLOBUS_SUCCESS)
    {
        char *                          errorstr;

        errorstr = globus_error_print_chain(globus_error_get(result));
        fprintf(stderr, "ERROR:\n\n%s\n\n", errorstr);
        globus_module_deactivate_all();
        return 1;
    }

    globus_module_deactivate_all();

    return 0;
}
