/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */


#ifndef GLOBUS_I_XIO_BUFFER_H
#define GLOBUS_I_XIO_BUFFER_H

#include "globus_xio_driver.h"
#include "globus_xio_buffer.h"

#define GLOBUS_XIO_BUFFER_DRIVER_MODULE GlobusXIOMyModule(buffer)

GlobusDebugDeclare(GLOBUS_XIO_BUFFER);

#define GLOBUS_XIO_BUFFER_DEBUG_TRACE          1 
#define GLOBUS_XIO_BUFFER_DEBUG_INTERNAL_TRACE 2
#define GLOBUS_XIO_BUFFER_DEBUG_MARKERS        4
#define GLOBUS_XIO_BUFFER_DEBUG_BUFFERS        8

#define GlobusXIOBufferDebugPrintf(LEVEL, MESSAGE)                           \
    GlobusDebugPrintf(GLOBUS_XIO_BUFFER, LEVEL, MESSAGE)

#define GlobusXIOBufferDebugInternalEnter()                                  \
    GlobusXIOBufferDebugPrintf(                                              \
        GLOBUS_XIO_BUFFER_DEBUG_INTERNAL_TRACE,                              \
        ("[%s] Entering\n", _xio_name))

#define GlobusXIOBufferDebugInternalExit()                                   \
    GlobusXIOBufferDebugPrintf(                                              \
        GLOBUS_XIO_BUFFER_DEBUG_INTERNAL_TRACE,                              \
        ("[%s] Exiting\n", _xio_name))

#define GlobusXIOBufferDebugInternalExitWithError()                          \
    GlobusXIOBufferDebugPrintf(                                              \
        GLOBUS_XIO_BUFFER_DEBUG_INTERNAL_TRACE,                              \
        ("[%s] Internal Exiting with error\n", _xio_name))

#define GlobusXIOBufferDebugEnter()                                          \
    GlobusXIOBufferDebugPrintf(                                              \
        GLOBUS_XIO_BUFFER_DEBUG_TRACE,                                       \
        ("[%s] Entering\n", _xio_name))

#define GlobusXIOBufferDebugExit()                                           \
    GlobusXIOBufferDebugPrintf(                                              \
        GLOBUS_XIO_BUFFER_DEBUG_TRACE,                                       \
        ("[%s] Exiting\n", _xio_name))

#define GlobusXIOBufferDebugExitWithError()                                  \
    GlobusXIOBufferDebugPrintf(                                              \
        GLOBUS_XIO_BUFFER_DEBUG_TRACE,                                       \
        ("[%s] Exiting with error\n", _xio_name))

#define GlobusXIOErrorBufferMarkerNotFound(MARKER)                           \
    globus_error_put(                                                        \
        globus_error_construct_error(                                        \
            GLOBUS_XIO_BUFFER_DRIVER_MODULE,                                 \
            GLOBUS_NULL,                                                     \
            GLOBUS_XIO_BUFFER_ERROR_MARKER_NOT_FOUND,                        \
            __FILE__,                                                        \
            _xio_name,                                                       \
            __LINE__,                                                        \
            "Failed to find buffer marked: %s",                              \
            MARKER))

#define GlobusXIOErrorBufferEmpty()                                          \
    globus_error_put(                                                        \
        globus_error_construct_error(                                        \
            GLOBUS_XIO_BUFFER_DRIVER_MODULE,                                 \
            GLOBUS_NULL,                                                     \
            GLOBUS_XIO_BUFFER_ERROR_BUFFER_EMPTY,                            \
            __FILE__,                                                        \
            _xio_name,                                                       \
            __LINE__,                                                        \
            ""))

#define GlobusXIOErrorBufferFlushFailed(cause)                               \
    globus_error_put(                                                        \
        globus_error_construct_error(                                        \
            GLOBUS_XIO_BUFFER_DRIVER_MODULE,                                 \
            GLOBUS_NULL,                                                     \
            GLOBUS_XIO_BUFFER_ERROR_FLUSH_FAILED,                            \
            __FILE__,                                                        \
            _xio_name,                                                       \
            __LINE__,                                                        \
            "Failed to flush buffered data"))

#define GlobusXIOErrorBufferSetMarkerFailed(marker)                          \
    globus_error_put(                                                        \
        globus_error_construct_error(                                        \
            GLOBUS_XIO_BUFFER_DRIVER_MODULE,                                 \
            GLOBUS_NULL,                                                     \
            GLOBUS_XIO_BUFFER_ERROR_SET_MARKER_FAILED,                       \
            __FILE__,                                                        \
            _xio_name,                                                       \
            __LINE__,                                                        \
            "Failed to set marker: %s", marker))


#define GlobusXIOErrorBufferEOF()                               \
    globus_error_put(                                           \
        globus_error_construct_error(                           \
                             GLOBUS_XIO_BUFFER_DRIVER_MODULE,   \
                             GLOBUS_NULL,                       \
                             GLOBUS_XIO_BUFFER_ERROR_EOF,       \
                             __FILE__,                          \
                             _xio_name,                         \
                             __LINE__,                          \
                             "Buffer EOF"))

#define GlobusXIOBufferErrorOutOfMemory                                 \
    globus_error_put(                                                   \
        globus_error_construct_error(                                   \
            GLOBUS_XIO_BUFFER_DRIVER_MODULE,                            \
            GLOBUS_NULL,                                                \
            GLOBUS_XIO_BUFFER_ERROR_OUT_OF_MEMORY,                      \
            __FILE__,                                                   \
            _xio_name,                                                  \
            __LINE__,                                                   \
            "Out of Memory"))

enum
{
    GLOBUS_L_XIO_BUFFER_READING_MODE,
    GLOBUS_L_XIO_BUFFER_WRITING_MODE
};

enum
{
    GLOBUS_L_XIO_BUFFERING_DISABLED,
    GLOBUS_L_XIO_BUFFERING_ENABLED,
    GLOBUS_L_XIO_BUFFERING_DONE
};

typedef struct globus_l_buffer_write_marker_s
{
    char *                              name;
    int                                 index;
} globus_l_buffer_write_marker_t;

typedef struct globus_l_buffer_read_marker_s
{
    char *                                 name;
    int                                    offset;
    int                                    buffer_index;
    struct globus_l_buffer_read_marker_s * next;
    struct globus_l_buffer_read_marker_s * prev;
} globus_l_buffer_read_marker_t;

typedef struct globus_l_read_buffer_element_s
{
    char *                              buffer;
    int                                 ref_count;
} globus_l_read_buffer_element_t;

typedef struct globus_l_read_buffer_s
{
    globus_l_read_buffer_element_t *    read_element;
    size_t                              length;
    int                                 start_pos;
} globus_l_read_buffer_t;

typedef struct globus_l_buffer_handle_s
{
    globus_xio_iovec_t *                write_buffers;
    globus_l_read_buffer_t *            read_buffers;
    int                                 read_buffers_available;
    int                                 buffer_count;
    size_t                              buffers_length;
    globus_hashtable_t                  buffer_map;
    int                                 traverse_position;
    int                                 traverse_buffer_index;
    globus_xio_iovec_t *                user_iovec;
    int                                 user_iovec_count;
    size_t                              wait_for;
    globus_l_buffer_read_marker_t *     current_read_marker;
    int                                 mode;
    int                                 io_mode;
    globus_xio_driver_handle_t          driver;
    int                                 current_write_buffer;
} globus_l_buffer_handle_t;

#define GloubsIXIOBufferGetBuffer(HANDLE, INDEX) \
    (HANDLE->read_buffers[INDEX].read_element->buffer + \
    HANDLE->read_buffers[INDEX].start_pos)

#define GlobusIXIOBufferGetLength(HANDLE, INDEX) \
    (HANDLE->read_buffers[INDEX].length)

#endif
