#!/usr/bin/env perl

#
# Portions of this file Copyright 1999-2005 University of Chicago
# Portions of this file Copyright 1999-2005 The University of Southern California.
#
# This file or a portion of this file is licensed under the
# terms of the Globus Toolkit Public License, found at
# http://www.globus.org/toolkit/download/license.html.
# If you redistribute this file, with or without
# modifications, you must include this notice in the file.
#


=pod

=head1 Tests for part of MDS functionality.

=cut

use strict;
use POSIX;
use Test;
use Cwd;

my $test_prog = 'mds-anon-gris-test';

my $diff = 'diff';
my @tests;
my @todo;

sub basic_func
{
   my ($errors,$rc) = ("",0);
   
   #$ENV{X509_CERT_DIR} = cwd();
   #$ENV{X509_USER_PROXY} = "testcred.pem";

   $rc = system("grid-info-search -x >$test_prog.log.stdout 2>$test_prog.log.stderr");

    if($rc != 0)
    {
         $errors .= "Test exited with $rc. ";
    }

    if(-r 'core')
    {
        $errors .= "Core file generated. ";
    }

# there should be no output on stderr at all.
    if(-s "$test_prog.log.stderr")
    {
        $errors .="Unexpected output on stderr. ";
    }

# check that we get some subtanstial amount of output (although this test
# doesn't look at it to see if it makes sense.)
# 5000 is an arbitrary size chosen by benc
   if(-s "$test_prog.log.stdout" < 5000)
   {
	$errors .="Query output was unreasonably small. ";
   }

#crosscheck count
#we count the number of dn: lines and compare it against the object count
#reported by grid-info-search
#If the output of multiple objects has got merged incorrectly (for example
#if there are missing blank links), this count may not match correctly.


   my $countdn = 0 + `cat $test_prog.log.stdout | grep "^dn: " | wc -l`;
   
#   print "Counted $countdn dn: lines.\n"; 

   my $respline = `grid-info-search -x | grep "^# numEntries:"`;
  
#   print "Respline is $respline";

   $respline =~ /^# numEntries: ([0-9]*)$/;

   my $countne = $1;

   if ($countne ne $countdn)
   {
      $errors .="Counting objects in two ways didn't match. ";
   }

   if($errors eq "")
   {
      ok('success', 'success');
      
      if( -e "$test_prog.log.stdout" )
      {
	 unlink("$test_prog.log.stdout");
      }
      
      if( -e "$test_prog.log.stderr" )
      {
	 unlink("$test_prog.log.stderr");
      } 
   }
   else
   {
      ok($errors, 'success');
   }

}

sub sig_handler
{
   if( -e "$test_prog.log.stdout" )
   {
      unlink("$test_prog.log.stdout");
   }

   if( -e "$test_prog.log.stderr" )
   {
      unlink("$test_prog.log.stderr");
   }
}

$SIG{'INT'}  = 'sig_handler';
$SIG{'QUIT'} = 'sig_handler';
$SIG{'KILL'} = 'sig_handler';


push(@tests, "basic_func();");

# Now that the tests are defined, set up the Test to deal with them.
plan tests => scalar(@tests), todo => \@todo;

# And run them all.
foreach (@tests)
{
   eval "&$_";
}
