#!/usr/bin/env perl

#
# Portions of this file Copyright 1999-2005 University of Chicago
# Portions of this file Copyright 1999-2005 The University of Southern California.
#
# This file or a portion of this file is licensed under the
# terms of the Globus Toolkit Public License, found at
# http://www.globus.org/toolkit/download/license.html.
# If you redistribute this file, with or without
# modifications, you must include this notice in the file.
#


=pod

=head1 Tests for part of MDS functionality.

This test will run a large number of identical queries on the server
and check that all the results are approximately the same size.

This test will incorrectly fail if, for example, someone mounts a new 
file system whilst the test is going on

=cut

use strict;
use POSIX;
use Test;
use Cwd;

my $test_prog = 'mds-auth-load-giis';

my $diff = 'diff';
my @tests;
my @todo;

# the range we will permit between the largest and smallest result set
# we need to have some tolerance because other stuff may be going on on 
# the system whilst we are performing the queries.
my $sizetolerance=60;

# the number of iterations we will perform
my $iterationcount=300;


sub basic_func
{
   my ($errors,$rc) = ("",0);

   my $loop;

   my $maxsize = -1;
   
   # 1000000 is hopefully sufficiently high to be infinite
   my $minsize = 1000000;

   my $currentsize;

   #$ENV{X509_CERT_DIR} = cwd();
   #$ENV{X509_USER_PROXY} = "testcred.pem";



   LOAD: for($loop=0;$loop<$iterationcount;$loop++)
   {
        $rc = system("grid-info-search -b 'mds-vo-name=site,o=grid' >$test_prog.log.stdout 2>$test_prog.log.stderr");

        if($rc != 0)
        {
             $errors .= "Test exited with $rc. ";
	     last LOAD;
        }

	$rc = system("$diff $test_prog.log.stderr $test_prog.stderr") / 256;
      
        if($rc != 0)
        {
             $errors .= "Test produced unexpected output, see $test_prog.log.stderr";
	     last LOAD;
        }

        $currentsize = -s "$test_prog.log.stdout";

        if($currentsize > $maxsize)
	{
            $maxsize=$currentsize;
	}
	if($currentsize < $minsize)
	{
            $minsize=$currentsize;
	}

    }

   if(($maxsize-$minsize) > $sizetolerance)
   {

      $errors .="Range of result sizes too large. ";

   }
   
   if($errors eq "")
   {
      ok('success', 'success');
      
      if( -e "$test_prog.log.stdout" )
      {
	 unlink("$test_prog.log.stdout");
      }
      
      if( -e "$test_prog.log.stderr" )
      {
	 unlink("$test_prog.log.stderr");
      } 
   }
   else
   {
      ok($errors, 'success');
   }

}

sub sig_handler
{
   if( -e "$test_prog.log.stdout" )
   {
      unlink("$test_prog.log.stdout");
   }

   if( -e "$test_prog.log.stderr" )
   {
      unlink("$test_prog.log.stderr");
   }
}

$SIG{'INT'}  = 'sig_handler';
$SIG{'QUIT'} = 'sig_handler';
$SIG{'KILL'} = 'sig_handler';


push(@tests, "basic_func();");

# Now that the tests are defined, set up the Test to deal with them.
plan tests => scalar(@tests), todo => \@todo;

# And run them all.
foreach (@tests)
{
   eval "&$_";
}
