#!/usr/bin/env perl

#
# Portions of this file Copyright 1999-2005 University of Chicago
# Portions of this file Copyright 1999-2005 The University of Southern California.
#
# This file or a portion of this file is licensed under the
# terms of the Globus Toolkit Public License, found at
# http://www.globus.org/toolkit/download/license.html.
# If you redistribute this file, with or without
# modifications, you must include this notice in the file.
#



require 5.005;

use strict;

use Cwd;
use Test::Harness;
use Getopt::Long;

use vars qw(@tests);

my $globus_location = $ENV{GLOBUS_LOCATION};

#$Test::Harness::verbose = 1;


# The auth tests needs to come first so we 
# can easily remove it when no certs are found

@tests = qw(mds-auth-load-giis.pl
            mds-anon-gris-test.pl
            mds-anon-giis-test.pl
            mds-errchk-bad-base.pl
            mds-anon-load-gris.pl
            mds-infoprov-chk.pl
            mds-server-stop.pl
            );

push(@INC, $ENV{GLOBUS_LOCATION} . "/lib/perl");

#
# run check for commands test seperately:
#

system("./mds-check-for-commands.pl");

my $runserver;

GetOptions( 'runserver' => \$runserver);

if(defined($runserver))
{
    start_mds();
}

if(0 != system("grid-proxy-info -exists -hours 2 >/dev/null 2>&1") / 256)
{
    print "Security proxy required to run the tests.\n";
    stop_mds();
    exit 1;
}



eval runtests(@tests);

$@ && print "$@";

if(defined($runserver))
{
    stop_mds();
}

exit 0;


sub start_mds()
{
    my $subject;
    my $hostname;
    my $fqdn;
    my $working_dir = cwd();
    my $server_subject;

    $ENV{GRIDMAP}="$working_dir/gridmap";
    $fqdn = `globus-hostname`;
    chomp($fqdn);
    $hostname = $fqdn;
    $hostname =~ s/\..*//;

    if( -f "ldapcerts/${hostname}cert.pem" && -r _ &&
        -f "ldapcerts/${hostname}key.pem" && -r _  &&
        ($server_subject = 
         `grid-cert-info -subject -file ldapcerts/${hostname}cert.pem`) 
        =~ /$fqdn/)
    {
        system("chmod go-rw ldapcerts/${hostname}key.pem");
        $ENV{X509_USER_CERT} = "$working_dir/ldapcerts/${hostname}cert.pem";
        $ENV{X509_USER_KEY} = "$working_dir/ldapcerts/${hostname}key.pem";
        $ENV{X509_RUN_AS_SERVER} = "1";
        $ENV{X509_CERT_DIR} = $working_dir;
    }
    else
    {
        # Fragile...
        shift(@tests);
    }

    if( 0 != system("./reconfig-port.sh >/dev/null 2>&1") / 256)
    {
        print "Unable to reconfigure MDS server port\n";
        exit 1;
    }
    
    if( 0 != system("globus-mds start >/dev/null 2>&1") / 256)
    {
        print "Unable to start MDS server\n";
        exit 1;
    }

    delete($ENV{X509_CERT_DIR});
    delete($ENV{X509_USER_CERT});
    delete($ENV{X509_USER_KEY});
    delete($ENV{X509_RUN_AS_SERVER});

    if(0 != system("grid-proxy-info -exists -hours 2 >/dev/null 2>&1") / 256)
    {
        $ENV{X509_CERT_DIR} = $working_dir;
        $ENV{X509_USER_PROXY} = "testcred.pem";
        system('chmod go-rw testcred.pem');
    }
    
    $subject = `grid-proxy-info -identity`;
    chomp($subject);
    
    
    if( 0 != system("grid-mapfile-add-entry -dn \"$subject\" -ln `whoami` -f $ENV{GRIDMAP} >/dev/null 2>&1") / 256)
    {

        print "Unable to create gridmap file\n";
        stop_mds();
        exit 1;
    }

    sleep(60);
}

sub stop_mds()
{
    if( 0 != system("globus-mds stop >/dev/null 2>&1") / 256)
    {
        print "Unable to stop MDS server\n";
        exit 1;
    }
}
