/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */

/*
 * Some data structures (lrclist and rlilist) require
 * locking scheme that supports multiple readers or 1 writer.  Following
 * functions support this.
 */

#include <globus_io.h>
#include <syslog.h>
#include "globus_rls_client.h"
#include "misc.h"
#include "lock.h"

extern int	loglevel;

int
lock_init(LOCK *lock)

{
  int	r;

  if ((r = globus_mutex_init(&lock->mtx, GLOBUS_NULL)) != 0) {
    logit(LOG_WARNING, "lock_init(%X,%d): globus_mutex_init: %d", lock, r);
    return GLOBUS_RLS_GLOBUSERR;
  }
  if ((r = globus_cond_init(&lock->cond, GLOBUS_NULL)) != 0) {
    logit(LOG_WARNING, "lock_init(%X,%d): globus_cond_init: %d", lock, r);
    return GLOBUS_RLS_GLOBUSERR;
  }
  lock->readers = 0;
  lock->writer = 0;
  return GLOBUS_RLS_SUCCESS;
}

void
lock_destroy(LOCK *lock)

{
  globus_cond_destroy(&lock->cond);
  globus_mutex_destroy(&lock->mtx);
}

void
lock_get(LOCK *lock, LOCKTYPE type)

{
  if (loglevel > 2)
    logit(LOG_DEBUG, "lock_get: %X %s", lock,
	  type == readlock ? "readlock" : "writelock");
  globus_mutex_lock(&lock->mtx);
  if (type == readlock) {
    while (lock->writer)
      globus_cond_wait(&lock->cond, &lock->mtx);
    lock->readers++;
    if (loglevel > 2)
      logit(LOG_DEBUG, "lock_get(%X): %d readers", lock, lock->readers);
  } else {
    while (lock->readers || lock->writer)
      globus_cond_wait(&lock->cond, &lock->mtx);
    lock->writer++;
    if (loglevel > 2)
      logit(LOG_DEBUG, "lock_get(%X): writelock acquired", lock);
  }
  globus_mutex_unlock(&lock->mtx);
}

void
lock_release(LOCK *lock, LOCKTYPE type)

{
  globus_mutex_lock(&lock->mtx);
  if (type == readlock) {
    if (--lock->readers == 0)
      globus_cond_broadcast(&lock->cond);
    if (loglevel > 2)
      logit(LOG_DEBUG, "lock_release(%X): %d readers", lock, lock->readers);
  } else {
    lock->writer = 0;
    globus_cond_broadcast(&lock->cond);
    if (loglevel > 2)
      logit(LOG_DEBUG, "lock_release(%X): writelock released", lock);
  }
  globus_mutex_unlock(&lock->mtx);
}
