/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.cas.impl.client;

import org.globus.cas.CASPortType;

import org.globus.cas.impl.CasConstants;

import org.globus.wsrf.utils.FaultHelper;

import org.globus.cas.types.ManageUserGroups;
import org.globus.cas.types.ManageObjectGroups;
import org.globus.cas.types.ManageServiceActionGroups;

import org.globus.cas.faults.CasFault;
import org.globus.cas.faults.NoPermissionFault;

/**
 * Command line client to add to a user, object or serviceAction group
 */
public class AddGroupEntry {
    
    static String msg = 
        " Usage: cas-group-add-entry [<options>] <groupType> <groupName> \n"
        + "      <groupMemberDetails> \n "
        + " Options are  -debug Runs with debug trace \n"
        + "              -c <instanceURl> Specify URL for CAS server\n"
        + "              -help Prints this message\n"
        + "              -v Prints version number\n"
        + "              -s Identity of CAS server. If not specified, will \n"
        + "                 default to host authorization.\n"
        + "              -m security mechanism. 'msg' for secure message\n"
        + "                 or 'conv' for secure conversation or 'trans' for\n"
        + "                 transport security. If unspecified, if instance\n"
        + "                 url is 'https', then trasport security is used \n"
        + "                 else mechanism defaults to secure message\n"
        + "              -p protection type, 'sig' signature and 'enc' \n"
        + "                 encryption, defaults to signature\n"
        + " <groupType> : Type of group to which member needs to be added.\n "
        + "               Should be \"user\" or \"object\" or \"serviceAction"
        + "\"\n" 
        + " <groupName> : Name of the group to which member needs to be added"
        + " \n <groupmemberDetails> : \n"
        + "    If \"user\", <nickname> \n"
        + "        <nickname> : Nickname of the user to be added \n" 
        + "    If \"object\", <objectSpecDesc> <objectSpec>\n"
        + "        <objectSpecDesc> : Type of object. Should be \"object\" \n"
        + "                           or \"user\" or \"userGroup\" or \n"
        + "                           \"serviceType\" or \"namespace\" or\n"
        + "                           \"trustAnchor\" \n"
        + "        <objectSpec> : Data to uniquely identify the CAS Object. \n"
        + "                       <objectNamespace> <objectName>, where \n"
        + "                       <objectNamespace> is Namespace the object\n"
        + "                       belongs to and <objectName> is name of the\n"
        + "                       object (or) nickname of the user (or) name\n"
        + "                       of user group (or) service type name (or)\n"
        + "                       nickname of the namespace or) nickname of\n"
        + "                       trust anchor respectively\n"
        + "    If \"serviceAction\", <serviceTypeName> <actionName> \n"
        + "        <serviceTypeName> : Service type that needs to be added\n"
        + "        <actionName> : Action name ";

    static boolean debug = false;

    public static void main(String args[]) throws Exception {

        if (args.length<1) {    
            System.out.println(msg);
            System.exit(0);
        }

        CasClientSetup clientSetup = new CasClientSetup(args, msg);
        String instanceURL =  clientSetup.getInstanceURL();     
        String serverIdentity = clientSetup.getServerIdentity();
        int startIndex = clientSetup.argsStartIndex;
        debug = clientSetup.debug;
        printMessage("CAS client to add a member to a group");

        if (args.length < startIndex + 2) {
            System.err.println("Not enough parameters");
            System.err.println(msg);
            System.exit(-1);
        }

        printMessage("Argument starting index is " + startIndex);
        String casObject = args[startIndex];    

        try {
            if (casObject.equals(CasConstants.USER_SPEC)) {
                CASPortType casPort = 
                    clientSetup.getCASPort(instanceURL, 3, serverIdentity);
                printMessage("User group " + args[startIndex+1] + " " 
                             + args[startIndex+2]);
                ManageUserGroups mngGp = new ManageUserGroups();
                mngGp.setRequestType(CasConstants.ADD_OPERATION);
                mngGp.setGroupName(args[startIndex+1]); 
                mngGp.setUserNickname(args[startIndex+2]);
                casPort.manageUserGroups(mngGp);
            } else if (casObject.equals(CasConstants.OBJECT_SPEC)) {
                CASPortType casPort = 
                    clientSetup.getCASPort(instanceURL, 4, serverIdentity);
                // swapped for desc and spec
                printMessage("Object group " + args[startIndex+1] + " " 
                             + args[startIndex+3] + " " + args[startIndex+2]);
                String objectSpec = args[startIndex+3];
                // If the member being added is object, then construct
                // <objectNamespace>|<objectName>
                if (args[startIndex+2].equals(CasConstants.OBJECT_SPEC)) {
                    printMessage("Group type is object group");
                    clientSetup.checkParameters(5);
                    objectSpec = args[startIndex+3] 
                        + CasConstants.OBJECTSPEC_DELIMITER 
                        + args[startIndex+4];
                }
                ManageObjectGroups mngGp = new ManageObjectGroups();
                mngGp.setRequestType(CasConstants.ADD_OPERATION);
                mngGp.setGroupName(args[startIndex+1]);
                mngGp.setObjectSpec(objectSpec);
                mngGp.setObjectSpecDesc(args[startIndex+2]);
                casPort.manageObjectGroups(mngGp);
            } else if (casObject.equals(CasConstants.SERVICEACTION_SPEC)) {
                CASPortType casPort = 
                    clientSetup.getCASPort(instanceURL, 4, serverIdentity);
                printMessage("Service Action group " + args[startIndex+1] 
                             + " " + args[startIndex+2] + " " 
                             + args[startIndex+3]);
                ManageServiceActionGroups mngGp = 
                    new ManageServiceActionGroups();
                mngGp.setRequestType(CasConstants.ADD_OPERATION);
                mngGp.setGroupName(args[startIndex+1]);
                mngGp.setServiceTypeName(args[startIndex+2]);
                mngGp.setActionName(args[startIndex+3]);
                casPort.manageServiceActionGroups(mngGp);
            } else {
                System.err.println("Erroneous object type " + casObject);
                System.err.println(msg);
                System.exit(-1);
            }
        } catch (NoPermissionFault noPermFault) {
            System.err.println(noPermFault.getDescription());
            System.exit(-1);
        } catch (CasFault casFault) {
            if (!debug) {
                String desc = 
                    (new FaultHelper(casFault)).getDescriptionAsString();
                System.err.println(desc);
            } else {
                System.err.println((new FaultHelper(casFault))
                                   .getStackTrace());
            }
            System.exit(-1);
        } catch (Exception exp) {
            if (debug) {
                System.err.println(exp.toString());
            } else {
                System.err.println(CasClientSetup.getErrorMsg(exp));
            }
            System.exit(-1);
        }
        System.out.println("Completed successfully");
    }
            
    private static void printMessage(String msg) {
        if (debug) {
            System.out.println(msg);
        }
    }
}
