/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.cas.impl.client;

import org.globus.cas.CASPortType;

import org.globus.cas.impl.CasConstants;

import org.globus.cas.types.CreateGroup;
import org.globus.cas.types.DeleteGroup;

import org.globus.cas.faults.CasFault;
import org.globus.cas.faults.NoPermissionFault;

import org.globus.wsrf.utils.FaultHelper;
/**
 * Command line client to create or delete from a user, object or 
 * serviceAction group
 */
public class GroupHandler {
    
    static String msg = 
        " Usage : cas-group-admin [<options>] <groupType> <operationType> \n"
        + "       [<userGpName>] <groupName>\n"
        + " Options are  -debug Runs with debug trace \n"
        + "              -c <instanceURl> Specify URL for CAS server\n"
        + "              -help Prints this message\n"
        + "              -v Prints version number\n"
        + "              -s Identity of CAS server. If not specified, will \n"
        + "                 defualt to host authorization.\n"
        + "              -m security mechanism. 'msg' for secure message\n"
        + "                 or 'conv' for secure conversation or 'trans' for\n"
        + "                 transport security. If unspecified, if instance\n"
        + "                 url is 'https', then trasport security is used \n"
        + "                 else mechanism defaults to secure message\n"
        + "              -p protection type, 'sig' signature and 'enc' \n"
        + "                 encryption, defaults to signature\n"
        + " <groupType> : Type of group this operation is on. Should be \n"
        + "               \"user\" or \"object\" or \"serviceAction\"\n"
        + " <operationType> : Type of operation on the group. Should be \n"
        + "                   \"create\" or \"delete\"\n"
        + " <userGpName> : Needed only when the <operationType> is \"create\""
        + "\n              Name of the user group to which to give all \n"
        + "                rights on the created group.\n"
        + " <groupName> : Name of the group that needs to be created or \n"
        + "               deleted.\n";

    static boolean debug = false;

    public static void main(String args[]) throws Exception {
        
        if (args.length<1) {
            System.out.println(msg);
            System.exit(0);
        }

        CasClientSetup clientSetup = new CasClientSetup(args, msg);
        String instanceURL =  clientSetup.getInstanceURL();     
        String serverIdentity = clientSetup.getServerIdentity();
        int startIndex = clientSetup.argsStartIndex;
        debug = clientSetup.debug;

        printMessage("CAS client to create or delete a group.");

        if (args.length < startIndex + 2) {
            System.err.println("Not enough parameters");
            System.err.println(msg);
            System.exit(-1);
        }

        String groupType = args[startIndex];
        String operation = args[startIndex+1];

        if ((!groupType.equals(CasConstants.USER_SPEC)) &&
            (!groupType.equals(CasConstants.OBJECT_SPEC)) && 
            (!groupType.equals(CasConstants.SERVICEACTION_SPEC))) {
            System.err.println("Erraneous groupType type " + groupType);
            System.err.println(msg);
            System.exit(-1);
        }
        
        if ((!operation.equals(CasConstants.CREATE_OPERATION)) &&
            (!operation.equals(CasConstants.DELETE_OPERATION))) {
            System.err.println("Erraneous operation type " + operation);
            System.err.println(msg);
            System.exit(-1);
        }
        
        try {
            if (operation.equals(CasConstants.CREATE_OPERATION)) {
                CASPortType casPort = 
                    clientSetup.getCASPort(instanceURL, 4, serverIdentity);
                printMessage("Create operation " + groupType + " " +
                             args[startIndex+2] + " " + args[startIndex+3]);
                CreateGroup createGroup = new CreateGroup();
                createGroup.setGroupType(groupType);
                createGroup.setUserGpName(args[startIndex+2]);
                createGroup.setGroupName(args[startIndex+3]);
                casPort.createGroup(createGroup);
            } else if (operation.equals(CasConstants.DELETE_OPERATION)) {
                CASPortType casPort = 
                    clientSetup.getCASPort(instanceURL, 3, serverIdentity);
                printMessage("Deletion operation " + groupType + " " 
                             + args[startIndex+2]);
                DeleteGroup deleteGroup = new DeleteGroup();
                deleteGroup.setGroupType(groupType);
                deleteGroup.setGroupName(args[startIndex+2]);
                casPort.deleteGroup(deleteGroup);
            }
        } catch (NoPermissionFault noPermFault) {
            System.err.println(noPermFault.getDescription());
            System.exit(-1);
        } catch (CasFault casFault) {
            if (!debug) {
                String desc = 
                    (new FaultHelper(casFault)).getDescriptionAsString();
                System.err.println(desc);
            } else {
                System.err.println((new FaultHelper(casFault))
                                   .getStackTrace());
            }
            System.exit(-1);
        } catch (Exception exp) {
            if (debug) {
                exp.printStackTrace();
                System.err.println(exp.toString());
            } else {
                System.err.println(CasClientSetup.getErrorMsg(exp));
            }
            System.exit(-1);
        }
        System.out.println("Completed successfully");
    }

    private static void printMessage(String msg) {
        if (debug) {
            System.out.println(msg);
        }
    }
}
