/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.cas.impl.client;

import org.globus.cas.CASPortType;

import org.globus.cas.types.GetGroupMembers;
import org.globus.cas.types.ArrayOfString;

import org.globus.cas.impl.CasConstants;

import org.globus.cas.faults.CasFault;
import org.globus.cas.faults.NoPermissionFault;

import org.globus.wsrf.utils.FaultHelper;
/**
 * Command line client to list members of a user, object or serviceAction group
 */
public class ListGroupEntries {
    
    static String msg = 
        " Usage: cas-group-list-entries [<options>] <groupType> <groupName> \n"
        + " Options are  -debug Runs with debug trace \n"
        + "              -c <instanceURl> Specify URL for CAS server\n"
        + "              -help Prints this message\n"
        + "              -v Prints version number\n"
        + "              -s Identity of CAS server. If not specified, will \n"
        + "                 defualt to host authorization.\n"
        + "              -m security mechanism. 'msg' for secure message\n"
        + "                 or 'conv' for secure conversation or 'trans' for\n"
        + "                 transport security. If unspecified, if instance\n"
        + "                 url is 'https', then trasport security is used \n"
        + "                 else mechanism defaults to secure message\n"
        + "              -p protection type, 'sig' signature and 'enc' \n"
        + "                 encryption, defaults to signature\n"
        + " <groupType> : Type of group which needs to be listed.\n "
        + "               Should be \"user\" or \"object\" or \"serviceAction"
        + "\"\n" 
        + " <groupName> : Name of the group which needs to be listed";

    static boolean debug = false;

    public static void main(String args[]) throws Exception {

        if (args.length<1) {    
            System.out.println(msg);
            System.exit(0);
        }

        CasClientSetup clientSetup = new CasClientSetup(args, msg);
        String instanceURL =  clientSetup.getInstanceURL();     
        String serverIdentity = clientSetup.getServerIdentity();
        int startIndex = clientSetup.argsStartIndex;
        debug = clientSetup.debug;

        printMessage("CAS client to list group members");

        if (args.length < startIndex + 1) {
            System.err.println("Not enough parameters");
            System.err.println(msg);
            System.exit(-1);
        }
        
        String groupType = args[startIndex].trim();     
        
        if ((groupType.equals(CasConstants.USER_SPEC)) ||
            (groupType.equals(CasConstants.OBJECT_SPEC)) ||
            (groupType.equals(CasConstants.SERVICEACTION_SPEC))) {
            printMessage("group " + groupType + " " + args[startIndex+1]);
            CASPortType casPort = 
                clientSetup.getCASPort(instanceURL, 2, serverIdentity);
            String[] gpMembers =null;
            try {
                GetGroupMembers gpMem = new GetGroupMembers();
                gpMem.setNameOfGroup(args[startIndex+1]);
                gpMem.setTypeOfGroup(groupType);
                ArrayOfString groupmemArray = casPort.getGroupMembers(gpMem);
                if (groupmemArray != null) {
                    gpMembers = groupmemArray.getStrings();
                }
            } catch (NoPermissionFault noPermFault) {
                System.err.println(noPermFault.getDescription());
                System.exit(-1);
            } catch (CasFault casFault) {
                if (!debug) {
                    String desc = 
                        (new FaultHelper(casFault)).getDescriptionAsString();
                    System.err.println(desc);
                } else {
                    System.err.println((new FaultHelper(casFault))
                                       .getStackTrace());
                }
                System.exit(-1);
            } catch (Exception exp) {
                if (debug) {
                    System.err.println(exp.toString());
                } else {
                    System.err.println(CasClientSetup.getErrorMsg(exp));
                }
                System.exit(-1);
            }
            
            if (gpMembers == null) {
                System.out.println(groupType.trim() + " group " 
                                   + args[startIndex+1] + " does not have any"
                                   + " members");
            } else {
                System.out.println("The group members are ");
                for (int i=0; i<gpMembers.length; i++) {
                    System.out.println(gpMembers[i]);
                }
            }
        } else {
            System.err.println("Erroneous type " + groupType);
            System.err.println(msg);
            System.exit(-1);
        }
        
        System.out.println("Completed successfully");
    }
    
    private static void printMessage(String msg) {
        if (debug) {
            System.out.println(msg);
        }
    }
}
