/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.cas.impl.client;

import org.globus.cas.CASPortType;

import org.globus.cas.types.VoidType;

import org.globus.cas.faults.CasFault;

import org.globus.wsrf.utils.FaultHelper;
/**
 * Command line client that returns the CAS nick name of the user.
 */
public class Whoami {

    static String msg = 
        " Usage: cas-whomai [<options>] \n"
        + " Options are  -debug Runs with debug trace \n"
        + "              -c <instanceURl> Specify URL for CAS server\n"
        + "              -help Prints this message\n"
        + "              -v Prints version number\n"
        + "              -s Identity of CAS server. If not specified, will \n"
        + "                 defualt to host authorization.\n"
        + "              -m security mechanism. 'msg' for secure message\n"
        + "                 or 'conv' for secure conversation or 'trans' for\n"
        + "                 transport security. If unspecified, if instance\n"
        + "                 url is 'https', then trasport security is used \n"
        + "                 else mechanism defaults to secure message\n"
        + "              -p protection type, 'sig' signature and 'enc' \n"
        + "                 encryption, defaults to signature\n";

    static boolean debug = false;
    
    public static void main(String args[]) throws Exception {
        
        if ((args.length == 1) && (args[0].equals("-help"))) {  
            System.out.println(msg);
            System.exit(0);
        }

        CasClientSetup clientSetup = new CasClientSetup(args, msg);
        String instanceURL =  clientSetup.getInstanceURL();
        String serverIdentity = clientSetup.getServerIdentity();
        debug = clientSetup.debug;      

        if (debug) {
            System.out.println("CAS client to return the CAS nickname of "
                               + " client.");
        }

        CASPortType casPort = 
            clientSetup.getCASPort(instanceURL, 0, serverIdentity);

        String userName = null;
        try {
            userName = casPort.whoami(new VoidType());
        } catch (CasFault casFault) {
            if (!debug) {
                String desc = 
                    (new FaultHelper(casFault)).getDescriptionAsString();
                System.err.println(desc);
            } else {
                System.err.println((new FaultHelper(casFault))
                                   .getStackTrace());
            }
            System.exit(-1);
        } catch (Exception exp) {
            if (debug) {
                System.err.println(exp.toString());
            } else {
                System.err.println(CasClientSetup.getErrorMsg(exp));
            }
            System.exit(-1);
        }

        if (userName == null) {
            System.out.println("This client does not have a nickname on this"
                               + " cas server ");
        } else {
            System.out.println("CAS nickname is " + userName);
        }

        System.out.println("Completed successfully");
    }
}
