/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.cas.impl;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.globus.cas.types.PolicyData;

import org.globus.cas.impl.databaseAccess.CasDBException;
import org.globus.cas.impl.databaseAccess.ObjectDataHandler;
import org.globus.cas.impl.databaseAccess.ServiceTypeActionHandler;

/**
 * Constants used by CAS  package
 */
public class CasConstants {

    static Log logger = LogFactory.getLog(CasConstants.class.getName() );

    final public static String SERVICE_NAME = "CASService";
    // Version number
    final public static String versionNumber = "1.0";

    // Cas OID
    final public static String OID = "1.3.6.1.4.1.3536.1.1.1.9";

    final public static String CAS_PROXY_TAG = "cas";

    final public static String CAS_NS = "http://www.globus.org/07/2004/cas";

    // Constants for SAML
    // FIXME: Dummy namespace URI
    final public static String RESOURCE_WILDCARD = "uri:samlResourceWildcard";
    final public static String ACTION_WILDCARD = "uri:samlActionWildcard";
    final public static String ACTION_NS_WILDCARD = "uri:samlActionNSWildcard";
    final public static String SAML_DECISION = "Permit";
    final public static String X509_FORMAT = "#X509SubjectName";
    final public static String X509_CONFIRMATION_METHOD = 
        "urn:oasis:names:tc:SAML:1.0:am:X509-PKI";

    // Comparison algorithms
    final public static String EXACT_ALGNAME = "ExactComparison";
    final public static String WILD_ALGNAME = "WildCardComparison";
    final public static String EXACT_ALG_CLASSNAME
        = "org.globus.cas.impl.databaseAccess.ExactComparison";
    final public static String WILD_ALG_CLASSNAME 
        = "org.globus.cas.impl.databaseAccess.WildCardComparison";
    
    // Operations
    final public static String ADD_OPERATION = "add";
    final public static String REMOVE_OPERATION = "remove";
    final public static String CREATE_OPERATION = "create";
    final public static String DELETE_OPERATION = "delete";
    
    // Object Specification
    final public static String OBJECT_SPEC = "object";
    final public static String OBJECTGP_SPEC = "objectGroup";
    final public static String SERVICETYPE_SPEC = "serviceType";
    final public static String NAMESPACE_SPEC = "namespace";
    // User Spec and Object Spec
    final public static String USER_SPEC = "user";
    final public static String USERGP_SPEC = "userGroup";
    // Action Spec
    final public static String SERVICEACTION_SPEC = "serviceAction";
    final public static String SERVICEACTIONGP_SPEC = "serviceActionGroup";

    final public static String TRUSTANCHOR_SPEC = "trustAnchor";
    final public static String POLICY_SPEC = "policy";
    // UserGroupData
    final public static String USER_COMMUNITY = "community";

    // ServiceType 
    final public static String SERVICETYPE_CAS = "cas";

    // service action delimiter serviceType/action
    final public static String SERVICEACTION_DELIMITER = "/";
    
    // object specification delimiter objectNamesapce/objectName
    final public static String OBJECTSPEC_DELIMITER = "|";

    // Action
    // Operate on everything
    final public static String ACTION_SU = "superuser";
    // add a trust anchor
    final public static String ACTION_ENROLL_ANCHOR = "enroll_trustAnchor";
    // add a user
    final public static String ACTION_ENROLL_USER = "enroll_user";
    // add a namespace
    final public static String ACTION_ENROLL_NAMESPACE = "enroll_namespace";
    // add an object
    final public static String ACTION_ENROLL_OBJECT = "enroll_object";
    // add a servicetype
    final public static String ACTION_CREATE_SERVICE_TYPE = 
        "create_service_type";

    // create a serviceTypeAction group entry
    final public static String ACTION_CREATE_SERVICEACTION_GROUP = 
        "create_serviceAction_group";
    // create a user group entry
    final public static String ACTION_CREATE_USER_GROUP = "create_user_group";
    // create an object group entry
    final public static String ACTION_CREATE_OBJECT_GROUP = 
        "create_object_group";
    // destroy a group entry
    final public static String ACTION_DELETE_GROUP = "destroy_group";
    // add a group entry
    final public static String ACTION_ADD_GROUP_ENTRY = "add_group_entry";
    // remove a group entry
    final public static String ACTION_REMOVE_GROUP_ENTRY = 
        "remove_group_entry";
    // remove an CASObject
    final public static String ACTION_UNENROLL = "unenroll";
    // Query on database
    final public static String ACTION_QUERY = "query";
    // grant all permissions on one object
    final public static String ACTION_GRANTALL = "grantAll";
    // grant permissions 
    final public static String ACTION_GRANT = "grant";
    // revoke permissions 
    final public static String ACTION_REVOKE = "revoke";

    // Namespace
    final public static String NAMESPACE_SELF = "casNamespace";

    // Object
    final public static String OBJECT_SELF = "casServer";

    public static PolicyData unenrollPolicy = new PolicyData();
    public static PolicyData trustEnrollPolicy = new PolicyData();    
    public static PolicyData userEnrollPolicy = new PolicyData();    
    public static PolicyData namespaceEnrollPolicy = new PolicyData();
    public static PolicyData objectEnrollPolicy = new PolicyData();
    public static PolicyData serviceTypeCreationPolicy = new PolicyData();
    public static PolicyData objectGpCreationPolicy = new PolicyData();
    public static PolicyData userGpCreationPolicy = new PolicyData();
    public static PolicyData serviceActionGpCreationPolicy = new PolicyData();
    public static PolicyData gpDeletionPolicy = new PolicyData();
    public static PolicyData gpAddEntryPolicy = new PolicyData();
    public static PolicyData gpDeleteEntryPolicy = new PolicyData();
    public static PolicyData grantAllPolicy = new PolicyData();
    public static PolicyData grantPolicy = new PolicyData();
    public static PolicyData revokePolicy = new PolicyData();
    public static PolicyData queryPolicy = new PolicyData();

    public static void initializePolicies() throws CasDBException {

        // casServer/NAMESPACE_SELF
        String selfObjectId = 
            Integer.toString(ObjectDataHandler.getObjectId(OBJECT_SELF, 
                                                           NAMESPACE_SELF));

        // cas/unenroll
        String unenrollId = 
            Integer.toString(ServiceTypeActionHandler.getServiceActionId(
                                                      SERVICETYPE_CAS, 
                                                      ACTION_UNENROLL));

        unenrollPolicy.setActionSpec(unenrollId);
        unenrollPolicy.setActionSpecDesc(SERVICEACTION_SPEC);

        // cas/enroll_trustAnchor       
        String trustEnrollId = 
            Integer.toString(ServiceTypeActionHandler.getServiceActionId(
                                                      SERVICETYPE_CAS, 
                                                      ACTION_ENROLL_ANCHOR));

        trustEnrollPolicy.setObjectSpec(selfObjectId);
        trustEnrollPolicy.setObjectSpecDesc(OBJECT_SPEC);
        trustEnrollPolicy.setActionSpec(trustEnrollId);
        trustEnrollPolicy.setActionSpecDesc(SERVICEACTION_SPEC);

        // cas/enroll_user
        String userEnrollId = 
            Integer.toString(ServiceTypeActionHandler.getServiceActionId(
                                                      SERVICETYPE_CAS, 
                                                      ACTION_ENROLL_USER));

        userEnrollPolicy.setObjectSpec(selfObjectId);
        userEnrollPolicy.setObjectSpecDesc(OBJECT_SPEC);
        userEnrollPolicy.setActionSpec(userEnrollId);
        userEnrollPolicy.setActionSpecDesc(SERVICEACTION_SPEC);

        // cas/enroll_namespace 
        String namespaceEnrollId = 
            Integer.toString(
                     ServiceTypeActionHandler.getServiceActionId(
                                              SERVICETYPE_CAS, 
                                              ACTION_ENROLL_NAMESPACE));

        namespaceEnrollPolicy.setObjectSpec(selfObjectId);
        namespaceEnrollPolicy.setObjectSpecDesc(OBJECT_SPEC);
        namespaceEnrollPolicy.setActionSpec(namespaceEnrollId);
        namespaceEnrollPolicy.setActionSpecDesc(SERVICEACTION_SPEC);

        // cas/enroll_object
        String objectEnrollId = 
            Integer.toString(
                        ServiceTypeActionHandler.getServiceActionId(
                                                 SERVICETYPE_CAS, 
                                                 ACTION_ENROLL_OBJECT));

        objectEnrollPolicy.setObjectSpec(selfObjectId);
        objectEnrollPolicy.setObjectSpecDesc(OBJECT_SPEC);
        objectEnrollPolicy.setActionSpec(objectEnrollId);
        objectEnrollPolicy.setActionSpecDesc(SERVICEACTION_SPEC);

        // cas/enroll_serviceType
        String serviceTypeCreationId = 
                Integer.toString(
                        ServiceTypeActionHandler.getServiceActionId(
                                                 SERVICETYPE_CAS, 
                                                 ACTION_CREATE_SERVICE_TYPE));

        serviceTypeCreationPolicy.setObjectSpec(selfObjectId);
        serviceTypeCreationPolicy.setObjectSpecDesc(OBJECT_SPEC);
        serviceTypeCreationPolicy.setActionSpec(serviceTypeCreationId);
        serviceTypeCreationPolicy.setActionSpecDesc(SERVICEACTION_SPEC);

        // cas/create_object_group
        String objectGpCreationId = 
            Integer.toString(
                    ServiceTypeActionHandler.getServiceActionId(
                                             SERVICETYPE_CAS, 
                                             ACTION_CREATE_OBJECT_GROUP));

        objectGpCreationPolicy.setObjectSpec(selfObjectId);
        objectGpCreationPolicy.setObjectSpecDesc(OBJECT_SPEC);
        objectGpCreationPolicy.setActionSpec(objectGpCreationId);
        objectGpCreationPolicy.setActionSpecDesc(SERVICEACTION_SPEC);

        // cas/create_user_group
        String userGpCreationId = 
                Integer.toString(
                        ServiceTypeActionHandler.getServiceActionId(
                                                 SERVICETYPE_CAS, 
                                                 ACTION_CREATE_USER_GROUP));

        userGpCreationPolicy.setObjectSpec(selfObjectId);
        userGpCreationPolicy.setObjectSpecDesc(OBJECT_SPEC);
        userGpCreationPolicy.setActionSpec(userGpCreationId);
        userGpCreationPolicy.setActionSpecDesc(SERVICEACTION_SPEC);

        // cas/create_serviceAction_group
        String serviceActionGpCreationId = 
            Integer.toString(
                    ServiceTypeActionHandler.getServiceActionId(
                    SERVICETYPE_CAS, 
                    ACTION_CREATE_SERVICEACTION_GROUP));

        serviceActionGpCreationPolicy.setObjectSpec(selfObjectId);
        serviceActionGpCreationPolicy.setObjectSpecDesc(OBJECT_SPEC);
        serviceActionGpCreationPolicy.setActionSpec(serviceActionGpCreationId);
        serviceActionGpCreationPolicy.setActionSpecDesc(SERVICEACTION_SPEC);

        // cas/delete_group
        String gpDeletionId = 
            Integer.toString(ServiceTypeActionHandler.getServiceActionId(
                                                      SERVICETYPE_CAS, 
                                                      ACTION_DELETE_GROUP));

        gpDeletionPolicy.setActionSpec(gpDeletionId);
        gpDeletionPolicy.setActionSpecDesc(SERVICEACTION_SPEC);

        // cas/add_group_entry
        String gpAddEntry = 
            Integer.toString(
                    ServiceTypeActionHandler.getServiceActionId(
                                             SERVICETYPE_CAS, 
                                             ACTION_ADD_GROUP_ENTRY));

        gpAddEntryPolicy.setActionSpec(gpAddEntry);
        gpAddEntryPolicy.setActionSpecDesc(SERVICEACTION_SPEC);

        // cas/delete_group_entry
        String gpDeleteEntry = 
            Integer.toString(
                        ServiceTypeActionHandler.getServiceActionId(
                                                 SERVICETYPE_CAS, 
                                                 ACTION_REMOVE_GROUP_ENTRY));

        gpDeleteEntryPolicy.setActionSpec(gpDeleteEntry);
        gpDeleteEntryPolicy.setActionSpecDesc(SERVICEACTION_SPEC);

        // cas/grantAll
        String grantAllId = 
            Integer.toString(
                             ServiceTypeActionHandler.getServiceActionId(
                                                      SERVICETYPE_CAS, 
                                                      ACTION_GRANTALL));

        grantAllPolicy.setActionSpec(grantAllId);
        grantAllPolicy.setActionSpecDesc(SERVICEACTION_SPEC);

        // cas/grant
        String grantId = 
            Integer.toString(
                             ServiceTypeActionHandler.getServiceActionId(
                                                      SERVICETYPE_CAS, 
                                                      ACTION_GRANT));

        grantPolicy.setObjectSpec(selfObjectId);
        grantPolicy.setObjectSpecDesc(OBJECT_SPEC);
        grantPolicy.setActionSpec(grantId);
        grantPolicy.setActionSpecDesc(SERVICEACTION_SPEC);

        // cas/revoke
        String revokeId = 
            Integer.toString(
                             ServiceTypeActionHandler.getServiceActionId(
                                                      SERVICETYPE_CAS, 
                                                      ACTION_REVOKE));

        revokePolicy.setActionSpec(revokeId);
        revokePolicy.setActionSpecDesc(SERVICEACTION_SPEC);

        // cas/query
        String queryId = 
            Integer.toString(
                             ServiceTypeActionHandler.getServiceActionId(
                                                      SERVICETYPE_CAS, 
                                                      ACTION_QUERY));

        queryPolicy.setObjectSpec(selfObjectId);
        queryPolicy.setObjectSpecDesc(OBJECT_SPEC);
        queryPolicy.setActionSpec(queryId);
        queryPolicy.setActionSpecDesc(SERVICEACTION_SPEC);
    }
}
