/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.cas.impl.bootstrap;

import org.globus.cas.impl.CasConstants;

import org.globus.cas.types.ServiceTypeData;
import org.globus.cas.types.NamespaceData;

import org.globus.cas.impl.databaseAccess.NamespaceDataHandler;
import org.globus.cas.impl.databaseAccess.ServiceTypeDataHandler;
import org.globus.cas.impl.databaseAccess.ServiceTypeActionHandler;

public class PopulateImplicitFTPData  {

    // Service type
    final public static String ftpServiceType = "file";

    // Actions
    final public static String readAction = "read";
    final public static String lookupAction = "lookup";
    final public static String writeAction = "write";
    final public static String createAction = "create";
    final public static String deleteAction = "delete";
    final public static String chdirAction = "chdir";

    // Namespace
    final public static String ftpWildcardNS = "FTPDirectoryTree";
    final public static String ftpExactNS = "FTPExact";
    
    public static void populateDb() throws Exception {
        
        // FTP Wildcard Namespace
        NamespaceData nsData1 = new NamespaceData();
        nsData1.setNickname(ftpWildcardNS);
        nsData1.setBasename(".");
        nsData1.setComparisonAlg(CasConstants.WILD_ALG_CLASSNAME);
        NamespaceDataHandler.storeObject(nsData1);

        // FTP Exact Namespace
        NamespaceData nsData2 = new NamespaceData();
        nsData2.setNickname(ftpExactNS);
        nsData2.setBasename(".");
        nsData2.setComparisonAlg(CasConstants.EXACT_ALG_CLASSNAME);
        NamespaceDataHandler.storeObject(nsData2);

        // ftp as service type
        ServiceTypeData serviceType = new ServiceTypeData();
        serviceType.setName(ftpServiceType);
        ServiceTypeDataHandler.storeObject(serviceType);

        // ftp/read
        ServiceTypeActionHandler.addServiceActionMapping(ftpServiceType, 
                                                         readAction);

        // ftp/lookup
        ServiceTypeActionHandler.addServiceActionMapping(ftpServiceType, 
                                                         lookupAction);

        // ftp/write
        ServiceTypeActionHandler.addServiceActionMapping(ftpServiceType, 
                                                         writeAction);

        // ftp/create
        ServiceTypeActionHandler.addServiceActionMapping(ftpServiceType, 
                                                         createAction);

        // ftp/delete
        ServiceTypeActionHandler.addServiceActionMapping(ftpServiceType, 
                                                         deleteAction);

        // ftp/chdir
        ServiceTypeActionHandler.addServiceActionMapping(ftpServiceType, 
                                                         chdirAction);
    }
}
