/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.cas.impl.databaseAccess;

import java.sql.Connection;
import java.sql.Statement;
import java.sql.SQLException;
import java.sql.ResultSet;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.globus.cas.types.ObjectData;

import org.globus.cas.impl.service.ObjectComparison;

import org.globus.util.I18n;

/**
 * Exact matching
 */
public class ExactComparison extends ObjectComparison {

    static Log logger = LogFactory.getLog(ExactComparison.class.getName());

    private static I18n i18n =
        I18n.getI18n("org.globus.cas.impl.databaseAccess.errors",
                     ExactComparison.class.getClassLoader());

    public ObjectData[] matchingSubset(ObjectData objData) 
        throws CasDBException {
        return null;
    }

    public ObjectData[] matchingSuperset(ObjectData objData)  
        throws CasDBException {
        return null;
    }

    // Policy object, request object
    public  boolean objectMatches(ObjectData policyObj, ObjectData requestObj) 
        throws CasDBException {

        // if not same namespace return false
        if (!policyObj.getObjectNamespace().equals(
                                            requestObj.getObjectNamespace()))
            return false;

        if (!policyObj.getObjectName().equals(requestObj.getObjectName()))
            return false;

        return true;
    }

    public boolean objectExists(ObjectData reqObj) 
        throws CasDBException {

        String query = "select * from" + CasDBConstants.TABLE_OBJECT 
            + " where (" + CasDBConstants.COL_NS_NICKNAME + "='" 
            + reqObj.getObjectNamespace() + "') and (" 
            + CasDBConstants.COL_OBJECT_NAME + "='" + reqObj.getObjectName() 
            + "')";
            
        Connection connection = CasDBStorage.getDBConnection();
        Statement statement = null;
        ResultSet resultSet = null;
        try {
            statement = connection.createStatement();
            resultSet = statement.executeQuery(query);
            if ((resultSet!=null) && (resultSet.next()))
                return true;
        }
        catch (SQLException exp) {
            logger.error(i18n.getMessage("existenceErr", new Object[]
                                         { "object", query} ), 
                         exp);
            throw new CasDBException(i18n.getMessage("objExistenceErr", 
                                                     new Object[] {
                                                         "object", "" })
                                     + exp.getMessage(), exp);
        }
        finally {
            // retuning connection irrespective of whether stmt
            // and result set are closed or not.
            CasDBStorage.returnDBConnection(connection);
            try {
                if (resultSet != null)
                    resultSet.close();
                if (statement != null)
                    statement.close();
            }
            catch (SQLException exp) {
                logger.warn(i18n.getMessage("existenceCloseErr", "object"),
                            exp);
            }
        }
        return false;
    }
}
