/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.cas.impl.databaseAccess;

import java.sql.Connection;
import java.sql.Statement;
import java.sql.SQLException;
import java.sql.ResultSet;

import java.util.Vector;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.globus.cas.types.CasObjectData;
import org.globus.cas.types.ObjectGroupData;
import org.globus.cas.types.ArrayOfString;

import org.globus.cas.impl.CasConstants;

import org.globus.util.I18n;

/**
 * Manipulate the data for ObjectGroups in the database
 */
public class ObjectGroupDataHandler {
    
    static Log logger = 
        LogFactory.getLog(ObjectGroupDataHandler.class.getName() );

 
    private static I18n i18n =
        I18n.getI18n("org.globus.cas.impl.databaseAccess.errors",
                     ObjectGroupDataHandler.class.getClassLoader());

    /**
     * Insert into group table 
     * Does not create a new object and will fail if the object specified 
     * does not exist
     */
    public static String storeObject(ObjectGroupData objectGpData) 
        throws CasDBException {

        Connection connection = CasDBStorage.getDBConnection();
        String groupName = objectGpData.getGroupName();
        String query = "insert into " + CasDBConstants.TABLE_OBJECTGP + " (" 
            + CasDBConstants.COL_OBJECTGP_NAME + ") values ('" + groupName 
            + "')";
        Statement statement = null;
        try {
            statement = connection.createStatement();
            logger.debug("store query is " + query);
            statement.executeUpdate(query);
        }
        catch (SQLException exp) {
            logger.error(i18n.getMessage("storeErr", new Object[] { 
                "object group", query }), exp);
            throw new CasDBException(i18n.getMessage("storeErr", new Object[] {
                "object group", "" } ) + exp.getMessage(), exp);
        }
        finally {
            // returning connection irrespective of whether stmt
            // and result set are closed or not.
            CasDBStorage.returnDBConnection(connection);
            try {
                if (statement != null)
                    statement.close();
            }
            catch (SQLException exp) {
                String err = "Error storing object group data. Could not "
                    + " close SQL statement.";
                logger.warn(err, exp);
            }
        }
        return groupName;
    }
    
    
    /**
     * List of object group
     */
    public static String[] list() throws CasDBException {
        String query = "select"  + CasDBConstants.COL_OBJECTGP_NAME + " from" 
            + CasDBConstants.TABLE_OBJECTGP;
        return CasDBStorage.runListQuery(query, 
                                         CasDBConstants.COL_OBJECTGP_NAME);
    }

    /** 
     * Retrieve object group object
     * Picks up values from object group table and object group entries table
     */
    public static CasObjectData retrieveObject(String groupName) 
        throws CasDBException {
        
        ObjectGroupData returnObject = null;
        Vector temp = null;
        Vector temp1 = null;    
        Connection connection = CasDBStorage.getDBConnection();
        String query = "select" + CasDBConstants.COL_OBJECT_SPEC + "," 
            + CasDBConstants.COL_OBJECT_SPEC_DESC + " from" 
            + CasDBConstants.TABLE_OBJECTGP_ENTRY + " where " 
            + CasDBConstants.COL_OBJECTGP_NAME + " ='" + groupName.trim() + "'";
        Statement statement = null;
        ResultSet resultSet = null;
        try {
            statement = connection.createStatement();
            resultSet = statement.executeQuery(query);  
            if ((resultSet != null) && (resultSet.next())) {
                temp = new Vector(resultSet.getFetchSize());            
                temp1 = new Vector(resultSet.getFetchSize());
                do {
                    temp.add(resultSet.getString(
                             CasDBConstants.COL_OBJECT_SPEC.trim()));
                    temp1.add(resultSet.getString(
                              CasDBConstants.COL_OBJECT_SPEC_DESC.trim()));
                } while (resultSet.next());
            }
        }
        catch (SQLException exp) {
            logger.error(i18n.getMessage("retrieveErr", new Object[] {
                "object group", query } ), exp);
            throw new CasDBException(i18n.getMessage("retrieveErr", 
                                                     new Object[] {
                                                         "object group", ""})
                                     + exp.getMessage() , exp);
        }
        finally {
            // returning connection irrespective of whether stmt
            // and result set are closed or not.
            CasDBStorage.returnDBConnection(connection);
            try {
                if (resultSet != null)
                    resultSet.close();
                if (statement != null)
                    statement.close();
            }
            catch (SQLException exp) {
                logger.warn(i18n.getMessage("retrieveErrClose", 
                                            "object group"), exp);
            }
        }
        
        ArrayOfString arrayOfObjSpec = null;
        ArrayOfString arrayOfObjSpecDesc = null;
        if (temp != null) {
            String objSpec[] = new String[temp.size()];
            String objSpecDesc[] = new String[temp.size()];
            temp.toArray(objSpec);
            temp1.toArray(objSpecDesc);
            arrayOfObjSpec = new ArrayOfString(objSpec);
            arrayOfObjSpecDesc = new ArrayOfString(objSpecDesc);
        }

        connection = CasDBStorage.getDBConnection();
        query = "select * from" + CasDBConstants.TABLE_OBJECTGP + " where" 
            + CasDBConstants.COL_OBJECTGP_NAME + " = '" + groupName.trim() 
            + "'";
        try {
            statement = connection.createStatement();
            resultSet = statement.executeQuery(query);
            if ((resultSet!=null) && (resultSet.next())) {
                returnObject = new ObjectGroupData();
                returnObject.setGroupName(
                             resultSet.getString(
                             CasDBConstants.COL_OBJECTGP_NAME.trim()));
                returnObject.setObjectSpecs(arrayOfObjSpec);
                returnObject.setObjectSpecsDesc(arrayOfObjSpecDesc);
            }
        }
        catch (SQLException exp) {
            logger.error(i18n.getMessage("retrieveErr", new Object[] {
                "object group", query}), exp);
            throw new CasDBException(i18n.getMessage("retrieveErr", 
                                                     new Object[] {
                                                         "object group", ""})
                                     + exp.getMessage() , exp);
        }
        finally {
            // returning connection irrespective of whether stmt
            // and result set are closed or not.
            CasDBStorage.returnDBConnection(connection);
            try {
                if (resultSet != null)
                    resultSet.close();
                if (statement != null)
                    statement.close();
            }
            catch (SQLException exp) {
                logger.warn(i18n.getMessage("retrieveErrClose", 
                                            "object group"), exp);
            }
        }
        return returnObject;
    }

    /** 
     * Deletes object group data
     * Manipulates object group table 
     */
    public static void deleteObject(String groupName) throws CasDBException {

        Connection connection = CasDBStorage.getDBConnection();
        String query = "delete from" + CasDBConstants.TABLE_OBJECTGP + " where"
            + CasDBConstants.COL_OBJECTGP_NAME + " ='" + groupName.trim() 
            + "'";
        Statement statement = null;
        int result = -1;
        try {
            statement = connection.createStatement();
            result = statement.executeUpdate(query);
        }
        catch (SQLException exp) {
            logger.error(i18n.getMessage("delObjErr", new Object[] {
                "object group", query }), exp);
            throw new CasDBException(i18n.getMessage("delObjErr", 
                                                     new Object[] {
                                                         "object group", "" })
                                     + exp.getMessage(), exp);
        }
        finally {
            // returning connection irrespective of whether stmt
            // and result set are closed or not.
            CasDBStorage.returnDBConnection(connection);
            try {
                if (statement != null)
                    statement.close();
            }
            catch (SQLException exp) {
                logger.warn(i18n.getMessage("delObjErrClose", "object group"),
                            exp);
            }
        }
        if (result <=0) {
            String err = i18n.getMessage("objGpDelError", groupName);
            logger.error(err);
            throw new CasDBException(err);
        }
    }

    private static String getObjectId(String objectName, String objectNamespace)
        throws CasDBException {
        
        String objectId = null;
        String query = "select" + CasDBConstants.COL_OBJECT_ID + " from" 
            + CasDBConstants.TABLE_OBJECT + " where (" 
            + CasDBConstants.COL_OBJECT_NAME + "='" + objectName + "') and (" 
            + CasDBConstants.COL_NS_NICKNAME + "='" + objectNamespace + "')";
        logger.debug("Query is " + query);
        Connection connection = CasDBStorage.getDBConnection();
        Statement statement = null;
        ResultSet resultSet = null;
        try {
            statement = connection.createStatement();
            resultSet = statement.executeQuery(query);
            if ((resultSet!=null) && (resultSet.next())) {
                logger.debug("setting objectid");
                objectId = 
                    resultSet.getString(CasDBConstants.COL_OBJECT_ID.trim());
            }
        }
        catch (SQLException exp) {
            logger.error(i18n.getMessage("objIdRetrErr", query), exp);
            throw new CasDBException(i18n.getMessage("objIdRetrErr", "")
                                     + exp.getMessage(), exp);
        }
        finally {
            // returning connection irrespective of whether stmt
            // and result set are closed or not.
            CasDBStorage.returnDBConnection(connection);
            try {
                if (resultSet != null)
                    resultSet.close();
                if (statement != null)
                    statement.close();
            }
            catch (SQLException exp) {
                logger.warn(i18n.getMessage("objIdRetrCloseErr"), exp);
            }
        }
        return objectId;
    }

    /** 
     * Adds an object specification and its description as a member of the group
     * ObjectSpec can be objectId(if object), user name, userGroup name, 
     * serviceType name, namespace nickname, trust anchor nickname
     * Corresponding objectSpecDesc are object, user, userGroup, serviceType,
     * namespace, trustAnchor
     * Ensure that consistency in the db is maintained and so entries for 
     * object spec and exists in the relevant tables
     */
    public static void addGroupMember(String groupName, String objectSpec, 
                                      String objectSpecDesc) 
        throws CasDBException {
        
        String baseErrMesg = i18n.getMessage("objGpMemAddErr", "");
        if ((groupName == null) || (objectSpec== null) || 
            (objectSpecDesc == null)) {
            String err = 
                i18n.getMessage("paramNull",
                                "Groupname/ObjectSpec/ObjectSpecDesc");
            logger.error(err);
            throw new CasDBException(err);
        }
        
        groupName = groupName.trim();
        objectSpec = objectSpec.trim();
        objectSpecDesc = objectSpecDesc.trim();
        
        boolean validSpec = false;
        // if objectSpecDesc is "object" check if objectSpec is in object 
        // table i.e objectid   
        if (objectSpecDesc.equals(CasConstants.OBJECT_SPEC)) {
            logger.debug("Object spec is \"object\". check if it exists in "
                         + "object table ");
            if (!CasDBStorage.rowExists(CasDBConstants.TABLE_OBJECT, 
                            CasDBConstants.COL_OBJECT_ID, objectSpec)) {
                String err = i18n.getMessage("notPresentInTable",
                                             new Object[] { "objectid",
                                                            objectSpec,
                                                            "object" });
                logger.error(err);
                throw new CasDBException(err);
            }
            validSpec = true;
        }

        // if objectSpecDesc is "user" check if objectSpec is in user 
        // table i.e username
        if (objectSpecDesc.equals(CasConstants.USER_SPEC)) {
            logger.debug("Object spec is \"user\". check if it exists in user"
                         + " table ");
            if (!CasDBStorage.rowExists(CasDBConstants.TABLE_USER, 
                            CasDBConstants.COL_USER_NICKNAME, objectSpec)) {
                String err = i18n.getMessage("notPresentInTable",
                                             new Object[] { "user name",
                                                            objectSpec,
                                                            "user table" });
                logger.error(err);
                throw new CasDBException(err);
            }
            validSpec = true;
        }
        
        // if objectSpecDesc is "userGroup" check if objectSpec is in user 
        // group table i.e userGroup name
        if (objectSpecDesc.equals(CasConstants.USERGP_SPEC)) {
            logger.debug("Object spec is \"userGroup\". check if it exists in"
                         + " user group table ");
            if (!CasDBStorage.rowExists(CasDBConstants.TABLE_USERGP, 
                            CasDBConstants.COL_USERGP_NAME, objectSpec)) {
                String err = i18n.getMessage("notPresentInTable",
                                             new Object[] { 
                                                 "user group name",
                                                 objectSpec,
                                                 "user group table" });
                logger.error(err);
                throw new CasDBException(err);
            }
            validSpec = true;
        }
        
        // if objectSpecDesc is "serviceType" check if objectSpec is in 
        // serviceType table i.e serviceType name
        if (objectSpecDesc.equals(CasConstants.SERVICETYPE_SPEC)) {
            logger.debug("Object spec is \"serviceType\". check if it exists "
                         + "in serviceType table ");
            if (!CasDBStorage.rowExists(CasDBConstants.TABLE_SERVICETYPE, 
                            CasDBConstants.COL_SERVICETYPE_NAME, objectSpec)) {
                String err = i18n.getMessage("notPresentInTable",
                                             new Object[] { 
                                                 "service type name",
                                                 objectSpec,
                                                 "service type table" });
                logger.error(err);
                throw new CasDBException(err);
            }
            validSpec = true;
        }
        
        // if objectSpecDesc is "namespace" check if objectSpec is in 
        // Namespace table i.e Namespace nick
        if (objectSpecDesc.equals(CasConstants.NAMESPACE_SPEC)) {
            logger.debug("Object spec is \"namespace\". check if it exists in "
                         + "namespace table ");
            if (!CasDBStorage.rowExists(CasDBConstants.TABLE_NAMESPACE, 
                            CasDBConstants.COL_NS_NICKNAME, objectSpec)) {
                String err = baseErrMesg 
                    + i18n.getMessage("notPresentInTable",
                                      new Object[] { 
                                          "namespace",
                                          objectSpec,
                                          "namespace table" });
                logger.error(err);
                throw new CasDBException(err);
            }
            validSpec = true;
        }
        
        // if objectSpecDesc is "trustAnchor" check if objectSpec is in 
        // TrustAnchor table i.e trustanchor nick
        if (objectSpecDesc.equals(CasConstants.TRUSTANCHOR_SPEC)) {
            logger.debug("Object spec is \"trustAnchor\". check if it exists "
                         + "in TrustAnchor table ");
            if (!CasDBStorage.rowExists(CasDBConstants.TABLE_TRUSTANCHOR, 
                            CasDBConstants.COL_TRUST_NICKNAME, objectSpec)) {
                String err = baseErrMesg 
                    + i18n.getMessage("notPresentInTable",
                                      new Object[] { 
                                          "TrustAnchor",
                                          objectSpec,
                                          "trustAnchor table" });
                logger.error(err);
                throw new CasDBException(err);
            }
            validSpec = true;
        }
        
        
        if (!validSpec) {
            String errMesg = 
                i18n.getMessage("invalidObjSpe", new Object[]
                    { CasConstants.OBJECT_SPEC, CasConstants.USER_SPEC,
                      CasConstants.USERGP_SPEC, CasConstants.SERVICETYPE_SPEC,
                      CasConstants.NAMESPACE_SPEC, 
                      CasConstants.TRUSTANCHOR_SPEC });
            logger.error(errMesg);
            throw new CasDBException(errMesg);
        }

        Connection connection = CasDBStorage.getDBConnection();
        String query = "insert into" + CasDBConstants.TABLE_OBJECTGP_ENTRY 
            + " ( " + CasDBConstants.COL_OBJECTGP_NAME + "," 
            + CasDBConstants.COL_OBJECT_SPEC + "," 
            + CasDBConstants.COL_OBJECT_SPEC_DESC + ") values ('" + groupName 
            + "','" + objectSpec + "','" + objectSpecDesc + "')";
        Statement statement = null;
        try {
            statement = connection.createStatement();
            logger.debug(" query is " + query);
            statement.executeUpdate(query);
            statement.close ();
        }
        catch (SQLException exp) {
            logger.error(i18n.getMessage("objGpMemAddErr", query), exp);
            throw new CasDBException(i18n.getMessage("objGpMemAddErr", "")
                                     + exp.getMessage(), exp);
        }
        finally {
            // returning connection irrespective of whether stmt
            // and result set are closed or not.
            CasDBStorage.returnDBConnection(connection);
            try {
                if (statement != null)
                    statement.close();
            }
            catch (SQLException exp) {
                logger.warn(i18n.getMessage("objGpMemAddCloseErr"), exp);
            }
        }
    }
    
    /** 
     * Removes the group member
     * Manipulates only group entry table
     */
    public static void removeGroupMember(String groupName, String objectSpec, 
                                         String objectSpecDesc) 
        throws CasDBException {
        
        if ((groupName == null) || (objectSpec== null) || 
            (objectSpecDesc == null)) {
            String err = 
                i18n.getMessage("paramNull", 
                                "Groupname/ObjectSpec/ObjectSpecDesc");
            logger.error(err);
            throw new CasDBException(err);
        }
        
        String query = "delete from" + CasDBConstants.TABLE_OBJECTGP_ENTRY
            + " where (" + CasDBConstants.COL_OBJECTGP_NAME + "='" 
            + groupName.trim() + "') and (" + CasDBConstants.COL_OBJECT_SPEC 
            + " ='" + objectSpec.trim() + "') and (" 
            + CasDBConstants.COL_OBJECT_SPEC_DESC.trim() + "='" 
            + objectSpecDesc + "')";
        Connection connection = CasDBStorage.getDBConnection();
        Statement statement = null;
        int result;
        try {
            statement = connection.createStatement();
            logger.debug(" query is " + query);
            result = statement.executeUpdate(query);
        }
        catch (SQLException exp) {
            logger.error(i18n.getMessage("delObjErr", new Object[] {
                "group member", query }), exp);
            throw new CasDBException(i18n.getMessage("delObjErr", 
                                                     new Object[] {
                                                         "group member", "" })
                                     + exp.getMessage(), exp);
        }
        finally {
            // returning connection irrespective of whether stmt
            // and result set are closed or not.
            CasDBStorage.returnDBConnection(connection);
            try {
                if (statement != null)
                    statement.close();
            }
            catch (SQLException exp) {
                logger.warn(i18n.getMessage("delObjErrClose", "object group"),
                            exp);
            }
        }
        if (result <=0) {
            String err = i18n.getMessage("objGpMemDelError", new Object[] {
                groupName, objectSpec, objectSpecDesc });
            logger.error(err);
            throw new CasDBException(err);
        }
    }

    public static void addObjectToGroup(String groupName, String objectName, 
                                        String objectNamespace) 
        throws CasDBException {
        
        String objectId = getObjectId(objectName, objectNamespace);
        if (objectId == null) {
            String err = i18n.getMessage("addGpErr") 
                + i18n.getMessage("objIdRetrErr", objectName + "/" 
                                  + objectNamespace);
            logger.error(err);
            throw new CasDBException(err);
        }
        addGroupMember(groupName, objectId, CasConstants.OBJECT_SPEC);
    }

    public static void removeObjectFromGroup(String groupName, 
                                             String objectName, 
                                             String objectNamespace) 
        throws CasDBException {
        
        String objectId = getObjectId(objectName, objectNamespace);
        if (objectId == null) {
            String err = i18n.getMessage("delObjErr") 
                + i18n.getMessage("objIdRetrErr", objectName + "/" 
                                  + objectNamespace);
            logger.error(err);
            throw new CasDBException(err);
        }
        removeGroupMember(groupName, objectId, CasConstants.OBJECT_SPEC);
    }

    public static String[] getObjectGroupsForMember(String objectSpec, 
                                                  String objectSpecDesc)
        throws CasDBException {
        logger.debug("ObjectGp Member for " + objectSpec.trim() + " "
                     + objectSpecDesc.trim());
        String query = "select" + CasDBConstants.COL_OBJECTGP_NAME + " from" 
            + CasDBConstants.TABLE_OBJECTGP_ENTRY + " where "
            + CasDBConstants.COL_OBJECT_SPEC + "='" + objectSpec.trim() 
            + "' and " + CasDBConstants.COL_OBJECT_SPEC_DESC + "='" 
            + objectSpecDesc.trim() + "'";
        Vector groupNames = null;
        Connection connection = CasDBStorage.getDBConnection();
        Statement statement = null;
        ResultSet resultSet= null;
        try {
            statement = connection.createStatement();
            resultSet = statement.executeQuery(query);  
            if ((resultSet != null) && (resultSet.next())) {
                groupNames = new Vector(resultSet.getFetchSize());
                do {
                    groupNames.add(
                        resultSet.getString(
                                  CasDBConstants.COL_OBJECTGP_NAME.trim()));
                } while (resultSet.next());
            }
        }
        catch (SQLException exp) {
            logger.error(i18n.getMessage("retrieveErr", new Object[] {
                "object group", query }), exp);
            throw new CasDBException(i18n.getMessage("retrieveErr", 
                                                     new Object[] {
                                                         "object group", "" })
                                     + exp.getMessage(), exp);
        }
        finally {
            // returning connection irrespective of whether stmt
            // and result set are closed or not.
            CasDBStorage.returnDBConnection(connection);
            try {
                if (resultSet != null)
                    resultSet.close();
                if (statement != null)
                    statement.close();
            }
            catch (SQLException exp) {
                logger.warn(i18n.getMessage("retrieveErrClose", 
                                            "object group"), exp);
            }
        }
        return CasDBStorage.vectorToStringArray(groupNames);
    }

    public static boolean isMember(String objectSpec, String objSpecDesc) 
        throws CasDBException {

        if ((objectSpec == null) || (objSpecDesc == null)) {
            String errMesg = "Parameter(s) null. Could not verify group "
                + "membership";
            logger.error(errMesg);
            throw new CasDBException(errMesg);
        }
        Connection connection = null;
        try {
            connection = CasDBStorage.getDBConnection();
            return isMemberNoValidate(connection, objectSpec, objSpecDesc);
        } finally {
            // returning connection irrespective of whether stmt
            // and result set are closed or not.
            CasDBStorage.returnDBConnection(connection);
        }
    }

    public static boolean isMember(Connection conn, String objectSpec, 
                                   String objSpecDesc) 
        throws CasDBException {

        if ((objectSpec == null) || (objSpecDesc == null)) {
            String errMesg = "Parameter(s) null. Could not verify group "
                + "membership";
            logger.error(errMesg);
            throw new CasDBException(errMesg);
        }
        return isMemberNoValidate(conn, objectSpec, objSpecDesc);
    }

    private static boolean isMemberNoValidate(Connection connection, 
                                              String objectSpec, 
                                              String objSpecDesc) 
        throws CasDBException {
        logger.debug("isMember check spec " + objectSpec.trim() + " desc " 
                     + objSpecDesc.trim());     
        String query = "select * from" + CasDBConstants.TABLE_OBJECTGP_ENTRY 
            + " where ((" + CasDBConstants.COL_OBJECT_SPEC + "='" 
            + objectSpec.trim() + "') and (" 
            + CasDBConstants.COL_OBJECT_SPEC_DESC + "='" + objSpecDesc.trim() 
            + "'))";
        logger.debug("\n Query is \n" + query + "\n");

        Statement statement = null;
        ResultSet resultSet = null;
        try {
            statement = connection.createStatement();
            resultSet = statement.executeQuery(query);
            if ((resultSet!=null) && (resultSet.next())) {
                logger.debug("There is an entry and hence is a member ");
                return true;
            }
        }
        catch (SQLException exp) {
            logger.error(i18n.getMessage("existenceErr", query), exp);
            throw new CasDBException(i18n.getMessage("existenceErr", "")
                                     + exp.getMessage(), exp);
        }
        finally {
            try {
                if (resultSet != null)
                    resultSet.close();
                if (statement != null)
                    statement.close();
            }
            catch (SQLException exp) {
                logger.warn(i18n.getMessage("dataExistenceCloseErr"), exp);
            }
        }
        logger.debug("No entry, not a member");

        return false;
    }
}
