/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.cas.impl.databaseAccess;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.globus.cas.types.PolicyData;
import org.globus.cas.types.ObjectData;
import org.globus.cas.types.CasObjectData;

import org.globus.cas.impl.CasConstants;

import org.globus.cas.impl.service.ObjectComparison;

/**
 * Used to evaluate permissions
 */
public class PermissionsHandler {
    
    static Log logger = LogFactory.getLog(PermissionsHandler.class.getName());
    
    public static boolean canEnrollTrustAnchor(String userName) 
	throws CasDBException {
	try {
	    return PermissionsEvaluator.userHasPermissions(
				     userName, 
				     CasConstants.trustEnrollPolicy);
	}
	catch (CasDBException exp) {
	    String err = "Could not ascertain enroll trust anchor priviledges "
		+ "for user. ";
	    logger.error(err, exp);
	    throw new CasDBException(err + exp.getMessage(), exp);
	}
    }

    public static boolean canUnenrollTrustAnchor(String userName, 
						 String nickname) 
	throws CasDBException {
	CasConstants.unenrollPolicy.setObjectSpec(nickname.trim());
	CasConstants.unenrollPolicy.setObjectSpecDesc(
				    CasConstants.TRUSTANCHOR_SPEC);
	try {
	    return PermissionsEvaluator.userHasPermissions(
				     userName, 
				     CasConstants.unenrollPolicy);
	}
	catch (CasDBException exp) {
	    String err = "Could not ascertain unenroll trust anchor "
		+ " priviledges for user. ";
	    logger.error(err, exp);
	    throw new CasDBException(err + exp.getMessage(), exp);
	}
    }

    public static boolean canEnrollUser(String userName) throws CasDBException {
	try {
	    return PermissionsEvaluator.userHasPermissions(
				     userName, 
				     CasConstants.userEnrollPolicy);
	}
	catch (CasDBException exp) {
	    String err = "Could not ascertain enroll user priviledges for "
		+ "user.";
	    logger.error(err, exp);
	    throw new CasDBException(err + exp.getMessage(), exp);
	}
    }

    public static boolean canUnenrollUser(String userName, String userNickname) 
	throws CasDBException {
	CasConstants.unenrollPolicy.setObjectSpec(userNickname.trim());
	CasConstants.unenrollPolicy.setObjectSpecDesc(
				    CasConstants.USER_SPEC);
	try {
	    return PermissionsEvaluator.userHasPermissions(userName, 
				     CasConstants.unenrollPolicy);
	}
	catch (CasDBException exp) {
	    String err = "Could not ascertain unenroll user priviledges for "
		+ "user.";
	    logger.error(err, exp);
	    throw new CasDBException(err + exp.getMessage(), exp);
	}
    }

    public static boolean canCreateGps(String type, String userName) 
	throws CasDBException {

	try {
	    if (type.trim().equals(CasConstants.USER_SPEC)) {
		return PermissionsEvaluator.userHasPermissions(
					 userName, 
					 CasConstants.userGpCreationPolicy);
	    } else if (type.trim().equals(CasConstants.OBJECT_SPEC)) {
		return PermissionsEvaluator.userHasPermissions(
					  userName, 
					  CasConstants.objectGpCreationPolicy);
	    } else if (type.trim().equals(CasConstants.SERVICEACTION_SPEC)) {
		return PermissionsEvaluator.userHasPermissions(
		       userName, 
		       CasConstants.serviceActionGpCreationPolicy);
	    } else {
		logger.error("type shoud be " + CasConstants.USER_SPEC 
			     + " or " + CasConstants.OBJECT_SPEC + " or " 
			     + CasConstants.SERVICEACTION_SPEC);
		throw new CasDBException("type shoud be " 
					 + CasConstants.USER_SPEC + " or " 
					 + CasConstants.OBJECT_SPEC + " or " 
					 + CasConstants.SERVICEACTION_SPEC);
	    }
	}
	catch (CasDBException exp) {
	    String err = "Could not ascertain create group priviledges for "
		+ "user.";
	    logger.error(err, exp);
	    throw new CasDBException(err + exp.getMessage(), exp);
	}
    }

    public static boolean canDeleteGps(String type, String userName, 
				       String gpName) 
	throws CasDBException {

	try {
	    CasConstants.gpDeletionPolicy.setObjectSpec(gpName.trim());
	    if (type.trim().equals(CasConstants.USER_SPEC)) {
		CasConstants.gpDeletionPolicy.setObjectSpecDesc(
					      CasConstants.USERGP_SPEC);
	    } else if (type.trim().equals(CasConstants.OBJECT_SPEC)) {
		CasConstants.gpDeletionPolicy.setObjectSpecDesc(
					      CasConstants.OBJECTGP_SPEC);
	    } else if (type.trim().equals(CasConstants.SERVICEACTION_SPEC)) {
		CasConstants.gpDeletionPolicy.setObjectSpecDesc(
			     CasConstants.SERVICEACTIONGP_SPEC);
	    } else {
		logger.error("type shoud be " + CasConstants.USER_SPEC 
			     + " or " + CasConstants.OBJECT_SPEC + " or "
			     + CasConstants.SERVICEACTION_SPEC);
		throw new CasDBException("type shoud be " 
					 + CasConstants.USER_SPEC + " or " 
					 + CasConstants.OBJECT_SPEC + " or " 
					 + CasConstants.SERVICEACTION_SPEC);
	    }
	    return PermissionsEvaluator.userHasPermissions(
				     userName, 
				     CasConstants.gpDeletionPolicy);
	}
	catch (CasDBException exp) {
	    String err = "Could not ascertain delete group priviledges for "
		+ "user.";
	    logger.error(err, exp);
	    throw new CasDBException(err + exp.getMessage(), exp);
	}
    }

    public static boolean canManipObjectGps(String type, String userName, 
					    String gpName)
	throws CasDBException {

	try {
	    if (type.trim().equals(CasConstants.ADD_OPERATION)) {
		CasConstants.gpAddEntryPolicy.setObjectSpec(gpName.trim());
		CasConstants.gpAddEntryPolicy.setObjectSpecDesc(
					      CasConstants.OBJECTGP_SPEC);
		return PermissionsEvaluator.userHasPermissions(
					 userName, 
					 CasConstants.gpAddEntryPolicy);
	    }
	    else if (type.trim().equals(CasConstants.REMOVE_OPERATION)) {
		CasConstants.gpDeleteEntryPolicy.setObjectSpec(gpName.trim());
		CasConstants.gpDeleteEntryPolicy.setObjectSpecDesc(
						 CasConstants.OBJECTGP_SPEC);
		return PermissionsEvaluator.userHasPermissions(
					 userName, 
					 CasConstants.gpDeleteEntryPolicy);
	    }
	    else {
		logger.error("type should be " + CasConstants.ADD_OPERATION 
			     + " or " + CasConstants.REMOVE_OPERATION);
		throw new CasDBException("type should be " 
					 + CasConstants.ADD_OPERATION + " or " 
					 + CasConstants.REMOVE_OPERATION);
	    }
	}
	catch (CasDBException exp) {
	    String err = "Could not ascertain add/remove from object group "
		+ " priviledges for user.";
	    logger.error(err + exp.toString());
	    throw new CasDBException(err + exp.getMessage(), exp);
	}
    }

    public static boolean canManipUserGps(String type, String userName, 
					  String gpName) 
	throws CasDBException {

	try {
	    if (type.trim().equals(CasConstants.ADD_OPERATION)) {
		CasConstants.gpAddEntryPolicy.setObjectSpec(gpName.trim());			
		CasConstants.gpAddEntryPolicy.setObjectSpecDesc(
					      CasConstants.USERGP_SPEC);
		return PermissionsEvaluator.userHasPermissions(
					 userName, 
					 CasConstants.gpAddEntryPolicy);
	    }
	    else if (type.trim().equals(CasConstants.REMOVE_OPERATION)) {
		CasConstants.gpDeleteEntryPolicy.setObjectSpec(gpName.trim());
		CasConstants.gpDeleteEntryPolicy.setObjectSpecDesc(
						 CasConstants.USERGP_SPEC);
		return PermissionsEvaluator.userHasPermissions(
					 userName, 
					 CasConstants.gpDeleteEntryPolicy);
	    }
	    else {
		logger.error("type should be " + CasConstants.ADD_OPERATION 
			     + " or " + CasConstants.REMOVE_OPERATION);
		throw new CasDBException("type should be " 
					 + CasConstants.ADD_OPERATION + " or " 
					 + CasConstants.REMOVE_OPERATION);
	    }
	}
	catch (CasDBException exp) {
	    String err = "Could not ascertain add/remove from user group "
		+ " priviledges for user.";
	    logger.error(err + exp.toString());
	    throw new CasDBException(err + exp.getMessage(), exp);
	}
    }

    public static boolean canManipServiceActionGps(String type, String userName,
						   String gpName) 
	throws CasDBException {

	try {
	    if (type.trim().equals(CasConstants.ADD_OPERATION)) {
		CasConstants.gpAddEntryPolicy.setObjectSpec(gpName.trim());			
		CasConstants.gpAddEntryPolicy.setObjectSpecDesc(
			     CasConstants.SERVICEACTIONGP_SPEC);
		return PermissionsEvaluator.userHasPermissions(
					 userName, 
					 CasConstants.gpAddEntryPolicy);
	    }
	    else if (type.trim().equals(CasConstants.REMOVE_OPERATION)) {
		CasConstants.gpDeleteEntryPolicy.setObjectSpec(gpName.trim());
		CasConstants.gpDeleteEntryPolicy.setObjectSpecDesc(
			     CasConstants.SERVICEACTIONGP_SPEC);
		return PermissionsEvaluator.userHasPermissions(
					 userName, 
					 CasConstants.gpDeleteEntryPolicy);
	    }
	    else {
		logger.error("type should be " + CasConstants.ADD_OPERATION 
			     + " or " + CasConstants.REMOVE_OPERATION);
		throw new CasDBException("type should be " 
					 + CasConstants.ADD_OPERATION + " or " 
					 + CasConstants.REMOVE_OPERATION);
	    }
	}
	catch (CasDBException exp) {
	    String err = "Could not ascertain add/remove from service action "
		+ "group priviledges for user.";
	    logger.error(err + exp.toString());
	    throw new CasDBException(err + exp.getMessage(), exp);
	}
    }

    public static boolean canRemoveServiceActionId(String userName, 
						   String serviceActionId) 
	throws CasDBException {
	String serviceAction = ServiceTypeActionHandler.getServiceActionMapping(
			       Integer.parseInt(serviceActionId));
	String serviceTypeName = serviceAction.substring(0, 
					       serviceAction.indexOf("/"));
	return canManipServiceActionMapping(CasConstants.REMOVE_OPERATION, 
					    userName, serviceTypeName);
    }

    public static boolean canManipServiceActionMapping(String type, 
						       String userName, 
						       String serviceType) 
	throws CasDBException {
	
	try {
	    if (type.trim().equals(CasConstants.ADD_OPERATION)) {
		CasConstants.gpAddEntryPolicy.setObjectSpec(serviceType.trim());
		CasConstants.gpAddEntryPolicy.setObjectSpecDesc(
					      CasConstants.SERVICETYPE_SPEC);
		return PermissionsEvaluator.userHasPermissions(
					 userName, 
					 CasConstants.gpAddEntryPolicy);
	    }
	    else if (type.trim().equals(CasConstants.REMOVE_OPERATION)) {
		CasConstants.gpDeleteEntryPolicy.setObjectSpec(
						 serviceType.trim());
		CasConstants.gpDeleteEntryPolicy.setObjectSpecDesc(
			     CasConstants.SERVICETYPE_SPEC);
		return PermissionsEvaluator.userHasPermissions(
					 userName, 
					 CasConstants.gpDeleteEntryPolicy);
	    }
	    else {
		logger.error("type should be " + CasConstants.ADD_OPERATION 
			     + " or " + CasConstants.REMOVE_OPERATION);
		throw new CasDBException("type should be " 
					 + CasConstants.ADD_OPERATION + " or " 
					 + CasConstants.REMOVE_OPERATION);
	    }
	}
	catch (CasDBException exp) {
	    String err = "Could not ascertain add/remove  service action "
		+ "mapping priviledges for user.";
	    logger.error(err + exp.toString());
	    throw new CasDBException(err + exp.getMessage(), exp);
	}
    }

    public static boolean canCreateNamespace(String userName) 
	throws CasDBException {
	try {
	    return PermissionsEvaluator.userHasPermissions(
				     userName, 
				     CasConstants.namespaceEnrollPolicy);
	}
	catch (CasDBException exp) {
	    String err = "Could not ascertain create namespace priviledges for "
		+ "user.";
	    logger.error(err + exp.toString());
	    throw new CasDBException(err + exp.getMessage(), exp);
	}
    }

    public static boolean canDeleteNamespace(String userName, String nsNickname)
	throws CasDBException {

	CasConstants.unenrollPolicy.setObjectSpec(nsNickname.trim());
	CasConstants.unenrollPolicy.setObjectSpecDesc(
				    CasConstants.NAMESPACE_SPEC);
	try {
	    return PermissionsEvaluator.userHasPermissions(userName, 
				     CasConstants.unenrollPolicy);
	}
	catch (CasDBException exp) {
	    String err = "Could not ascertain delete namespace priviledges for "
		+ "user.";
	    logger.error(err + exp.toString());
	    throw new CasDBException(err + exp.getMessage(), exp);
	}
    }

    private static boolean hasEnrollObjPermission(String userName, 
						  ObjectData objData) 
	throws CasDBException {
	
	PolicyData enrollObjPolicy  = CasConstants.objectEnrollPolicy;
	enrollObjPolicy.setObjectSpec(objData.getObjectId());
	enrollObjPolicy.setObjectSpecDesc(CasConstants.OBJECT_SPEC);
	return PermissionsEvaluator.userHasPermissions(userName, 
						       enrollObjPolicy);
    }

    private static boolean hasGrantAllObjPermission(String userName,
						    ObjectData objData)
	throws CasDBException {

	PolicyData grantAllObjPolicy  = CasConstants.grantAllPolicy;
	grantAllObjPolicy.setObjectSpec(objData.getObjectId());
	grantAllObjPolicy.setObjectSpecDesc(CasConstants.OBJECT_SPEC);
	return PermissionsEvaluator.userHasPermissions(userName, 
						       grantAllObjPolicy);
	
    }

    public static boolean canCreateObject(String userName, String namespaceNick,
					  String objectName)
	throws CasDBException {

	logger.debug("canCreateObj " + userName + " " + namespaceNick + "  " 
		     + objectName);
	// Need atleast cas/enrollObject	
	try {
	    if (!PermissionsEvaluator.userHasPermissions(
				     userName, 
				     CasConstants.objectEnrollPolicy)) {
		logger.debug("Does not have cas/enrollObject");
		return false;
	    }
	}
	catch (CasDBException exp) {
	    String err = "Could not ascertain create object priviledges for "
		+ "user.";
	    logger.error(err + exp.toString());
	    throw new CasDBException(err + exp.getMessage(), exp);
	}
	logger.debug("cas/enroll exists");
	
	// Test for object existence
	String compAlg = ObjectDataHandler.getComparisonAlg(namespaceNick);
	ObjectComparison objComp = 
	    ObjectDataHandler.getComparisonClass(compAlg);

	ObjectData objData = new ObjectData();
	objData.setObjectName(objectName);
	objData.setObjectNamespace(namespaceNick);
	// Check  if exact match exists
	if (objComp.exactMatchExists(objData)) {
	    logger.error("Object already exists");
	    throw new CasDBException("Object already exists");
	}
	logger.debug("no exact match ");

	// Check for superset. 
	ObjectData[] superSet = objComp.matchingSuperset(objData);
	boolean permissionTrue = false;
	// If its not null, then atleast one should have cas/enrollObject
	if (superSet != null) {
	    logger.debug("Super set is not null");
	    for (int i=0; i<superSet.length; i++) {
		if (hasEnrollObjPermission(userName, superSet[i])) {
		    permissionTrue = true;
		    break;
		}
	    }
	    if (!permissionTrue) {
		// FIXME maybe not throw exaception, just return false
		logger.debug("None of super set objects have cas/enrollObject");
		throw new CasDBException("Object already exists as super set "
					 + "and relevant permissions don't "
					 + "exist to create this oject");
	    }
	}
	logger.debug("Super set went thro'");

	// Check for subset
	// Every single object that is a subset shld have grantAll
	ObjectData[] subset = objComp.matchingSubset(objData);
	if (subset != null) {
	    logger.debug("Sub set is not null");
	    for (int i=0; i<subset.length; i++) {
		logger.debug("Subset " + subset[i].getObjectName());
		if (!hasGrantAllObjPermission(userName, subset[i])) {
		    // FIXME maybe not throw exaception, just return false
		    logger.debug("Subset member does not have permissions");
		    throw new CasDBException("Object already exists as subset"
					     + " and relevant permissions don't"
					     + " exist to create this oject");
		}
	    }
	}
	logger.debug("Subset went thro'");

	return true;
    }

    public static boolean canRemoveObject(String userName, String objName, 
					  String objNamespace) 
	throws CasDBException {
	
	try {
	    int objId = ObjectDataHandler.getObjectId(objName, objNamespace);
	    if (objId == -1) {
		logger.error("Object does not exist");
		throw new CasDBException("Object does not exist");
	    }
	    return canRemoveObjectId(userName, Integer.toString(objId));
	}
	catch (CasDBException exp) {
	    String err = "Could not ascertain remove object priviledges for "
		+ "user.";
	    logger.error(err + exp.toString());
	    throw new CasDBException(err + exp.getMessage(), exp);
	}
    }

    public static boolean canRemoveObjectId(String userName, String objId) 
	throws CasDBException {
	CasConstants.unenrollPolicy.setObjectSpec(objId.trim());
	CasConstants.unenrollPolicy.setObjectSpecDesc(
				    CasConstants.OBJECT_SPEC);
	try {
	    return PermissionsEvaluator.userHasPermissions(
				     userName, 
				     CasConstants.unenrollPolicy);
	}
	catch (CasDBException exp) {
	    String err = "Could not ascertain remove object priviledges for "
		+ "user.";
	    logger.error(err + exp.toString());
	    throw new CasDBException(err + exp.getMessage(), exp);
	}
    }

    public static boolean canCreateServiceType(String userName) 
	throws CasDBException {
	
	try {
	    return PermissionsEvaluator.userHasPermissions(
				     userName, 
				     CasConstants.serviceTypeCreationPolicy);
	}
	catch (CasDBException exp) {
	    String err = "Could not ascertain create service type priviledges "
		+ "for user.";
	    logger.error(err + exp.toString());
	    throw new CasDBException(err + exp.getMessage(), exp);
	}
    }

    public static boolean canDeleteServiceType(String userName, 
					       String serviceTypeName)
	throws CasDBException {
	
	CasConstants.unenrollPolicy.setObjectSpec(serviceTypeName.trim());
	CasConstants.unenrollPolicy.setObjectSpecDesc(
				    CasConstants.SERVICETYPE_SPEC);
	try {
	    return PermissionsEvaluator.userHasPermissions(
				     userName, 
				     CasConstants.unenrollPolicy);
	}
	catch (CasDBException exp) {
	    String err = "Could not ascertain delete service type priviledges "
		+ "for user.";
	    logger.error(err + exp.toString());
	    throw new CasDBException(err + exp.getMessage(), exp);
	}
    }

    public static boolean canGrant(String userName, String userGpName, 
				   CasObjectData objSpec, String objSpecDesc, 
				   String actionSpec, String actionSpecDesc) 
	throws CasDBException {

	String objString = PolicyDataHandler.getObjectSpecString(objSpec, 
								 objSpecDesc);
	return canGrant(userName, userGpName, objString, objSpecDesc, 
			actionSpec,actionSpecDesc);
    }

    public static boolean canGrant(String userName, String userGpName, 
				   String objSpec, String objSpecDesc, 
				   String actionSpec, String actionSpecDesc) 
	throws CasDBException {
	
	PolicyData policyData = new PolicyData();
	policyData.setActionSpec(actionSpec.trim());
	policyData.setActionSpecDesc(actionSpecDesc.trim());
	policyData.setObjectSpec(objSpec.trim());
	policyData.setObjectSpecDesc(objSpecDesc.trim());

	try {
	    if (!PermissionsEvaluator.userHasPermissions(userName, policyData)) {
		logger.warn("No permissions to perform the service/action that"
			     + " has to be granted");
		return false;
	    }
	    logger.debug("action is allowed, but grant not found");
	    return PermissionsEvaluator.userHasPermissions(
				     userName, 
				     CasConstants.grantPolicy);
	}
	catch (CasDBException exp) {
	    String err = "Could not ascertain grant rights priviledges for "
		+ "user.";
	    logger.error(err + exp.toString());
	    throw new CasDBException(err + exp.getMessage(), exp);
	}
    }

    public static boolean canRevoke(String userName, String policyId) 
	throws CasDBException {
	
	logger.debug("Policy id is " + policyId);

	if (policyId == null) {
	    logger.error("Policy id to check permission is null");
	    throw new CasDBException("Policy id to check permission is null");
	}

	CasConstants.revokePolicy.setObjectSpec(policyId.trim());
	CasConstants.revokePolicy.setObjectSpecDesc(
				  CasConstants.POLICY_SPEC);
	try {
	    return PermissionsEvaluator.userHasPermissions(
				     userName, 
				     CasConstants.revokePolicy);
	}
	catch (CasDBException exp) {
	    String err = "Could not ascertain revoke rights priviledges for "
		+ "user.";
	    logger.error(err + exp.toString());
	    throw new CasDBException(err + exp.getMessage(), exp);
	}
    }

    public static boolean canQuery(String userName) 
	throws CasDBException {

	try {
	    return (PermissionsEvaluator.userHasPermissions(
				      userName, 
				      CasConstants.queryPolicy)); 
	}
	catch (CasDBException exp) {
	    String err = "Could not ascertain query priviledges for user.";
	    logger.error(err + exp.toString());
	    throw new CasDBException(err + exp.getMessage(), exp);
	}
    }

    public static void grantAllPermission(String userGpName, String spec, 
					  String specDesc)
	throws CasDBException {
	CasConstants.grantAllPolicy.setUserGroupName(userGpName.trim());
	CasConstants.grantAllPolicy.setObjectSpec(spec.trim());
	CasConstants.grantAllPolicy.setObjectSpecDesc(specDesc.trim());
	PolicyDataHandler.storeObject(CasConstants.grantAllPolicy);
    }
    
    /**
     * Returns true if there is a semantically equivalent policy data 
     * in CAS databse. Super user is ignored, that is if the user is 
     * super user it is not considered, since this method is used to
     * evaluate external policies only
     */
    public static boolean isAuthorizedPolicy(String userName, 
					     PolicyData policyData)

	throws CasDBException {
	return PermissionsEvaluator.userHasPermissions(
				 userName.trim(),
				 policyData,
				 true);
    }
}
