/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.cas.impl.databaseAccess;

import java.sql.Connection;
import java.sql.Statement;
import java.sql.SQLException;
import java.sql.ResultSet;

import java.util.Vector;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.globus.cas.impl.CasConstants;

import org.globus.cas.types.UserData;
import org.globus.cas.types.ArrayOfString;
import org.globus.cas.types.CasObjectData;

import org.globus.util.I18n;

/**
 * User Data manipulation in the database
 */
public class UserDataHandler {

    static Log logger = LogFactory.getLog(UserDataHandler.class.getName() );
    
    private static I18n i18n =
        I18n.getI18n("org.globus.cas.impl.databaseAccess.errors",
                     UserDataHandler.class.getClassLoader());

    /**
     * Insert user data into user table 
     * Does not populate the user group entry table 
     * Does not create a new user group and will fail if the user group 
     * specified does not exist
     */
    public static String storeObject(UserData userData) throws CasDBException {

	Connection connection = CasDBStorage.getDBConnection();
	
	// Insert user table data
	String columnString = "insert into " + CasDBConstants.TABLE_USER 
            + " (" + CasDBConstants.COL_USER_NICKNAME + "," 
	    + CasDBConstants.COL_SUBJECTNAME + "," 
	    + CasDBConstants.COL_TRUST_NICKNAME + ") values ('";
	StringBuffer query = new StringBuffer(columnString);
	query.append(userData.getNickname()).append("','")
            .append(userData.getSubjectName()).append("','")
            .append(userData.getTrustAnchorName()).append("')");
	Statement statement = null;
	try {
	    statement = connection.createStatement();
	    logger.debug("store query is " + query.toString());
	    statement.executeUpdate(query.toString());
	}
	catch (SQLException exp) {
	    logger.error(i18n.getMessage("storeErr", new Object[] {
                "user ", query }), exp);
	    throw new CasDBException(i18n.getMessage("storeErr", new Object[] {
                "user data", exp.getMessage() }), exp);
	}
	finally {
	    // returning connection irrespective of whether stmt
	    // and result set are closed or not.
	    CasDBStorage.returnDBConnection(connection);
	    try {
		if (statement != null)
		    statement.close();
	    }
	    catch (SQLException exp) {
		logger.warn(i18n.getMessage("storeErrClose", "user data"),
                            exp);
	    }
	}
	return userData.getNickname();
    }

    /**
     * List of all users
     */
    public static String[] list() throws CasDBException {
	String query = "select" + CasDBConstants.COL_USER_NICKNAME + " from" 
	    + CasDBConstants.TABLE_USER;
	return CasDBStorage.runListQuery(query, 
					 CasDBConstants.COL_USER_NICKNAME);
    }

    /**
     * Retrieves user data
     * Picks up values from user table and user group entry table
     */
    public static CasObjectData retrieveObject(String userNickname) 
	throws CasDBException {

	logger.debug("retrieve Object called");
	UserData returnObject = null;

	Connection connection = CasDBStorage.getDBConnection();
	String query = "select" + CasDBConstants.COL_USERGP_NAME + " from" 
	    + CasDBConstants.TABLE_USERGP_ENTRY + " where " 
	    + CasDBConstants.COL_USER_NICKNAME + " ='" + userNickname.trim() 
	    + "'";
	logger.debug("query " + query);
	Vector groupNames = null;
	Statement statement = null;
	ResultSet resultSetGp = null;
	try {
	    statement = connection.createStatement();
	    resultSetGp = statement.executeQuery(query);
	    if ((resultSetGp!=null) && (resultSetGp.next())) {
		groupNames = new Vector(resultSetGp.getFetchSize());
		do {
		    groupNames.add((resultSetGp.getString(
			       CasDBConstants.COL_USERGP_NAME.trim())));
		} while (resultSetGp.next());
	    }
	}
	catch (SQLException exp) {
	    CasDBStorage.returnDBConnection(connection);
	    logger.error(i18n.getMessage("retrieveErr", new Object[] {
                "group data for user nickname " + userNickname, query}), exp);
	    throw new CasDBException(i18n.getMessage("retrieveErr", 
                       new Object[] { "group data for user nickname " 
                                      + userNickname, exp.getMessage()}), exp);
	}
	finally {
	    try {
		if (resultSetGp != null)
		    resultSetGp.close();
		if (statement != null)
		    statement.close();
	    }
	    catch (SQLException exp) {
		String err = "Error retrieving group data for user"
		    + " nickname " + userNickname;
		logger.warn(err, exp);
	    }
	}
	ArrayOfString arrayOfGpnames = null;
	if (groupNames != null) {
	    String groupNamesString[] = new String[groupNames.size()];
	    groupNames.toArray(groupNamesString);
            arrayOfGpnames = new ArrayOfString(groupNamesString);
	}

	query = "select * from" + CasDBConstants.TABLE_USER + " where" 
	    + CasDBConstants.COL_USER_NICKNAME + " = '" + userNickname.trim() 
	    + "'";
	ResultSet resultSet = null;
	try {
	    statement = connection.createStatement();
	    resultSet = statement.executeQuery(query);
	    if (resultSet!=null) {
		if (resultSet.next()) {
		   returnObject = new UserData();
		    returnObject.setNickname(
				 resultSet.getString(
				 CasDBConstants.COL_USER_NICKNAME.trim()));
		    returnObject.setSubjectName(
				 resultSet.getString(
				 CasDBConstants.COL_SUBJECTNAME.trim()));
		    returnObject.setTrustAnchorName(
				 resultSet.getString(
				 CasDBConstants.COL_TRUST_NICKNAME.trim()));
		    returnObject
                        .setGroupNames(arrayOfGpnames);
		}
	    }
	}
	catch (SQLException exp) {
	    logger.error(i18n.getMessage("retrieveErr", new Object[] {
                "user nickname " + userNickname, query }), exp);
	    throw new CasDBException(i18n.getMessage("retrieveErr", 
                      new Object[] { "user nickname " + userNickname, 
                                     exp.getMessage() }), exp);
	}
	finally {
	    // returning connection irrespective of whether stmt
	    // and result set are closed or not.
	    CasDBStorage.returnDBConnection(connection);
	    try {
		if (resultSet != null)
		    resultSet.close();
		if (statement != null)
		    statement.close();
	    }
	    catch (SQLException exp) {
		logger.warn(i18n.getMessage("retrieveErrClose", 
                                            "user nickname " + userNickname), 
                            exp);
	    }
	}

	return returnObject;
    }

    /**
     * Deletes user data
     * Manipulates user table. Entries in user group table for this user 
     * shld have been deleted
     */
    public static void deleteObject(String userNickname) 
        throws CasDBException {

	// User can be a member of object group, no SQL constraint
	// defines that. So check.
	if (ObjectGroupDataHandler.isMember(userNickname.trim(), 
					    CasConstants.USER_SPEC)) {
	    String errMesg = i18n.getMessage("memberOfGpErr", new Object[] {
                "User ", userNickname.trim() , "object"});
	    logger.debug(errMesg);
	    throw new CasDBException(errMesg);
	}

	Connection connection = CasDBStorage.getDBConnection();
	String query = "delete from" + CasDBConstants.TABLE_USER + " where" 
	    + CasDBConstants.COL_USER_NICKNAME + " ='" + userNickname.trim() 
	    + "'";
	Statement statement = null;
        int result;
	try {
	    statement = connection.createStatement();
	    result = statement.executeUpdate(query);
	}
	catch (SQLException exp) {
	    logger.error(i18n.getMessage("delObjErr", new Object[] {
                "user data with nickname " + userNickname, query }), exp);
	    throw new CasDBException(i18n.getMessage("delObjErr", 
                      new Object[] { "user data with nickname " + userNickname,
                                     exp.getMessage() }), exp);
	}
	finally {
	    // returning connection irrespective of whether stmt
	    // and result set are closed or not.
	    CasDBStorage.returnDBConnection(connection);
	    try {
		if (statement != null)
		    statement.close();
	    }
	    catch (SQLException exp) {
		logger.warn(i18n.getMessage("delObjErrClose", "user data"), 
                            exp);
	    }
	}
        if (result <=0) {
            String err = i18n.getMessage("userDelError", userNickname);
            logger.error(err);
            throw new CasDBException(err);
        }
    }

    /**
     * Retrives user nickname give subjectDN
     * 
     * @param subjectDN
     *        subject DN
     * @return user nickname if it exists in database, null otherwise
     */
    public static String getUserNickname(String subjectDN)
	throws CasDBException {
	
	String userNick = null;
	String query = "select" + CasDBConstants.COL_USER_NICKNAME + " from" 
	    + CasDBConstants.TABLE_USER + " where (" 
	    + CasDBConstants.COL_SUBJECTNAME + "='" + subjectDN.trim()
	    + "')";
	logger.debug("query is " + query);
	Connection connection = CasDBStorage.getDBConnection();
	Statement statement = null;
	ResultSet resultSet = null;
	try {
	    statement = connection.createStatement();
	    resultSet = statement.executeQuery(query);
	    if (resultSet!=null) {
		if (resultSet.next()) {
		    userNick = 
			resultSet.getString(
				  CasDBConstants.COL_USER_NICKNAME.trim());
		}
	    }
	}
	catch (SQLException exp) {
	    logger.error(i18n.getMessage("retrieveErr", new Object[] {
                "user nickname for subject DN " + subjectDN,
                query }), exp);
	    throw new CasDBException(i18n.getMessage("retrieveErr", 
                  new Object[] { "user nickname for subject DN " + subjectDN,
                                 exp.getMessage() }), exp);
	}
	finally {
	    // returning connection irrespective of whether stmt
	    // and result set are closed or not.
	    CasDBStorage.returnDBConnection(connection);
	    try {
		if (statement != null)
		    statement.close();
	    }
	    catch (SQLException exp) {
		logger.warn(i18n.getMessage("retrieveErrClose", 
                                            "user nickname"), exp);
	    }
	}
	logger.debug("userNick is " + userNick);
	return userNick;
    }
}
