/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.cas.impl.service;

import org.globus.cas.impl.CasConstants;

import org.globus.wsrf.Resource;
import org.globus.wsrf.ResourceProperty;
import org.globus.wsrf.ResourceProperties;
import org.globus.wsrf.ResourcePropertySet;

import org.globus.wsrf.impl.SimpleResourceProperty;
import org.globus.wsrf.impl.SimpleResourcePropertySet;

import org.globus.mds.usefulrp.ServiceMetaDataResourceProperty;

import javax.xml.namespace.QName;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class CasResource implements Resource, ResourceProperties {

    static Log logger = LogFactory.getLog(CasResource.class.getName());

    public static final javax.xml.namespace.QName RP_SET =
        new QName(CasConstants.CAS_NS, "CasRPSet");
    
    protected Object key;
    private ResourcePropertySet propSet;
    ResourceProperty serverDNProp = null;
    ServiceMetaDataResourceProperty metadata = null;    
    int maxAssertionLifetime = -1;
    String voDescription;
    String serverDN;

    public CasResource(int maxLifetime, String voDesc) {
        this.maxAssertionLifetime = maxLifetime;
        this.voDescription = voDesc;
        this.propSet = new SimpleResourcePropertySet(RP_SET);
        ResourceProperty prop = 
            new SimpleResourceProperty(new QName(CasConstants.CAS_NS, 
                                                 "VoDescription"));
        prop.add(this.voDescription);
        this.propSet.add(prop);
        serverDNProp = 
            new SimpleResourceProperty(new QName(CasConstants.CAS_NS, 
                                                 "ServerDN"));
        this.propSet.add(serverDNProp);
        
        // add a metadata RP
        try {
            metadata=new ServiceMetaDataResourceProperty();
            this.propSet.add(metadata);
        } catch(Exception e) {
            logger.warn(e);
        }          
    }

    // FIXME : Set RPs
    public void setRPs() {
    }

    public ResourcePropertySet getResourcePropertySet() {
        return this.propSet;
    }

    public int getMaxAssertionLifetime() {
        return this.maxAssertionLifetime;
    }

    public String getVoDescription() {
        return this.voDescription;
    }

    public String getServerDN() {
        return this.serverDN;
    }

    public void setServerDN(String serverDN) {
        this.serverDN = serverDN;
        if (this.serverDNProp != null) {
            this.serverDNProp.add(this.serverDN);
        }
    }
    
    public void setVersion(String version) {
        this.metadata.setVersion(version);
    }    
}
