/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.cas.impl.databaseAccess;

import java.sql.Connection;
import java.sql.Statement;
import java.sql.SQLException;

import java.io.IOException;
import java.io.FileInputStream;
import java.io.FileNotFoundException;

import java.util.Properties;

public class DatabaseDelete {

    public static void main(String[] args) throws Exception {

        if (args.length < 1) {
            System.out.println("Usage: DatabaseDelete dbPropFile");
            System.exit(0);
        }

        Properties prop = new Properties();
        try {
            prop.load(new FileInputStream(args[0]));
        } catch (FileNotFoundException fnfe) {
            System.err.println("Error: Database properties file " 
                               + args[0] + " not found");
            System.exit(-1);
        } catch (IOException ioe) {
            System.err.println("Error loading file " + args[0] + "\n"
                               + ioe.getMessage());
            System.exit(-1);
        }

        CasDBOptions casDbOption = 
            new CasDBOptions(prop.getProperty("dbDriver"), 
                             prop.getProperty("dbConnectionURL"),
                             prop.getProperty("dbUsername"), 
                             prop.getProperty("dbPassword"));
        String val = prop.getProperty("activeConnections");
        if (val != null) {
            casDbOption.setActiveConnections((new Integer(val)).intValue());
        }

        val = prop.getProperty("onExhaustAction");
        if (val != null) {
            casDbOption.setOnExhaustAction((new Integer(val)).byteValue());
        }

        val = prop.getProperty("maxWait");
        if (val != null) {
            casDbOption.setMaxWait((new Integer(val)).longValue());
        }

        val = prop.getProperty("idleConnections");
        if (val != null) {
            casDbOption.setIdleConnections((new Integer(val)).intValue());
        }
        
        try {
            CasDBStorage.setupDBConnection(casDbOption);
        } catch (CasDBException exp) {
            System.err.println("Erorr initializing database " 
                               + exp.getMessage());
            System.exit(-1);
        }
        try {
            deleteAllEntries();
        } catch (CasDBException exp) {
            System.err.println("Erorr deleting entries in database " 
                               + exp.getMessage());
            System.exit(-1);
        }

        System.out.println("Database entries succesfully deleted");
    }
    
    private static void deleteAllEntries() throws CasDBException {
        
        Connection connection = CasDBStorage.getDBConnection();
        Statement statement = null;
        try {
            statement = connection.createStatement();
            statement.addBatch("delete from " + CasDBConstants.TABLE_OBJECT);
            statement.addBatch("delete from " + 
                               CasDBConstants.TABLE_OBJECTGP_ENTRY);
            statement.addBatch("delete from " + CasDBConstants.TABLE_OBJECTGP);
            statement.addBatch("delete from " 
                               + CasDBConstants.TABLE_NAMESPACE);
            statement.addBatch("delete from " 
                               + CasDBConstants.TABLE_USERGP_ENTRY);
            statement.addBatch("delete from " + CasDBConstants.TABLE_USERGP);
            statement.addBatch("delete from " + CasDBConstants.TABLE_USER);
            statement.addBatch("delete from " 
                               + CasDBConstants.TABLE_TRUSTANCHOR);
            statement.addBatch("delete from " + 
                               CasDBConstants.TABLE_SERVICEACTIONGP_ENTRY);
            statement.addBatch("delete from " + 
                               CasDBConstants.TABLE_SERVICEACTIONGP);
            statement.addBatch("delete from " + 
                               CasDBConstants.TABLE_SERVICETYPE_ACTION);
            statement.addBatch("delete from " 
                               + CasDBConstants.TABLE_SERVICETYPE);
            statement.addBatch("delete from " + CasDBConstants.TABLE_POLICY);
            // sequence
            statement.addBatch("drop sequence service_action_seq");
            statement.addBatch("drop sequence object_seq");
            statement.addBatch("drop sequence policy_seq");
            statement.addBatch("create sequence service_action_seq");
            statement.addBatch("create sequence object_seq");	
            statement.addBatch("create sequence policy_seq");	
            statement.executeBatch();
        } catch (SQLException exp) {
            throw new CasDBException(exp.getMessage(), exp);
        } finally {
            CasDBStorage.returnDBConnection(connection);
            try {
                if (statement != null)
                    statement.close();
            }
            catch (SQLException exp) {
                System.err.println("Error closing stament");
            }
        }
    }
}
