/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.cas.impl.databaseAccess;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.util.Vector;

import org.globus.cas.types.ObjectData;
import org.globus.cas.types.NamespaceData;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class TestObjectComparison extends TestCase {

    static Log logger =
              LogFactory.getLog(TestObjectComparison.class.getName());

    public TestObjectComparison(String name){
	super(name);
    }

    public static Test suite() {
        return new TestSuite(TestObjectComparison.class);
    }

    public void testObjectComparison() throws Exception {

	WildCardComparison wildCard = new WildCardComparison();
	ExactComparison exact = new ExactComparison();

	String namespace1 = "namespace1";
	// Store name space data
	NamespaceData nsData = new NamespaceData();
	nsData.setNickname(namespace1);
	nsData.setBasename("basename");
	nsData.setComparisonAlg("ExactComparison");
	NamespaceDataHandler.storeObject(nsData);
	logger.debug("Store namespace went through " + nsData.toString());
	
	String namespace2 = "namespace2";
	nsData.setNickname(namespace2);
	nsData.setBasename("basename");
	nsData.setComparisonAlg("WildCardComparison");
	NamespaceDataHandler.storeObject(nsData);
	logger.debug("Store namespace went through " + nsData.toString());

	// Test wild card policy
	// WC Policy 1
	ObjectData wildCardPolicy = new ObjectData();
	wildCardPolicy.setObjectName("foo/*");
	wildCardPolicy.setObjectNamespace(namespace2);
	String w1 = ObjectDataHandler.storeObject(wildCardPolicy);

	// WC: equals
	ObjectData reqData1 = new ObjectData();
	reqData1.setObjectName("foo/*");
	reqData1.setObjectNamespace(namespace2);
	assertTrue(wildCard.objectMatches(wildCardPolicy, reqData1));
	assertTrue(wildCard.objectExists(reqData1));

	// WC is not equal
	reqData1.setObjectName("foo/");
	reqData1.setObjectNamespace(namespace2);
	assertTrue(!wildCard.objectMatches(wildCardPolicy, reqData1));
	assertTrue(!wildCard.objectExists(reqData1));

	// WC: is subset
	reqData1 = new ObjectData();
	reqData1.setObjectName("foo/ab/c");
	reqData1.setObjectNamespace(namespace2);
	assertTrue(wildCard.objectMatches(wildCardPolicy, reqData1));
	assertTrue(wildCard.objectExists(reqData1));

	// WC: not subset
	reqData1.setObjectName("foo/*/c");
	reqData1.setObjectNamespace(namespace2);
	assertTrue(wildCard.objectMatches(wildCardPolicy, reqData1));
	assertTrue(wildCard.objectExists(reqData1));
	
	// WC Policy 2 
	ObjectData wildCardPolicy1 = new ObjectData();
	wildCardPolicy1.setObjectName("abc/abc/c");
	wildCardPolicy1.setObjectNamespace(namespace2);
	String w2 = ObjectDataHandler.storeObject(wildCardPolicy1);

	ObjectDataHandler.deleteObjectForId(w1);

	// WC: Matches is false, exists is true. 
	reqData1 = new ObjectData();
	reqData1.setObjectName("abc/abc/*");
	reqData1.setObjectNamespace(namespace2);
	assertTrue(!wildCard.objectMatches(wildCardPolicy1, reqData1));
	assertTrue(wildCard.objectExists(reqData1));

	// * in the middle does not matter.
	reqData1.setObjectName("abc/*/c");
	reqData1.setObjectNamespace(namespace2);
	assertTrue(!wildCard.objectMatches(wildCardPolicy1, reqData1));
	assertTrue(!wildCard.objectExists(reqData1));

	// WC Policy 3
	wildCardPolicy1.setObjectName("foo/*/c/*");
	wildCardPolicy1.setObjectNamespace(namespace2);
	String w3 = ObjectDataHandler.storeObject(wildCardPolicy1);
	
	reqData1.setObjectName("foo/*/c");
	reqData1.setObjectNamespace(namespace2);
	assertTrue(!wildCard.objectMatches(wildCardPolicy1, reqData1));
	assertTrue(!wildCard.objectExists(reqData1));
	
	ObjectData exactPolicy = new ObjectData();
	exactPolicy.setObjectName("aa*/b*/*/cc");
	exactPolicy.setObjectNamespace(namespace1);
	String w4 = ObjectDataHandler.storeObject(exactPolicy);

 	ObjectData reqData3 = new ObjectData();
	reqData3.setObjectName("a*/b*/*/cc");
	reqData3.setObjectNamespace(namespace1);
 	assertTrue(!exact.objectMatches(exactPolicy, reqData3));
	assertTrue(!wildCard.objectExists(reqData3));

	reqData3.setObjectName("aa*/b*/*/cc");
	reqData3.setObjectNamespace(namespace1);
 	assertTrue(exact.objectMatches(exactPolicy, reqData3));
	assertTrue(wildCard.objectExists(reqData3));

	ObjectDataHandler.deleteObjectForId(w2);
	ObjectDataHandler.deleteObjectForId(w3);
	ObjectDataHandler.deleteObjectForId(w4);
	NamespaceDataHandler.deleteObject(namespace1);
	NamespaceDataHandler.deleteObject(namespace2);
    }

    public void testMatchingSets() throws Exception {
	
	WildCardComparison wildCard = new WildCardComparison();
	
	String namespace = "matchNS1";
	// Store name space data
	NamespaceData nsData = new NamespaceData();
	nsData.setNickname(namespace);
	nsData.setBasename("basename");
	nsData.setComparisonAlg("ExactComparison");
	NamespaceDataHandler.storeObject(nsData);

	String namespace2 = "matchNS2";
	nsData.setNickname(namespace2);
	nsData.setBasename("basename");
	nsData.setComparisonAlg("WildCardComparison");
	NamespaceDataHandler.storeObject(nsData);

	ObjectData wildCardPolicy1 = new ObjectData();
	wildCardPolicy1.setObjectName("foo/*");
	wildCardPolicy1.setObjectNamespace(namespace2);

	ObjectData wildCardPolicy2 = new ObjectData();
	wildCardPolicy2.setObjectName("foo/bar/*");
	wildCardPolicy2.setObjectNamespace(namespace2);

	ObjectData wildCardPolicy3 = new ObjectData();
	wildCardPolicy3.setObjectName("foo/bar/d1");
	wildCardPolicy3.setObjectNamespace(namespace2);

	ObjectData wildCardPolicy4 = new ObjectData();
	wildCardPolicy4.setObjectNamespace(namespace2);
	wildCardPolicy4.setObjectName("foo/bar/d2");

	ObjectData wildCardPolicy5 = new ObjectData();
	wildCardPolicy5.setObjectNamespace(namespace2);
	wildCardPolicy5.setObjectName("foo/bar/d2/*");

	ObjectData wildCardPolicy6 = new ObjectData();
	wildCardPolicy6.setObjectNamespace(namespace2);
	wildCardPolicy6.setObjectName("foo/bar/d3");

	ObjectData wildCardPolicy7 = new ObjectData();
	wildCardPolicy7.setObjectNamespace(namespace2);
	wildCardPolicy7.setObjectName("foo");

	ObjectData wildCardPolicy8 = new ObjectData();
	wildCardPolicy8.setObjectNamespace(namespace2);
	wildCardPolicy8.setObjectName("bar");

	Vector dbObjects = new Vector(8);
	dbObjects.add(wildCardPolicy1);
	dbObjects.add(wildCardPolicy2);	
	dbObjects.add(wildCardPolicy3);
	dbObjects.add(wildCardPolicy4);
	dbObjects.add(wildCardPolicy5);
	dbObjects.add(wildCardPolicy6);
	dbObjects.add(wildCardPolicy7);
	dbObjects.add(wildCardPolicy8);

	// Subset
	// Check for foo/bar/*
	ObjectData wildCardPolicy = new ObjectData();
	wildCardPolicy.setObjectNamespace(namespace2);
	wildCardPolicy.setObjectName("foo/bar/*");
	ObjectData[] objData = wildCard.matchForSubset(dbObjects, 
						       wildCardPolicy);
	assertTrue(objData != null);
	assertTrue(objData.length == 4);
	String[] validStrings = {"foo/bar/d1","foo/bar/d2", "foo/bar/d2/*", 
				 "foo/bar/d3"};
	assertTrue(verifySet(objData, validStrings));
	
	// chekc for foo/bar/
	wildCardPolicy.setObjectName("foo/bar");
	objData = wildCard.matchForSubset(dbObjects, wildCardPolicy);
	assertTrue(objData == null);

	// chekc for foo/bar/d3
	wildCardPolicy.setObjectName("foo/bar/d2/*");
	objData = wildCard.matchForSubset(dbObjects, wildCardPolicy);
	assertTrue(objData == null);
	
	// check for foo/*
	wildCardPolicy.setObjectName("foo/*");
	objData = wildCard.matchForSubset(dbObjects, wildCardPolicy);
	assertTrue(objData != null);
	String[] validString = {"foo/bar/d1","foo/bar/d2", "foo/bar/d2/*", 
				 "foo/bar/d3", "foo/bar/*" };
	assertTrue(verifySet(objData, validString));
	
	// Superset
	// Check for foo/bar/*
	wildCardPolicy.setObjectName("foo/bar/*");
	ObjectData[] objData1 = wildCard.matchForSuperset(dbObjects, 
							 wildCardPolicy);
	assertTrue(objData1 != null);
	assertTrue(objData1.length == 1);
	String[] validStr = {"foo/*"};
	assertTrue(verifySet(objData1, validStr));

	NamespaceDataHandler.deleteObject(namespace);
	NamespaceDataHandler.deleteObject(namespace2);

    }

    private boolean verifySet(ObjectData[] objData, 
			      String[] validString) 
	throws Exception {
	for (int i=0; i<objData.length; i++) {
	    String objName = objData[i].getObjectName();
	    int j=0;
	    while (j <validString.length) {
		if (objName.equals(validString[j])) {
		    logger.debug(objName + " " + validString[j]);
		    break;
		}
		j++;
	    }
	    if (j == validString.length)
		return false;
	}
	return true;
    }
}
