/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.cas.impl.databaseAccess;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.globus.cas.impl.CasConstants;

import org.globus.cas.types.UserData;
import org.globus.cas.types.ObjectData;
import org.globus.cas.types.PolicyData;
import org.globus.cas.types.NamespaceData;
import org.globus.cas.types.UserGroupData;
import org.globus.cas.types.ObjectGroupData;
import org.globus.cas.types.ServiceTypeData;
import org.globus.cas.types.TrustAnchorData;

import org.globus.cas.utils.CasStringUtils;

import java.util.Vector;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class TestPolicyData extends TestCase {

    static Log logger =
              LogFactory.getLog(TestPolicyData.class.getName());

    String externalObjId = null;
    String externalObjName = null;
    String externalObjNS = null;
    String externalServiceType = null;
    String addActionId = null;
    String createActionId = null;
    String externalPolicy = null;
    String compAlg = "ExactComparison";
    String trustNick  = "nickname";
    String userNick ="userNick";
    String rootGroup = "root";
    String groupName1 = "userGroup1";

    public TestPolicyData(String name){
	super(name);
    }

    public static Test suite() {
        return new TestSuite(TestPolicyData.class);
    }

    public void testPermissions() throws Exception {
	initializeUserData();
	policyDataTest();
	objectPermissionsTest();
        try {
            UserGroupDataHandler.removeGroupMember(rootGroup, userNick);
        } catch (CasDBException ex) {
            if (ex.getMessage().indexOf("may not exist") == -1) {
                throw ex;
            }
        }
	UserDataHandler.deleteObject(userNick);
	UserGroupDataHandler.deleteObject(rootGroup);
	UserGroupDataHandler.deleteObject(groupName1);
	TrustAnchorDataHandler.deleteObject(trustNick);
    }

    public void initializeUserData() throws Exception {

	String authMethod = "authMethod";
	String authData ="authData";

	// Store trust anchor data
	TrustAnchorData trustAnchor = new TrustAnchorData();
	trustAnchor.setNickname(trustNick);
	trustAnchor.setAuthMethod(authMethod);
	trustAnchor.setAuthData(authData);
	TrustAnchorDataHandler.storeObject(trustAnchor);
	logger.debug("Store object went through " + trustAnchor.toString());

	// Store user data 
	String subjectName = "userSubjectName";
	UserData userData = new UserData();
	userData.setNickname(userNick);
	userData.setSubjectName(subjectName);
	userData.setTrustAnchorName(trustNick);
	UserDataHandler.storeObject(userData);
	logger.debug("Store user data went through " + userData.toString());
    }

    public void policyDataTest() throws Exception {

	PolicyData policyData = new PolicyData();
	policyData.setUserGroupName(groupName1);
	policyData.setActionSpec("-1");
	policyData.setActionSpecDesc("serviceAction");
	policyData.setObjectSpec("-1");
	policyData.setObjectSpecDesc("object");

	// test without usergroup
	boolean expBool = false;
	try {
	    PolicyDataHandler.storeObject(policyData);
	}
	catch (CasDBException e) {
	    if ((e.getMessage()).indexOf(groupName1 
                                         + "\" does not exist") != -1)
		expBool = true;
            else
                logger.error(e);
	}
	assertTrue(expBool);

	// adding user group
	UserGroupData userGpData1 = new UserGroupData();
	userGpData1.setGroupName(groupName1);
	UserGroupDataHandler.storeObject(userGpData1);
	logger.debug("Store user group data went through " 
		     + userGpData1.toString());



	// adding user group
	String groupName = "userGroup";
	UserGroupData userGpData = new UserGroupData();
	userGpData.setGroupName(groupName);
	UserGroupDataHandler.storeObject(userGpData);
	logger.debug("Store user group data went through " 
		     + userGpData.toString());

	String nsNickname = "nickname";
	String basename = "basename1";

	// Store name space data
	NamespaceData nsData = new NamespaceData();
	nsData.setNickname(nsNickname);
	nsData.setBasename(basename);
	nsData.setComparisonAlg(compAlg);
	NamespaceDataHandler.storeObject(nsData);
	logger.debug("Store object went through " + nsData.toString());

	// adding an object without group
	String objectName = "objectName";
	String objectNamespace = nsNickname;
	ObjectData objectData = new ObjectData();
	objectData.setObjectName(objectName);
	objectData.setObjectNamespace(objectNamespace);
	String objectId = ObjectDataHandler.storeObject(objectData);
	assertTrue(objectId!=null);
	logger.debug("Store  object data went through id " + objectId + " " 
		     + objectData.toString());

	// Adding object group data
	String objectGroupName = "objectGroupName";
	ObjectGroupData objGroup = new ObjectGroupData();
	objGroup.setGroupName(objectGroupName);
	String objGroupId = ObjectGroupDataHandler.storeObject(objGroup);
	assertTrue(objGroupId != null);

	// serviceType name
	String serviceTypeName = "serviceTypeName";
	ServiceTypeData serviceTypeData = new ServiceTypeData();
	serviceTypeData.setName(serviceTypeName);
	String serviceId = ServiceTypeDataHandler.storeObject(serviceTypeData);
	assertTrue(serviceId!=null);
	logger.debug("Store  service type data went through");

	String actionId = "-1";
	policyData.setActionSpec(actionId);
	policyData.setActionSpecDesc("serviceAction");
	policyData.setObjectSpec(objectId);
	policyData.setObjectSpecDesc("object");
	// Test without service/action
	expBool = false;
	try {
	    PolicyDataHandler.storeObject(policyData);
	}
	catch (CasDBException e) {
	    if ((e.getMessage())
                .indexOf("does not exist in \"service action\" table") != -1)
		expBool = true;
            else
                logger.error(e);
	}
	assertTrue(expBool);

	String actionName1 = "actionName1";
	// add action mappings
	actionId = ServiceTypeActionHandler.addServiceActionMapping(
					    serviceTypeName, actionName1);
	assertTrue(actionId!=null);
	logger.debug("serviceType/actionName added");
	
	// create service_action group
	String serviceGpName = "serviceGroupName";
	ServiceTypeActionHandler.createServiceActionGroup(serviceGpName);
	logger.debug("created action group");

	policyData.setActionSpec(actionId);
	String policyId = PolicyDataHandler.storeObject(policyData);
	policyData.setPolicyId(policyId);
	PolicyData retrievedData = (PolicyData)PolicyDataHandler.retrieveObject(
								 policyId);
	assertTrue(retrievedData!=null);
	logger.debug("store and retrieve went through");
	assertTrue(retrievedData.equals(policyData));

	// storing with s/a gp and obj gp
	retrievedData = null;
	policyData = new PolicyData();
	policyData.setUserGroupName(groupName1);
	policyData.setActionSpec(serviceGpName);
	policyData.setActionSpecDesc("serviceActionGroup");
	policyData.setObjectSpec(objectGroupName);
	policyData.setObjectSpecDesc("objectGroup");
	String policyId1 = PolicyDataHandler.storeObject(policyData);
	assertTrue(policyId1!=null);
	policyData.setPolicyId(policyId1);
	retrievedData = (PolicyData)PolicyDataHandler.retrieveObject(policyId1);
	assertTrue(retrievedData!=null);
	logger.debug("store and retrieve went through");
	assertTrue(retrievedData.equals(policyData));

	// storing with user as objectspec
	policyData.setObjectSpec(userNick);
	policyData.setObjectSpecDesc("user");
	String policyIdUser = PolicyDataHandler.storeObject(policyData);
	assertTrue(policyIdUser!=null);
	policyData.setPolicyId(policyIdUser);
	retrievedData = (PolicyData)PolicyDataHandler.retrieveObject(
						      policyIdUser);
	assertTrue(retrievedData.equals(policyData));
	
	// storing with userGp as objectspec
	policyData.setObjectSpec(groupName1);
	policyData.setObjectSpecDesc("userGroup");
	String policyIdUserGp = PolicyDataHandler.storeObject(policyData);
	policyData.setPolicyId(policyIdUserGp);
	assertTrue(policyIdUserGp!=null);
	retrievedData = 
	    (PolicyData)PolicyDataHandler.retrieveObject(policyIdUserGp);
	assertTrue(retrievedData.equals(policyData));

	// storing with serviceType as objectspec
	policyData.setObjectSpec(serviceTypeName);
	policyData.setObjectSpecDesc("serviceType");
	String policyIdService = PolicyDataHandler.storeObject(policyData);
	assertTrue(policyIdService!=null);
	policyData.setPolicyId(policyIdService);
	retrievedData = (PolicyData)PolicyDataHandler.retrieveObject(
						      policyIdService);
	assertTrue(retrievedData.equals(policyData));

	// storing with namespace as objectspec
	policyData.setObjectSpec(nsNickname);
	policyData.setObjectSpecDesc("namespace");
	String policyIdNs = PolicyDataHandler.storeObject(policyData);
	assertTrue(policyIdNs!=null);
	policyData.setPolicyId(policyIdNs);
	retrievedData = 
	    (PolicyData)PolicyDataHandler.retrieveObject(policyIdNs);
	assertTrue(retrievedData.equals(policyData));

	// storing with trustAnchor as objectspec
	policyData.setObjectSpec(trustNick);
	policyData.setObjectSpecDesc("trustAnchor");
	String policyIdAnchor = PolicyDataHandler.storeObject(policyData);
	assertTrue(policyIdAnchor!=null);
	policyData.setPolicyId(policyIdAnchor);
	retrievedData = 
	    (PolicyData)PolicyDataHandler.retrieveObject(policyIdAnchor);
	assertTrue(retrievedData.equals(policyData));

	// Set cas/superuser on casServer 
	ServiceTypeData serTypeData = new ServiceTypeData();
	serTypeData.setName("cas");
	ServiceTypeDataHandler.storeObject(serTypeData);

	// Add mapping 
	String serviceActionId = 
	    ServiceTypeActionHandler.addServiceActionMapping("cas","superuser");

	// Add mapping 
	String grantAllId = 
	    ServiceTypeActionHandler.addServiceActionMapping("cas","grantAll");

	// storing with serviceActionGp as objectspec
	policyData.setActionSpec(grantAllId);
	policyData.setActionSpecDesc("serviceAction");
	policyData.setObjectSpec(serviceGpName);
	policyData.setObjectSpecDesc("serviceActionGroup");
	String policyIdServiceGp = PolicyDataHandler.storeObject(policyData);
	assertTrue(policyIdServiceGp!=null);
	policyData.setPolicyId(policyIdServiceGp);
	retrievedData = 
	    (PolicyData)PolicyDataHandler.retrieveObject(policyIdServiceGp);
	assertTrue(retrievedData.equals(policyData));

	// verify list method
	String listPolicy[] = PolicyDataHandler.list();
	assertTrue(CasStringUtils.stringInArray(listPolicy, policyIdServiceGp));

	// verify get policy id method
	String returnedPolicyId = PolicyDataHandler.getPolicyId(policyData);
	assertTrue(policyIdServiceGp != returnedPolicyId);

	// exception with wrong objectSpec type
	policyData.setObjectSpec(serviceGpName);
	policyData.setObjectSpecDesc("type");
	expBool = false;
	try {
	    PolicyDataHandler.storeObject(policyData);
	}
	catch (CasDBException e) {
	    if ((e.getMessage()).indexOf("Object spec is not set correctly") 
		!= -1)
		expBool = true;
	}
	assertTrue(expBool);
	
	// exception with wrong actionSpec type
	policyData.setActionSpec("-1");
	policyData.setActionSpecDesc("user");
	expBool = false;
	try {
	    PolicyDataHandler.storeObject(policyData);
	}
	catch (CasDBException e) {
	    if ((e.getMessage()).indexOf("Action spec is not set correctly") != 
		-1)
		expBool = true;
	}
	assertTrue(expBool);

	// Permission tests
	PolicyData p1 = new PolicyData();
	p1.setUserGroupName(groupName1);
	p1.setActionSpec(actionId);
	p1.setActionSpecDesc("serviceAction");
	p1.setObjectSpec(objectId);
	p1.setObjectSpecDesc("object");
	assertTrue(PermissionsEvaluator.hasPermissions(p1, false));
	logger.debug("P1 through!");

	PolicyData p2 = new PolicyData();
	p2.setUserGroupName(groupName1);
	p2.setObjectSpec("objectGroupName");
	p2.setObjectSpecDesc("objectGroup");
	p2.setActionSpec("1");
	p2.setActionSpecDesc("serviceAction");
	assertTrue(!PermissionsEvaluator.hasPermissions(p2, false));
	logger.debug("P2 through!");
				       
	// Add userNick to group
	UserGroupDataHandler.addGroupMember(groupName, "userNick");
	PolicyData p3 = new PolicyData();
	p3.setUserGroupName(groupName1);
	p3.setObjectSpec("objectGroupName");
	p3.setObjectSpecDesc("objectGroup");
	p3.setActionSpec("serviceGroupName");
	p3.setActionSpecDesc("serviceActionGroup");
	assertTrue(PermissionsEvaluator.hasPermissions(p3, false));
	assertTrue(!PermissionsEvaluator.userHasPermissions(userNick,p3));
	UserGroupDataHandler.addGroupMember(groupName1, userNick);
	assertTrue(PermissionsEvaluator.userHasPermissions(userNick, p3));
	logger.debug("P3 through!");

	// add an external object and gives add permission to usergroup1
	setupExternalObjects(groupName1);

	// Store namespace
	nsData = new NamespaceData();
	nsData.setNickname("casNamespace");
	nsData.setBasename("basename2");
	nsData.setComparisonAlg(compAlg);
	NamespaceDataHandler.storeObject(nsData);

	// store casServer as object
	objectData = new ObjectData();
	objectData.setObjectName("casServer");
	objectData.setObjectNamespace("casNamespace");
	String casServerId = ObjectDataHandler.storeObject(objectData);

	String objGpGrantAll = "objectGroupGrantAll";
	ObjectGroupData objGroup1 = new ObjectGroupData();
	objGroup1.setGroupName(objGpGrantAll);
	objGpGrantAll = ObjectGroupDataHandler.storeObject(objGroup1);
	assertTrue(objGpGrantAll != null);

	ObjectGroupDataHandler.addGroupMember(objGpGrantAll, casServerId, 
					      "object");
	ObjectGroupDataHandler.addGroupMember(objGpGrantAll, externalObjId, 
					      "object");

	// A new usergroup for superuser

	UserGroupData rootGp = new UserGroupData();
	rootGp.setGroupName(rootGroup);
	UserGroupDataHandler.storeObject(rootGp);
	
	// Set grantAll
	logger.debug("grant all perm");
	policyData = new PolicyData();
	policyData.setUserGroupName(rootGroup);
	policyData.setObjectSpec(objGpGrantAll);
	policyData.setObjectSpecDesc("objectGroup");
	policyData.setActionSpec(grantAllId);
	policyData.setActionSpecDesc("serviceAction");
	String grantAllPolicy = PolicyDataHandler.storeObject(policyData);
	

	// Add mapping 
	String addEntryId = 
	    ServiceTypeActionHandler.addServiceActionMapping(
				     "cas","add_group_entry");

	// now permission to add to this group must fail for 
	// userNick since grantAll on rootGp
	PolicyData grantTestPolicy = new PolicyData();
	grantTestPolicy.setObjectSpec(objGpGrantAll);
	grantTestPolicy.setObjectSpecDesc("objectGroup");
	grantTestPolicy.setActionSpec(addEntryId);
	grantTestPolicy.setActionSpecDesc("serviceAction");
	assertTrue(!PermissionsEvaluator.userHasPermissions(
				      userNick,grantTestPolicy));

	// add userNick to root Gp to test if grantAll worked.
	UserGroupDataHandler.addGroupMember(rootGroup, userNick);
	assertTrue(PermissionsEvaluator.userHasPermissions(
				      userNick,grantTestPolicy));


	// remove user from gp 
	UserGroupDataHandler.removeGroupMember(rootGroup, userNick);

	// Set super user permissions
	policyData = new PolicyData();
	policyData.setUserGroupName(groupName1);
	policyData.setObjectSpec(casServerId);
	policyData.setObjectSpecDesc("object");
	policyData.setActionSpec(serviceActionId);
	policyData.setActionSpecDesc("serviceAction");
	String superUserPolicy = PolicyDataHandler.storeObject(policyData);

	// test external policy list
	Vector applicablePolicy = 
	    ExternalPolicyEvaluator.getPolicies(userNick);
	assertTrue(applicablePolicy != null);
	int policyNum = 0;
	for (int i=0; i<applicablePolicy.size(); i++) {
	    Vector objPolicy = (Vector)applicablePolicy.get(i);
	    displayPolicyVector(objPolicy);
	    assertTrue(ensureOnlyExternalPolicy(objPolicy));
	    policyNum = policyNum + objPolicy.size();
	}
	assertTrue(policyNum == 2);

	UserGroupDataHandler.addGroupMember(rootGroup, userNick);
	applicablePolicy = 
	    ExternalPolicyEvaluator.getPolicies(userNick);
	assertTrue(applicablePolicy != null);
	policyNum = 0;
	for (int i=0; i<applicablePolicy.size(); i++) {
	    Vector objPolicy = (Vector)applicablePolicy.get(i);
	    displayPolicyVector(objPolicy);
	    assertTrue(ensureOnlyExternalPolicy(objPolicy));
	    policyNum = policyNum + objPolicy.size();
	}
	assertTrue(policyNum == 3);
	UserGroupDataHandler.removeGroupMember(rootGroup, userNick);

	// add grantAll for trustAnchor and Namespace
	policyData.setUserGroupName(groupName1);
	policyData.setObjectSpec(trustNick);
	policyData.setObjectSpecDesc("trustAnchor");
	policyData.setActionSpec(grantAllId);
	policyData.setActionSpecDesc("serviceAction");
	String trustAnchorPolicy = PolicyDataHandler.storeObject(policyData);
	
	ObjectGroupDataHandler.addGroupMember(objectGroupName, trustNick, 
					      "trustAnchor");
	
	// test all policy for trustAnchor
	PolicyData[] allApplicablePolicy = 
	    PolicyQueryHandler.getAllApplicablePolicy(
			      CasConstants.TRUSTANCHOR_SPEC,
			      trustNick);
	assertTrue(allApplicablePolicy != null);
	logger.debug("all policies " + allApplicablePolicy.length);
	displayPolicyArray(allApplicablePolicy);
	assertTrue(allApplicablePolicy.length == 3);

	// Remove group member 
	ObjectGroupDataHandler.removeGroupMember(objectGroupName, trustNick, 
						 "trustAnchor");
	// delete grantAll policy for trust anchor
	PolicyDataHandler.deleteObject(trustAnchorPolicy);

	// test all policy for namespace
	policyData.setObjectSpec(nsNickname);
	policyData.setObjectSpecDesc("namespace");
	String nsPolicy = PolicyDataHandler.storeObject(policyData);

	ObjectGroupDataHandler.addGroupMember(objectGroupName, nsNickname, 
					      "namespace");

	// test all policy for namespace
	allApplicablePolicy = 
	    PolicyQueryHandler.getAllApplicablePolicy(
			      CasConstants.NAMESPACE_SPEC,
			      nsNickname);
	assertTrue(allApplicablePolicy != null);
	logger.debug("all policies " + allApplicablePolicy.length);
	displayPolicyArray(allApplicablePolicy);
	assertTrue(allApplicablePolicy.length == 3);
	// Remove group member 
	ObjectGroupDataHandler.removeGroupMember(objectGroupName, nsNickname, 
						 "namespace");
	// delete grantAll policy for trust anchor
	PolicyDataHandler.deleteObject(nsPolicy);

	// test all policy for user
	allApplicablePolicy = 
	    PolicyQueryHandler.getAllApplicablePolicy(
						     CasConstants.USER_SPEC,
						     userNick);
	logger.debug("all policies " + allApplicablePolicy.length);
	displayPolicyArray(allApplicablePolicy);
	assertTrue(allApplicablePolicy.length == 11);

	try {
	    allApplicablePolicy = 
	    PolicyQueryHandler.getAllApplicablePolicy(
						     CasConstants.USER_SPEC,
						     "DummyUser");
	}
	catch (CasDBException exp) {
	    expBool = true;
	}
	assertTrue(expBool);

	// test all policy for user group
	allApplicablePolicy = 
	    PolicyQueryHandler.getAllApplicablePolicy(
			      CasConstants.USERGP_SPEC,
			      groupName1);
	assertTrue(allApplicablePolicy != null);
	assertTrue(allApplicablePolicy.length == 10);

	expBool = false;
	try {
	    allApplicablePolicy = 
		PolicyQueryHandler.getAllApplicablePolicy(
				  CasConstants.USERGP_SPEC,
				  "DummyUserGp");
	}
	catch (CasDBException dbExp) {
	    expBool = true;
	}
	assertTrue(expBool);
	    
	// Add casServerId to objectGroup also
	ObjectGroupDataHandler.addGroupMember(objectGroupName, casServerId, 
					      "object");
	
	// test all policy for object
	allApplicablePolicy = 
	    PolicyQueryHandler.getAllApplicablePolicy(
			      CasConstants.OBJECT_SPEC,
			      casServerId);
	assertTrue(allApplicablePolicy != null);
	assertTrue(allApplicablePolicy.length == 3);

	expBool = false;
	try {
	    allApplicablePolicy = 
		PolicyQueryHandler.getAllApplicablePolicy(
				  CasConstants.OBJECT_SPEC,
				  "DummyObjectId");
	}
	catch (CasDBException dbExp) {
	    expBool = true;
	}
	assertTrue(expBool);
	    
	
	// Remove casServerId
	ObjectGroupDataHandler.removeGroupMember(objectGroupName, casServerId, 
					      "object");

	// test all policy for object group
	allApplicablePolicy = 
	    PolicyQueryHandler.getAllApplicablePolicy(
			      CasConstants.OBJECTGP_SPEC,
			      objectGroupName);
	assertTrue(allApplicablePolicy != null);
	System.out.println("applicable Policy " + allApplicablePolicy.length);
	assertTrue(allApplicablePolicy.length == 1);

	expBool = false;
	try {
	    allApplicablePolicy = 
		PolicyQueryHandler.getAllApplicablePolicy(
				  CasConstants.OBJECT_SPEC,
				  "DummyObjectId");
	}
	catch (CasDBException dbExp) {
	    expBool = true;
	}
	assertTrue(expBool);

	// test all policy for service type
	allApplicablePolicy = 
	    PolicyQueryHandler.getAllApplicablePolicy(
			      CasConstants.SERVICETYPE_SPEC,
			      serviceTypeName);
	assertTrue(allApplicablePolicy != null);
	assertTrue(allApplicablePolicy.length == 1);

	expBool = false;
	try {
	    allApplicablePolicy = 
		PolicyQueryHandler.getAllApplicablePolicy(
				  CasConstants.SERVICETYPE_SPEC,
				  "DummyServiceType");
	}
	catch (CasDBException dbExp) {
	    expBool = true;
	}
	assertTrue(expBool);

	// Add SampleCounter service as part of serviceGroupName	
	ServiceTypeActionHandler.addServiceActionGpEntry(serviceGpName,
							 externalServiceType,
							 "add");
	allApplicablePolicy = 
	    PolicyQueryHandler.getAllApplicablePolicy(
			      CasConstants.SERVICETYPE_SPEC,
			      externalServiceType);
	assertTrue(allApplicablePolicy == null);

	// Remove group member
	ServiceTypeActionHandler.removeServiceActionGpEntry(serviceGpName,
		externalServiceType, "add");

	// add cas/grantAll to the serviceAction gp
	ServiceTypeActionHandler.addServiceActionGpEntry(serviceGpName,
							 "cas",
							 "grantAll");
	// test all policy for service/action
	allApplicablePolicy = 
	    PolicyQueryHandler.getAllApplicablePolicy(
			      CasConstants.SERVICEACTION_SPEC,
			      grantAllId);
	assertTrue(allApplicablePolicy != null);
	logger.debug("applicable policy length " + allApplicablePolicy.length);
	assertTrue(allApplicablePolicy.length == 9 );

	ServiceTypeActionHandler.removeServiceActionGpEntry(serviceGpName,
							 "cas",
							 "grantAll");
	
	// test all policy for serviceActionGp
	allApplicablePolicy = 
	    PolicyQueryHandler.getAllApplicablePolicy(
			      CasConstants.SERVICEACTIONGP_SPEC,
			      serviceGpName);
	assertTrue(allApplicablePolicy != null);
	assertTrue(allApplicablePolicy.length == 7);

	expBool = false;
	try {
	    allApplicablePolicy = 
		PolicyQueryHandler.getAllApplicablePolicy(
				  CasConstants.SERVICEACTIONGP_SPEC,
				  "DummyServiceGp");
	}
	catch (CasDBException dbExp) {
	    expBool = true;
	}
	assertTrue(expBool);


	// test allpolicy for user group
	UserGroupDataHandler.addGroupMember(rootGroup, userNick);
	allApplicablePolicy = 
	    PolicyQueryHandler.getAllApplicablePolicy(
						     CasConstants.USER_SPEC,
						     userNick);
	assertTrue(allApplicablePolicy.length == 12);
	UserGroupDataHandler.removeGroupMember(rootGroup, userNick);

	// Test if request for some other permission goes thro' 
	PolicyData p4 = new PolicyData();
	p4.setUserGroupName("randomUser");
	p4.setObjectSpec("userB");
	p4.setObjectSpecDesc("user");
	p4.setActionSpec("1");
	p4.setActionSpecDesc("serviceAction");
	assertTrue(PermissionsEvaluator.userHasPermissions(userNick,p4));
	logger.debug("P4 through - superuser case");
	logger.debug("testing userHasPermissions ignoring superser");
	assertTrue(!PermissionsEvaluator.userHasPermissions(userNick,p4, true));

	// userNick is super user, but still external create operation check 
	// should fail
	PolicyData extPerm = new PolicyData();
	extPerm.setUserGroupName("randomUser");
	extPerm.setObjectSpec(externalObjId);
	extPerm.setObjectSpecDesc("object");
	extPerm.setActionSpec(createActionId);
	extPerm.setActionSpecDesc("serviceAction");
	assertTrue(!PermissionsEvaluator.userHasPermissions(userNick,extPerm,
							 true));

	// some permission to group userNick is not a member of
	policyData.setUserGroupName(rootGroup);
	policyData.setObjectSpec(externalObjId);
	policyData.setObjectSpecDesc("object");
	policyData.setActionSpec(createActionId);
	policyData.setActionSpecDesc("serviceAction");
	String rootPolicy = PolicyDataHandler.storeObject(policyData);

	String[] currentPolicyList = PolicyDataHandler.list();
	logger.debug("Current " + currentPolicyList.length);
	assertTrue(currentPolicyList.length > 0);
	// all policies, not super user
	applicablePolicy = 
	    ExternalPolicyEvaluator.getPolicies(userNick);
	assertTrue(applicablePolicy != null);
	policyNum = 0;
	for (int i=0; i<applicablePolicy.size(); i++) {
	    Vector objPolicy = (Vector)applicablePolicy.get(i);
	    displayPolicyVector(objPolicy);
	    assertTrue(ensureOnlyExternalPolicy(objPolicy));
	    policyNum = policyNum + objPolicy.size();
	}
	logger.debug("external only " + policyNum);
	assertTrue(policyNum > 0);
	assertTrue(policyNum < currentPolicyList.length);

	allApplicablePolicy = 
	    PolicyQueryHandler.getAllApplicablePolicy(
						     CasConstants.USER_SPEC,
						     userNick);
	logger.debug("all policies " + allApplicablePolicy.length);
	displayPolicyArray(allApplicablePolicy);
	assertTrue(allApplicablePolicy.length > 0);

	// checks for given action id and user, all policies.
	// add another external service
	String externalServiceType2 = "SampleCounter2";
	ServiceTypeData serviceTypeData1 = new ServiceTypeData();
	serviceTypeData1.setName(externalServiceType2);
	ServiceTypeDataHandler.storeObject(serviceTypeData1);
	
	// add mapping
	ServiceTypeActionHandler.addServiceActionMapping(
				 externalServiceType2, "add");

	// add service action group members.
	ServiceTypeActionHandler.addServiceActionGpEntry(serviceGpName,
							 externalServiceType2,
							 "add");

	ServiceTypeActionHandler.addServiceActionGpEntry(serviceGpName,
							 externalServiceType,
							 "create");
	
	// Policy using service group on external object
	policyData.setUserGroupName(groupName1);
	policyData.setObjectSpec(objectId);
	policyData.setObjectSpecDesc("object");
	policyData.setActionSpec(serviceGpName);
	policyData.setActionSpecDesc("serviceActionGroup");
	String serviceGpPolicy = PolicyDataHandler.storeObject(policyData);

	// Policy on external object 
	policyData.setObjectSpec(externalObjId);
	policyData.setObjectSpecDesc("object");
	policyData.setActionSpec(createActionId);
	policyData.setActionSpecDesc("serviceAction");
	String externalActionId = PolicyDataHandler.storeObject(policyData);
	
	Vector actionIds = new Vector();
	actionIds.add(createActionId);
	applicablePolicy = 
	    ExternalPolicyEvaluator.getPoliciesForActions(userNick, actionIds);

	assertTrue(applicablePolicy != null);
	assertTrue(applicablePolicy.size() == 2);
	for (int i=0; i<applicablePolicy.size(); i++) {
	    Vector temp = (Vector)applicablePolicy.get(i);
	    assertTrue(ensureOnlyExternalPolicy(temp));
	}

	// call for getPoliciesForResource
	String testResource = "nickname|objectName";
	applicablePolicy = 
	    ExternalPolicyEvaluator.getPoliciesForResource(userNick, 
							   testResource);
	assertTrue(applicablePolicy != null);
	assertTrue(applicablePolicy.size() == 1);
	Vector temp = (Vector)applicablePolicy.get(0);
	assertTrue(temp.size() == 3);
	assertTrue(ensureOnlyExternalPolicy(temp));
	
	// policy for objectGroupGrantAll
	policyData.setObjectSpec(objGpGrantAll);
	policyData.setObjectSpecDesc("objectGroup");
	policyData.setActionSpec(createActionId);
	policyData.setActionSpecDesc("serviceAction");
	String gpGrantAllId = PolicyDataHandler.storeObject(policyData);
	
	// call for getPoliciesForResource
	testResource = "SampleServiceNS|CounterService/sample1";
	applicablePolicy = 
	    ExternalPolicyEvaluator.getPoliciesForResource(userNick, 
							   testResource);
	assertTrue(applicablePolicy != null);
	assertTrue(applicablePolicy.size() == 1);
	temp = (Vector)applicablePolicy.get(0);
	assertTrue(ensureOnlyExternalPolicy(temp));
	assertTrue(temp.size() == 2);
	
	// call to getPolicy with user. resource and action specified.
	applicablePolicy =
	    ExternalPolicyEvaluator.getPolicies(userNick, testResource,
						actionIds);
	assertTrue(applicablePolicy != null);
	assertTrue(applicablePolicy.size() == 1);
	temp = (Vector)applicablePolicy.get(0);
	assertTrue(ensureOnlyExternalPolicy(temp));
	assertTrue(temp.size() == 1);

	// Remove policy added for getPolicyFor* method
	PolicyDataHandler.deleteObject(gpGrantAllId);
	PolicyDataHandler.deleteObject(externalActionId);
	PolicyDataHandler.deleteObject(serviceGpPolicy);
	ServiceTypeActionHandler.removeServiceActionGpEntry(serviceGpName,
							 externalServiceType,
							 "create");
	ServiceTypeActionHandler.removeServiceActionGpEntry(serviceGpName,
							 externalServiceType2,
							 "add");
	ServiceTypeActionHandler.removeServiceActionMapping(
				 externalServiceType2, 
				 "add");
	
	ServiceTypeDataHandler.deleteObject(externalServiceType2);

	// Remove grantAll
	PolicyDataHandler.deleteObject(grantAllPolicy);
	ServiceTypeActionHandler.removeServiceActionMapping(grantAllId);
	ServiceTypeActionHandler.removeServiceActionMapping(addEntryId);
	ObjectGroupDataHandler.removeGroupMember(objGpGrantAll, casServerId, 
						 "object");
	ObjectGroupDataHandler.removeGroupMember(objGpGrantAll, externalObjId, 
						 "object");
	ObjectGroupDataHandler.deleteObject(objGpGrantAll);

	// Remove superuser permissions
	PolicyDataHandler.deleteObject(superUserPolicy);
	
	// Store community as a user group
	UserGroupData uGpData = new UserGroupData();
	uGpData.setGroupName("community");
	UserGroupDataHandler.storeObject(uGpData);

	// store commObject as object
	ObjectData objData = new ObjectData();
	objData.setObjectName("commObject");
	objData.setObjectNamespace("nickname");
	objectId = ObjectDataHandler.storeObject(objData);
       
	// store commService/commAction
	serTypeData = new ServiceTypeData();
	serTypeData.setName("commService");

	ServiceTypeDataHandler.storeObject(serTypeData);

	String commActionId = ServiceTypeActionHandler.addServiceActionMapping(
						       "commService", 
						       "commAction");

	// Set community permissions
	policyData = new PolicyData();
	policyData.setUserGroupName("community");
	policyData.setObjectSpec(objectId);
	policyData.setObjectSpecDesc("object");
	policyData.setActionSpec(commActionId);
	policyData.setActionSpecDesc("serviceAction");
	String commPolicy = PolicyDataHandler.storeObject(policyData);
	
	PolicyData p5 = new PolicyData();
	p5.setObjectSpec(objectId);
	p5.setObjectSpecDesc("object");
	p5.setActionSpec(commActionId);
	p5.setActionSpecDesc("serviceAction");
	assertTrue(PermissionsEvaluator.userHasPermissions(userNick,p5));
	logger.debug("P5 through - community positve case");
	// Remove community permissions
	PolicyDataHandler.deleteObject(commPolicy);

	// Test same permission again
	assertTrue(!PermissionsEvaluator.hasPermissions(p5, false));

	// Test remove all policy data for a given object
	policyData.setObjectSpec(casServerId);
	policyData.setObjectSpecDesc("object");
	commPolicy = PolicyDataHandler.storeObject(policyData);

	policyData = new PolicyData();
	policyData.setUserGroupName(groupName1);
	policyData.setObjectSpec(casServerId);
	policyData.setObjectSpecDesc("object");
	policyData.setActionSpec(serviceActionId);
	policyData.setActionSpecDesc("serviceAction");
	superUserPolicy = PolicyDataHandler.storeObject(policyData);

	PolicyDataHandler.deletePolicyForObject(casServerId,"object");
	assertTrue(PolicyDataHandler.retrieveObject(superUserPolicy) == null);
	assertTrue(PolicyDataHandler.retrieveObject(commPolicy) == null);

	// clean up policy
	PolicyDataHandler.deleteObject(policyId1);
	// check for objectGroupName policies being null
	allApplicablePolicy = 
	    PolicyQueryHandler.getAllApplicablePolicy(
			      CasConstants.OBJECTGP_SPEC,
			      objectGroupName);
	assertTrue(allApplicablePolicy == null);

	PolicyDataHandler.deleteObject(policyIdAnchor);
	PolicyDataHandler.deleteObject(policyIdUser);
	PolicyDataHandler.deleteObject(policyIdUserGp);
	PolicyDataHandler.deleteObject(policyIdService);
	PolicyDataHandler.deleteObject(policyIdNs);
	PolicyDataHandler.deleteObject(policyIdServiceGp);
	PolicyDataHandler.deleteObject(policyId);	
	// serviceTypeName shld not have policies
	allApplicablePolicy = 
	    PolicyQueryHandler.getAllApplicablePolicy(
			      CasConstants.SERVICETYPE_SPEC,
			      serviceTypeName);
	assertTrue(allApplicablePolicy == null);

	PolicyDataHandler.deleteObject(externalPolicy);	
	
	// At this point no applicable policy for user
	allApplicablePolicy = 
	    PolicyQueryHandler.getAllApplicablePolicy(
						     CasConstants.USER_SPEC,
						     userNick);
	assertTrue(allApplicablePolicy == null);
	PolicyDataHandler.deleteObject(rootPolicy);	
	// At this point no applicable policy for userGroup
	allApplicablePolicy = 
	    PolicyQueryHandler.getAllApplicablePolicy(
			      CasConstants.USERGP_SPEC,
			      rootGroup);
	assertTrue(allApplicablePolicy == null);

	// clean up
	ServiceTypeActionHandler.removeServiceActionMapping(
				 "commService", "commAction");
	ServiceTypeDataHandler.deleteObject("commService");
	ObjectDataHandler.deleteObject("commObject", "nickname");
	UserGroupDataHandler.deleteObject("community");
	ObjectDataHandler.deleteObject("casServer", "casNamespace");
	NamespaceDataHandler.deleteObject("casNamespace");
	ServiceTypeActionHandler.removeServiceActionMapping("cas", "superuser");
	ServiceTypeActionHandler.removeServiceActionMapping(externalServiceType,
							    "add");
	ServiceTypeActionHandler.removeServiceActionMapping(externalServiceType,
							    "create");
	ServiceTypeDataHandler.deleteObject(externalServiceType);
	ServiceTypeDataHandler.deleteObject("cas");
	ServiceTypeActionHandler.deleteServiceActionGroup(serviceGpName);
	ServiceTypeActionHandler.removeServiceActionMapping(
				 serviceTypeName, actionName1);
	ServiceTypeDataHandler.deleteObject(serviceTypeName);
	ObjectGroupDataHandler.deleteObject(objectGroupName);
	ObjectDataHandler.deleteObject(objectName, objectNamespace);
	ObjectDataHandler.deleteObject(externalObjName, externalObjNS);
	NamespaceDataHandler.deleteObject(externalObjNS);
	NamespaceDataHandler.deleteObject(nsNickname);
	UserGroupDataHandler.removeGroupMember(groupName1, "userNick");
	UserGroupDataHandler.removeGroupMember(groupName, "userNick");
	UserGroupDataHandler.deleteObject(groupName);

	String emptyPolicy[] = PolicyDataHandler.list();
	assertTrue(emptyPolicy == null);
    }

    public void objectPermissionsTest() throws Exception {

	String namespace2 = "wildCard";
	// Store name space data
	NamespaceData nsData = new NamespaceData();
	nsData.setNickname(namespace2);
	nsData.setBasename("basename");
	nsData.setComparisonAlg("WildCardComparison");
	NamespaceDataHandler.storeObject(nsData);

	ObjectData wildCardPolicy1 = new ObjectData();
	wildCardPolicy1.setObjectName("foo/bar");
	wildCardPolicy1.setObjectNamespace(namespace2);
	String w1 = ObjectDataHandler.storeObject(wildCardPolicy1);

	ObjectData wildCardPolicy2 = new ObjectData();
	wildCardPolicy2.setObjectName("foo/bar/d1");
	wildCardPolicy2.setObjectNamespace(namespace2);
	String w2 = ObjectDataHandler.storeObject(wildCardPolicy2);

	ObjectData wildCardPolicy3 = new ObjectData();
	wildCardPolicy3.setObjectName("foo/bar/d2");
	wildCardPolicy3.setObjectNamespace(namespace2);
	String w3 = ObjectDataHandler.storeObject(wildCardPolicy3);

	ObjectData wildCardPolicy5 = new ObjectData();
	wildCardPolicy5.setObjectNamespace(namespace2);
	wildCardPolicy5.setObjectName("foo/bar/d2/*");
	String w4 = ObjectDataHandler.storeObject(wildCardPolicy5);

	ObjectData wildCardPolicy4 = new ObjectData();
	wildCardPolicy4.setObjectNamespace(namespace2);
	wildCardPolicy4.setObjectName("bar");
	String bar = ObjectDataHandler.storeObject(wildCardPolicy4);

	// CAS Namespace
	NamespaceData nsData1 = new NamespaceData();
	nsData1.setNickname(CasConstants.NAMESPACE_SELF);
	nsData1.setBasename("baseName");
	nsData1.setComparisonAlg(CasConstants.EXACT_ALG_CLASSNAME);
	NamespaceDataHandler.storeObject(nsData1);

	// casServer as object
	ObjectData objData = new ObjectData();
	objData.setObjectName(CasConstants.OBJECT_SELF);
	objData.setObjectNamespace(CasConstants.NAMESPACE_SELF);
	String casServerId = ObjectDataHandler.storeObject(objData);

	// cas as service type
	ServiceTypeData serviceType = new ServiceTypeData();
	serviceType.setName(CasConstants.SERVICETYPE_CAS);
	ServiceTypeDataHandler.storeObject(serviceType);

	//cas/enroll_object
	String enrollId = ServiceTypeActionHandler.addServiceActionMapping(
				 CasConstants.SERVICETYPE_CAS, 
				 CasConstants.ACTION_ENROLL_OBJECT);

	String grantAll = ServiceTypeActionHandler.addServiceActionMapping(
				 CasConstants.SERVICETYPE_CAS, 
				 CasConstants.ACTION_GRANTALL);

	// Interested only in enroll object iniitializaton
	CasConstants.initializePolicies();

	// add user to user group
	// Add userNick to group
	UserGroupDataHandler.addGroupMember(rootGroup, userNick);
	// shld not have permissions sicne cas/enrollObject is not there
	assertFalse(PermissionsHandler.canCreateObject(userNick, namespace2, 
						       "fooabc"));
	PolicyData policyData = new PolicyData();
	policyData.setUserGroupName(rootGroup);
	policyData.setActionSpec(enrollId);
	policyData.setActionSpecDesc("serviceAction");
	policyData.setObjectSpec(casServerId);
	policyData.setObjectSpecDesc("object");
	PolicyDataHandler.storeObject(policyData);

	// Disjoint
	assertTrue(PermissionsHandler.canCreateObject(userNick, namespace2, 
						       "fooabc"));

	// Exact match
	boolean expBool = false;
	try {
	    PermissionsHandler.canCreateObject(userNick, namespace2, 
					       "foo/bar");
	} catch (CasDBException exp) {
	    if ((exp.getMessage().indexOf("Object already exists") == -1))
		fail("Object existance not detected");
	    expBool = true;
	}
	assertTrue(expBool);
	
	// Subset exist, super set doesnot - but permissiosn shld fail
	expBool = false;
	try {
	    PermissionsHandler.canCreateObject(userNick, namespace2, 
					       "foo/bar/*");
	} catch (CasDBException exp) {
	    logger.error(exp.getMessage());
	    if ((exp.getMessage().indexOf("Object already exists as subset "
					  + "and relevant permissions don't"
					  + " exist") == -1))
		fail("Object existance not detected");
	    expBool = true;
	}
	assertTrue(expBool);
	
	// gve permissions on one of the subset - shld fail.
	// set on foo/bar/d1
	policyData.setActionSpec(grantAll);
	policyData.setObjectSpec(w2);
	PolicyDataHandler.storeObject(policyData);

	expBool = false;
	try {
	    PermissionsHandler.canCreateObject(userNick, namespace2, 
					       "foo/bar/*");
	} catch (CasDBException exp) {
	    logger.debug(exp.getMessage());
	    if ((exp.getMessage().indexOf("Object already exists as subset "
					  + "and relevant permissions don't"
					  + " exist") == -1))
		fail("Object existance not detected");
	    expBool = true;
	}
	assertTrue(expBool);

	// set on foo/bar/d2
	policyData.setObjectSpec(w3);
	PolicyDataHandler.storeObject(policyData);

	// set on foo/bar/d2/*
	policyData.setObjectSpec(w4);
	PolicyDataHandler.storeObject(policyData);

	assertTrue(PermissionsHandler.canCreateObject(userNick, namespace2, 
						      "foo/bar/*"));
	ObjectData toStore = new ObjectData();
	toStore.setObjectName("foo/bar/*");
	toStore.setObjectNamespace(namespace2);
	String w5 = ObjectDataHandler.storeObject(toStore);

	expBool = false;
	try {
	    PermissionsHandler.canCreateObject(userNick, namespace2, 
					       "foo/bar/d3");
	} catch (CasDBException exp) {
	    logger.error(exp.getMessage());
	    if ((exp.getMessage().indexOf("Object already exists as super set"
					  + " and relevant permissions don't"
					  + " exist") == -1))
		fail("Object existance not detected");
	    expBool = true;
	}
	assertTrue(expBool);

	// Enroll object on super set
	policyData.setActionSpec(enrollId);	
	policyData.setObjectSpec(w5);
	PolicyDataHandler.storeObject(policyData);
	assertTrue(PermissionsHandler.canCreateObject(userNick, namespace2, 
						      "foo/bar/d3"));

	PolicyDataHandler.deletePolicyForObject(w5, namespace2);

	// Attempt adding foo/bar/d1/* - has no super and sub set
	assertTrue(PermissionsHandler.canCreateObject(userNick, namespace2, 
						      "foo/bar/d1/*"));

	// add foo/* as super set
	ObjectData wildCardPolicy6 = new ObjectData();
	wildCardPolicy6.setObjectNamespace(namespace2);
	wildCardPolicy6.setObjectName("foo/*");
	String fooStar = ObjectDataHandler.storeObject(wildCardPolicy6);
       
	// add foo/bar/d1/a1 and foo/bar/d1/a2 as subset
	ObjectData wildCardPolicy7 = new ObjectData();
	wildCardPolicy7.setObjectNamespace(namespace2);
	wildCardPolicy7.setObjectName("foo/bar/d1/a1");
	String temp1 = ObjectDataHandler.storeObject(wildCardPolicy7);

	ObjectData wildCardPolicy8 = new ObjectData();
	wildCardPolicy8.setObjectNamespace(namespace2);
	wildCardPolicy8.setObjectName("foo/bar/d1/a2");
	String temp2 = ObjectDataHandler.storeObject(wildCardPolicy8);

	// Has super set and subset with no permissions  - shld fail
	expBool = false;
	try {
	    assertTrue(PermissionsHandler.canCreateObject(userNick, namespace2, 
							  "foo/bar/d1/*"));
	} catch (CasDBException exp) {
	    if ((exp.getMessage().indexOf("Object already exists as subset and "
					  + "relevant permissions don't"
					  + " exist") == -1))
		fail("Object existance not detected");
	    expBool = true;
	}
	assertTrue(expBool);

	// cas/grantAll on subset
	policyData.setActionSpec(grantAll);
	policyData.setObjectSpec(temp1);
	PolicyDataHandler.storeObject(policyData);

	expBool = false;
	try {
	    assertTrue(PermissionsHandler.canCreateObject(userNick, namespace2, 
							  "foo/bar/d1/*"));
	} catch (CasDBException exp) {
	    if ((exp.getMessage().indexOf("Object already exists as subset and "
					  + "relevant permissions don't"
					  + " exist") == -1))
		fail("Object existance not detected");
	    expBool = true;
	}
	assertTrue(expBool);
	
	// Add object 
	objData.setObjectName("obj1");
	objData.setObjectNamespace(CasConstants.NAMESPACE_SELF);
	String exactId = ObjectDataHandler.storeObject(objData);

	expBool = false;
	try {
	    PermissionsHandler.canCreateObject(userNick, 
			       CasConstants.NAMESPACE_SELF, "obj1");
	} catch (CasDBException exp) {
	    if ((exp.getMessage().indexOf("Object already exists") == -1))
		fail("Object existance not detected");
	    expBool = true;
	}
	assertTrue(expBool);

	PolicyDataHandler.deletePolicyForObject(exactId, "object");
	PolicyDataHandler.deletePolicyForObject(temp1, "object");
	PolicyDataHandler.deletePolicyForObject(temp2, "object");
	PolicyDataHandler.deletePolicyForObject(fooStar, "object");
	PolicyDataHandler.deletePolicyForObject(w5, "object");
	PolicyDataHandler.deletePolicyForObject(w4, "object");
	PolicyDataHandler.deletePolicyForObject(w3, "object");
	PolicyDataHandler.deletePolicyForObject(w2, "object");
	PolicyDataHandler.deletePolicyForObject(w1, "object");
	PolicyDataHandler.deletePolicyForObject(casServerId, "object");
	ObjectDataHandler.deleteObjectForId(exactId);
	ObjectDataHandler.deleteObjectForId(temp1);
	ObjectDataHandler.deleteObjectForId(temp2);
	ObjectDataHandler.deleteObjectForId(fooStar);
	ObjectDataHandler.deleteObjectForId(w5);
	ObjectDataHandler.deleteObjectForId(w4);
	ObjectDataHandler.deleteObjectForId(w3);
	ObjectDataHandler.deleteObjectForId(w2);
	ObjectDataHandler.deleteObjectForId(w1);
	ObjectDataHandler.deleteObjectForId(bar);
	ObjectDataHandler.deleteObjectForId(casServerId);
	UserGroupDataHandler.removeGroupMember(rootGroup, userNick);
	ServiceTypeActionHandler.removeServiceActionMapping(
				 CasConstants.SERVICETYPE_CAS, 
				 CasConstants.ACTION_ENROLL_OBJECT);

	ServiceTypeActionHandler.removeServiceActionMapping(
				 CasConstants.SERVICETYPE_CAS, 
				 CasConstants.ACTION_GRANTALL);
	NamespaceDataHandler.deleteObject(CasConstants.NAMESPACE_SELF);
	NamespaceDataHandler.deleteObject(namespace2);

    }

    private void setupExternalObjects(String groupName1) throws Exception {
	    
	externalObjNS = "SampleServiceNS";	  
	NamespaceData nsData = new NamespaceData();
	nsData.setNickname(externalObjNS);
	nsData.setBasename("ogsa/services/samples");
	nsData.setComparisonAlg(CasConstants.WILD_ALG_CLASSNAME);
	NamespaceDataHandler.storeObject(nsData);

	ObjectData objectData = new ObjectData();
	externalObjName = "CounterService/*";
	objectData.setObjectName(externalObjName);
	objectData.setObjectNamespace(externalObjNS);
	externalObjId = ObjectDataHandler.storeObject(objectData);

	externalServiceType = "SampleCounter";
	ServiceTypeData serviceTypeData = new ServiceTypeData();
	serviceTypeData.setName(externalServiceType);
	ServiceTypeDataHandler.storeObject(serviceTypeData);
	
	String addAction = "add";
	String createAction = "create";
	addActionId = ServiceTypeActionHandler.addServiceActionMapping(
					    externalServiceType, addAction);
	createActionId = ServiceTypeActionHandler.addServiceActionMapping(
					    externalServiceType, createAction);
	
	// Set createAction on serviceType for userNick
	PolicyData policyData = new PolicyData();
	policyData.setUserGroupName(groupName1);
	policyData.setActionSpec(addActionId);
	policyData.setActionSpecDesc("serviceAction");
	policyData.setObjectSpec(externalObjId);
	policyData.setObjectSpecDesc("object");
	externalPolicy = PolicyDataHandler.storeObject(policyData);
    }

    private void displayPolicyArray(PolicyData[] policies) {
	logger.debug("displaying policy");
	for (int i=0; i<policies.length; i++) {
	    logger.debug(CasStringUtils.policyToString(policies[i]));
	}
    }

    private void displayPolicyVector(Vector policies) {
	logger.debug("displaying policy");
	for (int i=0; i<policies.size(); i++) {
	    logger.debug("Policy is " + CasStringUtils.policyToString(
					(PolicyData)policies.get(i)));
	}
    }

    private boolean ensureOnlyExternalPolicy(Vector policies) 
	throws Exception {
	int selfObjId = 
	    ObjectDataHandler.getObjectId(
					  CasConstants.OBJECT_SELF,
					  CasConstants.NAMESPACE_SELF);
	String objName = null;
	for (int i=0; i<policies.size(); i++) {
	    PolicyData policyData = (PolicyData)policies.get(i);
	    if (objName == null)
		objName = policyData.getObjectSpec();
	    if (policyData.getObjectSpec().equals(Integer.toString(selfObjId)))
		return false;
	    // Ensure an array has the same object name, since this method
	    // is called by methods that receove a Vector of Vector, with
	    // each inherant vector representing policies per resource.
	    if (!policyData.getObjectSpec().equals(objName))
		return false;
	}
	return true;
    }
       
}
