/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.cas.impl.databaseAccess;

import org.globus.cas.types.ServiceTypeData;

import org.globus.cas.utils.CasStringUtils;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class TestServiceTypeAction extends TestCase {

    static Log logger =
              LogFactory.getLog(TestServiceTypeAction.class.getName());

    public TestServiceTypeAction(String name){
        super(name);
    }

    public static Test suite() {
        return new TestSuite(TestServiceTypeAction.class);
    }

    public void testServiceTypeAction() throws Exception {
        
        String serviceTypeName = "serviceTypeName";
        ServiceTypeData serviceTypeData = new ServiceTypeData();
        serviceTypeData.setName(serviceTypeName);
        String serviceId = ServiceTypeDataHandler.storeObject(serviceTypeData);
        assertTrue(serviceId!=null);
        logger.debug("Store  service type data went through");

        String actionName1 = "actionName1";
        String actionName2 = "actionName2";
        // add action mappings
        String actionId1 = 
            ServiceTypeActionHandler.addServiceActionMapping(serviceTypeName, 
                                                             actionName1);
        assertTrue(actionId1!=null);
        logger.debug("actionName1 added");
        String actionId2 = 
            ServiceTypeActionHandler.addServiceActionMapping(serviceTypeName, 
                                                             actionName2);
        assertTrue(actionId2!=null);
        logger.debug("actionName2 added");

        // retrieve action mappings
        String[] serviceActionMapping = 
            ServiceTypeActionHandler.retrieveServiceActionMappings(
                                     serviceTypeName);
        assertTrue(serviceActionMapping!=null);
        logger.debug("mappings retrived for assertions");
        assertTrue(CasStringUtils.stringInArray(serviceActionMapping, 
                                                 actionName1));
        assertTrue(CasStringUtils.stringInArray(serviceActionMapping, 
                                                 actionName2));

        // test list
        String list[] = ServiceTypeActionHandler.listServiceActionMappings();
        assertTrue(list != null);
        assertTrue(CasStringUtils.stringInArray(list, serviceTypeName + "/" 
                                                + actionName1));
        assertTrue(CasStringUtils.stringInArray(list, serviceTypeName + "/" 
                                                + actionName2));

        // remove mappings
        ServiceTypeActionHandler.removeServiceActionMapping(serviceTypeName, 
                                                            actionName2);
        serviceActionMapping = 
            ServiceTypeActionHandler.retrieveServiceActionMappings(
                                     serviceTypeName);
        assertTrue(serviceActionMapping!=null);
        assertTrue(!CasStringUtils.stringInArray(serviceActionMapping, 
                                                  actionName2));
        logger.debug("actionName2 removed");
        // create service_action group
        String serviceGpName = "serviceGroupName";
        ServiceTypeActionHandler.createServiceActionGroup(serviceGpName);
        logger.debug("created action group");
        // add entries to group
        ServiceTypeActionHandler.addServiceActionGpEntry(serviceGpName, 
                                                         serviceTypeName, 
                                                         actionName1);
        ServiceTypeActionHandler.addServiceActionMapping(serviceTypeName, 
                                                         actionName2);
        ServiceTypeActionHandler.addServiceActionGpEntry(serviceGpName, 
                                                         serviceTypeName, 
                                                         actionName2);
        // retrieve entries
        String groupEntries[] = 
            ServiceTypeActionHandler.retrieveServiceActionGpEntries(
                                     serviceGpName);
        assertTrue(groupEntries!=null);
        assertTrue(CasStringUtils.stringInArray(groupEntries, serviceTypeName 
                                                + "/" + actionName1));
        assertTrue(CasStringUtils.stringInArray(groupEntries, serviceTypeName 
                                                + "/" + actionName2));

                   
        // create service_action group
        String serviceGpName1 = "serviceGroupName1";
        ServiceTypeActionHandler.createServiceActionGroup(serviceGpName1);
        // add entries (actionId1)
        ServiceTypeActionHandler.addServiceActionGpEntry(serviceGpName1, 
                                                         serviceTypeName, 
                                                         actionName1);
        
        // ActionId1 belongs to two groups, retrieve and check
        String[] actionIdGroups 
            = ServiceTypeActionHandler.retrieveServiceActionGroups(actionId1);
        assertTrue(actionIdGroups != null);
        assertTrue(CasStringUtils.stringInArray(actionIdGroups, 
                                                serviceGpName));
        assertTrue(CasStringUtils.stringInArray(actionIdGroups, 
                                                serviceGpName1));
        
        // test list
        list = ServiceTypeActionHandler.listServiceActionGroups();
        assertTrue(list != null);
        assertTrue(CasStringUtils.stringInArray(list, serviceGpName));
        assertTrue(CasStringUtils.stringInArray(list, serviceGpName1));

        // remove entries
        ServiceTypeActionHandler.removeServiceActionGpEntry(serviceGpName, 
                                                            serviceTypeName, 
                                                            actionName2);
        ServiceTypeActionHandler.removeServiceActionMapping(serviceTypeName, 
                                                            actionName2);      

        groupEntries = 
            ServiceTypeActionHandler.retrieveServiceActionGpEntries(
                                     serviceGpName);
        assertTrue(!CasStringUtils.stringInArray(groupEntries, serviceTypeName 
                                                  + "/" + actionName2));

        ServiceTypeActionHandler.removeServiceActionGpEntry(serviceGpName, 
                                                            serviceTypeName, 
                                                            actionName1);
      
        ServiceTypeActionHandler.removeServiceActionGpEntry(serviceGpName1, 
                                                            serviceTypeName, 
                                                            actionName1);
        ServiceTypeActionHandler.removeServiceActionMapping(serviceTypeName, 
                                                            actionName1);     

        ServiceTypeActionHandler.deleteServiceActionGroup(serviceGpName1);
        ServiceTypeActionHandler.deleteServiceActionGroup(serviceGpName);
        ServiceTypeDataHandler.deleteObject(serviceTypeName);
    }
}
