/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.cas.impl.databaseAccess;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.globus.cas.types.TrustAnchorData;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class TestTrustAnchorData extends TestCase {

    static Log logger =
              LogFactory.getLog(TestTrustAnchorData.class.getName());

    public TestTrustAnchorData(String name){
        super(name);
    }

    public static Test suite() {
        return new TestSuite(TestTrustAnchorData.class);
    }

    public void testTrustAnchorData() throws Exception {

        String nickname = "nickname" + System.currentTimeMillis();
        String authMethod = "authMethod";
        String authData ="authData";

        // Store trust anchor data
        TrustAnchorData trustAnchor = new TrustAnchorData();
        trustAnchor.setNickname(nickname);
        trustAnchor.setAuthMethod(authMethod);
        trustAnchor.setAuthData(authData);
        TrustAnchorDataHandler.storeObject(trustAnchor);
        logger.debug("Store object went through " + trustAnchor.toString());

        // List of all trust anchors
        String[] trustAnchorList = TrustAnchorDataHandler.list();
        assertTrue(trustAnchorList!=null);
        logger.debug("Trust anchor list is " + trustAnchorList);

        // Ensure that the stored and retrieved objects are the same
        TrustAnchorData retrieveAnchorData = 
            (TrustAnchorData)TrustAnchorDataHandler.retrieveObject(nickname);
        assertTrue(retrieveAnchorData!=null);
        logger.debug("Retrieved trust anchor " + retrieveAnchorData.toString());
        assertTrue(retrieveAnchorData.equals(trustAnchor));

        // Retrive nickname for authdata nad method
        String retrievedNick = TrustAnchorDataHandler.getNickname(authData, 
                                                                 authMethod);
        assertTrue(retrievedNick.equals(nickname));

        TrustAnchorDataHandler.deleteObject(nickname);
        TrustAnchorData deletedData = 
            (TrustAnchorData)TrustAnchorDataHandler.retrieveObject(nickname);
        assertTrue(deletedData == null);
    }
}
