/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.cas.impl.databaseAccess;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.globus.cas.types.UserData;
import org.globus.cas.types.UserGroupData;
import org.globus.cas.types.ArrayOfString;
import org.globus.cas.types.TrustAnchorData;

import org.globus.cas.utils.CasStringUtils;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class TestUserData extends TestCase {

    static Log logger = LogFactory.getLog(TestUserData.class.getName());

    public TestUserData(String name){
        super(name);
    }

    public static Test suite() {
        return new TestSuite(TestUserData.class);
    }
    
    public void testUserData() throws Exception {

        // Store Trust anchor data
        String nickname = "nickname1";
        String authMethod = "authMethod1";
        String authData ="authData1";
        TrustAnchorData trustAnchor = new TrustAnchorData();
        trustAnchor.setNickname(nickname);
        trustAnchor.setAuthMethod(authMethod);
        trustAnchor.setAuthData(authData);
        TrustAnchorDataHandler.storeObject(trustAnchor);
        logger.debug("Store trust anchor object went through");

        // UserGroup data store without usernames
        String groupName = "userGroup1";
        UserGroupData userGpData = new UserGroupData();
        userGpData.setGroupName(groupName);
        UserGroupDataHandler.storeObject(userGpData);
        logger.debug("Store user group data went through ");
        
        // Store user data 
        String userNick ="userNick1";
        String subjectName = "userSubjectName1";
        UserData userData1 = new UserData();
        userData1.setNickname(userNick);
        userData1.setSubjectName(subjectName);
        userData1.setTrustAnchorName(nickname);
        UserDataHandler.storeObject(userData1);
        logger.debug("Store user data " 
                     + CasStringUtils.userToString(userData1));

        // Check user nick retrieval
        String subjectToNick = UserDataHandler.getUserNickname(subjectName);
        assertTrue(subjectToNick.equals(userNick));
        
        // User data retrieve 
        UserData retrieveUserData = 
            (UserData)UserDataHandler.retrieveObject(userNick);
        assertTrue(retrieveUserData!=null);
        if (retrieveUserData.getGroupNames() == null) 
            logger.debug("retrive gp is null");
        else {
            logger.debug("retrive gp is not null");
            String str[] = retrieveUserData.getGroupNames().getStrings();
            logger.debug("str length " + str.length);
            for (int i=0; i<str.length; i++) 
                logger.debug(str[i]);
        }
        logger.debug("Retrieved user data " 
                     + CasStringUtils.userToString(retrieveUserData));
        assertTrue(userData1.equals(retrieveUserData));

        // User data list
        String[] list = UserDataHandler.list();
        assertTrue(list.length==1);
        logger.debug("List users went through");

        // User data delete 
        UserDataHandler.deleteObject(userNick);
        UserData deletedData1 = 
            (UserData)UserDataHandler.retrieveObject(userNick);
        assertTrue(deletedData1 == null);

        // User group retrieve
        UserGroupData retrieveUserGpData = 
            (UserGroupData)UserGroupDataHandler.retrieveObject(groupName);
        assertTrue(retrieveUserGpData!=null);
        logger.debug("retrieved user group data " 
                     + retrieveUserGpData.toString());
        assertTrue(retrieveUserGpData.equals(userGpData));

        // User group delete
        UserGroupDataHandler.deleteObject(groupName);
        UserGroupData deletedUserGpData = 
            (UserGroupData)UserGroupDataHandler.retrieveObject(groupName);
        assertTrue(deletedUserGpData==null);

        // To test adding group members
        // add user group object
        String groupName1 = "groupName2";
        UserGroupData ugData = new UserGroupData();
        ugData.setGroupName(groupName1);
        UserGroupDataHandler.storeObject(ugData);

        // add second group
        String groupName2 = "groupName3";
        userGpData = new UserGroupData();
        userGpData.setGroupName(groupName2);
        UserGroupDataHandler.storeObject(userGpData);

        // add user data
        UserData uData = new UserData();
        uData.setNickname(userNick);
        uData.setSubjectName("subject");
        uData.setTrustAnchorName(nickname);
        String temp1[] = new String[1];
        temp1[0] = groupName1;
        uData.setGroupNames(new ArrayOfString(temp1));
        UserDataHandler.storeObject(uData);

        // add user as member to the group
        UserGroupDataHandler.addGroupMember(groupName2, userNick);
        retrieveUserGpData = 
            (UserGroupData)UserGroupDataHandler.retrieveObject(groupName2);
        assertTrue(retrieveUserGpData.getUserNames() != null);
        String memberUsers[] = retrieveUserGpData.getUserNames().getStrings();
        assertTrue(memberUsers!=null);
        assertTrue(CasStringUtils.stringInArray(memberUsers, userNick));

        retrieveUserGpData = 
            (UserGroupData)UserGroupDataHandler.retrieveObject(groupName2);
        assertTrue(retrieveUserGpData.getUserNames() != null);
        memberUsers = retrieveUserGpData.getUserNames().getStrings();
        assertTrue(memberUsers!=null);
        assertTrue(CasStringUtils.stringInArray(memberUsers, userNick));

        // remove user from members list
        UserGroupDataHandler.removeGroupMember(groupName2, userNick);
        retrieveUserGpData = 
            (UserGroupData)UserGroupDataHandler.retrieveObject(groupName2);
        assertTrue(retrieveUserGpData.getUserNames() == null);

        // Clean up
        UserGroupDataHandler.deleteObject(groupName1);
        UserGroupDataHandler.deleteObject(groupName2);
        UserDataHandler.deleteObject(userNick);
        TrustAnchorDataHandler.deleteObject(nickname);
    }
}
