/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.cas.impl.service;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.w3c.dom.Element;
import org.apache.axis.message.MessageElement;

import org.globus.cas.types.SAMLAuthzQueryType;
import org.globus.cas.types.ServiceTypeData;
import org.globus.cas.types.UserGroupData;
import org.globus.cas.types.PolicyData;
import org.globus.cas.types.UserData;
import org.globus.cas.types.ObjectData;
import org.globus.cas.types.NamespaceData;
import org.globus.cas.types.ObjectGroupData;
import org.globus.cas.types.GetAssertionParam;
import org.globus.cas.types.ArrayOfSAMLAuthzQueryType;

import org.globus.cas.impl.databaseAccess.ObjectDataHandler;
import org.globus.cas.impl.databaseAccess.UserDataHandler;
import org.globus.cas.impl.databaseAccess.ObjectGroupDataHandler;
import org.globus.cas.impl.databaseAccess.UserGroupDataHandler;
import org.globus.cas.impl.databaseAccess.ServiceTypeDataHandler;
import org.globus.cas.impl.databaseAccess.ServiceTypeActionHandler;
import org.globus.cas.impl.databaseAccess.PolicyDataHandler;
import org.globus.cas.impl.databaseAccess.NamespaceDataHandler;

import org.globus.cas.faults.NoPermissionFault;

import org.globus.cas.impl.CasConstants;

import org.globus.cas.CASPortType;
import org.globus.wsrf.impl.security.SecurityMessageElement;

import org.opensaml.SAMLAction;
import org.opensaml.SAMLSubject;
import org.opensaml.SAMLAssertion;
import org.opensaml.SAMLAuthorizationDecisionQuery;
import org.opensaml.SAMLAuthorizationDecisionStatement;

import java.io.FileInputStream;

import java.util.Properties;
import java.util.Iterator;
import java.util.Date;
import java.util.Calendar;
import java.util.Vector;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class TestAssertionGeneration extends TestCase {

    static Log logger =
        LogFactory.getLog(TestAssertionGeneration.class.getName());

    static CASPortType casPort = null;

    int maxAssertionLifetime = 24 * 60 * 60;
    String objId = null;
    String enrollAnchor = null;
    String enrollUser = null;
    String enrollNs = null;
    String enrollObject = null;
    String enrollService = null;
    String createObjGp = null;
    String createUserGp = null;
    String createServiceActionGp = null;
    String addGroupEntry = null;
    String query = null;
    SAMLSubject samlSubject = null;
    Vector confMethods = null;

    public TestAssertionGeneration(String name){
        super(name);
    }

    public static Test suite() {
        return new TestSuite(TestAssertionGeneration.class);
    }

    public static void
        setParams(CASPortType casPort_) {
        casPort = casPort_;
    }

    public void testAssertionGeneration() throws Exception {
        initialSetup();
        emptyAssertionsTest();
        PopulateDB.addExternalObjects();
        secondUserTest();
        scenarioOneTest();
        scenarioTwoTest();
        scenarioThreeTest();
        scenarioFourTest();
        PopulateDB.scenarioFour(false);
        PopulateDB.scenarioThree(false);
        PopulateDB.scenarioTwo(false);
        PopulateDB.scenarioOne(false);
        PopulateDB.removeExternalObjects();
    }

    // Direct database minipulation to set up the db for testing
    private void initialSetup() throws Exception {

        String propFileName = System.getProperty("casTestProperties");
        Properties prop = new Properties();
        prop.load(new FileInputStream(propFileName));
        String subjectName = prop.getProperty("user1SubjectDN");
        String tempLifetime = prop.getProperty("maxAssertionLifetime");
        if ((tempLifetime != null) && (!(tempLifetime.trim().equals("")))) {
            maxAssertionLifetime = Integer.parseInt(tempLifetime);
        } else {
            logger.warn("Server's maximum lifetime value not set, "
                        + "using default value. Test might fail");
        }
        // Construct subject
        confMethods = new Vector(1);
        confMethods.add(CasConstants.X509_CONFIRMATION_METHOD);
        samlSubject = new SAMLSubject(subjectName,
                                      TestConstants.defTrustNickDN,
                                      CasConstants.X509_FORMAT,
                                      confMethods, null, null);

        objId =
            Integer.toString(ObjectDataHandler.getObjectId(
                                               CasConstants.OBJECT_SELF,
                                               CasConstants.NAMESPACE_SELF));
        enrollAnchor =
            Integer.toString(ServiceTypeActionHandler.getServiceActionId(
                             CasConstants.SERVICETYPE_CAS,
                             CasConstants.ACTION_ENROLL_ANCHOR));
        enrollUser =
            Integer.toString(
                    ServiceTypeActionHandler.getServiceActionId(
                                             CasConstants.SERVICETYPE_CAS,
                                             CasConstants.ACTION_ENROLL_USER));
        enrollNs = Integer.toString(
                           ServiceTypeActionHandler.getServiceActionId(
                           CasConstants.SERVICETYPE_CAS,
                           CasConstants.ACTION_ENROLL_NAMESPACE));
        enrollObject = Integer.toString(
                               ServiceTypeActionHandler.getServiceActionId(
                               CasConstants.SERVICETYPE_CAS,
                               CasConstants.ACTION_ENROLL_OBJECT));
        enrollService = Integer.toString(
                                ServiceTypeActionHandler.getServiceActionId(
                                CasConstants.SERVICETYPE_CAS,
                                CasConstants.ACTION_CREATE_SERVICE_TYPE));
        createObjGp = Integer.toString(
                              ServiceTypeActionHandler.getServiceActionId(
                              CasConstants.SERVICETYPE_CAS,
                              CasConstants.ACTION_CREATE_OBJECT_GROUP));
        createUserGp = Integer.toString(
                               ServiceTypeActionHandler.getServiceActionId(
                               CasConstants.SERVICETYPE_CAS,
                               CasConstants.ACTION_CREATE_USER_GROUP));
        createServiceActionGp =
            Integer.toString(
                             ServiceTypeActionHandler.getServiceActionId(
                             CasConstants.SERVICETYPE_CAS,
                             CasConstants.ACTION_CREATE_SERVICEACTION_GROUP));
        addGroupEntry =
            Integer.toString(ServiceTypeActionHandler.getServiceActionId(
                             CasConstants.SERVICETYPE_CAS,
                             CasConstants.ACTION_ADD_GROUP_ENTRY));

        query =
            Integer.toString(ServiceTypeActionHandler.getServiceActionId(
                             CasConstants.SERVICETYPE_CAS,
                             CasConstants.ACTION_QUERY));
    }

    private GetAssertionParam getAssertion(ArrayOfSAMLAuthzQueryType query,
                                           int lifetime) {
        GetAssertionParam getAssertion = new GetAssertionParam();
        getAssertion.setLifetime(lifetime);
        getAssertion.setSamlAuthzQuery(query);
        return getAssertion;
    }

    // test null id returned when there are no valid policies
    // test permissions to get assertion for another user
    public void emptyAssertionsTest() throws Exception {

        logger.debug("empty assertion test");
        SAMLAuthorizationDecisionQuery samlAuthQuery =
            constructMaxQuery(samlSubject);
        SAMLAuthzQueryType samlQuery = new SAMLAuthzQueryType();
        samlQuery.set_any(new MessageElement[] {
            new SecurityMessageElement((Element)samlAuthQuery.toDOM())});
        SAMLAuthzQueryType[] samlQueries = new SAMLAuthzQueryType[1];
        samlQueries[0] = samlQuery;
        ArrayOfSAMLAuthzQueryType arrayOfSamlAuthz =
            new ArrayOfSAMLAuthzQueryType(samlQueries);
        org.globus.cas.types.SAMLAssertion samlAssertion =
            casPort.getAssertion(getAssertion(arrayOfSamlAuthz, 100));
        assertTrue(samlAssertion == null);

        // resource - for CAS it would be name of obejct
        String objectName = "casNamespace|casServer";

        // Collection of actions to query on
        Vector actions = new Vector();
        // Conststruct an action
        SAMLAction samlAction = new SAMLAction("cas", "nonExistant");
        actions.add(samlAction);

        samlAuthQuery =
            new SAMLAuthorizationDecisionQuery(samlSubject, objectName,
                                               actions, null);

        samlQueries[0] = samlQuery;
        arrayOfSamlAuthz = new ArrayOfSAMLAuthzQueryType(samlQueries);
        samlAssertion =
            casPort.getAssertion(getAssertion(arrayOfSamlAuthz,100));
        assertTrue(samlAssertion == null);

        logger.debug("empty assertion test done");
    }

    public void secondUserTest() throws Exception {

        // Add another user
        String secondUserNick = "secondUser";
        String secondUserDN = "dummy DN";
        UserData userData = new UserData();
        userData.setNickname(secondUserNick);
        userData.setSubjectName(secondUserDN);
        userData.setTrustAnchorName(TestConstants.defTrustNick);
        UserDataHandler.storeObject(userData);

        // Add to user group with name userGroup1
        UserGroupDataHandler.addGroupMember("userGroup1", secondUserNick);

        // add policy for external object
        PolicyData policyData = new PolicyData();
        policyData.setUserGroupName("userGroup1");
        policyData.setActionSpec(PopulateDB.serviceGp1);
        policyData.setActionSpecDesc(CasConstants.SERVICEACTIONGP_SPEC);
        policyData.setObjectSpec(PopulateDB.objGp2);
        policyData.setObjectSpecDesc(CasConstants.OBJECTGP_SPEC);
        String secondUserPolicy = PolicyDataHandler.storeObject(policyData);

        // Try to get assertion for this user, shld fail
        SAMLSubject secondUserSubject =
            new SAMLSubject(secondUserDN, TestConstants.defTrustNickDN,
                            CasConstants.X509_FORMAT, confMethods, null, null);

        // Second user's query
        SAMLAuthorizationDecisionQuery secondAuthQuery =
            constructMaxQuery(secondUserSubject);
        SAMLAuthzQueryType secondSAMLQuery = new SAMLAuthzQueryType();
        secondSAMLQuery.set_any(new MessageElement[] {
            new SecurityMessageElement((Element)secondAuthQuery.toDOM())});

        // First user's query
        SAMLAuthorizationDecisionQuery samlAuthQuery =
            constructMaxQuery(samlSubject);
        SAMLAuthzQueryType samlQuery = new SAMLAuthzQueryType();
        samlQuery.set_any(new MessageElement[] {
            new SecurityMessageElement((Element)samlAuthQuery.toDOM())});

        SAMLAuthzQueryType[] samlQueries = new SAMLAuthzQueryType[2];
        samlQueries[0] = secondSAMLQuery;
        samlQueries[1] = samlQuery;
        ArrayOfSAMLAuthzQueryType arrayOfSamlAuthz =
            new ArrayOfSAMLAuthzQueryType(samlQueries);

        // Self + second user
        boolean exp = false;
        try {
            org.globus.cas.types.SAMLAssertion assertion =
                casPort.getAssertion(getAssertion(arrayOfSamlAuthz,
                                                  100));
        } catch (NoPermissionFault npf) {
            exp = true;
        }
        assertTrue(exp);

        samlQueries = new SAMLAuthzQueryType[1];
        samlQuery.set_any(new MessageElement[] {
            new SecurityMessageElement((Element)samlAuthQuery.toDOM())});
        samlQueries[0] = secondSAMLQuery;
        arrayOfSamlAuthz = new ArrayOfSAMLAuthzQueryType(samlQueries);

        // Second user query
        exp = false;
        try {
            org.globus.cas.types.SAMLAssertion assertion =
                casPort.getAssertion(getAssertion(arrayOfSamlAuthz,
                                                  100));
        } catch (NoPermissionFault npf) {
            exp = true;
        }
        assertTrue(exp);

        // store permission`
        policyData = new PolicyData();
        policyData.setUserGroupName(TestConstants.defUserGp);
        policyData.setObjectSpec(objId);
        policyData.setObjectSpecDesc(CasConstants.OBJECT_SPEC);
        policyData.setActionSpec(query);
        policyData.setActionSpecDesc(CasConstants.SERVICEACTION_SPEC);
        String casQueryPolicy = PolicyDataHandler.storeObject(policyData);

        // Get assertion
        arrayOfSamlAuthz = new ArrayOfSAMLAuthzQueryType(samlQueries);
        org.globus.cas.types.SAMLAssertion assertion =
            casPort.getAssertion(getAssertion(arrayOfSamlAuthz, 100));
        Iterator stmts = validateAssertion(assertion);
        SAMLAuthorizationDecisionStatement decisionStmt =
            (SAMLAuthorizationDecisionStatement)stmts.next();
        assertTrue(!stmts.hasNext());
        assertTrue(decisionStmt!= null);
        assertTrue(decisionStmt.getSubject() != null);
        assertTrue(decisionStmt.getSubject().getName().equals(
                                             secondUserSubject.getName()));
        String resource = decisionStmt.getResource();
        assertTrue(resource.indexOf(CasConstants.OBJECT_SELF) == -1);
        int actCnt = 0;
        Iterator actions = decisionStmt.getActions();
        while (actions.hasNext()) {
            SAMLAction samlAction = (SAMLAction)actions.next();
            assertTrue(samlAction.getData().equals("read"));
            String actionNs = samlAction.getNamespace();
            assertTrue((actionNs.equals(PopulateDB.serviceType1))||
                       (actionNs.equals(PopulateDB.serviceType2)));
            actCnt++;
        }
        assertTrue(actCnt == 2);


        // remove policy
        PolicyDataHandler.deleteObject(casQueryPolicy);

        // Remove second user policy
        PolicyDataHandler.deleteObject(secondUserPolicy);
        // remove membership
        UserGroupDataHandler.removeGroupMember("userGroup1", secondUserNick);
        UserDataHandler.deleteObject(secondUserNick);
    }


    public void scenarioOneTest() throws Exception {

        logger.debug("test one");
        PopulateDB.scenarioOne(true);
        // Max query = one resoruce - sample1, two actions
        SAMLAuthorizationDecisionQuery samlAuthQuery =
            constructMaxQuery(samlSubject);
        SAMLAuthzQueryType[] samlQueries = new SAMLAuthzQueryType[1];
        SAMLAuthzQueryType samlQuery = new SAMLAuthzQueryType();
        samlQuery.set_any(new MessageElement[] {
            new SecurityMessageElement((Element)samlAuthQuery.toDOM())});
        samlQueries[0] = samlQuery;
        ArrayOfSAMLAuthzQueryType arrayOfSamlAuthz =
            new ArrayOfSAMLAuthzQueryType(samlQueries);
        org.globus.cas.types.SAMLAssertion assertion =
            casPort.getAssertion(getAssertion(arrayOfSamlAuthz, 100));
        Iterator stmts = validateAssertion(assertion);
        // Verify one stsmt, two actions = read
        scenarioOneVerify((SAMLAuthorizationDecisionStatement)stmts.next(),
                          samlAuthQuery.getResource());
        // Ensure only one stmt was generated
        assertTrue(!stmts.hasNext());

        // one resource, two actions
        samlAuthQuery = constructScenarioOneQuery ();
        samlQuery.set_any(new MessageElement[] {
            new SecurityMessageElement((Element)samlAuthQuery.toDOM())});
        samlQueries[0] = samlQuery;
        logger.debug("Scenario one - test 3");
        arrayOfSamlAuthz = new ArrayOfSAMLAuthzQueryType(samlQueries);
        assertion =
            casPort.getAssertion(getAssertion(arrayOfSamlAuthz,100));
        assertTrue(assertion != null);
        logger.debug("Assertion is " + assertion);
        MessageElement[] msgElement = assertion.get_any();
        SAMLAssertion samlAssertion =
            new SAMLAssertion(msgElement[0].getAsDOM());
        samlAssertion.verify(false);
        stmts = samlAssertion.getStatements();
        // Verify one stsmt, two actions = read
        scenarioOneVerify((SAMLAuthorizationDecisionStatement)stmts.next(),
                          samlAuthQuery.getResource());
        // Ensure only one stmt was generated
        assertTrue(!stmts.hasNext());

        // Assertion lifetime verification
        Date notBefore = samlAssertion.getNotBefore();
        Date notOnOrAfter = samlAssertion.getNotOnOrAfter();
        Calendar calNotBefore = Calendar.getInstance();
        calNotBefore.setTime(notBefore);
        // add requested life time
        calNotBefore.add(Calendar.SECOND, 100);
        Calendar calNotAfter = Calendar.getInstance();
        calNotAfter.setTime(notOnOrAfter);
        // differnece shld be exacly requested life time,
        // since less than service life time
        assertTrue(calNotBefore.equals(calNotAfter));

        // Test assertion lifetime.
        // request greater than max time
        int requestLifetime = maxAssertionLifetime + 100;
        samlAuthQuery = constructMaxQuery(samlSubject);
        samlQuery.set_any(new MessageElement[] {
            new SecurityMessageElement((Element)samlAuthQuery.toDOM())});
        samlQueries[0] = samlQuery;
        logger.debug("Scenario one - test 4");
        arrayOfSamlAuthz = new ArrayOfSAMLAuthzQueryType(samlQueries);
        assertion =
            casPort.getAssertion(getAssertion(arrayOfSamlAuthz,
                                              requestLifetime));
        logger.debug("Assertion is " + assertion);
        assertTrue(assertion != null);
        MessageElement[] msgElement1 = assertion.get_any();
        SAMLAssertion samlAssertion1 =
            new SAMLAssertion(msgElement1[0].getAsDOM());
        samlAssertion1.verify(false);
        calNotBefore.setTime(samlAssertion1.getNotBefore());
        calNotAfter.setTime(samlAssertion1.getNotOnOrAfter());
        assertTrue(notBefore.before(notOnOrAfter));
        // since request was greater than max, it is set to max
        calNotBefore.add(Calendar.SECOND, maxAssertionLifetime);
        assertTrue(calNotBefore.equals(calNotAfter));

        // request less than maximum assertion lifetime
        requestLifetime = maxAssertionLifetime - 100;
        logger.debug("Scenario one - test 5");
        arrayOfSamlAuthz = new ArrayOfSAMLAuthzQueryType(samlQueries);
        assertion =
            casPort.getAssertion(getAssertion(arrayOfSamlAuthz,
                                              requestLifetime));
        logger.debug("Assertion is " + assertion);
        assertTrue(assertion != null);
        MessageElement[] msgElement2 = assertion.get_any();
        SAMLAssertion samlAssertion2 =
            new SAMLAssertion(msgElement2[0].getAsDOM());
        samlAssertion2.verify(false);
        calNotBefore.setTime(samlAssertion2.getNotBefore());
        calNotAfter.setTime(samlAssertion2.getNotOnOrAfter());
        assertTrue(notBefore.before(notOnOrAfter));
        // since request was greater than max, it is set to max
        calNotBefore.add(Calendar.SECOND, requestLifetime);
        assertTrue(calNotBefore.equals(calNotAfter));
    }

    // one resource, two actions
    public SAMLAuthorizationDecisionQuery constructScenarioOneQuery()
        throws Exception {

        // resource
        String resource = PopulateDB.nsNick + "|" + "ftp://sample1.org";
        // Collection of actions to query on
        Vector actions = new Vector();
        // Conststruct an action
        // true
        SAMLAction samlAction = new SAMLAction(PopulateDB.serviceType1, "read");
        actions.add(samlAction);
        // true
        samlAction = new SAMLAction(PopulateDB.serviceType2, "read");
        actions.add(samlAction);
        // false
        samlAction = new SAMLAction(PopulateDB.serviceType2, "write");
        actions.add(samlAction);

        return new SAMLAuthorizationDecisionQuery(samlSubject, resource,
                                               actions, null);
    }

    // one stmt with two actions
    public void scenarioOneVerify(SAMLAuthorizationDecisionStatement
                                  samlAuthzDecisionStmt, String resource)
    throws Exception {

        assertTrue(samlAuthzDecisionStmt!=null);
        String retResource = samlAuthzDecisionStmt.getResource();
        logger.debug("RETResource " + retResource + " resource " + resource);
        assertTrue(retResource != null);
        assertTrue(resource.indexOf(CasConstants.OBJECT_SELF) == -1);
        assertTrue(samlAuthzDecisionStmt.getSubject() != null);
        Iterator actionIter = samlAuthzDecisionStmt.getActions();
        assertTrue(actionIter != null);
        int actionCnt = 0;
        while (actionIter.hasNext()) {
            SAMLAction retAction = (SAMLAction)actionIter.next();
            assertTrue(retAction.getData().equals("read"));
            assertTrue((retAction.getNamespace().equals(
                        PopulateDB.serviceType1))
                       || (retAction.getNamespace().equals(
                           PopulateDB.serviceType2)));
            actionCnt++;
        }
        logger.debug("Action count is " + actionCnt);
        assertTrue(actionCnt == 2);
    }

    public void scenarioTwoTest() throws Exception {

        logger.debug("scenario two");
        PopulateDB.scenarioTwo(true);
        // Max query - 2 resources, sampel1 (2 actions), sample3 (2 action)
        SAMLAuthorizationDecisionQuery samlAuthQuery =
            constructMaxQuery(samlSubject);
        SAMLAuthzQueryType[] samlQueries = new SAMLAuthzQueryType[1];
        SAMLAuthzQueryType samlQuery = new SAMLAuthzQueryType();
        samlQuery.set_any(new MessageElement[] {
            new SecurityMessageElement((Element)samlAuthQuery.toDOM())});
        samlQueries[0] = samlQuery;
        logger.debug("Scenario two - test 1");
        ArrayOfSAMLAuthzQueryType arrayOfSamlAuthz =
            new ArrayOfSAMLAuthzQueryType(samlQueries);
        org.globus.cas.types.SAMLAssertion assertion =
            casPort.getAssertion(getAssertion(arrayOfSamlAuthz, 100));
        Iterator smts = validateAssertion(assertion);
        validateScenarioTwoMaxResult(smts);

        // sample3, file/read, dir/write, file/exec, only file/read is returned
        samlAuthQuery = constructScenarioTwoQuery();
        samlQuery.set_any(new MessageElement[] {
            new SecurityMessageElement((Element)samlAuthQuery.toDOM())});
        // Array of queries (scenario one query is also added)
        // sample1 file/read,dir/read
        samlQueries = new SAMLAuthzQueryType[2];
        samlQueries[0] = samlQuery;
        SAMLAuthorizationDecisionQuery samlAuthQuery1
            = constructScenarioOneQuery();
        samlQueries[1] = new SAMLAuthzQueryType();
        samlQueries[1].set_any(new MessageElement[] {
            new SecurityMessageElement((Element)samlAuthQuery1.toDOM())});
        logger.debug("Scenario two - test 2");
        arrayOfSamlAuthz = new ArrayOfSAMLAuthzQueryType(samlQueries);
        assertion =
            casPort.getAssertion(getAssertion(arrayOfSamlAuthz, 100));
        logger.debug("Assertion is " + assertion);
        assertTrue(assertion != null);
        MessageElement[] msgElement = assertion.get_any();
        SAMLAssertion samlAssertion =
            new SAMLAssertion(msgElement[0].getAsDOM());
        samlAssertion.verify(false);
        Iterator stmts = samlAssertion.getStatements();
        assertTrue(stmts.hasNext());
        SAMLAuthorizationDecisionStatement samlStmt
            = (SAMLAuthorizationDecisionStatement)stmts.next();
        scenarioTwoVerify(samlStmt, samlAuthQuery.getResource());
        // Second stmt
        assertTrue(stmts.hasNext());
        samlStmt = (SAMLAuthorizationDecisionStatement)stmts.next();
        scenarioOneVerify(samlStmt, samlAuthQuery1.getResource());
        // Ensure only two stmts was generated
        assertTrue(!stmts.hasNext());
    }

    private void validateScenarioTwoMaxResult(Iterator stmts) {
        assertTrue(stmts != null);
        int stmtCount=0;
        int sampleTrue = 0;
        int oldSample = 0;
        while (stmts.hasNext()) {
            stmtCount++;
            logger.debug("stmts " + stmtCount );
            SAMLAuthorizationDecisionStatement samlAuthzDecisionStmt
                = (SAMLAuthorizationDecisionStatement)stmts.next();
            logger.debug("stmt " + samlAuthzDecisionStmt.toString());
            assertTrue(samlAuthzDecisionStmt.getDecision().equals(
                                             CasConstants.SAML_DECISION));
            logger.debug("permit true");
            String resource = samlAuthzDecisionStmt.getResource();
            assertTrue(resource.indexOf(CasConstants.OBJECT_SELF) == -1);
            logger.debug("resource true");
            if (resource.equals(PopulateDB.nsNick + "|"
                                + "ftp://sample1.org")) {
                sampleTrue = 1;
            } else if (resource.equals(PopulateDB.nsNick + "|"
                                       + "ftp://sample3.org")) {
                sampleTrue = 2;
            }
            assertTrue(sampleTrue != 0);
            if (oldSample == 0) {
                oldSample = sampleTrue;
            } else {
                assertTrue(oldSample != sampleTrue);
            }
            Iterator actions = samlAuthzDecisionStmt.getActions();
            assertTrue(actions != null);
            int actCnt = 0;
            while (actions.hasNext()) {
                SAMLAction samlAction = (SAMLAction)actions.next();
                assertTrue(samlAction.getData().equals("read"));
                String actionNs = samlAction.getNamespace();
                assertTrue((actionNs.equals(PopulateDB.serviceType1))||
                           (actionNs.equals(PopulateDB.serviceType2)));
                actCnt++;
            }
            assertTrue(actCnt == 2);
            SAMLSubject subject = samlAuthzDecisionStmt.getSubject();
            assertTrue(subject != null);
        }
        logger.debug("stmt  done " + stmtCount);
        assertTrue(stmtCount == 2);
    }

    // one resource, one action
    private SAMLAuthorizationDecisionQuery constructScenarioTwoQuery()
        throws Exception {
        // resource
        String resource = PopulateDB.nsNick + "|" + "ftp://sample3.org";
        // Collection of actions to query on
        Vector actions = new Vector();
        // Conststruct an action
        // true
        SAMLAction samlAction = new SAMLAction(PopulateDB.serviceType1, "read");
        actions.add(samlAction);
        // false
        samlAction = new SAMLAction(PopulateDB.serviceType2, "write");
        actions.add(samlAction);
        // false
        samlAction = new SAMLAction(PopulateDB.serviceType1, "execute");
        return new SAMLAuthorizationDecisionQuery(samlSubject, resource,
                                               actions, null);
    }

    // one stmt, one action
    private void scenarioTwoVerify(SAMLAuthorizationDecisionStatement
                                      samlAuthzDecisionStmt, String resource)
        throws Exception {

        assertTrue(samlAuthzDecisionStmt!=null);
        String retResource = samlAuthzDecisionStmt.getResource();
        logger.debug("retResource " + retResource + " resource " + resource);
        assertTrue(retResource.equals(resource));
        assertTrue(samlAuthzDecisionStmt.getSubject() != null);
        Iterator actionIter = samlAuthzDecisionStmt.getActions();
        assertTrue(actionIter != null);
        int actionCnt = 0;
        while (actionIter.hasNext()) {
            SAMLAction retAction = (SAMLAction)actionIter.next();
            logger.debug(retAction.getData() + " " + retAction.getNamespace());
            assertTrue(retAction.getData().equals("read"));
            assertTrue(retAction.getNamespace().equals(
                                 PopulateDB.serviceType1));
            actionCnt++;
        }
        logger.debug("Action count " + actionCnt);
        assertTrue(actionCnt == 1);
    }

    public void scenarioThreeTest() throws Exception {
        logger.debug("scenario three");
        PopulateDB.scenarioThree(true);
        SAMLAuthorizationDecisionQuery samlAuthQuery =
            constructMaxQuery(samlSubject);
        SAMLAuthzQueryType[] samlQueries = new SAMLAuthzQueryType[1];
        SAMLAuthzQueryType samlQuery = new SAMLAuthzQueryType();
        samlQuery.set_any(new MessageElement[] {
            new SecurityMessageElement((Element)samlAuthQuery.toDOM())});
        samlQueries[0] = samlQuery;
        logger.debug("Scenario three - test 1");
        ArrayOfSAMLAuthzQueryType arrayOfSamlAuthz =
            new ArrayOfSAMLAuthzQueryType(samlQueries);
        org.globus.cas.types.SAMLAssertion assertion =
            casPort.getAssertion(getAssertion(arrayOfSamlAuthz, 100));
        Iterator smts = validateAssertion(assertion);
        validateScenarioMaxResult(smts, false);

        // resource
        String resource = PopulateDB.nsNick + "|" + "ftp://sample3.org";
        // Collection of actions to query on
        Vector actions = new Vector();
        // Conststruct an action
        // true
        SAMLAction samlAction = new SAMLAction(PopulateDB.serviceType1,
                                               "execute");
        actions.add(samlAction);
        // false
        samlAction = new SAMLAction(PopulateDB.serviceType2, "write");
        actions.add(samlAction);

        samlAuthQuery = new SAMLAuthorizationDecisionQuery(samlSubject,
                                                           resource,
                                                           actions, null);
        samlQuery = new SAMLAuthzQueryType();
        samlQuery.set_any(new MessageElement[] {
            new SecurityMessageElement((Element)samlAuthQuery.toDOM())});
        samlQueries = new SAMLAuthzQueryType[1];
        samlQueries[0] = samlQuery;
        samlQuery.set_any(new MessageElement[] {
            new SecurityMessageElement((Element)samlAuthQuery.toDOM())});
        logger.debug("Scenario three - test 2");
        arrayOfSamlAuthz = new ArrayOfSAMLAuthzQueryType(samlQueries);
        assertion =
            casPort.getAssertion(getAssertion(arrayOfSamlAuthz,100));
        logger.debug("Assertion is " + assertion);
        // wrong resource
        assertTrue(assertion == null);
        resource = PopulateDB.nsNick + "|" + "ftp://sample1.org";
        samlAuthQuery =
            new SAMLAuthorizationDecisionQuery(samlSubject, resource,
                                               actions, null);
        samlQueries = new SAMLAuthzQueryType[3];
        SAMLAuthzQueryType samlQuery0 = new SAMLAuthzQueryType();
        samlQuery0.set_any(new MessageElement[] {
            new SecurityMessageElement((Element)samlAuthQuery.toDOM())});
        samlQueries[0] = samlQuery0;
        logger.debug("samlAuth " + samlAuthQuery.getResource());
        SAMLAuthorizationDecisionQuery samlAuthQuery1
            = constructScenarioOneQuery ();
        SAMLAuthzQueryType samlQuery1 = new SAMLAuthzQueryType();
        samlQuery1.set_any(new MessageElement[] {
            new SecurityMessageElement((Element)samlAuthQuery1.toDOM())});
        samlQueries[1] = samlQuery1;
        logger.debug("samlAuth1 " + samlAuthQuery1.getResource());
        SAMLAuthorizationDecisionQuery samlAuthQuery2
            = constructScenarioTwoQuery ();
        SAMLAuthzQueryType samlQuery2 = new SAMLAuthzQueryType();
        samlQuery2.set_any(new MessageElement[] {
            new SecurityMessageElement((Element)samlAuthQuery2.toDOM())});
        samlQueries[2] = samlQuery2;
        logger.debug("samlAuth2 " + samlAuthQuery2.getResource());
        logger.debug("Scenario three - test 3");
        arrayOfSamlAuthz = new ArrayOfSAMLAuthzQueryType(samlQueries);
        assertion =
            casPort.getAssertion(getAssertion(arrayOfSamlAuthz,100));
        logger.debug("Assertion is " + assertion);
        // In this case three stmts will be gen per resource,
        // since there are threee queries.
        assertTrue(assertion != null);
        MessageElement[] msgElement = assertion.get_any();
        SAMLAssertion samlAssertion =
            new SAMLAssertion(msgElement[0].getAsDOM());
        logger.debug("Assertion " + samlAssertion.toString());
        Iterator stmts = samlAssertion.getStatements();
        assertTrue(stmts != null);
        assertTrue(stmts.hasNext());
        SAMLAuthorizationDecisionStatement samlAuthzDecisionStmt
            = (SAMLAuthorizationDecisionStatement)stmts.next();
        assertTrue(samlAuthzDecisionStmt!=null);
        String retResource = samlAuthzDecisionStmt.getResource();
        logger.debug("Returned resource " + retResource);
        scenarioThreeVerify(samlAuthzDecisionStmt);
        assertTrue(stmts.hasNext());
        samlAuthzDecisionStmt
            = (SAMLAuthorizationDecisionStatement)stmts.next();
        scenarioOneVerify(samlAuthzDecisionStmt,
                          samlAuthQuery1.getResource());
        assertTrue(stmts.hasNext());
        samlAuthzDecisionStmt
            = (SAMLAuthorizationDecisionStatement)stmts.next();
        scenarioTwoVerify(samlAuthzDecisionStmt,
                              samlAuthQuery2.getResource());

    }

    // if testFour is false, then scenario three or else scenario four
    private void validateScenarioMaxResult(Iterator stmts,
                                           boolean testFour) {
        assertTrue(stmts != null);
        int stmtCount=0;
        int sampleTrue = 0;
        int oldSample = 0;

        while (stmts.hasNext()) {
            stmtCount++;
            logger.debug("stmts " + stmtCount );
            SAMLAuthorizationDecisionStatement samlAuthzDecisionStmt
                = (SAMLAuthorizationDecisionStatement)stmts.next();
            logger.debug("stmt " + samlAuthzDecisionStmt.toString());
            assertTrue(samlAuthzDecisionStmt.getDecision().equals(
                                             CasConstants.SAML_DECISION));
            logger.debug("permit true");
            String resource = samlAuthzDecisionStmt.getResource();
            assertTrue(resource.indexOf(CasConstants.OBJECT_SELF) == -1);
            logger.debug("resource true");

            if (resource.equals(PopulateDB.nsNick + "|"
                                + "ftp://sample1.org")) {
                sampleTrue = 1;
            } else if (resource.equals(PopulateDB.nsNick + "|"
                                       + "ftp://sample2.org")) {
                sampleTrue = 2;
            } else if (resource.equals(PopulateDB.nsNick + "|"
                                       + "ftp://sample3.org")) {
                sampleTrue = 3;
            }
            assertTrue(sampleTrue != 0);
            assertTrue(oldSample != sampleTrue);
            oldSample = sampleTrue;
            Iterator actions = samlAuthzDecisionStmt.getActions();
            if ((sampleTrue == 1) || (sampleTrue == 2)) {
                int actCnt = 0;
                while (actions.hasNext()) {
                    SAMLAction samlAction = (SAMLAction)actions.next();
                    String actionNs = samlAction.getNamespace();
                    boolean opt1 = (samlAction.getData().equals("read")) &&
                        ((actionNs.equals(PopulateDB.serviceType1))||
                         (actionNs.equals(PopulateDB.serviceType2)));
                    boolean opt2 = (samlAction.getData().equals("execute")) &&
                        (actionNs.equals(PopulateDB.serviceType1));
                    boolean opt3 = (samlAction.getData().equals("random")) &&
                        (actionNs.equals(PopulateDB.serviceType3));
                    if (testFour) {
                        assertTrue(((sampleTrue == 1) &&
                                    (opt1 || opt2 || opt3))
                                   || ((sampleTrue == 2) && (opt1 || opt2)));
                    } else {
                        assertTrue(opt1 || opt2);
                    }
                    actCnt++;
                }
                if ((testFour) && (sampleTrue == 1))
                    assertTrue(actCnt == 4);
                else
                    assertTrue(actCnt == 3);
            } else if (sampleTrue == 3) {
                int actCnt = 0;
                while (actions.hasNext()) {
                    SAMLAction samlAction = (SAMLAction)actions.next();
                    assertTrue(samlAction.getData().equals("read"));
                    String actionNs = samlAction.getNamespace();
                    assertTrue((actionNs.equals(PopulateDB.serviceType1))||
                               (actionNs.equals(PopulateDB.serviceType2)));
                    actCnt++;
                }
                assertTrue(actCnt == 2);
            }
            SAMLSubject subject = samlAuthzDecisionStmt.getSubject();
            assertTrue(subject != null);
        }
        logger.debug("stmt  done " + stmtCount);
        assertTrue(stmtCount == 3);
    }

    private void scenarioThreeVerify(SAMLAuthorizationDecisionStatement stmt)
        throws Exception {

        assertTrue(stmt.getSubject() != null);
        Iterator actionIter = stmt.getActions();
        assertTrue(actionIter != null);
        int actionCnt = 0;
        while (actionIter.hasNext()) {
            SAMLAction retAction = (SAMLAction)actionIter.next();
            boolean opt1 = retAction.getData().equals("execute") &&
                retAction.getNamespace().equals(PopulateDB.serviceType1);
            assertTrue(opt1);
            actionCnt++;
        }
        logger.debug("action cnt " + actionCnt);
        assertTrue(actionCnt == 1);
    }

    public void scenarioFourTest() throws Exception {
        logger.debug("scenario four");
        PopulateDB.scenarioFour(true);
        SAMLAuthorizationDecisionQuery samlAuthQuery =
            constructMaxQuery(samlSubject);
        SAMLAuthzQueryType[] samlQueries = new SAMLAuthzQueryType[1];
        SAMLAuthzQueryType samlQuery = new SAMLAuthzQueryType();
        samlQuery.set_any(new MessageElement[] {
            new SecurityMessageElement((Element)samlAuthQuery.toDOM())});
        samlQueries[0] = samlQuery;
        logger.debug("Scenario four - test 1");
        ArrayOfSAMLAuthzQueryType arrayOfSamlAuthz =
            new ArrayOfSAMLAuthzQueryType(samlQueries);
        org.globus.cas.types.SAMLAssertion assertion =
            casPort.getAssertion(getAssertion(arrayOfSamlAuthz, 100));
        Iterator stmts = validateAssertion(assertion);
        validateScenarioMaxResult(stmts, true);
    }

    public Iterator
        validateAssertion(org.globus.cas.types.SAMLAssertion
                          assertion) throws Exception {

        assertTrue(assertion != null);
        MessageElement[] msgElement = assertion.get_any();
        SAMLAssertion samlAssertion =
            new SAMLAssertion(msgElement[0].getAsDOM());
        logger.debug("Assertion : " + samlAssertion.toString());
        samlAssertion.verify(false);

        return samlAssertion.getStatements();
    }

    // all resources, all actions.
    public SAMLAuthorizationDecisionQuery constructMaxQuery(SAMLSubject sub)
        throws Exception {
        String resource = CasConstants.RESOURCE_WILDCARD;
        Vector actions = new Vector();
        // Conststruct an action
        SAMLAction samlAction =
            new SAMLAction(CasConstants.ACTION_NS_WILDCARD,
                           CasConstants.ACTION_WILDCARD);
        actions.add(samlAction);
        return new SAMLAuthorizationDecisionQuery(sub, resource, actions, null);
    }
}

class PopulateDB {

    public static String nsNick = "FTPNamespace";
    public static String objId1 = null;
    public static String objId2 = null;
    public static String objId3 = null;
    public static String objGp1 = null;
    public static String objGp2 = null;
    public static String userGp = null;
    public static String serviceType1 = null;
    public static String serviceType2 = null;
    public static String serviceType3 = null;
    public static String serviceGp1 = null;
    public static String actionId1 = null;
    public static String actionId2 = null;
    public static String actionId3 = null;
    public static String actionId4 = null;
    public static String actionId5 = null;
    public static String actionId6 = null;

    public static String scenarioOnePolicy = null;
    public static String scenarioTwoPolicy = null;
    public static String scenarioThreePolicy1 = null;
    public static String scenarioThreePolicy2 = null;
    public static String scenarioFourPolicy1 = null;
    public static String scenarioFourPolicy2 = null;
    public static String implGp = null;

    public static void addExternalObjects() throws Exception {

        String basename = "basename1";
        String compAlg = "org.globus.cas.impl.databaseAccess.ExactComparison";
        NamespaceData nsData = new NamespaceData();
        nsData.setNickname(nsNick);
        nsData.setBasename(basename);
        nsData.setComparisonAlg(compAlg);
        NamespaceDataHandler.storeObject(nsData);

        // add external objects
        ObjectData objectData = new ObjectData();
        objectData.setObjectName("ftp://sample1.org");
        objectData.setObjectNamespace(nsNick);
        objId1 = ObjectDataHandler.storeObject(objectData);
        objectData.setObjectName("ftp://sample2.org");
        objId2 = ObjectDataHandler.storeObject(objectData);
        objectData.setObjectName("ftp://sample3.org");
        objId3 = ObjectDataHandler.storeObject(objectData);

        // add objectGroup
        objGp1 = "objectGroupName";
        ObjectGroupData objGroup = new ObjectGroupData();
        objGroup.setGroupName(objGp1);
        ObjectGroupDataHandler.storeObject(objGroup);

        // Add Obejct 1 and 2 to group
        ObjectGroupDataHandler.addObjectToGroup(objGp1, "ftp://sample1.org",
                                                nsNick);
        ObjectGroupDataHandler.addObjectToGroup(objGp1, "ftp://sample2.org",
                                                nsNick);

        // add object group with both implicit and external object
        objGp2 = "mixedObjGp";
        objGroup.setGroupName(objGp2);
        ObjectGroupDataHandler.storeObject(objGroup);
        // Add Obejct 1 and casServer
        ObjectGroupDataHandler.addObjectToGroup(objGp2, "ftp://sample1.org",
                                                nsNick);
        ObjectGroupDataHandler.addObjectToGroup(objGp2,
                                                CasConstants.OBJECT_SELF,
                                                CasConstants.NAMESPACE_SELF);

        // add service type
        serviceType1 = "fileType";
        ServiceTypeData serviceTypeData = new ServiceTypeData();
        serviceTypeData.setName(serviceType1);
        ServiceTypeDataHandler.storeObject(serviceTypeData);
        serviceType2 = "directory";
        serviceTypeData.setName(serviceType2);
        ServiceTypeDataHandler.storeObject(serviceTypeData);
        serviceType3 = "somthingElse";
        serviceTypeData.setName(serviceType3);
        ServiceTypeDataHandler.storeObject(serviceTypeData);

        // add action mapping
        actionId1 =
            ServiceTypeActionHandler.addServiceActionMapping(serviceType1,
                                                             "read");
        actionId2 =
            ServiceTypeActionHandler.addServiceActionMapping(serviceType1,
                                                             "write");
        actionId3 =
            ServiceTypeActionHandler.addServiceActionMapping(serviceType1,
                                                         "execute");
        actionId4 =
            ServiceTypeActionHandler.addServiceActionMapping(serviceType2,
                                                             "read");
        actionId5 =
            ServiceTypeActionHandler.addServiceActionMapping(serviceType2,
                                                             "write");
        actionId6 =
            ServiceTypeActionHandler.addServiceActionMapping(serviceType3,
                                                             "random");

        // add serviceActionGroup
        serviceGp1 = "readFtpGp";
        ServiceTypeActionHandler.createServiceActionGroup(serviceGp1);

        // add servicetype 1 and 2 selected actions
        ServiceTypeActionHandler.addServiceActionGpEntry(serviceGp1,
                                                         serviceType1,
                                                         "read");
        ServiceTypeActionHandler.addServiceActionGpEntry(serviceGp1,
                                                         serviceType2,
                                                         "read");
        // add another user group
        userGp = "externalUserGroup";
        UserGroupData userGpData = new UserGroupData();
        userGpData.setGroupName(userGp);
        UserGroupDataHandler.storeObject(userGpData);
    }

    static void removeExternalObjects() throws Exception {

        UserGroupDataHandler.deleteObject(userGp);
        ServiceTypeActionHandler.removeServiceActionGpEntry(serviceGp1,
                                                         serviceType1,
                                                         "read");
        ServiceTypeActionHandler.removeServiceActionGpEntry(serviceGp1,
                                                         serviceType2,
                                                         "read");
        ServiceTypeActionHandler.deleteServiceActionGroup(serviceGp1);

        ServiceTypeActionHandler.removeServiceActionMapping(serviceType1,
                                                         "read");
        ServiceTypeActionHandler.removeServiceActionMapping(serviceType1,
                                                         "write");
        ServiceTypeActionHandler.removeServiceActionMapping(serviceType1,
                                                         "execute");
        ServiceTypeActionHandler.removeServiceActionMapping(serviceType2,
                                                         "read");
        ServiceTypeActionHandler.removeServiceActionMapping(serviceType2,
                                                         "write");
        ServiceTypeActionHandler.removeServiceActionMapping(serviceType3,
                                                         "random");
        ServiceTypeDataHandler.deleteObject(serviceType3);
        ServiceTypeDataHandler.deleteObject(serviceType2);
        ServiceTypeDataHandler.deleteObject(serviceType1);

        ObjectGroupDataHandler.removeObjectFromGroup(objGp2,
                                                     "ftp://sample1.org",
                                                     nsNick);
        ObjectGroupDataHandler.removeObjectFromGroup(
                               objGp2,
                               CasConstants.OBJECT_SELF,
                               CasConstants.NAMESPACE_SELF);

        ObjectGroupDataHandler.deleteObject(objGp2);

        ObjectGroupDataHandler.removeObjectFromGroup(objGp1,
                                                     "ftp://sample1.org",
                                                     nsNick);
        ObjectGroupDataHandler.removeObjectFromGroup(
                                                     objGp1,
                                                     "ftp://sample2.org",
                                                     nsNick);
        ObjectGroupDataHandler.deleteObject(objGp1);
        ObjectDataHandler.deleteObjectForId(objId1);
        ObjectDataHandler.deleteObjectForId(objId2);
        ObjectDataHandler.deleteObjectForId(objId3);

    }

    // User belong to multiple user groups
    // Object Group of mixed objects
    // Action Group
    static void scenarioOne(boolean setup) throws Exception {

        if (setup) {
            UserGroupDataHandler.addGroupMember(userGp, TestConstants.user1);

            PolicyData policyData = new PolicyData();
            // Object Gp and action Gp
            policyData.setUserGroupName(TestConstants.defUserGp);
            policyData.setActionSpec(serviceGp1);
            policyData.setActionSpecDesc(CasConstants.SERVICEACTIONGP_SPEC);
            policyData.setObjectSpec(objGp2);
            policyData.setObjectSpecDesc(CasConstants.OBJECTGP_SPEC);
            scenarioOnePolicy = PolicyDataHandler.storeObject(policyData);
        }
        else {
            UserGroupDataHandler.removeGroupMember(userGp, TestConstants.user1);
            PolicyDataHandler.deleteObject(scenarioOnePolicy);
        }
    }

    // User belong to multiple user groups, user second gp
    // Object Group of mixed objects and an object
    // Action Group  (ading to scenario one)
    static void scenarioTwo(boolean setup) throws Exception {

        if (setup) {
            PolicyData policyData = new PolicyData();
            policyData.setUserGroupName(userGp);
            policyData.setActionSpec(serviceGp1);
            policyData.setActionSpecDesc(CasConstants.SERVICEACTIONGP_SPEC);
            policyData.setObjectSpec(objId3);
            policyData.setObjectSpecDesc(CasConstants.OBJECT_SPEC);
            scenarioTwoPolicy = PolicyDataHandler.storeObject(policyData);
        }
        else
            PolicyDataHandler.deleteObject(scenarioTwoPolicy);
    }

    // User belong to multiple user groups
    // Object Group of mixed objects, group of external  and an object
    // Action Group  (ading to scenario one) and an action
    static void scenarioThree(boolean setup) throws Exception {

        if (setup) {
            PolicyData policyData = new PolicyData();
            policyData.setUserGroupName(userGp);
            policyData.setActionSpec(serviceGp1);
            policyData.setActionSpecDesc(CasConstants.SERVICEACTIONGP_SPEC);
            policyData.setObjectSpec(objGp1);
            policyData.setObjectSpecDesc(CasConstants.OBJECTGP_SPEC);
            scenarioThreePolicy1 = PolicyDataHandler.storeObject(policyData);
            policyData.setActionSpec(actionId3);
            policyData.setActionSpecDesc(CasConstants.SERVICEACTION_SPEC);
            scenarioThreePolicy2 = PolicyDataHandler.storeObject(policyData);
            System.out.println("* actionId " + actionId3);
            System.out.println("* scenarioThreePolicy " + scenarioThreePolicy2);
        }
        else {
            PolicyDataHandler.deleteObject(scenarioThreePolicy1);
            PolicyDataHandler.deleteObject(scenarioThreePolicy2);
        }
    }

    // Add object/action, group of implicit objects
    static void scenarioFour(boolean setup) throws Exception {


        if (setup) {
            PolicyData policyData = new PolicyData();
            policyData.setUserGroupName(userGp);
            policyData.setActionSpec(actionId6);
            policyData.setActionSpecDesc(CasConstants.SERVICEACTION_SPEC);
            policyData.setObjectSpec(objId1);
            policyData.setObjectSpecDesc(CasConstants.OBJECT_SPEC);
            scenarioFourPolicy1 = PolicyDataHandler.storeObject(policyData);

            implGp = "implicitGroup";
            ObjectGroupData objGroup = new ObjectGroupData();
            objGroup.setGroupName(implGp);
            ObjectGroupDataHandler.storeObject(objGroup);

            // Add implicit object
            ObjectGroupDataHandler.addObjectToGroup(
                                   implGp,
                                   CasConstants.OBJECT_SELF,
                                   CasConstants.NAMESPACE_SELF);
            policyData.setActionSpec(actionId6);
            policyData.setActionSpecDesc(CasConstants.SERVICEACTION_SPEC);
            policyData.setObjectSpec(implGp);
            policyData.setObjectSpecDesc(CasConstants.OBJECTGP_SPEC);
            scenarioFourPolicy2 = PolicyDataHandler.storeObject(policyData);

        }
        else {
            PolicyDataHandler.deleteObject(scenarioFourPolicy1);
            PolicyDataHandler.deleteObject(scenarioFourPolicy2);
        }
    }
}
