/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.cas.impl.service;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.globus.cas.types.PolicyData;

import org.globus.cas.impl.CasConstants;

import org.globus.cas.impl.databaseAccess.PolicyDataHandler;
import org.globus.cas.impl.databaseAccess.ObjectDataHandler;
import org.globus.cas.impl.databaseAccess.ServiceTypeActionHandler;

import org.globus.cas.faults.NoPermissionFault;

import org.globus.cas.types.AddUser;
import org.globus.cas.types.CreateGroup;
import org.globus.cas.types.PolicyDetails;
import org.globus.cas.types.ManageUserGroups;
import org.globus.cas.types.CreateObjectNamespace;
import org.globus.cas.types.PolicyResponse;
import org.globus.cas.types.CreateServiceType;

import org.globus.cas.CASPortType;

import java.io.FileInputStream;
import java.util.Properties;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class TestGrantUser1 extends TestCase {

    static Log logger = LogFactory.getLog(TestGrantUser1.class.getName());
    static CASPortType casPort = null;

    String subjectName = null;
    String objId = null;
    String enrollAnchor = null;
    String enrollUser = null;
    String enrollNs = null;
    String enrollObject = null;
    String enrollService = null;
    String createObjGp = null;
    String createUserGp = null;
    String createServiceActionGp = null;
    String addGroupEntry = null;
    String unenrollId = null;
    String deleteGroupId = null;
    String deleteGroupEntry = null;
    String grantId = null;
    String revokeId = null;

    public TestGrantUser1(String name){
	super(name);
    }

    public static Test suite() {
        return new TestSuite(TestGrantUser1.class);
    }

    public static void 
	setParams(CASPortType casPort_) {
	casPort = casPort_;
    }

    // Direct database minipulation to set up the db for testing
    private void initialSetup() throws Exception {

	String propFileName = System.getProperty("casTestProperties");
	Properties prop = new Properties();
	prop.load(new FileInputStream(propFileName));
	subjectName = prop.getProperty("user2SubjectDN");
	
	objId = Integer.toString(ObjectDataHandler.getObjectId(
		CasConstants.OBJECT_SELF, CasConstants.NAMESPACE_SELF));
	enrollAnchor = 
	    Integer.toString(ServiceTypeActionHandler.getServiceActionId(
	    CasConstants.SERVICETYPE_CAS, CasConstants.ACTION_ENROLL_ANCHOR));
	enrollUser = 
	    Integer.toString(ServiceTypeActionHandler.getServiceActionId(
	    CasConstants.SERVICETYPE_CAS, CasConstants.ACTION_ENROLL_USER));
	enrollNs = 
	    Integer.toString(ServiceTypeActionHandler.getServiceActionId(
			     CasConstants.SERVICETYPE_CAS, 
			     CasConstants.ACTION_ENROLL_NAMESPACE));
	enrollObject = 
	    Integer.toString(ServiceTypeActionHandler.getServiceActionId(
	    CasConstants.SERVICETYPE_CAS, CasConstants.ACTION_ENROLL_OBJECT));
	enrollService = 
	    Integer.toString(ServiceTypeActionHandler.getServiceActionId(
	    CasConstants.SERVICETYPE_CAS, 
	    CasConstants.ACTION_CREATE_SERVICE_TYPE));
	createObjGp = 
	    Integer.toString(ServiceTypeActionHandler.getServiceActionId(
			     CasConstants.SERVICETYPE_CAS, 
			     CasConstants.ACTION_CREATE_OBJECT_GROUP));
	createUserGp = 
	    Integer.toString(ServiceTypeActionHandler.getServiceActionId(
			     CasConstants.SERVICETYPE_CAS, 
			     CasConstants.ACTION_CREATE_USER_GROUP));
	createServiceActionGp = 
	    Integer.toString(ServiceTypeActionHandler.getServiceActionId(
			     CasConstants.SERVICETYPE_CAS, 
			     CasConstants.ACTION_CREATE_SERVICEACTION_GROUP));
	addGroupEntry = 
	    Integer.toString(ServiceTypeActionHandler.getServiceActionId(
			     CasConstants.SERVICETYPE_CAS, 
			     CasConstants.ACTION_ADD_GROUP_ENTRY));
	grantId = 
	    Integer.toString(ServiceTypeActionHandler.getServiceActionId(
			     CasConstants.SERVICETYPE_CAS, 
			     CasConstants.ACTION_GRANT));
	revokeId = 
	    Integer.toString(ServiceTypeActionHandler.getServiceActionId(
			     CasConstants.SERVICETYPE_CAS, 
			     CasConstants.ACTION_REVOKE));
	unenrollId = 
	    Integer.toString(ServiceTypeActionHandler.getServiceActionId(
			     CasConstants.SERVICETYPE_CAS, 
			     CasConstants.ACTION_UNENROLL));
	deleteGroupId = 
	    Integer.toString(ServiceTypeActionHandler.getServiceActionId(
			     CasConstants.SERVICETYPE_CAS, 
			     CasConstants.ACTION_DELETE_GROUP));
	deleteGroupEntry = 
	    Integer.toString(ServiceTypeActionHandler.getServiceActionId(
			     CasConstants.SERVICETYPE_CAS, 
			     CasConstants.ACTION_REMOVE_GROUP_ENTRY));
    }
    
    public void testGrantRights() throws Exception {

	String selfObject = CasConstants.NAMESPACE_SELF 
	    + CasConstants.OBJECTSPEC_DELIMITER + CasConstants.OBJECT_SELF;
	initialSetup();

	PolicyData policyData = new PolicyData();
	policyData.setUserGroupName("userGroup");

	// Create user group
	casPort.createGroup(new CreateGroup(TestConstants.userGp2, "user", 
                                            TestConstants.userGp2));

	// Enroll User
	casPort.addUser(new AddUser(TestConstants.user2, subjectName,
                                    TestConstants.defTrustNick, 
                                    TestConstants.userGp2));


	// Create object group
	casPort.createGroup(new CreateGroup(TestConstants.objGp2, "object", 
                                            TestConstants.userGp2));

	// Create namespace
	casPort.createObjectNamespace(
                new CreateObjectNamespace(TestConstants.baseName, 
                                          TestConstants.compAlg,
                                          TestConstants.nsNick, 
                                          TestConstants.userGp2)); 

	// add to user group
	casPort.manageUserGroups(new ManageUserGroups(TestConstants.userGp2,
                                                      "add", 
                                                      TestConstants.user2));

	// Grant user1 permission to grant by direct DB manipulation
	policyData.setObjectSpec(objId);
	policyData.setObjectSpecDesc(CasConstants.OBJECT_SPEC);
	policyData.setActionSpec(grantId);
	policyData.setActionSpecDesc(CasConstants.SERVICEACTION_SPEC);
	PolicyDataHandler.storeObject(policyData);

	// grant permissions for stuff to the new user i.e userGroup2
        PolicyResponse res = casPort
            .grant(new PolicyDetails(CasConstants.SERVICETYPE_CAS + "/" 
                                     + CasConstants.ACTION_ENROLL_ANCHOR,
                                     CasConstants.SERVICEACTION_SPEC,
                                     selfObject,
                                     CasConstants.OBJECT_SPEC, 
                                     TestConstants.userGp2));
        PolicyData grantPolicy = (PolicyData)res.getPolicy();
	String policyId = grantPolicy.getPolicyId();

	boolean exp = false;
	try {
	    casPort
                .revoke(new PolicyDetails(CasConstants.SERVICETYPE_CAS + "/" 
                                          + CasConstants.ACTION_ENROLL_ANCHOR,
                                          CasConstants.SERVICEACTION_SPEC,
                                          selfObject,
                                          CasConstants.OBJECT_SPEC, 
                                          TestConstants.userGp2));
        } catch (NoPermissionFault fault) {
	    exp = true;
	}
	assertTrue(exp);

	// add cas/revoke permissions on  that policyId
	policyData.setObjectSpec(policyId);
	policyData.setObjectSpecDesc(CasConstants.POLICY_SPEC);
	policyData.setActionSpec(revokeId);
	policyData.setActionSpecDesc(CasConstants.SERVICEACTION_SPEC);
	PolicyDataHandler.storeObject(policyData);
	
	// Revoke should go through
	casPort.revoke(new PolicyDetails(CasConstants.SERVICETYPE_CAS + "/" 
                                         + CasConstants.ACTION_ENROLL_ANCHOR,
                                         CasConstants.SERVICEACTION_SPEC,
                                         selfObject,
                                         CasConstants.OBJECT_SPEC, 
                                         TestConstants.userGp2));
	
	// Ensure it was revoked by attempting to get the Policy Data object
	PolicyData revokedPolicy = 
	    (PolicyData)PolicyDataHandler.retrieveObject(policyId);
	assertTrue(revokedPolicy == null);
	
	// Restore granted rights
        PolicyDetails policyDetails = 
            new PolicyDetails(CasConstants.SERVICETYPE_CAS + 
                              CasConstants.SERVICEACTION_DELIMITER
                              + CasConstants.ACTION_ENROLL_ANCHOR,
                              CasConstants.SERVICEACTION_SPEC,
                              selfObject,
                              CasConstants.OBJECT_SPEC, 
                              TestConstants.userGp2);

        res = casPort.grant(policyDetails);
	grantPolicy = 	(PolicyData)res.getPolicy();
        // Store permissions on this policy
	policyData.setObjectSpec(grantPolicy.getPolicyId());
	PolicyDataHandler.storeObject(policyData);

	// Revoke with policyData
	casPort.revoke(new PolicyDetails(CasConstants.SERVICETYPE_CAS + "/" + 
                                         CasConstants.ACTION_ENROLL_ANCHOR, 
                                         CasConstants.SERVICEACTION_SPEC,
                                         selfObject,
                                         CasConstants.OBJECT_SPEC, 
                                         TestConstants.userGp2));

	// Restore granted rights
	casPort.grant(new PolicyDetails(CasConstants.SERVICETYPE_CAS + "/" 
                                        + CasConstants.ACTION_ENROLL_ANCHOR,
                                        CasConstants.SERVICEACTION_SPEC,
                                        selfObject,
                                        CasConstants.OBJECT_SPEC, 
                                        TestConstants.userGp2));
	
	// Grant new user permission to enroll user
	casPort.grant(new PolicyDetails(CasConstants.SERVICETYPE_CAS + "/" 
                                        + CasConstants.ACTION_ENROLL_USER,
                                        CasConstants.SERVICEACTION_SPEC,
                                        selfObject,
                                        CasConstants.OBJECT_SPEC, 
                                        TestConstants.userGp2));
	
	// Grant new user permission to enroll namespace
	casPort.grant(new PolicyDetails(CasConstants.SERVICETYPE_CAS + "/" + 
                                        CasConstants.ACTION_ENROLL_NAMESPACE,
                                        CasConstants.SERVICEACTION_SPEC,
                                        selfObject, 
                                        CasConstants.OBJECT_SPEC, 
                                        TestConstants.userGp2));

	// Grant new user permission to enroll object
	casPort.grant(new PolicyDetails(CasConstants.SERVICETYPE_CAS + "/" + 
                                        CasConstants.ACTION_ENROLL_OBJECT,
                                        CasConstants.SERVICEACTION_SPEC,
                                        selfObject, 
                                        CasConstants.OBJECT_SPEC, 
                                        TestConstants.userGp2));
	
	// Grant new user permission to enroll service
	casPort
            .grant(new PolicyDetails(CasConstants.SERVICETYPE_CAS + "/" + 
                                     CasConstants.ACTION_CREATE_SERVICE_TYPE,
                                     CasConstants.SERVICEACTION_SPEC,
                                     selfObject, 
                                     CasConstants.OBJECT_SPEC, 
                                     TestConstants.userGp2));
	
	// Grant new user permission to create user group
	casPort.grant(new PolicyDetails(CasConstants.SERVICETYPE_CAS + "/" + 
                                        CasConstants.ACTION_CREATE_USER_GROUP,
                                        CasConstants.SERVICEACTION_SPEC,
                                        selfObject, 
                                        CasConstants.OBJECT_SPEC, 
                                        TestConstants.userGp2));
	
	// Grant new user permission to create object group
	casPort
            .grant(new PolicyDetails(CasConstants.SERVICETYPE_CAS + "/" + 
                                     CasConstants.ACTION_CREATE_OBJECT_GROUP, 
                                     CasConstants.SERVICEACTION_SPEC,
                                     selfObject, 
                                     CasConstants.OBJECT_SPEC, 
                                     TestConstants.userGp2));

	// Grant new user permission to create s/a group
	casPort.grant(new PolicyDetails(CasConstants.SERVICETYPE_CAS + "/" + 
                                        CasConstants
                                        .ACTION_CREATE_SERVICEACTION_GROUP,
                                        CasConstants.SERVICEACTION_SPEC,
                                        selfObject, 
                                        CasConstants.OBJECT_SPEC, 
                                        TestConstants.userGp2));

	// create new gp and grantAll to default group . 
	// (to give permissions to user2)
	casPort.createGroup(new CreateGroup(TestConstants.objGpUser2, "object",
                                            TestConstants.defUserGp)); 

	// Grant new user permission to add entry into 
	// TestConstants.objectGpName
	casPort.grant(new PolicyDetails(CasConstants.SERVICETYPE_CAS + "/" 
                                        + CasConstants.ACTION_ADD_GROUP_ENTRY,
                                        CasConstants.SERVICEACTION_SPEC,
                                        TestConstants.objGpUser2, 
                                        CasConstants.OBJECTGP_SPEC,
                                        TestConstants.userGp2));
	
	// Grant new user permission to add entry into TestConstants.userGp
	casPort.grant(new PolicyDetails(CasConstants.SERVICETYPE_CAS + "/" 
                                        + CasConstants.ACTION_ADD_GROUP_ENTRY,
                                        CasConstants.SERVICEACTION_SPEC,
                                        TestConstants.userGp2, 
                                        CasConstants.USERGP_SPEC,
                                        TestConstants.userGp2));
	
	// create new service type and grantAll to default group 
	// (to give permissins to user2)
        CreateServiceType cst = 
            new CreateServiceType(TestConstants.serviceNameUser2, 
                                  TestConstants.defUserGp);
	casPort.createServiceType(cst);

	// Grant new user permission to add service action mapping
	casPort.grant(new PolicyDetails(CasConstants.SERVICETYPE_CAS + "/" 
                                        + CasConstants.ACTION_ADD_GROUP_ENTRY,
                                        CasConstants.SERVICEACTION_SPEC,
                                        TestConstants.serviceNameUser2, 
                                        CasConstants.SERVICETYPE_SPEC ,
                                        TestConstants.userGp2));
	
	// Grant new user permission to delete  TestConstants.objectGpName
	casPort.grant(new PolicyDetails(CasConstants.SERVICETYPE_CAS + "/" 
                                        + CasConstants.ACTION_DELETE_GROUP,
                                        CasConstants.SERVICEACTION_SPEC,
                                        TestConstants.objGp2, 
                                        CasConstants.OBJECTGP_SPEC,
                                        TestConstants.userGp2));
	
	// Grant new user permission to unenroll
	casPort.grant(new PolicyDetails(CasConstants.SERVICETYPE_CAS + "/" 
                                        + CasConstants.ACTION_UNENROLL,
                                        CasConstants.SERVICEACTION_SPEC,
                                        TestConstants.nsNick, 
                                        CasConstants.NAMESPACE_SPEC ,
                                        TestConstants.userGp2));
    }
}
