/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.cas.impl.service;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.globus.cas.types.ServiceTypeData;
import org.globus.cas.types.TrustAnchorData;
import org.globus.cas.types.UserData;
import org.globus.cas.types.UserGroupData;
import org.globus.cas.types.ObjectData;
import org.globus.cas.types.NamespaceData;
import org.globus.cas.types.ObjectGroupData;
import org.globus.cas.types.ArrayOfString;

import org.globus.cas.types.AddUser;
import org.globus.cas.types.CreateGroup;
import org.globus.cas.types.ManageServiceAction;
import org.globus.cas.types.ManageServiceActionGroups;
import org.globus.cas.types.ManageObjectGroups;
import org.globus.cas.types.AddTrustAnchor;
import org.globus.cas.types.CreateObject;
import org.globus.cas.types.CreateObjectNamespace;
import org.globus.cas.types.CreateServiceType;
import org.globus.cas.types.ManageUserGroups;

import org.globus.cas.utils.CasStringUtils;

import org.globus.cas.impl.databaseAccess.ObjectDataHandler;
import org.globus.cas.impl.databaseAccess.ObjectGroupDataHandler;
import org.globus.cas.impl.databaseAccess.UserDataHandler;
import org.globus.cas.impl.databaseAccess.UserGroupDataHandler;
import org.globus.cas.impl.databaseAccess.TrustAnchorDataHandler;
import org.globus.cas.impl.databaseAccess.ServiceTypeDataHandler;
import org.globus.cas.impl.databaseAccess.ServiceTypeActionHandler;
import org.globus.cas.impl.databaseAccess.NamespaceDataHandler;

import org.globus.cas.CASPortType;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class TestGrantUser2 extends TestCase {

    static Log logger = LogFactory.getLog(TestGrantUser2.class.getName());
    static CASPortType casPort = null;

    String groupName = null;
    String objId = null;
    String enrollAnchor = null;
    String enrollUser = null;
    String enrollNs = null;
    String enrollObject = null;
    String enrollService = null;
    String createObjGp = null;
    String createUserGp = null;
    String createServiceActionGp = null;
    String addGroupEntry = null;
    
    public TestGrantUser2(String name){
	super(name);
    }

    public static Test suite() {
        return new TestSuite(TestGrantUser2.class);
    }

    public static void setParams(CASPortType casPort_) {
	casPort = casPort_;
    }

    public void testEnrollData() throws Exception {

        assertTrue(casPort != null);
	// add enrollTrustAnchor
	casPort.addTrustAnchor(new AddTrustAnchor(TestConstants.authData,
                                                  TestConstants.authMeth,
                                                  TestConstants.trustNick,
                                                  TestConstants.userGp2));

	// Ensure it was added
	TrustAnchorData addedAnchor = new TrustAnchorData();
	addedAnchor.setNickname(TestConstants.trustNick);
	addedAnchor.setAuthMethod(TestConstants.authMeth);
	addedAnchor.setAuthData(TestConstants.authData);
	assertTrue(addedAnchor.equals(
		  (TrustAnchorData)TrustAnchorDataHandler.retrieveObject(
		  TestConstants.trustNick)));
	
	// attempt adding user
	casPort.addUser(new AddUser(TestConstants.userNick,
                                    TestConstants.subjectName, 
                                    TestConstants.trustNick,
                                    TestConstants.userGp2));

	// Ensure it was added
	UserData addedUserData = new UserData();
	addedUserData.setNickname(TestConstants.userNick);
	addedUserData.setSubjectName(TestConstants.subjectName);
	addedUserData.setTrustAnchorName(TestConstants.trustNick);
	assertTrue(addedUserData.equals(
		  (UserData)UserDataHandler.retrieveObject(
					    TestConstants.userNick)));

	// attempt adding namespace
	casPort.createObjectNamespace(
                new CreateObjectNamespace(TestConstants.user2BaseName,
                                          TestConstants.compAlg,
                                          TestConstants.user2NsNick,
                                          TestConstants.userGp2));

	// Ensure it was added
	NamespaceData nsData = new NamespaceData();
	nsData.setNickname(TestConstants.user2NsNick);
	nsData.setBasename(TestConstants.user2BaseName);
	nsData.setComparisonAlg(TestConstants.compAlg);
	assertTrue(nsData.equals(
		  (NamespaceData)NamespaceDataHandler.retrieveObject(
		  TestConstants.user2NsNick)));

	// attempt removing the older NS
	casPort.deleteObjectNamespace(TestConstants.nsNick);
	assertTrue(NamespaceDataHandler.retrieveObject(
					TestConstants.nsNick) == null);

	// attempt adding object
	casPort.createObject(new CreateObject(TestConstants.objName,
                                              TestConstants.user2NsNick,
                                              TestConstants.userGp2));

        // Ensure it was added
	ObjectData objData = new ObjectData();
	objData.setObjectName(TestConstants.objName);
	objData.setObjectNamespace(TestConstants.user2NsNick);
	ObjectData retrievedObj = 
	    (ObjectData)ObjectDataHandler.retrieveObject(
					  TestConstants.objName, 
					  TestConstants.user2NsNick);
	assertTrue(retrievedObj != null);
	// FIXME
	String addedObj = retrievedObj.getObjectId();
	objData.setObjectId(addedObj);
	assertTrue(objData.equals(retrievedObj));

	// attempt creating userGp
	casPort.createGroup(new CreateGroup(TestConstants.userGpName, "user",
                                            TestConstants.userGp2));
	// Ensure it was added
	UserGroupData userGpData = new UserGroupData();
	userGpData.setGroupName(TestConstants.userGpName);
	assertTrue(userGpData.equals(
		  (UserGroupData)UserGroupDataHandler.retrieveObject(
		  TestConstants.userGpName)));

	// attempt adding to user group
       	casPort.manageUserGroups(
                new ManageUserGroups(TestConstants.userGpName, "add", 
                                     TestConstants.userNick));
	// Ensure it was added
	userGpData = 
	    (UserGroupData)UserGroupDataHandler.retrieveObject(
						TestConstants.userGpName);
        assertTrue(userGpData.getUserNames() != null);
	assertTrue(CasStringUtils
                   .stringInArray(userGpData.getUserNames().getStrings(), 
                                  TestConstants.userNick));

	// attempt adding to user group created by user1
       	casPort.manageUserGroups(new ManageUserGroups(TestConstants.userGp2, 
                                                      "add", 
                                                      TestConstants.userNick));
	// Ensure it was added
	userGpData = 
	    (UserGroupData)UserGroupDataHandler.retrieveObject(
						TestConstants.userGp2);
        assertTrue(userGpData.getUserNames() != null);
	assertTrue(CasStringUtils
                   .stringInArray(userGpData.getUserNames().getStrings(), 
                                  TestConstants.userNick));
	
	// attempt creating objectGP
	casPort.createGroup(new CreateGroup(TestConstants.objectGpName, 
                                            "object", TestConstants.userGp2));

	// Ensure it was added
	ObjectGroupData objectGpData = new ObjectGroupData();
	objectGpData.setGroupName(TestConstants.objectGpName);
	assertTrue(objectGpData.equals((ObjectGroupData)ObjectGroupDataHandler.retrieveObject(TestConstants.objectGpName)));
	
	// attempt adding to object group
	String addedObjString = TestConstants.user2NsNick + "|" 
	    + TestConstants.objName;
	casPort.manageObjectGroups(
                new ManageObjectGroups(TestConstants.objectGpName, 
                                       addedObjString, "object", "add"));
	// Ensure it was added
	objectGpData = (ObjectGroupData)ObjectGroupDataHandler
            .retrieveObject(TestConstants.objectGpName);
        assertTrue(objectGpData.getObjectSpecs() != null);
	assertTrue(CasStringUtils
                   .stringInArray(objectGpData.getObjectSpecs().getStrings(),
                                  addedObj));

	// attempt adding to object group created by user1
	casPort.manageObjectGroups(
                new ManageObjectGroups(TestConstants.objGp2, 
                                       addedObjString, "object", "add"));
	objectGpData = 
	    (ObjectGroupData)ObjectGroupDataHandler.retrieveObject(
						    TestConstants.objGp2);
        assertTrue(objectGpData.getObjectSpecs() != null);
	assertTrue(CasStringUtils
                   .stringInArray(objectGpData.getObjectSpecs().getStrings(),
                                  addedObj));
	
	// attempting to createServiceType
	casPort.createServiceType(
                new CreateServiceType(TestConstants.serviceName2,
                                      TestConstants.userGp2));

	// Ensure it was added
	ServiceTypeData serviceType = new ServiceTypeData();
	serviceType.setName(TestConstants.serviceName2);
	assertTrue(serviceType.equals((ServiceTypeData)ServiceTypeDataHandler.retrieveObject(TestConstants.serviceName2)));

	// attempting to add serviceAction mapping
	casPort.manageServiceAction(
                new ManageServiceAction(TestConstants.actionName, "add", 
                                        TestConstants.serviceName2));

	// Ensure it was added
	String actNames[] = new String[1];
	actNames[0] = TestConstants.actionName;
	serviceType.setActionNames(new ArrayOfString(actNames));
	assertTrue(serviceType.equals((ServiceTypeData)ServiceTypeDataHandler.retrieveObject(TestConstants.serviceName2)));

	// attempt adding serviceAction mapping to service Type added by user1
	casPort.manageServiceAction(
                new ManageServiceAction(TestConstants.actionName, "add", 
                                        TestConstants.serviceNameUser2));

	String[] serviceAction = 
	ServiceTypeActionHandler.retrieveServiceActionMappings(
				 TestConstants.serviceNameUser2);
	assertTrue(CasStringUtils.stringInArray(serviceAction, 
						TestConstants.actionName));

	// attempt creating serviceActionGp
	casPort
            .createGroup(new CreateGroup(TestConstants.serviceActionGpName, 
                                         "serviceAction", 
                                         TestConstants.userGp2));

	// Ensure it was added
	String[] serviceActionGp = 
	    ServiceTypeActionHandler.listServiceActionGroups();
	assertTrue(CasStringUtils.stringInArray(serviceActionGp, 
				 TestConstants.serviceActionGpName));

	// attempt adding to serviceActionGp
	casPort
            .manageServiceActionGroups(new ManageServiceActionGroups(
                                           TestConstants.actionName,
                                           TestConstants.serviceActionGpName,
                                           "add",
                                           TestConstants.serviceNameUser2));

	// Ensure it was added
	String[] serviceActionGpEntry = 
	    ServiceTypeActionHandler.retrieveServiceActionGpEntries(
				     TestConstants.serviceActionGpName);
	assertTrue(CasStringUtils.stringInArray(serviceActionGpEntry, 
				 TestConstants.serviceNameUser2 + "/" + 
				 TestConstants.actionName));
    }	
}
