/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.cas.impl.service;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.globus.cas.faults.CasFault;
import org.globus.cas.faults.NoPermissionFault;

import org.globus.cas.types.UserData;
import org.globus.cas.types.PolicyData;
import org.globus.cas.types.ObjectData;
import org.globus.cas.types.UserGroupData;
import org.globus.cas.types.NamespaceData;
import org.globus.cas.types.ObjectGroupData;
import org.globus.cas.types.TrustAnchorData;
import org.globus.cas.types.ServiceTypeData;
import org.globus.cas.types.ArrayOfString;

import org.globus.cas.types.AddUser;
import org.globus.cas.types.CreateGroup;
import org.globus.cas.types.CreateObject;
import org.globus.cas.types.AddTrustAnchor;
import org.globus.cas.types.ManageUserGroups;
import org.globus.cas.types.ManageObjectGroups;
import org.globus.cas.types.ManageServiceAction;
import org.globus.cas.types.CreateServiceType;
import org.globus.cas.types.CreateObjectNamespace;
import org.globus.cas.types.ManageServiceActionGroups;

import org.globus.cas.impl.databaseAccess.UserDataHandler;
import org.globus.cas.impl.databaseAccess.ObjectDataHandler;
import org.globus.cas.impl.databaseAccess.PolicyDataHandler;
import org.globus.cas.impl.databaseAccess.UserGroupDataHandler;
import org.globus.cas.impl.databaseAccess.NamespaceDataHandler;
import org.globus.cas.impl.databaseAccess.TrustAnchorDataHandler;
import org.globus.cas.impl.databaseAccess.ServiceTypeDataHandler;
import org.globus.cas.impl.databaseAccess.ObjectGroupDataHandler;
import org.globus.cas.impl.databaseAccess.ServiceTypeActionHandler;

import org.globus.cas.utils.CasStringUtils;

import org.globus.cas.impl.CasConstants;

import org.globus.cas.CASPortType;

import java.io.FileInputStream;

import java.util.Properties;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class TestSelfEnrollPermissions extends TestCase {

    static Log logger = 
        LogFactory.getLog(TestSelfEnrollPermissions.class.getName());

    static CASPortType casPort = null;
    
    String objId = null;
    String enrollAnchor = null;
    String enrollUser = null;
    String enrollNs = null;
    String enrollObject = null;
    String enrollService = null;
    String createObjGp = null;
    String createUserGp = null;
    String createServiceActionGp = null;
    String addGroupEntry = null;

    public TestSelfEnrollPermissions(String name){
        super(name);
    }

    public static Test suite() {
        return new TestSuite(TestSelfEnrollPermissions.class);
    }

    public static void setParams(CASPortType casPort_) {
        casPort = casPort_;
    }

    // Direct database minipulation to set up the db for testing
    private void initialSetup() throws Exception {

        String propFileName = System.getProperty("casTestProperties");
        Properties prop = new Properties();
        prop.load(new FileInputStream(propFileName));
        String subjectName = prop.getProperty("user1SubjectDN");

        // Store Trust anchor data
        String authMethod = "authMethod";
        String authData = TestConstants.defTrustNickDN;
        TrustAnchorData trustAnchor = new TrustAnchorData();
        trustAnchor.setNickname(TestConstants.defTrustNick);
        trustAnchor.setAuthMethod(authMethod);
        trustAnchor.setAuthData(authData);
        TrustAnchorDataHandler.storeObject(trustAnchor);

        // Store user data 
        UserData userData1 = new UserData();
        userData1.setNickname(TestConstants.user1);
        userData1.setSubjectName(subjectName);
        userData1.setTrustAnchorName(TestConstants.defTrustNick);
        UserDataHandler.storeObject(userData1);
        logger.debug("Store user data with group went through ");

        // UserGroup data store without usernames
        UserGroupData userGpData = new UserGroupData();
        userGpData.setGroupName(TestConstants.defUserGp);
        UserGroupDataHandler.storeObject(userGpData);
        logger.debug("Store user group data went through ");

        // add to user group
        UserGroupDataHandler.addGroupMember(TestConstants.defUserGp, 
                                            TestConstants.user1);
        
        objId = 
            Integer.toString(ObjectDataHandler.getObjectId(
                                               CasConstants.OBJECT_SELF, 
                                               CasConstants.NAMESPACE_SELF));
        enrollAnchor = 
            Integer.toString(ServiceTypeActionHandler.getServiceActionId(
                             CasConstants.SERVICETYPE_CAS, 
                             CasConstants.ACTION_ENROLL_ANCHOR));
        enrollUser = 
            Integer.toString(
                    ServiceTypeActionHandler.getServiceActionId(
                                             CasConstants.SERVICETYPE_CAS, 
                                             CasConstants.ACTION_ENROLL_USER));
        enrollNs = Integer.toString(
                           ServiceTypeActionHandler.getServiceActionId(
                           CasConstants.SERVICETYPE_CAS, 
                           CasConstants.ACTION_ENROLL_NAMESPACE));
        enrollObject = Integer.toString(
                               ServiceTypeActionHandler.getServiceActionId(
                               CasConstants.SERVICETYPE_CAS, 
                               CasConstants.ACTION_ENROLL_OBJECT));
        enrollService = Integer.toString(
                                ServiceTypeActionHandler.getServiceActionId(
                                CasConstants.SERVICETYPE_CAS, 
                                CasConstants.ACTION_CREATE_SERVICE_TYPE));
        createObjGp = Integer.toString(
                              ServiceTypeActionHandler.getServiceActionId(
                              CasConstants.SERVICETYPE_CAS, 
                              CasConstants.ACTION_CREATE_OBJECT_GROUP));
        createUserGp = Integer.toString(
                               ServiceTypeActionHandler.getServiceActionId(
                               CasConstants.SERVICETYPE_CAS, 
                               CasConstants.ACTION_CREATE_USER_GROUP));
        createServiceActionGp = 
            Integer.toString(
                             ServiceTypeActionHandler.getServiceActionId(
                             CasConstants.SERVICETYPE_CAS, 
                             CasConstants.ACTION_CREATE_SERVICEACTION_GROUP));
        addGroupEntry = 
            Integer.toString(ServiceTypeActionHandler.getServiceActionId(
                             CasConstants.SERVICETYPE_CAS, 
                             CasConstants.ACTION_ADD_GROUP_ENTRY));
        
    }

    /* FIXME: test RPs.
    public void testServiceData() throws Exception {
        String propFileName = System.getProperty("casTestProperties");
        Properties prop = new Properties();
        prop.load(new FileInputStream(propFileName));
        String serverDN = prop.getProperty("serverDN");

        QName dnQName = new QName("http://ogsa.globus.org/base/cas", 
                                  "ServerDN");
        ExtensibilityType dnResult = 
            casPort.findServiceData(QueryHelper.getNamesQuery(dnQName));
        ServiceDataValuesType serviceData =
            AnyHelper.getAsServiceDataValues(dnResult);
        assertTrue(serviceData.get_any() != null);
        Object[] entries = 
            AnyHelper.getAsObject(serviceData, ServerDN.class); 
        String retVal = ((ServerDN)entries[0]).getServerDN();
        logger.debug(retVal);
        assertTrue(retVal != null);
        assertTrue(retVal.equals(serverDN));

    }
    */

    public void testEnrollData() throws Exception {

        initialSetup();

        PolicyData policyData = new PolicyData();
        policyData.setUserGroupName(TestConstants.defUserGp);

        // attempt adding trust anchor
        boolean exception = false;
        try {
            casPort
                .addTrustAnchor(new AddTrustAnchor(TestConstants.authData,
                                                   TestConstants.authMeth, 
                                                   TestConstants.trustNick,
                                                   TestConstants.defUserGp));
        } catch (NoPermissionFault casFault) {
            exception = true;
        }
        assertTrue(exception);

        // Set permission to add enrollTrustAnchor
        policyData.setObjectSpec(objId);
        policyData.setObjectSpecDesc(CasConstants.OBJECT_SPEC);
        policyData.setActionSpec(enrollAnchor);
        policyData.setActionSpecDesc(CasConstants.SERVICEACTION_SPEC);
        PolicyDataHandler.storeObject(policyData);

        // attempt adding with bad user group for grant all permissions
        exception = false;
        try {
            casPort.addTrustAnchor(new AddTrustAnchor(TestConstants.authData,
                                                      TestConstants.authMeth,
                                                      TestConstants.trustNick, 
                                                      "dummyUserGroup"));
        } catch (CasFault casFault) {
            if ((casFault.getDescription(0).toString()
                 .indexOf("does not exist"))!=-1) {
                exception = true;
            } else 
                logger.error("Error" + casFault.getDescription(0).toString());
        }
        assertTrue(exception);

        // Invoke method after permission has been added
        casPort.addTrustAnchor(new AddTrustAnchor(TestConstants.authData, 
                                                  TestConstants.authMeth,
                                                  TestConstants.trustNick,
                                                  TestConstants.defUserGp));
        // Ensure it was added
        TrustAnchorData addedAnchor = new TrustAnchorData();
        addedAnchor.setNickname(TestConstants.trustNick);
        addedAnchor.setAuthMethod(TestConstants.authMeth);
        addedAnchor.setAuthData(TestConstants.authData);
        assertTrue(addedAnchor.equals(
                  (TrustAnchorData)TrustAnchorDataHandler.retrieveObject(
                  TestConstants.trustNick)));
        
        // Attempt adding trust anchor same authdata i.e DN and authMethod
        exception = false;
        try {
            casPort.addTrustAnchor(new AddTrustAnchor(TestConstants.authData,
                                                      TestConstants.authMeth,
                                                      TestConstants.trustNick,
                                                      "DummyTrust"));
        } catch (CasFault casFault) {
            exception = true;
        }
        assertTrue(exception);

        // attempt adding user
        exception = false;
        try {
            casPort.addUser(new AddUser(TestConstants.userNick,
                                        TestConstants.subjectName,
                                        TestConstants.trustNick,
                                        TestConstants.defUserGp));
        } catch (NoPermissionFault casFault) {
            exception = true;
        }
        assertTrue(exception);
        // Set permission to add enrolluser
        policyData.setObjectSpec(objId);
        policyData.setObjectSpecDesc(CasConstants.OBJECT_SPEC);
        policyData.setActionSpec(enrollUser);
        policyData.setActionSpecDesc(CasConstants.SERVICEACTION_SPEC);
        PolicyDataHandler.storeObject(policyData);

        // attempt adding with bad user group for grant all permissions
        exception = false;
        try {
            casPort.addUser(new AddUser(TestConstants.userNick, 
                                        TestConstants.subjectName, 
                                        TestConstants.trustNick, 
                                        "dummyUserGp"));
        } catch (CasFault casFault) {
            if (casFault.getDescription(0).toString()
                .indexOf("does not exist")!=-1) {
                exception = true;
            } else {
                logger.error("Error" + casFault.getDescription(0).toString());
            }
        }
        assertTrue(exception);

        // Invoke method after permission has been added
        casPort.addUser(new AddUser(TestConstants.userNick,
                                    TestConstants.subjectName, 
                                    TestConstants.trustNick, 
                                    TestConstants.defUserGp));
        // Ensure it was added
        UserData addedUserData = new UserData();
        addedUserData.setNickname(TestConstants.userNick);
        addedUserData.setSubjectName(TestConstants.subjectName);
        addedUserData.setTrustAnchorName(TestConstants.trustNick);
        assertTrue(addedUserData.equals(
                   (UserData)UserDataHandler.retrieveObject(
                                             TestConstants.userNick)));

        // Attempt adding user same subject name and trust anchor nick  
        exception = false;
        try {
            casPort.addUser(new AddUser("DummyUserName", 
                                        TestConstants.subjectName, 
                                        TestConstants.trustNick, 
                                        TestConstants.defUserGp));
        } catch (CasFault casFault) {
            exception = true;
        }
        assertTrue(exception);

        // attempt adding namespace
        exception = false;
        try {
            casPort.createObjectNamespace(
                    new CreateObjectNamespace(TestConstants.baseName, 
                                              TestConstants.compAlg, 
                                              TestConstants.nsNick,
                                              TestConstants.defUserGp));
        } catch (NoPermissionFault casFault) {
            exception = true;
        }
        assertTrue(exception);
        // Set permission to enroll namespace
        policyData.setObjectSpec(objId);
        policyData.setObjectSpecDesc(CasConstants.OBJECT_SPEC);
        policyData.setActionSpec(enrollNs);
        policyData.setActionSpecDesc(CasConstants.SERVICEACTION_SPEC);
        PolicyDataHandler.storeObject(policyData);

        // attempt adding with bad user group for grant all permissions
        exception = false;
        try {
            casPort.createObjectNamespace(
                   new CreateObjectNamespace(TestConstants.baseName, 
                                             TestConstants.compAlg,
                                             TestConstants.nsNick,
                                             "DummyUserGp"));
        } catch (CasFault casFault) {
            if (casFault.getDescription(0).toString()
                .indexOf("does not exist")!=-1) {
                exception = true;
            }
        }
        assertTrue(exception);

        // Invoke method after permission has been added
        casPort.createObjectNamespace(
                new CreateObjectNamespace(TestConstants.baseName, 
                                          TestConstants.compAlg, 
                                          TestConstants.nsNick, 
                                          TestConstants.defUserGp));
        // Ensure it was added
        NamespaceData nsData = new NamespaceData();
        nsData.setNickname(TestConstants.nsNick);
        nsData.setBasename(TestConstants.baseName);
        nsData.setComparisonAlg(TestConstants.compAlg);
        assertTrue(nsData.equals(
                          (NamespaceData)NamespaceDataHandler.retrieveObject(
                          TestConstants.nsNick)));

        // attempt adding object
        exception = false;
        try {
            casPort.createObject(new CreateObject(TestConstants.objName,
                                                  TestConstants.nsNick,
                                                  TestConstants.defUserGp));
        } catch (NoPermissionFault casFault) {
            exception = true;
        }
        assertTrue(exception);
        // Set permission to add enrollObject
        policyData.setObjectSpec(objId);
        policyData.setObjectSpecDesc(CasConstants.OBJECT_SPEC);
        policyData.setActionSpec(enrollObject);
        policyData.setActionSpecDesc(CasConstants.SERVICEACTION_SPEC);
        PolicyDataHandler.storeObject(policyData);

        // attempt adding with bad user group for grant all permissions
        exception = false;
        try {
            casPort.createObject(new CreateObject(TestConstants.objName, 
                                                  TestConstants.nsNick,
                                                  "DummyUserGroup"));
        }
        catch (CasFault casFault) {
            if (casFault.getDescription(0).toString()
                .indexOf("does not exist")!=-1) {
                exception = true;
            }
        }
        assertTrue(exception);

        // Invoke method after permission has been added
        casPort.createObject(new CreateObject(TestConstants.objName, 
                                              TestConstants.nsNick,
                                              TestConstants.defUserGp));
        // Ensure it was added
        ObjectData objData = new ObjectData();
        objData.setObjectName(TestConstants.objName);
        objData.setObjectNamespace(TestConstants.nsNick);
        String addedObj = 
            Integer.toString(ObjectDataHandler.getObjectId(
                                               TestConstants.objName,
                                               TestConstants.nsNick));
        objData.setObjectId(addedObj);
        assertTrue(objData.equals(
                           (ObjectData)ObjectDataHandler.retrieveObject(
                           TestConstants.objName, 
                           TestConstants.nsNick)));

        // attempt creating userGp
        exception = false;
        try {
            casPort.createGroup(new CreateGroup(TestConstants.userGpName,
                                                "user", 
                                                TestConstants.defUserGp));
        } catch (NoPermissionFault casFault) {
            exception = true;
        }
        assertTrue(exception);
        // Set permission to add create user Gp
        policyData.setObjectSpec(objId);
        policyData.setObjectSpecDesc(CasConstants.OBJECT_SPEC);
        policyData.setActionSpec(createUserGp);
        logger.debug("create user gp " + createUserGp);
        policyData.setActionSpecDesc(CasConstants.SERVICEACTION_SPEC);
        PolicyDataHandler.storeObject(policyData);
        logger.debug("/n stored data " 
                     + CasStringUtils.policyToString(policyData));

        // attempt adding with bad user group for grant all permissions
        exception = false;
        try {
            casPort.createGroup(new CreateGroup("DummyUserGroup", "user", 
                                                TestConstants.userGpName));
        } catch (CasFault casFault) {
            if (casFault.getDescription(0).toString()
                .indexOf("does not exist")!=-1) {
                exception = true;
            }
        }
        assertTrue(exception);

        // Invoke method after permission has been added
        casPort.createGroup(new CreateGroup(TestConstants.userGpName, "user",
                                            TestConstants.defUserGp));
        // Ensure it was added
        UserGroupData userGpData = new UserGroupData();
        userGpData.setGroupName(TestConstants.userGpName);
        assertTrue(userGpData.equals(
                  (UserGroupData)UserGroupDataHandler.retrieveObject(
                  TestConstants.userGpName)));

        // attempt adding to user group, because of grantAll this should 
        // go through
        casPort.manageUserGroups(new ManageUserGroups(TestConstants.userGpName,
                                                      "add",
                                                      TestConstants.userNick));
        // Ensure it was added
        String temp[] = new String[1];
        temp[0] = TestConstants.userNick;
        userGpData.setUserNames(new ArrayOfString(temp));
        assertTrue(userGpData.equals(
                  (UserGroupData)UserGroupDataHandler.retrieveObject(
                  TestConstants.userGpName)));
        
        // attempt creating objectGP
        exception = false;
        try {
            casPort.createGroup(new CreateGroup(TestConstants.objectGpName,
                                                "object", 
                                                TestConstants.defUserGp));
        } catch (NoPermissionFault casFault) {
            exception = true;
        }
        assertTrue(exception);
        // Set permission to add create Obj Gp
        policyData.setObjectSpec(objId);
        policyData.setObjectSpecDesc(CasConstants.OBJECT_SPEC);
        policyData.setActionSpec(createObjGp);
        policyData.setActionSpecDesc(CasConstants.SERVICEACTION_SPEC);
        PolicyDataHandler.storeObject(policyData);

        // attempt adding with bad user group for grant all permissions
        exception = false;
        try {
            casPort.createGroup(new CreateGroup(TestConstants.objectGpName,
                                                "object", "DummyUserGroup"));
        } catch (CasFault casFault) {
            if (casFault.getDescription(0).toString()
                .indexOf("does not exist")!=-1) {
                exception = true;
            }
        }
        assertTrue(exception);

        // Invoke method after permission has been added
        casPort
            .createGroup(new CreateGroup(TestConstants.objectGpName,
                                         "object", TestConstants.defUserGp));
        // Ensure it was added
        ObjectGroupData objectGpData = new ObjectGroupData();
        objectGpData.setGroupName(TestConstants.objectGpName);
        assertTrue(objectGpData.equals(
                  (ObjectGroupData)ObjectGroupDataHandler.retrieveObject(
                  TestConstants.objectGpName)));
        
        // attempt adding to object group, because of grantAll this
        // should go through
        casPort.manageObjectGroups(
                new ManageObjectGroups(TestConstants.objectGpName, 
                                       TestConstants.nsNick + "|" 
                                       + TestConstants.objName, "object", 
                                       "add"));
        // Ensure it was added
        String objTemp[] = new String[1];
        objTemp[0] = addedObj;
        String objTemp1[] = new String[1];
        objTemp1[0] = CasConstants.OBJECT_SPEC;
        objectGpData.setObjectSpecs(new ArrayOfString(objTemp));
        objectGpData.setObjectSpecsDesc(new ArrayOfString(objTemp1));
        assertTrue(objectGpData.equals(
                  (ObjectGroupData)ObjectGroupDataHandler.retrieveObject(
                  TestConstants.objectGpName)));

        // attempting to createServiceType
        exception = false;
        try {
            casPort.createServiceType(
                    new CreateServiceType(TestConstants.serviceName,
                                          TestConstants.defUserGp));
        } catch (NoPermissionFault casFault) {
            exception = true;
        }
        assertTrue(exception);
        // Set permission to create service Type
        policyData.setObjectSpec(objId);
        policyData.setObjectSpecDesc(CasConstants.OBJECT_SPEC);
        policyData.setActionSpec(enrollService);
        policyData.setActionSpecDesc(CasConstants.SERVICEACTION_SPEC);
        PolicyDataHandler.storeObject(policyData);

        // attempt adding with bad user group for grant all permissions
        exception = false;
        try {
            casPort.createServiceType(
                    new CreateServiceType(TestConstants.serviceName, 
                                          "DummyUserGp"));
        } catch (CasFault casFault) {
            if (casFault.getDescription(0).toString()
                .indexOf("does not exist")!=-1) {
                exception = true;
            }
        }
        assertTrue(exception);

        // Invoke method after permission has been added
        casPort.createServiceType(
                new CreateServiceType(TestConstants.serviceName,
                                      TestConstants.defUserGp));
        // Ensure it was added
        ServiceTypeData serviceType = new ServiceTypeData();
        serviceType.setName(TestConstants.serviceName);
        assertTrue(serviceType.equals(
                  (ServiceTypeData)ServiceTypeDataHandler.retrieveObject(
                  TestConstants.serviceName)));

        // attempting to add serviceAction mapping, because of grantAll 
        // this should go through
        casPort.manageServiceAction(
                new ManageServiceAction(TestConstants.actionName, "add", 
                                        TestConstants.serviceName));
        // Ensure it was added
        String actNames[] = new String[1];
        actNames[0] = TestConstants.actionName;
        serviceType.setActionNames(new ArrayOfString(actNames));
        assertTrue(serviceType.equals(
                  (ServiceTypeData)ServiceTypeDataHandler.retrieveObject(
                  TestConstants.serviceName)));

        // attempt creating serviceActionGp
        exception = false;
        try {
            casPort.createGroup(
                    new CreateGroup(TestConstants.serviceActionGpName,
                                    "serviceAction", 
                                    TestConstants.defUserGp));
        } catch (NoPermissionFault  casFault) {
            exception = true;
        }
        assertTrue(exception);
        // Set permission to create serviceAction Gp
        policyData.setObjectSpec(objId);
        policyData.setObjectSpecDesc(CasConstants.OBJECT_SPEC);
        policyData.setActionSpec(createServiceActionGp);
        policyData.setActionSpecDesc(CasConstants.SERVICEACTION_SPEC);
        PolicyDataHandler.storeObject(policyData);
        // Invoke method after permission has been added
        casPort.createGroup(
                new CreateGroup(TestConstants.serviceActionGpName, 
                                "serviceAction", TestConstants.defUserGp));
        // Ensure it was added
        String[] serviceActionGp = 
            ServiceTypeActionHandler.listServiceActionGroups();
        assertTrue(CasStringUtils.stringInArray(serviceActionGp,
                                  TestConstants.serviceActionGpName));
        
        // attempt adding to serviceActionGp, because of grantAll this 
        // should go through
        casPort.manageServiceActionGroups(new ManageServiceActionGroups(
                                          TestConstants.actionName,
                                          TestConstants.serviceActionGpName,
                                          "add",
                                          TestConstants.serviceName));
        // Ensure it was added
        String[] serviceActionGpEntry = 
            ServiceTypeActionHandler.retrieveServiceActionGpEntries(
                                     TestConstants.serviceActionGpName);
        assertTrue(CasStringUtils.stringInArray(serviceActionGpEntry, 
                                                TestConstants.serviceName 
                                                + "/" 
                                                + TestConstants.actionName));
    }   
}
