/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.cas.utils;

import org.globus.cas.CASPortType;
import org.globus.cas.CommunityAuthorizationServiceLocator;

import org.globus.wsrf.impl.security.authentication.Constants;
import org.globus.wsrf.impl.security.authorization.Authorization;
import org.globus.wsrf.impl.security.authorization.NoAuthorization;
import org.globus.wsrf.impl.security.authorization.IdentityAuthorization;

import javax.xml.rpc.Stub;

import org.globus.axis.gsi.GSIConstants;

import java.net.URL;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class TestSetup {

    static Log logger = LogFactory.getLog(TestSetup.class.getName());
    String instanceUrl = null;
    CASPortType casPort = null;
    Object mechanism = Constants.GSI_TRANSPORT;
    String mechanismOption = "-m trans";
    Object protection = Constants.SIGNATURE;
    String protectionOption = "-p sig";
    TestSetup testSetup = null;
    Authorization authz = null;
    String authzOption = null;

    public TestSetup(String host, String port, String securityType, 
                     String protType) {
        this(host, port, securityType, protType, null);
    }

    public TestSetup(String host, String port, String securityType, 
                     String protType, String identity) {
        if (securityType != null) { 
            if (securityType.equalsIgnoreCase("message")) {
                this.mechanism = Constants.GSI_SEC_MSG;
                this.mechanismOption = "-m msg";
            } else if (securityType.equalsIgnoreCase("conversation")) {
                this.mechanism = Constants.GSI_SEC_CONV;
                this.mechanismOption = "-m conv";
            } else if (securityType.equalsIgnoreCase("transport")) {
                this.mechanism = Constants.GSI_TRANSPORT;
            } 
        }
        
        if (protType != null) {
            if  (protType.equalsIgnoreCase("signature")) {
                this.protection = Constants.SIGNATURE;
                
            } if (protType.equalsIgnoreCase("encryption")) {
                this.protection = Constants.ENCRYPTION;
                this.protectionOption = "-p enc";
            }
        } 

        if (host == null) {
            host = "127.0.0.1";
        }

        if (port == null) {
            port = "8080";
        }
        
        if (mechanism.equals(Constants.GSI_TRANSPORT)) {
            this.instanceUrl = "https://" + host + ":" + port 
                + "/wsrf/services/CASService";
        } else {
            this.instanceUrl = "http://" + host + ":" + port 
                + "/wsrf/services/CASService";
        }

        if (identity == null) {
            this.authz = NoAuthorization.getInstance();
        } else {
            this.authz = new IdentityAuthorization(identity);
            this.authzOption = "-s " + identity;
        }
    }

    public synchronized CASPortType getCasPort() throws Exception {

        if (this.casPort != null) {
            return this.casPort;
        }

        CommunityAuthorizationServiceLocator loc = 
            new CommunityAuthorizationServiceLocator();

	logger.debug("Instance url " + instanceUrl);
	this.casPort = 
	    loc.getCASPortTypePort(new URL(instanceUrl));
        ((Stub)casPort)._setProperty((String)mechanism, protection);

        ((Stub)casPort)._setProperty(Constants.AUTHORIZATION, 
                                     this.authz);

	return casPort;
    }

    public String getMechanismOption() {
        return this.mechanismOption;
    }

    public String getProtectionOption() {
        return this.protectionOption;
    }

    public String getAuthzOption() {
        return this.authzOption;
    }

    public String getInstanceUrl() {
        return this.instanceUrl;
    }
}
