/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */

#include "DelegationFactoryService_client.h"
#include "wsse_BinarySecurityTokenType.h"
#include "wsse_BinarySecurityToken.h"
#include "wsa_EndpointReferenceType.h"
#include "wsa_EndpointReference.h"
#include "wsdelegation_certType.h"
#include "globus_delegation_client_util.h"
#include "globus_soap_message.h"
#include "globus_wsrf_core_tools.h"
#include "globus_gsi_proxy.h"
#include "globus_ws_security.h"
#include "version.h"

#define GLOBUS_L_WSSE_BASE64BINARY                                          \
    "http://docs.oasis-open.org/wss/2004/01/"                               \
    "oasis-200401-wss-soap-message-security-1.0#Base64Binary"
#define GLOBUS_L_WSSE_PKIPATH                                               \
    "http://docs.oasis-open.org/wss/2004/01/"                               \
    "oasis-200401-wss-x509-token-profile-1.0#X509PKIPathv1"

#define GlobusLDelegationClientUtilErrorMemory(what__)                      \
    globus_error_put(                                                       \
        globus_error_wrap_errno_error(                                      \
            GLOBUS_DELEGATION_CLIENT_UTIL_MODULE,                           \
            (errno),                                                        \
            GLOBUS_DELEGATION_CLIENT_UTIL_ERROR_MEMORY,                     \
            __FILE__,                                                       \
            _globus_func_name,                                              \
            __LINE__,                                                       \
            "Error allocating %s",                                          \
            (what__)))
            
#define GlobusLDelegationClientUtilErrorInvalid(what__)                     \
    globus_error_put(                                                       \
        globus_error_construct_error(                                       \
            GLOBUS_DELEGATION_CLIENT_UTIL_MODULE,                           \
            NULL,                                                           \
            GLOBUS_DELEGATION_CLIENT_UTIL_ERROR_INVALID,                    \
            __FILE__,                                                       \
            _globus_func_name,                                              \
            __LINE__,                                                       \
            "Invalid parameter %s",                                         \
            (what__)))

#define GlobusLDelegationClientUtilErrorProtocol(what__)                    \
    globus_error_put(                                                       \
        globus_error_construct_error(                                       \
            GLOBUS_DELEGATION_CLIENT_UTIL_MODULE,                           \
            NULL,                                                           \
            GLOBUS_DELEGATION_CLIENT_UTIL_ERROR_PROTOCOL,                   \
            __FILE__,                                                       \
            _globus_func_name,                                              \
            __LINE__,                                                       \
            "Protocol error: %s",                                           \
            (what__)))

#define GlobusLDelegationClientUtilErrorActive()                            \
    globus_error_put(                                                       \
        globus_error_construct_error(                                       \
            GLOBUS_DELEGATION_CLIENT_UTIL_MODULE,                           \
            NULL,                                                           \
            GLOBUS_DELEGATION_CLIENT_UTIL_ERROR_ACTIVE,                     \
            __FILE__,                                                       \
            _globus_func_name,                                              \
            __LINE__,                                                       \
            "Handle already active"))

#define GlobusLDelegationClientUtilErrorCanceled(result__)                  \
    globus_error_put(                                                       \
        globus_error_construct_error(                                       \
            GLOBUS_DELEGATION_CLIENT_UTIL_MODULE,                           \
            (result__) == GLOBUS_SUCCESS ? NULL : globus_error_get(result__),\
            GLOBUS_DELEGATION_CLIENT_UTIL_ERROR_CANCELED,                   \
            __FILE__,                                                       \
            _globus_func_name,                                              \
            __LINE__,                                                       \
            "Operation was canceled."))

#define GlobusLDelegationClientUtilErrorWrappedObj(failed_func, result)     \
    globus_error_construct_error(                                           \
        GLOBUS_DELEGATION_CLIENT_UTIL_MODULE,                               \
        globus_error_get((result)),                                         \
        GLOBUS_DELEGATION_CLIENT_UTIL_ERROR_WRAPPED,                        \
        __FILE__,                                                           \
        _globus_func_name,                                                  \
        __LINE__,                                                           \
        "%s failed.",                                                       \
        (failed_func))

#define GlobusLDelegationClientUtilErrorWrapped(failed_func, result)        \
    globus_error_put(                                                       \
        GlobusLDelegationClientUtilErrorWrappedObj(failed_func, result))

#define GlobusLDelegationClientUtilErrorWsrfFault(failed_func, fault__)     \
    globus_error_put(                                                       \
        globus_error_construct_error(                                       \
            GLOBUS_DELEGATION_CLIENT_UTIL_MODULE,                           \
            globus_error_convert_wsrf_fault((wsbf_BaseFaultType *)(fault__)),\
            GLOBUS_DELEGATION_CLIENT_UTIL_ERROR_WSRF_FAULT,                 \
            __FILE__,                                                       \
            _globus_func_name,                                              \
            __LINE__,                                                       \
            "%s failed.",                                                   \
            (failed_func)))

struct globus_delegation_client_util_handle_s
{
    DelegationFactoryService_client_handle_t client_handle;
    globus_mutex_t                      lock;
    globus_bool_t                       active;
    globus_gsi_cred_handle_t            cred;
    globus_bool_t                       canceled;
    
    globus_gsi_cert_utils_cert_type_t   proxy_type;
    wsa_EndpointReferenceType           factory_epr;
    char *                              factory;
    globus_delegation_client_util_callback_t callback;
    void *                              user_arg;
};

typedef struct
{
    globus_mutex_t                      lock;
    globus_cond_t                       cond;
    globus_bool_t                       done;
    globus_object_t *                   error;
    wsa_EndpointReferenceType **        user_epr;
} globus_l_monitor_t;

globus_result_t
globus_delegation_client_util_handle_init(
    globus_delegation_client_util_handle_t * handle,
    globus_soap_message_attr_t               message_attr,
    globus_handler_chain_t                   handler_chain,
    const globus_gsi_cred_handle_t           user_cred,
    globus_delegation_client_util_attr_t     attr)
{
    globus_result_t                     result;
    globus_delegation_client_util_handle_t ihandle;
    DelegationFactoryService_client_handle_t client_handle;
    globus_xsd_type_registry_t          registry;
    GlobusFuncName(globus_delegation_client_util_handle_init);
    
    if(!message_attr)
    {
        result = GlobusLDelegationClientUtilErrorInvalid("message_attr");
        goto error_param;
    }
    
    result = DelegationFactoryService_client_init(
        &client_handle, message_attr, handler_chain);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusLDelegationClientUtilErrorWrapped(
            "DelegationFactoryService_client_init", result);
        goto error_client;
    }
    
    /* XXX hack, affects global data, no way to specify user specific types */
    DelegationFactoryService_client_get_registry(client_handle, &registry);
    globus_xsd_type_registry_insert(
        registry, &wsse_BinarySecurityTokenType_info, NULL);
    globus_xsd_type_registry_insert(
        registry, &wsa_EndpointReferenceType_info, NULL);
    
    ihandle = (globus_delegation_client_util_handle_t)
        globus_malloc(sizeof(*ihandle));
    if(!ihandle)
    {
        result = GlobusLDelegationClientUtilErrorMemory("ihandle");
        goto error_handle;
    }
    
    if(user_cred)
    {
        result = globus_gsi_cred_handle_copy(
            (globus_gsi_cred_handle_t) user_cred, &ihandle->cred);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusLDelegationClientUtilErrorWrapped(
                "globus_gsi_cred_handle_copy", result);
            goto error_init;
        }
    }
    else
    {
        result = globus_gsi_cred_handle_init(&ihandle->cred, NULL);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusLDelegationClientUtilErrorWrapped(
                "globus_gsi_cred_handle_init", result);
            goto error_init;
        }
        result = globus_gsi_cred_read(ihandle->cred, NULL);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusLDelegationClientUtilErrorWrapped(
                "globus_gsi_cred_read", result);
            goto error_read;
        }
    }
    
    globus_mutex_init(&ihandle->lock, NULL);
    ihandle->client_handle = client_handle;
    ihandle->active = GLOBUS_FALSE;
    
    *handle = ihandle;
    
    return GLOBUS_SUCCESS;

error_read:
    globus_gsi_cred_handle_destroy(ihandle->cred);
error_init:
    globus_free(ihandle);
error_handle:
error_client:
    DelegationFactoryService_client_destroy(client_handle);
error_param:
    *handle = NULL;
    return result;
}

void
globus_delegation_client_util_handle_destroy(
    globus_delegation_client_util_handle_t handle)
{
    GlobusFuncName(globus_delegation_client_util_handle_destroy);
    
    if(handle)
    {
        DelegationFactoryService_client_destroy(handle->client_handle);
        globus_gsi_cred_handle_destroy(handle->cred);
        globus_mutex_destroy(&handle->lock);
        globus_free(handle);
    }
}

globus_result_t
globus_delegation_client_util_cancel(
    globus_delegation_client_util_handle_t handle)
{
    GlobusFuncName(globus_delegation_client_util_cancel);
    
    globus_mutex_lock(&handle->lock);
    {
        if(handle->active)
        {
            handle->canceled = GLOBUS_TRUE;
            DelegationFactoryService_client_operation_cancel(
                handle->client_handle);
        }
    }
    globus_mutex_unlock(&handle->lock);
    
    return GLOBUS_SUCCESS;
}

/* call locked, returns unlocked */
static
void
globus_l_delegation_client_util_finish_op(
    globus_delegation_client_util_handle_t handle,
    globus_result_t                     result,
    const wsa_EndpointReferenceType *   delegated_epr)
{
    if(handle->factory)
    {
        globus_free(handle->factory);
    }
    else
    {
        wsa_EndpointReferenceType_destroy_contents(&handle->factory_epr);
    }
    
    handle->active = GLOBUS_FALSE;
    globus_mutex_unlock(&handle->lock);
    
    handle->callback(handle, result, delegated_epr, handle->user_arg);
}

static
globus_result_t
globus_l_wsse_token_to_cred(
    globus_gsi_cred_handle_t *          cred, 
    const wsse_BinarySecurityTokenType *sec_token)
{
    globus_result_t                     result;
    GlobusFuncName(globus_l_wsse_token_to_cred);

#if 0    
    /* XXX can't assume namespace prefix is wsse */
    if((sec_token->_EncodingType &&
        strcmp(sec_token->_EncodingType, "wsse:Base64Binary") != 0) ||
        (sec_token->_ValueType &&
        strcmp(sec_token->_ValueType, "wsse:PKIPath") != 0))
    {
        result = GlobusLDelegationClientUtilErrorProtocol(
            "Unexpected EncodingType and/or ValueType");
        goto error_protocol;
    }
#endif

    result = globus_ws_security_get_credential(
        (const unsigned char *) sec_token->base_value,
        strlen(sec_token->base_value), cred);
    if(result != GLOBUS_SUCCESS)
    {
        goto error_init;
    }
    
    return GLOBUS_SUCCESS;
    
error_init:
    return result;
}

static
globus_result_t
globus_l_create_delegated_cred(
    globus_gsi_cred_handle_t *          delegated_cred,
    globus_gsi_cred_handle_t            user_cred,
    globus_gsi_cred_handle_t            peer_cred,
    globus_gsi_cert_utils_cert_type_t   proxy_type)
{
    globus_gsi_proxy_handle_t           proxy;
    globus_result_t                     result;
    GlobusFuncName(globus_l_create_delegated_cred);
    
    result = globus_gsi_proxy_handle_init(&proxy, NULL);
    if(result != GLOBUS_SUCCESS)
    {
        goto error_init;
    }
    
    result = globus_gsi_proxy_handle_set_type(proxy, proxy_type);
    if(result != GLOBUS_SUCCESS)
    {
        goto error_setup;
    }
    
    result = globus_gsi_proxy_resign_cert(
        proxy, user_cred, peer_cred, delegated_cred);
    if(result != GLOBUS_SUCCESS)
    {
        goto error_setup;
    }
    
    globus_gsi_proxy_handle_destroy(proxy);
    
    return GLOBUS_SUCCESS;

error_setup:
    globus_gsi_proxy_handle_destroy(proxy);
error_init:
    return result;
}

static
globus_result_t
globus_l_cred_to_wsse_token(
    wsse_BinarySecurityTokenType **     sec_token,
    globus_gsi_cred_handle_t            delegated_cred)
{
    globus_result_t                     result;
    char *                              buffer;
    int                                 len;
    wsse_BinarySecurityTokenType *      token;
    GlobusFuncName(globus_l_cred_to_wsse_token);
    
    result = globus_ws_security_create_pkipath(
        delegated_cred, &buffer, &len);
    if(result != GLOBUS_SUCCESS)
    {
        goto error_create;
    }
    
    result = wsse_BinarySecurityTokenType_init(&token);
    if(result != GLOBUS_SUCCESS)
    {
        goto error_init;
    }
    
    /* XXX can't assume namespace prefix is wsse */
    result = xsd_anyURI_copy_cstr(
        &token->_EncodingType, GLOBUS_L_WSSE_BASE64BINARY);
    if(result != GLOBUS_SUCCESS)
    {
        goto error_setup;
    }
    
    result = xsd_anyURI_copy_cstr(
        &token->_ValueType, GLOBUS_L_WSSE_PKIPATH);
    if(result != GLOBUS_SUCCESS)
    {
        goto error_setup;
    }
    
    token->base_value = buffer;
    *sec_token = token;
    return GLOBUS_SUCCESS;

error_setup:
    wsse_BinarySecurityTokenType_destroy(token);
error_init:
    globus_free(buffer);
error_create:
    return result;
}

static
void
globus_l_delegation_client_util_delegate_cb(
    DelegationFactoryService_client_handle_t client_handle,
    void *                              user_args,
    globus_result_t                     result,
    const wst_RequestSecurityTokenResponseType * response,
    DelegationFactoryPortType_RequestSecurityToken_fault_t fault_type,
    const xsd_any *                     fault)
{
    globus_delegation_client_util_handle_t handle;
    const wsa_EndpointReferenceType *   delegated_epr = NULL;
    GlobusFuncName(globus_l_delegation_client_util_delegate_cb);
    
    handle = (globus_delegation_client_util_handle_t) user_args;
    
    globus_mutex_lock(&handle->lock);
    
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusLDelegationClientUtilErrorWrapped(
            "DelegationFactoryPortType_RequestSecurityToken callback",
            result);
        goto error;
    }
    
    globus_assert(response->any.elements[0].any_info == 
        &wsa_EndpointReferenceType_info ||
        response->any.elements[0].any_info == &wsa_EndpointReference_info);
    
    delegated_epr = (const wsa_EndpointReferenceType *)
        response->any.elements[0].value;
    
error:
    if(result != GLOBUS_SUCCESS && handle->canceled)
    {
        result = GlobusLDelegationClientUtilErrorCanceled(result);
    }
    globus_l_delegation_client_util_finish_op(handle, result, delegated_epr);
}

static
void
globus_l_delegation_client_util_get_cert_cb(
    DelegationFactoryService_client_handle_t client_handle,
    void *                              user_args,
    globus_result_t                     result,
    const wsrp_GetResourcePropertyResponseType * response,
    DelegationFactoryPortType_GetResourceProperty_fault_t fault_type,
    const xsd_any *                     fault)
{
    globus_delegation_client_util_handle_t handle;
    const wsdelegation_certType *       cert_chain;
    const wsse_BinarySecurityTokenType * sec_token;
    globus_gsi_cred_handle_t            peer_cred;
    globus_gsi_cred_handle_t            delegated_cred;
    wst_RequestSecurityTokenType        request;
    xsd_any *                           any;
    GlobusFuncName(globus_l_delegation_client_util_get_cert_cb);
    
    handle = (globus_delegation_client_util_handle_t) user_args;
    
    globus_mutex_lock(&handle->lock);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusLDelegationClientUtilErrorWrapped(
            "DelegationFactoryPortType_GetResourceProperty callback",
            result);
        goto error;
    }
    if(handle->canceled)
    {
        goto error;
    }
    
    globus_assert(response->any.elements[0].any_info == 
        &DelegationFactoryPortType_CertificateChain_rp_info ||
        response->any.elements[0].any_info == 
        &wsdelegation_certType_info);

    cert_chain = (const wsdelegation_certType *)
        response->any.elements[0].value;
    
    globus_assert(
        cert_chain->any.any_info == &wsse_BinarySecurityTokenType_info ||
        cert_chain->any.any_info == &wsse_BinarySecurityToken_info);
        
    sec_token = (const wsse_BinarySecurityTokenType *) cert_chain->any.value;
    
    result = globus_l_wsse_token_to_cred(&peer_cred, sec_token);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusLDelegationClientUtilErrorWrapped(
            "globus_l_wsse_token_to_cred", result);
        goto error;
    }
    
    result = globus_l_create_delegated_cred(
        &delegated_cred, handle->cred, peer_cred, handle->proxy_type);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusLDelegationClientUtilErrorWrapped(
            "globus_l_create_delegated_cred", result);
        goto error_delegate;
    }
    
    wst_RequestSecurityTokenType_init_contents(&request);
    any = xsd_any_array_push(&request.any);
    if(!any)
    {
        result = GlobusLDelegationClientUtilErrorMemory("any element");
        goto error_encode;
    }
    
    result = xsd_QName_copy(
        &any->element, &wsse_BinarySecurityToken_qname);
    if(result != GLOBUS_SUCCESS)
    {
        goto error_encode;
    }
    any->any_info = &wsse_BinarySecurityTokenType_info;
    
    result = globus_l_cred_to_wsse_token(
        (wsse_BinarySecurityTokenType **) &any->value, delegated_cred);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusLDelegationClientUtilErrorWrapped(
            "globus_l_cred_to_wsse_token", result);
        goto error_encode;
    }
    
    if(handle->factory)
    {
        result = DelegationFactoryPortType_RequestSecurityToken_register(
            handle->client_handle,
            handle->factory,
            &request,
            globus_l_delegation_client_util_delegate_cb,
            handle);
    }
    else
    {
        result = DelegationFactoryPortType_RequestSecurityToken_epr_register(
            handle->client_handle,
            &handle->factory_epr,
            &request,
            globus_l_delegation_client_util_delegate_cb,
            handle);
    }
    if(result != GLOBUS_SUCCESS)
    {
        goto error_register;
    }
    
    globus_mutex_unlock(&handle->lock);
    
    wst_RequestSecurityTokenType_destroy_contents(&request);
    globus_gsi_cred_handle_destroy(delegated_cred);
    globus_gsi_cred_handle_destroy(peer_cred);
    return;

error_register:
error_encode:
    wst_RequestSecurityTokenType_destroy_contents(&request);
    globus_gsi_cred_handle_destroy(delegated_cred);
error_delegate:
    globus_gsi_cred_handle_destroy(peer_cred);
error:
    if(handle->canceled)
    {
        result = GlobusLDelegationClientUtilErrorCanceled(result);
    }
    globus_l_delegation_client_util_finish_op(handle, result, NULL);
}

/* called locked */
static
globus_result_t
globus_l_delegation_client_util_delegate_register(
    globus_delegation_client_util_handle_t handle,
    globus_gsi_cert_utils_cert_type_t   proxy_type,
    globus_delegation_client_util_callback_t callback,
    void *                              user_arg)
{
    globus_result_t                     result;
    GlobusFuncName(globus_l_delegation_client_util_delegate_register);
    
    if(!callback)
    {
        result = GlobusLDelegationClientUtilErrorInvalid("callback");
        goto error_param;
    }
    
    if(handle->factory)
    {
        result = DelegationFactoryPortType_GetResourceProperty_register(
            handle->client_handle,
            handle->factory,
            &DelegationFactoryPortType_CertificateChain_rp_qname,
            globus_l_delegation_client_util_get_cert_cb,
            handle);
    }
    else
    {
        result = DelegationFactoryPortType_GetResourceProperty_epr_register(
            handle->client_handle,
            &handle->factory_epr,
            &DelegationFactoryPortType_CertificateChain_rp_qname,
            globus_l_delegation_client_util_get_cert_cb,
            handle);
    }
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusLDelegationClientUtilErrorWrapped(
            "DelegationFactoryPortType_GetResourceProperty_register", result);
        goto error_register;
    }
    
    handle->proxy_type = proxy_type;
    handle->callback = callback;
    handle->user_arg = user_arg;
    handle->canceled = GLOBUS_FALSE;
    handle->active = GLOBUS_TRUE;
    
    return GLOBUS_SUCCESS;

error_register:
error_param:
    return result;
}

globus_result_t
globus_delegation_client_util_delegate_epr_register(
    globus_delegation_client_util_handle_t handle,
    const wsa_EndpointReferenceType *   delegate_factory_epr,
    globus_gsi_cert_utils_cert_type_t   proxy_type,
    globus_delegation_client_util_callback_t callback,
    void *                              user_arg)
{
    globus_result_t                     result;
    GlobusFuncName(globus_delegation_client_util_delegate_epr_register);
    
    if(!delegate_factory_epr)
    {
        result = GlobusLDelegationClientUtilErrorInvalid(
            "delegate_factory_epr");
        goto error_param;
    }
    
    globus_mutex_lock(&handle->lock);
    {
        if(handle->active)
        {
            result = GlobusLDelegationClientUtilErrorActive();
            goto error_active;
        }
        
        handle->factory = NULL;
        result = wsa_EndpointReferenceType_copy_contents(
            &handle->factory_epr, delegate_factory_epr);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusLDelegationClientUtilErrorWrapped(
                "wsa_EndpointReferenceType_copy_contents", result);
            goto error_copy;
        }
        
        result = globus_l_delegation_client_util_delegate_register(
            handle, proxy_type, callback, user_arg);
        if(result != GLOBUS_SUCCESS)
        {
            goto error_register;
        }
    }
    globus_mutex_unlock(&handle->lock);
    
    return GLOBUS_SUCCESS;
    
error_register:
    wsa_EndpointReferenceType_destroy_contents(&handle->factory_epr);
error_copy:
error_active:
    globus_mutex_unlock(&handle->lock);
error_param:
    return result;
}

globus_result_t
globus_delegation_client_util_delegate_register(
    globus_delegation_client_util_handle_t handle,
    const char *                        delegate_factory,
    globus_gsi_cert_utils_cert_type_t   proxy_type,
    globus_delegation_client_util_callback_t callback,
    void *                              user_arg)
{
    globus_result_t                     result;
    GlobusFuncName(globus_delegation_client_util_delegate_register);
    
    if(!delegate_factory)
    {
        result = GlobusLDelegationClientUtilErrorInvalid("delegate_factory");
        goto error_param;
    }
    
    globus_mutex_lock(&handle->lock);
    {
        if(handle->active)
        {
            result = GlobusLDelegationClientUtilErrorActive();
            goto error_active;
        }
        
        handle->factory = globus_libc_strdup(delegate_factory);
        if(!handle->factory)
        {
            result = GlobusLDelegationClientUtilErrorMemory("handle->factory");
            goto error_factory;
        }
        
        result = globus_l_delegation_client_util_delegate_register(
            handle, proxy_type, callback, user_arg);
        if(result != GLOBUS_SUCCESS)
        {
            goto error_register;
        }
    }
    globus_mutex_unlock(&handle->lock);
    
    return GLOBUS_SUCCESS;
    
error_register:
    globus_free(handle->factory);
error_factory:
error_active:
    globus_mutex_unlock(&handle->lock);
error_param:
    return result;
}

static
void
globus_l_delegation_client_util_delegate_blocking_cb(
    globus_delegation_client_util_handle_t handle,
    globus_result_t                     result,
    const wsa_EndpointReferenceType *   delegated_epr,
    void *                              user_arg)
{
    globus_l_monitor_t *                monitor;
    
    monitor = (globus_l_monitor_t *) user_arg;
    
    globus_mutex_lock(&monitor->lock);
    {
        if(result != GLOBUS_SUCCESS)
        {
            monitor->error = globus_error_get(result);
        }
        else
        {
            result = wsa_EndpointReferenceType_copy(
                monitor->user_epr, delegated_epr);
            if(result != GLOBUS_SUCCESS)
            {
                monitor->error = GlobusLDelegationClientUtilErrorWrappedObj(
                    "wsa_EndpointReferenceType_copy", result);
            }
        }
        
        monitor->done = GLOBUS_TRUE;
        globus_cond_signal(&monitor->cond);
    }
    globus_mutex_unlock(&monitor->lock);
}

globus_result_t
globus_delegation_client_util_delegate_epr(
    globus_delegation_client_util_handle_t handle,
    const wsa_EndpointReferenceType *   delegate_factory_epr,
    globus_gsi_cert_utils_cert_type_t   proxy_type,
    wsa_EndpointReferenceType **        delegated_epr)
{
    globus_l_monitor_t                  monitor;
    globus_result_t                     result;
    GlobusFuncName(globus_delegation_client_util_delegate_epr);
    
    if(!delegated_epr)
    {
        result = GlobusLDelegationClientUtilErrorInvalid("delegated_epr");
        goto error_param;
    }
    
    globus_mutex_init(&monitor.lock, NULL);
    globus_cond_init(&monitor.cond, NULL);
    monitor.done = GLOBUS_FALSE;
    monitor.error = NULL;
    monitor.user_epr = delegated_epr;
    *delegated_epr = NULL;
    
    result = globus_delegation_client_util_delegate_epr_register(
        handle,
        delegate_factory_epr,
        proxy_type,
        globus_l_delegation_client_util_delegate_blocking_cb,
        &monitor);
    if(result != GLOBUS_SUCCESS)
    {
        goto error_register;
    }
    
    globus_mutex_lock(&monitor.lock);
    {
        while(!monitor.done)
        {
            globus_cond_wait(&monitor.cond, &monitor.lock);
        }
    }
    globus_mutex_unlock(&monitor.lock);
    
    globus_cond_destroy(&monitor.cond);
    globus_mutex_destroy(&monitor.lock);
    
    return monitor.error ? globus_error_put(monitor.error) : GLOBUS_SUCCESS ;

error_register:
    globus_cond_destroy(&monitor.cond);
    globus_mutex_destroy(&monitor.lock);
error_param:
    return result;
}

globus_result_t
globus_delegation_client_util_delegate(
    globus_delegation_client_util_handle_t handle,
    const char *                        delegate_factory,
    globus_gsi_cert_utils_cert_type_t   proxy_type,
    wsa_EndpointReferenceType **        delegated_epr)
{
    globus_l_monitor_t                  monitor;
    globus_result_t                     result;
    GlobusFuncName(globus_delegation_client_util_delegate);
    
    if(!delegated_epr)
    {
        result = GlobusLDelegationClientUtilErrorInvalid("delegated_epr");
        goto error_param;
    }
    
    globus_mutex_init(&monitor.lock, NULL);
    globus_cond_init(&monitor.cond, NULL);
    monitor.done = GLOBUS_FALSE;
    monitor.error = NULL;
    monitor.user_epr = delegated_epr;
    *delegated_epr = NULL;
    
    result = globus_delegation_client_util_delegate_register(
        handle,
        delegate_factory,
        proxy_type,
        globus_l_delegation_client_util_delegate_blocking_cb,
        &monitor);
    if(result != GLOBUS_SUCCESS)
    {
        goto error_register;
    }
    
    globus_mutex_lock(&monitor.lock);
    {
        while(!monitor.done)
        {
            globus_cond_wait(&monitor.cond, &monitor.lock);
        }
    }
    globus_mutex_unlock(&monitor.lock);
    
    globus_cond_destroy(&monitor.cond);
    globus_mutex_destroy(&monitor.lock);
    
    return monitor.error ? globus_error_put(monitor.error) : GLOBUS_SUCCESS ;

error_register:
    globus_cond_destroy(&monitor.cond);
    globus_mutex_destroy(&monitor.lock);
error_param:
    return result;
}

static
int
globus_l_delegation_client_util_activate(void)
{
    int                                 rc;
    
    rc = globus_module_activate(GLOBUS_COMMON_MODULE);
    if(rc != GLOBUS_SUCCESS)
    {
        goto error_common;
    }
    
    rc = globus_module_activate(GLOBUS_SOAP_MESSAGE_MODULE);
    if(rc != GLOBUS_SUCCESS)
    {
        goto error_soap;
    }
    
    rc = globus_module_activate(GLOBUS_GSI_CREDENTIAL_MODULE);
    if(rc != GLOBUS_SUCCESS)
    {
        goto error_cred;
    }
    
    rc = globus_module_activate(GLOBUS_GSI_PROXY_MODULE);
    if(rc != GLOBUS_SUCCESS)
    {
        goto error_proxy;
    }
    
    rc = globus_module_activate(DELEGATIONFACTORYSERVICE_MODULE);
    if(rc != GLOBUS_SUCCESS)
    {
        goto error_factory;
    }
    
    return GLOBUS_SUCCESS;

error_factory:
    globus_module_deactivate(GLOBUS_GSI_PROXY_MODULE);
error_proxy:
    globus_module_deactivate(GLOBUS_GSI_CREDENTIAL_MODULE);
error_cred:
    globus_module_deactivate(GLOBUS_SOAP_MESSAGE_MODULE);
error_soap:
    globus_module_deactivate(GLOBUS_COMMON_MODULE);
error_common:
    return rc;
}

static
int
globus_l_delegation_client_util_deactivate(void)
{
    globus_module_deactivate(DELEGATIONFACTORYSERVICE_MODULE);
    globus_module_deactivate(GLOBUS_GSI_PROXY_MODULE);
    globus_module_deactivate(GLOBUS_GSI_CREDENTIAL_MODULE);
    globus_module_deactivate(GLOBUS_SOAP_MESSAGE_MODULE);
    globus_module_deactivate(GLOBUS_COMMON_MODULE);

    return GLOBUS_SUCCESS;
}

globus_module_descriptor_t globus_i_delegation_client_util_module =
{
    "globus_delegation_client_util",
    globus_l_delegation_client_util_activate,
    globus_l_delegation_client_util_deactivate,
    NULL,
    NULL,
    &local_version
};
