/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.delegation.service;

import java.rmi.RemoteException;

import org.globus.wsrf.utils.AddressingUtils;

import org.globus.wsrf.container.ServiceHost;

import org.apache.axis.message.addressing.EndpointReferenceType;

import org.globus.wsrf.ResourceKey;
import org.globus.wsrf.ResourceContext;
import org.globus.wsrf.ResourceContextException;

import org.globus.delegationService.test.VoidType;

import org.globus.gsi.GlobusCredential;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class TestService {

    static Log logger = LogFactory.getLog(TestService.class.getName());

    public static final String SERVICE_NAME= "DelegationTestService";

    // Input is EPR to delegation service resource to fetch
    // credentials from.
    public EndpointReferenceType create(EndpointReferenceType epr) 
        throws RemoteException{

        System.out.println("EPR " + epr);
        ResourceContext ctx = null;
        try {
            ctx = ResourceContext.getResourceContext();
        } catch (ResourceContextException exp) {
            throw new RemoteException("Error getting resource context", exp);
        }
        
        TestHome home = (TestHome)ctx.getResourceHome();
        ResourceKey key = null;
        try {
            key = home.create(epr);
        } catch (Exception exp) {
            throw new RemoteException("", exp);
        }

        EndpointReferenceType testEpr = null;
        try {
            String addr = ServiceHost.getBaseURL() + "DelegationTestService";
            testEpr = AddressingUtils.createEndpointReference(addr, key);
        } catch (Exception exp) {
            throw new RemoteException("", exp);
        }
        
        return testEpr;
    }

    public int getCallbackCount(VoidType voidType) throws RemoteException {

        ResourceContext ctx = ResourceContext.getResourceContext();
        TestResource resource = (TestResource)ctx.getResource();

        // Print DN
        GlobusCredential credential = resource.getCredential();
        try {
            credential.verify();
        } catch (Exception exp) {
            throw new RemoteException("", exp);
        }
        System.out.println("verified delegated credential");

        if (logger.isDebugEnabled()) {
            try {
                credential.save(new java.io.FileOutputStream("delegatedCred"));
            } catch (Exception exp) {
                throw new RemoteException("", exp);
            }
                  
            logger.debug("Delegated credential");
            for (int i=0; i< credential.getCertificateChain().length; i++) {
                logger.debug(credential.getCertificateChain()[i]
                             .getSubjectDN());
            }
        }

        return resource.getCallbackCount();
    }
}
