
#ifndef _GLOBUS_I_RENDEZVOUS_H_
#define _GLOBUS_I_RENDEZVOUS_H_

#define SIMPLE_TOPIC_EXPRESSION "http://docs.oasis-open.org/wsn/2004/06/TopicExpression/Simple"

#include "globus_rendezvous.h"
#include "wsa_EndpointReferenceType.h"
#include "wsnt_TopicExpressionType.h"
#include "xsd_anyType.h"
#include "globus_service_engine.h"

typedef struct globus_rz_handle_attr_s
{
    globus_soap_message_attr_t   soap_message_attr;
} globus_i_rz_handle_attr_t;

typedef struct
{
    globus_result_t                     result;
    globus_rz_callback_func_t           callback;
    void *                              arg;
    globus_mutex_t                      mutex;
    globus_cond_t                       cond;
    globus_bool_t                       done;
    globus_bool_t                       unregister_done;
    wsa_EndpointReferenceType *         consumer_reference;
    wsa_EndpointReferenceType *         subscription_reference;
    globus_service_engine_t             service_engine;
    globus_bool_t                       stopped;
    globus_callback_handle_t            poll_handle;
} rz_l_callback_handle_t;

typedef struct globus_rz_handle_s
{
    rz_l_callback_handle_t *            callback_handle;
    globus_i_rz_handle_attr_t *         attr;
    wsa_EndpointReferenceType *         epr;
    int                                 poll_ready;
    globus_mutex_t                      poll_mutex;
} globus_i_rz_handle_t;

void
rz_l_notify_callback(
    void *                              arg,
    wsnt_TopicExpressionType *          topic,
    wsa_EndpointReferenceType *         producer,
    xsd_anyType *                       message);

void
rz_l_engine_stop_callback(
    globus_result_t                     result,
    globus_service_engine_t             engine,
    void *                              args);

globus_result_t
globus_l_rz_handle_attr_copy(
    globus_rz_handle_attr_t *           new_attr,
    globus_rz_handle_attr_t             attr);


/* error stuff */

#define GlobusRZErrorOutOfMemory(size)                          \
    globus_error_put(                                           \
        globus_error_construct_error(                           \
            GLOBUS_RENDEZVOUS_MODULE,                           \
            NULL,                                               \
            GLOBUS_RZ_ERROR_TYPE_OUT_OF_MEMORY,                 \
            __FILE__,                                           \
            NULL,                                               \
            __LINE__,                                           \
            "Memory allocation of size %d bytes failed",        \
            size))

#define GlobusRZErrorFailedAttrInit(result)                     \
    globus_error_put(                                           \
        globus_error_construct_error(                           \
            GLOBUS_RENDEZVOUS_MODULE,                           \
            (result ? globus_error_get(result) : NULL),         \
            GLOBUS_RZ_ERROR_TYPE_FAILED_ATTR_INIT,              \
            __FILE__,                                           \
            NULL,                                               \
            __LINE__,                                           \
            "Failed to initialize rendezvous attributes"))

#define GlobusRZErrorInitFailed(result)                         \
        globus_error_put(                                       \
            globus_error_construct_error(                       \
                GLOBUS_RENDEZVOUS_MODULE,                       \
                (result ? globus_error_get(result) : NULL),     \
                GLOBUS_RZ_ERROR_TYPE_INIT_FAILED,               \
                __FILE__,                                       \
                NULL,                                           \
                __LINE__,                                       \
                "Failed to initialize rendezvous handle"))

#define GlobusRZErrorRegisterFailed(result)                             \
        globus_error_put(                                               \
            globus_error_construct_error(                               \
                GLOBUS_RENDEZVOUS_MODULE,                               \
                (result ? globus_error_get(result) : NULL),             \
                GLOBUS_RZ_ERROR_TYPE_REGISTER_FAILED,                   \
                __FILE__,                                               \
                NULL,                                                   \
                __LINE__,                                               \
                "Failed while trying to perform register operation"))

#define GlobusRZErrorRequestAlreadyBegun()                              \
        globus_error_put(                                               \
            globus_error_construct_error(                               \
                GLOBUS_RENDEZVOUS_MODULE,                               \
                NULL,                                                   \
                GLOBUS_RZ_ERROR_TYPE_REQUEST_ALREADY_BEGUN,             \
                __FILE__,                                               \
                NULL,                                                   \
                __LINE__,                                               \
                "A data request has already begun on this handle"))

#define GlobusRZErrorDataRequestFailed(result)                  \
        globus_error_put(                                       \
            globus_error_construct_error(                       \
                GLOBUS_RENDEZVOUS_MODULE,                       \
                (result ? globus_error_get(result) : NULL),     \
                GLOBUS_RZ_ERROR_TYPE_DATA_REQUEST_FAILED,       \
                __FILE__,                                       \
                NULL,                                           \
                __LINE__,                                       \
                "Data Request Failed"))

#define GlobusRZErrorSubscribeFailed(result)                            \
        globus_error_put(                                               \
            globus_error_construct_error(                               \
                GLOBUS_RENDEZVOUS_MODULE,                               \
                (result ? globus_error_get(result) : NULL),             \
                GLOBUS_RZ_ERROR_TYPE_SUBSCRIBE_FAILED,                  \
                __FILE__,                                               \
                NULL,                                                   \
                __LINE__,                                               \
                "Failed to subscribe to RendezvousCompleted topic"))

#define GlobusRZErrorPollFailed(result)                         \
        globus_error_put(                                       \
            globus_error_construct_error(                       \
                GLOBUS_RENDEZVOUS_MODULE,                       \
                (result ? globus_error_get(result) : NULL),     \
                GLOBUS_RZ_ERROR_TYPE_POLL_FAILED,               \
                __FILE__,                                       \
                NULL,                                           \
                __LINE__,                                       \
                "Failed to poll RendezvousCompleted and "       \
                "RegisterData Resource Properties"))

#define GlobusRZErrorInvalidTopic(topic)                                \
        globus_error_put(                                               \
            globus_error_construct_error(                               \
                GLOBUS_RENDEZVOUS_MODULE,                               \
                NULL,                                                   \
                GLOBUS_RZ_ERROR_TYPE_INVALID_TOPIC,                     \
                __FILE__,                                               \
                NULL,                                                   \
                __LINE__,                                               \
                "Invalid topic dialect,\n\texpected: %s\n\treceived: %s\n", \
                SIMPLE_TOPIC_EXPRESSION,                                \
                topic))

#define GlobusRZErrorInvalidMessageType(message_qname)                  \
        globus_error_put(                                               \
            globus_error_construct_error(                               \
                GLOBUS_RENDEZVOUS_MODULE,                               \
                NULL,                                                   \
                GLOBUS_RZ_ERROR_TYPE_INVALID_MESSAGE_TYPE,              \
                __FILE__,                                               \
                NULL,                                                   \
                __LINE__,                                               \
                "Invalid Notification Message with type: {%s}%s",       \
                (message_qname ? message_qname->Namespace : "null"),    \
                (message_qname ? message_qname->local : "null")))

#define GlobusRZErrorInvalidRPArrayLength(_LENGTH_)                     \
        globus_error_put(                                               \
            globus_error_construct_error(                               \
                GLOBUS_RENDEZVOUS_MODULE,                               \
                NULL,                                                   \
                GLOBUS_RZ_ERROR_TYPE_INVALID_POLL_RESPONSE,             \
                __FILE__,                                               \
                NULL,                                                   \
                __LINE__,                                               \
                "Expected two RP values, only found %d", _LENGTH_))
        
#define GlobusRZErrorInvalidRPValue(_MESSAGE_, _QNAME_)                 \
        globus_error_put(                                               \
            globus_error_construct_error(                               \
                GLOBUS_RENDEZVOUS_MODULE,                               \
                NULL,                                                   \
                GLOBUS_RZ_ERROR_TYPE_INVALID_POLL_RESPONSE,             \
                __FILE__,                                               \
                NULL,                                                   \
                __LINE__,                                               \
                _MESSAGE_, (_QNAME_)->Namespace, (_QNAME_)->local))

#define GlobusRZErrorShutdownFailed(result)                             \
        globus_error_put(                                               \
            globus_error_construct_error(                               \
                GLOBUS_RENDEZVOUS_MODULE,                               \
                (result ? globus_error_get(result) : NULL),             \
                GLOBUS_RZ_ERROR_TYPE_SHUTDOWN_FAILED,                   \
                __FILE__,                                               \
                NULL,                                                   \
                __LINE__,                                               \
                "Failed to shutdown polling/notification setup"))

                
#endif
