/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */


#include "globus_i_rendezvous.h"
#include "globus_wsrf_core_tools.h"
#include "version.h"
#include "wsnt_TopicExpressionType.h"
#include "globus_notification_consumer.h"
#include "wsrendezvous_RendezvousCompleted.h"
#include "wsrendezvous_RegistrantData.h"
#include "wsrendezvous_StateChangeNotificationMessageWrapperType.h"
#include "globus_service_engine.h"
#include "SubscriptionManagerService_client.h"
#include "ManagedJobService_client.h"
#include "ManagedExecutableJobService_client.h"
#include "ManagedMultiJobService_client.h"

xsd_QName                               wsgram_ResourceID_qname =
{
    "http://www.globus.org/namespaces/2004/10/gram/job",
    "ResourceID"
};

static
int
rz_l_activate();

static
int
rz_l_deactivate();

globus_module_descriptor_t              globus_i_rz_module =
{
    "globus_rendezvous_module",
    rz_l_activate,
    rz_l_deactivate,
    NULL,
    NULL,
    &local_version
};

globus_result_t
globus_l_rz_handle_attr_copy(
    globus_rz_handle_attr_t *           new_attr,
    globus_rz_handle_attr_t             attr)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    globus_i_rz_handle_attr_t *         iattr;

    iattr = globus_malloc(sizeof(globus_i_rz_handle_attr_t));
    if(!iattr)
    {
        result = GlobusRZErrorOutOfMemory(
            sizeof(globus_i_rz_handle_attr_t));
        goto exit;
    }
    memset(iattr, 0, sizeof(globus_i_rz_handle_attr_t));

    result = globus_soap_message_attr_copy(
        &iattr->soap_message_attr,
        attr->soap_message_attr);
    if(result != GLOBUS_SUCCESS)
    {
        goto exit;
    }

    *new_attr = iattr;

exit:

    return result;
}

globus_result_t
globus_rz_handle_attr_init(
    globus_rz_handle_attr_t *           attr,
    int                                 count,
    ...)
{
    int                                 i = 0;
    globus_i_rz_handle_attr_t *         new_attr;
    va_list                             ap;
    globus_result_t                     result = GLOBUS_SUCCESS;

    new_attr = globus_malloc(sizeof(globus_i_rz_handle_attr_t));
    if(!new_attr)
    {
        result = GlobusRZErrorOutOfMemory(
            sizeof(globus_i_rz_handle_attr_t));
        goto exit;
    }
    memset(new_attr, 0, sizeof(globus_i_rz_handle_attr_t));

    result = globus_soap_message_attr_init(
        &new_attr->soap_message_attr);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusRZErrorFailedAttrInit(result);
        goto free_attr;
    }
    
    va_start(ap, count);
    for(i = 0; i < count; ++i)
    {
        const char * key = va_arg(ap, const char *);
        globus_rz_handle_attr_copy_func_t copyf = va_arg(ap, 
            globus_rz_handle_attr_copy_func_t);
        globus_rz_handle_attr_destroy_func_t destroyf = va_arg(ap,
            globus_rz_handle_attr_destroy_func_t);
        void * value = va_arg(ap, void *);
        result = globus_soap_message_attr_set(
            new_attr->soap_message_attr,
            key, copyf, destroyf, value); 
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusRZErrorFailedAttrInit(result);
            goto free_soap_attr;
        }
    }
    va_end(ap);

    *attr = new_attr;
    goto exit;

free_soap_attr:
    va_end(ap);
    globus_soap_message_attr_destroy(
        new_attr->soap_message_attr);

free_attr:
    globus_free(new_attr);

exit:
    return result;
}

void
globus_rz_handle_attr_destroy(
    globus_rz_handle_attr_t             attr)
{
    if(attr)
    {
        if(attr->soap_message_attr)
        {
            globus_soap_message_attr_destroy(attr->soap_message_attr);
        }
        globus_free(attr);
    }
}
 
globus_result_t
globus_rz_handle_init(
    globus_rz_handle_t *                handle,
    globus_rz_handle_attr_t             attr,
    const char *                        endpoint)
{
    globus_i_rz_handle_t *              new_handle;
    globus_result_t                     result = GLOBUS_SUCCESS;
    
    new_handle = globus_malloc(sizeof(globus_i_rz_handle_t));
    if(!new_handle)
    {
        result = GlobusRZErrorOutOfMemory(
            sizeof(globus_i_rz_handle_t));
        goto exit;
    }
    memset(new_handle, 0, sizeof(globus_i_rz_handle_t));
    
    globus_mutex_init(&new_handle->poll_mutex, NULL);

    if(attr)
    {
        result = globus_l_rz_handle_attr_copy(
            &new_handle->attr,
            attr);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusRZErrorInitFailed(result);
            goto destroy_handle;
        }
    }

    result = wsa_EndpointReferenceType_init(&new_handle->epr);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusRZErrorInitFailed(result);
        goto destroy_handle;
    }

    result = globus_wsrf_core_unwrap_endpoint_reference(
        endpoint, &wsgram_ResourceID_qname, new_handle->epr);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusRZErrorInitFailed(result);
        goto destroy_handle;
    }

    *handle = new_handle;
    goto exit;

destroy_handle:
    globus_rz_handle_destroy(new_handle);

exit:
    return result;
}

void
globus_rz_handle_destroy(
    globus_rz_handle_t                  handle)
{
    if(handle)
    {
        globus_mutex_destroy(&handle->poll_mutex);

        if(handle->attr)
        {
            globus_rz_handle_attr_destroy(handle->attr);
        }

        if(handle->epr)
        {
            wsa_EndpointReferenceType_destroy(handle->epr);
        }

        globus_free(handle);
    }
}

void
rz_l_notify_callback(
    void *                              arg,
    wsnt_TopicExpressionType *          topic,
    wsa_EndpointReferenceType *         producer,
    xsd_anyType *                       message)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    rz_l_callback_handle_t *            callback_handle =
    (rz_l_callback_handle_t *) arg;

    /* verify topic */
    if(!topic || !topic->_Dialect || 
       !strcmp(*topic->_Dialect, SIMPLE_TOPIC_EXPRESSION))
    {
        callback_handle->result = 
        GlobusRZErrorInvalidTopic(
            (*topic->_Dialect));
        goto exit;
    }

    if(message && 
       message->value &&
       (message->any_info ==
        &wsrendezvous_StateChangeNotificationMessageWrapperType_info))
    {
        wsrendezvous_StateChangeNotificationMessageWrapperType * sc = 
        (wsrendezvous_StateChangeNotificationMessageWrapperType *)
        message->value;

        globus_mutex_lock(&callback_handle->mutex);

        if(!callback_handle->done &&
           sc->stateChangeNotificationMessage.RendezvousCompleted)
        {
            callback_handle->callback(
                result,
                sc->stateChangeNotificationMessage.RegistrantData.value,
                sc->stateChangeNotificationMessage.RegistrantData.length,
                callback_handle->arg);
        }

        globus_mutex_unlock(&callback_handle->mutex);
    }
    else
    {
        xsd_QName *                     message_type =
        (message ? message->any_info->type : NULL);
        result = GlobusRZErrorInvalidMessageType(message_type);
        globus_mutex_lock(&callback_handle->mutex);
        callback_handle->callback(
                result,
                NULL,
                0,
                callback_handle->arg);
        globus_mutex_unlock(&callback_handle->mutex);
        goto exit;
    }

exit:

    callback_handle->result = result;
}

static
void
rz_l_unregister_poll_callback(
    void *                              arg)
{
    rz_l_callback_handle_t *            callback_handle = 
    (rz_l_callback_handle_t *) arg;
    
    globus_mutex_lock(&callback_handle->mutex);
    globus_cond_signal(&callback_handle->cond);
    callback_handle->unregister_done = 1;
    globus_mutex_unlock(&callback_handle->mutex);
}

void
rz_l_engine_stop_callback(
    globus_result_t                     result,
    globus_service_engine_t             engine,
    void *                              args)
{
    globus_rz_handle_t                  handle = (globus_rz_handle_t) args;

    globus_mutex_lock(&handle->callback_handle->mutex);
    handle->callback_handle->stopped = 1;
    globus_cond_signal(&handle->callback_handle->cond);
    globus_mutex_unlock(&handle->callback_handle->mutex);
}

globus_result_t
globus_rz_data_request_finished(
    globus_rz_handle_t                  handle)
{
    wsrl_DestroyType                    destroy;
    wsrl_DestroyResponseType *          destroyResponse;
    int                                 fault_type;
    xsd_any *                           fault;
    globus_result_t                     result = GLOBUS_SUCCESS;
    SubscriptionManagerService_client_handle_t subscription_client_handle 
                                        = NULL;
    
    if(handle->callback_handle)
    {
        globus_bool_t                   active;
        result = globus_callback_unregister(
            handle->callback_handle->poll_handle,
            rz_l_unregister_poll_callback,
            handle->callback_handle,
            &active);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusRZErrorShutdownFailed(result);
            goto exit;
        }
        
        globus_mutex_lock(&handle->callback_handle->mutex);
        while(!handle->callback_handle->unregister_done)
        {
            globus_cond_wait(
                &handle->callback_handle->cond,
                &handle->callback_handle->mutex);
        }
        globus_mutex_unlock(&handle->callback_handle->mutex);
        
        result = SubscriptionManagerService_client_init(
            &subscription_client_handle,
            (handle->attr ? handle->attr->soap_message_attr : NULL),
            NULL);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusRZErrorShutdownFailed(result);
            goto exit;
        }

        /* destroy Subscription */
        result = SubscriptionManager_Destroy_epr(
            subscription_client_handle,
            handle->callback_handle->subscription_reference,
            &destroy,
            &destroyResponse,
            (SubscriptionManager_Destroy_fault_t *) &fault_type,
            &fault);
        if(result != GLOBUS_SUCCESS)
        {
            if(fault)
            {
                xsd_any_destroy(fault);
            }
            result = GlobusRZErrorShutdownFailed(result);
            goto exit;
        }
        else
        {
            wsrl_DestroyResponseType_destroy(destroyResponse);
        }

        globus_notification_destroy_consumer(
            handle->callback_handle->consumer_reference);
        wsa_EndpointReferenceType_destroy(
            handle->callback_handle->consumer_reference);
        
        result = globus_service_engine_register_stop(
            handle->callback_handle->service_engine);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusRZErrorShutdownFailed(result);
            goto exit;
        }

        globus_mutex_lock(&handle->callback_handle->mutex);
        while(!handle->callback_handle->stopped)
        {
            globus_cond_wait(
                &handle->callback_handle->cond,
                &handle->callback_handle->mutex);
        }
        globus_mutex_unlock(&handle->callback_handle->mutex);

        globus_service_engine_destroy(
            handle->callback_handle->service_engine);
        globus_mutex_destroy(&handle->callback_handle->mutex);
        globus_cond_destroy(&handle->callback_handle->cond);
        globus_free(handle->callback_handle);
        handle->callback_handle = NULL;
    }

exit:

    if(subscription_client_handle)
    {
        SubscriptionManagerService_client_destroy(subscription_client_handle);
    }

    return result;
}

static
int
rz_l_activate()
{
    int                                 rc = GLOBUS_SUCCESS;

    rc = globus_module_activate(GLOBUS_NOTIFICATION_CONSUMER_MODULE);
    if(rc != GLOBUS_SUCCESS)
    {
        return rc;
    }

    rc = globus_module_activate(MANAGEDEXECUTABLEJOBSERVICE_MODULE);
    if(rc != GLOBUS_SUCCESS)
    {
        return rc;
    }

    rc = globus_module_activate(MANAGEDMULTIJOBSERVICE_MODULE);
    if(rc != GLOBUS_SUCCESS)
    {
        return rc;
    }
 
    rc = globus_module_activate(MANAGEDJOBSERVICE_MODULE);
    if(rc != GLOBUS_SUCCESS)
    {
        return rc;
    }

    rc = globus_module_activate(SUBSCRIPTIONMANAGERSERVICE_MODULE);
    if(rc != GLOBUS_SUCCESS)
    {
        return rc;
    }

    return rc;
}

static
int
rz_l_deactivate()
{
    int                                 rc = GLOBUS_SUCCESS;

    rc = globus_module_deactivate(SUBSCRIPTIONMANAGERSERVICE_MODULE);
    if(rc != GLOBUS_SUCCESS)
    {
        return rc;
    }

    rc = globus_module_deactivate(MANAGEDJOBSERVICE_MODULE);
    if(rc != GLOBUS_SUCCESS)
    {
        return rc;
    }

    rc = globus_module_deactivate(MANAGEDEXECUTABLEJOBSERVICE_MODULE);
    if(rc != GLOBUS_SUCCESS)
    {
        return rc;
    }

    rc = globus_module_deactivate(MANAGEDMULTIJOBSERVICE_MODULE);
    if(rc != GLOBUS_SUCCESS)
    {
        return rc;
    }

    rc = globus_module_deactivate(GLOBUS_NOTIFICATION_CONSUMER_MODULE);
    if(rc != GLOBUS_SUCCESS)
    {
        return rc;
    }
 
    return rc;
}
