/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */

#include "globus_i_globusrun_ws.h"

static
void
globus_l_cancel_cb(
    void *                              arg)
{
    ManagedJobService_client_handle_t   job_handle;
    
    job_handle = (ManagedJobService_client_handle_t) arg;
    ManagedJobService_client_operation_cancel(job_handle);
}

globus_result_t
globus_i_kill(
    globus_i_info_t *                   info)
{
    wsrl_DestroyType                    destroy;
    wsrl_DestroyResponseType *          response;
    ManagedJobPortType_Destroy_fault_t  fault_type;
    xsd_any *                           fault;
    globus_result_t                     result = GLOBUS_SUCCESS;
    ManagedJobService_client_handle_t   job_handle;
    globus_bool_t                       tls = GLOBUS_FALSE;

    if (strncmp(info->job_epr->Address.base_value, "https:", 6) == 0)
    {
        tls = GLOBUS_TRUE;
    }
    
    wsrl_DestroyType_init_contents(&destroy);
    
    result = ManagedJobService_client_init(
        &job_handle,
        info->message_attr,
        tls 
            ? info->tls_handler_chain
            : info->secmess_handler_chain);

    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusLErrorWrapResult("initializing MJS client", result);
        goto error;
    }
    
    if(globus_i_set_cancel_notify(globus_l_cancel_cb, job_handle))
    {
        result = GlobusLErrorCanceled();
        goto error_canceled;
    }
    
    result = ManagedJobPortType_Destroy_epr(
        job_handle,
        info->job_epr,
        &destroy,
        &response,
        &fault_type,
        &fault);
    if(result != GLOBUS_SUCCESS)
    {
        if(fault_type != GLOBUS_SUCCESS)
        {
            xsd_any_destroy(fault);
        }
        
        result = GlobusLErrorWrapResult("destroying job", result);
        goto error_op;
    }
    
    wsrl_DestroyResponseType_destroy(response);
    
error_op:
    globus_i_set_cancel_notify(NULL, NULL);
error_canceled:
    ManagedJobService_client_destroy(job_handle);
error:
    wsrl_DestroyType_destroy_contents(&destroy);
    return result;
}
