/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */

#include "globus_i_globusrun_ws.h"
#include "wsgram_ServiceLevelAgreementType.h"
#include "globusrunwsAnnotationType.h"
#include "ManagedExecutableJobService_client.h"
#include "wsgram_state.h"
#include "wsgram_holding.h"
#include "wsgram_fault.h"
#include "wsgram_exitCode.h"
#include "wsgram_serviceLevelAgreement.h"
#include "wsbf_DescriptionType.h"

static
void
globus_l_cancel_cb(
    void *                              arg)
{
    ManagedJobService_client_handle_t   job_handle;
    
    job_handle = (ManagedJobService_client_handle_t) arg;
    ManagedJobService_client_operation_cancel(job_handle);
}

/* XXX need to get things like holding, fault, and exit code
 * so client can return them
 */
globus_result_t
globus_i_status(
    globus_i_info_t *                   info)
{
    wsrp_GetMultipleResourcePropertiesResponseType * response;
    ManagedJobPortType_GetMultipleResourceProperties_fault_t fault_type;
    xsd_any *                           fault;
    globus_result_t                     result;
    ManagedJobService_client_handle_t   job_handle;
    xsd_QName *                         qname;
    wsrp_GetMultipleResourcePropertiesType gmrp;
    wsgram_StateEnumeration *           state = NULL;
    xsd_boolean *                       holding = NULL;
    xsd_int *                           exit_code = NULL;
    wsgram_FaultResourcePropertyType *  gram_fault = NULL;
    int                                 i;
    globus_bool_t                       tls = GLOBUS_FALSE;
    
    if (strncmp(info->job_epr->Address.base_value, "https:", 6) == 0)
    {
        tls = GLOBUS_TRUE;
    }
    wsrp_GetMultipleResourcePropertiesType_init_contents(&gmrp);
    qname = xsd_QName_array_push(&gmrp.ResourceProperty);
    if(!qname)
    {
        result = GlobusLErrorWrapResult(
            "querying job state", GlobusLErrorMemory("qname entry"));
        goto error_init;
    }
    
    result = xsd_QName_copy_contents(
        qname, &ManagedJobPortType_state_rp_qname);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusLErrorWrapResult("querying job state", result);
        goto error_init;
    }
    
    qname = xsd_QName_array_push(&gmrp.ResourceProperty);
    if(!qname)
    {
        result = GlobusLErrorWrapResult(
            "querying job state", GlobusLErrorMemory("qname entry"));
        goto error_init;
    }
    
    result = xsd_QName_copy_contents(
        qname, &ManagedJobPortType_holding_rp_qname);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusLErrorWrapResult("querying job state", result);
        goto error_init;
    }
    
    qname = xsd_QName_array_push(&gmrp.ResourceProperty);
    if(!qname)
    {
        result = GlobusLErrorWrapResult(
            "querying job state", GlobusLErrorMemory("qname entry"));
        goto error_init;
    }
    
    result = xsd_QName_copy_contents(
        qname, &ManagedJobPortType_fault_rp_qname);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusLErrorWrapResult("querying job state", result);
        goto error_init;
    }
    
    if(strstr(
        info->job_epr->Address.base_value,
        MANAGEDEXECUTABLEJOBSERVICE_BASE_PATH))
    {
        /* this probably isnt allowed within the MJS context, but it works */
        qname = xsd_QName_array_push(&gmrp.ResourceProperty);
        if(!qname)
        {
            result = GlobusLErrorWrapResult(
                "querying job state", GlobusLErrorMemory("qname entry"));
            goto error_init;
        }
        
        result = xsd_QName_copy_contents(
            qname, &ManagedExecutableJobPortType_exitCode_rp_qname);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusLErrorWrapResult("querying job state", result);
            goto error_init;
        }
    }

    result = ManagedJobService_client_init(
        &job_handle, info->message_attr,
        tls
            ? info->tls_handler_chain
            : info->secmess_handler_chain);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusLErrorWrapResult("initializing MJS client", result);
        goto error_client;
    }
    
    if(globus_i_set_cancel_notify(globus_l_cancel_cb, job_handle))
    {
        result = GlobusLErrorCanceled();
        goto error_canceled;
    }
    
    result = ManagedJobPortType_GetMultipleResourceProperties_epr(
        job_handle,
        info->job_epr,
        &gmrp,
        &response,
        &fault_type,
        &fault);
    if(result != GLOBUS_SUCCESS)
    {
        if(fault_type != GLOBUS_SUCCESS)
        {
            xsd_any_destroy(fault);
        }
        result = GlobusLErrorWrapResult("querying job state", result);
        goto error;
    }

    for(i = 0; i < response->any.length; i++)
    {
        if(response->any.elements[i].any_info == 
            &ManagedJobPortType_state_rp_info ||
	    response->any.elements[i].any_info ==
	    &wsgram_state_info)
        {
            globus_assert(!state && "Unexpected duplicate entry");
            
            state = (wsgram_StateEnumeration *) 
                response->any.elements[i].value;
        }
        else if(response->any.elements[i].any_info == 
            &ManagedJobPortType_holding_rp_info ||
	    response->any.elements[i].any_info ==
	    &wsgram_holding_info)
        {
            globus_assert(!holding && "Unexpected duplicate entry");
            
            holding = (xsd_boolean *) response->any.elements[i].value;
        }
        else if(response->any.elements[i].any_info == 
            &ManagedJobPortType_fault_rp_info ||
	    response->any.elements[i].any_info ==
	    &wsgram_fault_info)
        {
            globus_assert(!gram_fault && "Unexpected duplicate entry");
            
            gram_fault = (wsgram_FaultResourcePropertyType *)
                response->any.elements[i].value;
        }
        else if(response->any.elements[i].any_info == 
            &ManagedExecutableJobPortType_exitCode_rp_info ||
	    response->any.elements[i].any_info ==
	    &wsgram_exitCode_info)
        {
            globus_assert(!exit_code && "Unexpected duplicate entry");
            
            exit_code = (xsd_int *) response->any.elements[i].value;
        }
        else
        {
            result = GlobusLErrorInvalid("response",
                "expected only state, holding, fault and exitcode");
            goto error_response;
        }
    }
    
    if(!state || !holding)
    {
        result = GlobusLErrorInvalid("response", "state and holding");
        goto error_response;
    }
    
    printf("Current job state: %s%s\n", *state, *holding ? "-Hold" : "");

    if(strcmp(*state, wsgram_StateEnumeration_Done) == 0 ||
        strcmp(*state, wsgram_StateEnumeration_Failed) == 0)
    {
        if(exit_code)
        {
            info->job_exit_code = *exit_code;
        }
        
        if(gram_fault)
        {
            info->job_fault = globus_i_gram_fault_to_error(gram_fault);
        }
    }
    
error_response:
    wsrp_GetMultipleResourcePropertiesResponseType_destroy(response);
error:
    globus_i_set_cancel_notify(NULL, NULL);
error_canceled:
    ManagedJobService_client_destroy(job_handle);
error_client:
error_init:
    wsrp_GetMultipleResourcePropertiesType_destroy_contents(&gmrp);
    return result;
}

static
globus_object_t *
globus_l_error_convert_wsrf_fault(
    const wsbf_BaseFaultType *          fault);

static
globus_object_t *
globus_l_error_fault_multiple(
    const wsbf_BaseFaultType_array *    fault_array)
{
    globus_object_t *                   error;
    int                                 i;
    globus_object_t *                   f;
    
    error = globus_error_construct_multiple(NULL, 0, "");
    
    for(i = fault_array->length-1; i >= 0; i--)
    {
        f = globus_l_error_convert_wsrf_fault(&fault_array->elements[i]);

        if (f != GLOBUS_ERROR_NO_INFO)
        {
            globus_error_mutliple_add_chain( error, f, "");
        }
    }
    
    return error;
}

static
globus_object_t *
globus_l_error_convert_wsrf_fault(
    const wsbf_BaseFaultType *          fault)
{
    globus_object_t *                   cause;
    char *                              description = NULL;
    char **                             array;
    
    if(fault->FaultCause.length == 0)
    {
        cause = NULL;
    }
    else if(fault->FaultCause.length == 1)
    {
        cause = globus_l_error_convert_wsrf_fault(&fault->FaultCause.elements[0]);
    }
    else
    {
        cause = globus_l_error_fault_multiple(&fault->FaultCause);
    }
    if (cause == GLOBUS_ERROR_NO_INFO)
    {
        cause = NULL;
    }
    
    array = (char **) globus_malloc(
        sizeof(char *) * fault->Description.length * 2);
    if(array)
    {
        int                             i;
        int                             j = 0;
        
        for(i = fault->Description.length - 1; i >= 0; i--)
        {
            if ((strncmp(fault->Description.elements[i].base_value,
                        "org.", 4) != 0) &&
                (strncmp(fault->Description.elements[i].base_value,
                        "java.", 5) != 0))
            {
                if(j > 0)
                {
                    array[j++] = "\n";
                }
                array[j++] = fault->Description.elements[i].base_value;
            }
        }

        if (j > 0)
        {
            description = globus_libc_join((const char **)array, j);
        }
        globus_free(array);
        if (j == 0)
        {
            return GLOBUS_ERROR_NO_INFO;
        }
    }
    else
    {
        return GLOBUS_ERROR_NO_INFO;
    }
    
    return globus_error_construct_string(NULL, cause, "%s", description);
}

/* this function is a huge hack.  since all of the fault types have the same
 * base fault (wsbf_BaseFaultType), they'll all start with the same members.
 * hopefully the compiler always aligns the first members the same.
 */
globus_object_t *
globus_i_gram_fault_to_error(
    wsgram_FaultResourcePropertyType *  gram_fault)
{
    globus_object_t *                   error = NULL;
    wsbf_BaseFaultType *                fault;
    
    if(gram_fault->choice_value.type !=
        wsgram_FaultResourcePropertyType_undefined)
    {
        fault = (wsbf_BaseFaultType *) &gram_fault->choice_value.value.fault;
        error = globus_l_error_convert_wsrf_fault(fault);
    }
    
    return error;
}

globus_result_t
globus_i_query_job_description(
    globus_i_info_t *                   info,
    wsgram_CreateManagedJobInputType *  input)
{
    wsgram_ServiceLevelAgreementType *  sla;
    wsrp_GetResourcePropertyResponseType * response;
    ManagedJobPortType_GetResourceProperty_fault_t fault_type;
    xsd_any *                           fault;
    globus_result_t                     result;
    ManagedJobService_client_handle_t   job_handle;
    globus_xsd_type_registry_t          registry;
    globus_bool_t                       tls = GLOBUS_FALSE;
    
    if (strncmp(info->job_epr->Address.base_value, "https:", 6) == 0)
    {
        tls = GLOBUS_TRUE;
    }
    result = ManagedJobService_client_init(
        &job_handle,
        info->message_attr,
        tls 
            ? info->tls_handler_chain
            : info->secmess_handler_chain);

    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusLErrorWrapResult("initializing MJS client", result);
        goto error_client;
    }
    
    if(globus_i_set_cancel_notify(globus_l_cancel_cb, job_handle))
    {
        result = GlobusLErrorCanceled();
        goto error_canceled;
    }
    /* XXX hack, unable to set user registry yet.
     * this affects all users of client api
     */
    ManagedJobService_client_get_registry(job_handle, &registry);
    globus_xsd_type_registry_insert(
        registry, &globusrunwsAnnotationType_info, NULL);

    result = ManagedJobPortType_GetResourceProperty_epr(
        job_handle,
        info->job_epr,
        &ManagedJobPortType_serviceLevelAgreement_rp_qname,
        &response,
        &fault_type,
        &fault);
    if(result != GLOBUS_SUCCESS)
    {
        if(fault_type != GLOBUS_SUCCESS)
        {
            xsd_any_destroy(fault);
        }
        result = GlobusLErrorWrapResult("querying job submission", result);
        goto error;
    }
    
    globus_assert(response->any.elements[0].any_info == 
        &ManagedJobPortType_serviceLevelAgreement_rp_info ||
	response->any.elements[0].any_info ==
	&wsgram_serviceLevelAgreement_info);
    
    sla = (wsgram_ServiceLevelAgreementType *) response->any.elements[0].value;
    
    if(sla->choice_value.type == wsgram_ServiceLevelAgreementType_job)
    {
        input->choice_value.type = wsgram_CreateManagedJobInputType_job;
        result = wsgram_JobDescriptionType_copy_contents(
            &input->choice_value.value.job,
            &sla->choice_value.value.job);
    }
    else
    {
        globus_assert(sla->choice_value.type ==
            wsgram_ServiceLevelAgreementType_multiJob);
        
        input->choice_value.type = wsgram_CreateManagedJobInputType_multiJob;
        result = wsgram_MultiJobDescriptionType_copy_contents(
            &input->choice_value.value.multiJob,
            &sla->choice_value.value.multiJob);
    }
    
    wsrp_GetResourcePropertyResponseType_destroy(response);
    
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusLErrorWrapResult("copying job description", result);
        goto error;
    }
    
error:
    globus_i_set_cancel_notify(NULL, NULL);
error_canceled:
    ManagedJobService_client_destroy(job_handle);
error_client:
    return result;
}
