#ifndef GLOBUS_I_MJG_H_
#define GLOBUS_I_MJG_H_

#include "globus_ftp_client.h"
#include "globus_common.h"

#define GLOBUS_I_MJG_STREAMING_FINISH 1
#define GLOBUS_I_MJG_STREAMING_ABORT  2

#define GLOBUS_I_MJG_STREAMING_BUFFER_SIZE 65536
#define GLOBUS_I_MJG_STREAMING_IDLE_DELAY 10 /* seconds */

/* called for each url successfully added.  either on xfer error or stop
 * success
 */

typedef struct globus_i_streaming_s globus_i_streaming_t;

typedef void (*globus_i_streaming_cb_t)(
    globus_i_streaming_t *              handle,
    const char *                        url,
    globus_result_t                     result,
    void *                              user_arg);

struct globus_i_streaming_s
{
    globus_mutex_t                      lock;
    int                                 state;
    globus_ftp_client_handle_t          ftp_handle;
    globus_fifo_t                       ready_q;
    struct globus_l_streaming_entry_s * active;
    globus_list_t *                     oneshots;
    globus_reltime_t                    idle_delay;
    globus_callback_handle_t            finish_delay_handle;
    globus_i_streaming_cb_t             callback;
    void *                              user_arg;
    int                                 ref;
};


/* XXX needs GLOBUS_FTP_CLIENT_MODULE activated */
globus_result_t
globus_i_streaming_init(
    globus_i_streaming_t *              handle,
    globus_ftp_client_handleattr_t *    ftp_attr,
    globus_reltime_t *                  idle_delay,
    globus_i_streaming_cb_t             url_cb,
    void *                              user_arg);

/* note: keep track of how many urls are added 
 * -- you can add multiple entries using the same xio handle.
 *     the handle will only have one outstanding operation at a time
 */
globus_result_t
globus_i_streaming_add(
    globus_i_streaming_t *              handle,
    const char *                        source_url,
    globus_ftp_client_operationattr_t * ftp_attr,
    globus_xio_handle_t                 dest_handle);

/* FINISH goes through once more and fetches everything available,
 * ABORT stops all xfers
 * 
 * delay can be NULL, specifies time to wait before grabbing last of data when
 * using FINISH.
 */
void
globus_i_streaming_stop(
    globus_i_streaming_t *              handle,
    int                                 how,
    globus_reltime_t *                  delay);

/* note: make sure the url_cb has been called for all urls before destroying */
void
globus_i_streaming_destroy(
    globus_i_streaming_t *              handle);

#endif
