/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.exec.service.exec;

import java.util.LinkedList;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.globus.exec.generated.InternalStateEnumeration;
import org.globus.exec.service.factory.FactoryHomeConfiguration;
import org.globus.exec.service.factory.ManagedJobFactoryHome;
import org.globus.exec.utils.Resources;

import org.globus.util.I18n;
import org.globus.wsrf.ResourceKey;

class RunQueue
{
    /** Log4J logger */
    private static Log logger = LogFactory.getLog(RunQueue.class);
    static I18n i18n = I18n.getI18n(Resources.class.getName());

    LinkedList readyQueue = new LinkedList();

    public static final int RUN_QUEUE_STAGE_IN = 0;
    public static final int RUN_QUEUE_SUBMIT = 1;
    public static final int RUN_QUEUE_STAGE_OUT = 2;
    public static final int RUN_QUEUE_FILE_CLEAN_UP = 3;
    public static final int RUN_QUEUE_CACHE_CLEAN_UP = 4;
    public static final int RUN_QUEUE_OTHER = 5;
    private int id = RUN_QUEUE_OTHER;

    private RunQueue() { }

    RunQueue(
        int                                 queueId)
    {
        this.id = queueId;
    }

    public synchronized void add(ResourceKey resourceKey)
    {
        if (logger.isTraceEnabled())
        {
            logger.debug("Entering add()");
        }

        this.readyQueue.add(resourceKey);
        notify();

        if (logger.isDebugEnabled())
        {
            logger.debug("adding " + resourceKey.toString() + " to queue.");
            int queuedJobCount = readyQueue.size();
            StringBuffer asterisks = new StringBuffer();
            for (int index=0; index<queuedJobCount; index++)
            {
                asterisks.append("*");
            }
            logger.debug("\n" + getName() + " Graph: " + asterisks.toString());
        }

        if (logger.isTraceEnabled())
        {
            logger.trace("Leaving add()");
        }
    }

    public ResourceKey remove()
    {
        ResourceKey resourceKey = (ResourceKey) this.readyQueue.removeFirst();

        if (logger.isDebugEnabled())
        {
            logger.debug("removing " + resourceKey.toString() + " from queue.");
            int queuedJobCount = readyQueue.size();
            StringBuffer asterisks = new StringBuffer();
            for (int index=0; index<queuedJobCount; index++)
            {
                asterisks.append("*");
            }
            logger.debug("\n" + getName() + " Graph: " + asterisks.toString());
        }

        return resourceKey;
    }

    public int size()
    {
        return this.readyQueue.size();
    }

    public String getName()
    {
        return RunQueue.getName(this.id);
    }

    private static String getName(int queueId)
    {
        switch (queueId)
        {
            case RUN_QUEUE_STAGE_IN:
                return "RunQueue " + InternalStateEnumeration._StageIn;
            case RUN_QUEUE_SUBMIT:
                return "RunQueue " + InternalStateEnumeration._Submit;
            case RUN_QUEUE_STAGE_OUT:
                return "RunQueue " + InternalStateEnumeration._StageOut;
            case RUN_QUEUE_FILE_CLEAN_UP:
                return "RunQueue " + InternalStateEnumeration._FileCleanUp;
            case RUN_QUEUE_CACHE_CLEAN_UP:
                return "RunQueue " + InternalStateEnumeration._CacheCleanUp;
            case RUN_QUEUE_OTHER:
                return "RunQueue Other";
            default:
                return null;
        }
    }
}
