/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.exec.service.exec;

import java.util.LinkedList;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.globus.exec.generated.InternalStateEnumeration;
import org.globus.exec.service.factory.FactoryHomeConfiguration;
import org.globus.exec.service.factory.ManagedJobFactoryHome;
import org.globus.exec.utils.ManagedExecutableJobConstants;
import org.globus.exec.utils.Resources;

import org.globus.util.I18n;
import org.globus.wsrf.ResourceKey;

class RunQueueGroup
{
    /** Log4J logger */
    private static Log logger = LogFactory.getLog(RunQueue.class);
    static I18n i18n = I18n.getI18n(Resources.class.getName());

    //one queue each for StageIn, Submit, StageOut,
    //FileCleanUp, CacheCleanUp,and other
    private static final int DEFAULT_RUN_THREADS_PER_QUEUE = 3;
    private static final int RUN_QUEUE_COUNT = 6;
    private static final int DEFAULT_RUN_THREAD_COUNT
        = RUN_QUEUE_COUNT * DEFAULT_RUN_THREADS_PER_QUEUE;
    private int runThreadsPerQueue = DEFAULT_RUN_THREADS_PER_QUEUE;
    private int runThreadCount;
    RunQueue[] runQueues = null;

    void initialize(
        ManagedExecutableJobHome            home)
    {
        //get the configured number of run queues
        FactoryHomeConfiguration homeConfig
            = ManagedJobFactoryHome.getConfiguration();
        try
        {
            this.runThreadCount
                = Integer.parseInt(homeConfig.getRunQueueThreadCount());
        }
        catch (Exception e)
        {
            logger.warn(i18n.getMessage(Resources.BAD_RUN_QUEUE_THREAD_COUNT),
                        e);
            this.runThreadCount = RunQueueGroup.DEFAULT_RUN_THREAD_COUNT;
        }

        //assert that there isn't a silly setting
        if (this.runThreadCount < RunQueueGroup.DEFAULT_RUN_THREAD_COUNT)
        {
            logger.warn(i18n.getMessage(Resources.BAD_RUN_QUEUE_THREAD_COUNT));
            this.runThreadCount = RunQueueGroup.DEFAULT_RUN_THREAD_COUNT;
        }

        logger.info(i18n.getMessage(
            Resources.REPORT_RUN_QUEUE_THREAD_COUNT,
            new Integer(this.runThreadCount)));

        this.runQueues = new RunQueue[RUN_QUEUE_COUNT];

        //create all run queue and run thread instances
        for (int queueIndex=0; queueIndex<RUN_QUEUE_COUNT; queueIndex++)
        {
            this.runQueues[queueIndex] = new RunQueue(queueIndex);
            int threadCount = this.runThreadsPerQueue;
            if (queueIndex == (RUN_QUEUE_COUNT - 1))
            {
                threadCount = this.runThreadsPerQueue
                            + (this.runThreadCount % RUN_QUEUE_COUNT);
            }
            for (int threadIndex=0; threadIndex<threadCount; threadIndex++)
            {
                RunThread runThread = new RunThread(
                    this.runQueues[queueIndex],
                    threadIndex);
                runThread.setDaemon(true);
                runThread.start();
            }
        }
    }

    synchronized RunQueue getInstance(
        int                                 queueId)
    {
        RunQueue runQueue = null;

        if (queueId <= RunQueue.RUN_QUEUE_OTHER)
        {
            logger.debug("Getting RunQueue instance #" + queueId);
            runQueue = this.runQueues[queueId];
        }

        logger.debug("Returning RunQueue instance: " + runQueue);

        return runQueue;
    }

    public static synchronized RunQueue getInstance()
    {
        RunQueueGroup runQueueGroup = null;
        try
        {
            runQueueGroup
                = ManagedExecutableJobHome.getHome().getRunQueueGroup();
        }
        catch (Exception e)
        {
            throw new RuntimeException(e);
        }

        return runQueueGroup.getInstance(RunQueue.RUN_QUEUE_OTHER);
    }

    public static synchronized RunQueue getInstance(
        ManagedExecutableJobResource        resource)
    {
        InternalStateEnumeration internalState = resource.getInternalState();

        int queueId = RunQueue.RUN_QUEUE_OTHER;
        if (internalState.equals(InternalStateEnumeration.StageIn))
        {
            queueId = RunQueue.RUN_QUEUE_STAGE_IN;
        }
        else
        if (internalState.equals(InternalStateEnumeration.Submit))
        {
            queueId = RunQueue.RUN_QUEUE_SUBMIT;
        }
        else
        if (internalState.equals(InternalStateEnumeration.StageOut))

        {
            queueId = RunQueue.RUN_QUEUE_STAGE_OUT;
        }
        else
        if (   internalState.equals(InternalStateEnumeration.FileCleanUp)
            || internalState.equals(
                    InternalStateEnumeration.FailureFileCleanUp))
        {
            queueId = RunQueue.RUN_QUEUE_FILE_CLEAN_UP;
        }
        else
        if (   internalState.equals(InternalStateEnumeration.CacheCleanUp)
            || internalState.equals(
                    InternalStateEnumeration.FailureCacheCleanUp))
        {
            queueId = RunQueue.RUN_QUEUE_CACHE_CLEAN_UP;
        }

        RunQueueGroup runQueueGroup = null;
        try
        {
            runQueueGroup
                = ManagedExecutableJobHome.getHome().getRunQueueGroup();
        }
        catch (Exception e)
        {
            throw new RuntimeException(e);
        }

        return runQueueGroup.getInstance(queueId);
    }
}
