#!/usr/bin/perl

use strict;
use IO::Handle;

my $file=$ARGV[0];

STDOUT->autoflush(1);

my %duration_total_hash = (
    'createManagedJob' => 0,
    'processNoneState' => 0,
    'processStartState' => 0,
    'processStageInState' => 0,
    'processSubmitState' => 0,
    'processOpenStdoutState' => 0,
    'processOpenStderrState' => 0,
    'processWaitingForStateChangesState' => 0,
    'processStageOutState' => 0,
    'processCleanUpState' => 0,
    'processFileCleanUpState' => 0,
    'processCacheCleanUpState' => 0,
    'processDoneState' => 0,
    'remove' => 0
);
my $timingCount = 0;
my $timingName;
my $timingValue;

open FILE, "$file" or die "Error: unable to open file ".$file." for reading";
while (<FILE>)
{
    chomp;

#print "line: " . $_ . "\n";

    if (! /\.performance/)
    {
        next;
    }

    $timingName = $_;
    $timingName =~ s/^..*\] \[//;
    $timingName =~ s/]..*//;

    $timingValue = $_;
    $timingValue =~ s/^..*\] \[[^]]+\]\[Thread-[^\]]+\]\[//;
    $timingValue =~ s/\]//;

    $duration_total_hash{$timingName} += $timingValue;

    if (/\[remove\]/)
    {
        $timingCount++;
    }
}
close FILE;

my $total_duration = 0;

print "Timing Count: " . $timingCount . "\n\n";

#collect normalized timings into a multidimensional array
my @timings;
my $timingIndex = 0;
my @keys = keys(%duration_total_hash);
foreach my $key (@keys) {
    if ($duration_total_hash{$key} > 0)
    {
        $duration_total_hash{$key} /= ($timingCount * 1000);
    }

    $timings[$timingIndex++] = [$key, $duration_total_hash{$key}];

    $total_duration += $duration_total_hash{$key};
}

#sort timings in ascending order of duration
my @a;
my @b;
my @sortedTimings = sort { $a->[1] <=> $b->[1] } @timings;

#print sorted timings
for ($timingIndex=0; $timingIndex<$#sortedTimings; $timingIndex++)
{
    print "$sortedTimings[$timingIndex][0] duration average (seconds): $sortedTimings[$timingIndex][1]\n";
}

print "\ntotal duration average (seconds): " . $total_duration . "\n";

#my $execution_duration = $total_duration
                       #- $duration_total_hash{'createManagedJob'};
                       #- $duration_total_hash{'remove'};

#print "rough execution duration average (seconds): " . $execution_duration . "\n";
