/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.exec.service.test.throughput;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.Hashtable;
import java.util.Timer;
import java.util.TimerTask;
import java.util.Vector;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.globus.exec.generated.JobDescriptionType;
import org.globus.exec.utils.rsl.RSLHelper;
import org.globus.exec.utils.rsl.RSLParseException;

import org.globus.wsrf.utils.PerformanceLog;

/**
 *  Description of the Class
 *
 *@author     lane
 *@author     madduri
 *@created    July 6, 2004
 */
public class ThroughputTester {

    static Log logger = LogFactory.getLog(ThroughputTester.class.getName());

    String contactString = null;
    String factoryType = null;
    int load = 1;
    int parallelism = 1;
    long duration = 1;
    ClientThread[] clients = null;
    int completedCount = 0;
    int jobGrandTotal = 0;
    boolean durationElapsed = false;
    boolean delegationEnabled = true;
    int jobDuration = 5;
    String jobCommand = "";
    JobDescriptionType jobDescription = null;
    boolean polling = false;
    boolean jobDirUsageEnabled = false;

    PerformanceLog perfLog = new PerformanceLog(
            ThroughputTester.class.getName());
    PerformanceLog completePerfLog = new PerformanceLog(
            ThroughputTester.class.getName() + ".complete");


    /**
     *  Constructor for the ThroughputTester object
     */
    public ThroughputTester() { }


    /**
     *  Main processing method for the ThroughputTester object
     */
    public synchronized void run()
    {
        File logDir = new File(
            System.getProperty("user.dir") + "/throughput-logs/");
        logDir.mkdir();

        //start timing actual duration
        long startTime = System.currentTimeMillis();

        this.clients = new ClientThread[this.parallelism];
        for (int index = 0; index < this.clients.length; index++) {
            this.clients[index] = new ClientThread(this);
            new Thread(this.clients[index]).start();
        }

        //start a timer to stop the madness when the time is right
        QuitTimerTask quitTimerTask = new QuitTimerTask(this);
        Timer quitTimer = new Timer();
        quitTimer.schedule(quitTimerTask, this.duration * 1000); //milliseconds

        try {
            //wait for all clients to checkin with their job totals
            wait();
        } catch (Exception e) {
            logger.error("wait() canceled unexpectedly", e);
        }

        //calculate actual duration and jobs per minute across all clients
        System.out.println("Total Jobs Submitted: " + this.jobGrandTotal);
        long actualDuration = (System.currentTimeMillis() - startTime) / 1000;
        System.out.println("Actual Duration (seconds): " + actualDuration);
        double durationInMinutes = ((double) actualDuration) / 60;
        System.out.println("Actual Duration (minutes): " + durationInMinutes);
        double jobsPerMinute = Math.round(
                this.jobGrandTotal / durationInMinutes);
        System.out.println("Jobs Per Minute: " + jobsPerMinute);

        System.exit(0);
    }


    /**
     *  Description of the Method
     */
    synchronized void notifyError() {
        logger.error("unable to proceed with test");
        System.exit(1);
    }


    /**
     *  Description of the Method
     *
     *@param  clientIndex  Description of the Parameter
     *@param  jobTotal     Description of the Parameter
     */
    synchronized void notifyCompleted(int clientIndex, int jobTotal)
    {
        if (logger.isDebugEnabled()) {
            logger.debug("got completed signal from client #" + clientIndex);
            logger.debug("Thread job total: " + jobTotal);
        }
        this.completedCount++;
        this.jobGrandTotal += jobTotal;

        if (this.completedCount == this.clients.length) {
            notify();
            //notify wait() at end of run()
        }
    }


    /**
     *  Sets the contactString attribute of the ThroughputTester object
     *
     *@param  contactString  The new contactString value
     */
    public void setContactString(String contactString) {
        this.contactString = contactString;
    }


    /**
     *  Gets the contactString attribute of the ThroughputTester object
     *
     *@return    The contactString value
     */
    public String getContactString() {
        return this.contactString;
    }


    /**
     *  Sets the factoryType attribute of the ThroughputTester object
     *
     *@param  factoryType  The new factoryType value
     */
    public void setFactoryType(String factoryType) {
        this.factoryType = factoryType;
    }


    /**
     *  Gets the factoryType attribute of the ThroughputTester object
     *
     *@return    The factoryType value
     */
    public String getFactoryType() {
        return this.factoryType;
    }


    /**
     *  Sets the load attribute of the ThroughputTester object
     *
     *@param  load  The new load value
     */
    public void setLoad(int load) {
        this.load = load;
    }


    /**
     *  Gets the load attribute of the ThroughputTester object
     *
     *@return    The load value
     */
    public int getLoad() {
        return this.load;
    }


    /**
     *  Sets the parallelism attribute of the ThroughputTester object
     *
     *@param  parallelism  The new parallelism value
     */
    public void setParallelism(int parallelism) {
        this.parallelism = parallelism;
    }


    /**
     *  Gets the parallelism attribute of the ThroughputTester object
     *
     *@return    The parallelism value
     */
    public int getParallelism() {
        return this.parallelism;
    }


    /**
     *  Sets the length of time after which no more jobs will be submitted.
     *
     *  @param  duration test duration in seconds
     */
    public void setDuration(long duration) {
        this.duration = duration;
    }


    /**
     *  Sets the length of time after which no more jobs will be submitted.
     *
     *  @return test duration in seconds
     */
    public long getDuration() {
        return this.duration;
    }


    /**
     *  Gets the durationElapsed attribute of the ThroughputTester object
     *
     *@return    The durationElapsed value
     */
    public boolean isDurationElapsed() {
        return this.durationElapsed;
    }


    /**
     *  Sets the durationElapsed attribute of the ThroughputTester object
     *
     *@param  durationElapsed  The new durationElapsed value
     */
    void setDurationElapsed(boolean durationElapsed) {
        this.durationElapsed = durationElapsed;
    }

    /**
     *  Sets whether or not delegation should be performed
     *
     *@param  delegationEnabled  true for delegation, false for none
     */
    public void setDelegationEnabled(boolean delegationEnabled) {
        this.delegationEnabled = delegationEnabled;
    }

    /**
     *  Indicates whether or not delegation will be performed
     *
     *@return   true for delegation, false for none
     */
    public boolean isDelegationEnabled() {
        return this.delegationEnabled;
    }

    /**
     *  Sets the lentgh of time an individual job runs.
     *
     *  @param  jobDuration job duration in seconds
     */
    public void setJobDuration(int jobDuration)
    {
        this.jobDuration = jobDuration;
    }

    /**
     *  Gets the lentgh of time an individual job runs.
     *
     *  @return job duration in seconds
     */
    public int getJobDuration()
    {
        return this.jobDuration;
    }

    /**
     *  Sets the command each job will execute. This value overrides the default
     *  command of '/bin/sleep &lt;job duration&gt;', and thus causes
     *  jobDuration to be ignored.
     *
     *  @param  jobCommand the command to be executed for each job
     */
    public void setJobCommand(String jobCommand)
    {
        this.jobCommand = jobCommand;
    }

    /**
     *  Gets the command each job will execute.
     *
     *  @return  the command to be exenuted for each job
     */
    public String getJobCommand()
    {
        return this.jobCommand;
    }

    public void setJobDescriptionFilename(
        String                              jobDescriptionFilename)
        throws                              RSLParseException,
                                            FileNotFoundException
    {
        if (jobDescriptionFilename.length() == 0)
        {
            this.jobDescription = RSLHelper.makeSimpleJob(this.jobCommand);
            this.jobDescription.setStdout(
                System.getProperty("user.dir") + "/throughput-logs/stdout");
            this.jobDescription.setStderr(
                System.getProperty("user.dir") + "/throughput-logs/stderr");
        } else
        {
            File jobDescriptionFile = new File(jobDescriptionFilename);
            this.jobDescription = RSLHelper.readRSL(jobDescriptionFile);
        }
    }

    public JobDescriptionType getJobDescription()
    {
        return this.jobDescription;
    }

    public void setPolling(boolean polling)
    {
        this.polling = polling;
    }

    public boolean isPolling()
    {
        return this.polling;
    }


    /**
     *  Description of the Method
     *
     *@param  customMessage  Description of the Parameter
     */
    public static void printUsage(String customMessage) {
        StringBuffer usageMessage = new StringBuffer(customMessage);
        usageMessage.append("\nUsage:");
        usageMessage.append("\njava [JAVA OPTIONS] ");
        usageMessage.append(ThroughputTester.class.getName());
        usageMessage.append(" [OPTIONS]\n\nOPTIONS:\n");
        usageMessage.append("\n\t--help\t\t\t\tPrint this usage message.");
        usageMessage.append("\n\t--factory <contact string>");
        usageMessage.append("\tMJFS service address of the form");
        usageMessage.append("\n\t\t\t\t\t[protocol://]host[:[port]]");
        usageMessage.append("[/service path/service name]");
        usageMessage.append("\n\t--type <factory type>");
        usageMessage.append("\t\tMJFS type. Can be \"Fork\" (default), ");
        usageMessage.append("\n\t\t\t\t\t\"Condor\", \"Lsf\", or \"Pbs\"");
        usageMessage.append("\n\t--delegate <boolean>");
        usageMessage.append("\t\tEnable delegation.");
        usageMessage.append("\n\t\t\t\t\tEither \"true\" ");
        usageMessage.append("or \"false\" (default)");
        usageMessage.append("\n\t--job-duration <job duration>");
        usageMessage.append("\tDuration of an individual job ");
        usageMessage.append("\n\t\t\t\t\tin seconds (default is 5)");
        usageMessage.append("\n\t--load <load>\t\t\tPer-thread job load");
        usageMessage.append("\n\t--parallelism <parallelism>");
        usageMessage.append("\tNumber of client threads");
        usageMessage.append("\n\t--test-duration <test duration>");
        usageMessage.append("\tLoad maintenance duratuon in seconds");
        usageMessage.append("\n\t--command <job command>");
        usageMessage.append("\t\tCommand to be executed for each job ");
        usageMessage.append("\n\t\t\t\t\t(default is \'/bin/sleep ");
        usageMessage.append("<job duration>\')");
        usageMessage.append("\n\t--rsl <job description path>");
        usageMessage.append("\tPath pointing to the RSL to use instead.");
        usageMessage.append("\n\t--poll <boolean>");
        usageMessage.append("\t\tUse polling in place of notifications");
        usageMessage.append("\n\t\t\t\t\tfor getting job state. Either");
        usageMessage.append("\"true\" or\n\t\t\t\t\t\"false\" (default)");
        System.out.println(usageMessage.toString());
    }


    /**
     *  The main program for the ThroughputTester class
     *
     *@param  args  The command line arguments
     */
    public static void main(String[] args) {
        if (logger.isDebugEnabled()) {
            logger.debug("argument count: " + args.length);
        }
        Hashtable options = new Hashtable();
        options.put("help",         new Boolean(false));
        options.put("factory",      "");
        options.put("type",         "Fork");
        options.put("delegate",     "false");
        options.put("job-duration", "5");
        options.put("load",         "1");
        options.put("parallelism",  "1");
        options.put("test-duration","1");
        options.put("command",      "");
        options.put("rsl",          "");
        options.put("poll",         "false");
        String argName = null;
        String argValue = null;
        for (int index = 0; index < args.length; index++) {
            if (logger.isDebugEnabled()) {
                logger.debug("argument #" + index + ": " + args[index]);
            }
            if (args[index].startsWith("-"))
            {
                if (args[index].startsWith("--"))
                {
                    //handle '--' args
                    argName = args[index].substring(2);
                } else
                {
                    //handle '-' args
                    argName = args[index].substring(1);
                }

                if (logger.isDebugEnabled()) {
                    logger.debug("argument name: " + argName);
                }
                if (!options.containsKey(argName))
                {
                    logger.error("\nunknown option \"" + argName + "\"");
                    System.exit(1);
                }

                index++;
                try {
                    argValue = args[index];
                } catch (ArrayIndexOutOfBoundsException e) {
                    Object defaultValue = options.get(argName);
                    if ((defaultValue != null) &&
                            (defaultValue instanceof String)) {
                        ThroughputTester.printUsage(
                                "Error: value expected for " + argName);
                        System.exit(1);
                    }
                }
                if (logger.isDebugEnabled()) {
                    logger.debug("argument value: " + argValue);
                }
                if (((argValue != null) && (argValue.startsWith("-")))
                         || (argValue == null)) {
                    index--;
                    options.put(argName, new Boolean(true));
                } else {
                    options.put(argName, argValue);
                }
            } else {
                ThroughputTester.printUsage(
                        "Error: unexpected argument " + argName);
                System.exit(1);
            }
            argValue = null;
        }
        if (((Boolean) options.get("help")).booleanValue()) {
            ThroughputTester.printUsage("-- Help --");
            System.exit(0);
        }
        if (((String) options.get("factory")).length() < 0) {
            ThroughputTester.printUsage("Error: --factory <url> not specified");
            System.exit(1);
        }
        if (logger.isDebugEnabled()) {
            logger.debug("Factory Contact: "    + options.get("factory"));
            logger.debug("Factory Type: "       + options.get("type"));
            logger.debug("Delegation: "         + options.get("delegate"));
            logger.debug("Job Duration: "       + options.get("job-duration"));
            logger.debug("Per-Thread Job Load: "+ options.get("load"));
            logger.debug("Parallelism: "        + options.get("parallelism"));
            logger.debug("Load Maintenance Duration: "
                                                + options.get("test-duration"));
            logger.debug("Job Command: "        + options.get("command"));
            logger.debug("RSL: "                + options.get("rsl"));
            logger.debug("Poll: "               + options.get("poll"));
        }

        ThroughputTester harness = new ThroughputTester();
        harness.setContactString((String) options.get("factory"));
        harness.setFactoryType((String) options.get("type"));
        harness.setDelegationEnabled(
            Boolean.valueOf((String) options.get("delegate")).booleanValue());
        harness.setJobDuration(
            Integer.parseInt((String) options.get("job-duration")));
        harness.setLoad(Integer.parseInt((String) options.get("load")));
        harness.setParallelism(
            Integer.parseInt((String) options.get("parallelism")));
        harness.setDuration(
            Integer.parseInt((String) options.get("test-duration")));
        String command = (String) options.get("command");
        if (command.length() == 0)
        {
            command = "/bin/sleep " + harness.getJobDuration();
        }
        harness.setJobCommand(command);
        try {
            harness.setJobDescriptionFilename((String) options.get("rsl"));
        } catch(Exception e)
        {
            logger.error("Unable to load RSL", e);
            System.exit(1);
        }
        harness.setPolling(
            Boolean.valueOf((String) options.get("poll")).booleanValue());

        harness.run();
    }
}

/**
 *  Description of the Class
 *
 *@author     lane
 *@author     madduri
 *@created    July 6, 2004
 */
class QuitTimerTask extends TimerTask {


    ThroughputTester harness = null;


    /**
     *  Constructor for the QuitTimerTask object
     *
     *@param  harness  Description of the Parameter
     */
    public QuitTimerTask(ThroughputTester harness) {
        this.harness = harness;
    }


    /**
     *  Main processing method for the QuitTimerTask object
     */
    public void run() {
        this.harness.setDurationElapsed(true);
    }
}

