/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.exec.service.test;


import java.io.File;
import java.net.URL;
import java.security.cert.X509Certificate;
import java.util.Calendar;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import javax.xml.namespace.QName;
import javax.xml.rpc.Stub;

import org.oasis.wsn.Subscribe;
import org.oasis.wsn.SubscriptionManager;
import org.oasis.wsn.WSBaseNotificationServiceAddressingLocator;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.apache.axis.components.uuid.UUIDGen;
import org.apache.axis.components.uuid.UUIDGenFactory;
import org.apache.axis.message.MessageElement;
import org.apache.axis.message.addressing.AttributedURI;
import org.apache.axis.message.addressing.EndpointReferenceType;
import org.apache.axis.message.addressing.ReferencePropertiesType;
import org.apache.axis.types.URI;

import org.oasis.wsn.TopicExpressionType;
import org.oasis.wsrf.faults.BaseFaultType;
import org.oasis.wsrf.lifetime.Destroy;
import org.oasis.wsrf.lifetime.SetTerminationTime;
import org.oasis.wsrf.lifetime.SetTerminationTimeResponse;
import org.oasis.wsrf.properties.QueryExpressionType;
import org.oasis.wsrf.properties.GetMultipleResourceProperties_Element;
import org.oasis.wsrf.properties.GetMultipleResourcePropertiesResponse;
import org.oasis.wsrf.properties.GetResourcePropertyResponse;
import org.oasis.wsrf.properties.QueryResourceProperties_Element;
import org.oasis.wsrf.properties.QueryResourcePropertiesResponse;
import org.w3c.dom.Element;

import org.globus.axis.util.Util;
import org.globus.delegation.DelegationConstants;
import org.globus.delegation.DelegationUtil;
import org.globus.exec.generated.FaultResourcePropertyType;
import org.globus.exec.generated.CreateManagedJobInputType;
import org.globus.exec.generated.CreateManagedJobOutputType;
import org.globus.exec.generated.JobDescriptionType;
import org.globus.exec.generated.ManagedJobFactoryPortType;
import org.globus.exec.generated.ManagedJobPortType;
import org.globus.exec.generated.MultiJobDescriptionType;
import org.globus.exec.generated.ServiceLevelAgreementType;
import org.globus.exec.generated.StateChangeNotificationMessageType;
import org.globus.exec.generated.StateEnumeration;
import org.globus.exec.utils.ManagedJobConstants;
import org.globus.exec.utils.ManagedJobFactoryConstants;
import org.globus.rendezvous.client.ClientSecurityStrategy;
import org.globus.exec.utils.JobClientSecurityStrategy;
import org.globus.exec.utils.service.ManagedJobHelper;
import org.globus.exec.utils.client.ManagedJobClientHelper;
import org.globus.exec.utils.client.ManagedJobFactoryClientHelper;
import org.globus.exec.utils.rsl.RSLHelper;
import org.globus.gsi.GlobusCredential;
import org.globus.wsrf.NotificationConsumerManager;
import org.globus.wsrf.NotifyCallback;
import org.globus.wsrf.WSNConstants;
import org.globus.wsrf.WSRFConstants;
import org.globus.wsrf.container.ServiceContainer;
import org.globus.wsrf.encoding.ObjectDeserializer;
import org.globus.wsrf.impl.security.authentication.Constants;
import org.globus.wsrf.impl.security.authorization.Authorization;
import org.globus.wsrf.impl.security.authorization.HostAuthorization;
import org.globus.wsrf.impl.security.authorization.SelfAuthorization;
import org.globus.wsrf.impl.security.authorization.IdentityAuthorization;
import org.globus.wsrf.impl.security.descriptor.ClientSecurityDescriptor;
import org.globus.wsrf.impl.security.descriptor.GSISecureMsgAuthMethod;
import org.globus.wsrf.impl.security.descriptor.GSITransportAuthMethod;
import org.globus.wsrf.impl.security.descriptor.ResourceSecurityDescriptor;
import org.globus.wsrf.test.GridTestCase;
import org.globus.wsrf.test.GridTestSuite;
import org.globus.wsrf.utils.AddressingUtils;
import org.globus.wsrf.utils.AnyHelper;
import org.globus.wsrf.utils.XmlUtils;


public abstract class JobSubmissionTest extends GridTestCase {

    public JobSubmissionTest(String name) {
        super(name);
    }

    protected void setUp() throws Exception
    {
        super.setUp();

        setupSecurity();

        if (this.credentialEndpoint == null) {
            try {
                this.credentialEndpoint = delegate();
            } catch (Exception e) {
                throw new RuntimeException("Failed to delegate.", e);
            }
        }
    }

    private void setupSecurity()
    {
        Util.registerTransport();

        final Authorization DEFAULT_AUTHZ = HostAuthorization.getInstance();

        String authzIdentity = System.getProperty("authz.identity");
        if (logger.isDebugEnabled())
        {
            logger.debug("authz.identity: " + authzIdentity);
        }
        if ((authzIdentity == null) || authzIdentity.equals("")) {
            authorizationMethod = DEFAULT_AUTHZ;
        } else {
            authorizationMethod = new IdentityAuthorization(authzIdentity);
        }

        if (jobStubSecurity == null) {
            jobStubSecurity = new JobClientSecurityStrategy();
            ((JobClientSecurityStrategy)jobStubSecurity). //TODO generic ClientSecurityStrategy in Core
                setAuthorization(authorizationMethod);
        }

        if (jobFactoryStubSecurity == null) {
            jobFactoryStubSecurity = jobStubSecurity;
            //reuse: no need for extra object *for now*
        }

        if (subscriptionStubSecurity == null) {
            subscriptionStubSecurity = jobStubSecurity;
            //reuse: no need for extra object *for now*
        }

    }

    private static Authorization   authorizationMethod;
    private ClientSecurityStrategy jobStubSecurity;
    private ClientSecurityStrategy jobFactoryStubSecurity;
    private ClientSecurityStrategy subscriptionStubSecurity;

    protected void tearDown() {
        logger.trace("tearDown called");

        try {
            unsubscribe();
        }
        catch (Exception e) {
            logger.error(e);
            throw new RuntimeException("Job subscription cleanup failed");
        }

        try {
            destroyJob();
        } catch (Exception e) {
            logger.error(e);
            throw new RuntimeException("Job destruction failed");
        }

    }

    protected void destroyJob() throws Exception {
        if (this.jobEndpoint != null) {
            final ManagedJobPortType port = getSecureJobPort();
            logger.debug("destroying job resource");
            //noticed some bug trying to destroy while job not done.
            //let's do it on a thread so we can complain in a log for now
            Runnable activeDestroy =
                new Runnable()
                    {
                        public void run() {
                            try {
                                port.destroy(new Destroy());
                            }
                            catch (Exception e) {
                                throw new RuntimeException(
                                    "could not destroy job", e);
                            }
                            logger.debug("destroyed job resource");
                            jobEndpoint = null;
                        }
                    };
            new Thread(activeDestroy).start();
            int maxWaitMillis = getWaitTimeMillis();
            int waited = 0;
            while (jobEndpoint != null && waited < maxWaitMillis) {
                waited += 20;
                Thread.sleep(20);
            }
            if (waited >= maxWaitMillis && jobEndpoint != null) {
                String errorMessage =
                    "could not destroy job resource in " + maxWaitMillis +
                    " milliseconds! Probably a bug in GRAM service.";
                logger.error(errorMessage);
                throw new RuntimeException(errorMessage);
            }
        }
    }

    protected void unsubscribe() throws Exception {

        if (this.subscriptionEndpoint != null) {
            SubscriptionManager subscriptionPort
                = new WSBaseNotificationServiceAddressingLocator().
                    getSubscriptionManagerPort(this.subscriptionEndpoint);

            this.subscriptionStubSecurity.setStubPropertiesForDefaultOperation(
                (Stub) subscriptionPort);

            subscriptionPort.destroy(new Destroy());

            this.subscriptionEndpoint = null;
        }


        if (this.notificationConsumerEPR != null) {
            logger.debug("removing the notification consumer");
            notificationConsumerManager.removeNotificationConsumer(
                notificationConsumerEPR);
            this.notificationConsumerEPR = null;
        }

        if ( this.notificationConsumerManager != null &&
             this.notificationConsumerManager.isListening()) {
           logger.debug("stopping the consumer manager from listening");
           notificationConsumerManager.stopListening();
           this.notificationConsumerManager = null;
        }
    }

    protected EndpointReferenceType getJobEndpoint() {
        return this.jobEndpoint;
    }

    protected EndpointReferenceType getCredentialEndpoint() {
        return this.credentialEndpoint;
    }

    /**
     * The job description does not need to have the job and staging credentials
     * specified.
     * @return JobDescriptionType
     */
    protected JobDescriptionType getDefaultJobDescription() throws Exception {
        return RSLHelper.readRSL(new File(getDefaultRSLFilename()));
    }


    protected String getDefaultRSLFilename() {
        return SIMPLEST_JOB_RSL;
    }

    //TODO make private
    protected static EndpointReferenceType delegate() throws Exception
    {
        // Credential to sign with
        GlobusCredential credential = GlobusCredential.getDefaultCredential();

        String factoryUrl = TEST_CONTAINER.getBaseURL() +
                            DelegationConstants.FACTORY_PATH;
        if (logger.isDebugEnabled()) {
            logger.debug("Factory URL " + factoryUrl);
        }

        // lifetime in seconds
        int lifetime = 24 * 60 * 60; //1 day

        // Set security properties
        ClientSecurityDescriptor desc = new ClientSecurityDescriptor();
        desc.setAuthz(authorizationMethod);
        // FIXME: Set approprite security, for now setting transport siganature
        desc.setGSITransport(Constants.SIGNATURE);
        /* Sample code:
        if (mechanism.equals(Constants.GSI_SEC_MSG)) {
            desc.setGSISecureMsg((Integer)protection);
        } else if (mechanism.equals(Constants.GSI_SEC_CONV)) {
            desc.setGSISecureConv((Integer)protection);
        } else if (mechanism.equals(Constants.GSI_TRANSPORT)) {
            desc.setGSITransport((Integer)protection);
        }
        */
        
        // Get the public key to delegate on.
        EndpointReferenceType delegEpr =
            AddressingUtils.createEndpointReference(factoryUrl, null);
        X509Certificate[] certsToDelegateOn =
            DelegationUtil.getCertificateChainRP(delegEpr, desc);
        X509Certificate certToSign = certsToDelegateOn[0];

        // send to delegation service and get epr.
        EndpointReferenceType credentialEndpoint = DelegationUtil.delegate(
            factoryUrl,
            credential,
            certToSign,
            lifetime,
            false,
            desc);
        if (logger.isDebugEnabled()) {
            logger.debug("Delegated Credential Endpoint:\n"
                        + credentialEndpoint);
        }

        return credentialEndpoint;
    }


    private EndpointReferenceType getSecureNotificationConsumer(
        NotificationConsumerManager notificationConsumerManager,
        List topicPath,
        NotifyCallback callback) throws Exception
    {
        ResourceSecurityDescriptor securityDescriptor
            = this.jobStubSecurity.getSecurityDescriptor();

        EndpointReferenceType notificationConsumerEPR =
            notificationConsumerManager.createNotificationConsumer(
                topicPath,
                callback,
                securityDescriptor);
        return notificationConsumerEPR;
    }

    protected void submitJobAndWaitForCompletion(
        JobDescriptionType jobDescription)
        throws Exception
    {
        submitJobWithStateSubscription(jobDescription);

        int maxWaitMillis = getWaitTimeMillis();
        waitForJobCompletion(maxWaitMillis);
    }

    protected void submitJobAndWaitForState(
        JobDescriptionType jobDescription, StateEnumeration expectedState)
        throws Exception
    {
        submitJobWithStateSubscription(jobDescription);

        int maxWaitMillis = getWaitTimeMillis();
        waitForJobState(expectedState, maxWaitMillis);
    }

    protected void submitJobWithStateSubscription(
        JobDescriptionType jobDescription)
        throws Exception
    {
        this.callback = new TestNotifyCallback();
        QName topicQName = ManagedJobConstants.RP_STATE;
        /*
        this.notificationConsumerManager =
            NotificationConsumerManager.getInstance();
            */
        Map properties = new HashMap();
        properties.put(
            ServiceContainer.CLASS,
            "org.globus.wsrf.container.GSIServiceContainer");
        this.notificationConsumerManager
            = NotificationConsumerManager.getInstance(properties);
        this.notificationConsumerManager.startListening();
        List testTopicPath = new LinkedList();
        testTopicPath.add(topicQName);
        this.notificationConsumerEPR = getSecureNotificationConsumer(
            this.notificationConsumerManager, testTopicPath, callback);

        Subscribe request = new Subscribe();
        request.setConsumerReference(this.notificationConsumerEPR);
        TopicExpressionType topicExpression = new TopicExpressionType(
            WSNConstants.SIMPLE_TOPIC_DIALECT,
            topicQName);
        request.setTopicExpression(topicExpression);

        submitJob(getDefaultTerminationTime(),
                  jobDescription,
                  request);
    }


    /**
     *
     * @return int the time in milliseconds to wait for notifications
     * before failing the test
     */
    protected int getWaitTimeMillis() {
        return 30000;
    }

    private StateEnumeration pullJobState() throws Exception {
        ManagedJobPortType jobPortType =
             getSecureJobPort();
         GetResourcePropertyResponse response =
             jobPortType.getResourceProperty(ManagedJobConstants.RP_STATE);
         MessageElement [] any = response.get_any();
         assertTrue(any != null);
         assertTrue(any.length > 0);
         StateEnumeration state = (StateEnumeration) ObjectDeserializer.toObject(
             any[0], StateEnumeration.class);
         assertTrue(state != null);
         return state;
    }

/* apply to executable jobs only - not multi
    private int pullJobExitCode() throws Exception {
        ManagedJobPortType jobPortType =
             getSecureJobPort();
         GetResourcePropertyResponse response =
             jobPortType.getResourceProperty(ManagedExecutableJobConstants.RP_RP_STATE);
         MessageElement [] any = response.get_any();
         assertTrue(any != null);
         assertTrue(any.length > 0);
         Integer exitCode = (Integer) ObjectDeserializer.toObject(
             any[0], Integer.class);
         assertTrue(exitCode != null);
         return exitCode.intValue();
    }*/



    protected boolean isJobComplete() {
        return
            wasNotified(StateEnumeration.Done) ||
            wasNotified(StateEnumeration.Failed);
    }

    protected boolean wasNotified(StateEnumeration checkState)
    {
        logger.debug("entering wasNotified()");

        //see if we actually received the desired notification message
        Vector states = this.callback.getReceivedStates();
        if (states.contains(checkState)) {
            logger.debug("Was notified of state " + checkState.toString());
            return true;
        }

        logger.debug("Was NOT notified of state " + checkState.toString());

        return false;
    }

    protected void waitForCount(int count, int maxWait)
        throws Exception
    {
        int waited = 0;
        while(waited < maxWait &&
              callback.getNotifyCount() != count)
        {
            Thread.sleep(20);
            waited += 20;
        }

        assertEquals(ManagedJobConstants.RP_STATE,
                     this.callback.getNotificationTopic());
    }



    protected void waitForJobState(
        StateEnumeration expectedState,
        int maxWaitMillis)
        throws Exception
    {
        int waited = 0;
        this.jobState = pullJobState();
        logger.debug("pulling job state from service: " + jobState);
        while(!isJobComplete() && waited < maxWaitMillis)
        {
            Thread.sleep(20);
            waited += 20;
            if (callback.getJobState() != null) {
                this.jobState = callback.getJobState();
            }
        }

        if (logger.isDebugEnabled()) {
            String genericStatusMessage =
                "After waiting for completion, the state of the job is " +
                this.jobState;
            logger.debug(genericStatusMessage);
        }

        assertTrue("job not " + expectedState,
                   wasNotified(expectedState));//this.jobState.equals(expectedState));

        String hangingErrorMessage =
            "The job hasn't reached the expected state " + expectedState +
            " after waiting for notifications for " + maxWaitMillis +
            " milliseconds.\n";
        assertTrue(hangingErrorMessage, waited < maxWaitMillis);
    }

    protected void waitForJobCompletion(
        int maxWaitMillis)
        throws Exception
    {
        int waited = 0;
        this.jobState = pullJobState();
        logger.debug("pulling job state from service: " + jobState);
        while(!isJobComplete() && waited < maxWaitMillis)
        {
            Thread.sleep(20);
            waited += 20;
            if (callback.getJobState() != null) {
                this.jobState = callback.getJobState();
            }
        }

        if (logger.isDebugEnabled()) {
            String genericStatusMessage =
                "After waiting for completion, the state of the job is " +
                this.jobState;
            logger.debug(genericStatusMessage);
        }

        boolean jobCompleted = isJobComplete();
        String hangingErrorMessage =
            "The job hasn't completed after waiting for notifications for " +
            maxWaitMillis + " milliseconds.\n";
        assertTrue(hangingErrorMessage, jobCompleted);

        assertTrue("job not Done",
                   this.wasNotified(StateEnumeration.Done));

        if (logger.isDebugEnabled()) {
            StateEnumeration expectedLastState = StateEnumeration.Done;
            if (!this.jobState.equals(StateEnumeration.Done)) {
                logger.warn("The state " + expectedLastState +
                            " was not the last state received");
            }
        }

    }

    protected TestNotifyCallback getCallback() {
        return this.callback;
    }

    private StateEnumeration jobState;

    protected class TestNotifyCallback implements NotifyCallback
    {
        private int notifyCount = 0;
        private QName notificationTopic = null;
        private Object message = null;
        private BaseFaultType fault;
        private StateEnumeration state;
        private Vector receivedStates = new Vector();
        private boolean holding = false;
        private int exitCode;

        public synchronized int getNotifyCount()
        {
            return notifyCount;
        }

        public synchronized QName getNotificationTopic()
        {
            return notificationTopic;
        }

        public synchronized Object getMessage()
        {
            return message;
        }

        public synchronized Vector getReceivedStates()
        {
            return this.receivedStates;
        }

        public synchronized StateEnumeration getJobState()
        {
            return this.state;
        }

        public synchronized int getJobExitCode()
        {
            return exitCode;
        }

        public synchronized boolean isHolding()
        {
            return holding;
        }

        public synchronized void setHolding(boolean newHoldingValue)
        {
            this.holding = newHoldingValue;        }


        public synchronized BaseFaultType getFault() {
            return fault;
        }

        private void setFault(BaseFaultType fault)
            throws Exception
        {
            this.fault = fault;

            if ((this.fault != null) && logger.isErrorEnabled()) {
                // Call faultToString to get client-side localized
                // error string
                /**
                 * @todo uncomment when FaultUtils and its
                 * Gram and Core dependencies are ported
                String faultDescription =
                    FaultUtils.faultToString(fault);
                logger.error(faultDescription);
                */
                //log descriptions of each cause
                BaseFaultType[] causes = fault.getFaultCause();
                while (causes != null && causes[0] != null) {
                    if (causes[0] instanceof BaseFaultType) {
                        BaseFaultType gramFault = (BaseFaultType)causes[0];
                        /**
                         * @todo uncomment when FaultUtils and its
                         * Gram and Core dependencies are ported
                         *
                        logger.error(
                            FaultUtils.faultToString(gramFault);*/
                        logger.error(gramFault);//remove
                    } else {
                        logger.error(causes[0].getDescription()[0]);
                    }
                    //traversal of fault tree
                    causes = causes[0].getFaultCause();
                } //end while
            }
        }


        public synchronized void deliver(
            List topicPath,
            EndpointReferenceType producer,
            Object message)
        {
            this.message = message;
            this.notificationTopic = (QName) topicPath.get(0);
            this.notifyCount++;

            if (logger.isDebugEnabled()) {
                logger.debug("receiving notification");
                if (message instanceof Element) {
                    logger.debug("message is of type "
                                + message.getClass().getName());
                    logger.debug("message contents: \n"
                                + XmlUtils.toString((Element) message));
                }
            }

            try {
                StateChangeNotificationMessageType changeNotification =
                    (StateChangeNotificationMessageType)ObjectDeserializer
                    .toObject(
                        (Element) message,
                        StateChangeNotificationMessageType.class);

                //state
                StateEnumeration newState = changeNotification.getState();
                assertTrue(newState != null);
                this.state = newState;
                this.receivedStates.add(newState);
                logger.debug("Notification: job state is " + state.toString());
                if (state.equals(StateEnumeration.Failed)) {
                    setFault(getFaultFromRP(changeNotification.getFault()));
                }

                //exit code
                if (state.equals(StateEnumeration.Done) ||
                    state.equals(StateEnumeration.Failed)) {
                    exitCode = changeNotification.getExitCode();
                    logger.debug("Notification: job exit code is "
                            + Integer.toString(exitCode));
                }

                //holding flag
                boolean tempHolding = changeNotification.isHolding();
                if (!this.holding) {
                    this.holding = tempHolding;
                }
                logger.debug("Is the job holding: " + this.holding);

            } catch (Exception e) {
                String errorMessage = "Notification message processing FAILED:"+
                    "Could not get value or set new status.";
                logger.error(errorMessage, e);
                throw new RuntimeException(errorMessage, e);
            }
        }

    } //end class

    private BaseFaultType getFaultFromRP(FaultResourcePropertyType fault)
    {
        if (fault == null)
        {
            return null;
        }

        if (fault.getCredentialSerializationFault() != null) {
            return fault.getCredentialSerializationFault();
        } else if (fault.getExecutionFailedFault() != null) {
            return fault.getExecutionFailedFault();
        } else if (fault.getFilePermissionsFault() != null) {
            return fault.getFilePermissionsFault();
        } else if (fault.getInsufficientCredentialsFault() != null) {
            return fault.getInsufficientCredentialsFault();
        } else if (fault.getInternalFault() != null) {
            return fault.getInternalFault();
        } else if (fault.getInvalidCredentialsFault() != null) {
            return fault.getInvalidCredentialsFault();
        } else if (fault.getInvalidPathFault() != null) {
            return fault.getInvalidPathFault();
        } else if (fault.getServiceLevelAgreementFault() != null) {
            return fault.getServiceLevelAgreementFault();
        } else if (fault.getStagingFault() != null) {
            return fault.getStagingFault();
        } else if (fault.getUnsupportedFeatureFault() != null) {
            return fault.getUnsupportedFeatureFault();
        } else {
            return null;
        }
    }


    protected ManagedJobFactoryPortType getSecureFactory(String factoryType)
        throws Exception
    {
        URL factoryURL =
            new URL(TEST_CONTAINER.getBaseURL() +
                    ManagedJobFactoryConstants.SERVICE_NAME);
        logger.debug("using factory URL: " + factoryURL.toExternalForm());
        ManagedJobFactoryPortType factoryPort =
            ManagedJobFactoryClientHelper.getPort(factoryURL, factoryType);
        this.jobFactoryStubSecurity.setStubPropertiesForDefaultOperation(
            (Stub)factoryPort);
        return factoryPort;
    }

    protected ManagedJobPortType getSecureJobPort()
        throws Exception
    {
        return getSecureJobPort(this.jobEndpoint);
    }

    private ManagedJobPortType getSecureJobPort(EndpointReferenceType epr)
        throws Exception
    {
        ManagedJobPortType jobPort = ManagedJobClientHelper.getPort(epr);
        this.jobStubSecurity.setStubPropertiesForDefaultOperation(
            (Stub)jobPort);
        return jobPort;
    }

    protected String getFactoryType() {
        return factoryType;
    }

    protected void setFactoryType(String newFactoryType) {
        this.factoryType = newFactoryType;
    }

    private String factoryType =
        ManagedJobFactoryConstants.DEFAULT_FACTORY_TYPE;

    protected Calendar getDefaultTerminationTime() {
        Calendar initTermTime = Calendar.getInstance();
        initTermTime.add(Calendar.SECOND, 120);
        return initTermTime;
    }

    protected CreateManagedJobOutputType submitJob() throws Exception {
        return submitJob(this.getDefaultTerminationTime());
    }

    protected CreateManagedJobOutputType submitJob(
        Calendar initTermTime)
        throws Exception
    {
        return submitJob(
            initTermTime,
            this.getDefaultJobDescription());
    }


    protected CreateManagedJobOutputType submitJob(
        Calendar initTermTime,
        JobDescriptionType jobDescription)
        throws Exception
    {
        return submitJob(
            initTermTime,
            jobDescription,
            (Subscribe) null);
    }

    protected CreateManagedJobOutputType submitJob(
        Calendar                                initTermTime,
        JobDescriptionType                      jobDescription,
        Subscribe                               subscriptionRequest)
        throws Exception
    {
        return submitJob(initTermTime, jobDescription, null, subscriptionRequest);
    }

    protected CreateManagedJobOutputType submitJob(
        Calendar initTermTime,
        JobDescriptionType jobDescription,
        AttributedURI jobID)
        throws Exception
    {
        return submitJob(initTermTime, jobDescription, jobID, null);
    }

    protected CreateManagedJobOutputType submitJob(
        Calendar                                initTermTime,
        JobDescriptionType                      jobDescription,
        AttributedURI                           jobID,
        Subscribe                               subscriptionRequest)
        throws                                  Exception
    {
        logger.debug("submitting job");
        ManagedJobFactoryPortType factoryPort =
            getSecureFactory(getFactoryType());
        CreateManagedJobInputType jobCreationInput =
            new CreateManagedJobInputType();
        jobDescription.setJobCredentialEndpoint(
            this.getCredentialEndpoint());
        jobDescription.setStagingCredentialEndpoint(
            this.getCredentialEndpoint());
        if (jobDescription instanceof MultiJobDescriptionType) {
            jobCreationInput.setMultiJob(
                (MultiJobDescriptionType)jobDescription);
        } else {
            jobCreationInput.setJob(jobDescription);
        }
        if (jobID == null) {
            UUIDGen uuidgen = UUIDGenFactory.getUUIDGen();
            jobID = new AttributedURI("uuid:" +
                                      uuidgen.nextUUID());
        }
        jobCreationInput.setJobID(jobID);
        jobCreationInput.setInitialTerminationTime(initTermTime);
        if (subscriptionRequest != null) {
            jobCreationInput.setSubscribe(subscriptionRequest);
        }

        CreateManagedJobOutputType response
            = factoryPort.createManagedJob(jobCreationInput);

        this.jobEndpoint = response.getManagedJobEndpoint();
        this.subscriptionEndpoint = response.getSubscriptionEndpoint();

        if (logger.isDebugEnabled()) {
            Calendar terminationTime = response.getNewTerminationTime();
            Calendar serviceCurrentTime = response.getCurrentTime();
            if (terminationTime != null) {
                logger.debug(
                    "Termination time granted by the factory to the "
                    +"job resource: "
                    +terminationTime.getTime());
            } else {
                logger.debug(
                    "Termination time granted by the factory to the "
                    +"job resource is null");
            }
            logger.debug(
                "Current time seen by the factory service on creation: " +
                serviceCurrentTime.getTime());
        }
        return response;
    }

    private static final String GLOBUS_LOCATION
        = System.getProperty("GLOBUS_LOCATION");
    private static final String SHARE_PATH
        = GLOBUS_LOCATION + File.separator + "test" + File.separator
        + System.getProperty("package.name") + File.separator;
    private static final String SIMPLEST_JOB_RSL
        = SHARE_PATH + "simplest_job.xml";

    private EndpointReferenceType jobEndpoint;
    private NotificationConsumerManager notificationConsumerManager;
    private EndpointReferenceType notificationConsumerEPR;
    private EndpointReferenceType subscriptionEndpoint;

    private TestNotifyCallback callback;

    private static Log logger =
        LogFactory.getLog(JobSubmissionTest.class.getName());

    private static String queryString =
        "boolean(/*/*[namespace-uri()='http://docs.oasis-open.org/wsrf/2004/06/wsrf-WS-ResourceLifetime-1.2-draft-01.xsd' and local-name()='TerminationTime'])";

    private static EndpointReferenceType credentialEndpoint = null;


}
