/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.exec.service.test;


import java.io.File;
import java.net.URL;
import java.security.cert.X509Certificate;
import java.util.Calendar;
import java.util.LinkedList;
import java.util.List;
import java.util.Vector;

import javax.xml.namespace.QName;
import javax.xml.rpc.Stub;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.apache.axis.components.uuid.UUIDGen;
import org.apache.axis.components.uuid.UUIDGenFactory;
import org.apache.axis.message.MessageElement;
import org.apache.axis.message.addressing.AttributedURI;
import org.apache.axis.message.addressing.EndpointReferenceType;
import org.apache.axis.message.addressing.ReferencePropertiesType;
import org.apache.axis.types.URI;

import org.oasis.wsn.TopicExpressionType;
import org.oasis.wsrf.faults.BaseFaultType;
import org.oasis.wsrf.lifetime.Destroy;
import org.oasis.wsrf.lifetime.SetTerminationTime;
import org.oasis.wsrf.lifetime.SetTerminationTimeResponse;
import org.oasis.wsrf.properties.QueryExpressionType;
import org.oasis.wsrf.properties.GetMultipleResourceProperties_Element;
import org.oasis.wsrf.properties.GetMultipleResourcePropertiesResponse;
import org.oasis.wsrf.properties.GetResourcePropertyResponse;
import org.oasis.wsrf.properties.QueryResourceProperties_Element;
import org.oasis.wsrf.properties.QueryResourcePropertiesResponse;
import org.w3c.dom.Element;

import org.globus.delegation.DelegationConstants;
import org.globus.delegation.DelegationUtil;
import org.globus.exec.generated.CreateManagedJobInputType;
import org.globus.exec.generated.CreateManagedJobOutputType;
import org.globus.exec.generated.JobDescriptionType;
import org.globus.exec.generated.ManagedJobFactoryPortType;
import org.globus.exec.generated.ManagedJobPortType;
import org.globus.exec.generated.MultiJobDescriptionType;
import org.globus.exec.generated.ServiceLevelAgreementType;
import org.globus.exec.generated.StateChangeNotificationMessageType;
import org.globus.exec.generated.StateEnumeration;
import org.globus.exec.utils.ManagedJobConstants;
import org.globus.exec.utils.ManagedJobFactoryConstants;
import org.globus.exec.utils.service.ManagedJobHelper;
import org.globus.exec.utils.client.ManagedJobClientHelper;
import org.globus.exec.utils.client.ManagedJobFactoryClientHelper;
import org.globus.exec.utils.rsl.RSLHelper;
import org.globus.gsi.GlobusCredential;
import org.globus.wsrf.NotificationConsumerManager;
import org.globus.wsrf.NotifyCallback;
import org.globus.wsrf.WSNConstants;
import org.globus.wsrf.WSRFConstants;
import org.globus.wsrf.encoding.ObjectDeserializer;
import org.globus.wsrf.impl.security.authentication.Constants;
import org.globus.wsrf.impl.security.authorization.Authorization;
import org.globus.wsrf.impl.security.authorization.HostAuthorization;
import org.globus.wsrf.impl.security.descriptor.GSISecureMsgAuthMethod;
import org.globus.wsrf.impl.security.descriptor.ResourceSecurityDescriptor;
import org.globus.wsrf.test.GridTestCase;
import org.globus.wsrf.test.GridTestSuite;
import org.globus.wsrf.utils.AddressingUtils;
import org.globus.wsrf.utils.AnyHelper;
import org.globus.wsrf.utils.XmlUtils;


public class ManagedJobFactoryTest extends JobSubmissionTest
{
    private static Log logger =
        LogFactory.getLog(ManagedJobFactoryTest.class.getName());

    public ManagedJobFactoryTest(String name) {
        super(name);
    }

    public static void main(String [] args) {
        GridTestSuite testSuite = null;

        // run all
        if (testSuite == null) {
            testSuite = new GridTestSuite(
                ManagedJobFactoryTest.class.toString());
            testSuite.addTestSuite(ManagedJobFactoryTest.class);
        }

        junit.textui.TestRunner.run (testSuite);
    }

    /**
     * Create a job with null (indefinite) init term time,
     * and test initial termination time was set.
     * @throws Exception
     */
    public void testInitTermTime_Null()
        throws                              Exception
    {
        System.out.println("\nEntering testInitTermTime_Null()\n");

        Calendar initTermTime = null;

        CreateManagedJobOutputType response =
            submitJob(initTermTime);

        //==========compare input and output term time
        Calendar newTermTime = response.getNewTerminationTime();
        assertTrue(newTermTime == null);

        System.out.println("init term time\n");
        System.out.println("requested: " + "indefinite");
        System.out.println("scheduled: " + "indefinite");

        //=======now get TerminationTime RP and compare
        ManagedJobPortType port = getSecureJobPort();
        GetResourcePropertyResponse propResponse =
            port.getResourceProperty(WSRFConstants.TERMINATION_TIME);

        MessageElement [] any = propResponse.get_any();
        /*
        assertTrue(any == null);
        */
        System.out.println("TerminationTime Resource Property:\n");
        for (int i=0; i<any.length; i++) {
            Element rpElement = AnyHelper.toElement(any[i]);
            System.out.println(XmlUtils.toString(rpElement));
        }
        Object obj = ObjectDeserializer.toObject(any[0], Calendar.class);
        assertTrue(obj == null);
    }

    /**
     * Create a job with non-null init term time,
     * and test initial termination time was set.
     * @throws Exception
     */
    public void testInitTermTime_NonNull()
        throws                              Exception
    {
        System.out.println("\nEntering testInitTermTime_NonNull()\n");

        Calendar initTermTime = getDefaultTerminationTime();

        CreateManagedJobOutputType response
            = submitJob(initTermTime);

        //==========compare input and output term time
        Calendar newTermTime = response.getNewTerminationTime();
        assertTrue(newTermTime != null);
        System.out.println("init term time\n");
        System.out.println("requested: " + initTermTime.getTime());
        System.out.println("scheduled: " + newTermTime.getTime());
        assertTrue(newTermTime.getTime().equals(initTermTime.getTime()) ||
                   newTermTime.getTime().after(initTermTime.getTime()));

        //=======now get TerminationTime RP and compare too
        ManagedJobPortType port = getSecureJobPort();
        GetResourcePropertyResponse propResponse =
            port.getResourceProperty(WSRFConstants.TERMINATION_TIME);

        MessageElement [] any = propResponse.get_any();
        assertTrue(any != null);
        assertTrue(any.length > 0);
        Object obj = ObjectDeserializer.toObject(any[0], Calendar.class);
        assertTrue(obj instanceof Calendar);
        assertEquals(newTermTime.getTime(), ((Calendar)obj).getTime());
    }

    /**
     * Create a job with non-null init term time that's
     * before the current time, and test the call fails with a
     * fault.
     * This test should generate an error on the server side.
     * @throws Exception
     */
    public void testInitTermTime_Wrong()
        throws                              Exception
    {
        System.out.println("\nEntering testInitTermTime_Wrong()\n");

        Calendar initTermTime = Calendar.getInstance();
        initTermTime.set(Calendar.SECOND, initTermTime.get(Calendar.SECOND)-30);
        try {
            CreateManagedJobOutputType response
                = submitJob(initTermTime);
            fail();
        }
        catch (Exception e) {
            if (e instanceof BaseFaultType) {
                BaseFaultType baseFault = (BaseFaultType)e;
            }
            //TODO: test it is a wsrl:UnableToSetTerminationTimeFault.
            //Well, it seems it is only wrapped in a BaseFault...
        }

    }



    // get resource property 'LocalResourceManager' on Factory
    public void testResourceProperty_Factory_LocalResourceManager()
        throws                              Exception
    {
        System.out.println(
            "\nEntering testResourceProperty_Factory_LocalResourceManager()\n");

        ManagedJobFactoryPortType factoryPortType = getSecureFactory(
            ManagedJobFactoryConstants.FACTORY_TYPE.FORK);

        GetResourcePropertyResponse propResponse =
            factoryPortType.getResourceProperty(
                ManagedJobFactoryConstants.RP_LOCAL_RESOURCE_MANAGER);
        MessageElement [] any = propResponse.get_any();
        assertTrue(any != null);
        assertTrue(any.length > 0);
        String localResourceManager = (String)ObjectDeserializer.toObject(
                any[0],
                String.class);
        System.out.println("localResourceManager: " + localResourceManager);
        assertTrue(localResourceManager.equals(
            ManagedJobFactoryConstants.FACTORY_TYPE.FORK));

        /*  Array:
            GetResourcePropertyResponse propResponse =
                factoryPortType.getResourceProperty(ManagedJobFactoryProperties.
                        getTypeDesc().getFieldByName("template").getXmlName());
            MessageElement [] any = propResponse.get_any();
            assertTrue(any != null);
            //assertTrue(any.length > 0);
            for (int i=0; i<any.length; i++) {
                String template = (String)ObjectDeserializer.toObject(
                    any[i],
                    String.class);
                System.out.println("template " + i + ":\n" + template);
            }
        */
    }

    // get multiple resource properties from gram namespace
    public void testMultipleResourceProperties_Factory_GRAM()
        throws                              Exception
    {
        System.out.println(
            "\nEntering testMultipleResourceProperties_Factory_GRAM()\n");

        ManagedJobFactoryPortType factoryPort =
            getSecureFactory(getFactoryType());

        QName [] propNames =
            new QName[] {ManagedJobFactoryConstants.RP_LOCAL_RESOURCE_MANAGER,
                         ManagedJobFactoryConstants.RP_GLOBUS_LOCATION,
                         ManagedJobFactoryConstants.RP_CPU,
                         ManagedJobFactoryConstants.RP_MANUFACTURER,
                         ManagedJobFactoryConstants.RP_OS_NAME,
                         ManagedJobFactoryConstants.RP_OS_VERSION,
                         ManagedJobFactoryConstants.RP_CONDOR_ARCH,
                         ManagedJobFactoryConstants.RP_CONDOR_VERSION,
                         ManagedJobFactoryConstants.RP_SCRATCH_BASE_DIRECTORY
                     };

        //do test for optional RPs with specific test installation...?
        GetMultipleResourceProperties_Element requestProp =
            new GetMultipleResourceProperties_Element();
        requestProp.setResourceProperty(propNames);

        GetMultipleResourcePropertiesResponse responseProp =
            factoryPort.getMultipleResourceProperties(requestProp);

        MessageElement [] any = responseProp.get_any();
        assertTrue(any != null);

        System.out.println("Resources Properties:\n");
        for (int i=0; i<any.length; i++) {
            Element rpElement = AnyHelper.toElement(any[i]);
            System.out.println(XmlUtils.toString(rpElement));
            String value = (String)ObjectDeserializer.toObject(any[i],
                String.class);
            System.out.println("value of RP = " + value);

        }
        assertTrue(any[0] != null); //Local Resource Manager
        assertTrue(any[1] != null); //Globus Location
        //TODO add asserts for nullity when sure or not about presence
        //then add nillable to RPs or not

        assertTrue(any.length >= 2); //at least Globus Location and LRM
    }


    /**
     * Create a job, test the returned EPR.
     * @throws Exception
     */
    public void testJobEPR()
        throws                              Exception
    {
        System.out.println("\nEntering testJobEPR()\n");

        EndpointReferenceType jobEPR = submitJob(
            this.getDefaultTerminationTime(),
            this.getDefaultJobDescription()).getManagedJobEndpoint();
        //======= test EPR of job
        assertTrue(jobEPR != null);
        assertTrue(jobEPR.getAddress() != null);
        String address = jobEPR.getAddress().toString();
        System.out.println("job service URL:" + address);
        ReferencePropertiesType props = jobEPR.getProperties();
        assertTrue(props.get_any() != null);
        assertTrue(props.get_any().length > 0);
        for (int i=0; i<props.get_any().length; i++) {
            String referenceProperty =
                XmlUtils.toString(AnyHelper.toElement(props.get_any()[i]));
            System.out.println("reference property #" + i + "\n" +
                               referenceProperty);
        }
        assertTrue(props.get_any()[0] != null);
        //Reminder: client is not supposed to inspect the EPR contents!
        //we know it is a simple String value inside Text of XML key element.
        String keyValue = props.get_any()[0].getValue();
        System.out.println("job resource ID = " + keyValue);

    }

    public void testIdempotentCreate()
        throws                              Exception
    {
        System.out.println("\nEntering testIdempotentCreate()\n");

        UUIDGen uuidgen = UUIDGenFactory.getUUIDGen();
        AttributedURI jobID = new AttributedURI("uuid:" +
                                                uuidgen.nextUUID());
        CreateManagedJobOutputType reply =
            submitJob(null, this.getDefaultJobDescription(), jobID);
        EndpointReferenceType jobEPR = reply.getManagedJobEndpoint();
        reply = submitJob(null, this.getDefaultJobDescription(), jobID);
        assertEquals(ManagedJobHelper.getHandle(jobEPR),
                     ManagedJobHelper.getHandle(reply.getManagedJobEndpoint()));
    }
}
