/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.exec.service.test;


import java.io.StringReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.math.BigDecimal;
import java.net.MalformedURLException;
import java.net.URL;
import java.rmi.RemoteException;
import java.text.DateFormat;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Vector;
import java.util.List;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.Date;
import java.util.Arrays;

import javax.xml.rpc.ServiceException;
import javax.xml.rpc.Stub;
import javax.xml.soap.SOAPElement;
import javax.xml.namespace.QName;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.globus.axis.gsi.GSIConstants;

import org.apache.axis.message.addressing.AddressingHeaders;
import org.apache.axis.message.addressing.EndpointReferenceType;
import org.apache.axis.message.addressing.MessageID;
import org.apache.axis.types.URI.MalformedURIException;

import org.globus.exec.utils.client.ManagedJobFactoryClientHelper;
import org.globus.exec.utils.client.ManagedJobClientHelper;
import org.globus.exec.generated.CreateManagedJobInputType;
import org.globus.exec.generated.CreateManagedJobOutputType;
import org.globus.exec.generated.FaultType;
import org.globus.exec.generated.FilePairType;
import org.globus.exec.generated.JobDescriptionType;
import org.globus.exec.generated.ManagedJobFactoryPortType;
import org.globus.exec.generated.ManagedJobPortType;
import org.globus.exec.generated.StateChangeNotificationMessageType;
import org.globus.exec.generated.StateEnumeration;
import org.globus.exec.generated.StateEnumeration;
import org.globus.exec.utils.ManagedJobConstants;
import org.globus.exec.utils.ManagedJobFactoryConstants;
import org.globus.exec.utils.rsl.RSLHelper;
import org.globus.exec.utils.rsl.RSLParseException;
import org.globus.exec.utils.JobClientSecurityStrategy;

import org.globus.security.gridmap.GridMap;

import org.globus.wsrf.WSRFConstants;
import org.globus.wsrf.NotifyCallback;
import org.globus.wsrf.NotificationConsumerManager;
import org.globus.wsrf.NoSuchResourceException;
import org.globus.wsrf.encoding.ObjectDeserializer;
import org.globus.wsrf.impl.security.authentication.Constants;
import org.globus.wsrf.impl.security.authorization.Authorization;
import org.globus.wsrf.impl.security.authorization.HostAuthorization;
import org.globus.wsrf.impl.security.authorization.IdentityAuthorization;
import org.globus.wsrf.impl.security.authorization.SelfAuthorization;
import org.globus.wsrf.impl.security.descriptor.GSISecureMsgAuthMethod;
import org.globus.wsrf.impl.security.descriptor.ResourceSecurityDescriptor;
import org.globus.wsrf.utils.XmlUtils;

//import org.globus.resolver.ResolverUtils;

import org.ietf.jgss.GSSCredential;
import org.ietf.jgss.GSSException;

import org.oasis.wsn.SubscriptionManager;
import org.oasis.wsn.TopicExpressionType;
import org.oasis.wsn.WSBaseNotificationServiceAddressingLocator;
import org.oasis.wsrf.faults.BaseFaultType;
import org.oasis.wsrf.lifetime.SetTerminationTime;
import org.oasis.wsrf.lifetime.SetTerminationTimeResponse;
import org.oasis.wsrf.lifetime.ResourceUnknownFaultType;
import org.oasis.wsrf.properties.ResourcePropertyValueChangeNotificationType;

import org.w3c.dom.Element;
import org.w3c.dom.Text;

import java.util.List;

import java.rmi.RemoteException;

import javax.xml.namespace.QName;

// To be able to refer to Rendezvous services
import org.globus.wsrf.utils.AddressingUtils;
import org.globus.wsrf.encoding.ObjectDeserializer;

import org.globus.rendezvous.client.RendezvousConstants;
import org.globus.rendezvous.client.RendezvousHelper;
import org.globus.rendezvous.client.ClientSecurityStrategy;
import org.globus.rendezvous.client.CompletionNotifyCallbackImpl;
import org.globus.rendezvous.generated.RendezvousPortType;
import org.globus.rendezvous.generated.service.RendezvousServiceAddressingLocator;
import org.globus.rendezvous.generated.RegisterInput;
import org.globus.rendezvous.generated.RegisterResponse;
import org.globus.rendezvous.generated.RendezvousResourceProperties;
import org.globus.rendezvous.generated.CapacityReachedFaultType;
import org.globus.rendezvous.generated.RankTakenFaultType;

//For being a resource that accepts notifications
import org.globus.wsrf.NotifyCallback;
import org.globus.wsrf.WSRFConstants;
import org.globus.wsrf.NotificationConsumerManager;
import org.globus.wsrf.core.notification.ResourcePropertyValueChangeNotificationElementType;
import org.oasis.wsn.TopicExpressionType;
import org.oasis.wsrf.properties.ResourcePropertyValueChangeNotificationType;

// For being a resource that supports notifications
import org.globus.wsrf.ResourceProperty;
import org.globus.wsrf.ResourceException;
import org.globus.wsrf.ResourceProperties;
import org.globus.wsrf.ResourcePropertySet;
import org.globus.wsrf.Topic;
import org.globus.wsrf.TopicList;
import org.globus.wsrf.TopicListAccessor;

import org.globus.wsrf.impl.ReflectionResource;
import org.globus.wsrf.impl.ReflectionResourceProperty;
import org.globus.wsrf.impl.ResourcePropertyTopic;
import org.globus.wsrf.impl.SimpleResourcePropertySet;
import org.globus.wsrf.impl.SimpleTopicList;
import org.globus.wsrf.impl.SimpleTopic;
import org.globus.wsrf.impl.SimpleResourceProperty;


import org.globus.util.I18n;

import org.apache.axis.message.addressing.EndpointReferenceType;

// Schema types
import java.math.BigInteger;
import org.apache.axis.types.NonNegativeInteger;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


public class ProcessRendezvousJob {

    public ProcessRendezvousJob(EndpointReferenceType jobEndpoint,
                                EndpointReferenceType multijobEndpoint,
                                int subjobRank) {
        this.jobEndpoint = jobEndpoint;
        this.multijobEndpoint = multijobEndpoint;
        JobClientSecurityStrategy clientSecurity =
            new JobClientSecurityStrategy();
        //default authz is Host.
        this.invocationHelper = new RendezvousHelper(clientSecurity);
        this.subjobRank = subjobRank;
    }


    public ProcessRendezvousJob(EndpointReferenceType jobEndpoint,
                                EndpointReferenceType multijobEndpoint) {
        this(jobEndpoint, multijobEndpoint, RendezvousHelper.NO_DESIRED_RANK);
    }

    public class SystemProperties {
        public static final String TEST_JOB_HANDLE =
            "org.globus.exec.test.job.handle";
        public static final String TEST_MULTIJOB_HANDLE =
            "org.globus.exec.test.multijob.handle";
        public static final String TEST_SUBJOB_RANK =
            "org.globus.exec.test.subjob.rank";
        public static final String TEST_EXPECTED_SUBJOB_RANK =
            "org.globus.exec.test.subjob.rank.should.be";
    }

    public static void main(String[] args) throws Exception {
        System.out.println("=================================================");
        System.out.println("Test rendezvous job process started");

        //get handles to this subjob, and also to multijob if any
        String subJobHandle = System.getProperty(
            SystemProperties.TEST_JOB_HANDLE);
        System.out.println("job handle = " + subJobHandle);
        String multiJobHandle = System.getProperty(
            SystemProperties.TEST_MULTIJOB_HANDLE);
        System.out.println("multi-job handle = " +
                           multiJobHandle);

        EndpointReferenceType jobEndpoint =
            ManagedJobClientHelper.getEndpoint(subJobHandle);
            //ResolverUtils.resolve(subjobResolverHandle);
        EndpointReferenceType multiJobEndpoint = null;
        if (multiJobHandle != null &&
            !multiJobHandle.equals("")) {
            multiJobEndpoint =
                ManagedJobClientHelper.getEndpoint(multiJobHandle);
                //ResolverUtils.resolve(multijobResolverHandle);
        }

        String subjobRankString = System.getProperty(
            SystemProperties.TEST_SUBJOB_RANK);
        if (subjobRankString != null && !subjobRankString.equals("")) {
            int subjobRank = Integer.parseInt(subjobRankString);
            System.out.println("sub-job rank = " + subjobRank);
            int subjobRankShouldBe = Integer.parseInt(System.getProperty(
                SystemProperties.TEST_EXPECTED_SUBJOB_RANK));
            if (subjobRank != subjobRankShouldBe) {
                System.out.println("sub-job rank should be = " +
                                   subjobRankShouldBe);
                throw new RuntimeException("subjob rank seems wrong");
            }
            new ProcessRendezvousJob(jobEndpoint, multiJobEndpoint, subjobRank).
                doRendezvous();
        }
        else {
            new ProcessRendezvousJob(jobEndpoint, multiJobEndpoint).
                doRendezvous();
        }

        System.out.println("Test rendezvous job process finished");
        System.out.println("=================================================");
        System.out.println();
    }

    private void doRendezvous() throws Exception {

        //subscribe for notifications for when rendezvous is complete
        this.invocationHelper.subscribeForCompletionNotifications(
            this.jobEndpoint,
            new SubJobRendezvousCallback());

        //register process contact data
        int rank = this.invocationHelper.registerWithRendezvous(
            this.jobEndpoint,
            INPUT_DATA);

        System.out.println(
            "Registered application data with job rendezvous."+
            " Returned rank is: " + rank);


        waitForSubJobRendezvousToComplete();

        if (this.multijobEndpoint != null) {
            waitForMultiJobRendezvousToComplete();
        }
    }

    private void waitForSubJobRendezvousToComplete() throws Exception {
        int timeout = 10000;

        if (!this.subjobRendezvousComplete) {
            //Let the notification come in
            System.out.println("Waiting for top-level notification...");
            Thread.sleep(timeout);
        }

        System.out.println("Is sub-job level rendezvous done? " +
                           this.subjobRendezvousComplete);
        if (!this.subjobRendezvousComplete) {
            //let's pull
            boolean subDone =
                this.invocationHelper.isRendezvousDone(this.jobEndpoint);
            if (!subDone) {
                throw new RuntimeException(
                    "Test failed: sub-job rendezvous has not completed after " +
                    timeout + " milliseconds of waiting time.");
            }
        }
    }


    private void waitForMultiJobRendezvousToComplete() throws Exception {
        int timeout = 20000;
        if (!isDataReceived()) {
            //Let the notification come in
            System.out.println("Waiting for top-level notification...");
            Thread.sleep(timeout);
        }

        boolean dataReceived = isDataReceived();
        System.out.println("Has data for the whole rendezvous been obtained? "+
                           dataReceived);
        if (!dataReceived) {
            //let's pull
            boolean allDone =
                this.invocationHelper.isRendezvousDone(this.multijobEndpoint);
            if (!allDone) {
                throw new RuntimeException(
                    "Test failed: global rendezvous has not completed after " +
                    timeout + " milliseconds of waiting time.");
            }
        }
    }

    private synchronized boolean isDataReceived() {
        return (this.wholeData != null);
    }

    private class MultiJobRendezvousCallback
        extends CompletionNotifyCallbackImpl
    {
        public MultiJobRendezvousCallback() {
            super(new JobClientSecurityStrategy());
        }

        public void onRendezvousCompleted(byte[] bytes) {
            synchronized(System.out) {
                System.out.println(
                    "Notification: Multijob-level rendezvous completed");

                wholeData = bytes;

                //readMultiJob(bytes);
                System.out.println(
            "Doing stuff with the contact data after all the processes have rendezvous-ed");
            }
        }

    } //end MultiJobRendezvousCallback

    private class SubJobRendezvousCallback extends CompletionNotifyCallbackImpl
    {
        public SubJobRendezvousCallback() {
            super(new JobClientSecurityStrategy());
        }

        public void onRendezvousCompleted(byte[] dataSet) {
            synchronized(System.out) {
                System.out.println(
                    "Notification: sub-job level rendezvous completed");
                subjobRendezvousComplete = true;

                if (multijobEndpoint != null) {
                    //subscribe for notifications for when rendezvous is complete
                    try {
                        invocationHelper.subscribeForCompletionNotifications(
                            multijobEndpoint,
                            new MultiJobRendezvousCallback());
                        //note: in realistic scenario only process 0 needs
                        //whole contact data
                        //TODO: do this only if this is process 0
                    }
                    catch (Exception e) {
                        String errorMessage = "Could not subscribe with " +
                            "multijob for rendezvous notifications";
                        throw new RuntimeException(errorMessage, e);
                    }

                    //now register the entire data set for the sub
                    boolean alreadyRegistered = false;
                    try {
                        int rank = invocationHelper.registerWithRendezvous(
                            multijobEndpoint,
                            dataSet,
                            subjobRank);
                        if (rank != -1) {
                            System.out.println(
                                "Registered subjob data with multijob " +
                                "rendezvous.  Returned rank is: " + rank);
                        }
                        else {
                            System.out.println(
                                "multijob rendezvous was already complete");
                            //This will happen for any process after the first
                            //one. In a real application we would have only
                            //process 0 do this registration...
                        }
                    }
                    catch (RankTakenFaultType e) {
                        alreadyRegistered = true;
                    }
                    catch (CapacityReachedFaultType e) {
                        alreadyRegistered = true;
                    }
                    catch (Exception e) {
                        String errorMessage =
                            "An error occurred when registering the subjob";
                        System.err.println(errorMessage);
                        throw new RuntimeException(errorMessage, e);
                    }
                    if (alreadyRegistered) {
                        String errorMessage =
                        "Could not register whole subjob data with multijob. " +
                        "This is probably because another process has already "+
                        "done it, so it is not a test failure";
                        System.out.println(errorMessage);
                    }

                } //end if
            } //end synchronized
        } //end function

    } //end SubJobRendezvousCallback

    private EndpointReferenceType multijobEndpoint;
    private EndpointReferenceType jobEndpoint;
    private int subjobRank = RendezvousHelper.NO_DESIRED_RANK;

    private static final byte[] APPLICATION_BINARY_DATA = {01, 02, 03};
    private static final byte[] INPUT_DATA = RendezvousHelper.prependHeader(
        APPLICATION_BINARY_DATA, APPLICATION_BINARY_DATA.length);

    private byte[] wholeData;
    private boolean subjobRendezvousComplete;

    private RendezvousHelper invocationHelper;
}
