/*
* Portions of this file Copyright 1999-2005 University of Chicago
* Portions of this file Copyright 1999-2005 The University of Southern California.
*
* This file or a portion of this file is licensed under the
* terms of the Globus Toolkit Public License, found at
* http://www.globus.org/toolkit/download/license.html.
* If you redistribute this file, with or without
* modifications, you must include this notice in the file.
*/

package org.globus.mds.aggregator.impl;

/**
This is a bean used to pass in configuration for the aggregator framework
through JNDI.
*/
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.StringTokenizer;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class AggregatorConfiguration
{

    private static Log logger =
            LogFactory.getLog(AggregatorConfiguration.class.getName());
    
    public AggregatorConfiguration() { }

    private long entrySweeperInterval = 
        AggregatorServiceGroupResource.DEFAULT_SWEEP_PERIOD;
    
    private boolean notifyOnEntryLifetimeExtension = true;
    private boolean notifyOnEntryContentChange = false;
     
    private String entryRemovedCallbackClassName;    
    private String aggregatorSources;
    private String executableMappings;
    
    private ArrayList sources = new ArrayList();
    private Hashtable executables = new Hashtable();
    private AggregatorServiceGroupEntryRemovedCallback removeCallback = null;   
    
    public String getAggregatorSources() {
        return aggregatorSources;
    }

    public String getExecutableMappings() {
        return executableMappings;
    }
    
    public ArrayList getAggregatorSourceList() {
        return sources;
    }
    
    public Hashtable getExecutableMappingTable() {
        return executables;
    }
    
    public void setAggregatorSources(String s) 
    {
        StringTokenizer toker = new StringTokenizer(s);
        while (toker.hasMoreTokens()) {
            String source = toker.nextToken();
            sources.add(source);
        }
    }
    
    public void setExecutableMappings(String s) 
    {
        String entry;
        String name; 
        String value;
            
        StringTokenizer toker = new StringTokenizer(s);
        while (toker.hasMoreTokens()) 
        {
            entry = toker.nextToken(",");            
            entry.trim();
            name = entry.substring(0,entry.lastIndexOf("="));
            value = entry.substring(entry.lastIndexOf("=")+1);
            executables.put(name.trim(), value.trim());
        }
    }
    
    public String getEntryRemovedCallback() 
    {
        return this.entryRemovedCallbackClassName;
    }
    
    public void setEntryRemovedCallback(String className) 
    {
        if (className.length()==0) {
            return;
        }
        
        Class c = null;
        Object callback = null;
        try {
            c = Class.forName(className);
            callback = c.newInstance();
        } catch(Exception e) {
            logger.warn("Unable to create an instance of " 
                        + className + ": " + e);
        }
        if (callback instanceof AggregatorServiceGroupEntryRemovedCallback) {
            this.removeCallback = 
                (AggregatorServiceGroupEntryRemovedCallback) callback;
        }
    }
    public AggregatorServiceGroupEntryRemovedCallback getEntryRemovedCallbackInstance() 
    {
        return this.removeCallback;
    }    

    public long getEntrySweeperInterval() {
        return this.entrySweeperInterval;        
    }
    
    public void setEntrySweeperInterval(long val) {
        this.entrySweeperInterval = val;        
    }
    
    public boolean getNotifyOnEntryLifetimeExtensionValue() {
        return notifyOnEntryLifetimeExtension;        
    }
    
    public boolean getNotifyOnEntryContentChangeValue() {
        return notifyOnEntryContentChange;        
    }
    
    public String getNotifyOnEntryLifetimeExtension() {
        String ret = new Boolean(this.notifyOnEntryLifetimeExtension).toString();
        return ret;               
    }
    
    public String getNotifyOnEntryContentChange() {
        String ret = new Boolean(this.notifyOnEntryContentChange).toString();
        return ret;                  
    }
    
    public void setNotifyOnEntryLifetimeExtension(String bool) {
        this.notifyOnEntryLifetimeExtension = 
            Boolean.valueOf(bool.trim()).booleanValue();        
    }
    
    public void setNotifyOnEntryContentChange(String bool) {
        this.notifyOnEntryContentChange = 
            Boolean.valueOf(bool.trim()).booleanValue();
    }    
    
}
