/*
* Portions of this file Copyright 1999-2005 University of Chicago
* Portions of this file Copyright 1999-2005 The University of Southern California.
*
* This file or a portion of this file is licensed under the
* terms of the Globus Toolkit Public License, found at
* http://www.globus.org/toolkit/download/license.html.
* If you redistribute this file, with or without
* modifications, you must include this notice in the file.
*/

package org.globus.mds.aggregator.impl;

import org.globus.mds.aggregator.types.PairedKeyType;

import javax.xml.namespace.QName;
import javax.naming.Context;
import javax.naming.InitialContext;

import org.apache.axis.MessageContext;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.globus.wsrf.ResourceHome;
import org.globus.wsrf.ResourceKey;
import org.globus.wsrf.Resource;
import org.globus.wsrf.Constants;
import org.globus.wsrf.ResourceException;
import org.globus.wsrf.NoResourceHomeException;
import org.globus.wsrf.InvalidResourceKeyException;
import org.globus.wsrf.NoSuchResourceException;

import org.globus.wsrf.container.Lock;
import org.globus.wsrf.container.LockManager;

import org.globus.wsrf.impl.SimpleResourceKey;
import org.globus.wsrf.impl.ResourceHomeImpl;

import org.globus.wsrf.impl.servicegroup.ServiceGroupConstants;

import org.globus.util.I18n;
import org.globus.wsrf.utils.Resources;

/** Looks up a service group entry. A key for this service consists of the
    key to the containing service group and then a unique-in-servicegroup
    key. <br>
*/

public class AggregatorServiceGroupEntryHome extends ResourceHomeImpl 
{
    private static Log logger =
            LogFactory.getLog(AggregatorServiceGroupEntryHome.class.getName());
    
    private static I18n i18n = I18n.getI18n(Resources.class.getName());
    private ResourceHome groupHome = null;
    
    public AggregatorServiceGroupEntryHome() 
    {
        super();        
        
        try {
            this.setResourceClass(AggregatorServiceGroupEntryResource.class.getName());            
            this.setResourceKeyType(PairedKeyType.class.getName());
            this.setResourceKeyName(ServiceGroupConstants.ENTRY_KEY.toString()); 
            this.initialize();
        }  catch (Exception e) {
            logger.error("Exception during object creation: " + e);
        }                
    }
   
    public synchronized void initialize() 
        throws Exception 
    {
        super.initialize();
        
        // get the servicegroup home            
        /*this.groupHome= getGroupHome(); 
        if (this.groupHome == null) {
            throw new NoResourceHomeException();
        }  */       
    }
    
    public Resource find(ResourceKey key) 
        throws ResourceException, 
               InvalidResourceKeyException
    {
        if (key == null) {
            throw new InvalidResourceKeyException(
                i18n.getMessage("nullArgument", "key"));
        }

        Resource resource = null;
        AggregatorServiceGroupResource groupResource = null;
        SimpleResourceKey k = null;
        PairedKeyType pk = null;
        
        try {
            // extract ServiceGroup key from the composite key
            k = (SimpleResourceKey)key;
            pk = (PairedKeyType)k.getValue();

            ResourceKey groupKey = 
                new SimpleResourceKey(ServiceGroupConstants.KEY, pk.getGroupKey());  
            
            // get the servicegroup home and cache                 
            if (this.groupHome == null) {
                this.groupHome= getGroupHome(); 
            }
            // if we somehow made it here and groupHome is still null (unlikely),
            // throw an exception
            if (this.groupHome == null) {
                 throw new NoResourceHomeException();
            }                                        
            // special case if we use a singleton aggregator home
            if(this.groupHome instanceof SingletonAggregatorHome) {
                groupKey = null;
            }
            
            // retrieve servicegroup resource
            groupResource = 
                (AggregatorServiceGroupResource) groupHome.find(groupKey);
            
        } catch (Exception e) {
            logger.error("Exception looking up an entry key", e); 
            throw new ResourceException(e);
        }
            
        Lock lock = this.lockManager.getLock(key);
        try {
            lock.acquire();
        } catch (InterruptedException e) {
            throw new ResourceException(e);
        }
        
        try {
            // look up full key in servicegroup resource
            // group should never be null but test for NPR just in case
            if (groupResource != null) { 
                resource = (Resource)groupResource.entries.get(k);
                if (logger.isDebugEnabled()){
                    logger.debug("AggregatorServiceGroupEntryResource found: "+ resource);                
                }
            }            
        } catch(Exception e) {
            logger.error("Exception getting an entry key", e);
            throw new ResourceException(e);
        } finally {
            lock.release();
        }
        if (resource == null) {
            throw new NoSuchResourceException();
        }
        return resource;
    }
    
     public void remove(ResourceKey key) {
         if (logger.isDebugEnabled()) {
             logger.debug("Remove method called with key: " + key.toString());
         }
     }
     
    public ResourceHome getGroupHome()
        throws Exception {

        MessageContext ctx = MessageContext.getCurrentContext();

        String homeLocation = getGroupHomeLocation(ctx);
        Context initialContext = new InitialContext();

        return (ResourceHome) initialContext.lookup(homeLocation);
    }

    public String getGroupHomeLocation(MessageContext ctx)
        throws Exception {
        if (ctx == null) {
            throw new IllegalArgumentException("MessageContext ctx == null");
        }
        if (!(ctx instanceof MessageContext)) {
            throw new Exception("ctx must be Axis instance");
        }

        org.apache.axis.MessageContext context =
            (org.apache.axis.MessageContext)ctx;

        String servicePath = context.getTargetService();
        return Constants.JNDI_SERVICES_BASE_NAME + servicePath +
               "/groupHome";
    }    
}

