/*
* Portions of this file Copyright 1999-2005 University of Chicago
* Portions of this file Copyright 1999-2005 The University of Southern California.
*
* This file or a portion of this file is licensed under the
* terms of the Globus Toolkit Public License, found at
* http://www.globus.org/toolkit/download/license.html.
* If you redistribute this file, with or without
* modifications, you must include this notice in the file.
*/

/*
 * AggregatorSink.java
 */

package org.globus.mds.aggregator.impl;

import org.apache.axis.encoding.AnyContentType;

/**
 *  This interface must be implemented by a plugin for the AggregatorSource
 *  component of the aggregator and is generally type-specific to the message 
 *  data being processed.  It is a required parameter for any AggregatorSource.
 *  Also, for every AggregatorSource instance there can be only one connected 
 *  AggregatorSink instance.    
 */
public interface AggregatorSink {
    /** 
        Called to initialize any necessary state. This method should throw an 
        exception if the host AggregatorSource is not set. The underlying 
        implementation may also throw an exception during initialization if it 
        is not compatible with the host AggregationSource.  
        The implementation may use the optional parameters Object to read 
        implementation specific data or can just ignore processing it 
	altogether. 
     
        TODO: add custom exceptions here like SourceNotSet and 
	SourceTypeNotSupported
    */
    public void initialize(Object parameters) 
        throws AggregatorException, Exception;      
    
    /** 
        Called to free resources used by the sink.
    */
    public void terminate();
    
    /** 
        Gets the AggregatorSource connected to this sink
    */
    public AggregatorSource getSource(); 
    
    /** 
        Sets the AggregatorSource connected to this sink
    */
    public void setSource(AggregatorSource source);
    
    /** 
        Should be called by an AggregatorSource to deliver a message with 
	the assumption that the message data will be aggregated as its 
	native type and into whatever data storage format is 
        used by the underlying AggregatorSink class.  The context parameter 
	is the AggregatorServiceGroupEntryResource context parameter passed to 
        the hosting AggregationSource when the AggregationSource.addAggregation 
        method is invoked.

    */
    public void deliver(AnyContentType message,
                        AggregatorServiceGroupEntryResource context) 
        throws AggregatorException, Exception;
    
}
