/*
* Portions of this file Copyright 1999-2005 University of Chicago
* Portions of this file Copyright 1999-2005 The University of Southern California.
*
* This file or a portion of this file is licensed under the
* terms of the Globus Toolkit Public License, found at
* http://www.globus.org/toolkit/download/license.html.
* If you redistribute this file, with or without
* modifications, you must include this notice in the file.
*/
package org.globus.mds.aggregator.impl;

import javax.xml.namespace.QName;
import java.net.UnknownHostException;

import org.apache.axis.message.addressing.Address;
import org.apache.axis.message.addressing.EndpointReferenceType;
import org.apache.axis.message.addressing.ReferencePropertiesType;


import org.globus.wsrf.utils.AddressingUtils;
import org.globus.wsrf.impl.servicegroup.client.ServiceGroupRegistrationClient;
import org.globus.wsrf.config.ContainerConfig;
import org.globus.wsrf.container.ServiceHost;
import org.globus.mds.servicegroup.client.ServiceGroupRegistrationParameters;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.io.InputStream;
import java.io.FileInputStream;
import java.net.InetAddress;

public class AggregatorUtils 
{
    private static Log logger =
                LogFactory.getLog(AggregatorUtils.class.getName());
    
    // try to detect an EPR loopback condition
    // not perfect, but OK for now
    public static boolean detectLoopback(EndpointReferenceType remoteEPR,
                                         EndpointReferenceType localEPR) 
    {
        boolean loopbackDetected = false;
        try {
            // first try the basic string-based comparison to fail fast
            if (remoteEPR.getAddress().equals(localEPR.getAddress())) {
                loopbackDetected = true;                                     
            }
            else
            {
                InetAddress remoteAddr = null;
                InetAddress localAddr = null;
                // check to make sure we dont have two 'local' (or equivalent) 
                // host addresses - note this will force DNS name resolution 
                // to be used
                try {                 
                    remoteAddr = 
                        InetAddress.getByName(remoteEPR.getAddress().getHost());  

                    localAddr = 
                        InetAddress.getByName(localEPR.getAddress().getHost());

                } catch (UnknownHostException e) {
                    logger.warn("UnknownHostException: " + e.getMessage());
                    return false;
                }
                
                // check to see if the addresses resolved to the same IP address
                // or have special loopback characteristics
                // note - depends on JDK 1.4 or greater
                if (remoteAddr.isLoopbackAddress() || 
                    (remoteAddr.getHostAddress().equalsIgnoreCase(
                    localAddr.getHostAddress())))
                {
                    // if the entries are now suspect as local addresses, 
                    // then if the following matches, its enough to generate a warning
                    if ((remoteEPR.getAddress().getScheme().equalsIgnoreCase(
                        localEPR.getAddress().getScheme())) &&
                        (remoteEPR.getAddress().getPort()==
                         localEPR.getAddress().getPort()) &&
                         (remoteEPR.getAddress().getPath().equalsIgnoreCase(
                            localEPR.getAddress().getPath()))) 
                    {
                         loopbackDetected = true;     
                    }
                }
            }
        } catch (Exception e) {
            logger.warn("Unexpected Exception: ",e);
        }
        return loopbackDetected;
    }  

}
