/*
* Portions of this file Copyright 1999-2005 University of Chicago
* Portions of this file Copyright 1999-2005 The University of Southern California.
*
* This file or a portion of this file is licensed under the
* terms of the Globus Toolkit Public License, found at
* http://www.globus.org/toolkit/download/license.html.
* If you redistribute this file, with or without
* modifications, you must include this notice in the file.
*/

/*
 * ServiceGroupEntryAggregatorSink.java
 */

package org.globus.mds.aggregator.impl;

import org.globus.mds.aggregator.types.AggregatorContent;
import org.globus.mds.aggregator.types.AggregatorConfig;
import org.globus.mds.aggregator.types.AggregatorData;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.globus.wsrf.utils.AnyHelper;

import org.apache.axis.utils.XMLUtils;
import org.apache.axis.message.MessageElement;
import org.apache.axis.encoding.AnyContentType;

import org.w3c.dom.Element;
import java.util.Iterator;

/**
 *  The ServiceGroupEntryAggregatorSink stores aggregate data as 
 *  ServiceGroupEntry content.
 */
public class ServiceGroupEntryAggregatorSink implements AggregatorSink 
{
    private AggregatorSource source = null;
    private boolean initialized = false;
    
    private static Log logger =
            LogFactory.getLog(ServiceGroupEntryAggregatorSink.class.getName());
    
    /** Creates a new instance of ServiceGroupEntryAggregatorSink */
    public ServiceGroupEntryAggregatorSink() 
    {
    
    }
    
    public void initialize(Object parameters) 
        throws AggregatorException, Exception 
    {
        if (this.initialized) {
            this.terminate();
        }
        
        if (this.source == null) {
            throw new AggregatorException("Aggregator Source not set!");
        }

        this.initialized = true;       
    }
    
    public AggregatorSource getSource() {
        return this.source;
    }
    
    public void setSource(AggregatorSource source) {
        this.source = source;
    }

    public void deliver(AnyContentType message, 
                        AggregatorServiceGroupEntryResource entry) 
        throws AggregatorException, Exception 
    {
        MessageElement[] data; // don't initialise here so compiler checks init.
        if (entry == null) {
            logger.error("Deliver got a null entry parameter - bailing");
            return;
        }        
        if (message == null) {
            logger.warn("Deliver got a null message parameter - bailing");
            return;
        }        
       
        if (logger.isDebugEnabled()) {
            logger.debug("Processing message...");
        }
            
        data = message.get_any();
       
        // it might be the case that data is null here, if a 
        // delivery was succesful but with no values; this only 
        // matters for the debugging statements, not the main code.
 
        if (logger.isDebugEnabled()) 
        {
            if(data!=null) {
                logger.debug(
                    "Deliver successfully processed message content into " +
                    Integer.toString(data.length) + " message elements:");
                for (int i=0;i<data.length;i++) {
                    logger.debug("Message Element (" + 
                                 Integer.toString(i) + "): " +
                                 XMLUtils.ElementToString(data[i]));
                }
            } else {
                logger.debug("Deliver successuflly received no values.");
            }
        }

        logger.debug("Updating an AggregatorServiceGroupEntryResource");

        AggregatorContent content = entry.getContent();
        if ((content == null) || (content.getAggregatorData()==null)) {
            logger.error("Found null content or null content data during update");
            return;
        }
        
        content.getAggregatorData().set_any(data);
        entry.setContent(content);
       
    }    
    
    public void terminate() {
        if (!this.initialized) {
            return;
        }        
        this.initialized = false;        
    }    
    
}
