/*
* Portions of this file Copyright 1999-2005 University of Chicago
* Portions of this file Copyright 1999-2005 The University of Southern California.
*
* This file or a portion of this file is licensed under the
* terms of the Globus Toolkit Public License, found at
* http://www.globus.org/toolkit/download/license.html.
* If you redistribute this file, with or without
* modifications, you must include this notice in the file.
*/
package org.globus.mds.index.impl;

import java.rmi.RemoteException;
import java.util.LinkedList;
import java.util.List;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import javax.xml.namespace.QName;

import org.apache.axis.message.addressing.Address;
import org.apache.axis.message.addressing.EndpointReferenceType;
import org.apache.axis.message.addressing.ReferencePropertiesType;
import org.apache.axis.MessageContext;
import org.apache.axis.utils.XMLUtils;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.globus.wsrf.ResourceKey;
import org.globus.wsrf.ResourceProperty;
import org.globus.wsrf.Topic;
import org.globus.wsrf.TopicList;
import org.globus.wsrf.TopicListAccessor;
import org.globus.wsrf.Constants;

import org.globus.wsrf.impl.ReflectionResourceProperty;
import org.globus.wsrf.impl.SimpleResourcePropertySet;
import org.globus.wsrf.utils.FaultHelper;

import org.globus.mds.servicegroup.stubs.CreateResponse;
import org.globus.mds.servicegroup.stubs.Create;
import org.globus.mds.servicegroup.stubs.ServiceGroupFactoryCreateFaultType;

import org.globus.wsrf.Resource;
import org.globus.wsrf.ResourceHome;
import org.globus.wsrf.ResourceContext;
import org.globus.wsrf.ResourceProperties;
import org.globus.wsrf.ResourcePropertySet;

/**
 * IndexFactory service implementation.
 */
public class IndexFactoryService implements Resource
{
    private static Log logger =
            LogFactory.getLog(IndexFactoryService.class.getName());

    public CreateResponse create(Create request) 
        throws ServiceGroupFactoryCreateFaultType 
    {

        // hitCountRP.accumulate(1);

        CreateResponse response = new CreateResponse();
        
        EndpointReferenceType epr = new EndpointReferenceType();

        IndexHome home = null;
        IndexResource w = null;

        MessageContext ctx = MessageContext.getCurrentContext();

        String servicePath = ctx.getTargetService();

        String indexHomeName = 
            Constants.JNDI_SERVICES_BASE_NAME + servicePath  + "/groupHome";

        try {
            Context initialContext = new InitialContext();
            home =
                (IndexHome) initialContext.lookup(indexHomeName);

            w = home.create();
        } catch (NamingException e) {            
            ServiceGroupFactoryCreateFaultType fault = 
                new ServiceGroupFactoryCreateFaultType();
            FaultHelper faultHelper = new FaultHelper(fault);
            faultHelper.addFaultCause(e);
            faultHelper.addDescription("when performing JNDI lookup for "+indexHomeName);
            throw fault;            
        } catch (Exception e) {
            ServiceGroupFactoryCreateFaultType fault = 
                new ServiceGroupFactoryCreateFaultType();
            FaultHelper faultHelper = new FaultHelper(fault);
            faultHelper.addFaultCause(e);
            throw fault;    
        }
        
        ResourceKey key = w.getResourceKey();
        ReferencePropertiesType props = new ReferencePropertiesType();

        try {
            props.add(key.toSOAPElement());
        }
        catch(Exception e) {
            ServiceGroupFactoryCreateFaultType fault = 
                new ServiceGroupFactoryCreateFaultType();
            FaultHelper faultHelper = new FaultHelper(fault);
            faultHelper.addFaultCause(e);
            faultHelper.addDescription("Exception when serializing key in createIndex");
            throw fault;                        
        }
        MessageContext msgContext = MessageContext.getCurrentContext();
        String transportURL = 
            (String) msgContext.getProperty(MessageContext.TRANS_URL);
        // this is the URL to the IndexFactoryService
        // but what we actually want is the URL to the IndexService
        // XXX: yuck
        transportURL = transportURL.substring(0,transportURL.indexOf("Factory"))
                    + transportURL.substring(transportURL.indexOf("Factory")+7);
        logger.debug("Index factory service will use transport url "+transportURL);
        try {
            epr.setAddress(new Address(transportURL));
        } catch (Exception e) {
            ServiceGroupFactoryCreateFaultType fault = 
                new ServiceGroupFactoryCreateFaultType();
            FaultHelper faultHelper = new FaultHelper(fault);
            faultHelper.addFaultCause(e);
            throw fault;   
        }
        epr.setProperties(props);

        response.setEndpointReference(epr);
        return response;
    }

}

