/*
* Portions of this file Copyright 1999-2005 University of Chicago
* Portions of this file Copyright 1999-2005 The University of Southern California.
*
* This file or a portion of this file is licensed under the
* terms of the Globus Toolkit Public License, found at
* http://www.globus.org/toolkit/download/license.html.
* If you redistribute this file, with or without
* modifications, you must include this notice in the file.
*/
package org.globus.mds.index.impl;

import java.rmi.RemoteException;
import java.util.Calendar;
import java.util.List;
import javax.xml.namespace.QName;

import org.globus.wsrf.NotifyCallback;
import org.globus.wsrf.ResourceException;
import org.globus.wsrf.ResourceKey;
import org.globus.wsrf.ResourceProperties;
import org.globus.wsrf.ResourceProperty;
import org.globus.wsrf.ResourcePropertySet;
import org.globus.wsrf.TopicList;
import org.globus.wsrf.TopicListAccessor;
import org.globus.wsrf.Constants;
import org.globus.wsrf.WSRFConstants;
import org.globus.wsrf.NotificationConsumerManager;

import org.globus.wsrf.impl.ReflectionResourceProperty;
import org.globus.wsrf.impl.SimpleTopicList;
import org.globus.wsrf.impl.SimpleResourcePropertyMetaData;

import org.apache.axis.MessageContext;
import org.apache.axis.message.addressing.EndpointReferenceType;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.globus.wsrf.impl.servicegroup.EntryTopic;
import org.globus.wsrf.impl.servicegroup.EntryResourcePropertyTopic;
import org.globus.wsrf.impl.servicegroup.ServiceGroupConstants;

import org.globus.mds.aggregator.impl.AggregatorServiceGroupResource;
import org.globus.mds.aggregator.impl.ServiceGroupEntryAggregatorSink;
import org.globus.mds.aggregator.impl.AggregatorSource;
import org.globus.mds.aggregator.impl.AggregatorSink;
import org.globus.mds.aggregator.impl.AggregatorParamTypeNotSupportedException;
import org.globus.mds.aggregator.impl.AggregatorSinkTypeNotSupportedException;

/**
 * Resource Implementation of an Index resource.
 * Each index resource maintains an aggregation of resource properties
 * from other resources that have been registered to it.
 *
 * all of this index must remain in memory at all time.
 */
public class IndexResource extends AggregatorServiceGroupResource
                           implements TopicListAccessor
{
    private static Log logger =
            LogFactory.getLog(IndexResource.class.getName());

    private ResourceKey resourceKey = null;

    static NotificationConsumerManager consumerManager = 
        NotificationConsumerManager.getInstance();

    public static final QName RP_SET = 
        new QName("http://mds.globus.org/index", "IndexRP");

    protected Calendar terminationTime;
    
    private ResourcePropertySet propSet;
    private TopicList topicList;
    private String [] strings = null;
   
    public IndexResource() 
    {
        super.init(IndexResource.RP_SET);
        this.propSet = this.getResourcePropertySet();
        this.topicList = new SimpleTopicList(this);
        ResourceProperty prop = null;
        
        EntryResourcePropertyTopic rpTopic = 
            new EntryResourcePropertyTopic(
                this.propSet.get(ServiceGroupConstants.ENTRY));
        
        this.propSet.add(rpTopic);
        this.topicList.addTopic(rpTopic);
        
        try {
            prop = 
                new ReflectionResourceProperty(SimpleResourcePropertyMetaData.TERMINATION_TIME,
                                               this);
            this.propSet.add(prop);
            prop = 
                new ReflectionResourceProperty(SimpleResourcePropertyMetaData.CURRENT_TIME, 
                                               this);
            this.propSet.add(prop);                        
            
            // initialize aggregator
            this.initializeAggregator();
            
        } catch (Exception e) {
            e.printStackTrace();
        }        
    }

    /** Used to inform the resource of its key. */
    public void setResourceKey(ResourceKey k) 
        throws Exception
    {
        if(resourceKey==null) {
            resourceKey = k;
        } else {
            throw new Exception("Resource key can only be set once.");
        }
    }

    public ResourceKey getResourceKey() {
        return resourceKey;
    }


    public void setTerminationTime(Calendar time) {
        logger.debug("Set Termination time called: " + time.getTime());
        this.terminationTime = time;
    }
    
    public Calendar getTerminationTime() {
        return this.terminationTime;
    }

    public Calendar getCurrentTime() {
        return Calendar.getInstance();
    }

    public TopicList getTopicList() {
        return this.topicList;
    }

    /** returns an EPR to a newly created consumer, such that notifications
    sent to this consumer will be delivered to the callback. */
    public EndpointReferenceType getConsumer(NotifyCallback callback)
    {
        EndpointReferenceType consumerEPR;

        try {
            consumerEPR = consumerManager.createNotificationConsumer(callback);
        } catch(ResourceException re) {
            logger.error("Exception getting a consumer");
            return null;
        }
        return consumerEPR;
    }
    
    private void initializeAggregator() throws Exception
    {
        // initialize aggregator
        ServiceGroupEntryAggregatorSink sink = 
            new ServiceGroupEntryAggregatorSink();

        this.loadCompatibleSources(sink);        
    }
}

