/*
* Portions of this file Copyright 1999-2005 University of Chicago
* Portions of this file Copyright 1999-2005 The University of Southern California.
*
* This file or a portion of this file is licensed under the
* terms of the Globus Toolkit Public License, found at
* http://www.globus.org/toolkit/download/license.html.
* If you redistribute this file, with or without
* modifications, you must include this notice in the file.
*/
package org.globus.wsrf.impl.servicegroup;

import org.globus.wsrf.ResourceProperty;

import org.globus.wsrf.impl.ResourcePropertyTopic;
import org.globus.wsrf.impl.SimpleTopic;

import org.globus.wsrf.utils.Resources;

import org.globus.util.I18n;

/**
 * A ResourcePropertyTopic extension for Entry RP. 
 * The topic should only notify if active subscriptions are detected,
 * in order to avoid unnecessary Entry RP processing.
 */
public class EntryResourcePropertyTopic extends ResourcePropertyTopic
{
    private static I18n i18n = I18n.getI18n(Resources.class.getName());
       
    /** Creates a new instance of EntryResourcePropertyTopic */
    public EntryResourcePropertyTopic(ResourceProperty rp) 
    {
        if(rp == null)
        {
            throw new IllegalArgumentException(
                i18n.getMessage("nullArgument", "rp"));
        }
        this.rp = rp;
        this.topic = new EntryTopic(this.rp.getMetaData().getName());
    }        
    
    public void notify(Object obj) throws Exception
    {
        if (((EntryTopic)this.topic).hasActiveSubscriptions()) {
            super.notify(obj);
        }
    }    
}
