/*
* Portions of this file Copyright 1999-2005 University of Chicago
* Portions of this file Copyright 1999-2005 The University of Southern California.
*
* This file or a portion of this file is licensed under the
* terms of the Globus Toolkit Public License, found at
* http://www.globus.org/toolkit/download/license.html.
* If you redistribute this file, with or without
* modifications, you must include this notice in the file.
*/
package org.globus.wsrf.impl.servicegroup.inmemory;

import org.oasis.wsrf.servicegroup.Add;

import org.apache.axis.message.addressing.EndpointReferenceType;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.globus.wsrf.ResourceKey;
import org.globus.wsrf.ResourceProperties;
import org.globus.wsrf.ResourceProperty;
import org.globus.wsrf.ResourcePropertySet;

import org.globus.wsrf.impl.SimpleResourceKey;
import org.globus.wsrf.impl.SimpleResourcePropertySet;
import org.globus.wsrf.impl.SimpleResourceProperty;

import org.globus.wsrf.impl.servicegroup.ServiceGroupEntryResource;

import javax.xml.namespace.QName;

import org.globus.mds.inmemory._2004._07._13.PairedKeyType;

import org.globus.wsrf.impl.servicegroup.ServiceGroupConstants;

/** Implements a simple in-memory service group entry resource.
 */

public class InMemoryServiceGroupEntryResource 
       implements ServiceGroupEntryResource, ResourceProperties {

    private static Log logger =
            LogFactory.getLog(InMemoryServiceGroupEntryResource.class.getName());

    public static final QName RP_SET =
        new QName("http://mds.globus.org/inmemory/2004/07/13", 
                  "InMemoryEntryRP");

    ResourcePropertySet propSet;

    // Resource properties defined in ServiceGroupEntry
    ResourceProperty contentRP;
    ResourceProperty memberRP;
    ResourceProperty serviceGroupEPRRP;

    /** probably need to make this constructor take content as a
        parameter? or make the code that calls constructor then later on call
        set on the appropriate resource property.
    */
    public InMemoryServiceGroupEntryResource(ResourceKey groupKey,
                                             Add addMsg,
                                             EndpointReferenceType serviceGroupEPR) 
    {
        try {

            Object initialContent = addMsg.getContent();
            EndpointReferenceType memberEPR = addMsg.getMemberEPR();
            
            // key = new InMemoryServiceGroupEntryKey(groupKey, String.valueOf(hashCode()));
            PairedKeyType pk = new PairedKeyType();

            pk.setGroupKey((String)groupKey.getValue());
            pk.setEntryKey(String.valueOf(this.hashCode()));
            
            key = new SimpleResourceKey(ServiceGroupConstants.ENTRY_KEY, pk);

            propSet = new SimpleResourcePropertySet(RP_SET);

            contentRP = new SimpleResourceProperty(ServiceGroupConstants.CONTENT); 
            contentRP.add(initialContent);
            propSet.add(contentRP);

            memberRP = new SimpleResourceProperty(ServiceGroupConstants.MEMBER_EPR); 
            memberRP.add(memberEPR);
            propSet.add(memberRP);

            serviceGroupEPRRP = new SimpleResourceProperty(ServiceGroupConstants.SERVICE_GROUP_EPR); 
            serviceGroupEPRRP.add(serviceGroupEPR);
            propSet.add(serviceGroupEPRRP);

        } catch(Exception e) {
            logger.error("Exception constructing in memory SG entry: "+e);
        }   

    }

    public ResourcePropertySet getResourcePropertySet() {
        return propSet;
    }

    private ResourceKey key;

    public ResourceKey getKey() {
        return key;
    }

    private EndpointReferenceType entryEPR=null;

    public EndpointReferenceType getEntryEPR() {
        return entryEPR;
    }

    public void setEntryEPR(EndpointReferenceType epr) {
        entryEPR = epr;
    }

}

