/*
* Portions of this file Copyright 1999-2005 University of Chicago
* Portions of this file Copyright 1999-2005 The University of Southern California.
*
* This file or a portion of this file is licensed under the
* terms of the Globus Toolkit Public License, found at
* http://www.globus.org/toolkit/download/license.html.
* If you redistribute this file, with or without
* modifications, you must include this notice in the file.
*/
package org.globus.wsrf.impl.servicegroup.inmemory;

import java.rmi.RemoteException;
import java.util.LinkedList;
import java.util.List;


import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import javax.xml.namespace.QName;

import org.apache.axis.message.addressing.Address;
import org.apache.axis.message.addressing.EndpointReferenceType;
import org.apache.axis.message.addressing.ReferencePropertiesType;
import org.apache.axis.MessageContext;
import org.apache.axis.utils.XMLUtils;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.globus.wsrf.ResourceKey;
import org.globus.wsrf.ResourceProperty;
import org.globus.wsrf.Topic;
import org.globus.wsrf.TopicList;
import org.globus.wsrf.TopicListAccessor;
import org.globus.wsrf.Constants;

import org.globus.wsrf.impl.ReflectionResourceProperty;
import org.globus.wsrf.impl.SimpleResourceKey;
import org.globus.wsrf.impl.SimpleResourcePropertySet;
import org.globus.wsrf.impl.lifetime.DestroyProvider;
import org.globus.wsrf.impl.lifetime.SetTerminationTimeProvider;
import org.globus.wsrf.impl.notification.GetCurrentMessageProvider;
import org.globus.wsrf.impl.notification.SubscribeProvider;
import org.globus.wsrf.impl.properties.GetMultipleResourcePropertiesProvider;
import org.globus.wsrf.impl.properties.GetResourcePropertyProvider;
import org.globus.wsrf.impl.properties.QueryResourcePropertiesProvider;
import org.globus.wsrf.impl.properties.SetResourcePropertiesProvider;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import org.oasis.wsrf.lifetime.ResourceNotDestroyedFaultType;
import org.oasis.wsrf.lifetime.TerminationTimeChangeRejectedFaultType;
import org.oasis.wsrf.lifetime.UnableToSetTerminationTimeFaultType;

import org.oasis.wsn.InvalidTopicExpressionFaultType;
import org.oasis.wsn.NoCurrentMessageOnTopicFaultType;
import org.oasis.wsn.ResourceUnknownFaultType;
import org.oasis.wsn.SubscribeCreationFailedFaultType;
import org.oasis.wsn.TopicNotSupportedFaultType;
import org.oasis.wsn.TopicPathDialectUnknownFaultType;
import org.oasis.wsrf.properties.InvalidQueryExpressionFaultType;
import org.oasis.wsrf.properties.InvalidResourcePropertyQNameFaultType;
import org.oasis.wsrf.properties.InvalidSetResourcePropertiesRequestContentFaultType;
import org.oasis.wsrf.properties.QueryEvaluationErrorFaultType;
import org.oasis.wsrf.properties.SetResourcePropertyRequestFailedFaultType;
import org.oasis.wsrf.properties.UnableToModifyResourcePropertyFaultType;
import org.oasis.wsrf.properties.UnknownQueryExpressionDialectFaultType;
import org.oasis.wsrf.properties.GetMultipleResourceProperties_Element;
import org.oasis.wsrf.properties.GetMultipleResourcePropertiesResponse;
import org.oasis.wsrf.properties.GetResourcePropertyResponse;
import org.oasis.wsrf.properties.QueryResourceProperties_Element;
import org.oasis.wsrf.properties.QueryResourcePropertiesResponse;

import org.globus.mds.servicegroup.stubs.Create;
import org.globus.mds.servicegroup.stubs.CreateResponse;
import org.globus.mds.servicegroup.stubs.ServiceGroupFactoryPortType;


import org.globus.wsrf.ResourceHome;
import org.globus.wsrf.ResourceContext;
import org.globus.wsrf.ResourceProperties;
import org.globus.wsrf.ResourcePropertySet;




/**
 * A factory implementation for creating new InMemoryServiceGroups.
 */
public class InMemoryServiceGroupFactory implements ServiceGroupFactoryPortType,
                                            ResourceProperties 
{
    private static Log logger =
            LogFactory.getLog(InMemoryServiceGroupFactory.class.getName());

    // TODO get rid of this provider delegation and use Op Provider mech
    private QueryResourcePropertiesProvider queryResourcePropertiesProvider =
        new QueryResourcePropertiesProvider();
    
    private GetMultipleResourcePropertiesProvider getMultipleResourcePropertiesProvider = new GetMultipleResourcePropertiesProvider();
    
    private GetResourcePropertyProvider getResourcePropertyProvider =
        new GetResourcePropertyProvider();

    public CreateResponse create(Create request) 
        throws RemoteException {

        CreateResponse response = new CreateResponse();
        
        EndpointReferenceType epr = new EndpointReferenceType();

        InMemoryServiceGroupHome groupHome = null;
        InMemoryServiceGroupResource w = null;
        // get the groupHome
        try {
            Object o = getResourceJNDIParameter("groupHome");

            // it is probably a common configuration error that 
            // the incorrect home will be configured here
            try {
                groupHome = (InMemoryServiceGroupHome)o;
            } catch(ClassCastException cce) {
                throw new RemoteException("groupHome JNDI parameter should be an InMemoryServiceGroupHome, but was actually "+o.getClass());
            }
            // TODO: needs to be part of the ServiceGroupHome interface
            // to allow .create() ? Or a specialisation of that...?
            w = groupHome.create();
        } catch (Exception e) {
            throw new RemoteException("couldn't create new service group", e);
        }
       
        // TODO: this EPR formation is rather messy -- should use 
        // helper classes to do it more nicely.
 
        ResourceKey key = w.getKey();


        ReferencePropertiesType props = new ReferencePropertiesType();
        try {
            props.add(key.toSOAPElement());
        } catch(Exception e) {
            throw new RemoteException("When adding new ServiceGroup resource to home",e);
        }

        MessageContext msgContext = MessageContext.getCurrentContext();
        String transportURL = 
            (String) msgContext.getProperty(MessageContext.TRANS_URL);
        // this is the URL to the IndexFactoryService
        // but what we actually want is the URL to the IndexService
        // TODO: yuck
        transportURL = transportURL.substring(0,transportURL.indexOf("Factory"))
                    + transportURL.substring(transportURL.indexOf("Factory")+7);
        logger.debug("InMemoryServiceGroupHomeFactory Service will use transport URL "+transportURL);
        try {
            epr.setAddress(new Address(transportURL));
        } catch (Exception e) {
            throw new RemoteException("", e);
        }
        epr.setProperties(props);

        response.setEndpointReference(epr);

        return response;
    }

    // WS-Properties 

    public QueryResourcePropertiesResponse queryResourceProperties(QueryResourceProperties_Element request) 
        throws RemoteException, 
               InvalidResourcePropertyQNameFaultType,
               InvalidQueryExpressionFaultType,
               QueryEvaluationErrorFaultType,
               org.oasis.wsrf.properties.ResourceUnknownFaultType, 
               UnknownQueryExpressionDialectFaultType {
        return queryResourcePropertiesProvider.queryResourceProperties(request);
    }

    public GetMultipleResourcePropertiesResponse getMultipleResourceProperties(GetMultipleResourceProperties_Element request) 
        throws RemoteException, 
               InvalidResourcePropertyQNameFaultType, 
               org.oasis.wsrf.properties.ResourceUnknownFaultType {
        return getMultipleResourcePropertiesProvider.getMultipleResourceProperties(request);
    }
    
    public GetResourcePropertyResponse getResourceProperty(QName name) 
        throws RemoteException, 
               InvalidResourcePropertyQNameFaultType, 
               org.oasis.wsrf.properties.ResourceUnknownFaultType {
        return getResourcePropertyProvider.getResourceProperty(name);
    }






    // below is the code we need to act as a resource

    /** I don't think this gets used for anything in the core yet?*/
    public static final QName RP_SET =
        new QName("http://index.mil", "TODO");

    ResourcePropertySet propSet;


    {
        this.propSet = new SimpleResourcePropertySet(RP_SET);
    }

    public ResourcePropertySet getResourcePropertySet() {
        return this.propSet;
    }


    /** Returns a named parameter from the present service's JNDI block.

    <br>
    TODO: a unit test for this would be to call and check that we get the same 
    answer back as using the resource context getResourceHome call
    <br>
    TODO: this could be factored out into some other class? ResourceContext? */

    public static Object getResourceJNDIParameter(String name) throws Exception {

        org.apache.axis.MessageContext context =
            org.apache.axis.MessageContext.getCurrentContext();

        String servicePath = context.getTargetService();
        Context initialContext = new InitialContext();

        return initialContext.lookup(
            Constants.JNDI_BASE_NAME + servicePath + "/" +
            name);
    }

}

