/*
* Portions of this file Copyright 1999-2005 University of Chicago
* Portions of this file Copyright 1999-2005 The University of Southern California.
*
* This file or a portion of this file is licensed under the
* terms of the Globus Toolkit Public License, found at
* http://www.globus.org/toolkit/download/license.html.
* If you redistribute this file, with or without
* modifications, you must include this notice in the file.
*/
package org.globus.mds.trigger.impl;

import java.io.IOException;
import java.io.PrintStream;
import java.util.Calendar;
import java.util.Hashtable;
import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;
import java.rmi.RemoteException;

import javax.xml.namespace.QName;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;

import org.apache.axis.encoding.AnyContentType;
import org.apache.axis.message.Text;
import org.apache.axis.message.MessageElement;
import org.apache.axis.message.addressing.EndpointReferenceType;
import org.apache.axis.utils.XMLUtils;

import org.apache.xpath.XPathAPI;
import org.apache.xpath.objects.XBoolean;
import org.apache.xpath.objects.XNodeSet;
import org.apache.xpath.objects.XNumber;
import org.apache.xpath.objects.XObject;
import org.apache.xpath.objects.XString;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.globus.mds.aggregator.impl.AggregatorServiceGroupResource;
import org.globus.mds.aggregator.impl.AggregatorServiceGroupEntryResource;
import org.globus.mds.aggregator.impl.ServiceGroupEntryAggregatorSink;
import org.globus.mds.aggregator.impl.QueryAggregatorSource;
import org.globus.mds.aggregator.impl.AggregatorSource;
import org.globus.mds.aggregator.impl.AggregatorSink;

import org.globus.mds.aggregator.types.AggregatorContent;
import org.globus.mds.aggregator.types.AggregatorConfig;
import org.globus.mds.aggregator.types.AggregatorData;
import org.globus.mds.aggregator.types.PairedKeyType;

import org.globus.mds.trigger.types.ActionResult;
import org.globus.mds.trigger.types.TriggerRuleType;
import org.globus.mds.trigger.types.TriggerStatusType;

import org.globus.mds.usefulrp.AccumulatorResourceProperty;

import org.globus.wsrf.config.ContainerConfig;

import org.globus.wsrf.Constants;
import org.globus.wsrf.ResourceKey;
import org.globus.wsrf.ResourceProperties;
import org.globus.wsrf.ResourceProperty;
import org.globus.wsrf.ResourcePropertySet;
import org.globus.wsrf.TopicList;
import org.globus.wsrf.TopicListAccessor;

import org.globus.wsrf.encoding.DeserializationException;
import org.globus.wsrf.encoding.ObjectDeserializer;
import org.globus.wsrf.encoding.ObjectSerializer;

import org.globus.wsrf.impl.ReflectionResourceProperty;
import org.globus.wsrf.impl.SimpleTopicList;
import org.globus.wsrf.impl.SimpleResourcePropertyMetaData;

import org.globus.wsrf.impl.servicegroup.ServiceGroupConstants;
import org.globus.wsrf.impl.servicegroup.EntryResourcePropertyTopic;

import org.globus.wsrf.utils.AnyHelper;
import org.globus.wsrf.utils.XmlUtils;
import org.globus.wsrf.utils.Resources;

import org.globus.util.I18n;

import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;
import org.w3c.dom.CharacterData;

import org.xml.sax.SAXException;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.apache.axis.MessageContext;
/**
 * Implementation of a Trigger ServiceGroup resource. Each resource is also
 * an aggregator sink, in order to match trigger rules against received 
 * data.
 */
public class TriggerResource extends AggregatorServiceGroupResource
                             implements AggregatorSink, TopicListAccessor
{
    private static Log logger =
            LogFactory.getLog(TriggerResource.class.getName());

    private ResourceKey resourceKey = null;

    public static final QName RP_SET = 
        new QName("http://mds.globus.org/2004/08/trigger", "TriggerRP");
    
    public static final QName TRIGGERSTATUS = 
        new QName("http://mds.globus.org/2004/08/trigger/types", "TriggerStatus");

    // TODO: need to give the trigger its own RP doc to include this
    public static final QName FIRE_COUNT_RP = 
        new QName("http://mds.globus.org/2004/08/trigger", "ActionsFiredCount");

    private static I18n i18n = I18n.getI18n(Resources.class.getName());
 
    protected Calendar terminationTime;
    
    private Hashtable triggerTable;
    private ResourcePropertySet propSet;
    private TopicList topicList;
    private String [] strings = null;
    private int fooctr=0;
    private TriggerConfiguration triggerConfig = null;
    private String baseDir;
    
    AccumulatorResourceProperty fireCountRP;
 
    public TriggerResource() 
    {
        super.init(TriggerResource.RP_SET);
        
        this.baseDir = getBaseDirectory();
        this.propSet = this.getResourcePropertySet();
        this.topicList = new SimpleTopicList(this);
        ResourceProperty prop = null;
        
        EntryResourcePropertyTopic rpTopic = 
            new EntryResourcePropertyTopic(
                this.propSet.get(ServiceGroupConstants.ENTRY));
        
        this.propSet.add(rpTopic);
        this.topicList.addTopic(rpTopic);
        
        try {
            prop = 
                new ReflectionResourceProperty(
                    SimpleResourcePropertyMetaData.TERMINATION_TIME,
                    this);
            this.propSet.add(prop);
            prop = 
                new ReflectionResourceProperty(
                    SimpleResourcePropertyMetaData.CURRENT_TIME, 
                    this);
            this.propSet.add(prop);


            fireCountRP=new AccumulatorResourceProperty(FIRE_COUNT_RP);

            this.propSet.add(prop);

            // initialize aggregator 
            this.loadCompatibleSources(this);
            
            this.triggerTable = 
                this.getTriggerConfiguration().getExecutableMappingTable(); 
            if (logger.isDebugEnabled()) {
                logger.debug("Trigger resource contains the following executable mapping table: " 
                             + this.triggerTable.toString());
            }
        
        } catch (Exception e) {
            logger.error("Error during TriggerResource creation: ",e);
        }       
    }

    public TriggerConfiguration getTriggerConfiguration() 
    {
        if (this.triggerConfig != null) {
            return this.triggerConfig;
        }
        
        MessageContext ctx = MessageContext.getCurrentContext();

        String servicePath = ctx.getTargetService();

        String jndiName = Constants.JNDI_SERVICES_BASE_NAME + servicePath +
               "/triggerConfiguration";
        if (logger.isDebugEnabled()) {
            logger.debug("Will read configuration from jndi name: "+jndiName);
        }
        try {
            Context initialContext = new InitialContext();
            return this.triggerConfig =
                (TriggerConfiguration) initialContext.lookup(jndiName);

        } catch (NamingException e) {
            logger.error("when performing JNDI lookup for "+jndiName+": "+e );
        } catch (Exception e) {
            logger.error("when performing JNDI lookup for "+jndiName+": "+e );
        }
        return this.triggerConfig;
    }

    /** Used to inform the resource of its key. */
    public void setResourceKey(ResourceKey k) 
        throws Exception
    {
        if(resourceKey==null) {
            resourceKey = k;
        } else {
            throw new Exception("Resource key can only be set once.");
        }
    }

    public ResourceKey getResourceKey() {
        return resourceKey;
    }


    public void setTerminationTime(Calendar time) {
        if (logger.isDebugEnabled()) {
            logger.debug("Set Termination time called: " + time.getTime());
        }
        this.terminationTime = time;
    }
    
    public Calendar getTerminationTime() {
        return this.terminationTime;
    }

    public Calendar getCurrentTime() {
        return Calendar.getInstance();
    }

    public TopicList getTopicList() {
        return this.topicList;
    }



    // AggregatorSink methods

    /** Takes delivery of a message from an AggregatorSource and
      * applies the appropriate rules to it. 
      *
      * @param context this context should always
      * be a ServiceGroupEntry, which we expect to contain appropriate
      * trigger service rules.
      *
      * TODO: should there be any queuing/locking to stop multiple
      * triggers (either for same rule or for all rules) from firing
      * simultaneously?
      */
    public void deliver(AnyContentType messageObj, 
                        AggregatorServiceGroupEntryResource entry)
    {
        try 
        {
        if (logger.isDebugEnabled()) {
            logger.debug("In trigger service deliver, with message "+
            AnyHelper.toSingleString(messageObj));
        }

        MessageElement[] message = messageObj.get_any();
        if (message == null) {
            if (logger.isDebugEnabled()) {
                logger.debug("Trigger resource will not process null message, returning from function");
            }
            return;
        }
        
        // extract content

        //     get RP set from entry
        ResourcePropertySet rpSet = entry.getResourcePropertySet();

        //     get content RP from entry
        ResourceProperty contentRP = rpSet.get(ServiceGroupConstants.CONTENT);

        TriggerRuleType triggerRule = null;

        AggregatorContent content = entry.getContent();

        AggregatorConfig config = content.getAggregatorConfig();

        MessageElement[] any = config.get_any();
            
        if (logger.isDebugEnabled()) {
            logger.debug("Checking config for trigger rule:  "+
                AnyHelper.toSingleString(any));
        }
        
        // go over each config entry to see if it is a trigger rule
        for(int i=0;i<any.length;i++) {

            if (ObjectDeserializer.contains(
                        any[i],
                        TriggerRuleType.class)) {
    
                try {
                    triggerRule = (TriggerRuleType)
                            ObjectDeserializer.toObject(
                                any[i],
                                TriggerRuleType.class
                            );
            

                } catch(DeserializationException de) { 
                    logger.error("Could not deserialize trigger rule type: "+de);
                }
            }
            // TODO maybe we should have a test to spit out a warning if there
            // are two trigger rule types in content?
        }

        if(triggerRule == null) {
            logger.error("Trying to process trigger rule for an entry that does not have a trigger rule.");
            return;
        }
 
        AggregatorData aggregatorData = content.getAggregatorData();

        TriggerStatusType triggerStatus;
        // the aggregatorData in a trigger can only ever have one
        // entry, that being the trigger status
        try {
            triggerStatus = (TriggerStatusType)
                ObjectDeserializer.toObject(aggregatorData.get_any()[0],
                                            TriggerStatusType.class);
        } catch(Exception e) {
            // if this fails (for example, initially the aggregated data
            // may not be a TriggerStatusType), start off with a fresh new
            // triggerStatus
            triggerStatus = new TriggerStatusType();
            if (logger.isDebugEnabled()) {
                logger.debug("Unable to locate existing TriggerStatus, creating new trigger status object");
            }            
        }        

        // extract xpath expression
        String xpath = triggerRule.getMatchingRule();
        if (logger.isDebugEnabled()) {
            logger.debug("Processing trigger rule, XPath expression is: " + xpath);
        }

        // match expression

        // TODO namespaces: should extract namespaces from the bowels of the
        // message element, on the actual <MatchingRule> element, so that
        // we get *all* of the right definitions

        // if match  (TODO we also need handling of when a condition CEASES to be true)

        boolean fire = false;

        triggerStatus.setRuleLastCheckedAt(Calendar.getInstance());
        
        Document outputDoc = null;
        Document inputDoc = XmlUtils.newDocument();
        Node root = 
            inputDoc.appendChild(inputDoc.createElementNS(
                "http://mds.globus.org/2004/08/trigger","trigger:inputMessage"));
        
        try {         
            for(int i=0;message!=null && i<message.length;i++) 
            {

                if(message[i] == null) 
                {
                    if (logger.isDebugEnabled()) {
                        logger.debug("message element "+i+" is null.");
                    }
                } 
                else 
                {
                    root.appendChild(inputDoc.importNode(message[i], true)); 
                }                           
            }
            if (inputDoc.getDocumentElement().hasChildNodes()) {
                outputDoc = this.evaluateXPath(inputDoc, xpath);        
            } else {
                if (logger.isDebugEnabled()) {
                    logger.debug("XPath input document has no child nodes!");
                }                
            }
        }
        catch(Exception e) 
        {
            logger.error("When evaluating XPath rule: " + e );
        } 
        
        // we fire the trigger if any results were returned.
        if(outputDoc!=null && 
           outputDoc.getDocumentElement().hasChildNodes()) 
        {
            if (logger.isDebugEnabled()) {                
                logger.debug("Trigger scheduled to fire! XPath query result was: " + 
                             XmlUtils.toString(outputDoc));
            }
            fire=true;
        }        
        
        Calendar now = Calendar.getInstance();
        
        if(!fire)
        {
            triggerStatus.setConditionTrueSince(null);
        }

        if(fire && triggerStatus.getConditionTrueSince() == null) {
            triggerStatus.setConditionTrueSince(now);
        } 

        // perform rate limiting here. This if() will set fire back to
        // false if rate limiting applies.

        if(fire) 
        {
            //   if we are set to fire, then rate limit if:
            //              the conditionTrue has not been true 
            //              for long enough, 
            //              (now - conditionTrueSince) < minimumMatchTime
            
            if( triggerStatus.getConditionTrueSince()!=null &&
                    triggerRule.getMinimumMatchTime()!=null )  
            {
                long conditionTrueDuration = 
                    now.getTime().getTime() - 
                        triggerStatus.getConditionTrueSince().getTime().getTime();

                long minimumMatchTime = triggerRule.getMinimumMatchTime().intValue()*1000;
                
                if (conditionTrueDuration < minimumMatchTime) {
                    fire=false;
                    if (logger.isDebugEnabled()) {                
                        logger.debug("Inhibiting trigger action because condition not true for sufficient time: " + 
                                     "the condition has been true for " + 
                                     Long.toString(conditionTrueDuration) + 
                                     " milliseconds, while the minimum match time is set to " + 
                                     Long.toString(minimumMatchTime) + " milliseconds.");
                    }
                }
                else // mark the last time the condition was true
                {
                    Calendar last = Calendar.getInstance();
                    last.setTimeInMillis(now.getTime().getTime() - minimumMatchTime);
                    triggerStatus.setConditionTrueSince(last);
                }
            }
            //       or
            //              we have fired recently
            //              (now - actionFiredAt ) < minimumFiringInterval
            if( triggerStatus.getActionFiredAt()!= null &&
                triggerRule.getMinimumFiringInterval()!= null)
            {
                long actionFiredInterval = 
                    now.getTime().getTime() - 
                        triggerStatus.getActionFiredAt().getTime().getTime();
                
                long minimumFiringInterval = triggerRule.getMinimumFiringInterval().intValue()*1000;

                if (actionFiredInterval < minimumFiringInterval)
                {
                    fire=false;
                    if (logger.isDebugEnabled()) {
                        logger.debug("Inhibiting trigger action because minimumFiringInterval not met: " + 
                                     "the action was fired " + 
                                     Long.toString(actionFiredInterval) + 
                                     " milliseconds ago, while the minimum firing interval is set to " + 
                                     Long.toString(minimumFiringInterval) + " milliseconds.");                        
                    }
                }
            }
        }


        if(fire) 
        {
            Element actionResultElement = runAction(triggerRule, 
                                                    messageObj, entry, config);

            triggerStatus.setActionFiredAt(Calendar.getInstance());
	    fireCountRP.accumulate(1);
            
            if(actionResultElement == null) 
            {
                if (logger.isDebugEnabled()) {
                    logger.debug("runAction returned null");
                }
            }
            else
            {
                ActionResult actionResult = new ActionResult();

                MessageElement[] m = new MessageElement[] {
                    new MessageElement(actionResultElement)
                };

                actionResult.set_any(m);

                triggerStatus.setActionOutput(actionResult);                  
            }

        }

        // update content
        // by the time we reach here, triggerStatus should have had
        // all the updates it needs made       

        try { 
            aggregatorData.set_any( new MessageElement[] {
                (MessageElement)ObjectSerializer.toSOAPElement(
                    triggerStatus, TRIGGERSTATUS) } 
            );
            entry.setContent(content);            
        } catch(Exception e) {
            logger.error("Couldn't store action result into service group content", e);
        }         
        
        if (logger.isDebugEnabled()) {
            logger.debug("End of trigger resource deliver");
        }
        }
        catch (Exception e)
        {
            logger.error("Unhandled exception in trigger resource deliver method: " + e);
        }
    }

    public Document evaluateXPath(Document inputDoc, String xPathExpression) 
        throws Exception
    {
        if (logger.isDebugEnabled()) {
            logger.debug("Applying xpath " + 
                         xPathExpression +
                         " to " + 
                         XmlUtils.toString(inputDoc));
        }
        
        try 
        {
            Map namespaces = new HashMap();
            Element rootElement = inputDoc.getDocumentElement();
            collectNamespaces(rootElement,
                              namespaces, 
                              new HashMap());
            setNamespaces(rootElement, namespaces);

            XObject result = XPathAPI.eval(rootElement,
                                           xPathExpression,
                                           rootElement);

            Document outputDoc = XmlUtils.newDocument();
            
            Node root =
                outputDoc.appendChild(outputDoc.createElementNS(
                    "http://mds.globus.org/2004/08/trigger","trigger:xpathResult"));

            if (result instanceof XBoolean ||
                result instanceof XNumber ||
                result instanceof XString) {
                MessageElement element = 
                    new MessageElement(new Text( result.str() ));
                root.appendChild(outputDoc.importNode(element,true));
            } else if (result instanceof XNodeSet) {
                XNodeSet set = (XNodeSet)result;
                NodeList list = set.nodelist();
                for (int i=0;i<list.getLength();i++) {
                    Node node = list.item(i);
                    if (node instanceof Document) {
                         root.appendChild(
                            outputDoc.importNode(
                                new MessageElement(((Document)node).getDocumentElement()), true));
                    } else if (node instanceof Element) {
                         root.appendChild(
                            outputDoc.importNode(
                                new MessageElement((Element)node), true));
                    } else if (node instanceof CharacterData) {
                         root.appendChild(outputDoc.importNode(
                            new MessageElement(
                                new Text(((CharacterData)node).getData())), true ));
                    } else {
                        throw new Exception(i18n.getMessage(
                            "unsupportedXpathReturn", node.getClass().getName()));
                    }
                }
            } else {
                throw new Exception(i18n.getMessage(
                    "unsupportedXpathReturn", result));
            }

            return outputDoc;
          
        }
        catch(Exception e) 
        {
                logger.error("When evaluating XPath rule: "+e);
                // TODO should also put this in the entry status exposed
                // through the content.
                //throw e;
                return null;
        }        
    }

    private void collectNamespaces(Node node, Map namespaces, Map prefixes) 
    {
        NamedNodeMap attributes = node.getAttributes();
        if (attributes != null) {
            for (int i=0;i<attributes.getLength();i++) {
                Attr attr = (Attr)attributes.item(i);
                String name = attr.getName();
                String value = attr.getValue();
                if (name.startsWith("xmlns:")) {
                    if (namespaces.get(value) == null) {
                        // ns not defined
                        if (prefixes.get(name) != null) {
                            // find unique prefix
                            int j = 1;
                            do {
                                name = "xmlns:ns" + j++;
                            } while ( prefixes.get(name) != null);
                        }
                        prefixes.put(name, value);
                        namespaces.put(value, name);
                    }
                }
            }
        }
        NodeList children = node.getChildNodes();
        if (children != null) {
            for (int i=0;i<children.getLength();i++) {
                Node child = children.item(i);
                if (child.getNodeType() == Node.ELEMENT_NODE) {
                    collectNamespaces(child, namespaces, prefixes);
                }
            }
        }
    }

    private void setNamespaces(Element node, Map namespaces) {
        Iterator iter = namespaces.entrySet().iterator();
        while(iter.hasNext()) {
            Map.Entry entry = (Map.Entry)iter.next();
            node.setAttributeNS("http://www.w3.org/2000/xmlns/",
                                (String)entry.getValue(),
                                (String)entry.getKey());
        }
    }  
    
    /** Performs the appropriate action. <br>
        <ul>
          <li>TODO: feed in something on stdin (XPATH transformed XML)</li>
       </ul>
    */
    Element runAction(TriggerRuleType rule, AnyContentType message,
                      AggregatorServiceGroupEntryResource entry, 
                      AggregatorConfig config)
    {
        try 
        {
        
        if (logger.isDebugEnabled()) {
            logger.debug("Performing trigger action.");
        }

        // $GLOBUS_LOCATION /libexec/trigger/ <actionname>
        String commandline="";

        commandline += baseDir;  //GLOBUS LOCATION

        commandline += "/libexec/trigger/";

        String triggerName = rule.getActionScript();
        String triggerFile = (String)this.triggerTable.get(triggerName);              

        if (triggerFile == null) {
            String error =  
                "Could not locate a suitable executable file for trigger name (no action was taken): " 
                    + triggerName;
            logger.warn(error);
            Document response = this.createErrorDocument(error);
            return response.getDocumentElement();       
        }
            
        commandline += triggerFile; 
        if (logger.isDebugEnabled()) {
            logger.debug("Will execute: "+commandline);
        }

        Runtime runtime = Runtime.getRuntime();
        Process process = null;
        try {
            process = runtime.exec(commandline);
        } catch(IOException e) {
            String error = "Could not execute " +commandline+" with exception: "+e;
            logger.warn(error);
            Document response = this.createErrorDocument(error);
            return response.getDocumentElement();
        }


        // TODO make this into proper XSD
        
        PrintStream o = new PrintStream(process.getOutputStream());
        o.println("<fire>");

        o.println("<message>");
        MessageElement[] msgs=message.get_any();
        for(int i=0; i<msgs.length; i++) {
            o.println(msgs[i]);
        }
        o.println("</message>");

        try {
        
        o.println(ObjectSerializer.toString(entry.getMemberEPR(),
            new QName("","MemberEPR")));
        o.println(ObjectSerializer.toString(config,
            new QName("","AggregatorConfig")));

        } catch(Exception e) {
            logger.error("When serializing into action script stdin",e);
        }

        o.println("</fire>");

        o.close();

        Document responseDoc = null;
        String error = "";
        try {        
            responseDoc = XMLUtils.newDocument(process.getInputStream());
        } catch(Exception e) {
            error = "Error while parsing output from action script" + e;
            logger.error(error);
        }
        if (responseDoc == null) {
            responseDoc = this.createErrorDocument(error);
        }        

        return responseDoc.getDocumentElement();        
        
        }        
        catch (Exception e) 
        {
            logger.error("Unhandled exception in runAction: " + e);
            return null;
        }
    }

    private Document createErrorDocument(String error) 
        throws Exception
    {
        try 
        {            
            Document responseDoc = XMLUtils.newDocument();
            Node node = responseDoc.appendChild(
                responseDoc.createElementNS(
                    "http://mds.globus.org/2004/08/trigger",
                    "trigger:actionScriptRuntimeError")
            );
            node.setNodeValue(error);
            return responseDoc;
        }
        catch (Exception e) 
        {
            logger.error("Unhandled exception creating error response document" + e);
            throw e;
        }
    }
    
    private static String getBaseDirectory() 
    {
        return ContainerConfig.getBaseDirectory();
    }

    public void initialize(Object parameters) 
    {
            // NO OP
    }

    public void terminate()
    {
        // NO OP
    }

    public void setSource(AggregatorSource source)
    {
        // NO OP
    }

    public AggregatorSource getSource()
    {
        return null;
    }

}

