/*
* Portions of this file Copyright 1999-2005 University of Chicago
* Portions of this file Copyright 1999-2005 The University of Southern California.
*
* This file or a portion of this file is licensed under the
* terms of the Globus Toolkit Public License, found at
* http://www.globus.org/toolkit/download/license.html.
* If you redistribute this file, with or without
* modifications, you must include this notice in the file.
*/
package org.globus.mds.usefulrp;

import java.util.Date;
import java.util.Calendar;
import java.util.List;

import javax.xml.namespace.QName;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.apache.axis.MessageContext;
import org.globus.wsrf.utils.ContextUtils;

import org.globus.wsrf.impl.ReflectionResourceProperty;
import org.globus.wsrf.impl.SimpleResourcePropertyMetaData;
import org.globus.wsrf.ResourceProperty;


import org.globus.wsrf.impl.ReflectionResourceProperty;

import org.globus.mds.usefulrp.types.ServiceMetaData;

/** 
    An MDS ResourceProperty that will allow services to publish
    the service version and service start time. <br>        
*/

public class ServiceMetaDataResourceProperty extends ReflectionResourceProperty 
{
    private static Log logger =
            LogFactory.getLog(ServiceMetaDataResourceProperty.class.getName());

    public static final QName SERVICE_METADATA = 
        new QName("http://mds.globus.org/metadata/2005/02", "ServiceMetaDataInfo");

    // for now we are just going to set a version number in this class 
    // in order to cover the default uninitialized case.
    // calling code or subclassess may override the version number at any point 
    public static final String CURRENT_VERSION = "4.0.3";

    private ServiceMetaData metadata = new ServiceMetaData();

    /**  
    @throws java.lang.Exception passes along any exception encountered
        during construction of underlying ReflectionResourceProperty.
    */
    public ServiceMetaDataResourceProperty() throws Exception
    {
       this(SERVICE_METADATA, CURRENT_VERSION);
    }
    
    public ServiceMetaDataResourceProperty(String version) 
        throws Exception
    {
        this(SERVICE_METADATA, version);
    }
    
    public ServiceMetaDataResourceProperty(QName name, String version) 
        throws Exception 
    {
        super(new SimpleResourcePropertyMetaData(name));
        setObject(this);
        setPropertyName("ServiceMetaDataInfo");
        initialize();

        // set the start time to now.
        this.metadata.setStartTime(Calendar.getInstance());
        
        // set version based on argument
        this.metadata.setVersion(version);
        
        // set a worst-case default value for service type
        this.metadata.setServiceTypeName("unspecified");
        
        // try to set the service type name to the service name at runtime
        String serviceName = 
            ContextUtils.getTargetServicePath(MessageContext.getCurrentContext());        
        
        if ( (serviceName != null) && (serviceName.length() > 0) ) {
            this.metadata.setServiceTypeName(serviceName);
        }        
    }

    public Object getServiceMetaDataInfo() {
        return this.metadata;
    }
    
    public void setVersion(String version) {
        this.metadata.setVersion(version);    
    }
    
    public void setServiceTypeName(String typeName) {
        this.metadata.setServiceTypeName(typeName);
    }
}


