/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */

package org.globus.mds.webmds;

import java.util.Vector;
import org.globus.mds.webmds.ParameterException;

/**
 * Simple class to represent a name-value pair of Strings.
 */
public class Parameter {
    private String paramName;
    private String[] paramValues;

    /**
     * constructor.
     * @param name the parameter name.
     * @param values the parameter values.
     */
    public Parameter(String name, String[] values) {
	this.init(name, values);
    }

    /**
     * constructor.
     * @param name the parameter name.
     * @param value a single parameter value.
     */
    public Parameter(char[] name, char[] value) {
	String[] values = new String[1];
	values[0] = new String(value);
	this.init(new String(name), values);
    }

    /**
     * constructor.
     * @param name the parameter name.
     * @param value a single parameter value.
     */
    public static Parameter createSingleValuedParameter(String name, String value) {
	String[] values = new String[1];
	values[0] = value;
	return(new Parameter(name, values));
    }


    private void init(String name, String[] values) {
	String[] trimmedValues = null;
	if (values != null) {
	    trimmedValues = new String[values.length];
	    for (int i = 0; i < values.length; i ++) {
		if (values[i] == null) {
		    trimmedValues[i] = null;
		} else {
		    trimmedValues[i] = values[i].trim();
		    if (trimmedValues[i].length() == 0) {
			trimmedValues[i] = null;
		    }
		}
	    }
	}
	this.paramName = name;
	this.paramValues = trimmedValues;
    }

    /**
     * gets the parameter name
     * @return the parameter name
     */
    public String getName() {
	return(this.paramName);
    }

    /**
     * gets the parameter value
     * @return the parameter value
     */
    public String[] getValues() {
	return(this.paramValues);
    }

    public String getSingleValue() throws ParameterException {
	if (this.paramValues == null) {
	    throw new ParameterException("Parameter has no values");
	}
	if (this.paramValues.length != 1) {
	    throw new ParameterException("Parameter '" +
					 this.getName() +
					 "' has " +
					 Integer.toString(this.paramValues.length) +
					 " values.");
	}
	return(this.paramValues[0]);
    }
}
