/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */

package org.globus.mds.webmds;

import java.util.HashMap;
import java.util.Set;
import org.globus.mds.webmds.Parameter;

public class ParameterMap extends HashMap {
    public Object put(Object key, Object value) {
	if (key == null) {
	    throw new NullPointerException("Attempt to add map entry with null key");
	}
	Object returnValue = this.get(key);
	// throw ClassCastException if value isn't a parameter.
	Parameter param = (Parameter)value;
	super.put(key, value);
	return(returnValue);
    }

    public Object putParameter(Parameter param) throws ParameterMapException {
	if (param == null) {
	    throw new ParameterMapException(
		"attempt to add a null parameter to the map");
	}
	if (param.getName() == null) {
	    throw new ParameterMapException(
		"attempt to add a parameter with null name to the map");
	}
	return(this.put(param.getName(), param));
    }

    /**
     * prefixedSubMap -- takes a parameter set, finds the subset
     * whose names start with "prefix.", and returns a set that contains
     * those parameters, with the "prefix." stripped off their names.  For
     * example, if a parameter map contained these parameters:
     *                      {"abc", {"abc", "def"}},
     *                      {"foo.x", {"foo.x", "x1"}},
     *                      {"foo.y", {"foo.y", "y1"}},
     *                      {"foosball", {"foosball", "hello"}}
     *
     * prefixedSubMap("foo") would return a map containing {"x", {"x", "x1"}}
     * and {"y", {"y", "y1"}}.
     *
     * @param prefix the prefix to search for.
     * @returns the transformed map.
     */

    public ParameterMap prefixedSubMap(String prefix) throws ParameterMapException {
	String fixedPrefix;
	if (prefix == null) {
	    fixedPrefix = new String(".");
	} else {
	    fixedPrefix = new String(prefix + ".");
	}

	int prefixLength = fixedPrefix.length();

	Set keySet = this.keySet();
	if (keySet == null) {
	    return(null);
	}

	ParameterMap result = new ParameterMap();
	String[] keys = (String[]) keySet.toArray(new String[0]);
	for (int i = 0; i < keys.length; i++) {
	    String oldKey = keys[i];
	    if (oldKey.startsWith(fixedPrefix)) {
		Parameter oldParam = (Parameter)this.get(oldKey);
		String newKey = oldKey.substring(prefixLength);
		Parameter newParam = new Parameter(newKey, oldParam.getValues());
		result.putParameter(newParam);
	    }
	}
	if (result.isEmpty()) {
	    return(null);
	}
	return(result);
    }

    public String debugKeyString() throws ParameterMapException {
	String keyString = new String();
	Set keySet = this.keySet();
	if (keySet == null) {
	    return(null);
	}
	String[] keys = (String[]) keySet.toArray(new String[0]);
	for (int i = 0; i < keys.length; i++) {
	    if (i > 0) {
		keyString = keyString + ", ";
	    }
	    keyString = keyString + keys[i];
	}
	return(keyString);
    }

}
