/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */

package org.globus.mds.webmds;
import java.util.Vector;
import org.globus.mds.webmds.Parameter;
import org.globus.mds.webmds.ParameterMap;
import org.globus.mds.webmds.WebmdsUtil;
import java.io.IOException;
import javax.xml.transform.stream.StreamSource;
import javax.xml.transform.Source;
import org.globus.mds.webmds.WebmdsXmlSourceException;

/**
 * Interface called by WebmdsServlet to acquire an xml document.
 * @see WebmdsServlet
 */
public interface WebmdsXmlSource {
    /**
     * Function called by WebmdsServlet at invocation time to get an
     * xml document.
     *
     * @param name name of the XML source object to get.
     * @param initParams a ParameterMap of options corresponding to the
     * initialization parameters for this XML source.
     * @param requestParams a ParameterMap of options corresponding to
     * the parameters from this http request.
     * @return a javax.xml.transform.Source object representing a source
     * of XML.
     * @see ParameterMap
     */
    public Source getXmlSource(String name,
			       ParameterMap initParams,
			       ParameterMap requestParams)
	throws WebmdsXmlSourceException,
	       WebmdsConfigException,
  	       IOException;

    /**
     * Return information an end-user might find interesting about the
     * request.  This will be set as an xsl parameter when the results
     * are returned to the user.
     */
    public String getRequestInfo();

    /**
     * Initialize the XML source.
     * @param webmdsUtil a WebmdsUtil object with utility methods that the
     * xml source can use.
     */
    public void init(WebmdsUtil webmdsUtil);
}
