/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */

package org.globus.mds.webmds.test;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import com.meterware.httpunit.WebClient;
import com.meterware.httpunit.WebConversation;
import com.meterware.httpunit.WebRequest;
import com.meterware.httpunit.WebResponse;
import com.meterware.httpunit.WebTable;
import com.meterware.httpunit.GetMethodWebRequest;
import com.meterware.servletunit.ServletRunner;
import com.meterware.servletunit.ServletUnitClient;
import com.meterware.httpunit.HttpInternalErrorException;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import org.xml.sax.SAXException;
import java.net.MalformedURLException;

import java.util.regex.Pattern;
import java.util.regex.Matcher;

import java.net.URL;

import org.globus.mds.webmds.WebmdsServlet;
import org.globus.mds.webmds.WebmdsException;
import org.globus.mds.webmds.test.ServletTestFramework;

public class ServletXslTests extends ServletTestFramework {

    private ServletUnitClient simClient = null;
    private WebConversation httpClient = null;

    public ServletXslTests (String s) {
	super(s);
    }

    public static Test suite() 
    {
        return new TestSuite(ServletXslTests.class);
    }

    public void testWithServletRunner() throws Exception {
	System.out.println("Running " + this.getClass().getName() +
			   " tests with servlet simulator.");
	GetMethodWebRequest gr =
	    makeSRGetMethodWebRequest("testfileinfo", "servicegroupxsl");
	doTest(this.getDefaultSimClient(), gr);

	/*
	 * Test that the servlet will fail if the XSL source's
	 * suitableForXSL parameter is unset.
	 */
	gr = makeSRGetMethodWebRequest("testfileinfo", "testxslnull");
	doTestExpectFailure(this.getDefaultSimClient(), gr);

	/*
	 * Test that the servlet will fail if the XSL source's
	 * suitableForXSL parameter is false.
	 */
	gr = makeSRGetMethodWebRequest("testfileinfo", "testxslfalse");
	doTestExpectFailure(this.getDefaultSimClient(), gr);
    }

    public void testWithHttp() throws Exception {
	if (this.getDefaultHttpClient() != null) {
	    System.out.println("Running " + this.getClass().getName() +
			       " tests against server at " +
			       this.getServletURL());
	    GetMethodWebRequest gr =
		makeGetMethodWebRequest(this.getServletURL(),
					"testfileinfo",
					"servicegroupxsl");
	    doTest(this.getDefaultHttpClient(), gr);
	/*
	 * Test that the servlet will fail if the XSL source's
	 * suitableForXSL parameter is unset.
	 */
	gr = makeGetMethodWebRequest(this.getServletURL(),
				     "testfileinfo", "testxslnull");
	doTestExpectFailure(this.getDefaultHttpClient(), gr);

	/*
	 * Test that the servlet will fail if the XSL source's
	 * suitableForXSL parameter is false.
	 */
	gr = makeGetMethodWebRequest(this.getServletURL(),
				     "testfileinfo", "testxslfalse");
	doTestExpectFailure(this.getDefaultHttpClient(), gr);

	} else {
	    System.out.println("No " + SERVLET_URL_PROPERTY_NAME +
			       " property specified; skipping " +
			       this.getClass().getName() + " http tests");
	}

    }

    private void doTest(WebClient client, GetMethodWebRequest request)
    throws Exception {
	/*
	 * It would be nicer to read the testfileinfo config file rather
	 * than having the test/sample.xml path hardwired in.  But this will
	 * only cause a problem if someone changes the testfileinfo config
	 * to read a different file.
	 */
	WebResponse response = runFormTest(client, request, CONTENT_TYPE_HTML);
	WebTable[] tables = response.getTables();
	assertEquals(1, tables.length);
	WebTable table = tables[0];
	assertEquals(4, table.getColumnCount());
	assertEquals(11, table.getRowCount());

	int row = 0;
// Compare header row
	compareRow(table, row, "Resource Type", "ID", "Information", "");

// Compare the other rows
	compareCASRow(table, ++row, "127.0.0.1", "Serving VO:\\s+Description of VO of CAS service");
	compareServiceGroupRow(table, ++row, "128.9.72.106", "3", "3");
	compareGramRow(table, ++row, "128.9.72.106", "0", "0", "0");
	compareGramRow(table, ++row, "128.9.72.106", "1", "0", "0");
	compareRFTRow(table, ++row, "128.9.72.106", "0", "0", "5.5 GB", "3970");
	compareServiceGroupRow(table, ++row, "128.9.72.178", "4", "4");
	compareGramRow(table, ++row, "128.9.72.178", "0", "1", "10");
	compareRFTRow(table, ++row, "128.9.72.178", "0", "0", "8.62 GB", "84519");
	compareGramRow(table, ++row, "128.9.72.178", "2", "1", "10");
	compareGramRow(table, ++row, "128.9.72.178", "1", "1", "10");
    }

    public void compareRow(WebTable table,
			   int rowNumber,
			   String expectedColumn0,
			   String expectedColumn1,
			   String expectedColumn2,
			   String expectedColumn3)
	throws Exception {

	// first two columns are easy
	String[] expectedColumns = new String[2];
	expectedColumns[0] = expectedColumn0;
	expectedColumns[1] = expectedColumn1;
	compareRow(table, rowNumber, expectedColumns);


	// third has weird whitespace issues
/*
	System.out.println("comparing '" +
			   table.getCellAsText(rowNumber, 2).trim() +
			   "' to '" +
			   expectedColumn2 + "'");
*/			   
	assertTrue(
	    table.getCellAsText(rowNumber, 2).trim().matches(expectedColumn2));

	// fourth is easy
	assertEquals(expectedColumn3,
		     table.getCellAsText(rowNumber, 3).trim());

    }

    public String serviceGroupPattern(String directEntries, String descendants) {
	return(new String("This WS-ServiceGroup has\\s+" +
			  directEntries +
			  "\\s+direct entries,\\s+" +
			  descendants +
			  "\\s+including descendants."));
    }

    public void compareServiceGroupRow(WebTable table,
				       int rowNumber,
				       String ipaddr,
				       String directEntries,
				       String descendants) throws Exception {
	compareRow(table,
		   rowNumber,
		   "ServiceGroup",
		   ipaddr,
		   serviceGroupPattern(directEntries, descendants),
		   "detail");
    }

    public String gramPattern(String queues, String clusters, String hosts) {
	return(queues +
	       "\\s+queues, submitting to\\s+" +
	       clusters +
	       "\\s+cluster\\(s\\) of\\s+" +
	       hosts +
	       "\\s+host\\(s\\).");
    }

    public void compareGramRow(WebTable table,
			    int rowNumber,
			    String ipaddr,
			    String queues,
			    String clusters,
			    String hosts) throws Exception {
	compareRow(table,
		   rowNumber,
		   "GRAM",
		   ipaddr,
		   gramPattern(queues, clusters, hosts),
		   "detail");
    }

    public void compareRFTRow(WebTable table,
			      int rowNumber,
			      String ipaddr,
			      String activeTransferResources,
			      String activeFiles,
			      String amountTransferred,
			      String filesTransferred) throws Exception {
	compareRow(table,
		   rowNumber,
		   "RFT",
		   ipaddr,
		   rftPattern(activeTransferResources, activeFiles,
			      amountTransferred, filesTransferred),
		   "detail");
    }

    public void compareCASRow(WebTable table,
			      int rowNumber,
			      String ipaddr,
			      String description) throws Exception {
	compareRow(table,
		   rowNumber,
		   "CAS",
		   ipaddr,
		   description,
		   "detail");
    }


    String rftPattern(String activeTransferResources,
		      String activeFiles,
		      String amountTransferred,
		      String filesTransferred) {
	return(activeTransferResources +
	       "\\s+active transfer resources, transferring\\s+" +
	       activeFiles +
	       "\\s+files.\\s+" +
	       amountTransferred +
	       "\\s+transferred in\\s+" +
	       filesTransferred +
	       "\\s+files since start of database.");
    }

    private void doTestExpectFailure(WebClient client, GetMethodWebRequest request)
	throws Exception {
	boolean threwExceptionAsExpected = false;
	try {
	    WebResponse response = client.getResponse(request);
	} catch (HttpInternalErrorException e) {
	    threwExceptionAsExpected = true;
	}
	assertTrue(threwExceptionAsExpected);
    }
}
