/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */

package org.globus.mds.webmds.xmlSources.resourceProperties;

import org.globus.mds.webmds.xmlSources.resourceProperties.ResourcePropertyHelper;
import org.globus.mds.webmds.xmlSources.resourceProperties.ResourcePropertySourceException;
import org.globus.mds.webmds.Parameter;
import org.globus.mds.webmds.ParameterMap;
import org.globus.mds.webmds.WebmdsConfigException;
import org.globus.mds.webmds.xmlSources.xmlDomNode.WebmdsNodeSource;
import org.globus.mds.webmds.WebmdsUtil;
import java.lang.Boolean;
import javax.xml.namespace.QName;
import org.w3c.dom.Node;
import java.io.IOException;
import java.io.File;
import org.apache.axis.message.addressing.EndpointReferenceType;

/**
 * Class to get a resource property and return it as a DOM node.
 * This uses some configuration parameters from the servlet config file:
 * <dl>
 * <dt><b>endpoint</b>
 * (specified as <b>xmlSource.</b><i>sourceName</i>.<b>param.endpoint</b> in
 * the servlet configuration)
 * <dd>The endpoint name to be used in a getResourceProperty request.</dd>
 * <dt><b>endpointKeyName</b> and <b>endpointKeyValue</b> (specified as
 * <b>xmlSource.</b><i>sourceName</i>.<b>param.endpointKeyName</b> and
 * <b>xmlSource.</b><i>sourceName</i>.<b>param.endpointKeyValue</b> in
 * the servlet configuration).
 * <dd>An optional key-value pair that can be used to supply reference
 * properties to an endpoint specified by an
 * <b>endpoint</b> parameter.
 * <dt><b>allowUserEndpoints</b>
 * (specified as
 * <b>xmlSource.</b><i>sourceName</i>.<b>param.allowUserEndpoints</b> in
 * the servlet configuration)
 * <dd>If <b>true</b>, values of
 * <b>xmlSource.</b><i>sourceName</i>.<b>param.endpoint</b>,
 * <b>xmlSource.</b><i>sourceName</i>.<b>param.endpointKeyName</b>, and
 * <b>xmlSource.</b><i>sourceName</i>.<b>param.endpointKeyValue</b>
 * specified in the request will override the configured endpoint value.
 * <dt><b>endpointFile</b>
 * (specified as <b>xmlSource.</b><i>sourceName</i>.<b>param.endpointFile</b> in
 * the servlet configuration)
 * <dd>The name of a file containing an XML representation of an endpoint.
 * The <b>endpointFile</b> parameter can never be overridden by request 
 * arguments.

 *
 * </dl>
 *
 */
public class ResourcePropertyQueryNodeSource implements WebmdsNodeSource {
    private String requestInfoString = null;
    private WebmdsUtil webmdsUtil = null;

/**
 * Get a resource property and return it as a node.
 * @param configParams - configured parameters for this XML source
 * @param requestParams - request parameters for this XML source
 */
    public Node getNode(ParameterMap configParams, ParameterMap requestParams)
	throws ResourcePropertySourceException,
	WebmdsConfigException,
	IOException {
	String rpNamespace = null;
	String rpName = null;

	EndpointReferenceType endpoint =
	    ResourcePropertyConfigHelper.getEndpointReference(
		webmdsUtil, configParams, requestParams);
	Node node =
	    ResourcePropertyHelper.queryResourceProperties(endpoint);

	this.requestInfoString = "Endpoint: " + endpoint.toString();
	return(node);
    }

    /**
     * Returns a string containing the request endpoint.
     */
    public String getRequestInfo() {
	return(this.requestInfoString);
    }

    public void init(WebmdsUtil webmdsUtil) {
	this.webmdsUtil = webmdsUtil;
    }

}

