/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */

package org.globus.mds.webmds.xmlSources.resourceProperties.test;

import junit.framework.TestCase;
import junit.framework.Test;
import junit.framework.TestSuite;
import java.util.Properties;
import java.io.FileInputStream;
import org.apache.axis.message.addressing.EndpointReferenceType;

import org.globus.mds.webmds.xmlSources.resourceProperties.ResourcePropertyHelper;
import org.globus.mds.webmds.xmlSources.resourceProperties.ResourcePropertyConfigHelper;
import org.w3c.dom.Node;

import org.globus.mds.webmds.test.ServletTestFramework;
import org.globus.mds.webmds.xmlSources.resourceProperties.test.Constants;

public class ResourcePropertyHelperTest extends ServletTestFramework {
    private String secureEndpointString = null;
    private String insecureEndpointString = null;
    private String propFileName = System.getProperty("WebmdsTestProperties");

    public ResourcePropertyHelperTest(String name) {
	super(name);
    }

    public static Test suite() 
    {
        return new TestSuite(ResourcePropertyHelperTest.class);
    }

    // put this in so test could be run outside of junit
    public static void main(String[] args) 
    {
        ResourcePropertyHelperTest test = new ResourcePropertyHelperTest("ResourcePropertyHelperTest");
        try {
            test.testResourcePropertyHelperTest();
        } catch (Exception e) {
            System.out.print(e);
            e.printStackTrace();
        }
    }

    public void setUp() throws Exception {
	super.setUp();
	this.secureEndpointString = findPropertyValue(Constants.SECURE_SERVICE_PREFIX_PROPERTY_NAME) + "/DefaultIndexService";
	this.insecureEndpointString = findPropertyValue(Constants.INSECURE_SERVICE_PREFIX_PROPERTY_NAME) + "/DefaultIndexService";
    }

    public void testResourcePropertyHelperTest() throws Exception {
	doTest(this.insecureEndpointString);
	doTest(this.secureEndpointString);
    }

    public void doTest(String endpointString) throws Exception {
	System.out.println("querying resource properties at '" + endpointString + "'");
	EndpointReferenceType endpoint =
	    ResourcePropertyConfigHelper.getEndpointFromURLString(
		endpointString);
	Node resourceProperties =
	    ResourcePropertyHelper.queryResourceProperties(endpoint);
	assertNotNull(resourceProperties);
	assertEquals(Node.DOCUMENT_NODE, resourceProperties.getNodeType());
    }
}
