/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */

package org.globus.mds.webmds.xmlSources.xmlDomNode;
import java.util.Vector;
import org.globus.mds.webmds.Parameter;
import org.globus.mds.webmds.ParameterMap;
import org.globus.mds.webmds.WebmdsUtil;
import javax.servlet.ServletException;
import java.io.IOException;
import org.globus.mds.webmds.WebmdsXmlSourceException;
import org.globus.mds.webmds.WebmdsConfigException;
import org.globus.mds.webmds.xmlSources.xmlDomNode.WebmdsNodeSourceException;
import org.w3c.dom.Node;

/**
 * Interface called by WebmdsServlet to acquire an xml document.
 */
public interface WebmdsNodeSource {
    /**
     * Function called by WebmdsNodeSource at invocation time to get a
     * DOM node object.
     *
     * @param configParams a ParameterMap of options corresponding to the
     * configuration parameters for this XML source.
     * @param requestParams a ParameterMap of options corresponding to
     * the parameters from this http request.
     */
    public Node getNode(ParameterMap configParams,
			ParameterMap requestParams)
	throws WebmdsNodeSourceException,
	       WebmdsConfigException,
	       IOException;

    /**
     * Returns information about the request that might be interesting
     * to the end-user.
     */
    public String getRequestInfo();

    /**
     * Initialize the XML Node source.
     * @param webmdsUtil a WebmdsUtil object with utility methods that the
     * xml source can use.
     */
    public void init(WebmdsUtil webmdsUtil);
}
