#include "globus_common.h"
#include "version.h"
#include "RendezvousFactoryService_client.h"
#include "globus_service_engine.h"
#include "globus_notification_consumer.h"

typedef int (*test_case)(void);
#define SIZEOF_ARRAY(x) sizeof(x) / sizeof(x[0])

static const char BINARY_DATA_1[] = {1, 2, 3};
static char * contact = NULL;
wsa_EndpointReferenceType               sync_endpoint_reference;
wsa_EndpointReferenceType               sub_sync1_endpoint_reference;
wsa_EndpointReferenceType               sub_sync2_endpoint_reference;
wsa_EndpointReferenceType               sync_notification_reference;
wsa_EndpointReferenceType               sub_sync1_notification_reference;
wsa_EndpointReferenceType               sub_sync2_notification_reference;
RendezvousFactoryService_client_handle_t sync_handle;
RendezvousFactoryService_client_handle_t sub_sync1_handle;
RendezvousFactoryService_client_handle_t sub_sync2_handle;

wsrendezvous_registerInputType          registerInput;

globus_mutex_t lock;
globus_cond_t cond;
globus_bool_t done = 0;
globus_bool_t stopped = 0;;

xsd_QName rendezvous_completed_topic_qname =
{
    "http://www.globus.org/namespaces/2004/09/rendezvous",
    "RendezvousCompleted"
};

static
int 
rendezvous_test_init(void)
{
    int                                 rc;

    rc = globus_module_activate(GLOBUS_COMMON_MODULE);
    if (rc != GLOBUS_SUCCESS)
    {
        goto out;
    }
    rc = globus_module_activate(RENDEZVOUSFACTORYSERVICE_MODULE);
    if (rc != GLOBUS_SUCCESS)
    {
        goto deactivate_common_out;
    }
    rc = globus_module_activate(GLOBUS_NOTIFICATION_CONSUMER_MODULE);
    if (rc != GLOBUS_SUCCESS)
    {
        goto deactivate_rendezvous_out;
    }
    return 0;
deactivate_rendezvous_out:
    globus_module_deactivate(RENDEZVOUSFACTORYSERVICE_MODULE);
deactivate_common_out:
    globus_module_deactivate(GLOBUS_COMMON_MODULE);
out:
    return rc;
}

static
int
rendezvous_test_destroy(void)
{
    int                                 rc = 0;

    globus_module_deactivate(RENDEZVOUSFACTORYSERVICE_MODULE);
    globus_module_deactivate(GLOBUS_COMMON_MODULE);

    return rc;
}

static
void
register_callback(
    RendezvousFactoryService_client_handle_t handle,
    void *                              user_args,
    globus_result_t                     result,
    const wsrendezvous_registerResponseType *
                                        registerResponse,
    RendezvousFactoryPortType_register_fault_t fault_type,
    const xsd_any *                     fault)
{
}

static
void
top_rendezvous_notification_callback(
    void *                              arg,
    wsnt_TopicExpressionType *          topic,
    wsa_EndpointReferenceType *         producer,
    xsd_anyType *                       message)
{
    globus_mutex_lock(&lock);
    done = 1;
    globus_cond_signal(&cond);
    globus_mutex_unlock(&lock);
}

void engine_stop_callback(
    globus_result_t                     result,
    globus_service_engine_t             engine,
    void *                              args)
{
    globus_mutex_lock(&lock);
    stopped = 1;
    globus_cond_signal(&cond);
    globus_mutex_unlock(&lock);
}

static
void
sub_rendezvous_notification_callback(
    void *                              arg,
    wsnt_TopicExpressionType *          topic,
    wsa_EndpointReferenceType *         producer,
    xsd_anyType *                       message)
{
    globus_result_t                     result;

    globus_mutex_lock(&lock);

    registerInput.rank = -1;
    result = RendezvousFactoryPortType_register_epr_register(
        (RendezvousFactoryService_client_handle_t) arg,
        &sync_endpoint_reference,
        &registerInput,
        register_callback,
        NULL);

    if (result != GLOBUS_SUCCESS)
    {
        printf("# %s\n", globus_error_print_friendly(globus_error_peek(result)));
    }

    globus_mutex_unlock(&lock);
}

static
int
create_sync_and_subscribe(
    RendezvousFactoryService_client_handle_t 
                                        handle,
    globus_service_engine_t             engine,
    wsa_EndpointReferenceType *         sync_reference,
    wsa_EndpointReferenceType *         consumer_reference,
    globus_notification_consumer_func_t callback,
    void *                              arg)
{
    wsrendezvous_createSyncType *       createSync;
    wsrendezvous_createSyncResponseType *
                                        createSyncResponse = NULL;
    wsnt_SubscribeType *                Subscribe;
    wsnt_SubscribeResponseType *        SubscribeResponse;
    globus_result_t                     result = GLOBUS_SUCCESS;
    int                                 rc = 0;
    int                                 fault_type;
    xsd_any *                           fault = NULL;

    result = globus_notification_create_consumer(
        consumer_reference,
        engine,
        callback,
        arg);

    if (result != GLOBUS_SUCCESS)
    {
        rc = 1;

        goto out;
    }

    result = wsrendezvous_createSyncType_init(&createSync);
    if (result != GLOBUS_SUCCESS)
    {
        rc = 1;

        goto destroy_consumer_out;
    }

    createSync->Capacity = 2;
    result = RendezvousFactoryPortType_createSync(
        handle,
        contact,
        createSync,
        &createSyncResponse,
        (RendezvousFactoryPortType_createSync_fault_t *) &fault_type,
        &fault);
    if (result != GLOBUS_SUCCESS)
    {
        goto destroy_sync_out;
    }

    result = wsnt_SubscribeType_init(&Subscribe);
    if (result != GLOBUS_SUCCESS)
    {
        goto destroy_sync_response_out;
    }

    result = xsd_any_init(&Subscribe->TopicExpression.any);
    if (result != GLOBUS_SUCCESS)
    {
        goto destroy_subscribe_out;
    }
    Subscribe->TopicExpression.any->any_info = &xsd_QName_contents_info;
    result = xsd_QName_copy(
            (xsd_QName **) &Subscribe->TopicExpression.any->value,
            &rendezvous_completed_topic_qname);
    if (result != GLOBUS_SUCCESS)
    {
        goto destroy_subscribe_out;
    }

    result = xsd_anyURI_init_cstr(
            &Subscribe->TopicExpression._Dialect,
            globus_libc_strdup("http://docs.oasis-open.org/wsn/2004/06/TopicExpression/Simple"));
    if (result != GLOBUS_SUCCESS)
    {
        goto destroy_subscribe_out;
    }
    result = wsa_EndpointReferenceType_copy_contents(
            &Subscribe->ConsumerReference,
            consumer_reference);
    if (result != GLOBUS_SUCCESS)
    {
        goto destroy_subscribe_out;
    }

    result = RendezvousFactoryPortType_Subscribe_epr(
            handle,
            &createSyncResponse->EndpointReference,
            Subscribe,
            &SubscribeResponse,
            (RendezvousFactoryPortType_Subscribe_fault_t *) &fault_type,
            &fault);

    if (result != GLOBUS_SUCCESS)
    {
        goto destroy_subscribe_out;
    }
    result = wsa_EndpointReferenceType_copy_contents(
            sync_reference,
            &createSyncResponse->EndpointReference);
    if (result != GLOBUS_SUCCESS)
    {
        goto destroy_subscribe_response_out;
    }

destroy_subscribe_response_out:
    wsnt_SubscribeResponseType_destroy(SubscribeResponse);
destroy_subscribe_out:
    wsnt_SubscribeType_destroy(Subscribe);
destroy_sync_response_out:
    wsrendezvous_createSyncResponseType_destroy(createSyncResponse);
destroy_sync_out:
    wsrendezvous_createSyncType_destroy(createSync);
destroy_consumer_out:
    if (result != GLOBUS_SUCCESS || rc != 0)
    {
        globus_notification_destroy_consumer(consumer_reference);
    }
out:
    if (result != GLOBUS_SUCCESS || rc != 0)
    {
        return 1;
    }
    else
    {
        return 0;
    }
}

int
main(int argc, char * argv[])
{
    int                                 rc;
    int                                 fault_type = 0;
    xsd_any *                           fault = NULL;
    wsrendezvous_registerResponseType * registerResponse;
    globus_service_engine_t             engine;
    int                                 i;
    globus_result_t                     result;
    globus_abstime_t                    timeout;

    printf("1..1\n");
    fflush(stdout);
    rc = rendezvous_test_init();
    contact = argv[1];
    if (rc != GLOBUS_SUCCESS)
    {
        goto out;
    }

    globus_mutex_init(&lock, NULL);
    globus_cond_init(&cond, NULL);
    globus_mutex_lock(&lock);

    result = RendezvousFactoryService_client_init(
            &sync_handle,
            NULL,
            NULL);

    if (result != GLOBUS_SUCCESS)
    {
        rc = 1;

        goto destroy_rendezvous_out;
    }
    result = RendezvousFactoryService_client_init(
            &sub_sync1_handle,
            NULL,
            NULL);

    if (result != GLOBUS_SUCCESS)
    {
        rc = 1;

        goto destroy_sync_handle_out;
    }
    result = RendezvousFactoryService_client_init(
            &sub_sync2_handle,
            NULL,
            NULL);

    if (result != GLOBUS_SUCCESS)
    {
        rc = 1;

        goto destroy_sub_sync1_handle_out;
    }

    rc = globus_service_engine_init(
            &engine,
            NULL,
            NULL,
            NULL,
            GLOBUS_FALSE);

    if (rc != GLOBUS_SUCCESS)
    {
        rc = 1;

        goto destroy_sub_sync2_handle_out;
    }

    result = globus_service_engine_register_start(
            engine,
            engine_stop_callback,
            NULL);

    if (result != GLOBUS_SUCCESS)
    {
        rc = 1;
        goto destroy_engine_out;
    }

    rc = create_sync_and_subscribe(
            sync_handle,
            engine,
            &sync_endpoint_reference,
            &sync_notification_reference,
            top_rendezvous_notification_callback,
            NULL);

    if (rc != GLOBUS_SUCCESS)
    {
        goto stop_engine_out;
    }

    rc = create_sync_and_subscribe(
            sub_sync1_handle,
            engine,
            &sub_sync1_endpoint_reference,
            &sub_sync1_notification_reference,
            sub_rendezvous_notification_callback,
            sub_sync1_handle);

    if (rc != GLOBUS_SUCCESS)
    {
        goto stop_engine_out;
    }
    rc = create_sync_and_subscribe(
            sub_sync2_handle,
            engine,
            &sub_sync2_endpoint_reference,
            &sub_sync2_notification_reference,
            sub_rendezvous_notification_callback,
            sub_sync2_handle);
    if (rc != GLOBUS_SUCCESS)
    {
        goto stop_engine_out;
    }

    registerInput.data.value = globus_libc_malloc(sizeof(BINARY_DATA_1));
    registerInput.data.length = sizeof(BINARY_DATA_1);
    memcpy(registerInput.data.value, BINARY_DATA_1, sizeof BINARY_DATA_1);
    registerInput.rank = -1;

    for (i = 0; i < 2; i++)
    {
        result = RendezvousFactoryPortType_register_epr(
            sync_handle,
            &sub_sync1_endpoint_reference,
            &registerInput,
            &registerResponse,
            (RendezvousFactoryPortType_register_fault_t *) &fault_type,
            &fault);
        wsrendezvous_registerResponseType_destroy(registerResponse);
        if (result != GLOBUS_SUCCESS)
        {
            rc = 1;
            goto stop_engine_out;
        }

        result = RendezvousFactoryPortType_register_epr(
            sync_handle,
            &sub_sync2_endpoint_reference,
            &registerInput,
            &registerResponse,
            (RendezvousFactoryPortType_register_fault_t *) &fault_type,
            &fault);
        wsrendezvous_registerResponseType_destroy(registerResponse);
        if (result != GLOBUS_SUCCESS)
        {
            rc = 1;
            goto stop_engine_out;
        }
    }

    GlobusTimeAbstimeSet(timeout, 45, 0);
    while (rc != ETIMEDOUT && !done)
    {
        globus_cond_timedwait(&cond, &lock, &timeout);
    }

stop_engine_out:
    result = globus_service_engine_register_stop(engine);
    if (result != GLOBUS_SUCCESS)
    {
        goto destroy_engine_out;
    }

    while (!stopped)
    {
        globus_cond_wait(&cond, &lock);
    }
destroy_engine_out:
    globus_service_engine_destroy(engine);
destroy_sub_sync2_handle_out:
    RendezvousFactoryService_client_destroy(sub_sync2_handle);
destroy_sub_sync1_handle_out:
    RendezvousFactoryService_client_destroy(sub_sync1_handle);
destroy_sync_handle_out:
    RendezvousFactoryService_client_destroy(sync_handle);
destroy_rendezvous_out:
    globus_mutex_unlock(&lock);
    globus_mutex_destroy(&lock);
    globus_cond_destroy(&cond);

    rendezvous_test_destroy();
out:
    printf("%sok\n", rc != 0 ? "not ": "");
    return rc;
}
