/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.replica.replicator.impl;

import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;

import commonj.work.WorkManager;
import commonj.work.WorkItem;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.globus.util.I18n;

import org.globus.replica.rls.RLSClient;
import org.globus.replica.rls.RLSClient.LRC;
import org.globus.replica.rls.RLSException;

/**
 * Implements the <code>Work</code> required to process replication
 * requests.
 */
class CatalogQueryWork implements commonj.work.Work {

    // STATIC //

    private static RlsLrcGetPFNBulk operation = new RlsLrcGetPFNBulk();

    private static Log logger =
        LogFactory.getLog(CatalogQueryWork.class.getName());
    private static I18n i18n =
        I18n.getI18n(ReplicationConstants.MESSAGE_RESOURCE_BUNDLE,
                CatalogQueryWork.class.getClassLoader());

    // FIELDS //

    String lrcurl;
    ArrayList queryParams;
    String proxyFilename;
    int retryMax;
    int retryWait;
    boolean retryBackoff;
    int maxPerQuery;
    int queryTimeout;

    boolean failed = false;
    int rc = 0;
    String errmsg = null;
    ArrayList queryResults;


    // CONSTRUCTORS //

    CatalogQueryWork(String lrcurl,
                     ArrayList queryParams,
                     String proxyFilename,
                     int retryMax,
                     int retryWait,
                     boolean retryBackoff,
                     int maxPerQuery,
                     int queryTimeout) {
        this.lrcurl = lrcurl;
        this.queryParams = queryParams;
        this.proxyFilename = proxyFilename;
        this.retryMax = retryMax;
        this.retryWait = retryWait;
        this.retryBackoff = retryBackoff;
        this.maxPerQuery = maxPerQuery;
        this.queryTimeout = queryTimeout;
    }

    // METHODS //

    public void release() {
    }

    public boolean isDaemon() {
        return false;
    }

    public void run() {
        RLSClient rls = null;
        try {
            // Connect to LRC
            rls = RLSUtil.connect(this.lrcurl, this.proxyFilename,
                    this.retryMax, this.retryWait, this.retryBackoff);

            // Create results and perform operation
            this.queryResults = new java.util.ArrayList(0);
            RLSUtil.bulkOperation(rls, CatalogQueryWork.operation,
                    this.queryParams, this.queryResults,
                    this.maxPerQuery, this.queryTimeout);

            this.rc = RLSClient.RLS_SUCCESS;
        }
        catch (RLSException rlsex) {
            this.failed = true;
            this.rc = rlsex.GetRC();
            this.errmsg = rlsex.getMessage();
        }
        catch (InterruptedException intex) {
            this.failed = true;
            this.errmsg = intex.getMessage();
        }
        finally {
            if (rls!=null)
                try { RLSUtil.close(rls); } catch (Exception ignored) { }
        }
    }
}
