/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.replica.replicator.util;

import java.util.Collection;

/**
 * This interface specifies methods for filtering Replica Catalogs. Users of
 * ReplicationManager may not want to use every available Replica Catalog in
 * their Grid environment. The method(s) of this interface allows users an
 * interface to allow them to manipulate the lookup behavior of the 
 * ReplicationManager. Implementations of this interface MUST be thread safe.
 */
public interface ReplicaCatalogFilter {
    /**
     * This method filters a collection of Replica Catalog URLs based on an
     * unfiltered collection of Replica Catalog URLs. <b>Note:</b> 
     * Implementaors <b>MAY modify</b> the original collection.
     * @param catalogUrls A <code>Collection</code> of <code>String</code>s that
     * represent the URLs of Replica Catalogs.
     * @return A <code>Collection</code> containing <code>String</code>s
     * that represent the URLs of Replica Catalogs. <b>IMPORTANT:</b> 
     * Implementations <b>MUST NOT</b> return a <code>null</code> collection.
     * If no Catalogs URLs match the user criteria, the appropriate return
     * value will be an empty <code>Collection</code> object.
     * @since 1.0
     */
    public Collection filterCatalogUrls(Collection catalogUrls);
}
