/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */

package org.globus.transfer.reliable.service.database;


/**
 * This class is used to store database options used 
 * for connections from RFT to database
 */

public class RFTDatabaseOptions {
    
    private String driverName = null;
    
    private String userName = null;
    
    private String password = null;
    
    private String connectionString = null;
    
    private int maxActive = 20;
    
    private int maxIdle = 2;
    
    private int maxWait = -1;

    private byte onExhaustAction = 1;

    private boolean removeAbandoned = true;

    private int removeAbandonedTimeout = 60*60; // 1 hour
    

    public RFTDatabaseOptions() {
        //empty constructor
    }
    /**
     * @param driverName
     * @param userName
     * @param password
     * @param connectionString
     */
    public RFTDatabaseOptions(String driverName, String userName,
            String password, String connectionString,
            int maxActive, int maxIdle, int maxWait, 
            boolean removeAbandoned, int removeAbandonedTimeout) {
        this.driverName = driverName;
        this.userName = userName;
        this.password = password;
        this.connectionString = connectionString;
        this.maxActive = maxActive;
        this.maxIdle = maxIdle;
        this.maxWait = maxWait;
        this.removeAbandoned = removeAbandoned;
        this.removeAbandonedTimeout = removeAbandonedTimeout;
    }

    public RFTDatabaseOptions(String driverName, String userName,
            String password, String connectionString) {
        this.driverName = driverName;
        this.userName = userName;
        this.password = password;
        this.connectionString = connectionString;
    }

    /**
     * @return connectionString
     */
    public String getConnectionString() {
        return this.connectionString;
    }
    /**
     * @param connectionString
     */
    public void setConnectionString(String connectionString) {
        this.connectionString = connectionString;
    }
    /**
     * @return password
     */
    public String getPassword() {
        return this.password;
    }
    /**
     * @param password
     */
    public void setPassword(String password) {
        this.password = password;
    }
    /**
     * @return username
     */
    public String getUserName() {
        return this.userName;
    }
    /**
     * @param userName
     */
    public void setUserName(String userName) {
        this.userName = userName;
    }
    
    /**
     * @return driverName
     */
    public String getDriverName() {
        return this.driverName;
    }
    /**
     * @param driverName
     */
    public void setDriverName(String driverName) {
        this.driverName = driverName;
    }
    /**
     * @return Returns the maxActive.
     */
    public int getMaxActive() {
        return this.maxActive;
    }
    /**
     * @param maxActive The maxActive to set.
     */
    public void setMaxActive(int maxActive) {
        this.maxActive = maxActive;
    }
    /**
     * @return Returns the maxIdle.
     */
    public int getMaxIdle() {
        return this.maxIdle;
    }
    /**
     * @param maxIdle The maxIdle to set.
     */
    public void setMaxIdle(int maxIdle) {
        this.maxIdle = maxIdle;
    }
    /**
     * @return Returns the maxWait.
     */
    public int getMaxWait() {
        return this.maxWait;
    }
    /**
     * @param maxWait The maxWait to set.
     */
    public void setMaxWait(int maxWait) {
        this.maxWait = maxWait;
    }

    public byte getOnExhaustAction() {
        return this.onExhaustAction;
    }
    public void setOnExhaustAction(byte onExhaustAction) {
        this.onExhaustAction = onExhaustAction;
    }
    public void setRemoveAbandoned(boolean removeAbandoned) {
        this.removeAbandoned = true;
    }
    public boolean getRemoveAbandoned() {
        return this.removeAbandoned;
    }
    public int getRemoveAbandonedTimeout() {
        return this.removeAbandonedTimeout;
    }
    public void setRemoveAbandonedTimeout(int removeAbandonedTimeout) {
        this.removeAbandonedTimeout = removeAbandonedTimeout;
    }
}
