/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */

package org.globus.transfer.reliable.service.factory;

import javax.xml.namespace.QName;

import org.globus.wsrf.impl.ReflectionResourceProperty;
import org.globus.wsrf.impl.SimpleResourcePropertyMetaData;
import org.globus.transfer.reliable.service.database.ReliableFileTransferDbAdapter;
import org.globus.transfer.reliable.service.database.RftDBException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.globus.util.I18n;

/**
 * ActiveResourceInstances Resource Property
 */
public class ActiveResourceInstancesRP extends ReflectionResourceProperty {
    private int activeTransferResourceCount = 0;
    
    private static I18n i18n =
        I18n.getI18n("org.globus.transfer.reliable.errors", 
                ActiveResourceInstancesRP.class.getClassLoader());
    private static Log logger = 
        LogFactory.getLog(ActiveResourceInstancesRP.class.getName());

    /**
     * @param qname
     * @throws Exception
     */
    public ActiveResourceInstancesRP(QName qname) 
        throws Exception {
        super(new SimpleResourcePropertyMetaData(qname));
        setObject(this);
        setPropertyName("ActiveTransferResourceCount");
        initialize();
    }
    
    /**
     * Add this instance to RP
     */
    public void addInstance() {
        this.activeTransferResourceCount++;
    }

    /** Get the number of active resources
     * @return activeTransferResourceCount
     */
    public int getActiveTransferResourceCount() {
        try {
            this.activeTransferResourceCount = ReliableFileTransferDbAdapter.
                getDbAdapter().getActiveRequestIds().size();
        } catch (RftDBException dbEx) {
            if (logger.isDebugEnabled()) {
                logger.error(i18n.getMessage("dbRetrieveErr"), dbEx);
            }
        }
        return this.activeTransferResourceCount;
    }
}
