/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */

package org.globus.transfer.reliable.service.factory;

import org.apache.axis.message.addressing.EndpointReferenceType;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.globus.delegation.DelegationConstants;
import org.globus.wsrf.container.ServiceHost;
import org.globus.wsrf.impl.ReflectionResourceProperty;
import org.globus.wsrf.impl.SimpleResourcePropertyMetaData;
import org.globus.wsrf.utils.AddressingUtils;

import org.globus.transfer.reliable.service.RFTConstants;

public class DelegationServiceEndpoint extends ReflectionResourceProperty {
    private EndpointReferenceType delegationEPR = null;
    private static Log logger = LogFactory.getLog(DelegationServiceEndpoint
            .class.getName());
    private static DelegationServiceEndpoint resourceProperty = null;

    public static DelegationServiceEndpoint getInstance() 
    throws Exception {
        if (DelegationServiceEndpoint.resourceProperty == null) {
            DelegationServiceEndpoint.resourceProperty = new 
                DelegationServiceEndpoint();
        }
        return DelegationServiceEndpoint.resourceProperty;
    }

    private DelegationServiceEndpoint() throws Exception {
        super(new SimpleResourcePropertyMetaData(
            RFTConstants.DELEGATION_ENDPOINT_FACTORY));
        setObject(this);
        setPropertyName("DelegationServiceEndpoint");
        initialize();
    }
    
    public EndpointReferenceType getDelegationServiceEndpoint() {
        if (this.delegationEPR == null) {
            String serviceUrl = null;
            try {
                serviceUrl = ServiceHost.getBaseURL()
                    + DelegationConstants.FACTORY_PATH;
                this.delegationEPR = AddressingUtils.createEndpointReference(
                        serviceUrl, null);
            } catch (Exception e) {
                logger.error("Unable to set delegation service epr",e);
            }
        }
        return this.delegationEPR;
    }
}
