/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.transfer.reliable.service.factory;

import javax.xml.namespace.QName;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.globus.transfer.reliable.service.database.ReliableFileTransferDbAdapter;
import org.globus.transfer.reliable.service.database.RftDBException;
import org.globus.util.I18n;
import org.globus.wsrf.impl.ReflectionResourceProperty;
import org.globus.wsrf.impl.SimpleResourcePropertyMetaData;

/**
 *  Aggregation RP for total number of bytes transferres by this factory
 */
public class TotalNumberOfBytesTransferred extends ReflectionResourceProperty {
    private static Log logger = LogFactory.
        getLog(TotalNumberOfBytesTransferred.class.getName());
    private static I18n i18n =
        I18n.getI18n("org.globus.transfer.reliable.errors",
                TotalNumberOfBytesTransferred.class.getClassLoader());
    private long totalNumberOfBytesTransferred = 0;
    
    /**
     * @param qname
     * @throws Exception
     */
    public TotalNumberOfBytesTransferred(QName qname)
    throws Exception {
        super(new SimpleResourcePropertyMetaData(qname));
        setObject(this);
        setPropertyName("TotalNumberOfBytesTransferred");
        initialize();    
    }
    
    /**
     * @return totalNumberOfBytesTransferred
     */
    public long getTotalNumberOfBytesTransferred() {
        try {
            this.totalNumberOfBytesTransferred =
                ReliableFileTransferDbAdapter.getDbAdapter().
                getTotalNumberOfBytes();
        } catch (RftDBException dbEx) {
            if(logger.isDebugEnabled()) {
                logger.error(i18n.getMessage("dbRetrieveErr"), dbEx);
            }
        }
        return this.totalNumberOfBytesTransferred;
    }
    /**
     * @param 
     * totalNumberOfBytesTransferred The totalNumberOfBytesTransferred to set.
     */
    public void setTotalNumberOfBytesTransferred(
            long totalNumberOfBytesTransferred) {
        this.totalNumberOfBytesTransferred = totalNumberOfBytesTransferred;
    }
}
