/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.transfer.reliable.service.factory;

import javax.xml.namespace.QName;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.globus.transfer.reliable.service.database.ReliableFileTransferDbAdapter;
import org.globus.transfer.reliable.service.database.RftDBException;
import org.globus.util.I18n;
import org.globus.wsrf.impl.ReflectionResourceProperty;
import org.globus.wsrf.impl.SimpleResourcePropertyMetaData;

/**
 *
 */
public class TotalNumberOfTransfersRP extends ReflectionResourceProperty {
    private static Log logger = 
        LogFactory.getLog(TotalNumberOfTransfersRP.class.getName());
    private static I18n i18n =
        I18n.getI18n("org.globus.transfer.reliable.errors",
                TotalNumberOfTransfersRP.class.getClassLoader());
    private long totalNumberOfTransfers = 0;
    
    /**
     * @param qname
     * @throws Exception
     */
    public TotalNumberOfTransfersRP(QName qname)
        throws Exception {
        super(new SimpleResourcePropertyMetaData(qname));
        setObject(this);
        setPropertyName("TotalNumberOfTransfersCount");
        initialize();
    }

    /**
     * @param size
     */
    public void addTransfers(int size) {
        this.totalNumberOfTransfers += size;
    }

    /**
     * @return <code>totalNumberofTransfers</code>
     */
    public long getTotalNumberOfTransfersCount() {
        try {
            this.totalNumberOfTransfers = ReliableFileTransferDbAdapter.
            getDbAdapter().getTotalNumberOfTransfers();
        } catch (RftDBException dbEx) {
            if (logger.isDebugEnabled()) {
                logger.error(i18n.getMessage("dbRetrieveErr"),dbEx);
            }
        }
        return this.totalNumberOfTransfers;
    }
}
