/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.transfer.reliable.service.test;

import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;


import javax.xml.namespace.QName;
import javax.xml.rpc.Stub;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import javax.xml.soap.SOAPElement;

import org.apache.axis.message.MessageElement;
import org.apache.axis.message.addressing.Address;
import org.apache.axis.message.addressing.EndpointReferenceType;
import org.apache.axis.message.addressing.ReferencePropertiesType;
import org.globus.rft.generated.BaseRequestType;
import org.globus.rft.generated.OverallStatus;
import org.globus.rft.generated.ReliableFileTransferFactoryPortType;
import org.globus.rft.generated.ReliableFileTransferPortType;
import org.globus.rft.generated.RequestStatusType;
import org.globus.rft.generated.TotalBytesTransferredResource;
import org.globus.rft.generated.Start;
import org.globus.rft.generated.StartOutputType;
import org.globus.rft.generated.GetStatus;
import org.globus.rft.generated.GetStatusResponse;
import org.globus.rft.generated.Cancel;
import org.globus.rft.generated.CancelResponse;
import org.globus.rft.generated.GetStatusSet;
import org.globus.rft.generated.GetStatusSetResponse;
import org.globus.rft.generated.TransferStatusType;
import org.globus.rft.generated.service.ReliableFileTransferFactoryServiceAddressingLocator;
import org.globus.rft.generated.service.ReliableFileTransferServiceAddressingLocator;
import org.globus.transfer.reliable.service.RFTConstants;
import org.globus.transfer.reliable.client.BaseRFTClient;
import org.globus.transfer.reliable.client.ReliableFileDeleteClient;
import org.globus.transfer.reliable.client.ReliableFileTransferClient;
import org.globus.wsrf.NotificationConsumerManager;
import org.globus.wsrf.encoding.ObjectDeserializer;
import org.globus.wsrf.security.Constants;
import org.globus.wsrf.impl.security.authorization.SelfAuthorization;
import org.globus.wsrf.impl.security.util.AuthUtil;
import org.globus.wsrf.utils.AnyHelper;
import org.globus.wsrf.utils.XmlUtils;
import org.oasis.wsrf.properties.GetMultipleResourcePropertiesResponse;
import org.oasis.wsrf.properties.GetMultipleResourceProperties_Element;
import org.oasis.wsrf.properties.GetResourcePropertyResponse;
import org.w3c.dom.Element;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Unit tests for RFT
 */
public class RFTTest extends TestCase {
    private static Log logger = LogFactory.getLog(RFTTest.class.getName());
    public static String GLOBUS_LOCATION = System.getProperty("GLOBUS_LOCATION");
    public static String GLOBUS_SHARE= "/share/globus_wsrf_rft_test/";
    public static String PREFIX = GLOBUS_LOCATION + GLOBUS_SHARE;

    private final static String G_L = System.getProperty("GLOBUS_LOCATION");

    static final String SERVICE_URL_ROOT = "/wsrf/services/";

    static NotificationConsumerManager consumer = null;

    private static ReliableFileTransferFactoryServiceAddressingLocator 
    rftFactoryLocator = new ReliableFileTransferFactoryServiceAddressingLocator();

    private static ReliableFileTransferFactoryPortType factoryPort;

    private static ReliableFileTransferServiceAddressingLocator rftLocator = 
        new ReliableFileTransferServiceAddressingLocator();

    private static String source_file = "/tmp/rftTest.tmp";

    private static String dest_file = "/tmp/rftTest_Done.tmp";

    private static String source_file2 = "/tmp/RFT_TEST/rftTest.tmp";

    private static String rftTestDir = "/tmp/RFT_TEST";

    private boolean dir_transfer = false;

    private boolean tpt_transfer = false;

    private boolean set_transfer = false;

    private boolean delete = false;

    private boolean empty_dir = false;

    private boolean allOrNone = false;

    private boolean delete_dir = false;

    
    protected final static String TRANSFER = PREFIX + "/transfer.xfr";

    private static String DIR_TRANSFER = PREFIX + "/dir_transfer.xfr";

    private static String TPT_TRANSFER = PREFIX + "/tpt_transfer.xfr";

    private static String TRANSFER_SET = PREFIX + "/transfer_set.xfr";

    private static String DELETE_SET = PREFIX + "/delete.xfr";

    private static String EMPTY_DIR = PREFIX + "/empty_dir.xfr";

    private static String ALL_OR_NONE = PREFIX + "/all_or_none.xfr";
    
    private static String DELETE_DIR = PREFIX + "/delete_dir.xfr";

    private static String host = "127.0.0.1";

    private static int port = 8080;
    

    /**
     * Constructor for the RFTTest object
     * 
     * @param name
     */
    public RFTTest(String name) {
        super(name);
    }

    /**
     * The main program for the RFTTest class
     * 
     * @param as
     *            The command line arguments
     */
    public static void main(String[] as) {
        logger.debug("MAIN");
        TestSuite testSuite = null;

        if (testSuite == null) {
            testSuite = new TestSuite(RFTTest.class);
        }

        junit.textui.TestRunner.run(testSuite);
    }

    /**
     * A unit test suite for JUnit
     * 
     * @return The test suite
     */
    public static Test suite() {
        return new TestSuite(RFTTest.class);

    }

    /**
     * A unit test for JUnit
     * 
     * @exception Exception
     */
    public void testTransferEPR() throws Exception {
        logger.debug("testTransferEPR()");
        EndpointReferenceType transferEPR = createRFT(TRANSFER);
        //======= test EPR of transfer
        assertTrue(transferEPR != null);
        assertTrue(transferEPR.getAddress() != null);
        String address = transferEPR.getAddress().toString();
        System.out.println("rft service URL:" + address);
        ReferencePropertiesType props = transferEPR.getProperties();
        assertTrue(props.get_any() != null);
        assertTrue(props.get_any().length > 0);
        for (int i = 0; i < props.get_any().length; i++) {
            String referenceProperty = XmlUtils.toString(AnyHelper
                    .toElement(props.get_any()[i]));
            System.out.println("reference property #" + i + "\n"
                    + referenceProperty);
        }
        assertTrue(props.get_any()[0] != null);
        String keyValue = props.get_any()[0].getValue();
        System.out.println("transfer resource ID = " + keyValue);

    }

    /**
     * test for multiple starts fault
     * 
     * @throws Exception
     */
    public void testRepeatedlyStarted() throws Exception {
        logger.debug("testRepeatedlyStarted");
        ReliableFileTransferPortType rft = getRFTPort();
        StartOutputType startResponse = rft.start(new Start());
        assertTrue(startResponse != null);
        try {
            StartOutputType startResponse1 = rft.start(new Start());
        } catch(Exception es) {
            assertTrue(true);
        }
    }

    /**
     * Tests the actual transfer Need to create a file in /tmp and then transfer
     * it and test for the transfer
     * 
     * @exception Exception
     */
    public void testTransfer() throws Exception {
        logger.debug("testTransfer");
        //check if destination already exist
        cleanupDestination();
        setupFiles();
        Thread.sleep(10000);
        ReliableFileTransferPortType rft = getRFTPort();
        StartOutputType startResponse = rft.start(new Start());
        assertTrue(startResponse != null);
        File destFile = new File(dest_file);
        destFile.deleteOnExit();
        Thread.sleep(50000);
        assertTrue(destFile.exists());

    }

    /**
     * Test the getStatus operation
     * 
     * @throws Exception
     */
    public void testGetStatus() throws Exception {
        logger.debug("testGetStatus");
        setupFiles();
        ReliableFileTransferPortType rft = getRFTPort();
        StartOutputType startResponse = rft.start(new Start());
        assertTrue(startResponse != null);
        Thread.sleep(50000);
        GetStatusResponse getStatusResponse = rft.getStatus(new GetStatus(
                "gsiftp://localhost:2811/tmp/rftTest.tmp"));
        assertTrue(getStatusResponse != null);
        TransferStatusType transferStatus = getStatusResponse
                .getTransferStatus();
        System.out.println("Status: " + transferStatus.getStatus().toString());
        assertTrue(transferStatus != null);
    }

    /**
     * Tests the getStatusSet of rft portType
     * 
     * @throws Exception
     */
    public void testGetStatusSet() throws Exception {
        logger.debug("testGetStatusSet");
        setupFiles();
        this.set_transfer = true;
        ReliableFileTransferPortType rft = getRFTPort();
        StartOutputType startResponse = rft.start(new Start());
        assertTrue(startResponse != null);
        Thread.sleep(50000);
        GetStatusSetResponse getStatusSetResponse = rft
                .getStatusSet(new GetStatusSet(1, 9));
        assertTrue(getStatusSetResponse != null);
        TransferStatusType transferStatus[] = getStatusSetResponse
                .getTransferStatusSet();
        int length = transferStatus.length;
        System.out.println("Length of status set : " + length);
        assertTrue(length == 9);
        for (int i = 0; i < transferStatus.length; i++) {
            System.out.println("Status: "
                    + transferStatus[i].getStatus().toString());
            assertTrue(transferStatus[i] != null);
        }
        assertTrue(transferStatus != null);
    }

    /**
     * Tests the cancel operation
     * 
     * @throws Exception
     */
    public void testCancel() throws Exception {
        logger.debug("testCancel");
        setupFiles();
        this.set_transfer = true;
        ReliableFileTransferPortType rft = getRFTPort();
        StartOutputType startResponse = rft.start(new Start());
        assertTrue(startResponse != null);
        Thread.sleep(100);
        //Cancel the request
        Cancel cancelParams = new Cancel(-1, -1);
        CancelResponse response = rft.cancel(cancelParams);
        assertTrue(response != null);
        Thread.sleep(1000);
        GetStatusSetResponse getStatusSetResponse = rft
                .getStatusSet(new GetStatusSet(1, 9));
        assertTrue(getStatusSetResponse != null);
        TransferStatusType transferStatus[] = getStatusSetResponse
                .getTransferStatusSet();
        int length = transferStatus.length;
        System.out.println("Length of status set : " + length);
        assertTrue(length == 9);
        for (int i = 0; i < transferStatus.length; i++) {
            String status = transferStatus[i].getStatus().toString();
            System.out.println("Status: " + status);
            assertTrue(status.equals("Cancelled"));
        }
        assertTrue(transferStatus != null);
    }

    /**
     * tests the directory transfer
     * 
     * @throws Exception
     */
    public void testDirTransfer() throws Exception {
        logger.debug("testDirTransfer");
        File destFile = new File(source_file);
        destFile.delete();
        this.dir_transfer = true;
        setupTmpDir();
        Thread.sleep(10000);
        ReliableFileTransferPortType rft = getRFTPort();
        StartOutputType startResponse = rft.start(new Start());
        assertTrue(startResponse != null);
        Thread.sleep(50000);
        destFile = new File(source_file);
        assertTrue(destFile.exists());
    }

    /**
     * tests the third-party transfer
     * 
     * @throws Exception
     */
    public void testTptTransfer() throws Exception {
        logger.debug("testTptTransfer");
        //check if destination already exist
        this.tpt_transfer = true;
        cleanupDestination();
        setupFiles();
        Thread.sleep(10000);
        ReliableFileTransferPortType rft = getRFTPort();
        StartOutputType startResponse = rft.start(new Start());
        assertTrue(startResponse != null);
        File destFile = new File(dest_file);
        destFile.deleteOnExit();
        Thread.sleep(50000);
        assertTrue(destFile.exists());

    }

    /**
     * A unit test for JUnit
     * 
     * @exception Exception
     */
    public void testOverallStatus() throws Exception {
        logger.debug("testOverallStatus");
        setupFiles();
        ReliableFileTransferPortType rft = getRFTPort();
        StartOutputType startResponse = rft.start(new Start());
        GetResourcePropertyResponse response = rft
                .getResourceProperty(RFTConstants.OVERALL_STATUS_RESOURCE);
        SOAPElement[] any = response.get_any();
        assertTrue(any != null);
        assertTrue(any.length > 0);
        OverallStatus status = (OverallStatus) ObjectDeserializer.toObject(
                any[0], OverallStatus.class);
        System.out.println("status : " + status.getTransfersFinished());
        assertTrue(status != null);
    }

    /**
     * Test for RequestStatus RP
     * 
     * @throws Exception
     */
    public void testRequestStatusRP() throws Exception {
        logger.debug("testRequestStatusRP");
        setupFiles();
        this.set_transfer = true;
        ReliableFileTransferPortType rft = getRFTPort();
        StartOutputType startResponse = rft.start(new Start());
        Thread.sleep(3000);
        GetResourcePropertyResponse response = rft
                .getResourceProperty(RFTConstants.REQUEST_STATUS_RESOURCE);
        SOAPElement[] any = response.get_any();
        assertTrue(any != null);
        assertTrue(any.length > 0);
        RequestStatusType status = (RequestStatusType) ObjectDeserializer
                .toObject(any[0], RequestStatusType.class);
        System.out.println("request status : "
                + status.getRequestStatus().getValue());
        assertTrue(status != null);
    }

    /**
     * A unit test for JUnit
     * 
     * @exception Exception
     */
    public void testTotalBytesTransferred() throws Exception {
        logger.debug("testTotalBytesTransferred");
        ReliableFileTransferPortType rft = getRFTPort();
        StartOutputType startResponse = rft.start(new Start());
        Thread.sleep(10000);
        GetResourcePropertyResponse response = rft
                .getResourceProperty(
                        RFTConstants.TOTAL_BYTES_RESOURCE);
        SOAPElement[] any = response.get_any();
        assertTrue(any != null);
        assertTrue(any.length > 0);
        TotalBytesTransferredResource totalBytes = (TotalBytesTransferredResource) ObjectDeserializer
                .toObject(any[0], TotalBytesTransferredResource.class);
        System.out.println("Total bytes: " + totalBytes.getTotalBytes());
        assertTrue(totalBytes.getTotalBytes() > 0);
        //    cleanupDestination();
        //   cleanupSource();
    }

    /**
     * A unit test for JUnit
     * 
     * @exception Exception
     */
    public void testMultipleResourceProperties_Factory_RFT() throws Exception {
        logger.debug("testMultipleResourceProperties_Factory_RFT");
        ReliableFileTransferFactoryPortType factoryPort = BaseRFTClient
                .getFactoryPort(TestConfig.FACTORY_URL);
        BaseRFTClient.setSecurity((Stub)factoryPort);
        QName[] propNames = new QName[] { 
                RFTConstants.ACTIVE_RESOURCE_COUNT_FACTORY,
                RFTConstants.TOTAL_TRANSFER_COUNT_FACTORY,
                RFTConstants.DELEGATION_ENDPOINT_FACTORY,
                RFTConstants.RFT_FACTORY_START_TIME, 
                RFTConstants.TOTAL_BYTES_FACTORY,
                RFTConstants.TOTAL_ACTIVE_TRANSFERS_FACTORY };
        GetMultipleResourceProperties_Element requestProp = new GetMultipleResourceProperties_Element();
        requestProp.setResourceProperty(propNames);
        GetMultipleResourcePropertiesResponse responseProp = factoryPort
                .getMultipleResourceProperties(requestProp);
        MessageElement[] any = responseProp.get_any();
        assertTrue(any != null);

        System.out.println("Resource Properties of factory:\n");
        for (int i = 0; i < any.length; i++) {
            Element rpElement = AnyHelper.toElement(any[i]);
            System.out.println(XmlUtils.toString(rpElement));
        }
        assertTrue(any[0] != null);
        assertTrue(any[1] != null);
        assertTrue(any.length >= 6);
    }

    /**
     * A unit test for JUnit
     * 
     * @exception Exception
     */
    public void testMultipleResourceProperties_Resource_RFT() throws Exception {
        logger.debug("testMultipleResourceProperties_Resource_RFT");
        ReliableFileTransferPortType rft = getRFTPort();
        StartOutputType startResponse = rft.start(new Start());
        Thread.sleep(30000);

        QName[] propNamesInstance = new QName[] { 
                RFTConstants.OVERALL_STATUS_RESOURCE };
        GetMultipleResourceProperties_Element requestPropInstance = new GetMultipleResourceProperties_Element();
        requestPropInstance.setResourceProperty(propNamesInstance);
        GetMultipleResourcePropertiesResponse responsePropInstance = rft
                .getMultipleResourceProperties(requestPropInstance);
        SOAPElement[] anyInstance = responsePropInstance.get_any();

        assertTrue(anyInstance.length >= 1);

    }

    /**
     * Tests the delete functionality
     * @throws Exception
     */
    public void testDelete() throws Exception {
        logger.debug("testDelete");
        this.delete = true;
        ReliableFileTransferPortType rft = getRFTPort();
        StartOutputType startResponse = rft.start(new Start());
        assertTrue(startResponse != null);
        File destFile = new File(dest_file);
        Thread.sleep(5000);
        assertTrue(!destFile.exists());
    }

    public void testEmptyDir() throws Exception {
        logger.debug("testEmptyDir");
        this.empty_dir = true;
        // clean up stuff from /tmp/RFT_TEST/
        File dirCleanup = new File(rftTestDir);
        dirCleanup.delete();
        //Create it again
        dirCleanup.mkdir();
        ReliableFileTransferPortType rft = getRFTPort();
        StartOutputType startResponse = rft.start(new Start());
        assertTrue(startResponse != null);
        String dest_file1 = "/tmp/RFT_TEST2/rftTest_Done.tmp";
        File destFile = new File(dest_file1);
        Thread.sleep(5000);
        assertTrue(destFile.exists());
    }
    public void testDeleteDir() throws Exception {
        logger.debug("testDeleteDir");
        this.delete_dir = true;
        ReliableFileTransferPortType rft = getRFTPort();
        StartOutputType startResponse = rft.start(new Start());
        assertTrue(startResponse != null);
        File deleteDir = new File("/tmp/RFT_TEST/");
        Thread.sleep(5000);
        assertTrue(!deleteDir.exists());
    }

    public void testAllOrNone() throws Exception {
        logger.debug("testAllOrNone");
        this.allOrNone = true;
        ReliableFileTransferPortType rft = getRFTPort();
        StartOutputType startResponse = rft.start(new Start());
        assertTrue(startResponse != null);
        Thread.sleep(10000);
        File destFile1 = new File("/tmp/rftTest_Done1.tmp");
        assertTrue(!destFile1.exists());
    }

    private void cleanup() throws Exception {
        logger.debug("cleanup");
        File dir1 = new File(rftTestDir);
        dir1.delete();
        File dir2 = new File("/tmp/RFT_TEST2");
        dir2.delete();
        File testFile = new File(source_file);
        testFile.delete();
        File testFile2 = new File(dest_file);
        testFile2.delete();
    }
    /** Sets up temp files
     * @exception Exception
     */
    private static void setupFiles() throws Exception {
        logger.debug("setupFiles");
        File sourceFile = new File(source_file);
        File copyFile = new File(TRANSFER);
        FileReader in = new FileReader(copyFile);
        FileWriter out = new FileWriter(sourceFile);
        int c;

        while ((c = in.read()) != -1) {
            out.write(c);
        }

        in.close();
        out.close();
    }

    private void setupTmpDir() throws Exception {
        logger.debug("setupTmpDir");
        File dirFile = new File(rftTestDir);
        boolean dirExists = dirFile.mkdir();
        if (dirExists) {
            File sourceFile = new File(source_file2);
            File copyFile = new File(TRANSFER);
            FileReader in = new FileReader(copyFile);
            FileWriter out = new FileWriter(sourceFile);
            int c;

            while ((c = in.read()) != -1) {
                out.write(c);
            }
            in.close();
            out.close();
        }
    }

    /**
     * Cleans up the destination file 
     * @exception Exception
     */
    private void cleanupDestination() throws Exception {
        logger.debug("cleanupDestination");
        File destFile = new File(dest_file);
        if (destFile.exists()) {
            destFile.delete();
        }
    }

    /**
     * Gets the rFTPort attribute of the RFTTest object
     * 
     * @return The rFTPort value
     * @exception Exception
     */
    public ReliableFileTransferPortType getRFTPort() throws Exception {
        EndpointReferenceType rftEPR;
        if (this.dir_transfer) {
            rftEPR = createRFT(DIR_TRANSFER);
        } else if (this.tpt_transfer) {
            rftEPR = createRFT(TPT_TRANSFER);
        } else if (this.set_transfer) {
            rftEPR = createRFT(TRANSFER_SET);
        } else if (this.delete) {
            rftEPR = createRFT(DELETE_SET);
        } else if (this.empty_dir) {
            rftEPR = createRFT(EMPTY_DIR);
        } else if(this.allOrNone) {
            rftEPR = createRFT(ALL_OR_NONE);
       } else if(this.delete_dir) {
           rftEPR = createRFT(DELETE_DIR);
       } else {
            rftEPR = createRFT(TRANSFER);
        }

        rftEPR.setAddress(new Address(TestConfig.SERVICE_URL));
        ReliableFileTransferPortType rft = rftLocator
                .getReliableFileTransferPortTypePort(rftEPR);
        BaseRFTClient.setSecurity((Stub) rft);
        //For secure notifications
        return rft;
    }

    /**
     * 
     * @param reqFile
     * 
     * @return EndpointReferenceType 
     * @exception Exception
     */
    private EndpointReferenceType 
        createRFT(String reqFile) throws Exception {
        setOptions();
        EndpointReferenceType credEPR = BaseRFTClient.delegateCredential(
                TestConfig.HOST, TestConfig.PORT);
        BaseRequestType baseRequestType;
        if (this.delete) {
            baseRequestType = ReliableFileDeleteClient.getDeleteRequest(
                    reqFile, credEPR);
        } else if(this.delete_dir) {
            baseRequestType = ReliableFileDeleteClient.getDeleteRequest(
                reqFile, credEPR);
        } else {
            baseRequestType = ReliableFileTransferClient.getTransfer(reqFile,
                    credEPR);
        }
        return BaseRFTClient.createRFT(TestConfig.FACTORY_URL, baseRequestType);

    }

    public static void setOptions() {
        logger.debug("setOptions");
        BaseRFTClient.HOST = TestConfig.HOST;
        BaseRFTClient.PORT = TestConfig.PORT;
        BaseRFTClient.PROTOCOL = TestConfig.PROTOCOL;
        //Auth
        String authType = TestConfig.authType;
        if (authType.equals("GSI_SEC_MSG")) {
            BaseRFTClient.authType = Constants.GSI_SEC_MSG;
        } else if (authType.equals("GSI_SEC_CONV")) {
            BaseRFTClient.authType = Constants.GSI_SEC_CONV;
        } else if (authType.equals("GSI_TRANSPORT")) {
            BaseRFTClient.authType = Constants.GSI_TRANSPORT;
        }
        BaseRFTClient.authVal = Constants.SIGNATURE;
        //Authz
        BaseRFTClient.authzVal = AuthUtil
                .getClientAuthorization(TestConfig.authzVal);
        TestConfig.printConfig();
    }

}

