/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */

#ifndef GLOBUS_I_NOTIFICATION_CONSUMER_H
#define GLOBUS_I_NOTIFICATION_CONSUMER_H

#include "globus_notification_consumer.h"

#ifndef GLOBUS_DONT_DOCUMENT_INTERNAL

/**
 * Make a NULL_PARAM result
 */
#define GLOBUS_NOTIFICATION_CONSUMER_ERROR_NULL_PARAM(cause) \
    globus_error_put( \
        globus_error_construct_error( \
           GLOBUS_NOTIFICATION_CONSUMER_MODULE, \
           cause, \
           GLOBUS_NOTIFICATION_CONSUMER_ERROR_TYPE_NULL_PARAM, \
           __FILE__, \
           NULL, \
           __LINE__, \
           "Null parameter"))

/**
 * Make an UNKNOWN_RESOURCE result
 */
#define GLOBUS_NOTIFICATION_CONSUMER_ERROR_UNKNOWN_RESOURCE(cause) \
    globus_error_put( \
        globus_error_construct_error( \
           GLOBUS_NOTIFICATION_CONSUMER_MODULE, \
           cause, \
           GLOBUS_NOTIFICATION_CONSUMER_ERROR_TYPE_UNKNOWN_RESOURCE, \
           __FILE__, \
           NULL, \
           __LINE__, \
           "Unknown resource"))

/**
 * Make a DESERIALIZING_TOPIC result
 */
#define GLOBUS_NOTIFICATION_CONSUMER_ERROR_DESERIALIZING_TOPIC(cause) \
    globus_error_put( \
        globus_error_construct_error( \
           GLOBUS_NOTIFICATION_CONSUMER_MODULE, \
           cause, \
           GLOBUS_NOTIFICATION_CONSUMER_ERROR_TYPE_DESERIALIZING_TOPIC, \
           __FILE__, \
           NULL, \
           __LINE__, \
           "Unable to deserialize topic"))

/**
 * Make an UNABLE_TO_CREATE_CONSUMER result
 */
#define GLOBUS_NOTIFICATION_CONSUMER_ERROR_UNABLE_TO_CREATE_CONSUMER(cause) \
    globus_error_put( \
        globus_error_construct_error( \
           GLOBUS_NOTIFICATION_CONSUMER_MODULE, \
           cause, \
           GLOBUS_NOTIFICATION_CONSUMER_ERROR_TYPE_UNABLE_TO_CREATE_CONSUMER, \
           __FILE__, \
           NULL, \
           __LINE__, \
           "Unable to create consumer"))
/**
 * Namespace for the BaseNotification port type
 */
#define NOTIFICATION_NS \
    "http://www.globus.org/docs.oasis-open.org/wsn/2004/06/wsn-WS-BaseNotification-1.2-draft-01.wsdl"

/**
 * WS-Addressing Namespace
 */
#define WSADDRESSING_NS \
    "http://schemas.xmlsoap.org/ws/2004/03/addressing"

/**
 * Notification consumer resource-specific data structure.
 */
typedef struct
{
    /* The state machine is implemented in two places:
     * globus_notification_destroy_consumer() and Consumer_Notify_impl().
     * The state machine restricts the notification service to
     * dispatching one callback per resource at a time.
     */

    enum {
        /**
         * Consumer is totally idle. A call to destroy or notify will
         * happen immediately.
         */
        CONSUMER_OK,
        /**
         * In one thread, the consumer has unlocked in order that a 
         * callback be dispatched.
         * - Other calls to Consumer_Notify_impl() will block until
         *   the consumer is no longer active
         * - Calls to destroy will set the state to
         *   CONSUMER_ACTIVE_DESTROY_CALLED and then block.
         * When the callback returns, if the state isn't changed to
         * CONSUMER_ACTIVE_DESTROY_CALLED, then the state will return to
         * CONSUMER_OK. in either case, the condition variable will be
         * signalled.
         */
        CONSUMER_ACTIVE,
        /**
         * A thread has called globus_notification_destroy_consumer() while
         * a callback was active. The destroy thread will block. Another
         * thread which calls globus_notification_destroy_consumer() will
         * also block until the first to call that function destroys the
         * resource.
         */
        CONSUMER_ACTIVE_DESTROY_CALLED,
        /**
         * A thread is blocking in globus_resource_finish_and_destroy() until
         * other threads which might be waiting in Consumer_Notify_impl() or
         * globus_notification_destroy_consumer() realize the state has changed
         * and call globus_resource_finish()
         */
        CONSUMER_DESTROYED
    }                                   state;
    /** User-specified callback */
    globus_notification_consumer_func_t callback;
    /** User-specified callback argument */
    void *                              arg;
}
globus_i_notification_consumer_t;

extern globus_mutex_t globus_i_notification_lock;
extern globus_cond_t globus_i_notification_cond;
#endif /* GLOBUS_DONT_DOCUMENT_INTERNAL */

#endif /* GLOBUS_I_NOTIFICATION_CONSUMER_H */
