/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */

#include "CounterService_client.h"
#include "globus_service_engine.h"
#include "globus_notification_consumer.h"
#include "globus_xml_buffer.h"

#include "notif_ResourcePropertyValueChangeNotificationElementType.h"


static
void
notify_callback(
    void *                              arg,
    wsnt_TopicExpressionType *          topic,
    wsa_EndpointReferenceType *         producer,
    xsd_anyType *                       message);

static
void
engine_stop_callback (
    globus_result_t                     result,
    globus_service_engine_t             engine,
    void *                              args);

globus_cond_t cond;
globus_mutex_t mutex;
globus_service_engine_t engine;

int notify_value = 0;
globus_cond_t notify_cond;
globus_mutex_t notify_mutex;

int main(int argc, char * argv[])
{
    globus_soap_message_attr_t          attrs;
    globus_result_t                     result = GLOBUS_SUCCESS;
    CounterService_client_handle_t      client_handle;
    createCounterType                   createCounter;
    createCounterResponseType *         createCounterResponse = NULL;
    int *                               addResponse = NULL;
    int                                 add_value;
    int                                 add_result = 0;
    int                                 fault_type;
    xsd_any *                           fault;
    int                                 i = 0;
    globus_xml_buffer *                 counterKey = NULL;
    xsd_any_array                       referenceProperties;
    xsd_QName *                         topic_qname;
    wsnt_SubscribeType *                subscribe;
    wsnt_SubscribeResponseType *        subscribe_response = NULL;
    globus_bool_t                       done = GLOBUS_FALSE;

    if(argc < 2)
    {
        fprintf(stderr, "usage: test-counter <URL>\n");
        exit(1);
    }

    globus_module_activate(GLOBUS_SOAP_MESSAGE_MODULE);
    globus_module_activate(GLOBUS_NOTIFICATION_CONSUMER_MODULE);
    globus_module_activate(COUNTERSERVICE_MODULE);

    globus_mutex_init(&mutex, NULL);
    globus_cond_init(&cond, NULL);

    result = CounterService_client_init(&client_handle, NULL, NULL);
    if(result != GLOBUS_SUCCESS)
    {
        goto exit;
    }

/*    result = CounterService_client_attr_set(
        client_handle,
        GLOBUS_SOAP_MESSAGE_AUTHENTICATION_METHOD_KEY,
        NULL,
        NULL,
        (void *)GLOBUS_SOAP_MESSAGE_AUTH_SECURE);
*/
    result = CounterPortType_createCounter(
        client_handle,
        argv[1],
        &createCounter,
        &createCounterResponse,
        (CounterPortType_createCounter_fault_t *)&fault_type,
        &fault);
    if(result != GLOBUS_SUCCESS)
    {
        goto exit;
    }

    if(createCounterResponse && 
       createCounterResponse->EndpointReference.ReferenceProperties)
    {
        referenceProperties = 
        createCounterResponse->EndpointReference.ReferenceProperties->any;

        if(referenceProperties.length != 1)
        {
            fprintf(stderr, 
                    "ERROR: Invalid number (%d) of counterKey elements"
                    " in createCounterResponse\n", 
                    referenceProperties.length);
            exit(1);
        }

        printf("COUNTER KEY TYPE: {%s}%s\n", 
               referenceProperties.elements[0].any_info->type->Namespace, 
               referenceProperties.elements[0].any_info->type->local);
               
        if(referenceProperties.elements[0].any_info->type == 
           xsd_string_info.type)
        {
            printf("COUNTER KEY: %s\n", 
                   *(xsd_string *)referenceProperties.elements[0].value);
        }
        else if(referenceProperties.elements[0].any_info->type == 
                globus_xml_buffer_info.type)
        {
            counterKey = (globus_xml_buffer *)
            referenceProperties.elements[0].value;
            if(counterKey)
            {
                printf("COUNTER KEY: %*s\n", 
                       counterKey->length, counterKey->buffer);
            }
        }
    }

    wsnt_SubscribeType_init(&subscribe);

    result = globus_soap_message_attr_init(
	&attrs);

    globus_soap_message_attr_set(
        attrs,
        GLOBUS_SOAP_MESSAGE_AUTHZ_METHOD_KEY,
        NULL, NULL,
        (void *) GLOBUS_SOAP_MESSAGE_AUTHZ_HOST);

    result = globus_service_engine_init(
            &engine,
            attrs,
            NULL,
            NULL,
            GLOBUS_TRUE);

    globus_soap_message_attr_destroy(attrs);

    result = globus_notification_create_consumer(
        &subscribe->ConsumerReference,
        engine,
        notify_callback,
        &add_result);

    if (result != GLOBUS_SUCCESS)
    {
        printf("Error creating notification consumer: %s\n",
                globus_object_printable_to_string(globus_error_peek(result)));
    }

    xsd_any_init(&subscribe->TopicExpression.any);

    subscribe->TopicExpression.any->registry = NULL;
    subscribe->TopicExpression.any->any_info = &xsd_QName_contents_info;
    subscribe->TopicExpression.any->element = NULL;
    xsd_anyURI_init(&subscribe->TopicExpression._Dialect);
    *(subscribe->TopicExpression._Dialect) = globus_libc_strdup(
            "http://docs.oasis-open.org/wsn/2004/06/TopicExpression/Simple");
    xsd_QName_init(&topic_qname);
    topic_qname->Namespace = globus_libc_strdup("http://counter.com");
    topic_qname->local = globus_libc_strdup("Value");
    subscribe->TopicExpression.any->value = topic_qname;

    xsd_boolean_init(&subscribe->UseNotify);
    *(subscribe->UseNotify) = GLOBUS_TRUE;
    subscribe->Precondition = NULL;
    subscribe->Selector = NULL;
    subscribe->SubscriptionPolicy = NULL;
    subscribe->InitialTerminationTime = NULL;

    result = globus_service_engine_register_start(
        engine,
        engine_stop_callback,
        &done);

    result = CounterPortType_Subscribe_epr(
        client_handle,
        &createCounterResponse->EndpointReference,
        subscribe,
        &subscribe_response,
        (CounterPortType_Subscribe_fault_t *)&fault_type,
        &fault);
    if (result != GLOBUS_SUCCESS)
    {
        printf("subscribe failed: %s\n",
                globus_object_printable_to_string(globus_error_peek(result)));
    }

    add_value = 10;
    for(i = 0; i < 10; ++i)
    {
        result = CounterPortType_add_epr(
            client_handle,
            &createCounterResponse->EndpointReference,
            &add_value,
            &addResponse,
            (CounterPortType_add_fault_t *)&fault_type,
            &fault);
        if(result != GLOBUS_SUCCESS)
        {
            goto exit;
        }
    }

    globus_mutex_init(&notify_mutex, NULL);
    globus_cond_init(&notify_cond, NULL);
    globus_mutex_lock(&notify_mutex);
    while (notify_value < 100)
    {
        globus_cond_wait(&notify_cond, &notify_mutex);
    }
    globus_mutex_unlock(&notify_mutex);

    result = globus_service_engine_register_stop(engine);
    if(result != GLOBUS_SUCCESS)
    {
        goto exit;
    }

    globus_mutex_lock(&mutex);
    while (!done)
    {
        globus_cond_wait(&cond, &mutex);
    }
    globus_mutex_unlock(&mutex);

 exit:

    if(createCounterResponse)
    {
        createCounterResponseType_destroy(createCounterResponse);
    }

    CounterService_client_destroy(client_handle);

    if(result != GLOBUS_SUCCESS)
    {
        char *                          errorstr;

        errorstr = globus_error_print_chain(globus_error_get(result));
        fprintf(stderr, "ERROR:\n\n%s\n\n", errorstr);
        globus_module_activate(GLOBUS_NOTIFICATION_CONSUMER_MODULE);
        globus_module_activate(GLOBUS_SOAP_MESSAGE_MODULE);
        return 1;
    }

    globus_module_deactivate(COUNTERSERVICE_MODULE);
    globus_module_deactivate(GLOBUS_NOTIFICATION_CONSUMER_MODULE);
    globus_module_deactivate(GLOBUS_SOAP_MESSAGE_MODULE);

    return 0;
}

static
void
notify_callback(
    void *                              arg,
    wsnt_TopicExpressionType *          topic,
    wsa_EndpointReferenceType *         producer,
    xsd_anyType *                       message)
{
    fprintf(stdout, "NOTIFICATION RECEIVED\n");

    if(message && 
       (message->any_info == 
       &notif_ResourcePropertyValueChangeNotificationElementType_info))
    {
        notif_ResourcePropertyValueChangeNotificationElementType * rpvcne;

        rpvcne = (notif_ResourcePropertyValueChangeNotificationElementType *)
        message->value;

        if(rpvcne->ResourcePropertyValueChangeNotification.OldValue &&
           (rpvcne->ResourcePropertyValueChangeNotification.OldValue->any.
           any_info == (&CounterPortType_Value_rp_info)))
        {
            xsd_int *                   old_value;

            old_value = rpvcne->ResourcePropertyValueChangeNotification.
            OldValue->any.value;

            fprintf(stdout, "\tOLD VALUE: %d\n", *old_value);
        }

        if(rpvcne->ResourcePropertyValueChangeNotification.NewValue.any.
           any_info == (&CounterPortType_Value_rp_info))
        {
            xsd_int *                   new_value;

            new_value = rpvcne->ResourcePropertyValueChangeNotification.
            NewValue.any.value;

            fprintf(stdout, "\tNEW VALUE: %d\n", *new_value);
            
            globus_mutex_lock(&notify_mutex);
            notify_value = *new_value;
            globus_cond_signal(&notify_cond);
            globus_mutex_unlock(&notify_mutex);
        }
    }
}

static
void
engine_stop_callback (
    globus_result_t                     result,
    globus_service_engine_t             engine,
    void *                              args)
{
    globus_mutex_lock(&mutex);
    *(globus_bool_t *) args = GLOBUS_TRUE;
    globus_cond_signal(&cond);
    globus_mutex_unlock(&mutex);
}
