/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */

#ifndef WSRL_I_FAULT_TYPES_H
#define WSRL_I_FAULT_TYPES_H 1

#if WSRL_IMMEDIATE_RESOURCE_TERMINATION
#   ifndef NEED_RESOURCE_UNKNOWN_FAULT
#       define NEED_RESOURCE_UNKNOWN_FAULT 1
#   endif
#   ifndef NEED_RESOURCE_NOT_DESTROYED_FAULT
#       define NEED_RESOURCE_NOT_DESTROYED_FAULT 1
#   endif
#endif

#if WSRL_SCHEDULED_RESOURCE_TERMINATION
#   ifndef NEED_RESOURCE_UNKNOWN_FAULT
#       define NEED_RESOURCE_UNKNOWN_FAULT 1
#   endif
#   ifndef NEED_UNABLE_TO_SET_TERMINATION_TIME_FAULT
#       define NEED_UNABLE_TO_SET_TERMINATION_TIME_FAULT 1
#   endif
#   ifndef NEED_TERMINATION_TIME_CHANGE_REJECTED_FAULT
#       define NEED_TERMINATION_TIME_CHANGE_REJECTED_FAULT 1
#   endif
#endif

#if NEED_RESOURCE_UNKNOWN_FAULT
#include "wsrl_ResourceUnknownFaultType.h"

static
globus_result_t
wsrl_l_ResourceUnknownFaultType_create(
    char **                             fault_name,
    wsrl_ResourceUnknownFaultType **    fault)
{
    globus_result_t                     result;
    time_t                              timestamp = time(NULL);
    struct tm *                         tp;

    result = wsrl_ResourceUnknownFaultType_init(fault);

    if (result != GLOBUS_SUCCESS)
    {
        return result;
    }

    tp = globus_libc_gmtime_r(&timestamp, &(*fault)->Timestamp);
    globus_assert(tp != NULL);

    *fault_name = "ResourceUnknownFault";

    return result;
}
/* wsrl_l_ResourceUnknownFaultType_create() */
#endif /* NEED_RESOURCE_UNKNOWN_FAULT */

#if NEED_UNABLE_TO_SET_TERMINATION_TIME_FAULT
#include "wsrl_UnableToSetTerminationTimeFaultType.h"

static
globus_result_t
wsrl_l_UnableToSetTerminationTimeFaultType_create(
    char **                             fault_name,
    wsrl_UnableToSetTerminationTimeFaultType **
                                        fault)
{
    globus_result_t                     result;
    time_t                              timestamp = time(NULL);
    struct tm *                         tp;

    result = wsrl_UnableToSetTerminationTimeFaultType_init(fault);

    if (result != GLOBUS_SUCCESS)
    {
        return result;
    }

    tp = globus_libc_gmtime_r(
            &timestamp,
            &(*fault)->Timestamp);

    globus_assert(tp != NULL);

    *fault_name = "UnableToSetTerminationTimeFault";

    return result;
}
/* wsrl_l_UnableToSetTerminationTimeFaultType_create() */
#endif /* NEED_UNABLE_TO_SET_TERMINATION_TIME_FAULT */

#if NEED_TERMINATION_TIME_CHANGE_REJECTED_FAULT
#include "wsrl_TerminationTimeChangeRejectedFaultType.h"

static
globus_result_t
wsrl_l_TerminationTimeChangeRejectedFaultType_create(
    char **                             fault_name,
    wsrl_TerminationTimeChangeRejectedFaultType **
                                        fault)
{
    globus_result_t                     result;
    time_t                              timestamp = time(NULL);
    struct tm *                         tp;

    result = wsrl_TerminationTimeChangeRejectedFaultType_init(fault);

    if (result != GLOBUS_SUCCESS)
    {
        return result;
    }

    tp = globus_libc_gmtime_r(
            &timestamp,
            &(*fault)->Timestamp);

    globus_assert(tp != NULL);

    *fault_name = "TerminationChangeRejectedFault";

    return result;
}
/* wsrl_l_TerminationTimeChangeRejectedFault_create() */
#endif /* NEED_TERMINATION_TIME_CHANGE_REJECTED_FAULT */

#if NEED_RESOURCE_NOT_DESTROYED_FAULT
#include "wsrl_ResourceNotDestroyedFaultType.h"

static
globus_result_t
wsrl_l_ResourceNotDestroyedFaultType_create(
    char **                             fault_name,
    wsrl_ResourceNotDestroyedFaultType **
                                        fault)
{
    globus_result_t                     result;
    time_t                              timestamp = time(NULL);
    struct tm *                         tp;

    result = wsrl_ResourceNotDestroyedFaultType_init(fault);

    if (result != GLOBUS_SUCCESS)
    {
        return result;
    }

    tp = globus_libc_gmtime_r(
            &timestamp,
            &(*fault)->Timestamp);

    globus_assert(tp != NULL);

    *fault_name = "ResourceNotDestroyedFault";

    return result;
}
/* wsrl_l_resource_not_destroyed_fault() */
#endif /* NEED_RESOURCE_NOT_DESTROYED_FAULT */

#endif /* WSRL_I_FAULT_TYPES_H */
