/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */

#include "globus_soap_message.h"
#include "globus_wsrf_resource.h"
#include "globus_service_engine.h"
#include "globus_operation_provider.h"
#include "globus_service_registry.h"
#include "globus_wsrf_core_tools.h"
#include "version.h"

#include "wsrp_QueryResourcePropertiesType.h"
#include "wsrp_QueryResourcePropertiesResponseType.h"

#define WSRP_QUERY_RESOURCE_PROPERTIES 1
#define NEED_INVALID_QUERY_EXPRESSION_FAULT 0
#define NEED_QUERY_EVALUATION_ERROR_FAULT 0

#include "wsrp_i_faults.h"

/**
 * @page wsrp_QueryResourceProperties_provider QueryResourceProperties Provider
 *
 * <h2>Provider Usage Overview</h2>
 * This operation provider implements the QueryResourceProperties portType
 * defined in
 * http://docs.oasis-open.org/wsrf/2004/06/wsrf-WS-ResourceProperties-1.2-draft-01.wsdl .
 *
 * This portType consists of the @a QueryResourceProperties operation and
 * related faults.
 *
 * <b>NOTE:</b> This provider is stub which merely returns either a
 * ResourceUnknownFaultType or UnknownQueryExpressionDialectFaultType fault.
 *
 * The WSRF service engine will automatically register this provider to handle
 * the QueryResourceProperties operation for services which use the port type.
 *
 * <h2>Limitations</h2>
 * This provider is stub which merely returns either a
 * ResourceUnknownFaultType or UnknownQueryExpressionDialectFaultType fault.
 */
GlobusExtensionDeclareModule(globus_ws_resource_properties);

/* Prototypes */
static
globus_result_t
wsrp_QueryResourceProperties_init_provider(
    globus_service_engine_t             engine,
    globus_soap_message_handle_t        message,
    wsrp_QueryResourcePropertiesType *  QueryResourceProperties);

static
globus_result_t
wsrp_QueryResourceProperties_provider(
    globus_service_engine_t             engine,
    globus_soap_message_handle_t        message,
    globus_service_descriptor_t *       service,
    wsrp_QueryResourcePropertiesType *  QueryResourceProperties,
    wsrp_QueryResourcePropertiesResponseType *
                                        QueryResourcePropertiesResponse,
    char **                             fault_name,
    void **                             fault);

static
int
wsrp_QueryResourceProperties_activate(void);

static
int
wsrp_QueryResourceProperties_deactivate(void);

/* Local Variables */
static xsd_QName wsrp_QueryResourceProperties_qname =
{
    "http://www.globus.org/docs.oasis-open.org/wsrf/2004/06/wsrf-WS-ResourceProperties-1.2-draft-01.wsdl",
    "QueryResourceProperties"
};

static
globus_operation_provider_descriptor_t
wsrp_QueryResourceProperties_descriptor =
{
    &wsrp_QueryResourceProperties_qname,
    "QueryResourceProperties",
    (void *)wsrp_QueryResourceProperties_init_provider,
    (void *)wsrp_QueryResourceProperties_provider
};

GlobusExtensionDefineModule(wsrp_QueryResourceProperties) =
{
    "wsrp_QueryResourceProperties",
    wsrp_QueryResourceProperties_activate,
    wsrp_QueryResourceProperties_deactivate,
    NULL,
    NULL,
    &local_version
};

/* Implementation */

static
globus_result_t
wsrp_QueryResourceProperties_init_provider(
    globus_service_engine_t             engine,
    globus_soap_message_handle_t        message,
    wsrp_QueryResourcePropertiesType *  QueryResourceProperties)
{
    return GLOBUS_SUCCESS;
}
/* wsrp_QueryResourceProperties_init_provider() */

static
globus_result_t
wsrp_QueryResourceProperties_provider(
    globus_service_engine_t             engine,
    globus_soap_message_handle_t        message,
    globus_service_descriptor_t *       service,
    wsrp_QueryResourcePropertiesType *  QueryResourceProperties,
    wsrp_QueryResourcePropertiesResponseType *
                                        QueryResourcePropertiesResponse,
    char **                             fault_name,
    void **                             fault)
{
    globus_result_t                     result;
    globus_resource_t                   resource;

    result = globus_wsrf_core_get_resource(
        message,
        service,
        &resource);

    if (result != GLOBUS_SUCCESS || resource == NULL)
    {
        result = wsrp_l_ResourceUnknownFaultType_create(
            fault_name,
            (wsrp_ResourceUnknownFaultType **) fault);

        goto out;
    }
    /* We don't understand any XPATH query dialects yet */
    result = wsrp_l_UnknownQueryExpressionDialectFaultType_create(
            fault_name,
            (wsrp_UnknownQueryExpressionDialectFaultType **) fault);

    globus_resource_finish(resource);

out:
    return result;
}
/* wsrp_QueryResourceProperties_provider() */

static
int
wsrp_QueryResourceProperties_activate(void)
{
    int                                 res = 0;
    GlobusFuncName(wsrp_QueryResourceProperties_activate);
    

    res = globus_extension_registry_add(
        GLOBUS_OPERATION_PROVIDER_REGISTRY,
        &wsrp_QueryResourceProperties_qname,
        GlobusExtensionMyModule(globus_ws_resource_properties),
        &wsrp_QueryResourceProperties_descriptor);

    return res;
}
/* wsrp_QueryResourceProperties_activate(void) */

static
int
wsrp_QueryResourceProperties_deactivate(void)
{
    GlobusFuncName(wsrp_QueryResourceProperties_deactivate);

    globus_extension_registry_remove(
        GLOBUS_OPERATION_PROVIDER_REGISTRY,
        &wsrp_QueryResourceProperties_qname);

    return 0;
}
/* wsrp_QueryResourceProperties_deactivate() */
