/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */

#include "CounterService_client.h"
#include "globus_service_engine.h"
#include "globus_notification_consumer.h"
#include "globus_xml_buffer.h"

#include "wsrp_GetResourcePropertyResponseType.h"

int notify_value = 0;
globus_cond_t notify_cond;
globus_mutex_t notify_mutex;

int main(int argc, char * argv[])
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    CounterService_client_handle_t      client_handle;
    createCounterType                   createCounter;
    createCounterResponseType *         createCounterResponse = NULL;
    int *                               addResponse = NULL;
    int                                 add_value;
    int                                 fault_type;
    int                                 i = 0;
    xsd_any *                           fault;
    globus_xml_buffer *                 counterKey = NULL;
    xsd_any_array                       referenceProperties;
    xsd_QName                           get_rp_qname;
    wsrp_GetResourcePropertyResponseType *
                                        get_rp_response;

    if(argc < 2)
    {
        fprintf(stderr, "usage: test-counter <URL>\n");
        exit(1);
    }

    globus_module_activate(GLOBUS_SOAP_MESSAGE_MODULE);
    globus_module_activate(GLOBUS_NOTIFICATION_CONSUMER_MODULE);
    globus_module_activate(COUNTERSERVICE_MODULE);

    result = CounterService_client_init(&client_handle, NULL, NULL);
    if(result != GLOBUS_SUCCESS)
    {
        goto exit;
    }

    result = CounterPortType_createCounter(
        client_handle,
        argv[1],
        &createCounter,
        &createCounterResponse,
        (CounterPortType_createCounter_fault_t *)&fault_type,
        &fault);
    if(result != GLOBUS_SUCCESS)
    {
        goto exit;
    }

    if(createCounterResponse && 
       createCounterResponse->EndpointReference.ReferenceProperties)
    {
        referenceProperties = 
        createCounterResponse->EndpointReference.ReferenceProperties->any;

        if(referenceProperties.length != 1)
        {
            fprintf(stderr, 
                    "ERROR: Invalid number (%d) of counterKey elements"
                    " in createCounterResponse\n", 
                    referenceProperties.length);
            exit(1);
        }

        printf("COUNTER KEY TYPE: {%s}%s\n", 
               referenceProperties.elements[0].any_info->type->Namespace, 
               referenceProperties.elements[0].any_info->type->local);
               
        if(referenceProperties.elements[0].any_info->type == 
           xsd_string_info.type)
        {
            printf("COUNTER KEY: %s\n", 
                   *(xsd_string *)referenceProperties.elements[0].value);
        }
        else if(referenceProperties.elements[0].any_info->type == 
                globus_xml_buffer_info.type)
        {
            counterKey = (globus_xml_buffer *)
            referenceProperties.elements[0].value;
            if(counterKey)
            {
                printf("COUNTER KEY: %*s\n", 
                       counterKey->length, counterKey->buffer);
            }
        }
    }

    get_rp_qname.Namespace = globus_libc_strdup("http://counter.com");
    get_rp_qname.local = globus_libc_strdup("Value");

    add_value = 10;
    for(i = 0; i < 10; ++i)
    {
        result = CounterPortType_add_epr(
            client_handle,
            &createCounterResponse->EndpointReference,
            &add_value,
            &addResponse,
            (CounterPortType_add_fault_t *)&fault_type,
            &fault);
        if(result != GLOBUS_SUCCESS)
        {
            goto exit;
        }
        printf("Add result is %d\n", *(xsd_int*)addResponse);

        result = CounterPortType_GetResourceProperty_epr(
            client_handle,
            &createCounterResponse->EndpointReference,
            &get_rp_qname,
            &get_rp_response,
            (CounterPortType_GetResourceProperty_fault_t *)&fault_type,
            &fault);
        if (result != GLOBUS_SUCCESS)
        {
            printf("get_rp failed: %s\n",
                    globus_object_printable_to_string(globus_error_peek(result)));
            goto exit;
        }

        printf("RP Value is now %d\n",
                *(xsd_int*)get_rp_response->any.elements[0].value);
    }

 exit:

    if(createCounterResponse)
    {
        createCounterResponseType_destroy(createCounterResponse);
    }

    CounterService_client_destroy(client_handle);

    if(result != GLOBUS_SUCCESS)
    {
        char *                          errorstr;

        errorstr = globus_error_print_chain(globus_error_get(result));
        fprintf(stderr, "ERROR:\n\n%s\n\n", errorstr);
        globus_module_deactivate_all();
        return 1;
    }

    globus_module_deactivate_all();

    return 0;
}
