/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */

#include "WidgetService_skeleton.h"
#include "WidgetService_internal_skeleton.h"

#include "globus_wsrf_resource.h"
#include "globus_soap_message_handle.h"
#include "globus_libc.h"
#include "globus_uuid.h"
#include "globus_wsrf_core_tools.h"

#define WIDGET_NS "http://widgets.com"

static xsd_QName WidgetService_foo_qname =
{
    "http://widgets.com",
    "foo"
};

globus_result_t
WidgetService_init(
    globus_service_descriptor_t *       service_desc)
{
    int                                 rc;
    GlobusFuncName(WidgetService_init);

    WidgetServiceDebugEnter();

    rc = globus_module_activate(GLOBUS_WSRF_RESOURCE_MODULE);

    WidgetServiceDebugExit();

    return (rc == GLOBUS_SUCCESS) ? GLOBUS_SUCCESS : GLOBUS_FAILURE;
}
/* WidgetService_init() */

globus_result_t
WidgetService_finalize(
    globus_service_descriptor_t *       service_desc)
{
    int                                 rc;
    GlobusFuncName(WidgetService_finalize);

    WidgetServiceDebugEnter();

    /* do any service specific finalize stuff here, 
     * opposite of WidgetService_init
     */
    rc = globus_module_deactivate(GLOBUS_WSRF_RESOURCE_MODULE);

    WidgetServiceDebugExit();

    return (rc == GLOBUS_SUCCESS) ? GLOBUS_SUCCESS : GLOBUS_FAILURE;
}
/* WidgetService_finalize() */

/* Dummy implementations of functions defined by providers */
globus_result_t
WidgetPortType_GetResourceProperty_init(
    globus_service_engine_t             engine,
    globus_soap_message_handle_t        message,
    xsd_QName * GetResourceProperty)
{
    return GLOBUS_FAILURE;
}
/* WidgetPortType_GetResourceProperty_init() */

globus_result_t
WidgetPortType_GetResourceProperty_impl(
    globus_service_engine_t             engine,
    globus_soap_message_handle_t        message,
    globus_service_descriptor_t *       descriptor,
    xsd_QName * GetResourceProperty,
    wsrp_GetResourcePropertyResponseType * GetResourcePropertyResponse,
    const char **                       fault_name,
    void ** fault)
{
    return GLOBUS_FAILURE;
}
/* WidgetPortType_GetResourceProperty_impl() */

globus_result_t
WidgetPortType_GetMultipleResourceProperties_init(
    globus_service_engine_t             engine,
    globus_soap_message_handle_t        message,
    wsrp_GetMultipleResourcePropertiesType * GetMultipleResourceProperties)
{
    return GLOBUS_FAILURE;
}
/* WidgetPortType_GetMultipleResourceProperties_init() */

globus_result_t
WidgetPortType_GetMultipleResourceProperties_impl(
    globus_service_engine_t             engine,
    globus_soap_message_handle_t        message,
    globus_service_descriptor_t *       descriptor,
    wsrp_GetMultipleResourcePropertiesType * GetMultipleResourceProperties,
    wsrp_GetMultipleResourcePropertiesResponseType * GetMultipleResourcePropertiesResponse,
    const char **                       fault_name,
    void ** fault)
{
    return GLOBUS_FAILURE;
}
/* WidgetPortType_GetMultipleResourceProperties_impl() */

globus_result_t
WidgetPortType_SetResourceProperties_init(
    globus_service_engine_t             engine,
    globus_soap_message_handle_t        message,
    wsrp_SetResourcePropertiesType * SetResourceProperties)
{
    return GLOBUS_FAILURE;
}
/* WidgetPortType_SetResourceProperties_init() */

globus_result_t
WidgetPortType_SetResourceProperties_impl(
    globus_service_engine_t             engine,
    globus_soap_message_handle_t        message,
    globus_service_descriptor_t *       descriptor,
    wsrp_SetResourcePropertiesType * SetResourceProperties,
    wsrp_SetResourcePropertiesResponseType * SetResourcePropertiesResponse,
    const char **                       fault_name,
    void ** fault)
{
    return GLOBUS_FAILURE;
}
/* WidgetPortType_SetResourceProperties_impl() */

globus_result_t
WidgetPortType_QueryResourceProperties_init(
    globus_service_engine_t             engine,
    globus_soap_message_handle_t        message,
    wsrp_QueryResourcePropertiesType * QueryResourceProperties)
{
    return GLOBUS_FAILURE;
}
/* WidgetPortType_QueryResourceProperties_init() */

globus_result_t
WidgetPortType_QueryResourceProperties_impl(
    globus_service_engine_t             engine,
    globus_soap_message_handle_t        message,
    globus_service_descriptor_t *       descriptor,
    wsrp_QueryResourcePropertiesType * QueryResourceProperties,
    wsrp_QueryResourcePropertiesResponseType * QueryResourcePropertiesResponse,
    const char **                       fault_name,
    void ** fault)
{
    return GLOBUS_FAILURE;
}
/* WidgetPortType_QueryResourceProperties_impl() */

globus_result_t
WidgetPortType_Destroy_init(
    globus_service_engine_t             engine,
    globus_soap_message_handle_t        message,
    wsrl_DestroyType * Destroy)
{
    return GLOBUS_FAILURE;
}
/* WidgetPortType_Destroy_init() */

globus_result_t
WidgetPortType_Destroy_impl(
    globus_service_engine_t             engine,
    globus_soap_message_handle_t        message,
    globus_service_descriptor_t *       descriptor,
    wsrl_DestroyType * Destroy,
    wsrl_DestroyResponseType * DestroyResponse,
    const char **                       fault_name,
    void ** fault)
{
    return GLOBUS_FAILURE;
}
/* WidgetPortType_Destroy_impl() */

globus_result_t
WidgetPortType_SetTerminationTime_init(
    globus_service_engine_t             engine,
    globus_soap_message_handle_t        message,
    wsrl_SetTerminationTimeType * SetTerminationTime)
{
    return GLOBUS_FAILURE;
}
/* WidgetPortType_SetTerminationTime_init() */

globus_result_t
WidgetPortType_SetTerminationTime_impl(
    globus_service_engine_t             engine,
    globus_soap_message_handle_t        message,
    globus_service_descriptor_t *       descriptor,
    wsrl_SetTerminationTimeType * SetTerminationTime,
    wsrl_SetTerminationTimeResponseType * SetTerminationTimeResponse,
    const char **                       fault_name,
    void ** fault)
{
    return GLOBUS_FAILURE;
}
/* WidgetPortType_SetTerminationTime_impl() */


globus_result_t
WidgetPortType_createWidget_init(
    globus_service_engine_t             engine,
    globus_soap_message_handle_t        message,
    createWidgetType *                  createWidget)
{
    return GLOBUS_SUCCESS;
}
/* WidgetPortType_createWidget_init() */

globus_result_t
WidgetPortType_createWidget_impl(
    globus_service_engine_t             engine,
    globus_soap_message_handle_t        message,
    globus_service_descriptor_t *       descriptor,
    createWidgetType *                  createWidget,
    createWidgetResponseType *          createWidgetResponse,
    const char **                       fault_name,
    void **                             fault)
{
    globus_result_t                     result;
    globus_uuid_t                       uuid;
    xsd_any *                           reference_property;
    xsd_string_array *                  foo_value;
    globus_resource_t                   resource;
    int                                 rc;

    *fault_name = NULL;
    *fault = NULL;

    rc = globus_uuid_create(&uuid);
    if (rc != GLOBUS_SUCCESS)
    {
        goto out;
    }
    
    result = globus_resource_create(uuid.text, &resource);
    if (result != GLOBUS_SUCCESS)
    {
        goto out;
    }
    xsd_string_array_init(&foo_value);

    result = globus_resource_create_property(
        resource,
        &WidgetService_foo_qname,
        &xsd_string_array_info,
        foo_value);
    if (result != GLOBUS_SUCCESS)
    {
        goto destroy_out;
    }

    result = WidgetServiceInitResource(uuid.text);
    if (result != GLOBUS_SUCCESS)
    {
        goto destroy_out;
    }

    xsd_any_init(&reference_property);
    if (reference_property == NULL)
    {
        result = GLOBUS_FAILURE;
        goto destroy_out;
    }

    reference_property->any_info = &xsd_string_info;
    xsd_QName_init(&reference_property->element);
    if (reference_property->element == NULL)
    {
        result = GLOBUS_FAILURE;
        goto destroy_ref_property;
    }
    reference_property->element->Namespace = globus_libc_strdup(WIDGET_NS);
    if (reference_property->element->Namespace == NULL)
    {
        result = GLOBUS_FAILURE;
        goto destroy_ref_element;
    }
    reference_property->element->local = globus_libc_strdup("ResourceID");
    if (reference_property->element->local == NULL)
    {
        result = GLOBUS_FAILURE;
        goto destroy_ref_element_Namespace;
    }
    xsd_string_init((xsd_string **) &reference_property->value);
    (*(xsd_string*)reference_property->value) = globus_libc_strdup(uuid.text);
    if (reference_property->value == NULL)
    {
        result = GLOBUS_FAILURE;
        goto destroy_ref_element_local;
    }
    result = globus_wsrf_core_create_endpoint_reference(
            engine,
            WIDGETSERVICE_BASE_PATH,
            &reference_property,
            &createWidgetResponse->EndpointReference);
    if (result != GLOBUS_SUCCESS)
    {
        result = GLOBUS_FAILURE;
        goto destroy_ref_value;
    }

    return GLOBUS_SUCCESS;

destroy_ref_value:
    if (reference_property->value != NULL)
    {
        free(reference_property->value);
    }
    reference_property->value = NULL;
destroy_ref_element_local:
    free(reference_property->element->local);
    reference_property->element->local = NULL;
destroy_ref_element_Namespace:
    free(reference_property->element->Namespace);
    reference_property->element->Namespace = NULL;
destroy_ref_element:
    xsd_QName_destroy(reference_property->element);
destroy_ref_property:
    xsd_any_destroy(reference_property);
destroy_out:
    globus_resource_destroy(resource);
out:
    return result;
}
/* WidgetPortType_createWidget_impl() */
