/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */

#include "WidgetService_client.h"
#include "wsrl_DestroyType.h"
#include "wsrl_DestroyResponseType.h"

int
main(int argc, char *argv[])
{
    int                                 rc;
    WidgetService_client_handle_t       client_handle;
    createWidgetType                    createWidget;
    createWidgetResponseType *          createWidgetResponse = NULL;
    wsrl_DestroyType                    Destroy;
    wsrl_DestroyResponseType *          DestroyResponse = NULL;
    int                                 fault_type;
    xsd_any *                           fault = NULL;
    globus_result_t                     result = GLOBUS_SUCCESS;

    if (argc != 2)
    {
        fprintf(stderr, "Usage: %s ServiceURI\n", argv[0]);
        rc = 1;

        goto out;
    }

    rc = globus_module_activate(GLOBUS_SOAP_MESSAGE_MODULE);
    if (rc != GLOBUS_SUCCESS)
    {
        rc = 2;

        goto out;
    }
    result = WidgetService_client_init(&client_handle, NULL, NULL);
    if (result != GLOBUS_SUCCESS)
    {
        rc = 3;

        goto deactivate_out;
    }
    result = WidgetPortType_createWidget(
        client_handle,
        argv[1],
        &createWidget,
        &createWidgetResponse,
        (WidgetPortType_createWidget_fault_t *) &fault_type,
        &fault);

    if (result != GLOBUS_SUCCESS)
    {
        rc = 4;

        goto handle_destroy_out;
    }

    result = WidgetPortType_Destroy_epr(
        client_handle,
        &createWidgetResponse->EndpointReference,
        &Destroy,
        &DestroyResponse,
        (WidgetPortType_Destroy_fault_t *)&fault_type,
        &fault);
    if (result != GLOBUS_SUCCESS)
    {
        rc = 5;

        goto handle_destroy_out;
    }

handle_destroy_out:
    createWidgetResponseType_destroy(createWidgetResponse);
    wsrl_DestroyResponseType_destroy(DestroyResponse);
    WidgetService_client_destroy(client_handle);
deactivate_out:
    if(result != GLOBUS_SUCCESS)
    {
        char * error = globus_error_print_chain(globus_error_peek(result));
        fprintf(stderr, "%s\n", error);
        free(error);
    }

    globus_module_deactivate(GLOBUS_SOAP_MESSAGE_MODULE);
out:

    return rc;
}
/* main() */
