/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */


#ifndef _GLOBUS_I_WSRF_CORE_TOOLS_H_
#define _GLOBUS_I_WSRF_CORE_TOOLS_H_

#include "globus_common.h"

EXTERN_C_BEGIN

#include "globus_wsrf_core_tools.h"

GlobusDebugDeclare(GLOBUS_WSRF_CORE_TOOLS);

#define GLOBUS_L_WSRF_CORE_TOOLS_DEBUG_DEBUG 1
#define GLOBUS_L_WSRF_CORE_TOOLS_DEBUG_INFO  2
#define GLOBUS_L_WSRF_CORE_TOOLS_DEBUG_TRACE 4
#define GLOBUS_L_WSRF_CORE_TOOLS_DEBUG_WARN  8
#define GLOBUS_L_WSRF_CORE_TOOLS_DEBUG_ERROR 16

#define GlobusWSRFCoreToolsDebugPrintf(LEVEL, MESSAGE) \
    GlobusDebugPrintf(GLOBUS_WSRF_CORE_TOOLS, LEVEL, MESSAGE)

#define GlobusWSRFCoreToolsDebugEnter() \
    GlobusWSRFCoreToolsDebugPrintf(GLOBUS_L_WSRF_CORE_TOOLS_DEBUG_TRACE, \
                                   ("[%s] Entering\n", _globus_func_name))

#define GlobusWSRFCoreToolsDebugExit() \
    GlobusWSRFCoreToolsDebugPrintf(GLOBUS_L_WSRF_CORE_TOOLS_DEBUG_TRACE, \
                                   ("[%s] Exiting\n", _globus_func_name))

#define GlobusWSRFCoreToolsErrorInvalidReferenceProperties(qname)       \
globus_error_put(                                                       \
    globus_error_construct_error(                                       \
        GLOBUS_WSRF_CORE_TOOLS_MODULE,                                  \
        NULL,                                                           \
        GLOBUS_WSRF_CORE_TOOLS_ERROR_TYPE_INVALID_REF_PROP,             \
        __FILE__,                                                       \
        _globus_func_name,                                              \
        __LINE__,                                                       \
        "Invalid Reference Properties type in EPR.  "                   \
        "Expected {xsd}string, found {%s}%s",                           \
        (qname->Namespace ? "{" : ""),                                  \
        (qname->Namespace ? qname->Namespace : ""),                     \
        (qname->Namespace ? "}" : ""),                                  \
        qname->local))

#define GlobusWSRFCoreToolsErrorReferencePropertiesWrongCount(count)    \
globus_error_put(                                                       \
    globus_error_construct_error(                                       \
        GLOBUS_WSRF_CORE_TOOLS_MODULE,                                  \
        NULL,                                                           \
        GLOBUS_WSRF_CORE_TOOLS_ERROR_TYPE_REF_PROP_WRONG_COUNT,         \
        __FILE__,                                                       \
        _globus_func_name,                                              \
        __LINE__,                                                       \
        "Invalid Reference Properties length (%d) in EPR",              \
        count))

#define GlobusWSRFCoreToolsErrorNullReferenceProperty()         \
globus_error_put(                                               \
    globus_error_construct_error(                               \
        GLOBUS_WSRF_CORE_TOOLS_MODULE,                          \
        NULL,                                                   \
        GLOBUS_WSRF_CORE_TOOLS_ERROR_TYPE_NULL_REF_PROP,        \
        __FILE__,                                               \
        _globus_func_name,                                      \
        __LINE__,                                               \
        "Null Reference Property"))

#define GlobusWSRFCoreToolsErrorGetResourceFailed(result, message)       \
globus_error_put(                                                        \
    globus_error_construct_error(                                        \
        GLOBUS_WSRF_CORE_TOOLS_MODULE,                                   \
        result ? globus_error_get(result) : NULL,                        \
        GLOBUS_WSRF_CORE_TOOLS_ERROR_TYPE_GET_RESOURCE_FAILED,           \
        __FILE__,                                                        \
        _globus_func_name,                                               \
        __LINE__,                                                        \
        "Failed to get resource: %s", message))

#define GlobusWSRFCoreToolsErrorExportFailed(result, filename)  \
globus_error_put(                                               \
    globus_error_construct_error(                               \
        GLOBUS_WSRF_CORE_TOOLS_MODULE,                          \
        result ? globus_error_get(result) : NULL,               \
        GLOBUS_WSRF_CORE_TOOLS_ERROR_TYPE_EXPORT_FAILED,        \
        __FILE__,                                               \
        _globus_func_name,                                      \
        __LINE__,                                               \
        "Failed to export endpoint reference to file: %s",      \
        filename))

#define GlobusWSRFCoreToolsErrorImportFailed(result, filename)  \
globus_error_put(                                               \
    globus_error_construct_error(                               \
        GLOBUS_WSRF_CORE_TOOLS_MODULE,                          \
        result ? globus_error_get(result) : NULL,               \
        GLOBUS_WSRF_CORE_TOOLS_ERROR_TYPE_IMPORT_FAILED,        \
        __FILE__,                                               \
        _globus_func_name,                                      \
        __LINE__,                                               \
        "Failed to import endpoint reference from file: %s",    \
        filename))

EXTERN_C_END

#endif
