/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */


#ifndef _GLOBUS_I_HANDLER_WSSE_CRED_H_
#define _GLOBUS_I_HANDLER_WSSE_CRED_H_

#include "globus_common.h"

EXTERN_C_BEGIN

extern globus_module_descriptor_t       globus_extension_module;

#define GLOBUS_L_WS_SECURITY_DEBUG_TRACE   1
#define GLOBUS_L_WS_SECURITY_DEBUG_DEBUG   2
#define GLOBUS_L_WS_SECURITY_DEBUG_PKIPATH 4
#define GLOBUS_L_WS_SECURITY_DEBUG_C14N    8
#define GLOBUS_L_WS_SECURITY_DEBUG_SIGN    16
#define GLOBUS_L_WS_SECURITY_DEBUG_VERIFY  32

#include "globus_error_openssl.h"
#include "globus_gsi_credential.h"
#include "globus_soap_message.h"
#include "globus_soap_message_handle.h"

GlobusDebugDeclare(GLOBUS_HANDLER_WS_SECURE_MESSAGE);

#define GlobusWSSEDebug(LEVEL) \
    GlobusDebugTrue(GLOBUS_HANDLER_WS_SECURE_MESSAGE, LEVEL)

#define GlobusWSSEDebugPrintf(LEVEL, MESSAGE) \
    GlobusDebugPrintf(GLOBUS_HANDLER_WS_SECURE_MESSAGE, LEVEL, MESSAGE)

#define GlobusWSSEDebugEnter() \
    GlobusWSSEDebugPrintf(GLOBUS_L_WS_SECURITY_DEBUG_TRACE, \
                      ("[%s] Entering\n", _globus_func_name))

#define GlobusWSSEDebugExit() \
    GlobusWSSEDebugPrintf(GLOBUS_L_WS_SECURITY_DEBUG_TRACE, \
                      ("[%s] Exiting\n", _globus_func_name))

enum
{
    GLOBUS_WSSE_ERROR_GETTING_CREDENTIAL = 1024,
    GLOBUS_WSSE_ERROR_OPENSSL,
    GLOBUS_WSSE_ERROR_PKIPATH,
    GLOBUS_WSSE_ERROR_FAILED_SIGN,
    GLOBUS_WSSE_ERROR_FAILED_VERIFY,
    GLOBUS_WSSE_ERROR_FAILED_CLIENT_REQUEST,
    GLOBUS_WSSE_ERROR_FAILED_CLIENT_RESPONSE,
    GLOBUS_WSSE_ERROR_FAILED_SERVER_RESPONSE,
    GLOBUS_WSSE_ERROR_FAILED_SERVER_REQUEST,
    GLOBUS_WSSE_ERROR_CACHE,
    GLOBUS_WSSE_ERROR_DESERIALIZE_FAILED,
    GLOBUS_WSSE_ERROR_SECURITY_ELEMENT_NOT_FOUND,
    GLOBUS_WSSE_ERROR_FAILED_BINARY_SECURITY_TOKEN,
    GLOBUS_WSSE_ERROR_FAILED_DESERIALIZE_SIGNATURE,
    GLOBUS_WSSE_ERROR_FAILED_DESERIALIZE_TIMESTAMP,
    GLOBUS_WSSE_ERROR_EXPIRED,
    GLOBUS_WSSE_ERROR_WITH_ID_ATTR,
    GLOBUS_WSSE_ERROR_CALLOUT_INIT,
    GLOBUS_WSSE_ERROR_FAILED_DIGEST_VERIFY,
    GLOBUS_WSSE_ERROR_FAILED_SIGNATURE_VERIFY,
    GLOBUS_WSSE_ERROR_FAILED_ID_BUFFER_GET,
    GLOBUS_WSSE_ERROR_FAILED_REFERENCE_VERIFY,
    GLOBUS_WSSE_ERROR_OUT_OF_MEMORY,
    GLOBUS_WSSE_ERROR_FAILED_TREE_SEARCH
};
    
#define GlobusWSSEErrorGettingCredential(error, message)                \
    globus_error_put(                                                   \
        globus_error_construct_error(                                   \
            GlobusExtensionMyModule(globus_handler_ws_security),        \
            (error ? globus_error_get(error) : NULL),                   \
            GLOBUS_WSSE_ERROR_GETTING_CREDENTIAL,                       \
            __FILE__,                                                   \
            _globus_func_name,                                          \
            __LINE__,                                                   \
            "Failed to convert the PKIPath element "                    \
             "into a valid credential: %s",                             \
             message))

#define GlobusWSSEErrorOpenSSL(message)                                 \
    globus_error_put(                                                   \
        globus_error_wrap_openssl_error(                                \
            GlobusExtensionMyModule(globus_handler_ws_security),        \
            GLOBUS_WSSE_ERROR_OPENSSL,                                  \
            __FILE__,                                                   \
            _globus_func_name,                                          \
            __LINE__,                                                   \
            message))

#define GlobusWSSEErrorPKIPath(error, message)                          \
    globus_error_put(                                                   \
        globus_error_construct_error(                                   \
            GlobusExtensionMyModule(globus_handler_ws_security),        \
            (error ? globus_error_get(error) : NULL),                   \
            GLOBUS_WSSE_ERROR_PKIPATH,                                  \
            __FILE__,                                                   \
            _globus_func_name,                                          \
            __LINE__,                                                   \
            "Failed to convert the credential "                         \
             "into a valid PKIPath: %s",                                \
             message))

#define GlobusWSSEErrorCache(error, message)                            \
    globus_error_put(                                                   \
        globus_error_construct_error(                                   \
            GlobusExtensionMyModule(globus_handler_ws_security),        \
            (error ? globus_error_get(error) : NULL),                   \
            GLOBUS_WSSE_ERROR_CACHE,                                    \
            __FILE__,                                                   \
            _globus_func_name,                                          \
            __LINE__,                                                   \
            "Error with credential cache"))

#define GlobusWSSEErrorFailedSign(error)                                \
    globus_error_put(                                                   \
        globus_error_construct_error(                                   \
            GlobusExtensionMyModule(globus_handler_ws_security),        \
            (error ? globus_error_get(error) : NULL),                   \
            GLOBUS_WSSE_ERROR_FAILED_SIGN,                              \
            __FILE__,                                                   \
            _globus_func_name,                                          \
            __LINE__,                                                   \
            "Failed to sign the message"))

#define GlobusWSSEErrorFailedVerify(error, message)                     \
    globus_error_put(                                                   \
        globus_error_construct_error(                                   \
            GlobusExtensionMyModule(globus_handler_ws_security),        \
            (error ? globus_error_get(error) : NULL),                   \
            GLOBUS_WSSE_ERROR_FAILED_VERIFY,                            \
            __FILE__,                                                   \
            _globus_func_name,                                          \
            __LINE__,                                                   \
            "Failed to verify the message: %s", message))

#define GlobusWSSEErrorFailedSignatureVerify()                          \
    globus_error_put(                                                   \
        globus_error_construct_error(                                   \
            GlobusExtensionMyModule(globus_handler_ws_security),        \
            NULL,                                                       \
            GLOBUS_WSSE_ERROR_FAILED_SIGNATURE_VERIFY,                  \
            __FILE__,                                                   \
            _globus_func_name,                                          \
            __LINE__,                                                   \
            "Failed to verify signature"))

#define GlobusWSSEErrorFailedClientResponse(error)                      \
    globus_error_put(                                                   \
        globus_error_construct_error(                                   \
            GlobusExtensionMyModule(globus_handler_ws_security),        \
            (error ? globus_error_get(error) : NULL),                   \
            GLOBUS_WSSE_ERROR_FAILED_CLIENT_RESPONSE,                   \
            __FILE__,                                                   \
            _globus_func_name,                                          \
            __LINE__,                                                   \
            "Client Response handling failed"))

#define GlobusWSSEErrorFailedClientRequest(error)                       \
    globus_error_put(                                                   \
        globus_error_construct_error(                                   \
            GlobusExtensionMyModule(globus_handler_ws_security),        \
            (error ? globus_error_get(error) : NULL),                   \
            GLOBUS_WSSE_ERROR_FAILED_CLIENT_REQUEST,                    \
            __FILE__,                                                   \
            _globus_func_name,                                          \
            __LINE__,                                                   \
            "Client Request handling failed"))

#define GlobusWSSEErrorFailedServerResponse(error)                      \
    globus_error_put(                                                   \
        globus_error_construct_error(                                   \
            GlobusExtensionMyModule(globus_handler_ws_security),        \
            (error ? globus_error_get(error) : NULL),                   \
            GLOBUS_WSSE_ERROR_FAILED_SERVER_RESPONSE,                   \
            __FILE__,                                                   \
            _globus_func_name,                                          \
            __LINE__,                                                   \
            "Server Response handling failed"))

#define GlobusWSSEErrorFailedServerRequest(error)                       \
    globus_error_put(                                                   \
        globus_error_construct_error(                                   \
            GlobusExtensionMyModule(globus_handler_ws_security),        \
            (error ? globus_error_get(error) : NULL),                   \
            GLOBUS_WSSE_ERROR_FAILED_SERVER_REQUEST,                    \
            __FILE__,                                                   \
            _globus_func_name,                                          \
            __LINE__,                                                   \
            "Server Request handling failed"))

#define GlobusWSSEErrorDeserializeFailed(error, message)                \
    globus_error_put(                                                   \
        globus_error_construct_error(                                   \
            GlobusExtensionMyModule(globus_handler_ws_security),        \
            (error ? globus_error_get((globus_result_t)error) : NULL),  \
            GLOBUS_WSSE_ERROR_DESERIALIZE_FAILED,                       \
            __FILE__,                                                   \
            _globus_func_name,                                          \
            __LINE__,                                                   \
            "Error verifying WS-Security header: %s", message))

#define GlobusWSSEErrorSecurityElementNotFound()                        \
    globus_error_put(                                                   \
        globus_error_construct_error(                                   \
            GlobusExtensionMyModule(globus_handler_ws_security),        \
            NULL,                                                       \
            GLOBUS_WSSE_ERROR_SECURITY_ELEMENT_NOT_FOUND,               \
            __FILE__,                                                   \
            _globus_func_name,                                          \
            __LINE__,                                                   \
            "Error verifying WS-Security "                              \
            "header: Security element not found"))

#define GlobusWSSEErrorFailedBinarySecurityToken(message)               \
    globus_error_put(                                                   \
        globus_error_construct_error(                                   \
            GlobusExtensionMyModule(globus_handler_ws_security),        \
            NULL,                                                       \
            GLOBUS_WSSE_ERROR_FAILED_BINARY_SECURITY_TOKEN,             \
            __FILE__,                                                   \
            _globus_func_name,                                          \
            __LINE__,                                                   \
            "Failed to get Binary Security Token "                      \
            "from WS-Security header: %s", message))

#define GlobusWSSEErrorFailedDeserializeSignature(message)              \
    globus_error_put(                                                   \
        globus_error_construct_error(                                   \
            GlobusExtensionMyModule(globus_handler_ws_security),        \
            NULL,                                                       \
            GLOBUS_WSSE_ERROR_FAILED_DESERIALIZE_SIGNATURE,             \
            __FILE__,                                                   \
            _globus_func_name,                                          \
            __LINE__,                                                   \
            "Failed to deserialize signature: %s", message))


#define GlobusWSSEErrorFailedDeserializeTimestamp(message)              \
    globus_error_put(                                                   \
        globus_error_construct_error(                                   \
            GlobusExtensionMyModule(globus_handler_ws_security),        \
            NULL,                                                       \
            GLOBUS_WSSE_ERROR_FAILED_DESERIALIZE_TIMESTAMP,             \
            __FILE__,                                                   \
            _globus_func_name,                                          \
            __LINE__,                                                   \
            "Failed to deserialize timestamp: %s", message))

#define GlobusWSSEErrorExpired()                                        \
    globus_error_put(                                                   \
        globus_error_construct_error(                                   \
            GlobusExtensionMyModule(globus_handler_ws_security),        \
            NULL,                                                       \
            GLOBUS_WSSE_ERROR_EXPIRED,                                  \
            __FILE__,                                                   \
            _globus_func_name,                                          \
            __LINE__,                                                   \
            "The SecureMessage is expired"))

#define GlobusWSSEErrorWithIdAttr(RESULT, TYPE)                         \
    globus_error_put(                                                   \
        globus_error_construct_error(                                   \
            GlobusExtensionMyModule(globus_handler_ws_security),        \
            (RESULT ? globus_error_get(RESULT) : NULL),                 \
            GLOBUS_WSSE_ERROR_WITH_ID_ATTR,                             \
            __FILE__,                                                   \
            _globus_func_name,                                          \
            __LINE__,                                                   \
            "Error with Id attribute for QName: {%s}%s",                \
            TYPE->Namespace, TYPE->local))

#define GlobusWSSEErrorCalloutInit(RESULT)                                     \
    globus_error_put(                                                          \
        globus_error_construct_error(                                          \
            GlobusExtensionMyModule(globus_handler_ws_security),               \
            (RESULT ? globus_error_get(RESULT) : NULL),                        \
            GLOBUS_WSSE_ERROR_CALLOUT_INIT,                                    \
            __FILE__,                                                          \
            _globus_func_name,                                                 \
            __LINE__,                                                          \
            "Error in Callout init"))

#define GlobusWSSEErrorFailedDigestVerify(RESULT)                              \
    globus_error_put(                                                          \
        globus_error_construct_error(                                          \
            GlobusExtensionMyModule(globus_handler_ws_security),               \
            (RESULT ? globus_error_get(RESULT) : NULL),                        \
            GLOBUS_WSSE_ERROR_FAILED_DIGEST_VERIFY,                            \
            __FILE__,                                                          \
            _globus_func_name,                                                 \
            __LINE__,                                                          \
            "Failed to verify digest message"))

#define GlobusWSSEErrorFailedIdBufferGet(RESULT, ID)                           \
    globus_error_put(                                                          \
        globus_error_construct_error(                                          \
            GlobusExtensionMyModule(globus_handler_ws_security),               \
            (RESULT ? globus_error_get(RESULT) : NULL),                        \
            GLOBUS_WSSE_ERROR_FAILED_ID_BUFFER_GET,                            \
            __FILE__,                                                          \
            _globus_func_name,                                                 \
            __LINE__,                                                          \
            "Failed to get buffer marked by Id: %s", ID))

#define GlobusWSSEErrorFailedReferenceVerify(RESULT, ID)                       \
    globus_error_put(                                                          \
        globus_error_construct_error(                                          \
            GlobusExtensionMyModule(globus_handler_ws_security),               \
            (RESULT ? globus_error_get(RESULT) : NULL),                        \
            GLOBUS_WSSE_ERROR_FAILED_REFERENCE_VERIFY,                         \
            __FILE__,                                                          \
            _globus_func_name,                                                 \
            __LINE__,                                                          \
            "Failed to verify reference with Id: %s", ID))

#define GlobusWSSEErrorOutOfMemory(LENGTH)                              \
    globus_error_put(                                                   \
        globus_error_construct_error(                                   \
            GlobusExtensionMyModule(globus_handler_ws_security),        \
            NULL,                                                       \
            GLOBUS_WSSE_ERROR_OUT_OF_MEMORY,                            \
            __FILE__,                                                   \
            _globus_func_name,                                          \
            __LINE__,                                                   \
            "Memory allocation failed: %d bytes",                       \
            LENGTH))

#define GlobusWSSEErrorFailedTreeSearch(ELEMENT)                        \
    globus_error_put(                                                   \
	globus_error_construct_error(                                   \
	    GlobusExtensionMyModule(globus_handler_ws_security),        \
	    NULL,                                                       \
	    GLOBUS_WSSE_ERROR_FAILED_TREE_SEARCH,                       \
	    __FILE__,                                                   \
	    _globus_func_name,                                          \
	    __LINE__,                                                   \
	    "Failed Tree Search for element: {%s}%s",                   \
	    (ELEMENT)->Namespace ? (ELEMENT)->Namespace : "(null)",     \
	    (ELEMENT)->local ? (ELEMENT)->local : "(null)"))
    
	    
typedef struct globus_i_handler_wsse_cache_s
{
    globus_hashtable_t                  handlemap;
    globus_fifo_t                       handles;
    unsigned long                       capacity_limit;
    unsigned long                       entry_count;
    globus_reltime_t                    time_limit;
    unsigned long                       default_subject_hash;
} * globus_i_handler_wsse_cache_t;

typedef struct
{
    unsigned long                       subject;
    globus_gsi_cred_handle_t            credential;
    char *                              pkipath_buffer;
    int                                 pkipath_length;
    globus_abstime_t                    entry_time;
} globus_i_handler_wsse_pkipath_entry_t;

void 
globus_i_handler_wsse_cache_init (
    globus_i_handler_wsse_cache_t *     cache,
    int                                 capacity,
    globus_reltime_t                    reltime);

void
globus_i_handler_wsse_cache_destroy (
    globus_i_handler_wsse_cache_t       cache);

globus_result_t
globus_i_handler_wsse_cache_insert (
    globus_i_handler_wsse_cache_t       cache,
    globus_gsi_cred_handle_t            credential,
    unsigned long *                     hash);

globus_result_t
globus_i_handler_wsse_cache_lookup (
    globus_i_handler_wsse_cache_t       cache,
    unsigned long                       hash,
    globus_gsi_cred_handle_t *          credential,
    char **                             pkipath_buffer,
    int *                               pkipath_length);

globus_result_t
globus_i_handler_wsse_cache_remove (
    globus_i_handler_wsse_cache_t       cache,
    unsigned long                       hash,
    globus_gsi_cred_handle_t *          credential,
    char **                             pkipath_buffer,
    int *                               pkipath_length);

void
globus_i_handler_wsse_pkipath_entry_destroy(
    globus_i_handler_wsse_pkipath_entry_t * entry);

globus_result_t
globus_i_handler_wsse_get_credential(
    const unsigned char *               encoded_buff,
    int                                 encoded_length,
    globus_gsi_cred_handle_t *          new_cred);

globus_result_t
globus_i_handler_wsse_create_pkipath(
    globus_gsi_cred_handle_t            cred,
    char **                             encoded_buffer,
    int *                               encoded_length);

globus_result_t
globus_i_handler_wsse_get_pkipath(
    char **                             pkipath,
    int *                               pkipath_length);

globus_result_t
globus_i_handler_wsse_load_local_cred(
    globus_soap_message_handle_t        message_handle,
    globus_gsi_cred_handle_t *          credential,
    char **                             pkipath,
    int *                               pkipath_length);

globus_result_t
globus_i_handler_wsse_cred_pkipath_table_init();

void
globus_i_handler_wsse_cred_pkipath_table_destroy();

EXTERN_C_END

#endif
