/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */


#ifndef _GLOBUS_I_OPERATION_PROVIDER_H_
#define _GLOBUS_I_OPERATION_PROVIDER_H_

#include "globus_common.h"

EXTERN_C_BEGIN

#include "globus_operation_provider.h"

GlobusDebugDeclare(GLOBUS_OPERATION_PROVIDER);

enum globus_l_operation_provider_debug_levels
{
    GLOBUS_L_OPERATION_PROVIDER_DEBUG_TRACE = 1,
    GLOBUS_L_OPERATION_PROVIDER_DEBUG_ERROR = 2
};

#define GlobusOperationProviderDebugPrintf(level, message)              \
    GlobusDebugPrintf(GLOBUS_OPERATION_PROVIDER, level, message)

#define GlobusOperationProviderDebugEnter()             \
    GlobusOperationProviderDebugPrintf(                 \
        GLOBUS_L_OPERATION_PROVIDER_DEBUG_TRACE,        \
        ("[%s] Entering\n", _globus_func_name))

#define GlobusOperationProviderDebugExit()              \
    GlobusOperationProviderDebugPrintf(                 \
        GLOBUS_L_OPERATION_PROVIDER_DEBUG_TRACE,        \
        ("[%s] Exiting\n", _globus_func_name))

#define GlobusOperationProviderErrorRegistryLookupFailed(ENDPOINT)      \
    globus_error_put(                                                   \
        globus_error_construct_error(                                   \
            GLOBUS_OPERATION_PROVIDER_MODULE,                           \
            NULL,                                                       \
            GLOBUS_OPERATION_PROVIDER_ERROR_REGISTRY_LOOKUP_FAILED,     \
            __FILE__,                                                   \
            _globus_func_name,                                          \
            __LINE__,                                                   \
            "No operation provider found at endpoint: %s", ENDPOINT))

#define GlobusOperationProviderErrorFailedInit(CAUSE)                   \
    globus_error_put(                                                   \
                globus_error_construct_error(                           \
                    GLOBUS_OPERATION_PROVIDER_MODULE,                   \
                    ((CAUSE) ? globus_error_get(CAUSE) : NULL),         \
                    GLOBUS_OPERATION_PROVIDER_ERROR_FAILED_INIT,        \
                    __FILE__,                                           \
                    _globus_func_name,                                  \
                    __LINE__,                                           \
                    "Failed to initialize operation table"))

#define GlobusOperationProviderErrorOpNotFound(OP)                      \
    globus_error_put(                                                   \
                globus_error_construct_error(                           \
                    GLOBUS_OPERATION_PROVIDER_MODULE,                   \
                    NULL,                                               \
                    GLOBUS_OPERATION_PROVIDER_ERROR_OP_NOT_FOUND,       \
                    __FILE__,                                           \
                    _globus_func_name,                                  \
                    __LINE__,                                           \
                    "Failed to get operation: %s", OP))

#define GlobusOperationProviderErrorFailedTableInsert(OP)               \
    globus_error_put(                                                   \
        globus_error_construct_error(                                   \
            GLOBUS_OPERATION_PROVIDER_MODULE,                           \
            NULL,                                                       \
            GLOBUS_OPERATION_PROVIDER_ERROR_FAILED_TABLE_INSERT,        \
            __FILE__,                                                   \
            _globus_func_name,                                          \
            __LINE__,                                                   \
            "Failed to insert operation into table: %s", OP))

#define GlobusOperationProviderErrorFailedResourceInit(RESULT, OP)      \
    globus_error_put(                                                   \
        globus_error_construct_error(                                   \
            GLOBUS_OPERATION_PROVIDER_MODULE,                           \
            RESULT ? globus_error_get(RESULT) : NULL,                   \
            GLOBUS_OPERATION_PROVIDER_ERROR_FAILED_RESOURCE_INIT,       \
            __FILE__,                                                   \
            _globus_func_name,                                          \
            __LINE__,                                                   \
            "Failed to initialize resource with operation: {%s}%s",     \
            (OP)->Namespace, (OP)->local))

#define GlobusOperationProviderErrorOutOfMemory                         \
    globus_error_put(                                                   \
        globus_error_construct_error(                                   \
            GLOBUS_OPERATION_PROVIDER_MODULE,                           \
            GLOBUS_NULL,                                                \
            GLOBUS_OPERATION_PROVIDER_ERROR_OUT_OF_MEMORY,              \
            __FILE__,                                                   \
            _globus_func_name,                                          \
            __LINE__,                                                   \
            "Out of Memory"))

typedef struct globus_l_operation_table_s
{
    globus_hashtable_t                  table;
    char *                              Namespace;
} globus_l_operation_table_t;

EXTERN_C_END

#endif
