/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */


#ifndef _GLOBUS_I_SERVICE_ENGINE_H_
#define _GLOBUS_I_SERVICE_ENGINE_H_

#include "globus_common.h"

EXTERN_C_BEGIN

#include "globus_service_engine.h"
#include "globus_xio.h"

GlobusDebugDeclare(GLOBUS_SERVICE_ENGINE);

#define GLOBUS_L_SERVICE_ENGINE_DEBUG_DEBUG 1
#define GLOBUS_L_SERVICE_ENGINE_DEBUG_INFO  2
#define GLOBUS_L_SERVICE_ENGINE_DEBUG_TRACE 4
#define GLOBUS_L_SERVICE_ENGINE_DEBUG_WARN  8
#define GLOBUS_L_SERVICE_ENGINE_DEBUG_ERROR 16

#define GlobusServiceEngineDebug(LEVEL) \
    GlobusDebugTrue(GLOBUS_SERVICE_ENGINE, LEVEL)

#define GlobusServiceEngineDebugPrintf(LEVEL, MESSAGE) \
    GlobusDebugPrintf(GLOBUS_SERVICE_ENGINE, LEVEL, MESSAGE)

#define GlobusServiceEngineDebugEnter() \
    GlobusServiceEngineDebugPrintf(GLOBUS_L_SERVICE_ENGINE_DEBUG_TRACE, \
                                   ("[%s] Entering\n", _globus_func_name))

#define GlobusServiceEngineDebugExit() \
    GlobusServiceEngineDebugPrintf(GLOBUS_L_SERVICE_ENGINE_DEBUG_TRACE, \
                                   ("[%s] Exiting\n", _globus_func_name))

#define GlobusServiceEngineDebugPrintError(result, endpoint, action)    \
    GlobusServiceEngineDebugPrintf(GLOBUS_L_SERVICE_ENGINE_DEBUG_ERROR, \
                                   ("Service invocation failed:\n"      \
                                    "ENDPOINT: %s\nOPERATION: %s\n%s\n", \
                                    (endpoint ? endpoint : "Unknown"),  \
                                    (action ? action : "Unknown"),      \
                                    globus_error_print_friendly(        \
                                        globus_error_get(result))))

#define GLOBUS_SERVICE_ENGINE_ERROR_INVALID_CONTACT_OBJECT(contact)     \
    globus_error_construct_error(                                       \
        GLOBUS_SERVICE_ENGINE_MODULE,                                   \
        NULL,                                                           \
        GLOBUS_SERVICE_ENGINE_ERROR_TYPE_INVALID_CONTACT,               \
        __FILE__,                                                       \
        _globus_func_name,                                              \
        __LINE__,                                                       \
        "Invalid contact: %s.  needs to be listener port", contact)

#define GLOBUS_SERVICE_ENGINE_ERROR_INIT_FAILED_OBJECT(result)  \
    globus_error_construct_error(                               \
        GLOBUS_SERVICE_ENGINE_MODULE,                           \
        (result ? globus_error_get(result) : NULL),             \
        GLOBUS_SERVICE_ENGINE_ERROR_TYPE_INIT_FAILED,           \
        __FILE__,                                               \
        _globus_func_name,                                      \
        __LINE__,                                               \
        "Failed to initialize service engine")

#define GLOBUS_SERVICE_ENGINE_ERROR_UNSUPPORTED_MEDIA_TYPE_OBJECT(      \
    media_type)                                                         \
        globus_error_construct_error(                                   \
            GLOBUS_SERVICE_ENGINE_MODULE,                               \
            NULL,                                                       \
            GLOBUS_SERVICE_ENGINE_ERROR_TYPE_UNSUPPORTED_MEDIA_TYPE,    \
            __FILE__,                                                   \
            _globus_func_name,                                          \
            __LINE__,                                                   \
            "Unsupported media type: %s", media_type)

#define GLOBUS_SERVICE_ENGINE_ERROR_INVALID_URI_OBJECT(message, uri)    \
    globus_error_construct_error(                                       \
        GLOBUS_SERVICE_ENGINE_MODULE,                                   \
        NULL,                                                           \
        GLOBUS_SERVICE_ENGINE_ERROR_TYPE_INVALID_URI,                   \
        __FILE__,                                                       \
        _globus_func_name,                                              \
        __LINE__,                                                       \
        "Invalid URI for service: %s :%s", uri, message)

#define GLOBUS_SERVICE_ENGINE_ERROR_SERVICE_LOAD_FAILED_OBJECT(uri, cause)     \
    globus_error_construct_error(                                       \
        GLOBUS_SERVICE_ENGINE_MODULE,                                   \
        cause,                                                          \
        GLOBUS_SERVICE_ENGINE_ERROR_TYPE_SERVICE_LOAD_FAILED,           \
        __FILE__,                                                       \
        _globus_func_name,                                              \
        __LINE__,                                                       \
        "Service not found at path: %s", uri)

#define GLOBUS_SERVICE_ENGINE_ERROR_READ_REQUEST_FAILED_OBJECT(result)  \
    globus_error_construct_error(                                       \
        GLOBUS_SERVICE_ENGINE_MODULE,                                   \
        (result ? globus_error_get(result) : NULL),                     \
        GLOBUS_SERVICE_ENGINE_ERROR_TYPE_READ_REQUEST_FAILED,           \
        __FILE__,                                                       \
        _globus_func_name,                                              \
        __LINE__,                                                       \
        "Failed to read request message")

#define GLOBUS_SERVICE_ENGINE_ERROR_METHOD_NOT_ALLOWED_OBJECT(method)   \
    globus_error_construct_error(                                       \
        GLOBUS_SERVICE_ENGINE_MODULE,                                   \
        NULL,                                                           \
        GLOBUS_SERVICE_ENGINE_ERROR_TYPE_METHOD_NOT_ALLOWED,            \
        __FILE__,                                                       \
        _globus_func_name,                                              \
        __LINE__,                                                       \
        "Method %s not allowed by server", method)

#define GLOBUS_SERVICE_ENGINE_ERROR_FAILED_REQUEST_OBJECT(result)       \
    globus_error_construct_error(                                       \
        GLOBUS_SERVICE_ENGINE_MODULE,                                   \
        (result ? globus_error_get(result) : NULL),                     \
        GLOBUS_SERVICE_ENGINE_ERROR_TYPE_FAILED_REQUEST,                \
        __FILE__,                                                       \
        _globus_func_name,                                              \
        __LINE__,                                                       \
        "Failed Request")

#define GLOBUS_SERVICE_ENGINE_ERROR_ACCEPT_FAILED_OBJECT(result)        \
    globus_error_construct_error(                                       \
        GLOBUS_SERVICE_ENGINE_MODULE,                                   \
        (result ? globus_error_get(result) : NULL),                     \
        GLOBUS_SERVICE_ENGINE_ERROR_TYPE_ACCEPT_FAILED,                 \
        __FILE__,                                                       \
        _globus_func_name,                                              \
        __LINE__,                                                       \
        "Failed to accept new connection")

#define GLOBUS_SERVICE_ENGINE_ERROR_TRANSPORT_FAILED_OBJECT(    \
    result, message)                                            \
        globus_error_construct_error(                           \
            GLOBUS_SERVICE_ENGINE_MODULE,                       \
            (result ? globus_error_get(result) : NULL),         \
            GLOBUS_SERVICE_ENGINE_ERROR_TYPE_TRANSPORT_FAILED,  \
            __FILE__,                                           \
            _globus_func_name,                                  \
            __LINE__,                                           \
            "Transport Failed: %s", message)

#define GLOBUS_SERVICE_ENGINE_ERROR_FAILED_RESPONSE_OBJECT(result)      \
    globus_error_construct_error(                                       \
        GLOBUS_SERVICE_ENGINE_MODULE,                                   \
        (result ? globus_error_get(result) : NULL),                     \
        GLOBUS_SERVICE_ENGINE_ERROR_TYPE_FAILED_RESPONSE,               \
        __FILE__,                                                       \
        _globus_func_name,                                              \
        __LINE__,                                                       \
        "Failed sending response")

#define GLOBUS_SERVICE_ENGINE_ERROR_SENDING_FAULT_OBJECT(result)        \
    globus_error_construct_error(                                       \
        GLOBUS_SERVICE_ENGINE_MODULE,                                   \
        (result ? globus_error_get(result) : NULL),                     \
        GLOBUS_SERVICE_ENGINE_ERROR_TYPE_SENDING_FAULT,                 \
        __FILE__,                                                       \
        _globus_func_name,                                              \
        __LINE__,                                                       \
        "Failed sending fault")

#define GLOBUS_SERVICE_ENGINE_ERROR_ALREADY_STARTED_OBJECT      \
    globus_error_construct_error(                               \
        GLOBUS_SERVICE_ENGINE_MODULE,                           \
        (result ? globus_error_get(result) : NULL),             \
        GLOBUS_SERVICE_ENGINE_ERROR_TYPE_ALREADY_STARTED,       \
        __FILE__,                                               \
        _globus_func_name,                                      \
        __LINE__,                                               \
        "Engine is already started accepting connections")

#define GLOBUS_SERVICE_ENGINE_ERROR_STOP_FAILED_OBJECT(result)  \
    globus_error_construct_error(                               \
        GLOBUS_SERVICE_ENGINE_MODULE,                           \
        (result ? globus_error_get(result) : NULL),             \
        GLOBUS_SERVICE_ENGINE_ERROR_TYPE_STOP_FAILED,           \
        __FILE__,                                               \
        _globus_func_name,                                      \
        __LINE__,                                               \
        "Failed to stop engine accepting connections")

#define GLOBUS_SERVICE_ENGINE_ERROR_FAILED_REQUEST_PROCESS_OBJECT(      \
    result, message)                                                    \
        globus_error_construct_error(                                   \
            GLOBUS_SERVICE_ENGINE_MODULE,                               \
            (result ? globus_error_get(result) : NULL),                 \
            GLOBUS_SERVICE_ENGINE_ERROR_TYPE_FAILED_REQUEST_PROCESS,    \
            __FILE__,                                                   \
            _globus_func_name,                                          \
            __LINE__,                                                   \
            "Failed to process request: %s", message)

#define GLOBUS_SERVICE_ENGINE_ERROR_FAILED_START_OBJECT(        \
    result)                                                     \
        globus_error_construct_error(                           \
            GLOBUS_SERVICE_ENGINE_MODULE,                       \
            (result ? globus_error_get(result) : NULL),         \
            GLOBUS_SERVICE_ENGINE_ERROR_TYPE_FAILED_START,      \
            __FILE__,                                           \
            _globus_func_name,                                  \
            __LINE__,                                           \
            "Failed to start processing requests")

#define GLOBUS_SERVICE_ENGINE_ERROR_OPERATION_LOOKUP_FAILED_OBJECT(     \
    operation,                                                          \
    service)                                                            \
    globus_error_construct_error(                                       \
        GLOBUS_SERVICE_ENGINE_MODULE,                                   \
        (result ? globus_error_get(result) : NULL),                     \
        GLOBUS_SERVICE_ENGINE_ERROR_TYPE_OPERATION_LOOKUP_FAILED,       \
        __FILE__,                                                       \
        _globus_func_name,                                              \
        __LINE__,                                                       \
        "Failed to find operation: %s%s%s%s for service: %s%s%s%s",     \
        (((operation) && (operation)->Namespace) ? "{" : ""),           \
        (((operation) && (operation)->Namespace) ?                      \
         (operation)->Namespace : ""),                                  \
        (((operation) && (operation)->Namespace) ? "}" : ""),           \
        (((operation) && (operation)->local) ?                          \
                         (operation)->local : ""),                      \
        (((service) && (service)->Namespace) ? "{" : ""),               \
        (((service) && (service)->Namespace) ?                          \
                       (service)->Namespace : ""),                      \
        (((service) && (service)->Namespace) ? "}" : ""),               \
        (((service) && (service)->local) ?                              \
                       (service)->local : ""))

#define GLOBUS_SERVICE_ENGINE_ERROR_OP_INVOKE_FAILED_OBJECT(    \
    result, qname)                                              \
    globus_error_construct_error(                               \
        GLOBUS_SERVICE_ENGINE_MODULE,                           \
        (result ? globus_error_get(result) : NULL),             \
        GLOBUS_SERVICE_ENGINE_ERROR_TYPE_OP_INVOKE_FAILED,      \
        __FILE__,                                               \
        _globus_func_name,                                      \
        __LINE__,                                               \
        "Failed to invoke operation: %s%s%s%s",                 \
        (((qname) && (qname)->Namespace) ? "{" : ""),           \
        (((qname) && (qname)->Namespace) ?                      \
         (qname)->Namespace : ""),                              \
        (((qname) && (qname)->Namespace) ? "}" : ""),           \
        (((qname) && (qname)->local) ?                          \
                     (qname)->local : ""))

#define GLOBUS_SERVICE_ENGINE_ERROR_OP_DESERIALIZE_FAILED_OBJECT(       \
    result, qname)                                                      \
    globus_error_construct_error(                                       \
        GLOBUS_SERVICE_ENGINE_MODULE,                                   \
        (result ? globus_error_get(result) : NULL),                     \
        GLOBUS_SERVICE_ENGINE_ERROR_TYPE_OP_DESERIALIZE_FAILED,         \
        __FILE__,                                                       \
        _globus_func_name,                                              \
        __LINE__,                                                       \
        "Failed to deserialize operation: %s%s%s%s",                    \
        (((qname) && (qname)->Namespace) ? "{" : ""),                   \
        (((qname) && (qname)->Namespace) ?                              \
         (qname)->Namespace : ""),                                      \
        (((qname) && (qname)->Namespace) ? "}" : ""),                   \
        (((qname) && (qname)->local) ?                                  \
                     (qname)->local : ""))

#define GLOBUS_SERVICE_ENGINE_ERROR_SESSION_FAILED_OBJECT(      \
    result, qname)                                              \
    globus_error_construct_error(                               \
        GLOBUS_SERVICE_ENGINE_MODULE,                           \
        (result ? globus_error_get(result) : NULL),             \
        GLOBUS_SERVICE_ENGINE_ERROR_TYPE_SESSION_FAILED,        \
        __FILE__,                                               \
        _globus_func_name,                                      \
        __LINE__,                                               \
        "Session Failed: %s%s%s%s",                             \
        (((qname) && (qname)->Namespace) ? "{" : ""),           \
        (((qname) && (qname)->Namespace) ?                      \
         (qname)->Namespace : ""),                              \
        (((qname) && (qname)->Namespace) ? "}" : ""),           \
        (((qname) && (qname)->local) ?                          \
                     (qname)->local : ""))

#define GLOBUS_SERVICE_ENGINE_ERROR_HANDLER_LOAD_FAILED_OBJECT( \
    handler,                                                    \
    cause)                                                      \
    globus_error_construct_error(                               \
        GLOBUS_SERVICE_ENGINE_MODULE,                           \
        cause,                                                  \
        GLOBUS_SERVICE_ENGINE_ERROR_TYPE_HANDLER_LOAD_FAILED,   \
        __FILE__,                                               \
        _globus_func_name,                                      \
        __LINE__,                                               \
        "Handler Load Failed: %s", handler)

#define GLOBUS_SERVICE_ENGINE_ERROR_SESSION_START_FAILED_OBJECT(        \
    error)                                                              \
    globus_error_construct_error(                                       \
        GLOBUS_SERVICE_ENGINE_MODULE,                                   \
        (error ? globus_error_get(error) : NULL),                       \
        GLOBUS_SERVICE_ENGINE_ERROR_TYPE_SESSION_START_FAILED,          \
        __FILE__,                                                       \
        _globus_func_name,                                              \
        __LINE__,                                                       \
        "Session failed to start")

#define GLOBUS_SERVICE_ENGINE_ERROR_OUT_OF_MEMORY_OBJECT        \
    globus_error_construct_error(                               \
        GLOBUS_SERVICE_ENGINE_MODULE,                           \
        GLOBUS_NULL,                                            \
        GLOBUS_SERVICE_ENGINE_ERROR_TYPE_OUT_OF_MEMORY,         \
        __FILE__,                                               \
        _globus_func_name,                                      \
        __LINE__,                                               \
        "Out of Memory")
    
#define GlobusServiceEngineErrorInvalidContact(contact)                 \
    globus_error_put(                                                   \
        GLOBUS_SERVICE_ENGINE_ERROR_INVALID_CONTACT_OBJECT(contact))

#define GlobusServiceEngineErrorInitFailed(result)              \
    globus_error_put(                                           \
        GLOBUS_SERVICE_ENGINE_ERROR_INIT_FAILED_OBJECT(result))

#define GlobusServiceEngineErrorUnsupportedMediaType(media_type)        \
    globus_error_put(                                                   \
        GLOBUS_SERVICE_ENGINE_ERROR_UNSUPPORTED_MEDIA_TYPE_OBJECT(      \
            media_type))

#define GlobusServiceEngineErrorInvalidURI(message, uri)                \
    globus_error_put(                                                   \
        GLOBUS_SERVICE_ENGINE_ERROR_INVALID_URI_OBJECT(message, uri))

#define GlobusServiceEngineErrorServiceLoadFailed(uri, cause)           \
    globus_error_put(                                                   \
        GLOBUS_SERVICE_ENGINE_ERROR_SERVICE_LOAD_FAILED_OBJECT(uri, cause))

#define GlobusServiceEngineErrorReadRequestFailed(result)               \
    globus_error_put(                                                   \
        GLOBUS_SERVICE_ENGINE_ERROR_READ_REQUEST_FAILED_OBJECT(result))

#define GlobusServiceEngineErrorMethodNotAllowed(method)                \
    globus_error_put(                                                   \
        GLOBUS_SERVICE_ENGINE_ERROR_METHOD_NOT_ALLOWED_OBJECT(method))

#define GlobusServiceEngineErrorFailedRequest(result)                   \
    globus_error_put(                                                   \
        GLOBUS_SERVICE_ENGINE_ERROR_FAILED_REQUEST_OBJECT(result))

#define GlobusServiceEngineErrorAcceptFailed(result)                    \
    globus_error_put(                                                   \
        GLOBUS_SERVICE_ENGINE_ERROR_ACCEPT_FAILED_OBJECT(result))

#define GlobusServiceEngineErrorTransportFailed(result, message)        \
    globus_error_put(                                                   \
        GLOBUS_SERVICE_ENGINE_ERROR_TRANSPORT_FAILED_OBJECT(            \
            result, message))

#define GlobusServiceEngineErrorFailedResponse(result)                  \
    globus_error_put(                                                   \
        GLOBUS_SERVICE_ENGINE_ERROR_FAILED_RESPONSE_OBJECT(result))

#define GlobusServiceEngineErrorSendingFault(result)                    \
    globus_error_put(                                                   \
        GLOBUS_SERVICE_ENGINE_ERROR_SENDING_FAULT_OBJECT(result))

#define GlobusServiceEngineErrorAlreadyStarted()                \
    globus_error_put(                                           \
        GLOBUS_SERVICE_ENGINE_ERROR_ALREADY_STARTED_OBJECT)

#define GlobusServiceEngineErrorStopFailed(result)              \
    globus_error_put(                                           \
        GLOBUS_SERVICE_ENGINE_ERROR_STOP_FAILED_OBJECT(result))

#define GlobusServiceEngineErrorOperationLookupFailed(                  \
    operation, service)                                                 \
    globus_error_put(                                                   \
        GLOBUS_SERVICE_ENGINE_ERROR_OPERATION_LOOKUP_FAILED_OBJECT(     \
            operation, service))

#define GlobusServiceEngineErrorOpInvokeFailed(result, qname)   \
    globus_error_put(                                           \
        GLOBUS_SERVICE_ENGINE_ERROR_OP_INVOKE_FAILED_OBJECT(    \
            result, qname))

#define GlobusServiceEngineErrorOpDeserializeFailed(result, qname)      \
    globus_error_put(                                                   \
        GLOBUS_SERVICE_ENGINE_ERROR_OP_DESERIALIZE_FAILED_OBJECT(       \
            result, qname))

#define GlobusServiceEngineErrorSessionFailed(result, qname)    \
    globus_error_put(                                           \
        GLOBUS_SERVICE_ENGINE_ERROR_SESSION_FAILED_OBJECT(      \
            result, qname))

#define GlobusServiceEngineErrorFailedRequestProcess(result, message)   \
    globus_error_put(                                                   \
        GLOBUS_SERVICE_ENGINE_ERROR_FAILED_REQUEST_PROCESS_OBJECT(      \
            result, message))

#define GlobusServiceEngineErrorFailedStart(result)             \
    globus_error_put(                                           \
        GLOBUS_SERVICE_ENGINE_ERROR_FAILED_START_OBJECT(        \
            result))

#define GlobusServiceEngineErrorHandlerLoadFailed(handler, cause)       \
    globus_error_put(                                                   \
        GLOBUS_SERVICE_ENGINE_ERROR_HANDLER_LOAD_FAILED_OBJECT(         \
            handler, cause))

#define GlobusServiceEngineErrorSessionStartFailed(result)              \
    globus_error_put(                                                   \
        GLOBUS_SERVICE_ENGINE_ERROR_SESSION_START_FAILED_OBJECT(        \
            result))

#define GlobusServiceEngineErrorOutOfMemory                             \
    globus_error_put(                                                   \
        GLOBUS_SERVICE_ENGINE_ERROR_OUT_OF_MEMORY_OBJECT)



#define GlobusServiceEngineErrorCheck(result, error_type)       \
     result ? (globus_error_match(globus_error_peek(result),    \
                                  GLOBUS_SERVICE_ENGINE_MODULE, \
                                  error_type)) : 0

#define GlobusServiceEngineErrorCheckMethodNotAllowed(result)   \
    GlobusServiceEngineErrorCheck(                              \
        result,                                                 \
        GLOBUS_SERVICE_ENGINE_ERROR_TYPE_METHOD_NOT_ALLOWED)

#define GlobusServiceEngineErrorCheckUnsupportedMediaType(result)       \
    GlobusServiceEngineErrorCheck(                                      \
        result,                                                         \
        GLOBUS_SERVICE_ENGINE_ERROR_TYPE_UNSUPPORTED_MEDIA_TYPE)

typedef struct globus_l_service_engine_s
{
    globus_xio_server_t                 server;
    int                                 session_count;
    int                                 stop_sessions;
    globus_mutex_t                      mutex;
    globus_handler_chain_t              handlers;
    char *                              contact;
    globus_xsd_type_registry_t          registry;
    globus_soap_message_attr_t          attrs;
} globus_i_service_engine_t;

EXTERN_C_END

#endif
