/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */


#ifndef _GLOBUS_I_SOAP_MESSAGE_H_
#define _GLOBUS_I_SOAP_MESSAGE_H_

EXTERN_C_BEGIN

#include "globus_common.h"
#include "libxml/xmlreader.h"
#include "libxml/xmlwriter.h"
#include "globus_xio.h"
#include "globus_xsd_type_registry.h"
#include "globus_handler.h"

#define GLOBUS_I_SOAP_MESSAGE_READBUF_SIZE 500

extern globus_xio_stack_t               globus_i_soap_message_xio_stack;
extern globus_xio_driver_t              globus_i_soap_message_tcp_driver;
extern globus_xio_driver_t              globus_i_soap_message_http_driver;
extern globus_xio_driver_t              globus_i_soap_message_buffer_driver;

extern globus_xio_stack_t               globus_i_soap_message_xio_file_stack;
extern globus_xio_driver_t              globus_i_soap_message_file_driver;

extern globus_xio_stack_t               globus_i_soap_message_xio_https_stack;
extern globus_xio_driver_t              globus_i_soap_message_gsi_driver;

typedef struct globus_i_soap_message_namespace_s
{
    const xmlChar *                            ns;
    int                                        ref;
    struct globus_i_soap_message_namespace_s * next;
} globus_i_soap_message_namespace_t;

typedef struct globus_soap_message_handle_s
{
    globus_xio_handle_t                 xio_handle;
    xmlTextReaderPtr                    reader;
    globus_hashtable_t                  mustUnderstand_hash;
    xmlTextWriterPtr                    writer;
    int                                 buffer_eof;
    globus_soap_message_attr_t          attrs;
    globus_xsd_type_registry_t          registry;
    globus_handler_chain_t              chain;
    int                                 next_ready;
    globus_result_t                     result;
    globus_hashtable_t                  nsprefix_table;
    globus_i_soap_message_namespace_t * default_ns;
    int                                 c14n_subset;
    globus_hashtable_t                  c14n_nsprefix_table;
    globus_i_soap_message_namespace_t * c14n_default_ns;
    int                                 nsp_count;
    xmlDictPtr                          dict;
    int                                 response_semaphore;
    void *                              response_handle;
    globus_mutex_t                      mutex;
    globus_byte_t                       readbuf[GLOBUS_I_SOAP_MESSAGE_READBUF_SIZE];
    globus_mutex_t                      cancel_mutex;
    globus_bool_t                       cancelled;
    globus_hashtable_t                  callout_table;
    globus_bool_t                       verbose;
} globus_i_soap_message_handle_t;

typedef struct
{
    xsd_QName *                         type;
    globus_list_t *                     entries;
} globus_i_soap_message_callouts_t;

typedef struct
{
    globus_soap_message_callout_func_t  begin_callout;
    globus_soap_message_callout_func_t  end_callout;
    globus_soap_message_callout_func_t  attr_callout;
    globus_soap_message_callout_ns_func_t  ns_set_callout;
    globus_soap_message_callout_ns_func_t  ns_add_callout;
    globus_soap_message_callout_ns_func_t  ns_remove_callout;
} globus_i_soap_message_callout_entry_t;

void
globus_i_soap_message_namespace_destroy(
    globus_i_soap_message_namespace_t * ns_handle);

globus_result_t
globus_i_soap_message_reset_reader(
    globus_soap_message_handle_t        handle);

globus_result_t
globus_i_soap_message_setup_unbuffered_writer(
    globus_soap_message_handle_t        message);

globus_result_t
globus_i_soap_message_deserialize_read(
    globus_soap_message_handle_t        handle);

globus_result_t
globus_i_soap_message_deserialize_next_element(
    globus_soap_message_handle_t        handle);

globus_result_t
globus_i_soap_message_deserialize_next_content(
    globus_soap_message_handle_t             handle);

globus_result_t
globus_i_soap_message_setup_transport_security(
    globus_hashtable_t                  props,
    globus_xio_attr_t                   attr);

globus_result_t
globus_i_soap_message_serialize_context_attributes(
    globus_soap_message_handle_t        message_handle);

void
globus_i_soap_message_callouts_destroy(
    void *                              vcallouts);

EXTERN_C_END

#endif /* _GLOBUS_I_SOAP_MESSAGE_H_ */
