/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */


#include "globus_i_service_engine.h"
#include "globus_service_registry.h"
#include "globus_soap_message.h"
#include "globus_soap_message_attrs.h"
#include "globus_soap_message_fault.h"
#include "globus_xio_tcp_driver.h"
#include "globus_xio_http.h"
#include "globus_xio_buffer.h"
#include "version.h"
#include "globus_flavor.h"
#include "libxml/uri.h"
#include "globus_ws_addressing.h"
#include "globus_soap_message_markers.h"
#include "globus_i_soap_message.h"
#include "globus_usage.h"

#define SOAP_ACTION_HEADER              "SOAPAction"
#define CONTENT_TYPE_HEADER             "Content-Type"

#define GLOBUS_I_SERVICE_DESCRIPTOR_KEY       "GLOBUS_SERVICE_DESCRIPTOR"
#define GLOBUS_I_SERVICE_EXTENSION_HANDLE_KEY "GLOBUS_SERVICE_EXTENSION_HANDLE"
#define GLOBUS_I_SESSION_HANDLE_KEY           "GLOBUS_ENGINE_CALLBACK_HANDLE"
#define GLOBUS_I_SERVICE_OPERATION_QNAME_KEY  "GLOBUS_SERVICE_OPERATION_QNAME"

static globus_list_t *              globus_l_service_engine_extensions = NULL;
static globus_mutex_t               globus_l_service_engine_extensions_mutex;

globus_extension_registry_t             globus_service_registry =
{
    NULL,
    GLOBUS_FALSE,
    GLOBUS_FALSE
};

static int
globus_l_service_engine_activate();

static int
globus_l_service_engine_deactivate();

globus_module_descriptor_t              globus_l_service_engine_module =
{
    "globus_service_engine_module",
    globus_l_service_engine_activate,
    globus_l_service_engine_deactivate,
    NULL,
    NULL,
    &local_version
};

GlobusDebugDefine(GLOBUS_SERVICE_ENGINE);

typedef enum
{
    GLOBUS_L_SERVICE_ENGINE_REQUEST_READ_MESSAGE,
    GLOBUS_L_SERVICE_ENGINE_REQUEST_INIT_SERVICE_HANDLERS,
    GLOBUS_L_SERVICE_ENGINE_REQUEST_INIT_HANDLERS,
    GLOBUS_L_SERVICE_ENGINE_START_DESERIALIZE,
    GLOBUS_L_SERVICE_ENGINE_TRIGGER_INVOKE_HANDLERS,
    GLOBUS_L_SERVICE_ENGINE_REQUEST_HEADER_FAILED,
    GLOBUS_L_SERVICE_ENGINE_REQUEST_INVOKE_HANDLERS,
    GLOBUS_L_SERVICE_ENGINE_REQUEST_INVOKE_SERVICE_HANDLERS,
    GLOBUS_L_SERVICE_ENGINE_REQUEST_DESERIALIZE,
    GLOBUS_L_SERVICE_ENGINE_REQUEST_DESTROY_SERVICE_HANDLERS,
    GLOBUS_L_SERVICE_ENGINE_REQUEST_DESTROY_HANDLERS,
    GLOBUS_L_SERVICE_ENGINE_REQUEST_INVOKE_OPERATION,
    GLOBUS_L_SERVICE_ENGINE_REQUEST_READ_ANOTHER_MESSAGE,
    GLOBUS_L_SERVICE_ENGINE_RESPONSE_INIT_SERVICE_HANDLERS,
    GLOBUS_L_SERVICE_ENGINE_RESPONSE_INIT_HANDLERS,
    GLOBUS_L_SERVICE_ENGINE_RESPONSE_INVOKE_SERVICE_HANDLERS,
    GLOBUS_L_SERVICE_ENGINE_RESPONSE_INVOKE_HANDLERS,
    GLOBUS_L_SERVICE_ENGINE_RESPONSE_DESTROY_SERVICE_HANDLERS,
    GLOBUS_L_SERVICE_ENGINE_RESPONSE_DESTROY_HANDLERS,
    GLOBUS_L_SERVICE_ENGINE_RESPONSE_SEND,
    GLOBUS_L_SERVICE_ENGINE_RESPONSE_CLOSE,
    GLOBUS_L_SERVICE_ENGINE_RESPONSE_DONE
} globus_l_service_session_state_t;

static char * globus_l_service_engine_state_strings[22] =
{
    "GLOBUS_L_SERVICE_ENGINE_REQUEST_READ_MESSAGE",
    "GLOBUS_L_SERVICE_ENGINE_REQUEST_INIT_SERVICE_HANDLERS",
    "GLOBUS_L_SERVICE_ENGINE_REQUEST_INIT_HANDLERS",
    "GLOBUS_L_SERVICE_ENGINE_START_DESERIALIZE",
    "GLOBUS_L_SERVICE_ENGINE_TRIGGER_INVOKE_HANDLERS",
    "GLOBUS_L_SERVICE_ENGINE_REQUEST_HEADER_FAILED",
    "GLOBUS_L_SERVICE_ENGINE_REQUEST_INVOKE_HANDLERS",
    "GLOBUS_L_SERVICE_ENGINE_REQUEST_INVOKE_SERVICE_HANDLERS",
    "GLOBUS_L_SERVICE_ENGINE_REQUEST_DESERIALIZE",
    "GLOBUS_L_SERVICE_ENGINE_REQUEST_DESTROY_SERVICE_HANDLERS",
    "GLOBUS_L_SERVICE_ENGINE_REQUEST_DESTROY_HANDLERS",
    "GLOBUS_L_SERVICE_ENGINE_REQUEST_INVOKE_OPERATION",
    "GLOBUS_L_SERVICE_ENGINE_REQUEST_READ_ANOTHER_MESSAGE",
    "GLOBUS_L_SERVICE_ENGINE_RESPONSE_INIT_SERVICE_HANDLERS",
    "GLOBUS_L_SERVICE_ENGINE_RESPONSE_INIT_HANDLERS",
    "GLOBUS_L_SERVICE_ENGINE_RESPONSE_INVOKE_SERVICE_HANDLERS",
    "GLOBUS_L_SERVICE_ENGINE_RESPONSE_INVOKE_HANDLERS",
    "GLOBUS_L_SERVICE_ENGINE_RESPONSE_DESTROY_SERVICE_HANDLERS",
    "GLOBUS_L_SERVICE_ENGINE_RESPONSE_DESTROY_HANDLERS",
    "GLOBUS_L_SERVICE_ENGINE_RESPONSE_SEND",
    "GLOBUS_L_SERVICE_ENGINE_RESPONSE_CLOSE",
    "GLOBUS_L_SERVICE_ENGINE_RESPONSE_DONE"
};

typedef struct globus_l_session_callback_handle_s
{
    globus_service_engine_t                engine;
    globus_soap_message_handle_t           message;
    globus_service_session_callback_func_t callback;
    void *                                 args;
    globus_l_service_session_state_t       state;
    globus_result_t                        result;
    globus_result_t                        callback_result;
    globus_service_response_t              response;

    globus_service_session_callback_func_t process_callback;
    void *                                 process_args;
    globus_bool_t                          faulting;
} globus_l_session_callback_handle_t;

typedef struct
{
    globus_service_engine_t               engine;
    globus_service_engine_callback_func_t callback;
    void *                                args;
    globus_result_t                       result;
} globus_l_service_engine_callback_handle_t;

static
void
globus_l_service_session_callback(
    void *                              args);

static
void
globus_l_service_session_callback_handle_destroy(
    globus_l_session_callback_handle_t * callback_handle);

static
void
globus_l_service_engine_open_callback(
    globus_xio_handle_t                 handle,
    globus_result_t                     result,
    void *                              user_arg);

static
void
globus_l_service_engine_service_module_destroy(
    void *                              ext);

static
void
globus_l_service_engine_request_ready_callback(
    globus_xio_handle_t                 handle,
    globus_result_t                     result,
    globus_byte_t *                     buffer,
    globus_size_t                       len,
    globus_size_t                       nbytes,
    globus_xio_data_descriptor_t        data_desc,
    void *                              user_arg);

static
void
globus_l_service_engine_accept_callback(
    globus_xio_server_t                 server,
    globus_xio_handle_t                 new_handle,
    globus_result_t                     result,
    void *                              user_args);

static
void
globus_l_service_session_result_callback(
    globus_result_t                     result,
    void *                              args);

static
void
globus_l_service_engine_send_client_fault(
    void *                              args);

static
void 
globus_l_service_engine_session_complete_callback(
    globus_result_t                     result,
    globus_service_engine_t             engine,
    globus_soap_message_handle_t        session,
    void *                              args);

static
void
globus_l_service_engine_session_started_callback(
    globus_result_t                     result,
    globus_service_engine_t             engine,
    globus_soap_message_handle_t        session,
    void *                              args);

static globus_usage_stats_handle_t globus_l_service_engine_usage;

static int
globus_l_service_engine_activate()
{
    int                                 rc = 0;
    GlobusFuncName(globus_l_service_engine_activate);

    rc = globus_module_activate(GLOBUS_COMMON_MODULE);
    if(rc != GLOBUS_SUCCESS)
    {
        return rc;
    }

    GlobusDebugInit(GLOBUS_SERVICE_ENGINE, DEBUG INFO TRACE WARN ERROR);
    GlobusServiceEngineDebugEnter();

    rc = globus_module_activate(GLOBUS_SOAP_MESSAGE_MODULE);
    if(rc != GLOBUS_SUCCESS)
    {
        globus_module_deactivate(GLOBUS_COMMON_MODULE);
        goto error_exit;
    }
    
    rc = globus_module_activate(GLOBUS_OPERATION_PROVIDER_MODULE);
    if(rc != GLOBUS_SUCCESS)
    {
        globus_module_deactivate(GLOBUS_SOAP_MESSAGE_MODULE);
        globus_module_deactivate(GLOBUS_COMMON_MODULE);
        goto error_exit;
    }

    rc = globus_module_activate(GLOBUS_USAGE_MODULE);
    if(rc != GLOBUS_SUCCESS)
    {
        globus_module_deactivate(GLOBUS_OPERATION_PROVIDER_MODULE);
        globus_module_deactivate(GLOBUS_SOAP_MESSAGE_MODULE);
        globus_module_deactivate(GLOBUS_COMMON_MODULE);
        goto error_exit;
    }

    rc = globus_usage_stats_handle_init(
        &globus_l_service_engine_usage,
        4, 1, NULL);
    if(rc == GLOBUS_SUCCESS)
    {
	globus_usage_stats_send(globus_l_service_engine_usage, 0);
    }
    else
    {
	globus_object_free(globus_error_get(rc));
	rc = GLOBUS_SUCCESS;
    }

    globus_mutex_init(&globus_l_service_engine_extensions_mutex, NULL);
    
    goto exit;

 error_exit:

    GlobusServiceEngineDebugExit();
    GlobusDebugDestroy(GLOBUS_SERVICE_ENGINE);
    return rc;

 exit:

    GlobusServiceEngineDebugExit();
    return rc;
}

static
void
globus_l_service_engine_extension_destroy(
    void *                              sym)
{
    char *                              symbol = (char *) sym;

    globus_extension_deactivate(sym);
    globus_free(symbol);
}

static int
globus_l_service_engine_deactivate()
{
    int                                 rc = 0;
    GlobusFuncName(globus_l_service_engine_deactivate);
    GlobusServiceEngineDebugEnter();

    globus_usage_stats_handle_destroy(globus_l_service_engine_usage);
    
    globus_mutex_lock(&globus_l_service_engine_extensions_mutex);
    globus_list_destroy_all(globus_l_service_engine_extensions,
                            globus_l_service_engine_extension_destroy);
    globus_mutex_unlock(&globus_l_service_engine_extensions_mutex);
    globus_mutex_destroy(&globus_l_service_engine_extensions_mutex);

    rc = globus_module_deactivate(GLOBUS_USAGE_MODULE);
    if(rc != GLOBUS_SUCCESS)
    {
        goto exit;
    }

    rc = globus_module_deactivate(GLOBUS_OPERATION_PROVIDER_MODULE);
    if(rc != GLOBUS_SUCCESS)
    {
        goto exit;
    }

    rc = globus_module_deactivate(GLOBUS_SOAP_MESSAGE_MODULE);
    if(rc != GLOBUS_SUCCESS)
    {
        goto exit;
    }

 exit:

    GlobusServiceEngineDebugExit();
    GlobusDebugDestroy(GLOBUS_SERVICE_ENGINE);
    globus_module_deactivate(GLOBUS_COMMON_MODULE);

    return rc;
}

static
globus_bool_t
globus_l_service_engine_timeout_callback(
    globus_xio_handle_t                 handle,
    globus_xio_operation_type_t         type,
    void *                              user_arg)
{
    return GLOBUS_TRUE;
}

globus_result_t
globus_service_engine_init(
    globus_service_engine_t *           new_engine,
    globus_soap_message_attr_t          attrs,
    const char *                        port_contact,
    globus_handler_chain_t              handlers,
    globus_bool_t                       enable_https)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    int                                 port = 0;
    globus_i_service_engine_t *         engine = NULL;
    globus_xio_attr_t                   server_attr = NULL;
    char *                              contact = NULL;
    void *                              timeout = NULL;
    globus_soap_message_auth_method_t   secattr;
    GlobusFuncName(globus_service_engine_init);
    GlobusServiceEngineDebugEnter();

    engine = malloc(sizeof(globus_i_service_engine_t));
    if(!engine)
    {
        result = GlobusServiceEngineErrorOutOfMemory;
        goto exit;
    }
    memset(engine, 0, sizeof(globus_i_service_engine_t));

    if(!attrs)
    {
        result = globus_soap_message_attr_init(&engine->attrs);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusServiceEngineErrorInitFailed(result);
            goto exit;
        }
    }
    else
    {
        result = globus_soap_message_attr_copy(
            &engine->attrs, attrs);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusServiceEngineErrorInitFailed(result);
            goto exit;
        }
    }

    result = globus_xio_attr_init(&server_attr);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusServiceEngineErrorInitFailed(result);
        goto error;
    }
        
    if(port_contact)
    {
        port = atoi(port_contact);
        if(port < 0)
        {
            result = GlobusServiceEngineErrorInvalidContact(port_contact);
            goto error;
        }

        result = globus_xio_attr_cntl(server_attr, 
                                      globus_i_soap_message_tcp_driver,
                                      GLOBUS_XIO_TCP_SET_PORT, port);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusServiceEngineErrorInitFailed(result);
            goto error;
        }
        
        result = globus_xio_attr_cntl(server_attr, 
                                      globus_i_soap_message_tcp_driver,
                                      GLOBUS_XIO_TCP_SET_REUSEADDR,
                                      GLOBUS_TRUE);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusServiceEngineErrorInitFailed(result);
            goto error;
        }
    }
    result = globus_xio_attr_cntl(server_attr,
                                  globus_i_soap_message_tcp_driver,
                                  GLOBUS_XIO_TCP_SET_NODELAY,
                                  GLOBUS_TRUE);

    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusServiceEngineErrorInitFailed(result);
        goto error;
    }


    timeout = globus_soap_message_attr_get(
        engine->attrs, GLOBUS_SOAP_MESSAGE_TIMEOUT_KEY);
    if(timeout)
    {
        result = globus_xio_attr_cntl(
            server_attr,
            GLOBUS_NULL,
            GLOBUS_XIO_ATTR_SET_TIMEOUT_ALL,
            globus_l_service_engine_timeout_callback,
            timeout,
            GLOBUS_NULL);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusSoapMessageErrorFailedTransport(
                result, "Failed to set timeout for xio handle");
            goto exit;
        }
    }

    if(enable_https)
    {
        result = globus_i_soap_message_setup_transport_security(
            engine->attrs,
            server_attr);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusServiceEngineErrorInitFailed(result);
            goto error;
        }

        result = globus_xio_server_create(
            &engine->server,
            server_attr,
            globus_i_soap_message_xio_https_stack);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusServiceEngineErrorInitFailed(result);
            goto error;
        }
    }
    else
    {
        result = globus_xio_server_create(&engine->server, 
                                          server_attr, 
                                          globus_i_soap_message_xio_stack);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusServiceEngineErrorInitFailed(result);
            goto error;
        }
    }

    globus_mutex_init(&engine->mutex, NULL);

    globus_xio_server_get_contact_string(engine->server, &contact);
    engine->contact = globus_common_create_string(
        "http%s://%s/",
        (enable_https ? "s" : ""),
        contact);
    free(contact);

    if(!handlers)
    {
        int                             res = 0;
        res = globus_extension_activate(GLOBUS_HANDLER_WS_ADDRESSING_LIB);
        if(res != 0)
        {
            globus_object_t *           tmp = globus_error_get((globus_result_t) res);
            result = GlobusServiceEngineErrorHandlerLoadFailed(
                GLOBUS_HANDLER_WS_ADDRESSING_MODULE,
                tmp);
            goto error;
        }
        
        globus_mutex_lock(&globus_l_service_engine_extensions_mutex);
        globus_list_insert(&globus_l_service_engine_extensions,
                           strdup(GLOBUS_HANDLER_WS_ADDRESSING_LIB));
        globus_mutex_unlock(&globus_l_service_engine_extensions_mutex);

        globus_handler_chain_init(&engine->handlers);
        
        result = globus_handler_chain_push(
            engine->handlers, 
            GLOBUS_HANDLER_TYPE_REQUEST_ALL,
            GLOBUS_HANDLER_WS_ADDRESSING_SERVER);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusServiceEngineErrorInitFailed(result);
            goto error;
        }
        
        result = globus_handler_chain_push(
            engine->handlers, 
            GLOBUS_HANDLER_TYPE_RESPONSE_ALL,
            GLOBUS_HANDLER_WS_ADDRESSING_SERVER);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusServiceEngineErrorInitFailed(result);
            goto error;
        }

        secattr = (globus_soap_message_auth_method_t)
            globus_soap_message_attr_get(
                engine->attrs,
                GLOBUS_SOAP_MESSAGE_AUTHENTICATION_METHOD_KEY);

        if ((!enable_https) &&
                (secattr == GLOBUS_SOAP_MESSAGE_AUTH_SECURE ||
                 secattr == GLOBUS_SOAP_MESSAGE_AUTH_SECURE_MESSAGE))
        {
            res = globus_extension_activate("globus_handler_ws_secure_message");
            if(res != 0)
            {
                globus_object_t *       tmp = globus_error_get((globus_result_t) res);
                result = GlobusServiceEngineErrorHandlerLoadFailed(
                    "globus_handler_ws_secure_message",
                    tmp);
                goto error;
            }
            
            globus_mutex_lock(&globus_l_service_engine_extensions_mutex);
            globus_list_insert(&globus_l_service_engine_extensions,
                               strdup("globus_handler_ws_secure_message"));
            globus_mutex_unlock(&globus_l_service_engine_extensions_mutex);

            result = globus_handler_chain_push(
                engine->handlers,
                GLOBUS_HANDLER_TYPE_REQUEST,
                "globus_handler_ws_secure_message_server");
            if(result != GLOBUS_SUCCESS)
            {
                result = GlobusServiceEngineErrorInitFailed(result);
                goto error;
            }

            result = globus_handler_chain_push(
                engine->handlers,
                GLOBUS_HANDLER_TYPE_RESPONSE_ALL,
                "globus_handler_ws_secure_message_server");
            if(result != GLOBUS_SUCCESS)
            {
                result = GlobusServiceEngineErrorInitFailed(result);
                goto error;
            }
        }
    }
    else
    {
        globus_handler_chain_copy(&engine->handlers, handlers);
    }

    *new_engine = (globus_service_engine_t) engine;
    goto exit;

 error:

    if(server_attr)
    {
        globus_xio_attr_destroy(server_attr);
    }

    if(engine)
    {
        if(engine->server)
        {
            globus_xio_server_close(engine->server);
        }

        free(engine);
    }

 exit:

    GlobusServiceEngineDebugExit();
    return result;
}

globus_result_t
globus_service_engine_get_contact(
    globus_service_engine_t             engine,
    char **                             contact)
{
    *contact = globus_libc_strdup(engine->contact);
    return GLOBUS_SUCCESS;
}

void
globus_service_engine_destroy(
    globus_service_engine_t             engine)
{
    int                                 res;
    GlobusFuncName(globus_service_engine_destroy);
    GlobusServiceEngineDebugEnter();
    
    if(engine->handlers)
    {
        globus_handler_chain_destroy(engine->handlers);
    }

    if(engine->contact)
    {
        free(engine->contact);
    }

    if(engine->server)
    {
        globus_xio_server_close(engine->server);
    }

    res = globus_mutex_destroy(&engine->mutex);
    globus_assert_string((res == 0), "Failed to destroy mutex");

    if(engine->attrs)
    {
        globus_soap_message_attr_destroy(engine->attrs);
    }
    
    free(engine);
    GlobusServiceEngineDebugExit();
}

static
void
globus_l_service_session_callback_handle_destroy(
    globus_l_session_callback_handle_t * callback_handle)
{
    if(callback_handle)
    {
        if(callback_handle->result)
        {
            globus_object_free(globus_error_get(callback_handle->result));
        }

        if(callback_handle->callback_result)
        {
            globus_object_free(
                globus_error_get(callback_handle->callback_result));
        }
               
        free(callback_handle);
    }
}

static
void
globus_l_service_engine_open_callback(
    globus_xio_handle_t                 handle,
    globus_result_t                     result,
    void *                              user_arg)
{
    globus_l_session_callback_handle_t *callback_handle = NULL;
    static globus_byte_t                buffer[1];
    GlobusFuncName(globus_l_service_engine_open_callback);
    GlobusServiceEngineDebugEnter();

    callback_handle = (globus_l_session_callback_handle_t *) user_arg;

    globus_mutex_lock(&callback_handle->engine->mutex);
    if(result == GLOBUS_SUCCESS)
    {
        result = globus_xio_register_read(handle,
                buffer,
                0,
                0,
                NULL,
                globus_l_service_engine_request_ready_callback,
                callback_handle);
    }

    if (result != GLOBUS_SUCCESS)
    {
        globus_soap_message_handle_destroy(callback_handle->message);
        callback_handle->message = NULL;
        callback_handle->callback(
            result, callback_handle->engine,
            callback_handle->message, callback_handle->args);
    }
    globus_mutex_unlock(&callback_handle->engine->mutex);
    
    GlobusServiceEngineDebugExit();
}

static
globus_result_t
globus_l_service_engine_service_module_copy(
    void **                             ne,
    void *                              e)
{
    globus_extension_reference((globus_extension_handle_t)e);
    *ne = e;
    return GLOBUS_SUCCESS;
}

static
void
globus_l_service_engine_service_module_destroy(
    void *                              e)
{
    globus_extension_release((globus_extension_handle_t)e);
}

static
void
globus_l_service_engine_request_ready_callback(
    globus_xio_handle_t                 handle,
    globus_result_t                     result,
    globus_byte_t *                     buffer,
    globus_size_t                       len,
    globus_size_t                       nbytes,
    globus_xio_data_descriptor_t        data_desc,
    void *                              user_arg)
{
    const char *                        uriref = NULL;
    char *                              service_path = NULL;
    globus_xio_http_header_t *          soap_action = NULL;
    globus_xio_http_header_t *          content_type = NULL;
    globus_xio_http_header_t *          connection = NULL;
    char *                              method = NULL;
    char *                              uri = NULL;
    globus_xio_http_version_t           http_version;
    globus_hashtable_t                  headers;
    
    globus_l_session_callback_handle_t * callback_handle = NULL;
    GlobusFuncName(globus_l_service_engine_request_ready_callback);
    GlobusServiceEngineDebugEnter();

    callback_handle = (globus_l_session_callback_handle_t *) user_arg;

    if (result == GLOBUS_SUCCESS)
    {
        result = globus_xio_data_descriptor_cntl(
                data_desc,
                globus_i_soap_message_http_driver,
                GLOBUS_XIO_HTTP_GET_REQUEST,
                &method,
                &uri,
                &http_version,
                &headers);

    }

    if (method == NULL || result != GLOBUS_SUCCESS)
    {
        /* No more requests */
        if (callback_handle->process_callback != NULL)
        {
            callback_handle->state = GLOBUS_L_SERVICE_ENGINE_RESPONSE_CLOSE;
            globus_soap_message_handle_set_attr(
                callback_handle->message,
                GLOBUS_SOAP_MESSAGE_CONNECTION_KEY,
                globus_soap_message_attr_copy_string,
                globus_libc_free,
                (void *) "close");
        }
    }
    else if(!strcmp(method, "POST"))
    {
        globus_extension_handle_t       ext;
        globus_service_descriptor_t *   service_desc;

        soap_action = (globus_xio_http_header_t *)
        globus_hashtable_lookup(&headers, (void *) SOAP_ACTION_HEADER);
        if(!soap_action)
        {
            GlobusServiceEngineDebugPrintf(
                GLOBUS_L_SERVICE_ENGINE_DEBUG_DEBUG, 
                ("Service request: %s does not "
                 "have a SOAPAction header entry\n",
                 uri));
        }

        content_type = (globus_xio_http_header_t *)
        globus_hashtable_lookup(&headers, (void *) CONTENT_TYPE_HEADER);
        if(!content_type || !content_type->value || 
           !strstr(content_type->value, "text/xml"))
        {
            result = GlobusServiceEngineErrorUnsupportedMediaType(
                content_type->value);
            goto exit;
        }
        if (http_version != GLOBUS_XIO_HTTP_VERSION_1_0)
        {
            /*
             * Assume persistent connection with HTTP/1.1 unless told 
             * otherwise
             */
            connection = globus_hashtable_lookup(
                    &headers,
                    (void *) "Connection");
            if (connection && connection->value &&
                        strstr(connection->value, "close"))
            {
                result = globus_soap_message_handle_set_attr(
                    callback_handle->message,
                    GLOBUS_SOAP_MESSAGE_CONNECTION_KEY,
                    globus_soap_message_attr_copy_string,
                    globus_libc_free,
                    (void *) "close");
            }
        }
        else
        {
            result = globus_soap_message_handle_set_attr(
                callback_handle->message,
                GLOBUS_SOAP_MESSAGE_CONNECTION_KEY,
                globus_soap_message_attr_copy_string,
                globus_libc_free,
                (void *) "close");
        }
        if (result != GLOBUS_SUCCESS)
        {
            goto exit;
        }

        uriref = uri;
        while (*uriref == '/')
        {
            uriref++;
        }

        service_path = globus_common_create_string(
            GLOBUS_SERVICE_ENGINE_MODULE_PATH_PREFIX "/%s", uriref);
        service_desc = globus_extension_lookup(
            &ext,
            GLOBUS_SERVICE_REGISTRY,
            service_path);
        if(!service_desc)
        {
            int                         res = 0;

            res = globus_extension_activate(
                service_path);
            if(res != GLOBUS_SUCCESS)
            {
                globus_object_t *       tmp_err = globus_error_get((globus_result_t) res);
		if(GlobusServiceEngineDebug(
			GLOBUS_L_SERVICE_ENGINE_DEBUG_DEBUG))
		{
		    char * serror = globus_error_print_chain(
			tmp_err);
		    GlobusServiceEngineDebugPrintf(
			GLOBUS_L_SERVICE_ENGINE_DEBUG_DEBUG,
			("Module Activation Failed:\n\n%s\n",
			serror));
		    globus_free(serror);
		}

                result = GlobusServiceEngineErrorServiceLoadFailed(uri, tmp_err);
                goto exit;
            }

            globus_mutex_lock(&globus_l_service_engine_extensions_mutex);
            globus_list_insert(&globus_l_service_engine_extensions,
                               strdup(service_path));
            globus_mutex_unlock(&globus_l_service_engine_extensions_mutex);

            service_desc = (globus_service_descriptor_t *) 
            globus_extension_lookup(&ext, GLOBUS_SERVICE_REGISTRY, 
                                    service_path);
            if(!service_desc)
            {
                result = GlobusServiceEngineErrorServiceLoadFailed(uri, NULL);
                goto exit;
            }
        }

        result = globus_soap_message_handle_set_attr(
            callback_handle->message,
            GLOBUS_SOAP_MESSAGE_SERVICE_ENDPOINT_KEY,
            globus_soap_message_attr_copy_string,
            globus_libc_free,
            (void *)uri);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusServiceEngineErrorReadRequestFailed(result);
            goto exit;
        }

        result = globus_soap_message_handle_set_attr(
            callback_handle->message,
            GLOBUS_I_SERVICE_DESCRIPTOR_KEY,
            globus_l_service_engine_service_module_copy,
            globus_l_service_engine_service_module_destroy,
            (void *)ext);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusServiceEngineErrorReadRequestFailed(result);
            goto exit;
        }

        if(soap_action)
        {
            result = globus_soap_message_handle_set_attr(
                callback_handle->message,
                GLOBUS_SOAP_MESSAGE_SOAP_ACTION_KEY,
                globus_soap_message_attr_copy_string,
                globus_libc_free,
                (void *)soap_action->value);
            if(result != GLOBUS_SUCCESS)
            {
                result = GlobusServiceEngineErrorReadRequestFailed(result);
                goto exit;
            }
        }
    }
    else
    {
        result = GlobusServiceEngineErrorMethodNotAllowed(method);
        goto exit;
    }

    if (callback_handle->state ==
            GLOBUS_L_SERVICE_ENGINE_REQUEST_READ_ANOTHER_MESSAGE ||
        callback_handle->state ==
            GLOBUS_L_SERVICE_ENGINE_RESPONSE_CLOSE)
    {
        result = globus_callback_register_oneshot(
            NULL,
            &globus_i_reltime_zero,
            globus_l_service_session_callback,
            callback_handle);
        if(result == GLOBUS_SUCCESS)
        {
            goto no_callback_exit;
        }
        else
        {
            result = GlobusServiceEngineErrorFailedRequestProcess(
                result, "Failed to register callback to invoke operation");
            goto exit;
        }
    }
 exit:

    if(result != GLOBUS_SUCCESS)
    {
        callback_handle->result = result;
        globus_soap_message_handle_set_attr(
            callback_handle->message,
            GLOBUS_SOAP_MESSAGE_CONNECTION_KEY,
            globus_soap_message_attr_copy_string,
            globus_libc_free,
            (void *) "close");
    }

    callback_handle->callback(
        result, callback_handle->engine,
        callback_handle->message, callback_handle->args);

no_callback_exit:

    if(service_path)
    {
        free(service_path);
    }


    GlobusServiceEngineDebugExit();
}

static
void
globus_l_service_engine_accept_callback(
    globus_xio_server_t                 server,
    globus_xio_handle_t                 new_handle,
    globus_result_t                     result,
    void *                              user_args)
{
    char *                              peer_contact = NULL;
    globus_xio_attr_t                   attr;
    globus_soap_message_handle_t        message = NULL;
    globus_l_session_callback_handle_t * callback_handle;
    GlobusFuncName(globus_l_service_engine_accept_callback);
    GlobusServiceEngineDebugEnter();
    
    callback_handle = (globus_l_session_callback_handle_t *) user_args;

    if(result != GLOBUS_SUCCESS)
    {
        goto exit;
    }

    result = globus_soap_message_handle_init(&message, new_handle);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusServiceEngineErrorFailedRequest(result);
        goto exit;
    }
    globus_mutex_lock(&callback_handle->engine->mutex);
    result = globus_soap_message_handle_set_attrs(
        message,
        callback_handle->engine->attrs);
    if(result != GLOBUS_SUCCESS)
    {
        globus_mutex_unlock(&callback_handle->engine->mutex);
        result = GlobusServiceEngineErrorFailedRequest(result);
        goto exit;
    }
    result = globus_soap_message_handle_set_attr(
        message,
        GLOBUS_I_SESSION_HANDLE_KEY,
        NULL,
        NULL,
        callback_handle);
    if(result != GLOBUS_SUCCESS)
    {
        globus_mutex_unlock(&callback_handle->engine->mutex);
        result = GlobusServiceEngineErrorFailedRequest(result);
        goto exit;
    }

    globus_mutex_unlock(&callback_handle->engine->mutex);
        
    callback_handle->message = message;

    result = globus_xio_attr_init(&attr);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusServiceEngineErrorFailedRequest(result);
        goto exit;
    }

    result = globus_xio_handle_cntl(
        new_handle, globus_i_soap_message_tcp_driver,
        GLOBUS_XIO_TCP_GET_REMOTE_CONTACT,
        &peer_contact);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusServiceEngineErrorFailedRequest(result);
        goto exit;
    }

    result = globus_xio_register_open(
        new_handle, peer_contact, attr, 
        globus_l_service_engine_open_callback, callback_handle);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusServiceEngineErrorFailedRequest(result);
        goto exit;
    }

 exit:
    if (peer_contact != NULL)
    {
        globus_libc_free(peer_contact);
    }

    if(result != GLOBUS_SUCCESS)
    {
        callback_handle->callback(result, callback_handle->engine, 
                                  message, callback_handle->args);
        free(callback_handle);
        if(message)
        {
            globus_soap_message_handle_destroy(message);
        }
    }

    GlobusServiceEngineDebugExit();
}

globus_result_t
globus_service_engine_register_session(
    globus_service_engine_t                engine,
    globus_service_session_callback_func_t callback,
    void *                                 user_args)
{
    globus_l_session_callback_handle_t * callback_handle = NULL;
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_service_engine_register_session);
    GlobusServiceEngineDebugEnter();

    callback_handle = calloc(1, sizeof(globus_l_session_callback_handle_t));
    if(!callback_handle)
    {
        result = GlobusServiceEngineErrorOutOfMemory;
        goto exit;
    }

    callback_handle->callback = callback;
    callback_handle->args = user_args;
    callback_handle->engine = engine;
    callback_handle->faulting = GLOBUS_FALSE;

    result = globus_xio_server_register_accept(
        engine->server,
        globus_l_service_engine_accept_callback,
        callback_handle);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusServiceEngineErrorAcceptFailed(result);
        goto exit;
    }

 exit:

    GlobusServiceEngineDebugExit();
    return result;
}

static
void
globus_l_service_session_result_callback(
    globus_result_t                    result,
    void *                             args)
{
    globus_l_session_callback_handle_t * callback_handle;
    GlobusFuncName(globus_l_service_session_result_callback);
    GlobusServiceEngineDebugEnter();

    callback_handle = (globus_l_session_callback_handle_t *) args;
    callback_handle->result = result;

    globus_l_service_session_callback(callback_handle);

    GlobusServiceEngineDebugExit();
}

static
void
globus_l_service_session_callback(
    void *                              args)
{
    globus_result_t                      result = GLOBUS_SUCCESS;
    globus_service_descriptor_t *        service_desc = NULL;
    globus_extension_handle_t            ext;
    globus_l_session_callback_handle_t * callback_handle;
    globus_service_operation_descriptor_t * op_desc = NULL;
    xsd_QName *                          operation_name = NULL;
    xsd_QName *                          header_element = NULL;
    xsd_int                              must = 0;
    char *                               connection;
    globus_xio_handle_t                  xio_handle;
    static globus_byte_t                 buffer[1];
    GlobusFuncName(globus_l_service_session_callback);
    GlobusServiceEngineDebugEnter();

    callback_handle = (globus_l_session_callback_handle_t *) args;

    if(callback_handle->result != GLOBUS_SUCCESS)
    {
        if(!callback_handle->faulting)
        {
            result = callback_handle->result;
            goto error_send_fault;
        }
        else
        {
            char * errstr;

            errstr = globus_error_print_friendly(
                    globus_error_peek(
                        callback_handle->result));

            GlobusServiceEngineDebugPrintf(
                GLOBUS_L_SERVICE_ENGINE_DEBUG_DEBUG,
                ("error occurred while sending a fault: %s\n", 
                 errstr));

            free(errstr);

            callback_handle->result = GLOBUS_SUCCESS;
        }
    }

    GlobusServiceEngineDebugPrintf(
        GLOBUS_L_SERVICE_ENGINE_DEBUG_DEBUG,
        ("STATE: %s\n", 
	 globus_l_service_engine_state_strings[callback_handle->state]));

    switch(callback_handle->state)
    {

    case GLOBUS_L_SERVICE_ENGINE_REQUEST_READ_ANOTHER_MESSAGE:
        result = globus_soap_message_get_transport_handle(
            callback_handle->message,
            &xio_handle);
        if (result != GLOBUS_SUCCESS)
        {
            goto error_send_fault;
        }
        result = globus_xio_handle_cntl(
                xio_handle,
                globus_i_soap_message_buffer_driver,
                GLOBUS_XIO_BUFFER_ENABLE_READ_BUFFERING);
        if (result != GLOBUS_SUCCESS)
        {
            goto error_send_fault;
        }
        result = globus_i_soap_message_reset_reader(callback_handle->message);
        if (result != GLOBUS_SUCCESS)
        {
            goto error_send_fault;
        }
        /* FALLSTHROUGH */
    case GLOBUS_L_SERVICE_ENGINE_REQUEST_READ_MESSAGE:

        callback_handle->state =
        GLOBUS_L_SERVICE_ENGINE_REQUEST_INIT_SERVICE_HANDLERS;

        if(callback_handle->result != GLOBUS_SUCCESS)
        {
            result = callback_handle->result;
            goto error_send_fault;
        }

        result = globus_soap_message_register_read_request(
            callback_handle->message,
            globus_l_service_session_result_callback,
            callback_handle);
        if(result != GLOBUS_SUCCESS)
        {
            goto error_send_fault;
        }

        break;

    case GLOBUS_L_SERVICE_ENGINE_REQUEST_INIT_SERVICE_HANDLERS:

	callback_handle->state =
	GLOBUS_L_SERVICE_ENGINE_REQUEST_INIT_HANDLERS;

	if(callback_handle->result != GLOBUS_SUCCESS)
	{
	    result = callback_handle->result;
	    goto error_send_fault;
	}

	ext =
        (globus_extension_handle_t)
        globus_soap_message_handle_get_attr(
            callback_handle->message,
            GLOBUS_I_SERVICE_DESCRIPTOR_KEY);
        globus_assert_string(
            ext, "No SERVICE DESCRIPTOR attribute found!");
                
        service_desc = (globus_service_descriptor_t *) 
        globus_extension_reference(ext);
        
        globus_assert_string(
            service_desc, "No SERVICE DESCRIPTOR attribute found!");
        
        globus_mutex_lock(&service_desc->mutex);
        if(service_desc->handlers)
        {
	    globus_handler_chain_register_invoke(
		    service_desc->handlers,
		    GLOBUS_HANDLER_TYPE_REQUEST_INIT,
		    callback_handle->message,
		    globus_l_service_session_result_callback,
		    callback_handle);
        
	    globus_mutex_unlock(&service_desc->mutex);
	    service_desc = NULL;
	    globus_extension_release(ext);
	    break;
	}
	globus_mutex_unlock(&service_desc->mutex);
	service_desc = NULL;
	globus_extension_release(ext);
	    
    case GLOBUS_L_SERVICE_ENGINE_REQUEST_INIT_HANDLERS:
	
	callback_handle->state =
	GLOBUS_L_SERVICE_ENGINE_START_DESERIALIZE;

	if(callback_handle->result != GLOBUS_SUCCESS)
	{
	    result = callback_handle->result;
	    goto error_send_fault;
	}
	
        globus_mutex_lock(&callback_handle->engine->mutex);
        if(callback_handle->engine->handlers)
        {
            globus_handler_chain_register_invoke(
                callback_handle->engine->handlers,
		GLOBUS_HANDLER_TYPE_REQUEST_INIT,
                callback_handle->message,
                globus_l_service_session_result_callback,
                callback_handle);
	    globus_mutex_unlock(&callback_handle->engine->mutex);
	    break;
        }
        globus_mutex_unlock(&callback_handle->engine->mutex);
	
    case GLOBUS_L_SERVICE_ENGINE_START_DESERIALIZE:

        callback_handle->state = 
        GLOBUS_L_SERVICE_ENGINE_TRIGGER_INVOKE_HANDLERS;

        if(callback_handle->result != GLOBUS_SUCCESS)
        {
            result = callback_handle->result;
            goto error_send_fault;
        }

        result = globus_soap_message_deserialize_envelope(
            callback_handle->message);
        if(result != GLOBUS_SUCCESS)
        {
            goto error_send_fault;
        }

        result = globus_soap_message_deserialize_element_begin_close(
            callback_handle->message);
        if(result != GLOBUS_SUCCESS)
        {
            goto error_send_fault;
        }

        result = globus_soap_message_deserialize_header(
            callback_handle->message);
        if(result != GLOBUS_SUCCESS)
        {
            if(GlobusSoapMessageStatusElementNotFoundCheck(result))
            {
                globus_soap_message_deserialize_push_element(
                    callback_handle->message);
                result = GLOBUS_SUCCESS;
                callback_handle->state = 
                GLOBUS_L_SERVICE_ENGINE_REQUEST_HEADER_FAILED;
                callback_handle->result = result;
                globus_l_service_session_callback(
                    callback_handle);
                break;
            }

            goto error_send_fault;
        }

    case GLOBUS_L_SERVICE_ENGINE_TRIGGER_INVOKE_HANDLERS:

        callback_handle->state = 
        GLOBUS_L_SERVICE_ENGINE_TRIGGER_INVOKE_HANDLERS;

        if(callback_handle->result != GLOBUS_SUCCESS)
        {
            result = callback_handle->result;
            goto error_send_fault;
        }

        xsd_QName_init(&header_element);
        
        result = globus_soap_message_deserialize_element_unknown(
            callback_handle->message,
            header_element);
        if(result != GLOBUS_SUCCESS)
        {
            if(!GlobusSoapMessageStatusFailedElementCheck(result))
            {
                xsd_QName_destroy(header_element);
                goto error_send_fault;
            }
            else
            {
                callback_handle->state = 
                GLOBUS_L_SERVICE_ENGINE_REQUEST_DESERIALIZE;
                globus_callback_register_oneshot(
                    NULL,
                    &globus_i_reltime_zero,
                    globus_l_service_session_callback,
                    (void *)callback_handle);
                xsd_QName_destroy(header_element);
                break;
            }
        }

        result = globus_soap_message_deserialize_int_attribute(
            callback_handle->message,
            &soap_mustUnderstand_qname,
            &must);
        if(result == GLOBUS_SUCCESS && (must == 1))
        {
            globus_soap_message_add_required_header_element(
                callback_handle->message,
                header_element);
        }
        else if(result != GLOBUS_SUCCESS &&
                !GlobusSoapMessageStatusAttributeNotFoundCheck(result))
        {
            xsd_QName_destroy(header_element);
            goto error_send_fault;
        }

        globus_soap_message_deserialize_push_element(callback_handle->message);

	ext =
        (globus_extension_handle_t)
        globus_soap_message_handle_get_attr(
            callback_handle->message,
            GLOBUS_I_SERVICE_DESCRIPTOR_KEY);
        globus_assert_string(
            ext, "No SERVICE DESCRIPTOR attribute found!");
                
        service_desc = (globus_service_descriptor_t *) 
        globus_extension_reference(ext);
        
        globus_assert_string(
            service_desc, "No SERVICE DESCRIPTOR attribute found!");
        
        globus_mutex_lock(&service_desc->mutex);
        if(service_desc->handlers)
        {
            if(globus_handler_chain_has_trigger(
		    service_desc->handlers, header_element))
	    {
		result = globus_handler_chain_register_trigger(
		    service_desc->handlers,
		    callback_handle->message,
		    header_element,
		    globus_l_service_session_result_callback,
		    callback_handle);
		if(result == GLOBUS_SUCCESS)
		{
		    xsd_QName_destroy(header_element);
		    globus_mutex_unlock(&service_desc->mutex);
		    service_desc = NULL;
		    globus_extension_release(ext);
		    break;
		}
		else if(!GlobusHandlerStatusNotTriggeredCheck(result))
		{
		    xsd_QName_destroy(header_element);
		    globus_mutex_unlock(&service_desc->mutex);
		    service_desc = NULL;
		    globus_extension_release(ext);
		    goto error_send_fault;
		}
	    }
        }
        globus_mutex_unlock(&service_desc->mutex);
        service_desc = NULL;
        globus_extension_release(ext);

        globus_mutex_lock(&callback_handle->engine->mutex);
        if(callback_handle->engine->handlers)
        {
            result = globus_handler_chain_register_trigger(
                callback_handle->engine->handlers,
                callback_handle->message,
                header_element,
                globus_l_service_session_result_callback,
                callback_handle);
            if(result == GLOBUS_SUCCESS)
            {
                xsd_QName_destroy(header_element);
                globus_mutex_unlock(&callback_handle->engine->mutex);
                break;
            }
            else if(!GlobusHandlerStatusNotTriggeredCheck(result))
            {
                xsd_QName_destroy(header_element);
                globus_mutex_unlock(&callback_handle->engine->mutex);
                goto error_send_fault;
            }
        }
        globus_mutex_unlock(&callback_handle->engine->mutex);
        
        xsd_QName_destroy(header_element);
        result = globus_soap_message_deserialize_skip(
            callback_handle->message);
        if(result != GLOBUS_SUCCESS)
        {
            goto error_send_fault;
        }

        globus_callback_register_oneshot(
            NULL,
            &globus_i_reltime_zero,
            globus_l_service_session_callback,
            (void *)callback_handle);
        break;

    case GLOBUS_L_SERVICE_ENGINE_REQUEST_DESERIALIZE:
        
        if(callback_handle->result != GLOBUS_SUCCESS)
        {
            result = callback_handle->result;
            goto error_send_fault;
        }

	result = globus_soap_message_check_required_headers(
	    callback_handle->message);
	if(result != GLOBUS_SUCCESS)
	{
	    goto error_send_fault;
	}
	
        result = globus_soap_message_deserialize_header_end(
            callback_handle->message);
        if(result != GLOBUS_SUCCESS)
        {
            goto error_send_fault;
        }

    case GLOBUS_L_SERVICE_ENGINE_REQUEST_HEADER_FAILED:

        callback_handle->state = 
        GLOBUS_L_SERVICE_ENGINE_REQUEST_INVOKE_HANDLERS;

        if(callback_handle->result != GLOBUS_SUCCESS)
        {
            result = callback_handle->result;
            goto error_send_fault;
        }

        result = globus_soap_message_deserialize_body(
            callback_handle->message);
        if(result != GLOBUS_SUCCESS)
        {
            goto error_send_fault;
        }

        xsd_QName_init(&operation_name);

        result = globus_soap_message_deserialize_element_unknown(
            callback_handle->message, operation_name);
        if(result != GLOBUS_SUCCESS)
        {
            xsd_QName_destroy(operation_name);
            goto error_send_fault;
        }

        result = globus_soap_message_handle_set_attr(
            callback_handle->message,
            GLOBUS_I_SERVICE_OPERATION_QNAME_KEY,
            xsd_QName_copy_wrapper,
            xsd_QName_destroy_wrapper,
            operation_name);
        if(result != GLOBUS_SUCCESS)
        {
            xsd_QName_destroy(operation_name);
            goto error_send_fault;
        }

        globus_soap_message_deserialize_push_element(
            callback_handle->message);
        if(result != GLOBUS_SUCCESS)
        {
            goto error_send_fault;
        }

        ext = 
        (globus_extension_handle_t)
        globus_soap_message_handle_get_attr(
            callback_handle->message,
            GLOBUS_I_SERVICE_DESCRIPTOR_KEY);
        globus_assert_string(
            ext, "No SERVICE DESCRIPTOR attribute found!");

        service_desc = (globus_service_descriptor_t *) 
        globus_extension_reference(ext);

        globus_assert_string(
            service_desc, "No SERVICE DESCRIPTOR attribute found!");

        /* we use qn to map to operation deserialize function */
        globus_mutex_lock(&service_desc->mutex);
        if(service_desc->op_mapper)
        {
            op_desc = (globus_service_operation_descriptor_t *)
            globus_hashtable_lookup(
                &service_desc->op_mapper, 
                (void *)operation_name);
        }

        if(!op_desc)
        {
            result = GlobusServiceEngineErrorOperationLookupFailed(
                service_desc->name, operation_name);
            globus_mutex_unlock(&service_desc->mutex);
            service_desc = NULL;
            globus_extension_release(ext);
            goto error_send_fault;
        }

        result = op_desc->deserialize(
            callback_handle->engine,
            callback_handle->message,
            globus_l_service_session_result_callback,
            callback_handle);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusServiceEngineErrorOpDeserializeFailed(
                result, operation_name);
            globus_mutex_unlock(&service_desc->mutex);
            goto error_send_fault;
        }

        globus_mutex_unlock(&service_desc->mutex);
        service_desc = NULL;
        globus_extension_release(ext);

        break;

    case GLOBUS_L_SERVICE_ENGINE_REQUEST_INVOKE_HANDLERS:

        callback_handle->state = 
        GLOBUS_L_SERVICE_ENGINE_REQUEST_INVOKE_SERVICE_HANDLERS;

        if(callback_handle->result != GLOBUS_SUCCESS)
        {
            result = callback_handle->result;
            goto error_send_fault;
        }

        result = globus_soap_message_deserialize_body_end(
            callback_handle->message);
        if(result != GLOBUS_SUCCESS)
        {
            goto error_send_fault;
        }

        result = globus_soap_message_deserialize_envelope_end(
            callback_handle->message);
        if(result != GLOBUS_SUCCESS)
        {
            goto error_send_fault;
        }

        globus_mutex_lock(&callback_handle->engine->mutex);
        if(callback_handle->engine->handlers)
        {        
            globus_handler_chain_register_invoke(
                callback_handle->engine->handlers,
                GLOBUS_HANDLER_TYPE_REQUEST,
                callback_handle->message,
                globus_l_service_session_result_callback,
                callback_handle);
            globus_mutex_unlock(&callback_handle->engine->mutex);
            break;
        }
        globus_mutex_unlock(&callback_handle->engine->mutex);

    case GLOBUS_L_SERVICE_ENGINE_REQUEST_INVOKE_SERVICE_HANDLERS:

        callback_handle->state = 
        GLOBUS_L_SERVICE_ENGINE_REQUEST_DESTROY_HANDLERS;

        if(callback_handle->result != GLOBUS_SUCCESS)
        {
            result = callback_handle->result;
            goto error_send_fault;
        }

        ext =
        (globus_extension_handle_t)
        globus_soap_message_handle_get_attr(
            callback_handle->message,
            GLOBUS_I_SERVICE_DESCRIPTOR_KEY);
        globus_assert_string(
            ext, "No SERVICE DESCRIPTOR attribute found!");
        
        service_desc = (globus_service_descriptor_t *) 
        globus_extension_reference(ext);
        
        globus_assert_string(
            service_desc, "No SERVICE DESCRIPTOR attribute found!");
        
        globus_mutex_lock(&service_desc->mutex);
        if(service_desc->handlers)
        {
            globus_handler_chain_register_invoke(
                service_desc->handlers,
                GLOBUS_HANDLER_TYPE_REQUEST,
                callback_handle->message,
                globus_l_service_session_result_callback,
                callback_handle);
            globus_mutex_unlock(&service_desc->mutex);
            service_desc = NULL;
            globus_extension_release(ext);
            break;
        }
        globus_mutex_unlock(&service_desc->mutex);
        service_desc = NULL;
        globus_extension_release(ext);

    case GLOBUS_L_SERVICE_ENGINE_REQUEST_DESTROY_HANDLERS:

	callback_handle->state = 
	    GLOBUS_L_SERVICE_ENGINE_REQUEST_DESTROY_SERVICE_HANDLERS;

	if(callback_handle->result != GLOBUS_SUCCESS)
	{
	    result = callback_handle->result;
	    goto error_send_fault;
	}
		
        globus_mutex_lock(&callback_handle->engine->mutex);
        if(callback_handle->engine->handlers)
        {
            globus_handler_chain_register_invoke(
                callback_handle->engine->handlers,
		GLOBUS_HANDLER_TYPE_REQUEST_DESTROY,
                callback_handle->message,
                globus_l_service_session_result_callback,
                callback_handle);
	    globus_mutex_unlock(&callback_handle->engine->mutex);
	    break;
        }
        globus_mutex_unlock(&callback_handle->engine->mutex);

    case GLOBUS_L_SERVICE_ENGINE_REQUEST_DESTROY_SERVICE_HANDLERS:

        callback_handle->state = 
        GLOBUS_L_SERVICE_ENGINE_RESPONSE_INIT_SERVICE_HANDLERS;

        if(callback_handle->result != GLOBUS_SUCCESS)
        {
            result = callback_handle->result;
            goto error_send_fault;
	}
	
	ext =
        (globus_extension_handle_t)
        globus_soap_message_handle_get_attr(
            callback_handle->message,
            GLOBUS_I_SERVICE_DESCRIPTOR_KEY);
        globus_assert_string(
            ext, "No SERVICE DESCRIPTOR attribute found!");
                
        service_desc = (globus_service_descriptor_t *) 
        globus_extension_reference(ext);
        
        globus_assert_string(
            service_desc, "No SERVICE DESCRIPTOR attribute found!");
        
        globus_mutex_lock(&service_desc->mutex);
        if(service_desc->handlers)
        {
	    globus_handler_chain_register_invoke(
		    service_desc->handlers,
		    GLOBUS_HANDLER_TYPE_REQUEST_DESTROY,
		    callback_handle->message,
		    globus_l_service_session_result_callback,
		    callback_handle);
        
	    globus_mutex_unlock(&service_desc->mutex);
	    service_desc = NULL;
	    globus_extension_release(ext);
	    break;
	}
	globus_mutex_unlock(&service_desc->mutex);
	service_desc = NULL;
	globus_extension_release(ext); 
	
    case GLOBUS_L_SERVICE_ENGINE_RESPONSE_INIT_SERVICE_HANDLERS:
	
	if(callback_handle->result != GLOBUS_SUCCESS)
	{
	    result = callback_handle->result;
	    goto callback_exit;
	}
	
	callback_handle->state =
	    GLOBUS_L_SERVICE_ENGINE_RESPONSE_INIT_HANDLERS;

        ext =
        (globus_extension_handle_t)
        globus_soap_message_handle_get_attr(
            callback_handle->message,
            GLOBUS_I_SERVICE_DESCRIPTOR_KEY);
        globus_assert_string(
            ext, "No SERVICE DESCRIPTOR attribute found!");

        service_desc = 
        (globus_service_descriptor_t *) 
        globus_extension_reference(ext);
        
        globus_assert_string(
            service_desc, "No SERVICE DESCRIPTOR attribute found!");

        globus_mutex_unlock(&service_desc->mutex);
        if(service_desc->handlers)
        {
            globus_handler_chain_register_invoke(
                service_desc->handlers,
                GLOBUS_HANDLER_TYPE_RESPONSE_INIT,
                callback_handle->message,
                globus_l_service_session_result_callback,
                callback_handle);
            globus_mutex_unlock(&service_desc->mutex);
            service_desc = NULL;
            globus_extension_release(ext);
            break;
        }
        globus_mutex_unlock(&service_desc->mutex);

        service_desc = NULL;
        globus_extension_release(ext);

    case GLOBUS_L_SERVICE_ENGINE_RESPONSE_INIT_HANDLERS:
	
	if(callback_handle->result != GLOBUS_SUCCESS)
	{
	    result = callback_handle->result;
	    goto callback_exit;
	}
	
	callback_handle->state =
	    GLOBUS_L_SERVICE_ENGINE_REQUEST_INVOKE_OPERATION;

        globus_mutex_lock(&callback_handle->engine->mutex);
        if(callback_handle->engine->handlers)
        {
            globus_handler_chain_register_invoke(
                callback_handle->engine->handlers,
                GLOBUS_HANDLER_TYPE_RESPONSE_INIT,
                callback_handle->message,
                globus_l_service_session_result_callback,
                callback_handle);
            globus_mutex_unlock(&callback_handle->engine->mutex);
            break;
        }
        globus_mutex_unlock(&callback_handle->engine->mutex);
	

    case GLOBUS_L_SERVICE_ENGINE_REQUEST_INVOKE_OPERATION:
        
        if(callback_handle->result != GLOBUS_SUCCESS)
        {
            result = callback_handle->result;
            goto error_send_fault;
        }

        callback_handle->state = 
        GLOBUS_L_SERVICE_ENGINE_RESPONSE_INVOKE_SERVICE_HANDLERS;

        operation_name = globus_soap_message_handle_get_attr(
            callback_handle->message,
            GLOBUS_I_SERVICE_OPERATION_QNAME_KEY);
        globus_assert(operation_name);

        ext = 
        (globus_extension_handle_t)
        globus_soap_message_handle_get_attr(
            callback_handle->message,
            GLOBUS_I_SERVICE_DESCRIPTOR_KEY);
        globus_assert_string(
            ext, "No SERVICE DESCRIPTOR attribute found!");

        service_desc = (globus_service_descriptor_t *) 
        globus_extension_reference(ext);

        globus_assert_string(
            service_desc, "No SERVICE DESCRIPTOR attribute found!");

        /* we use qn to map to operation deserialize function */
        globus_mutex_lock(&service_desc->mutex);
        if(service_desc->op_mapper)
        {
            op_desc = (globus_service_operation_descriptor_t *)
            globus_hashtable_lookup(
                &service_desc->op_mapper, 
                (void *)operation_name);
        }

        if(!op_desc)
        {
            result = GlobusServiceEngineErrorOperationLookupFailed(
                service_desc->name, operation_name);
            globus_mutex_unlock(&service_desc->mutex);
            service_desc = NULL;
            globus_extension_release(ext);
            goto error_send_fault;
        }

        result = op_desc->invoke(
            callback_handle->engine,
            callback_handle->message,
            &callback_handle->response,
            globus_l_service_session_result_callback,
            callback_handle);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusServiceEngineErrorOpInvokeFailed(
                result, 
                operation_name);
            globus_mutex_unlock(&service_desc->mutex);
            goto error_send_fault;
        }

        if(callback_handle->response == GLOBUS_SERVICE_NO_RESPONSE)
        {
            callback_handle->state = GLOBUS_L_SERVICE_ENGINE_RESPONSE_SEND;
        }

        globus_mutex_unlock(&service_desc->mutex);
        service_desc = NULL;
        globus_extension_release(ext);

        break;

    case GLOBUS_L_SERVICE_ENGINE_RESPONSE_INVOKE_SERVICE_HANDLERS:
     
        if(callback_handle->result != GLOBUS_SUCCESS)
        {
            result = callback_handle->result;
            goto callback_exit;
        }
        
        callback_handle->state = 
        GLOBUS_L_SERVICE_ENGINE_RESPONSE_INVOKE_HANDLERS;
        
        result = globus_soap_message_set_write_position_to_marker(
            callback_handle->message, 
            GLOBUS_SOAP_MESSAGE_MARKER_HEADER_CONTENT);
        if(result != GLOBUS_SUCCESS)
        {
            goto callback_exit;
        }

        ext =
        (globus_extension_handle_t)
        globus_soap_message_handle_get_attr(
            callback_handle->message,
            GLOBUS_I_SERVICE_DESCRIPTOR_KEY);
        globus_assert_string(
            ext, "No SERVICE DESCRIPTOR attribute found!");

        service_desc = 
        (globus_service_descriptor_t *) 
        globus_extension_reference(ext);
        
        globus_assert_string(
            service_desc, "No SERVICE DESCRIPTOR attribute found!");

        globus_mutex_unlock(&service_desc->mutex);
        if(service_desc->handlers)
        {
            globus_handler_chain_register_invoke(
                service_desc->handlers,
                GLOBUS_HANDLER_TYPE_RESPONSE,
                callback_handle->message,
                globus_l_service_session_result_callback,
                callback_handle);
            globus_mutex_unlock(&service_desc->mutex);
            service_desc = NULL;
            globus_extension_release(ext);
            break;
        }
        globus_mutex_unlock(&service_desc->mutex);

        service_desc = NULL;
        globus_extension_release(ext);

    case GLOBUS_L_SERVICE_ENGINE_RESPONSE_INVOKE_HANDLERS:
        
        callback_handle->state =
        GLOBUS_L_SERVICE_ENGINE_RESPONSE_DESTROY_SERVICE_HANDLERS;
            
        if(callback_handle->result != GLOBUS_SUCCESS)
        {
            result = callback_handle->result;
            goto callback_exit;
        }

        globus_mutex_lock(&callback_handle->engine->mutex);
        if(callback_handle->engine->handlers)
        {
            globus_handler_chain_register_invoke(
                callback_handle->engine->handlers,
                GLOBUS_HANDLER_TYPE_RESPONSE,
                callback_handle->message,
                globus_l_service_session_result_callback,
                callback_handle);
            globus_mutex_unlock(&callback_handle->engine->mutex);
            break;
        }
        globus_mutex_unlock(&callback_handle->engine->mutex);
        
    case GLOBUS_L_SERVICE_ENGINE_RESPONSE_DESTROY_SERVICE_HANDLERS:

	if(callback_handle->result != GLOBUS_SUCCESS)
	{
	    result = callback_handle->result;
	    goto callback_exit;
	}

	callback_handle->state =
	    GLOBUS_L_SERVICE_ENGINE_RESPONSE_DESTROY_HANDLERS;

        if(callback_handle->response == GLOBUS_SERVICE_NO_RESPONSE)
        {
            result = globus_soap_message_setup_writer(
                callback_handle->message);
            if(result != GLOBUS_SUCCESS)
            {
                goto callback_exit;
            }
        }

        ext =
        (globus_extension_handle_t)
        globus_soap_message_handle_get_attr(
            callback_handle->message,
            GLOBUS_I_SERVICE_DESCRIPTOR_KEY);

        /* If the user passes in an invalid service name, the extension
         * handle will be NULL. We'll just skip service-specific handler
         * invocation in this case.
         */
        if (ext != NULL)
        {
            service_desc = 
            (globus_service_descriptor_t *) 
            globus_extension_reference(ext);
            
            globus_assert_string(
                service_desc, "No SERVICE DESCRIPTOR attribute found!");

            globus_mutex_unlock(&service_desc->mutex);
            if(service_desc->handlers)
            {
                globus_handler_chain_register_invoke(
                    service_desc->handlers,
                    GLOBUS_HANDLER_TYPE_RESPONSE_DESTROY,
                    callback_handle->message,
                    globus_l_service_session_result_callback,
                    callback_handle);
                globus_mutex_unlock(&service_desc->mutex);
                service_desc = NULL;
                globus_extension_release(ext);
                break;
            }
            globus_mutex_unlock(&service_desc->mutex);

            service_desc = NULL;
            globus_extension_release(ext);
        }

    case GLOBUS_L_SERVICE_ENGINE_RESPONSE_DESTROY_HANDLERS:

	if(callback_handle->result != GLOBUS_SUCCESS)
	{
	    result = callback_handle->result;
	    goto callback_exit;
	}
	
	callback_handle->state =
	    GLOBUS_L_SERVICE_ENGINE_RESPONSE_SEND;

        globus_mutex_lock(&callback_handle->engine->mutex);
        if(callback_handle->engine->handlers)
        {
            globus_handler_chain_register_invoke(
                callback_handle->engine->handlers,
                GLOBUS_HANDLER_TYPE_RESPONSE_DESTROY,
                callback_handle->message,
                globus_l_service_session_result_callback,
                callback_handle);
            globus_mutex_unlock(&callback_handle->engine->mutex);
            break;
        }
        globus_mutex_unlock(&callback_handle->engine->mutex);
	
    case GLOBUS_L_SERVICE_ENGINE_RESPONSE_SEND:

        callback_handle->state = 
        GLOBUS_L_SERVICE_ENGINE_RESPONSE_CLOSE;

        /* If there's no response, we setup the writer to clear the
         * buffer state so that we don't send the request back as
         * the unwanted response
         */
        if(callback_handle->response == GLOBUS_SERVICE_NO_RESPONSE)
        {
            result = globus_soap_message_setup_writer(
                callback_handle->message);
            if(result != GLOBUS_SUCCESS)
            {
                goto callback_exit;
            }
        }
        result = globus_soap_message_register_write_response(
            callback_handle->message,
            globus_l_service_session_result_callback,
            callback_handle);
        if(result != GLOBUS_SUCCESS)
        {
            goto callback_exit;
        }

        break;

    case GLOBUS_L_SERVICE_ENGINE_RESPONSE_CLOSE:
        connection = globus_soap_message_handle_get_attr(
            callback_handle->message,
            GLOBUS_SOAP_MESSAGE_CONNECTION_KEY);

        if (callback_handle->result == GLOBUS_SUCCESS
            && ((!connection) || (strcmp(connection, "close"))))
        {
            callback_handle->state =
                GLOBUS_L_SERVICE_ENGINE_REQUEST_READ_ANOTHER_MESSAGE;

            result = globus_soap_message_handle_reset(
                callback_handle->message);

            if (result != GLOBUS_SUCCESS)
            {
                goto close_it;
            }
            /* Register a new metadata read */
            result = globus_soap_message_get_transport_handle(
                callback_handle->message,
                &xio_handle);

            if (result != GLOBUS_SUCCESS)
            {
                goto close_it;
            }
            result = globus_xio_handle_cntl(
                    xio_handle,
                    globus_i_soap_message_buffer_driver,
                    GLOBUS_XIO_BUFFER_CLEAR);
            if (result != GLOBUS_SUCCESS)
            {
                goto close_it;
            }
            result = globus_xio_handle_cntl(
                    xio_handle,
                    globus_i_soap_message_buffer_driver,
                    GLOBUS_XIO_BUFFER_DISABLE_BUFFERING);
            if (result != GLOBUS_SUCCESS)
            {
                goto close_it;
            }

            result = globus_xio_register_read(
                    xio_handle,
                    buffer,
                    0,
                    0,
                    NULL,
                    globus_l_service_engine_request_ready_callback,
                    callback_handle);
            if (result == GLOBUS_SUCCESS)
            {
                break;
            }
        }
close_it:
        callback_handle->state = 
            GLOBUS_L_SERVICE_ENGINE_RESPONSE_DONE;
        
        if(callback_handle->result != GLOBUS_SUCCESS)
        {
            result = callback_handle->result;
            goto exit;
        }

        result = globus_soap_message_register_close(
            callback_handle->message,
            globus_l_service_session_result_callback,
            callback_handle);

        if(result != GLOBUS_SUCCESS)
        {
            goto callback_exit;
        }
        break;

    case GLOBUS_L_SERVICE_ENGINE_RESPONSE_DONE:

        if(callback_handle->result != GLOBUS_SUCCESS)
        {
            result = callback_handle->result;
            goto exit;
        }

        goto callback_exit;
    }

    goto exit;

 error_send_fault:

    if(service_desc)
    {
        service_desc = NULL;
        globus_extension_release(ext);
    }

    if(result != GLOBUS_SUCCESS)
    {
        char *                          error_str;

        error_str = globus_error_print_chain(
            globus_error_peek(result));
        GlobusServiceEngineDebugPrintf(
            GLOBUS_L_SERVICE_ENGINE_DEBUG_ERROR,
            ("SOAP message processing failed:\n%s\n",
             error_str));
        globus_free(error_str);

        callback_handle->result = result;
        /* need to send client FAULT as response */
        result = globus_callback_register_oneshot(
            NULL,
            &globus_i_reltime_zero,
            globus_l_service_engine_send_client_fault,
            callback_handle);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusServiceEngineErrorFailedRequestProcess(
                result, 
                "Failed to register callback "
                "to send client fault response");
        }
    }

    goto exit;

 callback_exit:

    if(result != GLOBUS_SUCCESS)
    {
        ext =
        (globus_extension_handle_t)
        globus_soap_message_handle_get_attr(
            callback_handle->message,
            GLOBUS_I_SERVICE_DESCRIPTOR_KEY);
        if(ext)
        {
            service_desc = 
            (globus_service_descriptor_t *) 
            globus_extension_reference(ext);

            globus_assert_string(service_desc,
                                 "No SERVICE DESCRIPTOR attribute found!");

            globus_mutex_lock(&service_desc->mutex);
            result = GlobusServiceEngineErrorSessionFailed(
                result, service_desc->name);
            globus_mutex_unlock(&service_desc->mutex);

            service_desc = NULL;
            globus_extension_release(ext);
        }
    }

    globus_mutex_lock(&callback_handle->engine->mutex);
    callback_handle->process_callback(
        result ? result : callback_handle->callback_result,
        callback_handle->engine,
        callback_handle->message,
        callback_handle->process_args);
    globus_mutex_unlock(&callback_handle->engine->mutex);
    
    globus_l_service_session_callback_handle_destroy(callback_handle);

 exit:
    GlobusServiceEngineDebugExit();
}

globus_result_t
globus_service_session_begin_response(
    globus_soap_message_handle_t        message)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_service_session_begin_response);
    GlobusServiceEngineDebugEnter();

    result = globus_soap_message_setup_writer(message);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusServiceEngineErrorTransportFailed(
            result, "Failed to setup message writer");
        goto exit;
    }

    result = globus_soap_message_serialize_envelope(message);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusServiceEngineErrorFailedResponse(result);
        goto exit;
    }

    result = globus_soap_message_serialize_header(message);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusServiceEngineErrorFailedResponse(result);
        goto exit;
    }

    result = globus_soap_message_serialize_header_begin_close(message);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusServiceEngineErrorFailedResponse(result);
        goto exit;
    }

    result = globus_soap_message_set_marker(
        message,
        GLOBUS_SOAP_MESSAGE_MARKER_HEADER_CONTENT);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusServiceEngineErrorFailedResponse(result);
        goto exit;
    }

    result = globus_soap_message_serialize_header_end(message);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusServiceEngineErrorFailedResponse(result);
        goto exit;
    }

    result = globus_soap_message_serialize_body(message);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusServiceEngineErrorFailedResponse(result);
        goto exit;
    }

    result = globus_soap_message_serialize_body_begin_close(
	message);
    if(result != GLOBUS_SUCCESS)
    {
	result = GlobusServiceEngineErrorFailedResponse(result);
	goto exit;
    }
    
 exit:

    GlobusServiceEngineDebugExit();
    return result;
}

globus_result_t
globus_service_session_end_response(
    globus_soap_message_handle_t        message)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_service_session_end_response);
    GlobusServiceEngineDebugEnter();
    
    result = globus_soap_message_serialize_body_end(message);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusServiceEngineErrorFailedResponse(result);
        goto exit;
    }

    result = globus_soap_message_serialize_envelope_end(message);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusServiceEngineErrorFailedResponse(result);
        goto exit;
    }
    
 exit:

    GlobusServiceEngineDebugExit();
    return result;
}

globus_result_t
globus_service_session_serialize_fault_response(
    globus_soap_message_handle_t        message,
    globus_soap_message_fault_t         fault)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_service_serialize_fault_response);
    GlobusServiceEngineDebugEnter();

    result = globus_soap_message_serialize_fault(message, fault);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusServiceEngineErrorSendingFault(result);
        goto exit;
    }

    result = globus_service_session_end_response(message);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusServiceEngineErrorSendingFault(result);
        goto exit;
    }

 exit:

    GlobusServiceEngineDebugExit();
    return result;
}

static
void
globus_l_service_engine_send_client_fault(
    void *                              args)
{
    globus_result_t                      result = GLOBUS_SUCCESS;
    globus_xio_handle_t                  xio_handle;
    globus_l_session_callback_handle_t * callback_handle;
    char *                               error_string;
    struct globus_soap_message_fault_s   fault;
    int                                  status_code = 200;
    GlobusFuncName(globus_l_service_engine_send_client_fault);
    GlobusServiceEngineDebugEnter();

    callback_handle = (globus_l_session_callback_handle_t *) args;
    callback_handle->faulting = GLOBUS_TRUE;

    fault.faultcode = "Client";

    error_string = globus_error_print_friendly(
        globus_error_peek(
            callback_handle->result));
    callback_handle->callback_result = callback_handle->result;
    callback_handle->result = GLOBUS_SUCCESS;
    fault.faultstring = error_string;
    fault.faultactor = NULL;
    fault.detail = NULL;
 
    result = globus_service_session_begin_response(
                callback_handle->message);
    if(result != GLOBUS_SUCCESS)
    {
        goto error_exit;
    }

    result = globus_service_session_serialize_fault_response(
        callback_handle->message, &fault);
    if(result != GLOBUS_SUCCESS)
    {
        goto error_exit;
    }

    if(GlobusServiceEngineErrorCheckMethodNotAllowed(
           callback_handle->result))
    {
        status_code = 405;
    }
    else if(GlobusServiceEngineErrorCheckUnsupportedMediaType(
                callback_handle->result))
    {
        status_code = 415;
    }
    else
    {
        status_code = 400;
    }

    result = globus_soap_message_get_transport_handle(
        callback_handle->message,
        &xio_handle);
    if(result != GLOBUS_SUCCESS)
    {
        goto error_exit;
    }

    result = globus_xio_handle_cntl(
        xio_handle,
        globus_i_soap_message_http_driver,
        GLOBUS_XIO_HTTP_HANDLE_SET_RESPONSE_STATUS_CODE,
        status_code);
    if(result != GLOBUS_SUCCESS)
    {
        goto error_exit;
    }

    result = globus_soap_message_set_write_position_to_marker(
        callback_handle->message, 
        GLOBUS_SOAP_MESSAGE_MARKER_HEADER_CONTENT);
    if(result != GLOBUS_SUCCESS)
    {
        goto error_exit;
    }

    result = globus_soap_message_handle_set_attr(
        callback_handle->message, 
        WSADDR_REQUEST_ENDPOINT_KEY,
        globus_soap_message_attr_copy_string,
        globus_libc_free,
        WSADDRESSING_ANON_NS);
    if(result != GLOBUS_SUCCESS)
    {
        goto error_exit;
    }

    result = globus_soap_message_handle_set_attr(
        callback_handle->message,
        WSADDR_ACTION_RESPONSE_KEY,
        globus_soap_message_attr_copy_string,
        globus_libc_free,
        WSADDRESSING_FAULT_NS);
    if(result != GLOBUS_SUCCESS)
    {
        goto error_exit;
    }

    callback_handle->state = GLOBUS_L_SERVICE_ENGINE_RESPONSE_INVOKE_HANDLERS;

    result = globus_callback_register_oneshot(
        NULL,
        &globus_i_reltime_zero,
        globus_l_service_session_callback,
        callback_handle);
    if(result != GLOBUS_SUCCESS)
    {
        goto error_exit;
    }
    
    goto exit;

 error_exit:

    globus_mutex_lock(&callback_handle->engine->mutex);
    callback_handle->callback(result,
                              callback_handle->engine,
                              callback_handle->message,
                              callback_handle->args);
    globus_mutex_unlock(&callback_handle->engine->mutex);
    globus_l_service_session_callback_handle_destroy(callback_handle);
    
 exit:

    GlobusServiceEngineDebugExit();
}

globus_result_t
globus_service_engine_register_process(
    globus_service_engine_t                engine,
    globus_soap_message_handle_t           message,
    globus_result_t                        fault_result,
    globus_service_session_callback_func_t callback,
    void *                                 user_args)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    globus_l_session_callback_handle_t * callback_handle;

    if(!message)
    {
        callback(fault_result, engine, NULL, user_args);
        goto exit;
    }

    callback_handle = globus_soap_message_handle_get_attr(
        message,
        GLOBUS_I_SESSION_HANDLE_KEY);
    globus_assert_string(
        callback_handle, "Session handle lookup failed for callback handle");

    callback_handle->process_callback = callback;
    callback_handle->process_args = user_args;

    if(fault_result != GLOBUS_SUCCESS)
    {
        char *                          error_str;

        error_str = globus_error_print_chain(
            globus_error_peek(fault_result));
        GlobusServiceEngineDebugPrintf(
            GLOBUS_L_SERVICE_ENGINE_DEBUG_ERROR,
            ("Error occurred during session accept:\n%s\n",
             error_str));
        globus_free(error_str);

        callback_handle->result = fault_result;

        /* need to send client FAULT as response */
        result = globus_callback_register_oneshot(
            NULL,
            &globus_i_reltime_zero,
            globus_l_service_engine_send_client_fault,
            callback_handle);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusServiceEngineErrorFailedRequestProcess(
                result, 
                "Failed to register callback "
                "to send client fault response");
            goto exit;
        }
        
        goto exit;
    }
    else
    {
        callback_handle->state = 
        GLOBUS_L_SERVICE_ENGINE_REQUEST_READ_MESSAGE;
        
        result = globus_callback_register_oneshot(
            NULL,
            &globus_i_reltime_zero,
            globus_l_service_session_callback,
            callback_handle);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusServiceEngineErrorFailedRequestProcess(
                result, "Failed to register callback to invoke operation");
            goto exit;
        }
    }
    
 exit:
    
    GlobusServiceEngineDebugExit();
    return result;
}

globus_result_t
globus_service_engine_set_handlers(
    globus_service_engine_t             engine,
    globus_handler_chain_t              chain)
{
    GlobusFuncName(globus_service_engine_set_handlers);
    GlobusServiceEngineDebugEnter();

    if(engine->handlers)
    {
        globus_handler_chain_destroy(engine->handlers);
    }

    engine->handlers = chain;

    GlobusServiceEngineDebugExit();
    return GLOBUS_SUCCESS;
}

static
void 
globus_l_service_engine_session_complete_callback(
    globus_result_t                     result,
    globus_service_engine_t             engine,
    globus_soap_message_handle_t        session,
    void *                              args)
{
    globus_l_service_engine_callback_handle_t * callback_handle;
    GlobusFuncName(globus_l_service_engine_session_complete_callback);
    GlobusServiceEngineDebugEnter();

    callback_handle = (globus_l_service_engine_callback_handle_t *) args;

    if(result != GLOBUS_SUCCESS)
    {
        char *                          endpoint = NULL; 
        char *                          action = NULL;

        if(session)
        {
            endpoint = globus_soap_message_handle_get_attr(
                session,
                GLOBUS_SOAP_MESSAGE_SERVICE_ENDPOINT_KEY);
            
            action = globus_soap_message_handle_get_attr(
                session,
                GLOBUS_SOAP_MESSAGE_SOAP_ACTION_KEY);
        }

        GlobusServiceEngineDebugPrintError(result, endpoint, action);
    }

    globus_mutex_lock(&engine->mutex);
    engine->session_count--;
    if(engine->stop_sessions && engine->session_count <= 0)
    {
        globus_mutex_unlock(&engine->mutex);

        callback_handle->callback(
            GLOBUS_SUCCESS, engine, callback_handle->args);

        globus_free(callback_handle);
    }
    else
    {
        globus_mutex_unlock(&engine->mutex);
    }

    globus_soap_message_handle_destroy(session);

    GlobusServiceEngineDebugExit();
}

static
void
globus_l_service_engine_session_started_callback(
    globus_result_t                     result,
    globus_service_engine_t             engine,
    globus_soap_message_handle_t        session,
    void *                              args)
{
    globus_result_t                     fault_result = GLOBUS_SUCCESS;
    globus_l_service_engine_callback_handle_t * callback_handle;
    GlobusFuncName(globus_l_service_engine_session_started_callback);
    GlobusServiceEngineDebugEnter();

    callback_handle = (globus_l_service_engine_callback_handle_t *) args;

    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusServiceEngineErrorSessionStartFailed(result);
        goto error_exit;
    }

    if(callback_handle->result != GLOBUS_SUCCESS)
    {
        fault_result = callback_handle->result;
    }
        
    globus_mutex_lock(&engine->mutex);
    engine->session_count++;
    if(!engine->stop_sessions)
    {
        result = globus_service_engine_register_session(
            engine,
            globus_l_service_engine_session_started_callback,
            callback_handle);
        if(result != GLOBUS_SUCCESS)
        {
            globus_mutex_unlock(&engine->mutex);
            /* no more sessions started */
            goto error_exit;
        }
    }
    
    globus_mutex_unlock(&engine->mutex);

    result = globus_service_engine_register_process(
        engine,
        session,
        fault_result,
        globus_l_service_engine_session_complete_callback,
        args);

    goto exit;

 error_exit:

    if(result != GLOBUS_SUCCESS)
    {
        char *                          endpoint = NULL; 
        char *                          action = NULL;

        if(session)
        {
            endpoint = globus_soap_message_handle_get_attr(
                session,
                GLOBUS_SOAP_MESSAGE_SERVICE_ENDPOINT_KEY);
            
            action = globus_soap_message_handle_get_attr(
                session,
                GLOBUS_SOAP_MESSAGE_SOAP_ACTION_KEY);
        }

        GlobusServiceEngineDebugPrintError(result, endpoint, action);
    }

    if (globus_xio_error_is_canceled(result))
    {
        result = GLOBUS_SUCCESS;
    }
    callback_handle->callback(result,
                              engine, 
                              callback_handle->args);
    
    if(engine->session_count <= 0)
    {
        free(callback_handle);
    }
    
 exit:

    GlobusServiceEngineDebugExit();
}


globus_result_t
globus_service_engine_register_start(
    globus_service_engine_t               engine,
    globus_service_engine_callback_func_t stop_callback,
    void *                                args)
{
    globus_l_service_engine_callback_handle_t * callback_handle;
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_service_engine_register_start);
    GlobusServiceEngineDebugEnter();

    if(engine->stop_sessions)
    {
        result = GlobusServiceEngineErrorAlreadyStarted();
        goto exit;
    }

    callback_handle = globus_malloc(
        sizeof(globus_l_service_engine_callback_handle_t));
    if(!callback_handle)
    {
        result = GlobusServiceEngineErrorOutOfMemory;
        goto exit;
    }
    memset(callback_handle, 0, sizeof(globus_l_service_engine_callback_handle_t));

    callback_handle->engine = engine;
    callback_handle->callback = stop_callback;
    callback_handle->args = args;

    result = globus_service_engine_register_session(
        engine,
        globus_l_service_engine_session_started_callback,
        callback_handle);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusServiceEngineErrorFailedStart(result);
    }

 exit:

    GlobusServiceEngineDebugExit();
    return result;
}

globus_result_t
globus_service_engine_register_stop(
    globus_service_engine_t               engine)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_service_engine_register_stop);
    GlobusServiceEngineDebugEnter();

    globus_mutex_lock(&engine->mutex);
    engine->stop_sessions = 1;
    globus_xio_server_cancel_accept(engine->server);
    globus_mutex_unlock(&engine->mutex);

    GlobusServiceEngineDebugExit();
    return result;
}
