/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */


#include "xsd_anyType.h"
#include "globus_soap_message.h"
#include "globus_soap_message_utils.h"
#include "globus_i_soap_message.h"
#include "globus_i_soap_message_wildcards.h"
#include "globus_xml_buffer.h"
#include "globus_i_xsd.h"

GLOBUS_I_XSD_DEFINE_QNAME(xsd, anyType);
GLOBUS_I_XSD_DEFINE_TYPE_INIT_ARRAY(xsd_anyType);
GLOBUS_I_XSD_DEFINE_TYPE_DESTROY_ARRAY(xsd_anyType);
GLOBUS_I_XSD_DEFINE_TYPE_COPY_ARRAY(xsd_anyType);
GLOBUS_I_XSD_DEFINE_TYPE_SERIALIZE_ARRAY(xsd_anyType);
GLOBUS_I_XSD_DEFINE_TYPE_DESERIALIZE_ARRAY(xsd_anyType);
GLOBUS_I_XSD_DEFINE_TYPE_ARRAY_PUSH(xsd_anyType);
GLOBUS_I_XSD_DEFINE_TYPE_INIT(xsd_anyType);
GLOBUS_I_XSD_DEFINE_TYPE_DESTROY(xsd_anyType);
GLOBUS_I_XSD_DEFINE_TYPE_COPY(xsd_anyType);
GLOBUS_I_XSD_DEFINE_INIT_CONTENTS_GENERIC(xsd_anyType);

globus_result_t
xsd_anyType_serialize_wrapper(
    xsd_QName *                         element_qname,
    void *                              instance,
    globus_soap_message_handle_t        message_handle,
    globus_xsd_element_options_t        options)
{
    return xsd_anyType_serialize(
        element_qname,
        (xsd_anyType *) instance,
        message_handle, options);
}

globus_result_t
xsd_anyType_deserialize_pointer_wrapper(
    xsd_QName *                         element_qname,
    void **                             ip,
    globus_soap_message_handle_t        message_handle,
    globus_xsd_element_options_t        options)
{
    return xsd_anyType_deserialize_pointer(
        element_qname,
        (xsd_anyType **) ip,
        message_handle, options);
}

struct globus_xsd_type_info_s xsd_anyType_info =
{
    &xsd_anyType_qname,
    xsd_anyType_serialize_wrapper,
    xsd_anyType_deserialize_pointer_wrapper,
    xsd_anyType_init_wrapper,
    xsd_anyType_destroy_wrapper,
    xsd_anyType_copy_wrapper,
    xsd_anyType_init_contents_wrapper,
    xsd_anyType_destroy_contents_wrapper,
    xsd_anyType_copy_contents_wrapper,
    sizeof(xsd_anyType),
    NULL,
    NULL,
    NULL
};

struct globus_xsd_type_info_s xsd_anyType_array_info =
{
    &xsd_anyType_qname,
    xsd_anyType_array_serialize_wrapper,
    xsd_anyType_array_deserialize_wrapper,
    xsd_anyType_array_init_wrapper,
    xsd_anyType_array_destroy_wrapper,
    xsd_anyType_array_copy_wrapper,
    xsd_anyType_array_init_contents_wrapper,
    xsd_anyType_array_destroy_contents_wrapper,
    xsd_anyType_array_copy_contents_wrapper,
    sizeof(xsd_anyType),
    xsd_anyType_array_push_wrapper,
    NULL,
    NULL
};

struct globus_xsd_type_info_s xsd_anyType_contents_info =
{
    &xsd_anyType_qname,
    xsd_anyType_serialize_wrapper,
    xsd_anyType_deserialize_pointer_wrapper,
    xsd_anyType_init_wrapper,
    xsd_anyType_destroy_wrapper,
    xsd_anyType_copy_wrapper,
    xsd_anyType_init_contents_wrapper,
    xsd_anyType_destroy_contents_wrapper,
    xsd_anyType_copy_contents_wrapper,
    sizeof(xsd_anyType),
    NULL,
    NULL,
    NULL
};

void
xsd_anyType_destroy_contents(
    xsd_anyType *                       instance)
{
    GlobusFuncName(xsd_anyType_destroy_contents);
    GlobusSoapMessageDebugEnter();

    if(instance && instance->value && instance->any_info)
    {
        instance->any_info->destroy(instance->value);
        instance->value = NULL;
    }

    GlobusSoapMessageDebugExit();
}

globus_result_t
xsd_anyType_copy_contents(
    xsd_anyType *                       dest,
    const xsd_anyType *                 src)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(xsd_anyType_copy_contents);
    GlobusSoapMessageDebugEnter();

    if(dest && src)
    {
        *dest = *src;
        if(src->value && src->any_info)
        {
            result = src->any_info->copy(&dest->value, src->value);
        }
    }

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
xsd_anyType_serialize(
    xsd_QName *                         element_qname,
    xsd_anyType *                       anyinst,
    globus_soap_message_handle_t        message_handle,
    globus_xsd_element_options_t        options)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(xsd_anyType_serialize);
    GlobusSoapMessageDebugEnter();

    /* serialize contents of type */
    result = anyinst->any_info->serialize(
        element_qname,
        anyinst->value,
        message_handle,
        options | GLOBUS_XSD_ELEMENT_TYPE_ATTR);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorSerializeFailed(
            result,
            "Serialization of xsd:any failed",
            element_qname);
        goto exit;
    }

 exit:

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
xsd_anyType_deserialize(
    xsd_QName *                         element_qname,
    xsd_anyType *                       instance,
    globus_soap_message_handle_t        message_handle,
    globus_xsd_element_options_t        options)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(xsd_anyType_deserialize);
    GlobusSoapMessageDebugEnter();

    result = globus_soap_message_deserialize_element(
        message_handle, element_qname);
    if(result != GLOBUS_SUCCESS)
    {
        if(GlobusSoapMessageStatusFailedWithTextCheck(result))
        {
            result = GLOBUS_SUCCESS;
        }
        else if(!GlobusSoapMessageStatusCheck(result))
        {
            result = GlobusSoapMessageErrorDeserializeFailed(
                result, element_qname);
            goto exit;
        }
        else
        {
            goto exit;
        }
    }

    if(!(options & GLOBUS_XSD_ELEMENT_CONTENTS_ONLY))
    {
        globus_soap_message_deserialize_push_element(message_handle);
    }

    GlobusSoapMessageDebugPrintf(
        GLOBUS_SOAP_MESSAGE_DEBUG_WILDCARDS, 
        ("\n\nDeserializing xsd:anyType\n"));

    result = globus_i_xsd_deserialize_from_xsi_type(
        &instance->any_info,
        instance->registry,
        message_handle);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            result, element_qname);
        goto exit;
    }

    if(!instance->any_info)
    {
        instance->any_info = (&globus_xml_buffer_contents_info);
    }

    GlobusSoapMessageDebugPrintf(
        GLOBUS_SOAP_MESSAGE_DEBUG_WILDCARDS, 
        ("Using any info: {%s}%s\n",
         instance->any_info->type->Namespace,
         instance->any_info->type->local));

    result = instance->any_info->deserialize(
        element_qname,
        &instance->value,
        message_handle,
        options);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            result, instance->any_info->type);
        goto exit;
    }

/*     if(options & GLOBUS_XSD_ELEMENT_CONTENTS_ONLY) */
/*     { */
/*         globus_soap_message_deserialize_push_element(message_handle); */
/*     } */

 exit:

    if(result != GLOBUS_SUCCESS)
    {
        xsd_anyType_destroy_contents(instance);
    }

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
xsd_anyType_deserialize_pointer(
    xsd_QName *                         element_qname,
    xsd_anyType **                      ip,
    globus_soap_message_handle_t        message_handle,
    globus_xsd_element_options_t        options)
{
    xsd_anyType *                       instance = NULL;
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(xsd_any_deserialize_pointer);
    GlobusSoapMessageDebugEnter();

    result = globus_soap_message_deserialize_element(
        message_handle, element_qname);
    if(result != GLOBUS_SUCCESS)
    {
        if(GlobusSoapMessageStatusElementNotFoundCheck(result) ||
           GlobusSoapMessageStatusFailedElementCheck(result))
        {
            *ip = NULL;
            result = GLOBUS_SUCCESS;
            goto exit;
        }

        if(!GlobusSoapMessageStatusCheck(result))
        {
            result = GlobusSoapMessageErrorDeserializeFailed(
                result, element_qname);
            goto exit;
        }
        goto exit;
    }


    GlobusSoapMessageDebugPrintf(
        GLOBUS_SOAP_MESSAGE_DEBUG_WILDCARDS, 
        ("\n\nDeserializing xsd:anyType\n"));

    result = xsd_anyType_init(&instance);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            result, &xsd_anyType_qname);
        goto exit;
    }

    result = globus_i_xsd_deserialize_from_xsi_type(
        &instance->any_info,
        instance->registry,
        message_handle);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            result, element_qname);
        goto exit;
    }

    if(!instance->any_info)
    {
        result = GlobusSoapMessageErrorNoXSITypeAttr();
        goto exit;
    }

    globus_soap_message_deserialize_push_element(message_handle);

    GlobusSoapMessageDebugPrintf(
        GLOBUS_SOAP_MESSAGE_DEBUG_WILDCARDS, 
        ("Using any info: {%s}%s\n",
         instance->any_info->type->Namespace,
         instance->any_info->type->local));

    result = instance->any_info->deserialize(
        element_qname,
        &instance->value,
        message_handle,
        options);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusSoapMessageErrorDeserializeFailed(
            result, instance->any_info->type);
        goto exit;
    }

    *ip = instance;

 exit:

    if(result != GLOBUS_SUCCESS && instance)
    {
        xsd_anyType_destroy(instance);
    }

    GlobusSoapMessageDebugExit();
    return result;
}
