/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */

#include "globus_i_xsd.h"
#include "globus_i_xsd_type_info.h"
#include "globus_soap_message.h"

GLOBUS_I_XSD_DEFINE_QNAME(xsd, hexBinary);
GLOBUS_I_XSD_DEFINE_TYPE_INFO(xsd_hexBinary);
GLOBUS_I_XSD_DEFINE_TYPE_FUNCTIONS(xsd_hexBinary);

GLOBUS_I_XSD_DEFINE_INIT_CONTENTS_GENERIC(xsd_hexBinary);

void
xsd_hexBinary_destroy_contents(
    xsd_hexBinary *                     instance)
{
    GlobusFuncName(xsd_hexBinary_destroy_contents);
    GlobusSoapMessageDebugEnter();

    if(instance)
    {
        if(instance->value)
        {
            free(instance->value);
            instance->value = NULL;
        }
    }

    GlobusSoapMessageDebugExit();
}

globus_result_t
xsd_hexBinary_copy_contents(
    xsd_hexBinary *                     dest,
    const xsd_hexBinary *               src)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(xsd_hexBinary_copy_contents);
    GlobusSoapMessageDebugEnter();

    if(dest && src)
    {
        dest->length = src->length;
        dest->value = globus_libc_strdup(src->value);
        if(!dest->value)
        {
            result = GlobusSoapMessageErrorOutOfMemory;
        }
    }

    GlobusSoapMessageDebugExit();

    return result;
}

globus_result_t
xsd_hexBinary_serialize_contents(
    xsd_QName *                         element,
    xsd_hexBinary *                     instance,
    globus_soap_message_handle_t        message_handle,
    globus_xsd_element_options_t        options)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(xsd_hexBinary_serialize_contents);
    GlobusSoapMessageDebugEnter();

    /* serialize contents of type */
    result = globus_soap_message_serialize_hexBinary(
        message_handle, instance);

    GlobusSoapMessageDebugExit();
    return result;
}

globus_result_t
xsd_hexBinary_deserialize_contents(
    xsd_QName *                         eqn,
    xsd_hexBinary *                     instance,
    globus_soap_message_handle_t        message_handle,
    globus_xsd_element_options_t        options)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(xsd_hexBinary_deserialize_contents);
    GlobusSoapMessageDebugEnter();
    
    result = globus_soap_message_deserialize_hexBinary(
        message_handle, instance);

    GlobusSoapMessageDebugExit();
    return result;
}
