#include "globus_service_engine.h"

/* Test case created in response to bug #4289 */

static volatile int                     done;
static globus_mutex_t                   mutex;
static globus_cond_t                    cond;

static
void
stop_callback(
    globus_result_t                     result,
    globus_service_engine_t             engine,
    void *                              args)
{
    globus_mutex_lock(&mutex);
    if (result == GLOBUS_SUCCESS)
    {
        done = 1;
    }
    else
    {
        done = 2;
    }
    globus_cond_signal(&cond);
    globus_mutex_unlock(&mutex);
}

int
main()
{
    int                                 rc;
    globus_result_t                     result;
    globus_service_engine_t             engine = NULL;

    printf("1..1\n");

    rc = globus_module_activate(GLOBUS_COMMON_MODULE);
    if (rc != GLOBUS_SUCCESS)
    {
        goto deactivate_out;
    }
    rc = globus_module_activate(GLOBUS_SERVICE_ENGINE_MODULE);
    if (rc != GLOBUS_SUCCESS)
    {
        goto deactivate_out;
    }

    globus_mutex_init(&mutex, NULL);
    globus_cond_init(&cond, NULL);
    done = GLOBUS_FALSE;

    result = globus_service_engine_init(
            &engine,
            NULL,
            NULL,
            NULL,
            GLOBUS_FALSE);

    if (result != GLOBUS_SUCCESS)
    {
        rc = 2;
        goto destroy_cond_out;
    }
    result = globus_service_engine_register_start(
            engine,
            stop_callback,
            NULL);
    if (result != GLOBUS_SUCCESS)
    {
        rc = 3;
        goto destroy_engine_out;
    }

    result = globus_service_engine_register_stop(engine);
    if (result != GLOBUS_SUCCESS)
    {
        rc = 4;

        goto destroy_engine_out;
    }

    globus_mutex_lock(&mutex);
    while (!done)
    {
        globus_cond_wait(&cond, &mutex);
    }
    globus_mutex_unlock(&mutex);

    rc = done - 1;

destroy_engine_out:
    globus_service_engine_destroy(engine);
destroy_cond_out:
    globus_cond_destroy(&cond);
    globus_mutex_destroy(&mutex);

deactivate_out:
    printf("%sok\n", rc == 0 ? "" : "not ");

    globus_module_deactivate_all();
    return rc;
}
