/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */

#include "globus_soap_message.h"
#include "globus_xsd_type_info.h"
#include "xsd_string.h"

typedef int (*test_case)(void);

#define SIZEOF_ARRAY(x) sizeof(x) / sizeof(x[0])

static
int
test_init()
{
    int rc;
    rc = globus_module_activate(GLOBUS_COMMON_MODULE);
    if (rc != 0)
    {
        return rc;
    }
    rc = globus_module_activate(GLOBUS_SOAP_MESSAGE_MODULE);
    return rc;
}

static
void
test_finalize()
{
    globus_module_deactivate(GLOBUS_COMMON_MODULE);
    globus_module_deactivate(GLOBUS_SOAP_MESSAGE_MODULE);
}

static
xsd_QName test_qname = {
    "http://www.globus.org/type_info",
    "test"
};

static
globus_result_t
test_init_func(
    void **                             obj)
{
    return xsd_string_init((xsd_string **) obj);
}

static
void
test_destroy_func(
    void *                              obj)
{
    xsd_string_destroy(obj);
}

static
globus_result_t
test_copy_func(
    void **                             dest_obj,
    void *                              src_obj)
{
    return xsd_string_copy((xsd_string **) dest_obj, (xsd_string *) src_obj);
}

static
globus_result_t
test_serialize_func(
    xsd_QName *                         element_name,
    void *                              instance,
    globus_soap_message_handle_t        soap_message_handle,
    globus_xsd_element_options_t        options)
{
    return xsd_string_serialize(
            element_name, instance, soap_message_handle, options);
}
/* test_serialize_func() */

static
globus_result_t
test_deserialize_func(
    xsd_QName *                         element_name,
    void **                             instance,
    globus_soap_message_handle_t        soap_message_handle,
    globus_xsd_element_options_t        options)
{
    return xsd_string_serialize(
            element_name, (xsd_string *) instance, soap_message_handle, options);
}

static
int 
init_destroy_test()
{
    globus_result_t                     result;
    globus_xsd_type_info_t              info;
    int                                 rc = 0;

    rc = test_init();
    if (rc != 0)
    {
        return 1;
    }

    result = globus_xsd_type_info_init_from_template(
            &info,
            &test_qname,
            &xsd_string_info);

    if (result != GLOBUS_SUCCESS)
    {
        rc = 1;

        goto out;
    }
    if (!xsd_QName_keyeq(info->type, &test_qname))
    {
        rc = 1;

        goto destroy_out;
    }
    if (info->initialize != xsd_string_info.initialize)
    {
        rc = 1;

        goto destroy_out;
    }

    if (info->destroy != xsd_string_info.destroy)
    {
        rc = 1;
        
        goto destroy_out;
    }
    if (info->copy != xsd_string_info.copy)
    {
        rc = 1;
        
        goto destroy_out;
    }
    if (info->serialize != xsd_string_info.serialize)
    {
        rc = 1;
        
        goto destroy_out;
    }
    if (info->deserialize != xsd_string_info.deserialize)
    {
        rc = 1;
        
        goto destroy_out;
    }
destroy_out:
    globus_xsd_type_info_destroy(info);
out:
    test_finalize();
    return rc;
}

static
int 
null_init_test()
{
    globus_result_t                     result;
    globus_xsd_type_info_t              info;
    int                                 rc = 0;

    rc = test_init();
    if (rc != 0)
    {
        return 1;
    }
    result = globus_xsd_type_info_init_from_template(
            NULL,
            &test_qname,
            &xsd_string_info);

    if (result == GLOBUS_SUCCESS)
    {
        rc = 1;

        goto out;
    }
    else if(! globus_error_match(globus_error_peek(result),
                GLOBUS_SOAP_MESSAGE_MODULE,
                GLOBUS_SOAP_MESSAGE_ERROR_TYPE_NULL_PARAM))
    {
        rc = 1;

        goto out;
    }
    result = globus_xsd_type_info_init_from_template(
            &info,
            NULL,
            &xsd_string_info);

    if (result == GLOBUS_SUCCESS)
    {
        rc = 1;

        goto out;
    }
    else if(! globus_error_match(globus_error_peek(result),
                GLOBUS_SOAP_MESSAGE_MODULE,
                GLOBUS_SOAP_MESSAGE_ERROR_TYPE_NULL_PARAM))
    {
        rc = 1;

        goto out;
    }
    result = globus_xsd_type_info_init_from_template(
            &info,
            &test_qname,
            NULL);

    if (result == GLOBUS_SUCCESS)
    {
        rc = 1;

        goto out;
    }
    else if(! globus_error_match(globus_error_peek(result),
                GLOBUS_SOAP_MESSAGE_MODULE,
                GLOBUS_SOAP_MESSAGE_ERROR_TYPE_NULL_PARAM))
    {
        rc = 1;

        goto out;
    }

out:
    test_finalize();
    return rc;
}

static
int 
null_destroy_test()
{
    int                                 rc = 0;

    rc = test_init();
    if (rc != 0)
    {
        return 1;
    }
    globus_xsd_type_info_destroy(NULL);
    test_finalize();
    return rc;
}

int main()
{
int i;
    int rc;
    int failed = 0;
    test_case tests[] =
    {
        /* Verify good behavior */
        init_destroy_test,

        /* Verify error handling */
        null_init_test,
        null_destroy_test,
    };

    printf("1..%d\n", SIZEOF_ARRAY(tests));

    for (i = 0; i < SIZEOF_ARRAY(tests); i++) {
        rc = (tests[i])();
        if (rc == 0)
        {
            printf("ok\n");
        }
        else
        {
            printf("not ok\n");
            failed++;
        }
    }
    return failed;

}
