/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */


#include "globus_common.h"
#include "version.h"
#include "globus_i_wsdl_config.h"

#define MAX_LINE_SIZE 1000

#ifdef WIN32
#define FILE_SEPARATOR "\\"
#else
#define FILE_SEPARATOR "/"
#endif

static int globus_l_wsdl_config_activate();
static int globus_l_wsdl_config_deactivate();

globus_module_descriptor_t              globus_i_wsdl_config_module =
{
    "globus_wsdl_config",
    globus_l_wsdl_config_activate,
    globus_l_wsdl_config_deactivate,
    GLOBUS_NULL,
    GLOBUS_NULL,
    &local_version
};

GlobusDebugDefine(GLOBUS_WSDL_CONFIG);

static int
globus_l_wsdl_config_activate()
{
    int                                 res = (int) GLOBUS_SUCCESS;
    
    res = (int) globus_module_activate(GLOBUS_COMMON_MODULE);
    GlobusDebugInit(GLOBUS_WSDL_CONFIG, ERROR TRACE INFO);
    return res;
}

static int
globus_l_wsdl_config_deactivate()
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    globus_module_deactivate(GLOBUS_COMMON_MODULE);
    GlobusDebugDestroy(GLOBUS_WSDL_CONFIG);
    return (int) result;
}

void
globus_i_wsdl_config_delete_entry(
    void *                              hash_entry)
{
    char **                             entry;

    entry = (char **) hash_entry;

    if(entry[0])
    {
        free(entry[0]);
    }

    if(entry[1])
    {
        free(entry[1]);
    }

    free(entry);
}

static
const char *
my_strpbrk(
    const char *                        str,
    size_t                              str_length,
    int *                               locator_index,
    const char *                        delims,
    int *                               delim_index);

char *
globus_wsdl_config_get_property(
    globus_hashtable_t                  props,
    const char *                        key)
{
    char **                             entry;

    entry = globus_hashtable_lookup(&props, (void *)key);
    if(!entry)
    {
        return NULL;
    }

    return (char *) entry[1];
}

globus_result_t
globus_wsdl_config_set_property(
    globus_hashtable_t *                props,
    const char *                        prop_string)
{
    globus_result_t                     result = GLOBUS_SUCCESS;
    int                                 res = 0;
    char *                              key;
    char *                              value;
    char *                              prop = NULL;
    char *                              pos;
    char *                              qpos;
    char *                              qpos_end;
    char *                              delim;
    int                                 line_length;
    int                                 index;
    int                                 delim_index;
    char **                             hash_entry;
    GlobusFuncName(globus_wsdl_config_set_property);
    GlobusWSDLConfigDebugEnter();

    prop = globus_libc_strdup(prop_string);
    pos = prop;
    line_length = strlen(prop);
    pos = (char *)my_strpbrk(pos, line_length, &index, "=", NULL);
    if(!pos)
    {
        result = GlobusWSDLConfigErrorNoEqualsProp(prop);
        goto exit;
    }
    
    *pos = '\0';

    key = prop;
    pos++;
    line_length -= (pos - prop);
    qpos = (char *) my_strpbrk(pos, line_length, &index, "\"'", &delim_index);
    if(!qpos)
    {
        value = pos;
    }
    else
    {
        line_length -= (qpos - pos);
        qpos++;
        if(delim_index == 0)
        {
            delim = "\"";
        }
        else
        {
            delim = "'";
        }
        
        qpos_end = (char *) my_strpbrk(qpos, line_length, &index, delim, NULL);
        if(!qpos_end)
        {
            result = GlobusWSDLConfigErrorNoEndQuoteProp(
                prop, delim);
            goto exit;
        }
        else
        {
            *qpos_end = '\0';
            value = qpos;
        }
    }

    hash_entry = globus_malloc(2 * sizeof(char *));
    if(!hash_entry)
    {
        result = GlobusWSDLConfigErrorOutOfMemory;
        goto exit;
    }
    
    hash_entry[0] = globus_libc_strdup(key);
    hash_entry[1] = globus_libc_strdup(value);

    res = globus_hashtable_insert(
        props, (void *) hash_entry[0], (void *) hash_entry);
    if(res != GLOBUS_SUCCESS)
    {
        result = GlobusWSDLConfigErrorHashInsertFailed();
        goto exit;
    }

 exit:

    if(prop)
    {
        free(prop);
    }

    GlobusWSDLConfigDebugExit();
    return result;
}

void
globus_wsdl_config_properties_destroy(
    globus_hashtable_t                  props)
{
    globus_hashtable_destroy_all(&props, globus_i_wsdl_config_delete_entry);
}

globus_result_t
globus_wsdl_config_properties_init(
    globus_hashtable_t *                props)
{
    GlobusFuncName(globus_wsdl_config_properties_init);
    GlobusWSDLConfigDebugEnter();
    
    globus_hashtable_init(props, 10, 
                          globus_hashtable_string_hash,
                          globus_hashtable_string_keyeq);
    globus_assert(props);
 
    GlobusWSDLConfigDebugExit();
    return GLOBUS_SUCCESS;
}


globus_result_t
globus_wsdl_config_load_properties(
    const char *                        filename,
    globus_hashtable_t *                properties)

{
    globus_result_t                     result = GLOBUS_SUCCESS;
    char                                line[MAX_LINE_SIZE];
    FILE *                              propstream = NULL;
    int                                 linenum = 0;
    char *                              key = NULL;
    char *                              value = NULL;
    char *                              new_value = NULL;
    size_t                              new_value_length = 0;
    char **                             hash_entry = NULL;
    GlobusFuncName(globus_wsdl_config_load_properties);
    GlobusWSDLConfigDebugEnter();

    globus_assert_string(properties, 
                         "NULL properties parameter passed to function");
    globus_assert_string(filename,
                         "NULL filename passed to function");

    propstream = fopen(filename, "r");
    if(!propstream)
    {
        result = GlobusWSDLConfigErrorFileOpen(filename);
        goto exit;
    }

    memset(line, 0, MAX_LINE_SIZE);
    while(fgets(line, MAX_LINE_SIZE, propstream) != NULL)
    {
        size_t                          line_length;
        char *                          pos = NULL;
        char *                          kpos = NULL;
        char *                          qpos = NULL;
        char *                          qpos_end = NULL;
        int                             index = 0;
        int                             delim_index = 0;
        char *                          delim = NULL;
        int                             res = 0;

        linenum++;
        line_length = strlen(line);

        pos = line;

        while(pos && (*pos == ' '))
        {
            ++pos;
        }

        if(pos && ((*pos == '#') || (*pos == '\n')))
        {
            continue;
        }

        if(line[line_length-1] == '\n')
        {
            line[line_length-1] = '\0';
        }

        kpos = pos;
        pos = (char *)my_strpbrk(pos, line_length, &index, "=", NULL);
        if(!pos)
        {
            result = GlobusWSDLConfigErrorNoEquals(filename, linenum);
            goto exit;
        }

        *pos = '\0';

        key = globus_libc_strdup(kpos);
        pos++;
        line_length -= (pos - line);
        qpos = (char *) my_strpbrk(pos, line_length, &index, "\"'", &delim_index);
        if(!qpos)
        {
            value = globus_libc_strdup(pos);
        }
        else
        {
            line_length -= (qpos - pos);
            qpos++;
            if(delim_index == 0)
            {
                delim = "\"";
            }
            else
            {
                delim = "'";
            }

            qpos_end = (char *) my_strpbrk(qpos, line_length, &index, delim, NULL);
            if(!qpos_end)
            {
                result = GlobusWSDLConfigErrorNoEndQuote(
                    filename, linenum, delim);
                goto exit;
            }
            else
            {
                *qpos_end = '\0';
                value = globus_libc_strdup(qpos);
            }
        }

        hash_entry = globus_malloc(2 * sizeof(char *));
        if(!hash_entry)
        {
            result = GlobusWSDLConfigErrorOutOfMemory;
            goto exit;
        }

        hash_entry[0] = globus_libc_strdup(key);
        hash_entry[1] = globus_libc_strdup(value);
        res = globus_hashtable_insert(
            properties, (void *) key, (void *) hash_entry);
        if(res != GLOBUS_SUCCESS)
        {
            result = GlobusWSDLConfigErrorHashInsertFailed();
            goto exit;
        }

        hash_entry = NULL;
        key = NULL;
        value = NULL;

        memset(line, 0, MAX_LINE_SIZE);
    }

    hash_entry = (char **)globus_hashtable_first(properties);
    while(hash_entry)
    {
        result = globus_wsdl_config_expand_refs(
            *properties, hash_entry[1], strlen(hash_entry[1]) + 1, 
            &new_value, &new_value_length);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusWSDLConfigErrorExpandingRefs(result, filename, linenum);
            goto exit;
        }

        if(hash_entry[1] != new_value)
        {
            free(hash_entry[1]);
            hash_entry[1] = globus_libc_strdup(new_value);
        }

        new_value = NULL;
        
        hash_entry = (char **)globus_hashtable_next(properties);
    }

 exit:
    
    if(propstream)
    {
        fclose(propstream);
    }

    if(key)
    {
        free(key);
    }

    if(value)
    {
        free(value);
    }

    if(new_value)
    {
        free(new_value);
    }

    if(hash_entry)
    {
        globus_free(hash_entry);
    }

    GlobusWSDLConfigDebugExit();
    return result;
}        
        
globus_result_t
globus_wsdl_config_expand_refs(
    globus_hashtable_t                  hash,
    char *                              value,
    size_t                              value_length,
    char **                             new_value,
    size_t *                            new_length)
{
    char *                              nvalue = NULL;
    size_t                              nvalue_length = 0;
    char *                              rp = NULL;
    char *                              prev_rp = NULL;
    size_t                              rp_length = 0;
    int                                 rp_index = 0;
    int                                 newrp_index = 0;
    char *                              rp_env = NULL;
    char *                              tmp_rp = NULL;
    int                                 tmp_rp_index = 0;
    char *                              envstring = NULL;
    globus_result_t                     result = GLOBUS_SUCCESS;
    char **                             hash_entry = NULL;

    GlobusFuncName(globus_wsdl_config_expand_refs);
    GlobusWSDLConfigDebugEnter();

    rp = (char *)value;
    prev_rp = rp;
    rp_length = value_length;
    rp = (char *)my_strpbrk(rp, rp_length, &rp_index, "$", NULL);

    if(!rp)
    {
        nvalue = value;
        nvalue_length = value_length;
    }

    while(rp)
    {
        rp_length -= rp_index;

        if(rp_index != 0)
        {
            if(rp[-1] == '\\')
            {
                rp++;
                rp_length--;
                rp = (char *)my_strpbrk(rp, rp_length, &newrp_index,
                                        "$", NULL);
                rp_length -= newrp_index;
                rp_index += newrp_index + 1;
            }
        }

        
        nvalue = realloc(nvalue, nvalue_length + rp_index);
        memcpy(nvalue + (nvalue_length - 1), prev_rp, rp_index);
        nvalue_length += rp_index;

        if(rp_length && (*(rp + 1) == '{'))
        {
            rp_env = rp + 2;
            rp_length -= 2;
        
            tmp_rp = (char *)my_strpbrk(rp_env, rp_length, &tmp_rp_index, "}", NULL);
            if(!tmp_rp)
            {
                result = GlobusWSDLConfigErrorNoCurly();
                goto error_exit;
            }
            
            envstring = malloc(tmp_rp_index + 1);
            memcpy(envstring, rp_env, tmp_rp_index);
            envstring[tmp_rp_index] = '\0';
            rp = tmp_rp + 1;
            rp_length -= tmp_rp_index - 1;
        }
        else
        {
            rp_env = rp + 1;
            tmp_rp = (char *)my_strpbrk(rp_env, rp_length, &tmp_rp_index, 
                                        FILE_SEPARATOR" ", NULL);
            if(tmp_rp)
            {
                rp = tmp_rp;
                rp_length -= tmp_rp_index;
                envstring = malloc(tmp_rp_index + 1);
                memcpy(envstring, rp_env, tmp_rp_index);
                envstring[tmp_rp_index] = '\0';
            }
            else
            {
                rp = tmp_rp;
                envstring = malloc(rp_length + 1);
                memcpy(envstring, rp_env, rp_length);
                envstring[rp_length] = '\0';
                rp_length = 0;
            }
        }

        hash_entry = globus_hashtable_lookup(&hash, (void *) envstring);
        if(hash_entry)
        {
            size_t                      tmp_length = strlen(hash_entry[1]);
            nvalue = realloc(nvalue, nvalue_length + tmp_length);
            memcpy(nvalue + (nvalue_length - 1), hash_entry[1], tmp_length);
            nvalue_length += tmp_length;
        }
        else
        {
            char *                      env_value;
            size_t                      tmp_length;
            env_value = getenv(envstring);
            if(!env_value)
            {
                result = GlobusWSDLConfigErrorNoEnv(envstring);
                goto error_exit;
            }

            tmp_length += strlen(env_value);
            nvalue = realloc(nvalue, nvalue_length + tmp_length);
            memcpy(nvalue + (nvalue_length - 1), env_value, tmp_length);
            nvalue_length += tmp_length;
        }

        if(envstring)
        {
            free(envstring);
            envstring = NULL;
        }

        if(rp)
        {
            prev_rp = rp;
            rp = (char *)my_strpbrk(rp, rp_length, &rp_index, "$", NULL);
        }
    }

    if(rp && rp_length)
    {
        nvalue = realloc(nvalue, nvalue_length + rp_length);
        memcpy(nvalue + (nvalue_length - 1), rp, rp_length);
        nvalue_length += rp_length;
    }
    
    nvalue[nvalue_length - 1] = '\0';

    *new_value = nvalue;
    *new_length = nvalue_length;
    
    goto exit;

 error_exit:
    
    if(nvalue)
    {
        free(nvalue);
    }
    
 exit:

    if(envstring)
    {
        free(envstring);
    }
    
    GlobusWSDLConfigDebugExit();
    return result;
}

static
const char *
my_strpbrk(
    const char *                        str,
    size_t                              str_length,
    int *                               locator_index,
    const char *                        delims,
    int *                               delim_index)
{
    const char *                        locator;
    int                                 delim_count, ind, i;
    
    ind = 0;
    i = 0;
    delim_count = strlen(delims);
    locator = str;
    while(*locator && ind < str_length)
    {
        for(i = 0; i < delim_count; ++i)
        {
            if(*locator == delims[i])
            {
                if(locator_index)
                {
                    *locator_index = ind;
                }
                
                if(delim_index)
                {
                    *delim_index = i;
                }
                return locator;
            }
        }
        
        locator++;
        ind++;
    }

    if(locator_index)
    {
        *locator_index = -1;
    }
    return NULL;
}
