/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */


#include "globus_common.h"
#include "libxml/xmlschemas.h"
#include "libxml/schemasInternals.h"
#include "libxml/uri.h"

#include "wsdl.h"

#include "globus_wsdl_template.h"
#include "globus_i_wsdl_parser.h"
#include "globus_wsdl_config.h"

#include "js/jsstddef.h"
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "js/jstypes.h"
#include "js/jsarena.h"
#include "js/jsutil.h"
#include "js/jsprf.h"
#include "js/jsapi.h"
#include "js/jsatom.h"
#include "js/jscntxt.h"
#include "js/jsdbgapi.h"
#include "js/jsemit.h"
#include "js/jsfun.h"
#include "js/jsgc.h"
#include "js/jslock.h"
#include "js/jsobj.h"
#include "js/jsparse.h"
#include "js/jsscope.h"
#include "js/jsscript.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>

#ifdef WIN32
#define FILE_SEPARATOR "\\"
#else
#define FILE_SEPARATOR "/"
#endif

#define GLOBUS_L_WSDL_JSCRIPT_UNBOUNDED (1 << 30)

size_t                                  gStackChunkSize = 8192;

typedef struct _StreamContext           StreamContext;
struct _StreamContext
{
    FILE *                              error_stream;
    FILE *                              out_stream;
    StreamContext *                     next;
};

typedef struct _Template                Template;
struct _Template
{
    char *                              filename;
    char *                              buffer;
    size_t                              length;
};

typedef struct _PrivateContext          PrivateContext;
struct _PrivateContext
{
    StreamContext *                     streams;
    globus_result_t                     result;
    globus_hashtable_t                  nsmap;
    int                                 free_nsmap;
    globus_hashtable_t                  templates;
    char *                              outdir;
    globus_list_t *                     template_paths;
};

static
void TemplateFree(
    void *                              template);

static JSBool
LoadTemplate(
    JSContext *                         ctx,
    JSObject *                          obj,
    uintN                               argc,
    jsval *                             argv,
    jsval *                             rval);

static JSBool
FileExists(
    JSContext *                         ctx,
    JSObject *                          obj,
    uintN                               argc,
    jsval *                             argv,
    jsval *                             rval);

static JSBool
CreateSubDir(
    JSContext *                         ctx,
    JSObject *                          obj,
    uintN                               argc,
    jsval *                             argv,
    jsval *                             rval);

static JSBool
URIBasePath(
    JSContext *                         ctx,
    JSObject *                          obj,
    uintN                               argc,
    jsval *                             argv,
    jsval *                             rval);
         
static JSBool
NamespaceToPrefix(
    JSContext *                         cx,
    JSObject *                          obj,
    uintN                               argc,
    jsval *                             argv,
    jsval *                             rval);

static JSBool
Debug(
    JSContext *                         cx,
    JSObject *                          obj,
    uintN                               argc,
    jsval *                             argv,
    jsval *                             rval);

static JSBool
Abort(
    JSContext *                         cx,
    JSObject *                          obj,
    uintN                               argc,
    jsval *                             argv,
    jsval *                             rval);

static JSBool
Print(
    JSContext *                         cx,
    JSObject *                          obj,
    uintN                               argc,
    jsval *                             argv,
    jsval *                             rval);

static JSBool
Error(
    JSContext *                         cx,
    JSObject *                          obj,
    uintN                               argc,
    jsval *                             argv,
    jsval *                             rval);

static JSBool
CanonicalizeString(
    JSContext *                         cx,
    JSObject *                          obj,
    uintN                               argc,
    jsval *                             argv,
    jsval *                             rval);

#define DEF_GET_PROP(_PREFIX_)                                  \
    static JSBool                                               \
    _PREFIX_ ## Getter(                                         \
        JSContext *                     cx,                     \
        JSObject *                      obj,                    \
        jsval                           id,                     \
        jsval *                         vp)

#define DEF_ENUM_PROP(_PREFIX_)                         \
    static JSBool                                       \
    _PREFIX_ ## Enumerate(                              \
        JSContext *                     cx,             \
        JSObject *                      obj,            \
        JSIterateOp                     enum_op,        \
        jsval *                         statep,         \
        jsid *                          idp)

#define DEF_CLASS(_PREFIX_)                                             \
    static JSClass _PREFIX_ ## _class =                                 \
    {                                                                   \
        #_PREFIX_,             JSCLASS_HAS_PRIVATE|JSCLASS_NEW_RESOLVE, \
        JS_PropertyStub,       JS_PropertyStub,                         \
        _PREFIX_ ## Getter,    JS_PropertyStub,                         \
        JS_EnumerateStub,      JS_ResolveStub,                          \
        JS_ConvertStub,        JS_FinalizeStub                          \
    };                                                                  \
    static JSObject * _PREFIX_ ## Class

#define DEF_LISTCLASS(_PREFIX_)                                 \
    static JSClass _PREFIX_ ## _class =                         \
    {                                                           \
        #_PREFIX_, JSCLASS_NEW_ENUMERATE|JSCLASS_HAS_PRIVATE,   \
        JS_PropertyStub,       JS_PropertyStub,                 \
        _PREFIX_ ## Getter,    JS_PropertyStub,                 \
        (JSEnumerateOp) _PREFIX_ ## Enumerate,                  \
        JS_ResolveStub,                                         \
        JS_ConvertStub,        JS_FinalizeStub                  \
    }

#define DEF_PROPS(_PREFIX_)                             \
    extern JSPropertySpec _PREFIX_ ## _properties[]

#define DEF_LIST(_PREFIX_)                      \
    DEF_ENUM_PROP(_PREFIX_);                    \
    DEF_GET_PROP(_PREFIX_);                     \
    DEF_LISTCLASS(_PREFIX_)

#define DEF_OBJECT(_PREFIX_)                    \
    DEF_GET_PROP(_PREFIX_);                     \
    DEF_PROPS(_PREFIX_);                        \
    DEF_CLASS(_PREFIX_)

DEF_OBJECT(xsdType);
DEF_OBJECT(xsdAttr);
DEF_OBJECT(xsdElement);
DEF_OBJECT(xsdWildcard);
DEF_OBJECT(xsdFacet);

DEF_OBJECT(xsdQName);
DEF_OBJECT(part);
DEF_OBJECT(operation);
DEF_OBJECT(binding);
DEF_OBJECT(portType);
DEF_OBJECT(message);
DEF_OBJECT(param);
DEF_OBJECT(fault);
DEF_OBJECT(port);
DEF_OBJECT(service);
DEF_OBJECT(bindingOperation);
DEF_OBJECT(bindingOperationFault);

DEF_OBJECT(soapBinding);
DEF_OBJECT(soapOperation);
DEF_OBJECT(soapBindingParam);
DEF_OBJECT(soapBody);
DEF_OBJECT(soapFault);
DEF_OBJECT(soapHeader);
DEF_OBJECT(soapAddress);

DEF_GET_PROP(xsdEnum);
DEF_CLASS(xsdEnum);

DEF_PROPS(xsdTypeEnum);
DEF_PROPS(xsdContentTypeEnum);
DEF_PROPS(xsdAttrEnum);
DEF_PROPS(operationTypeEnum);
DEF_PROPS(bindingType);
DEF_PROPS(soapOperationStyleType);
DEF_PROPS(soapUseType);

DEF_LIST(xsdSubTypes);
DEF_LIST(xsdAttrs);
DEF_LIST(xsdQNames);
DEF_LIST(parts);
DEF_LIST(operations);
DEF_LIST(faults);
DEF_LIST(ports);
DEF_LIST(bindingOperations);
DEF_LIST(bindingOperationFaults);
DEF_LIST(xsdFacets);

static void
internal_ErrorReporter(
    JSContext *                         cx,
    const char *                        message,
    JSErrorReport *                     report);

static JSFunctionSpec functions_defs[] = 
{
    {"load",                 LoadTemplate,       0},
    {"file_exists",          FileExists,         0},
    {"uri_base_path",        URIBasePath,        0},
    {"create_subdir",        CreateSubDir,       0},
    {"debug",                Debug,              0},
    {"abort",                Abort,              0},
    {"error",                Error,              0},
    {"print",                Print,              0},
    {"canonical",            CanonicalizeString, 0},
    {"NStoP",                NamespaceToPrefix,  0},
    {0}
};

static JSClass global_class = 
{
    "global", JSCLASS_NEW_RESOLVE,
    JS_PropertyStub,  JS_PropertyStub,
    JS_PropertyStub,  JS_PropertyStub,
    JS_EnumerateStub, JS_ResolveStub,
    JS_ConvertStub,   JS_FinalizeStub
};

static JSClass basic_class =
{
    "basic", JSCLASS_NEW_RESOLVE,
    JS_PropertyStub,  JS_PropertyStub,
    JS_PropertyStub,  JS_PropertyStub,
    JS_EnumerateStub, JS_ResolveStub,
    JS_ConvertStub,   JS_FinalizeStub
};

typedef struct _ScannerData             ScannerData;
struct _ScannerData
{
    wsdlSchemaPtr                       schema;
    xmlSchemaPtr                        xsdSchema;
    JSContext *                         context;
    JSObject *                          global;
    JSObject *                          types;
    JSObject *                          elements;
    JSObject *                          local_elements;
    JSObject *                          attributes;
    JSObject *                          local_attributes;
    JSObject *                          messages;
    JSObject *                          portTypes;
    JSObject *                          bindings;
    JSObject *                          services;
    globus_result_t                     result;
};

static
void
globus_i_wsdl_parser_import_scanner(
    void *                              payload,
    void *                              data,
    xmlChar *                           name);

static
void
globus_i_wsdl_parser_type_scanner(
    void *                              payload,
    void *                              data,
    xmlChar *                           name);

static
void
globus_i_wsdl_parser_element_scanner(
    void *                              payload,
    void *                              data,
    xmlChar *                           name);

static
void
globus_i_wsdl_parser_attribute_scanner(
    void *                              payload,
    void *                              data,
    xmlChar *                           name);

static
void
globus_i_wsdl_parser_message_scanner(
    void *                              payload,
    void *                              data,
    xmlChar *                           name);

static
void
globus_i_wsdl_parser_portType_scanner(
    void *                              payload,
    void *                              data,
    xmlChar *                           name);

static
void
globus_i_wsdl_parser_binding_scanner(
    void *                              payload,
    void *                              data,
    xmlChar *                           name);

static
void
globus_i_wsdl_parser_service_scanner(
    void *                              payload,
    void *                              data,
    xmlChar *                           name);

static
void
globus_i_wsdl_parser_wsdl_imports_scanner(
    void *                              payload,
    void *                              data,
    xmlChar *                           name);

static
globus_result_t
globus_i_wsdl_js_load_schema(
    JSContext *                         cx,
    wsdlSchemaPtr                       schema);

static 
globus_result_t
globus_i_wsdl_execute_script(
    JSContext *                         ctx,
    JSObject *                          glob,
    const char *                        tmpl_filename,
    const char *                        out_filename,
    const char *                        out_dir,
    globus_hashtable_t                  nsmap,
    globus_list_t *                     template_paths);

static
JSString *
globus_l_wsdl_jscript_qname_string(
    JSContext *                         ctx,
    const char *                        href,
    const char *                        local);

static
void
globus_i_wsdl_parser_classes_init(
    JSContext *                         cx,
    JSObject *                          glob)
{
    xsdTypeClass = JS_InitClass(cx, glob, NULL, &xsdType_class, NULL, 0,
                                xsdType_properties, NULL, NULL, NULL);

    xsdElementClass = JS_InitClass(cx, glob, NULL, &xsdElement_class, NULL, 0,
                                   xsdElement_properties, NULL, NULL, NULL);

    xsdAttrClass = JS_InitClass(cx, glob, NULL, &xsdAttr_class, NULL, 0,
                                xsdAttr_properties, NULL, NULL, NULL);

    xsdWildcardClass = JS_InitClass(cx, glob, NULL, &xsdWildcard_class, NULL, 0,
                                    xsdWildcard_properties, NULL, NULL, NULL);

    xsdFacetClass = JS_InitClass(cx, glob, NULL, &xsdFacet_class, NULL, 0,
                                 xsdFacet_properties, NULL, NULL, NULL);

    messageClass = JS_InitClass(cx, glob, NULL, &message_class, NULL, 0,
                                message_properties, NULL, NULL, NULL);

    portTypeClass = JS_InitClass(cx, glob, NULL, &portType_class, NULL, 0,
                                portType_properties, NULL, NULL, NULL);

    operationClass = JS_InitClass(cx, glob, NULL, &operation_class, NULL, 0,
                                  operation_properties, NULL, NULL, NULL);

    bindingClass = JS_InitClass(cx, glob, NULL, &binding_class, NULL, 0,
                                binding_properties, NULL, NULL, NULL);

    serviceClass = JS_InitClass(cx, glob, NULL, &service_class, NULL, 0,
                                service_properties, NULL, NULL, NULL);

    bindingOperationClass = JS_InitClass(cx, glob, NULL, 
                                         &bindingOperation_class, NULL, 0,
                                         bindingOperation_properties, 
                                         NULL, NULL, NULL);

    soapBindingClass = JS_InitClass(cx, glob, NULL, &soapBinding_class, NULL, 0,
                                    soapBinding_properties, NULL, NULL, NULL);

    soapOperationClass = JS_InitClass(cx, glob, NULL, &soapOperation_class, 
                                      NULL, 0, soapOperation_properties, 
                                      NULL, NULL, NULL);

}

static JSBool
URIBasePath(
    JSContext *                         cx, 
    JSObject *                          obj, 
    uintN                               argc,
    jsval *                             argv,
    jsval *                             rval)
{
    char *                              uri_base_path;
    JSString *                          str;
    char *                              uri_string;
    xmlURIPtr                           uri_handle;
    JSBool                              res = JS_TRUE;
    GlobusFuncName(URIBasePath);
    GlobusWSDLDebugEnter();

    if(argc < 1)
    {
        JS_ReportError(cx, 
                       "Wrong number of arguments "
                       "passed to function: %s",
                       argc, 2, "load(template, outfile)");
        res = JS_FALSE;
        goto exit;
    }

    str = JS_ValueToString(cx, argv[0]);
    if(!str)
    {
        JS_ReportError(cx,
                       "Failed to load template");
        res = JS_FALSE;
        goto exit;
    }
    
    uri_string = globus_common_create_string("%*s", 
                                             JS_GetStringLength(str),
                                             JS_GetStringBytes(str));
    
    uri_handle = xmlParseURI(uri_string);
    if(!uri_handle)
    {
        JS_ReportError(cx, "Failed to parse uri: %s", uri_string);
        res = JS_FALSE;
        goto exit;
    }

    uri_base_path = uri_handle->path;
    while((*uri_base_path) == '/')
    {
        ++uri_base_path;
    }

    str = JS_InternString(cx, uri_base_path);
    *rval = STRING_TO_JSVAL(str);

 exit:
    GlobusWSDLDebugExit();
    return res;
}

static JSBool
CreateSubDir(
    JSContext *                         cx, 
    JSObject *                          obj, 
    uintN                               argc,
    jsval *                             argv,
    jsval *                             rval)
{
    JSString *                          str;
    char *                              dir_string;
    JSBool                              res = JS_TRUE;
    PrivateContext *                    private = NULL;
    GlobusFuncName(CreateSubDir);
    GlobusWSDLDebugEnter();

    if(argc < 1)
    {
        JS_ReportError(cx, 
                       "Wrong number of arguments "
                       "passed to function: %s",
                       argc, 2, "load(template, outfile)");
        res = JS_FALSE;
        goto exit;
    }

    private = (PrivateContext *) JS_GetContextPrivate(cx);

    str = JS_ValueToString(cx, argv[0]);
    if(!str)
    {
        JS_ReportError(cx,
                       "Failed to load template");
        res = JS_FALSE;
        goto exit;
    }
    
    dir_string = globus_common_create_string(
        "%s%s%*s", 
        (private->outdir ? private->outdir : ""),
        (private->outdir ? FILE_SEPARATOR : ""),
        (int) JS_GetStringLength(str),
        JS_GetStringBytes(str));
    
    if(mkdir(dir_string, 00770) < 0)
    {
        if(errno != EEXIST)
        {
            globus_object_t *           errobj;
            char *                      errstr;
		
            errobj = globus_error_wrap_errno_error(
                    GLOBUS_WSDL_PARSER_MODULE,
                    errno,
                    GLOBUS_WSDL_ERROR_JSRUNTIME,
                    __FILE__,
                    _globus_func_name,
                    __LINE__,
                    "Failed to create directory: %s",
                    dir_string); 
            errstr = globus_error_print_friendly(errobj);
            globus_object_free(errobj);
            JS_ReportError(cx, "subdir %s creation failed:%d:%s", 
                           dir_string, errno, errstr);
            globus_free(errstr);
            free(dir_string);
            res = JS_FALSE;
            goto exit;
        }
    }

    free(dir_string);

    *rval = JSVAL_NULL;

 exit:
    GlobusWSDLDebugExit();
    return res;
}

static JSBool
LoadTemplate(
    JSContext *                         cx, 
    JSObject *                          obj, 
    uintN                               argc,
    jsval *                             argv,
    jsval *                             rval)
{
    char *                              script_buffer = NULL;
    size_t                              script_length = 0;
    jsval                               result;
    JSBool                              res = JS_TRUE;
    char *                              error = NULL;
    char *                              path = NULL;
    JSString *                          str;
    char                                filename[500];
    char                                out_filename[500];
    FILE *                              out_stream = NULL;
    PrivateContext *                    private = NULL;
    Template *                          template = NULL;
    StreamContext *                     new_streams = NULL;
    StreamContext *                     tmp_streams = NULL;
    globus_list_t *                     paths = NULL;
    struct stat                         statbuf;
    int                                 filename_found = 0;
    GlobusFuncName(LoadTemplate);
    GlobusWSDLDebugEnter();

    if(argc < 1)
    {
        JS_ReportError(cx, 
                       "Wrong number of arguments "
                       "passed to function: %s",
                       argc, 2, "load(template, outfile)");
        res = JS_FALSE;
        goto exit;
    }

    str = JS_ValueToString(cx, argv[0]);
    if(!str)
    {
        JS_ReportError(cx,
                       "Failed to load template");
        res = JS_FALSE;
        goto exit;
    }
    
    private = (PrivateContext *) JS_GetContextPrivate(cx);
    paths = private->template_paths;

    while(paths)
    {
        path = globus_list_first(paths);
        sprintf(filename, "%s/%*s", 
                path, (int) JS_GetStringLength(str), 
                JS_GetStringBytes(str));
        
        if(private->templates)
        {
            template = globus_hashtable_lookup(&private->templates, filename);
            if(template)
            {
                filename_found = 1;
                break;
            }
        }

        if(stat(filename, &statbuf))
        {
            if(errno != ENOENT)
            {
                char *                      errstr;
                result = GlobusWSDLErrorAccessingTemplateFile(errno, filename);
                errstr = globus_error_print_chain(globus_error_get(result));
                
                JS_ReportError(cx, errstr);
                res = JS_FALSE;
                goto exit;
            }
        }
        else
        {
            filename_found = 1;
            break;
        }
                
        paths = globus_list_rest(paths);
    }

    if(!filename_found)
    {
        sprintf(filename, "%*s", (int) JS_GetStringLength(str), JS_GetStringBytes(str));
        if(private->templates)
        {
            template = globus_hashtable_lookup(&private->templates, filename);
        }
    }

    if(!template)
    {
        if(stat(filename, &statbuf))
        {
            char *                      errstr;
            result = GlobusWSDLErrorAccessingTemplateFile(errno, filename);
            errstr = globus_error_print_chain(globus_error_get(result));

            JS_ReportError(cx, errstr);
            res = JS_FALSE;
            goto exit;
        }

        res = globus_wsdl_template_get_jsbuf(
            (const char *) filename, &script_buffer, &script_length, &error,
            '@', '$');
        if(res != 0)
        {
            if(error)
            {
                JS_ReportError(cx, "%s", error);
                free(error);
            }
            res = JS_FALSE;
            goto exit;
        }
        
        GlobusWSDLDebugPrintf(GLOBUS_WSDL_DEBUG_TEMPLATE,
                              ("TEMPLATE BUFFER:"
                               "\n----------------------\n"));
        GlobusWSDLDebugWrite(GLOBUS_WSDL_DEBUG_TEMPLATE,
                             script_length, script_buffer);
        GlobusWSDLDebugPrintf(GLOBUS_WSDL_DEBUG_TEMPLATE,
                              ("\n----------------------\n"));
        
        template = globus_malloc(sizeof(Template));
        if(!template)
        {
            JS_ReportError(cx,
                           "Memory allocation failed.");
            res = JS_FALSE;
            goto exit;
        }
        template->filename = globus_libc_strdup(filename);
        template->buffer = script_buffer;
        template->length = script_length;
        
        if(!private->templates)
        {
            globus_hashtable_init(&private->templates, 10, 
                                  globus_hashtable_string_hash, 
                                  globus_hashtable_string_keyeq);
        }
        
        globus_hashtable_insert(&private->templates, template->filename, template);
    }
    else
    {
        script_buffer = template->buffer;
        script_length = template->length;
    }
    
    if(argc > 1 && (argv[1] != JSVAL_NULL))
    {
        str = JS_ValueToString(cx, argv[1]);
        if(!str)
        {
            JS_ReportError(cx, 
                           "Failed to load template %s.  "
                           "Second argument to load function should "
                           "be output filename but is not of type string.",
                           filename);
            res = JS_FALSE;
            goto exit;
        }
        
        sprintf(out_filename, "%s%s%*s", 
                (private->outdir ? private->outdir : ""),
                (private->outdir ? FILE_SEPARATOR : ""),
                (int) JS_GetStringLength(str), 
                JS_GetStringBytes(str));
        
        out_stream = fopen(out_filename, "w+");
        if(!out_stream)
        {
            JS_ReportError(cx, "Failed to open output file %s for writing", 
                           out_filename);
            res = JS_FALSE;
            goto exit;
        }
    }

    globus_assert(private && private->streams);

    new_streams = (StreamContext *) malloc(sizeof(StreamContext));
    if(!new_streams)
    {
        JS_ReportOutOfMemory(cx);
        res = JS_FALSE;
        goto exit;
    }

    memset(new_streams, 0, sizeof(StreamContext));
    
    new_streams->next = private->streams;
    new_streams->error_stream = private->streams->error_stream;
    new_streams->out_stream = out_stream;
    private->streams = new_streams;

    JS_SetContextPrivate(cx, (void *) private);

    res = JS_EvaluateScript(cx, obj, script_buffer, 
                            script_length, filename, 0, &result);
    if(!res)
    {
        JS_ReportError(cx, "Failed to evaluate script: %s", filename);
        goto exit;
    }
    
/*     res = JS_ExecuteScript(cx, obj, script, &result); */
/*     JS_DestroyScript(cx, script); */

    private = JS_GetContextPrivate(cx);
    globus_assert(private);

    if(private->streams->out_stream)
    {
        fclose(private->streams->out_stream);
    }
    
    tmp_streams = private->streams;
    if(tmp_streams)
    {
        private->streams = private->streams->next;
        free(tmp_streams);
    }

    JS_SetContextPrivate(cx, (void *) private);

  exit:
    GlobusWSDLDebugExit();
    return res;
}    

static JSBool
FileExists(
    JSContext *                         cx, 
    JSObject *                          obj, 
    uintN                               argc,
    jsval *                             argv,
    jsval *                             rval)
{
    JSBool                              res = JS_TRUE;
    JSString *                          str;
    char                                filename[500];
    PrivateContext *                    private = NULL;
    struct stat                         statbuf;
    GlobusFuncName(FileExists);
    GlobusWSDLDebugEnter();

    if(argc < 1)
    {
        JS_ReportError(cx, 
                       "Wrong number of arguments "
                       "passed to function: %s",
                       argc, 2, "file_exists(filename)");
        res = JS_FALSE;
        goto exit;
    }

    str = JS_ValueToString(cx, argv[0]);
    if(!str)
    {
        JS_ReportError(cx,
                       "Failed to load filename");
        res = JS_FALSE;
        goto exit;
    }

    private = (PrivateContext *) JS_GetContextPrivate(cx);
    
    sprintf(filename, "%s%s%*s", 
            (private->outdir ? private->outdir : ""),
            (private->outdir ? FILE_SEPARATOR : ""),
            (int) JS_GetStringLength(str), 
            JS_GetStringBytes(str));

    if(!stat(filename, &statbuf))
    {
        *rval = BOOLEAN_TO_JSVAL(JS_TRUE);
    }
    else
    {
        *rval = BOOLEAN_TO_JSVAL(JS_FALSE);
    }


  exit:
    GlobusWSDLDebugExit();
    return res;
}    

static JSBool
CanonicalizeString(
    JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
    uintN i, n;
    JSString *str;
    char *s, *news;

    if(argc < 1)
    {
        JS_ReportError(cx, "function canonical called with no parameters");
        return JS_FALSE;
    }

    if(JSVAL_IS_NULL(argv[0]) || JSVAL_IS_VOID(argv[0]))
    {
        str = JS_InternString(cx, "NULL");
        *rval = STRING_TO_JSVAL(str);
        return JS_TRUE;
    }

    if(!JSVAL_IS_STRING(argv[0]))
    {
        JS_ReportError(cx, "parameter passed to canonical is not a string");
        return JS_FALSE;
    }

    str = JSVAL_TO_STRING(argv[0]);
    n = JSSTRING_LENGTH(str);
    news = (char *) JS_malloc(cx, (n + 1) * sizeof(char));
    if(!news)
    {
        JS_ReportOutOfMemory(cx);
        return JS_FALSE;
    }

    s = JS_GetStringBytes(str);
    for(i = 0; i < n; ++i)
    {
        switch(s[i])
        {
        case '-':
            news[i] = '_';
            break;
        case ' ':
            news[i] = '_';
            break;
        default:
            news[i] = s[i];
            break;
        }
    }
    
    news[n] = 0;
    str = JS_InternString(cx, news);
    JS_free(cx, news);
    if(!str)
    {
        JS_ReportError(cx, "Failed to internalize string in canonical function");
        return JS_FALSE;
    }

    *rval = STRING_TO_JSVAL(str);
    return JS_TRUE;
}

static JSBool
NamespaceToPrefix(JSContext *cx, JSObject *obj, 
                  uintN argc, jsval *argv, jsval *rval)
{
    globus_hashtable_t                  nsmap;
    PrivateContext *                    ctx;
    JSString *                          str;
    char *                              prefix;

    ctx = (PrivateContext *) JS_GetContextPrivate(cx);

    nsmap = (globus_hashtable_t) ctx->nsmap;
    if(!nsmap)
    {
        str = JS_InternString(cx, "");
        if(!str)
        {
            JS_ReportError(cx, 
                           "Failed to access namespace mapping table in NStoP");
            return JS_FALSE;
        }

        *rval = STRING_TO_JSVAL(str);
        return JS_TRUE;
    }

    str = JS_ValueToString(cx, argv[0]);
    if (!str)
    {
        JS_ReportError(cx, "Parameter passed to NStoP is not a string");
        return JS_FALSE;
    }
    
    prefix = globus_wsdl_config_get_property(nsmap, 
                                             (const char *)JS_GetStringBytes(str));
    if(!prefix)
    {
        str = JS_InternString(cx, "");
        if(!str)
        {
            JS_ReportError(cx, "NStoP failed: could not internalize string");
            return JS_FALSE;
        }

        *rval = STRING_TO_JSVAL(str);
        return JS_TRUE;
    }

    str = JS_InternString(cx, prefix);
    *rval = STRING_TO_JSVAL(str);
    return JS_TRUE;
}
    
        
static JSBool
Debug(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
    uintN i, n;
    JSString *str;

    for (i = n = 0; i < argc; i++) {
        str = JS_ValueToString(cx, argv[i]);
        if (!str)
        {
            JS_ReportError(
                cx, 
                "Argument %d to debug function is not a string", i);
            return JS_FALSE;
        }

        fprintf(stdout, "%s%s", i ? " " : "", JS_GetStringBytes(str));
    }

    n++;
    return JS_TRUE;
}

static JSBool
Error(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
    uintN i, n;
    JSString *str;
    char error_string[5000];
    int len = 0;
    PrivateContext * ctx_handle;
    
    ctx_handle = (PrivateContext *) JS_GetContextPrivate(cx);

    for (i = n = 0; i < argc; i++) {
        str = JS_ValueToString(cx, argv[i]);
        if (!str)
        {
            JS_ReportError(
                cx, 
                "Argument %d to debug function is not a string", i);
            return JS_FALSE;
        }

        len += sprintf(error_string + len, 
                       "%s%s", i ? " " : "", JS_GetStringBytes(str));
    }

    JS_ReportError(
        cx, "ERROR: %s", error_string);
    return JS_FALSE;
}

static JSBool
Abort(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
    abort();
}

static JSBool
Print(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
    uintN i, n;
    JSString *str;
    PrivateContext * ctx_handle;
    FILE * ostream;
    
    ctx_handle = (PrivateContext *) JS_GetContextPrivate(cx);
    if(!ctx_handle || !ctx_handle->streams || !ctx_handle->streams->out_stream)
    {
        ostream = stdout;
    }
    else
    {
        ostream = ctx_handle->streams->out_stream;
    }

    for (i = n = 0; i < argc; i++) {
        str = JS_ValueToString(cx, argv[i]);
        if (!str)
        {
            JS_ReportError(cx, "Argument %d to print function is not a string", i);
            return JS_FALSE;
        }

        fprintf(ostream, "%s%s", i ? " " : "", JS_GetStringBytes(str));
    }

    n++;
    return JS_TRUE;
}


static void
internal_ErrorReporter(
    JSContext *                         cx,
    const char *                        message,
    JSErrorReport *                     report)
{
    char *prefix, *tmp, *linebuf;
    PrivateContext * ctx_handle;
    GlobusFuncName(internal_ErrorReporting);
    GlobusWSDLDebugEnter();
    
    ctx_handle = (PrivateContext *) JS_GetContextPrivate(cx);
    if(!ctx_handle)
    {
        goto exit;
    }

    if (!report) {
        ctx_handle->result = globus_error_put(
            globus_error_construct_error(
                GLOBUS_WSDL_PARSER_MODULE, 
                (ctx_handle->result != GLOBUS_NULL)
                    ? globus_error_get(ctx_handle->result)
                    : NULL,
                GLOBUS_WSDL_ERROR_JAVASCRIPT,
                __FILE__,
                _globus_func_name,
                __LINE__,
                "%s\n", 
                message));
        goto exit;
    }

    /* Conditionally ignore reported warnings. */
    if (JSREPORT_IS_WARNING(report->flags) && 
        !GlobusWSDLDebug(GLOBUS_WSDL_DEBUG_WARN))
    {
        goto exit;
    }

    prefix = NULL;
    if (report->filename)
    {
        prefix = JS_smprintf("%s:", report->filename);
    }

    if (JSREPORT_IS_WARNING(report->flags)) {
        tmp = prefix;
        prefix = JS_smprintf("%s%swarning: ",
                             tmp ? tmp : "",
                             JSREPORT_IS_STRICT(report->flags) ? "strict " : "");
        JS_free(cx, tmp);
    }

    linebuf = NULL;
    if(report->linebuf)
    {
        linebuf = JS_smprintf("%s", report->linebuf);
    }

    ctx_handle->result = globus_error_put(
        globus_error_construct_error(
            GLOBUS_WSDL_PARSER_MODULE, 
                (ctx_handle->result != GLOBUS_NULL)
                    ? globus_error_get(ctx_handle->result)
                    : NULL,
            GLOBUS_WSDL_ERROR_JAVASCRIPT,
            __FILE__,
            _globus_func_name,
            __LINE__,
            (linebuf ? "%s%s at line:\n\t%s" : "%s%s%s"), 
            (prefix ? prefix : ""), 
            (message ? message : ""),
            (linebuf ? linebuf : "")));

    if(prefix)
    {
        JS_free(cx, prefix);
    }

    if(linebuf)
    {
        JS_free(cx, linebuf);
    }

    if(JSREPORT_IS_WARNING(report->flags))
    {
        globus_object_t *               warn_obj;
        warn_obj = globus_error_get(ctx_handle->result);
         
        if(GlobusWSDLDebug(GLOBUS_WSDL_DEBUG_WARN))
        {
            char *                          warn_str;
            
            warn_str = globus_error_print_chain(warn_obj);
            
            GlobusWSDLDebugPrintf(GLOBUS_WSDL_DEBUG_WARN,
                                  ("%s", warn_str));
            
            if(warn_str)
            {
                globus_free(warn_str);
            }
        }

        if(warn_obj)
        {
            globus_object_free(warn_obj);
        }

        ctx_handle->result = GLOBUS_SUCCESS;
    }

exit:

    GlobusWSDLDebugExit();
}

static
JSString *
globus_l_wsdl_jscript_qname_string(
    JSContext *                         ctx,
    const char *                        href,
    const char *                        local)
{
    JSString *                          str;
    char *                              qname;
    size_t                              href_length = 0;
    size_t                              local_length = 0;

    href_length = href ? strlen(href) : 0;
    local_length = local ? strlen(local) : 0;

    qname = malloc(href_length + local_length + 2);
    if(href)
    {
        memcpy(qname, href, href_length);
        qname[href_length] = '#';
        href_length++;
    }

    memcpy(qname + href_length, local, local_length);
    qname[href_length + local_length] = 0;
    
    str = JS_InternString(ctx, qname);

    return str;
}    

globus_result_t
globus_wsdl_execute(
    globus_list_t *                     schema_files,
    const char *                        tmpl_filename,
    const char *                        out_filename,
    const char *                        out_dir,
    globus_hashtable_t                  nsmap,
    globus_hashtable_t                  vars,
    globus_list_t *                     template_paths,
    globus_hashtable_t                  ignorable_namespaces,
    globus_hashtable_t                  include_namespaces,
    const int                           rt_size)
{
    JSRuntime *                         rt = NULL;
    JSContext *                         ctx = NULL;
    JSObject *                          glob = NULL;
    JSObject *                          ignore = NULL;
    char *                              ignore_entry = NULL;
    JSObject *                          include = NULL;
    char *                              include_entry = NULL;
    char **                             var_entry;
    globus_result_t                     result = GLOBUS_SUCCESS;
    char *                              sfile = NULL;
    globus_list_t *                     sfiles = NULL;
    globus_list_t *                     schemas = NULL;
    wsdlSchemaPtr                       schema = NULL;
    GlobusFuncName(globus_wsdl_execute);
    GlobusWSDLDebugEnter();

    rt = JS_NewRuntime((rt_size <= 0) ? (64L * 1024L * 1024L) : rt_size);
    if(!rt)
    {
        result = GlobusWSDLErrorJSRuntime(GLOBUS_NULL);
        goto exit;
    }
    
    ctx = JS_NewContext(rt, gStackChunkSize);
    if(!ctx)
    {
        result = GlobusWSDLErrorJSContext(GLOBUS_NULL);
        goto exit;
    }

    JS_SetErrorReporter(ctx, internal_ErrorReporter);

    glob = JS_NewObject(ctx, &global_class, NULL, NULL);
    if(!glob)
    {
        result = GlobusWSDLErrorJSContext(GLOBUS_NULL);
        goto exit;
    }

    if(!JS_InitStandardClasses(ctx, glob))
    {
        result = GlobusWSDLErrorJSContext(GLOBUS_NULL);
        goto exit;
    }

    globus_i_wsdl_parser_classes_init(ctx, glob);

    if(!JS_DefineFunctions(ctx, glob, functions_defs))
    {
        result = GlobusWSDLErrorJSContext(GLOBUS_NULL);
        goto exit;
    }

    sfiles = schema_files;
    while(sfiles)
    {
        sfile = globus_list_first(sfiles);

        result = globus_i_wsdl_parse_schema(sfile, &schema);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusWSDLErrorParsingSchema(result, sfile);
            goto exit;
        }

        result = globus_list_insert(&schemas, schema);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusWSDLErrorParsingSchema(result, sfile);
            goto exit;
        }

        result = globus_i_wsdl_js_load_schema(ctx, schema);
        if(result != GLOBUS_SUCCESS)
        {
            result = GlobusWSDLErrorLoadingSchema(result);
            goto exit;
        }

        sfiles = globus_list_rest(sfiles);
    }

    var_entry = globus_hashtable_first(&vars);
    while(var_entry)
    {
        if(!JS_DefineProperty(
               ctx, glob, var_entry[0],
               STRING_TO_JSVAL(JS_InternString(ctx, var_entry[1])),
               NULL, NULL,
               JSPROP_PERMANENT))
        {
            result = GlobusWSDLErrorLoadingSchema(GLOBUS_NULL);
            goto exit;
        }

        var_entry = globus_hashtable_next(&vars);
    }

    ignore = JS_NewObject(ctx, &basic_class, NULL, glob);
    if(!ignore)
    {
        result = GlobusWSDLErrorJSContext(GLOBUS_NULL);
        goto exit;
    }

    if(!JS_DefineProperty(
           ctx, glob, "ignore_namespaces", 
           OBJECT_TO_JSVAL(ignore), NULL, 
           NULL, JSPROP_READONLY|JSPROP_PERMANENT|JSPROP_ENUMERATE))
    {
        result = GlobusWSDLErrorLoadingSchema(GLOBUS_NULL);
        goto exit;
    }

    ignore_entry = globus_hashtable_first(&ignorable_namespaces);
    while(ignore_entry)
    {
        if(!JS_DefineProperty(
               ctx, ignore, ignore_entry, 
               STRING_TO_JSVAL(JS_InternString(ctx, ignore_entry)), 
               NULL, NULL, JSPROP_PERMANENT|JSPROP_READONLY|JSPROP_ENUMERATE))
        {
            result = GlobusWSDLErrorLoadingSchema(GLOBUS_NULL);
            goto exit;
        }

        ignore_entry = globus_hashtable_next(&ignorable_namespaces);
    }

    
    if(!include_namespaces)
    {
        if(!JS_DefineProperty(
               ctx, glob, "include_namespaces",
               JSVAL_NULL, NULL,
               NULL, JSPROP_READONLY|JSPROP_PERMANENT|JSPROP_ENUMERATE))
        {
            result = GlobusWSDLErrorLoadingSchema(GLOBUS_NULL);
            goto exit;
        }
    }
    else
    {

        include = JS_NewObject(ctx, &basic_class, NULL, glob);
        if(!include)
        {
            result = GlobusWSDLErrorJSContext(GLOBUS_NULL);
            goto exit;
        }

        if(!JS_DefineProperty(
               ctx, glob, "include_namespaces", 
               OBJECT_TO_JSVAL(include), NULL, 
               NULL, JSPROP_READONLY|JSPROP_PERMANENT|JSPROP_ENUMERATE))
        {
            result = GlobusWSDLErrorLoadingSchema(GLOBUS_NULL);
            goto exit;
        }
    
        include_entry = globus_hashtable_first(&include_namespaces);
        while(include_entry)
        {
            if(!JS_DefineProperty(
                   ctx, include, include_entry, 
                   STRING_TO_JSVAL(JS_InternString(ctx, include_entry)), 
                   NULL, NULL, 
                   JSPROP_PERMANENT|JSPROP_READONLY|JSPROP_ENUMERATE))
            {
                result = GlobusWSDLErrorLoadingSchema(GLOBUS_NULL);
                goto exit;
            }

            include_entry = globus_hashtable_next(&include_namespaces);
        }
    }

    result = globus_i_wsdl_execute_script(ctx, glob, 
                                          tmpl_filename, 
                                          out_filename,
                                          out_dir,
                                          nsmap,
                                          template_paths);
    if(result != GLOBUS_SUCCESS)
    {
        result = GlobusWSDLErrorExecutingScript(result, tmpl_filename);
        goto exit;
    }

  exit:
    
    if(result != GLOBUS_SUCCESS)
    {
        if(GlobusWSDLDebug(GLOBUS_WSDL_DEBUG_WARN))
        {
            globus_object_t *           error_obj;
            char *                      error_str;

            error_obj = globus_error_get(result);
            if(error_obj)
            {
                error_str = globus_error_print_chain(
                    error_obj);
            
                GlobusWSDLDebugPrintf(GLOBUS_WSDL_DEBUG_WARN, 
                                      ("[DEBUG::WSDL] %s:%d: %s", 
                                       _globus_func_name, 
                                       __LINE__, 
                                       error_str));
                globus_free(error_str);
                
                result = globus_error_put(error_obj);
            }
        }
    }

    if(schemas)
    {
        globus_list_destroy_all(schemas, (void(*)(void *))wsdlSchemaFree);
    }
    
    if(ctx)
    {
        JS_DestroyContext(ctx);
    }
    
    if(rt)
    {
        JS_DestroyRuntime(rt);
    }
    
    GlobusWSDLDebugExit();
    return result;
}

static
globus_result_t
globus_i_wsdl_js_load_schema(
    JSContext *                         cx,
    wsdlSchemaPtr                       schema)
{
    JSObject *                          glob = NULL;
    JSObject *                          wsdl = NULL;
    JSObject *                          types = NULL;
    JSObject *                          elements = NULL;
    JSObject *                          local_elements = NULL;
    JSObject *                          attributes = NULL;
    JSObject *                          local_attributes = NULL;
    JSObject *                          messages = NULL;
    JSObject *                          portTypes = NULL;
    JSObject *                          bindings = NULL;
    JSObject *                          services = NULL;
    JSObject *                          typeEnum = NULL;
    JSObject *                          attrEnum = NULL;
    JSObject *                          contentTypeEnum = NULL;
    JSObject *                          operationTypeEnum = NULL;
    JSObject *                          bindingType = NULL;
    JSObject *                          soapOperationStyleType = NULL;
    JSObject *                          soapUseType = NULL;
    ScannerData *                       data = NULL;
    jsval                               val;
    JSString *                          str;
    JSBool                              res = JS_TRUE;
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_i_wsdl_js_load_schema);
    GlobusWSDLDebugEnter();

    glob = JS_GetGlobalObject(cx);
    if(!glob)
    {
        result = GlobusWSDLErrorJSLoadSchema(GLOBUS_NULL);
        goto exit;
    }

    res = JS_GetProperty(cx, glob, "wsdl", &val);
    if(val == JSVAL_VOID)
    {
        wsdl = JS_NewObject(cx, &basic_class, NULL, glob);
        if(!wsdl)
        {
            result = GlobusWSDLErrorJSLoadSchema(GLOBUS_NULL);
            goto exit;
        }

        val = OBJECT_TO_JSVAL(wsdl);
        if(!JS_DefineProperty(cx, glob, "wsdl", val, NULL, NULL,
                              JSPROP_READONLY|JSPROP_PERMANENT))
        {
            result = GlobusWSDLErrorJSLoadSchema(GLOBUS_NULL);
            goto exit;
        }
    }
    else
    {
        wsdl = JSVAL_TO_OBJECT(val);
        if(!wsdl)
        {
            result = GlobusWSDLErrorJSLoadSchema(GLOBUS_NULL);
            goto exit;
        }
    }
    
    res = JS_GetProperty(cx, wsdl, "types", &val);
    if(val == JSVAL_VOID)
    {
        types = JS_NewObject(cx, &basic_class, NULL, wsdl);
        if(!types)
        {
            result = GlobusWSDLErrorJSLoadSchema(GLOBUS_NULL);
            goto exit;
        }

        val = OBJECT_TO_JSVAL(types);
        if(!JS_DefineProperty(cx, wsdl, "types", val, NULL, NULL,
                              JSPROP_READONLY|JSPROP_PERMANENT))
        {
            result = GlobusWSDLErrorJSLoadSchema(GLOBUS_NULL);
            goto exit;
        }
    }
    else
    {
        types = JSVAL_TO_OBJECT(val);
        if(!types)
        {
            result = GlobusWSDLErrorJSLoadSchema(GLOBUS_NULL);
            goto exit;
        }
    }

    res = JS_GetProperty(cx, wsdl, "elements", &val);
    if(val == JSVAL_VOID)
    {
        elements = JS_NewObject(cx, &basic_class, NULL, wsdl);
        if(!elements)
        {
            result = GlobusWSDLErrorJSLoadSchema(GLOBUS_NULL);
            goto exit;
        }

        val = OBJECT_TO_JSVAL(elements);
        if(!JS_DefineProperty(cx, wsdl, "elements", val, NULL, NULL,
                              JSPROP_READONLY|JSPROP_PERMANENT))
        {
            result = GlobusWSDLErrorJSLoadSchema(GLOBUS_NULL);
            goto exit;
        }
    }
    else
    {
        elements = JSVAL_TO_OBJECT(val);
        if(!elements)
        {
            result = GlobusWSDLErrorJSLoadSchema(GLOBUS_NULL);
            goto exit;
        }
    }

    res = JS_GetProperty(cx, wsdl, "local_elements", &val);
    if(val == JSVAL_VOID)
    {
        local_elements = JS_NewObject(cx, &basic_class, NULL, wsdl);
        if(!local_elements)
        {
            result = GlobusWSDLErrorJSLoadSchema(GLOBUS_NULL);
            goto exit;
        }

        val = OBJECT_TO_JSVAL(local_elements);
        if(!JS_DefineProperty(cx, wsdl, "local_elements", val, NULL, NULL,
                              JSPROP_READONLY|JSPROP_PERMANENT))
        {
            result = GlobusWSDLErrorJSLoadSchema(GLOBUS_NULL);
            goto exit;
        }
    }
    else
    {
        local_elements = JSVAL_TO_OBJECT(val);
        if(!local_elements)
        {
            result = GlobusWSDLErrorJSLoadSchema(GLOBUS_NULL);
            goto exit;
        }
    }

    res = JS_GetProperty(cx, wsdl, "attributes", &val);
    if(val == JSVAL_VOID)
    {
        attributes = JS_NewObject(cx, &basic_class, NULL, wsdl);
        if(!attributes)
        {
            result = GlobusWSDLErrorJSLoadSchema(GLOBUS_NULL);
            goto exit;
        }

        val = OBJECT_TO_JSVAL(attributes);
        if(!JS_DefineProperty(cx, wsdl, "attributes", val, NULL, NULL,
                              JSPROP_READONLY|JSPROP_PERMANENT))
        {
            result = GlobusWSDLErrorJSLoadSchema(GLOBUS_NULL);
            goto exit;
        }
    }
    else
    {
        attributes = JSVAL_TO_OBJECT(val);
        if(!attributes)
        {
            result = GlobusWSDLErrorJSLoadSchema(GLOBUS_NULL);
            goto exit;
        }
    }

    res = JS_GetProperty(cx, wsdl, "local_attributes", &val);
    if(val == JSVAL_VOID)
    {
        local_attributes = JS_NewObject(cx, &basic_class, NULL, wsdl);
        if(!local_attributes)
        {
            result = GlobusWSDLErrorJSLoadSchema(GLOBUS_NULL);
            goto exit;
        }

        val = OBJECT_TO_JSVAL(local_attributes);
        if(!JS_DefineProperty(cx, wsdl, "local_attributes", val, NULL, NULL,
                              JSPROP_READONLY|JSPROP_PERMANENT))
        {
            result = GlobusWSDLErrorJSLoadSchema(GLOBUS_NULL);
            goto exit;
        }
    }
    else
    {
        local_attributes = JSVAL_TO_OBJECT(val);
        if(!local_attributes)
        {
            result = GlobusWSDLErrorJSLoadSchema(GLOBUS_NULL);
            goto exit;
        }
    }

    res = JS_GetProperty(cx, wsdl, "messages", &val);
    if(val == JSVAL_VOID)
    {
        messages = JS_NewObject(cx, &basic_class, NULL, wsdl);
        if(!messages)
        {
            result = GlobusWSDLErrorJSLoadSchema(GLOBUS_NULL);
            goto exit;
        }

        val = OBJECT_TO_JSVAL(messages);
        if(!JS_DefineProperty(cx, wsdl, "messages", val, NULL, NULL,
                              JSPROP_READONLY|JSPROP_PERMANENT))
        {
            result = GlobusWSDLErrorJSLoadSchema(GLOBUS_NULL);
            goto exit;
        }
    }
    else
    {
        messages = JSVAL_TO_OBJECT(val);
        if(!messages)
        {
            result = GlobusWSDLErrorJSLoadSchema(GLOBUS_NULL);
            goto exit;
        }
    }

    res = JS_GetProperty(cx, wsdl, "portTypes", &val);
    if(val == JSVAL_VOID)
    {
        portTypes = JS_NewObject(cx, &basic_class, NULL, wsdl);
        if(!portTypes)
        {
            result = GlobusWSDLErrorJSLoadSchema(GLOBUS_NULL);
            goto exit;
        }

        val = OBJECT_TO_JSVAL(portTypes);
        if(!JS_DefineProperty(cx, wsdl, "portTypes", val, NULL, NULL,
                              JSPROP_READONLY|JSPROP_PERMANENT))
        {
            result = GlobusWSDLErrorJSLoadSchema(GLOBUS_NULL);
            goto exit;
        }
    }
    else
    {
        portTypes = JSVAL_TO_OBJECT(val);
        if(!portTypes)
        {
            result = GlobusWSDLErrorJSLoadSchema(GLOBUS_NULL);
            goto exit;
        }
    }

    res = JS_GetProperty(cx, wsdl, "bindings", &val);
    if(val == JSVAL_VOID)
    {
        bindings = JS_NewObject(cx, &basic_class, NULL, wsdl);
        if(!bindings)
        {
            result = GlobusWSDLErrorJSLoadSchema(GLOBUS_NULL);
            goto exit;
        }

        val = OBJECT_TO_JSVAL(bindings);
        if(!JS_DefineProperty(cx, wsdl, "bindings", val, NULL, NULL,
                              JSPROP_READONLY|JSPROP_PERMANENT))
        {
            result = GlobusWSDLErrorJSLoadSchema(GLOBUS_NULL);
            goto exit;
        }
    }
    else
    {
        bindings = JSVAL_TO_OBJECT(val);
        if(!bindings)
        {
            result = GlobusWSDLErrorJSLoadSchema(GLOBUS_NULL);
            goto exit;
        }
    }

    res = JS_GetProperty(cx, wsdl, "services", &val);
    if(val == JSVAL_VOID)
    {
        services = JS_NewObject(cx, &basic_class, NULL, wsdl);
        if(!services)
        {
            result = GlobusWSDLErrorJSLoadSchema(GLOBUS_NULL);
            goto exit;
        }

        val = OBJECT_TO_JSVAL(services);
        if(!JS_DefineProperty(cx, wsdl, "services", val, NULL, NULL,
                              JSPROP_READONLY|JSPROP_PERMANENT))
        {
            result = GlobusWSDLErrorJSLoadSchema(GLOBUS_NULL);
            goto exit;
        }
    }
    else
    {
        services = JSVAL_TO_OBJECT(val);
        if(!services)
        {
            result = GlobusWSDLErrorJSLoadSchema(GLOBUS_NULL);
            goto exit;
        }
    }

    data = (ScannerData *) malloc(sizeof(ScannerData));
    data->schema = schema;
    data->context = cx;
    data->global = glob;
    data->types = (void *)types;
    data->elements = (void *)elements;
    data->local_elements = (void *)local_elements;
    data->attributes = (void *)attributes;
    data->local_attributes = (void *)local_attributes;
    data->messages = (void *)messages;
    data->attributes = (void *)attributes;
    data->portTypes = (void *)portTypes;
    data->bindings = (void *)bindings;
    data->services = (void *)services;
    data->result = GLOBUS_SUCCESS;

    if(schema->schemaTypes)
    {
        data->xsdSchema = schema->schemaTypes;
        xmlHashScan(schema->schemaTypes->schemasImports, 
                    globus_i_wsdl_parser_import_scanner, data);
        if(data->result != GLOBUS_SUCCESS)
        {
            result = GlobusWSDLErrorJSLoadSchema(data->result);
            goto exit;
        }
        
        data->xsdSchema = schema->schemaTypes;
        xmlHashScan(schema->schemaTypes->typeDecl, 
                    globus_i_wsdl_parser_type_scanner, data);
        if(data->result != GLOBUS_SUCCESS)
        {
            result = GlobusWSDLErrorJSLoadSchema(data->result);
            goto exit;
        }
        
        xmlHashScan(schema->schemaTypes->groupDecl,
                    globus_i_wsdl_parser_type_scanner, data);
        if(data->result != GLOBUS_SUCCESS)
        {
            result = GlobusWSDLErrorJSLoadSchema(data->result);
            goto exit;
        }

        xmlHashScan(schema->schemaTypes->elemDecl, 
                    globus_i_wsdl_parser_element_scanner, data);
        if(data->result != GLOBUS_SUCCESS)
        {
            result = GlobusWSDLErrorJSLoadSchema(data->result);
            goto exit;
        }
        
        xmlHashScan(schema->schemaTypes->attrDecl, 
                    globus_i_wsdl_parser_attribute_scanner, 
                    data);
        if(data->result != GLOBUS_SUCCESS)
        {
            result = GlobusWSDLErrorJSLoadSchema(data->result);
            goto exit;
        }
    }

    typeEnum = JS_NewObject(cx, &xsdEnum_class, NULL, wsdl);
    if(!typeEnum)
    {
        result = GlobusWSDLErrorJSLoadSchema(GLOBUS_NULL);
        goto exit;
    }

    if(!JS_DefineProperties(cx, typeEnum, xsdTypeEnum_properties))
    {
        result = GlobusWSDLErrorJSLoadSchema(GLOBUS_NULL);
        goto exit;
    }

    val = OBJECT_TO_JSVAL(typeEnum);
    if(!JS_DefineProperty(cx, wsdl, "xsdType", val, NULL, NULL,
                          JSPROP_READONLY|JSPROP_PERMANENT))
    {
        result = GlobusWSDLErrorJSLoadSchema(GLOBUS_NULL);
        goto exit;
    }

    attrEnum = JS_NewObject(cx, &xsdEnum_class, NULL, wsdl);
    if(!attrEnum)
    {
        result = GlobusWSDLErrorJSLoadSchema(GLOBUS_NULL);
        goto exit;
    }

    if(!JS_DefineProperties(cx, attrEnum, xsdAttrEnum_properties))
    {
        result = GlobusWSDLErrorJSLoadSchema(GLOBUS_NULL);
        goto exit;
    }

    val = OBJECT_TO_JSVAL(attrEnum);
    if(!JS_DefineProperty(cx, wsdl, "xsdAttr", val, NULL, NULL,
                          JSPROP_READONLY|JSPROP_PERMANENT))
    {
        result = GlobusWSDLErrorJSLoadSchema(GLOBUS_NULL);
        goto exit;
    }

    contentTypeEnum = JS_NewObject(cx, &xsdEnum_class, NULL, wsdl);
    if(!contentTypeEnum)
    {
        result = GlobusWSDLErrorJSLoadSchema(GLOBUS_NULL);
        goto exit;
    }

    if(!JS_DefineProperties(cx, contentTypeEnum, 
                            xsdContentTypeEnum_properties))
    {
        result = GlobusWSDLErrorJSLoadSchema(GLOBUS_NULL);
        goto exit;
    }

    val = OBJECT_TO_JSVAL(contentTypeEnum);
    if(!JS_SetProperty(cx, wsdl, "xsdContentType", &val))
    {
        result = GlobusWSDLErrorJSLoadSchema(GLOBUS_NULL);
        goto exit;
    }

    operationTypeEnum = JS_NewObject(cx, &xsdEnum_class, NULL, wsdl);
    JS_DefineProperties(cx, operationTypeEnum, operationTypeEnum_properties);
    val = OBJECT_TO_JSVAL(operationTypeEnum);
    JS_SetProperty(cx, wsdl, "operationType", &val);

    bindingType = JS_NewObject(cx, &xsdEnum_class, NULL, wsdl);
    JS_DefineProperties(cx, bindingType, bindingType_properties);
    val = OBJECT_TO_JSVAL(bindingType);
    JS_SetProperty(cx, wsdl, "bindingType", &val);

    soapOperationStyleType = JS_NewObject(cx, &xsdEnum_class, NULL, wsdl);
    JS_DefineProperties(cx, soapOperationStyleType, 
                        soapOperationStyleType_properties);
    val = OBJECT_TO_JSVAL(soapOperationStyleType);
    JS_SetProperty(cx, wsdl, "soapOperationStyleType", &val);

    soapUseType = JS_NewObject(cx, &xsdEnum_class, NULL, wsdl);
    JS_DefineProperties(cx, soapUseType, 
                        soapUseType_properties);
    val = OBJECT_TO_JSVAL(soapUseType);
    JS_SetProperty(cx, wsdl, "soapUseType", &val);

    xmlHashScan(schema->messagesDecl, 
                globus_i_wsdl_parser_message_scanner, 
                data);
    if(data->result != GLOBUS_SUCCESS)
    {
        result = GlobusWSDLErrorJSLoadSchema(data->result);
        goto exit;
    }

    xmlHashScan(schema->portTypesDecl, 
                globus_i_wsdl_parser_portType_scanner, data);
    if(data->result != GLOBUS_SUCCESS)
    {
        result = GlobusWSDLErrorJSLoadSchema(data->result);
        goto exit;
    }

    xmlHashScan(schema->bindingsDecl, 
                globus_i_wsdl_parser_binding_scanner, 
                data);
    if(data->result != GLOBUS_SUCCESS)
    {
        result = GlobusWSDLErrorJSLoadSchema(data->result);
        goto exit;
    }

    xmlHashScan(schema->servicesDecl, 
                globus_i_wsdl_parser_service_scanner, 
                data);
    if(data->result != GLOBUS_SUCCESS)
    {
        result = GlobusWSDLErrorJSLoadSchema(data->result);
        goto exit;
    }

    xmlHashScan(schema->importsDecl, 
                globus_i_wsdl_parser_wsdl_imports_scanner, 
                data);
    if(data->result != GLOBUS_SUCCESS)
    {
        result = GlobusWSDLErrorJSLoadSchema(data->result);
        goto exit;
    }

    if(!schema->targetNamespace)
    {
        val = JSVAL_NULL;
    }
    else
    {
        str = JS_InternString(cx, schema->targetNamespace);
        if(!str)
        {
            result = GlobusWSDLErrorJSLoadSchema(GLOBUS_NULL);
            goto exit;
        }
        val = STRING_TO_JSVAL(str);
    }

    if(!JS_SetProperty(cx, wsdl, "targetNamespace", &val))
    {
        result = GlobusWSDLErrorJSLoadSchema(GLOBUS_NULL);
        goto exit;
    }

  exit:

    GlobusWSDLDebugExit(); 
    return result;
}

static
globus_result_t
globus_i_wsdl_js_load_imported_schema(
    JSContext *                         cx,
    wsdlSchemaPtr                       schema)
{
    JSObject *                          glob = NULL;
    JSObject *                          wsdl = NULL;
    JSObject *                          types = NULL;
    JSObject *                          elements = NULL;
    JSObject *                          local_elements = NULL;
    JSObject *                          attributes = NULL;    
    JSObject *                          local_attributes = NULL;
    JSObject *                          messages = NULL;
    JSObject *                          portTypes = NULL;
    JSObject *                          bindings = NULL;
    JSObject *                          services = NULL;
    ScannerData *                       data = NULL;
    jsval                               val;
    JSBool                              res = JS_TRUE;
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_i_wsdl_js_load_imported_schema);
    GlobusWSDLDebugEnter();

    glob = JS_GetGlobalObject(cx);
    if(!glob)
    {
        result = GlobusWSDLErrorJSLoadSchema(GLOBUS_NULL);
        goto exit;
    }

    res = JS_GetProperty(cx, glob, "wsdl", &val);
    globus_assert(res);

    wsdl = JSVAL_TO_OBJECT(val);
    if(!wsdl)
    {
        result = GlobusWSDLErrorJSLoadSchema(GLOBUS_NULL);
        goto exit;
    }

    res = JS_GetProperty(cx, wsdl, "types", &val);
    globus_assert(res);
    
    types = JSVAL_TO_OBJECT(val);
    if(!types)
    {
        result = GlobusWSDLErrorJSLoadSchema(GLOBUS_NULL);
        goto exit;
    }

    res = JS_GetProperty(cx, wsdl, "elements", &val);
    globus_assert(res);

    elements = JSVAL_TO_OBJECT(val);
    if(!elements)
    {
        result = GlobusWSDLErrorJSLoadSchema(GLOBUS_NULL);
        goto exit;
    }

    res = JS_GetProperty(cx, wsdl, "local_elements", &val);
    globus_assert(res);

    local_elements = JSVAL_TO_OBJECT(val);
    if(!local_elements)
    {
        result = GlobusWSDLErrorJSLoadSchema(GLOBUS_NULL);
        goto exit;
    }

    res = JS_GetProperty(cx, wsdl, "attributes", &val);
    globus_assert(res);

    attributes = JSVAL_TO_OBJECT(val);
    if(!attributes)
    {
        result = GlobusWSDLErrorJSLoadSchema(GLOBUS_NULL);
        goto exit;
    }

    res = JS_GetProperty(cx, wsdl, "local_attributes", &val);
    globus_assert(res);

    local_attributes = JSVAL_TO_OBJECT(val);
    if(!local_attributes)
    {
        result = GlobusWSDLErrorJSLoadSchema(GLOBUS_NULL);
        goto exit;
    }

    res = JS_GetProperty(cx, wsdl, "messages", &val);
    globus_assert(res);

    messages = JSVAL_TO_OBJECT(val);
    if(!messages)
    {
        result = GlobusWSDLErrorJSLoadSchema(GLOBUS_NULL);
        goto exit;
    }

    res = JS_GetProperty(cx, wsdl, "portTypes", &val);
    globus_assert(res);

    portTypes = JSVAL_TO_OBJECT(val);
    if(!portTypes)
    {
        result = GlobusWSDLErrorJSLoadSchema(GLOBUS_NULL);
        goto exit;
    }

    res = JS_GetProperty(cx, wsdl, "bindings", &val);
    globus_assert(res);

    bindings = JSVAL_TO_OBJECT(val);
    if(!bindings)
    {
        result = GlobusWSDLErrorJSLoadSchema(GLOBUS_NULL);
        goto exit;
    }

    res = JS_GetProperty(cx, wsdl, "services", &val);
    globus_assert(res);

    services = JSVAL_TO_OBJECT(val);
    if(!services)
    {
        result = GlobusWSDLErrorJSLoadSchema(GLOBUS_NULL);
        goto exit;
    }

    data = (ScannerData *) malloc(sizeof(ScannerData));
    data->schema = schema;
    data->context = cx;
    data->global = glob;
    data->types = (void *)types;
    data->elements = (void *)elements;
    data->attributes = (void *)attributes;
    data->local_elements = (void *)local_elements;
    data->local_attributes = (void *)local_attributes;
    data->messages = (void *)messages;
    data->portTypes = (void *)portTypes;
    data->bindings = (void *)bindings;
    data->services = (void *)services;
    data->result = GLOBUS_SUCCESS;

    if(schema->schemaTypes)
    {
        data->xsdSchema = schema->schemaTypes;
        xmlHashScan(schema->schemaTypes->schemasImports, 
                    globus_i_wsdl_parser_import_scanner, 
                    data);
        if(data->result != GLOBUS_SUCCESS)
        {
            result = GlobusWSDLErrorJSLoadSchema(data->result);
            goto exit;
        }
        
        data->xsdSchema = schema->schemaTypes;
        xmlHashScan(schema->schemaTypes->typeDecl, 
                    globus_i_wsdl_parser_type_scanner, data);
        if(data->result != GLOBUS_SUCCESS)
        {
            result = GlobusWSDLErrorJSLoadSchema(data->result);
            goto exit;
        }

        xmlHashScan(schema->schemaTypes->groupDecl, 
                    globus_i_wsdl_parser_type_scanner, data);
        if(data->result != GLOBUS_SUCCESS)
        {
            result = GlobusWSDLErrorJSLoadSchema(data->result);
            goto exit;
        }
        
        xmlHashScan(schema->schemaTypes->elemDecl, 
                    globus_i_wsdl_parser_element_scanner, data);
        if(data->result != GLOBUS_SUCCESS)
        {
            result = GlobusWSDLErrorJSLoadSchema(data->result);
            goto exit;
        }
        
        xmlHashScan(schema->schemaTypes->attrDecl, 
                    globus_i_wsdl_parser_attribute_scanner, 
                    data);
        if(data->result != GLOBUS_SUCCESS)
        {
            result = GlobusWSDLErrorJSLoadSchema(data->result);
            goto exit;
        }
    }

    xmlHashScan(schema->messagesDecl, 
                globus_i_wsdl_parser_message_scanner, data);
    if(data->result != GLOBUS_SUCCESS)
    {
        result = GlobusWSDLErrorJSLoadSchema(data->result);
        goto exit;
    }
    
    xmlHashScan(schema->portTypesDecl, 
                globus_i_wsdl_parser_portType_scanner, data);
    if(data->result != GLOBUS_SUCCESS)
    {
        result = GlobusWSDLErrorJSLoadSchema(data->result);
        goto exit;
    }

    xmlHashScan(schema->bindingsDecl, 
                globus_i_wsdl_parser_binding_scanner, data);
    if(data->result != GLOBUS_SUCCESS)
    {
        result = GlobusWSDLErrorJSLoadSchema(data->result);
        goto exit;
    }

    xmlHashScan(schema->servicesDecl, 
                globus_i_wsdl_parser_service_scanner, data);
    if(data->result != GLOBUS_SUCCESS)
    {
        result = GlobusWSDLErrorJSLoadSchema(data->result);
        goto exit;
    }

    xmlHashScan(schema->importsDecl, 
                globus_i_wsdl_parser_wsdl_imports_scanner, data);

  exit:

    GlobusWSDLDebugExit(); 
    return result;
}

static 
globus_result_t
globus_i_wsdl_execute_script(
    JSContext *                         ctx,
    JSObject *                          glob,
    const char *                        tmpl_filename,
    const char *                        out_filename,
    const char *                        out_dir,
    globus_hashtable_t                  nsmap,
    globus_list_t *                     template_paths)
{
    PrivateContext *                    data;
    JSString *                          argString;
    jsval                               resval;
    jsval                               load_args[2];
    globus_result_t                     result = GLOBUS_SUCCESS;
    int                                 argcount = 0;
    
    GlobusFuncName(globus_i_wsdl_execute_script);
    GlobusWSDLDebugEnter();
    
    data = (PrivateContext *) globus_malloc(sizeof(PrivateContext));
    if(!data)
    {
        result = GlobusWSDLErrorOutOfMemory;
        goto exit;
    }
    memset(data, 0, sizeof(PrivateContext));
    if(out_dir)
    {
        data->outdir = globus_libc_strdup(out_dir);
        if(mkdir(data->outdir, S_IRWXU|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH))
        {
            if(errno != EEXIST)
            {
                result = GlobusWSDLErrorDirCreate(data->outdir);
                goto exit;
            }
        }
    }

    data->streams = (StreamContext *) malloc(sizeof(StreamContext));
    globus_assert_string(data->streams, "Memory allocation failed.");
    memset(data->streams, 0, sizeof(StreamContext));

    if(!nsmap)
    {
        globus_hashtable_init(&data->nsmap, 10, 
                              globus_hashtable_string_hash,
                              globus_hashtable_string_keyeq);
        data->free_nsmap = 0;
    }
    else
    {
        data->nsmap = nsmap;
        data->free_nsmap = 0;
    }

    data->template_paths = template_paths;
    JS_SetContextPrivate(ctx, (void *)data);

    memset(load_args, 0, 2);

    argString = JS_InternString(ctx, tmpl_filename);
    load_args[0] = STRING_TO_JSVAL(argString);
    argcount++;

    if(out_filename)
    {
        argString = JS_InternString(ctx, out_filename);
        load_args[1] = STRING_TO_JSVAL(argString);
        argcount++;
    }
    else
    {
        load_args[1] = JSVAL_NULL;
        argcount++;
    }

    if(JS_CallFunctionName(
           ctx, glob, "load", argcount, load_args, &resval) != JS_TRUE)
    {
        result = GlobusWSDLErrorTemplateLoad(data->result, tmpl_filename);
        data->result = result;
        goto exit;
    }

  exit:

    data = (PrivateContext *) JS_GetContextPrivate(ctx);
    if(data)
    {
        if(data->streams)
        {
            free(data->streams);
        }

        if(data->nsmap && data->free_nsmap)
        {
            globus_wsdl_config_properties_destroy(data->nsmap);
        }

        if(data->templates)
        {
            globus_hashtable_destroy_all(&data->templates, TemplateFree);
        }

        if(data->outdir)
        {
            free(data->outdir);
        }

        free(data);
    }
    
    GlobusWSDLDebugExit();
    return result;
}

static
JSBool
xsdEnumGetter(
    JSContext *                         cx,
    JSObject *                          obj,
    jsval                               id,
    jsval *                             vp)
{
    jsval                               res = JS_TRUE;
    GlobusFuncName(xsdEnumGetter);
    GlobusWSDLDebugEnter();
    
    if(JSVAL_IS_INT(id))
    {
        *vp = id;
    }
    else
    {
        res = JS_FALSE;
    }

    GlobusWSDLDebugExit();
    return res;
}
    
static
JSBool
xsdAttrsGetter(
    JSContext *                         cx,
    JSObject *                          obj,
    jsval                               id,
    jsval *                             vp)
{
    xmlSchemaAttributePtr               attr;
    JSBool                              res = JS_TRUE;
    JSObject *                          attrObj;
    int                                 i = 0;
    GlobusFuncName(xsdAttrsGetter);
    GlobusWSDLDebugEnter();
    
    attr = (xmlSchemaAttributePtr) JS_GetPrivate(cx, obj);
    globus_assert(attr);

    if(JSVAL_IS_STRING(id))
    {
        while(attr)
        {
            JSString *                  name;
            
            name = JS_InternString(cx, attr->name);
            if(JS_CompareStrings(name, JSVAL_TO_STRING(id)))
            {
                break;
            }

            attr = attr->next;
        }
    }
    else if(JSVAL_IS_INT(id))
    {
        for(; i < JSVAL_TO_INT(id) && attr; ++i)
        {
            attr = attr->next;
        }
        globus_assert(attr);
    }
    else
    {
        attr = (xmlSchemaAttributePtr) JSVAL_TO_PRIVATE(id);
        globus_assert(attr);
    }

    attrObj = JS_NewObject(cx, &xsdAttr_class, xsdAttrClass, obj);
    globus_assert(attrObj);
    JS_SetPrivate(cx, attrObj, (void *)attr);
    JS_DefineProperties(cx, attrObj, xsdAttr_properties);

    *vp = OBJECT_TO_JSVAL(attrObj);

    GlobusWSDLDebugExit();
    return res;
}

static
JSBool
xsdFacetsGetter(
    JSContext *                         cx,
    JSObject *                          obj,
    jsval                               id,
    jsval *                             vp)
{
    xmlSchemaFacetPtr                   facet;
    JSObject *                          facetObj;
    int                                 i = 0;
    JSBool                              res = JS_TRUE;
    GlobusFuncName(xsdFacetsGetter);
    GlobusWSDLDebugEnter();

    facet = (xmlSchemaFacetPtr) JS_GetPrivate(cx, obj);
    globus_assert(facet);

    if(JSVAL_IS_STRING(id))
    {
        *vp = JSVAL_ZERO;
    }
    else if(JSVAL_IS_INT(id))
    {
        for(; i < JSVAL_TO_INT(id) && facet; ++i)
        {
            facet = facet->next;
        }
        globus_assert(facet);
    }
    else
    {
        facet = (xmlSchemaFacetPtr) JSVAL_TO_PRIVATE(id);
        globus_assert(facet);
    }

    facetObj = JS_NewObject(cx, &xsdFacet_class, xsdFacetClass, obj);
    globus_assert(facetObj);
    JS_SetPrivate(cx, facetObj, (void *)facet);
    JS_DefineProperties(cx, facetObj, xsdFacet_properties);
    *vp = OBJECT_TO_JSVAL(facetObj);

    GlobusWSDLDebugExit();
    return res;
}

static
JSBool
xsdFacetsEnumerate(
    JSContext *                         cx,
    JSObject *                          obj,
    JSIterateOp                         enum_op,
    jsval *                             statep,
    jsid *                              idp)
{
    JSBool                              res = JS_TRUE;
    xmlSchemaFacetPtr                   facet;
    int                                 index;
    int                                 i = 0;
    GlobusFuncName(xsdFacetsEnumerate);
    GlobusWSDLDebugEnter();

    facet = (xmlSchemaFacetPtr) JS_GetPrivate(cx, obj);
    globus_assert(facet);
    
    switch(enum_op)
    {
    case JSENUMERATE_INIT:
        
        *statep = INT_TO_JSVAL(0);
        break;

    case JSENUMERATE_NEXT:

        index = JSVAL_TO_INT(*statep);
        
        for(; i < index && facet; ++i)
        {
            facet = facet->next;
        }

        if(!facet)
        {
            *statep = JSVAL_NULL;
            *idp = JSVAL_ZERO;
            break;
        }

        *idp = *statep;
        *statep = INT_TO_JSVAL(index + 1);

        break;

    case JSENUMERATE_DESTROY:
            
        break;
    }

    GlobusWSDLDebugExit();
    return res;
}

static
JSBool
xsdSubTypesGetter(
    JSContext *                         cx,
    JSObject *                          obj,
    jsval                               id,
    jsval *                             vp)
{
    xmlSchemaTypePtr                    type;
    JSObject *                          typeObj;
    int                                 i = 0;
    JSBool                              res = JS_TRUE;
    GlobusFuncName(xsdSubTypesGetter);
    GlobusWSDLDebugEnter();

    type = (xmlSchemaTypePtr) JS_GetPrivate(cx, obj);
    globus_assert(type);

    if(JSVAL_IS_STRING(id))
    {
        while(type)
        {
            JSString *                  name;
            
            name = JS_InternString(cx, type->name);
            if(JS_CompareStrings(name, JSVAL_TO_STRING(id)))
            {
                break;
            }

            type = type->next;
        }
    }
    else if(JSVAL_IS_INT(id))
    {
        for(; i < JSVAL_TO_INT(id) && type; ++i)
        {
            type = type->next;
        }
        globus_assert(type);
    }
    else
    {
        type = (xmlSchemaTypePtr) JSVAL_TO_PRIVATE(id);
        globus_assert(type);
    }

    typeObj = JS_NewObject(cx, &xsdType_class, xsdTypeClass, obj);
    globus_assert(typeObj);
    JS_SetPrivate(cx, typeObj, (void *)type);
    JS_DefineProperties(cx, typeObj, xsdType_properties);
    *vp = OBJECT_TO_JSVAL(typeObj);

    GlobusWSDLDebugExit();
    return res;
}

static
JSBool
xsdSubTypesEnumerate(
    JSContext *                         cx,
    JSObject *                          obj,
    JSIterateOp                         enum_op,
    jsval *                             statep,
    jsid *                              idp)
{
    JSBool                              res = JS_TRUE;
    xmlSchemaTypePtr                    schemaType;
    int                                 index;
    int                                 i = 0;
    GlobusFuncName(xsdSubTypesEnumerate);
    GlobusWSDLDebugEnter();

    schemaType = (xmlSchemaTypePtr) JS_GetPrivate(cx, obj);
    globus_assert(schemaType);
    
    switch(enum_op)
    {
    case JSENUMERATE_INIT:
        
        *statep = INT_TO_JSVAL(i);
        if(idp)
        {
            *idp = JSVAL_ZERO;
        }
        break;

    case JSENUMERATE_NEXT:

        index = JSVAL_TO_INT(*statep);
        for(; i < index && schemaType; ++i)
        {
            schemaType = schemaType->next;
        }

        if(!schemaType)
        {
            *statep = JSVAL_NULL;
            *idp = JSVAL_ZERO;
            break;
        }

        *idp = *statep;
        *statep = INT_TO_JSVAL(index + 1);

        break;

    case JSENUMERATE_DESTROY:
            
        break;
    }

    GlobusWSDLDebugExit();
    return res;
}

static
JSBool
xsdAttrsEnumerate(
    JSContext *                         cx,
    JSObject *                          obj,
    JSIterateOp                         enum_op,
    jsval *                             statep,
    jsid *                              idp)
{
    JSBool                              res = JS_TRUE;
    xmlSchemaAttributePtr               schemaAttribute;
    int                                 i = 0;
    int                                 index;
    GlobusFuncName(xsdAttrsEnumerate);
    GlobusWSDLDebugEnter();

    schemaAttribute = (xmlSchemaAttributePtr) JS_GetPrivate(cx, obj);
    globus_assert(schemaAttribute);
    
    switch(enum_op)
    {
    case JSENUMERATE_INIT:
        
        *statep = INT_TO_JSVAL(0);
        break;

    case JSENUMERATE_NEXT:

        index = JSVAL_TO_INT(*statep);
        for(; i < index && schemaAttribute; ++i)
        {
            schemaAttribute = schemaAttribute->next;
        }

        if(!schemaAttribute)
        {
            *statep = JSVAL_NULL;
            *idp = JSVAL_ZERO;
            break;
        }

        *idp = *statep;
        *statep = INT_TO_JSVAL(index + 1);

        break;

    case JSENUMERATE_DESTROY:
            
        break;
    }
    
    GlobusWSDLDebugExit();
    return res;
}

static
JSBool
xsdFacetGetter(
    JSContext *                         cx,
    JSObject *                          obj,
    jsval                               jid,
    jsval *                             vp)
{
    int                                 idval;
    JSString *                          str;
    xmlSchemaFacetPtr                   facet;
    JSBool                              res = JS_TRUE;
    GlobusFuncName(xsdFacetGetter);
    GlobusWSDLDebugEnter();

    facet = (xmlSchemaFacetPtr) JS_GetPrivate(cx, obj);
    globus_assert(facet);

    if(!JSVAL_IS_INT(jid))
    {
        char *                          prop_name;

        prop_name = JS_GetStringBytes(JSVAL_TO_STRING(jid));
        JS_ReportError(cx, 
                       "Unknown property: .%s for facet", prop_name);
        res = JS_FALSE;
        goto exit;
    }
        
    idval = JSVAL_TO_INT(jid);
    switch(idval)
    {
    case type:

        *vp = INT_TO_JSVAL(facet->type);
        break;

    case value:

        if(!facet->value)
        {
            *vp = JSVAL_VOID;
            break;
        }

        str = JS_InternString(cx, facet->value);
        *vp = STRING_TO_JSVAL(str);
        break;

    case id:

        *vp = INT_TO_JSVAL(facet->id);
        break;
    
    case fixed:
 
        *vp = INT_TO_JSVAL(facet->fixed);
        break;

    default:

        *vp = JSVAL_VOID;
        JS_ReportError(cx, "Unknown property for facet with id: %d", idval);
        res = JS_FALSE;
    }

  exit:

    GlobusWSDLDebugExit();
    return res;
}

static
JSBool
xsdTypeGetter(
    JSContext *                         cx,
    JSObject *                          obj,
    jsval                               jid,
    jsval *                             vp)
{
    int                                 idval;
    JSObject *                          subTypesObj;
    JSObject *                          attribObj;
    JSObject *                          typeObj;
    JSObject *                          wildcardObj;
    JSObject *                          facetsObj;
    JSString *                          str;
    xmlSchemaTypePtr                    xsdType;
    JSBool                              res = JS_TRUE;
    GlobusFuncName(xsdTypeGetter);
    GlobusWSDLDebugEnter();

    xsdType = (xmlSchemaTypePtr) JS_GetPrivate(cx, obj);
    globus_assert(xsdType);

    if(!JSVAL_IS_INT(jid))
    {
        char *                          prop_name;

        prop_name = JS_GetStringBytes(JSVAL_TO_STRING(jid));
        JS_ReportError(
            cx, 
            "Unknown property: .%s for xsd:type element", prop_name);
        res = JS_FALSE;
        goto exit;
    }
        
    idval = JSVAL_TO_INT(jid);
    switch(idval)
    {
    case type:

        *vp = INT_TO_JSVAL(xsdType->type);
        break;

    case name:

        if(!xsdType->name)
        {
            *vp = JSVAL_VOID;
            break;
        }

        str = JS_InternString(cx, xsdType->name);
        *vp = STRING_TO_JSVAL(str);
        break;

    case id:

        *vp = INT_TO_JSVAL(xsdType->id);
        break;

    case subtypes:
        
        if(!xsdType->subtypes)
        {
            *vp = JSVAL_VOID;
            break;
        }

        subTypesObj = JS_NewObject(cx, &xsdSubTypes_class, NULL, obj);
        globus_assert(subTypesObj);
        JS_SetPrivate(cx, subTypesObj, xsdType->subtypes);

        *vp = OBJECT_TO_JSVAL(subTypesObj);
        break;
    
    case attributes:

        if(!xsdType->attributes)
        {
            *vp = JSVAL_VOID;
            break;
        }

        attribObj = JS_NewObject(cx, &xsdAttrs_class, NULL, obj);
        globus_assert(attribObj);
        JS_SetPrivate(cx, attribObj, xsdType->attributes);

        *vp = OBJECT_TO_JSVAL(attribObj);
        break;
    
    case minOccurs:

        *vp = INT_TO_JSVAL(xsdType->minOccurs);
        break;

    case maxOccurs:

        *vp = 
        (xsdType->maxOccurs == GLOBUS_L_WSDL_JSCRIPT_UNBOUNDED) ? 
        JS_GetPositiveInfinityValue(cx) :
        INT_TO_JSVAL(xsdType->maxOccurs);
        break;

    case ref:

        if(!xsdType->ref)
        {
            *vp = JSVAL_VOID;
            break;
        }

        str = JS_InternString(cx, xsdType->ref);
        *vp = STRING_TO_JSVAL(str);
        break;

    case refNs:

        if(!xsdType->refNs)
        {
            *vp = JSVAL_VOID;
            break;
        }

        str = JS_InternString(cx, xsdType->refNs);
        *vp = STRING_TO_JSVAL(str);
        break;

    case refQname:

        if(!xsdType->ref)
        {
            *vp = JSVAL_VOID;
            break;
        }
        else if(!xsdType->refNs)
        {
            str = JS_InternString(cx, xsdType->ref);
            *vp = STRING_TO_JSVAL(str);
            break;
        }
        else
        {
            str = globus_l_wsdl_jscript_qname_string(
                cx, xsdType->refNs, xsdType->ref);
            *vp = STRING_TO_JSVAL(str);
            break;
        }

    case base:

        if(!xsdType->base)
        {
            *vp = JSVAL_VOID;
            break;
        }

        if(xsdType->type == XML_SCHEMA_TYPE_UNION)
        {
            char * memberTypes = NULL;
            int len = 0;
            int memberTypes_length = 1;
            xsdQNamePtr qnames, qn;

            globus_i_wsdl_split_qnames(NULL, xsdType->node, xsdType->base, &qnames);
            qn = qnames;
            while(qn)
            {
                len = strlen(qn->Namespace) + strlen(qn->name) + 2;
                memberTypes = realloc(memberTypes, 
                                      (sizeof(char) * len) + 
                                      memberTypes_length);
                sprintf(memberTypes + memberTypes_length - 1, "%s#%s ", 
                        qn->Namespace, qn->name);
                memberTypes_length += len;

                qn = qn->next;
            }

            xsdQNameFree(qnames);

            str = JS_InternString(cx, memberTypes);
        }
        else
        {
            str = JS_InternString(cx, xsdType->base);
        }
        
        *vp = STRING_TO_JSVAL(str);
        break;

    case baseNs:

        if(!xsdType->baseNs)
        {
            *vp = JSVAL_VOID;
            break;
        }

        str = JS_InternString(cx, xsdType->baseNs);
        *vp = STRING_TO_JSVAL(str);
        break;

    case baseQName:

        if(!xsdType->base)
        {
            *vp = JSVAL_VOID;
            break;
        }

        if(!xsdType->baseNs)
        {
            str = JS_InternString(cx, xsdType->base);
            *vp = STRING_TO_JSVAL(str);
            break;
        }

        str = globus_l_wsdl_jscript_qname_string(
            cx, xsdType->baseNs, xsdType->base);
        *vp = STRING_TO_JSVAL(str);
        break;
        
    case baseType:

        if(!xsdType->baseType)
        {
            *vp = JSVAL_VOID;
            break;
        }

        typeObj = JS_NewObject(cx, &xsdType_class, xsdTypeClass, obj);
        globus_assert(typeObj);
        JS_SetPrivate(cx, typeObj, (void *)xsdType->baseType);
        JS_DefineProperties(cx, typeObj, xsdType_properties);
        *vp = OBJECT_TO_JSVAL(typeObj);
        break;

    case targetNamespace:

        if(!xsdType->targetNamespace)
        {
            *vp = JSVAL_VOID;
            break;
        }

        str = JS_InternString(cx, xsdType->targetNamespace);
        *vp = STRING_TO_JSVAL(str);
        break;

    case qname:

        if(!xsdType->name)
        {
            *vp = JSVAL_VOID;
            break;
        }
        else if(!xsdType->targetNamespace)
        {
            str = JS_InternString(cx, xsdType->name);
            *vp = STRING_TO_JSVAL(str);
            break;
        }
        else
        {
            str = globus_l_wsdl_jscript_qname_string(
                cx, xsdType->targetNamespace, xsdType->name);
            *vp = STRING_TO_JSVAL(str);
            break;
        }

    case attributeWildcard:

        if(!xsdType->attributeWildcard)
        {
            *vp = JSVAL_VOID;
            break;
        }

        wildcardObj = JS_NewObject(cx, 
                                   &xsdWildcard_class, 
                                   xsdWildcardClass, obj);
        globus_assert(wildcardObj);
        JS_SetPrivate(cx, wildcardObj, (void *)xsdType->attributeWildcard);
        JS_DefineProperties(cx, wildcardObj, xsdWildcard_properties);
        *vp = OBJECT_TO_JSVAL(wildcardObj);
        break;

    case facets:

        if(!xsdType->facets)
        {
            *vp = JSVAL_VOID;
            break;
        }

        facetsObj = JS_NewObject(cx, &xsdFacets_class, NULL, obj);
        globus_assert(facets);
        JS_SetPrivate(cx, facetsObj, xsdType->facets);

        *vp = OBJECT_TO_JSVAL(facetsObj);
        break;

    case namedType:

        if(xsdType->type != XML_SCHEMA_TYPE_ELEMENT || 
           !((xmlSchemaElementPtr)xsdType)->namedType)
        {
            *vp = JSVAL_VOID;
            break;
        }

        str = JS_InternString(cx, ((xmlSchemaElementPtr)xsdType)->namedType);
        *vp = STRING_TO_JSVAL(str);
        break;

    case namedTypeNs:

        if(((xmlSchemaElementPtr)xsdType)->type != XML_SCHEMA_TYPE_ELEMENT || 
           !((xmlSchemaElementPtr)xsdType)->namedTypeNs)
        {
            *vp = JSVAL_VOID;
            break;
        }

        str = JS_InternString(cx, ((xmlSchemaElementPtr)xsdType)->namedTypeNs);
        *vp = STRING_TO_JSVAL(str);
        break;

    default:

        *vp = JSVAL_VOID;
        JS_ReportError(cx, "Unknown property for xsd type with id: %d", idval);
        res = JS_FALSE;
    }

  exit:

    GlobusWSDLDebugExit();
    return res;
}

static
JSBool
xsdWildcardGetter(
    JSContext *                         cx,
    JSObject *                          obj,
    jsval                               jid,
    jsval *                             vp)
{
    int                                 idval;
    xmlSchemaWildcardPtr                xsdWildcard;
    JSBool                              res = JS_TRUE;
    GlobusFuncName(xsdWildcardGetter);
    GlobusWSDLDebugEnter();

    xsdWildcard = (xmlSchemaWildcardPtr) JS_GetPrivate(cx, obj);
    globus_assert(xsdWildcard);

    if(!JSVAL_IS_INT(jid))
    {
        char *                          prop_name;

        prop_name = JS_GetStringBytes(JSVAL_TO_STRING(jid));
        JS_ReportError(cx, 
                       "Unknown property: .%s for wildcard", prop_name);
        res = JS_FALSE;
        goto exit;
    }
        
    idval = JSVAL_TO_INT(jid);
    switch(idval)
    {
    case type:

        *vp = INT_TO_JSVAL(xsdWildcard->type);
        break;

    case id:

        *vp = INT_TO_JSVAL(xsdWildcard->id);
        break;

    case minOccurs:

        *vp = INT_TO_JSVAL(xsdWildcard->minOccurs);
        break;

    case maxOccurs:

        *vp = 
        (xsdWildcard->maxOccurs == GLOBUS_L_WSDL_JSCRIPT_UNBOUNDED) ? 
        JS_GetPositiveInfinityValue(cx) :
        INT_TO_JSVAL(xsdWildcard->maxOccurs);
        break;

    case any:

        *vp = INT_TO_JSVAL(xsdWildcard->any);
        break;
        
    default:

        *vp = JSVAL_VOID;
        JS_ReportError(cx, 
                       "Unknown property for xsd "
                       "wildcard with id: %d", idval);
        res = JS_FALSE;
    }

  exit:

    GlobusWSDLDebugExit();
    return res;
}

static
JSBool
xsdAttrGetter(
    JSContext *                         cx,
    JSObject *                          obj,
    jsval                               jid,
    jsval *                             vp)
{
    int                                 idval;
    xmlSchemaAttributePtr               attr;
    JSBool                              res = JS_TRUE;
    JSObject *                          subTypesObj;
    JSObject *                          attribObj;
    JSString *                          str;
    JSObject *                          typeObj;
    GlobusFuncName(xsdAttrGetter);
    GlobusWSDLDebugEnter();

    attr = (xmlSchemaAttributePtr) JS_GetPrivate(cx, obj);
    globus_assert(attr);

    if(!JSVAL_IS_INT(jid))
    {
        JS_ReportError(cx, 
                       "Invalid id for xsd attribute property, "
                       "expected int not string");
        res = JS_FALSE;
        goto exit;
    }
    
    
    idval = JSVAL_TO_INT(jid);
    switch(idval)
    {
    case type:
      
        *vp = INT_TO_JSVAL(attr->type);
        break;

    case name:

        if(!attr->name)
        {
            *vp = JSVAL_VOID;
            break;
        }

        str = JS_InternString(cx, attr->name);
        *vp = STRING_TO_JSVAL(str);
        break;

    case id:

        *vp = INT_TO_JSVAL(attr->id);
        break;

    case ref:

        if(!attr->ref)
        {
            *vp = JSVAL_VOID;
            break;
        }

        str = JS_InternString(cx, attr->ref);
        *vp = STRING_TO_JSVAL(str);
        break;

    case refNs:

        if(!attr->refNs)
        {
            *vp = JSVAL_VOID;
            break;
        }

        str = JS_InternString(cx, attr->refNs);
        *vp = STRING_TO_JSVAL(str);
        break;

    case refQname:

        if(!attr->ref)
        {
            *vp = JSVAL_VOID;
            break;
        }
        else if(!attr->refNs)
        {
            str = JS_InternString(cx, attr->ref);
            *vp = STRING_TO_JSVAL(str);
            break;
        }
        else
        {
            str = globus_l_wsdl_jscript_qname_string(
                cx, attr->refNs, attr->ref);
            *vp = STRING_TO_JSVAL(str);
            break;
        }

    case typeName:

        if(!attr->typeName)
        {
            *vp = JSVAL_VOID;
            break;
        }

        str = JS_InternString(cx, attr->typeName);
        *vp = STRING_TO_JSVAL(str);
        break;

    case typeNs:

        if(!attr->typeNs)
        {
            *vp = JSVAL_VOID;
            break;
        }

        str = JS_InternString(cx, attr->typeNs);
        *vp = STRING_TO_JSVAL(str);
        break;

    case typeQname:

        if(!attr->typeName)
        {
            *vp = JSVAL_VOID;
            break;
        }

        if(!attr->typeNs)
        {
            str = JS_InternString(cx, attr->typeNs);
            *vp = STRING_TO_JSVAL(str);
            break;
        }
        
        str = globus_l_wsdl_jscript_qname_string(cx, attr->typeNs, attr->typeName);
        *vp = STRING_TO_JSVAL(str);
        break;

    case subtypes:
        
        if(!attr->subtypes)
        {
            *vp = JSVAL_VOID;
            break;
        }

        subTypesObj = JS_NewObject(cx, &xsdSubTypes_class, NULL, obj);
        globus_assert(subTypesObj);
        JS_SetPrivate(cx, subTypesObj, attr->subtypes);

        *vp = OBJECT_TO_JSVAL(subTypesObj);
        break;
    
    case occurs:

        *vp = INT_TO_JSVAL(attr->occurs);
        break;

    case base:

        if(!attr->base)
        {
            *vp = JSVAL_VOID;
            break;
        }

        typeObj = JS_NewObject(cx, &xsdType_class, xsdTypeClass, obj);
        globus_assert(typeObj);
        JS_SetPrivate(cx, typeObj, (void *)attr->base);
        JS_DefineProperties(cx, typeObj, xsdType_properties);
        *vp = OBJECT_TO_JSVAL(typeObj);
        break;

    case defValue:
        
        if(!attr->defValue)
        {
            *vp = JSVAL_VOID;
            break;
        }
        
        str = JS_InternString(cx, attr->defValue);
        *vp = STRING_TO_JSVAL(str);
        break;

    case targetNamespace:

        if(!attr->targetNamespace)
        {
            str = JS_InternString(cx, "");
        }
        else
        {
            str = JS_InternString(cx, attr->targetNamespace);
        }

        *vp = STRING_TO_JSVAL(str);
        break;

    case qname:

        if(!attr->name)
        {
            *vp = JSVAL_VOID;
            break;
        }
        else if(!attr->targetNamespace)
        {
            str = JS_InternString(cx, attr->name);
            *vp = STRING_TO_JSVAL(str);
            break;
        }
        else
        {
            str = globus_l_wsdl_jscript_qname_string(
                cx, attr->targetNamespace, attr->name);
            *vp = STRING_TO_JSVAL(str);
            break;
        }

    case attributes:
      
        if(attr->type != XML_SCHEMA_TYPE_ATTRIBUTEGROUP)
        {
            *vp = JSVAL_VOID;
            break;
        }

        attribObj = JS_NewObject(cx, &xsdAttrs_class, NULL, obj);
        globus_assert(attribObj);

        JS_SetPrivate(cx, attribObj, 
                      ((xmlSchemaAttributeGroupPtr)attr)->attributes);

        *vp = OBJECT_TO_JSVAL(attribObj);
        break;
        
    default:

        *vp = JSVAL_VOID;
        JS_ReportError(cx, 
                       "Unknown property for xsd "
                       "attribute with id: %d", idval);
        res = JS_FALSE;
    }

  exit:

    GlobusWSDLDebugExit();
    return res;
}

static
JSBool
xsdElementGetter(
    JSContext *                         cx,
    JSObject *                          obj,
    jsval                               jid,
    jsval *                             vp)
{
    int                                 idval;
    xmlSchemaElementPtr                 element;
    JSString *                          str;
    JSObject *                          attribObj;
    JSBool                              res = JS_TRUE;
    JSObject *                          subTypesObj;
        
    GlobusFuncName(xsdElementGetter);
    GlobusWSDLDebugEnter();

    element = (xmlSchemaElementPtr) JS_GetPrivate(cx, obj);
    globus_assert(element);

    if(!JSVAL_IS_INT(jid))
    {
        JS_ReportError(cx, 
                       "Invalid id type (expected int) for "
                       "xsd element property");
        res = JS_FALSE;
        goto exit;
    }
    
    
    idval = JSVAL_TO_INT(jid);
    switch(idval)
    {
    case type:
      
        *vp = INT_TO_JSVAL(element->type);
        break;

    case name:

        if(!element->name)
        {
            *vp = JSVAL_VOID;
            break;
        }

        str = JS_InternString(cx, element->name);
        *vp = STRING_TO_JSVAL(str);
        break;

    case id:

        *vp = INT_TO_JSVAL(element->id);
        break;

    case subtypes:
        
        if(!element->subtypes)
        {
            *vp = JSVAL_VOID;
            break;
        }

        subTypesObj = JS_NewObject(cx, &xsdSubTypes_class, NULL, obj);
        globus_assert(subTypesObj);

        JS_SetPrivate(cx, subTypesObj, element->subtypes);

        *vp = OBJECT_TO_JSVAL(subTypesObj);
        break;
    
    case attributes:

        if(!element->attributes)
        {
            *vp = JSVAL_VOID;
            break;
        }

        attribObj = JS_NewObject(cx, &xsdAttrs_class, NULL, obj);
        globus_assert(attribObj);

        JS_SetPrivate(cx, attribObj, element->attributes);

        *vp = OBJECT_TO_JSVAL(attribObj);
        break;
    
    case minOccurs:

        *vp = INT_TO_JSVAL(element->minOccurs);
        break;

    case maxOccurs:

        *vp = 
        (element->maxOccurs == GLOBUS_L_WSDL_JSCRIPT_UNBOUNDED) ? 
        JS_GetPositiveInfinityValue(cx) :
        INT_TO_JSVAL(element->maxOccurs);
        break;

    case ref:

        if(!element->ref)
        {
            *vp = JSVAL_VOID;
            break;
        }

        str = JS_InternString(cx, element->ref);
        *vp = STRING_TO_JSVAL(str);
        break;

    case refNs:

        if(!element->refNs)
        {
            *vp = JSVAL_VOID;
            break;
        }

        str = JS_InternString(cx, element->refNs);
        *vp = STRING_TO_JSVAL(str);
        break;

    case refQname:

        if(!element->ref)
        {
            *vp = JSVAL_VOID;
            break;
        }
        else if(!element->refNs)
        {
            str = JS_InternString(cx, element->ref);
            *vp = STRING_TO_JSVAL(str);
            break;
        }
        else
        {
            str = globus_l_wsdl_jscript_qname_string(
                cx, element->refNs, element->ref);
            *vp = STRING_TO_JSVAL(str);
            break;
        }

    case targetNamespace:

        if(!element->targetNamespace)
        {
            str = JS_InternString(cx, "");
        }
        else
        {
            str = JS_InternString(cx, element->targetNamespace);
        }

        *vp = STRING_TO_JSVAL(str);
        break;
    
    case namedType:

        if(!element->namedType)
        {
            *vp = JSVAL_VOID;
            break;
        }

        str = JS_InternString(cx, element->namedType);
        *vp = STRING_TO_JSVAL(str);
        break;

    case namedTypeNs:

        if(!element->namedTypeNs)
        {
            *vp = JSVAL_VOID;
            break;
        }

        str = JS_InternString(cx, element->namedTypeNs);
        *vp = STRING_TO_JSVAL(str);
        break;

    case substGroup:

        if(!element->substGroup)
        {
            *vp = JSVAL_VOID;
            break;
        }

        str = JS_InternString(cx, element->substGroup);
        *vp = STRING_TO_JSVAL(str);
        break;

    case substGroupNs:

        if(!element->substGroupNs)
        {
            *vp = JSVAL_VOID;
            break;
        }

        str = JS_InternString(cx, element->substGroupNs);
        *vp = STRING_TO_JSVAL(str);
        break;

    case contentType:

        *vp = INT_TO_JSVAL(element->contentType);
        break;

    case qname:

        if(!element->name)
        {
            *vp = JSVAL_VOID;
            break;
        }
        else if(!element->targetNamespace)
        {
            str = JS_InternString(cx, element->name);
            *vp = STRING_TO_JSVAL(str);
            break;
        }
        else
        {
            str = globus_l_wsdl_jscript_qname_string(
                cx, element->targetNamespace, element->name);
            *vp = STRING_TO_JSVAL(str);
            break;
        }

    default:

        *vp = JSVAL_VOID;
        JS_ReportError(cx, "Unknown property with id %d for element", idval);
        res = JS_FALSE;
        goto exit;
    }

  exit:

    GlobusWSDLDebugExit();
    return res;
}

static
JSBool
messageGetter(
    JSContext *                         cx,
    JSObject *                          obj,
    jsval                               jid,
    jsval *                             vp)
{
    JSBool                              res = JS_TRUE;
    int                                 idval;
    JSString *                          str = NULL;
    JSObject *                          partsObj = NULL;
    wsdlMessagePtr                      message = NULL;
    GlobusFuncName(messageGetter);
    GlobusWSDLDebugEnter();

    message = (wsdlMessagePtr) JS_GetPrivate(cx, obj);
    globus_assert(message);

    if(!JSVAL_IS_INT(jid))
    {
        JS_ReportError(cx, 
                       "Invalid id type (expected int) for "
                       "wsdl element property");
        res = JS_FALSE;
        goto exit;
    }
    
    
    idval = JSVAL_TO_INT(jid);
    switch(idval)
    {

    case name:
    
        if(!message->name)
        {
            *vp = JSVAL_VOID;
            break;
        }

        str = JS_InternString(cx, message->name);
        globus_assert(str);

        *vp = STRING_TO_JSVAL(str);
        break;

    case parts:

        if(!message->parts)
        {
            *vp = JSVAL_VOID;
            break;
        }

        partsObj = JS_NewObject(cx, &parts_class, NULL, obj);
        globus_assert(partsObj);

        JS_SetPrivate(cx, partsObj, (void *) message->parts);

        *vp = OBJECT_TO_JSVAL(partsObj);
        break;
        
    case targetNamespace:

        if(!message->schema || !message->schema->targetNamespace)
        {
            *vp = JSVAL_VOID;
            break;
        }

        str = JS_InternString(cx, message->schema->targetNamespace);
        globus_assert(str);
        
        *vp = STRING_TO_JSVAL(str);
        break;

    default:

        *vp = JSVAL_VOID;
        JS_ReportError(cx, 
                       "Unknown message property with "
                       "id %d for element", idval);
        res = JS_FALSE;
        goto exit;
    }

  exit:

    GlobusWSDLDebugExit();
    return res;
}

#include "wsdlInternals.h"

static
JSBool
xsdQNamesGetter(
    JSContext *                         cx,
    JSObject *                          obj,
    jsval                               id,
    jsval *                             vp)
{
    xsdQNamePtr                         qn;
    JSObject *                          qnObj;
    JSBool                              res = JS_TRUE;
    JSString *                          str;
    int                                 i = 0;
    GlobusFuncName(xsdQNamesGetter);
    GlobusWSDLDebugEnter();

    qn = (xsdQNamePtr) JS_GetPrivate(cx, obj);
    globus_assert(qn);

    if(JSVAL_IS_STRING(id))
    {
        while(qn)
        {
            str = globus_l_wsdl_jscript_qname_string(
                cx, qn->Namespace, qn->name);
            *vp = STRING_TO_JSVAL(str);
            if(JS_CompareStrings(str, JSVAL_TO_STRING(id)))
            {
                break;
            }

            qn = qn->next;
        }
    }
    else if(JSVAL_IS_INT(id))
    {
        for(; i < JSVAL_TO_INT(id) && type; ++i)
        {
            qn = qn->next;
        }
        globus_assert(qn);  
    }
    else
    {
        qn = (xsdQNamePtr) JSVAL_TO_PRIVATE(id);
        globus_assert(qn);
    }

    qnObj = JS_NewObject(cx, &xsdQName_class, 
                         xsdQNameClass, obj);
    globus_assert(qnObj);
    JS_SetPrivate(cx, qnObj, (void *)qn);
    JS_DefineProperties(cx, qnObj, xsdQName_properties);
    *vp = OBJECT_TO_JSVAL(qnObj);

    GlobusWSDLDebugExit();
    return res;
}

static
JSBool
xsdQNamesEnumerate(
    JSContext *                         cx,
    JSObject *                          obj,
    JSIterateOp                         enum_op,
    jsval *                             statep,
    jsid *                              idp)
{
    xsdQNamePtr                         qn = NULL;
    JSBool                              res = JS_TRUE;
    int                                 i = 0;
    int                                 index;
    GlobusFuncName(xsdQNameEnumerate);
    GlobusWSDLDebugEnter();

    qn = (xsdQNamePtr) JS_GetPrivate(cx, obj);
    globus_assert(qn);
    
    switch(enum_op)
    {
    case JSENUMERATE_INIT:
        
        *statep = INT_TO_JSVAL(0);
        if(idp)
        {
            *idp = JSVAL_ZERO;
        }
        break;

    case JSENUMERATE_NEXT:

        index = JSVAL_TO_INT(*statep);
        for(; i < index && qn; ++i)
        {
            qn = qn->next;
        }

        if(!qn)
        {
            *statep = JSVAL_NULL;
            *idp = JSVAL_ZERO;
            break;
        }

        *idp = *statep;
        *statep = INT_TO_JSVAL(index + 1);

        break;

    case JSENUMERATE_DESTROY:
            
        break;
    }

    GlobusWSDLDebugExit();
    return res;
}

static
JSBool
xsdQNameGetter(
    JSContext *                         cx,
    JSObject *                          obj,
    jsval                               jid,
    jsval *                             vp)
{               
    xsdQNamePtr                         qn;
    JSBool                              res = JS_TRUE;
    jsval                               idval;
    JSString *                          str;
    GlobusFuncName(portTypeGetter);
    GlobusWSDLDebugEnter();

    qn = (xsdQNamePtr) JS_GetPrivate(cx, obj);
    globus_assert(qn);

    if(!JSVAL_IS_INT(jid))
    {
        JS_ReportError(cx, 
                       "Invalid id type (expected int) for "
                       "wsdl xsdQName property");
        res = JS_FALSE;
        goto exit;
    }
    
    idval = JSVAL_TO_INT(jid);
    switch(idval)
    {
    case name:

        if(!qn->name)
        {
            *vp = JSVAL_VOID;
            break;
        }

        str = JS_InternString(cx, qn->name);
        if(!str)
        {
            *vp = JSVAL_VOID;
            break;
        }

        *vp = STRING_TO_JSVAL(str);
        break;

    case Namespace:

        if(!qn->Namespace)
        {
            *vp = JSVAL_VOID;
            break;
        }

        str = JS_InternString(cx, qn->Namespace);
        if(!str)
        {
            *vp = JSVAL_VOID;
            break;
        }

        *vp = STRING_TO_JSVAL(str);
        break;

    case qname:

        if(!qn->name)
        {
            *vp = JSVAL_VOID;
            break;
        }
        else if(!qn->Namespace)
        {
            str = JS_InternString(cx, qn->name);
            if(!str)
            {
                *vp = JSVAL_VOID;
                break;
            }

            *vp = STRING_TO_JSVAL(str);
            break;
        }
        else
        {
            str = globus_l_wsdl_jscript_qname_string(
                cx, qn->Namespace, qn->name);
            *vp = STRING_TO_JSVAL(str);
            break;
        }
    
    default:

        *vp = JSVAL_VOID;
        JS_ReportError(cx, 
                       "Unknown xsdQName property "
                       "with id %d for element", idval);
        res = JS_FALSE;
        goto exit;
    }

  exit:

    GlobusWSDLDebugExit();
    return res;
}

static
JSBool
portTypeGetter(
    JSContext *                         cx,
    JSObject *                          obj,
    jsval                               jid,
    jsval *                             vp)
{
    JSBool                              res = JS_TRUE;
    JSString *                          str = NULL;
    int                                 idval;
    JSObject *                          operationsObj = NULL;
    JSObject *                          xsdQNamesObj = NULL;
    wsdlPortTypePtr                     portType = NULL;
    GlobusFuncName(portTypeGetter);
    GlobusWSDLDebugEnter();

    portType = (wsdlPortTypePtr) JS_GetPrivate(cx, obj);
    globus_assert(portType);

    if(!JSVAL_IS_INT(jid))
    {
        JS_ReportError(cx, 
                       "Invalid id type (expected int) for "
                       "wsdl PortType property");
        res = JS_FALSE;
        goto exit;
    }
    
    idval = JSVAL_TO_INT(jid);
    switch(idval)
    {
    case name:

        if(!portType->name)
        {
            *vp = JSVAL_VOID;
            break;
        }

        str = JS_InternString(cx, portType->name);
        if(!str)
        {
            *vp = JSVAL_VOID;
            break;
        }

        *vp = STRING_TO_JSVAL(str);
        break;

    case operations:

        if(!portType->operations)
        {
            *vp = JSVAL_VOID;
            break;
        }

        operationsObj = JS_NewObject(cx, &operations_class, NULL, obj);
        globus_assert(operationsObj);

        JS_SetPrivate(cx, operationsObj, (void *) portType->operations);
        
        *vp = OBJECT_TO_JSVAL(operationsObj);
        break;

    case ResourceProperties:

        if(!portType->ResourceProperties)
        {
            *vp = JSVAL_VOID;
            break;
        }

        xsdQNamesObj = JS_NewObject(cx, &xsdQNames_class, NULL, obj);
        globus_assert(xsdQNamesObj);
        
        JS_SetPrivate(cx, xsdQNamesObj, (void *) portType->ResourceProperties);

        *vp = OBJECT_TO_JSVAL(xsdQNamesObj);
        break;

    case implements:

        if(!portType->implements)
        {
            *vp = JSVAL_VOID;
            break;
        }

        xsdQNamesObj = JS_NewObject(cx, &xsdQNames_class, NULL, obj);
        globus_assert(xsdQNamesObj);

        JS_SetPrivate(cx, xsdQNamesObj, (void *) portType->implements);

        *vp = OBJECT_TO_JSVAL(xsdQNamesObj);
        break;

    case targetNamespace:

        if(!portType->schema || !portType->schema->targetNamespace)
        {
            *vp = JSVAL_VOID;
            break;
        }

        str = JS_InternString(cx, portType->schema->targetNamespace);
        globus_assert(str);
        
        *vp = STRING_TO_JSVAL(str);
        break;
    
    default:

        *vp = JSVAL_VOID;
        JS_ReportError(cx, 
                       "Unknown portType property "
                       "with id %d for element", idval);
        res = JS_FALSE;
        goto exit;
    }

  exit:

    GlobusWSDLDebugExit();
    return res;
}

static
JSBool
bindingGetter(
    JSContext *                         cx,
    JSObject *                          obj,
    jsval                               jid,
    jsval *                             vp)
{
    JSBool                              res = JS_TRUE;
    int                                 idval;
    JSString *                          str = NULL;
    JSObject *                          portTypeObj = NULL;
    JSObject *                          operationsObj = NULL;
    JSObject *                          soapBindingObj = NULL;
    wsdlBindingPtr                      binding = NULL;
    GlobusFuncName(bindingGetter);
    GlobusWSDLDebugEnter();

    binding = (wsdlBindingPtr) JS_GetPrivate(cx, obj);
    globus_assert(binding);

    if(!JSVAL_IS_INT(jid))
    {
        JS_ReportError(cx, 
                       "Invalid id type (expected int) for "
                       "wsdl binding property");
        res = JS_FALSE;
        goto exit;
    }
    
    
    idval = JSVAL_TO_INT(jid);
    switch(idval)
    {
    case name:

        if(!binding->name)
        {
            *vp = JSVAL_VOID;
            break;
        }

        str = JS_InternString(cx, binding->name);
        if(!str)
        {
            *vp = JSVAL_VOID;
            break;
        }

        *vp = STRING_TO_JSVAL(str);
        break;
        
    case type:

        if(!binding->type)
        {
            *vp = JSVAL_VOID;
            break;
        }

        portTypeObj = JS_NewObject(cx, &portType_class, 
                                   portTypeClass, obj);
        globus_assert(portTypeObj);
        JS_SetPrivate(cx, portTypeObj, (void *)binding->type);
        JS_DefineProperties(cx, portTypeObj, portType_properties);
        *vp = OBJECT_TO_JSVAL(portTypeObj);
        break;

    case operations:

        if(!binding->operations)
        {
            *vp = JSVAL_VOID;
            break;
        }

        operationsObj = JS_NewObject(cx, &bindingOperations_class, 
                                     NULL, obj);
        globus_assert(operationsObj);
        JS_SetPrivate(cx, operationsObj, (void *)binding->operations);

        *vp = OBJECT_TO_JSVAL(operationsObj);
        break;

    case soap:

        if(!binding->extension || (binding->extensionType != SOAP_BINDING))
        {
            *vp = JSVAL_VOID;
            break;
        }

        soapBindingObj = JS_NewObject(cx, &soapBinding_class, 
                                      soapBindingClass, obj);
        globus_assert(soapBindingObj);
        JS_SetPrivate(cx, soapBindingObj, 
                                (void *)binding->extension);
        JS_DefineProperties(cx, soapBindingObj, 
                                      soapBinding_properties);
        *vp = OBJECT_TO_JSVAL(soapBindingObj);
        break;
        
    case extensionType:

        *vp = INT_TO_JSVAL(binding->extensionType);
        break;

    case targetNamespace:

        if(!binding->schema || !binding->schema->targetNamespace)
        {
            *vp = JSVAL_VOID;
            break;
        }

        str = JS_InternString(cx, binding->schema->targetNamespace);
        globus_assert(str);
        
        *vp = STRING_TO_JSVAL(str);
        break;

    default:

        *vp = JSVAL_VOID;
        JS_ReportError(cx, 
                       "Unknown property for binding "
                       "with id %d for element", idval);
        res = JS_FALSE;
        goto exit;
    }

  exit:

    GlobusWSDLDebugExit();
    return res;
}

static
JSBool
bindingOperationsGetter(
    JSContext *                         cx,
    JSObject *                          obj,
    jsval                               id,
    jsval *                             vp)
{
    JSBool                              res = JS_TRUE;
    wsdlBindingOperationPtr             bop;
    JSObject *                          bopObj;
    int                                 i = 0;
    GlobusFuncName(bindingOperationsGetter);
    GlobusWSDLDebugEnter();

    bop = (wsdlBindingOperationPtr) JS_GetPrivate(cx, obj);
    globus_assert(bop);

    if(JSVAL_IS_STRING(id))
    {
        while(bop)
        {
            JSString *                  name;
            
            name = JS_InternString(cx, bop->name);
            if(JS_CompareStrings(name, JSVAL_TO_STRING(id)))
            {
                break;
            }

            bop = bop->next;
        }
    }
    else if(JSVAL_IS_INT(id))
    {
        for(; i < JSVAL_TO_INT(id) && bop; ++i)
        {
            bop = bop->next;
        }
        globus_assert(bop);
    }
    else
    {
        bop = (wsdlBindingOperationPtr) JSVAL_TO_PRIVATE(id);
        globus_assert(bop);
    }

    bopObj = JS_NewObject(cx, &bindingOperation_class, 
                          bindingOperationClass, obj);
    globus_assert(bopObj);
    JS_SetPrivate(cx, bopObj, (void *)bop);
    JS_DefineProperties(cx, bopObj, bindingOperation_properties);
    *vp = OBJECT_TO_JSVAL(bopObj);

    GlobusWSDLDebugExit();
    return res;
}

static
JSBool
bindingOperationsEnumerate(
    JSContext *                         cx,
    JSObject *                          obj,
    JSIterateOp                         enum_op,
    jsval *                             statep,
    jsid *                              idp)
{
    JSBool                              res = JS_TRUE;
    wsdlBindingOperationPtr             boperation;
    int                                 i = 0;
    int                                 index;
    GlobusFuncName(bindingOperationsEnumerate);
    GlobusWSDLDebugEnter();

    boperation = (wsdlBindingOperationPtr) JS_GetPrivate(cx, obj);
    globus_assert(boperation);
    
    switch(enum_op)
    {
    case JSENUMERATE_INIT:
        
        *statep = INT_TO_JSVAL(i);
        break;

    case JSENUMERATE_NEXT:

        index = JSVAL_TO_INT(*statep);
        for(; i < index && boperation; ++i)
        {
            boperation = boperation->next;
        }

        if(!boperation)
        {
            *statep = JSVAL_NULL;
            *idp = JSVAL_ZERO;
            break;
        }

        *idp = *statep;
        *statep = INT_TO_JSVAL(index + 1);

        break;

    case JSENUMERATE_DESTROY:
            
        break;
    }

    GlobusWSDLDebugExit();
    return res;
}

static
JSBool
bindingOperationGetter(
    JSContext *                         cx,
    JSObject *                          obj,
    jsval                               jid,
    jsval *                             vp)
{
    int                                 idval;
    JSString *                          str;
    JSBool                              res = JS_TRUE;
    JSObject *                          inObj = NULL;
    JSObject *                          outObj = NULL;
    JSObject *                          faultsObj = NULL;
    JSObject *                          soapObj = NULL;
    wsdlBindingOperationPtr             bop = NULL;
    GlobusFuncName(bindingOperationGetter);
    GlobusWSDLDebugEnter();
    
    bop = (wsdlBindingOperationPtr) JS_GetPrivate(cx, obj);
    globus_assert(bop);

    if(!JSVAL_IS_INT(jid))
    {
        JS_ReportError(cx, 
                       "Invalid id type (expected int) "
                       "for wsdl bindingOperation property");
        res = JS_FALSE;
        goto exit;
    }

    idval = JSVAL_TO_INT(jid);

    switch(idval)
    {
    case name:

        if(!bop->name)
        {
            *vp = JSVAL_VOID;
            break;
        }
        
        str = JS_InternString(cx, bop->name);
        globus_assert(str);

        *vp = STRING_TO_JSVAL(str);
        break;

    case input:

        if(!bop->input || (bop->inputType != SOAP_BINDING))
        {
            *vp = JSVAL_VOID;
            break;
        }

        inObj = JS_NewObject(cx, &soapBindingParam_class, 
                             soapBindingParamClass, obj);
        globus_assert(inObj);
        JS_SetPrivate(cx, inObj, (void *)bop->input);
        JS_DefineProperties(cx, inObj, soapBindingParam_properties);
        *vp = OBJECT_TO_JSVAL(inObj);
        break;
    
    case output:

        if(!bop->output || (bop->outputType != SOAP_BINDING))
        {
            *vp = JSVAL_VOID;
            break;
        }

        outObj = JS_NewObject(cx, &soapBindingParam_class, 
                              soapBindingParamClass, obj);
        globus_assert(outObj);
        JS_SetPrivate(cx, outObj, (void *)bop->output);
        JS_DefineProperties(cx, outObj, soapBindingParam_properties);
        *vp = OBJECT_TO_JSVAL(outObj);
        break;
    
    case inputType:

        *vp = INT_TO_JSVAL(bop->inputType);
        break;

    case outputType:

        *vp = INT_TO_JSVAL(bop->outputType);
        break;

    case faults:

        if(!bop->faults)
        {
            *vp = JSVAL_VOID;
            break;
        }

        faultsObj = JS_NewObject(cx, &bindingOperationFaults_class, 
                                 NULL, obj);
        globus_assert(faultsObj);
        JS_SetPrivate(cx, faultsObj, (void *) bop->faults);
        
        *vp = OBJECT_TO_JSVAL(faultsObj);
        break;

    case soap:

        if(!bop->extension || (bop->extensionType != SOAP_BINDING))
        {
            *vp = JSVAL_VOID;
            break;
        }

        soapObj = JS_NewObject(cx, &soapOperation_class, 
                               soapOperationClass, obj);
        globus_assert(soapObj);
        JS_SetPrivate(cx, soapObj, (void *)bop->extension);
        JS_DefineProperties(cx, soapObj, soapOperation_properties);
        *vp = OBJECT_TO_JSVAL(soapObj);
        break;

    default:

        *vp = JSVAL_VOID;
        JS_ReportError(cx, 
                       "Unknown binding operation property "
                       "with id %d for part", idval);
        res = JS_FALSE;
        goto exit;
    }

  exit:

    GlobusWSDLDebugExit();
    return res;
}

static
JSBool
bindingOperationFaultsGetter(
    JSContext *                         cx,
    JSObject *                          obj,
    jsval                               id,
    jsval *                             vp)
{
    wsdlBindingOperationFaultPtr        bopf;
    JSObject *                          bopfObj;
    JSBool                              res = JS_TRUE;
    int                                 i = 0;
    GlobusFuncName(bindingOperationFaultsGetter);
    GlobusWSDLDebugEnter();
    
    bopf = (wsdlBindingOperationFaultPtr) JS_GetPrivate(cx, obj);
    globus_assert(bopf);
    
    if(JSVAL_IS_STRING(id))
    {
        while(bopf)
        {
            JSString *                  name;
            
            name = JS_InternString(cx, bopf->name);
            if(JS_CompareStrings(name, JSVAL_TO_STRING(id)))
            {
                break;
            }
            
            bopf = bopf->next;
        }
    }
    else if(JSVAL_IS_INT(id))
    {
        for(; i < JSVAL_TO_INT(id) && bopf; ++i)
        {
            bopf = bopf->next;
        }
        globus_assert(bopf);
    }
    else
    {
        bopf = (wsdlBindingOperationFaultPtr) JSVAL_TO_PRIVATE(id);
        globus_assert(bopf);
    }
    
    bopfObj = JS_NewObject(cx, &bindingOperationFault_class, 
                           bindingOperationFaultClass, obj);
    globus_assert(bopfObj);
    JS_SetPrivate(cx, bopfObj, (void *)bopf);
    JS_DefineProperties(cx, bopfObj, 
                                  bindingOperationFault_properties);
    
    *vp = OBJECT_TO_JSVAL(bopfObj);
    
    GlobusWSDLDebugExit();
    return res;
}

static
JSBool
bindingOperationFaultsEnumerate(
    JSContext *                         cx,
    JSObject *                          obj,
    JSIterateOp                         enum_op,
    jsval *                             statep,
    jsid *                              idp)
{
    JSBool                              res = JS_TRUE;
    wsdlBindingOperationFaultPtr        bopf;
    int                                 i = 0;
    int                                 index;
    GlobusFuncName(bindingOperationFaultsEnumerate);
    GlobusWSDLDebugEnter();
    
    bopf = (wsdlBindingOperationFaultPtr) JS_GetPrivate(cx, obj);
    globus_assert(bopf);
    
    switch(enum_op)
    {
    case JSENUMERATE_INIT:
        
        *statep = INT_TO_JSVAL(i);
        break;

    case JSENUMERATE_NEXT:
        
        index = JSVAL_TO_INT(*statep);
        for(; i < index && bopf; ++i)
        {
            bopf = bopf->next;
        }

        if(!bopf)
        {
            *statep = JSVAL_NULL;
            *idp = JSVAL_ZERO;
            break;
        }
        
        *idp = *statep;
        *statep = INT_TO_JSVAL(index + 1);
        
        break;
        
    case JSENUMERATE_DESTROY:
        
        break;
    }

    GlobusWSDLDebugExit();
    return res;
}

static
JSBool
bindingOperationFaultGetter(
    JSContext *                         cx,
    JSObject *                          obj,
    jsval                               jid,
    jsval *                             vp)
{
    int                                 idval;
    JSString *                          str = NULL;
    JSBool                              res = JS_TRUE;
    wsdlBindingOperationFaultPtr        bopf = NULL;
    JSObject *                          soapObj = NULL;
    GlobusFuncName(bindingOperationFaultGetter);
    GlobusWSDLDebugEnter();
    
    bopf = (wsdlBindingOperationFaultPtr) JS_GetPrivate(cx, obj);
    globus_assert(bopf);
    
    if(!JSVAL_IS_INT(jid))
    {
        JS_ReportError(cx, 
                       "Invalid id type (expected int) "
                       "for wsdl bindingOperationFault property");
        res = JS_FALSE;
        goto exit;
    }
    
    idval = JSVAL_TO_INT(jid);
    
    switch(idval)
    {
    case name:

        if(!bopf->name)
        {
            *vp = JSVAL_VOID;
            break;
        }
        
        str = JS_InternString(cx, bopf->name);
        globus_assert(str);

        *vp = STRING_TO_JSVAL(str);
        break;

    case soap:

        if(!bopf->extension || (bopf->extensionType != SOAP_BINDING))
        {
            *vp = JSVAL_VOID;
            break;
        }

        soapObj = JS_NewObject(cx, &soapFault_class, soapFaultClass, obj);
        globus_assert(soapObj);
        JS_SetPrivate(cx, soapObj, bopf->extension);
        JS_DefineProperties(cx, soapObj, soapFault_properties);
        *vp = OBJECT_TO_JSVAL(soapObj);
        break;

    default:

        *vp = JSVAL_VOID;
        JS_ReportError(cx, "Unknown part property with id %d for part", idval);
        res = JS_FALSE;
        goto exit;
    }

  exit:

    GlobusWSDLDebugExit();
    return res;
}

static
JSBool
partsEnumerate(
    JSContext *                         cx,
    JSObject *                          obj,
    JSIterateOp                         enum_op,
    jsval *                             statep,
    jsid *                              idp)
{
    JSBool                              res = JS_TRUE;
    wsdlPartPtr                         parts;
    int                                 i = 0;
    int                                 index;
    GlobusFuncName(partsEnumerate);
    GlobusWSDLDebugEnter();

    parts = (wsdlPartPtr) JS_GetPrivate(cx, obj);
    globus_assert(parts);
    
    switch(enum_op)
    {
    case JSENUMERATE_INIT:
        
        *statep = INT_TO_JSVAL(i);
        break;

    case JSENUMERATE_NEXT:

        index = JSVAL_TO_INT(*statep);
        for(; i < index && parts; ++i)
        {
            parts = parts->next;
        }

        if(!parts)
        {
            *statep = JSVAL_NULL;
            *idp = JSVAL_ZERO;
            break;
        }

        *idp = *statep;
        *statep = INT_TO_JSVAL(index + 1);

        break;

    case JSENUMERATE_DESTROY:
            
        break;
    }

    GlobusWSDLDebugExit();
    return res;
}

static
JSBool
partsGetter(
    JSContext *                         cx,
    JSObject *                          obj,
    jsval                               id,
    jsval *                             vp)
{
    wsdlPartPtr                         part = NULL;
    JSObject *                          partObj = NULL;
    JSBool                              res = JS_TRUE;
    int                                 i = 0;
    GlobusFuncName(partsGetter);
    GlobusWSDLDebugEnter();

    part = (wsdlPartPtr) JS_GetPrivate(cx, obj);
    globus_assert(part);

    if(JSVAL_IS_STRING(id))
    {
        while(part)
        {
            JSString *                  name;
            
            name = JS_InternString(cx, part->name);
            if(JS_CompareStrings(name, JSVAL_TO_STRING(id)))
            {
                break;
            }

            part = part->next;
        }
    }
    else if(JSVAL_IS_INT(id))
    {
        for(; i < JSVAL_TO_INT(id) && part; ++i)
        {
            part = part->next;
        }
        globus_assert(part); 
    }

    partObj = JS_NewObject(cx, &part_class, 
                          partClass, obj);
    globus_assert(partObj);
    JS_SetPrivate(cx, partObj, (void *)part);
    JS_DefineProperties(cx, partObj, part_properties);

    *vp = OBJECT_TO_JSVAL(partObj);

    GlobusWSDLDebugExit();
    return res;
}

static
JSBool
partGetter(
    JSContext *                         cx,
    JSObject *                          obj,
    jsval                               jid,
    jsval *                             vp)
{
    int                                 idval;
    JSString *                          str = NULL;
    JSObject *                          typeObj = NULL;
    JSObject *                          elementObj = NULL;
    JSBool                              res = JS_TRUE;
    wsdlPartPtr                         part = NULL;
    GlobusFuncName(partGetter);
    GlobusWSDLDebugEnter();
    
    part = (wsdlPartPtr) JS_GetPrivate(cx, obj);
    globus_assert(part);

    if(!JSVAL_IS_INT(jid))
    {
        JS_ReportError(cx, 
                       "Invalid id type (expected int) "
                       "for wsdl part property");
        res = JS_FALSE;
        goto exit;
    }

    idval = JSVAL_TO_INT(jid);

    switch(idval)
    {
    case name:

        if(!part->name)
        {
            *vp = JSVAL_VOID;
            break;
        }
        
        str = JS_InternString(cx, part->name);
        globus_assert(str);

        *vp = STRING_TO_JSVAL(str);
        break;

    case type:

        if(!part->type)
        {
            *vp = JSVAL_VOID;
            break;
        }
        
        typeObj = JS_NewObject(cx, &xsdType_class, xsdTypeClass, obj);
        globus_assert(typeObj);
        JS_SetPrivate(cx, typeObj, (void *)part->type);
        JS_DefineProperties(cx, typeObj, xsdType_properties);
        *vp = OBJECT_TO_JSVAL(typeObj);
        break;

    case element:

        if(!part->element)
        {
            *vp = JSVAL_VOID;
            break;
        }

        elementObj = JS_NewObject(cx, &xsdElement_class, xsdElementClass, obj);
        globus_assert(elementObj);
        JS_SetPrivate(cx, elementObj, (void *)part->element);
        JS_DefineProperties(cx, elementObj, xsdElement_properties);
        *vp = OBJECT_TO_JSVAL(elementObj);
        break;

    default:

        *vp = JSVAL_VOID;
        JS_ReportError(cx, "Unknown part property with id %d for part", idval);
        res = JS_FALSE;
        goto exit;
    }

  exit:

    GlobusWSDLDebugExit();
    return res;
}

static
JSBool
operationsGetter(
    JSContext *                         cx,
    JSObject *                          obj,
    jsval                               id,
    jsval *                             vp)
{
    wsdlOperationPtr                    operation;
    JSObject *                          operationObj;
    JSBool                              res = JS_TRUE;
    int                                 i = 0;
    GlobusFuncName(operationsGetter);
    GlobusWSDLDebugEnter();

    operation = (wsdlOperationPtr) JS_GetPrivate(cx, obj);
    globus_assert(operation);

    if(JSVAL_IS_STRING(id))
    {
        while(operation)
        {
            JSString *                  name;
            
            name = JS_InternString(cx, operation->name);
            if(JS_CompareStrings(name, JSVAL_TO_STRING(id)))
            {
                break;
            }

            operation = operation->next;
        }
    }
    else if(JSVAL_IS_INT(id))
    {
        for(; i < JSVAL_TO_INT(id) && operation; ++i)
        {
            operation = operation->next;
        }
        globus_assert(operation);
    }
    else
    {
        operation = (wsdlOperationPtr) JSVAL_TO_PRIVATE(id);
        globus_assert(operation);
    }

    operationObj = JS_NewObject(cx, &operation_class, 
                                operationClass, obj);
    globus_assert(operationObj);
    JS_SetPrivate(cx, operationObj, (void *)operation);
    JS_DefineProperties(cx, operationObj, operation_properties);
    *vp = OBJECT_TO_JSVAL(operationObj);

    GlobusWSDLDebugExit();
    return res;
}

static
JSBool
operationsEnumerate(
    JSContext *                         cx,
    JSObject *                          obj,
    JSIterateOp                         enum_op,
    jsval *                             statep,
    jsid *                              idp)
{
    JSBool                              res = JS_TRUE;
    wsdlOperationPtr                    operations;
    int                                 i = 0;
    int                                 index;
    GlobusFuncName(operationsEnumerate);
    GlobusWSDLDebugEnter();

    operations = (wsdlOperationPtr) JS_GetPrivate(cx, obj);
    globus_assert(operations);
    
    switch(enum_op)
    {
    case JSENUMERATE_INIT:
        
        *statep = INT_TO_JSVAL(i);
        break;

    case JSENUMERATE_NEXT:

        index = JSVAL_TO_INT(*statep);
        for(; i < index && operations; ++i)
        {
            operations = operations->next;
        }

        if(!operations)
        {
            *statep = JSVAL_NULL;
            *idp = JSVAL_ZERO;
            break;
        }

        *idp = *statep;
        *statep = INT_TO_JSVAL(index + 1);

        break;

    case JSENUMERATE_DESTROY:
            
        break;
    }

    GlobusWSDLDebugExit();
    return res;
}

static
JSBool
operationGetter(
    JSContext *                         cx,
    JSObject *                          obj,
    jsval                               jid,
    jsval *                             vp)
{
    JSString *                          str = NULL;
    JSObject *                          paramObj = NULL;
    JSObject *                          faultsObj = NULL;
    int                                 idval;
    JSBool                              res = JS_TRUE;
    wsdlOperationPtr                    operation = NULL;
    GlobusFuncName(operationGetter);
    GlobusWSDLDebugEnter();
    
    operation = (wsdlOperationPtr) JS_GetPrivate(cx, obj);
    globus_assert(operation);

    if(!JSVAL_IS_INT(jid))
    {
        JS_ReportError(cx, 
                       "Invalid id type (expected int) "
                       "for wsdl operation property");
        res = JS_FALSE;
        goto exit;
    }

    idval = JSVAL_TO_INT(jid);

    switch(idval)
    {
    case name:

        if(!operation->name)
        {
            *vp = JSVAL_VOID;
            break;
        }
        
        str = JS_InternString(cx, operation->name);
        globus_assert(str);

        *vp = STRING_TO_JSVAL(str);
        break;

    case type:

        *vp = INT_TO_JSVAL(operation->type);
        break;

    case input:

        if(!operation->input)
        {
            *vp = JSVAL_VOID;
            break;
        }
        
        paramObj = JS_NewObject(cx, &param_class, paramClass, obj);
        globus_assert(paramObj);
        JS_SetPrivate(cx, paramObj, (void *)operation->input);
        JS_DefineProperties(cx, paramObj, param_properties);
        *vp = OBJECT_TO_JSVAL(paramObj);
        break;

    case output:

        if(!operation->output)
        {
            *vp = JSVAL_VOID;
            break;
        }
        
        paramObj = JS_NewObject(cx, &param_class, paramClass, obj);
        globus_assert(paramObj);
        JS_SetPrivate(cx, paramObj, (void *)operation->output);
        JS_DefineProperties(cx, paramObj, param_properties);
        *vp = OBJECT_TO_JSVAL(paramObj);
        break;

    case faults:

        if(!operation->faults)
        {
            *vp = JSVAL_VOID;
            break;
        }
        
        faultsObj = JS_NewObject(cx, &faults_class, NULL, obj);
        globus_assert(faultsObj);

        JS_SetPrivate(cx, faultsObj, (void *)operation->faults);
        *vp = OBJECT_TO_JSVAL(faultsObj);
        break;

    default:

        *vp = JSVAL_VOID;
        JS_ReportError(cx, "Unknown property with id %d for part", idval);
        res = JS_FALSE;
        goto exit;
    }

  exit:

    GlobusWSDLDebugExit();
    return res;
}


static
JSBool
faultsGetter(
    JSContext *                         cx,
    JSObject *                          obj,
    jsval                               id,
    jsval *                             vp)
{
    wsdlFaultPtr                        fault;
    JSObject *                          faultObj;
    JSBool                              res = JS_TRUE;
    int                                 i = 0;
    GlobusFuncName(faultsGetter);
    GlobusWSDLDebugEnter();

    fault = (wsdlFaultPtr) JS_GetPrivate(cx, obj);
    globus_assert(fault);

    if(JSVAL_IS_STRING(id))
    {
        while(fault)
        {
            JSString *                  name;
            
            name = JS_InternString(cx, fault->name);
            if(JS_CompareStrings(name, JSVAL_TO_STRING(id)))
            {
                break;
            }

            fault = fault->next;
        }
    }
    else if(JSVAL_IS_INT(id))
    {
        for(; i < JSVAL_TO_INT(id) && fault; ++i)
        {
            fault = fault->next;
        }
        globus_assert(fault);
    }
    else
    {
        fault = (wsdlFaultPtr) JSVAL_TO_PRIVATE(id);
        globus_assert(fault);
    }

    faultObj = JS_NewObject(cx, &fault_class, 
                            faultClass, obj);
    globus_assert(faultObj);
    JS_SetPrivate(cx, faultObj, (void *)fault);
    JS_DefineProperties(cx, faultObj, fault_properties);
    *vp = OBJECT_TO_JSVAL(faultObj);

    GlobusWSDLDebugExit();
    return res;
}

static
JSBool
faultsEnumerate(
    JSContext *                         cx,
    JSObject *                          obj,
    JSIterateOp                         enum_op,
    jsval *                             statep,
    jsid *                              idp)
{
    JSBool                              res = JS_TRUE;
    wsdlFaultPtr                        faults;
    int                                 i = 0;
    int                                 index;
    GlobusFuncName(faultsEnumerate);
    GlobusWSDLDebugEnter();

    faults = (wsdlFaultPtr) JS_GetPrivate(cx, obj);
    globus_assert(faults);
    
    switch(enum_op)
    {
    case JSENUMERATE_INIT:
        
        *statep = INT_TO_JSVAL(i);
        break;

    case JSENUMERATE_NEXT:

        index = JSVAL_TO_INT(*statep);
        for(; i < index && faults; ++i)
        {
            faults = faults->next;
        }
            
        if(!faults)
        {
            *statep = JSVAL_NULL;
            *idp = JSVAL_ZERO;
            break;
        }

        *idp = *statep;
        *statep = INT_TO_JSVAL(index + 1);

        break;

    case JSENUMERATE_DESTROY:
            
        break;
    }

    GlobusWSDLDebugExit();
    return res;
}

static
JSBool
faultGetter(
    JSContext *                         cx,
    JSObject *                          obj,
    jsval                               jid,
    jsval *                             vp)
{
    int                                 idval;
    JSString *                          str;
    JSObject *                          messageObj;
    JSBool                              res = JS_TRUE;
    wsdlFaultPtr                        fault = NULL;
    GlobusFuncName(faultGetter);
    GlobusWSDLDebugEnter();
    
    fault = (wsdlFaultPtr) JS_GetPrivate(cx, obj);
    globus_assert(fault);

    if(!JSVAL_IS_INT(jid))
    {
        JS_ReportError(cx, 
                       "Invalid id type (expected int) "
                       "for wsdl fault property");
        res = JS_FALSE;
        goto exit;
    }

    idval = JSVAL_TO_INT(jid);

    switch(idval)
    {
    case name:

        if(!fault->name)
        {
            *vp = JSVAL_VOID;
            break;
        }
        
        str = JS_InternString(cx, fault->name);
        globus_assert(str);

        *vp = STRING_TO_JSVAL(str);
        break;

    case message:

        if(!fault->message)
        {
            *vp = JSVAL_VOID;
            break;
        }

        messageObj = JS_NewObject(cx, &message_class, messageClass, obj);
        globus_assert(messageObj);
        JS_SetPrivate(cx, messageObj, (void *)fault->message);
        JS_DefineProperties(cx, messageObj, message_properties);
        *vp = OBJECT_TO_JSVAL(messageObj);
        break;

    default:

        *vp = JSVAL_VOID;
        JS_ReportError(cx, "Unknown fault property with id %d for part", idval);
        res = JS_FALSE;
        goto exit;
    }

  exit:

    GlobusWSDLDebugExit();
    return res;
}


static
JSBool
paramGetter(
    JSContext *                         cx,
    JSObject *                          obj,
    jsval                               jid,
    jsval *                             vp)
{
    int                                 idval;
    JSString *                          str;
    JSObject *                          messageObj;
    JSBool                              res = JS_TRUE;
    wsdlParamPtr                        param = NULL;
    GlobusFuncName(paramGetter);
    GlobusWSDLDebugEnter();
    
    param = (wsdlParamPtr) JS_GetPrivate(cx, obj);
    globus_assert(param);

    if(!JSVAL_IS_INT(jid))
    {
        JS_ReportError(cx, 
                       "Invalid id type (expected int) "
                       "for wsdl param property");
        res = JS_FALSE;
        goto exit;
    }

    idval = JSVAL_TO_INT(jid);

    switch(idval)
    {
    case name:

        if(!param->name)
        {
            *vp = JSVAL_VOID;
            break;
        }
        
        str = JS_InternString(cx, param->name);
        globus_assert(str);

        *vp = STRING_TO_JSVAL(str);
        break;

    case Action:

        if(!param->Action)
        {
            *vp = JSVAL_VOID;
            break;
        }

        str = JS_InternString(cx, param->Action);
        globus_assert(str);
        
        *vp = STRING_TO_JSVAL(str);
        break;

    case message:

        if(!param->message)
        {
            *vp = JSVAL_VOID;
            break;
        }

        messageObj = JS_NewObject(cx, &message_class, messageClass, obj);
        globus_assert(messageObj);
        JS_SetPrivate(cx, messageObj, (void *)param->message);
        JS_DefineProperties(cx, messageObj, message_properties);
        *vp = OBJECT_TO_JSVAL(messageObj);
        break;

    default:

        *vp = JSVAL_VOID;
        JS_ReportError(cx, "Unknown param property with id %d for part", idval);
        res = JS_FALSE;
        goto exit;
    }

  exit:

    GlobusWSDLDebugExit();
    return res;
}

static
JSBool
portsGetter(
    JSContext *                         cx,
    JSObject *                          obj,
    jsval                               id,
    jsval *                             vp)
{
    wsdlPortPtr                         port;
    JSObject *                          portObj;
    JSBool                              res = JS_TRUE;
    int                                 i = 0;
    GlobusFuncName(portsGetter);
    GlobusWSDLDebugEnter();

    port = (wsdlPortPtr) JS_GetPrivate(cx, obj);
    globus_assert(port);

    if(JSVAL_IS_STRING(id))
    {
        while(port)
        {
            JSString *                  name;
            
            name = JS_InternString(cx, port->name);
            if(JS_CompareStrings(name, JSVAL_TO_STRING(id)))
            {
                break;
            }

            port = port->next;
        }
    }
    else if(JSVAL_IS_INT(id))
    {
        for(; i < JSVAL_TO_INT(id) && port; ++i)
        {
            port = port->next;
        }
        globus_assert(port);
    }
    else
    {
        port = (wsdlPortPtr) JSVAL_TO_PRIVATE(id);
        globus_assert(port);
    }

    portObj = JS_NewObject(cx, &port_class, 
                           portClass, obj);
    globus_assert(portObj);
    JS_SetPrivate(cx, portObj, (void *)port);
    JS_DefineProperties(cx, portObj, port_properties);
    *vp = OBJECT_TO_JSVAL(portObj);

    GlobusWSDLDebugExit();
    return res;
}

static
JSBool
portsEnumerate(
    JSContext *                         cx,
    JSObject *                          obj,
    JSIterateOp                         enum_op,
    jsval *                             statep,
    jsid *                              idp)
{
    JSBool                              res = JS_TRUE;
    wsdlPortPtr                         ports;
    int                                 i = 0;
    int                                 index;
    GlobusFuncName(portsEnumerate);
    GlobusWSDLDebugEnter();

    ports = (wsdlPortPtr) JS_GetPrivate(cx, obj);
    globus_assert(ports);
    
    switch(enum_op)
    {
    case JSENUMERATE_INIT:
        
        *statep = INT_TO_JSVAL(i);
        break;

    case JSENUMERATE_NEXT:

        index = JSVAL_TO_INT(*statep);
        for(; i < index && ports; ++i)
        {
            ports = ports->next;
        }

        if(!ports)
        {
            *statep = JSVAL_NULL;
            *idp = JSVAL_ZERO;
            break;
        }

        *idp = *statep;
        *statep = INT_TO_JSVAL(index + 1);

        break;

    case JSENUMERATE_DESTROY:
            
        break;
    }

    GlobusWSDLDebugExit();
    return res;
}

static
JSBool
portGetter(
    JSContext *                         cx,
    JSObject *                          obj,
    jsval                               jid,
    jsval *                             vp)
{
    int                                 idval;
    JSString *                          str = NULL;
    JSObject *                          bindingObj = NULL;
    JSObject *                          soapObj = NULL;
    JSBool                              res = JS_TRUE;
    wsdlPortPtr                         port = NULL;
    GlobusFuncName(portGetter);
    GlobusWSDLDebugEnter();
    
    port = (wsdlPortPtr) JS_GetPrivate(cx, obj);
    globus_assert(port);

    if(!JSVAL_IS_INT(jid))
    {
        JS_ReportError(cx, 
                       "Invalid id type (expected int) "
                       "for wsdl port property");
        res = JS_FALSE;
        goto exit;
    }

    idval = JSVAL_TO_INT(jid);

    switch(idval)
    {
    case name:

        if(!port->name)
        {
            *vp = JSVAL_VOID;
            break;
        }
        
        str = JS_InternString(cx, port->name);
        globus_assert(str);

        *vp = STRING_TO_JSVAL(str);
        break;

    case binding:

        if(!port->binding)
        {
            *vp = JSVAL_VOID;
            break;
        }

        bindingObj = JS_NewObject(cx, &binding_class, bindingClass, obj);
        globus_assert(bindingObj);
        JS_SetPrivate(cx, bindingObj, (void *)port->binding);
        JS_DefineProperties(cx, bindingObj, binding_properties);
        
        *vp = OBJECT_TO_JSVAL(bindingObj);
        break;

    case soap:

        if(!port->extension || (port->extensionType != SOAP_BINDING))
        {
            *vp = JSVAL_VOID;
            break;
        }

        soapObj = JS_NewObject(cx, &soapAddress_class, soapAddressClass, obj);
        globus_assert(soapObj);
        JS_SetPrivate(cx, soapObj, (void *)port->extension);
        JS_DefineProperties(cx, soapObj, soapAddress_properties);
        *vp = OBJECT_TO_JSVAL(soapObj);
        break;

    default:

        *vp = JSVAL_VOID;
        JS_ReportError(cx, "Unknown port property with id %d for part", idval);
        res = JS_FALSE;
        goto exit;
    }

  exit:

    GlobusWSDLDebugExit();
    return res;
}

static
JSBool
serviceGetter(
    JSContext *                         cx,
    JSObject *                          obj,
    jsval                               jid,
    jsval *                             vp)
{
    int                                 idval;
    JSString *                          str = NULL;
    JSObject *                          portsObj = NULL;
    JSBool                              res = JS_TRUE;
    wsdlServicePtr                      service = NULL;
    GlobusFuncName(serviceGetter);
    GlobusWSDLDebugEnter();
    
    service = (wsdlServicePtr) JS_GetPrivate(cx, obj);
    globus_assert(service);

    if(!JSVAL_IS_INT(jid))
    {
        char *                          prop_name;

        prop_name = JS_GetStringBytes(JSVAL_TO_STRING(jid));
        JS_ReportError(cx, 
                       "Invalid property access .%s on wsdl service element",
                       prop_name);
        res = JS_FALSE;
        goto exit;
    }

    idval = JSVAL_TO_INT(jid);

    switch(idval)
    {
    case name:

        if(!service->name)
        {
            *vp = JSVAL_VOID;
            break;
        }
        
        str = JS_InternString(cx, service->name);
        globus_assert(str);

        *vp = STRING_TO_JSVAL(str);
        break;

    case ports:

        if(!service->ports)
        {
            *vp = JSVAL_VOID;
            break;
        }

        portsObj = JS_NewObject(cx, &ports_class, NULL, obj);
        globus_assert(portsObj);
        JS_SetPrivate(cx, portsObj, (void *) service->ports);
        *vp = OBJECT_TO_JSVAL(portsObj);
        break;

    case soap:

        *vp = JSVAL_VOID;
        break;
        
    case targetNamespace:

        if(!service->schema || !service->schema->targetNamespace)
        {
            *vp = JSVAL_VOID;
            break;
        }

        str = JS_InternString(cx, service->schema->targetNamespace);
        globus_assert(str);
        
        *vp = STRING_TO_JSVAL(str);
        break;

    default:

        *vp = JSVAL_VOID;
        JS_ReportError(cx, "Unknown service property with id %d for part", idval);
        res = JS_FALSE;
        goto exit;
    }

  exit:

    GlobusWSDLDebugExit();
    return res;
}

static
JSBool
soapBindingGetter(
    JSContext *                         cx,
    JSObject *                          obj,
    jsval                               jid,
    jsval *                             vp)
{
    JSString *                          str = NULL;
    int                                 idval;
    JSBool                              res = JS_TRUE;
    soapBindingPtr                      binding = NULL;
    GlobusFuncName(soapBindingGetter);
    GlobusWSDLDebugEnter();
    
    binding = (soapBindingPtr) JS_GetPrivate(cx, obj);
    globus_assert(binding);

    if(!JSVAL_IS_INT(jid))
    {
        JS_ReportError(cx, 
                       "Invalid id type (expected int) "
                       "for soap binding property");
        res = JS_FALSE;
        goto exit;
    }

    idval = JSVAL_TO_INT(jid);

    switch(idval)
    {
    case style:

        *vp = INT_TO_JSVAL(binding->style);
        break;

    case transport:

        if(!binding->transport)
        {
            *vp = JSVAL_VOID;
            break;
        }

        str = JS_InternString(cx, binding->transport);
        globus_assert(str);
        *vp = STRING_TO_JSVAL(str);
        break;

    default:

        *vp = JSVAL_VOID;
        JS_ReportError(cx, 
                       "Unknown soap binding property "
                       "with id %d for part", idval);
        res = JS_FALSE;
        goto exit;
    }

  exit:

    GlobusWSDLDebugExit();
    return res;
}

static
JSBool
soapOperationGetter(
    JSContext *                         cx,
    JSObject *                          obj,
    jsval                               jid,
    jsval *                             vp)
{
    JSString *                          str = NULL;
    int                                 idval;
    JSBool                              res = JS_TRUE;
    soapOperationPtr                    operation = NULL;
    GlobusFuncName(soapOperationGetter);
    GlobusWSDLDebugEnter();

    operation = (soapOperationPtr) JS_GetPrivate(cx, obj);
    globus_assert(operation);

    if(!JSVAL_IS_INT(jid))
    {
        JS_ReportError(cx, 
                       "Invalid id type (expected int) "
                       "for soap operation property");
        res = JS_FALSE;
        goto exit;
    }

    idval = JSVAL_TO_INT(jid);

    switch(idval)
    {
    case soapAction:
        
        if(!operation->soapAction)
        {
            *vp = JSVAL_VOID;
            break;
        }

        str = JS_InternString(cx, operation->soapAction);
        globus_assert(str);
        *vp = STRING_TO_JSVAL(str);
        break;

    case style:

        *vp = INT_TO_JSVAL(operation->style);
        break;


    default:

        *vp = JSVAL_VOID;
        JS_ReportError(cx, 
                       "Unknown soap operation property "
                       "with id %d", idval);
        res = JS_FALSE;
        goto exit;
    }

  exit:

    GlobusWSDLDebugExit();
    return res;
}

static
JSBool
soapHeaderGetter(
    JSContext *                         cx,
    JSObject *                          obj,
    jsval                               jid,
    jsval *                             vp)
{
    JSString *                          str = NULL;
    int                                 idval;
    JSBool                              res = JS_TRUE;
    soapHeaderPtr                       header = NULL;
    GlobusFuncName(soapHeaderGetter);
    GlobusWSDLDebugEnter();

    header = (soapHeaderPtr) JS_GetPrivate(cx, obj);
    globus_assert(header);

    if(!JSVAL_IS_INT(jid))
    {
        JS_ReportError(cx, 
                       "Invalid id type (expected int) "
                       "for soap header property");
        res = JS_FALSE;
        goto exit;
    }

    idval = JSVAL_TO_INT(jid);

    switch(idval)
    {
    case message:
        
        if(!header->message)
        {
            *vp = JSVAL_VOID;
            break;
        }

        str = JS_InternString(cx, header->message);
        globus_assert(str);
        *vp = STRING_TO_JSVAL(str);
        break;

    case parts:

        if(!header->parts)
        {
            *vp = JSVAL_VOID;
            break;
        }

        str = JS_InternString(cx, header->parts);
        globus_assert(str);
        *vp = STRING_TO_JSVAL(str);
        break;

    case use:

        *vp = INT_TO_JSVAL(header->use);
        break;

    case encodingStyle:

        if(!header->encodingStyle)
        {
            *vp = JSVAL_VOID;
            break;
        }

        str = JS_InternString(cx, header->encodingStyle);
        globus_assert(str);
        *vp = STRING_TO_JSVAL(str);
        break;

    case Namespace:

        if(!header->Namespace)
        {
            *vp = JSVAL_VOID;
        }

        str = JS_InternString(cx, header->Namespace);
        globus_assert(str);
        *vp = STRING_TO_JSVAL(str);
        break;

    default:

        *vp = JSVAL_VOID;
        JS_ReportError(cx, 
                       "Unknown soap header property "
                       "with id %d", idval);
        res = JS_FALSE;
        goto exit;
    }

  exit:

    GlobusWSDLDebugExit();
    return res;
}

static
JSBool
soapBodyGetter(
    JSContext *                         cx,
    JSObject *                          obj,
    jsval                               jid,
    jsval *                             vp)
{
    JSString *                          str = NULL;
    int                                 idval;
    JSBool                              res = JS_TRUE;
    soapBodyPtr                       body = NULL;
    GlobusFuncName(soapBodyGetter);
    GlobusWSDLDebugEnter();

    body = (soapBodyPtr) JS_GetPrivate(cx, obj);
    globus_assert(body);

    if(!JSVAL_IS_INT(jid))
    {
        JS_ReportError(cx, 
                       "Invalid id type (expected int) "
                       "for soap body property");
        res = JS_FALSE;
        goto exit;
    }

    idval = JSVAL_TO_INT(jid);

    switch(idval)
    {
    case parts:

        if(!body->parts)
        {
            *vp = JSVAL_VOID;
            break;
        }

        str = JS_InternString(cx, body->parts);
        globus_assert(str);
        *vp = STRING_TO_JSVAL(str);
        break;

    case use:

        *vp = INT_TO_JSVAL(body->use);
        break;

    case encodingStyle:

        if(!body->encodingStyle)
        {
            *vp = JSVAL_VOID;
            break;
        }

        str = JS_InternString(cx, body->encodingStyle);
        globus_assert(str);
        *vp = STRING_TO_JSVAL(str);
        break;

    case Namespace:

        if(!body->Namespace)
        {
            *vp = JSVAL_VOID;
        }

        str = JS_InternString(cx, body->Namespace);
        globus_assert(str);
        *vp = STRING_TO_JSVAL(str);
        break;

    default:

        *vp = JSVAL_VOID;
        JS_ReportError(cx, 
                       "Unknown soap body property "
                       "with id %d", idval);
        res = JS_FALSE;
        goto exit;
    }

  exit:

    GlobusWSDLDebugExit();
    return res;
}

static
JSBool
soapFaultGetter(
    JSContext *                         cx,
    JSObject *                          obj,
    jsval                               jid,
    jsval *                             vp)
{
    JSString *                          str = NULL;
    int                                 idval;
    JSBool                              res = JS_TRUE;
    soapFaultPtr                       fault = NULL;
    GlobusFuncName(soapFaultGetter);
    GlobusWSDLDebugEnter();

    fault = (soapFaultPtr) JS_GetPrivate(cx, obj);
    globus_assert(fault);

    if(!JSVAL_IS_INT(jid))
    {
        JS_ReportError(cx, 
                       "Invalid id type (expected int) "
                       "for soap fault property");
        res = JS_FALSE;
        goto exit;
    }

    idval = JSVAL_TO_INT(jid);

    switch(idval)
    {
    case use:

        *vp = INT_TO_JSVAL(fault->use);
        break;

    case encodingStyle:

        if(!fault->encodingStyle)
        {
            *vp = JSVAL_VOID;
            break;
        }

        str = JS_InternString(cx, fault->encodingStyle);
        globus_assert(str);
        *vp = STRING_TO_JSVAL(str);
        break;

    case Namespace:

        if(!fault->Namespace)
        {
            *vp = JSVAL_VOID;
        }

        str = JS_InternString(cx, fault->Namespace);
        globus_assert(str);
        *vp = STRING_TO_JSVAL(str);
        break;

    default:

        *vp = JSVAL_VOID;
        JS_ReportError(cx, 
                       "Unknown soap fault property "
                       "with id %d", idval);
        res = JS_FALSE;
        goto exit;
    }

  exit:

    GlobusWSDLDebugExit();
    return res;
}

static
JSBool
soapAddressGetter(
    JSContext *                         cx,
    JSObject *                          obj,
    jsval                               jid,
    jsval *                             vp)
{
    JSString *                          str = NULL;
    int                                 idval;
    JSBool                              res = JS_TRUE;
    soapAddressPtr                       address = NULL;
    GlobusFuncName(soapAddressGetter);
    GlobusWSDLDebugEnter();

    address = (soapAddressPtr) JS_GetPrivate(cx, obj);
    globus_assert(address);

    if(!JSVAL_IS_INT(jid))
    {
        JS_ReportError(cx, 
                       "Invalid id type (expected int) "
                       "for soap address property");
        res = JS_FALSE;
        goto exit;
    }

    idval = JSVAL_TO_INT(jid);

    switch(idval)
    {
    case location:

        if(!address->location)
        {
            *vp = JSVAL_VOID;
            break;
        }

        str = JS_InternString(cx, address->location);
        globus_assert(str);
        *vp = STRING_TO_JSVAL(str);
        break;

    default:

        *vp = JSVAL_VOID;
        JS_ReportError(cx, 
                       "Unknown soap address property "
                       "with id %d", idval);
        res = JS_FALSE;
        goto exit;
    }

  exit:

    GlobusWSDLDebugExit();
    return res;
}
    
static
JSBool
soapBindingParamGetter(
    JSContext *                         cx,
    JSObject *                          obj,
    jsval                               jid,
    jsval *                             vp)
{
    JSObject *                          newObj = NULL;
    int                                 idval;
    JSBool                              res = JS_TRUE;
    soapBindingParamPtr                 param = NULL;
    GlobusFuncName(soapBindingParamGetter);
    GlobusWSDLDebugEnter();
    
    param = (soapBindingParamPtr) JS_GetPrivate(cx, obj);
    globus_assert(param);

    if(!JSVAL_IS_INT(jid))
    {
        JS_ReportError(cx, 
                       "Invalid id type (expected int) "
                       "for soap binding param property");
        res = JS_FALSE;
        goto exit;
    }

    idval = JSVAL_TO_INT(jid);

    switch(idval)
    {
    case header:

        if(!param->header)
        {
            *vp = JSVAL_VOID;
            break;
        }

        newObj = JS_NewObject(cx, &soapHeader_class, soapHeaderClass, obj);
        globus_assert(newObj);
        JS_SetPrivate(cx, newObj, (void *)param->header);
        JS_DefineProperties(cx, newObj, soapHeader_properties);
        *vp = OBJECT_TO_JSVAL(newObj);
        break;

    case body:

        if(!param->body)
        {
            *vp = JSVAL_VOID;
            break;
        }

        newObj = JS_NewObject(cx, &soapBody_class, soapBodyClass, obj);
        globus_assert(newObj);
        JS_SetPrivate(cx, newObj, (void *)param->body);
        JS_DefineProperties(cx, newObj, soapBody_properties);
        *vp = OBJECT_TO_JSVAL(newObj);
        break;

    case fault:

        if(!param->fault)
        {
            *vp = JSVAL_VOID;
            break;
        }

        newObj = JS_NewObject(cx, &soapFault_class, soapFaultClass, obj);
        globus_assert(newObj);
        JS_SetPrivate(cx, newObj, (void *)param->fault);
        JS_DefineProperties(cx, newObj, soapFault_properties);
        *vp = OBJECT_TO_JSVAL(newObj);
        break;

    default:

        *vp = JSVAL_VOID;
        JS_ReportError(cx, 
                       "Unknown soap binding property "
                       "with id %d for part", idval);
        res = JS_FALSE;
        goto exit;
    }

  exit:

    GlobusWSDLDebugExit();
    return res;
}

static
void
globus_i_wsdl_parser_import_scanner(
    void *                              payload,
    void *                              data,
    xmlChar *                           name)
{
    xmlSchemaImportPtr                  import;
    xmlSchemaPtr                        schema;
    ScannerData *                       dt;
    GlobusFuncName(globus_i_wsdl_parser_import_scanner);
    GlobusWSDLDebugEnter();
    
    dt = (ScannerData *) data;
    import = (xmlSchemaImportPtr) payload;

    if(dt->result != GLOBUS_SUCCESS)
    {
        goto exit;
    }

    schema = import->schema;

    if(schema)
    {
        dt->xsdSchema = schema;
        xmlHashScan(schema->schemasImports, 
                    globus_i_wsdl_parser_import_scanner, data);
        if(dt->result != GLOBUS_SUCCESS)
        {
            dt->result = GlobusWSDLErrorJSLoadSchema(dt->result);
            goto exit;
        }

        dt->xsdSchema = schema;
        xmlHashScan(schema->typeDecl, globus_i_wsdl_parser_type_scanner, data);
        if(dt->result != GLOBUS_SUCCESS)
        {
            dt->result = GlobusWSDLErrorJSLoadSchema(dt->result);
            goto exit;
        }

        xmlHashScan(schema->groupDecl, globus_i_wsdl_parser_type_scanner, data);
        if(dt->result != GLOBUS_SUCCESS)
        {
            dt->result = GlobusWSDLErrorJSLoadSchema(dt->result);
            goto exit;
        }

        xmlHashScan(schema->elemDecl, 
                    globus_i_wsdl_parser_element_scanner, 
                    data);
        if(dt->result != GLOBUS_SUCCESS)
        {
            dt->result = GlobusWSDLErrorJSLoadSchema(dt->result);
            goto exit;
        }

        xmlHashScan(schema->attrDecl, 
                    globus_i_wsdl_parser_attribute_scanner, 
                    data);
        if(dt->result != GLOBUS_SUCCESS)
        {
            dt->result = GlobusWSDLErrorJSLoadSchema(dt->result);
            goto exit;
        }
    }

 exit:

    GlobusWSDLDebugExit();
}

static
void
globus_i_wsdl_parser_type_scanner(
    void *                              payload,
    void *                              data,
    xmlChar *                           name)
{
    xmlSchemaTypePtr                    xsdType;
    wsdlSchemaPtr                       schema;
    xmlSchemaPtr                        xsdSchema;
    JSContext *                         cx;
    jsval                               val;
    JSObject *                          typeObj;
    JSObject *                          types;
    JSObject *                          global;
    JSString *                          str;
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_i_wsdl_parser_type_scanner);
    GlobusWSDLDebugEnter();

    xsdType = (xmlSchemaTypePtr) payload;
    schema = ((ScannerData *)data)->schema;
    xsdSchema = ((ScannerData *)data)->xsdSchema;
    cx = ((ScannerData *)data)->context;
    global = ((ScannerData *)data)->global;
    types = ((ScannerData *)data)->types;
    result = ((ScannerData *)data)->result;

    if(result != GLOBUS_SUCCESS)
    {
        goto exit;
    }

    if(xsdType->targetNamespace)
    {
        str = globus_l_wsdl_jscript_qname_string(
            cx, xsdType->targetNamespace, xsdType->name);
    }
    else if(xsdSchema->targetNamespace)
    {
        xsdType->targetNamespace = xsdSchema->targetNamespace;
        str = globus_l_wsdl_jscript_qname_string(
            cx, xsdSchema->targetNamespace, xsdType->name);
    }
    else if(schema->targetNamespace)
    {
        xsdType->targetNamespace = schema->targetNamespace;
        str = globus_l_wsdl_jscript_qname_string(
            cx, schema->targetNamespace, xsdType->name);
    }
    else
    {
        JS_ReportError(cx, 
                       "Can't find namespace for type: %s", 
                       xsdType->name);
        result = GlobusWSDLErrorScanningTypes(
            GLOBUS_NULL, xsdType->name);
        goto exit;
    }

    typeObj = JS_NewObject(cx, &xsdType_class, xsdTypeClass, types);
    globus_assert(typeObj);
    JS_SetPrivate(cx, typeObj, (void *)xsdType);
    JS_DefineProperties(cx, typeObj, xsdType_properties);
    
    val = OBJECT_TO_JSVAL(typeObj);
    
    if(!JS_DefineProperty(
                      cx, types, (const char *)JS_GetStringBytes(str), 
                      val, NULL, NULL, 
                      JSPROP_ENUMERATE|JSPROP_READONLY|JSPROP_PERMANENT))
    {
        JS_ReportError(cx, "Failed to define type from parsed WSDL");
        result = GlobusWSDLErrorScanningTypes(
            GLOBUS_NULL, xsdType->name);
        goto exit;
    }

exit:

    GlobusWSDLDebugExit();
    ((ScannerData *)data)->result = result;
}    

static
void
globus_i_wsdl_parser_element_scanner(
    void *                              payload,
    void *                              data,
    xmlChar *                           name)
{
    xmlSchemaElementPtr                 element;
    wsdlSchemaPtr                       schema;
    xmlSchemaPtr                        xsdSchema;
    JSContext *                         cx;
    jsval                               val;
    JSObject *                          elements;
    JSObject *                          local_elements;
    JSObject *                          global;
    JSObject *                          elemObj;
    JSString *                          str;
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_i_wsdl_parser_element_scanner);
    GlobusWSDLDebugEnter();
    
    element = (xmlSchemaElementPtr) payload;
    schema = ((ScannerData *)data)->schema;
    xsdSchema = ((ScannerData *)data)->xsdSchema;
    cx = ((ScannerData *)data)->context;
    global = ((ScannerData *)data)->global;
    elements = ((ScannerData *)data)->elements;
    local_elements = ((ScannerData *)data)->local_elements;
    result = ((ScannerData *)data)->result;

    if(result != GLOBUS_SUCCESS)
    {
        goto exit;
    }


    if(!element->targetNamespace)
    {
        if((element->flags & XML_SCHEMAS_ELEM_GLOBAL) ||
           (xsdSchema->flags & XML_SCHEMAS_QUALIF_ELEM))
        {
            element->targetNamespace = xsdSchema->targetNamespace;
        }
    }
        
    str = globus_l_wsdl_jscript_qname_string(
        cx, element->targetNamespace, element->name);

    if(element->flags & XML_SCHEMAS_ELEM_GLOBAL)
    {
        elemObj = JS_NewObject(cx, &xsdElement_class, 
                               xsdElementClass, elements);
        globus_assert(elemObj);
        JS_SetPrivate(cx, elemObj, (void *)element);
        JS_DefineProperties(cx, elemObj, xsdElement_properties);
    
        val = OBJECT_TO_JSVAL(elemObj);
    
        if(!JS_DefineProperty(
               cx, elements, (const char *) JS_GetStringBytes(str), 
               val, NULL, NULL, 
               JSPROP_ENUMERATE|JSPROP_READONLY|JSPROP_PERMANENT))
        {
            JS_ReportError(cx, "Failed to define element from parsed WSDL");
            result = GlobusWSDLErrorScanningElements(
                GLOBUS_NULL, element->name);
            goto exit;
        }
    }
    else
    {
        elemObj = JS_NewObject(cx, &xsdElement_class, 
                               xsdElementClass, local_elements);
        globus_assert(elemObj);
        JS_SetPrivate(cx, elemObj, (void *)element);
        JS_DefineProperties(cx, elemObj, xsdElement_properties);
    
        val = OBJECT_TO_JSVAL(elemObj);
    
        if(!JS_DefineProperty(
               cx, local_elements, (const char *) JS_GetStringBytes(str), 
               val, NULL, NULL, 
               JSPROP_ENUMERATE|JSPROP_READONLY|JSPROP_PERMANENT))
        {
            JS_ReportError(cx, "Failed to define element from parsed WSDL");
            result = GlobusWSDLErrorScanningElements(
                GLOBUS_NULL, element->name);
            goto exit;
        }
    }

exit:

    GlobusWSDLDebugExit();
    ((ScannerData *)data)->result = result;
}    

static
void
globus_i_wsdl_parser_attribute_scanner(
    void *                              payload,
    void *                              data,
    xmlChar *                           name)
{
    xmlSchemaAttributePtr               attribute;
    wsdlSchemaPtr                       schema;
    xmlSchemaPtr                        xsdSchema;
    JSContext *                         cx;
    jsval                               val;
    JSObject *                          attributes;
    JSObject *                          local_attributes;
    JSObject *                          global;
    JSObject *                          attrObj;
    JSString *                          str;
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_i_wsdl_parser_attribute_scanner);
    GlobusWSDLDebugEnter();
    
    attribute = (xmlSchemaAttributePtr) payload;
    schema = ((ScannerData *)data)->schema;
    xsdSchema = ((ScannerData *)data)->xsdSchema;
    cx = ((ScannerData *)data)->context;
    global = ((ScannerData *)data)->global;
    attributes = ((ScannerData *)data)->attributes;
    local_attributes = ((ScannerData *)data)->local_attributes;
    result = ((ScannerData *)data)->result;

    if(result != GLOBUS_SUCCESS)
    {
        goto exit;
    }

    if(!attribute->targetNamespace)
    {
        if((attribute->flags & XML_SCHEMAS_ATTR_GLOBAL) ||
           (xsdSchema->flags & XML_SCHEMAS_QUALIF_ATTR))
        {
            attribute->targetNamespace = xsdSchema->targetNamespace;
        }
    }
    
    str = globus_l_wsdl_jscript_qname_string(
        cx, attribute->targetNamespace, attribute->name);

    if(attribute->flags & XML_SCHEMAS_ATTR_GLOBAL)
    {
        attrObj = JS_NewObject(
            cx, &xsdAttr_class, xsdAttrClass, attributes);
        globus_assert(attrObj);
        JS_SetPrivate(cx, attrObj, (void *)attribute);
        JS_DefineProperties(cx, attrObj, xsdAttr_properties);

        val = OBJECT_TO_JSVAL(attrObj);

        JS_DefineProperty(
            cx, attributes, (const char *) JS_GetStringBytes(str), 
            val, NULL, NULL, 
            JSPROP_ENUMERATE|JSPROP_READONLY|JSPROP_PERMANENT);
    }
    else
    {
        attrObj = JS_NewObject(
            cx, &xsdAttr_class, xsdAttrClass, local_attributes);
        globus_assert(attrObj);
        JS_SetPrivate(cx, attrObj, (void *)attribute);
        JS_DefineProperties(cx, attrObj, xsdAttr_properties);

        val = OBJECT_TO_JSVAL(attrObj);

        JS_DefineProperty(
            cx, local_attributes, (const char *) JS_GetStringBytes(str), 
            val, NULL, NULL, 
            JSPROP_ENUMERATE|JSPROP_READONLY|JSPROP_PERMANENT);
    }
        
 exit:
    
    GlobusWSDLDebugExit();
    ((ScannerData *)data)->result = result;
}

static
void
globus_i_wsdl_parser_message_scanner(
    void *                              payload,
    void *                              data,
    xmlChar *                           name)
{
    wsdlMessagePtr                      message;
    JSContext *                         cx;
    jsval                               val;
    JSObject *                          messages;
    JSObject *                          messageObj;
    JSObject *                          global;
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_i_wsdl_parser_message_scanner);
    GlobusWSDLDebugEnter();
    
    message = (wsdlMessagePtr) payload;
    cx = ((ScannerData *)data)->context;
    global = ((ScannerData *)data)->global;
    messages = ((ScannerData *)data)->messages;
    result = ((ScannerData *)data)->result;

    if(result != GLOBUS_SUCCESS)
    {
        goto exit;
    }

    messageObj = JS_NewObject(cx, &message_class, messageClass, messages);
    globus_assert(messageObj);
    JS_SetPrivate(cx, messageObj, (void *)message);
    JS_DefineProperties(cx, messageObj, message_properties);

    val = OBJECT_TO_JSVAL(messageObj);

    if(!JS_DefineProperty(cx, messages, (const char *)name, val, NULL, NULL, 
                          JSPROP_ENUMERATE|JSPROP_READONLY|JSPROP_PERMANENT))
    {
        JS_ReportError(cx, "Failed to define message from parsed WSDL");
        result = GlobusWSDLErrorScanningMessages(GLOBUS_NULL, message->name);
        goto exit;
    }


exit:

    GlobusWSDLDebugExit();
    ((ScannerData *)data)->result = result;
}    

static
void
globus_i_wsdl_parser_portType_scanner(
    void *                              payload,
    void *                              data,
    xmlChar *                           name)
{
    wsdlPortTypePtr                     portType;
    JSContext *                         cx;
    jsval                               val;
    JSObject *                          portTypes;
    JSObject *                          global;
    JSObject *                          portTypeObj;
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_i_wsdl_parser_portType_scanner);
    GlobusWSDLDebugEnter();
    
    portType = (wsdlPortTypePtr) payload;
    cx = ((ScannerData *)data)->context;
    global = ((ScannerData *)data)->global;
    portTypes = ((ScannerData *)data)->portTypes;
    result = ((ScannerData *)data)->result;

    if(result != GLOBUS_SUCCESS)
    {
        goto exit;
    }

    portTypeObj = JS_NewObject(cx, &portType_class, portTypeClass, portTypes);
    globus_assert(portTypeObj);
    JS_SetPrivate(cx, portTypeObj, (void *)portType);
    JS_DefineProperties(cx, portTypeObj, portType_properties);

    val = OBJECT_TO_JSVAL(portTypeObj);

    if(!JS_DefineProperty(cx, portTypes, (const char *)name, val, NULL, NULL, 
                          JSPROP_ENUMERATE|JSPROP_READONLY|JSPROP_PERMANENT))
    {
        JS_ReportError(cx, "Failed to define portType from parsed WSDL");
        result = GlobusWSDLErrorScanningPortTypes(GLOBUS_NULL, portType->name);
        goto exit;
    }


exit:

    GlobusWSDLDebugExit();
    ((ScannerData *)data)->result = result;
}    

static
void
globus_i_wsdl_parser_binding_scanner(
    void *                              payload,
    void *                              data,
    xmlChar *                           name)
{
    wsdlBindingPtr                      binding;
    JSContext *                         cx;
    jsval                               val;
    JSObject *                          bindings;
    JSObject *                          global;
    JSObject *                          bindingObj;
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_i_wsdl_parser_binding_scanner);
    GlobusWSDLDebugEnter();
    
    binding = (wsdlBindingPtr) payload;
    cx = ((ScannerData *)data)->context;
    global = ((ScannerData *)data)->global;
    bindings = ((ScannerData *)data)->bindings;
    result = ((ScannerData *)data)->result;

    if(result != GLOBUS_SUCCESS)
    {
        goto exit;
    }

    bindingObj = JS_NewObject(cx, &binding_class, bindingClass, bindings);
    globus_assert(bindingObj);
    JS_SetPrivate(cx, bindingObj, (void *)binding);

    val = OBJECT_TO_JSVAL(bindingObj);

    if(!JS_DefineProperty(cx, bindings, (const char *)name, val, NULL, NULL, 
                          JSPROP_ENUMERATE|JSPROP_READONLY|JSPROP_PERMANENT))
    {
        JS_ReportError(cx, "Failed to define binding from parsed WSDL");
        result = GlobusWSDLErrorScanningBindings(GLOBUS_NULL, binding->name);
        goto exit;
    }


exit:

    GlobusWSDLDebugExit();
    ((ScannerData *)data)->result = result;
}    

static
void
globus_i_wsdl_parser_service_scanner(
    void *                              payload,
    void *                              data,
    xmlChar *                           name)
{
    wsdlServicePtr                      service = NULL;
    JSContext *                         cx = NULL;
    jsval                               val;
    JSObject *                          services = NULL;
    JSObject *                          global = NULL;
    JSObject *                          serviceObj = NULL;
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_i_wsdl_parser_service_scanner);
    GlobusWSDLDebugEnter();
    
    service = (wsdlServicePtr) payload;
    cx = ((ScannerData *)data)->context;
    global = ((ScannerData *)data)->global;
    services = ((ScannerData *)data)->services;
    result = ((ScannerData *)data)->result;

    if(result != GLOBUS_SUCCESS)
    {
        goto exit;
    }

    serviceObj = JS_NewObject(cx, &service_class, serviceClass, services);
    globus_assert(serviceObj);
    JS_SetPrivate(cx, serviceObj, (void *)service);
    JS_DefineProperties(cx, serviceObj, service_properties);

    val = OBJECT_TO_JSVAL(serviceObj);

    if(!JS_DefineProperty(cx, services, (const char *)name, val, NULL, NULL, 
                          JSPROP_ENUMERATE|JSPROP_READONLY|JSPROP_PERMANENT))
    {
        JS_ReportError(cx, "Failed to define service from parsed WSDL");
        result = GlobusWSDLErrorScanningServices(GLOBUS_NULL, service->name);
        goto exit;
    }

exit:

    GlobusWSDLDebugExit();
    ((ScannerData *)data)->result = result;
}    

static
void
globus_i_wsdl_parser_wsdl_imports_scanner(
    void *                              payload,
    void *                              data,
    xmlChar *                           name)
{
    wsdlImportPtr                       import = NULL;
    JSContext *                         cx = NULL;
    JSObject *                          global = NULL;
    globus_result_t                     result = GLOBUS_SUCCESS;
    GlobusFuncName(globus_i_wsdl_parser_wsdl_imports_scanner);
    GlobusWSDLDebugEnter();
    
    import = (wsdlImportPtr) payload;
    cx = ((ScannerData *)data)->context;
    global = ((ScannerData *)data)->global;
    result = ((ScannerData *)data)->result;

    if(result != GLOBUS_SUCCESS)
    {
        goto exit;
    }

    ((ScannerData *)data)->schema = import->schema;
    result = globus_i_wsdl_js_load_imported_schema(cx, import->schema);
    if(result != GLOBUS_SUCCESS)
    {
        JS_ReportError(cx, "Failed to load WSDL schema");
        result = GlobusWSDLErrorScanningImports(
            GLOBUS_NULL, import->wsdlLocation);
        goto exit;
    }

exit:

    GlobusWSDLDebugExit();
    ((ScannerData *)data)->result = result;
}

static void
TemplateFree(
    void *                              template)
{
    Template *                          tmpl = (Template *) template;
    if(tmpl)
    {
        if(tmpl->filename)
        {
            free(tmpl->filename);
        }
        
        if(tmpl->buffer)
        {
            free(tmpl->buffer);
        }
        
        free(tmpl);
    }
}
