/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */

#include "globus_common.h"


#ifndef IN_LIBXML
#define IN_LIBXML
#include "libxml/xmlerror.h"
#undef  IN_LIBXML
#else
#include "libxml/xmlerror.h"
#endif

#include <string.h>
#include "libxml/xmlschemas.h"
#include "wsdl.h"
#include "libxml/parserInternals.h"
#include "libxml/uri.h"

const CONST_NODE WSDL_DEFINITIONS_NODE = {WSDL_NS, "definitions"};
const CONST_NODE WSDL_TYPES_NODE       = {WSDL_NS, "types"};
const CONST_NODE WSDL_MESSAGE_NODE     = {WSDL_NS, "message"};
const CONST_NODE WSDL_PART_NODE        = {WSDL_NS, "part"};
const CONST_NODE WSDL_PORTTYPE_NODE    = {WSDL_NS, "portType"};
const CONST_NODE WSDL_OPERATION_NODE   = {WSDL_NS, "operation"};
const CONST_NODE WSDL_INPUT_NODE       = {WSDL_NS, "input"};
const CONST_NODE WSDL_OUTPUT_NODE      = {WSDL_NS, "output"};
const CONST_NODE WSDL_FAULT_NODE       = {WSDL_NS, "fault"};
const CONST_NODE WSDL_BINDING_NODE     = {WSDL_NS, "binding"};
const CONST_NODE WSDL_SERVICE_NODE     = {WSDL_NS, "service"};
const CONST_NODE WSDL_PORT_NODE        = {WSDL_NS, "port"};
const CONST_NODE WSDL_IMPORT_NODE      = {WSDL_NS, "import"};
const CONST_NODE WSDL_DOCUMENTATION_NODE
                                       = {WSDL_NS, "documentation"};

const CONST_NODE SOAP_BINDING_NODE     = {SOAP_NS, "binding"};
const CONST_NODE SOAP_OPERATION_NODE   = {SOAP_NS, "operation"};
const CONST_NODE SOAP_BODY_NODE        = {SOAP_NS, "body"};
const CONST_NODE SOAP_HEADER_NODE      = {SOAP_NS, "header"};
const CONST_NODE SOAP_FAULT_NODE       = {SOAP_NS, "fault"};
const CONST_NODE SOAP_ADDRESS_NODE     = {SOAP_NS, "address"};

const CONST_NODE XSD_SCHEMA_NODE       = {XSD_NS, "schema"};
const CONST_NODE XSD_IMPORT_NODE       = {XSD_NS, "import"};

const CONST_NODE WSRP_RESOURCE_PROPERTIES_NODE = 
                                         {WSRP_NS, "ResourceProperties"};

/**
 * xmlSchemaIsBlank:
 * @str:  a string
 *
 * Check if a string is ignorable
 *
 * Returns 1 if the string is NULL or made of blanks chars, 0 otherwise
 */

/* copied from xmlschemas.c */

static int
xmlSchemaIsBlank(xmlChar * str)
{
    if (str == NULL)
        return (1);
    while (*str != 0) {
        if (!(IS_BLANK_CH(*str)))
            return (0);
        str++;
    }
    return (1);
}

#define IS_BLANK_NODE(n)						\
    (((n)->type == XML_TEXT_NODE) && (xmlSchemaIsBlank((n)->content)))

static
void
wsdlGenericError(wsdlParserCtxtPtr ctxt, xmlNodePtr node, int error,
          const char *msg, const xmlChar * str1, const xmlChar * str2);

static
int
xmlCompareNodeConst(xmlNodePtr node, const CONST_NODE comp);

static
int
xmlCompareNodes(xmlNodePtr node, xmlNodePtr comp);

static
int
GetChildNode(const xmlNodePtr node, CONST_NODE child, xmlNodePtr * childNode);

static
const xmlChar *
wsdlGetProp(
    wsdlParserCtxtPtr                   ctxt, 
    xmlNodePtr                          node, 
    const char *                        name);

static const xmlChar *
wsdlGetQNameProp(
    wsdlParserCtxtPtr                   ctxt,
    xmlNodePtr                          node,
    const char *                        name,
    const xmlChar **                    Namespace);

static void
wsdlErrorMemory(wsdlParserCtxtPtr ctxt,
                const char *extra, xmlNodePtr node);

static void
wsdlParserError(wsdlParserCtxtPtr ctxt, xmlNodePtr node, int error,
                const char *msg, const xmlChar * str1, const xmlChar * str2,
                const char *file, int line);

static
int
wsdlParseInternal(wsdlParserCtxtPtr     ctxt);

static void
wsdlCleanupDoc(wsdlParserCtxtPtr ctxt, xmlNodePtr root);

static
wsdlSchemaPtr
wsdlNewSchema(wsdlParserCtxtPtr         ctxt);

static
wsdlSchemaPtr
wsdlDoParse(wsdlParserCtxtPtr           ctxt,
            wsdlSchemaPtr               schema,
            xmlNodePtr                  node);

static wsdlPortPtr
wsdlParsePort(
    wsdlParserCtxtPtr                   ctxt,
    wsdlSchemaPtr                       schema,
    xmlNodePtr                          node);

static
void *
wsdlParseSoapOperation(
    wsdlParserCtxtPtr                   ctxt,
    wsdlSchemaPtr                       schema,
    xmlNodePtr                          node);

static
void *
wsdlParseSoapBindingParam(
    wsdlParserCtxtPtr                   ctxt,
    wsdlSchemaPtr                       schema,
    xmlNodePtr                          node);

static
void
wsdlSoapBindingParamFree(
    soapBindingParamPtr             param);

static
soapBindingPtr
wsdlParseSoapBinding(
    wsdlParserCtxtPtr                   ctxt,
    wsdlSchemaPtr                       schema,
    xmlNodePtr                          node);

static
void
wsdlBindingOperationFaultFree(
    wsdlBindingOperationFaultPtr        bopf);


static
void
wsdlBindingOperationFree(
    wsdlBindingOperationPtr             bop);

static wsdlBindingPtr
wsdlParseBinding(
    wsdlParserCtxtPtr                   ctxt,
    wsdlSchemaPtr                       schema,
    xmlNodePtr                          node);

static
void
wsdlFaultFree(
    wsdlFaultPtr                        fault);

static 
void
wsdlBindingFree(
    wsdlBindingPtr                      binding);

static 
void
wsdlBindingHEFree(
    void *                              value,
    xmlChar *                           name);

static
void
wsdlPortFree(
    wsdlPortPtr                         port);

static wsdlServicePtr
wsdlParseService(
    wsdlParserCtxtPtr                   ctxt,
    wsdlSchemaPtr                       schema,
    xmlNodePtr                          node);

static
void
wsdlServiceFree(
    wsdlServicePtr                      service);

static
void
wsdlServiceHEFree(
    void *                              value,
    xmlChar *                           name);

static
int
wsdlParseTypes(wsdlParserCtxtPtr        ctxt,
               wsdlSchemaPtr            schema,
               xmlNodePtr               node);

static        
wsdlPartPtr
wsdlParsePart(wsdlParserCtxtPtr         ctxt,
              wsdlSchemaPtr             schema,
              xmlNodePtr                node);

static
void
wsdlPartFree(
    wsdlPartPtr                         part);

static
wsdlMessagePtr
wsdlParseMessage(wsdlParserCtxtPtr      ctxt,
                 wsdlSchemaPtr          schema,
                 xmlNodePtr             node);

static
void
wsdlMessageFree(
    wsdlMessagePtr                      message);

static
void
wsdlMessageHEFree(
    void *                              value,
    xmlChar *                           name);

static
wsdlParserCtxtPtr
wsdlParseImport(wsdlParserCtxtPtr       ctxt,
                wsdlSchemaPtr           schema,
                xmlNodePtr              node);

static
int
wsdlParseSchemaImport(wsdlParserCtxtPtr ctxt,
                      wsdlSchemaPtr     schema,
                      xmlNodePtr        node);

static
wsdlParamPtr
wsdlParseParam(wsdlParserCtxtPtr        ctxt,
               wsdlSchemaPtr            schema,
               xmlNodePtr               node);

static
wsdlOperationPtr
wsdlParseOperation(wsdlParserCtxtPtr    ctxt,
                   wsdlSchemaPtr        schema,
                   xmlNodePtr           node);

static
void
wsdlOperationFree(
    wsdlOperationPtr                    op);

static
wsdlPortTypePtr
wsdlParsePortType(wsdlParserCtxtPtr     ctxt,
                  wsdlSchemaPtr         schema,
                  xmlNodePtr            node);

static
void
wsdlPortTypeFree(
    wsdlPortTypePtr                     portType);

static
void
wsdlPortTypeHEFree(
    void *                              value,
    xmlChar *                           name);

static
int
wsdlParseTopLevel(wsdlParserCtxtPtr     ctxt,
                  wsdlSchemaPtr         schema,
                  xmlNodePtr            node);

static
int
wsdlParseImports(wsdlParserCtxtPtr     ctxt,
                  wsdlSchemaPtr         schema,
                  xmlNodePtr            node);

static
void
wsdlCreateTypesHashScanner(
    void *                              import_payload,
    void *                              data,
    xmlChar *                           schema_namespace);

char *
globus_i_wsdl_my_strtok(
    const char *                        str,
    size_t                              str_length,
    int *                               token_start_index,
    int *                               token_end_index,
    const char *                        delims)
{
    const char *                        locator;
    const char *                        start_token;
    int                                 delim_count, ind, i, between_tokens, len;
    char *                              new_token = NULL;
    ind = 0;
    i = 0;
    between_tokens = 0;
    delim_count = strlen(delims);
    locator = str;
    
    for(i = 0; i < delim_count; ++i)
    {
        if(*locator == delims[i])
        {
            between_tokens = 1;
        }
    }

    while(between_tokens && locator && ind < str_length)
    {
        for(i = 0; i < delim_count; ++i)
        {
            if(*locator == delims[i])
            {
                break;
            }
        }

        if(i == delim_count)
        {
            between_tokens = 0;
            break;
        }

        locator++;
        ind++;
    }

    start_token = locator;
    if(token_start_index)
    {
        *token_start_index = ind;
    }

    while(!between_tokens && *locator && ind < str_length)
    {
        for(i = 0; i < delim_count; ++i)
        {
            if(*locator == delims[i])
            {
                if(token_end_index)
                {
                    *token_end_index = ind;
                }
                between_tokens = 1;
                break;
            }
        }

        locator++;
        ind++;
    }

    if(ind == str_length)
    {
        *token_end_index = -1;
    }
    else
    {
        ind--;
    }

    len = ind + (str - start_token);
    if(len > 0)
    {
        new_token = malloc(len+1);
        memcpy(new_token, start_token, len + 1);
        new_token[len] = '\0';
    }

    return new_token;
}

int
globus_i_wsdl_split_qnames(
    xmlChar *                           default_namespace,
    xmlNodePtr                          node,
    const xmlChar *                     qname_string,
    xsdQNamePtr *                       qnames)
{
    xmlChar *                           prefix;
    xmlChar *                           rpns;
    xmlChar *                           tmpstr;
    xmlChar *                           newstr;
    xmlChar *                           local;
    int                                 start_token;
    int                                 end_token;
    char *                              token;
    xsdQNamePtr                         qn = NULL;

    newstr = globus_libc_strdup(qname_string);
    tmpstr = newstr;
    token = globus_i_wsdl_my_strtok(tmpstr, strlen(tmpstr), &start_token, &end_token, "\n\t ");
    tmpstr += end_token;
        
    while(1)
    {
        xsdQNamePtr                     new_qn;
        xmlNsPtr                        ns;

        local = xmlSplitQName2(token, &prefix);
        if(!prefix)
        {
            if(default_namespace)
            {
                rpns = default_namespace;
            }
            else
            {
                ns = xmlSearchNs(node->doc, node, prefix);
                rpns = (xmlChar *) ns->href;
            }
        }
        else
        {
            ns = xmlSearchNs(node->doc, node, prefix);
            rpns = (xmlChar *) ns->href;
        }

        if(!local)
        {
            local = xmlStrdup(token);
        }

        new_qn = xmlMalloc(sizeof(xsdQName));
        memset(new_qn, 0, sizeof(xsdQName));

        new_qn->name = local;
        new_qn->Namespace = rpns;
        new_qn->next = qn;
        qn = new_qn;

        free(token);
        if(end_token < 0)
        {
            break;
        }

        token = globus_i_wsdl_my_strtok(tmpstr, strlen(tmpstr), &start_token, &end_token, "\n\t ");
        tmpstr += end_token;
        if(!token)
        {
            break;
        }

    }

    *qnames = qn;
    globus_free(newstr);
    return 0;
}

void
xsdQNameFree(
    xsdQNamePtr                         qname)
{
    if(qname)
    {
        if(qname->next)
        {
            xsdQNameFree(qname->next);
        }

        xmlFree(qname);
    }
}
        

static
int
xmlCompareNodeConst(xmlNodePtr node, const CONST_NODE comp)
{
    if(!node || !node->name)
    {
        return -1;
    }

    if(!xmlStrEqual(node->name, comp.name))
    {
        return -1;
    }

    if(!node->ns)
    {
        return -1;
    }

    if(!xmlStrEqual(node->ns->href, comp.ns))
    {
        return -1;
    }

    return 0;
}

static
int
xmlCompareNodes(xmlNodePtr node, xmlNodePtr comp)
{
    if(!node || !node->name || !comp || !comp->name)
    {
        return -1;
    }

    if(!xmlStrEqual(node->name, comp->name))
    {
        return -1;
    }

    if(!node->ns || !comp->ns)
    {
        return -1;
    }

    if(!xmlStrEqual(node->ns->href, comp->ns->href))
    {
        return -1;
    }

    return 0;
}

static
int
GetChildNode(const xmlNodePtr node, CONST_NODE child, xmlNodePtr * childNode)
{
    xmlNodePtr                          tmpNode;

    if(!node || !node->children)
    {
        return -1;
    }

    tmpNode = node->children;
    while(tmpNode)
    {
        if(tmpNode->type == XML_ELEMENT_NODE &&
           !xmlCompareNodeConst(tmpNode, child))
        {
            break;
        }
        else
        {
            tmpNode = tmpNode->next;
        }
    }

    if(!tmpNode)
    {
        return -1;
    }

    *childNode = tmpNode;
    return 0;
}

static
const xmlChar *
wsdlGetProp(
    wsdlParserCtxtPtr                   ctxt, 
    xmlNodePtr                          node, 
    const char *                        name)
{
    xmlChar *                           val;
    const xmlChar *                     ret;
    
    val = xmlGetProp(node, BAD_CAST name);
    if(val == NULL)
    {
        return (NULL);
    }

    ret = xmlDictLookup(ctxt->dict, val, -1);
    xmlFree(val);
    return(ret);
}

static const xmlChar *
wsdlGetQNameProp(
    wsdlParserCtxtPtr                   ctxt,
    xmlNodePtr                          node,
    const char *                        name,
    const xmlChar **                    Namespace)
{
    const xmlChar *                     val;
    xmlNsPtr                            ns;
    const xmlChar *                     ret;
    const xmlChar *                     prefix;
    int                                 len;

    *Namespace = NULL;
    val = wsdlGetProp(ctxt, node, name);
    if(val == NULL)
    {
        return (NULL);
    }
    
    ret = xmlSplitQName3(val, &len);
    if(ret == NULL)
    {
        return (val);
    }

    ret = xmlDictLookup(ctxt->dict, ret, -1);
    prefix = xmlDictLookup(ctxt->dict, val, len);

    ns = xmlSearchNs(node->doc, node, prefix);
    if(ns == NULL)
    {
        wsdlGenericError(ctxt, node, WSDL_ERR_PREFIX_UNDEFINED,
                 "Attribute %s: the QName prefix %s is undefined\n",
                 (const xmlChar *) name, prefix);
    }
    else
    {
        *Namespace = xmlDictLookup(ctxt->dict, ns->href, -1);
    }

    return (ret);
}

/**
 * wsdlSetParserErrors:
 * @ctxt:  a schema validation context
 * @err:  the error callback
 * @warn:  the warning callback
 * @ctx:  contextual data for the callbacks
 *
 * Set the callback functions used to handle errors for a validation context
 */
void
wsdlSetParserErrors(wsdlParserCtxtPtr ctxt,
                    wsdlErrorFunc err,
                    wsdlWarningFunc warn, void *ctx)
{
    if (ctxt == NULL)
        return;
    ctxt->err = err;
    ctxt->warn = warn;
    ctxt->userData = ctx;
}

/**
 * wsdlErrorMemory:
 * @node: a context node
 * @extra:  extra informations
 *
 * Handle an out of memory condition
 */
static void
wsdlErrorMemory(wsdlParserCtxtPtr ctxt,
                const char *extra, xmlNodePtr node)
{
    if (ctxt != NULL)
        ctxt->nberrors++;
    __xmlSimpleError(XML_FROM_SCHEMASP, XML_ERR_NO_MEMORY, node, NULL,
                     extra);
}

/**
 * wsdlError:
 * @ctxt: the parsing context
 * @node: the context node
 * @error: the error code
 * @msg: the error message
 * @str1: extra data
 * @str2: extra data
 * 
 * Handle a parser error
 */
static void
wsdlGenericError(wsdlParserCtxtPtr ctxt, xmlNodePtr node, int error,
          const char *msg, const xmlChar * str1, const xmlChar * str2)
{
    xmlGenericErrorFunc channel = NULL;
    xmlStructuredErrorFunc schannel = NULL;
    void *data = NULL;

    if (ctxt != NULL) {
        ctxt->nberrors++;
        channel = ctxt->err;
        data = ctxt->userData;
    }
    __xmlRaiseError(schannel, channel, data, ctxt, node, XML_FROM_SCHEMASP,
                    error, XML_ERR_ERROR, NULL, 0,
                    (const char *) str1, (const char *) str2, NULL, 0, 0,
                    msg, str1, str2);
}

static void
wsdlParserError(wsdlParserCtxtPtr ctxt, xmlNodePtr node, int error,
                const char *msg, const xmlChar * str1, const xmlChar * str2,
                const char *file, int line)
{
    xmlGenericErrorFunc channel = NULL;
    xmlStructuredErrorFunc schannel = NULL;
    void *data = NULL;

    if (ctxt != NULL) {
        ctxt->nberrors++;
        channel = ctxt->err;
        data = ctxt->userData;
    }
    __xmlRaiseError(schannel, channel, data, ctxt, node, XML_FROM_SCHEMASP,
                    error, XML_ERR_ERROR, file, (line != -1) ? line : 0,
                    (const char *) str1, (const char *) str2, NULL, 0, 0,
                    msg, str1, str2);
}    

wsdlParserCtxtPtr
wsdlNewParserCtxt(
    const char *                        URL)
{
    wsdlParserCtxtPtr                   ret;

    if(URL == NULL)
    {
        return (NULL);
    }

    ret = (wsdlParserCtxtPtr) xmlMalloc(sizeof(wsdlParserCtxt));
    if(ret == NULL)
    {
        wsdlErrorMemory(NULL, "allocating wsdl parser context", NULL);
        return (NULL);
    }

    memset(ret, 0, sizeof(wsdlParserCtxt));
    ret->dict = xmlDictCreate();
    if(!ret->dict)
    {
        wsdlGenericError(ret, NULL, WSDL_ERR_DICT, "creating dictionary for context",
                  NULL, NULL);
        xmlFree(ret);
        return (NULL);
    }
    
    ret->url = (xmlChar *) xmlDictLookup(ret->dict, (const xmlChar *) URL, -1);
    return (ret);
}

wsdlParserCtxtPtr
wsdlNewMemParserCtxt(
    const char *                        buffer,
    int                                 size)
{
    wsdlParserCtxtPtr                   ret;

    if((buffer == NULL) || (size <= 0))
    {
        return (NULL);
    }

    ret = (wsdlParserCtxtPtr) xmlMalloc(sizeof(wsdlParserCtxt));
    if(ret == NULL)
    {
        wsdlErrorMemory(NULL, "allocating wsdl parser context", NULL);
        return (NULL);
    }
    memset(ret, 0, sizeof(wsdlParserCtxt));
    ret->buffer = (char *) buffer;
    ret->size = size;
    ret->dict = xmlDictCreate();
    if(!ret->dict)
    {
        xmlFree(ret);
        return (NULL);
    }

    return (ret);
}

wsdlParserCtxtPtr
wsdlNewDocParserCtxt(
    xmlDocPtr                           doc)
{
    wsdlParserCtxtPtr                   ret;

    if(doc == NULL)
    {
        return (NULL);
    }

    ret = (wsdlParserCtxtPtr) xmlMalloc(sizeof(wsdlParserCtxt));
    if(ret == NULL)
    {
        wsdlErrorMemory(NULL, "allocating wsdl parser context", NULL);
        return (NULL);
    }
    memset(ret, 0, sizeof(wsdlParserCtxt));
    ret->doc = doc;
    ret->dict = xmlDictCreate();
    if(!ret->dict)
    {
        xmlFree(ret);
        return (NULL);
    }

    ret->preserve = 1;
    
    return (ret);
}

void
wsdlFreeParserCtxt(wsdlParserCtxtPtr    ctxt)
{
    if(ctxt == NULL)
    {
        return;
    }

    if(ctxt->doc != NULL && !ctxt->preserve)
    {
        xmlFreeDoc(ctxt->doc);
    }
    xmlDictFree(ctxt->dict);
    xmlFree(ctxt);
}

static
int
wsdlParseInternal(wsdlParserCtxtPtr     ctxt)
{
    if (ctxt->url != NULL) {
        ctxt->doc = xmlReadFile((const char *) ctxt->url, NULL, 
                                XML_PARSE_NOENT);
        if (ctxt->doc == NULL) {
	    wsdlGenericError(ctxt, NULL,
                     WSDL_ERR_FAILED_LOAD,
                     "wsdlParseInternal: could not load %s\n", 
                     ctxt->url, NULL);
            return -1;
        }
    } else if (ctxt->buffer != NULL) {
        ctxt->doc = xmlReadMemory(ctxt->buffer, ctxt->size, NULL, NULL,
                                  XML_PARSE_NOENT);
        if (ctxt->doc == NULL) {
	    wsdlGenericError(ctxt, NULL,
                     WSDL_ERR_FAILED_PARSE,
                     "wsdlParseInternal: could not parse\n",
                     NULL, NULL);
            return -1;
        }
        ctxt->doc->URL = xmlStrdup(BAD_CAST "in_memory_buffer");
        ctxt->url = (char *) xmlDictLookup(ctxt->dict, BAD_CAST "in_memory_buffer", -1);
    } else if (ctxt->doc != NULL) {
	ctxt->preserve = 1;
    } else {
	wsdlGenericError(ctxt, NULL,
                 WSDL_ERR_NOTHING_TO_PARSE,
                 "wsdlParseInternal: could not parse\n",
                 NULL, NULL);
        return -1;
    }

    return 0;
}

/**
 * unashamedly copied from xmlSchemaCleanupDoc
 */
static void
wsdlCleanupDoc(wsdlParserCtxtPtr ctxt, xmlNodePtr root)
{
    xmlNodePtr delete, cur;

    if ((ctxt == NULL) || (root == NULL)) return;

    /*
     * Remove all the blank text nodes
     */
    delete = NULL;
    cur = root;
    while (cur != NULL) {
        if (delete != NULL) {
            xmlUnlinkNode(delete);
            xmlFreeNode(delete);
            delete = NULL;
        }
        if (cur->type == XML_TEXT_NODE) {
            if (IS_BLANK_NODE(cur)) {
                if (xmlNodeGetSpacePreserve(cur) != 1) {
                    delete = cur;
                }
            }
        } else if ((cur->type != XML_ELEMENT_NODE) &&
                   (cur->type != XML_CDATA_SECTION_NODE)) {
            delete = cur;
            goto skip_children;
        }

        /*
         * Skip to next node
         */
        if (cur->children != NULL) {
            if ((cur->children->type != XML_ENTITY_DECL) &&
                (cur->children->type != XML_ENTITY_REF_NODE) &&
                (cur->children->type != XML_ENTITY_NODE)) {
                cur = cur->children;
                continue;
            }
        }
      skip_children:
        if (cur->next != NULL) {
            cur = cur->next;
            continue;
        }

        do {
            cur = cur->parent;
            if (cur == NULL)
                break;
            if (cur == root) {
                cur = NULL;
                break;
            }
            if (cur->next != NULL) {
                cur = cur->next;
                break;
            }
        } while (cur != NULL);
    }
    if (delete != NULL) {
        xmlUnlinkNode(delete);
        xmlFreeNode(delete);
        delete = NULL;
    }
}

static
wsdlSchemaPtr
wsdlNewSchema(wsdlParserCtxtPtr         ctxt)
{
    wsdlSchemaPtr                       ret;

    ret = (wsdlSchemaPtr) xmlMalloc(sizeof(wsdlSchema));
    if(ret == NULL)
    {
        wsdlErrorMemory(ctxt, "allocating schema", NULL);
        return (NULL);
    }
    memset(ret, 0, sizeof(wsdlSchema));
    xmlDictReference(ctxt->dict);
    ret->dict = ctxt->dict;
    ctxt->schema = ret;
    return (ret);
}
    

static
wsdlSchemaPtr
wsdlDoParse(wsdlParserCtxtPtr           ctxt,
            wsdlSchemaPtr               outer_schema,
            xmlNodePtr                  node)
{
    wsdlSchemaPtr                       schema = NULL;
    xmlNodePtr                          child = NULL;
    int                                 nberrors = 0;
    const xmlChar *                     val = NULL;
    int                                 res;
    
    if((ctxt == NULL) || (node == NULL))
    {
        return (NULL);
    }

    nberrors = ctxt->nberrors;
    ctxt->nberrors = 0;

    if(node->type == XML_ELEMENT_NODE &&
       !xmlCompareNodeConst(node, WSDL_DEFINITIONS_NODE))
    {
        schema = wsdlNewSchema(ctxt);
        if(schema == NULL)
        {
            return (NULL);
        }

        val = wsdlGetProp(ctxt, node, "targetNamespace");
        if(val != NULL)
        {
            schema->targetNamespace = xmlDictLookup(ctxt->dict, val, -1);
        }

        child = node->children;
        while(child)
        {
            res = wsdlParseImports(ctxt, schema, child);
            if(res != 0)
            {
                return (NULL);
            }
            child = child->next;
        }

        child = node->children;
        while(child)
        {
            res = wsdlParseTopLevel(ctxt, schema, child);
            if(res != 0)
            {
                long                            lno = xmlGetLineNo(node);
                wsdlParserError(ctxt, node,
                                WSDL_ERR_NOT_WSDL,
                                "Failed to parse top level element: %s",
                                child->name, NULL, ctxt->url, lno);
                return (NULL);
            }
            child = child->next;
        }
    }
    else if(node->type == XML_ELEMENT_NODE &&
            !xmlCompareNodeConst(node, XSD_SCHEMA_NODE))
    {
        if(!outer_schema)
        {
            long                        lno = xmlGetLineNo(node);
            wsdlParserError(ctxt, node,
                            WSDL_ERR_NOT_WSDL,
                            "Parser failed to find a 'definitions' element"
                            " in the document.  A 'schema' element appears "
                            "to be outermost node in the document",
                            NULL, NULL, ctxt->url, lno);
            return (NULL);
        }

        res = wsdlParseTypes(ctxt, outer_schema, node->parent);
        if(res != 0)
        {
            long                            lno = xmlGetLineNo(node);
            wsdlParserError(ctxt, node,
                            WSDL_ERR_NOT_WSDL,
                            "Failed to parser types in schema",
                            NULL, NULL, ctxt->url, lno);
            return (NULL);
        }
    }
    else
    {
        long                            lno = xmlGetLineNo(node);
        wsdlParserError(ctxt, node,
                        WSDL_ERR_NOT_WSDL,
                        "no 'definitions' element",
                        NULL, NULL, ctxt->url, lno);
        return (NULL);
    }

    if(ctxt->nberrors != 0)
    {
        if(schema != NULL)
        {
            wsdlSchemaFree(schema);
            schema = NULL;
        }
    }
    ctxt->nberrors = nberrors;
#ifdef DEBUG
    if(schema == NULL)
    {
        xmlGenericError(xmlGenericErrorContext,
                        "wsdlDoParse() failed\n");
    }
#endif
    
    return (schema);
}

wsdlSchemaPtr
wsdlParse(wsdlParserCtxtPtr             ctxt)
{
    int                                 res;
    wsdlSchemaPtr                       ret = NULL;
    xmlNodePtr                          root;
    
    res = wsdlParseInternal(ctxt);
    if(res != 0)
    {
        return NULL;
    }
    
    root = xmlDocGetRootElement(ctxt->doc);
    if(root == NULL)
    {
        wsdlParserError(ctxt, (xmlNodePtr) ctxt->doc,
                        WSDL_ERR_NOROOT,
                        "wsdl document has no root", 
                        NULL, NULL, ctxt->url, 0);
        if(!ctxt->preserve)
        {
            xmlFreeDoc(ctxt->doc);
        }
        return (NULL);
    }

    wsdlCleanupDoc(ctxt, root);

    ret = wsdlDoParse(ctxt, NULL, root);
    if(ret == NULL)
    {
        return (NULL);
    }

    return (ret);
}

wsdlSchemaPtr
wsdlParseXmlSchema(wsdlParserCtxtPtr             ctxt)
{
    xmlSchemaParserCtxtPtr              xmlschema_context = NULL;
    wsdlSchemaPtr                       schema = NULL;
    
    schema = wsdlNewSchema(ctxt);
    if(schema == NULL)
    {
        return (NULL);
    }

    xmlschema_context = xmlSchemaNewParserCtxt(ctxt->url);
    xmlSchemaSetParserErrors(xmlschema_context, 
                             (xmlSchemaValidityErrorFunc) ctxt->err, 
                             (xmlSchemaValidityWarningFunc) ctxt->warn, 
                             ctxt->userData);
        
    schema->schemaTypes = xmlSchemaParse(xmlschema_context, NULL);
    xmlSchemaFreeParserCtxt(xmlschema_context);

    return (schema);
}

static wsdlPortPtr
wsdlParsePort(
    wsdlParserCtxtPtr                   ctxt,
    wsdlSchemaPtr                       schema,
    xmlNodePtr                          node)
{
    xmlNodePtr                          tmpNode;
    xmlNodePtr                          childNode;
    wsdlPortPtr                         port = NULL;
    const xmlChar *                     binding_name;
    const xmlChar *                     binding_ns;

    tmpNode = node;

    if((ctxt == NULL) || (schema == NULL))
    {
        return (NULL);
    }
    
    port = (wsdlPortPtr) xmlMalloc(sizeof(wsdlPort));
    if(!port)
    {
        wsdlErrorMemory(ctxt, "allocating port", NULL);
        return (NULL);
    }
    memset(port, 0, sizeof(wsdlPort));
    port->node = tmpNode;
    port->name = wsdlGetProp(ctxt, tmpNode, "name");
    binding_name = wsdlGetQNameProp(ctxt, tmpNode, "binding", &binding_ns);
    port->binding = wsdlFindBinding(ctxt, schema, binding_name, binding_ns);
    if(!port->binding)
    {
        long                            lno = xmlGetLineNo(node);
        wsdlParserError(ctxt, node,
                        WSDL_ERR_NO_NODE,
                        "reference to binding {%s}%s could not be qualified",
                        binding_ns, binding_name, ctxt->url, lno);
        xmlFree(port);
        return (NULL);
    }

    childNode = tmpNode->children;

    while(childNode)
    {
        if(childNode &&
                !xmlCompareNodeConst(childNode, WSDL_DOCUMENTATION_NODE))
        {
            childNode = childNode->next;
        }
        else if(childNode && !xmlCompareNodeConst(childNode, SOAP_ADDRESS_NODE))
        {
            soapAddressPtr                  soap_addr;
            
            port->extensionType = SOAP_BINDING;
            soap_addr = (soapAddressPtr) xmlMalloc(sizeof(soapAddress));
            if(!soap_addr)
            {
                wsdlPortFree(port);
                return (NULL);
            }
            memset(soap_addr, 0, sizeof(soapAddress));
            
            soap_addr->location = wsdlGetProp(ctxt, childNode, "location");
            if(!soap_addr->location)
            {
                long                        lno = xmlGetLineNo(childNode);
                wsdlParserError(ctxt, childNode, WSDL_ERR_SOAP_BINDING,
                                "no location 'attribute' found for soap address element",
                                NULL, NULL, ctxt->url, lno);
                wsdlPortFree(port);
                return (NULL);
            }

            soap_addr->node = childNode;

            port->extension = (void *) soap_addr;

            return (port);
        }
        else
        {
            long                        lno = xmlGetLineNo(childNode);
            wsdlParserError(ctxt, childNode, WSDL_ERR_SOAP_BINDING,
                            "unexpected child of port element",
                            NULL, NULL, ctxt->url, lno);
            wsdlPortFree(port);
            return (NULL);
        }
    }
    return port;
}

static
void
wsdlPortFree(
    wsdlPortPtr                         port)
{
    if(port)
    {
        if(port->next)
        {
            wsdlPortFree(port->next);
        }

        if(port->extension)
        {
            if(port->extensionType == SOAP_BINDING)
            {
                xmlFree(port->extension);
            }
        }

        xmlFree(port);
    }
}
        
static
void *
wsdlParseSoapOperation(
    wsdlParserCtxtPtr                   ctxt,
    wsdlSchemaPtr                       schema,
    xmlNodePtr                          node)
{
    soapOperationPtr                    soap_op;
    const xmlChar *                     style;

    soap_op = (soapOperationPtr) xmlMalloc(sizeof(soapOperation));
    if(!soap_op)
    {
        wsdlErrorMemory(ctxt, "allocationg soap operation", NULL);
        return (NULL);
    }

    soap_op->soapAction = wsdlGetProp(ctxt, node, "soapAction");
    style = wsdlGetProp(ctxt, node, "style");
    if(style)
    {
        if(xmlStrEqual(style, "rpc"))
        {
            soap_op->style = RPC_STYLE;
        }
        else if(xmlStrEqual(style, "document"))
        {
            soap_op->style = DOCUMENT_STYLE;
        }
    }

    soap_op->node = node;

    return (soap_op);
}

static
void *
wsdlParseSoapBindingParam(
    wsdlParserCtxtPtr                   ctxt,
    wsdlSchemaPtr                       schema,
    xmlNodePtr                          node)
{
    xmlNodePtr                          tmpNode;
    soapBindingParamPtr                 soap_param;

    soap_param = (soapBindingParamPtr) xmlMalloc(sizeof(soapBindingParam));
    if(!soap_param)
    {
        wsdlErrorMemory(ctxt, "allocating soap binding param", NULL);
        return (NULL);
    }
    memset(soap_param, 0, sizeof(soapBindingParam));

    tmpNode = node;

    while(tmpNode)
    {
        if(!xmlCompareNodeConst(tmpNode, SOAP_BODY_NODE))
        {
            soapBodyPtr                     soap_body;
            const xmlChar *                 use;

            if(soap_param->body)
            {
                long                    lno = xmlGetLineNo(tmpNode);
                wsdlParserError(ctxt, tmpNode,
                                WSDL_ERR_SOAP_BINDING,
                                "multiple soap body elements defined",
                                NULL, NULL, ctxt->url, lno);
                wsdlSoapBindingParamFree(soap_param);
                return (NULL);
            }
                
            soap_body = (soapBodyPtr) xmlMalloc(sizeof(soapBody));
            if(!soap_body)
            {
                wsdlErrorMemory(ctxt, "allocating soap body", NULL);
                return (NULL);
            }
            memset(soap_body, 0, sizeof(soapBody));

            soap_body->encodingStyle = wsdlGetProp(ctxt, tmpNode, "encodingStyle");
            soap_body->parts = wsdlGetProp(ctxt, tmpNode, "parts");
            use = wsdlGetProp(ctxt, tmpNode, "use");
        
            if(use)
            {
                if(xmlStrEqual(use, "literal"))
                {
                    soap_body->use = LITERAL_USE;
                }
                else if(xmlStrEqual(use, "encoded"))
                {
                    soap_body->use = ENCODED_USE;
                }
                else
                {
                    long                        lno = xmlGetLineNo(tmpNode);
                    wsdlParserError(ctxt, tmpNode,
                                    WSDL_ERR_SOAP_BINDING,
                                    "Unknown soap body 'use' type: %s",
                                    use, NULL, ctxt->url, lno);
                    xmlFree(soap_body);
                    wsdlSoapBindingParamFree(soap_param);
                    return (NULL);
                }
            }

            soap_body->Namespace = wsdlGetProp(ctxt, tmpNode, "namespace");
            soap_body->node = tmpNode;
        
            soap_param->body = soap_body;
        }
        else if(!xmlCompareNodeConst(tmpNode, SOAP_HEADER_NODE))
        {
            soapHeaderPtr                   soap_header;
            const xmlChar *                 use;

            if(soap_param->header)
            {
                long                    lno = xmlGetLineNo(tmpNode);
                wsdlParserError(ctxt, tmpNode,
                                WSDL_ERR_SOAP_BINDING,
                                "multiple soap header elements defined",
                                NULL, NULL, ctxt->url, lno);
                wsdlSoapBindingParamFree(soap_param);
                return (NULL);
            }
                
            soap_header = (soapHeaderPtr) xmlMalloc(sizeof(soapHeader));
            if(!soap_header)
            {
                wsdlErrorMemory(ctxt, "allocating soap header", NULL);
                return (NULL);
            }
            memset(soap_header, 0, sizeof(soapHeader));

            soap_header->encodingStyle = wsdlGetProp(ctxt, tmpNode, "encodingStyle");
            soap_header->parts = wsdlGetProp(ctxt, tmpNode, "parts");
            use = wsdlGetProp(ctxt, tmpNode, "use");
        
            if(use)
            {
                if(xmlStrEqual(use, "literal"))
                {
                    soap_header->use = LITERAL_USE;
                }
                else if(xmlStrEqual(use, "encoded"))
                {
                    soap_header->use = ENCODED_USE;
                }
                else
                {
                    long                        lno = xmlGetLineNo(tmpNode);
                    wsdlParserError(ctxt, tmpNode,
                                    WSDL_ERR_SOAP_BINDING,
                                    "Unknown soap header use type: %s",
                                    use, NULL, ctxt->url, lno);
                    xmlFree(soap_header);
                    wsdlSoapBindingParamFree(soap_param);
                    return (NULL);
                }
            }

            soap_header->Namespace = wsdlGetProp(ctxt, tmpNode, "namespace");
            soap_header->node = tmpNode;
        
            soap_param->header = soap_header;
        }
        else if(!xmlCompareNodeConst(tmpNode, SOAP_FAULT_NODE))
        {
            soapFaultPtr                soap_fault;
            const xmlChar *             use;

            soap_fault = (soapFaultPtr) xmlMalloc(sizeof(soapFault));
            if(!soap_fault)
            {
                wsdlErrorMemory(ctxt, "allocating soap header", NULL);
                return (NULL);
            }
            memset(soap_fault, 0, sizeof(soapFault));

            soap_fault->encodingStyle = wsdlGetProp(ctxt, tmpNode, "encodingStyle");
            use = wsdlGetProp(ctxt, tmpNode, "use");
        
            if(use)
            {
                if(xmlStrEqual(use, "literal"))
                {
                    soap_fault->use = LITERAL_USE;
                }
                else if(xmlStrEqual(use, "encoded"))
                {
                    soap_fault->use = ENCODED_USE;
                }
                else
                {
                    long                        lno = xmlGetLineNo(tmpNode);
                    wsdlParserError(ctxt, tmpNode,
                                    WSDL_ERR_SOAP_BINDING,
                                    "Unknown soap fault use type: %s",
                                    use, NULL, ctxt->url, lno);
                    xmlFree(soap_fault);
                    wsdlSoapBindingParamFree(soap_param);
                    return (NULL);
                }
            }

            soap_fault->Namespace = wsdlGetProp(ctxt, tmpNode, "namespace");
            soap_fault->node = tmpNode;
        
            soap_param->fault = soap_fault;
        }

        tmpNode = tmpNode->next;
    }

    return (soap_param);
}

static
void
wsdlSoapBindingParamFree(
    soapBindingParamPtr             param)
{
    if(param)
    {
        if(param->header)
        {
            xmlFree(param->header);
        }
        
        if(param->body)
        {
            xmlFree(param->body);
        }
        
        if(param->fault)
        {
            xmlFree(param->fault);
        }

        xmlFree(param);
    }
}

static
soapBindingPtr
wsdlParseSoapBinding(
    wsdlParserCtxtPtr                   ctxt,
    wsdlSchemaPtr                       schema,
    xmlNodePtr                          node)
{
    const xmlChar *                     style;
    soapBindingPtr                      sb;

    sb = (soapBindingPtr) xmlMalloc(sizeof(soapBinding));
    if(!sb)
    {
        wsdlErrorMemory(ctxt, "allocating soap binding", NULL);
        return (NULL);
    }
    memset(sb, 0, sizeof(soapBinding));

    style = wsdlGetProp(ctxt, node, "style");
    if(style)
    {
        if(xmlStrEqual(style, "rpc"))
        {
            sb->style = RPC_STYLE;
        }
        else if(xmlStrEqual(style, "document"))
        {
            sb->style = DOCUMENT_STYLE;
        }
        else
        {
            long                        lno = xmlGetLineNo(node);
            wsdlParserError(ctxt, node,
                            WSDL_ERR_SOAP_BINDING,
                            "Invalid style choice: %s for soap binding",
                            style, NULL, ctxt->url, lno);
            xmlFree(sb);
            return (NULL);

        }
    }

    sb->transport = wsdlGetProp(ctxt, node, "transport");
    sb->node = node;

    return (sb);
}

static
void
wsdlBindingOperationFaultFree(
    wsdlBindingOperationFaultPtr        bopf)
{
    if(bopf)
    {
        if(bopf->next)
        {
            wsdlBindingOperationFaultFree(bopf->next);
        }

        if(bopf->extension)
        {
            if(bopf->extensionType == SOAP_BINDING)
            {
                wsdlSoapBindingParamFree((soapBindingParamPtr) bopf->extension);
            }
        }

        xmlFree(bopf);
    }
}

static
void
wsdlBindingOperationFree(
    wsdlBindingOperationPtr             bop)
{
    if(bop)
    {
        if(bop->next)
        {
            wsdlBindingOperationFree(bop->next);
        }

        if(bop->input)
        {
            if(bop->inputType == SOAP_BINDING)
            {
                wsdlSoapBindingParamFree(bop->input);
            }
        }
        if(bop->output)
        {
            if(bop->outputType == SOAP_BINDING)
            {
                wsdlSoapBindingParamFree(bop->output);
            }
        }
        if(bop->faults)
        {
            wsdlBindingOperationFaultFree(bop->faults);
        }
        if(bop->extension)
        {
            if(bop->extensionType == SOAP_BINDING)
            {
                xmlFree(bop->extension);
            }
        }
    }
}

static wsdlBindingPtr
wsdlParseBinding(
    wsdlParserCtxtPtr                   ctxt,
    wsdlSchemaPtr                       schema,
    xmlNodePtr                          node)
{
    xmlNodePtr                          tmpNode;
    wsdlBindingPtr                      binding;
    const xmlChar *                     port_type_name;
    const xmlChar *                     port_type_ns;
    xmlNodePtr                          sb_node;
    xmlNodePtr                          childNode;

    tmpNode = node;

    if((ctxt == NULL) || (schema == NULL))
    {
        return (NULL);
    }

    binding = xmlMalloc(sizeof(wsdlBinding));
    if(!binding)
    { 
        wsdlErrorMemory(ctxt, "allocating service", NULL);
        return (NULL);
    }
       
    memset(binding, 0, sizeof(wsdlService));
    binding->node = tmpNode;
    binding->name = wsdlGetProp(ctxt, tmpNode, "name");
    if(binding->name == NULL)
    {
        long                            lno = xmlGetLineNo(node);
        wsdlParserError(ctxt, node,
                        WSDL_ERR_NO_NODE,
                        "no 'name' attribute in binding element",
                        NULL, NULL, ctxt->url, lno);
        wsdlBindingFree(binding);
        return (NULL);
    }
    
    port_type_name = wsdlGetQNameProp(ctxt, tmpNode, "type", &port_type_ns);
    binding->type = wsdlFindPortType(ctxt, 
                                     schema, port_type_name, port_type_ns);
    if(!binding->type)
    {
        long                            lno = xmlGetLineNo(node);
        wsdlParserError(ctxt, node,
                        WSDL_ERR_NO_NODE,
                        "no port type matching {%s}%s for binding element",
                        port_type_ns, port_type_name, ctxt->url, lno);
        wsdlBindingFree(binding);
        return (NULL);
    }

    if(GetChildNode(tmpNode, SOAP_BINDING_NODE, &sb_node))
    {
        binding->extensionType = SOAP_BINDING;
        binding->extension = (void *) wsdlParseSoapBinding(ctxt, schema, sb_node);
    }

    childNode = tmpNode->children;
    while(childNode)
    {
        xmlNodePtr                      opChildNode = NULL;

        if(!xmlCompareNodeConst(childNode, WSDL_DOCUMENTATION_NODE))
        {
            /* Ignored for now */
        }
        else if(!xmlCompareNodeConst(childNode, SOAP_BINDING_NODE))
        {
            /* Already handled above */
        }
        else if(!xmlCompareNodeConst(childNode, WSDL_OPERATION_NODE))
        {
            wsdlBindingOperationPtr     bop;
            bop = xmlMalloc(sizeof(wsdlBindingOperation));
            if(!bop)
            {
                wsdlErrorMemory(ctxt, "allocating binding operation", NULL);
                wsdlBindingFree(binding);
                return (NULL);
            }
            memset(bop, 0, sizeof(wsdlBindingOperation));

            bop->name = wsdlGetProp(ctxt, childNode, "name");
            if(!bop->name)
            {
                long                    lno = xmlGetLineNo(childNode);
                wsdlParserError(ctxt, childNode,
                                WSDL_ERR_BINDING,
                                "the binding operation element does not "
                                "have a 'name' attribute",
                                NULL, NULL, ctxt->url, lno);
                wsdlBindingOperationFree(bop);
                wsdlBindingFree(binding);
                return (NULL);
            }
            
            opChildNode = childNode->children;
            while(opChildNode)
            {
                if(!xmlCompareNodeConst(opChildNode, WSDL_INPUT_NODE))
                {
                    xmlNodePtr          inputChildNode = opChildNode->children;
                    void *              soap_param;

                    if(bop->input)
                    {
                        long            lno = xmlGetLineNo(opChildNode);
                        wsdlParserError(ctxt, opChildNode,
                                        WSDL_ERR_SOAP_BINDING,
                                        "Only 1 input element is allowed "
                                        "in the soap binding",
                                        NULL, NULL, ctxt->url, lno);
                        wsdlBindingOperationFree(bop);
                        wsdlBindingFree(binding);
                        return (NULL);
                    }

                    if(inputChildNode)
                    {
                        soap_param = wsdlParseSoapBindingParam(ctxt, 
                                                               schema, 
                                                               inputChildNode);
                        if(!soap_param)
                        {
                            wsdlBindingOperationFree(bop);
                            wsdlBindingFree(binding);
                            return (NULL);
                        }

                        bop->inputType = SOAP_BINDING;
                        bop->input = soap_param;
                    }
                }
                if(!xmlCompareNodeConst(opChildNode, WSDL_OUTPUT_NODE))
                {
                    xmlNodePtr          outputChildNode = opChildNode->children;
                    void *              soap_param;

                    if(bop->output)
                    {
                        long            lno = xmlGetLineNo(opChildNode);
                        wsdlParserError(ctxt, opChildNode,
                                        WSDL_ERR_SOAP_BINDING,
                                        "Only 1 output element is allowed "
                                        "in the soap binding",
                                        NULL, NULL, ctxt->url, lno);
                        wsdlBindingOperationFree(bop);
                        wsdlBindingFree(binding);
                        return (NULL);
                    }

                    if(outputChildNode)
                    {
                        soap_param = wsdlParseSoapBindingParam(ctxt, 
                                                               schema, 
                                                               outputChildNode);
                        if(!soap_param)
                        {
                            wsdlBindingOperationFree(bop);
                            wsdlBindingFree(binding);
                            return (NULL);
                        }

                        bop->outputType = SOAP_BINDING;
                        bop->output = soap_param;
                    }
                }
                if(!xmlCompareNodeConst(opChildNode, WSDL_FAULT_NODE))
                {
                    xmlNodePtr          faultChildNode = opChildNode->children;
                    wsdlBindingOperationFaultPtr bopf;
                    void *              soap_param;

                    bopf = (wsdlBindingOperationFaultPtr) xmlMalloc(
                        sizeof(wsdlBindingOperationFault));
                    if(!bopf)
                    {
                        wsdlErrorMemory(ctxt, "allocating binding operation fault",
                                         NULL);
                        wsdlBindingOperationFree(bop);
                        wsdlBindingFree(binding);
                        return (NULL);
                    }
                    memset(bopf, 0, sizeof(wsdlBindingOperationFault));
                    
                    bopf->name =  wsdlGetProp(ctxt, opChildNode, "name");
                    if(!bopf->name)
                    {
                        long            lno = xmlGetLineNo(opChildNode);
                        wsdlParserError(ctxt, opChildNode, 
                                        WSDL_ERR_SOAP_BINDING,
                                        "binding operation fault "
                                        "requires 'name' attribute",
                                        NULL, NULL, ctxt->url, lno);
                        wsdlBindingOperationFaultFree(bopf);
                        wsdlBindingOperationFree(bop);
                        wsdlBindingFree(binding);
                        return (NULL);
                    }

                    if(faultChildNode)
                    {
                        soap_param = wsdlParseSoapBindingParam(ctxt, 
                                                               schema, 
                                                               faultChildNode);
                        if(!soap_param)
                        {
                            wsdlBindingOperationFaultFree(bopf);
                            wsdlBindingOperationFree(bop);
                            wsdlBindingFree(binding);
                            return (NULL);
                        }
                        
                        bopf->extension = soap_param;
                        bopf->extensionType = SOAP_BINDING;

                        bopf->next = bop->faults;
                        bop->faults = bopf;
                    }
                }
                else
                {
                    if(!xmlCompareNodeConst(opChildNode, SOAP_OPERATION_NODE))
                    {
                        bop->extensionType = SOAP_BINDING;
                        bop->extension = wsdlParseSoapOperation(ctxt, 
                                                                schema, 
                                                                opChildNode);
                        if(!bop->extension)
                        {
                            wsdlBindingOperationFree(bop);
                            wsdlBindingFree(binding);
                            return (NULL);
                        }
                    }
                }

                opChildNode = opChildNode->next;
            }

            if(!binding->operations)
            {
                binding->operations = bop;
            }
            else
            {
                bop->next = binding->operations;
                binding->operations = bop;
            }
        }
        else
        {
            long                        lno = xmlGetLineNo(childNode);

            wsdlParserError(ctxt, node,
                            WSDL_ERR_NOT_WSDL,
                            "Parser failed to find an 'operation' element"
                            " in the binding definition.",
                            NULL, NULL, ctxt->url, lno);
            return (NULL);
        }

        childNode = childNode->next;
    }

    binding->schema = schema;
    return (binding);
}                

static 
void
wsdlBindingFree(
    wsdlBindingPtr                      binding)
{
    if(binding)
    {
        if(binding->operations)
        {
            wsdlBindingOperationFree(binding->operations);
        }

        if(binding->extension)
        {
            if(binding->extensionType == SOAP_BINDING)
            {
                xmlFree(binding->extension);
            }
        }
    }
}

static
void
wsdlBindingHEFree(
    void *                              value,
    xmlChar *                           name)
{
    wsdlBindingFree((wsdlBindingPtr) value);
}

static wsdlServicePtr
wsdlParseService(
    wsdlParserCtxtPtr                   ctxt,
    wsdlSchemaPtr                       schema,
    xmlNodePtr                          node)
{
    xmlNodePtr                          tmpNode;
    wsdlServicePtr                      service;

    tmpNode = node;

    if((ctxt == NULL) || (schema == NULL))
    {
        return (NULL);
    }
    
    service = (wsdlServicePtr) xmlMalloc(sizeof(wsdlService));
    if(service == NULL)
    {
        wsdlErrorMemory(ctxt, "allocating service", NULL);
        return (NULL);
    }
    memset(service, 0, sizeof(wsdlService));
    service->node = tmpNode;
    service->name = wsdlGetProp(ctxt, tmpNode, "name");
    if(service->name == NULL)
    {
        long                            lno = xmlGetLineNo(node);
        wsdlParserError(ctxt, node,
                        WSDL_ERR_NO_NODE,
                        "no 'name' attribute in service element",
                        NULL, NULL, ctxt->url, lno);
        xmlFree(service);
        return (NULL);
    }

    tmpNode = tmpNode->children;
    while(tmpNode)
    {
        if(tmpNode->type == XML_ELEMENT_NODE &&
            !xmlCompareNodeConst(tmpNode, WSDL_DOCUMENTATION_NODE))
        {
            /* Ignored for now */
        }
        else if(tmpNode->type == XML_ELEMENT_NODE &&
           !xmlCompareNodeConst(tmpNode, WSDL_PORT_NODE))
        {
            wsdlPortPtr                 port;
            port = wsdlParsePort(ctxt, schema, tmpNode);
            if(!port)
            {
                xmlFree(service);
                return (NULL);
            }

            if(!service->ports)
            {
                service->ports = port;
            }
            else
            {
                port->next = service->ports;
                service->ports = port;
            }
        }
        else
        {
            long                        lno = xmlGetLineNo(node);
            wsdlParserError(ctxt, node,
                            WSDL_ERR_NOT_WSDL,
                            "Parser failed to find an 'port' element"
                            " in the portType definition.",
                            NULL, NULL, ctxt->url, lno);
            return (NULL);
        }

        tmpNode = tmpNode->next;
    }

    service->schema = schema;
    return (service);
}

static
void
wsdlServiceFree(
    wsdlServicePtr                      service)
{
    if(service)
    {
        if(service->ports)
        {
            wsdlPortFree(service->ports);
        }
        
        xmlFree(service);
    }
}        

static
void
wsdlServiceHEFree(
    void *                              value,
    xmlChar *                           name)
{
    wsdlServiceFree((wsdlServicePtr)value);
}

static
int
wsdlParseTypes(wsdlParserCtxtPtr        ctxt,
               wsdlSchemaPtr            schema,
               xmlNodePtr               node)
{
    xmlSchemaPtr                        xsd;
    xmlSchemaParserCtxtPtr              schemaCtxt;
    xmlDocPtr                           xsdDoc;
    xmlNsPtr                            tmp_ns;
    xmlNodePtr                          root;
    xmlNodePtr                          tnode;

    tnode = node->children;

    if(!xmlCompareNodeConst(tnode, WSDL_DOCUMENTATION_NODE))
    {
        tnode = tnode->next;
    }

    if(xmlCompareNodeConst(tnode, XSD_SCHEMA_NODE))
    {
        long                            lno;

        if(!xmlCompareNodeConst(node, XSD_SCHEMA_NODE))
        {
            tnode = node;
        }
        else
        {
            lno = xmlGetLineNo(node);
            wsdlParserError(ctxt, node,
                            WSDL_ERR_TYPES,
                            "expected 'schema' element but found '%s' instead",
                            NULL, NULL, ctxt->url, lno);
            return (-1);
        }
    }

    if(tnode)
    {
        xsdDoc = xmlNewDoc("1.0");
        xsdDoc->children = tnode;
        root = xmlDocGetRootElement(xsdDoc);

        if(root->nsDef)
        {
            tmp_ns = root->nsDef;
            while(tmp_ns->next)
            {
                tmp_ns = tmp_ns->next;
            }
            
            tmp_ns->next = xmlCopyNamespaceList(node->nsDef);
        }
        else
        {
            root->nsDef = xmlCopyNamespaceList(node->nsDef);
            tmp_ns = root->nsDef;
        }

        if(tmp_ns)
        {
            while(tmp_ns->next)
            {
                tmp_ns = tmp_ns->next;
            }

            tmp_ns->next = xmlCopyNamespaceList(
                (node->parent ? node->parent->nsDef : node->nsDef));
        }
        else
        {
            root->nsDef = xmlCopyNamespaceList(
                (node->parent ? node->parent->nsDef : node->nsDef));
        }
 
        /* need to add defined namespaces to types
         * doc from definitions and types */
        
        schemaCtxt = xmlSchemaNewDocParserCtxt(xsdDoc);
        if(!schemaCtxt)
        {
            return (-1);
        }
        
        xmlSchemaSetParserErrors(schemaCtxt, 
                                 (xmlSchemaValidityErrorFunc) ctxt->err, 
                                 (xmlSchemaValidityWarningFunc) ctxt->warn, 
                                 ctxt->userData);
        
        if(!schema->xmlSchemaImportsDecl)
        {
            schema->xmlSchemaImportsDecl = xmlHashCreate(10);
        }

        xmlHashScan(schema->importsDecl, 
                    wsdlCreateTypesHashScanner, 
                    schema->xmlSchemaImportsDecl);
        xsd = xmlSchemaParse(schemaCtxt, schema->xmlSchemaImportsDecl); 
        if(!xsd) 
        { 
            return (-1); 
        } 
        
        xmlSchemaFreeParserCtxt(schemaCtxt);
        schema->schemaTypes = xsd;
    }

    return (0);
}

xmlSchemaElementPtr
xmlSchemaGetElem(xmlSchemaPtr schema, const xmlChar * name,
                 const xmlChar * Namespace, int level);

xmlSchemaTypePtr
xmlSchemaGetType(xmlSchemaPtr schema, const xmlChar * name,
                 const xmlChar * Namespace);

typedef struct _wsdlFindType
{
    const xmlChar *                     typeLocal;
    const xmlChar *                     typeNs;
    xmlSchemaTypePtr                    type;
} * wsdlFindTypePtr;

static
void
wsdlFindTypeScanner(
    void *                              payload,
    void *                              data,
    xmlChar *                           name)
{
    wsdlImportPtr                       import = (wsdlImportPtr) payload;
    wsdlFindTypePtr                     handle = (wsdlFindTypePtr) data;

    if(!handle->type)
    {
        handle->type = xmlSchemaGetType(import->schema->schemaTypes, 
                                        handle->typeLocal, handle->typeNs);
    }
}

xmlSchemaTypePtr
wsdlFindType(wsdlParserCtxtPtr          ctxt,
             wsdlSchemaPtr              schema,
             const xmlChar *            typeLocal,
             const xmlChar *            typeNs)
{
    xmlSchemaTypePtr                    type;
    struct _wsdlFindType                find_type;

    type = xmlSchemaGetType(schema->schemaTypes, typeLocal, typeNs);
    if(type)
    {
        return (type);
    }

    if(schema->importsDecl)
    {
        find_type.typeLocal = typeLocal;
        find_type.typeNs = typeNs;
        find_type.type = NULL;
        xmlHashScan(schema->importsDecl, wsdlFindTypeScanner, &find_type);

        if(find_type.type)
        {
            return (find_type.type);
        }
    }

    return (NULL);
}

typedef struct _wsdlFindElement
{
    const xmlChar *                     elementLocal;
    const xmlChar *                     elementNs;
    xmlSchemaElementPtr                 element;
} * wsdlFindElementPtr;

static
void
wsdlFindElementScanner(
    void *                              payload,
    void *                              data,
    xmlChar *                           name)
{
    wsdlImportPtr                       import = (wsdlImportPtr) payload;
    wsdlFindElementPtr                  handle = (wsdlFindElementPtr) data;

    if(!handle->element)
    {
        handle->element = xmlSchemaGetElem(
            import->schema->schemaTypes, 
            handle->elementLocal, handle->elementNs, 1);
    }
}

xmlSchemaElementPtr
wsdlFindElement(wsdlParserCtxtPtr       ctxt,
                wsdlSchemaPtr           schema,
                const xmlChar *         elementLocal,
                const xmlChar *         elementNs)
{
    struct _wsdlFindElement             find_element;
    xmlSchemaElementPtr                 element;

    element = xmlSchemaGetElem(schema->schemaTypes, elementLocal, elementNs, 1);
    if(element)
    {
        return (element);
    }
    
    if(schema->importsDecl)
    {
        find_element.elementLocal = elementLocal;
        find_element.elementNs = elementNs;
        find_element.element = NULL;
        xmlHashScan(schema->importsDecl, wsdlFindElementScanner, &find_element);
        if(find_element.element)
        {
            return (find_element.element);
        }
    }

    return (NULL);
}

wsdlMessagePtr
wsdlFindMessage(
    wsdlParserCtxtPtr                   ctxt,
    wsdlSchemaPtr                       schema,
    const xmlChar *                     messageLocal,
    const xmlChar *                     messageNs)
{
    wsdlImportPtr                       import;
    wsdlMessagePtr                      message;

    message = xmlHashLookup2(schema->messagesDecl, messageLocal, messageNs);
    if(!message)
    {
        import = xmlHashLookup(schema->importsDecl, messageNs);
        if(!import)
        {
            return NULL;
        }

        message = wsdlFindMessage(ctxt, import->schema, messageLocal, messageNs);
        if(!message)
        {
            return NULL;
        }
    }

    return message;
}

wsdlPortTypePtr
wsdlFindPortType(
    wsdlParserCtxtPtr                   ctxt,
    wsdlSchemaPtr                       schema,
    const xmlChar *                     portTypeLocal,
    const xmlChar *                     portTypeNs)
{
    wsdlImportPtr                       import;
    wsdlPortTypePtr                     portType;

    portType = xmlHashLookup2(schema->portTypesDecl, portTypeLocal, portTypeNs);
    if(!portType)
    {
        import = xmlHashLookup(schema->importsDecl, portTypeNs);
        if(!import)
        {
            return NULL;
        }

        portType = wsdlFindPortType(ctxt, import->schema, portTypeLocal, portTypeNs);
        if(!portType)
        {
            return NULL;
        }
    }

    return portType;
}

wsdlBindingPtr
wsdlFindBinding(
    wsdlParserCtxtPtr                   ctxt,
    wsdlSchemaPtr                       schema,
    const xmlChar *                     bindingLocal,
    const xmlChar *                     bindingNs)
{
    wsdlImportPtr                       import;
    wsdlBindingPtr                      binding;

    binding = xmlHashLookup2(schema->bindingsDecl, bindingLocal, bindingNs);
    if(!binding)
    {
        import = xmlHashLookup(schema->importsDecl, bindingNs);
        if(!import)
        {
            return NULL;
        }

        binding = wsdlFindBinding(ctxt, import->schema, bindingLocal, bindingNs);
        if(!binding)
        {
            return NULL;
        }
    }

    return binding;
}

wsdlServicePtr
wsdlFindService(
    wsdlParserCtxtPtr                   ctxt,
    wsdlSchemaPtr                       schema,
    const xmlChar *                     serviceLocal,
    const xmlChar *                     serviceNs)
{
    wsdlImportPtr                       import;
    wsdlServicePtr                      service;

    service = xmlHashLookup2(schema->servicesDecl, serviceLocal, serviceNs);
    if(!service)
    {
        import = xmlHashLookup(schema->importsDecl, serviceNs);
        if(!import)
        {
            return NULL;
        }

        service = wsdlFindService(ctxt, import->schema, serviceLocal, serviceNs);
        if(!service)
        {
            return NULL;
        }
    }

    return service;
}

static        
wsdlPartPtr
wsdlParsePart(wsdlParserCtxtPtr         ctxt,
              wsdlSchemaPtr             schema,
              xmlNodePtr                node)
{
    wsdlPartPtr                         part;
    const xmlChar *                     typeLocal;
    const xmlChar *                     typeNs;
    const xmlChar *                     elementLocal;
    const xmlChar *                     elementNs;
    
    part = (wsdlPartPtr) xmlMalloc(sizeof(wsdlPart));
    if(!part)
    {
        wsdlErrorMemory(ctxt, "allocating part structure", node);
        return (NULL);
    }
    memset(part, 0, sizeof(wsdlPart));

    part->name = wsdlGetProp(ctxt, node, "name");

    part->node = node;

    typeLocal = wsdlGetQNameProp(ctxt, node, "type", &typeNs);
    if(typeLocal)
    {
        part->type = wsdlFindType(ctxt, schema, typeLocal, typeNs);
        if(!part->type)
        {
            long                        lno = xmlGetLineNo(node);
            wsdlParserError(ctxt, node,
                            WSDL_ERR_PART, 
                            "WSDL part references invalid type: {%s}%s\n",
                            typeNs, typeLocal, ctxt->url, lno);
            return (NULL);
        }
    }

    elementLocal = wsdlGetQNameProp(ctxt, node, "element", &elementNs);
    if(elementLocal)
    {
        part->element = wsdlFindElement(ctxt, schema, elementLocal, elementNs);
        if(!part->element)
        {
            long                        lno = xmlGetLineNo(node);
            
            wsdlParserError(ctxt, node,
                     WSDL_ERR_PART, 
                     "WSDL part references invalid element: {%s}%s\n",
                     elementNs, elementLocal, ctxt->url, lno);
            return (NULL);
        }
    }

    part->node = node;

    return (part);
}

static
void
wsdlPartFree(
    wsdlPartPtr                         part)
{
    if(part)
    {
        if(part->next)
        {
            wsdlPartFree(part->next);
        }

        xmlFree(part);
    }
}

static
wsdlMessagePtr
wsdlParseMessage(wsdlParserCtxtPtr      ctxt,
                 wsdlSchemaPtr          schema,
                 xmlNodePtr             node)
{
    wsdlMessagePtr                      message;
    wsdlPartPtr                         part;
    xmlNodePtr                          partNode;
    const xmlChar *                     typename = NULL;
    const xmlChar *                     elemname = NULL;
    
    message = (wsdlMessagePtr) xmlMalloc(sizeof(wsdlMessage));
    if(!message)
    {
        wsdlErrorMemory(ctxt, "allocating message structure", node);
        return (NULL);
    }
    memset(message, 0, sizeof(wsdlMessage));

    message->name = wsdlGetProp(ctxt, node, "name");
    if(!message->name)
    {
        long                            lno = xmlGetLineNo(node);
        wsdlParserError(ctxt, node,
                        WSDL_ERR_MESSAGE,
                        "The message element needs a 'name' attribute",
                        NULL, NULL, ctxt->url, lno);
        wsdlMessageFree(message);
        return (NULL);
    }

    message->node = node;

    partNode = node->children;
    while(partNode)
    {
        if(!xmlCompareNodeConst(partNode, WSDL_DOCUMENTATION_NODE))
        {
            /* Ignored for now */
        }
        else if (!xmlCompareNodeConst(partNode, WSDL_PART_NODE))
        {
            part = wsdlParsePart(ctxt, schema, partNode);
            if(!part)
            {
                wsdlMessageFree(message);
                return (NULL);
            }

            if(part->type && part->type->name)
            {
                typename = xmlDictLookup(ctxt->dict, part->type->name, -1);
            }

            if(part->element && part->element->name)
            {
                elemname = xmlDictLookup(ctxt->dict, part->element->name, -1);
            }

            if(!message->parts)
            {
                message->parts = part;
            }
            else
            {
                part->next = message->parts;
                message->parts = part;
            }
        }
        else if(xmlCompareNodeConst(partNode, WSDL_PART_NODE))
        {
            long                        lno = xmlGetLineNo(partNode);
            wsdlParserError(ctxt, node, WSDL_ERR_MESSAGE,
                           "The subelement of the %s message is not a part",
                           partNode->name, NULL, ctxt->url, lno);
            wsdlMessageFree(message);
            return (NULL);
        }

        partNode = partNode->next;
    }

    message->schema = schema;
    return (message);
}

static
void
wsdlMessageFree(
    wsdlMessagePtr                      message)
{
    if(message)
    {
        if(message->parts)
        {
            wsdlPartFree(message->parts);
        }

        xmlFree(message);
    }
}

static
void
wsdlMessageHEFree(
    void *                              value,
    xmlChar *                           name)
{
    wsdlMessageFree((wsdlMessagePtr)value);
}

static
int
wsdlParseSchemaImport(wsdlParserCtxtPtr ctxt,
                      wsdlSchemaPtr     schema,
                      xmlNodePtr        node)
{
    const xmlChar *                     Namespace;
    const xmlChar *                     schemaLocation;
    const xmlChar *                     url = NULL;
    xmlSchemaImportPtr                  xsd_import;
    xmlSchemaParserCtxtPtr              newCtxt;
    xmlSchemaPtr                        xsd_schema = NULL;
    

    Namespace = wsdlGetProp(ctxt, node, "location");
    schemaLocation = wsdlGetProp(ctxt, node, "schemaLocation");
    
    if(schemaLocation)
    {
        xmlChar *base = NULL;
        xmlChar *URI = NULL;
        xmlURIPtr check;

        url = xmlDictLookup(ctxt->dict, (const xmlChar *) schemaLocation, -1);
        if(!url)
        {
            long                        lno = xmlGetLineNo(node);
            wsdlParserError(ctxt, node,
                            WSDL_ERR_IMPORT,
                            "failed to get location of xml schema import",
                            NULL, NULL, ctxt->url, lno);
            return (-1);
        }

        check = xmlParseURI((const char *) url);
        if(check == NULL)
        {
            long                        lno = xmlGetLineNo(node);
            wsdlParserError(
                ctxt, node,
                WSDL_ERR_IMPORT,
                "Import schemaLocation attribute is not a URI: %s\n",
                schemaLocation, NULL, ctxt->url, lno);
            return (-1);
        }
        else
        {
            xmlFreeURI(check);
        }
        
        base = xmlNodeGetBase(node->doc, node);
        if(base == NULL)
        {
            URI = xmlBuildURI(url, node->doc->URL);
        }
        else
        {
            URI = xmlBuildURI(url, base);
            xmlFree(base);
        }
        
        if(URI != NULL)
        {
            url = xmlDictLookup(ctxt->dict, URI, -1);
            xmlFree(URI);
        }
    }

/*     if(!Namespace && !schema->targetNamespace) */
/*     { */
/*         long                            lno = xmlGetLineNo(node); */
/*         wsdlParserError(ctxt, node, */
/*                         WSDL_ERR_IMPORT, */
/*                         "import element does not define a namespace and " */
/*                         "enclosing wsdl does not define a targetNamespace",  */
/*                         NULL, NULL, ctxt->url, lno); */
/*         return (-1); */
/*     } */
    
/*     if(Namespace) */
/*     { */
/*         ns = xmlDictLookup(ctxt->dict, (const xmlChar *) Namespace, -1); */
/*         if(schema->targetNamespace == ns) /\* legal because of dict *\/ */
/*         { */
/*             long                        lno = xmlGetLineNo(node); */
/*             wsdlParserError(ctxt, node, */
/*                            WSDL_ERR_IMPORT, */
/*                            "import attribute namespace: %s\n" */
/*                            "cannot match targetNamespace of enclosing wsdl",  */
/*                            Namespace, NULL, ctxt->url, lno); */
/*             return (-1); */
/*         } */

/*     } */

    if(url)
    {
        newCtxt = xmlSchemaNewParserCtxt(url);
        xmlSchemaSetParserErrors(newCtxt, 
                                 (xmlSchemaValidityErrorFunc) ctxt->err, 
                                 (xmlSchemaValidityWarningFunc) ctxt->warn, 
                                 ctxt->userData);
    }
    else
    {
        long                            lno = xmlGetLineNo(node);
        wsdlParserError(
            ctxt, node,
            WSDL_ERR_IMPORT,
            "Could not find matching xml schema file for import: %s",
            url, NULL, ctxt->url, lno);
        return (-1);
    }

    xmlHashScan(schema->importsDecl, 
                wsdlCreateTypesHashScanner, 
                schema->xmlSchemaImportsDecl);
    xsd_schema = xmlSchemaParse(newCtxt, NULL);
    if(!xsd_schema)
    { 
        return (-1); 
    }

    xmlSchemaFreeParserCtxt(newCtxt);

    xsd_import = (xmlSchemaImportPtr) xmlMalloc(sizeof(xmlSchemaImport));
    if(xsd_import == NULL)
    {
        wsdlErrorMemory(ctxt, "allocating imported wsdl", NULL);
        return (-1);
    }
    
    memset(xsd_import, 0, sizeof(xmlSchemaImport));
    xsd_import->schemaLocation = xmlDictLookup(ctxt->dict, url, -1);
    xsd_import->schema = xsd_schema;
    
    if(!schema->xmlSchemaImportsDecl)
    {
        schema->xmlSchemaImportsDecl = xmlHashCreate(10);
    }
    
    if(xsd_schema->targetNamespace)
    {
        if(xmlHashAddEntry(
               schema->xmlSchemaImportsDecl, 
               xsd_schema->targetNamespace, 
               xsd_import) < 0)
        {
            long                        lno = xmlGetLineNo(
                (xmlNodePtr) ctxt->doc);
            wsdlParserError(ctxt, (xmlNodePtr) ctxt->doc,
                            WSDL_ERR_IMPORT,
                            "wsdl document imports %s multiple times",
                            url, NULL,
                            ctxt->url, lno);
            xmlSchemaFree(xsd_schema);
            xmlFree(xsd_import);
            return (-1);
        }
    }
    else
    {
        if(xmlHashAddEntry(
               schema->xmlSchemaImportsDecl, 
               schema->targetNamespace, 
               xsd_import) < 0)
        {
            long                        lno = xmlGetLineNo(
                (xmlNodePtr) ctxt->doc);
            wsdlParserError(ctxt, (xmlNodePtr) ctxt->doc,
                            WSDL_ERR_IMPORT,
                            "wsdl document imports %s multiple times",
                            url, NULL,
                            ctxt->url, lno);
            xmlSchemaFree(xsd_schema);
            xmlFree(xsd_import);
            return (-1);
        }
    }

    return 0;
}

static
wsdlParserCtxtPtr
wsdlParseImport(wsdlParserCtxtPtr       ctxt,
                wsdlSchemaPtr           schema,
                xmlNodePtr              node)
{
    const xmlChar *                     Namespace;
    const xmlChar *                     location;
    const xmlChar *                     url = NULL;
    const xmlChar *                     ns = NULL;
    wsdlParserCtxtPtr                   newCtxt;
    xmlNodePtr                          root;
    int                                 res;
    wsdlImportPtr                       import = NULL;
    
    Namespace = wsdlGetProp(ctxt, node, "namespace");
    location = wsdlGetProp(ctxt, node, "location");
    
    if(location)
    {
        xmlChar *base = NULL;
        xmlChar *URI = NULL;
        xmlURIPtr check;

        url = xmlDictLookup(ctxt->dict, (const xmlChar *) location, -1);
        if(!url)
        {
            long                        lno = xmlGetLineNo(node);
            wsdlParserError(ctxt, node, 
                            WSDL_ERR_IMPORT,
                            "Failed to get location of wsdl import", 
                            NULL, NULL, ctxt->url, lno);
            return (NULL);
        }

        check = xmlParseURI((const char *) url);
        if (check == NULL) {
            long                        lno = xmlGetLineNo(node);
            wsdlParserError(ctxt, node,
                           WSDL_ERR_IMPORT,
                           "Import location attribute is not an URI: %s\n",
                           location, NULL, ctxt->url, lno);
            return (NULL);
        } else {
            xmlFreeURI(check);
        }
	base = xmlNodeGetBase(node->doc, node);
	if (base == NULL) {
	    URI = xmlBuildURI(url, node->doc->URL);
	} else {
	    URI = xmlBuildURI(url, base);
	    xmlFree(base);
	}
	if (URI != NULL) {
            url = xmlDictLookup(ctxt->dict, URI, -1);
	    xmlFree(URI);
	}
    }

/*     if(!Namespace && !schema->targetNamespace) */
/*     { */
/*         long                            lno = xmlGetLineNo(node); */
/*         wsdlParserError(ctxt, node, */
/*                         WSDL_ERR_IMPORT, */
/*                         "import element does not define a namespace and " */
/*                         "enclosing wsdl does not define a targetNamespace",  */
/*                         NULL, NULL, ctxt->url, lno); */
/*         return (NULL); */
/*     } */
    
/*     if(Namespace) */
/*     { */
/*         ns = xmlDictLookup(ctxt->dict, (const xmlChar *) Namespace, -1); */
/*         if(schema->targetNamespace == ns) /\* legal because of dict *\/ */
/*         { */
/*             long                        lno = xmlGetLineNo(node); */
/*             wsdlParserError(ctxt, node, */
/*                            WSDL_ERR_IMPORT, */
/*                            "import attribute namespace: %s\n" */
/*                            "cannot match targetNamespace of enclosing wsdl",  */
/*                            Namespace, NULL, ctxt->url, lno); */
/*             return (NULL); */
/*         } */

/*     } */

    if(url)
    {
        newCtxt = wsdlNewParserCtxt(url);
        xmlDictFree(newCtxt->dict);
        xmlDictReference(ctxt->dict);
        newCtxt->dict = ctxt->dict;
        newCtxt->url = xmlDictLookup(newCtxt->dict, url, -1);
    }
    else
    {
        long                            lno = xmlGetLineNo(node);
        wsdlParserError(ctxt, node,
                       WSDL_ERR_IMPORT,
                       "Could not find matching wsdl file for import: %s",
                       url, NULL, ctxt->url, lno);
        return (NULL);
    }

    newCtxt->enclosingCtxt = ctxt;
    if(ns)
    {
        newCtxt->enclosingNs = ns;
    }
    else
    {
        newCtxt->enclosingNs = schema->targetNamespace;
    }

    res = wsdlParseInternal(newCtxt);
    if(res != 0)
    {
        return (NULL);
    }

    root = xmlDocGetRootElement(newCtxt->doc);
    if(root == NULL)
    {
        long                            lno = xmlGetLineNo(
            (xmlNodePtr) newCtxt->doc);
        wsdlParserError(newCtxt, (xmlNodePtr) newCtxt->doc,
                        WSDL_ERR_NOROOT,
                        "wsdl document has no root", NULL, NULL,
                        newCtxt->url, lno);
        if(!newCtxt->preserve)
        {
            xmlFreeDoc(newCtxt->doc);
        }
        return (NULL);
    }

    wsdlCleanupDoc(newCtxt, root);

    import = (wsdlImportPtr) xmlMalloc(sizeof(wsdlImport));
    if(import == NULL)
    {
        wsdlErrorMemory(newCtxt, "allocating imported wsdl", root);
        return (NULL);
    }

    memset(import, 0, sizeof(wsdlImport));
    import->wsdlLocation = xmlDictLookup(ctxt->dict, url, -1);
    import->schema = wsdlDoParse(newCtxt, schema, root);
    if(!import->schema)
    {
        long                            lno = xmlGetLineNo(
            (xmlNodePtr) newCtxt->doc);
        wsdlParserError(newCtxt, (xmlNodePtr) newCtxt->doc,
                        WSDL_ERR_IMPORT,
                        "Could not parse types in wsdl document", 
                        NULL, NULL,
                        newCtxt->url, lno);
        xmlFree(import);
        return (NULL);
    }

/*     if(Namespace) */
/*     { */
/*         ns = xmlDictLookup(ctxt->dict, (const xmlChar *) Namespace, -1); */
/*         if(import->schema->targetNamespace &&  */
/*            ns != import->schema->targetNamespace) */
/*         { */
/*             long                            lno = xmlGetLineNo( */
/*                 (xmlNodePtr) newCtxt->doc); */
/*             wsdlParserError( */
/*                 newCtxt, (xmlNodePtr) newCtxt->doc, */
/*                 WSDL_ERR_IMPORT, */
/*                 "import element has namespace attr: %s\n" */
/*                 "which does not match targetNamespace " */
/*                 "of imported document: %s", */
/*                 Namespace, import->schema->targetNamespace, */
/*                 newCtxt->url, lno); */
/*             wsdlSchemaFree(import->schema); */
/*             xmlFree(import); */
/*             return (NULL); */
/*         } */
/*     } */
/*     else */
/*     { */
/*         if(import->schema->targetNamespace) */
/*         { */
/*             long                            lno = xmlGetLineNo( */
/*                 (xmlNodePtr) newCtxt->doc); */

/*             wsdlParserError( */
/*                 newCtxt, (xmlNodePtr) newCtxt->doc, */
/*                 WSDL_ERR_IMPORT, */
/*                 "import element does not have namespace attribute " */
/*                 "but imported schema defines targetNamespace: %s", */
/*                 import->schema->targetNamespace, NULL, */
/*                 newCtxt->url, lno); */
/*             wsdlSchemaFree(import->schema); */
/*             xmlFree(import); */
/*             return (NULL); */
/*         } */

/*         import->schema->targetNamespace = xmlDictLookup( */
/*             ctxt->dict, newCtxt->enclosingNs, -1); */
/*     } */

    newCtxt->schema = import->schema;

    if(!schema->importsDecl)
    {
        schema->importsDecl = xmlHashCreate(10);
    }
        
    if(Namespace)
    {
        if(xmlHashAddEntry(schema->importsDecl, Namespace, import) < 0)
        {
            long                            lno = xmlGetLineNo(
            (xmlNodePtr) newCtxt->doc);
            wsdlParserError(newCtxt, (xmlNodePtr) newCtxt->doc,
                            WSDL_ERR_IMPORT,
                            "wsdl document imports %s multiple times", 
                            import->wsdlLocation, NULL,
                            newCtxt->url, lno);
            xmlFree(import);
            return (NULL);
        }
    }
    else
    {
        if(xmlHashAddEntry(schema->importsDecl, schema->targetNamespace, import) < 0)
        {
            long                            lno = xmlGetLineNo(
            (xmlNodePtr) newCtxt->doc);
            wsdlParserError(newCtxt, (xmlNodePtr) newCtxt->doc,
                            WSDL_ERR_IMPORT,
                            "wsdl document imports %s multiple times", 
                            import->wsdlLocation, NULL,
                            newCtxt->url, lno);
            xmlFree(import);
            return (NULL);
        }
    }

    return (newCtxt);
}

static
wsdlParamPtr
wsdlParseParam(wsdlParserCtxtPtr        ctxt,
               wsdlSchemaPtr            schema,
               xmlNodePtr               node)
{
    wsdlParamPtr                        param;
    const xmlChar *                     message_name;
    const xmlChar *                     message_ns;

    param = (wsdlParamPtr) xmlMalloc(sizeof(wsdlParam));
    if(!param)
    {
        wsdlErrorMemory(ctxt, "allocating structure for param", node);
        return (NULL);
    }
    memset(param, 0, sizeof(wsdlParam));
    
    param->name = wsdlGetProp(ctxt, node, "name");

    param->Action = wsdlGetProp(ctxt, node, "Action");

    param->node = node;
    
    message_name = wsdlGetQNameProp(ctxt, node, "message", &message_ns);
    if(!message_name)
    {
        long                            lno = xmlGetLineNo(node);
        wsdlParserError(ctxt, node, WSDL_ERR_PARAM,
                        "no 'message' attribute found for param element",
                        NULL, NULL, ctxt->url, lno);
        xmlFree(param);
        return (NULL);
    }

    param->message = wsdlFindMessage(ctxt, schema, message_name, message_ns);
    if(!param->message)
    {
        long                            lno = xmlGetLineNo(node);
        wsdlParserError(ctxt, node, WSDL_ERR_PARAM,
                        "invalid reference to message {%s}%s in param",
                        message_ns, message_name, ctxt->url, lno);
        xmlFree(param);
        return (NULL);
    }
    
    return (param);
}
    

static
wsdlFaultPtr
wsdlParseFault(wsdlParserCtxtPtr        ctxt,
               wsdlSchemaPtr            schema,
               xmlNodePtr               node)
{
    wsdlFaultPtr                        fault;
    const xmlChar *                     message_ns;
    const xmlChar *                     message_name;

    fault = (wsdlFaultPtr) xmlMalloc(sizeof(wsdlFault));
    if(!fault)
    {
        wsdlErrorMemory(ctxt, "allocating structure for param", node);
        return (NULL);
    }
    memset(fault, 0, sizeof(wsdlFault));
    
    fault->name = wsdlGetProp(ctxt, node, "name");
    if(!fault->name)
    {
        long                            lno = xmlGetLineNo(node);
        wsdlParserError(ctxt, node, WSDL_ERR_FAULT,
                        "no 'name' attribute found for fault element",
                        NULL, NULL, ctxt->url, lno);
        xmlFree(fault);
        return (NULL);
    }

    fault->node = node;
        
    message_name = wsdlGetQNameProp(ctxt, node, "message", &message_ns);
    if(!message_name)
    {
        long                            lno = xmlGetLineNo(node);
        wsdlParserError(ctxt, node, WSDL_ERR_FAULT,
                        "no 'message' attribute found for fault element",
                        NULL, NULL, ctxt->url, lno);
        xmlFree(fault);
        return (NULL);
    }

    fault->message = wsdlFindMessage(ctxt, schema, message_name, message_ns);
    if(!fault->message)
    {
        long                            lno = xmlGetLineNo(node);
        wsdlParserError(ctxt, node, WSDL_ERR_PARAM,
                        "invalid reference to message {%s}%s in fault",
                        message_ns, message_name, ctxt->url, lno);
        xmlFree(fault);
        return (NULL);
    }
    
    return (fault);
}

static
wsdlOperationPtr
wsdlParseOperation(wsdlParserCtxtPtr    ctxt,
                   wsdlSchemaPtr        schema,
                   xmlNodePtr           node)
{
    wsdlOperationPtr                    op;
    xmlNodePtr                          childNode;

    op = (wsdlOperationPtr) xmlMalloc(sizeof(wsdlOperation));
    if(!op)
    {
        wsdlErrorMemory(ctxt, "allocating structure for operation", node);
        return (NULL);
    }
    memset(op, 0, sizeof(wsdlOperation));

    op->name = wsdlGetProp(ctxt, node, "name");
    if(!op->name)
    {
        long                            lno = xmlGetLineNo(node);
        wsdlParserError(ctxt, node, WSDL_ERR_OP,
                        "operation must define a 'name' attribute",
                        NULL, NULL, ctxt->url, lno);
        
        return (NULL);
    }

    op->node = node;

    childNode = node->children;
    while(childNode)
    {
        if(!xmlCompareNodeConst(childNode, WSDL_DOCUMENTATION_NODE))
        {
            /* Ignored for now */
        }
        else if(!xmlCompareNodeConst(childNode, WSDL_INPUT_NODE))
        {
            op->type = (op->output) ? SOLICIT_RESPONSE_OPERATION : ONE_WAY_OPERATION;
            op->input = wsdlParseParam(ctxt, schema, childNode);
            if(!op->input)
            {
                return (NULL);
            }
        }
        else if(!xmlCompareNodeConst(childNode, WSDL_OUTPUT_NODE))
        {
            op->type = (op->input) ? REQUEST_RESPONSE_OPERATION : NOTIFICATION_OPERATION;
            op->output = wsdlParseParam(ctxt, schema, childNode);
            if(!op->output)
            {
                return (NULL);
            }
        }
        else if(!xmlCompareNodeConst(childNode, WSDL_FAULT_NODE))
        {
            wsdlFaultPtr                fault = NULL;
            fault = wsdlParseFault(ctxt, schema, childNode);
            if(!fault)
            {
                return (NULL);
            }

            if(!op->faults)
            {
                op->faults = fault;
            }
            else
            {
                fault->next = op->faults;
                op->faults = fault;
            }
        }
        else
        {
            /* unmatched operation child element */
            return (NULL);
        }

        childNode = childNode->next;
    }

    op->node = node;
    
    return (op);
}

static
void
wsdlOperationFree(
    wsdlOperationPtr                    op)
{
    if(op)
    {
        if(op->next)
        {
            wsdlOperationFree(op->next);
        }
        if(op->input)
        {
            xmlFree(op->input);
        }
        if(op->output)
        {
            xmlFree(op->output);
        }
        if(op->faults)
        {
            wsdlFaultFree(op->faults);
        }

        xmlFree(op);
    }
}

static
void
wsdlFaultFree(
    wsdlFaultPtr                        fault)
{
    if(fault)
    {
        if(fault->next)
        {
            wsdlFaultFree(fault->next);
        }
        
        xmlFree(fault);
    }
}

static
wsdlPortTypePtr
wsdlParsePortType(wsdlParserCtxtPtr     ctxt,
                  wsdlSchemaPtr         schema,
                  xmlNodePtr            node)
{
    xmlChar *                           rp_qnames;
    xmlChar *                           impls_qnames;
    wsdlPortTypePtr                     portType;
    wsdlOperationPtr                    operation;
    xmlNodePtr                          opNode;

    portType = (wsdlPortTypePtr) xmlMalloc(sizeof(wsdlPortType));
    if(!portType)
    {
        wsdlErrorMemory(ctxt, "allocating space for portType structure", node);
        return (NULL);
    }
    memset(portType, 0, sizeof(wsdlPortType));
    
    portType->name = wsdlGetProp(ctxt, node, "name");
    if(!portType->name)
    {
        long                            lno = xmlGetLineNo(node);
        wsdlParserError(ctxt, node, WSDL_ERR_PORT_TYPE,
                        "portType must define a 'name' attribute",
                        NULL, NULL, ctxt->url, lno);
        return (NULL);
    }

    portType->node = node;

    rp_qnames = xmlGetNsProp(node, "ResourceProperties", WSRP_NS);
    if(rp_qnames)
    {
        globus_i_wsdl_split_qnames(
            schema->targetNamespace,
            node, rp_qnames, &portType->ResourceProperties);
    }

    impls_qnames = xmlGetNsProp(node, "implements", GTWSDL_NS);
    if(impls_qnames)
    {
        globus_i_wsdl_split_qnames(
            schema->targetNamespace,
            node, 
            impls_qnames, &portType->implements);
    }
        
    opNode = node->children;
    while(opNode)
    {
        if(!xmlCompareNodeConst(opNode, WSDL_DOCUMENTATION_NODE))
        {
            /* Ignored for now */
        }
        else if(!xmlCompareNodeConst(opNode, WSDL_OPERATION_NODE))
        {
            operation = wsdlParseOperation(ctxt, schema, opNode);
            if(!operation)
            {
                return (NULL);
            }

            if(portType->operations)
            {
                operation->next = portType->operations;
            }

            portType->operations = operation;
        }
        else
        {
            long                        lno = xmlGetLineNo(opNode);
            wsdlParserError(ctxt, opNode, WSDL_ERR_PORT_TYPE,
                            "child element of 'portType' must be an operation",
                            NULL, NULL, ctxt->url, lno);
            return (NULL);
        }

        opNode = opNode->next;
    }

    portType->schema = schema;
    return (portType);
}

static
void
wsdlPortTypeFree(
    wsdlPortTypePtr                     portType)
{
    if(portType)
    {
        if(portType->operations)
        {
            wsdlOperationFree(portType->operations);
        }

        if(portType->ResourceProperties)
        {
            xsdQNameFree(portType->ResourceProperties);
        }

        if(portType->implements)
        {
            xsdQNameFree(portType->implements);
        }

        xmlFree(portType);
    }
}

static
void
wsdlPortTypeHEFree(
    void *                              value,
    xmlChar *                           name)
{
    wsdlPortTypeFree((wsdlPortTypePtr) value);
}

static
void
wsdlImportHEFree(
    void *                              value,
    xmlChar *                           name)
{
    wsdlImportPtr                       import;

    import = (wsdlImportPtr) value;

    if(import)
    {
        if(import->schema)
        {
            wsdlSchemaFree(import->schema);
        }

        xmlFree(import);
    }
}

void
wsdlSchemaFree(
    wsdlSchemaPtr                       schema)
{
    if(schema)
    {
        if(schema->schemaTypes)
        {
            xmlSchemaFree(schema->schemaTypes);
/*             xmlSchemaPtr                xsd = schema->schemaTypes; */
/*             while(xsd) */
/*             { */
/*                 xmlSchemaPtr            next_xsd = NULL; */

/*                 if(xsd->_private) */
/*                 { */
/*                     next_xsd = (xmlSchemaPtr) xsd->_private; */
/*                 } */
                
/*                 xsd->_private = NULL; */
/*                 xmlSchemaFree(xsd); */

/*                 xsd = next_xsd; */
/*             } */
        }

        if(schema->messagesDecl)
        {
            xmlHashFree(schema->messagesDecl, wsdlMessageHEFree);
        }

        if(schema->portTypesDecl)
        {
            xmlHashFree(schema->portTypesDecl, wsdlPortTypeHEFree);
        }

        if(schema->bindingsDecl)
        {
            xmlHashFree(schema->bindingsDecl, wsdlBindingHEFree);
        }

        if(schema->servicesDecl)
        {
            xmlHashFree(schema->servicesDecl, wsdlServiceHEFree);
        }

        if(schema->importsDecl)
        {
            xmlHashFree(schema->importsDecl, wsdlImportHEFree);
        }

        if(schema->dict)
        {
            xmlDictFree(schema->dict);
        }
    }
}

static
int
wsdlParseTopLevel(wsdlParserCtxtPtr     ctxt,
                  wsdlSchemaPtr         schema,
                  xmlNodePtr            node)
{
    int res = 0;
    xmlNodePtr                          child;
    
    child = node;
    if(!xmlCompareNodeConst(child, WSDL_TYPES_NODE))
    {
        res = wsdlParseTypes(ctxt, schema, child);
    }
    else if(!xmlCompareNodeConst(child, WSDL_MESSAGE_NODE))
    {
        wsdlMessagePtr                  new_message;
        const xmlChar *                 tns = NULL;
        
        new_message = wsdlParseMessage(ctxt, schema, child);
        if(!new_message)
        {
            return (-1);
        }
        
        if(!schema->messagesDecl)
        {
            schema->messagesDecl = xmlHashCreate(10);
        }

        if(schema->targetNamespace)
        {
            tns = xmlDictLookup(ctxt->dict, schema->targetNamespace, -1);
        }
        else if(child && child->ns && child->ns->href)
        {
            tns = xmlDictLookup(ctxt->dict, child->ns->href, -1);
        }
        
        xmlHashAddEntry2(
            schema->messagesDecl, new_message->name, tns, new_message);
    }
    else if(!xmlCompareNodeConst(child, WSDL_PORTTYPE_NODE))
    {
        wsdlPortTypePtr                 port_type;
        const xmlChar *                 tns;

        port_type = wsdlParsePortType(ctxt, schema, child);
        if(!port_type)
        {
            return (-1);
        }

        if(!schema->portTypesDecl)
        {
            schema->portTypesDecl = xmlHashCreate(10);
        }

        tns = schema->targetNamespace;
        if(!tns)
        {
            if(child->ns && child->ns->href)
            {
                tns = child->ns->href;
            }
        }
        
        xmlHashAddEntry2(
            schema->portTypesDecl, port_type->name, tns, port_type);
    }
    else if(!xmlCompareNodeConst(child, WSDL_BINDING_NODE))
    {
        wsdlBindingPtr                  binding;
        const xmlChar *                 tns;
        
        binding = wsdlParseBinding(ctxt, schema, child);
        if(!binding)
        {
            return (-1);
        }

        if(!schema->bindingsDecl)
        {
            schema->bindingsDecl = xmlHashCreate(10);
        }
        
        tns = schema->targetNamespace;
        if(!tns)
        {
            if(child->ns && child->ns->href)
            {
                tns = child->ns->href;
            }
        }
        
        xmlHashAddEntry2(schema->bindingsDecl, binding->name, tns, binding);
    }
    else if(!xmlCompareNodeConst(child, WSDL_SERVICE_NODE))
    {
        wsdlServicePtr                  service;
        const xmlChar *                 tns;
        
        service = wsdlParseService(ctxt, schema, child);
        if(!service)
        {
            return (-1);
        }
        
        if(!schema->servicesDecl)
        {
            schema->servicesDecl = xmlHashCreate(10);
        }

        tns = schema->targetNamespace;
        if(!tns)
        {
            if(child->ns && child->ns->href)
            {
                tns = child->ns->href;
            }
        }
        
        xmlHashAddEntry2(schema->servicesDecl, service->name, tns, service);
    }
    else if(!xmlCompareNodeConst(child, WSDL_IMPORT_NODE))
    {
        /* already handled */
    }
    else if(!xmlCompareNodeConst(child, XSD_IMPORT_NODE))
    {
        /* already handled */
    }
    else if(!xmlCompareNodeConst(child, WSDL_DOCUMENTATION_NODE))
    {
        /* Ignored for now */
    }
    else
    {
        long                            lno = xmlGetLineNo(node);
        res = -1;
        wsdlParserError(ctxt, node,
                        WSDL_ERR_TOPLEVEL,
                        "Failed to parse top level WSDL element %s",
                        child->name, NULL, ctxt->url, lno);
    }
    
    return (res);
}

static
int
wsdlParseImports(wsdlParserCtxtPtr     ctxt,
                 wsdlSchemaPtr         schema,
                 xmlNodePtr            node)
{
    int res = 0;
    xmlNodePtr                          child;
    
    child = node;
    if(!xmlCompareNodeConst(child, WSDL_IMPORT_NODE))
    {
        const char *                    tns;
        wsdlParserCtxtPtr               import_ctxt;
        tns = schema->targetNamespace;
        import_ctxt = wsdlParseImport(ctxt, schema, child);
        if(!import_ctxt)
        {
            return (-1);
        }
        schema->targetNamespace = tns;
    }
    else if(!xmlCompareNodeConst(child, XSD_IMPORT_NODE))
    {
        res = wsdlParseSchemaImport(ctxt, schema, child);
        if(res != 0)
        {
            return (-1);
        }
    }

    return (res);
}

static
void
xmlSchemaCreateTypesHashScanner(
    void *                              import_payload,
    void *                              data,
    xmlChar *                           schema_namespace);

static
void
wsdlCreateTypesHashScanner(
    void *                              import_payload,
    void *                              data,
    xmlChar *                           schema_namespace)
{
    wsdlImportPtr                       import;
    xmlHashTablePtr                     xsd_imports;
    xmlSchemaImportPtr                  xsd_import;

    xsd_imports = (xmlHashTablePtr) data;
    import = (wsdlImportPtr) import_payload;
    
    if(import->schema->schemaTypes)
    {
        xsd_import = (xmlSchemaImportPtr) xmlMalloc(sizeof(xmlSchemaImport));
        if(!xsd_import)
        {
            wsdlErrorMemory(
                NULL, "Memory allocation of schemaImport failed", NULL);
            return;
        }

        memset(xsd_import, 0, sizeof(xmlSchemaImport));

        xsd_import->schemaLocation = import->wsdlLocation;
        xsd_import->schema = import->schema->schemaTypes;
        xsd_import->preserve = 1;
	xmlHashAddEntry(
            xsd_imports, xsd_import->schema->targetNamespace, xsd_import);
        if(import->schema->xmlSchemaImportsDecl)
        {
            xmlHashScan(import->schema->xmlSchemaImportsDecl,
                        xmlSchemaCreateTypesHashScanner,
                        xsd_imports);
        }
    }
}

static
void
xmlSchemaCreateTypesHashScanner(
    void *                              import_payload,
    void *                              data,
    xmlChar *                           schema_namespace)
{
    xmlHashTablePtr                     xsd_imports;
    xmlSchemaImportPtr                  xsd_import;
    xmlSchemaImportPtr                  new_import;

    xsd_imports = (xmlHashTablePtr) data;
    xsd_import = (xmlSchemaImportPtr) import_payload;
    
    new_import = (xmlSchemaImportPtr) xmlMalloc(sizeof(xmlSchemaImport));
    if(!new_import)
    {
        wsdlErrorMemory(
            NULL, "Memory allocation of schemaImport failed", NULL);
        return;
    }

    memset(new_import, 0, sizeof(xmlSchemaImport));

    new_import->schemaLocation = xsd_import->schemaLocation;
    new_import->schema = xsd_import->schema;
    new_import->preserve = 1;
    xmlHashAddEntry(xsd_imports, schema_namespace, new_import);
}

