/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */


#ifndef __WSDL_INTERNALS_H__
#define __WSDL_INTERNALS_H__

#include "libxml/xmlschemas.h"
#include "libxml/schemasInternals.h"
#include "libxml/tree.h"

#ifdef __cplusplus
extern "C" { 
#endif

#if 0
}
#endif

typedef struct {
    const char * ns;
    const char * name;
} CONST_NODE;
    
#define WSDL_NS "http://schemas.xmlsoap.org/wsdl/"
#define SOAP_NS "http://schemas.xmlsoap.org/wsdl/soap/"
#define XSD_NS "http://www.w3.org/2001/XMLSchema"

#define WSRP_NS "http://docs.oasis-open.org/wsrf/2004/06/wsrf-WS-ResourceProperties-1.2-draft-01.xsd"
#define GTWSDL_NS "http://www.globus.org/namespaces/2004/01/GTWSDLExtensions"

#define WSA_NS "http://schemas.xmlsoap.org/ws/2004/08/addressing"

extern const CONST_NODE WSDL_DEFINITIONS_NODE;
extern const CONST_NODE WSDL_TYPES_NODE;
extern const CONST_NODE WSDL_MESSAGE_NODE;
extern const CONST_NODE WSDL_PART_NODE;
extern const CONST_NODE WSDL_PORTTYPE_NODE;
extern const CONST_NODE WSDL_OPERATION_NODE;
extern const CONST_NODE WSDL_INPUT_NODE;
extern const CONST_NODE WSDL_OUTPUT_NODE;
extern const CONST_NODE WSDL_FAULT_NODE;
extern const CONST_NODE WSDL_BINDING_NODE;
extern const CONST_NODE WSDL_SERVICE_NODE;
extern const CONST_NODE WSDL_PORT_NODE;
extern const CONST_NODE WSDL_IMPORT_NODE;

extern const CONST_NODE SOAP_BINDING_NODE;
extern const CONST_NODE SOAP_OPERATION_NODE;
extern const CONST_NODE SOAP_BODY_NODE;
extern const CONST_NODE SOAP_HEADER_NODE;
extern const CONST_NODE SOAP_FAULT_NODE;
extern const CONST_NODE SOAP_ADDRESS_NODE;

extern const CONST_NODE XSD_SCHEMA_NODE;

struct _xsdQName {
    const xmlChar *                     name;
    const xmlChar *                     Namespace;
    struct _xsdQName *                  next;
};

typedef struct _xsdQName                xsdQName;
typedef xsdQName *                      xsdQNamePtr;

struct _wsdlPart {
    const xmlChar *                     name;
    xmlSchemaTypePtr                    type;
    xmlSchemaElementPtr                 element;
    xmlNodePtr                          node;
    struct _wsdlPart *                  next;
};

typedef struct _wsdlPart                wsdlPart;
typedef wsdlPart *                      wsdlPartPtr;

struct _wsdlMessage {
    const xmlChar *                     name;
    wsdlPartPtr                         parts;
    xmlNodePtr                          node;
    struct _wsdlSchema *                schema;
};

typedef struct _wsdlMessage             wsdlMessage;
typedef wsdlMessage *                   wsdlMessagePtr;

typedef enum {
    ONE_WAY_OPERATION,
    REQUEST_RESPONSE_OPERATION,
    SOLICIT_RESPONSE_OPERATION,
    NOTIFICATION_OPERATION
} wsdlOperationType;

struct _wsdlParam {
    const xmlChar *                     name;
    wsdlMessagePtr                      message;
    const xmlChar *                     Action;

    xmlNodePtr                          node;
};

typedef struct _wsdlParam               wsdlParam;
typedef wsdlParam *                     wsdlParamPtr;

struct _wsdlFault {
    const xmlChar *                     name;
    wsdlMessagePtr                      message;
    void *                              extensions;
    struct _wsdlFault *                 next;
    xmlNodePtr                          node;
};

typedef struct _wsdlFault               wsdlFault;
typedef wsdlFault *                     wsdlFaultPtr;

struct _wsdlOperation {
    wsdlOperationType                   type;
    const xmlChar *                     name;

    wsdlParamPtr                        input;
    wsdlParamPtr                        output;
    wsdlFaultPtr                        faults;

    xmlNodePtr                          node;

    struct _wsdlOperation *             next;
};

typedef struct _wsdlOperation           wsdlOperation;
typedef wsdlOperation *                 wsdlOperationPtr;

struct _wsdlPortType {
    const xmlChar *                     name;
    wsdlOperationPtr                    operations;
    xsdQNamePtr                         ResourceProperties;
    xsdQNamePtr                         implements;
    xmlNodePtr                          node;
    struct _wsdlSchema *                schema;
};

typedef struct _wsdlPortType            wsdlPortType;
typedef wsdlPortType *                  wsdlPortTypePtr;

typedef enum {
    SOAP_BINDING
} wsdlBindingType;

struct _wsdlBindingOperationFault {
    const xmlChar *                     name;
    void *                              extension;
    wsdlBindingType                     extensionType;
    struct _wsdlBindingOperationFault * next;
};

typedef struct _wsdlBindingOperationFault wsdlBindingOperationFault;
typedef wsdlBindingOperationFault *       wsdlBindingOperationFaultPtr;
    
struct _wsdlBindingOperation {
    const xmlChar *                     name;
    void *                              input;
    wsdlBindingType                     inputType;
    void *                              output;
    wsdlBindingType                     outputType;
    wsdlBindingOperationFaultPtr        faults;
    void *                              extension;
    wsdlBindingType                     extensionType;

    xmlNodePtr                          node;
    struct _wsdlBindingOperation *      next;
};

typedef struct _wsdlBindingOperation    wsdlBindingOperation;
typedef wsdlBindingOperation *          wsdlBindingOperationPtr;

struct _wsdlBinding {
    const xmlChar *                     name;
    wsdlPortTypePtr                     type;
    wsdlBindingOperationPtr             operations;
    void *                              extension;
    wsdlBindingType                     extensionType;
    xmlNodePtr                          node;
    struct _wsdlSchema *                schema;
};

typedef struct _wsdlBinding             wsdlBinding;
typedef wsdlBinding *                   wsdlBindingPtr;

struct _wsdlPort {
    const xmlChar *                     name;
    wsdlBindingPtr                      binding;
    void *                              extension;
    wsdlBindingType                     extensionType;

    xmlNodePtr                          node;
    struct _wsdlPort *                  next;
};

typedef struct _wsdlPort                wsdlPort;
typedef wsdlPort *                      wsdlPortPtr;

struct _wsdlService {
    const xmlChar *                     name;
    wsdlPortPtr                         ports;
    void *                              extension;
    wsdlBindingType                     extensionType;

    xmlNodePtr                          node;
    struct _wsdlSchema *                schema;
};

typedef struct _wsdlService             wsdlService;
typedef wsdlService *                   wsdlServicePtr;

struct _wsdlSchema {

    const xmlChar *                     targetNamespace;
    xmlDocPtr                           doc;
    xmlSchemaPtr                        schemaTypes;
    xmlHashTablePtr                     messagesDecl;
    xmlHashTablePtr                     portTypesDecl;
    xmlHashTablePtr                     bindingsDecl;
    xmlHashTablePtr                     servicesDecl;
    xmlHashTablePtr                     importsDecl;
    xmlHashTablePtr                     xmlSchemaImportsDecl;
    xmlDictPtr                          dict;
};

typedef struct _wsdlSchema              wsdlSchema;
typedef wsdlSchema *                    wsdlSchemaPtr;

struct _wsdlImport {
    const xmlChar *                     wsdlLocation;
    wsdlSchemaPtr                       schema;
};

typedef struct _wsdlImport              wsdlImport;
typedef wsdlImport *                    wsdlImportPtr;

typedef enum {
    RPC_STYLE,
    DOCUMENT_STYLE
} soapOperationStyleType;

struct _soapBinding {
    soapOperationStyleType              style;
    const xmlChar *                     transport;
    xmlNodePtr                          node;
};

typedef struct _soapBinding             soapBinding;
typedef soapBinding *                   soapBindingPtr;

struct _soapOperation {
    const xmlChar *                     soapAction;
    soapOperationStyleType              style;
    xmlNodePtr                          node;
};

typedef struct _soapOperation           soapOperation;
typedef soapOperation *                 soapOperationPtr;

typedef enum {
    LITERAL_USE,
    ENCODED_USE
} soapUseType;

struct _soapBody {
    const xmlChar *                     encodingStyle;
    const xmlChar *                     parts;
    soapUseType                         use;
    const xmlChar *                     Namespace;
    xmlNodePtr                          node;
};

typedef struct _soapBody                soapBody;
typedef soapBody *                      soapBodyPtr;

struct _soapFault {
    const xmlChar *                     encodingStyle;
    soapUseType                         use;
    const xmlChar *                     Namespace;
    xmlNodePtr                          node;
};

typedef struct _soapFault               soapFault;
typedef soapFault *                     soapFaultPtr;

struct _soapHeader {
    const xmlChar *                     message;
    const xmlChar *                     parts;
    soapUseType                         use;
    const xmlChar *                     encodingStyle;
    const xmlChar *                     Namespace;
    xmlNodePtr                          node;
};

typedef struct _soapHeader              soapHeader;
typedef soapHeader *                    soapHeaderPtr;

struct _soapBindingParam {
    soapHeaderPtr                       header;
    soapBodyPtr                         body;
    soapFaultPtr                        fault;
};

typedef struct _soapBindingParam        soapBindingParam;
typedef soapBindingParam *              soapBindingParamPtr;

struct _soapAddress {
    const xmlChar *                     location;
    xmlNodePtr                          node;
};

typedef struct _soapAddress             soapAddress;
typedef soapAddress *                   soapAddressPtr;

#ifdef __cplusplus
}
#endif

#endif
