//(c) International Business Machines Corporation, 2005.
//(c) University of Edinburgh, 2005.
//See OGSA-DAI-Licence.txt for licencing information.

package uk.org.ogsadai.examples.clienttoolkit;

import uk.org.ogsadai.client.toolkit.GenericServiceFetcher;
import uk.org.ogsadai.client.toolkit.Response;
import uk.org.ogsadai.client.toolkit.activity.ActivityRequest;
import uk.org.ogsadai.client.toolkit.activity.delivery.DTOutputStream;
import uk.org.ogsadai.client.toolkit.activity.sql.SQLQuery;
import uk.org.ogsadai.client.toolkit.activity.sql.WebRowSet;
import uk.org.ogsadai.client.toolkit.activity.misc.DataStore;
import uk.org.ogsadai.client.toolkit.properties.RequestStatus;
import uk.org.ogsadai.client.toolkit.service.DataService;
import uk.org.ogsadai.client.toolkit.service.DataTransport;
import uk.org.ogsadai.client.toolkit.sessions.JoinExistingSession;
import uk.org.ogsadai.client.toolkit.sessions.JoinNewSession;
import uk.org.ogsadai.client.toolkit.sessions.Session;
import uk.org.ogsadai.engine.EngineConstants;


/**
 * This example shows how to query the request status and poll a data service
 * until a request completed.
 * 
 * @author The OGSA-DAI team.
 */
public class AsynchronousRequests {

    // Copyright statement
    private static final String COPYRIGHT_NOTICE =
        "(c) IBM Corp. 2002 - 2005. (c) The University of Edinburgh 2002 - 2005.";
    private static DataService mService;
    private static Session mSession;
    
    public static void main(String[] args) throws Exception {
        
        // Locate a data service and resource
                String handle = "http://localhost:8080/wsrf/services/ogsadai/DataService";
                String id = "MySQLResource";
                mService = GenericServiceFetcher.getInstance().getDataService(handle, id);
                System.out.println("Ready to connect to data service at " + handle);
        
        // Construct the asynchronous request
        ActivityRequest request = new ActivityRequest();
        DataStore store = new DataStore();
        store.addItem("one");
        store.addItem("two");
        store.addItem("three");
        DTOutputStream output = new DTOutputStream(store.getOutput());

        // Add activities to request
        request.add(store);
        request.add(output);
        request.setSessionRequirements(new JoinNewSession());
        
        // Perform the request
        System.out.println("Performing asynchronous request");
        Response response = mService.perform(request);
        System.out.println("Response:\n" + response.getAsString());
        
        // store the session 
        mSession = response.getSession();
        
        // Start a new thread which polls the data service resource until the current request
        // has completed
        new Thread(new RequestRunner()).start();
        
        // Get the service status
        System.out.println("Request status = " + mService.getStatus(mSession.getSessionID()));
        
        // Get the data transport for the output stream
        DataTransport dt = output.getDataTransport();

        // Get blocks from the output stream
        Object block = dt.getBlock();
        while (!block.equals(EngineConstants.NO_MORE_DATA)) {
            System.out.println("Received block: " + block);
            block = dt.getBlock();
        }
        
    }
    
    static class RequestRunner implements Runnable {
        
        // polls the service until the previous request has completed and  
        // then performs a query
        public void run() {
            try {
                System.out.println("THREAD: Starting to poll service until request completed");
                mService.pollUntilRequestCompleted(mSession.getSessionID(), 10);
                System.out.println("THREAD: First request completed");
                System.out.println("THREAD: Request status = " + mSession.getStatus());
                ActivityRequest request = new ActivityRequest();
                System.out.println("THREAD: About to run query request");
                SQLQuery query = new SQLQuery("select * from littleblackbook where id=9127");
                WebRowSet rowset = new WebRowSet(query.getOutput());
                request.add(query);
                request.add(rowset);
                request.setSessionRequirements(new JoinExistingSession(mSession));
                mService.perform(request);
                if (mSession.getStatus() == RequestStatus.COMPLETED) {
                    System.out.println("THREAD: Query request completed successfully");
                }
            }
            catch (Exception e) {
                e.printStackTrace();
            }
        }
        
    }
    

}
