//(c) International Business Machines Corporation, 2002 - 2005.
//(c) University of Edinburgh, 2002 - 2005.
//See OGSA-DAI-Licence.txt for licencing information.

package uk.org.ogsadai.examples.clienttoolkit;

import java.io.BufferedReader;
import java.io.InputStreamReader;

import uk.org.ogsadai.client.toolkit.GenericServiceFetcher;
import uk.org.ogsadai.client.toolkit.ResourceID;
import uk.org.ogsadai.client.toolkit.activity.ActivityRequest;
import uk.org.ogsadai.client.toolkit.activity.delivery.DTOutputStream;
import uk.org.ogsadai.client.toolkit.activity.delivery.DataTransportMode;
import uk.org.ogsadai.client.toolkit.activity.delivery.DeliverFromDT;
import uk.org.ogsadai.client.toolkit.activity.delivery.DeliverFromURL;
import uk.org.ogsadai.client.toolkit.activity.sql.SQLBulkLoad;
import uk.org.ogsadai.client.toolkit.activity.sql.SQLUpdate;
import uk.org.ogsadai.client.toolkit.activity.transform.XSLTransform;
import uk.org.ogsadai.client.toolkit.activity.xmldb.XPathQuery;
import uk.org.ogsadai.client.toolkit.service.DataService;
import uk.org.ogsadai.client.toolkit.sessions.JoinExistingSession;
import uk.org.ogsadai.client.toolkit.sessions.Session;

/**
 *  This is an example of data integration.
 *  An XPath query is performed on an XML database, the results are
 *  transformed into a WebRowSet and bulk loaded into a relational
 *  database.
 */
public class DataIntegrationExample {

    // Copyright statement
    private static final String COPYRIGHT_NOTICE =
        "(c) IBM Corp. 2002 - 2005. (c) The University of Edinburgh 2002 - 2005.";
    private static String serviceHandle = "http://localhost:8080/wsrf/services/ogsadai/DataService";

    public static ResourceID chooseResource(String question) throws Exception {

        // Locate a Data Service
        DataService service = GenericServiceFetcher.getInstance().getDataService(serviceHandle, null);
                ResourceID[] resources = service.getResourceIDs();
                for (int i=0; i<resources.length; i++) {
                    System.out.println(i + ") " + resources[i].getName());
                }

        System.out.print(question + " (0 - " + (resources.length-1) + "): \n");
        BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
        String choiceStr = br.readLine();
        int choice = Integer.parseInt(choiceStr);

        return resources[choice];
    }

    public static void main(String[] args) throws Exception {

        // Choose resource for creating sink
        ResourceID sinkResource = chooseResource("Select resource for sink");

        // Create a new table at the sink GDS
        String tableName = "mytable";
        String createTable = "create table if not exists " + tableName  +
                              " (id INTEGER, name VARCHAR(64), " +
                              "address VARCHAR(128), phone VARCHAR(20))";
        SQLUpdate create = new SQLUpdate( createTable );

        System.out.println("Creating new table at sink...");
        DataService sinkService = GenericServiceFetcher.getInstance().getDataService(serviceHandle, null);
        sinkService.setResourceID(sinkResource);
        sinkService.perform( create );
        System.out.println("Modified rows: " + create.getModifiedRowsCount());

        // Choose resource for source data service
        ResourceID sourceResource = chooseResource("Select resource for source");
        
        // Locate data service and resource
        DataService sourceService = GenericServiceFetcher.getInstance().getDataService(serviceHandle, null);
        sourceService.setResourceID(sourceResource);

        // Create a new session at the source
        System.out.println("Creating a new session at the source");
        Session session = sourceService.createSession();

        // Prepare source request
        // Deliver XSLT document to service
        String url = "http://localhost:8080/tutorial/Resource2Result.xsl";
        DeliverFromURL deliver = new DeliverFromURL( url );

        // XPath query
        XPathQuery query = new XPathQuery("/entry[@id<50]");
        
        // Transformation into WebRowSet XML
        XSLTransform transform = new XSLTransform();
        transform.setXMLInput( query.getOutput() );
        transform.setXSLTInput( deliver.getOutput() );

        // Output stream
        DTOutputStream outputStream = new DTOutputStream();
        outputStream.setInput(transform.getOutput());
        
        // Prepare request
        ActivityRequest sourceRequest = new ActivityRequest();
        sourceRequest.add( deliver );
        sourceRequest.add( query );
        sourceRequest.add( transform );
        sourceRequest.add( outputStream );
        sourceRequest.setSessionRequirements(new JoinExistingSession(session));

        // Perform request at source
        System.out.println("Starting source request...");
        sourceService.perform( sourceRequest );
        
        // Prepare sink request for bulk load
        DeliverFromDT deliverFromDT = new DeliverFromDT();
        deliverFromDT.setDataTransportInput(outputStream.getDataTransport());
        deliverFromDT.setDataTransportMode( DataTransportMode.BLOCK );

        SQLBulkLoad bulkload = new SQLBulkLoad( deliverFromDT.getOutput(), tableName );

        ActivityRequest sinkRequest = new ActivityRequest();
        sinkRequest.add( deliverFromDT );
        sinkRequest.add( bulkload );

        // Perform request at sink
        sinkService.perform(sinkRequest);
        System.out.println("Number of inserted rows at sink: " + bulkload.getInsertedRowsCount());

        // Destroy the session
        System.out.println("Destroying session at source.");
        sourceService.closeSession(session);
    }

}
