//(c) International Business Machines Corporation, 2002 - 2005. 
//(c) University of Edinburgh, 2002 - 2005.
//See OGSA-DAI-Licence.txt for licencing information.

package uk.org.ogsadai.examples.clienttoolkit;

import uk.org.ogsadai.client.toolkit.GenericServiceFetcher;
import uk.org.ogsadai.client.toolkit.Response;
import uk.org.ogsadai.client.toolkit.activity.ActivityRequest;
import uk.org.ogsadai.client.toolkit.activity.delivery.DataTransportMode;
import uk.org.ogsadai.client.toolkit.activity.delivery.DeliverFromDT;
import uk.org.ogsadai.client.toolkit.activity.delivery.DTOutputStream;
import uk.org.ogsadai.client.toolkit.activity.sql.SQLQuery;
import uk.org.ogsadai.client.toolkit.activity.sql.WebRowSet;
import uk.org.ogsadai.client.toolkit.service.DataService;
import uk.org.ogsadai.client.toolkit.sessions.JoinNewSession;
import uk.org.ogsadai.client.toolkit.sessions.Session;
import uk.org.ogsadai.client.toolkit.sessions.TerminateSession;

/**
 *  This example shows how the data transport operations can be used to pull
 *  data from one data service to another.
 */
public class DataTransportExample 
{
    // Copyright statement
    private static final String COPYRIGHT_NOTICE =
        "(c) IBM Corp. 2002 - 2005. (c) The University of Edinburgh 2002 - 2005.";
    public static void main(String[] args) throws Exception {

        String sourceHandle = 
            "http://localhost:8080/wsrf/services/ogsadai/DataService";
        String sourceResource = "MySQLResource";
        
        String sinkHandle = 
            "http://localhost:8080/wsrf/services/ogsadai/DataService";
        String sinkResource = "MySQLResource";

        // Locate the source and sink data services and resources
        DataService sourceService = 
            GenericServiceFetcher.getInstance().getDataService(
                sourceHandle, sourceResource);
        DataService sinkService = 
            GenericServiceFetcher.getInstance().getDataService(
                sinkHandle, sinkResource);
        
        // Perpare the source request
        SQLQuery sqlQuery = 
            new SQLQuery( "select * from littleblackbook where id<100" );
        WebRowSet rowset = new WebRowSet( sqlQuery.getOutput() );
        DTOutputStream outputStream = new DTOutputStream();
        outputStream.setInput(rowset.getOutput());
        ActivityRequest sourceRequest = new ActivityRequest();
        sourceRequest.add( sqlQuery );
        sourceRequest.add( rowset );
        sourceRequest.add( outputStream );
        sourceRequest.setSessionRequirements(new JoinNewSession());

        // Perform the source request, setting up the session stream
        System.out.println("Performing request at source resource.");
        Response response = sourceService.perform(sourceRequest);
        Session session = response.getSession();
        System.out.println("Response:\n" + response.getAsString());
        
        // Prepare sink request
        DeliverFromDT deliverFromDT = new DeliverFromDT();
        deliverFromDT.setDataTransportInput(outputStream.getDataTransport());
        deliverFromDT.setDataTransportMode(DataTransportMode.BLOCK);
        ActivityRequest sinkRequest = new ActivityRequest();
        sinkRequest.add(deliverFromDT);
        
        // Perform sink request and display response
        System.out.println("Performing request at sink resource");
        response = sinkService.perform(sinkRequest);
        System.out.println(response.getAsString());
        
        // Destroy the session at the source
        System.out.println("Destroying session at source.");
        sourceRequest = new ActivityRequest();
        sourceRequest.setSessionRequirements(new TerminateSession(session));
        sourceService.perform(sourceRequest);
    }
}
