//(c) International Business Machines Corporation, 2002 - 2005.
//(c) University of Edinburgh, 2002 - 2005.
//See OGSA-DAI-Licence.txt for licencing information.

package uk.org.ogsadai.examples.clienttoolkit;

import uk.org.ogsadai.client.toolkit.GenericServiceFetcher;
import uk.org.ogsadai.client.toolkit.Response;
import uk.org.ogsadai.client.toolkit.activity.ActivityRequest;
import uk.org.ogsadai.client.toolkit.activity.RequestComponent;
import uk.org.ogsadai.client.toolkit.activity.sql.SQLQuery;
import uk.org.ogsadai.client.toolkit.activity.sql.WebRowSet;
import uk.org.ogsadai.client.toolkit.service.DataService;

/**
 * This example shows how to construct more complicated requests
 * containing several queries.
 */
public class SQLQuerySequence {

    // Copyright statement
    private static final String COPYRIGHT_NOTICE =
        "(c) IBM Corp. 2002 - 2005. (c) The University of Edinburgh 2002 - 2005.";
    public static void main(String[] args) throws Exception {

                // set up service URL and resource ID
        String handle = "http://localhost:8080/wsrf/services/ogsadai/DataService";
        String id = "MySQLResource";
        // service handle may be provided as an argument
        if (args.length == 1) {
            handle = args[0];
        }

        // Locate a Data Service
        DataService service = GenericServiceFetcher.getInstance().getDataService(handle, id);
        System.out.println("Ready to connect to data service at " + service.getURL());

        // Prepare SQL queries
        SQLQuery query1  = new SQLQuery("select * from littleblackbook where name like '%Krause'");
        WebRowSet rowset1 = new WebRowSet(query1.getOutput());
        SQLQuery query2  = new SQLQuery("select * from littleblackbook where name like '%Sugden'");
        WebRowSet rowset2 = new WebRowSet(query2.getOutput());

        // Construct a request wrapping the queries
        ActivityRequest request = new ActivityRequest( 
                new RequestComponent[] { query1, rowset1, query2, rowset2 } );
        Response response = service.perform( request );
        System.out.println("Response:\n" + response.getAsString());

        // Alternative version: adding queries to an empty request
        request = new ActivityRequest();
        for (int i=0; i<20; i++) {
            String sql = "select * from littleblackbook where id = " + i;
            SQLQuery query = new SQLQuery(sql);
            WebRowSet rowset = new WebRowSet( query.getOutput() );
            request.add( query );
            request.add( rowset );
        }
        response = service.perform( request );
        System.out.println("Response:\n" + response.getAsString());

    }
}
