//(c) International Business Machines Corporation, 2005.
//(c) University of Edinburgh, 2005.
//See OGSA-DAI-Licence.txt for licencing information.

package uk.org.ogsadai.examples.clienttoolkit;

import java.io.File;
import java.sql.ResultSet;

import uk.org.ogsadai.client.toolkit.GenericServiceFetcher;
import uk.org.ogsadai.client.toolkit.activity.ActivityRequest;
import uk.org.ogsadai.client.toolkit.activity.sql.SQLQuery;
import uk.org.ogsadai.client.toolkit.activity.sql.WebRowSet;
import uk.org.ogsadai.client.toolkit.security.wsrf.SecurityConfigProperty;
import uk.org.ogsadai.client.toolkit.service.DataService;

/**
 * This example shows how to connect to a secure data service using message
 * level security.
 * 
 * @author The OGSA-DAI team.
 */

public class WSRFMessageLevelSecurityExample {

    // Copyright statement
    private static final String COPYRIGHT_NOTICE =
        "(c) IBM Corp. 2002 - 2005. (c) The University of Edinburgh 2002 - 2005.";
    public static void main(String[] args) throws Exception {
                
        // set up service URL and resource ID
        String handle = "http://myComputer.myCompany.com:8999/wsrf/services/ogsadai/SecureDataService";
        String id = "MySQLResource";
        // service handle may be provided as an argument
        if (args.length == 1) {
            handle = args[0];
        }

        // Locate a Data Service
        DataService secureService = GenericServiceFetcher.getInstance().getWSRFDataService(handle, id);
        
        // Setup message level security
        SecurityConfigProperty securityProperty = new SecurityConfigProperty(
                new File("examples/src/uk/org/ogsadai/examples/clienttoolkit/MLSecurityDescriptor.xml"));
        secureService.setConnectionProperty(securityProperty);
        
        System.out.println("Ready to connect to secure data service at " + secureService.getURL());

        // Perform a simple SQLQuery
                SQLQuery query = new SQLQuery("select * from littleblackbook where id='3475'");
        WebRowSet rowset = new WebRowSet( query.getOutput() );
        ActivityRequest request = new ActivityRequest();
        request.add( query );
        request.add( rowset );
        System.out.println("Performing SQL query...");
                secureService.perform( request );
        
        // Print out the names 
        ResultSet resultset = rowset.getResultSet();
        while (resultset.next()) {
            System.out.println("name = " + resultset.getString(2));
        }

    }
}
