//(c) International Business Machines Corporation, 2006.
//(c) University of Edinburgh, 2006.
//See OGSA-DAI-Licence.txt for licencing information.

package uk.org.ogsadai.examples.clienttoolkit;

import uk.org.ogsadai.client.CoGUtil;
import uk.org.ogsadai.client.toolkit.GenericServiceFetcher;
import uk.org.ogsadai.client.toolkit.Response;
import uk.org.ogsadai.client.toolkit.activity.ActivityRequest;
import uk.org.ogsadai.client.toolkit.activity.sql.SQLQuery;
import uk.org.ogsadai.client.toolkit.activity.sql.WebRowSet;
import uk.org.ogsadai.client.toolkit.security.wsrf.GSITransportEncryptionProperty;
import uk.org.ogsadai.client.toolkit.service.DataService;

/**
 * This example shows how to connect to a secure data service using
 * transport level security.
 * 
 * @author The OGSA-DAI Project Team.
 */
public class WSRFTransportLevelSecurityExample
{
    /** Copyright statement */
    private static final String COPYRIGHT_NOTICE =
        "(c) IBM Corp. 2002 - 2006. (c) The University of Edinburgh 2002 - 2006.";

    /**
     * Register transport security using
     * <code>CoGUtil.registerTransport()</code>
     */
    static
    {
        CoGUtil.registerTransport();
    }        
        
    public static void main(String[] args) throws Exception
    {
        if (!CoGUtil.isRegistered())
        {
            // We had an error registering transport security.
            System.out.println(CoGUtil.getRegistrationException().getLocalizedMessage());
            return;
        }
        
        // Set up service URL and resource ID.
        String handle = "https://myComputer.MyCompany.com:8443/wsrf/services/ogsadai/SecureDataService";
        String id = "MySQLResource";
        // service handle may be provided as an argument
        if (args.length == 1)
        {
            handle = args[0];
        }

        // Locate a data service
        DataService secureService = 
            GenericServiceFetcher.getInstance().getDataService(handle, id);
        
        // Setup encryption
        secureService.setConnectionProperty(new GSITransportEncryptionProperty());
        System.out.println("Ready to connect to secure data service at " + secureService.getURL());

        // Now perform a simple SQLQuery
        SQLQuery query = 
            new SQLQuery("select * from littleblackbook where id='3475'");
        WebRowSet rowset = new WebRowSet(query.getOutput());
        ActivityRequest request = new ActivityRequest();
        request.add(query);
        request.add(rowset);
        System.out.println("Performing SQL query...");
        Response response = secureService.perform(request);
        System.out.println("Response:\n" + response.getAsString());
    }
}
