//(c) University of Edinburgh, 2002 - 2006.
//See OGSA-DAI-Licence.txt for licencing information.

package uk.org.ogsadai.examples.demoFactory;

import java.io.File;

import uk.org.ogsadai.common.properties.Property;
import uk.org.ogsadai.dataresource.DataResourceAccessorConfigException;
import uk.org.ogsadai.dataresource.DataResourceAccessorMetaDataException;
import uk.org.ogsadai.dataresource.DataResourceAccessorSetupException;
import uk.org.ogsadai.dataresource.PersistInFiles;
import uk.org.ogsadai.dataresource.DataResourceAccessor;
import uk.org.ogsadai.service.resource.factory.ConfigDirectoryProvider;

/**
 * A very simple data resource accessor that allows activities to
 * obtain the configuration directory of the data service resource
 * that is using the accessor.
 * <p>
 * This functionality allows data service resource factory activities
 * to copy default configuration files from the parent data service
 * resource to a newly created child data service resource.
 * 
 * @author The OGSA-DAI Project Team
 */
public class DemoFactoryDataResourceAccessor 
    implements DataResourceAccessor, PersistInFiles, ConfigDirectoryProvider
{
    /** Copyright statement */
    private static final String COPYRIGHT_NOTICE = 
        "(c) University of Edinburgh 2002 - 2006.";

    /**
     * Location of configuration directory of parent data service
     * resource 
     */
    protected File mConfigDirectory;

    /* (non-Javadoc)
     * @see uk.org.ogsadai.dataresource.DataResourceAccessor#setResourceName(java.lang.String)
     */
    public void setResourceName(String resourceName)
    {
    }

    /* (non-Javadoc)
     * @see uk.org.ogsadai.dataresource.PersistedDataResourceAccessor#restoreFromConfig(java.io.File)
     */
    public void restoreFromConfig(File directory) 
        throws 
            DataResourceAccessorConfigException, 
            DataResourceAccessorMetaDataException, 
            DataResourceAccessorSetupException, 
            IllegalArgumentException
    {
        if (directory == null)
        {
            throw new IllegalArgumentException("directory must not be null");
        }
        if (! directory.isDirectory())
        {
            throw new IllegalArgumentException("directory " + 
                                               directory.getAbsolutePath() +
                                               " must be a directory");
        }
        // Just store the directory
        mConfigDirectory = directory;
    }

    /* (non-Javadoc)
     *
     * @return a zero element array. 
     *
     * @see uk.org.ogsadai.dataresource.DataResourceAccessor#getProperties()
     */
    public Property[] getProperties()
    {
        return new Property[]{};
    }

    /* (non-Javadoc)
     *
     * @see uk.org.ogsadai.service.resource.factory.ConfigDirectoryProvider#getConfigDirectory()
     */
    public File getConfigDirectory() 
    {
        return mConfigDirectory;
    }
}
