//(c) International Business Machines Corporation, 2002 - 2006. 
//(c) University of Edinburgh, 2002 - 2006.
//See OGSA-DAI-Licence.txt for licencing information.

package uk.org.ogsadai.examples.demoFactory;

import org.w3c.dom.Element;

import uk.org.ogsadai.activity.Activity;
import uk.org.ogsadai.activity.ActivityCreationException;
import uk.org.ogsadai.activity.ActivityExecutionException;
import uk.org.ogsadai.activity.ActivitySpecificationException;
import uk.org.ogsadai.activity.ActivityStreamNotFoundException;
import uk.org.ogsadai.common.BlockWriter;
import uk.org.ogsadai.service.OGSADAIConstants;

/** 
 * An activity that queries a data service resource created
 * by a demo data service resource factory resource.
 * Used to demonstrate dynamic data service resource creation.
 * <p>
 * For information on this activity see the OGSA-DAI user doc:
 * <code>OGSA-DAI/doc/interaction/activities/examples/demoQueryInstance.html
 * </code>
 *
 * @author OGSA-DAI team
 */
public class DemoQueryInstanceActivity extends Activity
{
    /** Copyright statement */
    private static final String COPYRIGHT_NOTICE = 
        "(c) IBM Corp. 2002 - 2006. (c) University of Edinburgh 2002 - 2006.";

    /** Activity output. */
    private BlockWriter mOutput;

    /**
     * Constructs a demo query instance activity using the specified XML
     * element.
     * 
     * @param element 
     *     <code>Element</code> to configure the activity with - from
     *     a perform document.
     * 
     * @throws ActivityCreationException
     *     If there is a problem constructing the activity due to an
     *     implementation error or OGSA-DAI confguration problem.
     * 
     * @throws ActivitySpecificationException
     *     If there is a problem constructing the activity due to an
     *     invalid setting in <code>element</code>.
     *     mistake such as an invalid setting.
     */
    public DemoQueryInstanceActivity(final Element element)
        throws ActivitySpecificationException, ActivityCreationException 
    {
        super(element);

        // Publish the activity output(s) by setting the inherited
        // mInternalOutputs fields.

        String outputName = ((Element) element.getElementsByTagNameNS(
                OGSADAIConstants.TYPES_NAMESPACE, "myOutput").item(0)).getAttribute("name");
        mInternalOutputs = new String[] {outputName};
    }

    /* (non-Javadoc)
     * @see uk.org.ogsadai.activity.Activity#initialise()
     */
    public void initialise()
        throws ActivitySpecificationException, ActivityExecutionException 
    {
        try
        {
            // Set-up references to the activity input and output.
            mOutput = mContext.getOutput(mInternalOutputs[0]);
        }
        catch (ActivityStreamNotFoundException e) 
        {
            throw new ActivitySpecificationException(getName(), e);
        }
    }

    /* (non-Javadoc)
     * @see uk.org.ogsadai.engine.Activity#processBlock()
     */
    public void processBlock()
    {
        // Get the data resource
        DemoNameProvider nameProvider =
            (DemoNameProvider) mContext.getDataResourceAccessor();

        mOutput.put(nameProvider.getName());
      
        setCompleted();
    }
}
