//(c) University of Edinburgh, 2002 - 2006.
//See OGSA-DAI-Licence.txt for licencing information.

package uk.org.ogsadai.examples.demoTransientFactory;

import java.io.File;

import uk.org.ogsadai.exception.DAIException;
import uk.org.ogsadai.common.properties.Property;
import uk.org.ogsadai.dataresource.DataResourceAccessorConfigException;
import uk.org.ogsadai.dataresource.DataResourceAccessorMetaDataException;
import uk.org.ogsadai.dataresource.DataResourceAccessorSetupException;
import uk.org.ogsadai.dataresource.PersistInFiles;
import uk.org.ogsadai.dataresource.DataResourceAccessor;
import uk.org.ogsadai.service.resource.config.ActivitiesConfig;
import uk.org.ogsadai.service.resource.config.FileBasedActivitiesConfig;
import uk.org.ogsadai.sessions.SessionComponentFactory;
import uk.org.ogsadai.sessions.SessionConfigurationLoader;

/**
 * A very simple data resource accessor that allows activities to obtain the
 * configuration directory of the data service resource that is using the
 * accessor.
 * <p>
 * This functionality allows data service resource factory activities to copy
 * default configuration files from the parent data service resource to a newly
 * created child data service resource.
 * 
 * @author The OGSA-DAI Project Team
 */
public class DemoTransientFactoryDataResourceAccessor
    implements
    DataResourceAccessor,
    PersistInFiles,
    ChildDataServiceResourceComponentProvider
{
    /** Copyright statement */
    private static final String COPYRIGHT_NOTICE = 
        "(c) University of Edinburgh 2002 - 2006.";

    /** Data service resource name */
    String mResourceName;
    
    /** Child activities config */
    ActivitiesConfig mChildActivitiesConfig;
    
    /** Child session component factory */
    SessionComponentFactory mSessionComponentFactory;
    
    /*
     * (non-Javadoc)
     * 
     * @see uk.org.ogsadai.dataresource.DataResourceAccessor#setResourceName(java.lang.String)
     */
    public void setResourceName(String resourceName)
    {
        mResourceName = resourceName;
    }

    /*
     * (non-Javadoc)
     * 
     * @see uk.org.ogsadai.dataresource.PersistedDataResourceAccessor#restoreFromConfig(java.io.File)
     */
    public void restoreFromConfig(File directory)
        throws 
            DataResourceAccessorConfigException,
            DataResourceAccessorMetaDataException,
            DataResourceAccessorSetupException,
            IllegalArgumentException
    {
        if (directory == null)
        {
            throw new IllegalArgumentException("directory must not be null");
        }
        if (!directory.isDirectory())
        {
            throw new IllegalArgumentException("directory "
                + directory.getAbsolutePath() + " must be a directory");
        }

        // Get the instance directory with details of config setting 
        // for child data service resources.
        File instanceDirectory = new File(directory,"instance");

        try
        {
            // Read the child activities config
            File childActivityConfigFile = 
                new File(instanceDirectory,"activityConfig.xml");
            mChildActivitiesConfig = 
                new FileBasedActivitiesConfig(childActivityConfigFile);
            
            // Read the child session component factory
            File childSessionConfigFile = 
                new File(instanceDirectory,"sessionConfig.xml");
            mSessionComponentFactory =
                new SessionConfigurationLoader(childSessionConfigFile);
        }
        catch(DAIException e)
        {
            throw new DataResourceAccessorConfigException(mResourceName,e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @return a zero element array.
     * 
     * @see uk.org.ogsadai.dataresource.DataResource#getProperties()
     */
    public Property[] getProperties()
    {
        return new Property[] {};
    }

    /* (non-Javadoc)
     * @see uk.org.ogsadai.examples.demoTransientFactory.ChildDataServiceResourceComponentProvider#getChildActivitiesConfig()
     */
    public ActivitiesConfig getChildActivitiesConfig()
    {
        return mChildActivitiesConfig;
    }

    /* (non-Javadoc)
     * @see uk.org.ogsadai.examples.demoTransientFactory.ChildDataServiceResourceComponentProvider#getChildSessionComponentFactory()
     */
    public SessionComponentFactory getChildSessionComponentFactory()
    {
        return mSessionComponentFactory;
    }

}