//(c) University of Edinburgh, 2002 - 2006.
//See OGSA-DAI-Licence.txt for licencing information.

package uk.org.ogsadai.examples.demoTransientFactory;

import javax.xml.namespace.QName;

import uk.org.ogsadai.common.properties.Property;
import uk.org.ogsadai.common.properties.PropertyValueException;
import uk.org.ogsadai.common.properties.StaticProperty;
import uk.org.ogsadai.dataresource.DataResourceAccessor;
import uk.org.ogsadai.dataresource.DataResourcePropertyNames;
import uk.org.ogsadai.examples.demoFactory.DemoNameProvider;

/**
 * 
 *
 * @author OGSA-DAI team
 */
public class DemoTransientInstanceDataResourceAccessor
    implements DataResourceAccessor, DemoNameProvider
{
    /** Copyright statement */
    private static final String COPYRIGHT_NOTICE = 
        "(c) University of Edinburgh 2002 - 2006.";

    /** Name of data service resource */
    private String mResourceName;
    
    /** A simple name value */
    private String mName; 

    /** Name of resource property holding the name value */
    public static final QName NAME_PROPERTY_QNAME =
        new QName(DataResourcePropertyNames.NAMESPACE, "name");
    
    /**
     * Creates the data resource accessor.
     * 
     * @param name 
     *    name value that will be stored within the accessor and is available
     *    to activties via the <code>DemoNameProvider</code> interface.
     *    
     */
    public DemoTransientInstanceDataResourceAccessor(String name)
    {
        mName = name;
    }

    /* (non-Javadoc)
     * @see uk.org.ogsadai.dataresource.DataResourceAccessor#setResourceName(java.lang.String)
     */
    public void setResourceName(String resourceName)
    {
        mResourceName = resourceName;
    }
    

    /* (non-Javadoc)
     * @see uk.org.ogsadai.dataresource.DataResourceAccessor#getProperties()
     */
    public Property[] getProperties()
    {
        Property prop = new StaticProperty();
        prop.setName(NAME_PROPERTY_QNAME);
        try 
        {
            prop.setValue(mName);
        } 
        catch (PropertyValueException e) 
        {
            // StaticProperty never throws this.
        }
        return new Property[]{prop};
    }
    
    /**
     * Gets the name value.
     * 
     * @return the name value.
     */
    public String getName()
    {
        return mName;
    }
}
