############################################################################
# Jason R. Lee, LBNL
#
#         $Author: nedelson $
#         $RCSfile: ftpd.py,v $
#         $Revision: 1.10 $
#         $Locker:  $
#         $Date: 2004/03/14 16:42:32 $
#
# See Copyright for copyright notice!
###########################################################################
"""
Python Globus File Transfer Protocol Daemon

Usage: python  ftpd.py [-d] [-n] [-p port] [-h]
        -d turn on debug 
        -n turn on autolog (must have netlogger lite installed)
        -p port portnumber to listen on
        -h this usage message

"""
import ftpdCommands
import string
import socket
from pyGlobus.ftpControlc import GLOBUS_FTP_CONTROL_AUTH_REQ_USER, GLOBUS_FTP_CONTROL_AUTH_REQ_PASS,GLOBUS_FTP_CONTROL_AUTH_REQ_GSSAPI
from pyGlobus.ftpControl import FtpControl
from pyGlobus.ftpControl import FtpControlServer
from pyGlobus.ftpControl import AuthInfo
from pyGlobus import ftpControlc
from pyGlobus.util import Buffer
from pyGlobus.util import GlobusException
from threading import *
from exceptions import Exception
import sys, getopt
from pyGlobus.ftpControl import FtpControlException
from pyGlobus.security import GSSCred
from pyGlobus.ftpControl import DCAU
## setup for autologing if its in our path
try:
    import nlauto,nllite
    autolog = 1
except:
    autolog = 0

ftp_version = "$Revision: 1.10 $"
ftp_date = "$Date: 2004/03/14 16:42:32 $"

# way too many globals here, clean up
g_port = 0
g_ftpControlServer = None

g_verbose = 0
g_dolog=0

g_commands = ftpdCommands.CommandMethods()
#############################################################################
def usage(m=None):
    if m: print "Error:", m
    print  __doc__
    sys.exit()

#############################################################################
def my_print(m):
        if g_verbose: 
            print m

#########################################################################

def dummy_callback( arg, handle, err):
    if err[0] != 0:
        print "dummy_callback here with an error:", err
        
#########################################################################

def command_process( arg, handle, err, command):
    my_print("in command_process")
    
    inst = ftpdCommands.Connections()
    state = inst.getFtpControlConnection(handle)
    control = state.connection

    if err[0] == 0:
        rawsplit = control.raw_command(command).split()
        cmd, args = rawsplit[0], string.join(rawsplit[1:])
        method = g_commands.getMethodForCommand(cmd)
        state.args = args
        try: apply(method,(handle,dummy_callback,0))
        except Exception, e: print "Error with command %s: %s" % (cmd,e)
    else:
        print "Error: No command processed"
    my_print("leaving command_process")
#########################################################################

def command_callback( arg , handle, err):

    my_print("in command callback")
    if err[0] != 0:
       print "Error in command callback, exiting ..."
       sys.exit(-1)
       
    inst = ftpdCommands.Connections()
    state = inst.getFtpControlConnection(handle)
    control = state.connection

    control.read_commands(command_process, 0)
    my_print("end command callback")

#########################################################################

def authorizeMe( arg, handle, err, auth_info):
    my_print("start authorizeMe")

    if err[0] == -1:
        print "Error in authenticaion... Did you run grid-proxy-init?"
        print "Error: ", err[1]  
        sys.exit(-1)
        
    inst = ftpdCommands.Connections()
    state = inst.getFtpControlConnection(handle)
    control = state.connection
    authResponse = """235 GSSAPI Authentication succeeded\r\n"""
    control.send_response(authResponse, command_callback, 0)

    my_print("end authorizeMe")
    
#########################################################################

def myauthenticate(arg, handle, err):

    my_print("in myauthenticate")

    if err[0] == -1:
       print "Error in myauthenticate, exiting ..."
       print err[1] 
       sys.exit(-1)
       
    inst = ftpdCommands.Connections()
    state = inst.getFtpControlConnection(handle)
    control = state.connection 

    flags = GLOBUS_FTP_CONTROL_AUTH_REQ_GSSAPI 

    try:
        result = g_ftpControlServer.authenticate(control, flags, authorizeMe, 0)
        my_print("authenticate result: %s " % result)
    except FtpControlException, ex:
        print "exception thrown"
        print ex
    my_print("end myauthenticate")

#########################################################################

def accept_msg( arg, handle, err):
    my_print("in accept_msg")

    if err[0] == -1: 
       print "Error in accept_msg exiting ..."
       print "Error: ", err[1]
       sys.exit(-1)

    inst = ftpdCommands.Connections()
    state = inst.getFtpControlConnection(handle)
    control = state.connection
    resp = "220 %s GSIFTP server (LBL PyGlobus Version %s %s ) ready\r\n" % ( 
                socket.getfqdn(), 
                ftp_version.split()[1], ftp_date.split()[1])

    control.send_response( resp , myauthenticate, 0)

    my_print( "leaving accept_msg")

#########################################################################

def simple_server(arg, shandle, err):
    my_print( "simple_server")

    if err[0] == -1:
       print "Error in simple_server exiting ..."
       print "Error: ", err[1]
       sys.exit(-1)
   
    ftpControl = FtpControl()

    # Seems to work now? Must use 'A' not 'N' 
    dcau = DCAU()
    dcau.set_mode('A')
    #dcau.set_mode('N')
    cred = GSSCred()
    ftpControl.local_dcau(dcau, cred)      
    inst = ftpdCommands.Connections()
    inst.addFtpControlConnection( ftpControl.get_handle(), ftpControl )

    try:
        g_ftpControlServer.accept( ftpControl, accept_msg, 0)
    except FtpControlException, ex:
        print "ex raised"
	print ex

    my_print("finished accept")

#########################################################################

def main(args):
    """Setup a few of the necessary conditions and starup the server, then
    sit on lock (easy way to do a event loop) and let the globus event lib
    callbacks do all the work.
    XXX: should add another flag for using in a non-gsi envirorment
    """
    global  g_ftpControlServer, g_port,g_verbose, g_dolog

    port = 5555

    try:
        options,prog_args = getopt.getopt(args[1:],'dhnp:')
    except getopt.GetoptError, E:
        usage(E)

    for opt,val in options:
        if opt == '-d': 
            g_verbose = 1
            ftpdCommands.g_verbose = 1
        elif opt == '-n':
            g_dolog = 1
        elif opt == '-p':
           port = int(val)
        elif opt == '-h': 
            usage()
        else:
            usage()

    ftpInstances = ftpdCommands.Connections()
    condV = Condition(Lock())

    if autolog and g_dolog:
        nlauto.setAutolog(nllite.LogOutputStream(sys.stderr))
        #nlauto.autologClass(FtpControl)
        #nlauto.autologClass(AuthInfo)
        nlauto.autologClass(FtpControlServer)

    try:
        g_ftpControlServer = FtpControlServer()
        condV.acquire()
        cond = 0
        newCbHandle,g_port = g_ftpControlServer.listen(port, simple_server,0)
    except GlobusException, ex: 
        print ex
        condV.release()
        sys.exit(-1)

    print "Listening on port ", g_port
    while cond == 0:
        condV.wait()
    condV.release()
    print "Shutting down..."
    sys.exit()

if __name__ == "__main__":
    main(sys.argv)

# vim: et ts=4 sw=4

