"""
FTP daemon commands.
Jason R. Lee, LBNL
See Copyright for copyright notice!
"""
from pyGlobus.ftpControl import FtpControl
from pyGlobus.ftpControlc import GLOBUS_FTP_CONTROL_MODE_EXTENDED_BLOCK
from pyGlobus.ftpControl import FtpControlServer
from pyGlobus.ftpControl import Parallelism
from pyGlobus.ftpControl import TcpBuffer
from pyGlobus.ftpControl import HostPort
from pyGlobus import ftpControlc
import os
from threading import *
from exceptions import Exception
import sys
from pyGlobus.util import Buffer

g_verbose = 0
g_transfers = 0
#############################################################################
def my_print(m):
    if g_verbose: print "%s" % m
#############################################################################
def check_file(arg, mode):
    path = "none"
    err = "none"
    # if write mode
    try:
        if mode == 'r':
            if not os.access(arg.args,os.F_OK): 
                err = "doesn't exit"
                raise
            if not os.access(arg.args, os.R_OK): 
                err = "can't read"
                raise
        # read mode
        elif mode == 'w':
            # file can't exist
            if os.access(arg.args, os.F_OK): 
                err = "exists"
                raise
            # have to be able to write to dir
            end = arg.args.rfind('/')
            if end == -1:
                path = arg.args
            else:
                path = arg.args[:end]
            if not os.access(path, os.W_OK): 
                err = "can't write"
                raise
        else:
            err = "unknown mode"
            raise 
        return True
    except:
        print "check_file: %s (%s:%s)?" % (err, path,arg.args)

        return False

#############################################################################
def sumfile(state):
    """Do an MD5 checksum of a file from offset to lenght
    """
    # some block size to use for cksumming

    bsize = 65536

    # don't deal with spaces in filenames (yet)
    if len(state.args.split()) != 4:
        return False

    method, start, length, file = state.args.split()

    if method.upper() != 'MD5':
        return False

    if not os.access(file,os.F_OK) or not os.access(file, os.R_OK): 
        return False

    # stuff that can throw exceptions
    try:
        f = open(file, 'rb')
        start = int(start)
        length = int(length)
        print "foo"
        import md5
    except:
        print "error"
        return False

    print "here"
    # if start is other then 0, seek there
    if (start != 0 ):
        f.seek(start)

    # if length is -1, set to end of file
    if length == -1:
        length = os.stat(file)[6]

    # if the size to sum is less then a block, just set 
    # blocksize to size the size to cksm
    if length < bsize:
        bsize = length

    b=f.read(bsize)
    length -= len(b)

    m=md5.new()
    m.update(b)

    # read until I either hit the end or have read length bytes
    while(len(b) and length > 0 ):
        b=f.read(bsize)
        length -= len(b)
        m.update(b)

    if length != 0:
        return False

    return m.hexdigest() 


############################################################################
def exit_cb( arg, handle, error):
    """ Used for debugging.
    """
    if error != "NONE":
        print "error is", error
    

#############################################################################
def dummy_cb( arg, handle, error):
    """Used for generic calls that requre a cb, but we don't
    care about the response (unless its a error code!)
    """
    if error:
        print "dummy_cb error is", error
    return

#############################################################################
def quit_cb( arg, handle, error):
    """Used for generic calls that requre a cb, but we don't
    care about the response (unless its a error code!)
    """
    if error:
        print 'quit cb = ', error
    inst = Connections()
    state = inst.getFtpControlConnection(handle)
    state.connection.quit( quit_cb2, None)
    return


def quit_cb2( arg, handle, error,response):
    """Used for generic calls that requre a cb, but we don't
    care about the response (unless its a error code!)
    """
    global g_transfers
    inst = Connections()
    inst.delFtpControlConnection(handle)
    
#############################################################################
def data_write_done_cb( arg, handle, error):
    """Called when we are done with a transfer to a client,
    clean up any old buffers, and maybe set the cond done
    for the transfer?
    """
    my_print("in data_write_done_cb with error"+str(error))
    #del arg.buf

    my_print("end data_write_done_cb")
    return

#############################################################################
def data_read_done_cb( arg, handle, error):
    """Called when we are done with a transfer to a client,
    clean up any old buffers, and maybe set the cond done
    for the transfer?
    """
    my_print("in data_read_done_cb")

    my_print( "end data_read_done_cb")
    return
   
#############################################################################
def data_read_cb(arg, handle, buffer, byte_type, length, offset, eof, error):
    """Read in data from the network and write it to disk. As long as
    we don't get an eof, keep registering to read from the network. at a 
    eof, close everyting up and call the dummy_cb to end thread.
    """
    my_print("in data_read_cb")
    inst = Connections()
    state = inst.getFtpControlConnection(handle)

    if error[0] != 0:
        print "Error on read! Aborting!!"
        sys.exit(-1)

    if eof == 0:
        state.fhandle.seek(offset)
        state.fhandle.write(buffer)
        state.connection.data_read(state.buf, data_read_cb, arg)
    else:
        if length > 0:
            state.fhandle.write(buffer)
        state.fhandle.flush()
        state.fhandle.close()
        state.connection.send_response("""226 Transfer complete.""",
                                            data_read_done_cb ,arg)
        my_print("     end of file")
    my_print("end data_read_cb")

#############################################################################
def data_connect_read_cb(arg, handle, stripe_ndx, reused, error):
    my_print( "in data_connect_read_cb")
    
    inst = Connections()
    state = inst.getFtpControlConnection(handle)

    # need to fix this to be dynamic (command line?)!
    max_buflen = 64*1024 
    state.buf = Buffer(max_buflen)

    # open up the file and call read!
    # check_file should have ensured we don't fail here
    state.fhandle = open(state.args, 'wb')
    state.filename = state.args
    
    state.connection.data_read(state.buf, data_read_cb, arg)
    my_print ("exit data_connect_read_cb")


#############################################################################
def data_write_cb(arg, handle, old_buf, byte_type, 
                  bufLen, old_offset, eof,err):
    """Keep calling this callback until the whole file is written out.
    once we reach the end of the file, call ourself again, this time with
    a zero length buffer and eof set. Hope this cleans up everything!

    """
    my_print("in data_write_done_cb")

    if err[0] != 0:
        print "in data_write_done_cb"

    inst = Connections()
    state = inst.getFtpControlConnection(handle)
    resp = """250 Requested file action okay, completed.\r\n"""
    #control = arg.connection

    if eof:
        my_print("sending NO more data (EOF)")
        state.connection.send_response(resp, dummy_cb, arg)
    else:
        offset = state.fhandle.tell()
        state.buf = state.fhandle.read(64*1024)
        buf_len = len(state.buf)
        eof = 0
        if buf_len == 0:
            eof = 1
        state.connection.data_write(state.buf, offset, eof, 
                                    data_write_cb, arg)

    my_print ("end data_write_cb")

#############################################################################

def data_connect_write_cb(arg, handle, stripe_ndx, reused, error):
    """This is the server writing out to the client, so we need to 
    open a file (check permissions, security, etc) read in a buffer
    full of data, and then write it out. If we got a eof on the read,
    we  mark the eof flag on the write. When we write out the data,
    setup the cb for writing the next chunk of data.
    """
    my_print("in data_connect_write_cb")

    inst = Connections()
    state = inst.getFtpControlConnection(handle)               
    offset = 0
    eof = 0
    max_len = 64*1024
    
    try:
        fhandle = open(state.args, 'rb')
        state.filename = state.args
    except Exception, E:
        print "error opening file(%s)" % E
        sys.exit(-1)

    state.buf = fhandle.read(max_len)
    state.fhandle=fhandle

    state.connection.data_write(state.buf, offset, eof, 
                       data_write_cb, arg)

    my_print("end data_connect_write_cb")
    
########################################################################i

class State:
    """This is a mini-class to keep track of the state-specific values
    of a connection. 
    """
    def __init__(self, conn): 
        self.condV = Condition(Lock())
        self.connection = conn
        self.filename = None
        self.args = None
        self.buf = None

    def __del__(self):
        del self.condV
        del self.args
        del self.connection
        del self.buf
        del self.filename

#########################################################################        
class Connections:
    """This is the class (global struct) holds the state for each
    indivual connection. This allows me to keep track of what various
    conditions associated with specific transfers (file, locks, args, etc)
    """
    init = 0
    FtpControlConnections = {}
    def __init__(self):
        if Connections.init:
            return
        Connections.init = 1
        Connections.FtpControlConnections ={}

    def addFtpControlConnection(self, handle, control):
        if self.FtpControlConnections.has_key(handle):
            raise Exception("control already exists")
        else:
            self.FtpControlConnections[handle] =  State(control)
            
    def delFtpControlConnection(self,handle):
        if not self.FtpControlConnections.has_key(handle):
            raise Exception("no such control")
        else:            
            del self.FtpControlConnections[handle]


    def getFtpControlConnection(self, handle):
        if self.FtpControlConnections.has_key(handle):
            return self.FtpControlConnections[handle]	
        return None     
#############################################################################

class CommandMethods:
    """Make a mapping of FTP commands to methods. each method is just
    the command name with '_do" prepended to it, so if you wanted to 
    implemnt the command 'MKDIR', you would define a function called
    _doMKDIR, and it would automagicly called with ftp sent over the
    command MKDIR.
    """

    MARKER = '_do'

    def __init__(self):
        self._commands = {}
        marker_len = len(self.MARKER)
        for method in dir(CommandMethods):
            method_obj = getattr(self,method)
            if method.startswith(self.MARKER):
                self._commands[method[marker_len:]] = method_obj
        return

#############################################################################

    def getMethodForCommand( self, action ):
        if self._commands.has_key(action):
            return self._commands[action]
        return self._doNotImplemented

#############################################################################

    def _doNotImplemented(self, handle, callback, arg=None):

        inst = Connections()
        state = inst.getFtpControlConnection(handle)
        state.connection.send_response("""502 Commmand Not Implemented\r\n""", 
                    callback, arg)

#############################################################################

    def _doSYST(self,handle, callback,arg=None):
        my_print( "start do_SYST")
        inst = Connections()
        state = inst.getFtpControlConnection(handle)
        state.connection.send_response("""215 *NIX Type: L8\r\n""", callback, arg)
        my_print( "end do_SYST")

#############################################################################
    def _do_CKSM( self, handle, callback, arg=None):
        my_print( "start do_CKSM")
        inst = Connections()
        state = inst.getFtpControlConnection(handle)
        sum = sumfile(state)
        if sum:
             state.connection.send_response("215 %s\r\n" % sum, callback, arg)
        else:
             state.connection.send_response("502 Commmand Not Implemented\r\n", 
                        quit_cb, arg)
        my_print( "end do_CKSM")
#############################################################################

    def _doQUIT(self,handle,callback,arg=None):
        my_print( "start do_QUIT")
        inst = Connections()
        state = inst.getFtpControlConnection(handle)
        state.connection.send_response("""221 Service closing control connection.\r\n""",quit_cb, arg)
        my_print( "end do_QUIT")

#############################################################################

    def _doTYPE(self,handle,callback,arg=None):
        my_print( "in _doTYPE")
        #my_print( "    arg: %s "% arg.args)
        inst = Connections()
        state = inst.getFtpControlConnection(handle)
        args = state.args
        
        state.connection.local_type(args)
        responseString = "200 Type is set to " + args +".\r\n"
        state.connection.send_response(responseString,callback, arg)
        my_print( "end _doTYPE")

#############################################################################

    def _doRETR(self,handle, callback,arg=None):
        my_print( "in _doRETR")
        inst = Connections()
        state = inst.getFtpControlConnection(handle)
        if check_file(state, 'r'):
            # setup for writing out data    
            state.connection.data_connect(data_connect_write_cb, arg)
            state.connection.send_response(
                                "150 File opened, sending data.\r\n", 
                                callback, arg)
        else:
            state.connection.send_response("550 %s: Permission denied.\r\n" %
                    state.args, dummy_cb, arg)
        my_print("end _doRETR")


#############################################################################

    def _doPASV(self,handle,callback, arg=None):
        my_print( "in _doPASV")
        inst = Connections()
        state = inst.getFtpControlConnection(handle)
        hostPort = state.connection.local_pasv()
        hpInfo = hostPort.get_info()
        responseString = "227 Entering Passive Mode "
        l = hpInfo[0].split('.')
        ipString = '('
        for s in l:
            ipString += s
            ipString +=','
		    
        high = str(hpInfo[1]/256)
        low = str(hpInfo[1]%256)
        ipString += high
        ipString +=','
        ipString +=low
        ipString +=').\r\n'
        state.connection.send_response(responseString + ipString, callback, arg)
        my_print( "end _doPASV")

#############################################################################

    def _doSTOR(self, handle, callback, arg=None):
        global g_cond    
        my_print("in _doSTOR")
        
        inst = Connections()
        state = inst.getFtpControlConnection(handle)
        my_print("    arg is %s" % state.args )
        # probably should do some checks on the file here,
        # make sure we arn't overwritting, its not a directory,
        # and that we can write to that location, etc.
        if check_file(state, 'w'):
            # probably should setup on a per transfer basis
            #wapper.condV = Condition(Lock())
            #wrapper.condV.acquire()
            #wrapper.cond = 1
            #wrapper.condV.release()
            state.connection.data_connect_read(data_connect_read_cb, arg)
            state.connection.send_response(
                        "150 Opening BINARY mode data connection.\r\n", 
                        callback, arg)
        else:
            state.connection.send_response(
                    "553 %s: Permission denied on server." % state.args, dummy_cb, arg)

        #question is: do we wait untile rx finishes before allowing
        # any more commands? (FTP spec says we don't HAVE to wait :-)
        #arg.condV.acquire()
        #while arg.cond:
            #print "waiting on connect_read_cb to finish"
            #arg.condV.wait()
        #arg.condV.release()

        my_print ("end _doSTOR")

#############################################################################

    def _doUSER(self, handle, callback, arg=None):
        my_print("in _doUser")
        #print "    arg is", state.args
        #print "    type(arg)", type(state.args)
        inst = Connections()
        state = inst.getFtpControlConnection(handle)
        state.connection.send_response("""230 User is logged in.\r\n""", 
                        callback, arg)
        my_print("end _doUser")

#############################################################################

    def _doPASS(self, handle, callback, arg=None):
        my_print( "in _do_PASS")
        inst = Connections()
        state = inst.getFtpControlConnection(handle)
        state.connection.send_response("""230 Password is fine.\r\n""", callback, arg)
        my_print( "end _do_PASS")

#############################################################################

    def _doFEAT(self, handle, callback, arg=None):
        my_print("in _doFEAT")
        inst = Connections()
        state = inst.getFtpControlConnection(handle)
        state.connection.send_response("""211-Extensions supported:\r\n REST STREAM\r\n ESTO\n ERET\r\n MDTM\r\n SIZE\r\n PARALLEL\r\n DCAU\r\n211 END\r\n""", callback, arg)
        my_print("end _doFEAT")

#############################################################################

    def _doENC(self, handle, callback, arg=None):
        """This is NOT tested, Matt R. put this here -- not sure why?
        """
        my_print("start _doENC")
        inst = Connections()
        state = inst.getFtpControlConnection(handle)
        encryptCmdString = "ENC " + state.args
        ret, cmdString = ftpControlc.command_decode(handle, 
                                                        encryptCmdString)
        print "HERE in _doENC"
        my_print(cmdString)
        my_print("    response string: I DON'T KNOW")
        my_print("end _doENC")

#############################################################################

    def _doMODE(self, handle, callback, arg=None):
        my_print("in _doMODE")
        # All I understand for now is extened mode 'E'
        inst = Connections()
        state = inst.getFtpControlConnection(handle) 
        if state.args == 'E':
            state.connection.local_mode('E')
            state.connection.send_response("""200 MODE %s ok.\r\n""" %
                                                    state.args, callback, arg)
        else:
            state.connection.send_response("""502 Command not implemented.\r\n""",
                                                    callback, arg)
        my_print("end _doMODE")

#############################################################################

    def _doOPTS(self, handle, callback, arg=None):
        """All I have implemted here is the 'RETR Paralleism' opt,
        there is a slew of them that could be implemented if one 
        wanted to :-)
        """
        inst = Connections()
        state = inst.getFtpControlConnection(handle) 
        my_print("in _doOPTS")
        parts = state.args.split()
        #print "    parts ==", parts
        if parts[0] == 'RETR' and parts[1][0:12] == 'Parallelism=':
            parallel = parts[1][12:].split(',')
            parallelism = Parallelism()
            parallelism.set_size( int(parallel[1]) )
            state.connection.local_parallelism(parallelism)
            state.connection.send_response("""200 Opts successful.\r\n""", 
                                                    callback, arg)
        else:
            state.connection.send_response("""502 Command not implemented.\r\n""", 
                                                    callback, arg)
        my_print("end _doOPTS")

#############################################################################
    def _doPORT(self, handle, callback, arg=None):
        my_print("in _do_PORT")
        # my_print "    args is", arg.args
        inst = Connections()
        state = inst.getFtpControlConnection(handle)
        parts = state.args.split(',')
        host = map( int, parts[:4] )
        port =  int(parts[4])*256+int(parts[5])
        hp = HostPort(host,port)
        state.connection.local_port(hp)
        state.connection.send_response("""200 Port successful.\r\n""", callback, arg)
        my_print("end _do_PORT")

#############################################################################
    def _doSITE(self, handle, callback, arg=None):
        my_print("in _doSITE")
        inst = Connections()
        state = inst.getFtpControlConnection(handle)
        parts = state.args.split()
        if parts[0] == 'BUFSIZE':
            size = int(parts[1])
            TcpBuf = TcpBuffer()
            TcpBuf.set_fixed(size)
            state.connection.local_tcp_buffer(TcpBuf)
            resp = "214 TCP window size set to %d bytes.\r\n" % size
        else:
            resp = "500 'SITE %s': command not understood.\r\n" % state.args
        state.connection.send_response(resp, callback, arg)
        my_print("in _do_SITE")

#############################################################################
    def _doSIZE(self, handle, callback, arg=None):
        my_print("in _doSIZE")
        inst = Connections()
        state = inst.getFtpControlConnection(handle)
        print state
        if check_file(state, 'r'):
            print "checked file"
            stat = os.stat(state.args)
            state.connection.send_response("213 %u\r\n" % stat[6], callback, arg)
        else:
            state.connection.send_response("550 %s: Permission denied.\r\n" %
                             state.args, dummy_cb, arg)
        my_print("end _doSIZE")

#############################################################################
    def _doPBSZ(self, handle, callback, arg=None):
        my_print("in _doPBSZ")
        inst = Connections()
        state = inst.getFtpControlConnection(handle)
        # print "    arg is", state.args
        state.connection.local_pbsz(int(state.args))
        state.connection.send_response("200 PBSZ=%s\r\n" % state.args, callback, arg)
        my_print("end _doPBSZ")

#############################################################################

if __name__ == "__main__":
    cm = CommandMethods()

# vim: et ts=4 sw=4

