############################################################################
# Keith R. Jackson, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################
from pyGlobus.ftpClient import *
from pyGlobus.util import Buffer
from pyGlobus.util import GlobusException
from pyGlobus.ftpControl import Parallelism
from pyGlobus.ftpControl import TcpBuffer
from pyGlobus.ftpControl import FtpControl
from pyGlobus.ftpControl import PARALLELISM_FIXED
from pyGlobus.ftpControl import PARALLELISM_NONE
from pyGlobus.ftpControl import MODE_EXTENDED_BLOCK
from threading import *
from exceptions import Exception
import sys, getopt, timing

MEGABYTE = 1048576

g_cond = 0
g_dest = None
g_ftpClient = None
g_buffer = None
g_oldCbHandle = None
g_newCbHandle = None
g_totalRead = 0
g_blockRead = 0
g_elapsedMillis = 0
g_timeit = 0

def done_func(cv, handle, error):
    global g_cond
    if not error == "NONE":
        print "The following error occured: %s" % error
    cv.acquire()
    g_cond = 1
    cv.notify()
    cv.release()
    return
    
def print_throughput( bytes, elapsedMillis ):
    global g_timeit
    if g_timeit:
        sec = elapsedMillis / 1000.0
        megabits = bytes*8/1000000.0
        print "avg throughput =",megabits/sec,"Mb/s"
    return

def data_func(cv, handle, buffer, bufHandle, bufLen, offset, eof, error):
    global g_dest, g_ftpClient, g_buffer, g_oldCbHandle, g_newCbHandle, g_bytesRead, g_totalRead, g_blockRead, g_elapsedMillis
    g_dest.seek(offset) 
    g_dest.write(buffer)
    g_blockRead = g_blockRead + int(bufLen)
    if ( g_blockRead > 16*MEGABYTE ):
        g_totalRead += g_blockRead
        if g_timeit:
            timing.finish()
            millis = timing.milli()
            print "Read 16MB, progress =",g_totalRead/MEGABYTE,"MB,",
            print_throughput(g_blockRead,millis)
            g_elapsedMillis += int(millis)
            timing.start()
        g_blockRead = 0
    if not eof:
        try:
            handle = g_ftpClient.register_read(g_buffer, data_func, 0)
            if not g_oldCbHandle == None:
                g_ftpClient.free_callback(g_oldCbHandle)
            g_oldCbHandle = g_newCbHandle
            g_newCbHandle = handle
        except Exception, e:
            print "sys.exc_type is %s" % sys.exc_type
            print "sys.exc_value is %s" % sys.exc_value
            print "sys.exc_traceback is %s" % sys.exc_traceback
            sys.exit(-1)
    else:
        if g_timeit:
            timing.finish()
            millis = timing.milli()
            g_totalRead += g_blockRead
	    print "Read ",g_totalRead,"bytes,",
            print_throughput(g_totalRead,g_elapsedMillis+millis)
    return


def test(url, dest, para, buf):
    global g_cond, g_dest, g_ftpClient, g_buffer, g_newCbHandle, g_timeit
    g_dest = open(dest, "w")
    print "Writing to ",dest," with parallelism =",para,
    print " and TCP buffer =",buf,"K"
    condV = Condition(Lock())
    try:
        handleAttr = HandleAttr()
        opAttr = OperationAttr()
	pattr = Parallelism()
	if para > 0:
		pattr.set_mode(PARALLELISM_FIXED)
		pattr.set_size(int(para))
	else:
		pattr.set_mode(PARALLELISM_NONE) 
	opAttr.set_parallelism(pattr)
	battr = TcpBuffer()
	battr.set_fixed(int(buf)*1024)
	opAttr.set_tcp_buffer(battr)
	opAttr.set_mode(MODE_EXTENDED_BLOCK)
        g_ftpClient = FtpClient(handleAttr)
        g_ftpClient.get(url,done_func, condV,opAttr)
        g_buffer = Buffer(64*1024)
        if g_timeit == 1:
            timing.start()
        handle = g_ftpClient.register_read(g_buffer, data_func, 0)
        g_newCbHandle = handle
    except GlobusException, ex:
        print ex
        sys.exit(-1)
    condV.acquire()
    while g_cond == 0:
        print "waiting on callbacks"
        condV.wait()
        condV.release()
    g_dest.close()
    del g_buffer
    del g_ftpClient

def usage():
    useStr = """Usage: parallel_get [-bhpt] -s <source url>
                            -d <destination file>
                            -p <parallelism> 
                            -b <TCP buffer in KB>
                            -t Turn on timing/throughput (default=off)"""
    print useStr

def getArgs():
    global g_timeit
    if len(sys.argv) == 1:
        usage()
        sys.exit(-1)
    try:
        opts, args = getopt.getopt(sys.argv[1:], "hts:d:p:b:")
    except getopt.GetoptError:
        usage()
        sys.exit(-1)
    dest = "/dev/null"
    para = 0
    buf  = 64
    timings = 0
    for o, a in opts:
        if o == "-h":
            usage()
            sys.exit(-1)
        elif o == "-s":
            url = a
        elif o == "-d":
            dest = a
	elif o == "-p":
	    para = a
        elif o == "-b":
	    buf = a
        elif o == "-t":
            timings = 1
    g_timeit = timings
    return url, dest, para, buf

url, dest, para, buf = getArgs()
test(url, dest, para, buf)

