############################################################################
# Keith R. Jackson, LBNL
# Joshua R. Boverhof, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################
from pyGlobus import ftpClient
from pyGlobus.util import GlobusException
from threading import Condition, Lock
from exceptions import Exception
from pyGlobus.ftpControl import Parallelism
from pyGlobus.ftpControl import ParallelismException
from pyGlobus.ftpControl import MODE_EXTENDED_BLOCK
import sys, getopt

g_cond = 0
THROUGHPUT_PLUGIN = 'ThoughputPerfPlugin'
PERFORMANCE_PLUGIN = 'PerformancePlugin'
RESTARTMARKER_PLUGIN = 'RestartMarkerPlugin'
def complete_CB(cv, handle, error):
    print "--> complete_CB called"
    global g_cond
    if not error == "NONE":
        print "The following error occured: %s" % error
    cv.acquire()
    g_cond = 1
    cv.notify()
    cv.release()
    return
# --------------------- RestartMarkerPlugin Callbacks ------------
def beginCB(handle, ftp_handle, src_url, dst_url, marker):
    print "*** beginCB ***"
    print "\thandle %(handle)s ftp_handle %(ftp_handle)s src_url %(src_url)s dst_url %(dst_url)s" %vars()
    return 0
def markerCB(handle, ftp_handle, marker):
    print "*** markerCB ***"
    print "\thandle %(handle)s ftp_handle %(ftp_handle)s marker %(marker)s" %vars()
    return
def completeCB(handle, ftp_handle, err, url):
    print "handle=%(handle)s ftp_handle=%(ftp_handle)s err=%(src_url)s url=%(url)s" %vars()
    return
# --------------------- ThroughPut Plugin Callbacks --------
def beginTP_CB(handle, ftp_handle, srcUrl, destUrl):
    print "*** beginTP_CB ***"
    print (("\thandle(%s)\n\tftp_handle(%s)\n\tsrc(%s)\n\tdst(%s)")
           %(handle, ftp_handle, srcUrl, destUrl))
    return
def stripeTP_CB(handle, ftp_handle, stripe_ndx, bytes, inst_thrput, avg_thrput):
    print "*** markerTP_CB *** "
    print "\thandle(%s)\n\tftp_handle(%s)\n\tstrip(%d)" %(handle, ftp_handle, stripe_ndx)
    print "\n\tbytes(%d) inst(%d bytes/sec) avg(%d bytes/sec)" %(bytes, inst_thrput, avg_thrput)
    return
def totalTP_CB(handle, ftp_handle, bytes, inst_thrput, avg_thrput):
    print "*** markerTP_CB *** "
    print (("\thandle(%s)\n\tftp_handle(%s)\n\tbytes(%d)\n\tinst(%d bytes/sec)\n\tavg(%d bytes/sec)")
           %(handle, ftp_handle, bytes, inst_thrput, avg_thrput))
    return
def completeTP_CB(handle, ftp_handle, success):
    print "*** completeTP_CB *** "
    print "\thandle(%s)\n\tftp_handle(%s)\n\tsuccess(%s)" %(handle, ftp_handle, success)
    return
# --------------------- PerformanceMarkerPlugin Callbacks --------
def beginPM_CB(handle, ftp_handle, srcUrl, destUrl, restart):
    print "*** beginPM_CB ***"
    msg = "\thandle(%s)\n\tftp_handle(%s)\n\tsrc(%s)\n\tdst(%s)\n\trestart(%d)"
    print ((msg %(handle, ftp_handle, srcUrl, destUrl, restart)))
    return
def markerPM_CB(handle, ftp_handle, time_stamp, time_stampL, stripe_ndx, num_stripes, nbytes):
    print "*** markerPM_CB ***"
    msg = "\thandle(%s)\n\tftp_handle(%s)\n\time(%d)\n\tstripe(%d)\n\tnum(%d)\n\tbytes(%d)"
    print ((msg %(handle, ftp_handle, time_stamp, stripe_ndx, num_stripes, nbytes)))
    #msg = "\thandle(%s)\n\tftp_handle(%s)\n\ttime(%d)\n\tstripe(%d)\n\tnum(%d)"
    #print ((msg %(handle, ftp_handle, time_stamp, stripe_ndx, num_stripes)))
    return
def completePM_CB(handle, ftp_handle, success):
    print "*** completePM_CB ***"
    msg = "\thandle(%s)\n\tftp_handle(%s)\n\tsuccess(%s)"
    print ((msg %(handle, ftp_handle, success)))
    return
# ----------------------------------------------------------------
# ----------------------------------------------------------------
def test(src, dst,para, *plugins):
    global g_cond
    print "Transfering from ",src," to ", dst, " with parallelism = ",para,
    print "Using plugins: "
    for i in plugins:
        if i ==  RESTARTMARKER_PLUGIN:
            print "RESTARTMARKER_PLUGIN ",
        elif i == THROUGHPUT_PLUGIN:
            print "THROUGHPUT_PLUGIN ",
        else:
            print "PERFORMANCE_PLUGIN "
            
    condV   = Condition(Lock())
    srcAttr = ftpClient.OperationAttr()
    dstAttr = ftpClient.OperationAttr()
    
    handleAttr = ftpClient.HandleAttr()
    srcAttr.set_mode(MODE_EXTENDED_BLOCK)
    dstAttr.set_mode(MODE_EXTENDED_BLOCK)
    if para != 0:
        try:
            parallelism = Parallelism()
            parallelism.set_mode(1)
            parallelism.set_size(int(para))
            srcAttr.set_parallelism(parallelism)
            dstAttr.set_parallelism(parallelism)
        except ParallelismException, ex:
            Log_exception(ex)
            parallelism = None

    for i in plugins:
        if i == RESTARTMARKER_PLUGIN:
            try:
                rm_plugin = ftpClient.RestartMarkerPlugin(beginCB,
                                                          markerCB,
                                                          completeCB,
                                                          ('test_restartmarker_plugin'))
            except ftpClient.PluginException, ex:
                for j in ex.args: print j
            else:
                handleAttr.add_plugin(rm_plugin)
        if i == THROUGHPUT_PLUGIN:
            try:
                tp_plugin = ftpClient.ThoughputPerfPlugin(beginTP_CB, stripeTP_CB,
                                                          totalTP_CB, completeTP_CB,
                                                          ('test_throughput_plugin'))
            except ftpClient.PluginException, ex:
                for j in ex.args: print j
            else:
                print i
                handleAttr.add_plugin(tp_plugin)
        elif i == PERFORMANCE_PLUGIN:
            try:
                pf_plugin = ftpClient.PerformanceMarkerPlugin(beginPM_CB, markerPM_CB, completePM_CB,
                                                          ('test_performance_plugin'))
            except ftpClient.PluginException, ex:
                for j in ex.args: print j
            else:
                print i
                handleAttr.add_plugin(pf_plugin)
    try:
        g_ftpClient = ftpClient.FtpClient(handleAttr)
        cb = g_ftpClient.third_party_transfer(src, dst, complete_CB, condV, srcAttr, dstAttr)
    except GlobusException, ex:
        print ex.args
        sys.exit(-1)
    condV.acquire()
    while g_cond == 0:
        print "waiting on callbacks"
        condV.wait()
        condV.release()
    del g_ftpClient

def usage():
    useStr = """Usage: third_party_transfer [-hrptm] -s <source url> -d <dst url> \n\tr[restart marker] p[performance] t[throughput] m[parallel streams]"""
    print useStr
    sys.exit(1)

def getArgs():
    url_src, url_dst, plugin_tuple = None, None, ()
    if len(sys.argv) <= 1:
       usage() 
    else:
        try:
            opts, args = getopt.getopt(sys.argv[1:], "m:hrpts:d:")
            #print opts
            #print args
            para = 0
        except getopt.GetoptError:
            usage()
        exclusive = 0
        for o, a in opts:
            if o == "-h":
                usage()
                sys.exit(-1)
            elif o == "-s":
                url_src = a
            elif o == "-d":
                url_dst = a
            elif o == "-t":
                if exclusive: usage()
                exclusive = 1
                plugin_tuple += (THROUGHPUT_PLUGIN,)
            elif o == "-p":
                if exclusive: usage()
                exclusive = 1
                plugin_tuple += (PERFORMANCE_PLUGIN,)
            elif o == "-r":
                plugin_tuple += (RESTARTMARKER_PLUGIN,)
            elif o == "-m":
                para = a
            else:
                usage()
            
    return url_src, url_dst, plugin_tuple, para

def main():
    url_src, url_dst, pt, para = getArgs()
    test(url_src, url_dst,para, *pt)

if __name__=='__main__':
        main()
