###########################################################################
#Dan Gunter, LBNL
#Copyright (c) 2005, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################
"""
Sample XIO client.
"""
__usage = """Usage: %s host port [#KB] [KB/sec]

Example: %s myhost.mydomain.org 12345 100
"""
# -- Imports
import os
import sys
import time
#
import pyGlobus.xio
from pyGlobus.xio import XioSocket

# -- Functions

sixty4_kb = '.' * (64*1024)
one_kb = '.' * 1024

def run(host,port,n,rate):
    sock = XioSocket()
    sock.connect((host,port))
    if rate:
        for i in xrange(n/rate):
            for j in xrange(rate):
                sock.sendall(one_kb)
            time.sleep(1)
    else:
        for i in xrange(n/64):
            sock.sendall(sixty4_kb)
        for i in xrange(n % 64):
            sock.sendall(one_kb)
    sock.close()
    
def main(args):
    if len(args) < 3:
        usage()
    host = args[1]
    try:
        port = int(args[2])
        if port < 0 or port > 65536:
            raise ValueError()
    except ValueError:
        usage("port argument must be a non-negative integer")
    n = 1000
    if len(args) > 3:
        try:
            n = int(args[3])
            if n < 1:
                raise ValueError()
        except ValueError:
            usage("# kilobytes argument must be an integer >= 1")
    r = None
    if len(args) > 4:
        try:
            r = int(args[4])
            if r < 1:
                raise ValueError()
        except ValueError:
            usage("KB/s argument must be an integer >= 1")
    t = time.time()
    run(host,port,n,r)
    e = time.time() - t
    print "Time to write %dKB = %gsec" % (n,e)

def usage(s=''):
    if s: print >>sys.stderr,s
    p = os.path.basename(sys.argv[0])
    print >>sys.stderr,__usage % (p,p)
    sys.exit(-1)
    
if __name__ == '__main__':
    main(sys.argv)
