###########################################################################
#Dan Gunter, LBNL
#Copyright (c) 2005, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################
"""
Sample XIO server.
"""
__usage = """Usage: %s [host] port

Example: %s 12345
"""
# -- Imports
import os
import sys
import time
#
from socket import SO_REUSEADDR, SOL_SOCKET
try:
    import pyGlobus.xio
    from pyGlobus.xio import XioSocket as socket
    from pyGlobus.xio import XioSocketException as socket_error
except ImportError:
    print "Warning: Could not import pyGlobus xio module"
    from socket import socket
    from socket import error as socket_error

# -- Functions

sixty4_kb = '.' * (64*1024)
one_kb = '.' * 1024

def run(host,port):
    lsock = socket()
    #lsock.setsockopt(SOL_SOCKET,SO_REUSEADDR,1)
    lsock.bind((host,port))
    lsock.listen(5)
    sock, peer = lsock.accept()
    t = time.time()
    n = 0
    while 1:
        try:
            data = sock.recv(65536)
            n += len(data)
        except socket_error:
            break
        if not data:
            break
    return n/1024,t

def main(args):
    if len(args) < 2:
        usage()
    if len(args) > 2:
        host = args[1]
        parg = 2
    else:
        host = ''
        parg = 1
    try:
        port = int(args[parg])
        if port < 0 or port > 65536:
            raise ValueError()
    except ValueError:
        usage("port argument must be a non-negative integer")
    while 1:
        n,t = run(host,port)
        e = time.time() - t
        print "Time to read %dKB = %gsec" % (n,e)

def usage(s=''):
    if s: print >>sys.stderr,s
    p = os.path.basename(sys.argv[0])
    print >>sys.stderr,__usage % (p,p)
    sys.exit(-1)
    
if __name__ == '__main__':
    main(sys.argv)
