############################################################################
# Keith R. Jackson, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################
"""
A module that forms rsl strings from python dictionaries
"""
## Attempt a version string
__author__  = "Keith R. Jackson"
__status__  = "alpha"
__version__ = "$Revision: 1.9 $"
__rcsid__   = "$Id: rsl.py,v 1.9 2004/01/30 20:49:25 nedelson Exp $"

from pyGlobus.util import GlobusException

try:
    import cStringIO as StringIO
except ImportError:
    import StringIO

class RSLException(GlobusException):
    """
    Errors in RSL modules

    A simple sub-class of GlobusException, it adds no new methods.

    **extends:**
        pyGlobus.util.GlobusException
    """
    pass

class RSL:
    def __init__(self, executable, **kw):
        self._executable = executable
        self._arguments = None
        self._count = None
        self._directory = None
        self._dry_run = None
        self._environment = None
        self._file_clean_up = None
        self._file_stage_in = None
        self._file_stage_in_shared = None
        self._file_stage_out = None
        self._gass_cache = None
        self._gram_my_job = None
        self._host_count = None
        self._jobType = None
        self._max_cpu_time = None
        self._max_memory = None
        self._max_wall_time = None
        self._min_memory = None
        self._project = None
        self._proxy_timeout = None
        self._queue = None
        self._remote_io_url = None
        self._restart = None
        self._rsl_substitution = None
        self._save_state = None
        self._scratch_dir = None
        self._stderr = None
        self._stderr_position = None
        self._stdin = None
        self._stdout = None
        self._stdout_position = None
        self._two_phase = None
        if kw.has_key('arguments'):
            val = kw['arguments']
            sio = StringIO.StringIO()
##             sio.write('"')
            if isinstance(val, list) or isinstance(val, tuple):
                i = 0
                for item in val:
                    if i > 0:
                        sio.write(' ')
                    sio.write('"%s"' % item)
                    i += 1
            else:
                sio.write('"%s"' % val)
##             sio.write('"')
            self._arguments = sio.getvalue()
            sio.close()
        if kw.has_key('count'):
            self._count = kw['count']
        if kw.has_key('directory'):
            self._directory = kw['directory']
        if kw.has_key('dry_run'):
            self._dry_run = kw['dry_run']
        if kw.has_key('environment'):
            val = kw['environment']
            sio = StringIO.StringIO()
            if isinstance(val, tuple):
                sio.write('(%s "%s")' % (val[0], val[1]))
            elif isinstance(val, list):
                for item in val:
                    sio.write('(%s "%s")' % (item[0], item[1]))
            else:
                ex = RSLException("environment must be either a list of " +
                                    "(env, val) tuples, or a single (env, val) tuple")
                raise ex
            self._environment = sio.getvalue()
            sio.close()
        if kw.has_key('file_clean_up'):
            val = kw['file_clean_up']
            if isinstance(val, str):
                self._file_clean_up
            else:
                sio = StringIO.StringIO()
                for item in val:
                    sio.write(item + ' ')
                self._file_clean_up = sio.getvalue()[:-1]
                sio.close()
        if kw.has_key('file_stage_in'):
            val = kw['file_stage_in']
            sio = StringIO.StringIO()
            for item in val:
                sio.write('(%s %s)' % (item[0], item[1]))
            self._file_stage_in = sio.getvalue()
            sio.close()
        if kw.has_key('file_stage_in_shared'):
            val = kw['file_stage_in_shared']
            sio = StringIO.StringIO()
            for item in val:
                sio.write('(%s %s)' % (item[0], item[1]))
            self._file_stage_in_shared = sio.getvalue()
            sio.close()
        if kw.has_key('file_stage_out'):
            val = kw['file_stage_out']
            sio = StringIO.StringIO()
            for item in val:
                sio.write('(%s %s)' % (item[0], item[1]))
            self._file_stage_out = sio.getvalue()
            sio.close()
        if kw.has_key('gass_cache'):
            self._gass_cache = kw['gass_cache']
        if kw.has_key('gram_my_job'):
            self._gram_my_job = kw['gram_my_job']
        if kw.has_key('host_count'):
            self._host_count = str(kw['host_count'])
        if kw.has_key('jobType'):
            self._jobType = kw['jobType']
        if kw.has_key('max_cpu_time'):
            self._max_cpu_time = str(kw['max_cpu_time'])
        if kw.has_key('max_memory'):
            self._max_memory = str(kw['max_memory'])
        if kw.has_key('max_wall_time'):
            self._max_wall_time = str(kw['max_wall_time'])
        if kw.has_key('min_memory'):
            self._min_memory = str(kw['min_memory'])
        if kw.has_key('project'):
            self._project = kw['project']
        if kw.has_key('proxy_timeout'):
            self._proxy_timeout = str(kw['proxy_timeout'])
        if kw.has_key('queue'):
            self._queue = kw['queue']
        if kw.has_key('remote_io_url'):
            self._remote_io_url = kw['remote_io_url']
        if kw.has_key('restart'):
            self._restart = kw['restart']
        if kw.has_key('rsl_substitution'):
            val = kw['rsl_substitution']
            sio = StringIO.StringIO()
            if isinstance(val, tuple):
                sio.write('(%s "%s")' % (val[0], val[1]))
            elif isinstance(val, list):
                for item in val:
                    sio.write('(%s "%s")' % (item[0], item[1]))
            else:
                ex = RSLException("rsl_substitution must be either a list of " +
                                    "(var, val) tuples, or a single (var, val) tuple")
                raise ex
            self._rsl_substitution = sio.getvalue()
            sio.close()
        if kw.has_key('save_state'):
            self._save_state = kw['save_state']
        if kw.has_key('scratch_dir'):
            self._scratch_dir = kw['scratch_dir']
        if kw.has_key('stderr'):
            self._stderr = kw['stderr']
        if kw.has_key('stderr_position'):
            self._stderr_position = str(kw['stderr_position'])
        if kw.has_key('stdin'):
            self._stdin = kw['stdin']
        if kw.has_key('stdout'):
            self._stdout = kw['stdout']
        if kw.has_key('stdout_position'):
            self._stdout_position = str(kw['stdout_position'])
        if kw.has_key('two_phase'):
            self._two_phase = str(kw['two_phase'])



    def __str__(self):
        sio = StringIO.StringIO()
        sio.write('&')
##         pass #print "sio is: %s" % `sio`
        sio.write('(executable=%s)' % self._executable)
        if self._arguments:
            sio.write('(arguments=%s)' % self._arguments)
        if self._count:
            sio.write('(count=%s)' % str(self._count))
        if self._directory:
            sio.write('(directory=%s)' % self._directory)
        if self._dry_run:
            sio.write('(dry_run=%s)' % self._dry_run)
        if self._environment:
            sio.write('(environment=%s)' % self._environment)
        if self._file_clean_up:
            sio.write('(file_clean_up=(%s))' % self._file_clean_up)
        if self._file_stage_in:
            sio.write('(file_stage_in=(%s))' % self._file_stage_in)
        if self._file_stage_in_shared:
            sio.write('(file_stage_in_shared=(%s))' % self._file_stage_in_shared)
        if self._file_stage_out:
            sio.write('(file_stage_out=(%s))' % self._file_stage_out)
        if self._gass_cache:
            sio.write('(gass_cache=%s)' % self._gass_cache)
        if self._gram_my_job:
            sio.write('(gram_my_job=%s)' % self._gram_my_job)
        if self._host_count:
            sio.write('(host_count=%s)' % self._host_count)
        if self._jobType:
            sio.write('(jobType=%s)' % self._jobType)
        if self._max_cpu_time:
            sio.write('(max_cpu_time=%s)' % self._max_cpu_time)
        if self._max_memory:
            sio.write('(max_memory=%s)' % self._max_memory)
        if self._max_wall_time:
            sio.write('(max_wall_time=%s)' % self._max_wall_time)
        if self._min_memory:
            sio.write('(min_memory=%s)' % self._min_memory)
        if self._project:
            sio.write('(project=%s)' % self._project)
        if self._proxy_timeout:
            sio.write('(proxy_timeout=%s)' % self._proxy_timeout)
        if self._queue:
            sio.write('(queue=%s)' % self._queue)
        if self._remote_io_url:
            sio.write('(remote_io_url=%s)' % self._remote_io_url)
        if self._restart:
            sio.write('(restart=%s)' % self._restart)
        if self._rsl_substitution:
            sio.write('(rsl_substitution=%s)' % self._rsl_substitution)
        if self._save_state:
            sio.write('(save_state=%s)' % self._save_state)
        if self._scratch_dir:
            sio.write('(scratch_dir=%s)' % self._scratch_dir)
        if self._stderr:
            sio.write('(stderr=%s)' % self._stderr)
        if self._stderr_position:
            sio.write('(stderr_position=%s)' % self._stderr_position)
        if self._stdin:
            sio.write('(stdin=%s)' % self._stdin)
        if self._stdout:
            sio.write('(stdout=%s)' % self._stdout)
        if self._stdout_position:
            sio.write('(stdout_position=%s)' % self._stdout_position)
        if self._two_phase:
            sio.write('(two_phase=%s)' % self._two_phase)


##         sio.write(')')
        val = sio.getvalue()
        sio.close()
        return val

