############################################################################
# Matt Rodriguez, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory
#(subject to receipt of any required approvals from the U.S. Dept. of Energy).
#All rights reserved.
########################################################################### 

from pyGlobus import security
import sys, os


everything_dict = { 'gramClient':   [ 
                  ("""What is the hostname of your GRAM server?""", 'gramhost.yourdomain', 'rm'),    
                  ("""What rsl string do you want to use to specify your job?""",'&(executable=/bin/date)', 'rsl'),
                  ("""What rsl string specifies a job that runs long enough to cancel?""",'&(executable=/bin/sleep)(arguments="10")','sleeper_rsl')
                                    ],
                    'gassServerEZ': [
                  ("""What is the hostname of your GRAM server?""", 'gramhost.yourdomain', 'rm'),
                  ("""What rsl string do you want to use to specify your job?""", '&(executable=/bin/date)', 'rsl')
                                    ],
                    'ftpClient':    [
                  ("""What source file do you want to do want to use for a GridFTP put?""", '/etc/group', 'src'),
                  ("""What is the url where the file will be placed?""", "gsiftp://scott.lbl.gov/home/portnoy/shared/pyGlobus-tests/ftpClient/ftpPutResult", "url"),
                  ("""Where do you want the file that you get to be placed?""", "./FtpResult", "dest"),
                  ("""What is the source url that you want to use in a 3rd party transfer?""", 'gsiftp://diesel.lbl.gov/etc/group', "srcUrl"),
                  ("""What is the destination url that you want to use in a 3rd party transfer?""",'gsiftp://portnoyc.lbl.gov/home/portnoy/shared/pyGlobus-tests/ftpClient/thirdpartyresult', 'destUrl'),
                  ("""What is the url of the directory that you want to be listed?""",'gsiftp://scott.lbl.gov/tmp','ftpList'),
                  ("""What is the directory that you want to be created? It will be automatically removed later.""", 'gsiftp://scott/home/portnoy/shared/pyGlobus-tests/ftpClient/ftpMkdirTest', 'ftpDir'),
                  ("""What is the url that you want to move a file?""", 'gsiftp://scott/home/portnoy/shared/pyGlobus-tests/ftpClient/ftpMkdirTest/test_ftpMove','fileName'),
                  ("""What is a bad url to use for an existence test?""", 'gsiftp://diesel.lbl.gov/etc/grou', 'badUrl'),
                  ("""What is the url of a big file that we can start a transfer then later abort it?""", 'gsiftp://diesel.lbl.gov/tmp/bigFile', "bigUrl"),
                  ("""How many streams do you want to use on a parallel transfer?""", "4", "streams"),
                  ("""Which file would you like the Debug plugin to write to? 1 is stdout, 2 is stderr, or just a path""", "1", "debugFile")
                                    ],
                  'ftpControl':     [
                  ("""What is the hostname of your gridftp server?""", "gridftphost.yourdomain", 'host')],
                  'gassCopy':       [
                  ("""What is a url from which you can make a GASS copy? """,
                  "gsiftp://portnoyc.lbl.gov/etc/group", "gsiftpSrcUrl"),
                  ("""What is a url to which you can make a GASS copy?""",
                  "gsiftp://scott.lbl.gov/tmp/foobar1234", "gsiftpDestUrl")
                                    ],
                  'gassTransfer':   [
                  ("""What is the source file that you want to transfer?""",
                   "/etc/group", "src"),
                  ("""What is the destination of the file that you want to get?""",               "/home/portnoy/shared/pyGlobus-tests/gassTransfer/gassGetResult", "destGet"),
                  ("""What is the destination of the file that you want to put?""", "/home/portnoy/shared/pyGlobus-tests/gassTransfer/gassPutResult", "destPut"),
                  ("""What is the destination of the file that you want to append?""", "/home/portnoy/shared/pyGlobus-tests/gassTransfer/gassAppendResult", "destAppend")
                                     ],
                  'gassCache':       [ 
                  ("""What is home directory where the gassCache information will be stored? This sets an environment variable GLOBUS_GASS_CACHE_DEFAULT.""", "gassCacheDirectory", "homeDir"),
                  ("""What is a url that you would like to add to the cache?""", "gsiftp://scott.lbl.gov/etc/group", "url"),
                  ("""What is a tag that you want to associate with that url?""", "gsiftp://scott.lbl.gov/etc/group", "tag")
                                     ],
                  'rlsClient':       [
                  
                  ("""What is the url of your rls server?""", "rls://bosshog.lbl.gov", "url"),
                  ("""What is the name of a logical page name that you would like to use in the rls tests?""", "testLfn", "lfn"), 
                  ("""What is the name of a physical page name that you would like to use in the rls tests? This must be an actual filename. """, "/etc/group", "pfn1"),
                  ("""What is the name of another physical page name that you would like to use in the rls tests? This must be an actual filename.""", "/etc/group", "pfn2"), 
                  ("""What is a regular expression that you would like to use to search for logical and physical page names in the RLS server's replica collection?""", "*", "regExp"),
                  ("""What is a physical file name that does not exist in the replica collection?""", "/etc/grou", "badFn")
                                     ],
                  'myProxy':         [
                  ("What is a host that is running a myproxy server?""","myproxy.ncsa.uiuc.edu", "host") 
                                     ],
                  'xio':             [
                  ("What is a port that a can be listened on for the xio tests?", "5555", "ListenPort")
                                     ],
                  'io':              [
                  ("What is a port that a can be listened on for the io tests?", "14381", "ListenPort1"),
                  ("What is another port that a can be listened on for the io tests?", "14383", "ListenPort2"),
                  ("What is another port that a can be listened on for the io tests?", "14385", "ListenPort3")
                                     ],
                 'GSISOAP':          [
                 ("""What is a port that we can have a GSISOAP server listen on?""", "8084", "HttpPort"),
                 ("""What is the hostname of the GSISOAP server? localhost is fine for the test.""", "localhost", "HttpHost")
                                     ],
                 'asynchio':         [
                 ("""What is a port that can be listened on for the asynchronous io tests?""", "14381", "serverport")
                                     ]             
                  }    
                  

def handle_question(question, default=""): 
    print question
    print '[',default,']'
    answer = sys.stdin.readline()
    if answer == '\n':
        return default
    else: 
        return answer.strip()


print """*****************************************************************"""
print """Welcome to the pyGlobus test configuration script builder.""" 
print """I'm going to ask a series of questions and for each question try to guess what a good answer will be."""
print """At the end of the script you should have a configuration file that will
let you run the pyGlobus testing framework."""
print """*****************************************************************"""

CONF_Q = "What do you want the name of the configuration file to be called?"
conf_file_default = "config_generated.txt"
conf_file = handle_question(CONF_Q,conf_file_default)

#Security
ISSUER_Q = "What is the issuer of the proxy that you want to use for the tests?"
s,d = security.grid_proxy_info()
issuer_default = d['issuer'] 
issuer = handle_question(ISSUER_Q,issuer_default) 
username = os.getenv("LOGNAME")


GRIDMAP_Q = "What do you want the name of the gridmap file to be called?"
gridmap_default = 'gridmap'
gridmap = handle_question(GRIDMAP_Q, gridmap_default)

GRIDMAPENV_Q = """Where do you want the gridmap file stored? This also sets a
GRIDMAP environment variable which is set to the location of the gridmap file."""
gridmapenv_default = os.path.join(os.getcwd(), gridmap)
gridmapenv = handle_question(GRIDMAPENV_Q, gridmapenv_default)

gridmap_file = open(gridmap, "w")
gridmap_file.write("\"%s\"" % issuer )
gridmap_file.write(" ")
gridmap_file.write(username)
f = open(conf_file, "w")
f.write('[security]\n')
f.write('gridmapenv = ')
f.write(gridmapenv)
f.write('\n')
f.write('\n')

for section in everything_dict:
    f.write('[%s]\n' % section)    
    for question_tuple in everything_dict[section]:
        answer = handle_question(question_tuple[0], question_tuple[1])
        f.write(question_tuple[2] + ' = ')
        f.write(answer)
        f.write('\n')
    f.write('\n')
