############################################################################
# Matt Rodriguez, LBNL
# Keith Beattie, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################

import base64, commands, cStringIO, getopt, socket, sys

from pyGlobus.security import GSSCred
from pyGlobus.security import GSSContext
from pyGlobus.security import GSSName, GSSMechs, GSSUsage, GSSBuffer
from pyGlobus.security import GSSContextException
from pyGlobus import gssc

from Ft.Xml import Domlette, XPath
from xml.ns import SOAP, SCHEMA

# The following dict is used when constructing xml messages, holding
# namespace definitions and other values needed in the xml.
xmlDict = {}
xmlDict['soapenv']    = SOAP.ENV
xmlDict['xsd']        = SCHEMA.XSD3
xmlDict['xsi']        = SCHEMA.XSI3
xmlDict['xmlns']      = "http://ogsa.globus.org/security/authentication"
xmlDict['mech-type']  = "http://www.globus.org/2002/04/xmlenc#gssapi-gsi"
xmlDict['ns1']        = "http://ogsa.globus.org/security/authentication/types"
xmlDict['context-id'] = 42

# The formatted tesmplate string for initTokenExchangeResponse
# messages
initTokenExchResp_tmpl = '''<soapenv:Envelope xmlns:soapenv="%(soapenv)s"
                  xmlns:xsd="%(xsd)s"
                  xmlns:xsi="%(xsi)s">
  <soapenv:Body>
    <initTokenExchangeResponse xmlns="%(xmlns)s">
      <ns1:contextTokenOut xsi:type="ns1:ContextTokenOutType"
                           ns1:continue-needed="%(continue_needed)s"
                           xmlns:ns1="%(ns1)s">
        <ns1:base64Token>%(b64Token)s</ns1:base64Token>
        <ns1:context-id>%(context-id)s</ns1:context-id>
      </ns1:contextTokenOut>
    </initTokenExchangeResponse>
  </soapenv:Body>
</soapenv:Envelope>
'''

# The formatted template string for continueTokenExchangeResponse
# messages.
contTokenExchResp_tmpl = '''<soapenv:Envelope xmlns:soapenv="%(soapenv)s"
                  xmlns:xsd="%(xsd)s"
                  xmlns:xsi="%(xsi)s">
 <soapenv:Body>
  <continueTokenExchangeResponse xmlns="%(xmlns)s">
   <ns1:contextTokenOut xsi:type="ns1:ContextTokenOutType"
                        ns1:continue-needed="%(continue_needed)s"
                        xmlns:ns1="%(ns1)s">
    <ns1:base64Token>%(b64Token)s</ns1:base64Token>
    <ns1:context-id>%(context-id)s</ns1:context-id>
   </ns1:contextTokenOut>
  </continueTokenExchangeResponse>
 </soapenv:Body>
</soapenv:Envelope>
'''

def acceptContext(host,port):

    name = GSSName()
    mechs = GSSMechs()
    usage = GSSUsage()
    usage.set_usage_accept()
    cred = GSSCred()
    cred.acquire_cred(name, mechs, usage)
    lifetime, cred_name = cred.inquire_cred()
    print "Cred:"
    print "      name: %s" % (cred_name.display())
    print "  lifetime: %d" % (lifetime)

    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM, 0)
    s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
    s.bind((host, port))
    s.listen(5)
    print "Listening on: %s:%s... " % (host, port)
    sock, addr = s.accept()
    print "Accept from %s:%s" % addr
    sockf = sock.makefile()
    
    ctx = GSSContext()

    uuid = commands.getoutput("uuidgen").split("-")

    # First, read the initTokenExchange message
    print "Reading initTokenExchange:"
    token, continue_needed, msg_size = readToken(sockf, context_id=uuid[4])
    print "  read %s bytes, cont=%s, null_token=%s" % (msg_size, continue_needed, token == None)
    
    # Calculate the responce token
    major, minor, token = ctx.accept_context(cred, token)

    # Then send the initTokenExchangeResponce
    print "Sending initTokenExchangeResponce:"
    continue_needed, msg_size = sendToken(sockf, initTokenExchResp_tmpl, major, token)
    print "  wrote %s bytes, cont=%s, null_token=%s" % (msg_size, continue_needed, token==None)

    # Now start the loop of continueTokenExchange messages.
    while continue_needed:

        print "Reading continueTokenExchange:"
        token, continue_needed, msg_size = readToken(sockf)
        print "  read %s bytes, cont=%s, null_token=%s" % (msg_size, continue_needed, token==None)
        major, minor, token = ctx.accept_context(cred, token)

        print "Sending continueTokenExchangeResponce:"
        continue_needed, msg_size = sendToken(sockf, contTokenExchResp_tmpl, major, token)
        print "  sent %s bytes, cont=%s, null_token=%s" % (msg_size, continue_needed, token==None)


    print "=== Context established ===\n"

    return sockf, ctx


def readToken(sockf, context_id=None):
    ''' readToken(sockf) -> token, continue_needed, msg_size

    Read a message from the given socket.  Return the binary token,
    the continue needed flag, and the size of the xml message read.
    If context_id is set, use it as the last 12 bytes of the
    context_id to use.'''

    # Read from the socket until a line ending with
    # '</soapenv:Envelope>' is read.

    msg = cStringIO.StringIO()
    line = ''
    while not line.endswith('</soapenv:Envelope>\n'):
        line = sockf.readline()
        msg.write(line)

    msg_str = msg.getvalue()
    dom = Domlette.NonvalidatingReader.parseString(msg_str, 'memory://')

    ctx = XPath.Context.Context(dom, processorNss={'ns1':xmlDict['ns1']})

    b64Token_el = XPath.Evaluate("//ns1:base64Token", context=ctx)[0]
    b64Token = b64Token_el.firstChild.data

    cn_attr = XPath.Evaluate("//child::*/@ns1:continue-needed", context=ctx)[0]
    cont_needed = (cn_attr.value.lower() == 'true')
    
    if context_id:
        context_id_el = XPath.Evaluate("//ns1:context-id", context=ctx)[0]
        uuid = context_id_el.firstChild.data.split("-")
        xmlDict['context-id'] = "%s-%s-%s-%s-%s" % (uuid[0], uuid[1], uuid[2], uuid[3], context_id)

    return base64.decodestring(b64Token_el.firstChild.data), cont_needed, len(msg_str)


def sendToken(sockf, msg_tmpl, major, token):
    ''' sendToken(sockf, msg_tmpl, major, token) -> continue_needed, msg_size

    Construct a message based on the msg_tmpl, with the given token,
    using major to determine the continue-needed flag, and write the
    message to the given socket (file).  Return a boolean
    continue_needed flag and the number of bytes written.'''

    # Set our parts in the xmlDict before forming the xml message
    xmlDict['b64Token'] = base64.encodestring(token)

    #print "  b64Token: '%(b64Token)s'" % xmlDict

    if major == gssc.GSS_S_CONTINUE_NEEDED:
        xmlDict['continue_needed'] = 'true'
    else:
        xmlDict['continue_needed'] = 'false'


    msg = msg_tmpl % xmlDict

    sockf.write(msg)
    sockf.flush()

    return xmlDict['continue_needed'] == 'true', len(msg)

def testSecContext(sockf, ctx):
    ''' Test the given security context by exchanging an encrypted message.'''

    xml_tmpl = '''<msg>%s</msg>
'''

    print "Testing message exchange:"

    xml_msg = sockf.readline()
    print "Received %s bytes" % (len(xml_msg))

    dom = Domlette.NonvalidatingReader.parseString(xml_msg, 'memory://')
    b64CyMsg = dom.firstChild.firstChild.data
    print "  base64'd crypto msg: %s" % (b64CyMsg)

    print "Decrypting message:"
    msg = ctx.unwrap(base64.decodestring(b64CyMsg))
    print "  clear msg: %s" % (msg)

    print "Generating & sending MIC:"
    mic = ctx.get_mic(msg)
    b64mic = base64.encodestring(mic).rstrip()
    print "  %s" % (b64mic)
    xml_mic = xml_tmpl % (b64mic)
    sockf.write(xml_mic)
    sockf.flush()
    print "  sent %s bytes" % (len(xml_mic))



def usage():
    useStr = """Usage: testAcceptSecContext[-u] -h <host> -p <port>"""
    print useStr


def getArgs():
    
    if len(sys.argv) == 1:
        usage()
        sys.exit(-1)
    try:
        opts, args = getopt.getopt(sys.argv[1:], "h:p:")
    except getopt.GetoptError:
        usage()
        sys.exit(-1)
    for o, a in opts:
        if o == "-u":
            usage()
            sys.exit(-1)
        elif o == "-p":
            port = a
        elif o == "-h":
            host = a
        else:
            usage()
            sys.exit(-1)

            
    return host, int(port) 


def main():
    host, port = getArgs()
    sockf, ctx = acceptContext(host,port)
    testSecContext(sockf, ctx)
    
if __name__ == "__main__": main()
