############################################################################
# Matt Rodriguez, LBNL
# Keith Beattie, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################

import base64, commands, cStringIO, getopt, httplib, sys

from pyGlobus.security import GSSCred
from pyGlobus.security import GSSContext
from pyGlobus.security import GSSName, GSSMechs, GSSUsage,GSSBuffer
from pyGlobus.security import GSSContextException
from pyGlobus import gssc
import ConfigParser

from Ft.Xml import Domlette, XPath
from xml.ns import SOAP, SCHEMA

config = ConfigParser.ConfigParser()
config.read('config.txt')


# The following dict is used when constructing xml messages, holding
# namespace definitions and other values needed in the xml.
xmlDict = {}
xmlDict['soapenv']    = SOAP.ENV
xmlDict['xsd']        = SCHEMA.XSD3
xmlDict['xsi']        = SCHEMA.XSI3
xmlDict['xmlns']      = config.get('GSISecContext','xmlns')     
xmlDict['mech-type']  = config.get('GSISecContext','mech-type')  
xmlDict['ns1']        = config.get('GSISecContext','ns1')        

# The formatted template string for initTokenExchange messages
initTokenExch_tmpl = '''<soapenv:Envelope xmlns:soapenv="%(soapenv)s"
                  xmlns:xsd="%(xsd)s"
                  xmlns:xsi="%(xsi)s">
  <soapenv:Body>
    <initTokenExchange xmlns="%(xmlns)s">
      <ns1:initContextToken ns1:continue-needed="%(continue_needed)s"
                            ns1:mechanism-type="%(mech-type)s"
                            xmlns:ns1="%(ns1)s">
        <ns1:base64Token>%(b64Token)s</ns1:base64Token>
        <ns1:context-id>%(context-id)s</ns1:context-id>
      </ns1:initContextToken>
    </initTokenExchange>
  </soapenv:Body>
</soapenv:Envelope>
'''

# The formatted template string for continueTokenExchange messages
contTokenExch_tmpl = '''<soapenv:Envelope xmlns:soapenv="%(soapenv)s"
                  xmlns:xsd="%(xsd)s"
                  xmlns:xsi="%(xsi)s">
  <soapenv:Body>
    <continueTokenExchange xmlns="%(xmlns)s">
      <ns1:contextToken ns1:continue-needed="%(continue_needed)s"
                        xmlns:ns1="%(ns1)s">
        <ns1:base64Token>%(b64Token)s</ns1:base64Token>
        <ns1:context-id>%(context-id)s</ns1:context-id>
      </ns1:contextToken>
    </continueTokenExchange>
  </soapenv:Body>
</soapenv:Envelope>
'''

def initContext(host,port):

    name = GSSName()
    mechs = GSSMechs()
    usage = GSSUsage()
    usage.set_usage_initiate()
    cred = GSSCred()
    cred.acquire_cred(name, mechs, usage)
    lifetime, cred_name = cred.inquire_cred()
    print "Cred:"
    print "      name: %s" % (cred_name.display())
    print "  lifetime: %d" % (lifetime)

    ctx = GSSContext()

    uuid = commands.getoutput("uuidgen").split("-")
    context_id = "%s-%s-%s-%s-%s" % ("0"*8, uuid[1], uuid[2], "0"*8, "0"*12)

    # initTokenExchange
    major, minor, token = ctx.init_context(cred, cred_name, "")
    print "initTokenExchange:"
    token, continue_needed = initTokenExch(host, port, major, token, context_id)

    # Now start the loop of continueTokenExchange messages.
    while continue_needed:

        major, minor, token = ctx.init_context(cred, cred_name, token)

        print "continueTokenExchange:"
        token, continue_needed = contTokenExch(host, port, major, token)
        print "  cont=%s, null token: %s" % (continue_needed, token == None)


    print "=== Context established ===\n"

    return ctx


def initTokenExch(host, port, major, token, context_id):
    ''' initTokenExch(host, port, major, token, context_id) -> token, continue_needed

    Do the send the initTokenExchange and receive the
    initTokenExchangeResponce, return the token and the value of the
    continue-needed attribute.'''
    global config
    # Set our parts in the xmlDict before forming the xml message
    xmlDict['b64Token'] = base64.encodestring(token)
    xmlDict['context-id'] = context_id

    if major == gssc.GSS_S_CONTINUE_NEEDED:
        xmlDict['continue_needed'] = 'true'
    else:
        xmlDict['continue_needed'] = 'false'

    # Fill in the formatted message template
    send_body = initTokenExch_tmpl % xmlDict

    # Construct the message and send it.
    req = httplib.HTTP(host, port)
    req.putrequest("POST", "/ogsa/services/samples/counter/soap-secure/CounterFactoryService/authService")
    req.putheader("Content-Type", 'text/plain; charset="utf-8"')
    req.putheader("Accept", "application/soap+xml, application/dime, multipart/related, text/*")
    req.putheader("User-Agent", "Axis/1.1")
    req.putheader("Cache-Control", "no-cache")
    req.putheader("Pragma", "no-cache")
    req.putheader("Host", host)
    req.putheader("SOAPAction", config.get('GSISecContext','SOAPAction') )
    req.putheader("Content-Length", str(len(send_body)))
    req.endheaders()
    req.send(send_body)

    print "  sent %s byte body" % (len(send_body))

    (status_code, message, reply_headers) = req.getreply()
    reply_body = req.getfile().read()

    print "  status: code %d, message %s. Reply body %s bytes" % (status_code, message, len(reply_body))

    # Now, parse the reply_body
    dom = Domlette.NonvalidatingReader.parseString(reply_body, 'memory://')

    ctx = XPath.Context.Context(dom, processorNss={'ns1':xmlDict['ns1']})

    # Get the continue_needed attribute
    cn_attr = XPath.Evaluate("//child::*/@ns1:continue-needed", context=ctx)[0]
    cont_needed = (cn_attr.value.lower() == 'true')

    # Get the context_id and set it for the remaining contTokenExch messages
    context_id_el = XPath.Evaluate("//ns1:context-id", context=ctx)[0]
    xmlDict['context-id'] = context_id_el.firstChild.data

    # And the token
    b64Token_el = XPath.Evaluate("//ns1:base64Token", context=ctx)[0]
    token = base64.decodestring(b64Token_el.firstChild.data)

    return token, cont_needed


def contTokenExch(host, port, major, token):
    ''' initTokenExch(host, port, major, token) -> token, continue_needed

    Send a continueTokenExchange and receive the
    continueTokenExchangeResponce, return the token and the value of
    the continue-needed attribute.'''

    global config
    # Set our parts in the xmlDict before forming the xml message
    xmlDict['b64Token'] = base64.encodestring(token)

    if major == gssc.GSS_S_CONTINUE_NEEDED:
        xmlDict['continue_needed'] = 'true'
    else:
        xmlDict['continue_needed'] = 'false'

    # Fill in the formatted message template
    send_body = contTokenExch_tmpl % xmlDict

    # Construct the message and send it.
    req = httplib.HTTP(host, port)
    req.putrequest("POST", "/ogsa/services/samples/counter/soap-secure/CounterFactoryService/authService")
    req.putheader("Content-Type", 'text/plain; charset="utf-8"')
    req.putheader("Accept", "application/soap+xml, application/dime, multipart/related, text/*")
    req.putheader("User-Agent", "Axis/1.1")
    req.putheader("Cache-Control", "no-cache")
    req.putheader("Pragma", "no-cache")
    req.putheader("Host", host)
    req.putheader("SOAPAction", config.get('GSISecContext','SOAPAction'))
    req.putheader("Content-Length", str(len(send_body)))
    req.endheaders()
    req.send(send_body)

    print "  sent %s byte body" % (len(send_body))

    (status_code, message, reply_headers) = req.getreply()
    reply_body = req.getfile().read()

    print "  status: code %d, message %s. Reply body %s bytes" % (status_code, message, len(reply_body))

    # Now, parse the reply_body
    dom = Domlette.NonvalidatingReader.parseString(reply_body, 'memory://')

    ctx = XPath.Context.Context(dom, processorNss={'ns1':xmlDict['ns1']})

    # Get the continue_needed attribute
    cn_attr = XPath.Evaluate("//child::*/@ns1:continue-needed", context=ctx)[0]
    cont_needed = (cn_attr.value.lower() == 'true')

    # And the token
    b64Token_el = XPath.Evaluate("//ns1:base64Token", context=ctx)[0]
    if b64Token_el.firstChild:
        token = base64.decodestring(b64Token_el.firstChild.data)
    else:
        token = None

    return token, cont_needed


def testSecContext(sockf, ctx):
    ''' Test the given security context by exchanging an encrypted message.'''

    xml_tmpl = '''<msg>%s</msg>
'''

    print "Testing message exchange:"

    msg = "The bodies are in the trunk"
    print "  msg: %s" % (msg)

    cyMsg = ctx.wrap(msg)
    b64CyMsg = base64.encodestring(cyMsg).rstrip()
    print "  base64'd crypto msg: %s" % (b64CyMsg)
    
    xml_msg = xml_tmpl % (b64CyMsg)
    sockf.write(xml_msg)
    sockf.flush()
    print "Sent %s bytes" % (len(xml_msg))

    print "Reading mic:"
    xml_mic = sockf.readline()
    print "  read %s bytes" % (len(xml_mic))
    dom = Domlette.NonvalidatingReader.parseString(xml_mic, 'memory://')
    b64mic = dom.firstChild.firstChild.data
    print "  base64' mic: %s" % (b64mic)

    try:
        verified = ctx.verify_mic(msg, base64.decodestring(b64mic))
    except GSSContextException, ex:
        print "Problem verifing mic"
        raise ex

    if verified:
        print "Message verification succeded"
    else:
        print "Message verification failed"



def usage():
    useStr = """Usage: testInitSecContext_xml.py [-u] -p <port> -h <host>"""
    print useStr


def getArgs():
    
    if len(sys.argv) == 1:
        usage()
        sys.exit(-1)
    try:
        opts, args = getopt.getopt(sys.argv[1:], "h:p:")
    except getopt.GetoptError:
        usage()
        sys.exit(-1)
    for o, a in opts:
        if o == "-u":
            usage()
            sys.exit(-1)
        elif o == "-p":
            port = a
        elif o == "-h":
            host = a
        else:
            usage()
            sys.exit(-1)

            
    return host, int(port) 


def main():
    global config
    if config == None:
        config = ConfigParser.ConfigParser()
        config.read('config.txt')

    host, port = getArgs()
    ctx = initContext(host,port)
    #testSecContext(ctx)

if __name__ == "__main__": main()
