############################################################################
# Matt Rodriguez, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################

from pyGlobus.security import GSSCred
from pyGlobus.security import GSSContext,ContextRequests
from pyGlobus.security import GSSName, GSSMechs, GSSUsage,GSSBuffer
from pyGlobus.security import GSSContextException
from pyGlobus import gssc
import socket,sys,getopt,base64, time

def readAll(sock):
    print "in readAll"
    recvString =""
    bytesRead = 0
    header = sock.recv(5)
    print "header:", header
    t = header.split('X')
    numBytes = header.split('X')[0]
    
    totalBytes = int(numBytes)
    while bytesRead < int(numBytes):
        recvString += sock.recv(totalBytes)
        bytesRead += len(recvString)
        
    return recvString

def sendAll(sock, sendString):
    print "in sendAll"
    stringLength = len(sendString)
    if stringLength == 0:
        return
    if stringLength > 99999:
        raise RuntimeError("sendAll can't send strings longer than 99,999 characters")
    xLength = 5 - len(str(stringLength))
    xString = ""
    for i in range(xLength):
        xString +='X'

    header = str(stringLength) + xString
    print "header is: ", header
    newSendString = header + sendString
    sock.sendall(newSendString)
    
def initContext(host,port):
    print "host is ", host
    print "port is ", port
    name = GSSName()
    mechs = GSSMechs()
    usage = GSSUsage()
    usage.set_usage_initiate()
    cred = GSSCred()
    cred.acquire_cred(name, mechs, usage)
    lifetime, cred_name = cred.inquire_cred()
    print "Cred info:  lifetime: " + str(lifetime) + "\nName: ", cred_name.display()

    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM, 0)
    s.connect((host,port))

    ctx = GSSContext()
    context_established = 0
    looped = 1
    input_token=""
    tokensSent = 0
    tokensRecv = 0
    while not context_established:
        requests = ContextRequests()
	#requests.set_delegation() 
        major, minor,tokens = ctx.init_context(cred,cred_name,input_token,requests)
   
        if  tokens != None:
            print "sending len(ctx_blob_send): ", len(tokens)
            ctx_blob_send =  tokens
            sendAll(s, ctx_blob_send)
            tokensSent += len(ctx_blob_send)
            
        if major == gssc.GSS_S_CONTINUE_NEEDED:
            ctx_blob_recv = readAll(s)
            print "len(ctx_blob_recv) ", len(ctx_blob_recv)
            tokensRecv += len(ctx_blob_recv)
            if len(ctx_blob_recv) == 0:
                print "Nothing received from the socket, exiting"
                sys.exit(1)
        else:
            context_established = 1

        if ctx_blob_recv != None:
            input_token = ctx_blob_recv
        print "Going around the loop: ", looped
        looped = looped + 1

    print "tokensSent: ", tokensSent
    print "tokensRecv: ", tokensRecv
    
    print "Context created"
    t = ctx.inquire()
    print t
    msg = "OZOozoOZO"
    print "Calling gss_wrap"
    print "msg: ",msg
    wrappedOut = ctx.wrap(msg)

    sendAll(s, wrappedOut)
    
    micRecv = readAll(s)
    print "received micString:", micRecv
    try:
        verified = ctx.verify_mic(msg, micRecv)
    except GSSContextException, ex:
        print "Problem verifing mic"
        raise ex
    if verified:
        print "Message verification succeded"
    else:
        print "Message verification failed"
def usage():
    useStr = """Usage: test_InitSecContext [-u] -p <port> -h <host>"""
    print useStr


def getArgs():
    
    if len(sys.argv) == 1:
        usage()
        sys.exit(-1)
    try:
        opts, args = getopt.getopt(sys.argv[1:], "h:p:")
    except getopt.GetoptError:
        usage()
        sys.exit(-1)
    for o, a in opts:
        if o == "-u":
            usage()
            sys.exit(-1)
        elif o == "-p":
            port = a
            print port
        elif o == "-h":
            host = a
            print host
        else:
            usage()
            sys.exit(-1)

            
    return host, int(port) 

host,port = getArgs()
initContext(host,port)
