############################################################################
# Keith R. Jackson, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################

import sys, getopt
from time import sleep

#from pyGlobus import ioc

from pyGlobus import utilc, ioc
from pyGlobus.io import GSITCPSocket, NetIOAttr, AuthData, TCPIOAttr
from pyGlobus.util import Buffer, GlobusException


def usage():
    useStr = "test_tcp_client -h host -p port [-d] to perform delegation"
    print useStr
    return

def parseArgs():
    if len(sys.argv) < 4:
        usage()
        sys.exit(-42)
    try:
        opts, args = getopt.getopt(sys.argv[1:], "dh:p:")
    except getopt.GetoptError:
        usage()
        sys.exit(-1)
    delegate = 0
    for o, a in opts:
        if o == "-h":
            host = a
        elif o == "-p":
            port = int(a)
        elif o == "-d":
            delegate = 1
        else:
            usage()
            sys.exit(-1)
    return host, port, delegate

def main():
    host, port, delegate = parseArgs()
    try:
        attr = TCPIOAttr()
        print "Init'd the attr"
        attr.set_authentication_mode(ioc.GLOBUS_IO_SECURE_AUTHENTICATION_MODE_GSSAPI)
        print "set the authentication mode"
        authData = AuthData()
        attr.set_authorization_mode(ioc.GLOBUS_IO_SECURE_AUTHORIZATION_MODE_SELF, authData)
        print "set the auth mode to SELF"
        attr.set_delegation_mode(ioc.GLOBUS_IO_SECURE_DELEGATION_MODE_FULL_PROXY)
        print "set the delegation mode to limited proxy"
        attr.set_channel_mode(ioc.GLOBUS_IO_SECURE_CHANNEL_MODE_CLEAR)
        print "set channel mode"
        soc = GSITCPSocket()
        print "got a socket object"
        soc.connect(host, port, attr)
        print "connected to host %s, on port %i" % (host, port)
        str = "Foobar, fOobar, foObar, fooBar, foobAr, foobaR"
        nBytes = soc.write(str, len(str))
        print "Wrote %i bytes out" % nBytes
        size = 18
        buf = Buffer(size)
        print "created a buffer"
        bytesRead = soc.read(buf, size, size)
        print "Read %i bytes" % bytesRead
        print "buf is: %s" % buf.as_string(bytesRead)
        if delegate:
            print 'As of **Thu Sep 12 14:22:44 PDT 2002**'
            print '\tdelegation is available only at connection setup.'
            '''
            print "Accepting delegation"
            cred = soc.accept_delegation([], [], 0)
            print "Accepted delegation"
            print "Exporting cred"
            print cred._handle
            where = cred.export_external()
            print "Exported to %s" % where
            '''
    except GlobusException, ex:
        print "caught an exception"
        print ex.args
        sys.exit(-1)
    return

if __name__ == '__main__':
    main()
    

    
