############################################################################
# Keith R. Jackson, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################

import sys, getopt
from pyGlobus import ioc
from pyGlobus.io import GSITCPSocket, NetIOAttr, AuthData, TCPIOAttr
from pyGlobus.util import Buffer, GlobusException

def auth_callback(arg, handle, identity, context):
    print "inside auth callback"
    print "id is: %s" % identity
    return 1
 
def usage():
    useStr = "test_tcp_server [-d] to perform delegation"
    print useStr
    return

def parseArgs():
    if len(sys.argv) > 2:
        usage()
        sys.exit(-42)
    try:
        opts, args = getopt.getopt(sys.argv[1:], "d")
    except getopt.GetoptError:
        usage()
        sys.exit(-1)
    delegate = 0
    for o, a in opts:
        if o == "-d":
            delegate = 1
        else:
            usage()
            sys.exit(-1)
    return delegate

def main():
    delegate = parseArgs()
    try:
        attr = TCPIOAttr()
        print "Init'd the attr"
        attr.set_authentication_mode(ioc.GLOBUS_IO_SECURE_AUTHENTICATION_MODE_GSSAPI)
        print "set the authentication mode"
        authData = AuthData()
        authData.set_callback(auth_callback, None)
        print "Set the auth callback"
        attr.set_authorization_mode(ioc.GLOBUS_IO_SECURE_AUTHORIZATION_MODE_CALLBACK, authData)
        print "set the auth mode to CALLBACK"
        attr.set_delegation_mode(ioc.GLOBUS_IO_SECURE_DELEGATION_MODE_LIMITED_PROXY)
        print "set the delegation mode to limited proxy"
        #attr.set_restrict_port(1)
        #print "set restrict port"
        attr.set_channel_mode(ioc.GLOBUS_IO_SECURE_CHANNEL_MODE_CLEAR)
        print "set channel mode"
        soc = GSITCPSocket()
        print "got a socket object"
        port = soc.create_listener(attr)
        print "created a listener on port %s" % port
        soc.listen()
        print "returned from listen"
        (childSoc, addr) = soc.accept(attr)
        print "accepted a connection"
        ctx = childSoc.get_security_context()
        print "got the security context"
        tuple = ctx.inquire()
        print "finished inquire_context"
        name = tuple[0].display()
        print "Source name is: %s" % name
        cred = childSoc.get_delegated_credential()
        print "Got the delegated credential"
        size = 46
        buf = Buffer(size)
        print "created a buffer"
        bytesRead = childSoc.read(buf, size, size)
        print "Read %i bytes" % bytesRead
        print "buf is: %s" % buf.as_string(bytesRead)
        print "Writing back buffer"
        str = "Fee, Fie, Foo, Fum"
        childSoc.write(str, len(str))
        if delegate:
            print "Delegating"
            childSoc.init_delegation(cred, [], [], 0)
        print "exiting"
        del cred
        print "deleted cred"
        del childSoc
        print "deleted childSoc"
        del soc
        print "deleted soc"
    except GlobusException, ex:
        print "caught an exception"
        print ex.args
        sys.exit(-1)
    return
    
if __name__ == '__main__':
    main()

    
