############################################################################
# Matt Rodriguez, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################

from pyGlobus.ftpClient import EasyFtpClient
from pyGlobus.ftpClientAttr   import OperationAttr
from pyGlobus.ftpControl import Parallelism
from pyGlobus.ftpControl import TcpBuffer
from pyGlobus.ftpControl import PARALLELISM_FIXED,MODE_EXTENDED_BLOCK
import unittest,ConfigParser
from threading import *
from exceptions import Exception
import time

config = None
config_file = 'config.txt'


class EasyFtpClientTestCaseOne(unittest.TestCase):
    def setUp(self):
        self._ftpC = EasyFtpClient()
        global config
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read(config_file)
        return
    
    def tearDown(self):
        return
        
    def checkMkdir(self):
        print "Unittest for mkdir" 
        dir = config.get("ftpClient", "ftpDir") 
        self._ftpC.mkdir(dir)
        print "DONE"
        
    def checkSize(self):
        print "Unittest for size"
        url = config.get("ftpClient", "srcUrl") 
        size = self._ftpC.size(url)
        print "Size of url", url , "is ", size
        print "DONE"
        
    def checkMTime(self):
        print "Unittest for modification time"
        url = config.get("ftpClient", "srcUrl")
        mt = self._ftpC.modification_time(url)
        seconds = mt.getSecs()
        timeStr = time.ctime(seconds)
        print "Modification time of ", url , " is ", timeStr
        print "in Seconds ", seconds
        print "DONE"

    def checkExists(self):
        print "Unittests for file existence"
        url = config.get("ftpClient", "badUrl") 
        exist = self._ftpC.exists(url)
        print "Exists: ", exist
        print "DONE"

        
class EasyFtpClientTestCasePuts(unittest.TestCase):
    def setUp(self):
        self._ftpC = EasyFtpClient()
        global config
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read(config_file)
        return
    
    def tearDown(self):
        del self._ftpC
        return

    def checkEasyPut(self):
        print "Unittest for easy_put"
        src = config.get("ftpClient", "src")
        url = config.get("ftpClient", "url")
        
        print "src file is: ",src
        print "Destination url is: ", url
        
        try:
            self._ftpC.put(src, url)
        except Exception, ex:
            raise
        print "DONE"

    def checkEasyPartialPut(self):
        print "Unittest for easy partial put"
        src = config.get("ftpClient", "src")
        url = config.get("ftpClient", "url") 
        url = url + '_partial' 
        
        try:
            self._ftpC.partial_put(src, url, 0, 2)
        except Exception, ex:
            raise
        print "DONE"

    def checkParallelPut(self):
        print "Unittest for parallel put"
        src = config.get("ftpClient", "src")
        url = config.get("ftpClient", "url")
        streams = config.get("ftpClient", "streams") 
        opAttr = OperationAttr()
        opAttr.set_mode(MODE_EXTENDED_BLOCK)
        pattr = Parallelism()
        pattr.set_mode(PARALLELISM_FIXED)
        pattr.set_size(int(streams))
        opAttr.set_parallelism(pattr)
        self._ftpC.put(src,url,opAttr)
        print "DONE"

class EasyFtpClientTestCaseGets(unittest.TestCase):
    def setUp(self):
        self._ftpC = EasyFtpClient()
        global config
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read(config_file)
        return
    
    def tearDown(self):
        return

    def checkEasyGet(self):
        print "Unittest for easy_get"
        dest = config.get("ftpClient", "dest")
        url = config.get("ftpClient", "srcUrl") 
        self._ftpC.get(dest, url)
        print "DONE"

    def checkParallelGet(self):
        print "Unittest for parallel get"
        dest = config.get("ftpClient", "dest")
        url = config.get("ftpClient", "srcUrl")
        streams = config.get("ftpClient", "streams") 
        opAttr = OperationAttr()
        opAttr.set_mode(MODE_EXTENDED_BLOCK)
        pattr = Parallelism()
        pattr.set_mode(PARALLELISM_FIXED)
        pattr.set_size(int(streams))
        opAttr.set_parallelism(pattr)
        self._ftpC.get(dest,url,opAttr)
        print "DONE"
        
class EasyFtpClientTestCaseTwo(unittest.TestCase):
    def setUp(self):
        self._ftpC = EasyFtpClient()
        global config
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read(config_file)
        return
    
    def tearDown(self):
        return

    def checkTPT(self):
        print "Unittest for third_party_transfer"
        srcUrl = config.get("ftpClient", "srcUrl")
        destUrl = config.get("ftpClient", "destUrl")
        destUrl += '_easy_matt' 
        self._ftpC.third_party_transfer(srcUrl, destUrl)
        print "DONE"

    def checkMove(self):
        print "Unittest for move"
        srcUrl = config.get("ftpClient", "url") 
        destUrl = config.get("ftpClient", "fileName") 
        self._ftpC.move(srcUrl, destUrl)
        print "DONE"
        
    def checkEasyList(self):
        print "Unittest for  list"
        url = config.get("ftpClient", "ftpList") 
        l = self._ftpC.list(url)
        print l
        print "DONE"
        
    def checkEasyVerboseList(self):
        print "Unittest for  verbose_list"
        url = config.get("ftpClient", "ftpList") 
        l = self._ftpC.verbose_list(url)
        print l
        print "DONE"

    
        
class EasyFtpClientTestCaseThree(unittest.TestCase):
    def setUp(self):
        self._ftpC = EasyFtpClient()
        global config
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read(config_file)
        return
    
    def tearDown(self):
        return

    def checkDelete(self):
        print "Unittest for delete" 
        fileName = config.get("ftpClient", "fileName") 
        self._ftpC.delete(fileName)
        print "DONE"

class EasyFtpClientTestCaseFour(unittest.TestCase):
    def setUp(self):
        self._ftpC = EasyFtpClient()
        global config
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read(config_file)
        return
    
    def tearDown(self):
        return

    def checkRmDir(self):
        print "Unittest for rmdir" 
        dir = config.get("ftpClient", "ftpDir") 
        self._ftpC.rmdir(dir)
        print "DONE"
        
    

def makeTestSuite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(EasyFtpClientTestCasePuts, "check"))
    suite.addTest(unittest.makeSuite(EasyFtpClientTestCaseGets, "check"))
    suite.addTest(unittest.makeSuite(EasyFtpClientTestCaseOne, "check"))
    suite.addTest(unittest.makeSuite(EasyFtpClientTestCaseTwo, "check"))
    suite.addTest(unittest.makeSuite(EasyFtpClientTestCaseThree, "check"))
    suite.addTest(unittest.makeSuite(EasyFtpClientTestCaseFour, "check"))
    return suite


def main():
    unittest.main(defaultTest="makeTestSuite")


if __name__ == "__main__" : main()
   
