############################################################################
# Matt Rodriguez, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################
from pyGlobus.ftpClientPlugin import PerformanceMarkerPlugin
from pyGlobus.ftpClientPlugin import DebugPlugin
from pyGlobus.ftpClientPlugin import RestartMarkerPlugin
from pyGlobus.ftpClientPlugin import RestartMarker
from pyGlobus.ftpClientPlugin import ThroughputPerfPlugin
from pyGlobus.ftpClient import EasyFtpClient
from pyGlobus.ftpClientAttr   import HandleAttr
from pyGlobus.ftpClientAttr   import OperationAttr
from pyGlobus.util import GlobusException
from pyGlobus import ftpControl
from pyGlobus import ftpClientc
import unittest,ConfigParser, sys
import time
from threading import *



config_file = 'config.txt'
config = None # ConfigParser object

   
class FtpPluginRestartMarkerTestCase(unittest.TestCase):
    "Test case for the FtpPlugin Module"
    def setUp(self):
        global config
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read(config_file)
            
        self.__marker_str = ''
        return
    
    
    def tearDown(self): 
        try:
            del self.__ftpClient
        except GlobusException, ex:
            print ex.args
            raise ex 
        return 


    def __beginCB(self, arg, handle, srcUrl, destUrl, marker):
        ret = 0
        print "inside the beginCB,good"
        if self.__marker_str:
            print "\tRestart Marker String: ", self.__marker_str
            ftpClientc.restart_marker_from_string(marker, self.__marker_str)
            print "\tout"
        ret = 1 
        return ret

    def __completeCB(self, arg, handle, err, url):
        print "inside the complete CB,good"
        if not err[0] == 0:
            print "got the following error: ", err
  
        return 
    
    def __marker_abort_CB(self, arg, handle, marker):
        print "inside the marker_abort_CB"
        ret, err, self.__marker_str = ftpClientc.restart_marker_to_string(marker)
        
        try:
            self.__ftpClient.abort()
        except Exception:
            print "Failed Abort"
        else:
            print "\tabort"
      
        print "\trestart marker string is:", self.__marker_str
        return
    
    def __markerCB(self, arg, handle, marker):
        print "inside the markerCB"
        ret, err, self.__marker_str = ftpClientc.restart_marker_to_string(marker)
        print "restart marker string is:", self.__marker_str
        return
    
    def checkRestartMarkerPlugin(self, offset=0L, endOffset=100L):
        """
        The problem with these RestartMarkerPlugin tests is that the marker 
        callbacks are not being called. They are not being called because
        markers are not being sent back. You probably need a really big file 
        to have the server send back marker responses. So not all aspects of
        the RestartMarkerPlugin are being tested. 
        """
        print "Checking restart marker plugin "   
        destUrl = config.get("ftpClient", "destUrl")
        srcUrl = config.get("ftpClient", "srcUrl")
        print "Transfering file from %s to %s" % (srcUrl, destUrl) 
        
        _markerCB_func = self.__marker_abort_CB
        
        try:
            handleAttr = HandleAttr()
            plugin = RestartMarkerPlugin(self.__beginCB,_markerCB_func,
                                        self.__completeCB, None)
            handleAttr.add_plugin(plugin)
            self.__ftpClient = EasyFtpClient(handleAttr)
            opAttr = OperationAttr()
            opAttr.set_mode(ftpControl.MODE_EXTENDED_BLOCK) 
            self.__ftpClient.third_party_transfer(srcUrl,destUrl,srcAttr=opAttr,destAttr=opAttr)
                                                      
        except GlobusException, ex:
            print "globus exception"
            print ex
            print "TEST FAILED" 
        print "DONE"
        return

    def checkPartialThirdPartyTransfer(self, offset=0L, endOffset=10L):
        print "Checking partial third party transfers with RestartMarkerPlugin"
        

        srcUrl = config.get("ftpClient", "srcUrl")
        destUrl = config.get("ftpClient", "destUrl")
        destUrl = destUrl + '_partial1'
        
        srcAttr = OperationAttr()
        srcAttr.set_mode(ftpControl.MODE_EXTENDED_BLOCK)
        dstAttr = OperationAttr()
        dstAttr.set_mode(ftpControl.MODE_EXTENDED_BLOCK) 
        marker  = RestartMarker()
        r_plugin = RestartMarkerPlugin(self.__beginCB, self.__markerCB,
                                     self.__completeCB, None)
        
        print "Starting 3 partial third party transfers from \n\tsrc(%s) dst(%s)" % (srcUrl, destUrl)
        for i in range(3):
            
            try: 
                handleAttr = HandleAttr()
                handleAttr.add_plugin(r_plugin)
                self.__ftpClient = EasyFtpClient(handleAttr) 
                cb = self.__ftpClient.partial_third_party_transfer(srcUrl,destUrl,
                                                                   offset, endOffset,
                                                                   srcAttr, dstAttr,
                                                                   marker)
                                                                  
            except GlobusException, ex:
                print "globus exception"
                print ex.args
                print "TEST FAILED"
                break
            else:
                offset = endOffset
                endOffset = offset + 1
            
        print "DONE"



class FtpPluginDebugTestCase(unittest.TestCase):
    def setUp(self):
        global config
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read(config_file) 
        self.__ftpC = None
        return
    
    
    def tearDown(self): 
        try:
            del self.__ftpClient
        except GlobusException, ex:
            print ex.args 
        
        return 

    

    def checkDebugPlugin(self):
        print "testing the Debug Plugin"
        debug_file_cfg = config.get("ftpClient", "debugFile")
        if debug_file_cfg == '1':
            debug_file = sys.stdout
            print "Sending debuging information to stdout"
        elif debug_file_cfg == '2':
            debug_file = sys.stderr
            print "Sending debugging information to stderr"
        else:
            print "Sending debugging information to ", debug_file_cfg
            debug_file = open(debug_file_cfg, "w")
        plugin = DebugPlugin(debug_file, "Running Unittest")
        handleAttr = HandleAttr()
        handleAttr.add_plugin(plugin)
        self.__ftpClient = EasyFtpClient(handleAttr)
        src = config.get("ftpClient", "src")
        url = config.get("ftpClient", "url")
        try:
            self.__ftpClient.put(src, url)
            
            
        except GlobusException, ex:
            print ex.args
            print "TEST FAILED" 

        print "DONE"
        
class FtpPluginPerformanceTestCase(unittest.TestCase):
    def setUp(self):
        global config
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read(config_file)
            
        self.__ftpClient = None
        return
    
    
    def tearDown(self): 
        try:
            del self.__ftpClient
        except GlobusException, ex:
            print ex.args 
        return 

    def __beginCB(self, arg, handle, srcUrl, destUrl, restart): 
        if self._url != destUrl:
            print "TEST FAILED" 
        return
    
   
    def __markerCB(self, arg, handle, time_stamp, time_stamp_length, stripe_ndx, num_stripes, nbytes):
        print "inside markerCB, printing timestamp information" 
        print "time_stamp: ", time.ctime(time_stamp) 
        
    def __completeCB(self, arg, handle, error):
        print "inside the complete CB, good" 
        return
    

    def checkPerformancePlugin(self):
        print "testing the  Performance Plugin"
        plugin = PerformanceMarkerPlugin(self.__beginCB, self.__markerCB,
                                      self.__completeCB, 0)
        handleAttr = HandleAttr()
        handleAttr.add_plugin(plugin)
        self.__ftpClient = EasyFtpClient(handleAttr)
        src = config.get("ftpClient", "src")
        url = config.get("ftpClient", "url")
        self._url = url
        try:
            
            opAttr = OperationAttr()
            opAttr.set_mode(ftpControl.MODE_EXTENDED_BLOCK)
            self.__ftpClient.put(src, url, opAttr)

        except GlobusException, ex:
            print ex.args
            print "TEST FAILED" 
            
        print "DONE"
        
class FtpPluginThroughPutTestCase(unittest.TestCase):
    """ Class that tests the ftpClientPlugin.ThroughputPerfPlugin class"""
    
    def setUp(self):
        global config
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read(config_file)
            
        self.__ftpClient = None
        return
    
    
    def tearDown(self):
        print "in tearDown" 
        try:
            del self.__ftpClient
        except GlobusException, ex:
            print ex.args

        print "leaving tearDown"
        return 

    def __beginCB(self, arg, handle, srcUrl, destUrl):
        if self._url != destUrl:
            print "TEST FAILED" 
        return 
        
    
    def __stripeCB(self, arg, handle, stripe_ndx, bytes, through_put, avg_through_put):
        print "Inside stripeCB, print average throughput" 
        print "average throughput:", avg_through_put

        
    def __totalCB(self, arg, handle,bytes, through_put, avg_through_put):
        print "Inside totalCB, printing throughput" 
        print "throughput: ", through_put
        
       
    def __completeCB(self, arg, handle, error):
        print "Transfer has completed" 
        if error[0] is not 0:
            print error
        return
    

    def checkThroughputPerfPlugin(self):
        print "testing the  ThroughPut Plugin"
        plugin = ThroughputPerfPlugin(self.__beginCB, self.__stripeCB,self.__totalCB, self.__completeCB, 0)
        handleAttr = HandleAttr()
        handleAttr.add_plugin(plugin)
        self.__ftpClient = EasyFtpClient(handleAttr)
        src = config.get("ftpClient", "src")
        url = config.get("ftpClient", "url")
        print "Transfering a file from %s to %s" % (src,url) 
        self._url = url 
        self.__file = open(src, "r")
        self.__condV = Condition(Lock())
        try:
            self.__ftpClient.put(src, url)
            
        except GlobusException, ex:
            print ex.args
            print "TEST FAILED" 

        print "DONE"
        
def makeTestSuite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(FtpPluginRestartMarkerTestCase, "check"))
    suite.addTest(unittest.makeSuite(FtpPluginDebugTestCase, "check"))
    suite.addTest(unittest.makeSuite(FtpPluginPerformanceTestCase, "check"))
    suite.addTest(unittest.makeSuite(FtpPluginThroughPutTestCase, "check"))
    return suite

def main():
    unittest.main(defaultTest="makeTestSuite")


if __name__ == "__main__" : main()
