############################################################################
# Matt Rodriguez, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################

from pyGlobus.util import GlobusException
from pyGlobus.ftpControl import FtpControl
from pyGlobus.ftpControl import Parallelism
from pyGlobus.ftpControl import HostPortArray
from pyGlobus.ftpControl import HostPort
from pyGlobus.ftpControl import TcpBuffer
from pyGlobus.ftpControl import DCAU
from pyGlobus.ftpControl import Response
from pyGlobus.security import GSSCred
import unittest,ConfigParser
from pyGlobus import ftpControl
from socket import gethostname,gethostbyname
from pyGlobus.ftpControl import AuthInfo
config = None
config_file = 'config.txt'
from threading import *
g_cond = 0

def closeCB(cv, handle, error, response):
    global g_cond
    if error[0] != 0:
        print "ERROR in callback"
        print error
        print "TEST FAILED" 
    cv.acquire()
    g_cond = 1
    cv.notify()
    cv.release()
    
     
    
def connectCB(cv, handle, stripe_ndx, reuse, error):
    global g_cond
    print "in connectCB"
    print "error: ", error
    print "stripe_ndex:", stripe_ndx
    
    print "reuse: ", reuse 
    
    print "leaving connnectCb"

def quitCB(cv, handle, stripe_ndx, reuse, error):
    global g_cond 
    print "in quitCb"
    cv.acquire()
    g_cond = 0
    cv.notify()
    cv.release()
    print "leaving quitCb"
    
def controlConnectCB(cv, handle, error, response):
    global g_cond 
    if error[0] != 0:
        print "ERROR in callback"
        print error
        print "TEST FAILED" 
    cv.acquire()
    g_cond = 1
    cv.notify()
    cv.release()
    

def authCB(cv, handle,error, response):
    global g_cond
    if error[0] != 0:
        print "ERROR in callback"
        print error
        print "TEST FAILED" 
    
    pyResponse = Response(response)
    info = pyResponse.info() 
    cv.acquire()
    g_cond = 1
    cv.notify()
    cv.release()
    
class FtpControlTest__basic(unittest.TestCase):
    def setUp(self):
	global config
	if config == None:
	    config = ConfigParser.ConfigParser()
	    config.read(config_file)
	self._ftpcontrol = FtpControl()
        self._host = config.get("ftpControl" , "host")
	
	
    def check_local_send_eof(self):
	print "Testing, setting local send eof"
	eof = 1
	self._ftpcontrol.local_send_eof(eof)
        print "DONE"

    def check_local_parallelism(self):
	print "Testing, setting parallelism information in the ftpControl handle"
	parallel = Parallelism()
	md = ftpControl.PARALLELISM_FIXED
	parallel.set_mode(md)
        self._ftpcontrol.local_parallelism(parallel)
        print "DONE"
    
    def check_local_mode(self):
	"""There is a pesky valgrind UMR bug to track down in this method MKR """
	print "Testing setting the local mode"
        #only STREAM and EXTENDED_MODE are supported for this call in the current
        #implementation
        mode = 'S'
        self._ftpcontrol.local_mode(mode)
        print "DONE"
	
    def check_local_tcp_buffer(self):
	print "Testing setting the tcp buffer size"
        tcpBuf = TcpBuffer()
        size = 100
        tcpBuf.set_fixed(size)
        self._ftpcontrol.local_tcp_buffer(tcpBuf)
        print "DONE"

    def check_local_dcau(self):
        print "Testing setting data channel authentication mode"	
	dcau = DCAU()
	mode = ftpControl.DCAU_SELF
	dcau.set_mode(mode)
	cred = GSSCred()
	self._ftpcontrol.local_dcau(dcau, cred)
        print "DONE"

    def check_local_pbsz(self):
	print "Testing setting the protection buffer size"
        size = 100
        self._ftpcontrol.local_pbsz(size)
        print "DONE" 
	
    def check_local_spas(self):
        print "Testing setting striped passive "
        size = 10
        hp_array = HostPortArray(size)
	try:
	    self._ftpcontrol.local_spas(hp_array)
        except GlobusException, ex:
            if str(ex).count( "this function is not implemented") <= 0:
                print "TEST FAILED" 
            else:
                print "TEST PASSED" 
	
    def check_local_spor(self):
	print "Testinng setting striped port"
        size = 10
        hp_array = HostPortArray(size)
	try:
	    hplisteners = self._ftpcontrol.local_spas(hp_array) 
	    self._ftpcontrol.local_spor(hplisteners)
             
	except GlobusException, ex:
            if str(ex).count( "this function is not implemented") <= 0:
                print "TEST FAILED" 
            else:
                print "TEST PASSED" 
	

    def implement_chek_data_connect(self):
	print "check_data_connect"
                
    def implement_check_data_write(self):
	print "check_data_write"

    def check_local_pasv(self): 
	global g_cond
        print "Testing, setting local passive and forcing close" 
        try:
            port = 2811
            cv = Condition(Lock())
            cv.acquire()
            g_cond = 0
            self._ftpcontrol.control_connect(self._host, port, controlConnectCB, cv)
            while g_cond == 0:
                cv.wait()
            cv.release()
            
            hostPort = self._ftpcontrol.local_pasv()
            t =  hostPort.get_info() 
            cv.acquire()
            g_cond = 0
            self._ftpcontrol.force_close(closeCB, cv)
            while g_cond == 0:
                cv.wait()
            cv.release() 

        except GlobusException, ex:
            print "Exception occured during testing"
            print ex
            print "TEST FAILED" 
      
        else:
            print "TEST PASSED"
        
        
    def check_local_port(self): 
        global g_cond
        print "Testing local port and forcing close"
        try: 
            host = gethostbyname(gethostname())
            newL = []
            hostList = host.split('.')
            for s in hostList:
                newL.append(int(s))
            hostPort = HostPort(newL, 7575) 
            port = 2811 
            cv = Condition(Lock())
            cv.acquire()
            g_cond = 0
            self._ftpcontrol.control_connect(self._host, port, controlConnectCB, cv)
            while g_cond == 0:
                cv.wait()
            cv.release()
            self._ftpcontrol.local_port(hostPort)
            cv.acquire()
            g_cond = 0
            self._ftpcontrol.force_close(closeCB, cv)
            while g_cond == 0:
                cv.wait()
            cv.release() 
        except GlobusException, ex: 
            print "Exception occured during testing"
            print ex
            print "TEST FAILED" 
        else:
            print "TEST PASSED"
            

    def check_local_type(self):
        print "Testing setting local type" 
        self._ftpcontrol.local_type('I')
        print "DONE"
        
    def implement_check_read_commands(self):
	print "check_read_commands"

    def implement_check_send_response(self):
	print "check_send_response"

    def implement_check_raw_command(self):
	print "check_raw_command"
    
    def implement_check_code(self):
	print "check_code"

    def implement_check_operation(self):
	print "check_operation"

    def implement_check_create_data_info(self):
	print "check_create_data_info"

    def implement_check_release_data_info(self):
	print "check_release_data_info"
    
    def implement_check_data_write_stripe(self):
	print "check_data_write_stripe"

    def implement_check_x_data_write_stripe(self):
	print "check_x_data_write_stripe"

    def check_authenticate(self):
	print "Testing connecting, authenticating, then force closing the connnection"
        global g_cond
        try:
            cv = Condition(Lock())
            host = self._host
            port = 2811
            g_cond = 0
            cv.acquire()
       	    self._ftpcontrol.control_connect(host, port, controlConnectCB, cv)
            while g_cond == 0:
                cv.wait()
            cv.release()
            cred = GSSCred()
            authInfo = AuthInfo(cred, True)
            g_cond = 0
            cv.acquire()
            self._ftpcontrol.authenticate(authInfo, True, authCB, cv)
            while g_cond == 0:
                cv.wait()
            cv.release()
            cv.acquire()
            g_cond = 0
            self._ftpcontrol.force_close(closeCB, cv)
            while g_cond == 0:
                cv.wait()
            cv.release() 
        except GlobusException, ex:
            print "Exception occured during testing"
            print ex
            print "TEST FAILED"
    
        print "DONE"

    def implement_check_send_command(self):
	print "check_send_command"

    def implement_check_abort(self):
	print "check_abort"
	
    def check_quit(self):
        pass
    
    def check_force_close(self):
        print "Testing connecting, then force closing the connnection"
        global g_cond
        try:
            cv = Condition(Lock())
            host = self._host
            port = 2811
            g_cond = 0
            cv.acquire()
            self._ftpcontrol.control_connect(host, port, controlConnectCB, cv)
            while g_cond == 0:
                cv.wait()
            cv.release()
            cv.acquire()
            g_cond = 0
            self._ftpcontrol.force_close(closeCB, cv)
            while g_cond == 0:
                cv.wait()
            cv.release() 
        except GlobusException, ex:
            print "Exception occured during testing"
            print ex
            print "TEST FAILED"
        print "DONE" 
    
    def implement_check_data_connect_read(self):
        print "check_data_connect_read"
        
    def implement_check_data_add_channels(self):
	print "check_data_add_channels"

    def implement_check_data_send_eof(self):
	print "check_data_send_eof"

    def implement_check_data_remove_channels(self):
	print "check_data_remove_channels"

    def implement_check_data_query_channels(self):
	print "check_data_query_channels"

    def implement_check_get_total_data_channels(self):
	print "check_get_total_data_channels"

    def implement_check_data_read(self):
	print "check_data_read"

    def implement_check_local_stru(self):
	print "check_local_stru"
	
class HostPortTest(unittest.TestCase):
    """There is a bug in the host port initialization function
       It takes a hostname when it should take an ip address
    """
    def check_CreateDel(self):
        print "Testing creation/destruction of a HostPort instance"
        try:
            host = gethostbyname(gethostname())
            port = 7575
            newL = []
            hostList = host.split('.')
            for s in hostList:
                newL.append(int(s))
            hostPort =  HostPort(newL,port)
            t = hostPort.get_info() 
            del hostPort
        except GlobusException, ex:
            print "Exception occured during testing"
            print ex
            print "TEST FAILED"
        else:
            print "TEST PASSED"
	
class AuthInfoTest(unittest.TestCase):

    def check_CreateDel(self):
        print "Testing creation/destruction of an AuthInfo instance "
        try:
            cred = GSSCred()
            cred.acquire_cred()
            lifetime,name = cred.inquire_cred()
            dn = name.display()
            authInfo = AuthInfo(cred, True, "mateo", "", "", dn)
            del authInfo
        except GlobusException, ex:
            print "Exception occured during testing"
            print ex
            print "TEST FAILED"
        else:
            print "TEST PASSED"

    def checkCreateDel(self):
        print "Testing creation/destruction of AuthInfo instance"
        try:
            cred = GSSCred()
            cred.acquire_cred()
            authInfo = AuthInfo(cred, True, "ftp", "user@", None, None)
            del authInfo
        except GlobusException, ex:
            print "Exception occured during testing"
            print ex
            print "TEST FAILED"
        else:
            print "TEST PASSED"
        
            
def makeTestSuite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(FtpControlTest__basic, "check"))
    suite.addTest(unittest.makeSuite(HostPortTest, "check"))
    suite.addTest(unittest.makeSuite(AuthInfoTest, "check"))
    return suite

def main():
    unittest.main(defaultTest="makeTestSuite")

if __name__ == "__main__" : main()
