############################################################################
# Matt Rodriguez, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################

import  unittest
from pyGlobus.security import GSICredentialAttr
from pyGlobus.security import GSICredential
from pyGlobus.security import EVP_Key
from pyGlobus.security import STACK_X509_Factory
import sha

class GSICredentialAttrTest(unittest.TestCase):

    def checkCreateDestroy(self):
        attr = GSICredentialAttr()
        del attr

class GSICredentialTest(unittest.TestCase):

    def setUp(self):
        self._cred = GSICredential()
        self._cred.read()
        
    def checkGetCert(self):
        print "Testing Getting user cert"
        cert = self._cred.get_cert()
        print "DONE"
        
    def checkGetKey(self):
        print "Testing Getting key"
        key = self._cred.get_key()
        print "DONE"
        
    def checkGetCertChain(self):
        print "Testing Getting certificate chain"
        certChain = self._cred.get_cert_chain()
        print "DONE"
        
class EVPKeyTestCase(unittest.TestCase):
    """
    Test case for the EVP_Key class
    """
    def setUp(self):
        self._cred = GSICredential()
        self._cred.read()
        
    def checkEVPKey(self):
        print "Testing EVPKey __init__ and del"
        evpKey = EVP_Key()
        del evpKey
        print "DONE"

    def checkSign(self):
        print "Testing signing"
        key = self._cred.get_key()
        signString = "Sign this please"
        signature = key.sign(sha.sha(signString).digest())
        del key
        print "DONE"
        
    def checkSignAndVerify(self):
        print "Testing Sign and Verify"
        key = self._cred.get_key()
        signString = "Sign this please"
        hashString = sha.sha(signString).digest()
        signature = key.sign(hashString)
        cert = self._cred.get_cert()
        pubKey = cert.get_public_key()
        verify = pubKey.verify(hashString, signature)
        if verify:
            print "TEST PASSED"
        else:
            print "TEST FAILED"
            
    def checkSignAndFailVerify(self):
        print "Testing Sing and Fail Verify"
        key = self._cred.get_key()
        signString = "Sign this please"
        hashString = sha.sha(signString).digest()
        signature = key.sign(hashString)
        cert = self._cred.get_cert()
        pubKey = cert.get_public_key()
        badString = "I didn't sign this"
        badHashString = sha.sha(badString).digest()
        verify = pubKey.verify(badHashString, signature)
        if verify:
            print "TEST FAILED"
        else:
            print "TEST PASSED"
            
class X509TestCase(unittest.TestCase):
    """
    Test case for the X509 objects
    """
    def setUp(self):
        self._cred = GSICredential()
        self._cred.read()

    def checkGetPublicKey(self):
        print "Testing get Public Key"
        cert = self._cred.get_cert()
        key = cert.get_public_key()
        print "DONE"

    def checkDerEncoding(self):
        print "Testing der encoding"
        cert = self._cred.get_cert()
        encoding = cert.get_der_encoding() 
        print "DONE"

        
class X509StackTestCase(unittest.TestCase):
    """
    Test case for STACK_X509 objects
    """
    def setUp(self):
        self._cred = GSICredential()
        self._cred.read()

    def checkDerEncoding(self):
        print "Testing get der encoding"
        certChain = self._cred.get_cert_chain()
        cert = self._cred.get_cert()
        certChain.insert(cert, 0)
        encoding = certChain.get_der_encoding()
        del certChain
        del cert
        print "DONE"

        
    def checkMakeFromDerEncodedSequence(self):
        print "Testing make from der encoded sequence"
        certChain = self._cred.get_cert_chain()
        cert = self._cred.get_cert()
        certChain.insert(cert, 1)
        encoding = certChain.get_der_encoding()
        stackFact = STACK_X509_Factory()
        stack = stackFact.createStackFromEncodedSequence(encoding)
        x1 = stack.pop()
        x2 = stack.pop()
        print "DONE"

    def checkMakeFromDerEncoded(self):
        print "Testing make from der encoded string"
        certChain = self._cred.get_cert_chain()
        cert = self._cred.get_cert()
        encoding = cert.get_der_encoding()
        parentCert = certChain.pop()
        encoding += cert.get_der_encoding()
        stackFact = STACK_X509_Factory()
        stack = stackFact.createStackFromEncodedString(encoding)
        c1 = stack.pop()
        c2 = stack.pop()
        print "DONE"
        
def makeTestSuite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(GSICredentialAttrTest, "check"))
    suite.addTest(unittest.makeSuite(GSICredentialTest, "check"))
    suite.addTest(unittest.makeSuite(EVPKeyTestCase, "check"))
    suite.addTest(unittest.makeSuite(X509TestCase, "check"))
    suite.addTest(unittest.makeSuite(X509StackTestCase, "check"))
    return suite

def main():
    unittest.main(defaultTest="makeTestSuite")


if __name__ == "__main__" : main()
