############################################################################
# Matt Rodriguez, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################

from pyGlobus import GSISOAP, ioc
from pyGlobus.io import AuthData, TCPIOAttr
from threading import *
import unittest, ConfigParser
import re

config = None
config_file = 'config.txt'
STRING = "spam, spam, spam, eggs, and spam"
regExp = re.compile(STRING)

def echo(s, _SOAPContext):
        cred = _SOAPContext.delegated_cred
        return s

def callback(arg, handle, identity, context):
        return 1

class testServer(Thread):

    def __init__(self, host, port):
        Thread.__init__(self)
	self.__threads = []
	self.__host = host
	self.__port = port
	
    def run(self):
        myTcpAttr = TCPIOAttr()
	myTcpAttr.set_authentication_mode(ioc.GLOBUS_IO_SECURE_AUTHENTICATION_MODE_GSSAPI)
	tcpAuthdata = AuthData()
	
	tcpAuthdata.set_callback(callback, None)
	myTcpAttr.set_authorization_mode(ioc.GLOBUS_IO_SECURE_AUTHORIZATION_MODE_CALLBACK, tcpAuthdata)
	myTcpAttr.set_delegation_mode(ioc.GLOBUS_IO_SECURE_DELEGATION_MODE_FULL_PROXY)
	myTcpAttr.set_channel_mode(ioc.GLOBUS_IO_SECURE_CHANNEL_MODE_GSI_WRAP)
	

	server = GSISOAP.SOAPServer(
		addr=(self.__host, self.__port),
		channel_mode=ioc.GLOBUS_IO_SECURE_CHANNEL_MODE_GSI_WRAP,
		delegation_mode=ioc.GLOBUS_IO_SECURE_DELEGATION_MODE_FULL_PROXY,
		tcpAttr = myTcpAttr,
		namespace = "urn:gtg-Echo"
		)
	server.registerFunction(GSISOAP.MethodSig(echo, keywords=0, context=1), "urn:gtg-Echo")
	
	
	server.handle_request()
	self.doJoins()
	server.server_close()
	print "TEST PASSED"
	
    def doJoins(self):
        for thr in self.__threads:
	    thr.join()
	    
    def insertClientThread(self, thread):
        self.__threads.insert(0,thread)	    

class testClient(Thread):

    def __init__(self, host, port):
        Thread.__init__(self)
	self.__host = host
	self.__port = port
	
    def run(self):
        url = "https://" + self.__host +':' + self.__port
        proxy = GSISOAP.SOAPProxy(url, namespace="urn:gtg-Echo")
	proxy.set_channel_mode = ioc.GLOBUS_IO_SECURE_CHANNEL_MODE_GSI_WRAP
	proxy.set_delegation_mode(ioc.GLOBUS_IO_SECURE_DELEGATION_MODE_NONE)
	recvString = proxy.echo(STRING)
	if regExp.match(recvString):
	    print "TEST PASSED"
	else:
	    print "TEST FAILED"

class GSISOAPTestCase(unittest.TestCase):
    def setUp(self):
        global config
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read(config_file)
	    
	
    def checkEcho(self):
        global server
	print "Starting GSISOAP server"
	host = config.get("GSISOAP", "HttpHost")
	port = config.get("GSISOAP", "HttpPort")
        server = testServer(host, int(port))
	server.start()
	
	
    def checkEchoClient(self):
        print "Starting SOAPProxy client"
        host = config.get("GSISOAP", "HttpHost")
	port = config.get("GSISOAP", "HttpPort")
        client = testClient(host, port)
	server.insertClientThread(client)
	client.start()
	server.join()

def makeTestSuite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(GSISOAPTestCase, "check"))
    return suite

def main():
    unittest.main(defaultTest="makeTestSuite")

if __name__ == "__main__" : main()

