############################################################################
# Joshua R. Boverhof, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################

import sys, getopt, unittest, time
from threading import *
from pyGlobus import ioc
from pyGlobus.io import GSITCPSocket, AuthData, TCPIOAttr
from pyGlobus.util import Buffer
import re

gPort = None
STRING1 = "Foobar, fOobar, foObar, fooBar, foobAr, foobaR"
STRING2 = "Fee, Fie, Foo, Fum"
regExpString = STRING1 + '|' + STRING2
regExp = re.compile(regExpString)

def auth_callback(arg, handle, identity, context): 
    return 1
 
def usage():
    useStr = "test_tcp_client -h host -p port"
    print useStr
    return

class GSITCPSocketCase(unittest.TestCase):
    def setUp(self):
        print "Begin GSITCPSocketServer"
        self._server = testClientServer(client=0)
        self._client = testClientServer(client=1)
        return
    def check_client_server(self):
        time.sleep(1)
        self._server.start()
        time.sleep(1)
        self._client.run(gPort)
        print  "DONE"
        return
    
class testClientServer(Thread):
    def __init__(self, client=None):
        Thread.__init__(self)
        self._client = client
        self.soc = GSITCPSocket()
        self.authData = AuthData()
        self.attr = TCPIOAttr()
        self.attr.set_authentication_mode(ioc.GLOBUS_IO_SECURE_AUTHENTICATION_MODE_GSSAPI)
        self.attr.set_delegation_mode(ioc.GLOBUS_IO_SECURE_DELEGATION_MODE_FULL_PROXY)
        self.attr.set_channel_mode(ioc.GLOBUS_IO_SECURE_CHANNEL_MODE_CLEAR)
        return
    def run(self,port=gPort,host='localhost'):
        if not self._client:
            self.authData.set_callback(auth_callback, None)
            self.attr.set_authorization_mode(\
                ioc.GLOBUS_IO_SECURE_AUTHORIZATION_MODE_CALLBACK, self.authData)
            self.init_Server()
        else:
            self.attr.set_authorization_mode(\
                ioc.GLOBUS_IO_SECURE_AUTHORIZATION_MODE_SELF, self.authData)
            self.init_Client(host, port)
        return
    def init_Server(self):
        port = self.soc.create_listener(self.attr) 
        global gPort
        gPort = port
        self.soc.listen() 
        (childSoc, addr) = self.soc.accept(self.attr) 
        ctx = childSoc.get_security_context() 
        tuple = ctx.inquire() 
        name = tuple[0].display() 
        cred = childSoc.get_delegated_credential() 
        size = 46
        buf = Buffer(size)
        
        bytesRead = childSoc.read(buf, size, size)
        if not regExp.match(buf.as_string(bytesRead)):
            print "Did not recieve expected data in init_Server"
            print "TEST FAILED"
        else:
            print "Server read data ok"
        childSoc.write(STRING2, len(STRING2))   
        del cred
        del childSoc
        del self.soc
        return 
    def init_Client(self,host,port):
        self.soc.connect(host, port, self.attr) 
        nBytes = self.soc.write(STRING1, len(STRING1)) 
        size = 18
        buf = Buffer(size) 
        bytesRead = self.soc.read(buf, size, size)
        
        
        if not regExp.match(buf.as_string(bytesRead)):
            print "Did not recieve expected data in init_Server"
            print "TEST FAILED"
        else:
            print "Client read data ok"
        return

def makeTestSuite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(GSITCPSocketCase, "check_"))
    return suite

def main():
    try:
        opts, args = getopt.getopt(sys.argv[1:], "h:p:s")
    except getopt.GetoptError:
        usage()
        sys.exit(-1)
    host = None
    if opts:
        for o, a in opts:
            if o == "-h":
                host = a
            elif o == "-p":
                port = int(a)
            elif o == "-s":
                testCS = testClientServer(client=0)
                testCS.run()
                break
            else:
                usage()
                sys.exit(-1)
        else:
            testCS = testClientServer(client=1)
            testCS.run(host,port)
    else:
        unittest.main(defaultTest="makeTestSuite")
    return
    
if __name__ == '__main__':
    main()

    
