############################################################################
# Joshua R. Boverhof, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################

import unittest, ConfigParser 
import SocketServer
from threading import *
from pyGlobus.io import GSITCPSocket,GSITCPSocketServer,TCPIOAttr,AuthData, GSITCPSocketException
from pyGlobus import ioc
from pyGlobus.util import GlobusException
import re

config = None
config_file = 'config.txt'
g_port = 0
server = None
STRING1 = "Foobar, fOobar, foObar, fooBar, foobAr, foobaR\n"
STRING2 = "Some DAta SOme DAtA\n"
STRING3 = "More DAta mORE daTA\n"
STRING4 = "Last Bit\n"
regExpString = STRING1 +'|' + STRING2 + '|' + STRING3 + '|' + STRING4 + '|' +""
regExp = re.compile(regExpString)

def auth_callback(arg, handle, identity, context):
        return 1

class TestServer2(Thread):
    def __init__(self):
	Thread.__init__(self)

    def setup(self, port):
       try:
           self.handle = GSITCPSocketServer(('',port), GSIRequestHandler)
       except GlobusException, ex:
           print "exception raised"
	   raise ex

    def run(self):
        try:
	     self.handle.handle_request()
	except Exception, ex:
	     print "TEST PASSED"
	     
    def server_close(self):
        self.handle.server_close()

    
class TestServer(Thread):
    def __init__(self):
        Thread.__init__(self)
	self.__threads = []
	
    def setup(self, port):
	global g_port
        g_port = port
	try:
	    self.handle = GSITCPSocketServer(('',g_port), GSIRequestHandler)
        except GlobusException, ex:
	    print ex.args
	    raise ex

    def insertClientThread(self, thread):
        self.__threads.insert(0,thread)

    def doJoins(self):
        for thr in self.__threads:
	    thr.join()
	    
    def run(self):
        self.handle.handle_request()
	self.handle.handle_request()
	self.handle.handle_request()
	#Make sure all of the client threads have finished before we close
	self.doJoins()
	self.server_close()
	
    def server_close(self):
        self.handle.server_close()
	

class GSIRequestHandler(SocketServer.StreamRequestHandler):
   """A minimalistic Request handler class that receives data from the
   client only. It prints it all out and then exits.
   """
      
   def handle(self):
      result = 'X'
      #print "GSIRequestHandler got a connection from %s" \
      #     % (self.client_address,)

      while result:
         result = self.rfile.readline()
	 #print result
	 if not regExp.match(result):
	     exceptString = "Did not receive expected Data, got: " +result
	     raise GSITCPSocketException(exceptString)


     
class TestClient(Thread):
    def __init__(self):
        Thread.__init__(self)
	                   
    def run(self):          
        host = "localhost"  
        try:
            attr = TCPIOAttr()
            attr.set_authentication_mode(ioc.GLOBUS_IO_SECURE_AUTHENTICATION_MODE_GSSAPI)
            authData = AuthData()
            attr.set_authorization_mode(ioc.GLOBUS_IO_SECURE_AUTHORIZATION_MODE_SELF, authData)
            attr.set_channel_mode(ioc.GLOBUS_IO_SECURE_CHANNEL_MODE_GSI_WRAP)
            attr.set_delegation_mode(ioc.GLOBUS_IO_SECURE_DELEGATION_MODE_LIMITED_PROXY)
            soc = GSITCPSocket()
            soc.connect(host, g_port, attr)
	    addrList = soc.get_remote_address()
	    str = STRING1
            nBytes = soc.write(str, len(str))
        except GlobusException, ex:
	    print "exception in testClient", ex.args
	    print "TEST FAILED"
	    raise ex
	print "TEST PASSED"

class TestClient2(Thread):
    def __init__(self):
        Thread.__init__(self)

    #Tests init_delegation, which doesn't currently work
    def run(self):               
        host = "localhost"
        try:
            attr = TCPIOAttr()
            attr.set_authentication_mode(ioc.GLOBUS_IO_SECURE_AUTHENTICATION_MODE_GSSAPI)
            authData = AuthData()
            attr.set_authorization_mode(ioc.GLOBUS_IO_SECURE_AUTHORIZATION_MODE_SELF, authData)
            attr.set_channel_mode(ioc.GLOBUS_IO_SECURE_CHANNEL_MODE_GSI_WRAP)
            attr.set_delegation_mode(ioc.GLOBUS_IO_SECURE_DELEGATION_MODE_LIMITED_PROXY)
            soc = GSITCPSocket()	    
            soc.connect(host, g_port, attr)
	    addrList = soc.get_remote_address()
	    str = " -------------------------------------------------------------- - - -- -- - - -\n"
	    cred = soc.get_delegated_credential()
	    soc.init_delegation(cred, [], [], 0)
	    nBytes = soc.write(str, len(str))
        except GlobusException, ex:
	    print "exception in testClient2_CAS", ex.args
	    raise ex

class TestClient3(Thread):
    def __init__(self):
        Thread.__init__(self)

    def run(self):            
        host = "localhost"
        try:
            attr = TCPIOAttr()
            attr.set_authentication_mode(ioc.GLOBUS_IO_SECURE_AUTHENTICATION_MODE_GSSAPI)
            authData = AuthData()
            attr.set_authorization_mode(ioc.GLOBUS_IO_SECURE_AUTHORIZATION_MODE_SELF, authData)
            attr.set_channel_mode(ioc.GLOBUS_IO_SECURE_CHANNEL_MODE_GSI_WRAP)
            attr.set_delegation_mode(ioc.GLOBUS_IO_SECURE_DELEGATION_MODE_LIMITED_PROXY)
            soc = GSITCPSocket()	    
            soc.connect(host, g_port, attr)
	    addrList = soc.get_remote_address()
	    str = STRING2 + STRING3 + STRING4  
	    phile = soc.makefile()
            nBytes = soc.send(str)
            nBytes = soc.sendall(str)
	    phileread = phile.read() 
        except GlobusException, ex:
	    print "exception in testClient3", ex.args
	    print "TEST FAILED"
	    raise ex
	print "TEST PASSED"
	    


class TestClient4(Thread):
    """
    Connects to the server
    Gets the security context
    examines the security context
    closes the socket
    """
    def __init__(self):
        Thread.__init__(self)
	                   
    def run(self):          
        host = "localhost"  
        try:
            attr = TCPIOAttr()
            attr.set_authentication_mode(ioc.GLOBUS_IO_SECURE_AUTHENTICATION_MODE_GSSAPI)
            authData = AuthData()
            attr.set_authorization_mode(ioc.GLOBUS_IO_SECURE_AUTHORIZATION_MODE_SELF, authData)
            attr.set_channel_mode(ioc.GLOBUS_IO_SECURE_CHANNEL_MODE_GSI_WRAP)
            attr.set_delegation_mode(ioc.GLOBUS_IO_SECURE_DELEGATION_MODE_LIMITED_PROXY)
            soc = GSITCPSocket()
            soc.connect(host, g_port, attr)
	    addrList = soc.get_remote_address()
	    sec = soc.get_security_context()  
            del sec
	    soc.close() 
        except GlobusException, ex:
	    print "exception in testClient", ex.args
	    print "TEST FAILED"
	    raise ex
	print "TEST PASSED"

class GSITCPSocketServerShutdownCase(unittest.TestCase):
    def setUp(self):
        global config
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read(config_file)
	    
    def checkSocketShutdown(self):
        print "Testing exception handling in GSITCPServer.get_request"
        try:
	    server = TestServer2()
	    port = config.get("io", "ListenPort3")
	    server.setup(int(port))
	    server.start()
	    server.server_close()
	except GlobusException, ex:
	    print ex.args
	    print "TEST FAILED"
	

    
	    
class GSITCPSocketServerCase(unittest.TestCase):
    
    def setUp(self):
        global config
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read(config_file)

    
    def checkSocketLoadTest1(self):    # loads test server
        """
	Starts up the Socket Server,
	closes the listening socket when
	"""
        global server
        try:
	    print "Starting Server Test"
	    server = TestServer()
	    port = config.get("io", "ListenPort1")
	    server.setup(int(port))
	    server.start()
	    
	except GlobusException, ex:
	     print "exception raised in Test1", ex.args
	     print "TEST FAILED"
	     raise ex

    def checkSocketLoadTest2(self):
        """
	Tests connect and write
	"""
        try:
	    print "Starting connect and write test"
	    self.client1 = TestClient()
	    server.insertClientThread(self.client1)
	    self.client1.start()
	    
	except GlobusException, ex:
	    print "exception raised in Test2"
	    raise ex
	
    def dontcheckSocketLoadTest3(self):    # loads testClient2()
        try:
	    self.client2 = TestClient2()
	    server.insertClientThread(self.client2)
	    self.client2.start()
        except GlobusException, ex:
	    print "exception raised in Test2"
	    raise ex  
	

    def checkSocketLoadTest4(self):    # loads testClient3()
	"""
	Tests send, sendall
	"""
        try:
	    print "Starting send, sendall test"
	    self.client3 = TestClient3()
	    server.insertClientThread(self.client3)
	    self.client3.start()
	    
        except GlobusException, ex:
	    print "exception raised in Test4"
	    raise ex 

    def checkSocketLoadTest5(self):
	"""
	Tests connect, get_security_context, close
	"""
        try:
	     print "Starting get_security_context test"
	     self.client4 = TestClient4()
	     server.insertClientThread(self.client4)
	     self.client4.start()
        except GlobusException, ex:
	     print "exception raised in Test5"
	     raise ex
   
   
        
def makeTestSuite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(GSITCPSocketServerCase, "check"))
    suite.addTest(unittest.makeSuite(GSITCPSocketServerShutdownCase, "check"))
    return suite

def main():
    unittest.main(defaultTest="makeTestSuite")

if __name__ == "__main__" : main()
