############################################################################
# Joshua R. Boverhof, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################

from pyGlobus.io import GSITCPSocket
from pyGlobus.io import TCPIOAttr
from pyGlobus.io import AuthData
from pyGlobus import ioc
from pyGlobus.util import Buffer
from pyGlobus.util import GlobusException
import  unittest
from threading import *

config = None
g_port = 0
def auth_callback(arg, handle, identity, context):
    return 1
    
class testServer(Thread):
    def run(self):
        global config, g_port
        try:
            attr = TCPIOAttr()
            attr.set_authentication_mode(ioc.GLOBUS_IO_SECURE_AUTHENTICATION_MODE_GSSAPI)
            authData = AuthData()
            authData.set_callback(auth_callback, None)
            attr.set_authorization_mode(ioc.GLOBUS_IO_SECURE_AUTHORIZATION_MODE_CALLBACK, authData)
            attr.set_channel_mode(ioc.GLOBUS_IO_SECURE_CHANNEL_MODE_GSI_WRAP)
	    attr.set_delegation_mode(ioc.GLOBUS_IO_SECURE_DELEGATION_MODE_LIMITED_PROXY)
            soc = GSITCPSocket()
            port = soc.create_listener(attr)
            g_port = port
            client_t = testClient()
            client_t.start()
	    soc.listen()
            childSoc, remote = soc.accept(attr)
            ctx = childSoc.get_security_context()
            tuple = ctx.inquire()
            name = tuple[0].display()
	    cred = childSoc.get_delegated_credential()
            size = 46
            buf = Buffer(size)
            bytesRead = childSoc.read(buf, size, size)
            recvStr = buf.as_string(bytesRead)
            if recvStr != "Foobar, fOobar, foObar, fooBar, foobAr, foobaR":
                print "TEST FAILED"
                print "received from socket: ", recvStr
            else:
                print "TEST PASSED"
            del cred
            del childSoc
            del soc
        except GlobusException, ex:
            print "caught an exception"
            print ex.args
            print "TEST FAILED" 
            return

class testClient(Thread):
    def run(self):
        host = "localhost"
        try: 
            attr = TCPIOAttr()
            attr.set_authentication_mode(ioc.GLOBUS_IO_SECURE_AUTHENTICATION_MODE_GSSAPI)
            authData = AuthData()
            authData.set_callback(auth_callback, None)
            attr.set_authorization_mode(ioc.GLOBUS_IO_SECURE_AUTHORIZATION_MODE_CALLBACK, authData) 
            attr.set_channel_mode(ioc.GLOBUS_IO_SECURE_CHANNEL_MODE_GSI_WRAP) 
            soc = GSITCPSocket()
            soc.connect(host, g_port, attr)
            str = "Foobar, fOobar, foObar, fooBar, foobAr, foobaR"
            nBytes = soc.write(str, len(str))
        except GlobusException, ex:
            print "caught an exception"
            print ex.args
            print "TEST FAILED" 
        return

class GSITcpTestCase(unittest.TestCase):
    
    def setUp(self):
       print "Testing reading/writing with GSITCPSockets."
       print "using various authentication/authorization/delegation attributes"


    def checkGSITcp(self):
        server_t = testServer()
        server_t.start()

        
def makeTestSuite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(GSITcpTestCase, "check"))
    return suite

def main():
    unittest.main(defaultTest="makeTestSuite")

if __name__ == "__main__" : main()
