############################################################################
# Matt Rodriguez, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################

import ConfigParser, unittest
from pyGlobus.security import GSSCred, grid_proxy_destroy
from pyGlobus.util import GlobusException
from pyGlobus.security import GSSName
from pyGlobus.security import GSSMechs
from pyGlobus.security import GSSUsage
from pyGlobus.security import grid_proxy_info
from pyGlobus.security import GSIException
"""
Unittest for the GSSCred module.
"""

config = None # ConfigParser object

class GSSCredTestCase(unittest.TestCase):
    """
    Test case for the GSSCred module.
    """
    def setUp(self):
        global config
        try:
            s,d = grid_proxy_info()
            if d['lifetime'] == 0:
                 self.fail("You need to have a valid proxy running this test") 
                 
        except GSIException, e:
            print e
            self.fail("You need to have a valid proxy running this test") 
            
       
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read("config.txt")
        self._cred = GSSCred()
        self._name = GSSName()
        self._mechs = GSSMechs()
        self._usage = GSSUsage()
        
    def checkGSSCred(self):
        print "Testing creating a destroying GSSCred instance"
        try:
            cred = GSSCred()
            del cred 
        except GlobusException, ex:
            print "Exception occured during testing."
            print "TEST FAILED"
        else:
            print "TEST PASSED"
        return
    
    def checkAcquireCred(self):
        print "Testing acquiring credential" 
        try: 
            self._cred.acquire_cred(self._name,self._mechs, self._usage)
        except GlobusException, ex:
            print "Exception occured during testing."
            print "TEST FAILED" 
        else: 
            print "TEST PASSED"

    def checkInquireCred(self):
        print "Testing acquiring and inquiring credential" 
        try: 
            self._cred.acquire_cred(self._name,self._mechs, self._usage)
            ret_lifetime, ret_name = self._cred.inquire_cred()
            print "Printing lifetime and DN information"
            print "lifetime is: ", ret_lifetime
            n = ret_name.display()
            print "name is: ", n
        except GlobusException, ex:
            print "Exception occured during testing."
            print "TEST FAILED" 
        else: 
            print "TEST PASSED" 
       

    def checkAquireCredLessTimeRequested(self):
        """
        The time requested argument in globus's current GSSAPI
        implementation of acquire_cred is ignored.
        """
        print "Testing acquire cred with time requested argument" 
        try:
            time_req = 800
            self._cred.acquire_cred(self._name,self._mechs, self._usage,time_requested=time_req) 
        except GlobusException, ex:
            print "Exception occured during testing."
            print "TEST FAILED" 
        else: 
            print "TEST PASSED" 

    def checkExportImportCred(self):
        print "Testing exporting and importing a credential." 
        try:
            self._cred.acquire_cred(self._name,self._mechs, self._usage)
            cred_location = self._cred.export_external()
            print "Exported cred to ", cred_location
            newCred = GSSCred()
            newCred.import_external(cred_location)
            ret_lifetime, ret_name = newCred.inquire_cred()
            print "lifetime is: ", ret_lifetime
            print "name is: ", ret_name.display() 
	    grid_proxy_destroy(cred_location)
	    
        except GlobusException, ex:
            print "Exception occured during testing."
            print "TEST FAILED" 
        else: 
            print "TEST PASSED" 
        
def makeTestSuite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(GSSCredTestCase, "check"))
    return suite

def main():
    unittest.main(defaultTest="makeTestSuite")


if __name__ == "__main__" : main()
    
                  

