############################################################################
# Matt Rodriguez, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################

import ConfigParser, unittest
from pyGlobus.security import GSSName
from pyGlobus.security import GSSMechs
from pyGlobus.security import GSSUsage
from pyGlobus.security import GSSBuffer
from pyGlobus.security import X509_Request


"""
Unittest for the all of the Simple GSS  classes  
"""

config = None # ConfigParser object

class GSSNameTestCase(unittest.TestCase):
    """
    Test case for the GSSName class.
    """

    def checkGSSName(self):
        global config
        print "Testing creation and destruction of a GSSName instance"
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read("config.txt")
        try:
            name = GSSName()
            del name 
        except GlobusException, ex:
            print "Exception occured during testing."
            print ex
            print "TEST FAILED" 
        else:
            print "TEST PASSED"

class GSSMechsTestCase(unittest.TestCase):
    """
    Test case for the GSSMechs class
    """

    def checkGSSMechs(self):
        global config
        print "Testing creation and destruction of a GSSMechs instance"
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read("config.txt")
        try:
            mech = GSSMechs()
            del mech
        except Exception, ex:
            print "Exception occured during testing. "
            print ex
            print "TEST FAILED"
        else:
            print "TEST PASSED"
        return

class GSSUsageTestCase(unittest.TestCase):
    """
    Test case for the GSSMechs class
    """
    def setUp(self):
        global config 
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read("config.txt")
            
    def checkGSSUsage(self):
        print "Testing creation and destruction of a GSSUsage instance"
        try:
            usage = GSSUsage()
            del usage
        except Exception, ex:
            print "Exception occured during testing. "
            print ex
            print "TEST FAILED"
        else:
            print "TEST PASSED"
        
        return

    def checkSetUsage(self):
        print "Testing setting GSSUsage instance"
        try:
            usage = GSSUsage()
            usage.set_usage_initiate()
            usage.set_usage_accept()
            del usage
            
        except Exception, ex:
            print "Exception occured during testing. "
            print ex
            print "TEST FAILED" 
        else:
            print "DONE"
        return

class X509RequestTestCase(unittest.TestCase):
    """
    Test case for the X509_Request class
    """
    def checkX509(self):
        print "Testing creation and destruction of a X509Req instance" 
        try:
            x509req = X509_Request()
            del x509req
        except Exception, ex:
            print "Exception occured during testing. "
            print ex
            print "TEST FAILED"
        else:
            print "TEST PASSED"    

class GSSBufferTestCase(unittest.TestCase):
    """
       Test case for the GSSBuffer class
       """
    def checkGSSBuffer(self):
        print "Testing creation an destruction of a GSSBuffer instance" 
        try:
            buf = GSSBuffer()
            del buf
        except Exception, ex:
            print "Exception occured during testing. "
            print ex
            print "TEST FAILED"
        else:
            print "TEST PASSED"    
    
        
    def checkBufferSet(self):
        print "testing setting and gettting GSSBuffer"
        try:
            buf = GSSBuffer()
            tmpstr = "Cream" 
            buf.set_buffer(tmpstr)
            output = buf.get_string()
            if output == tmpstr:
                print "TEST PASSED"
            else:
                print "TEST FAILED"
        except Exception, ex:
            print "Exception occured during testing. "
            print ex
            print "TEST FAILED"
      
     
            
            
        
def makeTestSuite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(GSSNameTestCase, "check"))
    suite.addTest(unittest.makeSuite(GSSMechsTestCase, "check"))
    suite.addTest(unittest.makeSuite(GSSUsageTestCase, "check"))
    suite.addTest(unittest.makeSuite(X509RequestTestCase, "check"))
    suite.addTest(unittest.makeSuite(GSSBufferTestCase, "check"))
    return suite

def main():
    unittest.main(defaultTest="makeTestSuite")


if __name__ == "__main__" : main()
    
                  

