############################################################################
# Keith R. Jackson, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################

from pyGlobus.gassCache import GassCache
from pyGlobus import gassCopy, gassCachec
from pyGlobus.util import GlobusException
import ConfigParser, unittest, os, socket


#The config file is ../../test/config.txt
config = None # ConfigParser object

SECTION = "gassCache"
URL = "url"
TAG = "tag"
HOME = "homeDir"
gSetEnvironment = True

#############################################################################
#  Unit test for the gassCache Module.
##############################################################################
class GassCacheTestCase(unittest.TestCase):
    "Test case for the GassCache module"

    def setUp(self):
        global config, gSetEnvironment
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read("config.txt")

        if gSetEnvironment:
            hostname = socket.gethostname()
            homeDir = os.environ["HOME"]
            gassDir = config.get(SECTION,HOME)
            homeDirFinal = homeDir +os.sep+ gassDir +os.sep+ hostname 
            os.environ.update({"GLOBUS_GASS_CACHE_DEFAULT": homeDirFinal})
            gSetEnvironment = False 

        return

    def check_cache_query(self): 
        print 'Testing cache_query expect success'
        url = config.get(SECTION, URL)     
        tag = config.get(SECTION, TAG)     
        try:
            gCache = GassCache()
            gCache.open()
            ret, file,timestamp  = gCache.add(url, tag, 1)
            if ret == gassCachec.GLOBUS_GASS_CACHE_ADD_EXISTS:
                ## File already exists.  Mark it 'done' and continue.
                ## Note that because we 'cleanup_tag' at the end of
                ## the test, this code path will probably not be
                ## executed unless somebody else has left the cache populated with this entry                
                ret = gCache.add_done(url, tag,timestamp)
            elif ret == gassCachec.GLOBUS_GASS_CACHE_ADD_NEW:
                ## File wasn't in cache.  Add it.
                g = gassCopy.GassCopy(handleAttr=gassCopy.HandleAttr(), handle=None)
                try:
                    ## DEK: Should check for success here (ret != GLOBUS_SUCCESS). Delete the cache entry if the transfer failed.
                    ## NJE: Library layer functions shouldn't return GLOBUS_SUCCESS, but rather throw exceptions.
                    g.copy_url_to_url(srcUrl=url, srcAttr=gassCopy.Attr(), destUrl="file://" + file, destAttr=gassCopy.Attr())
                except Exception, ex:
                    
                    gCache.delete(url, tag, timestamp, 1)                    
                ret = gCache.add_done(url, tag,timestamp)
            elif ret == GLOBUS_SUCCESS or ret == None:
                ret = gCache.add_done(url, tag,timestamp)
            else:
                print "TEST FAILED"
                raise RuntimeError, "return value is unexpected"
            gCache.close()
            gCache = GassCache()
            gCache.open()
            incache = gCache.cache_query(url,tag,False,timestamp,file)
            gCache.cleanup_tag(url, tag)
            gCache.close()
            del gCache
        except GlobusException, ex:
            print "Exception occured during testing."
            print ex
            print "TEST FAILED"
        print "DONE" 
        return

    def check_cache_query_failure(self):
        print 'Testing cache_query expect failure'
        url = config.get(SECTION, URL)      
        tag = config.get(SECTION, TAG)      

        try:
            gCache = GassCache()
            gCache.open()
            incache = gCache.cache_query(url,tag,False,0,"file")
            gCache.cleanup_tag(url, tag)
            gCache.close()
            del gCache
        except Exception, ex:
            
            if str(ex) == 'Invalid error code': 
                print "TEST PASSED"
            else:
                print "Exception was thrown but with an unexpected error message"
                print ex
                print "TEST FAILED"
        else:
            print "TEST FAILED" 
        return
    
    def check_cache_get_dirs(self):
        print 'Testing check_get_dirs'
        url = config.get(SECTION, URL)      
        tag = config.get(SECTION, TAG)      
        try:
            gCache = GassCache()
            gCache.open()
            ret = gCache.cache_get_dirs(url,tag)
            print ret
            del gCache
        except GlobusException, ex:
            print "Exception occured during testing."
            print ex
            print "TEST FAILED"
        print "DONE" 
        

    def check_mangle_tag(self):
        print 'Testing mangling tag'
        tag = config.get(SECTION, TAG)      
        try:
            gCache = GassCache()
            gCache.open()
            print 'mangling tag: ', tag
            mangle = gCache.mangle_tag(tag)
            print 'mangle: ',mangle
            if mangle ==None:
                print "TEST FAILED" 

            del gCache
        except GlobusException, ex:
            print "Exception occured during testing.", ex
            print ex
            print "TEST FAILED"
            
        print "DONE" 

    
    def check_mangle_url(self):
        print 'Testing mangling url'
        url = config.get(SECTION, URL)      
        try:
            gCache = GassCache()
            gCache.open()
            print 'url: ', url
            print 'mangle: ',gCache.mangle_url(url)
            del gCache
        except GlobusException, ex:
            print "Exception occured during testing."
            print ex
            print "TEST FAILED"
        print "DONE" 
    
    def checkGassAddCleanupTagCache(self):  
        print 'Testing adding a file then removing the tag to the file'
        url = config.get(SECTION, URL)      
        try:
            gCache = GassCache()
            gCache.open()
            ret, file,timestamp  = gCache.add(url, "foobar", 1)
            gCache.add_done(url, "foobar",timestamp)
            gCache.cleanup_tag(url, "foobar")
            gCache.close()
            del gCache
        except GlobusException, ex:
            print "Exception occured during testing."
            print ex
            print "TEST FAILED"
        print "DONE" 
        return
    
    def checkGassAddDeleteCache(self):
        print 'begin testing GassAddDeleteTagCache'
        url = config.get(SECTION, URL)      
        try:                                
            gCache = GassCache()
            gCache.open()
            ret, file,timestamp = gCache.add(url, "foobar", 1)
            gCache.add_done(url, "foobar",timestamp)
            timestamp = gCache.delete_start(url, "foobar")
            gCache.delete(url, "foobar", timestamp, 1)
            gCache.close()
            del gCache
        except GlobusException, ex:
            print "Exception occured during testing."
            print ex
            print "TEST FAILED"
        print "DONE"

    

            
def makeTestSuite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(GassCacheTestCase, "check"))
    return suite

def main():
    unittest.main(defaultTest="makeTestSuite")


if __name__ == "__main__" : main()
    
