############################################################################
# Matt Rodriguez, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################

from pyGlobus.gramClient import GramClient, JOB_STATE_PENDING, JOB_STATE_ACTIVE
from pyGlobus.gramClient import JOB_STATE_FAILED, JOB_STATE_DONE, JOB_STATE_ALL
from pyGlobus.util import GlobusException
import  ConfigParser, unittest
from threading import *
from pyGlobus.gassServerEZ import GassServerEZ, STDOUT_ENABLE

cond = 0
#point this to $PYTHONPATH/pyGlobus/test/config.txt
config_file = "config.txt"
config = None # ConfigParser object

NAME = "gassServerEZ"
RM = "rm"
RSL = "rsl"

def func(cv, contact, state, error): 
    global cond
    if state == JOB_STATE_PENDING:
        #print "Job is pending"
        pass
    elif state == JOB_STATE_ACTIVE:
        #print "Job is active"
        pass
    elif state == JOB_STATE_FAILED:
        #print "Job failed"
        cv.acquire()
        cond = 1
        cv.notify()
        cv.release()
    elif state == JOB_STATE_DONE:
        #print "Job is done"
        cv.acquire()
        cond = 1
        cv.notify()
        cv.release()


#############################################################################
#  Unit test for the gassServerEZ Module:
#
#  Same as the GramClient Module except this uses a gassServerEZ object which
#  is used to capture stdout on the remote machine on print it on the machine
#  that the test is run on
#
##############################################################################
class GassServerEZTestCase(unittest.TestCase):
    "Test case for the GramClient module"
    
    def checkGassServer(self):
        print "Submitting a job and capturing the output using a GassServer"
        global config, cond
        opts = STDOUT_ENABLE
        testServer = GassServerEZ(opts)
        u = testServer.getURL()
        print u
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read(config_file)
    
        rm = config.get(NAME, RM)
        r = config.get(NAME, RSL)
        rsl = r + "(stdout=" + u + "/dev/stdout)"
        #print "remote machine: ", rm 
        #print "rsl string: ", rsl  
        print "sending a job description %s to %s" % (rsl,rsl)
        condV = Condition(Lock())
        try:
            gramClient = GramClient()
            assert not gramClient == None, "Error creating the GramClient object"
            condV.acquire()
            callbackContact = gramClient.set_callback(func, condV) 
            jobContact = gramClient.submit_request(rm, rsl, JOB_STATE_ALL,
            callbackContact) 
            while cond == 0: 
                condV.wait()
                condV.release()
            gramClient.remove_callback(callbackContact) 
        except GlobusException, ex:
            print "Exception occured during testing" 
            print ex
            print "TEST FAILED"
        else:
            print "DONE"

def makeTestSuite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(GassServerEZTestCase, "check"))
    return suite

def main():
    unittest.main(defaultTest="makeTestSuite")


if __name__ == "__main__" : main()
    
                  

