############################################################################
# Matt Rodriguez, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################

import ConfigParser, unittest
from pyGlobus.gassTransfer import TransferReqAttr
from pyGlobus.gassTransfer import TransferClient
from pyGlobus.gassTransfer import TransferServer
from pyGlobus.gassTransfer import ListenerAttr
from pyGlobus.gassTransfer import TransferListener
from pyGlobus.gassTransfer import TransferData
from pyGlobus.util import GlobusException


from threading import *
from pyGlobus.util import Buffer
import sys

"""
Unittest for the TransferReqAttr module.
"""
config_file = "config.txt"
config = None # ConfigParser object

class TransferReqAttrTestCase(unittest.TestCase):
    """
    Test case for the TransferReqAttr class.
    """
        
    def checkTransferReqAttr(self):
        global config, config_file
        print "TransferReqAttrTestCase begin"
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read(config_file)
        try:
            name = TransferReqAttr("http")
            del name
        except GlobusException, ex:
            print "TEST FAILED"
            assert 0, ex.args
        print "TEST PASSED"
        return

    
class TestServer(Thread):

    def __init__(self):
        Thread.__init__(self)
        scheme = "http"
        self._server = TransferServer(scheme)
        self._reqAttr = TransferReqAttr(scheme)
        self._condV = Condition(Lock())
        self._url =  self._server.get_base_url()
        self._Trans = TransferData()
        self._length = 1024
        self._buffer_recv = Buffer(self._length)
        self._oldSendCbHandle = None
        self._newSendCbHandle = None
        self._oldRecvCbHandle = None
        self._newRecvCbHandle = None
        self._doSend = None
        self._doRecv = None
        
    def registerListenCallback(self, cv, request):
        self.callbackSynchronization()

    def registerAcceptCallback(self, cv, request):
        self.callbackSynchronization()

    def callbackSynchronization(self):
        self._condV.acquire()
        self._done = 1
        self._condV.notify()
        self._condV.release()
        
    def receiveBytesCallback(self,cv, request, bufHandle, bufLen,  eof):
        self._dest_file.write(bufHandle)
        self._dest_file.flush()
        if not eof:
            try:
                handle = self._Trans.receive_bytes(self._recv_request,self._buffer_recv,1, self.receiveBytesCallback, cv)
                
                if not self._oldRecvCbHandle == None:
                    self._Trans.free_callback(self._oldRecvCbHandle)
                    
                self._oldRecvCbHandle = self._newRecvCbHandle
                self._newRecvCbHandle = handle
            except Exception, e:
                print e.args
                sys.exit(-1)
        else:
            cv.acquire()
            self._done = 1
            cv.notify()
            cv.release()
        return

    def sendBytesCallback(self,cv, request, bufHandle, bufLen, eof):
        if not eof:
            #The send buffer needs to stay in scope until this callback has been called again
            self._buffer_send = self._src_file.read(self._length)
            
            try:
                
                handle = self._Trans.send_bytes(self._send_request,self._buffer_send,self._buffer_send == "", self.sendBytesCallback, cv)
                if not self._oldSendCbHandle == None:
                    self._Trans.free_callback(self._oldSendCbHandle)

                self._oldSendCbHandle = self._newSendCbHandle
                self._newSendCbHandle = handle
            except Exception, e:
                print e.args 
                sys.exit(-1)
            else:
                cv.acquire()
                self._done = 1
                cv.notify()
                cv.release()  
        return
    
    def getUrl(self):
        return self._url

    def setDest(self, dest):
        self._dest_file = dest

    def setSrc(self, src):
        self._src_file = src
        
    def run(self):
        self._condV.acquire()
        self._done = 0
        self._server.register_listen(self.registerListenCallback,self._condV)
        while self._done == 0:
            self._condV.wait()
        self._condV.release()
            
        self._done = 0
        self._condV.acquire()
        server_request = self._server.register_accept(self._reqAttr,self.registerAcceptCallback, self._condV)
        
        while self._done == 0:
            self._condV.wait()
        self._condV.release()

        self._server_request = self._server.authorize(server_request)
        self._recv_request = server_request
        
        if self._doRecv:
            self.receive()
        elif self._doSend:
            self.send()
        else:
            print "MUST SPECIFY do send or do recv"
            sys.exit(0)
            
    def receive(self):
        self._done = 0
        self._condV.acquire()
        self._Trans.receive_bytes(self._recv_request, self._buffer_recv,1 , self.receiveBytesCallback, self._condV)
        
        while self._done == 0:
            self._condV.wait()
        self._condV.release()
        del self._Trans
        self._dest_file.close()

    def send(self):
        self._buffer_send = self._src_file.read(self._length)
        self._send_request = self._server_request
        self._done = 0
        self._condV.acquire()
        self._newCbHandle = self._Trans.send_bytes(self._send_request,
                                                   self._buffer_send, self._buffer_send == "",
                                                   self.sendBytesCallback,self._condV)
        while self._done == 0:
            self._condV.wait()
        self._condV.release()
        del self._Trans
        self._src_file.close()
       
                
class TestClient(Thread):

    def __init__(self):
        Thread.__init__(self)
        self._client = TransferClient()
        self._condV = Condition(Lock())
        self._done = 1
        scheme = "http"
        self._reqAttr = TransferReqAttr(scheme)
        self._length = 1024
        self._buffer_recv = Buffer(self._length)
        self._Trans = TransferData()
        self._client_request = None
        self._oldSendCbHandle = None
        self._newSendCbHandle = None
        self._oldRecvCbHandle = None
        self._newRecvCbHandle = None
        self._src_file = None
        self._doSend = None
        self._doRecv = None
        self._doPut = None
        self._doGet = None
        self._doAppend = None

          
    def setSrc(self, srcFile):
        self._src_file = srcFile

    def setDest(self, destFile):
        self._dest_file = destFile
        
    def setUrl(self, url):
        self._url = url

    def registerPutCallback(self, cv, request):
        self.callbackSynchronization()

    def registerGetCallback(self, cv, request):
        self.callbackSynchronization()
        
    def callbackSynchronization(self):
        self._condV.acquire()
        self._done = 1
        self._condV.notify()
        self._condV.release()
              

    def sendBytesCallback(self,cv, request, bufHandle, bufLen, eof):
        if not eof:
            #The send buffer needs to stay in scope until this callback has been called again
            self._buffer_send = self._src_file.read(self._length)
            
            try:
                
                handle = self._Trans.send_bytes(self._send_request,self._buffer_send,self._buffer_send == "", self.sendBytesCallback, cv)
                if not self._oldSendCbHandle == None:
                    self._Trans.free_callback(self._oldSendCbHandle)

                self._oldSendCbHandle = self._newSendCbHandle
                self._newSendCbHandle = handle
            except Exception, e:
                print e.args
                sys.exit(-1)
            else:
                cv.acquire()
                self._done = 1
                cv.notify()
                cv.release()  
        return

    def receiveBytesCallback(self,cv, request, bufHandle, bufLen,  eof):
        self._dest_file.write(bufHandle)
        self._dest_file.flush()
        if not eof:
            try:
                handle = self._Trans.receive_bytes(self._recv_request,self._buffer_recv,1, self.receiveBytesCallback, cv)
                
                if not self._oldRecvCbHandle == None:
                    self._Trans.free_callback(self._oldRecvCbHandle)
                self._oldRecvCbHandle = self._newRecvCbHandle
                self._newRecvCbHandle = handle
            except Exception, e:
                print e.args
                sys.exit(-1)
        else:
            cv.acquire()
            self._done = 1
            cv.notify()
            cv.release()
        return
    
    def run(self):
        if self._doPut:
            self.put()
        elif self._doGet:
            self.get()
        elif self._doAppend:
            self.append()
        else:
            print "MUST SET client operation"
            sys.exit(0)

    def put(self):
        self._condV.acquire()
        self._done = 0
        client_request = self._client.register_put(self._reqAttr, self._url, self.registerPutCallback, self._condV)
        while self._done == 0:
            self._condV.wait()
        self._condV.release()
        self._send_request = client_request
        self.handleTransmission()

    def get(self):
        self._condV.acquire()
        self._done = 0
        client_request = self._client.register_get(self._reqAttr, self._url, self.registerGetCallback, self._condV)
        while self._done == 0:
            self._condV.wait()
        self._condV.release()
        self._recv_request = client_request
        self.handleTransmission()

    def append(self):
        self._condV.acquire()
        self._done = 0
        client_request = self._client.register_append(self._reqAttr, self._url, self.registerPutCallback, self._condV)
        
        while self._done == 0:
            self._condV.wait()
        self._condV.release()
        self._send_request = client_request
        self.handleTransmission()

    def handleTransmission(self):
        if self._doRecv:
            self.receive()
        elif self._doSend:
            self.send()
        else:
            print "MUST SPECIFY do send or do recv"
            sys.exit(0)
            
    def receive(self):
        self._done = 0
        self._condV.acquire()
        self._Trans.receive_bytes(self._recv_request, self._buffer_recv,1 , self.receiveBytesCallback, self._condV)
        
        while self._done == 0:
            self._condV.wait()
        self._condV.release()
        del self._Trans
        self._dest_file.close()
        
    def send(self):
        self._buffer_send = self._src_file.read(self._length)
        self._condV.acquire()
        self._done = 0
        self._Trans.send_bytes(self._send_request,self._buffer_send, self._buffer_send == "", self.sendBytesCallback,self._condV)  
        while self._done == 0:
            self._condV.wait()
        self._condV.release()
        del self._Trans
        self._src_file.close()


       
            
class GassTransferTestCase(unittest.TestCase):         
    """Test cases for the gassTransfer module"""
    def setUp(self):
        global config
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read(config_file)
    
   
    
    def checkGassGet(self):
       global config, config_file
       try:
           print "testing GASS Get with nonblocking calls "
           if config == None:
               config = ConfigParser.ConfigParser()
               config.read(config_file)
           
           src = config.get("gassTransfer", "src")
           dest = config.get("gassTransfer", "destGet") 
           server = TestServer()
           client = TestClient()
           dest_file = open(dest, "w")
           src_file = open(src, "r")
           server.setSrc(src_file)
           client.setDest(dest_file)
           server._doSend = 1
           client._doRecv = 1
           client._doGet = 1
           url = server.getUrl()
           url +=src
           client.setUrl(url)
           server.start()
           client.start()
           server.join()
           client.join()
           
       except GlobusException, ex:
           print ex.args
           print "TEST FAILED"
       print "DONE"

    def checkGassPut(self):
        print "testing GASS PUT with non blocking calls"
        try:
            
            global config, config_file
            if config == None:
                config = ConfigParser.ConfigParser()
                config.read(config_file)
           
            src = config.get("gassTransfer", "src")
            dest =  config.get("gassTransfer", "destPut") 

            server = TestServer()
            client = TestClient()
            dest_file = open(dest, "w")
            src_file = open(src, "r")
            server.setDest(dest_file)
            client.setSrc(src_file)
            url = server.getUrl()
            url +=dest
            client.setUrl(url)
            server._doRecv = 1
            client._doSend = 1
            client._doPut = 1
            server.start()
            client.start()
            server.join()
            client.join()
            
        except GlobusException, ex:
            print ex.args
            print "TEST FAILED"
        print "DONE"
    
    def checkGassAppend(self):
        print "testing GASS APPEND with non blocking calls"
        try:    
            global config, config_file
            if config == None:
                config = ConfigParser.ConfigParser()
                config.read(config_file)
            src = config.get("gassTransfer", "src")
            dest =  config.get("gassTransfer", "destAppend")
            server = TestServer()
            client = TestClient()
            dest_file = open(dest, "w")
            src_file = open(src, "r")
            server.setDest(dest_file)
            client.setSrc(src_file)
            url = server.getUrl()
            url +=dest
            client.setUrl(url)
            server._doRecv = 1
            client._doSend = 1
            client._doAppend = 1
            server.start()
            client.start()
            server.join()
            client.join()
            
        except GlobusException, ex:
            print ex.args
            print "TEST FAILED"
            
        print "DONE"

class GassTransferListenerTestCase(unittest.TestCase):

    def setUp(self):
        self._listener = TransferListener()
        self._done = 0
        
    def listenerCB(self,cv, listener):
        cv.acquire()
        self._done = 1
        cv.notify()
        cv.release()
        
    def checkListenerUrl(self):
        print "Checking Listener Url"
        url = self._listener.get_base_url()
        print "DONE"
        
    def checkCloseListener(self):
        print "checkCloseListener"
        cv = Condition(Lock())
        cv.acquire()
        self._listener.close_listener(self.listenerCB, cv)
        while self._done == 0:
            cv.wait()
        cv.release()
        print "DONE"

class GassTransferListenerAttrTestCase(unittest.TestCase):

    def setUp(self):
        self._listenerAttr = ListenerAttr()

    def checkListenerAttr(self):
        del self._listenerAttr
        
def makeTestSuite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TransferReqAttrTestCase, "check"))
    suite.addTest(unittest.makeSuite(GassTransferTestCase, "check"))
    suite.addTest(unittest.makeSuite(GassTransferListenerTestCase, "check"))
    suite.addTest(unittest.makeSuite(GassTransferListenerAttrTestCase, "check"))
    return suite

def main():
    unittest.main(defaultTest="makeTestSuite")


if __name__ == "__main__" : main()
