############################################################################
# Matt Rodriguez, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################

from pyGlobus.gramClient import GramClient, JOB_STATE_PENDING, JOB_STATE_ACTIVE
from pyGlobus.gramClient import JOB_STATE_FAILED, JOB_STATE_DONE, JOB_STATE_ALL, GramClientJobStateNames 
from pyGlobus.gramClient import GramClientAttr
from pyGlobus.security import GSSCred  
from pyGlobus.util import GlobusException
import ConfigParser, unittest, time
from threading import * 

config = None # ConfigParser object


def func(cv, contact, state, error, error2='default'): 
    str =  "**ENTERING PYTHON CALLBACK**    state="
    if state!=0: 
        str += GramClientJobStateNames[state] 

    if state == JOB_STATE_PENDING: pass
    elif state == JOB_STATE_ACTIVE: pass
    elif state == JOB_STATE_FAILED:
        if cv:
            cv.acquire()  
            cv.notify()
            cv.release()
            
    elif state == JOB_STATE_DONE:
        if cv:
            cv.acquire()
            cv.notify()
            cv.release() 
    return


        
#############################################################################
#  Unit test for the GramClient Module:
#  Given a remote machine and an rsl command, the test function creates a
#  GramClient object and which sends the rsl message to the remote machine
#  and if all goes well then the command is executed.
#
##############################################################################


class GramClientTestCase(unittest.TestCase):
    "Test case for the GramClient module"
    
    def checkGramClient(self):  
        global config
        print  "Testing gramClient, submitting a job"
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read("config.txt")
            
        rm = config.get("gramClient", "rm")
        rsl = config.get("gramClient", "rsl").strip('"')
        
        condV = Condition(Lock())
        try:
            gramClient = GramClient() 
            callbackContact = gramClient.set_callback(func, condV)
            print "submitting job description %s to %s" % (rsl, rm)
            condV.acquire() 
            jobContact = gramClient.submit_request(rm, rsl, JOB_STATE_ALL, callbackContact)
            condV.wait()
            condV.release()
            gramClient.remove_callback(callbackContact) 
        except GlobusException, ex:
            print "Exception occured during testing" 
            print ex
            print "TEST FAILED"
        else:
            print "DONE"

    def check_register_check_status(self):
        global config
        print  "Testing gramClient, submitting a job then checking the status"
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read("config.txt")
            
        rm = config.get("gramClient", "rm")
        rsl = config.get("gramClient", "rsl").strip('"')

        condV = Condition(Lock())
        try:
            gramClient = GramClient()
            gcAttr = GramClientAttr()
            assert not gramClient == None, "Error creating the GramClient object"
            callbackContact = gramClient.set_callback(func, condV)
            condV.acquire()
            print "submitting job description %s to %s" % (rsl, rm)
            jobContact = gramClient.submit_request(rm, rsl, JOB_STATE_ALL, callbackContact)
            gramClient.register_check_status(jobContact, gcAttr, func, None)            
            condV.wait()
            condV.release()
            gramClient.remove_callback(callbackContact)
        except GlobusException, ex:
            print "Exception occured during testing" 
            print ex
            print "TEST FAILED"
        else:
            print "DONE"
            
                        
    def check_unregister_callback(self):
        global config
        print  "Testing gramClient, submitting a job, registering/unregistering a callback"
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read("config.txt")            
        rm = config.get("gramClient", "rm")
        rsl = config.get("gramClient", "sleeper_rsl")        
        condV = Condition(Lock())
        try:
            gramClient = GramClient()
            assert not gramClient == None, "Error creating the GramClient object"
            callbackContact = gramClient.set_callback(func, condV)
            condV.acquire()
            print "submitting job description %s to %s" % (rsl, rm)
            jobContact = gramClient.submit_request(rm, rsl, JOB_STATE_ALL, callbackContact)
            gramClient.register_callback(jobContact, 0, callbackContact) 
            gramClient.unregister_callback(jobContact, callbackContact)
            condV.wait()
            condV.release()
            gramClient.remove_callback(callbackContact)
        except GlobusException, ex:
            print "Exception occured during testing" 
            print ex
            print "TEST FAILED"
        else:
            print "DONE"
            
            
    def check_register_callback_registration(self):
        global config
        print  "Testing gramClient, asynchronous callback registration"
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read("config.txt")

        rm = config.get("gramClient", "rm")
        rsl = config.get("gramClient", "sleeper_rsl")
        
        condV = Condition(Lock())
        try:
            gramClient = GramClient()            
            callbackContact = gramClient.set_callback(func, condV)
            gcAttr = GramClientAttr()
            cred = GSSCred()
            gcAttr.set_credential(cred) 
            condV.acquire()
            t1 = time.time()
            jobContact = gramClient.submit_request(rm, rsl, JOB_STATE_ALL, callbackContact)
            
            status = gramClient.register_callback_registration(jobContact, 0, 
                                                               callbackContact, gcAttr, func, None)
            
            condV.wait()
            condV.release()
            gramClient.remove_callback(callbackContact)
            t2 = time.time() 
            if t2-t1>20:
                print "there was a timeout problem in the callback registration test"
                print "The job failed to complete within 20 seconds"
                print "TEST FAILED"
                
        except GlobusException, s:
	    if str(s)=="globus_gram_client_attr_set_credential is unimplemented in this version of gram_client":
		print 'expected'
	    else:
	    	print 'unexpected!'
		assert 0, str(s)
        
        print "TEST PASSED"
   
    def check_register_callback(self): 
        global config
        print  "Testing gramClient, registering callback"
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read("config.txt")            
        rm = config.get("gramClient", "rm")
        rsl = config.get("gramClient", "sleeper_rsl")        
        condV = Condition(Lock())
        try:
            gramClient = GramClient() 
            callbackContact = gramClient.set_callback(func, condV)
            condV.acquire()
            jobContact = gramClient.submit_request(rm, rsl, JOB_STATE_ALL, callbackContact)
            gramClient.register_callback(jobContact, 0, callbackContact) 
            condV.wait()
            condV.release()
            gramClient.remove_callback(callbackContact) 
        except GlobusException, ex:
            print "Exception occured during testing" 
            print ex
            print "TEST FAILED"
        else:
            print "TEST PASSED"
        

    def check_check_status(self):
        global config
        print "Testing gramClient, checking the status of a job" 
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read("config.txt")
            
        rm = config.get("gramClient", "rm")
        rsl = config.get("gramClient", "sleeper_rsl")
        
        condV = Condition(Lock())
        try:
            gramClient = GramClient()
            assert not gramClient == None, "Error creating the GramClient object"
            callbackContact = gramClient.set_callback(func, condV)
            condV.acquire()
            jobContact = gramClient.submit_request(rm, rsl, JOB_STATE_ALL, callbackContact)
            status, tcode = gramClient.check_status(jobContact)
            condV.wait()
            condV.release()
            gramClient.remove_callback(callbackContact)
        except GlobusException, ex:
            print "Exception occured during testing" 
            print ex
            print "TEST FAILED"
        else:
            print "TEST PASSED"
            
        
    def checkRefreshCredentials(self):
        global config
        print  "Testing gramClient, refresh credentials"
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read("config.txt")
            
        rm = config.get("gramClient", "rm")
        rsl = config.get("gramClient", "sleeper_rsl")
        
        condV = Condition(Lock())
        try:
            gramClient = GramClient() 
            callbackContact = gramClient.set_callback(func, condV)
            condV.acquire()
            jobContact = gramClient.submit_request(rm, rsl, JOB_STATE_ALL, callbackContact)

            cred = GSSCred()
            jcontact = gramClient.refresh_credentials(jobContact,cred)
            condV.wait()
            condV.release()
            gramClient.remove_callback(callbackContact)

        except GlobusException, ex:
            if str(s).count('unimplemented') > 0:
	        print "refresh credentials is umimplemented in this version of GRAM"	
	    else: 
                print "TEST FAILED" 
        print "DONE"
    
    def checkVersion(self):
        """
        makes sure gramClient.version() is 2
        """
        global config
        print  "Checking the version of the gramClient protocol"
        try:
            gramClient = GramClient()
            version = gramClient.version() 
            if version == 2:
                print "TEST PASSED"
            else:
                print "TEST FAILED"
            
        except GlobusException, ex:
            print "Exception occured during testing"
            print ex
            print "TEST FAILED"
        

    def checkCancel(self):
        global config
        print "Testing gramClient, submitting a job then canceling it" 
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read("config.txt")

        rm = config.get("gramClient", "rm")
        rsl = config.get("gramClient", "sleeper_rsl")
        
        condV = Condition(Lock())
        try:
            gramClient = GramClient() 
            callbackContact = gramClient.set_callback(func, condV)
            condV.acquire()
            t1 = time.time()
            print "submitting job description %s to %s" % (rsl, rm)
            jobContact = gramClient.submit_request(rm, rsl, JOB_STATE_ALL, callbackContact)
            gramClient.cancel_job(jobContact)            
            condV.wait()
            condV.release()
            gramClient.remove_callback(callbackContact)
            t2 = time.time()
            if t2-t1>20:
                print "there was a timeout problem in the cancel test"
                print "The job was not canceled within 20 seconds"
                print "TEST FAILED"
            
        except GlobusException, ex:
            print "Exception occured during testing"
            print ex
            print "TEST FAILED"
        
        print "DONE"     
            

    def checkRegisterRefreshCredentials(self):
        global config
        print "Testing gramClient, asynchronously refreshing credentials" 
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read("config.txt")

        rm = config.get("gramClient", "rm")
        rsl = config.get("gramClient", "sleeper_rsl")        
        
        condV = Condition(Lock())
        try:
            gramClient = GramClient() 
            callbackContact = gramClient.set_callback(func, condV)
            gcAttr = GramClientAttr()

            condV.acquire()
            t1 = time.time()
            jobContact = gramClient.submit_request(rm, rsl, JOB_STATE_ALL, callbackContact)
            nocredconst = GSSCred(handle=None) 
            gramClient.register_refresh_credentials(jobContact, nocredconst, gcAttr, func, None)
            
            condV.wait()
            condV.release()
            gramClient.remove_callback(callbackContact)
            t2 = time.time()
            if t2-t1>20:
                print "there was a timeout problem in the cancel test"
                print "The job was not canceled within 20 seconds"
                print "TEST FAILED"
            

            
        except GlobusException, ex:
            if str(s).count('unimplemented') > 0:
		print 'expected'
	    else:
	    	print 'unexpected!'
                assert 0, ex.args
        
        print "DONE"     
        
    def checkCancelNonBlocking(self):
        """
        Tests register_cancel_job and submit_request
        """
        global config
        print  "Testing gramClient, asynchronously canceling a job"
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read("config.txt")

        rm = config.get("gramClient", "rm")
        rsl = config.get("gramClient", "sleeper_rsl")
        
        condV = Condition(Lock())
        try:
            gramClient = GramClient()            
            callbackContact = gramClient.set_callback(func, condV)
            gcAttr = GramClientAttr() 
            cred = GSSCred()
            gcAttr.set_credential(cred) 
        except GlobusException, s:
            if str(s).count('unimplemented') > 0:
                print "An unimplemented exception was raised."
                print "The features that are tested are not in this version of the gramClient libraries."
                print "DONE"
                return
	    else:
                print s
                print "TEST FAILED"
                self.fail("An unexpected exception was raised during testing")
        try:    
            condV.acquire()
            t1 = time.time() 
            jobContact = gramClient.submit_request(rm, rsl, JOB_STATE_ALL, callbackContact)
            gramClient.register_cancel_job(jobContact, gcAttr, func, None) 
            condV.wait()
            condV.release()
            gramClient.remove_callback(callbackContact)
            t2 = time.time() 
            if t2-t1>20:
                print "there was a timeout problem in the cancel test"
                print "The job was not canceled within 20 seconds"
                print "TEST FAILED" 
        except GlobusException, s: 
            print s
            print "TEST FAILED"
            self.fail("An unexpected exception was raised during testing") 
        print "DONE"
    
    def checkPing(self):
        global config
        print "Testing gramClient, ping" 
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read("config.txt")
        rm = config.get("gramClient", "rm")
        print "Pinging remote host: ", rm
        try:
            gramClient = GramClient() 
            gramClient.ping(rm)            
        except GlobusException, ex:
            print "Exception occured during testing"
            print ex
            print "TEST FAILED"
        else:
            print "TEST PASSED"
            
         

    def checkGramRegister_Sumbit_Request(self):
        global config
        print  "Testing gramClient, submitting a job asynchronously"
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read("config.txt")
            
        rm = config.get("gramClient", "rm")
        rsl = config.get("gramClient", "rsl")
        rsl = rsl.strip('"')
        condV = Condition(Lock())
        try:
            gramClient = GramClient() 
            gcAttr = GramClientAttr()
            cred = GSSCred()
            gcAttr.set_credential(cred) 
            callbackContact = gramClient.set_callback(func, condV)
            condV.acquire()
            jobContact = gramClient.register_submit_request(rm, rsl, 
                                                        JOB_STATE_ALL, callbackContact,
                                                        gcAttr, func, None)
           
            condV.wait()
            condV.release()
            gramClient.remove_callback(callbackContact) 
        except GlobusException, s:
            if str(s).count('unimplemented') > 0:
                print "An unimplemented exception was raised."
                print "The features that are tested are not in this version of the gramClient libraries."
            else:
                print s
                print "TEST FAILED"
                self.fail("An unexpected exception was raised during testing") 
         
        print "DONE"

class GramClientAttrTestCase(unittest.TestCase):
    
    def TestGetSets(self):
	self.cred = GSSCred()
        self.cred.acquire_cred()
        print "Testing GramClient Attribute"

	try:
	    gca  = GramClientAttr()
	    gca.set_credential(self.cred)  # tests set_credential()
	except GlobusException, s:
	    if str(s)=="globus_gram_client_attr_set_credential is unimplemented in this version of gram_client":
		print 'expected'
	    else:
	    	print 'unexpected!'
		assert 0, str(s)

	try: gca.get_credential()  # tests get_credential()
	except GlobusException, s:
            if str(s)=="globus_gram_client_attr_get_credential is unimplemented in this version of gram_client":
                print 'expected'
            else: print str(s)

        try: 
            handle = gca.get_handle()
        except GlobusException, s:
            if str(s)=="globus_gram_client_attr_get_credential is unimplemented in this version of gram_client":
                print 'expected'
            else: 
                print str(s)
            
        print "DONE"	


def makeTestSuite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(GramClientAttrTestCase,"Test")) 
    suite.addTest(unittest.makeSuite(GramClientTestCase, "check")) 
    return suite

def main():
    unittest.main(defaultTest="makeTestSuite")


if __name__ == "__main__" : main()
    
                  

