############################################################################
# Matt Rodriguez, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################

from pyGlobus.xio import GsiSocket as XioSocket
from pyGlobus.xio import XioSocketException
import string, md5, base64 
import threading 
import unittest

class TestClientRead(threading.Thread):
    def run(self):
        sock = XioSocket()
        sock.settimeout(3)
        sock.connect(("localhost", 6768)) 
        try: 
            sock.recv(1024) 
        except XioSocketException, ex:
            print ex
            print "TEST PASSED"
        else:
            print "TEST FAILED"
        sock.close()

class TimeoutTestCases(unittest.TestCase):
    """
    These test cases demonstrate a memory leak
    The server handle is not being free'd.
    """
    def checkAcceptTimeout(self):
        print "testing accept timeouts"
        try:
            sock = XioSocket()
            sock.settimeout(3)
            sock.bind(("localhost",6768))
            sock.listen(5) 
            newSock,endpoint = sock.accept()
        except XioSocketException, ex:
            print ex
            print "TEST PASSED" 
        else:
            print "TEST FAILED"

        sock.close()

    def checkReadTimeout(self):
        print "testing read timeouts"
        sock = XioSocket()
        sock.bind(("localhost",6768)) 
        sock.listen(5)
        client = TestClientRead()
        client.start()
        client.join()
        newSock, endpoint = sock.accept()
        newSock.close()
        sock.close()

class SocketOperations(unittest.TestCase):
   
    def checkSocketBind(self):
        print "testing the bind operation"
        try: 
            sock = XioSocket() 
            sock.bind(("localhost", 1234)) 
            sock.close()
        except Exception:
            print "TEST FAILED"
        else:
            print "TEST PASSED"
    
    def checkSocketNameNoListen(self):
        print "testing the getsockname operation"
        try:
            sock = XioSocket()
            sock.bind(("localhost", 1234))
            localname = sock.getsockname()
            print "localname: ", localname
            sock.close()
        except Exception:
            print "TEST FAILED"
            raise
        else:
            print "TEST PASSED"
    
    def checkSocketNameListen(self):
        print "testing the getsockname operation"
        try:
            sock = XioSocket()
            sock.bind(("localhost", 1234))
            sock.listen(5)
            localname = sock.getsockname()
            print "localname: ", localname
            sock.close()
        except Exception:
            print "TEST FAILED"
            raise
        else:
            print "TEST PASSED"
        
    def checkSocketBindBadPort(self):
        print "testing the bind operation with bad port argument"
        try:
            sock = XioSocket()
            sock.bind(("localhost", "1234"))
            sock.close()
        except TypeError:
            print "TEST PASSED"
        else:
            print "TEST FAILED"
        
    def checkSocketBindBadTuple(self):
        print "testing the bind operation with bad tuple argument"
        try:
            sock = XioSocket()
            sock.bind(["localhost", "1234"])
            sock.close()
        except TypeError:
            print "TEST PASSED"
        else:
            print "TEST FAILED"

    def checkSocketListenNoBind(self):
        print "testing the listen operation without bind"
        try:
            sock = XioSocket()
            sock.listen(3)
            sock.close()
        except XioSocketException:
            print "TEST FAILED"
        else:
            print "TEST PASSED"
           
class SocketEcho(threading.Thread):
    def __init__(self, port): 
        threading.Thread.__init__(self)
        self.sock = XioSocket()
        self.port = port 

    def run(self):
        try:
            
            self.sock.connect(("localhost", self.port))
            addr, port_con = self.sock.getsockname() 
            data = self.sock.recv(20) 
            self.data = data
            self.sock.send(data)
            self.sock.close() 
            if port_con != self.port or addr != '127.0.0.1':
                print "Problem with getsockname call"
                print "TEST FAILED"
        except XioSocketException:
            raise 


class SocketIOOperations(unittest.TestCase):
    
    def setUp(self):
        self.sock = XioSocket()
        self.port = 1235
        self.sock.bind(("localhost", self.port)) 
        self.sock.listen(5) 
        self.client_sock = SocketEcho(self.port)
        self.client_sock.start()

        
    def checkServer(self):
        print "Testing send and recv"
        try: 
            new_sock,end_point = self.sock.accept() 
            data_sent = new_sock.send("FooBar FooBar FooBar") 
            data  = new_sock.recv(20) 
            self.client_sock.join()
            self.sock.close() 
        except XioSocketException:
            raise
        else:
            print "DONE"
    
    def checkServerNames(self):
        print "Testing send,recv and getsockname"
        try: 
            new_sock,end_point = self.sock.accept() 
            localname = new_sock.getsockname() 
            data_sent = new_sock.send("FooBar FooBar FooBar") 
            data  = new_sock.recv(20) 
            self.client_sock.join()
            self.sock.close() 
        except XioSocketException:
            raise
        else:
            print "DONE"

class SocketRecvBigData(threading.Thread):
    def __init__(self, port):
        threading.Thread.__init__(self)
        self.sock = XioSocket()
        self.port = port 
    
    def run(self):
        try: 
            self.sock.connect(("localhost", self.port))
            name = self.sock.getsockname()
            total_len = 1049599 
            bytes_read = 0 
            data_list = []
            while bytes_read < total_len:
                data = self.sock.recv(pow(2,20))
                data_list.append(data)
                bytes_read += len(data)
                

            big_string = "".join(data_list)
            hasher = md5.new()
            hasher.update(big_string)
            hash_value = base64.encodestring(hasher.digest()) 
            self._hash_value = hash_value
            self.sock.close()
            
        except XioSocketException:
            print "exception raised in SocketRecvBigData"
            print str(ex)
            print "TEST FAILED" 
            

    def gethashvalue(self):
        return self._hash_value

class SocketIOSendAll(unittest.TestCase): 
    def checkBigDataSend(self):
        print "Testing Big Data Send and Recv"
        try:
            self.sock = XioSocket()
            self.port = 1235
            self.sock.bind(("localhost", self.port)) 
            self.sock.listen(5) 
            self.client_sock = SocketRecvBigData(self.port)
            self.client_sock.start() 
            new_sock,end_point = self.sock.accept() 
            data_send_piece = "" 
            for i in range(pow(2,7)):
                data_send_piece += '12345678'
            data_send_list = []
            for i in range(pow(2,10)):
                data_send_list.append(data_send_piece)
            big_string = string.join(data_send_list)        
            hasher = md5.new()
            hasher.update(big_string)
            hash_value = base64.encodestring(hasher.digest()) 
            data_sent = new_sock.sendall(big_string) 
            self.client_sock.join()
            recvd_hash = self.client_sock.gethashvalue()
            if recvd_hash == hash_value:
                print "TEST PASSED"
            else:
                print "TEST FAILED"
        except XioSocketException, ex:
            print "exception raised"
            print str(ex)
            print "TEST FAILED" 
            
class SocketTestOpts(unittest.TestCase): 
    
    def setUp(self):
        self.sock = XioSocket()


    def checkPort(self):
        print "Testing setting port"
        try:
            self.sock.setsockopt("GLOBUS_XIO_TCP_SET_PORT", 3333)
            vals = self.sock.getsockopt("GLOBUS_XIO_TCP_GET_PORT")
            if vals == 3333:
                print "TEST PASSED"
        except XioSocketException, ex:
            print "exception raised"
            print str(ex)
            print "TEST FAILED" 
        
    def checkConnectRange(self):
        print "Testing setting Connect Range"
        try:
            self.sock.setsockopt("GLOBUS_XIO_TCP_SET_CONNECT_RANGE", 4000, 5000)
            retvals = self.sock.getsockopt("GLOBUS_XIO_TCP_GET_CONNECT_RANGE")
            if retvals == (4000,5000):
                print "TEST PASSED"
        except XioSocketException, ex:
            print "exception raised"
            print str(ex)
            print "TEST FAILED" 
        
    def checkReuseAddr(self):
        print "Testing setting reuseaddr"
        try:
            self.sock.setsockopt("GLOBUS_XIO_TCP_SET_REUSEADDR", True)
            retvals = self.sock.getsockopt("GLOBUS_XIO_TCP_GET_REUSEADDR")
            if retvals == True:
                print "TEST PASSED"
        except XioSocketException, ex:
            print "exception raised"
            print str(ex)
            print "TEST FAILED" 
        
    def checkInterface(self):
        print "Testing getting/setting interface"
        try: 
            self.sock.bind(("localhost", 12345))
            self.sock.listen(5)
            self.sock.setsockopt("GLOBUS_XIO_TCP_SET_INTERFACE", "eth0")
            retvals = self.sock.getsockopt("GLOBUS_XIO_TCP_GET_INTERFACE")
            if retvals == "eth0":
                print "TEST PASSED"
            self.sock.close()
        except XioSocketException, ex:
            print "exception raised"
            print str(ex)
            print "TEST FAILED" 
         
    def checkLinger(self):
        print "Testing getting/setting linger"
        try: 
            self.sock.setsockopt("GLOBUS_XIO_TCP_SET_LINGER", True, 300)
            retvals = self.sock.getsockopt("GLOBUS_XIO_TCP_GET_LINGER")
            
            if retvals == (True,300):
                print "TEST PASSED"
            else:
                print "TEST FAILED"
           
        except XioSocketException, ex:
            print "exception raised"
            print str(ex)
            print "TEST FAILED"
    
    def checkLingerBadArguments(self):
        print "Testing setting interface with bad arguments"
        try: 
            self.sock.setsockopt("GLOBUS_XIO_TCP_SET_LINGER", True, True) 
           
        except XioSocketException, ex: 
            print "TEST PASSED"
        else:
            print "TEST FAILED"
    
    def checkInterfaceBadArg(self):
        print "Testing setting interface with a bad argument"
        try: 
            self.sock.setsockopt("GLOBUS_XIO_TCP_SET_INTERFACE", {}) 
        except XioSocketException, ex: 
            print "TEST PASSED" 
        else:
            print "TEST FAILED"
    
    def checkInterfaceTooManyArgs(self):
        print "Testing setting interface with too many args"
        try: 
            self.sock.setsockopt("GLOBUS_XIO_TCP_SET_INTERFACE",True,True,True) 
        except XioSocketException, ex: 
            print "TEST PASSED" 
        else:
            print "TEST FAILED"

class SocketTestNotImplemented(unittest.TestCase):
    def setUp(self):
        self.sock = XioSocket()

    def checkDup(self):
        print "Testing unimplemented dup method"
        try:
            self.sock.dup()
        except NotImplementedError:
            print "TEST PASSED"
        else:
            print "TEST FAILED"

    def checkMakefile(self):
        print "Testing unimplemented makefile method"
        try:
            self.sock.makefile()
        except NotImplementedError:
            print "TEST PASSED"
        else:
            print "TEST FAILED"

    def checkShutdown(self):
        print "Testing unimplemented shutdown method"
        try:
            self.sock.shutdown(1)
        except NotImplementedError:
            print "TEST PASSED"
        else:
            print "TEST FAILED"

    def checkFileno(self):
        print "Testing unimplemented fileno method"
        try:
            self.sock.fileno()
        except NotImplementedError:
            print "TEST PASSED"
        else:
            print "TEST FAILED"

def makeTestSuite():
    suite = unittest.TestSuite()
    #suite.addTest(unittest.makeSuite(TimeoutTestCases, "check"))
    #suite.addTest(unittest.makeSuite(SocketOperations, "check")) 
    suite.addTest(unittest.makeSuite(SocketIOOperations, "check")) 
    #suite.addTest(unittest.makeSuite(SocketIOSendAll, "check")) 
    #suite.addTest(unittest.makeSuite(SocketTestOpts, "check"))
    #suite.addTest(unittest.makeSuite(SocketTestNotImplemented, "check"))
    return suite

def main():
    unittest.main(defaultTest="makeTestSuite")


if __name__ == "__main__" : main()
