############################################################################
# Matt Rodriguez, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################
from pyGlobus import rlsClientc
from pyGlobus.rlsClient import RlsClient
from pyGlobus.rlsClient import RlsAttr
from pyGlobus.rlsClient import RlsClientException
from pyGlobus.rlsClient import RlsAttrException
import unittest,ConfigParser, time

config = None
config_file ="config.txt"

class RlsClientTest(unittest.TestCase):
    """
       Parent class which has the overrides the setUp and
       tearDown methods that all of the children testing classes use
       """

    def setUp(self):
	global config
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read(config_file)
	url = config.get("rlsClient", "url")
	self._client = RlsClient(url)

    def tearDown(self):
	del self._client

class RlsClientTestCaseCreate(RlsClientTest):
    """Unittest for the rlsClient module"""
    
    def check_lrc_create_lfn(self):
	lfn = config.get("rlsClient", "lfn")
	pfn = config.get("rlsClient", "pfn1")
	try:
	    self._client.lrc_create_lfn(lfn, pfn)
	except RlsClientException, ex:
	     print "Caught Exception"
	     print ex

class RlsClientTestCaseTest(RlsClientTest):

    def check_lrc_add(self):
	lfn = config.get("rlsClient", "lfn")
	pfn = config.get("rlsClient", "pfn2")
	try:
	    self._client.lrc_add(lfn, pfn)
	except RlsClientException, ex:
	     print "Caught Exception"
	     print ex    

    def check_lrc_get_lfn_wc(self):
	regExp = config.get("rlsClient", "regExp")
	try:
	    dict = self._client.lrc_get_lfn_wc(regExp)
	except RlsClientException, ex:
	     print "Caught Exception"
	     print ex.args
             print "TEST FAILED"
             return
    def check_lrc_get_pfn_wc(self):
	regExp = config.get("rlsClient", "regExp")
	try:
	    dict = self._client.lrc_get_pfn_wc(regExp)
	except RlsClientException, ex:
	     print "Caught Exception"
	     print ex
             print "TEST FAILED"
	     return
	
    def check_lrc_get_lfn(self):
	pfn = config.get("rlsClient", "pfn1")
	try:
	    list = self._client.lrc_get_lfn(pfn)
	except RlsClientException, ex:
	    print "Caught Exception"
	    print ex.args
            print "TEST FAILED"
	    return
	print list

    def check_lrc_exists(self):
	fn = config.get("rlsClient", "badFn")
	try:
	    exists = self._client.lrc_exists(fn,rlsClientc.globus_rls_obj_lrc_pfn)
	except RlsClientException, ex:
	    print "Caught Exception"
	    print ex
	    return
	if not exists:
            pass
        else:
            assert 0, "test failed!"
            
class LrcRlsAttrTestCase(RlsClientTest):

    def setUp(self):
	RlsClientTest.setUp(self)
	self._attr = RlsAttr()
	self._attr.set_name("testAttr")
	self._attr.set_type(rlsClientc.globus_rls_attr_type_str)
	self._attr.set_objtype(rlsClientc.globus_rls_obj_lrc_lfn)
	self._attr.set_val("Attribute string")
	try:
	    self._client.lrc_attr_create(self._attr)
	except RlsClientException, ex:
	    print "Caught Exception in setUp"
	    print ex

	lfn = config.get("rlsClient", "lfn")
	try:
	    self._client.lrc_attr_add(lfn, self._attr)
	except RlsClientException, ex:
	    print "Caught Exception"
	    print ex
	    
	
    def tearDown(self):
	lfn = config.get("rlsClient", "lfn")
	try:
	    self._client.lrc_attr_remove(lfn, self._attr)
	except RlsClientException, ex:
	    print "Caught Exception"
	    print ex
	    return
	print "attribute removed"
	try:
	    self._client.lrc_attr_delete(self._attr)
	except RlsClientException, ex:
	    print "Caught Exception"
	    print ex
	RlsClientTest.tearDown(self)
	

    def checkLrcAttrSearch(self):
	print "testing searchin for an RLS attribute in the LRC"
	attr = RlsAttr()
	attr.set_name("testAttr")
	attr.set_objtype(rlsClientc.globus_rls_obj_lrc_lfn)
	attr.set_val("testAttr")
	lfn =  config.get("rlsClient", "lfn") 
	try:
	    dict = self._client.lrc_attr_search(attr,rlsClientc.globus_rls_attr_op_all)
	except RlsClientException, ex:
	    print "Caught Exception"
	    print ex
            print "TEST FAILED"
            return
	print "Search dictionary: ", dict
	print "TEST PASSED"

    def checkLrcAttrGet(self):
	print "testing getting an rls attribute in the LRC"
	objtype = self._attr.get_objtype()
	name = self._attr.get_name()
	try:
	    list = self._client.lrc_attr_get(objtype, name)
	except RlsClientException, ex:
	    print "Caught Exception"
	    print ex
            print "TEST FAILED"
	    return
	for attr in list:
	    print "attr name: ", attr.get_name()
        print "TEST PASSED"
        
    def checkLrcAttrGetValue(self):
	print "testing retrieving an attributes value in the LRC"
	objtype = self._attr.get_objtype()
	name = self._attr.get_name()
	fn =  config.get("rlsClient", "lfn")

	try:
	    list = self._client.lrc_attr_value_get(fn,objtype, name)
	except RlsClientException, ex:
	    print "Caught Exception"
	    print ex
            print "TEST FAILED"
	    return
	for attr in list:
	    print "attr name: ", attr.get_name()
        print "TEST PASSED"

	
class RlsClientTestCaseClear(RlsClientTest):
             
    def check_lrc_delete(self):
        
	pfn = config.get("rlsClient", "pfn1")
	lfn =  config.get("rlsClient", "lfn")
	try:
	    self._client.lrc_delete(lfn,pfn)
	except RlsClientException, ex:
	    print "Caught Exception"
	    print ex
            print "TEST FAILED"
	    return
        print "TEST PASSED"

    def check_lrc_clear(self):
	print "Testing clearing the LRC of all LFNs"
	try:
	    self._client.lrc_clear()
	except RlsClientException, ex:
	    print "Caught Exception"
	    print ex
            print "TEST FAILED"
            return
        print "TEST PASSED"
        
class RlsClientTestCaseRli(RlsClientTest):

    def setUp(self):
	RlsClientTest.setUp(self)
	url = config.get("rlsClient", "url")
	try:
	    self._client.lrc_rli_add(url, "", 1)
	except RlsClientException, ex:
	   print "Caught Exception in setUp"
	   print ex.args
	lfn = config.get("rlsClient", "lfn")
	pfn = config.get("rlsClient", "pfn1")
	try:
	    self._client.lrc_create_lfn(lfn, pfn)
	except RlsClientException, ex:
	     print "Caught Exception in setUp, creating"
	     print ex.args
             
        lfn = config.get("rlsClient", "lfn")
	pfn = config.get("rlsClient", "pfn2")
	try:
	    self._client.lrc_add(lfn, pfn)
	except RlsClientException, ex:
	     print "Caught Exception"
	     print ex    
        
    def tearDown(self):
	url = config.get("rlsClient", "url")
	try:
	    self._client.lrc_rli_delete(url)
	except RlsClientException, ex:
	    print "Caught Exception"
	    print ex
	
        print "Test lrc_clear"
	try:
	    self._client.lrc_clear()
	except RlsClientException, ex:
	    print "Caught Exception"
	    print ex
        RlsClientTest.tearDown(self)
    
    def checkLrcRliList(self):
	print "Testing listing the RLI urls that the LRC sends updates"
	try:
	    dict = self._client.lrc_rli_list()
	except RlsClientException, ex:
	    print "Caught Exception"
	    print ex
            print "TEST FAILED"
            return
	print "dictionary: ", dict
        print "TEST PASSED"

    def checkLrcRliInfo(self):
	print "Testing retrieving info about the RLIs that the LRC sends updates"
	url = config.get("rlsClient", "url")
	try:
	    dict = self._client.lrc_rli_info(url)
	except RlsClientException, ex:
	    print "Caught Exception"
	    print ex
            print "TEST FAILED"
            return
	print "dictionary: ", dict
        print "TEST PASSED"

    def checkLrcRliGetPart(self):
	print "Testing retrieving RLI update partitions"
        url = config.get("rlsClient", "url")
	try:
	    list = self._client.lrc_rli_get_part(url)
	except RlsClientException, ex:
	    print "Caught Exception"
	    print ex
            print "TEST FAILED"
            return
	print "list is: ",list
        print "TEST PASSED"

    def checkLrcRliExists(self):
        print "Testing existence of lfn in the rli"
        lfn = config.get("rlsClient", "lfn")
        exists = 0
	try:
	    exists = self._client.rli_lfn_exists(lfn)
	except RlsClientException, ex:
	    print "Caught Exception"
	    print ex

	if exists == 1:
	    print "TEST PASSED"
	else:
            print "TEST FAILED"
	

    def checkRliGetLrc(self):
	print "testing  getting LRC from a RLI"
        lfn = "testLfn"
	pfn = "/etc/shells"
	try:
	    self._client.lrc_add(lfn, pfn)
	except RlsClientException, ex:
	     print "Caught Exception"
	     print ex  

	print "lfn Added"
	try:
	    list = self._client.rli_get_lrc(lfn)
	except RlsClientException, ex:
	    print "Caught Exception"
	    print ex
            print "TEST FAILED"
	    return
	print "RLI LRC list", list
        print "TEST PASSED"
        
    def checkRliGetLrcWc(self):
        print "Testing getting LRC from a RLI given an LFN pattern"
	url = config.get("rlsClient", "url")
	
	pattern = 'testLfn'
	try:
	    dict = self._client.rli_get_lrc(pattern)
	except RlsClientException, ex:
	    print "Caught Exception"
	    print ex
	    print "TEST FAILED"
            return
	print "RLI LRC dict", dict
        print "TEST PASSED"
        
class RlsAttrTestCase(unittest.TestCase):

    def setUp(self):
	self._attr = RlsAttr()

    def tearDown(self):
	print "DONE"
	
    def checkSetAndGetName(self):
	print "testing checkSetAndGetName" 
	name = "testAttr"
	self._attr.set_name(name)
	newName = self._attr.get_name()
	if name == newName:
	    print "TEST PASSED"
        else:
            print "TEST FAILED"

    def checkSetAndGetObjType(self):
	print "testing checkSetAndGetObjType"
	type = rlsClientc.globus_rls_obj_lrc_lfn
	self._attr.set_type(type)
	newType = self._attr.get_type()
	if type == newType:
	    print "TEST PASSED"
	else:
            print "TEST FAILED"
            
    def checkSetAndGetVal(self):
	print "testing checkSetAndGetVal"
	valStr = "Cream"
	self._attr.set_val(valStr)
	newValStr = self._attr.get_val()
	if valStr == newValStr:
	    print "Setting String val works"
            print "TEST PASSED"
        else:
            print "TEST FAILED"
	valFloat = 1.5
	self._attr.set_val(valFloat)
	newValFloat = self._attr.get_val()
	if valFloat == newValFloat:
	    print "Setting Float works"
            print "TEST PASSED"
        else:
            print "TEST FAILED"

	valInt = 5
	self._attr.set_val(valInt)
	newValInt = self._attr.get_val()
	if valInt == newValInt:
	    print "Setting Int works"
	    print "TEST PASSED"
        else:
            print "TEST FAILED"

    def checkSetAndGetValTime(self):
	print "checkSetAndValTime"
	try:
	    valTime = time.localtime()
	    vTime = tuple(valTime)
	    self._attr.set_val(vTime)
	    newValTime = self._attr.get_val()
	    
	    if valTime == newValTime:
		print "Setting time works"
                print "TEST PASSED"
            else:
                print "TEST FAILED"
	except RlsAttrException, ex:
	    print "Exception caught"
	    print ex
            print"TEST_FAILED"
	    
def makeTestSuite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(RlsClientTestCaseCreate, "check"))
    suite.addTest(unittest.makeSuite(RlsClientTestCaseTest, "check"))
    suite.addTest(unittest.makeSuite(LrcRlsAttrTestCase, "check"))
    suite.addTest(unittest.makeSuite(RlsAttrTestCase, "check"))
    suite.addTest(unittest.makeSuite(RlsClientTestCaseClear, "check"))
    suite.addTest(unittest.makeSuite(RlsClientTestCaseRli, "check"))
    
    
    return suite

def main():
    unittest.main(defaultTest="makeTestSuite")


if __name__ == "__main__" : main()
   
 
    
