############################################################################
# Matt Rodriguez, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################

import ConfigParser, unittest
from pyGlobus.security import check_keyfile, check_certfile
from pyGlobus.security import get_user_cert_filename
from pyGlobus.security import get_host_cert_filename
from pyGlobus.security import GSIException
from pyGlobus.security import get_cert_dir
from pyGlobus.security import get_proxy_filename
from pyGlobus.security import grid_proxy_info
from pyGlobus.security import gridmap
from pyGlobus.security import userok
from pyGlobus.security import map_local_user
import os

config=None
EXCEPTION_STRING_NO_HOST_CERT = "Could not find a valid certificate file: The host cert could not be found in"
EXCEPTION_STRING_NO_ROOT_PRIVELEGES = "Error with certificate filename"

class securityFunctionsTestCase(unittest.TestCase):
    """Unittests for the security modules functions
       3 of these tests require the user to have a ~/.gridmap file 
    """
    def setUp(self):
        global config
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read("config.txt")
        gridmap_file = config.get("security", "gridmapenv") 
        gridmap_env = os.path.join(os.getcwd(), gridmap_file) 
        os.putenv("GRIDMAP", gridmap_env)

    def tearDown(self):
        pass
        
    def checkGetHostCertFileName(self):
        print "Running test get host cert filename"
        try:
            files = get_host_cert_filename()
        except GSIException, ex:
            exceptString = str(ex)
            if exceptString.find(EXCEPTION_STRING_NO_HOST_CERT) > -1: 
                print "This machine does not have host cert and a host key"
                print "TEST PASSED"
            elif exceptString.find(EXCEPTION_STRING_NO_ROOT_PRIVELEGES) > -1:  
                print "You must be root to access the hostcert and hostkey"
                print "TEST PASSED"
            else:
                print 'got exception: ', exceptString
                raise ex
        
    def checkGetAndCertFile(self):
        print "Running test get user cert filename"
        try:
            certfile, keyfile = get_user_cert_filename()
            check_certfile(certfile)
            check_keyfile(keyfile)
        except GSIException, ex:
            print "TEST FAILED"
            print "Probably a configuration issue with certificates and keys"
            print ex
        else:
            print "TEST PASSED"

    def checkGetCertDir(self):
        print "Checking certificate directory"
        try:
            directory = get_cert_dir()
        except GSIException, ex:
            print "TEST FAILED"
            print "Probably a configuration issue with certificates"
            print ex
        else:
            print "TEST PASSED"

    def checkGetProxyFileName(self):
        print "Checking proxy file name"
        try: 
            proxyFile =get_proxy_filename() 
        except GSIException, ex:
            print "TEST FAILED"
            print ex
        else:
            print "TEST PASSED"

    def checkGridmap(self):
        print "Getting user from DN and gridmap file."
        DN, proxyDict = grid_proxy_info()
        l = DN.splitlines()
        subjectDN = l[1].split(': ')[1] 
        gridmap_user = gridmap(subjectDN)
        user = os.getenv("LOGNAME")
        if gridmap_user == user:
            print "TEST PASSED"
        else: 
            print "TEST FAILED"
        
    def checkUserok(self):
        print "Checking to see if the user is authorized by the gridmap file"
        DN,proxyDict = grid_proxy_info()
        l = DN.splitlines()
        subjectDN = l[1].split(': ')[1]
        user = os.getenv("LOGNAME") 
        ok  = userok(subjectDN, user)
        if ok == 0:
            print "TEST PASSED" 
        else:
            print "TEST FAILED"
            
    def checkMapLocalUser(self):
        print "Checking map local user" 
        user = os.getenv("LOGNAME")
        dn = map_local_user(user)
        DN,proxyDict = grid_proxy_info()
        l = DN.splitlines()
        subjectDN = l[1].split(': ')[1]
        if dn == subjectDN:
            print "TEST PASSED"
        else:
            print "TEST FAILED"

def makeTestSuite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(securityFunctionsTestCase, "check"))
    return suite


def main():
    unittest.main(defaultTest="makeTestSuite")


if __name__ == "__main__" : main()
