############################################################################
# Noah Edelson
#Copyright (c) 2004, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################

import sys, unittest, ConfigParser
import SocketServer
from threading import *
from pyGlobus import utilc
from pyGlobus.io import GSITCPSocket,GSITCPSocketServer,TCPIOAttr,AuthData, GSITCPSocketException
from pyGlobus import ioc
from pyGlobus.util import Buffer,GlobusException
import re, pyGlobus
import pyGlobus.ioAttr

config = None
g_port = 0
server = None
STRING1 = "Foobar, fOobar, foObar, fooBar, foobAr, foobaR\n"
STRING2 = "Some DAta SOme DAtA\n"
STRING3 = "More DAta mORE daTA\n"
STRING4 = "Last Bit\n"
regExpString = STRING1 +'|' + STRING2 + '|' + STRING3 + '|' + STRING4 + '|' +""
regExp = re.compile(regExpString)

def auth_callback(arg, handle, identity, context):
        return 1
def writeCB(cv, handle, result, data, datalen):
	if cv:
		cv.acquire()  
		cv.notify()
		cv.release()
        return 1
def connectCB(cv, handle, result):
	if cv:
		cv.acquire()  
		cv.notify()
		cv.release()
        return 1

def closeCB(cv, handle, result):
	#print 'cv: ', cv, 'handle: ', handle, 'result:', result
	if cv:
		cv.acquire()  
		cv.notify()
		cv.release()
        return 1


factoryMethods = ['pyGlobus.ioAttr.createTCPAttrDefault()',  
		  'pyGlobus.ioAttr.createTCPAttrSelfAuth()',
		  'pyGlobus.ioAttr.createTCPAttrDefaultSSL()',
		  'pyGlobus.ioAttr.createTCPAttrAlwaysAuth()']  
		  
	
class testServer(Thread):
    def __init__(self):
        Thread.__init__(self)
	self.__threads = []
	
    def setup(self, port):
	global g_port
        g_port = port
	try:
	    self.handle = GSITCPSocketServer(('',g_port), GSIRequestHandler)
        except GlobusException, ex:
	    print "exception raised"
	    raise ex

    def insertClientThread(self, thread):
        self.__threads.insert(0,thread)

    def doJoins(self):
        for thr in self.__threads:
	    thr.join()
	    
    def run(self):
        for factoryMethod in factoryMethods:
		self.handle.handle_request()
	#Make sure all of the client threads have finished before we close
	self.doJoins()
	self.server_close()
	
    def server_close(self):
        self.handle.server_close()
	

class GSIRequestHandler(SocketServer.StreamRequestHandler):
   """A minimalistic Request handler class that receives data from the
   client only. It prints it all out and then exits.
   """
      
   def handle(self):
      result = 'X'
      #print "GSIRequestHandler got a connection from %s" % (self.client_address,)

      while result:
         result = self.rfile.readline()
	 if not regExp.match(result):
	     exceptString = "Did not receive expected Data, got: " +result
	     raise GSITCPSocketException(exceptString)
      
class testClient(Thread):
    def __init__(self, fmethod):
        Thread.__init__(self)
	self.fmethod = fmethod

    def run(self):          
        host = "localhost"
	attr = eval(self.fmethod)
	if self.fmethod=='pyGlobus.ioAttr.createTCPAttrDefault()' or self.fmethod=='pyGlobus.ioAttr.createTCPAttrDefaultSSL()':
		attr.set_authentication_mode(ioc.GLOBUS_IO_SECURE_AUTHENTICATION_MODE_GSSAPI)
        try:
            authData = AuthData()
	    if self.fmethod=='pyGlobus.ioAttr.createTCPAttrDefault()' or self.fmethod=='pyGlobus.ioAttr.createTCPAttrDefaultSSL()':
		    attr.set_authorization_mode(ioc.GLOBUS_IO_SECURE_AUTHORIZATION_MODE_SELF, authData)
		    attr.set_channel_mode(ioc.GLOBUS_IO_SECURE_CHANNEL_MODE_GSI_WRAP)
		    attr.set_delegation_mode(ioc.GLOBUS_IO_SECURE_DELEGATION_MODE_LIMITED_PROXY)
            soc = GSITCPSocket()
	    
	    condV = Condition(Lock())
	    try:
		    condV.acquire()
		    soc.register_connect(host, g_port, attr, connectCB, condV)  # register connect
		    condV.wait()
		    condV.release()
	    except GlobusException, ex:
	        print '***PROBLEM WITH REGISTER CONNECT***'
		print 'TEST FAILED'
	        raise ex
	
	    addrList = soc.get_remote_address()       
	    str = STRING1
	    try:
		    condV.acquire()
		    nBytes = soc.register_write(STRING1, len(STRING1), writeCB, condV)  # register write 
		    condV.wait()
		    condV.release()
	    except GlobusException, ex:
	        print '***PROBLEM WITH REGISTER WRITE***'
		print 'TEST FAILED'
	        raise ex
	    try:
		    condV.acquire()
		    soc.register_close(closeCB, condV)        # register close
		    condV.wait()
		    condV.release()
	    except GlobusException, ex:
	        print '***PROBLEM WITH REGISTER CLOSE***'
		print 'TEST FAILED'
	        raise ex
	
        except GlobusException, ex:
	    print "exception in testClient", ex.args
	    print "TEST FAILED"
	    raise ex


	
class asynchIOTestCase(unittest.TestCase):
    
 
    def check_0_loadTestServer(self):    # loads test server
        """
	Starts up the Socket Server,
	closes the listening socket when
	"""
        global server
        try:
	    #print "Starting Server Test"
            config = ConfigParser.ConfigParser()
            config.read("config.txt")
	    portnum = int(config.get("asynchIO", "serverport"))
	    
	    server = testServer()
	    server.setup(portnum)
	    server.start()
	    
	except GlobusException, ex:
	     print "exception raised in Test1", ex.args
	     print "TEST FAILED"
	     raise ex

    def check_1_ioAttr_factory_tests(self):
        """
	Tests connect and write
	"""
	global server
        try:
	   for factoryMethod in factoryMethods:
		   self.client1 = testClient(factoryMethod)  
		   server.insertClientThread(self.client1)
		   self.client1.start()
		   #time.sleep(1)
	    
	except GlobusException, ex:
	    print "exception raised in Test2"
	    raise ex

	except GlobusException, ex:
	    print "exception raised in Test2"
	    raise ex
	
   
        
def makeTestSuite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(asynchIOTestCase, "check"))
    return suite

def main():
    unittest.main(defaultTest="makeTestSuite")

if __name__ == "__main__" : main()
