############################################################################
# Noah Edelson
#Copyright (c) 2004, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################

import sys, unittest, ConfigParser, time
import SocketServer
from threading import *
from pyGlobus import utilc
from pyGlobus.io import GSITCPSocket,GSITCPSocketServer,TCPIOAttr,AuthData, GSITCPSocketException
from pyGlobus import ioc
from pyGlobus.util import Buffer,GlobusException
import re, pyGlobus
import pyGlobus.ioAttr

config = None
config_file = 'config.txt'
g_port = 0
server = None
STRING1 = "Foobar, fOobar, foObar, fooBar, foobAr, foobaR\n"
STRING2 = "Some DAta SOme DAtA\n"
STRING3 = "More DAta mORE daTA\n"
STRING4 = "Last Bit\n"
regExpString = STRING1 +'|' + STRING2 + '|' + STRING3 + '|' + STRING4 + '|' +""
regExp = re.compile(regExpString)

def auth_callback(arg, handle, identity, context):
        return 1

factoryMethods = ['pyGlobus.ioAttr.createTCPAttrDefault()',  
		  'pyGlobus.ioAttr.createTCPAttrSelfAuth()',
		  'pyGlobus.ioAttr.createTCPAttrDefaultSSL()',
		  'pyGlobus.ioAttr.createTCPAttrAlwaysAuth()']  
		  

	
class testServer(Thread):
    def __init__(self):
        Thread.__init__(self)
	self.__threads = []
	
    def setup(self, port):
	global g_port
        g_port = port
	try:
	    self.handle = GSITCPSocketServer(('',g_port), GSIRequestHandler)
        except GlobusException, ex:
	    print "exception raised"
	    raise ex

    def insertClientThread(self, thread):
        self.__threads.insert(0,thread)

    def doJoins(self):
        for thr in self.__threads:
	    thr.join()
	    
    def run(self):
        for factoryMethod in factoryMethods:
		self.handle.handle_request()
	#Make sure all of the client threads have finished before we close
	self.doJoins()
	self.server_close()
	
    def server_close(self):
        self.handle.server_close()
	

class GSIRequestHandler(SocketServer.StreamRequestHandler):
   """A minimalistic Request handler class that receives data from the
   client only. It prints it all out and then exits.
   """
      
   def handle(self):
      result = 'X'
      #print "GSIRequestHandler got a connection from %s" % (self.client_address,)

      while result:
         result = self.rfile.readline()
	 if not regExp.match(result):
	     exceptString = "Did not receive expected Data, got: " +result
	     raise GSITCPSocketException(exceptString)
      
class testClient(Thread):
    def __init__(self, fmethod):
        Thread.__init__(self)
	self.fmethod = fmethod

    def run(self):          
        host = "localhost"
	attr = eval(self.fmethod)
	if self.fmethod=='pyGlobus.ioAttr.createTCPAttrDefault()' or self.fmethod=='pyGlobus.ioAttr.createTCPAttrDefaultSSL()':
	    attr.set_authentication_mode(ioc.GLOBUS_IO_SECURE_AUTHENTICATION_MODE_GSSAPI)
        try:
            authData = AuthData()
	    if self.fmethod=='pyGlobus.ioAttr.createTCPAttrDefault()' or self.fmethod=='pyGlobus.ioAttr.createTCPAttrDefaultSSL()':
	        attr.set_authorization_mode(ioc.GLOBUS_IO_SECURE_AUTHORIZATION_MODE_SELF, authData)
	        attr.set_channel_mode(ioc.GLOBUS_IO_SECURE_CHANNEL_MODE_GSI_WRAP)
	        attr.set_delegation_mode(ioc.GLOBUS_IO_SECURE_DELEGATION_MODE_LIMITED_PROXY)
            soc = GSITCPSocket()
            soc.connect(host, g_port, attr)
	    addrList = soc.get_remote_address()
	    str = STRING1
            nBytes = soc.write(STRING1, len(STRING1))
        except GlobusException, ex:
	    print "exception in testClient", ex.args
	    print "TEST FAILED"
	    raise ex

	
class ioAttrTestCase(unittest.TestCase):

    def setUp(self):
        global config
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read(config_file)
	
    def check_0_loadTestServer(self):    # loads test server
        """
	Starts up the Socket Server,
	closes the listening socket when
	"""
        global server
        print "Starting a GSITCPSocket Server."
        print "This server exits when all of the requests have been made and are completed."
        try:
	    server = testServer()
	    port = config.get("io", "ListenPort2")
	    server.setup(int(port))
	    server.start()
	    
	except GlobusException, ex:
	     print "exception raised in Test1", ex.args
	     print "TEST FAILED"
	     raise ex
        else:
             print "Server shutting down"
             print "TEST PASSED"
        

    def check_1_ioAttr_factory_tests(self):
        """
	Tests connect and write
	"""
        print "Testing io Attribute factory methods"
	global server
        try:
	   for factoryMethod in factoryMethods:
	       self.client1 = testClient(factoryMethod)  
	       server.insertClientThread(self.client1)
	       self.client1.start()
	    
	except GlobusException, ex:
	    print "exception raised in Test2"
       	    raise ex	
            print "TEST FAILED"
        else:
            print "TEST PASSED"
   
        
def makeTestSuite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(ioAttrTestCase, "check"))
    return suite

def main():
    unittest.main(defaultTest="makeTestSuite")

if __name__ == "__main__" : main()
