############################################################################
# Keith R. Jackson, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################

import unittest
from pyGlobus import utilc
from pyGlobus.util import getHostname
from pyGlobus.util import getEnv, setEnv, unSetEnv
from pyGlobus.util import GlobusXIODebug, GlobusXIODebugParseError
import socket

class UtilTestCase(unittest.TestCase):
    "Test case for the util module"
    
    def checkModule(self):
        "checking module activate and deactivate methods"
        ret = utilc.globus_module_activate(utilc.get_module())
        assert ret == 0, "Error activating the module"
        ret = utilc.globus_module_deactivate(utilc.get_module())
        assert ret == 0, "Error deactivating the module"
        ret = utilc.globus_module_activate(utilc.get_module())
        assert ret == 0, "Error activating the module"
        ret = utilc.globus_module_deactivate_all()
        assert ret == 0, "Error deactivating all modules"
        #print "TEST PASSED"

    def checkGetHostName(self):
        "checking getHostname"
        test_hostname = getHostname()
        real_hostname = socket.getfqdn() 
        assert test_hostname == real_hostname,"hostname does not match"

    def checkEnvironmet(self):
        "Testing setting and getting environment variables"
        variable = "testVar"
        value = "testValue"
        setEnv(variable, value)
        newVal = getEnv(variable)
        assert newVal == value,"setEnv failed"
        unSetEnv(variable)
        newVal = getEnv(variable)
        assert newVal == None,"unSetEnv failed"

    def checkGlobusXIODebug(self):
        "checking parsing of GLOBUS_XIO_DEBUG value"
        x = GlobusXIODebug("")
        assert(x.level is None and x.filename is None and x.overwrite is None)
        lvl = GlobusXIODebug.ERROR | GlobusXIODebug.INFO
        x = GlobusXIODebug("%d" % lvl)
        assert(x.level == lvl)
        try:
            x = GlobusXIODebug("garbage in garbage out")
            assert False, "expected exception with garbage input"
        except GlobusXIODebugParseError:
            pass
        
def makeTestSuite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(UtilTestCase, "check"))
    return suite

def main():
    unittest.main(defaultTest="makeTestSuite")

if __name__ == "__main__": main()
        
