############################################################################
# Matt Rodriguez, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################


from wxPython.wx import *
from pyGlobus.ftpClient import FtpClient
from pyGlobus.ftpClient import EasyFtpClient
from pyGlobus.ftpClient import OperationAttr
from pyGlobus.ftpClient import HandleAttr
from pyGlobus.ftpClient import RestartMarker
from pyGlobus.ftpClientPlugin import PluginException
from pyGlobus.ftpClientPlugin import DebugPlugin
from pyGlobus import ftpControl
from pyGlobus.util import Buffer
import ConfigParser, sys, os, stat, time, Queue, getopt
from pyGlobus.util import GlobusException
from threading import *
from thread import *
from exceptions import Exception



mWidgetCodes = {
    "ID_EXIT" : 102,
    "ID_ABOUT"  : 103,
    "ID_SERVERFILE" : 114,
    "ID_CLIENTFILE" : 115,
    "ID_GET" : 116,
    "ID_PUT" : 117,
    "ID_RIGHT_BMP" :3001,
    "ID_CANCEL": 3002,
    "ID_LEFT_BMP" : 3003,
    "ID_CONNECT" : 3004  
}

g_Log = None
g_config = None
g_Control = None
app = None

def LogPrintDebug(info):
    global g_Log
    if g_Log == None:
        return
    g_Log.debug_write(info)

def LogPrint(info):
    global g_Log
    if g_Log == None:
        return
    g_Log.write(info)
    
def init_globals():
    global g_config
    if g_config == None:
        g_config = ConfigParser.ConfigParser()
        g_config.read("./defaults.cfg")
        
    
    return
class Log:
    def __init__(self, name, debug_level = 0):
        
        self._file = open(name, "w")
        self._fileName = name
        self.write('*********************************************************\n')
        str = 'Begining Log of ftpClient at ' + time.ctime() +'\n'
        self.write(str)
        self._debug_level = debug_level
        
    def __del__(self):
        self._file.close()
        

    def write(self, info):
        info_string = ("%s%s%s" %(str(currentThread()),":    ", info))
        self._file.write(info_string)
        self._file.flush()

    def debug_write(self, info):
        if self._debug_level:
            self.write(info)



class MyApp(wxApp):
    def OnInit(self):
        global g_config
        if g_config == None:
            g_config = ConfigParser.ConfigParser()
            g_config.read("./defaults.cfg")

        frame = childFrame(NULL, -1, "FtpClient")
        frame.Show(true)
        self.SetTopWindow(frame)
        return true
    
###############################################################################
class childFrame(wxFrame):
    def __init__(self, parent, ID, title):
        SIZE_H = g_config.getint("Dimensions", "Size_H")
        SIZE_V = g_config.getint("Dimensions", "Size_V")
        wxFrame.__init__(self, None, ID, title,
                                 wxDefaultPosition, wxSize(SIZE_H, SIZE_V))

        sizer = wxBoxSizer(wxHORIZONTAL)
        self._gui = GuiPanel(self)
        sizer.Add(self._gui, 1, wxEXPAND)
        self.MakeMenu()
        self.SetAutoLayout(true)
        self.SetSizer(sizer)
        return

    def Quit(self, event):
        self.Close(true)
     
      
    def MakeMenu(self):
        fmenu = wxMenu()
        fmenu.Append(mWidgetCodes['ID_ABOUT'], "&About","Matt gets Gui")
        fmenu.AppendSeparator()
        fmenu.Append(mWidgetCodes['ID_EXIT'], "E&xit", "Terminate the program")   
        menuBar = wxMenuBar()
        menuBar.Append(fmenu,"&File")
        self.SetMenuBar(menuBar)
        EVT_MENU(self, mWidgetCodes['ID_EXIT'], self.Quit)
      

   

class GuiPanel(wxPanel):
    def __init__(self,parent):
        global g_Control
        wxPanel.__init__(self,parent, -1)
                            
        jobActions = Queue.Queue(0)
        fileAction1 = Queue.Queue(0)
        fileAction2 = Queue.Queue(0)
           
        self._client = LocalFileBrowser(self)
        self._client.addFiles()
	self._server = RemoteFileBrowser(self)
	self._topPanel = configPanel(self)
	self._transferbuttons = TransferButtonPanel(self, fileAction1)
        self._statusPanel = ProgressPanel(self)
        self._transferbuttons.setConfigs(self._topPanel)

        g_Control = Control(self)
        self.GuiSetup()
        return
    
    def GuiSetup(self, noClient = false):
        sz = wxBoxSizer(wxVERTICAL)
	sz1 = wxBoxSizer( wxHORIZONTAL )
        self._clientSizer = wxBoxSizer(wxHORIZONTAL)
        clientServer = wxBoxSizer(wxHORIZONTAL)
        self._clientSizer.Add(self._client, 2, wxEXPAND)
        clientServer.Add(self._clientSizer, 2, wxEXPAND)
	clientServer.Add(self._transferbuttons,1, wxEXPAND)
	clientServer.Add(self._server,2, wxEXPAND)
       
        sz.Add(self._topPanel, 1, wxEXPAND)
	sz.Add(clientServer,  3, wxEXPAND )
	sz.Add(self._statusPanel, 1, wxEXPAND)
    
        self.SetAutoLayout(true)
        self.SetSizer(sz)
        self._mainSizer = sz
        self.Layout()
        
    def getServer(self):
        return self._server

    def getClient(self):
        return self._client

    def removeClient(self):
        removed = self._mainSizer.Remove(self._clientSizer)
        del self._client
        
    def getClientType(self):
        return self._client.getType()
    
    def getConfig(self):
        return self._topPanel
    
    def getProgress(self):
        return self._statusPanel

    def setClientBrowser(self, browser):
        self._client = browser

    def setRemoteBrowser(self, browser):
        self._server = browser
    
    
class Control:
    """Class that will be instantiated once to handle events that require gathering information
       among the classes """
    
    def __init__(self, gui):
        self._gui = gui
        self._server = gui.getServer()
        self._client = gui.getClient()
        self._config = gui.getConfig()
        self._progress = gui.getProgress()

        self._Transfers = 0
        self._commands_dict = {}
        self._dict_mutex = Lock()
        self._server_view_mutex = Lock()
        self._client_view_mutex = Lock()
        self._progress_mutex = Lock()
        return


    def changeBrowsers(self,hostName):
        clientType = self._gui.getClientType()
        if hostName == 'localhost' and clientType == 'REMOTE':
            browser = LocalFileBrowser(self._gui, true)
            self._client = browser
            self._gui.setClientBrowser(browser)
            self._gui.GuiSetup()
            self._client.addFiles()
    
        if hostName != 'localhost' and clientType == 'LOCAL':
            self._gui.removeClient()
            browser = RemoteFileBrowser(self._gui, true)
            self._gui.setClientBrowser(browser)
            self._client = browser
            self._gui.GuiSetup()
            
   
    
    def doCancel(self):
        trans_ids = self._progress.getFileSelection()
        trans_ids
        for id in trans_ids:
            self._commands_dict[id].undo()
        

    def getUrlBase(self, browser):
        host = browser.getHost()
        path = browser.getPath()
        if path == '/':
            url = "gsiftp://" + host + path 
        else:
            url = "gsiftp://" + host + path +'/' 

        return url
    
    def getRemoteUrlBase(self):
        return self.getUrlBase(self._server)

    def getClientUrlBase(self):
        if self._client.getType() == 'REMOTE':
            return self.getUrlBase(self._client)

        path = self._client.getPath()
        if path != '/':
            path = path + '/'
        return path

    def doingTPT(self):
        return self._client.getType() == 'REMOTE'

    
    def verbose_list(self,path, side, existence_check):
        if side:
            host = self._client.getHost()
        else:
            host = self._server.getHost()
            
        url = "gsiftp://" + host + path
        ListC = ListCommand(url,side)
        self._Transfers += 1
        ListC.set_id(self._Transfers)
        self._commands_dict.update({self._Transfers:ListC})
        if existence_check:
            start_new_thread(ListCommand.checkExists,(ListC,))
        else:
            start_new_thread(ListCommand.run, (ListC,))
        
    def doPut(self):
        LogPrint("Doing a Put operation\n")
        
        
        dest_url_base = self.getRemoteUrlBase()
        destPath = self._server.getPath()
        src_base = self.getClientUrlBase()
        files = self._client.getFileSelection()
        for file in files:
            src = src_base + file
            dest_url = dest_url_base + file
            putC = PutCommand(src, dest_url)
            putC.setDirectory(destPath)
            self._Transfers +=1
            putC.set_id(self._Transfers)
            self._commands_dict.update({self._Transfers:putC}) 
            self._progress.addFile(file,self._Transfers, "   PUT   ")
            start_new_thread(PutCommand.run, (putC,dest_url_base))
        
    def doGet(self):
        LogPrint("Doing a Get operation\n")
        
        src_url_base = self.getRemoteUrlBase()
        destPath = self._client.getPath()
        dest_base = self.getClientUrlBase()
        files = self._server.getFileSelection()
        for file in files:
            src_url = src_url_base + file
            dest = dest_base +file
            getC = GetCommand(src_url, dest)
            getC.setDirectory(destPath)
            self._Transfers +=1
            getC.set_id(self._Transfers)
            self._commands_dict.update({self._Transfers:getC}) 
            self._progress.addFile(file,self._Transfers, "   GET   ")
            start_new_thread(GetCommand.run, (getC,dest_base))
            
    def getServerDir(self):
        return self._server.getPath()

    def getClientDir(self):
        return self._client.getPath()

    def updateServerView(self):
        self._server_view_mutex.acquire()
        print "updating server_view"
        self._server.updateView()
        self._server_view_mutex.release()

    def paintRemoteFiles(self, fileList, isClientSide):
        if isClientSide:
            self._client.paintRemoteFiles(fileList)
        else:
            self._server.paintRemoteFiles(fileList)
        
    def updateClientView(self):
        self._client_view_mutex.acquire()
        dbg_str = "Mutex acquired for updateClientView\n"
        LogPrintDebug(dbg_str)
        self._client.updateView()
        LogPrintDebug("Releasing mutex for view lock\n")
        self._client_view_mutex.release()
        
    def cleanProgress(self, trans_id):
        self._progress_mutex.acquire()
        str = ("%s%d\n" %("Cleaning Progress Panel, of transaction id: ",trans_id))
        LogPrintDebug(str)
        wxMutexGuiEnter()
        self._progress.removeFile(trans_id)
        wxMutexGuiLeave()
        self._progress_mutex.release()
        
    def removeCommand(self, trans_id):
        self._dict_mutex.acquire()
        LogPrintDebug("Acquiring mutex for  removeCommand\n")
        dbg_str = ("%s%d\n" %("Removing a command associated with Xact: ", trans_id))
        LogPrintDebug(dbg_str)
        dbg_str = ("%s%s\n" %("Removing a command from transaction Dictionary: ", str(self._commands_dict)))
        LogPrintDebug(dbg_str)
        del self._commands_dict[trans_id]
        dbg_str = ("%s%s\n" %("Command has been Removed: ", str(self._commands_dict)))
        LogPrintDebug(dbg_str)
        LogPrintDebug("Releasing mutex for removeCommand\n")
        self._dict_mutex.release()
        
class TransferCommand:
    def __init__(self, trans_type):
        self._trans_type = trans_type

    def set_id(self, id):
        self._trans_id = id

    def get_id(self):
        return self._trans_id

    def setDirectory(self, dir):
        self._dir = dir

    def getDirectory(self):
        return self._dir
    
    def run(self):
        raise NotImplementedError, "Method run should be overloaded in child classes" 

    def undo(self):
        raise NotImplementedError, "Method undo should be overloaded in child classes"
        
    
class ListCommand(TransferCommand):
    def __init__(self, url, isClientSide):
        TransferCommand.__init__(self,"LIST")
        self._url = url
        self._isClientSide = isClientSide
        self._ftpC = EasyFtpClient()

    def checkExists(self):
        exists = self._ftpC.exists(self._url)
        if not exists:
            print "The directory you tried to LIST does not exist"
            return None
        self.run()
        
    def run(self):
        
        try:
            fileList = self._ftpC.verbose_list(self._url)
        except GlobusException, ex:
            print "Exception: ", ex
            g_Control.removeCommand(self.get_id())
            raise ex
        g_Control.paintRemoteFiles(fileList, self._isClientSide)
        g_Control.removeCommand(self.get_id())
            
class PutCommand(TransferCommand):
    def __init__(self, src, dest_url):
        TransferCommand.__init__(self, "PUT")
        self._src = src
        self._dest_url = dest_url
        self._ftpC = EasyFtpClient()

    def undo(self):
        LogPrint("Aborting a put Command PutCommand\n")
        try:
            self._ftpC.abort()
        except GlobusException, ex:
            print "Undo Exception", ex
            g_Control.cleanProgress(self.get_id())
            g_Control.removeCommand(self.get_id())
            raise ex
        
        
    def run(self, dest_url_dir):
        try:
            dest_exists = self._ftpC.exists(dest_url_dir)
            if g_Control.doingTPT():
                src_exists = self._ftpC.exists(self._src)
                if src_exists and dest_exists:
                    self._ftpC.third_party_transfer(self._src, self._dest_url)
                else:
                    print "Source url or destination url failed existence check!!"
            else:
                if dest_exists:
                    self._ftpC.put(self._src, self._dest_url)
                else:
                    print "Destination url failed existence test!!"
                    
            print_str = ("%s%d\n" %( "Finished with put ftpC.put call, trans_id: " , self.get_id()))
            LogPrint(print_str)
        except GlobusException, ex:
            print "PutC Exception: ", ex
            g_Control.cleanProgress(self.get_id())
            g_Control.removeCommand(self.get_id())
            raise ex
        self.cleanUp()
        
    def cleanUp(self):
        currentDir = g_Control.getServerDir()
        getDir = self.getDirectory()
        if currentDir == getDir:
            g_Control.updateServerView()
            g_Control.cleanProgress(self.get_id())
        g_Control.removeCommand(self.get_id())
        dbg_str = ("%s%d\n" %( "Thread associatiated with put Xact finishes: ", self.get_id()))
        LogPrintDebug(dbg_str)
   
  
class GetCommand(TransferCommand):
    def __init__(self, src_url, dest):
        self._src_url = src_url
        self._dest = dest
        self._ftpC = EasyFtpClient()

    def undo(self):
        str =  "Aborting a get Command \n"
        LogPrint(str)
        try:
            
            self._ftpC.abort()
        except GlobusException, ex:
            print "Undo Exception", ex
            g_Control.cleanProgress(self.get_id())
            g_Control.removeCommand(self.get_id())
            raise ex
        
    def run(self, dest_dir):
        try:
            src_exists = self._ftpC.exists(self._src_url)
            if g_Control.doingTPT():
                dest_exists = self._ftpC.exists(dest_dir)
                if src_exists and dest_exists:
                    self._ftpC.third_party_transfer(self._src_url, self._dest)
                else:
                    print "Source url or destination url failed existence check!!"
                
            else:
                if src_exists:
                    self._ftpC.get(self._dest, self._src_url)
                else:
                    print "Source url failed existence check"
                    
            print_str= ("%s%d\n" %( "Finished with get ftpC.get call, trans_id: ", self.get_id()))
            LogPrint(print_str)
        except GlobusException, ex:
            print "GetC Exception: ", ex
            g_Control.cleanProgress(self.get_id())
            g_Control.removeCommand(self.get_id())
            raise ex
        self.cleanUp()
        
    def cleanUp(self):
        currentDir = g_Control.getClientDir()
        getDir = self.getDirectory()
        if currentDir == getDir:
            g_Control.updateClientView()  
            g_Control.cleanProgress(self.get_id())
            
        g_Control.removeCommand(self.get_id())
        dbg_str = ("%s%d\n" %( "Thread associatiated with get Xact finishes: " , self.get_id()))
        LogPrintDebug(dbg_str)
        
        
class Host(wxPanel):
    def __init__(self, parent, remoteOnly = false):
        wxPanel.__init__(self, parent, wxNewId())
        self._parent = parent
        self._host_txt = wxStaticText(self, -1, ' Host  ')
        self._host = HostBox(self)
        sz = wxBoxSizer(wxHORIZONTAL)
        sz.Add(self._host_txt,1, wxEXPAND)
        sz.Add(self._host,1, wxEXPAND)
        self.SetAutoLayout(true)
        self.SetSizer(sz)
        defaultPanelColor(self)

    def setHost(self, hostName):
        self._host.setHost(hostName)

    def changeHost(self, hostName):
        side = self._parent.isClientSide()
        if side:
            g_Control.changeBrowsers(hostName)

    def getHost(self):
        return self._host.GetValue()
    
class HostBox(wxComboBox):
    def __init__(self, parent, remoteOnly = false):
        hosts = self.getHostList()
        self._parent = parent
        self._id = wxNewId()
        if remoteOnly:
            hosts.remove('localhost')
        wxComboBox.__init__(self,parent,self._id, choices = hosts)
        EVT_COMBOBOX(self,self._id, self.changeHost)

    def getHostList(self):
        global g_config
        numHosts = g_config.getint("hostslist", "numHosts")
        r = range(numHosts)
        serverlist = []
        for x in r:
            name = g_config.get("hostslist" , str(r[x]))
            serverlist.append(name)

        return serverlist

    def changeHost(self, event):
        host = event.GetString()
        self._parent.changeHost(host)

    def setHost(self, hostName):
        self.SetValue(hostName)
        
##############################################################################
class FileBrowser(wxPanel):
    def __init__(self, parent,isClientSide):
        wxPanel.__init__(self, parent, wxNewId())
	self._host = None
        self._browserList = None
        self._pathSelector = None
        self._Pathtext = None
        self._isClientSide = isClientSide
        return
   
    def guiSetup(self):
        sz = wxBoxSizer(wxVERTICAL)
        sz1 = wxBoxSizer(wxVERTICAL)
        sz.Add(self._host, 0, wxEXPAND)
        sz.Add(self._Pathtext,0, wxEXPAND)
        sz1.Add(self._browserList,1, wxEXPAND)
        sz.Add(self._pathSelector,0,wxEXPAND)
        sz.Add(self._text,0, wxEXPAND)
        sz.Add(sz1, 1, wxEXPAND)
        self.SetAutoLayout(true)
        self.SetSizer(sz)
        defaultPanelColor(self)
        return
    
    def setPath(self, path):
        self._pathSelector.setPath(path)

    def getPath(self):
        return self._pathSelector.getPath()

    def getFileSelection(self):
        return self._browserList.getFileSelection()

    def getHost(self):
        return self._host.getHost()

    def getType(self):
        raise NotImplementedError, "Method should be overloaded in child Browser classes"

    def isClientSide(self):
        return self._isClientSide
##############################################################################        
         
class RemoteFileBrowser(FileBrowser):
    
    def __init__(self, parent, isClientSide = false):
        FileBrowser.__init__(self, parent, isClientSide) 
        self._host = Host(self, true)
	self._browserList = RemoteFileBrowserList(self)
	self._pathSelector = RemotePathSelector(self)
        self._Pathtext = wxStaticText(self,wxNewId(),  ' Path')
	self._text = wxStaticText(self,wxNewId(),  '[FileBrowser]')
	self.guiSetup()
	return

    def updateView(self):
        """Updates the view of file browser without an existence check"""
        self.addFiles(0)
        
    def addFiles(self, existence_check):
        self.requestFiles(existence_check)
    
    def requestFiles(self, existence_check):
        """Request the fileList from Control,
              Control spawns anther thread to get the fileList"""
        global g_Control
        path = self.getPath()
        g_Control.verbose_list(path,self._isClientSide, existence_check)
        #self._browserList.DeleteAllItems()

    def paintRemoteFiles(self, fileList):
        """Control sends the fileList Down to this method"""
        self._browserList.DeleteAllItems()
        self._browserList.paintRemoteFiles(fileList)

    def getType(self):
        return "REMOTE"

class LocalFileBrowser(FileBrowser):
    
    def __init__(self, parent, isClientSide = true):
        FileBrowser.__init__(self, parent,isClientSide)
        self._host = Host(self)
	self._browserList = LocalFileBrowserList(self)
	self._pathSelector = LocalPathSelector(self)
        self._Pathtext = wxStaticText(self,wxNewId(),  ' Path')
	self._text = wxStaticText(self,wxNewId(),  ' [FileBrowser] ')
	self._host.setHost('localhost')
	self.guiSetup()
	return
    
    def updateView(self):
        self.addFiles()
    
    def addFiles(self):
        self._browserList.DeleteAllItems()
        self._browserList.addFiles()
        
    def getType(self):
        return "LOCAL"
        
class PathSelector(wxComboBox):
    def __init__(self, browser):
	self._id = wxNewId()
	self._browser = browser
	wxComboBox.__init__(self,browser, self._id)
	EVT_COMBOBOX(self,self._id, self.changeDir)

    def getPath(self):
        return self.GetValue()
	    
    def fixView(self):
	self._browser.addFiles()

    def setPath(self, fullPath):
        self.SetValue(fullPath)

class LocalPathSelector(PathSelector):
    def __init__(self,browser):
        PathSelector.__init__(self,browser)
        self.setPath(os.getcwd())

    def changeDir(self, event):
	dir = event.GetString()
	p = PathInfo(dir)
	if p.isDir():
	    self._browser.setPath(dir)
	    self.fixView()

class RemotePathSelector(PathSelector):
     def __init__(self,browser):
        PathSelector.__init__(self,browser)

     def changeDir(self,event):
         self._browser.requestFiles(1)

         
class FileBrowserList(wxListCtrl):
    def __init__(self,parent):
        
	self._id = wxNewId()
	self._browser = parent
        self._fileSelected = []
	wxListCtrl.__init__(self,parent,self._id,style=wxLC_REPORT|wxSUNKEN_BORDER)
        EVT_LIST_ITEM_ACTIVATED(self,self._id, self.changeDir)
        EVT_LIST_ITEM_SELECTED(self, self._id, self.itemSelected)
        EVT_LIST_ITEM_DESELECTED(self, self._id, self.itemDeselected)
        EVT_LIST_DELETE_ALL_ITEMS(self, self._id, self.deleteAll)
        EVT_LIST_DELETE_ITEM(self, self._id, self.deleteItem)
	self._labels = ("Files", "Size", "Date", "Attribs")

	count = 0
	for label in self._labels:
	    self.InsertColumn(count, self._labels[count])
	    count += 1

	return

    def deleteAll(self, event):
        LogPrintDebug("in DeleteAll \n")

    def deleteItem(self, event):
        LogPrintDebug("in DeleteItem\n")
        
    def getFileInfo(self, file):
	path = self.getPath()
        fullPath = path +"/" + file
        p = PathInfo(fullPath)
	fsize = str(p.size())
	mtime = os.stat(path)[stat.ST_MTIME]
	mdate = time.ctime(mtime)
	perms = p._modeFlags()
	return file, fsize,mdate, perms

    def getPath(self):
        return self._browser._pathSelector.getPath()
	    
    def setPath(self, path):
        self._browser._pathSelector.setPath(path)
           	    
    def adjustColumnWidth(self):
	for count in range(len(self._labels)):
	    self.SetColumnWidth(count, wxLIST_AUTOSIZE)

    def itemSelected(self, event):
        dbg_str = "in itemSelected\n"
        LogPrintDebug(dbg_str)
        selected = event.GetText()
        dbg_str = "Selected Item is: " + selected + '\n'
        LogPrintDebug(dbg_str)
        self._fileSelected.append(selected)
        dbg_str = "self._fileSelected is: " + str(self._fileSelected) + '\n'
        LogPrintDebug(dbg_str)
         
    def itemDeselected(self, event):
        deselected = event.GetText()
        dbg_str = "Deselected Item is: " + deselected + '\n'
        LogPrintDebug(dbg_str)
        print dbg_str
        self._fileSelected.remove(deselected)
        dbg_str = "self._fileSelected is: " + str(self._fileSelected) + '\n'

    def getFileSelection(self):
        return self._fileSelected

    
class LocalFileBrowserList(FileBrowserList):
    def __init__(self, parent):
        FileBrowserList.__init__(self, parent)

    def changeDir(self,event):
	dir = event.GetText()
        tmpPath = self.getPath()
	if dir == "..":
            i = tmpPath.rfind("/")
            if i == 0:
                path = tmpPath[:i+1]
            else:
                path = tmpPath[:i]
	else:
            if tmpPath == "/":
                path = tmpPath + dir
            else:
                path  = tmpPath +"/" + dir

        p = PathInfo(path)
	if p.isDir():
            self.setPath(path)
            self.DeleteAllItems()
	    self.addFiles()
            self.itemDeselected(event)
            print self._fileSelected

    def addFiles(self):
        """Locks the Gui for painting"""
        LogPrintDebug("in LocalFileBrowserList addFiles\n")
	files = os.listdir(self.getPath())
	files.sort()
        dbg_str =("%s%d\n" %("Num files are ", len(files)))
        LogPrintDebug(dbg_str)
        p = PathInfo(self.getPath())
        LogPrintDebug("in LocalFileBrowserList addFiles 1 \n")
	if not p.isRootDir():
	    files.insert(0,'..')
            
        wxMutexGuiEnter()
	for i in files:
            file, fsize, mdate, perms = self.getFileInfo(i)
	    offset = self.GetItemCount()
            self.InsertStringItem(offset, file)
	    self.SetStringItem(offset,1,fsize)               
	    self.SetStringItem(offset,2,mdate) 
            self.SetStringItem(offset,3, perms)
	    self.adjustColumnWidth()
        wxMutexGuiLeave()
        LogPrintDebug("leaving LocalFileBrowserList addFiles\n")
            
class RemoteFileBrowserList(FileBrowserList):
    def __init__(self, parent):
        FileBrowserList.__init__(self, parent)
        self._fileInfo = {}
       
   
    def paintRemoteFiles(self, listDir):
        """Locks the Gui for painting"""
        files = self.readlines(listDir)
        #This gets rid of the total number return info
        files[0:1] = []
        path = self.getPath()
        wxMutexGuiEnter()
        if not path == "/":
            offset = self.GetItemCount()
            file = ".."
            perms = "d---------"
            self.InsertStringItem(offset, file)
            self.SetStringItem(offset, 3, perms)
            self._fileInfo.update({file:perms})
        for f in files:
            file,perms = self.parseFileString(f) 
            offset = self.GetItemCount()
            self.InsertStringItem(offset, file)
            #self.SetStringItem(offset,1,fsize)
            #self.SetStringItem(offset,2,mdate)
	    self.SetStringItem(offset,3, perms)
            self._fileInfo.update({file:perms})
            self.adjustColumnWidth()
        wxMutexGuiLeave()
        
    def parseFileString(self, file):
        str = file
        i = str.find(" ")
        attribs = str[:i]
        str = str[i+1:]
        i = str.rfind(" ")
        phyle = str[i+1:]
        return phyle, attribs

   
    def changeDir(self,event):
        print self._fileSelected
	dir = event.GetText()
        path = self.getPath()
        if dir == "..":
            i = path.rfind("/")
            if i == 0:
                fullPath = path[:i +1]
            else:
                fullPath = path[:i]
        else:
            if path == "/":
                fullPath = path + dir
            else:
                fullPath = path + "/" +dir

        
        perms = self._fileInfo[dir]
        if perms[0] == 'd':
            self.setPath(fullPath)
            self.itemDeselected(event)
            self._browser.requestFiles(1)
            
        
    def readline(self, str):
        """
        Return a single line as a string.
r        
        **Returns:**
        
        A string containing the next line in the Buffer.
        """
        i = str.find('\n')
        if i >= 0:
            rest = str[i+1:]
            line = str[:i]
        else:
            rest = ""
            line = rest
        return line, rest

    def readlines(self,str):
        """
        Return a list containing each line in the Buffer.

        **Returns:**
        
            A list of strings containing each line in the Buffer.
        """
        list = []
        while 1:
            line,str = self.readline(str)
            if line == "":
                break
            list.append(line)
            continue
        return list
    	
#######################################################################
class Actions:
    """
    Send command using parameters from model by
    placing a new Command subclass on the queue.
    """
    def __init__(self,queue):
        self._q = queue
        
    def setLog(self,log):
        self._log = log

    def _grab(self,thing):
        global g_model
        return g_model.getParam(thing)
  
############################################################################

class configPanel(wxPanel):
    def __init__(self, parent):
	wxPanel.__init__(self,parent, -1)
	
	self._txt2 = wxStaticText(self, -1, '   Port    ')
	self._portBox = wxComboBox(self, wxNewId())
	self._txt3 = wxStaticText(self, -1, '   User    ')
	self._userBox = wxComboBox(self, wxNewId())
	self._txt4 = wxStaticText(self, -1, '  Pass    ')
	self._passBox = wxTextCtrl(self, wxNewId(),style = wxTE_PASSWORD)
	choices = ["GSIFTP","GRIDFTP", "HTTP", "LOCAL", "SSH"]
	self._protoBox = wxChoice(self, wxNewId(), choices = choices)
        
        
	sz = wxBoxSizer(wxHORIZONTAL)
	sz.Add(self._txt2, 1, wxALIGN_CENTER)
	sz.Add(self._portBox, 1, wxEXPAND)
	sz.Add(self._txt3, 1, wxALIGN_CENTER)
	sz.Add(self._userBox, 2, wxEXPAND)
	sz.Add(self._txt4, 1, wxALIGN_CENTER)
	sz.Add(self._passBox, 2,  wxALIGN_CENTER)
	sz.Add(self._protoBox, 2,  wxALIGN_CENTER)

	self.SetSizer(sz)
        self.SetAutoLayout(true)
        defaultPanelColor(self)
        return

    def getUser(self):
        return self._userBox.GetValue()
    
    def getPort(self):
        return self._portBox.GetValue()

    def getProto(self):
        selection = self._protoBox.GetSelection()
        return self._protoBox.GetString(selection)

    def getPass(self):
        return self._passBox.GetValue()
    
    
class ProgressList(wxListCtrl):
    def __init__(self, parent):
        self._id = wxNewId()
        self._parent = parent
        self._fileSelected = []
        wxListCtrl.__init__(self, parent,self._id,style=wxLC_REPORT|wxSUNKEN_BORDER)
        self._labels = ("Transfer", "File")
        EVT_LIST_ITEM_SELECTED(self, self._id, self.itemSelected)
        EVT_LIST_ITEM_DESELECTED(self, self._id, self.itemDeselected)
        EVT_LIST_DELETE_ITEM(self, self._id, self.itemDeleted)
	count = 0
	for label in self._labels:
	    self.InsertColumn(count, self._labels[count])
	    count += 1

    def addFile(self, file,trans_id, transferType):   
        
        dbg_str = ("%s%s%d\n" %("in ADD file for ProgressList: ", file, trans_id)) 
        LogPrintDebug(dbg_str)
        offset = self.GetItemCount()
        self.InsertStringItem(offset, transferType)
        id  = self.SetStringItem(offset,1, file)
        self.adjustColumnWidth()
        dbg_str =("%s%d\n" %("File is at index: ", id))
        LogPrintDebug(dbg_str)
        return offset

    def removeFile(self, id):
        """Locks the GUI"""
        dbg_str = ("%s%d\n"%("in Remove File in Progress List, index is: ", id))
        LogPrintDebug(dbg_str)
        wxMutexGuiEnter()
        deleted = self.DeleteItem(id)
        wxMutexGuiLeave()
        dbg_str = ("%s%d\n" %("in Remove File in Progress List, deleted is: ", deleted))
        LogPrintDebug(dbg_str)
        
    def itemDeleted(self, id):
        LogPrintDebug("In item Deleted\n")
        
    def adjustColumnWidth(self):
	for count in range(len(self._labels)):
	    self.SetColumnWidth(count, wxLIST_AUTOSIZE)   
       
    def itemSelected(self, event):
        selected = event.GetIndex()
        self._fileSelected.append(selected)

    def itemDeselected(self, event):
        deselected = event.GetIndex()
        self._fileSelected.remove(deselected)

    def getFileSelection(self):
        return self._fileSelected
    
class ProgressPanel(wxPanel):
    def __init__(self, parent):
        wxPanel.__init__(self,parent, -1)
        self._list = ProgressList(self)
        self._transfer_dict = {}
        self._mutex = Lock()
        sz = wxBoxSizer(wxVERTICAL)
        sz.Add(self._list, 1, wxEXPAND)
        self.SetSizer(sz)
        self.SetAutoLayout(true)
        defaultPanelColor(self)
        return

    def addFile(self, file,trans_id, transferType):
        self._mutex.acquire()
        LogPrintDebug("Mutex acquired for addFile \n")
        id = self._list.addFile(file, trans_id, transferType)
        LogPrintDebug("Adding a File to the Progress Panel\n")
        self._transfer_dict.update({trans_id: id})
        dbg_str = "Progress Panel transfer dictionary: " + str(self._transfer_dict) + '\n'
        LogPrintDebug(dbg_str)
        LogPrintDebug("Mutex released in ProgressPanel addFile\n")
        self._mutex.release()
        
    def removeFile(self, trans_id):
        self._mutex.acquire()
        LogPrintDebug("Mutex acquired for RemoveFile in Progress Panel\n")

        id = self._transfer_dict[trans_id]

        
        self._list.removeFile(id)
        del self._transfer_dict[trans_id]
        for item in self._transfer_dict.items():
            if item[1] > id:
                newId = item[1] - 1
                tmpD = {item[0]: newId}
                self._transfer_dict.update(tmpD)
        dbg_str = "Progress Panel transfer dictionary: " + str(self._transfer_dict) + '\n'
        LogPrintDebug(dbg_str)
        LogPrintDebug("Mutex released in removeFile\n")
        self._mutex.release()

        
    def getFileSelection(self):
        index_list = self._list.getFileSelection()
        trans_id_list = []
        for index in index_list:
            for item in self._transfer_dict.items():
                if index == item[1]:
                    trans_id_list.append(item[0])
                    
        return trans_id_list
        
         
############################################################################
class BitmapButtonPanel(wxPanel):
    def __init__(self,parent,queue,buttons):
        """
        The 'action' is the class which contains the button action
        functions.
        Set up and draw the buttons.
        """
	#actions = ConnectActions(queue)
	wxPanel.__init__(self,parent,-1)
        self._action = queue
        self.buttons = buttons
	runaround = 10
	blist = self.buttons.items()
	sz = wxBoxSizer( wxVERTICAL)
        sz1 = wxBoxSizer(wxVERTICAL)
        sz.Add(sz1,1, wxEXPAND)
        count = 0
	blist.sort(lambda x,y: cmp(x[1],y[1]))
        for b in blist:
	    bmp = wxBitmap(b[0], wxBITMAP_TYPE_BMP)
	    btn = wxBitmapButton(self,b[1],bmp)
            EVT_BUTTON(self,b[1], self.OnClick)
	    sz.Add(btn,1, wxALIGN_CENTER)
            count +=1
            if not count == len(blist):
                sz.Add(sz1,1.5, wxEXPAND)

        sz.Add(sz1,1, wxEXPAND)
        self.SetAutoLayout(1)
	self.SetSizer(sz)
	defaultPanelColor(self)
	return
############################################################################
class TransferActions(Actions):
    def __init__(self,queue):
	Actions.__init__(self, queue)
	return
    
class TransferButtonPanel(BitmapButtonPanel):
    def __init__(self,parent, queue):
	actions = TransferActions(queue)
	BitmapButtonPanel.__init__(self, parent, actions,
		      {'images/right_arrow.bmp': mWidgetCodes['ID_RIGHT_BMP'],
		       'images/cancel.bmp': mWidgetCodes['ID_CANCEL'],
			 'images/left_arrow.bmp': mWidgetCodes['ID_LEFT_BMP']})

    def setConfigs(self, configPanel):
        self._configPanel = configPanel
        
    def OnClick(self, event):
        global g_Control
        id = event.GetId()
        if id == self.buttons['images/right_arrow.bmp']:
            g_Control.doPut()
        if id == self.buttons['images/left_arrow.bmp']:
            g_Control.doGet()
	if id == self.buttons['images/cancel.bmp']:
	    g_Control.doCancel()
           


############################################################################
class PathInfo:
    """Object to represent a pathname and the information from the
    associated inode.

    pathinfo.py:  Object for manipulating information about pathnames
    and their related i-nodes.

    Written by John W. Shipman (john@nmt.edu), New Mexico Tech
    Computer Center, Socorro, NM 87801, in March 1997.

    Freely available under the terms of the GNU General Public
    License.

    This class defines methods for the ten standard components
    of the os.stat() value, giving information about file status.


    
    """

#----------------------------------------------------------------
# State
#----------------------------------------------------------------
# .path == The path passed to __init__
#--
# .stat == The file status tuple
#--
# .mode == The mode bits from the file status tuple
#--

    def __init__ ( self, pathName ):
        self.path  =  pathName
        self.stat  =  os.lstat(pathName)
        self.mode  =  self.stat[stat.ST_MODE]

    def __str__ ( self ):
        baseName = (os.path.split(self.path))[1]
        return ( "%s  %s  %8d %s" %
                 ( self._modeFlags(), self._modTime(), self.size(),
                   baseName ) )

    def _modeFlags ( self ):
        return self._dirFlag() + self._permFlags()

    def _dirFlag ( self ): 
        """ [ if self is a directory -> return "d"
              else if self is a symbolic link -> return "l"
              else if self is a regular file -> return "-"
              else -> return "?"
            ]
        """
        if stat.S_ISLNK(self.mode):
            return "l"
        elif stat.S_ISDIR(self.mode):
            return "d"
        elif stat.S_ISREG(self.mode):
            return "-"
        else:
            return "?"

    def _permFlags ( self ):
        """Return the `rwxrwxrwx' part of ls output
        """
        return ( self._rwx ( self.mode & stat.S_IRUSR,
                             self.mode & stat.S_IWUSR,
                             self.mode & stat.S_IXUSR ) +
                 self._rwx ( self.mode & stat.S_IRGRP,
                             self.mode & stat.S_IWGRP,
                             self.mode & stat.S_IXGRP ) +
                 self._rwx ( self.mode & stat.S_IROTH,
                             self.mode & stat.S_IWOTH,
                             self.mode & stat.S_IXOTH ) )

    def _rwx ( self, r, w, x ):
        """ [ return a string of the form "abc" where:
                a  :=  "r" if r is nonzero, else "-"
                b  :=  "w" if w is nonzero, else "-"
                c  :=  "x" if x is nonzero, else "-"
            ]
        """
        return ( self._dasher ( r, "r" ) +
                 self._dasher ( w, "w" ) +
                 self._dasher ( x, "x" ) )

    def _dasher ( self, b, c ):
        """ [ if b is true -> return c
              else -> return "-"
            ]
        """
        if b:
            return c
        else:
            return "-"

    def _modTime ( self ):
        """ [ return self's modification time as "yyyy-dd-mm hh:mm:ss"
            ]
        """
        return time.strftime ( "%Y-%m-%d %H:%M:%S",
                               time.localtime ( self.stat[stat.ST_MTIME] ) )

    def size ( self ):
        return self.stat[stat.ST_SIZE]

    def ownerCanRead ( self ):
        return self.mode & stat.S_IRUSR

    def ownerCanWrite ( self ):
        return self.mode & stat.S_IWUSR

    def ownerCanExec ( self ):
        return self.mode & stat.S_IXUSR

    def groupCanRead ( self ):
        return self.mode & stat.S_IRGRP

    def groupCanWrite ( self ):
        return self.mode & stat.S_IWGRP

    def groupCanExec ( self ):
        return self.mode & stat.S_IXGRP

    def worldCanRead ( self ):
        return self.mode & stat.S_IROTH

    def worldCanWrite ( self ):
        return self.mode & stat.S_IWOTH

    def worldCanExec ( self ):
        return self.mode & stat.S_IXOTH

    def isDir ( self ):
        return stat.S_ISDIR ( self.mode )

    def isLink ( self ):
        return stat.S_ISLNK ( self.mode )

    def isFile ( self ):
        return stat.S_ISREG ( self.mode )

    def isRootDir(self):
	return self.path == '/'
               
	

     
###############################################################################

def actionButtonColor( button ):
    """
    Set default color for buttons
    """
    button.SetForegroundColour( wxBLACK )
    
###############################################################################


def defaultPanelColor( win ):
    """
    Set default color for panels
    """
    BGColor = getColor("BGColor")
    win.SetBackgroundColour(BGColor)
    FGColor = getColor("FGColor")
    win.SetForegroundColour(FGColor)
    return

def getColor(colorString):
    r = range(3)
    list = []
    for x in r:
        tmpString = colorString + str(r[x])
        colorNum = g_config.getint("Colors", tmpString)
        list.append(colorNum)

    color = wxColour(list[0], list[1], list[2])
    return color


def main():
    global g_Log
    try:
        opt, arg = getopt.getopt(sys.argv[1:], 'dl')
    except getopt.error:
        print "bad usage expecting -l"
        print opt    
    for o,a  in opt:
        if   o == '-l' and a == '':
            g_Log = Log('./log_ftpClient')
        elif o == '-l' and not a == '':
            g_Log = Log(a)

        elif o == '-d'  and a == '':
            g_Log = Log('./log_ftpClient', 1)
        elif o == '-d' and not a =='':
            g_Log = Log(a, 1)
        else:
            g_Log = None
            
    app = MyApp(0)
    app.MainLoop()

if __name__ == "__main__": main()
