# ------------------------------------------------------------------------------
# 
# ------------------------------------------------------------------------------
import time, ConfigParser, os, sys, threading, getopt
from pyGlobus import ftpControl
from pyGlobus.ftpClient import FtpClient, HandleAttr, OperationAttr, DebugPlugin, \
     RestartMarkerPlugin, RestartMarker
from pyGlobus import gramClient
from pyGlobus.util import Buffer, GlobusException

NAME = 'pyGridAdmin'
DEBUG_LEVEL = 0

class simpleGridTests:
    def __init__(self):
        self._ftpC = simpleFtpClient()
        self._gc = simpleGramClient()
        return
    def run_authentication(self, host):
        raise NotImplementedError, 'not implemented'
    def run_job_test(self, host, test='/bin/date'):
        raise NotImplementedError, 'not implemented'
    # ----------------------------------
    # hosts_d: success -> None
    #          failure -> error message
    def gatekeeper_authentication(self, hosts_l):
        hosts_d = {}
        for i in hosts_l:
            err = self.run_authentication(i)
            hosts_d.update({i:err})
        return hosts_d
    def gatekeeper_job_test(self, hosts_l):
        hosts_d = {}
        for i in hosts_l:
            err = self.run_job_test(i)
            hosts_d.update({i:err})
        return hosts_d   

    
class GridTests1(simpleGridTests):
    def __init__(self):
        simpleGridTests.__init__(self)
        return
    def run_authentication(self, host):
        fd_r, fd_w = os.pipe()
        pid = os.fork()
        if not pid:
            os.close(fd_r)
            sys.stdout = fd_w
            sys.stderr = fd_w
            os.execl('/home/local/grid/globus/globus-2.0/beta/redhat-7.2/bin/globusrun',
                     NAME, '-a', '-r', host)
            
        os.close(fd_w)
        msg = os.read(fd_r, 1024)
        os.close(fd_r)
        print "PARENT:\n", msg
        return
    def run_job_test(self, host, test='/bin/date'):
        return


    
class simpleFtpClient:
     def __init__(self):
        
        self._ftpClient = None
        self._condV = threading.Condition(threading.Lock())

     def verbose_list(self, host, path):
        url  = "gsiftp://" + host + path
        self._result = ''
        self._buffer = Buffer(64*1024)
        handleAttr = HandleAttr()
        if DEBUG_LEVEL:
            try:
                debug_plugin = DebugPlugin(sys.stdout, ('DebugPlugin'))
            except PluginException, ex:
                print "NO DEBUG PLUGIN"
            else:
                handleAttr.add_plugin(debug_plugin)
                
        self._ftpClient = FtpClient(handleAttr)
        opAttr = OperationAttr()
        opAttr.set_mode(ftpControl.MODE_EXTENDED_BLOCK)
        try:
            self._ftpClient.verbose_list(url,opAttr, self.done_func, self._condV)
            self._cb_handle = self._ftpClient.register_read(self._buffer, self.small_read_cb, 0)
        except Exception, ex:
            raise
        else:
            self._condV.acquire()
            self._condV.wait()
            self._condV.release()
            self._ftpClient.free_callback(self._cb_handle)
        return self._result

     def small_read_cb(self, arg, handle, buffer, bufHandle, bufLen, offset, eof, error):
         print "small_read_cb"
         self._result += str(buffer)
         if not eof:
             try:
                 handle = self._ftpClient.register_read(self._buffer, self.read_cb, 0)
                 if self._cb_handle != None:
                     self._ftpClient.free_callback(self._cb_handle)
                 self._cb_handle = handle

             except Exception, ex:
                 raise
         return
    
     def read_cb(self, fd, handle, buffer, bufHandle, bufLen, offset, eof, error):
         if DEBUG_LEVEL:
             print "read_cb"
             
         fd.write(buffer)
         if not eof:
             try:
                 handle = self._ftpClient.register_read(self._buffer, self.read_cb, fd)
                 
                 if self._cb_handle != None:
                     self._ftpClient.free_callback(self._cb_handle)
                 
                 self._cb_handle = handle
                
             except Exception, ex:
                 fd.close()
                 raise
         return
    # --------------------------------------------------------------------------
    # --------------------------------------------------------------------------
     def get(self, src_url, dst_file):
         self._cb_handle = None
         self._buffer = Buffer(64*1024)
         fd = open(dst_file, "w")
         handleAttr = HandleAttr()
         #plugin = RestartMarkerPlugin(self.beginCB, self.markerCB, self.completeCB, None)
         #handleAttr.add_plugin(plugin)
         self._ftpClient = FtpClient(handleAttr)
         if DEBUG_LEVEL:
             try:
                 debug_plugin = DebugPlugin(sys.stdout, "Get")
             except PluginException, ex:
                 print "NO DEBUG PLUGIN"
             else:
                 handleAttr.add_plugin(debug_plugin)
         try:                  
             plugin = RestartMarkerPlugin(self.beginCB, self.markerCB, self.completeCB, None)
         except PluginException, ex:
             if DEBUG_LEVEL:
                 print "NO RESTART PLUGIN"
             else:
                 handleAttr.add_plugin(plugin)
         try:
             self._ftpClient.get(src_url, self.done_func, self._condV)
             handle = self._ftpClient.register_read(self._buffer, self.read_cb, fd)
             
         except GlobusException, ex:
             fd.close()
             raise
         
         self._condV.acquire()
         self._condV.wait()
         self._condV.release()
         fd.close()
         self._ftpClient.free_callback(handle)
    
     def put(self, src_file, dst_url):
         print "Put"
         self._cb_handle = None
         fd = open(src_file, "r")
         handleAttr = HandleAttr()
         if DEBUG_LEVEL:
             try:
                 debug_plugin = DebugPlugin(sys.stdout, "Put")
             except PluginException, ex:
                 print "NO DEBUG PLUGIN"
             else:
                 handleAttr.add_plugin(debug_plugin)
         try:                  
             plugin = RestartMarkerPlugin(self.beginCB, self.markerCB, self.completeCB, None)
         except PluginException, ex:
             if DEBUG_LEVEL:
                 print "NO RESTART PLUGIN"
         else:
             handleAttr.add_plugin(plugin)
            
         self._ftpClient = FtpClient(handleAttr)
         opAttr = OperationAttr()
         opAttr.set_mode(ftpControl.MODE_EXTENDED_BLOCK)
         self._ftpClient.put(dst_url, self.done_func, self._condV, opAttr)
         size = 64*1024
         oset = 0
         self._buffer = fd.read(size)
         try:
             handle = self._ftpClient.register_write(self._buffer, len(self._buffer), oset,
                                                             self._buffer == "", self.write_cb, fd)
         except GlobusException, ex:
             fd.close()
             raise
         
         self._condV.acquire()
         self._condV.wait()
         self._condV.release()
         fd.close()
         self._ftpClient.free_callback(handle)
         return
    
     def write_cb(self, fd, handle, buffer, bufHandle, bufLen, offset, eof, error):
         if DEBUG_LEVEL:
             print "write_cb: "
         if not eof:
             buf = fd.read(bufLen)
             bufLen = len(buf)
             try:
                
                 handle = self._ftpClient.register_write(buf, len(buf),
                                                    offset + bufLen, buf == "",
                                                    self.write_cb, fd)
                 if self._cb_handle != None:
                     self._ftpClient.free_callback(self._cb_handle)
                 self._cb_handle = handle
             except Exception, ex:
                 print "Exception: ", ex
                 fd.close()
                 raise

         if DEBUG_LEVEL:
             print "leaving write_cb"
         return
    
     def done_func(self, cv, handle, error):
         if DEBUG_LEVEL:
             print "Xinside done func"
             print "XXX: ", error

         if error or (error != "NONE"):
             print "The following error occured: %s" %(error)
         else:
             print "No Errors"
         cv.acquire()
         cv.notify()
         cv.release()
         return
    
     def beginCB(self, arg, handle, srcUrl, destUrl, marker):
         print "beginCB"
         return 0
    
     def markerCB(self, arg, handle, marker):
         print "markerCB"
         return
    
     def completeCB(self, handle, err, url):
         print "completeCB"
         return

class simpleGramClient:
    def __init__(self):
        try:
            self._gc = gramClient.GramClient()
        except gramClient.GramClientException, ex:
            print "\nCredentials Expired: run grid-proxy-init or wgpi first\n"
            sys.exit(-1)
        return

     # --------------------------------------------------------------------------
    # --------------------------------------------------------------------------
    def run_authentication(self, host):
        args = None
        try:
            self._gc.ping(host)
        except gramClient.GramClientException, ex:
            args = ex.args
        return args
    # --------------------------------------------------------------------------
    # --------------------------------------------------------------------------
    def run_job_test(self, host, test='/bin/date'):
        self._condV.acquire(1)
        cb = self._gc.set_callback(self.run_job_cb, self._condV)
        args = None
        try:
            tp = self._gc.submit_request(host, "&(executable=/bin/date)",
                                         gramClient.JOB_STATE_ALL, cb)
        except gramClient.GramClientException, ex:
            args = ex.args
        except Exception, ex:
            args = ex.args
        else:
            self._condV.wait()
        self._condV.release()
        self._gc.remove_callback(cb)
        return args
    def run_job_cb(self, cv, contact, state, error):
        if DEBUG_LEVEL:
            if state == 1:   print "Job is pending"
            elif state == 2: print "Job is active"
            elif state == 4: print "Job failed"
            elif state == 8: print "Job is done"
        if state == 4 or state == 8:
            cv.acquire()
            cv.notify()
            cv.release()
        return
    
    
        
    # --------------------------------------------------------------------------
    # --------------------------------------------------------------------------
  
    
   
   
class AuthenticationError(Exception):
    def __init__(self, args=None):
        Exception.__init__(self, args)
        #self.args = args
        return
    
def main():
    global DEBUG_LEVEL
    init_d = {'command_line':0, 'debug_level':0, 'authenticate':0,
              'submit_job':0, 'put':0, 'get':0, 'list':0}
    try:
        opt, arg = getopt.getopt(sys.argv[1:], 'cajk:d:n:p:g:l:')
    except getopt.error:
        usage(sys.argv)
    for o,a  in opt:
        if   o == '-c': init_d.update({'command_line':1})
        elif o == '-a': init_d.update({'authenticate':1})
        elif o == '-j': init_d.update({'submit_job':1})
        elif o == '-k':
            g = eval(a)
            if type(g) != type([]): usage(sys.argv)
            init_d.update({'gatekeeper_list':eval(a)})
        elif o == '-d': init_d.update({'debug_level':int(a)})
        elif o == '-n': init_d.update({'netlog_url':a})
        elif o == '-p': init_d.update({'put':eval(a)})
        elif o == '-g': init_d.update({'get':eval(a)})
        elif o == '-l': init_d.update({'list':eval(a)})
        else: usage(sys.argv)
        
    if init_d.get('debug_level'):
        DEBUG_LEVEL = init_d.get('debug_level')
    if init_d.get('command_line'):
        gc = simpleGramClient()
        ftpC = simpleFtpClient()
        #gt = simpleGridTests()
        gk_l = init_d.get('gatekeeper_list')
        if init_d.get('authenticate'):
            auth_d = gc.gatekeeper_authentication(gk_l)
            print "Authentication ---- "
            for i in auth_d.keys():  print "%s: %s" %(i, auth_d.get(i))
        if init_d.get('submit_job'):
            job_d = gc.gatekeeper_job_test(gk_l)
            print "JobManager -------- "
            for i in auth_d.keys():print "%s: %s" %(i, job_d.get(i))
        if init_d.get('put'):
            src_file, dst_url = init_d.get('put')
            result = ftpC.put(src_file, dst_url)
            print "put -------- "
            print "\tresult = ", result
        if init_d.get('get'):
            src_url, dst_file = init_d.get('get')
            result = ftpC.get(src_url, dst_file )
            print "get -------- "
        if init_d.get('list'):
            host, path = init_d.get('list')
            result = ftpC.verbose_list(host, path )
            print "get -------- \n", result

    """
    else:
        FtpClientGui.init_globals()
        app = GridAdminApp()
        app.MainLoop()
    """
    return


    
def usage(argList):
    print "usage:\n" + argList[0] + '\n'
    for o,a in argList[1:]:
        if o[0] == '-' and a[0] == '-':
            print ('\t' + o + ' ' + a + '\n')
        else:
            print ('\t' + o + '\n\t' + a + '\n')
    sys.exit(1)

if __name__ == "__main__": main()
