############################################################################
# Noah J. Edelson, LBNL
# Copyright (c) 2003, The Regents of the University of California,
# through Lawrence Berkeley National Laboratory
# (subject to receipt of any required approvals from the U.S. Dept. of Energy).
# All rights reserved.
###########################################################################
 
import sys
import os
import re
import string
import glob
from distutils.spawn import spawn

"""
enforce good first tripple quote alignment
write function which gives another arg to deal_xxx, that is- firstline, itemline?
"""

noisy     = 0

formatIt  = 1
warningOn = 1
returnOn  = 1
paramOn   = 1
raisesOn  = 1
extendsOn = 1



def findmode(line):
    if  line.find("**XXX**") > -1 and warningOn:
        return '@warning'
    if  (line.find("**Returns:**") > -1 or line.find("**Returns**") > -1 or line.find('*Returns*') > -1) and returnOn:
        return '@ret'
    if (line.find("**Arguments:**") > -1 or line.find("**Arguments**") > -1 or line.find("**Args**") > -1 \
        or line.find("**Arguements") > -1 or line.find("**Arguements:**") > -1) and paramOn:
        return '@param'
    if (line.find("**Raises:**") > -1 or line.find("**Raises**") > -1 or line.find("*raises*") > -1 or \
        line.find("*Raises*") > -1 or line.find("**Throws**") > -1) and raisesOn:
	return '@raise'
    if (line.find("**Extends:**") > -1 or line.find("**Extends**") > -1) and extendsOn:
        return '@extends'
    return 'none'

def indent_line(line, indent, debug=0):
    a = 0
    padding = ' '
    if debug == -1:
	padding = '*'
    if debug == -2:
	padding = '%'
    while line[0]==' ' or line[0]=='\t':
	line = line[1:]   # clear white space
    line = padding*indent + line # indent ' ' times 
    return line

def deal_param(line, paramname, indent):
    if line.find('- *') > -1:
        line = line.replace("- *%s*" % paramname, "@param %s:" % paramname)
    else:
        line = line.replace("*%s*" % paramname, "@param %s:" % paramname)

   # if -1 < indent_line(line,0).find("None") < 6 and line.find("*") and line.__len__() < 8:
	#print '$$$$',line
	#return ' '
    return line

def hackit(line):
    line = line.replace('-@param','@param')
    line = line.replace('L{unimplemented}','B{unimplemented}')
    line = line.replace('L{GassServerEZExceptions}','L{GassServerEZException}')
    line = line.replace('L{pyGlobus.security.GSI.GSIException}','L{pyGlobus.security.GSIException}')
    line = line.replace('L{pyGlobus.io.IOBase.IOBaseException}','L{pyGlobus.io.IOBaseException}')
    line = line.replace('L{pyGlobus.io.IOBase.IOBase}','L{pyGlobus.io.IOBaseException}')
    line = line.replace('L{GSSCredException}','B{GSSCredException}')

    return line

def deal_return(line, paramname, indent):    
    if line.find('-') > -1 and line.find('-') < 4:
	line = indent_line(line, indent+2)
    else:
	line = indent_line(line, indent)
    return line

def deal_raise(line, paramname, indent):

    line = indent_line(line, 0)
    apattern = ["A ", "a ", "An ", "an "]

    if line.find('one') > -1 and line.__len__() < 5:
	return ''
    for word in apattern:

	if (line.find(word) > -1 and line.find(word) < 4 and line.find('xception') > -1) or line.find('aises a')>-1:   # search through apattern
	    if line.find('aises a') > -1:
		paramname = ""	
		start = line.find('aises a')+8
		substring = line[start:]
		end   = substring.find(" ")-1
		paramname = line[start:start+end+1]
	    else:
		paramname = ""	
		start = line.find(word)+word.__len__()
		substring = line[start:]
		end   = substring.find(" ")-1
		paramname = line[start:start+end+1]

	    line = '@raise ' + paramname + ': A' + indent_line(line[start-1:], indent - 'raises'.__len__())

	    line = line.replace('A  '+paramname, 'A L{'+paramname+'}')
	    line = indent_line(line, indent)
	    #print 'rxxx: ',line

    line = indent_line(line,indent)
    return line


def deal_extends(line, paramname, indent):
    #line = indent_line(line, indent)
    if line.count("*") > 1:
	line = '- extends L{' + paramname + '}\n'        # change to L to cause bad-link import module epydoc crash. <sigh>
	line = '   ' + indent_line(line,indent)
	#print line
    return line

def throwaway(line):
    if findmode(line) == 'none':
	return 0
    else:
	return 1

# ****************************************************************************
# ****************************************************************************
# ****************************************************************************
# ****************************************************************************

def get_secondary(line, lastmode, indent):
    paramname = get_param(line) 
    offset = indent
    if lastmode == '@param':
	line = deal_param(line,paramname, offset)
	if line.find('none') > -1 or line.find('None') > -1 and indent_line(line,0).__len__() < 8:
	    return ''
	return line
    if lastmode == '@ret':
	line = deal_return(line, paramname, offset)
	return line
    if lastmode == '@raise':
	line = deal_raise(line, paramname, offset)
	return line
    if lastmode == '@extends':
	line = deal_extends(line, paramname, offset)
	return line

# ****************************************************************************

def get_primary(line, indent):
    tokenmode = findmode(line)
    if tokenmode == '@param':  # first occurance, skip text out
	return ''
    if tokenmode == '@warning':  # fix later
	line = line.replace("***XXX***", "@warning: ")
	line = line.replace("**XXX**", "@warning: ")
	line = line.replace("*XXX*", "@warning: ")
	return line
    if tokenmode == '@ret':
	line = '@return:\n'
	return line
    if tokenmode == '@raise':
	line = ""
	return line 
    if tokenmode == '@extends':
	return ' '
    return ' '

# ****************************************************************************
def get_param(line):
    paramstart = line.find('*')+1
    paramend = paramstart + line[paramstart:].find('*')
    paramname = line[paramstart:paramend]		    
    if paramname.endswith(':'):
	paramname = paramname[:-1]    # strip the : from the paramname
    return paramname


# ****************************************************************************

lastmode = 'none'

def docstring_format(line, indent):
    global lastmode
    line = indent_line(line, indent)
    if findmode(line)!='none':                              # primary (first) line
	lastmode = findmode(line) 
	line     = get_primary(line, indent)
	if lastmode=='@ret':
	    line = indent_line('@return:', indent)
    else:                                                   # who knows!?
	if lastmode=='none':
	    if indent_line(line,indent).find('-') > -1 and indent_line(line,indent).find('-') <= 2:
		line =  line
	    else:
		line = indent_line(line,indent)
		
	else:                                               # secondary (assigned) lines
	    line = get_secondary(line, lastmode, indent)
    return line

# ****************************************************************************
# ****************************************************************************
def epydoc_format(fp):
    global lastmode
    data = ''
    in_docstring = 0
    indent = 0
    def_indent = 0
    class_indent = 0
    lastindent = 'none'

    while 1:
	line = fp.readline()
	if line.__len__() < 1:
	    break

	if line.find("class ") > -1 and line.find(":") > -1 and not in_docstring:
	    class_indent = line.find("class ") + 4
	    lastindent = 'class'

        if line.find("def ") > -1  and not in_docstring:
            def_indent = line.find("def ") + 4 
            lastindent = 'def'

	if line.find('"""') > -1:
	    if line.count('"""') > 1:                     # more than one """ per line
		data += line
		continue

	    in_docstring = not in_docstring

	    if not in_docstring:                          # lower """ handler
		line = indent_line(line, indent)
		lastmode = 'none'
		data += '\n'+line+'\n'
		continue

	    else:                                         # uppper """ handler
		#indent  =  line.find('"""')	    
		#indent +=  line.find('\t') + 1
		if lastindent == 'class':
		    indent  = class_indent
		    line    =  indent_line('\n'+line+'\n', indent)		
		elif lastindent == 'def':
		    indent = def_indent
		    line    =  indent_line(line+'\n', indent)		
		else:
		    indent  =  def_indent #class_indent
		    line    =  indent_line(line, indent)		
		data += line
		#print line
		continue

	if in_docstring:
	    if formatIt:
		line = docstring_format(line,indent)      # program-heart-thread
		if line:
		    line = hackit(line)
		    barspace = line.find('-') - indent 
		    if barspace > -1 and barspace < 4 :
			line = '    ' + line
		    data = data + line
	else:                                             # PROGRAM CODE BELOW. DO NOT FORMAT.
	    data += line

    return data
# ****************************************************************************
# ****************************************************************************

def replace_old_source():
    for fname in sys.argv:
	if fname.find(".epydocify.py") > -1 and fname != "epydocify.py":	    
	    newname = fname[:-13]
	    if noisy:
		print fname,'\t-->\t' ,newname
	    spawn(['mv', fname, newname])
	else:
	    if noisy:
		print fname, ' is not a valid epydocify file.'

def cleanshite():
    rmfiles = glob.glob("*.epydocify.py")
    for x in rmfiles:
	spawn(['rm',x])
	    
def main():
    for arg in sys.argv:
        if arg == '-help' or arg == '--help':
            print '\n\nepydocify: a HAPPYDOCish -> EPYDOC docstring converter'
            print 'USAGE: python epydocify.py [infile.py] [--clean] [--replace-old-source dir]'
            print '--replace-old-source   easy way to cp *.epydocify.py -> *.py'
            print '--clean                easy way to rm *.epydocify.py'
            print '--help                 this help text\n\n'
	    sys.exit(0)
	if arg == '--replace-old-source':
	    replace_old_source()
	    return
	if arg == '--clean':
	    cleanshite()
	    return
    for arg in sys.argv[1:]:
	if arg.find(".epydocify.py") > -1:
	    continue
	fp = open(arg, "rt")
	data = fp.read()
	fp.seek(0)

	print "processing: ", arg
	data2 = epydoc_format(fp)

	fp.close()
	fp = open(arg+".epydocify.py","wt")
	fp.write(data2)
	fp.close()

if __name__ == "__main__" : main()
 
