############################################################################
# Keith R. Jackson, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################

"""
A module to generate the filelist file needed by GPT.
"""

from distutils.util import get_platform
import glob, os, sys, string
import distutils.sysconfig
import types

LIBDIR = 'lib'
libdir_list = distutils.sysconfig.get_config_vars("LIBDIR")
for libdirs in libdir_list:
    if type(libdirs) is types.StringType: 
        if libdirs.find('lib64') != -1: 
            LIBDIR = 'lib64' 

def _replace_dirname(l, platDir):
    '''
    Given a list of filenames, it will replace the "build/lib.linux-i686-2.2"
    with "lib/python". This is needed for making the GPT filelist.
    '''
    newL = [] 
    for i in l:
        i = string.replace(i, 'build', LIBDIR , 1)
        i = string.replace(i, platDir, 'python', 1)
        newL.append(i)
    return newL

def _add_doc_dir(l):
    """
    Fixes the path for the docs to include pyGlobus.
    """
    newL = []
    for i in l:
        i = string.replace(i, 'doc', os.path.join('doc', 'pyGlobus'), 1)
        newL.append(i)
    return newL

flist = []
plat = get_platform()
vers = sys.version_info
platDir = 'lib.%s-%i.%i' % (plat, vers[0], vers[1])
dir = 'build%s%s%spyGlobus%s*.py' % (os.sep, platDir, os.sep, os.sep)
dir2 = 'build%s%s%spyGlobus%stest%s*.py' % (os.sep, platDir, os.sep, 
                                                        os.sep, os.sep)
dir3 = 'build%s%s%spyGlobus%s*.so' % (os.sep, platDir, os.sep, os.sep)
tmp = glob.glob(dir)
tmp = _replace_dirname(tmp, platDir)
flist.extend(tmp)
for i in tmp:
    flist.append(i + 'c')
tmp = glob.glob(dir2)
tmp = _replace_dirname(tmp, platDir)
flist.extend(tmp)
for i in tmp:
    flist.append(i + 'c')
tmp = glob.glob(dir3)
flist.extend(_replace_dirname(tmp, platDir))
tmp = glob.glob('doc/*.css')
tmp = _add_doc_dir(tmp)
flist.extend(tmp)
tmp = glob.glob('doc/*.html')
tmp = _add_doc_dir(tmp)
flist.extend(tmp)
tmp = glob.glob('doc/private/*.html')
tmp = _add_doc_dir(tmp)
flist.extend(tmp)
tmp = glob.glob('doc/private/*.css')
tmp = _add_doc_dir(tmp)
flist.extend(tmp)
tmp = glob.glob('doc/public/*.html')
tmp = _add_doc_dir(tmp)
flist.extend(tmp)
tmp = glob.glob('doc/public/*.css')
tmp = _add_doc_dir(tmp)
flist.extend(tmp) 
print flist
f = open("filelist", "w")
for line in flist:
    f.write(line)
    f.write('\n')
f.close()
