#!/usr/local/bin/python
############################################################################
# Keith R. Jackson, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################

## Attempt a version string
__author__  = "Keith R. Jackson"
__status__  = "alpha"
__version__ = "$Revision: 1.10 $"
__rcsid__   = "$Id: pyglobusrun.py,v 1.10 2003/11/13 21:17:16 dek Exp $"

import getopt, sys, os
from threading import *
from pyGlobus.gramClient import GramClient, GramClientException
from pyGlobus import gramClient
from pyGlobus.gassServerEZ import GassServerEZ, STDOUT_ENABLE, STDERR_ENABLE
from pyGlobus.logUtil import logging , netlogger
PRINT_DEBUG  = 0
APPLICATION_EVENT_NAME = 'pyglobusrun'
NETLOGGER_DEST = 'x-netlog://netlogger.lbl.gov:14830'

errorMessage = None

def print_debug(printStr):
    if PRINT_DEBUG:
	print printStr
   	


def func(usrVars, contact, state, error):
    cv = usrVars[0]
    logger = usrVars[1]
    netlogger_writer = usrVars[2]
    if state == 1:
        print_debug("Job is pending")
        logger.info("%s Job is pending" % APPLICATION_EVENT_NAME)
        netlogger_writer.write(2, "%s.jobState.pending" % APPLICATION_EVENT_NAME,
                               "VAL=%d", 1)
    elif state == 2:
        print_debug("Job is active")
        logger.info("%s Job is active" % APPLICATION_EVENT_NAME)
        netlogger_writer.write(2, "%s.jobState.active" % APPLICATION_EVENT_NAME,
                               "VAL=%d", 1)
    elif state == 4:
        print_debug("Job failed")
        logger.error("%s error status: %s" % (APPLICATION_EVENT_NAME, error))
        netlogger_writer.write(0, "%s.jobState.done" % APPLICATION_EVENT_NAME,
                               "VAL=%d", 1)
        cv.acquire()
        global errorMessage
        errorMessage = error
        cv.notify()
        cv.release()
    elif state == 8:
        print_debug("Job is done")
        logger.info("%s Job has completed." % APPLICATION_EVENT_NAME)
        netlogger_writer.write(2, "%s.jobState.done" % APPLICATION_EVENT_NAME,
                               "VAL=%d", 1)
        cv.acquire()
        cv.notify()
        cv.release()
    else:
        print "Unrecognized state=", state
        
def pyglobusrun(rm, rsl, logger, netlogger_writer, outOn, callbackFunc=None):
    if outOn:
        opts = STDOUT_ENABLE | STDERR_ENABLE
        outServer = GassServerEZ(opts)
        outUrl = outServer.getURL()
        rsl = "%s(stdout=%s/dev/stdout)(stderr=%s/dev/stderr)" % (rsl, outUrl, outUrl)
    condV = Condition(Lock())
    logger.info("%s resource contact is: %s" % (APPLICATION_EVENT_NAME, rm))
    logger.info("%s RSL is: %s" % (APPLICATION_EVENT_NAME, rsl))
    netlogger_writer.write(2, "%s.start" % APPLICATION_EVENT_NAME, "VAL=%d CONTACT=%s", 1, rm)
##     try:
    gC = GramClient()
    gC.debug()
    if callbackFunc == None:
        cb = func
    else:
        cb = callbackFunc
    callbackContact = gC.set_callback(cb, (condV, logger,
                                             netlogger_writer))
    jobContact = gC.submit_request(rm, rsl, gramClient.JOB_STATE_ALL,
                                   callbackContact)
##     except GramClientException, ex:
##         print ex.args
##         sys.exit(-1)
    print_debug("sumbitted the job, waiting for callbacks")
    condV.acquire()
    condV.wait()
    print_debug("made it out of the wait")
    condV.release()
##     try:
    print_debug("trying to remove the callback")
    gC.remove_callback(callbackContact)
    print_debug("removed callback")
##     except GramClientException, ex:
##         print ex.args
##         sys.exit(-1)
    global errorMessage
    if errorMessage == None:
        netlogger_writer.write(2, "%s.end" % APPLICATION_EVENT_NAME, "VAL=%d ERROR=%s", 1, "None")
    else:
        netlogger_writer.write(0, "%s.end" % APPLICATION_EVENT_NAME, "VAL=%d ERROR=%s", -1, errorMessage)

def usage():
    useStr = """Usage: pyglobusrun [options] rsl
       OPTIONS 
            -r <resource manager contact> 
            -l <log level> | --logging-level=<log level>
                specify the Python logging package logging level (CRITICAL/ERROR, WARN, INFO, DEBUG)
            -n | --netlogger 
                Force netlogger to be enabled even if NETLOGGER_ON environment variable is not set
            --netlogger-destination=<destination> 
                specify the netlogger destination URL 
            --netlogger-level=<level> 
                specify the netlogger log level (ERROR/FATAL, WARN, INFO, DEBUG) 
            -o | --output-enable
                Use the GASS Server module to redirect standard out and standard
                error to pyglobusrun.
                
            NOTE: specifying a logging level will mask all messages logged with a priority level 
            "below" that of the set level.  IE, if you set the logging level to INFO, then DEBUG 
            level messages will not appear.  
    """
    print useStr
    
def getArgs():
    global PRINT_DEBUG 
    contact = rsl = netlogger_level = netlogger_dest = logger_level = None
    outOn = 0
    argLen = len(sys.argv)
    if argLen == 1:
        usage()
        sys.exit(-1)
    try:
        opts, args = getopt.getopt(sys.argv[1:], "hd:r:l:n:o", ["help", "usage", "logging-level=", 
                                        "netlogger", "netlogger-destination=", 
                                        "output-enable", "netlogger-level="])
    except getopt.GetoptError:
        usage()
        sys.exit(-1)
    for o, a in opts:
        if o in ("-h", "--help", "--usage"):
            usage()
            sys.exit(0)
        if o in ("-v", "--version"):
            print "pyglobusrun version %s" % __version__
            sys.exit(0)
        elif o == "-d":
	    PRINT_DEBUG = 1 
        elif o == "-r":
            contact = a
        elif o in ("-n", "--netlogger"):
            os.putenv("NETLOGGER_ON", "yes")
        elif o in ("-o", "--output-enable"):
            outOn = 1
        elif o == "--netlogger-destination":
            netlogger_dest = a

        elif o == "--netlogger-level":
            if a == 'DEBUG':
                netlogger_level = netlogger.Writer.DEBUG_LVL
            elif a == 'INFO':
                netlogger_level = netlogger.Writer.INFO_LVL
            elif a == 'WARN':
                netlogger_level = netlogger.Writer.WARN_LVL
            elif a == 'ERROR':
                netlogger_level = netlogger.Writer.ERROR_LVL
            elif a == 'FATAL':
                netlogger_level = netlogger.Writer.FATAL_LVL
            else:
                print "Unrecognized NetLogger logging level '%s'" % a
                sys.exit(1)
        elif o in ("-l", "--logging-level"):
            if a == 'DEBUG':
                logger_level = logging.DEBUG
            elif a == 'INFO':
                logger_level = logging.INFO
            elif a == 'WARNING':
                logger_level = logging.WARNING
            elif a == 'ERROR':
                logger_level = logging.ERROR
            elif a == 'CRITICAL':
                logger_level = logging.CRITICAL
            else:
                print "Unrecognized logging level '%s'" % a
                sys.exit(1)
    rsl = sys.argv[argLen -1]
    return contact, rsl, netlogger_dest, netlogger_level, logger_level, outOn

def main():
    rm, rsl, netlogger_dest, netlogger_level, logger_level, outOn = getArgs()
    ## set up error logging before anything else
    logger = logging.getLogger('pyglobusrun')
    ## TODO: command-line selectable stream type and warning level
    logger_handler = logging.StreamHandler(sys.stdout)
    logger_formatter = logging.Formatter('%(asctime)s %(levelname)s %(message)s')
    logger_handler.setFormatter(logger_formatter)
    logger.addHandler(logger_handler)
    logger.setLevel(logger_level)
    program = APPLICATION_EVENT_NAME;
    flags = netlogger.OutputFlags.ULM | netlogger.OutputFlags.FLUSH
    netlogger_writer = netlogger.open(netlogger_dest, flags = flags, 
                                    program = program, 
                                    threaded = 0, backup = None) 
    if netlogger_level: 
        netlogger_writer.setLevel(netlogger_level)
    logger.info("%s Programming Starting" % APPLICATION_EVENT_NAME)
    netlogger_writer.write(2, "%s_PROG_START" % APPLICATION_EVENT_NAME, "VAL=%d", 1)
    pyglobusrun(rm, rsl, logger, netlogger_writer, outOn)
    logger.info("%s Programming Ending" % APPLICATION_EVENT_NAME)
    netlogger_writer.write(2, "%s_PROG_END" % APPLICATION_EVENT_NAME, "VAL=%d", 1)
    logging.shutdown()

if __name__ == '__main__':
    main()

