/*############################################################################
# Keith R. Jackson, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################*/
%module ftpClientc
%{
#undef HAVE_LIMITS_H
#undef SIZEOF_OFF_T
#undef socklen_t
#include "globus_common.h"
#include "globus_ftp_client.h"
#include "globus_ftp_client_debug_plugin.h"
#include "globus_ftp_client_restart_marker_plugin.h"
#include "globus_ftp_client_perf_plugin.h"
#include "globus_ftp_client_throughput_plugin.h"
#include "pyGlobus_common.h"
/*
#if defined(GLOBUS_BUILD_WITH_NETLOGGER)
#include "globus_ftp_client_throughput_nl_plugin.h"
#endif
*/
%}

%include ./src/pyGlobus-typemaps.i

%types(globus_off_t*);
%types(globus_abstime_t*);
%types(globus_byte_t*);
%types(globus_ftp_control_auth_info_t*);
%types(globus_ftp_control_response_t*); 
%types(globus_object_t*);

typedef int globus_bool_t; 

%typemap(python, in) FILE * {
    if (!PyFile_Check($input)) {
        PyErr_SetString(PyExc_TypeError, "expected PyFile");
        return NULL;
    }
    $1=PyFile_AsFile($input);
}

%apply char **OUTPUT { char **usr, char** passwd, char **account,
                       char **subject, char** markerString };



%init %{
    PyEval_InitThreads();
    python_interp = PyThreadState_Get()->interp;
    init_globus_thread_state_key(&python_thread_state_key);
%}

%{

static globus_thread_key_t python_thread_state_key;
static PyInterpreterState *python_interp;




typedef struct {
     PyObject* cb1;
     PyObject* cb2;
     PyObject* cb3;
     PyObject* arg;
 } callbackStruct2;

typedef struct {
     PyObject* cb1;
     PyObject* cb2;
     PyObject* cb3;
     PyObject* cb4;
     PyObject* arg;
 } callbackStruct3;



typedef enum {
    COPY,
    DESTROY,
    GET,
    PUT,
    TRANSFER,
    ABORT,
    CONNECT,
    PLUGIN_DELETE,
    AUTH,
    READ,
    MOVE,
    LIST,
    PLUGIN_SIZE,
    VERBOSELIST,
    RMDIR,
    MKDIR,
    MODIFICATIONTIME,
    WRITE,
    DATA,
    COMMAND,
    RESPONSE,
    FAULT,
    COMPLETE
} pythonPluginList;

typedef struct {
  PyObject *python_function_list;
  PyObject *python_user_specific;
} pyglobus_plugin_t;

/* forward-declare the pyglobus_plugin_init function because it's used by C functions before it's defined */

static
globus_ftp_client_plugin_t *
pyglobus_plugin_init(PyObject *python_function_list);




PyObject * check_error(globus_object_t *  error){
    PyObject *tuple, *err;
    
    if(!(tuple = PyTuple_New(2))) {
    	PyErr_SetString(PyExc_RuntimeError,"check_error, PyTuple_New");	
    }
    else{ 
	if (error) {  /* SAME AS IF(ERROR==GLOBUS_SUCCESS)*/
            char *tmpstr;
	    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
	    tmpstr = globus_object_printable_to_string(error);
            PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
            globus_libc_free(tmpstr);
        }
	else{
	    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
            Py_INCREF(Py_None);
            PyTuple_SET_ITEM(tuple, 1, Py_None);
	}
    }	
    
    if ((err = PyErr_Occurred()) != NULL){
	PyErr_Print();
    }
    return tuple;
}


static void PythonCompleteCallback(void* user_data, 
                                   globus_ftp_client_handle_t* handle,
                                   globus_object_t* error) {
    
    PyObject *func, *arglist, *result, *arg, *ptr, *errorObject;
    char* err;
    callbackStruct* cbStruct = (callbackStruct*)user_data;
    PyEval_AcquireThread(get_python_thread_state(python_thread_state_key, python_interp));
    func = cbStruct->callback;
    arg = cbStruct->arg;
    ptr = SWIG_NewPointerObj((void*)handle, 
			     SWIGTYPE_p_globus_ftp_client_handle_t, 0);

    errorObject = check_error(error);
    arglist = Py_BuildValue("(OOO)", arg, ptr, errorObject);
    result = PyEval_CallObject(func, arglist);
    if (result == NULL) {
      PyErr_Print();
    }
    Py_DECREF(ptr);
    Py_DECREF(arglist);
    Py_XDECREF(result);
    Py_XDECREF(errorObject);
    PyEval_ReleaseThread(get_python_thread_state(python_thread_state_key, python_interp));
    return;
}


static void PythonDataCallback(void* user_data,
                               globus_ftp_client_handle_t* handle,
                               globus_object_t* error,
                               globus_byte_t* buffer,
                               globus_size_t length,
                               globus_off_t offset,
                               globus_bool_t eof) {
    PyObject *func, *arglist, *result, *arg, *buf;
    PyObject  *oSet, *ptr, *ptr2, *oldBuf, *errorObject;
    char* err;
    callbackStruct4 *cbStruct;
    PyEval_AcquireThread(get_python_thread_state(python_thread_state_key, python_interp));
    cbStruct = (callbackStruct4*)user_data;
    func = cbStruct->callback;
    arg = cbStruct->arg;
    oldBuf = cbStruct->buffer;
    if (sizeof(globus_off_t) == 8) {
        /* XXX, This assumes globus_off_t is a long long.  This code will
           probably have to be ifdef'd by platform to account for 
           different types for size_t */
        oSet = PyLong_FromLongLong(offset);
    }
    else {
        oSet = PyInt_FromLong(offset);
    }
    ptr = SWIG_NewPointerObj((void*)handle, 
			     SWIGTYPE_p_globus_ftp_client_handle_t, 0);
    ptr2 = SWIG_NewPointerObj((void*)buffer, SWIGTYPE_p_globus_byte_t, 0);
    buf = PyBuffer_FromReadWriteMemory((void*)buffer, length * sizeof(globus_byte_t));
    errorObject = check_error(error);
    arglist = Py_BuildValue("(OOOOiOiO)", arg, ptr, buf, ptr2, length, 
                            oSet, (int)eof,errorObject);
    result = PyEval_CallObject(func, arglist);
    if (result == NULL) {
      PyErr_Print();
    }
    Py_XDECREF(oldBuf);
    Py_DECREF(arglist);
    Py_XDECREF(result);
    Py_DECREF(oSet);
    Py_DECREF(buf);
    Py_DECREF(ptr);
    Py_DECREF(ptr2);
    Py_XDECREF(errorObject);
    PyEval_ReleaseThread(get_python_thread_state(python_thread_state_key, python_interp));
    return;
}
/*
// ------------------------------------------------------------------------------------
//  RestartMarker Callbacks
//
//
// ------------------------------------------------------------------------------------
*/
static void PythonRestartMarkerPluginCompleteCallback(void* user_data, 
                                                      globus_ftp_client_handle_t* handle,
                                                      globus_object_t* error,
                                                      const char* url) {
    PyObject *func, *arglist, *result, *arg, *ptr, *errorObject;
    char* err;
    callbackStruct2* cbStruct = (callbackStruct2*)user_data;
    PyEval_AcquireThread(get_python_thread_state(python_thread_state_key, python_interp));
    func = cbStruct->cb3;
    arg = cbStruct->arg;
    ptr = SWIG_NewPointerObj((void*)handle, 
			     SWIGTYPE_p_globus_ftp_client_handle_t, 0);
    errorObject = check_error(error);
    arglist = Py_BuildValue("(OOOs)", arg, ptr, errorObject, url);
    result = PyEval_CallObject(func, arglist);
    if (result == NULL) {
      PyErr_Print();
    }
    Py_DECREF(ptr);
    Py_DECREF(arglist);
    Py_XDECREF(result);
    Py_XDECREF(errorObject);
    PyEval_ReleaseThread(get_python_thread_state(python_thread_state_key, python_interp));
    return;
}

globus_bool_t PythonRestartMarkerPluginBeginCallback(void* user_data, 
                                                     globus_ftp_client_handle_t *handle,
                                                     const char* srcUrl,
                                                     const char* destUrl,
                                                     globus_ftp_client_restart_marker_t* marker) {
    PyObject *func, *arglist, *result, *arg, *ptr, *markerPtr;
    globus_bool_t ret;
    callbackStruct2* cbStruct = (callbackStruct2*)user_data;
    PyEval_AcquireThread(get_python_thread_state(python_thread_state_key, python_interp));
    func = cbStruct->cb1;
    arg = cbStruct->arg;
    ptr = SWIG_NewPointerObj((void*)handle, 
			     SWIGTYPE_p_globus_ftp_client_handle_t, 0);
    markerPtr = SWIG_NewPointerObj((void*)marker, 
                                   SWIGTYPE_p_globus_ftp_client_restart_marker_t, 0);
    arglist = Py_BuildValue("(OOssO)", arg, ptr, srcUrl, destUrl, markerPtr);
    result = PyEval_CallObject(func, arglist);
    if (result == NULL) {
      PyErr_Print();
    }
    if (PyInt_AsLong(result) == GLOBUS_SUCCESS) {
        ret = GLOBUS_SUCCESS;
    }
    else {
        ret = GLOBUS_FALSE;
    }
    Py_DECREF(ptr);
    Py_DECREF(markerPtr);
    Py_DECREF(arglist);
    Py_XDECREF(result);
    PyEval_ReleaseThread(get_python_thread_state(python_thread_state_key, python_interp));
    return ret;
}

void PythonRestartMarkerPluginMarkerCallback(void* user_data, 
                                            globus_ftp_client_handle_t *handle,
                                            globus_ftp_client_restart_marker_t* marker) {
    PyObject *func, *arglist, *result, *arg, *ptr, *markerPtr;
    callbackStruct2* cbStruct = (callbackStruct2*)user_data;
    PyEval_AcquireThread(get_python_thread_state(python_thread_state_key, python_interp));
    func = cbStruct->cb2;
    arg = cbStruct->arg;
    ptr = SWIG_NewPointerObj((void*)handle, 
			     SWIGTYPE_p_globus_ftp_client_handle_t, 0);
    markerPtr = SWIG_NewPointerObj((void*)marker, 
                                   SWIGTYPE_p_globus_ftp_client_restart_marker_t, 0);
    arglist = Py_BuildValue("(OOO)", arg, ptr, markerPtr);
    result = PyEval_CallObject(func, arglist);
    if (result == NULL) {
      PyErr_Print();
    }
    Py_DECREF(ptr);
    Py_DECREF(markerPtr);
    Py_DECREF(arglist);
    Py_XDECREF(result);
    PyEval_ReleaseThread(get_python_thread_state(python_thread_state_key, python_interp));
    return;
}
/*
// ------------------------------------------------------------------------------------
//  Performance Callbacks
//
//
// ------------------------------------------------------------------------------------
*/
static void 
PythonPerfMarkerPluginCompleteCallback(void* user_data, 
				       globus_ftp_client_handle_t* handle,
				       globus_bool_t success)
 {
    PyObject *func, *arglist, *result, *arg, *ptr, *tuple;
    char* err;
    callbackStruct2* cbStruct = (callbackStruct2*)user_data;
    PyEval_AcquireThread(get_python_thread_state(python_thread_state_key, python_interp));
    func = cbStruct->cb3;
    arg = cbStruct->arg;
    ptr = SWIG_NewPointerObj((void*)handle, 
			     SWIGTYPE_p_globus_ftp_client_handle_t, 0);

    if(!(tuple = PyTuple_New(2))) {
    	PyErr_SetString(PyExc_RuntimeError,"PythonPerfMarkerPluginCompleteCallback, PyTuple_New");	
    }
    else{ 
	if (success) {  
	    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
            Py_INCREF(Py_None);
            PyTuple_SET_ITEM(tuple, 1, Py_None);  
        }
	else{
	    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
            PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", "Error Transfer interrupted"));
	}
    }
	
   
    arglist = Py_BuildValue("(OOO)", arg, ptr, tuple);
    result = PyEval_CallObject(func, arglist);
    if (result == NULL) {
      PyErr_Print();
    }
    Py_DECREF(ptr);
    Py_DECREF(arglist);
    Py_XDECREF(result);
    PyEval_ReleaseThread(get_python_thread_state(python_thread_state_key, python_interp));
    return;
}

static void 
PythonPerfMarkerPluginBeginCallback(void* user_data, 
					 globus_ftp_client_handle_t *handle,
					 const char* srcUrl,
					 const char* destUrl,
					 globus_bool_t restart) {
    PyObject *func, *arglist, *result, *arg, *ptr;
    callbackStruct2* cbStruct = (callbackStruct2*)user_data;
    PyEval_AcquireThread(get_python_thread_state(python_thread_state_key, python_interp));
    func = cbStruct->cb1;
    arg = cbStruct->arg;
    ptr = SWIG_NewPointerObj((void*)handle, 
			     SWIGTYPE_p_globus_ftp_client_handle_t, 0);
    arglist = Py_BuildValue("(OOssi)", arg, ptr, srcUrl, destUrl, (int)restart);
    result = PyEval_CallObject(func, arglist);
    if (result == NULL) {
      PyErr_Print();
    }
    Py_DECREF(ptr);
    Py_DECREF(arglist);
    Py_XDECREF(result);
    PyEval_ReleaseThread(get_python_thread_state(python_thread_state_key, python_interp));
    return;
}

static void 
PythonPerfMarkerPluginMarkerCallback(void* user_data, 
					  globus_ftp_client_handle_t *handle,
					  long time_stamp_int,
					  char time_stamp_length,
					  int strip_ndx,
					  int num_stripes,
					  globus_off_t nbytes) {
    PyObject *func, *arglist, *result, *arg, *ptr, *tBytesPtr, *timeStampLength;
    callbackStruct2* cbStruct = (callbackStruct2*)user_data;
    PyEval_AcquireThread(get_python_thread_state(python_thread_state_key, python_interp));
    func = cbStruct->cb2;
    arg = cbStruct->arg;

    if (sizeof(globus_off_t) == 8) {
        /* XXX, This assumes globus_off_t is a long long.  This code will
           probably have to be ifdef'd by platform to account for 
           different types for size_t */
	tBytesPtr = PyLong_FromLongLong(nbytes);
    }
    else {
        tBytesPtr = PyInt_FromLong(nbytes);
    }
    ptr = SWIG_NewPointerObj((void*)handle, 
			     SWIGTYPE_p_globus_ftp_client_handle_t, 0);
    
    arglist = Py_BuildValue("(OOlciiO)", arg, ptr, time_stamp_int,time_stamp_length , 
    			    strip_ndx, num_stripes, tBytesPtr);
    result = PyEval_CallObject(func, arglist);
    if (result == NULL) {
      PyErr_Print();
    }
    Py_DECREF(tBytesPtr);
    Py_DECREF(ptr);
    Py_DECREF(arglist);
    Py_XDECREF(result);
    PyEval_ReleaseThread(get_python_thread_state(python_thread_state_key, python_interp));
    return;
}
/*
// ------------------------------------------------------------------------------------
//  Throughput Callbacks
//
//
// ------------------------------------------------------------------------------------
*/
void PythonThroughputPluginBeginCallback(void *user_data,
					 globus_ftp_client_handle_t *handle,
					 const char *source_url, 
					 const char *dest_url) {
    PyObject *func, *arglist, *result, *arg, *ptr;
    callbackStruct3* cbStruct = (callbackStruct3*)user_data;
    PyEval_AcquireThread(get_python_thread_state(python_thread_state_key, python_interp));
    func = cbStruct->cb1;
    arg = cbStruct->arg;
    ptr = SWIG_NewPointerObj((void*)handle, 
			     SWIGTYPE_p_globus_ftp_client_handle_t, 0);
    arglist = Py_BuildValue("(OOss)", arg, ptr, source_url, dest_url);
    result = PyEval_CallObject(func, arglist);
    if (result == NULL) {
      PyErr_Print();
    }
    Py_DECREF(ptr);
    Py_DECREF(arglist);
    Py_XDECREF(result);
    PyEval_ReleaseThread(get_python_thread_state(python_thread_state_key, python_interp));
    return;
}
void PythonThroughputPluginStripeCallback(void *user_data, 
					  globus_ftp_client_handle_t *handle,
					  int stripe_ndx,
					  globus_off_t nbytes,
					  float instantaneous_throughput,
					  float avg_throughput) {
    PyObject *func, *arglist, *result, *arg, *ptr, *tBytesPtr;
    callbackStruct3* cbStruct = (callbackStruct3*)user_data;
    PyEval_AcquireThread(get_python_thread_state(python_thread_state_key, python_interp));
    func = cbStruct->cb2;
    arg = cbStruct->arg;
    if (sizeof(globus_off_t) == 8) {
        /* XXX, This assumes globus_off_t is a long long.  This code will
           probably have to be ifdef'd by platform to account for 
           different types for size_t */
	tBytesPtr = PyLong_FromLongLong(nbytes);
    }
    else {
        tBytesPtr = PyInt_FromLong(nbytes);
    }
    ptr = SWIG_NewPointerObj((void*)handle, 
			     SWIGTYPE_p_globus_ftp_client_handle_t, 0);
    arglist = Py_BuildValue("(OOiOff)", arg, ptr, stripe_ndx, tBytesPtr, 
			    instantaneous_throughput, avg_throughput);
    result = PyEval_CallObject(func, arglist);
    if (result == NULL) {
	PyErr_Print();
    }
    Py_DECREF(ptr);
    Py_DECREF(arglist);
    Py_XDECREF(result);
    PyEval_ReleaseThread(get_python_thread_state(python_thread_state_key, python_interp));
    return;
}
void PythonThroughputPluginTotalCallback(void *user_data,
					 globus_ftp_client_handle_t *handle,
					 globus_off_t nbytes,
					 float instantaneous_throughput,
					 float avg_throughput) {
    PyObject *func, *arglist, *result, *arg, *ptr, *tBytesPtr;
    callbackStruct3* cbStruct = (callbackStruct3*)user_data;
    PyEval_AcquireThread(get_python_thread_state(python_thread_state_key, python_interp));
    func = cbStruct->cb3;
    arg = cbStruct->arg;
    if (sizeof(globus_off_t) == 8) {
        /* XXX, This assumes globus_off_t is a long long.  This code will
           probably have to be ifdef'd by platform to account for 
           different types for size_t */
	tBytesPtr = PyLong_FromLongLong(nbytes);
    }
    else {
        tBytesPtr = PyInt_FromLong(nbytes);
    }
    ptr = SWIG_NewPointerObj((void*)handle, 
			     SWIGTYPE_p_globus_ftp_client_handle_t, 0);
    arglist = Py_BuildValue("(OOOff)", arg, ptr, tBytesPtr,
			    instantaneous_throughput, avg_throughput);
    result = PyEval_CallObject(func, arglist);
    if (result == NULL) {
      PyErr_Print();
    }
    Py_DECREF(ptr);
    Py_DECREF(arglist);
    Py_XDECREF(result);
    PyEval_ReleaseThread(get_python_thread_state(python_thread_state_key, python_interp));
    return;
}
void PythonThroughputPluginCompleteCallback(void *user_data,
					    globus_ftp_client_handle_t *handle,
					    globus_bool_t success) {
    PyObject *func, *arglist, *result, *arg, *ptr, *tuple;
    callbackStruct3* cbStruct = (callbackStruct3*)user_data;
    PyEval_AcquireThread(get_python_thread_state(python_thread_state_key, python_interp));
    func = cbStruct->cb4;
    arg = cbStruct->arg;
    ptr = SWIG_NewPointerObj((void*)handle, 
			     SWIGTYPE_p_globus_ftp_client_handle_t, 0);
    if(!(tuple = PyTuple_New(2))) {
    	PyErr_SetString(PyExc_RuntimeError,"PythonThroughputPluginCompleteCallback, PyTuple_New");	
    }
    else{ 
	if (success) {  
	    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
            Py_INCREF(Py_None);
            PyTuple_SET_ITEM(tuple, 1, Py_None);  
        }
	else{
	    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
            PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", "Error Transfer interrupted"));
	}
    }
   
    arglist = Py_BuildValue("(OOO)", arg, ptr, tuple);
    result = PyEval_CallObject(func, arglist);
    if (result == NULL) {
      PyErr_Print();
    }
    Py_DECREF(ptr);
    Py_DECREF(arglist);
    Py_XDECREF(result);
    PyEval_ReleaseThread(get_python_thread_state(python_thread_state_key, python_interp));
    return;
}

/* C to Python dispatch functions for the Python plugin infrastructure */


static globus_ftp_client_plugin_t *
PythonPluginCopy(globus_ftp_client_plugin_t* self, 
		 void *plugin_specific) {
    globus_ftp_client_plugin_t * newguy;
    PyObject *ptemp;
    PyObject *func, *result, *arglist;
    pyglobus_plugin_t *pyglobus_plugin = (pyglobus_plugin_t *)plugin_specific;

/*     printf("PythonPluginCopy called\n");  */

    /* currently, we don't acquire the thread here because this
       function is always called via an interpreter context (?) */

/*     PyEval_AcquireThread(get_python_thread_state(python_thread_state_key, python_interp)); */
/*     printf("PythonPluginCopy end acquire thread\n");  */
    newguy = pyglobus_plugin_init(pyglobus_plugin->python_function_list);
    func = PyList_GetItem(pyglobus_plugin->python_function_list, COPY);  
    ptemp = SWIG_NewPointerObj((void*)self, SWIGTYPE_p_globus_ftp_client_plugin_t, 0);  
    arglist = Py_BuildValue("(OO)", ptemp, pyglobus_plugin->python_user_specific);  
    result = PyEval_CallObject(func, arglist);  
    if (result == NULL) {
      PyErr_Print();
    }
    Py_DECREF(ptemp);
    Py_DECREF(arglist);  
    Py_XDECREF(result);  
    
/*     PyEval_ReleaseThread(get_python_thread_state(python_thread_state_key, python_interp));  */
    return newguy;
}

static void
PythonPluginDestroy(globus_ftp_client_plugin_t* self, void *plugin_specific) {
    PyObject *ptemp;
    PyObject *func, *py_result, *arglist;
    pyglobus_plugin_t *pyglobus_plugin = (pyglobus_plugin_t *)plugin_specific;
    globus_result_t result;

/*     printf("PythonPluginDestroy called\n");  */

/*     PyEval_AcquireThread(get_python_thread_state(python_thread_state_key, python_interp)); */

    func = PyList_GetItem(pyglobus_plugin->python_function_list, DESTROY);
    ptemp = SWIG_NewPointerObj((void*)self, SWIGTYPE_p_globus_ftp_client_plugin_t, 0);
    arglist = Py_BuildValue("(OO)", ptemp, pyglobus_plugin->python_user_specific);
    py_result = PyEval_CallObject(func, arglist);
    if (py_result == NULL) {
      PyErr_Print();
    }
    Py_DECREF(ptemp);
    Py_DECREF(arglist);
    Py_XDECREF(py_result);
    
    Py_DECREF(pyglobus_plugin->python_function_list);
    Py_DECREF(Py_None);
    globus_free(pyglobus_plugin);

/*     PyEval_ReleaseThread(get_python_thread_state(python_thread_state_key, python_interp)); */
    globus_ftp_client_plugin_destroy(self);
}

static void
PythonPluginGet(globus_ftp_client_plugin_t* self, 
		void *plugin_specific,
                globus_ftp_client_handle_t* handle, const char* url,
                const globus_ftp_client_operationattr_t * attr, globus_bool_t restart) {
    PyObject *ptemp, *ptemp1, *ptemp2;
    PyObject *func, *result, *arglist;
    pyglobus_plugin_t *pyglobus_plugin = (pyglobus_plugin_t *)plugin_specific;
    PyEval_AcquireThread(get_python_thread_state(python_thread_state_key, python_interp));

/*     printf("PythonPluginGet called\n");  */

    func = PyList_GetItem(pyglobus_plugin->python_function_list, GET);
    ptemp = SWIG_NewPointerObj((void*)self, SWIGTYPE_p_globus_ftp_client_plugin_t, 0);
    ptemp1 = SWIG_NewPointerObj((void*)handle, SWIGTYPE_p_globus_ftp_client_handle_t, 0);
    ptemp2 = SWIG_NewPointerObj((void*)attr, SWIGTYPE_p_globus_ftp_client_operationattr_t, 0);
    arglist = Py_BuildValue("(OOsOiO)", ptemp, ptemp1, url, ptemp2, (int)restart, pyglobus_plugin->python_user_specific);
    result = PyEval_CallObject(func, arglist);
    if (result == NULL) {
      PyErr_Print();
    }
    Py_DECREF(ptemp);
    Py_DECREF(ptemp1);
    Py_DECREF(ptemp2);
    Py_DECREF(arglist);
    Py_XDECREF(result);
    PyEval_ReleaseThread(get_python_thread_state(python_thread_state_key, python_interp));
    return;
}

static void
PythonPluginPut(globus_ftp_client_plugin_t* self, 
		void *plugin_specific,
                globus_ftp_client_handle_t* handle, 
		const char* url,
                const globus_ftp_client_operationattr_t * attr, 
		globus_bool_t restart) {
    PyObject *ptemp, *ptemp1, *ptemp2;
    PyObject *func, *result, *arglist;
    pyglobus_plugin_t *pyglobus_plugin = (pyglobus_plugin_t *)plugin_specific;
    PyEval_AcquireThread(get_python_thread_state(python_thread_state_key, python_interp));

/*     printf("PythonPluginPut called\n");  */

    func = PyList_GetItem(pyglobus_plugin->python_function_list, PUT);
    ptemp = SWIG_NewPointerObj((void*)self, SWIGTYPE_p_globus_ftp_client_plugin_t, 0);
    ptemp1 = SWIG_NewPointerObj((void*)handle, SWIGTYPE_p_globus_ftp_client_handle_t, 0);
    ptemp2 = SWIG_NewPointerObj((void*)attr, SWIGTYPE_p_globus_ftp_client_operationattr_t, 0);
    arglist = Py_BuildValue("(OOsOiO)", ptemp, ptemp1, url, ptemp2, (int)restart, pyglobus_plugin->python_user_specific);
    result = PyEval_CallObject(func, arglist);
    if (result == NULL) {
      PyErr_Print();
    }
    Py_DECREF(ptemp);
    Py_DECREF(ptemp1);
    Py_DECREF(ptemp2);
    Py_DECREF(arglist);
    Py_XDECREF(result);
    PyEval_ReleaseThread(get_python_thread_state(python_thread_state_key, python_interp));
    return;

}

static void
PythonPluginThirdPartyTransfer(globus_ftp_client_plugin_t* self, 
		     void *plugin_specific,
                     globus_ftp_client_handle_t* handle, const char* srcUrl,
                     const globus_ftp_client_operationattr_t * srcAttr, 
                     const char* dstUrl, 
                     const globus_ftp_client_operationattr_t* dstAttr, 
                     globus_bool_t restart) {
    PyObject *ptemp, *ptemp1, *ptemp2, *ptemp3;
    PyObject *func, *result, *arglist;
    pyglobus_plugin_t *pyglobus_plugin = (pyglobus_plugin_t *)plugin_specific;
    PyEval_AcquireThread(get_python_thread_state(python_thread_state_key, python_interp));

/*     printf("PythonPluginThirdPartyTransfer called\n");  */

    func = PyList_GetItem(pyglobus_plugin->python_function_list, TRANSFER);
    ptemp = SWIG_NewPointerObj((void*)self, SWIGTYPE_p_globus_ftp_client_plugin_t, 0);
    ptemp1 = SWIG_NewPointerObj((void*)handle, SWIGTYPE_p_globus_ftp_client_handle_t, 0);
    ptemp2 = SWIG_NewPointerObj((void*)srcAttr, SWIGTYPE_p_globus_ftp_client_operationattr_t, 0);
    ptemp3 = SWIG_NewPointerObj((void*)dstAttr, SWIGTYPE_p_globus_ftp_client_operationattr_t, 0);
    arglist = Py_BuildValue("(OOsOsOiO)", ptemp, ptemp1, srcUrl, ptemp2, dstUrl, ptemp3, (int)restart, pyglobus_plugin->python_user_specific);
    result = PyEval_CallObject(func, arglist);
    if (result == NULL) {
      PyErr_Print();
    }
    Py_DECREF(ptemp);
    Py_DECREF(ptemp1);
    Py_DECREF(ptemp2);
    Py_DECREF(ptemp3);
    Py_DECREF(arglist);
    Py_XDECREF(result);
    PyEval_ReleaseThread(get_python_thread_state(python_thread_state_key, python_interp));
    return;
}

static void
PythonPluginAbort(globus_ftp_client_plugin_t* self, 
		    void *plugin_specific,
                  globus_ftp_client_handle_t* handle) {
    PyObject *ptemp, *ptemp1;
    PyObject *func, *result, *arglist;
    pyglobus_plugin_t *pyglobus_plugin = (pyglobus_plugin_t *)plugin_specific;
    PyEval_AcquireThread(get_python_thread_state(python_thread_state_key, python_interp));

/*     printf("PythonPluginThirdPartyTransfer called\n");  */

    func = PyList_GetItem(pyglobus_plugin->python_function_list, TRANSFER);
    ptemp = SWIG_NewPointerObj((void*)self, SWIGTYPE_p_globus_ftp_client_plugin_t, 0);
    ptemp1 = SWIG_NewPointerObj((void*)handle, SWIGTYPE_p_globus_ftp_client_handle_t, 0);
    arglist = Py_BuildValue("(OOO)", ptemp, ptemp1, pyglobus_plugin->python_user_specific);
    result = PyEval_CallObject(func, arglist);
    if (result == NULL) {
      PyErr_Print();
    }
    Py_DECREF(ptemp);
    Py_DECREF(ptemp1);
    Py_DECREF(arglist);
    Py_XDECREF(result);
    PyEval_ReleaseThread(get_python_thread_state(python_thread_state_key, python_interp));
    return;
}

static void
PythonPluginConnect(globus_ftp_client_plugin_t* self, 
		    void *plugin_specific,
                    globus_ftp_client_handle_t* handle,
                    const char* url) {
    PyObject *ptemp, *ptemp1;
    PyObject *func, *result, *arglist;
    pyglobus_plugin_t *pyglobus_plugin = (pyglobus_plugin_t *)plugin_specific;
    PyEval_AcquireThread(get_python_thread_state(python_thread_state_key, python_interp));

/*     printf("PythonPluginConnect called\n");  */

    func = PyList_GetItem(pyglobus_plugin->python_function_list, CONNECT);
    ptemp = SWIG_NewPointerObj((void*)self, SWIGTYPE_p_globus_ftp_client_plugin_t, 0);
    ptemp1 = SWIG_NewPointerObj((void*)handle, SWIGTYPE_p_globus_ftp_client_handle_t, 0);
    arglist = Py_BuildValue("(OOsO)", ptemp, ptemp1, url, pyglobus_plugin->python_user_specific);
    result = PyEval_CallObject(func, arglist);
    if (result == NULL) {
      PyErr_Print();
    }
    Py_DECREF(ptemp);
    Py_DECREF(ptemp1);
    Py_DECREF(arglist);
    Py_XDECREF(result);
    PyEval_ReleaseThread(get_python_thread_state(python_thread_state_key, python_interp));
    return;
}

static void
PythonPluginDelete(globus_ftp_client_plugin_t* self, 
		    void *plugin_specific,
                 globus_ftp_client_handle_t* handle,
                 const char* url,
                     const globus_ftp_client_operationattr_t* attr, 
                     globus_bool_t restart) {
    PyObject *ptemp, *ptemp1, *ptemp2;
    PyObject *func, *result, *arglist;
    pyglobus_plugin_t *pyglobus_plugin = (pyglobus_plugin_t *)plugin_specific;
    PyEval_AcquireThread(get_python_thread_state(python_thread_state_key, python_interp));

/*     printf("PythonPluginDelete called\n");  */

    func = PyList_GetItem(pyglobus_plugin->python_function_list, PLUGIN_DELETE);
    ptemp = SWIG_NewPointerObj((void*)self, SWIGTYPE_p_globus_ftp_client_plugin_t, 0);
    ptemp1 = SWIG_NewPointerObj((void*)handle, SWIGTYPE_p_globus_ftp_client_handle_t, 0);
    ptemp2 = SWIG_NewPointerObj((void*)attr, SWIGTYPE_p_globus_ftp_client_operationattr_t, 0);
    arglist = Py_BuildValue("(OOsOiO)", ptemp, ptemp1, url, ptemp2, (int)restart, pyglobus_plugin->python_user_specific);
    result = PyEval_CallObject(func, arglist);
    if (result == NULL) {
      PyErr_Print();
    }
    Py_DECREF(ptemp);
    Py_DECREF(ptemp1);
    Py_DECREF(ptemp2);
    Py_DECREF(arglist);
    Py_XDECREF(result);
    PyEval_ReleaseThread(get_python_thread_state(python_thread_state_key, python_interp));
    return;
}

static void
PythonPluginAuthenticate(globus_ftp_client_plugin_t* self, 
		    void *plugin_specific,
                 globus_ftp_client_handle_t* handle,
                 const char* url,
                 const globus_ftp_control_auth_info_t* authInfo) {
    PyObject *ptemp, *ptemp1, *ptemp2;
    PyObject *func, *result, *arglist;
    pyglobus_plugin_t *pyglobus_plugin = (pyglobus_plugin_t *)plugin_specific;
    PyEval_AcquireThread(get_python_thread_state(python_thread_state_key, python_interp));

/*     printf("PythonPluginAuthenticate called\n");  */

    func = PyList_GetItem(pyglobus_plugin->python_function_list, AUTH); 
    ptemp = SWIG_NewPointerObj((void*)self, SWIGTYPE_p_globus_ftp_client_plugin_t, 0); 
    ptemp1 = SWIG_NewPointerObj((void*)handle, SWIGTYPE_p_globus_ftp_client_handle_t, 0); 
    ptemp2 = SWIG_NewPointerObj((void*)authInfo, SWIGTYPE_p_globus_ftp_control_auth_info_t, 0); 
    arglist = Py_BuildValue("(OOsOO)", ptemp, ptemp1, url, ptemp2, pyglobus_plugin->python_user_specific); 
    result = PyEval_CallObject(func, arglist); 
    if (result == NULL) { 
      PyErr_Print(); 
    } 
    Py_DECREF(ptemp);
    Py_DECREF(ptemp1);
    Py_DECREF(ptemp2);
    Py_DECREF(arglist); 
    Py_XDECREF(result);  
    PyEval_ReleaseThread(get_python_thread_state(python_thread_state_key, python_interp));
    return;
}

static void
PythonPluginRead(globus_ftp_client_plugin_t* self, 
		    void *plugin_specific,
                 globus_ftp_client_handle_t* handle,
                 const globus_byte_t* buffer,
                 globus_size_t bufferLen) {
    PyObject *ptemp, *ptemp1, *ptemp2;
    PyObject *func, *result, *arglist;
    pyglobus_plugin_t *pyglobus_plugin = (pyglobus_plugin_t *)plugin_specific;
    PyEval_AcquireThread(get_python_thread_state(python_thread_state_key, python_interp));

/*     printf("PythonPluginRead called\n");  */

    func = PyList_GetItem(pyglobus_plugin->python_function_list, READ);
    ptemp = SWIG_NewPointerObj((void*)self, SWIGTYPE_p_globus_ftp_client_plugin_t, 0);
    ptemp1 = SWIG_NewPointerObj((void*)handle, SWIGTYPE_p_globus_ftp_client_handle_t, 0);
    ptemp2 = SWIG_NewPointerObj((void*)buffer, SWIGTYPE_p_globus_byte_t, 0);
    arglist = Py_BuildValue("(OOOiO)", ptemp, ptemp1, ptemp2, (int)bufferLen, pyglobus_plugin->python_user_specific);
    result = PyEval_CallObject(func, arglist);
    if (result == NULL) {
      PyErr_Print();
    }
    Py_DECREF(ptemp);
    Py_DECREF(ptemp1);
    Py_DECREF(ptemp2);
    Py_DECREF(arglist);
    Py_XDECREF(result);
    PyEval_ReleaseThread(get_python_thread_state(python_thread_state_key, python_interp));
    return;
}

static void
PythonPluginMove(globus_ftp_client_plugin_t* self, 
		    void *plugin_specific,
                 globus_ftp_client_handle_t* handle,
                 const char* source_url,
                 const char* dest_url,
                     const globus_ftp_client_operationattr_t* attr, 
		 globus_bool_t result) {		 
    PyObject *ptemp, *ptemp1, *ptemp2;
    PyObject *func, *py_result, *arglist;
    pyglobus_plugin_t *pyglobus_plugin = (pyglobus_plugin_t *)plugin_specific;
    PyEval_AcquireThread(get_python_thread_state(python_thread_state_key, python_interp));

/*     printf("PythonPluginMove called\n");  */

    func = PyList_GetItem(pyglobus_plugin->python_function_list, MOVE);
    ptemp = SWIG_NewPointerObj((void*)self, SWIGTYPE_p_globus_ftp_client_plugin_t, 0);
    ptemp1 = SWIG_NewPointerObj((void*)handle, SWIGTYPE_p_globus_ftp_client_handle_t, 0);
    ptemp2 = SWIG_NewPointerObj((void*)attr, SWIGTYPE_p_globus_byte_t, 0);
    arglist = Py_BuildValue("(OOssOiO)", ptemp, ptemp1, source_url, dest_url, ptemp2, (int) result, pyglobus_plugin->python_user_specific);
    py_result = PyEval_CallObject(func, arglist);
    if (py_result == NULL) {
      PyErr_Print();
    }
    Py_DECREF(ptemp);
    Py_DECREF(ptemp1);
    Py_DECREF(ptemp2);
    Py_DECREF(arglist);
    Py_XDECREF(py_result);
    PyEval_ReleaseThread(get_python_thread_state(python_thread_state_key, python_interp));
    return;
}

static void
PythonPluginList(globus_ftp_client_plugin_t* self, 
		 void *plugin_specific,
                 globus_ftp_client_handle_t* handle,
                 const char* url,
		 const globus_ftp_client_operationattr_t* attr, 
		 globus_bool_t result) {		 
    PyObject *ptemp, *ptemp1, *ptemp2;
    PyObject *func, *py_result, *arglist;
    pyglobus_plugin_t *pyglobus_plugin = (pyglobus_plugin_t *)plugin_specific;
    PyEval_AcquireThread(get_python_thread_state(python_thread_state_key, python_interp));

/*     printf("PythonPluginList called\n");  */

    func = PyList_GetItem(pyglobus_plugin->python_function_list, LIST);
    ptemp = SWIG_NewPointerObj((void*)self, SWIGTYPE_p_globus_ftp_client_plugin_t, 0);
    ptemp1 = SWIG_NewPointerObj((void*)handle, SWIGTYPE_p_globus_ftp_client_handle_t, 0);
    ptemp2 = SWIG_NewPointerObj((void*)attr, SWIGTYPE_p_globus_byte_t, 0);
    arglist = Py_BuildValue("(OOsOiO)", ptemp, ptemp1, url, ptemp2, (int) result, pyglobus_plugin->python_user_specific);
    py_result = PyEval_CallObject(func, arglist);
    if (py_result == NULL) {
      PyErr_Print();
    }
    Py_DECREF(ptemp);
    Py_DECREF(ptemp1);
    Py_DECREF(ptemp2);
    Py_DECREF(arglist);
    Py_XDECREF(py_result);
    PyEval_ReleaseThread(get_python_thread_state(python_thread_state_key, python_interp));
    return;
}

static void
PythonPluginSize(globus_ftp_client_plugin_t* self, 
		    void *plugin_specific,
                 globus_ftp_client_handle_t* handle,
                 const char* url,
                     const globus_ftp_client_operationattr_t* attr, 
		 globus_bool_t result) {		 
    PyObject *ptemp, *ptemp1, *ptemp2;
    PyObject *func, *py_result, *arglist;
    pyglobus_plugin_t *pyglobus_plugin = (pyglobus_plugin_t *)plugin_specific;
    PyEval_AcquireThread(get_python_thread_state(python_thread_state_key, python_interp));

/*     printf("PythonPluginSize called\n");  */

    func = PyList_GetItem(pyglobus_plugin->python_function_list, PLUGIN_SIZE);
    ptemp = SWIG_NewPointerObj((void*)self, SWIGTYPE_p_globus_ftp_client_plugin_t, 0);
    ptemp1 = SWIG_NewPointerObj((void*)handle, SWIGTYPE_p_globus_ftp_client_handle_t, 0);
    ptemp2 = SWIG_NewPointerObj((void*)attr, SWIGTYPE_p_globus_byte_t, 0);
    arglist = Py_BuildValue("(OOssOiO)", ptemp, ptemp1, url, ptemp2, (int) result, pyglobus_plugin->python_user_specific);
    py_result = PyEval_CallObject(func, arglist);
    if (py_result == NULL) {
      PyErr_Print();
    }
    Py_DECREF(ptemp);
    Py_DECREF(ptemp1);
    Py_DECREF(ptemp2);
    Py_DECREF(arglist);
    Py_XDECREF(py_result);
    PyEval_ReleaseThread(get_python_thread_state(python_thread_state_key, python_interp));
    return;
}

static void
PythonPluginVerboseList(globus_ftp_client_plugin_t* self, 
		    void *plugin_specific,
                 globus_ftp_client_handle_t* handle,
                 const char* url,
                     const globus_ftp_client_operationattr_t* attr, 
		 globus_bool_t result) {		 
    PyObject *ptemp, *ptemp1, *ptemp2;
    PyObject *func, *py_result, *arglist;
    pyglobus_plugin_t *pyglobus_plugin = (pyglobus_plugin_t *)plugin_specific;
    PyEval_AcquireThread(get_python_thread_state(python_thread_state_key, python_interp));

/*     printf("PythonPluginVerboseList called\n");  */

    func = PyList_GetItem(pyglobus_plugin->python_function_list, VERBOSELIST);
    ptemp = SWIG_NewPointerObj((void*)self, SWIGTYPE_p_globus_ftp_client_plugin_t, 0);
    ptemp1 = SWIG_NewPointerObj((void*)handle, SWIGTYPE_p_globus_ftp_client_handle_t, 0);
    ptemp2 = SWIG_NewPointerObj((void*)attr, SWIGTYPE_p_globus_byte_t, 0);
    arglist = Py_BuildValue("(OOsOiO)", ptemp, ptemp1, url, ptemp2, (int) result, pyglobus_plugin->python_user_specific);
    py_result = PyEval_CallObject(func, arglist);
    if (py_result == NULL) {
      PyErr_Print();
    }
    Py_DECREF(ptemp);
    Py_DECREF(ptemp1);
    Py_DECREF(ptemp2);       
    Py_DECREF(arglist);
    Py_XDECREF(py_result);
    PyEval_ReleaseThread(get_python_thread_state(python_thread_state_key, python_interp));
    return;
}

static void
PythonPluginRmdir(globus_ftp_client_plugin_t* self, 
		    void *plugin_specific,
                 globus_ftp_client_handle_t* handle,
                 const char* url,
                     const globus_ftp_client_operationattr_t* attr, 
		 globus_bool_t result) {		 
    PyObject *ptemp, *ptemp1, *ptemp2;
    PyObject *func, *py_result, *arglist;
    pyglobus_plugin_t *pyglobus_plugin = (pyglobus_plugin_t *)plugin_specific;
    PyEval_AcquireThread(get_python_thread_state(python_thread_state_key, python_interp));

/*     printf("PythonPluginRmdir called\n");  */

    func = PyList_GetItem(pyglobus_plugin->python_function_list, RMDIR);
    ptemp = SWIG_NewPointerObj((void*)self, SWIGTYPE_p_globus_ftp_client_plugin_t, 0);
    ptemp1 = SWIG_NewPointerObj((void*)handle, SWIGTYPE_p_globus_ftp_client_handle_t, 0);
    ptemp2 = SWIG_NewPointerObj((void*)attr, SWIGTYPE_p_globus_byte_t, 0);
    arglist = Py_BuildValue("(OOsOiO)", ptemp, ptemp1, url, ptemp2, (int) result, pyglobus_plugin->python_user_specific);
    py_result = PyEval_CallObject(func, arglist);
    if (py_result == NULL) {
      PyErr_Print();
    }
    Py_DECREF(ptemp);
    Py_DECREF(ptemp1);
    Py_DECREF(ptemp2);
    Py_DECREF(arglist);
    Py_XDECREF(py_result);
    PyEval_ReleaseThread(get_python_thread_state(python_thread_state_key, python_interp));
    return;
}

static void
PythonPluginMkdir(globus_ftp_client_plugin_t* self, 
		    void *plugin_specific,
                 globus_ftp_client_handle_t* handle,
                 const char* url,
                     const globus_ftp_client_operationattr_t* attr, 
		 globus_bool_t result) {		 
    PyObject *ptemp, *ptemp1, *ptemp2;
    PyObject *func, *py_result, *arglist;
    pyglobus_plugin_t *pyglobus_plugin = (pyglobus_plugin_t *)plugin_specific;
    PyEval_AcquireThread(get_python_thread_state(python_thread_state_key, python_interp));

/*     printf("PythonPluginMkdir called\n");  */

    func = PyList_GetItem(pyglobus_plugin->python_function_list, MKDIR);
    ptemp = SWIG_NewPointerObj((void*)self, SWIGTYPE_p_globus_ftp_client_plugin_t, 0);
    ptemp1 = SWIG_NewPointerObj((void*)handle, SWIGTYPE_p_globus_ftp_client_handle_t, 0);
    ptemp2 = SWIG_NewPointerObj((void*)attr, SWIGTYPE_p_globus_byte_t, 0);
    arglist = Py_BuildValue("(OOsOiO)", ptemp, ptemp1, url, ptemp2, (int) result, pyglobus_plugin->python_user_specific);
    py_result = PyEval_CallObject(func, arglist);
    if (py_result == NULL) {
      PyErr_Print();
    }
    Py_DECREF(ptemp);
    Py_DECREF(ptemp1);
    Py_DECREF(ptemp2);       
    Py_DECREF(arglist);
    Py_XDECREF(py_result);
    PyEval_ReleaseThread(get_python_thread_state(python_thread_state_key, python_interp));
    return;
}

static void
PythonPluginModificationTime(globus_ftp_client_plugin_t* self, 
		    void *plugin_specific,
                 globus_ftp_client_handle_t* handle,
                 const char* url,
                     const globus_ftp_client_operationattr_t* attr, 
		 globus_bool_t result) {		 
    PyObject *ptemp, *ptemp1, *ptemp2;
    PyObject *func, *py_result, *arglist;
    pyglobus_plugin_t *pyglobus_plugin = (pyglobus_plugin_t *)plugin_specific;
    PyEval_AcquireThread(get_python_thread_state(python_thread_state_key, python_interp));

/*     printf("PythonPluginModificationTime called\n");  */

    func = PyList_GetItem(pyglobus_plugin->python_function_list, MODIFICATIONTIME);
    ptemp = SWIG_NewPointerObj((void*)self, SWIGTYPE_p_globus_ftp_client_plugin_t, 0);
    ptemp1 = SWIG_NewPointerObj((void*)handle, SWIGTYPE_p_globus_ftp_client_handle_t, 0);
    ptemp2 = SWIG_NewPointerObj((void*)attr, SWIGTYPE_p_globus_byte_t, 0);
    arglist = Py_BuildValue("(OOssOiO)", ptemp, ptemp1, url, ptemp2, (int) result, pyglobus_plugin->python_user_specific);
    py_result = PyEval_CallObject(func, arglist);
    if (py_result == NULL) {
      PyErr_Print();
    }
    Py_DECREF(ptemp);
    Py_DECREF(ptemp1);
    Py_DECREF(ptemp2); 
    Py_DECREF(arglist);
    Py_XDECREF(py_result);
    PyEval_ReleaseThread(get_python_thread_state(python_thread_state_key, python_interp));
    return;
}

static void
PythonPluginWrite(globus_ftp_client_plugin_t* self, 
		    void *plugin_specific,
                  globus_ftp_client_handle_t* handle,
                  const globus_byte_t* buffer,
                  globus_size_t bufferLen,
                  globus_off_t offset, globus_bool_t eof) {
    PyObject *ptemp, *ptemp1, *ptemp2;
    PyObject *func, *py_result, *arglist;
    pyglobus_plugin_t *pyglobus_plugin = (pyglobus_plugin_t *)plugin_specific;

/* this is causing hanging- if we acquire() the first time the write
   is called, then it hangs because we already have the GIL, if we
   don't all it then the second time we are called it crashed because
   we don't have the GIL */

/*     PyEval_AcquireThread(get_python_thread_state(python_thread_state_key, python_interp)); */

/*     printf("PythonPluginWrite called\n");  */
    return;

#if defined NOTHING
    func = PyList_GetItem(pyglobus_plugin->python_function_list, WRITE);
    ptemp = SWIG_NewPointerObj((void*)self, SWIGTYPE_p_globus_ftp_client_plugin_t, 0);
    ptemp1 = SWIG_NewPointerObj((void*)handle, SWIGTYPE_p_globus_ftp_client_handle_t, 0);
    ptemp2 = SWIG_NewPointerObj((void*)buffer, SWIGTYPE_p_globus_byte_t, 0);
    arglist = Py_BuildValue("(OOOiiiO)", ptemp, ptemp1, ptemp2, (int)bufferLen, (int)offset, (int)eof, pyglobus_plugin->python_user_specific);
    py_result = PyEval_CallObject(func, arglist);
    if (py_result == NULL) {
	PyErr_Print();
    }
    Py_DECREF(ptemp);
    Py_DECREF(ptemp1);
    Py_DECREF(ptemp2); 
    Py_DECREF(arglist);
    Py_XDECREF(py_result);
    PyEval_ReleaseThread(get_python_thread_state(python_thread_state_key, python_interp)); 
    return;
#endif
}

static void
PythonPluginData(globus_ftp_client_plugin_t* self, 
		    void *plugin_specific,
                 globus_ftp_client_handle_t* handle,
                 globus_object_t* error, 
                 const globus_byte_t* buffer,
                 globus_size_t bufferLen,
                 globus_off_t offset, globus_bool_t eof) {
    PyObject *ptemp, *ptemp1, *ptemp2;
    PyObject *func, *result, *arglist;
    pyglobus_plugin_t *pyglobus_plugin = (pyglobus_plugin_t *)plugin_specific;
    PyEval_AcquireThread(get_python_thread_state(python_thread_state_key, python_interp));

/*     printf("PythonPluginData called\n");  */

    func = PyList_GetItem(pyglobus_plugin->python_function_list, DATA);
    ptemp = SWIG_NewPointerObj((void*)self, SWIGTYPE_p_globus_ftp_client_plugin_t, 0);
    ptemp1 = SWIG_NewPointerObj((void*)handle, SWIGTYPE_p_globus_ftp_client_handle_t, 0);
    ptemp2 = SWIG_NewPointerObj((void*)buffer, SWIGTYPE_p_globus_byte_t, 0);
    arglist = Py_BuildValue("(OOOiiiO)", ptemp, ptemp1, ptemp2, (int)bufferLen, (int)offset, (int)eof, pyglobus_plugin->python_user_specific);
    result = PyEval_CallObject(func, arglist);
    if (result == NULL) {
      PyErr_Print();
    }
    Py_DECREF(ptemp);
    Py_DECREF(ptemp1);
    Py_DECREF(ptemp2); 
    Py_DECREF(arglist);
    Py_XDECREF(result);
    PyEval_ReleaseThread(get_python_thread_state(python_thread_state_key, python_interp));
    return;
}

static void
PythonPluginCommand(globus_ftp_client_plugin_t* self, 
		    void *plugin_specific,
                    globus_ftp_client_handle_t* handle,
                    const char* url, const char* command) {
    PyObject *ptemp, *ptemp1;
    PyObject *func, *result, *arglist;
    pyglobus_plugin_t *pyglobus_plugin = (pyglobus_plugin_t *)plugin_specific;
    PyEval_AcquireThread(get_python_thread_state(python_thread_state_key, python_interp));

/*     printf("PythonPluginCommand called\n");  */

    func = PyList_GetItem(pyglobus_plugin->python_function_list, COMMAND);
    ptemp = SWIG_NewPointerObj((void*)self, SWIGTYPE_p_globus_ftp_client_plugin_t, 0);
    ptemp1 = SWIG_NewPointerObj((void*)handle, SWIGTYPE_p_globus_ftp_client_handle_t, 0);
    arglist = Py_BuildValue("(OOssO)", ptemp, ptemp1, url, command, pyglobus_plugin->python_user_specific);
    result = PyEval_CallObject(func, arglist);
    if (result == NULL) {
      PyErr_Print();
    }
    Py_DECREF(ptemp);
    Py_DECREF(ptemp1); 
    Py_DECREF(arglist);
    Py_XDECREF(result);
    PyEval_ReleaseThread(get_python_thread_state(python_thread_state_key, python_interp));
    return;
}

static void
PythonPluginResponse(globus_ftp_client_plugin_t* self, 
		    void *plugin_specific,
                     globus_ftp_client_handle_t* handle,
                     const char* url, globus_object_t* error,
                     const globus_ftp_control_response_t* response) {
    PyObject *ptr, *ptr2, *ptr3;
    PyObject *func, *result, *arglist, *errorObject;
    pyglobus_plugin_t *pyglobus_plugin = (pyglobus_plugin_t *)plugin_specific;
    PyEval_AcquireThread(get_python_thread_state(python_thread_state_key, python_interp));

/*     printf("PythonPluginResponse called\n");  */

    func = PyList_GetItem(pyglobus_plugin->python_function_list, RESPONSE);
    ptr = SWIG_NewPointerObj((void*)self, SWIGTYPE_p_globus_ftp_client_plugin_t, 0);
    ptr2 = SWIG_NewPointerObj((void*)handle, SWIGTYPE_p_globus_ftp_client_handle_t, 0);

    ptr3 = SWIG_NewPointerObj((void*)response, SWIGTYPE_p_globus_ftp_control_response_t, 0);
    
    errorObject = check_error(error);
    arglist = Py_BuildValue("(OOsOOO)", ptr, ptr2, url, errorObject, ptr3, pyglobus_plugin->python_user_specific);
    result = PyEval_CallObject(func, arglist);
    if (result == NULL) {
      PyErr_Print();
    }
    Py_DECREF(ptr);
    Py_DECREF(ptr2);
    Py_DECREF(ptr3); 
    Py_DECREF(arglist);
    Py_XDECREF(result);
    Py_XDECREF(errorObject);
    PyEval_ReleaseThread(get_python_thread_state(python_thread_state_key, python_interp));
    return;
}

static void
PythonPluginFault(globus_ftp_client_plugin_t* self, 
		    void *plugin_specific,
                  globus_ftp_client_handle_t* handle,
                  const char* url, globus_object_t* error) {
    PyObject *ptr, *ptr2, *errorObject;
    PyObject *func, *result, *arglist;
    pyglobus_plugin_t *pyglobus_plugin = (pyglobus_plugin_t *)plugin_specific;
    PyEval_AcquireThread(get_python_thread_state(python_thread_state_key, python_interp));

/*     printf("PythonPluginFault called\n");  */

    func = PyList_GetItem(pyglobus_plugin->python_function_list, FAULT);
    ptr = SWIG_NewPointerObj((void*)self, SWIGTYPE_p_globus_ftp_client_plugin_t, 0);
    ptr2 = SWIG_NewPointerObj((void*)handle, SWIGTYPE_p_globus_ftp_client_handle_t, 0);
   
    errorObject = check_error(error);
    arglist = Py_BuildValue("(OOsOO)", ptr, ptr2, url, errorObject, pyglobus_plugin->python_user_specific);
    result = PyEval_CallObject(func, arglist);
    if (result == NULL) {
      PyErr_Print();
    }
    Py_DECREF(ptr);
    Py_DECREF(ptr2);
    Py_DECREF(arglist);
    Py_XDECREF(result);
    Py_XDECREF(errorObject);
    PyEval_ReleaseThread(get_python_thread_state(python_thread_state_key, python_interp));
    return;
}

static void
PythonPluginComplete(globus_ftp_client_plugin_t* self, 
		    void *plugin_specific,
		     globus_ftp_client_handle_t* handle) {
    PyObject *ptemp, *ptemp1;
    PyObject *func, *result, *arglist;
    pyglobus_plugin_t *pyglobus_plugin = (pyglobus_plugin_t *)plugin_specific;
    PyEval_AcquireThread(get_python_thread_state(python_thread_state_key, python_interp));

/*     printf("PythonPluginComplete called\n");  */

    func = PyList_GetItem(pyglobus_plugin->python_function_list, COMPLETE);
    ptemp = SWIG_NewPointerObj((void*)self, SWIGTYPE_p_globus_ftp_client_plugin_t, 0);
    ptemp1 = SWIG_NewPointerObj((void*)handle, SWIGTYPE_p_globus_ftp_client_handle_t, 0);
    arglist = Py_BuildValue("(OOO)", ptemp, ptemp1, pyglobus_plugin->python_user_specific);
    result = PyEval_CallObject(func, arglist);
    if (result == NULL) {
      PyErr_Print();
    }
    Py_DECREF(ptemp);
    Py_DECREF(ptemp1);
    Py_DECREF(arglist);
    Py_XDECREF(result);
    PyEval_ReleaseThread(get_python_thread_state(python_thread_state_key, python_interp));
    return;
}
%}

%inline %{
globus_module_descriptor_t*
get_module(void) {
    return GLOBUS_FTP_CLIENT_MODULE;
}

/* Unlike the globus-defined modules we have to define this data here */

/* I have no idea what this data structure means */
static
globus_version_t local_version = 
{
    1,
    8,
    1042214906,
    28
};

static
int
globus_ftp_client_pyglobus_plugin_activate(void)
{
    return globus_module_activate(GLOBUS_FTP_CLIENT_MODULE);
}

static
int
globus_ftp_client_pyglobus_plugin_deactivate(void)
{
    return globus_module_deactivate(GLOBUS_FTP_CLIENT_MODULE);
}

static
globus_module_descriptor_t globus_ftp_client_pyglobus_plugin_module =
{
    "globus_ftp_client_pyglobus_plugin",
    globus_ftp_client_pyglobus_plugin_activate,
    globus_ftp_client_pyglobus_plugin_deactivate,
    GLOBUS_NULL,
    GLOBUS_NULL,
    &local_version
};
 
globus_module_descriptor_t*
get_pyglobus_plugin_module(void) {
    return &globus_ftp_client_pyglobus_plugin_module;
}

globus_module_descriptor_t*
get_debug_plugin_module(void) {
    return GLOBUS_FTP_CLIENT_DEBUG_PLUGIN_MODULE;
}

globus_module_descriptor_t*
get_restart_marker_plugin_module(void) {
    return GLOBUS_FTP_CLIENT_RESTART_MARKER_PLUGIN_MODULE;
}
globus_module_descriptor_t*
get_performance_marker_plugin_module(void) {
    return GLOBUS_FTP_CLIENT_PERF_PLUGIN_MODULE;
}
globus_module_descriptor_t*
get_throughput_plugin_module(void) {
    return GLOBUS_FTP_CLIENT_THROUGHPUT_PLUGIN_MODULE;
}
/*
globus_module_descriptor_t*
get_netlogger_plugin_module(void) {
    return GLOBUS_FTP_CLIENT_RESTART_MARKER_PLUGIN_MODULE;
}
*/
PyObject*
handleattr_init() {
    globus_result_t ret;
    PyObject *tuple, *ptr;
    char* tmpstr;
    globus_ftp_client_handleattr_t* attr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "handleattr_init: PyTuple_New()");
        return NULL;
    }
    attr = (globus_ftp_client_handleattr_t*)globus_malloc(sizeof(globus_ftp_client_handleattr_t));
    ret = globus_ftp_client_handleattr_init(attr);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
        globus_libc_free(attr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        ptr = SWIG_NewPointerObj((void*)attr, 
				 SWIGTYPE_p_globus_ftp_client_handleattr_t, 0);
        PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

PyObject*
handleattr_copy(globus_ftp_client_handleattr_t* srcAttr) {
    globus_result_t ret;
    PyObject *tuple, *ptr;
    char* tmpstr;
    globus_ftp_client_handleattr_t* attr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "handleattr_copy: PyTuple_New()");
        return NULL;
    }
    attr = (globus_ftp_client_handleattr_t*)globus_malloc(sizeof(globus_ftp_client_handleattr_t));
    ret = globus_ftp_client_handleattr_copy(attr, srcAttr);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
        globus_libc_free(attr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        ptr = SWIG_NewPointerObj((void*)attr, 
				 SWIGTYPE_p_globus_ftp_client_handleattr_t, 0);
        PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

PyObject*
handleattr_destroy(globus_ftp_client_handleattr_t* attr) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "handleattr_destroy: PyTuple_New()");
        return NULL;
    }
    ret = globus_ftp_client_handleattr_destroy(attr);
    globus_free(attr);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject*
handleattr_add_cached_url(globus_ftp_client_handleattr_t* attr,
                          const char* url) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "handleattr_add_cached_url: PyTuple_New()");
        return NULL;
    }
    ret = globus_ftp_client_handleattr_add_cached_url(attr, url);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject*
handleattr_remove_cached_url(globus_ftp_client_handleattr_t* attr,
                          const char* url) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "handleattr_remove_cached_url: PyTuple_New()");
        return NULL;
    }
    ret = globus_ftp_client_handleattr_remove_cached_url(attr, url);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject*
handleattr_set_cache_all(globus_ftp_client_handleattr_t* attr,
                         globus_bool_t cacheAll) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "handleattr_set_cache_all: PyTuple_New()");
        return NULL;
    }
    ret = globus_ftp_client_handleattr_set_cache_all(attr, cacheAll);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject*
handleattr_get_cache_all(globus_ftp_client_handleattr_t* attr,
                         globus_bool_t* boolRet) {
    globus_result_t ret;
    PyObject* list;
    char* tmpstr;
    list = PyList_New(0);
    ret = globus_ftp_client_handleattr_get_cache_all(attr, boolRet);
    if (ret != GLOBUS_SUCCESS) {
        PyList_Append(list, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyList_Append(list, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyList_Append(list, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyList_Append(list, Py_None);
    }
    return list;
}
/*
PyObject*
handleattr_set_netlogger(globus_ftp_client_handleattr_t* attr,
                         globus_netlogger_handle_t* nl_handle) {
    globus_result_t ret;
    PyObject *tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "handleattr_set_netlogger: PyTuple_New()");
        return NULL;
    }
    ret = globus_ftp_client_handleattr_set_netlogger(attr, nl_handle);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
}
*/
PyObject*
handleattr_add_plugin(globus_ftp_client_handleattr_t* handleattr,
                  globus_ftp_client_plugin_t* plugin) {
    globus_result_t ret;
    PyObject *tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "handleattr_add_plugin: PyTuple_New()");
        return NULL;
    }
    ret = globus_ftp_client_handleattr_add_plugin(handleattr, plugin);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject*
handleattr_remove_plugin(globus_ftp_client_handleattr_t* handleattr,
                     globus_ftp_client_plugin_t* plugin) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "handleattr_remove_plugin: PyTuple_New()");
        return NULL;
    }
    ret = globus_ftp_client_handleattr_remove_plugin(handleattr, plugin);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    globus_free(handleattr);
    return tuple;
}

PyObject*
operationattr_init() {
    globus_result_t ret;
    PyObject *tuple, *ptr;
    char* tmpstr;
    globus_ftp_client_operationattr_t* attr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "operationattr_init: PyTuple_New()");
        return NULL;
    }
    attr = (globus_ftp_client_operationattr_t*)globus_malloc(sizeof(globus_ftp_client_operationattr_t));
    ret = globus_ftp_client_operationattr_init(attr);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
        globus_libc_free(attr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        ptr = SWIG_NewPointerObj((void*)attr, 
				 SWIGTYPE_p_globus_ftp_client_operationattr_t, 0);
        PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

PyObject*
operationattr_destroy(globus_ftp_client_operationattr_t* attr) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "operationattr_destroy: PyTuple_New()");
        return NULL;
    }
    ret = globus_ftp_client_operationattr_destroy(attr);
    globus_free(attr);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject*
operationattr_copy(const globus_ftp_client_operationattr_t* srcAttr) {
    globus_result_t ret;
    PyObject *tuple, *ptr;
    char* tmpstr;
    globus_ftp_client_operationattr_t* attr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "operationattr_copy: PyTuple_New()");
        return NULL;
    }
    attr = (globus_ftp_client_operationattr_t*)globus_malloc(sizeof(globus_ftp_client_operationattr_t));
    ret = globus_ftp_client_operationattr_copy(attr, srcAttr);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
        globus_libc_free(attr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        ptr = SWIG_NewPointerObj((void*)attr, 
				 SWIGTYPE_p_globus_ftp_client_operationattr_t, 0);
        PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

PyObject*
operationattr_set_append(globus_ftp_client_operationattr_t* attr, globus_bool_t append) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "operationattr_set_append: PyTuple_New()");
        return NULL;
    }
    ret = globus_ftp_client_operationattr_set_append(attr, append);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject*
operationattr_get_append(globus_ftp_client_operationattr_t* attr, globus_bool_t* boolRet) {
    globus_result_t ret;
    PyObject* list;
    char* tmpstr;
    list = PyList_New(0);
    ret = globus_ftp_client_operationattr_get_append(attr, boolRet);
    if (ret != GLOBUS_SUCCESS) {
        PyList_Append(list, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyList_Append(list, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyList_Append(list, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyList_Append(list, Py_None);
    }
    return list;
}

PyObject*
operationattr_set_resume_third_party_transfer(globus_ftp_client_operationattr_t* attr, 
                                              globus_bool_t resume) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                       "operationattr_set_resume_third_party_transfer: PyTuple_New()");
        return NULL;
    }
    ret = globus_ftp_client_operationattr_set_resume_third_party_transfer(attr, resume);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject*
operationattr_get_resume_third_party_transfer(globus_ftp_client_operationattr_t* attr, 
                                                          globus_bool_t* boolRet) {
    globus_result_t ret;
    PyObject* list;
    char* tmpstr;
    list = PyList_New(0);
    ret = globus_ftp_client_operationattr_get_resume_third_party_transfer(attr, boolRet);
    if (ret != GLOBUS_SUCCESS) {
        PyList_Append(list, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyList_Append(list, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyList_Append(list, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyList_Append(list, Py_None);
    }
    return list;
}

PyObject*
operationattr_set_authorization(globus_ftp_client_operationattr_t* attr, 
                  gss_cred_id_t* credential, PyObject* usr,
                  PyObject *passwd, PyObject* acct, 
                  PyObject *sbjct) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    char *user, *password, *account, *subject;

    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "operationattr_set_authorization: PyTuple_New()");
        return NULL;
    }
    if (usr == Py_None) {
        user = GLOBUS_NULL;
    }
    else {
        user = PyString_AsString(usr);
    }
    if (passwd == Py_None) {
        password = GLOBUS_NULL;
    }
    else {
      password = PyString_AsString(passwd);
    }
    /* The globus documentation is vague on whether account can be passed in as NULL */
    if (acct == Py_None) {
      account = GLOBUS_NULL;
    }
    else {
      account = PyString_AsString(acct);
    }
    /* The globus documentation is vague on whether subject can be passed in as NULL */
    if (sbjct == Py_None) {
      subject = GLOBUS_NULL;
    }
    else {
      subject = PyString_AsString(sbjct);
    }

    ret = globus_ftp_client_operationattr_set_authorization(attr, *credential,
                                                user, password, account, subject);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject*
operationattr_get_authorization(globus_ftp_client_operationattr_t* attr, 
                                char** usr, char** passwd, char** account, 
				char** subject) {
    globus_result_t ret;
    PyObject* list;
    char* tmpstr;
    gss_cred_id_t *credential;
    PyObject * ptr;


    list = PyList_New(0);
    credential = (gss_cred_id_t*)globus_malloc(sizeof(gss_cred_id_t));
    *credential =  GSS_C_NO_CREDENTIAL;
    
    ret = globus_ftp_client_operationattr_get_authorization(attr, credential, usr, passwd,
							    account, subject);
    
    if (ret != GLOBUS_SUCCESS) {
        PyList_Append(list, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyList_Append(list, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyList_Append(list, Py_BuildValue("i", 0));
	ptr = SWIG_NewPointerObj((void *)credential, SWIGTYPE_p_gss_cred_id_t, 0);
	PyList_Append(list, ptr);
	
    }
    return list;
}

PyObject*
operationattr_set_dcau(globus_ftp_client_operationattr_t* attr,
                  const globus_ftp_control_dcau_t* dcau) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "operationattr_set_dcau: PyTuple_New()");
        return NULL;
    }
    ret = globus_ftp_client_operationattr_set_dcau(attr, dcau);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject*
operationattr_get_dcau(globus_ftp_client_operationattr_t* attr) {
    globus_result_t ret;
    PyObject *tuple, *ptr;
    char* tmpstr;
    globus_ftp_control_dcau_t* dcau;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "operationattr_get_dcau: PyTuple_New()");
        return NULL;
    }
    dcau = (globus_ftp_control_dcau_t*)globus_malloc(sizeof(
                                                         globus_ftp_control_dcau_t));
    ret = globus_ftp_client_operationattr_get_dcau(attr, dcau);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
        globus_libc_free(dcau);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        ptr = SWIG_NewPointerObj((void*)dcau, 
				 SWIGTYPE_p_globus_ftp_control_dcau_t, 0);
        PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

PyObject*
operationattr_set_data_protection(globus_ftp_client_operationattr_t* attr, 
                                  char protection) {
    globus_result_t ret;
    globus_ftp_control_protection_t prt;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "operationattr_set_data_protection: PyTuple_New()");
        return NULL;
    }
    if (protection == 'C')
        prt = GLOBUS_FTP_CONTROL_PROTECTION_CLEAR;
    else if (protection == 'S')
        prt = GLOBUS_FTP_CONTROL_PROTECTION_SAFE;
    else if (protection == 'E')
        prt = GLOBUS_FTP_CONTROL_PROTECTION_CONFIDENTIAL;
    else if (protection == 'P')
        prt = GLOBUS_FTP_CONTROL_PROTECTION_PRIVATE;
    ret = globus_ftp_client_operationattr_set_data_protection(attr, prt);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject*
operationattr_get_data_protection(globus_ftp_client_operationattr_t* attr) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    char prt[2];
    globus_ftp_control_protection_t protection;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "get_mode: PyTuple_New()");
        return NULL;
    }
    ret = globus_ftp_client_operationattr_get_data_protection(attr, &protection);
    if (protection == GLOBUS_FTP_CONTROL_MODE_NONE)
        prt[0] = 'C';
    else if (protection == GLOBUS_FTP_CONTROL_MODE_STREAM)
        prt[0] = 'S';
    else if (protection == GLOBUS_FTP_CONTROL_MODE_EXTENDED_BLOCK)
        prt[0] = 'E';
    else if (protection == GLOBUS_FTP_CONTROL_MODE_COMPRESSED)
        prt[0] = 'P';
    prt[1] = '\0';
    if (ret != GLOBUS_SUCCESS) 
    {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
        return tuple;
    }
    else 
    {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", prt));
        return tuple;
    }
}

PyObject*
operationattr_set_control_protection(globus_ftp_client_operationattr_t* attr, 
                                  char protection) {
    globus_result_t ret;
    globus_ftp_control_protection_t prt;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "operationattr_set_control_protection: PyTuple_New()");
        return NULL;
    }
    if (protection == 'C')
        prt = GLOBUS_FTP_CONTROL_PROTECTION_CLEAR;
    else if (protection == 'S')
        prt = GLOBUS_FTP_CONTROL_PROTECTION_SAFE;
    else if (protection == 'E')
        prt = GLOBUS_FTP_CONTROL_PROTECTION_CONFIDENTIAL;
    else if (protection == 'P')
        prt = GLOBUS_FTP_CONTROL_PROTECTION_PRIVATE;
    ret = globus_ftp_client_operationattr_set_control_protection(attr, prt);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject*
operationattr_get_control_protection(globus_ftp_client_operationattr_t* attr) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    char prt[2];
    globus_ftp_control_protection_t protection;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "get_mode: PyTuple_New()");
        return NULL;
    }
    ret = globus_ftp_client_operationattr_get_control_protection(attr, &protection);
    if (protection == GLOBUS_FTP_CONTROL_MODE_NONE)
        prt[0] = 'C';
    else if (protection == GLOBUS_FTP_CONTROL_MODE_STREAM)
        prt[0] = 'S';
    else if (protection == GLOBUS_FTP_CONTROL_MODE_EXTENDED_BLOCK)
        prt[0] = 'E';
    else if (protection == GLOBUS_FTP_CONTROL_MODE_COMPRESSED)
        prt[0] = 'P';
    prt[1] = '\0';
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
	return tuple;
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", prt));
        return tuple;
    }
}

PyObject*
operationattr_set_layout(globus_ftp_client_operationattr_t* attr,
                  const globus_ftp_control_layout_t* layout) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "operationattr_set_layout: PyTuple_New()");
        return NULL;
    }
    ret = globus_ftp_client_operationattr_set_layout(attr, layout);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject*
operationattr_get_layout(globus_ftp_client_operationattr_t* attr) {
    globus_result_t ret;
    PyObject *tuple, *ptr;
    char* tmpstr;
    globus_ftp_control_layout_t* layout;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "operationattr_get_layout: PyTuple_New()");
        return NULL;
    }
    layout = (globus_ftp_control_layout_t*)globus_malloc(sizeof(
                                                         globus_ftp_control_layout_t));
    ret = globus_ftp_client_operationattr_get_layout(attr, layout);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
        globus_libc_free(layout);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        ptr = SWIG_NewPointerObj((void*)layout, 
				 SWIGTYPE_p_globus_ftp_control_layout_t, 0);
        PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

PyObject*
operationattr_set_mode(globus_ftp_client_operationattr_t* attr, char mode) {
    globus_result_t ret;
    globus_ftp_control_mode_t md;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "operationattr_set_mode: PyTuple_New()");
        return NULL;
    }
    if (mode == '0')
        md = GLOBUS_FTP_CONTROL_MODE_NONE;
    else if (mode == 'S')
        md = GLOBUS_FTP_CONTROL_MODE_STREAM;
    else if (mode == 'B')
        md = GLOBUS_FTP_CONTROL_MODE_BLOCK;
    else if (mode == 'E')
        md = GLOBUS_FTP_CONTROL_MODE_EXTENDED_BLOCK;
    else if (mode == 'C')
        md = GLOBUS_FTP_CONTROL_MODE_COMPRESSED;
    ret = globus_ftp_client_operationattr_set_mode(attr, md);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
       
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject*
operationattr_get_mode(globus_ftp_client_operationattr_t* attr) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    char mode[2];
    globus_ftp_control_mode_t md;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "get_mode: PyTuple_New()");
        return NULL;
    }
    ret = globus_ftp_client_operationattr_get_mode(attr, &md);
    if (md == GLOBUS_FTP_CONTROL_MODE_NONE)
        mode[0] = '0';
    else if (md == GLOBUS_FTP_CONTROL_MODE_STREAM)
        mode[0] = 'S';
    else if (md == GLOBUS_FTP_CONTROL_MODE_BLOCK)
        mode[0] = 'B';
    else if (md == GLOBUS_FTP_CONTROL_MODE_EXTENDED_BLOCK)
        mode[0] = 'E';
    else if (md == GLOBUS_FTP_CONTROL_MODE_COMPRESSED)
        mode[0] = 'C';
    else 
        mode[0] = 'Z';
    mode[1] = '\0';
    if (ret != GLOBUS_SUCCESS)
    {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else 
    {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", mode));
    }
    return tuple;
}

PyObject*
operationattr_set_parallelism(globus_ftp_client_operationattr_t* attr,
                     const globus_ftp_control_parallelism_t* par) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "handle_destroy: PyTuple_New()");
        return NULL;
    }
    ret = globus_ftp_client_operationattr_set_parallelism(attr, par);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject*
operationattr_get_parallelism(globus_ftp_client_operationattr_t* attr) {
    globus_result_t ret;
    PyObject *tuple, *ptr;
    char* tmpstr;
    globus_ftp_control_parallelism_t* par;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "attr_init: PyTuple_New()");
        return NULL;
    }
    par = (globus_ftp_control_parallelism_t*)globus_malloc(sizeof(
                                                        globus_ftp_control_parallelism_t));
    ret = globus_ftp_client_operationattr_get_parallelism(attr, par);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        ptr = SWIG_NewPointerObj((void*)par, 
				 SWIGTYPE_p_globus_ftp_control_parallelism_t, 0);
        PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

PyObject*
operationattr_set_tcp_buffer(globus_ftp_client_operationattr_t* attr,
                    const globus_ftp_control_tcpbuffer_t* buffer) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "operationattr_set_tcp_buffer: PyTuple_New()");
        return NULL;
    }
    ret = globus_ftp_client_operationattr_set_tcp_buffer(attr, buffer);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject*
operationattr_get_tcp_buffer(globus_ftp_client_operationattr_t* attr) {
    globus_result_t ret;
    PyObject *tuple, *ptr;
    char* tmpstr;
    globus_ftp_control_tcpbuffer_t* buffer;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "operationattr_get_tcp_buffer: PyTuple_New()");
        return NULL;
    }
    buffer = (globus_ftp_control_tcpbuffer_t*)globus_malloc(sizeof(
                                                      globus_ftp_control_tcpbuffer_t));
    ret = globus_ftp_client_operationattr_get_tcp_buffer(attr, buffer);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        ptr = SWIG_NewPointerObj((void*)buffer, 
				 SWIGTYPE_p_globus_ftp_control_tcpbuffer_t, 0);
        PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

PyObject*
operationattr_set_type(globus_ftp_client_operationattr_t* attr, char type) {
    globus_result_t ret;
    char* tmpstr;
    PyObject* tuple;
    globus_ftp_control_type_t tp;
    if (type == '0')
        tp = GLOBUS_FTP_CONTROL_TYPE_NONE;
    else if (type == 'A')
        tp = GLOBUS_FTP_CONTROL_TYPE_ASCII;
    else if (type == 'E')
        tp = GLOBUS_FTP_CONTROL_TYPE_EBCDIC;
    else if (type == 'I')
        tp = GLOBUS_FTP_CONTROL_TYPE_IMAGE;
    else if (type == 'L')
        tp = GLOBUS_FTP_CONTROL_TYPE_LOCAL;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "operationattr_get_type: PyTuple_New()");
        return NULL;
    }
    ret = globus_ftp_client_operationattr_set_type(attr, tp);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject*
operationattr_get_type(globus_ftp_client_operationattr_t* attr) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    char type[2];
    globus_ftp_control_type_t tp;
    ret = globus_ftp_client_operationattr_get_type(attr, &tp);
    if (tp == GLOBUS_FTP_CONTROL_TYPE_NONE)
        type[0] = '0';
    else if (tp == GLOBUS_FTP_CONTROL_TYPE_ASCII)
        type[0] = 'A';
    else if (tp == GLOBUS_FTP_CONTROL_TYPE_EBCDIC)
        type[0] = 'E';
    else if (tp == GLOBUS_FTP_CONTROL_TYPE_IMAGE)
        type[0] = 'I';
    else if (tp == GLOBUS_FTP_CONTROL_TYPE_LOCAL)
        type[0] = 'L';
    else 
        type[0] = 'Z';
    type[1] = '\0';
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "operationattr_get_type: PyTuple_New()");
        return NULL;
    }
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", type));
    }
    return tuple;
}

PyObject*
restart_marker_init() {
    globus_result_t ret;
    PyObject *tuple, *ptr;
    char* tmpstr;
    globus_ftp_client_restart_marker_t* marker;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "restart_marker_init: PyTuple_New()");
        return NULL;
    }
    marker = (globus_ftp_client_restart_marker_t*)globus_malloc(sizeof(
                                               globus_ftp_client_restart_marker_t));
    ret = globus_ftp_client_restart_marker_init(marker);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
        globus_libc_free(marker);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        ptr = SWIG_NewPointerObj((void*)marker, 
				 SWIGTYPE_p_globus_ftp_client_restart_marker_t, 0);
        PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

PyObject*
restart_marker_destroy(globus_ftp_client_restart_marker_t* marker) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "restart_marker_destroy: PyTuple_New()");
        return NULL;
    }
    ret = globus_ftp_client_restart_marker_destroy(marker);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    globus_free(marker);
    return tuple;
}

PyObject*
restart_marker_copy(globus_ftp_client_restart_marker_t* srcMarker) {
    globus_result_t ret;
    PyObject *tuple, *ptr;
    char* tmpstr;
    globus_ftp_client_restart_marker_t* marker;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "restart_marker_copy: PyTuple_New()");
        return NULL;
    }
    marker = (globus_ftp_client_restart_marker_t*)globus_malloc(sizeof(
                                       globus_ftp_client_restart_marker_t));
    ret = globus_ftp_client_restart_marker_copy(marker, srcMarker);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
        globus_libc_free(marker);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        ptr = SWIG_NewPointerObj((void*)marker, 
				 SWIGTYPE_p_globus_ftp_client_restart_marker_t, 0);
        PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

PyObject*
restart_marker_insert_range(globus_ftp_client_restart_marker_t* marker,
                            PyObject* pOffset, PyObject* pEndOffset) {
    globus_result_t ret;
    char* tmpstr;
    PyObject* tuple;
    globus_off_t offset, endOffset;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "restart_marker_insert_range: PyTuple_New()");
        return NULL;
    }
    if (sizeof(globus_off_t) == 8) {
        /* XXX. this assumes off_t is a long long.  It will
           probably be necessary to ifdef this code for various
           platforms.  This works on 64bit Solaris 7, but I don't
           know about others. */
        if (PyLong_Check(pOffset)) {
            offset = (globus_off_t)PyLong_AsLongLong(pOffset);
        }
        else {
            offset = (globus_off_t)PyInt_AsLong(pOffset);
        }
        if (PyLong_Check(pEndOffset)) {
            endOffset = (globus_off_t)PyLong_AsLongLong(pEndOffset);
        }
        else {
            endOffset = (globus_off_t)PyInt_AsLong(pEndOffset);
        }
    }
    else {
        offset = (globus_off_t)PyInt_AsLong(pOffset);
        endOffset = (globus_off_t)PyInt_AsLong(pEndOffset);
    }
    ret = globus_ftp_client_restart_marker_insert_range(marker, offset, endOffset);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;    
}

PyObject*
restart_marker_set_offset(globus_ftp_client_restart_marker_t* marker,
                          PyObject* pOffset) {
    globus_result_t ret;
    char* tmpstr;
    PyObject* tuple;
    globus_off_t offset;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "restart_marker_set_offset: PyTuple_New()");
        return NULL;
    }
    if ((sizeof(globus_off_t) == 8) && (PyLong_Check(pOffset))) {
        /* XXX. this assumes off_t is a long long.  It will
           probably be necessary to ifdef this code for various
           platforms.  This works on 64bit Solaris 7, but I don't
           know about others. */
        offset = (globus_off_t)PyLong_AsLongLong(pOffset);
    }
    else {
        offset = (globus_off_t)PyInt_AsLong(pOffset);
    }
    ret = globus_ftp_client_restart_marker_set_offset(marker, offset);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;    
}

PyObject*
restart_marker_set_ascii_offset(globus_ftp_client_restart_marker_t* marker,
                                PyObject* pOffset, PyObject* pAsciiOffset) {
    globus_result_t ret;
    char* tmpstr;
    PyObject* tuple;
    globus_off_t offset, asciiOffset;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "restart_marker_set_ascii_offset: PyTuple_New()");
        return NULL;
    }
    if (sizeof(globus_off_t) == 8) {
        /* XXX. this assumes off_t is a long long.  It will
           probably be necessary to ifdef this code for various
           platforms.  This works on 64bit Solaris 7, but I don't
           know about others. */
        if (PyLong_Check(pOffset)) {
            offset = (globus_off_t)PyLong_AsLongLong(pOffset);
        }
        else {
            offset = (globus_off_t)PyInt_AsLong(pOffset);
        }
        if (PyLong_Check(pAsciiOffset)) {
            asciiOffset = (globus_off_t)PyLong_AsLongLong(pAsciiOffset);
        }
        else {
            asciiOffset = (globus_off_t)PyInt_AsLong(pAsciiOffset);
        }
    }
    else {
        offset = (globus_off_t)PyInt_AsLong(pOffset);
        asciiOffset = (globus_off_t)PyInt_AsLong(pAsciiOffset);
    }
    ret = globus_ftp_client_restart_marker_set_ascii_offset(marker,
                                                            offset,
                                                            asciiOffset);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject*
restart_marker_get_total(globus_ftp_client_restart_marker_t* marker) {
    globus_result_t ret;
    char* tmpstr;
    PyObject* tuple;
    PyObject* oSet;
    globus_off_t offset;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "restart_marker_get_total: PyTuple_New()");
        return NULL;
    }
    ret = globus_ftp_client_restart_marker_get_total(marker, &offset);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        if (sizeof(globus_off_t) == 8) {
            /* XXX, This assumes globus_off_t is a long long.  This code will
               probably have to be ifdef'd by platform to account for 
               different types for size_t */
            oSet = PyLong_FromLongLong(offset);
        }
        else {
            oSet = PyInt_FromLong(offset);
        }
        PyTuple_SET_ITEM(tuple, 1, oSet);
    }
    return tuple;
}

PyObject*
restart_marker_to_string(globus_ftp_client_restart_marker_t* marker,
                         char** markerString) {
    globus_result_t ret;
    char* tmpstr;
    PyObject* list;
    list = PyList_New(0);
    ret = globus_ftp_client_restart_marker_to_string(marker, markerString);
    if (ret != GLOBUS_SUCCESS) {
        PyList_Append(list, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyList_Append(list, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyList_Append(list, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyList_Append(list, Py_None);
    }
    return list;
}

PyObject*
restart_marker_from_string(globus_ftp_client_restart_marker_t* marker,
                           const char* markerString) {
    globus_result_t ret;
    char* tmpstr;
    PyObject* tuple;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "restart_marker_from_string: PyTuple_New()");
        return NULL;
    }
    ret = globus_ftp_client_restart_marker_from_string(marker, markerString);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject*
handle_init(globus_ftp_client_handleattr_t* attr) {
    globus_result_t ret;
    PyObject *tuple, *ptr;
    char* tmpstr;
    globus_ftp_client_handle_t* handle;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "handle_init: PyTuple_New()");
        return NULL;
    }
    handle = (globus_ftp_client_handle_t*)globus_malloc(sizeof(globus_ftp_client_handle_t));
    ret = globus_ftp_client_handle_init(handle, attr);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
        globus_libc_free(handle);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        ptr = SWIG_NewPointerObj((void*)handle, 
				 SWIGTYPE_p_globus_ftp_client_handle_t, 0);
        PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

PyObject*
handle_destroy(globus_ftp_client_handle_t* handle) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "handle_destroy: PyTuple_New()");
        return NULL;
    }
    ret = globus_ftp_client_handle_destroy(handle);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    globus_free(handle);
    return tuple;
}

PyObject*
handle_add_plugin(globus_ftp_client_handle_t* handle,
                  globus_ftp_client_plugin_t* plugin) {
    globus_result_t ret;
    PyObject *tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "handle_add_plugin: PyTuple_New()");
        return NULL;
    }
    ret = globus_ftp_client_handle_add_plugin(handle, plugin);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject*
handle_remove_plugin(globus_ftp_client_handle_t* handle,
                     globus_ftp_client_plugin_t* plugin) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "handle_remove_plugin: PyTuple_New()");
        return NULL;
    }
    ret = globus_ftp_client_handle_remove_plugin(handle, plugin);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    globus_free(handle);
    return tuple;
}

PyObject*
handle_cache_url_state(globus_ftp_client_handle_t* handle,
                       const char* url) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "handle_cache_url_state: PyTuple_New()");
        return NULL;
    }
    ret = globus_ftp_client_handle_cache_url_state(handle, url);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    globus_free(handle);
    return tuple;
}

PyObject*
handle_flush_url_state(globus_ftp_client_handle_t* handle,
                       const char* url) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "handle_flush_url_state: PyTuple_New()");
        return NULL;
    }
    ret = globus_ftp_client_handle_flush_url_state(handle, url);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    globus_free(handle);
    return tuple;
}

PyObject*
handle_set_user_pointer(globus_ftp_client_handle_t* handle,
			PyObject* obj) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "handle_set_user_pointer: PyTuple_New()");
        return NULL;
    }
    ret = globus_ftp_client_handle_set_user_pointer(handle, (void*)obj);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        Py_INCREF(obj);  /*How does this ref get freed?*/
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject*
handle_get_user_pointer(const globus_ftp_client_handle_t* handle) {
    globus_result_t ret;
    PyObject* tuple;
    void** ptr;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "handle_get_user_pointer: PyTuple_New()");
        return NULL;
    }
    ret = globus_ftp_client_handle_get_user_pointer(handle, ptr);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("o",(PyObject*)*ptr));
    }
    return tuple;
}

PyObject*
py_delete(globus_ftp_client_handle_t* handle, const char* url,
       globus_ftp_client_operationattr_t* attr,
       PyObject* pyfunc, PyObject* arg) {
    
    globus_result_t ret;
    PyObject *tuple, *ptr;
    char* tmpstr;
    callbackStruct* cbStruct;
	
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "client_delete: PyTuple_New()");
        return NULL;
    }
    cbStruct = (callbackStruct*)globus_malloc(sizeof(callbackStruct));
    Py_XINCREF(pyfunc);
    Py_XINCREF(arg);
    cbStruct->callback = pyfunc;
    cbStruct->arg = arg;
   
    ret = globus_ftp_client_delete(handle, url, attr, 
                                PythonCompleteCallback, (void*)cbStruct);
    
   

    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
        globus_libc_free(cbStruct);
    }
    else {
        ptr = SWIG_NewPointerObj((void*)cbStruct, 
				 SWIGTYPE_p_callbackStruct, 0);
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

PyObject*
py_mkdir(globus_ftp_client_handle_t* handle, const char* url,
         globus_ftp_client_operationattr_t* attr,
         PyObject* pyfunc, PyObject* arg) {
    globus_result_t ret;
    PyObject *tuple, *ptr;
    char* tmpstr;
    callbackStruct* cbStruct;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "client_mkdir: PyTuple_New()");
        return NULL;
    }
    cbStruct = (callbackStruct*)globus_malloc(sizeof(callbackStruct));
    Py_XINCREF(pyfunc);
    Py_XINCREF(arg);
    cbStruct->callback = pyfunc;
    cbStruct->arg = arg;
    ret = globus_ftp_client_mkdir(handle, url, attr, 
                                  PythonCompleteCallback, (void*)cbStruct);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
        globus_libc_free(cbStruct);
    }
    else {
        ptr = SWIG_NewPointerObj((void*)cbStruct, SWIGTYPE_p_callbackStruct, 0);
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

PyObject*
py_rmdir(globus_ftp_client_handle_t* handle, const char* url,
         globus_ftp_client_operationattr_t* attr,
         PyObject* pyfunc, PyObject* arg) {
    globus_result_t ret;
    PyObject *tuple, *ptr;
    char* tmpstr;
    callbackStruct* cbStruct;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "client_mkdir: PyTuple_New()");
        return NULL;
    }
    cbStruct = (callbackStruct*)globus_malloc(sizeof(callbackStruct));
    Py_XINCREF(pyfunc);
    Py_XINCREF(arg);
    cbStruct->callback = pyfunc;
    cbStruct->arg = arg;
    ret = globus_ftp_client_rmdir(handle, url, attr, 
                                  PythonCompleteCallback, (void*)cbStruct);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
        globus_libc_free(cbStruct);
    }
    else {
        ptr = SWIG_NewPointerObj((void*)cbStruct, SWIGTYPE_p_callbackStruct, 0);
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

PyObject*
py_list(globus_ftp_client_handle_t* handle, const char* url,
        globus_ftp_client_operationattr_t* attr,
        PyObject* pyfunc, PyObject* arg) {
    globus_result_t ret;
    PyObject *tuple, *ptr;
    char* tmpstr;
    callbackStruct* cbStruct;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "client_list: PyTuple_New()");
        return NULL;
    }
    cbStruct = (callbackStruct*)globus_malloc(sizeof(callbackStruct));
    Py_XINCREF(pyfunc);
    Py_XINCREF(arg);
    cbStruct->callback = pyfunc;
    cbStruct->arg = arg;
    ret = globus_ftp_client_list(handle, url, attr, 
                                 PythonCompleteCallback, (void*)cbStruct);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
        globus_libc_free(cbStruct);
    }
    else {
        ptr = SWIG_NewPointerObj((void*)cbStruct, SWIGTYPE_p_callbackStruct, 0);
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

PyObject*
verbose_list(globus_ftp_client_handle_t* handle, const char* url,
              globus_ftp_client_operationattr_t* attr,
              PyObject* pyfunc, PyObject* arg) {
    globus_result_t ret;
    PyObject *tuple, *ptr;
    char* tmpstr;
    callbackStruct* cbStruct;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "verbose_list: PyTuple_New()");
        return NULL;
    }
    cbStruct = (callbackStruct*)globus_malloc(sizeof(callbackStruct));
    Py_XINCREF(pyfunc);
    Py_XINCREF(arg);
    cbStruct->callback = pyfunc;
    cbStruct->arg = arg;
    ret = globus_ftp_client_verbose_list(handle, url, attr, 
                                         PythonCompleteCallback, (void*)cbStruct);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
        globus_libc_free(cbStruct);
    }
    else {
        ptr = SWIG_NewPointerObj((void*)cbStruct, SWIGTYPE_p_callbackStruct, 0);
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}


PyObject*
size(globus_ftp_client_handle_t* handle, const char* url,
              globus_ftp_client_operationattr_t* attr,PyObject* pyfunc, PyObject* arg) {
    
    globus_result_t ret;
    PyObject *tuple, * ptr, *ptr2;
    char *tmpStr;
    callbackStruct* cbStruct;
    globus_off_t *fileSize;
    if (!(tuple=PyTuple_New(3)))
    {
        PyErr_SetString(PyExc_RuntimeError, "size: PyTuple_New()");
        return NULL;
    }
    
    fileSize = (globus_off_t*)globus_malloc(sizeof(globus_off_t));
    cbStruct = (callbackStruct*)globus_malloc(sizeof(callbackStruct));
    Py_XINCREF(pyfunc);
    Py_XINCREF(arg);
    cbStruct->callback = pyfunc;
    cbStruct->arg = arg;
    

    ret = globus_ftp_client_size(handle, url, attr, fileSize, 
                                         PythonCompleteCallback, (void*)cbStruct);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
	PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("i", ret));
        tmpStr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 2, Py_BuildValue("s", tmpStr));
        globus_libc_free(tmpStr);
        globus_libc_free(cbStruct);
    }
    else 
    {
	
        ptr = SWIG_NewPointerObj((void*)cbStruct, SWIGTYPE_p_callbackStruct, 0);
      	PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
	ptr2 = SWIG_NewPointerObj((void*)fileSize, SWIGTYPE_p_globus_off_t, 0);
	Py_XINCREF(ptr2);
	PyTuple_SET_ITEM(tuple, 1, ptr2);
	PyTuple_SET_ITEM(tuple, 2, ptr);
    }
    return tuple;
}


	
PyObject *
exists(globus_ftp_client_handle_t* handle, const char* url,
              globus_ftp_client_operationattr_t* attr,PyObject* pyfunc, PyObject* arg) {
    
    globus_result_t ret;
    PyObject *tuple, * ptr;
    char *tmpStr;
    callbackStruct* cbStruct;
    
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "size: PyTuple_New()");
        return NULL;
    }
    
    cbStruct = (callbackStruct*)globus_malloc(sizeof(callbackStruct));
    Py_XINCREF(pyfunc);
    Py_XINCREF(arg);
    cbStruct->callback = pyfunc;
    cbStruct->arg = arg;
    

    ret = globus_ftp_client_exists(handle, url, attr,  
                                         PythonCompleteCallback, (void*)cbStruct);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpStr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpStr));
        globus_libc_free(tmpStr);
        globus_libc_free(cbStruct);
    }
    else 
    {
        ptr = SWIG_NewPointerObj((void*)cbStruct, SWIGTYPE_p_callbackStruct, 0);
      	PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
	PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

PyObject*
modification_time(globus_ftp_client_handle_t* handle, const char* url,
              globus_ftp_client_operationattr_t* attr,
              PyObject* pyfunc, PyObject* arg) {
    
    globus_result_t ret;
    PyObject *tuple, * ptr, *ptr2;
    char *tmpStr;
    callbackStruct* cbStruct;
    globus_abstime_t * modTime;

    if (!(tuple=PyTuple_New(3)))
    {
        PyErr_SetString(PyExc_RuntimeError, "size: PyTuple_New()");
        return NULL;
    }
     cbStruct = (callbackStruct*)globus_malloc(sizeof(callbackStruct));
     modTime = (globus_abstime_t *)globus_malloc(sizeof(globus_abstime_t));
    Py_XINCREF(pyfunc);
    Py_XINCREF(arg);
    cbStruct->callback = pyfunc;
    cbStruct->arg = arg;
    ret = globus_ftp_client_modification_time(handle, url, attr, modTime,
                                         PythonCompleteCallback, (void*)cbStruct);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpStr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpStr));
        PyTuple_SET_ITEM(tuple, 2, Py_BuildValue("i", ret));
	globus_libc_free(tmpStr);
        globus_libc_free(cbStruct);
    }
    else {
        ptr = SWIG_NewPointerObj((void*)cbStruct, SWIGTYPE_p_callbackStruct, 0);
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        PyTuple_SET_ITEM(tuple, 1, ptr);
	ptr2 = SWIG_NewPointerObj((void*)modTime, SWIGTYPE_p_globus_abstime_t, 0);
	Py_XINCREF(ptr2);
	PyTuple_SET_ITEM(tuple, 2, ptr2);
    }
    return tuple;
}

PyObject*
move(globus_ftp_client_handle_t* handle, const char* srcUrl,
     const char* destUrl, globus_ftp_client_operationattr_t* attr,
     PyObject* pyfunc, PyObject* arg) {
    callbackStruct *cbStruct;
    globus_result_t ret;
    PyObject *tuple, *ptr;
    char* tmpstr;
    
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "verbose_list: PyTuple_New()");
        return NULL;
    }
    cbStruct = (callbackStruct*)globus_malloc(sizeof(callbackStruct));
    Py_XINCREF(pyfunc);
    Py_XINCREF(arg);
    cbStruct->callback = pyfunc;
    cbStruct->arg = arg;
    ret = globus_ftp_client_move(handle, srcUrl, destUrl, attr, 
                                         PythonCompleteCallback, (void*)cbStruct);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
        globus_libc_free(cbStruct);
    }
    else {
        ptr = SWIG_NewPointerObj((void*)cbStruct, SWIGTYPE_p_callbackStruct, 0);
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

PyObject*
get(globus_ftp_client_handle_t* handle, const char* url,
           globus_ftp_client_operationattr_t* attr, 
           globus_ftp_client_restart_marker_t* marker,
           PyObject* pyfunc, PyObject* arg) {
    globus_result_t ret;
    PyObject *tuple, *ptr;
    char* tmpstr;
    callbackStruct* cbStruct;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "client_get: PyTuple_New()");
        return NULL;
    }
    cbStruct = (callbackStruct*)globus_malloc(sizeof(callbackStruct));
    Py_XINCREF(pyfunc);
    Py_XINCREF(arg);
    cbStruct->callback = pyfunc;
    cbStruct->arg = arg;
    Py_BEGIN_ALLOW_THREADS
    ret = globus_ftp_client_get(handle, url, attr, marker, 
                                PythonCompleteCallback, (void*)cbStruct);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
        globus_libc_free(cbStruct);
    }
    else {
        ptr = SWIG_NewPointerObj((void*)cbStruct, SWIGTYPE_p_callbackStruct, 0);
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

PyObject*
partial_get(globus_ftp_client_handle_t* handle, const char* url,
            globus_ftp_client_operationattr_t* attr, 
            globus_ftp_client_restart_marker_t* marker,
            PyObject* pyOffset, PyObject* pyEndOffset,
	    PyObject* pyfunc, PyObject* arg) {
    globus_result_t ret;
    PyObject *tuple, *ptr;
    char* tmpstr;
    globus_off_t offset, endOffset;
    callbackStruct* cbStruct;
   
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "client_get: PyTuple_New()");
        return NULL;
    }
    cbStruct = (callbackStruct*)globus_malloc(sizeof(callbackStruct));
    Py_XINCREF(pyfunc);
    Py_XINCREF(arg);
    cbStruct->callback = pyfunc;
    cbStruct->arg = arg;
    if (sizeof(globus_off_t) == 8) {
        /* XXX. this assumes off_t is a long long.  It will
           probably be necessary to ifdef this code for various
           platforms.  This works on 64bit Solaris 7, but I don't
           know about others. */
        if (PyLong_Check(pyOffset)) {
            offset = (globus_off_t)PyLong_AsLongLong(pyOffset);
        }
        else {
            offset = (globus_off_t)PyInt_AsLong(pyOffset);
        }
        if (PyLong_Check(pyEndOffset)) {
            endOffset = (globus_off_t)PyLong_AsLongLong(pyEndOffset);
        }
        else {
            endOffset = (globus_off_t)PyInt_AsLong(pyEndOffset);
        }
    }
    else {
        offset = (globus_off_t)PyInt_AsLong(pyOffset);
        endOffset = (globus_off_t)PyInt_AsLong(pyEndOffset);
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_ftp_client_partial_get(handle, url, attr, marker, 
					offset, endOffset,
					PythonCompleteCallback, (void*)cbStruct);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
        globus_libc_free(cbStruct);
    }
    else {
        ptr = SWIG_NewPointerObj((void*)cbStruct, SWIGTYPE_p_callbackStruct, 0);
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

PyObject*
extended_get(globus_ftp_client_handle_t* handle, const char* url,
             globus_ftp_client_operationattr_t* attr, 
             globus_ftp_client_restart_marker_t* marker,
             const char* eret_alg_str,
             PyObject* pyfunc, PyObject* arg) {
    globus_result_t ret;
    PyObject *tuple, *ptr;
    char* tmpstr;
    callbackStruct* cbStruct;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "client_extended_get: PyTuple_New()");
        return NULL;
    }
    cbStruct = (callbackStruct*)globus_malloc(sizeof(callbackStruct));
    Py_XINCREF(pyfunc);
    Py_XINCREF(arg);
    cbStruct->callback = pyfunc;
    cbStruct->arg = arg;
    Py_BEGIN_ALLOW_THREADS
    ret = globus_ftp_client_extended_get(handle, url, attr, marker, 
                                         eret_alg_str, 
                                         PythonCompleteCallback, (void*)cbStruct);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
        globus_libc_free(cbStruct);
    }
    else {
        ptr = SWIG_NewPointerObj((void*)cbStruct, SWIGTYPE_p_callbackStruct, 0);
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

PyObject*
put(globus_ftp_client_handle_t* handle, const char* url,
           globus_ftp_client_operationattr_t* attr, 
           globus_ftp_client_restart_marker_t* marker,
           PyObject* pyfunc, PyObject* arg) {
    globus_result_t ret;
    PyObject *tuple, *ptr;
    char* tmpstr;
    callbackStruct* cbStruct;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "client_put: PyTuple_New()");
        return NULL;
    }
    cbStruct = (callbackStruct*)globus_malloc(sizeof(callbackStruct));
    Py_XINCREF(pyfunc);
    Py_XINCREF(arg);
    cbStruct->callback = pyfunc;
    cbStruct->arg = arg;
    
    Py_BEGIN_ALLOW_THREADS
    ret = globus_ftp_client_put(handle, url, attr, marker, 
                                PythonCompleteCallback, (void*)cbStruct);
    
    Py_END_ALLOW_THREADS
	
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
        globus_libc_free(cbStruct);
    }
    else {
        ptr = SWIG_NewPointerObj((void*)cbStruct, SWIGTYPE_p_callbackStruct, 0);
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

PyObject*
partial_put(globus_ftp_client_handle_t* handle, const char* url,
	    globus_ftp_client_operationattr_t* attr,
	    globus_ftp_client_restart_marker_t* marker,
	    PyObject* pyOffset, PyObject* pyEndOffset,
	    PyObject* pyfunc, PyObject* arg) {
    globus_result_t ret;
    PyObject *tuple, *ptr;
    char* tmpstr;
    globus_off_t offset, endOffset;
    callbackStruct* cbStruct;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "client_partial_put: PyTuple_New()");
        return NULL;
    }
    cbStruct = (callbackStruct*)globus_malloc(sizeof(callbackStruct));
    Py_XINCREF(pyfunc);
    Py_XINCREF(arg);
    cbStruct->callback = pyfunc;
    cbStruct->arg = arg;
    if (sizeof(globus_off_t) == 8) {
        /* XXX. this assumes off_t is a long long.  It will
           probably be necessary to ifdef this code for various
           platforms.  This works on 64bit Solaris 7, but I don't
           know about others. */
        if (PyLong_Check(pyOffset)) {
            offset = (globus_off_t)PyLong_AsLongLong(pyOffset);
        }
        else {
            offset = (globus_off_t)PyInt_AsLong(pyOffset);
        }
        if (PyLong_Check(pyEndOffset)) {
            endOffset = (globus_off_t)PyLong_AsLongLong(pyEndOffset);
        }
        else {
            endOffset = (globus_off_t)PyInt_AsLong(pyEndOffset);
        }
    }
    else {
        offset = (globus_off_t)PyInt_AsLong(pyOffset);
        endOffset = (globus_off_t)PyInt_AsLong(pyEndOffset);
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_ftp_client_partial_put(handle, url, attr, marker,
					offset, endOffset,
					PythonCompleteCallback, (void*)cbStruct);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
        globus_libc_free(cbStruct);
    }
    else {
        ptr = SWIG_NewPointerObj((void*)cbStruct, SWIGTYPE_p_callbackStruct, 0);
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

PyObject*
extended_put(globus_ftp_client_handle_t* handle, const char* url,
             globus_ftp_client_operationattr_t* attr, 
             globus_ftp_client_restart_marker_t* marker,
             const char* esto_alg_str,
             PyObject* pyfunc, PyObject* arg) {
    globus_result_t ret;
    PyObject *tuple, *ptr;
    char* tmpstr;
    callbackStruct* cbStruct;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "client_extended_put: PyTuple_New()");
        return NULL;
    }
    cbStruct = (callbackStruct*)globus_malloc(sizeof(callbackStruct));
    Py_XINCREF(pyfunc);
    Py_XINCREF(arg);
    cbStruct->callback = pyfunc;
    cbStruct->arg = arg;
    
    Py_BEGIN_ALLOW_THREADS
    ret = globus_ftp_client_extended_put(handle, url, attr, marker, esto_alg_str, 
                                PythonCompleteCallback, (void*)cbStruct);
    
    Py_END_ALLOW_THREADS
	
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
        globus_libc_free(cbStruct);
    }
    else {
        ptr = SWIG_NewPointerObj((void*)cbStruct, SWIGTYPE_p_callbackStruct, 0);
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

PyObject*
third_party_transfer(globus_ftp_client_handle_t* handle, const char* srcUrl,
                     globus_ftp_client_operationattr_t* srcAttr, const char* dstUrl,
                     globus_ftp_client_operationattr_t* dstAttr, 
                     globus_ftp_client_restart_marker_t* marker,
                     PyObject* pyfunc, PyObject* arg) {
    globus_result_t ret;
    PyObject *tuple, *ptr;
    char* tmpstr;
    callbackStruct* cbStruct;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "client_get: PyTuple_New()");
        return NULL;
    }
    cbStruct = (callbackStruct*)globus_malloc(sizeof(callbackStruct));
    Py_XINCREF(pyfunc);
    Py_XINCREF(arg);
    cbStruct->callback = pyfunc;
    cbStruct->arg = arg;
    Py_BEGIN_ALLOW_THREADS
    ret = globus_ftp_client_third_party_transfer(handle, srcUrl, srcAttr, 
                                                 dstUrl, dstAttr, marker, 
                                                 PythonCompleteCallback, 
                                                 (void*)cbStruct);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
        globus_libc_free(cbStruct);
    }
    else {
        ptr = SWIG_NewPointerObj((void*)cbStruct, SWIGTYPE_p_callbackStruct, 0);
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

PyObject*
partial_third_party_transfer(globus_ftp_client_handle_t* handle, 
			     const char* srcUrl, 
			     globus_ftp_client_operationattr_t* srcAttr, 
			     const char* dstUrl,
			     globus_ftp_client_operationattr_t* dstAttr, 
			     globus_ftp_client_restart_marker_t* marker,
			     PyObject* pyOffset, PyObject* pyEndOffset,
			     PyObject* pyfunc, PyObject* arg) {
    globus_result_t ret;
    PyObject *tuple, *ptr;
    char* tmpstr;
    globus_off_t offset, endOffset;
    callbackStruct* cbStruct;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "client_third_party: PyTuple_New()");
        return NULL;
    }
    cbStruct = (callbackStruct*)globus_malloc(sizeof(callbackStruct));
    Py_XINCREF(pyfunc);
    Py_XINCREF(arg);
    cbStruct->callback = pyfunc;
    cbStruct->arg = arg;
    if (sizeof(globus_off_t) == 8) {
        /* XXX. this assumes off_t is a long long.  It will
           probably be necessary to ifdef this code for various
           platforms.  This works on 64bit Solaris 7, but I don't
           know about others. */
        if (PyLong_Check(pyOffset)) {
            offset = (globus_off_t)PyLong_AsLongLong(pyOffset);
        }
        else {
            offset = (globus_off_t)PyInt_AsLong(pyOffset);
        }
        if (PyLong_Check(pyEndOffset)) {
            endOffset = (globus_off_t)PyLong_AsLongLong(pyEndOffset);
        }
        else {
            endOffset = (globus_off_t)PyInt_AsLong(pyEndOffset);
        }
    }
    else {
        offset = (globus_off_t)PyInt_AsLong(pyOffset);
        endOffset = (globus_off_t)PyInt_AsLong(pyEndOffset);
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_ftp_client_partial_third_party_transfer(handle, srcUrl, srcAttr, 
							 dstUrl, dstAttr, marker,
							 offset, endOffset,
							 PythonCompleteCallback, 
							 (void*)cbStruct);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
        globus_libc_free(cbStruct);
    }
    else {
        ptr = SWIG_NewPointerObj((void*)cbStruct, SWIGTYPE_p_callbackStruct, 0);
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

PyObject*
extended_third_party_transfer(globus_ftp_client_handle_t* handle, 
                              const char* srcUrl,
                              globus_ftp_client_operationattr_t* srcAttr, 
                              const char* eret_alg_str,
                              const char* dstUrl,
                              globus_ftp_client_operationattr_t* dstAttr,
                              const char* esto_alg_str,
                              globus_ftp_client_restart_marker_t* marker,
                              PyObject* pyfunc, PyObject* arg) {
    globus_result_t ret;
    PyObject *tuple, *ptr;
    char* tmpstr;
    callbackStruct* cbStruct;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "client_extended_third_party: PyTuple_New()");
        return NULL;
    }
    cbStruct = (callbackStruct*)globus_malloc(sizeof(callbackStruct));
    Py_XINCREF(pyfunc);
    Py_XINCREF(arg);
    cbStruct->callback = pyfunc;
    cbStruct->arg = arg;
    Py_BEGIN_ALLOW_THREADS
    ret = globus_ftp_client_extended_third_party_transfer(handle, srcUrl, 
                                                          srcAttr, eret_alg_str, 
                                                          dstUrl, dstAttr, 
                                                          esto_alg_str, marker, 
                                                          PythonCompleteCallback, 
                                                          (void*)cbStruct);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
        globus_libc_free(cbStruct);
    }
    else {
        ptr = SWIG_NewPointerObj((void*)cbStruct, SWIGTYPE_p_callbackStruct, 0);
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

PyObject*
py_abort(globus_ftp_client_handle_t* handle) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "abort: PyTuple_New()");
        return NULL;
    }
    ret = globus_ftp_client_abort(handle);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
	Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
} 
	
PyObject*
register_read(globus_ftp_client_handle_t* handle, PyObject* pyBuffer, 
	      globus_byte_t * buffer,int size, PyObject* pyfunc, PyObject* arg) {
    globus_result_t ret;
    PyObject *tuple, *ptr;
    char* tmpstr;
    callbackStruct4* cbStruct;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "register_read: PyTuple_New()");
        return NULL;
    }
    cbStruct = (callbackStruct4*)globus_malloc(sizeof(callbackStruct4));
    Py_XINCREF(pyfunc);
    Py_XINCREF(arg);
    Py_XINCREF(pyBuffer);
    cbStruct->callback = pyfunc;
    cbStruct->arg = arg;
    cbStruct->buffer = pyBuffer;
    ret = globus_ftp_client_register_read(handle, buffer, size, 
                                          PythonDataCallback, cbStruct);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
        globus_libc_free(cbStruct);
    }
    else {
        ptr = SWIG_NewPointerObj((void*)cbStruct, SWIGTYPE_p_callbackStruct, 0);
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

PyObject*
register_write(globus_ftp_client_handle_t* handle, PyObject* buffer,
               int size, PyObject* offSet, globus_bool_t eof, 
               PyObject* pyfunc, PyObject* arg) {
    globus_result_t ret;
    PyObject *tuple, *ptr;
    char* tmpstr;
    globus_off_t offset;
    globus_byte_t* buf;
    int sizeNew;
    callbackStruct4* cbStruct;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "client_get: PyTuple_New()");
        return NULL;
    }
    cbStruct = (callbackStruct4*)globus_malloc(sizeof(callbackStruct4));
    Py_XINCREF(pyfunc);
    Py_XINCREF(arg);
    Py_XINCREF(buffer);
    cbStruct->callback = pyfunc;
    cbStruct->arg = arg;
    cbStruct->buffer = buffer;	 
    if ((sizeof(globus_off_t) == 8) && (PyLong_Check(offSet))) {
        /* XXX. this assumes off_t is a long long.  It will
           probably be necessary to ifdef this code for various
           platforms.  This works on 64bit Solaris 7, but I don't
           know about others. */
        offset = (globus_off_t)PyLong_AsLongLong(offSet);
    }
    else {
        offset = (globus_off_t)PyInt_AsLong(offSet);
    }
    
    PyString_AsStringAndSize(buffer, (char**)&buf, &sizeNew);
    ret = globus_ftp_client_register_write(handle, buf, sizeNew, offset, eof, 
                                          PythonDataCallback, cbStruct);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
        globus_libc_free(cbStruct);
    }
    else {
        ptr = SWIG_NewPointerObj((void*)cbStruct, SWIGTYPE_p_callbackStruct, 0);
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

void
free_callback(callbackStruct* cbStruct) {
    Py_XDECREF(cbStruct->callback);
    Py_XDECREF(cbStruct->arg);
    globus_free(cbStruct);
}

void
free_callback2(callbackStruct2* cbStruct) {
    Py_XDECREF(cbStruct->cb1);
    Py_XDECREF(cbStruct->cb2);
    Py_XDECREF(cbStruct->cb3);
    Py_XDECREF(cbStruct->arg);
    globus_free(cbStruct);
}

void
free_callback3(callbackStruct3* cbStruct) {
    Py_XDECREF(cbStruct->cb1);
    Py_XDECREF(cbStruct->cb2);
    Py_XDECREF(cbStruct->cb3);
    Py_XDECREF(cbStruct->cb4);
    Py_XDECREF(cbStruct->arg);
    globus_free(cbStruct);
}

PyObject*
debug_plugin_init(FILE* stream, const char* text) {
    globus_ftp_client_plugin_t* plugin;
    globus_result_t ret;
    char* tmpstr;
    PyObject *tuple, *ptr;

    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "debug_plugin_init: PyTuple_New()");
        return NULL;
    }
    plugin = (globus_ftp_client_plugin_t*)globus_malloc(sizeof(globus_ftp_client_plugin_t));
    ret = globus_ftp_client_debug_plugin_init(plugin, stream, text);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
        globus_libc_free(plugin);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        ptr = SWIG_NewPointerObj((void*)plugin, 
				 SWIGTYPE_p_globus_ftp_client_plugin_t, 0);
        PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

PyObject*
debug_plugin_destroy(globus_ftp_client_plugin_t* plugin) {
    globus_result_t ret;
    PyObject *tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "debug_plugin_destroy: PyTuple_New()");
        return NULL;
    }
    ret = globus_ftp_client_debug_plugin_destroy(plugin);
    globus_free(plugin);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject*
restart_marker_plugin_init(PyObject* beginCB, PyObject* markerCB, 
                           PyObject* completeCB, PyObject* arg) {
    globus_ftp_client_plugin_t* plugin;
    globus_result_t ret;
    char* tmpstr;
    PyObject *tuple, *ptr, *ptr2;
    callbackStruct2* cbStruct;
    if (!(tuple=PyTuple_New(3)))
    {
        PyErr_SetString(PyExc_RuntimeError, "restart_marker_plugin_init: PyTuple_New()");
        return NULL;
    }
    cbStruct = (callbackStruct2*)globus_malloc(sizeof(callbackStruct2));
    Py_XINCREF(beginCB);
    Py_XINCREF(markerCB);
    Py_XINCREF(completeCB);
    Py_XINCREF(arg);
    cbStruct->cb1 = beginCB;
    cbStruct->cb2 = markerCB;
    cbStruct->cb3 = completeCB;
    cbStruct->arg = arg;
    plugin = (globus_ftp_client_plugin_t*)globus_malloc(sizeof(globus_ftp_client_plugin_t));
    ret = globus_ftp_client_restart_marker_plugin_init(plugin, PythonRestartMarkerPluginBeginCallback, 
                                                       PythonRestartMarkerPluginMarkerCallback,
                                                       PythonRestartMarkerPluginCompleteCallback, (void*)cbStruct);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 2, Py_None);
        globus_libc_free(tmpstr);
        globus_libc_free(plugin);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        ptr = SWIG_NewPointerObj((void*)plugin, 
				 SWIGTYPE_p_globus_ftp_client_plugin_t, 0);
         ptr2 = SWIG_NewPointerObj((void*)cbStruct, 
				 SWIGTYPE_p_callbackStruct2, 0);
        PyTuple_SET_ITEM(tuple, 1, ptr);
        PyTuple_SET_ITEM(tuple, 2, ptr2);
    }
    return tuple;
}

PyObject*
restart_marker_plugin_destroy(globus_ftp_client_plugin_t* plugin) {
    globus_result_t ret;
    PyObject *tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "restart_marker_plugin_destroy: PyTuple_New()");
        return NULL;
    }
    ret = globus_ftp_client_restart_marker_plugin_destroy(plugin);
    globus_free(plugin);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject*
performance_marker_plugin_init(PyObject* beginCB, PyObject* markerCB, 
			       PyObject* completeCB, PyObject* arg) {
    globus_ftp_client_plugin_t* plugin;
    globus_result_t ret;
    char* tmpstr;
    PyObject *tuple, *ptr, *ptr2;
    callbackStruct2* cbStruct;
    if (!(tuple=PyTuple_New(3)))
    {
        PyErr_SetString(PyExc_RuntimeError, "restart_marker_plugin_init: PyTuple_New()");
        return NULL;
    }
    cbStruct = (callbackStruct2*)globus_malloc(sizeof(callbackStruct2));
    Py_XINCREF(beginCB);
    Py_XINCREF(markerCB);
    Py_XINCREF(completeCB);
    Py_XINCREF(arg);
    cbStruct->cb1 = beginCB;
    cbStruct->cb2 = markerCB;
    cbStruct->cb3 = completeCB;
    cbStruct->arg = arg;
    plugin = (globus_ftp_client_plugin_t*)globus_malloc(sizeof(globus_ftp_client_plugin_t));
    ret = globus_ftp_client_perf_plugin_init(plugin, PythonPerfMarkerPluginBeginCallback, 
					     PythonPerfMarkerPluginMarkerCallback,
					     PythonPerfMarkerPluginCompleteCallback, (void*)cbStruct);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 2, Py_None);
        globus_libc_free(tmpstr);
        globus_libc_free(plugin);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        ptr = SWIG_NewPointerObj((void*)plugin, 
				 SWIGTYPE_p_globus_ftp_client_plugin_t, 0);
	ptr2 = SWIG_NewPointerObj((void*)cbStruct, 
				  SWIGTYPE_p_callbackStruct2, 0);
        PyTuple_SET_ITEM(tuple, 1, ptr);
        PyTuple_SET_ITEM(tuple, 2, ptr2);
    }
    return tuple;
}

PyObject*
performance_marker_plugin_destroy(globus_ftp_client_plugin_t* plugin) {
    globus_result_t ret;
    PyObject *tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "performance_marker_plugin_destroy: PyTuple_New()");
        return NULL;
    }
    ret = globus_ftp_client_perf_plugin_destroy(plugin);
    globus_free(plugin);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}
PyObject*
throughput_plugin_init(PyObject* beginCB, PyObject* stripeCB, PyObject* totalCB,
		       PyObject* completeCB, PyObject* arg) {
    globus_ftp_client_plugin_t* plugin;
    globus_result_t ret;
    char* tmpstr;
    PyObject *tuple, *ptr, *ptr2;
    callbackStruct3* cbStruct;

    if (!(tuple=PyTuple_New(3)))
    {
        PyErr_SetString(PyExc_RuntimeError, "throughput_plugin_init: PyTuple_New()");
        return NULL;
    }
    cbStruct = (callbackStruct3*)globus_malloc(sizeof(callbackStruct3));
    Py_XINCREF(beginCB);
    Py_XINCREF(stripeCB);
    Py_XINCREF(totalCB);
    Py_XINCREF(completeCB);
    Py_XINCREF(arg);
    cbStruct->cb1 = beginCB;
    cbStruct->cb2 = stripeCB;
    cbStruct->cb3 = totalCB;
    cbStruct->cb4 = completeCB;
    cbStruct->arg = arg;
    plugin = (globus_ftp_client_plugin_t*)globus_malloc(sizeof(globus_ftp_client_plugin_t));
    ret = globus_ftp_client_throughput_plugin_init(plugin, PythonThroughputPluginBeginCallback, 
						   PythonThroughputPluginStripeCallback,
						   PythonThroughputPluginTotalCallback,
						   PythonThroughputPluginCompleteCallback, (void*)cbStruct);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 2, Py_None);
        globus_libc_free(tmpstr);
        globus_libc_free(plugin);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        ptr = SWIG_NewPointerObj((void*)plugin, 
				 SWIGTYPE_p_globus_ftp_client_plugin_t, 0);
	ptr2 = SWIG_NewPointerObj((void*)cbStruct, 
				  SWIGTYPE_p_callbackStruct3, 0);
        PyTuple_SET_ITEM(tuple, 1, ptr);
        PyTuple_SET_ITEM(tuple, 2, ptr2);
    }
    return tuple;
}
PyObject*
throughput_plugin_destroy(globus_ftp_client_plugin_t* plugin) {
    globus_result_t ret;
    PyObject *tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "throughput_plugin_destroy: PyTuple_New()");
        return NULL;
    }
    ret = globus_ftp_client_throughput_plugin_destroy(plugin);
    globus_free(plugin);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

/* ----------------------------- 
PyObject*
netlogger_plugin_init(PyObject* nl_url, PyObject* prog_name, PyObject* opaque_string) {
    globus_ftp_client_plugin_t* plugin;
    globus_result_t ret;
    char* tmpstr;
    char* nl_url_p;
    char* prog_name_p;
    char* opaque_string_p;
    PyObject *tuple, *ptr;

    if (!(tuple=PyTuple_New(3)))
    {
        PyErr_SetString(PyExc_RuntimeError, "netlogger_plugin_init: PyTuple_New()");
        return NULL;
    }
#if defined(GLOBUS_BUILD_WITH_NETLOGGER)
    Py_XINCREF(nl_url);
    Py_XINCREF(prog_name);
    Py_XINCREF(opaque_string);
    nl_url_p = PyString_AsString(nl_url);
    prog_name_p = PyString_AsString(prog_name);
    opaque_string_p = PyString_AsString(opaque_string);
    plugin = (globus_ftp_client_plugin_t*)globus_malloc(sizeof(globus_ftp_client_plugin_t));
    ret = globus_ftp_client_throughput_nl_plugin_init(plugin, nl_url_p, 
						      prog_name_p, opaque_string_p);
    
    if (ret != GLOBUS_SUCCESS) {
	PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
	tmpstr = globus_object_printable_to_string(globus_error_get(ret));
	PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
	globus_libc_free(tmpstr);
	globus_libc_free(plugin);
    }
    else {
	PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
	ptr = SWIG_NewPointerObj((void*)plugin, 
				 SWIGTYPE_p_globus_ftp_client_plugin_t, 0);
	PyTuple_SET_ITEM(tuple, 1, ptr);
    }
#else
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", "not compiled with support for netlogger"));
#endif  
    return tuple;
}

PyObject*
netlogger_plugin_destroy(globus_ftp_client_plugin_t* plugin) {
    globus_result_t ret;
    PyObject *tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "netlogger_plugin_destroy: PyTuple_New()");
        return NULL;
    }
#if defined(GLOBUS_BUILD_WITH_NETLOGGER)
    ret = globus_ftp_client_restart_marker_plugin_destroy(plugin);
    globus_free(plugin);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
#else
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", "not compiled with support for netlogger"));
#endif  
    return tuple;
}
*/

static globus_ftp_client_plugin_t *
pyglobus_plugin_init(PyObject *python_function_list) {
    globus_ftp_client_plugin_t* plugin;
    pyglobus_plugin_t *pyglobus_plugin;
    globus_result_t result;


/*     PyEval_AcquireThread(get_python_thread_state(python_thread_state_key, python_interp));  */
    /* TODO: properly check for errors and do cleanup */
    plugin = (globus_ftp_client_plugin_t*)globus_malloc(sizeof(globus_ftp_client_plugin_t));
    pyglobus_plugin = (pyglobus_plugin_t *)globus_malloc(sizeof(pyglobus_plugin_t));
    pyglobus_plugin->python_function_list = python_function_list;
    Py_INCREF(python_function_list);
    pyglobus_plugin->python_user_specific = Py_None;
    Py_INCREF(Py_None);

/*     Py_BEGIN_ALLOW_THREADS */
    result = globus_ftp_client_plugin_init(plugin, "pyglobus_plugin", GLOBUS_FTP_CLIENT_CMD_MASK_ALL, pyglobus_plugin);

    globus_ftp_client_plugin_set_get_func(plugin, PythonPluginGet);
    globus_ftp_client_plugin_set_copy_func(plugin, PythonPluginCopy);
    globus_ftp_client_plugin_set_destroy_func(plugin, PythonPluginDestroy);
    globus_ftp_client_plugin_set_connect_func(plugin, PythonPluginConnect);
    globus_ftp_client_plugin_set_authenticate_func(plugin, PythonPluginAuthenticate);
    globus_ftp_client_plugin_set_delete_func(plugin, PythonPluginDelete);
    globus_ftp_client_plugin_set_mkdir_func(plugin, PythonPluginMkdir);
    globus_ftp_client_plugin_set_rmdir_func(plugin, PythonPluginRmdir);
    globus_ftp_client_plugin_set_list_func(plugin, PythonPluginList);
    globus_ftp_client_plugin_set_verbose_list_func(plugin, PythonPluginVerboseList);
    globus_ftp_client_plugin_set_move_func(plugin, PythonPluginMove);
    globus_ftp_client_plugin_set_get_func(plugin, PythonPluginGet);
    globus_ftp_client_plugin_set_put_func(plugin, PythonPluginPut);
    globus_ftp_client_plugin_set_third_party_transfer_func(plugin, PythonPluginThirdPartyTransfer);
    globus_ftp_client_plugin_set_modification_time_func(plugin, PythonPluginModificationTime);
    globus_ftp_client_plugin_set_size_func(plugin, PythonPluginSize);
    globus_ftp_client_plugin_set_abort_func(plugin, PythonPluginAbort);
    globus_ftp_client_plugin_set_read_func(plugin, PythonPluginRead);
    globus_ftp_client_plugin_set_write_func(plugin, PythonPluginWrite);
    globus_ftp_client_plugin_set_data_func(plugin, PythonPluginData);
    globus_ftp_client_plugin_set_command_func(plugin, PythonPluginCommand);
    globus_ftp_client_plugin_set_response_func(plugin, PythonPluginResponse);
    globus_ftp_client_plugin_set_fault_func(plugin, PythonPluginFault);
    globus_ftp_client_plugin_set_complete_func(plugin, PythonPluginComplete);

/*     Py_END_ALLOW_THREADS */

/*       PyEval_ReleaseThread(get_python_thread_state(python_thread_state_key, python_interp));  */

    return plugin;
}
%}
