/*############################################################################
# Keith R. Jackson, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################*/
%module gassCachec
%{
#undef HAVE_LIMITS_H
#undef SIZEOF_OFF_T
#undef socklen_t
#include "globus_gass_cache.h"
%}
typedef int globus_bool_t; 

%include ./src/pyGlobus-typemaps.i
/*
%apply char **OUTPUT { char **file };
%apply unsigned long *ulongRet { unsigned long *timestamp };
*/


%constant GLOBUS_GASS_CACHE_ADD_NEW = 1;
%constant GLOBUS_GASS_CACHE_URL_NOT_FOUND = 2;
%constant GLOBUS_GASS_CACHE_ADD_EXISTS = 3;

%inline %{
globus_module_descriptor_t* get_module(void) {
    return GLOBUS_GASS_CACHE_MODULE;
}

PyObject*
cache_open(PyObject* pyPath) {
    int ret;
    PyObject *tuple, *ptr;
    char *path;
    globus_gass_cache_t *handle;
    const char *tmpstr;

    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "cache_open: PyTuple_New()");
        return NULL;
    }
    handle = (globus_gass_cache_t*)globus_malloc(sizeof(globus_gass_cache_t));
    if (pyPath == Py_None)
    {
        path = NULL;
    }
    else 
    {
        path = PyString_AsString(pyPath);
    }
    
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gass_cache_open(path, handle);
    Py_END_ALLOW_THREADS
	 
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS) 
    {
	tmpstr = globus_gass_cache_error_string(ret);
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s",tmpstr));
    }
    else 
    {
        ptr = SWIG_NewPointerObj((void*)handle, SWIGTYPE_p_globus_gass_cache_t, 0);
        PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

void
handle_free(globus_gass_cache_t *handle) {
    globus_free(handle);
    return;
}

PyObject *
cache_close(globus_gass_cache_t *handle) {
    int ret;
    const char * tmpstr;
    PyObject * tuple;

    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "cache_close: PyTuple_New()");
        return NULL;
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gass_cache_close(handle);
    Py_END_ALLOW_THREADS
    
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if ( ret != GLOBUS_SUCCESS)
    {
	tmpstr = globus_gass_cache_error_string(ret);
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s",tmpstr));
    }
    else
    {
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple,1, Py_None);
    }
    return tuple;
	
}


PyObject *
cache_add(globus_gass_cache_t *handle,
          char *url, char *tag, globus_bool_t create) 
{
    int ret;
    PyObject * tuple;
    const char * tmpstr;
    unsigned long timestamp;
    char * local_filename = NULL;
 
    if (!(tuple=PyTuple_New(3)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "cache_add: PyTuple_New()");
        return NULL;
    }

    Py_BEGIN_ALLOW_THREADS
    ret = globus_gass_cache_add(*handle, url, tag, create, &timestamp, &local_filename);
    Py_END_ALLOW_THREADS

    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));	
    
    /*return codes 0,1,2,3 are okay MKR*/

    if ( ! (ret == 0 || ret == 1 || ret == 2 || ret ==3))
    {
        tmpstr = globus_gass_cache_error_string(ret);
	PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s",tmpstr));
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 2, Py_None);
    }    
    else
    {
	PyTuple_SET_ITEM(tuple,1,Py_BuildValue("s",local_filename));
	PyTuple_SET_ITEM(tuple,2, Py_BuildValue("i", timestamp));
    }
    /*globus_gass_cache_add allocates memory for local filename which needs to be freed by the caller*/
    globus_free(local_filename);
    return tuple;	
}

PyObject *
add_done(globus_gass_cache_t *cache_handle, const char *url, const char *tag, unsigned long timestamp)
{
    int ret;
    PyObject * tuple;
    const char * tmpstr;
    unsigned long timestamp2 = 0;

    if (!(tuple=PyTuple_New(2)))
    {
    PyErr_SetString(PyExc_RuntimeError, 
    "add_done: PyTuple_New()");
    return NULL;
    } 

	
    Py_BEGIN_ALLOW_THREADS   
	ret = globus_gass_cache_add_done(*cache_handle, url, tag, timestamp2);
    Py_END_ALLOW_THREADS
	
    PyTuple_SET_ITEM(tuple,0,Py_BuildValue("i",ret));
    if( ret != GLOBUS_SUCCESS)
    {   
	    tmpstr = globus_gass_cache_error_string(ret);
	    PyTuple_SET_ITEM(tuple,1,Py_BuildValue("s",tmpstr));
    }
    else 
    {
	    Py_INCREF(Py_None);
	    PyTuple_SET_ITEM(tuple,1, Py_None);
    }

    return tuple;
}

PyObject *
delete_start(globus_gass_cache_t *cache_handle, const char *url, 
	     const char *tag)
{
    int ret;
    PyObject * tuple;
    const  char * tmpstr;
    unsigned long timestamp;

    if (!(tuple=PyTuple_New(2)))
    {
    PyErr_SetString(PyExc_RuntimeError, 
    "delete_start: PyTuple_New()");
    return NULL;
    } 

    Py_BEGIN_ALLOW_THREADS   
	ret = globus_gass_cache_delete_start(*cache_handle, url, tag, &timestamp);
    Py_END_ALLOW_THREADS
	 
    PyTuple_SET_ITEM(tuple,0,Py_BuildValue("i",ret));
    if( ret != GLOBUS_SUCCESS)
    {   
	    tmpstr = globus_gass_cache_error_string(ret);
	    PyTuple_SET_ITEM(tuple,1,Py_BuildValue("s",tmpstr));
    }
    else 
    {
	    Py_INCREF(Py_None);
	    PyTuple_SET_ITEM(tuple,1, Py_BuildValue("i", timestamp));
    }

    return tuple;

}

PyObject *
do_delete(globus_gass_cache_t *cache_handle, const char *url, 
	     const char *tag, unsigned long timestamp, globus_bool_t is_locked)
{
    int ret;
    PyObject * tuple;
    const  char * tmpstr;

    if (!(tuple=PyTuple_New(2)))
    {
    PyErr_SetString(PyExc_RuntimeError, 
    "delete: PyTuple_New()");
    return NULL;
    } 

    Py_BEGIN_ALLOW_THREADS   
	ret = globus_gass_cache_delete(*cache_handle, url, tag, timestamp, is_locked);
    Py_END_ALLOW_THREADS
	 
    PyTuple_SET_ITEM(tuple,0,Py_BuildValue("i",ret));
    if( ret != GLOBUS_SUCCESS)
    {   
	    tmpstr = globus_gass_cache_error_string(ret);
	    PyTuple_SET_ITEM(tuple,1,Py_BuildValue("s",tmpstr));
    }
    else 
    {
	    Py_INCREF(Py_None);
	    PyTuple_SET_ITEM(tuple,1, Py_None);
    }

    return tuple;

}

PyObject *
cleanup_tag(globus_gass_cache_t *cache_handle, const char *url, 
	    const char *tag)
{
    int ret;
    PyObject * tuple;
    const  char * tmpstr;

    if (!(tuple=PyTuple_New(2)))
    {
    PyErr_SetString(PyExc_RuntimeError, 
    "cleanup_tag: PyTuple_New()");
    return NULL;
    } 
	
    Py_BEGIN_ALLOW_THREADS   
	ret = globus_gass_cache_cleanup_tag(*cache_handle, url, tag);
    Py_END_ALLOW_THREADS
	 
    PyTuple_SET_ITEM(tuple,0,Py_BuildValue("i",ret));
    if( ret != GLOBUS_SUCCESS)
    {   
	    tmpstr = globus_gass_cache_error_string(ret);
	    PyTuple_SET_ITEM(tuple,1,Py_BuildValue("s",tmpstr));
    }
    else 
    {
	    Py_INCREF(Py_None);
	    PyTuple_SET_ITEM(tuple,1, Py_None);
    }

    return tuple;

}

PyObject * 
cache_query(globus_gass_cache_t *cache_handle, const char *url,
            const char *tag, globus_bool_t wait_for_lock,
  	    unsigned long timestamp)
{
    globus_bool_t is_locked = GLOBUS_TRUE;

    char *local_filename= NULL;
    int ret;
    PyObject * tuple;
    const  char * tmpstr;

    if (!(tuple=PyTuple_New(2)))    {
    PyErr_SetString(PyExc_RuntimeError, "cache_query: PyTuple_New()");
    return NULL;
    } 
    
    Py_BEGIN_ALLOW_THREADS   
	ret = globus_gass_cache_query(*cache_handle, url, tag, wait_for_lock, \
				      &timestamp, &local_filename, &is_locked);
    Py_END_ALLOW_THREADS
   
	globus_free(local_filename); 
    PyTuple_SET_ITEM(tuple,0,Py_BuildValue("i",ret));
    if( ret != GLOBUS_SUCCESS)
    {   
	    tmpstr = globus_gass_cache_error_string(ret);
	    PyTuple_SET_ITEM(tuple,1,Py_BuildValue("s",tmpstr));
    }
    else 
    {
	if(is_locked) PyTuple_SET_ITEM(tuple,1, Py_True);
	else PyTuple_SET_ITEM(tuple,1, Py_False);
    }

    return tuple;          
}

PyObject *
mangle_url(globus_gass_cache_t *cache_handle,const char *url)
{
    int ret;
    int length;
    int max_mangled_length;
    char * mangled_string;
    const char * errString;
    PyObject * tuple, *pyString;
    mangled_string = NULL;
	
    if (!(tuple=PyTuple_New(2)))
    {
    PyErr_SetString(PyExc_RuntimeError, 
    "mangle_url: PyTuple_New()");
    return NULL;
    } 

    Py_BEGIN_ALLOW_THREADS                                               
	    ret = globus_gass_cache_mangle_url(*cache_handle, url, &mangled_string, &length);
    Py_END_ALLOW_THREADS
    PyTuple_SET_ITEM(tuple,0,Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS)
    {
         errString = globus_gass_cache_error_string(ret); 
   	 PyTuple_SET_ITEM(tuple,1,Py_BuildValue("s", errString));
    }	
    else
    {
	PyTuple_SET_ITEM(tuple,1,Py_BuildValue("s",mangled_string)); 
	globus_free(mangled_string);
    }	 

	return tuple;
}
 
PyObject *
mangle_tag(const globus_gass_cache_t *cache_handle,const char *url)
{

  int ret;
  int length= -1;
  int max_mangled_length;
  char * mangled_string;
  const char * errString;
  PyObject * tuple, *pyString;
  mangled_string = NULL;
	
  if (!(tuple=PyTuple_New(2)))
  {
      PyErr_SetString(PyExc_RuntimeError, 
		      "mangle_tagPyTuple_New()");
      return NULL;
  } 

    Py_BEGIN_ALLOW_THREADS                                               
    ret = globus_gass_cache_mangle_tag(*cache_handle, url, &mangled_string, &length);
    Py_END_ALLOW_THREADS

	PyTuple_SET_ITEM(tuple,0,Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS)
    {
	errString = globus_gass_cache_error_string(ret); 
	PyTuple_SET_ITEM(tuple,1,Py_BuildValue("s", errString));
    }	
    else
    {	
	PyTuple_SET_ITEM(tuple,1,Py_BuildValue("s",mangled_string)); 
	globus_free(mangled_string);
    }	 
    
    return tuple;
}   



PyObject *
cache_get_dirs(const globus_gass_cache_t *cache_handle,
               const char *url,const char *tag)
{
    int ret;
    char *global_root;
    char *local_root;
    char *tmp_root;
    char *log_root; 
    char *global_dir; 
    char *local_dir;
    const char * errString;
    PyObject * tuple;
	
    if (!(tuple=PyTuple_New(7)))
    {
    PyErr_SetString(PyExc_RuntimeError, 
    "cache_get_dirs: PyTuple_New()");
    return NULL;
    } 
    
    Py_BEGIN_ALLOW_THREADS                                               
	    ret = globus_gass_cache_get_dirs(*cache_handle, url, 
                                             tag,&global_root,
                                             &local_root, &tmp_root, 
                                             &log_root, &global_dir, 
                                             &local_dir);
    Py_END_ALLOW_THREADS
    PyTuple_SET_ITEM(tuple,0,Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS)
    {
         errString = globus_gass_cache_error_string(ret);  	
	 PyTuple_SET_ITEM(tuple,1,Py_BuildValue("s",errString));
	 Py_INCREF(Py_None);
         PyTuple_SET_ITEM(tuple,2,Py_None);
         Py_INCREF(Py_None);
         PyTuple_SET_ITEM(tuple,3,Py_None);
         Py_INCREF(Py_None);
         PyTuple_SET_ITEM(tuple,4,Py_None);
         Py_INCREF(Py_None);
         PyTuple_SET_ITEM(tuple,5,Py_None);
	 Py_INCREF(Py_None);
         PyTuple_SET_ITEM(tuple,6,Py_None);
	 Py_INCREF(Py_None);
   }
   else
   {
	 PyTuple_SET_ITEM(tuple,1,Py_BuildValue("s",global_root));
         PyTuple_SET_ITEM(tuple,2,Py_BuildValue("s",local_root));
         PyTuple_SET_ITEM(tuple,3,Py_BuildValue("s",tmp_root));
         PyTuple_SET_ITEM(tuple,4,Py_BuildValue("s",log_root));
         PyTuple_SET_ITEM(tuple,5,Py_BuildValue("s",global_dir));
         PyTuple_SET_ITEM(tuple,6,Py_BuildValue("s",local_dir)); 
   }
    globus_free(global_root);
    globus_free(local_root);
    globus_free(tmp_root);
    globus_free(log_root); 
    globus_free(global_dir); 
    globus_free(local_dir);
    
    return tuple;
}



%}
