/*############################################################################
# Keith R. Jackson, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################*/
%module gassCopyc
%{
#undef SIZEOF_OFF_T
#undef HAVE_LIMITS_H
#undef socklen_t
#include "globus_common.h"
#include "globus_object.h"
#include "globus_gass_copy.h"
#include "pyGlobus_common.h"
%}

%include ./src/pyGlobus-typemaps.i

%typemap(python, argout) globus_gass_copy_url_mode_t* mode {
    PyObject* o;
    o = PyInt_FromLong(*$1);
    if ((!$result) || ($result == Py_None)) {
        $result = o;
    } else {
        if (!PyList_Check($result)) {
            PyObject *o2 = $result;
            $result = PyList_New(0);
            PyList_Append($result, o2);
            Py_XDECREF(o2);
        }
        PyList_Append($result, o);
        Py_XDECREF(o);
    }
}

%typemap(python, argout) globus_gass_copy_status_t* status {
    PyObject* o;
    o = PyInt_FromLong(*$1);
    if ((!$result) || ($result == Py_None)) {
        $result = o;
    } else {
        if (!PyList_Check($result)) {
            PyObject *o2 = $result;
            $result = PyList_New(0);
            PyList_Append($result, o2);
            Py_XDECREF(o2);
        }
        PyList_Append($result, o);
        Py_XDECREF(o);
    }
}

%typemap(python, in, numinputs=1) 
globus_gass_copy_url_mode_t* mode(globus_gass_copy_url_mode_t temp) {
    $1 = &temp;
}

%typemap(python, in, numinputs=1) 
globus_gass_copy_status_t* status(globus_gass_copy_status_t temp) {
    $1 = &temp;
}

%init %{
    PyEval_InitThreads();
    python_interp = PyThreadState_Get()->interp;
    init_globus_thread_state_key(&python_thread_state_key);
%}

%{

static globus_thread_key_t python_thread_state_key;
static PyInterpreterState *python_interp; 


PyObject * check_error(globus_object_t *  error){
    PyObject *tuple, *err;

    if(!(tuple = PyTuple_New(2))) {
    	PyErr_SetString(PyExc_RuntimeError,"PythonCallback, PyTuple_New");	
    }
    else{ 
	if (error) {  
            char *tmpstr;
	    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
	    tmpstr = globus_object_printable_to_string(error);
            PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
            globus_libc_free(tmpstr);
        }
	else{
	    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
            Py_INCREF(Py_None);
            PyTuple_SET_ITEM(tuple, 1, Py_None);
	}
    }	
    
    if ((err = PyErr_Occurred()) != NULL){ 
	PyErr_Print();
    }
    return tuple;
}


static void PythonCallback(void* user_data, 
                           globus_gass_copy_handle_t* handle, globus_object_t* error) {
    PyObject *func, *arglist, *result, *arg, *ptr, *errorObject;
    char* err;
    callbackStruct* cbStruct = (callbackStruct*)user_data;
    PyEval_AcquireThread(get_python_thread_state(python_thread_state_key, python_interp));;
    func = cbStruct->callback;
    arg = cbStruct->arg;
    ptr = SWIG_NewPointerObj((void*)handle, 
			     SWIGTYPE_p_globus_gass_copy_handle_t, 0);

    errorObject = check_error(error);
    arglist = Py_BuildValue("(OsO)", arg, ptr, errorObject);
    result = PyEval_CallObject(func, arglist);
    if (result == NULL) {
      PyErr_Print();
    }
    Py_DECREF(ptr);
    Py_DECREF(arglist);
    Py_XDECREF(result);
    Py_XDECREF(errorObject);

    PyEval_ReleaseThread(get_python_thread_state(python_thread_state_key, python_interp));
    return; 
}
%}

%inline %{
globus_module_descriptor_t* 
get_module(void) {
    return GLOBUS_GASS_COPY_MODULE;
}

PyObject*
handle_init(globus_gass_copy_handleattr_t* attr) {
    globus_result_t ret;
    PyObject *tuple, *ptr;
    char* tmpstr;
    globus_gass_copy_handle_t* handle;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "handle_init: PyTuple_New()");
        return NULL;
    }
    handle = (globus_gass_copy_handle_t*)globus_libc_malloc(
                                                      sizeof(globus_gass_copy_handle_t));
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gass_copy_handle_init(handle, attr);
    Py_END_ALLOW_THREADS
	PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS) {        
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
        globus_libc_free(handle);
    }
    else {
        ptr = SWIG_NewPointerObj((void*)handle, 
				 SWIGTYPE_p_globus_gass_copy_handle_t, 0);
        PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

PyObject*
handle_destroy(globus_gass_copy_handle_t* handle) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "handle_destroy: PyTuple_New()");
        return NULL;
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gass_copy_handle_destroy(handle);
    Py_END_ALLOW_THREADS
	PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS) {
       
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    globus_free(handle);
    return tuple;
}

PyObject*
attr_init() {
    globus_result_t ret;
    PyObject *tuple, *ptr;
    char* tmpstr;
    globus_gass_copy_attr_t* attr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "attr_init: PyTuple_New()");
        return NULL;
    }
    attr = (globus_gass_copy_attr_t*)globus_libc_malloc(sizeof(globus_gass_copy_attr_t));
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gass_copy_attr_init(attr);
    Py_END_ALLOW_THREADS
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS) {
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
        globus_libc_free(attr);
    }
    else {
        ptr = SWIG_NewPointerObj((void*)attr, 
				 SWIGTYPE_p_globus_gass_copy_attr_t, 0);
        PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

void
attr_destroy(globus_gass_copy_attr_t* attr) {
    globus_libc_free(attr);
    return;
}

PyObject*
handleattr_init() {
    globus_result_t ret;
    PyObject *tuple, *ptr;
    char* tmpstr;
    globus_gass_copy_handleattr_t* attr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "attr_init: PyTuple_New()");
        return NULL;
    }
    attr = (globus_gass_copy_handleattr_t*)globus_libc_malloc(sizeof(
                                                        globus_gass_copy_handleattr_t));
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gass_copy_handleattr_init(attr);
    Py_END_ALLOW_THREADS
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS) {
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
        globus_libc_free(attr);
    }
    else {
        ptr = SWIG_NewPointerObj((void*)attr, 
				 SWIGTYPE_p_globus_gass_copy_handleattr_t, 0);
        PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

void
handleattr_destroy(globus_gass_copy_handleattr_t* attr) {
    globus_libc_free(attr);
    return;
}

PyObject*
handleattr_set_ftp(globus_gass_copy_handleattr_t* attr, 
                   globus_ftp_client_handleattr_t* ftpAttr) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "handleattr_set_ftp: PyTuple_New()");
        return NULL;
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gass_copy_handleattr_set_ftp_attr(attr, ftpAttr);
    Py_END_ALLOW_THREADS
     PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS) {
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject*
copy_url_to_url(globus_gass_copy_handle_t* handle, char* srcUrl, 
                globus_gass_copy_attr_t* srcAttr, char* destUrl,
                globus_gass_copy_attr_t* dstAttr ) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "copy_url_to_url: PyTuple_New()");
        return NULL;
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gass_copy_url_to_url(handle, srcUrl, srcAttr, 
                                           destUrl, dstAttr);
    Py_END_ALLOW_THREADS
	 PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS) {
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject*
copy_url_to_handle(globus_gass_copy_handle_t* handle, char* srcUrl,
                   globus_gass_copy_attr_t* srcAttr, 
                   globus_io_handle_t* dstHandle) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "copy_url_to_handle: PyTuple_New()");
        return NULL;
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gass_copy_url_to_handle(handle, srcUrl, srcAttr, dstHandle);
    Py_END_ALLOW_THREADS
    	 PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS) {
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject*
copy_handle_to_url(globus_gass_copy_handle_t* handle, 
                   globus_io_handle_t* srcHandle, char* dstUrl,
                   globus_gass_copy_attr_t* dstAttr) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "copy_url_to_handle: PyTuple_New()");
        return NULL;
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gass_copy_handle_to_url(handle, srcHandle, dstUrl, dstAttr);
    Py_END_ALLOW_THREADS
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS) {
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject*
register_copy_url_to_url(globus_gass_copy_handle_t* handle, 
                         char* srcUrl, globus_gass_copy_attr_t* srcAttr,
                         char* dstUrl, globus_gass_copy_attr_t* dstAttr,
                         PyObject* pyfunc, PyObject* arg) {
    globus_result_t ret;
    PyObject *tuple, *cbPtr;
    char* tmpstr;
    callbackStruct* cbStruct;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "register_copy_url_to_url: PyTuple_New()");
        return NULL;
    }
    cbStruct = (callbackStruct*)globus_malloc(sizeof(callbackStruct));
    Py_XINCREF(pyfunc);
    Py_XINCREF(arg);
    cbStruct->callback = pyfunc;
    cbStruct->arg = arg;
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gass_copy_register_url_to_url(handle, srcUrl, 
                                                    srcAttr, dstUrl,
                                                    dstAttr, 
                                                    PythonCallback,
                                                    (void*)cbStruct);
    Py_END_ALLOW_THREADS
      PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
     if (ret != GLOBUS_SUCCESS) {
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
        globus_libc_free(cbStruct);
    }
    else {
        cbPtr = SWIG_NewPointerObj((void*)cbStruct, 
				   SWIGTYPE_p_callbackStruct, 0);
        PyTuple_SET_ITEM(tuple, 1, cbPtr);
    }
    return tuple;
}

PyObject*
register_copy_handle_to_url(globus_gass_copy_handle_t* handle, 
                            globus_io_handle_t* srcHandle, char* dstUrl, 
                            globus_gass_copy_attr_t* dstAttr,
                            PyObject* pyfunc, PyObject* arg) {
    globus_result_t ret;
    PyObject *tuple, *cbPtr;
    char* tmpstr;   
    callbackStruct* cbStruct;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "register_copy_url_to_url: PyTuple_New()");
        return NULL;
    }
    cbStruct = (callbackStruct*)globus_malloc(sizeof(callbackStruct));
    Py_XINCREF(pyfunc);
    Py_XINCREF(arg);
    cbStruct->callback = pyfunc;
    cbStruct->arg = arg;
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gass_copy_register_handle_to_url(handle, srcHandle, 
                                                       dstUrl, dstAttr, 
                                                       PythonCallback,
                                                       (void*)cbStruct);
    Py_END_ALLOW_THREADS
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS) {
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
        globus_libc_free(cbStruct);
    }
    else {
        cbPtr = SWIG_NewPointerObj((void*)cbStruct, SWIGTYPE_p_callbackStruct, 0);
        PyTuple_SET_ITEM(tuple, 1, cbPtr);
    }
    return tuple;
}

PyObject*
register_copy_url_to_handle(globus_gass_copy_handle_t* handle, 
                            char* srcUrl, globus_gass_copy_attr_t* srcAttr,
                            globus_io_handle_t* dstHandle, 
                            PyObject* pyfunc, PyObject* arg) {
    globus_result_t ret;
    PyObject *tuple, *cbPtr;
    char* tmpstr;   
    callbackStruct* cbStruct;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "register_copy_url_to_url: PyTuple_New()");
        return NULL;
    }
    cbStruct = (callbackStruct*)globus_malloc(sizeof(callbackStruct));
    Py_XINCREF(pyfunc);
    Py_XINCREF(arg);
    cbStruct->callback = pyfunc;
    cbStruct->arg = arg;
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gass_copy_register_url_to_handle(handle, srcUrl, 
                                                       srcAttr, dstHandle,
                                                       PythonCallback,
                                                       (void*)cbStruct);
    Py_END_ALLOW_THREADS
	PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS) {
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
        globus_libc_free(cbStruct);
    }
    else {
        cbPtr = SWIG_NewPointerObj((void*)cbStruct, SWIGTYPE_p_callbackStruct, 0);
        PyTuple_SET_ITEM(tuple, 1, cbPtr);
    }
    return tuple;
}

PyObject*
set_buffer_length(globus_gass_copy_handle_t* handle, int length) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "handle_destroy: PyTuple_New()");
        return NULL;
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gass_copy_set_buffer_length(handle, length);
    Py_END_ALLOW_THREADS
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS) {
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject*
get_url_mode(char* url, globus_gass_copy_url_mode_t* mode) {
    globus_result_t ret;
    PyObject* list;
    char* tmpstr;
    list = PyList_New(0);
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gass_copy_get_url_mode(url, mode);
    Py_END_ALLOW_THREADS
    PyList_Append(list, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS) {
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyList_Append(list, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        Py_INCREF(Py_None);
        PyList_Append(list, Py_None); 
    }
    return list;
}

PyObject*
get_status(globus_gass_copy_handle_t* handle, 
           globus_gass_copy_status_t* status) {
    globus_result_t ret;
    PyObject* list;
    char* tmpstr;
    list = PyList_New(0);
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gass_copy_get_status(handle, status);
    Py_END_ALLOW_THREADS
    PyList_Append(list, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS) {
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyList_Append(list, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        Py_INCREF(Py_None);
        PyList_Append(list, Py_None); 
    }
    return list;
}

PyObject*
flush_url_state(globus_gass_copy_handle_t* handle, char* url) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "handle_destroy: PyTuple_New()");
        return NULL;
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gass_copy_flush_url_state(handle, url);
    Py_END_ALLOW_THREADS
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS) {
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject*
cache_url_state(globus_gass_copy_handle_t* handle, char* url) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "handle_destroy: PyTuple_New()");
        return NULL;
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gass_copy_cache_url_state(handle, url);
    Py_END_ALLOW_THREADS
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS) {
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject*
attr_set_io(globus_gass_copy_attr_t* attr, globus_io_attr_t* ioAttr) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "handle_destroy: PyTuple_New()");
        return NULL;
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gass_copy_attr_set_io(attr, ioAttr);
    Py_END_ALLOW_THREADS
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS) {
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject*
attr_set_gass(globus_gass_copy_attr_t* attr, 
              globus_gass_transfer_requestattr_t* gassAttr) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "handle_destroy: PyTuple_New()");
        return NULL;
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gass_copy_attr_set_gass(attr, gassAttr);
    Py_END_ALLOW_THREADS
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS) {
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject*
attr_set_ftp(globus_gass_copy_attr_t* attr, 
             globus_ftp_client_operationattr_t* ftpAttr) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "handle_destroy: PyTuple_New()");
        return NULL;
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gass_copy_attr_set_ftp(attr, ftpAttr);
    Py_END_ALLOW_THREADS
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS) {
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

void
free_callback(callbackStruct* cbStruct) {
    Py_XDECREF(cbStruct->callback);
    Py_XDECREF(cbStruct->arg);
    globus_free(cbStruct);
}
%}
