/*############################################################################
# Keith R. Jackson, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################*/
%module gassTransferc
%{
#undef HAVE_LIMITS_H
#undef SIZEOF_OFF_T
#undef socklen_t
#include "globus_common.h"
#include "globus_object.h"
#include "globus_gass_transfer.h"
#include "pyGlobus_common.h"
%}

%include ./src/pyGlobus-typemaps.i
typedef int globus_bool_t;
%types(void*);
%apply globus_bool_t *boolRet { globus_bool_t *connect_reuse };


%init %{
    PyEval_InitThreads();
    python_interp = PyThreadState_Get()->interp;
    init_globus_thread_state_key(&python_thread_state_key);
%}

%{

static globus_thread_key_t python_thread_state_key;
static PyInterpreterState *python_interp; 

static void PythonGassTransferCallback(void* user_data, 
				       globus_gass_transfer_request_t request) {

    PyObject *func, *arglist, *result, *arg, *ptr;
    callbackStruct* cbStruct = (callbackStruct*)user_data;
    PyEval_AcquireThread(get_python_thread_state(python_thread_state_key, python_interp));;
    func = cbStruct->callback;
    arg = cbStruct->arg;
    ptr = SWIG_NewPointerObj((void*)&request, SWIGTYPE_p_globus_gass_transfer_request_t,0);
    arglist = Py_BuildValue("(OO)", arg, ptr);
    result = PyEval_CallObject(func, arglist);
    if (result == NULL) {
      PyErr_Print();
    }
    Py_DECREF(ptr);
    Py_DECREF(arglist);
    Py_XDECREF(result);
    PyEval_ReleaseThread(get_python_thread_state(python_thread_state_key, python_interp));
    return;
}

static void PythonGassTransferCloseCallback(void* user_data, 
				       globus_gass_transfer_listener_t listener) {

    PyObject *func, *arglist, *result, *arg, *ptr;
    callbackStruct* cbStruct = (callbackStruct*)user_data;
    PyEval_AcquireThread(get_python_thread_state(python_thread_state_key, python_interp));;
    func = cbStruct->callback;
    arg = cbStruct->arg;
    ptr = SWIG_NewPointerObj((void*)&listener, SWIGTYPE_p_globus_gass_transfer_listener_t,0);
    arglist = Py_BuildValue("(OO)", arg, ptr);
    result = PyEval_CallObject(func, arglist);
    if (result == NULL) {
      PyErr_Print();
    }
    Py_DECREF(ptr);
    Py_DECREF(arglist);
    Py_XDECREF(result);
    PyEval_ReleaseThread(get_python_thread_state(python_thread_state_key, python_interp));
    return;
}

static void PythonGassTransferBytesCallback(void * user_data,
					    globus_gass_transfer_request_t request, 
					    globus_byte_t * bytes, globus_size_t length, 
					    globus_bool_t last_data){
	
    PyObject *func, *arglist, *result, *arg, *ptr, *buffer;
    callbackStruct* cbStruct = (callbackStruct*)user_data;
    PyEval_AcquireThread(get_python_thread_state(python_thread_state_key, python_interp));;
    func = cbStruct->callback;
    arg = cbStruct->arg;
    ptr = SWIG_NewPointerObj((void*)&request, SWIGTYPE_p_globus_gass_transfer_request_t,0);
    buffer = PyBuffer_FromReadWriteMemory((void*)bytes, length*sizeof(globus_byte_t));
    arglist = Py_BuildValue("(OOOii)", arg, ptr, buffer,length, (int)last_data);
    result = PyEval_CallObject(func, arglist);
    if (result == NULL) {
      PyErr_Print();
    }
    
    Py_DECREF(ptr);
    Py_DECREF(arglist);
    Py_DECREF(buffer);
    Py_XDECREF(result);
    PyEval_ReleaseThread(get_python_thread_state(python_thread_state_key, python_interp));
    
    return;

 }
%}

%inline %{

globus_module_descriptor_t* 
get_module(void) {
    return GLOBUS_GASS_TRANSFER_MODULE;
}

void
free_callback(callbackStruct * cbStruct){
    Py_XDECREF(cbStruct->callback);
    Py_XDECREF(cbStruct->arg);
    globus_free(cbStruct);
}    

PyObject*
reqattr_init(char* urlScheme) {
    int ret;
    PyObject *tuple, *ptr;
    globus_gass_transfer_requestattr_t* attr;
    if (!(tuple=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, "attr_init: PyTuple_New()");
        return NULL;
    }
    attr = (globus_gass_transfer_requestattr_t*)globus_libc_malloc(sizeof(globus_gass_transfer_requestattr_t));
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gass_transfer_requestattr_init(attr, urlScheme);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
	PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", "Error creating request attribute"));
        globus_libc_free(attr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        ptr = SWIG_NewPointerObj((void*)attr, 
				 SWIGTYPE_p_globus_gass_transfer_requestattr_t, 0);
        PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

int 
reqattr_destroy(globus_gass_transfer_requestattr_t* attr) {
    int ret;   
    ret = globus_gass_transfer_requestattr_destroy(attr);
    globus_libc_free(attr);
    return ret;
}

int 
request_destroy(globus_gass_transfer_request_t* request){
    int ret;
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gass_transfer_request_destroy(*request);
    Py_END_ALLOW_THREADS	
    return ret;
}

PyObject*
listenerattr_init(char * urlScheme){
    int ret;
    PyObject *tuple,*ptr;
    globus_gass_transfer_listenerattr_t* attr;
    
    if (!(tuple=PyTuple_New(2))){
	PyErr_SetString(PyExc_RuntimeError, "attr_init: PyTuple_New()");
	return NULL;
    }
	
    attr = (globus_gass_transfer_listenerattr_t*)globus_libc_malloc(sizeof(globus_gass_transfer_listenerattr_t));
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gass_transfer_listenerattr_init(attr, urlScheme);
    Py_END_ALLOW_THREADS
	
     if (ret != GLOBUS_SUCCESS) 
     {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
	PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", "Error creating listener attribute"));
        globus_libc_free(attr);
     }
     else {
	PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
	ptr = SWIG_NewPointerObj((void*)attr, 
				 SWIGTYPE_p_globus_gass_transfer_listenerattr_t, 0);
	PyTuple_SET_ITEM(tuple, 1, ptr);
     }
    return tuple;
}

int
listenerattr_destroy(globus_gass_transfer_listenerattr_t * attr){

    int ret;
    /*You'd think that there would be a listenerattr destroy method but there isn't
    Its declared here  globus_gass_transfer.h:687:globus_gass_transfer_listenerattr_destroy(
    But it was never implemented, MKR */
    ret = globus_gass_transfer_requestattr_destroy(attr);
    globus_libc_free(attr);
    return ret;
}



PyObject *
register_get(globus_gass_transfer_requestattr_t *attr,
	     char*  url, PyObject * pyfunc, PyObject* arg){
    int ret;
    PyObject * list, *ptr, *ptr2;
    callbackStruct * cbStruct;
    globus_gass_transfer_request_t * request;
    
    request = (globus_gass_transfer_request_t *)globus_malloc(sizeof(globus_gass_transfer_request_t));
    cbStruct = (callbackStruct *)globus_malloc(sizeof(callbackStruct));
    Py_XINCREF(pyfunc);
    Py_XINCREF(arg);
    cbStruct->callback = pyfunc;
    cbStruct->arg = arg;
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gass_transfer_register_get(request, attr, url, PythonGassTransferCallback, cbStruct);
    Py_END_ALLOW_THREADS
    list = PyList_New(0);
    if (ret != GLOBUS_SUCCESS){
	 PyList_Append(list, Py_BuildValue("i", ret));
	 PyList_Append(list, Py_BuildValue("s", "Error with the register get call"));
	 Py_INCREF(Py_None);
	 PyList_Append(list, Py_None);
	 globus_libc_free(cbStruct);
      }
    else{
	ptr = SWIG_NewPointerObj((void*)cbStruct, SWIGTYPE_p_callbackStruct,0);
	ptr2 = SWIG_NewPointerObj((void*)request, SWIGTYPE_p_globus_gass_transfer_request_t,0);
	PyList_Append(list, Py_BuildValue("i", ret));
	PyList_Append(list, ptr);
	PyList_Append(list, ptr2);
    }
    return list;
} 

PyObject *
get(globus_gass_transfer_requestattr_t *attr, char*  url){
    int ret;
    PyObject *tuple, *ptr;
    globus_gass_transfer_request_t * request;
    request = (globus_gass_transfer_request_t *)globus_malloc(sizeof(globus_gass_transfer_request_t));
    if (!(tuple=PyTuple_New(2))){
	PyErr_SetString(PyExc_RuntimeError, 
			"get:PyTuple_New()");
	return NULL;
    }
    
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gass_transfer_get(request, attr, url);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS){
	PyTuple_SET_ITEM(tuple,0, Py_BuildValue("i", ret));
	PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", "Error with the get call"));
    }
    else{
	PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i",0));
	ptr = SWIG_NewPointerObj((void *)request, SWIGTYPE_p_globus_gass_transfer_request_t,0);
	PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

PyObject *
register_put(globus_gass_transfer_requestattr_t *attr,int length,
	     char*  url, PyObject * pyfunc, PyObject* arg){
      int ret;
      PyObject * list, *ptr, *ptr2;
      callbackStruct * cbStruct;
      globus_gass_transfer_request_t * request;
      
      request = (globus_gass_transfer_request_t *)globus_malloc(sizeof(globus_gass_transfer_request_t));
      cbStruct = (callbackStruct *)globus_malloc(sizeof(callbackStruct));
      Py_XINCREF(pyfunc);
      Py_XINCREF(arg);
      cbStruct->callback = pyfunc;
      cbStruct->arg = arg;
       
      
      Py_BEGIN_ALLOW_THREADS
      ret = globus_gass_transfer_register_put(request, attr, url, length,
					      PythonGassTransferCallback, 
					      cbStruct);
      Py_END_ALLOW_THREADS
	  
      list = PyList_New(0);
      if (ret != GLOBUS_SUCCESS) {
	  PyList_Append(list, Py_BuildValue("i", ret));
	  PyList_Append(list, Py_BuildValue("s", "Error with the register put call"));
	  Py_INCREF(Py_None);
	  PyList_Append(list, Py_None);
	  globus_libc_free(cbStruct);
      }
      else {
	  ptr = SWIG_NewPointerObj((void*)cbStruct, SWIGTYPE_p_callbackStruct,0);
	  ptr2 = SWIG_NewPointerObj((void*)request, SWIGTYPE_p_globus_gass_transfer_request_t,0);
	  PyList_Append(list, Py_BuildValue("i", ret));
	  PyList_Append(list, ptr);
	  PyList_Append(list, ptr2);
      }
    
      return list;
}

PyObject *
put(globus_gass_transfer_requestattr_t *attr, char*  url, int length){
      int ret;
      PyObject *tuple, *ptr;
      globus_gass_transfer_request_t *request;
      request = (globus_gass_transfer_request_t *)globus_malloc(sizeof(globus_gass_transfer_request_t));
    
      if (!(tuple=PyTuple_New(2))){
	 PyErr_SetString(PyExc_RuntimeError, 
			 "put:PyTuple_New()");
	 return NULL;
      }

      Py_BEGIN_ALLOW_THREADS
      ret = globus_gass_transfer_put(request, attr, url,length);
      Py_END_ALLOW_THREADS
      if (ret != GLOBUS_SUCCESS){
	  PyTuple_SET_ITEM(tuple,0, Py_BuildValue("i", ret));
	  PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", "Error with the put call"));
      }
      else{
	  PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i",0));
	  ptr = SWIG_NewPointerObj((void *)request, SWIGTYPE_p_globus_gass_transfer_request_t,0);
	  PyTuple_SET_ITEM(tuple, 1, ptr);
      }
      return tuple;
}

PyObject *
register_append( globus_gass_transfer_requestattr_t *attr,
	     int length, char*  url, PyObject * pyfunc, PyObject* arg){
    int ret;
    PyObject * list, *ptr, *ptr2;
    callbackStruct * cbStruct;
    globus_gass_transfer_request_t * request;
    request = (globus_gass_transfer_request_t *)globus_malloc(sizeof(globus_gass_transfer_request_t));
    cbStruct = (callbackStruct *)globus_malloc(sizeof(callbackStruct));
    Py_XINCREF(pyfunc);
    Py_XINCREF(arg);
    cbStruct->callback = pyfunc;
    cbStruct->arg = arg;
    
    
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gass_transfer_register_append(request, attr, url,length,
					       PythonGassTransferCallback, 
					       cbStruct);
    Py_END_ALLOW_THREADS
      
      
    list = PyList_New(0); 
    if (ret != GLOBUS_SUCCESS) {
	PyList_Append(list, Py_BuildValue("i", ret));
	PyList_Append(list, Py_BuildValue("s", "Error with the register append call"));
	Py_INCREF(Py_None);
	PyList_Append(list, Py_None);
	globus_libc_free(cbStruct);
    }
    else{
	ptr = SWIG_NewPointerObj((void*)cbStruct, SWIGTYPE_p_callbackStruct,0);
	ptr2 = SWIG_NewPointerObj((void*)request, SWIGTYPE_p_globus_gass_transfer_request_t,0);
	PyList_Append(list, Py_BuildValue("i", ret));
	PyList_Append(list, ptr);
	PyList_Append(list, ptr2);
      }
    
       return list;
}

PyObject *
append( globus_gass_transfer_requestattr_t *attr, char*  url, int length){
    int ret;
    PyObject *tuple, *ptr;
    globus_gass_transfer_request_t *request;
    request = (globus_gass_transfer_request_t *)globus_malloc(sizeof(globus_gass_transfer_request_t));
    if (!(tuple=PyTuple_New(2))){
	PyErr_SetString(PyExc_RuntimeError, 
			"append:PyTuple_New()");
	return NULL;
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gass_transfer_append(request, attr, url,length);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS){
	PyTuple_SET_ITEM(tuple,0, Py_BuildValue("i", ret));
	PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", "Error with the append call"));
    }
    else{
	PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i",0));
	ptr = SWIG_NewPointerObj((void *)request, SWIGTYPE_p_globus_gass_transfer_request_t,0);
	PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

PyObject *
create_listener(globus_gass_transfer_listenerattr_t *attr, char *scheme){
    int ret;
    PyObject *tuple, *ptr;
    globus_gass_transfer_listener_t *listener;
    listener = (globus_gass_transfer_listener_t *)globus_malloc(sizeof(globus_gass_transfer_listener_t));
    if (!(tuple=PyTuple_New(2))){
	PyErr_SetString(PyExc_RuntimeError, 
			"create_listener:PyTuple_New()");
	return NULL;
    }

    Py_BEGIN_ALLOW_THREADS
    ret = globus_gass_transfer_create_listener(listener, attr,scheme);
    Py_END_ALLOW_THREADS
    if(ret != GLOBUS_SUCCESS){
	 PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
	 PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", "Error with the create listener call"));
	 globus_libc_free(listener);
    }
    else
    {
	PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
	ptr = SWIG_NewPointerObj((void *)listener, SWIGTYPE_p_globus_gass_transfer_listener_t, 0);
	PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;	  
}

int 
listener_destroy(globus_gass_transfer_listener_t listener){
	/*Calling an internal globus function, this function changes the state of the 
	globus_handle_table so I think it is a better option than just doing a globus_free 
	MKR */
	int ret;
	ret = globus_i_gass_transfer_listener_destroy(listener);
	return ret;	
}

PyObject *
close_listener(globus_gass_transfer_listener_t * listener, 
             PyObject * pyfunc, PyObject * arg){ 
    int ret;
    PyObject *list, *ptr, *ptr2;
    callbackStruct *cbStruct;
    cbStruct = (callbackStruct *)globus_malloc(sizeof(callbackStruct));
    Py_INCREF(pyfunc);
    Py_INCREF(arg);
    cbStruct->callback = pyfunc;
    cbStruct->arg = arg;
    
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gass_transfer_close_listener(*listener, PythonGassTransferCloseCallback, cbStruct);
    Py_END_ALLOW_THREADS
	
    list = PyList_New(0); 
    if (ret != GLOBUS_SUCCESS){
	PyList_Append(list, Py_BuildValue("i", ret));
	PyList_Append(list, Py_BuildValue("s", "Error with the close listener call"));
	Py_INCREF(Py_None);
	PyList_Append(list, Py_None);
	globus_libc_free(cbStruct);
    }
    else {
	ptr = SWIG_NewPointerObj((void*)cbStruct, SWIGTYPE_p_callbackStruct,0);
	ptr2 = SWIG_NewPointerObj((void*)listener, SWIGTYPE_p_globus_gass_transfer_listener_t,0);
	PyList_Append(list, Py_BuildValue("i", ret));
	PyList_Append(list, ptr);
	PyList_Append(list, ptr2);
    }
    return list;
}

PyObject *
register_listen(globus_gass_transfer_listener_t *listener, PyObject * pyfunc, PyObject * arg){
    int ret;
    PyObject *list, *ptr, *ptr2;
    callbackStruct *cbStruct;
    cbStruct = (callbackStruct *)globus_malloc(sizeof(callbackStruct));
    Py_INCREF(pyfunc);
    Py_INCREF(arg);
    cbStruct->callback = pyfunc;
    cbStruct->arg = arg;
  
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gass_transfer_register_listen(*listener, PythonGassTransferCloseCallback, cbStruct);
    Py_END_ALLOW_THREADS
    list = PyList_New(0);
    if (ret != GLOBUS_SUCCESS) {
	PyList_Append(list, Py_BuildValue("i", ret));
	PyList_Append(list, Py_BuildValue("s", "Error with the register listen call"));
	Py_INCREF(Py_None);
	PyList_Append(list, Py_None);
	globus_libc_free(cbStruct);
    }
    else {
	ptr = SWIG_NewPointerObj((void*)cbStruct, SWIGTYPE_p_callbackStruct,0);
	ptr2 = SWIG_NewPointerObj((void*)listener, SWIGTYPE_p_globus_gass_transfer_listener_t,0);
	PyList_Append(list, Py_BuildValue("i", ret));
	PyList_Append(list, ptr);
	PyList_Append(list, ptr2);
    }
    return list;
}

PyObject * 
register_accept(  globus_gass_transfer_requestattr_t * attr, 
                 globus_gass_transfer_listener_t listener, 
		 PyObject * pyfunc, PyObject * arg){
    int ret;
    PyObject *list, *ptr, *ptr2;
    callbackStruct *cbStruct;
    globus_gass_transfer_request_t * request; 
    cbStruct = (callbackStruct *)globus_malloc(sizeof(callbackStruct));
    request = (globus_gass_transfer_request_t *)globus_malloc(sizeof(globus_gass_transfer_request_t));
    Py_INCREF(pyfunc);
    Py_INCREF(arg);
    cbStruct->callback = pyfunc;
    cbStruct->arg = arg;
    
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gass_transfer_register_accept(request,attr,listener, PythonGassTransferCallback, cbStruct);
    Py_END_ALLOW_THREADS                               
    list = PyList_New(0);                    
    if (ret != GLOBUS_SUCCESS) {
	PyList_Append(list, Py_BuildValue("i", ret));
	PyList_Append(list, Py_BuildValue("s", "Error with the register accept call"));
	Py_INCREF(Py_None);
	PyList_Append(list, Py_None);
	globus_libc_free(cbStruct);
    }
    else {
	ptr = SWIG_NewPointerObj((void*)cbStruct, SWIGTYPE_p_callbackStruct,0);
	ptr2 = SWIG_NewPointerObj((void*)request, SWIGTYPE_p_globus_gass_transfer_request_t,0);
	PyList_Append(list, Py_BuildValue("i", ret));
	PyList_Append(list, ptr);
	PyList_Append(list, ptr2);
    }
        
    return list;
}          

PyObject *
get_user_pointer(globus_gass_transfer_listener_t listener){
    void * ret;
    PyObject * tuple, * ptr;
    if (!(tuple=PyTuple_New(2))){
	PyErr_SetString(PyExc_RuntimeError, 
			"get_user_pointer:PyTuple_New()");
	return NULL;
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gass_transfer_listener_get_user_pointer(listener);
    Py_END_ALLOW_THREADS
    if(ret == GLOBUS_NULL){
	PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
	PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", "Error with the get user pointer call"));
	}
    else{
	PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
	ptr = SWIG_NewPointerObj((void *)ret, SWIGTYPE_p_globus_gass_transfer_listener_t, 0);
	PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

PyObject * 
set_user_pointer(globus_gass_transfer_listener_t listener,void * user_pointer){
    int ret;
    PyObject * tuple, *ptr;
    if (!(tuple=PyTuple_New(2))){
	PyErr_SetString(PyExc_RuntimeError, 
			"get_user_pointer:PyTuple_New()");
	return NULL;
    }
    
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gass_transfer_listener_set_user_pointer(listener, user_pointer);
    Py_END_ALLOW_THREADS
    if(ret != GLOBUS_SUCCESS){
	PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
	PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", "Error with the set user pointer call"));
    }
    else{
	PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
	ptr = SWIG_NewPointerObj((void *)user_pointer, SWIGTYPE_p_globus_gass_transfer_listener_t, 0);
	PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

char *
get_base_url(globus_gass_transfer_listener_t *listener){
    char * ret;
    ret = globus_gass_transfer_listener_get_base_url(*listener);
    return ret;
}

char *
get_url(globus_gass_transfer_request_t * request){
    char * ret;
    ret = globus_gass_transfer_request_get_url(*request);
    return ret;
}

PyObject *
refer(globus_gass_transfer_request_t * request, char ** urls, globus_size_t num_urls){
    int ret;
    PyObject * tuple, *ptr;
    if (!(tuple=PyTuple_New(2))){
	PyErr_SetString(PyExc_RuntimeError, 
			"refer:PyTuple_New()");
	return NULL;
    }	
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gass_transfer_refer(*request, urls, num_urls);
    Py_END_ALLOW_THREADS
    if(ret != GLOBUS_SUCCESS){
	PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
	PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", "Error with the refer call"));		 
    }
    else{
	PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
	ptr = SWIG_NewPointerObj((void *)request, SWIGTYPE_p_globus_gass_transfer_request_t, 0);
	PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;	
}

PyObject *
authorize(globus_gass_transfer_request_t *request, int total_length){
    int ret;
    PyObject * tuple, *ptr;
    
    if (!(tuple=PyTuple_New(2))){
	PyErr_SetString(PyExc_RuntimeError, 
			"authorize:PyTuple_New()");
	return NULL;
    }	
    
    
    Py_BEGIN_ALLOW_THREADS    
    ret = globus_gass_transfer_authorize(*request, total_length);
    Py_END_ALLOW_THREADS
	
    if(ret != GLOBUS_SUCCESS){
	PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
	PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", "Error with the authorize call"));	
    }
    else{
	PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
	ptr = SWIG_NewPointerObj((void *)request, SWIGTYPE_p_globus_gass_transfer_request_t, 0);
	PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;	
}	

PyObject *
deny(globus_gass_transfer_request_t *request, int reason, char * message){ 
    int ret;
    PyObject * tuple, *ptr;
    
    if (!(tuple=PyTuple_New(2))){
	PyErr_SetString(PyExc_RuntimeError, 
			"deny:PyTuple_New()");
	return NULL;
    }	
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gass_transfer_deny(*request, reason, message);
    Py_END_ALLOW_THREADS
    printf("ret is %d\n", ret);
    if(ret != GLOBUS_SUCCESS){
	PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
	PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", "Error with the deny call"));		
    }
    else{
	PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
	ptr = SWIG_NewPointerObj((void *)request, SWIGTYPE_p_globus_gass_transfer_request_t, 0);
	PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;	
}

PyObject *
send_bytes(globus_gass_transfer_request_t request,PyObject * buffer, 
	   int send_length, globus_bool_t last_data,
	   PyObject * pyfunc, PyObject *arg){
    
    int ret;
    PyObject *list, *ptr;
    callbackStruct *cbStruct;
    globus_byte_t * buf;
    int sizeNew;
    cbStruct = (callbackStruct *)globus_malloc(sizeof(callbackStruct));
    Py_INCREF(pyfunc);
    Py_INCREF(arg);
    cbStruct->callback = pyfunc;
    cbStruct->arg = arg;
	
    ret = PyString_AsStringAndSize(buffer, (char**)&buf, &sizeNew);
    
    
    Py_BEGIN_ALLOW_THREADS
   ret = globus_gass_transfer_send_bytes(request,buf,sizeNew,
					 last_data,
					 PythonGassTransferBytesCallback,
					 cbStruct);
    Py_END_ALLOW_THREADS                               
    list = PyList_New(0);                    
    if (ret != GLOBUS_SUCCESS) {
	PyList_Append(list, Py_BuildValue("i", ret));
	PyList_Append(list, Py_BuildValue("s", "Error with the send bytes call"));
	globus_libc_free(cbStruct);
    }
    else {
	ptr = SWIG_NewPointerObj((void*)cbStruct, SWIGTYPE_p_callbackStruct,0);
	PyList_Append(list, Py_BuildValue("i", ret));
	PyList_Append(list, ptr);
	
    }
    return list;
}         
 
PyObject *
receive_bytes(globus_gass_transfer_request_t *request,globus_byte_t * bytes, 
	       int max_length, int wait_for_length, 
	       PyObject * pyfunc, PyObject *arg){
    int ret;
    PyObject *list, *ptr;
    callbackStruct *cbStruct;
    cbStruct = (callbackStruct *)globus_malloc(sizeof(callbackStruct));
    Py_INCREF(pyfunc);
    Py_INCREF(arg);
    cbStruct->callback = pyfunc;
    cbStruct->arg = arg;

    Py_BEGIN_ALLOW_THREADS
    ret = globus_gass_transfer_receive_bytes(*request,bytes,max_length,wait_for_length,
						     PythonGassTransferBytesCallback, cbStruct);
    Py_END_ALLOW_THREADS                               
    list = PyList_New(0);                    
    if (ret != GLOBUS_SUCCESS) {
	PyList_Append(list, Py_BuildValue("i", ret));
	PyList_Append(list, Py_BuildValue("s", "Error with the receive bytes call"));
	globus_libc_free(cbStruct);
    }
    else {
	ptr = SWIG_NewPointerObj((void*)cbStruct, SWIGTYPE_p_callbackStruct,0);
	PyList_Append(list, Py_BuildValue("i", ret));
	PyList_Append(list, ptr);
    }
    return list;
}          

int
get_request_type(globus_gass_transfer_request_t *request){
    globus_gass_transfer_request_type_t type;
    type = globus_gass_transfer_request_get_type(*request);
    return (int)type;
}

int 
get_request_status(globus_gass_transfer_request_t * request){
    int ret;
    ret = globus_gass_transfer_request_get_status(*request);
    return ret;
}  

int
set_subject(globus_gass_transfer_request_t * request, char * subject){
    int ret;
    ret = globus_gass_transfer_request_set_subject(*request, subject);
    return ret;
}

char *
get_subject(globus_gass_transfer_request_t * request){
    char * ret;
    ret = globus_gass_transfer_request_get_subject(*request);
    return ret;
}

int 
set_length(globus_gass_transfer_request_t *request, int length){
    globus_gass_transfer_request_set_length(*request, length);
   
}

int
get_length(globus_gass_transfer_request_t *request){
    int ret;
    ret = globus_gass_transfer_request_get_length(*request);
    return ret;
}

int
get_denial_reason(globus_gass_transfer_request_t *request){
    int ret;
    ret = globus_gass_transfer_request_get_denial_reason(*request);
    return ret;
}

char *
get_denial_message(globus_gass_transfer_request_t *request){
    char * ret;
    ret = globus_gass_transfer_request_get_denial_message(*request);
    return ret;
}

int
set_proxy_url(globus_gass_transfer_requestattr_t *attr, char *proxy_url){
    int ret;
    ret = globus_gass_transfer_requestattr_set_proxy_url(attr, proxy_url);
    return ret;
}

int 
get_proxy_url(globus_gass_transfer_requestattr_t *attr, char **proxy_url){
    int ret;
    ret = globus_gass_transfer_requestattr_get_proxy_url(attr, proxy_url);
    return ret;
}

int
set_block_size(globus_gass_transfer_requestattr_t* attr, globus_size_t blocksize){
    int ret;
    ret = globus_gass_transfer_requestattr_set_block_size(attr,blocksize);
    return ret;
}

int
get_block_size(globus_gass_transfer_requestattr_t* attr, globus_size_t* blocksize){
    int ret;
    ret = globus_gass_transfer_requestattr_get_block_size(attr,blocksize);
    return ret;
}

int
set_file_mode(globus_gass_transfer_requestattr_t* attr, globus_gass_transfer_file_mode_t file_mode){
   int ret;
   ret = globus_gass_transfer_requestattr_set_file_mode(attr, file_mode);
   return ret;
}

int
get_file_mode(globus_gass_transfer_requestattr_t* attr, globus_gass_transfer_file_mode_t *file_mode){
   int ret;
   ret = globus_gass_transfer_requestattr_get_file_mode(attr, file_mode);
   return ret;
}

int
set_connection_reuse(globus_gass_transfer_requestattr_t* attr, globus_bool_t connect_reuse){
   int ret;
   ret = globus_gass_transfer_requestattr_set_connection_reuse(attr, connect_reuse); 
   return ret;
}

int 
get_connection_reuse(globus_gass_transfer_requestattr_t* attr, globus_bool_t* connect_reuse){
   int ret;
   ret = globus_gass_transfer_requestattr_get_connection_reuse(attr, connect_reuse);
   return ret;
}

int 
set_socket_sndbuf(globus_gass_transfer_requestattr_t* attr, int sndbuf){
   int ret;
   ret = globus_gass_transfer_requestattr_set_socket_sndbuf(attr, sndbuf);
   return ret;
}

int 
get_socket_sndbuf(globus_gass_transfer_requestattr_t* attr, int *sndbuf){
   int ret;
   ret = globus_gass_transfer_requestattr_get_socket_sndbuf(attr, sndbuf);
   return ret;
}

int
set_socket_rcvbuf(globus_gass_transfer_requestattr_t* attr, int rcvbuf){
   int ret;
   ret = globus_gass_transfer_requestattr_set_socket_rcvbuf(attr, rcvbuf);
   return ret;
}

int
get_socket_rcvbuf(globus_gass_transfer_requestattr_t* attr, int *rcvbuf){
   int ret;
   ret = globus_gass_transfer_requestattr_get_socket_rcvbuf(attr, rcvbuf);
   return ret;
}

int
set_socket_nodelay(globus_gass_transfer_requestattr_t* attr,globus_bool_t nodelay){
   int ret;
   ret = globus_gass_transfer_requestattr_set_socket_nodelay(attr, nodelay);
   return ret;
}

int
get_socket_nodelay(globus_gass_transfer_requestattr_t* attr,globus_bool_t *nodelay){
   int ret;
   ret = globus_gass_transfer_requestattr_get_socket_nodelay(attr, nodelay);
   return ret;
}

int
set_authorization(globus_gass_transfer_requestattr_t* attr, 
		  globus_gass_transfer_authorization_t mode, char * subject){
   int ret;
   ret = globus_gass_transfer_secure_requestattr_set_authorization(attr, mode, subject);
   return ret;
}

int
get_authorization(globus_gass_transfer_requestattr_t* attr, 
		  globus_gass_transfer_authorization_t *mode, char ** subject){
   int ret;
   ret = globus_gass_transfer_secure_requestattr_get_authorization(attr, mode, subject);
   return ret;
}


%}
