/*############################################################################
# Keith R. Jackson, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################*/
%module gramClientc
%{
#undef SIZEOF_OFF_T
#undef HAVE_LIMITS_H
#undef socklen_t
#include "globus_gram_client.h"
#include "pyGlobus_common.h"
%}

%include ./src/pyGlobus-typemaps.i

%apply char **OUTPUT { const char **callback_contact,
			   char **callback_contact };

%apply int *OUTPUT { int* job_status, int *failure_code };

%init %{
    PyEval_InitThreads();
    python_interp = PyThreadState_Get()->interp;
    init_globus_thread_state_key(&python_thread_state_key);
%}

%{

static globus_thread_key_t python_thread_state_key;
static PyInterpreterState *python_interp; 

PyObject * check_error(globus_gram_protocol_error_t error){
    PyObject *tuple, *err;
    char *tmpstr;
    int tmp;

    if(!(tuple = PyTuple_New(2))){
    	PyErr_SetString(PyExc_RuntimeError,"PythonCallback, PyTuple_New");	
    }
    else{ 
	if (error){  
	    tmpstr =  (char *) globus_gram_client_error_string(error); 
	    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", error));
            PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));	    
        }
	else{
	    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
            Py_INCREF(Py_None);
            PyTuple_SET_ITEM(tuple, 1, Py_None);
	}
    }	
    
    if ((err =  PyErr_Occurred()) != NULL){
	PyErr_Print();
    }
    return tuple;
}

PyObject * check_errori(int error){
    PyObject *tuple;
    PyObject* err;
    char *tmpstr;
    int tmp;

    if(!(tuple = PyTuple_New(2))){
    	PyErr_SetString(PyExc_RuntimeError,"PythonCallback, PyTuple_New");	
    }
    else{ 
	if (error){  
	    tmpstr =  (char *) globus_gram_client_error_string(error); 
	    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", error));
            PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));	    
        }
	else{
	    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
            Py_INCREF(Py_None);
            PyTuple_SET_ITEM(tuple, 1, Py_None);
	}
    }	
    
    if ((err = PyErr_Occurred()) != NULL){
	PyErr_Print();
    }
    return tuple;
}



static void PythonCallback(void* user_data, char* job_contact,
                           int state, int errorcode) {
    PyObject *func, *arglist, *result, *arg;
    PyObject *error_object, *tuple, *err;
    callbackStruct* cbStruct = (callbackStruct*)user_data;
    PyEval_AcquireThread(get_python_thread_state(python_thread_state_key, python_interp));
    func = cbStruct->callback;
    arg = cbStruct->arg;

    error_object = check_errori(errorcode);
    arglist = Py_BuildValue("(OsiO)", arg, job_contact, state, error_object);
    result = PyEval_CallObject(func, arglist);
    if (result == NULL){
       PyErr_Print();
    }
    Py_DECREF(arglist);
    Py_XDECREF(result);
    Py_DECREF(error_object);
    PyEval_ReleaseThread(get_python_thread_state(python_thread_state_key, python_interp));
    return; 
}


static void
NonBlockingPythonCallback(void *user_data, 
                globus_gram_protocol_error_t failure_code, 
                const char *job_contact,
                globus_gram_protocol_job_state_t job_state,
                globus_gram_protocol_error_t job_failure_code) {

    PyObject *func, *arglist, *result, *arg;
    PyObject *errorObject1, *errorObject2, *tuple, *err;
    callbackStruct *cbStruct = (callbackStruct*)user_data;
    assert(cbStruct);
    PyEval_AcquireThread(get_python_thread_state(python_thread_state_key, python_interp));
    func = cbStruct->callback; 
    arg = cbStruct->arg;
    errorObject1 = check_error(job_failure_code);
    errorObject2 = check_error(failure_code);
    arglist = Py_BuildValue("(OsiOO)", arg, job_contact, (int)job_state, errorObject1, errorObject2);
    result = PyEval_CallObject(func, arglist);
    if (result == NULL){
       PyErr_Print();
    }
    
    Py_DECREF(arglist);
    Py_XDECREF(result);
    Py_DECREF(errorObject1);
    Py_DECREF(errorObject2); 
    PyEval_ReleaseThread(get_python_thread_state(python_thread_state_key, python_interp));
    return;
}
%}

%inline %{
globus_module_descriptor_t* get_module(void) {
    return GLOBUS_GRAM_CLIENT_MODULE;
}

PyObject*
callback_allow(PyObject *pyfunc, PyObject *arg, char ** callback_contact) {
    int ret;
    PyObject *list, *cbPtr;
    callbackStruct* cbStruct;
    cbStruct = (callbackStruct*)globus_malloc(sizeof(callbackStruct));
    Py_XINCREF(pyfunc);
    Py_XINCREF(arg);
    cbStruct->callback = pyfunc;
    cbStruct->arg = arg;
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gram_client_callback_allow(PythonCallback, cbStruct,
					    callback_contact);
    Py_END_ALLOW_THREADS
    cbPtr = SWIG_NewPointerObj((void*)cbStruct, SWIGTYPE_p_callbackStruct, 0);
    list = PyList_New(0);
    PyList_Append(list, Py_BuildValue("i", ret));
    PyList_Append(list, cbPtr);
    return list; 
}
 
PyObject *
ping(char * resource_manager_contact) {
    int ret;
    PyObject* tuple;
    const char * tmpstr;
    
    if (!(tuple=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, "ping: PyTuple_New()");
        return NULL;    
    }

    Py_BEGIN_ALLOW_THREADS
    ret = globus_gram_client_ping(resource_manager_contact);
    Py_END_ALLOW_THREADS
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS) {
        tmpstr = globus_gram_client_error_string(ret);
	PyTuple_SET_ITEM(tuple,1,Py_BuildValue("s", tmpstr));
    }
    else {
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 1, Py_None); 
    }
    return tuple;
}

PyObject*
job_request(char * resource_manager_contact, const char * description,
            const int job_state_mask, const char * callback_contact) {
    int ret;
    PyObject* tuple;
    char* job_contact;
    char * tmpstr;
    if (!(tuple=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, "job_request: PyTuple_New()");
        return NULL;
    }
    ret = globus_gram_client_job_request(resource_manager_contact,
                                         description, job_state_mask,
                                         callback_contact, &job_contact);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple,1,Py_BuildValue("s", "An error occurred in job_request."));
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 1, Py_None); 
    }
    else {
	PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", job_contact));
    }
    return tuple;
}

PyObject *
job_status(const char* job_contact) {
    int ret;
    PyObject* tuple;
    const char * tmpstr;
    int job_status, failure_code;

     if (!(tuple=PyTuple_New(3))) {
        PyErr_SetString(PyExc_RuntimeError, "job_status: PyTuple_New()");
        return NULL;
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gram_client_job_status(job_contact, &job_status, &failure_code);
    Py_END_ALLOW_THREADS
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS) {
        tmpstr = globus_gram_client_error_string(ret);
        PyTuple_SET_ITEM(tuple,1,Py_BuildValue("s", tmpstr));
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 2, Py_None);	
    }
    else {
	PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("i", job_status));
	PyTuple_SET_ITEM(tuple, 2, Py_BuildValue("i", failure_code));
	
    }
    return tuple;
}

PyObject *
job_cancel(char* job_contact) {
    int ret;
    PyObject* tuple;
    const char * tmpstr;
    if (!(tuple=PyTuple_New(2)))    {
        PyErr_SetString(PyExc_RuntimeError, "job_cancel: PyTuple_New()");
        return NULL;
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gram_client_job_cancel(job_contact);
    Py_END_ALLOW_THREADS
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS) {
        tmpstr = globus_gram_client_error_string(ret);
        PyTuple_SET_ITEM(tuple,1,Py_BuildValue("s", tmpstr));
    }
    else {
	PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", job_contact));
    }
    return tuple;
}



PyObject *
job_refresh_credentials(char *job_contact, gss_cred_id_t *cred) {
    int ret;
    PyObject* tuple;
    const char * tmpstr;
    if (!(tuple=PyTuple_New(2)))    {
        PyErr_SetString(PyExc_RuntimeError, "job_refresh_credentials: PyTuple_New()");
        return NULL;
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gram_client_job_refresh_credentials(job_contact, *cred);
    Py_END_ALLOW_THREADS
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));

    if (ret != GLOBUS_SUCCESS) {
        tmpstr = globus_gram_client_error_string(ret);
        PyTuple_SET_ITEM(tuple,1,Py_BuildValue("s", tmpstr));
    }
    else {
	PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", job_contact));
    }
    return tuple;
}



PyObject*
register_job_refresh_credentials(char* job_contact, gss_cred_id_t *cred, 
                  globus_gram_client_attr_t *gattr, PyObject *pyfunc,
                  PyObject *arg) {
    int ret;
    PyObject *list, *cbPtr;
    callbackStruct* cbStruct;
    cbStruct = (callbackStruct*)globus_malloc(sizeof(callbackStruct));
    Py_XINCREF(pyfunc);
    Py_XINCREF(arg);
    cbStruct->callback = pyfunc;
    cbStruct->arg = arg;
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gram_client_register_job_refresh_credentials(job_contact, 
                                        *cred, *gattr, NonBlockingPythonCallback, cbStruct);
    Py_END_ALLOW_THREADS
    cbPtr = SWIG_NewPointerObj((void*)cbStruct, SWIGTYPE_p_callbackStruct, 0);
    list = PyList_New(0);
    PyList_Append(list, Py_BuildValue("i", ret));
    PyList_Append(list, cbPtr);
    return list; 
}

PyObject*
register_job_request(const char* rm_contact, const char *description,
                                        int job_state_mask, 
                                        const char *callback_contact,
                                        globus_gram_client_attr_t *gattr,
                                        PyObject *pyfunc, PyObject *arg) { 
    int ret;
    PyObject *list, *cbPtr;
    callbackStruct* cbStruct;
    cbStruct = (callbackStruct*)globus_malloc(sizeof(callbackStruct));
    Py_XINCREF(pyfunc);
    Py_XINCREF(arg);
    cbStruct->callback = pyfunc; 
    cbStruct->arg = arg;   
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gram_client_register_job_request(rm_contact, description,
                                        job_state_mask, callback_contact,
                                        *gattr, NonBlockingPythonCallback, cbStruct);
    Py_END_ALLOW_THREADS
    cbPtr = SWIG_NewPointerObj((void*)cbStruct, SWIGTYPE_p_callbackStruct, 0);
    list = PyList_New(0);
    PyList_Append(list, Py_BuildValue("i", ret));
    PyList_Append(list, cbPtr);
    return list;
}

PyObject*
register_callback_registration(const char* job_contact, int job_state_mask, 
                                        const char *callback_contact,
                                        globus_gram_client_attr_t *gattr,
                                        PyObject *pyfunc, PyObject *arg) { 
    int ret;
    PyObject *list, *cbPtr;
    callbackStruct* cbStruct;
    cbStruct = (callbackStruct*)globus_malloc(sizeof(callbackStruct));
    Py_XINCREF(pyfunc);
    Py_XINCREF(arg);
    cbStruct->callback = pyfunc; 
    cbStruct->arg = arg;   
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gram_client_register_job_callback_registration(job_contact, 
                                        job_state_mask, callback_contact,
                                        *gattr, NonBlockingPythonCallback, cbStruct);
    Py_END_ALLOW_THREADS
    cbPtr = SWIG_NewPointerObj((void*)cbStruct, SWIGTYPE_p_callbackStruct, 0);
    list = PyList_New(0);
    PyList_Append(list, Py_BuildValue("i", ret));
    PyList_Append(list, cbPtr);
    return list;
}

void
free_nb_callback(callbackStruct *cbStruct) {
    int ret;
    Py_XDECREF(cbStruct->callback);
    Py_XDECREF(cbStruct->arg);
    globus_free(cbStruct);
    return;
}

int
remove_callback(char* callback_contact, callbackStruct* cbStruct) {
    int ret;
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gram_client_callback_disallow(callback_contact);
    Py_END_ALLOW_THREADS
    Py_XDECREF(cbStruct->callback);
    Py_XDECREF(cbStruct->arg);
    globus_free(cbStruct);
    return ret;
}

PyObject*
callback_register(char* job_contact, const int job_state_mask, 
                  const char* callback_contact) {
    int ret;
    int failure_code, job_status;
    PyObject *tuple, *ptr;
    const char *tmpstr;
    gss_cred_id_t *cred;

    if (!(tuple=PyTuple_New(2))) {               
        PyErr_SetString(PyExc_RuntimeError, "callback_register: PyTuple_New()");
        return NULL;
    }	
    Py_BEGIN_ALLOW_THREADS

    ret = globus_gram_client_job_callback_register(job_contact, job_state_mask,
                                                   callback_contact, &job_status,
                                                   &failure_code);

    Py_END_ALLOW_THREADS

    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));


    if (ret != GLOBUS_SUCCESS) {
        tmpstr = globus_gram_client_error_string(ret);
	PyTuple_SET_ITEM(tuple,1,Py_BuildValue("s", tmpstr));
    }
    else {
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None); 
    } 
    return tuple;
}

PyObject*
callback_unregister(char* job_contact, const char* callback_contact) {
    int ret;
    int failure_code, job_status;
    PyObject *tuple, *ptr;
    const char *tmpstr;
    gss_cred_id_t *cred;

    if (!(tuple=PyTuple_New(2))) {               
        PyErr_SetString(PyExc_RuntimeError, "callback_unregister: PyTuple_New()");
        return NULL;
    }

    Py_BEGIN_ALLOW_THREADS
    ret = globus_gram_client_job_callback_unregister(job_contact, callback_contact,
                                                     &job_status, &failure_code);
    Py_END_ALLOW_THREADS

    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));

    if (ret != GLOBUS_SUCCESS) {
        tmpstr = globus_gram_client_error_string(ret);
	PyTuple_SET_ITEM(tuple,1,Py_BuildValue("s", tmpstr));
    }
    else {
       PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("i",job_status)); 
    } 
    return tuple;
}


PyObject*
client_attr_init() {
    PyObject *tuple, *ptr;
    int ret;
    globus_gram_client_attr_t *attr;
    const char * tmpstr;

    if (!(tuple=PyTuple_New(2))) {               
        PyErr_SetString(PyExc_RuntimeError, "client_attr_init: PyTuple_New()");
        return NULL;
    }
    attr = (globus_gram_client_attr_t*)globus_malloc(sizeof(globus_gram_client_attr_t));
    ret = globus_gram_client_attr_init(attr);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS) {
	tmpstr = globus_gram_client_error_string(ret);
        PyTuple_SET_ITEM(tuple,1,Py_BuildValue("s", tmpstr));
        /* Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None); */
    }
    else {
        ptr = SWIG_NewPointerObj((void*)attr, SWIGTYPE_p_globus_gram_client_attr_t, 0);
        PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

int
client_attr_destroy(globus_gram_client_attr_t *attr) {
    int ret;
    ret = globus_gram_client_attr_destroy(attr);
    globus_free(attr);
    return ret;
}

PyObject *
client_attr_set_credential(globus_gram_client_attr_t *attr, gss_cred_id_t *cred) {
    int ret;
    PyObject *tuple;
    const char * tmpstr;
    if (!(tuple=PyTuple_New(2))) {
        PyErr_SetString(PyExc_RuntimeError, "client_attr_set_credential: PyTuple_New()");
        return NULL;
    }
    
    ret = globus_gram_client_attr_set_credential(*attr, *cred);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS) {
        tmpstr = globus_gram_client_error_string(ret);
	PyTuple_SET_ITEM(tuple,1,Py_BuildValue("s", tmpstr));
    }
    else{
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    
    return tuple;
}

PyObject*
client_attr_get_credential(globus_gram_client_attr_t *attr) {
    PyObject *tuple, *ptr;
    int ret;
    const char *tmpstr;
    gss_cred_id_t *cred;
    if (!(tuple=PyTuple_New(2))) {
        PyErr_SetString(PyExc_RuntimeError, "client_attr_get_credential: PyTuple_New()");
        return NULL;
    }
    cred = (gss_cred_id_t*)globus_malloc(sizeof(gss_cred_id_t));
    ret = globus_gram_client_attr_get_credential(*attr, cred);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS) {
        tmpstr = globus_gram_client_error_string(ret);
	PyTuple_SET_ITEM(tuple,1,Py_BuildValue("s", tmpstr));
    }
    else {
        ptr = SWIG_NewPointerObj((void*)cred, SWIGTYPE_p_gss_cred_id_t, 0);
        PyTuple_SET_ITEM(tuple, 1, ptr); 
    } 
    return tuple;
}

int
client_set_credentials(gss_cred_id_t *cred) {
    return globus_gram_client_set_credentials(*cred);
}

PyObject *
register_job_cancel(
    const char *			job_contact,
    globus_gram_client_attr_t          *attr,
    PyObject *                          pyfunc,
    PyObject *				arg) {
    int ret=0;
    PyObject *list, *cbPtr;
    callbackStruct* cbStruct;
    PyObject *tuple, *ptr;
    const char *tmpstr;
    gss_cred_id_t *cred;

    cbStruct = (callbackStruct*) globus_malloc(sizeof(callbackStruct));
    Py_XINCREF(pyfunc);
    Py_XINCREF(arg);
    cbStruct->callback = pyfunc; 
    cbStruct->arg = arg;   
    if (!(tuple=PyTuple_New(2))) {
        PyErr_SetString(PyExc_RuntimeError, "client_attr_get_credential: PyTuple_New()");
        return NULL;
    }

    Py_BEGIN_ALLOW_THREADS
    ret = globus_gram_client_register_job_cancel(job_contact, GLOBUS_GRAM_CLIENT_NO_ATTR, NonBlockingPythonCallback,cbStruct); 
    Py_END_ALLOW_THREADS
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS) {
        tmpstr = globus_gram_client_error_string(ret);
	PyTuple_SET_ITEM(tuple,1,Py_BuildValue("s", tmpstr));
    }
    else {
	cbPtr = SWIG_NewPointerObj((void*)cbStruct, SWIGTYPE_p_callbackStruct, 0);
	Py_XINCREF(cbPtr);
	PyTuple_SET_ITEM(tuple, 1, cbPtr); 
    } 
    return tuple;
}

PyObject *
register_job_status(
    const char *			job_contact,
    globus_gram_client_attr_t		attr,
    PyObject *                          pyfunc,
    PyObject *				arg) {
    int ret=0;
    PyObject *list, *cbPtr;
    callbackStruct* cbStruct;
    PyObject *tuple, *ptr;
    const char *tmpstr;
    gss_cred_id_t *cred;
    cbStruct = (callbackStruct*)globus_malloc(sizeof(callbackStruct));

    if (!(tuple=PyTuple_New(2))) {
        PyErr_SetString(PyExc_RuntimeError, "client_attr_get_credential: PyTuple_New()");
        return NULL;
    }

    Py_XINCREF(pyfunc);
    Py_XINCREF(arg);
    cbStruct->callback = pyfunc; 
    cbStruct->arg = arg;  
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gram_client_register_job_status(job_contact, attr, NonBlockingPythonCallback,cbStruct);
    Py_END_ALLOW_THREADS

    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS) {
        tmpstr = globus_gram_client_error_string(ret);
	PyTuple_SET_ITEM(tuple,1,Py_BuildValue("s", tmpstr));
    }
    else {
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject *
register_job_signal(
    const char *			job_contact,
    globus_gram_protocol_job_signal_t	signal,
    const char *			signal_arg,
    globus_gram_client_attr_t		attr,
    PyObject *                          pyfunc,
    PyObject *				arg) {
    PyObject *tuple, *ptr;
    const char *tmpstr;
    gss_cred_id_t *cred;
    PyObject *list, *cbPtr;
    callbackStruct* cbStruct;
    int ret=0; 

    cbStruct = (callbackStruct*)globus_malloc(sizeof(callbackStruct));
    Py_XINCREF(pyfunc);
    Py_XINCREF(arg);
    cbStruct->callback = pyfunc; 
    cbStruct->arg = arg;   
    
    Py_BEGIN_ALLOW_THREADS
    ret=globus_gram_client_register_job_signal(job_contact, signal, signal_arg,attr,NonBlockingPythonCallback, cbStruct);
    Py_END_ALLOW_THREADS
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS) {
        tmpstr = globus_gram_client_error_string(ret);
	PyTuple_SET_ITEM(tuple,1,Py_BuildValue("s", tmpstr));
    }
    else {
        ptr = SWIG_NewPointerObj((void*)cred, SWIGTYPE_p_gss_cred_id_t, 0);
        PyTuple_SET_ITEM(tuple, 1, ptr); 
    } 
    return tuple;
}

PyObject *
job_signal(
    const char *			job_contact,
    globus_gram_protocol_job_signal_t	signal,
    const char *			signal_arg,
    int *				job_status,
    int *				failure_code) {
    int ret=0;
    PyObject *tuple, *ptr;
    const char *tmpstr;
    gss_cred_id_t *cred;

    Py_BEGIN_ALLOW_THREADS
    ret = globus_gram_client_job_signal(job_contact, signal, signal_arg, job_status, failure_code);
    Py_END_ALLOW_THREADS
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS) {
        tmpstr = globus_gram_client_error_string(ret);
	PyTuple_SET_ITEM(tuple,1,Py_BuildValue("s", tmpstr));
    }
    else {
        ptr = SWIG_NewPointerObj((void*)cred, SWIGTYPE_p_gss_cred_id_t, 0);
        PyTuple_SET_ITEM(tuple, 1, ptr); 
    } 
    return tuple;
}

PyObject * 
register_ping(
    const char *			resource_manager_contact,
    globus_gram_client_attr_t		attr,
    PyObject *                          pyfunc,
    PyObject*				arg) {
    int ret=0;
    PyObject *list, *cbPtr;
    callbackStruct* cbStruct;
    PyObject *tuple, *ptr;
    const char *tmpstr;
    gss_cred_id_t *cred;

    cbStruct = (callbackStruct*)globus_malloc(sizeof(callbackStruct));
    Py_XINCREF(pyfunc);
    Py_XINCREF(arg);
    cbStruct->callback = pyfunc; 
    cbStruct->arg = arg;   
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gram_client_register_ping(resource_manager_contact, attr,\
	 NonBlockingPythonCallback, cbStruct);
    Py_END_ALLOW_THREADS
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS) {
        tmpstr = globus_gram_client_error_string(ret);
	PyTuple_SET_ITEM(tuple,1,Py_BuildValue("s", tmpstr));
    }
    else {
        ptr = SWIG_NewPointerObj((void*)cred, SWIGTYPE_p_gss_cred_id_t, 0);
        PyTuple_SET_ITEM(tuple, 1, ptr); 
    } 
    return tuple;
}

PyObject *
register_job_callback_registration(
    const char *			job_contact,
    int					job_state_mask,
    const char *			callback_contact,
    globus_gram_client_attr_t		attr,
    PyObject *                          pyfunc,
    PyObject *				arg) {
    int ret=0;
    PyObject *tuple, *ptr;
    const char *tmpstr;
    gss_cred_id_t *cred;
    PyObject *list, *cbPtr;
    callbackStruct* cbStruct;
    cbStruct = (callbackStruct*)globus_malloc(sizeof(callbackStruct));
    Py_XINCREF(pyfunc);
    Py_XINCREF(arg);
    cbStruct->callback = pyfunc; 
    cbStruct->arg = arg;   
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gram_client_register_job_callback_registration(job_contact, job_state_mask, \
          callback_contact, attr, NonBlockingPythonCallback, cbStruct);
    Py_END_ALLOW_THREADS
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS) {
        tmpstr = globus_gram_client_error_string(ret);
	PyTuple_SET_ITEM(tuple,1,Py_BuildValue("s", tmpstr));
    }
    else {
        ptr = SWIG_NewPointerObj((void*)cred, SWIGTYPE_p_gss_cred_id_t, 0);
        PyTuple_SET_ITEM(tuple, 1, ptr); 
    } 
    return tuple;
}




%}        
extern const char*
globus_gram_protocol_error_string(int error_code);

extern void
globus_gram_client_debug(void);

extern int
globus_gram_client_version(void);

