/*############################################################################
# Matt Rodriguez, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################*/
%module gssc
%{
#undef SIZEOF_OFF_T
#undef HAVE_LIMITS_H
#undef socklen_t
#include "globus_common.h"
#include "globus_gss_assist.h"
#include "gssapi.h"
#include "globus_io.h"
#include "globus_gsi_credential.h"
#include "globus_gsi_proxy.h"
#include "globus_gsi_cert_utils.h"
#include "globus_gsi_system_config.h"
#include "globus_gsi_system_config_constants.h"
#include "globus_openssl.h" 
%}

%include ./src/pyGlobus-typemaps.i


%constant GSS_EXPORT_CRED_TO_BUFFER=0;
%constant GSS_EXPORT_CRED_EXTERNAL=1;

%types(gss_OID*);
%types(gss_buffer_t);

%constant GSS_C_GSS_CODE=1;
%constant GSS_C_MECH_CODE=2;

%constant GSS_S_COMPLETE=0;
%constant GSS_C_BOTH=0;
%constant GSS_C_INITIATE=1;
%constant GSS_C_ACCEPT=2;

%constant GSS_C_DELEG_FLAG=1;
%constant GSS_C_MUTUAL_FLAG=2;
%constant GSS_C_REPLAY_FLAG=4;
%constant GSS_C_SEQUENCE_FLAG=8;
%constant GSS_C_CONF_FLAG=6;
%constant GSS_C_INTEG_FLAG=32;
%constant GSS_C_ANON_FLAG=64;
%constant GSS_C_PROT_READY_FLAG=128;
%constant GSS_C_TRANS_FLAG=256;

%constant GSS_C_GLOBUS_LIMITED_DELEG_PROXY_FLAG=4096;
%constant GSS_C_GLOBUS_LIMITED_PROXY_FLAG        = 192;
%constant GSS_C_GLOBUS_SSL_COMPATABLE          = 16384;

%constant GSS_C_CALLING_ERROR_OFFSET=24;
%constant GSS_C_ROUTINE_ERROR_OFFSET=16;
%constant GSS_C_SUPPLEMENTARY_OFFSET=0;
%constant GSS_C_CALLING_ERROR_MASK=0377ul;
%constant GSS_C_ROUTINE_ERROR_MASK=0377ul;
%constant GSS_C_SUPPLEMENTARY_MASK=0177777ul;
%constant GSS_S_BAD_MECH=(1ul << GSS_C_ROUTINE_ERROR_OFFSET);
%constant GSS_S_BAD_NAME=(2ul << GSS_C_ROUTINE_ERROR_OFFSET);
%constant GSS_S_BAD_NAMETYPE=(3ul << GSS_C_ROUTINE_ERROR_OFFSET);
%constant GSS_S_BAD_BINDINGS=(4ul << GSS_C_ROUTINE_ERROR_OFFSET);
%constant GSS_S_BAD_STATUS=(5ul << GSS_C_ROUTINE_ERROR_OFFSET);
%constant GSS_S_BAD_SIG=(6ul << GSS_C_ROUTINE_ERROR_OFFSET);
%constant GSS_S_BAD_MIC=GSS_S_BAD_SIG;
%constant GSS_S_NO_CRED=(7ul << GSS_C_ROUTINE_ERROR_OFFSET);
%constant GSS_S_NO_CONTEXT=(8ul << GSS_C_ROUTINE_ERROR_OFFSET);
%constant GSS_S_DEFECTIVE_TOKEN=(9ul << GSS_C_ROUTINE_ERROR_OFFSET);
%constant GSS_S_DEFECTIVE_CREDENTIAL=(10ul << GSS_C_ROUTINE_ERROR_OFFSET);
%constant GSS_S_CREDENTIALS_EXPIRED=(11ul << GSS_C_ROUTINE_ERROR_OFFSET);
%constant GSS_S_CONTEXT_EXPIRED=(12ul << GSS_C_ROUTINE_ERROR_OFFSET);
%constant GSS_S_FAILURE=(13ul << GSS_C_ROUTINE_ERROR_OFFSET);
%constant GSS_S_BAD_QOP=(14ul << GSS_C_ROUTINE_ERROR_OFFSET);
%constant GSS_S_UNAUTHORIZED=(15ul << GSS_C_ROUTINE_ERROR_OFFSET);
%constant GSS_S_UNAVAILABLE=(16ul << GSS_C_ROUTINE_ERROR_OFFSET);
%constant GSS_S_DUPLICATE_ELEMENT=(17ul << GSS_C_ROUTINE_ERROR_OFFSET);
%constant GSS_S_NAME_NOT_MN=(18ul << GSS_C_ROUTINE_ERROR_OFFSET);
%constant GSS_S_CONTINUE_NEEDED=(1ul << (GSS_C_SUPPLEMENTARY_OFFSET + 0));
%constant GSS_S_DUPLICATE_TOKEN=(1ul << (GSS_C_SUPPLEMENTARY_OFFSET + 1));
%constant GSS_S_OLD_TOKEN=(1ul << (GSS_C_SUPPLEMENTARY_OFFSET + 2));
%constant GSS_S_UNSEQ_TOKEN=(1ul << (GSS_C_SUPPLEMENTARY_OFFSET + 3));
%constant GSS_S_GAP_TOKEN=(1ul << (GSS_C_SUPPLEMENTARY_OFFSET + 4));

%constant GSS_C_INDEFINITE=0xfffffffful;

%apply char **OUTPUT { char **charRet };
%apply SWIGTYPE_gss_buffer_t {gss_buffer_t buf, gss_buffer_t input_token}
%apply SWIGTYPE_gss_buffer_t {gss_buffer_t msg_token, gss_buffer_t msg_buffer};

extern gss_OID GSS_C_NT_USER_NAME;
extern gss_OID GSS_C_NT_MACHINE_UID_NAME;
extern gss_OID GSS_C_NT_STRING_UID_NAME;
extern gss_OID GSS_C_NT_HOSTBASED_SERVICE_X;
extern gss_OID GSS_C_NT_HOSTBASED_SERVICE;
extern gss_OID GSS_C_NT_ANONYMOUS;
extern gss_OID GSS_C_NT_EXPORT_NAME;


%inline %{ 

globus_module_descriptor_t* get_module(void) {
    return GLOBUS_GSI_GSS_ASSIST_MODULE;
}

globus_module_descriptor_t* get_module_proxy(void){
    return GLOBUS_GSI_PROXY_MODULE;
} 

globus_module_descriptor_t* get_module_callback(void){
    return GLOBUS_GSI_CALLBACK_MODULE;
}

globus_module_descriptor_t* get_module_openssl(void){
    return GLOBUS_OPENSSL_MODULE;
}

globus_module_descriptor_t* get_module_credential(void){
    return GLOBUS_GSI_CREDENTIAL_MODULE;
}

gss_cred_id_t*
get_no_cred_const_ptr() {
    gss_cred_id_t* cred;
    cred = (gss_cred_id_t*)globus_malloc(sizeof(gss_cred_id_t));
    *cred = GSS_C_NO_CREDENTIAL;
    return cred;
}

void
free_cred_ptr(gss_cred_id_t* cred){
    globus_free(cred);
}

PyObject*
release_cred(gss_cred_id_t* cred) {
    OM_uint32 major_status;
    OM_uint32 minor_status;
    PyObject* tuple;
    if (!(tuple=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, 
                        "release_cred: PyTuple_New()");
        return NULL;
    }
    major_status = gss_release_cred(&minor_status, cred);
    globus_free(cred);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", major_status));
    PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("i", minor_status));
    return tuple;
} 

PyObject *
create_empty_oid_set(){
   PyObject * ptr, *tuple;
   OM_uint32 minor_status = 0;
   OM_uint32 major_status = 0;
   gss_OID_set * mechs;
   
   if (!(tuple=PyTuple_New(3))){
        PyErr_SetString(PyExc_RuntimeError, 
                        "create_empty_oid_set:: PyTuple_New()");
        return NULL;
   }	
   mechs = (gss_OID_set *)globus_malloc(sizeof(gss_OID_set));
   major_status = gss_create_empty_oid_set(&minor_status, mechs);
   PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", major_status));
   PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("i", minor_status));
   
   if (major_status != GSS_S_COMPLETE){
       Py_INCREF(Py_None);
       PyTuple_SET_ITEM(tuple, 2, Py_None);
   }
   else{
       ptr = SWIG_NewPointerObj((void *)mechs, SWIGTYPE_p_gss_OID_set, 0);
       PyTuple_SET_ITEM(tuple, 2, ptr);
   }	

   return tuple;
}	

PyObject *
new_X509(){
    X509 * x509;
    PyObject * ptr;
    x509 = X509_new();
    ptr =  SWIG_NewPointerObj((void *)x509, SWIGTYPE_p_X509,0);
    return ptr;
}

void
free_X509(X509* x){ 
    X509_free(x);   
}

PyObject *
sign_X509(X509 *x509, EVP_PKEY * pkey){
    PyObject *ptr, *tuple;
    int ret;
    if (!(tuple=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, 
                        "sign_X509:: PyTuple_New()");
        return NULL;
    }

    ret = X509_sign(x509, pkey,EVP_md5());
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    ptr = SWIG_NewPointerObj((void *)x509, SWIGTYPE_p_X509, 0);
    PyTuple_SET_ITEM(tuple, 1, ptr);
    return tuple;

}

PyObject *
get_pubkey(X509 * x509){
    PyObject * ptr;
    EVP_PKEY * pkey  = NULL;
    pkey = X509_get_pubkey(x509);
    ptr = SWIG_NewPointerObj((void *)pkey, SWIGTYPE_p_EVP_PKEY,0);
    return ptr;
}

PyObject *
new_X509_REQ(){
    X509_REQ * x509_req;
    PyObject *ptr;
    x509_req =  X509_REQ_new(); 
    ptr = SWIG_NewPointerObj((void *)x509_req, SWIGTYPE_p_X509_REQ,0);
    return ptr;	
}

void
free_X509_REQ(X509_REQ* req){
    X509_REQ_free(req);   

}


void
free_EVP_PKEY(EVP_PKEY * pkey){   
    EVP_PKEY_free(pkey);
       
}

PyObject *
new_EVP_PKEY() {
    EVP_PKEY * pkey;
    PyObject *ptr;
    pkey = EVP_PKEY_new(); 
    ptr = SWIG_NewPointerObj((void *)pkey, SWIGTYPE_p_EVP_PKEY,0);
    return ptr;
}

PyObject *
sign_rsa(EVP_PKEY *pkey, PyObject * pySignString){
    PyObject * ptr, *err, *tuple;
    int ret = 0;
    int sign_string_len = 0;
    char * sign_string = NULL;
    unsigned char * sign_buf;
    int buf_len = 0;
    unsigned int real_buf_len = 0;
    
    if (PyString_Check(pySignString)){
        sign_string_len = PyString_Size(pySignString);
        sign_string = PyString_AsString(pySignString);
        if((err = PyErr_Occurred()) !=NULL){
            PyErr_Print();
            return NULL;
        }
    }
    if (!(tuple=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, 
                        "sign_rsa:: PyTuple_New()");
        return NULL;
    }	
    /*Need to be able to handle different keys, i.e. DSA, and different HMAC schemes
      MKR
    */
    buf_len = RSA_size(EVP_PKEY_get1_RSA(pkey));
    sign_buf = (unsigned char *) globus_malloc(sizeof(unsigned char) * buf_len);

    ret = RSA_sign(NID_sha1, (unsigned char *) sign_string, sign_string_len, sign_buf, &real_buf_len,EVP_PKEY_get1_RSA(pkey));
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if ( ret == 0){
	PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", "RSA_sign did not succeed"));
	globus_free(sign_buf);
    }
    else{
	ptr = PyString_FromStringAndSize( (const char *)sign_buf, buf_len);
	PyTuple_SET_ITEM(tuple,1,ptr);	
    }

    return tuple;			
}

int
verify_rsa(EVP_PKEY *pkey, PyObject * pyVerifyString, PyObject *pySignatureString){
    PyObject * ptr, *err;
    int ret = 0;
    int verify_string_len = 0;
    int signature_string_len = 0;
    char * signature_string = NULL;
    char * verify_string = NULL;
    
    if (PyString_Check(pyVerifyString)){
        verify_string_len = PyString_Size(pyVerifyString);
        verify_string = PyString_AsString(pyVerifyString);
        if((err = PyErr_Occurred()) !=NULL){
            PyErr_Print();
            return -1;
        }
    }
 
    if (PyString_Check(pySignatureString)){
        signature_string_len = PyString_Size(pySignatureString);
        signature_string = PyString_AsString(pySignatureString);
        if((err = PyErr_Occurred()) !=NULL){
            PyErr_Print();
            return -1;
        }
    }
 
    /*Need to be able to handle different keys, i.e. DSA, and different HMAC schemes
      MKR
    */
    ret = RSA_verify(NID_sha1, (unsigned char *)verify_string,
		     verify_string_len, (unsigned char *)signature_string, 
		     signature_string_len, EVP_PKEY_get1_RSA(pkey));
    return ret;
			
}

PyObject *
release_oid_set(gss_OID_set * mechs){
   PyObject  *tuple;
   OM_uint32 minor_status = 0;
   OM_uint32 major_status = 0;
   
   if (!(tuple=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, 
                        "release_oid_set:: PyTuple_New()");
        return NULL;
   }	

   major_status = gss_release_oid_set(&minor_status, mechs);
   globus_free(mechs);
   PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", major_status));
   PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("i", minor_status));
   
   return tuple;
}

PyObject *
get_usage_both(){
    PyObject * ptr;
    gss_cred_usage_t * usage;
    usage = (gss_cred_usage_t *)globus_malloc(sizeof(gss_cred_usage_t));
    *usage = GSS_C_BOTH;
    ptr = SWIG_NewPointerObj((void *)usage, SWIGTYPE_p_gss_cred_usage_t, 0); 
    return ptr;
}		

PyObject *
get_usage_initiate(){

    PyObject * ptr;
    gss_cred_usage_t * usage;
    usage = (gss_cred_usage_t *)globus_malloc(sizeof(gss_cred_usage_t));
    *usage = GSS_C_INITIATE;
    ptr = SWIG_NewPointerObj((void *)usage, SWIGTYPE_p_gss_cred_usage_t, 0); 
    return ptr;
}

PyObject *
get_usage_accept(){
 
    PyObject * ptr;
    gss_cred_usage_t * usage;
    usage = (gss_cred_usage_t *)globus_malloc(sizeof(gss_cred_usage_t));
    *usage = GSS_C_ACCEPT;
    ptr = SWIG_NewPointerObj((void *)usage, SWIGTYPE_p_gss_cred_usage_t, 0); 
    return ptr;
}

void 
free_usage(gss_cred_usage_t *usage){
  globus_free(usage);
}

PyObject*
acquire_cred(gss_name_t *name, gss_OID_set * desired_mechs,gss_cred_usage_t *cred_usage, PyObject * py_time_req){
    OM_uint32 minor_status;	
    OM_uint32 major_status;
    OM_uint32 time_rec;
    OM_uint32 time_requested;
    gss_cred_id_t * output_cred;
    gss_OID_set  * actual_mechs;
    PyObject * ptr, * ptr2;
    PyObject * tuple;
    
    
    if (!(tuple=PyTuple_New(4))){
	PyErr_SetString(PyExc_RuntimeError, 
			"acquire_cred:: PyTuple_New()");
	return NULL;
    }
    time_requested = PyInt_AsUnsignedLongMask(py_time_req); 
    actual_mechs = (gss_OID_set *)globus_malloc(sizeof(gss_OID_set *)); 
    output_cred = (gss_cred_id_t *)globus_malloc(sizeof(gss_cred_id_t));	
    major_status = gss_acquire_cred(&minor_status, *name, time_requested, 
                                    *desired_mechs, *cred_usage, output_cred, 
                                    actual_mechs, &time_rec);
    
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", major_status));
    PyTuple_SET_ITEM(tuple,1, Py_BuildValue("i", minor_status));
    if (major_status != GSS_S_COMPLETE){
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 2, Py_None);
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 3, Py_None);
    }
    else{
	ptr = SWIG_NewPointerObj((void *)output_cred, SWIGTYPE_p_gss_cred_id_t,0);
	PyTuple_SET_ITEM(tuple, 2, ptr);
	ptr = SWIG_NewPointerObj((void *)actual_mechs, SWIGTYPE_p_gss_OID_set,0);
	PyTuple_SET_ITEM(tuple, 3, ptr);
    }
    return tuple;
}

PyObject *
cred_handle_attr_init(){
    
    PyObject * tuple, *ptr;
    char * tmpstr;
    globus_result_t ret;
    globus_gsi_cred_handle_attrs_t * attr;
    
    if (!(tuple=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, 
                        "cred_handle_attr_init:: PyTuple_New()");
        return NULL;
    }
    
    attr = (globus_gsi_cred_handle_attrs_t *)globus_malloc(sizeof(globus_gsi_cred_handle_attrs_t));
    ret = globus_gsi_cred_handle_attrs_init(attr);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret !=GLOBUS_SUCCESS){
	tmpstr = globus_object_printable_to_string(globus_error_get(ret));
       	PyTuple_SET_ITEM(tuple,1, Py_BuildValue("s", tmpstr));
	globus_free(tmpstr);
    }
    else{
	ptr = SWIG_NewPointerObj((void *)attr, SWIGTYPE_p_globus_gsi_cred_handle_attrs_t,0);
	PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

PyObject *
cred_handle_attr_destroy( globus_gsi_cred_handle_attrs_t * attr){
    
    PyObject * tuple;
    char * tmpstr;
    globus_result_t ret;
    
    if (!(tuple=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, 
                        "cred_handle_attr_destroy:: PyTuple_New()");
        return NULL;
    }
    
  
    ret = globus_gsi_cred_handle_attrs_destroy(*attr);
    globus_free(attr);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret !=GLOBUS_SUCCESS){
	tmpstr = globus_object_printable_to_string(globus_error_get(ret));
       	PyTuple_SET_ITEM(tuple,1, Py_BuildValue("s", tmpstr));
	globus_free(tmpstr);
    }
    else{
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject *
cred_handle_init(globus_gsi_cred_handle_attrs_t * attr){
    PyObject * tuple, *ptr;
    char * tmpstr;
    globus_result_t ret;
    globus_gsi_cred_handle_t * handle;
    if (!(tuple=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, 
                        "cred_handle_init:: PyTuple_New()");
        return NULL;
    }
    
    handle = (globus_gsi_cred_handle_t *)globus_malloc(sizeof(globus_gsi_cred_handle_t));
    ret = globus_gsi_cred_handle_init(handle, *attr);
    
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret !=GLOBUS_SUCCESS){
	tmpstr = globus_object_printable_to_string(globus_error_get(ret));
       	PyTuple_SET_ITEM(tuple,1, Py_BuildValue("s", tmpstr));
	globus_free(tmpstr);
    }
    else{
        ptr = SWIG_NewPointerObj((void *)handle, SWIGTYPE_p_globus_gsi_cred_handle_t,0);
        PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}
PyObject *
cred_handle_destroy(globus_gsi_cred_handle_t * cred_handle){
    
    PyObject * tuple;
    char * tmpstr;
    globus_result_t ret;

    if (!(tuple=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, 
                        "cred_handle_destroy:: PyTuple_New()");
        return NULL;
    }
    ret = globus_gsi_cred_handle_destroy(*cred_handle);
    globus_free(cred_handle);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    
    if (ret !=GLOBUS_SUCCESS){
	tmpstr = globus_object_printable_to_string(globus_error_get(ret));
       	PyTuple_SET_ITEM(tuple,1, Py_BuildValue("s", tmpstr));
	globus_free(tmpstr);
    }
    else{
	Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject *
cred_read(globus_gsi_cred_handle_t * cred_handle){
    PyObject * tuple;
    char * tmpstr;
    globus_result_t ret;

    if (!(tuple=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, 
                        "cred_read:: PyTuple_New()");
        return NULL;
    }
    
    /*The second argument is a X509_NAME, desired subject 
      We don't have a class that wraps X509_NAMEs in pyGlobus
      and we don't need one right now so I'm calling this with NULL
      MKR
    */
    ret = globus_gsi_cred_read(*cred_handle, NULL);

    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret !=GLOBUS_SUCCESS){
	tmpstr = globus_object_printable_to_string(globus_error_get(ret));
       	PyTuple_SET_ITEM(tuple,1, Py_BuildValue("s", tmpstr));
	globus_free(tmpstr);
    }
    else{
	Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject *
cred_get_cert(globus_gsi_cred_handle_t * cred_handle){
    
    PyObject * tuple, *ptr;
    char * tmpstr;
    globus_result_t ret;
    X509 * cert = NULL;

    if (!(tuple=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, 
                        "cred_get_cert:: PyTuple_New()");
        return NULL;
    }
    
    
    ret = globus_gsi_cred_get_cert(*cred_handle, &cert);

    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret !=GLOBUS_SUCCESS){
	tmpstr = globus_object_printable_to_string(globus_error_get(ret));
       	PyTuple_SET_ITEM(tuple,1, Py_BuildValue("s", tmpstr));
	globus_free(tmpstr);
    }
    else{
	ptr = SWIG_NewPointerObj((void *)cert, SWIGTYPE_p_X509,0);
        PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

PyObject *
cred_get_cert_chain(globus_gsi_cred_handle_t * cred_handle){
    PyObject * tuple, *ptr;
    char * tmpstr;
    globus_result_t ret;
    STACK_OF(X509) * cert_chain;

    if (!(tuple=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, 
                        "cred_get_cert_chain:: PyTuple_New()");
        return NULL;
    }
    
    ret = globus_gsi_cred_get_cert_chain(*cred_handle, &cert_chain);

    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret !=GLOBUS_SUCCESS){
	tmpstr = globus_object_printable_to_string(globus_error_get(ret));
       	PyTuple_SET_ITEM(tuple,1, Py_BuildValue("s", tmpstr));
	globus_free(tmpstr);
    }
    else{
	ptr = SWIG_NewPointerObj((void *)cert_chain, SWIGTYPE_p_STACK,0);
        PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

PyObject *
cred_get_key(globus_gsi_cred_handle_t * cred_handle){
    PyObject * tuple, *ptr;
    char * tmpstr;
    globus_result_t ret;
    EVP_PKEY * pkey;
    
    if (!(tuple=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, 
                        "cred_get_cert_chain:: PyTuple_New()");
        return NULL;
    }
    
    pkey = EVP_PKEY_new();
    ret = globus_gsi_cred_get_key(*cred_handle, &pkey);

    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret !=GLOBUS_SUCCESS){
	tmpstr = globus_object_printable_to_string(globus_error_get(ret));
       	PyTuple_SET_ITEM(tuple,1, Py_BuildValue("s", tmpstr));
	globus_free(tmpstr);
    }
    else{
	ptr = SWIG_NewPointerObj((void *)pkey, SWIGTYPE_p_EVP_PKEY,0);
        PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

void
free_stack(STACK * stack){
    sk_X509_free((STACK_OF(X509) *)stack);
}

PyObject *
get_der_encoding_stack(STACK * stack){
    PyObject * tuple, *encodedString;
    globus_result_t ret;
    unsigned char * encoding;
    int len;
    if (!(tuple=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, 
                        "get_der_encoding:: PyTuple_New()");
        return NULL;
    }

    encoding = ASN1_seq_pack((STACK_OF(X509)*) stack, i2d_X509, NULL, &len);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
    encodedString = PyString_FromStringAndSize((const char *)encoding, len);
    globus_free(encoding);
    PyTuple_SET_ITEM(tuple, 1, encodedString);
    return tuple;
     
}

PyObject *
make_stack_from_der_sequence(PyObject * pyEncodedString){
    PyObject * ptr, *err;
    STACK_OF(X509) * certs = NULL;
    int encoded_string_len = 0;
    char * encoded_string = NULL;

    if (PyString_Check(pyEncodedString)){
        encoded_string_len = PyString_Size(pyEncodedString);
        encoded_string = PyString_AsString(pyEncodedString);
        if((err = PyErr_Occurred()) !=NULL){
            PyErr_Print();
            return NULL;
        }
    }

    certs = ASN1_seq_unpack((unsigned char *)encoded_string, encoded_string_len,  
			    (char *(*)())d2i_X509, (void(*)())X509_free );
    
    ptr = SWIG_NewPointerObj((void *)certs,SWIGTYPE_p_STACK, 0);
    return ptr;

}

PyObject *
make_stack_from_der_string(PyObject * pyEncodedString){
    PyObject * ptr, *err;
    STACK_OF(X509) * certs = NULL;
    int encoded_string_len = 0;
    char * encoded_string = NULL;
    BIO *bio;
    X509 * temp_cert= NULL;
    if (PyString_Check(pyEncodedString)){
        encoded_string_len = PyString_Size(pyEncodedString);
        encoded_string = PyString_AsString(pyEncodedString);
        if((err = PyErr_Occurred()) !=NULL){
            PyErr_Print();
            return NULL;
        }
    }

    bio = BIO_new_mem_buf(encoded_string, encoded_string_len);
    certs = sk_X509_new_null();
    while ( temp_cert = d2i_X509_bio(bio, NULL)){
	sk_X509_push(certs, X509_dup(temp_cert));
    }
    ptr = SWIG_NewPointerObj((void *)certs,SWIGTYPE_p_STACK, 0);
    return ptr;

}

void
insert_X509(STACK * stack, X509 * x509, int pos){
    sk_X509_insert((STACK_OF(X509) *)stack, x509, pos);
}

PyObject *
stack_pop(STACK * stack){

    PyObject * ptr;
    X509 * x509 = NULL;
    x509 = sk_X509_pop((STACK_OF(X509) *)stack);
    ptr = SWIG_NewPointerObj((void *)x509, SWIGTYPE_p_X509, 0);
    return ptr;
}

PyObject *
import_name(PyObject* nameStr, const gss_OID nameType) {
    OM_uint32 major_status;
    OM_uint32 minor_status;
    PyObject *tuple, *ptr;
    gss_buffer_t buf;
    gss_name_t* name;

    if (!(tuple=PyTuple_New(3))){
        PyErr_SetString(PyExc_RuntimeError, 
                        "import_name: PyTuple_New()");
        return NULL;
    }

    if (!PyString_Check(nameStr)) {
	PyErr_SetString(PyExc_TypeError, "not a string");
	return NULL;
    }

    buf = (gss_buffer_desc*)globus_malloc(sizeof(gss_buffer_desc));
    name = (gss_name_t *)globus_malloc(sizeof(gss_name_t));	
    buf->length = PyString_Size(nameStr);
    buf->value = PyString_AsString(nameStr);
    major_status = gss_import_name(&minor_status, buf, nameType, name);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", major_status));
    PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("i", minor_status));

    if (major_status != GSS_S_COMPLETE) {
	Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 2, Py_None);
    }
    else {
	ptr = SWIG_NewPointerObj((void*)name, SWIGTYPE_p_gss_name_t, 0);
	PyTuple_SET_ITEM(tuple, 2, ptr);
    }

    globus_free(buf);
    return tuple;
}

PyObject *
get_default_name(){
    PyObject * ptr;
    gss_name_t *name;
    name = (gss_name_t *)globus_malloc(sizeof(gss_name_t));
    *name = GSS_C_NO_NAME;
    ptr = SWIG_NewPointerObj((void*)name, SWIGTYPE_p_gss_name_t, 0);
    return ptr;

}	

PyObject*
display_name(gss_name_t* name) {
    PyObject* tuple;
    OM_uint32         major = 0;
    OM_uint32         minor = 0;
    gss_OID           display_oid;
    gss_buffer_t      output_buf;
    char val_buf[1000];

    if (!(tuple=PyTuple_New(3))){
        PyErr_SetString(PyExc_RuntimeError, "display_name: PyTuple_New()");
        return NULL;
    }
    
    output_buf = (gss_buffer_desc *)globus_malloc(sizeof(gss_buffer_desc)); 	
    output_buf->value = val_buf;

    major = gss_display_name(&minor, *name, output_buf, &display_oid);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", major));
    PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("i", minor));

    if (major != GSS_S_COMPLETE) {
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 2, Py_None);
    }
    else {
	PyTuple_SET_ITEM(tuple, 2, Py_BuildValue("s#", (char*)output_buf->value, 
						 (int)output_buf->length));
	gss_release_buffer(&minor, output_buf);
    }
    globus_free(output_buf);
    return tuple;
}
 
PyObject*
release_name(gss_name_t* name) {
    PyObject* tuple;
    OM_uint32 major_status;
    OM_uint32 minor_status;

    if (!(tuple=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, 
                        "release_name: PyTuple_New()");
        return NULL;
    }

    major_status = gss_release_name(&minor_status, name);
    globus_free(name);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", major_status));
    PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("i", minor_status));
    return tuple;
}

PyObject*
init_sec_context(gss_ctx_id_t *ctx, const gss_cred_id_t *initiator_cred,
		 gss_name_t *target_name, gss_buffer_t  input_token, int request_flags) {

    PyObject* tuple;
    PyObject* ptr, *ptr1;
    OM_uint32 major_status;
    OM_uint32 minor_status;
    OM_uint32 req_flags = (OM_uint32)request_flags;
    gss_buffer_t output_token;
    gss_OID actual_mech;
    OM_uint32 ret_flags;
    OM_uint32 time_rec;
    gss_OID mech_type = NULL;
    
    if (!(tuple=PyTuple_New(4))){
       PyErr_SetString(PyExc_RuntimeError, 
		       "init_sec_context: PyTuple_New()");
       return NULL;
    }

    output_token = (gss_buffer_t)globus_malloc(sizeof(gss_buffer_desc));
    output_token->length = 0;
    output_token->value = NULL;
   
    /*Right now time_req is GSS_C_INDEFINITE, input_channel_bindings are set, mech_type is NULL
    If you want you can pass actual_mech, ret_flags, time_rec back up */
    
   
    major_status = gss_init_sec_context(&minor_status, *initiator_cred, ctx,
					*target_name,mech_type, 
					req_flags,GSS_C_INDEFINITE,
					GSS_C_NO_CHANNEL_BINDINGS, 
					input_token, &actual_mech,
					output_token,&ret_flags, &time_rec);
  
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", major_status));
    PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("i", minor_status));
 
    if (major_status != GSS_S_COMPLETE && major_status != GSS_S_CONTINUE_NEEDED) {
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 2, Py_None);
    	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 3, Py_None);
    }
    else {
	ptr =SWIG_NewPointerObj((void *)output_token,SWIGTYPE_gss_buffer_t,0);
	PyTuple_SET_ITEM(tuple, 2,ptr);
	ptr1 = SWIG_NewPointerObj((void *)ctx,SWIGTYPE_p_gss_ctx_id_t, 0);
	PyTuple_SET_ITEM(tuple,3,ptr1);
    }
    return tuple;
} 

PyObject *
accept_sec_context(gss_ctx_id_t *ctx, gss_cred_id_t *acceptor_cred_handle, gss_buffer_t input_token){

   PyObject * tuple;
   PyObject * ptr, *ptr1, *ptr2;
   OM_uint32 major_status = 0;
   OM_uint32 minor_status= 0;
   gss_buffer_t output_token;
   const gss_channel_bindings_t input_chan_bindings = GSS_C_NO_CHANNEL_BINDINGS;
   /*These are variables that could be passed up to the Python API when major_status is GSS_C_COMPLETE */
   gss_name_t client_name = GSS_C_NO_NAME;
   gss_OID mech_type;
   gss_cred_id_t  *delegated_cred_handle;
   OM_uint32 ret_flags;
   OM_uint32 time_rec;
   
   if (!(tuple=PyTuple_New(5))){
       PyErr_SetString(PyExc_RuntimeError, 
		       "accept_sec_context: PyTuple_New()");
       return NULL;
   }
  
   output_token = (gss_buffer_t)globus_malloc(sizeof(gss_buffer_desc));
   output_token->length = 0;
   output_token->value = NULL;
   delegated_cred_handle = (gss_cred_id_t *)globus_malloc(sizeof(gss_cred_id_t));
   *delegated_cred_handle = GSS_C_NO_CREDENTIAL;
   
   major_status = gss_accept_sec_context(&minor_status, ctx,*acceptor_cred_handle,
					 input_token, input_chan_bindings, &client_name,
					 &mech_type,output_token,&ret_flags, 
					 &time_rec,delegated_cred_handle);
 				  
   PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", major_status));
   PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("i", minor_status));
   ptr2 = SWIG_NewPointerObj((void *)delegated_cred_handle,SWIGTYPE_p_gss_cred_id_t, 0);
   PyTuple_SET_ITEM(tuple,4,ptr2);
	
   if (major_status != GSS_S_COMPLETE && major_status != GSS_S_CONTINUE_NEEDED) {
       Py_INCREF(Py_None);
       PyTuple_SET_ITEM(tuple, 2, Py_None);
       Py_INCREF(Py_None);
       PyTuple_SET_ITEM(tuple, 3, Py_None);
   }
   else {
       ptr =SWIG_NewPointerObj((void *)output_token,SWIGTYPE_gss_buffer_t,0);
       PyTuple_SET_ITEM(tuple, 2,ptr);
       ptr1 = SWIG_NewPointerObj((void *)ctx,SWIGTYPE_p_gss_ctx_id_t, 0);
       PyTuple_SET_ITEM(tuple,3,ptr1);
       
   }
   
   return tuple;
}


PyObject*
create_context() {
    PyObject* ptr;
    gss_ctx_id_t *ctx; 
    ctx = (gss_ctx_id_t*)globus_malloc(sizeof(gss_ctx_id_t));
    *ctx = GSS_C_NO_CONTEXT;
    ptr = SWIG_NewPointerObj((void *)ctx, SWIGTYPE_p_gss_ctx_id_t,0);
    return ptr;
}

PyObject*
delete_context(gss_ctx_id_t* ctx) {
    PyObject* tuple;
    OM_uint32 major, minor;
    if (!(tuple=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, 
                        "delete_context: PyTuple_New()");
        return NULL;
    }
    
    major = gss_delete_sec_context(&minor, ctx, GSS_C_NO_BUFFER);
    globus_free(ctx);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", major));
    PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("i", minor));
    return tuple;
}
 
PyObject*
inquire_context(gss_ctx_id_t* ctx) {
    PyObject *tuple, *ptr, *ptr2, *ptr3;
    OM_uint32 major, minor, lifetime, ctxFlags;
    int local, open;
    gss_name_t *srcName, *targetName;
    gss_OID* mech;
    
    if (!(tuple=PyTuple_New(9))){
        PyErr_SetString(PyExc_RuntimeError, 
                        "inquire_context: PyTuple_New()");
        return NULL;
    }
    
    mech = (gss_OID*)globus_malloc(sizeof(gss_OID));
    srcName = (gss_name_t*)globus_malloc(sizeof(gss_name_t));
    targetName = (gss_name_t*)globus_malloc(sizeof(gss_name_t));
    major = gss_inquire_context(&minor, *ctx, srcName, targetName,
				&lifetime, mech, &ctxFlags, &local, &open);
   
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", major));
    PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("i", minor));
    if (major != GSS_S_COMPLETE) {
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 2, Py_None);
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 3, Py_None);
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 4, Py_None);
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 5, Py_None);
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 6, Py_None);
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 7, Py_None);
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 8, Py_None);
    }
    else {
	ptr = SWIG_NewPointerObj((void*)srcName, SWIGTYPE_p_gss_name_t, 0);
	PyTuple_SET_ITEM(tuple, 2, ptr);
	ptr2 = SWIG_NewPointerObj((void*)targetName, SWIGTYPE_p_gss_name_t, 0);
	PyTuple_SET_ITEM(tuple, 3, ptr2);
	PyTuple_SET_ITEM(tuple, 4, Py_BuildValue("i", lifetime));
	ptr3 = SWIG_NewPointerObj((void*)mech, SWIGTYPE_p_gss_OID, 0);
	PyTuple_SET_ITEM(tuple, 5, ptr3);
	PyTuple_SET_ITEM(tuple, 6, Py_BuildValue("i", ctxFlags));
	PyTuple_SET_ITEM(tuple, 7, Py_BuildValue("i", local));
	PyTuple_SET_ITEM(tuple, 8, Py_BuildValue("i", open));
    }
       
    return tuple;

} 

PyObject*
get_mic(gss_ctx_id_t* ctx, gss_buffer_t msg_buffer){
    OM_uint32 major, minor;
    gss_qop_t  qop_t;
    gss_buffer_t msg_token;
    PyObject *tuple, *ptr;

    if (!(tuple=PyTuple_New(3))){
        PyErr_SetString(PyExc_RuntimeError, 
                        "get_mic: PyTuple_New()");
        return NULL;
    }
    msg_token = (gss_buffer_t)globus_malloc(sizeof(gss_buffer_desc));
    msg_token->length = 0;
    msg_token->value = NULL;
    qop_t = GSS_C_QOP_DEFAULT;
    major = gss_get_mic(&minor, *ctx, qop_t, msg_buffer, msg_token);
    PyTuple_SET_ITEM(tuple, 0,  Py_BuildValue("i", major));
    PyTuple_SET_ITEM(tuple, 1,  Py_BuildValue("i", minor));
    ptr = SWIG_NewPointerObj((void *)msg_token, SWIGTYPE_gss_buffer_t, 0);
    PyTuple_SET_ITEM(tuple, 2, ptr);
    return tuple;    
}

PyObject *
verify_mic(gss_ctx_id_t *ctx, gss_buffer_t msg_buffer, gss_buffer_t msg_token){
    OM_uint32 major, minor;
    gss_qop_t  qop_t;
    PyObject *tuple, *ptr;

    if (!(tuple=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, 
                        "get_mic: PyTuple_New()");
        return NULL;
    }
    
    
    major = gss_verify_mic(&minor, *ctx, msg_buffer,msg_token, &qop_t);
    PyTuple_SET_ITEM(tuple, 0,  Py_BuildValue("i", major));
    PyTuple_SET_ITEM(tuple, 1,  Py_BuildValue("i", minor));
    return tuple;	
}

PyObject *
wrap(gss_ctx_id_t * ctx, gss_buffer_t msg_buffer){
    
    OM_uint32 major, minor;
    gss_qop_t  qop_t;
    PyObject *tuple, *ptr;
    int conf_req_flag = 1;
    int conf_state = 1;
    gss_buffer_t output_buffer;
    output_buffer = (gss_buffer_t)globus_malloc(sizeof(gss_buffer_desc));
    output_buffer->value = NULL;
    output_buffer->length = 0;
    if (!(tuple=PyTuple_New(3))){
        PyErr_SetString(PyExc_RuntimeError, 
                        "wrap: PyTuple_New()");
        return NULL;
    }
    
    qop_t = GSS_C_QOP_DEFAULT;
    major = gss_wrap(&minor, *ctx,conf_req_flag, qop_t, msg_buffer,&conf_state, output_buffer);
    PyTuple_SET_ITEM(tuple, 0,  Py_BuildValue("i", major));
    PyTuple_SET_ITEM(tuple, 1,  Py_BuildValue("i", minor));
    ptr = SWIG_NewPointerObj((void*)output_buffer, SWIGTYPE_gss_buffer_t,0);
    PyTuple_SET_ITEM(tuple, 2,  ptr);
    return tuple;
}

PyObject *
unwrap(gss_ctx_id_t * ctx, gss_buffer_t msg_buffer){
    OM_uint32 major, minor;
    gss_qop_t  qop_t;
    PyObject *tuple, *ptr;
    int conf_req_flag = 1;
    int conf_state = 1;
    gss_buffer_t output_buffer;
    output_buffer = (gss_buffer_t)globus_malloc(sizeof(gss_buffer_desc));
    output_buffer->value = NULL;
    output_buffer->length = 0;
    if (!(tuple=PyTuple_New(3))){
        PyErr_SetString(PyExc_RuntimeError, 
                        "wrap: PyTuple_New()");
        return NULL;
    }
    
    qop_t = GSS_C_QOP_DEFAULT;
    major = gss_unwrap(&minor, *ctx, msg_buffer, output_buffer, &conf_state, &qop_t);
    PyTuple_SET_ITEM(tuple, 0,  Py_BuildValue("i", major));
    PyTuple_SET_ITEM(tuple, 1,  Py_BuildValue("i", minor));
    ptr = SWIG_NewPointerObj((void*)output_buffer, SWIGTYPE_gss_buffer_t,0);
    PyTuple_SET_ITEM(tuple, 2,  ptr);
    return tuple;
}

PyObject *
set_buffer(PyObject * bufstring){
    PyObject * ptr;
    char * newstr;
    char * tmpstr;
    gss_buffer_t buf;    
    globus_size_t string_size;
    
    buf = (gss_buffer_t)globus_malloc(sizeof(gss_buffer_desc));
    string_size = PyString_Size(bufstring);
    newstr = (char *) globus_malloc(sizeof(char) * string_size );
    tmpstr = PyString_AsString(bufstring);
    memcpy(newstr, tmpstr,string_size);
    buf->value = newstr;
    buf->length = string_size;
    ptr = SWIG_NewPointerObj((void*)buf, SWIGTYPE_gss_buffer_t,0);
    return ptr;
}


PyObject *
get_buffer_string(gss_buffer_t  buf){
    PyObject *pystr;    
    pystr = PyString_FromStringAndSize(buf->value, buf->length);
    return pystr;
}

PyObject *
get_empty_buffer(){
    PyObject *ptr;
    gss_buffer_t  buf;
    buf = (gss_buffer_t)globus_malloc(sizeof(gss_buffer_desc));
    buf->value = NULL;
    buf->length = 0;
    ptr = SWIG_NewPointerObj((void *)buf,SWIGTYPE_gss_buffer_t,0);
    return ptr;

}


PyObject * 
free_buffer(gss_buffer_t buf){
    PyObject * tuple;
    OM_uint32 major_status;
    OM_uint32 minor_status;
    if (!(tuple=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, 
                        "free_buffer: PyTuple_New()");
        return NULL;
    }
    
    major_status =  gss_release_buffer(&minor_status, buf);
    globus_free(buf);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", major_status));
    PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("i", minor_status));
    return tuple;
}	

gss_buffer_t
get_no_buffer_const() {
    return GSS_C_NO_BUFFER;
}

/* JRB Argument minor_status is not used */
PyObject*
display_status(int major_status, int minor_status){
    PyObject* list;
    OM_uint32 maj, min, status_type, msgCtx;
    gss_buffer_desc status;
    msgCtx = 0;
    list = PyList_New(0);
    status_type = GSS_C_GSS_CODE;
    do {
	maj = gss_display_status(&min, major_status, status_type,
				 GSS_C_NULL_OID, &msgCtx, &status);
	PyList_Append(list, Py_BuildValue("s#", 
                                         (char*)status.value,
					  status.length));
	gss_release_buffer(&min, &status);
    } while (msgCtx != 0 || 
	(status_type == GSS_C_GSS_CODE && (status_type = GSS_C_MECH_CODE )));
    return list;
}

#ifdef CAS
 
PyObject*
inquire_context_by_oid(gss_ctx_id_t* ctx, PyObject* pyOID) {
    PyObject *tuple, *list;
    OM_uint32 major, minor;
    gss_buffer_set_t set;
    gss_OID_desc oid;
    if (!(tuple=PyTuple_New(3))) {
        PyErr_SetString(PyExc_RuntimeError, 
                        "inquire_context_by_oid: PyTuple_New()");
        return NULL;
    }
    PyString_AsStringAndSize(pyOID,
                             (char **) &(oid.elements),
                             (int *) &(oid.length));
    major = gss_inquire_sec_context_by_oid(&minor, *ctx, &oid, &set);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", major));
    PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("i", minor));
    if (major != GSS_S_COMPLETE) {
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 2, Py_None);
    }
    else {
        int i;
	list = PyList_New(0);
	for (i=0; i < set.count; i++) {
	    gss_buffer_desc buf = set->elements[i];
	    PyList_Append(list, Py_BuildValue("s#", 
					      (char*)buf.value,
					      buf.length));
	}
	PyTuple_SET_ITEM(tuple, 2, list);
    }
    return tuple;
} 

PyObject*
inquire_cred_by_oid(gss_cred_id_t* cred, PyObject* pyOID) {
    PyObject *tuple, *list;
    OM_uint32 major, minor;
    gss_buffer_set_t set;
    gss_OID_desc oid;
    if (!(tuple=PyTuple_New(3))) {
        PyErr_SetString(PyExc_RuntimeError, 
                        "inquire_cred_by_oid: PyTuple_New()");
        return NULL;
    }
    PyString_AsStringAndSize(pyOID,
                             (char **) &(oid.elements),
                             (int *) &(oid.length));
    major = gss_inquire_cred_by_oid(&minor, *cred, &oid, &set);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", major));
    PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("i", minor));
    if (major != GSS_S_COMPLETE) {
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 2, Py_None);
    }
    else {
        int i;
	list = PyList_New(0);
	for (i=0; i < set.count; i++) {
	    gss_buffer_desc buf = set->elements[i];
	    PyList_Append(list, Py_BuildValue("s#", 
					      (char*)buf.value,
					      buf.length));
	}
	PyTuple_SET_ITEM(tuple, 2, list);
    }
    return tuple;
}

PyObject*
accept_delegation(gss_ctx_id_t* ctx, PyObject* oids, 
		  PyObject* buffers, OM_uint32 time_req,
		  globus_io_handle_t* io_handle) {
    PyObject *item, *ptr;
    globus_result_t ret;
    OM_uint32 major, minor;
    OM_uint32 time_rec_buf;
    gss_OID_set_desc oidSet;
    gss_buffer_set_desc bufferSet;
    gss_buffer_t buf;
    gss_buffer_desc output_token;
    gss_buffer_desc input_token;
    gss_cred_id_t *cred;
    gss_OID oid, mech;
    globus_size_t read, lenSize, written;
    int oidsLen, i, bufsLen;
    long sendLen;
    long readLen;
    char *globus_error_string = NULL;

    cred = (gss_cred_id_t*)globus_malloc(sizeof(gss_cred_id_t));
    *cred = GSS_C_NO_CREDENTIAL;

    lenSize = sizeof(input_token.length);
    oidsLen = PyList_Size(oids);
    mech = NULL;
    if (oidsLen > 0)
    {
        oidSet.count = oidsLen;
        bufsLen = PyList_Size(buffers);
        bufferSet.count = bufsLen;
        oidSet.elements = (gss_OID_desc *)globus_malloc(sizeof(gss_OID_desc) *
                                                        oidsLen);
        for (i=0; i < oidsLen; i++) {
            oid = &(oidSet.elements[i]);
            item = PyList_GetItem(oids, i);
            PyString_AsStringAndSize(item,
                                     (char **) &(oid->elements),
                                     (int *) &(oid->length));
        }

        bufferSet.elements = (gss_buffer_desc*)globus_malloc(
            sizeof(gss_buffer_desc) * bufsLen);
        for (i=0; i < bufsLen; i++) {
            buf = &(bufferSet.elements[i]);
            item = PyList_GetItem(buffers, i);
            PyString_AsStringAndSize(item,
                                     (char **) &(buf->value),
                                     (int *) &(buf->length));
        }
    }
    else
    {
        oidSet.count = 0;
        oidSet.elements = NULL;
        bufferSet.count = 0;
        bufferSet.elements = NULL;
    }
    do {
	Py_BEGIN_ALLOW_THREADS
#ifdef DEBUG
        printf("Reading input token length\n");
#endif
	ret = globus_io_read(io_handle, (globus_byte_t*)&readLen,
			     sizeof(readLen), sizeof(readLen), &read);
	Py_END_ALLOW_THREADS
	if (!(ret == GLOBUS_SUCCESS)) {
            globus_error_string = globus_object_printable_to_string(globus_error_get(ret));
	    PyErr_SetString(PyExc_RuntimeError, globus_error_string);
            globus_libc_free(globus_error_string);
	    if (oidsLen > 0) {
		globus_free(oidSet.elements);
	    }
	    return NULL;
	}
	Py_BEGIN_ALLOW_THREADS
	input_token.length = ntohl(readLen);
#ifdef DEBUG
        printf("Reading %d byte input token\n", input_token.length);
#endif
	input_token.value = globus_malloc(input_token.length);
	ret = globus_io_read(io_handle, (globus_byte_t*)input_token.value,
			     input_token.length, input_token.length, &read);
	Py_END_ALLOW_THREADS
	if (!(ret == GLOBUS_SUCCESS)) {
	    globus_error_string = globus_object_printable_to_string(globus_error_get(ret));
	    PyErr_SetString(PyExc_RuntimeError, globus_error_string);
	    globus_libc_free(globus_error_string);
	    if (oidsLen > 0) {
		globus_free(oidSet.elements);
	    }
	    return NULL;
	}
	Py_BEGIN_ALLOW_THREADS
#ifdef DEBUG
        printf("Calling gss_accept_delegation()\n");
#endif
	major = gss_accept_delegation(&minor, *ctx, 
				      &oidSet, &bufferSet, &input_token,
				      time_req, &time_rec_buf,
				      cred, &mech, &output_token);
	Py_END_ALLOW_THREADS
	if (major != GSS_S_COMPLETE && major != GSS_S_CONTINUE_NEEDED) {
            globus_gss_assist_display_status_str(&globus_error_string,
                                                 "Accepting delegation",
                                                 major, minor, 0);
            gss_release_buffer(&minor, &input_token);
	    PyErr_SetString(PyExc_RuntimeError, globus_error_string);
	    if (oidsLen > 0) {
		globus_free(oidSet.elements);
	    }
	    return NULL;
	}
        gss_release_buffer(&minor, &input_token);
#ifdef DEBUG
        printf("gss_accept_delegation() returned %d\n", major);
#endif
        if (output_token.length > 0) 
        {
	    Py_BEGIN_ALLOW_THREADS
#ifdef DEBUG
            printf("Writing %d byte token\n", output_token.length);
#endif
            sendLen = htonl(output_token.length);
            ret = globus_io_write(io_handle, (globus_byte_t*)&sendLen,
                                  sizeof(sendLen), &written);
	    Py_END_ALLOW_THREADS
            if (!(ret == GLOBUS_SUCCESS)) {
                globus_error_string = globus_object_printable_to_string(globus_error_get(ret));
                PyErr_SetString(PyExc_RuntimeError, globus_error_string);
                globus_libc_free(globus_error_string);
		if (oidsLen > 0) {
		    globus_free(oidSet.elements);
		}
                return NULL;
            }
	    Py_BEGIN_ALLOW_THREADS
            ret = globus_io_write(io_handle, output_token.value, 
                                  output_token.length, &written);
	    Py_END_ALLOW_THREADS
            if (!(ret == GLOBUS_SUCCESS)) {
                globus_error_string = globus_object_printable_to_string(globus_error_get(ret));
                PyErr_SetString(PyExc_RuntimeError, globus_error_string);
                globus_libc_free(globus_error_string);
		if (oidsLen > 0) {
		    globus_free(oidSet.elements);
		}
                return NULL;
            }
            gss_release_buffer(&minor, &output_token);
        }
    } while (major == GSS_S_CONTINUE_NEEDED);
#ifdef DEBUG
    printf("Returning good credential %x\n", cred);
#endif
    ptr = SWIG_NewPointerObj((void*)cred, SWIGTYPE_p_gss_cred_id_t, 0);
    if (oidsLen > 0) {
	globus_free(oidSet.elements);
    }
    return ptr;
}

PyObject*
init_delegation(gss_ctx_id_t* ctx, gss_cred_id_t* cred, 
		PyObject* oids, PyObject* buffers,
		OM_uint32 time_req,
		globus_io_handle_t* io_handle) {
    PyObject *item;
    globus_result_t ret;
    OM_uint32 major, minor;
    gss_OID_set_desc oidSet;
    gss_buffer_set_desc bufferSet;
    gss_buffer_t buf;
    gss_buffer_desc output_token;
    gss_buffer_t input_token = GSS_C_NO_BUFFER;
    gss_buffer_desc input_token_buf;
    gss_OID oid;
    gss_OID mech = NULL;
    globus_size_t lenSize;
    globus_size_t written;
    globus_size_t read;
    int oidsLen, i, bufsLen;
    long sendLen;
    long readLen;
    char *globus_error_string;
    
    oidsLen = PyList_Size(oids);

    if (oidsLen > 0) 
    {
        oidSet.count = oidsLen;
        bufsLen = PyList_Size(buffers);
        bufferSet.count = bufsLen;
        oidSet.elements = (gss_OID_desc*)globus_malloc(sizeof(gss_OID_desc) *
                                                       oidsLen);
        for (i=0; i < oidsLen; i++) {
            oid = &(oidSet.elements[i]);
            item = PyList_GetItem(oids, i);
            PyString_AsStringAndSize(item,
                                     (char **) &(oid->elements),
                                     (int *) &(oid->length));
        }

        bufferSet.elements = (gss_buffer_desc*)globus_malloc(
            sizeof(gss_buffer_desc) * bufsLen);
        for (i=0; i < bufsLen; i++) {
            buf = &(bufferSet.elements[i]);
            item = PyList_GetItem(buffers, i);
            PyString_AsStringAndSize(item,
                                     (char **) &(buf->value),
                                     (int *) &(buf->length));
        }
    }
    else
    {
        oidSet.count = 0;
        oidSet.elements = NULL;
        bufferSet.count = 0;
        bufferSet.elements = NULL;
    }
    do {
	Py_BEGIN_ALLOW_THREADS
#ifdef DEBUG
        printf("DEBUG: Calling gss_init_delegation\n");
#endif
	major = gss_init_delegation(&minor, *ctx, *cred, mech,
				    &oidSet, &bufferSet, 
				    input_token, time_req, &output_token);
#ifdef DEBUG
        printf("DEBUG: gss_init_delegation returned %d\n", major);
#endif
	Py_END_ALLOW_THREADS
	if (major != GSS_S_COMPLETE && major != GSS_S_CONTINUE_NEEDED) {
            globus_gss_assist_display_status_str(&globus_error_string,
                                                 "Initing delegation",
                                                 major, minor, 0);
            gss_release_buffer(&minor, input_token);
	    PyErr_SetString(PyExc_RuntimeError, globus_error_string);
	    if (oidsLen > 0) {
		globus_free(oidSet.elements);
	    }
	    return NULL;
	}

        /* This call is ok if input_token == GSS_C_NO_BUFFER */
        gss_release_buffer(&minor, input_token);

        if (output_token.length > 0)
        {
	    Py_BEGIN_ALLOW_THREADS
#ifdef DEBUG
            printf("Writing %d byte token\n", output_token.length);
#endif
            sendLen = htonl(output_token.length);
            ret = globus_io_write(io_handle, (globus_byte_t*)&sendLen,
                                  sizeof(sendLen), &written);
	    Py_END_ALLOW_THREADS
            if (!(ret == GLOBUS_SUCCESS)) {
                globus_error_string = globus_object_printable_to_string(globus_error_get(ret));
                PyErr_SetString(PyExc_RuntimeError, globus_error_string);
                globus_libc_free(globus_error_string);
		if (oidsLen > 0) {
		    globus_free(oidSet.elements);
		}
                return NULL;
            }
	    Py_BEGIN_ALLOW_THREADS
            ret = globus_io_write(io_handle, output_token.value, 
                                  output_token.length, &written);
	    Py_END_ALLOW_THREADS
            if (!(ret == GLOBUS_SUCCESS)) {
                globus_error_string = globus_object_printable_to_string(globus_error_get(ret));
                PyErr_SetString(PyExc_RuntimeError, globus_error_string);
                globus_libc_free(globus_error_string);
		if (oidsLen > 0) {
		    globus_free(oidSet.elements);
		}
                return NULL;
            }
            gss_release_buffer(&minor, &output_token);
        }
        
	if (major == GSS_S_CONTINUE_NEEDED) {
	    Py_BEGIN_ALLOW_THREADS
	    ret = globus_io_read(io_handle, (globus_byte_t*)&readLen,
				 sizeof(readLen), sizeof(readLen), &read);
	    Py_END_ALLOW_THREADS
	    if (!(ret == GLOBUS_SUCCESS)) {
                globus_error_string = globus_object_printable_to_string(globus_error_get(ret));
                PyErr_SetString(PyExc_RuntimeError, globus_error_string);
                globus_libc_free(globus_error_string);
		if (oidsLen > 0) {
		    globus_free(oidSet.elements);
		}
                return NULL;
	    }
	    Py_BEGIN_ALLOW_THREADS
	    input_token_buf.length = ntohl(readLen);
#ifdef DEBUG
            printf("Reading %d byte input token\n", input_token_buf.length);
#endif
	    input_token_buf.value = globus_malloc(input_token_buf.length);  
	    ret = globus_io_read(io_handle,
                                 (globus_byte_t*)input_token_buf.value,
				 input_token_buf.length,
                                 input_token_buf.length,
                                 &read);
	    Py_END_ALLOW_THREADS
	    if (!(ret == GLOBUS_SUCCESS)) {
                globus_error_string = globus_object_printable_to_string(globus_error_get(ret));
                PyErr_SetString(PyExc_RuntimeError, globus_error_string);
                globus_libc_free(globus_error_string);
		if (oidsLen > 0) {
		    globus_free(oidSet.elements);
		}
                return NULL;
	    }
            input_token = &input_token_buf;
	}
    } while (major == GSS_S_CONTINUE_NEEDED);
    Py_INCREF(Py_None);
    return Py_None;
}

#endif /* CAS */

PyObject*
export_cred(gss_cred_id_t *cred,
            int option_req){
    OM_uint32 major;
    OM_uint32 minor;
    gss_OID p_desired_mech = NULL;
    gss_buffer_desc export_buffer;
    PyObject* tuple;

#ifdef DEBUG
    printf("Exporting cred %x (%x)\n", cred, *cred);
#endif

    if (!(tuple=PyTuple_New(3))) {
	PyErr_SetString(PyExc_RuntimeError, 
			"export_cred: PyTuple_New()");
	return NULL;
    }
    major = gss_export_cred(&minor,
                            *cred,
                            p_desired_mech,
                            option_req,
                            &export_buffer);

    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", major));
    PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("i", minor));

#ifdef DEBUG
    printf("Export done. major = %d\n", major);
#endif
    if (major == GLOBUS_SUCCESS){
        PyTuple_SET_ITEM(tuple, 2,
                         Py_BuildValue("s#", (char *) export_buffer.value,
                                       export_buffer.length));
        gss_release_buffer(&minor, &export_buffer);
    }
    else{
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 2, Py_None);
    }

    return tuple;
}
                
PyObject*
import_cred(int option_req, PyObject* credHandle, int time){    
    OM_uint32 major;
    OM_uint32 minor;
    OM_uint32 timeRec;
    gss_OID p_desired_mech = NULL;
    gss_buffer_t buf;
    PyObject* tuple;
    gss_cred_id_t* cred;
    PyObject* ptr;

    if (!(tuple=PyTuple_New(4))) {
	PyErr_SetString(PyExc_RuntimeError, 
			"import_cred: PyTuple_New()");
	return NULL;
    }
    cred = (gss_cred_id_t*)globus_malloc(sizeof(gss_cred_id_t));
    buf = (gss_buffer_desc*)globus_malloc(sizeof(gss_buffer_desc));
    buf->length = PyString_Size(credHandle);
    buf->value = PyString_AsString(credHandle);
    major = gss_import_cred(&minor,
                            cred,
                            p_desired_mech,
                            option_req,
                            buf,
                            (OM_uint32)time,
                            &timeRec);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", major));
    PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("i", minor));
    if (major == GLOBUS_SUCCESS){
        ptr = SWIG_NewPointerObj((void*)cred, SWIGTYPE_p_gss_cred_id_t, 0);
        PyTuple_SET_ITEM(tuple, 2, ptr);
        PyTuple_SET_ITEM(tuple, 3, Py_BuildValue("i", timeRec));
    }
    else{
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 2, Py_None);
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 3, Py_None);
    }
    globus_free(buf);
    return tuple;
}

PyObject*
inquire_cred(gss_cred_id_t* cred) {
    PyObject *tuple,*name_p;
    OM_uint32 major;
    OM_uint32 minor;
    gss_name_t *name;
    OM_uint32 life;
    gss_cred_usage_t cred_usage;
    gss_OID_set output_mechs;

    if (!(tuple=PyTuple_New(4))) {
        PyErr_SetString(PyExc_RuntimeError, 
                        "inquire_cred: PyTuple_New()");
        return NULL;
    }
    name = (gss_name_t*)globus_malloc(sizeof(gss_name_t));
    major = gss_inquire_cred(&minor, *cred, name, &life, 
                              &cred_usage, &output_mechs);

    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", major));
    PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("i", minor));
    if (major != GSS_S_COMPLETE) {
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 2, Py_None);
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 3, Py_None);
    }
    else {
      
	PyTuple_SET_ITEM(tuple, 2, Py_BuildValue("i", life));
	name_p = SWIG_NewPointerObj((void*)name, SWIGTYPE_p_gss_name_t, 0);
	PyTuple_SET_ITEM(tuple, 3, name_p);

    }
    return tuple;
}

PyObject *
assist_gridmap(char * globususerid){

    PyObject * tuple;
    char * useridp; 
    int ret;

    if (!(tuple=PyTuple_New(2))) {
        PyErr_SetString(PyExc_RuntimeError, 
                        "assist_gridmap:: PyTuple_New()");
        return NULL;
    }
    
    ret = globus_gss_assist_gridmap(globususerid, &useridp);
    if (ret != GLOBUS_SUCCESS){
	PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    else{
	PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", GLOBUS_SUCCESS));
	PyTuple_SET_ITEM(tuple,1, Py_BuildValue("s", useridp));
	globus_free(useridp);
    }
    return tuple;

}

int
assist_userok(char * globususerid, char * userid){
    int ret; 

    ret = globus_gss_assist_userok(globususerid, userid);
    return ret;
}

PyObject *
assist_map_local_user(char * local_user){
    PyObject * tuple;
    char * globusidp;
    int ret;
    
    if (!(tuple=PyTuple_New(2))) {
        PyErr_SetString(PyExc_RuntimeError, 
                        "assist_map_local_user:: PyTuple_New()");
        return NULL;
    }
    
    ret = globus_gss_assist_map_local_user(local_user, &globusidp);
    if (ret != GLOBUS_SUCCESS){
	PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    else{
	PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", GLOBUS_SUCCESS));
	PyTuple_SET_ITEM(tuple,1, Py_BuildValue("s", globusidp));
	globus_free(globusidp);
    }
    return tuple;

}

%}
