/*############################################################################
# Keith R. Jackson, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################*/
%module ioc
%{
#include "globus_common.h"
#include "globus_io.h"
#include "pyGlobus_common.h"
%}

%include ./src/pyGlobus-typemaps.i

typedef int globus_bool_t;

#ifdef CAS
typedef enum { 
    GLOBUS_IO_SECURE_AUTHENTICATION_MODE_NONE = 0,
    GLOBUS_IO_SECURE_AUTHENTICATION_MODE_GSSAPI = 1,
    GLOBUS_IO_SECURE_AUTHENTICATION_MODE_MUTUAL = 1,
    GLOBUS_IO_SECURE_AUTHENTICATION_MODE_ANONYMOUS = 2
} globus_io_secure_authentication_mode_t;
#else
typedef enum {
    GLOBUS_IO_SECURE_AUTHENTICATION_MODE_NONE = 0,
    GLOBUS_IO_SECURE_AUTHENTICATION_MODE_GSSAPI
} globus_io_secure_authentication_mode_t;
#endif /*CAS*/

typedef enum
{
    GLOBUS_IO_SECURE_AUTHORIZATION_MODE_NONE = 0,
    GLOBUS_IO_SECURE_AUTHORIZATION_MODE_SELF,
    GLOBUS_IO_SECURE_AUTHORIZATION_MODE_IDENTITY,
    GLOBUS_IO_SECURE_AUTHORIZATION_MODE_HOST,
    GLOBUS_IO_SECURE_AUTHORIZATION_MODE_CALLBACK
} globus_io_secure_authorization_mode_t;

typedef enum
{
    GLOBUS_IO_SECURE_CHANNEL_MODE_CLEAR = 0,
    GLOBUS_IO_SECURE_CHANNEL_MODE_GSI_WRAP,
    GLOBUS_IO_SECURE_CHANNEL_MODE_SSL_WRAP
} globus_io_secure_channel_mode_t;
   
typedef enum
{
    GLOBUS_IO_SECURE_PROTECTION_MODE_NONE = 0,
    GLOBUS_IO_SECURE_PROTECTION_MODE_SAFE,
    GLOBUS_IO_SECURE_PROTECTION_MODE_PRIVATE
} globus_io_secure_protection_mode_t;

typedef enum
{
    GLOBUS_IO_SECURE_DELEGATION_MODE_NONE = 0,
    GLOBUS_IO_SECURE_DELEGATION_MODE_LIMITED_PROXY,
    GLOBUS_IO_SECURE_DELEGATION_MODE_FULL_PROXY
} globus_io_secure_delegation_mode_t;

%init %{
    PyEval_InitThreads();
    python_interp = PyThreadState_Get()->interp;
    init_globus_thread_state_key(&python_thread_state_key);
%}

%{
static globus_thread_key_t python_thread_state_key;
static PyInterpreterState *python_interp; 

static globus_bool_t PythonAuthorizationCallback(void* user_data, 
						 globus_io_handle_t* handle,
						 globus_result_t rslt, 
						 char* identity,
						 gss_ctx_id_t* context) {
    PyObject *func, *arglist, *result, *arg, *ptr, *ptr2, *err;
    globus_bool_t ret;
    callbackStruct* cbStruct = (callbackStruct*)user_data;
    if (rslt != GLOBUS_SUCCESS) {
	return GLOBUS_FALSE;
    }
    PyEval_AcquireThread(get_python_thread_state(python_thread_state_key, python_interp));
    func = cbStruct->callback;
    arg = cbStruct->arg;
    ptr = SWIG_NewPointerObj((void*)handle, SWIGTYPE_p_globus_io_handle_t, 0);
    ptr2 = SWIG_NewPointerObj((void*)context, SWIGTYPE_p_gss_ctx_id_t, 0);
    arglist = Py_BuildValue("(OOsO)", arg, ptr, identity, ptr2);
    result = PyEval_CallObject(func, arglist);
    if (result == NULL) {
      PyErr_Print();
    }

    Py_DECREF(ptr);
    Py_DECREF(ptr2);
    Py_DECREF(arglist);
    Py_XDECREF(result);
    ret = (int)PyInt_AsLong(result);
    PyEval_ReleaseThread(get_python_thread_state(python_thread_state_key, python_interp));
    return ret;
}

static void PythonIOCallback(void* user_data, globus_io_handle_t* handle,
			      globus_result_t rslt) {
    PyObject *func, *arglist, *result, *arg, *ptr, *tuple,*err;
    callbackStruct* cbStruct = (callbackStruct*)user_data;
    PyEval_AcquireThread(get_python_thread_state(python_thread_state_key, python_interp));
    func = cbStruct->callback;
    arg = cbStruct->arg;
    ptr = SWIG_NewPointerObj((void*)handle, SWIGTYPE_p_globus_io_handle_t, 0);
    if(!(tuple = PyTuple_New(2))){
    	PyErr_SetString(PyExc_RuntimeError,"PythonIOCallback, PyTuple_New");	
    }
    else{
	PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", rslt));
	if (rslt != GLOBUS_SUCCESS) {
            char *tmpstr;
            tmpstr = globus_object_printable_to_string(globus_error_get(rslt));
            PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
            globus_libc_free(tmpstr);
        }
	else{
            Py_INCREF(Py_None);
            PyTuple_SET_ITEM(tuple, 1, Py_None);
	}
    }	
    if ((err = PyErr_Occurred()) != NULL){
	PyErr_Print();
    } 	
    arglist = Py_BuildValue("(OOO)", arg, ptr, tuple);
    result = PyEval_CallObject(func, arglist);
    if (result == NULL) {
      PyErr_Print();
    }
    Py_DECREF(ptr);
    Py_DECREF(tuple);
    Py_DECREF(arglist);
    Py_XDECREF(result);
    PyEval_ReleaseThread(get_python_thread_state(python_thread_state_key, python_interp));
    return;
}
 
static void PythonReadCallback(void* user_data, globus_io_handle_t* handle,
			       globus_result_t rslt,
			       globus_byte_t* buf, globus_size_t nbytes) {
    PyObject *func, *arglist, *result, *arg, *buffer, *ptr, *tuple, *err;
    callbackStruct* cbStruct = (callbackStruct*)user_data;
    PyEval_AcquireThread(get_python_thread_state(python_thread_state_key, python_interp));
    func = cbStruct->callback;
    arg = cbStruct->arg;
    ptr = SWIG_NewPointerObj((void*)handle, SWIGTYPE_p_globus_io_handle_t, 0);
    if(!(tuple = PyTuple_New(2))){
    	PyErr_SetString(PyExc_RuntimeError,"PythonReadCallback, PyTuple_New");	
    }
    else{
	PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", rslt));
	if (rslt != GLOBUS_SUCCESS) {
            char *tmpstr;
            tmpstr = globus_object_printable_to_string(globus_error_get(rslt));
            PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
            globus_libc_free(tmpstr);
        }
	else{
            Py_INCREF(Py_None);
            PyTuple_SET_ITEM(tuple, 1, Py_None);
	}
    }	
    if ((err = PyErr_Occurred()) != NULL){
	PyErr_Print();
    } 		
    buffer = PyBuffer_FromReadWriteMemory((void*)buf, nbytes*sizeof(globus_byte_t));
    arglist = Py_BuildValue("(OOOOi)", arg, ptr, tuple, buffer, nbytes);
    result = PyEval_CallObject(func, arglist);
    if (result == NULL) {
      PyErr_Print();
    }
    Py_DECREF(ptr);
    Py_DECREF(tuple);
    Py_DECREF(buffer);
    Py_DECREF(arglist);
    Py_XDECREF(result);
    PyEval_ReleaseThread(get_python_thread_state(python_thread_state_key, python_interp));
    return;
}

static void PythonRecvCallback(void* user_data, globus_io_handle_t* handle,
			       globus_result_t rslt, globus_byte_t* buf, 
			       globus_size_t nbytes, int flags) {
    PyObject *func, *arglist, *result, *arg, *buffer, *ptr, *tuple, *err;
    callbackStruct* cbStruct = (callbackStruct*)user_data;
    PyEval_AcquireThread(get_python_thread_state(python_thread_state_key, python_interp));
    func = cbStruct->callback;
    arg = cbStruct->arg;
    ptr = SWIG_NewPointerObj((void*)handle, SWIGTYPE_p_globus_io_handle_t, 0);
    if(!(tuple = PyTuple_New(2))){
    	PyErr_SetString(PyExc_RuntimeError,"PythonRecvCallback, PyTuple_New");	
    }
    else{
	PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", rslt));
	if (rslt != GLOBUS_SUCCESS) {
            char *tmpstr;
            tmpstr = globus_object_printable_to_string(globus_error_get(rslt));
            PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
            globus_libc_free(tmpstr);
        }
	else{
            Py_INCREF(Py_None);
            PyTuple_SET_ITEM(tuple, 1, Py_None);
	}
    }	
     if ((err = PyErr_Occurred()) != NULL){
	PyErr_Print();
    } 
    buffer = PyBuffer_FromReadWriteMemory((void*)buf, nbytes*sizeof(globus_byte_t));
    arglist = Py_BuildValue("(OOOOii)", arg, ptr, tuple, buffer, nbytes, flags);
    result = PyEval_CallObject(func, arglist);
    if (result == NULL) {
      PyErr_Print();
    }
    Py_DECREF(ptr);
    Py_DECREF(tuple);
    Py_DECREF(buffer);
    Py_DECREF(arglist);
    Py_XDECREF(result);
    PyEval_ReleaseThread(get_python_thread_state(python_thread_state_key, python_interp));
    return;
}

static void PythonWriteCallback(void* user_data, globus_io_handle_t* handle,
				globus_result_t rslt,
				globus_byte_t* buf, globus_size_t nbytes) {
    PyObject *func, *arglist, *result, *arg, *buffer, *ptr, *tuple, *err;
    callbackStruct* cbStruct = (callbackStruct*)user_data;
    PyEval_AcquireThread(get_python_thread_state(python_thread_state_key, python_interp));
    func = cbStruct->callback;
    arg = cbStruct->arg;
    ptr = SWIG_NewPointerObj((void*)handle, SWIGTYPE_p_globus_io_handle_t, 0);
    if(!(tuple = PyTuple_New(2))){
    	PyErr_SetString(PyExc_RuntimeError,"PythonWriteCallback, PyTuple_New");	
    }
    else{
	PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", rslt));
	if (rslt != GLOBUS_SUCCESS) {
            char *tmpstr;
            tmpstr = globus_object_printable_to_string(globus_error_get(rslt));
            PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
            globus_libc_free(tmpstr);
        }
	else{
            Py_INCREF(Py_None);
            PyTuple_SET_ITEM(tuple, 1, Py_None);
	}
    }	
    if ((err = PyErr_Occurred()) != NULL){
	PyErr_Print();
    } 
    buffer = PyBuffer_FromReadWriteMemory((void*)buf, nbytes*sizeof(globus_byte_t));
    arglist = Py_BuildValue("(OOOOi)", arg, ptr, tuple, buffer, nbytes);
    result = PyEval_CallObject(func, arglist);
    if (result == NULL) {
      PyErr_Print();
    }
    Py_DECREF(ptr);
    Py_DECREF(tuple);
    Py_DECREF(buffer);
    Py_DECREF(arglist);
    Py_XDECREF(result);
    PyEval_ReleaseThread(get_python_thread_state(python_thread_state_key, python_interp));
    return;
}
%}


%inline %{
globus_module_descriptor_t* 
get_module(void) {
    return GLOBUS_IO_MODULE;
}
gss_cred_id_t
get_no_cred_const() {
    return GSS_C_NO_CREDENTIAL;
}
 
PyObject*
file_open(char* path, int flags, int mode, globus_io_attr_t* attr) {
    globus_result_t ret;
    PyObject* tuple, *ptr;
    char* tmpstr;
    globus_io_handle_t* handle;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "file_open: PyTuple_New()");
        return NULL;
    }
    handle = (globus_io_handle_t*)globus_libc_malloc(sizeof(globus_io_handle_t));
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_file_open(path, flags, mode, attr, handle);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
        globus_libc_free(handle);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        ptr = SWIG_NewPointerObj((void*)handle, SWIGTYPE_p_globus_io_handle_t, 0);
        PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

PyObject*
fileattr_init() {
    globus_result_t ret;
    PyObject* tuple, *ptr;
    char* tmpstr;
    globus_io_attr_t* attr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "fileattr_init: PyTuple_New()");
        return NULL;
    }
    attr = (globus_io_attr_t*)globus_libc_malloc(sizeof(globus_io_attr_t));
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_fileattr_init(attr);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
        globus_libc_free(attr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        ptr = SWIG_NewPointerObj((void*)attr, SWIGTYPE_p_globus_io_attr_t, 0);
        PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

PyObject*
fileattr_destroy(globus_io_attr_t* attr) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "fileattr_destroy: PyTuple_New()");
        return NULL;
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_fileattr_destroy(attr);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    globus_libc_free(attr);
    return tuple;
}

PyObject*
io_attr_set_file_type(globus_io_attr_t* attr, int file_type) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    globus_io_file_type_t type;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "io_attr_set_file_type: PyTuple_New()");
        return NULL;
    }
    if (file_type == GLOBUS_IO_FILE_TYPE_TEXT) {
	type = GLOBUS_IO_FILE_TYPE_TEXT;
    } else {
	type = GLOBUS_IO_FILE_TYPE_BINARY;
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_attr_set_file_type(attr, type);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject*
tcpattr_init() {
    globus_result_t ret;
    PyObject* tuple, *ptr;
    char* tmpstr;
    globus_io_attr_t* attr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "tcpattr_init: PyTuple_New()");
        return NULL;
    }
    attr = (globus_io_attr_t*)globus_libc_malloc(sizeof(globus_io_attr_t));
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_tcpattr_init(attr);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
        globus_libc_free(attr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        ptr = SWIG_NewPointerObj((void*)attr, SWIGTYPE_p_globus_io_attr_t, 0);
        PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

PyObject*
tcpattr_destroy(globus_io_attr_t* attr) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "tcpattr_destroy: PyTuple_New()");
        return NULL;
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_tcpattr_destroy(attr);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    globus_libc_free(attr);
    return tuple;
}


PyObject*
attr_set_tcp_restrict_port(globus_io_attr_t* attr, globus_bool_t restrct) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "attr_set_tcp_restrict_port: PyTuple_New()");
        return NULL;
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_attr_set_tcp_restrict_port(attr, restrct);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
        globus_libc_free(attr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple; 
}

PyObject*
attr_get_tcp_restrict_port(globus_io_attr_t* attr, globus_bool_t* boolRet) {
    globus_result_t ret;
    PyObject* list;
    char* tmpstr;
    list = PyList_New(0);
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_attr_get_tcp_restrict_port(attr, boolRet);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyList_Append(list, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyList_Append(list, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyList_Append(list, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyList_Append(list, Py_None);
    }
    return list;
} 

PyObject*
attr_set_socket_reuseaddr(globus_io_attr_t* attr, globus_bool_t reuse) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "attr_set_socket_reuseaddr: PyTuple_New()");
        return NULL;
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_attr_set_socket_reuseaddr(attr, reuse);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject*
attr_get_socket_reuseaddr(globus_io_attr_t* attr, globus_bool_t* boolRet) {
    globus_result_t ret;
    PyObject* list;
    char* tmpstr;
    list = PyList_New(0);
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_attr_get_socket_reuseaddr(attr, boolRet);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyList_Append(list, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyList_Append(list, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyList_Append(list, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyList_Append(list, Py_None);
    }
    return list;
}

PyObject*
attr_set_socket_keepalive(globus_io_attr_t* attr, globus_bool_t keepAlive) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "attr_set_socket_keepalive: PyTuple_New()");
        return NULL;
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_attr_set_socket_keepalive(attr, keepAlive);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject*
attr_get_socket_keepalive(globus_io_attr_t* attr, globus_bool_t* boolRet) {
    globus_result_t ret;
    PyObject* list;
    char* tmpstr;
    list = PyList_New(0);
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_attr_get_socket_keepalive(attr, boolRet);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyList_Append(list, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyList_Append(list, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyList_Append(list, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyList_Append(list, Py_None);
    }
    return list;
}

PyObject*
attr_set_socket_linger(globus_io_attr_t* attr, 
		       globus_bool_t linger, int lingerTime) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "attr_set_socket_linger: PyTuple_New()");
        return NULL;
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_attr_set_socket_linger(attr, linger, lingerTime);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject*
attr_get_socket_linger(globus_io_attr_t* attr, 
		       globus_bool_t* boolRet, int* intRet) {
    globus_result_t ret;
    PyObject* list;
    char* tmpstr;
    list = PyList_New(0);
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_attr_get_socket_linger(attr, boolRet, intRet);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyList_Append(list, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyList_Append(list, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyList_Append(list, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyList_Append(list, Py_None);
    }
    return list;
}

PyObject*
attr_set_socket_oobinline(globus_io_attr_t* attr, globus_bool_t oobinline) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "attr_set_socket_oobinline: PyTuple_New()");
        return NULL;
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_attr_set_socket_oobinline(attr, oobinline);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject*
attr_get_socket_oobinline(globus_io_attr_t* attr, globus_bool_t* boolRet) {
    globus_result_t ret;
    PyObject* list;
    char* tmpstr;
    list = PyList_New(0);
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_attr_get_socket_oobinline(attr, boolRet);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyList_Append(list, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyList_Append(list, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyList_Append(list, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyList_Append(list, Py_None);
    }
    return list;
}

PyObject*
attr_set_socket_sndbuf(globus_io_attr_t* attr, int sndBuf) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "attr_set_socket_sndbuf: PyTuple_New()");
        return NULL;
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_attr_set_socket_sndbuf(attr, sndBuf);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject*
attr_get_socket_sndbuf(globus_io_attr_t* attr, int* intRet) {
    globus_result_t ret;
    PyObject* list;
    char* tmpstr;
    list = PyList_New(0);
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_attr_get_socket_sndbuf(attr, intRet);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyList_Append(list, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyList_Append(list, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyList_Append(list, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyList_Append(list, Py_None);
    }
    return list;
}

PyObject*
attr_set_socket_rcvbuf(globus_io_attr_t* attr, int rcvBuf) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "attr_set_socket_rcvbuf: PyTuple_New()");
        return NULL;
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_attr_set_socket_rcvbuf(attr, rcvBuf);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject*
attr_get_socket_rcvbuf(globus_io_attr_t* attr, int* intRet) {
    globus_result_t ret;
    PyObject* list;
    char* tmpstr;
    list = PyList_New(0);
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_attr_get_socket_rcvbuf(attr, intRet);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyList_Append(list, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyList_Append(list, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyList_Append(list, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyList_Append(list, Py_None);
    }
    return list;
}

PyObject*
attr_set_tcp_nodelay(globus_io_attr_t* attr, globus_bool_t nodelay) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "attr_set_tcp_nodelay: PyTuple_New()");
        return NULL;
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_attr_set_tcp_nodelay(attr, nodelay);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject*
attr_get_tcp_nodelay(globus_io_attr_t* attr, globus_bool_t* boolRet) {
    globus_result_t ret;
    PyObject* list;
    char* tmpstr;
    list = PyList_New(0);
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_attr_get_tcp_nodelay(attr, boolRet);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyList_Append(list, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyList_Append(list, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyList_Append(list, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyList_Append(list, Py_None);
    }
    return list;
}

PyObject*
attr_set_tcp_interface(globus_io_attr_t* attr, char* iface) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "attr_set_tcp_interface: PyTuple_New()");
        return NULL;
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_attr_set_tcp_interface(attr, iface);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject*
attr_get_tcp_interface(globus_io_attr_t* attr, char** charRet) {
    globus_result_t ret;
    PyObject* list;
    char* tmpstr;
    list = PyList_New(0);
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_attr_get_tcp_interface(attr, charRet);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyList_Append(list, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyList_Append(list, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyList_Append(list, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyList_Append(list, Py_None);
    }
    return list;
}

PyObject*
attr_set_secure_authentication_mode(globus_io_attr_t* attr,
				    globus_io_secure_authentication_mode_t mode,
				    gss_cred_id_t *cred) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "attr_set_secure_authentication_mode: PyTuple_New()");
        return NULL;
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_attr_set_secure_authentication_mode(attr, mode, *cred);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
} 

PyObject*
attr_get_secure_authentication_mode(globus_io_attr_t* attr) {
    globus_result_t ret;
    PyObject* tuple, *ptr;
    char* tmpstr;
    globus_io_secure_authentication_mode_t mode;
    gss_cred_id_t* credPtr = (gss_cred_id_t*)malloc(sizeof(gss_cred_id_t));
    if (!(tuple=PyTuple_New(3)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "attr_get_secure_authentication_mode: PyTuple_New()");
        return NULL;
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_attr_get_secure_authentication_mode(attr, &mode, credPtr);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 2, Py_None);
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
	PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("i", mode));
        ptr = SWIG_NewPointerObj((void*)credPtr, SWIGTYPE_p_gss_cred_id_t, 0);
        PyTuple_SET_ITEM(tuple, 2, ptr);
    }
    return tuple;
}

PyObject*
attr_set_secure_authorization_mode(globus_io_attr_t* attr,
				   globus_io_secure_authorization_mode_t mode,
				   globus_io_secure_authorization_data_t* data) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "attr_set_secure_authorization_mode: PyTuple_New()");
        return NULL;
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_attr_set_secure_authorization_mode(attr, mode, data);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
} 

PyObject*
attr_get_secure_authorization_mode(globus_io_attr_t* attr,
				   globus_io_secure_authorization_data_t* data) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    globus_io_secure_authorization_mode_t mode;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "attr_get_secure_authorization_mode: PyTuple_New()");
        return NULL;
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_attr_get_secure_authorization_mode(attr, &mode, data);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
	PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("i", mode));
    }
    return tuple;
}

PyObject*
authorization_data_init() {
    globus_result_t ret;
    PyObject* tuple, *ptr;
    char* tmpstr;
    globus_io_secure_authorization_data_t* data;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "authorization_data_init: PyTuple_New()");
        return NULL;
    }
    data = (globus_io_secure_authorization_data_t*)
	globus_malloc(sizeof(globus_io_secure_authorization_data_t));
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_secure_authorization_data_initialize(data);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        ptr = SWIG_NewPointerObj((void*)data, 
				 SWIGTYPE_p_globus_io_secure_authorization_data_t, 0);
        PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}
 
PyObject*
authorization_data_destroy(globus_io_secure_authorization_data_t* data) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "authorization_data_destroy: PyTuple_New()");
        return NULL;
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_secure_authorization_data_destroy(data);
    Py_END_ALLOW_THREADS
    globus_free(data);
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject*
authorization_data_set_identity(globus_io_secure_authorization_data_t* data,
				char* identity) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "authorization_data_set_identity: PyTuple_New()");
        return NULL;
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_secure_authorization_data_set_identity(data, identity);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
} 

PyObject*
authorization_data_get_identity(globus_io_secure_authorization_data_t* data,
				char** charRet) {
    globus_result_t ret;
    PyObject* list;
    char* tmpstr;
    list = PyList_New(0);
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_secure_authorization_data_get_identity(data, charRet);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyList_Append(list, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyList_Append(list, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyList_Append(list, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyList_Append(list, Py_None);
    }
    return list;
} 

PyObject*
authorization_data_set_callback(globus_io_secure_authorization_data_t* data,
				PyObject* pyfunc, PyObject* arg) {
    globus_result_t ret;
    PyObject *list, *cbPtr;
    char* tmpstr;
    callbackStruct* cbStruct;
    list = PyList_New(0);
    cbStruct = (callbackStruct*)globus_malloc(sizeof(callbackStruct));
    Py_XINCREF(pyfunc);
    Py_XINCREF(arg);
    cbStruct->callback = pyfunc;
    cbStruct->arg = arg;
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_secure_authorization_data_set_callback(data, 
				  (globus_io_secure_authorization_callback_t)PythonAuthorizationCallback, cbStruct);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
	PyList_Append(list, Py_BuildValue("i", -1));
	tmpstr = globus_object_printable_to_string(globus_error_get(ret));
	PyList_Append(list, Py_BuildValue("s", tmpstr));
	globus_libc_free(tmpstr);
	globus_libc_free(cbStruct);
    } else {
	cbPtr = SWIG_NewPointerObj((void*)cbStruct, SWIGTYPE_p_callbackStruct, 0);
	PyList_Append(list, Py_BuildValue("i", ret));
	PyList_Append(list, cbPtr);
    }
    
    return list;
}

PyObject*
attr_set_secure_channel_mode(globus_io_attr_t* attr,
			     globus_io_secure_channel_mode_t mode) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "attr_set_secure_channel_mode: PyTuple_New()");
        return NULL;
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_attr_set_secure_channel_mode(attr, mode);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
} 

PyObject*
attr_get_secure_channel_mode(globus_io_attr_t* attr) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    globus_io_secure_channel_mode_t mode;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "attr_get_secure_channel_mode: PyTuple_New()");
        return NULL;
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_attr_get_secure_channel_mode(attr, &mode);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
	PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("i", mode));
    }
    return tuple;
}

PyObject*
attr_set_secure_protection_mode(globus_io_attr_t* attr,
				globus_io_secure_protection_mode_t mode) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "attr_set_secure_protection_mode: PyTuple_New()");
        return NULL;
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_attr_set_secure_protection_mode(attr, mode);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
} 

PyObject*
attr_get_secure_protection_mode(globus_io_attr_t* attr) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    globus_io_secure_protection_mode_t mode;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "attr_get_secure_protection_mode: PyTuple_New()");
        return NULL;
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_attr_get_secure_protection_mode(attr, &mode);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
	PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("i", mode));
    }
    return tuple;
}

PyObject*
attr_set_secure_delegation_mode(globus_io_attr_t* attr,
				globus_io_secure_delegation_mode_t mode) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "attr_set_secure_delegation_mode: PyTuple_New()");
        return NULL;
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_attr_set_secure_delegation_mode(attr, mode);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
} 

PyObject*
attr_get_secure_delegation_mode(globus_io_attr_t* attr) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    globus_io_secure_delegation_mode_t mode;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "attr_get_secure_delegation_mode: PyTuple_New()");
        return NULL;
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_attr_get_secure_delegation_mode(attr, &mode);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
	PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("i", mode));
    }
    return tuple;
}

PyObject*
tcp_register_connect(char* host, int port, 
		     globus_io_attr_t* attr, PyObject* pyfunc, 
		     PyObject* arg) {
    globus_result_t ret;
    PyObject *list, *ptr, *ptr2;
    char* tmpstr;
    globus_io_handle_t* handle;
    callbackStruct* cbStruct;
    handle = (globus_io_handle_t*)globus_malloc(sizeof(globus_io_handle_t));
    cbStruct = (callbackStruct*)globus_malloc(sizeof(callbackStruct));
    Py_XINCREF(pyfunc);
    Py_XINCREF(arg);
    cbStruct->callback = pyfunc;
    cbStruct->arg = arg;
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_tcp_register_connect(host, port, attr, 
					 PythonIOCallback,
					 cbStruct, handle);
    Py_END_ALLOW_THREADS
    list = PyList_New(0);
    if (ret != GLOBUS_SUCCESS) 
    {
        PyList_Append(list, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyList_Append(list, Py_BuildValue("s", tmpstr));
	Py_INCREF(Py_None);
	PyList_Append(list, Py_None);
        globus_libc_free(tmpstr);
	globus_libc_free(cbStruct);
    }
    else 
    {
        ptr = SWIG_NewPointerObj((void*)cbStruct, SWIGTYPE_p_callbackStruct, 0);
        ptr2 = SWIG_NewPointerObj((void*)handle, SWIGTYPE_p_globus_io_handle_t, 0);
        PyList_Append(list, Py_BuildValue("i", ret));
        PyList_Append(list, ptr);
        PyList_Append(list, ptr2);
    }
    
    return list;
} 

PyObject*
tcp_connect(char* host, int port, 
	    globus_io_attr_t* attr) {
    globus_result_t ret;
    PyObject* tuple, *ptr;
    char* tmpstr;
    globus_io_handle_t* handle;
    handle = (globus_io_handle_t*)globus_malloc(sizeof(globus_io_handle_t));
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "tcp_connect: PyTuple_New()");
        return NULL;
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_tcp_connect(host, port, attr, handle);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        ptr = SWIG_NewPointerObj((void*)handle, SWIGTYPE_p_globus_io_handle_t, 0);
        PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

PyObject*
tcp_create_listener(unsigned short port, int backlog, globus_io_attr_t* attr) {
    globus_result_t ret;
    PyObject* tuple, *ptr;
    char* tmpstr;
    globus_io_handle_t* handle;
    handle = (globus_io_handle_t*)globus_malloc(sizeof(globus_io_handle_t));
    if (!(tuple=PyTuple_New(3)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "tcp_create_listener: PyTuple_New()");
        return NULL;
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_tcp_create_listener(&port, backlog, attr, handle);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 2, Py_None);
        globus_libc_free(tmpstr);
        globus_libc_free(handle);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("i", port));
        ptr = SWIG_NewPointerObj((void*)handle, SWIGTYPE_p_globus_io_handle_t, 0);
        PyTuple_SET_ITEM(tuple, 2, ptr);
    }
    return tuple;
}

PyObject*
tcp_register_accept(globus_io_handle_t* listenerHandle, 
		    globus_io_attr_t* attr, 
		    PyObject* pyfunc, PyObject* arg) {
    globus_result_t ret;
    PyObject *list, *ptr, *ptr2;
    globus_io_handle_t* newHandle;
    callbackStruct* cbStruct;
    cbStruct = (callbackStruct*)globus_malloc(sizeof(callbackStruct));
    newHandle = (globus_io_handle_t*)globus_malloc(sizeof(globus_io_handle_t));
    Py_XINCREF(pyfunc);
    Py_XINCREF(arg);
    cbStruct->callback = pyfunc;
    cbStruct->arg = arg;
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_tcp_register_accept(listenerHandle, attr, 
					newHandle, 
					PythonIOCallback, cbStruct);
    Py_END_ALLOW_THREADS
    ptr = SWIG_NewPointerObj((void*)cbStruct, SWIGTYPE_p_callbackStruct, 0);
    ptr2 = SWIG_NewPointerObj((void*)newHandle, SWIGTYPE_p_globus_io_handle_t, 0); 
    list = PyList_New(0);
    PyList_Append(list, Py_BuildValue("i", ret));
    PyList_Append(list, ptr);
    PyList_Append(list, ptr2);
    return list;
} 

PyObject*
tcp_accept(globus_io_handle_t* listenerHandle, 
	   globus_io_attr_t* attr) {
    globus_result_t ret;
    PyObject* tuple, *ptr;
    char* tmpstr;
    globus_io_handle_t* newHandle;
    newHandle = (globus_io_handle_t*)globus_malloc(sizeof(globus_io_handle_t));
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "tcp_accept: PyTuple_New()");
        return NULL;
    }
    
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_tcp_accept(listenerHandle, attr, newHandle);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        ptr = SWIG_NewPointerObj((void*)newHandle, 
				 SWIGTYPE_p_globus_io_handle_t, 0);
	PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

PyObject*
tcp_register_listen(globus_io_handle_t* handle, 
		    PyObject* pyfunc, PyObject* arg) {
    globus_result_t ret;
    PyObject *list, *ptr;
    callbackStruct* cbStruct;
    cbStruct = (callbackStruct*)globus_malloc(sizeof(callbackStruct));
    Py_XINCREF(pyfunc);
    Py_XINCREF(arg);
    cbStruct->callback = pyfunc;
    cbStruct->arg = arg;
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_tcp_register_listen(handle,
					PythonIOCallback, cbStruct);
    Py_END_ALLOW_THREADS
    ptr = SWIG_NewPointerObj((void*)cbStruct, SWIGTYPE_p_callbackStruct, 0);
    list = PyList_New(0);
    PyList_Append(list, Py_BuildValue("i", ret));
    PyList_Append(list, ptr);
    return list;
} 

PyObject*
tcp_listen(globus_io_handle_t* handle) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "tcp_listen: PyTuple_New()");
        return NULL;
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_tcp_listen(handle);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject*
tcp_get_local_address(globus_io_handle_t* handle) {
    globus_result_t ret;
    PyObject* list;
    char* tmpstr;
    char* hostStr;
    int host[4];
    unsigned short ushortRet;
    list = PyList_New(0);
    hostStr = (char*)globus_malloc(32*sizeof(char));
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_tcp_get_local_address(handle, host, &ushortRet);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyList_Append(list, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyList_Append(list, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyList_Append(list, Py_BuildValue("i", 0));
        sprintf(hostStr, "%d.%d.%d.%d", host[0], host[1], host[2], host[3]);
        PyList_Append(list, Py_BuildValue("s", hostStr));
        PyList_Append(list, Py_BuildValue("i", ushortRet));
    }
    return list;
}

PyObject*
tcp_get_remote_address(globus_io_handle_t* handle) {
    globus_result_t ret;
    PyObject* list;
    char* tmpstr;
    char* hostStr;
    int host[4];
    unsigned short ushortRet;
    list = PyList_New(0);
    hostStr = (char*)globus_malloc(32*sizeof(char));
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_tcp_get_remote_address(handle, host, &ushortRet);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyList_Append(list, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyList_Append(list, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyList_Append(list, Py_BuildValue("i", 0));
        sprintf(hostStr, "%d.%d.%d.%d", host[0], host[1], host[2], host[3]);
        PyList_Append(list, Py_BuildValue("s", hostStr));
        PyList_Append(list, Py_BuildValue("i", ushortRet));
        globus_free(hostStr);
    }
    return list;
}

PyObject*
tcp_get_security_context(globus_io_handle_t* handle) {
    globus_result_t ret;
    PyObject *tuple, *ptr;
    char* tmpstr;
    gss_ctx_id_t* context;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "tcp_get_security_context: PyTuple_New()");
        return NULL;
    }
    context = (gss_ctx_id_t*)globus_malloc(sizeof(gss_ctx_id_t));
    ret = globus_io_tcp_get_security_context(handle, context);
    if (ret != GLOBUS_SUCCESS) {
	PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
	ptr = SWIG_NewPointerObj((void*)context, SWIGTYPE_p_gss_ctx_id_t, 0);
	PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}
 
PyObject*
tcp_get_delegated_credential(globus_io_handle_t* handle) {
    globus_result_t ret;
    PyObject *tuple, *ptr;
    char* tmpstr;
    gss_cred_id_t* cred;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "tcp_get_security_context: PyTuple_New()");
        return NULL;
    }
    cred = (gss_cred_id_t*)globus_malloc(sizeof(gss_cred_id_t));
    ret = globus_io_tcp_get_delegated_credential(handle, cred);
    if (ret != GLOBUS_SUCCESS) {
	PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
	ptr = SWIG_NewPointerObj((void*)cred, SWIGTYPE_p_gss_cred_id_t, 0);
	PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

PyObject*
io_register_close(globus_io_handle_t* handle, 
		  PyObject* pyfunc, PyObject* arg) {
    globus_result_t ret;
    PyObject *list, *ptr;
    callbackStruct* cbStruct;
    cbStruct = (callbackStruct*)globus_malloc(sizeof(callbackStruct));
    Py_XINCREF(pyfunc);
    Py_XINCREF(arg);
    cbStruct->callback = pyfunc;
    cbStruct->arg = arg;
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_register_close(handle, PythonIOCallback, cbStruct);
    Py_END_ALLOW_THREADS
    ptr = SWIG_NewPointerObj((void*)cbStruct, SWIGTYPE_p_callbackStruct, 0);
    list = PyList_New(0);
    PyList_Append(list, Py_BuildValue("i", ret));
    PyList_Append(list, ptr);
    return list;
} 

PyObject*
io_close(globus_io_handle_t* handle) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "io_close: PyTuple_New()");
        return NULL;
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_close(handle);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject*
io_cancel(globus_io_handle_t* handle, globus_bool_t performCallbacks) {
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "io_close: PyTuple_New()");
        return NULL;
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_cancel(handle, performCallbacks);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject*
io_register_read(globus_io_handle_t* handle, globus_byte_t* buffer,
		 int maxBytes, int waitForBytes,
		 PyObject* pyfunc, PyObject* arg) {
    globus_result_t ret;
    PyObject *tuple, *ptr;
    char* tmpstr;
    callbackStruct* cbStruct;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "io_register_read: PyTuple_New()");
        return NULL;
    }
    cbStruct = (callbackStruct*)globus_malloc(sizeof(callbackStruct));
    Py_XINCREF(pyfunc);
    Py_XINCREF(arg);
    cbStruct->callback = pyfunc;
    cbStruct->arg = arg;
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_register_read(handle, buffer, maxBytes, waitForBytes,
				  PythonReadCallback, cbStruct);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
        globus_libc_free(cbStruct);
    }
    else {
        ptr = SWIG_NewPointerObj((void*)cbStruct, SWIGTYPE_p_callbackStruct, 0);
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

PyObject*
io_try_read(globus_io_handle_t* handle, globus_byte_t* buffer,
	    int maxBytes) {
    globus_result_t ret;
    PyObject *tuple;
    char* tmpstr;
    globus_size_t bytesRead;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "io_try_read: PyTuple_New()");
        return NULL;
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_try_read(handle, buffer, maxBytes, &bytesRead);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("i", bytesRead));
    }
    return tuple;
}

PyObject*
io_read(globus_io_handle_t* handle, globus_byte_t* buffer,
	int maxBytes, int waitForBytes) {
    globus_result_t ret;
    PyObject *tuple;
    char* tmpstr;
    globus_size_t bytesRead;
    globus_object_t *err;

    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "io_read: PyTuple_New()");
        return NULL;
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_read(handle, buffer, maxBytes, waitForBytes, &bytesRead);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
	err = globus_error_get(ret);
	if (globus_object_get_type(err) == GLOBUS_IO_ERROR_TYPE_EOF) {
	    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
	    PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("i", bytesRead));
	}
	else {
	    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
	    tmpstr = globus_object_printable_to_string(err);
	    PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
	    globus_libc_free(tmpstr);
	}
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("i", bytesRead));
    }
    return tuple;
}

PyObject*
io_register_write(globus_io_handle_t* handle, PyObject* buffer,
		  int nBytes, PyObject* pyfunc, PyObject* arg) {
    globus_result_t ret;
    PyObject *tuple, *ptr;
    char* tmpstr;
    globus_byte_t* buf;
    callbackStruct* cbStruct;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "io_register_write: PyTuple_New()");
        return NULL;
    }
    cbStruct = (callbackStruct*)globus_malloc(sizeof(callbackStruct));
    Py_XINCREF(pyfunc);
    Py_XINCREF(arg);
    cbStruct->callback = pyfunc;
    cbStruct->arg = arg;
    buf = (globus_byte_t*)PyString_AsString(buffer);
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_register_write(handle, buf, nBytes,
				   PythonWriteCallback, cbStruct);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
        globus_libc_free(cbStruct);
    }
    else {
        ptr = SWIG_NewPointerObj((void*)cbStruct, SWIGTYPE_p_callbackStruct, 0);
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

PyObject*
io_try_write(globus_io_handle_t* handle, PyObject* buffer,
	    int maxBytes) {
    globus_result_t ret;
    PyObject *tuple;
    char* tmpstr;
    globus_byte_t* buf;
    globus_size_t bytesWritten;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "io_try_write: PyTuple_New()");
        return NULL;
    }
    buf = (globus_byte_t*)PyString_AsString(buffer);
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_try_write(handle, buf, maxBytes, &bytesWritten);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("i", bytesWritten));
    }
    return tuple;
}

PyObject*
io_write(globus_io_handle_t* handle, PyObject* buffer,
	 int nBytes) {
    globus_result_t ret;
    PyObject *tuple;
    globus_byte_t* buf;
    char* tmpstr;
    globus_size_t bytesWritten;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "io_write: PyTuple_New()");
        return NULL;
    }
    buf = (globus_byte_t*)PyString_AsString(buffer);
    Py_BEGIN_ALLOW_THREADS
    ret = globus_io_write(handle, buf, nBytes, &bytesWritten);
    Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("i", bytesWritten));
    }
    return tuple;
}

void
free_handle(globus_io_handle_t* handle) {
    globus_libc_free(handle);
    return;
}
 
void
free_callback(callbackStruct* cbStruct) {
    Py_XDECREF(cbStruct->callback);
    Py_XDECREF(cbStruct->arg);
    globus_free(cbStruct);
}

void
dummy(gss_ctx_id_t* ctx, void* f, gss_cred_id_t* cred) {
    return;
} 

PyObject *
io_try_send(globus_io_handle_t* handle, PyObject* buffer, int flags, int maxBytes){
   globus_result_t ret;
   PyObject * tuple;
   char * tmpstr;
   globus_byte_t* buf;
   globus_size_t bytesSent;
   if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "io_try_send: PyTuple_New()");
        return NULL;
    }
   buf = (globus_byte_t*)PyString_AsString(buffer);
   Py_BEGIN_ALLOW_THREADS
      ret = globus_io_try_send(handle,buf,maxBytes,flags,&bytesSent); 
   Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else
    {
       PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
       PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("i", bytesSent));
    }
   return tuple;

}
PyObject *
io_send(globus_io_handle_t* handle, PyObject* buffer, int flags, int maxBytes){
   globus_result_t ret;
   PyObject * tuple;
   globus_byte_t* buf;
   char *tmpstr;
   globus_size_t bytesSent;
   if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "io_send: PyTuple_New()");
        return NULL;
    }
   buf = (globus_byte_t*)PyString_AsString(buffer);
   Py_BEGIN_ALLOW_THREADS
      ret = globus_io_send(handle,buf,maxBytes,flags,&bytesSent); 
   Py_END_ALLOW_THREADS
    if (ret != GLOBUS_SUCCESS) {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else
    {
       PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
       PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("i", bytesSent));
    }
   return tuple;
}
   

%}
