/*############################################################################
# Keith Jackson, LBNL
# See Copyright for copyright notice!
###########################################################################*/
%module myProxyc
%{
#include "globus_gsi_proxy.h"
#include "globus_gsi_credential.h"
#include "myproxy.h"
#include "myproxy_delegation.h"
#include <string.h>
%}

%include ./src/pyGlobus-typemaps.i

%{
/*Helper function for get_info */
void
print_info(myproxy_creds_t *creds,PyObject * py_info_dict )
{
    
    int ret = 0; 
    char proxyString[1024];
    for (; creds; creds = creds->next) {
        time_t time_diff = 0, now = 0;
        long days = 0;
	
       
        if (creds->owner_name)
	{  
	    ret = PyDict_SetItemString(py_info_dict,"owner", PyString_FromString(creds->owner_name));
	}
   
	if (creds->username)
	{
	    PyDict_SetItemString(py_info_dict, "username", PyString_FromString(strdup(creds->username)));
	}
    
	if (creds->credname)
	{
	    PyDict_SetItemString(py_info_dict, "name", PyString_FromString(creds->credname));
	}
	
        if (creds->creddesc)   
	{
	    PyDict_SetItemString(py_info_dict, "description", PyString_FromString(creds->creddesc));
	}
	    
        if (creds->retrievers)
	{
	    PyDict_SetItemString(py_info_dict, "retrieval policy", PyString_FromString(creds->retrievers));
	}
        if (creds->renewers)
	{
	    PyDict_SetItemString(py_info_dict, "renewal policy", PyString_FromString(creds->renewers));
	}
        if (creds->lockmsg)
	{
	    PyDict_SetItemString(py_info_dict, "locked", PyString_FromString(creds->lockmsg));
        }
	 
	now = time(0);
        if (creds->end_time > now) {
            time_diff = creds->end_time - now;
            days = time_diff / 86400.0;
        }
	
        sprintf(proxyString,"%ld:%02ld:%02ld",
               (long)(time_diff / 3600),
               (long)(time_diff % 3600) / 60,
               (long)time_diff % 60 );
        
	PyDict_SetItemString(py_info_dict, "Days Left", PyInt_FromLong(days));
	PyDict_SetItemString(py_info_dict, "Time Left", PyString_FromString(proxyString));
	
    }
 
}
#define SECONDS_PER_HOUR 3600
%}

%inline %{ 

int 
set_passphrase(myproxy_request_t * client_request, PyObject * py_passwd){ 
   int len = 0;
   PyObject * err;
   char * passwd_string = NULL;

   if (PyString_Check(py_passwd)){
       passwd_string = PyString_AsString(py_passwd); 
       len = PyString_Size(py_passwd);
       if((err = PyErr_Occurred()) !=NULL){
            PyErr_Print();
            return -1; 
       }

    }
   if (len > MAX_PASS_LEN){
       fprintf(stderr, "The passphrase was too long\n");
       return -1;
       }
       
   strncpy(client_request->passphrase,passwd_string, len);
   return 0;
}
 
int
get_delegation(myproxy_socket_attrs_t *socket_attrs,
	       myproxy_request_t *client_request, 
	       myproxy_response_t  *server_response,
               char *hostname, int port, 
	       char *certfile, char *outfile,
	       int lifetime, char *username, int interactive) {
   
    int len;
    int ret;
   
    client_request->proxy_lifetime = 60*60*lifetime;
    socket_attrs->pshost = strdup(hostname);
    socket_attrs->psport = port;
    client_request->username = strdup(username);
    
    if (interactive){
        len = myproxy_read_passphrase(client_request->passphrase, sizeof(client_request->passphrase),NULL);
    }
    
    client_request->credname = NULL;
    client_request->retrievers = NULL;
    /* The myproxy_get_delegation function signature changed between
    versions 1.17 and 2.x.x. The 2.x.x versions have a MYPROXY_VERSION_MAJOR
    that is 2, the earlier versions do not correspond to the packaging
    version so this MAJOR version is equal to 0.
    */
    #if MYPROXY_VERSION_MAJOR 
    ret = myproxy_get_delegation(socket_attrs, client_request,
                               NULL, server_response, outfile);
    #else
    ret = myproxy_get_delegation(socket_attrs, client_request,
                                 server_response, outfile);
    #endif 
    if (ret !=0) {
	return 1;
    }

    return 0;
}
PyObject *
myproxy_init()
{
    PyObject * ptr1, * ptr2, *ptr3, *tuple;
    myproxy_socket_attrs_t *socket_attrs;
    myproxy_request_t *client_request;
    myproxy_response_t *server_response;

    if (!(tuple=PyTuple_New(3)))
    {
        PyErr_SetString(PyExc_RuntimeError, "myproxy_init: PyTuple_New()");
        return NULL;
    }
    socket_attrs = malloc(sizeof(*socket_attrs));
    memset(socket_attrs, 0, sizeof(*socket_attrs));

    client_request = malloc(sizeof(*client_request));
    memset(client_request, 0, sizeof(*client_request));

    server_response = malloc(sizeof(*server_response));
    memset(server_response, 0, sizeof(*server_response));
    
    ptr1 = SWIG_NewPointerObj((void*)socket_attrs, 
			      SWIGTYPE_p_myproxy_socket_attrs_t, 0);

    ptr2 = SWIG_NewPointerObj((void*)client_request, 
			      SWIGTYPE_p_myproxy_request_t, 0);

    ptr3 = SWIG_NewPointerObj((void*)server_response, 
			      SWIGTYPE_p_myproxy_response_t, 0);
     
    PyTuple_SET_ITEM(tuple, 0, ptr1);
    PyTuple_SET_ITEM(tuple, 1, ptr2);
    PyTuple_SET_ITEM(tuple, 2, ptr3);
    return tuple;
}

void 
myproxy_destroy(myproxy_socket_attrs_t *socket_attrs,myproxy_request_t * client_request
		, myproxy_response_t * server_response)
{
    
     myproxy_free(socket_attrs, client_request, server_response);
}




int
proxy_init(myproxy_socket_attrs_t *socket_attrs, 
	     myproxy_request_t * client_request, 
	     myproxy_response_t * server_response, 
	     char * pshost, int port, int lifetime, 
	     char * proxyfile, char * username)
{
    int cred_lifetime, hours;
    float days;
    char request_buffer[1024];
    int requestlen;
    int rval;
    int ret;
    myproxy_log_use_stream (stderr);
   
   
    /* setup defaults */
    client_request->version = malloc(strlen(MYPROXY_VERSION) + 1);
    strcpy(client_request->version, MYPROXY_VERSION);
    client_request->command_type = MYPROXY_PUT_PROXY;

    socket_attrs->pshost = strdup(pshost);
   
    /* client_request stores the lifetime of proxies delegated by the server */
    client_request->proxy_lifetime = SECONDS_PER_HOUR * MYPROXY_DEFAULT_DELEG_HOURS;

    
    socket_attrs->psport = port;
    client_request->username = strdup(username);
    

    /* Allow user to provide a passphrase */ 
    rval = myproxy_read_verified_passphrase(client_request->passphrase, sizeof(    client_request->passphrase), NULL);

    if (rval == -1) 
    {
	fprintf(stderr, "%s\n", verror_get_string());
    }
    

    /* Set up client socket attributes */
    if (myproxy_init_client(socket_attrs) < 0) {
        fprintf(stderr, "%s\n",
                verror_get_string());
        
    }

    /* Authenticate client to server */
    if (myproxy_authenticate_init(socket_attrs, proxyfile) < 0) {
        fprintf(stderr, "%s\n",
                verror_get_string());
        
    }

    /* Serialize client request object */
    requestlen = myproxy_serialize_request(client_request,
                                           request_buffer, sizeof(request_buffer));
    if (requestlen < 0) {
        fprintf(stderr, "%s\n",verror_get_string());
        
    }

    /* Send request to the myproxy-server */
        if (myproxy_send(socket_attrs, request_buffer, requestlen) < 0) {
        fprintf(stderr, "%s\n",
                verror_get_string());
        
    }

    /* Continue unless the response is not OK */
    if (myproxy_recv_response(socket_attrs, server_response) != 0) {
        fprintf(stderr, "%s\n", verror_get_string());
        
    }
   
    /* Delegate credentials to server using the default lifetime of the cert. */
    if (myproxy_init_delegation(socket_attrs, proxyfile, lifetime,
                                NULL /* no passphrase */) < 0) {
        fprintf(stderr, "%s\n",
                verror_get_string());
       
    }


    /* Get final response from server */
    if (myproxy_recv_response(socket_attrs, server_response) != 0) {
        fprintf(stderr, "%s\n", verror_get_string());
        
    }
    
    hours = (int)(lifetime/SECONDS_PER_HOUR);
    days = (float)(hours/24.0);
    printf("A proxy valid for %d hours (%.1f days) for user %s now exists on %s.\n",
           hours, days, client_request->username, socket_attrs->pshost);

   
    return 0;

}


PyObject *
proxy_info(myproxy_socket_attrs_t *socket_attrs,
	   myproxy_request_t * client_request, 
	   myproxy_response_t * server_response, 
	   char *hostname, int port, char * username, 
	   int dn_as_username)
{

    char * response_buffer;
    char request_buffer[1024];
    int requestlen, responselen;
    PyObject *tuple, *py_info_dict;
    char * tmpstr;
  
    
    
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "proxy_info: PyTuple_New()");
        return NULL;
    }
    strncpy(client_request->passphrase, "DUMMY-PASSPHRASE",
            sizeof(client_request->passphrase));

    client_request->command_type = MYPROXY_INFO_PROXY;
    /* Set up client socket attributes */
    socket_attrs->pshost = strdup(hostname);
    if (myproxy_init_client(socket_attrs) < 0) {
	tmpstr =  verror_get_string();
	PyTuple_SET_ITEM(tuple, 0,  Py_BuildValue("i",1));
	PyTuple_SET_ITEM(tuple, 1,  Py_BuildValue("s",tmpstr));
        return tuple;
    }

    /* Authenticate client to server */
    if (myproxy_authenticate_init(socket_attrs, NULL /* Default proxy */) < 0) {
	tmpstr =  verror_get_string();
	PyTuple_SET_ITEM(tuple, 0,  Py_BuildValue("i",1));
	PyTuple_SET_ITEM(tuple, 1,  Py_BuildValue("s",tmpstr));
        return tuple;
    }
    if (client_request->username == NULL) { /* set default username */
        if (dn_as_username) {
	    char *username = NULL;
            if (ssl_get_base_subject_file(NULL,
                                          &username)) 
	    {
		PyTuple_SET_ITEM(tuple, 0,  Py_BuildValue("i",1));
                PyTuple_SET_ITEM(tuple, 1,  Py_BuildValue("s","Cannot get subject name from your certificate\n"));
                return tuple;
            }
        }
        client_request->username = strdup(username);
     }
    /* Serialize client request object */
    requestlen = myproxy_serialize_request(client_request,
                                           request_buffer, sizeof(request_buffer));

    if (requestlen < 0) {
        fprintf(stderr, "Error: %s", verror_get_string());
        exit(1);
    }
        /* Send request to the myproxy-server */
    if (myproxy_send(socket_attrs, request_buffer, requestlen) < 0) {
	tmpstr =  verror_get_string();
	PyTuple_SET_ITEM(tuple, 0,  Py_BuildValue("i",1));
	PyTuple_SET_ITEM(tuple, 1,  Py_BuildValue("s",tmpstr));
        return tuple;
    }

    /* Receive a response from the server */
    responselen = myproxy_recv_ex(socket_attrs, &response_buffer);
    if (responselen < 0) {
        tmpstr =  verror_get_string();
	PyTuple_SET_ITEM(tuple, 0,  Py_BuildValue("i",1));
	PyTuple_SET_ITEM(tuple, 1,  Py_BuildValue("s",tmpstr));
        return tuple;

    }

    /* Make a response object from the response buffer */

    if (myproxy_deserialize_response(server_response, response_buffer, responselen) < 0) {
        fprintf(stderr, "%s\n",verror_get_string());
        exit(1);
    }
    free(response_buffer);
    response_buffer = NULL;

    /* Check version */
    if (strcmp(server_response->version, MYPROXY_VERSION) != 0) {
        fprintf(stderr, "Invalid version number received from server\n");
        exit(1);
    }

    /* Check response */
    switch(server_response->response_type) {
    case MYPROXY_ERROR_RESPONSE:
        fprintf(stderr, "Received ERROR_RESPONSE: %s\n",
                server_response->error_string);
        exit(1);
        break;
    case MYPROXY_OK_RESPONSE:
	py_info_dict = PyDict_New();
	PyDict_SetItemString(py_info_dict, "username", PyString_FromString(client_request->username));
	print_info(server_response->info_creds,py_info_dict ); 
        break;
    default:
        fprintf(stderr, "Invalid response type received.\n");
        exit(1);
        break;
    }
    
    PyTuple_SET_ITEM(tuple, 0,  Py_BuildValue("i",0));
    PyTuple_SET_ITEM(tuple, 1, py_info_dict);
    return tuple;
}


PyObject *
proxy_destroy(myproxy_socket_attrs_t *socket_attrs,
	   myproxy_request_t * client_request, 
	   myproxy_response_t * server_response, 
	   char *hostname, int port, char * username, 
	   int dn_as_username)
{
    char request_buffer[1024], *response_buffer = NULL;
    int requestlen, responselen;
    PyObject *tuple;
    char *tmpstr;

     if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "proxy_destroy: PyTuple_New()");
        return NULL;
    }
    client_request->command_type = MYPROXY_DESTROY_PROXY;
    socket_attrs->pshost = strdup(hostname);
    client_request->proxy_lifetime = 0;
        strncpy(client_request->passphrase, "DUMMY-PASSPHRASE",
            sizeof(client_request->passphrase));

    /* Set up client socket attributes */
    if (myproxy_init_client(socket_attrs) < 0) {
	PyTuple_SET_ITEM(tuple, 0,  Py_BuildValue("i",1));
	PyTuple_SET_ITEM(tuple, 1,  Py_BuildValue("s","error in myproxy_init_client():"));
       
        return tuple;
    }

    /* Authenticate client to server */
    if (myproxy_authenticate_init(socket_attrs, NULL /* Default proxy */) < 0) {
        PyTuple_SET_ITEM(tuple, 0,  Py_BuildValue("i",1));
	PyTuple_SET_ITEM(tuple, 1,  Py_BuildValue("s","error in myproxy_authenticate_init(): "));
	return tuple;
    }

    if (client_request->username == NULL) { /* set default username */
        if (dn_as_username) {
	    char *username = NULL;
            if (ssl_get_base_subject_file(NULL,
                                          &username)) {
		PyTuple_SET_ITEM(tuple, 0,  Py_BuildValue("i",1));
		PyTuple_SET_ITEM(tuple, 1,  Py_BuildValue("s","Cannot get subject name from your certificate\n"));
                return tuple;
            }
        }
        client_request->username = strdup(username);
     }

    /* Serialize client request object */
    requestlen = myproxy_serialize_request(client_request,
                                           request_buffer, sizeof(request_buffer));

    if (requestlen < 0) {
	PyTuple_SET_ITEM(tuple, 0,  Py_BuildValue("i",1));
        PyTuple_SET_ITEM(tuple, 1,  Py_BuildValue("s","error in myproxy_serialize_request()\n"));
        return tuple;
    }

    /* Send request to the myproxy-server */
    if (myproxy_send(socket_attrs, request_buffer, requestlen) < 0) {
	PyTuple_SET_ITEM(tuple, 0,  Py_BuildValue("i",1));
	PyTuple_SET_ITEM(tuple, 1,  Py_BuildValue("s","error in myproxy_send_request(): %s,\n"));
        return tuple;
    }
        /* Receive a response from the server */
    responselen = myproxy_recv_ex(socket_attrs, &response_buffer);
    if (responselen < 0) {
	PyTuple_SET_ITEM(tuple, 0,  Py_BuildValue("i",1));
	tmpstr =  verror_get_string();
	PyTuple_SET_ITEM(tuple, 1,  Py_BuildValue("s",tmpstr));
        return tuple;
    }

    /* Make a response object from the response buffer */
    if (myproxy_deserialize_response(server_response, response_buffer, responselen) < 0) {
	PyTuple_SET_ITEM(tuple, 0,  Py_BuildValue("i",1));
        PyTuple_SET_ITEM(tuple, 1,  Py_BuildValue("s", "error in myproxy_deserialize_response()\n"));
        return tuple;
    }
    free(response_buffer);
    response_buffer = NULL;

    /* Check version */
    if (strcmp(server_response->version, MYPROXY_VERSION) != 0) {
	PyTuple_SET_ITEM(tuple, 0,  Py_BuildValue("i",1));
	PyTuple_SET_ITEM(tuple, 1,  Py_BuildValue("s","Invalid version number received from server\n"));
        return tuple;
    }

    /* Check response */
    switch(server_response->response_type) {
    case MYPROXY_ERROR_RESPONSE:
        tmpstr = server_response->error_string;
	PyTuple_SET_ITEM(tuple, 0,  Py_BuildValue("i",1));
	PyTuple_SET_ITEM(tuple, 1,  Py_BuildValue("s",tmpstr));
        return tuple;
    case MYPROXY_OK_RESPONSE:
        if (client_request->credname) {
            printf("MyProxy credential '%s' for user %s was successfully removed.\n",
                   client_request->credname, client_request->username);
        } else {
            printf("Default MyProxy credential for user %s was successfully removed.\n",
                   client_request->username);
        }
        break;
    default:
	PyTuple_SET_ITEM(tuple, 0,  Py_BuildValue("i",1));
	PyTuple_SET_ITEM(tuple, 1,  Py_BuildValue("s","Invalid response type received.\n"));
        return tuple;
    }
    
    PyTuple_SET_ITEM(tuple, 0,  Py_BuildValue("i",0));
    Py_INCREF(Py_None);
    PyTuple_SET_ITEM(tuple, 1, Py_None);
    return tuple;
}

PyObject *
set_delegation(myproxy_socket_attrs_t *socket_attrs,myproxy_request_t * client_request)
{
    PyObject * ptr1, * ptr2, *tuple;
    int ret;

    if (!(tuple=PyTuple_New(3)))
    {
        PyErr_SetString(PyExc_RuntimeError, "myproxy_init: PyTuple_New()");
        return NULL;
    }  

    ret = myproxy_set_delegation_defaults(socket_attrs, client_request);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i",ret));
    if (ret != 0)
    {
	PyTuple_SET_ITEM(tuple,1, Py_BuildValue("s", "Could not set delegation_defaults"));
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple,2, Py_None);
    }
    else
    {
	ptr1 = SWIG_NewPointerObj((void*)socket_attrs, 
			      SWIGTYPE_p_myproxy_socket_attrs_t, 0);
	
	ptr2 = SWIG_NewPointerObj((void*)client_request, 
			      SWIGTYPE_p_myproxy_request_t, 0);
	PyTuple_SET_ITEM(tuple, 1, ptr1);
	PyTuple_SET_ITEM(tuple, 2, ptr2);
    }
    return tuple;
	
} 

%}
