/*############################################################################
# Matt Rodriguez, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################*/
%module proxyc
%{
#undef SIZEOF_OFF_T
#undef HAVE_LIMITS_H
#undef socklen_t
#include "globus_gsi_credential.h"
#include "globus_gsi_proxy.h"
#include "globus_gsi_cert_utils.h"
#include "globus_gsi_system_config.h"
#include "globus_gsi_system_config_constants.h"
#include "globus_openssl.h" 
%}

%include ./src/pyGlobus-typemaps.i

typedef enum globus_gsi_cert_utils_cert_type_e
{
    GLOBUS_GSI_CERT_UTILS_TYPE_EEC,
    GLOBUS_GSI_CERT_UTILS_TYPE_CA,
    GLOBUS_GSI_CERT_UTILS_TYPE_GSI_3_IMPERSONATION_PROXY,
    GLOBUS_GSI_CERT_UTILS_TYPE_GSI_3_INDEPENDENT_PROXY,
    GLOBUS_GSI_CERT_UTILS_TYPE_GSI_3_LIMITED_PROXY,
    GLOBUS_GSI_CERT_UTILS_TYPE_GSI_3_RESTRICTED_PROXY,
    GLOBUS_GSI_CERT_UTILS_TYPE_GSI_2_PROXY,
    GLOBUS_GSI_CERT_UTILS_TYPE_GSI_2_LIMITED_PROXY
} globus_gsi_cert_utils_cert_type_t;

%{
static int
globus_i_gsi_proxy_utils_clear_and_remove(
    char *                              filename,
    int                                 flag) 
{
    int                                 f;
    int                                 rec;
    int                                 left;
    long                                size;
    char                                msg[65] 
        = "Destroyed by globus_proxy_destroy\r\n";

    if (flag)
        fprintf(stderr, "Would remove %s\n", filename ? filename : "(null)");
    else
    {
        f = open(filename, O_RDWR);
        if (f) 
        {
            size = lseek(f, 0L, SEEK_END);
            lseek(f, 0L, SEEK_SET);
            if (size > 0) 
            {
                rec = size / 64;
                left = size - rec * 64;
                while (rec)
                {
                    write(f, msg, 64);
                    rec--;
                }
                if (left) 
                    write(f, msg, left);
            }
            close(f);
        }
        remove(filename);
    }
    return 0;
}

static int
globus_i_gsi_proxy_utils_pwstdin_callback(
    char *                              buf, 
    int                                 num, 
    int                                 w)
{
    int                                 i;

    if (!(fgets(buf, num, stdin))) {
        fprintf(stderr, "Failed to read pass-phrase from stdin\n");
        return -1;
    }
    i = strlen(buf);
    if (buf[i-1] == '\n') {
        buf[i-1] = '\0';
        i--;
    }
    return i;       

}

static void
globus_i_gsi_proxy_utils_key_gen_callback(int p, int n, void * dummy)
{
    char c='B';


    if (p == 0) c='.';
    if (p == 1) c='+';
    if (p == 2) c='*';
    if (p == 3) c='\n';
    fputc(c, stdout);
    fflush(stdout);
}

struct pwcb_data
 {
     PyObject *passphrase;
     PyObject *error;
 };

 static globus_thread_key_t pwcb_thread_key;

 /*
  * Sigh. Thread-specific storage support for password callbacks.
  * Need this because the globus password callback interface doesn't
  * support passing an argument.
  */
 static void init_pwcb_key()
 {
      int rc;
 
      rc = globus_thread_key_create(&pwcb_thread_key, NULL);
      if (rc != 0)
      {
        fprintf(stderr, "Error creating pwcb_thread_key: %d\n", rc);
      }
  }

static int
pwcb_set_data(PyObject *passphrase, char **err_string)
  {
      struct pwcb_data *d;
      int rc;
 
      *err_string = 0;
 
      d = (struct pwcb_data *) globus_malloc(sizeof(struct pwcb_data));
      if (d == 0)
      {
        *err_string = "pwcb_set_data(): globus_malloc failed";
        return GLOBUS_FAILURE;
      }
      d->passphrase = passphrase;
 
      rc = globus_thread_setspecific(pwcb_thread_key, (void *) d);
      if (rc != 0)
      {
        static char errbuf[128];
        snprintf(errbuf, sizeof(errbuf), "pwcb_set_data(): globus_thread_setspecific failed with %d", rc);
        *err_string = errbuf;
        return GLOBUS_FAILURE;
      }
 
      return GLOBUS_SUCCESS;
  }
  /*
   * Get the thread-specific callback data.
   * Return 0 for success, nonzero for failure.
   * Fills in the arguments from the registered cb data and deletes
   * that data.
   */
static int pwcb_get_data(struct pwcb_data **data)
  {
      struct pwcb_data *d;
      int rc;
 
      d = (struct pwcb_data *) globus_thread_getspecific(pwcb_thread_key);
 
      if (d == 0)
      {
        printf("pwcb_get_data(): No password data registered\n");
        *data = 0;
 
        return -1;
      }
 
      *data = d;
      return 0;
  }
static int pwcb_remove_data()
  {
      int rc;
      struct pwcb_data *d;
 
      d = (struct pwcb_data *) globus_thread_getspecific(pwcb_thread_key);
 
      if (d == 0)
      {
        printf("pwcb_get_data(): No password data registered\n");
 
        return -1;
      }
 
      rc = globus_thread_setspecific(pwcb_thread_key, (void *) 0);
      if (rc != 0)
      {
        printf("pwcb_get_data(): globus_thread_setspecific (reset to null) failed with %d\n", rc);
        return -1;
      }
 
      globus_free(d);
      return 0;
  }

static int convertPassphrase(PyObject *pp, char *buf, int len)
  {
      if (PyString_Check(pp))
      {
	  int ssiz, nchars;
	  ssiz = PyString_Size(pp);
 
	  nchars = ssiz > len ? len : ssiz;
	  strncpy(buf, PyString_AsString(pp), nchars);
	  return nchars;
      }
      
      /*
       * not a string, check for list of integers.
       */
      
      if (PyList_Check(pp))
      {
	  /*
	   * It's a list. Scan once to determine if all are valid ints.
	   */
	  
	  int i, n;
 
        n = PyList_Size(pp);
 
        for (i = 0; i < n; i++  )
        {
            PyObject *d;
            d = PyList_GetItem(pp, i);
 
            /*
             * Handle list-of-chars too.
             */
 
           if (PyString_Check(d))
            {
                if (PyString_Size(d) != 1)
                {
                    PyErr_SetString(PyExc_RuntimeError, "convertPassphrase: list-style passphrase contains a more than single-char string");
                    return 0;
                }
            }
            else if (!PyInt_Check(d))
            {
                PyErr_SetString(PyExc_RuntimeError, "convertPassphrase: list-style passphrase contains a non-integer");
                return 0;
            }
        }
 
        /*
         * We're okay. Create the passphrase from the list.
         *
         * Check length. Be paranoid and leave a char for zero-term.
         *
         */
 
        if (n > (len - 1))
            n = len - 1;
 
        for (i = 0; i < n; i++  )
        {
            PyObject *d;
            long val = 0;
            d = PyList_GetItem(pp, i);
 
            if (PyInt_Check(d))
            {
                val = PyInt_AsLong(d);
            }

            else if (PyString_Check(d))
            {
                char *s;
                s = PyString_AsString(d);
                val = s[0];
            }
            buf[i] = (char) val;
            /*
             * We're done with the list, scrub it.
             */
            PyList_SetItem(pp, i, PyInt_FromLong(0));
            val = 0;
        }
 
 
        buf[n] = '\0';
 
        return n;
      }
 
      /*
       * Otherwise, this wasn't a valid passphrase.
       */
 
      PyErr_SetString(PyExc_RuntimeError, "convertPassphrase: Invalid passphrase type");
      return 0;
  }
static int invokePythonCallback(PyObject *cb, char *buf, int len, int rwflag)
{
      PyObject *arglist, *ret;
 
      if (!PyCallable_Check(cb))
      {
        PyErr_SetString(PyExc_RuntimeError, "invokePythonCallback received non-callable callback");
        return -1;
      }
 
      arglist = Py_BuildValue("(i)", rwflag);
      printf("calling Python function\n");
      ret = PyEval_CallObject(cb, arglist);
 
      if (PyErr_Occurred() != NULL)
      {
        printf("Error invoking passphrase callback\n");
        PyErr_Print();
        return -1;
      }
 
      return convertPassphrase(ret, buf, len);
}
  /*
   * Invoke the Python passphrase callback.
   * We use the thread-specific state to determine the python
   * callback to be invoked, or to find the string that should just
   * be returend as the passphrase.
   *
   * We just pass along rwflag to the python cb. We expect a string to be returned.
   */
static int invokePassphraseCallback(char *buf, int len, int rwflag, void *cb_arg)
{
    PyObject *arglist, *ret;
    struct pwcb_data *data;
    int rc;
    
    if (pwcb_get_data(&data) != 0)
    {
        /*
         * We couldn't get the thread-specific data.
         *
         * Raise a python exception.
         */
	
        PyErr_SetString(PyExc_RuntimeError, "Could not read thread-specific data in passphrase callback");
        return -1;
    }
    
    /*
     * Check the type of the passphrase passed.
     *
     * If it's a string, return that as the passphrase.
     *
     * If it's a list of integers, convert to a string and return.
     *
     * If it's a callable, call it, and handle the return as
     * above.
     *
     */
    
    if (PyCallable_Check(data->passphrase))
    {
        rc = invokePythonCallback(data->passphrase, buf, len, rwflag);
    }
    else
    {
        rc = convertPassphrase(data->passphrase, buf, len);
    }
    return rc;
}

static invokePythonDebugCallback(PyObject *cb, int debug, char *format, ...)
{
    PyObject *arglist, *ret;
    int rc;
    static char buf[1024];
    va_list args;
    va_start(args, format);
    vsnprintf(buf, sizeof(buf), format, args);
    va_end(args);

    if (PyFile_Check(cb))
    {
	FILE *fp = PyFile_AsFile(cb);

	vfprintf(fp, format, args);
	return;
    }

    if (!PyCallable_Check(cb))
    {
	if (debug)
	{
	    globus_libc_fprintf(stderr, buf);
	}
	return;
    }
    else
    {
	arglist = Py_BuildValue("(s)", buf);
	ret = PyEval_CallObject(cb, arglist);
	if (PyErr_Occurred() != NULL)
	{
	    printf("Error invoking debug callback\n");
	    PyErr_Print();
	    PyErr_Clear();
	}
    }
    
}

static void
null_key_gen_callback(int p, int n, void * dummy)
{
    return;
}
 PyObject *
get_globus_error_chain( globus_object_t *error)
{
    char *                              error_string;
    char *                              tmp;
    int                                 length = 1;
    globus_object_t *                   current_error;

    PyObject * errorList;

    errorList = PyList_New(0);

    current_error = error;

    do
    {
	/*
	 * This stuff didn't work well.
	int etype;
	char *short_desc;
	char *long_desc;
	globus_module_descriptor_t *source_mod;
	PyObject *tup;

	etype = globus_error_get_type(current_error);
	source_mod = globus_error_get_source(current_error);
	short_desc = globus_error_get_short_desc(current_error);
	*/
	
        tmp = globus_object_printable_to_string(current_error);

        if(tmp != NULL)
	{
	    PyList_Append(errorList, PyString_FromString(tmp));
	    globus_free(tmp);
	}
    }
    while(current_error = globus_error_get_cause(current_error));
    
    return errorList;
}

PyObject *
return_error(
    globus_result_t                     result)
{
    PyObject * tuple;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "return_error: PyTuple_New()");
        return NULL;
    }
    assert(result != GLOBUS_SUCCESS);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", result));
    PyTuple_SET_ITEM(tuple, 1, get_globus_error_chain(globus_error_get(result)));
    return tuple;
} 
PyObject *
return_error_extended(
    globus_result_t                     result)
{
    PyObject * tuple;
    if (!(tuple=PyTuple_New(3)))
    {
        PyErr_SetString(PyExc_RuntimeError, "return_error: PyTuple_New()");
        return NULL;
    }
    assert(result != GLOBUS_SUCCESS);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", result));
    PyTuple_SET_ITEM(tuple, 1, get_globus_error_chain(globus_error_get(result)));
    Py_INCREF(Py_None);
    PyTuple_SET_ITEM(tuple,2, Py_None);
    return tuple;
} 
#define GLOBUS_GSI_PROXY_GENERIC_POLICY_OID "1.3.6.1.4.1.3536.1.222"
#define GLOBUS_GSI_PROXY_GENERIC_POLICY_SN  "GENERICPOLICY"
#define GLOBUS_GSI_PROXY_GENERIC_POLICY_LN  "Generic Policy Object"
%}

%init %{
    init_pwcb_key();
%}

%inline %{

PyObject *
get_cert_dir() {
    PyObject *tuple;
    globus_result_t ret;
    char *cert_dir, *error_str;
    if (!(tuple=PyTuple_New(2))) {
        PyErr_SetString(PyExc_RuntimeError, 
                        "get_cert_dir:: PyTuple_New()");
        return NULL;
    }	
    ret = GLOBUS_GSI_SYSCONFIG_GET_CERT_DIR(&cert_dir);
    if (!(ret == GLOBUS_SUCCESS)) {
        error_str = globus_object_printable_to_string(globus_error_get(ret));
	    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
	    PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", error_str));
        globus_libc_free(error_str);
    } 
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", cert_dir));
        globus_libc_free(cert_dir);
    }
    return tuple;
}    

PyObject *
check_keyfile(const char* filename) {
    PyObject *tuple;
    globus_result_t ret;
    char *error_str;
    if (!(tuple=PyTuple_New(2))) {
        PyErr_SetString(PyExc_RuntimeError, 
                        "check_keyfile:: PyTuple_New()");
        return NULL;
    }	
    ret = GLOBUS_GSI_SYSCONFIG_CHECK_KEYFILE(filename);
    if (!(ret == GLOBUS_SUCCESS)) {
        error_str = globus_object_printable_to_string(globus_error_get(ret));
	    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
	    PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", error_str));
        globus_libc_free(error_str);
    } 
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject *
check_certfile(const char* filename) {
    PyObject *tuple;
    globus_result_t ret;
    char *error_str;
    if (!(tuple=PyTuple_New(2))) {
        PyErr_SetString(PyExc_RuntimeError, 
                        "check_certfile:: PyTuple_New()");
        return NULL;
    }	
    ret = GLOBUS_GSI_SYSCONFIG_CHECK_CERTFILE(filename);
    if (!(ret == GLOBUS_SUCCESS)) {
        error_str = globus_object_printable_to_string(globus_error_get(ret));
	    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
	    PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", error_str));
        globus_libc_free(error_str);
    } 
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject *
get_user_cert_filename() {
    PyObject *tuple;
    globus_result_t ret;
    char *cert_file, *key_file, *error_str;
    if (!(tuple=PyTuple_New(3))) {
        PyErr_SetString(PyExc_RuntimeError, 
                        "get_user_cert_filename:: PyTuple_New()");
        return NULL;
    }	
    ret = GLOBUS_GSI_SYSCONFIG_GET_USER_CERT_FILENAME(&cert_file, &key_file);
    if (!(ret == GLOBUS_SUCCESS)) {
        error_str = globus_object_printable_to_string(globus_error_get(ret));
	    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
	    PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", error_str));
	    Py_INCREF(Py_None);
	    PyTuple_SET_ITEM(tuple, 2, Py_None);
        globus_libc_free(error_str);
    } 
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", cert_file));
        PyTuple_SET_ITEM(tuple, 2, Py_BuildValue("s", key_file));
        globus_libc_free(cert_file);
        globus_libc_free(key_file);
    }
    return tuple;
}    

PyObject *
get_host_cert_filename() {
    PyObject *tuple;
    globus_result_t ret;
    char *cert_file, *key_file, *error_str;
    if (!(tuple=PyTuple_New(3))) {
        PyErr_SetString(PyExc_RuntimeError,
                        "get_host_cert_filename:: PyTuple_New()");
        return NULL;
    }
    ret = GLOBUS_GSI_SYSCONFIG_GET_HOST_CERT_FILENAME(&cert_file, &key_file);
    if (ret != GLOBUS_SUCCESS) {
        error_str = globus_object_printable_to_string(globus_error_get(ret));
            PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
            PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", error_str));
            Py_INCREF(Py_None);
            PyTuple_SET_ITEM(tuple, 2, Py_None);
        globus_libc_free(error_str);
    }
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", cert_file));
        PyTuple_SET_ITEM(tuple, 2, Py_BuildValue("s", key_file));
        globus_libc_free(cert_file);
        globus_libc_free(key_file);
    }
    return tuple;

}    

PyObject *
get_service_cert_filename(char *service_name) {
    PyObject *tuple;
    globus_result_t ret;
    char *cert_file, *key_file, *error_str;
    if (!(tuple=PyTuple_New(3))) {
        PyErr_SetString(PyExc_RuntimeError, 
                        "get_service_cert_filename:: PyTuple_New()");
        return NULL;
    }	
    ret = GLOBUS_GSI_SYSCONFIG_GET_SERVICE_CERT_FILENAME(service_name, &cert_file, &key_file);
    if (!(ret == GLOBUS_SUCCESS)) {
        error_str = globus_object_printable_to_string(globus_error_get(ret));
	    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
	    PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", error_str));
	    Py_INCREF(Py_None);
	    PyTuple_SET_ITEM(tuple, 2, Py_None);
        globus_libc_free(error_str);
    } 
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", cert_file));
        PyTuple_SET_ITEM(tuple, 2, Py_BuildValue("s", key_file));
        globus_libc_free(cert_file);
        globus_libc_free(key_file);
    }
    return tuple;
}    

PyObject *
get_proxy_filename(int proxy_file_type) {
    PyObject *tuple;
    globus_result_t ret;
    char *filename, *error_str;
    if (!(tuple=PyTuple_New(2))) {
        PyErr_SetString(PyExc_RuntimeError, 
                        "get_proxy_filename:: PyTuple_New()");
        return NULL;
    }	
    ret = GLOBUS_GSI_SYSCONFIG_GET_PROXY_FILENAME(&filename, 
                                (globus_gsi_proxy_file_type_t)proxy_file_type);
    if (!(ret == GLOBUS_SUCCESS)) {
        error_str = globus_object_printable_to_string(globus_error_get(ret));
	    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
	    PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", error_str));
        globus_libc_free(error_str);
    } 
    else {
        PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", 0));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", filename));
        globus_libc_free(filename);
    }
    return tuple;
}    

PyObject * 
grid_proxy_init(int debug,
		int verify,
		int key_bits,
		int valid,
		PyObject *py_proxy_out_filename,
		PyObject *passphraseSpec,
		PyObject *debugCB,
		PyObject *py_ca_cert_dir,
		PyObject *py_user_cert_filename,
		PyObject *py_user_key_filename,
		int cert_type)
  {
     globus_result_t                     result  = GLOBUS_SUCCESS;
     globus_gsi_proxy_handle_t           proxy_handle = NULL;
     globus_gsi_proxy_handle_attrs_t     proxy_handle_attrs = NULL;
     globus_gsi_callback_data_t          callback_data = NULL;
     globus_gsi_cred_handle_attrs_t      cred_handle_attrs = NULL;
     globus_gsi_cred_handle_t            cred_handle = NULL;
     globus_gsi_cred_handle_t            proxy_cred_handle = NULL;
     BIO *                               pem_proxy_bio = NULL;
     time_t                              goodtill;
     time_t                              lifetime;
     char *                              policy_buf = NULL;
     size_t                              policy_buf_len = 0;
     char *                              policy_filename = NULL;
     char *                              policy_language = NULL;
     char *                              proxy_out_filename = NULL;
     char *                              ca_cert_dir = NULL;
     char *                              user_cert_filename = NULL;
     char *                              user_key_filename = NULL;
     char *                              tmp_user_cert_filename = NULL;
     char *                              tmp_user_key_filename = NULL;
     int                                 policy_NID;
     int                                 path_length = -1;
     int                                 (*pw_cb)() = NULL;
     int                                 return_value = 0;
    
  
      PyObject * tuple;
      PyObject * errTuple;
      PyObject * err;
  
     if (!(tuple=PyTuple_New(2))) {
          PyErr_SetString(PyExc_RuntimeError, 
                         "grid_proxy_init: PyTuple_New()");
          return NULL;
      }
     
     if (PyString_Check(py_proxy_out_filename)){
	proxy_out_filename = PyString_AsString(py_proxy_out_filename);
        if((err = PyErr_Occurred()) !=NULL){
	    PyErr_Print();
	    return NULL;
	}
     }
     if (PyString_Check(py_ca_cert_dir)){
	ca_cert_dir = PyString_AsString(py_ca_cert_dir);
        if((err = PyErr_Occurred()) !=NULL){
	    PyErr_Print();
	    return NULL;
	}
     }
     
     if (PyString_Check(py_user_cert_filename)){
	user_cert_filename = PyString_AsString(py_user_cert_filename);
        if((err = PyErr_Occurred()) !=NULL){
	    PyErr_Print();
	    return NULL;
	}
     }
     if (PyString_Check(py_user_key_filename)){
	user_key_filename = PyString_AsString(py_user_key_filename);
        if((err = PyErr_Occurred()) !=NULL){
	    PyErr_Print();
	    return NULL;
	}
     }
        
    
     result = globus_gsi_proxy_handle_attrs_init(&proxy_handle_attrs);
     
     if(result != GLOBUS_SUCCESS)
     {
 
	 invokePythonDebugCallback(debugCB,debug,
				   "\n\nERROR: Couldn't initialize "
				   "the proxy handle attributes.\n");
	 
	  
	 errTuple = return_error(result);
	 return errTuple;
     }
 
     /* set the key bits for the proxy cert in the proxy handle
      * attributes
      */
     result = globus_gsi_proxy_handle_attrs_set_keybits(
         proxy_handle_attrs, key_bits);
     if(result != GLOBUS_SUCCESS)
     {
	 invokePythonDebugCallback(debugCB,debug,
                             "\n\nERROR: Couldn't set the key bits for "
                             "the private key of the proxy certificate\n");
         errTuple = return_error(result);
 	    return errTuple;
     }
     
     result = globus_gsi_proxy_handle_attrs_set_key_gen_callback(
         proxy_handle_attrs, 
         null_key_gen_callback);
     if(result != GLOBUS_SUCCESS)
     {
	 invokePythonDebugCallback(debugCB,debug,
				   "\n\nERROR: Couldn't set the key generation callback function\n");
            
         errTuple = return_error(result);
	 return errTuple;
     }
 
     result = globus_gsi_proxy_handle_init(&proxy_handle, proxy_handle_attrs);
     if(result != GLOBUS_SUCCESS)
     {
	 invokePythonDebugCallback(debugCB,debug,
             "\n\nERROR: Couldn't initialize the proxy handle\n");
         errTuple = return_error(result);
 	return errTuple;
     }
 
     result = globus_gsi_proxy_handle_attrs_destroy(proxy_handle_attrs);
     if(result != GLOBUS_SUCCESS)
     {
	 invokePythonDebugCallback(debugCB,debug,
                             "\n\nERROR: Couldn't destroy proxy "
                             "handle attributes.\n");
         errTuple = return_error(result);
 	return errTuple;
     }
     
     /* set the time valid in the proxy handle
      * used to be hours - now the time valid needs to be set in minutes 
      */
     result = globus_gsi_proxy_handle_set_time_valid(proxy_handle, valid);
 
     if(result != GLOBUS_SUCCESS)
     {
	 invokePythonDebugCallback(debugCB,debug,
                             "\n\nERROR: Couldn't set the validity time "
                             "of the proxy cert to %d minutes.\n", valid);
         errTuple = return_error(result);
 	return errTuple;
     }
 
     /* set the type of proxy to be generated
      */
     result = globus_gsi_proxy_handle_set_type(proxy_handle, cert_type);
 
     if(result != GLOBUS_SUCCESS)
     {
	 invokePythonDebugCallback(debugCB,debug,
                             "\n\nERROR: Couldn't set the type"
                             "of the proxy cert\n");
         errTuple = return_error(result);
 	    return errTuple;
     }
     
     if(!user_cert_filename || !user_key_filename)
    {
        result = GLOBUS_GSI_SYSCONFIG_GET_USER_CERT_FILENAME(
            user_cert_filename ? NULL : &tmp_user_cert_filename,
            user_key_filename ? NULL : &tmp_user_key_filename);
        if(result != GLOBUS_SUCCESS)
        {
            invokePythonDebugCallback(debugCB,debug,
                                "\n\nERROR: Couldn't find valid credentials "
                                "to generate a proxy.\n");
            errTuple = return_error(result);
	    return errTuple;
        }

        if(tmp_user_cert_filename &&
           tmp_user_cert_filename == tmp_user_key_filename)
        {
            /* supposed to be a pkcs12 formated credential */
            user_cert_filename = user_key_filename
                               = tmp_user_key_filename;
        }
    }

    if(!user_cert_filename)
    {
        user_cert_filename = tmp_user_cert_filename;
    }

    if(!user_key_filename)
    {
        user_key_filename = tmp_user_key_filename;
    }
    

     
    invokePythonDebugCallback(debugCB,debug,
			      "\nUser Cert File: %s\nUser Key File: %s\n",
			      user_cert_filename, user_key_filename);
    
    invokePythonDebugCallback(debugCB,debug,
			      "\nTrusted CA Cert Dir: %s\n", 
			      ca_cert_dir ? ca_cert_dir : "(null)");
    
 
     if(!ca_cert_dir && verify)
     {
        result = GLOBUS_GSI_SYSCONFIG_GET_CERT_DIR(&ca_cert_dir);
        if(result != GLOBUS_SUCCESS)
        {
	    invokePythonDebugCallback(debugCB,debug,
                "\n\nERROR: Couldn't find a valid trusted certificate "
                "directory\n");
            errTuple = return_error(result);
	    return errTuple;
        }
    }

   
     
     invokePythonDebugCallback(debugCB,debug,
			       "\nTrusted CA Cert Dir: %s\n", 
			       ca_cert_dir ? ca_cert_dir : "(null)");
     

     
    if(!proxy_out_filename)
    {
        result = GLOBUS_GSI_SYSCONFIG_GET_PROXY_FILENAME(
            &proxy_out_filename,
            GLOBUS_PROXY_FILE_OUTPUT);
        if(result != GLOBUS_SUCCESS)
        {
	    invokePythonDebugCallback(debugCB,debug,
                "\n\nERROR: Couldn't find a valid location "
                "to write the proxy file\n");
            errTuple = return_error(result);
	    return errTuple;
        }
    }
    else
     {
         /* verify that the directory path of proxy_out_filename
          * exists and is writeable
          */
         char *                          proxy_absolute_path = NULL;
         char *                          temp_filename = NULL;
         char *                          temp_dir = NULL;
 
         /* first, make absolute path */
         result = GLOBUS_GSI_SYSCONFIG_MAKE_ABSOLUTE_PATH_FOR_FILENAME(
             proxy_out_filename,
             &proxy_absolute_path);
         if(result != GLOBUS_SUCCESS)
         {
 	    invokePythonDebugCallback(debugCB,debug,
                 "\n\nERROR: Can't create the absolute path "
                 "of the proxy filename: %s",
                 proxy_out_filename);
             errTuple = return_error(result);
 	    return errTuple;
         }
 

         proxy_out_filename = proxy_absolute_path;
 
         /* then split */
         result = GLOBUS_GSI_SYSCONFIG_SPLIT_DIR_AND_FILENAME(
             proxy_absolute_path,
             &temp_dir,
             &temp_filename);
         if(result != GLOBUS_SUCCESS)
         {
 	    invokePythonDebugCallback(debugCB,debug,
                 "\n\nERROR: Can't split the full path into "
                 "directory and filename. The full path is: %s", 
                 proxy_absolute_path);
             if(proxy_absolute_path)
             {
                 free(proxy_absolute_path);
                 proxy_absolute_path = NULL;
             }
             errTuple = return_error(result);
 	    return errTuple;
         }
                 
         result = GLOBUS_GSI_SYSCONFIG_DIR_EXISTS(temp_dir);
         if(result != GLOBUS_SUCCESS)
         {
 	    invokePythonDebugCallback(debugCB,debug,
 				      "\n\nERROR: %s is not a valid directory for writing the "
 				      "proxy certificate\n\n",
 				      temp_dir);
 
             if(temp_dir)
             {
                 free(temp_dir);
                 temp_dir = NULL;
             }
             
             if(temp_filename)
             {
                 free(temp_filename);
                 temp_filename = NULL;
             }
 
            
            
	     errTuple = return_error(result);
	     return errTuple; 
         }
 
         if(temp_dir);
         {
             free(temp_dir);
             temp_dir = NULL;
         }
         
         if(temp_filename)
         {
             free(temp_filename);
             temp_filename = NULL;
         }
     }
 
    
    invokePythonDebugCallback(debugCB,debug,
			      "\nOutput File: %s\n", proxy_out_filename);
    
 
     result = globus_gsi_cred_handle_attrs_init(&cred_handle_attrs);
     if(result != GLOBUS_SUCCESS)
     {
	 invokePythonDebugCallback(debugCB,debug,
                             "\n\nERROR: Couldn't initialize credential "
                             "handle attributes\n");
         errTuple = return_error(result);
 	    return errTuple;
     }
     
     result = globus_gsi_cred_handle_init(&cred_handle, cred_handle_attrs);
     if(result != GLOBUS_SUCCESS)
     {
	 invokePythonDebugCallback(debugCB,debug,
                             "\n\nERROR: Couldn't initialize credential "
                             "handle\n");
         errTuple = return_error(result);
 	    return errTuple;
     }
 
     result = globus_gsi_cred_handle_attrs_destroy(cred_handle_attrs);
     if(result != GLOBUS_SUCCESS)
     {
	 invokePythonDebugCallback(debugCB,debug,
                             "\n\nERROR: Couldn't destroy credential "
                             "handle attributes.\n");
         errTuple = return_error(result);
 	    return errTuple;
     }
 
     if(strstr(user_cert_filename, ".p12"))
     {
	 /* we have a pkcs12 credential */
	 result = globus_gsi_cred_read_pkcs12(
	     cred_handle,
	     user_cert_filename);
	 if(result != GLOBUS_SUCCESS)
	 {
	    invokePythonDebugCallback(debugCB,debug,
		 "\n\nERROR: Couldn't read in PKCS12 credential "
		 "from file: %s\n", user_cert_filename);
	     errTuple = return_error(result);
            return errTuple;
	    
	 }

	 if (debug)
	 {
	     char *                          subject = NULL;
	     result = globus_gsi_cred_get_subject_name(cred_handle, &subject);
	     if(result != GLOBUS_SUCCESS)
	     {
		 invokePythonDebugCallback(debugCB,debug,
		     "\n\nERROR: The subject name of the "
		     "user certificate could "
                    "not be retrieved\n");
		 errTuple = return_error(result);
		 return errTuple;

	     }
	     
            printf("Your identity: %s\n", subject);
            if(subject)
            {
                free(subject);
                subject = NULL;
            }
	 }
    }
     else
    {
	result = globus_gsi_cred_read_cert(
	    cred_handle,
	    user_cert_filename);
	
	if(result != GLOBUS_SUCCESS)
	{
	    invokePythonDebugCallback(debugCB,debug,
				      "\n\nERROR: Couldn't read user certificate\n"
				      "cert file location: %s\n\n", 
				      user_cert_filename);
	    errTuple = return_error(result);
	    return errTuple;
	}
	 	
	if (passphraseSpec != Py_None)
	{
	    /*
	     * Stash the passphrase information in thread-specific storage.
	     */
	    char *err;
            int rc =  GLOBUS_SUCCESS;
	    rc = pwcb_set_data(passphraseSpec, &err);
	    
	    
	    if (rc != GLOBUS_SUCCESS)
	    {
		/*
 		 * Badness; the set_data failed somehow. Raise an exception
 		 * with the given error string.
 		 */
		
		PyErr_SetString(PyExc_RuntimeError, err);
		return NULL;
	    }
	    
	    /** First see if a python exception has already been raised.
	     * If so, we hit a problem in the python callback stuff somewhere.
	     */
	    
	    result = globus_gsi_cred_read_key(
		cred_handle,
		user_key_filename,
		invokePassphraseCallback);
	 
	    /*
	     * And remove it now we're done.
	     */
	    pwcb_remove_data();
	    
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	}	 
	else
	{
	    EVP_set_pw_prompt(debug? "Enter GRID pass phrase:" :
			      "Enter GRID pass phrase for this identity:");
	    /*Read in the passphrase from the command prompt*/
	    result = globus_gsi_cred_read_key(
		cred_handle,
		user_key_filename,
		pw_cb);
	}
	
 
	
	/*
	 * Otherwise return the error.
	 */
	if (result != GLOBUS_SUCCESS)
	{
	    globus_object_t * error; 
	    error = globus_error_get(result);
	    
	    if(globus_error_match_openssl_error(error,
						ERR_LIB_PEM,
						PEM_F_PEM_DO_HEADER,
						PEM_R_BAD_DECRYPT)
	       == GLOBUS_TRUE)
	    { 
		invokePythonDebugCallback(debugCB,debug,
					  "\n\nERROR: Couldn't read user key: Bad passphrase\n"
					  "key file location: %s\n\n",
					  user_key_filename);
	    }
	    else
	    {
		invokePythonDebugCallback(debugCB,debug,
					  "\n\nERROR: Couldn't read user key.\n"
					  "key file location: %s\n\n",
					  user_key_filename);
	    }
	    
	    result = globus_error_put(error);
	    
	    errTuple = return_error(result);
	    return errTuple;
	    
	}
   
    }
     
     /* add path length constraint */
     
     if(path_length >= 0)
     {
	 result = globus_gsi_proxy_handle_set_pathlen(proxy_handle,
						      path_length);
	 if(result != GLOBUS_SUCCESS)
	 {
	     invokePythonDebugCallback(debugCB,debug,
				       "\n\nERROR: Can't set the path "
				       "length in the proxy handle\n");
	     errTuple = return_error(result);
	     return errTuple;
	 }
     }
     
    
     invokePythonDebugCallback(debugCB,debug,
			       "Creating proxy ");
	 
     
     result = globus_gsi_proxy_create_signed(
         proxy_handle,
         cred_handle,
         &proxy_cred_handle);
     if(result != GLOBUS_SUCCESS)
     {
	 invokePythonDebugCallback(debugCB,debug,
				  "\n\nERROR: Couldn't create proxy certificate\n");
	errTuple = return_error(result);
	return errTuple;
     }
     
    
    
     invokePythonDebugCallback(debugCB,debug,
				  " Done\n");
     
     
     if(verify)
     {
         result = globus_gsi_callback_data_init(&callback_data);
         if(result != GLOBUS_SUCCESS)
         {
	     invokePythonDebugCallback(debugCB,debug,
				       "\n\nERROR: Couldn't initialize callback data "
				       "for credential verification\n");
             errTuple = return_error(result);
	     return errTuple;
         }
 
         result = globus_gsi_callback_set_cert_dir(
             callback_data,
             ca_cert_dir);
         if(result != GLOBUS_SUCCESS)
         {
	     invokePythonDebugCallback(debugCB,debug,
				       "certificate directory in the callback "
				       "data\n");
             errTuple = return_error(result);
	     return errTuple;
         }
	 
         result = globus_gsi_cred_verify_cert_chain(
             proxy_cred_handle,
             callback_data);
         if(result != GLOBUS_SUCCESS)
         {
	     invokePythonDebugCallback(debugCB,debug,
				       "\n\nERROR: Couldn't verify the authenticity of the user's "
				       "credential to generate a proxy from.\n");
             errTuple = return_error(result);
	     return errTuple;
         }
	 
	 invokePythonDebugCallback(debugCB,debug,
				   "Proxy Verify OK\n");
     }
     
     result = globus_gsi_cred_write_proxy(proxy_cred_handle,
                                          proxy_out_filename);
     if(result != GLOBUS_SUCCESS)
     {
	 invokePythonDebugCallback(debugCB,debug,
				   "\n\nERROR: The proxy credential could not be "
				   "written to the output file.\n");
         errTuple = return_error(result);
	 return errTuple;
     }
     
     result = globus_gsi_cred_get_lifetime(
         cred_handle,
         &lifetime);
     if(result != GLOBUS_SUCCESS)
     {
	 invokePythonDebugCallback(debugCB,debug,
				   "\n\nERROR: Can't get the lifetime of the proxy "
				   "credential.\n");
         errTuple = return_error(result);
	 return errTuple;
     }
 
     result = globus_gsi_cred_get_goodtill(
         proxy_cred_handle,
         &goodtill);
     if(result != GLOBUS_SUCCESS)
     {
	 invokePythonDebugCallback(debugCB,debug,
				   "\n\nERROR: Can't get the expiration date of the "
				   "proxy credential.\n");
         errTuple = return_error(result);
	 return errTuple;
     }
 
     if(lifetime < 0)
     {
	 invokePythonDebugCallback(debugCB,debug,
				   "\n\nERROR: Your certificate has expired: %s\n\n", 
				   asctime(localtime(&goodtill)));
     }
     else if(lifetime < (valid * 60))
     {
	 invokePythonDebugCallback(debugCB,debug,
				   "\nWarning: your certificate and proxy will expire %s "
				   "which is within the requested lifetime of the proxy\n",
				   asctime(localtime(&goodtill)));
         return_value = 1;
     }
     else 
     {
	 invokePythonDebugCallback(debugCB,debug, 
				   "Your proxy is valid until: %s", 
				   asctime(localtime(&goodtill)));
     }
 
     BIO_free(pem_proxy_bio);
 
     globus_gsi_proxy_handle_destroy(proxy_handle);
     globus_gsi_cred_handle_destroy(cred_handle);
     globus_gsi_cred_handle_destroy(proxy_cred_handle);
     globus_gsi_callback_data_destroy(callback_data);
 
     PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", GLOBUS_SUCCESS));
     
     Py_INCREF(Py_None); 
     PyTuple_SET_ITEM(tuple, 1, Py_None);
     return tuple;
}

PyObject *
grid_proxy_info(int verbose, PyObject * py_proxy_filename)
{
    int                                 strength          = 0;
    int                                 bits              = 0;
    int                                 time_valid        = 0;
    int                                 exists_flag       = 0;
    int                                 time_valid_flag   = 0;
    int                                 hours_flag        = 0;
    int                                 bits_flag         = 0;
    int                                 is_valid          = 0;
    char *                              proxy_filename = NULL;
    char *                              subject;
    char *                              issuer;
    char *                              identity;
    globus_gsi_cert_utils_cert_type_t   cert_type;
    char *                              cert_type_name;
    time_t                              lifetime;
    globus_gsi_cred_handle_t            proxy_cred = NULL;
    X509 *                              proxy_cert = NULL;
    EVP_PKEY *                          proxy_pubkey = NULL;
    globus_result_t                     result; 

    char proxy_string[500];
    PyObject * tuple;
    PyObject * errTuple;
    PyObject * py_proxy_string;
    PyObject * py_proxy_dict;
    PyObject * py_tmp_string;
    PyObject * err;
    int free_file = 0;
    
    if (!(tuple=PyTuple_New(3))) {
        PyErr_SetString(PyExc_RuntimeError,
                        "grid_proxy_info: PyTuple_New()");
        return NULL;
    }
    if (PyString_Check(py_proxy_filename)){
	proxy_filename = PyString_AsString(py_proxy_filename);
        if((err = PyErr_Occurred()) !=NULL){
	    PyErr_Print();
	    return NULL;
	}
    }
    if(proxy_filename)
    {
        result = GLOBUS_GSI_SYSCONFIG_CHECK_KEYFILE(proxy_filename);

        if(result != GLOBUS_SUCCESS)
        {
            globus_libc_fprintf(
                stderr,
                "\n\nERROR: Couldn't find a valid proxy.\n");
            errTuple = return_error_extended(result);
	    return errTuple;
        }
    }
    else
    { 
        result = GLOBUS_GSI_SYSCONFIG_GET_PROXY_FILENAME(
            &proxy_filename,
            GLOBUS_PROXY_FILE_INPUT);
        if(result != GLOBUS_SUCCESS)
        {
            globus_libc_fprintf(
                stderr,
                "\n\nERROR: Couldn't find a valid proxy.\n");
            errTuple = return_error_extended(result);
	    return errTuple;
        }
	free_file = 1;
    }

    result = globus_gsi_cred_handle_init(&proxy_cred, NULL);
    if(result != GLOBUS_SUCCESS)
    {
        globus_libc_fprintf(
            stderr,
            "\n\nERROR: Couldn't initialize proxy credential handle\n");
        errTuple = return_error_extended(result);
	return errTuple;
    }
    
    result = globus_gsi_cred_read_proxy(proxy_cred, proxy_filename);
    if(result != GLOBUS_SUCCESS)
    {
        globus_libc_fprintf(
            stderr,
            "\n\nERROR: Couldn't read proxy from: %s\n", proxy_filename);
        errTuple = return_error_extended(result);
	return errTuple;
    }

    result = globus_gsi_cred_get_cert(proxy_cred, &proxy_cert);
    if(result != GLOBUS_SUCCESS)
    {
        globus_libc_fprintf(
            stderr,
            "\n\nERROR: Couldn't get the proxy certificate from "
            "the proxy credential.\n");
        errTuple = return_error_extended(result);
	return errTuple;
    }

    if ((proxy_pubkey = X509_get_pubkey(proxy_cert)) == NULL)
    {
        globus_libc_fprintf(
            stderr,
            "\n\nERROR: unable to load public key from proxy\n");
        errTuple = return_error_extended(result);
	return errTuple;
        
    }

    /* The things we will need to know below: subject, issuer,
       strength, validity, type */

    /* subject */
    result = globus_gsi_cred_get_subject_name(proxy_cred, &subject);
    if(result != GLOBUS_SUCCESS)
    {
        globus_libc_fprintf(
            stderr,
            "\n\nERROR: Couldn't get a valid subject "
            "name from the proxy credential.\n");
        errTuple = return_error_extended(result);
	return errTuple;
    }

    /* issuer */

    result = globus_gsi_cred_get_issuer_name(proxy_cred, &issuer);
    if(result != GLOBUS_SUCCESS)
    {
        globus_libc_fprintf(
            stderr,
            "\n\nERROR: Couldn't get a valid issuer "
            "name from the proxy credential.\n");
        errTuple = return_error_extended(result);
	return errTuple;
    }
    /* issuer */

    result = globus_gsi_cred_get_identity_name(proxy_cred, &identity);
    if(result != GLOBUS_SUCCESS)
    {
        globus_libc_fprintf(
            stderr,
            "\n\nERROR: Couldn't get a valid identity "
            "name from the proxy credential.\n");
        errTuple = return_error_extended(result);
	return errTuple;
    }
    
    /* validity: set time_diff to time to expiration (in seconds) */
    result = globus_gsi_cred_get_lifetime(proxy_cred,
                                          &lifetime);
    if(result != GLOBUS_SUCCESS)
    {
        globus_libc_fprintf(
            stderr,
            "\n\nERROR: Couldn't get a valid lifetime "
            "for the proxy credential.\n");
        errTuple = return_error_extended(result);
	return errTuple;
    }

    /* strength: set strength to key size (in bits) */
    strength = 8 * EVP_PKEY_size(proxy_pubkey);

    /* check if proxy is valid in our own defined sense */
    if (exists_flag)
    {
        is_valid = (lifetime >= (time_valid * 60)) && 
                   (strength >= bits) ? 0 : 1;
    }
    else
    {
        is_valid = 0;
    }

    /* type: restricted, limited or full */
    result = globus_gsi_cred_get_cert_type(proxy_cred,
                                           &cert_type);
    if(result != GLOBUS_SUCCESS)
    {
        globus_libc_fprintf(
            stderr,
            "\n\nERROR: Couldn't get the proxy type "
            "from the proxy credential\n");
        errTuple = return_error_extended(result);
	return errTuple;
    }
    
    switch(cert_type)
    {
      case GLOBUS_GSI_CERT_UTILS_TYPE_GSI_3_IMPERSONATION_PROXY:
        cert_type_name = "Proxy draft compliant impersonation proxy";
        break;
      case GLOBUS_GSI_CERT_UTILS_TYPE_GSI_3_INDEPENDENT_PROXY:
        cert_type_name = "Proxy draft compliant independent proxy";
        break;
      case GLOBUS_GSI_CERT_UTILS_TYPE_GSI_3_LIMITED_PROXY:
        cert_type_name = "Proxy draft compliant limited proxy";
        break;
      case GLOBUS_GSI_CERT_UTILS_TYPE_GSI_3_RESTRICTED_PROXY:
        cert_type_name = "Proxy draft compliant restricted proxy";
        break;
      case GLOBUS_GSI_CERT_UTILS_TYPE_GSI_2_PROXY:
        cert_type_name = "full legacy globus proxy";
        break;
      case GLOBUS_GSI_CERT_UTILS_TYPE_GSI_2_LIMITED_PROXY:
        cert_type_name = "limited legacy globus proxy";
        break;
      case GLOBUS_GSI_CERT_UTILS_TYPE_EEC:
        cert_type_name = "end entity credential";
        break;        
      default:
        globus_libc_fprintf(
            stderr,
            "\n\nERROR: Not a proxy\n\n");
       cert_type_name = 0;
    }
    
        sprintf(proxy_string,"subject  : %s\n" 
               "issuer   : %s\n"
               "identity : %s\n" 
               "type     : %s\n" 
               "strength : %d bits\n"
               "path     : %s\n"
               "timeleft : ",
               subject,
               issuer,
               identity,
		cert_type_name ? cert_type_name : "Not a proxy",
               strength,
               proxy_filename);
        
	py_proxy_string = PyString_FromString(proxy_string);

	if (lifetime <= 0)
            lifetime = 0; 
        
	sprintf(proxy_string, "%ld:%02ld:%02ld\n",
                   (long)(lifetime / 3600),
                   (long)(lifetime % 3600) / 60,
                   (long)lifetime % 60 );

      py_tmp_string = PyString_FromString(proxy_string);

      
      PyString_Concat(&py_proxy_string, py_tmp_string);

      if (lifetime > 3600 * 24)
               {
                sprintf(proxy_string, "  (%.1f days)\n", (float)(lifetime / 3600) / 24.0);
                py_tmp_string = PyString_FromString(proxy_string);
                PyString_Concat(&py_proxy_string, py_tmp_string);

            }
    py_proxy_dict = PyDict_New();

    PyDict_SetItemString(py_proxy_dict, "subject", PyString_FromString(subject)
);
    PyDict_SetItemString(py_proxy_dict, "issuer", PyString_FromString(issuer));
    
    if (cert_type_name == GLOBUS_NULL)
    {
	Py_INCREF(Py_None);
	PyDict_SetItemString(py_proxy_dict, "type", Py_None);
    }
    else
    {
	PyDict_SetItemString(py_proxy_dict, "type", PyString_FromString(cert_type_name));
    }
    PyDict_SetItemString(py_proxy_dict, "type_num", PyInt_FromLong(cert_type));
    PyDict_SetItemString(py_proxy_dict, "strength", PyInt_FromLong(strength));
    PyDict_SetItemString(py_proxy_dict, "path", PyString_FromString(proxy_filename));
    PyDict_SetItemString(py_proxy_dict, "subject", PyString_FromString(subject)
);
    PyDict_SetItemString(py_proxy_dict, "lifetime", PyInt_FromLong(lifetime));
    

    free(subject);
    free(issuer);
    free(identity);
    if(free_file)
    {
	globus_free(proxy_filename);
    }
    Py_DECREF(py_tmp_string);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", GLOBUS_SUCCESS));
    PyTuple_SET_ITEM(tuple, 1, py_proxy_string);
    PyTuple_SET_ITEM(tuple, 2, py_proxy_dict);
    return tuple;

}

PyObject *
grid_proxy_destroy(PyObject * py_default_full_file)
{
   

    char *                              default_full_file = NULL;
    globus_result_t                     result = GLOBUS_SUCCESS;
    globus_bool_t                       free_file = 0;
    PyObject *                          tuple;
    PyObject *                          errTuple;
    PyObject *                          err;

    if (!(tuple=PyTuple_New(2))) {
        PyErr_SetString(PyExc_RuntimeError,
                        "grid_proxy_destroy:: PyTuple_New()");
        return NULL;
    }
    if (PyString_Check(py_default_full_file)){
	default_full_file = PyString_AsString(py_default_full_file);
        if((err = PyErr_Occurred()) !=NULL){
	    PyErr_Print();
	    return NULL;
	}
     }
   
    if(default_full_file==NULL)
    {
        result = GLOBUS_GSI_SYSCONFIG_GET_PROXY_FILENAME(&default_full_file,
                                                         GLOBUS_PROXY_FILE_INPUT);
        if(result != GLOBUS_SUCCESS)
        {
            globus_libc_fprintf(
                stderr,
                "\nERROR: Proxy file doesn't exist or has bad permissions\n");
             errTuple = return_error(result);
	return errTuple;
        }
	free_file = 1;
    }

    
    globus_i_gsi_proxy_utils_clear_and_remove(default_full_file, 0);
        
    /* 
     * no options, remove the default file, which is the ENV
     * or the /tmp/x509up_u<uid> file
     */


 done:

    if(free_file)
    { 
        free(default_full_file);
    }

    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", GLOBUS_SUCCESS));
    Py_INCREF(Py_None);
    PyTuple_SET_ITEM(tuple, 1, Py_None);
    return tuple;

}

%}
