/*############################################################################
# Matt Rodriguez, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################*/
#include "globus_common.h"
#include "pyGlobus_common.h"


void init_globus_thread_state_key(globus_thread_key_t *python_thread_state_key)
{
    int rc;

    rc = globus_thread_key_create(python_thread_state_key,  python_thread_state_destroy_cb);
    if (rc != 0)
    {
    printf("Error creating python_thread_state_key: %d\n", rc);
    }
}

/* DEK: added static to io.i patch from Access Grid node when integrating into more source files */
PyThreadState *pyGlobus_createThreadState(PyInterpreterState *python_interp)
{
    PyThreadState *state;
    state = PyThreadState_New(python_interp);
    return state;
}

/*
 * Retrieve the per-thread Python thread state.
 * If it doesn't exist yet, create one and put it in thread-specific  storage.
 */
PyThreadState *get_python_thread_state(globus_thread_key_t python_thread_state_key,PyInterpreterState *python_interp)
{
    PyThreadState *state = (PyThreadState *)  globus_thread_getspecific(python_thread_state_key);

    if (state == 0)
    {
    int rc;

    state = pyGlobus_createThreadState(python_interp);
    
    /*pyGlobus_dprintf("Creating new python thread state %x for thread  %x\n"      state, globus_thread_self()); */

    rc = globus_thread_setspecific(python_thread_state_key, (void *)  state);
    if (rc != 0)
    {
        pyGlobus_dprintf("globus_thread_setspecific failed: %d\n", rc);
    }
    }
    return state;
}

/* DEK: added static to io.i patch from Access Grid node when integrating into more source files */
void pyGlobus_destroyThreadState(PyThreadState *state, int haveLock)
{
    /*pyGlobus_dprintf("Destroying thread state for thread %x: %x\n",
      globus_thread_self(), state); */
       
    /*
     * We need to hold the interp lock to clear out
     * the thread state before we can delete it.
     */

    if (!haveLock)
    PyEval_AcquireThread(state);
    /*pyGlobus_dprintf("* Have thread state in destroy_cb\n");*/
    PyThreadState_Clear(state);
    PyThreadState_DeleteCurrent();
}

void python_thread_state_destroy_cb(void *val)
{
    PyThreadState *state;
    state = (PyThreadState *) val;
    pyGlobus_destroyThreadState(state, 0);
    /*pyGlobus_dprintf("* Leaving in destroy_cb\n");*/
}

