/*############################################################################
# Matt Rodriguez, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################*/


%module rlsClientc
%{
#include "globus_rls_client.h"
%}

typedef enum {
  rls_pattern_unix,     /**< Unix file globbing chars (*, ?)    */
  rls_pattern_sql       /**< SQL "like" wildcards (%, _)        */
} globus_rls_pattern_t;

typedef enum {
  globus_rls_obj_lrc_lfn,       /**< LRC Logical File Name. */
  globus_rls_obj_lrc_pfn,       /**< LRC Physical File Name. */
  globus_rls_obj_rli_lfn,       /**< RLI Logical File Name. */
  globus_rls_obj_rli_lrc        /**< RLI LRC URL. */
} globus_rls_obj_type_t;

typedef enum {
  globus_rls_attr_type_date,    /**< Date (time_t). */
  globus_rls_attr_type_flt,     /**< Floating point (double). */
  globus_rls_attr_type_int,     /**< Integer (int). */
  globus_rls_attr_type_str      /**< String (char *). */
} globus_rls_attr_type_t;

typedef enum {
  globus_rls_attr_op_all,       /**< All values returned.               */
  globus_rls_attr_op_eq,        /**< Values matching operand 1 returned.*/
  globus_rls_attr_op_ne,        /**< Values not matching operand 1.     */
  globus_rls_attr_op_gt,        /**< Values greater than operand 1.     */
  globus_rls_attr_op_ge,        /**< Values greater than or equal to op1*/
  globus_rls_attr_op_lt,        /**< Values less than operand 1.        */
  globus_rls_attr_op_le,        /**< Values less than or equal to op1.  */
  globus_rls_attr_op_btw,       /**< Values between operand1 and 2.     */
  globus_rls_attr_op_like,      /**< Strings "like" operand1 (SQL like) */
} globus_rls_attr_op_t;



#define FRLI_BLOOMFILTER        0x1
#define GLOBUS_RLS_LFN_NEXIST           12
#define GLOBUS_RLS_PFN_NEXIST           14
%inline %{

globus_module_descriptor_t* get_module(void) {
    return  GLOBUS_RLS_CLIENT_MODULE;
}

PyObject *
client_connect(char *rls_server)
{
    globus_result_t		ret;
    PyObject                   *tuple, *ptr;
    char                        tmpstr[256];
    int                         rc; 
    globus_rls_handle_t	       *handle = NULL;

    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "client_connect: PyTuple_New()");
        return NULL;
    }
    
    handle = (globus_rls_handle_t*)globus_malloc(sizeof(globus_rls_handle_t));	
    
     Py_BEGIN_ALLOW_THREADS
     ret = globus_rls_client_connect(rls_server, &handle);
     Py_END_ALLOW_THREADS

    
   
    if (ret != GLOBUS_SUCCESS)
    {
	globus_rls_client_error_info(ret, &rc,tmpstr,256,GLOBUS_FALSE); 
	PyTuple_SET_ITEM(tuple,0, Py_BuildValue("i", rc));
       	PyTuple_SET_ITEM(tuple,1, Py_BuildValue("s", tmpstr));
       
    }
    else
    {
	PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
	ptr = SWIG_NewPointerObj((void *)handle, SWIGTYPE_p_globus_rls_handle_t, 0);
	PyTuple_SET_ITEM(tuple, 1, ptr);	
    }	
  
    return tuple; 
   
}


PyObject *
client_close(globus_rls_handle_t * handle)
{
    globus_result_t             ret;
    PyObject                   *tuple;
    char                        tmpstr[256];
    int                         rc;

    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "client_close:: PyTuple_New()");
        return NULL;
    }
     Py_BEGIN_ALLOW_THREADS
    ret = globus_rls_client_close(handle);
     Py_END_ALLOW_THREADS

    
    if (ret != GLOBUS_SUCCESS)
    {
	globus_rls_client_error_info(ret, &rc,tmpstr,256,GLOBUS_FALSE); 
	PyTuple_SET_ITEM(tuple,0, Py_BuildValue("i", rc));
	PyTuple_SET_ITEM(tuple,1, Py_BuildValue("s", tmpstr));

    }
    else
    {
	PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
	Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }	
    return tuple; 

}

PyObject *
lrc_clear(globus_rls_handle_t * handle)
{
    globus_result_t             ret;
    PyObject                   *tuple;
    char                        tmpstr[256];
    int                         rc;
 
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "lrc_clear:: PyTuple_New()");
        return NULL;
    }

     Py_BEGIN_ALLOW_THREADS
     ret = globus_rls_client_lrc_clear(handle);
     Py_END_ALLOW_THREADS
      
    
     if (ret != GLOBUS_SUCCESS)
     {
	 globus_rls_client_error_info(ret, &rc,tmpstr,256,GLOBUS_FALSE); 
	 PyTuple_SET_ITEM(tuple,0, Py_BuildValue("i", rc));
	 PyTuple_SET_ITEM(tuple,1, Py_BuildValue("s", tmpstr));
     }
     else
     {
	 PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
	 Py_INCREF(Py_None);
	 PyTuple_SET_ITEM(tuple, 1, Py_None);
     }	
     return tuple; 
    
	 

}
PyObject *
lrc_get_pfn(globus_rls_handle_t *handle, char *lfn, int offset, int reslimit)
{
  globus_result_t	ret;
  int			n;
  int			i;
  globus_list_t	       *glist;
  globus_list_t	       *p;
  globus_rls_string2_t *str2;
  PyObject	       *list, *tuple;
  char                  tmpstr[256];
  int                   rc;

  if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "lrc_get_pfn:: PyTuple_New()");
        return NULL;
    }
  Py_BEGIN_ALLOW_THREADS
      ret = globus_rls_client_lrc_get_pfn(handle, lfn, &offset, reslimit,&glist);
  Py_END_ALLOW_THREADS
      
     
  /* query the LRC using the LFN with result stuffed into glist */
  if (ret != GLOBUS_SUCCESS){
      /* send back the non-zero integer return value to indicate failure
       at the rls API level */
      
      globus_rls_client_error_info(ret, &rc,tmpstr,256,GLOBUS_FALSE); 
      PyTuple_SET_ITEM(tuple,0, Py_BuildValue("i", rc));
      PyList_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
      return tuple;
  } 
  PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
  /* query glist for length and create python list obj with 
   necessary length */
  n = globus_list_size(glist);
  list = PyList_New(n);

  /* populate python list with PFNs stored in glist */
  p = glist;
  for(i = 0; i < n; i++){ 
      str2 = (globus_rls_string2_t *) globus_list_first(p);
      p = globus_list_rest(p);
      PyList_SetItem(list, i, PyString_FromString(str2 -> s2));
  }

  /* we can now free the glist */
  
  globus_rls_client_free_list(glist);
  PyTuple_SET_ITEM(tuple, 1, list);
  return tuple;
}

PyObject *
lrc_get_pfn_wc(globus_rls_handle_t *handle, char *lfnpat, globus_rls_pattern_t pattype, 
	       int offset, int reslimit)
{
  globus_result_t	ret;
  int			n;
  int			i;
  globus_list_t	       *glist;
  globus_list_t	       *p;
  globus_rls_string2_t *str2;
  char                  tmpstr[256];
  int                   rc;
  PyObject	       *dict, *tuple;
  

   if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "lrc_get_pfn_wc: PyTuple_New()");
        return NULL;
    }
  Py_BEGIN_ALLOW_THREADS 
   ret = globus_rls_client_lrc_get_pfn_wc(handle, lfnpat, pattype, &offset, reslimit,
					  &glist);
   Py_END_ALLOW_THREADS

  /* query the LRC using the LFN pattern with result stuffed into glist*/
  if (ret != GLOBUS_SUCCESS)
  {
      /* send back the non-zero integer return value to indicate failure   
       at the rls API level */
      globus_rls_client_error_info(ret, &rc,tmpstr,256,GLOBUS_FALSE); 
      PyTuple_SET_ITEM(tuple,0, Py_BuildValue("i", rc));
      PyTuple_SET_ITEM(tuple,1, Py_BuildValue("s", tmpstr));
      return tuple;
  } 

    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));

  /* query glist for length and create python dict to store results */
  n = globus_list_size(glist);
  dict = PyDict_New();

  /* populate python dict with LFNs as keys and PFNs as values */
  p = glist;
  for(i = 0; i < n; i++){ 
      str2 = (globus_rls_string2_t *) globus_list_first(p);
      p = globus_list_rest(p);
      PyDict_SetItemString(dict, str2 -> s1, PyString_FromString(str2 -> s2));
  }

  PyTuple_SET_ITEM(tuple, 1, dict);
  /* we can now free the glist */
  globus_rls_client_free_list(glist);

  return tuple;
}

PyObject *
lrc_get_lfn_wc(globus_rls_handle_t *handle, char *pfnpat, 
	       globus_rls_pattern_t pattype, int offset, int reslimit)
{
  globus_result_t	ret;
  int			n;
  int			i;
  globus_list_t	       *glist;
  globus_list_t	       *p;
  globus_rls_string2_t *str2;
  char                  tmpstr[256];
  int                   rc;
  PyObject	       *dict, *tuple;

   if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "lrc_get_lfn_wc: PyTuple_New()");
        return NULL;
    }

   Py_BEGIN_ALLOW_THREADS 
       ret = globus_rls_client_lrc_get_lfn_wc(handle, pfnpat, 
					      pattype, &offset, 
					      reslimit, &glist);
   Py_END_ALLOW_THREADS
     
  /* query the LRC using the PFN pattern with result stuffed into glist */
  if ( ret != GLOBUS_SUCCESS){
      /* send back the non-zero integer return value to indicate failure
       at the rls API level */
      globus_rls_client_error_info(ret, &rc,tmpstr,256,GLOBUS_FALSE); 
      PyTuple_SET_ITEM(tuple,0, Py_BuildValue("i", rc));
      PyTuple_SET_ITEM(tuple,1, Py_BuildValue("s", tmpstr));
      return tuple;
  } 
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));  
  /* query glist for length and create python dict to store results*/
  n = globus_list_size(glist);
  dict = PyDict_New();

  /* populate python dict with PFNs as keys and LFNs as values */
  p = glist;
  for(i = 0; i < n; i++){ 
      str2 = (globus_rls_string2_t *) globus_list_first(p);
      p = globus_list_rest(p);
      PyDict_SetItemString(dict, str2 -> s2, PyString_FromString(str2 -> s1));
  }

  /* we can now free the glist */
  globus_rls_client_free_list(glist);
  PyTuple_SET_ITEM(tuple,1, dict);
  
  return tuple;

}

PyObject *
lrc_create(globus_rls_handle_t *handle, char *lfn, char *pfn)
{
    globus_result_t             ret;
    PyObject                   *tuple;
    char                        tmpstr[256];
    int                         rc;

    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "lrc_create:: PyTuple_New()");
        return NULL;
    }

     Py_BEGIN_ALLOW_THREADS
     ret = globus_rls_client_lrc_create(handle,lfn, pfn);
     Py_END_ALLOW_THREADS
      
     PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
     if (ret != GLOBUS_SUCCESS)
     {
	 globus_rls_client_error_info(ret, &rc,tmpstr,256,GLOBUS_FALSE); 
	 PyTuple_SET_ITEM(tuple,0, Py_BuildValue("i", rc));
	 PyTuple_SET_ITEM(tuple,1, Py_BuildValue("s", tmpstr));
     }
     else
     {
	 Py_INCREF(Py_None);
	 PyTuple_SET_ITEM(tuple, 1, Py_None);
     }	
     return tuple; 
 
}   

PyObject *
lrc_add(globus_rls_handle_t *handle, char *lfn, char *pfn)
{
    globus_result_t             ret;
    PyObject                   *tuple;
    char                        tmpstr[256];
    int                         rc;

    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "lrc_add: PyTuple_New()");
        return NULL;
    }

     Py_BEGIN_ALLOW_THREADS
     ret = globus_rls_client_lrc_add(handle,lfn, pfn);
     Py_END_ALLOW_THREADS
      
     
     if (ret != GLOBUS_SUCCESS)
     {
	 globus_rls_client_error_info(ret, &rc,tmpstr,256,GLOBUS_FALSE); 
	 PyTuple_SET_ITEM(tuple,0, Py_BuildValue("i", rc));
	 PyTuple_SET_ITEM(tuple,1, Py_BuildValue("s", tmpstr));
     }
     else
     {
	 PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
	 Py_INCREF(Py_None);
	 PyTuple_SET_ITEM(tuple, 1, Py_None);
     }	
     return tuple; 
 
} 

PyObject *
lrc_delete(globus_rls_handle_t *handle, char *lfn, char *pfn)
{
    globus_result_t             ret;
    PyObject                   *tuple;
    char                        tmpstr[256];
    int                         rc;

    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "lrc_delete: PyTuple_New()");
        return NULL;
    }

     Py_BEGIN_ALLOW_THREADS
     ret = globus_rls_client_lrc_delete(handle,lfn, pfn);
     Py_END_ALLOW_THREADS
      
     
     if (ret != GLOBUS_SUCCESS)
     {
	 globus_rls_client_error_info(ret, &rc,tmpstr,256,GLOBUS_FALSE); 
	 PyTuple_SET_ITEM(tuple,0, Py_BuildValue("i", rc));
	 PyTuple_SET_ITEM(tuple,1, Py_BuildValue("s", tmpstr));
     }
     else
     {
	 PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
	 Py_INCREF(Py_None);
	 PyTuple_SET_ITEM(tuple, 1, Py_None);
     }	
     return tuple; 
 
} 
/*Needs to be redone */
PyObject *
lrc_exists(globus_rls_handle_t *handle, char *lfn, globus_rls_obj_type_t  objtype)
{
    globus_result_t             ret;
    PyObject                   *tuple;
    char                        tmpstr[256];
    int                         rc;

    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "lrc_exists: PyTuple_New()");
        return NULL;
    }

     Py_BEGIN_ALLOW_THREADS
     ret = globus_rls_client_lrc_exists(handle,lfn, objtype);
     Py_END_ALLOW_THREADS
      
     
     if (ret != GLOBUS_SUCCESS  )
     { 
	 globus_rls_client_error_info(ret, &rc,tmpstr,256,GLOBUS_FALSE); 
	 PyTuple_SET_ITEM(tuple,0, Py_BuildValue("i", rc));
	 PyTuple_SET_ITEM(tuple,1, Py_BuildValue("s", tmpstr));
     }
     else
     {
	 PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
	 Py_INCREF(Py_None);
	 PyTuple_SET_ITEM(tuple, 1, Py_None);
     }	
     return tuple; 
 
} 

PyObject *
lrc_attr_create(globus_rls_handle_t *handle, char *name, 
		globus_rls_obj_type_t  objtype, globus_rls_attr_type_t type)
{
    globus_result_t             ret;
    PyObject                   *tuple;
    char                       tmpstr[256];
    int                        rc;
    
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "lrc_attr_create: PyTuple_New()");
        return NULL;
    }

     Py_BEGIN_ALLOW_THREADS
     ret = globus_rls_client_lrc_attr_create(handle, name, objtype, type);
     Py_END_ALLOW_THREADS
    
     if (ret != GLOBUS_SUCCESS)
     {
	 globus_rls_client_error_info(ret, &rc,tmpstr,256,GLOBUS_FALSE); 
	 PyTuple_SET_ITEM(tuple,0, Py_BuildValue("i", rc));
	 PyTuple_SET_ITEM(tuple,1, Py_BuildValue("s", tmpstr));
     }
     else
     {
	   
	 PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
	 Py_INCREF(Py_None);
	 PyTuple_SET_ITEM(tuple, 1, Py_None);
     }	
     return tuple; 
 
} 

PyObject *
lrc_attr_search(globus_rls_handle_t *handle, char *name, 
		globus_rls_obj_type_t objtype,globus_rls_attr_op_t op,
		globus_rls_attribute_t *operand1,globus_rls_attribute_t *operand2,
		int offset,int reslimit)
{
    globus_result_t			ret;
    int					n;
    int					i;
    PyObject			       *dict, *tuple;
    globus_list_t		       *glist;
    globus_list_t		       *p;
    globus_rls_attribute_object_t      *res;
    globus_rls_attribute_t		myattr;
    char			       *mykey;
    char                               tmpstr[256];
    PyObject			       *myval;
    int                                 rc;

    
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "lrc_attr_search: PyTuple_New()");
        return NULL;
    }
    /* query the LRC with results stuffed into glist */
    Py_BEGIN_ALLOW_THREADS 
    ret = globus_rls_client_lrc_attr_search(handle, name, objtype, op, 
					    operand1, operand2, &offset, 
					    reslimit, &glist);
    Py_END_ALLOW_THREADS
     
    if (ret != GLOBUS_SUCCESS)
    {
	globus_rls_client_error_info(ret, &rc,tmpstr,256,GLOBUS_FALSE); 
	PyTuple_SET_ITEM(tuple,0, Py_BuildValue("i", rc));
	PyTuple_SET_ITEM(tuple,1, Py_BuildValue("s", tmpstr));
     
	return tuple;
    }
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));

    /* query glist for length and create a new python dict */
    n = globus_list_size(glist);
    dict = PyDict_New();

    /* loop picking off names and values
     returning them as keys and values in a dictionary */
    p = glist;
    for (i = 0; i < n; i++){
	res = (globus_rls_attribute_object_t *) globus_list_first(p);
	p = globus_list_rest(p);
	mykey = res -> key;
	myattr = res -> attr;

	switch (myattr.type){
	    case globus_rls_attr_type_date:
		myval = PyInt_FromLong((long) myattr.val.t);
		break;
	    case globus_rls_attr_type_flt:
		myval = PyFloat_FromDouble((double) myattr.val.d);
		break;
	    case globus_rls_attr_type_int:
		myval = PyInt_FromLong((long) myattr.val.i);
		break;
	    case globus_rls_attr_type_str:
		myval = PyString_FromString((char *) myattr.val.s);
		break;
	}

	PyDict_SetItemString(dict, mykey, myval);
    }


    /* free the glist */
    globus_rls_client_free_list(glist);

    PyTuple_SET_ITEM(tuple,1, dict);
    return tuple;

}

PyObject *
lrc_attr_delete(globus_rls_handle_t *handle, char *name, 
		globus_rls_obj_type_t objtype, int clear)
{
    globus_result_t        ret;
    PyObject              *tuple;
    char                   tmpstr[256];
    int                    rc;

    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "lrc_attr_delete: PyTuple_New()");
        return NULL;
    }
    /* query the LRC with results stuffed into glist */
    Py_BEGIN_ALLOW_THREADS 
	ret = globus_rls_client_lrc_attr_delete(handle, name, objtype, (globus_bool_t) clear);
    Py_END_ALLOW_THREADS
    
    
    if (ret != GLOBUS_SUCCESS)
    {
	globus_rls_client_error_info(ret, &rc,tmpstr,256,GLOBUS_FALSE); 
	PyTuple_SET_ITEM(tuple,0, Py_BuildValue("i", rc));
	PyTuple_SET_ITEM(tuple,1, Py_BuildValue("s", tmpstr));
    }
    else
    {
	PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple,1, Py_None);
    }
	
       return tuple;
}

PyObject *
lrc_attr_add(globus_rls_handle_t *handle, char *key, globus_rls_attribute_t *attr)
{
    globus_result_t        ret;
    PyObject              *tuple;
    char                   tmpstr[256];
    int                    rc;

    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "lrc_attr_add: PyTuple_New()");
        return NULL;
    }
   
    Py_BEGIN_ALLOW_THREADS 
	ret = globus_rls_client_lrc_attr_add(handle, key, attr);
    Py_END_ALLOW_THREADS
    
   
    if (ret != GLOBUS_SUCCESS)
    {
	globus_rls_client_error_info(ret, &rc,tmpstr,256,GLOBUS_FALSE); 
	PyTuple_SET_ITEM(tuple,0, Py_BuildValue("i", rc));
	PyTuple_SET_ITEM(tuple,1, Py_BuildValue("s", tmpstr));
    }
    else
    {
	PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple,1, Py_None);
    }
	
       return tuple;
}
PyObject *
lrc_attr_remove(globus_rls_handle_t *handle, char *key, globus_rls_attribute_t *attr)
{
    globus_result_t        ret;
    PyObject              *tuple;
    char                   tmpstr[256];
    int                    rc;  

    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "lrc_attr_remove: PyTuple_New()");
        return NULL;
    }
    /* query the LRC with results stuffed into glist */
    Py_BEGIN_ALLOW_THREADS 
	ret = globus_rls_client_lrc_attr_remove(handle, key, attr);
    Py_END_ALLOW_THREADS
    
    
    if (ret != GLOBUS_SUCCESS)
    {
	globus_rls_client_error_info(ret, &rc,tmpstr,256,GLOBUS_FALSE); 
	PyTuple_SET_ITEM(tuple,0, Py_BuildValue("i", rc));
	PyTuple_SET_ITEM(tuple,1, Py_BuildValue("s", tmpstr));
	
    }
    else
    {
	PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple,1, Py_None);
    }
	
       return tuple;
}

PyObject *
lrc_attr_get(globus_rls_handle_t *handle, char *name,
	     globus_rls_obj_type_t objtype)
{
    globus_result_t		ret;		
    int				i;
    int				n;
    char		       *searchname;
    char		       *empty = "";
    char		       *stringval;
    char                        tmpstr[256];
    int                         rc;
    globus_list_t	       *glist;
    globus_list_t	       *p;
    PyObject		       *dict;
    PyObject		       *list;
    PyObject		       *newval;
    PyObject                   *tuple;
    globus_rls_attribute_t     *attr;
    
    
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "lrc_attr_get: PyTuple_New()");
        return NULL;
    }
    /* query the LRC with results stuffed into glist */
    if (strcmp(name, empty) == 0){ 
	searchname = (char *) NULL; 
    }
    else { 
	searchname = name;
    }
    
    Py_BEGIN_ALLOW_THREADS
    ret = globus_rls_client_lrc_attr_get(handle, searchname, objtype, &glist);
    Py_END_ALLOW_THREADS
    
   
    if (ret != GLOBUS_SUCCESS){
	globus_rls_client_error_info(ret, &rc,tmpstr,256,GLOBUS_FALSE); 
	PyTuple_SET_ITEM(tuple,0, Py_BuildValue("i", rc));
	PyTuple_SET_ITEM(tuple,1, Py_BuildValue("s", tmpstr));
	return tuple;
    }
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    /* query glist for length and create a new python dict */
    n = globus_list_size(glist);
    dict = PyDict_New();

    /* loop picking off names and values and returning them
     as keys and list of values */
    p = glist;
    for (i = 0; i < n; i++){
	attr = (globus_rls_attribute_t *) globus_list_first(p);
	p = globus_list_rest(p);

	switch (attr -> type){
	    case globus_rls_attr_type_date:
		newval = PyInt_FromLong((long) attr -> val.t);
		break;
	    case globus_rls_attr_type_flt:
		newval = PyFloat_FromDouble((double) attr -> val.d);
		break;
	    case globus_rls_attr_type_int:
		newval = PyInt_FromLong((long) attr -> val.i);
		break;
	    case globus_rls_attr_type_str:
		stringval = (char *) attr -> val.s;
		if (stringval == NULL){
		    newval = PyString_FromString("");
		}
		else {
			newval = PyString_FromString((char *) attr -> val.s);
		}
		break;
	}

	list = PyList_New(3);
	PyList_SetItem(list, 0, PyInt_FromLong((long) attr -> objtype));
	PyList_SetItem(list, 1, PyInt_FromLong((long) attr -> type));
	PyList_SetItem(list, 2, newval);

	PyDict_SetItemString(dict, attr -> name, list);
    }

    fflush(stdout);


    /* free the glist */
    globus_rls_client_free_list(glist);
    PyTuple_SET_ITEM(tuple,1, dict);
    return tuple;

}

PyObject *
lrc_attr_value_get(globus_rls_handle_t *handle, char *fn, 
		   globus_rls_obj_type_t objtype, char * attrname)
{
    globus_result_t		ret;
    int				i;
    int				n;
    char		       *searchname;
    char		       *empty = "";
    char		       *stringval;
    char                        tmpstr[256];
    int                         rc;
    globus_list_t	       *glist;
    globus_list_t	       *p;
    PyObject		       *dict;
    PyObject		       *list;
    PyObject		       *newval;
    PyObject                   *tuple;
    globus_rls_attribute_t     *attr;

    /* query the LRC with results stuffed into glist */
    if (strcmp(attrname, empty) == 0){ 
	searchname = (char *) NULL; 
    }
    else { 
	searchname = attrname;
    }
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "lrc_attr_value_get: PyTuple_New()");
        return NULL;
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_rls_client_lrc_attr_value_get(handle, fn, searchname, objtype, &glist);
    Py_END_ALLOW_THREADS
    
   
    if (ret != GLOBUS_SUCCESS)
    {
	globus_rls_client_error_info(ret, &rc,tmpstr,256,GLOBUS_FALSE); 
	PyTuple_SET_ITEM(tuple,0, Py_BuildValue("i", rc));
	PyTuple_SET_ITEM(tuple,1, Py_BuildValue("s", tmpstr));
	return tuple;
    }
     PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    /* query glist for length and create a new python dict */
    n = globus_list_size(glist);
    dict = PyDict_New();

    /* loop picking off names and values and returning them
     as keys and list of values */
    p = glist;
    for (i = 0; i < n; i++){
	attr = (globus_rls_attribute_t *) globus_list_first(p);
	p = globus_list_rest(p);

	switch (attr -> type){
	    case globus_rls_attr_type_date:
		newval = PyInt_FromLong((long) attr -> val.t);
		break;
	    case globus_rls_attr_type_flt:
		newval = PyFloat_FromDouble((double) attr -> val.d);
		break;
	    case globus_rls_attr_type_int:
		newval = PyInt_FromLong((long) attr -> val.i);
		break;
	    case globus_rls_attr_type_str:
		stringval = (char *) attr -> val.s;
		if (stringval == NULL){
		    newval = PyString_FromString("");
		}
		else {
			newval = PyString_FromString((char *) attr -> val.s);
		}
		break;
	}

	list = PyList_New(3);
	PyList_SetItem(list, 0, PyInt_FromLong((long) attr -> objtype));
	PyList_SetItem(list, 1, PyInt_FromLong((long) attr -> type));
	PyList_SetItem(list, 2, newval);

	PyDict_SetItemString(dict, attr -> name, list);
    }


    /* free the glist */
    globus_rls_client_free_list(glist);
    
    PyTuple_SET_ITEM(tuple,1, dict);
    return tuple;
    
}

PyObject *
lrc_rli_add(globus_rls_handle_t *handle, char *rli_url, int flags, char * pattern)
{
    globus_result_t        ret;
    PyObject              *tuple;
    char                   tmpstr[256];
    int                    rc;
    int                    flags_passed; 
    
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "lrc_rli_add: PyTuple_New()");
        return NULL;
    }
    /* query the LRC with results stuffed into glist */
    flags_passed = 0;
    if (flags != 0)
    {
       flags_passed = FRLI_BLOOMFILTER;
    } 

    Py_BEGIN_ALLOW_THREADS 
	ret = globus_rls_client_lrc_rli_add(handle,rli_url, flags_passed,pattern);
    Py_END_ALLOW_THREADS
    
    
    if (ret != GLOBUS_SUCCESS)
    {
	globus_rls_client_error_info(ret, &rc,tmpstr,256,GLOBUS_FALSE); 
	PyTuple_SET_ITEM(tuple,0, Py_BuildValue("i", rc));
	PyTuple_SET_ITEM(tuple,1, Py_BuildValue("s", tmpstr));
      
    }
    else
    {
	PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple,1, Py_None);
    }
	
       return tuple;
}

PyObject *
lrc_rli_delete(globus_rls_handle_t *handle, char *rli_url, char * pattern)
{
    globus_result_t        ret;
    PyObject              *tuple;
    char                   tmpstr[256];
    int                    rc;

    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "lrc_rli_delete: PyTuple_New()");
        return NULL;
    }
    /* query the LRC with results stuffed into glist */
    Py_BEGIN_ALLOW_THREADS 
	ret = globus_rls_client_lrc_rli_delete(handle,rli_url,pattern);
    Py_END_ALLOW_THREADS
    
    
    if (ret != GLOBUS_SUCCESS)
    {
	globus_rls_client_error_info(ret, &rc,tmpstr,256,GLOBUS_FALSE); 
	PyTuple_SET_ITEM(tuple,0, Py_BuildValue("i", rc));
	PyTuple_SET_ITEM(tuple,1, Py_BuildValue("s", tmpstr));
	
    }
    else
    {
	PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple,1, Py_None);
    }
	
       return tuple;
}

PyObject *
lrc_get_lfn(globus_rls_handle_t *handle, char *pfn, int offset, int reslimit)
{
  globus_result_t	ret;
  int			n;
  int			i;
  globus_list_t	       *glist;
  globus_list_t	       *p;
  globus_rls_string2_t *str2;
  char                  tmpstr[256];
  int                   rc;
  PyObject	       *list;
  PyObject             *tuple; 

  if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "lrc_get_lfn: PyTuple_New()");
        return NULL;
    }
  /* query the LRC using the PFN with result stuffed into glist */
   Py_BEGIN_ALLOW_THREADS
       ret = globus_rls_client_lrc_get_lfn(handle, pfn, &offset, reslimit,&glist);
   Py_END_ALLOW_THREADS
   
    
   if (ret != GLOBUS_SUCCESS)
   {
       globus_rls_client_error_info(ret, &rc,tmpstr,256,GLOBUS_FALSE); 
       PyTuple_SET_ITEM(tuple,0, Py_BuildValue("i", rc));
       PyTuple_SET_ITEM(tuple,1, Py_BuildValue("s", tmpstr));
       return tuple;  
   } 
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));  
  /* query glist for length and create python list obj with 
   necessary length */
  n = globus_list_size(glist);
  list = PyList_New(n);

  /* populate python list with LFNs stored in glist*/
  p = glist;
  for(i = 0; i < n; i++){ 
      str2 = (globus_rls_string2_t *) globus_list_first(p);
      p = globus_list_rest(p);
      PyList_SetItem(list, i, PyString_FromString(str2 -> s1));
  }

  /* we can now free the glist */
  globus_rls_client_free_list(glist);
  PyTuple_SET_ITEM(tuple,1, list);
  return tuple;
}


PyObject *
lrc_rli_list(globus_rls_handle_t *handle)
{
    globus_result_t		ret;
    int				i;
    int				n;
    globus_list_t	       *glist;
    globus_list_t	       *p;
    PyObject		       *dict;
    PyObject		       *list;
    PyObject		       *tuple;
    globus_rls_rli_info_t      *info;
    char                        tmpstr[256]; 
    int                         rc;                      
	 
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "lrc_rli_list: PyTuple_New()");
        return NULL;
    }
    /* query the LRC with results stuffed into glist */
     Py_BEGIN_ALLOW_THREADS
     ret = globus_rls_client_lrc_rli_list(handle, &glist);
      Py_END_ALLOW_THREADS
   
     
   if (ret != GLOBUS_SUCCESS)
   {
       globus_rls_client_error_info(ret, &rc,tmpstr,256,GLOBUS_FALSE); 
       PyTuple_SET_ITEM(tuple,0, Py_BuildValue("i", rc));
       PyTuple_SET_ITEM(tuple,1, Py_BuildValue("s", tmpstr));
       return tuple;  
   } 
     PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));  
     /* query glist for length and create a new python dict*/
    n = globus_list_size(glist);
    dict = PyDict_New();

    /* loop picking off names and values and returning them
     as keys and list of values */
    p = glist;
    for (i = 0; i < n; i++){
	info = (globus_rls_rli_info_t *) globus_list_first(p);
	p = globus_list_rest(p);

	list = PyList_New(3);
	PyList_SetItem(list, 0, PyInt_FromLong((long) info -> updateinterval));
	PyList_SetItem(list, 1, PyInt_FromLong((long) info -> flags));
	PyList_SetItem(list, 2, PyInt_FromLong((long) info -> lastupdate));

	PyDict_SetItemString(dict, info -> url, list);

    }

    /* free the glist */
    globus_rls_client_free_list(glist);
    PyTuple_SET_ITEM(tuple,1, dict);
    return tuple;

}

PyObject *
lrc_rli_info(globus_rls_handle_t *handle, char *url)
{
    globus_result_t		ret;
    PyObject		       *dict;
    PyObject		       *list, *tuple;
    globus_rls_rli_info_t	info;
    char                        tmpstr[256]; 
    int                         rc;

    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "lrc_rli_info: PyTuple_New()");
        return NULL;
    }

     Py_BEGIN_ALLOW_THREADS
    ret = globus_rls_client_lrc_rli_info(handle, url, &info);
       Py_END_ALLOW_THREADS
   
     
     
     if (ret != GLOBUS_SUCCESS)
     {
	 globus_rls_client_error_info(ret, &rc,tmpstr,256,GLOBUS_FALSE); 
	 PyTuple_SET_ITEM(tuple,0, Py_BuildValue("i", rc));
	 PyTuple_SET_ITEM(tuple,1, Py_BuildValue("s", tmpstr));
	 return tuple;  
     } 
     
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret)); 
    /* create a new python dict */
    dict = PyDict_New();

    list = PyList_New(3);
    PyList_SetItem(list, 0, PyInt_FromLong((long) info.updateinterval));
    PyList_SetItem(list, 1, PyInt_FromLong((long) info.flags));
    PyList_SetItem(list, 2, PyInt_FromLong((long) info.lastupdate));

    PyDict_SetItemString(dict, info.url, list);

    PyTuple_SET_ITEM(tuple, 1, dict);
    return tuple;

}


PyObject *
lrc_rli_get_part(globus_rls_handle_t *handle, char *rli_url, char *pattern)
{
    globus_result_t		ret;
    int				i;
    int				n;
    globus_list_t	       *glist;
    globus_list_t	       *p;
    PyObject		       *list, *tuple;
    PyObject		       *shortList;
    globus_rls_string2_t       *info;
    char                        tmpstr[256]; 
    int                         rc;
 
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "lrc_rli_get_part: PyTuple_New()");
        return NULL;
    }
    /* query the LRC with results stuffed into glist */
     Py_BEGIN_ALLOW_THREADS
     ret = globus_rls_client_lrc_rli_get_part(handle, rli_url, pattern, &glist);
     Py_END_ALLOW_THREADS
   
    
     
     if (ret != GLOBUS_SUCCESS)
     {
	 globus_rls_client_error_info(ret, &rc,tmpstr,256,GLOBUS_FALSE); 
	 PyTuple_SET_ITEM(tuple,0, Py_BuildValue("i", rc));
	 PyTuple_SET_ITEM(tuple,1, Py_BuildValue("s", tmpstr));
	 return tuple;  
     } 
   
       PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret)); 
    /* query glist for length and create a new python dict */
    n = globus_list_size(glist);
    list = PyList_New(n);

    /* loop picking off names and values and returning them
     as keys and list of values */
    p = glist;
    for (i = 0; i < n; i++){
	info = (globus_rls_string2_t *) globus_list_first(p);
	p = globus_list_rest(p);

	shortList = PyList_New(2);

	PyList_SetItem(shortList, 0, PyString_FromString(info -> s1));
	PyList_SetItem(shortList, 1, PyString_FromString(info -> s2));

	PyList_SetItem(list, i, shortList);

    }

    /* free the glist */
    globus_rls_client_free_list(glist);
     PyTuple_SET_ITEM(tuple, 1, list);
    return tuple;
   
}

PyObject *
rli_exists(globus_rls_handle_t *handle, char *key, 
	   globus_rls_obj_type_t objtype)
{
     globus_result_t	ret;
     PyObject          *tuple;
     char               tmpstr[256];
     int                rc;
     
     if (!(tuple=PyTuple_New(2)))
     {
	 PyErr_SetString(PyExc_RuntimeError, 
			 "rli_exists: PyTuple_New()");
	 return NULL;
     }
     /* query the LRC with results stuffed into glist */
     Py_BEGIN_ALLOW_THREADS
     ret = globus_rls_client_rli_exists(handle, key, objtype);
     Py_END_ALLOW_THREADS

     if (ret != GLOBUS_SUCCESS)
     {
	 globus_rls_client_error_info(ret, &rc,tmpstr,256,GLOBUS_FALSE); 
	 PyTuple_SET_ITEM(tuple,0, Py_BuildValue("i", rc));
	 PyTuple_SET_ITEM(tuple,1, Py_BuildValue("s", tmpstr)); 
     }
     else
     {
	 PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret)); 
	 Py_INCREF(Py_None);
	 PyTuple_SET_ITEM(tuple, 1, Py_None);
     }
     
     return tuple;

}

PyObject *
rli_get_lrc(globus_rls_handle_t *handle, char *lfn, int offset, int reslimit)
{
  globus_result_t	ret;
  int			n;
  int			i;
  globus_list_t	       *glist;
  globus_list_t	       *p;
  globus_rls_string2_t *str2;
  PyObject	       *list, *tuple;
  char                  tmpstr[256];
  int                   rc;

  if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "rli_get_lrc: PyTuple_New()");
        return NULL;
    }
  Py_BEGIN_ALLOW_THREADS
      ret = globus_rls_client_rli_get_lrc(handle, lfn, &offset, reslimit, &glist);
  Py_END_ALLOW_THREADS
   
 
     
  if (ret != GLOBUS_SUCCESS)
  {
      globus_rls_client_error_info(ret, &rc,tmpstr,256,GLOBUS_FALSE); 
      PyTuple_SET_ITEM(tuple,0, Py_BuildValue("i", rc));
      PyTuple_SET_ITEM(tuple,1, Py_BuildValue("s", tmpstr));
      return tuple;  
  } 
   PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret)); 
  /* query glist for length and create python list obj with 
   necessary length */
  n = globus_list_size(glist);
  list = PyList_New(n);

  /* populate python list with LRCs stored in glist */
  p = glist;
  for(i = 0; i < n; i++){ 
      str2 = (globus_rls_string2_t *) globus_list_first(p);
      p = globus_list_rest(p);
      PyList_SetItem(list, i, PyString_FromString(str2 -> s2));
  }

  /* we can now free the glist */
  globus_rls_client_free_list(glist);
  PyTuple_SET_ITEM(tuple, 1, list);
  return tuple;
  
}


PyObject *
rli_get_lrc_wc(globus_rls_handle_t *handle, char *lfnpat,
	       globus_rls_pattern_t pattype, int offset, int reslimit)
{
  globus_result_t	ret;
  int			n;
  int			i;
  globus_list_t	       *glist;
  globus_list_t	       *p;
  globus_rls_string2_t *str2;
  PyObject	       *dict, *tuple;
  char                 tmpstr[256]; 
  int                  rc;

  if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "rli_get_lrc_wc: PyTuple_New()");
        return NULL;
    }

  Py_BEGIN_ALLOW_THREADS
  ret = globus_rls_client_rli_get_lrc_wc(handle, lfnpat, pattype, &offset, reslimit,
					 &glist);
   Py_END_ALLOW_THREADS
     
  if (ret != GLOBUS_SUCCESS)
  {
      globus_rls_client_error_info(ret, &rc,tmpstr,256,GLOBUS_FALSE); 
      PyTuple_SET_ITEM(tuple,0, Py_BuildValue("i", rc));
      PyTuple_SET_ITEM(tuple,1, Py_BuildValue("s", tmpstr));
      return tuple;  
  } 
  
   PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
  /* query glist for length and create python dict to store results */
  n = globus_list_size(glist);
  dict = PyDict_New();

  /* populate python dict with LFNs as keys and LRCs as values */
  p = glist;
  for(i = 0; i < n; i++){ 
      str2 = (globus_rls_string2_t *) globus_list_first(p);
      p = globus_list_rest(p);
      PyDict_SetItemString(dict, str2 -> s1, PyString_FromString(str2 -> s2));
  }

  /* we can now free the glist */
  globus_rls_client_free_list(glist);
  PyTuple_SET_ITEM(tuple, 1, dict);
  return tuple;
}

PyObject *
rli_lrc_list(globus_rls_handle_t *handle)
{
    globus_result_t		ret;
    int				i;
    int				n;
    globus_list_t	       *glist;
    globus_list_t	       *p;
    PyObject		       *list;
    PyObject		       *shortList;
    PyObject                   *tuple; 
    globus_rls_lrc_info_t      *info;
    char                        tmpstr[256];
    int                         rc;

    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "rli_lrc_list: PyTuple_New()");
        return NULL;
    }
    /* query the LRC with results stuffed into glist */
     Py_BEGIN_ALLOW_THREADS
     ret  = globus_rls_client_rli_lrc_list(handle, &glist);
     Py_END_ALLOW_THREADS
    
  if (ret != GLOBUS_SUCCESS)
  {
      globus_rls_client_error_info(ret, &rc,tmpstr,256,GLOBUS_FALSE); 
      PyTuple_SET_ITEM(tuple,0, Py_BuildValue("i", rc));
      PyTuple_SET_ITEM(tuple,1, Py_BuildValue("s", tmpstr));
      return tuple;  
  } 
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    /* query glist for length and create a new python dict */
    n = globus_list_size(glist);
    list = PyList_New(n);

    /* loop picking off names and values */
    p = glist;
    for (i = 0; i < n; i++){
	info = (globus_rls_lrc_info_t *) globus_list_first(p);
	p = globus_list_rest(p);

	shortList = PyList_New(2);

	PyList_SetItem(shortList, 0, PyString_FromString(info -> url));
	PyList_SetItem(shortList, 1, PyInt_FromLong((long) info -> lastupdate));

	PyList_SetItem(list, i, shortList);

    }

    /* free the glist */
    globus_rls_client_free_list(glist);
     PyTuple_SET_ITEM(tuple, 1, list);
     return tuple;

}

PyObject *
attr_init()
{
    globus_rls_attribute_t     *attr;
    PyObject                   *ptr;
    attr = globus_malloc(sizeof(globus_rls_attribute_t));
    ptr = SWIG_NewPointerObj((void *)attr, SWIGTYPE_p_globus_rls_attribute_t, 0);
    return ptr;
}

void
attr_free(globus_rls_attribute_t *attr)
{
    
    globus_free(attr);
}

void  
attr_set_name(globus_rls_attribute_t *attr, char * name)  
{
    attr->name = name;
}
 
PyObject *
attr_get_name(globus_rls_attribute_t *attr)
{
    char * name;
    PyObject * ptr;
    name = attr->name;
    ptr = Py_BuildValue("s", name);
    return ptr;
    
}   

void
attr_set_objtype(globus_rls_attribute_t *attr, globus_rls_obj_type_t objtype)
{
    attr->objtype = objtype;
}

globus_rls_obj_type_t
attr_get_objtype(globus_rls_attribute_t *attr)
{
    return attr->objtype;
}

void 
attr_set_type(globus_rls_attribute_t *attr, globus_rls_attr_type_t type)
{
    attr->type = type;
}

globus_rls_attr_type_t
attr_get_type(globus_rls_attribute_t *attr)
{
    return attr->type;
}


void
attr_set_time_value(globus_rls_attribute_t *attr, long time_long )
{
    char *timestr;
    attr->val.t = time_long;
    
}

void
attr_set_float_value(globus_rls_attribute_t *attr, double d)
{
    attr->val.d = d;
}

void
attr_set_int_value(globus_rls_attribute_t *attr, int i)
{
    attr->val.i = i;
}

void
attr_set_string_value(globus_rls_attribute_t *attr, char * s)
{
    attr->val.s = s;
}

long
attr_get_time_value(globus_rls_attribute_t *attr)
{
    time_t t;
    t = attr->val.t;  
    return t;
   
}

double
attr_get_float_value(globus_rls_attribute_t *attr)
{
    return attr->val.d;
}

int
attr_get_int_value(globus_rls_attribute_t *attr)
{
    return attr->val.i;
}

char *
attr_get_string_value(globus_rls_attribute_t *attr)
{
    return attr->val.s;
}

%}
