/*############################################################################
# Matt Rodriguez, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################*/
%module xioc
%{
#include "globus_common.h"
#include "globus_xio.h"
#include "globus_xio_tcp_driver.h"
#include "globus_xio_file_driver.h"
#include "globus_xio_types.h"
#include "pyGlobus_common.h"
%}

%include ./src/pyGlobus-typemaps.i
%types(globus_xio_server_t*);
typedef int globus_bool_t;

typedef enum 
{
    /**
     *  target/server attrs
     */
    /* globus_xio_system_handle_t       handle */
    GLOBUS_XIO_TCP_SET_HANDLE,
    
    /**
     *  target/server attrs cntls, handle cntl
     */
    /* globus_xio_system_handle_t *     handle_out */
    GLOBUS_XIO_TCP_GET_HANDLE,
    
    /**
     *  server attrs
     */
    /* const char *                     service_name */
    GLOBUS_XIO_TCP_SET_SERVICE,
    /* char **                          service_name_out */
    GLOBUS_XIO_TCP_GET_SERVICE,
    /* int                              listener_port */
    GLOBUS_XIO_TCP_SET_PORT,
    /* int *                            listener_port_out */
    GLOBUS_XIO_TCP_GET_PORT,
    /* int                              listener_backlog */
    GLOBUS_XIO_TCP_SET_BACKLOG,
    /* int *                            listener_backlog_out */
    GLOBUS_XIO_TCP_GET_BACKLOG,
    /* int                              listener_min_port */
    /* int                              listener_max_port */
    GLOBUS_XIO_TCP_SET_LISTEN_RANGE,
    /* int *                            listener_min_port_out */
    /* int *                            listener_max_port_out */
    GLOBUS_XIO_TCP_GET_LISTEN_RANGE,
    
    /**
     *  handle/server attrs
     */
    /* const char *                     interface */
    GLOBUS_XIO_TCP_SET_INTERFACE,
    /* char **                          interface_out */
    GLOBUS_XIO_TCP_GET_INTERFACE,
    /* globus_bool_t                    restrict_port */
    GLOBUS_XIO_TCP_SET_RESTRICT_PORT,
    /* globus_bool_t *                  restrict_port_out */
    GLOBUS_XIO_TCP_GET_RESTRICT_PORT,
    /* globus_bool_t                    resuseaddr */
    GLOBUS_XIO_TCP_SET_REUSEADDR,
    /* globus_bool_t *                  resuseaddr_out */
    GLOBUS_XIO_TCP_GET_REUSEADDR,
    /* globus_bool_t                    no_ipv6 */
    GLOBUS_XIO_TCP_SET_NO_IPV6,
    /* globus_bool_t *                  no_ipv6_out */
    GLOBUS_XIO_TCP_GET_NO_IPV6,
    
    /**
     *  handle attrs
     */
    /* int                              connector_min_port */
    /* int                              connector_max_port */
    GLOBUS_XIO_TCP_SET_CONNECT_RANGE,
    /* int *                            connector_min_port_out */
    /* int *                            connector_max_port_out */
    GLOBUS_XIO_TCP_GET_CONNECT_RANGE,
    
    /**
     *  handle attrs/cntls
     */
    /* globus_bool_t                    keepalive */
    GLOBUS_XIO_TCP_SET_KEEPALIVE,
    /* globus_bool_t *                  keepalive_out */
    GLOBUS_XIO_TCP_GET_KEEPALIVE,
    /* globus_bool_t                    linger */
    /* int                              linger_time */
    GLOBUS_XIO_TCP_SET_LINGER,
    /* globus_bool_t *                  linger_out */
    /* int *                            linger_time_out */
    GLOBUS_XIO_TCP_GET_LINGER,
    /* globus_bool_t                    oobinline */
    GLOBUS_XIO_TCP_SET_OOBINLINE,
    /* globus_bool_t *                  oobinline_out */
    GLOBUS_XIO_TCP_GET_OOBINLINE,
    /* int                              sndbuf */
    GLOBUS_XIO_TCP_SET_SNDBUF,
    /* int *                            sndbuf_out */
    GLOBUS_XIO_TCP_GET_SNDBUF,
    /* int                              rcvbuf */
    GLOBUS_XIO_TCP_SET_RCVBUF,
    /* int *                            rcvbuf_out */
    GLOBUS_XIO_TCP_GET_RCVBUF,
    /* globus_bool_t                    nodelay */
    GLOBUS_XIO_TCP_SET_NODELAY,
    /* globus_bool_t *                  nodelay_out */
    GLOBUS_XIO_TCP_GET_NODELAY,
    
    /**
     * data descriptors
     */
    /* int                              send_flags */
    GLOBUS_XIO_TCP_SET_SEND_FLAGS,
    /* int *                            send_flags_out */
    GLOBUS_XIO_TCP_GET_SEND_FLAGS,
    
    /**
     * handle/server/target cntls
     */
    /* char **                          contact_string_out */
    GLOBUS_XIO_TCP_GET_LOCAL_CONTACT,
    /* char **                          contact_string_out */
    GLOBUS_XIO_TCP_GET_LOCAL_NUMERIC_CONTACT,
    /* char **                          contact_string_out */
    GLOBUS_XIO_TCP_GET_REMOTE_CONTACT,
    /* char **                          contact_string_out */
    GLOBUS_XIO_TCP_GET_REMOTE_NUMERIC_CONTACT
    
} globus_xio_tcp_cmd_t;



typedef enum
{
    /* char **                          contact_string_out */
    GLOBUS_XIO_GET_LOCAL_CONTACT = 12345,

    /* char **                          contact_string_out */
    GLOBUS_XIO_GET_LOCAL_NUMERIC_CONTACT, 

    /* char **                          contact_string_out */
    GLOBUS_XIO_GET_REMOTE_CONTACT, 

    /* char **                          contact_string_out */
    GLOBUS_XIO_GET_REMOTE_NUMERIC_CONTACT, 

    /* globus_off_t                     offset */
    GLOBUS_XIO_SEEK

} globus_xio_handle_cmd_t; 

/* ===========================================================================
 *   NetLogger start
 */
/* If the variable NETLOGGER is defined,
 * this will add NetLogger write methods into
 * the xio_read/write calls, which will transparently
 * merge with the (less efficient) writes going
 * on at the Python level.
 * -Dan Gunter <dkgunter@lbl.gov> Nov 2005
 */
%{
int g_id = 0;
#ifdef NETLOGGER
#include "nl_log.h"

/**
 * Call these macros to write NetLogger events.
 * b = buflen, y = #bytes
 */
#define NL_EVENT_READ_START(b) fmtrec_b (nl_xio_rec[0],b)
#define NL_EVENT_READ_END(b,y) fmtrec_bb(nl_xio_rec[1],b,y)
#define NL_EVENT_WRITE_START(b) fmtrec_b (nl_xio_rec[2],b)
#define NL_EVENT_WRITE_END(b,y) fmtrec_bb(nl_xio_rec[3],b,y)

#define NL_XIO_MAXREC 1024
char    nl_xio_recbuf[NL_XIO_MAXREC];
NL_rec_t *nl_xio_rec[4];
NL_rfmt_t *nl_xio_rfp;
int        nl_xio_fd = -1;
int        nl_xio_off = 1;
unsigned int  nl_xio_seq;

#define NL_XIO_BUFLEN_FLD 4
#define NL_XIO_BYTES_FLD  5
/* # fields in the two record types */
#define NL_XIO_B_RECSZ 5
#define NL_XIO_BB_RECSZ 6

static void nl_xio_init(int fd);
static void xio_nl_on(int fd);
static void xio_nl_off(void);
static NL_rec_t *makerec(const char *event, int size);
static void addbuflen(NL_rec_t *recp);
static void addbytes(NL_rec_t *recp);

/* Initialize output */
void xio_nl_init(void) {
    NL_rec_t *recp;

    recp = nl_xio_rec[0] = 
	makerec("xio.XioSocket.read.start",NL_XIO_B_RECSZ);
    addbuflen(recp);
    recp = nl_xio_rec[1] = 
	makerec("xio.XioSocket.read.end",NL_XIO_BB_RECSZ);
    addbuflen(recp), addbytes(recp);    
    recp = nl_xio_rec[2] = 
	makerec("xio.XioSocket.write.start",NL_XIO_B_RECSZ);
    addbuflen(recp);    
    recp = nl_xio_rec[3] = 
	makerec("xio.XioSocket.write.end",NL_XIO_BB_RECSZ);
    addbuflen(recp), addbytes(recp);    
 
    nl_xio_rfp = NL_rfmt();
    nl_xio_seq = 0U;
}

/* Turn output on/off */
void xio_nl_on(int fd) { nl_xio_off = 0; nl_xio_fd = fd; }
void xio_nl_off() { nl_xio_off = 1; }

/* Finalize output */
void xio_nl_finalize(void) {
}

/**
 * Construct a single record with DATE, LVL, and EVNT fields
 * 'event' is name of EVNT, and size is total # of fields for record.
 */
NL_rec_t *makerec(const char *event, int size) 
{
    int ival;
    char sval[NL_MAX_STR];
    char *hostname;
    NL_rec_t *recp;

    recp = NL_rec(size);    
    NL_rec_add(recp, NL_fld(NL_FLD_DATE, NL_FLD_DATE_LEN, sval, 
                            sizeof(struct timeval),NL_time));
    NL_rec_add(recp, NL_fld(NL_FLD_LVL, NL_FLD_LVL_LEN, "DEBUG", 
                            5, NL_string));
    NL_rec_add(recp, NL_fld(NL_FLD_EVENT, NL_FLD_EVENT_LEN, (char*)event,
                            strlen(event), NL_string));    
    hostname = ipaddr(); /* defined in nl_log.h */
    if ( NULL == hostname ) {
	hostname = strdup("0.0.0.0");
    }
    NL_rec_add(recp, NL_fld(NL_FLD_HOST, NL_FLD_HOST_LEN, hostname,
                            strlen(hostname), NL_string));
    return recp;
}

/** Add 'buflen' integer field to record. */
void addbuflen(NL_rec_t *recp) 
{
    int ival = 0;
    NL_rec_add(recp, NL_fld("buflen", 6,  &ival, sizeof(ival), NL_int));
}

/** Add 'bytes' integer fields to record */
void addbytes(NL_rec_t *recp) 
{
    int ival = 0;
    NL_rec_add(recp, NL_fld("bytes", 5,  &ival, sizeof(ival), NL_int));
}


#define FMTREC_FINISH                                                     \
    memcpy( recp->fields[NL_dtfld]->value, &tv, sizeof(struct timeval));  \
    len = NL_rfmt_format(nl_xio_rfp, recp, nl_xio_recbuf, NL_XIO_MAXREC); \
    write(nl_xio_fd,nl_xio_recbuf,len);

/** Format a record with a buflen field. */
void fmtrec_b(NL_rec_t *recp, int buflen)
{
    int len;
    struct timeval tv;

    if ( nl_xio_fd < 0 || nl_xio_off ) return;
    gettimeofday( &tv, 0 );

    memcpy( recp->fields[NL_dtfld]->value, &tv, sizeof(struct timeval));    
    memcpy(((recp->fields)[NL_XIO_BUFLEN_FLD])->value, &buflen, sizeof(int));

    FMTREC_FINISH
}

/** Format a record with a buflen and a bytes field */
void fmtrec_bb(NL_rec_t *recp, int buflen, int bytes)
{
    int len;
    struct timeval tv;

    if ( nl_xio_fd < 0 || nl_xio_off ) return;
    gettimeofday( &tv, 0 );

    memcpy(((recp->fields)[NL_XIO_BUFLEN_FLD])->value, &buflen, sizeof(int));
    memcpy(((recp->fields)[NL_XIO_BYTES_FLD])->value, &bytes, sizeof(int));

    FMTREC_FINISH
}

#else

  void xio_nl_init() { ; }
  void xio_nl_on(int fd) { ; }
  void xio_nl_off(void) { ; }
  void xio_nl_finalize(void) { ; }
#define NL_EVENT_READ_START(b)
#define NL_EVENT_READ_END(b,y)
#define NL_EVENT_WRITE_START(b)
#define NL_EVENT_WRITE_END(b,y)

#endif
%}

void xio_nl_init(void);
void xio_nl_on(int fd);
void xio_nl_off(void);
void xio_nl_finalize(void);
int g_id;

/* NetLogger end
 * ======================================================================
 */

%init %{
    PyEval_InitThreads();
    python_interp = PyThreadState_Get()->interp;
    init_globus_thread_state_key(&python_thread_state_key);
%}

%{

static globus_thread_key_t python_thread_state_key;
static PyInterpreterState *python_interp;
 

static void PythonXioAcceptCallback(globus_xio_server_t server, globus_xio_handle_t handle, globus_result_t rslt, void * user_data)
{
    PyObject * func, *arglist, *ptr,*ptr1, *arg,*err,*result,*tuple;
    callbackStruct* cbStruct = (callbackStruct*)user_data;
    globus_xio_handle_t *g_handle;
    PyEval_AcquireThread(get_python_thread_state(python_thread_state_key, python_interp));
    func = cbStruct->callback;
    arg = cbStruct->arg;
    g_handle = (globus_xio_handle_t*)globus_malloc(sizeof(globus_xio_handle_t));
    *g_handle = handle;
    ptr = SWIG_NewPointerObj((void*)&server, SWIGTYPE_p_globus_xio_server_t, 0);
    ptr1 = SWIG_NewPointerObj((void*)g_handle, SWIGTYPE_p_globus_xio_handle_t, 0);
   
     if(!(tuple = PyTuple_New(2))){
    	PyErr_SetString(PyExc_RuntimeError,"PythonXioAcceptCallback, PyTuple_New");
    }
     else{
	PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", rslt));
	if (rslt != GLOBUS_SUCCESS) {
            char *tmpstr;
            tmpstr = globus_object_printable_to_string(globus_error_get(rslt));
            PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
            globus_libc_free(tmpstr);
        }
	else{
            Py_INCREF(Py_None);
            PyTuple_SET_ITEM(tuple, 1, Py_None);
	}
     } 
         if ((err = PyErr_Occurred()) != NULL){
	PyErr_Print();
    } 	
  
    arglist = Py_BuildValue("(OOOO)", arg, ptr,ptr1, tuple);
    result = PyEval_CallObject(func, arglist);
    if (result == NULL) {
      PyErr_Print();
      PyErr_Clear();
    }
    Py_DECREF(arglist);
    Py_XDECREF(result);
    Py_DECREF(ptr);
    Py_DECREF(ptr1);
    PyEval_ReleaseThread(get_python_thread_state(python_thread_state_key, python_interp));
    return;
    

}

static void PythonXioCallback(globus_xio_handle_t handle, globus_result_t rslt, void * user_data)
{
    PyObject * func, *arglist, *ptr,*arg, *err, *result, *tuple;
    callbackStruct* cbStruct = (callbackStruct*)user_data;
    PyEval_AcquireThread(get_python_thread_state(python_thread_state_key, python_interp));
    func = cbStruct->callback;
    arg = cbStruct->arg;
    ptr = SWIG_NewPointerObj((void*)&handle, SWIGTYPE_p_globus_xio_handle_t, 0);
     if(!(tuple = PyTuple_New(2))){
    	PyErr_SetString(PyExc_RuntimeError,"PythonXioCallback, PyTuple_New");	
    }
     else{
	PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", rslt));
	if (rslt != GLOBUS_SUCCESS) {
            char *tmpstr;
            tmpstr = globus_object_printable_to_string(globus_error_get(rslt));
            PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
            globus_libc_free(tmpstr);
        }
	else{
            Py_INCREF(Py_None);
            PyTuple_SET_ITEM(tuple, 1, Py_None);
	}
     } 
         if ((err = PyErr_Occurred()) != NULL){
	PyErr_Print();
    } 	
  
    arglist = Py_BuildValue("(OOO)", arg, ptr, tuple);
    result = PyEval_CallObject(func, arglist);
    if (result == NULL) {
      PyErr_Print();
      PyErr_Clear();
    }
    Py_DECREF(arglist);
    Py_XDECREF(result);
    Py_DECREF(ptr);
    PyEval_ReleaseThread(get_python_thread_state(python_thread_state_key, python_interp));
    return;
}

static void PythonXioServerCallback(globus_xio_server_t server, void * user_data)
{
    PyObject * func, *arglist, *ptr,*arg, *err, *result;
    callbackStruct* cbStruct = (callbackStruct*)user_data;
    PyEval_AcquireThread(get_python_thread_state(python_thread_state_key, python_interp));
    func = cbStruct->callback;
    arg = cbStruct->arg;
    ptr = SWIG_NewPointerObj((void*)&server, SWIGTYPE_p_globus_xio_server_t, 0);
  
    arglist = Py_BuildValue("(OO)", arg, ptr);
    result = PyEval_CallObject(func, arglist);
    if (result == NULL) {
      PyErr_Print();
      PyErr_Clear();
    }
    Py_DECREF(arglist);
    Py_XDECREF(result);
    Py_DECREF(ptr);
    PyEval_ReleaseThread(get_python_thread_state(python_thread_state_key, python_interp));
    return;
}

static void PythonXioDataCallback(globus_xio_handle_t handle, globus_result_t rslt, globus_byte_t* buf, globus_size_t len,globus_size_t nbytes, globus_xio_data_descriptor_t data_desc, void * user_data)
{
    PyObject * func, *arglist, *ptr, *tuple, *buffer,*oldBuf, *ptr1, *arg, *err, *result;
    callbackStruct4* cbStruct = (callbackStruct4*)user_data;
    PyEval_AcquireThread(get_python_thread_state(python_thread_state_key, python_interp));
    func = cbStruct->callback;
    arg = cbStruct->arg;
    oldBuf = cbStruct->buffer;
    ptr = SWIG_NewPointerObj((void*)&handle, SWIGTYPE_p_globus_xio_handle_t, 0);
    if(!(tuple = PyTuple_New(2))){
    	PyErr_SetString(PyExc_RuntimeError,"PythonXioDataCallback, PyTuple_New");	
    }
     else{
	PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", rslt));
	if (rslt != GLOBUS_SUCCESS) {
            char *tmpstr;
            tmpstr = globus_object_printable_to_string(globus_error_get(rslt));
            PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
            globus_libc_free(tmpstr);
        }
	else{
            Py_INCREF(Py_None);
            PyTuple_SET_ITEM(tuple, 1, Py_None);
	}
     } 
         if ((err = PyErr_Occurred()) != NULL){
	PyErr_Print();
    } 	
    
    buffer = PyBuffer_FromReadWriteMemory((void *)buf, nbytes*sizeof(globus_byte_t));
    ptr1 = SWIG_NewPointerObj((void *) &data_desc, SWIGTYPE_p_globus_xio_data_descriptor_t,0);
    arglist = Py_BuildValue("(OOOOiO)", arg, ptr, tuple,buffer,nbytes,ptr1);
    result = PyEval_CallObject(func, arglist);
    if (result == NULL) {
      PyErr_Print();
      PyErr_Clear();
    }

    Py_DECREF(arglist);
    Py_XDECREF(result);
    Py_DECREF(buffer);
    Py_DECREF(ptr);
    Py_DECREF(ptr1);
    PyEval_ReleaseThread(get_python_thread_state(python_thread_state_key, python_interp));
    return;
}

globus_bool_t
PythonXioTimeoutCallback( globus_xio_handle_t handle,globus_xio_operation_type_t op,void  * user_arg)
{
    
    return GLOBUS_TRUE;
}

globus_bool_t
PythonXioTimeoutServerCallback( globus_xio_server_t server, globus_xio_operation_type_t op)
{
    
    return GLOBUS_TRUE;
}



%}

%inline%{
globus_module_descriptor_t*
get_module(void){
    return GLOBUS_XIO_MODULE;
}
void
free_callback(callbackStruct* cbStruct) {
    Py_XDECREF(cbStruct->callback);
    Py_XDECREF(cbStruct->arg);
    globus_free(cbStruct);
}

void
free_callback4(callbackStruct4 *cbStruct){
    Py_XDECREF(cbStruct->callback);
    Py_XDECREF(cbStruct->arg);
    Py_XDECREF(cbStruct->buffer);
    globus_free(cbStruct);
}    

PyObject *
attr_init()
{
    globus_xio_attr_t * attr;
    globus_result_t ret;
    PyObject *tuple, *ptr;
    char *tmpstr;

    if (!(tuple=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, "attr_init: PyTuple_New()");
        return NULL;
    }
    
    attr = (globus_xio_attr_t*)globus_malloc(sizeof(globus_xio_attr_t));
    ret = globus_xio_attr_init(attr);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    
    if (ret != GLOBUS_SUCCESS)
    {
	tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else{
	ptr = SWIG_NewPointerObj((void*)attr, SWIGTYPE_p_globus_xio_attr_t, 0);
	PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

PyObject *
attr_destroy(globus_xio_attr_t * attr)
{
    globus_result_t ret;
    PyObject *tuple;
    char *tmpstr;

    if (!(tuple=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, "attr_destroy: PyTuple_New()");
        return NULL;
    }
    ret = globus_xio_attr_destroy(*attr);
    globus_free(attr);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    
    if (ret != GLOBUS_SUCCESS){
	tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else{
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}


PyObject *
attr_cntl_get_string(globus_xio_attr_t * attr, globus_xio_driver_t * driver, int cmd, PyObject *empty)
{
   globus_result_t ret;
   PyObject *tuple;
   char *tmpstr;
   char * ret_string = GLOBUS_NULL;
   
   if (!(tuple=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, "attr_cntl: PyTuple_New()");
        return NULL;
    } 
    ret = globus_xio_attr_cntl(*attr, *driver, cmd, &ret_string); 
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));

    if (ret != GLOBUS_SUCCESS){
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
        if (ret_string != GLOBUS_NULL){
            globus_free(ret_string);
        } 
    }
    else{ 
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", ret_string));
        globus_free(ret_string);
        
    }
    return tuple;
}

    
PyObject *
attr_cntl_get_bool(globus_xio_attr_t * attr, globus_xio_driver_t * driver, int cmd, PyObject *empty)
{
   globus_result_t ret;
   PyObject *tuple;
   char *tmpstr;
   globus_bool_t ret_bool; 
   
   if (!(tuple=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, "attr_cntl: PyTuple_New()");
        return NULL;
    } 
    ret = globus_xio_attr_cntl(*attr, *driver, cmd, &ret_bool); 
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));

    if (ret != GLOBUS_SUCCESS){
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else if (ret_bool) {
        Py_INCREF(Py_True);
        PyTuple_SET_ITEM(tuple, 1, Py_True);
    }
    else{
        Py_INCREF(Py_False);
        PyTuple_SET_ITEM(tuple, 1, Py_False); 
    }
    
    return tuple; 
}

PyObject *
attr_cntl_get_bool_int(globus_xio_attr_t * attr, globus_xio_driver_t * driver, int cmd, PyObject *empty)
{
    globus_result_t ret;
    PyObject *tuple1, *tuple2;
    char *tmpstr;
    globus_bool_t ret_bool; 
    int ret_int;
    
    if (!(tuple1=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, "attr_cntl: PyTuple_New()");
        return NULL;
    } 
    if (!(tuple2=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, "attr_cntl: PyTuple_New()");
        return NULL;
    } 
    ret = globus_xio_attr_cntl(*attr, *driver, cmd, &ret_bool, &ret_int); 
    PyTuple_SET_ITEM(tuple1, 0, Py_BuildValue("i", ret));

    if (ret != GLOBUS_SUCCESS){
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple1, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr); 
    }
    else{
        if (ret_bool){ 
            Py_INCREF(Py_True);   
            PyTuple_SET_ITEM(tuple2, 0, Py_True);
        } 
        else {
            Py_INCREF(Py_False);   
            PyTuple_SET_ITEM(tuple2, 0, Py_False);
            
        }            

        PyTuple_SET_ITEM(tuple2, 1, Py_BuildValue("i", ret_int));
        PyTuple_SET_ITEM(tuple1, 1, tuple2);
         
    }
    return tuple1; 
} 

PyObject *
attr_cntl_get_int(globus_xio_attr_t * attr, globus_xio_driver_t * driver, int cmd, PyObject *empty)
{
   globus_result_t ret;
   PyObject *tuple;
   char *tmpstr;
   int ret_int;
   
   if (!(tuple=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, "attr_cntl: PyTuple_New()");
        return NULL;
    } 
    ret = globus_xio_attr_cntl(*attr, *driver, cmd, &ret_int); 
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret)); 
    if (ret != GLOBUS_SUCCESS){
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else{ 
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("i", ret_int));
    }
    return tuple; 
}


PyObject *
attr_cntl_get_int_int(globus_xio_attr_t * attr, globus_xio_driver_t * driver, int cmd, PyObject *empty)
{
    globus_result_t ret;
    PyObject *tuple1, *tuple2;
    char *tmpstr;
    int ret_int1, ret_int2;
    
    if (!(tuple1=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, "attr_cntl: PyTuple_New()");
        return NULL;
    } 
    if (!(tuple2=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, "attr_cntl: PyTuple_New()");
        return NULL;
    } 
    ret = globus_xio_attr_cntl(*attr, *driver, cmd, &ret_int1, &ret_int2); 
    PyTuple_SET_ITEM(tuple1, 0, Py_BuildValue("i", ret));

    if (ret != GLOBUS_SUCCESS){
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple1, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple1, 2, Py_None);
    }
    else{ 
        PyTuple_SET_ITEM(tuple2, 0, Py_BuildValue("i", ret_int1)); 
        PyTuple_SET_ITEM(tuple2, 1, Py_BuildValue("i", ret_int2));
        PyTuple_SET_ITEM(tuple1, 1, tuple2);
    }
    return tuple1; 
}

/*Handles variable length args as long as there just ints and strings
and not longer than 2.*/
PyObject *
attr_cntl_set(globus_xio_attr_t * attr, globus_xio_driver_t * driver, int cmd, PyObject * arg_tuple)
{
    globus_result_t ret;
    PyObject *tuple, *py_arg1, *py_arg2;
    char *tmpstr, *arg_string1;
    int arg_int1, arg_int2; 
    const char * UNHANDLED_ARG = "Unhandled argument type in attr_cntl_set. This is not a globus error\n"; 
    if (!(tuple=PyTuple_New(2))){
         PyErr_SetString(PyExc_RuntimeError, "attr_cntl: PyTuple_New()");
         return NULL;
     } 
    
    if (PyTuple_GET_SIZE(arg_tuple)== 1){ 
        py_arg1 = PyTuple_GET_ITEM(arg_tuple,0); 
        if (PyInt_Check(py_arg1)){
            arg_int1 = PyInt_AsLong(py_arg1); 
            ret = globus_xio_attr_cntl(*attr, *driver, cmd, arg_int1); 
        }
        else if (PyString_Check(py_arg1)){ 
            arg_string1 = PyString_AsString(py_arg1); 
            ret = globus_xio_attr_cntl(*attr, *driver, cmd, arg_string1); 
        }     
        else {
            PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", -1));
            PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s",UNHANDLED_ARG)); 
            return tuple;
        }
      
    }
    if (PyTuple_GET_SIZE(arg_tuple) == 2){ 
        py_arg1 = PyTuple_GET_ITEM(arg_tuple,0); 
        py_arg2 = PyTuple_GET_ITEM(arg_tuple,1); 
        arg_int1 = PyInt_AsLong(py_arg1);     
        arg_int2 = PyLong_AsLong(py_arg2);     
        ret = globus_xio_attr_cntl(*attr, *driver, cmd, arg_int1, arg_int2); 
    }    
     
   PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
 
    if (ret != GLOBUS_SUCCESS){
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else{
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
    
} 

/*Specialized timeout functions */
PyObject *
attr_cntl_set_timeout_accept(globus_xio_attr_t * attr, int seconds){
    globus_result_t ret;
    PyObject *tuple;
    char * tmpstr;
    globus_reltime_t delay;

    if (!(tuple=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, "set_timeout: PyTuple_New()");
        return NULL;
    }
    GlobusTimeReltimeSet(delay, seconds, 0);
    ret = globus_xio_attr_cntl(*attr, NULL, GLOBUS_XIO_ATTR_SET_TIMEOUT_ACCEPT , PythonXioTimeoutCallback, &delay);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    
    if (ret != GLOBUS_SUCCESS){
	tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else{
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject *
attr_cntl_set_timeout_all(globus_xio_attr_t * attr, int seconds)
{
    globus_result_t ret;
    PyObject *tuple;
    char * tmpstr;
    globus_reltime_t delay;

    if (!(tuple=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, "set_timeout: PyTuple_New()");
        return NULL;
    }
    GlobusTimeReltimeSet(delay, seconds, 0);
    ret = globus_xio_attr_cntl(*attr, NULL, GLOBUS_XIO_ATTR_SET_TIMEOUT_ALL , PythonXioTimeoutCallback, &delay, NULL);
     PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    
     if (ret != GLOBUS_SUCCESS){
	tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else{
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
} 

PyObject *
attr_copy(globus_xio_attr_t * src_attr)
{
   globus_result_t ret;
   globus_xio_attr_t * dst_attr;
   PyObject *tuple, *ptr;
   char *tmpstr;
  
   if (!(tuple=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, "attr_copy: PyTuple_New()");
        return NULL;
    }
    
    dst_attr = (globus_xio_attr_t *)globus_malloc(sizeof(globus_xio_attr_t)); 
    ret = globus_xio_attr_copy(dst_attr,*src_attr); 
    if (ret != GLOBUS_SUCCESS){
	tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else{ 
        ptr = SWIG_NewPointerObj((void *)dst_attr, SWIGTYPE_p_globus_xio_attr_t,0);
	PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}


PyObject *
stack_init(globus_xio_attr_t * attr)
{
    globus_result_t ret;
    PyObject *tuple, *ptr;
    char *tmpstr;
    globus_xio_stack_t * stack;
    
    if (!(tuple=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, "stack_init: PyTuple_New()");
        return NULL;
    }
    stack = (globus_xio_stack_t *)globus_malloc(sizeof(globus_xio_stack_t));
    ret = globus_xio_stack_init(stack, *attr);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS){
	tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else{
	ptr = SWIG_NewPointerObj((void*)stack, SWIGTYPE_p_globus_xio_stack_t, 0);
	PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}
 
PyObject *
stack_destroy(globus_xio_stack_t *stack)
{
    globus_result_t ret;
    PyObject *tuple;
    char *tmpstr;

    if (!(tuple=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, "stack_destroy: PyTuple_New()");
        return NULL;
    }
    ret = globus_xio_stack_destroy(*stack);
    globus_free(stack);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS){
	tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else{
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
   
}
PyObject *
push_driver(globus_xio_stack_t * stack, globus_xio_driver_t * driver)
{
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;

    if (!(tuple=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, "push_driver: PyTuple_New()");
        return NULL;
    }
    ret = globus_xio_stack_push_driver(*stack, *driver);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS){
	tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else{
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

void
driver_free(globus_xio_driver_t * handle)
{
    globus_free(handle);
}

PyObject *
driver_load(const char * driver_string)
{
    globus_xio_driver_t *handle;
    globus_result_t ret;
    PyObject* tuple, *ptr;
    char* tmpstr;

    if (!(tuple=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, "driver_load: PyTuple_New()");
        return NULL;
    }
    handle = (globus_xio_driver_t *)globus_malloc(sizeof(globus_xio_driver_t));
    ret = globus_xio_driver_load(driver_string, handle);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS)
    {
	tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else{
	ptr = SWIG_NewPointerObj((void*)handle, SWIGTYPE_p_globus_xio_driver_t, 0);
	PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

PyObject *
driver_unload(globus_xio_driver_t * handle)
{
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;

    if (!(tuple=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, "driver_unload: PyTuple_New()");
        return NULL;
    }
    ret = globus_xio_driver_unload(*handle);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    
    if (ret != GLOBUS_SUCCESS){
	tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else{
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}


void
handle_free(globus_xio_handle_t *handle)
{
    globus_free(handle);
}

PyObject *
xio_handle_create(globus_xio_stack_t *stack)
{
    globus_result_t ret;
    PyObject* tuple, *ptr;
    char* tmpstr;
    globus_xio_handle_t * handle;

    if (!(tuple=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, "xio_handle_create: PyTuple_New()");
        return NULL;
    }
   
    handle = (globus_xio_handle_t *)globus_malloc(sizeof(globus_xio_handle_t));
    ret = globus_xio_handle_create(handle, *stack);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS){
	tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else{
	ptr = SWIG_NewPointerObj((void*)handle, SWIGTYPE_p_globus_xio_handle_t, 0);
	PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
} 

PyObject *
xio_open( globus_xio_attr_t * user_attr, const char * contact_string, globus_xio_handle_t* handle){
    globus_result_t ret;
    PyObject* tuple, *ptr;
    char* tmpstr;

    if (!(tuple=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, "xio_open: PyTuple_New()");
        return NULL;
    }


    ret = globus_xio_open(*handle, contact_string, *user_attr);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS){
	tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else{
	ptr = SWIG_NewPointerObj((void*)handle, SWIGTYPE_p_globus_xio_handle_t, 0);
	PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}


PyObject *
xio_register_open( globus_xio_attr_t * user_attr, const char * contact_string,globus_xio_handle_t * handle, PyObject *pyfunc,PyObject *arg){
    
    globus_result_t ret;
    PyObject* tuple, *ptr, *ptr1;
    char* tmpstr;
    callbackStruct* cbStruct;
    cbStruct = (callbackStruct*)globus_malloc(sizeof(callbackStruct));
    Py_XINCREF(pyfunc);
    Py_XINCREF(arg);
    cbStruct->callback = pyfunc;
    cbStruct->arg = arg;

    if (!(tuple=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, "xio_register_open: PyTuple_New()");
        return NULL;
    }
    
    ret = globus_xio_register_open(*handle, contact_string, *user_attr,PythonXioCallback,cbStruct);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS){
	tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
	globus_libc_free(cbStruct);
    }
    else{
	ptr = SWIG_NewPointerObj((void*)cbStruct, SWIGTYPE_p_callbackStruct,0);
	PyTuple_SET_ITEM(tuple, 1,ptr);
    }
    return tuple;
}

PyObject *
xio_close(globus_xio_handle_t * handle, globus_xio_attr_t * user_attr){
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr;

    if (!(tuple=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, "xio_close: PyTuple_New()");
        return NULL;
    }

    Py_BEGIN_ALLOW_THREADS
    ret = globus_xio_close(*handle, *user_attr);
    Py_END_ALLOW_THREADS
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    
    if (ret != GLOBUS_SUCCESS){
	tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else{
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject *
xio_register_close(globus_xio_handle_t * handle, globus_xio_attr_t * user_attr,PyObject *pyfunc, PyObject *arg){
    globus_result_t ret;
    PyObject* tuple, *ptr;
    char* tmpstr;
    callbackStruct* cbStruct;
    cbStruct = (callbackStruct*)globus_malloc(sizeof(callbackStruct));
    Py_XINCREF(pyfunc);
    Py_XINCREF(arg);
    cbStruct->callback = pyfunc;
    cbStruct->arg = arg;

    if (!(tuple=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, "xio_register_close: PyTuple_New()");
        return NULL;
    }

    Py_BEGIN_ALLOW_THREADS
    ret = globus_xio_register_close(*handle, *user_attr,PythonXioCallback, cbStruct);
    Py_END_ALLOW_THREADS
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS){
	tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
	globus_libc_free(cbStruct);
    }
    else{
	ptr = SWIG_NewPointerObj((void *)cbStruct, SWIGTYPE_p_callbackStruct, 0);
	PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

PyObject *
xio_read(globus_xio_handle_t * handle, globus_byte_t * buffer, int buffer_length, int waitforbytes,   PyObject  * py_data_desc){
    
    globus_result_t ret;
    PyObject * tuple, * buf, * obj, * attr_name;
    char* tmpstr;
    globus_size_t nbytes;
    globus_xio_data_descriptor_t data_desc = NULL;
    globus_xio_data_descriptor_t  * data_desc_ptr = &data_desc;
 
    NL_EVENT_READ_START(buffer_length);

    if (!(tuple=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, "xio_read: PyTuple_New()");
        return NULL;
    }  
    
    if (py_data_desc != Py_None){
        attr_name = PyString_FromString("_handle");
        obj = PyObject_GetAttr(py_data_desc, attr_name);
        SWIG_Python_ConvertPtr(obj, (void **)&data_desc_ptr,
        SWIGTYPE_p_globus_xio_data_descriptor_t, SWIG_POINTER_EXCEPTION | 0);
    }
 
    Py_BEGIN_ALLOW_THREADS
     ret = globus_xio_read(*handle,buffer, buffer_length, waitforbytes, &nbytes, data_desc);
    Py_END_ALLOW_THREADS
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS){
	tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else{ 
        buf = PyBuffer_FromMemory((void *)buffer,nbytes * sizeof(globus_byte_t));       
        PyTuple_SET_ITEM(tuple,1, Py_BuildValue("O", buf)); 
	Py_DECREF(buf); 
	
    }
    
    NL_EVENT_READ_END(buffer_length,nbytes);

    return tuple;
}

PyObject *
xio_register_read(globus_xio_handle_t * handle, PyObject * pyBuffer, globus_byte_t * buffer, int buffer_length, int waitforbytes,   globus_xio_data_descriptor_t  *data_desc, PyObject *pyfunc, PyObject *arg){
    
    globus_result_t ret;
    PyObject* tuple, *buf, *ptr;
    char* tmpstr;
    callbackStruct4* cbStruct;
    cbStruct = (callbackStruct4*)globus_malloc(sizeof(callbackStruct4));
 
    Py_XINCREF(pyfunc);
    Py_XINCREF(arg);
    Py_XINCREF(pyBuffer);
    cbStruct->callback = pyfunc;
    cbStruct->arg = arg;
    cbStruct->buffer = pyBuffer;

    if (!(tuple=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, "xio_register_read: PyTuple_New()");
        return NULL;
    }  
    Py_BEGIN_ALLOW_THREADS
    ret = globus_xio_register_read(*handle,buffer, buffer_length, waitforbytes,*data_desc, PythonXioDataCallback, cbStruct);
    Py_END_ALLOW_THREADS
   
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS){
	tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
	globus_libc_free(cbStruct);
    }
    else{
	ptr = SWIG_NewPointerObj((void *) cbStruct, SWIGTYPE_p_callbackStruct4, 0);
	PyTuple_SET_ITEM(tuple,1, ptr);	
    }
    return tuple;
}

PyObject *
xio_write(globus_xio_handle_t * handle, PyObject * buffer,int buffer_length, int waitforbytes, PyObject * py_data_desc){

    globus_result_t ret;
    PyObject * tuple, * obj, * attr_name;
    char * tmpstr;
    globus_size_t nbytes;
    globus_byte_t * buf;
    int buf_len, buf_ret; 
    globus_xio_data_descriptor_t data_desc = NULL;
    globus_xio_data_descriptor_t *data_desc_ptr = &data_desc;
     
    
    NL_EVENT_WRITE_START(buffer_length);

    if (!(tuple=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, "xio_write: PyTuple_New()");
        return NULL;
    }  
    
    if (py_data_desc != Py_None){ 
        attr_name = PyString_FromString("_handle");
        obj = PyObject_GetAttr(py_data_desc, attr_name);
        SWIG_Python_ConvertPtr(obj, (void **)&data_desc_ptr, 
        SWIGTYPE_p_globus_xio_data_descriptor_t, SWIG_POINTER_EXCEPTION | 0); 
    }
    
    buf_ret = PyObject_AsReadBuffer(buffer, (const void **)&buf, &buf_len); 
    
    if (buf_ret == -1){
        PyErr_SetString(PyExc_RuntimeError, "xio_write: PyObject_AsReadBuffer failed");
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_xio_write(*handle,buf, buffer_length, waitforbytes, &nbytes, data_desc);
    Py_END_ALLOW_THREADS
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    
    if (ret != GLOBUS_SUCCESS){
	tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else{
	PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("i",nbytes));
    }

    NL_EVENT_WRITE_END(buffer_length,nbytes);

    return tuple;

}

PyObject *
xio_register_write(globus_xio_handle_t * handle, PyObject * buffer,int buffer_length, int waitforbytes, globus_xio_data_descriptor_t * data_desc, PyObject *pyfunc, PyObject *arg){

    globus_result_t ret;
    PyObject* tuple, *ptr;
    char* tmpstr;
    int nbytes;
    globus_byte_t * buf;
    callbackStruct4* cbStruct;
    
    if (!(tuple=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, "xio_write: PyTuple_New()");
        return NULL;
    }  
    cbStruct = (callbackStruct4*)globus_malloc(sizeof(callbackStruct4));
    Py_XINCREF(pyfunc);
    Py_XINCREF(arg);
    Py_XINCREF(buffer);
    cbStruct->callback = pyfunc;
    cbStruct->arg = arg;
    cbStruct->buffer = buffer;	 
    buf = (globus_byte_t*)PyString_AsString(buffer);
    Py_BEGIN_ALLOW_THREADS
     ret = globus_xio_register_write(*handle,buf, buffer_length, waitforbytes, *data_desc, PythonXioDataCallback,cbStruct);
    Py_END_ALLOW_THREADS
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS){
	tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else{
	ptr = SWIG_NewPointerObj((void *) cbStruct, SWIGTYPE_p_callbackStruct4, 0);
	PyTuple_SET_ITEM(tuple,1, ptr);
    }
    return tuple;

}

PyObject *
xio_handle_cntl_return_string( globus_xio_handle_t *handle, globus_xio_driver_t *driver, int cmd){
    globus_result_t ret;
    PyObject *tuple;
    char * tmpstr;
    char * ret_string = GLOBUS_NULL;

    if (!(tuple=PyTuple_New(2))){
       PyErr_SetString(PyExc_RuntimeError, "xio_handle_cntl_return_string: PyTuple_New()");
       return NULL;
    }  
    
    ret = globus_xio_handle_cntl(*handle, *driver, cmd, &ret_string);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS){
        tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr); 
        if(ret_string != GLOBUS_NULL){ 
            globus_free(ret_string);
        }
    }
    else{
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", ret_string));
        globus_free(ret_string); 
        } 

    return tuple;
}    

PyObject *
server_create( globus_xio_attr_t * attr, globus_xio_stack_t *stack){ 
    globus_result_t ret;
    globus_xio_server_t * server;
    PyObject * tuple, *ptr;
    char * tmpstr; 
    
    if (!(tuple=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, "server_create: PyTuple_New()");
        return NULL;
    }  
    server = (globus_xio_server_t *)globus_malloc(sizeof(globus_xio_server_t)); 
    ret = globus_xio_server_create(server, *attr, *stack);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS){
	tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else{
        ptr = SWIG_NewPointerObj((void *)server, SWIGTYPE_p_globus_xio_server_t,0);
        PyTuple_SET_ITEM(tuple,1,ptr);	
    
    }
    return tuple;
}

void
server_free(globus_xio_server_t *server)
{
    globus_free(server);
}

PyObject *
server_cntl_return_string(globus_xio_server_t * server, globus_xio_driver_t *driver, int cmd)
{ 
    globus_result_t ret; 
    PyObject * tuple;
    char * tmpstr; 
    char * ret_string = GLOBUS_NULL;
    /**Another function that needs to have some support for va_list**/ 
    if (!(tuple=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, "server_cntl: PyTuple_New()");
        return NULL;
    }  
    ret = globus_xio_server_cntl(*server, *driver, cmd, &ret_string);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS)
    {
	tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_free(tmpstr);
        globus_free(ret_string);
    }
    else{
        Py_INCREF(Py_None); 
        PyTuple_SET_ITEM(tuple,1,Py_BuildValue("s", ret_string));	
        globus_free(ret_string); 
    }
    return tuple;
}

PyObject *
get_contact_string(globus_xio_server_t * server)
{
    globus_result_t ret; 
    PyObject* tuple;
    char* tmpstr; 
    char *contact_string=NULL;

    if (!(tuple=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, "server_get_contact: PyTuple_New()");
        return NULL;
    }  
    ret = globus_xio_server_get_contact_string(*server,&contact_string);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS)
    {
	tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    
    }
    else{
        Py_INCREF(Py_None); 
        PyTuple_SET_ITEM(tuple,1,Py_BuildValue("s",contact_string));	
	globus_free(contact_string);
    }
    return tuple;
}
PyObject *
server_close(globus_xio_server_t * server)
{ 
    globus_result_t ret; 
    PyObject* tuple;
    char* tmpstr; 
   
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "server_close: PyTuple_New()");
        return NULL;
    }  
    Py_BEGIN_ALLOW_THREADS
    ret = globus_xio_server_close(*server);
    Py_END_ALLOW_THREADS
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS)
    {
	tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else{
        Py_INCREF(Py_None); 
        PyTuple_SET_ITEM(tuple,1,Py_None);	
    
    }
    return tuple;
}

PyObject *
server_register_close(globus_xio_server_t * server, PyObject * pyfunc, PyObject *arg)
{ 
    globus_result_t ret; 
    PyObject* tuple, *ptr;
    char* tmpstr; 
    callbackStruct *cbStruct;
    Py_XINCREF(pyfunc);
    Py_XINCREF(arg);
    cbStruct = (callbackStruct *)globus_malloc(sizeof(callbackStruct));
    cbStruct->callback = pyfunc;
    cbStruct->arg = arg;

    if (!(tuple=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, "server_register_close: PyTuple_New()");
        return NULL;
    }  
    Py_BEGIN_ALLOW_THREADS
    ret = globus_xio_server_register_close(*server,PythonXioServerCallback, cbStruct);
    Py_END_ALLOW_THREADS
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS){
	tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else{
        ptr = SWIG_NewPointerObj((void *)cbStruct, SWIGTYPE_p_callbackStruct, 0);
        PyTuple_SET_ITEM(tuple,1,ptr);	
    }
    return tuple;
}

PyObject *
server_register_accept(globus_xio_server_t * server, PyObject * pyfunc, PyObject *arg)
{ 
    globus_result_t ret; 
    PyObject* tuple, *ptr;
    char* tmpstr; 
    callbackStruct* cbStruct;
    cbStruct = (callbackStruct*)globus_malloc(sizeof(callbackStruct));
    Py_XINCREF(pyfunc);
    Py_XINCREF(arg);
    cbStruct->callback = pyfunc;
    cbStruct->arg = arg;

    if (!(tuple=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, "server_register_accept: PyTuple_New()");
        return NULL;
    }  
  
    Py_BEGIN_ALLOW_THREADS
    ret = globus_xio_server_register_accept(*server,PythonXioAcceptCallback, cbStruct);
    Py_END_ALLOW_THREADS
    
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS){
	tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
	globus_libc_free(cbStruct);
    }
    else{
       ptr = SWIG_NewPointerObj((void *)cbStruct, SWIGTYPE_p_callbackStruct, 0);
       PyTuple_SET_ITEM(tuple,1, ptr); 
    }
    return tuple;
}

PyObject *
server_accept(globus_xio_server_t * server)
{ 
    globus_result_t ret; 
    PyObject* tuple, *ptr;
    char* tmpstr; 
    globus_xio_handle_t * handle;

    if (!(tuple=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, "server_accept: PyTuple_New()");
        return NULL;
    }  
    handle = (globus_xio_handle_t *)globus_malloc(sizeof(globus_xio_handle_t)); 
    Py_BEGIN_ALLOW_THREADS
    ret = globus_xio_server_accept(handle, *server);
    Py_END_ALLOW_THREADS
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS){
	tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else{
       ptr = SWIG_NewPointerObj((void *)handle, SWIGTYPE_p_globus_xio_handle_t, 0);
       PyTuple_SET_ITEM(tuple,1, ptr); 
    }
    return tuple;
}


PyObject *
server_cancel_accept(globus_xio_server_t * server)
{ 
    globus_result_t ret; 
    PyObject* tuple;
    char* tmpstr; 
 
    if (!(tuple=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, "server_cancel_accept: PyTuple_New()");
        return NULL;
    }  
    Py_BEGIN_ALLOW_THREADS
    ret = globus_xio_server_cancel_accept( *server);
    Py_END_ALLOW_THREADS
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS){
	tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else{
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple,1, Py_None);
    }
    return tuple;
}

PyObject *
descriptor_init(globus_xio_handle_t *handle)
{ 
    globus_result_t ret;
    PyObject* tuple, *ptr;
    globus_xio_data_descriptor_t * data;
    char* tmpstr; 
    
    if (!(tuple=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, "descriptor_init: PyTuple_New()");
        return NULL;
    }  
    
    data = (globus_xio_data_descriptor_t *)globus_malloc(sizeof(globus_xio_data_descriptor_t)); 
    ret = globus_xio_data_descriptor_init(data, *handle);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS){
	tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else{
        ptr = SWIG_NewPointerObj((void *)data, SWIGTYPE_p_globus_xio_data_descriptor_t,0);
        PyTuple_SET_ITEM(tuple,1,ptr);	
    
    }
    return tuple;
}

PyObject *
descriptor_destroy(globus_xio_data_descriptor_t *data)
{ 
    globus_result_t ret;
    PyObject* tuple;
    char* tmpstr; 
    
    if (!(tuple=PyTuple_New(2))){
        PyErr_SetString(PyExc_RuntimeError, "descriptor_destroy: PyTuple_New()");
        return NULL;
    }  
 
    ret = globus_xio_data_descriptor_destroy(*data);
    globus_free(data);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS){
	tmpstr = globus_object_printable_to_string(globus_error_get(ret));
        PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", tmpstr));
        globus_libc_free(tmpstr);
    }
    else{
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 1, Py_None);
    
    }
    return tuple;
}
%}
