#!/usr/bin/env python
############################################################################
# Joshua R. Boverhof, LBNL
# See LBNLCopyright for copyright notice!
###########################################################################
import sys, time, urlparse, optparse
import ZSI
from twisted.python import log
from twisted.internet import selectreactor
from twisted.internet import reactor
from twisted.internet.defer import succeed, waitForDeferred, deferredGenerator, Deferred
import pyGridWare.generated.stubs.counterservice as COUNTER
from pyGridWare.utility.scripts.client import GetOptParser, GetPortKWArgs, SetUp
from pyGridWare.wsrf.faults import BaseFault


############################################################################
# Counter
#     add
#     createCounter
############################################################################

############################################################################
# ResourceLifetime
#     Destroy
#     GetCurrentMessage
#     SetTerminationTime
############################################################################

############################################################################
# ResourceProperties
#     GetResourceProperty
############################################################################

def BlockAdd(d, locator, **kw):
    '''waiting for createResponse
    '''
    w = waitForDeferred(d)
    yield w

    response = w.getResult()
    kw['endPointReference'] = response._EndpointReference
    port = locator.getCounterPortType(**kw)
    l = []
    sum = 0

    # fire off a bunch of adds
    request = COUNTER.AddInputMessage(1)
    for i in range(10):
        d = port.add(request)
        l.append(d)
        sum += request
    BlockAdd2(l, port, sum)
BlockAdd = deferredGenerator(BlockAdd)


def BlockAdd2(dl, port, sum, **kw):
    '''wait for BlockAdd deferreds then fire a bunch more.
    '''
    for d in dl:
        w = waitForDeferred(d)
        yield w
        w.getResult()

    l = []
    # fire off a bunch of adds
    request = COUNTER.AddInputMessage(1)
    for i in range(10):
        d = port.add(request)
        l.append(d)
        sum += request

    BlockGetRP(l, port, sum)
BlockAdd2 = deferredGenerator(BlockAdd2)


def BlockGetRP(l, port, sum):
    '''wait on all outstanding operations to finish, then grab CounterRP value
    and assert that it matches expected value.
    '''
    for d in l:
        w = waitForDeferred(d)
        yield w
        w.getResult()

    def cb(response):
        assert response._any[0] == sum, \
            'GetResourcePropertyResponse(%d) and sum(%d) should be equal' %(response._any[0],sum)

    d = port.GetResourceProperty(\
            COUNTER.GetResourcePropertyRequest(("http://counter.com", "Value")),
    )
    d.addCallback(cb)
    BlockDestroy(d, port)
BlockGetRP = deferredGenerator(BlockGetRP)


def BlockDestroy(d, port):
    w = waitForDeferred(d)
    yield w

    response = w.getResult()
    d = port.Destroy( COUNTER.DestroyRequest() )
    BlockAddFail(d, port, 13)
BlockDestroy = deferredGenerator(BlockDestroy)


def BlockAddFail(d, port, value):
    w = waitForDeferred(d)
    yield w

    d = port.add( COUNTER.AddInputMessage(value) )
    d.addErrback(Failure)
BlockAddFail = deferredGenerator(BlockAddFail)


def Failure(fail):
    if fail.check(ZSI.fault.Fault) is not ZSI.fault.Fault:
        # don't know what this is??
        return fail
    reactor.callWhenRunning(reactor.stop)


def main(**kw):
    locator = COUNTER.CounterServiceLocator()
    print kw
    port = locator.getCounterPortType(**kw)
    port.__class__.defer = True
    d = port.createCounter( COUNTER.CreateCounterRequest() )
    d.addErrback(Failure)
    d = BlockAdd(d, locator, **kw)
    return port
 
if __name__ == '__main__':
    op = GetOptParser('CounterService')
    (options, args) = op.parse_args()
    SetUp(options)
    kw = GetPortKWArgs(options)
    reactor.callWhenRunning(main, **kw)
    reactor.run()

