#!/usr/bin/env python
############################################################################
# Matt Rodriguez, LBNL
# See LBNLCopyright for copyright notice!
###########################################################################
import  base64, sys
import pyGridWare.generated.stubs.delegationservice as DS
from pyGridWare.utility.scripts.client import \
    GetOptParser, GetPortKWArgs, SetUp, GetConfigParser

import ZSI
from ZSI.wstools.Namespaces import OASIS
from ZSI.TC import _get_global_element_declaration
from twisted.internet import reactor   
try:
    from pyGsi import proxy 
except ImportError:
    print "Need to have pyGsi installed."
    sys.exit(0)
try:
    from M2Crypto import X509
except ImportError:
    print "Need to have M2Crypto installed."
    sys.exit(0)

DFPortType = DS.DelegationFactoryPortTypeSOAPBindingSOAP
GED = _get_global_element_declaration

class QName(tuple):
    '''Use to specify a Qualified Name (namespace,name) --> prefix:name'''
    typecode = ZSI.TC.QName()
    
def DelegateCredential(**kw):
    """Returns an EPR to the Delegated Resource
    """
    # -------------------------------------------
    # Get CertificatChain from DelegationFactoryService
    # -------------------------------------------
    
    kw['endPointReference'] = None
    dPort = DFPortType(**kw)
    property = ("http://www.globus.org/08/2004/delegationService",
                "CertificateChain")
    getReq = DS.GetResourcePropertyRequest(QName(property))
    getRsp = dPort.GetResourceProperty(getReq)

    # -------------------------------------------
    # Grab BinarySecurityToken, Create X509 Stack (PKIPath)
    # -------------------------------------------
    certChain = getRsp._any[0]
    bstoken = certChain._any
    #assert isinstance(certChain.typecode, \
    #    GED('http://www.globus.org/08/2004/delegationService', 
    #        'CertificateChain')), \
    #        'Expecting CertificateChain'
    stack = X509.new_stack_from_der(base64.decodestring(bstoken)) 
    delegation_cert = stack.pop()
    new_delegation_cert = X509.X509()
    new_delegation_cert.set_pubkey(delegation_cert.get_pubkey())
    
    # -------------------------------------------
    # Grab Credential, use to sign 1st cred in PKIPath .
    #    FIX: Currently just inserting credential into chain
    # -------------------------------------------
    proxy_cred = proxy.Proxy()
    proxy_cred.read() 
    proxy_cert = proxy_cred.getcert()
    proxy_key = proxy_cred.getkey()
    issuer_cert = proxy_cred.getissuer()
    
    bst_stack = X509.X509_Stack()
    bst_stack.push(issuer_cert)
    bst_stack.push(proxy_cert)
    
    pf_keywords = {}
    proxyfile = proxy.get_proxy_filename()
    pf_keywords['cert'] = proxyfile
    pf_keywords['key'] = proxyfile
    pf_keywords['valid'] = (12,0)
    pf_keywords['full'] = kw['full']
    proxy_factory = proxy.ProxyFactory(pf_keywords)
    proxy_factory.set_proxycert(new_delegation_cert)
    proxy_factory.generate()
    new_proxy = proxy_factory.getproxy()
    bst_stack.push(new_proxy.getcert())
    der_encoding = bst_stack.as_der()
    # -------------------------------------------
    # RequestSecurityToken
    # -------------------------------------------
    
    BST = GED(OASIS.WSSE,"BinarySecurityToken").pyclass
    bst = BST(base64.encodestring(der_encoding))

    bst._attrs = {}
    #These namespaces should be moved to ZSI.wstools.Namespaces
    bst._attrs['EncodingType'] =\
        "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-soap-message-security-1.0#Base64Binary"
    bst._attrs['ValueType'] =\
         "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-x509-token-profile-1.0#X509PKIPathv1"
    rstokMsg = DS.RequestSecurityTokenMsg()
    rstokMsg._any = [bst,]
    rstokMsgR = dPort.RequestSecurityToken(rstokMsg)
    # -------------------------------------------
    # EPR of delegated credential.
    # -------------------------------------------
    epr = rstokMsgR._any[0]

    return epr

    
def main(options):
    cp = GetConfigParser()
    path = cp.get('Paths', 'DelegationFactoryService')
    kw = GetPortKWArgs(options)
    kw['full'] = not options.limited
    delEPR = DelegateCredential(**kw)
     
if __name__ == "__main__":
    parser = GetOptParser('DelegationFactoryService')
    parser.add_option("-l", "--limited", action="store_true", dest="limited",
                       default=False, help="flag to set limited delegation")
    (options, args) = parser.parse_args()
    SetUp(options)
    reactor.callWhenRunning(main, options)
    reactor.run()
