#!/usr/bin/env python
############################################################################
# Joshua R. Boverhof, LBNL
# See LBNLCopyright for copyright notice!
###########################################################################
import commands, sys, time, urlparse, optparse, base64, warnings
import ZSI
from ZSI.wstools.Namespaces import WSA200403
from ZSI.TC import _get_type_definition, _get_global_element_declaration as GED,\
    b64decode, b64encode, ElementDeclaration, String

# M2Crypto imports
try:
    import pyGsi.proxy
except ImportError:
    print """REQUIRED DEPENDENCIES:
    
REQUIRES a patched M2Crypto(as of Revision 321), openssl-0.9.8,
and proxy_tools.  

  Quick Directions:
    %svn co https://bosshog.lbl.gov:/repos/proxy_tools
    %export PYTHONPATH=$PYTHONPATH:$PWD/proxy_tools
    %svn co http://svn.osafoundation.org/m2crypto/trunk
    %cd trunk
    %patch -p0 <../proxy_tools/m2.patch
    %python setup.py build

"""
    sys.exit()

from M2Crypto import X509
from twisted.python import log
from twisted.internet import reactor

from pyGridWare.utility.scripts.client import \
    GetOptParser, GetPortKWArgs, SetUp, GetConfigParser
import pyGridWare.generated.stubs.managedjobfactory as MJF
import pyGridWare.generated.types.managedjobservice
import pyGridWare.generated.stubs.delegationservice as DS

from pyGridWare.notification.NotificationConsumer import \
    NotificationConsumer

from pyGridWare.generated.services.basenotification.WS_BaseNotificationService import \
    WS_BaseNotificationServiceWSRF as NotificationConsumerService


MJFPortType = MJF.ManagedJobFactoryPortTypeSOAPBindingSOAP
DFPortType = DS.DelegationFactoryPortTypeSOAPBindingSOAP

DESC = """WS-GRAM Client
    Example: ./ManagedJobClient.py -u https://fakehost:9999/wsrf/services/ManagedJobFactoryService -d1 -m /bin/touch /tmp/touchme"""

#######################################################################
# Runtime Patch
#######################################################################
for what in filter(lambda what: what.pname in ('job','multiJob'), 
                    MJF.createManagedJobInputMessage.typecode.ofwhat):
    what.minOccurs = 0


#######################################################################
# Functions
#
#######################################################################

def CreateManagedJob(mjfEPR, cEPR, delEPR=None, *args, **kw):
    '''Essentially copying an EPR obtained from delegation serivce over
    to jobCredentialEndpoint and staggingCredentialEndpoint.

    mjfEPR -- ManagedJobFactory EPR
    cEPR   -- NotificationConsumer EPR
    delEPR -- DelegationService EPR (optional)

    '''
    # -------------------------------------------
    # Get Port to ManangedJobFactoryService
    # -------------------------------------------
    mPort = MJFPortType(endPointReference=mjfEPR, **kw)
    request = MJF.createManagedJobInputMessage()
    job = request.Job = request.new_job()
    request.InitialTerminationTime = time.gmtime(time.time()+10000)
    request.JobID = 'uuid:%s' %commands.getoutput('uuidgen')

    # -------------------------------------------
    # Subscribing to a RP of the resource to be created.
    # ManagedJobPortType -- Resource Properties: 
    #     GED ("http://www.globus.org/namespaces/2004/10/gram/job","managedJobResourceProperties")
    #        element ref ("http://www.globus.org/namespaces/2004/10/gram/job/types","state")
    # -------------------------------------------
    sub = request.Subscribe = request.new_Subscribe()
    sub.ConsumerReference = cEPR
    top = sub.TopicExpression = sub.new_TopicExpression()
    # TODO: Fix pyclass_type mixed content
    top._text = QName(('http://www.globus.org/namespaces/2004/10/gram/job/types','state'))
    top.set_attribute_Dialect(\
        'http://docs.oasis-open.org/wsn/2004/06/TopicExpression/Simple'
        )

    # set up job factoryEndpoint
    epr = job.FactoryEndpoint = job.new_factoryEndpoint()
    epr.Address = mjfEPR._Address
    epr.ReferenceProperties = epr.new_ReferenceProperties()
    epr.ReferenceProperties.Any = mjfEPR._ReferenceProperties._any
    if delEPR is not None:
        # set up job jobCredentialEndpoint: DelegationService
        job.JobCredentialEndpoint = job.new_jobCredentialEndpoint()
        job.JobCredentialEndpoint.Address = delEPR._Address
        job.JobCredentialEndpoint.ReferenceProperties =\
            job.JobCredentialEndpoint.new_ReferenceProperties()
        job.JobCredentialEndpoint.ReferenceProperties._any = \
            delEPR._ReferenceProperties._any

        # set up job stagingCredentialEndpoint: Delegation Service
        job.StagingCredentialEndpoint = job.new_stagingCredentialEndpoint()
        job.StagingCredentialEndpoint.Address = delEPR._Address
        job.StagingCredentialEndpoint.ReferenceProperties = \
            job.StagingCredentialEndpoint.new_ReferenceProperties()
        job.StagingCredentialEndpoint.ReferenceProperties._any = \
            delEPR._ReferenceProperties._any
#    else:
#        request._job._jobCredentialEndpoint = None
#        request._job._stagingCredentialEndpoint = None

    # job description
    if len(args):
        job.Executable = args[0]
        #TODO: argument is maxOccurs="unbounded" but expects "1"
        #job.Argument = args[1:]
        job._argument = args[1:]

#    request._job._localUserId = None
#    request._job._holdState = None 
#    request._job._directory = None
#    request._job._environment = None
#    request._job._stdin = None
#    request._job._stdout = None
#    request._job._stderr = None
#    request._job._count = None
#    request._job._libraryPath = None
#    request._job._hostCount = None
#    request._job._project = None
#    request._job._queue = None
#    request._job._maxTime = None
#    request._job._maxWallTime = None
#    request._job._maxCpuTime = None
#    request._job._maxMemory = None
#    request._job._minMemory = None
#    request._job._jobType = None
#    request._job._fileStageIn = None
#    request._job._fileStageOut = None
#    request._job._fileCleanUp = None
#    request._job._extensions = None

    return mPort.createManagedJob(request)


def DelegateCredential(**kw):
    """Returns an EPR to the Delegated Resource
    """
    # -------------------------------------------
    # Get CertificatChain from DelegationFactoryService
    # -------------------------------------------
    kw['endPointReference'] = None
    dPort = DFPortType(**kw)
    property = ("http://www.globus.org/08/2004/delegationService","CertificateChain")
    getReq = DS.GetResourcePropertyRequest(QName(property))
    getRsp = dPort.GetResourceProperty(getReq)

    # -------------------------------------------
    # Grab BinarySecurityToken, Create X509 Stack (PKIPath)
    # -------------------------------------------
    certChain = getRsp._any[0]
    bstoken = certChain._any
    assert isinstance(certChain.typecode, \
        GED('http://www.globus.org/08/2004/delegationService', 'CertificateChain')), \
        'Expecting CertificateChain'
    #assert isinstance(bstoken.typecode, \
    #    GED('http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd', 
    #        'BinarySecurityToken')), \
    #    'Expecting BinarySecurityToken'
    stack = X509.new_stack_from_der(base64.decodestring(bstoken)) 
    
    # -------------------------------------------
    # Grab Credential, use to sign 1st cred in PKIPath .
    #    FIX: Currently just inserting credential into chain
    # -------------------------------------------
    proxy = pyGsi.proxy.Proxy()
    proxy.read() 
    proxy_cert = proxy.getcert()
    proxy_key = proxy.getkey()
    issuer_cert = proxy.getissuer()
    
    bst_stack = X509.X509_Stack()
    bst_stack.push(issuer_cert)
    bst_stack.push(proxy_cert)
    der_encoding = bst_stack.as_der()
    # -------------------------------------------
    # RequestSecurityToken
    # -------------------------------------------
    bst = GED("http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd", 
              "BinarySecurityToken").pyclass(base64.encodestring(der_encoding))


    bst._attrs = {}
    bst._attrs['EncodingType'] =\
        "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-soap-message-security-1.0#Base64Binary"
    bst._attrs['ValueType'] =\
         "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-x509-token-profile-1.0#X509PKIPathv1"
    rstokMsg = DS.RequestSecurityTokenMsg()
    rstokMsg._any = [bst,]
    rstokMsgR = dPort.RequestSecurityToken(rstokMsg)
    # -------------------------------------------
    # EPR of delegated credential.
    # -------------------------------------------
    epr = rstokMsgR._any[0]
    assert isinstance(epr.typecode, \
        GED('http://schemas.xmlsoap.org/ws/2004/03/addressing','EndpointReference')), \
        'Expecting a EndpointReference'

    return epr


#######################################################################
# classes 
#######################################################################

class QName(tuple):
    '''Use to specify a Qualified Name (namespace,name) --> prefix:name'''
    typecode = ZSI.TC.QName()


class NotificationHandler(NotificationConsumerService):
    '''Notification Callback handler.'''
    def __init__(self, **kw):
        NotificationConsumerService.__init__(self, **kw)

    def wsa_Notify(self, ps, address, **kw):
        request,response = NotificationConsumerService.wsa_Notify(self, ps, address, **kw)

        log.msg("Received Notification")
        for nm in request.NotificationMessage:
            #log.msg("  NotificationMessage: ", nm)
            #log.msg("    Topic: ", nm.Topic)
            #log.msg("    ProducerReference: ", nm.ProducerReference)

            if nm.Message.StateChangeNotificationMessage.State == 'Done':
                log.msg("exit code: ", nm.Message.StateChangeNotificationMessage.ExitCode)
                log.msg("holding:   ", nm.Message.StateChangeNotificationMessage.Holding)
                reactor.callLater(0.1, reactor.stop)

            #TODO: Not sure what to use this for..
            #if nm.Message.StateChangeNotificationMessage.Holding

        return request,response


#######################################################################
# main 
#######################################################################
class DELEGATION:
    NONE = 'none'
    FULL = 'full'
    LIMITED = 'limited'
    OPTIONS =  (NONE,FULL,LIMITED)


class OPERATION:
    VALIDATE = 'validate'
    SUBMIT = 'submit'
    MONITOR = 'monitor'
    STATUS = 'status'
    KILL = 'kill'
    OPTIONS = (VALIDATE,SUBMIT,MONITOR,STATUS,KILL)                


WSGRAM = 'http://www.globus.org/namespaces/2004/10/gram/job'
EndpointReference = GED(WSA200403.ADDRESS, 'EndpointReference').pyclass
class ResourceID(str):
    typecode = String((WSGRAM,"ResourceID"), typed=False)

def main(options, args):
    """
    """
    kw = GetPortKWArgs(options)
    mjfURL = options.url

    # -------------------------------------------
    # Create EPR for "Fork"
    # -------------------------------------------
    mjfEPR = EndpointReference()
    mjfEPR.ReferenceProperties = mjfEPR.new_ReferenceProperties()
    mjfEPR.ReferenceProperties.Any = ResourceID('Fork')
    mjfEPR.Address = mjfEPR.new_Address(mjfURL)

    # -------------------------------------------
    # Create NotificationConsumer and get EPR
    # -------------------------------------------
    consumer = NotificationConsumer(notificationClass=NotificationHandler)
    consumer.start()
    cEPR = consumer.getEndPointReference()
    delEPR = None
    if options.delegate != DELEGATION.NONE:
        # -------------------------------------------
        # Get Port to ManangedJobFactoryService
        # -------------------------------------------
        #mPort = MJFPortType(url=mjfURL, endPointReference=mjfEPR, **kw)
        kwc = kw.copy()
        kwc['url'] = mjfEPR._Address
        mPort = MJFPortType(endPointReference=mjfEPR, **kwc)

        # -------------------------------------------
        # Create GetMultipleResourcePropertiesRequest
        # -------------------------------------------
        multiReq = MJF.GetMultipleResourcePropertiesRequest()
        multiReq.ResourceProperty = QName((WSGRAM, 'delegationFactoryEndpoint'))
        multiReq.ResourceProperty.append(\
            QName((WSGRAM, 'stagingDelegationFactoryEndpoint')),
            )
        multiRsp = mPort.GetMultipleResourceProperties(multiReq)

        # -------------------------------------------
        # Delegate Credential
        # -------------------------------------------
        scheme,netloc,url,params,query,fragment = urlparse.urlparse(mjfURL)
        params,query,fragment = '','',''
        cp = GetConfigParser()
        path = cp.get('Paths', 'DelegationFactoryService')
        dcURL = urlparse.urlunparse((scheme,netloc,path,params,query,fragment))
        kwc['url'] = dcURL
        delEPR = DelegateCredential(**kwc)

    # -------------------------------------------
    # Create Manged Job
    # -------------------------------------------
    kw['url'] = mjfURL
    if options.operation == OPERATION.SUBMIT:
        CreateManagedJob(mjfEPR, cEPR, delEPR, *args, **kw)
    else:
        warnings.warn('')


if __name__ == '__main__':
    parser = GetOptParser('ManagedJobFactoryService', 
                 usage="usage: %prog [options] command arg1 ...", description=DESC)

    # TODO: m2crypto ONLY...
    parser.add_option('-t', "--delegate", action="store", dest="delegate", 
        choices=DELEGATION.OPTIONS, default=DELEGATION.NONE,
        help="choose from %s" %str(DELEGATION.OPTIONS))
    parser.add_option('-o', "--operation", action="store", dest="operation", 
        choices=OPERATION.OPTIONS, default=OPERATION.SUBMIT, 
        help="choose from %s" %str(OPERATION.OPTIONS))
    
    (options, args) = parser.parse_args()
    if len(args) < 1: 
        print "Exit: No command provided"
        sys.exit(-1)

    SetUp(options)

    reactor.callWhenRunning(main, options, args)
    reactor.run()
