#!/usr/bin/env python
############################################################################
# Joshua R. Boverhof, LBNL
# See LBNLCopyright for copyright notice!
###########################################################################
from twisted.python import log
from twisted.internet import reactor

import ZSI
import pyGridWare.generated.stubs.counterservice as COUNTER
import pyGridWare.generated.stubs.submanagerservice as SUBM
from pyGridWare.notification.NotificationConsumer import \
    NotificationConsumer, NotificationConsumerService
from pyGridWare.utility.scripts.client import GetOptParser, GetPortKWArgs, SetUp


PORT = None
class NCService(NotificationConsumerService):
    def wsa_Notify(self, ps, address, **kw):
        request,response = NotificationConsumerService.wsa_Notify(self, ps, address)
        reactor.callWhenRunning(PORT.Destroy, COUNTER.DestroyRequest())
        reactor.callLater(1, reactor.stop)
        return request,response

###########################################################################
# Functions
###########################################################################

def Subscribe(port, consumer):
    """Need to specify the prefix-namespace mapping in the XML instance 
    to deal with QNames passed via the TopicExpression mixed text content. 
    Do this by using the QName TypeCode.

    port -- Subscription Client
    consumer -- Notification Consumer
    """
    class TopicExpression:
        class QName(tuple):
            typecode = ZSI.TC.QName()

        def __init__(self, namespaceURI, name):
            self._attrs = {}
            self._attrs['Dialect'] = \
                "http://docs.oasis-open.org/wsn/2004/06/TopicExpression/Simple"
            self._text = TopicExpression.QName((namespaceURI,name))
    
    request = COUNTER.SubscribeRequest()
    request._ConsumerReference = consumer.getEndPointReference()
    request._TopicExpression = TopicExpression("http://counter.com", "Value")
    request._UseNotify = True
    msg = port.Subscribe(request)
    return msg


def main(**kw):
    locator = COUNTER.CounterServiceLocator()
    port = locator.getCounterPortType(**kw)
    response = port.createCounter(COUNTER.CreateCounterRequest())

    # Get New port w/ EPR
    global PORT
    kw['endPointReference'] = response._EndpointReference
    PORT = locator.getCounterPortType(**kw)
    consumer = NotificationConsumer(notificationClass=NCService)
    consumer.start()

    sub_rsp = Subscribe(PORT, consumer)
    locator = SUBM.SubscriptionManagerServiceLocator()
    kw['endPointReference'] = sub_rsp._SubscriptionReference
    kw['url'] = sub_rsp._SubscriptionReference._Address
    sm_port = locator.getSubscriptionManager(**kw)

    # Queue Events
    if options.pause:
        reactor.callWhenRunning(sm_port.PauseSubscription, SUBM.PauseSubscriptionRequest())
        reactor.callWhenRunning(PORT.add, COUNTER.AddInputMessage(13))
        reactor.callWhenRunning(sm_port.ResumeSubscription, SUBM.ResumeSubscriptionRequest())
        reactor.callWhenRunning(PORT.add, COUNTER.AddInputMessage(13))
    else:
        reactor.callWhenRunning(PORT.add, COUNTER.AddInputMessage(13))

 
if __name__ == '__main__':
    op = GetOptParser('CounterService')
    op.add_option("-p", "--pause",
                  action="store_true", dest="pause", default=False,
                  help="tell subscription manager to pause ")

    (options, args) = op.parse_args()
    SetUp(options)
    kw = GetPortKWArgs(options)

    reactor.callWhenRunning(main, **kw)
    reactor.run()
