#!/usr/bin/env python
############################################################################
# Joshua R. Boverhof, LBNL
# See LBNLCopyright for copyright notice!
###########################################################################
import os, sys, time, urlparse, optparse, socket, warnings
import ZSI
from ZSI.TC import _get_global_element_declaration as GED
from ZSI.wstools.Namespaces import *

import pyGridWare.generated.stubs.performancecounter as COUNTER
from pyGridWare.utility.scripts.client import GetOptParser, GetPortKWArgs, SetUp,\
    GetConfigParser
from pyGridWare.notification.NotificationConsumer import NotificationConsumer,\
    NotificationConsumerService
from pyGridWare.generated.services.basenotification.WS_BaseNotificationService import \
    WS_BaseNotificationServiceWSRF as NCServiceInterface

try:
    from pyGridWare.security.gss.GssSignatureHandler import GssSignatureHandler
    from pyGridWare.security.gss.Authenticate import GssAuthenticationHandler
    from pyGridWare.security.Utility import MessageSigUtility
    from pyGridWare.security.gss.GssService import GssSecureResourceMixIn
except ImportError, ex:
    warnings.warn("No XML Signatures")
    SIGNATURES = False
else:
    SIGNATURES = True

from twisted.python import log
from twisted.internet import reactor, main
from twisted.internet.defer import execute

############################################################################
# GLOBALS
############################################################################
USE_ZSI_CLIENT = COUNTER.client.__name__ == ZSI.client.__name__
CP = None
SEC_PATHS = 'Paths'
COUNT = 0
DIV = 10
LOOPS = 0
SCENARIO = 0
_TARGETNAMESPACE = "http://perf.counter.com"
DIALECT = "http://docs.oasis-open.org/wsn/2004/06/TopicExpression/Simple"
VALUE = GED(_TARGETNAMESPACE, 'Value')()
RESULTS_FILE = open('benchmarks.txt', 'a+')
OPTIONS = None


############################################################################
#  Scenarios
############################################################################

def Scenario1(locator, **kw):
    """
    Scenario 1: no WS-Security, no https, client and service co-located
    Scenario 4: no WS-Security, no https, client and service NOT co-located
    """
    scheme, netloc, path, params, query, fragment = urlparse.urlparse(OPTIONS.url)
    if scheme == 'https':
        raise RuntimeError, 'Scenario 1,4 no https'
    if OPTIONS.secureConv:
        raise RuntimeError, 'Scenario 1,4 no WS-Security'

    global SCENARIO
    host,port = netloc.split(':')
    if socket.getfqdn(host) == socket.getfqdn(socket.gethostname()):
        SCENARIO = 1
        RESULTS_FILE.write("\nSCENARIO1\n")
    else:
        SCENARIO = 4
        RESULTS_FILE.write("\nSCENARIO4\n")

    RESULTS_FILE.write("\tCLIENT: %s\n" %socket.getfqdn(socket.gethostname()))
    RESULTS_FILE.write("\tSERVER: %s\n" %OPTIONS.url)
    Run(locator, kw)


def Scenario2(locator, **kw):
    """
    Scenario 2: X509 signing via WS-Security, no https, client and service co-located
    Scenario 5: X509 signing via WS-Security, no https, client and service NOT co-located
    """
    if OPTIONS.url.endswith('SecurePerfCounterService') is False:
        raise RuntimeError, 'Bad URL, Expecting SecurePerfCounterService'
    if OPTIONS.ssl:  
        raise RuntimeError, 'Scenario 2,5 no https'
    if not OPTIONS.secureConv:
        raise RuntimeError, 'Scenario 2,5 requires secure conversation'

    scheme, netloc, path, params, query, fragment = urlparse.urlparse(kw['url'])
    global SCENARIO
    host,port = netloc.split(':')
    if socket.getfqdn(host) == socket.getfqdn(socket.gethostname()):
        SCENARIO = 2
        RESULTS_FILE.write("\nSCENARIO2\n")
    else:
        SCENARIO = 5
        RESULTS_FILE.write("\nSCENARIO5\n")
    RESULTS_FILE.write("\tCLIENT: %s\n" %socket.getfqdn(socket.gethostname()))
    RESULTS_FILE.write("\tSERVER: %s\n" %kw['url'])

    Run(locator, kw)


def Scenario3(locator, **kw):
    """Scenario 3: https, client and service co-located
       Scenario 6: https, client and service NOT co-located 
    """
    if USE_ZSI_CLIENT is True:
        from pyGlobus.io import GSIHTTPConnection
        from pyGlobus.ioc import GLOBUS_IO_SECURE_CHANNEL_MODE_SSL_WRAP,\
            GLOBUS_IO_SECURE_DELEGATION_MODE_NONE
        kw['transport'] = GSIHTTPConnection
        kw['transdict'] = \
            {'channel_mode':GLOBUS_IO_SECURE_CHANNEL_MODE_SSL_WRAP,
             'delegation_mode':GLOBUS_IO_SECURE_DELEGATION_MODE_NONE,
            }

    if not OPTIONS.ssl:  
        raise RuntimeError, 'Scenario 3,6 require https'

    scheme, netloc, path, params, query, fragment = urlparse.urlparse(kw['url'])
    if scheme != 'https':
        raise RuntimeError, 'Scenario 3,6 require https'

    global SCENARIO
    host,port = netloc.split(':')
    if socket.getfqdn(host) == socket.getfqdn(socket.gethostname()):
        SCENARIO = 3
        RESULTS_FILE.write("\nSCENARIO3\n")
    else:
        SCENARIO = 6
        RESULTS_FILE.write("\nSCENARIO6\n")
    RESULTS_FILE.write("\tCLIENT: %s\n" %socket.getfqdn(socket.gethostname()))
    RESULTS_FILE.write("\tSERVER: %s\n" %kw['url'])
    Run(locator, kw)


############################################################################
#  Utility classes.
############################################################################
class Holder: pass
class ValueWrapper(int):
    typecode = VALUE

class TopicExpression:
    def __init__(self, value):
        self._attrs = {}
        self._attrs['Dialect'] = DIALECT
        self._text = value


############################################################################
#  Tests
############################################################################

def Test1(locator, **kw):
    global LOOPS
    LOOPS = 10000/DIV
    print "---------------------------"
    print "Test 1: GetResourceProperty"
    port = locator.getCounterPortType(**kw)
    request = COUNTER.CreateCounterRequest()
    response = port.createCounter(request)

    kw['endPointReference'] = response._EndpointReference
    port = locator.getCounterPortType(**kw)

    msg = COUNTER.SetResourcePropertiesRequest()
    msg._Update = Holder()
    msg._Update._any = [ValueWrapper(100)]
    port.SetResourceProperties(msg)

    global COUNT
    elapsedTime = stopTime = startTime = 0.0
    #startTime = time.time()
    tests = []

    COUNT = 0
    min = 1
    max = 0
    DOGET = False
    while COUNT < LOOPS:
        #startTime = time.time()
        startTime = time.time()
        request = COUNTER.GetResourcePropertyRequest((VALUE.nspname, VALUE.pname,))
        msg = port.GetResourceProperty(request)
        stopTime = time.time()

        tests.append(stopTime-startTime)
        elapsedTime += tests[-1]
        if min > tests[-1]: min = tests[-1]
        if max < tests[-1]: max = tests[-1]
        COUNT += 1

    #stopTime = time.time()
    RESULTS_FILE.write("\t1) time to GetResourceProperty(%d): %s\n" %(COUNT,elapsedTime))
    RESULTS_FILE.write("\t    mean: %sms\n" %((elapsedTime/len(tests))*1000.0))
    RESULTS_FILE.write("\t    max: %sms\n" %(max*1000.0))
    RESULTS_FILE.write("\t    min: %sms\n" %(min*1000.0))
    print "time to GetResourceProperty: ", elapsedTime
    return tests


def Test2(locator, **kw):
    global LOOPS
    LOOPS = 10000/DIV
    print "---------------------------"
    print "Test 2: SetResourceProperty"
    port = locator.getCounterPortType(**kw)
    request = COUNTER.CreateCounterRequest()
    response = port.createCounter(request)

    kw['endPointReference'] = response._EndpointReference
    port = locator.getCounterPortType(**kw)

    request = COUNTER.SetResourcePropertiesRequest()
    request._Update = Holder()
    request._Update._any = [ValueWrapper(100)]
    msg = port.SetResourceProperties(request)

    global COUNT
    elapsedTime = stopTime = startTime = 0
    COUNT = 0
    min = 1.0
    max = 0.0
    tests = []
    while COUNT < LOOPS:
        startTime = time.time()
        request = COUNTER.SetResourcePropertiesRequest()
        request._Update = Holder()
        request._Update._any = [ValueWrapper(COUNT)]
        msg = port.SetResourceProperties(request)
        stopTime = time.time()

        tests.append(stopTime-startTime)
        elapsedTime += tests[-1]
        if min > tests[-1]: min = tests[-1]
        if max < tests[-1]: max = tests[-1]
        COUNT += 1

    RESULTS_FILE.write("\t2) time to SetResourceProperty(%d): %s\n" %(COUNT,elapsedTime))
    RESULTS_FILE.write("\t    mean: %sms\n" %((elapsedTime/len(tests))*1000.0))
    RESULTS_FILE.write("\t    max: %sms\n" %(max*1000.0))
    RESULTS_FILE.write("\t    min: %sms\n" %(min*1000.0))
    print "time to SetResourceProperty: ", elapsedTime
    return tests


def Test3(locator, **kw):
    global LOOPS
    LOOPS = 1000/DIV
    print "---------------------------"
    print "Test 3: CreateResource"

    port = locator.getCounterPortType(**kw)

    global COUNT
    elapsedTime = stopTime = startTime = 0
    startTime = time.time()

    min = 1.0
    max = 0.0
    tests = []
    while COUNT < LOOPS:
        startTime = time.time()
        request = COUNTER.CreateCounterRequest()
        response = port.createCounter(request)
        stopTime = time.time()

        tests.append(stopTime-startTime)
        elapsedTime += tests[-1]
        if min > tests[-1]: min = tests[-1]
        if max < tests[-1]: max = tests[-1]
        COUNT += 1

    RESULTS_FILE.write("\t3) time to CreateResource(%d): %s\n" %(COUNT,elapsedTime))
    RESULTS_FILE.write("\t    mean: %sms\n" %((elapsedTime/len(tests))*1000.0))
    RESULTS_FILE.write("\t    max: %sms\n" %(max*1000.0))
    RESULTS_FILE.write("\t    min: %sms\n" %(min*1000.0))
    print "time to CreateResource: ", elapsedTime
    return tests


def Test4(locator, **kw):
    global LOOPS
    LOOPS = 1000/DIV
    print "---------------------------"
    print "Test 4: DestroyResource"

    responses = []
    eprs = []
    port = locator.getCounterPortType(**kw)
    i = 0
    while i < LOOPS:
        port = locator.getCounterPortType(**kw)
        request = COUNTER.CreateCounterRequest()
        response = port.createCounter(request)
        responses.append(response)
        eprs.append(response._EndpointReference)

        request = COUNTER.SetResourcePropertiesRequest()
        request._Update = Holder()
        request._Update._any = [ValueWrapper(100)]
        port.endPointReference = eprs[-1]
        msg = port.SetResourceProperties(request)
        i += 1

    global COUNT
    min = 1.0
    max = 0.0
    tests = []
    COUNT = 0
    port = locator.getCounterPortType(**kw)
    elapsedTime = stopTime = startTime = 0
    startTime = time.time()
    for epr in eprs:
        port.endPointReference = epr
        startTime = time.time()
        request = COUNTER.DestroyRequest()
        port.Destroy(request)
        stopTime = time.time()
        tests.append(stopTime-startTime)
        elapsedTime += tests[-1]
        if min > tests[-1]: min = tests[-1]
        if max < tests[-1]: max = tests[-1]
        COUNT += 1


    stopTime = time.time()
    elapsedTime = elapsedTime + (stopTime - startTime)
    RESULTS_FILE.write("\t4) time to DestroyResource(%d): %s\n" %(LOOPS,elapsedTime))
    RESULTS_FILE.write("\t    mean: %sms\n" %((elapsedTime/len(tests))*1000.0))
    RESULTS_FILE.write("\t    max: %sms\n" %(max*1000.0))
    RESULTS_FILE.write("\t    min: %sms\n" %(min*1000.0))
    print "time to DestroyResource: ", elapsedTime
    return tests


def Test5(locator, **kw):
    global LOOPS
    LOOPS = 1000/DIV
    print "---------------------------"
    print "Test 5: Notification"

    if not kw.has_key('sig_handler'):
        notificationClass=NCS
    else:
        notificationClass=SignedNCS

    def cb():
       reactor.callLater(0.001, SetResourceProperty, port, notificationClass)
    notificationClass.cb = staticmethod(cb)
    
    consumer = NotificationConsumer(notificationClass=notificationClass)
    scheme, netloc, path, params, query, fragment = urlparse.urlparse(kw['url'])
    if scheme == 'https':
        assert kw.has_key('contextFactory'), 'SSL REQUIRES contextFactory'
        #CERTFILE = CP.get('security', 'certfile') or '%s/%s' %(os.environ['HOME'],'.globus/usercert.pem')
        #KEYFILE = CP.get('security', 'keyfile') or '%s/%s' %(os.environ['HOME'],'.globus/userkey.pem')
        consumer.start(ssl=True, contextFactory=kw['contextFactory'])
    else:
        consumer.start()

    port = locator.getCounterPortType(**kw)
    request = COUNTER.CreateCounterRequest()
    response = port.createCounter(request)

    kw['endPointReference'] = response._EndpointReference
    port = locator.getCounterPortType(**kw)
    request = COUNTER.SetResourcePropertiesRequest()
    request._Update = Holder()
    request._Update._any = [ValueWrapper(100)]
    msg = port.SetResourceProperties(request)

    request = COUNTER.SubscribeRequest()
    epr = consumer.getEndPointReference()
    request._ConsumerReference = epr
    prefix = "counter"
    request._TopicExpression = TopicExpression("%s:%s" %(prefix,VALUE.pname))
    request._UseNotify = True
    port.binding.nsdict[prefix] = VALUE.nspname
    msg = port.Subscribe(request)

    reactor.callWhenRunning(SetResourceProperty, port, notificationClass)
    #reactor.addSystemEventTrigger('before', 'shutdown', WriteTime, startTime)


def SetResourceProperty(port, notificationClass):
    notificationClass.setStartTime()
    request = COUNTER.SetResourcePropertiesRequest()
    request._Update = Holder()
    request._Update._any = [ValueWrapper(15)]
    msg = port.SetResourceProperties(request)


class NCS(NotificationConsumerService):
    min = 100.0
    max = 0.0
    cb = None
    tests = []
    startTime = 0

    def __init__(self, post='/wsrf/services/NotificationConsumerService', **kw):
        NotificationConsumerService.__init__(self, post, **kw)

    def setStartTime(cls):
        cls.startTime = time.time()
    setStartTime = classmethod(setStartTime)
    def getStartTime(cls):
        return cls.startTime
    getStartTime = classmethod(getStartTime)

    def wsa_Notify(self, ps, address):
        NotificationConsumerService.wsa_Notify(self, ps, address)
        stopTime = time.time()
        elapsedTime = stopTime-self.getStartTime()
        NCS.tests.append(elapsedTime)
        if NCS.min > NCS.tests[-1]: NCS.min = NCS.tests[-1]
        if NCS.max < NCS.tests[-1]: NCS.max = NCS.tests[-1]
        global COUNT
        COUNT += 1
        if COUNT == LOOPS: 
            RESULTS_FILE.write("\t5) time for Notification(%d): % ss\n" %(len(NCS.tests),sum(NCS.tests)))
            RESULTS_FILE.write("\t    mean: %sms\n" %((sum(NCS.tests)/len(NCS.tests))*1000.0))
            RESULTS_FILE.write("\t    max: %sms\n" %(NCS.max*1000.0))
            RESULTS_FILE.write("\t    min: %sms\n" %(NCS.min*1000.0))
            WriteOutData(SCENARIO, 5, NCS.tests)
            NCS.tests = []
            reactor.stop()
            return
        NCS.cb()

if SIGNATURES:
    class SignedNCS(GssSecureResourceMixIn, NCServiceInterface):
        cb = None
        def __init__(self, post='/wsrf/services/NotificationConsumerService', **kw):
            GssSecureResourceMixIn.__init__(self, post)

        def setStartTime(cls):
            cls.startTime = time.time()
        setStartTime = classmethod(setStartTime)
        def getStartTime(cls):
            return cls.startTime
        getStartTime = classmethod(getStartTime)

        def wsa_Notify(self, ps, address):
            stopTime = time.time()
            elapsedTime = stopTime-self.getStartTime()
            NCS.tests.append(elapsedTime)
            if NCS.min > NCS.tests[-1]: NCS.min = NCS.tests[-1]
            if NCS.max < NCS.tests[-1]: NCS.max = NCS.tests[-1]
            global COUNT
            COUNT += 1
            if COUNT == LOOPS: 
                RESULTS_FILE.write("\t5) time for Notification(%d): % ss\n" %(len(NCS.tests),sum(NCS.tests)))
                RESULTS_FILE.write("\t    mean: %sms\n" %((sum(NCS.tests)/len(NCS.tests))*1000.0))
                RESULTS_FILE.write("\t    max: %sms\n" %(NCS.max*1000.0))
                RESULTS_FILE.write("\t    min: %sms\n" %(NCS.min*1000.0))
                WriteOutData(SCENARIO, 5, NCS.tests)
                NCS.tests = []
                reactor.stop()
                return
            SignedNCS.cb()


############################################################################
#  Run
############################################################################

def Run(locator, kw):
    l = [Test1, Test2, Test3, Test4, Test5]

    if OPTIONS.number is None:
        for test in l: 
            tests = test(locator, **kw)
            if tests is not None: 
                WriteOutData(SCENARIO, OPTIONS.number, tests)
                reactor.stop()
    else:
        print "RUN Test %d" %OPTIONS.number
        test = l[OPTIONS.number-1] 
        tests = test(locator, **kw)
        if tests is not None: 
            WriteOutData(SCENARIO, OPTIONS.number, tests)
            try:
                reactor.stop()
            except RuntimeError, ex:
                # Twisted-2.0 
                # RuntimeError: can't stop reactor that isn't running
                pass
                
 
def WriteOutData(snum, tnum, tests):
    if not OPTIONS.data:
       return
    print "WriteOutData"
    fName = 'Data/Scenario%dTest%d.txt' %(SCENARIO,tnum)
    fd = open(fName, 'w+')
    fd.write('#--------------------------------------------\n')
    fd.write('# Scenario -- %d\n' %SCENARIO)
    fd.write('#     Test -- %d\n' %tnum)
    fd.write('#--------------------------------------------\n')
    fd.write('[tests]\n')
    fd.write('data = %s\n\n' %tests)
    fd.close()


def main():
    global CP,DIV,OPTIONS
    locator = COUNTER.CounterServiceLocator()
    op = GetOptParser('PerfCounterService')
    op.add_option("-m", "--scenario",
                  action="store", dest="scenario", default=1, type="int",
                  help="scenario to run 1-3")
    op.add_option("-n", "--number",
                  action="store", dest="number", default=None, type="int",
                  help="Test number to run 1-4")
    op.add_option("-a", "--div",
                  action="store", dest="div", default=DIV, type="int",
                  help="DIVISOR")
    op.add_option("-b", "--data",
                  action="store", dest="data", default=0, type="int",
                  help="Produce Data")
    (OPTIONS, args) = op.parse_args()
    SetUp(OPTIONS)
    kw = GetPortKWArgs(OPTIONS)
    CP = GetConfigParser()
    DIV = OPTIONS.div
    OPTIONS.ssl = 0
    if OPTIONS.scenario == 1:
        method = Scenario1
    elif OPTIONS.scenario == 2:
        method = Scenario2
    elif OPTIONS.scenario == 3:
        OPTIONS.ssl = 1
        method = Scenario3
    else:
        raise RuntimeError, 'Scenario must be 1-6, not %d' %OPTIONS.scenario
    kw['wsAddressURI'] = WSA200408.ADDRESS
    reactor.callWhenRunning(method, locator, **kw)


if __name__ == '__main__':
    main()
    reactor.run()
