#!/usr/bin/env python
############################################################################
# Joshua R. Boverhof, LBNL
# See LBNLCopyright for copyright notice!
###########################################################################
import os, sys, time, urlparse, optparse, socket, warnings
import ZSI
from ZSI.wstools.Namespaces import *
from ZSI.TC import _get_global_element_declaration as GED

from twisted.python import log
from twisted.internet import reactor, main
from twisted.internet.defer import execute

from pyGridWare.utility.scripts.client import \
    GetOptParser, GetPortKWArgs, SetUp, GetConfigParser

import pyGridWare.generated.types.printerservice
import pyGridWare.generated.stubs.printerservice as CLIENT
from pyGridWare.generated.properties.printerservice.PrinterService import PrinterPort
from ZSI.wstools.logging import setBasicLoggerDEBUG
setBasicLoggerDEBUG()

"""
client for WSRF Interop 2.0 testing.
"""

############################################################################
# GLOBALS
############################################################################
OPTIONS = None
SCENARIOS = ( 'A1', 'A2', 'A3', 'A4', 'B1', 'B2', 'B3', 'B3v', 'C', 'D', 'E')

############################################################################
# Utility
############################################################################
class Holder: pass


def Test():
    if OPTIONS.scenario == 'A1':
        pass
    elif OPTIONS.scenario == 'A2':
        pass
    elif OPTIONS.scenario == 'A3':
        ps = ZSI.parse.ParsedSoap(GET_RESOURCE_PROPERTIES_DOCUMENT_RESPONSE)
        pyobj = ps.Parse(CLIENT.GetResourcePropertyDocumentResponse.typecode)
        print pyobj
        for k,v in pyobj.__dict__.items():
            print "\t%s -- " %k,v
            for k1,k2 in v.__dict__.items():
                print "\t\t%s -- " %k1,k2
    


############################################################################
#  Tests
############################################################################
"""
 Run Scenario A1	 - 	GetResourceProperty
 Run Scenario A2	 - 	GetMultipleResourceProperties
 Run Scenario A3	 - 	GetResourcePropertyDocument
 Run Scenario A4	 - 	QueryResourceProperties
"""
def ScenarioA(**kw):
    locator = CLIENT.PrinterFactoryServiceLocator()
    port = locator.getCreatePrinter(**kw)
    response = port.CreatePrinter(CLIENT.CreatePrinterRequest())

    locator = CLIENT.PrinterServiceLocator()
    kw['endPointReference'] = response._printer_reference
    kw['url'] = response._printer_reference._Address
    port = locator.getPrinter(**kw)

    if OPTIONS.scenario == 'A1': 
        msg = CLIENT.GetResourcePropertyRequest(\
                  ("http://docs.oasis-open.org/wsrf/2005/01/wsrf-Interop-2.0-draft-03.xsd", "queued_job_count"),
              )
        response = port.GetResourceProperty(msg)
    elif OPTIONS.scenario == 'A2': 
        msg = CLIENT.GetMultipleResourcePropertiesRequest()
        msg._ResourceProperty = [
            ("http://docs.oasis-open.org/wsrf/2005/01/wsrf-Interop-2.0-draft-03.xsd", "printer_state"),
            ("http://docs.oasis-open.org/wsrf/2005/01/wsrf-Interop-2.0-draft-03.xsd", "queued_job_count"),
        ]
        response = port.GetMultipleResourceProperties(msg)
    elif OPTIONS.scenario == 'A3': 
        msg = CLIENT.GetResourcePropertyDocumentRequest()
        response = port.GetResourcePropertyDocument(msg)
    elif OPTIONS.scenario == 'A4': 
        msg = CLIENT.QueryResourcePropertiesRequest()
        msg._QueryExpression = Holder()
        msg._QueryExpression._text = "contains(/*/*[namespace-uri()='http://docs.oasis-open.org/wsrf/2005/01/wsrf-Interop-2.0-draft-0.3.xsd' and local-name()='operations supported'],\"Print Job\")"
        response = port.QueryResourceProperties(msg)
    else:
        warnings.warn('No such scenario %s' %OPTIONS.scenario)



"""
 Run Scenario B1	 - 	UpdateResourceProperties
 Run Scenario B2	 - 	SetResourceProperties (update)
 Run Scenario B3	 - 	SetResourceProperties (insert, delete)
 Run Scenario B3 variant	 - 	SetResourceProperties (insert, insert fail ,delete)
"""

"""
 Run Scenario C	 - 	Print_Job (create a job)
"""
"""
 Run Scenario D	 - 	Destroy the Job
"""
"""
 Run Scenario E	 - 	This scenario tests scheduled termination
"""


def main():
    global OPTIONS
    op = GetOptParser('PerfCounterService')
    op.add_option("-n", "--scenario",
                  action="store", dest="scenario", default=SCENARIOS[0], choices=SCENARIOS,
                  help="Scenario to run ")
    op.add_option("-a", "--test",
                  action="store", dest="test", default="0", choices=("0","1"),
                  help="Scenario to run ")

    (OPTIONS, args) = op.parse_args()
    SetUp(OPTIONS)
    kw = GetPortKWArgs(OPTIONS)
    OPTIONS.ssl = 0
    kw['wsAddressURI'] = WSA200408.ADDRESS

    if int(OPTIONS.test) == 1:
        Test()
    elif OPTIONS.scenario.startswith('A'):
        ScenarioA(**kw)
    reactor.stop()

GET_RESOURCE_PROPERTIES_DOCUMENT_RESPONSE = """
<soapenv:Envelope xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:wsa="http://schemas.xmlsoap.org/ws/2004/08/addressing">
 <soapenv:Header>
  <wsa:MessageID soapenv:mustUnderstand="0">uuid:edcbfc20-971b-11d9-b02c-da1dc2931fe4</wsa:MessageID>
  <wsa:To soapenv:mustUnderstand="0">http://schemas.xmlsoap.org/ws/2004/08/addressing/role/anonymous</wsa:To>
  <wsa:Action soapenv:mustUnderstand="0">http://docs.oasis-open.org/wsrf/2005/01/wsrf-Interop-2.0-draft-03.wsdl/GetResourcePropertyDocumentResponse</wsa:Action>
  <wsa:From soapenv:mustUnderstand="0" xmlns:ns1="http://docs.oasis-open.org/wsrf/2005/01/wsrf-Interop-2.0-draft-03.wsdl">
   <Address xmlns="http://schemas.xmlsoap.org/ws/2004/08/addressing">http://12.35.246.160:8080/wsrf/services/PrinterPort</Address>
   <ReferenceProperties xmlns="http://schemas.xmlsoap.org/ws/2004/08/addressing">
    <ns1:ResourceID xmlns:ns1="http://docs.oasis-open.org/wsrf/2005/01/wsrf-Interop-2.0-draft-03.wsdl" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:type="xsd:string">Printer125</ns1:ResourceID>
   </ReferenceProperties>
  </wsa:From>
  <wsa:RelatesTo RelationshipType="wsa:Response" soapenv:mustUnderstand="0">uuid:1111087940.18</wsa:RelatesTo>
 </soapenv:Header>
 <soapenv:Body>
  <wsrf:GetResourcePropertyDocumentResponse xmlns:wsrf="http://docs.oasis-open.org/wsrf/2004/11/wsrf-WS-ResourceProperties-1.2-draft-05.xsd" xmlns:wsrf1="http://docs.oasis-open.org/wsrf/2005/01/wsrf-Interop-2.0-draft-03.xsd" xmlns:add="http://schemas.xmlsoap.org/ws/2004/08/addressing" xmlns:wsrf2="http://docs.oasis-open.org/wsrf/2005/01/wsrf-Interop-2.0-draft-03.wsdl">
   <wsrf1:printer_properties xmlns:wsrf1="http://docs.oasis-open.org/wsrf/2005/01/wsrf-Interop-2.0-draft-03.xsd">
    <wsrf:QueryExpressionDialect>http://www.w3.org/TR/1999/REC-xpath-19991116</wsrf:QueryExpressionDialect>
    <wsrf1:printer_reference>
     <add:Address xmlns:add="http://schemas.xmlsoap.org/ws/2004/08/addressing">http://12.35.246.160:8080/wsrf/services/PrinterPort</add:Address>
     <add:ReferenceProperties xmlns:add="http://schemas.xmlsoap.org/ws/2004/08/addressing">
      <wsrf2:ResourceID xmlns:wsrf2="http://docs.oasis-open.org/wsrf/2005/01/wsrf-Interop-2.0-draft-03.wsdl">Printer125</wsrf2:ResourceID>
     </add:ReferenceProperties>
    </wsrf1:printer_reference>
    <wsrf1:printer_name>Printer125</wsrf1:printer_name>
    <wsrf1:printer_state>Idle</wsrf1:printer_state>
    <wsrf1:printer_is_accepting_jobs>false</wsrf1:printer_is_accepting_jobs>
    <wsrf1:queued_job_count>0</wsrf1:queued_job_count>
    <wsrf1:operations_supported>Print_Job Create_Job Send_Document</wsrf1:operations_supported>
    <wsrf1:document_format_supported>
     <wsrf1:mimeMediaType>text/plain</wsrf1:mimeMediaType>
    </wsrf1:document_format_supported>
   </wsrf1:printer_properties>
  </wsrf:GetResourcePropertyDocumentResponse>
 </soapenv:Body>
</soapenv:Envelope>
"""



if __name__ == '__main__':
    main()
    reactor.run()
