############################################################################
# Joshua R. Boverhof, LBNL
# See LBNLCopyright for copyright notice!
###########################################################################
import os, socket, warnings
from ConfigParser import ConfigParser
from twisted.application import service, internet
from twisted.web.server import Site
from twisted.web.resource import Resource
from twisted.web import static, script
from ZSI.address import Address

from pyGridWare.plugin import setDefaultPlugin
###########################################################################
# Grab configuration information
#
###########################################################################
CP = ConfigParser()
CP.read('server-config.txt')
PORT = CP.getint('Location', 'port')
HOST = socket.gethostbyname(socket.gethostname())
DEBUG = CP.getint('logging', 'debug')
SIGN_NOTIFY = CP.getint('WSSecurity', 'notifications')
USE_SSL = CP.getint('security', 'ssl')
CERTFILE = CP.get('security', 'certfile') or '%s/%s' %(os.environ['HOME'],'.globus/usercert.pem')
KEYFILE = CP.get('security', 'keyfile') or '%s/%s' %(os.environ['HOME'],'.globus/userkey.pem')
CALIST = filter(lambda s: s, map(lambda s: s.strip(), CP.get('security', 'ca').split(' ')))
AUTHSTYLE = CP.get('security', 'authorization')
ACL = CP.get('authorization', AUTHSTYLE)

SEC_PLUGINS = 'plugins'
for opt in CP.options(SEC_PLUGINS):
    setDefaultPlugin(opt, CP.get(SEC_PLUGINS, opt))


###########################################################################
# Debugging -- if config debug is 0, turn off logging.
#
###########################################################################
if DEBUG:
    from ZSI.wstools.logging import setBasicLoggerDEBUG
    setBasicLoggerDEBUG()
else:
    from twisted.python.log import removeObserver, theLogPublisher
    for ob in theLogPublisher.observers: 
        removeObserver(ob)


###########################################################################
# pyGridWare imports
#
###########################################################################
from pyGridWare.wsrf.notification.SubscriptionManager import SubscriptionManager
from pyGridWare.services.SubscriptionManager import SubscriptionManager as SMServiceBinding
#from pyGridWare.services.CounterService import Counter
#from pyGridWare.services.PerfCounterService import Counter as PCounter
from pyGridWare.utility.http import ChunkedHTTPChannel

#SECURE_MESSAGE=True
#try:
#    from pyGridWare.services.SecureConversationService import SecureConversationService
#    from pyGridWare.services.SecureCounterService import SecureCounter
#    from pyGridWare.services.SecurePerfCounterService import Counter as PSCounter
#except ImportError,ex:
#    SECURE_MESSAGE=False

###########################################################################
# SetUpResourceTree
#
###########################################################################
def SetUpResourceTree(root):
    # Discover WS-Address URI
    addr = Address()
    addr.setUp()
    
    # Path "wsrf/core"
    core = Resource()
    root.putChild('wsrf', Resource())
    root.getStaticEntity('wsrf').putChild('core', core)

    smbinding = SMServiceBinding()
    core.putChild('SubscriptionManagerService', smbinding)
    transdict = {'wsAddressURI':addr.wsAddressURI}
    if SIGN_NOTIFY and SECURE_MESSAGE is True:
        from pyGridWare.security.gss.GssSignatureHandler import GssSignatureHandler
        transdict['sig_handler'] = GssSignatureHandler()
        transdict['sig_handler'].setSecureMessage()
    
    # Check for SSL and grab pyOpenSSL Context
    # TODO: Check for M2Crypto and setup SSL Context
    if isinstance(root.server, internet.SSLServer):
        transdict['contextFactory'] = root.server.args[1]
        
    manager = SubscriptionManager.getInstance(smbinding.__class__, transdict)
    
    # Path "wsrf/services": Drop in directory "services"
    dropins = static.File("services")
    dropins.ignoreExt(".rpy")
    dropins.processors = {".rpy": script.ResourceScript}
    root.getStaticEntity('wsrf').putChild('services', dropins)
    
    return root


###########################################################################
# Set up Application
#
###########################################################################
Site.protocol = ChunkedHTTPChannel
root = Resource()
siteFactory = Site(root)
port = None
kw = {}

if USE_SSL:
    from OpenSSL import SSL
    from pyGridWare.utility import ssl 
    kw = {'sslmethod':SSL.SSLv3_METHOD, 'calist':CALIST}
    ContextFactoryClass = ssl.ServerContextFactory
    if AUTHSTYLE == 'acl':
        kw['aclfile'] = ACL
        ContextFactoryClass = ssl.ACLServerContextFactory
    elif AUTHSTYLE == 'gridmap':
        kw['aclfile'] = ACL
        ContextFactoryClass = ssl.GridMapServerContextFactory

    ctxFactory = ContextFactoryClass(KEYFILE, CERTFILE, **kw)
    port = internet.SSLServer(PORT, siteFactory, ctxFactory)
else:
    port = internet.TCPServer(PORT, siteFactory)

root.__class__.server = port
SetUpResourceTree(root)
application = service.Application("WSRF-Container")
port.setServiceParent(application)
