#!/usr/bin/env python
"""
http://c2.com/cgi/wiki?PythonSingleton
By Gary Robinson, grobinson@transpose.com. No rights reserved -- placed in the public domain -- which is only reasonable considering how much it owes to other people's version which are in the public domain. The idea of using a metaclass came from a comment on Gary's blog (see http://www.garyrobinson.net/2004/03/python_singleto.html#comments). Not guaranteed to be fit for any particular purpose.
"""

class SingletonException(Exception):
    pass


class MetaSingleton(type):

    def __new__(metaclass, strName, tupBases, dict):
        if dict.has_key('__new__'):
            raise SingletonException, 'Can not override __new__ in a Singleton'
        return super(MetaSingleton,metaclass).__new__(metaclass, strName, tupBases, dict)


    def __call__(cls, *lstArgs, **dictArgs):
        raise SingletonException, 'Singletons may only be instantiated through getInstance()'


class Singleton(object):

    __metaclass__ = MetaSingleton


    def getInstance(cls, *lstArgs, **kw):
        """
        Call this to instantiate an instance or retrieve the existing instance.
        If the singleton requires args to be instantiated, include them the first
        time you call getInstance.        
        """
        if cls._isInstantiated():
            if len(lstArgs) != 0:
                raise SingletonException, 'If no supplied args, singleton must already be instantiated, or __init__ must require no args'
        else:
            #if len(lstArgs) != cls._getConstructionArgCountNotCountingSelf():  
            #    raise SingletonException, 'If the singleton requires __init__ args, supply them on first instantiation'
            if (len(lstArgs) + len(kw)) != cls._getConstructionArgCountNotCountingSelf():  
                raise SingletonException, 'If the singleton requires __init__ args, supply them on first instantiation'
            instance = cls.__new__(cls)
            instance.__init__(*lstArgs, **kw)
            cls.cInstance = instance
        return cls.cInstance
    getInstance = classmethod(getInstance)


    def _isInstantiated(cls):
        return hasattr(cls, 'cInstance')
    _isInstantiated = classmethod(_isInstantiated)  


    def _getConstructionArgCountNotCountingSelf(cls):
        return cls.__init__.im_func.func_code.co_argcount - 1
    _getConstructionArgCountNotCountingSelf = classmethod(_getConstructionArgCountNotCountingSelf)


    def _forgetClassInstanceReferenceForTesting(cls):
        """
        This is designed for convenience in testing -- sometimes you 
        want to get rid of a singleton during test code to see what
        happens when you call getInstance() under a new situation.


        To really delete the object, all external references to it
        also need to be deleted.
        """
        delattr(cls,'cInstance') 
    _forgetClassInstanceReferenceForTesting = classmethod(_forgetClassInstanceReferenceForTesting)

class Test(Singleton):

    def __init__(self):
        self._size = 10

    def set(self, var):
        self._size = var

    def get(self):
        return self._size

    size = property(get, set, None, "test this")

if __name__ == '__main__':
    a = Test.getInstance()
    b = Test.getInstance()
    c = Test.getInstance()

    
    assert a.size == b.size
    assert a.size == c.size
    print "SIZE: ", a.size

    b.size = 11
    assert a.size == b.size
    assert a.size == c.size
    print "SIZE: ", a.size

