import warnings
from twisted.plugin import getPlugIns as _getPlugIns
import pyGridWare.plugins

DEFAULT = {}
def setDefaultPlugin(interface_name, default_name):
    """Set a default plugin to be returned via a
    interface name.  Store everything as lower.

    @param interface_name: Name of interface class.

    @param default_name: Name of default plugin.

    @return: None
    """
    DEFAULT[interface_name.lower()] = default_name.lower()


def getPlugIn(interface):
    """Retrieve a plugin implementing the given interface beneath the 
    pyGridWare.plugins module.

    @param interface: An interface class.  Only plugins which
    implement this interface will be returned.

    @return: An iterator of plugins.
    """
    it = _getPlugIns(interface, pyGridWare.plugins)
    default = DEFAULT.get(interface.__name__.lower())
    plug = None

    for plug in it:
        if default is None: 
            try: 
                it.next()
            except StopIteration: 
                pass
            else: 
                warnings.warn(
                   'Found multiple %s plugins and no default was specified, choosing plugin %s'\
                   %(interface, plug),
                )
            break
        if default == plug.__name__.lower():
            break
    else:
        if plug is None:
            raise RuntimeError,\
                'failed to find any plugins for interface %s' %(interface)

        raise RuntimeError,\
            'failed to find a plugin named %s for interface %s' %(default,interface)

    return plug
