###########################################################################
# Joshua R. Boverhof, LBNL
# See Copyright for copyright notice!
###########################################################################
import time
DISABLED = False
try:
    import MySQLdb
except ImportError:
    DISABLED = True
    
from zope.interface import classProvides, implements
from twisted.internet import reactor
from twisted.plugin import IPlugin

from pyGridWare.wsrf.faults.PropertiesFaults import ResourceUnknownFault
from pyGridWare.resource.interfaces import IResourceHome, IPersistent
from ConfigParser import SafeConfigParser as ConfigParser,\
    NoSectionError, NoOptionError


class MySQLResourceHome:
    '''MySQL backend plugin.
    TODO: unstable.
    '''
    if DISABLED is False:
        implements(IResourceHome, IPersistent)
        classProvides(IPlugin, IResourceHome, IPersistent)
    
    resourceID = 2222
    CONFIG = '~/my.cnf'


    def __init__(self):
        self.resources = {}
        self.__loaded = False
        
    # IResourceHome
    def add(self, ctx, key=None):
        '''set a Nofication Listener in the topicListeners for each
        resource created.  Whenever a RP is changed --> callback.
        '''
        if key: ctx.resourceID = key
        key = ctx.resourceID
        if key > MySQLResourceHome.resourceID:
            MySQLResourceHome.resourceID = key
            self.create()
        self.resources[key] = ctx
        
        # Notify me of changes
        def cb(*args): self.save(key)
        ctx.properties.topicListeners.append(cb)


    def create(self):
        key = MySQLResourceHome.resourceID 
        MySQLResourceHome.resourceID += 1
        return key

    def delete(self, key):
        del self.resources[key]
        self.save(key)

    def get(self, key):
        if self.resources.has_key(key) is False:
            raise ResourceUnknownFault(\
                    timestamp=time.gmtime(),
                    originator=None,
                    errorcode=None,
                    description=None,
                    faultcause=None
            )
        return self.resources[key]
    
    def getResources(self):
        return self.resources.values()
        
    def setTerminationTime(self, key, terminationTime):
        self.resources[key].TerminationTime = terminationTime

    DB = 'wsrf'
    class ROWS:
        id = 'id'
        context = 'context'
        contextModule = 'contextModule'
        properties = 'properties'
        propertiesModule = 'propertiesModule'
        url = 'url'
        xml = 'xml'
        
    # IPersistent
    def load(self):
        '''load up from wsrf DATABASE.
        '''
        db = MySQLResourceHome.DB
        r = MySQLResourceHome.ROWS
        
        if self.__loaded is not False:
            raise RuntimeError, 'Already loaded'
        
        reactor.addSystemEventTrigger('before', 'shutdown', self.save)
        
        kw = {}
        if MySQLResourceHome.CONFIG is not None:
            kw['read_default_file'] = MySQLResourceHome.CONFIG
            
        conn = MySQLdb.connect(**kw)
        
        self.__loaded = True
        cursor = conn.cursor(MySQLdb.cursors.DictCursor)
        
        # get all resource ids
        cursor.execute ("SELECT * FROM wsrf")
        
        max = MySQLResourceHome.resourceID
        for d in cursor.fetchall():
            # import context & properties
            for i in ('from %(contextModule)s import %(context)s' %d,
                'from %(propertiesModule)s import %(properties)s' %d):
                exec(i)
                
            # create instance of context & properties
            ctx_class = eval(d[r.context])
            prop_class = eval(d[r.properties])
            ctx = ctx_class(prop_class)
            ctx.properties.setUp(d[r.xml])
            id = int(d[r.id])
            ctx.resourceID = id
            ctx.url = d[r.url]
            self.add(ctx)
            if id > max: max = id
        else:
            return
        
        MySQLResourceHome.resourceID = max + 1
        
    def save(self, key=None):
        '''save ResourceProperties corresponding to key, or all if key is None.
        key -- resource ID
        '''
        db = MySQLResourceHome.DB
        r = MySQLResourceHome.ROWS
        
        kw = {}
        if MySQLResourceHome.CONFIG is not None:
            kw['read_default_file'] = MySQLResourceHome.CONFIG
            
        conn = MySQLdb.connect(**kw)
        cursor = conn.cursor(MySQLdb.cursors.DictCursor)
        try:
            cursor.execute("USE wsrf")
        except OperationalError:
            # create the table
            cursor.execute("BEGIN")
            cursor.execute ("""
               CREATE TABLE wsrf
               (
                   id CHAR(40),
                   url CHAR(100),
                   contextModule CHAR(400), 
                   context CHAR(100), 
                   propertiesModule CHAR(400),
                   properties CHAR(100), 
                   xml CHAR(1000)
               )
            """)
            cursor.execute("COMMIT")
        
        cursor.execute("BEGIN")
        
        resources = []
        if key is None:
            resources = self.resources.items()
        elif self.resources.has_key(key):
            resources.append((key,self.resources[key]))
        else:
            cursor.execute("""DELETE FROM wsrf WHERE id = %s""", key)
            
        for k,ctx in resources:
            d = {'id':k, 
                 'url':ctx.url, 
                 'contextModule':ctx.__class__.__module__, 
                 'context':ctx.__class__.__name__, 
                 'propertiesModule':ctx.__class__.__name__,
                 'properties':ctx.properties.__class__.__name__, 
                 'xml':str(ctx.properties),
            }
            cursor.execute("""
               INSERT INTO wsrf (id, url, contextModule, context, propertiesModule, properties, xml)
               VALUES
                   (%(id)s, %(url)s, %(contextModule)s, %(context)s,
                    %(propertiesModule)s, %(properties)s, %(xml)s
                    )""", d)
                    
        cursor.execute("COMMIT")
        cursor.close()
        conn.close()
        
        
        
        