###########################################################################
# Joshua R. Boverhof, LBNL
# See Copyright for copyright notice!
###########################################################################
import time
from zope.interface import classProvides, implements
from twisted.internet import reactor
from twisted.plugin import IPlugin

from pyGridWare.wsrf.faults.PropertiesFaults import ResourceUnknownFault
from pyGridWare.resource.interfaces import IResourceHome, IPersistent
from ConfigParser import SafeConfigParser as ConfigParser,\
    NoSectionError, NoOptionError


class PersistentResourceHome:
    '''Persistent Resource Home, uses a file w/ConfigParser syntax 
    to save ResourceProperties of all resource context properties. 
    All instances will share a single file.
    '''
    implements(IResourceHome, IPersistent)
    classProvides(IPlugin, IResourceHome, IPersistent)
    
    resourceID = 7777
    FILE_NAME = 'persistent_resource_config.txt'

    def __init__(self):
        self.resources = {}
        self.__loaded = False
        
    # IResourceHome
    def add(self, ctx, key=None):
        '''set a Nofication Listener in the topicListeners for each
        resource created.  Whenever a RP is changed --> callback.
        '''
        if key: ctx.resourceID = key
        key = ctx.resourceID
        if int(key) > PersistentResourceHome.resourceID:
            PersistentResourceHome.resourceID = int(key)
            self.create()
        self.resources[str(key)] = ctx
        
        # Notify me of changes
        def cb(*args): self.save(key)
        ctx.properties.topicListeners.append(cb)


    def create(self):
        key = PersistentResourceHome.resourceID 
        PersistentResourceHome.resourceID += 1
        return key

    def delete(self, key):
        del self.resources[str(key)]
        self.save(key)

    def get(self, key):
        if self.resources.has_key(str(key)) is False:
            raise ResourceUnknownFault(\
                    timestamp=time.gmtime(),
                    originator=None,
                    errorcode=None,
                    description=None,
                    faultcause=None
            )
        return self.resources[str(key)]
    
    def getResources(self):
        return self.resources.values()
        
    def setTerminationTime(self, key, terminationTime):
        self.resources[str(key)].TerminationTime = terminationTime

    # IPersistent
    def load(self):
        section = '%s.%s' %(self.__class__.__module__,self.__class__.__name__)
        if self.__loaded is not False:
            raise RuntimeError, 'Already loaded %s' %section
        self.__loaded = True
        
        reactor.addSystemEventTrigger('before', 'shutdown', self.save)
        cp = ConfigParser()
        try:
            cp.readfp(open(self.FILE_NAME, 'r'))
        except IOError:
            return
            
        try:
            gen = cp.items(section)
        except NoSectionError:
            return
            
        try:
            ctx_mod,ctx_class =  eval(cp.get(section, 'context'))
            prop_mod, prop_class =  eval(cp.get(section, 'properties'))
            url = cp.get(section, 'url')
        except NoOptionError:
            # guess we have no entries
            pass
        else:
            # import context & properties
            for i in ('from %s import %s' %(ctx_mod,ctx_class), \
                'from %s import %s' %(prop_mod,prop_class)):
                exec(i)
                
            # create instance of context & properties
            ctx_class = eval(ctx_class)
            prop_class = eval(prop_class)
            for k,v in gen:
                if k in ('context','properties','url'): continue
                ctx = ctx_class(prop_class)
                ctx.properties.setUp(v)
                ctx.resourceID = int(k)
                ctx.url = url
                self.add(ctx)
        
    def save(self, key=None):
        '''save ResourceProperties corresponding to key, or all if key is None.
        key -- resource ID
        '''
        section = '%s.%s' %(self.__class__.__module__,self.__class__.__name__)
        cp = ConfigParser()
        
        try:
            fp = open(self.FILE_NAME)
        except IOError:
            pass
        else:
            cp.readfp(fp)
            fp.close()
        
        context = None
        if key is not None:
            key = str(key)
            context = self.resources.get(key)
            if context is not None:
                if cp.has_section(section) is False:
                    cp.add_section(section)
                cp.set(section, key, str(context.properties))
            elif cp.has_section(section) is True:
                for option in (key,'url','context','properties'):
                    try: cp.remove_option(section, option)
                    except NoOptionError: pass
        else:
            # save all resources I know about
            for key,context in self.resources.items():
                cp.set(section, str(key), str(context.properties))
        
        if context is not None:
            cp.set(section, 'url', context.url)
            cp.set(section, 'context', '("%s","%s")'\
                %(context.__class__.__module__, context.__class__.__name__))
            cp.set(section, 'properties', '("%s","%s")' \
                %(context.__class__.__module__, context.properties.__class__.__name__))
                
        cp.write(open(self.FILE_NAME, 'w'))
        

