############################################################################
# Joshua R. Boverhof, LBNL
# See LBNLCopyright for copyright notice!
###########################################################################
import time
from ZSI.wstools.Namespaces import OASIS
from pyGridWare.wsrf.properties.ResourceProperties import *
from pyGridWare.resource import IPersistent


class ResourceContext(object):
    __slots__ = ['_url','_resourceID','_properties']
    home = None
    
    def __init__(self, properties_class):
        '''properties -- ResourceProperties instance
        ''' 
        self.setResourceID(0)
        self.url = None
        self.properties = properties_class()
        self.properties.context = self
        self.properties.setUp()

    def getAddress(self):
        return self._url
    def setAddress(self, address):
        self._url = address
    url = property(getAddress, setAddress, None,
       'URL of service')

    def getResourceID(self):
        return self._resourceID
    def setResourceID(self, resourceID):
        self._resourceID = resourceID
    resourceID = property(getResourceID, setResourceID, None, 
       'Resource Identifier')

    def getResourceProperties(self):
        return self._properties
    def setResourceProperties(self, properties):
        if isinstance(properties, ResourceProperties) is False:
            raise TypeError, 'expecting a %s instance' %ResourceProperties.__name__
        self._properties = properties
    properties = property(getResourceProperties, setResourceProperties, None,
       'Resource Properties')

    def save(self):
        '''invoke when resource context/properties change 
        to manually save state.
        '''
        home = self.getResourceHome()
        if IPersistent.implementedBy(home):
            home.save(self.getResourceID())
            
    # class methods
    def getResourceHome(cls):
        return cls.home
    getResourceHome = classmethod(getResourceHome)


class ScheduledResourceTerminationContext(ResourceContext):
    '''Base ResourceContext for all Resources that declare
    WS-Lifetime resourceProperties.  Returns times as UTC
    floats, stores as UTC tuple in resourceProperties.
    '''
    __slots__ = []

    def __init__(self, properties_class):
        '''Initialize Lifetime properties.
        ''' 
        ResourceContext.__init__(self, properties_class)

    def getCurrentTime(self):
        '''return a UTC float
        '''
        t = self.properties.CurrentTime
        return time.mktime(t) - time.timezone
    def setCurrentTime(self, seconds):
        assert isinstance(seconds, float), \
            'ResourceContext, CurrentTime expecting a float'
        self.properties.CurrentTime = tuple(time.gmtime(seconds))
    CurrentTime = property(getCurrentTime, setCurrentTime, None, 'Current Time')

    def getTerminationTime(self):
        '''return a UTC float
        '''
        t = self.properties.TerminationTime
        return time.mktime(t) - time.timezone

    def setTerminationTime(self, seconds):
        '''store in properties as a gmtime tuple.
        '''
        assert isinstance(seconds, float) or isinstance(seconds, int), \
            'ResourceContext, TerminationTime expecting a float'
        self.properties.TerminationTime = tuple(time.gmtime(seconds))
    TerminationTime = property(getTerminationTime, setTerminationTime, None, 'Termination Time')


class SubscriptionManagerContext(ScheduledResourceTerminationContext):
    '''Base ResourceContext for all Resources that declare
    WS-BaseNotification SubscriptionManagerRP. 
    '''
