###########################################################################
# Joshua R. Boverhof, LBNL
# See Copyright for copyright notice!
###########################################################################
import time
from zope.interface import implements
from twisted.internet import reactor
from pyGridWare.resource.ResourceSweeper import ResourceSweeper
from pyGridWare.wsrf.faults.PropertiesFaults import ResourceUnknownFault
from pyGridWare.resource import IResourceHome, IPersistent
        

class SimpleResourceHome:
    '''Simple Resource Home registers itself with a ResourceSweeper,
    which finds WS-Lifetime Resources and if scheduled termination
    time is expired it immediately deletes the aforesaid resource(s).
    '''
    implements(IResourceHome)
    resourceID = 1000

    def __init__(self):
        self.resources = {}

    def add(self, ctx, key=None):
        if key: ctx.resourceID = key
        key = ctx.resourceID
        try: 
            if int(key) > SimpleResourceHome.resourceID:
                SimpleResourceHome.resourceID = int(key)
                self.create()
        except ValueError:
            # allow anything
            SimpleResourceHome.resourceID = str(key)
            
        self.resources[str(key)] = ctx

    def create(self):
        key = SimpleResourceHome.resourceID 
        SimpleResourceHome.resourceID += 1
        return key

    def delete(self, key):
        del self.resources[str(key)]

    def get(self, key):
        if self.resources.has_key(str(key)) is False:
            raise ResourceUnknownFault(\
                    timestamp=time.gmtime(),
                    originator=None,
                    errorcode=None,
                    description=None,
                    faultcause=None
            )
        return self.resources[str(key)]
    
    def getResources(self):
        return self.resources.values()
    
    def setTerminationTime(self, key, terminationTime):
        self.resources[str(key)].TerminationTime = terminationTime


