############################################################################
# Joshua R. Boverhof, LBNL
# See Copyright for copyright notice!
###########################################################################
import time, weakref
from twisted.internet import reactor
from twisted.python import log
from pyGridWare.resource.ResourceContext import ScheduledResourceTerminationContext


class ResourceSweeper:
    def __init__(self, resourceHome, delay):
        self.delay = delay
        self.resourceHome = resourceHome
        self.delayedCall = reactor.callLater(self.delay, self.sweep)

    def getSweeperDelay(self):
        return self._delay
    def setSweeperDelay(self, delay):
        self._delay = delay
        assert self.delayedCall.active(), "Resource Sweeper delayedCall not active"
        self.delayedCall.reset(delay)
    delay = property(getSweeperDelay, setSweeperDelay, None, "Sweeper Delay in Seconds")

    def getResourceHome(self):
        return self._resourceHome()
    def setResourceHome(self, home):
        self._resourceHome = weakref.ref(home)
    resourceHome = property(getResourceHome, setResourceHome, None, "Resource Home")

    def sweep(self):
        currentTime = self.delayedCall.getTime()
        log.debug("==> Delayed Call Sweep -- %.2f" %currentTime)
        for resource in self.resourceHome.getResources():
            if isinstance(resource, ScheduledResourceTerminationContext) is False:
                continue
            resource.CurrentTime = currentTime
            terminationTime = resource.TerminationTime
            if terminationTime < currentTime:
                key = resource.getResourceID()
                log.debug("* Sweep terminate %s" %key)
                self.resourceHome.delete(key)
        self.delayedCall = reactor.callLater(self.delay, self.sweep)

