############################################################################
# Joshua R. Boverhof, LBNL
# See LBNLCopyright for copyright notice!
###########################################################################
from zope.interface import Interface


class IResourceHome(Interface):
    '''Interface for ResourceHome, any class implementing this interface
    is expected to handle lifetime management of resources added.
    '''
    def add(self, ctx, key=None):
        '''save resource context referenced via key.
        '''
    def create(self):
        '''create a new key.
        '''
    def delete(self, key):
        '''immediately delete resource context referenced via key.
        '''
    def get(self, key):
        '''get resource context via key.
        '''
    def setTerminationTime(self, key, terminationTime):
        '''set terminationTime of resource, and schedule deletion of 
        resource context referenced via key.
        '''
    def getResources(self):
        '''return all resource contexts.
        '''
    def getResourceContext(self, ps, address):
        '''return resource context.
        '''


class IPersistent(Interface):
    '''Interface for Persistent ResourceHome, any class implementing 
    this interface is expected to save all resource.
    '''
    def save(self, key=None):
        '''If key is None save all resources, else save only 
        resource specified via key.
        '''
    def load(self):
        '''Implement this method to load up state upon initialization.
        '''