############################################################################
# Joshua R. Boverhof, LBNL
# See Copyright for copyright notice!
##########################################################################
import types
import ZSI
from ZSI.wstools.Namespaces import OASIS


class MessageSigUtility:
    '''This class is used to mark a typecode message to
    be signed.  This must be done before serialization.
    When serialized all nodes with "wsu:Id" attribute
    will be processed by the SignatureHandler.
    '''
    indx = 0

    def Sign(cls, request, typecode=None):
        '''request must either have a typecode attribute, or
        a typecode must be provided as a parameter. The typecode 
        parameter can be a ZSI.TC.Typecode instance, or a class
        definition with a typecode class variable that is.

        This method will add a dictionary to to the request based
        on the "attrs_aname" specified by the typecode if it's not
        already present.
        '''
        key = (OASIS.UTILITY,"Id")
        if typecode is None:
            typecode = getattr(request, 'typecode', typecode)
        elif not isinstance(typecode, ZSI.TC.TypeCode):
            typecode = getattr(typecode, 'typecode', None)

        assert typecode is not None, \
            'Must provide a typecode with Request'
        assert isinstance(typecode, ZSI.TC.TypeCode), \
            'typecode must be a ZSI.TC.TypeCode, got a %s' %type(typecode)
        if not hasattr(request, typecode.attrs_aname):
            try:
                setattr(request, typecode.attrs_aname, {})
            except AttributeError, ex:
                raise TypeError, 'can\'t set attributes on type %s' %type(request)

        attr_dict = getattr(request, typecode.attrs_aname)
        attr_dict[key] = cls.GetUniqueId()
    Sign = classmethod(Sign) 

    def GetUniqueId(cls):
        cls.indx += 1
        return "digestSource%d" %cls.indx
    GetUniqueId = classmethod(GetUniqueId)



