############################################################################
# Matt Rodriguez, LBNL
# See Copyright for copyright notice!
###########################################################################
from ZSI.wstools.Namespaces import SOAP, OASIS, DSIG
import sha

class SignatureHandlerError(RuntimeError):
    pass

class BaseSignatureHandler:
    """
    The parent class for all of the Signature handlers.
    This class will handle tasks which are not specific
    to gss signing
    """
    tokenRef = 10101
    def signHeaderElements(self, sw):
        '''Set Id attribute for SOAP-ENV:Header child elements.
        '''
        processorNss={'SOAP-ENV':SOAP.ENV}
        sw.dom.setContext(processorNss=processorNss)
        nodes = sw.dom.evaluate(expression='//SOAP-ENV:Header')
        headerNode = nodes[0]
        headerNodes = headerNode.getElements(None, None)
        for node in headerNodes:
            idVal = self.getTokenRefString()
            node.setAttributeNS(OASIS.UTILITY, 'Id', idVal)

    def signBodyElement(self, sw):
        '''Set Id attribute for SOAP-ENV:Body child element.
        '''
        processorNss={'SOAP-ENV':SOAP.ENV}
        sw.dom.setContext(processorNss=processorNss)
        nodes = sw.dom.evaluate(expression='//SOAP-ENV:Body/*')
        if len(nodes) != 1:
            if len(nodes) == 0:
                raise SignatureHandlerError('No element in SOAP-ENV:Body')
            raise SignatureHandlerError('Multiple elements in SOAP-ENV:Body')

        idVal = self.getTokenRefString()
        nodes[0].setAttributeNS(OASIS.UTILITY, 'Id', idVal)

    def getDigestValueFromSoapWriter(self, sw, uri, algorithm=DSIG.C14N, **kw):
        '''TODO: looks incomplete, no algo?
        '''
        if algorithm == DSIG.C14N_EXCL and not kw.has_key('unsuppressedPrefixes'):
            kw['unsuppressedPrefixes'] = []

        digExpString = '//*[@wsu:Id="' + uri[1:] +'"]'
        if not sw.dom.isContextInitialized():
            raise SignatureHandlerError('SoapWriter ElementProxy has no context')
        nodes = sw.dom.evaluate(expression=digExpString)
        node = nodes[0]
        cxml = node.canonicalize(algorithm, **kw)
        digestValue = sha.sha(cxml).digest()
        return digestValue

    def getDigestValueFromDom(self, element, uri, algorithm=DSIG.C14N, **kw):
        '''If exclusive c14n can specify unsuppressedPrefix list, else ignored.
        KeyWord Arguments:
            algorithm --
            unsuppressedPrefixes --
        '''
        if algorithm == DSIG.C14N_EXCL and not kw.has_key('unsuppressedPrefixes'):
            kw['unsuppressedPrefixes'] = []

        uriExpString = '//*[@wsu:Id="' + uri[1:] +'"]'
        if not element.isContextInitialized():
            raise SignatureHandlerError('ElementProxy has no context')
        uriNodes = element.evaluate(expression=uriExpString)
        l = len(uriNodes)
        if l == 0: raise VerifyError("uri not found: " + uri)
        if l > 1: raise VerifyError("more than one uri found with uri: " + uri)
        cxml = uriNodes[0].canonicalize(algorithm, **kw)
        digestValue = sha.sha(cxml).digest()
        return digestValue

    def getTokenRefString(self):
        """
        Return an unique number to identify an element
        for the element's id attribute
        """
        BaseSignatureHandler.tokenRef +=1
        return str(BaseSignatureHandler.tokenRef)
