#!/usr/bin/env python
############################################################################
# Joshua R. Boverhof, LBNL
# See LBNLCopyright for copyright notice!
###########################################################################
import urlparse
import ZSI.TC
from ZSI.TC import _get_global_element_declaration as GED
from ZSI.wstools.Namespaces import SOAP, BEA, WSSE,GLOBUS
from pyGridWare.generated.stubs.secureconversation import *
from pyGridWare.security.gss.GssSecureConversation import GssSecurityContext
from pyGridWare.security.gss.GssSignatureHandler import GssSignatureHandler


class RequestType(str):
    typecode = GED("http://schemas.xmlsoap.org/ws/2004/04/trust","RequestType")

        
class Token(str):
    typecode = GED("http://schemas.xmlsoap.org/ws/2004/04/trust","TokenType")


class BinaryExchange(str):
    typecode = GED("http://schemas.xmlsoap.org/ws/2004/04/trust","BinaryExchange")

    def __new__(cls, value):
        s = str.__new__(cls, value)
        s._attrs = {}
        return s


class Holder:
    pass

class GssAuthenticationHandler:
    mechanism = "http://www.globus.org/2002/04/xmlenc#gssapi-gsi"
    namespaceURI = "http://wsrf.globus.org/security/authentication/secureconv/types"

    def getMechanism(cls):
        return cls.mechanism
    getMechanism = classmethod(getMechanism)

    def getNamespace(cls):
        return cls.namespaceURI
    getNamespace = classmethod(getNamespace)

    def continueCreateContext(self, port,ctx, response):
        ctxId = response._attrs['Context']
       
        #serverTokens = response._any[(WSSE.TRUST,"BinaryExchange")][0]
        serverTokens = None
        for any in response._any:
            if hasattr(any, 'typecode') \
                and (any.typecode.nspname,any.typecode.pname) == (WSSE.TRUST,"BinaryExchange"):
                serverTokens = any
                break
        else:
            raise RuntimeError, '%s continueCreateContext expecting BinaryExchange' %self.__class__

        request = RequestSecurityTokenResponseMsg()
        request._any = []
        request._attrs = {}
        #I have to put the context Id into the ._attrs member
        request._attribute_typecode_dict = {}
        request._attribute_typecode_dict['Content'] = ZSI.TC.String()
        request._attrs['Context'] = ctxId
        tokens = ctx.initSecurityContext(serverTokens)
        binEx = BinaryExchange(tokens)
        binEx._attrs['EncodingType'] = \
            "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-soap-message-security-1.0#Base64Binary"
        binEx._attrs['ValueType'] = \
            "http://www.globus.org/ws/2004/09/security/sc#GSSAPI_GSI_TOKEN"
        request._any.append(binEx)

        response = port.RequestSecurityTokenResponse(request)
        return response

    def createContext(self,port,ctx):
        tokens = ctx.initSecurityContext()
        binEx = BinaryExchange(tokens)
        request = RequestSecurityTokenMsg()
        request._any = []
        request._any.append(RequestType('http://schemas.xmlsoap.org/ws/2004/04/security/trust/Issue'))
        request._any.append(Token('http://schemas.xmlsoap.org/ws/2004/04/security/sc/sct'))
        request._any.append(binEx)
        request._any[-1]._attrs['EncodingType'] = \
            "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-soap-message-security-1.0#Base64Binary"
        request._any[-1]._attrs['ValueType'] = \
            "http://www.globus.org/ws/2004/09/security/sc#GSSAPI_GSI_TOKEN"
        response = port.RequestSecurityToken(request)
        return response

    def authenticate(self, delegation=True, **kw):
        context = GssSecurityContext()
        mechanism = GssAuthenticationHandler.getMechanism()
        namespaceURI = GssAuthenticationHandler.getNamespace()
        mechanism_attr_key = (namespaceURI,"mechanism-type")
        continue_attr_key = (namespaceURI,"continue-needed")

        if delegation is True:
            context.setDelegation()

        # Initialize Port instance
        locator = SecureConversationServiceLocator()
        port = locator.getSecureConversation(**kw)
        response = self.createContext(port, context)

        # When the context has been created the response has
        # a SecurityContextToken
        done = False
        secureContextToken = address = None
        while secureContextToken is None:
            response = self.continueCreateContext(port,context,response)
            for any in response._any:
                if hasattr(any, 'typecode') is False: 
                    raise RuntimeError, 'No typecode'
                key = (any.typecode.nspname, any.typecode.pname)
                if key == (BEA.SECCONV, 'SecurityContextToken'):
                    secureContextToken = any

        #secureContextToken._Identifier
        for any in secureContextToken._any:
            key = (any.typecode.nspname, any.typecode.pname)
            if key == (GLOBUS.SECCONV, 'ContextAddress'):
                address = any
                break
        else:
            raise RuntimeError, 'Missing ContextAddress.'

        for any in address._ReferenceProperties._any:
            #if hasattr(any, 'typecode') is False: continue
            #key = (any.typecode.nspname, any.typecode.pname)
            #if key == (GLOBUS.CORE, 'SecureContextKey'):
            #    contextID = any
            #    break
            contextID = any
            break
        else:
            raise RuntimeError, 'Can not find context key'

        context.setContextID(contextID)
        home = GssSecurityContext.getResourceHome()
        home.add(context, contextID)
        return contextID


