############################################################################
# Matt Rodriguez, LBNL
# See Copyright for copyright notice!
###########################################################################
from pyGlobus.security import GSICredential, STACK_X509_Factory


class GSIAuthentication:
    """
    This class performs signing,verification and authentication
    information. It is another layer about the GSICredential
    class from pyGlobus
    """

    def __init__(self):
        self.__cred = GSICredential()
        self.__cred.read()
        
            
    def getPkiPath(self):
        """
        Returns the DER encoded 
        """
        certChain = self.__cred.get_cert_chain()
        cert = self.__cred.get_cert()
        certChain.insert(cert, 1)
        encoding = certChain.get_der_encoding()
        return encoding

    def sign(self, signStr):
        """
        signs the data in the SignStr, the data
        must be hashed using SHA1, and the private
        key obtained from the user proxy must be a RSA
        key

        **Arguments**

        - *signStr*
        A string of sha1 hashed data to sign

        **Returns**
        The signed string
        """
        pkey = self.__cred.get_key()
        signedData = pkey.sign(signStr)
        return signedData

    def verify(self, certChainInfo, hash, signature):
        """
        verifies the signature

        **Arguments**

        - *certChainInfo*
        A DER encoded set of certificates, this method
        assumes that the 2nd certificate has the public key.
        The 1st certificate is the proxy which does not
        have the public key

        - *hash*
        A sha1 hash of the data, this is compared to the
        decrypting the signature with the public key
        and if it is equal, then it verifies

        - *signature*
        The signature that you wish to verify

        **Returns**
        Returns True if it verifies and False if it does not verify
        """
        
        stackFact = STACK_X509_Factory()
        stack = stackFact.createStackFromEncodedSequence(certChainInfo)
        pubKeyX509 = stack.pop()
        #pubKeyX509 = stack.pop()
        pubKey = pubKeyX509.get_public_key()
        verify = pubKey.verify(hash, signature)
        return verify

    
