############################################################################
# Joshua R. Boverhof, LBNL
# See Copyright for copyright notice!
###########################################################################
import string

from Ft.Lib import Uuid
from pyGlobus.security import GSSCred, GSSContext, GSSContextException
from pyGlobus.security import GSSName, GSSMechs, GSSUsage, GSSBuffer
from pyGlobus.security import ContextRequests
from pyGridWare.resource.ResourceHome import SimpleResourceHome
import base64

class SecurityManager:
    pass

class GSSManager(SecurityManager):
    """
    """
    def createCredential(self):
        cred = GSSCred()
        name, mechs, usage  = GSSName(), GSSMechs(), GSSUsage()
        usage.set_usage_initiate()
        cred.acquire_cred(name, mechs, usage)
        return cred

    def createContext(self, cred):
        lifetime, credName = cred.inquire_cred()
        context = GSSContext()
        requests = ContextRequests()
        return context, requests, lifetime, credName
           

class GssSecurityContextHome(SimpleResourceHome):
    """
    """


class GssSecurityContext:
    """Associates a GSSContext with a context Id. It is used for GSI secure converstation.
    """
    home = GssSecurityContextHome()

    def __init__(self):
        manager = GSSManager()
        self._contextID = None
        self._cred = manager.createCredential()
        self._GSSContext, self._requests, lifetime, self._credName = \
            manager.createContext(self._cred)

        self._contextCreated = False
        
    def getContext(self):
        """org.ietf.jgss.GSSContext getContext()
        """
        return self._GSSContext

    def getContextID(self):
        """java.lang.String getContextId()
        """
        return self._contextID

    def getDelegatedCredential(self):
        """
        org.ietf.jgss.GSSCredential getDelegatedCredential() 
        """
        pass

    def getResourceHome(cls):
        return cls.home 
    getResourceHome = classmethod(getResourceHome)
   
    def setCredential(self, cred):
        self._cred = cred

    def setContext(self, context):
        self._GSSContext = context

    def setContextID(self, contextID):
        self._contextID = contextID

    def setDelegation(self):
        self._requests.set_delegation()

    def setEncryption(self):
        self._requests.set_confidentiality()

    def hashCode(self):
        """init context-id
        """
        hcode = Uuid.UuidAsString(Uuid.GenerateUuid())
        tmpList = hcode.split('-')
        tmpList[3] = '0' * 8
        tmpList[4] = '0' * 12
        return(string.join(tmpList, '-'))

    def contextCreated(self):
        return self._contextCreated
    
    def acceptSecurityContext(self, inToken64):
        """
        **Arguments**

        - *inToken* base64 encoded tokens, which contain
        authentication information
        
        """
        tokens = base64.decodestring(inToken64)
        major,minor,outToken = \
            self._GSSContext.accept_context(\
                acceptor_cred=self._cred, 
                inputTokenString=tokens)

        if major == 0:
            self._contextCreated = True
            
        tokens64 = base64.encodestring(outToken)
        outTokenSplit = tokens64.split('\n')
        outTokens = ""
        for token in outTokenSplit:
            outTokens +=token

        return outTokens
        

    def initSecurityContext(self, inToken64=""):
        """
        XXX pyGlobus throws a segmentation Fault if inToken=None

        The ContextRequests object needs to be the last argument to the
        init_context calls that you make.

        **Arguments**
        - *inToken* - must be base64 encoded
        
        **Returns**
        a string of base64 encoded tokens that has no newlines
        """
        inToken = ''
        if not inToken64 == '':
            inToken = base64.decodestring(inToken64)
            
        major,minor,outToken = \
                    self._GSSContext.init_context(
                    init_cred=self._cred, 
                    target_name=self._credName, 
                    inputTokenString=inToken, 
                    requests=self._requests)

        
        tokens64 = base64.encodestring(outToken)
        outTokenSplit = tokens64.split('\n')
        outTokens = ""
        for token in outTokenSplit:
            outTokens +=token

        return outTokens

    def getMIC(self, signedInfoC14n):
        signatureValue = self._GSSContext.get_mic(signedInfoC14n)
        return signatureValue

    def verifyMIC(self, signedInfoC14n, signatureValue):
        return self._GSSContext.verify_mic(signedInfoC14n, signatureValue)

    def wrap(self, messageStr):
        return self._GSSContext.wrap(messageStr)

    def unwrap(self, encryptedStr):
        return self._GSSContext.unwrap(encryptedStr)



