############################################################################
# Joshua R. Boverhof, LBNL
# See Copyright for copyright notice!
###########################################################################
from pyGridWare.security.m2.M2SecurityContext import M2SecurityContext


class ContextToken:
    def __init__(self):
        self._base64Token = None
        self._context_id = None
        self._attrs = {}

    def setBase64Token(self, base64Token):
        self._base64Token = base64Token
        
    def setContextID(self, context_id):
        self._context_id = context_id

    def setAttributeContinueNeeded(self, continue_needed):
        key = ("http://wsrf.globus.org/security/authentication/secureconv/types","continue-needed")
        self._attrs[key] = continue_needed


class GssSecureConversation:
    MECHANISM = "http://www.globus.org/2002/04/xmlenc#gssapi-gsi"

    def initTokenExchange(cls, request, response):
        key = ("http://wsrf.globus.org/security/authentication/secureconv/types","mechanism-type")
        mech = request._initContextToken._attrs[key]
        if mech != cls.MECHANISM:
            raise RuntimeError, 'Bad Mechanism(%s)' %mech
        base64Token = request._initContextToken._base64Token
        context_id = request._initContextToken._context_id
        context = M2SecurityContext()

        home = M2SecurityContext.getResourceHome()
        home.create(context, context_id)

        response._contextTokenOut = ContextToken()
        response._contextTokenOut.setBase64Token(context.acceptSecurityContext(base64Token))
        response._contextTokenOut.setContextID(context_id)
        response._contextTokenOut.setAttributeContinueNeeded(1)
    initTokenExchange = classmethod(initTokenExchange)

    def continueTokenExchange(cls, request, response):
        base64Token = request._contextToken._base64Token
        context_id = request._contextToken._context_id

        home = M2SecurityContext.getResourceHome()
        context = home.get(context_id)

        response._contextTokenOut = ContextToken()
        token = context.acceptSecurityContext(base64Token)
        response._contextTokenOut.setBase64Token(token)
        if token:
            response._contextTokenOut.setAttributeContinueNeeded(1)
        else:
            response._contextTokenOut.setAttributeContinueNeeded(0)
        response._contextTokenOut.setContextID(context_id)
    continueTokenExchange = classmethod(continueTokenExchange)




