############################################################################
# Matt Rodriguez, LBNL
# See Copyright for copyright notice!
###########################################################################
"""
Module that implements that SecureMessage protocol using M2Crypto
"""
#from pyGlobus.security import GSICredential, STACK_X509_Factory
from pyGsi import proxy
from M2Crypto import X509
from M2Crypto import RSA

class M2Authentication:
    """
    This class performs signing,verification and authentication
    information. It is another layer about the GSICredential
    class from pyGlobus
    """

    def __init__(self):
        self.__cred = proxy.Proxy()
        self.__cred.read()
        
            
    def getPkiPath(self):
        """
        Returns the DER encoded certificate chain
        """
        cert_chain = X509.X509_Stack() 
        proxy_cert = self.__cred.getcert()
        issuer_cert = self.__cred.getissuer()
        cert_chain.push(issuer_cert)
        cert_chain.push(proxy_cert)
        encoding = cert_chain.as_der()
        return encoding

    def sign(self, signStr):
        """
        signs the data in the SignStr, the data
        must be hashed using SHA1, and the private
        key obtained from the user proxy must be a RSA
        key

        **Arguments**

        - *signStr*
        A string of sha1 hashed data to sign

        **Returns**
        The signed string
        """
        pkey = self.__cred.getkey()
        signedData = pkey.sign(signStr)
        return signedData

    def verify(self, certChainInfo, hash, signature):
        """
        verifies the signature

        **Arguments**

        - *certChainInfo*
        A DER encoded set of certificates, this method
        assumes that the 2nd certificate has the public key.
        The 1st certificate is the proxy which does not
        have the public key

        - *hash*
        A sha1 hash of the data, this is compared to the
        decrypting the signature with the public key
        and if it is equal, then it verifies

        - *signature*
        The signature that you wish to verify

        **Returns**
        Returns True if it verifies and False if it does not verify
        """
        stack = X509.new_stack_from_der(certChainInfo) 
        pubKeyX509 = stack.pop()
        pubKey = pubKeyX509.get_pubkey()
        #pubKeyPem = pubKey.as_pem()
        #rsa = RSA.load_key_string(pubKeyPem)
        rsa = pubKey.get_rsa()
        verify = rsa.verify(hash, signature)
        return verify

    
