############################################################################
# Joshua R. Boverhof, LBNL
# See Copyright for copyright notice!
########################################################################### 

from Ft.Lib import Uuid
from pyGridWare.resource.ResourceHome import SimpleResourceHome
import base64

from M2Crypto import SSL
from M2Crypto import BIO
from pyGsi import proxy
import pyGsi._mic

class SecurityManager:
    pass

class M2Manager(SecurityManager):
    """
    """
    def createCredential(self, proxypath=None):
        cred = proxy.Proxy()
        cred.read()
        return cred

    def createContext(self, cred):
        #TODO: What is cred for?
        #TODO: initiate keyword??
        context = M2Context() 
        return context
           

class GssSecurityContextHome(SimpleResourceHome):
    """
    """
class M2Context:
    #TODO: Naming is strange.  M2SecurityContext contains
    #      an M2Context which contains a SSL.Context???
    #
    def __init__(self, initiate=True):
        self._context = SSL.Context('sslv3')
        self._context.set_info_callback()
        proxypath = proxy.get_proxy_filename()
        self._context.load_cert_chain(proxypath)
        self._delegate = False
        self._connection = SSL.Connection(self._context)
        self._readbio = BIO.MemoryBuffer()
        self._writebio = BIO.MemoryBuffer()
        self._sslbio = BIO.SSLBio()
        self._connection.set_bio(self._readbio, self._writebio)
        self._sslbio.set_ssl(self._connection)
        if initiate:
            self._connection.set_connect_state()
        else:
            self._connection.set_accept_state()

    def initContext(self, inTokens=None):
        
        if inTokens is not None:
            write_len = 0
            token_length = len(inTokens)
            while write_len < token_length: 
                write_len += self._readbio.write(inTokens[write_len:])
        
        ret = self._sslbio.do_handshake()
        handshake_complete = False
        if ret <= 0:
            if not self._sslbio.should_retry() or not self._sslbio.should_read():
                raise RuntimeError("""problem with sslbio
                                     This should probably be an exception""")
        else:
            handshake_complete = True
            if self._delegate:
                self._sslbio.write("D")
            else:
                self._sslbio.write("0")

        outTokens = self._writebio.read()
        
        return handshake_complete, outTokens
    
    def acceptContext(self, inTokens=None):
        self._readbio.write(inTokens)
        ret = self._sslbio.do_handshake()
        handshake_complete = False
        if ret <= 0:
            if not self._sslbio.should_retry() or not self._sslbio.should_read():
                raise RuntimeError("""problem with sslbio
                                      This should probably be an exception""")
        else:
            handshake_complete = True
        
        outTokens = self._writebio.read()
        return handshake_complete, outTokens

    def setDelegation(self, val):
        """
        Set the delegation member variable
        """
        self._delegate = val
       
    def _getSSL(self):
        """
        Returns the PyCObject that is contained within the
        connection instance. Should only be used within this
        module.
        """
        return self._connection.ssl

class M2SecurityContext:
    """
    Associates a GSSContext with a context Id. It is used 
    for GSI secure converstation.
    """
    home = GssSecurityContextHome()

    #TODO: How do I use a cred/keypair with this setup?
    #TODO: Make manager a class attribute
    manager = M2Manager()

    def __init__(self):
        self._contextID = None
        self._cred = self.manager.createCredential()
        self._context = self.manager.createContext(self._cred)
        self._contextCreated = False
        self._delegate = False

    def getContext(self):
        """org.ietf.jgss.GSSContext getContext()
        """
        return self._context

    def getContextID(self):
        """java.lang.String getContextId()
        """
        return self._contextID

    def getDelegatedCredential(self):
        """
        org.ietf.jgss.GSSCredential getDelegatedCredential() 
        """
        pass

    @classmethod
    def getResourceHome(cls):
        return cls.home 
    
    def setCredential(self, cred):
        self._cred = cred

    def setContext(self, context):
        self._context = context

    def setContextID(self, contextID):
        self._contextID = contextID

    def hashCode(self):
        """init context-id
        """
        hcode = Uuid.UuidAsString(Uuid.GenerateUuid())
        tmpList = hcode.split('-')
        tmpList[3] = '0' * 8
        
        tmpList[4] = '0' * 12
        return("-".join(tmpList))

    def contextCreated(self):
        return self._contextCreated
    
    def acceptSecurityContext(self, inToken64=None):
        """
        **Arguments**

        - *inToken* base64 encoded tokens, which contain
        authentication information
        
        **Returns**
        a string of base64 encoded tokens that has no newlines
        """
        tokens = None
        if inToken64 is not None:
            tokens = base64.decodestring(inToken64)
        handshake_complete, outToken = self._context.acceptContext(tokens)

        if handshake_complete:
            self._contextCreated = True
            
        tokens64 = base64.encodestring(outToken)
        outTokenSplit = tokens64.split('\n')
        outTokens = ""
        for token in outTokenSplit:
            outTokens += token

        return outTokens
        

    def initSecurityContext(self, inToken64=None):
        """
        **Arguments**
        - *inToken* - must be base64 encoded
        
        **Returns**
        a string of base64 encoded tokens that has no newlines
        """
        inToken = None
        if inToken64 is not None:
            inToken = base64.decodestring(inToken64)
            
        handshake_complete, outToken = self._context.initContext(inToken)
        
       
       
        tokens64 = base64.encodestring(outToken)
        outTokenSplit = tokens64.split('\n')
        outTokens = ""
        for token in outTokenSplit:
            outTokens += token

        return outTokens
    
    def setDelegation(self):
        """
        If delegation is done, then after
        the handshake completes write 'D' into
        the sslbio, otherwise write 0
        """
        self._delegate = True
     
    def getMIC(self, signedInfoC14n):
        """
        Create a message integrity code from 
        the cannonicalized message
        """
        signature_value = pyGsi._mic.ssl_generate_mic(self._context._getSSL(),
                                                      signedInfoC14n)
        return signature_value

    def verifyMIC(self, signedInfoC14n, signatureValue):
        """
        Verify a the message integrity code from the
        cannonicalized message
        """
        return pyGsi._mic.ssl_verify_mic(self._context._getSSL(), 
                                         signedInfoC14n, signatureValue)

"""
    def wrap(self, messageStr):
        return self._GSSContext.wrap(messageStr)

    def unwrap(self, encryptedStr):
        return self._GSSContext.unwrap(encryptedStr)

""" 
