############################################################################
# Joshua R. Boverhof, LBNL
# See LBNLCopyright for copyright notice!
###########################################################################
import time

from twisted.internet import reactor

from pyGridWare.generated.services.stubs.JobRunnerService_services_server import *
#from pyGridWare.generated.resource.JobRunner_context import *
from pyGridWare.generated.resource.JobRunner_context import Context as JobRunnerContext

from pyGridWare.addressing.AddressingUtils import AddressingUtils
from pyGridWare.lifetime.ResourceLifetime import \
    ImmediateResourceTermination, ScheduledResourceTermination

from pyGridWare.services.SubscriptionManager import \
    SubscriptionManager

from twisted.internet.protocol import ProcessProtocol
from twisted.internet.error import ProcessDone, ProcessTerminated
from twisted.python.failure import Failure

class JobRunnerProcessProtocol(ProcessProtocol):
    def __init__(self, rp):
        self.rp = rp

    def connectionMade(self):
        print "connectionMade"
        self.rp.set_status("connectioNMade")
        self.transport.closeStdin()
        
    def inConnectionLost(self):
        print "inConnectionLost"
        self.rp.set_status("inConnectionLost")

    def outConnectionLost(self):
        print "outConnectionLost"
        self.rp.set_status("outConnectionLost")
        
    def outReceived(self, data):
        print "outReceived"
        self.rp.set_status("outReceived, data: %s" % data)
        
    def errReceived(self, data):
        print "errReceived"
        self.rp.set_status("errReceived, data: %s" % data)

    def processEnded(self, status):
        print "processEnded", status.value.exitCode
        
        if isinstance(status.value, ProcessDone):
            self.rp.set_status("processEnded (terminated normally)")

        elif isinstance(status.value, ProcessTerminated):
            self.rp.set_status("processEnded (terminated abnormally: status: %d)" % \
                               status.value.exitCode)
            
        print "EXIT CODE: ", type(status.value.exitCode)
        print "RP: ", self.rp
        self.rp.set_returnCode(0)
        
class JobRunner(JobRunnerService):

    def wsa_create(self, ps, address, **kw):
        request,response = JobRunnerService.wsa_create(self, ps, address, **kw)

        # Create Resource Context
        ctx = JobRunnerContext()
        url = self.getServiceURL()
        ctx.setAddress(url)
        home = ctx.getResourceHome()
        key = home.create()
        home.add(ctx, key)
        ctx.setResourceID(key)

        # Access resource properties, and set status 0
        rp = ctx.properties

        rp.set_status('INITIAL')
        rp.set_returnCode(-1)
        rp.set_executable(request._executable)
        rp.set_argument(request._argument)

        #args = [request._executable]
        #args.extend(request._argument)
        #reactor.spawnProcess(JobRunnerProcessProtocol(rp), 
        #    request._executable, 
        #    request._argument
        #)

        # Create EndPointReference and set in response
        epr = AddressingUtils.createEndpointReference(ctx)
        response._EndpointReference = epr
        return request,response

    def wsa_run(self, ps, address, **kw):
        request,response = JobRunnerService.wsa_run(self, ps, address, **kw)

        # Get Resource Context
        home = JobRunnerContext.getResourceHome()
        ctx = home.getResourceContext(ps, address)

        rp = ctx.properties
        ex = rp.get_executable()
        args = rp.get_argument()
        reactor.spawnProcess(JobRunnerProcessProtocol(rp), ex, args)
        rp.set_status('RUNNING')

        return request,response

    def wsa_Destroy(self, ps, address, **kw):
        request,response = JobRunnerService.wsa_Destroy(self, ps, address, **kw)

        # Get Resource Context
        home = JobRunnerContext.getResourceHome()
        ctx = home.getResourceContext(ps, address)

        # Access resource properties, and set status 0
        #rp = ctx.properties
        #rp.set_status('DESTROY')
        #rp.set_returnCode(666)

        # Terminate Resource
        #ImmediateResourceTermination.Destroy(ctx)
        return request,response

    def wsa_getMultipleResourceProperties(self, ps, address, **kw):
        request,response = JobRunnerService.wsa_getMultipleResourceProperties(self, ps, address, **kw)

        # Get Resource Context
        home = JobRunnerContext.getResourceHome()
        ctx = home.getResourceContext(ps, address)

        namespaceURI,name = request
        property = ctx.properties.getResourceProperty(namespaceURI,name)

        # assign return values to response object
        response._any = property
        return request,response

    def wsa_GetResourceProperty(self, ps, address, **kw):
        request,response = JobRunnerService.wsa_GetResourceProperty(self, ps, address, **kw)

        # Get Resource Context
        home = JobRunnerContext.getResourceHome()
        ctx = home.getResourceContext(ps, address)

        print "-"*90
        print "GET REQUEST: ", request
        print "-"*90
        namespaceURI,name = request
        property = ctx.properties.getResourceProperty(namespaceURI,name)

        # assign return values to response object
        response._any = property
        return request,response

    def wsa_queryResourceProperties(self, ps, address, **kw):
        request,response = JobRunnerService.wsa_queryResourceProperties(self, ps, address, **kw)
        return request,response

    def wsa_setResourceProperty(self, ps, address, **kw):
        request,response = JobRunnerService.wsa_setResourceProperty(self, ps, address, **kw)
        return request,response

    def wsa_setTermTime(self, ps, address, **kw):
        request,response = JobRunnerService.wsa_setTermTime(self, ps, address, **kw)

        # Get Resource Context
        home = JobRunnerContext.getResourceHome()
        ctx = home.getResourceContext(ps, address)

        # Schedule Termination
        ScheduledResourceTermination.SetTerminationTime(ctx, request._RequestedTerminationTime)

        # assign return values to response object
        response._NewTerminationTime = time.gmtime(ctx.getTerminationTime())
        response._CurrentTime = time.gmtime(time.time())
        return request,response



