############################################################################
# Joshua R. Boverhof, LBNL
# See Copyright for copyright notice!
###########################################################################
import ZSI.TC 
from ZSI.TC import _get_global_element_declaration as GED
from ZSI.TC import ElementDeclaration
from ZSI.wstools.Namespaces import WSSE, BEA, GLOBUS, WSTRUST, OASIS

from pyGridWare.security.m2.M2SecurityContext import M2SecurityContext
from pyGridWare.security.m2.M2SecurityContext import M2Context

# generated code
import pyGridWare.generated.types.secureconversation
from pyGridWare.generated.services.secureconversation.SecureConversationService import \
    SecureConversationServiceWSRF as WSRFService
    

"""SecureConversation service that uses M2Crypto 
"""

# Typecodes
RequestType = GED(WSTRUST.BASE,"RequestType")
ContextAddress = GED(GLOBUS.SECCONV,"ContextAddress")
SecurityContextToken = GED(BEA.SECCONV,"SecurityContextToken")
TokenType = GED(WSTRUST.BASE,"TokenType")
BinaryExchange = GED(WSTRUST.BASE,"BinaryExchange")


# pyclass
#
##TODO: As of ZSI-2.0RC2 Union has no pyclass
#  GED RequestType --> GTD RequestTypeOpenEnum -->
#  union of anyURI and RequestTypeEnum
# 
class RequestTypePyclass(str): typecode = RequestType


# Context Key that will work with Globus Client.
class GSISecureKey(str):
    typecode = ZSI.TC.String(pname=(GLOBUS.CORE,"SecureContextKey"))




class SecureConversationService(WSRFService):

    def wsa_RequestSecurityToken(self, ps, address, **kw):
        request,response = WSRFService.wsa_RequestSecurityToken(self, ps, address, **kw)
        for any in request.Any:
            if hasattr(any, 'typecode') and any.typecode is BinaryExchange:
                break
        else:
            raise RuntimeError, \
                '%s RequestSecurityToken did not receive a BinaryExchange token' %self.__class__

        context = M2SecurityContext()
        ##TODO: Remove Super hack
        context._context = M2Context(initiate=False)
        outTokens = context.acceptSecurityContext(any)
        
        home = M2SecurityContext.getResourceHome()
        #key = home.create()
        key = hashCode()
        home.add(context, key)
    
        #response.set_attribute_Context('uuid:%d' %key)
        response.set_attribute_Context('uuid:%s' %key)


        ##TODO: WSTRUST.ISSUE is wrong??
        #response.Any = RequestTypePyclass(WSTRUST.ISSUE)
        response.Any = RequestTypePyclass("http://schemas.xmlsoap.org/ws/2004/04/security/trust/Issue")
        response.Any.append(TokenType.pyclass(BEA.SCTOKEN))
        
        binEx = BinaryExchange.pyclass(outTokens)
        binEx.set_attribute_EncodingType(OASIS.X509TOKEN.Base64Binary)
        binEx.set_attribute_ValueType(GLOBUS.TOKEN)
        response.Any.append(binEx)
        
        return request,response

    def wsa_RequestSecurityTokenResponse(self, ps, address, **kw):
        request,response = WSRFService.wsa_RequestSecurityTokenResponse(self, ps, address, **kw)
        for any in request.Any:
            if hasattr(any, 'typecode') and any.typecode is BinaryExchange:
                break
        else:
            raise RuntimeError, \
                '%s RequestSecurityTokenResponse did not receive a BinaryExchange token' %(
                    self.__class__)

        tokens = any
        contextID = request.get_attribute_Context()
        response.set_attribute_Context(contextID)
        
        # remove uuid:
        key = contextID.split(':')[1]
        home = M2SecurityContext.getResourceHome()
        context = home.get(key)
        
        outTokens = context.acceptSecurityContext(tokens)
        if context.contextCreated():
            response.Any = RequestTypePyclass("http://schemas.xmlsoap.org/ws/2004/04/security/trust/Issue")
            response.Any.append(TokenType.pyclass(BEA.SCTOKEN))

            sct = SecurityContextToken.pyclass()
            response.Any.append(sct)

            for what in sct.typecode.ofwhat: what.typed = False
            sct.Identifier = sct.new_Identifier(contextID)

            sct.Any = ctxAddr = ContextAddress.pyclass()
            ctxAddr.Address = ctxAddr.new_Address(address._addressTo)
            ctxAddr.ReferenceProperties = ctxAddr.new_ReferenceProperties()
            ctxAddr.ReferenceProperties.Any = GSISecureKey(key)
        else:
            response.Any = binEx = BinaryExchange.pyclass(outTokens)
            binEx.set_attribute_EncodingType(OASIS.X509TOKEN.Base64Binary)
            binEx.set_attribute_ValueType(GLOBUS.TOKEN)
            
        return request,response
    
    
   
from Ft.Lib import Uuid
def hashCode():
    """init context-id
    """
    hcode = Uuid.UuidAsString(Uuid.GenerateUuid())
    print hcode
    tmpList = hcode.split('-')
    tmpList[3] = '0' * 8

    tmpList[4] = '0' * 12
    return("-".join(tmpList)) 
