############################################################################
# Joshua R. Boverhof, LBNL
# See LBNLCopyright for copyright notice!
###########################################################################
import time
from pyGridWare.generated.services.stubs.WidgetService_services_server import *
from pyGridWare.generated.resource.WidgetService_context import *

from pyGridWare.addressing.AddressingUtils import AddressingUtils
from pyGridWare.lifetime.ResourceLifetime import \
    ImmediateResourceTermination, ScheduledResourceTermination


class Widget(WidgetService):

    def wsa_create(self, ps, address, **kw):
        request,response = WidgetService.wsa_create(self, ps, address, **kw)

        # Create Resource Context
        ctx = WidgetContext()
        url = self.getServiceURL()
        ctx.setAddress(url)
        home = ctx.getResourceHome()
        key = home.create()
        home.add(ctx, key)
        ctx.setResourceID(key)

        # Create EndPointReference and set in response
        epr = AddressingUtils.createEndpointReference(ctx)
        response._EndpointReference = epr
        return request,response

    def wsa_destroy(self, ps, address, **kw):
        request,response = WidgetService.wsa_destroy(self, ps, address, **kw)

        # Get Resource Context
        home = WidgetContext.getResourceHome()
        ctx = home.getResourceContext(ps, address)

        # Terminate Resource
        ImmediateResourceTermination.Destroy(ctx)
        return request,response

    def wsa_getMultipleResourceProperties(self, ps, address, **kw):
        request,response = WidgetService.wsa_getMultipleResourceProperties(self, ps, address, **kw)

        # Get Resource Context
        home = WidgetContext.getResourceHome()
        ctx = home.getResourceContext(ps, address)

        namespaceURI,name = request
        property = ctx.properties.getResourceProperty(namespaceURI,name)

        # assign return values to response object
        response._any = property
        return request,response

    def wsa_getResourceProperty(self, ps, address, **kw):
        request,response = WidgetService.wsa_getResourceProperty(self, ps, address, **kw)

        # Get Resource Context
        home = WidgetContext.getResourceHome()
        ctx = home.getResourceContext(ps, address)

        namespaceURI,name = request
        property = ctx.properties.getResourceProperty(namespaceURI,name)
        print "PROPERTY: ", property
        print "\tc: ", property.typecode

        # assign return values to response object
        response._any = property
        return request,response

    def wsa_queryResourceProperties(self, ps, address, **kw):
        request,response = WidgetService.wsa_queryResourceProperties(self, ps, address, **kw)
        return request,response

    def wsa_setResourceProperty(self, ps, address, **kw):
        response = WidgetService.wsa_setResourceProperty(self, ps, address, **kw)
        return response

    def wsa_setTermTime(self, ps, address, **kw):
        response = WidgetService.wsa_setTermTime(self, ps, address, **kw)

        # Get Resource Context
        home = WidgetContext.getResourceHome()
        ctx = home.getResourceContext(ps, address)

        # Schedule Termination
        ScheduledResourceTermination.SetTerminationTime(ctx, request._RequestedTerminationTime)

        # assign return values to response object
        response._NewTerminationTime = time.gmtime(ctx.getTerminationTime())
        response._CurrentTime = time.gmtime(time.time())
        return response



