import os, pydoc, sys
import os.path

from distutils import log
from distutils.command.build_py import build_py
from distutils.util import convert_path

from setuptools import Command
from pyGridWare.utility.generate.Modules import NR
from pyGridWare.utility.generate.Modules import CLIENT, TYPES


def find_packages_modules(where='.'):
    #pack,mod,mod_file 
    """Return a list all Python packages found within directory 'where'
    """
    out = []
    stack=[(convert_path(where), '')]
    while stack:
        where,prefix = stack.pop(0)
        for name in os.listdir(where):
            fn = os.path.join(where,name)
            #if (os.path.isdir(fn) and
            #    os.path.isfile(os.path.join(fn,'__init__.py'))
            #):
            #    out.append(prefix+name); stack.append((fn,prefix+name+'.'))
            if (os.path.isdir(fn) and
                os.path.isfile(os.path.join(fn,'__init__.py'))):
                stack.append((fn,prefix+name+'.'))
                continue

            if name == '__init__.py' or name.endswith('.pyc'): 
                continue

            out.append((prefix, name.split('.py')[0])) 
            
    return out


class PyDocForGeneratedModules(Command):
    """Create pydoc documentation for generated modules.
    """
    srcdir = "pyGridWare/generated"
    destdir = "docs/API"

    user_options = [
        ("srcdir=", "s", "directory containing the source files (default: %s) %srcdir"),
        ("destdir=", "d", "destination output directory (default: %s)" %destdir),
         
    ]

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def _writedoc(self, thing, forceload=0):
        """Write HTML documentation to a file in the current directory.
        """
        try:
            object, name = pydoc.resolve(thing, forceload)
            page = pydoc.html.page(pydoc.describe(object), pydoc.html.document(object, name))
            fname = os.path.join(self.destdir, name + '.html')
            file = open(fname, 'w')
            file.write(page)
            file.close()
            print 'wrote', fname
        except (ImportError, pydoc.ErrorDuringImport), value:
            log.warn(str(value))
        else:
            return name + '.html'
            

    def _writeclientdoc(self, thing, forceload=0):
        """Write HTML documentation to a file in the current directory.
        """
        docmodule = pydoc.HTMLDoc.docmodule
        def strongarm(self, object, name=None, mod=None, *ignored):
            result = docmodule(self, object, name, mod, *ignored)

            # Grab all the aliases to pyclasses and create links.
            nonmembers = []
            push = nonmembers.append
            for k,v in inspect.getmembers(object, inspect.isclass):
                if inspect.getmodule(v) is not object and getattr(v,'typecode',None) is not None:
                    push('<a href="%s.html">%s</a>: pyclass alias<br/>' %(v.__name__,k))

            result += self.bigsection('Aliases', '#ffffff', '#eeaa77', ''.join(nonmembers))
            return result

        pydoc.HTMLDoc.docmodule = strongarm
        try:
            object, name = pydoc.resolve(thing, forceload)
            page = pydoc.html.page(pydoc.describe(object), pydoc.html.document(object, name))
            name = os.path.join(self.destdir, name + '.html')
            file = open(name, 'w')
            file.write(page)
            file.close()
            print 'wroteclient', name
        except (ImportError, pydoc.ErrorDuringImport), value:
            log.warn(str(value))

        pydoc.HTMLDoc.docmodule = docmodule

    def _writetypesdoc(self, thing, forceload=0):
        """Write HTML documentation to a file in the current directory.
        """
        try:
            object, name = pydoc.resolve(thing, forceload)
            name = os.path.join(self.destdir, name + '.html')
        except (ImportError, pydoc.ErrorDuringImport), value:
            log.warn(str(value))
            return
            
        # inner classes
        cdict = {}
        fdict = {}
        elements_dict = {}
        types_dict = {}
        for kname,klass in inspect.getmembers(thing, inspect.isclass):
            if thing is not inspect.getmodule(klass):
                continue
            
            cdict[kname] = inspect.getmembers(klass, inspect.isclass)
            for iname,iklass in cdict[kname]:
                key = (kname,iname)
                fdict[key] = self._writedoc(iklass)
                try:
                    typecode = iklass()
                except TypeError:
                    try:
                        typecode = iklass(None)
                    except (AttributeError,RuntimeError), ex:
                        types_dict[iname] = self._writebrokedoc(ex, iname)
                    else:
                        types_dict[iname] = None
                        if typecode.pyclass is not None:
                            types_dict[iname] = self._writedoc(typecode.pyclass)
                            
                except (AttributeError,RuntimeError), ex:
                    elements_dict[iname] = self._writebrokedoc(ex, iname)
                else:
                    elements_dict[iname] = None
                    if typecode.pyclass is not None:                        
                        elements_dict[iname] = self._writedoc(typecode.pyclass)
                    
        def strongarm(self, object, name=None, mod=None, funcs={}, classes={}, *ignored):
            """Produce HTML documentation for a class object."""
            realname = object.__name__
            name = name or realname
            bases = object.__bases__
            object, name = pydoc.resolve(object, forceload)
            contents = []
            push = contents.append
            if name == realname:
                title = '<a name="%s">class <strong>%s</strong></a>' % (
                    name, realname)
            else:   
                title = '<strong>%s</strong> = <a name="%s">class %s</a>' % (
                    name, name, realname)
                    
            mdict = {}
            if bases:
                parents = []
                for base in bases:
                    parents.append(self.classlink(base, object.__module__))
                title = title + '(%s)' % pydoc.join(parents, ', ')
                
            doc = self.markup(pydoc.getdoc(object), self.preformat, funcs, classes, mdict)
            doc = doc and '<tt>%s<br>&nbsp;</tt>' % doc
            for iname,iclass in cdict[name]:
                fname = fdict[(name,iname)]

                if elements_dict.has_key(iname):
                    push('class <a href="%s">%s</a>: element declaration typecode<br/>'\
                        %(fname,iname))
                    pyclass = elements_dict[iname]
                    if pyclass is not None:
                        push('<ul>instance attributes:')
                        push('<li><a href="%s">pyclass</a>: instances serializable to XML<br/></li>'\
                            %elements_dict[iname])
                        push('</ul>')
                elif types_dict.has_key(iname):
                    push('class <a href="%s">%s</a>: type definition typecode<br/>' %(fname,iname))
                    pyclass = types_dict[iname]
                    if pyclass is not None:
                        push('<ul>instance attributes:')
                        push('<li><a href="%s">pyclass</a>: instances serializable to XML<br/></li>'\
                              %types_dict[iname])
                        push('</ul>')
                else:
                    push('class <a href="%s">%s</a>: TODO not sure what this is<br/>' %(fname,iname))
                    
            contents = ''.join(contents)
            return self.section(title, '#000000', '#ffc8d8', contents, 3, doc) 
        
        doclass = pydoc.HTMLDoc.docclass
        pydoc.HTMLDoc.docclass = strongarm
                
        try:
            page = pydoc.html.page(pydoc.describe(object), pydoc.html.document(object, name))
            file = open(name, 'w')
            file.write(page)
            file.close()
            print 'wrotetypes', name
        except (ImportError, pydoc.ErrorDuringImport), value:
            log.warn(str(value))
            
        pydoc.HTMLDoc.docclass = doclass
        

            
    def _writebrokedoc(self, ex, name, forceload=0):
        try:
            fname = os.path.join(self.destdir, name + '.html')
            page = pydoc.html.page(pydoc.describe(ex), pydoc.html.document(str(ex), fname))
            file = open(fname, 'w')
            file.write(page)
            file.close()
            print 'wrotebroke', fname
        except (ImportError, pydoc.ErrorDuringImport), value:
            log.warn(str(value))
            
        return name + '.html'

    def run(self):
        """if generate is specified generate modules.
        """
        """create pydoc html pages
        """
        ok = True
        if os.path.isdir(self.destdir):
            log.warn("To generate API docs manually Remove directory %s" %self.destdir)
            sys.exit(-1)

        os.mkdir(self.destdir)
        for p in self.distribution.packages:
            self._writedoc(p)

        #build_py = self.distribution.command_obj.get('build_py')
        client_pack,types_pack = NR.Module(CLIENT),NR.Module(TYPES)
        #for pack,mod,mod_file in build_py.find_all_modules():
        #for pack,mod,mod_file in find_all_modules(self.srcdir):
        for pack,mod in find_packages_modules(self.srcdir):
            name = 'pyGridWare.generated.%s%s' %(pack,mod)
            try:
                exec('import ' + name)
                e = eval(name)
            except Exception,ex:
                self._writebrokedoc(ex, name)
                log.warn(str(ex))
                continue
            
            if pack == client_pack:
                self._writeclientdoc(e)
                continue
            
            if pack == types_pack:
                self._writetypesdoc(e)
                continue
       
            try: 
                self._writedoc(e)
            except IndexError,ex:
                self._writebrokedoc(ex, name)
                log.warn(str(ex))
                continue

        return ok


