############################################################################
# Joshua R. Boverhof, LBNL
# See Copyright for copyright notice!
###########################################################################
from Ft.Xml.Domlette import NonvalidatingReaderBase, NonvalidatingReader
#from ZSI.wstools.c14n import Canonicalize
from ZSI.wstools.Namespaces import SCHEMA, XMLNS
#from ZSI.wstools.Utility import DOMException
from ZSI.wstools.Utility import MessageInterface

from elementtree import ElementTree
#from lxml import etree as ElementTree

class DomletteReader(NonvalidatingReaderBase):
    '''Used with ZSI.parse.ParsedSoap
    '''
    fromString = NonvalidatingReaderBase.parseString
    fromStream = NonvalidatingReaderBase.parseStream

class ElementProxyException(Exception):
    '''TODO: Should probably rename DOMException ElementProxyException.
    '''

class ElementTreeProxy(MessageInterface):
    '''ElementTree wrapper
    TODO: add c14n for message signatures, issue with "getPrefix"
    '''

    def __init__(self, sw, message=None):
        '''Initialize. 
           sw -- SoapWriter
        '''
        assert message is None, 'TODO'
        self._etree = None

    def __str__(self):
        return self.toString()
        
    def toString(self):
        return ElementTree.tostring(self._etree)

    #############################################
    #Methods used in TypeCodes
    #############################################
    def createAppendElement(self, namespaceURI, localName, prefix=None):
        '''Create a new element (namespaceURI,name), append it
           to current node, and return the newly created node.
        Keyword arguments:
            namespaceURI -- namespace of element to create
            localName -- local name of new element
            prefix -- if namespaceURI is not defined, declare prefix.  defaults
                to 'ns1' if left unspecified.
        '''
        assert prefix is None, 'TODO: can really do this'
        e = ElementTree.Element("{%s}%s" %(namespaceURI, localName))
        self._etree.append(e)
        
        eproxy = ElementTreeProxy(None)
        eproxy._etree = e
        return eproxy

    def createAppendTextNode(self, pyobj):
        '''TODO: obviously mixed text content cannot be accurately
        represented via this interface.  Only 1 possible text node/element
        '''
        self._etree.text = pyobj

    def createDocument(self, namespaceURI, localName):
        self._etree = ElementTree.Element('{%s}%s' %(namespaceURI, localName))
    
    def getElement(self, namespaceURI, localName):
        for e in self._etree.getchildren():
            l = e.tag.strip('{').split('}')
            if not namespaceURI:
                if len(l) == 1 and l[0] == localName:
                    eproxy = ElementTreeProxy(None)
                    eproxy._etree = e
                    return eproxy
            elif len(l) == 2 and l[0] == namespaceURI and l[1] == localName:
                eproxy = ElementTreeProxy(None)
                eproxy._etree = e
                return eproxy
                
        raise ElementProxyException,\
            'No such element(%s,%s)' %(namespaceURI,localName)
        
    def getPrefix(self, namespaceURI):
        '''TODO: this is not possible w/elementTree since namespace prefix
        mappings aren't done until serialization.  completely abstraced out.
        '''
        raise NotImplementedError, 'this func isnt going to work'
        
    def setAttributeNS(self, namespaceURI, localName, value):
        '''
        Keyword arguments:
            namespaceURI -- namespace of attribute to create, None is for
                attributes in no namespace.
            localName -- local name of new attribute
            value -- value of new attribute
        ''' 
        self._etree.attrib["{%s}%s" %(namespaceURI, localName)] = value
        
    def setAttributeType(self, namespaceURI, localName):
        '''xsi:type attribute, value must be a QName
        '''
        self.setAttributeNS(SCHEMA.XSI3, 'type', 
            ElementTree.QName('{%s}%s' %(namespaceURI,localName))
        )
        
    def setNamespaceAttribute(self, prefix, namespaceURI):
        '''TODO: Not sure how to force this to be used by ElementTree
        Keyword arguments:
            prefix -- xmlns prefix
            namespaceURI -- value of prefix
        '''
        self._etree.attrib["xmlns:%s" %prefix] = namespaceURI
