##########################################################################
# Joshua R. Boverhof, LBNL
# See Copyright for copyright notice!
###########################################################################
import string, warnings
from cStringIO import StringIO

from ZSI.wstools.WSDLTools import SoapBinding
from ZSI.wstools.Namespaces import WSRF_V1_2, WSA200403
from ZSI.generate.wsdl2dispatch import WSAServiceModuleWriter
from ZSI.generate.wsdl2python import WriteServiceModule, \
    ServiceOperationsClassContainer, ServiceOperationContainer
from ZSI.generate.wsdl2python import ServiceDescription as ClientServiceDescription
from ZSI.generate.containers import ID1, ID2, ID3, ID4, ID5, \
    SimpleTypeContainer, TypecodeContainerBase, TypesHeaderContainer
from ZSI.generate.utility import NamespaceAliasDict as NAD
from ZSI.generate import WsdlGeneratorError

from Utility import HEADER, CLIENT, SERVICE, SERVICE_STUBS, TYPES,\
     PROPERTIES, RESOURCE
from Utility import NameResolver, GetResourcePropertiesFromPort,\
    GetPropertiesBasesFromRP, GetContextClassBasesFromPort

NR = NameResolver

"""
Functions
    WriteTypesModule
    WriteClientStubModule
    WriteServiceModules
    WriteServiceStubModule

    _CreatePropertiesModule
    _CreateContextModule

"""
try:
    TypesHeaderContainer.imports.append('import pyGridWare.patches')
except AttributeError, ex:
    warnings.warn('Update ZSI')


def WriteTypesModule(section, wsdl):
    '''Create/Write types module.
    '''
    wsm = WriteServiceModule(wsdl, addressing=True)
    file_name = NR.GetTypesModuleFileName(section)
    fd = open(file_name, 'w+')
    wsm.writeTypes(fd)
    fd.close()


def WriteClientStubModule(section, wsdl):
    '''Create/Write Client Skeleton module.
    '''
    wsm = WriteServiceModule(wsdl, addressing=True)
    wsm.setTypesModulePath(NR.Module(TYPES))
    wsm.setTypesModuleName(NR.GetTypesModuleName(section))
    wsm.setClientModulePath(NR.Module(CLIENT))
    wsm.gatherNamespaces()

    file_name = NR.GetTypesModuleFileName(section)
    fd = open(file_name, 'w+')
    wsm.writeTypes(fd)
    fd.close()

    kw = {}
    try:
        import Ft
    except ImportError, ex:
        warnings.warn('4Suite not found.  No XML Signatures, Slower performance.')
    else:
        kw['readerclass'] = 'DomletteReader'
        kw['writerclass'] = 'DomletteElementProxy'

    file_name = NR.GetClientModuleFileName(section)
    fd = open(file_name, 'w+')

    class MessageWriter:
        def __init__(self):
            self.s = StringIO()
 
        def __str__(self):
            return self.s.getvalue()

        def setUp(self, soc, port, input=True):
            operation = port.getBinding().getPortType().operations.get(soc.getOperationName())
            if input is True:
                message = operation.getInputMessage()
            else:
                message = operation.getOutputMessage()

            parts = message.parts.data.values()
            assert len(parts) == 1,\
                'doc/lit message(%s) must have one part found(%d)' %(message.name, len(parts))
            assert parts[0].element is not None,\
                'expecting part element attribute for message(%s)' %message.name

            func_mangle = soc.mangle
            #func_alias = NAD.getAlias
            msg_name = func_mangle(message.name)
            #prefix = func_alias(parts[0].element[0])
            nspname,pname = parts[0].element
            #tcname = '%s_Dec' %func_mangle(pname)
            #print >>self.s, '%s = %s.%s().pyclass' %(msg_name, prefix, tcname)
            print >>self.s, '%s = GED("%s","%s").pyclass' %(msg_name, nspname, pname)

    class TwistedServiceOperationContainer(ServiceOperationContainer):
        #
        # XXX
        # MUST override a big method to do simple things.  
        #
        def _setContent(self):
            '''create string representation of operation.
            '''
            kwstring = 'kw = {}'
            tCheck = 'if isinstance(request, %s) is False:' % self.inputName
            bindArgs = ''
            if self.encodingStyle is not None:
                bindArgs = 'encoding="%s", ' %self.encodingStyle
    
            if self.useWSA:
                wsactionIn = 'wsaction = "%s"' % self.inputAction
                wsactionOut = 'wsaction = "%s"' % self.outputAction
                bindArgs += 'wsaction=wsaction, endPointReference=self.endPointReference, '
                responseArgs = ', wsaction=wsaction'
            else:
                wsactionIn = '# no input wsaction'
                wsactionOut = '# no output wsaction'
                responseArgs = ''

            bindArgs += '**kw)'

            method = [
                '%sdef %s(self, request):' % (ID1, self.name),
                '%s"""input parameters for operation %s' % (ID2, self.name),
                '%s       request -- %s' %(ID2, self.inputName), 
                '%s   return -- %s' %(ID2, self.outputName), 
                '%s"""' % (ID2),
                '%s%s' % (ID2, tCheck),
                '%sraise TypeError, "%%s incorrect request type" %% (%s)' %(ID3, 'request.__class__'),
                '%s%s' % (ID2, kwstring),
                '%s%s' % (ID2, wsactionIn),
                '%sd = self.binding.Send(None, None, request, soapaction="%s", %s'\
                %(ID2, self.soapaction, bindArgs),
            ]
            if self.outputName:
                response = ['%s%s' % (ID2, wsactionOut),]
                # XXX add some code to return deferred
                response.append('%sif self.defer is True:' %ID2)
                response.append('%sd.addCallback(self.binding.getResponseSOAPMessage,' %ID3)
                response.append('%sreplytype=%s.typecode, address=d.address,' %(ID4,self.outputName))
                response.append('%swsaction=wsaction)' %ID4)
                if self.outputSimpleType:
                    response.append('%sd.addCallback(lambda response: %s(response))' %(ID3, self.outputName))
                response.append('%sreturn d' %ID3)
                # XXX 

                response.append('%sresponse = self.binding.Receive(%s.typecode%s)' %(ID2, self.outputName, responseArgs))
                if self.outputSimpleType:
                    response.append('%sreturn %s(response)' %(ID2, self.outputName))
                else:
                    rCheck = 'if isinstance(response, %s.typecode.pyclass) is False:' %self.outputName

                    # XXX added line
                    response.append('%s%s' %(ID2, rCheck))
                    response.append('%sraise TypeError, "%%s incorrect response type" %% (%s)' %(ID3, 'response.__class__'))

                    if self.do_extended:
                        partsList = self.getOperation().getOutputMessage().parts.values()
                        subNames = GetPartsSubNames(partsList, self._wsdl)
                        args = []
                        for pa in subNames:
                            args += pa

                        for arg in args:
                            response.append('%s%s = response.%s' % (ID2, self.mangle(arg), self.getAttributeName(arg)) )
                        margs = ",".join(args)
                        response.append("%sreturn %s" % (ID2, margs) )
                    else:
                        response.append('%sreturn response' %ID2)
                method += response

            self.writeArray(method)

            

    class TwistedServiceOperationsClassContainer(ServiceOperationsClassContainer):
        #
        # XXX MUST Subclass/Overide _setContent to add a single line. 
        #
        operationclass = TwistedServiceOperationContainer
        
        def _setContent(self):
            if self.useWSA is True:
                ctorArgs = 'endPointReference=None, **kw'
                epr      = 'self.endPointReference = endPointReference'
            else:
                ctorArgs = '**kw'
                epr      = '# no ws-addressing'

            if self.rProp:
                rprop = 'kw.setdefault("ResourceProperties", ("%s","%s"))'\
                    %(self.rProp[0], self.rProp[1])
            else:
                rprop = '# no resource properties'

            methods = [\
                '# Methods',
                'class %s%s:' % (self.bName, self.clientClassSuffix),
                '%sdefer = False' %ID1,
                '%sdef __init__(self, url, %s):' % (ID1, ctorArgs),
                '%skw.setdefault("readerclass", %s)' % (ID2, self.readerclass),
                '%skw.setdefault("writerclass", %s)' % (ID2, self.writerclass),
                '%skw.setdefault("wsAddressingURL", "%s")' % (ID2, WSA200403.ADDRESS),
                '%s%s' % (ID2, rprop),
                '%sself.binding = client.Binding(url=url, **kw)' %ID2,
                '%s%s' % (ID2,epr),
                ]

            for op in self.operations:
                methods += [ op.getvalue() ]

            self.writeArray(methods)
            

    class SD(ClientServiceDescription):
        def __init__(self, addressing=False, **kw):
            ClientServiceDescription.__init__(self, addressing, **kw)
            self.imports.basic = [
                '\nimport urlparse, types',
                'import ZSI',
                'from ZSI.TC import _get_global_element_declaration as GED',
                'from ZSI.TCcompound import Struct',
                'from pyGridWare.utility.web import client',
                'from pyGridWare.utility.dom import DomletteReader, DomletteElementProxy',
            ]
        def fromWsdl(self, service, **kw):
            self.imports.setTypesModuleName(self.typesModuleName)
            self.imports.appendImport(kw.get('imports', []))

            self.locator.setUp(service)

            for port in service.ports:
                sop_container = TwistedServiceOperationsClassContainer(useWSA=self.wsAddressing)
                try:
                    sop_container.setUp(port)
                except Wsdl2PythonError, ex:
                    self.logger.warning('Skipping port(%s)' %port.name)
                    if len(ex.args):
                        self.logger.warning(ex.args[0])
                else:
                    sop_container.setReaderClass(kw.get('readerclass'))
                    sop_container.setWriterClass(kw.get('writerclass'))
                    for soc in sop_container.operations:
                        if soc.hasInput() is True:
                            mw = MessageWriter()
                            mw.setUp(soc, port, input=True)
                            self.messages.append(mw)

                            if soc.hasOutput() is True:
                                mw = MessageWriter()
                                mw.setUp(soc, port, input=False)
                                self.messages.append(mw)

                    self.methods.append(sop_container)


    wsm.writeClient(fd, SD, **kw)
    fd.close()


def WriteServiceModules(section, wsdl, packages=None, simple=True, web2=False):
    '''service module combines all generated modules together
    and provides defined WSRF functionality.
    
    KeyWord Args:
        packages --
        simple -- use SimpleResourceHome, else use IPersistent plugin.
        web2 -- use twisted.web2 
    '''
    imports = []
    lifetime_import = 'pyGridWare.wsrf.lifetime.ResourceLifetime'
    if packages is not None: NR.packages = packages

    for service in wsdl.services:
        imports = []
        args = WriteServiceStubModule(section, service, simple=simple, web2=web2)
        if args is None: 
            continue
        ss, ssmod, pmmod, rpmod = args
        kw = {\
            'id0':ss.getIndent(1), 
            'id1':ss.getIndent(2), 
            'id2':ss.getIndent(3), 
            'id3':ss.getIndent(4),
        }
        importfrom = {ssmod:['*'],}
        kw['base_class'] = ss.getClassName(service.name)
        kw['service_class'] = '%(base_class)sWSRF' %kw
        
        sc = StringIO()
        print >>sc, ''
        print >>sc, ''
        print >>sc, 'class %(service_class)s(%(base_class)s):' %kw
        print >>sc, ''
        print >>sc, '%(id0)sdef GetResourceContext(ps, address):' %kw
        print >>sc, '%(id1)s"""get a resource context"""' %kw
        print >>sc, '%(id1)sreturn ManagerHome.getInstance().getResourceContext(ps, address)' %kw
        print >>sc, '%(id0)sGetResourceContext = staticmethod(GetResourceContext)' %kw

        soap_ports = \
            filter(\
                lambda p: p.getBinding().findBinding(SoapBinding) is not None,
                service.ports
            )
        if rpmod is not None: importfrom[rpmod] = ['ManagerHome']
        for port in soap_ports:
            rp = port.getPortType().getResourceProperties()
            if rp is None:
                continue

            kw['context_class'] = NR.GetContextClassNameFromPort(port)
            kw['port'] = port.name
            if rpmod is not None: importfrom[rpmod].append(kw['context_class'])
            print >>sc, ''
            print >>sc, '%(id0)sdef NewResourceContext%(port)s(cls, url):' %kw
            print >>sc, '%(id1)s"""create a new resource context"""' %kw
            print >>sc, '%(id1)sctx = %(context_class)s()' %kw
            print >>sc, '%(id1)sctx.setAddress(url)' %kw
            print >>sc, '%(id1)shome = ctx.getResourceHome()' %kw
            print >>sc, '%(id1)skey = home.create()' %kw
            print >>sc, '%(id1)shome.add(ctx, key)' %kw
            print >>sc, '%(id1)sctx.setResourceID(key)' %kw
            print >>sc, '%(id1)sreturn ctx' %kw
            print >>sc, '%(id0)sNewResourceContext%(port)s = classmethod(NewResourceContext%(port)s)' %kw

        for op in [op for p in soap_ports for op in p.getPortType().operations]:
            kw['method'] = ss.getMethodName(op.name)
            inmsg = getattr(op.input, 'message', None)
            outmsg = getattr(op.output, 'message', None)
            if inmsg is None: 
                continue

            print >>sc, ''
            print >>sc, '%(id0)sdef %(method)s(self, ps, address, **kw):' %kw
            nsuri,name = inmsg
            print >>sc, '%(id1)s' %kw + '#%s %s' %(nsuri,name)

            #if outmsg is None:
            #    print >>sc, '%(id1)srequest,response = %(base_class)s.%(method)s(self, ps, address)' %kw
            #else:
            print >>sc, '%(id1)srequest,response = %(base_class)s.%(method)s(self, ps, address)' %kw
            
            # Look for WSRF Messages
            if nsuri in WSRF_V1_2.LIFETIME.WSDL_LIST:
                importfrom[lifetime_import] = ('ImmediateResourceTermination', 'ScheduledResourceTermination')
                print>>sc, '%(id1)sctx = %(service_class)s.GetResourceContext(ps, address)' %kw
                if name == 'DestroyRequest':
                    print>>sc, '%(id1)sImmediateResourceTermination.Destroy(ctx)' %kw
                elif name == 'SetTerminationTimeRequest':
                    imports.append('time')
                    print>>sc, '%(id1)sScheduledResourceTermination.SetTerminationTime(ctx, request._RequestedTerminationTime)' %kw
                    print>>sc, '%(id1)sresponse._NewTerminationTime = time.gmtime(ctx.TerminationTime)' %kw
                    print>>sc, '%(id1)sresponse._CurrentTime = time.gmtime(ctx.CurrentTime)' %kw
                else: 
                    warnings.warn('unknown lifetime message (%s,%s)' %(nsuri,name))
            elif nsuri in WSRF_V1_2.BASENOTIFICATION.WSDL_LIST:
                importfrom['pyGridWare.wsrf.notification.SubscriptionManager'] = ('SubscriptionManager',)
                
                # NotificationConsumer Notify operation not included.  SubscriptionManager is 
                # on the client-side w/respect to the NotificationConsumer.
                if name != 'Notify':
                    print>>sc, '%(id1)sctx = self.GetResourceContext(ps, address)' %kw
                    print>>sc, '%(id1)sassert SubscriptionManager._isInstantiated(),"SubscriptionManager is not instantiated"' %kw
                    print>>sc, '%(id1)smanager = SubscriptionManager.getInstance()' %kw
                
                if name == 'SubscribeRequest':
                    print>>sc, '%(id1)sresponse._SubscriptionReference = manager.subscribe(request, ctx)' %kw
                elif name == 'GetCurrentMessageRequest':
                    print>>sc, '%(id1)sresponse._any = manager.current(request, ctx)' %kw
                else: 
                    warnings.warn('unknown base notification message (%s,%s)' %(nsuri,name))
        
            elif nsuri in WSRF_V1_2.PROPERTIES.WSDL_LIST:
                print>>sc, '%(id1)sctx = ManagerHome.getInstance().getResourceContext(ps, address)' %kw
                if name == 'GetResourcePropertyDocumentRequest':
                    print>>sc, '%(id1)snamespaceURI,element = request' %kw
                    print>>sc, '%(id1)sdocument = ctx.properties.getResourcePropertiesDocument()' %kw
                    print>>sc, '%(id1)sresponse._any = document' %kw
                elif name == 'GetResourcePropertyRequest':
                    print>>sc, '%(id1)snamespaceURI,element = request' %kw
                    print>>sc, '%(id1)sproperty = ctx.properties.getResourceProperty(namespaceURI,element)' %kw
                    print>>sc, '%(id1)sresponse._any = [property]' %kw
                elif name == 'GetMultipleResourcePropertiesRequest':
                    print>>sc, '%(id1)sresponse._any = []' %kw
                    print>>sc, '%(id1)sfor nsuri,name in request._ResourceProperty:' %kw
                    print>>sc, '%(id2)sresponse._any.append(ctx.properties.getResourceProperty(nsuri, name))' %kw
                elif name == 'SetResourcePropertiesRequest':
                    print>>sc, '%(id1)sif request._Insert is not None:' %kw
                    print>>sc, '%(id2)sraise NotImplemetedError, "SetResourceProperties insert not implemented"' %kw
                    print>>sc, '%(id1)selif request._Update is not None:' %kw
                    print>>sc, '%(id2)sfor any in request._Update._any:' %kw
                    print>>sc, '%(id3)snspname,pname = any.typecode.nspname,any.typecode.pname' %kw
                    print>>sc, '%(id3)sctx.properties.setResourceProperty(nspname, pname, any)' %kw
                    print>>sc, '%(id1)selif request._Delete is not None:' %kw
                    print>>sc, '%(id2)sraise NotImplemetedError, "SetResourceProperties delete not implemented"' %kw
                    print>>sc, '%(id1)selse:' %kw
                    print>>sc, '%(id2)sraise RuntimeError, "unknown request for SetResourceProperties."' %kw
    
                elif name == 'InsertResourcePropertiesRequest':
                    pass
                elif name == 'UpdateResourcePropertiesRequest':
                    pass
                elif name == 'DeleteResourcePropertiesRequest':
                    pass
                elif name == 'QueryResourcePropertiesRequest':
                    print>>sc, '%(id1)sresponse._any = ctx.properties.query(request._QueryExpression, response)' %kw
                else: 
                    warnings.warn('unknown resource properties message (%s,%s)' %(nsuri,name))

            else:
                pass
            
            print>>sc, '%(id1)sreturn request,response' %kw

        s = StringIO()
        print >>s, HEADER
        s.writelines(['import %s\n' %i for i in imports])
        s.writelines(['from %s import %s\n' %(k,vv) for k,v in importfrom.items() for vv in v])
        print >>s, sc.getvalue()

        fd = open(NR.GetServiceModuleFileName(section, service), 'w+')
        fd.write(s.getvalue())
        fd.close()
        sc.close()
        s.close()



def WriteServiceStubModule(section, service, simple=True, **kw):
    '''Create/Write Service Skeleton module.
    
    KeyWord Args:
        simple -- Use simple Resource Home, else use Persistent plugin.
    '''
    wsdl = service.getWSDL()
    ssmod = pmmod = rpmod = None

    class GridServiceDescription(WSAServiceModuleWriter):
        '''Specify how each service skeleton should be created.
        '''
        def __init__(self, base='pyGridWare.utility.web.resource.WSResource', prefix='wsa', strict=False):
            WSAServiceModuleWriter.__init__(self, base=base, prefix=prefix, strict=strict)
            self.setClientModulePath(NR.Module(CLIENT))
            
        def setUpInitDef(self, service):
            '''set __init__ function
            '''
        def createMethodBody(msgInName, msgOutName, **kw):
            '''return a tuple of strings containing the body of a method.
            msgInName -- None or a str
            msgOutName --  None or a str
            '''
            body = []
            request = response = None
            if msgOutName is not None:
                response = '%s()' %msgOutName
            if msgInName is not None:
                body.append('request = ps.Parse(%s.typecode)' %msgInName)
                request = 'request'
                
            body.append('return %(request)s, %(response)s'\
                         %{'request':request, 'response':response}
            )
            
            return tuple(body)
        createMethodBody = staticmethod(createMethodBody)
            
            
    if kw.has_key('web2'):
        if kw['web2']: kw['base'] = 'pyGridWare.utility.web2.resource.WSResource'
        del kw['web2']
 
    ss = GridServiceDescription(**kw)
    ss.setClientModuleName(NR.GetClientModuleName(section))
    try:
        ss.fromWSDL(wsdl)
    except WsdlGeneratorError, ex:
        warnings.warn('%s -- skip server stub generation(%s)' %(section, string.join(ex.args)))
        return

    fname =  NR.GetServiceStubModuleFileName(section, service)
    ssmod = NR.GetModFromFile(fname)
    fd = open(fname, 'w+')
    ss.write(fd)
    fd.close()

    # Create Properties template
    # Assumption of 1service/1port
    if len(wsdl.services) > 1:
        warnings.warn('number of services(%d) may cause problems..' %len(wsdl.services))

    if len(wsdl.services) == 0:
        warnings.warn('WSDL Definition(%s) has no service defined ..' %len(wsdl.name))
        return

    kw = {}
    ged = None
    bases = []

    if len(service.ports) != 1: 
        warnings.warn('number of ports(%d) may cause problems..' %len(service.ports))

    skip = True 
    for p in service.ports:
        els = GetResourcePropertiesFromPort(p)
        if els is None:
            continue 
        skip = False
        fname = NR.GetPropertiesModuleFileName(section, p)
        fd = open(fname, 'w+')
        fd.write(\
            _CreatePropertiesModule(section, p)
        )
        fd.close()

        pmmod = NR.GetModFromFile(fname)

    if skip is False:
        fname = NR.GetContextModuleFileName(section, service)
        rpmod = NR.GetModFromFile(fname)
        fd = open(fname, 'w+')
        fd.write(\
            _CreateContextModule(section, service, simple=simple)
        )
        fd.close()

    return ss, ssmod, pmmod, rpmod


def _CreateContextModule(section, service, simple=True):
    '''
    KeyWord Args:
        simple -- use SimpleResourceHome.
    '''
    wsdl = service.getWSDL()
    kw = {\
        'id1':' '*4,
        'id2':' '*8,
        'targetNamespace':wsdl.targetNamespace, 
        'Key':'%sKey' %wsdl.name or '',
        'bases':'',
        'init':'',
    }

    h = StringIO()
    print >>h, HEADER
    print >>h, 'import ZSI'
    print >>h, 'from pyGridWare.patterns import Singleton'
    print >>h, 'from pyGridWare.resource.ResourceHomeManager import ResourceHomeManager'
    print >>h, 'from pyGridWare.addressing.AddressingUtils import AddressingUtils'
    print >>h, 'from pyGridWare.resource.ResourceContext import ResourceContext'
    
    if simple is False:
        print >>h, 'from pyGridWare.plugin import getPlugIn'
        print >>h, 'from pyGridWare.resource.interfaces import IPersistent'
        kw['resource_home'] = 'getPlugIn(IPersistent)()'
    else:  
        print >>h, 'from pyGridWare.resource.ResourceHome import SimpleResourceHome'
        kw['resource_home'] = 'SimpleResourceHome()'

    s = StringIO()
    print >>s, ''
    print >>s, ''
    print >>s, 'class Key(int):'
    print >>s, '%(id1)stypecode = ZSI.TC.Iint(pname=(\'%(targetNamespace)s\',\'%(Key)s\'))' %kw
    print >>s, ''
    print >>s, ''
    print >>s, 'class ManagerHome(ResourceHomeManager, Singleton):'
    print >>s, '%(id1)sdef getResourceContext(self, ps, address):' %kw
    print >>s, '%(id2)skey = AddressingUtils.getResourceID(ps, Key.typecode)' %kw
    print >>s, '%(id2)sreturn self.get(key)' %kw

    # should check only soap ports
    for port in service.ports:
        rp = port.getPortType().getResourceProperties()
        if rp is None:
            continue

        kw['init'] = ''
        kw['bases'] = ''
        kw['base'] = kw['class'] = kw['module'] = None
        for b in GetContextClassBasesFromPort(port):
            kw['base'] = b.alias
            kw['class'] = b.name
            kw['module'] = b.module
            print >>h, 'from %(module)s import %(class)s as %(base)s' %kw
            kw['init'] += '%(id2)s%(base)s.__init__(self, properties_class)\n' %kw
            kw['bases'] += '%s, ' %b.alias
        del kw['base'],kw['class'],kw['module']

        kw['context_class'] = NR.GetContextClassNameFromPort(port)
        kw['prop_class'] = NR.GetPropertiesClassNameFromPort(port)
        kw['module'] = NR.GetModFromFile(NR.GetPropertiesModuleFileName(section, port))

        print >>h, 'from %(module)s import %(prop_class)s' %kw

        print >>s, ''
        print >>s, ''
        print >>s, 'class %(context_class)s(%(bases)sResourceContext):' %kw
        print >>s, '%(id1)sdef __init__(self, properties_class=%(prop_class)s):' %kw
        print >>s, '%(id2)sResourceContext.__init__(self, properties_class)' %kw
        print >>s, '%(init)s' %kw
        print >>s, '%(id1)sdef setResourceID(self, resourceID):' %kw
        print >>s, '%(id2)sResourceContext.setResourceID(self, Key(resourceID))' %kw
        print >>s, '%(context_class)s.home = ManagerHome.getInstance(%(resource_home)s)' %kw
        # Singleton getInstance called ONCE w/args.
        kw['resource_home'] = ''

    print >>h, s.getvalue()
    return h.getvalue()


def _CreatePropertiesModule(section, port):
    '''
    section -- 
    port --
    '''
    kw = {'id1':' '*4, 'id2':' '*8, }
    wsdl = port.getWSDL() 
    s_top = StringIO()
    s_top.write(HEADER)
    s_top.write('\nimport ZSI.TC\n')
    s_top.write('\nfrom ZSI.TC import _get_global_element_declaration as GED')
    #s_top.write('\nfrom pyGridWare.properties.ResourceProperties import *')
    s_top.write('\nimport %s\n' %(NR.Module(TYPES, NR.GetTypesModuleName(section))))
    s_top.write('\nfrom pyGridWare.wsrf.properties.ResourceProperties import ResourceProperties')

    s_tail = StringIO()
    pt = port.getPortType()
    kw['class_name'] = NR.GetPropertiesClassNameFromPort(port)
    rp = pt.getResourceProperties()
    if rp is None: 
        return

    nsuri,name = rp
    kw['targetNamespace'],kw['name'] = nsuri,name
    ged = wsdl.types[nsuri].elements[name]

    basestr, initstr = '', ''
    kw['base'] = kw['module'] = kw['class'] = None
    for b in GetPropertiesBasesFromRP(ged):
        kw['base'] = b.alias
        kw['module'] = b.module
        kw['class'] = b.name
        s_top.write('\nfrom %(module)s import %(class)s as %(base)s' %kw)
        basestr += '%(base)s,' %kw
        initstr += '\n%(id2)s%(base)s.__init__(self)' %kw
    del kw['base'],kw['module'],kw['class']

    kw['bases'] = basestr
    s_body = StringIO()
    s_body.write('\nclass %(class_name)s(%(bases)sResourceProperties):' %kw)
    s_body.write('\n%(id1)sdeclaration = GED("%(targetNamespace)s","%(name)s")\n' %kw)
    s_body.write('\n%(id1)sdef __init__(self):' %kw)
    s_body.write('\n%(id2)sResourceProperties.__init__(self)' %kw)
    s_body.write(initstr + '\n')

    # Create property for each element reference
    type_def = ged.getTypeDefinition()
    model_group = type_def.content
    extra_els = map(lambda er: er.getElementDeclaration(), model_group.content)
    for e in extra_els:
        kw['targetNamespace'],kw['name'] = e.getTargetNamespace(),e.getAttribute('name')
        s_body.write('\n%(id1)sdef get%(name)s(self):' %kw)
        s_body.write('\n%(id2)sreturn self.getResourceProperty("%(targetNamespace)s","%(name)s")' %kw)
        s_body.write('\n%(id1)sdef set%(name)s(self, pyobj):' %kw)
        s_body.write('\n%(id2)sreturn self.setResourceProperty("%(targetNamespace)s","%(name)s", pyobj)' %kw)
        s_body.write('\n%(id1)s%(name)s = property(get%(name)s, set%(name)s, None, "RP element %(name)s")\n' %kw)

    s_top.write(s_body.getvalue())
    return s_top.getvalue()


