###########################################################################
# Monte M. Goode, LBNL
# Joshua R. Boverhof, LBNL
# See Copyright for copyright notice!
###########################################################################
from twisted.web import http


def GetURLFromRequest(request):
    if isinstance(request, http.Request) is False:
        raise TypeError, '%s instance expected' %http.Request
    if request.transport.TLS == 0:
        localURL = 'http://%s:%d%s'\
            %(request.host.host, request.host.port, request.path)
    else:
        localURL = 'https://%s:%d%s'\
            %(request.host.host, request.host.port, request.path)
    return localURL
    
    
class ChunkedHTTPChannel(http.HTTPChannel):

    chunked = False

    maxHeaders = 500 # max number of headers allowed per request
    
    length = 0
    persistent = 1
    __header = ''
    __first_line = 1
    __content = None

    # set in instances or subclasses
    requestFactory = http.Request

    _savedTimeOut = None

    def __init__(self):
        http.HTTPChannel.__init__(self)

        self.chunks = []

    def lineReceived(self, line):
        self.resetTimeout()

        if self.__first_line:
            # if this connection is not persistent, drop any data which
            # the client (illegally) sent after the last request.
            if not self.persistent:
                self.dataReceived = self.lineReceived = lambda *args: None
                return

            # IE sends an extraneous empty line (\r\n) after a POST request;
            # eat up such a line, but only ONCE
            if not line and self.__first_line == 1:
                self.__first_line = 2
                return

            # create a new Request object
            request = self.requestFactory(self, len(self.requests))
            self.requests.append(request)
            
            self.__first_line = 0
            parts = line.split()
            if len(parts) != 3:
                self.transport.write("HTTP/1.1 400 Bad Request\r\n\r\n")
                self.transport.loseConnection()
                return
            command, request, version = parts
            self._command = command
            self._path = request
            self._version = version
        elif line == '':
            if self.__header:
                self.headerReceived(self.__header)
            self.__header = ''
            self.allHeadersReceived()
            if self.length == 0 and self.chunked == False:
                self.allContentReceived()
            else:
                self.setRawMode()
        elif line[0] in ' \t':
            self.__header = self.__header+'\n'+line
        else:
            if self.__header:
                self.headerReceived(self.__header)
            self.__header = line

    def allContentReceived(self):
        self.chunked = False
        while self.chunks: self.chunks.pop()
        http.HTTPChannel.allContentReceived(self)

    def headerReceived(self, line):
        """Do pre-processing (for content-length) and store this header away.
        """
        header, data = line.split(':', 1)
        header = header.lower()
        data = data.strip()
        if header == 'content-length':
            self.length = int(data)
        if header == 'transfer-encoding' and data == 'chunked':
            self.chunked = True
        reqHeaders = self.requests[-1].received_headers
        reqHeaders[header] = data
        if len(reqHeaders) > self.maxHeaders:
            self.transport.write("HTTP/1.1 400 Bad Request\r\n\r\n")
            self.transport.loseConnection()

    def rawDataReceived(self, data):
        if self.chunked:
            self.processChunkedData(data)
        else:
            http.HTTPChannel.rawDataReceived(self, data)

    def processChunkedData(self, data):

        self.chunks.append(data)
        
        # XXX: The following line needs to be watched
        if ''.join(self.chunks).find('\r\n0\r\n') > -1:
            buffer  = ''
            for c in ''.join(self.chunks):
                buffer += c
                try:
                    data, buffer = http.fromChunk(buffer)
                    self.requests[-1].handleContentChunk(data)
                except ValueError:
                    pass
                
            self.allContentReceived()

