###########################################################################
# Monte M. Goode, LBNL
# Joshua R. Boverhof, LBNL
# See Copyright for copyright notice!
###########################################################################

# $Id: resource.py,v 1.8 2006/03/31 06:26:58 boverhof Exp $

import os,sys,warnings
from cStringIO import StringIO
from twisted import copyright
TWISTED_VERSION = (2,0,0)
try:
    if tuple(map(lambda i:int(i), copyright.version.split('.'))) < TWISTED_VERSION:
        warnings.warn(\
            'Twisted-%s or later is required, found: %s' %(TWISTED_VERSION, copyright.version),
            RuntimeWarning,
        )
except:
    warnings.warn(\
        'Twisted-%s or later is required, found: %s' %(TWISTED_VERSION, copyright.version),
        RuntimeWarning,
    )

from zope.interface import classProvides, implements, Interface
from twisted.web.error import NoResource
from twisted.web.server import NOT_DONE_YET
from twisted.web.resource import Resource
from twisted.python import log, failure

from ZSI import _get_element_nsuri_name, EvaluateException, ParseException
from ZSI.parse import ParsedSoap
from ZSI.writer import SoapWriter
from ZSI import fault
from ZSI.address import Address
from ZSI.ServiceContainer import WSActionException, WSActionNotSpecified
from ZSI.twisted.WSresource import WSResource as _WSResource,\
    CallbackChainInterface, DataHandler, DefaultHandlerChain, WSAddressHandler,\
    HandlerChainInterface

#from pyGridWare.security.interfaces import IWSSecurity
from pyGridWare.utility.http import GetURLFromRequest
try:
    from pyGridWare.utility.dom import DomletteReader, DomletteElementProxy
except ImportError: 
    DomletteReader = DomletteElementProxy = None
    
from pyGridWare.wsrf.faults import BaseFault


# 
# Swap in 4Suite Domlette for Performance and XPath support
# 
DataHandler.readerClass = DomletteReader
DataHandler.writerClass = DomletteElementProxy


class CallbackHandler:
    classProvides(CallbackChainInterface)

    @classmethod
    def processRequest(cls, ps, **kw):
        """invokes callback that should return a (request,response) tuple.
        representing the SOAP request and response respectively.
        ps -- ParsedSoap instance representing HTTP Body.
        request -- twisted.web.server.Request
        """
        resource = kw['resource']
        request = kw['request']
        method = resource.root[_get_element_nsuri_name(ps.body_root)]
        if method is None:
            raise UnknownRequestException,\
                'failed to map request to a method: action(%s), root%s'\
                %(action,_get_element_nsuri_name(ps.body_root))
                
        try:
            req_pyobj,rsp_pyobj =\
                getattr(resource, method)(ps, ps.address, request=request)
        except TypeError, ex:
            log.err(
                'ERROR: service %s.%s is broken, method MUST return request, response' %(
                    resource.__class__, method)
            )
            raise
        except Exception, ex:
            log.err('failure when calling bound method')
            raise
        
        return rsp_pyobj


class HandlerChainFactory:
    protocol = DefaultHandlerChain
    
    @classmethod
    def newInstance(cls):
        return cls.protocol(CallbackHandler, DataHandler, WSAddressHandler())

    
class WSResource(_WSResource):
    '''
    class variables:
        encoding  
        soapAction -- Must override to set input Action values.
           Dictionary of soapAction keys, and operation name values.
           These are specified in the WSDL soap bindings. There must be a 
           class method matching the operation name value.  If WS-Action is
           used the keys are WS-Action request values, according to the spec
           if soapAction and WS-Action is specified they must be equal.
           
        wsAction -- Must override to set output Action values.
           Dictionary of operation name keys and WS-Action 
           response values.  These values are specified by the portType.

        root -- dictionary of root element keys, and operation name values.
    '''
    soapAction = {}
    wsAction = {}
    root = {}
    factory = HandlerChainFactory



#class FileServerResource(Resource, object):
#    '''Represents a root directory to serve files out of.
#    '''
#    def __init__(self, root='wsdl'):
#        '''root -- root directory path.
#        '''
#        self.root = root
#        Resource.__init__(self)
#
#    def render_GET(self, request):
#        s = StringIO()
#        if request.path.startswith('/%s' %self.root) is False:
#            raise NoResource, request.path
#
#        rpath = request.path.split(self.root, 1)[1].lstrip('/')
#        fpath = os.path.join(self.root, rpath)
#        if os.path.isfile(fpath) is True:
#            for l in open(fpath,'r'): s.write(l)
#            return s.getvalue()
#
#        raise NoResource, 'request(%s), no file(%s)' %(request.path,fpath)
#
#    def getChild(self, path, request):
#        '''so can serve GET requests and act as directory.
#        '''
#        return self

