############################################################################
# Joshua R. Boverhof, LBNL
# See Copyright for copyright notice!
###########################################################################
from ZSI.writer import SoapWriter
from ZSI.TC import _get_global_element_declaration as GED
from ZSI.wstools.Namespaces import OASIS
from pyGridWare.generated.types import basefaults

__all__=['PropertiesFaults',]


class BaseFault(Exception):
    ''' 
    '''
    typecode = GED(OASIS.BASEFAULTS, "BaseFault")

    def __init__(self, timestamp=None, originator=None, errorcode=None, 
                 description=None, faultcause=None):
        '''Parameters/KeyWord arguments
        Timestamp -- gDateTime
        Originator -- EPR
        ErrorCode --
        Description --
        FaultCause -- <any>
        Instance Data:
           fault -- pyobj representation of fault
        '''
        fault = self.__class__.typecode.pyclass()
        fault._Timestamp = timestamp
        fault._Originator = originator
        fault._ErrorCode = errorcode
        fault._Description = description
        fault._FaultCause = faultcause
        self.fault = fault
        Exception.__init__(self, fault)

    def __str__(self):
        kw = {\
            'mod':self.__class__.__module__, 
            'class':self.__class__, 
            'id':id(self),
            'xml':'',
        }
        try:
            sw = SoapWriter(envelope=False, header=False)
            sw.serialize(self.fault)
            kw['xml'] = '\n%s' %str(sw)
        except:
            raise

        return '<%(mod)s.%(class)s instance %(id)x %(xml)s>' %kw

    def getFault(self, fault):
        return self._fault
    def setFault(self, fault):
        self._fault = fault
    fault = property(getFault, setFault, None, '')


import PropertiesFaults
def WrapFaultInException(pyobj):
    element = (pyobj.typecode.nspname, pyobj.typecode.pname)
    for pyclass in (BaseFault, PropertiesFaults.ResourceUnknownFault):
        if (pyclass.typecode.nspname, pyclass.typecode.pname) != element:
            continue
        break
    else:
        return RuntimeException(\
            'unknown fault(%s,%s): ' %(pyobj.typecode.nspname, pyobj.typecode.pname), pyobj
        )

    ex = pyclass()
    ex.fault = pyobj
    return ex



