############################################################################
# Joshua R. Boverhof, LBNL
# See LBNLCopyright for copyright notice!
###########################################################################
import socket, urlparse
import ZSI
from twisted.python import log
from twisted.internet import reactor
from twisted.web.server import Site
from twisted.web.resource import Resource
from twisted.internet.error import CannotListenError

from pyGridWare.addressing.AddressingUtils import AddressingUtils
from pyGridWare.wsrf.properties.ResourceProperties import ResourceProperties

# Enable HTTP chunking
from pyGridWare.utility.http import ChunkedHTTPChannel
Site.protocol = ChunkedHTTPChannel

# Generated Code
from pyGridWare.generated.services.basenotification.WS_BaseNotificationService import \
    WS_BaseNotificationServiceWSRF as NCServiceInterface
from pyGridWare.generated.services.basenotification.WS_BaseNotificationService import \
    ManagerHome, NotificationProducerContext

import pyGridWare.generated.types.notificationconsumerservice


"""
  Notification Consumer portType does not contain ResourceProperties
  so must manually set up a context.
"""


class NotificationConsumerService(NCServiceInterface):
    '''Default NotificaionConsumerService to receive
    Notify messsages.
    '''

    def wsa_Notify(self, ps, address, **kw):
        log.msg("=====NCS Notification: ", debug=True)
        request,response = NCServiceInterface.wsa_Notify(self, ps, address)

        log.msg("Notification: ", request)
        pyobj = request._NotificationMessage[0]
        log.msg("  NotificationMessage: ", pyobj)
        log.msg("    Topic: ", pyobj._Topic)
        log.msg("    ProducerReference: ", pyobj._ProducerReference)
        log.msg("    Message: ", pyobj._Message)
        pyobj = pyobj._Message._ResourcePropertyValueChangeNotification
        log.msg("        NewValue: ", pyobj._NewValue)
        log.msg("          Value: ", pyobj._NewValue._any)
        if pyobj._OldValue is not None:
            log.msg("        OldValue: ", pyobj._OldValue)
            log.msg("          Value: ", pyobj._OldValue._any)
        return request,response


class NotificationConsumer(object):
    '''A NotificationConsumer sets up a NotificationConsumerService 
    resource at the desired path.  Must call start to begin listening,
    and to set up the endPointReference.
    '''

    def __init__(self, path='/wsrf/services/NotificationConsumerService', 
        notificationClass=NotificationConsumerService):
        '''
        notificationClass -- Notification Consumer Service 
        '''
        self.context = None
        self.path = path
        self.consumer = None
        self.factory = None
        self.port = None

        assert issubclass(notificationClass, NCServiceInterface), \
            'notificationClass must be NotificationConsumer'
        root = Resource()
        resource = root
        token = '/'
        ipath = path.strip(token)
        for key in ipath.split(token)[:-1]:
            resource.putChild(key, Resource())
            resource = resource.getStaticEntity(key)

        service_key = ipath.split(token)[-1]
        self.consumer = notificationClass()
        resource.putChild(service_key, self.consumer)
        self.factory = Site(root)

    def getEndPointReference(self):
        '''NotificationConsumer EndpointReference
        '''
        if self.context is None:
            raise RuntimeError, 'Must start NotificationConsumer to initialize EPR.'

        return AddressingUtils.createEndpointReference(self.context)

    def start(self, ssl=False, keyfile=None, certfile=None, contextFactory=None, interface=None):
        scheme = 'http'
        self.consumer.ssl = ssl
        interface = interface or socket.gethostbyname(socket.gethostname())
        try:
            if ssl is True:
                if contextFactory is None:
                    from twisted.internet import ssl as SSL
                    contextFactory = SSL.DefaultOpenSSLContextFactory(keyfile, certfile)
                self.port = reactor.listenSSL(0, self.factory, contextFactory, interface=interface)
                scheme = 'https'
            else:
                self.port = reactor.listenTCP(0, self.factory, interface=interface)
        except CannotListenError, ex:
            raise

        #hostname = socket.getfqdn(self.port.getHost().host)
        hostname = interface
        netloc = '%s:%s' %(hostname,self.port.getHost().port)
        url = urlparse.urlunparse((scheme,netloc,self.path,'','',''))
        self.context = NotificationConsumerResource.NewResourceContextNotificationConsumerPort(url)


class NotificationConsumerResource:
    '''Helper classes for specifying SOAP Data
    '''

    def NewResourceContextNotificationConsumerPort(cls, url):
        '''create a new resource context
        '''
        ctx = NotificationConsumerResource.NotificationConsumerContext()
        ctx.setAddress(url)
        home = ctx.getResourceHome()
        key = home.create()
        home.add(ctx, key)
        ctx.setResourceID(key)
        return ctx
    NewResourceContextNotificationConsumerPort = classmethod(NewResourceContextNotificationConsumerPort)

    class NotificationConsumerContext(NotificationProducerContext):
        class EmptyResourceProperties(ResourceProperties):
            '''This does nothing since there are no ResourceProperties.
            '''
            def __init__(self): 
                return
            def setUp(self, xml=None): 
                return

        def __init__(self, properties_class=EmptyResourceProperties):
            NotificationProducerContext.__init__(self, properties_class)

    NotificationConsumerContext.home = ManagerHome.getInstance()



