############################################################################
# Joshua R. Boverhof, LBNL
# See Copyright for copyright notice!
# Generated by setup.py
###########################################################################

import time
from pyGridWare.wsrf.lifetime.ResourceLifetime import ImmediateResourceTermination
from pyGridWare.wsrf.lifetime.ResourceLifetime import ScheduledResourceTermination
from pyGridWare.wsrf.notification.SubscriptionManager import SubscriptionManager
from generated.counter_service.services.stubs.counter_service.CounterService import *
from generated.counter_service.resource.counter_service.CounterService import ManagerHome
from generated.counter_service.resource.counter_service.CounterService import CounterPortTypeContext


class CounterServiceWSRF(CounterService):

    def GetResourceContext(ps, address):
        """get a resource context"""
        return ManagerHome.getInstance().getResourceContext(ps, address)
    GetResourceContext = staticmethod(GetResourceContext)

    def NewResourceContextCounterPortTypePort(cls, url):
        """create a new resource context"""
        ctx = CounterPortTypeContext()
        ctx.setAddress(url)
        home = ctx.getResourceHome()
        key = home.create()
        home.add(ctx, key)
        ctx.setResourceID(key)
        return ctx
    NewResourceContextCounterPortTypePort = classmethod(NewResourceContextCounterPortTypePort)

    def wsa_createCounter(self, ps, address, **kw):
        #http://counter.com CreateCounterRequest
        request,response = CounterService.wsa_createCounter(self, ps, address)
        return request,response

    def wsa_add(self, ps, address, **kw):
        #http://counter.com AddInputMessage
        request,response = CounterService.wsa_add(self, ps, address)
        return request,response

    def wsa_GetResourceProperty(self, ps, address, **kw):
        #http://docs.oasis-open.org/wsrf/2004/06/wsrf-WS-ResourceProperties-1.2-draft-01.wsdl GetResourcePropertyRequest
        request,response = CounterService.wsa_GetResourceProperty(self, ps, address)
        ctx = ManagerHome.getInstance().getResourceContext(ps, address)
        namespaceURI,element = request
        property = ctx.properties.getResourceProperty(namespaceURI,element)
        response._any = [property]
        return request,response

    def wsa_Subscribe(self, ps, address, **kw):
        #http://docs.oasis-open.org/wsn/2004/06/wsn-WS-BaseNotification-1.2-draft-01.wsdl SubscribeRequest
        request,response = CounterService.wsa_Subscribe(self, ps, address)
        ctx = self.GetResourceContext(ps, address)
        assert SubscriptionManager._isInstantiated(),"SubscriptionManager is not instantiated"
        manager = SubscriptionManager.getInstance()
        response._SubscriptionReference = manager.subscribe(request, ctx)
        return request,response

    def wsa_GetCurrentMessage(self, ps, address, **kw):
        #http://docs.oasis-open.org/wsn/2004/06/wsn-WS-BaseNotification-1.2-draft-01.wsdl GetCurrentMessageRequest
        request,response = CounterService.wsa_GetCurrentMessage(self, ps, address)
        ctx = self.GetResourceContext(ps, address)
        assert SubscriptionManager._isInstantiated(),"SubscriptionManager is not instantiated"
        manager = SubscriptionManager.getInstance()
        response._any = manager.current(request, ctx)
        return request,response

    def wsa_Destroy(self, ps, address, **kw):
        #http://docs.oasis-open.org/wsrf/2004/06/wsrf-WS-ResourceLifetime-1.2-draft-01.wsdl DestroyRequest
        request,response = CounterService.wsa_Destroy(self, ps, address)
        ctx = CounterServiceWSRF.GetResourceContext(ps, address)
        ImmediateResourceTermination.Destroy(ctx)
        return request,response

    def wsa_SetTerminationTime(self, ps, address, **kw):
        #http://docs.oasis-open.org/wsrf/2004/06/wsrf-WS-ResourceLifetime-1.2-draft-01.wsdl SetTerminationTimeRequest
        request,response = CounterService.wsa_SetTerminationTime(self, ps, address)
        ctx = CounterServiceWSRF.GetResourceContext(ps, address)
        ScheduledResourceTermination.SetTerminationTime(ctx, request._RequestedTerminationTime)
        response._NewTerminationTime = time.gmtime(ctx.TerminationTime)
        response._CurrentTime = time.gmtime(ctx.CurrentTime)
        return request,response

    def wsa_QueryResourceProperties(self, ps, address, **kw):
        #http://docs.oasis-open.org/wsrf/2004/06/wsrf-WS-ResourceProperties-1.2-draft-01.wsdl QueryResourcePropertiesRequest
        request,response = CounterService.wsa_QueryResourceProperties(self, ps, address)
        ctx = ManagerHome.getInstance().getResourceContext(ps, address)
        response._any = ctx.properties.query(request._QueryExpression, response)
        return request,response

    def wsa_GetMultipleResourceProperties(self, ps, address, **kw):
        #http://docs.oasis-open.org/wsrf/2004/06/wsrf-WS-ResourceProperties-1.2-draft-01.wsdl GetMultipleResourcePropertiesRequest
        request,response = CounterService.wsa_GetMultipleResourceProperties(self, ps, address)
        ctx = ManagerHome.getInstance().getResourceContext(ps, address)
        response._any = []
        for nsuri,name in request._ResourceProperty:
            response._any.append(ctx.properties.getResourceProperty(nsuri, name))
        return request,response

