#!/usr/bin/env python
##########################################################################
# Joshua R. Boverhof, LBNL
# See Copyright for copyright notice!
###########################################################################
import optparse, os, sys, signal, time, warnings, unittest
from distutils.core import setup
from distutils.dist import Distribution
from distutils.command.clean import clean as _clean
from distutils.command.build import build as _build
from distutils.dir_util import remove_tree
from os.path import isfile
from ConfigParser import ConfigParser

from twisted.internet import reactor
from twisted.python import log

from ZSI import ZSIException
from ZSI.wstools.WSDLTools import WSDLReader
from ZSI.wstools.Namespaces import WSA200403


from pyGridWare.wsrf.notification.NotificationConsumer import \
    NotificationConsumer, NotificationConsumerService
    
import pyGridWare.wsrf.faults

# WARNING:
if sys.version_info[:3] < (2,4,0):
     warnings.warn('Expecting python2.4 or later, found version ' + str(sys.version_info))

sys.path.append('suites')
import suites

PID = None



"""MyTestProgram: 
Expects the module containing each test case to define two globals.  If
you do not, createTests will break (AttributeError).
    WSDL -- path to WSDL file.
    SECTION -- name of generated stub module.
 
"""
class MyTestProgram(unittest.TestProgram):
    USAGE = """\
Usage: %(progName)s [options] [test] [...]

Options:
  -h, --help       Show this message
  -w, --off        Turn off wsdl to python generation.
  -v, --verbose    Verbose output
  -q, --quiet      Minimal output
  -f, --future     Use twisted.web2

Examples:
  %(progName)s                               - run default set of tests
  %(progName)s MyTestSuite                   - run suite 'MyTestSuite'
  %(progName)s MyTestCase.testSomething      - run MyTestCase.testSomething
  %(progName)s MyTestCase                    - run all 'test*' test methods
                                               in MyTestCase
"""

    def __init__(self, module='__main__', defaultTest=None,
                 argv=None, testRunner=None, testLoader=unittest.defaultTestLoader):
        self.generate = True
        if type(module) == type(''):
            self.module = __import__(module)
            for part in module.split('.')[1:]:
                self.module = getattr(self.module, part)
        else:
            self.module = module
        if argv is None:
            argv = sys.argv
        self.useWeb2 = False
        self.verbosity = 1
        self.defaultTest = defaultTest
        self.testRunner = testRunner
        self.testLoader = testLoader
        self.progName = os.path.basename(argv[0])
        if self.parseArgs(argv):
            self.runTests()

    def usageExit(self, msg=None):
        if msg: print msg
        print self.USAGE % self.__dict__
        reactor.callWhenRunning(reactor.stop)

    def parseArgs(self, argv):
        import getopt
        global PID
        try:
            options, args = getopt.getopt(argv[1:], 'hHvwqf',
                                          ['help','verbose','off','quiet', 'future'])
            for opt, value in options:
                if opt in ('-h','-H','--help'):
                    self.usageExit()
                    return False
                if opt in ('-w','--off'):
                    self.generate = False
                if opt in ('-q','--quiet'):
                    self.verbosity = 0
                if opt in ('-v','--verbose'):
                    self.verbosity = 2
                if opt in ('-f','--future'):
                    self.useWeb2 = True
            if len(args) == 0 and self.defaultTest is None:
                self.test = self.testLoader.loadTestsFromModule(self.module)
                return
            if len(args) > 0:
                self.testNames = args
            else:
                self.testNames = (self.defaultTest,)
            self.createTests()
        except getopt.error, msg:
            self.usageExit(msg)
            return False
        except RuntimeError, ex:
            log.err(ex)
            return False

        tac = '../bin/server-config.tac'
        if self.useWeb2:
            tac = '../bin/web2-server-config.tac'

        PID = os.spawnlp(os.P_NOWAIT, 'twistd', 'twistd', '-noy', 
                          tac, '-l', 'server.log')
                          
        print "wait for server process to initialize..."
        time.sleep(5)
        return True

    def createTests(self):
        # -v
        if self.verbosity > 1:
            log.startLogging(sys.stdout)

        unittest.TestProgram.createTests(self)
        if self.generate is False:
            return
     
        tests = []
        def flatten(i): 
            if isinstance(i, unittest.TestCase): tests.append(i)
            if isinstance(i, unittest.TestSuite): 
                    for j in i._tests: flatten(j)
        flatten(self.test)
 
        modset = set(map(lambda t:eval(t.__module__), tests))
        BigFunk = lambda f,w: os.path.realpath(os.path.join(os.path.dirname(f),w))
        args = map(lambda mod: BigFunk(mod.__file__,mod.WSDL), modset)
        try:
            exit = os.spawnlp(os.P_WAIT, 'wsdl2web.py', 'wsdl2web.py', 
                               '--force', *args)
        except OSError, ex:
            print "TODO: Not sure what is going on here?"
            #reactor.callLater(0.1, reactor.stop)
            #raise
            return
        
        if os.WIFEXITED(exit) and exit == 0: 
            return
            
        msg = 'wsdl2web.py exited with signal#: %d' %exit
        if os.WIFEXITED(exit):
            msg = 'wsdl2web.py exited with exit status: %d' %exit
        
        reactor.callLater(0.1, reactor.stop)
        raise RuntimeError, msg

      
    def runTests(self):
        log.msg("RUN TESTS")
        if self.testRunner is None:
            self.testRunner = unittest.TextTestRunner(verbosity=self.verbosity)
        result = self.testRunner.run(self.test)
        log.msg("SUCCESS?: ", result, debug=True)
        log.msg("SUCCESS?: ", result.wasSuccessful(), debug=True)

        # TODO: HERE NEED TO WAIT FOR NOTIFICATIONS??
        # assume notification test will stop reactor?
        #if async is False:
        if True:
            reactor.callLater(0.1, reactor.stop)
            reactor.callLater(0.1, os.kill, PID, signal.SIGTERM)



def main():
    reactor.callWhenRunning(MyTestProgram, defaultTest="suites.counter.WSRFCounterTestSuite")
    reactor.run()


if __name__ == '__main__': 
    main()
