#!/usr/bin/env python
##########################################################################
# Joshua R. Boverhof, LBNL
# See Copyright for copyright notice!
###########################################################################
import optparse, os, sys, signal, time, warnings, unittest
#from distutils import log
from distutils.core import setup
from distutils.dist import Distribution
from distutils.command.clean import clean as _clean
from distutils.command.build import build as _build
from distutils.dir_util import remove_tree
from os.path import isfile
from ConfigParser import ConfigParser

from twisted.internet import reactor
from twisted.python import log

from ZSI import ZSIException
from ZSI.wstools.WSDLTools import WSDLReader
from ZSI.wstools.Namespaces import WSA200403

#from pyGridWare.utility.generate.Utility import CLIENT,SERVICE,\
#    SERVICE_STUBS,TYPES,PROPERTIES,RESOURCE, NameResolver as NR
#from pyGridWare.utility.generate.Modules import WriteServiceModules,\
#    WriteClientStubModule, WriteTypesModule

from pyGridWare.wsrf.notification.NotificationConsumer import \
    NotificationConsumer, NotificationConsumerService
    
import pyGridWare.wsrf.faults
    
#NR.d[CLIENT] = 'gen.stubs'
#NR.d[SERVICE] = 'gen.services'
#NR.d[SERVICE_STUBS] = 'gen.services.stubs'
#NR.d[TYPES] = 'gen.types'
#NR.d[PROPERTIES] = 'gen.properties'
#NR.d[RESOURCE] = 'gen.resource'

#log.startLogging(sys.stdout)
ASYNC = False

from counter import WSDL, CounterTestSuite, CreateTestCase, AddTestCase, DestroyTestCase

class BenchmarkTestSuite(CounterTestSuite):
    def __init__(self, tests=()):
        CounterTestSuite.__init__(self, tests=tests)
        self.times = {}

    def mark_start(self, test):
        i = id(test)
        n = test.__class__.__name__
        t = time.time()
        self.times[i] = [n,t]

    def mark_end(self, test):
        t = time.time()
        i = id(test)
        self.times[i].append(t)
        self.times[i] = tuple(self.times[i])

    def run(self, result):
        '''Factory method first.  Each factory method call will change the EPR
        for subsequent calls.
        '''
        epr = None
        for test in self._tests:
            if result.shouldStop: break
            
            test.setEPR(epr)
            self.mark_start(test)
            test(result)
            self.mark_end(test)
            epr = test.getEPR()
 
        self.print_stats()
        return result

    def print_stats(self):
        hi = None
        low = None        
        sum = 0.0
        values = self.times.values()[1:-1]
        for test,beg,end in values:
            t1 = (end-beg)   
            sum += t1
            if hi is None or hi<t1: hi = t1 
            if low is None or low>t1: low = t1 

        print "STATISTICS"
        print "\tLOW  -- %6.2fms" %(low*1000.0)
        print "\tHI   -- %6.2fms" %(hi*1000.0)
        print "\tMEAN -- %6.2fms" %(sum*1000.0/len(values))
        print 



def Benchmark100TestSuite():
    tests=[CreateTestCase()]
    for i in range(100): tests.append(AddTestCase())
    tests.append(DestroyTestCase())

    return BenchmarkTestSuite(tests=tuple(tests))


