#!/usr/bin/env python
##########################################################################
# Joshua R. Boverhof, LBNL
# See Copyright for copyright notice!
###########################################################################
import optparse, os, sys, signal, time, warnings, unittest
#from distutils import log
from distutils.core import setup
from distutils.dist import Distribution
from distutils.command.clean import clean as _clean
from distutils.command.build import build as _build
from distutils.dir_util import remove_tree
from os.path import isfile
from ConfigParser import ConfigParser

from twisted.internet import reactor
from twisted.python import log

from ZSI import ZSIException
from ZSI.wstools.WSDLTools import WSDLReader
from ZSI.wstools.Namespaces import WSA200403

from pyGridWare.utility.generate.Utility import CLIENT,SERVICE,\
    SERVICE_STUBS,TYPES,PROPERTIES,RESOURCE, NameResolver as NR
from pyGridWare.utility.generate.Modules import WriteServiceModules,\
    WriteClientStubModule, WriteTypesModule

from pyGridWare.wsrf.notification.NotificationConsumer import \
    NotificationConsumer, NotificationConsumerService
    
import pyGridWare.wsrf.faults

# 
# DONT BE STUPID, JUST WALK AWAY
# 

import httplib
_UNKNOWN = httplib._UNKNOWN
BIGFAT = 8192
class BigFatHTTPResponse(httplib.HTTPResponse):
    def __init__(self, sock, debuglevel=0, strict=0, method=None):
        self.fp = sock.makefile('rb', BIGFAT)
        self.debuglevel = debuglevel
        self.strict = strict
        self._method = method

        self.msg = None

        # from the Status-Line of the response
        self.version = _UNKNOWN # HTTP-Version
        self.status = _UNKNOWN  # Status-Code
        self.reason = _UNKNOWN  # Reason-Phrase

        self.chunked = _UNKNOWN         # is "chunked" being used?
        self.chunk_left = _UNKNOWN      # bytes left to read in current chunk
        self.length = _UNKNOWN          # number of bytes left in response
        self.will_close = _UNKNOWN      # conn will close at end of response

    def _check_close(self):
        conn = self.msg.getheader('connection')
        if self.version == 11:
            # An HTTP/1.1 proxy is assumed to stay open unless
            # explicitly closed.
            conn = self.msg.getheader('connection')
            if conn and "close" in conn.lower():
                return True
            return False

        # Some HTTP/1.0 implementations have support for persistent
        # connections, using rules different than HTTP/1.1.

        # For older HTTP, Keep-Alive indiciates persistent connection.
        if self.msg.getheader('keep-alive'):
            return False

        # At least Akamai returns a "Connection: Keep-Alive" header,
        # which was supposed to be sent by the client.
        if conn and "keep-alive" in conn.lower():
            return False

        # Proxy-Connection is a netscape hack.
        pconn = self.msg.getheader('proxy-connection')
        if pconn and "keep-alive" in pconn.lower():
            return False

        # otherwise, assume it will close
        return True

#httplib.HTTPConnection.response_class = BigFatHTTPResponse

class BigFatHTTPConnection(httplib.HTTPConnection):
    response_class = BigFatHTTPResponse
    def __init__(self, host, port=None, strict=None):
        self.going_already = False 
        httplib.HTTPConnection.__init__(self, host, port=port, strict=strict)

    def connect(self):
        '''To do keep alive only connect once, prevent ZSI from
        doing it for each flow.
        '''
        msg = "getaddrinfo returns an empty list"
        if self.going_already: return
        print "CONNECT "*10
        httplib.HTTPConnection.connect(self)
        self.going_already = True

    def close(self):
        '''To do keep alive we want to keep the connection open,
        prevent ZSI from closing it.
        '''
        #httplib.HTTPConnection.close(self)
        return


XINST = None
def Transport(netloc, x, **kw):
    '''Returns the same HTTPConnection to ZSI.
    '''
    global XINST
    if XINST is None: XINST = BigFatHTTPConnection(netloc, x, **kw)
    return XINST
   
TRANSPORT = Transport


from benchmarks import WSDL, BenchmarkTestSuite,\
    CreateTestCase, AddTestCase, DestroyTestCase


class BigFatTestSuite(BenchmarkTestSuite):
    def run(self, result):
        '''Factory method first.  Each factory method call will change the EPR
        for subsequent calls. Swap in TRANSPORT.
        '''
        warnings.warn(\
            'EXPERIMENTAL: Running this test suite will override a bunch of modules to test keep alives in httplib and ZSI!!!'
        )

        import gen.stubs.counter
        import ZSI.client
        # SWAP OUT client
        gen.stubs.counter.client = ZSI.client

        epr = None
        for test in self._tests:
            if result.shouldStop: break
            
            bound = test.setUp
            this = bound.im_self
            def trap():
                bound()
                this.port.binding.transport = TRANSPORT
            test.setUp = trap
            
            test.setEPR(epr)
            self.mark_start(test)
            test(result)
            self.mark_end(test)
            epr = test.getEPR()

        # print stats.
        self.print_stats()

        # close it.
        httplib.HTTPConnection.close(XINST)

        return result


def Benchmark100TestSuite():
    tests=[CreateTestCase()]
    #for i in range(100): tests.append(AddTestCase())
    for i in range(1): tests.append(AddTestCase())
    tests.append(DestroyTestCase())

    return BigFatTestSuite(tests=tuple(tests))


