package org.globus.workspace;

import org.apache.axis.client.Stub;
import org.apache.axis.message.addressing.Address;
import org.apache.axis.message.addressing.EndpointReferenceType;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.globus.axis.gsi.GSIConstants;
import org.globus.gsi.gssapi.GlobusGSSCredentialImpl;
import org.globus.gsi.jaas.JaasGssUtil;
import org.globus.gsi.jaas.UserNamePrincipal;
import org.globus.util.I18n;
import org.globus.workspace.generated.PolicyQuery;
import org.globus.workspace.generated.QueryAccessPolicyResponse;
import org.globus.workspace.generated.WorkspacePortType;
import org.globus.workspace.generated.WorkspaceServiceAddressingLocator;
import org.globus.workspace.impl.service.WorkspaceHome;
import org.globus.wsrf.Constants;
import org.globus.wsrf.impl.security.authorization.exceptions.AuthorizationException;
import org.globus.wsrf.impl.security.authorization.exceptions.CloseException;
import org.globus.wsrf.impl.security.authorization.exceptions.InitializeException;
import org.globus.wsrf.impl.security.authorization.exceptions.InvalidPolicyException;
import org.globus.wsrf.impl.security.util.AuthUtil;
import org.globus.wsrf.security.SecurityManager;
import org.globus.wsrf.security.authorization.PDP;
import org.globus.wsrf.security.authorization.PDPConfig;
import org.globus.wsrf.utils.PerformanceLog;
import org.w3c.dom.Node;

import javax.naming.InitialContext;
import javax.security.auth.Subject;
import javax.xml.namespace.QName;
import javax.xml.rpc.handler.MessageContext;
import java.rmi.RemoteException;

public class QueryPDP implements PDP {

    private static Log logger =
        LogFactory.getLog(QueryPDP.class.getName());

    private static PerformanceLog performanceLogger = new PerformanceLog(
                QueryPDP.class.getName() + ".performance");

    public static final String QueryPDP_I18N =
        "org.globus.workspace.querypdperrors";

    public static final String WORKSPACE_URL_KEY =
        "workspaceService";

    public static final String ALL_MAPPINGS_KEY =
        "allMappings";

    public static final String SERVICE_PATH =
            "WorkspaceService";

    private static I18n i18n =
            I18n.getI18n(QueryPDP_I18N,
                         QueryPDP.class.getClassLoader());

    String servicePath = null;
    private String workspaceServiceURL = null;
    private WorkspacePortType servicePort = null;
    private boolean allMappings = true;
    private WorkspaceHome local = null;

    public String[] getPolicyNames() {
        return null;
    }

    public Node getPolicy(Node node) throws InvalidPolicyException {
        return null;
    }

    public Node setPolicy(Node node) throws InvalidPolicyException {
        return null;
    }

    public boolean isPermitted(Subject peerSubject,
                               MessageContext messageContext,
                               QName operation)
            throws AuthorizationException {

        logger.debug("workspace query authz");

        if (peerSubject == null) {
            throw new IllegalArgumentException(
                i18n.getMessage("noPeerSubject"));
        }

        String peerIdentity = AuthUtil.getIdentity(peerSubject);

        if (peerIdentity == null) {
            logger.debug(i18n.getMessage("anonPeer"));
            throw new AuthorizationException(i18n.getMessage("anonPeer"));
        }

        logger.debug("Operation " + operation + " called by subject: " +
                        peerIdentity);

        String[] usernames = null;
        try {
            usernames = queryService(peerIdentity);
        } catch (RemoteException e) {
            String err = i18n.getMessage("queryService");
            logger.error(err, e);
            throw new AuthorizationException(err);
        }

        if (usernames != null) {
            // if usernames.length == 0, usernames is set
            // to null in queryService()
            for (int i = 0; i < usernames.length; i++) {
                logger.debug("adding: " + usernames[i]);
                peerSubject.getPrincipals()
                    .add(new UserNamePrincipal(usernames[i]));
            }
            return true;
        } else {
            return false;
        }

    }

    public void initialize(PDPConfig config,
                           String name,
                           String id)
            throws InitializeException {

        this.servicePath = id;
        logger.debug("service " + this.servicePath);

        Object workspaceURL = config.getProperty(name, WORKSPACE_URL_KEY);
        String url = (String) workspaceURL;

        if ((url == null) || (url.equals(""))) {

            try {
                performanceLogger.start();
                this.local = getServiceHome();
                performanceLogger.stop("jndi lookup");
            } catch (WorkspaceException e) {
                logger.debug(e);
                throw new InitializeException("",e);
            }

            logger.warn(i18n.getMessage("noWorkspaceServiceURL"));

        } else {
            this.workspaceServiceURL = url;
        }

        try {
            if (this.local == null) {
                performanceLogger.start();
                setServicePort();
                performanceLogger.stop("port creation");
            }
        } catch (RemoteException e) {
            throw new InitializeException("",e);
        }

        Object allMappings = config.getProperty(name, ALL_MAPPINGS_KEY);
        if (allMappings == null) {
            logger.debug(i18n.getMessage("noMappingsConfig"));
        } else {
            if (((String)allMappings).equalsIgnoreCase("false")) {
                this.allMappings = false;
                logger.debug(i18n.getMessage("mappingsConfigFalse"));
            } else {
                logger.debug(i18n.getMessage("mappingsConfigTrue"));
            }
        }

    }

    public void close() throws CloseException {
        //
    }


    /**
     * Initialize the service port.
     */
    private void setServicePort() throws RemoteException {

        logger.debug("Setting up service locator");

        EndpointReferenceType epr = new EndpointReferenceType();
        try {
            epr.setAddress(new Address(this.workspaceServiceURL));
        } catch (Exception e) {
            String err = i18n.getMessage("errorEPRCreate");
            logger.debug(err,e);
            throw new RemoteException(err);
        }

        WorkspaceServiceAddressingLocator serviceLocator =
            new WorkspaceServiceAddressingLocator();

        logger.debug("Creating service port");

        try {
            this.servicePort = serviceLocator.getWorkspacePortTypePort(epr);
        } catch (Exception e) {
            String err = i18n.getMessage("errorServicePortCreate");
            logger.debug(err,e);
            throw new RemoteException(err);
        }

        logger.debug("epr scheme: " + epr.getAddress().getScheme());
        if (epr.getAddress().getScheme().startsWith("https")) {
            org.apache.axis.MessageContext ctx =
                    org.apache.axis.MessageContext.getCurrentContext();
            org.globus.wsrf.security.SecurityManager manager =
                    SecurityManager.getManager(ctx);
            Subject serviceSubject= manager.getServiceSubject();

            GlobusGSSCredentialImpl credential =
                (GlobusGSSCredentialImpl)JaasGssUtil.
                    getCredential(serviceSubject);

           ((Stub)this.servicePort)._setProperty(GSIConstants.GSI_CREDENTIALS,
                   credential);
           ((Stub)this.servicePort)._setProperty(GSIConstants.GSI_TRANSPORT,
                   org.globus.wsrf.impl.security.authentication.
                   Constants.ENCRYPTION);

        } else if (epr.getAddress().getScheme().startsWith("http")) {
            String err = i18n.getMessage("unsupportedSchemeHTTP");
            logger.debug(err);
            throw new RemoteException(err);
        } else {
            String err = i18n.getMessage("unsupportedScheme",
                    epr.getAddress().getScheme());
            throw new RemoteException(err);
        }

        logger.debug("successful service port creation");

        return;
    }

    private String[] queryService(String entry)
            throws RemoteException {

        PolicyQuery query = new PolicyQuery(this.allMappings, entry, entry);
        performanceLogger.start();
        QueryAccessPolicyResponse resp = null;
        String[] usernames = null;
        if (this.local == null) {
            resp = this.servicePort.queryAccessPolicy(query);
            if (resp == null) {
                return null;
            }
            usernames = resp.getUsername();
        } else {
            usernames = this.local.queryAccessPolicy(query);
        }
        performanceLogger.stop("query call");

        if (usernames != null) {
            if (usernames.length == 0) {
                return null;
            } else {
                logger.debug("query returned "
                        + usernames.length + " usernames");
                return usernames;
            }
        } else {
            return null;
        }
    }

    private WorkspaceHome getServiceHome() throws WorkspaceException{
        try {
            InitialContext context = new InitialContext();
            logger.debug(Constants.JNDI_SERVICES_BASE_NAME +
                         WorkspaceConstants.SERVICE_PATH +
                         Constants.HOME_NAME);
            return (WorkspaceHome)context
                .lookup(Constants.JNDI_SERVICES_BASE_NAME +
                        WorkspaceConstants.SERVICE_PATH +
                        Constants.HOME_NAME);
        } catch (Exception exp) {
            throw new WorkspaceException(exp);
        }
    }


}
