/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */

#include <stdio.h>
#include <ctype.h>
#include <errno.h>
#include <pwd.h>
#include <grp.h>
#include <sys/wait.h>
#include <sys/types.h>
#include <signal.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <unistd.h>
#include <sys/fsuid.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "lcmaps_return_poolindex_without_gsi.h"
#include "errors.h"
#include "gridmapdir.h"
#include "log.h"
#include "util.h"


/* prototypes */
int poolindex(char *dn, char *fqan_list[], int nfqan, int mapcount);
int delete_by_dn_attr(char *dn, char *fqan_list[], int nfqan);
int unquarantine(char *input);
int delete_by_index(char *poolindex);

/* Global variables */
uid_t g_real_uid;
uid_t g_euid;
gid_t g_gid;
char *g_gridmapdir  = NULL;

/* __________________________________________________
   main
   _______________________________________________{{{ */
int main(int argc, char *argv[])
{
  char *cp, *prog;
  char *dn = NULL; 
  char *countstr = NULL;
  int mapcount = -1;
  char **fqan_list = NULL; 
  int nfqan = -1;
  int fqanoffset = 5;
  int ret = -1;
  char *index = NULL;

  g_euid = geteuid();
  g_real_uid = getuid();
  
  /* drop suid if g_real_uid is different than runner */
  seteuid(g_real_uid);
  
  /*  debug_init() is called by parse_config_file */
  
  /* get given path to configuration file */
  if ((argc < 3) || (strncmp(argv[1], "-conf",6) != 0)) {
    goto help; 
    /* it's time to include an arbitrary arg handler */
  }

  /* parse config file (and check its permissions) - results are 
     placed in global variables.  Guaranteed to return with full
     set of options, otherwise parse_config_file will return
     ENVIRONMENT_CONFIG_FILE.  */
     
  /* TODO: time to move this all to a typical arg parsing for loop, this
     is ugly now */
  ret = parse_config_file(argv[0], argv[2], g_real_uid, g_euid);
  if (ret != SUCCESS) {
    goto exit_free;
  }
  
  if (g_RUN_EXPIRY_SETUID >= 0)
  {
    if (g_RUN_EXPIRY_SETUID != g_euid) {
      fprintf(stderr, "Must run with euid %d.\n", g_RUN_EXPIRY_SETUID);
      ret = ENVIRONMENT;
      goto exit_free;
    }
  }
  
  if (g_POOL_LOGFILE != NULL) {
    if (log_init()) {
      fprintf(stderr, "cannot initialize logging\n");
      ret = ENVIRONMENT;
      goto exit_free;
    }
  }
  
  /* convention: using null fqan_list when nfaqn is 0 
     but dn must be !null */
  /* maybe it's time to include an arbitrary arg handler */
     
  if (argc == 5 && strncmp(argv[3], "-unlease_poolindex",19) == 0) {
    index = safe_dup(argv[4]);  /*safe_dup calls exit on error*/
    
    /* for now, gridmapdir must be set in .conf AND lcmaps.db 
       for this to work, todo: could parse lcmaps.db here */
    g_gridmapdir = g_LCMAPS_GRIDMAPDIR;
    if(setenv("MAPDIR", g_LCMAPS_GRIDMAPDIR,0) != 0) {
      fprintf(stderr,"cannot set GRIDMAPDIR: %s\n",g_LCMAPS_GRIDMAPDIR);
      ret = ENVIRONMENT;
      goto exit_free;
    }
    
    ret = delete_by_index(index);
    free(index);
    goto exit_free;
  }
  else if (argc == 5 && strncmp(argv[3], "-unquarantine",14) == 0)
  {
    /* for now, gridmapdir must be set in .conf AND lcmaps.db 
       for this to work, todo: could parse lcmaps.db here */
    g_gridmapdir = g_LCMAPS_GRIDMAPDIR;
    if(setenv("MAPDIR", g_LCMAPS_GRIDMAPDIR,0) != 0) {
      fprintf(stderr,"cannot set GRIDMAPDIR: %s\n",g_LCMAPS_GRIDMAPDIR);
      ret = ENVIRONMENT;
      goto exit_free;
    }
    ret = unquarantine(safe_dup(argv[4]));
    log_msg("RESULT: %d\n",ret);
    goto exit_free;
  }
  else if (argc >= 5)
  {
    int i;
    
    /* DN can not be absent */ 
    dn = safe_dup(argv[4]);
    
    /* hack for the moment until move to arbitrary arg handling */
    if (strncmp(dn, "-count",7) == 0) {
      if (argc >= 7) {
        countstr = safe_dup(argv[5]);
        debug_msg(5,"mapcount arg = %s",countstr);
        mapcount = atoi(countstr);
        if (strncmp(countstr, "0",2) == 0) {
          mapcount = 0;
        }
        if (mapcount < 1) {
          if (strncmp(countstr, "0",2) == 0) {
            mapcount = 0;
          } else {
            fprintf(stderr, "-count must be an integer and >= 0\n"); 
            goto help;
          }
        }
        debug_msg(5,"mapcount int = %d",mapcount);
        free(countstr);
        free(dn);
        dn = NULL;
        dn = safe_dup(argv[6]);
      
      } else {
        goto help;
      }
    }
    
    debug_msg(5,"dn = %s",dn);
    
    if (mapcount > -1) {
      fqanoffset = 7;
    }
    
    /* if more args, they are fqan's */
    nfqan = argc - fqanoffset;
    
    if (nfqan == 0) {
      fqan_list = NULL;
    } else {
    
      debug_msg(5,"nfqan = %d", nfqan);
      
      fqan_list = (char **) malloc(sizeof(char *) * nfqan);
      
      for (i = fqanoffset; i < nfqan + fqanoffset; i++) {
        debug_msg(5,"fqan%2d = %s",i-fqanoffset+1,argv[i]);
        fqan_list[i-fqanoffset] = safe_dup(argv[i]);
      }
    }
    
    if (strncmp(argv[3], "-return_poolindex",18) == 0) {
      ret = poolindex(dn, fqan_list, nfqan, mapcount);
      log_msg("RESULT: %d\n",ret);
      goto exit_free;
      
    }  
    
    /* disabling option entirely */
    /*} else if (strncmp(argv[3], "-unlease_dnattr",16) == 0) { */
      /* don't use this option! */

      /* note, MAPDIR env var set by LCMAPS code */
      /* for now, gridmapdir must be set in .conf AND lcmaps.db 
       for this to work, todo: could parse lcmaps.db here */
      
      /*g_gridmapdir = g_LCMAPS_GRIDMAPDIR;
      ret = delete_by_dn_attr(dn, fqan_list, nfqan);
      goto exit_free;
    } */
    
    
    /* falls through otherwise */
    
  }
  help:
    cp = strrchr(argv[0], '/');
    prog = cp == NULL ? argv[0] : cp + 1;
    
    fprintf(stderr, "Syntax:\n"); 
    fprintf(stderr, "   %s -conf file -return_poolindex  \"DN\" [attr1] [attr2] [...]\n", prog);
    fprintf(stderr, "   %s -conf file -return_poolindex  -count # \"DN\" [attr1] [attr2] [...]\n", prog);
    fprintf(stderr, "   %s -conf file -unlease_poolindex \"urlencoded_index\"\n", prog);
    fprintf(stderr, "   %s -conf file -unquarantine      \"local_acct\"\n", prog);
    ret = SYNTAX;
  
  exit_free:
    if (dn) free (dn);
    if (fqan_list) {
      int i;
      for (i = 0; i < nfqan; i++) {
        if (fqan_list[i]) free (fqan_list[i]);
      }
      free (fqan_list);
    }
    if (g_LCMAPS_DB_FILE) free (g_LCMAPS_DB_FILE);
    if (g_LCMAPS_DEBUG_LEVEL) free (g_LCMAPS_DEBUG_LEVEL);
    if (g_LCMAPS_GRIDMAPDIR) free (g_LCMAPS_GRIDMAPDIR);
    if (g_LCMAPS_LOG_FILE) free (g_LCMAPS_LOG_FILE);
    if (g_LCMAPS_LOG_STRING) free (g_LCMAPS_LOG_STRING);
    if (g_LCMAPS_POLICY_NAME) free (g_LCMAPS_POLICY_NAME);
    if (g_EXPIRY_PROG) free (g_EXPIRY_PROG);
    if (g_POOL_LOGFILE) free (g_POOL_LOGFILE);
    if (g_POOL_LOGSEP) free (g_POOL_LOGSEP);
    return ret;
  
} /* main() }}} */

/* __________________________________________________
   poolindex
   _______________________________________________{{{ */

int poolindex(char *dn, char *fqan_list[], int nfqan, int mapcount)
{
  int ret = -1;
  int i;
  char *gridmapdir = NULL;
  struct passwd *pwd = NULL;
  struct group *grp = NULL;
  char *dup;
  lcmaps_account_info_t lcmaps_account;
  
  debug_msg(8,"dn = %x",&dn);
  
  if (nfqan > 0) {
    log_msg("poolindex called with:\n");
    log_msg("     DN     = %s\n",dn);
    for (i = 0; i < nfqan; i++) {
      log_msg("     fqan%2d = %s\n",i+1,fqan_list[i]);
    }
    
  } else {
    log_msg("poolindex called with DN=%s and no FQANs\n",dn);
  }
  
  ret = lcmaps_poolindex(dn, fqan_list, nfqan, mapcount, &lcmaps_account, &gridmapdir);
  if (ret) {
    log_msg("call to LCMAPS fails\n");
    return ret;
  }
  
  /* communicate result in form:
   * poolindex
   * unixname
   * primgroup
   * secgrp1  <- if exists
   * secgrp2
   * ...
   */
   
   if (lcmaps_account.poolindex == NULL) {
     fprintf(stderr,"Call to LCMAPS succeeded, yet no poolindex?\n");
     log_msg("Call to LCMAPS succeeded, yet no poolindex?\n");
     return POOL_INDEX_FAILURE;
   }
   
   dup = safe_dup(lcmaps_account.poolindex);
   if (dup) {
     log_msg("found:\n");
     log_msg("     poolindex       =  %s\n",dup);
     printf("%s\n",dup);
   } else {
     return POOL_INDEX_FAILURE;
   }
      
   if (lcmaps_account.uid >= 0) {
     //log_msg("uid             =  %d\n",(int) lcmaps_account.uid);
     if ((pwd = getpwuid(lcmaps_account.uid)) == NULL) {
         fprintf(stderr, "uid %d does not exist\n", (int)lcmaps_account.uid);
         return GENERAL_FAILURE;
     }
     
     log_msg("     unixname        =  %s\n",pwd->pw_name);
     printf("%s\n",pwd->pw_name);
     //todo free memory in pwd fields
   } else {
     return POOL_INDEX_FAILURE;
   }
   
   if (lcmaps_account.npgid > 0) {
     if ((grp = getgrgid(lcmaps_account.pgid_list[0])) == NULL) {
         fprintf(stderr, "gid %d does not exist\n", lcmaps_account.pgid_list[0]);
         return GENERAL_FAILURE;
     }
     log_msg("     primary group   =  %s\n",grp->gr_name);
     printf("%s\n",grp->gr_name);
   } else {
     return POOL_INDEX_FAILURE;
   }
   
   
   if ((int)lcmaps_account.nsgid > 0) {
     /*log_msg("num secgroups   =  %d\n",(int)lcmaps_account.nsgid); */
     
     for (i = 0; i < lcmaps_account.nsgid; i++) {
       /*log_msg("secgroup %2d    =  %d\n", i+1, (int)lcmaps_account.sgid_list[i]);*/
       if ((grp = getgrgid(lcmaps_account.sgid_list[i])) == NULL) {
         fprintf(stderr, "gid %d does not exist\n", lcmaps_account.pgid_list[i]);
         return GENERAL_FAILURE;
       }
       log_msg("     secgroup %2d    =  %s\n",i+1,grp->gr_name);
       printf("%s\n",grp->gr_name);
       //todo: free memory in grp fields
     }
   }
     
  return SUCCESS;
  
} /* poolindex() }}} */

/* __________________________________________________
   delete with what you would pass to -return_poolindex
   _______________________________________________{{{ */
   
   /* disabling */
   
   /*
   
int delete_by_dn_attr(char *dn, char *fqan_list[], int nfqan)
{
  char *func = "delete_by_dn_attr()";
  int i;
  int ret;
  lcmaps_account_info_t lcmaps_account;
  char *gridmapdir = NULL;
  
  
  debug_msg(8,func);
  
  if (nfqan > 0) {
    log_msg("delete_by_dn_attr called with:\n");
    log_msg("     DN     = %s\n",dn);
    for (i = 0; i < nfqan; i++) {
      log_msg("     fqan%2d = %s\n",i+1,fqan_list[i]);
    }
    
  } else {
    log_msg("delete_by_dn_attr called with DN=%s and no FQANs\n",dn);
  }
  
  */
  /* this unfortunately secures a lease for millisecond if the lease
     does not exist !!  This interface (delete_by_dn_attr) is not used
     by GT4 service (that keeps track of straight poolindex instead).
   */
   
   /*
  ret = lcmaps_poolindex(dn, fqan_list, nfqan, &lcmaps_account, &gridmapdir);
  if (ret) {
    log_msg("call to LCMAPS fails\n");
    return ret;
  }
  
  debug_msg(5, "found poolindex: %s", lcmaps_account.poolindex);
  
  return delete_by_index(lcmaps_account.poolindex);
} */
/* delete_by_dn_attr() }}} */
  



/* __________________________________________________
   delete with known poolindex
   
   Called directly when using -unlease_poolindex subcommand.
   Workspace Service only uses this interface.
   _______________________________________________{{{ */
int delete_by_index(char *poolindex)
{
  char *args[3];
  pid_t pid;
  int status;
  struct stat statbuf;
  uid_t run_uid;

  int ret;
  unsigned short quarantine = QUARANTINE_BASE;
  uid_t uid = -1;
  struct passwd *pwd;
  char *func = "delete_by_index()";
  
  log_msg("deleting poolindex: %s\n", poolindex);
  
  /* always quarantine, even if just for the unlink */
  quarantine |= QUARANTINE;  
  if (g_DO_QUARANTINE == 1) {
    log_msg("   with quarantine\n");
  }

  /* unlink DN */
  
  ret = gridmapdir_unlink_index(poolindex, &uid, quarantine);
  if (ret) {
    debug_msg(8,"%s: gridmapdir_unlink_dn() fails",func);
    log_msg("gridmapdir_unlink_dn fails\n",func);
    log_msg("RESULT %d\n",ret);
    return ret; 
  }
  debug_msg(8,"%s: uid returned: %d",func,uid);
  
  if (uid < 100) {
    debug_msg(8,"%s: not cleaning or killing for user: %d",func,uid);
    log_msg("will not clean or kill for user: %d\n",uid);
    log_msg("RESULT %d\n",GENERAL_FAILURE);
    return GENERAL_FAILURE;
  }
  
  if ((pwd = getpwuid(uid)) == NULL) {
    fprintf(stderr, "uid %d not found\n", uid);
    log_msg("uid %d not found\n",uid);
    log_msg("RESULT %d\n",GENERAL_FAILURE);
    return GENERAL_FAILURE;
  }
  
  if (!pwd->pw_name) {
    debug_msg(1,"pw_name null");
    log_msg("uid %d not found\n",uid);
    log_msg("RESULT %d\n",GENERAL_FAILURE);
    return GENERAL_FAILURE;
  }
    
  /* At this point the lease has expired and the account should be returned
   * to an acceptable state.  */
  
  if (g_EXPIRY_PROG == NULL) {
    
    if (g_DO_QUARANTINE == 1) {
      log_msg("RESULT %d\n",SUCCESS);
      return SUCCESS;
    }
  
    /* release to pool */
    return unquarantine(pwd->pw_name);
  }
  
  /* Otherwise g_EXPIRY_PROG exists, attempt to clean account */

  /* sanity checks */
  
  /* 1. file exists and is executable? */
  ret = stat(g_EXPIRY_PROG, &statbuf);
  if (ret) {
    log_msg("Pool expiry program (%s) does not exist\n", g_EXPIRY_PROG);
    fprintf(stderr, "Pool expiry program (%s) does not exist\n", g_EXPIRY_PROG);
    log_msg("RESULT %d\n",EXPIRY_PROG_CHECK);
    log_msg("leaving account quarantined\n"); 
    fprintf(stderr,"leaving account quarantined\n"); 
    return EXPIRY_PROG_CHECK;
  }
  
  if (statbuf.st_mode & S_IFREG) {
    debug_msg(8,"%s: expiry script regular file",func);
  } else {
    log_msg("Pool expiry program (%s) not a regular file\n", g_EXPIRY_PROG);
    fprintf(stderr, "Pool expiry program (%s) not a regular file\n", g_EXPIRY_PROG);
    log_msg("RESULT %d\n",EXPIRY_PROG_CHECK);
    log_msg("leaving account quarantined\n"); 
    fprintf(stderr,"leaving account quarantined\n"); 
    return EXPIRY_PROG_CHECK;
  }
  
  if (statbuf.st_mode & S_IXUSR) {
    debug_msg(8,"%s: expiry script executable by owner",func);
  } else {
    log_msg("Pool expiry program (%s) not executable by owner\n", g_EXPIRY_PROG);
    fprintf(stderr, "Pool expiry program (%s) not executable by owner\n", g_EXPIRY_PROG);
    log_msg("RESULT %d\n",EXPIRY_PROG_CHECK);
    log_msg("leaving account quarantined\n"); 
    fprintf(stderr,"leaving account quarantined\n"); 
    return EXPIRY_PROG_CHECK;
  }
  
  
  /*  2. file is owned by runner and not modifiable by others? */
  if (g_RUN_EXPIRY_SETUID >= 0) {
      run_uid = g_RUN_EXPIRY_SETUID;
  } else {
      run_uid = g_real_uid;
  }

  if (statbuf.st_uid != run_uid) {
    log_msg("Pool expiry program (%s) not owned by %d\n", g_EXPIRY_PROG,run_uid);
    fprintf(stderr, "Pool expiry program (%s) not owned by %d\n", g_EXPIRY_PROG,run_uid);
    log_msg("RESULT %d\n",EXPIRY_PROG_CHECK);
    log_msg("leaving account quarantined\n"); 
    fprintf(stderr,"leaving account quarantined\n"); 
    return EXPIRY_PROG_CHECK;
  }
  debug_msg(8,"%s: expiry script st_mode: %x",func,statbuf.st_mode);
  
  if (statbuf.st_mode & S_IWGRP) {
    log_msg("Pool expiry program (%s) writable by group\n", g_EXPIRY_PROG);
    fprintf(stderr, "Pool expiry program (%s) writable by group\n", g_EXPIRY_PROG);
    log_msg("RESULT %d\n",EXPIRY_PROG_CHECK);
    log_msg("leaving account quarantined\n"); 
    fprintf(stderr,"leaving account quarantined\n"); 
    return EXPIRY_PROG_CHECK;
  } else {
    debug_msg(8,"%s: expiry script not writable by group",func);
  }
  if (statbuf.st_mode & S_IWOTH) {
    log_msg("Pool expiry program (%s) writable by world\n", g_EXPIRY_PROG);
    fprintf(stderr, "Pool expiry program (%s) writable by world\n", g_EXPIRY_PROG);
    log_msg("RESULT %d\n",EXPIRY_PROG_CHECK);
    log_msg("leaving account quarantined\n"); 
    fprintf(stderr,"leaving account quarantined\n"); 
    return EXPIRY_PROG_CHECK;
  } else {
    debug_msg(8,"%s: expiry script not writable by world",func);
  }
  
  
  args[0] = g_EXPIRY_PROG;
  args[1] = pwd->pw_name;
  args[2] = NULL;
  
  debug_msg(3,"%s: Expiring via: %s %s",func, g_EXPIRY_PROG, pwd->pw_name);
 
  if ((pid = fork()) == -1)
  {
    perror("fork");
  }
  else if (pid)
  {
    while (waitpid(pid, &status, 0) == -1 && errno == EINTR);
  }
  else
  {
    
    if (g_RUN_EXPIRY_SETUID >= 0) {
      seteuid(g_RUN_EXPIRY_SETUID);
      setuid(g_RUN_EXPIRY_SETUID);
    }
    execv(args[0], args);
    perror("execv");
    log_msg("leaving account quarantined\n"); 
    fprintf(stderr,"leaving account quarantined\n"); 
    return GENERAL_FAILURE;
  }
  
  /* translate return value */
  if (WIFEXITED(status)) {
    debug_msg(3,"%s: Pool expiry program exit code %d", func, WEXITSTATUS(status));
    
    if (WEXITSTATUS(status)) {
      fprintf(stderr, "Pool expiry program exit code %d.\n", WEXITSTATUS(status));
      log_msg("RESULT: %d\n",EXPIRY_PROG_FAILURE);
      log_msg("leaving account quarantined\n"); 
      fprintf(stderr,"leaving account quarantined\n"); 
      return EXPIRY_PROG_FAILURE;
    } else {
      
      if (g_DO_QUARANTINE == 1) {
        log_msg("RESULT %d\n",SUCCESS);
        return SUCCESS;
      }
      
      /* release to pool */
      return unquarantine(pwd->pw_name);
    }
  } else {
    if (WIFSIGNALED(status)) {
      fprintf(stderr, "Pool expiry program died by signal %d", WTERMSIG(status));
    }         
    log_msg("RESULT: %d\n",EXPIRY_PROG_FAILURE);
    log_msg("leaving account quarantined\n"); 
    fprintf(stderr,"leaving account quarantined\n"); 
    return EXPIRY_PROG_FAILURE;
  }
  
    
}  /* delete_by_index }}} */

/* __________________________________________________
   unquarantine
   _______________________________________________{{{ */
   
int unquarantine(char *input) {
  char *func = "exec_unquarantine()";
  char *quarantinesuffix = QUARANTINE_SUFFIX;
  char *quarpath = NULL;
  char *otherlnk = NULL;
  struct stat statbuf;
  int ret;
  int nsuffix;
  int nother;
  
  debug_msg(8,func);
  
  /* As an example, a lease is secured by hardlinking "pool001" with the
     urlencoded DN+attrs = %2fo%3ddutchgrid%2f
     
     In .conf, user chooses to secure quarantine by username or encoded
     lease name + pool username.
     So either "pool001.quarantined" or "%2fo%3ddutchgrid%2f.pool001.quarantined"
     is the hardlink.
     
     The unquarantine code receives "pool001", looks at the hardlink to pool001, and
     unquarantines if this other file has the quarantine suffix.
          
  */
  
  if (input == NULL) { 
    fprintf(stderr, "null arg to exec_unquarantine\n");
    return GENERAL_FAILURE;
  } else {
    log_msg("exec_unquarantine called with input=%s\n",input);
  }
  
  /* search for an 'otherlink' of /mapdir/input */
     
  otherlnk = otherlink(input,g_gridmapdir);
  if (!otherlnk) {
    debug_msg(3,"%s can't remove: no quarantine for: %s",func,input);
    fprintf(stderr,"can't remove: no quarantine for: %s\n",input);
    return GRIDMAPDIR_NO_ASSOCIATION;
  }
  
  /* Found something linked to /mapdir/input, see if it is suffixed
     with quarantine suffix */
  debug_msg(8,"looking at %s",input);
  debug_msg(8,"found otherlink %s",otherlnk);
  
  nsuffix = strlen(quarantinesuffix);
  debug_msg(8,"characters in suffix %s: %d",quarantinesuffix,nsuffix);
  
  nother = strlen(otherlnk);
  debug_msg(8,"characters in otherlink %s: %d",otherlnk,nother);
  
  if (nsuffix > nother) {
    debug_msg(8,"nsuffix > nother");
    debug_msg(3,"%s can't remove: no quarantine for: %s",func,input);
    fprintf(stderr,"can't remove: no quarantine for: %s\n",input);
    return GRIDMAPDIR_NO_ASSOCIATION;
  }
  
  debug_msg(8,"otherlink + %d chars: %s",nother-nsuffix,otherlnk + (nother-nsuffix) );
  
  ret = strncmp(otherlnk + (nother-nsuffix), quarantinesuffix, nsuffix);
  if (ret) {
    debug_msg(8,"no suffix");
    debug_msg(3,"%s can't remove: no quarantine for: %s",func,input);
    fprintf(stderr,"can't remove: no quarantine for: %s\n",input);
    return GRIDMAPDIR_NO_ASSOCIATION;
  }
  
  /* if equal, then otherlnk is the file to delete */
  
  free(quarpath);
  quarpath = NULL;
  
  quarpath = (char *) malloc(strlen(g_gridmapdir) + 2 + strlen(otherlnk));
  if (!quarpath) {
    log_msg("%s problem, malloc",func);
    fprintf(stderr,"%s problem, malloc\n",func);
    return GENERAL_FAILURE;
  }
  sprintf(quarpath, "%s/%s", g_gridmapdir,otherlnk);
  debug_msg(8,"%s new quarpath: %s",func,quarpath);
  ret = stat(quarpath, &statbuf);
  if (ret) {
    log_msg("%s: cannot unquarantine %s, file does not exist: %s",func, input, quarpath);
    fprintf(stderr,"cannot unquarantine %s, file does not exist: %s\n", input, quarpath);
    return BADARG;
  }

  if ((statbuf.st_nlink > 2) || (statbuf.st_nlink < 2)) {
    log_msg("%s problem, unquarantine file %s has %d links",func, quarpath, statbuf.st_nlink);
    fprintf(stderr,"%s problem, unquarantine file %s has %d links\n",func, quarpath, statbuf.st_nlink);
    return ENVIRONMENT;
  }
  
  ret = unlink(quarpath);
  if (ret) { 
    debug_msg(1,"%s: cannot unlink %s", func, quarpath);
    fprintf(stderr,"%s: cannot unlink %s\n", func, quarpath);
    return GENERAL_FAILURE;
  }
  debug_msg(8,"%s: unlinked %s",func,quarpath);
  log_msg("unlinked %s\n",quarpath);
  
  return SUCCESS;
} /* unquarantine() }}} */



