/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */

#include <stdio.h>
#include <ctype.h>
#include <dirent.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <unistd.h>
#include <time.h>

#include "log.h"
#include "util.h"

/* __________________________________________________
   log_init
   __________________________________________________ */

int log_init() {
  debug_msg(8,"log_init()");
  
  if (g_POOL_LOGFILE != NULL) {

    if ((g_log_fp = fopen(g_POOL_LOGFILE, "a")) == NULL) {
      return -1;
    } else if (g_POOL_LOGSEP != NULL) {
      fprintf(g_log_fp,g_POOL_LOGSEP);
      fprintf(g_log_fp,"\n\n");
      return 0;
    }
  }
  g_log_fp = NULL;  /* causes no-op in log_msg */
  return 0;
} /* log_init() */

/* __________________________________________________
   log_msg
   (based on lcmaps)
   __________________________________________________ */

void log_msg(char *fmt, ...) {
  va_list     pvar;
  char        buf[MAX_LOG_BUFFER_SIZE];
  char *      datetime = NULL;
  char *      tmpbuf = NULL;
  int         res;
  
  /*debug_msg(8,"log_msg()");*/
  
  if (g_log_fp == NULL)
  {
    /* NO-OP, see log_init(), no POOL_LOGFILE defined */
    return;
  }
  
  if (fmt == NULL) {
    fprintf(stderr,"log_msg() error: no message\n");
    return;
  }
  
  va_start(pvar, fmt);
  res=vsnprintf(buf,MAX_LOG_BUFFER_SIZE,fmt,pvar);
  va_end(pvar);
  if ( (res >= MAX_LOG_BUFFER_SIZE) || (res < 0) )
  {
    fprintf(stderr,"debug_msg(): string too long (> %d)\n",
            MAX_LOG_BUFFER_SIZE);
  }
  
  datetime = get_time();
  
  /*debug_msg(8,"time: %s",datetime);*/

  tmpbuf = (char *) malloc ((strlen(datetime) + strlen(buf) + strlen(" : ")) * sizeof(char) + 1);
  strcpy(tmpbuf, datetime);
  strcat(tmpbuf, " : ");
  strcat(tmpbuf, buf);
  
  fprintf(g_log_fp,"%s", tmpbuf);
  /*debug_msg(5,"logging: %s",tmpbuf);*/
  
  fflush(g_log_fp);

  if (datetime) free(datetime);
  if (tmpbuf) free(tmpbuf);
  
}  /* log_msg() */


/* __________________________________________________
   debug_init
   (from lcmaps)
   __________________________________________________ */

int debug_init() {
  char * debug_env_ = NULL;
  char * debug_env = NULL;
  
  g_debug_fp = (FILE *) DEBUG_FD;
  
  if ( (debug_env_ = getenv("POOL_DEBUG_LEVEL")) )
  {
      /* convert into integer */
      int j = 0;
      
      debug_env = safe_dup(debug_env_);

      for (j = 0; j < strlen(debug_env); j++)
      {
          if (!isdigit((debug_env)[j]))
          {
              fprintf(stderr,"debug_init(): found non-digit in environment variable in \"POOL_DEBUG_LEVEL\" = %s\n", debug_env);
              free(debug_env);
              return 1;
          }
      }
      g_debug_level = atoi(debug_env);
      if (g_debug_level < 0)
      {
          fprintf(stderr,"debug_init(): environment variable in \"POOL_DEBUG_LEVEL\" should be >= 0\n");
          free(debug_env);
          return 1;
      }
  }
  else if (g_DEBUG_LEVEL > 0 ) 
  {
    g_debug_level = g_DEBUG_LEVEL;
  } 
  else
  {
      g_debug_level = 0;
  }
          
  if(debug_env) free(debug_env);
  
  return 0;

} /* debug_init() */

/* __________________________________________________
   debug_msg
   (from lcmaps)
   __________________________________________________ */

void debug_msg(int debuglevel, char *fmt, ...) {
  va_list pvar;
  char    buf[MAX_LOG_BUFFER_SIZE];
  int     res;
  
  if (fmt == NULL) {
    fprintf(stderr,"debug_msg() error: no message\n");
    return;
  }
  
  va_start(pvar, fmt);
  res=vsnprintf(buf,MAX_LOG_BUFFER_SIZE,fmt,pvar);
  va_end(pvar);
  if ( (res >= MAX_LOG_BUFFER_SIZE) || (res < 0) )
  {
    fprintf(stderr,"debug_msg(): string too long (> %d)\n",
            MAX_LOG_BUFFER_SIZE);
  }

  if (debuglevel <= g_debug_level) {
    fprintf(g_debug_fp,"%d: %s\n", debuglevel, buf);
    fflush(g_debug_fp);
  } 
} /* debug_msg() */

/* __________________________________________________
   get_time
   caller must release mem malloced
   __________________________________________________ */
char * get_time() {
  int         res;
  time_t      clock;
  struct tm * tmp = NULL;
  char *      tmpbuf = NULL;
  
  time(&clock);
  tmp = localtime(&clock);

  tmpbuf = malloc(sizeof(char) * 20);

  res=snprintf(tmpbuf, 20, "%04d-%02d-%02d.%02d:%02d:%02d",
         tmp->tm_year + 1900, tmp->tm_mon + 1, tmp->tm_mday,
         tmp->tm_hour, tmp->tm_min, tmp->tm_sec);
  if ( (res >= 20) || (res < 0) )
  {
    fprintf(stderr,"log_msg(): date string too long (> %d)\n", 20);
  }
  
  return tmpbuf;
  
}
