"""
Portions of this file Copyright 1999-2005 University of Chicago
Portions of this file Copyright 1999-2005 The University of Southern California.

This file or a portion of this file is licensed under the
terms of the Globus Toolkit Public License, found at
http://www.globus.org/toolkit/download/license.html.
If you redistribute this file, with or without
modifications, you must include this notice in the file.
"""
import unittest, random, os, sys, re, time, string, urllib

# Comment out the next line to remove the question at the beginning
ASKQUESTION = ""

TESTCONF = "test.conf"
EXE = "../source/lcmaps_poolindex"
POOL = "gpool"
NUMPOOL = 10
INFO = None
NOEDITDIR = "do-not-edit"
BOGUS_EXPIRY_PROG = "bogus_expire.sh"

# args
ARG_CONF = "-conf"
ARG_POOLINDEX = "-return_poolindex"
ARG_UNLEASE = "-unlease_poolindex"
ARG_UNQUAR = "-unquarantine"

# keys in conf file
DO_QUAR = "do quarantine"
QUAR_BY_INDEX = "quarantine by poolindex"
LOGFILE = "pool logfile"
LOGSEP = "log separator"
DEBUG_LEVEL = "debug level"
LCMAPS_LOG_FILE = "LCMAPS_LOG_FILE"
LCMAPS_DB_FILE = "LCMAPS_DB_FILE"
LCMAPS_GRIDMAPDIR = "LCMAPS_GRIDMAPDIR"
LCMAPS_DEBUG_LEVEL = "LCMAPS_DEBUG_LEVEL"
LCMAPS_POLICY_NAME = "LCMAPS_POLICY_NAME"
LCMAPS_LOG_STRING = "LCMAPS_LOG_STRING"
EXPIRY_PROG = "expiry program"
EXPIRY_SETUID = "run expiry program setuid"

#########################
# class: LCMAPSTestCase #
#########################

class LCMAPSTestCase(unittest.TestCase):
    ###########
    # setUp() #
    ###########
    def setUp(self):
        """Run before every single test"""
        if not INFO: raise NoInfo("No information object present, not using main()?")
        INFO.testnum += 1
        print ""
        print "---- Test #%d, setting up ----\n" % INFO.testnum
        self.something = "hi"
        self.clearGridmapdir()
        self.makePool()
        
    ##############
    # tearDown() #
    ##############
    def tearDown(self):
        """Run after every single test"""
        self.something = None
        print "\n---- Test #%d is done ----\n" % INFO.testnum
       
    #####################
    # clearGridmapdir() #
    #####################
    def clearGridmapdir(self):
        print "  - Wiping gridmapdir."
        # assuming we don't have to os.walk over any dirs under gridmapdir
        for f in os.listdir(INFO.gridmapdir):
            os.remove(os.path.join(INFO.gridmapdir, f))
            
    ##############
    # makePool() #
    ##############
    def makePool(self):
        print "  - Creating new pool."
        for i in range(NUMPOOL):
            num = string.zfill(i,3) # using %3d print mask doesn't get the zeroes
            name = "%s%s" % (POOL,num)
            f = open(os.path.join(INFO.gridmapdir, name), "w")
            f.close()
            
    ##############
    # findPool() #
    ##############
    def findPool(self, gridmapfile):
        """Returns DN:pool map from the gridmapfile"""
        pattern = r'\s*\"(.*?)\" (.*)' # use non-greedy for dn
        beg = re.compile(pattern)
        f = open(gridmapfile,"r")
        dnpools = []
        lines = f.readlines()
        for line in lines:
            mobj = beg.search(line)
            if mobj:
                # only include pools
                if mobj.groups()[1][0] == ".":
                    dnpool = [mobj.groups()[0], mobj.groups()[1]]
                    dnpools.append(dnpool)
        f.close()
        if len(dnpools) > 0:
            for i in dnpools:
                print "  - Found pool '%s' for DN '%s'" % (i[1],i[0])
            return dnpools
        else:
            return None
            
    ##################
    # checkForPool() #
    ##################
    def checkForPool(self, pool):
        """Checks if file exists in gridmapdir"""
        poolen = len(pool)
        for f in os.listdir(INFO.gridmapdir):
            if f[:poolen] == pool:
                print "  - Pool '%s' exists" % pool
                return True
        return False
        
    ###############
    # urlencode() #
    ###############
    def urlencode(self,text):
        """Python's urllib.urlencode() annoyingly only takes keypairs."""
        return urllib.quote_plus(text)
        
    #################
    # unurlencode() #
    #################
    def unurlencode(self,text):
        """Python's urllib.urlencode() annoyingly only takes keypairs."""
        return urllib.unquote_plus(text)
        
    #########
    # run() #
    #########
    def run(self,command):
        """Run command and return stdout (line by line) and exit status."""
        print "  - Attempting command: " + command
        ret = os.popen(command)
        stdout = ret.read()
        code = ret.close()
        stdoutlines = []
        for line in stdout.split("\n"):
            stdoutlines.append(line)
        if not code:
            code = 0
        else:
            code = code >> 8
        print "  - exit code: " + str(code)
        # stripping last line, which is empty
        stdoutlines = stdoutlines[:-1]
        return [code,stdoutlines]
        
################################
# class: TestGridmapdirEntries #
################################

class TestGridmapdirEntries(LCMAPSTestCase):
    
    ############
    # testDN() #
    ############
    def testDN(self):
        """Check for a valid DN -> pool mapping in the configured grid-mapfile, attempt to lease by DN, check for lease, and then unlease, checks for no lease"""
        print "  ### Testing lease by DN. ###"
        dnpoolmap = self.findPool(INFO.gridmapfile)
        assert dnpoolmap, "No DN to pool mappings for any DNs in grid-mapfile '%s'" % INFO.gridmapfile
        # pick a DN at random
        idx = random.choice(range(0, len(dnpoolmap), 1))
        dn = dnpoolmap[idx][0]
        pool = dnpoolmap[idx][1]
        assert self.checkForPool(pool[1:]), "Pool in grid-mapfile does not exist: '%s'" % pool
        expectedlease = self.urlencode(dn)
        command = "%s %s %s %s \"%s\"" % (EXE,ARG_CONF,TESTCONF,ARG_POOLINDEX,dn)
        result = self.run(command)
        assert result[0] == 0, "poolindex fails with exit code: %d" % result[0]
        stdout = result[1]
        assert len(stdout) >= 3, "Did not receive 3 pieces of information via stdout"
        print "  - poolindex reportedly successful"
        poolindex = stdout[0]
        print "  - received poolindex: " + poolindex
        assert dn.lower() == self.unurlencode(poolindex), "poolindex is not the URL encdoded DN"
        print "  - poolindex is the urlencoded DN"
        print "  - account: " + stdout[1]
        print "  - group: " + stdout[2]
        
    ############
    # test2() #
    ############
    #def test2(self):
    #    assert 1 == 1, 'one doesnt equal one'
        
    ############
    # test3() #
    ############
    #def test3(self):
    #    assert 1 == 1, 'one doesnt equal one'
        
#################
# class: NoInfo #
#################

class NoInfo(Exception):
    def __init__(self, msg):
        self.msg = msg    
        
######################
# class: information #
######################

class information:
    """hack to only get information once, not the right pyunit way, as
    it makes the tests need to be run from a custom main().  This mostly
    means you can't use the pyunit GUI if you wanted to."""
    
    def __init__(self):
        print "\n-> Initializing:"
        self.testnum = 0
        # test config file must be present, sending the C program
        # a bogus config file is a unit test
        if not os.path.exists(TESTCONF):
            errorstr = "     - Test config file '%s' does not exist.\n     Redefine TESTCONF at the top of this file or create file '%s' in this directory.\n" % (TESTCONF,TESTCONF)
            raise NoInfo(errorstr)
            
        # let exceptions fly:
        try:
            f = open(TESTCONF, "r")
            self.gridmapdir = self.parseArg(f,LCMAPS_GRIDMAPDIR)
            self.do_quarantine = self.getBooleanArg(f,DO_QUAR)
            self.do_quarantine_index = self.getBooleanArg(f,QUAR_BY_INDEX)
            self.logfile = self.parseArg(f,LOGFILE)
            self.lcmapslogfile = self.parseArg(f,LCMAPS_LOG_FILE)
            self.lcmapsDBfile = self.parseArg(f,LCMAPS_DB_FILE)
            self.lcmapsdebug = self.parseArg(f,LCMAPS_DEBUG_LEVEL)
            self.lcmapspolicy = self.parseArg(f,LCMAPS_POLICY_NAME)
            self.lcmapslogstring= self.parseArg(f,LCMAPS_POLICY_NAME)
            self.expiryprog = self.parseArg(f,EXPIRY_PROG)
            self.expiryuid = self.parseArg(f,EXPIRY_SETUID)
            self.gridmapfile = None
            self.groupmapfile = None
        finally:
            f.close()
        print "\n     executable           =", self.pp(EXE)
        print "     configuration file   =", self.pp(TESTCONF)
        print "     gridmapdir           =", self.pp(self.gridmapdir)
        print "     expect quarantine?   =", self.pp(self.do_quarantine)
        print "     quarantine by index? =", self.pp(self.do_quarantine_index)
        print "     logfile              =", self.pp(self.logfile)
        print "     LCMAPS logfile       =", self.pp(self.lcmapslogfile)
        print "     LCMAPS log string    =", self.pp(self.lcmapslogstring)
        print "     LCMAPS DB file       =", self.pp(self.lcmapsDBfile)
        print "     LCMAPS debug level   =", self.pp(self.lcmapsdebug)
        print "     LCMAPS policy name   =", self.pp(self.lcmapspolicy)
        print "     expiry program       =", self.pp(self.expiryprog)
        print "     expiry setuid uid    =", self.pp(self.expiryuid)

        print "\n-> Validating:"
        # throws NoInfo exception if something is invalid or missing
        print self.validate()
        print "\n   Result:"
        print "   -------"
        print "   Configuration is valid."
                
        print "\n-> From DB file:\n"
        print "     grid-mapfile  =", self.pp(self.gridmapfile)
        print "     group-mapfile =", self.pp(self.groupmapfile)
        print "\n-> Done.\n"
    
        
    ##############
    # validate() #
    ##############

    def validate(self):
        """Returns without exception if all necessary settings are present and valid"""
        okstr = "\n   Valid:\n   ------\n"
        errorstr = "\n   Invalid:\n   --------\n"
        tripexception = False
        
        # 1. gridmapdir must be present
        if not self.gridmapdir:
            errorstr += "     - Test gridmapdir is not set.\n"
            tripexception = True            
        elif not os.path.exists(self.gridmapdir):
            errorstr += "     - Test gridmapdir '%s' does not exist.\n" % self.gridmapdir
            tripexception = True
        else:
            okstr += "     - gridmapdir exists.\n"
            
        # 2. gridmapdir must be writable, since C program does 
        #    not call LCMAPS setuid
            try:
                tmp = "tmpfile"+str(time.time())
                tmp = os.path.join(self.gridmapdir,tmp)
                f = open(tmp,"w")
                f.close()
            except:
                exception_type = sys.exc_type
                try:
                    exceptname = exception_type.__name__ 
                except AttributeError:
                    exceptname = exception_type
                errorstr += "     - Problem writing to gridmapdir: %s: %s\n" % (str(exceptname), str(sys.exc_value))
                tripexception = True
            else:
                okstr += "     - gridmapdir writable.\n"
            
        # 3. LCMAPS DB file exists?
        #    This will be caught by the C program, but why make the user hunt
        #    down this error as the unit tests fail?  There are separate unit
        #    tests for bogus and missing configuration options that are required.
        if not self.lcmapsDBfile:
            errorstr += "     - LCMAPS DB file is not set.\n"
            tripexception = True        
        elif not os.path.exists(self.lcmapsDBfile):
            errorstr += "     - LCMAPS DB file '%s' does not exist.\n" % self.lcmapsDBfile
            tripexception = True        
        else:
            okstr += "     - LCMAPS DB file exists.\n"
            
        # 4. readable?
            try:
                f = open(self.lcmapsDBfile, "r")
            except:
                exception_type = sys.exc_type
                try:
                    exceptname = exception_type.__name__ 
                except AttributeError:
                    exceptname = exception_type
                errorstr += "     - Problem reading test LCMAPS DB file: %s: %s\n" % (str(exceptname), str(sys.exc_value))
                tripexception = True
            else:
            
        # 5. Does DB file have matching gridmapdirs as the env variable we 
        #    pass LCMAPS via the configuration file's LCMAPS_GRIDMAPDIR setting?
        #    If so, then these tests will test for the wrong things.
                
                # only handling case where all of the LCMAPS policies are pointing to
                # the same gridmapdir! 
                found = self.parseLCMAPSDBArg(f,"-gridmapdir")
                if len(found) == 0:
                    errorstr += "     - Problem reading test LCMAPS DB file: cannot find -gridmapdir settings\n"
                    tripexception = True
                else:
                    found = map(string.strip, found)
                    foundstr = ', '.join(found)
                    okstr += "     - Found %d gridmapdir settings in LCMAPS DB file: '%s'\n" % (len(found),foundstr)
                    onebad = False
                    for i in found:
                        if i != self.gridmapdir:
                            onebad = True
                    if not onebad:
                        okstr += "     - gridmapdir settings in LCMAPS DB file all match configuration file gridmapdir setting\n"
                    else:
                        errorstr += "     - gridmapdir settings in LCMAPS DB file do NOT all match configuration file gridmapdir setting, can't handle this\n"
                        tripexception = True
                        
        # 6. Does DB file have matching grid-mapfiles?
                
                # only handling case where all of the LCMAPS policies are pointing to
                # the same grid-mapfile! 
                found = self.parseLCMAPSDBArg(f,"-gridmapfile")
                if len(found) == 0:
                    errorstr += "     - Problem reading test LCMAPS DB file: cannot find -gridmapfile settings\n"
                    tripexception = True
                else:
                    found = map(string.strip, found)
                    foundstr = ', '.join(found)
                    okstr += "     - Found %d gridmapfile settings in LCMAPS DB file: '%s'\n" % (len(found),foundstr)
                    self.gridmapfile = found[0]
                    onebad = False
                    for i in found:
                        if i != self.gridmapfile:
                            onebad = True
                    if not onebad:
                        okstr += "     - gridmapfile settings in LCMAPS DB file are all the same file\n"
                    else:
                        errorstr += "     - gridmapfile settings in LCMAPS DB file are NOT all the same, can't handle this\n"
                        tripexception = True
                        
        # 7. Does DB file have matching group-mapfiles?
                
                # only handling case where all of the LCMAPS policies are pointing to
                # the same grid-mapfile! 
                found = self.parseLCMAPSDBArg(f,"-groupmapfile")
                if len(found) == 0:
                    errorstr += "     - Problem reading test LCMAPS DB file: cannot find -groupmapfile settings\n"
                    tripexception = True
                else:
                    found = map(string.strip, found)
                    foundstr = ', '.join(found)
                    okstr += "     - Found %d groupmapfile settings in LCMAPS DB file: '%s'\n" % (len(found),foundstr)
                    self.groupmapfile = found[0]
                    onebad = False
                    for i in found:
                        if i != self.groupmapfile:
                            onebad = True
                    if not onebad:
                        okstr += "     - groupmapfile settings in LCMAPS DB file are all the same file\n"
                    else:
                        errorstr += "     - -groupmapfile settings in LCMAPS DB file do NOT all match, can't handle this\n"
                        tripexception = True                        

        # 8. All other LCMAPS settings present? 
        #    (if values invalid in some other way, LCMAPS will throw an error)
        if not self.lcmapslogfile:
            errorstr += "     - LCMAPS log file is not set.\n"
            tripexception = True              
        elif not os.path.exists(self.lcmapslogfile):
            errorstr += "     - LCMAPS log file '%s' does not exist.\n" % self.lcmapslogfile
            tripexception = True
        else:
            okstr += "     - LCMAPS log file exists.\n"
            
        if not self.lcmapslogstring:
            errorstr += "     - LCMAPS log string is not set.\n"
            tripexception = True              
        else:
            okstr += "     - LCMAPS log string exists.\n"       
            
        if not self.lcmapsdebug:
            errorstr += "     - LCMAPS debug level is not set.\n"
            tripexception = True              
        else:
            okstr += "     - LCMAPS debug level exists.\n"     
            
        if not self.lcmapspolicy:
            errorstr += "     - LCMAPS policy name is not set.\n"
            tripexception = True              
        else:
            okstr += "     - LCMAPS policy name exists.\n"    
            
        # 9.  Executable exists and is executable?
        if not EXE:
            errorstr += "     - executable is not set.\n"
            tripexception = True              
        elif not os.path.exists(EXE):
            errorstr += "     - executable '%s' does not exist.\n" % EXE
            tripexception = True 
        else:
            try:
                # will even work in a setuid environment
                x = os.access(EXE, os.X_OK)
                if x:
                    okstr += "     - '%s' is executable.\n" % EXE
                else:
                    errorstr += "     - '%s' is NOT executable.\n" % EXE
                    tripexception = True
            except:
                exception_type = sys.exc_type
                try:
                    exceptname = exception_type.__name__ 
                except AttributeError:
                    exceptname = exception_type
                errorstr += "     - Problem testing executable: %s: %s\n" % (str(exceptname), str(sys.exc_value))
                tripexception = True            
                
        # 10. Expiry program set?
        #    We do not get into testing that for real, just use the bogus script
        #    in the do-not-edit directory.
        #    Don't re-write if there is another problem.
        if not tripexception:
            repl = False
            add = False
            if self.expiryprog:
                default = os.path.join(NOEDITDIR,BOGUS_EXPIRY_PROG)
                if self.expiryprog != default:
                    # do not trip exception unless we can't replace it
                    errorstr += "     - Expiry program is set, replacing setting with '%s'" % self.expiryprog
                    repl = True
            else:
                self.expiryprog = os.path.join(NOEDITDIR,BOGUS_EXPIRY_PROG)
                # do not trip exception unless we can't replace it
                errorstr += "     - Expiry program is not set, adding '%s'" % self.expiryprog
                add = True
            try:
                new = "\n" + EXPIRY_PROG + " = " + self.expiryprog + "\n"
                if repl and not add:
                    before = r'\s*'
                    re_ = re.compile(before+EXPIRY_PROG)
                    self.replaceFileLines(TESTCONF, re_, new)
                elif add and not repl:
                    self.addFileLine(TESTCONF, new)
                else:
                    okstr += "     - Expiry program is set, matches default already"
            except:
                exception_type = sys.exc_type
                try:
                    exceptname = exception_type.__name__ 
                except AttributeError:
                    exceptname = exception_type
                errorstr += "     - Problem re-writing configuration file: %s: %s\n" % (str(exceptname), str(sys.exc_value))
                tripexception = True
        
        if tripexception:
            raise NoInfo(okstr+errorstr)
        else:
            if errorstr == "\n   Invalid:\n   --------\n":
                return okstr
            else:
                return okstr+errorstr

                
    #################
    # addFileLine() #
    #################
            
    def addFileLine(self,path,new):
        """add line at end"""
        try:
            f = open(path,'a')
            f.write(new)
        finally:
            f.close()
            
    ######################
    # replaceFileLines() #
    ######################
            
    def replaceFileLines(self,path,regex,new):
        """matches line against regex (using match, not search), replaces with 
        entirely new line; I've seen a slicker way to do this with fileinput"""
        f = open(path)
        try:
            lines = f.readlines()
            idx = 0
            for line in lines:
                mobj = regex.match(line)
                if mobj:
                    lines[idx] = new
                idx += 1
            f.close()
            f = open(path, 'w')
            for line in lines:
                f.write(line)
        finally:
            f.close()
            
    ########
    # pp() #
    ########
            
    def pp(self,x):
        """Pretty print a string vs. other"""
        if x == True or x == None or x == False:
            return " " + str(x)
        else:
            return "'" + str(x) + "'"

    ######################
    # parseLCMAPSDBArg() #
    ######################

    def parseLCMAPSDBArg(self, f, key):
        """Given a key and LCMAPS DB file, return list of all the values matching key"""
        after = r' (.*?)\"' # use non-greedy
        beg = re.compile(key+after)
        f.seek(0)
        lines = f.readlines()
        found = []
        for line in lines:
            mobj = beg.search(line)
            if mobj:
                found.append(mobj.groups()[0])        
        return found
        
    ##############
    # parseArg() #
    ##############
        
    def parseArg(self, conf, key):
        """Given a key and conf file, return the value"""
        
        before = r'\s*'
        after = r'\s*=\s*(.*)'
        beg = re.compile(before+key+after)
        
        conf.seek(0)
        lines = conf.readlines()
        found = None
        for line in lines:
            mobj = beg.match(line)
            if mobj:
                found = mobj.groups()[0]
                break
        if not found: 
            return None
        else:
            # regex doesn't catch trailing whitespace
            return found.strip()
            
    ###################
    # getBooleanArg() #
    ###################
            
    def getBooleanArg(self, conf, key):
        """Given a key and conf file, return the boolean translated to Python"""
        
        ret = self.parseArg(conf,key)
        
        # Currently, all lcmaps_poolindex boolean configs default to false.
        if not ret:
            return False # only for pretty print, False == None ... 
        ret = ret.lower()
        # already stripped by parseArg
        if ret == "true":
            return True
        else:
            return False

##########
# main() #
##########

if __name__ == '__main__':
    try:
        INFO = information()
    except NoInfo,err:
        print err.msg
        print "\n   Result:"
        print "   -------"
        print "\n   Initialization problem:"
        sys.exit(1)
        
    try:
        ASKQUESTION
    except:
        # ASKQUESTION does not exist
        pass
    else:
        resp = raw_input("""
WARNING:
This could have serious effects depending on your configuration, 
including leasing accounts from valid pools.

This program deletes ALL files in the configured gridmapdir for
every test.  If your gridmapdir is set to a writable but invalid
directory, it will delete ALL of those files.

You can disable this question by commenting out the ASKQUESTION 
variable at the top of this program.

Continue with these settings? (y/n) """)
        if not resp:
            print "You didn't respond, implying no."
            sys.exit(1)
        elif resp[0] == "n" or resp[0] == "N":
            print "You choose no."
            sys.exit(1)
        elif resp[0] == "y" or resp[0] == "Y":
           print "-> You responded yes."
           pass
        else:
            print "Response not y/n, implying no."
            sys.exit(1)
            
    print "-> Starting tests:"
    unittest.main()
    
