/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.workspace.impl.authz;

import java.util.ArrayList;
import java.util.Iterator;

import java.io.IOException;
import java.io.Serializable;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

/**
 * @see org.globus.workspace.impl.service.WorkspaceHome#create(org.globus.workspace.generated.types.AccountInformation, String, String, String, org.apache.axis.types.Duration, org.apache.axis.types.Duration)
 */
public class VomsPDPPolicy implements Serializable {

    private ArrayList DNs = null; //todo: Set?
    private ArrayList attrs = null;

    public VomsPDPPolicy(String[] _DNs, String[] _attrs) {
        this.DNs = new ArrayList();
        if (_DNs != null) {
            for (int i = 0; i < _DNs.length; i++) {
                this.DNs.add(_DNs[i]);
            }
        }
        this.attrs = new ArrayList();
        if (_attrs != null) {
            for (int i = 0; i < _attrs.length; i++) {
                this.attrs.add(_attrs[i]);
            }
        }
    }
    // encapsulate details of storage method
    public Iterator getDNs() {
        return this.DNs.iterator();
    }

    public Iterator getAttrs() {
        return this.attrs.iterator();
    }

    /**
     * @param DN The DN to add to policy.
     * @return true if policy was altered.
     */
    public boolean addDN(String DN) {
        if (!this.DNs.contains(DN)) {
            this.DNs.add(DN);
            return true;
        }
        return false;
    }

    /**
     * @param DN The DN to remove from policy.
     * @return true if policy was altered.
     */
    public boolean removeDN(String DN) {
        return this.DNs.remove(DN);
    }

    /**
     * @param attr The attribute to add to policy.
     * @return true if policy was altered.
     */
    public boolean addAttr(String attr) {
        if (!this.attrs.contains(attr)) {
            this.attrs.add(attr);
            return true;
        }
        return false;
    }

    /**
     * @param attr The attribute to remove from policy.
     * @return true if policy was altered.
     */
    public boolean removeAttr(String attr) {
        return this.attrs.remove(attr);
    }
        
    /**
     * Delete all entries.
     */
    public void clearPolicy() {
        this.DNs.clear();
        this.attrs.clear();
    }

    public void setDNs(ArrayList DNs) {
        this.DNs = DNs;
    }
    
    public void setAttrs(ArrayList attrs) {
        this.attrs = attrs;
    }
    protected void readObject(ObjectInputStream ois) 
        throws IOException, ClassNotFoundException {

        this.DNs = (ArrayList)ois.readObject();
        this.attrs = (ArrayList)ois.readObject();
    }

    protected void writeObject(ObjectOutputStream oos) throws IOException {

        oos.writeObject(this.DNs);
        oos.writeObject(this.attrs);
    }
}
