/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.workspace.impl.factory;

import java.util.Iterator;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.globus.workspace.WorkspaceConstants;
import org.globus.workspace.factory.WorkspaceFactoryResource;
import org.globus.wsrf.impl.ResourceHomeImpl;
import org.globus.wsrf.impl.SimpleResourceKey;
import org.globus.wsrf.utils.FilePersistenceHelper;

import javax.xml.namespace.QName;

public class WorkspaceFactoryHome extends ResourceHomeImpl {

    static Log logger = 
        LogFactory.getLog(WorkspaceFactoryHome.class.getName());

   private FilePersistenceHelper persistenceHelper = null;

    String defaultTTL = null;
    String maxTTL = null;
    String maxWorkspaces = "1";

    static final QName RESOURCE_KEY_QNAME =
        new QName(WorkspaceConstants.NS, "ResourceID");

    // todo: this gets called twice for some reason?
    public synchronized void initialize() throws Exception {
        this.keyTypeName = RESOURCE_KEY_QNAME;
        super.initialize();

        // maxWorkspaces has a default of one
        if ((this.defaultTTL == null) || (this.maxTTL == null)) {
                String err = "MaxTTL/defaultTTL configuration was not set";
                logger.debug(err);
                throw new Exception(err);
        }

        Integer ttl = new Integer(this.defaultTTL);;
        Integer maxttl = new Integer(this.maxTTL);
        Integer maxws = new Integer(this.maxWorkspaces);
        // Look in persistence directory
         boolean added = false;
        List listOfKeys = getPersistenceHelper().list();
        if (listOfKeys != null) {
            Iterator keyIterator = listOfKeys.iterator();
            while (keyIterator.hasNext()) {
                String keyValue = (String)keyIterator.next();
                SimpleResourceKey key = getResourceKey(keyValue);
                WorkspaceFactoryResource factoryResource =
                    (WorkspaceFactoryResource)createNewInstance();

                if (WorkspaceFactoryResource.class
                    .isAssignableFrom(this.resourceClass)) {
                    ((WorkspaceFactoryResource)factoryResource).load(key);
                }

                 /*Reset these values.  This allows the jndi-config to be
                   changed without needing to delete the persisted factory
                   resource file.  A database would make this all easier, just
                   leaving it hackish until then. */
                ((WorkspaceFactoryResourceImpl)factoryResource).
                        reinitialize(ttl, maxttl, maxws);

                logger.debug("Adding key " + key);
                this.add(key, factoryResource);
                added = true;
                logger.debug("loaded factory resource from persistence dir");
            }
        }

        if (added == false) {
            SimpleResourceKey key = getResourceKey("default");
            WorkspaceFactoryResource factoryResource =
                    (WorkspaceFactoryResource)createNewInstance();

            factoryResource.initialize(key, ttl, maxttl, maxws);
            logger.debug("Adding key " +  key);
            this.add(key, factoryResource);
        }

        // would be nice to make this INFO, but not until
        // "this gets called twice for some reason?" is figured out
        logger.debug("\nWORKSPACE FACTORY CONFIGURATIONS:" +
                    "\nDefault TTL: " + this.defaultTTL +
                    "\nMax TTL: " + this.maxTTL +
                    "\nMax workspaces per credential: " + this.maxWorkspaces);
    }

    public synchronized FilePersistenceHelper getPersistenceHelper() {
        if (this.persistenceHelper == null) {
            try {
                this.persistenceHelper =
                    new FilePersistenceHelper(this.resourceClass, ".ser");
            } catch (Exception e) {
                throw new RuntimeException(e.getMessage());
            }
        }
        return persistenceHelper;
    }
    
    
    public static final SimpleResourceKey getResourceKey(String keyValue) {
        return new SimpleResourceKey(RESOURCE_KEY_QNAME, keyValue);
    }

    public void setDefaultTTL(String ttl) {
        this.defaultTTL = ttl;
    }

    public String getDefaultTTL() {
        return this.defaultTTL;
    }

    public void setMaxTTL(String ttl) {
        this.maxTTL = ttl;
    }

    public String getMaxTTL() {
        return this.maxTTL;
    }

    public void setMaxWorkspaces(String max) {
        this.maxWorkspaces = max;
    }

    public String getMaxWorkspaces() {
        return this.maxWorkspaces;
    }


}
