/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.workspace.impl.service;

import org.apache.axis.types.Duration;
import org.globus.workspace.ReturnException;
import org.globus.workspace.WorkspaceConstants;
import org.globus.workspace.WorkspaceException;
import org.globus.workspace.WorkspaceUtil;
import org.globus.workspace.database.PersistenceAdapter;

import java.sql.SQLException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedList;
import java.io.File;

public class LeasedAccountsResource extends WorkspaceResourceImpl {

    private String poolName = null;

    private String status = WorkspaceConstants.LEASED_ACCOUNT_STATUS_ACTIVE;

    private static String expirePath = null;

    // If using LeasedAccountResource, the resource load in Home.init
    // removes the active ones from the available-account hashmap (via
    // LeasedAccountResource.loadInMech).  But we also need to
    // remove any quarantined accounts from the hashmap at container
    // start.
    public static void initMech(WorkspaceHome home) throws WorkspaceException {
        expirePath = home.getLeasedExpireProgram();
        logger.debug("expirePath = " + expirePath);
        if (expirePath == null) {
            logger.warn("no expiry program configured");
        } else {
            File f = new File(expirePath);
            if (!f.isAbsolute()) {
                throw new WorkspaceException("expire script must be" +
                        " an absolute path");
            }
        }

        Hashtable quarantined = getPersistenceAdapter().getQuarantined();

        if (quarantined.size() == 0) {
            logger.debug("no quarantined accounts persisted");
            return;
        }

        if (logger.isDebugEnabled()) {
            logger.debug("quarantined accounts in persistence:");
            Enumeration x = quarantined.keys();
            while (x.hasMoreElements()) {
                String poolname = (String) x.nextElement();
                LinkedList localnames = (LinkedList)quarantined.get(poolname);
                logger.debug("poolname = '" + x.nextElement() + "':");
                Iterator iter = localnames.iterator();
                while (iter.hasNext()) {
                    logger.debug("    " + iter.next());
                }
            }
        }

        Hashtable ht = getPersistenceAdapter().getAccountsPool();
        Enumeration pools = ht.keys();
        while (pools.hasMoreElements()) {
            String pool = (String)pools.nextElement();
            if (!quarantined.containsKey(pool)) {
                continue;
            }
            LinkedList localnames = (LinkedList) ht.get(pool);
            LinkedList qnames = (LinkedList) quarantined.get(pool);
            Iterator iter = localnames.iterator();
            while (iter.hasNext()) {
                String name = (String) iter.next();
                if (qnames.contains(name)) {
                    logger.debug("removed quarantined account '" + name +
                            "' from pool " + pool + "'s available accounts");
                    localnames.remove(name);
                }
            }
        }

    }


    /*
	 * (non-Javadoc)
	 * 
	 * @see org.globus.workspace.service.WorkspaceResource#initialize(java.lang.String,
	 *      java.lang.String, java.lang.String, java.lang.String,
	 *      org.apache.axis.types.Duration, org.apache.axis.types.Duration)
	 */
    public void initialize(String _creationMechanism, String _callerDN,
                           String _callerUserName, String _globusLocation,
                           Duration _defaultTTL, Duration _maxTTL) throws WorkspaceException {

        super.initialize(_creationMechanism, _callerDN, _callerUserName,
                _globusLocation, _defaultTTL, _maxTTL);
        logger.debug("initialized LeasedAccountsResource");
    }

    /*
      * (non-Javadoc)
      *
      * @see org.globus.workspace.impl.service.WorkspaceResourceImpl#createAccountUsingMech()
      */
    protected void createAccountUsingMech() throws WorkspaceException {
        // Read the pool account files and attributes and come up with a match
        // Update the table in the database
        logger.debug("callerDN :" + this.callerDN);

        logger.debug("Primary Attr: " + this.primaryAttr);
        this.poolName = getPersistenceAdapter().getPoolName(this.callerDN,
                this.primaryAttr, this.creatorAttributesVector);

        if (this.poolName != null) {
            LinkedList localNames = getPersistenceAdapter().getPoolAccounts(
                this.poolName);

            if (localNames == null) {
                throw new WorkspaceException("Found pool mapping '"
                        + this.poolName + "' for client, but no " +
                        "pool configuration for it");
            }

            // find a name in localNames that is not in usedLocalNamesOfPool
            if (localNames.size() > 0) {
                // Make a database query to get list of valid names
                this.localName = (String) localNames.remove(0);
                logger.debug("localName " + this.localName);
            } else {
                throw new WorkspaceException("No free local account");
            }
            this.status = WorkspaceConstants.LEASED_ACCOUNT_STATUS_ACTIVE;
        }
    }

    /*
      * (non-Javadoc)
      *
      * @see org.globus.workspace.impl.service.WorkspaceResourceImpl#deleteAccountUsingMech()
      */
    protected void deleteAccountUsingMech() throws WorkspaceException {

        if (expirePath != null) {
            // always quarantine first, if there is a problem with remove
            // script, leave quarantined.  If it is successful, then add
            // the account back into the available pool.

            getPersistenceAdapter().quarantine(this.localName, this.poolName);

            String[] command = {expirePath, this.localName};
            String returnString;
            try {
                returnString = WorkspaceUtil.runCommand(command);
                logger.debug(returnString);
            } catch (ReturnException e) {
                String err = "problem with expiration callout, command " +
                        "returned with:\nEXIT CODE: " + e.retval +
                        "\nSTDERR: '" + e.stderr + "'\nSTDOUT: '" +
                        e.stdout + "'";
                logger.error(err);
                logger.info("QUARANTINED: " + this.localName
                        + " (from pool '" + this.poolName + "')");
                throw new WorkspaceException(err);
            }
        }

        try {
            LinkedList temp = getPersistenceAdapter().
                    getPoolAccounts(this.poolName);
            if (temp == null) {
                throw new WorkspaceException("getPoolAccounts should " +
                        "never return null here");
            }
            temp.add(this.localName);
            getPersistenceAdapter().getAccountsPool().put(this.poolName, temp);
            getPersistenceAdapter()
                    .deleteLeasedAccountMapping(this.poolName, this.localName);
        } catch (SQLException dbex) {
            throw new WorkspaceException("Error deleting", dbex);
        }
    }

    /*
      * Make a database call to store this
      */
    protected void storeInMech(String localName, PersistenceAdapter adapter)
            throws Exception {
        this.localName = localName;
        // Add this localname to database
        adapter.storeLeasedAccount(this.localName, this.poolName, this.status);
    }

    /*
      * Load the resource specific data here
      */
    protected void loadInMech(String localName, PersistenceAdapter adapter)
            throws Exception {
        logger.debug("reloading " + localName);
        this.localName = localName;
        this.poolName = adapter.getLeasedAccountPoolName(localName);
        logger.debug("poolname : " + this.poolName);
        // Get poolname from database and remove localname from hashmap
        LinkedList list = (LinkedList) adapter.getAccountsPool()
                .get(this.poolName);
        logger.debug("Size of this list : " + list.size());
        list.remove(this.localName);

        adapter.getAccountsPool().put(this.poolName, list);
    }

    public String getLocalAccountName() {
        return this.localName;
    }
}
