/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.workspace.impl.service;

import org.globus.workspace.WorkspaceException;
import org.globus.workspace.WorkspaceUtil;
import org.globus.workspace.ReturnException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.axis.types.Duration;

import java.io.File;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import java.util.StringTokenizer;

public class PoolAccountsResource extends WorkspaceResourceImpl {

    static Log logger =
        LogFactory.getLog(WorkspaceResourceImpl.class.getName());

    /* set in initialize() */
    protected String executable = null;

    protected final static String BASE_COMMAND = "bin" + File.separator
        + "poolaccts_setuid";
    protected final static String POOLINDEX_COMMAND = "-return_poolindex";
    protected final static String DELETE_COMMAND = "-delete_dn";


    //overriding to set executable
    public void initialize(String _creationMechanism,
                       String _callerDN,
                       String _callerUserName,
                       String _globusLocation,
                       Duration _defaultTTL,
                       Duration _maxTTL)
        throws WorkspaceException {

        super.initialize(_creationMechanism, _callerDN, _callerUserName,
                _globusLocation, _defaultTTL, _maxTTL);

        if (_globusLocation.endsWith(File.separator)) {
            executable = _globusLocation + BASE_COMMAND;
        } else {
            executable = _globusLocation + File.separator + BASE_COMMAND;
        }
        logger.debug("Executable " + executable);

    }


    /*
     * BEGIN setuid interface related methods
     */

    protected void createAccountUsingMech() throws WorkspaceException {

        String command[] = null;
        if (this.primaryAttr != null) {
            command =  new String[4];
            command[0] = executable;
            command[1] = POOLINDEX_COMMAND;
            command[2] = this.callerDN;
            command[3] = this.primaryAttr;
        } else {
            command =  new String[3];
            command[0] = executable;
            command[1] = POOLINDEX_COMMAND;
            command[2] = this.callerDN;
        }

        String returnString = null;
        try {
            returnString = WorkspaceUtil.runCommand(command);
        } catch (ReturnException e) {
            translateReturnValue(e);
        }

        logger.debug("Return string " + returnString);
        StringTokenizer st = new StringTokenizer(returnString,":");
        if (st.hasMoreTokens()) {
            this.localName = st.nextToken();
            if (st.hasMoreTokens()) {
                this.primaryGroup = st.nextToken();
            } else {
                throw new WorkspaceException("no group returned from " +
                        "pool impl");
            }
        } else {
            throw new WorkspaceException("no account returned from " +
                    "pool impl");
        }
        logger.debug("Local name " + this.localName);
        logger.debug("Primary group " + this.primaryGroup);
    }

    protected void deleteAccountUsingMech() throws WorkspaceException {
        String command[] =  new String[3];
        command[0] = executable;
        command[1] = DELETE_COMMAND;
        command[2] = this.callerDN;
        try {
            WorkspaceUtil.runCommand(command);
        } catch (ReturnException e) {
            translateReturnValue(e);
        }
    }

    private void translateReturnValue(ReturnException e)
            throws WorkspaceException {

        /* #define GENERAL_FAILURE 255 */
        if (e.retval == 255) {
            if (e.stderr != null) {
            throw new WorkspaceException("failure running pool" +
                    " account setuid\nSTDERR: " + e.stderr);
            } else {
            throw new WorkspaceException("failure running pool" +
                        " account setuid");
            }
        }
        /* #define ENVIRONMENT 1 */
        if (e.retval == 1) {
            if (e.stderr != null) {
            throw new WorkspaceException("pool account setuid" +
                    " setup problem\nSTDERR: " + e.stderr);
            } else {
            throw new WorkspaceException("pool account setuid" +
                    " setup problem");
            }
        }
        /* #define SYNTAX 2 */
        if (e.retval == 2) {
            if (e.stderr != null) {
            throw new WorkspaceException("pool account service" +
                    " setup problem\nSTDERR: " + e.stderr);
            } else {
            throw new WorkspaceException("pool account service" +
                    "setup problem");
            }
        }
        /* #define BADARG 3 */
        if (e.retval == 3) {
            if (e.stderr != null) {
            throw new WorkspaceException("problem with parameters, " +
                    "STDERR: " + e.stderr);
            } else {
            throw new WorkspaceException("problem with parameters");
            }
        }
        /* #define POOL_INDEX_FAILURE 4 */
        if (e.retval == 4) {
            if (e.stderr != null) {
            throw new WorkspaceException("problem with mappings, " +
                    "STDERR: " + e.stderr);
            } else {
            throw new WorkspaceException("problem with mappings");
            }
        }
        /* #define NO_POOL_FOR_USER 5 */
        if (e.retval == 5) {
            if (e.stderr != null) {
            throw new WorkspaceException("found user for DN, but no" +
                    "pool account, STDERR: " + e.stderr);
            } else {
            throw new WorkspaceException("found user for DN, but no" +
                    "pool account,");
            }
        }
        /* #define NO_USER 6 */
        if (e.retval == 6) {
            if (e.stderr != null) {
            throw new WorkspaceException("no mapping for this DN, " +
                    "STDERR: " + e.stderr);
            } else {
            throw new WorkspaceException("no mapping for this DN");
            }
        }
        /* #define POOL_FULL 7 */
        if (e.retval == 7) {
            if (e.stderr != null) {
            throw new WorkspaceException("pool is exhausted, " +
                    "STDERR: " + e.stderr);
            } else {
            throw new WorkspaceException("pool is exhausted");
            }
        }
        /* #define GRIDMAPDIR_NO_ASSOCIATION 8 */
        if (e.retval == 8) {
            if (e.stderr != null) {
            throw new WorkspaceException("pooling environment problem, " +
                    "STDERR: " + e.stderr);
            } else {
            throw new WorkspaceException("pooling environment problem");
            }
        }


        /*****
         * EXPIRY_PROG_FAILURE & EXPIRY_PROG_CHECK are both errors that happen
         * after a lease has been unleased and quarantined.  Perhaps don't throw
         * exception because this resource, which represents the lease, must
         * succeed in its destruction?
         */

        /* #define EXPIRY_PROG_FAILURE 9 */
        if (e.retval == 9) {
            if (e.stderr != null) {
            throw new WorkspaceException("expire account program failure, " +
                    "STDERR: " + e.stderr);
            } else {
            throw new WorkspaceException("expire account program failure");
            }
        }

        /* #define EXPIRY_PROG_CHECK 10 */
        if (e.retval == 10) {
            if (e.stderr != null) {
            throw new WorkspaceException("expire account environment problem," +
                    " STDERR: " + e.stderr);
            } else {
            throw new WorkspaceException("expire account environment problem");
            }
        }

        /* #define NO_GROUP 11 */
        if (e.retval == 11) {
            if (e.stderr != null) {
            throw new WorkspaceException("no group mapping for attribute, " +
                    "STDERR: " + e.stderr);
            } else {
            throw new WorkspaceException("no group mapping for attribute");
            }
        }

        /* others? */
        if (e.stderr != null) {
        throw new WorkspaceException("pooling problem, implementation " +
                "returns error #" + e.retval + ", STDERR: " + e.stderr);
        } else {
        throw new WorkspaceException("pooling problem, implementation" +
                "returns error #" + e.retval);
        }

    }

    /*
     * END setuid interface related methods
     */

    protected void storeInMech(ObjectOutputStream oos) throws Exception {

        oos.writeObject(this.executable);
    }

    protected void loadInMech(ObjectInputStream ois) throws Exception {

        this.executable = (String)ois.readObject();
    }
}
