/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.workspace.impl.service;

import org.apache.axis.types.Duration;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.globus.workspace.WorkspaceException;
import org.globus.workspace.WorkspaceUtil;

import java.io.File;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

public class UnixAccountsResource extends WorkspaceResourceImpl {


    final static String BASE_COMMAND = "bin" + File.separator
        + "adduser_setuid";
    final static String ADDUSER_COMMAND = "-adduser";
    final static String GETQUOTA_COMMAND = "-getquota";
    final static String SETQUOTA_COMMAND = "-setquota";
    final static String DELUSER_COMMAND = "-deluser";
    String executable = null;

    static Log logger =
        LogFactory.getLog(UnixAccountsResource.class.getName());

    public void initialize(String _creationMechanism,
                           String _callerDN,
                           String _callerUserName,
                           String _globusLocation,
                           Duration _defaultTTL,
                           Duration _maxTTL)
            throws WorkspaceException {


        super.initialize(_creationMechanism, _callerDN, _callerUserName, 
                _globusLocation, _defaultTTL, _maxTTL);

        if (_globusLocation.endsWith(File.separator)) {
            executable = _globusLocation + BASE_COMMAND;
        } else {
            executable = _globusLocation + File.separator + BASE_COMMAND;
        }
        logger.error("Executable " + executable);
    }

    protected void createAccountUsingMech() throws WorkspaceException {

        // Create actual account
        String command[] =  new String[4];
        command[0] = executable;
        command[1] = ADDUSER_COMMAND;
        command[2] = this.callerDN;
        command[3] = this.callerUsername;

        String err = "Error creating new account ";
        String returnString = WorkspaceUtil.runCommand(command, err);
	    logger.debug("Return string " + returnString);
	    this.localName = returnString.substring(0, returnString.indexOf(":"));
	    logger.debug("Local name " + this.localName);
        this.primaryGroup = returnString.substring(returnString.indexOf(":"));
	    logger.debug("Primary group " + this.primaryGroup);
    }

    protected void deleteAccountUsingMech() throws WorkspaceException {
        String command[] =  new String[3];
        command[0] = executable;
        command[1] = DELUSER_COMMAND;
        command[2] = this.localName;
        String err = "Error deleting account " + this.localName;
        WorkspaceUtil.runCommand(command, err);
    }

    protected void constructGroupMapInput() throws WorkspaceException {
        //nothing to do
    }

    protected void storeInMech(ObjectOutputStream oos) throws Exception {
        oos.writeObject(this.executable);
    }

    protected void loadInMech(ObjectInputStream ois) throws Exception {

        this.executable = (String)ois.readObject();
    }
}
