/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.workspace.service;

import org.apache.axis.types.Duration;

import org.globus.workspace.WorkspaceException;
import org.globus.workspace.LimitException;
import org.globus.workspace.generated.types.AccountInformation;
import org.globus.workspace.generated.types.VOMSAttributes;

import org.globus.wsrf.ResourceKey;
import org.globus.wsrf.RemoveCallback;
import org.globus.wsrf.ResourceLifetime;
import org.globus.wsrf.ResourceProperties;
import org.globus.wsrf.ResourceException;
import org.globus.wsrf.NoSuchResourceException;
import org.globus.wsrf.InvalidResourceKeyException;

import org.globus.wsrf.security.SecureResource;

public interface WorkspaceResource
    extends ResourceProperties, ResourceLifetime, RemoveCallback, 
            SecureResource {
    
    public void initialize(String creationMechanism, String callerDN,
                           String callerUserName, String globusLocation,
                           Duration factoryTTL, Duration maxTTL)
            throws WorkspaceException;

    public String createAccount(AccountInformation info)
            throws WorkspaceException, LimitException;
    
    public void addAuthzIdentity(String str) throws WorkspaceException;

    public void addAccessIdentity(String str) throws WorkspaceException;

    public boolean removeAuthzIdentity(String str) throws WorkspaceException;

    public boolean removeAccessIdentity(String str) throws WorkspaceException;

    public String getLocalAccountName() throws WorkspaceException;

    public String getOwnerDN() throws WorkspaceException;

    public VOMSAttributes getVOMSAttributes() throws WorkspaceException;

    public void store() throws ResourceException;

    public void load(ResourceKey key) throws ResourceException,
                                             NoSuchResourceException,
                                             InvalidResourceKeyException;
}
