/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.workspace.impl.service;

import org.apache.axis.client.Stub;
import org.apache.axis.message.addressing.AttributedURI;
import org.apache.axis.message.addressing.EndpointReferenceType;
import org.apache.axis.types.Duration;
import org.globus.gsi.GlobusCredential;
import org.globus.workspace.TestConstants;
import org.globus.workspace.WorkspaceConstants;
import org.globus.workspace.generated.PolicyQuery;
import org.globus.workspace.generated.QueryAccessPolicyResponse;
import org.globus.workspace.generated.WorkspaceFactoryPortType;
import org.globus.workspace.generated.WorkspaceFactoryServiceAddressingLocator;
import org.globus.workspace.generated.WorkspacePortType;
import org.globus.workspace.generated.WorkspaceServiceAddressingLocator;
import org.globus.workspace.generated.types.AccountFaultType;
import org.globus.workspace.generated.types.AccountInformation;
import org.globus.workspace.generated.types.AccountReference;
import org.globus.workspace.generated.types.VOMSAttributes;
import org.globus.workspace.generated.types.VoidType;
import org.globus.workspace.impl.factory.WorkspaceFactoryHome;
import org.globus.wsrf.ResourceKey;
import org.globus.wsrf.encoding.ObjectDeserializer;
import org.globus.wsrf.encoding.ObjectSerializer;
import org.globus.wsrf.impl.security.authentication.Constants;
import org.globus.wsrf.impl.security.authorization.NoAuthorization;
import org.globus.wsrf.test.GridTestCase;
import org.globus.wsrf.utils.AddressingUtils;
import org.globus.wsrf.utils.FaultHelper;
import org.globus.wsrf.utils.FilePersistenceHelper;
import org.xml.sax.InputSource;

import javax.xml.namespace.QName;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;

import org.globus.wsrf.utils.AnyHelper;

import org.globus.wsrf.NoSuchResourceException;
import org.oasis.wsrf.properties.WSResourcePropertiesServiceAddressingLocator;
import org.oasis.wsrf.properties.GetMultipleResourcePropertiesResponse;
import org.oasis.wsrf.properties.GetMultipleResourceProperties_Element;
import org.oasis.wsrf.properties.GetMultipleResourceProperties_PortType;

public class WorkspaceServiceTest extends GridTestCase {

    static String mechanism = null;
    static Object protection = null;
    String ownerIdentity = null;

    private static boolean postPackage = false;
    private static File testResourceEPRFile = new File("testResourceEPR");

    public static void setPostPackage(boolean _postPackage) {
        postPackage = _postPackage;
    }

    public WorkspaceServiceTest(String name) throws Exception {
        super(name);
        // FIXME: proxy file.
        // assuming use of default proxy
        GlobusCredential credential = GlobusCredential.getDefaultCredential();
        assertTrue(credential != null);
        this.ownerIdentity = credential.getIdentity();
    }

    public void test() throws Exception {

        assertTrue(TEST_CONTAINER != null);

        // username 1
        String userName1 = TestConstants.CREATE_LOCAL_NAME + "1";

            // username 2
        String userName2 = TestConstants.CREATE_LOCAL_NAME + "4";

        WorkspacePortType servicePort =  null;

        WorkspaceServiceAddressingLocator serviceLocator =
            new WorkspaceServiceAddressingLocator();        

        EndpointReferenceType workspaceEPR2 = null;

        if (!postPackage) {
            String factoryUrl = 
                TEST_CONTAINER.getBaseURL() + TestConstants.TEST_FACTORY_NAME;

            // locator
            WorkspaceFactoryServiceAddressingLocator locator = 
                new WorkspaceFactoryServiceAddressingLocator();
            // key
            ResourceKey key = WorkspaceFactoryHome.getResourceKey("default");
            // EPR
            EndpointReferenceType epr =
                AddressingUtils.createEndpointReference(factoryUrl, key);
            // port reference
            WorkspaceFactoryPortType factoryPort = 
                locator.getWorkspaceFactoryPortTypePort(epr);

            // use default
            AccountInformation accountInfo = new AccountInformation();
            
            // without security
            boolean worked = false;
            try {
                //AccountReference accountRef =
                    factoryPort.createAccount(accountInfo);
            } catch (Exception exp) {
                if (exp.getMessage()
                    .indexOf("GSI Secure Message authentication required") 
                    != -1) {
                    worked = true;
                } else {
                    exp.printStackTrace();
                }
            }
            assertTrue(worked);

            // set security
            setSecurity((Stub) factoryPort);

            // create account
            AccountReference accountRef1 = 
                factoryPort.createAccount(accountInfo);
            assertTrue(accountRef1 != null);
            
            // EPR
            EndpointReferenceType workspaceEPR1 = 
                accountRef1.getEndpointReference();
            System.out.println(workspaceEPR1);
            assertTrue(workspaceEPR1.toString().indexOf(userName1) != -1);

            // ttl  - 10
            accountInfo.setTTL(new Duration(false, 0, 0, 0, 0, 0, 20));
            worked = false;
            try {
                //AccountReference accountRef =
                    factoryPort.createAccount(accountInfo);
            } catch (AccountFaultType exp) {
                FaultHelper helper = new FaultHelper(exp);
                if (helper.getMessage()
                    .indexOf("invalid time to live") != -1) {
                    worked = true;
                } else {
                    exp.printStackTrace();
                }
            }
            assertTrue(worked);
            
            // ttl > maxTTL 
            accountInfo.setTTL(new Duration(false, 0, 0, 0, 0, 100, 0));
            worked = false;
            try {
                //AccountReference accountRef =
                    factoryPort.createAccount(accountInfo);
            } catch (AccountFaultType exp) {
                FaultHelper helper = new FaultHelper(exp);
                if (helper.getMessage()
                    .indexOf("unauthorized time to live") != -1) {
                    worked = true;
                } else {
                    exp.printStackTrace();
                }
            }
            assertTrue(worked);

            // 40 mins
            accountInfo.setTTL(new Duration(false, 0, 0, 0, 0, 40, 0));
            
            // create account
            AccountReference accountRef2 = 
                factoryPort.createAccount(accountInfo);
            assertTrue(accountRef2 != null);
            
            // EPR
            workspaceEPR2 = accountRef2.getEndpointReference();
            assertTrue(workspaceEPR2.toString().indexOf(userName2) != -1);
            
            // resource 1 test
            resourceTest1(workspaceEPR1, userName1);


            saveEprToFile(workspaceEPR2);

        } else {
            workspaceEPR2 = readEprFromFile();
            AttributedURI uri = workspaceEPR2.getAddress();
            uri.setPort(TEST_CONTAINER.getPort());
            workspaceEPR2.setAddress(uri);
        }
        
        servicePort = serviceLocator.getWorkspacePortTypePort(workspaceEPR2);
        setSecurity((Stub) servicePort);        

        // methods on service
        resourceTest2(servicePort, userName2);

        // resource properties test, before and after persistence.
        resourcePropertiesTest(workspaceEPR2, userName2);

        if (postPackage) {
            servicePort.destroy(new org.oasis.wsrf.lifetime.Destroy());
            
            boolean worked = false;
            try {
                servicePort.getOwnerDN(new VoidType());
            } catch (AccountFaultType exp) {
                FaultHelper helper = new FaultHelper(exp);
                if (helper.getMessage()
                    .indexOf("No resource found") != -1) {
                    worked = true;
                } else {
                    exp.printStackTrace();
                }
            }
            assertTrue(worked);

            // clean up
            testResourceEPRFile.delete();
        }
    }

    private void resourceTest2(WorkspacePortType servicePort, String userName) 
        throws Exception {

        // true in both cases
        assertTrue(userName.equals(servicePort
                                    .getLocalAccountName(new VoidType())));

        // owner dn        
        assertTrue(this.ownerIdentity
                   .equals(servicePort.getOwnerDN(new VoidType())));

        // add acces policy - only in setup
        if (!postPackage) {
            servicePort.addAccessIdentity("tester2DN");
        }

        // all mappings ?
        // true in both cases
        QueryAccessPolicyResponse response = 
            servicePort.queryAccessPolicy(new PolicyQuery(true, "someAttr",
                                                          "tester2DN"));
        String[] userNames = response.getUsername();
        for (int i=0; i<userNames.length; i++) {
            System.out.println("username " + userNames[i]);
        }
        System.out.println("Matched " + userNames[0] + " user " + userName);
        assertTrue(userName.equals(userNames[0]));
    }

    private void resourceTest1(EndpointReferenceType workspaceEPR1, 
                               String userName1) throws Exception {
        
        // username
        WorkspaceServiceAddressingLocator serviceLocator =
            new WorkspaceServiceAddressingLocator();
        WorkspacePortType servicePort = 
            serviceLocator.getWorkspacePortTypePort(workspaceEPR1);
        setSecurity((Stub) servicePort);
        assertTrue(userName1.equals(servicePort
                                    .getLocalAccountName(new VoidType())));

        // owner dn        
        assertTrue(this.ownerIdentity
                   .equals(servicePort.getOwnerDN(new VoidType())));
        
        // VOMS attribute - null for now
        // FIXME: use of VOMS attribute
        VOMSAttributes attrs = servicePort.getVOMSAttributes(new VoidType());
        assertTrue(attrs.getFqan() == null);

        // add dummy DNs
        servicePort.addAuthzIdentity("FOO");

        // Query access policy
        QueryAccessPolicyResponse response = 
            servicePort.queryAccessPolicy(new PolicyQuery(true, "someAttr",
                                                          "tester1DN"));
        assertTrue(response.getUsername() == null);

        // add acces policy
        servicePort.addAccessIdentity("tester1DN");
        
        // query
        response = 
            servicePort.queryAccessPolicy(new PolicyQuery(true, "someAttr",
                                                          "tester1DN"));

        String matchedUserName = response.getUsername()[0];
        assertTrue(userName1.equals(matchedUserName));

        servicePort.removeAuthzIdentity("FOO");
        // remove caller identity
        servicePort.removeAuthzIdentity(this.ownerIdentity);

        // query shoudl still work.        
        response = 
            servicePort.queryAccessPolicy(new PolicyQuery(true, "someAttr",
                                                          "tester1DN"));
        assertTrue(userName1.equals(response.getUsername()[0]));
        
        // any other call should fail.
        boolean worked = false;
        try {
            servicePort.getOwnerDN(new VoidType());
        } catch (Exception exp) {
            if ((exp.getMessage().indexOf(this.ownerIdentity) != -1) &&
                (exp.getMessage().indexOf("not authorized to use ") != -1)) {
                worked = true;
            }
        }
        assertTrue(worked);

        // Only way to destroy this now is to remove persisted file
        /*
        FilePersistenceHelper persistenceHelper =
            new FilePersistenceHelper(TestWorkspaceResourceImpl.class, ".ser");
        File file = persistenceHelper.getKeyAsFile(userName1);
        file.delete();
        */
    }

    private static void setSecurity(Stub stub) {

        stub._setProperty(Constants.GSI_SEC_MSG, Constants.SIGNATURE);
        stub._setProperty(Constants.AUTHORIZATION,
            NoAuthorization.getInstance());
    }

    private EndpointReferenceType readEprFromFile() throws Exception {
        FileInputStream in = null;
        EndpointReferenceType ref = null;
        try {
            in = new FileInputStream(testResourceEPRFile);
            ref = (EndpointReferenceType)ObjectDeserializer
                .deserialize(new InputSource(in),
                             EndpointReferenceType.class);
        } finally {
            if (in != null) {
                try { in.close(); } catch (Exception e) {}
            }
        }
        return ref;
    }

    private void saveEprToFile(EndpointReferenceType ref)
        throws Exception {
        String eprStr = ObjectSerializer.toString(ref,
                                                  new QName("", "EPR"));
        FileWriter writer = null;
        try {
            writer = new FileWriter(testResourceEPRFile);
            writer.write(eprStr);
        } finally {
            if (writer != null) {
                try { writer.close(); } catch (Exception e) {}
            }
        }
    }

    private void resourcePropertiesTest(EndpointReferenceType epr, 
                                        String userName) 
        throws Exception {

        // not including policy, ttl stuff and vomsAttribute
        QName[] qnames = new QName[4];
        qnames[0] = new QName(WorkspaceConstants.NS, "CreationMechanism");
        qnames[1] = new QName(WorkspaceConstants.NS, "AccountType");
        qnames[2] = new QName(WorkspaceConstants.NS, "LocalName");
        qnames[3] = new QName(WorkspaceConstants.NS, "Policy");

        GetMultipleResourceProperties_Element request =
            new GetMultipleResourceProperties_Element();
        request.setResourceProperty(qnames);

        WSResourcePropertiesServiceAddressingLocator locator =
            new WSResourcePropertiesServiceAddressingLocator();

        GetMultipleResourceProperties_PortType port =
            locator.getGetMultipleResourcePropertiesPort(epr);
        setSecurity((Stub)port);
        
        GetMultipleResourcePropertiesResponse response =
            port.getMultipleResourceProperties(request);

        String str = AnyHelper.toSingleString(response);
        assertTrue((str.indexOf("default") != -1) && 
                   (str.indexOf("IndependentAccount") != -1) &&
                   (str.indexOf(userName) != -1) && 
                   (str.indexOf("tester2DN") != -1));
    }
}
